/*!
 * Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * Neither the name of Anaconda nor the names of any contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
*/
(function(root, factory) {
  const bokeh = factory();
  if (root.Bokeh === undefined) {
    root.Bokeh = bokeh;
  }
  const Bokeh = root.Bokeh;
  Bokeh[bokeh.version] = bokeh;
})(this, function() {
  var define;
  var parent_require = typeof require === "function" && require
  return (function(modules, entry, aliases, externals) {
    if (aliases === undefined) aliases = {};
    if (externals === undefined) externals = {};

    var cache = {};

    var normalize = function(name) {
      if (typeof name === "number")
        return name;

      if (name === "bokehjs")
        return entry;

      var prefix = "@bokehjs/"
      if (name.slice(0, prefix.length) === prefix)
        name = name.slice(prefix.length)

      var alias = aliases[name]
      if (alias != null)
        return alias;

      var trailing = name.length > 0 && name[name.lenght-1] === "/";
      var index = aliases[name + (trailing ? "" : "/") + "index"];
      if (index != null)
        return index;

      return name;
    }

    var require = function(name) {
      var mod = cache[name];
      if (!mod) {
        var id = normalize(name);

        mod = cache[id];
        if (!mod) {
          if (!modules[id]) {
            if (externals[id] === false || (externals[id] == true && parent_require)) {
              try {
                mod = {exports: externals[id] ? parent_require(id) : {}};
                cache[id] = cache[name] = mod;
                return mod.exports;
              } catch (e) {}
            }

            var err = new Error("Cannot find module '" + name + "'");
            err.code = 'MODULE_NOT_FOUND';
            throw err;
          }

          mod = {exports: {}};
          cache[id] = cache[name] = mod;
          modules[id].call(mod.exports, require, mod, mod.exports);
        } else
          cache[name] = mod;
      }

      return mod.exports;
    }

    var main = require(entry);
    main.require = require;

    if (typeof Proxy !== "undefined") {
      // allow Bokeh.loader["@bokehjs/module/name"] syntax
      main.loader = new Proxy({}, {
        get: function(_obj, module) {
          return require(module);
        }
      });
    }

    main.register_plugin = function(plugin_modules, plugin_entry, plugin_aliases, plugin_externals) {
      if (plugin_aliases === undefined) plugin_aliases = {};
      if (plugin_externals === undefined) plugin_externals = {};

      for (var name in plugin_modules) {
        modules[name] = plugin_modules[name];
      }

      for (var name in plugin_aliases) {
        aliases[name] = plugin_aliases[name];
      }

      for (var name in plugin_externals) {
        externals[name] = plugin_externals[name];
      }

      var plugin = require(plugin_entry);

      for (var name in plugin) {
        main[name] = plugin[name];
      }

      return plugin;
    }

    return main;
  })
([
/* main.js */ function _(require, module, exports) {
    var __asyncValues = (this && this.__asyncValues) || function (o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const yargs_1 = require(1) /* yargs */;
    const path_1 = require("path");
    const sys_1 = require(43) /* ./sys */;
    const build_1 = require(58) /* ./build */;
    const compile_1 = require(823) /* ./compile */;
    async function read_stdin() {
        var e_1, _a;
        const stdin = process.stdin;
        stdin.setEncoding("utf-8");
        stdin.resume();
        let data = "";
        try {
            for (var stdin_1 = __asyncValues(stdin), stdin_1_1; stdin_1_1 = await stdin_1.next(), !stdin_1_1.done;) {
                const chunk = stdin_1_1.value;
                data += chunk;
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (stdin_1_1 && !stdin_1_1.done && (_a = stdin_1.return))
                    await _a.call(stdin_1);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        return data;
    }
    function reply(data) {
        process.stdout.write(JSON.stringify(data));
        process.stdout.write("\n");
    }
    async function compile() {
        if (yargs_1.argv.file != null) {
            const input = {
                code: yargs_1.argv.code != null ? yargs_1.argv.code : sys_1.read(yargs_1.argv.file),
                lang: yargs_1.argv.lang || "typescript",
                file: yargs_1.argv.file,
                bokehjs_dir: yargs_1.argv.bokehjsDir || "./build",
            };
            return await compile_1.compile_and_resolve_deps(input);
        }
        else {
            const input = JSON.parse(await read_stdin());
            return await compile_1.compile_and_resolve_deps(input);
        }
    }
    async function main() {
        const cmd = yargs_1.argv._[0];
        if (cmd == "build") {
            try {
                const base_dir = path_1.resolve(yargs_1.argv.baseDir);
                const bokehjs_dir = path_1.resolve(yargs_1.argv.bokehjsDir);
                const rebuild = yargs_1.argv.rebuild;
                const bokeh_version = yargs_1.argv.bokehVersion;
                const result = await build_1.build(base_dir, bokehjs_dir, { rebuild, bokeh_version });
                process.exit(result ? 0 : 1);
            }
            catch (error) {
                console.log(error.stack);
                process.exit(1);
            }
        }
        else if (cmd == "init") {
            try {
                const base_dir = path_1.resolve(yargs_1.argv.baseDir);
                const bokehjs_dir = path_1.resolve(yargs_1.argv.bokehjsDir);
                const interactive = yargs_1.argv.interactive;
                const bokehjs_version = yargs_1.argv.bokehjsVersion;
                const bokeh_version = yargs_1.argv.bokehVersion;
                const result = await build_1.init(base_dir, bokehjs_dir, { interactive, bokehjs_version, bokeh_version });
                process.exit(result ? 0 : 1);
            }
            catch (error) {
                console.log(error.stack);
                process.exit(1);
            }
        }
        else {
            try {
                reply(await compile());
            }
            catch (error) {
                reply({ error: error.stack });
            }
        }
    }
    main();
},
/* yargs/index.js */ function _(require, module, exports) {
    // classic singleton yargs API, to use yargs
    // without running as a singleton do:
    // require('yargs/yargs')(process.argv.slice(2))
    const yargs = require(2) /* ./yargs */;
    const processArgv = require(34) /* ./lib/process-argv */;
    Argv(processArgv.getProcessArgvWithoutBin());
    module.exports = Argv;
    function Argv(processArgs, cwd) {
        const argv = yargs(processArgs, cwd, require);
        singletonify(argv);
        return argv;
    }
    /*  Hack an instance of Argv with process.argv into Argv
        so people can do
        require('yargs')(['--beeble=1','-z','zizzle']).argv
        to parse a list of args and
        require('yargs').argv
        to get a parsed version of process.argv.
    */
    function singletonify(inst) {
        Object.keys(inst).forEach((key) => {
            if (key === 'argv') {
                Argv.__defineGetter__(key, inst.__lookupGetter__(key));
            }
            else if (typeof inst[key] === 'function') {
                Argv[key] = inst[key].bind(inst);
            }
            else {
                Argv.__defineGetter__('$0', () => {
                    return inst.$0;
                });
                Argv.__defineGetter__('parsed', () => {
                    return inst.parsed;
                });
            }
        });
    }
},
/* yargs/yargs.js */ function _(require, module, exports) {
    // an async function fails early in Node.js versions prior to 8.
    async function requiresNode8OrGreater() { }
    requiresNode8OrGreater();
    const argsert = require(3) /* ./lib/argsert */;
    const fs = require('fs');
    const Command = require(4) /* ./lib/command */;
    const Completion = require(14) /* ./lib/completion */;
    const Parser = require(7) /* yargs-parser */;
    const path = require('path');
    const Usage = require(16) /* ./lib/usage */;
    const Validation = require(30) /* ./lib/validation */;
    const Y18n = require(32) /* y18n */;
    const objFilter = require(21) /* ./lib/obj-filter */;
    const setBlocking = require(22) /* set-blocking */;
    const applyExtends = require(33) /* ./lib/apply-extends */;
    const { globalMiddlewareFactory } = require(6) /* ./lib/middleware */;
    const YError = require(13) /* ./lib/yerror */;
    const processArgv = require(34) /* ./lib/process-argv */;
    exports = module.exports = Yargs;
    function Yargs(processArgs, cwd, parentRequire) {
        processArgs = processArgs || []; // handle calling yargs().
        const self = {};
        let command = null;
        let completion = null;
        let groups = {};
        const globalMiddleware = [];
        let output = '';
        const preservedGroups = {};
        let usage = null;
        let validation = null;
        let handlerFinishCommand = null;
        const y18n = Y18n({
            directory: path.resolve(__dirname, './locales'),
            updateFiles: false
        });
        self.middleware = globalMiddlewareFactory(globalMiddleware, self);
        if (!cwd)
            cwd = process.cwd();
        self.scriptName = function (scriptName) {
            self.customScriptName = true;
            self.$0 = scriptName;
            return self;
        };
        // ignore the node bin, specify this in your
        // bin file with #!/usr/bin/env node
        if (/\b(node|iojs|electron)(\.exe)?$/.test(process.argv[0])) {
            self.$0 = process.argv.slice(1, 2);
        }
        else {
            self.$0 = process.argv.slice(0, 1);
        }
        self.$0 = self.$0
            .map((x, i) => {
            const b = rebase(cwd, x);
            return x.match(/^(\/|([a-zA-Z]:)?\\)/) && b.length < x.length ? b : x;
        })
            .join(' ').trim();
        if (process.env._ !== undefined && processArgv.getProcessArgvBin() === process.env._) {
            self.$0 = process.env._.replace(`${path.dirname(process.execPath)}/`, '');
        }
        // use context object to keep track of resets, subcommand execution, etc
        // submodules should modify and check the state of context as necessary
        const context = { resets: -1, commands: [], fullCommands: [], files: [] };
        self.getContext = () => context;
        // puts yargs back into an initial state. any keys
        // that have been set to "global" will not be reset
        // by this action.
        let options;
        self.resetOptions = self.reset = function resetOptions(aliases) {
            context.resets++;
            aliases = aliases || {};
            options = options || {};
            // put yargs back into an initial state, this
            // logic is used to build a nested command
            // hierarchy.
            const tmpOptions = {};
            tmpOptions.local = options.local ? options.local : [];
            tmpOptions.configObjects = options.configObjects ? options.configObjects : [];
            // if a key has been explicitly set as local,
            // we should reset it before passing options to command.
            const localLookup = {};
            tmpOptions.local.forEach((l) => {
                localLookup[l] = true;
                (aliases[l] || []).forEach((a) => {
                    localLookup[a] = true;
                });
            });
            // add all groups not set to local to preserved groups
            Object.assign(preservedGroups, Object.keys(groups).reduce((acc, groupName) => {
                const keys = groups[groupName].filter(key => !(key in localLookup));
                if (keys.length > 0) {
                    acc[groupName] = keys;
                }
                return acc;
            }, {}));
            // groups can now be reset
            groups = {};
            const arrayOptions = [
                'array', 'boolean', 'string', 'skipValidation',
                'count', 'normalize', 'number',
                'hiddenOptions'
            ];
            const objectOptions = [
                'narg', 'key', 'alias', 'default', 'defaultDescription',
                'config', 'choices', 'demandedOptions', 'demandedCommands', 'coerce',
                'deprecatedOptions'
            ];
            arrayOptions.forEach((k) => {
                tmpOptions[k] = (options[k] || []).filter(k => !localLookup[k]);
            });
            objectOptions.forEach((k) => {
                tmpOptions[k] = objFilter(options[k], (k, v) => !localLookup[k]);
            });
            tmpOptions.envPrefix = options.envPrefix;
            options = tmpOptions;
            // if this is the first time being executed, create
            // instances of all our helpers -- otherwise just reset.
            usage = usage ? usage.reset(localLookup) : Usage(self, y18n);
            validation = validation ? validation.reset(localLookup) : Validation(self, usage, y18n);
            command = command ? command.reset() : Command(self, usage, validation, globalMiddleware);
            if (!completion)
                completion = Completion(self, usage, command);
            completionCommand = null;
            output = '';
            exitError = null;
            hasOutput = false;
            self.parsed = false;
            return self;
        };
        self.resetOptions();
        // temporary hack: allow "freezing" of reset-able state for parse(msg, cb)
        const frozens = [];
        function freeze() {
            const frozen = {};
            frozens.push(frozen);
            frozen.options = options;
            frozen.configObjects = options.configObjects.slice(0);
            frozen.exitProcess = exitProcess;
            frozen.groups = groups;
            usage.freeze();
            validation.freeze();
            command.freeze();
            frozen.strict = strict;
            frozen.strictCommands = strictCommands;
            frozen.completionCommand = completionCommand;
            frozen.output = output;
            frozen.exitError = exitError;
            frozen.hasOutput = hasOutput;
            frozen.parsed = self.parsed;
            frozen.parseFn = parseFn;
            frozen.parseContext = parseContext;
            frozen.handlerFinishCommand = handlerFinishCommand;
        }
        function unfreeze() {
            const frozen = frozens.pop();
            options = frozen.options;
            options.configObjects = frozen.configObjects;
            exitProcess = frozen.exitProcess;
            groups = frozen.groups;
            output = frozen.output;
            exitError = frozen.exitError;
            hasOutput = frozen.hasOutput;
            self.parsed = frozen.parsed;
            usage.unfreeze();
            validation.unfreeze();
            command.unfreeze();
            strict = frozen.strict;
            strictCommands = frozen.strictCommands;
            completionCommand = frozen.completionCommand;
            parseFn = frozen.parseFn;
            parseContext = frozen.parseContext;
            handlerFinishCommand = frozen.handlerFinishCommand;
        }
        self.boolean = function (keys) {
            argsert('<array|string>', [keys], arguments.length);
            populateParserHintArray('boolean', keys);
            return self;
        };
        self.array = function (keys) {
            argsert('<array|string>', [keys], arguments.length);
            populateParserHintArray('array', keys);
            return self;
        };
        self.number = function (keys) {
            argsert('<array|string>', [keys], arguments.length);
            populateParserHintArray('number', keys);
            return self;
        };
        self.normalize = function (keys) {
            argsert('<array|string>', [keys], arguments.length);
            populateParserHintArray('normalize', keys);
            return self;
        };
        self.count = function (keys) {
            argsert('<array|string>', [keys], arguments.length);
            populateParserHintArray('count', keys);
            return self;
        };
        self.string = function (keys) {
            argsert('<array|string>', [keys], arguments.length);
            populateParserHintArray('string', keys);
            return self;
        };
        self.requiresArg = function (keys, value) {
            argsert('<array|string|object> [number]', [keys], arguments.length);
            // If someone configures nargs at the same time as requiresArg,
            // nargs should take precedent,
            // see: https://github.com/yargs/yargs/pull/1572
            // TODO: make this work with aliases, using a check similar to
            // checkAllAliases() in yargs-parser.
            if (typeof keys === 'string' && options.narg[keys]) {
                return self;
            }
            else {
                populateParserHintObject(self.requiresArg, false, 'narg', keys, NaN);
            }
            return self;
        };
        self.skipValidation = function (keys) {
            argsert('<array|string>', [keys], arguments.length);
            populateParserHintArray('skipValidation', keys);
            return self;
        };
        function populateParserHintArray(type, keys, value) {
            keys = [].concat(keys);
            keys.forEach((key) => {
                key = sanitizeKey(key);
                options[type].push(key);
            });
        }
        self.nargs = function (key, value) {
            argsert('<string|object|array> [number]', [key, value], arguments.length);
            populateParserHintObject(self.nargs, false, 'narg', key, value);
            return self;
        };
        self.choices = function (key, value) {
            argsert('<object|string|array> [string|array]', [key, value], arguments.length);
            populateParserHintObject(self.choices, true, 'choices', key, value);
            return self;
        };
        self.alias = function (key, value) {
            argsert('<object|string|array> [string|array]', [key, value], arguments.length);
            populateParserHintObject(self.alias, true, 'alias', key, value);
            return self;
        };
        // TODO: actually deprecate self.defaults.
        self.default = self.defaults = function (key, value, defaultDescription) {
            argsert('<object|string|array> [*] [string]', [key, value, defaultDescription], arguments.length);
            if (defaultDescription)
                options.defaultDescription[key] = defaultDescription;
            if (typeof value === 'function') {
                if (!options.defaultDescription[key])
                    options.defaultDescription[key] = usage.functionDescription(value);
                value = value.call();
            }
            populateParserHintObject(self.default, false, 'default', key, value);
            return self;
        };
        self.describe = function (key, desc) {
            argsert('<object|string|array> [string]', [key, desc], arguments.length);
            populateParserHintObject(self.describe, false, 'key', key, true);
            usage.describe(key, desc);
            return self;
        };
        self.demandOption = function (keys, msg) {
            argsert('<object|string|array> [string]', [keys, msg], arguments.length);
            populateParserHintObject(self.demandOption, false, 'demandedOptions', keys, msg);
            return self;
        };
        self.coerce = function (keys, value) {
            argsert('<object|string|array> [function]', [keys, value], arguments.length);
            populateParserHintObject(self.coerce, false, 'coerce', keys, value);
            return self;
        };
        function populateParserHintObject(builder, isArray, type, key, value) {
            if (Array.isArray(key)) {
                const temp = Object.create(null);
                // an array of keys with one value ['x', 'y', 'z'], function parse () {}
                key.forEach((k) => {
                    temp[k] = value;
                });
                builder(temp);
            }
            else if (typeof key === 'object') {
                // an object of key value pairs: {'x': parse () {}, 'y': parse() {}}
                Object.keys(key).forEach((k) => {
                    builder(k, key[k]);
                });
            }
            else {
                key = sanitizeKey(key);
                // a single key value pair 'x', parse() {}
                if (isArray) {
                    options[type][key] = (options[type][key] || []).concat(value);
                }
                else {
                    options[type][key] = value;
                }
            }
        }
        // TODO(bcoe): in future major versions move more objects towards
        // Object.create(null):
        function sanitizeKey(key) {
            if (key === '__proto__')
                return '___proto___';
            return key;
        }
        function deleteFromParserHintObject(optionKey) {
            // delete from all parsing hints:
            // boolean, array, key, alias, etc.
            Object.keys(options).forEach((hintKey) => {
                const hint = options[hintKey];
                if (Array.isArray(hint)) {
                    if (~hint.indexOf(optionKey))
                        hint.splice(hint.indexOf(optionKey), 1);
                }
                else if (typeof hint === 'object') {
                    delete hint[optionKey];
                }
            });
            // now delete the description from usage.js.
            delete usage.getDescriptions()[optionKey];
        }
        self.config = function config(key, msg, parseFn) {
            argsert('[object|string] [string|function] [function]', [key, msg, parseFn], arguments.length);
            // allow a config object to be provided directly.
            if (typeof key === 'object') {
                key = applyExtends(key, cwd, self.getParserConfiguration()['deep-merge-config']);
                options.configObjects = (options.configObjects || []).concat(key);
                return self;
            }
            // allow for a custom parsing function.
            if (typeof msg === 'function') {
                parseFn = msg;
                msg = null;
            }
            key = key || 'config';
            self.describe(key, msg || usage.deferY18nLookup('Path to JSON config file'));
            (Array.isArray(key) ? key : [key]).forEach((k) => {
                options.config[k] = parseFn || true;
            });
            return self;
        };
        self.example = function (cmd, description) {
            argsert('<string> [string]', [cmd, description], arguments.length);
            usage.example(cmd, description);
            return self;
        };
        self.command = function (cmd, description, builder, handler, middlewares) {
            argsert('<string|array|object> [string|boolean] [function|object] [function] [array]', [cmd, description, builder, handler, middlewares], arguments.length);
            command.addHandler(cmd, description, builder, handler, middlewares);
            return self;
        };
        self.commandDir = function (dir, opts) {
            argsert('<string> [object]', [dir, opts], arguments.length);
            const req = parentRequire || require;
            command.addDirectory(dir, self.getContext(), req, require(35) /* get-caller-file */(), opts);
            return self;
        };
        // TODO: deprecate self.demand in favor of
        // .demandCommand() .demandOption().
        self.demand = self.required = self.require = function demand(keys, max, msg) {
            // you can optionally provide a 'max' key,
            // which will raise an exception if too many '_'
            // options are provided.
            if (Array.isArray(max)) {
                max.forEach((key) => {
                    self.demandOption(key, msg);
                });
                max = Infinity;
            }
            else if (typeof max !== 'number') {
                msg = max;
                max = Infinity;
            }
            if (typeof keys === 'number') {
                self.demandCommand(keys, max, msg, msg);
            }
            else if (Array.isArray(keys)) {
                keys.forEach((key) => {
                    self.demandOption(key, msg);
                });
            }
            else {
                if (typeof msg === 'string') {
                    self.demandOption(keys, msg);
                }
                else if (msg === true || typeof msg === 'undefined') {
                    self.demandOption(keys);
                }
            }
            return self;
        };
        self.demandCommand = function demandCommand(min, max, minMsg, maxMsg) {
            argsert('[number] [number|string] [string|null|undefined] [string|null|undefined]', [min, max, minMsg, maxMsg], arguments.length);
            if (typeof min === 'undefined')
                min = 1;
            if (typeof max !== 'number') {
                minMsg = max;
                max = Infinity;
            }
            self.global('_', false);
            options.demandedCommands._ = {
                min,
                max,
                minMsg,
                maxMsg
            };
            return self;
        };
        self.getDemandedOptions = () => {
            argsert([], 0);
            return options.demandedOptions;
        };
        self.getDemandedCommands = () => {
            argsert([], 0);
            return options.demandedCommands;
        };
        self.deprecateOption = function deprecateOption(option, message) {
            argsert('<string> [string|boolean]', [option, message], arguments.length);
            options.deprecatedOptions[option] = message;
            return self;
        };
        self.getDeprecatedOptions = () => {
            argsert([], 0);
            return options.deprecatedOptions;
        };
        self.implies = function (key, value) {
            argsert('<string|object> [number|string|array]', [key, value], arguments.length);
            validation.implies(key, value);
            return self;
        };
        self.conflicts = function (key1, key2) {
            argsert('<string|object> [string|array]', [key1, key2], arguments.length);
            validation.conflicts(key1, key2);
            return self;
        };
        self.usage = function (msg, description, builder, handler) {
            argsert('<string|null|undefined> [string|boolean] [function|object] [function]', [msg, description, builder, handler], arguments.length);
            if (description !== undefined) {
                // .usage() can be used as an alias for defining
                // a default command.
                if ((msg || '').match(/^\$0( |$)/)) {
                    return self.command(msg, description, builder, handler);
                }
                else {
                    throw new YError('.usage() description must start with $0 if being used as alias for .command()');
                }
            }
            else {
                usage.usage(msg);
                return self;
            }
        };
        self.epilogue = self.epilog = function (msg) {
            argsert('<string>', [msg], arguments.length);
            usage.epilog(msg);
            return self;
        };
        self.fail = function (f) {
            argsert('<function>', [f], arguments.length);
            usage.failFn(f);
            return self;
        };
        self.onFinishCommand = function (f) {
            argsert('<function>', [f], arguments.length);
            handlerFinishCommand = f;
            return self;
        };
        self.getHandlerFinishCommand = () => handlerFinishCommand;
        self.check = function (f, _global) {
            argsert('<function> [boolean]', [f, _global], arguments.length);
            validation.check(f, _global !== false);
            return self;
        };
        self.global = function global(globals, global) {
            argsert('<string|array> [boolean]', [globals, global], arguments.length);
            globals = [].concat(globals);
            if (global !== false) {
                options.local = options.local.filter(l => globals.indexOf(l) === -1);
            }
            else {
                globals.forEach((g) => {
                    if (options.local.indexOf(g) === -1)
                        options.local.push(g);
                });
            }
            return self;
        };
        self.pkgConf = function pkgConf(key, rootPath) {
            argsert('<string> [string]', [key, rootPath], arguments.length);
            let conf = null;
            // prefer cwd to require-main-filename in this method
            // since we're looking for e.g. "nyc" config in nyc consumer
            // rather than "yargs" config in nyc (where nyc is the main filename)
            const obj = pkgUp(rootPath || cwd);
            // If an object exists in the key, add it to options.configObjects
            if (obj[key] && typeof obj[key] === 'object') {
                conf = applyExtends(obj[key], rootPath || cwd, self.getParserConfiguration()['deep-merge-config']);
                options.configObjects = (options.configObjects || []).concat(conf);
            }
            return self;
        };
        const pkgs = {};
        function pkgUp(rootPath) {
            const npath = rootPath || '*';
            if (pkgs[npath])
                return pkgs[npath];
            const findUp = require(36) /* find-up */;
            let obj = {};
            try {
                let startDir = rootPath || require(42) /* require-main-filename */(parentRequire || require);
                // When called in an environment that lacks require.main.filename, such as a jest test runner,
                // startDir is already process.cwd(), and should not be shortened.
                // Whether or not it is _actually_ a directory (e.g., extensionless bin) is irrelevant, find-up handles it.
                if (!rootPath && path.extname(startDir)) {
                    startDir = path.dirname(startDir);
                }
                const pkgJsonPath = findUp.sync('package.json', {
                    cwd: startDir
                });
                obj = JSON.parse(fs.readFileSync(pkgJsonPath));
            }
            catch (noop) { }
            pkgs[npath] = obj || {};
            return pkgs[npath];
        }
        let parseFn = null;
        let parseContext = null;
        self.parse = function parse(args, shortCircuit, _parseFn) {
            argsert('[string|array] [function|boolean|object] [function]', [args, shortCircuit, _parseFn], arguments.length);
            freeze();
            if (typeof args === 'undefined') {
                const argv = self._parseArgs(processArgs);
                const tmpParsed = self.parsed;
                unfreeze();
                // TODO: remove this compatibility hack when we release yargs@15.x:
                self.parsed = tmpParsed;
                return argv;
            }
            // a context object can optionally be provided, this allows
            // additional information to be passed to a command handler.
            if (typeof shortCircuit === 'object') {
                parseContext = shortCircuit;
                shortCircuit = _parseFn;
            }
            // by providing a function as a second argument to
            // parse you can capture output that would otherwise
            // default to printing to stdout/stderr.
            if (typeof shortCircuit === 'function') {
                parseFn = shortCircuit;
                shortCircuit = null;
            }
            // completion short-circuits the parsing process,
            // skipping validation, etc.
            if (!shortCircuit)
                processArgs = args;
            if (parseFn)
                exitProcess = false;
            const parsed = self._parseArgs(args, shortCircuit);
            completion.setParsed(self.parsed);
            if (parseFn)
                parseFn(exitError, parsed, output);
            unfreeze();
            return parsed;
        };
        self._getParseContext = () => parseContext || {};
        self._hasParseCallback = () => !!parseFn;
        self.option = self.options = function option(key, opt) {
            argsert('<string|object> [object]', [key, opt], arguments.length);
            if (typeof key === 'object') {
                Object.keys(key).forEach((k) => {
                    self.options(k, key[k]);
                });
            }
            else {
                if (typeof opt !== 'object') {
                    opt = {};
                }
                options.key[key] = true; // track manually set keys.
                if (opt.alias)
                    self.alias(key, opt.alias);
                const deprecate = opt.deprecate || opt.deprecated;
                if (deprecate) {
                    self.deprecateOption(key, deprecate);
                }
                const demand = opt.demand || opt.required || opt.require;
                // A required option can be specified via "demand: true".
                if (demand) {
                    self.demand(key, demand);
                }
                if (opt.demandOption) {
                    self.demandOption(key, typeof opt.demandOption === 'string' ? opt.demandOption : undefined);
                }
                if ('conflicts' in opt) {
                    self.conflicts(key, opt.conflicts);
                }
                if ('default' in opt) {
                    self.default(key, opt.default);
                }
                if ('implies' in opt) {
                    self.implies(key, opt.implies);
                }
                if ('nargs' in opt) {
                    self.nargs(key, opt.nargs);
                }
                if (opt.config) {
                    self.config(key, opt.configParser);
                }
                if (opt.normalize) {
                    self.normalize(key);
                }
                if ('choices' in opt) {
                    self.choices(key, opt.choices);
                }
                if ('coerce' in opt) {
                    self.coerce(key, opt.coerce);
                }
                if ('group' in opt) {
                    self.group(key, opt.group);
                }
                if (opt.boolean || opt.type === 'boolean') {
                    self.boolean(key);
                    if (opt.alias)
                        self.boolean(opt.alias);
                }
                if (opt.array || opt.type === 'array') {
                    self.array(key);
                    if (opt.alias)
                        self.array(opt.alias);
                }
                if (opt.number || opt.type === 'number') {
                    self.number(key);
                    if (opt.alias)
                        self.number(opt.alias);
                }
                if (opt.string || opt.type === 'string') {
                    self.string(key);
                    if (opt.alias)
                        self.string(opt.alias);
                }
                if (opt.count || opt.type === 'count') {
                    self.count(key);
                }
                if (typeof opt.global === 'boolean') {
                    self.global(key, opt.global);
                }
                if (opt.defaultDescription) {
                    options.defaultDescription[key] = opt.defaultDescription;
                }
                if (opt.skipValidation) {
                    self.skipValidation(key);
                }
                const desc = opt.describe || opt.description || opt.desc;
                self.describe(key, desc);
                if (opt.hidden) {
                    self.hide(key);
                }
                if (opt.requiresArg) {
                    self.requiresArg(key);
                }
            }
            return self;
        };
        self.getOptions = () => options;
        self.positional = function (key, opts) {
            argsert('<string> <object>', [key, opts], arguments.length);
            if (context.resets === 0) {
                throw new YError(".positional() can only be called in a command's builder function");
            }
            // .positional() only supports a subset of the configuration
            // options available to .option().
            const supportedOpts = ['default', 'defaultDescription', 'implies', 'normalize',
                'choices', 'conflicts', 'coerce', 'type', 'describe',
                'desc', 'description', 'alias'];
            opts = objFilter(opts, (k, v) => {
                let accept = supportedOpts.indexOf(k) !== -1;
                // type can be one of string|number|boolean.
                if (k === 'type' && ['string', 'number', 'boolean'].indexOf(v) === -1)
                    accept = false;
                return accept;
            });
            // copy over any settings that can be inferred from the command string.
            const fullCommand = context.fullCommands[context.fullCommands.length - 1];
            const parseOptions = fullCommand ? command.cmdToParseOptions(fullCommand) : {
                array: [],
                alias: {},
                default: {},
                demand: {}
            };
            Object.keys(parseOptions).forEach((pk) => {
                if (Array.isArray(parseOptions[pk])) {
                    if (parseOptions[pk].indexOf(key) !== -1)
                        opts[pk] = true;
                }
                else {
                    if (parseOptions[pk][key] && !(pk in opts))
                        opts[pk] = parseOptions[pk][key];
                }
            });
            self.group(key, usage.getPositionalGroupName());
            return self.option(key, opts);
        };
        self.group = function group(opts, groupName) {
            argsert('<string|array> <string>', [opts, groupName], arguments.length);
            const existing = preservedGroups[groupName] || groups[groupName];
            if (preservedGroups[groupName]) {
                // we now only need to track this group name in groups.
                delete preservedGroups[groupName];
            }
            const seen = {};
            groups[groupName] = (existing || []).concat(opts).filter((key) => {
                if (seen[key])
                    return false;
                return (seen[key] = true);
            });
            return self;
        };
        // combine explicit and preserved groups. explicit groups should be first
        self.getGroups = () => Object.assign({}, groups, preservedGroups);
        // as long as options.envPrefix is not undefined,
        // parser will apply env vars matching prefix to argv
        self.env = function (prefix) {
            argsert('[string|boolean]', [prefix], arguments.length);
            if (prefix === false)
                options.envPrefix = undefined;
            else
                options.envPrefix = prefix || '';
            return self;
        };
        self.wrap = function (cols) {
            argsert('<number|null|undefined>', [cols], arguments.length);
            usage.wrap(cols);
            return self;
        };
        let strict = false;
        self.strict = function (enabled) {
            argsert('[boolean]', [enabled], arguments.length);
            strict = enabled !== false;
            return self;
        };
        self.getStrict = () => strict;
        let strictCommands = false;
        self.strictCommands = function (enabled) {
            argsert('[boolean]', [enabled], arguments.length);
            strictCommands = enabled !== false;
            return self;
        };
        self.getStrictCommands = () => strictCommands;
        let parserConfig = {};
        self.parserConfiguration = function parserConfiguration(config) {
            argsert('<object>', [config], arguments.length);
            parserConfig = config;
            return self;
        };
        self.getParserConfiguration = () => parserConfig;
        self.showHelp = function (level) {
            argsert('[string|function]', [level], arguments.length);
            if (!self.parsed)
                self._parseArgs(processArgs); // run parser, if it has not already been executed.
            if (command.hasDefaultCommand()) {
                context.resets++; // override the restriction on top-level positoinals.
                command.runDefaultBuilderOn(self, true);
            }
            usage.showHelp(level);
            return self;
        };
        let versionOpt = null;
        self.version = function version(opt, msg, ver) {
            const defaultVersionOpt = 'version';
            argsert('[boolean|string] [string] [string]', [opt, msg, ver], arguments.length);
            // nuke the key previously configured
            // to return version #.
            if (versionOpt) {
                deleteFromParserHintObject(versionOpt);
                usage.version(undefined);
                versionOpt = null;
            }
            if (arguments.length === 0) {
                ver = guessVersion();
                opt = defaultVersionOpt;
            }
            else if (arguments.length === 1) {
                if (opt === false) { // disable default 'version' key.
                    return self;
                }
                ver = opt;
                opt = defaultVersionOpt;
            }
            else if (arguments.length === 2) {
                ver = msg;
                msg = null;
            }
            versionOpt = typeof opt === 'string' ? opt : defaultVersionOpt;
            msg = msg || usage.deferY18nLookup('Show version number');
            usage.version(ver || undefined);
            self.boolean(versionOpt);
            self.describe(versionOpt, msg);
            return self;
        };
        function guessVersion() {
            const obj = pkgUp();
            return obj.version || 'unknown';
        }
        let helpOpt = null;
        self.addHelpOpt = self.help = function addHelpOpt(opt, msg) {
            const defaultHelpOpt = 'help';
            argsert('[string|boolean] [string]', [opt, msg], arguments.length);
            // nuke the key previously configured
            // to return help.
            if (helpOpt) {
                deleteFromParserHintObject(helpOpt);
                helpOpt = null;
            }
            if (arguments.length === 1) {
                if (opt === false)
                    return self;
            }
            // use arguments, fallback to defaults for opt and msg
            helpOpt = typeof opt === 'string' ? opt : defaultHelpOpt;
            self.boolean(helpOpt);
            self.describe(helpOpt, msg || usage.deferY18nLookup('Show help'));
            return self;
        };
        const defaultShowHiddenOpt = 'show-hidden';
        options.showHiddenOpt = defaultShowHiddenOpt;
        self.addShowHiddenOpt = self.showHidden = function addShowHiddenOpt(opt, msg) {
            argsert('[string|boolean] [string]', [opt, msg], arguments.length);
            if (arguments.length === 1) {
                if (opt === false)
                    return self;
            }
            const showHiddenOpt = typeof opt === 'string' ? opt : defaultShowHiddenOpt;
            self.boolean(showHiddenOpt);
            self.describe(showHiddenOpt, msg || usage.deferY18nLookup('Show hidden options'));
            options.showHiddenOpt = showHiddenOpt;
            return self;
        };
        self.hide = function hide(key) {
            argsert('<string|object>', [key], arguments.length);
            options.hiddenOptions.push(key);
            return self;
        };
        self.showHelpOnFail = function showHelpOnFail(enabled, message) {
            argsert('[boolean|string] [string]', [enabled, message], arguments.length);
            usage.showHelpOnFail(enabled, message);
            return self;
        };
        var exitProcess = true;
        self.exitProcess = function (enabled) {
            argsert('[boolean]', [enabled], arguments.length);
            if (typeof enabled !== 'boolean') {
                enabled = true;
            }
            exitProcess = enabled;
            return self;
        };
        self.getExitProcess = () => exitProcess;
        var completionCommand = null;
        self.completion = function (cmd, desc, fn) {
            argsert('[string] [string|boolean|function] [function]', [cmd, desc, fn], arguments.length);
            // a function to execute when generating
            // completions can be provided as the second
            // or third argument to completion.
            if (typeof desc === 'function') {
                fn = desc;
                desc = null;
            }
            // register the completion command.
            completionCommand = cmd || completionCommand || 'completion';
            if (!desc && desc !== false) {
                desc = 'generate completion script';
            }
            self.command(completionCommand, desc);
            // a function can be provided
            if (fn)
                completion.registerFunction(fn);
            return self;
        };
        self.showCompletionScript = function ($0, cmd) {
            argsert('[string] [string]', [$0, cmd], arguments.length);
            $0 = $0 || self.$0;
            _logger.log(completion.generateCompletionScript($0, cmd || completionCommand || 'completion'));
            return self;
        };
        self.getCompletion = function (args, done) {
            argsert('<array> <function>', [args, done], arguments.length);
            completion.getCompletion(args, done);
        };
        self.locale = function (locale) {
            argsert('[string]', [locale], arguments.length);
            if (arguments.length === 0) {
                guessLocale();
                return y18n.getLocale();
            }
            detectLocale = false;
            y18n.setLocale(locale);
            return self;
        };
        self.updateStrings = self.updateLocale = function (obj) {
            argsert('<object>', [obj], arguments.length);
            detectLocale = false;
            y18n.updateLocale(obj);
            return self;
        };
        let detectLocale = true;
        self.detectLocale = function (detect) {
            argsert('<boolean>', [detect], arguments.length);
            detectLocale = detect;
            return self;
        };
        self.getDetectLocale = () => detectLocale;
        var hasOutput = false;
        var exitError = null;
        // maybe exit, always capture
        // context about why we wanted to exit.
        self.exit = (code, err) => {
            hasOutput = true;
            exitError = err;
            if (exitProcess)
                process.exit(code);
        };
        // we use a custom logger that buffers output,
        // so that we can print to non-CLIs, e.g., chat-bots.
        const _logger = {
            log() {
                const args = [];
                for (let i = 0; i < arguments.length; i++)
                    args.push(arguments[i]);
                if (!self._hasParseCallback())
                    console.log.apply(console, args);
                hasOutput = true;
                if (output.length)
                    output += '\n';
                output += args.join(' ');
            },
            error() {
                const args = [];
                for (let i = 0; i < arguments.length; i++)
                    args.push(arguments[i]);
                if (!self._hasParseCallback())
                    console.error.apply(console, args);
                hasOutput = true;
                if (output.length)
                    output += '\n';
                output += args.join(' ');
            }
        };
        self._getLoggerInstance = () => _logger;
        // has yargs output an error our help
        // message in the current execution context.
        self._hasOutput = () => hasOutput;
        self._setHasOutput = () => {
            hasOutput = true;
        };
        let recommendCommands;
        self.recommendCommands = function (recommend) {
            argsert('[boolean]', [recommend], arguments.length);
            recommendCommands = typeof recommend === 'boolean' ? recommend : true;
            return self;
        };
        self.getUsageInstance = () => usage;
        self.getValidationInstance = () => validation;
        self.getCommandInstance = () => command;
        self.terminalWidth = () => {
            argsert([], 0);
            return typeof process.stdout.columns !== 'undefined' ? process.stdout.columns : null;
        };
        Object.defineProperty(self, 'argv', {
            get: () => self._parseArgs(processArgs),
            enumerable: true
        });
        self._parseArgs = function parseArgs(args, shortCircuit, _calledFromCommand, commandIndex) {
            let skipValidation = !!_calledFromCommand;
            args = args || processArgs;
            options.__ = y18n.__;
            options.configuration = self.getParserConfiguration();
            const populateDoubleDash = !!options.configuration['populate--'];
            const config = Object.assign({}, options.configuration, {
                'populate--': true
            });
            const parsed = Parser.detailed(args, Object.assign({}, options, {
                configuration: config
            }));
            let argv = parsed.argv;
            if (parseContext)
                argv = Object.assign({}, argv, parseContext);
            const aliases = parsed.aliases;
            argv.$0 = self.$0;
            self.parsed = parsed;
            try {
                guessLocale(); // guess locale lazily, so that it can be turned off in chain.
                // while building up the argv object, there
                // are two passes through the parser. If completion
                // is being performed short-circuit on the first pass.
                if (shortCircuit) {
                    return (populateDoubleDash || _calledFromCommand) ? argv : self._copyDoubleDash(argv);
                }
                // if there's a handler associated with a
                // command defer processing to it.
                if (helpOpt) {
                    // consider any multi-char helpOpt alias as a valid help command
                    // unless all helpOpt aliases are single-char
                    // note that parsed.aliases is a normalized bidirectional map :)
                    const helpCmds = [helpOpt]
                        .concat(aliases[helpOpt] || [])
                        .filter(k => k.length > 1);
                    // check if help should trigger and strip it from _.
                    if (~helpCmds.indexOf(argv._[argv._.length - 1])) {
                        argv._.pop();
                        argv[helpOpt] = true;
                    }
                }
                const handlerKeys = command.getCommands();
                const requestCompletions = completion.completionKey in argv;
                const skipRecommendation = argv[helpOpt] || requestCompletions;
                const skipDefaultCommand = skipRecommendation && (handlerKeys.length > 1 || handlerKeys[0] !== '$0');
                if (argv._.length) {
                    if (handlerKeys.length) {
                        let firstUnknownCommand;
                        for (let i = (commandIndex || 0), cmd; argv._[i] !== undefined; i++) {
                            cmd = String(argv._[i]);
                            if (~handlerKeys.indexOf(cmd) && cmd !== completionCommand) {
                                // commands are executed using a recursive algorithm that executes
                                // the deepest command first; we keep track of the position in the
                                // argv._ array that is currently being executed.
                                const innerArgv = command.runCommand(cmd, self, parsed, i + 1);
                                return populateDoubleDash ? innerArgv : self._copyDoubleDash(innerArgv);
                            }
                            else if (!firstUnknownCommand && cmd !== completionCommand) {
                                firstUnknownCommand = cmd;
                                break;
                            }
                        }
                        // run the default command, if defined
                        if (command.hasDefaultCommand() && !skipDefaultCommand) {
                            const innerArgv = command.runCommand(null, self, parsed);
                            return populateDoubleDash ? innerArgv : self._copyDoubleDash(innerArgv);
                        }
                        // recommend a command if recommendCommands() has
                        // been enabled, and no commands were found to execute
                        if (recommendCommands && firstUnknownCommand && !skipRecommendation) {
                            validation.recommendCommands(firstUnknownCommand, handlerKeys);
                        }
                    }
                    // generate a completion script for adding to ~/.bashrc.
                    if (completionCommand && ~argv._.indexOf(completionCommand) && !requestCompletions) {
                        if (exitProcess)
                            setBlocking(true);
                        self.showCompletionScript();
                        self.exit(0);
                    }
                }
                else if (command.hasDefaultCommand() && !skipDefaultCommand) {
                    const innerArgv = command.runCommand(null, self, parsed);
                    return populateDoubleDash ? innerArgv : self._copyDoubleDash(innerArgv);
                }
                // we must run completions first, a user might
                // want to complete the --help or --version option.
                if (requestCompletions) {
                    if (exitProcess)
                        setBlocking(true);
                    // we allow for asynchronous completions,
                    // e.g., loading in a list of commands from an API.
                    const completionArgs = args.slice(args.indexOf(`--${completion.completionKey}`) + 1);
                    completion.getCompletion(completionArgs, (completions) => {
                        ;
                        (completions || []).forEach((completion) => {
                            _logger.log(completion);
                        });
                        self.exit(0);
                    });
                    return (populateDoubleDash || _calledFromCommand) ? argv : self._copyDoubleDash(argv);
                }
                // Handle 'help' and 'version' options
                // if we haven't already output help!
                if (!hasOutput) {
                    Object.keys(argv).forEach((key) => {
                        if (key === helpOpt && argv[key]) {
                            if (exitProcess)
                                setBlocking(true);
                            skipValidation = true;
                            self.showHelp('log');
                            self.exit(0);
                        }
                        else if (key === versionOpt && argv[key]) {
                            if (exitProcess)
                                setBlocking(true);
                            skipValidation = true;
                            usage.showVersion();
                            self.exit(0);
                        }
                    });
                }
                // Check if any of the options to skip validation were provided
                if (!skipValidation && options.skipValidation.length > 0) {
                    skipValidation = Object.keys(argv).some(key => options.skipValidation.indexOf(key) >= 0 && argv[key] === true);
                }
                // If the help or version options where used and exitProcess is false,
                // or if explicitly skipped, we won't run validations.
                if (!skipValidation) {
                    if (parsed.error)
                        throw new YError(parsed.error.message);
                    // if we're executed via bash completion, don't
                    // bother with validation.
                    if (!requestCompletions) {
                        self._runValidation(argv, aliases, {}, parsed.error);
                    }
                }
            }
            catch (err) {
                if (err instanceof YError)
                    usage.fail(err.message, err);
                else
                    throw err;
            }
            return (populateDoubleDash || _calledFromCommand) ? argv : self._copyDoubleDash(argv);
        };
        // to simplify the parsing of positionals in commands,
        // we temporarily populate '--' rather than _, with arguments
        // after the '--' directive. After the parse, we copy these back.
        self._copyDoubleDash = function (argv) {
            if (!argv._ || !argv['--'])
                return argv;
            argv._.push.apply(argv._, argv['--']);
            // TODO(bcoe): refactor command parsing such that this delete is not
            // necessary: https://github.com/yargs/yargs/issues/1482
            try {
                delete argv['--'];
            }
            catch (_err) { }
            return argv;
        };
        self._runValidation = function runValidation(argv, aliases, positionalMap, parseErrors) {
            if (parseErrors)
                throw new YError(parseErrors.message);
            validation.nonOptionCount(argv);
            validation.requiredArguments(argv);
            let failedStrictCommands = false;
            if (strictCommands) {
                failedStrictCommands = validation.unknownCommands(argv, aliases, positionalMap);
            }
            if (strict && !failedStrictCommands) {
                validation.unknownArguments(argv, aliases, positionalMap);
            }
            validation.customChecks(argv, aliases);
            validation.limitedChoices(argv);
            validation.implications(argv);
            validation.conflicting(argv);
        };
        function guessLocale() {
            if (!detectLocale)
                return;
            const locale = process.env.LC_ALL || process.env.LC_MESSAGES || process.env.LANG || process.env.LANGUAGE || 'en_US';
            self.locale(locale.replace(/[.:].*/, ''));
        }
        // an app should almost always have --version and --help,
        // if you *really* want to disable this use .help(false)/.version(false).
        self.help();
        self.version();
        return self;
    }
    // allow consumers to directly use the version of yargs-parser used by yargs
    exports.Parser = Parser;
    // rebase an absolute path to a relative one with respect to a base directory
    // exported for tests
    exports.rebase = rebase;
    function rebase(base, dir) {
        return path.relative(base, dir);
    }
},
/* yargs/lib/argsert.js */ function _(require, module, exports) {
    // hoisted due to circular dependency on command.
    module.exports = argsert;
    const command = require(4) /* ./command */();
    const YError = require(13) /* ./yerror */;
    const positionName = ['first', 'second', 'third', 'fourth', 'fifth', 'sixth'];
    function argsert(expected, callerArguments, length) {
        // TODO: should this eventually raise an exception.
        try {
            // preface the argument description with "cmd", so
            // that we can run it through yargs' command parser.
            let position = 0;
            let parsed = { demanded: [], optional: [] };
            if (typeof expected === 'object') {
                length = callerArguments;
                callerArguments = expected;
            }
            else {
                parsed = command.parseCommand(`cmd ${expected}`);
            }
            const args = [].slice.call(callerArguments);
            while (args.length && args[args.length - 1] === undefined)
                args.pop();
            length = length || args.length;
            if (length < parsed.demanded.length) {
                throw new YError(`Not enough arguments provided. Expected ${parsed.demanded.length} but received ${args.length}.`);
            }
            const totalCommands = parsed.demanded.length + parsed.optional.length;
            if (length > totalCommands) {
                throw new YError(`Too many arguments provided. Expected max ${totalCommands} but received ${length}.`);
            }
            parsed.demanded.forEach((demanded) => {
                const arg = args.shift();
                const observedType = guessType(arg);
                const matchingTypes = demanded.cmd.filter(type => type === observedType || type === '*');
                if (matchingTypes.length === 0)
                    argumentTypeError(observedType, demanded.cmd, position, false);
                position += 1;
            });
            parsed.optional.forEach((optional) => {
                if (args.length === 0)
                    return;
                const arg = args.shift();
                const observedType = guessType(arg);
                const matchingTypes = optional.cmd.filter(type => type === observedType || type === '*');
                if (matchingTypes.length === 0)
                    argumentTypeError(observedType, optional.cmd, position, true);
                position += 1;
            });
        }
        catch (err) {
            console.warn(err.stack);
        }
    }
    function guessType(arg) {
        if (Array.isArray(arg)) {
            return 'array';
        }
        else if (arg === null) {
            return 'null';
        }
        return typeof arg;
    }
    function argumentTypeError(observedType, allowedTypes, position, optional) {
        throw new YError(`Invalid ${positionName[position] || 'manyith'} argument. Expected ${allowedTypes.join(' or ')} but received ${observedType}.`);
    }
},
/* yargs/lib/command.js */ function _(require, module, exports) {
    const inspect = require('util').inspect;
    const isPromise = require(5) /* ./is-promise */;
    const { applyMiddleware, commandMiddlewareFactory } = require(6) /* ./middleware */;
    const path = require('path');
    const Parser = require(7) /* yargs-parser */;
    const DEFAULT_MARKER = /(^\*)|(^\$0)/;
    // handles parsing positional arguments,
    // and populating argv with said positional
    // arguments.
    module.exports = function command(yargs, usage, validation, globalMiddleware) {
        const self = {};
        let handlers = {};
        let aliasMap = {};
        let defaultCommand;
        globalMiddleware = globalMiddleware || [];
        self.addHandler = function addHandler(cmd, description, builder, handler, commandMiddleware) {
            let aliases = [];
            const middlewares = commandMiddlewareFactory(commandMiddleware);
            handler = handler || (() => { });
            if (Array.isArray(cmd)) {
                aliases = cmd.slice(1);
                cmd = cmd[0];
            }
            else if (typeof cmd === 'object') {
                let command = (Array.isArray(cmd.command) || typeof cmd.command === 'string') ? cmd.command : moduleName(cmd);
                if (cmd.aliases)
                    command = [].concat(command).concat(cmd.aliases);
                self.addHandler(command, extractDesc(cmd), cmd.builder, cmd.handler, cmd.middlewares);
                return;
            }
            // allow a module to be provided instead of separate builder and handler
            if (typeof builder === 'object' && builder.builder && typeof builder.handler === 'function') {
                self.addHandler([cmd].concat(aliases), description, builder.builder, builder.handler, builder.middlewares);
                return;
            }
            // parse positionals out of cmd string
            const parsedCommand = self.parseCommand(cmd);
            // remove positional args from aliases only
            aliases = aliases.map(alias => self.parseCommand(alias).cmd);
            // check for default and filter out '*''
            let isDefault = false;
            const parsedAliases = [parsedCommand.cmd].concat(aliases).filter((c) => {
                if (DEFAULT_MARKER.test(c)) {
                    isDefault = true;
                    return false;
                }
                return true;
            });
            // standardize on $0 for default command.
            if (parsedAliases.length === 0 && isDefault)
                parsedAliases.push('$0');
            // shift cmd and aliases after filtering out '*'
            if (isDefault) {
                parsedCommand.cmd = parsedAliases[0];
                aliases = parsedAliases.slice(1);
                cmd = cmd.replace(DEFAULT_MARKER, parsedCommand.cmd);
            }
            // populate aliasMap
            aliases.forEach((alias) => {
                aliasMap[alias] = parsedCommand.cmd;
            });
            if (description !== false) {
                usage.command(cmd, description, isDefault, aliases);
            }
            handlers[parsedCommand.cmd] = {
                original: cmd,
                description: description,
                handler,
                builder: builder || {},
                middlewares,
                demanded: parsedCommand.demanded,
                optional: parsedCommand.optional
            };
            if (isDefault)
                defaultCommand = handlers[parsedCommand.cmd];
        };
        self.addDirectory = function addDirectory(dir, context, req, callerFile, opts) {
            opts = opts || {};
            // disable recursion to support nested directories of subcommands
            if (typeof opts.recurse !== 'boolean')
                opts.recurse = false;
            // exclude 'json', 'coffee' from require-directory defaults
            if (!Array.isArray(opts.extensions))
                opts.extensions = ['js'];
            // allow consumer to define their own visitor function
            const parentVisit = typeof opts.visit === 'function' ? opts.visit : o => o;
            // call addHandler via visitor function
            opts.visit = function visit(obj, joined, filename) {
                const visited = parentVisit(obj, joined, filename);
                // allow consumer to skip modules with their own visitor
                if (visited) {
                    // check for cyclic reference
                    // each command file path should only be seen once per execution
                    if (~context.files.indexOf(joined))
                        return visited;
                    // keep track of visited files in context.files
                    context.files.push(joined);
                    self.addHandler(visited);
                }
                return visited;
            };
            require(11) /* require-directory */({ require: req, filename: callerFile }, dir, opts);
        };
        // lookup module object from require()d command and derive name
        // if module was not require()d and no name given, throw error
        function moduleName(obj) {
            const mod = require(12) /* which-module */(obj);
            if (!mod)
                throw new Error(`No command name given for module: ${inspect(obj)}`);
            return commandFromFilename(mod.filename);
        }
        // derive command name from filename
        function commandFromFilename(filename) {
            return path.basename(filename, path.extname(filename));
        }
        function extractDesc(obj) {
            for (let keys = ['describe', 'description', 'desc'], i = 0, l = keys.length, test; i < l; i++) {
                test = obj[keys[i]];
                if (typeof test === 'string' || typeof test === 'boolean')
                    return test;
            }
            return false;
        }
        self.parseCommand = function parseCommand(cmd) {
            const extraSpacesStrippedCommand = cmd.replace(/\s{2,}/g, ' ');
            const splitCommand = extraSpacesStrippedCommand.split(/\s+(?![^[]*]|[^<]*>)/);
            const bregex = /\.*[\][<>]/g;
            const parsedCommand = {
                cmd: (splitCommand.shift()).replace(bregex, ''),
                demanded: [],
                optional: []
            };
            splitCommand.forEach((cmd, i) => {
                let variadic = false;
                cmd = cmd.replace(/\s/g, '');
                if (/\.+[\]>]/.test(cmd) && i === splitCommand.length - 1)
                    variadic = true;
                if (/^\[/.test(cmd)) {
                    parsedCommand.optional.push({
                        cmd: cmd.replace(bregex, '').split('|'),
                        variadic
                    });
                }
                else {
                    parsedCommand.demanded.push({
                        cmd: cmd.replace(bregex, '').split('|'),
                        variadic
                    });
                }
            });
            return parsedCommand;
        };
        self.getCommands = () => Object.keys(handlers).concat(Object.keys(aliasMap));
        self.getCommandHandlers = () => handlers;
        self.hasDefaultCommand = () => !!defaultCommand;
        self.runCommand = function runCommand(command, yargs, parsed, commandIndex) {
            let aliases = parsed.aliases;
            const commandHandler = handlers[command] || handlers[aliasMap[command]] || defaultCommand;
            const currentContext = yargs.getContext();
            let numFiles = currentContext.files.length;
            const parentCommands = currentContext.commands.slice();
            // what does yargs look like after the builder is run?
            let innerArgv = parsed.argv;
            let innerYargs = null;
            let positionalMap = {};
            if (command) {
                currentContext.commands.push(command);
                currentContext.fullCommands.push(commandHandler.original);
            }
            if (typeof commandHandler.builder === 'function') {
                // a function can be provided, which builds
                // up a yargs chain and possibly returns it.
                innerYargs = commandHandler.builder(yargs.reset(parsed.aliases));
                if (!innerYargs || (typeof innerYargs._parseArgs !== 'function')) {
                    innerYargs = yargs;
                }
                if (shouldUpdateUsage(innerYargs)) {
                    innerYargs.getUsageInstance().usage(usageFromParentCommandsCommandHandler(parentCommands, commandHandler), commandHandler.description);
                }
                innerArgv = innerYargs._parseArgs(null, null, true, commandIndex);
                aliases = innerYargs.parsed.aliases;
            }
            else if (typeof commandHandler.builder === 'object') {
                // as a short hand, an object can instead be provided, specifying
                // the options that a command takes.
                innerYargs = yargs.reset(parsed.aliases);
                if (shouldUpdateUsage(innerYargs)) {
                    innerYargs.getUsageInstance().usage(usageFromParentCommandsCommandHandler(parentCommands, commandHandler), commandHandler.description);
                }
                Object.keys(commandHandler.builder).forEach((key) => {
                    innerYargs.option(key, commandHandler.builder[key]);
                });
                innerArgv = innerYargs._parseArgs(null, null, true, commandIndex);
                aliases = innerYargs.parsed.aliases;
            }
            if (!yargs._hasOutput()) {
                positionalMap = populatePositionals(commandHandler, innerArgv, currentContext, yargs);
            }
            const middlewares = globalMiddleware.slice(0).concat(commandHandler.middlewares);
            applyMiddleware(innerArgv, yargs, middlewares, true);
            // we apply validation post-hoc, so that custom
            // checks get passed populated positional arguments.
            if (!yargs._hasOutput())
                yargs._runValidation(innerArgv, aliases, positionalMap, yargs.parsed.error);
            if (commandHandler.handler && !yargs._hasOutput()) {
                yargs._setHasOutput();
                // to simplify the parsing of positionals in commands,
                // we temporarily populate '--' rather than _, with arguments
                const populateDoubleDash = !!yargs.getOptions().configuration['populate--'];
                if (!populateDoubleDash)
                    yargs._copyDoubleDash(innerArgv);
                innerArgv = applyMiddleware(innerArgv, yargs, middlewares, false);
                let handlerResult;
                if (isPromise(innerArgv)) {
                    handlerResult = innerArgv.then(argv => commandHandler.handler(argv));
                }
                else {
                    handlerResult = commandHandler.handler(innerArgv);
                }
                const handlerFinishCommand = yargs.getHandlerFinishCommand();
                if (isPromise(handlerResult)) {
                    yargs.getUsageInstance().cacheHelpMessage();
                    handlerResult
                        .then(value => {
                        if (handlerFinishCommand) {
                            handlerFinishCommand(value);
                        }
                    })
                        .catch(error => {
                        try {
                            yargs.getUsageInstance().fail(null, error);
                        }
                        catch (err) {
                            // fail's throwing would cause an unhandled rejection.
                        }
                    })
                        .then(() => {
                        yargs.getUsageInstance().clearCachedHelpMessage();
                    });
                }
                else {
                    if (handlerFinishCommand) {
                        handlerFinishCommand(handlerResult);
                    }
                }
            }
            if (command) {
                currentContext.commands.pop();
                currentContext.fullCommands.pop();
            }
            numFiles = currentContext.files.length - numFiles;
            if (numFiles > 0)
                currentContext.files.splice(numFiles * -1, numFiles);
            return innerArgv;
        };
        function shouldUpdateUsage(yargs) {
            return !yargs.getUsageInstance().getUsageDisabled() &&
                yargs.getUsageInstance().getUsage().length === 0;
        }
        function usageFromParentCommandsCommandHandler(parentCommands, commandHandler) {
            const c = DEFAULT_MARKER.test(commandHandler.original) ? commandHandler.original.replace(DEFAULT_MARKER, '').trim() : commandHandler.original;
            const pc = parentCommands.filter((c) => { return !DEFAULT_MARKER.test(c); });
            pc.push(c);
            return `$0 ${pc.join(' ')}`;
        }
        self.runDefaultBuilderOn = function (yargs) {
            if (shouldUpdateUsage(yargs)) {
                // build the root-level command string from the default string.
                const commandString = DEFAULT_MARKER.test(defaultCommand.original)
                    ? defaultCommand.original : defaultCommand.original.replace(/^[^[\]<>]*/, '$0 ');
                yargs.getUsageInstance().usage(commandString, defaultCommand.description);
            }
            const builder = defaultCommand.builder;
            if (typeof builder === 'function') {
                builder(yargs);
            }
            else {
                Object.keys(builder).forEach((key) => {
                    yargs.option(key, builder[key]);
                });
            }
        };
        // transcribe all positional arguments "command <foo> <bar> [apple]"
        // onto argv.
        function populatePositionals(commandHandler, argv, context, yargs) {
            argv._ = argv._.slice(context.commands.length); // nuke the current commands
            const demanded = commandHandler.demanded.slice(0);
            const optional = commandHandler.optional.slice(0);
            const positionalMap = {};
            validation.positionalCount(demanded.length, argv._.length);
            while (demanded.length) {
                const demand = demanded.shift();
                populatePositional(demand, argv, positionalMap);
            }
            while (optional.length) {
                const maybe = optional.shift();
                populatePositional(maybe, argv, positionalMap);
            }
            argv._ = context.commands.concat(argv._);
            postProcessPositionals(argv, positionalMap, self.cmdToParseOptions(commandHandler.original));
            return positionalMap;
        }
        function populatePositional(positional, argv, positionalMap, parseOptions) {
            const cmd = positional.cmd[0];
            if (positional.variadic) {
                positionalMap[cmd] = argv._.splice(0).map(String);
            }
            else {
                if (argv._.length)
                    positionalMap[cmd] = [String(argv._.shift())];
            }
        }
        // we run yargs-parser against the positional arguments
        // applying the same parsing logic used for flags.
        function postProcessPositionals(argv, positionalMap, parseOptions) {
            // combine the parsing hints we've inferred from the command
            // string with explicitly configured parsing hints.
            const options = Object.assign({}, yargs.getOptions());
            options.default = Object.assign(parseOptions.default, options.default);
            options.alias = Object.assign(parseOptions.alias, options.alias);
            options.array = options.array.concat(parseOptions.array);
            delete options.config; //  don't load config when processing positionals.
            const unparsed = [];
            Object.keys(positionalMap).forEach((key) => {
                positionalMap[key].map((value) => {
                    if (options.configuration['unknown-options-as-args'])
                        options.key[key] = true;
                    unparsed.push(`--${key}`);
                    unparsed.push(value);
                });
            });
            // short-circuit parse.
            if (!unparsed.length)
                return;
            const config = Object.assign({}, options.configuration, {
                'populate--': true
            });
            const parsed = Parser.detailed(unparsed, Object.assign({}, options, {
                configuration: config
            }));
            if (parsed.error) {
                yargs.getUsageInstance().fail(parsed.error.message, parsed.error);
            }
            else {
                // only copy over positional keys (don't overwrite
                // flag arguments that were already parsed).
                const positionalKeys = Object.keys(positionalMap);
                Object.keys(positionalMap).forEach((key) => {
                    [].push.apply(positionalKeys, parsed.aliases[key]);
                });
                Object.keys(parsed.argv).forEach((key) => {
                    if (positionalKeys.indexOf(key) !== -1) {
                        // any new aliases need to be placed in positionalMap, which
                        // is used for validation.
                        if (!positionalMap[key])
                            positionalMap[key] = parsed.argv[key];
                        argv[key] = parsed.argv[key];
                    }
                });
            }
        }
        self.cmdToParseOptions = function (cmdString) {
            const parseOptions = {
                array: [],
                default: {},
                alias: {},
                demand: {}
            };
            const parsed = self.parseCommand(cmdString);
            parsed.demanded.forEach((d) => {
                const cmds = d.cmd.slice(0);
                const cmd = cmds.shift();
                if (d.variadic) {
                    parseOptions.array.push(cmd);
                    parseOptions.default[cmd] = [];
                }
                cmds.forEach((c) => {
                    parseOptions.alias[cmd] = c;
                });
                parseOptions.demand[cmd] = true;
            });
            parsed.optional.forEach((o) => {
                const cmds = o.cmd.slice(0);
                const cmd = cmds.shift();
                if (o.variadic) {
                    parseOptions.array.push(cmd);
                    parseOptions.default[cmd] = [];
                }
                cmds.forEach((c) => {
                    parseOptions.alias[cmd] = c;
                });
            });
            return parseOptions;
        };
        self.reset = () => {
            handlers = {};
            aliasMap = {};
            defaultCommand = undefined;
            return self;
        };
        // used by yargs.parse() to freeze
        // the state of commands such that
        // we can apply .parse() multiple times
        // with the same yargs instance.
        const frozens = [];
        self.freeze = () => {
            const frozen = {};
            frozens.push(frozen);
            frozen.handlers = handlers;
            frozen.aliasMap = aliasMap;
            frozen.defaultCommand = defaultCommand;
        };
        self.unfreeze = () => {
            const frozen = frozens.pop();
            handlers = frozen.handlers;
            aliasMap = frozen.aliasMap;
            defaultCommand = frozen.defaultCommand;
        };
        return self;
    };
},
/* yargs/lib/is-promise.js */ function _(require, module, exports) {
    module.exports = function isPromise(maybePromise) {
        return !!maybePromise && !!maybePromise.then && (typeof maybePromise.then === 'function');
    };
},
/* yargs/lib/middleware.js */ function _(require, module, exports) {
    // hoisted due to circular dependency on command.
    module.exports = {
        applyMiddleware,
        commandMiddlewareFactory,
        globalMiddlewareFactory
    };
    const isPromise = require(5) /* ./is-promise */;
    const argsert = require(3) /* ./argsert */;
    function globalMiddlewareFactory(globalMiddleware, context) {
        return function (callback, applyBeforeValidation = false) {
            argsert('<array|function> [boolean]', [callback, applyBeforeValidation], arguments.length);
            if (Array.isArray(callback)) {
                for (let i = 0; i < callback.length; i++) {
                    if (typeof callback[i] !== 'function') {
                        throw Error('middleware must be a function');
                    }
                    callback[i].applyBeforeValidation = applyBeforeValidation;
                }
                Array.prototype.push.apply(globalMiddleware, callback);
            }
            else if (typeof callback === 'function') {
                callback.applyBeforeValidation = applyBeforeValidation;
                globalMiddleware.push(callback);
            }
            return context;
        };
    }
    function commandMiddlewareFactory(commandMiddleware) {
        if (!commandMiddleware)
            return [];
        return commandMiddleware.map(middleware => {
            middleware.applyBeforeValidation = false;
            return middleware;
        });
    }
    function applyMiddleware(argv, yargs, middlewares, beforeValidation) {
        const beforeValidationError = new Error('middleware cannot return a promise when applyBeforeValidation is true');
        return middlewares
            .reduce((accumulation, middleware) => {
            if (middleware.applyBeforeValidation !== beforeValidation) {
                return accumulation;
            }
            if (isPromise(accumulation)) {
                return accumulation
                    .then(initialObj => Promise.all([initialObj, middleware(initialObj, yargs)]))
                    .then(([initialObj, middlewareObj]) => Object.assign(initialObj, middlewareObj));
            }
            else {
                const result = middleware(argv, yargs);
                if (beforeValidation && isPromise(result))
                    throw beforeValidationError;
                return isPromise(result)
                    ? result.then(middlewareObj => Object.assign(accumulation, middlewareObj))
                    : Object.assign(accumulation, result);
            }
        }, argv);
    }
},
/* yargs-parser/index.js */ function _(require, module, exports) {
    var camelCase = require(8) /* camelcase */;
    var decamelize = require(9) /* decamelize */;
    var path = require('path');
    var tokenizeArgString = require(10) /* ./lib/tokenize-arg-string */;
    var util = require('util');
    function parse(args, opts) {
        if (!opts)
            opts = {};
        // allow a string argument to be passed in rather
        // than an argv array.
        args = tokenizeArgString(args);
        // aliases might have transitive relationships, normalize this.
        var aliases = combineAliases(opts.alias || {});
        var configuration = Object.assign({
            'short-option-groups': true,
            'camel-case-expansion': true,
            'dot-notation': true,
            'parse-numbers': true,
            'boolean-negation': true,
            'negation-prefix': 'no-',
            'duplicate-arguments-array': true,
            'flatten-duplicate-arrays': true,
            'populate--': false,
            'combine-arrays': false,
            'set-placeholder-key': false,
            'halt-at-non-option': false,
            'strip-aliased': false,
            'strip-dashed': false
        }, opts.configuration);
        var defaults = opts.default || {};
        var configObjects = opts.configObjects || [];
        var envPrefix = opts.envPrefix;
        var notFlagsOption = configuration['populate--'];
        var notFlagsArgv = notFlagsOption ? '--' : '_';
        var newAliases = {};
        // allow a i18n handler to be passed in, default to a fake one (util.format).
        var __ = opts.__ || util.format;
        var error = null;
        var flags = {
            aliases: {},
            arrays: {},
            bools: {},
            strings: {},
            numbers: {},
            counts: {},
            normalize: {},
            configs: {},
            defaulted: {},
            nargs: {},
            coercions: {},
            keys: []
        };
        var negative = /^-[0-9]+(\.[0-9]+)?/;
        var negatedBoolean = new RegExp('^--' + configuration['negation-prefix'] + '(.+)');
        [].concat(opts.array).filter(Boolean).forEach(function (opt) {
            var key = opt.key || opt;
            // assign to flags[bools|strings|numbers]
            const assignment = Object.keys(opt).map(function (key) {
                return ({
                    boolean: 'bools',
                    string: 'strings',
                    number: 'numbers'
                })[key];
            }).filter(Boolean).pop();
            // assign key to be coerced
            if (assignment) {
                flags[assignment][key] = true;
            }
            flags.arrays[key] = true;
            flags.keys.push(key);
        });
        [].concat(opts.boolean).filter(Boolean).forEach(function (key) {
            flags.bools[key] = true;
            flags.keys.push(key);
        });
        [].concat(opts.string).filter(Boolean).forEach(function (key) {
            flags.strings[key] = true;
            flags.keys.push(key);
        });
        [].concat(opts.number).filter(Boolean).forEach(function (key) {
            flags.numbers[key] = true;
            flags.keys.push(key);
        });
        [].concat(opts.count).filter(Boolean).forEach(function (key) {
            flags.counts[key] = true;
            flags.keys.push(key);
        });
        [].concat(opts.normalize).filter(Boolean).forEach(function (key) {
            flags.normalize[key] = true;
            flags.keys.push(key);
        });
        Object.keys(opts.narg || {}).forEach(function (k) {
            flags.nargs[k] = opts.narg[k];
            flags.keys.push(k);
        });
        Object.keys(opts.coerce || {}).forEach(function (k) {
            flags.coercions[k] = opts.coerce[k];
            flags.keys.push(k);
        });
        if (Array.isArray(opts.config) || typeof opts.config === 'string') {
            ;
            [].concat(opts.config).filter(Boolean).forEach(function (key) {
                flags.configs[key] = true;
            });
        }
        else {
            Object.keys(opts.config || {}).forEach(function (k) {
                flags.configs[k] = opts.config[k];
            });
        }
        // create a lookup table that takes into account all
        // combinations of aliases: {f: ['foo'], foo: ['f']}
        extendAliases(opts.key, aliases, opts.default, flags.arrays);
        // apply default values to all aliases.
        Object.keys(defaults).forEach(function (key) {
            (flags.aliases[key] || []).forEach(function (alias) {
                defaults[alias] = defaults[key];
            });
        });
        var argv = { _: [] };
        Object.keys(flags.bools).forEach(function (key) {
            if (Object.prototype.hasOwnProperty.call(defaults, key)) {
                setArg(key, defaults[key]);
                setDefaulted(key);
            }
        });
        var notFlags = [];
        for (var i = 0; i < args.length; i++) {
            var arg = args[i];
            var broken;
            var key;
            var letters;
            var m;
            var next;
            var value;
            // -- separated by =
            if (arg.match(/^--.+=/) || (!configuration['short-option-groups'] && arg.match(/^-.+=/))) {
                // Using [\s\S] instead of . because js doesn't support the
                // 'dotall' regex modifier. See:
                // http://stackoverflow.com/a/1068308/13216
                m = arg.match(/^--?([^=]+)=([\s\S]*)$/);
                // nargs format = '--f=monkey washing cat'
                if (checkAllAliases(m[1], flags.nargs)) {
                    args.splice(i + 1, 0, m[2]);
                    i = eatNargs(i, m[1], args);
                    // arrays format = '--f=a b c'
                }
                else if (checkAllAliases(m[1], flags.arrays) && args.length > i + 1) {
                    args.splice(i + 1, 0, m[2]);
                    i = eatArray(i, m[1], args);
                }
                else {
                    setArg(m[1], m[2]);
                }
            }
            else if (arg.match(negatedBoolean) && configuration['boolean-negation']) {
                key = arg.match(negatedBoolean)[1];
                setArg(key, false);
                // -- seperated by space.
            }
            else if (arg.match(/^--.+/) || (!configuration['short-option-groups'] && arg.match(/^-[^-]+/))) {
                key = arg.match(/^--?(.+)/)[1];
                // nargs format = '--foo a b c'
                if (checkAllAliases(key, flags.nargs)) {
                    i = eatNargs(i, key, args);
                    // array format = '--foo a b c'
                }
                else if (checkAllAliases(key, flags.arrays) && args.length > i + 1) {
                    i = eatArray(i, key, args);
                }
                else {
                    next = flags.nargs[key] === 0 ? undefined : args[i + 1];
                    if (next !== undefined && (!next.match(/^-/) ||
                        next.match(negative)) &&
                        !checkAllAliases(key, flags.bools) &&
                        !checkAllAliases(key, flags.counts)) {
                        setArg(key, next);
                        i++;
                    }
                    else if (/^(true|false)$/.test(next)) {
                        setArg(key, next);
                        i++;
                    }
                    else {
                        setArg(key, defaultValue(key));
                    }
                }
                // dot-notation flag seperated by '='.
            }
            else if (arg.match(/^-.\..+=/)) {
                m = arg.match(/^-([^=]+)=([\s\S]*)$/);
                setArg(m[1], m[2]);
                // dot-notation flag seperated by space.
            }
            else if (arg.match(/^-.\..+/)) {
                next = args[i + 1];
                key = arg.match(/^-(.\..+)/)[1];
                if (next !== undefined && !next.match(/^-/) &&
                    !checkAllAliases(key, flags.bools) &&
                    !checkAllAliases(key, flags.counts)) {
                    setArg(key, next);
                    i++;
                }
                else {
                    setArg(key, defaultValue(key));
                }
            }
            else if (arg.match(/^-[^-]+/) && !arg.match(negative)) {
                letters = arg.slice(1, -1).split('');
                broken = false;
                for (var j = 0; j < letters.length; j++) {
                    next = arg.slice(j + 2);
                    if (letters[j + 1] && letters[j + 1] === '=') {
                        value = arg.slice(j + 3);
                        key = letters[j];
                        // nargs format = '-f=monkey washing cat'
                        if (checkAllAliases(key, flags.nargs)) {
                            args.splice(i + 1, 0, value);
                            i = eatNargs(i, key, args);
                            // array format = '-f=a b c'
                        }
                        else if (checkAllAliases(key, flags.arrays) && args.length > i + 1) {
                            args.splice(i + 1, 0, value);
                            i = eatArray(i, key, args);
                        }
                        else {
                            setArg(key, value);
                        }
                        broken = true;
                        break;
                    }
                    if (next === '-') {
                        setArg(letters[j], next);
                        continue;
                    }
                    // current letter is an alphabetic character and next value is a number
                    if (/[A-Za-z]/.test(letters[j]) &&
                        /^-?\d+(\.\d*)?(e-?\d+)?$/.test(next)) {
                        setArg(letters[j], next);
                        broken = true;
                        break;
                    }
                    if (letters[j + 1] && letters[j + 1].match(/\W/)) {
                        setArg(letters[j], next);
                        broken = true;
                        break;
                    }
                    else {
                        setArg(letters[j], defaultValue(letters[j]));
                    }
                }
                key = arg.slice(-1)[0];
                if (!broken && key !== '-') {
                    // nargs format = '-f a b c'
                    if (checkAllAliases(key, flags.nargs)) {
                        i = eatNargs(i, key, args);
                        // array format = '-f a b c'
                    }
                    else if (checkAllAliases(key, flags.arrays) && args.length > i + 1) {
                        i = eatArray(i, key, args);
                    }
                    else {
                        next = args[i + 1];
                        if (next !== undefined && (!/^(-|--)[^-]/.test(next) ||
                            next.match(negative)) &&
                            !checkAllAliases(key, flags.bools) &&
                            !checkAllAliases(key, flags.counts)) {
                            setArg(key, next);
                            i++;
                        }
                        else if (/^(true|false)$/.test(next)) {
                            setArg(key, next);
                            i++;
                        }
                        else {
                            setArg(key, defaultValue(key));
                        }
                    }
                }
            }
            else if (arg === '--') {
                notFlags = args.slice(i + 1);
                break;
            }
            else if (configuration['halt-at-non-option']) {
                notFlags = args.slice(i);
                break;
            }
            else {
                argv._.push(maybeCoerceNumber('_', arg));
            }
        }
        // order of precedence:
        // 1. command line arg
        // 2. value from env var
        // 3. value from config file
        // 4. value from config objects
        // 5. configured default value
        applyEnvVars(argv, true); // special case: check env vars that point to config file
        applyEnvVars(argv, false);
        setConfig(argv);
        setConfigObjects();
        applyDefaultsAndAliases(argv, flags.aliases, defaults);
        applyCoercions(argv);
        if (configuration['set-placeholder-key'])
            setPlaceholderKeys(argv);
        // for any counts either not in args or without an explicit default, set to 0
        Object.keys(flags.counts).forEach(function (key) {
            if (!hasKey(argv, key.split('.')))
                setArg(key, 0);
        });
        // '--' defaults to undefined.
        if (notFlagsOption && notFlags.length)
            argv[notFlagsArgv] = [];
        notFlags.forEach(function (key) {
            argv[notFlagsArgv].push(key);
        });
        if (configuration['camel-case-expansion'] && configuration['strip-dashed']) {
            Object.keys(argv).filter(key => key !== '--' && key.includes('-')).forEach(key => {
                delete argv[key];
            });
        }
        if (configuration['strip-aliased']) {
            // XXX Switch to [].concat(...Object.values(aliases)) once node.js 6 is dropped
            ;
            [].concat(...Object.keys(aliases).map(k => aliases[k])).forEach(alias => {
                if (configuration['camel-case-expansion']) {
                    delete argv[alias.split('.').map(prop => camelCase(prop)).join('.')];
                }
                delete argv[alias];
            });
        }
        // how many arguments should we consume, based
        // on the nargs option?
        function eatNargs(i, key, args) {
            var ii;
            const toEat = checkAllAliases(key, flags.nargs);
            // nargs will not consume flag arguments, e.g., -abc, --foo,
            // and terminates when one is observed.
            var available = 0;
            for (ii = i + 1; ii < args.length; ii++) {
                if (!args[ii].match(/^-[^0-9]/))
                    available++;
                else
                    break;
            }
            if (available < toEat)
                error = Error(__('Not enough arguments following: %s', key));
            const consumed = Math.min(available, toEat);
            for (ii = i + 1; ii < (consumed + i + 1); ii++) {
                setArg(key, args[ii]);
            }
            return (i + consumed);
        }
        // if an option is an array, eat all non-hyphenated arguments
        // following it... YUM!
        // e.g., --foo apple banana cat becomes ["apple", "banana", "cat"]
        function eatArray(i, key, args) {
            var start = i + 1;
            var argsToSet = [];
            var multipleArrayFlag = i > 0;
            for (var ii = i + 1; ii < args.length; ii++) {
                if (/^-/.test(args[ii]) && !negative.test(args[ii])) {
                    if (ii === start) {
                        setArg(key, defaultForType('array'));
                    }
                    multipleArrayFlag = true;
                    break;
                }
                i = ii;
                argsToSet.push(args[ii]);
            }
            if (multipleArrayFlag) {
                setArg(key, argsToSet.map(function (arg) {
                    return processValue(key, arg);
                }));
            }
            else {
                argsToSet.forEach(function (arg) {
                    setArg(key, arg);
                });
            }
            return i;
        }
        function setArg(key, val) {
            unsetDefaulted(key);
            if (/-/.test(key) && configuration['camel-case-expansion']) {
                var alias = key.split('.').map(function (prop) {
                    return camelCase(prop);
                }).join('.');
                addNewAlias(key, alias);
            }
            var value = processValue(key, val);
            var splitKey = key.split('.');
            setKey(argv, splitKey, value);
            // handle populating aliases of the full key
            if (flags.aliases[key] && flags.aliases[key].forEach) {
                flags.aliases[key].forEach(function (x) {
                    x = x.split('.');
                    setKey(argv, x, value);
                });
            }
            // handle populating aliases of the first element of the dot-notation key
            if (splitKey.length > 1 && configuration['dot-notation']) {
                ;
                (flags.aliases[splitKey[0]] || []).forEach(function (x) {
                    x = x.split('.');
                    // expand alias with nested objects in key
                    var a = [].concat(splitKey);
                    a.shift(); // nuke the old key.
                    x = x.concat(a);
                    setKey(argv, x, value);
                });
            }
            // Set normalize getter and setter when key is in 'normalize' but isn't an array
            if (checkAllAliases(key, flags.normalize) && !checkAllAliases(key, flags.arrays)) {
                var keys = [key].concat(flags.aliases[key] || []);
                keys.forEach(function (key) {
                    argv.__defineSetter__(key, function (v) {
                        val = path.normalize(v);
                    });
                    argv.__defineGetter__(key, function () {
                        return typeof val === 'string' ? path.normalize(val) : val;
                    });
                });
            }
        }
        function addNewAlias(key, alias) {
            if (!(flags.aliases[key] && flags.aliases[key].length)) {
                flags.aliases[key] = [alias];
                newAliases[alias] = true;
            }
            if (!(flags.aliases[alias] && flags.aliases[alias].length)) {
                addNewAlias(alias, key);
            }
        }
        function processValue(key, val) {
            // strings may be quoted, clean this up as we assign values.
            if (typeof val === 'string' &&
                (val[0] === "'" || val[0] === '"') &&
                val[val.length - 1] === val[0]) {
                val = val.substring(1, val.length - 1);
            }
            // handle parsing boolean arguments --foo=true --bar false.
            if (checkAllAliases(key, flags.bools) || checkAllAliases(key, flags.counts)) {
                if (typeof val === 'string')
                    val = val === 'true';
            }
            var value = maybeCoerceNumber(key, val);
            // increment a count given as arg (either no value or value parsed as boolean)
            if (checkAllAliases(key, flags.counts) && (isUndefined(value) || typeof value === 'boolean')) {
                value = increment;
            }
            // Set normalized value when key is in 'normalize' and in 'arrays'
            if (checkAllAliases(key, flags.normalize) && checkAllAliases(key, flags.arrays)) {
                if (Array.isArray(val))
                    value = val.map(path.normalize);
                else
                    value = path.normalize(val);
            }
            return value;
        }
        function maybeCoerceNumber(key, value) {
            if (!checkAllAliases(key, flags.strings) && !checkAllAliases(key, flags.coercions)) {
                const shouldCoerceNumber = isNumber(value) && configuration['parse-numbers'] && (Number.isSafeInteger(Math.floor(value)));
                if (shouldCoerceNumber || (!isUndefined(value) && checkAllAliases(key, flags.numbers)))
                    value = Number(value);
            }
            return value;
        }
        // set args from config.json file, this should be
        // applied last so that defaults can be applied.
        function setConfig(argv) {
            var configLookup = {};
            // expand defaults/aliases, in-case any happen to reference
            // the config.json file.
            applyDefaultsAndAliases(configLookup, flags.aliases, defaults);
            Object.keys(flags.configs).forEach(function (configKey) {
                var configPath = argv[configKey] || configLookup[configKey];
                if (configPath) {
                    try {
                        var config = null;
                        var resolvedConfigPath = path.resolve(process.cwd(), configPath);
                        if (typeof flags.configs[configKey] === 'function') {
                            try {
                                config = flags.configs[configKey](resolvedConfigPath);
                            }
                            catch (e) {
                                config = e;
                            }
                            if (config instanceof Error) {
                                error = config;
                                return;
                            }
                        }
                        else {
                            config = require(resolvedConfigPath);
                        }
                        setConfigObject(config);
                    }
                    catch (ex) {
                        if (argv[configKey])
                            error = Error(__('Invalid JSON config file: %s', configPath));
                    }
                }
            });
        }
        // set args from config object.
        // it recursively checks nested objects.
        function setConfigObject(config, prev) {
            Object.keys(config).forEach(function (key) {
                var value = config[key];
                var fullKey = prev ? prev + '.' + key : key;
                // if the value is an inner object and we have dot-notation
                // enabled, treat inner objects in config the same as
                // heavily nested dot notations (foo.bar.apple).
                if (typeof value === 'object' && value !== null && !Array.isArray(value) && configuration['dot-notation']) {
                    // if the value is an object but not an array, check nested object
                    setConfigObject(value, fullKey);
                }
                else {
                    // setting arguments via CLI takes precedence over
                    // values within the config file.
                    if (!hasKey(argv, fullKey.split('.')) || (flags.defaulted[fullKey]) || (flags.arrays[fullKey] && configuration['combine-arrays'])) {
                        setArg(fullKey, value);
                    }
                }
            });
        }
        // set all config objects passed in opts
        function setConfigObjects() {
            if (typeof configObjects === 'undefined')
                return;
            configObjects.forEach(function (configObject) {
                setConfigObject(configObject);
            });
        }
        function applyEnvVars(argv, configOnly) {
            if (typeof envPrefix === 'undefined')
                return;
            var prefix = typeof envPrefix === 'string' ? envPrefix : '';
            Object.keys(process.env).forEach(function (envVar) {
                if (prefix === '' || envVar.lastIndexOf(prefix, 0) === 0) {
                    // get array of nested keys and convert them to camel case
                    var keys = envVar.split('__').map(function (key, i) {
                        if (i === 0) {
                            key = key.substring(prefix.length);
                        }
                        return camelCase(key);
                    });
                    if (((configOnly && flags.configs[keys.join('.')]) || !configOnly) && (!hasKey(argv, keys) || flags.defaulted[keys.join('.')])) {
                        setArg(keys.join('.'), process.env[envVar]);
                    }
                }
            });
        }
        function applyCoercions(argv) {
            var coerce;
            var applied = {};
            Object.keys(argv).forEach(function (key) {
                if (!applied.hasOwnProperty(key)) { // If we haven't already coerced this option via one of its aliases
                    coerce = checkAllAliases(key, flags.coercions);
                    if (typeof coerce === 'function') {
                        try {
                            var value = coerce(argv[key]);
                            ([].concat(flags.aliases[key] || [], key)).forEach(ali => {
                                applied[ali] = argv[ali] = value;
                            });
                        }
                        catch (err) {
                            error = err;
                        }
                    }
                }
            });
        }
        function setPlaceholderKeys(argv) {
            flags.keys.forEach((key) => {
                // don't set placeholder keys for dot notation options 'foo.bar'.
                if (~key.indexOf('.'))
                    return;
                if (typeof argv[key] === 'undefined')
                    argv[key] = undefined;
            });
            return argv;
        }
        function applyDefaultsAndAliases(obj, aliases, defaults) {
            Object.keys(defaults).forEach(function (key) {
                if (!hasKey(obj, key.split('.'))) {
                    setKey(obj, key.split('.'), defaults[key]);
                    (aliases[key] || []).forEach(function (x) {
                        if (hasKey(obj, x.split('.')))
                            return;
                        setKey(obj, x.split('.'), defaults[key]);
                    });
                }
            });
        }
        function hasKey(obj, keys) {
            var o = obj;
            if (!configuration['dot-notation'])
                keys = [keys.join('.')];
            keys.slice(0, -1).forEach(function (key) {
                o = (o[key] || {});
            });
            var key = keys[keys.length - 1];
            if (typeof o !== 'object')
                return false;
            else
                return key in o;
        }
        function setKey(obj, keys, value) {
            var o = obj;
            if (!configuration['dot-notation'])
                keys = [keys.join('.')];
            keys.slice(0, -1).forEach(function (key, index) {
                // TODO(bcoe): in the next major version of yargs, switch to
                // Object.create(null) for dot notation:
                key = sanitizeKey(key);
                if (typeof o === 'object' && o[key] === undefined) {
                    o[key] = {};
                }
                if (typeof o[key] !== 'object' || Array.isArray(o[key])) {
                    // ensure that o[key] is an array, and that the last item is an empty object.
                    if (Array.isArray(o[key])) {
                        o[key].push({});
                    }
                    else {
                        o[key] = [o[key], {}];
                    }
                    // we want to update the empty object at the end of the o[key] array, so set o to that object
                    o = o[key][o[key].length - 1];
                }
                else {
                    o = o[key];
                }
            });
            // TODO(bcoe): in the next major version of yargs, switch to
            // Object.create(null) for dot notation:
            const key = sanitizeKey(keys[keys.length - 1]);
            const isTypeArray = checkAllAliases(keys.join('.'), flags.arrays);
            const isValueArray = Array.isArray(value);
            let duplicate = configuration['duplicate-arguments-array'];
            // nargs has higher priority than duplicate
            if (!duplicate && checkAllAliases(key, flags.nargs)) {
                duplicate = true;
                if ((!isUndefined(o[key]) && flags.nargs[key] === 1) || (Array.isArray(o[key]) && o[key].length === flags.nargs[key])) {
                    o[key] = undefined;
                }
            }
            if (value === increment) {
                o[key] = increment(o[key]);
            }
            else if (Array.isArray(o[key])) {
                if (duplicate && isTypeArray && isValueArray) {
                    o[key] = configuration['flatten-duplicate-arrays'] ? o[key].concat(value) : (Array.isArray(o[key][0]) ? o[key] : [o[key]]).concat([value]);
                }
                else if (!duplicate && Boolean(isTypeArray) === Boolean(isValueArray)) {
                    o[key] = value;
                }
                else {
                    o[key] = o[key].concat([value]);
                }
            }
            else if (o[key] === undefined && isTypeArray) {
                o[key] = isValueArray ? value : [value];
            }
            else if (duplicate && !(o[key] === undefined || checkAllAliases(key, flags.bools) || checkAllAliases(keys.join('.'), flags.bools) || checkAllAliases(key, flags.counts))) {
                o[key] = [o[key], value];
            }
            else {
                o[key] = value;
            }
        }
        // extend the aliases list with inferred aliases.
        function extendAliases(...args) {
            args.forEach(function (obj) {
                Object.keys(obj || {}).forEach(function (key) {
                    // short-circuit if we've already added a key
                    // to the aliases array, for example it might
                    // exist in both 'opts.default' and 'opts.key'.
                    if (flags.aliases[key])
                        return;
                    flags.aliases[key] = [].concat(aliases[key] || []);
                    // For "--option-name", also set argv.optionName
                    flags.aliases[key].concat(key).forEach(function (x) {
                        if (/-/.test(x) && configuration['camel-case-expansion']) {
                            var c = camelCase(x);
                            if (c !== key && flags.aliases[key].indexOf(c) === -1) {
                                flags.aliases[key].push(c);
                                newAliases[c] = true;
                            }
                        }
                    });
                    // For "--optionName", also set argv['option-name']
                    flags.aliases[key].concat(key).forEach(function (x) {
                        if (x.length > 1 && /[A-Z]/.test(x) && configuration['camel-case-expansion']) {
                            var c = decamelize(x, '-');
                            if (c !== key && flags.aliases[key].indexOf(c) === -1) {
                                flags.aliases[key].push(c);
                                newAliases[c] = true;
                            }
                        }
                    });
                    flags.aliases[key].forEach(function (x) {
                        flags.aliases[x] = [key].concat(flags.aliases[key].filter(function (y) {
                            return x !== y;
                        }));
                    });
                });
            });
        }
        // check if a flag is set for any of a key's aliases.
        function checkAllAliases(key, flag) {
            var isSet = false;
            var toCheck = [].concat(flags.aliases[key] || [], key);
            toCheck.forEach(function (key) {
                if (flag[key])
                    isSet = flag[key];
            });
            return isSet;
        }
        function setDefaulted(key) {
            [].concat(flags.aliases[key] || [], key).forEach(function (k) {
                flags.defaulted[k] = true;
            });
        }
        function unsetDefaulted(key) {
            [].concat(flags.aliases[key] || [], key).forEach(function (k) {
                delete flags.defaulted[k];
            });
        }
        // make a best effor to pick a default value
        // for an option based on name and type.
        function defaultValue(key) {
            if (!checkAllAliases(key, flags.bools) &&
                !checkAllAliases(key, flags.counts) &&
                `${key}` in defaults) {
                return defaults[key];
            }
            else {
                return defaultForType(guessType(key));
            }
        }
        // return a default value, given the type of a flag.,
        // e.g., key of type 'string' will default to '', rather than 'true'.
        function defaultForType(type) {
            var def = {
                boolean: true,
                string: '',
                number: undefined,
                array: []
            };
            return def[type];
        }
        // given a flag, enforce a default type.
        function guessType(key) {
            var type = 'boolean';
            if (checkAllAliases(key, flags.strings))
                type = 'string';
            else if (checkAllAliases(key, flags.numbers))
                type = 'number';
            else if (checkAllAliases(key, flags.arrays))
                type = 'array';
            return type;
        }
        function isNumber(x) {
            if (x === null || x === undefined)
                return false;
            // if loaded from config, may already be a number.
            if (typeof x === 'number')
                return true;
            // hexadecimal.
            if (/^0x[0-9a-f]+$/i.test(x))
                return true;
            // don't treat 0123 as a number; as it drops the leading '0'.
            if (x.length > 1 && x[0] === '0')
                return false;
            return /^[-]?(?:\d+(?:\.\d*)?|\.\d+)(e[-+]?\d+)?$/.test(x);
        }
        function isUndefined(num) {
            return num === undefined;
        }
        return {
            argv: argv,
            error: error,
            aliases: flags.aliases,
            newAliases: newAliases,
            configuration: configuration
        };
    }
    // if any aliases reference each other, we should
    // merge them together.
    function combineAliases(aliases) {
        var aliasArrays = [];
        var change = true;
        var combined = {};
        // turn alias lookup hash {key: ['alias1', 'alias2']} into
        // a simple array ['key', 'alias1', 'alias2']
        Object.keys(aliases).forEach(function (key) {
            aliasArrays.push([].concat(aliases[key], key));
        });
        // combine arrays until zero changes are
        // made in an iteration.
        while (change) {
            change = false;
            for (var i = 0; i < aliasArrays.length; i++) {
                for (var ii = i + 1; ii < aliasArrays.length; ii++) {
                    var intersect = aliasArrays[i].filter(function (v) {
                        return aliasArrays[ii].indexOf(v) !== -1;
                    });
                    if (intersect.length) {
                        aliasArrays[i] = aliasArrays[i].concat(aliasArrays[ii]);
                        aliasArrays.splice(ii, 1);
                        change = true;
                        break;
                    }
                }
            }
        }
        // map arrays back to the hash-lookup (de-dupe while
        // we're at it).
        aliasArrays.forEach(function (aliasArray) {
            aliasArray = aliasArray.filter(function (v, i, self) {
                return self.indexOf(v) === i;
            });
            combined[aliasArray.pop()] = aliasArray;
        });
        return combined;
    }
    // this function should only be called when a count is given as an arg
    // it is NOT called to set a default value
    // thus we can start the count at 1 instead of 0
    function increment(orig) {
        return orig !== undefined ? orig + 1 : 1;
    }
    function Parser(args, opts) {
        var result = parse(args.slice(), opts);
        return result.argv;
    }
    // parse arguments and return detailed
    // meta information, aliases, etc.
    Parser.detailed = function (args, opts) {
        return parse(args.slice(), opts);
    };
    // TODO(bcoe): in the next major version of yargs, switch to
    // Object.create(null) for dot notation:
    function sanitizeKey(key) {
        if (key === '__proto__')
            return '___proto___';
        return key;
    }
    module.exports = Parser;
},
/* camelcase/index.js */ function _(require, module, exports) {
    const preserveCamelCase = string => {
        let isLastCharLower = false;
        let isLastCharUpper = false;
        let isLastLastCharUpper = false;
        for (let i = 0; i < string.length; i++) {
            const character = string[i];
            if (isLastCharLower && /[a-zA-Z]/.test(character) && character.toUpperCase() === character) {
                string = string.slice(0, i) + '-' + string.slice(i);
                isLastCharLower = false;
                isLastLastCharUpper = isLastCharUpper;
                isLastCharUpper = true;
                i++;
            }
            else if (isLastCharUpper && isLastLastCharUpper && /[a-zA-Z]/.test(character) && character.toLowerCase() === character) {
                string = string.slice(0, i - 1) + '-' + string.slice(i - 1);
                isLastLastCharUpper = isLastCharUpper;
                isLastCharUpper = false;
                isLastCharLower = true;
            }
            else {
                isLastCharLower = character.toLowerCase() === character && character.toUpperCase() !== character;
                isLastLastCharUpper = isLastCharUpper;
                isLastCharUpper = character.toUpperCase() === character && character.toLowerCase() !== character;
            }
        }
        return string;
    };
    const camelCase = (input, options) => {
        if (!(typeof input === 'string' || Array.isArray(input))) {
            throw new TypeError('Expected the input to be `string | string[]`');
        }
        options = Object.assign({
            pascalCase: false
        }, options);
        const postProcess = x => options.pascalCase ? x.charAt(0).toUpperCase() + x.slice(1) : x;
        if (Array.isArray(input)) {
            input = input.map(x => x.trim())
                .filter(x => x.length)
                .join('-');
        }
        else {
            input = input.trim();
        }
        if (input.length === 0) {
            return '';
        }
        if (input.length === 1) {
            return options.pascalCase ? input.toUpperCase() : input.toLowerCase();
        }
        const hasUpperCase = input !== input.toLowerCase();
        if (hasUpperCase) {
            input = preserveCamelCase(input);
        }
        input = input
            .replace(/^[_.\- ]+/, '')
            .toLowerCase()
            .replace(/[_.\- ]+(\w|$)/g, (_, p1) => p1.toUpperCase())
            .replace(/\d+(\w|$)/g, m => m.toUpperCase());
        return postProcess(input);
    };
    module.exports = camelCase;
    // TODO: Remove this for the next major release
    module.exports.default = camelCase;
},
/* decamelize/index.js */ function _(require, module, exports) {
    module.exports = function (str, sep) {
        if (typeof str !== 'string') {
            throw new TypeError('Expected a string');
        }
        sep = typeof sep === 'undefined' ? '_' : sep;
        return str
            .replace(/([a-z\d])([A-Z])/g, '$1' + sep + '$2')
            .replace(/([A-Z]+)([A-Z][a-z\d]+)/g, '$1' + sep + '$2')
            .toLowerCase();
    };
},
/* yargs-parser/lib/tokenize-arg-string.js */ function _(require, module, exports) {
    // take an un-split argv string and tokenize it.
    module.exports = function (argString) {
        if (Array.isArray(argString)) {
            return argString.map(e => typeof e !== 'string' ? e + '' : e);
        }
        argString = argString.trim();
        var i = 0;
        var prevC = null;
        var c = null;
        var opening = null;
        var args = [];
        for (var ii = 0; ii < argString.length; ii++) {
            prevC = c;
            c = argString.charAt(ii);
            // split on spaces unless we're in quotes.
            if (c === ' ' && !opening) {
                if (!(prevC === ' ')) {
                    i++;
                }
                continue;
            }
            // don't split the string if we're in matching
            // opening or closing single and double quotes.
            if (c === opening) {
                opening = null;
            }
            else if ((c === "'" || c === '"') && !opening) {
                opening = c;
            }
            if (!args[i])
                args[i] = '';
            args[i] += c;
        }
        return args;
    };
},
/* require-directory/index.js */ function _(require, module, exports) {
    var fs = require('fs'), join = require('path').join, resolve = require('path').resolve, dirname = require('path').dirname, defaultOptions = {
        extensions: ['js', 'json', 'coffee'],
        recurse: true,
        rename: function (name) {
            return name;
        },
        visit: function (obj) {
            return obj;
        }
    };
    function checkFileInclusion(path, filename, options) {
        return (
        // verify file has valid extension
        (new RegExp('\\.(' + options.extensions.join('|') + ')$', 'i').test(filename)) &&
            // if options.include is a RegExp, evaluate it and make sure the path passes
            !(options.include && options.include instanceof RegExp && !options.include.test(path)) &&
            // if options.include is a function, evaluate it and make sure the path passes
            !(options.include && typeof options.include === 'function' && !options.include(path, filename)) &&
            // if options.exclude is a RegExp, evaluate it and make sure the path doesn't pass
            !(options.exclude && options.exclude instanceof RegExp && options.exclude.test(path)) &&
            // if options.exclude is a function, evaluate it and make sure the path doesn't pass
            !(options.exclude && typeof options.exclude === 'function' && options.exclude(path, filename)));
    }
    function requireDirectory(m, path, options) {
        var retval = {};
        // path is optional
        if (path && !options && typeof path !== 'string') {
            options = path;
            path = null;
        }
        // default options
        options = options || {};
        for (var prop in defaultOptions) {
            if (typeof options[prop] === 'undefined') {
                options[prop] = defaultOptions[prop];
            }
        }
        // if no path was passed in, assume the equivelant of __dirname from caller
        // otherwise, resolve path relative to the equivalent of __dirname
        path = !path ? dirname(m.filename) : resolve(dirname(m.filename), path);
        // get the path of each file in specified directory, append to current tree node, recurse
        fs.readdirSync(path).forEach(function (filename) {
            var joined = join(path, filename), files, key, obj;
            if (fs.statSync(joined).isDirectory() && options.recurse) {
                // this node is a directory; recurse
                files = requireDirectory(m, joined, options);
                // exclude empty directories
                if (Object.keys(files).length) {
                    retval[options.rename(filename, joined, filename)] = files;
                }
            }
            else {
                if (joined !== m.filename && checkFileInclusion(joined, filename, options)) {
                    // hash node key shouldn't include file extension
                    key = filename.substring(0, filename.lastIndexOf('.'));
                    obj = m.require(joined);
                    retval[options.rename(key, joined, filename)] = options.visit(obj, joined, filename) || obj;
                }
            }
        });
        return retval;
    }
    module.exports = requireDirectory;
    module.exports.defaults = defaultOptions;
},
/* which-module/index.js */ function _(require, module, exports) {
    module.exports = function whichModule(exported) {
        for (var i = 0, files = Object.keys(require.cache), mod; i < files.length; i++) {
            mod = require.cache[files[i]];
            if (mod.exports === exported)
                return mod;
        }
        return null;
    };
},
/* yargs/lib/yerror.js */ function _(require, module, exports) {
    function YError(msg) {
        this.name = 'YError';
        this.message = msg || 'yargs error';
        Error.captureStackTrace(this, YError);
    }
    YError.prototype = Object.create(Error.prototype);
    YError.prototype.constructor = YError;
    module.exports = YError;
},
/* yargs/lib/completion.js */ function _(require, module, exports) {
    const path = require('path');
    // add bash completions to your
    //  yargs-powered applications.
    module.exports = function completion(yargs, usage, command) {
        const self = {
            completionKey: 'get-yargs-completions'
        };
        let aliases;
        self.setParsed = function setParsed(parsed) {
            aliases = parsed.aliases;
        };
        const zshShell = (process.env.SHELL && process.env.SHELL.indexOf('zsh') !== -1) ||
            (process.env.ZSH_NAME && process.env.ZSH_NAME.indexOf('zsh') !== -1);
        // get a list of completion commands.
        // 'args' is the array of strings from the line to be completed
        self.getCompletion = function getCompletion(args, done) {
            const completions = [];
            const current = args.length ? args[args.length - 1] : '';
            const argv = yargs.parse(args, true);
            const parentCommands = yargs.getContext().commands;
            // a custom completion function can be provided
            // to completion().
            if (completionFunction) {
                if (completionFunction.length < 3) {
                    const result = completionFunction(current, argv);
                    // promise based completion function.
                    if (typeof result.then === 'function') {
                        return result.then((list) => {
                            process.nextTick(() => { done(list); });
                        }).catch((err) => {
                            process.nextTick(() => { throw err; });
                        });
                    }
                    // synchronous completion function.
                    return done(result);
                }
                else {
                    // asynchronous completion function
                    return completionFunction(current, argv, (completions) => {
                        done(completions);
                    });
                }
            }
            const handlers = command.getCommandHandlers();
            for (let i = 0, ii = args.length; i < ii; ++i) {
                if (handlers[args[i]] && handlers[args[i]].builder) {
                    const builder = handlers[args[i]].builder;
                    if (typeof builder === 'function') {
                        const y = yargs.reset();
                        builder(y);
                        return y.argv;
                    }
                }
            }
            if (!current.match(/^-/) && parentCommands[parentCommands.length - 1] !== current) {
                usage.getCommands().forEach((usageCommand) => {
                    const commandName = command.parseCommand(usageCommand[0]).cmd;
                    if (args.indexOf(commandName) === -1) {
                        if (!zshShell) {
                            completions.push(commandName);
                        }
                        else {
                            const desc = usageCommand[1] || '';
                            completions.push(commandName.replace(/:/g, '\\:') + ':' + desc);
                        }
                    }
                });
            }
            if (current.match(/^-/) || (current === '' && completions.length === 0)) {
                const descs = usage.getDescriptions();
                const options = yargs.getOptions();
                Object.keys(options.key).forEach((key) => {
                    const negable = !!options.configuration['boolean-negation'] && options.boolean.includes(key);
                    // If the key and its aliases aren't in 'args', add the key to 'completions'
                    let keyAndAliases = [key].concat(aliases[key] || []);
                    if (negable)
                        keyAndAliases = keyAndAliases.concat(keyAndAliases.map(key => `no-${key}`));
                    function completeOptionKey(key) {
                        const notInArgs = keyAndAliases.every(val => args.indexOf(`--${val}`) === -1);
                        if (notInArgs) {
                            const startsByTwoDashes = s => /^--/.test(s);
                            const isShortOption = s => /^[^0-9]$/.test(s);
                            const dashes = !startsByTwoDashes(current) && isShortOption(key) ? '-' : '--';
                            if (!zshShell) {
                                completions.push(dashes + key);
                            }
                            else {
                                const desc = descs[key] || '';
                                completions.push(dashes + `${key.replace(/:/g, '\\:')}:${desc.replace('__yargsString__:', '')}`);
                            }
                        }
                    }
                    completeOptionKey(key);
                    if (negable && !!options.default[key])
                        completeOptionKey(`no-${key}`);
                });
            }
            done(completions);
        };
        // generate the completion script to add to your .bashrc.
        self.generateCompletionScript = function generateCompletionScript($0, cmd) {
            const templates = require(15) /* ./completion-templates */;
            let script = zshShell ? templates.completionZshTemplate : templates.completionShTemplate;
            const name = path.basename($0);
            // add ./to applications not yet installed as bin.
            if ($0.match(/\.js$/))
                $0 = `./${$0}`;
            script = script.replace(/{{app_name}}/g, name);
            script = script.replace(/{{completion_command}}/g, cmd);
            return script.replace(/{{app_path}}/g, $0);
        };
        // register a function to perform your own custom
        // completions., this function can be either
        // synchrnous or asynchronous.
        let completionFunction = null;
        self.registerFunction = (fn) => {
            completionFunction = fn;
        };
        return self;
    };
},
/* yargs/lib/completion-templates.js */ function _(require, module, exports) {
    exports.completionShTemplate =
        `###-begin-{{app_name}}-completions-###
#
# yargs command completion script
#
# Installation: {{app_path}} {{completion_command}} >> ~/.bashrc
#    or {{app_path}} {{completion_command}} >> ~/.bash_profile on OSX.
#
_yargs_completions()
{
    local cur_word args type_list

    cur_word="\${COMP_WORDS[COMP_CWORD]}"
    args=("\${COMP_WORDS[@]}")

    # ask yargs to generate completions.
    type_list=$({{app_path}} --get-yargs-completions "\${args[@]}")

    COMPREPLY=( $(compgen -W "\${type_list}" -- \${cur_word}) )

    # if no match was found, fall back to filename completion
    if [ \${#COMPREPLY[@]} -eq 0 ]; then
      COMPREPLY=()
    fi

    return 0
}
complete -o default -F _yargs_completions {{app_name}}
###-end-{{app_name}}-completions-###
`;
    exports.completionZshTemplate = `###-begin-{{app_name}}-completions-###
#
# yargs command completion script
#
# Installation: {{app_path}} {{completion_command}} >> ~/.zshrc
#    or {{app_path}} {{completion_command}} >> ~/.zsh_profile on OSX.
#
_{{app_name}}_yargs_completions()
{
  local reply
  local si=$IFS
  IFS=$'\n' reply=($(COMP_CWORD="$((CURRENT-1))" COMP_LINE="$BUFFER" COMP_POINT="$CURSOR" {{app_path}} --get-yargs-completions "\${words[@]}"))
  IFS=$si
  _describe 'values' reply
}
compdef _{{app_name}}_yargs_completions {{app_name}}
###-end-{{app_name}}-completions-###
`;
},
/* yargs/lib/usage.js */ function _(require, module, exports) {
    // this file handles outputting usage instructions,
    // failures, etc. keeps logging in one place.
    const decamelize = require(9) /* decamelize */;
    const stringWidth = require(17) /* string-width */;
    const objFilter = require(21) /* ./obj-filter */;
    const path = require('path');
    const setBlocking = require(22) /* set-blocking */;
    const YError = require(13) /* ./yerror */;
    module.exports = function usage(yargs, y18n) {
        const __ = y18n.__;
        const self = {};
        // methods for ouputting/building failure message.
        const fails = [];
        self.failFn = function failFn(f) {
            fails.push(f);
        };
        let failMessage = null;
        let showHelpOnFail = true;
        self.showHelpOnFail = function showHelpOnFailFn(enabled, message) {
            if (typeof enabled === 'string') {
                message = enabled;
                enabled = true;
            }
            else if (typeof enabled === 'undefined') {
                enabled = true;
            }
            failMessage = message;
            showHelpOnFail = enabled;
            return self;
        };
        let failureOutput = false;
        self.fail = function fail(msg, err) {
            const logger = yargs._getLoggerInstance();
            if (fails.length) {
                for (let i = fails.length - 1; i >= 0; --i) {
                    fails[i](msg, err, self);
                }
            }
            else {
                if (yargs.getExitProcess())
                    setBlocking(true);
                // don't output failure message more than once
                if (!failureOutput) {
                    failureOutput = true;
                    if (showHelpOnFail) {
                        yargs.showHelp('error');
                        logger.error();
                    }
                    if (msg || err)
                        logger.error(msg || err);
                    if (failMessage) {
                        if (msg || err)
                            logger.error('');
                        logger.error(failMessage);
                    }
                }
                err = err || new YError(msg);
                if (yargs.getExitProcess()) {
                    return yargs.exit(1);
                }
                else if (yargs._hasParseCallback()) {
                    return yargs.exit(1, err);
                }
                else {
                    throw err;
                }
            }
        };
        // methods for ouputting/building help (usage) message.
        let usages = [];
        let usageDisabled = false;
        self.usage = (msg, description) => {
            if (msg === null) {
                usageDisabled = true;
                usages = [];
                return;
            }
            usageDisabled = false;
            usages.push([msg, description || '']);
            return self;
        };
        self.getUsage = () => {
            return usages;
        };
        self.getUsageDisabled = () => {
            return usageDisabled;
        };
        self.getPositionalGroupName = () => {
            return __('Positionals:');
        };
        let examples = [];
        self.example = (cmd, description) => {
            examples.push([cmd, description || '']);
        };
        let commands = [];
        self.command = function command(cmd, description, isDefault, aliases) {
            // the last default wins, so cancel out any previously set default
            if (isDefault) {
                commands = commands.map((cmdArray) => {
                    cmdArray[2] = false;
                    return cmdArray;
                });
            }
            commands.push([cmd, description || '', isDefault, aliases]);
        };
        self.getCommands = () => commands;
        let descriptions = {};
        self.describe = function describe(key, desc) {
            if (typeof key === 'object') {
                Object.keys(key).forEach((k) => {
                    self.describe(k, key[k]);
                });
            }
            else {
                descriptions[key] = desc;
            }
        };
        self.getDescriptions = () => descriptions;
        let epilogs = [];
        self.epilog = (msg) => {
            epilogs.push(msg);
        };
        let wrapSet = false;
        let wrap;
        self.wrap = (cols) => {
            wrapSet = true;
            wrap = cols;
        };
        function getWrap() {
            if (!wrapSet) {
                wrap = windowWidth();
                wrapSet = true;
            }
            return wrap;
        }
        const deferY18nLookupPrefix = '__yargsString__:';
        self.deferY18nLookup = str => deferY18nLookupPrefix + str;
        const defaultGroup = __('Options:');
        self.help = function help() {
            if (cachedHelpMessage)
                return cachedHelpMessage;
            normalizeAliases();
            // handle old demanded API
            const base$0 = yargs.customScriptName ? yargs.$0 : path.basename(yargs.$0);
            const demandedOptions = yargs.getDemandedOptions();
            const demandedCommands = yargs.getDemandedCommands();
            const deprecatedOptions = yargs.getDeprecatedOptions();
            const groups = yargs.getGroups();
            const options = yargs.getOptions();
            let keys = [];
            keys = keys.concat(Object.keys(descriptions));
            keys = keys.concat(Object.keys(demandedOptions));
            keys = keys.concat(Object.keys(demandedCommands));
            keys = keys.concat(Object.keys(options.default));
            keys = keys.filter(filterHiddenOptions);
            keys = Object.keys(keys.reduce((acc, key) => {
                if (key !== '_')
                    acc[key] = true;
                return acc;
            }, {}));
            const theWrap = getWrap();
            const ui = require(23) /* cliui */({
                width: theWrap,
                wrap: !!theWrap
            });
            // the usage string.
            if (!usageDisabled) {
                if (usages.length) {
                    // user-defined usage.
                    usages.forEach((usage) => {
                        ui.div(`${usage[0].replace(/\$0/g, base$0)}`);
                        if (usage[1]) {
                            ui.div({ text: `${usage[1]}`, padding: [1, 0, 0, 0] });
                        }
                    });
                    ui.div();
                }
                else if (commands.length) {
                    let u = null;
                    // demonstrate how commands are used.
                    if (demandedCommands._) {
                        u = `${base$0} <${__('command')}>\n`;
                    }
                    else {
                        u = `${base$0} [${__('command')}]\n`;
                    }
                    ui.div(`${u}`);
                }
            }
            // your application's commands, i.e., non-option
            // arguments populated in '_'.
            if (commands.length) {
                ui.div(__('Commands:'));
                const context = yargs.getContext();
                const parentCommands = context.commands.length ? `${context.commands.join(' ')} ` : '';
                if (yargs.getParserConfiguration()['sort-commands'] === true) {
                    commands = commands.sort((a, b) => a[0].localeCompare(b[0]));
                }
                commands.forEach((command) => {
                    const commandString = `${base$0} ${parentCommands}${command[0].replace(/^\$0 ?/, '')}`; // drop $0 from default commands.
                    ui.span({
                        text: commandString,
                        padding: [0, 2, 0, 2],
                        width: maxWidth(commands, theWrap, `${base$0}${parentCommands}`) + 4
                    }, { text: command[1] });
                    const hints = [];
                    if (command[2])
                        hints.push(`[${__('default')}]`);
                    if (command[3] && command[3].length) {
                        hints.push(`[${__('aliases:')} ${command[3].join(', ')}]`);
                    }
                    if (hints.length) {
                        ui.div({ text: hints.join(' '), padding: [0, 0, 0, 2], align: 'right' });
                    }
                    else {
                        ui.div();
                    }
                });
                ui.div();
            }
            // perform some cleanup on the keys array, making it
            // only include top-level keys not their aliases.
            const aliasKeys = (Object.keys(options.alias) || [])
                .concat(Object.keys(yargs.parsed.newAliases) || []);
            keys = keys.filter(key => !yargs.parsed.newAliases[key] && aliasKeys.every(alias => (options.alias[alias] || []).indexOf(key) === -1));
            // populate 'Options:' group with any keys that have not
            // explicitly had a group set.
            if (!groups[defaultGroup])
                groups[defaultGroup] = [];
            addUngroupedKeys(keys, options.alias, groups);
            // display 'Options:' table along with any custom tables:
            Object.keys(groups).forEach((groupName) => {
                if (!groups[groupName].length)
                    return;
                // if we've grouped the key 'f', but 'f' aliases 'foobar',
                // normalizedKeys should contain only 'foobar'.
                const normalizedKeys = groups[groupName].filter(filterHiddenOptions).map((key) => {
                    if (~aliasKeys.indexOf(key))
                        return key;
                    for (let i = 0, aliasKey; (aliasKey = aliasKeys[i]) !== undefined; i++) {
                        if (~(options.alias[aliasKey] || []).indexOf(key))
                            return aliasKey;
                    }
                    return key;
                });
                if (normalizedKeys.length < 1)
                    return;
                ui.div(groupName);
                // actually generate the switches string --foo, -f, --bar.
                const switches = normalizedKeys.reduce((acc, key) => {
                    acc[key] = [key].concat(options.alias[key] || [])
                        .map(sw => {
                        // for the special positional group don't
                        // add '--' or '-' prefix.
                        if (groupName === self.getPositionalGroupName())
                            return sw;
                        else {
                            return (
                            // matches yargs-parser logic in which single-digits
                            // aliases declared with a boolean type are now valid
                            /^[0-9]$/.test(sw)
                                ? ~options.boolean.indexOf(key) ? '-' : '--'
                                : sw.length > 1 ? '--' : '-') + sw;
                        }
                    })
                        .join(', ');
                    return acc;
                }, {});
                normalizedKeys.forEach((key) => {
                    const kswitch = switches[key];
                    let desc = descriptions[key] || '';
                    let type = null;
                    if (~desc.lastIndexOf(deferY18nLookupPrefix))
                        desc = __(desc.substring(deferY18nLookupPrefix.length));
                    if (~options.boolean.indexOf(key))
                        type = `[${__('boolean')}]`;
                    if (~options.count.indexOf(key))
                        type = `[${__('count')}]`;
                    if (~options.string.indexOf(key))
                        type = `[${__('string')}]`;
                    if (~options.normalize.indexOf(key))
                        type = `[${__('string')}]`;
                    if (~options.array.indexOf(key))
                        type = `[${__('array')}]`;
                    if (~options.number.indexOf(key))
                        type = `[${__('number')}]`;
                    const extra = [
                        (key in deprecatedOptions) ? (typeof deprecatedOptions[key] === 'string'
                            ? `[${__('deprecated: %s', deprecatedOptions[key])}]`
                            : `[${__('deprecated')}]`) : null,
                        type,
                        (key in demandedOptions) ? `[${__('required')}]` : null,
                        options.choices && options.choices[key] ? `[${__('choices:')} ${self.stringifiedValues(options.choices[key])}]` : null,
                        defaultString(options.default[key], options.defaultDescription[key])
                    ].filter(Boolean).join(' ');
                    ui.span({ text: kswitch, padding: [0, 2, 0, 2], width: maxWidth(switches, theWrap) + 4 }, desc);
                    if (extra)
                        ui.div({ text: extra, padding: [0, 0, 0, 2], align: 'right' });
                    else
                        ui.div();
                });
                ui.div();
            });
            // describe some common use-cases for your application.
            if (examples.length) {
                ui.div(__('Examples:'));
                examples.forEach((example) => {
                    example[0] = example[0].replace(/\$0/g, base$0);
                });
                examples.forEach((example) => {
                    if (example[1] === '') {
                        ui.div({
                            text: example[0],
                            padding: [0, 2, 0, 2]
                        });
                    }
                    else {
                        ui.div({
                            text: example[0],
                            padding: [0, 2, 0, 2],
                            width: maxWidth(examples, theWrap) + 4
                        }, {
                            text: example[1]
                        });
                    }
                });
                ui.div();
            }
            // the usage string.
            if (epilogs.length > 0) {
                const e = epilogs.map(epilog => epilog.replace(/\$0/g, base$0)).join('\n');
                ui.div(`${e}\n`);
            }
            // Remove the trailing white spaces
            return ui.toString().replace(/\s*$/, '');
        };
        // return the maximum width of a string
        // in the left-hand column of a table.
        function maxWidth(table, theWrap, modifier) {
            let width = 0;
            // table might be of the form [leftColumn],
            // or {key: leftColumn}
            if (!Array.isArray(table)) {
                table = Object.keys(table).map(key => [table[key]]);
            }
            table.forEach((v) => {
                width = Math.max(stringWidth(modifier ? `${modifier} ${v[0]}` : v[0]), width);
            });
            // if we've enabled 'wrap' we should limit
            // the max-width of the left-column.
            if (theWrap)
                width = Math.min(width, parseInt(theWrap * 0.5, 10));
            return width;
        }
        // make sure any options set for aliases,
        // are copied to the keys being aliased.
        function normalizeAliases() {
            // handle old demanded API
            const demandedOptions = yargs.getDemandedOptions();
            const options = yargs.getOptions();
            (Object.keys(options.alias) || []).forEach((key) => {
                options.alias[key].forEach((alias) => {
                    // copy descriptions.
                    if (descriptions[alias])
                        self.describe(key, descriptions[alias]);
                    // copy demanded.
                    if (alias in demandedOptions)
                        yargs.demandOption(key, demandedOptions[alias]);
                    // type messages.
                    if (~options.boolean.indexOf(alias))
                        yargs.boolean(key);
                    if (~options.count.indexOf(alias))
                        yargs.count(key);
                    if (~options.string.indexOf(alias))
                        yargs.string(key);
                    if (~options.normalize.indexOf(alias))
                        yargs.normalize(key);
                    if (~options.array.indexOf(alias))
                        yargs.array(key);
                    if (~options.number.indexOf(alias))
                        yargs.number(key);
                });
            });
        }
        // if yargs is executing an async handler, we take a snapshot of the
        // help message to display on failure:
        let cachedHelpMessage;
        self.cacheHelpMessage = function () {
            cachedHelpMessage = this.help();
        };
        // however this snapshot must be cleared afterwards
        // not to be be used by next calls to parse
        self.clearCachedHelpMessage = function () {
            cachedHelpMessage = undefined;
        };
        // given a set of keys, place any keys that are
        // ungrouped under the 'Options:' grouping.
        function addUngroupedKeys(keys, aliases, groups) {
            let groupedKeys = [];
            let toCheck = null;
            Object.keys(groups).forEach((group) => {
                groupedKeys = groupedKeys.concat(groups[group]);
            });
            keys.forEach((key) => {
                toCheck = [key].concat(aliases[key]);
                if (!toCheck.some(k => groupedKeys.indexOf(k) !== -1)) {
                    groups[defaultGroup].push(key);
                }
            });
            return groupedKeys;
        }
        function filterHiddenOptions(key) {
            return yargs.getOptions().hiddenOptions.indexOf(key) < 0 || yargs.parsed.argv[yargs.getOptions().showHiddenOpt];
        }
        self.showHelp = (level) => {
            const logger = yargs._getLoggerInstance();
            if (!level)
                level = 'error';
            const emit = typeof level === 'function' ? level : logger[level];
            emit(self.help());
        };
        self.functionDescription = (fn) => {
            const description = fn.name ? decamelize(fn.name, '-') : __('generated-value');
            return ['(', description, ')'].join('');
        };
        self.stringifiedValues = function stringifiedValues(values, separator) {
            let string = '';
            const sep = separator || ', ';
            const array = [].concat(values);
            if (!values || !array.length)
                return string;
            array.forEach((value) => {
                if (string.length)
                    string += sep;
                string += JSON.stringify(value);
            });
            return string;
        };
        // format the default-value-string displayed in
        // the right-hand column.
        function defaultString(value, defaultDescription) {
            let string = `[${__('default:')} `;
            if (value === undefined && !defaultDescription)
                return null;
            if (defaultDescription) {
                string += defaultDescription;
            }
            else {
                switch (typeof value) {
                    case 'string':
                        string += `"${value}"`;
                        break;
                    case 'object':
                        string += JSON.stringify(value);
                        break;
                    default:
                        string += value;
                }
            }
            return `${string}]`;
        }
        // guess the width of the console window, max-width 80.
        function windowWidth() {
            const maxWidth = 80;
            // CI is not a TTY
            /* c8 ignore next 2 */
            if (typeof process === 'object' && process.stdout && process.stdout.columns) {
                return Math.min(maxWidth, process.stdout.columns);
            }
            else {
                return maxWidth;
            }
        }
        // logic for displaying application version.
        let version = null;
        self.version = (ver) => {
            version = ver;
        };
        self.showVersion = () => {
            const logger = yargs._getLoggerInstance();
            logger.log(version);
        };
        self.reset = function reset(localLookup) {
            // do not reset wrap here
            // do not reset fails here
            failMessage = null;
            failureOutput = false;
            usages = [];
            usageDisabled = false;
            epilogs = [];
            examples = [];
            commands = [];
            descriptions = objFilter(descriptions, (k, v) => !localLookup[k]);
            return self;
        };
        const frozens = [];
        self.freeze = function freeze() {
            const frozen = {};
            frozens.push(frozen);
            frozen.failMessage = failMessage;
            frozen.failureOutput = failureOutput;
            frozen.usages = usages;
            frozen.usageDisabled = usageDisabled;
            frozen.epilogs = epilogs;
            frozen.examples = examples;
            frozen.commands = commands;
            frozen.descriptions = descriptions;
        };
        self.unfreeze = function unfreeze() {
            const frozen = frozens.pop();
            failMessage = frozen.failMessage;
            failureOutput = frozen.failureOutput;
            usages = frozen.usages;
            usageDisabled = frozen.usageDisabled;
            epilogs = frozen.epilogs;
            examples = frozen.examples;
            commands = frozen.commands;
            descriptions = frozen.descriptions;
        };
        return self;
    };
},
/* string-width/index.js */ function _(require, module, exports) {
    const stripAnsi = require(18) /* strip-ansi */;
    const isFullwidthCodePoint = require(20) /* is-fullwidth-code-point */;
    module.exports = str => {
        if (typeof str !== 'string' || str.length === 0) {
            return 0;
        }
        str = stripAnsi(str);
        let width = 0;
        for (let i = 0; i < str.length; i++) {
            const code = str.codePointAt(i);
            // Ignore control characters
            if (code <= 0x1F || (code >= 0x7F && code <= 0x9F)) {
                continue;
            }
            // Ignore combining characters
            if (code >= 0x300 && code <= 0x36F) {
                continue;
            }
            // Surrogates
            if (code > 0xFFFF) {
                i++;
            }
            width += isFullwidthCodePoint(code) ? 2 : 1;
        }
        return width;
    };
},
/* strip-ansi/index.js */ function _(require, module, exports) {
    const ansiRegex = require(19) /* ansi-regex */;
    module.exports = input => typeof input === 'string' ? input.replace(ansiRegex(), '') : input;
},
/* ansi-regex/index.js */ function _(require, module, exports) {
    module.exports = () => {
        const pattern = [
            '[\\u001B\\u009B][[\\]()#;?]*(?:(?:(?:[a-zA-Z\\d]*(?:;[a-zA-Z\\d]*)*)?\\u0007)',
            '(?:(?:\\d{1,4}(?:;\\d{0,4})*)?[\\dA-PRZcf-ntqry=><~]))'
        ].join('|');
        return new RegExp(pattern, 'g');
    };
},
/* is-fullwidth-code-point/index.js */ function _(require, module, exports) {
    /* eslint-disable yoda */
    module.exports = x => {
        if (Number.isNaN(x)) {
            return false;
        }
        // code points are derived from:
        // http://www.unix.org/Public/UNIDATA/EastAsianWidth.txt
        if (x >= 0x1100 && (x <= 0x115f || // Hangul Jamo
            x === 0x2329 || // LEFT-POINTING ANGLE BRACKET
            x === 0x232a || // RIGHT-POINTING ANGLE BRACKET
            // CJK Radicals Supplement .. Enclosed CJK Letters and Months
            (0x2e80 <= x && x <= 0x3247 && x !== 0x303f) ||
            // Enclosed CJK Letters and Months .. CJK Unified Ideographs Extension A
            (0x3250 <= x && x <= 0x4dbf) ||
            // CJK Unified Ideographs .. Yi Radicals
            (0x4e00 <= x && x <= 0xa4c6) ||
            // Hangul Jamo Extended-A
            (0xa960 <= x && x <= 0xa97c) ||
            // Hangul Syllables
            (0xac00 <= x && x <= 0xd7a3) ||
            // CJK Compatibility Ideographs
            (0xf900 <= x && x <= 0xfaff) ||
            // Vertical Forms
            (0xfe10 <= x && x <= 0xfe19) ||
            // CJK Compatibility Forms .. Small Form Variants
            (0xfe30 <= x && x <= 0xfe6b) ||
            // Halfwidth and Fullwidth Forms
            (0xff01 <= x && x <= 0xff60) ||
            (0xffe0 <= x && x <= 0xffe6) ||
            // Kana Supplement
            (0x1b000 <= x && x <= 0x1b001) ||
            // Enclosed Ideographic Supplement
            (0x1f200 <= x && x <= 0x1f251) ||
            // CJK Unified Ideographs Extension B .. Tertiary Ideographic Plane
            (0x20000 <= x && x <= 0x3fffd))) {
            return true;
        }
        return false;
    };
},
/* yargs/lib/obj-filter.js */ function _(require, module, exports) {
    module.exports = function objFilter(original, filter) {
        const obj = {};
        filter = filter || ((k, v) => true);
        Object.keys(original || {}).forEach((key) => {
            if (filter(key, original[key])) {
                obj[key] = original[key];
            }
        });
        return obj;
    };
},
/* set-blocking/index.js */ function _(require, module, exports) {
    module.exports = function (blocking) {
        [process.stdout, process.stderr].forEach(function (stream) {
            if (stream._handle && stream.isTTY && typeof stream._handle.setBlocking === 'function') {
                stream._handle.setBlocking(blocking);
            }
        });
    };
},
/* cliui/index.js */ function _(require, module, exports) {
    var stringWidth = require(17) /* string-width */;
    var stripAnsi = require(18) /* strip-ansi */;
    var wrap = require(24) /* wrap-ansi */;
    var align = {
        right: alignRight,
        center: alignCenter
    };
    var top = 0;
    var right = 1;
    var bottom = 2;
    var left = 3;
    function UI(opts) {
        this.width = opts.width;
        this.wrap = opts.wrap;
        this.rows = [];
    }
    UI.prototype.span = function () {
        var cols = this.div.apply(this, arguments);
        cols.span = true;
    };
    UI.prototype.resetOutput = function () {
        this.rows = [];
    };
    UI.prototype.div = function () {
        if (arguments.length === 0)
            this.div('');
        if (this.wrap && this._shouldApplyLayoutDSL.apply(this, arguments)) {
            return this._applyLayoutDSL(arguments[0]);
        }
        var cols = [];
        for (var i = 0, arg; (arg = arguments[i]) !== undefined; i++) {
            if (typeof arg === 'string')
                cols.push(this._colFromString(arg));
            else
                cols.push(arg);
        }
        this.rows.push(cols);
        return cols;
    };
    UI.prototype._shouldApplyLayoutDSL = function () {
        return arguments.length === 1 && typeof arguments[0] === 'string' &&
            /[\t\n]/.test(arguments[0]);
    };
    UI.prototype._applyLayoutDSL = function (str) {
        var _this = this;
        var rows = str.split('\n');
        var leftColumnWidth = 0;
        // simple heuristic for layout, make sure the
        // second column lines up along the left-hand.
        // don't allow the first column to take up more
        // than 50% of the screen.
        rows.forEach(function (row) {
            var columns = row.split('\t');
            if (columns.length > 1 && stringWidth(columns[0]) > leftColumnWidth) {
                leftColumnWidth = Math.min(Math.floor(_this.width * 0.5), stringWidth(columns[0]));
            }
        });
        // generate a table:
        //  replacing ' ' with padding calculations.
        //  using the algorithmically generated width.
        rows.forEach(function (row) {
            var columns = row.split('\t');
            _this.div.apply(_this, columns.map(function (r, i) {
                return {
                    text: r.trim(),
                    padding: _this._measurePadding(r),
                    width: (i === 0 && columns.length > 1) ? leftColumnWidth : undefined
                };
            }));
        });
        return this.rows[this.rows.length - 1];
    };
    UI.prototype._colFromString = function (str) {
        return {
            text: str,
            padding: this._measurePadding(str)
        };
    };
    UI.prototype._measurePadding = function (str) {
        // measure padding without ansi escape codes
        var noAnsi = stripAnsi(str);
        return [0, noAnsi.match(/\s*$/)[0].length, 0, noAnsi.match(/^\s*/)[0].length];
    };
    UI.prototype.toString = function () {
        var _this = this;
        var lines = [];
        _this.rows.forEach(function (row, i) {
            _this.rowToString(row, lines);
        });
        // don't display any lines with the
        // hidden flag set.
        lines = lines.filter(function (line) {
            return !line.hidden;
        });
        return lines.map(function (line) {
            return line.text;
        }).join('\n');
    };
    UI.prototype.rowToString = function (row, lines) {
        var _this = this;
        var padding;
        var rrows = this._rasterize(row);
        var str = '';
        var ts;
        var width;
        var wrapWidth;
        rrows.forEach(function (rrow, r) {
            str = '';
            rrow.forEach(function (col, c) {
                ts = ''; // temporary string used during alignment/padding.
                width = row[c].width; // the width with padding.
                wrapWidth = _this._negatePadding(row[c]); // the width without padding.
                ts += col;
                for (var i = 0; i < wrapWidth - stringWidth(col); i++) {
                    ts += ' ';
                }
                // align the string within its column.
                if (row[c].align && row[c].align !== 'left' && _this.wrap) {
                    ts = align[row[c].align](ts, wrapWidth);
                    if (stringWidth(ts) < wrapWidth)
                        ts += new Array(width - stringWidth(ts)).join(' ');
                }
                // apply border and padding to string.
                padding = row[c].padding || [0, 0, 0, 0];
                if (padding[left])
                    str += new Array(padding[left] + 1).join(' ');
                str += addBorder(row[c], ts, '| ');
                str += ts;
                str += addBorder(row[c], ts, ' |');
                if (padding[right])
                    str += new Array(padding[right] + 1).join(' ');
                // if prior row is span, try to render the
                // current row on the prior line.
                if (r === 0 && lines.length > 0) {
                    str = _this._renderInline(str, lines[lines.length - 1]);
                }
            });
            // remove trailing whitespace.
            lines.push({
                text: str.replace(/ +$/, ''),
                span: row.span
            });
        });
        return lines;
    };
    function addBorder(col, ts, style) {
        if (col.border) {
            if (/[.']-+[.']/.test(ts))
                return '';
            else if (ts.trim().length)
                return style;
            else
                return '  ';
        }
        return '';
    }
    // if the full 'source' can render in
    // the target line, do so.
    UI.prototype._renderInline = function (source, previousLine) {
        var leadingWhitespace = source.match(/^ */)[0].length;
        var target = previousLine.text;
        var targetTextWidth = stringWidth(target.trimRight());
        if (!previousLine.span)
            return source;
        // if we're not applying wrapping logic,
        // just always append to the span.
        if (!this.wrap) {
            previousLine.hidden = true;
            return target + source;
        }
        if (leadingWhitespace < targetTextWidth)
            return source;
        previousLine.hidden = true;
        return target.trimRight() + new Array(leadingWhitespace - targetTextWidth + 1).join(' ') + source.trimLeft();
    };
    UI.prototype._rasterize = function (row) {
        var _this = this;
        var i;
        var rrow;
        var rrows = [];
        var widths = this._columnWidths(row);
        var wrapped;
        // word wrap all columns, and create
        // a data-structure that is easy to rasterize.
        row.forEach(function (col, c) {
            // leave room for left and right padding.
            col.width = widths[c];
            if (_this.wrap)
                wrapped = wrap(col.text, _this._negatePadding(col), { hard: true }).split('\n');
            else
                wrapped = col.text.split('\n');
            if (col.border) {
                wrapped.unshift('.' + new Array(_this._negatePadding(col) + 3).join('-') + '.');
                wrapped.push("'" + new Array(_this._negatePadding(col) + 3).join('-') + "'");
            }
            // add top and bottom padding.
            if (col.padding) {
                for (i = 0; i < (col.padding[top] || 0); i++)
                    wrapped.unshift('');
                for (i = 0; i < (col.padding[bottom] || 0); i++)
                    wrapped.push('');
            }
            wrapped.forEach(function (str, r) {
                if (!rrows[r])
                    rrows.push([]);
                rrow = rrows[r];
                for (var i = 0; i < c; i++) {
                    if (rrow[i] === undefined)
                        rrow.push('');
                }
                rrow.push(str);
            });
        });
        return rrows;
    };
    UI.prototype._negatePadding = function (col) {
        var wrapWidth = col.width;
        if (col.padding)
            wrapWidth -= (col.padding[left] || 0) + (col.padding[right] || 0);
        if (col.border)
            wrapWidth -= 4;
        return wrapWidth;
    };
    UI.prototype._columnWidths = function (row) {
        var _this = this;
        var widths = [];
        var unset = row.length;
        var unsetWidth;
        var remainingWidth = this.width;
        // column widths can be set in config.
        row.forEach(function (col, i) {
            if (col.width) {
                unset--;
                widths[i] = col.width;
                remainingWidth -= col.width;
            }
            else {
                widths[i] = undefined;
            }
        });
        // any unset widths should be calculated.
        if (unset)
            unsetWidth = Math.floor(remainingWidth / unset);
        widths.forEach(function (w, i) {
            if (!_this.wrap)
                widths[i] = row[i].width || stringWidth(row[i].text);
            else if (w === undefined)
                widths[i] = Math.max(unsetWidth, _minWidth(row[i]));
        });
        return widths;
    };
    // calculates the minimum width of
    // a column, based on padding preferences.
    function _minWidth(col) {
        var padding = col.padding || [];
        var minWidth = 1 + (padding[left] || 0) + (padding[right] || 0);
        if (col.border)
            minWidth += 4;
        return minWidth;
    }
    function getWindowWidth() {
        if (typeof process === 'object' && process.stdout && process.stdout.columns)
            return process.stdout.columns;
    }
    function alignRight(str, width) {
        str = str.trim();
        var padding = '';
        var strWidth = stringWidth(str);
        if (strWidth < width) {
            padding = new Array(width - strWidth + 1).join(' ');
        }
        return padding + str;
    }
    function alignCenter(str, width) {
        str = str.trim();
        var padding = '';
        var strWidth = stringWidth(str.trim());
        if (strWidth < width) {
            padding = new Array(parseInt((width - strWidth) / 2, 10) + 1).join(' ');
        }
        return padding + str;
    }
    module.exports = function (opts) {
        opts = opts || {};
        return new UI({
            width: (opts || {}).width || getWindowWidth() || 80,
            wrap: typeof opts.wrap === 'boolean' ? opts.wrap : true
        });
    };
},
/* wrap-ansi/index.js */ function _(require, module, exports) {
    const stringWidth = require(17) /* string-width */;
    const stripAnsi = require(18) /* strip-ansi */;
    const ansiStyles = require(25) /* ansi-styles */;
    const ESCAPES = new Set([
        '\u001B',
        '\u009B'
    ]);
    const END_CODE = 39;
    const wrapAnsi = code => `${ESCAPES.values().next().value}[${code}m`;
    // Calculate the length of words split on ' ', ignoring
    // the extra characters added by ansi escape codes
    const wordLengths = string => string.split(' ').map(character => stringWidth(character));
    // Wrap a long word across multiple rows
    // Ansi escape codes do not count towards length
    const wrapWord = (rows, word, columns) => {
        const characters = [...word];
        let insideEscape = false;
        let visible = stringWidth(stripAnsi(rows[rows.length - 1]));
        for (const [index, character] of characters.entries()) {
            const characterLength = stringWidth(character);
            if (visible + characterLength <= columns) {
                rows[rows.length - 1] += character;
            }
            else {
                rows.push(character);
                visible = 0;
            }
            if (ESCAPES.has(character)) {
                insideEscape = true;
            }
            else if (insideEscape && character === 'm') {
                insideEscape = false;
                continue;
            }
            if (insideEscape) {
                continue;
            }
            visible += characterLength;
            if (visible === columns && index < characters.length - 1) {
                rows.push('');
                visible = 0;
            }
        }
        // It's possible that the last row we copy over is only
        // ansi escape characters, handle this edge-case
        if (!visible && rows[rows.length - 1].length > 0 && rows.length > 1) {
            rows[rows.length - 2] += rows.pop();
        }
    };
    // Trims spaces from a string ignoring invisible sequences
    const stringVisibleTrimSpacesRight = str => {
        const words = str.split(' ');
        let last = words.length;
        while (last > 0) {
            if (stringWidth(words[last - 1]) > 0) {
                break;
            }
            last--;
        }
        if (last === words.length) {
            return str;
        }
        return words.slice(0, last).join(' ') + words.slice(last).join('');
    };
    // The wrap-ansi module can be invoked
    // in either 'hard' or 'soft' wrap mode
    //
    // 'hard' will never allow a string to take up more
    // than columns characters
    //
    // 'soft' allows long words to expand past the column length
    const exec = (string, columns, options = {}) => {
        if (options.trim !== false && string.trim() === '') {
            return '';
        }
        let pre = '';
        let ret = '';
        let escapeCode;
        const lengths = wordLengths(string);
        let rows = [''];
        for (const [index, word] of string.split(' ').entries()) {
            if (options.trim !== false) {
                rows[rows.length - 1] = rows[rows.length - 1].trimLeft();
            }
            let rowLength = stringWidth(rows[rows.length - 1]);
            if (index !== 0) {
                if (rowLength >= columns && (options.wordWrap === false || options.trim === false)) {
                    // If we start with a new word but the current row length equals the length of the columns, add a new row
                    rows.push('');
                    rowLength = 0;
                }
                if (rowLength > 0 || options.trim === false) {
                    rows[rows.length - 1] += ' ';
                    rowLength++;
                }
            }
            // In 'hard' wrap mode, the length of a line is
            // never allowed to extend past 'columns'
            if (options.hard && lengths[index] > columns) {
                const remainingColumns = (columns - rowLength);
                const breaksStartingThisLine = 1 + Math.floor((lengths[index] - remainingColumns - 1) / columns);
                const breaksStartingNextLine = Math.floor((lengths[index] - 1) / columns);
                if (breaksStartingNextLine < breaksStartingThisLine) {
                    rows.push('');
                }
                wrapWord(rows, word, columns);
                continue;
            }
            if (rowLength + lengths[index] > columns && rowLength > 0 && lengths[index] > 0) {
                if (options.wordWrap === false && rowLength < columns) {
                    wrapWord(rows, word, columns);
                    continue;
                }
                rows.push('');
            }
            if (rowLength + lengths[index] > columns && options.wordWrap === false) {
                wrapWord(rows, word, columns);
                continue;
            }
            rows[rows.length - 1] += word;
        }
        if (options.trim !== false) {
            rows = rows.map(stringVisibleTrimSpacesRight);
        }
        pre = rows.join('\n');
        for (const [index, character] of [...pre].entries()) {
            ret += character;
            if (ESCAPES.has(character)) {
                const code = parseFloat(/\d[^m]*/.exec(pre.slice(index, index + 4)));
                escapeCode = code === END_CODE ? null : code;
            }
            const code = ansiStyles.codes.get(Number(escapeCode));
            if (escapeCode && code) {
                if (pre[index + 1] === '\n') {
                    ret += wrapAnsi(code);
                }
                else if (character === '\n') {
                    ret += wrapAnsi(escapeCode);
                }
            }
        }
        return ret;
    };
    // For each newline, invoke the method separately
    module.exports = (string, columns, options) => {
        return String(string)
            .normalize()
            .split('\n')
            .map(line => exec(line, columns, options))
            .join('\n');
    };
},
/* ansi-styles/index.js */ function _(require, module, exports) {
    const wrapAnsi16 = (fn, offset) => (...args) => {
        const code = fn(...args);
        return `\u001B[${code + offset}m`;
    };
    const wrapAnsi256 = (fn, offset) => (...args) => {
        const code = fn(...args);
        return `\u001B[${38 + offset};5;${code}m`;
    };
    const wrapAnsi16m = (fn, offset) => (...args) => {
        const rgb = fn(...args);
        return `\u001B[${38 + offset};2;${rgb[0]};${rgb[1]};${rgb[2]}m`;
    };
    const ansi2ansi = n => n;
    const rgb2rgb = (r, g, b) => [r, g, b];
    const setLazyProperty = (object, property, get) => {
        Object.defineProperty(object, property, {
            get: () => {
                const value = get();
                Object.defineProperty(object, property, {
                    value,
                    enumerable: true,
                    configurable: true
                });
                return value;
            },
            enumerable: true,
            configurable: true
        });
    };
    /** @type {typeof import('color-convert')} */
    let colorConvert;
    const makeDynamicStyles = (wrap, targetSpace, identity, isBackground) => {
        if (colorConvert === undefined) {
            colorConvert = require(26) /* color-convert */;
        }
        const offset = isBackground ? 10 : 0;
        const styles = {};
        for (const [sourceSpace, suite] of Object.entries(colorConvert)) {
            const name = sourceSpace === 'ansi16' ? 'ansi' : sourceSpace;
            if (sourceSpace === targetSpace) {
                styles[name] = wrap(identity, offset);
            }
            else if (typeof suite === 'object') {
                styles[name] = wrap(suite[targetSpace], offset);
            }
        }
        return styles;
    };
    function assembleStyles() {
        const codes = new Map();
        const styles = {
            modifier: {
                reset: [0, 0],
                // 21 isn't widely supported and 22 does the same thing
                bold: [1, 22],
                dim: [2, 22],
                italic: [3, 23],
                underline: [4, 24],
                inverse: [7, 27],
                hidden: [8, 28],
                strikethrough: [9, 29]
            },
            color: {
                black: [30, 39],
                red: [31, 39],
                green: [32, 39],
                yellow: [33, 39],
                blue: [34, 39],
                magenta: [35, 39],
                cyan: [36, 39],
                white: [37, 39],
                // Bright color
                blackBright: [90, 39],
                redBright: [91, 39],
                greenBright: [92, 39],
                yellowBright: [93, 39],
                blueBright: [94, 39],
                magentaBright: [95, 39],
                cyanBright: [96, 39],
                whiteBright: [97, 39]
            },
            bgColor: {
                bgBlack: [40, 49],
                bgRed: [41, 49],
                bgGreen: [42, 49],
                bgYellow: [43, 49],
                bgBlue: [44, 49],
                bgMagenta: [45, 49],
                bgCyan: [46, 49],
                bgWhite: [47, 49],
                // Bright color
                bgBlackBright: [100, 49],
                bgRedBright: [101, 49],
                bgGreenBright: [102, 49],
                bgYellowBright: [103, 49],
                bgBlueBright: [104, 49],
                bgMagentaBright: [105, 49],
                bgCyanBright: [106, 49],
                bgWhiteBright: [107, 49]
            }
        };
        // Alias bright black as gray (and grey)
        styles.color.gray = styles.color.blackBright;
        styles.bgColor.bgGray = styles.bgColor.bgBlackBright;
        styles.color.grey = styles.color.blackBright;
        styles.bgColor.bgGrey = styles.bgColor.bgBlackBright;
        for (const [groupName, group] of Object.entries(styles)) {
            for (const [styleName, style] of Object.entries(group)) {
                styles[styleName] = {
                    open: `\u001B[${style[0]}m`,
                    close: `\u001B[${style[1]}m`
                };
                group[styleName] = styles[styleName];
                codes.set(style[0], style[1]);
            }
            Object.defineProperty(styles, groupName, {
                value: group,
                enumerable: false
            });
        }
        Object.defineProperty(styles, 'codes', {
            value: codes,
            enumerable: false
        });
        styles.color.close = '\u001B[39m';
        styles.bgColor.close = '\u001B[49m';
        setLazyProperty(styles.color, 'ansi', () => makeDynamicStyles(wrapAnsi16, 'ansi16', ansi2ansi, false));
        setLazyProperty(styles.color, 'ansi256', () => makeDynamicStyles(wrapAnsi256, 'ansi256', ansi2ansi, false));
        setLazyProperty(styles.color, 'ansi16m', () => makeDynamicStyles(wrapAnsi16m, 'rgb', rgb2rgb, false));
        setLazyProperty(styles.bgColor, 'ansi', () => makeDynamicStyles(wrapAnsi16, 'ansi16', ansi2ansi, true));
        setLazyProperty(styles.bgColor, 'ansi256', () => makeDynamicStyles(wrapAnsi256, 'ansi256', ansi2ansi, true));
        setLazyProperty(styles.bgColor, 'ansi16m', () => makeDynamicStyles(wrapAnsi16m, 'rgb', rgb2rgb, true));
        return styles;
    }
    // Make the export immutable
    Object.defineProperty(module, 'exports', {
        enumerable: true,
        get: assembleStyles
    });
},
/* color-convert/index.js */ function _(require, module, exports) {
    const conversions = require(27) /* ./conversions */;
    const route = require(29) /* ./route */;
    const convert = {};
    const models = Object.keys(conversions);
    function wrapRaw(fn) {
        const wrappedFn = function (...args) {
            const arg0 = args[0];
            if (arg0 === undefined || arg0 === null) {
                return arg0;
            }
            if (arg0.length > 1) {
                args = arg0;
            }
            return fn(args);
        };
        // Preserve .conversion property if there is one
        if ('conversion' in fn) {
            wrappedFn.conversion = fn.conversion;
        }
        return wrappedFn;
    }
    function wrapRounded(fn) {
        const wrappedFn = function (...args) {
            const arg0 = args[0];
            if (arg0 === undefined || arg0 === null) {
                return arg0;
            }
            if (arg0.length > 1) {
                args = arg0;
            }
            const result = fn(args);
            // We're assuming the result is an array here.
            // see notice in conversions.js; don't use box types
            // in conversion functions.
            if (typeof result === 'object') {
                for (let len = result.length, i = 0; i < len; i++) {
                    result[i] = Math.round(result[i]);
                }
            }
            return result;
        };
        // Preserve .conversion property if there is one
        if ('conversion' in fn) {
            wrappedFn.conversion = fn.conversion;
        }
        return wrappedFn;
    }
    models.forEach(fromModel => {
        convert[fromModel] = {};
        Object.defineProperty(convert[fromModel], 'channels', { value: conversions[fromModel].channels });
        Object.defineProperty(convert[fromModel], 'labels', { value: conversions[fromModel].labels });
        const routes = route(fromModel);
        const routeModels = Object.keys(routes);
        routeModels.forEach(toModel => {
            const fn = routes[toModel];
            convert[fromModel][toModel] = wrapRounded(fn);
            convert[fromModel][toModel].raw = wrapRaw(fn);
        });
    });
    module.exports = convert;
},
/* color-convert/conversions.js */ function _(require, module, exports) {
    /* MIT license */
    /* eslint-disable no-mixed-operators */
    const cssKeywords = require(28) /* color-name */;
    // NOTE: conversions should only return primitive values (i.e. arrays, or
    //       values that give correct `typeof` results).
    //       do not use box values types (i.e. Number(), String(), etc.)
    const reverseKeywords = {};
    for (const key of Object.keys(cssKeywords)) {
        reverseKeywords[cssKeywords[key]] = key;
    }
    const convert = {
        rgb: { channels: 3, labels: 'rgb' },
        hsl: { channels: 3, labels: 'hsl' },
        hsv: { channels: 3, labels: 'hsv' },
        hwb: { channels: 3, labels: 'hwb' },
        cmyk: { channels: 4, labels: 'cmyk' },
        xyz: { channels: 3, labels: 'xyz' },
        lab: { channels: 3, labels: 'lab' },
        lch: { channels: 3, labels: 'lch' },
        hex: { channels: 1, labels: ['hex'] },
        keyword: { channels: 1, labels: ['keyword'] },
        ansi16: { channels: 1, labels: ['ansi16'] },
        ansi256: { channels: 1, labels: ['ansi256'] },
        hcg: { channels: 3, labels: ['h', 'c', 'g'] },
        apple: { channels: 3, labels: ['r16', 'g16', 'b16'] },
        gray: { channels: 1, labels: ['gray'] }
    };
    module.exports = convert;
    // Hide .channels and .labels properties
    for (const model of Object.keys(convert)) {
        if (!('channels' in convert[model])) {
            throw new Error('missing channels property: ' + model);
        }
        if (!('labels' in convert[model])) {
            throw new Error('missing channel labels property: ' + model);
        }
        if (convert[model].labels.length !== convert[model].channels) {
            throw new Error('channel and label counts mismatch: ' + model);
        }
        const { channels, labels } = convert[model];
        delete convert[model].channels;
        delete convert[model].labels;
        Object.defineProperty(convert[model], 'channels', { value: channels });
        Object.defineProperty(convert[model], 'labels', { value: labels });
    }
    convert.rgb.hsl = function (rgb) {
        const r = rgb[0] / 255;
        const g = rgb[1] / 255;
        const b = rgb[2] / 255;
        const min = Math.min(r, g, b);
        const max = Math.max(r, g, b);
        const delta = max - min;
        let h;
        let s;
        if (max === min) {
            h = 0;
        }
        else if (r === max) {
            h = (g - b) / delta;
        }
        else if (g === max) {
            h = 2 + (b - r) / delta;
        }
        else if (b === max) {
            h = 4 + (r - g) / delta;
        }
        h = Math.min(h * 60, 360);
        if (h < 0) {
            h += 360;
        }
        const l = (min + max) / 2;
        if (max === min) {
            s = 0;
        }
        else if (l <= 0.5) {
            s = delta / (max + min);
        }
        else {
            s = delta / (2 - max - min);
        }
        return [h, s * 100, l * 100];
    };
    convert.rgb.hsv = function (rgb) {
        let rdif;
        let gdif;
        let bdif;
        let h;
        let s;
        const r = rgb[0] / 255;
        const g = rgb[1] / 255;
        const b = rgb[2] / 255;
        const v = Math.max(r, g, b);
        const diff = v - Math.min(r, g, b);
        const diffc = function (c) {
            return (v - c) / 6 / diff + 1 / 2;
        };
        if (diff === 0) {
            h = 0;
            s = 0;
        }
        else {
            s = diff / v;
            rdif = diffc(r);
            gdif = diffc(g);
            bdif = diffc(b);
            if (r === v) {
                h = bdif - gdif;
            }
            else if (g === v) {
                h = (1 / 3) + rdif - bdif;
            }
            else if (b === v) {
                h = (2 / 3) + gdif - rdif;
            }
            if (h < 0) {
                h += 1;
            }
            else if (h > 1) {
                h -= 1;
            }
        }
        return [
            h * 360,
            s * 100,
            v * 100
        ];
    };
    convert.rgb.hwb = function (rgb) {
        const r = rgb[0];
        const g = rgb[1];
        let b = rgb[2];
        const h = convert.rgb.hsl(rgb)[0];
        const w = 1 / 255 * Math.min(r, Math.min(g, b));
        b = 1 - 1 / 255 * Math.max(r, Math.max(g, b));
        return [h, w * 100, b * 100];
    };
    convert.rgb.cmyk = function (rgb) {
        const r = rgb[0] / 255;
        const g = rgb[1] / 255;
        const b = rgb[2] / 255;
        const k = Math.min(1 - r, 1 - g, 1 - b);
        const c = (1 - r - k) / (1 - k) || 0;
        const m = (1 - g - k) / (1 - k) || 0;
        const y = (1 - b - k) / (1 - k) || 0;
        return [c * 100, m * 100, y * 100, k * 100];
    };
    function comparativeDistance(x, y) {
        /*
            See https://en.m.wikipedia.org/wiki/Euclidean_distance#Squared_Euclidean_distance
        */
        return (((x[0] - y[0]) ** 2) +
            ((x[1] - y[1]) ** 2) +
            ((x[2] - y[2]) ** 2));
    }
    convert.rgb.keyword = function (rgb) {
        const reversed = reverseKeywords[rgb];
        if (reversed) {
            return reversed;
        }
        let currentClosestDistance = Infinity;
        let currentClosestKeyword;
        for (const keyword of Object.keys(cssKeywords)) {
            const value = cssKeywords[keyword];
            // Compute comparative distance
            const distance = comparativeDistance(rgb, value);
            // Check if its less, if so set as closest
            if (distance < currentClosestDistance) {
                currentClosestDistance = distance;
                currentClosestKeyword = keyword;
            }
        }
        return currentClosestKeyword;
    };
    convert.keyword.rgb = function (keyword) {
        return cssKeywords[keyword];
    };
    convert.rgb.xyz = function (rgb) {
        let r = rgb[0] / 255;
        let g = rgb[1] / 255;
        let b = rgb[2] / 255;
        // Assume sRGB
        r = r > 0.04045 ? (((r + 0.055) / 1.055) ** 2.4) : (r / 12.92);
        g = g > 0.04045 ? (((g + 0.055) / 1.055) ** 2.4) : (g / 12.92);
        b = b > 0.04045 ? (((b + 0.055) / 1.055) ** 2.4) : (b / 12.92);
        const x = (r * 0.4124) + (g * 0.3576) + (b * 0.1805);
        const y = (r * 0.2126) + (g * 0.7152) + (b * 0.0722);
        const z = (r * 0.0193) + (g * 0.1192) + (b * 0.9505);
        return [x * 100, y * 100, z * 100];
    };
    convert.rgb.lab = function (rgb) {
        const xyz = convert.rgb.xyz(rgb);
        let x = xyz[0];
        let y = xyz[1];
        let z = xyz[2];
        x /= 95.047;
        y /= 100;
        z /= 108.883;
        x = x > 0.008856 ? (x ** (1 / 3)) : (7.787 * x) + (16 / 116);
        y = y > 0.008856 ? (y ** (1 / 3)) : (7.787 * y) + (16 / 116);
        z = z > 0.008856 ? (z ** (1 / 3)) : (7.787 * z) + (16 / 116);
        const l = (116 * y) - 16;
        const a = 500 * (x - y);
        const b = 200 * (y - z);
        return [l, a, b];
    };
    convert.hsl.rgb = function (hsl) {
        const h = hsl[0] / 360;
        const s = hsl[1] / 100;
        const l = hsl[2] / 100;
        let t2;
        let t3;
        let val;
        if (s === 0) {
            val = l * 255;
            return [val, val, val];
        }
        if (l < 0.5) {
            t2 = l * (1 + s);
        }
        else {
            t2 = l + s - l * s;
        }
        const t1 = 2 * l - t2;
        const rgb = [0, 0, 0];
        for (let i = 0; i < 3; i++) {
            t3 = h + 1 / 3 * -(i - 1);
            if (t3 < 0) {
                t3++;
            }
            if (t3 > 1) {
                t3--;
            }
            if (6 * t3 < 1) {
                val = t1 + (t2 - t1) * 6 * t3;
            }
            else if (2 * t3 < 1) {
                val = t2;
            }
            else if (3 * t3 < 2) {
                val = t1 + (t2 - t1) * (2 / 3 - t3) * 6;
            }
            else {
                val = t1;
            }
            rgb[i] = val * 255;
        }
        return rgb;
    };
    convert.hsl.hsv = function (hsl) {
        const h = hsl[0];
        let s = hsl[1] / 100;
        let l = hsl[2] / 100;
        let smin = s;
        const lmin = Math.max(l, 0.01);
        l *= 2;
        s *= (l <= 1) ? l : 2 - l;
        smin *= lmin <= 1 ? lmin : 2 - lmin;
        const v = (l + s) / 2;
        const sv = l === 0 ? (2 * smin) / (lmin + smin) : (2 * s) / (l + s);
        return [h, sv * 100, v * 100];
    };
    convert.hsv.rgb = function (hsv) {
        const h = hsv[0] / 60;
        const s = hsv[1] / 100;
        let v = hsv[2] / 100;
        const hi = Math.floor(h) % 6;
        const f = h - Math.floor(h);
        const p = 255 * v * (1 - s);
        const q = 255 * v * (1 - (s * f));
        const t = 255 * v * (1 - (s * (1 - f)));
        v *= 255;
        switch (hi) {
            case 0:
                return [v, t, p];
            case 1:
                return [q, v, p];
            case 2:
                return [p, v, t];
            case 3:
                return [p, q, v];
            case 4:
                return [t, p, v];
            case 5:
                return [v, p, q];
        }
    };
    convert.hsv.hsl = function (hsv) {
        const h = hsv[0];
        const s = hsv[1] / 100;
        const v = hsv[2] / 100;
        const vmin = Math.max(v, 0.01);
        let sl;
        let l;
        l = (2 - s) * v;
        const lmin = (2 - s) * vmin;
        sl = s * vmin;
        sl /= (lmin <= 1) ? lmin : 2 - lmin;
        sl = sl || 0;
        l /= 2;
        return [h, sl * 100, l * 100];
    };
    // http://dev.w3.org/csswg/css-color/#hwb-to-rgb
    convert.hwb.rgb = function (hwb) {
        const h = hwb[0] / 360;
        let wh = hwb[1] / 100;
        let bl = hwb[2] / 100;
        const ratio = wh + bl;
        let f;
        // Wh + bl cant be > 1
        if (ratio > 1) {
            wh /= ratio;
            bl /= ratio;
        }
        const i = Math.floor(6 * h);
        const v = 1 - bl;
        f = 6 * h - i;
        if ((i & 0x01) !== 0) {
            f = 1 - f;
        }
        const n = wh + f * (v - wh); // Linear interpolation
        let r;
        let g;
        let b;
        /* eslint-disable max-statements-per-line,no-multi-spaces */
        switch (i) {
            default:
            case 6:
            case 0:
                r = v;
                g = n;
                b = wh;
                break;
            case 1:
                r = n;
                g = v;
                b = wh;
                break;
            case 2:
                r = wh;
                g = v;
                b = n;
                break;
            case 3:
                r = wh;
                g = n;
                b = v;
                break;
            case 4:
                r = n;
                g = wh;
                b = v;
                break;
            case 5:
                r = v;
                g = wh;
                b = n;
                break;
        }
        /* eslint-enable max-statements-per-line,no-multi-spaces */
        return [r * 255, g * 255, b * 255];
    };
    convert.cmyk.rgb = function (cmyk) {
        const c = cmyk[0] / 100;
        const m = cmyk[1] / 100;
        const y = cmyk[2] / 100;
        const k = cmyk[3] / 100;
        const r = 1 - Math.min(1, c * (1 - k) + k);
        const g = 1 - Math.min(1, m * (1 - k) + k);
        const b = 1 - Math.min(1, y * (1 - k) + k);
        return [r * 255, g * 255, b * 255];
    };
    convert.xyz.rgb = function (xyz) {
        const x = xyz[0] / 100;
        const y = xyz[1] / 100;
        const z = xyz[2] / 100;
        let r;
        let g;
        let b;
        r = (x * 3.2406) + (y * -1.5372) + (z * -0.4986);
        g = (x * -0.9689) + (y * 1.8758) + (z * 0.0415);
        b = (x * 0.0557) + (y * -0.2040) + (z * 1.0570);
        // Assume sRGB
        r = r > 0.0031308
            ? ((1.055 * (r ** (1.0 / 2.4))) - 0.055)
            : r * 12.92;
        g = g > 0.0031308
            ? ((1.055 * (g ** (1.0 / 2.4))) - 0.055)
            : g * 12.92;
        b = b > 0.0031308
            ? ((1.055 * (b ** (1.0 / 2.4))) - 0.055)
            : b * 12.92;
        r = Math.min(Math.max(0, r), 1);
        g = Math.min(Math.max(0, g), 1);
        b = Math.min(Math.max(0, b), 1);
        return [r * 255, g * 255, b * 255];
    };
    convert.xyz.lab = function (xyz) {
        let x = xyz[0];
        let y = xyz[1];
        let z = xyz[2];
        x /= 95.047;
        y /= 100;
        z /= 108.883;
        x = x > 0.008856 ? (x ** (1 / 3)) : (7.787 * x) + (16 / 116);
        y = y > 0.008856 ? (y ** (1 / 3)) : (7.787 * y) + (16 / 116);
        z = z > 0.008856 ? (z ** (1 / 3)) : (7.787 * z) + (16 / 116);
        const l = (116 * y) - 16;
        const a = 500 * (x - y);
        const b = 200 * (y - z);
        return [l, a, b];
    };
    convert.lab.xyz = function (lab) {
        const l = lab[0];
        const a = lab[1];
        const b = lab[2];
        let x;
        let y;
        let z;
        y = (l + 16) / 116;
        x = a / 500 + y;
        z = y - b / 200;
        const y2 = y ** 3;
        const x2 = x ** 3;
        const z2 = z ** 3;
        y = y2 > 0.008856 ? y2 : (y - 16 / 116) / 7.787;
        x = x2 > 0.008856 ? x2 : (x - 16 / 116) / 7.787;
        z = z2 > 0.008856 ? z2 : (z - 16 / 116) / 7.787;
        x *= 95.047;
        y *= 100;
        z *= 108.883;
        return [x, y, z];
    };
    convert.lab.lch = function (lab) {
        const l = lab[0];
        const a = lab[1];
        const b = lab[2];
        let h;
        const hr = Math.atan2(b, a);
        h = hr * 360 / 2 / Math.PI;
        if (h < 0) {
            h += 360;
        }
        const c = Math.sqrt(a * a + b * b);
        return [l, c, h];
    };
    convert.lch.lab = function (lch) {
        const l = lch[0];
        const c = lch[1];
        const h = lch[2];
        const hr = h / 360 * 2 * Math.PI;
        const a = c * Math.cos(hr);
        const b = c * Math.sin(hr);
        return [l, a, b];
    };
    convert.rgb.ansi16 = function (args, saturation = null) {
        const [r, g, b] = args;
        let value = saturation === null ? convert.rgb.hsv(args)[2] : saturation; // Hsv -> ansi16 optimization
        value = Math.round(value / 50);
        if (value === 0) {
            return 30;
        }
        let ansi = 30
            + ((Math.round(b / 255) << 2)
                | (Math.round(g / 255) << 1)
                | Math.round(r / 255));
        if (value === 2) {
            ansi += 60;
        }
        return ansi;
    };
    convert.hsv.ansi16 = function (args) {
        // Optimization here; we already know the value and don't need to get
        // it converted for us.
        return convert.rgb.ansi16(convert.hsv.rgb(args), args[2]);
    };
    convert.rgb.ansi256 = function (args) {
        const r = args[0];
        const g = args[1];
        const b = args[2];
        // We use the extended greyscale palette here, with the exception of
        // black and white. normal palette only has 4 greyscale shades.
        if (r === g && g === b) {
            if (r < 8) {
                return 16;
            }
            if (r > 248) {
                return 231;
            }
            return Math.round(((r - 8) / 247) * 24) + 232;
        }
        const ansi = 16
            + (36 * Math.round(r / 255 * 5))
            + (6 * Math.round(g / 255 * 5))
            + Math.round(b / 255 * 5);
        return ansi;
    };
    convert.ansi16.rgb = function (args) {
        let color = args % 10;
        // Handle greyscale
        if (color === 0 || color === 7) {
            if (args > 50) {
                color += 3.5;
            }
            color = color / 10.5 * 255;
            return [color, color, color];
        }
        const mult = (~~(args > 50) + 1) * 0.5;
        const r = ((color & 1) * mult) * 255;
        const g = (((color >> 1) & 1) * mult) * 255;
        const b = (((color >> 2) & 1) * mult) * 255;
        return [r, g, b];
    };
    convert.ansi256.rgb = function (args) {
        // Handle greyscale
        if (args >= 232) {
            const c = (args - 232) * 10 + 8;
            return [c, c, c];
        }
        args -= 16;
        let rem;
        const r = Math.floor(args / 36) / 5 * 255;
        const g = Math.floor((rem = args % 36) / 6) / 5 * 255;
        const b = (rem % 6) / 5 * 255;
        return [r, g, b];
    };
    convert.rgb.hex = function (args) {
        const integer = ((Math.round(args[0]) & 0xFF) << 16)
            + ((Math.round(args[1]) & 0xFF) << 8)
            + (Math.round(args[2]) & 0xFF);
        const string = integer.toString(16).toUpperCase();
        return '000000'.substring(string.length) + string;
    };
    convert.hex.rgb = function (args) {
        const match = args.toString(16).match(/[a-f0-9]{6}|[a-f0-9]{3}/i);
        if (!match) {
            return [0, 0, 0];
        }
        let colorString = match[0];
        if (match[0].length === 3) {
            colorString = colorString.split('').map(char => {
                return char + char;
            }).join('');
        }
        const integer = parseInt(colorString, 16);
        const r = (integer >> 16) & 0xFF;
        const g = (integer >> 8) & 0xFF;
        const b = integer & 0xFF;
        return [r, g, b];
    };
    convert.rgb.hcg = function (rgb) {
        const r = rgb[0] / 255;
        const g = rgb[1] / 255;
        const b = rgb[2] / 255;
        const max = Math.max(Math.max(r, g), b);
        const min = Math.min(Math.min(r, g), b);
        const chroma = (max - min);
        let grayscale;
        let hue;
        if (chroma < 1) {
            grayscale = min / (1 - chroma);
        }
        else {
            grayscale = 0;
        }
        if (chroma <= 0) {
            hue = 0;
        }
        else if (max === r) {
            hue = ((g - b) / chroma) % 6;
        }
        else if (max === g) {
            hue = 2 + (b - r) / chroma;
        }
        else {
            hue = 4 + (r - g) / chroma;
        }
        hue /= 6;
        hue %= 1;
        return [hue * 360, chroma * 100, grayscale * 100];
    };
    convert.hsl.hcg = function (hsl) {
        const s = hsl[1] / 100;
        const l = hsl[2] / 100;
        const c = l < 0.5 ? (2.0 * s * l) : (2.0 * s * (1.0 - l));
        let f = 0;
        if (c < 1.0) {
            f = (l - 0.5 * c) / (1.0 - c);
        }
        return [hsl[0], c * 100, f * 100];
    };
    convert.hsv.hcg = function (hsv) {
        const s = hsv[1] / 100;
        const v = hsv[2] / 100;
        const c = s * v;
        let f = 0;
        if (c < 1.0) {
            f = (v - c) / (1 - c);
        }
        return [hsv[0], c * 100, f * 100];
    };
    convert.hcg.rgb = function (hcg) {
        const h = hcg[0] / 360;
        const c = hcg[1] / 100;
        const g = hcg[2] / 100;
        if (c === 0.0) {
            return [g * 255, g * 255, g * 255];
        }
        const pure = [0, 0, 0];
        const hi = (h % 1) * 6;
        const v = hi % 1;
        const w = 1 - v;
        let mg = 0;
        /* eslint-disable max-statements-per-line */
        switch (Math.floor(hi)) {
            case 0:
                pure[0] = 1;
                pure[1] = v;
                pure[2] = 0;
                break;
            case 1:
                pure[0] = w;
                pure[1] = 1;
                pure[2] = 0;
                break;
            case 2:
                pure[0] = 0;
                pure[1] = 1;
                pure[2] = v;
                break;
            case 3:
                pure[0] = 0;
                pure[1] = w;
                pure[2] = 1;
                break;
            case 4:
                pure[0] = v;
                pure[1] = 0;
                pure[2] = 1;
                break;
            default:
                pure[0] = 1;
                pure[1] = 0;
                pure[2] = w;
        }
        /* eslint-enable max-statements-per-line */
        mg = (1.0 - c) * g;
        return [
            (c * pure[0] + mg) * 255,
            (c * pure[1] + mg) * 255,
            (c * pure[2] + mg) * 255
        ];
    };
    convert.hcg.hsv = function (hcg) {
        const c = hcg[1] / 100;
        const g = hcg[2] / 100;
        const v = c + g * (1.0 - c);
        let f = 0;
        if (v > 0.0) {
            f = c / v;
        }
        return [hcg[0], f * 100, v * 100];
    };
    convert.hcg.hsl = function (hcg) {
        const c = hcg[1] / 100;
        const g = hcg[2] / 100;
        const l = g * (1.0 - c) + 0.5 * c;
        let s = 0;
        if (l > 0.0 && l < 0.5) {
            s = c / (2 * l);
        }
        else if (l >= 0.5 && l < 1.0) {
            s = c / (2 * (1 - l));
        }
        return [hcg[0], s * 100, l * 100];
    };
    convert.hcg.hwb = function (hcg) {
        const c = hcg[1] / 100;
        const g = hcg[2] / 100;
        const v = c + g * (1.0 - c);
        return [hcg[0], (v - c) * 100, (1 - v) * 100];
    };
    convert.hwb.hcg = function (hwb) {
        const w = hwb[1] / 100;
        const b = hwb[2] / 100;
        const v = 1 - b;
        const c = v - w;
        let g = 0;
        if (c < 1) {
            g = (v - c) / (1 - c);
        }
        return [hwb[0], c * 100, g * 100];
    };
    convert.apple.rgb = function (apple) {
        return [(apple[0] / 65535) * 255, (apple[1] / 65535) * 255, (apple[2] / 65535) * 255];
    };
    convert.rgb.apple = function (rgb) {
        return [(rgb[0] / 255) * 65535, (rgb[1] / 255) * 65535, (rgb[2] / 255) * 65535];
    };
    convert.gray.rgb = function (args) {
        return [args[0] / 100 * 255, args[0] / 100 * 255, args[0] / 100 * 255];
    };
    convert.gray.hsl = function (args) {
        return [0, 0, args[0]];
    };
    convert.gray.hsv = convert.gray.hsl;
    convert.gray.hwb = function (gray) {
        return [0, 100, gray[0]];
    };
    convert.gray.cmyk = function (gray) {
        return [0, 0, 0, gray[0]];
    };
    convert.gray.lab = function (gray) {
        return [gray[0], 0, 0];
    };
    convert.gray.hex = function (gray) {
        const val = Math.round(gray[0] / 100 * 255) & 0xFF;
        const integer = (val << 16) + (val << 8) + val;
        const string = integer.toString(16).toUpperCase();
        return '000000'.substring(string.length) + string;
    };
    convert.rgb.gray = function (rgb) {
        const val = (rgb[0] + rgb[1] + rgb[2]) / 3;
        return [val / 255 * 100];
    };
},
/* color-name/index.js */ function _(require, module, exports) {
    module.exports = {
        "aliceblue": [240, 248, 255],
        "antiquewhite": [250, 235, 215],
        "aqua": [0, 255, 255],
        "aquamarine": [127, 255, 212],
        "azure": [240, 255, 255],
        "beige": [245, 245, 220],
        "bisque": [255, 228, 196],
        "black": [0, 0, 0],
        "blanchedalmond": [255, 235, 205],
        "blue": [0, 0, 255],
        "blueviolet": [138, 43, 226],
        "brown": [165, 42, 42],
        "burlywood": [222, 184, 135],
        "cadetblue": [95, 158, 160],
        "chartreuse": [127, 255, 0],
        "chocolate": [210, 105, 30],
        "coral": [255, 127, 80],
        "cornflowerblue": [100, 149, 237],
        "cornsilk": [255, 248, 220],
        "crimson": [220, 20, 60],
        "cyan": [0, 255, 255],
        "darkblue": [0, 0, 139],
        "darkcyan": [0, 139, 139],
        "darkgoldenrod": [184, 134, 11],
        "darkgray": [169, 169, 169],
        "darkgreen": [0, 100, 0],
        "darkgrey": [169, 169, 169],
        "darkkhaki": [189, 183, 107],
        "darkmagenta": [139, 0, 139],
        "darkolivegreen": [85, 107, 47],
        "darkorange": [255, 140, 0],
        "darkorchid": [153, 50, 204],
        "darkred": [139, 0, 0],
        "darksalmon": [233, 150, 122],
        "darkseagreen": [143, 188, 143],
        "darkslateblue": [72, 61, 139],
        "darkslategray": [47, 79, 79],
        "darkslategrey": [47, 79, 79],
        "darkturquoise": [0, 206, 209],
        "darkviolet": [148, 0, 211],
        "deeppink": [255, 20, 147],
        "deepskyblue": [0, 191, 255],
        "dimgray": [105, 105, 105],
        "dimgrey": [105, 105, 105],
        "dodgerblue": [30, 144, 255],
        "firebrick": [178, 34, 34],
        "floralwhite": [255, 250, 240],
        "forestgreen": [34, 139, 34],
        "fuchsia": [255, 0, 255],
        "gainsboro": [220, 220, 220],
        "ghostwhite": [248, 248, 255],
        "gold": [255, 215, 0],
        "goldenrod": [218, 165, 32],
        "gray": [128, 128, 128],
        "green": [0, 128, 0],
        "greenyellow": [173, 255, 47],
        "grey": [128, 128, 128],
        "honeydew": [240, 255, 240],
        "hotpink": [255, 105, 180],
        "indianred": [205, 92, 92],
        "indigo": [75, 0, 130],
        "ivory": [255, 255, 240],
        "khaki": [240, 230, 140],
        "lavender": [230, 230, 250],
        "lavenderblush": [255, 240, 245],
        "lawngreen": [124, 252, 0],
        "lemonchiffon": [255, 250, 205],
        "lightblue": [173, 216, 230],
        "lightcoral": [240, 128, 128],
        "lightcyan": [224, 255, 255],
        "lightgoldenrodyellow": [250, 250, 210],
        "lightgray": [211, 211, 211],
        "lightgreen": [144, 238, 144],
        "lightgrey": [211, 211, 211],
        "lightpink": [255, 182, 193],
        "lightsalmon": [255, 160, 122],
        "lightseagreen": [32, 178, 170],
        "lightskyblue": [135, 206, 250],
        "lightslategray": [119, 136, 153],
        "lightslategrey": [119, 136, 153],
        "lightsteelblue": [176, 196, 222],
        "lightyellow": [255, 255, 224],
        "lime": [0, 255, 0],
        "limegreen": [50, 205, 50],
        "linen": [250, 240, 230],
        "magenta": [255, 0, 255],
        "maroon": [128, 0, 0],
        "mediumaquamarine": [102, 205, 170],
        "mediumblue": [0, 0, 205],
        "mediumorchid": [186, 85, 211],
        "mediumpurple": [147, 112, 219],
        "mediumseagreen": [60, 179, 113],
        "mediumslateblue": [123, 104, 238],
        "mediumspringgreen": [0, 250, 154],
        "mediumturquoise": [72, 209, 204],
        "mediumvioletred": [199, 21, 133],
        "midnightblue": [25, 25, 112],
        "mintcream": [245, 255, 250],
        "mistyrose": [255, 228, 225],
        "moccasin": [255, 228, 181],
        "navajowhite": [255, 222, 173],
        "navy": [0, 0, 128],
        "oldlace": [253, 245, 230],
        "olive": [128, 128, 0],
        "olivedrab": [107, 142, 35],
        "orange": [255, 165, 0],
        "orangered": [255, 69, 0],
        "orchid": [218, 112, 214],
        "palegoldenrod": [238, 232, 170],
        "palegreen": [152, 251, 152],
        "paleturquoise": [175, 238, 238],
        "palevioletred": [219, 112, 147],
        "papayawhip": [255, 239, 213],
        "peachpuff": [255, 218, 185],
        "peru": [205, 133, 63],
        "pink": [255, 192, 203],
        "plum": [221, 160, 221],
        "powderblue": [176, 224, 230],
        "purple": [128, 0, 128],
        "rebeccapurple": [102, 51, 153],
        "red": [255, 0, 0],
        "rosybrown": [188, 143, 143],
        "royalblue": [65, 105, 225],
        "saddlebrown": [139, 69, 19],
        "salmon": [250, 128, 114],
        "sandybrown": [244, 164, 96],
        "seagreen": [46, 139, 87],
        "seashell": [255, 245, 238],
        "sienna": [160, 82, 45],
        "silver": [192, 192, 192],
        "skyblue": [135, 206, 235],
        "slateblue": [106, 90, 205],
        "slategray": [112, 128, 144],
        "slategrey": [112, 128, 144],
        "snow": [255, 250, 250],
        "springgreen": [0, 255, 127],
        "steelblue": [70, 130, 180],
        "tan": [210, 180, 140],
        "teal": [0, 128, 128],
        "thistle": [216, 191, 216],
        "tomato": [255, 99, 71],
        "turquoise": [64, 224, 208],
        "violet": [238, 130, 238],
        "wheat": [245, 222, 179],
        "white": [255, 255, 255],
        "whitesmoke": [245, 245, 245],
        "yellow": [255, 255, 0],
        "yellowgreen": [154, 205, 50]
    };
},
/* color-convert/route.js */ function _(require, module, exports) {
    const conversions = require(27) /* ./conversions */;
    /*
        This function routes a model to all other models.
    
        all functions that are routed have a property `.conversion` attached
        to the returned synthetic function. This property is an array
        of strings, each with the steps in between the 'from' and 'to'
        color models (inclusive).
    
        conversions that are not possible simply are not included.
    */
    function buildGraph() {
        const graph = {};
        // https://jsperf.com/object-keys-vs-for-in-with-closure/3
        const models = Object.keys(conversions);
        for (let len = models.length, i = 0; i < len; i++) {
            graph[models[i]] = {
                // http://jsperf.com/1-vs-infinity
                // micro-opt, but this is simple.
                distance: -1,
                parent: null
            };
        }
        return graph;
    }
    // https://en.wikipedia.org/wiki/Breadth-first_search
    function deriveBFS(fromModel) {
        const graph = buildGraph();
        const queue = [fromModel]; // Unshift -> queue -> pop
        graph[fromModel].distance = 0;
        while (queue.length) {
            const current = queue.pop();
            const adjacents = Object.keys(conversions[current]);
            for (let len = adjacents.length, i = 0; i < len; i++) {
                const adjacent = adjacents[i];
                const node = graph[adjacent];
                if (node.distance === -1) {
                    node.distance = graph[current].distance + 1;
                    node.parent = current;
                    queue.unshift(adjacent);
                }
            }
        }
        return graph;
    }
    function link(from, to) {
        return function (args) {
            return to(from(args));
        };
    }
    function wrapConversion(toModel, graph) {
        const path = [graph[toModel].parent, toModel];
        let fn = conversions[graph[toModel].parent][toModel];
        let cur = graph[toModel].parent;
        while (graph[cur].parent) {
            path.unshift(graph[cur].parent);
            fn = link(conversions[graph[cur].parent][cur], fn);
            cur = graph[cur].parent;
        }
        fn.conversion = path;
        return fn;
    }
    module.exports = function (fromModel) {
        const graph = deriveBFS(fromModel);
        const conversion = {};
        const models = Object.keys(graph);
        for (let len = models.length, i = 0; i < len; i++) {
            const toModel = models[i];
            const node = graph[toModel];
            if (node.parent === null) {
                // No possible conversion, or this node is the source model.
                continue;
            }
            conversion[toModel] = wrapConversion(toModel, graph);
        }
        return conversion;
    };
},
/* yargs/lib/validation.js */ function _(require, module, exports) {
    const argsert = require(3) /* ./argsert */;
    const objFilter = require(21) /* ./obj-filter */;
    const specialKeys = ['$0', '--', '_'];
    // validation-type-stuff, missing params,
    // bad implications, custom checks.
    module.exports = function validation(yargs, usage, y18n) {
        const __ = y18n.__;
        const __n = y18n.__n;
        const self = {};
        // validate appropriate # of non-option
        // arguments were provided, i.e., '_'.
        self.nonOptionCount = function nonOptionCount(argv) {
            const demandedCommands = yargs.getDemandedCommands();
            // don't count currently executing commands
            const _s = argv._.length - yargs.getContext().commands.length;
            if (demandedCommands._ && (_s < demandedCommands._.min || _s > demandedCommands._.max)) {
                if (_s < demandedCommands._.min) {
                    if (demandedCommands._.minMsg !== undefined) {
                        usage.fail(
                        // replace $0 with observed, $1 with expected.
                        demandedCommands._.minMsg ? demandedCommands._.minMsg.replace(/\$0/g, _s).replace(/\$1/, demandedCommands._.min) : null);
                    }
                    else {
                        usage.fail(__n('Not enough non-option arguments: got %s, need at least %s', 'Not enough non-option arguments: got %s, need at least %s', _s, _s, demandedCommands._.min));
                    }
                }
                else if (_s > demandedCommands._.max) {
                    if (demandedCommands._.maxMsg !== undefined) {
                        usage.fail(
                        // replace $0 with observed, $1 with expected.
                        demandedCommands._.maxMsg ? demandedCommands._.maxMsg.replace(/\$0/g, _s).replace(/\$1/, demandedCommands._.max) : null);
                    }
                    else {
                        usage.fail(__n('Too many non-option arguments: got %s, maximum of %s', 'Too many non-option arguments: got %s, maximum of %s', _s, _s, demandedCommands._.max));
                    }
                }
            }
        };
        // validate the appropriate # of <required>
        // positional arguments were provided:
        self.positionalCount = function positionalCount(required, observed) {
            if (observed < required) {
                usage.fail(__n('Not enough non-option arguments: got %s, need at least %s', 'Not enough non-option arguments: got %s, need at least %s', observed, observed, required));
            }
        };
        // make sure all the required arguments are present.
        self.requiredArguments = function requiredArguments(argv) {
            const demandedOptions = yargs.getDemandedOptions();
            let missing = null;
            Object.keys(demandedOptions).forEach((key) => {
                if (!Object.prototype.hasOwnProperty.call(argv, key) || typeof argv[key] === 'undefined') {
                    missing = missing || {};
                    missing[key] = demandedOptions[key];
                }
            });
            if (missing) {
                const customMsgs = [];
                Object.keys(missing).forEach((key) => {
                    const msg = missing[key];
                    if (msg && customMsgs.indexOf(msg) < 0) {
                        customMsgs.push(msg);
                    }
                });
                const customMsg = customMsgs.length ? `\n${customMsgs.join('\n')}` : '';
                usage.fail(__n('Missing required argument: %s', 'Missing required arguments: %s', Object.keys(missing).length, Object.keys(missing).join(', ') + customMsg));
            }
        };
        // check for unknown arguments (strict-mode).
        self.unknownArguments = function unknownArguments(argv, aliases, positionalMap) {
            const commandKeys = yargs.getCommandInstance().getCommands();
            const unknown = [];
            const currentContext = yargs.getContext();
            Object.keys(argv).forEach((key) => {
                if (specialKeys.indexOf(key) === -1 &&
                    !Object.prototype.hasOwnProperty.call(positionalMap, key) &&
                    !Object.prototype.hasOwnProperty.call(yargs._getParseContext(), key) &&
                    !self.isValidAndSomeAliasIsNotNew(key, aliases)) {
                    unknown.push(key);
                }
            });
            if ((currentContext.commands.length > 0) || (commandKeys.length > 0)) {
                argv._.slice(currentContext.commands.length).forEach((key) => {
                    if (commandKeys.indexOf(key) === -1) {
                        unknown.push(key);
                    }
                });
            }
            if (unknown.length > 0) {
                usage.fail(__n('Unknown argument: %s', 'Unknown arguments: %s', unknown.length, unknown.join(', ')));
            }
        };
        self.unknownCommands = function unknownCommands(argv, aliases, positionalMap) {
            const commandKeys = yargs.getCommandInstance().getCommands();
            const unknown = [];
            const currentContext = yargs.getContext();
            if ((currentContext.commands.length > 0) || (commandKeys.length > 0)) {
                argv._.slice(currentContext.commands.length).forEach((key) => {
                    if (commandKeys.indexOf(key) === -1) {
                        unknown.push(key);
                    }
                });
            }
            if (unknown.length > 0) {
                usage.fail(__n('Unknown command: %s', 'Unknown commands: %s', unknown.length, unknown.join(', ')));
                return true;
            }
            else {
                return false;
            }
        };
        // check for a key that is not an alias, or for which every alias is new,
        // implying that it was invented by the parser, e.g., during camelization
        self.isValidAndSomeAliasIsNotNew = function isValidAndSomeAliasIsNotNew(key, aliases) {
            if (!Object.prototype.hasOwnProperty.call(aliases, key)) {
                return false;
            }
            const newAliases = yargs.parsed.newAliases;
            for (const a of [key, ...aliases[key]]) {
                if (!Object.prototype.hasOwnProperty.call(newAliases, a) || !newAliases[key]) {
                    return true;
                }
            }
            return false;
        };
        // validate arguments limited to enumerated choices
        self.limitedChoices = function limitedChoices(argv) {
            const options = yargs.getOptions();
            const invalid = {};
            if (!Object.keys(options.choices).length)
                return;
            Object.keys(argv).forEach((key) => {
                if (specialKeys.indexOf(key) === -1 &&
                    Object.prototype.hasOwnProperty.call(options.choices, key)) {
                    [].concat(argv[key]).forEach((value) => {
                        // TODO case-insensitive configurability
                        if (options.choices[key].indexOf(value) === -1 &&
                            value !== undefined) {
                            invalid[key] = (invalid[key] || []).concat(value);
                        }
                    });
                }
            });
            const invalidKeys = Object.keys(invalid);
            if (!invalidKeys.length)
                return;
            let msg = __('Invalid values:');
            invalidKeys.forEach((key) => {
                msg += `\n  ${__('Argument: %s, Given: %s, Choices: %s', key, usage.stringifiedValues(invalid[key]), usage.stringifiedValues(options.choices[key]))}`;
            });
            usage.fail(msg);
        };
        // custom checks, added using the `check` option on yargs.
        let checks = [];
        self.check = function check(f, global) {
            checks.push({
                func: f,
                global
            });
        };
        self.customChecks = function customChecks(argv, aliases) {
            for (let i = 0, f; (f = checks[i]) !== undefined; i++) {
                const func = f.func;
                let result = null;
                try {
                    result = func(argv, aliases);
                }
                catch (err) {
                    usage.fail(err.message ? err.message : err, err);
                    continue;
                }
                if (!result) {
                    usage.fail(__('Argument check failed: %s', func.toString()));
                }
                else if (typeof result === 'string' || result instanceof Error) {
                    usage.fail(result.toString(), result);
                }
            }
        };
        // check implications, argument foo implies => argument bar.
        let implied = {};
        self.implies = function implies(key, value) {
            argsert('<string|object> [array|number|string]', [key, value], arguments.length);
            if (typeof key === 'object') {
                Object.keys(key).forEach((k) => {
                    self.implies(k, key[k]);
                });
            }
            else {
                yargs.global(key);
                if (!implied[key]) {
                    implied[key] = [];
                }
                if (Array.isArray(value)) {
                    value.forEach((i) => self.implies(key, i));
                }
                else {
                    implied[key].push(value);
                }
            }
        };
        self.getImplied = function getImplied() {
            return implied;
        };
        function keyExists(argv, val) {
            // convert string '1' to number 1
            const num = Number(val);
            val = isNaN(num) ? val : num;
            if (typeof val === 'number') {
                // check length of argv._
                val = argv._.length >= val;
            }
            else if (val.match(/^--no-.+/)) {
                // check if key/value doesn't exist
                val = val.match(/^--no-(.+)/)[1];
                val = !argv[val];
            }
            else {
                // check if key/value exists
                val = argv[val];
            }
            return val;
        }
        self.implications = function implications(argv) {
            const implyFail = [];
            Object.keys(implied).forEach((key) => {
                const origKey = key;
                (implied[key] || []).forEach((value) => {
                    let key = origKey;
                    const origValue = value;
                    key = keyExists(argv, key);
                    value = keyExists(argv, value);
                    if (key && !value) {
                        implyFail.push(` ${origKey} -> ${origValue}`);
                    }
                });
            });
            if (implyFail.length) {
                let msg = `${__('Implications failed:')}\n`;
                implyFail.forEach((value) => {
                    msg += (value);
                });
                usage.fail(msg);
            }
        };
        let conflicting = {};
        self.conflicts = function conflicts(key, value) {
            argsert('<string|object> [array|string]', [key, value], arguments.length);
            if (typeof key === 'object') {
                Object.keys(key).forEach((k) => {
                    self.conflicts(k, key[k]);
                });
            }
            else {
                yargs.global(key);
                if (!conflicting[key]) {
                    conflicting[key] = [];
                }
                if (Array.isArray(value)) {
                    value.forEach((i) => self.conflicts(key, i));
                }
                else {
                    conflicting[key].push(value);
                }
            }
        };
        self.getConflicting = () => conflicting;
        self.conflicting = function conflictingFn(argv) {
            Object.keys(argv).forEach((key) => {
                if (conflicting[key]) {
                    conflicting[key].forEach((value) => {
                        // we default keys to 'undefined' that have been configured, we should not
                        // apply conflicting check unless they are a value other than 'undefined'.
                        if (value && argv[key] !== undefined && argv[value] !== undefined) {
                            usage.fail(__('Arguments %s and %s are mutually exclusive', key, value));
                        }
                    });
                }
            });
        };
        self.recommendCommands = function recommendCommands(cmd, potentialCommands) {
            const distance = require(31) /* ./levenshtein */;
            const threshold = 3; // if it takes more than three edits, let's move on.
            potentialCommands = potentialCommands.sort((a, b) => b.length - a.length);
            let recommended = null;
            let bestDistance = Infinity;
            for (let i = 0, candidate; (candidate = potentialCommands[i]) !== undefined; i++) {
                const d = distance(cmd, candidate);
                if (d <= threshold && d < bestDistance) {
                    bestDistance = d;
                    recommended = candidate;
                }
            }
            if (recommended)
                usage.fail(__('Did you mean %s?', recommended));
        };
        self.reset = function reset(localLookup) {
            implied = objFilter(implied, (k, v) => !localLookup[k]);
            conflicting = objFilter(conflicting, (k, v) => !localLookup[k]);
            checks = checks.filter(c => c.global);
            return self;
        };
        const frozens = [];
        self.freeze = function freeze() {
            const frozen = {};
            frozens.push(frozen);
            frozen.implied = implied;
            frozen.checks = checks;
            frozen.conflicting = conflicting;
        };
        self.unfreeze = function unfreeze() {
            const frozen = frozens.pop();
            implied = frozen.implied;
            checks = frozen.checks;
            conflicting = frozen.conflicting;
        };
        return self;
    };
},
/* yargs/lib/levenshtein.js */ function _(require, module, exports) {
    // Compute the edit distance between the two given strings
    module.exports = function levenshtein(a, b) {
        if (a.length === 0)
            return b.length;
        if (b.length === 0)
            return a.length;
        const matrix = [];
        // increment along the first column of each row
        let i;
        for (i = 0; i <= b.length; i++) {
            matrix[i] = [i];
        }
        // increment each column in the first row
        let j;
        for (j = 0; j <= a.length; j++) {
            matrix[0][j] = j;
        }
        // Fill in the rest of the matrix
        for (i = 1; i <= b.length; i++) {
            for (j = 1; j <= a.length; j++) {
                if (b.charAt(i - 1) === a.charAt(j - 1)) {
                    matrix[i][j] = matrix[i - 1][j - 1];
                }
                else {
                    matrix[i][j] = Math.min(matrix[i - 1][j - 1] + 1, // substitution
                    Math.min(matrix[i][j - 1] + 1, // insertion
                    matrix[i - 1][j] + 1)); // deletion
                }
            }
        }
        return matrix[b.length][a.length];
    };
},
/* y18n/index.js */ function _(require, module, exports) {
    var fs = require('fs');
    var path = require('path');
    var util = require('util');
    function Y18N(opts) {
        // configurable options.
        opts = opts || {};
        this.directory = opts.directory || './locales';
        this.updateFiles = typeof opts.updateFiles === 'boolean' ? opts.updateFiles : true;
        this.locale = opts.locale || 'en';
        this.fallbackToLanguage = typeof opts.fallbackToLanguage === 'boolean' ? opts.fallbackToLanguage : true;
        // internal stuff.
        this.cache = {};
        this.writeQueue = [];
    }
    Y18N.prototype.__ = function () {
        if (typeof arguments[0] !== 'string') {
            return this._taggedLiteral.apply(this, arguments);
        }
        var args = Array.prototype.slice.call(arguments);
        var str = args.shift();
        var cb = function () { }; // start with noop.
        if (typeof args[args.length - 1] === 'function')
            cb = args.pop();
        cb = cb || function () { }; // noop.
        if (!this.cache[this.locale])
            this._readLocaleFile();
        // we've observed a new string, update the language file.
        if (!this.cache[this.locale][str] && this.updateFiles) {
            this.cache[this.locale][str] = str;
            // include the current directory and locale,
            // since these values could change before the
            // write is performed.
            this._enqueueWrite([this.directory, this.locale, cb]);
        }
        else {
            cb();
        }
        return util.format.apply(util, [this.cache[this.locale][str] || str].concat(args));
    };
    Y18N.prototype._taggedLiteral = function (parts) {
        var args = arguments;
        var str = '';
        parts.forEach(function (part, i) {
            var arg = args[i + 1];
            str += part;
            if (typeof arg !== 'undefined') {
                str += '%s';
            }
        });
        return this.__.apply(null, [str].concat([].slice.call(arguments, 1)));
    };
    Y18N.prototype._enqueueWrite = function (work) {
        this.writeQueue.push(work);
        if (this.writeQueue.length === 1)
            this._processWriteQueue();
    };
    Y18N.prototype._processWriteQueue = function () {
        var _this = this;
        var work = this.writeQueue[0];
        // destructure the enqueued work.
        var directory = work[0];
        var locale = work[1];
        var cb = work[2];
        var languageFile = this._resolveLocaleFile(directory, locale);
        var serializedLocale = JSON.stringify(this.cache[locale], null, 2);
        fs.writeFile(languageFile, serializedLocale, 'utf-8', function (err) {
            _this.writeQueue.shift();
            if (_this.writeQueue.length > 0)
                _this._processWriteQueue();
            cb(err);
        });
    };
    Y18N.prototype._readLocaleFile = function () {
        var localeLookup = {};
        var languageFile = this._resolveLocaleFile(this.directory, this.locale);
        try {
            localeLookup = JSON.parse(fs.readFileSync(languageFile, 'utf-8'));
        }
        catch (err) {
            if (err instanceof SyntaxError) {
                err.message = 'syntax error in ' + languageFile;
            }
            if (err.code === 'ENOENT')
                localeLookup = {};
            else
                throw err;
        }
        this.cache[this.locale] = localeLookup;
    };
    Y18N.prototype._resolveLocaleFile = function (directory, locale) {
        var file = path.resolve(directory, './', locale + '.json');
        if (this.fallbackToLanguage && !this._fileExistsSync(file) && ~locale.lastIndexOf('_')) {
            // attempt fallback to language only
            var languageFile = path.resolve(directory, './', locale.split('_')[0] + '.json');
            if (this._fileExistsSync(languageFile))
                file = languageFile;
        }
        return file;
    };
    // this only exists because fs.existsSync() "will be deprecated"
    // see https://nodejs.org/api/fs.html#fs_fs_existssync_path
    Y18N.prototype._fileExistsSync = function (file) {
        try {
            return fs.statSync(file).isFile();
        }
        catch (err) {
            return false;
        }
    };
    Y18N.prototype.__n = function () {
        var args = Array.prototype.slice.call(arguments);
        var singular = args.shift();
        var plural = args.shift();
        var quantity = args.shift();
        var cb = function () { }; // start with noop.
        if (typeof args[args.length - 1] === 'function')
            cb = args.pop();
        if (!this.cache[this.locale])
            this._readLocaleFile();
        var str = quantity === 1 ? singular : plural;
        if (this.cache[this.locale][singular]) {
            str = this.cache[this.locale][singular][quantity === 1 ? 'one' : 'other'];
        }
        // we've observed a new string, update the language file.
        if (!this.cache[this.locale][singular] && this.updateFiles) {
            this.cache[this.locale][singular] = {
                one: singular,
                other: plural
            };
            // include the current directory and locale,
            // since these values could change before the
            // write is performed.
            this._enqueueWrite([this.directory, this.locale, cb]);
        }
        else {
            cb();
        }
        // if a %d placeholder is provided, add quantity
        // to the arguments expanded by util.format.
        var values = [str];
        if (~str.indexOf('%d'))
            values.push(quantity);
        return util.format.apply(util, values.concat(args));
    };
    Y18N.prototype.setLocale = function (locale) {
        this.locale = locale;
    };
    Y18N.prototype.getLocale = function () {
        return this.locale;
    };
    Y18N.prototype.updateLocale = function (obj) {
        if (!this.cache[this.locale])
            this._readLocaleFile();
        for (var key in obj) {
            this.cache[this.locale][key] = obj[key];
        }
    };
    module.exports = function (opts) {
        var y18n = new Y18N(opts);
        // bind all functions to y18n, so that
        // they can be used in isolation.
        for (var key in y18n) {
            if (typeof y18n[key] === 'function') {
                y18n[key] = y18n[key].bind(y18n);
            }
        }
        return y18n;
    };
},
/* yargs/lib/apply-extends.js */ function _(require, module, exports) {
    const fs = require('fs');
    const path = require('path');
    const YError = require(13) /* ./yerror */;
    let previouslyVisitedConfigs = [];
    function checkForCircularExtends(cfgPath) {
        if (previouslyVisitedConfigs.indexOf(cfgPath) > -1) {
            throw new YError(`Circular extended configurations: '${cfgPath}'.`);
        }
    }
    function getPathToDefaultConfig(cwd, pathToExtend) {
        return path.resolve(cwd, pathToExtend);
    }
    function mergeDeep(config1, config2) {
        const target = {};
        const isObject = obj => obj && typeof obj === 'object' && !Array.isArray(obj);
        Object.assign(target, config1);
        for (const key of Object.keys(config2)) {
            if (isObject(config2[key]) && isObject(target[key])) {
                target[key] = mergeDeep(config1[key], config2[key]);
            }
            else {
                target[key] = config2[key];
            }
        }
        return target;
    }
    function applyExtends(config, cwd, mergeExtends) {
        let defaultConfig = {};
        if (Object.prototype.hasOwnProperty.call(config, 'extends')) {
            if (typeof config.extends !== 'string')
                return defaultConfig;
            const isPath = /\.json|\..*rc$/.test(config.extends);
            let pathToDefault = null;
            if (!isPath) {
                try {
                    pathToDefault = require.resolve(config.extends);
                }
                catch (err) {
                    // most likely this simply isn't a module.
                }
            }
            else {
                pathToDefault = getPathToDefaultConfig(cwd, config.extends);
            }
            // maybe the module uses key for some other reason,
            // err on side of caution.
            if (!pathToDefault && !isPath)
                return config;
            checkForCircularExtends(pathToDefault);
            previouslyVisitedConfigs.push(pathToDefault);
            defaultConfig = isPath ? JSON.parse(fs.readFileSync(pathToDefault, 'utf8')) : require(config.extends);
            delete config.extends;
            defaultConfig = applyExtends(defaultConfig, path.dirname(pathToDefault), mergeExtends);
        }
        previouslyVisitedConfigs = [];
        return mergeExtends ? mergeDeep(defaultConfig, config) : Object.assign({}, defaultConfig, config);
    }
    module.exports = applyExtends;
},
/* yargs/lib/process-argv.js */ function _(require, module, exports) {
    function getProcessArgvBinIndex() {
        // The binary name is the first command line argument for:
        // - bundled Electron apps: bin argv1 argv2 ... argvn
        if (isBundledElectronApp())
            return 0;
        // or the second one (default) for:
        // - standard node apps: node bin.js argv1 argv2 ... argvn
        // - unbundled Electron apps: electron bin.js argv1 arg2 ... argvn
        return 1;
    }
    function isBundledElectronApp() {
        // process.defaultApp is either set by electron in an electron unbundled app, or undefined
        // see https://github.com/electron/electron/blob/master/docs/api/process.md#processdefaultapp-readonly
        return isElectronApp() && !process.defaultApp;
    }
    function isElectronApp() {
        // process.versions.electron is either set by electron, or undefined
        // see https://github.com/electron/electron/blob/master/docs/api/process.md#processversionselectron-readonly
        return !!process.versions.electron;
    }
    function getProcessArgvWithoutBin() {
        return process.argv.slice(getProcessArgvBinIndex() + 1);
    }
    function getProcessArgvBin() {
        return process.argv[getProcessArgvBinIndex()];
    }
    module.exports = {
        getProcessArgvBin,
        getProcessArgvWithoutBin
    };
},
/* get-caller-file/index.js */ function _(require, module, exports) {
    // Call this function in a another function to find out the file from
    // which that function was called from. (Inspects the v8 stack trace)
    //
    // Inspired by http://stackoverflow.com/questions/13227489
    module.exports = function getCallerFile(position) {
        if (position === void 0) {
            position = 2;
        }
        if (position >= Error.stackTraceLimit) {
            throw new TypeError('getCallerFile(position) requires position be less then Error.stackTraceLimit but position was: `' + position + '` and Error.stackTraceLimit was: `' + Error.stackTraceLimit + '`');
        }
        var oldPrepareStackTrace = Error.prepareStackTrace;
        Error.prepareStackTrace = function (_, stack) { return stack; };
        var stack = new Error().stack;
        Error.prepareStackTrace = oldPrepareStackTrace;
        if (stack !== null && typeof stack === 'object') {
            // stack[0] holds this file
            // stack[1] holds where this function was called
            // stack[2] holds the file we're interested in
            return stack[position] ? stack[position].getFileName() : undefined;
        }
    };
},
/* find-up/index.js */ function _(require, module, exports) {
    const path = require('path');
    const locatePath = require(37) /* locate-path */;
    module.exports = (filename, opts = {}) => {
        const startDir = path.resolve(opts.cwd || '');
        const { root } = path.parse(startDir);
        const filenames = [].concat(filename);
        return new Promise(resolve => {
            (function find(dir) {
                locatePath(filenames, { cwd: dir }).then(file => {
                    if (file) {
                        resolve(path.join(dir, file));
                    }
                    else if (dir === root) {
                        resolve(null);
                    }
                    else {
                        find(path.dirname(dir));
                    }
                });
            })(startDir);
        });
    };
    module.exports.sync = (filename, opts = {}) => {
        let dir = path.resolve(opts.cwd || '');
        const { root } = path.parse(dir);
        const filenames = [].concat(filename);
        // eslint-disable-next-line no-constant-condition
        while (true) {
            const file = locatePath.sync(filenames, { cwd: dir });
            if (file) {
                return path.join(dir, file);
            }
            if (dir === root) {
                return null;
            }
            dir = path.dirname(dir);
        }
    };
},
/* locate-path/index.js */ function _(require, module, exports) {
    const path = require('path');
    const pathExists = require(38) /* path-exists */;
    const pLocate = require(39) /* p-locate */;
    module.exports = (iterable, options) => {
        options = Object.assign({
            cwd: process.cwd()
        }, options);
        return pLocate(iterable, el => pathExists(path.resolve(options.cwd, el)), options);
    };
    module.exports.sync = (iterable, options) => {
        options = Object.assign({
            cwd: process.cwd()
        }, options);
        for (const el of iterable) {
            if (pathExists.sync(path.resolve(options.cwd, el))) {
                return el;
            }
        }
    };
},
/* path-exists/index.js */ function _(require, module, exports) {
    const fs = require('fs');
    module.exports = fp => new Promise(resolve => {
        fs.access(fp, err => {
            resolve(!err);
        });
    });
    module.exports.sync = fp => {
        try {
            fs.accessSync(fp);
            return true;
        }
        catch (err) {
            return false;
        }
    };
},
/* p-locate/index.js */ function _(require, module, exports) {
    const pLimit = require(40) /* p-limit */;
    class EndError extends Error {
        constructor(value) {
            super();
            this.value = value;
        }
    }
    // The input can also be a promise, so we `Promise.resolve()` it
    const testElement = (el, tester) => Promise.resolve(el).then(tester);
    // The input can also be a promise, so we `Promise.all()` them both
    const finder = el => Promise.all(el).then(val => val[1] === true && Promise.reject(new EndError(val[0])));
    module.exports = (iterable, tester, opts) => {
        opts = Object.assign({
            concurrency: Infinity,
            preserveOrder: true
        }, opts);
        const limit = pLimit(opts.concurrency);
        // Start all the promises concurrently with optional limit
        const items = [...iterable].map(el => [el, limit(testElement, el, tester)]);
        // Check the promises either serially or concurrently
        const checkLimit = pLimit(opts.preserveOrder ? 1 : Infinity);
        return Promise.all(items.map(el => checkLimit(finder, el)))
            .then(() => { })
            .catch(err => err instanceof EndError ? err.value : Promise.reject(err));
    };
},
/* p-limit/index.js */ function _(require, module, exports) {
    const pTry = require(41) /* p-try */;
    const pLimit = concurrency => {
        if (!((Number.isInteger(concurrency) || concurrency === Infinity) && concurrency > 0)) {
            return Promise.reject(new TypeError('Expected `concurrency` to be a number from 1 and up'));
        }
        const queue = [];
        let activeCount = 0;
        const next = () => {
            activeCount--;
            if (queue.length > 0) {
                queue.shift()();
            }
        };
        const run = (fn, resolve, ...args) => {
            activeCount++;
            const result = pTry(fn, ...args);
            resolve(result);
            result.then(next, next);
        };
        const enqueue = (fn, resolve, ...args) => {
            if (activeCount < concurrency) {
                run(fn, resolve, ...args);
            }
            else {
                queue.push(run.bind(null, fn, resolve, ...args));
            }
        };
        const generator = (fn, ...args) => new Promise(resolve => enqueue(fn, resolve, ...args));
        Object.defineProperties(generator, {
            activeCount: {
                get: () => activeCount
            },
            pendingCount: {
                get: () => queue.length
            }
        });
        return generator;
    };
    module.exports = pLimit;
    module.exports.default = pLimit;
},
/* p-try/index.js */ function _(require, module, exports) {
    const pTry = (fn, ...arguments_) => new Promise(resolve => {
        resolve(fn(...arguments_));
    });
    module.exports = pTry;
    // TODO: remove this in the next major version
    module.exports.default = pTry;
},
/* require-main-filename/index.js */ function _(require, module, exports) {
    module.exports = function (_require) {
        _require = _require || require;
        var main = _require.main;
        if (main && isIISNode(main))
            return handleIISNode(main);
        else
            return main ? main.filename : process.cwd();
    };
    function isIISNode(main) {
        return /\\iisnode\\/.test(main.filename);
    }
    function handleIISNode(main) {
        if (!main.children.length) {
            return main.filename;
        }
        else {
            return main.children[0].filename;
        }
    }
},
/* sys.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = __importStar(require(44) /* typescript */);
    const crypto_1 = __importDefault(require("crypto"));
    const path_1 = require("path");
    function scan(path, extensions, exclude, include, depth) {
        return ts.sys.readDirectory(path, extensions, exclude, include, depth).map((p) => path_1.normalize(p));
    }
    exports.scan = scan;
    function glob(...patterns) {
        return scan(".", undefined, undefined, patterns);
    }
    exports.glob = glob;
    exports.read = ts.sys.readFile;
    exports.write = ts.sys.writeFile;
    exports.file_exists = ts.sys.fileExists;
    exports.directory_exists = ts.sys.directoryExists;
    function rename(path, options) {
        let { dir, name, ext } = path_1.parse(path);
        if (options.dir != null) {
            if (options.base != null)
                dir = dir.replace(options.base, options.dir);
            else
                dir = options.dir;
        }
        if (options.name != null)
            name = options.name(name);
        if (options.ext != null)
            ext = options.ext;
        return path_1.format({ dir, name, ext });
    }
    exports.rename = rename;
    function hash(data) {
        return crypto_1.default.createHash("sha256").update(data).digest("hex");
    }
    exports.hash = hash;
    function hash_file(path) {
        const contents = exports.read(path);
        return contents != null ? hash(contents) : null;
    }
    exports.hash_file = hash_file;
    function read_json(path) {
        const data = exports.read(path);
        if (data == null)
            return undefined;
        else {
            try {
                return JSON.parse(data);
            }
            catch (_a) {
                return undefined;
            }
        }
    }
    exports.read_json = read_json;
},
/* typescript/lib/typescript.js */ function _(require, module, exports) {
    var __spreadArrays = (this && this.__spreadArrays) || function () {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    };
    var __assign = (this && this.__assign) || function () {
        __assign = Object.assign || function (t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    var __generator = (this && this.__generator) || function (thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    };
    var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    };
    var __rest = (this && this.__rest) || function (s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    };
    var __extends = (this && this.__extends) || (function () {
        var extendStatics = function (d, b) {
            extendStatics = Object.setPrototypeOf ||
                ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
                function (d, b) { for (var p in b)
                    if (b.hasOwnProperty(p))
                        d[p] = b[p]; };
            return extendStatics(d, b);
        };
        return function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
    })();
    /* @internal */
    var ts;
    (function (ts) {
        function createMapShim() {
            /** Create a MapLike with good performance. */
            function createDictionaryObject() {
                var map = Object.create(/*prototype*/ null); // eslint-disable-line no-null/no-null
                // Using 'delete' on an object causes V8 to put the object in dictionary mode.
                // This disables creation of hidden classes, which are expensive when an object is
                // constantly changing shape.
                map.__ = undefined;
                delete map.__;
                return map;
            }
            var MapIterator = /** @class */ (function () {
                function MapIterator(currentEntry, selector) {
                    this.currentEntry = currentEntry;
                    this.selector = selector;
                }
                MapIterator.prototype.next = function () {
                    // Navigate to the next entry.
                    while (this.currentEntry) {
                        var skipNext = !!this.currentEntry.skipNext;
                        this.currentEntry = this.currentEntry.nextEntry;
                        if (!skipNext) {
                            break;
                        }
                    }
                    if (this.currentEntry) {
                        return { value: this.selector(this.currentEntry.key, this.currentEntry.value), done: false };
                    }
                    else {
                        return { value: undefined, done: true };
                    }
                };
                return MapIterator;
            }());
            return /** @class */ (function () {
                function class_1() {
                    this.data = createDictionaryObject();
                    this.size = 0;
                    // Create a first (stub) map entry that will not contain a key
                    // and value but serves as starting point for iterators.
                    this.firstEntry = {};
                    // When the map is empty, the last entry is the same as the
                    // first one.
                    this.lastEntry = this.firstEntry;
                }
                class_1.prototype.get = function (key) {
                    var entry = this.data[key];
                    return entry && entry.value;
                };
                class_1.prototype.set = function (key, value) {
                    if (!this.has(key)) {
                        this.size++;
                        // Create a new entry that will be appended at the
                        // end of the linked list.
                        var newEntry = {
                            key: key,
                            value: value
                        };
                        this.data[key] = newEntry;
                        // Adjust the references.
                        var previousLastEntry = this.lastEntry;
                        previousLastEntry.nextEntry = newEntry;
                        newEntry.previousEntry = previousLastEntry;
                        this.lastEntry = newEntry;
                    }
                    else {
                        this.data[key].value = value;
                    }
                    return this;
                };
                class_1.prototype.has = function (key) {
                    // eslint-disable-next-line no-in-operator
                    return key in this.data;
                };
                class_1.prototype.delete = function (key) {
                    if (this.has(key)) {
                        this.size--;
                        var entry = this.data[key];
                        delete this.data[key];
                        // Adjust the linked list references of the neighbor entries.
                        var previousEntry = entry.previousEntry;
                        previousEntry.nextEntry = entry.nextEntry;
                        if (entry.nextEntry) {
                            entry.nextEntry.previousEntry = previousEntry;
                        }
                        // When the deleted entry was the last one, we need to
                        // adjust the lastEntry reference.
                        if (this.lastEntry === entry) {
                            this.lastEntry = previousEntry;
                        }
                        // Adjust the forward reference of the deleted entry
                        // in case an iterator still references it. This allows us
                        // to throw away the entry, but when an active iterator
                        // (which points to the current entry) continues, it will
                        // navigate to the entry that originally came before the
                        // current one and skip it.
                        entry.previousEntry = undefined;
                        entry.nextEntry = previousEntry;
                        entry.skipNext = true;
                        return true;
                    }
                    return false;
                };
                class_1.prototype.clear = function () {
                    this.data = createDictionaryObject();
                    this.size = 0;
                    // Reset the linked list. Note that we must adjust the forward
                    // references of the deleted entries to ensure iterators stuck
                    // in the middle of the list don't continue with deleted entries,
                    // but can continue with new entries added after the clear()
                    // operation.
                    var firstEntry = this.firstEntry;
                    var currentEntry = firstEntry.nextEntry;
                    while (currentEntry) {
                        var nextEntry = currentEntry.nextEntry;
                        currentEntry.previousEntry = undefined;
                        currentEntry.nextEntry = firstEntry;
                        currentEntry.skipNext = true;
                        currentEntry = nextEntry;
                    }
                    firstEntry.nextEntry = undefined;
                    this.lastEntry = firstEntry;
                };
                class_1.prototype.keys = function () {
                    return new MapIterator(this.firstEntry, function (key) { return key; });
                };
                class_1.prototype.values = function () {
                    return new MapIterator(this.firstEntry, function (_key, value) { return value; });
                };
                class_1.prototype.entries = function () {
                    return new MapIterator(this.firstEntry, function (key, value) { return [key, value]; });
                };
                class_1.prototype.forEach = function (action) {
                    var iterator = this.entries();
                    while (true) {
                        var iterResult = iterator.next();
                        if (iterResult.done) {
                            break;
                        }
                        var _a = iterResult.value, key = _a[0], value = _a[1];
                        action(value, key);
                    }
                };
                return class_1;
            }());
        }
        ts.createMapShim = createMapShim;
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        // WARNING: The script `configurePrerelease.ts` uses a regexp to parse out these values.
        // If changing the text in this section, be sure to test `configurePrerelease` too.
        ts.versionMajorMinor = "3.8";
        /** The version of the TypeScript compiler release */
        ts.version = "3.8.3";
        /**
         * Returns the native Map implementation if it is available and compatible (i.e. supports iteration).
         */
        /* @internal */
        function tryGetNativeMap() {
            // eslint-disable-next-line no-in-operator
            return typeof Map !== "undefined" && "entries" in Map.prototype ? Map : undefined;
        }
        ts.tryGetNativeMap = tryGetNativeMap;
        /* @internal */
        ts.Map = tryGetNativeMap() || (function () {
            // NOTE: createMapShim will be defined for typescriptServices.js but not for tsc.js, so we must test for it.
            if (typeof ts.createMapShim === "function") {
                return ts.createMapShim();
            }
            throw new Error("TypeScript requires an environment that provides a compatible native Map implementation.");
        })();
        /* @internal */
        var Comparison;
        (function (Comparison) {
            Comparison[Comparison["LessThan"] = -1] = "LessThan";
            Comparison[Comparison["EqualTo"] = 0] = "EqualTo";
            Comparison[Comparison["GreaterThan"] = 1] = "GreaterThan";
        })(Comparison = ts.Comparison || (ts.Comparison = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        ts.emptyArray = [];
        /** Create a new map. */
        function createMap() {
            return new ts.Map();
        }
        ts.createMap = createMap;
        /** Create a new map from an array of entries. */
        function createMapFromEntries(entries) {
            var map = createMap();
            for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
                var _a = entries_1[_i], key = _a[0], value = _a[1];
                map.set(key, value);
            }
            return map;
        }
        ts.createMapFromEntries = createMapFromEntries;
        /** Create a new map from a template object is provided, the map will copy entries from it. */
        function createMapFromTemplate(template) {
            var map = new ts.Map();
            // Copies keys/values from template. Note that for..in will not throw if
            // template is undefined, and instead will just exit the loop.
            for (var key in template) {
                if (hasOwnProperty.call(template, key)) {
                    map.set(key, template[key]);
                }
            }
            return map;
        }
        ts.createMapFromTemplate = createMapFromTemplate;
        function length(array) {
            return array ? array.length : 0;
        }
        ts.length = length;
        /**
         * Iterates through 'array' by index and performs the callback on each element of array until the callback
         * returns a truthy value, then returns that value.
         * If no such value is found, the callback is applied to each element of array and undefined is returned.
         */
        function forEach(array, callback) {
            if (array) {
                for (var i = 0; i < array.length; i++) {
                    var result = callback(array[i], i);
                    if (result) {
                        return result;
                    }
                }
            }
            return undefined;
        }
        ts.forEach = forEach;
        /**
         * Like `forEach`, but iterates in reverse order.
         */
        function forEachRight(array, callback) {
            if (array) {
                for (var i = array.length - 1; i >= 0; i--) {
                    var result = callback(array[i], i);
                    if (result) {
                        return result;
                    }
                }
            }
            return undefined;
        }
        ts.forEachRight = forEachRight;
        /** Like `forEach`, but suitable for use with numbers and strings (which may be falsy). */
        function firstDefined(array, callback) {
            if (array === undefined) {
                return undefined;
            }
            for (var i = 0; i < array.length; i++) {
                var result = callback(array[i], i);
                if (result !== undefined) {
                    return result;
                }
            }
            return undefined;
        }
        ts.firstDefined = firstDefined;
        function firstDefinedIterator(iter, callback) {
            while (true) {
                var iterResult = iter.next();
                if (iterResult.done) {
                    return undefined;
                }
                var result = callback(iterResult.value);
                if (result !== undefined) {
                    return result;
                }
            }
        }
        ts.firstDefinedIterator = firstDefinedIterator;
        function zipWith(arrayA, arrayB, callback) {
            var result = [];
            ts.Debug.assertEqual(arrayA.length, arrayB.length);
            for (var i = 0; i < arrayA.length; i++) {
                result.push(callback(arrayA[i], arrayB[i], i));
            }
            return result;
        }
        ts.zipWith = zipWith;
        function zipToIterator(arrayA, arrayB) {
            ts.Debug.assertEqual(arrayA.length, arrayB.length);
            var i = 0;
            return {
                next: function () {
                    if (i === arrayA.length) {
                        return { value: undefined, done: true };
                    }
                    i++;
                    return { value: [arrayA[i - 1], arrayB[i - 1]], done: false };
                }
            };
        }
        ts.zipToIterator = zipToIterator;
        function zipToMap(keys, values) {
            ts.Debug.assert(keys.length === values.length);
            var map = createMap();
            for (var i = 0; i < keys.length; ++i) {
                map.set(keys[i], values[i]);
            }
            return map;
        }
        ts.zipToMap = zipToMap;
        /**
         * Iterates through `array` by index and performs the callback on each element of array until the callback
         * returns a falsey value, then returns false.
         * If no such value is found, the callback is applied to each element of array and `true` is returned.
         */
        function every(array, callback) {
            if (array) {
                for (var i = 0; i < array.length; i++) {
                    if (!callback(array[i], i)) {
                        return false;
                    }
                }
            }
            return true;
        }
        ts.every = every;
        function find(array, predicate) {
            for (var i = 0; i < array.length; i++) {
                var value = array[i];
                if (predicate(value, i)) {
                    return value;
                }
            }
            return undefined;
        }
        ts.find = find;
        function findLast(array, predicate) {
            for (var i = array.length - 1; i >= 0; i--) {
                var value = array[i];
                if (predicate(value, i)) {
                    return value;
                }
            }
            return undefined;
        }
        ts.findLast = findLast;
        /** Works like Array.prototype.findIndex, returning `-1` if no element satisfying the predicate is found. */
        function findIndex(array, predicate, startIndex) {
            for (var i = startIndex || 0; i < array.length; i++) {
                if (predicate(array[i], i)) {
                    return i;
                }
            }
            return -1;
        }
        ts.findIndex = findIndex;
        function findLastIndex(array, predicate, startIndex) {
            for (var i = startIndex === undefined ? array.length - 1 : startIndex; i >= 0; i--) {
                if (predicate(array[i], i)) {
                    return i;
                }
            }
            return -1;
        }
        ts.findLastIndex = findLastIndex;
        /**
         * Returns the first truthy result of `callback`, or else fails.
         * This is like `forEach`, but never returns undefined.
         */
        function findMap(array, callback) {
            for (var i = 0; i < array.length; i++) {
                var result = callback(array[i], i);
                if (result) {
                    return result;
                }
            }
            return ts.Debug.fail();
        }
        ts.findMap = findMap;
        function contains(array, value, equalityComparer) {
            if (equalityComparer === void 0) {
                equalityComparer = equateValues;
            }
            if (array) {
                for (var _i = 0, array_1 = array; _i < array_1.length; _i++) {
                    var v = array_1[_i];
                    if (equalityComparer(v, value)) {
                        return true;
                    }
                }
            }
            return false;
        }
        ts.contains = contains;
        function arraysEqual(a, b, equalityComparer) {
            if (equalityComparer === void 0) {
                equalityComparer = equateValues;
            }
            return a.length === b.length && a.every(function (x, i) { return equalityComparer(x, b[i]); });
        }
        ts.arraysEqual = arraysEqual;
        function indexOfAnyCharCode(text, charCodes, start) {
            for (var i = start || 0; i < text.length; i++) {
                if (contains(charCodes, text.charCodeAt(i))) {
                    return i;
                }
            }
            return -1;
        }
        ts.indexOfAnyCharCode = indexOfAnyCharCode;
        function countWhere(array, predicate) {
            var count = 0;
            if (array) {
                for (var i = 0; i < array.length; i++) {
                    var v = array[i];
                    if (predicate(v, i)) {
                        count++;
                    }
                }
            }
            return count;
        }
        ts.countWhere = countWhere;
        function filter(array, f) {
            if (array) {
                var len = array.length;
                var i = 0;
                while (i < len && f(array[i]))
                    i++;
                if (i < len) {
                    var result = array.slice(0, i);
                    i++;
                    while (i < len) {
                        var item = array[i];
                        if (f(item)) {
                            result.push(item);
                        }
                        i++;
                    }
                    return result;
                }
            }
            return array;
        }
        ts.filter = filter;
        function filterMutate(array, f) {
            var outIndex = 0;
            for (var i = 0; i < array.length; i++) {
                if (f(array[i], i, array)) {
                    array[outIndex] = array[i];
                    outIndex++;
                }
            }
            array.length = outIndex;
        }
        ts.filterMutate = filterMutate;
        function clear(array) {
            array.length = 0;
        }
        ts.clear = clear;
        function map(array, f) {
            var result;
            if (array) {
                result = [];
                for (var i = 0; i < array.length; i++) {
                    result.push(f(array[i], i));
                }
            }
            return result;
        }
        ts.map = map;
        function mapIterator(iter, mapFn) {
            return {
                next: function () {
                    var iterRes = iter.next();
                    return iterRes.done ? iterRes : { value: mapFn(iterRes.value), done: false };
                }
            };
        }
        ts.mapIterator = mapIterator;
        function sameMap(array, f) {
            if (array) {
                for (var i = 0; i < array.length; i++) {
                    var item = array[i];
                    var mapped = f(item, i);
                    if (item !== mapped) {
                        var result = array.slice(0, i);
                        result.push(mapped);
                        for (i++; i < array.length; i++) {
                            result.push(f(array[i], i));
                        }
                        return result;
                    }
                }
            }
            return array;
        }
        ts.sameMap = sameMap;
        /**
         * Flattens an array containing a mix of array or non-array elements.
         *
         * @param array The array to flatten.
         */
        function flatten(array) {
            var result = [];
            for (var _i = 0, array_2 = array; _i < array_2.length; _i++) {
                var v = array_2[_i];
                if (v) {
                    if (isArray(v)) {
                        addRange(result, v);
                    }
                    else {
                        result.push(v);
                    }
                }
            }
            return result;
        }
        ts.flatten = flatten;
        /**
         * Maps an array. If the mapped value is an array, it is spread into the result.
         *
         * @param array The array to map.
         * @param mapfn The callback used to map the result into one or more values.
         */
        function flatMap(array, mapfn) {
            var result;
            if (array) {
                for (var i = 0; i < array.length; i++) {
                    var v = mapfn(array[i], i);
                    if (v) {
                        if (isArray(v)) {
                            result = addRange(result, v);
                        }
                        else {
                            result = append(result, v);
                        }
                    }
                }
            }
            return result || ts.emptyArray;
        }
        ts.flatMap = flatMap;
        function flatMapToMutable(array, mapfn) {
            var result = [];
            if (array) {
                for (var i = 0; i < array.length; i++) {
                    var v = mapfn(array[i], i);
                    if (v) {
                        if (isArray(v)) {
                            addRange(result, v);
                        }
                        else {
                            result.push(v);
                        }
                    }
                }
            }
            return result;
        }
        ts.flatMapToMutable = flatMapToMutable;
        function flatMapIterator(iter, mapfn) {
            var first = iter.next();
            if (first.done) {
                return ts.emptyIterator;
            }
            var currentIter = getIterator(first.value);
            return {
                next: function () {
                    while (true) {
                        var currentRes = currentIter.next();
                        if (!currentRes.done) {
                            return currentRes;
                        }
                        var iterRes = iter.next();
                        if (iterRes.done) {
                            return iterRes;
                        }
                        currentIter = getIterator(iterRes.value);
                    }
                },
            };
            function getIterator(x) {
                var res = mapfn(x);
                return res === undefined ? ts.emptyIterator : isArray(res) ? arrayIterator(res) : res;
            }
        }
        ts.flatMapIterator = flatMapIterator;
        function sameFlatMap(array, mapfn) {
            var result;
            if (array) {
                for (var i = 0; i < array.length; i++) {
                    var item = array[i];
                    var mapped = mapfn(item, i);
                    if (result || item !== mapped || isArray(mapped)) {
                        if (!result) {
                            result = array.slice(0, i);
                        }
                        if (isArray(mapped)) {
                            addRange(result, mapped);
                        }
                        else {
                            result.push(mapped);
                        }
                    }
                }
            }
            return result || array;
        }
        ts.sameFlatMap = sameFlatMap;
        function mapAllOrFail(array, mapFn) {
            var result = [];
            for (var i = 0; i < array.length; i++) {
                var mapped = mapFn(array[i], i);
                if (mapped === undefined) {
                    return undefined;
                }
                result.push(mapped);
            }
            return result;
        }
        ts.mapAllOrFail = mapAllOrFail;
        function mapDefined(array, mapFn) {
            var result = [];
            if (array) {
                for (var i = 0; i < array.length; i++) {
                    var mapped = mapFn(array[i], i);
                    if (mapped !== undefined) {
                        result.push(mapped);
                    }
                }
            }
            return result;
        }
        ts.mapDefined = mapDefined;
        function mapDefinedIterator(iter, mapFn) {
            return {
                next: function () {
                    while (true) {
                        var res = iter.next();
                        if (res.done) {
                            return res;
                        }
                        var value = mapFn(res.value);
                        if (value !== undefined) {
                            return { value: value, done: false };
                        }
                    }
                }
            };
        }
        ts.mapDefinedIterator = mapDefinedIterator;
        function mapDefinedMap(map, mapValue, mapKey) {
            if (mapKey === void 0) {
                mapKey = identity;
            }
            var result = createMap();
            map.forEach(function (value, key) {
                var mapped = mapValue(value, key);
                if (mapped !== undefined) {
                    result.set(mapKey(key), mapped);
                }
            });
            return result;
        }
        ts.mapDefinedMap = mapDefinedMap;
        ts.emptyIterator = { next: function () { return ({ value: undefined, done: true }); } };
        function singleIterator(value) {
            var done = false;
            return {
                next: function () {
                    var wasDone = done;
                    done = true;
                    return wasDone ? { value: undefined, done: true } : { value: value, done: false };
                }
            };
        }
        ts.singleIterator = singleIterator;
        function spanMap(array, keyfn, mapfn) {
            var result;
            if (array) {
                result = [];
                var len = array.length;
                var previousKey = void 0;
                var key = void 0;
                var start = 0;
                var pos = 0;
                while (start < len) {
                    while (pos < len) {
                        var value = array[pos];
                        key = keyfn(value, pos);
                        if (pos === 0) {
                            previousKey = key;
                        }
                        else if (key !== previousKey) {
                            break;
                        }
                        pos++;
                    }
                    if (start < pos) {
                        var v = mapfn(array.slice(start, pos), previousKey, start, pos);
                        if (v) {
                            result.push(v);
                        }
                        start = pos;
                    }
                    previousKey = key;
                    pos++;
                }
            }
            return result;
        }
        ts.spanMap = spanMap;
        function mapEntries(map, f) {
            if (!map) {
                return undefined;
            }
            var result = createMap();
            map.forEach(function (value, key) {
                var _a = f(key, value), newKey = _a[0], newValue = _a[1];
                result.set(newKey, newValue);
            });
            return result;
        }
        ts.mapEntries = mapEntries;
        function some(array, predicate) {
            if (array) {
                if (predicate) {
                    for (var _i = 0, array_3 = array; _i < array_3.length; _i++) {
                        var v = array_3[_i];
                        if (predicate(v)) {
                            return true;
                        }
                    }
                }
                else {
                    return array.length > 0;
                }
            }
            return false;
        }
        ts.some = some;
        /** Calls the callback with (start, afterEnd) index pairs for each range where 'pred' is true. */
        function getRangesWhere(arr, pred, cb) {
            var start;
            for (var i = 0; i < arr.length; i++) {
                if (pred(arr[i])) {
                    start = start === undefined ? i : start;
                }
                else {
                    if (start !== undefined) {
                        cb(start, i);
                        start = undefined;
                    }
                }
            }
            if (start !== undefined)
                cb(start, arr.length);
        }
        ts.getRangesWhere = getRangesWhere;
        function concatenate(array1, array2) {
            if (!some(array2))
                return array1;
            if (!some(array1))
                return array2;
            return __spreadArrays(array1, array2);
        }
        ts.concatenate = concatenate;
        function selectIndex(_, i) {
            return i;
        }
        function indicesOf(array) {
            return array.map(selectIndex);
        }
        ts.indicesOf = indicesOf;
        function deduplicateRelational(array, equalityComparer, comparer) {
            // Perform a stable sort of the array. This ensures the first entry in a list of
            // duplicates remains the first entry in the result.
            var indices = indicesOf(array);
            stableSortIndices(array, indices, comparer);
            var last = array[indices[0]];
            var deduplicated = [indices[0]];
            for (var i = 1; i < indices.length; i++) {
                var index = indices[i];
                var item = array[index];
                if (!equalityComparer(last, item)) {
                    deduplicated.push(index);
                    last = item;
                }
            }
            // restore original order
            deduplicated.sort();
            return deduplicated.map(function (i) { return array[i]; });
        }
        function deduplicateEquality(array, equalityComparer) {
            var result = [];
            for (var _i = 0, array_4 = array; _i < array_4.length; _i++) {
                var item = array_4[_i];
                pushIfUnique(result, item, equalityComparer);
            }
            return result;
        }
        /**
         * Deduplicates an unsorted array.
         * @param equalityComparer An `EqualityComparer` used to determine if two values are duplicates.
         * @param comparer An optional `Comparer` used to sort entries before comparison, though the
         * result will remain in the original order in `array`.
         */
        function deduplicate(array, equalityComparer, comparer) {
            return array.length === 0 ? [] :
                array.length === 1 ? array.slice() :
                    comparer ? deduplicateRelational(array, equalityComparer, comparer) :
                        deduplicateEquality(array, equalityComparer);
        }
        ts.deduplicate = deduplicate;
        /**
         * Deduplicates an array that has already been sorted.
         */
        function deduplicateSorted(array, comparer) {
            if (array.length === 0)
                return ts.emptyArray;
            var last = array[0];
            var deduplicated = [last];
            for (var i = 1; i < array.length; i++) {
                var next = array[i];
                switch (comparer(next, last)) {
                    // equality comparison
                    case true:
                    // relational comparison
                    // falls through
                    case 0 /* EqualTo */:
                        continue;
                    case -1 /* LessThan */:
                        // If `array` is sorted, `next` should **never** be less than `last`.
                        return ts.Debug.fail("Array is unsorted.");
                }
                deduplicated.push(last = next);
            }
            return deduplicated;
        }
        function insertSorted(array, insert, compare) {
            if (array.length === 0) {
                array.push(insert);
                return;
            }
            var insertIndex = binarySearch(array, insert, identity, compare);
            if (insertIndex < 0) {
                array.splice(~insertIndex, 0, insert);
            }
        }
        ts.insertSorted = insertSorted;
        function sortAndDeduplicate(array, comparer, equalityComparer) {
            return deduplicateSorted(sort(array, comparer), equalityComparer || comparer || compareStringsCaseSensitive);
        }
        ts.sortAndDeduplicate = sortAndDeduplicate;
        function arrayIsEqualTo(array1, array2, equalityComparer) {
            if (equalityComparer === void 0) {
                equalityComparer = equateValues;
            }
            if (!array1 || !array2) {
                return array1 === array2;
            }
            if (array1.length !== array2.length) {
                return false;
            }
            for (var i = 0; i < array1.length; i++) {
                if (!equalityComparer(array1[i], array2[i], i)) {
                    return false;
                }
            }
            return true;
        }
        ts.arrayIsEqualTo = arrayIsEqualTo;
        function compact(array) {
            var result;
            if (array) {
                for (var i = 0; i < array.length; i++) {
                    var v = array[i];
                    if (result || !v) {
                        if (!result) {
                            result = array.slice(0, i);
                        }
                        if (v) {
                            result.push(v);
                        }
                    }
                }
            }
            return result || array;
        }
        ts.compact = compact;
        /**
         * Gets the relative complement of `arrayA` with respect to `arrayB`, returning the elements that
         * are not present in `arrayA` but are present in `arrayB`. Assumes both arrays are sorted
         * based on the provided comparer.
         */
        function relativeComplement(arrayA, arrayB, comparer) {
            if (!arrayB || !arrayA || arrayB.length === 0 || arrayA.length === 0)
                return arrayB;
            var result = [];
            loopB: for (var offsetA = 0, offsetB = 0; offsetB < arrayB.length; offsetB++) {
                if (offsetB > 0) {
                    // Ensure `arrayB` is properly sorted.
                    ts.Debug.assertGreaterThanOrEqual(comparer(arrayB[offsetB], arrayB[offsetB - 1]), 0 /* EqualTo */);
                }
                loopA: for (var startA = offsetA; offsetA < arrayA.length; offsetA++) {
                    if (offsetA > startA) {
                        // Ensure `arrayA` is properly sorted. We only need to perform this check if
                        // `offsetA` has changed since we entered the loop.
                        ts.Debug.assertGreaterThanOrEqual(comparer(arrayA[offsetA], arrayA[offsetA - 1]), 0 /* EqualTo */);
                    }
                    switch (comparer(arrayB[offsetB], arrayA[offsetA])) {
                        case -1 /* LessThan */:
                            // If B is less than A, B does not exist in arrayA. Add B to the result and
                            // move to the next element in arrayB without changing the current position
                            // in arrayA.
                            result.push(arrayB[offsetB]);
                            continue loopB;
                        case 0 /* EqualTo */:
                            // If B is equal to A, B exists in arrayA. Move to the next element in
                            // arrayB without adding B to the result or changing the current position
                            // in arrayA.
                            continue loopB;
                        case 1 /* GreaterThan */:
                            // If B is greater than A, we need to keep looking for B in arrayA. Move to
                            // the next element in arrayA and recheck.
                            continue loopA;
                    }
                }
            }
            return result;
        }
        ts.relativeComplement = relativeComplement;
        function sum(array, prop) {
            var result = 0;
            for (var _i = 0, array_5 = array; _i < array_5.length; _i++) {
                var v = array_5[_i];
                result += v[prop];
            }
            return result;
        }
        ts.sum = sum;
        function append(to, value) {
            if (value === undefined)
                return to;
            if (to === undefined)
                return [value];
            to.push(value);
            return to;
        }
        ts.append = append;
        /**
         * Gets the actual offset into an array for a relative offset. Negative offsets indicate a
         * position offset from the end of the array.
         */
        function toOffset(array, offset) {
            return offset < 0 ? array.length + offset : offset;
        }
        function addRange(to, from, start, end) {
            if (from === undefined || from.length === 0)
                return to;
            if (to === undefined)
                return from.slice(start, end);
            start = start === undefined ? 0 : toOffset(from, start);
            end = end === undefined ? from.length : toOffset(from, end);
            for (var i = start; i < end && i < from.length; i++) {
                if (from[i] !== undefined) {
                    to.push(from[i]);
                }
            }
            return to;
        }
        ts.addRange = addRange;
        /**
         * @return Whether the value was added.
         */
        function pushIfUnique(array, toAdd, equalityComparer) {
            if (contains(array, toAdd, equalityComparer)) {
                return false;
            }
            else {
                array.push(toAdd);
                return true;
            }
        }
        ts.pushIfUnique = pushIfUnique;
        /**
         * Unlike `pushIfUnique`, this can take `undefined` as an input, and returns a new array.
         */
        function appendIfUnique(array, toAdd, equalityComparer) {
            if (array) {
                pushIfUnique(array, toAdd, equalityComparer);
                return array;
            }
            else {
                return [toAdd];
            }
        }
        ts.appendIfUnique = appendIfUnique;
        function stableSortIndices(array, indices, comparer) {
            // sort indices by value then position
            indices.sort(function (x, y) { return comparer(array[x], array[y]) || compareValues(x, y); });
        }
        /**
         * Returns a new sorted array.
         */
        function sort(array, comparer) {
            return (array.length === 0 ? array : array.slice().sort(comparer));
        }
        ts.sort = sort;
        function arrayIterator(array) {
            var i = 0;
            return { next: function () {
                    if (i === array.length) {
                        return { value: undefined, done: true };
                    }
                    else {
                        i++;
                        return { value: array[i - 1], done: false };
                    }
                } };
        }
        ts.arrayIterator = arrayIterator;
        function arrayReverseIterator(array) {
            var i = array.length;
            return {
                next: function () {
                    if (i === 0) {
                        return { value: undefined, done: true };
                    }
                    else {
                        i--;
                        return { value: array[i], done: false };
                    }
                }
            };
        }
        ts.arrayReverseIterator = arrayReverseIterator;
        /**
         * Stable sort of an array. Elements equal to each other maintain their relative position in the array.
         */
        function stableSort(array, comparer) {
            var indices = indicesOf(array);
            stableSortIndices(array, indices, comparer);
            return indices.map(function (i) { return array[i]; });
        }
        ts.stableSort = stableSort;
        function rangeEquals(array1, array2, pos, end) {
            while (pos < end) {
                if (array1[pos] !== array2[pos]) {
                    return false;
                }
                pos++;
            }
            return true;
        }
        ts.rangeEquals = rangeEquals;
        /**
         * Returns the element at a specific offset in an array if non-empty, `undefined` otherwise.
         * A negative offset indicates the element should be retrieved from the end of the array.
         */
        function elementAt(array, offset) {
            if (array) {
                offset = toOffset(array, offset);
                if (offset < array.length) {
                    return array[offset];
                }
            }
            return undefined;
        }
        ts.elementAt = elementAt;
        /**
         * Returns the first element of an array if non-empty, `undefined` otherwise.
         */
        function firstOrUndefined(array) {
            return array.length === 0 ? undefined : array[0];
        }
        ts.firstOrUndefined = firstOrUndefined;
        function first(array) {
            ts.Debug.assert(array.length !== 0);
            return array[0];
        }
        ts.first = first;
        /**
         * Returns the last element of an array if non-empty, `undefined` otherwise.
         */
        function lastOrUndefined(array) {
            return array.length === 0 ? undefined : array[array.length - 1];
        }
        ts.lastOrUndefined = lastOrUndefined;
        function last(array) {
            ts.Debug.assert(array.length !== 0);
            return array[array.length - 1];
        }
        ts.last = last;
        /**
         * Returns the only element of an array if it contains only one element, `undefined` otherwise.
         */
        function singleOrUndefined(array) {
            return array && array.length === 1
                ? array[0]
                : undefined;
        }
        ts.singleOrUndefined = singleOrUndefined;
        function singleOrMany(array) {
            return array && array.length === 1
                ? array[0]
                : array;
        }
        ts.singleOrMany = singleOrMany;
        function replaceElement(array, index, value) {
            var result = array.slice(0);
            result[index] = value;
            return result;
        }
        ts.replaceElement = replaceElement;
        /**
         * Performs a binary search, finding the index at which `value` occurs in `array`.
         * If no such index is found, returns the 2's-complement of first index at which
         * `array[index]` exceeds `value`.
         * @param array A sorted array whose first element must be no larger than number
         * @param value The value to be searched for in the array.
         * @param keySelector A callback used to select the search key from `value` and each element of
         * `array`.
         * @param keyComparer A callback used to compare two keys in a sorted array.
         * @param offset An offset into `array` at which to start the search.
         */
        function binarySearch(array, value, keySelector, keyComparer, offset) {
            return binarySearchKey(array, keySelector(value), keySelector, keyComparer, offset);
        }
        ts.binarySearch = binarySearch;
        /**
         * Performs a binary search, finding the index at which an object with `key` occurs in `array`.
         * If no such index is found, returns the 2's-complement of first index at which
         * `array[index]` exceeds `key`.
         * @param array A sorted array whose first element must be no larger than number
         * @param key The key to be searched for in the array.
         * @param keySelector A callback used to select the search key from each element of `array`.
         * @param keyComparer A callback used to compare two keys in a sorted array.
         * @param offset An offset into `array` at which to start the search.
         */
        function binarySearchKey(array, key, keySelector, keyComparer, offset) {
            if (!some(array)) {
                return -1;
            }
            var low = offset || 0;
            var high = array.length - 1;
            while (low <= high) {
                var middle = low + ((high - low) >> 1);
                var midKey = keySelector(array[middle]);
                switch (keyComparer(midKey, key)) {
                    case -1 /* LessThan */:
                        low = middle + 1;
                        break;
                    case 0 /* EqualTo */:
                        return middle;
                    case 1 /* GreaterThan */:
                        high = middle - 1;
                        break;
                }
            }
            return ~low;
        }
        ts.binarySearchKey = binarySearchKey;
        function reduceLeft(array, f, initial, start, count) {
            if (array && array.length > 0) {
                var size = array.length;
                if (size > 0) {
                    var pos = start === undefined || start < 0 ? 0 : start;
                    var end = count === undefined || pos + count > size - 1 ? size - 1 : pos + count;
                    var result = void 0;
                    if (arguments.length <= 2) {
                        result = array[pos];
                        pos++;
                    }
                    else {
                        result = initial;
                    }
                    while (pos <= end) {
                        result = f(result, array[pos], pos);
                        pos++;
                    }
                    return result;
                }
            }
            return initial;
        }
        ts.reduceLeft = reduceLeft;
        var hasOwnProperty = Object.prototype.hasOwnProperty;
        /**
         * Indicates whether a map-like contains an own property with the specified key.
         *
         * @param map A map-like.
         * @param key A property key.
         */
        function hasProperty(map, key) {
            return hasOwnProperty.call(map, key);
        }
        ts.hasProperty = hasProperty;
        /**
         * Gets the value of an owned property in a map-like.
         *
         * @param map A map-like.
         * @param key A property key.
         */
        function getProperty(map, key) {
            return hasOwnProperty.call(map, key) ? map[key] : undefined;
        }
        ts.getProperty = getProperty;
        /**
         * Gets the owned, enumerable property keys of a map-like.
         */
        function getOwnKeys(map) {
            var keys = [];
            for (var key in map) {
                if (hasOwnProperty.call(map, key)) {
                    keys.push(key);
                }
            }
            return keys;
        }
        ts.getOwnKeys = getOwnKeys;
        function getAllKeys(obj) {
            var result = [];
            do {
                var names = Object.getOwnPropertyNames(obj);
                for (var _i = 0, names_1 = names; _i < names_1.length; _i++) {
                    var name = names_1[_i];
                    pushIfUnique(result, name);
                }
            } while (obj = Object.getPrototypeOf(obj));
            return result;
        }
        ts.getAllKeys = getAllKeys;
        function getOwnValues(sparseArray) {
            var values = [];
            for (var key in sparseArray) {
                if (hasOwnProperty.call(sparseArray, key)) {
                    values.push(sparseArray[key]);
                }
            }
            return values;
        }
        ts.getOwnValues = getOwnValues;
        function arrayFrom(iterator, map) {
            var result = [];
            for (var iterResult = iterator.next(); !iterResult.done; iterResult = iterator.next()) {
                result.push(map ? map(iterResult.value) : iterResult.value);
            }
            return result;
        }
        ts.arrayFrom = arrayFrom;
        function assign(t) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            for (var _a = 0, args_1 = args; _a < args_1.length; _a++) {
                var arg = args_1[_a];
                if (arg === undefined)
                    continue;
                for (var p in arg) {
                    if (hasProperty(arg, p)) {
                        t[p] = arg[p];
                    }
                }
            }
            return t;
        }
        ts.assign = assign;
        /**
         * Performs a shallow equality comparison of the contents of two map-likes.
         *
         * @param left A map-like whose properties should be compared.
         * @param right A map-like whose properties should be compared.
         */
        function equalOwnProperties(left, right, equalityComparer) {
            if (equalityComparer === void 0) {
                equalityComparer = equateValues;
            }
            if (left === right)
                return true;
            if (!left || !right)
                return false;
            for (var key in left) {
                if (hasOwnProperty.call(left, key)) {
                    if (!hasOwnProperty.call(right, key))
                        return false;
                    if (!equalityComparer(left[key], right[key]))
                        return false;
                }
            }
            for (var key in right) {
                if (hasOwnProperty.call(right, key)) {
                    if (!hasOwnProperty.call(left, key))
                        return false;
                }
            }
            return true;
        }
        ts.equalOwnProperties = equalOwnProperties;
        function arrayToMap(array, makeKey, makeValue) {
            if (makeValue === void 0) {
                makeValue = identity;
            }
            var result = createMap();
            for (var _i = 0, array_6 = array; _i < array_6.length; _i++) {
                var value = array_6[_i];
                var key = makeKey(value);
                if (key !== undefined)
                    result.set(key, makeValue(value));
            }
            return result;
        }
        ts.arrayToMap = arrayToMap;
        function arrayToNumericMap(array, makeKey, makeValue) {
            if (makeValue === void 0) {
                makeValue = identity;
            }
            var result = [];
            for (var _i = 0, array_7 = array; _i < array_7.length; _i++) {
                var value = array_7[_i];
                result[makeKey(value)] = makeValue(value);
            }
            return result;
        }
        ts.arrayToNumericMap = arrayToNumericMap;
        function arrayToMultiMap(values, makeKey, makeValue) {
            if (makeValue === void 0) {
                makeValue = identity;
            }
            var result = createMultiMap();
            for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
                var value = values_1[_i];
                result.add(makeKey(value), makeValue(value));
            }
            return result;
        }
        ts.arrayToMultiMap = arrayToMultiMap;
        function group(values, getGroupId, resultSelector) {
            if (resultSelector === void 0) {
                resultSelector = identity;
            }
            return arrayFrom(arrayToMultiMap(values, getGroupId).values(), resultSelector);
        }
        ts.group = group;
        function clone(object) {
            var result = {};
            for (var id in object) {
                if (hasOwnProperty.call(object, id)) {
                    result[id] = object[id];
                }
            }
            return result;
        }
        ts.clone = clone;
        /**
         * Creates a new object by adding the own properties of `second`, then the own properties of `first`.
         *
         * NOTE: This means that if a property exists in both `first` and `second`, the property in `first` will be chosen.
         */
        function extend(first, second) {
            var result = {};
            for (var id in second) {
                if (hasOwnProperty.call(second, id)) {
                    result[id] = second[id];
                }
            }
            for (var id in first) {
                if (hasOwnProperty.call(first, id)) {
                    result[id] = first[id];
                }
            }
            return result;
        }
        ts.extend = extend;
        function copyProperties(first, second) {
            for (var id in second) {
                if (hasOwnProperty.call(second, id)) {
                    first[id] = second[id];
                }
            }
        }
        ts.copyProperties = copyProperties;
        function maybeBind(obj, fn) {
            return fn ? fn.bind(obj) : undefined;
        }
        ts.maybeBind = maybeBind;
        function mapMap(map, f) {
            var result = createMap();
            map.forEach(function (t, key) { return result.set.apply(result, (f(t, key))); });
            return result;
        }
        ts.mapMap = mapMap;
        function createMultiMap() {
            var map = createMap();
            map.add = multiMapAdd;
            map.remove = multiMapRemove;
            return map;
        }
        ts.createMultiMap = createMultiMap;
        function multiMapAdd(key, value) {
            var values = this.get(key);
            if (values) {
                values.push(value);
            }
            else {
                this.set(key, values = [value]);
            }
            return values;
        }
        function multiMapRemove(key, value) {
            var values = this.get(key);
            if (values) {
                unorderedRemoveItem(values, value);
                if (!values.length) {
                    this.delete(key);
                }
            }
        }
        /**
         * Tests whether a value is an array.
         */
        function isArray(value) {
            return Array.isArray ? Array.isArray(value) : value instanceof Array;
        }
        ts.isArray = isArray;
        function toArray(value) {
            return isArray(value) ? value : [value];
        }
        ts.toArray = toArray;
        /**
         * Tests whether a value is string
         */
        function isString(text) {
            return typeof text === "string";
        }
        ts.isString = isString;
        function isNumber(x) {
            return typeof x === "number";
        }
        ts.isNumber = isNumber;
        function tryCast(value, test) {
            return value !== undefined && test(value) ? value : undefined;
        }
        ts.tryCast = tryCast;
        function cast(value, test) {
            if (value !== undefined && test(value))
                return value;
            return ts.Debug.fail("Invalid cast. The supplied value " + value + " did not pass the test '" + ts.Debug.getFunctionName(test) + "'.");
        }
        ts.cast = cast;
        /** Does nothing. */
        function noop(_) { }
        ts.noop = noop;
        /** Do nothing and return false */
        function returnFalse() { return false; }
        ts.returnFalse = returnFalse;
        /** Do nothing and return true */
        function returnTrue() { return true; }
        ts.returnTrue = returnTrue;
        /** Do nothing and return undefined */
        function returnUndefined() { return undefined; }
        ts.returnUndefined = returnUndefined;
        /** Returns its argument. */
        function identity(x) { return x; }
        ts.identity = identity;
        /** Returns lower case string */
        function toLowerCase(x) { return x.toLowerCase(); }
        ts.toLowerCase = toLowerCase;
        // We convert the file names to lower case as key for file name on case insensitive file system
        // While doing so we need to handle special characters (eg \u0130) to ensure that we dont convert
        // it to lower case, fileName with its lowercase form can exist along side it.
        // Handle special characters and make those case sensitive instead
        //
        // |-#--|-Unicode--|-Char code-|-Desc-------------------------------------------------------------------|
        // | 1. | i        | 105       | Ascii i                                                                |
        // | 2. | I        | 73        | Ascii I                                                                |
        // |-------- Special characters ------------------------------------------------------------------------|
        // | 3. | \u0130   | 304       | Uppper case I with dot above                                           |
        // | 4. | i,\u0307 | 105,775   | i, followed by 775: Lower case of (3rd item)                           |
        // | 5. | I,\u0307 | 73,775    | I, followed by 775: Upper case of (4th item), lower case is (4th item) |
        // | 6. | \u0131   | 305       | Lower case i without dot, upper case is I (2nd item)                   |
        // | 7. | \u00DF   | 223       | Lower case sharp s                                                     |
        //
        // Because item 3 is special where in its lowercase character has its own
        // upper case form we cant convert its case.
        // Rest special characters are either already in lower case format or
        // they have corresponding upper case character so they dont need special handling
        //
        // But to avoid having to do string building for most common cases, also ignore
        // a-z, 0-9, \u0131, \u00DF, \, /, ., : and space
        var fileNameLowerCaseRegExp = /[^\u0130\u0131\u00DFa-z0-9\\/:\-_\. ]+/g;
        /**
         * Case insensitive file systems have descripencies in how they handle some characters (eg. turkish Upper case I with dot on top - \u0130)
         * This function is used in places where we want to make file name as a key on these systems
         * It is possible on mac to be able to refer to file name with I with dot on top as a fileName with its lower case form
         * But on windows we cannot. Windows can have fileName with I with dot on top next to its lower case and they can not each be referred with the lowercase forms
         * Technically we would want this function to be platform sepcific as well but
         * our api has till now only taken caseSensitive as the only input and just for some characters we dont want to update API and ensure all customers use those api
         * We could use upper case and we would still need to deal with the descripencies but
         * we want to continue using lower case since in most cases filenames are lowercasewe and wont need any case changes and avoid having to store another string for the key
         * So for this function purpose, we go ahead and assume character I with dot on top it as case sensitive since its very unlikely to use lower case form of that special character
         */
        function toFileNameLowerCase(x) {
            return fileNameLowerCaseRegExp.test(x) ?
                x.replace(fileNameLowerCaseRegExp, toLowerCase) :
                x;
        }
        ts.toFileNameLowerCase = toFileNameLowerCase;
        /** Throws an error because a function is not implemented. */
        function notImplemented() {
            throw new Error("Not implemented");
        }
        ts.notImplemented = notImplemented;
        function memoize(callback) {
            var value;
            return function () {
                if (callback) {
                    value = callback();
                    callback = undefined;
                }
                return value;
            };
        }
        ts.memoize = memoize;
        function compose(a, b, c, d, e) {
            if (!!e) {
                var args_2 = [];
                for (var i = 0; i < arguments.length; i++) {
                    args_2[i] = arguments[i];
                }
                return function (t) { return reduceLeft(args_2, function (u, f) { return f(u); }, t); };
            }
            else if (d) {
                return function (t) { return d(c(b(a(t)))); };
            }
            else if (c) {
                return function (t) { return c(b(a(t))); };
            }
            else if (b) {
                return function (t) { return b(a(t)); };
            }
            else if (a) {
                return function (t) { return a(t); };
            }
            else {
                return function (t) { return t; };
            }
        }
        ts.compose = compose;
        var AssertionLevel;
        (function (AssertionLevel) {
            AssertionLevel[AssertionLevel["None"] = 0] = "None";
            AssertionLevel[AssertionLevel["Normal"] = 1] = "Normal";
            AssertionLevel[AssertionLevel["Aggressive"] = 2] = "Aggressive";
            AssertionLevel[AssertionLevel["VeryAggressive"] = 3] = "VeryAggressive";
        })(AssertionLevel = ts.AssertionLevel || (ts.AssertionLevel = {}));
        function equateValues(a, b) {
            return a === b;
        }
        ts.equateValues = equateValues;
        /**
         * Compare the equality of two strings using a case-sensitive ordinal comparison.
         *
         * Case-sensitive comparisons compare both strings one code-point at a time using the integer
         * value of each code-point after applying `toUpperCase` to each string. We always map both
         * strings to their upper-case form as some unicode characters do not properly round-trip to
         * lowercase (such as `ẞ` (German sharp capital s)).
         */
        function equateStringsCaseInsensitive(a, b) {
            return a === b
                || a !== undefined
                    && b !== undefined
                    && a.toUpperCase() === b.toUpperCase();
        }
        ts.equateStringsCaseInsensitive = equateStringsCaseInsensitive;
        /**
         * Compare the equality of two strings using a case-sensitive ordinal comparison.
         *
         * Case-sensitive comparisons compare both strings one code-point at a time using the
         * integer value of each code-point.
         */
        function equateStringsCaseSensitive(a, b) {
            return equateValues(a, b);
        }
        ts.equateStringsCaseSensitive = equateStringsCaseSensitive;
        function compareComparableValues(a, b) {
            return a === b ? 0 /* EqualTo */ :
                a === undefined ? -1 /* LessThan */ :
                    b === undefined ? 1 /* GreaterThan */ :
                        a < b ? -1 /* LessThan */ :
                            1 /* GreaterThan */;
        }
        /**
         * Compare two numeric values for their order relative to each other.
         * To compare strings, use any of the `compareStrings` functions.
         */
        function compareValues(a, b) {
            return compareComparableValues(a, b);
        }
        ts.compareValues = compareValues;
        /**
         * Compare two TextSpans, first by `start`, then by `length`.
         */
        function compareTextSpans(a, b) {
            return compareValues(a === null || a === void 0 ? void 0 : a.start, b === null || b === void 0 ? void 0 : b.start) || compareValues(a === null || a === void 0 ? void 0 : a.length, b === null || b === void 0 ? void 0 : b.length);
        }
        ts.compareTextSpans = compareTextSpans;
        function min(a, b, compare) {
            return compare(a, b) === -1 /* LessThan */ ? a : b;
        }
        ts.min = min;
        /**
         * Compare two strings using a case-insensitive ordinal comparison.
         *
         * Ordinal comparisons are based on the difference between the unicode code points of both
         * strings. Characters with multiple unicode representations are considered unequal. Ordinal
         * comparisons provide predictable ordering, but place "a" after "B".
         *
         * Case-insensitive comparisons compare both strings one code-point at a time using the integer
         * value of each code-point after applying `toUpperCase` to each string. We always map both
         * strings to their upper-case form as some unicode characters do not properly round-trip to
         * lowercase (such as `áºž` (German sharp capital s)).
         */
        function compareStringsCaseInsensitive(a, b) {
            if (a === b)
                return 0 /* EqualTo */;
            if (a === undefined)
                return -1 /* LessThan */;
            if (b === undefined)
                return 1 /* GreaterThan */;
            a = a.toUpperCase();
            b = b.toUpperCase();
            return a < b ? -1 /* LessThan */ : a > b ? 1 /* GreaterThan */ : 0 /* EqualTo */;
        }
        ts.compareStringsCaseInsensitive = compareStringsCaseInsensitive;
        /**
         * Compare two strings using a case-sensitive ordinal comparison.
         *
         * Ordinal comparisons are based on the difference between the unicode code points of both
         * strings. Characters with multiple unicode representations are considered unequal. Ordinal
         * comparisons provide predictable ordering, but place "a" after "B".
         *
         * Case-sensitive comparisons compare both strings one code-point at a time using the integer
         * value of each code-point.
         */
        function compareStringsCaseSensitive(a, b) {
            return compareComparableValues(a, b);
        }
        ts.compareStringsCaseSensitive = compareStringsCaseSensitive;
        function getStringComparer(ignoreCase) {
            return ignoreCase ? compareStringsCaseInsensitive : compareStringsCaseSensitive;
        }
        ts.getStringComparer = getStringComparer;
        /**
         * Creates a string comparer for use with string collation in the UI.
         */
        var createUIStringComparer = (function () {
            var defaultComparer;
            var enUSComparer;
            var stringComparerFactory = getStringComparerFactory();
            return createStringComparer;
            function compareWithCallback(a, b, comparer) {
                if (a === b)
                    return 0 /* EqualTo */;
                if (a === undefined)
                    return -1 /* LessThan */;
                if (b === undefined)
                    return 1 /* GreaterThan */;
                var value = comparer(a, b);
                return value < 0 ? -1 /* LessThan */ : value > 0 ? 1 /* GreaterThan */ : 0 /* EqualTo */;
            }
            function createIntlCollatorStringComparer(locale) {
                // Intl.Collator.prototype.compare is bound to the collator. See NOTE in
                // http://www.ecma-international.org/ecma-402/2.0/#sec-Intl.Collator.prototype.compare
                var comparer = new Intl.Collator(locale, { usage: "sort", sensitivity: "variant" }).compare;
                return function (a, b) { return compareWithCallback(a, b, comparer); };
            }
            function createLocaleCompareStringComparer(locale) {
                // if the locale is not the default locale (`undefined`), use the fallback comparer.
                if (locale !== undefined)
                    return createFallbackStringComparer();
                return function (a, b) { return compareWithCallback(a, b, compareStrings); };
                function compareStrings(a, b) {
                    return a.localeCompare(b);
                }
            }
            function createFallbackStringComparer() {
                // An ordinal comparison puts "A" after "b", but for the UI we want "A" before "b".
                // We first sort case insensitively.  So "Aaa" will come before "baa".
                // Then we sort case sensitively, so "aaa" will come before "Aaa".
                //
                // For case insensitive comparisons we always map both strings to their
                // upper-case form as some unicode characters do not properly round-trip to
                // lowercase (such as `áºž` (German sharp capital s)).
                return function (a, b) { return compareWithCallback(a, b, compareDictionaryOrder); };
                function compareDictionaryOrder(a, b) {
                    return compareStrings(a.toUpperCase(), b.toUpperCase()) || compareStrings(a, b);
                }
                function compareStrings(a, b) {
                    return a < b ? -1 /* LessThan */ : a > b ? 1 /* GreaterThan */ : 0 /* EqualTo */;
                }
            }
            function getStringComparerFactory() {
                // If the host supports Intl, we use it for comparisons using the default locale.
                if (typeof Intl === "object" && typeof Intl.Collator === "function") {
                    return createIntlCollatorStringComparer;
                }
                // If the host does not support Intl, we fall back to localeCompare.
                // localeCompare in Node v0.10 is just an ordinal comparison, so don't use it.
                if (typeof String.prototype.localeCompare === "function" &&
                    typeof String.prototype.toLocaleUpperCase === "function" &&
                    "a".localeCompare("B") < 0) {
                    return createLocaleCompareStringComparer;
                }
                // Otherwise, fall back to ordinal comparison:
                return createFallbackStringComparer;
            }
            function createStringComparer(locale) {
                // Hold onto common string comparers. This avoids constantly reallocating comparers during
                // tests.
                if (locale === undefined) {
                    return defaultComparer || (defaultComparer = stringComparerFactory(locale));
                }
                else if (locale === "en-US") {
                    return enUSComparer || (enUSComparer = stringComparerFactory(locale));
                }
                else {
                    return stringComparerFactory(locale);
                }
            }
        })();
        var uiComparerCaseSensitive;
        var uiLocale;
        function getUILocale() {
            return uiLocale;
        }
        ts.getUILocale = getUILocale;
        function setUILocale(value) {
            if (uiLocale !== value) {
                uiLocale = value;
                uiComparerCaseSensitive = undefined;
            }
        }
        ts.setUILocale = setUILocale;
        /**
         * Compare two strings in a using the case-sensitive sort behavior of the UI locale.
         *
         * Ordering is not predictable between different host locales, but is best for displaying
         * ordered data for UI presentation. Characters with multiple unicode representations may
         * be considered equal.
         *
         * Case-sensitive comparisons compare strings that differ in base characters, or
         * accents/diacritic marks, or case as unequal.
         */
        function compareStringsCaseSensitiveUI(a, b) {
            var comparer = uiComparerCaseSensitive || (uiComparerCaseSensitive = createUIStringComparer(uiLocale));
            return comparer(a, b);
        }
        ts.compareStringsCaseSensitiveUI = compareStringsCaseSensitiveUI;
        function compareProperties(a, b, key, comparer) {
            return a === b ? 0 /* EqualTo */ :
                a === undefined ? -1 /* LessThan */ :
                    b === undefined ? 1 /* GreaterThan */ :
                        comparer(a[key], b[key]);
        }
        ts.compareProperties = compareProperties;
        /** True is greater than false. */
        function compareBooleans(a, b) {
            return compareValues(a ? 1 : 0, b ? 1 : 0);
        }
        ts.compareBooleans = compareBooleans;
        /**
         * Given a name and a list of names that are *not* equal to the name, return a spelling suggestion if there is one that is close enough.
         * Names less than length 3 only check for case-insensitive equality, not Levenshtein distance.
         *
         * If there is a candidate that's the same except for case, return that.
         * If there is a candidate that's within one edit of the name, return that.
         * Otherwise, return the candidate with the smallest Levenshtein distance,
         *    except for candidates:
         *      * With no name
         *      * Whose length differs from the target name by more than 0.34 of the length of the name.
         *      * Whose levenshtein distance is more than 0.4 of the length of the name
         *        (0.4 allows 1 substitution/transposition for every 5 characters,
         *         and 1 insertion/deletion at 3 characters)
         */
        function getSpellingSuggestion(name, candidates, getName) {
            var maximumLengthDifference = Math.min(2, Math.floor(name.length * 0.34));
            var bestDistance = Math.floor(name.length * 0.4) + 1; // If the best result isn't better than this, don't bother.
            var bestCandidate;
            var justCheckExactMatches = false;
            var nameLowerCase = name.toLowerCase();
            for (var _i = 0, candidates_1 = candidates; _i < candidates_1.length; _i++) {
                var candidate = candidates_1[_i];
                var candidateName = getName(candidate);
                if (candidateName !== undefined && Math.abs(candidateName.length - nameLowerCase.length) <= maximumLengthDifference) {
                    var candidateNameLowerCase = candidateName.toLowerCase();
                    if (candidateNameLowerCase === nameLowerCase) {
                        if (candidateName === name) {
                            continue;
                        }
                        return candidate;
                    }
                    if (justCheckExactMatches) {
                        continue;
                    }
                    if (candidateName.length < 3) {
                        // Don't bother, user would have noticed a 2-character name having an extra character
                        continue;
                    }
                    // Only care about a result better than the best so far.
                    var distance = levenshteinWithMax(nameLowerCase, candidateNameLowerCase, bestDistance - 1);
                    if (distance === undefined) {
                        continue;
                    }
                    if (distance < 3) {
                        justCheckExactMatches = true;
                        bestCandidate = candidate;
                    }
                    else {
                        ts.Debug.assert(distance < bestDistance); // Else `levenshteinWithMax` should return undefined
                        bestDistance = distance;
                        bestCandidate = candidate;
                    }
                }
            }
            return bestCandidate;
        }
        ts.getSpellingSuggestion = getSpellingSuggestion;
        function levenshteinWithMax(s1, s2, max) {
            var previous = new Array(s2.length + 1);
            var current = new Array(s2.length + 1);
            /** Represents any value > max. We don't care about the particular value. */
            var big = max + 1;
            for (var i = 0; i <= s2.length; i++) {
                previous[i] = i;
            }
            for (var i = 1; i <= s1.length; i++) {
                var c1 = s1.charCodeAt(i - 1);
                var minJ = i > max ? i - max : 1;
                var maxJ = s2.length > max + i ? max + i : s2.length;
                current[0] = i;
                /** Smallest value of the matrix in the ith column. */
                var colMin = i;
                for (var j = 1; j < minJ; j++) {
                    current[j] = big;
                }
                for (var j = minJ; j <= maxJ; j++) {
                    var dist = c1 === s2.charCodeAt(j - 1)
                        ? previous[j - 1]
                        : Math.min(/*delete*/ previous[j] + 1, /*insert*/ current[j - 1] + 1, /*substitute*/ previous[j - 1] + 2);
                    current[j] = dist;
                    colMin = Math.min(colMin, dist);
                }
                for (var j = maxJ + 1; j <= s2.length; j++) {
                    current[j] = big;
                }
                if (colMin > max) {
                    // Give up -- everything in this column is > max and it can't get better in future columns.
                    return undefined;
                }
                var temp = previous;
                previous = current;
                current = temp;
            }
            var res = previous[s2.length];
            return res > max ? undefined : res;
        }
        function endsWith(str, suffix) {
            var expectedPos = str.length - suffix.length;
            return expectedPos >= 0 && str.indexOf(suffix, expectedPos) === expectedPos;
        }
        ts.endsWith = endsWith;
        function removeSuffix(str, suffix) {
            return endsWith(str, suffix) ? str.slice(0, str.length - suffix.length) : str;
        }
        ts.removeSuffix = removeSuffix;
        function tryRemoveSuffix(str, suffix) {
            return endsWith(str, suffix) ? str.slice(0, str.length - suffix.length) : undefined;
        }
        ts.tryRemoveSuffix = tryRemoveSuffix;
        function stringContains(str, substring) {
            return str.indexOf(substring) !== -1;
        }
        ts.stringContains = stringContains;
        /**
         * Takes a string like "jquery-min.4.2.3" and returns "jquery"
         */
        function removeMinAndVersionNumbers(fileName) {
            // Match a "." or "-" followed by a version number or 'min' at the end of the name
            var trailingMinOrVersion = /[.-]((min)|(\d+(\.\d+)*))$/;
            // The "min" or version may both be present, in either order, so try applying the above twice.
            return fileName.replace(trailingMinOrVersion, "").replace(trailingMinOrVersion, "");
        }
        ts.removeMinAndVersionNumbers = removeMinAndVersionNumbers;
        /** Remove an item from an array, moving everything to its right one space left. */
        function orderedRemoveItem(array, item) {
            for (var i = 0; i < array.length; i++) {
                if (array[i] === item) {
                    orderedRemoveItemAt(array, i);
                    return true;
                }
            }
            return false;
        }
        ts.orderedRemoveItem = orderedRemoveItem;
        /** Remove an item by index from an array, moving everything to its right one space left. */
        function orderedRemoveItemAt(array, index) {
            // This seems to be faster than either `array.splice(i, 1)` or `array.copyWithin(i, i+ 1)`.
            for (var i = index; i < array.length - 1; i++) {
                array[i] = array[i + 1];
            }
            array.pop();
        }
        ts.orderedRemoveItemAt = orderedRemoveItemAt;
        function unorderedRemoveItemAt(array, index) {
            // Fill in the "hole" left at `index`.
            array[index] = array[array.length - 1];
            array.pop();
        }
        ts.unorderedRemoveItemAt = unorderedRemoveItemAt;
        /** Remove the *first* occurrence of `item` from the array. */
        function unorderedRemoveItem(array, item) {
            return unorderedRemoveFirstItemWhere(array, function (element) { return element === item; });
        }
        ts.unorderedRemoveItem = unorderedRemoveItem;
        /** Remove the *first* element satisfying `predicate`. */
        function unorderedRemoveFirstItemWhere(array, predicate) {
            for (var i = 0; i < array.length; i++) {
                if (predicate(array[i])) {
                    unorderedRemoveItemAt(array, i);
                    return true;
                }
            }
            return false;
        }
        function createGetCanonicalFileName(useCaseSensitiveFileNames) {
            return useCaseSensitiveFileNames ? identity : toFileNameLowerCase;
        }
        ts.createGetCanonicalFileName = createGetCanonicalFileName;
        function patternText(_a) {
            var prefix = _a.prefix, suffix = _a.suffix;
            return prefix + "*" + suffix;
        }
        ts.patternText = patternText;
        /**
         * Given that candidate matches pattern, returns the text matching the '*'.
         * E.g.: matchedText(tryParsePattern("foo*baz"), "foobarbaz") === "bar"
         */
        function matchedText(pattern, candidate) {
            ts.Debug.assert(isPatternMatch(pattern, candidate));
            return candidate.substring(pattern.prefix.length, candidate.length - pattern.suffix.length);
        }
        ts.matchedText = matchedText;
        /** Return the object corresponding to the best pattern to match `candidate`. */
        function findBestPatternMatch(values, getPattern, candidate) {
            var matchedValue;
            // use length of prefix as betterness criteria
            var longestMatchPrefixLength = -1;
            for (var _i = 0, values_2 = values; _i < values_2.length; _i++) {
                var v = values_2[_i];
                var pattern = getPattern(v);
                if (isPatternMatch(pattern, candidate) && pattern.prefix.length > longestMatchPrefixLength) {
                    longestMatchPrefixLength = pattern.prefix.length;
                    matchedValue = v;
                }
            }
            return matchedValue;
        }
        ts.findBestPatternMatch = findBestPatternMatch;
        function startsWith(str, prefix) {
            return str.lastIndexOf(prefix, 0) === 0;
        }
        ts.startsWith = startsWith;
        function removePrefix(str, prefix) {
            return startsWith(str, prefix) ? str.substr(prefix.length) : str;
        }
        ts.removePrefix = removePrefix;
        function tryRemovePrefix(str, prefix, getCanonicalFileName) {
            if (getCanonicalFileName === void 0) {
                getCanonicalFileName = identity;
            }
            return startsWith(getCanonicalFileName(str), getCanonicalFileName(prefix)) ? str.substring(prefix.length) : undefined;
        }
        ts.tryRemovePrefix = tryRemovePrefix;
        function isPatternMatch(_a, candidate) {
            var prefix = _a.prefix, suffix = _a.suffix;
            return candidate.length >= prefix.length + suffix.length &&
                startsWith(candidate, prefix) &&
                endsWith(candidate, suffix);
        }
        function and(f, g) {
            return function (arg) { return f(arg) && g(arg); };
        }
        ts.and = and;
        function or() {
            var fs = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                fs[_i] = arguments[_i];
            }
            return function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                for (var _a = 0, fs_1 = fs; _a < fs_1.length; _a++) {
                    var f = fs_1[_a];
                    if (f.apply(void 0, args)) {
                        return true;
                    }
                }
                return false;
            };
        }
        ts.or = or;
        function not(fn) {
            return function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return !fn.apply(void 0, args);
            };
        }
        ts.not = not;
        function assertType(_) { }
        ts.assertType = assertType;
        function singleElementArray(t) {
            return t === undefined ? undefined : [t];
        }
        ts.singleElementArray = singleElementArray;
        function enumerateInsertsAndDeletes(newItems, oldItems, comparer, inserted, deleted, unchanged) {
            unchanged = unchanged || noop;
            var newIndex = 0;
            var oldIndex = 0;
            var newLen = newItems.length;
            var oldLen = oldItems.length;
            while (newIndex < newLen && oldIndex < oldLen) {
                var newItem = newItems[newIndex];
                var oldItem = oldItems[oldIndex];
                var compareResult = comparer(newItem, oldItem);
                if (compareResult === -1 /* LessThan */) {
                    inserted(newItem);
                    newIndex++;
                }
                else if (compareResult === 1 /* GreaterThan */) {
                    deleted(oldItem);
                    oldIndex++;
                }
                else {
                    unchanged(oldItem, newItem);
                    newIndex++;
                    oldIndex++;
                }
            }
            while (newIndex < newLen) {
                inserted(newItems[newIndex++]);
            }
            while (oldIndex < oldLen) {
                deleted(oldItems[oldIndex++]);
            }
        }
        ts.enumerateInsertsAndDeletes = enumerateInsertsAndDeletes;
        function fill(length, cb) {
            var result = Array(length);
            for (var i = 0; i < length; i++) {
                result[i] = cb(i);
            }
            return result;
        }
        ts.fill = fill;
        function cartesianProduct(arrays) {
            var result = [];
            cartesianProductWorker(arrays, result, /*outer*/ undefined, 0);
            return result;
        }
        ts.cartesianProduct = cartesianProduct;
        function cartesianProductWorker(arrays, result, outer, index) {
            for (var _i = 0, _a = arrays[index]; _i < _a.length; _i++) {
                var element = _a[_i];
                var inner = void 0;
                if (outer) {
                    inner = outer.slice();
                    inner.push(element);
                }
                else {
                    inner = [element];
                }
                if (index === arrays.length - 1) {
                    result.push(inner);
                }
                else {
                    cartesianProductWorker(arrays, result, inner, index + 1);
                }
            }
        }
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var Debug;
        (function (Debug) {
            /* eslint-disable prefer-const */
            Debug.currentAssertionLevel = 0 /* None */;
            Debug.isDebugging = false;
            /* eslint-enable prefer-const */
            function shouldAssert(level) {
                return Debug.currentAssertionLevel >= level;
            }
            Debug.shouldAssert = shouldAssert;
            function assert(expression, message, verboseDebugInfo, stackCrawlMark) {
                if (!expression) {
                    if (verboseDebugInfo) {
                        message += "\r\nVerbose Debug Information: " + (typeof verboseDebugInfo === "string" ? verboseDebugInfo : verboseDebugInfo());
                    }
                    fail(message ? "False expression: " + message : "False expression.", stackCrawlMark || assert);
                }
            }
            Debug.assert = assert;
            function assertEqual(a, b, msg, msg2) {
                if (a !== b) {
                    var message = msg ? msg2 ? msg + " " + msg2 : msg : "";
                    fail("Expected " + a + " === " + b + ". " + message);
                }
            }
            Debug.assertEqual = assertEqual;
            function assertLessThan(a, b, msg) {
                if (a >= b) {
                    fail("Expected " + a + " < " + b + ". " + (msg || ""));
                }
            }
            Debug.assertLessThan = assertLessThan;
            function assertLessThanOrEqual(a, b) {
                if (a > b) {
                    fail("Expected " + a + " <= " + b);
                }
            }
            Debug.assertLessThanOrEqual = assertLessThanOrEqual;
            function assertGreaterThanOrEqual(a, b) {
                if (a < b) {
                    fail("Expected " + a + " >= " + b);
                }
            }
            Debug.assertGreaterThanOrEqual = assertGreaterThanOrEqual;
            function fail(message, stackCrawlMark) {
                debugger;
                var e = new Error(message ? "Debug Failure. " + message : "Debug Failure.");
                if (Error.captureStackTrace) {
                    Error.captureStackTrace(e, stackCrawlMark || fail);
                }
                throw e;
            }
            Debug.fail = fail;
            function assertDefined(value, message) {
                // eslint-disable-next-line no-null/no-null
                if (value === undefined || value === null)
                    return fail(message);
                return value;
            }
            Debug.assertDefined = assertDefined;
            function assertEachDefined(value, message) {
                for (var _i = 0, value_1 = value; _i < value_1.length; _i++) {
                    var v = value_1[_i];
                    assertDefined(v, message);
                }
                return value;
            }
            Debug.assertEachDefined = assertEachDefined;
            function assertNever(member, message, stackCrawlMark) {
                if (message === void 0) {
                    message = "Illegal value:";
                }
                var detail = typeof member === "object" && ts.hasProperty(member, "kind") && ts.hasProperty(member, "pos") && formatSyntaxKind ? "SyntaxKind: " + formatSyntaxKind(member.kind) : JSON.stringify(member);
                return fail(message + " " + detail, stackCrawlMark || assertNever);
            }
            Debug.assertNever = assertNever;
            function getFunctionName(func) {
                if (typeof func !== "function") {
                    return "";
                }
                else if (func.hasOwnProperty("name")) {
                    return func.name;
                }
                else {
                    var text = Function.prototype.toString.call(func);
                    var match = /^function\s+([\w\$]+)\s*\(/.exec(text);
                    return match ? match[1] : "";
                }
            }
            Debug.getFunctionName = getFunctionName;
            function formatSymbol(symbol) {
                return "{ name: " + ts.unescapeLeadingUnderscores(symbol.escapedName) + "; flags: " + formatSymbolFlags(symbol.flags) + "; declarations: " + ts.map(symbol.declarations, function (node) { return formatSyntaxKind(node.kind); }) + " }";
            }
            Debug.formatSymbol = formatSymbol;
            /**
             * Formats an enum value as a string for debugging and debug assertions.
             */
            function formatEnum(value, enumObject, isFlags) {
                if (value === void 0) {
                    value = 0;
                }
                var members = getEnumMembers(enumObject);
                if (value === 0) {
                    return members.length > 0 && members[0][0] === 0 ? members[0][1] : "0";
                }
                if (isFlags) {
                    var result = "";
                    var remainingFlags = value;
                    for (var _i = 0, members_1 = members; _i < members_1.length; _i++) {
                        var _a = members_1[_i], enumValue = _a[0], enumName = _a[1];
                        if (enumValue > value) {
                            break;
                        }
                        if (enumValue !== 0 && enumValue & value) {
                            result = "" + result + (result ? "|" : "") + enumName;
                            remainingFlags &= ~enumValue;
                        }
                    }
                    if (remainingFlags === 0) {
                        return result;
                    }
                }
                else {
                    for (var _b = 0, members_2 = members; _b < members_2.length; _b++) {
                        var _c = members_2[_b], enumValue = _c[0], enumName = _c[1];
                        if (enumValue === value) {
                            return enumName;
                        }
                    }
                }
                return value.toString();
            }
            Debug.formatEnum = formatEnum;
            function getEnumMembers(enumObject) {
                var result = [];
                for (var name in enumObject) {
                    var value = enumObject[name];
                    if (typeof value === "number") {
                        result.push([value, name]);
                    }
                }
                return ts.stableSort(result, function (x, y) { return ts.compareValues(x[0], y[0]); });
            }
            function formatSyntaxKind(kind) {
                return formatEnum(kind, ts.SyntaxKind, /*isFlags*/ false);
            }
            Debug.formatSyntaxKind = formatSyntaxKind;
            function formatNodeFlags(flags) {
                return formatEnum(flags, ts.NodeFlags, /*isFlags*/ true);
            }
            Debug.formatNodeFlags = formatNodeFlags;
            function formatModifierFlags(flags) {
                return formatEnum(flags, ts.ModifierFlags, /*isFlags*/ true);
            }
            Debug.formatModifierFlags = formatModifierFlags;
            function formatTransformFlags(flags) {
                return formatEnum(flags, ts.TransformFlags, /*isFlags*/ true);
            }
            Debug.formatTransformFlags = formatTransformFlags;
            function formatEmitFlags(flags) {
                return formatEnum(flags, ts.EmitFlags, /*isFlags*/ true);
            }
            Debug.formatEmitFlags = formatEmitFlags;
            function formatSymbolFlags(flags) {
                return formatEnum(flags, ts.SymbolFlags, /*isFlags*/ true);
            }
            Debug.formatSymbolFlags = formatSymbolFlags;
            function formatTypeFlags(flags) {
                return formatEnum(flags, ts.TypeFlags, /*isFlags*/ true);
            }
            Debug.formatTypeFlags = formatTypeFlags;
            function formatObjectFlags(flags) {
                return formatEnum(flags, ts.ObjectFlags, /*isFlags*/ true);
            }
            Debug.formatObjectFlags = formatObjectFlags;
            function failBadSyntaxKind(node, message) {
                return fail((message || "Unexpected node.") + "\r\nNode " + formatSyntaxKind(node.kind) + " was unexpected.", failBadSyntaxKind);
            }
            Debug.failBadSyntaxKind = failBadSyntaxKind;
            Debug.assertEachNode = shouldAssert(1 /* Normal */)
                ? function (nodes, test, message) { return assert(test === undefined || ts.every(nodes, test), message || "Unexpected node.", function () { return "Node array did not pass test '" + getFunctionName(test) + "'."; }, Debug.assertEachNode); }
                : ts.noop;
            Debug.assertNode = shouldAssert(1 /* Normal */)
                ? function (node, test, message) { return assert(test === undefined || test(node), message || "Unexpected node.", function () { return "Node " + formatSyntaxKind(node.kind) + " did not pass test '" + getFunctionName(test) + "'."; }, Debug.assertNode); }
                : ts.noop;
            Debug.assertNotNode = shouldAssert(1 /* Normal */)
                ? function (node, test, message) { return assert(test === undefined || !test(node), message || "Unexpected node.", function () { return "Node " + formatSyntaxKind(node.kind) + " should not have passed test '" + getFunctionName(test) + "'."; }, Debug.assertNode); }
                : ts.noop;
            Debug.assertOptionalNode = shouldAssert(1 /* Normal */)
                ? function (node, test, message) { return assert(test === undefined || node === undefined || test(node), message || "Unexpected node.", function () { return "Node " + formatSyntaxKind(node.kind) + " did not pass test '" + getFunctionName(test) + "'."; }, Debug.assertOptionalNode); }
                : ts.noop;
            Debug.assertOptionalToken = shouldAssert(1 /* Normal */)
                ? function (node, kind, message) { return assert(kind === undefined || node === undefined || node.kind === kind, message || "Unexpected node.", function () { return "Node " + formatSyntaxKind(node.kind) + " was not a '" + formatSyntaxKind(kind) + "' token."; }, Debug.assertOptionalToken); }
                : ts.noop;
            Debug.assertMissingNode = shouldAssert(1 /* Normal */)
                ? function (node, message) { return assert(node === undefined, message || "Unexpected node.", function () { return "Node " + formatSyntaxKind(node.kind) + " was unexpected'."; }, Debug.assertMissingNode); }
                : ts.noop;
            var isDebugInfoEnabled = false;
            var extendedDebugModule;
            function extendedDebug() {
                enableDebugInfo();
                if (!extendedDebugModule) {
                    throw new Error("Debugging helpers could not be loaded.");
                }
                return extendedDebugModule;
            }
            function printControlFlowGraph(flowNode) {
                return console.log(formatControlFlowGraph(flowNode));
            }
            Debug.printControlFlowGraph = printControlFlowGraph;
            function formatControlFlowGraph(flowNode) {
                return extendedDebug().formatControlFlowGraph(flowNode);
            }
            Debug.formatControlFlowGraph = formatControlFlowGraph;
            function attachFlowNodeDebugInfo(flowNode) {
                if (isDebugInfoEnabled) {
                    if (!("__debugFlowFlags" in flowNode)) { // eslint-disable-line no-in-operator
                        Object.defineProperties(flowNode, {
                            __debugFlowFlags: { get: function () { return formatEnum(this.flags, ts.FlowFlags, /*isFlags*/ true); } },
                            __debugToString: { value: function () { return formatControlFlowGraph(this); } }
                        });
                    }
                }
            }
            Debug.attachFlowNodeDebugInfo = attachFlowNodeDebugInfo;
            /**
             * Injects debug information into frequently used types.
             */
            function enableDebugInfo() {
                if (isDebugInfoEnabled)
                    return;
                // Add additional properties in debug mode to assist with debugging.
                Object.defineProperties(ts.objectAllocator.getSymbolConstructor().prototype, {
                    __debugFlags: { get: function () { return formatSymbolFlags(this.flags); } }
                });
                Object.defineProperties(ts.objectAllocator.getTypeConstructor().prototype, {
                    __debugFlags: { get: function () { return formatTypeFlags(this.flags); } },
                    __debugObjectFlags: { get: function () { return this.flags & 524288 /* Object */ ? formatObjectFlags(this.objectFlags) : ""; } },
                    __debugTypeToString: { value: function () { return this.checker.typeToString(this); } },
                });
                var nodeConstructors = [
                    ts.objectAllocator.getNodeConstructor(),
                    ts.objectAllocator.getIdentifierConstructor(),
                    ts.objectAllocator.getTokenConstructor(),
                    ts.objectAllocator.getSourceFileConstructor()
                ];
                for (var _i = 0, nodeConstructors_1 = nodeConstructors; _i < nodeConstructors_1.length; _i++) {
                    var ctor = nodeConstructors_1[_i];
                    if (!ctor.prototype.hasOwnProperty("__debugKind")) {
                        Object.defineProperties(ctor.prototype, {
                            __debugKind: { get: function () { return formatSyntaxKind(this.kind); } },
                            __debugNodeFlags: { get: function () { return formatNodeFlags(this.flags); } },
                            __debugModifierFlags: { get: function () { return formatModifierFlags(ts.getModifierFlagsNoCache(this)); } },
                            __debugTransformFlags: { get: function () { return formatTransformFlags(this.transformFlags); } },
                            __debugIsParseTreeNode: { get: function () { return ts.isParseTreeNode(this); } },
                            __debugEmitFlags: { get: function () { return formatEmitFlags(ts.getEmitFlags(this)); } },
                            __debugGetText: {
                                value: function (includeTrivia) {
                                    if (ts.nodeIsSynthesized(this))
                                        return "";
                                    var parseNode = ts.getParseTreeNode(this);
                                    var sourceFile = parseNode && ts.getSourceFileOfNode(parseNode);
                                    return sourceFile ? ts.getSourceTextOfNodeFromSourceFile(sourceFile, parseNode, includeTrivia) : "";
                                }
                            }
                        });
                    }
                }
                // attempt to load extended debugging information
                try {
                    if (ts.sys && ts.sys.require) {
                        var basePath = ts.getDirectoryPath(ts.resolvePath(ts.sys.getExecutingFilePath()));
                        var result = ts.sys.require(basePath, "./compiler-debug");
                        if (!result.error) {
                            result.module.init(ts);
                            extendedDebugModule = result.module;
                        }
                    }
                }
                catch (_a) {
                    // do nothing
                }
                isDebugInfoEnabled = true;
            }
            Debug.enableDebugInfo = enableDebugInfo;
        })(Debug = ts.Debug || (ts.Debug = {}));
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        /** Gets a timestamp with (at least) ms resolution */
        ts.timestamp = typeof performance !== "undefined" && performance.now ? function () { return performance.now(); } : Date.now ? Date.now : function () { return +(new Date()); };
    })(ts || (ts = {}));
    /*@internal*/
    /** Performance measurements for the compiler. */
    var ts;
    (function (ts) {
        var performance;
        (function (performance) {
            // NOTE: cannot use ts.noop as core.ts loads after this
            var profilerEvent = typeof onProfilerEvent === "function" && onProfilerEvent.profiler === true ? onProfilerEvent : function () { };
            var enabled = false;
            var profilerStart = 0;
            var counts;
            var marks;
            var measures;
            function createTimerIf(condition, measureName, startMarkName, endMarkName) {
                return condition ? createTimer(measureName, startMarkName, endMarkName) : performance.nullTimer;
            }
            performance.createTimerIf = createTimerIf;
            function createTimer(measureName, startMarkName, endMarkName) {
                var enterCount = 0;
                return {
                    enter: enter,
                    exit: exit
                };
                function enter() {
                    if (++enterCount === 1) {
                        mark(startMarkName);
                    }
                }
                function exit() {
                    if (--enterCount === 0) {
                        mark(endMarkName);
                        measure(measureName, startMarkName, endMarkName);
                    }
                    else if (enterCount < 0) {
                        ts.Debug.fail("enter/exit count does not match.");
                    }
                }
            }
            performance.createTimer = createTimer;
            performance.nullTimer = { enter: ts.noop, exit: ts.noop };
            /**
             * Marks a performance event.
             *
             * @param markName The name of the mark.
             */
            function mark(markName) {
                if (enabled) {
                    marks.set(markName, ts.timestamp());
                    counts.set(markName, (counts.get(markName) || 0) + 1);
                    profilerEvent(markName);
                }
            }
            performance.mark = mark;
            /**
             * Adds a performance measurement with the specified name.
             *
             * @param measureName The name of the performance measurement.
             * @param startMarkName The name of the starting mark. If not supplied, the point at which the
             *      profiler was enabled is used.
             * @param endMarkName The name of the ending mark. If not supplied, the current timestamp is
             *      used.
             */
            function measure(measureName, startMarkName, endMarkName) {
                if (enabled) {
                    var end = endMarkName && marks.get(endMarkName) || ts.timestamp();
                    var start = startMarkName && marks.get(startMarkName) || profilerStart;
                    measures.set(measureName, (measures.get(measureName) || 0) + (end - start));
                }
            }
            performance.measure = measure;
            /**
             * Gets the number of times a marker was encountered.
             *
             * @param markName The name of the mark.
             */
            function getCount(markName) {
                return counts && counts.get(markName) || 0;
            }
            performance.getCount = getCount;
            /**
             * Gets the total duration of all measurements with the supplied name.
             *
             * @param measureName The name of the measure whose durations should be accumulated.
             */
            function getDuration(measureName) {
                return measures && measures.get(measureName) || 0;
            }
            performance.getDuration = getDuration;
            /**
             * Iterate over each measure, performing some action
             *
             * @param cb The action to perform for each measure
             */
            function forEachMeasure(cb) {
                measures.forEach(function (measure, key) {
                    cb(key, measure);
                });
            }
            performance.forEachMeasure = forEachMeasure;
            /** Enables (and resets) performance measurements for the compiler. */
            function enable() {
                counts = ts.createMap();
                marks = ts.createMap();
                measures = ts.createMap();
                enabled = true;
                profilerStart = ts.timestamp();
            }
            performance.enable = enable;
            /** Disables performance measurements for the compiler. */
            function disable() {
                enabled = false;
            }
            performance.disable = disable;
        })(performance = ts.performance || (ts.performance = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var nullLogger = {
            logEvent: ts.noop,
            logErrEvent: ts.noop,
            logPerfEvent: ts.noop,
            logInfoEvent: ts.noop,
            logStartCommand: ts.noop,
            logStopCommand: ts.noop,
            logStartUpdateProgram: ts.noop,
            logStopUpdateProgram: ts.noop,
            logStartUpdateGraph: ts.noop,
            logStopUpdateGraph: ts.noop,
            logStartResolveModule: ts.noop,
            logStopResolveModule: ts.noop,
            logStartParseSourceFile: ts.noop,
            logStopParseSourceFile: ts.noop,
            logStartReadFile: ts.noop,
            logStopReadFile: ts.noop,
            logStartBindFile: ts.noop,
            logStopBindFile: ts.noop,
            logStartScheduledOperation: ts.noop,
            logStopScheduledOperation: ts.noop,
        };
        // Load optional module to enable Event Tracing for Windows
        // See https://github.com/microsoft/typescript-etw for more information
        var etwModule;
        try {
            // require() will throw an exception if the module is not installed
            // It may also return undefined if not installed properly
            etwModule = require("@microsoft/typescript-etw");
        }
        catch (e) {
            etwModule = undefined;
        }
        /** Performance logger that will generate ETW events if possible - check for `logEvent` member, as `etwModule` will be `{}` when browserified */
        ts.perfLogger = etwModule && etwModule.logEvent ? etwModule : nullLogger;
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        // https://semver.org/#spec-item-2
        // > A normal version number MUST take the form X.Y.Z where X, Y, and Z are non-negative
        // > integers, and MUST NOT contain leading zeroes. X is the major version, Y is the minor
        // > version, and Z is the patch version. Each element MUST increase numerically.
        //
        // NOTE: We differ here in that we allow X and X.Y, with missing parts having the default
        // value of `0`.
        var versionRegExp = /^(0|[1-9]\d*)(?:\.(0|[1-9]\d*)(?:\.(0|[1-9]\d*)(?:\-([a-z0-9-.]+))?(?:\+([a-z0-9-.]+))?)?)?$/i;
        // https://semver.org/#spec-item-9
        // > A pre-release version MAY be denoted by appending a hyphen and a series of dot separated
        // > identifiers immediately following the patch version. Identifiers MUST comprise only ASCII
        // > alphanumerics and hyphen [0-9A-Za-z-]. Identifiers MUST NOT be empty. Numeric identifiers
        // > MUST NOT include leading zeroes.
        var prereleaseRegExp = /^(?:0|[1-9]\d*|[a-z-][a-z0-9-]*)(?:\.(?:0|[1-9]\d*|[a-z-][a-z0-9-]*))*$/i;
        // https://semver.org/#spec-item-10
        // > Build metadata MAY be denoted by appending a plus sign and a series of dot separated
        // > identifiers immediately following the patch or pre-release version. Identifiers MUST
        // > comprise only ASCII alphanumerics and hyphen [0-9A-Za-z-]. Identifiers MUST NOT be empty.
        var buildRegExp = /^[a-z0-9-]+(?:\.[a-z0-9-]+)*$/i;
        // https://semver.org/#spec-item-9
        // > Numeric identifiers MUST NOT include leading zeroes.
        var numericIdentifierRegExp = /^(0|[1-9]\d*)$/;
        /**
         * Describes a precise semantic version number, https://semver.org
         */
        var Version = /** @class */ (function () {
            function Version(major, minor, patch, prerelease, build) {
                if (minor === void 0) {
                    minor = 0;
                }
                if (patch === void 0) {
                    patch = 0;
                }
                if (prerelease === void 0) {
                    prerelease = "";
                }
                if (build === void 0) {
                    build = "";
                }
                if (typeof major === "string") {
                    var result = ts.Debug.assertDefined(tryParseComponents(major), "Invalid version");
                    (major = result.major, minor = result.minor, patch = result.patch, prerelease = result.prerelease, build = result.build);
                }
                ts.Debug.assert(major >= 0, "Invalid argument: major");
                ts.Debug.assert(minor >= 0, "Invalid argument: minor");
                ts.Debug.assert(patch >= 0, "Invalid argument: patch");
                ts.Debug.assert(!prerelease || prereleaseRegExp.test(prerelease), "Invalid argument: prerelease");
                ts.Debug.assert(!build || buildRegExp.test(build), "Invalid argument: build");
                this.major = major;
                this.minor = minor;
                this.patch = patch;
                this.prerelease = prerelease ? prerelease.split(".") : ts.emptyArray;
                this.build = build ? build.split(".") : ts.emptyArray;
            }
            Version.tryParse = function (text) {
                var result = tryParseComponents(text);
                if (!result)
                    return undefined;
                var major = result.major, minor = result.minor, patch = result.patch, prerelease = result.prerelease, build = result.build;
                return new Version(major, minor, patch, prerelease, build);
            };
            Version.prototype.compareTo = function (other) {
                // https://semver.org/#spec-item-11
                // > Precedence is determined by the first difference when comparing each of these
                // > identifiers from left to right as follows: Major, minor, and patch versions are
                // > always compared numerically.
                //
                // https://semver.org/#spec-item-11
                // > Precedence for two pre-release versions with the same major, minor, and patch version
                // > MUST be determined by comparing each dot separated identifier from left to right until
                // > a difference is found [...]
                //
                // https://semver.org/#spec-item-11
                // > Build metadata does not figure into precedence
                if (this === other)
                    return 0 /* EqualTo */;
                if (other === undefined)
                    return 1 /* GreaterThan */;
                return ts.compareValues(this.major, other.major)
                    || ts.compareValues(this.minor, other.minor)
                    || ts.compareValues(this.patch, other.patch)
                    || comparePrerelaseIdentifiers(this.prerelease, other.prerelease);
            };
            Version.prototype.increment = function (field) {
                switch (field) {
                    case "major": return new Version(this.major + 1, 0, 0);
                    case "minor": return new Version(this.major, this.minor + 1, 0);
                    case "patch": return new Version(this.major, this.minor, this.patch + 1);
                    default: return ts.Debug.assertNever(field);
                }
            };
            Version.prototype.toString = function () {
                var result = this.major + "." + this.minor + "." + this.patch;
                if (ts.some(this.prerelease))
                    result += "-" + this.prerelease.join(".");
                if (ts.some(this.build))
                    result += "+" + this.build.join(".");
                return result;
            };
            Version.zero = new Version(0, 0, 0);
            return Version;
        }());
        ts.Version = Version;
        function tryParseComponents(text) {
            var match = versionRegExp.exec(text);
            if (!match)
                return undefined;
            var major = match[1], _a = match[2], minor = _a === void 0 ? "0" : _a, _b = match[3], patch = _b === void 0 ? "0" : _b, _c = match[4], prerelease = _c === void 0 ? "" : _c, _d = match[5], build = _d === void 0 ? "" : _d;
            if (prerelease && !prereleaseRegExp.test(prerelease))
                return undefined;
            if (build && !buildRegExp.test(build))
                return undefined;
            return {
                major: parseInt(major, 10),
                minor: parseInt(minor, 10),
                patch: parseInt(patch, 10),
                prerelease: prerelease,
                build: build
            };
        }
        function comparePrerelaseIdentifiers(left, right) {
            // https://semver.org/#spec-item-11
            // > When major, minor, and patch are equal, a pre-release version has lower precedence
            // > than a normal version.
            if (left === right)
                return 0 /* EqualTo */;
            if (left.length === 0)
                return right.length === 0 ? 0 /* EqualTo */ : 1 /* GreaterThan */;
            if (right.length === 0)
                return -1 /* LessThan */;
            // https://semver.org/#spec-item-11
            // > Precedence for two pre-release versions with the same major, minor, and patch version
            // > MUST be determined by comparing each dot separated identifier from left to right until
            // > a difference is found [...]
            var length = Math.min(left.length, right.length);
            for (var i = 0; i < length; i++) {
                var leftIdentifier = left[i];
                var rightIdentifier = right[i];
                if (leftIdentifier === rightIdentifier)
                    continue;
                var leftIsNumeric = numericIdentifierRegExp.test(leftIdentifier);
                var rightIsNumeric = numericIdentifierRegExp.test(rightIdentifier);
                if (leftIsNumeric || rightIsNumeric) {
                    // https://semver.org/#spec-item-11
                    // > Numeric identifiers always have lower precedence than non-numeric identifiers.
                    if (leftIsNumeric !== rightIsNumeric)
                        return leftIsNumeric ? -1 /* LessThan */ : 1 /* GreaterThan */;
                    // https://semver.org/#spec-item-11
                    // > identifiers consisting of only digits are compared numerically
                    var result = ts.compareValues(+leftIdentifier, +rightIdentifier);
                    if (result)
                        return result;
                }
                else {
                    // https://semver.org/#spec-item-11
                    // > identifiers with letters or hyphens are compared lexically in ASCII sort order.
                    var result = ts.compareStringsCaseSensitive(leftIdentifier, rightIdentifier);
                    if (result)
                        return result;
                }
            }
            // https://semver.org/#spec-item-11
            // > A larger set of pre-release fields has a higher precedence than a smaller set, if all
            // > of the preceding identifiers are equal.
            return ts.compareValues(left.length, right.length);
        }
        /**
         * Describes a semantic version range, per https://github.com/npm/node-semver#ranges
         */
        var VersionRange = /** @class */ (function () {
            function VersionRange(spec) {
                this._alternatives = spec ? ts.Debug.assertDefined(parseRange(spec), "Invalid range spec.") : ts.emptyArray;
            }
            VersionRange.tryParse = function (text) {
                var sets = parseRange(text);
                if (sets) {
                    var range = new VersionRange("");
                    range._alternatives = sets;
                    return range;
                }
                return undefined;
            };
            VersionRange.prototype.test = function (version) {
                if (typeof version === "string")
                    version = new Version(version);
                return testDisjunction(version, this._alternatives);
            };
            VersionRange.prototype.toString = function () {
                return formatDisjunction(this._alternatives);
            };
            return VersionRange;
        }());
        ts.VersionRange = VersionRange;
        // https://github.com/npm/node-semver#range-grammar
        //
        // range-set    ::= range ( logical-or range ) *
        // range        ::= hyphen | simple ( ' ' simple ) * | ''
        // logical-or   ::= ( ' ' ) * '||' ( ' ' ) *
        var logicalOrRegExp = /\s*\|\|\s*/g;
        var whitespaceRegExp = /\s+/g;
        // https://github.com/npm/node-semver#range-grammar
        //
        // partial      ::= xr ( '.' xr ( '.' xr qualifier ? )? )?
        // xr           ::= 'x' | 'X' | '*' | nr
        // nr           ::= '0' | ['1'-'9'] ( ['0'-'9'] ) *
        // qualifier    ::= ( '-' pre )? ( '+' build )?
        // pre          ::= parts
        // build        ::= parts
        // parts        ::= part ( '.' part ) *
        // part         ::= nr | [-0-9A-Za-z]+
        var partialRegExp = /^([xX*0]|[1-9]\d*)(?:\.([xX*0]|[1-9]\d*)(?:\.([xX*0]|[1-9]\d*)(?:-([a-z0-9-.]+))?(?:\+([a-z0-9-.]+))?)?)?$/i;
        // https://github.com/npm/node-semver#range-grammar
        //
        // hyphen       ::= partial ' - ' partial
        var hyphenRegExp = /^\s*([a-z0-9-+.*]+)\s+-\s+([a-z0-9-+.*]+)\s*$/i;
        // https://github.com/npm/node-semver#range-grammar
        //
        // simple       ::= primitive | partial | tilde | caret
        // primitive    ::= ( '<' | '>' | '>=' | '<=' | '=' ) partial
        // tilde        ::= '~' partial
        // caret        ::= '^' partial
        var rangeRegExp = /^\s*(~|\^|<|<=|>|>=|=)?\s*([a-z0-9-+.*]+)$/i;
        function parseRange(text) {
            var alternatives = [];
            for (var _i = 0, _a = text.trim().split(logicalOrRegExp); _i < _a.length; _i++) {
                var range = _a[_i];
                if (!range)
                    continue;
                var comparators = [];
                var match = hyphenRegExp.exec(range);
                if (match) {
                    if (!parseHyphen(match[1], match[2], comparators))
                        return undefined;
                }
                else {
                    for (var _b = 0, _c = range.split(whitespaceRegExp); _b < _c.length; _b++) {
                        var simple = _c[_b];
                        var match_1 = rangeRegExp.exec(simple);
                        if (!match_1 || !parseComparator(match_1[1], match_1[2], comparators))
                            return undefined;
                    }
                }
                alternatives.push(comparators);
            }
            return alternatives;
        }
        function parsePartial(text) {
            var match = partialRegExp.exec(text);
            if (!match)
                return undefined;
            var major = match[1], _a = match[2], minor = _a === void 0 ? "*" : _a, _b = match[3], patch = _b === void 0 ? "*" : _b, prerelease = match[4], build = match[5];
            var version = new Version(isWildcard(major) ? 0 : parseInt(major, 10), isWildcard(major) || isWildcard(minor) ? 0 : parseInt(minor, 10), isWildcard(major) || isWildcard(minor) || isWildcard(patch) ? 0 : parseInt(patch, 10), prerelease, build);
            return { version: version, major: major, minor: minor, patch: patch };
        }
        function parseHyphen(left, right, comparators) {
            var leftResult = parsePartial(left);
            if (!leftResult)
                return false;
            var rightResult = parsePartial(right);
            if (!rightResult)
                return false;
            if (!isWildcard(leftResult.major)) {
                comparators.push(createComparator(">=", leftResult.version));
            }
            if (!isWildcard(rightResult.major)) {
                comparators.push(isWildcard(rightResult.minor) ? createComparator("<", rightResult.version.increment("major")) :
                    isWildcard(rightResult.patch) ? createComparator("<", rightResult.version.increment("minor")) :
                        createComparator("<=", rightResult.version));
            }
            return true;
        }
        function parseComparator(operator, text, comparators) {
            var result = parsePartial(text);
            if (!result)
                return false;
            var version = result.version, major = result.major, minor = result.minor, patch = result.patch;
            if (!isWildcard(major)) {
                switch (operator) {
                    case "~":
                        comparators.push(createComparator(">=", version));
                        comparators.push(createComparator("<", version.increment(isWildcard(minor) ? "major" :
                            "minor")));
                        break;
                    case "^":
                        comparators.push(createComparator(">=", version));
                        comparators.push(createComparator("<", version.increment(version.major > 0 || isWildcard(minor) ? "major" :
                            version.minor > 0 || isWildcard(patch) ? "minor" :
                                "patch")));
                        break;
                    case "<":
                    case ">=":
                        comparators.push(createComparator(operator, version));
                        break;
                    case "<=":
                    case ">":
                        comparators.push(isWildcard(minor) ? createComparator(operator === "<=" ? "<" : ">=", version.increment("major")) :
                            isWildcard(patch) ? createComparator(operator === "<=" ? "<" : ">=", version.increment("minor")) :
                                createComparator(operator, version));
                        break;
                    case "=":
                    case undefined:
                        if (isWildcard(minor) || isWildcard(patch)) {
                            comparators.push(createComparator(">=", version));
                            comparators.push(createComparator("<", version.increment(isWildcard(minor) ? "major" : "minor")));
                        }
                        else {
                            comparators.push(createComparator("=", version));
                        }
                        break;
                    default:
                        // unrecognized
                        return false;
                }
            }
            else if (operator === "<" || operator === ">") {
                comparators.push(createComparator("<", Version.zero));
            }
            return true;
        }
        function isWildcard(part) {
            return part === "*" || part === "x" || part === "X";
        }
        function createComparator(operator, operand) {
            return { operator: operator, operand: operand };
        }
        function testDisjunction(version, alternatives) {
            // an empty disjunction is treated as "*" (all versions)
            if (alternatives.length === 0)
                return true;
            for (var _i = 0, alternatives_1 = alternatives; _i < alternatives_1.length; _i++) {
                var alternative = alternatives_1[_i];
                if (testAlternative(version, alternative))
                    return true;
            }
            return false;
        }
        function testAlternative(version, comparators) {
            for (var _i = 0, comparators_1 = comparators; _i < comparators_1.length; _i++) {
                var comparator = comparators_1[_i];
                if (!testComparator(version, comparator.operator, comparator.operand))
                    return false;
            }
            return true;
        }
        function testComparator(version, operator, operand) {
            var cmp = version.compareTo(operand);
            switch (operator) {
                case "<": return cmp < 0;
                case "<=": return cmp <= 0;
                case ">": return cmp > 0;
                case ">=": return cmp >= 0;
                case "=": return cmp === 0;
                default: return ts.Debug.assertNever(operator);
            }
        }
        function formatDisjunction(alternatives) {
            return ts.map(alternatives, formatAlternative).join(" || ") || "*";
        }
        function formatAlternative(comparators) {
            return ts.map(comparators, formatComparator).join(" ");
        }
        function formatComparator(comparator) {
            return "" + comparator.operator + comparator.operand;
        }
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        // token > SyntaxKind.Identifier => token is a keyword
        // Also, If you add a new SyntaxKind be sure to keep the `Markers` section at the bottom in sync
        var SyntaxKind;
        (function (SyntaxKind) {
            SyntaxKind[SyntaxKind["Unknown"] = 0] = "Unknown";
            SyntaxKind[SyntaxKind["EndOfFileToken"] = 1] = "EndOfFileToken";
            SyntaxKind[SyntaxKind["SingleLineCommentTrivia"] = 2] = "SingleLineCommentTrivia";
            SyntaxKind[SyntaxKind["MultiLineCommentTrivia"] = 3] = "MultiLineCommentTrivia";
            SyntaxKind[SyntaxKind["NewLineTrivia"] = 4] = "NewLineTrivia";
            SyntaxKind[SyntaxKind["WhitespaceTrivia"] = 5] = "WhitespaceTrivia";
            // We detect and preserve #! on the first line
            SyntaxKind[SyntaxKind["ShebangTrivia"] = 6] = "ShebangTrivia";
            // We detect and provide better error recovery when we encounter a git merge marker.  This
            // allows us to edit files with git-conflict markers in them in a much more pleasant manner.
            SyntaxKind[SyntaxKind["ConflictMarkerTrivia"] = 7] = "ConflictMarkerTrivia";
            // Literals
            SyntaxKind[SyntaxKind["NumericLiteral"] = 8] = "NumericLiteral";
            SyntaxKind[SyntaxKind["BigIntLiteral"] = 9] = "BigIntLiteral";
            SyntaxKind[SyntaxKind["StringLiteral"] = 10] = "StringLiteral";
            SyntaxKind[SyntaxKind["JsxText"] = 11] = "JsxText";
            SyntaxKind[SyntaxKind["JsxTextAllWhiteSpaces"] = 12] = "JsxTextAllWhiteSpaces";
            SyntaxKind[SyntaxKind["RegularExpressionLiteral"] = 13] = "RegularExpressionLiteral";
            SyntaxKind[SyntaxKind["NoSubstitutionTemplateLiteral"] = 14] = "NoSubstitutionTemplateLiteral";
            // Pseudo-literals
            SyntaxKind[SyntaxKind["TemplateHead"] = 15] = "TemplateHead";
            SyntaxKind[SyntaxKind["TemplateMiddle"] = 16] = "TemplateMiddle";
            SyntaxKind[SyntaxKind["TemplateTail"] = 17] = "TemplateTail";
            // Punctuation
            SyntaxKind[SyntaxKind["OpenBraceToken"] = 18] = "OpenBraceToken";
            SyntaxKind[SyntaxKind["CloseBraceToken"] = 19] = "CloseBraceToken";
            SyntaxKind[SyntaxKind["OpenParenToken"] = 20] = "OpenParenToken";
            SyntaxKind[SyntaxKind["CloseParenToken"] = 21] = "CloseParenToken";
            SyntaxKind[SyntaxKind["OpenBracketToken"] = 22] = "OpenBracketToken";
            SyntaxKind[SyntaxKind["CloseBracketToken"] = 23] = "CloseBracketToken";
            SyntaxKind[SyntaxKind["DotToken"] = 24] = "DotToken";
            SyntaxKind[SyntaxKind["DotDotDotToken"] = 25] = "DotDotDotToken";
            SyntaxKind[SyntaxKind["SemicolonToken"] = 26] = "SemicolonToken";
            SyntaxKind[SyntaxKind["CommaToken"] = 27] = "CommaToken";
            SyntaxKind[SyntaxKind["QuestionDotToken"] = 28] = "QuestionDotToken";
            SyntaxKind[SyntaxKind["LessThanToken"] = 29] = "LessThanToken";
            SyntaxKind[SyntaxKind["LessThanSlashToken"] = 30] = "LessThanSlashToken";
            SyntaxKind[SyntaxKind["GreaterThanToken"] = 31] = "GreaterThanToken";
            SyntaxKind[SyntaxKind["LessThanEqualsToken"] = 32] = "LessThanEqualsToken";
            SyntaxKind[SyntaxKind["GreaterThanEqualsToken"] = 33] = "GreaterThanEqualsToken";
            SyntaxKind[SyntaxKind["EqualsEqualsToken"] = 34] = "EqualsEqualsToken";
            SyntaxKind[SyntaxKind["ExclamationEqualsToken"] = 35] = "ExclamationEqualsToken";
            SyntaxKind[SyntaxKind["EqualsEqualsEqualsToken"] = 36] = "EqualsEqualsEqualsToken";
            SyntaxKind[SyntaxKind["ExclamationEqualsEqualsToken"] = 37] = "ExclamationEqualsEqualsToken";
            SyntaxKind[SyntaxKind["EqualsGreaterThanToken"] = 38] = "EqualsGreaterThanToken";
            SyntaxKind[SyntaxKind["PlusToken"] = 39] = "PlusToken";
            SyntaxKind[SyntaxKind["MinusToken"] = 40] = "MinusToken";
            SyntaxKind[SyntaxKind["AsteriskToken"] = 41] = "AsteriskToken";
            SyntaxKind[SyntaxKind["AsteriskAsteriskToken"] = 42] = "AsteriskAsteriskToken";
            SyntaxKind[SyntaxKind["SlashToken"] = 43] = "SlashToken";
            SyntaxKind[SyntaxKind["PercentToken"] = 44] = "PercentToken";
            SyntaxKind[SyntaxKind["PlusPlusToken"] = 45] = "PlusPlusToken";
            SyntaxKind[SyntaxKind["MinusMinusToken"] = 46] = "MinusMinusToken";
            SyntaxKind[SyntaxKind["LessThanLessThanToken"] = 47] = "LessThanLessThanToken";
            SyntaxKind[SyntaxKind["GreaterThanGreaterThanToken"] = 48] = "GreaterThanGreaterThanToken";
            SyntaxKind[SyntaxKind["GreaterThanGreaterThanGreaterThanToken"] = 49] = "GreaterThanGreaterThanGreaterThanToken";
            SyntaxKind[SyntaxKind["AmpersandToken"] = 50] = "AmpersandToken";
            SyntaxKind[SyntaxKind["BarToken"] = 51] = "BarToken";
            SyntaxKind[SyntaxKind["CaretToken"] = 52] = "CaretToken";
            SyntaxKind[SyntaxKind["ExclamationToken"] = 53] = "ExclamationToken";
            SyntaxKind[SyntaxKind["TildeToken"] = 54] = "TildeToken";
            SyntaxKind[SyntaxKind["AmpersandAmpersandToken"] = 55] = "AmpersandAmpersandToken";
            SyntaxKind[SyntaxKind["BarBarToken"] = 56] = "BarBarToken";
            SyntaxKind[SyntaxKind["QuestionToken"] = 57] = "QuestionToken";
            SyntaxKind[SyntaxKind["ColonToken"] = 58] = "ColonToken";
            SyntaxKind[SyntaxKind["AtToken"] = 59] = "AtToken";
            SyntaxKind[SyntaxKind["QuestionQuestionToken"] = 60] = "QuestionQuestionToken";
            /** Only the JSDoc scanner produces BacktickToken. The normal scanner produces NoSubstitutionTemplateLiteral and related kinds. */
            SyntaxKind[SyntaxKind["BacktickToken"] = 61] = "BacktickToken";
            // Assignments
            SyntaxKind[SyntaxKind["EqualsToken"] = 62] = "EqualsToken";
            SyntaxKind[SyntaxKind["PlusEqualsToken"] = 63] = "PlusEqualsToken";
            SyntaxKind[SyntaxKind["MinusEqualsToken"] = 64] = "MinusEqualsToken";
            SyntaxKind[SyntaxKind["AsteriskEqualsToken"] = 65] = "AsteriskEqualsToken";
            SyntaxKind[SyntaxKind["AsteriskAsteriskEqualsToken"] = 66] = "AsteriskAsteriskEqualsToken";
            SyntaxKind[SyntaxKind["SlashEqualsToken"] = 67] = "SlashEqualsToken";
            SyntaxKind[SyntaxKind["PercentEqualsToken"] = 68] = "PercentEqualsToken";
            SyntaxKind[SyntaxKind["LessThanLessThanEqualsToken"] = 69] = "LessThanLessThanEqualsToken";
            SyntaxKind[SyntaxKind["GreaterThanGreaterThanEqualsToken"] = 70] = "GreaterThanGreaterThanEqualsToken";
            SyntaxKind[SyntaxKind["GreaterThanGreaterThanGreaterThanEqualsToken"] = 71] = "GreaterThanGreaterThanGreaterThanEqualsToken";
            SyntaxKind[SyntaxKind["AmpersandEqualsToken"] = 72] = "AmpersandEqualsToken";
            SyntaxKind[SyntaxKind["BarEqualsToken"] = 73] = "BarEqualsToken";
            SyntaxKind[SyntaxKind["CaretEqualsToken"] = 74] = "CaretEqualsToken";
            // Identifiers and PrivateIdentifiers
            SyntaxKind[SyntaxKind["Identifier"] = 75] = "Identifier";
            SyntaxKind[SyntaxKind["PrivateIdentifier"] = 76] = "PrivateIdentifier";
            // Reserved words
            SyntaxKind[SyntaxKind["BreakKeyword"] = 77] = "BreakKeyword";
            SyntaxKind[SyntaxKind["CaseKeyword"] = 78] = "CaseKeyword";
            SyntaxKind[SyntaxKind["CatchKeyword"] = 79] = "CatchKeyword";
            SyntaxKind[SyntaxKind["ClassKeyword"] = 80] = "ClassKeyword";
            SyntaxKind[SyntaxKind["ConstKeyword"] = 81] = "ConstKeyword";
            SyntaxKind[SyntaxKind["ContinueKeyword"] = 82] = "ContinueKeyword";
            SyntaxKind[SyntaxKind["DebuggerKeyword"] = 83] = "DebuggerKeyword";
            SyntaxKind[SyntaxKind["DefaultKeyword"] = 84] = "DefaultKeyword";
            SyntaxKind[SyntaxKind["DeleteKeyword"] = 85] = "DeleteKeyword";
            SyntaxKind[SyntaxKind["DoKeyword"] = 86] = "DoKeyword";
            SyntaxKind[SyntaxKind["ElseKeyword"] = 87] = "ElseKeyword";
            SyntaxKind[SyntaxKind["EnumKeyword"] = 88] = "EnumKeyword";
            SyntaxKind[SyntaxKind["ExportKeyword"] = 89] = "ExportKeyword";
            SyntaxKind[SyntaxKind["ExtendsKeyword"] = 90] = "ExtendsKeyword";
            SyntaxKind[SyntaxKind["FalseKeyword"] = 91] = "FalseKeyword";
            SyntaxKind[SyntaxKind["FinallyKeyword"] = 92] = "FinallyKeyword";
            SyntaxKind[SyntaxKind["ForKeyword"] = 93] = "ForKeyword";
            SyntaxKind[SyntaxKind["FunctionKeyword"] = 94] = "FunctionKeyword";
            SyntaxKind[SyntaxKind["IfKeyword"] = 95] = "IfKeyword";
            SyntaxKind[SyntaxKind["ImportKeyword"] = 96] = "ImportKeyword";
            SyntaxKind[SyntaxKind["InKeyword"] = 97] = "InKeyword";
            SyntaxKind[SyntaxKind["InstanceOfKeyword"] = 98] = "InstanceOfKeyword";
            SyntaxKind[SyntaxKind["NewKeyword"] = 99] = "NewKeyword";
            SyntaxKind[SyntaxKind["NullKeyword"] = 100] = "NullKeyword";
            SyntaxKind[SyntaxKind["ReturnKeyword"] = 101] = "ReturnKeyword";
            SyntaxKind[SyntaxKind["SuperKeyword"] = 102] = "SuperKeyword";
            SyntaxKind[SyntaxKind["SwitchKeyword"] = 103] = "SwitchKeyword";
            SyntaxKind[SyntaxKind["ThisKeyword"] = 104] = "ThisKeyword";
            SyntaxKind[SyntaxKind["ThrowKeyword"] = 105] = "ThrowKeyword";
            SyntaxKind[SyntaxKind["TrueKeyword"] = 106] = "TrueKeyword";
            SyntaxKind[SyntaxKind["TryKeyword"] = 107] = "TryKeyword";
            SyntaxKind[SyntaxKind["TypeOfKeyword"] = 108] = "TypeOfKeyword";
            SyntaxKind[SyntaxKind["VarKeyword"] = 109] = "VarKeyword";
            SyntaxKind[SyntaxKind["VoidKeyword"] = 110] = "VoidKeyword";
            SyntaxKind[SyntaxKind["WhileKeyword"] = 111] = "WhileKeyword";
            SyntaxKind[SyntaxKind["WithKeyword"] = 112] = "WithKeyword";
            // Strict mode reserved words
            SyntaxKind[SyntaxKind["ImplementsKeyword"] = 113] = "ImplementsKeyword";
            SyntaxKind[SyntaxKind["InterfaceKeyword"] = 114] = "InterfaceKeyword";
            SyntaxKind[SyntaxKind["LetKeyword"] = 115] = "LetKeyword";
            SyntaxKind[SyntaxKind["PackageKeyword"] = 116] = "PackageKeyword";
            SyntaxKind[SyntaxKind["PrivateKeyword"] = 117] = "PrivateKeyword";
            SyntaxKind[SyntaxKind["ProtectedKeyword"] = 118] = "ProtectedKeyword";
            SyntaxKind[SyntaxKind["PublicKeyword"] = 119] = "PublicKeyword";
            SyntaxKind[SyntaxKind["StaticKeyword"] = 120] = "StaticKeyword";
            SyntaxKind[SyntaxKind["YieldKeyword"] = 121] = "YieldKeyword";
            // Contextual keywords
            SyntaxKind[SyntaxKind["AbstractKeyword"] = 122] = "AbstractKeyword";
            SyntaxKind[SyntaxKind["AsKeyword"] = 123] = "AsKeyword";
            SyntaxKind[SyntaxKind["AssertsKeyword"] = 124] = "AssertsKeyword";
            SyntaxKind[SyntaxKind["AnyKeyword"] = 125] = "AnyKeyword";
            SyntaxKind[SyntaxKind["AsyncKeyword"] = 126] = "AsyncKeyword";
            SyntaxKind[SyntaxKind["AwaitKeyword"] = 127] = "AwaitKeyword";
            SyntaxKind[SyntaxKind["BooleanKeyword"] = 128] = "BooleanKeyword";
            SyntaxKind[SyntaxKind["ConstructorKeyword"] = 129] = "ConstructorKeyword";
            SyntaxKind[SyntaxKind["DeclareKeyword"] = 130] = "DeclareKeyword";
            SyntaxKind[SyntaxKind["GetKeyword"] = 131] = "GetKeyword";
            SyntaxKind[SyntaxKind["InferKeyword"] = 132] = "InferKeyword";
            SyntaxKind[SyntaxKind["IsKeyword"] = 133] = "IsKeyword";
            SyntaxKind[SyntaxKind["KeyOfKeyword"] = 134] = "KeyOfKeyword";
            SyntaxKind[SyntaxKind["ModuleKeyword"] = 135] = "ModuleKeyword";
            SyntaxKind[SyntaxKind["NamespaceKeyword"] = 136] = "NamespaceKeyword";
            SyntaxKind[SyntaxKind["NeverKeyword"] = 137] = "NeverKeyword";
            SyntaxKind[SyntaxKind["ReadonlyKeyword"] = 138] = "ReadonlyKeyword";
            SyntaxKind[SyntaxKind["RequireKeyword"] = 139] = "RequireKeyword";
            SyntaxKind[SyntaxKind["NumberKeyword"] = 140] = "NumberKeyword";
            SyntaxKind[SyntaxKind["ObjectKeyword"] = 141] = "ObjectKeyword";
            SyntaxKind[SyntaxKind["SetKeyword"] = 142] = "SetKeyword";
            SyntaxKind[SyntaxKind["StringKeyword"] = 143] = "StringKeyword";
            SyntaxKind[SyntaxKind["SymbolKeyword"] = 144] = "SymbolKeyword";
            SyntaxKind[SyntaxKind["TypeKeyword"] = 145] = "TypeKeyword";
            SyntaxKind[SyntaxKind["UndefinedKeyword"] = 146] = "UndefinedKeyword";
            SyntaxKind[SyntaxKind["UniqueKeyword"] = 147] = "UniqueKeyword";
            SyntaxKind[SyntaxKind["UnknownKeyword"] = 148] = "UnknownKeyword";
            SyntaxKind[SyntaxKind["FromKeyword"] = 149] = "FromKeyword";
            SyntaxKind[SyntaxKind["GlobalKeyword"] = 150] = "GlobalKeyword";
            SyntaxKind[SyntaxKind["BigIntKeyword"] = 151] = "BigIntKeyword";
            SyntaxKind[SyntaxKind["OfKeyword"] = 152] = "OfKeyword";
            // Parse tree nodes
            // Names
            SyntaxKind[SyntaxKind["QualifiedName"] = 153] = "QualifiedName";
            SyntaxKind[SyntaxKind["ComputedPropertyName"] = 154] = "ComputedPropertyName";
            // Signature elements
            SyntaxKind[SyntaxKind["TypeParameter"] = 155] = "TypeParameter";
            SyntaxKind[SyntaxKind["Parameter"] = 156] = "Parameter";
            SyntaxKind[SyntaxKind["Decorator"] = 157] = "Decorator";
            // TypeMember
            SyntaxKind[SyntaxKind["PropertySignature"] = 158] = "PropertySignature";
            SyntaxKind[SyntaxKind["PropertyDeclaration"] = 159] = "PropertyDeclaration";
            SyntaxKind[SyntaxKind["MethodSignature"] = 160] = "MethodSignature";
            SyntaxKind[SyntaxKind["MethodDeclaration"] = 161] = "MethodDeclaration";
            SyntaxKind[SyntaxKind["Constructor"] = 162] = "Constructor";
            SyntaxKind[SyntaxKind["GetAccessor"] = 163] = "GetAccessor";
            SyntaxKind[SyntaxKind["SetAccessor"] = 164] = "SetAccessor";
            SyntaxKind[SyntaxKind["CallSignature"] = 165] = "CallSignature";
            SyntaxKind[SyntaxKind["ConstructSignature"] = 166] = "ConstructSignature";
            SyntaxKind[SyntaxKind["IndexSignature"] = 167] = "IndexSignature";
            // Type
            SyntaxKind[SyntaxKind["TypePredicate"] = 168] = "TypePredicate";
            SyntaxKind[SyntaxKind["TypeReference"] = 169] = "TypeReference";
            SyntaxKind[SyntaxKind["FunctionType"] = 170] = "FunctionType";
            SyntaxKind[SyntaxKind["ConstructorType"] = 171] = "ConstructorType";
            SyntaxKind[SyntaxKind["TypeQuery"] = 172] = "TypeQuery";
            SyntaxKind[SyntaxKind["TypeLiteral"] = 173] = "TypeLiteral";
            SyntaxKind[SyntaxKind["ArrayType"] = 174] = "ArrayType";
            SyntaxKind[SyntaxKind["TupleType"] = 175] = "TupleType";
            SyntaxKind[SyntaxKind["OptionalType"] = 176] = "OptionalType";
            SyntaxKind[SyntaxKind["RestType"] = 177] = "RestType";
            SyntaxKind[SyntaxKind["UnionType"] = 178] = "UnionType";
            SyntaxKind[SyntaxKind["IntersectionType"] = 179] = "IntersectionType";
            SyntaxKind[SyntaxKind["ConditionalType"] = 180] = "ConditionalType";
            SyntaxKind[SyntaxKind["InferType"] = 181] = "InferType";
            SyntaxKind[SyntaxKind["ParenthesizedType"] = 182] = "ParenthesizedType";
            SyntaxKind[SyntaxKind["ThisType"] = 183] = "ThisType";
            SyntaxKind[SyntaxKind["TypeOperator"] = 184] = "TypeOperator";
            SyntaxKind[SyntaxKind["IndexedAccessType"] = 185] = "IndexedAccessType";
            SyntaxKind[SyntaxKind["MappedType"] = 186] = "MappedType";
            SyntaxKind[SyntaxKind["LiteralType"] = 187] = "LiteralType";
            SyntaxKind[SyntaxKind["ImportType"] = 188] = "ImportType";
            // Binding patterns
            SyntaxKind[SyntaxKind["ObjectBindingPattern"] = 189] = "ObjectBindingPattern";
            SyntaxKind[SyntaxKind["ArrayBindingPattern"] = 190] = "ArrayBindingPattern";
            SyntaxKind[SyntaxKind["BindingElement"] = 191] = "BindingElement";
            // Expression
            SyntaxKind[SyntaxKind["ArrayLiteralExpression"] = 192] = "ArrayLiteralExpression";
            SyntaxKind[SyntaxKind["ObjectLiteralExpression"] = 193] = "ObjectLiteralExpression";
            SyntaxKind[SyntaxKind["PropertyAccessExpression"] = 194] = "PropertyAccessExpression";
            SyntaxKind[SyntaxKind["ElementAccessExpression"] = 195] = "ElementAccessExpression";
            SyntaxKind[SyntaxKind["CallExpression"] = 196] = "CallExpression";
            SyntaxKind[SyntaxKind["NewExpression"] = 197] = "NewExpression";
            SyntaxKind[SyntaxKind["TaggedTemplateExpression"] = 198] = "TaggedTemplateExpression";
            SyntaxKind[SyntaxKind["TypeAssertionExpression"] = 199] = "TypeAssertionExpression";
            SyntaxKind[SyntaxKind["ParenthesizedExpression"] = 200] = "ParenthesizedExpression";
            SyntaxKind[SyntaxKind["FunctionExpression"] = 201] = "FunctionExpression";
            SyntaxKind[SyntaxKind["ArrowFunction"] = 202] = "ArrowFunction";
            SyntaxKind[SyntaxKind["DeleteExpression"] = 203] = "DeleteExpression";
            SyntaxKind[SyntaxKind["TypeOfExpression"] = 204] = "TypeOfExpression";
            SyntaxKind[SyntaxKind["VoidExpression"] = 205] = "VoidExpression";
            SyntaxKind[SyntaxKind["AwaitExpression"] = 206] = "AwaitExpression";
            SyntaxKind[SyntaxKind["PrefixUnaryExpression"] = 207] = "PrefixUnaryExpression";
            SyntaxKind[SyntaxKind["PostfixUnaryExpression"] = 208] = "PostfixUnaryExpression";
            SyntaxKind[SyntaxKind["BinaryExpression"] = 209] = "BinaryExpression";
            SyntaxKind[SyntaxKind["ConditionalExpression"] = 210] = "ConditionalExpression";
            SyntaxKind[SyntaxKind["TemplateExpression"] = 211] = "TemplateExpression";
            SyntaxKind[SyntaxKind["YieldExpression"] = 212] = "YieldExpression";
            SyntaxKind[SyntaxKind["SpreadElement"] = 213] = "SpreadElement";
            SyntaxKind[SyntaxKind["ClassExpression"] = 214] = "ClassExpression";
            SyntaxKind[SyntaxKind["OmittedExpression"] = 215] = "OmittedExpression";
            SyntaxKind[SyntaxKind["ExpressionWithTypeArguments"] = 216] = "ExpressionWithTypeArguments";
            SyntaxKind[SyntaxKind["AsExpression"] = 217] = "AsExpression";
            SyntaxKind[SyntaxKind["NonNullExpression"] = 218] = "NonNullExpression";
            SyntaxKind[SyntaxKind["MetaProperty"] = 219] = "MetaProperty";
            SyntaxKind[SyntaxKind["SyntheticExpression"] = 220] = "SyntheticExpression";
            // Misc
            SyntaxKind[SyntaxKind["TemplateSpan"] = 221] = "TemplateSpan";
            SyntaxKind[SyntaxKind["SemicolonClassElement"] = 222] = "SemicolonClassElement";
            // Element
            SyntaxKind[SyntaxKind["Block"] = 223] = "Block";
            SyntaxKind[SyntaxKind["EmptyStatement"] = 224] = "EmptyStatement";
            SyntaxKind[SyntaxKind["VariableStatement"] = 225] = "VariableStatement";
            SyntaxKind[SyntaxKind["ExpressionStatement"] = 226] = "ExpressionStatement";
            SyntaxKind[SyntaxKind["IfStatement"] = 227] = "IfStatement";
            SyntaxKind[SyntaxKind["DoStatement"] = 228] = "DoStatement";
            SyntaxKind[SyntaxKind["WhileStatement"] = 229] = "WhileStatement";
            SyntaxKind[SyntaxKind["ForStatement"] = 230] = "ForStatement";
            SyntaxKind[SyntaxKind["ForInStatement"] = 231] = "ForInStatement";
            SyntaxKind[SyntaxKind["ForOfStatement"] = 232] = "ForOfStatement";
            SyntaxKind[SyntaxKind["ContinueStatement"] = 233] = "ContinueStatement";
            SyntaxKind[SyntaxKind["BreakStatement"] = 234] = "BreakStatement";
            SyntaxKind[SyntaxKind["ReturnStatement"] = 235] = "ReturnStatement";
            SyntaxKind[SyntaxKind["WithStatement"] = 236] = "WithStatement";
            SyntaxKind[SyntaxKind["SwitchStatement"] = 237] = "SwitchStatement";
            SyntaxKind[SyntaxKind["LabeledStatement"] = 238] = "LabeledStatement";
            SyntaxKind[SyntaxKind["ThrowStatement"] = 239] = "ThrowStatement";
            SyntaxKind[SyntaxKind["TryStatement"] = 240] = "TryStatement";
            SyntaxKind[SyntaxKind["DebuggerStatement"] = 241] = "DebuggerStatement";
            SyntaxKind[SyntaxKind["VariableDeclaration"] = 242] = "VariableDeclaration";
            SyntaxKind[SyntaxKind["VariableDeclarationList"] = 243] = "VariableDeclarationList";
            SyntaxKind[SyntaxKind["FunctionDeclaration"] = 244] = "FunctionDeclaration";
            SyntaxKind[SyntaxKind["ClassDeclaration"] = 245] = "ClassDeclaration";
            SyntaxKind[SyntaxKind["InterfaceDeclaration"] = 246] = "InterfaceDeclaration";
            SyntaxKind[SyntaxKind["TypeAliasDeclaration"] = 247] = "TypeAliasDeclaration";
            SyntaxKind[SyntaxKind["EnumDeclaration"] = 248] = "EnumDeclaration";
            SyntaxKind[SyntaxKind["ModuleDeclaration"] = 249] = "ModuleDeclaration";
            SyntaxKind[SyntaxKind["ModuleBlock"] = 250] = "ModuleBlock";
            SyntaxKind[SyntaxKind["CaseBlock"] = 251] = "CaseBlock";
            SyntaxKind[SyntaxKind["NamespaceExportDeclaration"] = 252] = "NamespaceExportDeclaration";
            SyntaxKind[SyntaxKind["ImportEqualsDeclaration"] = 253] = "ImportEqualsDeclaration";
            SyntaxKind[SyntaxKind["ImportDeclaration"] = 254] = "ImportDeclaration";
            SyntaxKind[SyntaxKind["ImportClause"] = 255] = "ImportClause";
            SyntaxKind[SyntaxKind["NamespaceImport"] = 256] = "NamespaceImport";
            SyntaxKind[SyntaxKind["NamedImports"] = 257] = "NamedImports";
            SyntaxKind[SyntaxKind["ImportSpecifier"] = 258] = "ImportSpecifier";
            SyntaxKind[SyntaxKind["ExportAssignment"] = 259] = "ExportAssignment";
            SyntaxKind[SyntaxKind["ExportDeclaration"] = 260] = "ExportDeclaration";
            SyntaxKind[SyntaxKind["NamedExports"] = 261] = "NamedExports";
            SyntaxKind[SyntaxKind["NamespaceExport"] = 262] = "NamespaceExport";
            SyntaxKind[SyntaxKind["ExportSpecifier"] = 263] = "ExportSpecifier";
            SyntaxKind[SyntaxKind["MissingDeclaration"] = 264] = "MissingDeclaration";
            // Module references
            SyntaxKind[SyntaxKind["ExternalModuleReference"] = 265] = "ExternalModuleReference";
            // JSX
            SyntaxKind[SyntaxKind["JsxElement"] = 266] = "JsxElement";
            SyntaxKind[SyntaxKind["JsxSelfClosingElement"] = 267] = "JsxSelfClosingElement";
            SyntaxKind[SyntaxKind["JsxOpeningElement"] = 268] = "JsxOpeningElement";
            SyntaxKind[SyntaxKind["JsxClosingElement"] = 269] = "JsxClosingElement";
            SyntaxKind[SyntaxKind["JsxFragment"] = 270] = "JsxFragment";
            SyntaxKind[SyntaxKind["JsxOpeningFragment"] = 271] = "JsxOpeningFragment";
            SyntaxKind[SyntaxKind["JsxClosingFragment"] = 272] = "JsxClosingFragment";
            SyntaxKind[SyntaxKind["JsxAttribute"] = 273] = "JsxAttribute";
            SyntaxKind[SyntaxKind["JsxAttributes"] = 274] = "JsxAttributes";
            SyntaxKind[SyntaxKind["JsxSpreadAttribute"] = 275] = "JsxSpreadAttribute";
            SyntaxKind[SyntaxKind["JsxExpression"] = 276] = "JsxExpression";
            // Clauses
            SyntaxKind[SyntaxKind["CaseClause"] = 277] = "CaseClause";
            SyntaxKind[SyntaxKind["DefaultClause"] = 278] = "DefaultClause";
            SyntaxKind[SyntaxKind["HeritageClause"] = 279] = "HeritageClause";
            SyntaxKind[SyntaxKind["CatchClause"] = 280] = "CatchClause";
            // Property assignments
            SyntaxKind[SyntaxKind["PropertyAssignment"] = 281] = "PropertyAssignment";
            SyntaxKind[SyntaxKind["ShorthandPropertyAssignment"] = 282] = "ShorthandPropertyAssignment";
            SyntaxKind[SyntaxKind["SpreadAssignment"] = 283] = "SpreadAssignment";
            // Enum
            SyntaxKind[SyntaxKind["EnumMember"] = 284] = "EnumMember";
            // Unparsed
            SyntaxKind[SyntaxKind["UnparsedPrologue"] = 285] = "UnparsedPrologue";
            SyntaxKind[SyntaxKind["UnparsedPrepend"] = 286] = "UnparsedPrepend";
            SyntaxKind[SyntaxKind["UnparsedText"] = 287] = "UnparsedText";
            SyntaxKind[SyntaxKind["UnparsedInternalText"] = 288] = "UnparsedInternalText";
            SyntaxKind[SyntaxKind["UnparsedSyntheticReference"] = 289] = "UnparsedSyntheticReference";
            // Top-level nodes
            SyntaxKind[SyntaxKind["SourceFile"] = 290] = "SourceFile";
            SyntaxKind[SyntaxKind["Bundle"] = 291] = "Bundle";
            SyntaxKind[SyntaxKind["UnparsedSource"] = 292] = "UnparsedSource";
            SyntaxKind[SyntaxKind["InputFiles"] = 293] = "InputFiles";
            // JSDoc nodes
            SyntaxKind[SyntaxKind["JSDocTypeExpression"] = 294] = "JSDocTypeExpression";
            // The * type
            SyntaxKind[SyntaxKind["JSDocAllType"] = 295] = "JSDocAllType";
            // The ? type
            SyntaxKind[SyntaxKind["JSDocUnknownType"] = 296] = "JSDocUnknownType";
            SyntaxKind[SyntaxKind["JSDocNullableType"] = 297] = "JSDocNullableType";
            SyntaxKind[SyntaxKind["JSDocNonNullableType"] = 298] = "JSDocNonNullableType";
            SyntaxKind[SyntaxKind["JSDocOptionalType"] = 299] = "JSDocOptionalType";
            SyntaxKind[SyntaxKind["JSDocFunctionType"] = 300] = "JSDocFunctionType";
            SyntaxKind[SyntaxKind["JSDocVariadicType"] = 301] = "JSDocVariadicType";
            // https://jsdoc.app/about-namepaths.html
            SyntaxKind[SyntaxKind["JSDocNamepathType"] = 302] = "JSDocNamepathType";
            SyntaxKind[SyntaxKind["JSDocComment"] = 303] = "JSDocComment";
            SyntaxKind[SyntaxKind["JSDocTypeLiteral"] = 304] = "JSDocTypeLiteral";
            SyntaxKind[SyntaxKind["JSDocSignature"] = 305] = "JSDocSignature";
            SyntaxKind[SyntaxKind["JSDocTag"] = 306] = "JSDocTag";
            SyntaxKind[SyntaxKind["JSDocAugmentsTag"] = 307] = "JSDocAugmentsTag";
            SyntaxKind[SyntaxKind["JSDocAuthorTag"] = 308] = "JSDocAuthorTag";
            SyntaxKind[SyntaxKind["JSDocClassTag"] = 309] = "JSDocClassTag";
            SyntaxKind[SyntaxKind["JSDocPublicTag"] = 310] = "JSDocPublicTag";
            SyntaxKind[SyntaxKind["JSDocPrivateTag"] = 311] = "JSDocPrivateTag";
            SyntaxKind[SyntaxKind["JSDocProtectedTag"] = 312] = "JSDocProtectedTag";
            SyntaxKind[SyntaxKind["JSDocReadonlyTag"] = 313] = "JSDocReadonlyTag";
            SyntaxKind[SyntaxKind["JSDocCallbackTag"] = 314] = "JSDocCallbackTag";
            SyntaxKind[SyntaxKind["JSDocEnumTag"] = 315] = "JSDocEnumTag";
            SyntaxKind[SyntaxKind["JSDocParameterTag"] = 316] = "JSDocParameterTag";
            SyntaxKind[SyntaxKind["JSDocReturnTag"] = 317] = "JSDocReturnTag";
            SyntaxKind[SyntaxKind["JSDocThisTag"] = 318] = "JSDocThisTag";
            SyntaxKind[SyntaxKind["JSDocTypeTag"] = 319] = "JSDocTypeTag";
            SyntaxKind[SyntaxKind["JSDocTemplateTag"] = 320] = "JSDocTemplateTag";
            SyntaxKind[SyntaxKind["JSDocTypedefTag"] = 321] = "JSDocTypedefTag";
            SyntaxKind[SyntaxKind["JSDocPropertyTag"] = 322] = "JSDocPropertyTag";
            // Synthesized list
            SyntaxKind[SyntaxKind["SyntaxList"] = 323] = "SyntaxList";
            // Transformation nodes
            SyntaxKind[SyntaxKind["NotEmittedStatement"] = 324] = "NotEmittedStatement";
            SyntaxKind[SyntaxKind["PartiallyEmittedExpression"] = 325] = "PartiallyEmittedExpression";
            SyntaxKind[SyntaxKind["CommaListExpression"] = 326] = "CommaListExpression";
            SyntaxKind[SyntaxKind["MergeDeclarationMarker"] = 327] = "MergeDeclarationMarker";
            SyntaxKind[SyntaxKind["EndOfDeclarationMarker"] = 328] = "EndOfDeclarationMarker";
            SyntaxKind[SyntaxKind["SyntheticReferenceExpression"] = 329] = "SyntheticReferenceExpression";
            // Enum value count
            SyntaxKind[SyntaxKind["Count"] = 330] = "Count";
            // Markers
            SyntaxKind[SyntaxKind["FirstAssignment"] = 62] = "FirstAssignment";
            SyntaxKind[SyntaxKind["LastAssignment"] = 74] = "LastAssignment";
            SyntaxKind[SyntaxKind["FirstCompoundAssignment"] = 63] = "FirstCompoundAssignment";
            SyntaxKind[SyntaxKind["LastCompoundAssignment"] = 74] = "LastCompoundAssignment";
            SyntaxKind[SyntaxKind["FirstReservedWord"] = 77] = "FirstReservedWord";
            SyntaxKind[SyntaxKind["LastReservedWord"] = 112] = "LastReservedWord";
            SyntaxKind[SyntaxKind["FirstKeyword"] = 77] = "FirstKeyword";
            SyntaxKind[SyntaxKind["LastKeyword"] = 152] = "LastKeyword";
            SyntaxKind[SyntaxKind["FirstFutureReservedWord"] = 113] = "FirstFutureReservedWord";
            SyntaxKind[SyntaxKind["LastFutureReservedWord"] = 121] = "LastFutureReservedWord";
            SyntaxKind[SyntaxKind["FirstTypeNode"] = 168] = "FirstTypeNode";
            SyntaxKind[SyntaxKind["LastTypeNode"] = 188] = "LastTypeNode";
            SyntaxKind[SyntaxKind["FirstPunctuation"] = 18] = "FirstPunctuation";
            SyntaxKind[SyntaxKind["LastPunctuation"] = 74] = "LastPunctuation";
            SyntaxKind[SyntaxKind["FirstToken"] = 0] = "FirstToken";
            SyntaxKind[SyntaxKind["LastToken"] = 152] = "LastToken";
            SyntaxKind[SyntaxKind["FirstTriviaToken"] = 2] = "FirstTriviaToken";
            SyntaxKind[SyntaxKind["LastTriviaToken"] = 7] = "LastTriviaToken";
            SyntaxKind[SyntaxKind["FirstLiteralToken"] = 8] = "FirstLiteralToken";
            SyntaxKind[SyntaxKind["LastLiteralToken"] = 14] = "LastLiteralToken";
            SyntaxKind[SyntaxKind["FirstTemplateToken"] = 14] = "FirstTemplateToken";
            SyntaxKind[SyntaxKind["LastTemplateToken"] = 17] = "LastTemplateToken";
            SyntaxKind[SyntaxKind["FirstBinaryOperator"] = 29] = "FirstBinaryOperator";
            SyntaxKind[SyntaxKind["LastBinaryOperator"] = 74] = "LastBinaryOperator";
            SyntaxKind[SyntaxKind["FirstStatement"] = 225] = "FirstStatement";
            SyntaxKind[SyntaxKind["LastStatement"] = 241] = "LastStatement";
            SyntaxKind[SyntaxKind["FirstNode"] = 153] = "FirstNode";
            SyntaxKind[SyntaxKind["FirstJSDocNode"] = 294] = "FirstJSDocNode";
            SyntaxKind[SyntaxKind["LastJSDocNode"] = 322] = "LastJSDocNode";
            SyntaxKind[SyntaxKind["FirstJSDocTagNode"] = 306] = "FirstJSDocTagNode";
            SyntaxKind[SyntaxKind["LastJSDocTagNode"] = 322] = "LastJSDocTagNode";
            /* @internal */ SyntaxKind[SyntaxKind["FirstContextualKeyword"] = 122] = "FirstContextualKeyword";
            /* @internal */ SyntaxKind[SyntaxKind["LastContextualKeyword"] = 152] = "LastContextualKeyword";
        })(SyntaxKind = ts.SyntaxKind || (ts.SyntaxKind = {}));
        var NodeFlags;
        (function (NodeFlags) {
            NodeFlags[NodeFlags["None"] = 0] = "None";
            NodeFlags[NodeFlags["Let"] = 1] = "Let";
            NodeFlags[NodeFlags["Const"] = 2] = "Const";
            NodeFlags[NodeFlags["NestedNamespace"] = 4] = "NestedNamespace";
            NodeFlags[NodeFlags["Synthesized"] = 8] = "Synthesized";
            NodeFlags[NodeFlags["Namespace"] = 16] = "Namespace";
            NodeFlags[NodeFlags["OptionalChain"] = 32] = "OptionalChain";
            NodeFlags[NodeFlags["ExportContext"] = 64] = "ExportContext";
            NodeFlags[NodeFlags["ContainsThis"] = 128] = "ContainsThis";
            NodeFlags[NodeFlags["HasImplicitReturn"] = 256] = "HasImplicitReturn";
            NodeFlags[NodeFlags["HasExplicitReturn"] = 512] = "HasExplicitReturn";
            NodeFlags[NodeFlags["GlobalAugmentation"] = 1024] = "GlobalAugmentation";
            NodeFlags[NodeFlags["HasAsyncFunctions"] = 2048] = "HasAsyncFunctions";
            NodeFlags[NodeFlags["DisallowInContext"] = 4096] = "DisallowInContext";
            NodeFlags[NodeFlags["YieldContext"] = 8192] = "YieldContext";
            NodeFlags[NodeFlags["DecoratorContext"] = 16384] = "DecoratorContext";
            NodeFlags[NodeFlags["AwaitContext"] = 32768] = "AwaitContext";
            NodeFlags[NodeFlags["ThisNodeHasError"] = 65536] = "ThisNodeHasError";
            NodeFlags[NodeFlags["JavaScriptFile"] = 131072] = "JavaScriptFile";
            NodeFlags[NodeFlags["ThisNodeOrAnySubNodesHasError"] = 262144] = "ThisNodeOrAnySubNodesHasError";
            NodeFlags[NodeFlags["HasAggregatedChildData"] = 524288] = "HasAggregatedChildData";
            // These flags will be set when the parser encounters a dynamic import expression or 'import.meta' to avoid
            // walking the tree if the flags are not set. However, these flags are just a approximation
            // (hence why it's named "PossiblyContainsDynamicImport") because once set, the flags never get cleared.
            // During editing, if a dynamic import is removed, incremental parsing will *NOT* clear this flag.
            // This means that the tree will always be traversed during module resolution, or when looking for external module indicators.
            // However, the removal operation should not occur often and in the case of the
            // removal, it is likely that users will add the import anyway.
            // The advantage of this approach is its simplicity. For the case of batch compilation,
            // we guarantee that users won't have to pay the price of walking the tree if a dynamic import isn't used.
            /* @internal */ NodeFlags[NodeFlags["PossiblyContainsDynamicImport"] = 1048576] = "PossiblyContainsDynamicImport";
            /* @internal */ NodeFlags[NodeFlags["PossiblyContainsImportMeta"] = 2097152] = "PossiblyContainsImportMeta";
            NodeFlags[NodeFlags["JSDoc"] = 4194304] = "JSDoc";
            /* @internal */ NodeFlags[NodeFlags["Ambient"] = 8388608] = "Ambient";
            /* @internal */ NodeFlags[NodeFlags["InWithStatement"] = 16777216] = "InWithStatement";
            NodeFlags[NodeFlags["JsonFile"] = 33554432] = "JsonFile";
            /* @internal */ NodeFlags[NodeFlags["TypeCached"] = 67108864] = "TypeCached";
            NodeFlags[NodeFlags["BlockScoped"] = 3] = "BlockScoped";
            NodeFlags[NodeFlags["ReachabilityCheckFlags"] = 768] = "ReachabilityCheckFlags";
            NodeFlags[NodeFlags["ReachabilityAndEmitFlags"] = 2816] = "ReachabilityAndEmitFlags";
            // Parsing context flags
            NodeFlags[NodeFlags["ContextFlags"] = 25358336] = "ContextFlags";
            // Exclude these flags when parsing a Type
            NodeFlags[NodeFlags["TypeExcludesFlags"] = 40960] = "TypeExcludesFlags";
            // Represents all flags that are potentially set once and
            // never cleared on SourceFiles which get re-used in between incremental parses.
            // See the comment above on `PossiblyContainsDynamicImport` and `PossiblyContainsImportMeta`.
            /* @internal */ NodeFlags[NodeFlags["PermanentlySetIncrementalFlags"] = 3145728] = "PermanentlySetIncrementalFlags";
        })(NodeFlags = ts.NodeFlags || (ts.NodeFlags = {}));
        var ModifierFlags;
        (function (ModifierFlags) {
            ModifierFlags[ModifierFlags["None"] = 0] = "None";
            ModifierFlags[ModifierFlags["Export"] = 1] = "Export";
            ModifierFlags[ModifierFlags["Ambient"] = 2] = "Ambient";
            ModifierFlags[ModifierFlags["Public"] = 4] = "Public";
            ModifierFlags[ModifierFlags["Private"] = 8] = "Private";
            ModifierFlags[ModifierFlags["Protected"] = 16] = "Protected";
            ModifierFlags[ModifierFlags["Static"] = 32] = "Static";
            ModifierFlags[ModifierFlags["Readonly"] = 64] = "Readonly";
            ModifierFlags[ModifierFlags["Abstract"] = 128] = "Abstract";
            ModifierFlags[ModifierFlags["Async"] = 256] = "Async";
            ModifierFlags[ModifierFlags["Default"] = 512] = "Default";
            ModifierFlags[ModifierFlags["Const"] = 2048] = "Const";
            ModifierFlags[ModifierFlags["HasComputedFlags"] = 536870912] = "HasComputedFlags";
            ModifierFlags[ModifierFlags["AccessibilityModifier"] = 28] = "AccessibilityModifier";
            // Accessibility modifiers and 'readonly' can be attached to a parameter in a constructor to make it a property.
            ModifierFlags[ModifierFlags["ParameterPropertyModifier"] = 92] = "ParameterPropertyModifier";
            ModifierFlags[ModifierFlags["NonPublicAccessibilityModifier"] = 24] = "NonPublicAccessibilityModifier";
            ModifierFlags[ModifierFlags["TypeScriptModifier"] = 2270] = "TypeScriptModifier";
            ModifierFlags[ModifierFlags["ExportDefault"] = 513] = "ExportDefault";
            ModifierFlags[ModifierFlags["All"] = 3071] = "All";
        })(ModifierFlags = ts.ModifierFlags || (ts.ModifierFlags = {}));
        var JsxFlags;
        (function (JsxFlags) {
            JsxFlags[JsxFlags["None"] = 0] = "None";
            /** An element from a named property of the JSX.IntrinsicElements interface */
            JsxFlags[JsxFlags["IntrinsicNamedElement"] = 1] = "IntrinsicNamedElement";
            /** An element inferred from the string index signature of the JSX.IntrinsicElements interface */
            JsxFlags[JsxFlags["IntrinsicIndexedElement"] = 2] = "IntrinsicIndexedElement";
            JsxFlags[JsxFlags["IntrinsicElement"] = 3] = "IntrinsicElement";
        })(JsxFlags = ts.JsxFlags || (ts.JsxFlags = {}));
        /* @internal */
        var RelationComparisonResult;
        (function (RelationComparisonResult) {
            RelationComparisonResult[RelationComparisonResult["Succeeded"] = 1] = "Succeeded";
            RelationComparisonResult[RelationComparisonResult["Failed"] = 2] = "Failed";
            RelationComparisonResult[RelationComparisonResult["Reported"] = 4] = "Reported";
            RelationComparisonResult[RelationComparisonResult["ReportsUnmeasurable"] = 8] = "ReportsUnmeasurable";
            RelationComparisonResult[RelationComparisonResult["ReportsUnreliable"] = 16] = "ReportsUnreliable";
            RelationComparisonResult[RelationComparisonResult["ReportsMask"] = 24] = "ReportsMask";
        })(RelationComparisonResult = ts.RelationComparisonResult || (ts.RelationComparisonResult = {}));
        /*@internal*/
        var GeneratedIdentifierFlags;
        (function (GeneratedIdentifierFlags) {
            // Kinds
            GeneratedIdentifierFlags[GeneratedIdentifierFlags["None"] = 0] = "None";
            GeneratedIdentifierFlags[GeneratedIdentifierFlags["Auto"] = 1] = "Auto";
            GeneratedIdentifierFlags[GeneratedIdentifierFlags["Loop"] = 2] = "Loop";
            GeneratedIdentifierFlags[GeneratedIdentifierFlags["Unique"] = 3] = "Unique";
            GeneratedIdentifierFlags[GeneratedIdentifierFlags["Node"] = 4] = "Node";
            GeneratedIdentifierFlags[GeneratedIdentifierFlags["KindMask"] = 7] = "KindMask";
            // Flags
            GeneratedIdentifierFlags[GeneratedIdentifierFlags["ReservedInNestedScopes"] = 8] = "ReservedInNestedScopes";
            GeneratedIdentifierFlags[GeneratedIdentifierFlags["Optimistic"] = 16] = "Optimistic";
            GeneratedIdentifierFlags[GeneratedIdentifierFlags["FileLevel"] = 32] = "FileLevel";
        })(GeneratedIdentifierFlags = ts.GeneratedIdentifierFlags || (ts.GeneratedIdentifierFlags = {}));
        var TokenFlags;
        (function (TokenFlags) {
            TokenFlags[TokenFlags["None"] = 0] = "None";
            /* @internal */
            TokenFlags[TokenFlags["PrecedingLineBreak"] = 1] = "PrecedingLineBreak";
            /* @internal */
            TokenFlags[TokenFlags["PrecedingJSDocComment"] = 2] = "PrecedingJSDocComment";
            /* @internal */
            TokenFlags[TokenFlags["Unterminated"] = 4] = "Unterminated";
            /* @internal */
            TokenFlags[TokenFlags["ExtendedUnicodeEscape"] = 8] = "ExtendedUnicodeEscape";
            TokenFlags[TokenFlags["Scientific"] = 16] = "Scientific";
            TokenFlags[TokenFlags["Octal"] = 32] = "Octal";
            TokenFlags[TokenFlags["HexSpecifier"] = 64] = "HexSpecifier";
            TokenFlags[TokenFlags["BinarySpecifier"] = 128] = "BinarySpecifier";
            TokenFlags[TokenFlags["OctalSpecifier"] = 256] = "OctalSpecifier";
            /* @internal */
            TokenFlags[TokenFlags["ContainsSeparator"] = 512] = "ContainsSeparator";
            /* @internal */
            TokenFlags[TokenFlags["UnicodeEscape"] = 1024] = "UnicodeEscape";
            /* @internal */
            TokenFlags[TokenFlags["BinaryOrOctalSpecifier"] = 384] = "BinaryOrOctalSpecifier";
            /* @internal */
            TokenFlags[TokenFlags["NumericLiteralFlags"] = 1008] = "NumericLiteralFlags";
        })(TokenFlags = ts.TokenFlags || (ts.TokenFlags = {}));
        // NOTE: Ensure this is up-to-date with src/debug/debug.ts
        var FlowFlags;
        (function (FlowFlags) {
            FlowFlags[FlowFlags["Unreachable"] = 1] = "Unreachable";
            FlowFlags[FlowFlags["Start"] = 2] = "Start";
            FlowFlags[FlowFlags["BranchLabel"] = 4] = "BranchLabel";
            FlowFlags[FlowFlags["LoopLabel"] = 8] = "LoopLabel";
            FlowFlags[FlowFlags["Assignment"] = 16] = "Assignment";
            FlowFlags[FlowFlags["TrueCondition"] = 32] = "TrueCondition";
            FlowFlags[FlowFlags["FalseCondition"] = 64] = "FalseCondition";
            FlowFlags[FlowFlags["SwitchClause"] = 128] = "SwitchClause";
            FlowFlags[FlowFlags["ArrayMutation"] = 256] = "ArrayMutation";
            FlowFlags[FlowFlags["Call"] = 512] = "Call";
            FlowFlags[FlowFlags["Referenced"] = 1024] = "Referenced";
            FlowFlags[FlowFlags["Shared"] = 2048] = "Shared";
            FlowFlags[FlowFlags["PreFinally"] = 4096] = "PreFinally";
            FlowFlags[FlowFlags["AfterFinally"] = 8192] = "AfterFinally";
            FlowFlags[FlowFlags["Label"] = 12] = "Label";
            FlowFlags[FlowFlags["Condition"] = 96] = "Condition";
        })(FlowFlags = ts.FlowFlags || (ts.FlowFlags = {}));
        var OperationCanceledException = /** @class */ (function () {
            function OperationCanceledException() {
            }
            return OperationCanceledException;
        }());
        ts.OperationCanceledException = OperationCanceledException;
        /*@internal*/
        var RefFileKind;
        (function (RefFileKind) {
            RefFileKind[RefFileKind["Import"] = 0] = "Import";
            RefFileKind[RefFileKind["ReferenceFile"] = 1] = "ReferenceFile";
            RefFileKind[RefFileKind["TypeReferenceDirective"] = 2] = "TypeReferenceDirective";
        })(RefFileKind = ts.RefFileKind || (ts.RefFileKind = {}));
        /* @internal */
        var StructureIsReused;
        (function (StructureIsReused) {
            StructureIsReused[StructureIsReused["Not"] = 0] = "Not";
            StructureIsReused[StructureIsReused["SafeModules"] = 1] = "SafeModules";
            StructureIsReused[StructureIsReused["Completely"] = 2] = "Completely";
        })(StructureIsReused = ts.StructureIsReused || (ts.StructureIsReused = {}));
        /** Return code used by getEmitOutput function to indicate status of the function */
        var ExitStatus;
        (function (ExitStatus) {
            // Compiler ran successfully.  Either this was a simple do-nothing compilation (for example,
            // when -version or -help was provided, or this was a normal compilation, no diagnostics
            // were produced, and all outputs were generated successfully.
            ExitStatus[ExitStatus["Success"] = 0] = "Success";
            // Diagnostics were produced and because of them no code was generated.
            ExitStatus[ExitStatus["DiagnosticsPresent_OutputsSkipped"] = 1] = "DiagnosticsPresent_OutputsSkipped";
            // Diagnostics were produced and outputs were generated in spite of them.
            ExitStatus[ExitStatus["DiagnosticsPresent_OutputsGenerated"] = 2] = "DiagnosticsPresent_OutputsGenerated";
            // When build skipped because passed in project is invalid
            ExitStatus[ExitStatus["InvalidProject_OutputsSkipped"] = 3] = "InvalidProject_OutputsSkipped";
            // When build is skipped because project references form cycle
            ExitStatus[ExitStatus["ProjectReferenceCycle_OutputsSkipped"] = 4] = "ProjectReferenceCycle_OutputsSkipped";
            /** @deprecated Use ProjectReferenceCycle_OutputsSkipped instead. */
            ExitStatus[ExitStatus["ProjectReferenceCycle_OutputsSkupped"] = 4] = "ProjectReferenceCycle_OutputsSkupped";
        })(ExitStatus = ts.ExitStatus || (ts.ExitStatus = {}));
        /* @internal */
        var UnionReduction;
        (function (UnionReduction) {
            UnionReduction[UnionReduction["None"] = 0] = "None";
            UnionReduction[UnionReduction["Literal"] = 1] = "Literal";
            UnionReduction[UnionReduction["Subtype"] = 2] = "Subtype";
        })(UnionReduction = ts.UnionReduction || (ts.UnionReduction = {}));
        /* @internal */
        var ContextFlags;
        (function (ContextFlags) {
            ContextFlags[ContextFlags["None"] = 0] = "None";
            ContextFlags[ContextFlags["Signature"] = 1] = "Signature";
            ContextFlags[ContextFlags["NoConstraints"] = 2] = "NoConstraints";
            ContextFlags[ContextFlags["Completions"] = 4] = "Completions";
        })(ContextFlags = ts.ContextFlags || (ts.ContextFlags = {}));
        // NOTE: If modifying this enum, must modify `TypeFormatFlags` too!
        var NodeBuilderFlags;
        (function (NodeBuilderFlags) {
            NodeBuilderFlags[NodeBuilderFlags["None"] = 0] = "None";
            // Options
            NodeBuilderFlags[NodeBuilderFlags["NoTruncation"] = 1] = "NoTruncation";
            NodeBuilderFlags[NodeBuilderFlags["WriteArrayAsGenericType"] = 2] = "WriteArrayAsGenericType";
            NodeBuilderFlags[NodeBuilderFlags["GenerateNamesForShadowedTypeParams"] = 4] = "GenerateNamesForShadowedTypeParams";
            NodeBuilderFlags[NodeBuilderFlags["UseStructuralFallback"] = 8] = "UseStructuralFallback";
            NodeBuilderFlags[NodeBuilderFlags["ForbidIndexedAccessSymbolReferences"] = 16] = "ForbidIndexedAccessSymbolReferences";
            NodeBuilderFlags[NodeBuilderFlags["WriteTypeArgumentsOfSignature"] = 32] = "WriteTypeArgumentsOfSignature";
            NodeBuilderFlags[NodeBuilderFlags["UseFullyQualifiedType"] = 64] = "UseFullyQualifiedType";
            NodeBuilderFlags[NodeBuilderFlags["UseOnlyExternalAliasing"] = 128] = "UseOnlyExternalAliasing";
            NodeBuilderFlags[NodeBuilderFlags["SuppressAnyReturnType"] = 256] = "SuppressAnyReturnType";
            NodeBuilderFlags[NodeBuilderFlags["WriteTypeParametersInQualifiedName"] = 512] = "WriteTypeParametersInQualifiedName";
            NodeBuilderFlags[NodeBuilderFlags["MultilineObjectLiterals"] = 1024] = "MultilineObjectLiterals";
            NodeBuilderFlags[NodeBuilderFlags["WriteClassExpressionAsTypeLiteral"] = 2048] = "WriteClassExpressionAsTypeLiteral";
            NodeBuilderFlags[NodeBuilderFlags["UseTypeOfFunction"] = 4096] = "UseTypeOfFunction";
            NodeBuilderFlags[NodeBuilderFlags["OmitParameterModifiers"] = 8192] = "OmitParameterModifiers";
            NodeBuilderFlags[NodeBuilderFlags["UseAliasDefinedOutsideCurrentScope"] = 16384] = "UseAliasDefinedOutsideCurrentScope";
            NodeBuilderFlags[NodeBuilderFlags["UseSingleQuotesForStringLiteralType"] = 268435456] = "UseSingleQuotesForStringLiteralType";
            // Error handling
            NodeBuilderFlags[NodeBuilderFlags["AllowThisInObjectLiteral"] = 32768] = "AllowThisInObjectLiteral";
            NodeBuilderFlags[NodeBuilderFlags["AllowQualifedNameInPlaceOfIdentifier"] = 65536] = "AllowQualifedNameInPlaceOfIdentifier";
            NodeBuilderFlags[NodeBuilderFlags["AllowAnonymousIdentifier"] = 131072] = "AllowAnonymousIdentifier";
            NodeBuilderFlags[NodeBuilderFlags["AllowEmptyUnionOrIntersection"] = 262144] = "AllowEmptyUnionOrIntersection";
            NodeBuilderFlags[NodeBuilderFlags["AllowEmptyTuple"] = 524288] = "AllowEmptyTuple";
            NodeBuilderFlags[NodeBuilderFlags["AllowUniqueESSymbolType"] = 1048576] = "AllowUniqueESSymbolType";
            NodeBuilderFlags[NodeBuilderFlags["AllowEmptyIndexInfoType"] = 2097152] = "AllowEmptyIndexInfoType";
            // Errors (cont.)
            NodeBuilderFlags[NodeBuilderFlags["AllowNodeModulesRelativePaths"] = 67108864] = "AllowNodeModulesRelativePaths";
            /* @internal */ NodeBuilderFlags[NodeBuilderFlags["DoNotIncludeSymbolChain"] = 134217728] = "DoNotIncludeSymbolChain";
            NodeBuilderFlags[NodeBuilderFlags["IgnoreErrors"] = 70221824] = "IgnoreErrors";
            // State
            NodeBuilderFlags[NodeBuilderFlags["InObjectTypeLiteral"] = 4194304] = "InObjectTypeLiteral";
            NodeBuilderFlags[NodeBuilderFlags["InTypeAlias"] = 8388608] = "InTypeAlias";
            NodeBuilderFlags[NodeBuilderFlags["InInitialEntityName"] = 16777216] = "InInitialEntityName";
            NodeBuilderFlags[NodeBuilderFlags["InReverseMappedType"] = 33554432] = "InReverseMappedType";
        })(NodeBuilderFlags = ts.NodeBuilderFlags || (ts.NodeBuilderFlags = {}));
        // Ensure the shared flags between this and `NodeBuilderFlags` stay in alignment
        var TypeFormatFlags;
        (function (TypeFormatFlags) {
            TypeFormatFlags[TypeFormatFlags["None"] = 0] = "None";
            TypeFormatFlags[TypeFormatFlags["NoTruncation"] = 1] = "NoTruncation";
            TypeFormatFlags[TypeFormatFlags["WriteArrayAsGenericType"] = 2] = "WriteArrayAsGenericType";
            // hole because there's a hole in node builder flags
            TypeFormatFlags[TypeFormatFlags["UseStructuralFallback"] = 8] = "UseStructuralFallback";
            // hole because there's a hole in node builder flags
            TypeFormatFlags[TypeFormatFlags["WriteTypeArgumentsOfSignature"] = 32] = "WriteTypeArgumentsOfSignature";
            TypeFormatFlags[TypeFormatFlags["UseFullyQualifiedType"] = 64] = "UseFullyQualifiedType";
            // hole because `UseOnlyExternalAliasing` is here in node builder flags, but functions which take old flags use `SymbolFormatFlags` instead
            TypeFormatFlags[TypeFormatFlags["SuppressAnyReturnType"] = 256] = "SuppressAnyReturnType";
            // hole because `WriteTypeParametersInQualifiedName` is here in node builder flags, but functions which take old flags use `SymbolFormatFlags` for this instead
            TypeFormatFlags[TypeFormatFlags["MultilineObjectLiterals"] = 1024] = "MultilineObjectLiterals";
            TypeFormatFlags[TypeFormatFlags["WriteClassExpressionAsTypeLiteral"] = 2048] = "WriteClassExpressionAsTypeLiteral";
            TypeFormatFlags[TypeFormatFlags["UseTypeOfFunction"] = 4096] = "UseTypeOfFunction";
            TypeFormatFlags[TypeFormatFlags["OmitParameterModifiers"] = 8192] = "OmitParameterModifiers";
            TypeFormatFlags[TypeFormatFlags["UseAliasDefinedOutsideCurrentScope"] = 16384] = "UseAliasDefinedOutsideCurrentScope";
            TypeFormatFlags[TypeFormatFlags["UseSingleQuotesForStringLiteralType"] = 268435456] = "UseSingleQuotesForStringLiteralType";
            // Error Handling
            TypeFormatFlags[TypeFormatFlags["AllowUniqueESSymbolType"] = 1048576] = "AllowUniqueESSymbolType";
            // TypeFormatFlags exclusive
            TypeFormatFlags[TypeFormatFlags["AddUndefined"] = 131072] = "AddUndefined";
            TypeFormatFlags[TypeFormatFlags["WriteArrowStyleSignature"] = 262144] = "WriteArrowStyleSignature";
            // State
            TypeFormatFlags[TypeFormatFlags["InArrayType"] = 524288] = "InArrayType";
            TypeFormatFlags[TypeFormatFlags["InElementType"] = 2097152] = "InElementType";
            TypeFormatFlags[TypeFormatFlags["InFirstTypeArgument"] = 4194304] = "InFirstTypeArgument";
            TypeFormatFlags[TypeFormatFlags["InTypeAlias"] = 8388608] = "InTypeAlias";
            /** @deprecated */ TypeFormatFlags[TypeFormatFlags["WriteOwnNameForAnyLike"] = 0] = "WriteOwnNameForAnyLike";
            TypeFormatFlags[TypeFormatFlags["NodeBuilderFlagsMask"] = 277904747] = "NodeBuilderFlagsMask";
        })(TypeFormatFlags = ts.TypeFormatFlags || (ts.TypeFormatFlags = {}));
        var SymbolFormatFlags;
        (function (SymbolFormatFlags) {
            SymbolFormatFlags[SymbolFormatFlags["None"] = 0] = "None";
            // Write symbols's type argument if it is instantiated symbol
            // eg. class C<T> { p: T }   <-- Show p as C<T>.p here
            //     var a: C<number>;
            //     var p = a.p; <--- Here p is property of C<number> so show it as C<number>.p instead of just C.p
            SymbolFormatFlags[SymbolFormatFlags["WriteTypeParametersOrArguments"] = 1] = "WriteTypeParametersOrArguments";
            // Use only external alias information to get the symbol name in the given context
            // eg.  module m { export class c { } } import x = m.c;
            // When this flag is specified m.c will be used to refer to the class instead of alias symbol x
            SymbolFormatFlags[SymbolFormatFlags["UseOnlyExternalAliasing"] = 2] = "UseOnlyExternalAliasing";
            // Build symbol name using any nodes needed, instead of just components of an entity name
            SymbolFormatFlags[SymbolFormatFlags["AllowAnyNodeKind"] = 4] = "AllowAnyNodeKind";
            // Prefer aliases which are not directly visible
            SymbolFormatFlags[SymbolFormatFlags["UseAliasDefinedOutsideCurrentScope"] = 8] = "UseAliasDefinedOutsideCurrentScope";
            // Skip building an accessible symbol chain
            /* @internal */ SymbolFormatFlags[SymbolFormatFlags["DoNotIncludeSymbolChain"] = 16] = "DoNotIncludeSymbolChain";
        })(SymbolFormatFlags = ts.SymbolFormatFlags || (ts.SymbolFormatFlags = {}));
        /* @internal */
        var SymbolAccessibility;
        (function (SymbolAccessibility) {
            SymbolAccessibility[SymbolAccessibility["Accessible"] = 0] = "Accessible";
            SymbolAccessibility[SymbolAccessibility["NotAccessible"] = 1] = "NotAccessible";
            SymbolAccessibility[SymbolAccessibility["CannotBeNamed"] = 2] = "CannotBeNamed";
        })(SymbolAccessibility = ts.SymbolAccessibility || (ts.SymbolAccessibility = {}));
        /* @internal */
        var SyntheticSymbolKind;
        (function (SyntheticSymbolKind) {
            SyntheticSymbolKind[SyntheticSymbolKind["UnionOrIntersection"] = 0] = "UnionOrIntersection";
            SyntheticSymbolKind[SyntheticSymbolKind["Spread"] = 1] = "Spread";
        })(SyntheticSymbolKind = ts.SyntheticSymbolKind || (ts.SyntheticSymbolKind = {}));
        var TypePredicateKind;
        (function (TypePredicateKind) {
            TypePredicateKind[TypePredicateKind["This"] = 0] = "This";
            TypePredicateKind[TypePredicateKind["Identifier"] = 1] = "Identifier";
            TypePredicateKind[TypePredicateKind["AssertsThis"] = 2] = "AssertsThis";
            TypePredicateKind[TypePredicateKind["AssertsIdentifier"] = 3] = "AssertsIdentifier";
        })(TypePredicateKind = ts.TypePredicateKind || (ts.TypePredicateKind = {}));
        /** Indicates how to serialize the name for a TypeReferenceNode when emitting decorator metadata */
        /* @internal */
        var TypeReferenceSerializationKind;
        (function (TypeReferenceSerializationKind) {
            // The TypeReferenceNode could not be resolved.
            // The type name should be emitted using a safe fallback.
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["Unknown"] = 0] = "Unknown";
            // The TypeReferenceNode resolves to a type with a constructor
            // function that can be reached at runtime (e.g. a `class`
            // declaration or a `var` declaration for the static side
            // of a type, such as the global `Promise` type in lib.d.ts).
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["TypeWithConstructSignatureAndValue"] = 1] = "TypeWithConstructSignatureAndValue";
            // The TypeReferenceNode resolves to a Void-like, Nullable, or Never type.
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["VoidNullableOrNeverType"] = 2] = "VoidNullableOrNeverType";
            // The TypeReferenceNode resolves to a Number-like type.
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["NumberLikeType"] = 3] = "NumberLikeType";
            // The TypeReferenceNode resolves to a BigInt-like type.
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["BigIntLikeType"] = 4] = "BigIntLikeType";
            // The TypeReferenceNode resolves to a String-like type.
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["StringLikeType"] = 5] = "StringLikeType";
            // The TypeReferenceNode resolves to a Boolean-like type.
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["BooleanType"] = 6] = "BooleanType";
            // The TypeReferenceNode resolves to an Array-like type.
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["ArrayLikeType"] = 7] = "ArrayLikeType";
            // The TypeReferenceNode resolves to the ESSymbol type.
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["ESSymbolType"] = 8] = "ESSymbolType";
            // The TypeReferenceNode resolved to the global Promise constructor symbol.
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["Promise"] = 9] = "Promise";
            // The TypeReferenceNode resolves to a Function type or a type with call signatures.
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["TypeWithCallSignature"] = 10] = "TypeWithCallSignature";
            // The TypeReferenceNode resolves to any other type.
            TypeReferenceSerializationKind[TypeReferenceSerializationKind["ObjectType"] = 11] = "ObjectType";
        })(TypeReferenceSerializationKind = ts.TypeReferenceSerializationKind || (ts.TypeReferenceSerializationKind = {}));
        var SymbolFlags;
        (function (SymbolFlags) {
            SymbolFlags[SymbolFlags["None"] = 0] = "None";
            SymbolFlags[SymbolFlags["FunctionScopedVariable"] = 1] = "FunctionScopedVariable";
            SymbolFlags[SymbolFlags["BlockScopedVariable"] = 2] = "BlockScopedVariable";
            SymbolFlags[SymbolFlags["Property"] = 4] = "Property";
            SymbolFlags[SymbolFlags["EnumMember"] = 8] = "EnumMember";
            SymbolFlags[SymbolFlags["Function"] = 16] = "Function";
            SymbolFlags[SymbolFlags["Class"] = 32] = "Class";
            SymbolFlags[SymbolFlags["Interface"] = 64] = "Interface";
            SymbolFlags[SymbolFlags["ConstEnum"] = 128] = "ConstEnum";
            SymbolFlags[SymbolFlags["RegularEnum"] = 256] = "RegularEnum";
            SymbolFlags[SymbolFlags["ValueModule"] = 512] = "ValueModule";
            SymbolFlags[SymbolFlags["NamespaceModule"] = 1024] = "NamespaceModule";
            SymbolFlags[SymbolFlags["TypeLiteral"] = 2048] = "TypeLiteral";
            SymbolFlags[SymbolFlags["ObjectLiteral"] = 4096] = "ObjectLiteral";
            SymbolFlags[SymbolFlags["Method"] = 8192] = "Method";
            SymbolFlags[SymbolFlags["Constructor"] = 16384] = "Constructor";
            SymbolFlags[SymbolFlags["GetAccessor"] = 32768] = "GetAccessor";
            SymbolFlags[SymbolFlags["SetAccessor"] = 65536] = "SetAccessor";
            SymbolFlags[SymbolFlags["Signature"] = 131072] = "Signature";
            SymbolFlags[SymbolFlags["TypeParameter"] = 262144] = "TypeParameter";
            SymbolFlags[SymbolFlags["TypeAlias"] = 524288] = "TypeAlias";
            SymbolFlags[SymbolFlags["ExportValue"] = 1048576] = "ExportValue";
            SymbolFlags[SymbolFlags["Alias"] = 2097152] = "Alias";
            SymbolFlags[SymbolFlags["Prototype"] = 4194304] = "Prototype";
            SymbolFlags[SymbolFlags["ExportStar"] = 8388608] = "ExportStar";
            SymbolFlags[SymbolFlags["Optional"] = 16777216] = "Optional";
            SymbolFlags[SymbolFlags["Transient"] = 33554432] = "Transient";
            SymbolFlags[SymbolFlags["Assignment"] = 67108864] = "Assignment";
            SymbolFlags[SymbolFlags["ModuleExports"] = 134217728] = "ModuleExports";
            /* @internal */
            SymbolFlags[SymbolFlags["All"] = 67108863] = "All";
            SymbolFlags[SymbolFlags["Enum"] = 384] = "Enum";
            SymbolFlags[SymbolFlags["Variable"] = 3] = "Variable";
            SymbolFlags[SymbolFlags["Value"] = 111551] = "Value";
            SymbolFlags[SymbolFlags["Type"] = 788968] = "Type";
            SymbolFlags[SymbolFlags["Namespace"] = 1920] = "Namespace";
            SymbolFlags[SymbolFlags["Module"] = 1536] = "Module";
            SymbolFlags[SymbolFlags["Accessor"] = 98304] = "Accessor";
            // Variables can be redeclared, but can not redeclare a block-scoped declaration with the
            // same name, or any other value that is not a variable, e.g. ValueModule or Class
            SymbolFlags[SymbolFlags["FunctionScopedVariableExcludes"] = 111550] = "FunctionScopedVariableExcludes";
            // Block-scoped declarations are not allowed to be re-declared
            // they can not merge with anything in the value space
            SymbolFlags[SymbolFlags["BlockScopedVariableExcludes"] = 111551] = "BlockScopedVariableExcludes";
            SymbolFlags[SymbolFlags["ParameterExcludes"] = 111551] = "ParameterExcludes";
            SymbolFlags[SymbolFlags["PropertyExcludes"] = 0] = "PropertyExcludes";
            SymbolFlags[SymbolFlags["EnumMemberExcludes"] = 900095] = "EnumMemberExcludes";
            SymbolFlags[SymbolFlags["FunctionExcludes"] = 110991] = "FunctionExcludes";
            SymbolFlags[SymbolFlags["ClassExcludes"] = 899503] = "ClassExcludes";
            SymbolFlags[SymbolFlags["InterfaceExcludes"] = 788872] = "InterfaceExcludes";
            SymbolFlags[SymbolFlags["RegularEnumExcludes"] = 899327] = "RegularEnumExcludes";
            SymbolFlags[SymbolFlags["ConstEnumExcludes"] = 899967] = "ConstEnumExcludes";
            SymbolFlags[SymbolFlags["ValueModuleExcludes"] = 110735] = "ValueModuleExcludes";
            SymbolFlags[SymbolFlags["NamespaceModuleExcludes"] = 0] = "NamespaceModuleExcludes";
            SymbolFlags[SymbolFlags["MethodExcludes"] = 103359] = "MethodExcludes";
            SymbolFlags[SymbolFlags["GetAccessorExcludes"] = 46015] = "GetAccessorExcludes";
            SymbolFlags[SymbolFlags["SetAccessorExcludes"] = 78783] = "SetAccessorExcludes";
            SymbolFlags[SymbolFlags["TypeParameterExcludes"] = 526824] = "TypeParameterExcludes";
            SymbolFlags[SymbolFlags["TypeAliasExcludes"] = 788968] = "TypeAliasExcludes";
            SymbolFlags[SymbolFlags["AliasExcludes"] = 2097152] = "AliasExcludes";
            SymbolFlags[SymbolFlags["ModuleMember"] = 2623475] = "ModuleMember";
            SymbolFlags[SymbolFlags["ExportHasLocal"] = 944] = "ExportHasLocal";
            SymbolFlags[SymbolFlags["BlockScoped"] = 418] = "BlockScoped";
            SymbolFlags[SymbolFlags["PropertyOrAccessor"] = 98308] = "PropertyOrAccessor";
            SymbolFlags[SymbolFlags["ClassMember"] = 106500] = "ClassMember";
            /* @internal */
            SymbolFlags[SymbolFlags["ExportSupportsDefaultModifier"] = 112] = "ExportSupportsDefaultModifier";
            /* @internal */
            SymbolFlags[SymbolFlags["ExportDoesNotSupportDefaultModifier"] = -113] = "ExportDoesNotSupportDefaultModifier";
            /* @internal */
            // The set of things we consider semantically classifiable.  Used to speed up the LS during
            // classification.
            SymbolFlags[SymbolFlags["Classifiable"] = 2885600] = "Classifiable";
            /* @internal */
            SymbolFlags[SymbolFlags["LateBindingContainer"] = 6256] = "LateBindingContainer";
        })(SymbolFlags = ts.SymbolFlags || (ts.SymbolFlags = {}));
        /* @internal */
        var EnumKind;
        (function (EnumKind) {
            EnumKind[EnumKind["Numeric"] = 0] = "Numeric";
            EnumKind[EnumKind["Literal"] = 1] = "Literal"; // Literal enum (each member has a TypeFlags.EnumLiteral type)
        })(EnumKind = ts.EnumKind || (ts.EnumKind = {}));
        /* @internal */
        var CheckFlags;
        (function (CheckFlags) {
            CheckFlags[CheckFlags["Instantiated"] = 1] = "Instantiated";
            CheckFlags[CheckFlags["SyntheticProperty"] = 2] = "SyntheticProperty";
            CheckFlags[CheckFlags["SyntheticMethod"] = 4] = "SyntheticMethod";
            CheckFlags[CheckFlags["Readonly"] = 8] = "Readonly";
            CheckFlags[CheckFlags["ReadPartial"] = 16] = "ReadPartial";
            CheckFlags[CheckFlags["WritePartial"] = 32] = "WritePartial";
            CheckFlags[CheckFlags["HasNonUniformType"] = 64] = "HasNonUniformType";
            CheckFlags[CheckFlags["HasLiteralType"] = 128] = "HasLiteralType";
            CheckFlags[CheckFlags["ContainsPublic"] = 256] = "ContainsPublic";
            CheckFlags[CheckFlags["ContainsProtected"] = 512] = "ContainsProtected";
            CheckFlags[CheckFlags["ContainsPrivate"] = 1024] = "ContainsPrivate";
            CheckFlags[CheckFlags["ContainsStatic"] = 2048] = "ContainsStatic";
            CheckFlags[CheckFlags["Late"] = 4096] = "Late";
            CheckFlags[CheckFlags["ReverseMapped"] = 8192] = "ReverseMapped";
            CheckFlags[CheckFlags["OptionalParameter"] = 16384] = "OptionalParameter";
            CheckFlags[CheckFlags["RestParameter"] = 32768] = "RestParameter";
            CheckFlags[CheckFlags["DeferredType"] = 65536] = "DeferredType";
            CheckFlags[CheckFlags["Synthetic"] = 6] = "Synthetic";
            CheckFlags[CheckFlags["Discriminant"] = 192] = "Discriminant";
            CheckFlags[CheckFlags["Partial"] = 48] = "Partial";
        })(CheckFlags = ts.CheckFlags || (ts.CheckFlags = {}));
        var InternalSymbolName;
        (function (InternalSymbolName) {
            InternalSymbolName["Call"] = "__call";
            InternalSymbolName["Constructor"] = "__constructor";
            InternalSymbolName["New"] = "__new";
            InternalSymbolName["Index"] = "__index";
            InternalSymbolName["ExportStar"] = "__export";
            InternalSymbolName["Global"] = "__global";
            InternalSymbolName["Missing"] = "__missing";
            InternalSymbolName["Type"] = "__type";
            InternalSymbolName["Object"] = "__object";
            InternalSymbolName["JSXAttributes"] = "__jsxAttributes";
            InternalSymbolName["Class"] = "__class";
            InternalSymbolName["Function"] = "__function";
            InternalSymbolName["Computed"] = "__computed";
            InternalSymbolName["Resolving"] = "__resolving__";
            InternalSymbolName["ExportEquals"] = "export=";
            InternalSymbolName["Default"] = "default";
            InternalSymbolName["This"] = "this";
        })(InternalSymbolName = ts.InternalSymbolName || (ts.InternalSymbolName = {}));
        /* @internal */
        var NodeCheckFlags;
        (function (NodeCheckFlags) {
            NodeCheckFlags[NodeCheckFlags["TypeChecked"] = 1] = "TypeChecked";
            NodeCheckFlags[NodeCheckFlags["LexicalThis"] = 2] = "LexicalThis";
            NodeCheckFlags[NodeCheckFlags["CaptureThis"] = 4] = "CaptureThis";
            NodeCheckFlags[NodeCheckFlags["CaptureNewTarget"] = 8] = "CaptureNewTarget";
            NodeCheckFlags[NodeCheckFlags["SuperInstance"] = 256] = "SuperInstance";
            NodeCheckFlags[NodeCheckFlags["SuperStatic"] = 512] = "SuperStatic";
            NodeCheckFlags[NodeCheckFlags["ContextChecked"] = 1024] = "ContextChecked";
            NodeCheckFlags[NodeCheckFlags["AsyncMethodWithSuper"] = 2048] = "AsyncMethodWithSuper";
            NodeCheckFlags[NodeCheckFlags["AsyncMethodWithSuperBinding"] = 4096] = "AsyncMethodWithSuperBinding";
            NodeCheckFlags[NodeCheckFlags["CaptureArguments"] = 8192] = "CaptureArguments";
            NodeCheckFlags[NodeCheckFlags["EnumValuesComputed"] = 16384] = "EnumValuesComputed";
            NodeCheckFlags[NodeCheckFlags["LexicalModuleMergesWithClass"] = 32768] = "LexicalModuleMergesWithClass";
            NodeCheckFlags[NodeCheckFlags["LoopWithCapturedBlockScopedBinding"] = 65536] = "LoopWithCapturedBlockScopedBinding";
            NodeCheckFlags[NodeCheckFlags["ContainsCapturedBlockScopeBinding"] = 131072] = "ContainsCapturedBlockScopeBinding";
            NodeCheckFlags[NodeCheckFlags["CapturedBlockScopedBinding"] = 262144] = "CapturedBlockScopedBinding";
            NodeCheckFlags[NodeCheckFlags["BlockScopedBindingInLoop"] = 524288] = "BlockScopedBindingInLoop";
            NodeCheckFlags[NodeCheckFlags["ClassWithBodyScopedClassBinding"] = 1048576] = "ClassWithBodyScopedClassBinding";
            NodeCheckFlags[NodeCheckFlags["BodyScopedClassBinding"] = 2097152] = "BodyScopedClassBinding";
            NodeCheckFlags[NodeCheckFlags["NeedsLoopOutParameter"] = 4194304] = "NeedsLoopOutParameter";
            NodeCheckFlags[NodeCheckFlags["AssignmentsMarked"] = 8388608] = "AssignmentsMarked";
            NodeCheckFlags[NodeCheckFlags["ClassWithConstructorReference"] = 16777216] = "ClassWithConstructorReference";
            NodeCheckFlags[NodeCheckFlags["ConstructorReferenceInClass"] = 33554432] = "ConstructorReferenceInClass";
            NodeCheckFlags[NodeCheckFlags["ContainsClassWithPrivateIdentifiers"] = 67108864] = "ContainsClassWithPrivateIdentifiers";
        })(NodeCheckFlags = ts.NodeCheckFlags || (ts.NodeCheckFlags = {}));
        var TypeFlags;
        (function (TypeFlags) {
            TypeFlags[TypeFlags["Any"] = 1] = "Any";
            TypeFlags[TypeFlags["Unknown"] = 2] = "Unknown";
            TypeFlags[TypeFlags["String"] = 4] = "String";
            TypeFlags[TypeFlags["Number"] = 8] = "Number";
            TypeFlags[TypeFlags["Boolean"] = 16] = "Boolean";
            TypeFlags[TypeFlags["Enum"] = 32] = "Enum";
            TypeFlags[TypeFlags["BigInt"] = 64] = "BigInt";
            TypeFlags[TypeFlags["StringLiteral"] = 128] = "StringLiteral";
            TypeFlags[TypeFlags["NumberLiteral"] = 256] = "NumberLiteral";
            TypeFlags[TypeFlags["BooleanLiteral"] = 512] = "BooleanLiteral";
            TypeFlags[TypeFlags["EnumLiteral"] = 1024] = "EnumLiteral";
            TypeFlags[TypeFlags["BigIntLiteral"] = 2048] = "BigIntLiteral";
            TypeFlags[TypeFlags["ESSymbol"] = 4096] = "ESSymbol";
            TypeFlags[TypeFlags["UniqueESSymbol"] = 8192] = "UniqueESSymbol";
            TypeFlags[TypeFlags["Void"] = 16384] = "Void";
            TypeFlags[TypeFlags["Undefined"] = 32768] = "Undefined";
            TypeFlags[TypeFlags["Null"] = 65536] = "Null";
            TypeFlags[TypeFlags["Never"] = 131072] = "Never";
            TypeFlags[TypeFlags["TypeParameter"] = 262144] = "TypeParameter";
            TypeFlags[TypeFlags["Object"] = 524288] = "Object";
            TypeFlags[TypeFlags["Union"] = 1048576] = "Union";
            TypeFlags[TypeFlags["Intersection"] = 2097152] = "Intersection";
            TypeFlags[TypeFlags["Index"] = 4194304] = "Index";
            TypeFlags[TypeFlags["IndexedAccess"] = 8388608] = "IndexedAccess";
            TypeFlags[TypeFlags["Conditional"] = 16777216] = "Conditional";
            TypeFlags[TypeFlags["Substitution"] = 33554432] = "Substitution";
            TypeFlags[TypeFlags["NonPrimitive"] = 67108864] = "NonPrimitive";
            /* @internal */
            TypeFlags[TypeFlags["AnyOrUnknown"] = 3] = "AnyOrUnknown";
            /* @internal */
            TypeFlags[TypeFlags["Nullable"] = 98304] = "Nullable";
            TypeFlags[TypeFlags["Literal"] = 2944] = "Literal";
            TypeFlags[TypeFlags["Unit"] = 109440] = "Unit";
            TypeFlags[TypeFlags["StringOrNumberLiteral"] = 384] = "StringOrNumberLiteral";
            /* @internal */
            TypeFlags[TypeFlags["StringOrNumberLiteralOrUnique"] = 8576] = "StringOrNumberLiteralOrUnique";
            /* @internal */
            TypeFlags[TypeFlags["DefinitelyFalsy"] = 117632] = "DefinitelyFalsy";
            TypeFlags[TypeFlags["PossiblyFalsy"] = 117724] = "PossiblyFalsy";
            /* @internal */
            TypeFlags[TypeFlags["Intrinsic"] = 67359327] = "Intrinsic";
            /* @internal */
            TypeFlags[TypeFlags["Primitive"] = 131068] = "Primitive";
            TypeFlags[TypeFlags["StringLike"] = 132] = "StringLike";
            TypeFlags[TypeFlags["NumberLike"] = 296] = "NumberLike";
            TypeFlags[TypeFlags["BigIntLike"] = 2112] = "BigIntLike";
            TypeFlags[TypeFlags["BooleanLike"] = 528] = "BooleanLike";
            TypeFlags[TypeFlags["EnumLike"] = 1056] = "EnumLike";
            TypeFlags[TypeFlags["ESSymbolLike"] = 12288] = "ESSymbolLike";
            TypeFlags[TypeFlags["VoidLike"] = 49152] = "VoidLike";
            /* @internal */
            TypeFlags[TypeFlags["DisjointDomains"] = 67238908] = "DisjointDomains";
            TypeFlags[TypeFlags["UnionOrIntersection"] = 3145728] = "UnionOrIntersection";
            TypeFlags[TypeFlags["StructuredType"] = 3670016] = "StructuredType";
            TypeFlags[TypeFlags["TypeVariable"] = 8650752] = "TypeVariable";
            TypeFlags[TypeFlags["InstantiableNonPrimitive"] = 58982400] = "InstantiableNonPrimitive";
            TypeFlags[TypeFlags["InstantiablePrimitive"] = 4194304] = "InstantiablePrimitive";
            TypeFlags[TypeFlags["Instantiable"] = 63176704] = "Instantiable";
            TypeFlags[TypeFlags["StructuredOrInstantiable"] = 66846720] = "StructuredOrInstantiable";
            /* @internal */
            TypeFlags[TypeFlags["ObjectFlagsType"] = 3899393] = "ObjectFlagsType";
            /* @internal */
            TypeFlags[TypeFlags["Simplifiable"] = 25165824] = "Simplifiable";
            // 'Narrowable' types are types where narrowing actually narrows.
            // This *should* be every type other than null, undefined, void, and never
            TypeFlags[TypeFlags["Narrowable"] = 133970943] = "Narrowable";
            TypeFlags[TypeFlags["NotUnionOrUnit"] = 67637251] = "NotUnionOrUnit";
            /* @internal */
            TypeFlags[TypeFlags["NotPrimitiveUnion"] = 66994211] = "NotPrimitiveUnion";
            // The following flags are aggregated during union and intersection type construction
            /* @internal */
            TypeFlags[TypeFlags["IncludesMask"] = 68943871] = "IncludesMask";
            // The following flags are used for different purposes during union and intersection type construction
            /* @internal */
            TypeFlags[TypeFlags["IncludesStructuredOrInstantiable"] = 262144] = "IncludesStructuredOrInstantiable";
            /* @internal */
            TypeFlags[TypeFlags["IncludesNonWideningType"] = 2097152] = "IncludesNonWideningType";
            /* @internal */
            TypeFlags[TypeFlags["IncludesWildcard"] = 4194304] = "IncludesWildcard";
            /* @internal */
            TypeFlags[TypeFlags["IncludesEmptyObject"] = 8388608] = "IncludesEmptyObject";
            // The following flag is used for different purposes by maybeTypeOfKind
            /* @internal */
            TypeFlags[TypeFlags["GenericMappedType"] = 131072] = "GenericMappedType";
        })(TypeFlags = ts.TypeFlags || (ts.TypeFlags = {}));
        var ObjectFlags;
        (function (ObjectFlags) {
            ObjectFlags[ObjectFlags["Class"] = 1] = "Class";
            ObjectFlags[ObjectFlags["Interface"] = 2] = "Interface";
            ObjectFlags[ObjectFlags["Reference"] = 4] = "Reference";
            ObjectFlags[ObjectFlags["Tuple"] = 8] = "Tuple";
            ObjectFlags[ObjectFlags["Anonymous"] = 16] = "Anonymous";
            ObjectFlags[ObjectFlags["Mapped"] = 32] = "Mapped";
            ObjectFlags[ObjectFlags["Instantiated"] = 64] = "Instantiated";
            ObjectFlags[ObjectFlags["ObjectLiteral"] = 128] = "ObjectLiteral";
            ObjectFlags[ObjectFlags["EvolvingArray"] = 256] = "EvolvingArray";
            ObjectFlags[ObjectFlags["ObjectLiteralPatternWithComputedProperties"] = 512] = "ObjectLiteralPatternWithComputedProperties";
            ObjectFlags[ObjectFlags["ContainsSpread"] = 1024] = "ContainsSpread";
            ObjectFlags[ObjectFlags["ReverseMapped"] = 2048] = "ReverseMapped";
            ObjectFlags[ObjectFlags["JsxAttributes"] = 4096] = "JsxAttributes";
            ObjectFlags[ObjectFlags["MarkerType"] = 8192] = "MarkerType";
            ObjectFlags[ObjectFlags["JSLiteral"] = 16384] = "JSLiteral";
            ObjectFlags[ObjectFlags["FreshLiteral"] = 32768] = "FreshLiteral";
            ObjectFlags[ObjectFlags["ArrayLiteral"] = 65536] = "ArrayLiteral";
            ObjectFlags[ObjectFlags["ObjectRestType"] = 131072] = "ObjectRestType";
            /* @internal */
            ObjectFlags[ObjectFlags["PrimitiveUnion"] = 262144] = "PrimitiveUnion";
            /* @internal */
            ObjectFlags[ObjectFlags["ContainsWideningType"] = 524288] = "ContainsWideningType";
            /* @internal */
            ObjectFlags[ObjectFlags["ContainsObjectOrArrayLiteral"] = 1048576] = "ContainsObjectOrArrayLiteral";
            /* @internal */
            ObjectFlags[ObjectFlags["NonInferrableType"] = 2097152] = "NonInferrableType";
            ObjectFlags[ObjectFlags["ClassOrInterface"] = 3] = "ClassOrInterface";
            /* @internal */
            ObjectFlags[ObjectFlags["RequiresWidening"] = 1572864] = "RequiresWidening";
            /* @internal */
            ObjectFlags[ObjectFlags["PropagatingFlags"] = 3670016] = "PropagatingFlags";
        })(ObjectFlags = ts.ObjectFlags || (ts.ObjectFlags = {}));
        /* @internal */
        var VarianceFlags;
        (function (VarianceFlags) {
            VarianceFlags[VarianceFlags["Invariant"] = 0] = "Invariant";
            VarianceFlags[VarianceFlags["Covariant"] = 1] = "Covariant";
            VarianceFlags[VarianceFlags["Contravariant"] = 2] = "Contravariant";
            VarianceFlags[VarianceFlags["Bivariant"] = 3] = "Bivariant";
            VarianceFlags[VarianceFlags["Independent"] = 4] = "Independent";
            VarianceFlags[VarianceFlags["VarianceMask"] = 7] = "VarianceMask";
            VarianceFlags[VarianceFlags["Unmeasurable"] = 8] = "Unmeasurable";
            VarianceFlags[VarianceFlags["Unreliable"] = 16] = "Unreliable";
            VarianceFlags[VarianceFlags["AllowsStructuralFallback"] = 24] = "AllowsStructuralFallback";
        })(VarianceFlags = ts.VarianceFlags || (ts.VarianceFlags = {}));
        /* @internal */
        var JsxReferenceKind;
        (function (JsxReferenceKind) {
            JsxReferenceKind[JsxReferenceKind["Component"] = 0] = "Component";
            JsxReferenceKind[JsxReferenceKind["Function"] = 1] = "Function";
            JsxReferenceKind[JsxReferenceKind["Mixed"] = 2] = "Mixed";
        })(JsxReferenceKind = ts.JsxReferenceKind || (ts.JsxReferenceKind = {}));
        var SignatureKind;
        (function (SignatureKind) {
            SignatureKind[SignatureKind["Call"] = 0] = "Call";
            SignatureKind[SignatureKind["Construct"] = 1] = "Construct";
        })(SignatureKind = ts.SignatureKind || (ts.SignatureKind = {}));
        /* @internal */
        var SignatureFlags;
        (function (SignatureFlags) {
            SignatureFlags[SignatureFlags["None"] = 0] = "None";
            SignatureFlags[SignatureFlags["HasRestParameter"] = 1] = "HasRestParameter";
            SignatureFlags[SignatureFlags["HasLiteralTypes"] = 2] = "HasLiteralTypes";
            SignatureFlags[SignatureFlags["IsInnerCallChain"] = 4] = "IsInnerCallChain";
            SignatureFlags[SignatureFlags["IsOuterCallChain"] = 8] = "IsOuterCallChain";
            // We do not propagate `IsInnerCallChain` to instantiated signatures, as that would result in us
            // attempting to add `| undefined` on each recursive call to `getReturnTypeOfSignature` when
            // instantiating the return type.
            SignatureFlags[SignatureFlags["PropagatingFlags"] = 3] = "PropagatingFlags";
            SignatureFlags[SignatureFlags["CallChainFlags"] = 12] = "CallChainFlags";
        })(SignatureFlags = ts.SignatureFlags || (ts.SignatureFlags = {}));
        var IndexKind;
        (function (IndexKind) {
            IndexKind[IndexKind["String"] = 0] = "String";
            IndexKind[IndexKind["Number"] = 1] = "Number";
        })(IndexKind = ts.IndexKind || (ts.IndexKind = {}));
        var InferencePriority;
        (function (InferencePriority) {
            InferencePriority[InferencePriority["NakedTypeVariable"] = 1] = "NakedTypeVariable";
            InferencePriority[InferencePriority["HomomorphicMappedType"] = 2] = "HomomorphicMappedType";
            InferencePriority[InferencePriority["PartialHomomorphicMappedType"] = 4] = "PartialHomomorphicMappedType";
            InferencePriority[InferencePriority["MappedTypeConstraint"] = 8] = "MappedTypeConstraint";
            InferencePriority[InferencePriority["ContravariantConditional"] = 16] = "ContravariantConditional";
            InferencePriority[InferencePriority["ReturnType"] = 32] = "ReturnType";
            InferencePriority[InferencePriority["LiteralKeyof"] = 64] = "LiteralKeyof";
            InferencePriority[InferencePriority["NoConstraints"] = 128] = "NoConstraints";
            InferencePriority[InferencePriority["AlwaysStrict"] = 256] = "AlwaysStrict";
            InferencePriority[InferencePriority["MaxValue"] = 512] = "MaxValue";
            InferencePriority[InferencePriority["PriorityImpliesCombination"] = 104] = "PriorityImpliesCombination";
            InferencePriority[InferencePriority["Circularity"] = -1] = "Circularity";
        })(InferencePriority = ts.InferencePriority || (ts.InferencePriority = {}));
        /* @internal */
        var InferenceFlags;
        (function (InferenceFlags) {
            InferenceFlags[InferenceFlags["None"] = 0] = "None";
            InferenceFlags[InferenceFlags["NoDefault"] = 1] = "NoDefault";
            InferenceFlags[InferenceFlags["AnyDefault"] = 2] = "AnyDefault";
            InferenceFlags[InferenceFlags["SkippedGenericFunction"] = 4] = "SkippedGenericFunction";
        })(InferenceFlags = ts.InferenceFlags || (ts.InferenceFlags = {}));
        /**
         * Ternary values are defined such that
         * x & y is False if either x or y is False.
         * x & y is Maybe if either x or y is Maybe, but neither x or y is False.
         * x & y is True if both x and y are True.
         * x | y is False if both x and y are False.
         * x | y is Maybe if either x or y is Maybe, but neither x or y is True.
         * x | y is True if either x or y is True.
         */
        /* @internal */
        var Ternary;
        (function (Ternary) {
            Ternary[Ternary["False"] = 0] = "False";
            Ternary[Ternary["Maybe"] = 1] = "Maybe";
            Ternary[Ternary["True"] = -1] = "True";
        })(Ternary = ts.Ternary || (ts.Ternary = {}));
        /* @internal */
        var AssignmentDeclarationKind;
        (function (AssignmentDeclarationKind) {
            AssignmentDeclarationKind[AssignmentDeclarationKind["None"] = 0] = "None";
            /// exports.name = expr
            AssignmentDeclarationKind[AssignmentDeclarationKind["ExportsProperty"] = 1] = "ExportsProperty";
            /// module.exports = expr
            AssignmentDeclarationKind[AssignmentDeclarationKind["ModuleExports"] = 2] = "ModuleExports";
            /// className.prototype.name = expr
            AssignmentDeclarationKind[AssignmentDeclarationKind["PrototypeProperty"] = 3] = "PrototypeProperty";
            /// this.name = expr
            AssignmentDeclarationKind[AssignmentDeclarationKind["ThisProperty"] = 4] = "ThisProperty";
            // F.name = expr
            AssignmentDeclarationKind[AssignmentDeclarationKind["Property"] = 5] = "Property";
            // F.prototype = { ... }
            AssignmentDeclarationKind[AssignmentDeclarationKind["Prototype"] = 6] = "Prototype";
            // Object.defineProperty(x, 'name', { value: any, writable?: boolean (false by default) });
            // Object.defineProperty(x, 'name', { get: Function, set: Function });
            // Object.defineProperty(x, 'name', { get: Function });
            // Object.defineProperty(x, 'name', { set: Function });
            AssignmentDeclarationKind[AssignmentDeclarationKind["ObjectDefinePropertyValue"] = 7] = "ObjectDefinePropertyValue";
            // Object.defineProperty(exports || module.exports, 'name', ...);
            AssignmentDeclarationKind[AssignmentDeclarationKind["ObjectDefinePropertyExports"] = 8] = "ObjectDefinePropertyExports";
            // Object.defineProperty(Foo.prototype, 'name', ...);
            AssignmentDeclarationKind[AssignmentDeclarationKind["ObjectDefinePrototypeProperty"] = 9] = "ObjectDefinePrototypeProperty";
        })(AssignmentDeclarationKind = ts.AssignmentDeclarationKind || (ts.AssignmentDeclarationKind = {}));
        var DiagnosticCategory;
        (function (DiagnosticCategory) {
            DiagnosticCategory[DiagnosticCategory["Warning"] = 0] = "Warning";
            DiagnosticCategory[DiagnosticCategory["Error"] = 1] = "Error";
            DiagnosticCategory[DiagnosticCategory["Suggestion"] = 2] = "Suggestion";
            DiagnosticCategory[DiagnosticCategory["Message"] = 3] = "Message";
        })(DiagnosticCategory = ts.DiagnosticCategory || (ts.DiagnosticCategory = {}));
        /* @internal */
        function diagnosticCategoryName(d, lowerCase) {
            if (lowerCase === void 0) {
                lowerCase = true;
            }
            var name = DiagnosticCategory[d.category];
            return lowerCase ? name.toLowerCase() : name;
        }
        ts.diagnosticCategoryName = diagnosticCategoryName;
        var ModuleResolutionKind;
        (function (ModuleResolutionKind) {
            ModuleResolutionKind[ModuleResolutionKind["Classic"] = 1] = "Classic";
            ModuleResolutionKind[ModuleResolutionKind["NodeJs"] = 2] = "NodeJs";
        })(ModuleResolutionKind = ts.ModuleResolutionKind || (ts.ModuleResolutionKind = {}));
        var WatchFileKind;
        (function (WatchFileKind) {
            WatchFileKind[WatchFileKind["FixedPollingInterval"] = 0] = "FixedPollingInterval";
            WatchFileKind[WatchFileKind["PriorityPollingInterval"] = 1] = "PriorityPollingInterval";
            WatchFileKind[WatchFileKind["DynamicPriorityPolling"] = 2] = "DynamicPriorityPolling";
            WatchFileKind[WatchFileKind["UseFsEvents"] = 3] = "UseFsEvents";
            WatchFileKind[WatchFileKind["UseFsEventsOnParentDirectory"] = 4] = "UseFsEventsOnParentDirectory";
        })(WatchFileKind = ts.WatchFileKind || (ts.WatchFileKind = {}));
        var WatchDirectoryKind;
        (function (WatchDirectoryKind) {
            WatchDirectoryKind[WatchDirectoryKind["UseFsEvents"] = 0] = "UseFsEvents";
            WatchDirectoryKind[WatchDirectoryKind["FixedPollingInterval"] = 1] = "FixedPollingInterval";
            WatchDirectoryKind[WatchDirectoryKind["DynamicPriorityPolling"] = 2] = "DynamicPriorityPolling";
        })(WatchDirectoryKind = ts.WatchDirectoryKind || (ts.WatchDirectoryKind = {}));
        var PollingWatchKind;
        (function (PollingWatchKind) {
            PollingWatchKind[PollingWatchKind["FixedInterval"] = 0] = "FixedInterval";
            PollingWatchKind[PollingWatchKind["PriorityInterval"] = 1] = "PriorityInterval";
            PollingWatchKind[PollingWatchKind["DynamicPriority"] = 2] = "DynamicPriority";
        })(PollingWatchKind = ts.PollingWatchKind || (ts.PollingWatchKind = {}));
        var ModuleKind;
        (function (ModuleKind) {
            ModuleKind[ModuleKind["None"] = 0] = "None";
            ModuleKind[ModuleKind["CommonJS"] = 1] = "CommonJS";
            ModuleKind[ModuleKind["AMD"] = 2] = "AMD";
            ModuleKind[ModuleKind["UMD"] = 3] = "UMD";
            ModuleKind[ModuleKind["System"] = 4] = "System";
            // NOTE: ES module kinds should be contiguous to more easily check whether a module kind is *any* ES module kind.
            //       Non-ES module kinds should not come between ES2015 (the earliest ES module kind) and ESNext (the last ES
            //       module kind).
            ModuleKind[ModuleKind["ES2015"] = 5] = "ES2015";
            ModuleKind[ModuleKind["ES2020"] = 6] = "ES2020";
            ModuleKind[ModuleKind["ESNext"] = 99] = "ESNext";
        })(ModuleKind = ts.ModuleKind || (ts.ModuleKind = {}));
        var JsxEmit;
        (function (JsxEmit) {
            JsxEmit[JsxEmit["None"] = 0] = "None";
            JsxEmit[JsxEmit["Preserve"] = 1] = "Preserve";
            JsxEmit[JsxEmit["React"] = 2] = "React";
            JsxEmit[JsxEmit["ReactNative"] = 3] = "ReactNative";
        })(JsxEmit = ts.JsxEmit || (ts.JsxEmit = {}));
        var ImportsNotUsedAsValues;
        (function (ImportsNotUsedAsValues) {
            ImportsNotUsedAsValues[ImportsNotUsedAsValues["Remove"] = 0] = "Remove";
            ImportsNotUsedAsValues[ImportsNotUsedAsValues["Preserve"] = 1] = "Preserve";
            ImportsNotUsedAsValues[ImportsNotUsedAsValues["Error"] = 2] = "Error";
        })(ImportsNotUsedAsValues = ts.ImportsNotUsedAsValues || (ts.ImportsNotUsedAsValues = {}));
        var NewLineKind;
        (function (NewLineKind) {
            NewLineKind[NewLineKind["CarriageReturnLineFeed"] = 0] = "CarriageReturnLineFeed";
            NewLineKind[NewLineKind["LineFeed"] = 1] = "LineFeed";
        })(NewLineKind = ts.NewLineKind || (ts.NewLineKind = {}));
        var ScriptKind;
        (function (ScriptKind) {
            ScriptKind[ScriptKind["Unknown"] = 0] = "Unknown";
            ScriptKind[ScriptKind["JS"] = 1] = "JS";
            ScriptKind[ScriptKind["JSX"] = 2] = "JSX";
            ScriptKind[ScriptKind["TS"] = 3] = "TS";
            ScriptKind[ScriptKind["TSX"] = 4] = "TSX";
            ScriptKind[ScriptKind["External"] = 5] = "External";
            ScriptKind[ScriptKind["JSON"] = 6] = "JSON";
            /**
             * Used on extensions that doesn't define the ScriptKind but the content defines it.
             * Deferred extensions are going to be included in all project contexts.
             */
            ScriptKind[ScriptKind["Deferred"] = 7] = "Deferred";
        })(ScriptKind = ts.ScriptKind || (ts.ScriptKind = {}));
        var ScriptTarget;
        (function (ScriptTarget) {
            ScriptTarget[ScriptTarget["ES3"] = 0] = "ES3";
            ScriptTarget[ScriptTarget["ES5"] = 1] = "ES5";
            ScriptTarget[ScriptTarget["ES2015"] = 2] = "ES2015";
            ScriptTarget[ScriptTarget["ES2016"] = 3] = "ES2016";
            ScriptTarget[ScriptTarget["ES2017"] = 4] = "ES2017";
            ScriptTarget[ScriptTarget["ES2018"] = 5] = "ES2018";
            ScriptTarget[ScriptTarget["ES2019"] = 6] = "ES2019";
            ScriptTarget[ScriptTarget["ES2020"] = 7] = "ES2020";
            ScriptTarget[ScriptTarget["ESNext"] = 99] = "ESNext";
            ScriptTarget[ScriptTarget["JSON"] = 100] = "JSON";
            ScriptTarget[ScriptTarget["Latest"] = 99] = "Latest";
        })(ScriptTarget = ts.ScriptTarget || (ts.ScriptTarget = {}));
        var LanguageVariant;
        (function (LanguageVariant) {
            LanguageVariant[LanguageVariant["Standard"] = 0] = "Standard";
            LanguageVariant[LanguageVariant["JSX"] = 1] = "JSX";
        })(LanguageVariant = ts.LanguageVariant || (ts.LanguageVariant = {}));
        var WatchDirectoryFlags;
        (function (WatchDirectoryFlags) {
            WatchDirectoryFlags[WatchDirectoryFlags["None"] = 0] = "None";
            WatchDirectoryFlags[WatchDirectoryFlags["Recursive"] = 1] = "Recursive";
        })(WatchDirectoryFlags = ts.WatchDirectoryFlags || (ts.WatchDirectoryFlags = {}));
        /* @internal */
        var CharacterCodes;
        (function (CharacterCodes) {
            CharacterCodes[CharacterCodes["nullCharacter"] = 0] = "nullCharacter";
            CharacterCodes[CharacterCodes["maxAsciiCharacter"] = 127] = "maxAsciiCharacter";
            CharacterCodes[CharacterCodes["lineFeed"] = 10] = "lineFeed";
            CharacterCodes[CharacterCodes["carriageReturn"] = 13] = "carriageReturn";
            CharacterCodes[CharacterCodes["lineSeparator"] = 8232] = "lineSeparator";
            CharacterCodes[CharacterCodes["paragraphSeparator"] = 8233] = "paragraphSeparator";
            CharacterCodes[CharacterCodes["nextLine"] = 133] = "nextLine";
            // Unicode 3.0 space characters
            CharacterCodes[CharacterCodes["space"] = 32] = "space";
            CharacterCodes[CharacterCodes["nonBreakingSpace"] = 160] = "nonBreakingSpace";
            CharacterCodes[CharacterCodes["enQuad"] = 8192] = "enQuad";
            CharacterCodes[CharacterCodes["emQuad"] = 8193] = "emQuad";
            CharacterCodes[CharacterCodes["enSpace"] = 8194] = "enSpace";
            CharacterCodes[CharacterCodes["emSpace"] = 8195] = "emSpace";
            CharacterCodes[CharacterCodes["threePerEmSpace"] = 8196] = "threePerEmSpace";
            CharacterCodes[CharacterCodes["fourPerEmSpace"] = 8197] = "fourPerEmSpace";
            CharacterCodes[CharacterCodes["sixPerEmSpace"] = 8198] = "sixPerEmSpace";
            CharacterCodes[CharacterCodes["figureSpace"] = 8199] = "figureSpace";
            CharacterCodes[CharacterCodes["punctuationSpace"] = 8200] = "punctuationSpace";
            CharacterCodes[CharacterCodes["thinSpace"] = 8201] = "thinSpace";
            CharacterCodes[CharacterCodes["hairSpace"] = 8202] = "hairSpace";
            CharacterCodes[CharacterCodes["zeroWidthSpace"] = 8203] = "zeroWidthSpace";
            CharacterCodes[CharacterCodes["narrowNoBreakSpace"] = 8239] = "narrowNoBreakSpace";
            CharacterCodes[CharacterCodes["ideographicSpace"] = 12288] = "ideographicSpace";
            CharacterCodes[CharacterCodes["mathematicalSpace"] = 8287] = "mathematicalSpace";
            CharacterCodes[CharacterCodes["ogham"] = 5760] = "ogham";
            CharacterCodes[CharacterCodes["_"] = 95] = "_";
            CharacterCodes[CharacterCodes["$"] = 36] = "$";
            CharacterCodes[CharacterCodes["_0"] = 48] = "_0";
            CharacterCodes[CharacterCodes["_1"] = 49] = "_1";
            CharacterCodes[CharacterCodes["_2"] = 50] = "_2";
            CharacterCodes[CharacterCodes["_3"] = 51] = "_3";
            CharacterCodes[CharacterCodes["_4"] = 52] = "_4";
            CharacterCodes[CharacterCodes["_5"] = 53] = "_5";
            CharacterCodes[CharacterCodes["_6"] = 54] = "_6";
            CharacterCodes[CharacterCodes["_7"] = 55] = "_7";
            CharacterCodes[CharacterCodes["_8"] = 56] = "_8";
            CharacterCodes[CharacterCodes["_9"] = 57] = "_9";
            CharacterCodes[CharacterCodes["a"] = 97] = "a";
            CharacterCodes[CharacterCodes["b"] = 98] = "b";
            CharacterCodes[CharacterCodes["c"] = 99] = "c";
            CharacterCodes[CharacterCodes["d"] = 100] = "d";
            CharacterCodes[CharacterCodes["e"] = 101] = "e";
            CharacterCodes[CharacterCodes["f"] = 102] = "f";
            CharacterCodes[CharacterCodes["g"] = 103] = "g";
            CharacterCodes[CharacterCodes["h"] = 104] = "h";
            CharacterCodes[CharacterCodes["i"] = 105] = "i";
            CharacterCodes[CharacterCodes["j"] = 106] = "j";
            CharacterCodes[CharacterCodes["k"] = 107] = "k";
            CharacterCodes[CharacterCodes["l"] = 108] = "l";
            CharacterCodes[CharacterCodes["m"] = 109] = "m";
            CharacterCodes[CharacterCodes["n"] = 110] = "n";
            CharacterCodes[CharacterCodes["o"] = 111] = "o";
            CharacterCodes[CharacterCodes["p"] = 112] = "p";
            CharacterCodes[CharacterCodes["q"] = 113] = "q";
            CharacterCodes[CharacterCodes["r"] = 114] = "r";
            CharacterCodes[CharacterCodes["s"] = 115] = "s";
            CharacterCodes[CharacterCodes["t"] = 116] = "t";
            CharacterCodes[CharacterCodes["u"] = 117] = "u";
            CharacterCodes[CharacterCodes["v"] = 118] = "v";
            CharacterCodes[CharacterCodes["w"] = 119] = "w";
            CharacterCodes[CharacterCodes["x"] = 120] = "x";
            CharacterCodes[CharacterCodes["y"] = 121] = "y";
            CharacterCodes[CharacterCodes["z"] = 122] = "z";
            CharacterCodes[CharacterCodes["A"] = 65] = "A";
            CharacterCodes[CharacterCodes["B"] = 66] = "B";
            CharacterCodes[CharacterCodes["C"] = 67] = "C";
            CharacterCodes[CharacterCodes["D"] = 68] = "D";
            CharacterCodes[CharacterCodes["E"] = 69] = "E";
            CharacterCodes[CharacterCodes["F"] = 70] = "F";
            CharacterCodes[CharacterCodes["G"] = 71] = "G";
            CharacterCodes[CharacterCodes["H"] = 72] = "H";
            CharacterCodes[CharacterCodes["I"] = 73] = "I";
            CharacterCodes[CharacterCodes["J"] = 74] = "J";
            CharacterCodes[CharacterCodes["K"] = 75] = "K";
            CharacterCodes[CharacterCodes["L"] = 76] = "L";
            CharacterCodes[CharacterCodes["M"] = 77] = "M";
            CharacterCodes[CharacterCodes["N"] = 78] = "N";
            CharacterCodes[CharacterCodes["O"] = 79] = "O";
            CharacterCodes[CharacterCodes["P"] = 80] = "P";
            CharacterCodes[CharacterCodes["Q"] = 81] = "Q";
            CharacterCodes[CharacterCodes["R"] = 82] = "R";
            CharacterCodes[CharacterCodes["S"] = 83] = "S";
            CharacterCodes[CharacterCodes["T"] = 84] = "T";
            CharacterCodes[CharacterCodes["U"] = 85] = "U";
            CharacterCodes[CharacterCodes["V"] = 86] = "V";
            CharacterCodes[CharacterCodes["W"] = 87] = "W";
            CharacterCodes[CharacterCodes["X"] = 88] = "X";
            CharacterCodes[CharacterCodes["Y"] = 89] = "Y";
            CharacterCodes[CharacterCodes["Z"] = 90] = "Z";
            CharacterCodes[CharacterCodes["ampersand"] = 38] = "ampersand";
            CharacterCodes[CharacterCodes["asterisk"] = 42] = "asterisk";
            CharacterCodes[CharacterCodes["at"] = 64] = "at";
            CharacterCodes[CharacterCodes["backslash"] = 92] = "backslash";
            CharacterCodes[CharacterCodes["backtick"] = 96] = "backtick";
            CharacterCodes[CharacterCodes["bar"] = 124] = "bar";
            CharacterCodes[CharacterCodes["caret"] = 94] = "caret";
            CharacterCodes[CharacterCodes["closeBrace"] = 125] = "closeBrace";
            CharacterCodes[CharacterCodes["closeBracket"] = 93] = "closeBracket";
            CharacterCodes[CharacterCodes["closeParen"] = 41] = "closeParen";
            CharacterCodes[CharacterCodes["colon"] = 58] = "colon";
            CharacterCodes[CharacterCodes["comma"] = 44] = "comma";
            CharacterCodes[CharacterCodes["dot"] = 46] = "dot";
            CharacterCodes[CharacterCodes["doubleQuote"] = 34] = "doubleQuote";
            CharacterCodes[CharacterCodes["equals"] = 61] = "equals";
            CharacterCodes[CharacterCodes["exclamation"] = 33] = "exclamation";
            CharacterCodes[CharacterCodes["greaterThan"] = 62] = "greaterThan";
            CharacterCodes[CharacterCodes["hash"] = 35] = "hash";
            CharacterCodes[CharacterCodes["lessThan"] = 60] = "lessThan";
            CharacterCodes[CharacterCodes["minus"] = 45] = "minus";
            CharacterCodes[CharacterCodes["openBrace"] = 123] = "openBrace";
            CharacterCodes[CharacterCodes["openBracket"] = 91] = "openBracket";
            CharacterCodes[CharacterCodes["openParen"] = 40] = "openParen";
            CharacterCodes[CharacterCodes["percent"] = 37] = "percent";
            CharacterCodes[CharacterCodes["plus"] = 43] = "plus";
            CharacterCodes[CharacterCodes["question"] = 63] = "question";
            CharacterCodes[CharacterCodes["semicolon"] = 59] = "semicolon";
            CharacterCodes[CharacterCodes["singleQuote"] = 39] = "singleQuote";
            CharacterCodes[CharacterCodes["slash"] = 47] = "slash";
            CharacterCodes[CharacterCodes["tilde"] = 126] = "tilde";
            CharacterCodes[CharacterCodes["backspace"] = 8] = "backspace";
            CharacterCodes[CharacterCodes["formFeed"] = 12] = "formFeed";
            CharacterCodes[CharacterCodes["byteOrderMark"] = 65279] = "byteOrderMark";
            CharacterCodes[CharacterCodes["tab"] = 9] = "tab";
            CharacterCodes[CharacterCodes["verticalTab"] = 11] = "verticalTab";
        })(CharacterCodes = ts.CharacterCodes || (ts.CharacterCodes = {}));
        var Extension;
        (function (Extension) {
            Extension["Ts"] = ".ts";
            Extension["Tsx"] = ".tsx";
            Extension["Dts"] = ".d.ts";
            Extension["Js"] = ".js";
            Extension["Jsx"] = ".jsx";
            Extension["Json"] = ".json";
            Extension["TsBuildInfo"] = ".tsbuildinfo";
        })(Extension = ts.Extension || (ts.Extension = {}));
        /* @internal */
        var TransformFlags;
        (function (TransformFlags) {
            TransformFlags[TransformFlags["None"] = 0] = "None";
            // Facts
            // - Flags used to indicate that a node or subtree contains syntax that requires transformation.
            TransformFlags[TransformFlags["ContainsTypeScript"] = 1] = "ContainsTypeScript";
            TransformFlags[TransformFlags["ContainsJsx"] = 2] = "ContainsJsx";
            TransformFlags[TransformFlags["ContainsESNext"] = 4] = "ContainsESNext";
            TransformFlags[TransformFlags["ContainsES2020"] = 8] = "ContainsES2020";
            TransformFlags[TransformFlags["ContainsES2019"] = 16] = "ContainsES2019";
            TransformFlags[TransformFlags["ContainsES2018"] = 32] = "ContainsES2018";
            TransformFlags[TransformFlags["ContainsES2017"] = 64] = "ContainsES2017";
            TransformFlags[TransformFlags["ContainsES2016"] = 128] = "ContainsES2016";
            TransformFlags[TransformFlags["ContainsES2015"] = 256] = "ContainsES2015";
            TransformFlags[TransformFlags["ContainsGenerator"] = 512] = "ContainsGenerator";
            TransformFlags[TransformFlags["ContainsDestructuringAssignment"] = 1024] = "ContainsDestructuringAssignment";
            // Markers
            // - Flags used to indicate that a subtree contains a specific transformation.
            TransformFlags[TransformFlags["ContainsTypeScriptClassSyntax"] = 2048] = "ContainsTypeScriptClassSyntax";
            TransformFlags[TransformFlags["ContainsLexicalThis"] = 4096] = "ContainsLexicalThis";
            TransformFlags[TransformFlags["ContainsRestOrSpread"] = 8192] = "ContainsRestOrSpread";
            TransformFlags[TransformFlags["ContainsObjectRestOrSpread"] = 16384] = "ContainsObjectRestOrSpread";
            TransformFlags[TransformFlags["ContainsComputedPropertyName"] = 32768] = "ContainsComputedPropertyName";
            TransformFlags[TransformFlags["ContainsBlockScopedBinding"] = 65536] = "ContainsBlockScopedBinding";
            TransformFlags[TransformFlags["ContainsBindingPattern"] = 131072] = "ContainsBindingPattern";
            TransformFlags[TransformFlags["ContainsYield"] = 262144] = "ContainsYield";
            TransformFlags[TransformFlags["ContainsAwait"] = 524288] = "ContainsAwait";
            TransformFlags[TransformFlags["ContainsHoistedDeclarationOrCompletion"] = 1048576] = "ContainsHoistedDeclarationOrCompletion";
            TransformFlags[TransformFlags["ContainsDynamicImport"] = 2097152] = "ContainsDynamicImport";
            TransformFlags[TransformFlags["ContainsClassFields"] = 4194304] = "ContainsClassFields";
            // Please leave this as 1 << 29.
            // It is the maximum bit we can set before we outgrow the size of a v8 small integer (SMI) on an x86 system.
            // It is a good reminder of how much room we have left
            TransformFlags[TransformFlags["HasComputedFlags"] = 536870912] = "HasComputedFlags";
            // Assertions
            // - Bitmasks that are used to assert facts about the syntax of a node and its subtree.
            TransformFlags[TransformFlags["AssertTypeScript"] = 1] = "AssertTypeScript";
            TransformFlags[TransformFlags["AssertJsx"] = 2] = "AssertJsx";
            TransformFlags[TransformFlags["AssertESNext"] = 4] = "AssertESNext";
            TransformFlags[TransformFlags["AssertES2020"] = 8] = "AssertES2020";
            TransformFlags[TransformFlags["AssertES2019"] = 16] = "AssertES2019";
            TransformFlags[TransformFlags["AssertES2018"] = 32] = "AssertES2018";
            TransformFlags[TransformFlags["AssertES2017"] = 64] = "AssertES2017";
            TransformFlags[TransformFlags["AssertES2016"] = 128] = "AssertES2016";
            TransformFlags[TransformFlags["AssertES2015"] = 256] = "AssertES2015";
            TransformFlags[TransformFlags["AssertGenerator"] = 512] = "AssertGenerator";
            TransformFlags[TransformFlags["AssertDestructuringAssignment"] = 1024] = "AssertDestructuringAssignment";
            // Scope Exclusions
            // - Bitmasks that exclude flags from propagating out of a specific context
            //   into the subtree flags of their container.
            TransformFlags[TransformFlags["OuterExpressionExcludes"] = 536870912] = "OuterExpressionExcludes";
            TransformFlags[TransformFlags["PropertyAccessExcludes"] = 536870912] = "PropertyAccessExcludes";
            TransformFlags[TransformFlags["NodeExcludes"] = 536870912] = "NodeExcludes";
            TransformFlags[TransformFlags["ArrowFunctionExcludes"] = 538920960] = "ArrowFunctionExcludes";
            TransformFlags[TransformFlags["FunctionExcludes"] = 538925056] = "FunctionExcludes";
            TransformFlags[TransformFlags["ConstructorExcludes"] = 538923008] = "ConstructorExcludes";
            TransformFlags[TransformFlags["MethodOrAccessorExcludes"] = 538923008] = "MethodOrAccessorExcludes";
            TransformFlags[TransformFlags["PropertyExcludes"] = 536875008] = "PropertyExcludes";
            TransformFlags[TransformFlags["ClassExcludes"] = 536905728] = "ClassExcludes";
            TransformFlags[TransformFlags["ModuleExcludes"] = 537991168] = "ModuleExcludes";
            TransformFlags[TransformFlags["TypeExcludes"] = -2] = "TypeExcludes";
            TransformFlags[TransformFlags["ObjectLiteralExcludes"] = 536922112] = "ObjectLiteralExcludes";
            TransformFlags[TransformFlags["ArrayLiteralOrCallOrNewExcludes"] = 536879104] = "ArrayLiteralOrCallOrNewExcludes";
            TransformFlags[TransformFlags["VariableDeclarationListExcludes"] = 537018368] = "VariableDeclarationListExcludes";
            TransformFlags[TransformFlags["ParameterExcludes"] = 536870912] = "ParameterExcludes";
            TransformFlags[TransformFlags["CatchClauseExcludes"] = 536887296] = "CatchClauseExcludes";
            TransformFlags[TransformFlags["BindingPatternExcludes"] = 536879104] = "BindingPatternExcludes";
            // Propagating flags
            // - Bitmasks for flags that should propagate from a child
            TransformFlags[TransformFlags["PropertyNamePropagatingFlags"] = 4096] = "PropertyNamePropagatingFlags";
            // Masks
            // - Additional bitmasks
        })(TransformFlags = ts.TransformFlags || (ts.TransformFlags = {}));
        var EmitFlags;
        (function (EmitFlags) {
            EmitFlags[EmitFlags["None"] = 0] = "None";
            EmitFlags[EmitFlags["SingleLine"] = 1] = "SingleLine";
            EmitFlags[EmitFlags["AdviseOnEmitNode"] = 2] = "AdviseOnEmitNode";
            EmitFlags[EmitFlags["NoSubstitution"] = 4] = "NoSubstitution";
            EmitFlags[EmitFlags["CapturesThis"] = 8] = "CapturesThis";
            EmitFlags[EmitFlags["NoLeadingSourceMap"] = 16] = "NoLeadingSourceMap";
            EmitFlags[EmitFlags["NoTrailingSourceMap"] = 32] = "NoTrailingSourceMap";
            EmitFlags[EmitFlags["NoSourceMap"] = 48] = "NoSourceMap";
            EmitFlags[EmitFlags["NoNestedSourceMaps"] = 64] = "NoNestedSourceMaps";
            EmitFlags[EmitFlags["NoTokenLeadingSourceMaps"] = 128] = "NoTokenLeadingSourceMaps";
            EmitFlags[EmitFlags["NoTokenTrailingSourceMaps"] = 256] = "NoTokenTrailingSourceMaps";
            EmitFlags[EmitFlags["NoTokenSourceMaps"] = 384] = "NoTokenSourceMaps";
            EmitFlags[EmitFlags["NoLeadingComments"] = 512] = "NoLeadingComments";
            EmitFlags[EmitFlags["NoTrailingComments"] = 1024] = "NoTrailingComments";
            EmitFlags[EmitFlags["NoComments"] = 1536] = "NoComments";
            EmitFlags[EmitFlags["NoNestedComments"] = 2048] = "NoNestedComments";
            EmitFlags[EmitFlags["HelperName"] = 4096] = "HelperName";
            EmitFlags[EmitFlags["ExportName"] = 8192] = "ExportName";
            EmitFlags[EmitFlags["LocalName"] = 16384] = "LocalName";
            EmitFlags[EmitFlags["InternalName"] = 32768] = "InternalName";
            EmitFlags[EmitFlags["Indented"] = 65536] = "Indented";
            EmitFlags[EmitFlags["NoIndentation"] = 131072] = "NoIndentation";
            EmitFlags[EmitFlags["AsyncFunctionBody"] = 262144] = "AsyncFunctionBody";
            EmitFlags[EmitFlags["ReuseTempVariableScope"] = 524288] = "ReuseTempVariableScope";
            EmitFlags[EmitFlags["CustomPrologue"] = 1048576] = "CustomPrologue";
            EmitFlags[EmitFlags["NoHoisting"] = 2097152] = "NoHoisting";
            EmitFlags[EmitFlags["HasEndOfDeclarationMarker"] = 4194304] = "HasEndOfDeclarationMarker";
            EmitFlags[EmitFlags["Iterator"] = 8388608] = "Iterator";
            EmitFlags[EmitFlags["NoAsciiEscaping"] = 16777216] = "NoAsciiEscaping";
            /*@internal*/ EmitFlags[EmitFlags["TypeScriptClassWrapper"] = 33554432] = "TypeScriptClassWrapper";
            /*@internal*/ EmitFlags[EmitFlags["NeverApplyImportHelper"] = 67108864] = "NeverApplyImportHelper";
        })(EmitFlags = ts.EmitFlags || (ts.EmitFlags = {}));
        /**
         * Used by the checker, this enum keeps track of external emit helpers that should be type
         * checked.
         */
        /* @internal */
        var ExternalEmitHelpers;
        (function (ExternalEmitHelpers) {
            ExternalEmitHelpers[ExternalEmitHelpers["Extends"] = 1] = "Extends";
            ExternalEmitHelpers[ExternalEmitHelpers["Assign"] = 2] = "Assign";
            ExternalEmitHelpers[ExternalEmitHelpers["Rest"] = 4] = "Rest";
            ExternalEmitHelpers[ExternalEmitHelpers["Decorate"] = 8] = "Decorate";
            ExternalEmitHelpers[ExternalEmitHelpers["Metadata"] = 16] = "Metadata";
            ExternalEmitHelpers[ExternalEmitHelpers["Param"] = 32] = "Param";
            ExternalEmitHelpers[ExternalEmitHelpers["Awaiter"] = 64] = "Awaiter";
            ExternalEmitHelpers[ExternalEmitHelpers["Generator"] = 128] = "Generator";
            ExternalEmitHelpers[ExternalEmitHelpers["Values"] = 256] = "Values";
            ExternalEmitHelpers[ExternalEmitHelpers["Read"] = 512] = "Read";
            ExternalEmitHelpers[ExternalEmitHelpers["Spread"] = 1024] = "Spread";
            ExternalEmitHelpers[ExternalEmitHelpers["SpreadArrays"] = 2048] = "SpreadArrays";
            ExternalEmitHelpers[ExternalEmitHelpers["Await"] = 4096] = "Await";
            ExternalEmitHelpers[ExternalEmitHelpers["AsyncGenerator"] = 8192] = "AsyncGenerator";
            ExternalEmitHelpers[ExternalEmitHelpers["AsyncDelegator"] = 16384] = "AsyncDelegator";
            ExternalEmitHelpers[ExternalEmitHelpers["AsyncValues"] = 32768] = "AsyncValues";
            ExternalEmitHelpers[ExternalEmitHelpers["ExportStar"] = 65536] = "ExportStar";
            ExternalEmitHelpers[ExternalEmitHelpers["MakeTemplateObject"] = 131072] = "MakeTemplateObject";
            ExternalEmitHelpers[ExternalEmitHelpers["ClassPrivateFieldGet"] = 262144] = "ClassPrivateFieldGet";
            ExternalEmitHelpers[ExternalEmitHelpers["ClassPrivateFieldSet"] = 524288] = "ClassPrivateFieldSet";
            ExternalEmitHelpers[ExternalEmitHelpers["FirstEmitHelper"] = 1] = "FirstEmitHelper";
            ExternalEmitHelpers[ExternalEmitHelpers["LastEmitHelper"] = 524288] = "LastEmitHelper";
            // Helpers included by ES2015 for..of
            ExternalEmitHelpers[ExternalEmitHelpers["ForOfIncludes"] = 256] = "ForOfIncludes";
            // Helpers included by ES2017 for..await..of
            ExternalEmitHelpers[ExternalEmitHelpers["ForAwaitOfIncludes"] = 32768] = "ForAwaitOfIncludes";
            // Helpers included by ES2017 async generators
            ExternalEmitHelpers[ExternalEmitHelpers["AsyncGeneratorIncludes"] = 12288] = "AsyncGeneratorIncludes";
            // Helpers included by yield* in ES2017 async generators
            ExternalEmitHelpers[ExternalEmitHelpers["AsyncDelegatorIncludes"] = 53248] = "AsyncDelegatorIncludes";
            // Helpers included by ES2015 spread
            ExternalEmitHelpers[ExternalEmitHelpers["SpreadIncludes"] = 1536] = "SpreadIncludes";
        })(ExternalEmitHelpers = ts.ExternalEmitHelpers || (ts.ExternalEmitHelpers = {}));
        var EmitHint;
        (function (EmitHint) {
            EmitHint[EmitHint["SourceFile"] = 0] = "SourceFile";
            EmitHint[EmitHint["Expression"] = 1] = "Expression";
            EmitHint[EmitHint["IdentifierName"] = 2] = "IdentifierName";
            EmitHint[EmitHint["MappedTypeParameter"] = 3] = "MappedTypeParameter";
            EmitHint[EmitHint["Unspecified"] = 4] = "Unspecified";
            EmitHint[EmitHint["EmbeddedStatement"] = 5] = "EmbeddedStatement";
            EmitHint[EmitHint["JsxAttributeValue"] = 6] = "JsxAttributeValue";
        })(EmitHint = ts.EmitHint || (ts.EmitHint = {}));
        /*@internal*/
        var BundleFileSectionKind;
        (function (BundleFileSectionKind) {
            BundleFileSectionKind["Prologue"] = "prologue";
            BundleFileSectionKind["EmitHelpers"] = "emitHelpers";
            BundleFileSectionKind["NoDefaultLib"] = "no-default-lib";
            BundleFileSectionKind["Reference"] = "reference";
            BundleFileSectionKind["Type"] = "type";
            BundleFileSectionKind["Lib"] = "lib";
            BundleFileSectionKind["Prepend"] = "prepend";
            BundleFileSectionKind["Text"] = "text";
            BundleFileSectionKind["Internal"] = "internal";
            // comments?
        })(BundleFileSectionKind = ts.BundleFileSectionKind || (ts.BundleFileSectionKind = {}));
        var ListFormat;
        (function (ListFormat) {
            ListFormat[ListFormat["None"] = 0] = "None";
            // Line separators
            ListFormat[ListFormat["SingleLine"] = 0] = "SingleLine";
            ListFormat[ListFormat["MultiLine"] = 1] = "MultiLine";
            ListFormat[ListFormat["PreserveLines"] = 2] = "PreserveLines";
            ListFormat[ListFormat["LinesMask"] = 3] = "LinesMask";
            // Delimiters
            ListFormat[ListFormat["NotDelimited"] = 0] = "NotDelimited";
            ListFormat[ListFormat["BarDelimited"] = 4] = "BarDelimited";
            ListFormat[ListFormat["AmpersandDelimited"] = 8] = "AmpersandDelimited";
            ListFormat[ListFormat["CommaDelimited"] = 16] = "CommaDelimited";
            ListFormat[ListFormat["AsteriskDelimited"] = 32] = "AsteriskDelimited";
            ListFormat[ListFormat["DelimitersMask"] = 60] = "DelimitersMask";
            ListFormat[ListFormat["AllowTrailingComma"] = 64] = "AllowTrailingComma";
            // Whitespace
            ListFormat[ListFormat["Indented"] = 128] = "Indented";
            ListFormat[ListFormat["SpaceBetweenBraces"] = 256] = "SpaceBetweenBraces";
            ListFormat[ListFormat["SpaceBetweenSiblings"] = 512] = "SpaceBetweenSiblings";
            // Brackets/Braces
            ListFormat[ListFormat["Braces"] = 1024] = "Braces";
            ListFormat[ListFormat["Parenthesis"] = 2048] = "Parenthesis";
            ListFormat[ListFormat["AngleBrackets"] = 4096] = "AngleBrackets";
            ListFormat[ListFormat["SquareBrackets"] = 8192] = "SquareBrackets";
            ListFormat[ListFormat["BracketsMask"] = 15360] = "BracketsMask";
            ListFormat[ListFormat["OptionalIfUndefined"] = 16384] = "OptionalIfUndefined";
            ListFormat[ListFormat["OptionalIfEmpty"] = 32768] = "OptionalIfEmpty";
            ListFormat[ListFormat["Optional"] = 49152] = "Optional";
            // Other
            ListFormat[ListFormat["PreferNewLine"] = 65536] = "PreferNewLine";
            ListFormat[ListFormat["NoTrailingNewLine"] = 131072] = "NoTrailingNewLine";
            ListFormat[ListFormat["NoInterveningComments"] = 262144] = "NoInterveningComments";
            ListFormat[ListFormat["NoSpaceIfEmpty"] = 524288] = "NoSpaceIfEmpty";
            ListFormat[ListFormat["SingleElement"] = 1048576] = "SingleElement";
            // Precomputed Formats
            ListFormat[ListFormat["Modifiers"] = 262656] = "Modifiers";
            ListFormat[ListFormat["HeritageClauses"] = 512] = "HeritageClauses";
            ListFormat[ListFormat["SingleLineTypeLiteralMembers"] = 768] = "SingleLineTypeLiteralMembers";
            ListFormat[ListFormat["MultiLineTypeLiteralMembers"] = 32897] = "MultiLineTypeLiteralMembers";
            ListFormat[ListFormat["TupleTypeElements"] = 528] = "TupleTypeElements";
            ListFormat[ListFormat["UnionTypeConstituents"] = 516] = "UnionTypeConstituents";
            ListFormat[ListFormat["IntersectionTypeConstituents"] = 520] = "IntersectionTypeConstituents";
            ListFormat[ListFormat["ObjectBindingPatternElements"] = 525136] = "ObjectBindingPatternElements";
            ListFormat[ListFormat["ArrayBindingPatternElements"] = 524880] = "ArrayBindingPatternElements";
            ListFormat[ListFormat["ObjectLiteralExpressionProperties"] = 526226] = "ObjectLiteralExpressionProperties";
            ListFormat[ListFormat["ArrayLiteralExpressionElements"] = 8914] = "ArrayLiteralExpressionElements";
            ListFormat[ListFormat["CommaListElements"] = 528] = "CommaListElements";
            ListFormat[ListFormat["CallExpressionArguments"] = 2576] = "CallExpressionArguments";
            ListFormat[ListFormat["NewExpressionArguments"] = 18960] = "NewExpressionArguments";
            ListFormat[ListFormat["TemplateExpressionSpans"] = 262144] = "TemplateExpressionSpans";
            ListFormat[ListFormat["SingleLineBlockStatements"] = 768] = "SingleLineBlockStatements";
            ListFormat[ListFormat["MultiLineBlockStatements"] = 129] = "MultiLineBlockStatements";
            ListFormat[ListFormat["VariableDeclarationList"] = 528] = "VariableDeclarationList";
            ListFormat[ListFormat["SingleLineFunctionBodyStatements"] = 768] = "SingleLineFunctionBodyStatements";
            ListFormat[ListFormat["MultiLineFunctionBodyStatements"] = 1] = "MultiLineFunctionBodyStatements";
            ListFormat[ListFormat["ClassHeritageClauses"] = 0] = "ClassHeritageClauses";
            ListFormat[ListFormat["ClassMembers"] = 129] = "ClassMembers";
            ListFormat[ListFormat["InterfaceMembers"] = 129] = "InterfaceMembers";
            ListFormat[ListFormat["EnumMembers"] = 145] = "EnumMembers";
            ListFormat[ListFormat["CaseBlockClauses"] = 129] = "CaseBlockClauses";
            ListFormat[ListFormat["NamedImportsOrExportsElements"] = 525136] = "NamedImportsOrExportsElements";
            ListFormat[ListFormat["JsxElementOrFragmentChildren"] = 262144] = "JsxElementOrFragmentChildren";
            ListFormat[ListFormat["JsxElementAttributes"] = 262656] = "JsxElementAttributes";
            ListFormat[ListFormat["CaseOrDefaultClauseStatements"] = 163969] = "CaseOrDefaultClauseStatements";
            ListFormat[ListFormat["HeritageClauseTypes"] = 528] = "HeritageClauseTypes";
            ListFormat[ListFormat["SourceFileStatements"] = 131073] = "SourceFileStatements";
            ListFormat[ListFormat["Decorators"] = 49153] = "Decorators";
            ListFormat[ListFormat["TypeArguments"] = 53776] = "TypeArguments";
            ListFormat[ListFormat["TypeParameters"] = 53776] = "TypeParameters";
            ListFormat[ListFormat["Parameters"] = 2576] = "Parameters";
            ListFormat[ListFormat["IndexSignatureParameters"] = 8848] = "IndexSignatureParameters";
            ListFormat[ListFormat["JSDocComment"] = 33] = "JSDocComment";
        })(ListFormat = ts.ListFormat || (ts.ListFormat = {}));
        /* @internal */
        var PragmaKindFlags;
        (function (PragmaKindFlags) {
            PragmaKindFlags[PragmaKindFlags["None"] = 0] = "None";
            /**
             * Triple slash comment of the form
             * /// <pragma-name argname="value" />
             */
            PragmaKindFlags[PragmaKindFlags["TripleSlashXML"] = 1] = "TripleSlashXML";
            /**
             * Single line comment of the form
             * // @pragma-name argval1 argval2
             * or
             * /// @pragma-name argval1 argval2
             */
            PragmaKindFlags[PragmaKindFlags["SingleLine"] = 2] = "SingleLine";
            /**
             * Multiline non-jsdoc pragma of the form
             * /* @pragma-name argval1 argval2 * /
             */
            PragmaKindFlags[PragmaKindFlags["MultiLine"] = 4] = "MultiLine";
            PragmaKindFlags[PragmaKindFlags["All"] = 7] = "All";
            PragmaKindFlags[PragmaKindFlags["Default"] = 7] = "Default";
        })(PragmaKindFlags = ts.PragmaKindFlags || (ts.PragmaKindFlags = {}));
        // While not strictly a type, this is here because `PragmaMap` needs to be here to be used with `SourceFile`, and we don't
        //  fancy effectively defining it twice, once in value-space and once in type-space
        /* @internal */
        ts.commentPragmas = {
            "reference": {
                args: [
                    { name: "types", optional: true, captureSpan: true },
                    { name: "lib", optional: true, captureSpan: true },
                    { name: "path", optional: true, captureSpan: true },
                    { name: "no-default-lib", optional: true }
                ],
                kind: 1 /* TripleSlashXML */
            },
            "amd-dependency": {
                args: [{ name: "path" }, { name: "name", optional: true }],
                kind: 1 /* TripleSlashXML */
            },
            "amd-module": {
                args: [{ name: "name" }],
                kind: 1 /* TripleSlashXML */
            },
            "ts-check": {
                kind: 2 /* SingleLine */
            },
            "ts-nocheck": {
                kind: 2 /* SingleLine */
            },
            "jsx": {
                args: [{ name: "factory" }],
                kind: 4 /* MultiLine */
            },
        };
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        /**
         * djb2 hashing algorithm
         * http://www.cse.yorku.ca/~oz/hash.html
         */
        /* @internal */
        function generateDjb2Hash(data) {
            var acc = 5381;
            for (var i = 0; i < data.length; i++) {
                acc = ((acc << 5) + acc) + data.charCodeAt(i);
            }
            return acc.toString();
        }
        ts.generateDjb2Hash = generateDjb2Hash;
        /**
         * Set a high stack trace limit to provide more information in case of an error.
         * Called for command-line and server use cases.
         * Not called if TypeScript is used as a library.
         */
        /* @internal */
        function setStackTraceLimit() {
            if (Error.stackTraceLimit < 100) { // Also tests that we won't set the property if it doesn't exist.
                Error.stackTraceLimit = 100;
            }
        }
        ts.setStackTraceLimit = setStackTraceLimit;
        var FileWatcherEventKind;
        (function (FileWatcherEventKind) {
            FileWatcherEventKind[FileWatcherEventKind["Created"] = 0] = "Created";
            FileWatcherEventKind[FileWatcherEventKind["Changed"] = 1] = "Changed";
            FileWatcherEventKind[FileWatcherEventKind["Deleted"] = 2] = "Deleted";
        })(FileWatcherEventKind = ts.FileWatcherEventKind || (ts.FileWatcherEventKind = {}));
        /* @internal */
        var PollingInterval;
        (function (PollingInterval) {
            PollingInterval[PollingInterval["High"] = 2000] = "High";
            PollingInterval[PollingInterval["Medium"] = 500] = "Medium";
            PollingInterval[PollingInterval["Low"] = 250] = "Low";
        })(PollingInterval = ts.PollingInterval || (ts.PollingInterval = {}));
        /* @internal */
        ts.missingFileModifiedTime = new Date(0); // Any subsequent modification will occur after this time
        function createPollingIntervalBasedLevels(levels) {
            var _a;
            return _a = {},
                _a[PollingInterval.Low] = levels.Low,
                _a[PollingInterval.Medium] = levels.Medium,
                _a[PollingInterval.High] = levels.High,
                _a;
        }
        var defaultChunkLevels = { Low: 32, Medium: 64, High: 256 };
        var pollingChunkSize = createPollingIntervalBasedLevels(defaultChunkLevels);
        /* @internal */
        ts.unchangedPollThresholds = createPollingIntervalBasedLevels(defaultChunkLevels);
        /* @internal */
        function setCustomPollingValues(system) {
            if (!system.getEnvironmentVariable) {
                return;
            }
            var pollingIntervalChanged = setCustomLevels("TSC_WATCH_POLLINGINTERVAL", PollingInterval);
            pollingChunkSize = getCustomPollingBasedLevels("TSC_WATCH_POLLINGCHUNKSIZE", defaultChunkLevels) || pollingChunkSize;
            ts.unchangedPollThresholds = getCustomPollingBasedLevels("TSC_WATCH_UNCHANGEDPOLLTHRESHOLDS", defaultChunkLevels) || ts.unchangedPollThresholds;
            function getLevel(envVar, level) {
                return system.getEnvironmentVariable(envVar + "_" + level.toUpperCase());
            }
            function getCustomLevels(baseVariable) {
                var customLevels;
                setCustomLevel("Low");
                setCustomLevel("Medium");
                setCustomLevel("High");
                return customLevels;
                function setCustomLevel(level) {
                    var customLevel = getLevel(baseVariable, level);
                    if (customLevel) {
                        (customLevels || (customLevels = {}))[level] = Number(customLevel);
                    }
                }
            }
            function setCustomLevels(baseVariable, levels) {
                var customLevels = getCustomLevels(baseVariable);
                if (customLevels) {
                    setLevel("Low");
                    setLevel("Medium");
                    setLevel("High");
                    return true;
                }
                return false;
                function setLevel(level) {
                    levels[level] = customLevels[level] || levels[level];
                }
            }
            function getCustomPollingBasedLevels(baseVariable, defaultLevels) {
                var customLevels = getCustomLevels(baseVariable);
                return (pollingIntervalChanged || customLevels) &&
                    createPollingIntervalBasedLevels(customLevels ? __assign(__assign({}, defaultLevels), customLevels) : defaultLevels);
            }
        }
        ts.setCustomPollingValues = setCustomPollingValues;
        /* @internal */
        function createDynamicPriorityPollingWatchFile(host) {
            var watchedFiles = [];
            var changedFilesInLastPoll = [];
            var lowPollingIntervalQueue = createPollingIntervalQueue(PollingInterval.Low);
            var mediumPollingIntervalQueue = createPollingIntervalQueue(PollingInterval.Medium);
            var highPollingIntervalQueue = createPollingIntervalQueue(PollingInterval.High);
            return watchFile;
            function watchFile(fileName, callback, defaultPollingInterval) {
                var file = {
                    fileName: fileName,
                    callback: callback,
                    unchangedPolls: 0,
                    mtime: getModifiedTime(fileName)
                };
                watchedFiles.push(file);
                addToPollingIntervalQueue(file, defaultPollingInterval);
                return {
                    close: function () {
                        file.isClosed = true;
                        // Remove from watchedFiles
                        ts.unorderedRemoveItem(watchedFiles, file);
                        // Do not update polling interval queue since that will happen as part of polling
                    }
                };
            }
            function createPollingIntervalQueue(pollingInterval) {
                var queue = [];
                queue.pollingInterval = pollingInterval;
                queue.pollIndex = 0;
                queue.pollScheduled = false;
                return queue;
            }
            function pollPollingIntervalQueue(queue) {
                queue.pollIndex = pollQueue(queue, queue.pollingInterval, queue.pollIndex, pollingChunkSize[queue.pollingInterval]);
                // Set the next polling index and timeout
                if (queue.length) {
                    scheduleNextPoll(queue.pollingInterval);
                }
                else {
                    ts.Debug.assert(queue.pollIndex === 0);
                    queue.pollScheduled = false;
                }
            }
            function pollLowPollingIntervalQueue(queue) {
                // Always poll complete list of changedFilesInLastPoll
                pollQueue(changedFilesInLastPoll, PollingInterval.Low, /*pollIndex*/ 0, changedFilesInLastPoll.length);
                // Finally do the actual polling of the queue
                pollPollingIntervalQueue(queue);
                // Schedule poll if there are files in changedFilesInLastPoll but no files in the actual queue
                // as pollPollingIntervalQueue wont schedule for next poll
                if (!queue.pollScheduled && changedFilesInLastPoll.length) {
                    scheduleNextPoll(PollingInterval.Low);
                }
            }
            function pollQueue(queue, pollingInterval, pollIndex, chunkSize) {
                // Max visit would be all elements of the queue
                var needsVisit = queue.length;
                var definedValueCopyToIndex = pollIndex;
                for (var polled = 0; polled < chunkSize && needsVisit > 0; nextPollIndex(), needsVisit--) {
                    var watchedFile = queue[pollIndex];
                    if (!watchedFile) {
                        continue;
                    }
                    else if (watchedFile.isClosed) {
                        queue[pollIndex] = undefined;
                        continue;
                    }
                    polled++;
                    var fileChanged = onWatchedFileStat(watchedFile, getModifiedTime(watchedFile.fileName));
                    if (watchedFile.isClosed) {
                        // Closed watcher as part of callback
                        queue[pollIndex] = undefined;
                    }
                    else if (fileChanged) {
                        watchedFile.unchangedPolls = 0;
                        // Changed files go to changedFilesInLastPoll queue
                        if (queue !== changedFilesInLastPoll) {
                            queue[pollIndex] = undefined;
                            addChangedFileToLowPollingIntervalQueue(watchedFile);
                        }
                    }
                    else if (watchedFile.unchangedPolls !== ts.unchangedPollThresholds[pollingInterval]) {
                        watchedFile.unchangedPolls++;
                    }
                    else if (queue === changedFilesInLastPoll) {
                        // Restart unchangedPollCount for unchanged file and move to low polling interval queue
                        watchedFile.unchangedPolls = 1;
                        queue[pollIndex] = undefined;
                        addToPollingIntervalQueue(watchedFile, PollingInterval.Low);
                    }
                    else if (pollingInterval !== PollingInterval.High) {
                        watchedFile.unchangedPolls++;
                        queue[pollIndex] = undefined;
                        addToPollingIntervalQueue(watchedFile, pollingInterval === PollingInterval.Low ? PollingInterval.Medium : PollingInterval.High);
                    }
                    if (queue[pollIndex]) {
                        // Copy this file to the non hole location
                        if (definedValueCopyToIndex < pollIndex) {
                            queue[definedValueCopyToIndex] = watchedFile;
                            queue[pollIndex] = undefined;
                        }
                        definedValueCopyToIndex++;
                    }
                }
                // Return next poll index
                return pollIndex;
                function nextPollIndex() {
                    pollIndex++;
                    if (pollIndex === queue.length) {
                        if (definedValueCopyToIndex < pollIndex) {
                            // There are holes from nextDefinedValueIndex to end of queue, change queue size
                            queue.length = definedValueCopyToIndex;
                        }
                        pollIndex = 0;
                        definedValueCopyToIndex = 0;
                    }
                }
            }
            function pollingIntervalQueue(pollingInterval) {
                switch (pollingInterval) {
                    case PollingInterval.Low:
                        return lowPollingIntervalQueue;
                    case PollingInterval.Medium:
                        return mediumPollingIntervalQueue;
                    case PollingInterval.High:
                        return highPollingIntervalQueue;
                }
            }
            function addToPollingIntervalQueue(file, pollingInterval) {
                pollingIntervalQueue(pollingInterval).push(file);
                scheduleNextPollIfNotAlreadyScheduled(pollingInterval);
            }
            function addChangedFileToLowPollingIntervalQueue(file) {
                changedFilesInLastPoll.push(file);
                scheduleNextPollIfNotAlreadyScheduled(PollingInterval.Low);
            }
            function scheduleNextPollIfNotAlreadyScheduled(pollingInterval) {
                if (!pollingIntervalQueue(pollingInterval).pollScheduled) {
                    scheduleNextPoll(pollingInterval);
                }
            }
            function scheduleNextPoll(pollingInterval) {
                pollingIntervalQueue(pollingInterval).pollScheduled = host.setTimeout(pollingInterval === PollingInterval.Low ? pollLowPollingIntervalQueue : pollPollingIntervalQueue, pollingInterval, pollingIntervalQueue(pollingInterval));
            }
            function getModifiedTime(fileName) {
                return host.getModifiedTime(fileName) || ts.missingFileModifiedTime;
            }
        }
        ts.createDynamicPriorityPollingWatchFile = createDynamicPriorityPollingWatchFile;
        function createUseFsEventsOnParentDirectoryWatchFile(fsWatch, useCaseSensitiveFileNames) {
            // One file can have multiple watchers
            var fileWatcherCallbacks = ts.createMultiMap();
            var dirWatchers = ts.createMap();
            var toCanonicalName = ts.createGetCanonicalFileName(useCaseSensitiveFileNames);
            return nonPollingWatchFile;
            function nonPollingWatchFile(fileName, callback, _pollingInterval, fallbackOptions) {
                var filePath = toCanonicalName(fileName);
                fileWatcherCallbacks.add(filePath, callback);
                var dirPath = ts.getDirectoryPath(filePath) || ".";
                var watcher = dirWatchers.get(dirPath) ||
                    createDirectoryWatcher(ts.getDirectoryPath(fileName) || ".", dirPath, fallbackOptions);
                watcher.referenceCount++;
                return {
                    close: function () {
                        if (watcher.referenceCount === 1) {
                            watcher.close();
                            dirWatchers.delete(dirPath);
                        }
                        else {
                            watcher.referenceCount--;
                        }
                        fileWatcherCallbacks.remove(filePath, callback);
                    }
                };
            }
            function createDirectoryWatcher(dirName, dirPath, fallbackOptions) {
                var watcher = fsWatch(dirName, 1 /* Directory */, function (_eventName, relativeFileName) {
                    // When files are deleted from disk, the triggered "rename" event would have a relativefileName of "undefined"
                    if (!ts.isString(relativeFileName)) {
                        return;
                    }
                    var fileName = ts.getNormalizedAbsolutePath(relativeFileName, dirName);
                    // Some applications save a working file via rename operations
                    var callbacks = fileName && fileWatcherCallbacks.get(toCanonicalName(fileName));
                    if (callbacks) {
                        for (var _i = 0, callbacks_1 = callbacks; _i < callbacks_1.length; _i++) {
                            var fileCallback = callbacks_1[_i];
                            fileCallback(fileName, FileWatcherEventKind.Changed);
                        }
                    }
                }, 
                /*recursive*/ false, PollingInterval.Medium, fallbackOptions);
                watcher.referenceCount = 0;
                dirWatchers.set(dirPath, watcher);
                return watcher;
            }
        }
        /* @internal */
        function createSingleFileWatcherPerName(watchFile, useCaseSensitiveFileNames) {
            var cache = ts.createMap();
            var callbacksCache = ts.createMultiMap();
            var toCanonicalFileName = ts.createGetCanonicalFileName(useCaseSensitiveFileNames);
            return function (fileName, callback, pollingInterval, options) {
                var path = toCanonicalFileName(fileName);
                var existing = cache.get(path);
                if (existing) {
                    existing.refCount++;
                }
                else {
                    cache.set(path, {
                        watcher: watchFile(fileName, function (fileName, eventKind) { return ts.forEach(callbacksCache.get(path), function (cb) { return cb(fileName, eventKind); }); }, pollingInterval, options),
                        refCount: 1
                    });
                }
                callbacksCache.add(path, callback);
                return {
                    close: function () {
                        var watcher = ts.Debug.assertDefined(cache.get(path));
                        callbacksCache.remove(path, callback);
                        watcher.refCount--;
                        if (watcher.refCount)
                            return;
                        cache.delete(path);
                        ts.closeFileWatcherOf(watcher);
                    }
                };
            };
        }
        ts.createSingleFileWatcherPerName = createSingleFileWatcherPerName;
        /**
         * Returns true if file status changed
         */
        /*@internal*/
        function onWatchedFileStat(watchedFile, modifiedTime) {
            var oldTime = watchedFile.mtime.getTime();
            var newTime = modifiedTime.getTime();
            if (oldTime !== newTime) {
                watchedFile.mtime = modifiedTime;
                watchedFile.callback(watchedFile.fileName, getFileWatcherEventKind(oldTime, newTime));
                return true;
            }
            return false;
        }
        ts.onWatchedFileStat = onWatchedFileStat;
        /*@internal*/
        function getFileWatcherEventKind(oldTime, newTime) {
            return oldTime === 0
                ? FileWatcherEventKind.Created
                : newTime === 0
                    ? FileWatcherEventKind.Deleted
                    : FileWatcherEventKind.Changed;
        }
        ts.getFileWatcherEventKind = getFileWatcherEventKind;
        /*@internal*/
        ts.ignoredPaths = ["/node_modules/.", "/.git", "/.#"];
        /*@internal*/
        ts.sysLog = ts.noop; // eslint-disable-line prefer-const
        /*@internal*/
        function setSysLog(logger) {
            ts.sysLog = logger;
        }
        ts.setSysLog = setSysLog;
        /**
         * Watch the directory recursively using host provided method to watch child directories
         * that means if this is recursive watcher, watch the children directories as well
         * (eg on OS that dont support recursive watch using fs.watch use fs.watchFile)
         */
        /*@internal*/
        function createDirectoryWatcherSupportingRecursive(host) {
            var cache = ts.createMap();
            var callbackCache = ts.createMultiMap();
            var cacheToUpdateChildWatches = ts.createMap();
            var timerToUpdateChildWatches;
            var filePathComparer = ts.getStringComparer(!host.useCaseSensitiveFileNames);
            var toCanonicalFilePath = ts.createGetCanonicalFileName(host.useCaseSensitiveFileNames);
            return function (dirName, callback, recursive, options) {
                return recursive ?
                    createDirectoryWatcher(dirName, options, callback) :
                    host.watchDirectory(dirName, callback, recursive, options);
            };
            /**
             * Create the directory watcher for the dirPath.
             */
            function createDirectoryWatcher(dirName, options, callback) {
                var dirPath = toCanonicalFilePath(dirName);
                var directoryWatcher = cache.get(dirPath);
                if (directoryWatcher) {
                    directoryWatcher.refCount++;
                }
                else {
                    directoryWatcher = {
                        watcher: host.watchDirectory(dirName, function (fileName) {
                            if (isIgnoredPath(fileName))
                                return;
                            if (options === null || options === void 0 ? void 0 : options.synchronousWatchDirectory) {
                                // Call the actual callback
                                invokeCallbacks(dirPath, fileName);
                                // Iterate through existing children and update the watches if needed
                                updateChildWatches(dirName, dirPath, options);
                            }
                            else {
                                nonSyncUpdateChildWatches(dirName, dirPath, fileName, options);
                            }
                        }, /*recursive*/ false, options),
                        refCount: 1,
                        childWatches: ts.emptyArray
                    };
                    cache.set(dirPath, directoryWatcher);
                    updateChildWatches(dirName, dirPath, options);
                }
                var callbackToAdd = callback && { dirName: dirName, callback: callback };
                if (callbackToAdd) {
                    callbackCache.add(dirPath, callbackToAdd);
                }
                return {
                    dirName: dirName,
                    close: function () {
                        var directoryWatcher = ts.Debug.assertDefined(cache.get(dirPath));
                        if (callbackToAdd)
                            callbackCache.remove(dirPath, callbackToAdd);
                        directoryWatcher.refCount--;
                        if (directoryWatcher.refCount)
                            return;
                        cache.delete(dirPath);
                        ts.closeFileWatcherOf(directoryWatcher);
                        directoryWatcher.childWatches.forEach(ts.closeFileWatcher);
                    }
                };
            }
            function invokeCallbacks(dirPath, fileNameOrInvokeMap) {
                var fileName;
                var invokeMap;
                if (ts.isString(fileNameOrInvokeMap)) {
                    fileName = fileNameOrInvokeMap;
                }
                else {
                    invokeMap = fileNameOrInvokeMap;
                }
                // Call the actual callback
                callbackCache.forEach(function (callbacks, rootDirName) {
                    if (invokeMap && invokeMap.has(rootDirName))
                        return;
                    if (rootDirName === dirPath || (ts.startsWith(dirPath, rootDirName) && dirPath[rootDirName.length] === ts.directorySeparator)) {
                        if (invokeMap) {
                            invokeMap.set(rootDirName, true);
                        }
                        else {
                            callbacks.forEach(function (_a) {
                                var callback = _a.callback;
                                return callback(fileName);
                            });
                        }
                    }
                });
            }
            function nonSyncUpdateChildWatches(dirName, dirPath, fileName, options) {
                // Iterate through existing children and update the watches if needed
                var parentWatcher = cache.get(dirPath);
                if (parentWatcher && host.directoryExists(dirName)) {
                    // Schedule the update and postpone invoke for callbacks
                    scheduleUpdateChildWatches(dirName, dirPath, options);
                    return;
                }
                // Call the actual callbacks and remove child watches
                invokeCallbacks(dirPath, fileName);
                removeChildWatches(parentWatcher);
            }
            function scheduleUpdateChildWatches(dirName, dirPath, options) {
                if (!cacheToUpdateChildWatches.has(dirPath)) {
                    cacheToUpdateChildWatches.set(dirPath, { dirName: dirName, options: options });
                }
                if (timerToUpdateChildWatches) {
                    host.clearTimeout(timerToUpdateChildWatches);
                    timerToUpdateChildWatches = undefined;
                }
                timerToUpdateChildWatches = host.setTimeout(onTimerToUpdateChildWatches, 1000);
            }
            function onTimerToUpdateChildWatches() {
                timerToUpdateChildWatches = undefined;
                ts.sysLog("sysLog:: onTimerToUpdateChildWatches:: " + cacheToUpdateChildWatches.size);
                var start = ts.timestamp();
                var invokeMap = ts.createMap();
                while (!timerToUpdateChildWatches && cacheToUpdateChildWatches.size) {
                    var _a = cacheToUpdateChildWatches.entries().next(), _b = _a.value, dirPath = _b[0], _c = _b[1], dirName = _c.dirName, options = _c.options, done = _a.done;
                    ts.Debug.assert(!done);
                    cacheToUpdateChildWatches.delete(dirPath);
                    // Because the child refresh is fresh, we would need to invalidate whole root directory being watched
                    // to ensure that all the changes are reflected at this time
                    invokeCallbacks(dirPath, invokeMap);
                    updateChildWatches(dirName, dirPath, options);
                }
                ts.sysLog("sysLog:: invokingWatchers:: " + (ts.timestamp() - start) + "ms:: " + cacheToUpdateChildWatches.size);
                callbackCache.forEach(function (callbacks, rootDirName) {
                    if (invokeMap.has(rootDirName)) {
                        callbacks.forEach(function (_a) {
                            var callback = _a.callback, dirName = _a.dirName;
                            return callback(dirName);
                        });
                    }
                });
                var elapsed = ts.timestamp() - start;
                ts.sysLog("sysLog:: Elapsed " + elapsed + "ms:: onTimerToUpdateChildWatches:: " + cacheToUpdateChildWatches.size + " " + timerToUpdateChildWatches);
            }
            function removeChildWatches(parentWatcher) {
                if (!parentWatcher)
                    return;
                var existingChildWatches = parentWatcher.childWatches;
                parentWatcher.childWatches = ts.emptyArray;
                for (var _i = 0, existingChildWatches_1 = existingChildWatches; _i < existingChildWatches_1.length; _i++) {
                    var childWatcher = existingChildWatches_1[_i];
                    childWatcher.close();
                    removeChildWatches(cache.get(toCanonicalFilePath(childWatcher.dirName)));
                }
            }
            function updateChildWatches(dirName, dirPath, options) {
                // Iterate through existing children and update the watches if needed
                var parentWatcher = cache.get(dirPath);
                if (parentWatcher) {
                    parentWatcher.childWatches = watchChildDirectories(dirName, parentWatcher.childWatches, options);
                }
            }
            /**
             * Watch the directories in the parentDir
             */
            function watchChildDirectories(parentDir, existingChildWatches, options) {
                var newChildWatches;
                ts.enumerateInsertsAndDeletes(host.directoryExists(parentDir) ? ts.mapDefined(host.getAccessibleSortedChildDirectories(parentDir), function (child) {
                    var childFullName = ts.getNormalizedAbsolutePath(child, parentDir);
                    // Filter our the symbolic link directories since those arent included in recursive watch
                    // which is same behaviour when recursive: true is passed to fs.watch
                    return !isIgnoredPath(childFullName) && filePathComparer(childFullName, ts.normalizePath(host.realpath(childFullName))) === 0 /* EqualTo */ ? childFullName : undefined;
                }) : ts.emptyArray, existingChildWatches, function (child, childWatcher) { return filePathComparer(child, childWatcher.dirName); }, createAndAddChildDirectoryWatcher, ts.closeFileWatcher, addChildDirectoryWatcher);
                return newChildWatches || ts.emptyArray;
                /**
                 * Create new childDirectoryWatcher and add it to the new ChildDirectoryWatcher list
                 */
                function createAndAddChildDirectoryWatcher(childName) {
                    var result = createDirectoryWatcher(childName, options);
                    addChildDirectoryWatcher(result);
                }
                /**
                 * Add child directory watcher to the new ChildDirectoryWatcher list
                 */
                function addChildDirectoryWatcher(childWatcher) {
                    (newChildWatches || (newChildWatches = [])).push(childWatcher);
                }
            }
            function isIgnoredPath(path) {
                return ts.some(ts.ignoredPaths, function (searchPath) { return isInPath(path, searchPath); });
            }
            function isInPath(path, searchPath) {
                if (ts.stringContains(path, searchPath))
                    return true;
                if (host.useCaseSensitiveFileNames)
                    return false;
                return ts.stringContains(toCanonicalFilePath(path), searchPath);
            }
        }
        ts.createDirectoryWatcherSupportingRecursive = createDirectoryWatcherSupportingRecursive;
        /*@internal*/
        var FileSystemEntryKind;
        (function (FileSystemEntryKind) {
            FileSystemEntryKind[FileSystemEntryKind["File"] = 0] = "File";
            FileSystemEntryKind[FileSystemEntryKind["Directory"] = 1] = "Directory";
        })(FileSystemEntryKind = ts.FileSystemEntryKind || (ts.FileSystemEntryKind = {}));
        /*@internal*/
        function createFileWatcherCallback(callback) {
            return function (_fileName, eventKind) { return callback(eventKind === FileWatcherEventKind.Changed ? "change" : "rename", ""); };
        }
        ts.createFileWatcherCallback = createFileWatcherCallback;
        function createFsWatchCallbackForFileWatcherCallback(fileName, callback, fileExists) {
            return function (eventName) {
                if (eventName === "rename") {
                    callback(fileName, fileExists(fileName) ? FileWatcherEventKind.Created : FileWatcherEventKind.Deleted);
                }
                else {
                    // Change
                    callback(fileName, FileWatcherEventKind.Changed);
                }
            };
        }
        function createFsWatchCallbackForDirectoryWatcherCallback(directoryName, callback) {
            return function (eventName, relativeFileName) {
                // In watchDirectory we only care about adding and removing files (when event name is
                // "rename"); changes made within files are handled by corresponding fileWatchers (when
                // event name is "change")
                if (eventName === "rename") {
                    // When deleting a file, the passed baseFileName is null
                    callback(!relativeFileName ? directoryName : ts.normalizePath(ts.combinePaths(directoryName, relativeFileName)));
                }
            };
        }
        /*@internal*/
        function createSystemWatchFunctions(_a) {
            var pollingWatchFile = _a.pollingWatchFile, getModifiedTime = _a.getModifiedTime, setTimeout = _a.setTimeout, clearTimeout = _a.clearTimeout, fsWatch = _a.fsWatch, fileExists = _a.fileExists, useCaseSensitiveFileNames = _a.useCaseSensitiveFileNames, fsSupportsRecursiveFsWatch = _a.fsSupportsRecursiveFsWatch, directoryExists = _a.directoryExists, getAccessibleSortedChildDirectories = _a.getAccessibleSortedChildDirectories, realpath = _a.realpath, tscWatchFile = _a.tscWatchFile, useNonPollingWatchers = _a.useNonPollingWatchers, tscWatchDirectory = _a.tscWatchDirectory;
            var dynamicPollingWatchFile;
            var nonPollingWatchFile;
            var hostRecursiveDirectoryWatcher;
            return {
                watchFile: watchFile,
                watchDirectory: watchDirectory
            };
            function watchFile(fileName, callback, pollingInterval, options) {
                options = updateOptionsForWatchFile(options, useNonPollingWatchers);
                var watchFileKind = ts.Debug.assertDefined(options.watchFile);
                switch (watchFileKind) {
                    case ts.WatchFileKind.FixedPollingInterval:
                        return pollingWatchFile(fileName, callback, PollingInterval.Low, /*options*/ undefined);
                    case ts.WatchFileKind.PriorityPollingInterval:
                        return pollingWatchFile(fileName, callback, pollingInterval, /*options*/ undefined);
                    case ts.WatchFileKind.DynamicPriorityPolling:
                        return ensureDynamicPollingWatchFile()(fileName, callback, pollingInterval, /*options*/ undefined);
                    case ts.WatchFileKind.UseFsEvents:
                        return fsWatch(fileName, 0 /* File */, createFsWatchCallbackForFileWatcherCallback(fileName, callback, fileExists), 
                        /*recursive*/ false, pollingInterval, ts.getFallbackOptions(options));
                    case ts.WatchFileKind.UseFsEventsOnParentDirectory:
                        if (!nonPollingWatchFile) {
                            nonPollingWatchFile = createUseFsEventsOnParentDirectoryWatchFile(fsWatch, useCaseSensitiveFileNames);
                        }
                        return nonPollingWatchFile(fileName, callback, pollingInterval, ts.getFallbackOptions(options));
                    default:
                        ts.Debug.assertNever(watchFileKind);
                }
            }
            function ensureDynamicPollingWatchFile() {
                return dynamicPollingWatchFile ||
                    (dynamicPollingWatchFile = createDynamicPriorityPollingWatchFile({ getModifiedTime: getModifiedTime, setTimeout: setTimeout }));
            }
            function updateOptionsForWatchFile(options, useNonPollingWatchers) {
                if (options && options.watchFile !== undefined)
                    return options;
                switch (tscWatchFile) {
                    case "PriorityPollingInterval":
                        // Use polling interval based on priority when create watch using host.watchFile
                        return { watchFile: ts.WatchFileKind.PriorityPollingInterval };
                    case "DynamicPriorityPolling":
                        // Use polling interval but change the interval depending on file changes and their default polling interval
                        return { watchFile: ts.WatchFileKind.DynamicPriorityPolling };
                    case "UseFsEvents":
                        // Use notifications from FS to watch with falling back to fs.watchFile
                        return generateWatchFileOptions(ts.WatchFileKind.UseFsEvents, ts.PollingWatchKind.PriorityInterval, options);
                    case "UseFsEventsWithFallbackDynamicPolling":
                        // Use notifications from FS to watch with falling back to dynamic watch file
                        return generateWatchFileOptions(ts.WatchFileKind.UseFsEvents, ts.PollingWatchKind.DynamicPriority, options);
                    case "UseFsEventsOnParentDirectory":
                        useNonPollingWatchers = true;
                    // fall through
                    default:
                        return useNonPollingWatchers ?
                            // Use notifications from FS to watch with falling back to fs.watchFile
                            generateWatchFileOptions(ts.WatchFileKind.UseFsEventsOnParentDirectory, ts.PollingWatchKind.PriorityInterval, options) :
                            // Default to do not use fixed polling interval
                            { watchFile: ts.WatchFileKind.FixedPollingInterval };
                }
            }
            function generateWatchFileOptions(watchFile, fallbackPolling, options) {
                var defaultFallbackPolling = options === null || options === void 0 ? void 0 : options.fallbackPolling;
                return {
                    watchFile: watchFile,
                    fallbackPolling: defaultFallbackPolling === undefined ?
                        fallbackPolling :
                        defaultFallbackPolling
                };
            }
            function watchDirectory(directoryName, callback, recursive, options) {
                if (fsSupportsRecursiveFsWatch) {
                    return fsWatch(directoryName, 1 /* Directory */, createFsWatchCallbackForDirectoryWatcherCallback(directoryName, callback), recursive, PollingInterval.Medium, ts.getFallbackOptions(options));
                }
                if (!hostRecursiveDirectoryWatcher) {
                    hostRecursiveDirectoryWatcher = createDirectoryWatcherSupportingRecursive({
                        useCaseSensitiveFileNames: useCaseSensitiveFileNames,
                        directoryExists: directoryExists,
                        getAccessibleSortedChildDirectories: getAccessibleSortedChildDirectories,
                        watchDirectory: nonRecursiveWatchDirectory,
                        realpath: realpath,
                        setTimeout: setTimeout,
                        clearTimeout: clearTimeout
                    });
                }
                return hostRecursiveDirectoryWatcher(directoryName, callback, recursive, options);
            }
            function nonRecursiveWatchDirectory(directoryName, callback, recursive, options) {
                ts.Debug.assert(!recursive);
                options = updateOptionsForWatchDirectory(options);
                var watchDirectoryKind = ts.Debug.assertDefined(options.watchDirectory);
                switch (watchDirectoryKind) {
                    case ts.WatchDirectoryKind.FixedPollingInterval:
                        return pollingWatchFile(directoryName, function () { return callback(directoryName); }, PollingInterval.Medium, 
                        /*options*/ undefined);
                    case ts.WatchDirectoryKind.DynamicPriorityPolling:
                        return ensureDynamicPollingWatchFile()(directoryName, function () { return callback(directoryName); }, PollingInterval.Medium, 
                        /*options*/ undefined);
                    case ts.WatchDirectoryKind.UseFsEvents:
                        return fsWatch(directoryName, 1 /* Directory */, createFsWatchCallbackForDirectoryWatcherCallback(directoryName, callback), recursive, PollingInterval.Medium, ts.getFallbackOptions(options));
                    default:
                        ts.Debug.assertNever(watchDirectoryKind);
                }
            }
            function updateOptionsForWatchDirectory(options) {
                if (options && options.watchDirectory !== undefined)
                    return options;
                switch (tscWatchDirectory) {
                    case "RecursiveDirectoryUsingFsWatchFile":
                        // Use polling interval based on priority when create watch using host.watchFile
                        return { watchDirectory: ts.WatchDirectoryKind.FixedPollingInterval };
                    case "RecursiveDirectoryUsingDynamicPriorityPolling":
                        // Use polling interval but change the interval depending on file changes and their default polling interval
                        return { watchDirectory: ts.WatchDirectoryKind.DynamicPriorityPolling };
                    default:
                        var defaultFallbackPolling = options === null || options === void 0 ? void 0 : options.fallbackPolling;
                        return {
                            watchDirectory: ts.WatchDirectoryKind.UseFsEvents,
                            fallbackPolling: defaultFallbackPolling !== undefined ?
                                defaultFallbackPolling :
                                undefined
                        };
                }
            }
        }
        ts.createSystemWatchFunctions = createSystemWatchFunctions;
        /**
         * patch writefile to create folder before writing the file
         */
        /*@internal*/
        function patchWriteFileEnsuringDirectory(sys) {
            // patch writefile to create folder before writing the file
            var originalWriteFile = sys.writeFile;
            sys.writeFile = function (path, data, writeBom) {
                return ts.writeFileEnsuringDirectories(path, data, !!writeBom, function (path, data, writeByteOrderMark) { return originalWriteFile.call(sys, path, data, writeByteOrderMark); }, function (path) { return sys.createDirectory(path); }, function (path) { return sys.directoryExists(path); });
            };
        }
        ts.patchWriteFileEnsuringDirectory = patchWriteFileEnsuringDirectory;
        function getNodeMajorVersion() {
            if (typeof process === "undefined") {
                return undefined;
            }
            var version = process.version;
            if (!version) {
                return undefined;
            }
            var dot = version.indexOf(".");
            if (dot === -1) {
                return undefined;
            }
            return parseInt(version.substring(1, dot));
        }
        ts.getNodeMajorVersion = getNodeMajorVersion;
        // TODO: GH#18217 this is used as if it's certainly defined in many places.
        // eslint-disable-next-line prefer-const
        ts.sys = (function () {
            // NodeJS detects "\uFEFF" at the start of the string and *replaces* it with the actual
            // byte order mark from the specified encoding. Using any other byte order mark does
            // not actually work.
            var byteOrderMarkIndicator = "\uFEFF";
            function getNodeSystem() {
                var nativePattern = /^native |^\([^)]+\)$|^(internal[\\/]|[a-zA-Z0-9_\s]+(\.js)?$)/;
                var _fs = require("fs");
                var _path = require("path");
                var _os = require("os");
                // crypto can be absent on reduced node installations
                var _crypto;
                try {
                    _crypto = require("crypto");
                }
                catch (_a) {
                    _crypto = undefined;
                }
                var activeSession;
                var profilePath = "./profile.cpuprofile";
                var Buffer = require("buffer").Buffer;
                var nodeVersion = getNodeMajorVersion();
                var isNode4OrLater = nodeVersion >= 4;
                var isLinuxOrMacOs = process.platform === "linux" || process.platform === "darwin";
                var platform = _os.platform();
                var useCaseSensitiveFileNames = isFileSystemCaseSensitive();
                var fsSupportsRecursiveFsWatch = isNode4OrLater && (process.platform === "win32" || process.platform === "darwin");
                var _b = createSystemWatchFunctions({
                    pollingWatchFile: createSingleFileWatcherPerName(fsWatchFileWorker, useCaseSensitiveFileNames),
                    getModifiedTime: getModifiedTime,
                    setTimeout: setTimeout,
                    clearTimeout: clearTimeout,
                    fsWatch: fsWatch,
                    useCaseSensitiveFileNames: useCaseSensitiveFileNames,
                    fileExists: fileExists,
                    // Node 4.0 `fs.watch` function supports the "recursive" option on both OSX and Windows
                    // (ref: https://github.com/nodejs/node/pull/2649 and https://github.com/Microsoft/TypeScript/issues/4643)
                    fsSupportsRecursiveFsWatch: fsSupportsRecursiveFsWatch,
                    directoryExists: directoryExists,
                    getAccessibleSortedChildDirectories: function (path) { return getAccessibleFileSystemEntries(path).directories; },
                    realpath: realpath,
                    tscWatchFile: process.env.TSC_WATCHFILE,
                    useNonPollingWatchers: process.env.TSC_NONPOLLING_WATCHER,
                    tscWatchDirectory: process.env.TSC_WATCHDIRECTORY,
                }), watchFile = _b.watchFile, watchDirectory = _b.watchDirectory;
                var nodeSystem = {
                    args: process.argv.slice(2),
                    newLine: _os.EOL,
                    useCaseSensitiveFileNames: useCaseSensitiveFileNames,
                    write: function (s) {
                        process.stdout.write(s);
                    },
                    writeOutputIsTTY: function () {
                        return process.stdout.isTTY;
                    },
                    readFile: readFile,
                    writeFile: writeFile,
                    watchFile: watchFile,
                    watchDirectory: watchDirectory,
                    resolvePath: function (path) { return _path.resolve(path); },
                    fileExists: fileExists,
                    directoryExists: directoryExists,
                    createDirectory: function (directoryName) {
                        if (!nodeSystem.directoryExists(directoryName)) {
                            // Wrapped in a try-catch to prevent crashing if we are in a race
                            // with another copy of ourselves to create the same directory
                            try {
                                _fs.mkdirSync(directoryName);
                            }
                            catch (e) {
                                if (e.code !== "EEXIST") {
                                    // Failed for some other reason (access denied?); still throw
                                    throw e;
                                }
                            }
                        }
                    },
                    getExecutingFilePath: function () {
                        return __filename;
                    },
                    getCurrentDirectory: function () {
                        return process.cwd();
                    },
                    getDirectories: getDirectories,
                    getEnvironmentVariable: function (name) {
                        return process.env[name] || "";
                    },
                    readDirectory: readDirectory,
                    getModifiedTime: getModifiedTime,
                    setModifiedTime: setModifiedTime,
                    deleteFile: deleteFile,
                    createHash: _crypto ? createSHA256Hash : generateDjb2Hash,
                    createSHA256Hash: _crypto ? createSHA256Hash : undefined,
                    getMemoryUsage: function () {
                        if (global.gc) {
                            global.gc();
                        }
                        return process.memoryUsage().heapUsed;
                    },
                    getFileSize: function (path) {
                        try {
                            var stat = _fs.statSync(path);
                            if (stat.isFile()) {
                                return stat.size;
                            }
                        }
                        catch ( /*ignore*/_a) { /*ignore*/ }
                        return 0;
                    },
                    exit: function (exitCode) {
                        disableCPUProfiler(function () { return process.exit(exitCode); });
                    },
                    enableCPUProfiler: enableCPUProfiler,
                    disableCPUProfiler: disableCPUProfiler,
                    realpath: realpath,
                    debugMode: ts.some(process.execArgv, function (arg) { return /^--(inspect|debug)(-brk)?(=\d+)?$/i.test(arg); }),
                    tryEnableSourceMapsForHost: function () {
                        try {
                            require(45) /* source-map-support */.install();
                        }
                        catch (_a) {
                            // Could not enable source maps.
                        }
                    },
                    setTimeout: setTimeout,
                    clearTimeout: clearTimeout,
                    clearScreen: function () {
                        process.stdout.write("\x1Bc");
                    },
                    setBlocking: function () {
                        if (process.stdout && process.stdout._handle && process.stdout._handle.setBlocking) {
                            process.stdout._handle.setBlocking(true);
                        }
                    },
                    bufferFrom: bufferFrom,
                    base64decode: function (input) { return bufferFrom(input, "base64").toString("utf8"); },
                    base64encode: function (input) { return bufferFrom(input).toString("base64"); },
                    require: function (baseDir, moduleName) {
                        try {
                            var modulePath = ts.resolveJSModule(moduleName, baseDir, nodeSystem);
                            return { module: require(modulePath), modulePath: modulePath, error: undefined };
                        }
                        catch (error) {
                            return { module: undefined, modulePath: undefined, error: error };
                        }
                    }
                };
                return nodeSystem;
                /**
                 * Uses the builtin inspector APIs to capture a CPU profile
                 * See https://nodejs.org/api/inspector.html#inspector_example_usage for details
                 */
                function enableCPUProfiler(path, cb) {
                    if (activeSession) {
                        cb();
                        return false;
                    }
                    var inspector = require("inspector");
                    if (!inspector || !inspector.Session) {
                        cb();
                        return false;
                    }
                    var session = new inspector.Session();
                    session.connect();
                    session.post("Profiler.enable", function () {
                        session.post("Profiler.start", function () {
                            activeSession = session;
                            profilePath = path;
                            cb();
                        });
                    });
                    return true;
                }
                /**
                 * Strips non-TS paths from the profile, so users with private projects shouldn't
                 * need to worry about leaking paths by submitting a cpu profile to us
                 */
                function cleanupPaths(profile) {
                    var externalFileCounter = 0;
                    var remappedPaths = ts.createMap();
                    var normalizedDir = ts.normalizeSlashes(__dirname);
                    // Windows rooted dir names need an extra `/` prepended to be valid file:/// urls
                    var fileUrlRoot = "file://" + (ts.getRootLength(normalizedDir) === 1 ? "" : "/") + normalizedDir;
                    for (var _i = 0, _a = profile.nodes; _i < _a.length; _i++) {
                        var node = _a[_i];
                        if (node.callFrame.url) {
                            var url = ts.normalizeSlashes(node.callFrame.url);
                            if (ts.containsPath(fileUrlRoot, url, useCaseSensitiveFileNames)) {
                                node.callFrame.url = ts.getRelativePathToDirectoryOrUrl(fileUrlRoot, url, fileUrlRoot, ts.createGetCanonicalFileName(useCaseSensitiveFileNames), /*isAbsolutePathAnUrl*/ true);
                            }
                            else if (!nativePattern.test(url)) {
                                node.callFrame.url = (remappedPaths.has(url) ? remappedPaths : remappedPaths.set(url, "external" + externalFileCounter + ".js")).get(url);
                                externalFileCounter++;
                            }
                        }
                    }
                    return profile;
                }
                function disableCPUProfiler(cb) {
                    if (activeSession && activeSession !== "stopping") {
                        var s_1 = activeSession;
                        activeSession.post("Profiler.stop", function (err, _a) {
                            var profile = _a.profile;
                            if (!err) {
                                try {
                                    if (_fs.statSync(profilePath).isDirectory()) {
                                        profilePath = _path.join(profilePath, (new Date()).toISOString().replace(/:/g, "-") + "+P" + process.pid + ".cpuprofile");
                                    }
                                }
                                catch (_b) {
                                    // do nothing and ignore fallible fs operation
                                }
                                try {
                                    _fs.mkdirSync(_path.dirname(profilePath), { recursive: true });
                                }
                                catch (_c) {
                                    // do nothing and ignore fallible fs operation
                                }
                                _fs.writeFileSync(profilePath, JSON.stringify(cleanupPaths(profile)));
                            }
                            activeSession = undefined;
                            s_1.disconnect();
                            cb();
                        });
                        activeSession = "stopping";
                        return true;
                    }
                    else {
                        cb();
                        return false;
                    }
                }
                function bufferFrom(input, encoding) {
                    // See https://github.com/Microsoft/TypeScript/issues/25652
                    return Buffer.from && Buffer.from !== Int8Array.from
                        ? Buffer.from(input, encoding)
                        : new Buffer(input, encoding);
                }
                function isFileSystemCaseSensitive() {
                    // win32\win64 are case insensitive platforms
                    if (platform === "win32" || platform === "win64") {
                        return false;
                    }
                    // If this file exists under a different case, we must be case-insensitve.
                    return !fileExists(swapCase(__filename));
                }
                /** Convert all lowercase chars to uppercase, and vice-versa */
                function swapCase(s) {
                    return s.replace(/\w/g, function (ch) {
                        var up = ch.toUpperCase();
                        return ch === up ? ch.toLowerCase() : up;
                    });
                }
                function fsWatchFileWorker(fileName, callback, pollingInterval) {
                    _fs.watchFile(fileName, { persistent: true, interval: pollingInterval }, fileChanged);
                    var eventKind;
                    return {
                        close: function () { return _fs.unwatchFile(fileName, fileChanged); }
                    };
                    function fileChanged(curr, prev) {
                        // previous event kind check is to ensure we recongnize the file as previously also missing when it is restored or renamed twice (that is it disappears and reappears)
                        // In such case, prevTime returned is same as prev time of event when file was deleted as per node documentation
                        var isPreviouslyDeleted = +prev.mtime === 0 || eventKind === FileWatcherEventKind.Deleted;
                        if (+curr.mtime === 0) {
                            if (isPreviouslyDeleted) {
                                // Already deleted file, no need to callback again
                                return;
                            }
                            eventKind = FileWatcherEventKind.Deleted;
                        }
                        else if (isPreviouslyDeleted) {
                            eventKind = FileWatcherEventKind.Created;
                        }
                        // If there is no change in modified time, ignore the event
                        else if (+curr.mtime === +prev.mtime) {
                            return;
                        }
                        else {
                            // File changed
                            eventKind = FileWatcherEventKind.Changed;
                        }
                        callback(fileName, eventKind);
                    }
                }
                function fsWatch(fileOrDirectory, entryKind, callback, recursive, fallbackPollingInterval, fallbackOptions) {
                    var options;
                    var lastDirectoryPartWithDirectorySeparator;
                    var lastDirectoryPart;
                    if (isLinuxOrMacOs) {
                        lastDirectoryPartWithDirectorySeparator = fileOrDirectory.substr(fileOrDirectory.lastIndexOf(ts.directorySeparator));
                        lastDirectoryPart = lastDirectoryPartWithDirectorySeparator.slice(ts.directorySeparator.length);
                    }
                    /** Watcher for the file system entry depending on whether it is missing or present */
                    var watcher = !fileSystemEntryExists(fileOrDirectory, entryKind) ?
                        watchMissingFileSystemEntry() :
                        watchPresentFileSystemEntry();
                    return {
                        close: function () {
                            // Close the watcher (either existing file system entry watcher or missing file system entry watcher)
                            watcher.close();
                            watcher = undefined;
                        }
                    };
                    /**
                     * Invoke the callback with rename and update the watcher if not closed
                     * @param createWatcher
                     */
                    function invokeCallbackAndUpdateWatcher(createWatcher) {
                        ts.sysLog("sysLog:: " + fileOrDirectory + ":: Changing watcher to " + (createWatcher === watchPresentFileSystemEntry ? "Present" : "Missing") + "FileSystemEntryWatcher");
                        // Call the callback for current directory
                        callback("rename", "");
                        // If watcher is not closed, update it
                        if (watcher) {
                            watcher.close();
                            watcher = createWatcher();
                        }
                    }
                    /**
                     * Watch the file or directory that is currently present
                     * and when the watched file or directory is deleted, switch to missing file system entry watcher
                     */
                    function watchPresentFileSystemEntry() {
                        // Node 4.0 `fs.watch` function supports the "recursive" option on both OSX and Windows
                        // (ref: https://github.com/nodejs/node/pull/2649 and https://github.com/Microsoft/TypeScript/issues/4643)
                        if (options === undefined) {
                            if (fsSupportsRecursiveFsWatch) {
                                options = { persistent: true, recursive: !!recursive };
                            }
                            else {
                                options = { persistent: true };
                            }
                        }
                        try {
                            var presentWatcher = _fs.watch(fileOrDirectory, options, isLinuxOrMacOs ?
                                callbackChangingToMissingFileSystemEntry :
                                callback);
                            // Watch the missing file or directory or error
                            presentWatcher.on("error", function () { return invokeCallbackAndUpdateWatcher(watchMissingFileSystemEntry); });
                            return presentWatcher;
                        }
                        catch (e) {
                            // Catch the exception and use polling instead
                            // Eg. on linux the number of watches are limited and one could easily exhaust watches and the exception ENOSPC is thrown when creating watcher at that point
                            // so instead of throwing error, use fs.watchFile
                            return watchPresentFileSystemEntryWithFsWatchFile();
                        }
                    }
                    function callbackChangingToMissingFileSystemEntry(event, relativeName) {
                        // because relativeName is not guaranteed to be correct we need to check on each rename with few combinations
                        // Eg on ubuntu while watching app/node_modules the relativeName is "node_modules" which is neither relative nor full path
                        return event === "rename" &&
                            (!relativeName ||
                                relativeName === lastDirectoryPart ||
                                relativeName.lastIndexOf(lastDirectoryPartWithDirectorySeparator) === relativeName.length - lastDirectoryPartWithDirectorySeparator.length) &&
                            !fileSystemEntryExists(fileOrDirectory, entryKind) ?
                            invokeCallbackAndUpdateWatcher(watchMissingFileSystemEntry) :
                            callback(event, relativeName);
                    }
                    /**
                     * Watch the file or directory using fs.watchFile since fs.watch threw exception
                     * Eg. on linux the number of watches are limited and one could easily exhaust watches and the exception ENOSPC is thrown when creating watcher at that point
                     */
                    function watchPresentFileSystemEntryWithFsWatchFile() {
                        ts.sysLog("sysLog:: " + fileOrDirectory + ":: Changing to fsWatchFile");
                        return watchFile(fileOrDirectory, createFileWatcherCallback(callback), fallbackPollingInterval, fallbackOptions);
                    }
                    /**
                     * Watch the file or directory that is missing
                     * and switch to existing file or directory when the missing filesystem entry is created
                     */
                    function watchMissingFileSystemEntry() {
                        return watchFile(fileOrDirectory, function (_fileName, eventKind) {
                            if (eventKind === FileWatcherEventKind.Created && fileSystemEntryExists(fileOrDirectory, entryKind)) {
                                // Call the callback for current file or directory
                                // For now it could be callback for the inner directory creation,
                                // but just return current directory, better than current no-op
                                invokeCallbackAndUpdateWatcher(watchPresentFileSystemEntry);
                            }
                        }, fallbackPollingInterval, fallbackOptions);
                    }
                }
                function readFileWorker(fileName, _encoding) {
                    var buffer;
                    try {
                        buffer = _fs.readFileSync(fileName);
                    }
                    catch (e) {
                        return undefined;
                    }
                    var len = buffer.length;
                    if (len >= 2 && buffer[0] === 0xFE && buffer[1] === 0xFF) {
                        // Big endian UTF-16 byte order mark detected. Since big endian is not supported by node.js,
                        // flip all byte pairs and treat as little endian.
                        len &= ~1; // Round down to a multiple of 2
                        for (var i = 0; i < len; i += 2) {
                            var temp = buffer[i];
                            buffer[i] = buffer[i + 1];
                            buffer[i + 1] = temp;
                        }
                        return buffer.toString("utf16le", 2);
                    }
                    if (len >= 2 && buffer[0] === 0xFF && buffer[1] === 0xFE) {
                        // Little endian UTF-16 byte order mark detected
                        return buffer.toString("utf16le", 2);
                    }
                    if (len >= 3 && buffer[0] === 0xEF && buffer[1] === 0xBB && buffer[2] === 0xBF) {
                        // UTF-8 byte order mark detected
                        return buffer.toString("utf8", 3);
                    }
                    // Default is UTF-8 with no byte order mark
                    return buffer.toString("utf8");
                }
                function readFile(fileName, _encoding) {
                    ts.perfLogger.logStartReadFile(fileName);
                    var file = readFileWorker(fileName, _encoding);
                    ts.perfLogger.logStopReadFile();
                    return file;
                }
                function writeFile(fileName, data, writeByteOrderMark) {
                    ts.perfLogger.logEvent("WriteFile: " + fileName);
                    // If a BOM is required, emit one
                    if (writeByteOrderMark) {
                        data = byteOrderMarkIndicator + data;
                    }
                    var fd;
                    try {
                        fd = _fs.openSync(fileName, "w");
                        _fs.writeSync(fd, data, /*position*/ undefined, "utf8");
                    }
                    finally {
                        if (fd !== undefined) {
                            _fs.closeSync(fd);
                        }
                    }
                }
                function getAccessibleFileSystemEntries(path) {
                    ts.perfLogger.logEvent("ReadDir: " + (path || "."));
                    try {
                        var entries = _fs.readdirSync(path || ".", { withFileTypes: true });
                        var files = [];
                        var directories = [];
                        for (var _i = 0, entries_2 = entries; _i < entries_2.length; _i++) {
                            var dirent = entries_2[_i];
                            // withFileTypes is not supported before Node 10.10.
                            var entry = typeof dirent === "string" ? dirent : dirent.name;
                            // This is necessary because on some file system node fails to exclude
                            // "." and "..". See https://github.com/nodejs/node/issues/4002
                            if (entry === "." || entry === "..") {
                                continue;
                            }
                            var stat = void 0;
                            if (typeof dirent === "string" || dirent.isSymbolicLink()) {
                                var name = ts.combinePaths(path, entry);
                                try {
                                    stat = _fs.statSync(name);
                                }
                                catch (e) {
                                    continue;
                                }
                            }
                            else {
                                stat = dirent;
                            }
                            if (stat.isFile()) {
                                files.push(entry);
                            }
                            else if (stat.isDirectory()) {
                                directories.push(entry);
                            }
                        }
                        files.sort();
                        directories.sort();
                        return { files: files, directories: directories };
                    }
                    catch (e) {
                        return ts.emptyFileSystemEntries;
                    }
                }
                function readDirectory(path, extensions, excludes, includes, depth) {
                    return ts.matchFiles(path, extensions, excludes, includes, useCaseSensitiveFileNames, process.cwd(), depth, getAccessibleFileSystemEntries, realpath);
                }
                function fileSystemEntryExists(path, entryKind) {
                    try {
                        var stat = _fs.statSync(path);
                        switch (entryKind) {
                            case 0 /* File */: return stat.isFile();
                            case 1 /* Directory */: return stat.isDirectory();
                            default: return false;
                        }
                    }
                    catch (e) {
                        return false;
                    }
                }
                function fileExists(path) {
                    return fileSystemEntryExists(path, 0 /* File */);
                }
                function directoryExists(path) {
                    return fileSystemEntryExists(path, 1 /* Directory */);
                }
                function getDirectories(path) {
                    return getAccessibleFileSystemEntries(path).directories.slice();
                }
                function realpath(path) {
                    try {
                        return _fs.realpathSync(path);
                    }
                    catch (_a) {
                        return path;
                    }
                }
                function getModifiedTime(path) {
                    try {
                        return _fs.statSync(path).mtime;
                    }
                    catch (e) {
                        return undefined;
                    }
                }
                function setModifiedTime(path, time) {
                    try {
                        _fs.utimesSync(path, time, time);
                    }
                    catch (e) {
                        return;
                    }
                }
                function deleteFile(path) {
                    try {
                        return _fs.unlinkSync(path);
                    }
                    catch (e) {
                        return;
                    }
                }
                function createSHA256Hash(data) {
                    var hash = _crypto.createHash("sha256");
                    hash.update(data);
                    return hash.digest("hex");
                }
            }
            function getChakraSystem() {
                var realpath = ChakraHost.realpath && (function (path) { return ChakraHost.realpath(path); });
                return {
                    newLine: ChakraHost.newLine || "\r\n",
                    args: ChakraHost.args,
                    useCaseSensitiveFileNames: !!ChakraHost.useCaseSensitiveFileNames,
                    write: ChakraHost.echo,
                    readFile: function (path, _encoding) {
                        // encoding is automatically handled by the implementation in ChakraHost
                        return ChakraHost.readFile(path);
                    },
                    writeFile: function (path, data, writeByteOrderMark) {
                        // If a BOM is required, emit one
                        if (writeByteOrderMark) {
                            data = byteOrderMarkIndicator + data;
                        }
                        ChakraHost.writeFile(path, data);
                    },
                    resolvePath: ChakraHost.resolvePath,
                    fileExists: ChakraHost.fileExists,
                    deleteFile: ChakraHost.deleteFile,
                    getModifiedTime: ChakraHost.getModifiedTime,
                    setModifiedTime: ChakraHost.setModifiedTime,
                    directoryExists: ChakraHost.directoryExists,
                    createDirectory: ChakraHost.createDirectory,
                    getExecutingFilePath: function () { return ChakraHost.executingFile; },
                    getCurrentDirectory: function () { return ChakraHost.currentDirectory; },
                    getDirectories: ChakraHost.getDirectories,
                    getEnvironmentVariable: ChakraHost.getEnvironmentVariable || (function () { return ""; }),
                    readDirectory: function (path, extensions, excludes, includes, _depth) {
                        var pattern = ts.getFileMatcherPatterns(path, excludes, includes, !!ChakraHost.useCaseSensitiveFileNames, ChakraHost.currentDirectory);
                        return ChakraHost.readDirectory(path, extensions, pattern.basePaths, pattern.excludePattern, pattern.includeFilePattern, pattern.includeDirectoryPattern);
                    },
                    exit: ChakraHost.quit,
                    realpath: realpath
                };
            }
            var sys;
            if (typeof ChakraHost !== "undefined") {
                sys = getChakraSystem();
            }
            else if (typeof process !== "undefined" && process.nextTick && !process.browser && typeof require !== "undefined") {
                // process and process.nextTick checks if current environment is node-like
                // process.browser check excludes webpack and browserify
                sys = getNodeSystem();
            }
            if (sys) {
                // patch writefile to create folder before writing the file
                patchWriteFileEnsuringDirectory(sys);
            }
            return sys;
        })();
        if (ts.sys && ts.sys.getEnvironmentVariable) {
            setCustomPollingValues(ts.sys);
            ts.Debug.currentAssertionLevel = /^development$/i.test(ts.sys.getEnvironmentVariable("NODE_ENV"))
                ? 1 /* Normal */
                : 0 /* None */;
        }
        if (ts.sys && ts.sys.debugMode) {
            ts.Debug.isDebugging = true;
        }
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        /**
         * Internally, we represent paths as strings with '/' as the directory separator.
         * When we make system calls (eg: LanguageServiceHost.getDirectory()),
         * we expect the host to correctly handle paths in our specified format.
         */
        ts.directorySeparator = "/";
        var altDirectorySeparator = "\\";
        var urlSchemeSeparator = "://";
        var backslashRegExp = /\\/g;
        //// Path Tests
        /**
         * Determines whether a charCode corresponds to `/` or `\`.
         */
        function isAnyDirectorySeparator(charCode) {
            return charCode === 47 /* slash */ || charCode === 92 /* backslash */;
        }
        ts.isAnyDirectorySeparator = isAnyDirectorySeparator;
        /**
         * Determines whether a path starts with a URL scheme (e.g. starts with `http://`, `ftp://`, `file://`, etc.).
         */
        function isUrl(path) {
            return getEncodedRootLength(path) < 0;
        }
        ts.isUrl = isUrl;
        /**
         * Determines whether a path is an absolute disk path (e.g. starts with `/`, or a dos path
         * like `c:`, `c:\` or `c:/`).
         */
        function isRootedDiskPath(path) {
            return getEncodedRootLength(path) > 0;
        }
        ts.isRootedDiskPath = isRootedDiskPath;
        /**
         * Determines whether a path consists only of a path root.
         */
        function isDiskPathRoot(path) {
            var rootLength = getEncodedRootLength(path);
            return rootLength > 0 && rootLength === path.length;
        }
        ts.isDiskPathRoot = isDiskPathRoot;
        /**
         * Determines whether a path starts with an absolute path component (i.e. `/`, `c:/`, `file://`, etc.).
         *
         * ```ts
         * // POSIX
         * pathIsAbsolute("/path/to/file.ext") === true
         * // DOS
         * pathIsAbsolute("c:/path/to/file.ext") === true
         * // URL
         * pathIsAbsolute("file:///path/to/file.ext") === true
         * // Non-absolute
         * pathIsAbsolute("path/to/file.ext") === false
         * pathIsAbsolute("./path/to/file.ext") === false
         * ```
         */
        function pathIsAbsolute(path) {
            return getEncodedRootLength(path) !== 0;
        }
        ts.pathIsAbsolute = pathIsAbsolute;
        /**
         * Determines whether a path starts with a relative path component (i.e. `.` or `..`).
         */
        function pathIsRelative(path) {
            return /^\.\.?($|[\\/])/.test(path);
        }
        ts.pathIsRelative = pathIsRelative;
        function hasExtension(fileName) {
            return ts.stringContains(getBaseFileName(fileName), ".");
        }
        ts.hasExtension = hasExtension;
        function fileExtensionIs(path, extension) {
            return path.length > extension.length && ts.endsWith(path, extension);
        }
        ts.fileExtensionIs = fileExtensionIs;
        function fileExtensionIsOneOf(path, extensions) {
            for (var _i = 0, extensions_1 = extensions; _i < extensions_1.length; _i++) {
                var extension = extensions_1[_i];
                if (fileExtensionIs(path, extension)) {
                    return true;
                }
            }
            return false;
        }
        ts.fileExtensionIsOneOf = fileExtensionIsOneOf;
        /**
         * Determines whether a path has a trailing separator (`/` or `\\`).
         */
        function hasTrailingDirectorySeparator(path) {
            return path.length > 0 && isAnyDirectorySeparator(path.charCodeAt(path.length - 1));
        }
        ts.hasTrailingDirectorySeparator = hasTrailingDirectorySeparator;
        //// Path Parsing
        function isVolumeCharacter(charCode) {
            return (charCode >= 97 /* a */ && charCode <= 122 /* z */) ||
                (charCode >= 65 /* A */ && charCode <= 90 /* Z */);
        }
        function getFileUrlVolumeSeparatorEnd(url, start) {
            var ch0 = url.charCodeAt(start);
            if (ch0 === 58 /* colon */)
                return start + 1;
            if (ch0 === 37 /* percent */ && url.charCodeAt(start + 1) === 51 /* _3 */) {
                var ch2 = url.charCodeAt(start + 2);
                if (ch2 === 97 /* a */ || ch2 === 65 /* A */)
                    return start + 3;
            }
            return -1;
        }
        /**
         * Returns length of the root part of a path or URL (i.e. length of "/", "x:/", "//server/share/, file:///user/files").
         * If the root is part of a URL, the twos-complement of the root length is returned.
         */
        function getEncodedRootLength(path) {
            if (!path)
                return 0;
            var ch0 = path.charCodeAt(0);
            // POSIX or UNC
            if (ch0 === 47 /* slash */ || ch0 === 92 /* backslash */) {
                if (path.charCodeAt(1) !== ch0)
                    return 1; // POSIX: "/" (or non-normalized "\")
                var p1 = path.indexOf(ch0 === 47 /* slash */ ? ts.directorySeparator : altDirectorySeparator, 2);
                if (p1 < 0)
                    return path.length; // UNC: "//server" or "\\server"
                return p1 + 1; // UNC: "//server/" or "\\server\"
            }
            // DOS
            if (isVolumeCharacter(ch0) && path.charCodeAt(1) === 58 /* colon */) {
                var ch2 = path.charCodeAt(2);
                if (ch2 === 47 /* slash */ || ch2 === 92 /* backslash */)
                    return 3; // DOS: "c:/" or "c:\"
                if (path.length === 2)
                    return 2; // DOS: "c:" (but not "c:d")
            }
            // URL
            var schemeEnd = path.indexOf(urlSchemeSeparator);
            if (schemeEnd !== -1) {
                var authorityStart = schemeEnd + urlSchemeSeparator.length;
                var authorityEnd = path.indexOf(ts.directorySeparator, authorityStart);
                if (authorityEnd !== -1) { // URL: "file:///", "file://server/", "file://server/path"
                    // For local "file" URLs, include the leading DOS volume (if present).
                    // Per https://www.ietf.org/rfc/rfc1738.txt, a host of "" or "localhost" is a
                    // special case interpreted as "the machine from which the URL is being interpreted".
                    var scheme = path.slice(0, schemeEnd);
                    var authority = path.slice(authorityStart, authorityEnd);
                    if (scheme === "file" && (authority === "" || authority === "localhost") &&
                        isVolumeCharacter(path.charCodeAt(authorityEnd + 1))) {
                        var volumeSeparatorEnd = getFileUrlVolumeSeparatorEnd(path, authorityEnd + 2);
                        if (volumeSeparatorEnd !== -1) {
                            if (path.charCodeAt(volumeSeparatorEnd) === 47 /* slash */) {
                                // URL: "file:///c:/", "file://localhost/c:/", "file:///c%3a/", "file://localhost/c%3a/"
                                return ~(volumeSeparatorEnd + 1);
                            }
                            if (volumeSeparatorEnd === path.length) {
                                // URL: "file:///c:", "file://localhost/c:", "file:///c$3a", "file://localhost/c%3a"
                                // but not "file:///c:d" or "file:///c%3ad"
                                return ~volumeSeparatorEnd;
                            }
                        }
                    }
                    return ~(authorityEnd + 1); // URL: "file://server/", "http://server/"
                }
                return ~path.length; // URL: "file://server", "http://server"
            }
            // relative
            return 0;
        }
        /**
         * Returns length of the root part of a path or URL (i.e. length of "/", "x:/", "//server/share/, file:///user/files").
         *
         * For example:
         * ```ts
         * getRootLength("a") === 0                   // ""
         * getRootLength("/") === 1                   // "/"
         * getRootLength("c:") === 2                  // "c:"
         * getRootLength("c:d") === 0                 // ""
         * getRootLength("c:/") === 3                 // "c:/"
         * getRootLength("c:\\") === 3                // "c:\\"
         * getRootLength("//server") === 7            // "//server"
         * getRootLength("//server/share") === 8      // "//server/"
         * getRootLength("\\\\server") === 7          // "\\\\server"
         * getRootLength("\\\\server\\share") === 8   // "\\\\server\\"
         * getRootLength("file:///path") === 8        // "file:///"
         * getRootLength("file:///c:") === 10         // "file:///c:"
         * getRootLength("file:///c:d") === 8         // "file:///"
         * getRootLength("file:///c:/path") === 11    // "file:///c:/"
         * getRootLength("file://server") === 13      // "file://server"
         * getRootLength("file://server/path") === 14 // "file://server/"
         * getRootLength("http://server") === 13      // "http://server"
         * getRootLength("http://server/path") === 14 // "http://server/"
         * ```
         */
        function getRootLength(path) {
            var rootLength = getEncodedRootLength(path);
            return rootLength < 0 ? ~rootLength : rootLength;
        }
        ts.getRootLength = getRootLength;
        function getDirectoryPath(path) {
            path = normalizeSlashes(path);
            // If the path provided is itself the root, then return it.
            var rootLength = getRootLength(path);
            if (rootLength === path.length)
                return path;
            // return the leading portion of the path up to the last (non-terminal) directory separator
            // but not including any trailing directory separator.
            path = removeTrailingDirectorySeparator(path);
            return path.slice(0, Math.max(rootLength, path.lastIndexOf(ts.directorySeparator)));
        }
        ts.getDirectoryPath = getDirectoryPath;
        function getBaseFileName(path, extensions, ignoreCase) {
            path = normalizeSlashes(path);
            // if the path provided is itself the root, then it has not file name.
            var rootLength = getRootLength(path);
            if (rootLength === path.length)
                return "";
            // return the trailing portion of the path starting after the last (non-terminal) directory
            // separator but not including any trailing directory separator.
            path = removeTrailingDirectorySeparator(path);
            var name = path.slice(Math.max(getRootLength(path), path.lastIndexOf(ts.directorySeparator) + 1));
            var extension = extensions !== undefined && ignoreCase !== undefined ? getAnyExtensionFromPath(name, extensions, ignoreCase) : undefined;
            return extension ? name.slice(0, name.length - extension.length) : name;
        }
        ts.getBaseFileName = getBaseFileName;
        function tryGetExtensionFromPath(path, extension, stringEqualityComparer) {
            if (!ts.startsWith(extension, "."))
                extension = "." + extension;
            if (path.length >= extension.length && path.charCodeAt(path.length - extension.length) === 46 /* dot */) {
                var pathExtension = path.slice(path.length - extension.length);
                if (stringEqualityComparer(pathExtension, extension)) {
                    return pathExtension;
                }
            }
        }
        function getAnyExtensionFromPathWorker(path, extensions, stringEqualityComparer) {
            if (typeof extensions === "string") {
                return tryGetExtensionFromPath(path, extensions, stringEqualityComparer) || "";
            }
            for (var _i = 0, extensions_2 = extensions; _i < extensions_2.length; _i++) {
                var extension = extensions_2[_i];
                var result = tryGetExtensionFromPath(path, extension, stringEqualityComparer);
                if (result)
                    return result;
            }
            return "";
        }
        function getAnyExtensionFromPath(path, extensions, ignoreCase) {
            // Retrieves any string from the final "." onwards from a base file name.
            // Unlike extensionFromPath, which throws an exception on unrecognized extensions.
            if (extensions) {
                return getAnyExtensionFromPathWorker(removeTrailingDirectorySeparator(path), extensions, ignoreCase ? ts.equateStringsCaseInsensitive : ts.equateStringsCaseSensitive);
            }
            var baseFileName = getBaseFileName(path);
            var extensionIndex = baseFileName.lastIndexOf(".");
            if (extensionIndex >= 0) {
                return baseFileName.substring(extensionIndex);
            }
            return "";
        }
        ts.getAnyExtensionFromPath = getAnyExtensionFromPath;
        function pathComponents(path, rootLength) {
            var root = path.substring(0, rootLength);
            var rest = path.substring(rootLength).split(ts.directorySeparator);
            if (rest.length && !ts.lastOrUndefined(rest))
                rest.pop();
            return __spreadArrays([root], rest);
        }
        /**
         * Parse a path into an array containing a root component (at index 0) and zero or more path
         * components (at indices > 0). The result is not normalized.
         * If the path is relative, the root component is `""`.
         * If the path is absolute, the root component includes the first path separator (`/`).
         *
         * ```ts
         * // POSIX
         * getPathComponents("/path/to/file.ext") === ["/", "path", "to", "file.ext"]
         * getPathComponents("/path/to/") === ["/", "path", "to"]
         * getPathComponents("/") === ["/"]
         * // DOS
         * getPathComponents("c:/path/to/file.ext") === ["c:/", "path", "to", "file.ext"]
         * getPathComponents("c:/path/to/") === ["c:/", "path", "to"]
         * getPathComponents("c:/") === ["c:/"]
         * getPathComponents("c:") === ["c:"]
         * // URL
         * getPathComponents("http://typescriptlang.org/path/to/file.ext") === ["http://typescriptlang.org/", "path", "to", "file.ext"]
         * getPathComponents("http://typescriptlang.org/path/to/") === ["http://typescriptlang.org/", "path", "to"]
         * getPathComponents("http://typescriptlang.org/") === ["http://typescriptlang.org/"]
         * getPathComponents("http://typescriptlang.org") === ["http://typescriptlang.org"]
         * getPathComponents("file://server/path/to/file.ext") === ["file://server/", "path", "to", "file.ext"]
         * getPathComponents("file://server/path/to/") === ["file://server/", "path", "to"]
         * getPathComponents("file://server/") === ["file://server/"]
         * getPathComponents("file://server") === ["file://server"]
         * getPathComponents("file:///path/to/file.ext") === ["file:///", "path", "to", "file.ext"]
         * getPathComponents("file:///path/to/") === ["file:///", "path", "to"]
         * getPathComponents("file:///") === ["file:///"]
         * getPathComponents("file://") === ["file://"]
         */
        function getPathComponents(path, currentDirectory) {
            if (currentDirectory === void 0) {
                currentDirectory = "";
            }
            path = combinePaths(currentDirectory, path);
            return pathComponents(path, getRootLength(path));
        }
        ts.getPathComponents = getPathComponents;
        //// Path Formatting
        /**
         * Formats a parsed path consisting of a root component (at index 0) and zero or more path
         * segments (at indices > 0).
         *
         * ```ts
         * getPathFromPathComponents(["/", "path", "to", "file.ext"]) === "/path/to/file.ext"
         * ```
         */
        function getPathFromPathComponents(pathComponents) {
            if (pathComponents.length === 0)
                return "";
            var root = pathComponents[0] && ensureTrailingDirectorySeparator(pathComponents[0]);
            return root + pathComponents.slice(1).join(ts.directorySeparator);
        }
        ts.getPathFromPathComponents = getPathFromPathComponents;
        //// Path Normalization
        /**
         * Normalize path separators, converting `\` into `/`.
         */
        function normalizeSlashes(path) {
            return path.replace(backslashRegExp, ts.directorySeparator);
        }
        ts.normalizeSlashes = normalizeSlashes;
        /**
         * Reduce an array of path components to a more simplified path by navigating any
         * `"."` or `".."` entries in the path.
         */
        function reducePathComponents(components) {
            if (!ts.some(components))
                return [];
            var reduced = [components[0]];
            for (var i = 1; i < components.length; i++) {
                var component = components[i];
                if (!component)
                    continue;
                if (component === ".")
                    continue;
                if (component === "..") {
                    if (reduced.length > 1) {
                        if (reduced[reduced.length - 1] !== "..") {
                            reduced.pop();
                            continue;
                        }
                    }
                    else if (reduced[0])
                        continue;
                }
                reduced.push(component);
            }
            return reduced;
        }
        ts.reducePathComponents = reducePathComponents;
        /**
         * Combines paths. If a path is absolute, it replaces any previous path. Relative paths are not simplified.
         *
         * ```ts
         * // Non-rooted
         * combinePaths("path", "to", "file.ext") === "path/to/file.ext"
         * combinePaths("path", "dir", "..", "to", "file.ext") === "path/dir/../to/file.ext"
         * // POSIX
         * combinePaths("/path", "to", "file.ext") === "/path/to/file.ext"
         * combinePaths("/path", "/to", "file.ext") === "/to/file.ext"
         * // DOS
         * combinePaths("c:/path", "to", "file.ext") === "c:/path/to/file.ext"
         * combinePaths("c:/path", "c:/to", "file.ext") === "c:/to/file.ext"
         * // URL
         * combinePaths("file:///path", "to", "file.ext") === "file:///path/to/file.ext"
         * combinePaths("file:///path", "file:///to", "file.ext") === "file:///to/file.ext"
         * ```
         */
        function combinePaths(path) {
            var paths = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                paths[_i - 1] = arguments[_i];
            }
            if (path)
                path = normalizeSlashes(path);
            for (var _a = 0, paths_1 = paths; _a < paths_1.length; _a++) {
                var relativePath = paths_1[_a];
                if (!relativePath)
                    continue;
                relativePath = normalizeSlashes(relativePath);
                if (!path || getRootLength(relativePath) !== 0) {
                    path = relativePath;
                }
                else {
                    path = ensureTrailingDirectorySeparator(path) + relativePath;
                }
            }
            return path;
        }
        ts.combinePaths = combinePaths;
        /**
         * Combines and resolves paths. If a path is absolute, it replaces any previous path. Any
         * `.` and `..` path components are resolved. Trailing directory separators are preserved.
         *
         * ```ts
         * resolvePath("/path", "to", "file.ext") === "path/to/file.ext"
         * resolvePath("/path", "to", "file.ext/") === "path/to/file.ext/"
         * resolvePath("/path", "dir", "..", "to", "file.ext") === "path/to/file.ext"
         * ```
         */
        function resolvePath(path) {
            var paths = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                paths[_i - 1] = arguments[_i];
            }
            return normalizePath(ts.some(paths) ? combinePaths.apply(void 0, __spreadArrays([path], paths)) : normalizeSlashes(path));
        }
        ts.resolvePath = resolvePath;
        /**
         * Parse a path into an array containing a root component (at index 0) and zero or more path
         * components (at indices > 0). The result is normalized.
         * If the path is relative, the root component is `""`.
         * If the path is absolute, the root component includes the first path separator (`/`).
         *
         * ```ts
         * getNormalizedPathComponents("to/dir/../file.ext", "/path/") === ["/", "path", "to", "file.ext"]
         */
        function getNormalizedPathComponents(path, currentDirectory) {
            return reducePathComponents(getPathComponents(path, currentDirectory));
        }
        ts.getNormalizedPathComponents = getNormalizedPathComponents;
        function getNormalizedAbsolutePath(fileName, currentDirectory) {
            return getPathFromPathComponents(getNormalizedPathComponents(fileName, currentDirectory));
        }
        ts.getNormalizedAbsolutePath = getNormalizedAbsolutePath;
        function normalizePath(path) {
            path = normalizeSlashes(path);
            var normalized = getPathFromPathComponents(reducePathComponents(getPathComponents(path)));
            return normalized && hasTrailingDirectorySeparator(path) ? ensureTrailingDirectorySeparator(normalized) : normalized;
        }
        ts.normalizePath = normalizePath;
        function getPathWithoutRoot(pathComponents) {
            if (pathComponents.length === 0)
                return "";
            return pathComponents.slice(1).join(ts.directorySeparator);
        }
        function getNormalizedAbsolutePathWithoutRoot(fileName, currentDirectory) {
            return getPathWithoutRoot(getNormalizedPathComponents(fileName, currentDirectory));
        }
        ts.getNormalizedAbsolutePathWithoutRoot = getNormalizedAbsolutePathWithoutRoot;
        function toPath(fileName, basePath, getCanonicalFileName) {
            var nonCanonicalizedPath = isRootedDiskPath(fileName)
                ? normalizePath(fileName)
                : getNormalizedAbsolutePath(fileName, basePath);
            return getCanonicalFileName(nonCanonicalizedPath);
        }
        ts.toPath = toPath;
        function normalizePathAndParts(path) {
            path = normalizeSlashes(path);
            var _a = reducePathComponents(getPathComponents(path)), root = _a[0], parts = _a.slice(1);
            if (parts.length) {
                var joinedParts = root + parts.join(ts.directorySeparator);
                return { path: hasTrailingDirectorySeparator(path) ? ensureTrailingDirectorySeparator(joinedParts) : joinedParts, parts: parts };
            }
            else {
                return { path: root, parts: parts };
            }
        }
        ts.normalizePathAndParts = normalizePathAndParts;
        function removeTrailingDirectorySeparator(path) {
            if (hasTrailingDirectorySeparator(path)) {
                return path.substr(0, path.length - 1);
            }
            return path;
        }
        ts.removeTrailingDirectorySeparator = removeTrailingDirectorySeparator;
        function ensureTrailingDirectorySeparator(path) {
            if (!hasTrailingDirectorySeparator(path)) {
                return path + ts.directorySeparator;
            }
            return path;
        }
        ts.ensureTrailingDirectorySeparator = ensureTrailingDirectorySeparator;
        /**
         * Ensures a path is either absolute (prefixed with `/` or `c:`) or dot-relative (prefixed
         * with `./` or `../`) so as not to be confused with an unprefixed module name.
         *
         * ```ts
         * ensurePathIsNonModuleName("/path/to/file.ext") === "/path/to/file.ext"
         * ensurePathIsNonModuleName("./path/to/file.ext") === "./path/to/file.ext"
         * ensurePathIsNonModuleName("../path/to/file.ext") === "../path/to/file.ext"
         * ensurePathIsNonModuleName("path/to/file.ext") === "./path/to/file.ext"
         * ```
         */
        function ensurePathIsNonModuleName(path) {
            return !pathIsAbsolute(path) && !pathIsRelative(path) ? "./" + path : path;
        }
        ts.ensurePathIsNonModuleName = ensurePathIsNonModuleName;
        function changeAnyExtension(path, ext, extensions, ignoreCase) {
            var pathext = extensions !== undefined && ignoreCase !== undefined ? getAnyExtensionFromPath(path, extensions, ignoreCase) : getAnyExtensionFromPath(path);
            return pathext ? path.slice(0, path.length - pathext.length) + (ts.startsWith(ext, ".") ? ext : "." + ext) : path;
        }
        ts.changeAnyExtension = changeAnyExtension;
        //// Path Comparisons
        // check path for these segments: '', '.'. '..'
        var relativePathSegmentRegExp = /(^|\/)\.{0,2}($|\/)/;
        function comparePathsWorker(a, b, componentComparer) {
            if (a === b)
                return 0 /* EqualTo */;
            if (a === undefined)
                return -1 /* LessThan */;
            if (b === undefined)
                return 1 /* GreaterThan */;
            // NOTE: Performance optimization - shortcut if the root segments differ as there would be no
            //       need to perform path reduction.
            var aRoot = a.substring(0, getRootLength(a));
            var bRoot = b.substring(0, getRootLength(b));
            var result = ts.compareStringsCaseInsensitive(aRoot, bRoot);
            if (result !== 0 /* EqualTo */) {
                return result;
            }
            // NOTE: Performance optimization - shortcut if there are no relative path segments in
            //       the non-root portion of the path
            var aRest = a.substring(aRoot.length);
            var bRest = b.substring(bRoot.length);
            if (!relativePathSegmentRegExp.test(aRest) && !relativePathSegmentRegExp.test(bRest)) {
                return componentComparer(aRest, bRest);
            }
            // The path contains a relative path segment. Normalize the paths and perform a slower component
            // by component comparison.
            var aComponents = reducePathComponents(getPathComponents(a));
            var bComponents = reducePathComponents(getPathComponents(b));
            var sharedLength = Math.min(aComponents.length, bComponents.length);
            for (var i = 1; i < sharedLength; i++) {
                var result_1 = componentComparer(aComponents[i], bComponents[i]);
                if (result_1 !== 0 /* EqualTo */) {
                    return result_1;
                }
            }
            return ts.compareValues(aComponents.length, bComponents.length);
        }
        /**
         * Performs a case-sensitive comparison of two paths. Path roots are always compared case-insensitively.
         */
        function comparePathsCaseSensitive(a, b) {
            return comparePathsWorker(a, b, ts.compareStringsCaseSensitive);
        }
        ts.comparePathsCaseSensitive = comparePathsCaseSensitive;
        /**
         * Performs a case-insensitive comparison of two paths.
         */
        function comparePathsCaseInsensitive(a, b) {
            return comparePathsWorker(a, b, ts.compareStringsCaseInsensitive);
        }
        ts.comparePathsCaseInsensitive = comparePathsCaseInsensitive;
        function comparePaths(a, b, currentDirectory, ignoreCase) {
            if (typeof currentDirectory === "string") {
                a = combinePaths(currentDirectory, a);
                b = combinePaths(currentDirectory, b);
            }
            else if (typeof currentDirectory === "boolean") {
                ignoreCase = currentDirectory;
            }
            return comparePathsWorker(a, b, ts.getStringComparer(ignoreCase));
        }
        ts.comparePaths = comparePaths;
        function containsPath(parent, child, currentDirectory, ignoreCase) {
            if (typeof currentDirectory === "string") {
                parent = combinePaths(currentDirectory, parent);
                child = combinePaths(currentDirectory, child);
            }
            else if (typeof currentDirectory === "boolean") {
                ignoreCase = currentDirectory;
            }
            if (parent === undefined || child === undefined)
                return false;
            if (parent === child)
                return true;
            var parentComponents = reducePathComponents(getPathComponents(parent));
            var childComponents = reducePathComponents(getPathComponents(child));
            if (childComponents.length < parentComponents.length) {
                return false;
            }
            var componentEqualityComparer = ignoreCase ? ts.equateStringsCaseInsensitive : ts.equateStringsCaseSensitive;
            for (var i = 0; i < parentComponents.length; i++) {
                var equalityComparer = i === 0 ? ts.equateStringsCaseInsensitive : componentEqualityComparer;
                if (!equalityComparer(parentComponents[i], childComponents[i])) {
                    return false;
                }
            }
            return true;
        }
        ts.containsPath = containsPath;
        /**
         * Determines whether `fileName` starts with the specified `directoryName` using the provided path canonicalization callback.
         * Comparison is case-sensitive between the canonical paths.
         *
         * @deprecated Use `containsPath` if possible.
         */
        function startsWithDirectory(fileName, directoryName, getCanonicalFileName) {
            var canonicalFileName = getCanonicalFileName(fileName);
            var canonicalDirectoryName = getCanonicalFileName(directoryName);
            return ts.startsWith(canonicalFileName, canonicalDirectoryName + "/") || ts.startsWith(canonicalFileName, canonicalDirectoryName + "\\");
        }
        ts.startsWithDirectory = startsWithDirectory;
        //// Relative Paths
        function getPathComponentsRelativeTo(from, to, stringEqualityComparer, getCanonicalFileName) {
            var fromComponents = reducePathComponents(getPathComponents(from));
            var toComponents = reducePathComponents(getPathComponents(to));
            var start;
            for (start = 0; start < fromComponents.length && start < toComponents.length; start++) {
                var fromComponent = getCanonicalFileName(fromComponents[start]);
                var toComponent = getCanonicalFileName(toComponents[start]);
                var comparer = start === 0 ? ts.equateStringsCaseInsensitive : stringEqualityComparer;
                if (!comparer(fromComponent, toComponent))
                    break;
            }
            if (start === 0) {
                return toComponents;
            }
            var components = toComponents.slice(start);
            var relative = [];
            for (; start < fromComponents.length; start++) {
                relative.push("..");
            }
            return __spreadArrays([""], relative, components);
        }
        ts.getPathComponentsRelativeTo = getPathComponentsRelativeTo;
        function getRelativePathFromDirectory(fromDirectory, to, getCanonicalFileNameOrIgnoreCase) {
            ts.Debug.assert((getRootLength(fromDirectory) > 0) === (getRootLength(to) > 0), "Paths must either both be absolute or both be relative");
            var getCanonicalFileName = typeof getCanonicalFileNameOrIgnoreCase === "function" ? getCanonicalFileNameOrIgnoreCase : ts.identity;
            var ignoreCase = typeof getCanonicalFileNameOrIgnoreCase === "boolean" ? getCanonicalFileNameOrIgnoreCase : false;
            var pathComponents = getPathComponentsRelativeTo(fromDirectory, to, ignoreCase ? ts.equateStringsCaseInsensitive : ts.equateStringsCaseSensitive, getCanonicalFileName);
            return getPathFromPathComponents(pathComponents);
        }
        ts.getRelativePathFromDirectory = getRelativePathFromDirectory;
        function convertToRelativePath(absoluteOrRelativePath, basePath, getCanonicalFileName) {
            return !isRootedDiskPath(absoluteOrRelativePath)
                ? absoluteOrRelativePath
                : getRelativePathToDirectoryOrUrl(basePath, absoluteOrRelativePath, basePath, getCanonicalFileName, /*isAbsolutePathAnUrl*/ false);
        }
        ts.convertToRelativePath = convertToRelativePath;
        function getRelativePathFromFile(from, to, getCanonicalFileName) {
            return ensurePathIsNonModuleName(getRelativePathFromDirectory(getDirectoryPath(from), to, getCanonicalFileName));
        }
        ts.getRelativePathFromFile = getRelativePathFromFile;
        function getRelativePathToDirectoryOrUrl(directoryPathOrUrl, relativeOrAbsolutePath, currentDirectory, getCanonicalFileName, isAbsolutePathAnUrl) {
            var pathComponents = getPathComponentsRelativeTo(resolvePath(currentDirectory, directoryPathOrUrl), resolvePath(currentDirectory, relativeOrAbsolutePath), ts.equateStringsCaseSensitive, getCanonicalFileName);
            var firstComponent = pathComponents[0];
            if (isAbsolutePathAnUrl && isRootedDiskPath(firstComponent)) {
                var prefix = firstComponent.charAt(0) === ts.directorySeparator ? "file://" : "file:///";
                pathComponents[0] = prefix + firstComponent;
            }
            return getPathFromPathComponents(pathComponents);
        }
        ts.getRelativePathToDirectoryOrUrl = getRelativePathToDirectoryOrUrl;
        function forEachAncestorDirectory(directory, callback) {
            while (true) {
                var result = callback(directory);
                if (result !== undefined) {
                    return result;
                }
                var parentPath = getDirectoryPath(directory);
                if (parentPath === directory) {
                    return undefined;
                }
                directory = parentPath;
            }
        }
        ts.forEachAncestorDirectory = forEachAncestorDirectory;
    })(ts || (ts = {}));
    // <auto-generated />
    // generated from './diagnosticInformationMap.generated.ts' by 'src/compiler'
    /* @internal */
    var ts;
    (function (ts) {
        function diag(code, category, key, message, reportsUnnecessary, elidedInCompatabilityPyramid) {
            return { code: code, category: category, key: key, message: message, reportsUnnecessary: reportsUnnecessary, elidedInCompatabilityPyramid: elidedInCompatabilityPyramid };
        }
        ts.Diagnostics = {
            Unterminated_string_literal: diag(1002, ts.DiagnosticCategory.Error, "Unterminated_string_literal_1002", "Unterminated string literal."),
            Identifier_expected: diag(1003, ts.DiagnosticCategory.Error, "Identifier_expected_1003", "Identifier expected."),
            _0_expected: diag(1005, ts.DiagnosticCategory.Error, "_0_expected_1005", "'{0}' expected."),
            A_file_cannot_have_a_reference_to_itself: diag(1006, ts.DiagnosticCategory.Error, "A_file_cannot_have_a_reference_to_itself_1006", "A file cannot have a reference to itself."),
            The_parser_expected_to_find_a_to_match_the_token_here: diag(1007, ts.DiagnosticCategory.Error, "The_parser_expected_to_find_a_to_match_the_token_here_1007", "The parser expected to find a '}' to match the '{' token here."),
            Trailing_comma_not_allowed: diag(1009, ts.DiagnosticCategory.Error, "Trailing_comma_not_allowed_1009", "Trailing comma not allowed."),
            Asterisk_Slash_expected: diag(1010, ts.DiagnosticCategory.Error, "Asterisk_Slash_expected_1010", "'*/' expected."),
            An_element_access_expression_should_take_an_argument: diag(1011, ts.DiagnosticCategory.Error, "An_element_access_expression_should_take_an_argument_1011", "An element access expression should take an argument."),
            Unexpected_token: diag(1012, ts.DiagnosticCategory.Error, "Unexpected_token_1012", "Unexpected token."),
            A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma: diag(1013, ts.DiagnosticCategory.Error, "A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma_1013", "A rest parameter or binding pattern may not have a trailing comma."),
            A_rest_parameter_must_be_last_in_a_parameter_list: diag(1014, ts.DiagnosticCategory.Error, "A_rest_parameter_must_be_last_in_a_parameter_list_1014", "A rest parameter must be last in a parameter list."),
            Parameter_cannot_have_question_mark_and_initializer: diag(1015, ts.DiagnosticCategory.Error, "Parameter_cannot_have_question_mark_and_initializer_1015", "Parameter cannot have question mark and initializer."),
            A_required_parameter_cannot_follow_an_optional_parameter: diag(1016, ts.DiagnosticCategory.Error, "A_required_parameter_cannot_follow_an_optional_parameter_1016", "A required parameter cannot follow an optional parameter."),
            An_index_signature_cannot_have_a_rest_parameter: diag(1017, ts.DiagnosticCategory.Error, "An_index_signature_cannot_have_a_rest_parameter_1017", "An index signature cannot have a rest parameter."),
            An_index_signature_parameter_cannot_have_an_accessibility_modifier: diag(1018, ts.DiagnosticCategory.Error, "An_index_signature_parameter_cannot_have_an_accessibility_modifier_1018", "An index signature parameter cannot have an accessibility modifier."),
            An_index_signature_parameter_cannot_have_a_question_mark: diag(1019, ts.DiagnosticCategory.Error, "An_index_signature_parameter_cannot_have_a_question_mark_1019", "An index signature parameter cannot have a question mark."),
            An_index_signature_parameter_cannot_have_an_initializer: diag(1020, ts.DiagnosticCategory.Error, "An_index_signature_parameter_cannot_have_an_initializer_1020", "An index signature parameter cannot have an initializer."),
            An_index_signature_must_have_a_type_annotation: diag(1021, ts.DiagnosticCategory.Error, "An_index_signature_must_have_a_type_annotation_1021", "An index signature must have a type annotation."),
            An_index_signature_parameter_must_have_a_type_annotation: diag(1022, ts.DiagnosticCategory.Error, "An_index_signature_parameter_must_have_a_type_annotation_1022", "An index signature parameter must have a type annotation."),
            An_index_signature_parameter_type_must_be_either_string_or_number: diag(1023, ts.DiagnosticCategory.Error, "An_index_signature_parameter_type_must_be_either_string_or_number_1023", "An index signature parameter type must be either 'string' or 'number'."),
            readonly_modifier_can_only_appear_on_a_property_declaration_or_index_signature: diag(1024, ts.DiagnosticCategory.Error, "readonly_modifier_can_only_appear_on_a_property_declaration_or_index_signature_1024", "'readonly' modifier can only appear on a property declaration or index signature."),
            Accessibility_modifier_already_seen: diag(1028, ts.DiagnosticCategory.Error, "Accessibility_modifier_already_seen_1028", "Accessibility modifier already seen."),
            _0_modifier_must_precede_1_modifier: diag(1029, ts.DiagnosticCategory.Error, "_0_modifier_must_precede_1_modifier_1029", "'{0}' modifier must precede '{1}' modifier."),
            _0_modifier_already_seen: diag(1030, ts.DiagnosticCategory.Error, "_0_modifier_already_seen_1030", "'{0}' modifier already seen."),
            _0_modifier_cannot_appear_on_a_class_element: diag(1031, ts.DiagnosticCategory.Error, "_0_modifier_cannot_appear_on_a_class_element_1031", "'{0}' modifier cannot appear on a class element."),
            super_must_be_followed_by_an_argument_list_or_member_access: diag(1034, ts.DiagnosticCategory.Error, "super_must_be_followed_by_an_argument_list_or_member_access_1034", "'super' must be followed by an argument list or member access."),
            Only_ambient_modules_can_use_quoted_names: diag(1035, ts.DiagnosticCategory.Error, "Only_ambient_modules_can_use_quoted_names_1035", "Only ambient modules can use quoted names."),
            Statements_are_not_allowed_in_ambient_contexts: diag(1036, ts.DiagnosticCategory.Error, "Statements_are_not_allowed_in_ambient_contexts_1036", "Statements are not allowed in ambient contexts."),
            A_declare_modifier_cannot_be_used_in_an_already_ambient_context: diag(1038, ts.DiagnosticCategory.Error, "A_declare_modifier_cannot_be_used_in_an_already_ambient_context_1038", "A 'declare' modifier cannot be used in an already ambient context."),
            Initializers_are_not_allowed_in_ambient_contexts: diag(1039, ts.DiagnosticCategory.Error, "Initializers_are_not_allowed_in_ambient_contexts_1039", "Initializers are not allowed in ambient contexts."),
            _0_modifier_cannot_be_used_in_an_ambient_context: diag(1040, ts.DiagnosticCategory.Error, "_0_modifier_cannot_be_used_in_an_ambient_context_1040", "'{0}' modifier cannot be used in an ambient context."),
            _0_modifier_cannot_be_used_with_a_class_declaration: diag(1041, ts.DiagnosticCategory.Error, "_0_modifier_cannot_be_used_with_a_class_declaration_1041", "'{0}' modifier cannot be used with a class declaration."),
            _0_modifier_cannot_be_used_here: diag(1042, ts.DiagnosticCategory.Error, "_0_modifier_cannot_be_used_here_1042", "'{0}' modifier cannot be used here."),
            _0_modifier_cannot_appear_on_a_data_property: diag(1043, ts.DiagnosticCategory.Error, "_0_modifier_cannot_appear_on_a_data_property_1043", "'{0}' modifier cannot appear on a data property."),
            _0_modifier_cannot_appear_on_a_module_or_namespace_element: diag(1044, ts.DiagnosticCategory.Error, "_0_modifier_cannot_appear_on_a_module_or_namespace_element_1044", "'{0}' modifier cannot appear on a module or namespace element."),
            A_0_modifier_cannot_be_used_with_an_interface_declaration: diag(1045, ts.DiagnosticCategory.Error, "A_0_modifier_cannot_be_used_with_an_interface_declaration_1045", "A '{0}' modifier cannot be used with an interface declaration."),
            Top_level_declarations_in_d_ts_files_must_start_with_either_a_declare_or_export_modifier: diag(1046, ts.DiagnosticCategory.Error, "Top_level_declarations_in_d_ts_files_must_start_with_either_a_declare_or_export_modifier_1046", "Top-level declarations in .d.ts files must start with either a 'declare' or 'export' modifier."),
            A_rest_parameter_cannot_be_optional: diag(1047, ts.DiagnosticCategory.Error, "A_rest_parameter_cannot_be_optional_1047", "A rest parameter cannot be optional."),
            A_rest_parameter_cannot_have_an_initializer: diag(1048, ts.DiagnosticCategory.Error, "A_rest_parameter_cannot_have_an_initializer_1048", "A rest parameter cannot have an initializer."),
            A_set_accessor_must_have_exactly_one_parameter: diag(1049, ts.DiagnosticCategory.Error, "A_set_accessor_must_have_exactly_one_parameter_1049", "A 'set' accessor must have exactly one parameter."),
            A_set_accessor_cannot_have_an_optional_parameter: diag(1051, ts.DiagnosticCategory.Error, "A_set_accessor_cannot_have_an_optional_parameter_1051", "A 'set' accessor cannot have an optional parameter."),
            A_set_accessor_parameter_cannot_have_an_initializer: diag(1052, ts.DiagnosticCategory.Error, "A_set_accessor_parameter_cannot_have_an_initializer_1052", "A 'set' accessor parameter cannot have an initializer."),
            A_set_accessor_cannot_have_rest_parameter: diag(1053, ts.DiagnosticCategory.Error, "A_set_accessor_cannot_have_rest_parameter_1053", "A 'set' accessor cannot have rest parameter."),
            A_get_accessor_cannot_have_parameters: diag(1054, ts.DiagnosticCategory.Error, "A_get_accessor_cannot_have_parameters_1054", "A 'get' accessor cannot have parameters."),
            Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value: diag(1055, ts.DiagnosticCategory.Error, "Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Prom_1055", "Type '{0}' is not a valid async function return type in ES5/ES3 because it does not refer to a Promise-compatible constructor value."),
            Accessors_are_only_available_when_targeting_ECMAScript_5_and_higher: diag(1056, ts.DiagnosticCategory.Error, "Accessors_are_only_available_when_targeting_ECMAScript_5_and_higher_1056", "Accessors are only available when targeting ECMAScript 5 and higher."),
            An_async_function_or_method_must_have_a_valid_awaitable_return_type: diag(1057, ts.DiagnosticCategory.Error, "An_async_function_or_method_must_have_a_valid_awaitable_return_type_1057", "An async function or method must have a valid awaitable return type."),
            The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member: diag(1058, ts.DiagnosticCategory.Error, "The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_t_1058", "The return type of an async function must either be a valid promise or must not contain a callable 'then' member."),
            A_promise_must_have_a_then_method: diag(1059, ts.DiagnosticCategory.Error, "A_promise_must_have_a_then_method_1059", "A promise must have a 'then' method."),
            The_first_parameter_of_the_then_method_of_a_promise_must_be_a_callback: diag(1060, ts.DiagnosticCategory.Error, "The_first_parameter_of_the_then_method_of_a_promise_must_be_a_callback_1060", "The first parameter of the 'then' method of a promise must be a callback."),
            Enum_member_must_have_initializer: diag(1061, ts.DiagnosticCategory.Error, "Enum_member_must_have_initializer_1061", "Enum member must have initializer."),
            Type_is_referenced_directly_or_indirectly_in_the_fulfillment_callback_of_its_own_then_method: diag(1062, ts.DiagnosticCategory.Error, "Type_is_referenced_directly_or_indirectly_in_the_fulfillment_callback_of_its_own_then_method_1062", "Type is referenced directly or indirectly in the fulfillment callback of its own 'then' method."),
            An_export_assignment_cannot_be_used_in_a_namespace: diag(1063, ts.DiagnosticCategory.Error, "An_export_assignment_cannot_be_used_in_a_namespace_1063", "An export assignment cannot be used in a namespace."),
            The_return_type_of_an_async_function_or_method_must_be_the_global_Promise_T_type: diag(1064, ts.DiagnosticCategory.Error, "The_return_type_of_an_async_function_or_method_must_be_the_global_Promise_T_type_1064", "The return type of an async function or method must be the global Promise<T> type."),
            In_ambient_enum_declarations_member_initializer_must_be_constant_expression: diag(1066, ts.DiagnosticCategory.Error, "In_ambient_enum_declarations_member_initializer_must_be_constant_expression_1066", "In ambient enum declarations member initializer must be constant expression."),
            Unexpected_token_A_constructor_method_accessor_or_property_was_expected: diag(1068, ts.DiagnosticCategory.Error, "Unexpected_token_A_constructor_method_accessor_or_property_was_expected_1068", "Unexpected token. A constructor, method, accessor, or property was expected."),
            Unexpected_token_A_type_parameter_name_was_expected_without_curly_braces: diag(1069, ts.DiagnosticCategory.Error, "Unexpected_token_A_type_parameter_name_was_expected_without_curly_braces_1069", "Unexpected token. A type parameter name was expected without curly braces."),
            _0_modifier_cannot_appear_on_a_type_member: diag(1070, ts.DiagnosticCategory.Error, "_0_modifier_cannot_appear_on_a_type_member_1070", "'{0}' modifier cannot appear on a type member."),
            _0_modifier_cannot_appear_on_an_index_signature: diag(1071, ts.DiagnosticCategory.Error, "_0_modifier_cannot_appear_on_an_index_signature_1071", "'{0}' modifier cannot appear on an index signature."),
            A_0_modifier_cannot_be_used_with_an_import_declaration: diag(1079, ts.DiagnosticCategory.Error, "A_0_modifier_cannot_be_used_with_an_import_declaration_1079", "A '{0}' modifier cannot be used with an import declaration."),
            Invalid_reference_directive_syntax: diag(1084, ts.DiagnosticCategory.Error, "Invalid_reference_directive_syntax_1084", "Invalid 'reference' directive syntax."),
            Octal_literals_are_not_available_when_targeting_ECMAScript_5_and_higher_Use_the_syntax_0: diag(1085, ts.DiagnosticCategory.Error, "Octal_literals_are_not_available_when_targeting_ECMAScript_5_and_higher_Use_the_syntax_0_1085", "Octal literals are not available when targeting ECMAScript 5 and higher. Use the syntax '{0}'."),
            _0_modifier_cannot_appear_on_a_constructor_declaration: diag(1089, ts.DiagnosticCategory.Error, "_0_modifier_cannot_appear_on_a_constructor_declaration_1089", "'{0}' modifier cannot appear on a constructor declaration."),
            _0_modifier_cannot_appear_on_a_parameter: diag(1090, ts.DiagnosticCategory.Error, "_0_modifier_cannot_appear_on_a_parameter_1090", "'{0}' modifier cannot appear on a parameter."),
            Only_a_single_variable_declaration_is_allowed_in_a_for_in_statement: diag(1091, ts.DiagnosticCategory.Error, "Only_a_single_variable_declaration_is_allowed_in_a_for_in_statement_1091", "Only a single variable declaration is allowed in a 'for...in' statement."),
            Type_parameters_cannot_appear_on_a_constructor_declaration: diag(1092, ts.DiagnosticCategory.Error, "Type_parameters_cannot_appear_on_a_constructor_declaration_1092", "Type parameters cannot appear on a constructor declaration."),
            Type_annotation_cannot_appear_on_a_constructor_declaration: diag(1093, ts.DiagnosticCategory.Error, "Type_annotation_cannot_appear_on_a_constructor_declaration_1093", "Type annotation cannot appear on a constructor declaration."),
            An_accessor_cannot_have_type_parameters: diag(1094, ts.DiagnosticCategory.Error, "An_accessor_cannot_have_type_parameters_1094", "An accessor cannot have type parameters."),
            A_set_accessor_cannot_have_a_return_type_annotation: diag(1095, ts.DiagnosticCategory.Error, "A_set_accessor_cannot_have_a_return_type_annotation_1095", "A 'set' accessor cannot have a return type annotation."),
            An_index_signature_must_have_exactly_one_parameter: diag(1096, ts.DiagnosticCategory.Error, "An_index_signature_must_have_exactly_one_parameter_1096", "An index signature must have exactly one parameter."),
            _0_list_cannot_be_empty: diag(1097, ts.DiagnosticCategory.Error, "_0_list_cannot_be_empty_1097", "'{0}' list cannot be empty."),
            Type_parameter_list_cannot_be_empty: diag(1098, ts.DiagnosticCategory.Error, "Type_parameter_list_cannot_be_empty_1098", "Type parameter list cannot be empty."),
            Type_argument_list_cannot_be_empty: diag(1099, ts.DiagnosticCategory.Error, "Type_argument_list_cannot_be_empty_1099", "Type argument list cannot be empty."),
            Invalid_use_of_0_in_strict_mode: diag(1100, ts.DiagnosticCategory.Error, "Invalid_use_of_0_in_strict_mode_1100", "Invalid use of '{0}' in strict mode."),
            with_statements_are_not_allowed_in_strict_mode: diag(1101, ts.DiagnosticCategory.Error, "with_statements_are_not_allowed_in_strict_mode_1101", "'with' statements are not allowed in strict mode."),
            delete_cannot_be_called_on_an_identifier_in_strict_mode: diag(1102, ts.DiagnosticCategory.Error, "delete_cannot_be_called_on_an_identifier_in_strict_mode_1102", "'delete' cannot be called on an identifier in strict mode."),
            A_for_await_of_statement_is_only_allowed_within_an_async_function_or_async_generator: diag(1103, ts.DiagnosticCategory.Error, "A_for_await_of_statement_is_only_allowed_within_an_async_function_or_async_generator_1103", "A 'for-await-of' statement is only allowed within an async function or async generator."),
            A_continue_statement_can_only_be_used_within_an_enclosing_iteration_statement: diag(1104, ts.DiagnosticCategory.Error, "A_continue_statement_can_only_be_used_within_an_enclosing_iteration_statement_1104", "A 'continue' statement can only be used within an enclosing iteration statement."),
            A_break_statement_can_only_be_used_within_an_enclosing_iteration_or_switch_statement: diag(1105, ts.DiagnosticCategory.Error, "A_break_statement_can_only_be_used_within_an_enclosing_iteration_or_switch_statement_1105", "A 'break' statement can only be used within an enclosing iteration or switch statement."),
            Jump_target_cannot_cross_function_boundary: diag(1107, ts.DiagnosticCategory.Error, "Jump_target_cannot_cross_function_boundary_1107", "Jump target cannot cross function boundary."),
            A_return_statement_can_only_be_used_within_a_function_body: diag(1108, ts.DiagnosticCategory.Error, "A_return_statement_can_only_be_used_within_a_function_body_1108", "A 'return' statement can only be used within a function body."),
            Expression_expected: diag(1109, ts.DiagnosticCategory.Error, "Expression_expected_1109", "Expression expected."),
            Type_expected: diag(1110, ts.DiagnosticCategory.Error, "Type_expected_1110", "Type expected."),
            A_default_clause_cannot_appear_more_than_once_in_a_switch_statement: diag(1113, ts.DiagnosticCategory.Error, "A_default_clause_cannot_appear_more_than_once_in_a_switch_statement_1113", "A 'default' clause cannot appear more than once in a 'switch' statement."),
            Duplicate_label_0: diag(1114, ts.DiagnosticCategory.Error, "Duplicate_label_0_1114", "Duplicate label '{0}'."),
            A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement: diag(1115, ts.DiagnosticCategory.Error, "A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement_1115", "A 'continue' statement can only jump to a label of an enclosing iteration statement."),
            A_break_statement_can_only_jump_to_a_label_of_an_enclosing_statement: diag(1116, ts.DiagnosticCategory.Error, "A_break_statement_can_only_jump_to_a_label_of_an_enclosing_statement_1116", "A 'break' statement can only jump to a label of an enclosing statement."),
            An_object_literal_cannot_have_multiple_properties_with_the_same_name_in_strict_mode: diag(1117, ts.DiagnosticCategory.Error, "An_object_literal_cannot_have_multiple_properties_with_the_same_name_in_strict_mode_1117", "An object literal cannot have multiple properties with the same name in strict mode."),
            An_object_literal_cannot_have_multiple_get_Slashset_accessors_with_the_same_name: diag(1118, ts.DiagnosticCategory.Error, "An_object_literal_cannot_have_multiple_get_Slashset_accessors_with_the_same_name_1118", "An object literal cannot have multiple get/set accessors with the same name."),
            An_object_literal_cannot_have_property_and_accessor_with_the_same_name: diag(1119, ts.DiagnosticCategory.Error, "An_object_literal_cannot_have_property_and_accessor_with_the_same_name_1119", "An object literal cannot have property and accessor with the same name."),
            An_export_assignment_cannot_have_modifiers: diag(1120, ts.DiagnosticCategory.Error, "An_export_assignment_cannot_have_modifiers_1120", "An export assignment cannot have modifiers."),
            Octal_literals_are_not_allowed_in_strict_mode: diag(1121, ts.DiagnosticCategory.Error, "Octal_literals_are_not_allowed_in_strict_mode_1121", "Octal literals are not allowed in strict mode."),
            Variable_declaration_list_cannot_be_empty: diag(1123, ts.DiagnosticCategory.Error, "Variable_declaration_list_cannot_be_empty_1123", "Variable declaration list cannot be empty."),
            Digit_expected: diag(1124, ts.DiagnosticCategory.Error, "Digit_expected_1124", "Digit expected."),
            Hexadecimal_digit_expected: diag(1125, ts.DiagnosticCategory.Error, "Hexadecimal_digit_expected_1125", "Hexadecimal digit expected."),
            Unexpected_end_of_text: diag(1126, ts.DiagnosticCategory.Error, "Unexpected_end_of_text_1126", "Unexpected end of text."),
            Invalid_character: diag(1127, ts.DiagnosticCategory.Error, "Invalid_character_1127", "Invalid character."),
            Declaration_or_statement_expected: diag(1128, ts.DiagnosticCategory.Error, "Declaration_or_statement_expected_1128", "Declaration or statement expected."),
            Statement_expected: diag(1129, ts.DiagnosticCategory.Error, "Statement_expected_1129", "Statement expected."),
            case_or_default_expected: diag(1130, ts.DiagnosticCategory.Error, "case_or_default_expected_1130", "'case' or 'default' expected."),
            Property_or_signature_expected: diag(1131, ts.DiagnosticCategory.Error, "Property_or_signature_expected_1131", "Property or signature expected."),
            Enum_member_expected: diag(1132, ts.DiagnosticCategory.Error, "Enum_member_expected_1132", "Enum member expected."),
            Variable_declaration_expected: diag(1134, ts.DiagnosticCategory.Error, "Variable_declaration_expected_1134", "Variable declaration expected."),
            Argument_expression_expected: diag(1135, ts.DiagnosticCategory.Error, "Argument_expression_expected_1135", "Argument expression expected."),
            Property_assignment_expected: diag(1136, ts.DiagnosticCategory.Error, "Property_assignment_expected_1136", "Property assignment expected."),
            Expression_or_comma_expected: diag(1137, ts.DiagnosticCategory.Error, "Expression_or_comma_expected_1137", "Expression or comma expected."),
            Parameter_declaration_expected: diag(1138, ts.DiagnosticCategory.Error, "Parameter_declaration_expected_1138", "Parameter declaration expected."),
            Type_parameter_declaration_expected: diag(1139, ts.DiagnosticCategory.Error, "Type_parameter_declaration_expected_1139", "Type parameter declaration expected."),
            Type_argument_expected: diag(1140, ts.DiagnosticCategory.Error, "Type_argument_expected_1140", "Type argument expected."),
            String_literal_expected: diag(1141, ts.DiagnosticCategory.Error, "String_literal_expected_1141", "String literal expected."),
            Line_break_not_permitted_here: diag(1142, ts.DiagnosticCategory.Error, "Line_break_not_permitted_here_1142", "Line break not permitted here."),
            or_expected: diag(1144, ts.DiagnosticCategory.Error, "or_expected_1144", "'{' or ';' expected."),
            Declaration_expected: diag(1146, ts.DiagnosticCategory.Error, "Declaration_expected_1146", "Declaration expected."),
            Import_declarations_in_a_namespace_cannot_reference_a_module: diag(1147, ts.DiagnosticCategory.Error, "Import_declarations_in_a_namespace_cannot_reference_a_module_1147", "Import declarations in a namespace cannot reference a module."),
            Cannot_use_imports_exports_or_module_augmentations_when_module_is_none: diag(1148, ts.DiagnosticCategory.Error, "Cannot_use_imports_exports_or_module_augmentations_when_module_is_none_1148", "Cannot use imports, exports, or module augmentations when '--module' is 'none'."),
            File_name_0_differs_from_already_included_file_name_1_only_in_casing: diag(1149, ts.DiagnosticCategory.Error, "File_name_0_differs_from_already_included_file_name_1_only_in_casing_1149", "File name '{0}' differs from already included file name '{1}' only in casing."),
            const_declarations_must_be_initialized: diag(1155, ts.DiagnosticCategory.Error, "const_declarations_must_be_initialized_1155", "'const' declarations must be initialized."),
            const_declarations_can_only_be_declared_inside_a_block: diag(1156, ts.DiagnosticCategory.Error, "const_declarations_can_only_be_declared_inside_a_block_1156", "'const' declarations can only be declared inside a block."),
            let_declarations_can_only_be_declared_inside_a_block: diag(1157, ts.DiagnosticCategory.Error, "let_declarations_can_only_be_declared_inside_a_block_1157", "'let' declarations can only be declared inside a block."),
            Unterminated_template_literal: diag(1160, ts.DiagnosticCategory.Error, "Unterminated_template_literal_1160", "Unterminated template literal."),
            Unterminated_regular_expression_literal: diag(1161, ts.DiagnosticCategory.Error, "Unterminated_regular_expression_literal_1161", "Unterminated regular expression literal."),
            An_object_member_cannot_be_declared_optional: diag(1162, ts.DiagnosticCategory.Error, "An_object_member_cannot_be_declared_optional_1162", "An object member cannot be declared optional."),
            A_yield_expression_is_only_allowed_in_a_generator_body: diag(1163, ts.DiagnosticCategory.Error, "A_yield_expression_is_only_allowed_in_a_generator_body_1163", "A 'yield' expression is only allowed in a generator body."),
            Computed_property_names_are_not_allowed_in_enums: diag(1164, ts.DiagnosticCategory.Error, "Computed_property_names_are_not_allowed_in_enums_1164", "Computed property names are not allowed in enums."),
            A_computed_property_name_in_an_ambient_context_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type: diag(1165, ts.DiagnosticCategory.Error, "A_computed_property_name_in_an_ambient_context_must_refer_to_an_expression_whose_type_is_a_literal_t_1165", "A computed property name in an ambient context must refer to an expression whose type is a literal type or a 'unique symbol' type."),
            A_computed_property_name_in_a_class_property_declaration_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type: diag(1166, ts.DiagnosticCategory.Error, "A_computed_property_name_in_a_class_property_declaration_must_refer_to_an_expression_whose_type_is_a_1166", "A computed property name in a class property declaration must refer to an expression whose type is a literal type or a 'unique symbol' type."),
            A_computed_property_name_in_a_method_overload_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type: diag(1168, ts.DiagnosticCategory.Error, "A_computed_property_name_in_a_method_overload_must_refer_to_an_expression_whose_type_is_a_literal_ty_1168", "A computed property name in a method overload must refer to an expression whose type is a literal type or a 'unique symbol' type."),
            A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type: diag(1169, ts.DiagnosticCategory.Error, "A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_1169", "A computed property name in an interface must refer to an expression whose type is a literal type or a 'unique symbol' type."),
            A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type: diag(1170, ts.DiagnosticCategory.Error, "A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type__1170", "A computed property name in a type literal must refer to an expression whose type is a literal type or a 'unique symbol' type."),
            A_comma_expression_is_not_allowed_in_a_computed_property_name: diag(1171, ts.DiagnosticCategory.Error, "A_comma_expression_is_not_allowed_in_a_computed_property_name_1171", "A comma expression is not allowed in a computed property name."),
            extends_clause_already_seen: diag(1172, ts.DiagnosticCategory.Error, "extends_clause_already_seen_1172", "'extends' clause already seen."),
            extends_clause_must_precede_implements_clause: diag(1173, ts.DiagnosticCategory.Error, "extends_clause_must_precede_implements_clause_1173", "'extends' clause must precede 'implements' clause."),
            Classes_can_only_extend_a_single_class: diag(1174, ts.DiagnosticCategory.Error, "Classes_can_only_extend_a_single_class_1174", "Classes can only extend a single class."),
            implements_clause_already_seen: diag(1175, ts.DiagnosticCategory.Error, "implements_clause_already_seen_1175", "'implements' clause already seen."),
            Interface_declaration_cannot_have_implements_clause: diag(1176, ts.DiagnosticCategory.Error, "Interface_declaration_cannot_have_implements_clause_1176", "Interface declaration cannot have 'implements' clause."),
            Binary_digit_expected: diag(1177, ts.DiagnosticCategory.Error, "Binary_digit_expected_1177", "Binary digit expected."),
            Octal_digit_expected: diag(1178, ts.DiagnosticCategory.Error, "Octal_digit_expected_1178", "Octal digit expected."),
            Unexpected_token_expected: diag(1179, ts.DiagnosticCategory.Error, "Unexpected_token_expected_1179", "Unexpected token. '{' expected."),
            Property_destructuring_pattern_expected: diag(1180, ts.DiagnosticCategory.Error, "Property_destructuring_pattern_expected_1180", "Property destructuring pattern expected."),
            Array_element_destructuring_pattern_expected: diag(1181, ts.DiagnosticCategory.Error, "Array_element_destructuring_pattern_expected_1181", "Array element destructuring pattern expected."),
            A_destructuring_declaration_must_have_an_initializer: diag(1182, ts.DiagnosticCategory.Error, "A_destructuring_declaration_must_have_an_initializer_1182", "A destructuring declaration must have an initializer."),
            An_implementation_cannot_be_declared_in_ambient_contexts: diag(1183, ts.DiagnosticCategory.Error, "An_implementation_cannot_be_declared_in_ambient_contexts_1183", "An implementation cannot be declared in ambient contexts."),
            Modifiers_cannot_appear_here: diag(1184, ts.DiagnosticCategory.Error, "Modifiers_cannot_appear_here_1184", "Modifiers cannot appear here."),
            Merge_conflict_marker_encountered: diag(1185, ts.DiagnosticCategory.Error, "Merge_conflict_marker_encountered_1185", "Merge conflict marker encountered."),
            A_rest_element_cannot_have_an_initializer: diag(1186, ts.DiagnosticCategory.Error, "A_rest_element_cannot_have_an_initializer_1186", "A rest element cannot have an initializer."),
            A_parameter_property_may_not_be_declared_using_a_binding_pattern: diag(1187, ts.DiagnosticCategory.Error, "A_parameter_property_may_not_be_declared_using_a_binding_pattern_1187", "A parameter property may not be declared using a binding pattern."),
            Only_a_single_variable_declaration_is_allowed_in_a_for_of_statement: diag(1188, ts.DiagnosticCategory.Error, "Only_a_single_variable_declaration_is_allowed_in_a_for_of_statement_1188", "Only a single variable declaration is allowed in a 'for...of' statement."),
            The_variable_declaration_of_a_for_in_statement_cannot_have_an_initializer: diag(1189, ts.DiagnosticCategory.Error, "The_variable_declaration_of_a_for_in_statement_cannot_have_an_initializer_1189", "The variable declaration of a 'for...in' statement cannot have an initializer."),
            The_variable_declaration_of_a_for_of_statement_cannot_have_an_initializer: diag(1190, ts.DiagnosticCategory.Error, "The_variable_declaration_of_a_for_of_statement_cannot_have_an_initializer_1190", "The variable declaration of a 'for...of' statement cannot have an initializer."),
            An_import_declaration_cannot_have_modifiers: diag(1191, ts.DiagnosticCategory.Error, "An_import_declaration_cannot_have_modifiers_1191", "An import declaration cannot have modifiers."),
            Module_0_has_no_default_export: diag(1192, ts.DiagnosticCategory.Error, "Module_0_has_no_default_export_1192", "Module '{0}' has no default export."),
            An_export_declaration_cannot_have_modifiers: diag(1193, ts.DiagnosticCategory.Error, "An_export_declaration_cannot_have_modifiers_1193", "An export declaration cannot have modifiers."),
            Export_declarations_are_not_permitted_in_a_namespace: diag(1194, ts.DiagnosticCategory.Error, "Export_declarations_are_not_permitted_in_a_namespace_1194", "Export declarations are not permitted in a namespace."),
            Catch_clause_variable_cannot_have_a_type_annotation: diag(1196, ts.DiagnosticCategory.Error, "Catch_clause_variable_cannot_have_a_type_annotation_1196", "Catch clause variable cannot have a type annotation."),
            Catch_clause_variable_cannot_have_an_initializer: diag(1197, ts.DiagnosticCategory.Error, "Catch_clause_variable_cannot_have_an_initializer_1197", "Catch clause variable cannot have an initializer."),
            An_extended_Unicode_escape_value_must_be_between_0x0_and_0x10FFFF_inclusive: diag(1198, ts.DiagnosticCategory.Error, "An_extended_Unicode_escape_value_must_be_between_0x0_and_0x10FFFF_inclusive_1198", "An extended Unicode escape value must be between 0x0 and 0x10FFFF inclusive."),
            Unterminated_Unicode_escape_sequence: diag(1199, ts.DiagnosticCategory.Error, "Unterminated_Unicode_escape_sequence_1199", "Unterminated Unicode escape sequence."),
            Line_terminator_not_permitted_before_arrow: diag(1200, ts.DiagnosticCategory.Error, "Line_terminator_not_permitted_before_arrow_1200", "Line terminator not permitted before arrow."),
            Import_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_import_Asterisk_as_ns_from_mod_import_a_from_mod_import_d_from_mod_or_another_module_format_instead: diag(1202, ts.DiagnosticCategory.Error, "Import_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_import_Asterisk_as_1202", "Import assignment cannot be used when targeting ECMAScript modules. Consider using 'import * as ns from \"mod\"', 'import {a} from \"mod\"', 'import d from \"mod\"', or another module format instead."),
            Export_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_export_default_or_another_module_format_instead: diag(1203, ts.DiagnosticCategory.Error, "Export_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_export_default_or__1203", "Export assignment cannot be used when targeting ECMAScript modules. Consider using 'export default' or another module format instead."),
            Re_exporting_a_type_when_the_isolatedModules_flag_is_provided_requires_using_export_type: diag(1205, ts.DiagnosticCategory.Error, "Re_exporting_a_type_when_the_isolatedModules_flag_is_provided_requires_using_export_type_1205", "Re-exporting a type when the '--isolatedModules' flag is provided requires using 'export type'."),
            Decorators_are_not_valid_here: diag(1206, ts.DiagnosticCategory.Error, "Decorators_are_not_valid_here_1206", "Decorators are not valid here."),
            Decorators_cannot_be_applied_to_multiple_get_Slashset_accessors_of_the_same_name: diag(1207, ts.DiagnosticCategory.Error, "Decorators_cannot_be_applied_to_multiple_get_Slashset_accessors_of_the_same_name_1207", "Decorators cannot be applied to multiple get/set accessors of the same name."),
            All_files_must_be_modules_when_the_isolatedModules_flag_is_provided: diag(1208, ts.DiagnosticCategory.Error, "All_files_must_be_modules_when_the_isolatedModules_flag_is_provided_1208", "All files must be modules when the '--isolatedModules' flag is provided."),
            Invalid_use_of_0_Class_definitions_are_automatically_in_strict_mode: diag(1210, ts.DiagnosticCategory.Error, "Invalid_use_of_0_Class_definitions_are_automatically_in_strict_mode_1210", "Invalid use of '{0}'. Class definitions are automatically in strict mode."),
            A_class_declaration_without_the_default_modifier_must_have_a_name: diag(1211, ts.DiagnosticCategory.Error, "A_class_declaration_without_the_default_modifier_must_have_a_name_1211", "A class declaration without the 'default' modifier must have a name."),
            Identifier_expected_0_is_a_reserved_word_in_strict_mode: diag(1212, ts.DiagnosticCategory.Error, "Identifier_expected_0_is_a_reserved_word_in_strict_mode_1212", "Identifier expected. '{0}' is a reserved word in strict mode."),
            Identifier_expected_0_is_a_reserved_word_in_strict_mode_Class_definitions_are_automatically_in_strict_mode: diag(1213, ts.DiagnosticCategory.Error, "Identifier_expected_0_is_a_reserved_word_in_strict_mode_Class_definitions_are_automatically_in_stric_1213", "Identifier expected. '{0}' is a reserved word in strict mode. Class definitions are automatically in strict mode."),
            Identifier_expected_0_is_a_reserved_word_in_strict_mode_Modules_are_automatically_in_strict_mode: diag(1214, ts.DiagnosticCategory.Error, "Identifier_expected_0_is_a_reserved_word_in_strict_mode_Modules_are_automatically_in_strict_mode_1214", "Identifier expected. '{0}' is a reserved word in strict mode. Modules are automatically in strict mode."),
            Invalid_use_of_0_Modules_are_automatically_in_strict_mode: diag(1215, ts.DiagnosticCategory.Error, "Invalid_use_of_0_Modules_are_automatically_in_strict_mode_1215", "Invalid use of '{0}'. Modules are automatically in strict mode."),
            Identifier_expected_esModule_is_reserved_as_an_exported_marker_when_transforming_ECMAScript_modules: diag(1216, ts.DiagnosticCategory.Error, "Identifier_expected_esModule_is_reserved_as_an_exported_marker_when_transforming_ECMAScript_modules_1216", "Identifier expected. '__esModule' is reserved as an exported marker when transforming ECMAScript modules."),
            Export_assignment_is_not_supported_when_module_flag_is_system: diag(1218, ts.DiagnosticCategory.Error, "Export_assignment_is_not_supported_when_module_flag_is_system_1218", "Export assignment is not supported when '--module' flag is 'system'."),
            Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_the_experimentalDecorators_option_in_your_tsconfig_or_jsconfig_to_remove_this_warning: diag(1219, ts.DiagnosticCategory.Error, "Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_t_1219", "Experimental support for decorators is a feature that is subject to change in a future release. Set the 'experimentalDecorators' option in your 'tsconfig' or 'jsconfig' to remove this warning."),
            Generators_are_only_available_when_targeting_ECMAScript_2015_or_higher: diag(1220, ts.DiagnosticCategory.Error, "Generators_are_only_available_when_targeting_ECMAScript_2015_or_higher_1220", "Generators are only available when targeting ECMAScript 2015 or higher."),
            Generators_are_not_allowed_in_an_ambient_context: diag(1221, ts.DiagnosticCategory.Error, "Generators_are_not_allowed_in_an_ambient_context_1221", "Generators are not allowed in an ambient context."),
            An_overload_signature_cannot_be_declared_as_a_generator: diag(1222, ts.DiagnosticCategory.Error, "An_overload_signature_cannot_be_declared_as_a_generator_1222", "An overload signature cannot be declared as a generator."),
            _0_tag_already_specified: diag(1223, ts.DiagnosticCategory.Error, "_0_tag_already_specified_1223", "'{0}' tag already specified."),
            Signature_0_must_be_a_type_predicate: diag(1224, ts.DiagnosticCategory.Error, "Signature_0_must_be_a_type_predicate_1224", "Signature '{0}' must be a type predicate."),
            Cannot_find_parameter_0: diag(1225, ts.DiagnosticCategory.Error, "Cannot_find_parameter_0_1225", "Cannot find parameter '{0}'."),
            Type_predicate_0_is_not_assignable_to_1: diag(1226, ts.DiagnosticCategory.Error, "Type_predicate_0_is_not_assignable_to_1_1226", "Type predicate '{0}' is not assignable to '{1}'."),
            Parameter_0_is_not_in_the_same_position_as_parameter_1: diag(1227, ts.DiagnosticCategory.Error, "Parameter_0_is_not_in_the_same_position_as_parameter_1_1227", "Parameter '{0}' is not in the same position as parameter '{1}'."),
            A_type_predicate_is_only_allowed_in_return_type_position_for_functions_and_methods: diag(1228, ts.DiagnosticCategory.Error, "A_type_predicate_is_only_allowed_in_return_type_position_for_functions_and_methods_1228", "A type predicate is only allowed in return type position for functions and methods."),
            A_type_predicate_cannot_reference_a_rest_parameter: diag(1229, ts.DiagnosticCategory.Error, "A_type_predicate_cannot_reference_a_rest_parameter_1229", "A type predicate cannot reference a rest parameter."),
            A_type_predicate_cannot_reference_element_0_in_a_binding_pattern: diag(1230, ts.DiagnosticCategory.Error, "A_type_predicate_cannot_reference_element_0_in_a_binding_pattern_1230", "A type predicate cannot reference element '{0}' in a binding pattern."),
            An_export_assignment_can_only_be_used_in_a_module: diag(1231, ts.DiagnosticCategory.Error, "An_export_assignment_can_only_be_used_in_a_module_1231", "An export assignment can only be used in a module."),
            An_import_declaration_can_only_be_used_in_a_namespace_or_module: diag(1232, ts.DiagnosticCategory.Error, "An_import_declaration_can_only_be_used_in_a_namespace_or_module_1232", "An import declaration can only be used in a namespace or module."),
            An_export_declaration_can_only_be_used_in_a_module: diag(1233, ts.DiagnosticCategory.Error, "An_export_declaration_can_only_be_used_in_a_module_1233", "An export declaration can only be used in a module."),
            An_ambient_module_declaration_is_only_allowed_at_the_top_level_in_a_file: diag(1234, ts.DiagnosticCategory.Error, "An_ambient_module_declaration_is_only_allowed_at_the_top_level_in_a_file_1234", "An ambient module declaration is only allowed at the top level in a file."),
            A_namespace_declaration_is_only_allowed_in_a_namespace_or_module: diag(1235, ts.DiagnosticCategory.Error, "A_namespace_declaration_is_only_allowed_in_a_namespace_or_module_1235", "A namespace declaration is only allowed in a namespace or module."),
            The_return_type_of_a_property_decorator_function_must_be_either_void_or_any: diag(1236, ts.DiagnosticCategory.Error, "The_return_type_of_a_property_decorator_function_must_be_either_void_or_any_1236", "The return type of a property decorator function must be either 'void' or 'any'."),
            The_return_type_of_a_parameter_decorator_function_must_be_either_void_or_any: diag(1237, ts.DiagnosticCategory.Error, "The_return_type_of_a_parameter_decorator_function_must_be_either_void_or_any_1237", "The return type of a parameter decorator function must be either 'void' or 'any'."),
            Unable_to_resolve_signature_of_class_decorator_when_called_as_an_expression: diag(1238, ts.DiagnosticCategory.Error, "Unable_to_resolve_signature_of_class_decorator_when_called_as_an_expression_1238", "Unable to resolve signature of class decorator when called as an expression."),
            Unable_to_resolve_signature_of_parameter_decorator_when_called_as_an_expression: diag(1239, ts.DiagnosticCategory.Error, "Unable_to_resolve_signature_of_parameter_decorator_when_called_as_an_expression_1239", "Unable to resolve signature of parameter decorator when called as an expression."),
            Unable_to_resolve_signature_of_property_decorator_when_called_as_an_expression: diag(1240, ts.DiagnosticCategory.Error, "Unable_to_resolve_signature_of_property_decorator_when_called_as_an_expression_1240", "Unable to resolve signature of property decorator when called as an expression."),
            Unable_to_resolve_signature_of_method_decorator_when_called_as_an_expression: diag(1241, ts.DiagnosticCategory.Error, "Unable_to_resolve_signature_of_method_decorator_when_called_as_an_expression_1241", "Unable to resolve signature of method decorator when called as an expression."),
            abstract_modifier_can_only_appear_on_a_class_method_or_property_declaration: diag(1242, ts.DiagnosticCategory.Error, "abstract_modifier_can_only_appear_on_a_class_method_or_property_declaration_1242", "'abstract' modifier can only appear on a class, method, or property declaration."),
            _0_modifier_cannot_be_used_with_1_modifier: diag(1243, ts.DiagnosticCategory.Error, "_0_modifier_cannot_be_used_with_1_modifier_1243", "'{0}' modifier cannot be used with '{1}' modifier."),
            Abstract_methods_can_only_appear_within_an_abstract_class: diag(1244, ts.DiagnosticCategory.Error, "Abstract_methods_can_only_appear_within_an_abstract_class_1244", "Abstract methods can only appear within an abstract class."),
            Method_0_cannot_have_an_implementation_because_it_is_marked_abstract: diag(1245, ts.DiagnosticCategory.Error, "Method_0_cannot_have_an_implementation_because_it_is_marked_abstract_1245", "Method '{0}' cannot have an implementation because it is marked abstract."),
            An_interface_property_cannot_have_an_initializer: diag(1246, ts.DiagnosticCategory.Error, "An_interface_property_cannot_have_an_initializer_1246", "An interface property cannot have an initializer."),
            A_type_literal_property_cannot_have_an_initializer: diag(1247, ts.DiagnosticCategory.Error, "A_type_literal_property_cannot_have_an_initializer_1247", "A type literal property cannot have an initializer."),
            A_class_member_cannot_have_the_0_keyword: diag(1248, ts.DiagnosticCategory.Error, "A_class_member_cannot_have_the_0_keyword_1248", "A class member cannot have the '{0}' keyword."),
            A_decorator_can_only_decorate_a_method_implementation_not_an_overload: diag(1249, ts.DiagnosticCategory.Error, "A_decorator_can_only_decorate_a_method_implementation_not_an_overload_1249", "A decorator can only decorate a method implementation, not an overload."),
            Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5: diag(1250, ts.DiagnosticCategory.Error, "Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_1250", "Function declarations are not allowed inside blocks in strict mode when targeting 'ES3' or 'ES5'."),
            Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Class_definitions_are_automatically_in_strict_mode: diag(1251, ts.DiagnosticCategory.Error, "Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Class_d_1251", "Function declarations are not allowed inside blocks in strict mode when targeting 'ES3' or 'ES5'. Class definitions are automatically in strict mode."),
            Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Modules_are_automatically_in_strict_mode: diag(1252, ts.DiagnosticCategory.Error, "Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Modules_1252", "Function declarations are not allowed inside blocks in strict mode when targeting 'ES3' or 'ES5'. Modules are automatically in strict mode."),
            _0_tag_cannot_be_used_independently_as_a_top_level_JSDoc_tag: diag(1253, ts.DiagnosticCategory.Error, "_0_tag_cannot_be_used_independently_as_a_top_level_JSDoc_tag_1253", "'{0}' tag cannot be used independently as a top level JSDoc tag."),
            A_const_initializer_in_an_ambient_context_must_be_a_string_or_numeric_literal_or_literal_enum_reference: diag(1254, ts.DiagnosticCategory.Error, "A_const_initializer_in_an_ambient_context_must_be_a_string_or_numeric_literal_or_literal_enum_refere_1254", "A 'const' initializer in an ambient context must be a string or numeric literal or literal enum reference."),
            A_definite_assignment_assertion_is_not_permitted_in_this_context: diag(1255, ts.DiagnosticCategory.Error, "A_definite_assignment_assertion_is_not_permitted_in_this_context_1255", "A definite assignment assertion '!' is not permitted in this context."),
            A_rest_element_must_be_last_in_a_tuple_type: diag(1256, ts.DiagnosticCategory.Error, "A_rest_element_must_be_last_in_a_tuple_type_1256", "A rest element must be last in a tuple type."),
            A_required_element_cannot_follow_an_optional_element: diag(1257, ts.DiagnosticCategory.Error, "A_required_element_cannot_follow_an_optional_element_1257", "A required element cannot follow an optional element."),
            Definite_assignment_assertions_can_only_be_used_along_with_a_type_annotation: diag(1258, ts.DiagnosticCategory.Error, "Definite_assignment_assertions_can_only_be_used_along_with_a_type_annotation_1258", "Definite assignment assertions can only be used along with a type annotation."),
            Module_0_can_only_be_default_imported_using_the_1_flag: diag(1259, ts.DiagnosticCategory.Error, "Module_0_can_only_be_default_imported_using_the_1_flag_1259", "Module '{0}' can only be default-imported using the '{1}' flag"),
            Keywords_cannot_contain_escape_characters: diag(1260, ts.DiagnosticCategory.Error, "Keywords_cannot_contain_escape_characters_1260", "Keywords cannot contain escape characters."),
            Already_included_file_name_0_differs_from_file_name_1_only_in_casing: diag(1261, ts.DiagnosticCategory.Error, "Already_included_file_name_0_differs_from_file_name_1_only_in_casing_1261", "Already included file name '{0}' differs from file name '{1}' only in casing."),
            with_statements_are_not_allowed_in_an_async_function_block: diag(1300, ts.DiagnosticCategory.Error, "with_statements_are_not_allowed_in_an_async_function_block_1300", "'with' statements are not allowed in an async function block."),
            await_expressions_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules: diag(1308, ts.DiagnosticCategory.Error, "await_expressions_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules_1308", "'await' expressions are only allowed within async functions and at the top levels of modules."),
            can_only_be_used_in_an_object_literal_property_inside_a_destructuring_assignment: diag(1312, ts.DiagnosticCategory.Error, "can_only_be_used_in_an_object_literal_property_inside_a_destructuring_assignment_1312", "'=' can only be used in an object literal property inside a destructuring assignment."),
            The_body_of_an_if_statement_cannot_be_the_empty_statement: diag(1313, ts.DiagnosticCategory.Error, "The_body_of_an_if_statement_cannot_be_the_empty_statement_1313", "The body of an 'if' statement cannot be the empty statement."),
            Global_module_exports_may_only_appear_in_module_files: diag(1314, ts.DiagnosticCategory.Error, "Global_module_exports_may_only_appear_in_module_files_1314", "Global module exports may only appear in module files."),
            Global_module_exports_may_only_appear_in_declaration_files: diag(1315, ts.DiagnosticCategory.Error, "Global_module_exports_may_only_appear_in_declaration_files_1315", "Global module exports may only appear in declaration files."),
            Global_module_exports_may_only_appear_at_top_level: diag(1316, ts.DiagnosticCategory.Error, "Global_module_exports_may_only_appear_at_top_level_1316", "Global module exports may only appear at top level."),
            A_parameter_property_cannot_be_declared_using_a_rest_parameter: diag(1317, ts.DiagnosticCategory.Error, "A_parameter_property_cannot_be_declared_using_a_rest_parameter_1317", "A parameter property cannot be declared using a rest parameter."),
            An_abstract_accessor_cannot_have_an_implementation: diag(1318, ts.DiagnosticCategory.Error, "An_abstract_accessor_cannot_have_an_implementation_1318", "An abstract accessor cannot have an implementation."),
            A_default_export_can_only_be_used_in_an_ECMAScript_style_module: diag(1319, ts.DiagnosticCategory.Error, "A_default_export_can_only_be_used_in_an_ECMAScript_style_module_1319", "A default export can only be used in an ECMAScript-style module."),
            Type_of_await_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member: diag(1320, ts.DiagnosticCategory.Error, "Type_of_await_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member_1320", "Type of 'await' operand must either be a valid promise or must not contain a callable 'then' member."),
            Type_of_yield_operand_in_an_async_generator_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member: diag(1321, ts.DiagnosticCategory.Error, "Type_of_yield_operand_in_an_async_generator_must_either_be_a_valid_promise_or_must_not_contain_a_cal_1321", "Type of 'yield' operand in an async generator must either be a valid promise or must not contain a callable 'then' member."),
            Type_of_iterated_elements_of_a_yield_Asterisk_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member: diag(1322, ts.DiagnosticCategory.Error, "Type_of_iterated_elements_of_a_yield_Asterisk_operand_must_either_be_a_valid_promise_or_must_not_con_1322", "Type of iterated elements of a 'yield*' operand must either be a valid promise or must not contain a callable 'then' member."),
            Dynamic_imports_are_only_supported_when_the_module_flag_is_set_to_es2020_esnext_commonjs_amd_system_or_umd: diag(1323, ts.DiagnosticCategory.Error, "Dynamic_imports_are_only_supported_when_the_module_flag_is_set_to_es2020_esnext_commonjs_amd_system__1323", "Dynamic imports are only supported when the '--module' flag is set to 'es2020', 'esnext', 'commonjs', 'amd', 'system', or 'umd'."),
            Dynamic_import_must_have_one_specifier_as_an_argument: diag(1324, ts.DiagnosticCategory.Error, "Dynamic_import_must_have_one_specifier_as_an_argument_1324", "Dynamic import must have one specifier as an argument."),
            Specifier_of_dynamic_import_cannot_be_spread_element: diag(1325, ts.DiagnosticCategory.Error, "Specifier_of_dynamic_import_cannot_be_spread_element_1325", "Specifier of dynamic import cannot be spread element."),
            Dynamic_import_cannot_have_type_arguments: diag(1326, ts.DiagnosticCategory.Error, "Dynamic_import_cannot_have_type_arguments_1326", "Dynamic import cannot have type arguments"),
            String_literal_with_double_quotes_expected: diag(1327, ts.DiagnosticCategory.Error, "String_literal_with_double_quotes_expected_1327", "String literal with double quotes expected."),
            Property_value_can_only_be_string_literal_numeric_literal_true_false_null_object_literal_or_array_literal: diag(1328, ts.DiagnosticCategory.Error, "Property_value_can_only_be_string_literal_numeric_literal_true_false_null_object_literal_or_array_li_1328", "Property value can only be string literal, numeric literal, 'true', 'false', 'null', object literal or array literal."),
            _0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write_0: diag(1329, ts.DiagnosticCategory.Error, "_0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write__1329", "'{0}' accepts too few arguments to be used as a decorator here. Did you mean to call it first and write '@{0}()'?"),
            A_property_of_an_interface_or_type_literal_whose_type_is_a_unique_symbol_type_must_be_readonly: diag(1330, ts.DiagnosticCategory.Error, "A_property_of_an_interface_or_type_literal_whose_type_is_a_unique_symbol_type_must_be_readonly_1330", "A property of an interface or type literal whose type is a 'unique symbol' type must be 'readonly'."),
            A_property_of_a_class_whose_type_is_a_unique_symbol_type_must_be_both_static_and_readonly: diag(1331, ts.DiagnosticCategory.Error, "A_property_of_a_class_whose_type_is_a_unique_symbol_type_must_be_both_static_and_readonly_1331", "A property of a class whose type is a 'unique symbol' type must be both 'static' and 'readonly'."),
            A_variable_whose_type_is_a_unique_symbol_type_must_be_const: diag(1332, ts.DiagnosticCategory.Error, "A_variable_whose_type_is_a_unique_symbol_type_must_be_const_1332", "A variable whose type is a 'unique symbol' type must be 'const'."),
            unique_symbol_types_may_not_be_used_on_a_variable_declaration_with_a_binding_name: diag(1333, ts.DiagnosticCategory.Error, "unique_symbol_types_may_not_be_used_on_a_variable_declaration_with_a_binding_name_1333", "'unique symbol' types may not be used on a variable declaration with a binding name."),
            unique_symbol_types_are_only_allowed_on_variables_in_a_variable_statement: diag(1334, ts.DiagnosticCategory.Error, "unique_symbol_types_are_only_allowed_on_variables_in_a_variable_statement_1334", "'unique symbol' types are only allowed on variables in a variable statement."),
            unique_symbol_types_are_not_allowed_here: diag(1335, ts.DiagnosticCategory.Error, "unique_symbol_types_are_not_allowed_here_1335", "'unique symbol' types are not allowed here."),
            An_index_signature_parameter_type_cannot_be_a_type_alias_Consider_writing_0_Colon_1_Colon_2_instead: diag(1336, ts.DiagnosticCategory.Error, "An_index_signature_parameter_type_cannot_be_a_type_alias_Consider_writing_0_Colon_1_Colon_2_instead_1336", "An index signature parameter type cannot be a type alias. Consider writing '[{0}: {1}]: {2}' instead."),
            An_index_signature_parameter_type_cannot_be_a_union_type_Consider_using_a_mapped_object_type_instead: diag(1337, ts.DiagnosticCategory.Error, "An_index_signature_parameter_type_cannot_be_a_union_type_Consider_using_a_mapped_object_type_instead_1337", "An index signature parameter type cannot be a union type. Consider using a mapped object type instead."),
            infer_declarations_are_only_permitted_in_the_extends_clause_of_a_conditional_type: diag(1338, ts.DiagnosticCategory.Error, "infer_declarations_are_only_permitted_in_the_extends_clause_of_a_conditional_type_1338", "'infer' declarations are only permitted in the 'extends' clause of a conditional type."),
            Module_0_does_not_refer_to_a_value_but_is_used_as_a_value_here: diag(1339, ts.DiagnosticCategory.Error, "Module_0_does_not_refer_to_a_value_but_is_used_as_a_value_here_1339", "Module '{0}' does not refer to a value, but is used as a value here."),
            Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0: diag(1340, ts.DiagnosticCategory.Error, "Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0_1340", "Module '{0}' does not refer to a type, but is used as a type here. Did you mean 'typeof import('{0}')'?"),
            Type_arguments_cannot_be_used_here: diag(1342, ts.DiagnosticCategory.Error, "Type_arguments_cannot_be_used_here_1342", "Type arguments cannot be used here."),
            The_import_meta_meta_property_is_only_allowed_when_the_module_option_is_esnext_or_system: diag(1343, ts.DiagnosticCategory.Error, "The_import_meta_meta_property_is_only_allowed_when_the_module_option_is_esnext_or_system_1343", "The 'import.meta' meta-property is only allowed when the '--module' option is 'esnext' or 'system'."),
            A_label_is_not_allowed_here: diag(1344, ts.DiagnosticCategory.Error, "A_label_is_not_allowed_here_1344", "'A label is not allowed here."),
            An_expression_of_type_void_cannot_be_tested_for_truthiness: diag(1345, ts.DiagnosticCategory.Error, "An_expression_of_type_void_cannot_be_tested_for_truthiness_1345", "An expression of type 'void' cannot be tested for truthiness"),
            This_parameter_is_not_allowed_with_use_strict_directive: diag(1346, ts.DiagnosticCategory.Error, "This_parameter_is_not_allowed_with_use_strict_directive_1346", "This parameter is not allowed with 'use strict' directive."),
            use_strict_directive_cannot_be_used_with_non_simple_parameter_list: diag(1347, ts.DiagnosticCategory.Error, "use_strict_directive_cannot_be_used_with_non_simple_parameter_list_1347", "'use strict' directive cannot be used with non-simple parameter list."),
            Non_simple_parameter_declared_here: diag(1348, ts.DiagnosticCategory.Error, "Non_simple_parameter_declared_here_1348", "Non-simple parameter declared here."),
            use_strict_directive_used_here: diag(1349, ts.DiagnosticCategory.Error, "use_strict_directive_used_here_1349", "'use strict' directive used here."),
            Print_the_final_configuration_instead_of_building: diag(1350, ts.DiagnosticCategory.Message, "Print_the_final_configuration_instead_of_building_1350", "Print the final configuration instead of building."),
            An_identifier_or_keyword_cannot_immediately_follow_a_numeric_literal: diag(1351, ts.DiagnosticCategory.Error, "An_identifier_or_keyword_cannot_immediately_follow_a_numeric_literal_1351", "An identifier or keyword cannot immediately follow a numeric literal."),
            A_bigint_literal_cannot_use_exponential_notation: diag(1352, ts.DiagnosticCategory.Error, "A_bigint_literal_cannot_use_exponential_notation_1352", "A bigint literal cannot use exponential notation."),
            A_bigint_literal_must_be_an_integer: diag(1353, ts.DiagnosticCategory.Error, "A_bigint_literal_must_be_an_integer_1353", "A bigint literal must be an integer."),
            readonly_type_modifier_is_only_permitted_on_array_and_tuple_literal_types: diag(1354, ts.DiagnosticCategory.Error, "readonly_type_modifier_is_only_permitted_on_array_and_tuple_literal_types_1354", "'readonly' type modifier is only permitted on array and tuple literal types."),
            A_const_assertions_can_only_be_applied_to_references_to_enum_members_or_string_number_boolean_array_or_object_literals: diag(1355, ts.DiagnosticCategory.Error, "A_const_assertions_can_only_be_applied_to_references_to_enum_members_or_string_number_boolean_array__1355", "A 'const' assertions can only be applied to references to enum members, or string, number, boolean, array, or object literals."),
            Did_you_mean_to_mark_this_function_as_async: diag(1356, ts.DiagnosticCategory.Error, "Did_you_mean_to_mark_this_function_as_async_1356", "Did you mean to mark this function as 'async'?"),
            An_enum_member_name_must_be_followed_by_a_or: diag(1357, ts.DiagnosticCategory.Error, "An_enum_member_name_must_be_followed_by_a_or_1357", "An enum member name must be followed by a ',', '=', or '}'."),
            Tagged_template_expressions_are_not_permitted_in_an_optional_chain: diag(1358, ts.DiagnosticCategory.Error, "Tagged_template_expressions_are_not_permitted_in_an_optional_chain_1358", "Tagged template expressions are not permitted in an optional chain."),
            Identifier_expected_0_is_a_reserved_word_that_cannot_be_used_here: diag(1359, ts.DiagnosticCategory.Error, "Identifier_expected_0_is_a_reserved_word_that_cannot_be_used_here_1359", "Identifier expected. '{0}' is a reserved word that cannot be used here."),
            Did_you_mean_to_parenthesize_this_function_type: diag(1360, ts.DiagnosticCategory.Error, "Did_you_mean_to_parenthesize_this_function_type_1360", "Did you mean to parenthesize this function type?"),
            _0_cannot_be_used_as_a_value_because_it_was_imported_using_import_type: diag(1361, ts.DiagnosticCategory.Error, "_0_cannot_be_used_as_a_value_because_it_was_imported_using_import_type_1361", "'{0}' cannot be used as a value because it was imported using 'import type'."),
            _0_cannot_be_used_as_a_value_because_it_was_exported_using_export_type: diag(1362, ts.DiagnosticCategory.Error, "_0_cannot_be_used_as_a_value_because_it_was_exported_using_export_type_1362", "'{0}' cannot be used as a value because it was exported using 'export type'."),
            A_type_only_import_can_specify_a_default_import_or_named_bindings_but_not_both: diag(1363, ts.DiagnosticCategory.Error, "A_type_only_import_can_specify_a_default_import_or_named_bindings_but_not_both_1363", "A type-only import can specify a default import or named bindings, but not both."),
            Convert_to_type_only_export: diag(1364, ts.DiagnosticCategory.Message, "Convert_to_type_only_export_1364", "Convert to type-only export"),
            Convert_all_re_exported_types_to_type_only_exports: diag(1365, ts.DiagnosticCategory.Message, "Convert_all_re_exported_types_to_type_only_exports_1365", "Convert all re-exported types to type-only exports"),
            Split_into_two_separate_import_declarations: diag(1366, ts.DiagnosticCategory.Message, "Split_into_two_separate_import_declarations_1366", "Split into two separate import declarations"),
            Split_all_invalid_type_only_imports: diag(1367, ts.DiagnosticCategory.Message, "Split_all_invalid_type_only_imports_1367", "Split all invalid type-only imports"),
            Specify_emit_Slashchecking_behavior_for_imports_that_are_only_used_for_types: diag(1368, ts.DiagnosticCategory.Message, "Specify_emit_Slashchecking_behavior_for_imports_that_are_only_used_for_types_1368", "Specify emit/checking behavior for imports that are only used for types"),
            Did_you_mean_0: diag(1369, ts.DiagnosticCategory.Message, "Did_you_mean_0_1369", "Did you mean '{0}'?"),
            Only_ECMAScript_imports_may_use_import_type: diag(1370, ts.DiagnosticCategory.Error, "Only_ECMAScript_imports_may_use_import_type_1370", "Only ECMAScript imports may use 'import type'."),
            This_import_is_never_used_as_a_value_and_must_use_import_type_because_the_importsNotUsedAsValues_is_set_to_error: diag(1371, ts.DiagnosticCategory.Error, "This_import_is_never_used_as_a_value_and_must_use_import_type_because_the_importsNotUsedAsValues_is__1371", "This import is never used as a value and must use 'import type' because the 'importsNotUsedAsValues' is set to 'error'."),
            Convert_to_type_only_import: diag(1373, ts.DiagnosticCategory.Message, "Convert_to_type_only_import_1373", "Convert to type-only import"),
            Convert_all_imports_not_used_as_a_value_to_type_only_imports: diag(1374, ts.DiagnosticCategory.Message, "Convert_all_imports_not_used_as_a_value_to_type_only_imports_1374", "Convert all imports not used as a value to type-only imports"),
            await_expressions_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_file_has_no_imports_or_exports_Consider_adding_an_empty_export_to_make_this_file_a_module: diag(1375, ts.DiagnosticCategory.Error, "await_expressions_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_fi_1375", "'await' expressions are only allowed at the top level of a file when that file is a module, but this file has no imports or exports. Consider adding an empty 'export {}' to make this file a module."),
            _0_was_imported_here: diag(1376, ts.DiagnosticCategory.Message, "_0_was_imported_here_1376", "'{0}' was imported here."),
            _0_was_exported_here: diag(1377, ts.DiagnosticCategory.Message, "_0_was_exported_here_1377", "'{0}' was exported here."),
            Top_level_await_expressions_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_the_target_option_is_set_to_es2017_or_higher: diag(1378, ts.DiagnosticCategory.Error, "Top_level_await_expressions_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_t_1378", "Top-level 'await' expressions are only allowed when the 'module' option is set to 'esnext' or 'system', and the 'target' option is set to 'es2017' or higher."),
            An_import_alias_cannot_reference_a_declaration_that_was_exported_using_export_type: diag(1379, ts.DiagnosticCategory.Error, "An_import_alias_cannot_reference_a_declaration_that_was_exported_using_export_type_1379", "An import alias cannot reference a declaration that was exported using 'export type'."),
            An_import_alias_cannot_reference_a_declaration_that_was_imported_using_import_type: diag(1380, ts.DiagnosticCategory.Error, "An_import_alias_cannot_reference_a_declaration_that_was_imported_using_import_type_1380", "An import alias cannot reference a declaration that was imported using 'import type'."),
            Only_named_exports_may_use_export_type: diag(1383, ts.DiagnosticCategory.Error, "Only_named_exports_may_use_export_type_1383", "Only named exports may use 'export type'."),
            The_types_of_0_are_incompatible_between_these_types: diag(2200, ts.DiagnosticCategory.Error, "The_types_of_0_are_incompatible_between_these_types_2200", "The types of '{0}' are incompatible between these types."),
            The_types_returned_by_0_are_incompatible_between_these_types: diag(2201, ts.DiagnosticCategory.Error, "The_types_returned_by_0_are_incompatible_between_these_types_2201", "The types returned by '{0}' are incompatible between these types."),
            Call_signature_return_types_0_and_1_are_incompatible: diag(2202, ts.DiagnosticCategory.Error, "Call_signature_return_types_0_and_1_are_incompatible_2202", "Call signature return types '{0}' and '{1}' are incompatible.", /*reportsUnnecessary*/ undefined, /*elidedInCompatabilityPyramid*/ true),
            Construct_signature_return_types_0_and_1_are_incompatible: diag(2203, ts.DiagnosticCategory.Error, "Construct_signature_return_types_0_and_1_are_incompatible_2203", "Construct signature return types '{0}' and '{1}' are incompatible.", /*reportsUnnecessary*/ undefined, /*elidedInCompatabilityPyramid*/ true),
            Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1: diag(2204, ts.DiagnosticCategory.Error, "Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1_2204", "Call signatures with no arguments have incompatible return types '{0}' and '{1}'.", /*reportsUnnecessary*/ undefined, /*elidedInCompatabilityPyramid*/ true),
            Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1: diag(2205, ts.DiagnosticCategory.Error, "Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1_2205", "Construct signatures with no arguments have incompatible return types '{0}' and '{1}'.", /*reportsUnnecessary*/ undefined, /*elidedInCompatabilityPyramid*/ true),
            Duplicate_identifier_0: diag(2300, ts.DiagnosticCategory.Error, "Duplicate_identifier_0_2300", "Duplicate identifier '{0}'."),
            Initializer_of_instance_member_variable_0_cannot_reference_identifier_1_declared_in_the_constructor: diag(2301, ts.DiagnosticCategory.Error, "Initializer_of_instance_member_variable_0_cannot_reference_identifier_1_declared_in_the_constructor_2301", "Initializer of instance member variable '{0}' cannot reference identifier '{1}' declared in the constructor."),
            Static_members_cannot_reference_class_type_parameters: diag(2302, ts.DiagnosticCategory.Error, "Static_members_cannot_reference_class_type_parameters_2302", "Static members cannot reference class type parameters."),
            Circular_definition_of_import_alias_0: diag(2303, ts.DiagnosticCategory.Error, "Circular_definition_of_import_alias_0_2303", "Circular definition of import alias '{0}'."),
            Cannot_find_name_0: diag(2304, ts.DiagnosticCategory.Error, "Cannot_find_name_0_2304", "Cannot find name '{0}'."),
            Module_0_has_no_exported_member_1: diag(2305, ts.DiagnosticCategory.Error, "Module_0_has_no_exported_member_1_2305", "Module '{0}' has no exported member '{1}'."),
            File_0_is_not_a_module: diag(2306, ts.DiagnosticCategory.Error, "File_0_is_not_a_module_2306", "File '{0}' is not a module."),
            Cannot_find_module_0: diag(2307, ts.DiagnosticCategory.Error, "Cannot_find_module_0_2307", "Cannot find module '{0}'."),
            Module_0_has_already_exported_a_member_named_1_Consider_explicitly_re_exporting_to_resolve_the_ambiguity: diag(2308, ts.DiagnosticCategory.Error, "Module_0_has_already_exported_a_member_named_1_Consider_explicitly_re_exporting_to_resolve_the_ambig_2308", "Module {0} has already exported a member named '{1}'. Consider explicitly re-exporting to resolve the ambiguity."),
            An_export_assignment_cannot_be_used_in_a_module_with_other_exported_elements: diag(2309, ts.DiagnosticCategory.Error, "An_export_assignment_cannot_be_used_in_a_module_with_other_exported_elements_2309", "An export assignment cannot be used in a module with other exported elements."),
            Type_0_recursively_references_itself_as_a_base_type: diag(2310, ts.DiagnosticCategory.Error, "Type_0_recursively_references_itself_as_a_base_type_2310", "Type '{0}' recursively references itself as a base type."),
            A_class_may_only_extend_another_class: diag(2311, ts.DiagnosticCategory.Error, "A_class_may_only_extend_another_class_2311", "A class may only extend another class."),
            An_interface_can_only_extend_an_object_type_or_intersection_of_object_types_with_statically_known_members: diag(2312, ts.DiagnosticCategory.Error, "An_interface_can_only_extend_an_object_type_or_intersection_of_object_types_with_statically_known_me_2312", "An interface can only extend an object type or intersection of object types with statically known members."),
            Type_parameter_0_has_a_circular_constraint: diag(2313, ts.DiagnosticCategory.Error, "Type_parameter_0_has_a_circular_constraint_2313", "Type parameter '{0}' has a circular constraint."),
            Generic_type_0_requires_1_type_argument_s: diag(2314, ts.DiagnosticCategory.Error, "Generic_type_0_requires_1_type_argument_s_2314", "Generic type '{0}' requires {1} type argument(s)."),
            Type_0_is_not_generic: diag(2315, ts.DiagnosticCategory.Error, "Type_0_is_not_generic_2315", "Type '{0}' is not generic."),
            Global_type_0_must_be_a_class_or_interface_type: diag(2316, ts.DiagnosticCategory.Error, "Global_type_0_must_be_a_class_or_interface_type_2316", "Global type '{0}' must be a class or interface type."),
            Global_type_0_must_have_1_type_parameter_s: diag(2317, ts.DiagnosticCategory.Error, "Global_type_0_must_have_1_type_parameter_s_2317", "Global type '{0}' must have {1} type parameter(s)."),
            Cannot_find_global_type_0: diag(2318, ts.DiagnosticCategory.Error, "Cannot_find_global_type_0_2318", "Cannot find global type '{0}'."),
            Named_property_0_of_types_1_and_2_are_not_identical: diag(2319, ts.DiagnosticCategory.Error, "Named_property_0_of_types_1_and_2_are_not_identical_2319", "Named property '{0}' of types '{1}' and '{2}' are not identical."),
            Interface_0_cannot_simultaneously_extend_types_1_and_2: diag(2320, ts.DiagnosticCategory.Error, "Interface_0_cannot_simultaneously_extend_types_1_and_2_2320", "Interface '{0}' cannot simultaneously extend types '{1}' and '{2}'."),
            Excessive_stack_depth_comparing_types_0_and_1: diag(2321, ts.DiagnosticCategory.Error, "Excessive_stack_depth_comparing_types_0_and_1_2321", "Excessive stack depth comparing types '{0}' and '{1}'."),
            Type_0_is_not_assignable_to_type_1: diag(2322, ts.DiagnosticCategory.Error, "Type_0_is_not_assignable_to_type_1_2322", "Type '{0}' is not assignable to type '{1}'."),
            Cannot_redeclare_exported_variable_0: diag(2323, ts.DiagnosticCategory.Error, "Cannot_redeclare_exported_variable_0_2323", "Cannot redeclare exported variable '{0}'."),
            Property_0_is_missing_in_type_1: diag(2324, ts.DiagnosticCategory.Error, "Property_0_is_missing_in_type_1_2324", "Property '{0}' is missing in type '{1}'."),
            Property_0_is_private_in_type_1_but_not_in_type_2: diag(2325, ts.DiagnosticCategory.Error, "Property_0_is_private_in_type_1_but_not_in_type_2_2325", "Property '{0}' is private in type '{1}' but not in type '{2}'."),
            Types_of_property_0_are_incompatible: diag(2326, ts.DiagnosticCategory.Error, "Types_of_property_0_are_incompatible_2326", "Types of property '{0}' are incompatible."),
            Property_0_is_optional_in_type_1_but_required_in_type_2: diag(2327, ts.DiagnosticCategory.Error, "Property_0_is_optional_in_type_1_but_required_in_type_2_2327", "Property '{0}' is optional in type '{1}' but required in type '{2}'."),
            Types_of_parameters_0_and_1_are_incompatible: diag(2328, ts.DiagnosticCategory.Error, "Types_of_parameters_0_and_1_are_incompatible_2328", "Types of parameters '{0}' and '{1}' are incompatible."),
            Index_signature_is_missing_in_type_0: diag(2329, ts.DiagnosticCategory.Error, "Index_signature_is_missing_in_type_0_2329", "Index signature is missing in type '{0}'."),
            Index_signatures_are_incompatible: diag(2330, ts.DiagnosticCategory.Error, "Index_signatures_are_incompatible_2330", "Index signatures are incompatible."),
            this_cannot_be_referenced_in_a_module_or_namespace_body: diag(2331, ts.DiagnosticCategory.Error, "this_cannot_be_referenced_in_a_module_or_namespace_body_2331", "'this' cannot be referenced in a module or namespace body."),
            this_cannot_be_referenced_in_current_location: diag(2332, ts.DiagnosticCategory.Error, "this_cannot_be_referenced_in_current_location_2332", "'this' cannot be referenced in current location."),
            this_cannot_be_referenced_in_constructor_arguments: diag(2333, ts.DiagnosticCategory.Error, "this_cannot_be_referenced_in_constructor_arguments_2333", "'this' cannot be referenced in constructor arguments."),
            this_cannot_be_referenced_in_a_static_property_initializer: diag(2334, ts.DiagnosticCategory.Error, "this_cannot_be_referenced_in_a_static_property_initializer_2334", "'this' cannot be referenced in a static property initializer."),
            super_can_only_be_referenced_in_a_derived_class: diag(2335, ts.DiagnosticCategory.Error, "super_can_only_be_referenced_in_a_derived_class_2335", "'super' can only be referenced in a derived class."),
            super_cannot_be_referenced_in_constructor_arguments: diag(2336, ts.DiagnosticCategory.Error, "super_cannot_be_referenced_in_constructor_arguments_2336", "'super' cannot be referenced in constructor arguments."),
            Super_calls_are_not_permitted_outside_constructors_or_in_nested_functions_inside_constructors: diag(2337, ts.DiagnosticCategory.Error, "Super_calls_are_not_permitted_outside_constructors_or_in_nested_functions_inside_constructors_2337", "Super calls are not permitted outside constructors or in nested functions inside constructors."),
            super_property_access_is_permitted_only_in_a_constructor_member_function_or_member_accessor_of_a_derived_class: diag(2338, ts.DiagnosticCategory.Error, "super_property_access_is_permitted_only_in_a_constructor_member_function_or_member_accessor_of_a_der_2338", "'super' property access is permitted only in a constructor, member function, or member accessor of a derived class."),
            Property_0_does_not_exist_on_type_1: diag(2339, ts.DiagnosticCategory.Error, "Property_0_does_not_exist_on_type_1_2339", "Property '{0}' does not exist on type '{1}'."),
            Only_public_and_protected_methods_of_the_base_class_are_accessible_via_the_super_keyword: diag(2340, ts.DiagnosticCategory.Error, "Only_public_and_protected_methods_of_the_base_class_are_accessible_via_the_super_keyword_2340", "Only public and protected methods of the base class are accessible via the 'super' keyword."),
            Property_0_is_private_and_only_accessible_within_class_1: diag(2341, ts.DiagnosticCategory.Error, "Property_0_is_private_and_only_accessible_within_class_1_2341", "Property '{0}' is private and only accessible within class '{1}'."),
            An_index_expression_argument_must_be_of_type_string_number_symbol_or_any: diag(2342, ts.DiagnosticCategory.Error, "An_index_expression_argument_must_be_of_type_string_number_symbol_or_any_2342", "An index expression argument must be of type 'string', 'number', 'symbol', or 'any'."),
            This_syntax_requires_an_imported_helper_named_1_which_does_not_exist_in_0_Consider_upgrading_your_version_of_0: diag(2343, ts.DiagnosticCategory.Error, "This_syntax_requires_an_imported_helper_named_1_which_does_not_exist_in_0_Consider_upgrading_your_ve_2343", "This syntax requires an imported helper named '{1}' which does not exist in '{0}'. Consider upgrading your version of '{0}'."),
            Type_0_does_not_satisfy_the_constraint_1: diag(2344, ts.DiagnosticCategory.Error, "Type_0_does_not_satisfy_the_constraint_1_2344", "Type '{0}' does not satisfy the constraint '{1}'."),
            Argument_of_type_0_is_not_assignable_to_parameter_of_type_1: diag(2345, ts.DiagnosticCategory.Error, "Argument_of_type_0_is_not_assignable_to_parameter_of_type_1_2345", "Argument of type '{0}' is not assignable to parameter of type '{1}'."),
            Call_target_does_not_contain_any_signatures: diag(2346, ts.DiagnosticCategory.Error, "Call_target_does_not_contain_any_signatures_2346", "Call target does not contain any signatures."),
            Untyped_function_calls_may_not_accept_type_arguments: diag(2347, ts.DiagnosticCategory.Error, "Untyped_function_calls_may_not_accept_type_arguments_2347", "Untyped function calls may not accept type arguments."),
            Value_of_type_0_is_not_callable_Did_you_mean_to_include_new: diag(2348, ts.DiagnosticCategory.Error, "Value_of_type_0_is_not_callable_Did_you_mean_to_include_new_2348", "Value of type '{0}' is not callable. Did you mean to include 'new'?"),
            This_expression_is_not_callable: diag(2349, ts.DiagnosticCategory.Error, "This_expression_is_not_callable_2349", "This expression is not callable."),
            Only_a_void_function_can_be_called_with_the_new_keyword: diag(2350, ts.DiagnosticCategory.Error, "Only_a_void_function_can_be_called_with_the_new_keyword_2350", "Only a void function can be called with the 'new' keyword."),
            This_expression_is_not_constructable: diag(2351, ts.DiagnosticCategory.Error, "This_expression_is_not_constructable_2351", "This expression is not constructable."),
            Conversion_of_type_0_to_type_1_may_be_a_mistake_because_neither_type_sufficiently_overlaps_with_the_other_If_this_was_intentional_convert_the_expression_to_unknown_first: diag(2352, ts.DiagnosticCategory.Error, "Conversion_of_type_0_to_type_1_may_be_a_mistake_because_neither_type_sufficiently_overlaps_with_the__2352", "Conversion of type '{0}' to type '{1}' may be a mistake because neither type sufficiently overlaps with the other. If this was intentional, convert the expression to 'unknown' first."),
            Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1: diag(2353, ts.DiagnosticCategory.Error, "Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1_2353", "Object literal may only specify known properties, and '{0}' does not exist in type '{1}'."),
            This_syntax_requires_an_imported_helper_but_module_0_cannot_be_found: diag(2354, ts.DiagnosticCategory.Error, "This_syntax_requires_an_imported_helper_but_module_0_cannot_be_found_2354", "This syntax requires an imported helper but module '{0}' cannot be found."),
            A_function_whose_declared_type_is_neither_void_nor_any_must_return_a_value: diag(2355, ts.DiagnosticCategory.Error, "A_function_whose_declared_type_is_neither_void_nor_any_must_return_a_value_2355", "A function whose declared type is neither 'void' nor 'any' must return a value."),
            An_arithmetic_operand_must_be_of_type_any_number_bigint_or_an_enum_type: diag(2356, ts.DiagnosticCategory.Error, "An_arithmetic_operand_must_be_of_type_any_number_bigint_or_an_enum_type_2356", "An arithmetic operand must be of type 'any', 'number', 'bigint' or an enum type."),
            The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access: diag(2357, ts.DiagnosticCategory.Error, "The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access_2357", "The operand of an increment or decrement operator must be a variable or a property access."),
            The_left_hand_side_of_an_instanceof_expression_must_be_of_type_any_an_object_type_or_a_type_parameter: diag(2358, ts.DiagnosticCategory.Error, "The_left_hand_side_of_an_instanceof_expression_must_be_of_type_any_an_object_type_or_a_type_paramete_2358", "The left-hand side of an 'instanceof' expression must be of type 'any', an object type or a type parameter."),
            The_right_hand_side_of_an_instanceof_expression_must_be_of_type_any_or_of_a_type_assignable_to_the_Function_interface_type: diag(2359, ts.DiagnosticCategory.Error, "The_right_hand_side_of_an_instanceof_expression_must_be_of_type_any_or_of_a_type_assignable_to_the_F_2359", "The right-hand side of an 'instanceof' expression must be of type 'any' or of a type assignable to the 'Function' interface type."),
            The_left_hand_side_of_an_in_expression_must_be_of_type_any_string_number_or_symbol: diag(2360, ts.DiagnosticCategory.Error, "The_left_hand_side_of_an_in_expression_must_be_of_type_any_string_number_or_symbol_2360", "The left-hand side of an 'in' expression must be of type 'any', 'string', 'number', or 'symbol'."),
            The_right_hand_side_of_an_in_expression_must_be_of_type_any_an_object_type_or_a_type_parameter: diag(2361, ts.DiagnosticCategory.Error, "The_right_hand_side_of_an_in_expression_must_be_of_type_any_an_object_type_or_a_type_parameter_2361", "The right-hand side of an 'in' expression must be of type 'any', an object type or a type parameter."),
            The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type: diag(2362, ts.DiagnosticCategory.Error, "The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type_2362", "The left-hand side of an arithmetic operation must be of type 'any', 'number', 'bigint' or an enum type."),
            The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type: diag(2363, ts.DiagnosticCategory.Error, "The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type_2363", "The right-hand side of an arithmetic operation must be of type 'any', 'number', 'bigint' or an enum type."),
            The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access: diag(2364, ts.DiagnosticCategory.Error, "The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access_2364", "The left-hand side of an assignment expression must be a variable or a property access."),
            Operator_0_cannot_be_applied_to_types_1_and_2: diag(2365, ts.DiagnosticCategory.Error, "Operator_0_cannot_be_applied_to_types_1_and_2_2365", "Operator '{0}' cannot be applied to types '{1}' and '{2}'."),
            Function_lacks_ending_return_statement_and_return_type_does_not_include_undefined: diag(2366, ts.DiagnosticCategory.Error, "Function_lacks_ending_return_statement_and_return_type_does_not_include_undefined_2366", "Function lacks ending return statement and return type does not include 'undefined'."),
            This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap: diag(2367, ts.DiagnosticCategory.Error, "This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap_2367", "This condition will always return '{0}' since the types '{1}' and '{2}' have no overlap."),
            Type_parameter_name_cannot_be_0: diag(2368, ts.DiagnosticCategory.Error, "Type_parameter_name_cannot_be_0_2368", "Type parameter name cannot be '{0}'."),
            A_parameter_property_is_only_allowed_in_a_constructor_implementation: diag(2369, ts.DiagnosticCategory.Error, "A_parameter_property_is_only_allowed_in_a_constructor_implementation_2369", "A parameter property is only allowed in a constructor implementation."),
            A_rest_parameter_must_be_of_an_array_type: diag(2370, ts.DiagnosticCategory.Error, "A_rest_parameter_must_be_of_an_array_type_2370", "A rest parameter must be of an array type."),
            A_parameter_initializer_is_only_allowed_in_a_function_or_constructor_implementation: diag(2371, ts.DiagnosticCategory.Error, "A_parameter_initializer_is_only_allowed_in_a_function_or_constructor_implementation_2371", "A parameter initializer is only allowed in a function or constructor implementation."),
            Parameter_0_cannot_be_referenced_in_its_initializer: diag(2372, ts.DiagnosticCategory.Error, "Parameter_0_cannot_be_referenced_in_its_initializer_2372", "Parameter '{0}' cannot be referenced in its initializer."),
            Initializer_of_parameter_0_cannot_reference_identifier_1_declared_after_it: diag(2373, ts.DiagnosticCategory.Error, "Initializer_of_parameter_0_cannot_reference_identifier_1_declared_after_it_2373", "Initializer of parameter '{0}' cannot reference identifier '{1}' declared after it."),
            Duplicate_string_index_signature: diag(2374, ts.DiagnosticCategory.Error, "Duplicate_string_index_signature_2374", "Duplicate string index signature."),
            Duplicate_number_index_signature: diag(2375, ts.DiagnosticCategory.Error, "Duplicate_number_index_signature_2375", "Duplicate number index signature."),
            A_super_call_must_be_the_first_statement_in_the_constructor_when_a_class_contains_initialized_properties_parameter_properties_or_private_identifiers: diag(2376, ts.DiagnosticCategory.Error, "A_super_call_must_be_the_first_statement_in_the_constructor_when_a_class_contains_initialized_proper_2376", "A 'super' call must be the first statement in the constructor when a class contains initialized properties, parameter properties, or private identifiers."),
            Constructors_for_derived_classes_must_contain_a_super_call: diag(2377, ts.DiagnosticCategory.Error, "Constructors_for_derived_classes_must_contain_a_super_call_2377", "Constructors for derived classes must contain a 'super' call."),
            A_get_accessor_must_return_a_value: diag(2378, ts.DiagnosticCategory.Error, "A_get_accessor_must_return_a_value_2378", "A 'get' accessor must return a value."),
            Getter_and_setter_accessors_do_not_agree_in_visibility: diag(2379, ts.DiagnosticCategory.Error, "Getter_and_setter_accessors_do_not_agree_in_visibility_2379", "Getter and setter accessors do not agree in visibility."),
            get_and_set_accessor_must_have_the_same_type: diag(2380, ts.DiagnosticCategory.Error, "get_and_set_accessor_must_have_the_same_type_2380", "'get' and 'set' accessor must have the same type."),
            A_signature_with_an_implementation_cannot_use_a_string_literal_type: diag(2381, ts.DiagnosticCategory.Error, "A_signature_with_an_implementation_cannot_use_a_string_literal_type_2381", "A signature with an implementation cannot use a string literal type."),
            Specialized_overload_signature_is_not_assignable_to_any_non_specialized_signature: diag(2382, ts.DiagnosticCategory.Error, "Specialized_overload_signature_is_not_assignable_to_any_non_specialized_signature_2382", "Specialized overload signature is not assignable to any non-specialized signature."),
            Overload_signatures_must_all_be_exported_or_non_exported: diag(2383, ts.DiagnosticCategory.Error, "Overload_signatures_must_all_be_exported_or_non_exported_2383", "Overload signatures must all be exported or non-exported."),
            Overload_signatures_must_all_be_ambient_or_non_ambient: diag(2384, ts.DiagnosticCategory.Error, "Overload_signatures_must_all_be_ambient_or_non_ambient_2384", "Overload signatures must all be ambient or non-ambient."),
            Overload_signatures_must_all_be_public_private_or_protected: diag(2385, ts.DiagnosticCategory.Error, "Overload_signatures_must_all_be_public_private_or_protected_2385", "Overload signatures must all be public, private or protected."),
            Overload_signatures_must_all_be_optional_or_required: diag(2386, ts.DiagnosticCategory.Error, "Overload_signatures_must_all_be_optional_or_required_2386", "Overload signatures must all be optional or required."),
            Function_overload_must_be_static: diag(2387, ts.DiagnosticCategory.Error, "Function_overload_must_be_static_2387", "Function overload must be static."),
            Function_overload_must_not_be_static: diag(2388, ts.DiagnosticCategory.Error, "Function_overload_must_not_be_static_2388", "Function overload must not be static."),
            Function_implementation_name_must_be_0: diag(2389, ts.DiagnosticCategory.Error, "Function_implementation_name_must_be_0_2389", "Function implementation name must be '{0}'."),
            Constructor_implementation_is_missing: diag(2390, ts.DiagnosticCategory.Error, "Constructor_implementation_is_missing_2390", "Constructor implementation is missing."),
            Function_implementation_is_missing_or_not_immediately_following_the_declaration: diag(2391, ts.DiagnosticCategory.Error, "Function_implementation_is_missing_or_not_immediately_following_the_declaration_2391", "Function implementation is missing or not immediately following the declaration."),
            Multiple_constructor_implementations_are_not_allowed: diag(2392, ts.DiagnosticCategory.Error, "Multiple_constructor_implementations_are_not_allowed_2392", "Multiple constructor implementations are not allowed."),
            Duplicate_function_implementation: diag(2393, ts.DiagnosticCategory.Error, "Duplicate_function_implementation_2393", "Duplicate function implementation."),
            This_overload_signature_is_not_compatible_with_its_implementation_signature: diag(2394, ts.DiagnosticCategory.Error, "This_overload_signature_is_not_compatible_with_its_implementation_signature_2394", "This overload signature is not compatible with its implementation signature."),
            Individual_declarations_in_merged_declaration_0_must_be_all_exported_or_all_local: diag(2395, ts.DiagnosticCategory.Error, "Individual_declarations_in_merged_declaration_0_must_be_all_exported_or_all_local_2395", "Individual declarations in merged declaration '{0}' must be all exported or all local."),
            Duplicate_identifier_arguments_Compiler_uses_arguments_to_initialize_rest_parameters: diag(2396, ts.DiagnosticCategory.Error, "Duplicate_identifier_arguments_Compiler_uses_arguments_to_initialize_rest_parameters_2396", "Duplicate identifier 'arguments'. Compiler uses 'arguments' to initialize rest parameters."),
            Declaration_name_conflicts_with_built_in_global_identifier_0: diag(2397, ts.DiagnosticCategory.Error, "Declaration_name_conflicts_with_built_in_global_identifier_0_2397", "Declaration name conflicts with built-in global identifier '{0}'."),
            Duplicate_identifier_this_Compiler_uses_variable_declaration_this_to_capture_this_reference: diag(2399, ts.DiagnosticCategory.Error, "Duplicate_identifier_this_Compiler_uses_variable_declaration_this_to_capture_this_reference_2399", "Duplicate identifier '_this'. Compiler uses variable declaration '_this' to capture 'this' reference."),
            Expression_resolves_to_variable_declaration_this_that_compiler_uses_to_capture_this_reference: diag(2400, ts.DiagnosticCategory.Error, "Expression_resolves_to_variable_declaration_this_that_compiler_uses_to_capture_this_reference_2400", "Expression resolves to variable declaration '_this' that compiler uses to capture 'this' reference."),
            Duplicate_identifier_super_Compiler_uses_super_to_capture_base_class_reference: diag(2401, ts.DiagnosticCategory.Error, "Duplicate_identifier_super_Compiler_uses_super_to_capture_base_class_reference_2401", "Duplicate identifier '_super'. Compiler uses '_super' to capture base class reference."),
            Expression_resolves_to_super_that_compiler_uses_to_capture_base_class_reference: diag(2402, ts.DiagnosticCategory.Error, "Expression_resolves_to_super_that_compiler_uses_to_capture_base_class_reference_2402", "Expression resolves to '_super' that compiler uses to capture base class reference."),
            Subsequent_variable_declarations_must_have_the_same_type_Variable_0_must_be_of_type_1_but_here_has_type_2: diag(2403, ts.DiagnosticCategory.Error, "Subsequent_variable_declarations_must_have_the_same_type_Variable_0_must_be_of_type_1_but_here_has_t_2403", "Subsequent variable declarations must have the same type.  Variable '{0}' must be of type '{1}', but here has type '{2}'."),
            The_left_hand_side_of_a_for_in_statement_cannot_use_a_type_annotation: diag(2404, ts.DiagnosticCategory.Error, "The_left_hand_side_of_a_for_in_statement_cannot_use_a_type_annotation_2404", "The left-hand side of a 'for...in' statement cannot use a type annotation."),
            The_left_hand_side_of_a_for_in_statement_must_be_of_type_string_or_any: diag(2405, ts.DiagnosticCategory.Error, "The_left_hand_side_of_a_for_in_statement_must_be_of_type_string_or_any_2405", "The left-hand side of a 'for...in' statement must be of type 'string' or 'any'."),
            The_left_hand_side_of_a_for_in_statement_must_be_a_variable_or_a_property_access: diag(2406, ts.DiagnosticCategory.Error, "The_left_hand_side_of_a_for_in_statement_must_be_a_variable_or_a_property_access_2406", "The left-hand side of a 'for...in' statement must be a variable or a property access."),
            The_right_hand_side_of_a_for_in_statement_must_be_of_type_any_an_object_type_or_a_type_parameter_but_here_has_type_0: diag(2407, ts.DiagnosticCategory.Error, "The_right_hand_side_of_a_for_in_statement_must_be_of_type_any_an_object_type_or_a_type_parameter_but_2407", "The right-hand side of a 'for...in' statement must be of type 'any', an object type or a type parameter, but here has type '{0}'."),
            Setters_cannot_return_a_value: diag(2408, ts.DiagnosticCategory.Error, "Setters_cannot_return_a_value_2408", "Setters cannot return a value."),
            Return_type_of_constructor_signature_must_be_assignable_to_the_instance_type_of_the_class: diag(2409, ts.DiagnosticCategory.Error, "Return_type_of_constructor_signature_must_be_assignable_to_the_instance_type_of_the_class_2409", "Return type of constructor signature must be assignable to the instance type of the class."),
            The_with_statement_is_not_supported_All_symbols_in_a_with_block_will_have_type_any: diag(2410, ts.DiagnosticCategory.Error, "The_with_statement_is_not_supported_All_symbols_in_a_with_block_will_have_type_any_2410", "The 'with' statement is not supported. All symbols in a 'with' block will have type 'any'."),
            Property_0_of_type_1_is_not_assignable_to_string_index_type_2: diag(2411, ts.DiagnosticCategory.Error, "Property_0_of_type_1_is_not_assignable_to_string_index_type_2_2411", "Property '{0}' of type '{1}' is not assignable to string index type '{2}'."),
            Property_0_of_type_1_is_not_assignable_to_numeric_index_type_2: diag(2412, ts.DiagnosticCategory.Error, "Property_0_of_type_1_is_not_assignable_to_numeric_index_type_2_2412", "Property '{0}' of type '{1}' is not assignable to numeric index type '{2}'."),
            Numeric_index_type_0_is_not_assignable_to_string_index_type_1: diag(2413, ts.DiagnosticCategory.Error, "Numeric_index_type_0_is_not_assignable_to_string_index_type_1_2413", "Numeric index type '{0}' is not assignable to string index type '{1}'."),
            Class_name_cannot_be_0: diag(2414, ts.DiagnosticCategory.Error, "Class_name_cannot_be_0_2414", "Class name cannot be '{0}'."),
            Class_0_incorrectly_extends_base_class_1: diag(2415, ts.DiagnosticCategory.Error, "Class_0_incorrectly_extends_base_class_1_2415", "Class '{0}' incorrectly extends base class '{1}'."),
            Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2: diag(2416, ts.DiagnosticCategory.Error, "Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2_2416", "Property '{0}' in type '{1}' is not assignable to the same property in base type '{2}'."),
            Class_static_side_0_incorrectly_extends_base_class_static_side_1: diag(2417, ts.DiagnosticCategory.Error, "Class_static_side_0_incorrectly_extends_base_class_static_side_1_2417", "Class static side '{0}' incorrectly extends base class static side '{1}'."),
            Type_of_computed_property_s_value_is_0_which_is_not_assignable_to_type_1: diag(2418, ts.DiagnosticCategory.Error, "Type_of_computed_property_s_value_is_0_which_is_not_assignable_to_type_1_2418", "Type of computed property's value is '{0}', which is not assignable to type '{1}'."),
            Class_0_incorrectly_implements_interface_1: diag(2420, ts.DiagnosticCategory.Error, "Class_0_incorrectly_implements_interface_1_2420", "Class '{0}' incorrectly implements interface '{1}'."),
            A_class_can_only_implement_an_object_type_or_intersection_of_object_types_with_statically_known_members: diag(2422, ts.DiagnosticCategory.Error, "A_class_can_only_implement_an_object_type_or_intersection_of_object_types_with_statically_known_memb_2422", "A class can only implement an object type or intersection of object types with statically known members."),
            Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_accessor: diag(2423, ts.DiagnosticCategory.Error, "Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_access_2423", "Class '{0}' defines instance member function '{1}', but extended class '{2}' defines it as instance member accessor."),
            Class_0_defines_instance_member_property_1_but_extended_class_2_defines_it_as_instance_member_function: diag(2425, ts.DiagnosticCategory.Error, "Class_0_defines_instance_member_property_1_but_extended_class_2_defines_it_as_instance_member_functi_2425", "Class '{0}' defines instance member property '{1}', but extended class '{2}' defines it as instance member function."),
            Class_0_defines_instance_member_accessor_1_but_extended_class_2_defines_it_as_instance_member_function: diag(2426, ts.DiagnosticCategory.Error, "Class_0_defines_instance_member_accessor_1_but_extended_class_2_defines_it_as_instance_member_functi_2426", "Class '{0}' defines instance member accessor '{1}', but extended class '{2}' defines it as instance member function."),
            Interface_name_cannot_be_0: diag(2427, ts.DiagnosticCategory.Error, "Interface_name_cannot_be_0_2427", "Interface name cannot be '{0}'."),
            All_declarations_of_0_must_have_identical_type_parameters: diag(2428, ts.DiagnosticCategory.Error, "All_declarations_of_0_must_have_identical_type_parameters_2428", "All declarations of '{0}' must have identical type parameters."),
            Interface_0_incorrectly_extends_interface_1: diag(2430, ts.DiagnosticCategory.Error, "Interface_0_incorrectly_extends_interface_1_2430", "Interface '{0}' incorrectly extends interface '{1}'."),
            Enum_name_cannot_be_0: diag(2431, ts.DiagnosticCategory.Error, "Enum_name_cannot_be_0_2431", "Enum name cannot be '{0}'."),
            In_an_enum_with_multiple_declarations_only_one_declaration_can_omit_an_initializer_for_its_first_enum_element: diag(2432, ts.DiagnosticCategory.Error, "In_an_enum_with_multiple_declarations_only_one_declaration_can_omit_an_initializer_for_its_first_enu_2432", "In an enum with multiple declarations, only one declaration can omit an initializer for its first enum element."),
            A_namespace_declaration_cannot_be_in_a_different_file_from_a_class_or_function_with_which_it_is_merged: diag(2433, ts.DiagnosticCategory.Error, "A_namespace_declaration_cannot_be_in_a_different_file_from_a_class_or_function_with_which_it_is_merg_2433", "A namespace declaration cannot be in a different file from a class or function with which it is merged."),
            A_namespace_declaration_cannot_be_located_prior_to_a_class_or_function_with_which_it_is_merged: diag(2434, ts.DiagnosticCategory.Error, "A_namespace_declaration_cannot_be_located_prior_to_a_class_or_function_with_which_it_is_merged_2434", "A namespace declaration cannot be located prior to a class or function with which it is merged."),
            Ambient_modules_cannot_be_nested_in_other_modules_or_namespaces: diag(2435, ts.DiagnosticCategory.Error, "Ambient_modules_cannot_be_nested_in_other_modules_or_namespaces_2435", "Ambient modules cannot be nested in other modules or namespaces."),
            Ambient_module_declaration_cannot_specify_relative_module_name: diag(2436, ts.DiagnosticCategory.Error, "Ambient_module_declaration_cannot_specify_relative_module_name_2436", "Ambient module declaration cannot specify relative module name."),
            Module_0_is_hidden_by_a_local_declaration_with_the_same_name: diag(2437, ts.DiagnosticCategory.Error, "Module_0_is_hidden_by_a_local_declaration_with_the_same_name_2437", "Module '{0}' is hidden by a local declaration with the same name."),
            Import_name_cannot_be_0: diag(2438, ts.DiagnosticCategory.Error, "Import_name_cannot_be_0_2438", "Import name cannot be '{0}'."),
            Import_or_export_declaration_in_an_ambient_module_declaration_cannot_reference_module_through_relative_module_name: diag(2439, ts.DiagnosticCategory.Error, "Import_or_export_declaration_in_an_ambient_module_declaration_cannot_reference_module_through_relati_2439", "Import or export declaration in an ambient module declaration cannot reference module through relative module name."),
            Import_declaration_conflicts_with_local_declaration_of_0: diag(2440, ts.DiagnosticCategory.Error, "Import_declaration_conflicts_with_local_declaration_of_0_2440", "Import declaration conflicts with local declaration of '{0}'."),
            Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module: diag(2441, ts.DiagnosticCategory.Error, "Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_2441", "Duplicate identifier '{0}'. Compiler reserves name '{1}' in top level scope of a module."),
            Types_have_separate_declarations_of_a_private_property_0: diag(2442, ts.DiagnosticCategory.Error, "Types_have_separate_declarations_of_a_private_property_0_2442", "Types have separate declarations of a private property '{0}'."),
            Property_0_is_protected_but_type_1_is_not_a_class_derived_from_2: diag(2443, ts.DiagnosticCategory.Error, "Property_0_is_protected_but_type_1_is_not_a_class_derived_from_2_2443", "Property '{0}' is protected but type '{1}' is not a class derived from '{2}'."),
            Property_0_is_protected_in_type_1_but_public_in_type_2: diag(2444, ts.DiagnosticCategory.Error, "Property_0_is_protected_in_type_1_but_public_in_type_2_2444", "Property '{0}' is protected in type '{1}' but public in type '{2}'."),
            Property_0_is_protected_and_only_accessible_within_class_1_and_its_subclasses: diag(2445, ts.DiagnosticCategory.Error, "Property_0_is_protected_and_only_accessible_within_class_1_and_its_subclasses_2445", "Property '{0}' is protected and only accessible within class '{1}' and its subclasses."),
            Property_0_is_protected_and_only_accessible_through_an_instance_of_class_1: diag(2446, ts.DiagnosticCategory.Error, "Property_0_is_protected_and_only_accessible_through_an_instance_of_class_1_2446", "Property '{0}' is protected and only accessible through an instance of class '{1}'."),
            The_0_operator_is_not_allowed_for_boolean_types_Consider_using_1_instead: diag(2447, ts.DiagnosticCategory.Error, "The_0_operator_is_not_allowed_for_boolean_types_Consider_using_1_instead_2447", "The '{0}' operator is not allowed for boolean types. Consider using '{1}' instead."),
            Block_scoped_variable_0_used_before_its_declaration: diag(2448, ts.DiagnosticCategory.Error, "Block_scoped_variable_0_used_before_its_declaration_2448", "Block-scoped variable '{0}' used before its declaration."),
            Class_0_used_before_its_declaration: diag(2449, ts.DiagnosticCategory.Error, "Class_0_used_before_its_declaration_2449", "Class '{0}' used before its declaration."),
            Enum_0_used_before_its_declaration: diag(2450, ts.DiagnosticCategory.Error, "Enum_0_used_before_its_declaration_2450", "Enum '{0}' used before its declaration."),
            Cannot_redeclare_block_scoped_variable_0: diag(2451, ts.DiagnosticCategory.Error, "Cannot_redeclare_block_scoped_variable_0_2451", "Cannot redeclare block-scoped variable '{0}'."),
            An_enum_member_cannot_have_a_numeric_name: diag(2452, ts.DiagnosticCategory.Error, "An_enum_member_cannot_have_a_numeric_name_2452", "An enum member cannot have a numeric name."),
            The_type_argument_for_type_parameter_0_cannot_be_inferred_from_the_usage_Consider_specifying_the_type_arguments_explicitly: diag(2453, ts.DiagnosticCategory.Error, "The_type_argument_for_type_parameter_0_cannot_be_inferred_from_the_usage_Consider_specifying_the_typ_2453", "The type argument for type parameter '{0}' cannot be inferred from the usage. Consider specifying the type arguments explicitly."),
            Variable_0_is_used_before_being_assigned: diag(2454, ts.DiagnosticCategory.Error, "Variable_0_is_used_before_being_assigned_2454", "Variable '{0}' is used before being assigned."),
            Type_argument_candidate_1_is_not_a_valid_type_argument_because_it_is_not_a_supertype_of_candidate_0: diag(2455, ts.DiagnosticCategory.Error, "Type_argument_candidate_1_is_not_a_valid_type_argument_because_it_is_not_a_supertype_of_candidate_0_2455", "Type argument candidate '{1}' is not a valid type argument because it is not a supertype of candidate '{0}'."),
            Type_alias_0_circularly_references_itself: diag(2456, ts.DiagnosticCategory.Error, "Type_alias_0_circularly_references_itself_2456", "Type alias '{0}' circularly references itself."),
            Type_alias_name_cannot_be_0: diag(2457, ts.DiagnosticCategory.Error, "Type_alias_name_cannot_be_0_2457", "Type alias name cannot be '{0}'."),
            An_AMD_module_cannot_have_multiple_name_assignments: diag(2458, ts.DiagnosticCategory.Error, "An_AMD_module_cannot_have_multiple_name_assignments_2458", "An AMD module cannot have multiple name assignments."),
            Module_0_declares_1_locally_but_it_is_not_exported: diag(2459, ts.DiagnosticCategory.Error, "Module_0_declares_1_locally_but_it_is_not_exported_2459", "Module '{0}' declares '{1}' locally, but it is not exported."),
            Module_0_declares_1_locally_but_it_is_exported_as_2: diag(2460, ts.DiagnosticCategory.Error, "Module_0_declares_1_locally_but_it_is_exported_as_2_2460", "Module '{0}' declares '{1}' locally, but it is exported as '{2}'."),
            Type_0_is_not_an_array_type: diag(2461, ts.DiagnosticCategory.Error, "Type_0_is_not_an_array_type_2461", "Type '{0}' is not an array type."),
            A_rest_element_must_be_last_in_a_destructuring_pattern: diag(2462, ts.DiagnosticCategory.Error, "A_rest_element_must_be_last_in_a_destructuring_pattern_2462", "A rest element must be last in a destructuring pattern."),
            A_binding_pattern_parameter_cannot_be_optional_in_an_implementation_signature: diag(2463, ts.DiagnosticCategory.Error, "A_binding_pattern_parameter_cannot_be_optional_in_an_implementation_signature_2463", "A binding pattern parameter cannot be optional in an implementation signature."),
            A_computed_property_name_must_be_of_type_string_number_symbol_or_any: diag(2464, ts.DiagnosticCategory.Error, "A_computed_property_name_must_be_of_type_string_number_symbol_or_any_2464", "A computed property name must be of type 'string', 'number', 'symbol', or 'any'."),
            this_cannot_be_referenced_in_a_computed_property_name: diag(2465, ts.DiagnosticCategory.Error, "this_cannot_be_referenced_in_a_computed_property_name_2465", "'this' cannot be referenced in a computed property name."),
            super_cannot_be_referenced_in_a_computed_property_name: diag(2466, ts.DiagnosticCategory.Error, "super_cannot_be_referenced_in_a_computed_property_name_2466", "'super' cannot be referenced in a computed property name."),
            A_computed_property_name_cannot_reference_a_type_parameter_from_its_containing_type: diag(2467, ts.DiagnosticCategory.Error, "A_computed_property_name_cannot_reference_a_type_parameter_from_its_containing_type_2467", "A computed property name cannot reference a type parameter from its containing type."),
            Cannot_find_global_value_0: diag(2468, ts.DiagnosticCategory.Error, "Cannot_find_global_value_0_2468", "Cannot find global value '{0}'."),
            The_0_operator_cannot_be_applied_to_type_symbol: diag(2469, ts.DiagnosticCategory.Error, "The_0_operator_cannot_be_applied_to_type_symbol_2469", "The '{0}' operator cannot be applied to type 'symbol'."),
            Symbol_reference_does_not_refer_to_the_global_Symbol_constructor_object: diag(2470, ts.DiagnosticCategory.Error, "Symbol_reference_does_not_refer_to_the_global_Symbol_constructor_object_2470", "'Symbol' reference does not refer to the global Symbol constructor object."),
            A_computed_property_name_of_the_form_0_must_be_of_type_symbol: diag(2471, ts.DiagnosticCategory.Error, "A_computed_property_name_of_the_form_0_must_be_of_type_symbol_2471", "A computed property name of the form '{0}' must be of type 'symbol'."),
            Spread_operator_in_new_expressions_is_only_available_when_targeting_ECMAScript_5_and_higher: diag(2472, ts.DiagnosticCategory.Error, "Spread_operator_in_new_expressions_is_only_available_when_targeting_ECMAScript_5_and_higher_2472", "Spread operator in 'new' expressions is only available when targeting ECMAScript 5 and higher."),
            Enum_declarations_must_all_be_const_or_non_const: diag(2473, ts.DiagnosticCategory.Error, "Enum_declarations_must_all_be_const_or_non_const_2473", "Enum declarations must all be const or non-const."),
            const_enum_member_initializers_can_only_contain_literal_values_and_other_computed_enum_values: diag(2474, ts.DiagnosticCategory.Error, "const_enum_member_initializers_can_only_contain_literal_values_and_other_computed_enum_values_2474", "const enum member initializers can only contain literal values and other computed enum values."),
            const_enums_can_only_be_used_in_property_or_index_access_expressions_or_the_right_hand_side_of_an_import_declaration_or_export_assignment_or_type_query: diag(2475, ts.DiagnosticCategory.Error, "const_enums_can_only_be_used_in_property_or_index_access_expressions_or_the_right_hand_side_of_an_im_2475", "'const' enums can only be used in property or index access expressions or the right hand side of an import declaration or export assignment or type query."),
            A_const_enum_member_can_only_be_accessed_using_a_string_literal: diag(2476, ts.DiagnosticCategory.Error, "A_const_enum_member_can_only_be_accessed_using_a_string_literal_2476", "A const enum member can only be accessed using a string literal."),
            const_enum_member_initializer_was_evaluated_to_a_non_finite_value: diag(2477, ts.DiagnosticCategory.Error, "const_enum_member_initializer_was_evaluated_to_a_non_finite_value_2477", "'const' enum member initializer was evaluated to a non-finite value."),
            const_enum_member_initializer_was_evaluated_to_disallowed_value_NaN: diag(2478, ts.DiagnosticCategory.Error, "const_enum_member_initializer_was_evaluated_to_disallowed_value_NaN_2478", "'const' enum member initializer was evaluated to disallowed value 'NaN'."),
            Property_0_does_not_exist_on_const_enum_1: diag(2479, ts.DiagnosticCategory.Error, "Property_0_does_not_exist_on_const_enum_1_2479", "Property '{0}' does not exist on 'const' enum '{1}'."),
            let_is_not_allowed_to_be_used_as_a_name_in_let_or_const_declarations: diag(2480, ts.DiagnosticCategory.Error, "let_is_not_allowed_to_be_used_as_a_name_in_let_or_const_declarations_2480", "'let' is not allowed to be used as a name in 'let' or 'const' declarations."),
            Cannot_initialize_outer_scoped_variable_0_in_the_same_scope_as_block_scoped_declaration_1: diag(2481, ts.DiagnosticCategory.Error, "Cannot_initialize_outer_scoped_variable_0_in_the_same_scope_as_block_scoped_declaration_1_2481", "Cannot initialize outer scoped variable '{0}' in the same scope as block scoped declaration '{1}'."),
            The_left_hand_side_of_a_for_of_statement_cannot_use_a_type_annotation: diag(2483, ts.DiagnosticCategory.Error, "The_left_hand_side_of_a_for_of_statement_cannot_use_a_type_annotation_2483", "The left-hand side of a 'for...of' statement cannot use a type annotation."),
            Export_declaration_conflicts_with_exported_declaration_of_0: diag(2484, ts.DiagnosticCategory.Error, "Export_declaration_conflicts_with_exported_declaration_of_0_2484", "Export declaration conflicts with exported declaration of '{0}'."),
            The_left_hand_side_of_a_for_of_statement_must_be_a_variable_or_a_property_access: diag(2487, ts.DiagnosticCategory.Error, "The_left_hand_side_of_a_for_of_statement_must_be_a_variable_or_a_property_access_2487", "The left-hand side of a 'for...of' statement must be a variable or a property access."),
            Type_0_must_have_a_Symbol_iterator_method_that_returns_an_iterator: diag(2488, ts.DiagnosticCategory.Error, "Type_0_must_have_a_Symbol_iterator_method_that_returns_an_iterator_2488", "Type '{0}' must have a '[Symbol.iterator]()' method that returns an iterator."),
            An_iterator_must_have_a_next_method: diag(2489, ts.DiagnosticCategory.Error, "An_iterator_must_have_a_next_method_2489", "An iterator must have a 'next()' method."),
            The_type_returned_by_the_0_method_of_an_iterator_must_have_a_value_property: diag(2490, ts.DiagnosticCategory.Error, "The_type_returned_by_the_0_method_of_an_iterator_must_have_a_value_property_2490", "The type returned by the '{0}()' method of an iterator must have a 'value' property."),
            The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern: diag(2491, ts.DiagnosticCategory.Error, "The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern_2491", "The left-hand side of a 'for...in' statement cannot be a destructuring pattern."),
            Cannot_redeclare_identifier_0_in_catch_clause: diag(2492, ts.DiagnosticCategory.Error, "Cannot_redeclare_identifier_0_in_catch_clause_2492", "Cannot redeclare identifier '{0}' in catch clause."),
            Tuple_type_0_of_length_1_has_no_element_at_index_2: diag(2493, ts.DiagnosticCategory.Error, "Tuple_type_0_of_length_1_has_no_element_at_index_2_2493", "Tuple type '{0}' of length '{1}' has no element at index '{2}'."),
            Using_a_string_in_a_for_of_statement_is_only_supported_in_ECMAScript_5_and_higher: diag(2494, ts.DiagnosticCategory.Error, "Using_a_string_in_a_for_of_statement_is_only_supported_in_ECMAScript_5_and_higher_2494", "Using a string in a 'for...of' statement is only supported in ECMAScript 5 and higher."),
            Type_0_is_not_an_array_type_or_a_string_type: diag(2495, ts.DiagnosticCategory.Error, "Type_0_is_not_an_array_type_or_a_string_type_2495", "Type '{0}' is not an array type or a string type."),
            The_arguments_object_cannot_be_referenced_in_an_arrow_function_in_ES3_and_ES5_Consider_using_a_standard_function_expression: diag(2496, ts.DiagnosticCategory.Error, "The_arguments_object_cannot_be_referenced_in_an_arrow_function_in_ES3_and_ES5_Consider_using_a_stand_2496", "The 'arguments' object cannot be referenced in an arrow function in ES3 and ES5. Consider using a standard function expression."),
            This_module_can_only_be_referenced_with_ECMAScript_imports_Slashexports_by_turning_on_the_0_flag_and_referencing_its_default_export: diag(2497, ts.DiagnosticCategory.Error, "This_module_can_only_be_referenced_with_ECMAScript_imports_Slashexports_by_turning_on_the_0_flag_and_2497", "This module can only be referenced with ECMAScript imports/exports by turning on the '{0}' flag and referencing its default export."),
            Module_0_uses_export_and_cannot_be_used_with_export_Asterisk: diag(2498, ts.DiagnosticCategory.Error, "Module_0_uses_export_and_cannot_be_used_with_export_Asterisk_2498", "Module '{0}' uses 'export =' and cannot be used with 'export *'."),
            An_interface_can_only_extend_an_identifier_Slashqualified_name_with_optional_type_arguments: diag(2499, ts.DiagnosticCategory.Error, "An_interface_can_only_extend_an_identifier_Slashqualified_name_with_optional_type_arguments_2499", "An interface can only extend an identifier/qualified-name with optional type arguments."),
            A_class_can_only_implement_an_identifier_Slashqualified_name_with_optional_type_arguments: diag(2500, ts.DiagnosticCategory.Error, "A_class_can_only_implement_an_identifier_Slashqualified_name_with_optional_type_arguments_2500", "A class can only implement an identifier/qualified-name with optional type arguments."),
            A_rest_element_cannot_contain_a_binding_pattern: diag(2501, ts.DiagnosticCategory.Error, "A_rest_element_cannot_contain_a_binding_pattern_2501", "A rest element cannot contain a binding pattern."),
            _0_is_referenced_directly_or_indirectly_in_its_own_type_annotation: diag(2502, ts.DiagnosticCategory.Error, "_0_is_referenced_directly_or_indirectly_in_its_own_type_annotation_2502", "'{0}' is referenced directly or indirectly in its own type annotation."),
            Cannot_find_namespace_0: diag(2503, ts.DiagnosticCategory.Error, "Cannot_find_namespace_0_2503", "Cannot find namespace '{0}'."),
            Type_0_must_have_a_Symbol_asyncIterator_method_that_returns_an_async_iterator: diag(2504, ts.DiagnosticCategory.Error, "Type_0_must_have_a_Symbol_asyncIterator_method_that_returns_an_async_iterator_2504", "Type '{0}' must have a '[Symbol.asyncIterator]()' method that returns an async iterator."),
            A_generator_cannot_have_a_void_type_annotation: diag(2505, ts.DiagnosticCategory.Error, "A_generator_cannot_have_a_void_type_annotation_2505", "A generator cannot have a 'void' type annotation."),
            _0_is_referenced_directly_or_indirectly_in_its_own_base_expression: diag(2506, ts.DiagnosticCategory.Error, "_0_is_referenced_directly_or_indirectly_in_its_own_base_expression_2506", "'{0}' is referenced directly or indirectly in its own base expression."),
            Type_0_is_not_a_constructor_function_type: diag(2507, ts.DiagnosticCategory.Error, "Type_0_is_not_a_constructor_function_type_2507", "Type '{0}' is not a constructor function type."),
            No_base_constructor_has_the_specified_number_of_type_arguments: diag(2508, ts.DiagnosticCategory.Error, "No_base_constructor_has_the_specified_number_of_type_arguments_2508", "No base constructor has the specified number of type arguments."),
            Base_constructor_return_type_0_is_not_an_object_type_or_intersection_of_object_types_with_statically_known_members: diag(2509, ts.DiagnosticCategory.Error, "Base_constructor_return_type_0_is_not_an_object_type_or_intersection_of_object_types_with_statically_2509", "Base constructor return type '{0}' is not an object type or intersection of object types with statically known members."),
            Base_constructors_must_all_have_the_same_return_type: diag(2510, ts.DiagnosticCategory.Error, "Base_constructors_must_all_have_the_same_return_type_2510", "Base constructors must all have the same return type."),
            Cannot_create_an_instance_of_an_abstract_class: diag(2511, ts.DiagnosticCategory.Error, "Cannot_create_an_instance_of_an_abstract_class_2511", "Cannot create an instance of an abstract class."),
            Overload_signatures_must_all_be_abstract_or_non_abstract: diag(2512, ts.DiagnosticCategory.Error, "Overload_signatures_must_all_be_abstract_or_non_abstract_2512", "Overload signatures must all be abstract or non-abstract."),
            Abstract_method_0_in_class_1_cannot_be_accessed_via_super_expression: diag(2513, ts.DiagnosticCategory.Error, "Abstract_method_0_in_class_1_cannot_be_accessed_via_super_expression_2513", "Abstract method '{0}' in class '{1}' cannot be accessed via super expression."),
            Classes_containing_abstract_methods_must_be_marked_abstract: diag(2514, ts.DiagnosticCategory.Error, "Classes_containing_abstract_methods_must_be_marked_abstract_2514", "Classes containing abstract methods must be marked abstract."),
            Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2: diag(2515, ts.DiagnosticCategory.Error, "Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2_2515", "Non-abstract class '{0}' does not implement inherited abstract member '{1}' from class '{2}'."),
            All_declarations_of_an_abstract_method_must_be_consecutive: diag(2516, ts.DiagnosticCategory.Error, "All_declarations_of_an_abstract_method_must_be_consecutive_2516", "All declarations of an abstract method must be consecutive."),
            Cannot_assign_an_abstract_constructor_type_to_a_non_abstract_constructor_type: diag(2517, ts.DiagnosticCategory.Error, "Cannot_assign_an_abstract_constructor_type_to_a_non_abstract_constructor_type_2517", "Cannot assign an abstract constructor type to a non-abstract constructor type."),
            A_this_based_type_guard_is_not_compatible_with_a_parameter_based_type_guard: diag(2518, ts.DiagnosticCategory.Error, "A_this_based_type_guard_is_not_compatible_with_a_parameter_based_type_guard_2518", "A 'this'-based type guard is not compatible with a parameter-based type guard."),
            An_async_iterator_must_have_a_next_method: diag(2519, ts.DiagnosticCategory.Error, "An_async_iterator_must_have_a_next_method_2519", "An async iterator must have a 'next()' method."),
            Duplicate_identifier_0_Compiler_uses_declaration_1_to_support_async_functions: diag(2520, ts.DiagnosticCategory.Error, "Duplicate_identifier_0_Compiler_uses_declaration_1_to_support_async_functions_2520", "Duplicate identifier '{0}'. Compiler uses declaration '{1}' to support async functions."),
            Expression_resolves_to_variable_declaration_0_that_compiler_uses_to_support_async_functions: diag(2521, ts.DiagnosticCategory.Error, "Expression_resolves_to_variable_declaration_0_that_compiler_uses_to_support_async_functions_2521", "Expression resolves to variable declaration '{0}' that compiler uses to support async functions."),
            The_arguments_object_cannot_be_referenced_in_an_async_function_or_method_in_ES3_and_ES5_Consider_using_a_standard_function_or_method: diag(2522, ts.DiagnosticCategory.Error, "The_arguments_object_cannot_be_referenced_in_an_async_function_or_method_in_ES3_and_ES5_Consider_usi_2522", "The 'arguments' object cannot be referenced in an async function or method in ES3 and ES5. Consider using a standard function or method."),
            yield_expressions_cannot_be_used_in_a_parameter_initializer: diag(2523, ts.DiagnosticCategory.Error, "yield_expressions_cannot_be_used_in_a_parameter_initializer_2523", "'yield' expressions cannot be used in a parameter initializer."),
            await_expressions_cannot_be_used_in_a_parameter_initializer: diag(2524, ts.DiagnosticCategory.Error, "await_expressions_cannot_be_used_in_a_parameter_initializer_2524", "'await' expressions cannot be used in a parameter initializer."),
            Initializer_provides_no_value_for_this_binding_element_and_the_binding_element_has_no_default_value: diag(2525, ts.DiagnosticCategory.Error, "Initializer_provides_no_value_for_this_binding_element_and_the_binding_element_has_no_default_value_2525", "Initializer provides no value for this binding element and the binding element has no default value."),
            A_this_type_is_available_only_in_a_non_static_member_of_a_class_or_interface: diag(2526, ts.DiagnosticCategory.Error, "A_this_type_is_available_only_in_a_non_static_member_of_a_class_or_interface_2526", "A 'this' type is available only in a non-static member of a class or interface."),
            The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary: diag(2527, ts.DiagnosticCategory.Error, "The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary_2527", "The inferred type of '{0}' references an inaccessible '{1}' type. A type annotation is necessary."),
            A_module_cannot_have_multiple_default_exports: diag(2528, ts.DiagnosticCategory.Error, "A_module_cannot_have_multiple_default_exports_2528", "A module cannot have multiple default exports."),
            Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_containing_async_functions: diag(2529, ts.DiagnosticCategory.Error, "Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_containing_async_func_2529", "Duplicate identifier '{0}'. Compiler reserves name '{1}' in top level scope of a module containing async functions."),
            Property_0_is_incompatible_with_index_signature: diag(2530, ts.DiagnosticCategory.Error, "Property_0_is_incompatible_with_index_signature_2530", "Property '{0}' is incompatible with index signature."),
            Object_is_possibly_null: diag(2531, ts.DiagnosticCategory.Error, "Object_is_possibly_null_2531", "Object is possibly 'null'."),
            Object_is_possibly_undefined: diag(2532, ts.DiagnosticCategory.Error, "Object_is_possibly_undefined_2532", "Object is possibly 'undefined'."),
            Object_is_possibly_null_or_undefined: diag(2533, ts.DiagnosticCategory.Error, "Object_is_possibly_null_or_undefined_2533", "Object is possibly 'null' or 'undefined'."),
            A_function_returning_never_cannot_have_a_reachable_end_point: diag(2534, ts.DiagnosticCategory.Error, "A_function_returning_never_cannot_have_a_reachable_end_point_2534", "A function returning 'never' cannot have a reachable end point."),
            Enum_type_0_has_members_with_initializers_that_are_not_literals: diag(2535, ts.DiagnosticCategory.Error, "Enum_type_0_has_members_with_initializers_that_are_not_literals_2535", "Enum type '{0}' has members with initializers that are not literals."),
            Type_0_cannot_be_used_to_index_type_1: diag(2536, ts.DiagnosticCategory.Error, "Type_0_cannot_be_used_to_index_type_1_2536", "Type '{0}' cannot be used to index type '{1}'."),
            Type_0_has_no_matching_index_signature_for_type_1: diag(2537, ts.DiagnosticCategory.Error, "Type_0_has_no_matching_index_signature_for_type_1_2537", "Type '{0}' has no matching index signature for type '{1}'."),
            Type_0_cannot_be_used_as_an_index_type: diag(2538, ts.DiagnosticCategory.Error, "Type_0_cannot_be_used_as_an_index_type_2538", "Type '{0}' cannot be used as an index type."),
            Cannot_assign_to_0_because_it_is_not_a_variable: diag(2539, ts.DiagnosticCategory.Error, "Cannot_assign_to_0_because_it_is_not_a_variable_2539", "Cannot assign to '{0}' because it is not a variable."),
            Cannot_assign_to_0_because_it_is_a_read_only_property: diag(2540, ts.DiagnosticCategory.Error, "Cannot_assign_to_0_because_it_is_a_read_only_property_2540", "Cannot assign to '{0}' because it is a read-only property."),
            The_target_of_an_assignment_must_be_a_variable_or_a_property_access: diag(2541, ts.DiagnosticCategory.Error, "The_target_of_an_assignment_must_be_a_variable_or_a_property_access_2541", "The target of an assignment must be a variable or a property access."),
            Index_signature_in_type_0_only_permits_reading: diag(2542, ts.DiagnosticCategory.Error, "Index_signature_in_type_0_only_permits_reading_2542", "Index signature in type '{0}' only permits reading."),
            Duplicate_identifier_newTarget_Compiler_uses_variable_declaration_newTarget_to_capture_new_target_meta_property_reference: diag(2543, ts.DiagnosticCategory.Error, "Duplicate_identifier_newTarget_Compiler_uses_variable_declaration_newTarget_to_capture_new_target_me_2543", "Duplicate identifier '_newTarget'. Compiler uses variable declaration '_newTarget' to capture 'new.target' meta-property reference."),
            Expression_resolves_to_variable_declaration_newTarget_that_compiler_uses_to_capture_new_target_meta_property_reference: diag(2544, ts.DiagnosticCategory.Error, "Expression_resolves_to_variable_declaration_newTarget_that_compiler_uses_to_capture_new_target_meta__2544", "Expression resolves to variable declaration '_newTarget' that compiler uses to capture 'new.target' meta-property reference."),
            A_mixin_class_must_have_a_constructor_with_a_single_rest_parameter_of_type_any: diag(2545, ts.DiagnosticCategory.Error, "A_mixin_class_must_have_a_constructor_with_a_single_rest_parameter_of_type_any_2545", "A mixin class must have a constructor with a single rest parameter of type 'any[]'."),
            Property_0_has_conflicting_declarations_and_is_inaccessible_in_type_1: diag(2546, ts.DiagnosticCategory.Error, "Property_0_has_conflicting_declarations_and_is_inaccessible_in_type_1_2546", "Property '{0}' has conflicting declarations and is inaccessible in type '{1}'."),
            The_type_returned_by_the_0_method_of_an_async_iterator_must_be_a_promise_for_a_type_with_a_value_property: diag(2547, ts.DiagnosticCategory.Error, "The_type_returned_by_the_0_method_of_an_async_iterator_must_be_a_promise_for_a_type_with_a_value_pro_2547", "The type returned by the '{0}()' method of an async iterator must be a promise for a type with a 'value' property."),
            Type_0_is_not_an_array_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator: diag(2548, ts.DiagnosticCategory.Error, "Type_0_is_not_an_array_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator_2548", "Type '{0}' is not an array type or does not have a '[Symbol.iterator]()' method that returns an iterator."),
            Type_0_is_not_an_array_type_or_a_string_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator: diag(2549, ts.DiagnosticCategory.Error, "Type_0_is_not_an_array_type_or_a_string_type_or_does_not_have_a_Symbol_iterator_method_that_returns__2549", "Type '{0}' is not an array type or a string type or does not have a '[Symbol.iterator]()' method that returns an iterator."),
            Property_0_does_not_exist_on_type_1_Did_you_mean_2: diag(2551, ts.DiagnosticCategory.Error, "Property_0_does_not_exist_on_type_1_Did_you_mean_2_2551", "Property '{0}' does not exist on type '{1}'. Did you mean '{2}'?"),
            Cannot_find_name_0_Did_you_mean_1: diag(2552, ts.DiagnosticCategory.Error, "Cannot_find_name_0_Did_you_mean_1_2552", "Cannot find name '{0}'. Did you mean '{1}'?"),
            Computed_values_are_not_permitted_in_an_enum_with_string_valued_members: diag(2553, ts.DiagnosticCategory.Error, "Computed_values_are_not_permitted_in_an_enum_with_string_valued_members_2553", "Computed values are not permitted in an enum with string valued members."),
            Expected_0_arguments_but_got_1: diag(2554, ts.DiagnosticCategory.Error, "Expected_0_arguments_but_got_1_2554", "Expected {0} arguments, but got {1}."),
            Expected_at_least_0_arguments_but_got_1: diag(2555, ts.DiagnosticCategory.Error, "Expected_at_least_0_arguments_but_got_1_2555", "Expected at least {0} arguments, but got {1}."),
            Expected_0_arguments_but_got_1_or_more: diag(2556, ts.DiagnosticCategory.Error, "Expected_0_arguments_but_got_1_or_more_2556", "Expected {0} arguments, but got {1} or more."),
            Expected_at_least_0_arguments_but_got_1_or_more: diag(2557, ts.DiagnosticCategory.Error, "Expected_at_least_0_arguments_but_got_1_or_more_2557", "Expected at least {0} arguments, but got {1} or more."),
            Expected_0_type_arguments_but_got_1: diag(2558, ts.DiagnosticCategory.Error, "Expected_0_type_arguments_but_got_1_2558", "Expected {0} type arguments, but got {1}."),
            Type_0_has_no_properties_in_common_with_type_1: diag(2559, ts.DiagnosticCategory.Error, "Type_0_has_no_properties_in_common_with_type_1_2559", "Type '{0}' has no properties in common with type '{1}'."),
            Value_of_type_0_has_no_properties_in_common_with_type_1_Did_you_mean_to_call_it: diag(2560, ts.DiagnosticCategory.Error, "Value_of_type_0_has_no_properties_in_common_with_type_1_Did_you_mean_to_call_it_2560", "Value of type '{0}' has no properties in common with type '{1}'. Did you mean to call it?"),
            Object_literal_may_only_specify_known_properties_but_0_does_not_exist_in_type_1_Did_you_mean_to_write_2: diag(2561, ts.DiagnosticCategory.Error, "Object_literal_may_only_specify_known_properties_but_0_does_not_exist_in_type_1_Did_you_mean_to_writ_2561", "Object literal may only specify known properties, but '{0}' does not exist in type '{1}'. Did you mean to write '{2}'?"),
            Base_class_expressions_cannot_reference_class_type_parameters: diag(2562, ts.DiagnosticCategory.Error, "Base_class_expressions_cannot_reference_class_type_parameters_2562", "Base class expressions cannot reference class type parameters."),
            The_containing_function_or_module_body_is_too_large_for_control_flow_analysis: diag(2563, ts.DiagnosticCategory.Error, "The_containing_function_or_module_body_is_too_large_for_control_flow_analysis_2563", "The containing function or module body is too large for control flow analysis."),
            Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor: diag(2564, ts.DiagnosticCategory.Error, "Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor_2564", "Property '{0}' has no initializer and is not definitely assigned in the constructor."),
            Property_0_is_used_before_being_assigned: diag(2565, ts.DiagnosticCategory.Error, "Property_0_is_used_before_being_assigned_2565", "Property '{0}' is used before being assigned."),
            A_rest_element_cannot_have_a_property_name: diag(2566, ts.DiagnosticCategory.Error, "A_rest_element_cannot_have_a_property_name_2566", "A rest element cannot have a property name."),
            Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations: diag(2567, ts.DiagnosticCategory.Error, "Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations_2567", "Enum declarations can only merge with namespace or other enum declarations."),
            Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators: diag(2569, ts.DiagnosticCategory.Error, "Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterati_2569", "Type '{0}' is not an array type or a string type. Use compiler option '--downlevelIteration' to allow iterating of iterators."),
            Object_is_of_type_unknown: diag(2571, ts.DiagnosticCategory.Error, "Object_is_of_type_unknown_2571", "Object is of type 'unknown'."),
            Rest_signatures_are_incompatible: diag(2572, ts.DiagnosticCategory.Error, "Rest_signatures_are_incompatible_2572", "Rest signatures are incompatible."),
            Property_0_is_incompatible_with_rest_element_type: diag(2573, ts.DiagnosticCategory.Error, "Property_0_is_incompatible_with_rest_element_type_2573", "Property '{0}' is incompatible with rest element type."),
            A_rest_element_type_must_be_an_array_type: diag(2574, ts.DiagnosticCategory.Error, "A_rest_element_type_must_be_an_array_type_2574", "A rest element type must be an array type."),
            No_overload_expects_0_arguments_but_overloads_do_exist_that_expect_either_1_or_2_arguments: diag(2575, ts.DiagnosticCategory.Error, "No_overload_expects_0_arguments_but_overloads_do_exist_that_expect_either_1_or_2_arguments_2575", "No overload expects {0} arguments, but overloads do exist that expect either {1} or {2} arguments."),
            Property_0_is_a_static_member_of_type_1: diag(2576, ts.DiagnosticCategory.Error, "Property_0_is_a_static_member_of_type_1_2576", "Property '{0}' is a static member of type '{1}'"),
            Return_type_annotation_circularly_references_itself: diag(2577, ts.DiagnosticCategory.Error, "Return_type_annotation_circularly_references_itself_2577", "Return type annotation circularly references itself."),
            Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_types_Slashnode: diag(2580, ts.DiagnosticCategory.Error, "Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_types_Slashnode_2580", "Cannot find name '{0}'. Do you need to install type definitions for node? Try `npm i @types/node`."),
            Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_types_Slashjquery: diag(2581, ts.DiagnosticCategory.Error, "Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_types_Slashjquery_2581", "Cannot find name '{0}'. Do you need to install type definitions for jQuery? Try `npm i @types/jquery`."),
            Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_types_Slashjest_or_npm_i_types_Slashmocha: diag(2582, ts.DiagnosticCategory.Error, "Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_types_Slashje_2582", "Cannot find name '{0}'. Do you need to install type definitions for a test runner? Try `npm i @types/jest` or `npm i @types/mocha`."),
            Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_es2015_or_later: diag(2583, ts.DiagnosticCategory.Error, "Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_2583", "Cannot find name '{0}'. Do you need to change your target library? Try changing the `lib` compiler option to es2015 or later."),
            Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_include_dom: diag(2584, ts.DiagnosticCategory.Error, "Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_2584", "Cannot find name '{0}'. Do you need to change your target library? Try changing the `lib` compiler option to include 'dom'."),
            _0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_es2015_or_later: diag(2585, ts.DiagnosticCategory.Error, "_0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Do_you_need_to_change_your_target_library_2585", "'{0}' only refers to a type, but is being used as a value here. Do you need to change your target library? Try changing the `lib` compiler option to es2015 or later."),
            Enum_type_0_circularly_references_itself: diag(2586, ts.DiagnosticCategory.Error, "Enum_type_0_circularly_references_itself_2586", "Enum type '{0}' circularly references itself."),
            JSDoc_type_0_circularly_references_itself: diag(2587, ts.DiagnosticCategory.Error, "JSDoc_type_0_circularly_references_itself_2587", "JSDoc type '{0}' circularly references itself."),
            Cannot_assign_to_0_because_it_is_a_constant: diag(2588, ts.DiagnosticCategory.Error, "Cannot_assign_to_0_because_it_is_a_constant_2588", "Cannot assign to '{0}' because it is a constant."),
            Type_instantiation_is_excessively_deep_and_possibly_infinite: diag(2589, ts.DiagnosticCategory.Error, "Type_instantiation_is_excessively_deep_and_possibly_infinite_2589", "Type instantiation is excessively deep and possibly infinite."),
            Expression_produces_a_union_type_that_is_too_complex_to_represent: diag(2590, ts.DiagnosticCategory.Error, "Expression_produces_a_union_type_that_is_too_complex_to_represent_2590", "Expression produces a union type that is too complex to represent."),
            Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_types_Slashnode_and_then_add_node_to_the_types_field_in_your_tsconfig: diag(2591, ts.DiagnosticCategory.Error, "Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_types_Slashnode_and_th_2591", "Cannot find name '{0}'. Do you need to install type definitions for node? Try `npm i @types/node` and then add `node` to the types field in your tsconfig."),
            Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_types_Slashjquery_and_then_add_jquery_to_the_types_field_in_your_tsconfig: diag(2592, ts.DiagnosticCategory.Error, "Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_types_Slashjquery_an_2592", "Cannot find name '{0}'. Do you need to install type definitions for jQuery? Try `npm i @types/jquery` and then add `jquery` to the types field in your tsconfig."),
            Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_types_Slashjest_or_npm_i_types_Slashmocha_and_then_add_jest_or_mocha_to_the_types_field_in_your_tsconfig: diag(2593, ts.DiagnosticCategory.Error, "Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_types_Slashje_2593", "Cannot find name '{0}'. Do you need to install type definitions for a test runner? Try `npm i @types/jest` or `npm i @types/mocha` and then add `jest` or `mocha` to the types field in your tsconfig."),
            This_module_is_declared_with_using_export_and_can_only_be_used_with_a_default_import_when_using_the_0_flag: diag(2594, ts.DiagnosticCategory.Error, "This_module_is_declared_with_using_export_and_can_only_be_used_with_a_default_import_when_using_the__2594", "This module is declared with using 'export =', and can only be used with a default import when using the '{0}' flag."),
            JSX_element_attributes_type_0_may_not_be_a_union_type: diag(2600, ts.DiagnosticCategory.Error, "JSX_element_attributes_type_0_may_not_be_a_union_type_2600", "JSX element attributes type '{0}' may not be a union type."),
            The_return_type_of_a_JSX_element_constructor_must_return_an_object_type: diag(2601, ts.DiagnosticCategory.Error, "The_return_type_of_a_JSX_element_constructor_must_return_an_object_type_2601", "The return type of a JSX element constructor must return an object type."),
            JSX_element_implicitly_has_type_any_because_the_global_type_JSX_Element_does_not_exist: diag(2602, ts.DiagnosticCategory.Error, "JSX_element_implicitly_has_type_any_because_the_global_type_JSX_Element_does_not_exist_2602", "JSX element implicitly has type 'any' because the global type 'JSX.Element' does not exist."),
            Property_0_in_type_1_is_not_assignable_to_type_2: diag(2603, ts.DiagnosticCategory.Error, "Property_0_in_type_1_is_not_assignable_to_type_2_2603", "Property '{0}' in type '{1}' is not assignable to type '{2}'."),
            JSX_element_type_0_does_not_have_any_construct_or_call_signatures: diag(2604, ts.DiagnosticCategory.Error, "JSX_element_type_0_does_not_have_any_construct_or_call_signatures_2604", "JSX element type '{0}' does not have any construct or call signatures."),
            JSX_element_type_0_is_not_a_constructor_function_for_JSX_elements: diag(2605, ts.DiagnosticCategory.Error, "JSX_element_type_0_is_not_a_constructor_function_for_JSX_elements_2605", "JSX element type '{0}' is not a constructor function for JSX elements."),
            Property_0_of_JSX_spread_attribute_is_not_assignable_to_target_property: diag(2606, ts.DiagnosticCategory.Error, "Property_0_of_JSX_spread_attribute_is_not_assignable_to_target_property_2606", "Property '{0}' of JSX spread attribute is not assignable to target property."),
            JSX_element_class_does_not_support_attributes_because_it_does_not_have_a_0_property: diag(2607, ts.DiagnosticCategory.Error, "JSX_element_class_does_not_support_attributes_because_it_does_not_have_a_0_property_2607", "JSX element class does not support attributes because it does not have a '{0}' property."),
            The_global_type_JSX_0_may_not_have_more_than_one_property: diag(2608, ts.DiagnosticCategory.Error, "The_global_type_JSX_0_may_not_have_more_than_one_property_2608", "The global type 'JSX.{0}' may not have more than one property."),
            JSX_spread_child_must_be_an_array_type: diag(2609, ts.DiagnosticCategory.Error, "JSX_spread_child_must_be_an_array_type_2609", "JSX spread child must be an array type."),
            _0_is_defined_as_an_accessor_in_class_1_but_is_overridden_here_in_2_as_an_instance_property: diag(2610, ts.DiagnosticCategory.Error, "_0_is_defined_as_an_accessor_in_class_1_but_is_overridden_here_in_2_as_an_instance_property_2610", "'{0}' is defined as an accessor in class '{1}', but is overridden here in '{2}' as an instance property."),
            _0_is_defined_as_a_property_in_class_1_but_is_overridden_here_in_2_as_an_accessor: diag(2611, ts.DiagnosticCategory.Error, "_0_is_defined_as_a_property_in_class_1_but_is_overridden_here_in_2_as_an_accessor_2611", "'{0}' is defined as a property in class '{1}', but is overridden here in '{2}' as an accessor."),
            Property_0_will_overwrite_the_base_property_in_1_If_this_is_intentional_add_an_initializer_Otherwise_add_a_declare_modifier_or_remove_the_redundant_declaration: diag(2612, ts.DiagnosticCategory.Error, "Property_0_will_overwrite_the_base_property_in_1_If_this_is_intentional_add_an_initializer_Otherwise_2612", "Property '{0}' will overwrite the base property in '{1}'. If this is intentional, add an initializer. Otherwise, add a 'declare' modifier or remove the redundant declaration."),
            Module_0_has_no_default_export_Did_you_mean_to_use_import_1_from_0_instead: diag(2613, ts.DiagnosticCategory.Error, "Module_0_has_no_default_export_Did_you_mean_to_use_import_1_from_0_instead_2613", "Module '{0}' has no default export. Did you mean to use 'import { {1} } from {0}' instead?"),
            Module_0_has_no_exported_member_1_Did_you_mean_to_use_import_1_from_0_instead: diag(2614, ts.DiagnosticCategory.Error, "Module_0_has_no_exported_member_1_Did_you_mean_to_use_import_1_from_0_instead_2614", "Module '{0}' has no exported member '{1}'. Did you mean to use 'import {1} from {0}' instead?"),
            Cannot_augment_module_0_with_value_exports_because_it_resolves_to_a_non_module_entity: diag(2649, ts.DiagnosticCategory.Error, "Cannot_augment_module_0_with_value_exports_because_it_resolves_to_a_non_module_entity_2649", "Cannot augment module '{0}' with value exports because it resolves to a non-module entity."),
            A_member_initializer_in_a_enum_declaration_cannot_reference_members_declared_after_it_including_members_defined_in_other_enums: diag(2651, ts.DiagnosticCategory.Error, "A_member_initializer_in_a_enum_declaration_cannot_reference_members_declared_after_it_including_memb_2651", "A member initializer in a enum declaration cannot reference members declared after it, including members defined in other enums."),
            Merged_declaration_0_cannot_include_a_default_export_declaration_Consider_adding_a_separate_export_default_0_declaration_instead: diag(2652, ts.DiagnosticCategory.Error, "Merged_declaration_0_cannot_include_a_default_export_declaration_Consider_adding_a_separate_export_d_2652", "Merged declaration '{0}' cannot include a default export declaration. Consider adding a separate 'export default {0}' declaration instead."),
            Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1: diag(2653, ts.DiagnosticCategory.Error, "Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1_2653", "Non-abstract class expression does not implement inherited abstract member '{0}' from class '{1}'."),
            Exported_external_package_typings_file_cannot_contain_tripleslash_references_Please_contact_the_package_author_to_update_the_package_definition: diag(2654, ts.DiagnosticCategory.Error, "Exported_external_package_typings_file_cannot_contain_tripleslash_references_Please_contact_the_pack_2654", "Exported external package typings file cannot contain tripleslash references. Please contact the package author to update the package definition."),
            Exported_external_package_typings_file_0_is_not_a_module_Please_contact_the_package_author_to_update_the_package_definition: diag(2656, ts.DiagnosticCategory.Error, "Exported_external_package_typings_file_0_is_not_a_module_Please_contact_the_package_author_to_update_2656", "Exported external package typings file '{0}' is not a module. Please contact the package author to update the package definition."),
            JSX_expressions_must_have_one_parent_element: diag(2657, ts.DiagnosticCategory.Error, "JSX_expressions_must_have_one_parent_element_2657", "JSX expressions must have one parent element."),
            Type_0_provides_no_match_for_the_signature_1: diag(2658, ts.DiagnosticCategory.Error, "Type_0_provides_no_match_for_the_signature_1_2658", "Type '{0}' provides no match for the signature '{1}'."),
            super_is_only_allowed_in_members_of_object_literal_expressions_when_option_target_is_ES2015_or_higher: diag(2659, ts.DiagnosticCategory.Error, "super_is_only_allowed_in_members_of_object_literal_expressions_when_option_target_is_ES2015_or_highe_2659", "'super' is only allowed in members of object literal expressions when option 'target' is 'ES2015' or higher."),
            super_can_only_be_referenced_in_members_of_derived_classes_or_object_literal_expressions: diag(2660, ts.DiagnosticCategory.Error, "super_can_only_be_referenced_in_members_of_derived_classes_or_object_literal_expressions_2660", "'super' can only be referenced in members of derived classes or object literal expressions."),
            Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module: diag(2661, ts.DiagnosticCategory.Error, "Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module_2661", "Cannot export '{0}'. Only local declarations can be exported from a module."),
            Cannot_find_name_0_Did_you_mean_the_static_member_1_0: diag(2662, ts.DiagnosticCategory.Error, "Cannot_find_name_0_Did_you_mean_the_static_member_1_0_2662", "Cannot find name '{0}'. Did you mean the static member '{1}.{0}'?"),
            Cannot_find_name_0_Did_you_mean_the_instance_member_this_0: diag(2663, ts.DiagnosticCategory.Error, "Cannot_find_name_0_Did_you_mean_the_instance_member_this_0_2663", "Cannot find name '{0}'. Did you mean the instance member 'this.{0}'?"),
            Invalid_module_name_in_augmentation_module_0_cannot_be_found: diag(2664, ts.DiagnosticCategory.Error, "Invalid_module_name_in_augmentation_module_0_cannot_be_found_2664", "Invalid module name in augmentation, module '{0}' cannot be found."),
            Invalid_module_name_in_augmentation_Module_0_resolves_to_an_untyped_module_at_1_which_cannot_be_augmented: diag(2665, ts.DiagnosticCategory.Error, "Invalid_module_name_in_augmentation_Module_0_resolves_to_an_untyped_module_at_1_which_cannot_be_augm_2665", "Invalid module name in augmentation. Module '{0}' resolves to an untyped module at '{1}', which cannot be augmented."),
            Exports_and_export_assignments_are_not_permitted_in_module_augmentations: diag(2666, ts.DiagnosticCategory.Error, "Exports_and_export_assignments_are_not_permitted_in_module_augmentations_2666", "Exports and export assignments are not permitted in module augmentations."),
            Imports_are_not_permitted_in_module_augmentations_Consider_moving_them_to_the_enclosing_external_module: diag(2667, ts.DiagnosticCategory.Error, "Imports_are_not_permitted_in_module_augmentations_Consider_moving_them_to_the_enclosing_external_mod_2667", "Imports are not permitted in module augmentations. Consider moving them to the enclosing external module."),
            export_modifier_cannot_be_applied_to_ambient_modules_and_module_augmentations_since_they_are_always_visible: diag(2668, ts.DiagnosticCategory.Error, "export_modifier_cannot_be_applied_to_ambient_modules_and_module_augmentations_since_they_are_always__2668", "'export' modifier cannot be applied to ambient modules and module augmentations since they are always visible."),
            Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_declarations: diag(2669, ts.DiagnosticCategory.Error, "Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_2669", "Augmentations for the global scope can only be directly nested in external modules or ambient module declarations."),
            Augmentations_for_the_global_scope_should_have_declare_modifier_unless_they_appear_in_already_ambient_context: diag(2670, ts.DiagnosticCategory.Error, "Augmentations_for_the_global_scope_should_have_declare_modifier_unless_they_appear_in_already_ambien_2670", "Augmentations for the global scope should have 'declare' modifier unless they appear in already ambient context."),
            Cannot_augment_module_0_because_it_resolves_to_a_non_module_entity: diag(2671, ts.DiagnosticCategory.Error, "Cannot_augment_module_0_because_it_resolves_to_a_non_module_entity_2671", "Cannot augment module '{0}' because it resolves to a non-module entity."),
            Cannot_assign_a_0_constructor_type_to_a_1_constructor_type: diag(2672, ts.DiagnosticCategory.Error, "Cannot_assign_a_0_constructor_type_to_a_1_constructor_type_2672", "Cannot assign a '{0}' constructor type to a '{1}' constructor type."),
            Constructor_of_class_0_is_private_and_only_accessible_within_the_class_declaration: diag(2673, ts.DiagnosticCategory.Error, "Constructor_of_class_0_is_private_and_only_accessible_within_the_class_declaration_2673", "Constructor of class '{0}' is private and only accessible within the class declaration."),
            Constructor_of_class_0_is_protected_and_only_accessible_within_the_class_declaration: diag(2674, ts.DiagnosticCategory.Error, "Constructor_of_class_0_is_protected_and_only_accessible_within_the_class_declaration_2674", "Constructor of class '{0}' is protected and only accessible within the class declaration."),
            Cannot_extend_a_class_0_Class_constructor_is_marked_as_private: diag(2675, ts.DiagnosticCategory.Error, "Cannot_extend_a_class_0_Class_constructor_is_marked_as_private_2675", "Cannot extend a class '{0}'. Class constructor is marked as private."),
            Accessors_must_both_be_abstract_or_non_abstract: diag(2676, ts.DiagnosticCategory.Error, "Accessors_must_both_be_abstract_or_non_abstract_2676", "Accessors must both be abstract or non-abstract."),
            A_type_predicate_s_type_must_be_assignable_to_its_parameter_s_type: diag(2677, ts.DiagnosticCategory.Error, "A_type_predicate_s_type_must_be_assignable_to_its_parameter_s_type_2677", "A type predicate's type must be assignable to its parameter's type."),
            Type_0_is_not_comparable_to_type_1: diag(2678, ts.DiagnosticCategory.Error, "Type_0_is_not_comparable_to_type_1_2678", "Type '{0}' is not comparable to type '{1}'."),
            A_function_that_is_called_with_the_new_keyword_cannot_have_a_this_type_that_is_void: diag(2679, ts.DiagnosticCategory.Error, "A_function_that_is_called_with_the_new_keyword_cannot_have_a_this_type_that_is_void_2679", "A function that is called with the 'new' keyword cannot have a 'this' type that is 'void'."),
            A_0_parameter_must_be_the_first_parameter: diag(2680, ts.DiagnosticCategory.Error, "A_0_parameter_must_be_the_first_parameter_2680", "A '{0}' parameter must be the first parameter."),
            A_constructor_cannot_have_a_this_parameter: diag(2681, ts.DiagnosticCategory.Error, "A_constructor_cannot_have_a_this_parameter_2681", "A constructor cannot have a 'this' parameter."),
            get_and_set_accessor_must_have_the_same_this_type: diag(2682, ts.DiagnosticCategory.Error, "get_and_set_accessor_must_have_the_same_this_type_2682", "'get' and 'set' accessor must have the same 'this' type."),
            this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation: diag(2683, ts.DiagnosticCategory.Error, "this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_2683", "'this' implicitly has type 'any' because it does not have a type annotation."),
            The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1: diag(2684, ts.DiagnosticCategory.Error, "The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1_2684", "The 'this' context of type '{0}' is not assignable to method's 'this' of type '{1}'."),
            The_this_types_of_each_signature_are_incompatible: diag(2685, ts.DiagnosticCategory.Error, "The_this_types_of_each_signature_are_incompatible_2685", "The 'this' types of each signature are incompatible."),
            _0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead: diag(2686, ts.DiagnosticCategory.Error, "_0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead_2686", "'{0}' refers to a UMD global, but the current file is a module. Consider adding an import instead."),
            All_declarations_of_0_must_have_identical_modifiers: diag(2687, ts.DiagnosticCategory.Error, "All_declarations_of_0_must_have_identical_modifiers_2687", "All declarations of '{0}' must have identical modifiers."),
            Cannot_find_type_definition_file_for_0: diag(2688, ts.DiagnosticCategory.Error, "Cannot_find_type_definition_file_for_0_2688", "Cannot find type definition file for '{0}'."),
            Cannot_extend_an_interface_0_Did_you_mean_implements: diag(2689, ts.DiagnosticCategory.Error, "Cannot_extend_an_interface_0_Did_you_mean_implements_2689", "Cannot extend an interface '{0}'. Did you mean 'implements'?"),
            An_import_path_cannot_end_with_a_0_extension_Consider_importing_1_instead: diag(2691, ts.DiagnosticCategory.Error, "An_import_path_cannot_end_with_a_0_extension_Consider_importing_1_instead_2691", "An import path cannot end with a '{0}' extension. Consider importing '{1}' instead."),
            _0_is_a_primitive_but_1_is_a_wrapper_object_Prefer_using_0_when_possible: diag(2692, ts.DiagnosticCategory.Error, "_0_is_a_primitive_but_1_is_a_wrapper_object_Prefer_using_0_when_possible_2692", "'{0}' is a primitive, but '{1}' is a wrapper object. Prefer using '{0}' when possible."),
            _0_only_refers_to_a_type_but_is_being_used_as_a_value_here: diag(2693, ts.DiagnosticCategory.Error, "_0_only_refers_to_a_type_but_is_being_used_as_a_value_here_2693", "'{0}' only refers to a type, but is being used as a value here."),
            Namespace_0_has_no_exported_member_1: diag(2694, ts.DiagnosticCategory.Error, "Namespace_0_has_no_exported_member_1_2694", "Namespace '{0}' has no exported member '{1}'."),
            Left_side_of_comma_operator_is_unused_and_has_no_side_effects: diag(2695, ts.DiagnosticCategory.Error, "Left_side_of_comma_operator_is_unused_and_has_no_side_effects_2695", "Left side of comma operator is unused and has no side effects.", /*reportsUnnecessary*/ true),
            The_Object_type_is_assignable_to_very_few_other_types_Did_you_mean_to_use_the_any_type_instead: diag(2696, ts.DiagnosticCategory.Error, "The_Object_type_is_assignable_to_very_few_other_types_Did_you_mean_to_use_the_any_type_instead_2696", "The 'Object' type is assignable to very few other types. Did you mean to use the 'any' type instead?"),
            An_async_function_or_method_must_return_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option: diag(2697, ts.DiagnosticCategory.Error, "An_async_function_or_method_must_return_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_in_2697", "An async function or method must return a 'Promise'. Make sure you have a declaration for 'Promise' or include 'ES2015' in your `--lib` option."),
            Spread_types_may_only_be_created_from_object_types: diag(2698, ts.DiagnosticCategory.Error, "Spread_types_may_only_be_created_from_object_types_2698", "Spread types may only be created from object types."),
            Static_property_0_conflicts_with_built_in_property_Function_0_of_constructor_function_1: diag(2699, ts.DiagnosticCategory.Error, "Static_property_0_conflicts_with_built_in_property_Function_0_of_constructor_function_1_2699", "Static property '{0}' conflicts with built-in property 'Function.{0}' of constructor function '{1}'."),
            Rest_types_may_only_be_created_from_object_types: diag(2700, ts.DiagnosticCategory.Error, "Rest_types_may_only_be_created_from_object_types_2700", "Rest types may only be created from object types."),
            The_target_of_an_object_rest_assignment_must_be_a_variable_or_a_property_access: diag(2701, ts.DiagnosticCategory.Error, "The_target_of_an_object_rest_assignment_must_be_a_variable_or_a_property_access_2701", "The target of an object rest assignment must be a variable or a property access."),
            _0_only_refers_to_a_type_but_is_being_used_as_a_namespace_here: diag(2702, ts.DiagnosticCategory.Error, "_0_only_refers_to_a_type_but_is_being_used_as_a_namespace_here_2702", "'{0}' only refers to a type, but is being used as a namespace here."),
            The_operand_of_a_delete_operator_must_be_a_property_reference: diag(2703, ts.DiagnosticCategory.Error, "The_operand_of_a_delete_operator_must_be_a_property_reference_2703", "The operand of a 'delete' operator must be a property reference."),
            The_operand_of_a_delete_operator_cannot_be_a_read_only_property: diag(2704, ts.DiagnosticCategory.Error, "The_operand_of_a_delete_operator_cannot_be_a_read_only_property_2704", "The operand of a 'delete' operator cannot be a read-only property."),
            An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option: diag(2705, ts.DiagnosticCategory.Error, "An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_de_2705", "An async function or method in ES5/ES3 requires the 'Promise' constructor.  Make sure you have a declaration for the 'Promise' constructor or include 'ES2015' in your `--lib` option."),
            Required_type_parameters_may_not_follow_optional_type_parameters: diag(2706, ts.DiagnosticCategory.Error, "Required_type_parameters_may_not_follow_optional_type_parameters_2706", "Required type parameters may not follow optional type parameters."),
            Generic_type_0_requires_between_1_and_2_type_arguments: diag(2707, ts.DiagnosticCategory.Error, "Generic_type_0_requires_between_1_and_2_type_arguments_2707", "Generic type '{0}' requires between {1} and {2} type arguments."),
            Cannot_use_namespace_0_as_a_value: diag(2708, ts.DiagnosticCategory.Error, "Cannot_use_namespace_0_as_a_value_2708", "Cannot use namespace '{0}' as a value."),
            Cannot_use_namespace_0_as_a_type: diag(2709, ts.DiagnosticCategory.Error, "Cannot_use_namespace_0_as_a_type_2709", "Cannot use namespace '{0}' as a type."),
            _0_are_specified_twice_The_attribute_named_0_will_be_overwritten: diag(2710, ts.DiagnosticCategory.Error, "_0_are_specified_twice_The_attribute_named_0_will_be_overwritten_2710", "'{0}' are specified twice. The attribute named '{0}' will be overwritten."),
            A_dynamic_import_call_returns_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option: diag(2711, ts.DiagnosticCategory.Error, "A_dynamic_import_call_returns_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES20_2711", "A dynamic import call returns a 'Promise'. Make sure you have a declaration for 'Promise' or include 'ES2015' in your `--lib` option."),
            A_dynamic_import_call_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option: diag(2712, ts.DiagnosticCategory.Error, "A_dynamic_import_call_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declarat_2712", "A dynamic import call in ES5/ES3 requires the 'Promise' constructor.  Make sure you have a declaration for the 'Promise' constructor or include 'ES2015' in your `--lib` option."),
            Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_property_1_in_0_with_0_1: diag(2713, ts.DiagnosticCategory.Error, "Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_p_2713", "Cannot access '{0}.{1}' because '{0}' is a type, but not a namespace. Did you mean to retrieve the type of the property '{1}' in '{0}' with '{0}[\"{1}\"]'?"),
            The_expression_of_an_export_assignment_must_be_an_identifier_or_qualified_name_in_an_ambient_context: diag(2714, ts.DiagnosticCategory.Error, "The_expression_of_an_export_assignment_must_be_an_identifier_or_qualified_name_in_an_ambient_context_2714", "The expression of an export assignment must be an identifier or qualified name in an ambient context."),
            Abstract_property_0_in_class_1_cannot_be_accessed_in_the_constructor: diag(2715, ts.DiagnosticCategory.Error, "Abstract_property_0_in_class_1_cannot_be_accessed_in_the_constructor_2715", "Abstract property '{0}' in class '{1}' cannot be accessed in the constructor."),
            Type_parameter_0_has_a_circular_default: diag(2716, ts.DiagnosticCategory.Error, "Type_parameter_0_has_a_circular_default_2716", "Type parameter '{0}' has a circular default."),
            Subsequent_property_declarations_must_have_the_same_type_Property_0_must_be_of_type_1_but_here_has_type_2: diag(2717, ts.DiagnosticCategory.Error, "Subsequent_property_declarations_must_have_the_same_type_Property_0_must_be_of_type_1_but_here_has_t_2717", "Subsequent property declarations must have the same type.  Property '{0}' must be of type '{1}', but here has type '{2}'."),
            Duplicate_property_0: diag(2718, ts.DiagnosticCategory.Error, "Duplicate_property_0_2718", "Duplicate property '{0}'."),
            Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated: diag(2719, ts.DiagnosticCategory.Error, "Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated_2719", "Type '{0}' is not assignable to type '{1}'. Two different types with this name exist, but they are unrelated."),
            Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclass: diag(2720, ts.DiagnosticCategory.Error, "Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclas_2720", "Class '{0}' incorrectly implements class '{1}'. Did you mean to extend '{1}' and inherit its members as a subclass?"),
            Cannot_invoke_an_object_which_is_possibly_null: diag(2721, ts.DiagnosticCategory.Error, "Cannot_invoke_an_object_which_is_possibly_null_2721", "Cannot invoke an object which is possibly 'null'."),
            Cannot_invoke_an_object_which_is_possibly_undefined: diag(2722, ts.DiagnosticCategory.Error, "Cannot_invoke_an_object_which_is_possibly_undefined_2722", "Cannot invoke an object which is possibly 'undefined'."),
            Cannot_invoke_an_object_which_is_possibly_null_or_undefined: diag(2723, ts.DiagnosticCategory.Error, "Cannot_invoke_an_object_which_is_possibly_null_or_undefined_2723", "Cannot invoke an object which is possibly 'null' or 'undefined'."),
            Module_0_has_no_exported_member_1_Did_you_mean_2: diag(2724, ts.DiagnosticCategory.Error, "Module_0_has_no_exported_member_1_Did_you_mean_2_2724", "Module '{0}' has no exported member '{1}'. Did you mean '{2}'?"),
            Class_name_cannot_be_Object_when_targeting_ES5_with_module_0: diag(2725, ts.DiagnosticCategory.Error, "Class_name_cannot_be_Object_when_targeting_ES5_with_module_0_2725", "Class name cannot be 'Object' when targeting ES5 with module {0}."),
            Cannot_find_lib_definition_for_0: diag(2726, ts.DiagnosticCategory.Error, "Cannot_find_lib_definition_for_0_2726", "Cannot find lib definition for '{0}'."),
            Cannot_find_lib_definition_for_0_Did_you_mean_1: diag(2727, ts.DiagnosticCategory.Error, "Cannot_find_lib_definition_for_0_Did_you_mean_1_2727", "Cannot find lib definition for '{0}'. Did you mean '{1}'?"),
            _0_is_declared_here: diag(2728, ts.DiagnosticCategory.Message, "_0_is_declared_here_2728", "'{0}' is declared here."),
            Property_0_is_used_before_its_initialization: diag(2729, ts.DiagnosticCategory.Error, "Property_0_is_used_before_its_initialization_2729", "Property '{0}' is used before its initialization."),
            An_arrow_function_cannot_have_a_this_parameter: diag(2730, ts.DiagnosticCategory.Error, "An_arrow_function_cannot_have_a_this_parameter_2730", "An arrow function cannot have a 'this' parameter."),
            Implicit_conversion_of_a_symbol_to_a_string_will_fail_at_runtime_Consider_wrapping_this_expression_in_String: diag(2731, ts.DiagnosticCategory.Error, "Implicit_conversion_of_a_symbol_to_a_string_will_fail_at_runtime_Consider_wrapping_this_expression_i_2731", "Implicit conversion of a 'symbol' to a 'string' will fail at runtime. Consider wrapping this expression in 'String(...)'."),
            Cannot_find_module_0_Consider_using_resolveJsonModule_to_import_module_with_json_extension: diag(2732, ts.DiagnosticCategory.Error, "Cannot_find_module_0_Consider_using_resolveJsonModule_to_import_module_with_json_extension_2732", "Cannot find module '{0}'. Consider using '--resolveJsonModule' to import module with '.json' extension"),
            Property_0_was_also_declared_here: diag(2733, ts.DiagnosticCategory.Error, "Property_0_was_also_declared_here_2733", "Property '{0}' was also declared here."),
            It_is_highly_likely_that_you_are_missing_a_semicolon: diag(2734, ts.DiagnosticCategory.Error, "It_is_highly_likely_that_you_are_missing_a_semicolon_2734", "It is highly likely that you are missing a semicolon."),
            Did_you_mean_for_0_to_be_constrained_to_type_new_args_Colon_any_1: diag(2735, ts.DiagnosticCategory.Error, "Did_you_mean_for_0_to_be_constrained_to_type_new_args_Colon_any_1_2735", "Did you mean for '{0}' to be constrained to type 'new (...args: any[]) => {1}'?"),
            Operator_0_cannot_be_applied_to_type_1: diag(2736, ts.DiagnosticCategory.Error, "Operator_0_cannot_be_applied_to_type_1_2736", "Operator '{0}' cannot be applied to type '{1}'."),
            BigInt_literals_are_not_available_when_targeting_lower_than_ES2020: diag(2737, ts.DiagnosticCategory.Error, "BigInt_literals_are_not_available_when_targeting_lower_than_ES2020_2737", "BigInt literals are not available when targeting lower than ES2020."),
            An_outer_value_of_this_is_shadowed_by_this_container: diag(2738, ts.DiagnosticCategory.Message, "An_outer_value_of_this_is_shadowed_by_this_container_2738", "An outer value of 'this' is shadowed by this container."),
            Type_0_is_missing_the_following_properties_from_type_1_Colon_2: diag(2739, ts.DiagnosticCategory.Error, "Type_0_is_missing_the_following_properties_from_type_1_Colon_2_2739", "Type '{0}' is missing the following properties from type '{1}': {2}"),
            Type_0_is_missing_the_following_properties_from_type_1_Colon_2_and_3_more: diag(2740, ts.DiagnosticCategory.Error, "Type_0_is_missing_the_following_properties_from_type_1_Colon_2_and_3_more_2740", "Type '{0}' is missing the following properties from type '{1}': {2}, and {3} more."),
            Property_0_is_missing_in_type_1_but_required_in_type_2: diag(2741, ts.DiagnosticCategory.Error, "Property_0_is_missing_in_type_1_but_required_in_type_2_2741", "Property '{0}' is missing in type '{1}' but required in type '{2}'."),
            The_inferred_type_of_0_cannot_be_named_without_a_reference_to_1_This_is_likely_not_portable_A_type_annotation_is_necessary: diag(2742, ts.DiagnosticCategory.Error, "The_inferred_type_of_0_cannot_be_named_without_a_reference_to_1_This_is_likely_not_portable_A_type_a_2742", "The inferred type of '{0}' cannot be named without a reference to '{1}'. This is likely not portable. A type annotation is necessary."),
            No_overload_expects_0_type_arguments_but_overloads_do_exist_that_expect_either_1_or_2_type_arguments: diag(2743, ts.DiagnosticCategory.Error, "No_overload_expects_0_type_arguments_but_overloads_do_exist_that_expect_either_1_or_2_type_arguments_2743", "No overload expects {0} type arguments, but overloads do exist that expect either {1} or {2} type arguments."),
            Type_parameter_defaults_can_only_reference_previously_declared_type_parameters: diag(2744, ts.DiagnosticCategory.Error, "Type_parameter_defaults_can_only_reference_previously_declared_type_parameters_2744", "Type parameter defaults can only reference previously declared type parameters."),
            This_JSX_tag_s_0_prop_expects_type_1_which_requires_multiple_children_but_only_a_single_child_was_provided: diag(2745, ts.DiagnosticCategory.Error, "This_JSX_tag_s_0_prop_expects_type_1_which_requires_multiple_children_but_only_a_single_child_was_pr_2745", "This JSX tag's '{0}' prop expects type '{1}' which requires multiple children, but only a single child was provided."),
            This_JSX_tag_s_0_prop_expects_a_single_child_of_type_1_but_multiple_children_were_provided: diag(2746, ts.DiagnosticCategory.Error, "This_JSX_tag_s_0_prop_expects_a_single_child_of_type_1_but_multiple_children_were_provided_2746", "This JSX tag's '{0}' prop expects a single child of type '{1}', but multiple children were provided."),
            _0_components_don_t_accept_text_as_child_elements_Text_in_JSX_has_the_type_string_but_the_expected_type_of_1_is_2: diag(2747, ts.DiagnosticCategory.Error, "_0_components_don_t_accept_text_as_child_elements_Text_in_JSX_has_the_type_string_but_the_expected_t_2747", "'{0}' components don't accept text as child elements. Text in JSX has the type 'string', but the expected type of '{1}' is '{2}'."),
            Cannot_access_ambient_const_enums_when_the_isolatedModules_flag_is_provided: diag(2748, ts.DiagnosticCategory.Error, "Cannot_access_ambient_const_enums_when_the_isolatedModules_flag_is_provided_2748", "Cannot access ambient const enums when the '--isolatedModules' flag is provided."),
            _0_refers_to_a_value_but_is_being_used_as_a_type_here: diag(2749, ts.DiagnosticCategory.Error, "_0_refers_to_a_value_but_is_being_used_as_a_type_here_2749", "'{0}' refers to a value, but is being used as a type here."),
            The_implementation_signature_is_declared_here: diag(2750, ts.DiagnosticCategory.Error, "The_implementation_signature_is_declared_here_2750", "The implementation signature is declared here."),
            Circularity_originates_in_type_at_this_location: diag(2751, ts.DiagnosticCategory.Error, "Circularity_originates_in_type_at_this_location_2751", "Circularity originates in type at this location."),
            The_first_export_default_is_here: diag(2752, ts.DiagnosticCategory.Error, "The_first_export_default_is_here_2752", "The first export default is here."),
            Another_export_default_is_here: diag(2753, ts.DiagnosticCategory.Error, "Another_export_default_is_here_2753", "Another export default is here."),
            super_may_not_use_type_arguments: diag(2754, ts.DiagnosticCategory.Error, "super_may_not_use_type_arguments_2754", "'super' may not use type arguments."),
            No_constituent_of_type_0_is_callable: diag(2755, ts.DiagnosticCategory.Error, "No_constituent_of_type_0_is_callable_2755", "No constituent of type '{0}' is callable."),
            Not_all_constituents_of_type_0_are_callable: diag(2756, ts.DiagnosticCategory.Error, "Not_all_constituents_of_type_0_are_callable_2756", "Not all constituents of type '{0}' are callable."),
            Type_0_has_no_call_signatures: diag(2757, ts.DiagnosticCategory.Error, "Type_0_has_no_call_signatures_2757", "Type '{0}' has no call signatures."),
            Each_member_of_the_union_type_0_has_signatures_but_none_of_those_signatures_are_compatible_with_each_other: diag(2758, ts.DiagnosticCategory.Error, "Each_member_of_the_union_type_0_has_signatures_but_none_of_those_signatures_are_compatible_with_each_2758", "Each member of the union type '{0}' has signatures, but none of those signatures are compatible with each other."),
            No_constituent_of_type_0_is_constructable: diag(2759, ts.DiagnosticCategory.Error, "No_constituent_of_type_0_is_constructable_2759", "No constituent of type '{0}' is constructable."),
            Not_all_constituents_of_type_0_are_constructable: diag(2760, ts.DiagnosticCategory.Error, "Not_all_constituents_of_type_0_are_constructable_2760", "Not all constituents of type '{0}' are constructable."),
            Type_0_has_no_construct_signatures: diag(2761, ts.DiagnosticCategory.Error, "Type_0_has_no_construct_signatures_2761", "Type '{0}' has no construct signatures."),
            Each_member_of_the_union_type_0_has_construct_signatures_but_none_of_those_signatures_are_compatible_with_each_other: diag(2762, ts.DiagnosticCategory.Error, "Each_member_of_the_union_type_0_has_construct_signatures_but_none_of_those_signatures_are_compatible_2762", "Each member of the union type '{0}' has construct signatures, but none of those signatures are compatible with each other."),
            Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_for_of_will_always_send_0: diag(2763, ts.DiagnosticCategory.Error, "Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_for_of_will_always_s_2763", "Cannot iterate value because the 'next' method of its iterator expects type '{1}', but for-of will always send '{0}'."),
            Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_spread_will_always_send_0: diag(2764, ts.DiagnosticCategory.Error, "Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_spread_will_al_2764", "Cannot iterate value because the 'next' method of its iterator expects type '{1}', but array spread will always send '{0}'."),
            Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_destructuring_will_always_send_0: diag(2765, ts.DiagnosticCategory.Error, "Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_destructuring__2765", "Cannot iterate value because the 'next' method of its iterator expects type '{1}', but array destructuring will always send '{0}'."),
            Cannot_delegate_iteration_to_value_because_the_next_method_of_its_iterator_expects_type_1_but_the_containing_generator_will_always_send_0: diag(2766, ts.DiagnosticCategory.Error, "Cannot_delegate_iteration_to_value_because_the_next_method_of_its_iterator_expects_type_1_but_the_co_2766", "Cannot delegate iteration to value because the 'next' method of its iterator expects type '{1}', but the containing generator will always send '{0}'."),
            The_0_property_of_an_iterator_must_be_a_method: diag(2767, ts.DiagnosticCategory.Error, "The_0_property_of_an_iterator_must_be_a_method_2767", "The '{0}' property of an iterator must be a method."),
            The_0_property_of_an_async_iterator_must_be_a_method: diag(2768, ts.DiagnosticCategory.Error, "The_0_property_of_an_async_iterator_must_be_a_method_2768", "The '{0}' property of an async iterator must be a method."),
            No_overload_matches_this_call: diag(2769, ts.DiagnosticCategory.Error, "No_overload_matches_this_call_2769", "No overload matches this call."),
            The_last_overload_gave_the_following_error: diag(2770, ts.DiagnosticCategory.Error, "The_last_overload_gave_the_following_error_2770", "The last overload gave the following error."),
            The_last_overload_is_declared_here: diag(2771, ts.DiagnosticCategory.Error, "The_last_overload_is_declared_here_2771", "The last overload is declared here."),
            Overload_0_of_1_2_gave_the_following_error: diag(2772, ts.DiagnosticCategory.Error, "Overload_0_of_1_2_gave_the_following_error_2772", "Overload {0} of {1}, '{2}', gave the following error."),
            Did_you_forget_to_use_await: diag(2773, ts.DiagnosticCategory.Error, "Did_you_forget_to_use_await_2773", "Did you forget to use 'await'?"),
            This_condition_will_always_return_true_since_the_function_is_always_defined_Did_you_mean_to_call_it_instead: diag(2774, ts.DiagnosticCategory.Error, "This_condition_will_always_return_true_since_the_function_is_always_defined_Did_you_mean_to_call_it__2774", "This condition will always return true since the function is always defined. Did you mean to call it instead?"),
            Assertions_require_every_name_in_the_call_target_to_be_declared_with_an_explicit_type_annotation: diag(2775, ts.DiagnosticCategory.Error, "Assertions_require_every_name_in_the_call_target_to_be_declared_with_an_explicit_type_annotation_2775", "Assertions require every name in the call target to be declared with an explicit type annotation."),
            Assertions_require_the_call_target_to_be_an_identifier_or_qualified_name: diag(2776, ts.DiagnosticCategory.Error, "Assertions_require_the_call_target_to_be_an_identifier_or_qualified_name_2776", "Assertions require the call target to be an identifier or qualified name."),
            The_operand_of_an_increment_or_decrement_operator_may_not_be_an_optional_property_access: diag(2777, ts.DiagnosticCategory.Error, "The_operand_of_an_increment_or_decrement_operator_may_not_be_an_optional_property_access_2777", "The operand of an increment or decrement operator may not be an optional property access."),
            The_target_of_an_object_rest_assignment_may_not_be_an_optional_property_access: diag(2778, ts.DiagnosticCategory.Error, "The_target_of_an_object_rest_assignment_may_not_be_an_optional_property_access_2778", "The target of an object rest assignment may not be an optional property access."),
            The_left_hand_side_of_an_assignment_expression_may_not_be_an_optional_property_access: diag(2779, ts.DiagnosticCategory.Error, "The_left_hand_side_of_an_assignment_expression_may_not_be_an_optional_property_access_2779", "The left-hand side of an assignment expression may not be an optional property access."),
            The_left_hand_side_of_a_for_in_statement_may_not_be_an_optional_property_access: diag(2780, ts.DiagnosticCategory.Error, "The_left_hand_side_of_a_for_in_statement_may_not_be_an_optional_property_access_2780", "The left-hand side of a 'for...in' statement may not be an optional property access."),
            The_left_hand_side_of_a_for_of_statement_may_not_be_an_optional_property_access: diag(2781, ts.DiagnosticCategory.Error, "The_left_hand_side_of_a_for_of_statement_may_not_be_an_optional_property_access_2781", "The left-hand side of a 'for...of' statement may not be an optional property access."),
            _0_needs_an_explicit_type_annotation: diag(2782, ts.DiagnosticCategory.Message, "_0_needs_an_explicit_type_annotation_2782", "'{0}' needs an explicit type annotation."),
            Import_declaration_0_is_using_private_name_1: diag(4000, ts.DiagnosticCategory.Error, "Import_declaration_0_is_using_private_name_1_4000", "Import declaration '{0}' is using private name '{1}'."),
            Type_parameter_0_of_exported_class_has_or_is_using_private_name_1: diag(4002, ts.DiagnosticCategory.Error, "Type_parameter_0_of_exported_class_has_or_is_using_private_name_1_4002", "Type parameter '{0}' of exported class has or is using private name '{1}'."),
            Type_parameter_0_of_exported_interface_has_or_is_using_private_name_1: diag(4004, ts.DiagnosticCategory.Error, "Type_parameter_0_of_exported_interface_has_or_is_using_private_name_1_4004", "Type parameter '{0}' of exported interface has or is using private name '{1}'."),
            Type_parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1: diag(4006, ts.DiagnosticCategory.Error, "Type_parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1_4006", "Type parameter '{0}' of constructor signature from exported interface has or is using private name '{1}'."),
            Type_parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1: diag(4008, ts.DiagnosticCategory.Error, "Type_parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1_4008", "Type parameter '{0}' of call signature from exported interface has or is using private name '{1}'."),
            Type_parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1: diag(4010, ts.DiagnosticCategory.Error, "Type_parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1_4010", "Type parameter '{0}' of public static method from exported class has or is using private name '{1}'."),
            Type_parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1: diag(4012, ts.DiagnosticCategory.Error, "Type_parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1_4012", "Type parameter '{0}' of public method from exported class has or is using private name '{1}'."),
            Type_parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1: diag(4014, ts.DiagnosticCategory.Error, "Type_parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1_4014", "Type parameter '{0}' of method from exported interface has or is using private name '{1}'."),
            Type_parameter_0_of_exported_function_has_or_is_using_private_name_1: diag(4016, ts.DiagnosticCategory.Error, "Type_parameter_0_of_exported_function_has_or_is_using_private_name_1_4016", "Type parameter '{0}' of exported function has or is using private name '{1}'."),
            Implements_clause_of_exported_class_0_has_or_is_using_private_name_1: diag(4019, ts.DiagnosticCategory.Error, "Implements_clause_of_exported_class_0_has_or_is_using_private_name_1_4019", "Implements clause of exported class '{0}' has or is using private name '{1}'."),
            extends_clause_of_exported_class_0_has_or_is_using_private_name_1: diag(4020, ts.DiagnosticCategory.Error, "extends_clause_of_exported_class_0_has_or_is_using_private_name_1_4020", "'extends' clause of exported class '{0}' has or is using private name '{1}'."),
            extends_clause_of_exported_interface_0_has_or_is_using_private_name_1: diag(4022, ts.DiagnosticCategory.Error, "extends_clause_of_exported_interface_0_has_or_is_using_private_name_1_4022", "'extends' clause of exported interface '{0}' has or is using private name '{1}'."),
            Exported_variable_0_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4023, ts.DiagnosticCategory.Error, "Exported_variable_0_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named_4023", "Exported variable '{0}' has or is using name '{1}' from external module {2} but cannot be named."),
            Exported_variable_0_has_or_is_using_name_1_from_private_module_2: diag(4024, ts.DiagnosticCategory.Error, "Exported_variable_0_has_or_is_using_name_1_from_private_module_2_4024", "Exported variable '{0}' has or is using name '{1}' from private module '{2}'."),
            Exported_variable_0_has_or_is_using_private_name_1: diag(4025, ts.DiagnosticCategory.Error, "Exported_variable_0_has_or_is_using_private_name_1_4025", "Exported variable '{0}' has or is using private name '{1}'."),
            Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4026, ts.DiagnosticCategory.Error, "Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot__4026", "Public static property '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named."),
            Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2: diag(4027, ts.DiagnosticCategory.Error, "Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4027", "Public static property '{0}' of exported class has or is using name '{1}' from private module '{2}'."),
            Public_static_property_0_of_exported_class_has_or_is_using_private_name_1: diag(4028, ts.DiagnosticCategory.Error, "Public_static_property_0_of_exported_class_has_or_is_using_private_name_1_4028", "Public static property '{0}' of exported class has or is using private name '{1}'."),
            Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4029, ts.DiagnosticCategory.Error, "Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_name_4029", "Public property '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named."),
            Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2: diag(4030, ts.DiagnosticCategory.Error, "Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4030", "Public property '{0}' of exported class has or is using name '{1}' from private module '{2}'."),
            Public_property_0_of_exported_class_has_or_is_using_private_name_1: diag(4031, ts.DiagnosticCategory.Error, "Public_property_0_of_exported_class_has_or_is_using_private_name_1_4031", "Public property '{0}' of exported class has or is using private name '{1}'."),
            Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2: diag(4032, ts.DiagnosticCategory.Error, "Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2_4032", "Property '{0}' of exported interface has or is using name '{1}' from private module '{2}'."),
            Property_0_of_exported_interface_has_or_is_using_private_name_1: diag(4033, ts.DiagnosticCategory.Error, "Property_0_of_exported_interface_has_or_is_using_private_name_1_4033", "Property '{0}' of exported interface has or is using private name '{1}'."),
            Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2: diag(4034, ts.DiagnosticCategory.Error, "Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_name_1_from_private_mod_4034", "Parameter type of public static setter '{0}' from exported class has or is using name '{1}' from private module '{2}'."),
            Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_private_name_1: diag(4035, ts.DiagnosticCategory.Error, "Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_private_name_1_4035", "Parameter type of public static setter '{0}' from exported class has or is using private name '{1}'."),
            Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2: diag(4036, ts.DiagnosticCategory.Error, "Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2_4036", "Parameter type of public setter '{0}' from exported class has or is using name '{1}' from private module '{2}'."),
            Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_private_name_1: diag(4037, ts.DiagnosticCategory.Error, "Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_private_name_1_4037", "Parameter type of public setter '{0}' from exported class has or is using private name '{1}'."),
            Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4038, ts.DiagnosticCategory.Error, "Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_external_modul_4038", "Return type of public static getter '{0}' from exported class has or is using name '{1}' from external module {2} but cannot be named."),
            Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2: diag(4039, ts.DiagnosticCategory.Error, "Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_4039", "Return type of public static getter '{0}' from exported class has or is using name '{1}' from private module '{2}'."),
            Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_private_name_1: diag(4040, ts.DiagnosticCategory.Error, "Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_private_name_1_4040", "Return type of public static getter '{0}' from exported class has or is using private name '{1}'."),
            Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4041, ts.DiagnosticCategory.Error, "Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_4041", "Return type of public getter '{0}' from exported class has or is using name '{1}' from external module {2} but cannot be named."),
            Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2: diag(4042, ts.DiagnosticCategory.Error, "Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2_4042", "Return type of public getter '{0}' from exported class has or is using name '{1}' from private module '{2}'."),
            Return_type_of_public_getter_0_from_exported_class_has_or_is_using_private_name_1: diag(4043, ts.DiagnosticCategory.Error, "Return_type_of_public_getter_0_from_exported_class_has_or_is_using_private_name_1_4043", "Return type of public getter '{0}' from exported class has or is using private name '{1}'."),
            Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1: diag(4044, ts.DiagnosticCategory.Error, "Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_name_0_from_private_mod_4044", "Return type of constructor signature from exported interface has or is using name '{0}' from private module '{1}'."),
            Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_0: diag(4045, ts.DiagnosticCategory.Error, "Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_0_4045", "Return type of constructor signature from exported interface has or is using private name '{0}'."),
            Return_type_of_call_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1: diag(4046, ts.DiagnosticCategory.Error, "Return_type_of_call_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1_4046", "Return type of call signature from exported interface has or is using name '{0}' from private module '{1}'."),
            Return_type_of_call_signature_from_exported_interface_has_or_is_using_private_name_0: diag(4047, ts.DiagnosticCategory.Error, "Return_type_of_call_signature_from_exported_interface_has_or_is_using_private_name_0_4047", "Return type of call signature from exported interface has or is using private name '{0}'."),
            Return_type_of_index_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1: diag(4048, ts.DiagnosticCategory.Error, "Return_type_of_index_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1_4048", "Return type of index signature from exported interface has or is using name '{0}' from private module '{1}'."),
            Return_type_of_index_signature_from_exported_interface_has_or_is_using_private_name_0: diag(4049, ts.DiagnosticCategory.Error, "Return_type_of_index_signature_from_exported_interface_has_or_is_using_private_name_0_4049", "Return type of index signature from exported interface has or is using private name '{0}'."),
            Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named: diag(4050, ts.DiagnosticCategory.Error, "Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_external_module__4050", "Return type of public static method from exported class has or is using name '{0}' from external module {1} but cannot be named."),
            Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_private_module_1: diag(4051, ts.DiagnosticCategory.Error, "Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_private_module_1_4051", "Return type of public static method from exported class has or is using name '{0}' from private module '{1}'."),
            Return_type_of_public_static_method_from_exported_class_has_or_is_using_private_name_0: diag(4052, ts.DiagnosticCategory.Error, "Return_type_of_public_static_method_from_exported_class_has_or_is_using_private_name_0_4052", "Return type of public static method from exported class has or is using private name '{0}'."),
            Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named: diag(4053, ts.DiagnosticCategory.Error, "Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_c_4053", "Return type of public method from exported class has or is using name '{0}' from external module {1} but cannot be named."),
            Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_private_module_1: diag(4054, ts.DiagnosticCategory.Error, "Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_private_module_1_4054", "Return type of public method from exported class has or is using name '{0}' from private module '{1}'."),
            Return_type_of_public_method_from_exported_class_has_or_is_using_private_name_0: diag(4055, ts.DiagnosticCategory.Error, "Return_type_of_public_method_from_exported_class_has_or_is_using_private_name_0_4055", "Return type of public method from exported class has or is using private name '{0}'."),
            Return_type_of_method_from_exported_interface_has_or_is_using_name_0_from_private_module_1: diag(4056, ts.DiagnosticCategory.Error, "Return_type_of_method_from_exported_interface_has_or_is_using_name_0_from_private_module_1_4056", "Return type of method from exported interface has or is using name '{0}' from private module '{1}'."),
            Return_type_of_method_from_exported_interface_has_or_is_using_private_name_0: diag(4057, ts.DiagnosticCategory.Error, "Return_type_of_method_from_exported_interface_has_or_is_using_private_name_0_4057", "Return type of method from exported interface has or is using private name '{0}'."),
            Return_type_of_exported_function_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named: diag(4058, ts.DiagnosticCategory.Error, "Return_type_of_exported_function_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named_4058", "Return type of exported function has or is using name '{0}' from external module {1} but cannot be named."),
            Return_type_of_exported_function_has_or_is_using_name_0_from_private_module_1: diag(4059, ts.DiagnosticCategory.Error, "Return_type_of_exported_function_has_or_is_using_name_0_from_private_module_1_4059", "Return type of exported function has or is using name '{0}' from private module '{1}'."),
            Return_type_of_exported_function_has_or_is_using_private_name_0: diag(4060, ts.DiagnosticCategory.Error, "Return_type_of_exported_function_has_or_is_using_private_name_0_4060", "Return type of exported function has or is using private name '{0}'."),
            Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4061, ts.DiagnosticCategory.Error, "Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_can_4061", "Parameter '{0}' of constructor from exported class has or is using name '{1}' from external module {2} but cannot be named."),
            Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_private_module_2: diag(4062, ts.DiagnosticCategory.Error, "Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_private_module_2_4062", "Parameter '{0}' of constructor from exported class has or is using name '{1}' from private module '{2}'."),
            Parameter_0_of_constructor_from_exported_class_has_or_is_using_private_name_1: diag(4063, ts.DiagnosticCategory.Error, "Parameter_0_of_constructor_from_exported_class_has_or_is_using_private_name_1_4063", "Parameter '{0}' of constructor from exported class has or is using private name '{1}'."),
            Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2: diag(4064, ts.DiagnosticCategory.Error, "Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_name_1_from_private_mod_4064", "Parameter '{0}' of constructor signature from exported interface has or is using name '{1}' from private module '{2}'."),
            Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1: diag(4065, ts.DiagnosticCategory.Error, "Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1_4065", "Parameter '{0}' of constructor signature from exported interface has or is using private name '{1}'."),
            Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2: diag(4066, ts.DiagnosticCategory.Error, "Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2_4066", "Parameter '{0}' of call signature from exported interface has or is using name '{1}' from private module '{2}'."),
            Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1: diag(4067, ts.DiagnosticCategory.Error, "Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1_4067", "Parameter '{0}' of call signature from exported interface has or is using private name '{1}'."),
            Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4068, ts.DiagnosticCategory.Error, "Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_external_module__4068", "Parameter '{0}' of public static method from exported class has or is using name '{1}' from external module {2} but cannot be named."),
            Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_private_module_2: diag(4069, ts.DiagnosticCategory.Error, "Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_private_module_2_4069", "Parameter '{0}' of public static method from exported class has or is using name '{1}' from private module '{2}'."),
            Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1: diag(4070, ts.DiagnosticCategory.Error, "Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1_4070", "Parameter '{0}' of public static method from exported class has or is using private name '{1}'."),
            Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4071, ts.DiagnosticCategory.Error, "Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_c_4071", "Parameter '{0}' of public method from exported class has or is using name '{1}' from external module {2} but cannot be named."),
            Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_private_module_2: diag(4072, ts.DiagnosticCategory.Error, "Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_private_module_2_4072", "Parameter '{0}' of public method from exported class has or is using name '{1}' from private module '{2}'."),
            Parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1: diag(4073, ts.DiagnosticCategory.Error, "Parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1_4073", "Parameter '{0}' of public method from exported class has or is using private name '{1}'."),
            Parameter_0_of_method_from_exported_interface_has_or_is_using_name_1_from_private_module_2: diag(4074, ts.DiagnosticCategory.Error, "Parameter_0_of_method_from_exported_interface_has_or_is_using_name_1_from_private_module_2_4074", "Parameter '{0}' of method from exported interface has or is using name '{1}' from private module '{2}'."),
            Parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1: diag(4075, ts.DiagnosticCategory.Error, "Parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1_4075", "Parameter '{0}' of method from exported interface has or is using private name '{1}'."),
            Parameter_0_of_exported_function_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4076, ts.DiagnosticCategory.Error, "Parameter_0_of_exported_function_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named_4076", "Parameter '{0}' of exported function has or is using name '{1}' from external module {2} but cannot be named."),
            Parameter_0_of_exported_function_has_or_is_using_name_1_from_private_module_2: diag(4077, ts.DiagnosticCategory.Error, "Parameter_0_of_exported_function_has_or_is_using_name_1_from_private_module_2_4077", "Parameter '{0}' of exported function has or is using name '{1}' from private module '{2}'."),
            Parameter_0_of_exported_function_has_or_is_using_private_name_1: diag(4078, ts.DiagnosticCategory.Error, "Parameter_0_of_exported_function_has_or_is_using_private_name_1_4078", "Parameter '{0}' of exported function has or is using private name '{1}'."),
            Exported_type_alias_0_has_or_is_using_private_name_1: diag(4081, ts.DiagnosticCategory.Error, "Exported_type_alias_0_has_or_is_using_private_name_1_4081", "Exported type alias '{0}' has or is using private name '{1}'."),
            Default_export_of_the_module_has_or_is_using_private_name_0: diag(4082, ts.DiagnosticCategory.Error, "Default_export_of_the_module_has_or_is_using_private_name_0_4082", "Default export of the module has or is using private name '{0}'."),
            Type_parameter_0_of_exported_type_alias_has_or_is_using_private_name_1: diag(4083, ts.DiagnosticCategory.Error, "Type_parameter_0_of_exported_type_alias_has_or_is_using_private_name_1_4083", "Type parameter '{0}' of exported type alias has or is using private name '{1}'."),
            Conflicting_definitions_for_0_found_at_1_and_2_Consider_installing_a_specific_version_of_this_library_to_resolve_the_conflict: diag(4090, ts.DiagnosticCategory.Error, "Conflicting_definitions_for_0_found_at_1_and_2_Consider_installing_a_specific_version_of_this_librar_4090", "Conflicting definitions for '{0}' found at '{1}' and '{2}'. Consider installing a specific version of this library to resolve the conflict."),
            Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2: diag(4091, ts.DiagnosticCategory.Error, "Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2_4091", "Parameter '{0}' of index signature from exported interface has or is using name '{1}' from private module '{2}'."),
            Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_private_name_1: diag(4092, ts.DiagnosticCategory.Error, "Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_private_name_1_4092", "Parameter '{0}' of index signature from exported interface has or is using private name '{1}'."),
            Property_0_of_exported_class_expression_may_not_be_private_or_protected: diag(4094, ts.DiagnosticCategory.Error, "Property_0_of_exported_class_expression_may_not_be_private_or_protected_4094", "Property '{0}' of exported class expression may not be private or protected."),
            Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4095, ts.DiagnosticCategory.Error, "Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_4095", "Public static method '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named."),
            Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2: diag(4096, ts.DiagnosticCategory.Error, "Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4096", "Public static method '{0}' of exported class has or is using name '{1}' from private module '{2}'."),
            Public_static_method_0_of_exported_class_has_or_is_using_private_name_1: diag(4097, ts.DiagnosticCategory.Error, "Public_static_method_0_of_exported_class_has_or_is_using_private_name_1_4097", "Public static method '{0}' of exported class has or is using private name '{1}'."),
            Public_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4098, ts.DiagnosticCategory.Error, "Public_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named_4098", "Public method '{0}' of exported class has or is using name '{1}' from external module {2} but cannot be named."),
            Public_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2: diag(4099, ts.DiagnosticCategory.Error, "Public_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2_4099", "Public method '{0}' of exported class has or is using name '{1}' from private module '{2}'."),
            Public_method_0_of_exported_class_has_or_is_using_private_name_1: diag(4100, ts.DiagnosticCategory.Error, "Public_method_0_of_exported_class_has_or_is_using_private_name_1_4100", "Public method '{0}' of exported class has or is using private name '{1}'."),
            Method_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2: diag(4101, ts.DiagnosticCategory.Error, "Method_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2_4101", "Method '{0}' of exported interface has or is using name '{1}' from private module '{2}'."),
            Method_0_of_exported_interface_has_or_is_using_private_name_1: diag(4102, ts.DiagnosticCategory.Error, "Method_0_of_exported_interface_has_or_is_using_private_name_1_4102", "Method '{0}' of exported interface has or is using private name '{1}'."),
            Type_parameter_0_of_exported_mapped_object_type_is_using_private_name_1: diag(4103, ts.DiagnosticCategory.Error, "Type_parameter_0_of_exported_mapped_object_type_is_using_private_name_1_4103", "Type parameter '{0}' of exported mapped object type is using private name '{1}'."),
            The_type_0_is_readonly_and_cannot_be_assigned_to_the_mutable_type_1: diag(4104, ts.DiagnosticCategory.Error, "The_type_0_is_readonly_and_cannot_be_assigned_to_the_mutable_type_1_4104", "The type '{0}' is 'readonly' and cannot be assigned to the mutable type '{1}'."),
            Private_or_protected_member_0_cannot_be_accessed_on_a_type_parameter: diag(4105, ts.DiagnosticCategory.Error, "Private_or_protected_member_0_cannot_be_accessed_on_a_type_parameter_4105", "Private or protected member '{0}' cannot be accessed on a type parameter."),
            Parameter_0_of_accessor_has_or_is_using_private_name_1: diag(4106, ts.DiagnosticCategory.Error, "Parameter_0_of_accessor_has_or_is_using_private_name_1_4106", "Parameter '{0}' of accessor has or is using private name '{1}'."),
            Parameter_0_of_accessor_has_or_is_using_name_1_from_private_module_2: diag(4107, ts.DiagnosticCategory.Error, "Parameter_0_of_accessor_has_or_is_using_name_1_from_private_module_2_4107", "Parameter '{0}' of accessor has or is using name '{1}' from private module '{2}'."),
            Parameter_0_of_accessor_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named: diag(4108, ts.DiagnosticCategory.Error, "Parameter_0_of_accessor_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named_4108", "Parameter '{0}' of accessor has or is using name '{1}' from external module '{2}' but cannot be named."),
            Type_arguments_for_0_circularly_reference_themselves: diag(4109, ts.DiagnosticCategory.Error, "Type_arguments_for_0_circularly_reference_themselves_4109", "Type arguments for '{0}' circularly reference themselves."),
            Tuple_type_arguments_circularly_reference_themselves: diag(4110, ts.DiagnosticCategory.Error, "Tuple_type_arguments_circularly_reference_themselves_4110", "Tuple type arguments circularly reference themselves."),
            The_current_host_does_not_support_the_0_option: diag(5001, ts.DiagnosticCategory.Error, "The_current_host_does_not_support_the_0_option_5001", "The current host does not support the '{0}' option."),
            Cannot_find_the_common_subdirectory_path_for_the_input_files: diag(5009, ts.DiagnosticCategory.Error, "Cannot_find_the_common_subdirectory_path_for_the_input_files_5009", "Cannot find the common subdirectory path for the input files."),
            File_specification_cannot_end_in_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0: diag(5010, ts.DiagnosticCategory.Error, "File_specification_cannot_end_in_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0_5010", "File specification cannot end in a recursive directory wildcard ('**'): '{0}'."),
            Cannot_read_file_0_Colon_1: diag(5012, ts.DiagnosticCategory.Error, "Cannot_read_file_0_Colon_1_5012", "Cannot read file '{0}': {1}."),
            Failed_to_parse_file_0_Colon_1: diag(5014, ts.DiagnosticCategory.Error, "Failed_to_parse_file_0_Colon_1_5014", "Failed to parse file '{0}': {1}."),
            Unknown_compiler_option_0: diag(5023, ts.DiagnosticCategory.Error, "Unknown_compiler_option_0_5023", "Unknown compiler option '{0}'."),
            Compiler_option_0_requires_a_value_of_type_1: diag(5024, ts.DiagnosticCategory.Error, "Compiler_option_0_requires_a_value_of_type_1_5024", "Compiler option '{0}' requires a value of type {1}."),
            Unknown_compiler_option_0_Did_you_mean_1: diag(5025, ts.DiagnosticCategory.Error, "Unknown_compiler_option_0_Did_you_mean_1_5025", "Unknown compiler option '{0}'. Did you mean '{1}'?"),
            Could_not_write_file_0_Colon_1: diag(5033, ts.DiagnosticCategory.Error, "Could_not_write_file_0_Colon_1_5033", "Could not write file '{0}': {1}."),
            Option_project_cannot_be_mixed_with_source_files_on_a_command_line: diag(5042, ts.DiagnosticCategory.Error, "Option_project_cannot_be_mixed_with_source_files_on_a_command_line_5042", "Option 'project' cannot be mixed with source files on a command line."),
            Option_isolatedModules_can_only_be_used_when_either_option_module_is_provided_or_option_target_is_ES2015_or_higher: diag(5047, ts.DiagnosticCategory.Error, "Option_isolatedModules_can_only_be_used_when_either_option_module_is_provided_or_option_target_is_ES_5047", "Option 'isolatedModules' can only be used when either option '--module' is provided or option 'target' is 'ES2015' or higher."),
            Option_0_cannot_be_specified_when_option_target_is_ES3: diag(5048, ts.DiagnosticCategory.Error, "Option_0_cannot_be_specified_when_option_target_is_ES3_5048", "Option '{0}' cannot be specified when option 'target' is 'ES3'."),
            Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided: diag(5051, ts.DiagnosticCategory.Error, "Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided_5051", "Option '{0} can only be used when either option '--inlineSourceMap' or option '--sourceMap' is provided."),
            Option_0_cannot_be_specified_without_specifying_option_1: diag(5052, ts.DiagnosticCategory.Error, "Option_0_cannot_be_specified_without_specifying_option_1_5052", "Option '{0}' cannot be specified without specifying option '{1}'."),
            Option_0_cannot_be_specified_with_option_1: diag(5053, ts.DiagnosticCategory.Error, "Option_0_cannot_be_specified_with_option_1_5053", "Option '{0}' cannot be specified with option '{1}'."),
            A_tsconfig_json_file_is_already_defined_at_Colon_0: diag(5054, ts.DiagnosticCategory.Error, "A_tsconfig_json_file_is_already_defined_at_Colon_0_5054", "A 'tsconfig.json' file is already defined at: '{0}'."),
            Cannot_write_file_0_because_it_would_overwrite_input_file: diag(5055, ts.DiagnosticCategory.Error, "Cannot_write_file_0_because_it_would_overwrite_input_file_5055", "Cannot write file '{0}' because it would overwrite input file."),
            Cannot_write_file_0_because_it_would_be_overwritten_by_multiple_input_files: diag(5056, ts.DiagnosticCategory.Error, "Cannot_write_file_0_because_it_would_be_overwritten_by_multiple_input_files_5056", "Cannot write file '{0}' because it would be overwritten by multiple input files."),
            Cannot_find_a_tsconfig_json_file_at_the_specified_directory_Colon_0: diag(5057, ts.DiagnosticCategory.Error, "Cannot_find_a_tsconfig_json_file_at_the_specified_directory_Colon_0_5057", "Cannot find a tsconfig.json file at the specified directory: '{0}'."),
            The_specified_path_does_not_exist_Colon_0: diag(5058, ts.DiagnosticCategory.Error, "The_specified_path_does_not_exist_Colon_0_5058", "The specified path does not exist: '{0}'."),
            Invalid_value_for_reactNamespace_0_is_not_a_valid_identifier: diag(5059, ts.DiagnosticCategory.Error, "Invalid_value_for_reactNamespace_0_is_not_a_valid_identifier_5059", "Invalid value for '--reactNamespace'. '{0}' is not a valid identifier."),
            Option_paths_cannot_be_used_without_specifying_baseUrl_option: diag(5060, ts.DiagnosticCategory.Error, "Option_paths_cannot_be_used_without_specifying_baseUrl_option_5060", "Option 'paths' cannot be used without specifying '--baseUrl' option."),
            Pattern_0_can_have_at_most_one_Asterisk_character: diag(5061, ts.DiagnosticCategory.Error, "Pattern_0_can_have_at_most_one_Asterisk_character_5061", "Pattern '{0}' can have at most one '*' character."),
            Substitution_0_in_pattern_1_can_have_at_most_one_Asterisk_character: diag(5062, ts.DiagnosticCategory.Error, "Substitution_0_in_pattern_1_can_have_at_most_one_Asterisk_character_5062", "Substitution '{0}' in pattern '{1}' can have at most one '*' character."),
            Substitutions_for_pattern_0_should_be_an_array: diag(5063, ts.DiagnosticCategory.Error, "Substitutions_for_pattern_0_should_be_an_array_5063", "Substitutions for pattern '{0}' should be an array."),
            Substitution_0_for_pattern_1_has_incorrect_type_expected_string_got_2: diag(5064, ts.DiagnosticCategory.Error, "Substitution_0_for_pattern_1_has_incorrect_type_expected_string_got_2_5064", "Substitution '{0}' for pattern '{1}' has incorrect type, expected 'string', got '{2}'."),
            File_specification_cannot_contain_a_parent_directory_that_appears_after_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0: diag(5065, ts.DiagnosticCategory.Error, "File_specification_cannot_contain_a_parent_directory_that_appears_after_a_recursive_directory_wildca_5065", "File specification cannot contain a parent directory ('..') that appears after a recursive directory wildcard ('**'): '{0}'."),
            Substitutions_for_pattern_0_shouldn_t_be_an_empty_array: diag(5066, ts.DiagnosticCategory.Error, "Substitutions_for_pattern_0_shouldn_t_be_an_empty_array_5066", "Substitutions for pattern '{0}' shouldn't be an empty array."),
            Invalid_value_for_jsxFactory_0_is_not_a_valid_identifier_or_qualified_name: diag(5067, ts.DiagnosticCategory.Error, "Invalid_value_for_jsxFactory_0_is_not_a_valid_identifier_or_qualified_name_5067", "Invalid value for 'jsxFactory'. '{0}' is not a valid identifier or qualified-name."),
            Adding_a_tsconfig_json_file_will_help_organize_projects_that_contain_both_TypeScript_and_JavaScript_files_Learn_more_at_https_Colon_Slash_Slashaka_ms_Slashtsconfig: diag(5068, ts.DiagnosticCategory.Error, "Adding_a_tsconfig_json_file_will_help_organize_projects_that_contain_both_TypeScript_and_JavaScript__5068", "Adding a tsconfig.json file will help organize projects that contain both TypeScript and JavaScript files. Learn more at https://aka.ms/tsconfig."),
            Option_0_cannot_be_specified_without_specifying_option_1_or_option_2: diag(5069, ts.DiagnosticCategory.Error, "Option_0_cannot_be_specified_without_specifying_option_1_or_option_2_5069", "Option '{0}' cannot be specified without specifying option '{1}' or option '{2}'."),
            Option_resolveJsonModule_cannot_be_specified_without_node_module_resolution_strategy: diag(5070, ts.DiagnosticCategory.Error, "Option_resolveJsonModule_cannot_be_specified_without_node_module_resolution_strategy_5070", "Option '--resolveJsonModule' cannot be specified without 'node' module resolution strategy."),
            Option_resolveJsonModule_can_only_be_specified_when_module_code_generation_is_commonjs_amd_es2015_or_esNext: diag(5071, ts.DiagnosticCategory.Error, "Option_resolveJsonModule_can_only_be_specified_when_module_code_generation_is_commonjs_amd_es2015_or_5071", "Option '--resolveJsonModule' can only be specified when module code generation is 'commonjs', 'amd', 'es2015' or 'esNext'."),
            Unknown_build_option_0: diag(5072, ts.DiagnosticCategory.Error, "Unknown_build_option_0_5072", "Unknown build option '{0}'."),
            Build_option_0_requires_a_value_of_type_1: diag(5073, ts.DiagnosticCategory.Error, "Build_option_0_requires_a_value_of_type_1_5073", "Build option '{0}' requires a value of type {1}."),
            Option_incremental_can_only_be_specified_using_tsconfig_emitting_to_single_file_or_when_option_tsBuildInfoFile_is_specified: diag(5074, ts.DiagnosticCategory.Error, "Option_incremental_can_only_be_specified_using_tsconfig_emitting_to_single_file_or_when_option_tsBui_5074", "Option '--incremental' can only be specified using tsconfig, emitting to single file or when option `--tsBuildInfoFile` is specified."),
            _0_is_assignable_to_the_constraint_of_type_1_but_1_could_be_instantiated_with_a_different_subtype_of_constraint_2: diag(5075, ts.DiagnosticCategory.Error, "_0_is_assignable_to_the_constraint_of_type_1_but_1_could_be_instantiated_with_a_different_subtype_of_5075", "'{0}' is assignable to the constraint of type '{1}', but '{1}' could be instantiated with a different subtype of constraint '{2}'."),
            _0_and_1_operations_cannot_be_mixed_without_parentheses: diag(5076, ts.DiagnosticCategory.Error, "_0_and_1_operations_cannot_be_mixed_without_parentheses_5076", "'{0}' and '{1}' operations cannot be mixed without parentheses."),
            Unknown_build_option_0_Did_you_mean_1: diag(5077, ts.DiagnosticCategory.Error, "Unknown_build_option_0_Did_you_mean_1_5077", "Unknown build option '{0}'. Did you mean '{1}'?"),
            Unknown_watch_option_0: diag(5078, ts.DiagnosticCategory.Error, "Unknown_watch_option_0_5078", "Unknown watch option '{0}'."),
            Unknown_watch_option_0_Did_you_mean_1: diag(5079, ts.DiagnosticCategory.Error, "Unknown_watch_option_0_Did_you_mean_1_5079", "Unknown watch option '{0}'. Did you mean '{1}'?"),
            Watch_option_0_requires_a_value_of_type_1: diag(5080, ts.DiagnosticCategory.Error, "Watch_option_0_requires_a_value_of_type_1_5080", "Watch option '{0}' requires a value of type {1}."),
            Cannot_find_a_tsconfig_json_file_at_the_current_directory_Colon_0: diag(5081, ts.DiagnosticCategory.Error, "Cannot_find_a_tsconfig_json_file_at_the_current_directory_Colon_0_5081", "Cannot find a tsconfig.json file at the current directory: {0}."),
            Generates_a_sourcemap_for_each_corresponding_d_ts_file: diag(6000, ts.DiagnosticCategory.Message, "Generates_a_sourcemap_for_each_corresponding_d_ts_file_6000", "Generates a sourcemap for each corresponding '.d.ts' file."),
            Concatenate_and_emit_output_to_single_file: diag(6001, ts.DiagnosticCategory.Message, "Concatenate_and_emit_output_to_single_file_6001", "Concatenate and emit output to single file."),
            Generates_corresponding_d_ts_file: diag(6002, ts.DiagnosticCategory.Message, "Generates_corresponding_d_ts_file_6002", "Generates corresponding '.d.ts' file."),
            Specify_the_location_where_debugger_should_locate_map_files_instead_of_generated_locations: diag(6003, ts.DiagnosticCategory.Message, "Specify_the_location_where_debugger_should_locate_map_files_instead_of_generated_locations_6003", "Specify the location where debugger should locate map files instead of generated locations."),
            Specify_the_location_where_debugger_should_locate_TypeScript_files_instead_of_source_locations: diag(6004, ts.DiagnosticCategory.Message, "Specify_the_location_where_debugger_should_locate_TypeScript_files_instead_of_source_locations_6004", "Specify the location where debugger should locate TypeScript files instead of source locations."),
            Watch_input_files: diag(6005, ts.DiagnosticCategory.Message, "Watch_input_files_6005", "Watch input files."),
            Redirect_output_structure_to_the_directory: diag(6006, ts.DiagnosticCategory.Message, "Redirect_output_structure_to_the_directory_6006", "Redirect output structure to the directory."),
            Do_not_erase_const_enum_declarations_in_generated_code: diag(6007, ts.DiagnosticCategory.Message, "Do_not_erase_const_enum_declarations_in_generated_code_6007", "Do not erase const enum declarations in generated code."),
            Do_not_emit_outputs_if_any_errors_were_reported: diag(6008, ts.DiagnosticCategory.Message, "Do_not_emit_outputs_if_any_errors_were_reported_6008", "Do not emit outputs if any errors were reported."),
            Do_not_emit_comments_to_output: diag(6009, ts.DiagnosticCategory.Message, "Do_not_emit_comments_to_output_6009", "Do not emit comments to output."),
            Do_not_emit_outputs: diag(6010, ts.DiagnosticCategory.Message, "Do_not_emit_outputs_6010", "Do not emit outputs."),
            Allow_default_imports_from_modules_with_no_default_export_This_does_not_affect_code_emit_just_typechecking: diag(6011, ts.DiagnosticCategory.Message, "Allow_default_imports_from_modules_with_no_default_export_This_does_not_affect_code_emit_just_typech_6011", "Allow default imports from modules with no default export. This does not affect code emit, just typechecking."),
            Skip_type_checking_of_declaration_files: diag(6012, ts.DiagnosticCategory.Message, "Skip_type_checking_of_declaration_files_6012", "Skip type checking of declaration files."),
            Do_not_resolve_the_real_path_of_symlinks: diag(6013, ts.DiagnosticCategory.Message, "Do_not_resolve_the_real_path_of_symlinks_6013", "Do not resolve the real path of symlinks."),
            Only_emit_d_ts_declaration_files: diag(6014, ts.DiagnosticCategory.Message, "Only_emit_d_ts_declaration_files_6014", "Only emit '.d.ts' declaration files."),
            Specify_ECMAScript_target_version_Colon_ES3_default_ES5_ES2015_ES2016_ES2017_ES2018_ES2019_ES2020_or_ESNEXT: diag(6015, ts.DiagnosticCategory.Message, "Specify_ECMAScript_target_version_Colon_ES3_default_ES5_ES2015_ES2016_ES2017_ES2018_ES2019_ES2020_or_6015", "Specify ECMAScript target version: 'ES3' (default), 'ES5', 'ES2015', 'ES2016', 'ES2017', 'ES2018', 'ES2019', 'ES2020', or 'ESNEXT'."),
            Specify_module_code_generation_Colon_none_commonjs_amd_system_umd_es2015_es2020_or_ESNext: diag(6016, ts.DiagnosticCategory.Message, "Specify_module_code_generation_Colon_none_commonjs_amd_system_umd_es2015_es2020_or_ESNext_6016", "Specify module code generation: 'none', 'commonjs', 'amd', 'system', 'umd', 'es2015', 'es2020', or 'ESNext'."),
            Print_this_message: diag(6017, ts.DiagnosticCategory.Message, "Print_this_message_6017", "Print this message."),
            Print_the_compiler_s_version: diag(6019, ts.DiagnosticCategory.Message, "Print_the_compiler_s_version_6019", "Print the compiler's version."),
            Compile_the_project_given_the_path_to_its_configuration_file_or_to_a_folder_with_a_tsconfig_json: diag(6020, ts.DiagnosticCategory.Message, "Compile_the_project_given_the_path_to_its_configuration_file_or_to_a_folder_with_a_tsconfig_json_6020", "Compile the project given the path to its configuration file, or to a folder with a 'tsconfig.json'."),
            Syntax_Colon_0: diag(6023, ts.DiagnosticCategory.Message, "Syntax_Colon_0_6023", "Syntax: {0}"),
            options: diag(6024, ts.DiagnosticCategory.Message, "options_6024", "options"),
            file: diag(6025, ts.DiagnosticCategory.Message, "file_6025", "file"),
            Examples_Colon_0: diag(6026, ts.DiagnosticCategory.Message, "Examples_Colon_0_6026", "Examples: {0}"),
            Options_Colon: diag(6027, ts.DiagnosticCategory.Message, "Options_Colon_6027", "Options:"),
            Version_0: diag(6029, ts.DiagnosticCategory.Message, "Version_0_6029", "Version {0}"),
            Insert_command_line_options_and_files_from_a_file: diag(6030, ts.DiagnosticCategory.Message, "Insert_command_line_options_and_files_from_a_file_6030", "Insert command line options and files from a file."),
            Starting_compilation_in_watch_mode: diag(6031, ts.DiagnosticCategory.Message, "Starting_compilation_in_watch_mode_6031", "Starting compilation in watch mode..."),
            File_change_detected_Starting_incremental_compilation: diag(6032, ts.DiagnosticCategory.Message, "File_change_detected_Starting_incremental_compilation_6032", "File change detected. Starting incremental compilation..."),
            KIND: diag(6034, ts.DiagnosticCategory.Message, "KIND_6034", "KIND"),
            FILE: diag(6035, ts.DiagnosticCategory.Message, "FILE_6035", "FILE"),
            VERSION: diag(6036, ts.DiagnosticCategory.Message, "VERSION_6036", "VERSION"),
            LOCATION: diag(6037, ts.DiagnosticCategory.Message, "LOCATION_6037", "LOCATION"),
            DIRECTORY: diag(6038, ts.DiagnosticCategory.Message, "DIRECTORY_6038", "DIRECTORY"),
            STRATEGY: diag(6039, ts.DiagnosticCategory.Message, "STRATEGY_6039", "STRATEGY"),
            FILE_OR_DIRECTORY: diag(6040, ts.DiagnosticCategory.Message, "FILE_OR_DIRECTORY_6040", "FILE OR DIRECTORY"),
            Generates_corresponding_map_file: diag(6043, ts.DiagnosticCategory.Message, "Generates_corresponding_map_file_6043", "Generates corresponding '.map' file."),
            Compiler_option_0_expects_an_argument: diag(6044, ts.DiagnosticCategory.Error, "Compiler_option_0_expects_an_argument_6044", "Compiler option '{0}' expects an argument."),
            Unterminated_quoted_string_in_response_file_0: diag(6045, ts.DiagnosticCategory.Error, "Unterminated_quoted_string_in_response_file_0_6045", "Unterminated quoted string in response file '{0}'."),
            Argument_for_0_option_must_be_Colon_1: diag(6046, ts.DiagnosticCategory.Error, "Argument_for_0_option_must_be_Colon_1_6046", "Argument for '{0}' option must be: {1}."),
            Locale_must_be_of_the_form_language_or_language_territory_For_example_0_or_1: diag(6048, ts.DiagnosticCategory.Error, "Locale_must_be_of_the_form_language_or_language_territory_For_example_0_or_1_6048", "Locale must be of the form <language> or <language>-<territory>. For example '{0}' or '{1}'."),
            Unsupported_locale_0: diag(6049, ts.DiagnosticCategory.Error, "Unsupported_locale_0_6049", "Unsupported locale '{0}'."),
            Unable_to_open_file_0: diag(6050, ts.DiagnosticCategory.Error, "Unable_to_open_file_0_6050", "Unable to open file '{0}'."),
            Corrupted_locale_file_0: diag(6051, ts.DiagnosticCategory.Error, "Corrupted_locale_file_0_6051", "Corrupted locale file {0}."),
            Raise_error_on_expressions_and_declarations_with_an_implied_any_type: diag(6052, ts.DiagnosticCategory.Message, "Raise_error_on_expressions_and_declarations_with_an_implied_any_type_6052", "Raise error on expressions and declarations with an implied 'any' type."),
            File_0_not_found: diag(6053, ts.DiagnosticCategory.Error, "File_0_not_found_6053", "File '{0}' not found."),
            File_0_has_an_unsupported_extension_The_only_supported_extensions_are_1: diag(6054, ts.DiagnosticCategory.Error, "File_0_has_an_unsupported_extension_The_only_supported_extensions_are_1_6054", "File '{0}' has an unsupported extension. The only supported extensions are {1}."),
            Suppress_noImplicitAny_errors_for_indexing_objects_lacking_index_signatures: diag(6055, ts.DiagnosticCategory.Message, "Suppress_noImplicitAny_errors_for_indexing_objects_lacking_index_signatures_6055", "Suppress noImplicitAny errors for indexing objects lacking index signatures."),
            Do_not_emit_declarations_for_code_that_has_an_internal_annotation: diag(6056, ts.DiagnosticCategory.Message, "Do_not_emit_declarations_for_code_that_has_an_internal_annotation_6056", "Do not emit declarations for code that has an '@internal' annotation."),
            Specify_the_root_directory_of_input_files_Use_to_control_the_output_directory_structure_with_outDir: diag(6058, ts.DiagnosticCategory.Message, "Specify_the_root_directory_of_input_files_Use_to_control_the_output_directory_structure_with_outDir_6058", "Specify the root directory of input files. Use to control the output directory structure with --outDir."),
            File_0_is_not_under_rootDir_1_rootDir_is_expected_to_contain_all_source_files: diag(6059, ts.DiagnosticCategory.Error, "File_0_is_not_under_rootDir_1_rootDir_is_expected_to_contain_all_source_files_6059", "File '{0}' is not under 'rootDir' '{1}'. 'rootDir' is expected to contain all source files."),
            Specify_the_end_of_line_sequence_to_be_used_when_emitting_files_Colon_CRLF_dos_or_LF_unix: diag(6060, ts.DiagnosticCategory.Message, "Specify_the_end_of_line_sequence_to_be_used_when_emitting_files_Colon_CRLF_dos_or_LF_unix_6060", "Specify the end of line sequence to be used when emitting files: 'CRLF' (dos) or 'LF' (unix)."),
            NEWLINE: diag(6061, ts.DiagnosticCategory.Message, "NEWLINE_6061", "NEWLINE"),
            Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_null_on_command_line: diag(6064, ts.DiagnosticCategory.Error, "Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_null_on_command_line_6064", "Option '{0}' can only be specified in 'tsconfig.json' file or set to 'null' on command line."),
            Enables_experimental_support_for_ES7_decorators: diag(6065, ts.DiagnosticCategory.Message, "Enables_experimental_support_for_ES7_decorators_6065", "Enables experimental support for ES7 decorators."),
            Enables_experimental_support_for_emitting_type_metadata_for_decorators: diag(6066, ts.DiagnosticCategory.Message, "Enables_experimental_support_for_emitting_type_metadata_for_decorators_6066", "Enables experimental support for emitting type metadata for decorators."),
            Enables_experimental_support_for_ES7_async_functions: diag(6068, ts.DiagnosticCategory.Message, "Enables_experimental_support_for_ES7_async_functions_6068", "Enables experimental support for ES7 async functions."),
            Specify_module_resolution_strategy_Colon_node_Node_js_or_classic_TypeScript_pre_1_6: diag(6069, ts.DiagnosticCategory.Message, "Specify_module_resolution_strategy_Colon_node_Node_js_or_classic_TypeScript_pre_1_6_6069", "Specify module resolution strategy: 'node' (Node.js) or 'classic' (TypeScript pre-1.6)."),
            Initializes_a_TypeScript_project_and_creates_a_tsconfig_json_file: diag(6070, ts.DiagnosticCategory.Message, "Initializes_a_TypeScript_project_and_creates_a_tsconfig_json_file_6070", "Initializes a TypeScript project and creates a tsconfig.json file."),
            Successfully_created_a_tsconfig_json_file: diag(6071, ts.DiagnosticCategory.Message, "Successfully_created_a_tsconfig_json_file_6071", "Successfully created a tsconfig.json file."),
            Suppress_excess_property_checks_for_object_literals: diag(6072, ts.DiagnosticCategory.Message, "Suppress_excess_property_checks_for_object_literals_6072", "Suppress excess property checks for object literals."),
            Stylize_errors_and_messages_using_color_and_context_experimental: diag(6073, ts.DiagnosticCategory.Message, "Stylize_errors_and_messages_using_color_and_context_experimental_6073", "Stylize errors and messages using color and context (experimental)."),
            Do_not_report_errors_on_unused_labels: diag(6074, ts.DiagnosticCategory.Message, "Do_not_report_errors_on_unused_labels_6074", "Do not report errors on unused labels."),
            Report_error_when_not_all_code_paths_in_function_return_a_value: diag(6075, ts.DiagnosticCategory.Message, "Report_error_when_not_all_code_paths_in_function_return_a_value_6075", "Report error when not all code paths in function return a value."),
            Report_errors_for_fallthrough_cases_in_switch_statement: diag(6076, ts.DiagnosticCategory.Message, "Report_errors_for_fallthrough_cases_in_switch_statement_6076", "Report errors for fallthrough cases in switch statement."),
            Do_not_report_errors_on_unreachable_code: diag(6077, ts.DiagnosticCategory.Message, "Do_not_report_errors_on_unreachable_code_6077", "Do not report errors on unreachable code."),
            Disallow_inconsistently_cased_references_to_the_same_file: diag(6078, ts.DiagnosticCategory.Message, "Disallow_inconsistently_cased_references_to_the_same_file_6078", "Disallow inconsistently-cased references to the same file."),
            Specify_library_files_to_be_included_in_the_compilation: diag(6079, ts.DiagnosticCategory.Message, "Specify_library_files_to_be_included_in_the_compilation_6079", "Specify library files to be included in the compilation."),
            Specify_JSX_code_generation_Colon_preserve_react_native_or_react: diag(6080, ts.DiagnosticCategory.Message, "Specify_JSX_code_generation_Colon_preserve_react_native_or_react_6080", "Specify JSX code generation: 'preserve', 'react-native', or 'react'."),
            File_0_has_an_unsupported_extension_so_skipping_it: diag(6081, ts.DiagnosticCategory.Message, "File_0_has_an_unsupported_extension_so_skipping_it_6081", "File '{0}' has an unsupported extension, so skipping it."),
            Only_amd_and_system_modules_are_supported_alongside_0: diag(6082, ts.DiagnosticCategory.Error, "Only_amd_and_system_modules_are_supported_alongside_0_6082", "Only 'amd' and 'system' modules are supported alongside --{0}."),
            Base_directory_to_resolve_non_absolute_module_names: diag(6083, ts.DiagnosticCategory.Message, "Base_directory_to_resolve_non_absolute_module_names_6083", "Base directory to resolve non-absolute module names."),
            Deprecated_Use_jsxFactory_instead_Specify_the_object_invoked_for_createElement_when_targeting_react_JSX_emit: diag(6084, ts.DiagnosticCategory.Message, "Deprecated_Use_jsxFactory_instead_Specify_the_object_invoked_for_createElement_when_targeting_react__6084", "[Deprecated] Use '--jsxFactory' instead. Specify the object invoked for createElement when targeting 'react' JSX emit"),
            Enable_tracing_of_the_name_resolution_process: diag(6085, ts.DiagnosticCategory.Message, "Enable_tracing_of_the_name_resolution_process_6085", "Enable tracing of the name resolution process."),
            Resolving_module_0_from_1: diag(6086, ts.DiagnosticCategory.Message, "Resolving_module_0_from_1_6086", "======== Resolving module '{0}' from '{1}'. ========"),
            Explicitly_specified_module_resolution_kind_Colon_0: diag(6087, ts.DiagnosticCategory.Message, "Explicitly_specified_module_resolution_kind_Colon_0_6087", "Explicitly specified module resolution kind: '{0}'."),
            Module_resolution_kind_is_not_specified_using_0: diag(6088, ts.DiagnosticCategory.Message, "Module_resolution_kind_is_not_specified_using_0_6088", "Module resolution kind is not specified, using '{0}'."),
            Module_name_0_was_successfully_resolved_to_1: diag(6089, ts.DiagnosticCategory.Message, "Module_name_0_was_successfully_resolved_to_1_6089", "======== Module name '{0}' was successfully resolved to '{1}'. ========"),
            Module_name_0_was_not_resolved: diag(6090, ts.DiagnosticCategory.Message, "Module_name_0_was_not_resolved_6090", "======== Module name '{0}' was not resolved. ========"),
            paths_option_is_specified_looking_for_a_pattern_to_match_module_name_0: diag(6091, ts.DiagnosticCategory.Message, "paths_option_is_specified_looking_for_a_pattern_to_match_module_name_0_6091", "'paths' option is specified, looking for a pattern to match module name '{0}'."),
            Module_name_0_matched_pattern_1: diag(6092, ts.DiagnosticCategory.Message, "Module_name_0_matched_pattern_1_6092", "Module name '{0}', matched pattern '{1}'."),
            Trying_substitution_0_candidate_module_location_Colon_1: diag(6093, ts.DiagnosticCategory.Message, "Trying_substitution_0_candidate_module_location_Colon_1_6093", "Trying substitution '{0}', candidate module location: '{1}'."),
            Resolving_module_name_0_relative_to_base_url_1_2: diag(6094, ts.DiagnosticCategory.Message, "Resolving_module_name_0_relative_to_base_url_1_2_6094", "Resolving module name '{0}' relative to base url '{1}' - '{2}'."),
            Loading_module_as_file_Slash_folder_candidate_module_location_0_target_file_type_1: diag(6095, ts.DiagnosticCategory.Message, "Loading_module_as_file_Slash_folder_candidate_module_location_0_target_file_type_1_6095", "Loading module as file / folder, candidate module location '{0}', target file type '{1}'."),
            File_0_does_not_exist: diag(6096, ts.DiagnosticCategory.Message, "File_0_does_not_exist_6096", "File '{0}' does not exist."),
            File_0_exist_use_it_as_a_name_resolution_result: diag(6097, ts.DiagnosticCategory.Message, "File_0_exist_use_it_as_a_name_resolution_result_6097", "File '{0}' exist - use it as a name resolution result."),
            Loading_module_0_from_node_modules_folder_target_file_type_1: diag(6098, ts.DiagnosticCategory.Message, "Loading_module_0_from_node_modules_folder_target_file_type_1_6098", "Loading module '{0}' from 'node_modules' folder, target file type '{1}'."),
            Found_package_json_at_0: diag(6099, ts.DiagnosticCategory.Message, "Found_package_json_at_0_6099", "Found 'package.json' at '{0}'."),
            package_json_does_not_have_a_0_field: diag(6100, ts.DiagnosticCategory.Message, "package_json_does_not_have_a_0_field_6100", "'package.json' does not have a '{0}' field."),
            package_json_has_0_field_1_that_references_2: diag(6101, ts.DiagnosticCategory.Message, "package_json_has_0_field_1_that_references_2_6101", "'package.json' has '{0}' field '{1}' that references '{2}'."),
            Allow_javascript_files_to_be_compiled: diag(6102, ts.DiagnosticCategory.Message, "Allow_javascript_files_to_be_compiled_6102", "Allow javascript files to be compiled."),
            Option_0_should_have_array_of_strings_as_a_value: diag(6103, ts.DiagnosticCategory.Error, "Option_0_should_have_array_of_strings_as_a_value_6103", "Option '{0}' should have array of strings as a value."),
            Checking_if_0_is_the_longest_matching_prefix_for_1_2: diag(6104, ts.DiagnosticCategory.Message, "Checking_if_0_is_the_longest_matching_prefix_for_1_2_6104", "Checking if '{0}' is the longest matching prefix for '{1}' - '{2}'."),
            Expected_type_of_0_field_in_package_json_to_be_1_got_2: diag(6105, ts.DiagnosticCategory.Message, "Expected_type_of_0_field_in_package_json_to_be_1_got_2_6105", "Expected type of '{0}' field in 'package.json' to be '{1}', got '{2}'."),
            baseUrl_option_is_set_to_0_using_this_value_to_resolve_non_relative_module_name_1: diag(6106, ts.DiagnosticCategory.Message, "baseUrl_option_is_set_to_0_using_this_value_to_resolve_non_relative_module_name_1_6106", "'baseUrl' option is set to '{0}', using this value to resolve non-relative module name '{1}'."),
            rootDirs_option_is_set_using_it_to_resolve_relative_module_name_0: diag(6107, ts.DiagnosticCategory.Message, "rootDirs_option_is_set_using_it_to_resolve_relative_module_name_0_6107", "'rootDirs' option is set, using it to resolve relative module name '{0}'."),
            Longest_matching_prefix_for_0_is_1: diag(6108, ts.DiagnosticCategory.Message, "Longest_matching_prefix_for_0_is_1_6108", "Longest matching prefix for '{0}' is '{1}'."),
            Loading_0_from_the_root_dir_1_candidate_location_2: diag(6109, ts.DiagnosticCategory.Message, "Loading_0_from_the_root_dir_1_candidate_location_2_6109", "Loading '{0}' from the root dir '{1}', candidate location '{2}'."),
            Trying_other_entries_in_rootDirs: diag(6110, ts.DiagnosticCategory.Message, "Trying_other_entries_in_rootDirs_6110", "Trying other entries in 'rootDirs'."),
            Module_resolution_using_rootDirs_has_failed: diag(6111, ts.DiagnosticCategory.Message, "Module_resolution_using_rootDirs_has_failed_6111", "Module resolution using 'rootDirs' has failed."),
            Do_not_emit_use_strict_directives_in_module_output: diag(6112, ts.DiagnosticCategory.Message, "Do_not_emit_use_strict_directives_in_module_output_6112", "Do not emit 'use strict' directives in module output."),
            Enable_strict_null_checks: diag(6113, ts.DiagnosticCategory.Message, "Enable_strict_null_checks_6113", "Enable strict null checks."),
            Unknown_option_excludes_Did_you_mean_exclude: diag(6114, ts.DiagnosticCategory.Error, "Unknown_option_excludes_Did_you_mean_exclude_6114", "Unknown option 'excludes'. Did you mean 'exclude'?"),
            Raise_error_on_this_expressions_with_an_implied_any_type: diag(6115, ts.DiagnosticCategory.Message, "Raise_error_on_this_expressions_with_an_implied_any_type_6115", "Raise error on 'this' expressions with an implied 'any' type."),
            Resolving_type_reference_directive_0_containing_file_1_root_directory_2: diag(6116, ts.DiagnosticCategory.Message, "Resolving_type_reference_directive_0_containing_file_1_root_directory_2_6116", "======== Resolving type reference directive '{0}', containing file '{1}', root directory '{2}'. ========"),
            Resolving_using_primary_search_paths: diag(6117, ts.DiagnosticCategory.Message, "Resolving_using_primary_search_paths_6117", "Resolving using primary search paths..."),
            Resolving_from_node_modules_folder: diag(6118, ts.DiagnosticCategory.Message, "Resolving_from_node_modules_folder_6118", "Resolving from node_modules folder..."),
            Type_reference_directive_0_was_successfully_resolved_to_1_primary_Colon_2: diag(6119, ts.DiagnosticCategory.Message, "Type_reference_directive_0_was_successfully_resolved_to_1_primary_Colon_2_6119", "======== Type reference directive '{0}' was successfully resolved to '{1}', primary: {2}. ========"),
            Type_reference_directive_0_was_not_resolved: diag(6120, ts.DiagnosticCategory.Message, "Type_reference_directive_0_was_not_resolved_6120", "======== Type reference directive '{0}' was not resolved. ========"),
            Resolving_with_primary_search_path_0: diag(6121, ts.DiagnosticCategory.Message, "Resolving_with_primary_search_path_0_6121", "Resolving with primary search path '{0}'."),
            Root_directory_cannot_be_determined_skipping_primary_search_paths: diag(6122, ts.DiagnosticCategory.Message, "Root_directory_cannot_be_determined_skipping_primary_search_paths_6122", "Root directory cannot be determined, skipping primary search paths."),
            Resolving_type_reference_directive_0_containing_file_1_root_directory_not_set: diag(6123, ts.DiagnosticCategory.Message, "Resolving_type_reference_directive_0_containing_file_1_root_directory_not_set_6123", "======== Resolving type reference directive '{0}', containing file '{1}', root directory not set. ========"),
            Type_declaration_files_to_be_included_in_compilation: diag(6124, ts.DiagnosticCategory.Message, "Type_declaration_files_to_be_included_in_compilation_6124", "Type declaration files to be included in compilation."),
            Looking_up_in_node_modules_folder_initial_location_0: diag(6125, ts.DiagnosticCategory.Message, "Looking_up_in_node_modules_folder_initial_location_0_6125", "Looking up in 'node_modules' folder, initial location '{0}'."),
            Containing_file_is_not_specified_and_root_directory_cannot_be_determined_skipping_lookup_in_node_modules_folder: diag(6126, ts.DiagnosticCategory.Message, "Containing_file_is_not_specified_and_root_directory_cannot_be_determined_skipping_lookup_in_node_mod_6126", "Containing file is not specified and root directory cannot be determined, skipping lookup in 'node_modules' folder."),
            Resolving_type_reference_directive_0_containing_file_not_set_root_directory_1: diag(6127, ts.DiagnosticCategory.Message, "Resolving_type_reference_directive_0_containing_file_not_set_root_directory_1_6127", "======== Resolving type reference directive '{0}', containing file not set, root directory '{1}'. ========"),
            Resolving_type_reference_directive_0_containing_file_not_set_root_directory_not_set: diag(6128, ts.DiagnosticCategory.Message, "Resolving_type_reference_directive_0_containing_file_not_set_root_directory_not_set_6128", "======== Resolving type reference directive '{0}', containing file not set, root directory not set. ========"),
            Resolving_real_path_for_0_result_1: diag(6130, ts.DiagnosticCategory.Message, "Resolving_real_path_for_0_result_1_6130", "Resolving real path for '{0}', result '{1}'."),
            Cannot_compile_modules_using_option_0_unless_the_module_flag_is_amd_or_system: diag(6131, ts.DiagnosticCategory.Error, "Cannot_compile_modules_using_option_0_unless_the_module_flag_is_amd_or_system_6131", "Cannot compile modules using option '{0}' unless the '--module' flag is 'amd' or 'system'."),
            File_name_0_has_a_1_extension_stripping_it: diag(6132, ts.DiagnosticCategory.Message, "File_name_0_has_a_1_extension_stripping_it_6132", "File name '{0}' has a '{1}' extension - stripping it."),
            _0_is_declared_but_its_value_is_never_read: diag(6133, ts.DiagnosticCategory.Error, "_0_is_declared_but_its_value_is_never_read_6133", "'{0}' is declared but its value is never read.", /*reportsUnnecessary*/ true),
            Report_errors_on_unused_locals: diag(6134, ts.DiagnosticCategory.Message, "Report_errors_on_unused_locals_6134", "Report errors on unused locals."),
            Report_errors_on_unused_parameters: diag(6135, ts.DiagnosticCategory.Message, "Report_errors_on_unused_parameters_6135", "Report errors on unused parameters."),
            The_maximum_dependency_depth_to_search_under_node_modules_and_load_JavaScript_files: diag(6136, ts.DiagnosticCategory.Message, "The_maximum_dependency_depth_to_search_under_node_modules_and_load_JavaScript_files_6136", "The maximum dependency depth to search under node_modules and load JavaScript files."),
            Cannot_import_type_declaration_files_Consider_importing_0_instead_of_1: diag(6137, ts.DiagnosticCategory.Error, "Cannot_import_type_declaration_files_Consider_importing_0_instead_of_1_6137", "Cannot import type declaration files. Consider importing '{0}' instead of '{1}'."),
            Property_0_is_declared_but_its_value_is_never_read: diag(6138, ts.DiagnosticCategory.Error, "Property_0_is_declared_but_its_value_is_never_read_6138", "Property '{0}' is declared but its value is never read.", /*reportsUnnecessary*/ true),
            Import_emit_helpers_from_tslib: diag(6139, ts.DiagnosticCategory.Message, "Import_emit_helpers_from_tslib_6139", "Import emit helpers from 'tslib'."),
            Auto_discovery_for_typings_is_enabled_in_project_0_Running_extra_resolution_pass_for_module_1_using_cache_location_2: diag(6140, ts.DiagnosticCategory.Error, "Auto_discovery_for_typings_is_enabled_in_project_0_Running_extra_resolution_pass_for_module_1_using__6140", "Auto discovery for typings is enabled in project '{0}'. Running extra resolution pass for module '{1}' using cache location '{2}'."),
            Parse_in_strict_mode_and_emit_use_strict_for_each_source_file: diag(6141, ts.DiagnosticCategory.Message, "Parse_in_strict_mode_and_emit_use_strict_for_each_source_file_6141", "Parse in strict mode and emit \"use strict\" for each source file."),
            Module_0_was_resolved_to_1_but_jsx_is_not_set: diag(6142, ts.DiagnosticCategory.Error, "Module_0_was_resolved_to_1_but_jsx_is_not_set_6142", "Module '{0}' was resolved to '{1}', but '--jsx' is not set."),
            Module_0_was_resolved_as_locally_declared_ambient_module_in_file_1: diag(6144, ts.DiagnosticCategory.Message, "Module_0_was_resolved_as_locally_declared_ambient_module_in_file_1_6144", "Module '{0}' was resolved as locally declared ambient module in file '{1}'."),
            Module_0_was_resolved_as_ambient_module_declared_in_1_since_this_file_was_not_modified: diag(6145, ts.DiagnosticCategory.Message, "Module_0_was_resolved_as_ambient_module_declared_in_1_since_this_file_was_not_modified_6145", "Module '{0}' was resolved as ambient module declared in '{1}' since this file was not modified."),
            Specify_the_JSX_factory_function_to_use_when_targeting_react_JSX_emit_e_g_React_createElement_or_h: diag(6146, ts.DiagnosticCategory.Message, "Specify_the_JSX_factory_function_to_use_when_targeting_react_JSX_emit_e_g_React_createElement_or_h_6146", "Specify the JSX factory function to use when targeting 'react' JSX emit, e.g. 'React.createElement' or 'h'."),
            Resolution_for_module_0_was_found_in_cache_from_location_1: diag(6147, ts.DiagnosticCategory.Message, "Resolution_for_module_0_was_found_in_cache_from_location_1_6147", "Resolution for module '{0}' was found in cache from location '{1}'."),
            Directory_0_does_not_exist_skipping_all_lookups_in_it: diag(6148, ts.DiagnosticCategory.Message, "Directory_0_does_not_exist_skipping_all_lookups_in_it_6148", "Directory '{0}' does not exist, skipping all lookups in it."),
            Show_diagnostic_information: diag(6149, ts.DiagnosticCategory.Message, "Show_diagnostic_information_6149", "Show diagnostic information."),
            Show_verbose_diagnostic_information: diag(6150, ts.DiagnosticCategory.Message, "Show_verbose_diagnostic_information_6150", "Show verbose diagnostic information."),
            Emit_a_single_file_with_source_maps_instead_of_having_a_separate_file: diag(6151, ts.DiagnosticCategory.Message, "Emit_a_single_file_with_source_maps_instead_of_having_a_separate_file_6151", "Emit a single file with source maps instead of having a separate file."),
            Emit_the_source_alongside_the_sourcemaps_within_a_single_file_requires_inlineSourceMap_or_sourceMap_to_be_set: diag(6152, ts.DiagnosticCategory.Message, "Emit_the_source_alongside_the_sourcemaps_within_a_single_file_requires_inlineSourceMap_or_sourceMap__6152", "Emit the source alongside the sourcemaps within a single file; requires '--inlineSourceMap' or '--sourceMap' to be set."),
            Transpile_each_file_as_a_separate_module_similar_to_ts_transpileModule: diag(6153, ts.DiagnosticCategory.Message, "Transpile_each_file_as_a_separate_module_similar_to_ts_transpileModule_6153", "Transpile each file as a separate module (similar to 'ts.transpileModule')."),
            Print_names_of_generated_files_part_of_the_compilation: diag(6154, ts.DiagnosticCategory.Message, "Print_names_of_generated_files_part_of_the_compilation_6154", "Print names of generated files part of the compilation."),
            Print_names_of_files_part_of_the_compilation: diag(6155, ts.DiagnosticCategory.Message, "Print_names_of_files_part_of_the_compilation_6155", "Print names of files part of the compilation."),
            The_locale_used_when_displaying_messages_to_the_user_e_g_en_us: diag(6156, ts.DiagnosticCategory.Message, "The_locale_used_when_displaying_messages_to_the_user_e_g_en_us_6156", "The locale used when displaying messages to the user (e.g. 'en-us')"),
            Do_not_generate_custom_helper_functions_like_extends_in_compiled_output: diag(6157, ts.DiagnosticCategory.Message, "Do_not_generate_custom_helper_functions_like_extends_in_compiled_output_6157", "Do not generate custom helper functions like '__extends' in compiled output."),
            Do_not_include_the_default_library_file_lib_d_ts: diag(6158, ts.DiagnosticCategory.Message, "Do_not_include_the_default_library_file_lib_d_ts_6158", "Do not include the default library file (lib.d.ts)."),
            Do_not_add_triple_slash_references_or_imported_modules_to_the_list_of_compiled_files: diag(6159, ts.DiagnosticCategory.Message, "Do_not_add_triple_slash_references_or_imported_modules_to_the_list_of_compiled_files_6159", "Do not add triple-slash references or imported modules to the list of compiled files."),
            Deprecated_Use_skipLibCheck_instead_Skip_type_checking_of_default_library_declaration_files: diag(6160, ts.DiagnosticCategory.Message, "Deprecated_Use_skipLibCheck_instead_Skip_type_checking_of_default_library_declaration_files_6160", "[Deprecated] Use '--skipLibCheck' instead. Skip type checking of default library declaration files."),
            List_of_folders_to_include_type_definitions_from: diag(6161, ts.DiagnosticCategory.Message, "List_of_folders_to_include_type_definitions_from_6161", "List of folders to include type definitions from."),
            Disable_size_limitations_on_JavaScript_projects: diag(6162, ts.DiagnosticCategory.Message, "Disable_size_limitations_on_JavaScript_projects_6162", "Disable size limitations on JavaScript projects."),
            The_character_set_of_the_input_files: diag(6163, ts.DiagnosticCategory.Message, "The_character_set_of_the_input_files_6163", "The character set of the input files."),
            Emit_a_UTF_8_Byte_Order_Mark_BOM_in_the_beginning_of_output_files: diag(6164, ts.DiagnosticCategory.Message, "Emit_a_UTF_8_Byte_Order_Mark_BOM_in_the_beginning_of_output_files_6164", "Emit a UTF-8 Byte Order Mark (BOM) in the beginning of output files."),
            Do_not_truncate_error_messages: diag(6165, ts.DiagnosticCategory.Message, "Do_not_truncate_error_messages_6165", "Do not truncate error messages."),
            Output_directory_for_generated_declaration_files: diag(6166, ts.DiagnosticCategory.Message, "Output_directory_for_generated_declaration_files_6166", "Output directory for generated declaration files."),
            A_series_of_entries_which_re_map_imports_to_lookup_locations_relative_to_the_baseUrl: diag(6167, ts.DiagnosticCategory.Message, "A_series_of_entries_which_re_map_imports_to_lookup_locations_relative_to_the_baseUrl_6167", "A series of entries which re-map imports to lookup locations relative to the 'baseUrl'."),
            List_of_root_folders_whose_combined_content_represents_the_structure_of_the_project_at_runtime: diag(6168, ts.DiagnosticCategory.Message, "List_of_root_folders_whose_combined_content_represents_the_structure_of_the_project_at_runtime_6168", "List of root folders whose combined content represents the structure of the project at runtime."),
            Show_all_compiler_options: diag(6169, ts.DiagnosticCategory.Message, "Show_all_compiler_options_6169", "Show all compiler options."),
            Deprecated_Use_outFile_instead_Concatenate_and_emit_output_to_single_file: diag(6170, ts.DiagnosticCategory.Message, "Deprecated_Use_outFile_instead_Concatenate_and_emit_output_to_single_file_6170", "[Deprecated] Use '--outFile' instead. Concatenate and emit output to single file"),
            Command_line_Options: diag(6171, ts.DiagnosticCategory.Message, "Command_line_Options_6171", "Command-line Options"),
            Basic_Options: diag(6172, ts.DiagnosticCategory.Message, "Basic_Options_6172", "Basic Options"),
            Strict_Type_Checking_Options: diag(6173, ts.DiagnosticCategory.Message, "Strict_Type_Checking_Options_6173", "Strict Type-Checking Options"),
            Module_Resolution_Options: diag(6174, ts.DiagnosticCategory.Message, "Module_Resolution_Options_6174", "Module Resolution Options"),
            Source_Map_Options: diag(6175, ts.DiagnosticCategory.Message, "Source_Map_Options_6175", "Source Map Options"),
            Additional_Checks: diag(6176, ts.DiagnosticCategory.Message, "Additional_Checks_6176", "Additional Checks"),
            Experimental_Options: diag(6177, ts.DiagnosticCategory.Message, "Experimental_Options_6177", "Experimental Options"),
            Advanced_Options: diag(6178, ts.DiagnosticCategory.Message, "Advanced_Options_6178", "Advanced Options"),
            Provide_full_support_for_iterables_in_for_of_spread_and_destructuring_when_targeting_ES5_or_ES3: diag(6179, ts.DiagnosticCategory.Message, "Provide_full_support_for_iterables_in_for_of_spread_and_destructuring_when_targeting_ES5_or_ES3_6179", "Provide full support for iterables in 'for-of', spread, and destructuring when targeting 'ES5' or 'ES3'."),
            Enable_all_strict_type_checking_options: diag(6180, ts.DiagnosticCategory.Message, "Enable_all_strict_type_checking_options_6180", "Enable all strict type-checking options."),
            List_of_language_service_plugins: diag(6181, ts.DiagnosticCategory.Message, "List_of_language_service_plugins_6181", "List of language service plugins."),
            Scoped_package_detected_looking_in_0: diag(6182, ts.DiagnosticCategory.Message, "Scoped_package_detected_looking_in_0_6182", "Scoped package detected, looking in '{0}'"),
            Reusing_resolution_of_module_0_to_file_1_from_old_program: diag(6183, ts.DiagnosticCategory.Message, "Reusing_resolution_of_module_0_to_file_1_from_old_program_6183", "Reusing resolution of module '{0}' to file '{1}' from old program."),
            Reusing_module_resolutions_originating_in_0_since_resolutions_are_unchanged_from_old_program: diag(6184, ts.DiagnosticCategory.Message, "Reusing_module_resolutions_originating_in_0_since_resolutions_are_unchanged_from_old_program_6184", "Reusing module resolutions originating in '{0}' since resolutions are unchanged from old program."),
            Disable_strict_checking_of_generic_signatures_in_function_types: diag(6185, ts.DiagnosticCategory.Message, "Disable_strict_checking_of_generic_signatures_in_function_types_6185", "Disable strict checking of generic signatures in function types."),
            Enable_strict_checking_of_function_types: diag(6186, ts.DiagnosticCategory.Message, "Enable_strict_checking_of_function_types_6186", "Enable strict checking of function types."),
            Enable_strict_checking_of_property_initialization_in_classes: diag(6187, ts.DiagnosticCategory.Message, "Enable_strict_checking_of_property_initialization_in_classes_6187", "Enable strict checking of property initialization in classes."),
            Numeric_separators_are_not_allowed_here: diag(6188, ts.DiagnosticCategory.Error, "Numeric_separators_are_not_allowed_here_6188", "Numeric separators are not allowed here."),
            Multiple_consecutive_numeric_separators_are_not_permitted: diag(6189, ts.DiagnosticCategory.Error, "Multiple_consecutive_numeric_separators_are_not_permitted_6189", "Multiple consecutive numeric separators are not permitted."),
            Whether_to_keep_outdated_console_output_in_watch_mode_instead_of_clearing_the_screen: diag(6191, ts.DiagnosticCategory.Message, "Whether_to_keep_outdated_console_output_in_watch_mode_instead_of_clearing_the_screen_6191", "Whether to keep outdated console output in watch mode instead of clearing the screen."),
            All_imports_in_import_declaration_are_unused: diag(6192, ts.DiagnosticCategory.Error, "All_imports_in_import_declaration_are_unused_6192", "All imports in import declaration are unused.", /*reportsUnnecessary*/ true),
            Found_1_error_Watching_for_file_changes: diag(6193, ts.DiagnosticCategory.Message, "Found_1_error_Watching_for_file_changes_6193", "Found 1 error. Watching for file changes."),
            Found_0_errors_Watching_for_file_changes: diag(6194, ts.DiagnosticCategory.Message, "Found_0_errors_Watching_for_file_changes_6194", "Found {0} errors. Watching for file changes."),
            Resolve_keyof_to_string_valued_property_names_only_no_numbers_or_symbols: diag(6195, ts.DiagnosticCategory.Message, "Resolve_keyof_to_string_valued_property_names_only_no_numbers_or_symbols_6195", "Resolve 'keyof' to string valued property names only (no numbers or symbols)."),
            _0_is_declared_but_never_used: diag(6196, ts.DiagnosticCategory.Error, "_0_is_declared_but_never_used_6196", "'{0}' is declared but never used.", /*reportsUnnecessary*/ true),
            Include_modules_imported_with_json_extension: diag(6197, ts.DiagnosticCategory.Message, "Include_modules_imported_with_json_extension_6197", "Include modules imported with '.json' extension"),
            All_destructured_elements_are_unused: diag(6198, ts.DiagnosticCategory.Error, "All_destructured_elements_are_unused_6198", "All destructured elements are unused.", /*reportsUnnecessary*/ true),
            All_variables_are_unused: diag(6199, ts.DiagnosticCategory.Error, "All_variables_are_unused_6199", "All variables are unused.", /*reportsUnnecessary*/ true),
            Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0: diag(6200, ts.DiagnosticCategory.Error, "Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0_6200", "Definitions of the following identifiers conflict with those in another file: {0}"),
            Conflicts_are_in_this_file: diag(6201, ts.DiagnosticCategory.Message, "Conflicts_are_in_this_file_6201", "Conflicts are in this file."),
            Project_references_may_not_form_a_circular_graph_Cycle_detected_Colon_0: diag(6202, ts.DiagnosticCategory.Error, "Project_references_may_not_form_a_circular_graph_Cycle_detected_Colon_0_6202", "Project references may not form a circular graph. Cycle detected: {0}"),
            _0_was_also_declared_here: diag(6203, ts.DiagnosticCategory.Message, "_0_was_also_declared_here_6203", "'{0}' was also declared here."),
            and_here: diag(6204, ts.DiagnosticCategory.Message, "and_here_6204", "and here."),
            All_type_parameters_are_unused: diag(6205, ts.DiagnosticCategory.Error, "All_type_parameters_are_unused_6205", "All type parameters are unused"),
            package_json_has_a_typesVersions_field_with_version_specific_path_mappings: diag(6206, ts.DiagnosticCategory.Message, "package_json_has_a_typesVersions_field_with_version_specific_path_mappings_6206", "'package.json' has a 'typesVersions' field with version-specific path mappings."),
            package_json_does_not_have_a_typesVersions_entry_that_matches_version_0: diag(6207, ts.DiagnosticCategory.Message, "package_json_does_not_have_a_typesVersions_entry_that_matches_version_0_6207", "'package.json' does not have a 'typesVersions' entry that matches version '{0}'."),
            package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_match_module_name_2: diag(6208, ts.DiagnosticCategory.Message, "package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_ma_6208", "'package.json' has a 'typesVersions' entry '{0}' that matches compiler version '{1}', looking for a pattern to match module name '{2}'."),
            package_json_has_a_typesVersions_entry_0_that_is_not_a_valid_semver_range: diag(6209, ts.DiagnosticCategory.Message, "package_json_has_a_typesVersions_entry_0_that_is_not_a_valid_semver_range_6209", "'package.json' has a 'typesVersions' entry '{0}' that is not a valid semver range."),
            An_argument_for_0_was_not_provided: diag(6210, ts.DiagnosticCategory.Message, "An_argument_for_0_was_not_provided_6210", "An argument for '{0}' was not provided."),
            An_argument_matching_this_binding_pattern_was_not_provided: diag(6211, ts.DiagnosticCategory.Message, "An_argument_matching_this_binding_pattern_was_not_provided_6211", "An argument matching this binding pattern was not provided."),
            Did_you_mean_to_call_this_expression: diag(6212, ts.DiagnosticCategory.Message, "Did_you_mean_to_call_this_expression_6212", "Did you mean to call this expression?"),
            Did_you_mean_to_use_new_with_this_expression: diag(6213, ts.DiagnosticCategory.Message, "Did_you_mean_to_use_new_with_this_expression_6213", "Did you mean to use 'new' with this expression?"),
            Enable_strict_bind_call_and_apply_methods_on_functions: diag(6214, ts.DiagnosticCategory.Message, "Enable_strict_bind_call_and_apply_methods_on_functions_6214", "Enable strict 'bind', 'call', and 'apply' methods on functions."),
            Using_compiler_options_of_project_reference_redirect_0: diag(6215, ts.DiagnosticCategory.Message, "Using_compiler_options_of_project_reference_redirect_0_6215", "Using compiler options of project reference redirect '{0}'."),
            Found_1_error: diag(6216, ts.DiagnosticCategory.Message, "Found_1_error_6216", "Found 1 error."),
            Found_0_errors: diag(6217, ts.DiagnosticCategory.Message, "Found_0_errors_6217", "Found {0} errors."),
            Module_name_0_was_successfully_resolved_to_1_with_Package_ID_2: diag(6218, ts.DiagnosticCategory.Message, "Module_name_0_was_successfully_resolved_to_1_with_Package_ID_2_6218", "======== Module name '{0}' was successfully resolved to '{1}' with Package ID '{2}'. ========"),
            Type_reference_directive_0_was_successfully_resolved_to_1_with_Package_ID_2_primary_Colon_3: diag(6219, ts.DiagnosticCategory.Message, "Type_reference_directive_0_was_successfully_resolved_to_1_with_Package_ID_2_primary_Colon_3_6219", "======== Type reference directive '{0}' was successfully resolved to '{1}' with Package ID '{2}', primary: {3}. ========"),
            package_json_had_a_falsy_0_field: diag(6220, ts.DiagnosticCategory.Message, "package_json_had_a_falsy_0_field_6220", "'package.json' had a falsy '{0}' field."),
            Disable_use_of_source_files_instead_of_declaration_files_from_referenced_projects: diag(6221, ts.DiagnosticCategory.Message, "Disable_use_of_source_files_instead_of_declaration_files_from_referenced_projects_6221", "Disable use of source files instead of declaration files from referenced projects."),
            Emit_class_fields_with_Define_instead_of_Set: diag(6222, ts.DiagnosticCategory.Message, "Emit_class_fields_with_Define_instead_of_Set_6222", "Emit class fields with Define instead of Set."),
            Generates_a_CPU_profile: diag(6223, ts.DiagnosticCategory.Message, "Generates_a_CPU_profile_6223", "Generates a CPU profile."),
            Disable_solution_searching_for_this_project: diag(6224, ts.DiagnosticCategory.Message, "Disable_solution_searching_for_this_project_6224", "Disable solution searching for this project."),
            Specify_strategy_for_watching_file_Colon_FixedPollingInterval_default_PriorityPollingInterval_DynamicPriorityPolling_UseFsEvents_UseFsEventsOnParentDirectory: diag(6225, ts.DiagnosticCategory.Message, "Specify_strategy_for_watching_file_Colon_FixedPollingInterval_default_PriorityPollingInterval_Dynami_6225", "Specify strategy for watching file: 'FixedPollingInterval' (default), 'PriorityPollingInterval', 'DynamicPriorityPolling', 'UseFsEvents', 'UseFsEventsOnParentDirectory'."),
            Specify_strategy_for_watching_directory_on_platforms_that_don_t_support_recursive_watching_natively_Colon_UseFsEvents_default_FixedPollingInterval_DynamicPriorityPolling: diag(6226, ts.DiagnosticCategory.Message, "Specify_strategy_for_watching_directory_on_platforms_that_don_t_support_recursive_watching_natively__6226", "Specify strategy for watching directory on platforms that don't support recursive watching natively: 'UseFsEvents' (default), 'FixedPollingInterval', 'DynamicPriorityPolling'."),
            Specify_strategy_for_creating_a_polling_watch_when_it_fails_to_create_using_file_system_events_Colon_FixedInterval_default_PriorityInterval_DynamicPriority: diag(6227, ts.DiagnosticCategory.Message, "Specify_strategy_for_creating_a_polling_watch_when_it_fails_to_create_using_file_system_events_Colon_6227", "Specify strategy for creating a polling watch when it fails to create using file system events: 'FixedInterval' (default), 'PriorityInterval', 'DynamicPriority'."),
            Synchronously_call_callbacks_and_update_the_state_of_directory_watchers_on_platforms_that_don_t_support_recursive_watching_natively: diag(6228, ts.DiagnosticCategory.Message, "Synchronously_call_callbacks_and_update_the_state_of_directory_watchers_on_platforms_that_don_t_supp_6228", "Synchronously call callbacks and update the state of directory watchers on platforms that don't support recursive watching natively."),
            Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_false_or_null_on_command_line: diag(6230, ts.DiagnosticCategory.Error, "Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_false_or_null_on_command_line_6230", "Option '{0}' can only be specified in 'tsconfig.json' file or set to 'false' or 'null' on command line."),
            Projects_to_reference: diag(6300, ts.DiagnosticCategory.Message, "Projects_to_reference_6300", "Projects to reference"),
            Enable_project_compilation: diag(6302, ts.DiagnosticCategory.Message, "Enable_project_compilation_6302", "Enable project compilation"),
            Composite_projects_may_not_disable_declaration_emit: diag(6304, ts.DiagnosticCategory.Error, "Composite_projects_may_not_disable_declaration_emit_6304", "Composite projects may not disable declaration emit."),
            Output_file_0_has_not_been_built_from_source_file_1: diag(6305, ts.DiagnosticCategory.Error, "Output_file_0_has_not_been_built_from_source_file_1_6305", "Output file '{0}' has not been built from source file '{1}'."),
            Referenced_project_0_must_have_setting_composite_Colon_true: diag(6306, ts.DiagnosticCategory.Error, "Referenced_project_0_must_have_setting_composite_Colon_true_6306", "Referenced project '{0}' must have setting \"composite\": true."),
            File_0_is_not_listed_within_the_file_list_of_project_1_Projects_must_list_all_files_or_use_an_include_pattern: diag(6307, ts.DiagnosticCategory.Error, "File_0_is_not_listed_within_the_file_list_of_project_1_Projects_must_list_all_files_or_use_an_includ_6307", "File '{0}' is not listed within the file list of project '{1}'. Projects must list all files or use an 'include' pattern."),
            Cannot_prepend_project_0_because_it_does_not_have_outFile_set: diag(6308, ts.DiagnosticCategory.Error, "Cannot_prepend_project_0_because_it_does_not_have_outFile_set_6308", "Cannot prepend project '{0}' because it does not have 'outFile' set"),
            Output_file_0_from_project_1_does_not_exist: diag(6309, ts.DiagnosticCategory.Error, "Output_file_0_from_project_1_does_not_exist_6309", "Output file '{0}' from project '{1}' does not exist"),
            Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2: diag(6350, ts.DiagnosticCategory.Message, "Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2_6350", "Project '{0}' is out of date because oldest output '{1}' is older than newest input '{2}'"),
            Project_0_is_up_to_date_because_newest_input_1_is_older_than_oldest_output_2: diag(6351, ts.DiagnosticCategory.Message, "Project_0_is_up_to_date_because_newest_input_1_is_older_than_oldest_output_2_6351", "Project '{0}' is up to date because newest input '{1}' is older than oldest output '{2}'"),
            Project_0_is_out_of_date_because_output_file_1_does_not_exist: diag(6352, ts.DiagnosticCategory.Message, "Project_0_is_out_of_date_because_output_file_1_does_not_exist_6352", "Project '{0}' is out of date because output file '{1}' does not exist"),
            Project_0_is_out_of_date_because_its_dependency_1_is_out_of_date: diag(6353, ts.DiagnosticCategory.Message, "Project_0_is_out_of_date_because_its_dependency_1_is_out_of_date_6353", "Project '{0}' is out of date because its dependency '{1}' is out of date"),
            Project_0_is_up_to_date_with_d_ts_files_from_its_dependencies: diag(6354, ts.DiagnosticCategory.Message, "Project_0_is_up_to_date_with_d_ts_files_from_its_dependencies_6354", "Project '{0}' is up to date with .d.ts files from its dependencies"),
            Projects_in_this_build_Colon_0: diag(6355, ts.DiagnosticCategory.Message, "Projects_in_this_build_Colon_0_6355", "Projects in this build: {0}"),
            A_non_dry_build_would_delete_the_following_files_Colon_0: diag(6356, ts.DiagnosticCategory.Message, "A_non_dry_build_would_delete_the_following_files_Colon_0_6356", "A non-dry build would delete the following files: {0}"),
            A_non_dry_build_would_build_project_0: diag(6357, ts.DiagnosticCategory.Message, "A_non_dry_build_would_build_project_0_6357", "A non-dry build would build project '{0}'"),
            Building_project_0: diag(6358, ts.DiagnosticCategory.Message, "Building_project_0_6358", "Building project '{0}'..."),
            Updating_output_timestamps_of_project_0: diag(6359, ts.DiagnosticCategory.Message, "Updating_output_timestamps_of_project_0_6359", "Updating output timestamps of project '{0}'..."),
            delete_this_Project_0_is_up_to_date_because_it_was_previously_built: diag(6360, ts.DiagnosticCategory.Message, "delete_this_Project_0_is_up_to_date_because_it_was_previously_built_6360", "delete this - Project '{0}' is up to date because it was previously built"),
            Project_0_is_up_to_date: diag(6361, ts.DiagnosticCategory.Message, "Project_0_is_up_to_date_6361", "Project '{0}' is up to date"),
            Skipping_build_of_project_0_because_its_dependency_1_has_errors: diag(6362, ts.DiagnosticCategory.Message, "Skipping_build_of_project_0_because_its_dependency_1_has_errors_6362", "Skipping build of project '{0}' because its dependency '{1}' has errors"),
            Project_0_can_t_be_built_because_its_dependency_1_has_errors: diag(6363, ts.DiagnosticCategory.Message, "Project_0_can_t_be_built_because_its_dependency_1_has_errors_6363", "Project '{0}' can't be built because its dependency '{1}' has errors"),
            Build_one_or_more_projects_and_their_dependencies_if_out_of_date: diag(6364, ts.DiagnosticCategory.Message, "Build_one_or_more_projects_and_their_dependencies_if_out_of_date_6364", "Build one or more projects and their dependencies, if out of date"),
            Delete_the_outputs_of_all_projects: diag(6365, ts.DiagnosticCategory.Message, "Delete_the_outputs_of_all_projects_6365", "Delete the outputs of all projects"),
            Enable_verbose_logging: diag(6366, ts.DiagnosticCategory.Message, "Enable_verbose_logging_6366", "Enable verbose logging"),
            Show_what_would_be_built_or_deleted_if_specified_with_clean: diag(6367, ts.DiagnosticCategory.Message, "Show_what_would_be_built_or_deleted_if_specified_with_clean_6367", "Show what would be built (or deleted, if specified with '--clean')"),
            Build_all_projects_including_those_that_appear_to_be_up_to_date: diag(6368, ts.DiagnosticCategory.Message, "Build_all_projects_including_those_that_appear_to_be_up_to_date_6368", "Build all projects, including those that appear to be up to date"),
            Option_build_must_be_the_first_command_line_argument: diag(6369, ts.DiagnosticCategory.Error, "Option_build_must_be_the_first_command_line_argument_6369", "Option '--build' must be the first command line argument."),
            Options_0_and_1_cannot_be_combined: diag(6370, ts.DiagnosticCategory.Error, "Options_0_and_1_cannot_be_combined_6370", "Options '{0}' and '{1}' cannot be combined."),
            Updating_unchanged_output_timestamps_of_project_0: diag(6371, ts.DiagnosticCategory.Message, "Updating_unchanged_output_timestamps_of_project_0_6371", "Updating unchanged output timestamps of project '{0}'..."),
            Project_0_is_out_of_date_because_output_of_its_dependency_1_has_changed: diag(6372, ts.DiagnosticCategory.Message, "Project_0_is_out_of_date_because_output_of_its_dependency_1_has_changed_6372", "Project '{0}' is out of date because output of its dependency '{1}' has changed"),
            Updating_output_of_project_0: diag(6373, ts.DiagnosticCategory.Message, "Updating_output_of_project_0_6373", "Updating output of project '{0}'..."),
            A_non_dry_build_would_update_timestamps_for_output_of_project_0: diag(6374, ts.DiagnosticCategory.Message, "A_non_dry_build_would_update_timestamps_for_output_of_project_0_6374", "A non-dry build would update timestamps for output of project '{0}'"),
            A_non_dry_build_would_update_output_of_project_0: diag(6375, ts.DiagnosticCategory.Message, "A_non_dry_build_would_update_output_of_project_0_6375", "A non-dry build would update output of project '{0}'"),
            Cannot_update_output_of_project_0_because_there_was_error_reading_file_1: diag(6376, ts.DiagnosticCategory.Message, "Cannot_update_output_of_project_0_because_there_was_error_reading_file_1_6376", "Cannot update output of project '{0}' because there was error reading file '{1}'"),
            Cannot_write_file_0_because_it_will_overwrite_tsbuildinfo_file_generated_by_referenced_project_1: diag(6377, ts.DiagnosticCategory.Error, "Cannot_write_file_0_because_it_will_overwrite_tsbuildinfo_file_generated_by_referenced_project_1_6377", "Cannot write file '{0}' because it will overwrite '.tsbuildinfo' file generated by referenced project '{1}'"),
            Enable_incremental_compilation: diag(6378, ts.DiagnosticCategory.Message, "Enable_incremental_compilation_6378", "Enable incremental compilation"),
            Composite_projects_may_not_disable_incremental_compilation: diag(6379, ts.DiagnosticCategory.Error, "Composite_projects_may_not_disable_incremental_compilation_6379", "Composite projects may not disable incremental compilation."),
            Specify_file_to_store_incremental_compilation_information: diag(6380, ts.DiagnosticCategory.Message, "Specify_file_to_store_incremental_compilation_information_6380", "Specify file to store incremental compilation information"),
            Project_0_is_out_of_date_because_output_for_it_was_generated_with_version_1_that_differs_with_current_version_2: diag(6381, ts.DiagnosticCategory.Message, "Project_0_is_out_of_date_because_output_for_it_was_generated_with_version_1_that_differs_with_curren_6381", "Project '{0}' is out of date because output for it was generated with version '{1}' that differs with current version '{2}'"),
            Skipping_build_of_project_0_because_its_dependency_1_was_not_built: diag(6382, ts.DiagnosticCategory.Message, "Skipping_build_of_project_0_because_its_dependency_1_was_not_built_6382", "Skipping build of project '{0}' because its dependency '{1}' was not built"),
            Project_0_can_t_be_built_because_its_dependency_1_was_not_built: diag(6383, ts.DiagnosticCategory.Message, "Project_0_can_t_be_built_because_its_dependency_1_was_not_built_6383", "Project '{0}' can't be built because its dependency '{1}' was not built"),
            Have_recompiles_in_incremental_and_watch_assume_that_changes_within_a_file_will_only_affect_files_directly_depending_on_it: diag(6384, ts.DiagnosticCategory.Message, "Have_recompiles_in_incremental_and_watch_assume_that_changes_within_a_file_will_only_affect_files_di_6384", "Have recompiles in '--incremental' and '--watch' assume that changes within a file will only affect files directly depending on it."),
            The_expected_type_comes_from_property_0_which_is_declared_here_on_type_1: diag(6500, ts.DiagnosticCategory.Message, "The_expected_type_comes_from_property_0_which_is_declared_here_on_type_1_6500", "The expected type comes from property '{0}' which is declared here on type '{1}'"),
            The_expected_type_comes_from_this_index_signature: diag(6501, ts.DiagnosticCategory.Message, "The_expected_type_comes_from_this_index_signature_6501", "The expected type comes from this index signature."),
            The_expected_type_comes_from_the_return_type_of_this_signature: diag(6502, ts.DiagnosticCategory.Message, "The_expected_type_comes_from_the_return_type_of_this_signature_6502", "The expected type comes from the return type of this signature."),
            Print_names_of_files_that_are_part_of_the_compilation_and_then_stop_processing: diag(6503, ts.DiagnosticCategory.Message, "Print_names_of_files_that_are_part_of_the_compilation_and_then_stop_processing_6503", "Print names of files that are part of the compilation and then stop processing."),
            File_0_is_a_JavaScript_file_Did_you_mean_to_enable_the_allowJs_option: diag(6504, ts.DiagnosticCategory.Error, "File_0_is_a_JavaScript_file_Did_you_mean_to_enable_the_allowJs_option_6504", "File '{0}' is a JavaScript file. Did you mean to enable the 'allowJs' option?"),
            Variable_0_implicitly_has_an_1_type: diag(7005, ts.DiagnosticCategory.Error, "Variable_0_implicitly_has_an_1_type_7005", "Variable '{0}' implicitly has an '{1}' type."),
            Parameter_0_implicitly_has_an_1_type: diag(7006, ts.DiagnosticCategory.Error, "Parameter_0_implicitly_has_an_1_type_7006", "Parameter '{0}' implicitly has an '{1}' type."),
            Member_0_implicitly_has_an_1_type: diag(7008, ts.DiagnosticCategory.Error, "Member_0_implicitly_has_an_1_type_7008", "Member '{0}' implicitly has an '{1}' type."),
            new_expression_whose_target_lacks_a_construct_signature_implicitly_has_an_any_type: diag(7009, ts.DiagnosticCategory.Error, "new_expression_whose_target_lacks_a_construct_signature_implicitly_has_an_any_type_7009", "'new' expression, whose target lacks a construct signature, implicitly has an 'any' type."),
            _0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type: diag(7010, ts.DiagnosticCategory.Error, "_0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type_7010", "'{0}', which lacks return-type annotation, implicitly has an '{1}' return type."),
            Function_expression_which_lacks_return_type_annotation_implicitly_has_an_0_return_type: diag(7011, ts.DiagnosticCategory.Error, "Function_expression_which_lacks_return_type_annotation_implicitly_has_an_0_return_type_7011", "Function expression, which lacks return-type annotation, implicitly has an '{0}' return type."),
            Construct_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type: diag(7013, ts.DiagnosticCategory.Error, "Construct_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type_7013", "Construct signature, which lacks return-type annotation, implicitly has an 'any' return type."),
            Function_type_which_lacks_return_type_annotation_implicitly_has_an_0_return_type: diag(7014, ts.DiagnosticCategory.Error, "Function_type_which_lacks_return_type_annotation_implicitly_has_an_0_return_type_7014", "Function type, which lacks return-type annotation, implicitly has an '{0}' return type."),
            Element_implicitly_has_an_any_type_because_index_expression_is_not_of_type_number: diag(7015, ts.DiagnosticCategory.Error, "Element_implicitly_has_an_any_type_because_index_expression_is_not_of_type_number_7015", "Element implicitly has an 'any' type because index expression is not of type 'number'."),
            Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type: diag(7016, ts.DiagnosticCategory.Error, "Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type_7016", "Could not find a declaration file for module '{0}'. '{1}' implicitly has an 'any' type."),
            Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature: diag(7017, ts.DiagnosticCategory.Error, "Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature_7017", "Element implicitly has an 'any' type because type '{0}' has no index signature."),
            Object_literal_s_property_0_implicitly_has_an_1_type: diag(7018, ts.DiagnosticCategory.Error, "Object_literal_s_property_0_implicitly_has_an_1_type_7018", "Object literal's property '{0}' implicitly has an '{1}' type."),
            Rest_parameter_0_implicitly_has_an_any_type: diag(7019, ts.DiagnosticCategory.Error, "Rest_parameter_0_implicitly_has_an_any_type_7019", "Rest parameter '{0}' implicitly has an 'any[]' type."),
            Call_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type: diag(7020, ts.DiagnosticCategory.Error, "Call_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type_7020", "Call signature, which lacks return-type annotation, implicitly has an 'any' return type."),
            _0_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_and_is_referenced_directly_or_indirectly_in_its_own_initializer: diag(7022, ts.DiagnosticCategory.Error, "_0_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_and_is_referenced_directly_or__7022", "'{0}' implicitly has type 'any' because it does not have a type annotation and is referenced directly or indirectly in its own initializer."),
            _0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions: diag(7023, ts.DiagnosticCategory.Error, "_0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_reference_7023", "'{0}' implicitly has return type 'any' because it does not have a return type annotation and is referenced directly or indirectly in one of its return expressions."),
            Function_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions: diag(7024, ts.DiagnosticCategory.Error, "Function_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_ref_7024", "Function implicitly has return type 'any' because it does not have a return type annotation and is referenced directly or indirectly in one of its return expressions."),
            Generator_implicitly_has_yield_type_0_because_it_does_not_yield_any_values_Consider_supplying_a_return_type_annotation: diag(7025, ts.DiagnosticCategory.Error, "Generator_implicitly_has_yield_type_0_because_it_does_not_yield_any_values_Consider_supplying_a_retu_7025", "Generator implicitly has yield type '{0}' because it does not yield any values. Consider supplying a return type annotation."),
            JSX_element_implicitly_has_type_any_because_no_interface_JSX_0_exists: diag(7026, ts.DiagnosticCategory.Error, "JSX_element_implicitly_has_type_any_because_no_interface_JSX_0_exists_7026", "JSX element implicitly has type 'any' because no interface 'JSX.{0}' exists."),
            Unreachable_code_detected: diag(7027, ts.DiagnosticCategory.Error, "Unreachable_code_detected_7027", "Unreachable code detected.", /*reportsUnnecessary*/ true),
            Unused_label: diag(7028, ts.DiagnosticCategory.Error, "Unused_label_7028", "Unused label.", /*reportsUnnecessary*/ true),
            Fallthrough_case_in_switch: diag(7029, ts.DiagnosticCategory.Error, "Fallthrough_case_in_switch_7029", "Fallthrough case in switch."),
            Not_all_code_paths_return_a_value: diag(7030, ts.DiagnosticCategory.Error, "Not_all_code_paths_return_a_value_7030", "Not all code paths return a value."),
            Binding_element_0_implicitly_has_an_1_type: diag(7031, ts.DiagnosticCategory.Error, "Binding_element_0_implicitly_has_an_1_type_7031", "Binding element '{0}' implicitly has an '{1}' type."),
            Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation: diag(7032, ts.DiagnosticCategory.Error, "Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation_7032", "Property '{0}' implicitly has type 'any', because its set accessor lacks a parameter type annotation."),
            Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation: diag(7033, ts.DiagnosticCategory.Error, "Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation_7033", "Property '{0}' implicitly has type 'any', because its get accessor lacks a return type annotation."),
            Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined: diag(7034, ts.DiagnosticCategory.Error, "Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined_7034", "Variable '{0}' implicitly has type '{1}' in some locations where its type cannot be determined."),
            Try_npm_install_types_Slash_1_if_it_exists_or_add_a_new_declaration_d_ts_file_containing_declare_module_0: diag(7035, ts.DiagnosticCategory.Error, "Try_npm_install_types_Slash_1_if_it_exists_or_add_a_new_declaration_d_ts_file_containing_declare_mod_7035", "Try `npm install @types/{1}` if it exists or add a new declaration (.d.ts) file containing `declare module '{0}';`"),
            Dynamic_import_s_specifier_must_be_of_type_string_but_here_has_type_0: diag(7036, ts.DiagnosticCategory.Error, "Dynamic_import_s_specifier_must_be_of_type_string_but_here_has_type_0_7036", "Dynamic import's specifier must be of type 'string', but here has type '{0}'."),
            Enables_emit_interoperability_between_CommonJS_and_ES_Modules_via_creation_of_namespace_objects_for_all_imports_Implies_allowSyntheticDefaultImports: diag(7037, ts.DiagnosticCategory.Message, "Enables_emit_interoperability_between_CommonJS_and_ES_Modules_via_creation_of_namespace_objects_for__7037", "Enables emit interoperability between CommonJS and ES Modules via creation of namespace objects for all imports. Implies 'allowSyntheticDefaultImports'."),
            Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cause_a_failure_at_runtime_Consider_using_a_default_import_or_import_require_here_instead: diag(7038, ts.DiagnosticCategory.Message, "Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cau_7038", "Type originates at this import. A namespace-style import cannot be called or constructed, and will cause a failure at runtime. Consider using a default import or import require here instead."),
            Mapped_object_type_implicitly_has_an_any_template_type: diag(7039, ts.DiagnosticCategory.Error, "Mapped_object_type_implicitly_has_an_any_template_type_7039", "Mapped object type implicitly has an 'any' template type."),
            If_the_0_package_actually_exposes_this_module_consider_sending_a_pull_request_to_amend_https_Colon_Slash_Slashgithub_com_SlashDefinitelyTyped_SlashDefinitelyTyped_Slashtree_Slashmaster_Slashtypes_Slash_1: diag(7040, ts.DiagnosticCategory.Error, "If_the_0_package_actually_exposes_this_module_consider_sending_a_pull_request_to_amend_https_Colon_S_7040", "If the '{0}' package actually exposes this module, consider sending a pull request to amend 'https://github.com/DefinitelyTyped/DefinitelyTyped/tree/master/types/{1}`"),
            The_containing_arrow_function_captures_the_global_value_of_this: diag(7041, ts.DiagnosticCategory.Error, "The_containing_arrow_function_captures_the_global_value_of_this_7041", "The containing arrow function captures the global value of 'this'."),
            Module_0_was_resolved_to_1_but_resolveJsonModule_is_not_used: diag(7042, ts.DiagnosticCategory.Error, "Module_0_was_resolved_to_1_but_resolveJsonModule_is_not_used_7042", "Module '{0}' was resolved to '{1}', but '--resolveJsonModule' is not used."),
            Variable_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage: diag(7043, ts.DiagnosticCategory.Suggestion, "Variable_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage_7043", "Variable '{0}' implicitly has an '{1}' type, but a better type may be inferred from usage."),
            Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage: diag(7044, ts.DiagnosticCategory.Suggestion, "Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage_7044", "Parameter '{0}' implicitly has an '{1}' type, but a better type may be inferred from usage."),
            Member_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage: diag(7045, ts.DiagnosticCategory.Suggestion, "Member_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage_7045", "Member '{0}' implicitly has an '{1}' type, but a better type may be inferred from usage."),
            Variable_0_implicitly_has_type_1_in_some_locations_but_a_better_type_may_be_inferred_from_usage: diag(7046, ts.DiagnosticCategory.Suggestion, "Variable_0_implicitly_has_type_1_in_some_locations_but_a_better_type_may_be_inferred_from_usage_7046", "Variable '{0}' implicitly has type '{1}' in some locations, but a better type may be inferred from usage."),
            Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage: diag(7047, ts.DiagnosticCategory.Suggestion, "Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage_7047", "Rest parameter '{0}' implicitly has an 'any[]' type, but a better type may be inferred from usage."),
            Property_0_implicitly_has_type_any_but_a_better_type_for_its_get_accessor_may_be_inferred_from_usage: diag(7048, ts.DiagnosticCategory.Suggestion, "Property_0_implicitly_has_type_any_but_a_better_type_for_its_get_accessor_may_be_inferred_from_usage_7048", "Property '{0}' implicitly has type 'any', but a better type for its get accessor may be inferred from usage."),
            Property_0_implicitly_has_type_any_but_a_better_type_for_its_set_accessor_may_be_inferred_from_usage: diag(7049, ts.DiagnosticCategory.Suggestion, "Property_0_implicitly_has_type_any_but_a_better_type_for_its_set_accessor_may_be_inferred_from_usage_7049", "Property '{0}' implicitly has type 'any', but a better type for its set accessor may be inferred from usage."),
            _0_implicitly_has_an_1_return_type_but_a_better_type_may_be_inferred_from_usage: diag(7050, ts.DiagnosticCategory.Suggestion, "_0_implicitly_has_an_1_return_type_but_a_better_type_may_be_inferred_from_usage_7050", "'{0}' implicitly has an '{1}' return type, but a better type may be inferred from usage."),
            Parameter_has_a_name_but_no_type_Did_you_mean_0_Colon_1: diag(7051, ts.DiagnosticCategory.Error, "Parameter_has_a_name_but_no_type_Did_you_mean_0_Colon_1_7051", "Parameter has a name but no type. Did you mean '{0}: {1}'?"),
            Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature_Did_you_mean_to_call_1: diag(7052, ts.DiagnosticCategory.Error, "Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature_Did_you_mean_to_call_1_7052", "Element implicitly has an 'any' type because type '{0}' has no index signature. Did you mean to call '{1}'?"),
            Element_implicitly_has_an_any_type_because_expression_of_type_0_can_t_be_used_to_index_type_1: diag(7053, ts.DiagnosticCategory.Error, "Element_implicitly_has_an_any_type_because_expression_of_type_0_can_t_be_used_to_index_type_1_7053", "Element implicitly has an 'any' type because expression of type '{0}' can't be used to index type '{1}'."),
            No_index_signature_with_a_parameter_of_type_0_was_found_on_type_1: diag(7054, ts.DiagnosticCategory.Error, "No_index_signature_with_a_parameter_of_type_0_was_found_on_type_1_7054", "No index signature with a parameter of type '{0}' was found on type '{1}'."),
            _0_which_lacks_return_type_annotation_implicitly_has_an_1_yield_type: diag(7055, ts.DiagnosticCategory.Error, "_0_which_lacks_return_type_annotation_implicitly_has_an_1_yield_type_7055", "'{0}', which lacks return-type annotation, implicitly has an '{1}' yield type."),
            You_cannot_rename_this_element: diag(8000, ts.DiagnosticCategory.Error, "You_cannot_rename_this_element_8000", "You cannot rename this element."),
            You_cannot_rename_elements_that_are_defined_in_the_standard_TypeScript_library: diag(8001, ts.DiagnosticCategory.Error, "You_cannot_rename_elements_that_are_defined_in_the_standard_TypeScript_library_8001", "You cannot rename elements that are defined in the standard TypeScript library."),
            import_can_only_be_used_in_TypeScript_files: diag(8002, ts.DiagnosticCategory.Error, "import_can_only_be_used_in_TypeScript_files_8002", "'import ... =' can only be used in TypeScript files."),
            export_can_only_be_used_in_TypeScript_files: diag(8003, ts.DiagnosticCategory.Error, "export_can_only_be_used_in_TypeScript_files_8003", "'export =' can only be used in TypeScript files."),
            Type_parameter_declarations_can_only_be_used_in_TypeScript_files: diag(8004, ts.DiagnosticCategory.Error, "Type_parameter_declarations_can_only_be_used_in_TypeScript_files_8004", "Type parameter declarations can only be used in TypeScript files."),
            implements_clauses_can_only_be_used_in_TypeScript_files: diag(8005, ts.DiagnosticCategory.Error, "implements_clauses_can_only_be_used_in_TypeScript_files_8005", "'implements' clauses can only be used in TypeScript files."),
            _0_declarations_can_only_be_used_in_TypeScript_files: diag(8006, ts.DiagnosticCategory.Error, "_0_declarations_can_only_be_used_in_TypeScript_files_8006", "'{0}' declarations can only be used in TypeScript files."),
            Type_aliases_can_only_be_used_in_TypeScript_files: diag(8008, ts.DiagnosticCategory.Error, "Type_aliases_can_only_be_used_in_TypeScript_files_8008", "Type aliases can only be used in TypeScript files."),
            The_0_modifier_can_only_be_used_in_TypeScript_files: diag(8009, ts.DiagnosticCategory.Error, "The_0_modifier_can_only_be_used_in_TypeScript_files_8009", "The '{0}' modifier can only be used in TypeScript files."),
            Type_annotations_can_only_be_used_in_TypeScript_files: diag(8010, ts.DiagnosticCategory.Error, "Type_annotations_can_only_be_used_in_TypeScript_files_8010", "Type annotations can only be used in TypeScript files."),
            Type_arguments_can_only_be_used_in_TypeScript_files: diag(8011, ts.DiagnosticCategory.Error, "Type_arguments_can_only_be_used_in_TypeScript_files_8011", "Type arguments can only be used in TypeScript files."),
            Parameter_modifiers_can_only_be_used_in_TypeScript_files: diag(8012, ts.DiagnosticCategory.Error, "Parameter_modifiers_can_only_be_used_in_TypeScript_files_8012", "Parameter modifiers can only be used in TypeScript files."),
            Non_null_assertions_can_only_be_used_in_TypeScript_files: diag(8013, ts.DiagnosticCategory.Error, "Non_null_assertions_can_only_be_used_in_TypeScript_files_8013", "Non-null assertions can only be used in TypeScript files."),
            Type_assertion_expressions_can_only_be_used_in_TypeScript_files: diag(8016, ts.DiagnosticCategory.Error, "Type_assertion_expressions_can_only_be_used_in_TypeScript_files_8016", "Type assertion expressions can only be used in TypeScript files."),
            Octal_literal_types_must_use_ES2015_syntax_Use_the_syntax_0: diag(8017, ts.DiagnosticCategory.Error, "Octal_literal_types_must_use_ES2015_syntax_Use_the_syntax_0_8017", "Octal literal types must use ES2015 syntax. Use the syntax '{0}'."),
            Octal_literals_are_not_allowed_in_enums_members_initializer_Use_the_syntax_0: diag(8018, ts.DiagnosticCategory.Error, "Octal_literals_are_not_allowed_in_enums_members_initializer_Use_the_syntax_0_8018", "Octal literals are not allowed in enums members initializer. Use the syntax '{0}'."),
            Report_errors_in_js_files: diag(8019, ts.DiagnosticCategory.Message, "Report_errors_in_js_files_8019", "Report errors in .js files."),
            JSDoc_types_can_only_be_used_inside_documentation_comments: diag(8020, ts.DiagnosticCategory.Error, "JSDoc_types_can_only_be_used_inside_documentation_comments_8020", "JSDoc types can only be used inside documentation comments."),
            JSDoc_typedef_tag_should_either_have_a_type_annotation_or_be_followed_by_property_or_member_tags: diag(8021, ts.DiagnosticCategory.Error, "JSDoc_typedef_tag_should_either_have_a_type_annotation_or_be_followed_by_property_or_member_tags_8021", "JSDoc '@typedef' tag should either have a type annotation or be followed by '@property' or '@member' tags."),
            JSDoc_0_is_not_attached_to_a_class: diag(8022, ts.DiagnosticCategory.Error, "JSDoc_0_is_not_attached_to_a_class_8022", "JSDoc '@{0}' is not attached to a class."),
            JSDoc_0_1_does_not_match_the_extends_2_clause: diag(8023, ts.DiagnosticCategory.Error, "JSDoc_0_1_does_not_match_the_extends_2_clause_8023", "JSDoc '@{0} {1}' does not match the 'extends {2}' clause."),
            JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name: diag(8024, ts.DiagnosticCategory.Error, "JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_8024", "JSDoc '@param' tag has name '{0}', but there is no parameter with that name."),
            Class_declarations_cannot_have_more_than_one_augments_or_extends_tag: diag(8025, ts.DiagnosticCategory.Error, "Class_declarations_cannot_have_more_than_one_augments_or_extends_tag_8025", "Class declarations cannot have more than one `@augments` or `@extends` tag."),
            Expected_0_type_arguments_provide_these_with_an_extends_tag: diag(8026, ts.DiagnosticCategory.Error, "Expected_0_type_arguments_provide_these_with_an_extends_tag_8026", "Expected {0} type arguments; provide these with an '@extends' tag."),
            Expected_0_1_type_arguments_provide_these_with_an_extends_tag: diag(8027, ts.DiagnosticCategory.Error, "Expected_0_1_type_arguments_provide_these_with_an_extends_tag_8027", "Expected {0}-{1} type arguments; provide these with an '@extends' tag."),
            JSDoc_may_only_appear_in_the_last_parameter_of_a_signature: diag(8028, ts.DiagnosticCategory.Error, "JSDoc_may_only_appear_in_the_last_parameter_of_a_signature_8028", "JSDoc '...' may only appear in the last parameter of a signature."),
            JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_It_would_match_arguments_if_it_had_an_array_type: diag(8029, ts.DiagnosticCategory.Error, "JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_It_would_match_arguments_if_it_h_8029", "JSDoc '@param' tag has name '{0}', but there is no parameter with that name. It would match 'arguments' if it had an array type."),
            The_type_of_a_function_declaration_must_match_the_function_s_signature: diag(8030, ts.DiagnosticCategory.Error, "The_type_of_a_function_declaration_must_match_the_function_s_signature_8030", "The type of a function declaration must match the function's signature."),
            You_cannot_rename_a_module_via_a_global_import: diag(8031, ts.DiagnosticCategory.Error, "You_cannot_rename_a_module_via_a_global_import_8031", "You cannot rename a module via a global import."),
            Qualified_name_0_is_not_allowed_without_a_leading_param_object_1: diag(8032, ts.DiagnosticCategory.Error, "Qualified_name_0_is_not_allowed_without_a_leading_param_object_1_8032", "Qualified name '{0}' is not allowed without a leading '@param {object} {1}'."),
            Only_identifiers_Slashqualified_names_with_optional_type_arguments_are_currently_supported_in_a_class_extends_clause: diag(9002, ts.DiagnosticCategory.Error, "Only_identifiers_Slashqualified_names_with_optional_type_arguments_are_currently_supported_in_a_clas_9002", "Only identifiers/qualified-names with optional type arguments are currently supported in a class 'extends' clause."),
            class_expressions_are_not_currently_supported: diag(9003, ts.DiagnosticCategory.Error, "class_expressions_are_not_currently_supported_9003", "'class' expressions are not currently supported."),
            Language_service_is_disabled: diag(9004, ts.DiagnosticCategory.Error, "Language_service_is_disabled_9004", "Language service is disabled."),
            Declaration_emit_for_this_file_requires_using_private_name_0_An_explicit_type_annotation_may_unblock_declaration_emit: diag(9005, ts.DiagnosticCategory.Error, "Declaration_emit_for_this_file_requires_using_private_name_0_An_explicit_type_annotation_may_unblock_9005", "Declaration emit for this file requires using private name '{0}'. An explicit type annotation may unblock declaration emit."),
            Declaration_emit_for_this_file_requires_using_private_name_0_from_module_1_An_explicit_type_annotation_may_unblock_declaration_emit: diag(9006, ts.DiagnosticCategory.Error, "Declaration_emit_for_this_file_requires_using_private_name_0_from_module_1_An_explicit_type_annotati_9006", "Declaration emit for this file requires using private name '{0}' from module '{1}'. An explicit type annotation may unblock declaration emit."),
            JSX_attributes_must_only_be_assigned_a_non_empty_expression: diag(17000, ts.DiagnosticCategory.Error, "JSX_attributes_must_only_be_assigned_a_non_empty_expression_17000", "JSX attributes must only be assigned a non-empty 'expression'."),
            JSX_elements_cannot_have_multiple_attributes_with_the_same_name: diag(17001, ts.DiagnosticCategory.Error, "JSX_elements_cannot_have_multiple_attributes_with_the_same_name_17001", "JSX elements cannot have multiple attributes with the same name."),
            Expected_corresponding_JSX_closing_tag_for_0: diag(17002, ts.DiagnosticCategory.Error, "Expected_corresponding_JSX_closing_tag_for_0_17002", "Expected corresponding JSX closing tag for '{0}'."),
            JSX_attribute_expected: diag(17003, ts.DiagnosticCategory.Error, "JSX_attribute_expected_17003", "JSX attribute expected."),
            Cannot_use_JSX_unless_the_jsx_flag_is_provided: diag(17004, ts.DiagnosticCategory.Error, "Cannot_use_JSX_unless_the_jsx_flag_is_provided_17004", "Cannot use JSX unless the '--jsx' flag is provided."),
            A_constructor_cannot_contain_a_super_call_when_its_class_extends_null: diag(17005, ts.DiagnosticCategory.Error, "A_constructor_cannot_contain_a_super_call_when_its_class_extends_null_17005", "A constructor cannot contain a 'super' call when its class extends 'null'."),
            An_unary_expression_with_the_0_operator_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses: diag(17006, ts.DiagnosticCategory.Error, "An_unary_expression_with_the_0_operator_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_ex_17006", "An unary expression with the '{0}' operator is not allowed in the left-hand side of an exponentiation expression. Consider enclosing the expression in parentheses."),
            A_type_assertion_expression_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses: diag(17007, ts.DiagnosticCategory.Error, "A_type_assertion_expression_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Con_17007", "A type assertion expression is not allowed in the left-hand side of an exponentiation expression. Consider enclosing the expression in parentheses."),
            JSX_element_0_has_no_corresponding_closing_tag: diag(17008, ts.DiagnosticCategory.Error, "JSX_element_0_has_no_corresponding_closing_tag_17008", "JSX element '{0}' has no corresponding closing tag."),
            super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class: diag(17009, ts.DiagnosticCategory.Error, "super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class_17009", "'super' must be called before accessing 'this' in the constructor of a derived class."),
            Unknown_type_acquisition_option_0: diag(17010, ts.DiagnosticCategory.Error, "Unknown_type_acquisition_option_0_17010", "Unknown type acquisition option '{0}'."),
            super_must_be_called_before_accessing_a_property_of_super_in_the_constructor_of_a_derived_class: diag(17011, ts.DiagnosticCategory.Error, "super_must_be_called_before_accessing_a_property_of_super_in_the_constructor_of_a_derived_class_17011", "'super' must be called before accessing a property of 'super' in the constructor of a derived class."),
            _0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2: diag(17012, ts.DiagnosticCategory.Error, "_0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2_17012", "'{0}' is not a valid meta-property for keyword '{1}'. Did you mean '{2}'?"),
            Meta_property_0_is_only_allowed_in_the_body_of_a_function_declaration_function_expression_or_constructor: diag(17013, ts.DiagnosticCategory.Error, "Meta_property_0_is_only_allowed_in_the_body_of_a_function_declaration_function_expression_or_constru_17013", "Meta-property '{0}' is only allowed in the body of a function declaration, function expression, or constructor."),
            JSX_fragment_has_no_corresponding_closing_tag: diag(17014, ts.DiagnosticCategory.Error, "JSX_fragment_has_no_corresponding_closing_tag_17014", "JSX fragment has no corresponding closing tag."),
            Expected_corresponding_closing_tag_for_JSX_fragment: diag(17015, ts.DiagnosticCategory.Error, "Expected_corresponding_closing_tag_for_JSX_fragment_17015", "Expected corresponding closing tag for JSX fragment."),
            JSX_fragment_is_not_supported_when_using_jsxFactory: diag(17016, ts.DiagnosticCategory.Error, "JSX_fragment_is_not_supported_when_using_jsxFactory_17016", "JSX fragment is not supported when using --jsxFactory"),
            JSX_fragment_is_not_supported_when_using_an_inline_JSX_factory_pragma: diag(17017, ts.DiagnosticCategory.Error, "JSX_fragment_is_not_supported_when_using_an_inline_JSX_factory_pragma_17017", "JSX fragment is not supported when using an inline JSX factory pragma"),
            Unknown_type_acquisition_option_0_Did_you_mean_1: diag(17018, ts.DiagnosticCategory.Error, "Unknown_type_acquisition_option_0_Did_you_mean_1_17018", "Unknown type acquisition option '{0}'. Did you mean '{1}'?"),
            Circularity_detected_while_resolving_configuration_Colon_0: diag(18000, ts.DiagnosticCategory.Error, "Circularity_detected_while_resolving_configuration_Colon_0_18000", "Circularity detected while resolving configuration: {0}"),
            A_path_in_an_extends_option_must_be_relative_or_rooted_but_0_is_not: diag(18001, ts.DiagnosticCategory.Error, "A_path_in_an_extends_option_must_be_relative_or_rooted_but_0_is_not_18001", "A path in an 'extends' option must be relative or rooted, but '{0}' is not."),
            The_files_list_in_config_file_0_is_empty: diag(18002, ts.DiagnosticCategory.Error, "The_files_list_in_config_file_0_is_empty_18002", "The 'files' list in config file '{0}' is empty."),
            No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2: diag(18003, ts.DiagnosticCategory.Error, "No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2_18003", "No inputs were found in config file '{0}'. Specified 'include' paths were '{1}' and 'exclude' paths were '{2}'."),
            File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module: diag(80001, ts.DiagnosticCategory.Suggestion, "File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module_80001", "File is a CommonJS module; it may be converted to an ES6 module."),
            This_constructor_function_may_be_converted_to_a_class_declaration: diag(80002, ts.DiagnosticCategory.Suggestion, "This_constructor_function_may_be_converted_to_a_class_declaration_80002", "This constructor function may be converted to a class declaration."),
            Import_may_be_converted_to_a_default_import: diag(80003, ts.DiagnosticCategory.Suggestion, "Import_may_be_converted_to_a_default_import_80003", "Import may be converted to a default import."),
            JSDoc_types_may_be_moved_to_TypeScript_types: diag(80004, ts.DiagnosticCategory.Suggestion, "JSDoc_types_may_be_moved_to_TypeScript_types_80004", "JSDoc types may be moved to TypeScript types."),
            require_call_may_be_converted_to_an_import: diag(80005, ts.DiagnosticCategory.Suggestion, "require_call_may_be_converted_to_an_import_80005", "'require' call may be converted to an import."),
            This_may_be_converted_to_an_async_function: diag(80006, ts.DiagnosticCategory.Suggestion, "This_may_be_converted_to_an_async_function_80006", "This may be converted to an async function."),
            await_has_no_effect_on_the_type_of_this_expression: diag(80007, ts.DiagnosticCategory.Suggestion, "await_has_no_effect_on_the_type_of_this_expression_80007", "'await' has no effect on the type of this expression."),
            Numeric_literals_with_absolute_values_equal_to_2_53_or_greater_are_too_large_to_be_represented_accurately_as_integers: diag(80008, ts.DiagnosticCategory.Suggestion, "Numeric_literals_with_absolute_values_equal_to_2_53_or_greater_are_too_large_to_be_represented_accur_80008", "Numeric literals with absolute values equal to 2^53 or greater are too large to be represented accurately as integers."),
            Add_missing_super_call: diag(90001, ts.DiagnosticCategory.Message, "Add_missing_super_call_90001", "Add missing 'super()' call"),
            Make_super_call_the_first_statement_in_the_constructor: diag(90002, ts.DiagnosticCategory.Message, "Make_super_call_the_first_statement_in_the_constructor_90002", "Make 'super()' call the first statement in the constructor"),
            Change_extends_to_implements: diag(90003, ts.DiagnosticCategory.Message, "Change_extends_to_implements_90003", "Change 'extends' to 'implements'"),
            Remove_unused_declaration_for_Colon_0: diag(90004, ts.DiagnosticCategory.Message, "Remove_unused_declaration_for_Colon_0_90004", "Remove unused declaration for: '{0}'"),
            Remove_import_from_0: diag(90005, ts.DiagnosticCategory.Message, "Remove_import_from_0_90005", "Remove import from '{0}'"),
            Implement_interface_0: diag(90006, ts.DiagnosticCategory.Message, "Implement_interface_0_90006", "Implement interface '{0}'"),
            Implement_inherited_abstract_class: diag(90007, ts.DiagnosticCategory.Message, "Implement_inherited_abstract_class_90007", "Implement inherited abstract class"),
            Add_0_to_unresolved_variable: diag(90008, ts.DiagnosticCategory.Message, "Add_0_to_unresolved_variable_90008", "Add '{0}.' to unresolved variable"),
            Remove_destructuring: diag(90009, ts.DiagnosticCategory.Message, "Remove_destructuring_90009", "Remove destructuring"),
            Remove_variable_statement: diag(90010, ts.DiagnosticCategory.Message, "Remove_variable_statement_90010", "Remove variable statement"),
            Remove_template_tag: diag(90011, ts.DiagnosticCategory.Message, "Remove_template_tag_90011", "Remove template tag"),
            Remove_type_parameters: diag(90012, ts.DiagnosticCategory.Message, "Remove_type_parameters_90012", "Remove type parameters"),
            Import_0_from_module_1: diag(90013, ts.DiagnosticCategory.Message, "Import_0_from_module_1_90013", "Import '{0}' from module \"{1}\""),
            Change_0_to_1: diag(90014, ts.DiagnosticCategory.Message, "Change_0_to_1_90014", "Change '{0}' to '{1}'"),
            Add_0_to_existing_import_declaration_from_1: diag(90015, ts.DiagnosticCategory.Message, "Add_0_to_existing_import_declaration_from_1_90015", "Add '{0}' to existing import declaration from \"{1}\""),
            Declare_property_0: diag(90016, ts.DiagnosticCategory.Message, "Declare_property_0_90016", "Declare property '{0}'"),
            Add_index_signature_for_property_0: diag(90017, ts.DiagnosticCategory.Message, "Add_index_signature_for_property_0_90017", "Add index signature for property '{0}'"),
            Disable_checking_for_this_file: diag(90018, ts.DiagnosticCategory.Message, "Disable_checking_for_this_file_90018", "Disable checking for this file"),
            Ignore_this_error_message: diag(90019, ts.DiagnosticCategory.Message, "Ignore_this_error_message_90019", "Ignore this error message"),
            Initialize_property_0_in_the_constructor: diag(90020, ts.DiagnosticCategory.Message, "Initialize_property_0_in_the_constructor_90020", "Initialize property '{0}' in the constructor"),
            Initialize_static_property_0: diag(90021, ts.DiagnosticCategory.Message, "Initialize_static_property_0_90021", "Initialize static property '{0}'"),
            Change_spelling_to_0: diag(90022, ts.DiagnosticCategory.Message, "Change_spelling_to_0_90022", "Change spelling to '{0}'"),
            Declare_method_0: diag(90023, ts.DiagnosticCategory.Message, "Declare_method_0_90023", "Declare method '{0}'"),
            Declare_static_method_0: diag(90024, ts.DiagnosticCategory.Message, "Declare_static_method_0_90024", "Declare static method '{0}'"),
            Prefix_0_with_an_underscore: diag(90025, ts.DiagnosticCategory.Message, "Prefix_0_with_an_underscore_90025", "Prefix '{0}' with an underscore"),
            Rewrite_as_the_indexed_access_type_0: diag(90026, ts.DiagnosticCategory.Message, "Rewrite_as_the_indexed_access_type_0_90026", "Rewrite as the indexed access type '{0}'"),
            Declare_static_property_0: diag(90027, ts.DiagnosticCategory.Message, "Declare_static_property_0_90027", "Declare static property '{0}'"),
            Call_decorator_expression: diag(90028, ts.DiagnosticCategory.Message, "Call_decorator_expression_90028", "Call decorator expression"),
            Add_async_modifier_to_containing_function: diag(90029, ts.DiagnosticCategory.Message, "Add_async_modifier_to_containing_function_90029", "Add async modifier to containing function"),
            Replace_infer_0_with_unknown: diag(90030, ts.DiagnosticCategory.Message, "Replace_infer_0_with_unknown_90030", "Replace 'infer {0}' with 'unknown'"),
            Replace_all_unused_infer_with_unknown: diag(90031, ts.DiagnosticCategory.Message, "Replace_all_unused_infer_with_unknown_90031", "Replace all unused 'infer' with 'unknown'"),
            Import_default_0_from_module_1: diag(90032, ts.DiagnosticCategory.Message, "Import_default_0_from_module_1_90032", "Import default '{0}' from module \"{1}\""),
            Add_default_import_0_to_existing_import_declaration_from_1: diag(90033, ts.DiagnosticCategory.Message, "Add_default_import_0_to_existing_import_declaration_from_1_90033", "Add default import '{0}' to existing import declaration from \"{1}\""),
            Add_parameter_name: diag(90034, ts.DiagnosticCategory.Message, "Add_parameter_name_90034", "Add parameter name"),
            Declare_a_private_field_named_0: diag(90053, ts.DiagnosticCategory.Message, "Declare_a_private_field_named_0_90053", "Declare a private field named '{0}'."),
            Convert_function_to_an_ES2015_class: diag(95001, ts.DiagnosticCategory.Message, "Convert_function_to_an_ES2015_class_95001", "Convert function to an ES2015 class"),
            Convert_function_0_to_class: diag(95002, ts.DiagnosticCategory.Message, "Convert_function_0_to_class_95002", "Convert function '{0}' to class"),
            Extract_to_0_in_1: diag(95004, ts.DiagnosticCategory.Message, "Extract_to_0_in_1_95004", "Extract to {0} in {1}"),
            Extract_function: diag(95005, ts.DiagnosticCategory.Message, "Extract_function_95005", "Extract function"),
            Extract_constant: diag(95006, ts.DiagnosticCategory.Message, "Extract_constant_95006", "Extract constant"),
            Extract_to_0_in_enclosing_scope: diag(95007, ts.DiagnosticCategory.Message, "Extract_to_0_in_enclosing_scope_95007", "Extract to {0} in enclosing scope"),
            Extract_to_0_in_1_scope: diag(95008, ts.DiagnosticCategory.Message, "Extract_to_0_in_1_scope_95008", "Extract to {0} in {1} scope"),
            Annotate_with_type_from_JSDoc: diag(95009, ts.DiagnosticCategory.Message, "Annotate_with_type_from_JSDoc_95009", "Annotate with type from JSDoc"),
            Annotate_with_types_from_JSDoc: diag(95010, ts.DiagnosticCategory.Message, "Annotate_with_types_from_JSDoc_95010", "Annotate with types from JSDoc"),
            Infer_type_of_0_from_usage: diag(95011, ts.DiagnosticCategory.Message, "Infer_type_of_0_from_usage_95011", "Infer type of '{0}' from usage"),
            Infer_parameter_types_from_usage: diag(95012, ts.DiagnosticCategory.Message, "Infer_parameter_types_from_usage_95012", "Infer parameter types from usage"),
            Convert_to_default_import: diag(95013, ts.DiagnosticCategory.Message, "Convert_to_default_import_95013", "Convert to default import"),
            Install_0: diag(95014, ts.DiagnosticCategory.Message, "Install_0_95014", "Install '{0}'"),
            Replace_import_with_0: diag(95015, ts.DiagnosticCategory.Message, "Replace_import_with_0_95015", "Replace import with '{0}'."),
            Use_synthetic_default_member: diag(95016, ts.DiagnosticCategory.Message, "Use_synthetic_default_member_95016", "Use synthetic 'default' member."),
            Convert_to_ES6_module: diag(95017, ts.DiagnosticCategory.Message, "Convert_to_ES6_module_95017", "Convert to ES6 module"),
            Add_undefined_type_to_property_0: diag(95018, ts.DiagnosticCategory.Message, "Add_undefined_type_to_property_0_95018", "Add 'undefined' type to property '{0}'"),
            Add_initializer_to_property_0: diag(95019, ts.DiagnosticCategory.Message, "Add_initializer_to_property_0_95019", "Add initializer to property '{0}'"),
            Add_definite_assignment_assertion_to_property_0: diag(95020, ts.DiagnosticCategory.Message, "Add_definite_assignment_assertion_to_property_0_95020", "Add definite assignment assertion to property '{0}'"),
            Add_all_missing_members: diag(95022, ts.DiagnosticCategory.Message, "Add_all_missing_members_95022", "Add all missing members"),
            Infer_all_types_from_usage: diag(95023, ts.DiagnosticCategory.Message, "Infer_all_types_from_usage_95023", "Infer all types from usage"),
            Delete_all_unused_declarations: diag(95024, ts.DiagnosticCategory.Message, "Delete_all_unused_declarations_95024", "Delete all unused declarations"),
            Prefix_all_unused_declarations_with_where_possible: diag(95025, ts.DiagnosticCategory.Message, "Prefix_all_unused_declarations_with_where_possible_95025", "Prefix all unused declarations with '_' where possible"),
            Fix_all_detected_spelling_errors: diag(95026, ts.DiagnosticCategory.Message, "Fix_all_detected_spelling_errors_95026", "Fix all detected spelling errors"),
            Add_initializers_to_all_uninitialized_properties: diag(95027, ts.DiagnosticCategory.Message, "Add_initializers_to_all_uninitialized_properties_95027", "Add initializers to all uninitialized properties"),
            Add_definite_assignment_assertions_to_all_uninitialized_properties: diag(95028, ts.DiagnosticCategory.Message, "Add_definite_assignment_assertions_to_all_uninitialized_properties_95028", "Add definite assignment assertions to all uninitialized properties"),
            Add_undefined_type_to_all_uninitialized_properties: diag(95029, ts.DiagnosticCategory.Message, "Add_undefined_type_to_all_uninitialized_properties_95029", "Add undefined type to all uninitialized properties"),
            Change_all_jsdoc_style_types_to_TypeScript: diag(95030, ts.DiagnosticCategory.Message, "Change_all_jsdoc_style_types_to_TypeScript_95030", "Change all jsdoc-style types to TypeScript"),
            Change_all_jsdoc_style_types_to_TypeScript_and_add_undefined_to_nullable_types: diag(95031, ts.DiagnosticCategory.Message, "Change_all_jsdoc_style_types_to_TypeScript_and_add_undefined_to_nullable_types_95031", "Change all jsdoc-style types to TypeScript (and add '| undefined' to nullable types)"),
            Implement_all_unimplemented_interfaces: diag(95032, ts.DiagnosticCategory.Message, "Implement_all_unimplemented_interfaces_95032", "Implement all unimplemented interfaces"),
            Install_all_missing_types_packages: diag(95033, ts.DiagnosticCategory.Message, "Install_all_missing_types_packages_95033", "Install all missing types packages"),
            Rewrite_all_as_indexed_access_types: diag(95034, ts.DiagnosticCategory.Message, "Rewrite_all_as_indexed_access_types_95034", "Rewrite all as indexed access types"),
            Convert_all_to_default_imports: diag(95035, ts.DiagnosticCategory.Message, "Convert_all_to_default_imports_95035", "Convert all to default imports"),
            Make_all_super_calls_the_first_statement_in_their_constructor: diag(95036, ts.DiagnosticCategory.Message, "Make_all_super_calls_the_first_statement_in_their_constructor_95036", "Make all 'super()' calls the first statement in their constructor"),
            Add_qualifier_to_all_unresolved_variables_matching_a_member_name: diag(95037, ts.DiagnosticCategory.Message, "Add_qualifier_to_all_unresolved_variables_matching_a_member_name_95037", "Add qualifier to all unresolved variables matching a member name"),
            Change_all_extended_interfaces_to_implements: diag(95038, ts.DiagnosticCategory.Message, "Change_all_extended_interfaces_to_implements_95038", "Change all extended interfaces to 'implements'"),
            Add_all_missing_super_calls: diag(95039, ts.DiagnosticCategory.Message, "Add_all_missing_super_calls_95039", "Add all missing super calls"),
            Implement_all_inherited_abstract_classes: diag(95040, ts.DiagnosticCategory.Message, "Implement_all_inherited_abstract_classes_95040", "Implement all inherited abstract classes"),
            Add_all_missing_async_modifiers: diag(95041, ts.DiagnosticCategory.Message, "Add_all_missing_async_modifiers_95041", "Add all missing 'async' modifiers"),
            Add_ts_ignore_to_all_error_messages: diag(95042, ts.DiagnosticCategory.Message, "Add_ts_ignore_to_all_error_messages_95042", "Add '@ts-ignore' to all error messages"),
            Annotate_everything_with_types_from_JSDoc: diag(95043, ts.DiagnosticCategory.Message, "Annotate_everything_with_types_from_JSDoc_95043", "Annotate everything with types from JSDoc"),
            Add_to_all_uncalled_decorators: diag(95044, ts.DiagnosticCategory.Message, "Add_to_all_uncalled_decorators_95044", "Add '()' to all uncalled decorators"),
            Convert_all_constructor_functions_to_classes: diag(95045, ts.DiagnosticCategory.Message, "Convert_all_constructor_functions_to_classes_95045", "Convert all constructor functions to classes"),
            Generate_get_and_set_accessors: diag(95046, ts.DiagnosticCategory.Message, "Generate_get_and_set_accessors_95046", "Generate 'get' and 'set' accessors"),
            Convert_require_to_import: diag(95047, ts.DiagnosticCategory.Message, "Convert_require_to_import_95047", "Convert 'require' to 'import'"),
            Convert_all_require_to_import: diag(95048, ts.DiagnosticCategory.Message, "Convert_all_require_to_import_95048", "Convert all 'require' to 'import'"),
            Move_to_a_new_file: diag(95049, ts.DiagnosticCategory.Message, "Move_to_a_new_file_95049", "Move to a new file"),
            Remove_unreachable_code: diag(95050, ts.DiagnosticCategory.Message, "Remove_unreachable_code_95050", "Remove unreachable code"),
            Remove_all_unreachable_code: diag(95051, ts.DiagnosticCategory.Message, "Remove_all_unreachable_code_95051", "Remove all unreachable code"),
            Add_missing_typeof: diag(95052, ts.DiagnosticCategory.Message, "Add_missing_typeof_95052", "Add missing 'typeof'"),
            Remove_unused_label: diag(95053, ts.DiagnosticCategory.Message, "Remove_unused_label_95053", "Remove unused label"),
            Remove_all_unused_labels: diag(95054, ts.DiagnosticCategory.Message, "Remove_all_unused_labels_95054", "Remove all unused labels"),
            Convert_0_to_mapped_object_type: diag(95055, ts.DiagnosticCategory.Message, "Convert_0_to_mapped_object_type_95055", "Convert '{0}' to mapped object type"),
            Convert_namespace_import_to_named_imports: diag(95056, ts.DiagnosticCategory.Message, "Convert_namespace_import_to_named_imports_95056", "Convert namespace import to named imports"),
            Convert_named_imports_to_namespace_import: diag(95057, ts.DiagnosticCategory.Message, "Convert_named_imports_to_namespace_import_95057", "Convert named imports to namespace import"),
            Add_or_remove_braces_in_an_arrow_function: diag(95058, ts.DiagnosticCategory.Message, "Add_or_remove_braces_in_an_arrow_function_95058", "Add or remove braces in an arrow function"),
            Add_braces_to_arrow_function: diag(95059, ts.DiagnosticCategory.Message, "Add_braces_to_arrow_function_95059", "Add braces to arrow function"),
            Remove_braces_from_arrow_function: diag(95060, ts.DiagnosticCategory.Message, "Remove_braces_from_arrow_function_95060", "Remove braces from arrow function"),
            Convert_default_export_to_named_export: diag(95061, ts.DiagnosticCategory.Message, "Convert_default_export_to_named_export_95061", "Convert default export to named export"),
            Convert_named_export_to_default_export: diag(95062, ts.DiagnosticCategory.Message, "Convert_named_export_to_default_export_95062", "Convert named export to default export"),
            Add_missing_enum_member_0: diag(95063, ts.DiagnosticCategory.Message, "Add_missing_enum_member_0_95063", "Add missing enum member '{0}'"),
            Add_all_missing_imports: diag(95064, ts.DiagnosticCategory.Message, "Add_all_missing_imports_95064", "Add all missing imports"),
            Convert_to_async_function: diag(95065, ts.DiagnosticCategory.Message, "Convert_to_async_function_95065", "Convert to async function"),
            Convert_all_to_async_functions: diag(95066, ts.DiagnosticCategory.Message, "Convert_all_to_async_functions_95066", "Convert all to async functions"),
            Add_unknown_conversion_for_non_overlapping_types: diag(95069, ts.DiagnosticCategory.Message, "Add_unknown_conversion_for_non_overlapping_types_95069", "Add 'unknown' conversion for non-overlapping types"),
            Add_unknown_to_all_conversions_of_non_overlapping_types: diag(95070, ts.DiagnosticCategory.Message, "Add_unknown_to_all_conversions_of_non_overlapping_types_95070", "Add 'unknown' to all conversions of non-overlapping types"),
            Add_missing_new_operator_to_call: diag(95071, ts.DiagnosticCategory.Message, "Add_missing_new_operator_to_call_95071", "Add missing 'new' operator to call"),
            Add_missing_new_operator_to_all_calls: diag(95072, ts.DiagnosticCategory.Message, "Add_missing_new_operator_to_all_calls_95072", "Add missing 'new' operator to all calls"),
            Add_names_to_all_parameters_without_names: diag(95073, ts.DiagnosticCategory.Message, "Add_names_to_all_parameters_without_names_95073", "Add names to all parameters without names"),
            Enable_the_experimentalDecorators_option_in_your_configuration_file: diag(95074, ts.DiagnosticCategory.Message, "Enable_the_experimentalDecorators_option_in_your_configuration_file_95074", "Enable the 'experimentalDecorators' option in your configuration file"),
            Convert_parameters_to_destructured_object: diag(95075, ts.DiagnosticCategory.Message, "Convert_parameters_to_destructured_object_95075", "Convert parameters to destructured object"),
            Allow_accessing_UMD_globals_from_modules: diag(95076, ts.DiagnosticCategory.Message, "Allow_accessing_UMD_globals_from_modules_95076", "Allow accessing UMD globals from modules."),
            Extract_type: diag(95077, ts.DiagnosticCategory.Message, "Extract_type_95077", "Extract type"),
            Extract_to_type_alias: diag(95078, ts.DiagnosticCategory.Message, "Extract_to_type_alias_95078", "Extract to type alias"),
            Extract_to_typedef: diag(95079, ts.DiagnosticCategory.Message, "Extract_to_typedef_95079", "Extract to typedef"),
            Infer_this_type_of_0_from_usage: diag(95080, ts.DiagnosticCategory.Message, "Infer_this_type_of_0_from_usage_95080", "Infer 'this' type of '{0}' from usage"),
            Add_const_to_unresolved_variable: diag(95081, ts.DiagnosticCategory.Message, "Add_const_to_unresolved_variable_95081", "Add 'const' to unresolved variable"),
            Add_const_to_all_unresolved_variables: diag(95082, ts.DiagnosticCategory.Message, "Add_const_to_all_unresolved_variables_95082", "Add 'const' to all unresolved variables"),
            Add_await: diag(95083, ts.DiagnosticCategory.Message, "Add_await_95083", "Add 'await'"),
            Add_await_to_initializer_for_0: diag(95084, ts.DiagnosticCategory.Message, "Add_await_to_initializer_for_0_95084", "Add 'await' to initializer for '{0}'"),
            Fix_all_expressions_possibly_missing_await: diag(95085, ts.DiagnosticCategory.Message, "Fix_all_expressions_possibly_missing_await_95085", "Fix all expressions possibly missing 'await'"),
            Remove_unnecessary_await: diag(95086, ts.DiagnosticCategory.Message, "Remove_unnecessary_await_95086", "Remove unnecessary 'await'"),
            Remove_all_unnecessary_uses_of_await: diag(95087, ts.DiagnosticCategory.Message, "Remove_all_unnecessary_uses_of_await_95087", "Remove all unnecessary uses of 'await'"),
            Enable_the_jsx_flag_in_your_configuration_file: diag(95088, ts.DiagnosticCategory.Message, "Enable_the_jsx_flag_in_your_configuration_file_95088", "Enable the '--jsx' flag in your configuration file"),
            Add_await_to_initializers: diag(95089, ts.DiagnosticCategory.Message, "Add_await_to_initializers_95089", "Add 'await' to initializers"),
            Extract_to_interface: diag(95090, ts.DiagnosticCategory.Message, "Extract_to_interface_95090", "Extract to interface"),
            Convert_to_a_bigint_numeric_literal: diag(95091, ts.DiagnosticCategory.Message, "Convert_to_a_bigint_numeric_literal_95091", "Convert to a bigint numeric literal"),
            Convert_all_to_bigint_numeric_literals: diag(95092, ts.DiagnosticCategory.Message, "Convert_all_to_bigint_numeric_literals_95092", "Convert all to bigint numeric literals"),
            Convert_const_to_let: diag(95093, ts.DiagnosticCategory.Message, "Convert_const_to_let_95093", "Convert 'const' to 'let'"),
            Prefix_with_declare: diag(95094, ts.DiagnosticCategory.Message, "Prefix_with_declare_95094", "Prefix with 'declare'"),
            Prefix_all_incorrect_property_declarations_with_declare: diag(95095, ts.DiagnosticCategory.Message, "Prefix_all_incorrect_property_declarations_with_declare_95095", "Prefix all incorrect property declarations with 'declare'"),
            Convert_to_template_string: diag(95096, ts.DiagnosticCategory.Message, "Convert_to_template_string_95096", "Convert to template string"),
            Add_export_to_make_this_file_into_a_module: diag(95097, ts.DiagnosticCategory.Message, "Add_export_to_make_this_file_into_a_module_95097", "Add 'export {}' to make this file into a module"),
            Set_the_target_option_in_your_configuration_file_to_0: diag(95098, ts.DiagnosticCategory.Message, "Set_the_target_option_in_your_configuration_file_to_0_95098", "Set the 'target' option in your configuration file to '{0}'"),
            Set_the_module_option_in_your_configuration_file_to_0: diag(95099, ts.DiagnosticCategory.Message, "Set_the_module_option_in_your_configuration_file_to_0_95099", "Set the 'module' option in your configuration file to '{0}'"),
            No_value_exists_in_scope_for_the_shorthand_property_0_Either_declare_one_or_provide_an_initializer: diag(18004, ts.DiagnosticCategory.Error, "No_value_exists_in_scope_for_the_shorthand_property_0_Either_declare_one_or_provide_an_initializer_18004", "No value exists in scope for the shorthand property '{0}'. Either declare one or provide an initializer."),
            Classes_may_not_have_a_field_named_constructor: diag(18006, ts.DiagnosticCategory.Error, "Classes_may_not_have_a_field_named_constructor_18006", "Classes may not have a field named 'constructor'."),
            JSX_expressions_may_not_use_the_comma_operator_Did_you_mean_to_write_an_array: diag(18007, ts.DiagnosticCategory.Error, "JSX_expressions_may_not_use_the_comma_operator_Did_you_mean_to_write_an_array_18007", "JSX expressions may not use the comma operator. Did you mean to write an array?"),
            Private_identifiers_cannot_be_used_as_parameters: diag(18009, ts.DiagnosticCategory.Error, "Private_identifiers_cannot_be_used_as_parameters_18009", "Private identifiers cannot be used as parameters"),
            An_accessibility_modifier_cannot_be_used_with_a_private_identifier: diag(18010, ts.DiagnosticCategory.Error, "An_accessibility_modifier_cannot_be_used_with_a_private_identifier_18010", "An accessibility modifier cannot be used with a private identifier."),
            The_operand_of_a_delete_operator_cannot_be_a_private_identifier: diag(18011, ts.DiagnosticCategory.Error, "The_operand_of_a_delete_operator_cannot_be_a_private_identifier_18011", "The operand of a 'delete' operator cannot be a private identifier."),
            constructor_is_a_reserved_word: diag(18012, ts.DiagnosticCategory.Error, "constructor_is_a_reserved_word_18012", "'#constructor' is a reserved word."),
            Property_0_is_not_accessible_outside_class_1_because_it_has_a_private_identifier: diag(18013, ts.DiagnosticCategory.Error, "Property_0_is_not_accessible_outside_class_1_because_it_has_a_private_identifier_18013", "Property '{0}' is not accessible outside class '{1}' because it has a private identifier."),
            The_property_0_cannot_be_accessed_on_type_1_within_this_class_because_it_is_shadowed_by_another_private_identifier_with_the_same_spelling: diag(18014, ts.DiagnosticCategory.Error, "The_property_0_cannot_be_accessed_on_type_1_within_this_class_because_it_is_shadowed_by_another_priv_18014", "The property '{0}' cannot be accessed on type '{1}' within this class because it is shadowed by another private identifier with the same spelling."),
            Property_0_in_type_1_refers_to_a_different_member_that_cannot_be_accessed_from_within_type_2: diag(18015, ts.DiagnosticCategory.Error, "Property_0_in_type_1_refers_to_a_different_member_that_cannot_be_accessed_from_within_type_2_18015", "Property '{0}' in type '{1}' refers to a different member that cannot be accessed from within type '{2}'."),
            Private_identifiers_are_not_allowed_outside_class_bodies: diag(18016, ts.DiagnosticCategory.Error, "Private_identifiers_are_not_allowed_outside_class_bodies_18016", "Private identifiers are not allowed outside class bodies."),
            The_shadowing_declaration_of_0_is_defined_here: diag(18017, ts.DiagnosticCategory.Error, "The_shadowing_declaration_of_0_is_defined_here_18017", "The shadowing declaration of '{0}' is defined here"),
            The_declaration_of_0_that_you_probably_intended_to_use_is_defined_here: diag(18018, ts.DiagnosticCategory.Error, "The_declaration_of_0_that_you_probably_intended_to_use_is_defined_here_18018", "The declaration of '{0}' that you probably intended to use is defined here"),
            _0_modifier_cannot_be_used_with_a_private_identifier: diag(18019, ts.DiagnosticCategory.Error, "_0_modifier_cannot_be_used_with_a_private_identifier_18019", "'{0}' modifier cannot be used with a private identifier"),
            A_method_cannot_be_named_with_a_private_identifier: diag(18022, ts.DiagnosticCategory.Error, "A_method_cannot_be_named_with_a_private_identifier_18022", "A method cannot be named with a private identifier."),
            An_accessor_cannot_be_named_with_a_private_identifier: diag(18023, ts.DiagnosticCategory.Error, "An_accessor_cannot_be_named_with_a_private_identifier_18023", "An accessor cannot be named with a private identifier."),
            An_enum_member_cannot_be_named_with_a_private_identifier: diag(18024, ts.DiagnosticCategory.Error, "An_enum_member_cannot_be_named_with_a_private_identifier_18024", "An enum member cannot be named with a private identifier."),
            can_only_be_used_at_the_start_of_a_file: diag(18026, ts.DiagnosticCategory.Error, "can_only_be_used_at_the_start_of_a_file_18026", "'#!' can only be used at the start of a file."),
            Compiler_reserves_name_0_when_emitting_private_identifier_downlevel: diag(18027, ts.DiagnosticCategory.Error, "Compiler_reserves_name_0_when_emitting_private_identifier_downlevel_18027", "Compiler reserves name '{0}' when emitting private identifier downlevel."),
            Private_identifiers_are_only_available_when_targeting_ECMAScript_2015_and_higher: diag(18028, ts.DiagnosticCategory.Error, "Private_identifiers_are_only_available_when_targeting_ECMAScript_2015_and_higher_18028", "Private identifiers are only available when targeting ECMAScript 2015 and higher."),
            Private_identifiers_are_not_allowed_in_variable_declarations: diag(18029, ts.DiagnosticCategory.Error, "Private_identifiers_are_not_allowed_in_variable_declarations_18029", "Private identifiers are not allowed in variable declarations."),
            An_optional_chain_cannot_contain_private_identifiers: diag(18030, ts.DiagnosticCategory.Error, "An_optional_chain_cannot_contain_private_identifiers_18030", "An optional chain cannot contain private identifiers."),
        };
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        var _a;
        /* @internal */
        function tokenIsIdentifierOrKeyword(token) {
            return token >= 75 /* Identifier */;
        }
        ts.tokenIsIdentifierOrKeyword = tokenIsIdentifierOrKeyword;
        /* @internal */
        function tokenIsIdentifierOrKeywordOrGreaterThan(token) {
            return token === 31 /* GreaterThanToken */ || tokenIsIdentifierOrKeyword(token);
        }
        ts.tokenIsIdentifierOrKeywordOrGreaterThan = tokenIsIdentifierOrKeywordOrGreaterThan;
        var textToKeywordObj = (_a = {
            abstract: 122 /* AbstractKeyword */,
            any: 125 /* AnyKeyword */,
            as: 123 /* AsKeyword */,
            asserts: 124 /* AssertsKeyword */,
            bigint: 151 /* BigIntKeyword */,
            boolean: 128 /* BooleanKeyword */,
            break: 77 /* BreakKeyword */,
            case: 78 /* CaseKeyword */,
            catch: 79 /* CatchKeyword */,
            class: 80 /* ClassKeyword */,
            continue: 82 /* ContinueKeyword */,
            const: 81 /* ConstKeyword */
        },
            _a["" + "constructor"] = 129 /* ConstructorKeyword */,
            _a.debugger = 83 /* DebuggerKeyword */,
            _a.declare = 130 /* DeclareKeyword */,
            _a.default = 84 /* DefaultKeyword */,
            _a.delete = 85 /* DeleteKeyword */,
            _a.do = 86 /* DoKeyword */,
            _a.else = 87 /* ElseKeyword */,
            _a.enum = 88 /* EnumKeyword */,
            _a.export = 89 /* ExportKeyword */,
            _a.extends = 90 /* ExtendsKeyword */,
            _a.false = 91 /* FalseKeyword */,
            _a.finally = 92 /* FinallyKeyword */,
            _a.for = 93 /* ForKeyword */,
            _a.from = 149 /* FromKeyword */,
            _a.function = 94 /* FunctionKeyword */,
            _a.get = 131 /* GetKeyword */,
            _a.if = 95 /* IfKeyword */,
            _a.implements = 113 /* ImplementsKeyword */,
            _a.import = 96 /* ImportKeyword */,
            _a.in = 97 /* InKeyword */,
            _a.infer = 132 /* InferKeyword */,
            _a.instanceof = 98 /* InstanceOfKeyword */,
            _a.interface = 114 /* InterfaceKeyword */,
            _a.is = 133 /* IsKeyword */,
            _a.keyof = 134 /* KeyOfKeyword */,
            _a.let = 115 /* LetKeyword */,
            _a.module = 135 /* ModuleKeyword */,
            _a.namespace = 136 /* NamespaceKeyword */,
            _a.never = 137 /* NeverKeyword */,
            _a.new = 99 /* NewKeyword */,
            _a.null = 100 /* NullKeyword */,
            _a.number = 140 /* NumberKeyword */,
            _a.object = 141 /* ObjectKeyword */,
            _a.package = 116 /* PackageKeyword */,
            _a.private = 117 /* PrivateKeyword */,
            _a.protected = 118 /* ProtectedKeyword */,
            _a.public = 119 /* PublicKeyword */,
            _a.readonly = 138 /* ReadonlyKeyword */,
            _a.require = 139 /* RequireKeyword */,
            _a.global = 150 /* GlobalKeyword */,
            _a.return = 101 /* ReturnKeyword */,
            _a.set = 142 /* SetKeyword */,
            _a.static = 120 /* StaticKeyword */,
            _a.string = 143 /* StringKeyword */,
            _a.super = 102 /* SuperKeyword */,
            _a.switch = 103 /* SwitchKeyword */,
            _a.symbol = 144 /* SymbolKeyword */,
            _a.this = 104 /* ThisKeyword */,
            _a.throw = 105 /* ThrowKeyword */,
            _a.true = 106 /* TrueKeyword */,
            _a.try = 107 /* TryKeyword */,
            _a.type = 145 /* TypeKeyword */,
            _a.typeof = 108 /* TypeOfKeyword */,
            _a.undefined = 146 /* UndefinedKeyword */,
            _a.unique = 147 /* UniqueKeyword */,
            _a.unknown = 148 /* UnknownKeyword */,
            _a.var = 109 /* VarKeyword */,
            _a.void = 110 /* VoidKeyword */,
            _a.while = 111 /* WhileKeyword */,
            _a.with = 112 /* WithKeyword */,
            _a.yield = 121 /* YieldKeyword */,
            _a.async = 126 /* AsyncKeyword */,
            _a.await = 127 /* AwaitKeyword */,
            _a.of = 152 /* OfKeyword */,
            _a);
        var textToKeyword = ts.createMapFromTemplate(textToKeywordObj);
        var textToToken = ts.createMapFromTemplate(__assign(__assign({}, textToKeywordObj), { "{": 18 /* OpenBraceToken */, "}": 19 /* CloseBraceToken */, "(": 20 /* OpenParenToken */, ")": 21 /* CloseParenToken */, "[": 22 /* OpenBracketToken */, "]": 23 /* CloseBracketToken */, ".": 24 /* DotToken */, "...": 25 /* DotDotDotToken */, ";": 26 /* SemicolonToken */, ",": 27 /* CommaToken */, "<": 29 /* LessThanToken */, ">": 31 /* GreaterThanToken */, "<=": 32 /* LessThanEqualsToken */, ">=": 33 /* GreaterThanEqualsToken */, "==": 34 /* EqualsEqualsToken */, "!=": 35 /* ExclamationEqualsToken */, "===": 36 /* EqualsEqualsEqualsToken */, "!==": 37 /* ExclamationEqualsEqualsToken */, "=>": 38 /* EqualsGreaterThanToken */, "+": 39 /* PlusToken */, "-": 40 /* MinusToken */, "**": 42 /* AsteriskAsteriskToken */, "*": 41 /* AsteriskToken */, "/": 43 /* SlashToken */, "%": 44 /* PercentToken */, "++": 45 /* PlusPlusToken */, "--": 46 /* MinusMinusToken */, "<<": 47 /* LessThanLessThanToken */, "</": 30 /* LessThanSlashToken */, ">>": 48 /* GreaterThanGreaterThanToken */, ">>>": 49 /* GreaterThanGreaterThanGreaterThanToken */, "&": 50 /* AmpersandToken */, "|": 51 /* BarToken */, "^": 52 /* CaretToken */, "!": 53 /* ExclamationToken */, "~": 54 /* TildeToken */, "&&": 55 /* AmpersandAmpersandToken */, "||": 56 /* BarBarToken */, "?": 57 /* QuestionToken */, "??": 60 /* QuestionQuestionToken */, "?.": 28 /* QuestionDotToken */, ":": 58 /* ColonToken */, "=": 62 /* EqualsToken */, "+=": 63 /* PlusEqualsToken */, "-=": 64 /* MinusEqualsToken */, "*=": 65 /* AsteriskEqualsToken */, "**=": 66 /* AsteriskAsteriskEqualsToken */, "/=": 67 /* SlashEqualsToken */, "%=": 68 /* PercentEqualsToken */, "<<=": 69 /* LessThanLessThanEqualsToken */, ">>=": 70 /* GreaterThanGreaterThanEqualsToken */, ">>>=": 71 /* GreaterThanGreaterThanGreaterThanEqualsToken */, "&=": 72 /* AmpersandEqualsToken */, "|=": 73 /* BarEqualsToken */, "^=": 74 /* CaretEqualsToken */, "@": 59 /* AtToken */, "`": 61 /* BacktickToken */ }));
        /*
            As per ECMAScript Language Specification 3th Edition, Section 7.6: Identifiers
            IdentifierStart ::
                Can contain Unicode 3.0.0 categories:
                Uppercase letter (Lu),
                Lowercase letter (Ll),
                Titlecase letter (Lt),
                Modifier letter (Lm),
                Other letter (Lo), or
                Letter number (Nl).
            IdentifierPart :: =
                Can contain IdentifierStart + Unicode 3.0.0 categories:
                Non-spacing mark (Mn),
                Combining spacing mark (Mc),
                Decimal number (Nd), or
                Connector punctuation (Pc).
    
            Codepoint ranges for ES3 Identifiers are extracted from the Unicode 3.0.0 specification at:
            http://www.unicode.org/Public/3.0-Update/UnicodeData-3.0.0.txt
        */
        var unicodeES3IdentifierStart = [170, 170, 181, 181, 186, 186, 192, 214, 216, 246, 248, 543, 546, 563, 592, 685, 688, 696, 699, 705, 720, 721, 736, 740, 750, 750, 890, 890, 902, 902, 904, 906, 908, 908, 910, 929, 931, 974, 976, 983, 986, 1011, 1024, 1153, 1164, 1220, 1223, 1224, 1227, 1228, 1232, 1269, 1272, 1273, 1329, 1366, 1369, 1369, 1377, 1415, 1488, 1514, 1520, 1522, 1569, 1594, 1600, 1610, 1649, 1747, 1749, 1749, 1765, 1766, 1786, 1788, 1808, 1808, 1810, 1836, 1920, 1957, 2309, 2361, 2365, 2365, 2384, 2384, 2392, 2401, 2437, 2444, 2447, 2448, 2451, 2472, 2474, 2480, 2482, 2482, 2486, 2489, 2524, 2525, 2527, 2529, 2544, 2545, 2565, 2570, 2575, 2576, 2579, 2600, 2602, 2608, 2610, 2611, 2613, 2614, 2616, 2617, 2649, 2652, 2654, 2654, 2674, 2676, 2693, 2699, 2701, 2701, 2703, 2705, 2707, 2728, 2730, 2736, 2738, 2739, 2741, 2745, 2749, 2749, 2768, 2768, 2784, 2784, 2821, 2828, 2831, 2832, 2835, 2856, 2858, 2864, 2866, 2867, 2870, 2873, 2877, 2877, 2908, 2909, 2911, 2913, 2949, 2954, 2958, 2960, 2962, 2965, 2969, 2970, 2972, 2972, 2974, 2975, 2979, 2980, 2984, 2986, 2990, 2997, 2999, 3001, 3077, 3084, 3086, 3088, 3090, 3112, 3114, 3123, 3125, 3129, 3168, 3169, 3205, 3212, 3214, 3216, 3218, 3240, 3242, 3251, 3253, 3257, 3294, 3294, 3296, 3297, 3333, 3340, 3342, 3344, 3346, 3368, 3370, 3385, 3424, 3425, 3461, 3478, 3482, 3505, 3507, 3515, 3517, 3517, 3520, 3526, 3585, 3632, 3634, 3635, 3648, 3654, 3713, 3714, 3716, 3716, 3719, 3720, 3722, 3722, 3725, 3725, 3732, 3735, 3737, 3743, 3745, 3747, 3749, 3749, 3751, 3751, 3754, 3755, 3757, 3760, 3762, 3763, 3773, 3773, 3776, 3780, 3782, 3782, 3804, 3805, 3840, 3840, 3904, 3911, 3913, 3946, 3976, 3979, 4096, 4129, 4131, 4135, 4137, 4138, 4176, 4181, 4256, 4293, 4304, 4342, 4352, 4441, 4447, 4514, 4520, 4601, 4608, 4614, 4616, 4678, 4680, 4680, 4682, 4685, 4688, 4694, 4696, 4696, 4698, 4701, 4704, 4742, 4744, 4744, 4746, 4749, 4752, 4782, 4784, 4784, 4786, 4789, 4792, 4798, 4800, 4800, 4802, 4805, 4808, 4814, 4816, 4822, 4824, 4846, 4848, 4878, 4880, 4880, 4882, 4885, 4888, 4894, 4896, 4934, 4936, 4954, 5024, 5108, 5121, 5740, 5743, 5750, 5761, 5786, 5792, 5866, 6016, 6067, 6176, 6263, 6272, 6312, 7680, 7835, 7840, 7929, 7936, 7957, 7960, 7965, 7968, 8005, 8008, 8013, 8016, 8023, 8025, 8025, 8027, 8027, 8029, 8029, 8031, 8061, 8064, 8116, 8118, 8124, 8126, 8126, 8130, 8132, 8134, 8140, 8144, 8147, 8150, 8155, 8160, 8172, 8178, 8180, 8182, 8188, 8319, 8319, 8450, 8450, 8455, 8455, 8458, 8467, 8469, 8469, 8473, 8477, 8484, 8484, 8486, 8486, 8488, 8488, 8490, 8493, 8495, 8497, 8499, 8505, 8544, 8579, 12293, 12295, 12321, 12329, 12337, 12341, 12344, 12346, 12353, 12436, 12445, 12446, 12449, 12538, 12540, 12542, 12549, 12588, 12593, 12686, 12704, 12727, 13312, 19893, 19968, 40869, 40960, 42124, 44032, 55203, 63744, 64045, 64256, 64262, 64275, 64279, 64285, 64285, 64287, 64296, 64298, 64310, 64312, 64316, 64318, 64318, 64320, 64321, 64323, 64324, 64326, 64433, 64467, 64829, 64848, 64911, 64914, 64967, 65008, 65019, 65136, 65138, 65140, 65140, 65142, 65276, 65313, 65338, 65345, 65370, 65382, 65470, 65474, 65479, 65482, 65487, 65490, 65495, 65498, 65500,];
        var unicodeES3IdentifierPart = [170, 170, 181, 181, 186, 186, 192, 214, 216, 246, 248, 543, 546, 563, 592, 685, 688, 696, 699, 705, 720, 721, 736, 740, 750, 750, 768, 846, 864, 866, 890, 890, 902, 902, 904, 906, 908, 908, 910, 929, 931, 974, 976, 983, 986, 1011, 1024, 1153, 1155, 1158, 1164, 1220, 1223, 1224, 1227, 1228, 1232, 1269, 1272, 1273, 1329, 1366, 1369, 1369, 1377, 1415, 1425, 1441, 1443, 1465, 1467, 1469, 1471, 1471, 1473, 1474, 1476, 1476, 1488, 1514, 1520, 1522, 1569, 1594, 1600, 1621, 1632, 1641, 1648, 1747, 1749, 1756, 1759, 1768, 1770, 1773, 1776, 1788, 1808, 1836, 1840, 1866, 1920, 1968, 2305, 2307, 2309, 2361, 2364, 2381, 2384, 2388, 2392, 2403, 2406, 2415, 2433, 2435, 2437, 2444, 2447, 2448, 2451, 2472, 2474, 2480, 2482, 2482, 2486, 2489, 2492, 2492, 2494, 2500, 2503, 2504, 2507, 2509, 2519, 2519, 2524, 2525, 2527, 2531, 2534, 2545, 2562, 2562, 2565, 2570, 2575, 2576, 2579, 2600, 2602, 2608, 2610, 2611, 2613, 2614, 2616, 2617, 2620, 2620, 2622, 2626, 2631, 2632, 2635, 2637, 2649, 2652, 2654, 2654, 2662, 2676, 2689, 2691, 2693, 2699, 2701, 2701, 2703, 2705, 2707, 2728, 2730, 2736, 2738, 2739, 2741, 2745, 2748, 2757, 2759, 2761, 2763, 2765, 2768, 2768, 2784, 2784, 2790, 2799, 2817, 2819, 2821, 2828, 2831, 2832, 2835, 2856, 2858, 2864, 2866, 2867, 2870, 2873, 2876, 2883, 2887, 2888, 2891, 2893, 2902, 2903, 2908, 2909, 2911, 2913, 2918, 2927, 2946, 2947, 2949, 2954, 2958, 2960, 2962, 2965, 2969, 2970, 2972, 2972, 2974, 2975, 2979, 2980, 2984, 2986, 2990, 2997, 2999, 3001, 3006, 3010, 3014, 3016, 3018, 3021, 3031, 3031, 3047, 3055, 3073, 3075, 3077, 3084, 3086, 3088, 3090, 3112, 3114, 3123, 3125, 3129, 3134, 3140, 3142, 3144, 3146, 3149, 3157, 3158, 3168, 3169, 3174, 3183, 3202, 3203, 3205, 3212, 3214, 3216, 3218, 3240, 3242, 3251, 3253, 3257, 3262, 3268, 3270, 3272, 3274, 3277, 3285, 3286, 3294, 3294, 3296, 3297, 3302, 3311, 3330, 3331, 3333, 3340, 3342, 3344, 3346, 3368, 3370, 3385, 3390, 3395, 3398, 3400, 3402, 3405, 3415, 3415, 3424, 3425, 3430, 3439, 3458, 3459, 3461, 3478, 3482, 3505, 3507, 3515, 3517, 3517, 3520, 3526, 3530, 3530, 3535, 3540, 3542, 3542, 3544, 3551, 3570, 3571, 3585, 3642, 3648, 3662, 3664, 3673, 3713, 3714, 3716, 3716, 3719, 3720, 3722, 3722, 3725, 3725, 3732, 3735, 3737, 3743, 3745, 3747, 3749, 3749, 3751, 3751, 3754, 3755, 3757, 3769, 3771, 3773, 3776, 3780, 3782, 3782, 3784, 3789, 3792, 3801, 3804, 3805, 3840, 3840, 3864, 3865, 3872, 3881, 3893, 3893, 3895, 3895, 3897, 3897, 3902, 3911, 3913, 3946, 3953, 3972, 3974, 3979, 3984, 3991, 3993, 4028, 4038, 4038, 4096, 4129, 4131, 4135, 4137, 4138, 4140, 4146, 4150, 4153, 4160, 4169, 4176, 4185, 4256, 4293, 4304, 4342, 4352, 4441, 4447, 4514, 4520, 4601, 4608, 4614, 4616, 4678, 4680, 4680, 4682, 4685, 4688, 4694, 4696, 4696, 4698, 4701, 4704, 4742, 4744, 4744, 4746, 4749, 4752, 4782, 4784, 4784, 4786, 4789, 4792, 4798, 4800, 4800, 4802, 4805, 4808, 4814, 4816, 4822, 4824, 4846, 4848, 4878, 4880, 4880, 4882, 4885, 4888, 4894, 4896, 4934, 4936, 4954, 4969, 4977, 5024, 5108, 5121, 5740, 5743, 5750, 5761, 5786, 5792, 5866, 6016, 6099, 6112, 6121, 6160, 6169, 6176, 6263, 6272, 6313, 7680, 7835, 7840, 7929, 7936, 7957, 7960, 7965, 7968, 8005, 8008, 8013, 8016, 8023, 8025, 8025, 8027, 8027, 8029, 8029, 8031, 8061, 8064, 8116, 8118, 8124, 8126, 8126, 8130, 8132, 8134, 8140, 8144, 8147, 8150, 8155, 8160, 8172, 8178, 8180, 8182, 8188, 8255, 8256, 8319, 8319, 8400, 8412, 8417, 8417, 8450, 8450, 8455, 8455, 8458, 8467, 8469, 8469, 8473, 8477, 8484, 8484, 8486, 8486, 8488, 8488, 8490, 8493, 8495, 8497, 8499, 8505, 8544, 8579, 12293, 12295, 12321, 12335, 12337, 12341, 12344, 12346, 12353, 12436, 12441, 12442, 12445, 12446, 12449, 12542, 12549, 12588, 12593, 12686, 12704, 12727, 13312, 19893, 19968, 40869, 40960, 42124, 44032, 55203, 63744, 64045, 64256, 64262, 64275, 64279, 64285, 64296, 64298, 64310, 64312, 64316, 64318, 64318, 64320, 64321, 64323, 64324, 64326, 64433, 64467, 64829, 64848, 64911, 64914, 64967, 65008, 65019, 65056, 65059, 65075, 65076, 65101, 65103, 65136, 65138, 65140, 65140, 65142, 65276, 65296, 65305, 65313, 65338, 65343, 65343, 65345, 65370, 65381, 65470, 65474, 65479, 65482, 65487, 65490, 65495, 65498, 65500,];
        /*
            As per ECMAScript Language Specification 5th Edition, Section 7.6: ISyntaxToken Names and Identifiers
            IdentifierStart ::
                Can contain Unicode 6.2 categories:
                Uppercase letter (Lu),
                Lowercase letter (Ll),
                Titlecase letter (Lt),
                Modifier letter (Lm),
                Other letter (Lo), or
                Letter number (Nl).
            IdentifierPart ::
                Can contain IdentifierStart + Unicode 6.2 categories:
                Non-spacing mark (Mn),
                Combining spacing mark (Mc),
                Decimal number (Nd),
                Connector punctuation (Pc),
                <ZWNJ>, or
                <ZWJ>.
    
            Codepoint ranges for ES5 Identifiers are extracted from the Unicode 6.2 specification at:
            http://www.unicode.org/Public/6.2.0/ucd/UnicodeData.txt
        */
        var unicodeES5IdentifierStart = [170, 170, 181, 181, 186, 186, 192, 214, 216, 246, 248, 705, 710, 721, 736, 740, 748, 748, 750, 750, 880, 884, 886, 887, 890, 893, 902, 902, 904, 906, 908, 908, 910, 929, 931, 1013, 1015, 1153, 1162, 1319, 1329, 1366, 1369, 1369, 1377, 1415, 1488, 1514, 1520, 1522, 1568, 1610, 1646, 1647, 1649, 1747, 1749, 1749, 1765, 1766, 1774, 1775, 1786, 1788, 1791, 1791, 1808, 1808, 1810, 1839, 1869, 1957, 1969, 1969, 1994, 2026, 2036, 2037, 2042, 2042, 2048, 2069, 2074, 2074, 2084, 2084, 2088, 2088, 2112, 2136, 2208, 2208, 2210, 2220, 2308, 2361, 2365, 2365, 2384, 2384, 2392, 2401, 2417, 2423, 2425, 2431, 2437, 2444, 2447, 2448, 2451, 2472, 2474, 2480, 2482, 2482, 2486, 2489, 2493, 2493, 2510, 2510, 2524, 2525, 2527, 2529, 2544, 2545, 2565, 2570, 2575, 2576, 2579, 2600, 2602, 2608, 2610, 2611, 2613, 2614, 2616, 2617, 2649, 2652, 2654, 2654, 2674, 2676, 2693, 2701, 2703, 2705, 2707, 2728, 2730, 2736, 2738, 2739, 2741, 2745, 2749, 2749, 2768, 2768, 2784, 2785, 2821, 2828, 2831, 2832, 2835, 2856, 2858, 2864, 2866, 2867, 2869, 2873, 2877, 2877, 2908, 2909, 2911, 2913, 2929, 2929, 2947, 2947, 2949, 2954, 2958, 2960, 2962, 2965, 2969, 2970, 2972, 2972, 2974, 2975, 2979, 2980, 2984, 2986, 2990, 3001, 3024, 3024, 3077, 3084, 3086, 3088, 3090, 3112, 3114, 3123, 3125, 3129, 3133, 3133, 3160, 3161, 3168, 3169, 3205, 3212, 3214, 3216, 3218, 3240, 3242, 3251, 3253, 3257, 3261, 3261, 3294, 3294, 3296, 3297, 3313, 3314, 3333, 3340, 3342, 3344, 3346, 3386, 3389, 3389, 3406, 3406, 3424, 3425, 3450, 3455, 3461, 3478, 3482, 3505, 3507, 3515, 3517, 3517, 3520, 3526, 3585, 3632, 3634, 3635, 3648, 3654, 3713, 3714, 3716, 3716, 3719, 3720, 3722, 3722, 3725, 3725, 3732, 3735, 3737, 3743, 3745, 3747, 3749, 3749, 3751, 3751, 3754, 3755, 3757, 3760, 3762, 3763, 3773, 3773, 3776, 3780, 3782, 3782, 3804, 3807, 3840, 3840, 3904, 3911, 3913, 3948, 3976, 3980, 4096, 4138, 4159, 4159, 4176, 4181, 4186, 4189, 4193, 4193, 4197, 4198, 4206, 4208, 4213, 4225, 4238, 4238, 4256, 4293, 4295, 4295, 4301, 4301, 4304, 4346, 4348, 4680, 4682, 4685, 4688, 4694, 4696, 4696, 4698, 4701, 4704, 4744, 4746, 4749, 4752, 4784, 4786, 4789, 4792, 4798, 4800, 4800, 4802, 4805, 4808, 4822, 4824, 4880, 4882, 4885, 4888, 4954, 4992, 5007, 5024, 5108, 5121, 5740, 5743, 5759, 5761, 5786, 5792, 5866, 5870, 5872, 5888, 5900, 5902, 5905, 5920, 5937, 5952, 5969, 5984, 5996, 5998, 6000, 6016, 6067, 6103, 6103, 6108, 6108, 6176, 6263, 6272, 6312, 6314, 6314, 6320, 6389, 6400, 6428, 6480, 6509, 6512, 6516, 6528, 6571, 6593, 6599, 6656, 6678, 6688, 6740, 6823, 6823, 6917, 6963, 6981, 6987, 7043, 7072, 7086, 7087, 7098, 7141, 7168, 7203, 7245, 7247, 7258, 7293, 7401, 7404, 7406, 7409, 7413, 7414, 7424, 7615, 7680, 7957, 7960, 7965, 7968, 8005, 8008, 8013, 8016, 8023, 8025, 8025, 8027, 8027, 8029, 8029, 8031, 8061, 8064, 8116, 8118, 8124, 8126, 8126, 8130, 8132, 8134, 8140, 8144, 8147, 8150, 8155, 8160, 8172, 8178, 8180, 8182, 8188, 8305, 8305, 8319, 8319, 8336, 8348, 8450, 8450, 8455, 8455, 8458, 8467, 8469, 8469, 8473, 8477, 8484, 8484, 8486, 8486, 8488, 8488, 8490, 8493, 8495, 8505, 8508, 8511, 8517, 8521, 8526, 8526, 8544, 8584, 11264, 11310, 11312, 11358, 11360, 11492, 11499, 11502, 11506, 11507, 11520, 11557, 11559, 11559, 11565, 11565, 11568, 11623, 11631, 11631, 11648, 11670, 11680, 11686, 11688, 11694, 11696, 11702, 11704, 11710, 11712, 11718, 11720, 11726, 11728, 11734, 11736, 11742, 11823, 11823, 12293, 12295, 12321, 12329, 12337, 12341, 12344, 12348, 12353, 12438, 12445, 12447, 12449, 12538, 12540, 12543, 12549, 12589, 12593, 12686, 12704, 12730, 12784, 12799, 13312, 19893, 19968, 40908, 40960, 42124, 42192, 42237, 42240, 42508, 42512, 42527, 42538, 42539, 42560, 42606, 42623, 42647, 42656, 42735, 42775, 42783, 42786, 42888, 42891, 42894, 42896, 42899, 42912, 42922, 43000, 43009, 43011, 43013, 43015, 43018, 43020, 43042, 43072, 43123, 43138, 43187, 43250, 43255, 43259, 43259, 43274, 43301, 43312, 43334, 43360, 43388, 43396, 43442, 43471, 43471, 43520, 43560, 43584, 43586, 43588, 43595, 43616, 43638, 43642, 43642, 43648, 43695, 43697, 43697, 43701, 43702, 43705, 43709, 43712, 43712, 43714, 43714, 43739, 43741, 43744, 43754, 43762, 43764, 43777, 43782, 43785, 43790, 43793, 43798, 43808, 43814, 43816, 43822, 43968, 44002, 44032, 55203, 55216, 55238, 55243, 55291, 63744, 64109, 64112, 64217, 64256, 64262, 64275, 64279, 64285, 64285, 64287, 64296, 64298, 64310, 64312, 64316, 64318, 64318, 64320, 64321, 64323, 64324, 64326, 64433, 64467, 64829, 64848, 64911, 64914, 64967, 65008, 65019, 65136, 65140, 65142, 65276, 65313, 65338, 65345, 65370, 65382, 65470, 65474, 65479, 65482, 65487, 65490, 65495, 65498, 65500,];
        var unicodeES5IdentifierPart = [170, 170, 181, 181, 186, 186, 192, 214, 216, 246, 248, 705, 710, 721, 736, 740, 748, 748, 750, 750, 768, 884, 886, 887, 890, 893, 902, 902, 904, 906, 908, 908, 910, 929, 931, 1013, 1015, 1153, 1155, 1159, 1162, 1319, 1329, 1366, 1369, 1369, 1377, 1415, 1425, 1469, 1471, 1471, 1473, 1474, 1476, 1477, 1479, 1479, 1488, 1514, 1520, 1522, 1552, 1562, 1568, 1641, 1646, 1747, 1749, 1756, 1759, 1768, 1770, 1788, 1791, 1791, 1808, 1866, 1869, 1969, 1984, 2037, 2042, 2042, 2048, 2093, 2112, 2139, 2208, 2208, 2210, 2220, 2276, 2302, 2304, 2403, 2406, 2415, 2417, 2423, 2425, 2431, 2433, 2435, 2437, 2444, 2447, 2448, 2451, 2472, 2474, 2480, 2482, 2482, 2486, 2489, 2492, 2500, 2503, 2504, 2507, 2510, 2519, 2519, 2524, 2525, 2527, 2531, 2534, 2545, 2561, 2563, 2565, 2570, 2575, 2576, 2579, 2600, 2602, 2608, 2610, 2611, 2613, 2614, 2616, 2617, 2620, 2620, 2622, 2626, 2631, 2632, 2635, 2637, 2641, 2641, 2649, 2652, 2654, 2654, 2662, 2677, 2689, 2691, 2693, 2701, 2703, 2705, 2707, 2728, 2730, 2736, 2738, 2739, 2741, 2745, 2748, 2757, 2759, 2761, 2763, 2765, 2768, 2768, 2784, 2787, 2790, 2799, 2817, 2819, 2821, 2828, 2831, 2832, 2835, 2856, 2858, 2864, 2866, 2867, 2869, 2873, 2876, 2884, 2887, 2888, 2891, 2893, 2902, 2903, 2908, 2909, 2911, 2915, 2918, 2927, 2929, 2929, 2946, 2947, 2949, 2954, 2958, 2960, 2962, 2965, 2969, 2970, 2972, 2972, 2974, 2975, 2979, 2980, 2984, 2986, 2990, 3001, 3006, 3010, 3014, 3016, 3018, 3021, 3024, 3024, 3031, 3031, 3046, 3055, 3073, 3075, 3077, 3084, 3086, 3088, 3090, 3112, 3114, 3123, 3125, 3129, 3133, 3140, 3142, 3144, 3146, 3149, 3157, 3158, 3160, 3161, 3168, 3171, 3174, 3183, 3202, 3203, 3205, 3212, 3214, 3216, 3218, 3240, 3242, 3251, 3253, 3257, 3260, 3268, 3270, 3272, 3274, 3277, 3285, 3286, 3294, 3294, 3296, 3299, 3302, 3311, 3313, 3314, 3330, 3331, 3333, 3340, 3342, 3344, 3346, 3386, 3389, 3396, 3398, 3400, 3402, 3406, 3415, 3415, 3424, 3427, 3430, 3439, 3450, 3455, 3458, 3459, 3461, 3478, 3482, 3505, 3507, 3515, 3517, 3517, 3520, 3526, 3530, 3530, 3535, 3540, 3542, 3542, 3544, 3551, 3570, 3571, 3585, 3642, 3648, 3662, 3664, 3673, 3713, 3714, 3716, 3716, 3719, 3720, 3722, 3722, 3725, 3725, 3732, 3735, 3737, 3743, 3745, 3747, 3749, 3749, 3751, 3751, 3754, 3755, 3757, 3769, 3771, 3773, 3776, 3780, 3782, 3782, 3784, 3789, 3792, 3801, 3804, 3807, 3840, 3840, 3864, 3865, 3872, 3881, 3893, 3893, 3895, 3895, 3897, 3897, 3902, 3911, 3913, 3948, 3953, 3972, 3974, 3991, 3993, 4028, 4038, 4038, 4096, 4169, 4176, 4253, 4256, 4293, 4295, 4295, 4301, 4301, 4304, 4346, 4348, 4680, 4682, 4685, 4688, 4694, 4696, 4696, 4698, 4701, 4704, 4744, 4746, 4749, 4752, 4784, 4786, 4789, 4792, 4798, 4800, 4800, 4802, 4805, 4808, 4822, 4824, 4880, 4882, 4885, 4888, 4954, 4957, 4959, 4992, 5007, 5024, 5108, 5121, 5740, 5743, 5759, 5761, 5786, 5792, 5866, 5870, 5872, 5888, 5900, 5902, 5908, 5920, 5940, 5952, 5971, 5984, 5996, 5998, 6000, 6002, 6003, 6016, 6099, 6103, 6103, 6108, 6109, 6112, 6121, 6155, 6157, 6160, 6169, 6176, 6263, 6272, 6314, 6320, 6389, 6400, 6428, 6432, 6443, 6448, 6459, 6470, 6509, 6512, 6516, 6528, 6571, 6576, 6601, 6608, 6617, 6656, 6683, 6688, 6750, 6752, 6780, 6783, 6793, 6800, 6809, 6823, 6823, 6912, 6987, 6992, 7001, 7019, 7027, 7040, 7155, 7168, 7223, 7232, 7241, 7245, 7293, 7376, 7378, 7380, 7414, 7424, 7654, 7676, 7957, 7960, 7965, 7968, 8005, 8008, 8013, 8016, 8023, 8025, 8025, 8027, 8027, 8029, 8029, 8031, 8061, 8064, 8116, 8118, 8124, 8126, 8126, 8130, 8132, 8134, 8140, 8144, 8147, 8150, 8155, 8160, 8172, 8178, 8180, 8182, 8188, 8204, 8205, 8255, 8256, 8276, 8276, 8305, 8305, 8319, 8319, 8336, 8348, 8400, 8412, 8417, 8417, 8421, 8432, 8450, 8450, 8455, 8455, 8458, 8467, 8469, 8469, 8473, 8477, 8484, 8484, 8486, 8486, 8488, 8488, 8490, 8493, 8495, 8505, 8508, 8511, 8517, 8521, 8526, 8526, 8544, 8584, 11264, 11310, 11312, 11358, 11360, 11492, 11499, 11507, 11520, 11557, 11559, 11559, 11565, 11565, 11568, 11623, 11631, 11631, 11647, 11670, 11680, 11686, 11688, 11694, 11696, 11702, 11704, 11710, 11712, 11718, 11720, 11726, 11728, 11734, 11736, 11742, 11744, 11775, 11823, 11823, 12293, 12295, 12321, 12335, 12337, 12341, 12344, 12348, 12353, 12438, 12441, 12442, 12445, 12447, 12449, 12538, 12540, 12543, 12549, 12589, 12593, 12686, 12704, 12730, 12784, 12799, 13312, 19893, 19968, 40908, 40960, 42124, 42192, 42237, 42240, 42508, 42512, 42539, 42560, 42607, 42612, 42621, 42623, 42647, 42655, 42737, 42775, 42783, 42786, 42888, 42891, 42894, 42896, 42899, 42912, 42922, 43000, 43047, 43072, 43123, 43136, 43204, 43216, 43225, 43232, 43255, 43259, 43259, 43264, 43309, 43312, 43347, 43360, 43388, 43392, 43456, 43471, 43481, 43520, 43574, 43584, 43597, 43600, 43609, 43616, 43638, 43642, 43643, 43648, 43714, 43739, 43741, 43744, 43759, 43762, 43766, 43777, 43782, 43785, 43790, 43793, 43798, 43808, 43814, 43816, 43822, 43968, 44010, 44012, 44013, 44016, 44025, 44032, 55203, 55216, 55238, 55243, 55291, 63744, 64109, 64112, 64217, 64256, 64262, 64275, 64279, 64285, 64296, 64298, 64310, 64312, 64316, 64318, 64318, 64320, 64321, 64323, 64324, 64326, 64433, 64467, 64829, 64848, 64911, 64914, 64967, 65008, 65019, 65024, 65039, 65056, 65062, 65075, 65076, 65101, 65103, 65136, 65140, 65142, 65276, 65296, 65305, 65313, 65338, 65343, 65343, 65345, 65370, 65382, 65470, 65474, 65479, 65482, 65487, 65490, 65495, 65498, 65500,];
        /**
         * Generated by scripts/regenerate-unicode-identifier-parts.js on node v12.4.0 with unicode 12.1
         * based on http://www.unicode.org/reports/tr31/ and https://www.ecma-international.org/ecma-262/6.0/#sec-names-and-keywords
         * unicodeESNextIdentifierStart corresponds to the ID_Start and Other_ID_Start property, and
         * unicodeESNextIdentifierPart corresponds to ID_Continue, Other_ID_Continue, plus ID_Start and Other_ID_Start
         */
        var unicodeESNextIdentifierStart = [65, 90, 97, 122, 170, 170, 181, 181, 186, 186, 192, 214, 216, 246, 248, 705, 710, 721, 736, 740, 748, 748, 750, 750, 880, 884, 886, 887, 890, 893, 895, 895, 902, 902, 904, 906, 908, 908, 910, 929, 931, 1013, 1015, 1153, 1162, 1327, 1329, 1366, 1369, 1369, 1376, 1416, 1488, 1514, 1519, 1522, 1568, 1610, 1646, 1647, 1649, 1747, 1749, 1749, 1765, 1766, 1774, 1775, 1786, 1788, 1791, 1791, 1808, 1808, 1810, 1839, 1869, 1957, 1969, 1969, 1994, 2026, 2036, 2037, 2042, 2042, 2048, 2069, 2074, 2074, 2084, 2084, 2088, 2088, 2112, 2136, 2144, 2154, 2208, 2228, 2230, 2237, 2308, 2361, 2365, 2365, 2384, 2384, 2392, 2401, 2417, 2432, 2437, 2444, 2447, 2448, 2451, 2472, 2474, 2480, 2482, 2482, 2486, 2489, 2493, 2493, 2510, 2510, 2524, 2525, 2527, 2529, 2544, 2545, 2556, 2556, 2565, 2570, 2575, 2576, 2579, 2600, 2602, 2608, 2610, 2611, 2613, 2614, 2616, 2617, 2649, 2652, 2654, 2654, 2674, 2676, 2693, 2701, 2703, 2705, 2707, 2728, 2730, 2736, 2738, 2739, 2741, 2745, 2749, 2749, 2768, 2768, 2784, 2785, 2809, 2809, 2821, 2828, 2831, 2832, 2835, 2856, 2858, 2864, 2866, 2867, 2869, 2873, 2877, 2877, 2908, 2909, 2911, 2913, 2929, 2929, 2947, 2947, 2949, 2954, 2958, 2960, 2962, 2965, 2969, 2970, 2972, 2972, 2974, 2975, 2979, 2980, 2984, 2986, 2990, 3001, 3024, 3024, 3077, 3084, 3086, 3088, 3090, 3112, 3114, 3129, 3133, 3133, 3160, 3162, 3168, 3169, 3200, 3200, 3205, 3212, 3214, 3216, 3218, 3240, 3242, 3251, 3253, 3257, 3261, 3261, 3294, 3294, 3296, 3297, 3313, 3314, 3333, 3340, 3342, 3344, 3346, 3386, 3389, 3389, 3406, 3406, 3412, 3414, 3423, 3425, 3450, 3455, 3461, 3478, 3482, 3505, 3507, 3515, 3517, 3517, 3520, 3526, 3585, 3632, 3634, 3635, 3648, 3654, 3713, 3714, 3716, 3716, 3718, 3722, 3724, 3747, 3749, 3749, 3751, 3760, 3762, 3763, 3773, 3773, 3776, 3780, 3782, 3782, 3804, 3807, 3840, 3840, 3904, 3911, 3913, 3948, 3976, 3980, 4096, 4138, 4159, 4159, 4176, 4181, 4186, 4189, 4193, 4193, 4197, 4198, 4206, 4208, 4213, 4225, 4238, 4238, 4256, 4293, 4295, 4295, 4301, 4301, 4304, 4346, 4348, 4680, 4682, 4685, 4688, 4694, 4696, 4696, 4698, 4701, 4704, 4744, 4746, 4749, 4752, 4784, 4786, 4789, 4792, 4798, 4800, 4800, 4802, 4805, 4808, 4822, 4824, 4880, 4882, 4885, 4888, 4954, 4992, 5007, 5024, 5109, 5112, 5117, 5121, 5740, 5743, 5759, 5761, 5786, 5792, 5866, 5870, 5880, 5888, 5900, 5902, 5905, 5920, 5937, 5952, 5969, 5984, 5996, 5998, 6000, 6016, 6067, 6103, 6103, 6108, 6108, 6176, 6264, 6272, 6312, 6314, 6314, 6320, 6389, 6400, 6430, 6480, 6509, 6512, 6516, 6528, 6571, 6576, 6601, 6656, 6678, 6688, 6740, 6823, 6823, 6917, 6963, 6981, 6987, 7043, 7072, 7086, 7087, 7098, 7141, 7168, 7203, 7245, 7247, 7258, 7293, 7296, 7304, 7312, 7354, 7357, 7359, 7401, 7404, 7406, 7411, 7413, 7414, 7418, 7418, 7424, 7615, 7680, 7957, 7960, 7965, 7968, 8005, 8008, 8013, 8016, 8023, 8025, 8025, 8027, 8027, 8029, 8029, 8031, 8061, 8064, 8116, 8118, 8124, 8126, 8126, 8130, 8132, 8134, 8140, 8144, 8147, 8150, 8155, 8160, 8172, 8178, 8180, 8182, 8188, 8305, 8305, 8319, 8319, 8336, 8348, 8450, 8450, 8455, 8455, 8458, 8467, 8469, 8469, 8472, 8477, 8484, 8484, 8486, 8486, 8488, 8488, 8490, 8505, 8508, 8511, 8517, 8521, 8526, 8526, 8544, 8584, 11264, 11310, 11312, 11358, 11360, 11492, 11499, 11502, 11506, 11507, 11520, 11557, 11559, 11559, 11565, 11565, 11568, 11623, 11631, 11631, 11648, 11670, 11680, 11686, 11688, 11694, 11696, 11702, 11704, 11710, 11712, 11718, 11720, 11726, 11728, 11734, 11736, 11742, 12293, 12295, 12321, 12329, 12337, 12341, 12344, 12348, 12353, 12438, 12443, 12447, 12449, 12538, 12540, 12543, 12549, 12591, 12593, 12686, 12704, 12730, 12784, 12799, 13312, 19893, 19968, 40943, 40960, 42124, 42192, 42237, 42240, 42508, 42512, 42527, 42538, 42539, 42560, 42606, 42623, 42653, 42656, 42735, 42775, 42783, 42786, 42888, 42891, 42943, 42946, 42950, 42999, 43009, 43011, 43013, 43015, 43018, 43020, 43042, 43072, 43123, 43138, 43187, 43250, 43255, 43259, 43259, 43261, 43262, 43274, 43301, 43312, 43334, 43360, 43388, 43396, 43442, 43471, 43471, 43488, 43492, 43494, 43503, 43514, 43518, 43520, 43560, 43584, 43586, 43588, 43595, 43616, 43638, 43642, 43642, 43646, 43695, 43697, 43697, 43701, 43702, 43705, 43709, 43712, 43712, 43714, 43714, 43739, 43741, 43744, 43754, 43762, 43764, 43777, 43782, 43785, 43790, 43793, 43798, 43808, 43814, 43816, 43822, 43824, 43866, 43868, 43879, 43888, 44002, 44032, 55203, 55216, 55238, 55243, 55291, 63744, 64109, 64112, 64217, 64256, 64262, 64275, 64279, 64285, 64285, 64287, 64296, 64298, 64310, 64312, 64316, 64318, 64318, 64320, 64321, 64323, 64324, 64326, 64433, 64467, 64829, 64848, 64911, 64914, 64967, 65008, 65019, 65136, 65140, 65142, 65276, 65313, 65338, 65345, 65370, 65382, 65470, 65474, 65479, 65482, 65487, 65490, 65495, 65498, 65500, 65536, 65547, 65549, 65574, 65576, 65594, 65596, 65597, 65599, 65613, 65616, 65629, 65664, 65786, 65856, 65908, 66176, 66204, 66208, 66256, 66304, 66335, 66349, 66378, 66384, 66421, 66432, 66461, 66464, 66499, 66504, 66511, 66513, 66517, 66560, 66717, 66736, 66771, 66776, 66811, 66816, 66855, 66864, 66915, 67072, 67382, 67392, 67413, 67424, 67431, 67584, 67589, 67592, 67592, 67594, 67637, 67639, 67640, 67644, 67644, 67647, 67669, 67680, 67702, 67712, 67742, 67808, 67826, 67828, 67829, 67840, 67861, 67872, 67897, 67968, 68023, 68030, 68031, 68096, 68096, 68112, 68115, 68117, 68119, 68121, 68149, 68192, 68220, 68224, 68252, 68288, 68295, 68297, 68324, 68352, 68405, 68416, 68437, 68448, 68466, 68480, 68497, 68608, 68680, 68736, 68786, 68800, 68850, 68864, 68899, 69376, 69404, 69415, 69415, 69424, 69445, 69600, 69622, 69635, 69687, 69763, 69807, 69840, 69864, 69891, 69926, 69956, 69956, 69968, 70002, 70006, 70006, 70019, 70066, 70081, 70084, 70106, 70106, 70108, 70108, 70144, 70161, 70163, 70187, 70272, 70278, 70280, 70280, 70282, 70285, 70287, 70301, 70303, 70312, 70320, 70366, 70405, 70412, 70415, 70416, 70419, 70440, 70442, 70448, 70450, 70451, 70453, 70457, 70461, 70461, 70480, 70480, 70493, 70497, 70656, 70708, 70727, 70730, 70751, 70751, 70784, 70831, 70852, 70853, 70855, 70855, 71040, 71086, 71128, 71131, 71168, 71215, 71236, 71236, 71296, 71338, 71352, 71352, 71424, 71450, 71680, 71723, 71840, 71903, 71935, 71935, 72096, 72103, 72106, 72144, 72161, 72161, 72163, 72163, 72192, 72192, 72203, 72242, 72250, 72250, 72272, 72272, 72284, 72329, 72349, 72349, 72384, 72440, 72704, 72712, 72714, 72750, 72768, 72768, 72818, 72847, 72960, 72966, 72968, 72969, 72971, 73008, 73030, 73030, 73056, 73061, 73063, 73064, 73066, 73097, 73112, 73112, 73440, 73458, 73728, 74649, 74752, 74862, 74880, 75075, 77824, 78894, 82944, 83526, 92160, 92728, 92736, 92766, 92880, 92909, 92928, 92975, 92992, 92995, 93027, 93047, 93053, 93071, 93760, 93823, 93952, 94026, 94032, 94032, 94099, 94111, 94176, 94177, 94179, 94179, 94208, 100343, 100352, 101106, 110592, 110878, 110928, 110930, 110948, 110951, 110960, 111355, 113664, 113770, 113776, 113788, 113792, 113800, 113808, 113817, 119808, 119892, 119894, 119964, 119966, 119967, 119970, 119970, 119973, 119974, 119977, 119980, 119982, 119993, 119995, 119995, 119997, 120003, 120005, 120069, 120071, 120074, 120077, 120084, 120086, 120092, 120094, 120121, 120123, 120126, 120128, 120132, 120134, 120134, 120138, 120144, 120146, 120485, 120488, 120512, 120514, 120538, 120540, 120570, 120572, 120596, 120598, 120628, 120630, 120654, 120656, 120686, 120688, 120712, 120714, 120744, 120746, 120770, 120772, 120779, 123136, 123180, 123191, 123197, 123214, 123214, 123584, 123627, 124928, 125124, 125184, 125251, 125259, 125259, 126464, 126467, 126469, 126495, 126497, 126498, 126500, 126500, 126503, 126503, 126505, 126514, 126516, 126519, 126521, 126521, 126523, 126523, 126530, 126530, 126535, 126535, 126537, 126537, 126539, 126539, 126541, 126543, 126545, 126546, 126548, 126548, 126551, 126551, 126553, 126553, 126555, 126555, 126557, 126557, 126559, 126559, 126561, 126562, 126564, 126564, 126567, 126570, 126572, 126578, 126580, 126583, 126585, 126588, 126590, 126590, 126592, 126601, 126603, 126619, 126625, 126627, 126629, 126633, 126635, 126651, 131072, 173782, 173824, 177972, 177984, 178205, 178208, 183969, 183984, 191456, 194560, 195101];
        var unicodeESNextIdentifierPart = [48, 57, 65, 90, 95, 95, 97, 122, 170, 170, 181, 181, 183, 183, 186, 186, 192, 214, 216, 246, 248, 705, 710, 721, 736, 740, 748, 748, 750, 750, 768, 884, 886, 887, 890, 893, 895, 895, 902, 906, 908, 908, 910, 929, 931, 1013, 1015, 1153, 1155, 1159, 1162, 1327, 1329, 1366, 1369, 1369, 1376, 1416, 1425, 1469, 1471, 1471, 1473, 1474, 1476, 1477, 1479, 1479, 1488, 1514, 1519, 1522, 1552, 1562, 1568, 1641, 1646, 1747, 1749, 1756, 1759, 1768, 1770, 1788, 1791, 1791, 1808, 1866, 1869, 1969, 1984, 2037, 2042, 2042, 2045, 2045, 2048, 2093, 2112, 2139, 2144, 2154, 2208, 2228, 2230, 2237, 2259, 2273, 2275, 2403, 2406, 2415, 2417, 2435, 2437, 2444, 2447, 2448, 2451, 2472, 2474, 2480, 2482, 2482, 2486, 2489, 2492, 2500, 2503, 2504, 2507, 2510, 2519, 2519, 2524, 2525, 2527, 2531, 2534, 2545, 2556, 2556, 2558, 2558, 2561, 2563, 2565, 2570, 2575, 2576, 2579, 2600, 2602, 2608, 2610, 2611, 2613, 2614, 2616, 2617, 2620, 2620, 2622, 2626, 2631, 2632, 2635, 2637, 2641, 2641, 2649, 2652, 2654, 2654, 2662, 2677, 2689, 2691, 2693, 2701, 2703, 2705, 2707, 2728, 2730, 2736, 2738, 2739, 2741, 2745, 2748, 2757, 2759, 2761, 2763, 2765, 2768, 2768, 2784, 2787, 2790, 2799, 2809, 2815, 2817, 2819, 2821, 2828, 2831, 2832, 2835, 2856, 2858, 2864, 2866, 2867, 2869, 2873, 2876, 2884, 2887, 2888, 2891, 2893, 2902, 2903, 2908, 2909, 2911, 2915, 2918, 2927, 2929, 2929, 2946, 2947, 2949, 2954, 2958, 2960, 2962, 2965, 2969, 2970, 2972, 2972, 2974, 2975, 2979, 2980, 2984, 2986, 2990, 3001, 3006, 3010, 3014, 3016, 3018, 3021, 3024, 3024, 3031, 3031, 3046, 3055, 3072, 3084, 3086, 3088, 3090, 3112, 3114, 3129, 3133, 3140, 3142, 3144, 3146, 3149, 3157, 3158, 3160, 3162, 3168, 3171, 3174, 3183, 3200, 3203, 3205, 3212, 3214, 3216, 3218, 3240, 3242, 3251, 3253, 3257, 3260, 3268, 3270, 3272, 3274, 3277, 3285, 3286, 3294, 3294, 3296, 3299, 3302, 3311, 3313, 3314, 3328, 3331, 3333, 3340, 3342, 3344, 3346, 3396, 3398, 3400, 3402, 3406, 3412, 3415, 3423, 3427, 3430, 3439, 3450, 3455, 3458, 3459, 3461, 3478, 3482, 3505, 3507, 3515, 3517, 3517, 3520, 3526, 3530, 3530, 3535, 3540, 3542, 3542, 3544, 3551, 3558, 3567, 3570, 3571, 3585, 3642, 3648, 3662, 3664, 3673, 3713, 3714, 3716, 3716, 3718, 3722, 3724, 3747, 3749, 3749, 3751, 3773, 3776, 3780, 3782, 3782, 3784, 3789, 3792, 3801, 3804, 3807, 3840, 3840, 3864, 3865, 3872, 3881, 3893, 3893, 3895, 3895, 3897, 3897, 3902, 3911, 3913, 3948, 3953, 3972, 3974, 3991, 3993, 4028, 4038, 4038, 4096, 4169, 4176, 4253, 4256, 4293, 4295, 4295, 4301, 4301, 4304, 4346, 4348, 4680, 4682, 4685, 4688, 4694, 4696, 4696, 4698, 4701, 4704, 4744, 4746, 4749, 4752, 4784, 4786, 4789, 4792, 4798, 4800, 4800, 4802, 4805, 4808, 4822, 4824, 4880, 4882, 4885, 4888, 4954, 4957, 4959, 4969, 4977, 4992, 5007, 5024, 5109, 5112, 5117, 5121, 5740, 5743, 5759, 5761, 5786, 5792, 5866, 5870, 5880, 5888, 5900, 5902, 5908, 5920, 5940, 5952, 5971, 5984, 5996, 5998, 6000, 6002, 6003, 6016, 6099, 6103, 6103, 6108, 6109, 6112, 6121, 6155, 6157, 6160, 6169, 6176, 6264, 6272, 6314, 6320, 6389, 6400, 6430, 6432, 6443, 6448, 6459, 6470, 6509, 6512, 6516, 6528, 6571, 6576, 6601, 6608, 6618, 6656, 6683, 6688, 6750, 6752, 6780, 6783, 6793, 6800, 6809, 6823, 6823, 6832, 6845, 6912, 6987, 6992, 7001, 7019, 7027, 7040, 7155, 7168, 7223, 7232, 7241, 7245, 7293, 7296, 7304, 7312, 7354, 7357, 7359, 7376, 7378, 7380, 7418, 7424, 7673, 7675, 7957, 7960, 7965, 7968, 8005, 8008, 8013, 8016, 8023, 8025, 8025, 8027, 8027, 8029, 8029, 8031, 8061, 8064, 8116, 8118, 8124, 8126, 8126, 8130, 8132, 8134, 8140, 8144, 8147, 8150, 8155, 8160, 8172, 8178, 8180, 8182, 8188, 8255, 8256, 8276, 8276, 8305, 8305, 8319, 8319, 8336, 8348, 8400, 8412, 8417, 8417, 8421, 8432, 8450, 8450, 8455, 8455, 8458, 8467, 8469, 8469, 8472, 8477, 8484, 8484, 8486, 8486, 8488, 8488, 8490, 8505, 8508, 8511, 8517, 8521, 8526, 8526, 8544, 8584, 11264, 11310, 11312, 11358, 11360, 11492, 11499, 11507, 11520, 11557, 11559, 11559, 11565, 11565, 11568, 11623, 11631, 11631, 11647, 11670, 11680, 11686, 11688, 11694, 11696, 11702, 11704, 11710, 11712, 11718, 11720, 11726, 11728, 11734, 11736, 11742, 11744, 11775, 12293, 12295, 12321, 12335, 12337, 12341, 12344, 12348, 12353, 12438, 12441, 12447, 12449, 12538, 12540, 12543, 12549, 12591, 12593, 12686, 12704, 12730, 12784, 12799, 13312, 19893, 19968, 40943, 40960, 42124, 42192, 42237, 42240, 42508, 42512, 42539, 42560, 42607, 42612, 42621, 42623, 42737, 42775, 42783, 42786, 42888, 42891, 42943, 42946, 42950, 42999, 43047, 43072, 43123, 43136, 43205, 43216, 43225, 43232, 43255, 43259, 43259, 43261, 43309, 43312, 43347, 43360, 43388, 43392, 43456, 43471, 43481, 43488, 43518, 43520, 43574, 43584, 43597, 43600, 43609, 43616, 43638, 43642, 43714, 43739, 43741, 43744, 43759, 43762, 43766, 43777, 43782, 43785, 43790, 43793, 43798, 43808, 43814, 43816, 43822, 43824, 43866, 43868, 43879, 43888, 44010, 44012, 44013, 44016, 44025, 44032, 55203, 55216, 55238, 55243, 55291, 63744, 64109, 64112, 64217, 64256, 64262, 64275, 64279, 64285, 64296, 64298, 64310, 64312, 64316, 64318, 64318, 64320, 64321, 64323, 64324, 64326, 64433, 64467, 64829, 64848, 64911, 64914, 64967, 65008, 65019, 65024, 65039, 65056, 65071, 65075, 65076, 65101, 65103, 65136, 65140, 65142, 65276, 65296, 65305, 65313, 65338, 65343, 65343, 65345, 65370, 65382, 65470, 65474, 65479, 65482, 65487, 65490, 65495, 65498, 65500, 65536, 65547, 65549, 65574, 65576, 65594, 65596, 65597, 65599, 65613, 65616, 65629, 65664, 65786, 65856, 65908, 66045, 66045, 66176, 66204, 66208, 66256, 66272, 66272, 66304, 66335, 66349, 66378, 66384, 66426, 66432, 66461, 66464, 66499, 66504, 66511, 66513, 66517, 66560, 66717, 66720, 66729, 66736, 66771, 66776, 66811, 66816, 66855, 66864, 66915, 67072, 67382, 67392, 67413, 67424, 67431, 67584, 67589, 67592, 67592, 67594, 67637, 67639, 67640, 67644, 67644, 67647, 67669, 67680, 67702, 67712, 67742, 67808, 67826, 67828, 67829, 67840, 67861, 67872, 67897, 67968, 68023, 68030, 68031, 68096, 68099, 68101, 68102, 68108, 68115, 68117, 68119, 68121, 68149, 68152, 68154, 68159, 68159, 68192, 68220, 68224, 68252, 68288, 68295, 68297, 68326, 68352, 68405, 68416, 68437, 68448, 68466, 68480, 68497, 68608, 68680, 68736, 68786, 68800, 68850, 68864, 68903, 68912, 68921, 69376, 69404, 69415, 69415, 69424, 69456, 69600, 69622, 69632, 69702, 69734, 69743, 69759, 69818, 69840, 69864, 69872, 69881, 69888, 69940, 69942, 69951, 69956, 69958, 69968, 70003, 70006, 70006, 70016, 70084, 70089, 70092, 70096, 70106, 70108, 70108, 70144, 70161, 70163, 70199, 70206, 70206, 70272, 70278, 70280, 70280, 70282, 70285, 70287, 70301, 70303, 70312, 70320, 70378, 70384, 70393, 70400, 70403, 70405, 70412, 70415, 70416, 70419, 70440, 70442, 70448, 70450, 70451, 70453, 70457, 70459, 70468, 70471, 70472, 70475, 70477, 70480, 70480, 70487, 70487, 70493, 70499, 70502, 70508, 70512, 70516, 70656, 70730, 70736, 70745, 70750, 70751, 70784, 70853, 70855, 70855, 70864, 70873, 71040, 71093, 71096, 71104, 71128, 71133, 71168, 71232, 71236, 71236, 71248, 71257, 71296, 71352, 71360, 71369, 71424, 71450, 71453, 71467, 71472, 71481, 71680, 71738, 71840, 71913, 71935, 71935, 72096, 72103, 72106, 72151, 72154, 72161, 72163, 72164, 72192, 72254, 72263, 72263, 72272, 72345, 72349, 72349, 72384, 72440, 72704, 72712, 72714, 72758, 72760, 72768, 72784, 72793, 72818, 72847, 72850, 72871, 72873, 72886, 72960, 72966, 72968, 72969, 72971, 73014, 73018, 73018, 73020, 73021, 73023, 73031, 73040, 73049, 73056, 73061, 73063, 73064, 73066, 73102, 73104, 73105, 73107, 73112, 73120, 73129, 73440, 73462, 73728, 74649, 74752, 74862, 74880, 75075, 77824, 78894, 82944, 83526, 92160, 92728, 92736, 92766, 92768, 92777, 92880, 92909, 92912, 92916, 92928, 92982, 92992, 92995, 93008, 93017, 93027, 93047, 93053, 93071, 93760, 93823, 93952, 94026, 94031, 94087, 94095, 94111, 94176, 94177, 94179, 94179, 94208, 100343, 100352, 101106, 110592, 110878, 110928, 110930, 110948, 110951, 110960, 111355, 113664, 113770, 113776, 113788, 113792, 113800, 113808, 113817, 113821, 113822, 119141, 119145, 119149, 119154, 119163, 119170, 119173, 119179, 119210, 119213, 119362, 119364, 119808, 119892, 119894, 119964, 119966, 119967, 119970, 119970, 119973, 119974, 119977, 119980, 119982, 119993, 119995, 119995, 119997, 120003, 120005, 120069, 120071, 120074, 120077, 120084, 120086, 120092, 120094, 120121, 120123, 120126, 120128, 120132, 120134, 120134, 120138, 120144, 120146, 120485, 120488, 120512, 120514, 120538, 120540, 120570, 120572, 120596, 120598, 120628, 120630, 120654, 120656, 120686, 120688, 120712, 120714, 120744, 120746, 120770, 120772, 120779, 120782, 120831, 121344, 121398, 121403, 121452, 121461, 121461, 121476, 121476, 121499, 121503, 121505, 121519, 122880, 122886, 122888, 122904, 122907, 122913, 122915, 122916, 122918, 122922, 123136, 123180, 123184, 123197, 123200, 123209, 123214, 123214, 123584, 123641, 124928, 125124, 125136, 125142, 125184, 125259, 125264, 125273, 126464, 126467, 126469, 126495, 126497, 126498, 126500, 126500, 126503, 126503, 126505, 126514, 126516, 126519, 126521, 126521, 126523, 126523, 126530, 126530, 126535, 126535, 126537, 126537, 126539, 126539, 126541, 126543, 126545, 126546, 126548, 126548, 126551, 126551, 126553, 126553, 126555, 126555, 126557, 126557, 126559, 126559, 126561, 126562, 126564, 126564, 126567, 126570, 126572, 126578, 126580, 126583, 126585, 126588, 126590, 126590, 126592, 126601, 126603, 126619, 126625, 126627, 126629, 126633, 126635, 126651, 131072, 173782, 173824, 177972, 177984, 178205, 178208, 183969, 183984, 191456, 194560, 195101, 917760, 917999];
        function lookupInUnicodeMap(code, map) {
            // Bail out quickly if it couldn't possibly be in the map.
            if (code < map[0]) {
                return false;
            }
            // Perform binary search in one of the Unicode range maps
            var lo = 0;
            var hi = map.length;
            var mid;
            while (lo + 1 < hi) {
                mid = lo + (hi - lo) / 2;
                // mid has to be even to catch a range's beginning
                mid -= mid % 2;
                if (map[mid] <= code && code <= map[mid + 1]) {
                    return true;
                }
                if (code < map[mid]) {
                    hi = mid;
                }
                else {
                    lo = mid + 2;
                }
            }
            return false;
        }
        /* @internal */ function isUnicodeIdentifierStart(code, languageVersion) {
            return languageVersion >= 2 /* ES2015 */ ?
                lookupInUnicodeMap(code, unicodeESNextIdentifierStart) :
                languageVersion === 1 /* ES5 */ ? lookupInUnicodeMap(code, unicodeES5IdentifierStart) :
                    lookupInUnicodeMap(code, unicodeES3IdentifierStart);
        }
        ts.isUnicodeIdentifierStart = isUnicodeIdentifierStart;
        function isUnicodeIdentifierPart(code, languageVersion) {
            return languageVersion >= 2 /* ES2015 */ ?
                lookupInUnicodeMap(code, unicodeESNextIdentifierPart) :
                languageVersion === 1 /* ES5 */ ? lookupInUnicodeMap(code, unicodeES5IdentifierPart) :
                    lookupInUnicodeMap(code, unicodeES3IdentifierPart);
        }
        function makeReverseMap(source) {
            var result = [];
            source.forEach(function (value, name) {
                result[value] = name;
            });
            return result;
        }
        var tokenStrings = makeReverseMap(textToToken);
        function tokenToString(t) {
            return tokenStrings[t];
        }
        ts.tokenToString = tokenToString;
        /* @internal */
        function stringToToken(s) {
            return textToToken.get(s);
        }
        ts.stringToToken = stringToToken;
        /* @internal */
        function computeLineStarts(text) {
            var result = new Array();
            var pos = 0;
            var lineStart = 0;
            while (pos < text.length) {
                var ch = text.charCodeAt(pos);
                pos++;
                switch (ch) {
                    case 13 /* carriageReturn */:
                        if (text.charCodeAt(pos) === 10 /* lineFeed */) {
                            pos++;
                        }
                    // falls through
                    case 10 /* lineFeed */:
                        result.push(lineStart);
                        lineStart = pos;
                        break;
                    default:
                        if (ch > 127 /* maxAsciiCharacter */ && isLineBreak(ch)) {
                            result.push(lineStart);
                            lineStart = pos;
                        }
                        break;
                }
            }
            result.push(lineStart);
            return result;
        }
        ts.computeLineStarts = computeLineStarts;
        function getPositionOfLineAndCharacter(sourceFile, line, character, allowEdits) {
            return sourceFile.getPositionOfLineAndCharacter ?
                sourceFile.getPositionOfLineAndCharacter(line, character, allowEdits) :
                computePositionOfLineAndCharacter(getLineStarts(sourceFile), line, character, sourceFile.text, allowEdits);
        }
        ts.getPositionOfLineAndCharacter = getPositionOfLineAndCharacter;
        /* @internal */
        function computePositionOfLineAndCharacter(lineStarts, line, character, debugText, allowEdits) {
            if (line < 0 || line >= lineStarts.length) {
                if (allowEdits) {
                    // Clamp line to nearest allowable value
                    line = line < 0 ? 0 : line >= lineStarts.length ? lineStarts.length - 1 : line;
                }
                else {
                    ts.Debug.fail("Bad line number. Line: " + line + ", lineStarts.length: " + lineStarts.length + " , line map is correct? " + (debugText !== undefined ? ts.arraysEqual(lineStarts, computeLineStarts(debugText)) : "unknown"));
                }
            }
            var res = lineStarts[line] + character;
            if (allowEdits) {
                // Clamp to nearest allowable values to allow the underlying to be edited without crashing (accuracy is lost, instead)
                // TODO: Somehow track edits between file as it was during the creation of sourcemap we have and the current file and
                // apply them to the computed position to improve accuracy
                return res > lineStarts[line + 1] ? lineStarts[line + 1] : typeof debugText === "string" && res > debugText.length ? debugText.length : res;
            }
            if (line < lineStarts.length - 1) {
                ts.Debug.assert(res < lineStarts[line + 1]);
            }
            else if (debugText !== undefined) {
                ts.Debug.assert(res <= debugText.length); // Allow single character overflow for trailing newline
            }
            return res;
        }
        ts.computePositionOfLineAndCharacter = computePositionOfLineAndCharacter;
        /* @internal */
        function getLineStarts(sourceFile) {
            return sourceFile.lineMap || (sourceFile.lineMap = computeLineStarts(sourceFile.text));
        }
        ts.getLineStarts = getLineStarts;
        /* @internal */
        /**
         * We assume the first line starts at position 0 and 'position' is non-negative.
         */
        function computeLineAndCharacterOfPosition(lineStarts, position) {
            var lineNumber = ts.binarySearch(lineStarts, position, ts.identity, ts.compareValues);
            if (lineNumber < 0) {
                // If the actual position was not found,
                // the binary search returns the 2's-complement of the next line start
                // e.g. if the line starts at [5, 10, 23, 80] and the position requested was 20
                // then the search will return -2.
                //
                // We want the index of the previous line start, so we subtract 1.
                // Review 2's-complement if this is confusing.
                lineNumber = ~lineNumber - 1;
                ts.Debug.assert(lineNumber !== -1, "position cannot precede the beginning of the file");
            }
            return {
                line: lineNumber,
                character: position - lineStarts[lineNumber]
            };
        }
        ts.computeLineAndCharacterOfPosition = computeLineAndCharacterOfPosition;
        function getLineAndCharacterOfPosition(sourceFile, position) {
            return computeLineAndCharacterOfPosition(getLineStarts(sourceFile), position);
        }
        ts.getLineAndCharacterOfPosition = getLineAndCharacterOfPosition;
        function isWhiteSpaceLike(ch) {
            return isWhiteSpaceSingleLine(ch) || isLineBreak(ch);
        }
        ts.isWhiteSpaceLike = isWhiteSpaceLike;
        /** Does not include line breaks. For that, see isWhiteSpaceLike. */
        function isWhiteSpaceSingleLine(ch) {
            // Note: nextLine is in the Zs space, and should be considered to be a whitespace.
            // It is explicitly not a line-break as it isn't in the exact set specified by EcmaScript.
            return ch === 32 /* space */ ||
                ch === 9 /* tab */ ||
                ch === 11 /* verticalTab */ ||
                ch === 12 /* formFeed */ ||
                ch === 160 /* nonBreakingSpace */ ||
                ch === 133 /* nextLine */ ||
                ch === 5760 /* ogham */ ||
                ch >= 8192 /* enQuad */ && ch <= 8203 /* zeroWidthSpace */ ||
                ch === 8239 /* narrowNoBreakSpace */ ||
                ch === 8287 /* mathematicalSpace */ ||
                ch === 12288 /* ideographicSpace */ ||
                ch === 65279 /* byteOrderMark */;
        }
        ts.isWhiteSpaceSingleLine = isWhiteSpaceSingleLine;
        function isLineBreak(ch) {
            // ES5 7.3:
            // The ECMAScript line terminator characters are listed in Table 3.
            //     Table 3: Line Terminator Characters
            //     Code Unit Value     Name                    Formal Name
            //     \u000A              Line Feed               <LF>
            //     \u000D              Carriage Return         <CR>
            //     \u2028              Line separator          <LS>
            //     \u2029              Paragraph separator     <PS>
            // Only the characters in Table 3 are treated as line terminators. Other new line or line
            // breaking characters are treated as white space but not as line terminators.
            return ch === 10 /* lineFeed */ ||
                ch === 13 /* carriageReturn */ ||
                ch === 8232 /* lineSeparator */ ||
                ch === 8233 /* paragraphSeparator */;
        }
        ts.isLineBreak = isLineBreak;
        function isDigit(ch) {
            return ch >= 48 /* _0 */ && ch <= 57 /* _9 */;
        }
        /* @internal */
        function isOctalDigit(ch) {
            return ch >= 48 /* _0 */ && ch <= 55 /* _7 */;
        }
        ts.isOctalDigit = isOctalDigit;
        function couldStartTrivia(text, pos) {
            // Keep in sync with skipTrivia
            var ch = text.charCodeAt(pos);
            switch (ch) {
                case 13 /* carriageReturn */:
                case 10 /* lineFeed */:
                case 9 /* tab */:
                case 11 /* verticalTab */:
                case 12 /* formFeed */:
                case 32 /* space */:
                case 47 /* slash */:
                // starts of normal trivia
                // falls through
                case 60 /* lessThan */:
                case 124 /* bar */:
                case 61 /* equals */:
                case 62 /* greaterThan */:
                    // Starts of conflict marker trivia
                    return true;
                case 35 /* hash */:
                    // Only if its the beginning can we have #! trivia
                    return pos === 0;
                default:
                    return ch > 127 /* maxAsciiCharacter */;
            }
        }
        ts.couldStartTrivia = couldStartTrivia;
        /* @internal */
        function skipTrivia(text, pos, stopAfterLineBreak, stopAtComments) {
            if (stopAtComments === void 0) {
                stopAtComments = false;
            }
            if (ts.positionIsSynthesized(pos)) {
                return pos;
            }
            // Keep in sync with couldStartTrivia
            while (true) {
                var ch = text.charCodeAt(pos);
                switch (ch) {
                    case 13 /* carriageReturn */:
                        if (text.charCodeAt(pos + 1) === 10 /* lineFeed */) {
                            pos++;
                        }
                    // falls through
                    case 10 /* lineFeed */:
                        pos++;
                        if (stopAfterLineBreak) {
                            return pos;
                        }
                        continue;
                    case 9 /* tab */:
                    case 11 /* verticalTab */:
                    case 12 /* formFeed */:
                    case 32 /* space */:
                        pos++;
                        continue;
                    case 47 /* slash */:
                        if (stopAtComments) {
                            break;
                        }
                        if (text.charCodeAt(pos + 1) === 47 /* slash */) {
                            pos += 2;
                            while (pos < text.length) {
                                if (isLineBreak(text.charCodeAt(pos))) {
                                    break;
                                }
                                pos++;
                            }
                            continue;
                        }
                        if (text.charCodeAt(pos + 1) === 42 /* asterisk */) {
                            pos += 2;
                            while (pos < text.length) {
                                if (text.charCodeAt(pos) === 42 /* asterisk */ && text.charCodeAt(pos + 1) === 47 /* slash */) {
                                    pos += 2;
                                    break;
                                }
                                pos++;
                            }
                            continue;
                        }
                        break;
                    case 60 /* lessThan */:
                    case 124 /* bar */:
                    case 61 /* equals */:
                    case 62 /* greaterThan */:
                        if (isConflictMarkerTrivia(text, pos)) {
                            pos = scanConflictMarkerTrivia(text, pos);
                            continue;
                        }
                        break;
                    case 35 /* hash */:
                        if (pos === 0 && isShebangTrivia(text, pos)) {
                            pos = scanShebangTrivia(text, pos);
                            continue;
                        }
                        break;
                    default:
                        if (ch > 127 /* maxAsciiCharacter */ && (isWhiteSpaceLike(ch))) {
                            pos++;
                            continue;
                        }
                        break;
                }
                return pos;
            }
        }
        ts.skipTrivia = skipTrivia;
        // All conflict markers consist of the same character repeated seven times.  If it is
        // a <<<<<<< or >>>>>>> marker then it is also followed by a space.
        var mergeConflictMarkerLength = "<<<<<<<".length;
        function isConflictMarkerTrivia(text, pos) {
            ts.Debug.assert(pos >= 0);
            // Conflict markers must be at the start of a line.
            if (pos === 0 || isLineBreak(text.charCodeAt(pos - 1))) {
                var ch = text.charCodeAt(pos);
                if ((pos + mergeConflictMarkerLength) < text.length) {
                    for (var i = 0; i < mergeConflictMarkerLength; i++) {
                        if (text.charCodeAt(pos + i) !== ch) {
                            return false;
                        }
                    }
                    return ch === 61 /* equals */ ||
                        text.charCodeAt(pos + mergeConflictMarkerLength) === 32 /* space */;
                }
            }
            return false;
        }
        function scanConflictMarkerTrivia(text, pos, error) {
            if (error) {
                error(ts.Diagnostics.Merge_conflict_marker_encountered, pos, mergeConflictMarkerLength);
            }
            var ch = text.charCodeAt(pos);
            var len = text.length;
            if (ch === 60 /* lessThan */ || ch === 62 /* greaterThan */) {
                while (pos < len && !isLineBreak(text.charCodeAt(pos))) {
                    pos++;
                }
            }
            else {
                ts.Debug.assert(ch === 124 /* bar */ || ch === 61 /* equals */);
                // Consume everything from the start of a ||||||| or ======= marker to the start
                // of the next ======= or >>>>>>> marker.
                while (pos < len) {
                    var currentChar = text.charCodeAt(pos);
                    if ((currentChar === 61 /* equals */ || currentChar === 62 /* greaterThan */) && currentChar !== ch && isConflictMarkerTrivia(text, pos)) {
                        break;
                    }
                    pos++;
                }
            }
            return pos;
        }
        var shebangTriviaRegex = /^#!.*/;
        /*@internal*/
        function isShebangTrivia(text, pos) {
            // Shebangs check must only be done at the start of the file
            ts.Debug.assert(pos === 0);
            return shebangTriviaRegex.test(text);
        }
        ts.isShebangTrivia = isShebangTrivia;
        /*@internal*/
        function scanShebangTrivia(text, pos) {
            var shebang = shebangTriviaRegex.exec(text)[0];
            pos = pos + shebang.length;
            return pos;
        }
        ts.scanShebangTrivia = scanShebangTrivia;
        /**
         * Invokes a callback for each comment range following the provided position.
         *
         * Single-line comment ranges include the leading double-slash characters but not the ending
         * line break. Multi-line comment ranges include the leading slash-asterisk and trailing
         * asterisk-slash characters.
         *
         * @param reduce If true, accumulates the result of calling the callback in a fashion similar
         *      to reduceLeft. If false, iteration stops when the callback returns a truthy value.
         * @param text The source text to scan.
         * @param pos The position at which to start scanning.
         * @param trailing If false, whitespace is skipped until the first line break and comments
         *      between that location and the next token are returned. If true, comments occurring
         *      between the given position and the next line break are returned.
         * @param cb The callback to execute as each comment range is encountered.
         * @param state A state value to pass to each iteration of the callback.
         * @param initial An initial value to pass when accumulating results (when "reduce" is true).
         * @returns If "reduce" is true, the accumulated value. If "reduce" is false, the first truthy
         *      return value of the callback.
         */
        function iterateCommentRanges(reduce, text, pos, trailing, cb, state, initial) {
            var pendingPos;
            var pendingEnd;
            var pendingKind;
            var pendingHasTrailingNewLine;
            var hasPendingCommentRange = false;
            var collecting = trailing;
            var accumulator = initial;
            if (pos === 0) {
                collecting = true;
                var shebang = getShebang(text);
                if (shebang) {
                    pos = shebang.length;
                }
            }
            scan: while (pos >= 0 && pos < text.length) {
                var ch = text.charCodeAt(pos);
                switch (ch) {
                    case 13 /* carriageReturn */:
                        if (text.charCodeAt(pos + 1) === 10 /* lineFeed */) {
                            pos++;
                        }
                    // falls through
                    case 10 /* lineFeed */:
                        pos++;
                        if (trailing) {
                            break scan;
                        }
                        collecting = true;
                        if (hasPendingCommentRange) {
                            pendingHasTrailingNewLine = true;
                        }
                        continue;
                    case 9 /* tab */:
                    case 11 /* verticalTab */:
                    case 12 /* formFeed */:
                    case 32 /* space */:
                        pos++;
                        continue;
                    case 47 /* slash */:
                        var nextChar = text.charCodeAt(pos + 1);
                        var hasTrailingNewLine = false;
                        if (nextChar === 47 /* slash */ || nextChar === 42 /* asterisk */) {
                            var kind = nextChar === 47 /* slash */ ? 2 /* SingleLineCommentTrivia */ : 3 /* MultiLineCommentTrivia */;
                            var startPos = pos;
                            pos += 2;
                            if (nextChar === 47 /* slash */) {
                                while (pos < text.length) {
                                    if (isLineBreak(text.charCodeAt(pos))) {
                                        hasTrailingNewLine = true;
                                        break;
                                    }
                                    pos++;
                                }
                            }
                            else {
                                while (pos < text.length) {
                                    if (text.charCodeAt(pos) === 42 /* asterisk */ && text.charCodeAt(pos + 1) === 47 /* slash */) {
                                        pos += 2;
                                        break;
                                    }
                                    pos++;
                                }
                            }
                            if (collecting) {
                                if (hasPendingCommentRange) {
                                    accumulator = cb(pendingPos, pendingEnd, pendingKind, pendingHasTrailingNewLine, state, accumulator);
                                    if (!reduce && accumulator) {
                                        // If we are not reducing and we have a truthy result, return it.
                                        return accumulator;
                                    }
                                }
                                pendingPos = startPos;
                                pendingEnd = pos;
                                pendingKind = kind;
                                pendingHasTrailingNewLine = hasTrailingNewLine;
                                hasPendingCommentRange = true;
                            }
                            continue;
                        }
                        break scan;
                    default:
                        if (ch > 127 /* maxAsciiCharacter */ && (isWhiteSpaceLike(ch))) {
                            if (hasPendingCommentRange && isLineBreak(ch)) {
                                pendingHasTrailingNewLine = true;
                            }
                            pos++;
                            continue;
                        }
                        break scan;
                }
            }
            if (hasPendingCommentRange) {
                accumulator = cb(pendingPos, pendingEnd, pendingKind, pendingHasTrailingNewLine, state, accumulator);
            }
            return accumulator;
        }
        function forEachLeadingCommentRange(text, pos, cb, state) {
            return iterateCommentRanges(/*reduce*/ false, text, pos, /*trailing*/ false, cb, state);
        }
        ts.forEachLeadingCommentRange = forEachLeadingCommentRange;
        function forEachTrailingCommentRange(text, pos, cb, state) {
            return iterateCommentRanges(/*reduce*/ false, text, pos, /*trailing*/ true, cb, state);
        }
        ts.forEachTrailingCommentRange = forEachTrailingCommentRange;
        function reduceEachLeadingCommentRange(text, pos, cb, state, initial) {
            return iterateCommentRanges(/*reduce*/ true, text, pos, /*trailing*/ false, cb, state, initial);
        }
        ts.reduceEachLeadingCommentRange = reduceEachLeadingCommentRange;
        function reduceEachTrailingCommentRange(text, pos, cb, state, initial) {
            return iterateCommentRanges(/*reduce*/ true, text, pos, /*trailing*/ true, cb, state, initial);
        }
        ts.reduceEachTrailingCommentRange = reduceEachTrailingCommentRange;
        function appendCommentRange(pos, end, kind, hasTrailingNewLine, _state, comments) {
            if (!comments) {
                comments = [];
            }
            comments.push({ kind: kind, pos: pos, end: end, hasTrailingNewLine: hasTrailingNewLine });
            return comments;
        }
        function getLeadingCommentRanges(text, pos) {
            return reduceEachLeadingCommentRange(text, pos, appendCommentRange, /*state*/ undefined, /*initial*/ undefined);
        }
        ts.getLeadingCommentRanges = getLeadingCommentRanges;
        function getTrailingCommentRanges(text, pos) {
            return reduceEachTrailingCommentRange(text, pos, appendCommentRange, /*state*/ undefined, /*initial*/ undefined);
        }
        ts.getTrailingCommentRanges = getTrailingCommentRanges;
        /** Optionally, get the shebang */
        function getShebang(text) {
            var match = shebangTriviaRegex.exec(text);
            if (match) {
                return match[0];
            }
        }
        ts.getShebang = getShebang;
        function isIdentifierStart(ch, languageVersion) {
            return ch >= 65 /* A */ && ch <= 90 /* Z */ || ch >= 97 /* a */ && ch <= 122 /* z */ ||
                ch === 36 /* $ */ || ch === 95 /* _ */ ||
                ch > 127 /* maxAsciiCharacter */ && isUnicodeIdentifierStart(ch, languageVersion);
        }
        ts.isIdentifierStart = isIdentifierStart;
        function isIdentifierPart(ch, languageVersion) {
            return ch >= 65 /* A */ && ch <= 90 /* Z */ || ch >= 97 /* a */ && ch <= 122 /* z */ ||
                ch >= 48 /* _0 */ && ch <= 57 /* _9 */ || ch === 36 /* $ */ || ch === 95 /* _ */ ||
                ch > 127 /* maxAsciiCharacter */ && isUnicodeIdentifierPart(ch, languageVersion);
        }
        ts.isIdentifierPart = isIdentifierPart;
        /* @internal */
        function isIdentifierText(name, languageVersion) {
            var ch = codePointAt(name, 0);
            if (!isIdentifierStart(ch, languageVersion)) {
                return false;
            }
            for (var i = charSize(ch); i < name.length; i += charSize(ch)) {
                if (!isIdentifierPart(ch = codePointAt(name, i), languageVersion)) {
                    return false;
                }
            }
            return true;
        }
        ts.isIdentifierText = isIdentifierText;
        // Creates a scanner over a (possibly unspecified) range of a piece of text.
        function createScanner(languageVersion, skipTrivia, languageVariant, textInitial, onError, start, length) {
            if (languageVariant === void 0) {
                languageVariant = 0 /* Standard */;
            }
            var text = textInitial;
            // Current position (end position of text of current token)
            var pos;
            // end of text
            var end;
            // Start position of whitespace before current token
            var startPos;
            // Start position of text of current token
            var tokenPos;
            var token;
            var tokenValue;
            var tokenFlags;
            var inJSDocType = 0;
            setText(text, start, length);
            var scanner = {
                getStartPos: function () { return startPos; },
                getTextPos: function () { return pos; },
                getToken: function () { return token; },
                getTokenPos: function () { return tokenPos; },
                getTokenText: function () { return text.substring(tokenPos, pos); },
                getTokenValue: function () { return tokenValue; },
                hasUnicodeEscape: function () { return (tokenFlags & 1024 /* UnicodeEscape */) !== 0; },
                hasExtendedUnicodeEscape: function () { return (tokenFlags & 8 /* ExtendedUnicodeEscape */) !== 0; },
                hasPrecedingLineBreak: function () { return (tokenFlags & 1 /* PrecedingLineBreak */) !== 0; },
                isIdentifier: function () { return token === 75 /* Identifier */ || token > 112 /* LastReservedWord */; },
                isReservedWord: function () { return token >= 77 /* FirstReservedWord */ && token <= 112 /* LastReservedWord */; },
                isUnterminated: function () { return (tokenFlags & 4 /* Unterminated */) !== 0; },
                getTokenFlags: function () { return tokenFlags; },
                reScanGreaterToken: reScanGreaterToken,
                reScanSlashToken: reScanSlashToken,
                reScanTemplateToken: reScanTemplateToken,
                scanJsxIdentifier: scanJsxIdentifier,
                scanJsxAttributeValue: scanJsxAttributeValue,
                reScanJsxAttributeValue: reScanJsxAttributeValue,
                reScanJsxToken: reScanJsxToken,
                reScanLessThanToken: reScanLessThanToken,
                reScanQuestionToken: reScanQuestionToken,
                scanJsxToken: scanJsxToken,
                scanJsDocToken: scanJsDocToken,
                scan: scan,
                getText: getText,
                setText: setText,
                setScriptTarget: setScriptTarget,
                setLanguageVariant: setLanguageVariant,
                setOnError: setOnError,
                setTextPos: setTextPos,
                setInJSDocType: setInJSDocType,
                tryScan: tryScan,
                lookAhead: lookAhead,
                scanRange: scanRange,
            };
            if (ts.Debug.isDebugging) {
                Object.defineProperty(scanner, "__debugShowCurrentPositionInText", {
                    get: function () {
                        var text = scanner.getText();
                        return text.slice(0, scanner.getStartPos()) + "║" + text.slice(scanner.getStartPos());
                    },
                });
            }
            return scanner;
            function error(message, errPos, length) {
                if (errPos === void 0) {
                    errPos = pos;
                }
                if (onError) {
                    var oldPos = pos;
                    pos = errPos;
                    onError(message, length || 0);
                    pos = oldPos;
                }
            }
            function scanNumberFragment() {
                var start = pos;
                var allowSeparator = false;
                var isPreviousTokenSeparator = false;
                var result = "";
                while (true) {
                    var ch = text.charCodeAt(pos);
                    if (ch === 95 /* _ */) {
                        tokenFlags |= 512 /* ContainsSeparator */;
                        if (allowSeparator) {
                            allowSeparator = false;
                            isPreviousTokenSeparator = true;
                            result += text.substring(start, pos);
                        }
                        else if (isPreviousTokenSeparator) {
                            error(ts.Diagnostics.Multiple_consecutive_numeric_separators_are_not_permitted, pos, 1);
                        }
                        else {
                            error(ts.Diagnostics.Numeric_separators_are_not_allowed_here, pos, 1);
                        }
                        pos++;
                        start = pos;
                        continue;
                    }
                    if (isDigit(ch)) {
                        allowSeparator = true;
                        isPreviousTokenSeparator = false;
                        pos++;
                        continue;
                    }
                    break;
                }
                if (text.charCodeAt(pos - 1) === 95 /* _ */) {
                    error(ts.Diagnostics.Numeric_separators_are_not_allowed_here, pos - 1, 1);
                }
                return result + text.substring(start, pos);
            }
            function scanNumber() {
                var start = pos;
                var mainFragment = scanNumberFragment();
                var decimalFragment;
                var scientificFragment;
                if (text.charCodeAt(pos) === 46 /* dot */) {
                    pos++;
                    decimalFragment = scanNumberFragment();
                }
                var end = pos;
                if (text.charCodeAt(pos) === 69 /* E */ || text.charCodeAt(pos) === 101 /* e */) {
                    pos++;
                    tokenFlags |= 16 /* Scientific */;
                    if (text.charCodeAt(pos) === 43 /* plus */ || text.charCodeAt(pos) === 45 /* minus */)
                        pos++;
                    var preNumericPart = pos;
                    var finalFragment = scanNumberFragment();
                    if (!finalFragment) {
                        error(ts.Diagnostics.Digit_expected);
                    }
                    else {
                        scientificFragment = text.substring(end, preNumericPart) + finalFragment;
                        end = pos;
                    }
                }
                var result;
                if (tokenFlags & 512 /* ContainsSeparator */) {
                    result = mainFragment;
                    if (decimalFragment) {
                        result += "." + decimalFragment;
                    }
                    if (scientificFragment) {
                        result += scientificFragment;
                    }
                }
                else {
                    result = text.substring(start, end); // No need to use all the fragments; no _ removal needed
                }
                if (decimalFragment !== undefined || tokenFlags & 16 /* Scientific */) {
                    checkForIdentifierStartAfterNumericLiteral(start, decimalFragment === undefined && !!(tokenFlags & 16 /* Scientific */));
                    return {
                        type: 8 /* NumericLiteral */,
                        value: "" + +result // if value is not an integer, it can be safely coerced to a number
                    };
                }
                else {
                    tokenValue = result;
                    var type = checkBigIntSuffix(); // if value is an integer, check whether it is a bigint
                    checkForIdentifierStartAfterNumericLiteral(start);
                    return { type: type, value: tokenValue };
                }
            }
            function checkForIdentifierStartAfterNumericLiteral(numericStart, isScientific) {
                if (!isIdentifierStart(codePointAt(text, pos), languageVersion)) {
                    return;
                }
                var identifierStart = pos;
                var length = scanIdentifierParts().length;
                if (length === 1 && text[identifierStart] === "n") {
                    if (isScientific) {
                        error(ts.Diagnostics.A_bigint_literal_cannot_use_exponential_notation, numericStart, identifierStart - numericStart + 1);
                    }
                    else {
                        error(ts.Diagnostics.A_bigint_literal_must_be_an_integer, numericStart, identifierStart - numericStart + 1);
                    }
                }
                else {
                    error(ts.Diagnostics.An_identifier_or_keyword_cannot_immediately_follow_a_numeric_literal, identifierStart, length);
                    pos = identifierStart;
                }
            }
            function scanOctalDigits() {
                var start = pos;
                while (isOctalDigit(text.charCodeAt(pos))) {
                    pos++;
                }
                return +(text.substring(start, pos));
            }
            /**
             * Scans the given number of hexadecimal digits in the text,
             * returning -1 if the given number is unavailable.
             */
            function scanExactNumberOfHexDigits(count, canHaveSeparators) {
                var valueString = scanHexDigits(/*minCount*/ count, /*scanAsManyAsPossible*/ false, canHaveSeparators);
                return valueString ? parseInt(valueString, 16) : -1;
            }
            /**
             * Scans as many hexadecimal digits as are available in the text,
             * returning "" if the given number of digits was unavailable.
             */
            function scanMinimumNumberOfHexDigits(count, canHaveSeparators) {
                return scanHexDigits(/*minCount*/ count, /*scanAsManyAsPossible*/ true, canHaveSeparators);
            }
            function scanHexDigits(minCount, scanAsManyAsPossible, canHaveSeparators) {
                var valueChars = [];
                var allowSeparator = false;
                var isPreviousTokenSeparator = false;
                while (valueChars.length < minCount || scanAsManyAsPossible) {
                    var ch = text.charCodeAt(pos);
                    if (canHaveSeparators && ch === 95 /* _ */) {
                        tokenFlags |= 512 /* ContainsSeparator */;
                        if (allowSeparator) {
                            allowSeparator = false;
                            isPreviousTokenSeparator = true;
                        }
                        else if (isPreviousTokenSeparator) {
                            error(ts.Diagnostics.Multiple_consecutive_numeric_separators_are_not_permitted, pos, 1);
                        }
                        else {
                            error(ts.Diagnostics.Numeric_separators_are_not_allowed_here, pos, 1);
                        }
                        pos++;
                        continue;
                    }
                    allowSeparator = canHaveSeparators;
                    if (ch >= 65 /* A */ && ch <= 70 /* F */) {
                        ch += 97 /* a */ - 65 /* A */; // standardize hex literals to lowercase
                    }
                    else if (!((ch >= 48 /* _0 */ && ch <= 57 /* _9 */) ||
                        (ch >= 97 /* a */ && ch <= 102 /* f */))) {
                        break;
                    }
                    valueChars.push(ch);
                    pos++;
                    isPreviousTokenSeparator = false;
                }
                if (valueChars.length < minCount) {
                    valueChars = [];
                }
                if (text.charCodeAt(pos - 1) === 95 /* _ */) {
                    error(ts.Diagnostics.Numeric_separators_are_not_allowed_here, pos - 1, 1);
                }
                return String.fromCharCode.apply(String, valueChars);
            }
            function scanString(jsxAttributeString) {
                if (jsxAttributeString === void 0) {
                    jsxAttributeString = false;
                }
                var quote = text.charCodeAt(pos);
                pos++;
                var result = "";
                var start = pos;
                while (true) {
                    if (pos >= end) {
                        result += text.substring(start, pos);
                        tokenFlags |= 4 /* Unterminated */;
                        error(ts.Diagnostics.Unterminated_string_literal);
                        break;
                    }
                    var ch = text.charCodeAt(pos);
                    if (ch === quote) {
                        result += text.substring(start, pos);
                        pos++;
                        break;
                    }
                    if (ch === 92 /* backslash */ && !jsxAttributeString) {
                        result += text.substring(start, pos);
                        result += scanEscapeSequence();
                        start = pos;
                        continue;
                    }
                    if (isLineBreak(ch) && !jsxAttributeString) {
                        result += text.substring(start, pos);
                        tokenFlags |= 4 /* Unterminated */;
                        error(ts.Diagnostics.Unterminated_string_literal);
                        break;
                    }
                    pos++;
                }
                return result;
            }
            /**
             * Sets the current 'tokenValue' and returns a NoSubstitutionTemplateLiteral or
             * a literal component of a TemplateExpression.
             */
            function scanTemplateAndSetTokenValue() {
                var startedWithBacktick = text.charCodeAt(pos) === 96 /* backtick */;
                pos++;
                var start = pos;
                var contents = "";
                var resultingToken;
                while (true) {
                    if (pos >= end) {
                        contents += text.substring(start, pos);
                        tokenFlags |= 4 /* Unterminated */;
                        error(ts.Diagnostics.Unterminated_template_literal);
                        resultingToken = startedWithBacktick ? 14 /* NoSubstitutionTemplateLiteral */ : 17 /* TemplateTail */;
                        break;
                    }
                    var currChar = text.charCodeAt(pos);
                    // '`'
                    if (currChar === 96 /* backtick */) {
                        contents += text.substring(start, pos);
                        pos++;
                        resultingToken = startedWithBacktick ? 14 /* NoSubstitutionTemplateLiteral */ : 17 /* TemplateTail */;
                        break;
                    }
                    // '${'
                    if (currChar === 36 /* $ */ && pos + 1 < end && text.charCodeAt(pos + 1) === 123 /* openBrace */) {
                        contents += text.substring(start, pos);
                        pos += 2;
                        resultingToken = startedWithBacktick ? 15 /* TemplateHead */ : 16 /* TemplateMiddle */;
                        break;
                    }
                    // Escape character
                    if (currChar === 92 /* backslash */) {
                        contents += text.substring(start, pos);
                        contents += scanEscapeSequence();
                        start = pos;
                        continue;
                    }
                    // Speculated ECMAScript 6 Spec 11.8.6.1:
                    // <CR><LF> and <CR> LineTerminatorSequences are normalized to <LF> for Template Values
                    if (currChar === 13 /* carriageReturn */) {
                        contents += text.substring(start, pos);
                        pos++;
                        if (pos < end && text.charCodeAt(pos) === 10 /* lineFeed */) {
                            pos++;
                        }
                        contents += "\n";
                        start = pos;
                        continue;
                    }
                    pos++;
                }
                ts.Debug.assert(resultingToken !== undefined);
                tokenValue = contents;
                return resultingToken;
            }
            function scanEscapeSequence() {
                pos++;
                if (pos >= end) {
                    error(ts.Diagnostics.Unexpected_end_of_text);
                    return "";
                }
                var ch = text.charCodeAt(pos);
                pos++;
                switch (ch) {
                    case 48 /* _0 */:
                        return "\0";
                    case 98 /* b */:
                        return "\b";
                    case 116 /* t */:
                        return "\t";
                    case 110 /* n */:
                        return "\n";
                    case 118 /* v */:
                        return "\v";
                    case 102 /* f */:
                        return "\f";
                    case 114 /* r */:
                        return "\r";
                    case 39 /* singleQuote */:
                        return "\'";
                    case 34 /* doubleQuote */:
                        return "\"";
                    case 117 /* u */:
                        // '\u{DDDDDDDD}'
                        if (pos < end && text.charCodeAt(pos) === 123 /* openBrace */) {
                            tokenFlags |= 8 /* ExtendedUnicodeEscape */;
                            pos++;
                            return scanExtendedUnicodeEscape();
                        }
                        tokenFlags |= 1024 /* UnicodeEscape */;
                        // '\uDDDD'
                        return scanHexadecimalEscape(/*numDigits*/ 4);
                    case 120 /* x */:
                        // '\xDD'
                        return scanHexadecimalEscape(/*numDigits*/ 2);
                    // when encountering a LineContinuation (i.e. a backslash and a line terminator sequence),
                    // the line terminator is interpreted to be "the empty code unit sequence".
                    case 13 /* carriageReturn */:
                        if (pos < end && text.charCodeAt(pos) === 10 /* lineFeed */) {
                            pos++;
                        }
                    // falls through
                    case 10 /* lineFeed */:
                    case 8232 /* lineSeparator */:
                    case 8233 /* paragraphSeparator */:
                        return "";
                    default:
                        return String.fromCharCode(ch);
                }
            }
            function scanHexadecimalEscape(numDigits) {
                var escapedValue = scanExactNumberOfHexDigits(numDigits, /*canHaveSeparators*/ false);
                if (escapedValue >= 0) {
                    return String.fromCharCode(escapedValue);
                }
                else {
                    error(ts.Diagnostics.Hexadecimal_digit_expected);
                    return "";
                }
            }
            function scanExtendedUnicodeEscape() {
                var escapedValueString = scanMinimumNumberOfHexDigits(1, /*canHaveSeparators*/ false);
                var escapedValue = escapedValueString ? parseInt(escapedValueString, 16) : -1;
                var isInvalidExtendedEscape = false;
                // Validate the value of the digit
                if (escapedValue < 0) {
                    error(ts.Diagnostics.Hexadecimal_digit_expected);
                    isInvalidExtendedEscape = true;
                }
                else if (escapedValue > 0x10FFFF) {
                    error(ts.Diagnostics.An_extended_Unicode_escape_value_must_be_between_0x0_and_0x10FFFF_inclusive);
                    isInvalidExtendedEscape = true;
                }
                if (pos >= end) {
                    error(ts.Diagnostics.Unexpected_end_of_text);
                    isInvalidExtendedEscape = true;
                }
                else if (text.charCodeAt(pos) === 125 /* closeBrace */) {
                    // Only swallow the following character up if it's a '}'.
                    pos++;
                }
                else {
                    error(ts.Diagnostics.Unterminated_Unicode_escape_sequence);
                    isInvalidExtendedEscape = true;
                }
                if (isInvalidExtendedEscape) {
                    return "";
                }
                return utf16EncodeAsString(escapedValue);
            }
            // Current character is known to be a backslash. Check for Unicode escape of the form '\uXXXX'
            // and return code point value if valid Unicode escape is found. Otherwise return -1.
            function peekUnicodeEscape() {
                if (pos + 5 < end && text.charCodeAt(pos + 1) === 117 /* u */) {
                    var start_1 = pos;
                    pos += 2;
                    var value = scanExactNumberOfHexDigits(4, /*canHaveSeparators*/ false);
                    pos = start_1;
                    return value;
                }
                return -1;
            }
            function peekExtendedUnicodeEscape() {
                if (languageVersion >= 2 /* ES2015 */ && codePointAt(text, pos + 1) === 117 /* u */ && codePointAt(text, pos + 2) === 123 /* openBrace */) {
                    var start_2 = pos;
                    pos += 3;
                    var escapedValueString = scanMinimumNumberOfHexDigits(1, /*canHaveSeparators*/ false);
                    var escapedValue = escapedValueString ? parseInt(escapedValueString, 16) : -1;
                    pos = start_2;
                    return escapedValue;
                }
                return -1;
            }
            function scanIdentifierParts() {
                var result = "";
                var start = pos;
                while (pos < end) {
                    var ch = codePointAt(text, pos);
                    if (isIdentifierPart(ch, languageVersion)) {
                        pos += charSize(ch);
                    }
                    else if (ch === 92 /* backslash */) {
                        ch = peekExtendedUnicodeEscape();
                        if (ch >= 0 && isIdentifierPart(ch, languageVersion)) {
                            pos += 3;
                            tokenFlags |= 8 /* ExtendedUnicodeEscape */;
                            result += scanExtendedUnicodeEscape();
                            start = pos;
                            continue;
                        }
                        ch = peekUnicodeEscape();
                        if (!(ch >= 0 && isIdentifierPart(ch, languageVersion))) {
                            break;
                        }
                        tokenFlags |= 1024 /* UnicodeEscape */;
                        result += text.substring(start, pos);
                        result += utf16EncodeAsString(ch);
                        // Valid Unicode escape is always six characters
                        pos += 6;
                        start = pos;
                    }
                    else {
                        break;
                    }
                }
                result += text.substring(start, pos);
                return result;
            }
            function getIdentifierToken() {
                // Reserved words are between 2 and 11 characters long and start with a lowercase letter
                var len = tokenValue.length;
                if (len >= 2 && len <= 11) {
                    var ch = tokenValue.charCodeAt(0);
                    if (ch >= 97 /* a */ && ch <= 122 /* z */) {
                        var keyword = textToKeyword.get(tokenValue);
                        if (keyword !== undefined) {
                            return token = keyword;
                        }
                    }
                }
                return token = 75 /* Identifier */;
            }
            function scanBinaryOrOctalDigits(base) {
                var value = "";
                // For counting number of digits; Valid binaryIntegerLiteral must have at least one binary digit following B or b.
                // Similarly valid octalIntegerLiteral must have at least one octal digit following o or O.
                var separatorAllowed = false;
                var isPreviousTokenSeparator = false;
                while (true) {
                    var ch = text.charCodeAt(pos);
                    // Numeric separators are allowed anywhere within a numeric literal, except not at the beginning, or following another separator
                    if (ch === 95 /* _ */) {
                        tokenFlags |= 512 /* ContainsSeparator */;
                        if (separatorAllowed) {
                            separatorAllowed = false;
                            isPreviousTokenSeparator = true;
                        }
                        else if (isPreviousTokenSeparator) {
                            error(ts.Diagnostics.Multiple_consecutive_numeric_separators_are_not_permitted, pos, 1);
                        }
                        else {
                            error(ts.Diagnostics.Numeric_separators_are_not_allowed_here, pos, 1);
                        }
                        pos++;
                        continue;
                    }
                    separatorAllowed = true;
                    if (!isDigit(ch) || ch - 48 /* _0 */ >= base) {
                        break;
                    }
                    value += text[pos];
                    pos++;
                    isPreviousTokenSeparator = false;
                }
                if (text.charCodeAt(pos - 1) === 95 /* _ */) {
                    // Literal ends with underscore - not allowed
                    error(ts.Diagnostics.Numeric_separators_are_not_allowed_here, pos - 1, 1);
                }
                return value;
            }
            function checkBigIntSuffix() {
                if (text.charCodeAt(pos) === 110 /* n */) {
                    tokenValue += "n";
                    // Use base 10 instead of base 2 or base 8 for shorter literals
                    if (tokenFlags & 384 /* BinaryOrOctalSpecifier */) {
                        tokenValue = ts.parsePseudoBigInt(tokenValue) + "n";
                    }
                    pos++;
                    return 9 /* BigIntLiteral */;
                }
                else { // not a bigint, so can convert to number in simplified form
                    // Number() may not support 0b or 0o, so use parseInt() instead
                    var numericValue = tokenFlags & 128 /* BinarySpecifier */
                        ? parseInt(tokenValue.slice(2), 2) // skip "0b"
                        : tokenFlags & 256 /* OctalSpecifier */
                            ? parseInt(tokenValue.slice(2), 8) // skip "0o"
                            : +tokenValue;
                    tokenValue = "" + numericValue;
                    return 8 /* NumericLiteral */;
                }
            }
            function scan() {
                var _a;
                startPos = pos;
                tokenFlags = 0 /* None */;
                var asteriskSeen = false;
                while (true) {
                    tokenPos = pos;
                    if (pos >= end) {
                        return token = 1 /* EndOfFileToken */;
                    }
                    var ch = codePointAt(text, pos);
                    // Special handling for shebang
                    if (ch === 35 /* hash */ && pos === 0 && isShebangTrivia(text, pos)) {
                        pos = scanShebangTrivia(text, pos);
                        if (skipTrivia) {
                            continue;
                        }
                        else {
                            return token = 6 /* ShebangTrivia */;
                        }
                    }
                    switch (ch) {
                        case 10 /* lineFeed */:
                        case 13 /* carriageReturn */:
                            tokenFlags |= 1 /* PrecedingLineBreak */;
                            if (skipTrivia) {
                                pos++;
                                continue;
                            }
                            else {
                                if (ch === 13 /* carriageReturn */ && pos + 1 < end && text.charCodeAt(pos + 1) === 10 /* lineFeed */) {
                                    // consume both CR and LF
                                    pos += 2;
                                }
                                else {
                                    pos++;
                                }
                                return token = 4 /* NewLineTrivia */;
                            }
                        case 9 /* tab */:
                        case 11 /* verticalTab */:
                        case 12 /* formFeed */:
                        case 32 /* space */:
                        case 160 /* nonBreakingSpace */:
                        case 5760 /* ogham */:
                        case 8192 /* enQuad */:
                        case 8193 /* emQuad */:
                        case 8194 /* enSpace */:
                        case 8195 /* emSpace */:
                        case 8196 /* threePerEmSpace */:
                        case 8197 /* fourPerEmSpace */:
                        case 8198 /* sixPerEmSpace */:
                        case 8199 /* figureSpace */:
                        case 8200 /* punctuationSpace */:
                        case 8201 /* thinSpace */:
                        case 8202 /* hairSpace */:
                        case 8203 /* zeroWidthSpace */:
                        case 8239 /* narrowNoBreakSpace */:
                        case 8287 /* mathematicalSpace */:
                        case 12288 /* ideographicSpace */:
                        case 65279 /* byteOrderMark */:
                            if (skipTrivia) {
                                pos++;
                                continue;
                            }
                            else {
                                while (pos < end && isWhiteSpaceSingleLine(text.charCodeAt(pos))) {
                                    pos++;
                                }
                                return token = 5 /* WhitespaceTrivia */;
                            }
                        case 33 /* exclamation */:
                            if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                                if (text.charCodeAt(pos + 2) === 61 /* equals */) {
                                    return pos += 3, token = 37 /* ExclamationEqualsEqualsToken */;
                                }
                                return pos += 2, token = 35 /* ExclamationEqualsToken */;
                            }
                            pos++;
                            return token = 53 /* ExclamationToken */;
                        case 34 /* doubleQuote */:
                        case 39 /* singleQuote */:
                            tokenValue = scanString();
                            return token = 10 /* StringLiteral */;
                        case 96 /* backtick */:
                            return token = scanTemplateAndSetTokenValue();
                        case 37 /* percent */:
                            if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                                return pos += 2, token = 68 /* PercentEqualsToken */;
                            }
                            pos++;
                            return token = 44 /* PercentToken */;
                        case 38 /* ampersand */:
                            if (text.charCodeAt(pos + 1) === 38 /* ampersand */) {
                                return pos += 2, token = 55 /* AmpersandAmpersandToken */;
                            }
                            if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                                return pos += 2, token = 72 /* AmpersandEqualsToken */;
                            }
                            pos++;
                            return token = 50 /* AmpersandToken */;
                        case 40 /* openParen */:
                            pos++;
                            return token = 20 /* OpenParenToken */;
                        case 41 /* closeParen */:
                            pos++;
                            return token = 21 /* CloseParenToken */;
                        case 42 /* asterisk */:
                            if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                                return pos += 2, token = 65 /* AsteriskEqualsToken */;
                            }
                            if (text.charCodeAt(pos + 1) === 42 /* asterisk */) {
                                if (text.charCodeAt(pos + 2) === 61 /* equals */) {
                                    return pos += 3, token = 66 /* AsteriskAsteriskEqualsToken */;
                                }
                                return pos += 2, token = 42 /* AsteriskAsteriskToken */;
                            }
                            pos++;
                            if (inJSDocType && !asteriskSeen && (tokenFlags & 1 /* PrecedingLineBreak */)) {
                                // decoration at the start of a JSDoc comment line
                                asteriskSeen = true;
                                continue;
                            }
                            return token = 41 /* AsteriskToken */;
                        case 43 /* plus */:
                            if (text.charCodeAt(pos + 1) === 43 /* plus */) {
                                return pos += 2, token = 45 /* PlusPlusToken */;
                            }
                            if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                                return pos += 2, token = 63 /* PlusEqualsToken */;
                            }
                            pos++;
                            return token = 39 /* PlusToken */;
                        case 44 /* comma */:
                            pos++;
                            return token = 27 /* CommaToken */;
                        case 45 /* minus */:
                            if (text.charCodeAt(pos + 1) === 45 /* minus */) {
                                return pos += 2, token = 46 /* MinusMinusToken */;
                            }
                            if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                                return pos += 2, token = 64 /* MinusEqualsToken */;
                            }
                            pos++;
                            return token = 40 /* MinusToken */;
                        case 46 /* dot */:
                            if (isDigit(text.charCodeAt(pos + 1))) {
                                tokenValue = scanNumber().value;
                                return token = 8 /* NumericLiteral */;
                            }
                            if (text.charCodeAt(pos + 1) === 46 /* dot */ && text.charCodeAt(pos + 2) === 46 /* dot */) {
                                return pos += 3, token = 25 /* DotDotDotToken */;
                            }
                            pos++;
                            return token = 24 /* DotToken */;
                        case 47 /* slash */:
                            // Single-line comment
                            if (text.charCodeAt(pos + 1) === 47 /* slash */) {
                                pos += 2;
                                while (pos < end) {
                                    if (isLineBreak(text.charCodeAt(pos))) {
                                        break;
                                    }
                                    pos++;
                                }
                                if (skipTrivia) {
                                    continue;
                                }
                                else {
                                    return token = 2 /* SingleLineCommentTrivia */;
                                }
                            }
                            // Multi-line comment
                            if (text.charCodeAt(pos + 1) === 42 /* asterisk */) {
                                pos += 2;
                                if (text.charCodeAt(pos) === 42 /* asterisk */ && text.charCodeAt(pos + 1) !== 47 /* slash */) {
                                    tokenFlags |= 2 /* PrecedingJSDocComment */;
                                }
                                var commentClosed = false;
                                while (pos < end) {
                                    var ch_1 = text.charCodeAt(pos);
                                    if (ch_1 === 42 /* asterisk */ && text.charCodeAt(pos + 1) === 47 /* slash */) {
                                        pos += 2;
                                        commentClosed = true;
                                        break;
                                    }
                                    if (isLineBreak(ch_1)) {
                                        tokenFlags |= 1 /* PrecedingLineBreak */;
                                    }
                                    pos++;
                                }
                                if (!commentClosed) {
                                    error(ts.Diagnostics.Asterisk_Slash_expected);
                                }
                                if (skipTrivia) {
                                    continue;
                                }
                                else {
                                    if (!commentClosed) {
                                        tokenFlags |= 4 /* Unterminated */;
                                    }
                                    return token = 3 /* MultiLineCommentTrivia */;
                                }
                            }
                            if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                                return pos += 2, token = 67 /* SlashEqualsToken */;
                            }
                            pos++;
                            return token = 43 /* SlashToken */;
                        case 48 /* _0 */:
                            if (pos + 2 < end && (text.charCodeAt(pos + 1) === 88 /* X */ || text.charCodeAt(pos + 1) === 120 /* x */)) {
                                pos += 2;
                                tokenValue = scanMinimumNumberOfHexDigits(1, /*canHaveSeparators*/ true);
                                if (!tokenValue) {
                                    error(ts.Diagnostics.Hexadecimal_digit_expected);
                                    tokenValue = "0";
                                }
                                tokenValue = "0x" + tokenValue;
                                tokenFlags |= 64 /* HexSpecifier */;
                                return token = checkBigIntSuffix();
                            }
                            else if (pos + 2 < end && (text.charCodeAt(pos + 1) === 66 /* B */ || text.charCodeAt(pos + 1) === 98 /* b */)) {
                                pos += 2;
                                tokenValue = scanBinaryOrOctalDigits(/* base */ 2);
                                if (!tokenValue) {
                                    error(ts.Diagnostics.Binary_digit_expected);
                                    tokenValue = "0";
                                }
                                tokenValue = "0b" + tokenValue;
                                tokenFlags |= 128 /* BinarySpecifier */;
                                return token = checkBigIntSuffix();
                            }
                            else if (pos + 2 < end && (text.charCodeAt(pos + 1) === 79 /* O */ || text.charCodeAt(pos + 1) === 111 /* o */)) {
                                pos += 2;
                                tokenValue = scanBinaryOrOctalDigits(/* base */ 8);
                                if (!tokenValue) {
                                    error(ts.Diagnostics.Octal_digit_expected);
                                    tokenValue = "0";
                                }
                                tokenValue = "0o" + tokenValue;
                                tokenFlags |= 256 /* OctalSpecifier */;
                                return token = checkBigIntSuffix();
                            }
                            // Try to parse as an octal
                            if (pos + 1 < end && isOctalDigit(text.charCodeAt(pos + 1))) {
                                tokenValue = "" + scanOctalDigits();
                                tokenFlags |= 32 /* Octal */;
                                return token = 8 /* NumericLiteral */;
                            }
                        // This fall-through is a deviation from the EcmaScript grammar. The grammar says that a leading zero
                        // can only be followed by an octal digit, a dot, or the end of the number literal. However, we are being
                        // permissive and allowing decimal digits of the form 08* and 09* (which many browsers also do).
                        // falls through
                        case 49 /* _1 */:
                        case 50 /* _2 */:
                        case 51 /* _3 */:
                        case 52 /* _4 */:
                        case 53 /* _5 */:
                        case 54 /* _6 */:
                        case 55 /* _7 */:
                        case 56 /* _8 */:
                        case 57 /* _9 */:
                            (_a = scanNumber(), token = _a.type, tokenValue = _a.value);
                            return token;
                        case 58 /* colon */:
                            pos++;
                            return token = 58 /* ColonToken */;
                        case 59 /* semicolon */:
                            pos++;
                            return token = 26 /* SemicolonToken */;
                        case 60 /* lessThan */:
                            if (isConflictMarkerTrivia(text, pos)) {
                                pos = scanConflictMarkerTrivia(text, pos, error);
                                if (skipTrivia) {
                                    continue;
                                }
                                else {
                                    return token = 7 /* ConflictMarkerTrivia */;
                                }
                            }
                            if (text.charCodeAt(pos + 1) === 60 /* lessThan */) {
                                if (text.charCodeAt(pos + 2) === 61 /* equals */) {
                                    return pos += 3, token = 69 /* LessThanLessThanEqualsToken */;
                                }
                                return pos += 2, token = 47 /* LessThanLessThanToken */;
                            }
                            if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                                return pos += 2, token = 32 /* LessThanEqualsToken */;
                            }
                            if (languageVariant === 1 /* JSX */ &&
                                text.charCodeAt(pos + 1) === 47 /* slash */ &&
                                text.charCodeAt(pos + 2) !== 42 /* asterisk */) {
                                return pos += 2, token = 30 /* LessThanSlashToken */;
                            }
                            pos++;
                            return token = 29 /* LessThanToken */;
                        case 61 /* equals */:
                            if (isConflictMarkerTrivia(text, pos)) {
                                pos = scanConflictMarkerTrivia(text, pos, error);
                                if (skipTrivia) {
                                    continue;
                                }
                                else {
                                    return token = 7 /* ConflictMarkerTrivia */;
                                }
                            }
                            if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                                if (text.charCodeAt(pos + 2) === 61 /* equals */) {
                                    return pos += 3, token = 36 /* EqualsEqualsEqualsToken */;
                                }
                                return pos += 2, token = 34 /* EqualsEqualsToken */;
                            }
                            if (text.charCodeAt(pos + 1) === 62 /* greaterThan */) {
                                return pos += 2, token = 38 /* EqualsGreaterThanToken */;
                            }
                            pos++;
                            return token = 62 /* EqualsToken */;
                        case 62 /* greaterThan */:
                            if (isConflictMarkerTrivia(text, pos)) {
                                pos = scanConflictMarkerTrivia(text, pos, error);
                                if (skipTrivia) {
                                    continue;
                                }
                                else {
                                    return token = 7 /* ConflictMarkerTrivia */;
                                }
                            }
                            pos++;
                            return token = 31 /* GreaterThanToken */;
                        case 63 /* question */:
                            pos++;
                            if (text.charCodeAt(pos) === 46 /* dot */ && !isDigit(text.charCodeAt(pos + 1))) {
                                pos++;
                                return token = 28 /* QuestionDotToken */;
                            }
                            if (text.charCodeAt(pos) === 63 /* question */) {
                                pos++;
                                return token = 60 /* QuestionQuestionToken */;
                            }
                            return token = 57 /* QuestionToken */;
                        case 91 /* openBracket */:
                            pos++;
                            return token = 22 /* OpenBracketToken */;
                        case 93 /* closeBracket */:
                            pos++;
                            return token = 23 /* CloseBracketToken */;
                        case 94 /* caret */:
                            if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                                return pos += 2, token = 74 /* CaretEqualsToken */;
                            }
                            pos++;
                            return token = 52 /* CaretToken */;
                        case 123 /* openBrace */:
                            pos++;
                            return token = 18 /* OpenBraceToken */;
                        case 124 /* bar */:
                            if (isConflictMarkerTrivia(text, pos)) {
                                pos = scanConflictMarkerTrivia(text, pos, error);
                                if (skipTrivia) {
                                    continue;
                                }
                                else {
                                    return token = 7 /* ConflictMarkerTrivia */;
                                }
                            }
                            if (text.charCodeAt(pos + 1) === 124 /* bar */) {
                                return pos += 2, token = 56 /* BarBarToken */;
                            }
                            if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                                return pos += 2, token = 73 /* BarEqualsToken */;
                            }
                            pos++;
                            return token = 51 /* BarToken */;
                        case 125 /* closeBrace */:
                            pos++;
                            return token = 19 /* CloseBraceToken */;
                        case 126 /* tilde */:
                            pos++;
                            return token = 54 /* TildeToken */;
                        case 64 /* at */:
                            pos++;
                            return token = 59 /* AtToken */;
                        case 92 /* backslash */:
                            var extendedCookedChar = peekExtendedUnicodeEscape();
                            if (extendedCookedChar >= 0 && isIdentifierStart(extendedCookedChar, languageVersion)) {
                                pos += 3;
                                tokenFlags |= 8 /* ExtendedUnicodeEscape */;
                                tokenValue = scanExtendedUnicodeEscape() + scanIdentifierParts();
                                return token = getIdentifierToken();
                            }
                            var cookedChar = peekUnicodeEscape();
                            if (cookedChar >= 0 && isIdentifierStart(cookedChar, languageVersion)) {
                                pos += 6;
                                tokenFlags |= 1024 /* UnicodeEscape */;
                                tokenValue = String.fromCharCode(cookedChar) + scanIdentifierParts();
                                return token = getIdentifierToken();
                            }
                            error(ts.Diagnostics.Invalid_character);
                            pos++;
                            return token = 0 /* Unknown */;
                        case 35 /* hash */:
                            if (pos !== 0 && text[pos + 1] === "!") {
                                error(ts.Diagnostics.can_only_be_used_at_the_start_of_a_file);
                                pos++;
                                return token = 0 /* Unknown */;
                            }
                            pos++;
                            if (isIdentifierStart(ch = text.charCodeAt(pos), languageVersion)) {
                                pos++;
                                while (pos < end && isIdentifierPart(ch = text.charCodeAt(pos), languageVersion))
                                    pos++;
                                tokenValue = text.substring(tokenPos, pos);
                                if (ch === 92 /* backslash */) {
                                    tokenValue += scanIdentifierParts();
                                }
                            }
                            else {
                                tokenValue = "#";
                                error(ts.Diagnostics.Invalid_character);
                            }
                            return token = 76 /* PrivateIdentifier */;
                        default:
                            if (isIdentifierStart(ch, languageVersion)) {
                                pos += charSize(ch);
                                while (pos < end && isIdentifierPart(ch = codePointAt(text, pos), languageVersion))
                                    pos += charSize(ch);
                                tokenValue = text.substring(tokenPos, pos);
                                if (ch === 92 /* backslash */) {
                                    tokenValue += scanIdentifierParts();
                                }
                                return token = getIdentifierToken();
                            }
                            else if (isWhiteSpaceSingleLine(ch)) {
                                pos += charSize(ch);
                                continue;
                            }
                            else if (isLineBreak(ch)) {
                                tokenFlags |= 1 /* PrecedingLineBreak */;
                                pos += charSize(ch);
                                continue;
                            }
                            error(ts.Diagnostics.Invalid_character);
                            pos += charSize(ch);
                            return token = 0 /* Unknown */;
                    }
                }
            }
            function reScanGreaterToken() {
                if (token === 31 /* GreaterThanToken */) {
                    if (text.charCodeAt(pos) === 62 /* greaterThan */) {
                        if (text.charCodeAt(pos + 1) === 62 /* greaterThan */) {
                            if (text.charCodeAt(pos + 2) === 61 /* equals */) {
                                return pos += 3, token = 71 /* GreaterThanGreaterThanGreaterThanEqualsToken */;
                            }
                            return pos += 2, token = 49 /* GreaterThanGreaterThanGreaterThanToken */;
                        }
                        if (text.charCodeAt(pos + 1) === 61 /* equals */) {
                            return pos += 2, token = 70 /* GreaterThanGreaterThanEqualsToken */;
                        }
                        pos++;
                        return token = 48 /* GreaterThanGreaterThanToken */;
                    }
                    if (text.charCodeAt(pos) === 61 /* equals */) {
                        pos++;
                        return token = 33 /* GreaterThanEqualsToken */;
                    }
                }
                return token;
            }
            function reScanSlashToken() {
                if (token === 43 /* SlashToken */ || token === 67 /* SlashEqualsToken */) {
                    var p = tokenPos + 1;
                    var inEscape = false;
                    var inCharacterClass = false;
                    while (true) {
                        // If we reach the end of a file, or hit a newline, then this is an unterminated
                        // regex.  Report error and return what we have so far.
                        if (p >= end) {
                            tokenFlags |= 4 /* Unterminated */;
                            error(ts.Diagnostics.Unterminated_regular_expression_literal);
                            break;
                        }
                        var ch = text.charCodeAt(p);
                        if (isLineBreak(ch)) {
                            tokenFlags |= 4 /* Unterminated */;
                            error(ts.Diagnostics.Unterminated_regular_expression_literal);
                            break;
                        }
                        if (inEscape) {
                            // Parsing an escape character;
                            // reset the flag and just advance to the next char.
                            inEscape = false;
                        }
                        else if (ch === 47 /* slash */ && !inCharacterClass) {
                            // A slash within a character class is permissible,
                            // but in general it signals the end of the regexp literal.
                            p++;
                            break;
                        }
                        else if (ch === 91 /* openBracket */) {
                            inCharacterClass = true;
                        }
                        else if (ch === 92 /* backslash */) {
                            inEscape = true;
                        }
                        else if (ch === 93 /* closeBracket */) {
                            inCharacterClass = false;
                        }
                        p++;
                    }
                    while (p < end && isIdentifierPart(text.charCodeAt(p), languageVersion)) {
                        p++;
                    }
                    pos = p;
                    tokenValue = text.substring(tokenPos, pos);
                    token = 13 /* RegularExpressionLiteral */;
                }
                return token;
            }
            /**
             * Unconditionally back up and scan a template expression portion.
             */
            function reScanTemplateToken() {
                ts.Debug.assert(token === 19 /* CloseBraceToken */, "'reScanTemplateToken' should only be called on a '}'");
                pos = tokenPos;
                return token = scanTemplateAndSetTokenValue();
            }
            function reScanJsxToken() {
                pos = tokenPos = startPos;
                return token = scanJsxToken();
            }
            function reScanLessThanToken() {
                if (token === 47 /* LessThanLessThanToken */) {
                    pos = tokenPos + 1;
                    return token = 29 /* LessThanToken */;
                }
                return token;
            }
            function reScanQuestionToken() {
                ts.Debug.assert(token === 60 /* QuestionQuestionToken */, "'reScanQuestionToken' should only be called on a '??'");
                pos = tokenPos + 1;
                return token = 57 /* QuestionToken */;
            }
            function scanJsxToken() {
                startPos = tokenPos = pos;
                if (pos >= end) {
                    return token = 1 /* EndOfFileToken */;
                }
                var char = text.charCodeAt(pos);
                if (char === 60 /* lessThan */) {
                    if (text.charCodeAt(pos + 1) === 47 /* slash */) {
                        pos += 2;
                        return token = 30 /* LessThanSlashToken */;
                    }
                    pos++;
                    return token = 29 /* LessThanToken */;
                }
                if (char === 123 /* openBrace */) {
                    pos++;
                    return token = 18 /* OpenBraceToken */;
                }
                // First non-whitespace character on this line.
                var firstNonWhitespace = 0;
                var lastNonWhitespace = -1;
                // These initial values are special because the first line is:
                // firstNonWhitespace = 0 to indicate that we want leading whitespace,
                while (pos < end) {
                    // We want to keep track of the last non-whitespace (but including
                    // newlines character for hitting the end of the JSX Text region)
                    if (!isWhiteSpaceSingleLine(char)) {
                        lastNonWhitespace = pos;
                    }
                    char = text.charCodeAt(pos);
                    if (char === 123 /* openBrace */) {
                        break;
                    }
                    if (char === 60 /* lessThan */) {
                        if (isConflictMarkerTrivia(text, pos)) {
                            pos = scanConflictMarkerTrivia(text, pos, error);
                            return token = 7 /* ConflictMarkerTrivia */;
                        }
                        break;
                    }
                    if (lastNonWhitespace > 0)
                        lastNonWhitespace++;
                    // FirstNonWhitespace is 0, then we only see whitespaces so far. If we see a linebreak, we want to ignore that whitespaces.
                    // i.e (- : whitespace)
                    //      <div>----
                    //      </div> becomes <div></div>
                    //
                    //      <div>----</div> becomes <div>----</div>
                    if (isLineBreak(char) && firstNonWhitespace === 0) {
                        firstNonWhitespace = -1;
                    }
                    else if (!isWhiteSpaceLike(char)) {
                        firstNonWhitespace = pos;
                    }
                    pos++;
                }
                var endPosition = lastNonWhitespace === -1 ? pos : lastNonWhitespace;
                tokenValue = text.substring(startPos, endPosition);
                return firstNonWhitespace === -1 ? 12 /* JsxTextAllWhiteSpaces */ : 11 /* JsxText */;
            }
            // Scans a JSX identifier; these differ from normal identifiers in that
            // they allow dashes
            function scanJsxIdentifier() {
                if (tokenIsIdentifierOrKeyword(token)) {
                    // An identifier or keyword has already been parsed - check for a `-` and then append it and everything after it to the token
                    // Do note that this means that `scanJsxIdentifier` effectively _mutates_ the visible token without advancing to a new token
                    // Any caller should be expecting this behavior and should only read the pos or token value after calling it.
                    while (pos < end) {
                        var ch = text.charCodeAt(pos);
                        if (ch === 45 /* minus */) {
                            tokenValue += "-";
                            pos++;
                            continue;
                        }
                        var oldPos = pos;
                        tokenValue += scanIdentifierParts(); // reuse `scanIdentifierParts` so unicode escapes are handled
                        if (pos === oldPos) {
                            break;
                        }
                    }
                }
                return token;
            }
            function scanJsxAttributeValue() {
                startPos = pos;
                switch (text.charCodeAt(pos)) {
                    case 34 /* doubleQuote */:
                    case 39 /* singleQuote */:
                        tokenValue = scanString(/*jsxAttributeString*/ true);
                        return token = 10 /* StringLiteral */;
                    default:
                        // If this scans anything other than `{`, it's a parse error.
                        return scan();
                }
            }
            function reScanJsxAttributeValue() {
                pos = tokenPos = startPos;
                return scanJsxAttributeValue();
            }
            function scanJsDocToken() {
                startPos = tokenPos = pos;
                tokenFlags = 0 /* None */;
                if (pos >= end) {
                    return token = 1 /* EndOfFileToken */;
                }
                var ch = codePointAt(text, pos);
                pos += charSize(ch);
                switch (ch) {
                    case 9 /* tab */:
                    case 11 /* verticalTab */:
                    case 12 /* formFeed */:
                    case 32 /* space */:
                        while (pos < end && isWhiteSpaceSingleLine(text.charCodeAt(pos))) {
                            pos++;
                        }
                        return token = 5 /* WhitespaceTrivia */;
                    case 64 /* at */:
                        return token = 59 /* AtToken */;
                    case 10 /* lineFeed */:
                    case 13 /* carriageReturn */:
                        tokenFlags |= 1 /* PrecedingLineBreak */;
                        return token = 4 /* NewLineTrivia */;
                    case 42 /* asterisk */:
                        return token = 41 /* AsteriskToken */;
                    case 123 /* openBrace */:
                        return token = 18 /* OpenBraceToken */;
                    case 125 /* closeBrace */:
                        return token = 19 /* CloseBraceToken */;
                    case 91 /* openBracket */:
                        return token = 22 /* OpenBracketToken */;
                    case 93 /* closeBracket */:
                        return token = 23 /* CloseBracketToken */;
                    case 60 /* lessThan */:
                        return token = 29 /* LessThanToken */;
                    case 62 /* greaterThan */:
                        return token = 31 /* GreaterThanToken */;
                    case 61 /* equals */:
                        return token = 62 /* EqualsToken */;
                    case 44 /* comma */:
                        return token = 27 /* CommaToken */;
                    case 46 /* dot */:
                        return token = 24 /* DotToken */;
                    case 96 /* backtick */:
                        return token = 61 /* BacktickToken */;
                    case 92 /* backslash */:
                        pos--;
                        var extendedCookedChar = peekExtendedUnicodeEscape();
                        if (extendedCookedChar >= 0 && isIdentifierStart(extendedCookedChar, languageVersion)) {
                            pos += 3;
                            tokenFlags |= 8 /* ExtendedUnicodeEscape */;
                            tokenValue = scanExtendedUnicodeEscape() + scanIdentifierParts();
                            return token = getIdentifierToken();
                        }
                        var cookedChar = peekUnicodeEscape();
                        if (cookedChar >= 0 && isIdentifierStart(cookedChar, languageVersion)) {
                            pos += 6;
                            tokenFlags |= 1024 /* UnicodeEscape */;
                            tokenValue = String.fromCharCode(cookedChar) + scanIdentifierParts();
                            return token = getIdentifierToken();
                        }
                        pos++;
                        return token = 0 /* Unknown */;
                }
                if (isIdentifierStart(ch, languageVersion)) {
                    var char = ch;
                    while (pos < end && isIdentifierPart(char = codePointAt(text, pos), languageVersion) || text.charCodeAt(pos) === 45 /* minus */)
                        pos += charSize(char);
                    tokenValue = text.substring(tokenPos, pos);
                    if (char === 92 /* backslash */) {
                        tokenValue += scanIdentifierParts();
                    }
                    return token = getIdentifierToken();
                }
                else {
                    return token = 0 /* Unknown */;
                }
            }
            function speculationHelper(callback, isLookahead) {
                var savePos = pos;
                var saveStartPos = startPos;
                var saveTokenPos = tokenPos;
                var saveToken = token;
                var saveTokenValue = tokenValue;
                var saveTokenFlags = tokenFlags;
                var result = callback();
                // If our callback returned something 'falsy' or we're just looking ahead,
                // then unconditionally restore us to where we were.
                if (!result || isLookahead) {
                    pos = savePos;
                    startPos = saveStartPos;
                    tokenPos = saveTokenPos;
                    token = saveToken;
                    tokenValue = saveTokenValue;
                    tokenFlags = saveTokenFlags;
                }
                return result;
            }
            function scanRange(start, length, callback) {
                var saveEnd = end;
                var savePos = pos;
                var saveStartPos = startPos;
                var saveTokenPos = tokenPos;
                var saveToken = token;
                var saveTokenValue = tokenValue;
                var saveTokenFlags = tokenFlags;
                setText(text, start, length);
                var result = callback();
                end = saveEnd;
                pos = savePos;
                startPos = saveStartPos;
                tokenPos = saveTokenPos;
                token = saveToken;
                tokenValue = saveTokenValue;
                tokenFlags = saveTokenFlags;
                return result;
            }
            function lookAhead(callback) {
                return speculationHelper(callback, /*isLookahead*/ true);
            }
            function tryScan(callback) {
                return speculationHelper(callback, /*isLookahead*/ false);
            }
            function getText() {
                return text;
            }
            function setText(newText, start, length) {
                text = newText || "";
                end = length === undefined ? text.length : start + length;
                setTextPos(start || 0);
            }
            function setOnError(errorCallback) {
                onError = errorCallback;
            }
            function setScriptTarget(scriptTarget) {
                languageVersion = scriptTarget;
            }
            function setLanguageVariant(variant) {
                languageVariant = variant;
            }
            function setTextPos(textPos) {
                ts.Debug.assert(textPos >= 0);
                pos = textPos;
                startPos = textPos;
                tokenPos = textPos;
                token = 0 /* Unknown */;
                tokenValue = undefined;
                tokenFlags = 0 /* None */;
            }
            function setInJSDocType(inType) {
                inJSDocType += inType ? 1 : -1;
            }
        }
        ts.createScanner = createScanner;
        /* @internal */
        var codePointAt = String.prototype.codePointAt ? function (s, i) { return s.codePointAt(i); } : function codePointAt(str, i) {
            // from https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/codePointAt
            var size = str.length;
            // Account for out-of-bounds indices:
            if (i < 0 || i >= size) {
                return undefined; // String.codePointAt returns `undefined` for OOB indexes
            }
            // Get the first code unit
            var first = str.charCodeAt(i);
            // check if it’s the start of a surrogate pair
            if (first >= 0xD800 && first <= 0xDBFF && size > i + 1) { // high surrogate and there is a next code unit
                var second = str.charCodeAt(i + 1);
                if (second >= 0xDC00 && second <= 0xDFFF) { // low surrogate
                    // https://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
                    return (first - 0xD800) * 0x400 + second - 0xDC00 + 0x10000;
                }
            }
            return first;
        };
        /* @internal */
        function charSize(ch) {
            if (ch >= 0x10000) {
                return 2;
            }
            return 1;
        }
        // Derived from the 10.1.1 UTF16Encoding of the ES6 Spec.
        function utf16EncodeAsStringFallback(codePoint) {
            ts.Debug.assert(0x0 <= codePoint && codePoint <= 0x10FFFF);
            if (codePoint <= 65535) {
                return String.fromCharCode(codePoint);
            }
            var codeUnit1 = Math.floor((codePoint - 65536) / 1024) + 0xD800;
            var codeUnit2 = ((codePoint - 65536) % 1024) + 0xDC00;
            return String.fromCharCode(codeUnit1, codeUnit2);
        }
        var utf16EncodeAsStringWorker = String.fromCodePoint ? function (codePoint) { return String.fromCodePoint(codePoint); } : utf16EncodeAsStringFallback;
        /* @internal */
        function utf16EncodeAsString(codePoint) {
            return utf16EncodeAsStringWorker(codePoint);
        }
        ts.utf16EncodeAsString = utf16EncodeAsString;
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        function isExternalModuleNameRelative(moduleName) {
            // TypeScript 1.0 spec (April 2014): 11.2.1
            // An external module name is "relative" if the first term is "." or "..".
            // Update: We also consider a path like `C:\foo.ts` "relative" because we do not search for it in `node_modules` or treat it as an ambient module.
            return ts.pathIsRelative(moduleName) || ts.isRootedDiskPath(moduleName);
        }
        ts.isExternalModuleNameRelative = isExternalModuleNameRelative;
        function sortAndDeduplicateDiagnostics(diagnostics) {
            return ts.sortAndDeduplicate(diagnostics, ts.compareDiagnostics);
        }
        ts.sortAndDeduplicateDiagnostics = sortAndDeduplicateDiagnostics;
        function getDefaultLibFileName(options) {
            switch (options.target) {
                case 99 /* ESNext */:
                    return "lib.esnext.full.d.ts";
                case 7 /* ES2020 */:
                    return "lib.es2020.full.d.ts";
                case 6 /* ES2019 */:
                    return "lib.es2019.full.d.ts";
                case 5 /* ES2018 */:
                    return "lib.es2018.full.d.ts";
                case 4 /* ES2017 */:
                    return "lib.es2017.full.d.ts";
                case 3 /* ES2016 */:
                    return "lib.es2016.full.d.ts";
                case 2 /* ES2015 */:
                    return "lib.es6.d.ts"; // We don't use lib.es2015.full.d.ts due to breaking change.
                default:
                    return "lib.d.ts";
            }
        }
        ts.getDefaultLibFileName = getDefaultLibFileName;
        function textSpanEnd(span) {
            return span.start + span.length;
        }
        ts.textSpanEnd = textSpanEnd;
        function textSpanIsEmpty(span) {
            return span.length === 0;
        }
        ts.textSpanIsEmpty = textSpanIsEmpty;
        function textSpanContainsPosition(span, position) {
            return position >= span.start && position < textSpanEnd(span);
        }
        ts.textSpanContainsPosition = textSpanContainsPosition;
        /* @internal */
        function textRangeContainsPositionInclusive(span, position) {
            return position >= span.pos && position <= span.end;
        }
        ts.textRangeContainsPositionInclusive = textRangeContainsPositionInclusive;
        // Returns true if 'span' contains 'other'.
        function textSpanContainsTextSpan(span, other) {
            return other.start >= span.start && textSpanEnd(other) <= textSpanEnd(span);
        }
        ts.textSpanContainsTextSpan = textSpanContainsTextSpan;
        function textSpanOverlapsWith(span, other) {
            return textSpanOverlap(span, other) !== undefined;
        }
        ts.textSpanOverlapsWith = textSpanOverlapsWith;
        function textSpanOverlap(span1, span2) {
            var overlap = textSpanIntersection(span1, span2);
            return overlap && overlap.length === 0 ? undefined : overlap;
        }
        ts.textSpanOverlap = textSpanOverlap;
        function textSpanIntersectsWithTextSpan(span, other) {
            return decodedTextSpanIntersectsWith(span.start, span.length, other.start, other.length);
        }
        ts.textSpanIntersectsWithTextSpan = textSpanIntersectsWithTextSpan;
        function textSpanIntersectsWith(span, start, length) {
            return decodedTextSpanIntersectsWith(span.start, span.length, start, length);
        }
        ts.textSpanIntersectsWith = textSpanIntersectsWith;
        function decodedTextSpanIntersectsWith(start1, length1, start2, length2) {
            var end1 = start1 + length1;
            var end2 = start2 + length2;
            return start2 <= end1 && end2 >= start1;
        }
        ts.decodedTextSpanIntersectsWith = decodedTextSpanIntersectsWith;
        function textSpanIntersectsWithPosition(span, position) {
            return position <= textSpanEnd(span) && position >= span.start;
        }
        ts.textSpanIntersectsWithPosition = textSpanIntersectsWithPosition;
        function textSpanIntersection(span1, span2) {
            var start = Math.max(span1.start, span2.start);
            var end = Math.min(textSpanEnd(span1), textSpanEnd(span2));
            return start <= end ? createTextSpanFromBounds(start, end) : undefined;
        }
        ts.textSpanIntersection = textSpanIntersection;
        function createTextSpan(start, length) {
            if (start < 0) {
                throw new Error("start < 0");
            }
            if (length < 0) {
                throw new Error("length < 0");
            }
            return { start: start, length: length };
        }
        ts.createTextSpan = createTextSpan;
        function createTextSpanFromBounds(start, end) {
            return createTextSpan(start, end - start);
        }
        ts.createTextSpanFromBounds = createTextSpanFromBounds;
        function textChangeRangeNewSpan(range) {
            return createTextSpan(range.span.start, range.newLength);
        }
        ts.textChangeRangeNewSpan = textChangeRangeNewSpan;
        function textChangeRangeIsUnchanged(range) {
            return textSpanIsEmpty(range.span) && range.newLength === 0;
        }
        ts.textChangeRangeIsUnchanged = textChangeRangeIsUnchanged;
        function createTextChangeRange(span, newLength) {
            if (newLength < 0) {
                throw new Error("newLength < 0");
            }
            return { span: span, newLength: newLength };
        }
        ts.createTextChangeRange = createTextChangeRange;
        ts.unchangedTextChangeRange = createTextChangeRange(createTextSpan(0, 0), 0); // eslint-disable-line prefer-const
        /**
         * Called to merge all the changes that occurred across several versions of a script snapshot
         * into a single change.  i.e. if a user keeps making successive edits to a script we will
         * have a text change from V1 to V2, V2 to V3, ..., Vn.
         *
         * This function will then merge those changes into a single change range valid between V1 and
         * Vn.
         */
        function collapseTextChangeRangesAcrossMultipleVersions(changes) {
            if (changes.length === 0) {
                return ts.unchangedTextChangeRange;
            }
            if (changes.length === 1) {
                return changes[0];
            }
            // We change from talking about { { oldStart, oldLength }, newLength } to { oldStart, oldEnd, newEnd }
            // as it makes things much easier to reason about.
            var change0 = changes[0];
            var oldStartN = change0.span.start;
            var oldEndN = textSpanEnd(change0.span);
            var newEndN = oldStartN + change0.newLength;
            for (var i = 1; i < changes.length; i++) {
                var nextChange = changes[i];
                // Consider the following case:
                // i.e. two edits.  The first represents the text change range { { 10, 50 }, 30 }.  i.e. The span starting
                // at 10, with length 50 is reduced to length 30.  The second represents the text change range { { 30, 30 }, 40 }.
                // i.e. the span starting at 30 with length 30 is increased to length 40.
                //
                //      0         10        20        30        40        50        60        70        80        90        100
                //      -------------------------------------------------------------------------------------------------------
                //                |                                                 /
                //                |                                            /----
                //  T1            |                                       /----
                //                |                                  /----
                //                |                             /----
                //      -------------------------------------------------------------------------------------------------------
                //                                     |                            \
                //                                     |                               \
                //   T2                                |                                 \
                //                                     |                                   \
                //                                     |                                      \
                //      -------------------------------------------------------------------------------------------------------
                //
                // Merging these turns out to not be too difficult.  First, determining the new start of the change is trivial
                // it's just the min of the old and new starts.  i.e.:
                //
                //      0         10        20        30        40        50        60        70        80        90        100
                //      ------------------------------------------------------------*------------------------------------------
                //                |                                                 /
                //                |                                            /----
                //  T1            |                                       /----
                //                |                                  /----
                //                |                             /----
                //      ----------------------------------------$-------------------$------------------------------------------
                //                .                    |                            \
                //                .                    |                               \
                //   T2           .                    |                                 \
                //                .                    |                                   \
                //                .                    |                                      \
                //      ----------------------------------------------------------------------*--------------------------------
                //
                // (Note the dots represent the newly inferred start.
                // Determining the new and old end is also pretty simple.  Basically it boils down to paying attention to the
                // absolute positions at the asterisks, and the relative change between the dollar signs. Basically, we see
                // which if the two $'s precedes the other, and we move that one forward until they line up.  in this case that
                // means:
                //
                //      0         10        20        30        40        50        60        70        80        90        100
                //      --------------------------------------------------------------------------------*----------------------
                //                |                                                                     /
                //                |                                                                /----
                //  T1            |                                                           /----
                //                |                                                      /----
                //                |                                                 /----
                //      ------------------------------------------------------------$------------------------------------------
                //                .                    |                            \
                //                .                    |                               \
                //   T2           .                    |                                 \
                //                .                    |                                   \
                //                .                    |                                      \
                //      ----------------------------------------------------------------------*--------------------------------
                //
                // In other words (in this case), we're recognizing that the second edit happened after where the first edit
                // ended with a delta of 20 characters (60 - 40).  Thus, if we go back in time to where the first edit started
                // that's the same as if we started at char 80 instead of 60.
                //
                // As it so happens, the same logic applies if the second edit precedes the first edit.  In that case rather
                // than pushing the first edit forward to match the second, we'll push the second edit forward to match the
                // first.
                //
                // In this case that means we have { oldStart: 10, oldEnd: 80, newEnd: 70 } or, in TextChangeRange
                // semantics: { { start: 10, length: 70 }, newLength: 60 }
                //
                // The math then works out as follows.
                // If we have { oldStart1, oldEnd1, newEnd1 } and { oldStart2, oldEnd2, newEnd2 } then we can compute the
                // final result like so:
                //
                // {
                //      oldStart3: Min(oldStart1, oldStart2),
                //      oldEnd3: Max(oldEnd1, oldEnd1 + (oldEnd2 - newEnd1)),
                //      newEnd3: Max(newEnd2, newEnd2 + (newEnd1 - oldEnd2))
                // }
                var oldStart1 = oldStartN;
                var oldEnd1 = oldEndN;
                var newEnd1 = newEndN;
                var oldStart2 = nextChange.span.start;
                var oldEnd2 = textSpanEnd(nextChange.span);
                var newEnd2 = oldStart2 + nextChange.newLength;
                oldStartN = Math.min(oldStart1, oldStart2);
                oldEndN = Math.max(oldEnd1, oldEnd1 + (oldEnd2 - newEnd1));
                newEndN = Math.max(newEnd2, newEnd2 + (newEnd1 - oldEnd2));
            }
            return createTextChangeRange(createTextSpanFromBounds(oldStartN, oldEndN), /*newLength*/ newEndN - oldStartN);
        }
        ts.collapseTextChangeRangesAcrossMultipleVersions = collapseTextChangeRangesAcrossMultipleVersions;
        function getTypeParameterOwner(d) {
            if (d && d.kind === 155 /* TypeParameter */) {
                for (var current = d; current; current = current.parent) {
                    if (isFunctionLike(current) || isClassLike(current) || current.kind === 246 /* InterfaceDeclaration */) {
                        return current;
                    }
                }
            }
        }
        ts.getTypeParameterOwner = getTypeParameterOwner;
        function isParameterPropertyDeclaration(node, parent) {
            return ts.hasModifier(node, 92 /* ParameterPropertyModifier */) && parent.kind === 162 /* Constructor */;
        }
        ts.isParameterPropertyDeclaration = isParameterPropertyDeclaration;
        function isEmptyBindingPattern(node) {
            if (isBindingPattern(node)) {
                return ts.every(node.elements, isEmptyBindingElement);
            }
            return false;
        }
        ts.isEmptyBindingPattern = isEmptyBindingPattern;
        function isEmptyBindingElement(node) {
            if (isOmittedExpression(node)) {
                return true;
            }
            return isEmptyBindingPattern(node.name);
        }
        ts.isEmptyBindingElement = isEmptyBindingElement;
        function walkUpBindingElementsAndPatterns(binding) {
            var node = binding.parent;
            while (isBindingElement(node.parent)) {
                node = node.parent.parent;
            }
            return node.parent;
        }
        ts.walkUpBindingElementsAndPatterns = walkUpBindingElementsAndPatterns;
        function getCombinedFlags(node, getFlags) {
            if (isBindingElement(node)) {
                node = walkUpBindingElementsAndPatterns(node);
            }
            var flags = getFlags(node);
            if (node.kind === 242 /* VariableDeclaration */) {
                node = node.parent;
            }
            if (node && node.kind === 243 /* VariableDeclarationList */) {
                flags |= getFlags(node);
                node = node.parent;
            }
            if (node && node.kind === 225 /* VariableStatement */) {
                flags |= getFlags(node);
            }
            return flags;
        }
        function getCombinedModifierFlags(node) {
            return getCombinedFlags(node, ts.getModifierFlags);
        }
        ts.getCombinedModifierFlags = getCombinedModifierFlags;
        // Returns the node flags for this node and all relevant parent nodes.  This is done so that
        // nodes like variable declarations and binding elements can returned a view of their flags
        // that includes the modifiers from their container.  i.e. flags like export/declare aren't
        // stored on the variable declaration directly, but on the containing variable statement
        // (if it has one).  Similarly, flags for let/const are store on the variable declaration
        // list.  By calling this function, all those flags are combined so that the client can treat
        // the node as if it actually had those flags.
        function getCombinedNodeFlags(node) {
            return getCombinedFlags(node, function (n) { return n.flags; });
        }
        ts.getCombinedNodeFlags = getCombinedNodeFlags;
        /**
         * Checks to see if the locale is in the appropriate format,
         * and if it is, attempts to set the appropriate language.
         */
        function validateLocaleAndSetLanguage(locale, sys, errors) {
            var matchResult = /^([a-z]+)([_\-]([a-z]+))?$/.exec(locale.toLowerCase());
            if (!matchResult) {
                if (errors) {
                    errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Locale_must_be_of_the_form_language_or_language_territory_For_example_0_or_1, "en", "ja-jp"));
                }
                return;
            }
            var language = matchResult[1];
            var territory = matchResult[3];
            // First try the entire locale, then fall back to just language if that's all we have.
            // Either ways do not fail, and fallback to the English diagnostic strings.
            if (!trySetLanguageAndTerritory(language, territory, errors)) {
                trySetLanguageAndTerritory(language, /*territory*/ undefined, errors);
            }
            // Set the UI locale for string collation
            ts.setUILocale(locale);
            function trySetLanguageAndTerritory(language, territory, errors) {
                var compilerFilePath = ts.normalizePath(sys.getExecutingFilePath());
                var containingDirectoryPath = ts.getDirectoryPath(compilerFilePath);
                var filePath = ts.combinePaths(containingDirectoryPath, language);
                if (territory) {
                    filePath = filePath + "-" + territory;
                }
                filePath = sys.resolvePath(ts.combinePaths(filePath, "diagnosticMessages.generated.json"));
                if (!sys.fileExists(filePath)) {
                    return false;
                }
                // TODO: Add codePage support for readFile?
                var fileContents = "";
                try {
                    fileContents = sys.readFile(filePath);
                }
                catch (e) {
                    if (errors) {
                        errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Unable_to_open_file_0, filePath));
                    }
                    return false;
                }
                try {
                    // this is a global mutation (or live binding update)!
                    ts.setLocalizedDiagnosticMessages(JSON.parse(fileContents));
                }
                catch (_a) {
                    if (errors) {
                        errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Corrupted_locale_file_0, filePath));
                    }
                    return false;
                }
                return true;
            }
        }
        ts.validateLocaleAndSetLanguage = validateLocaleAndSetLanguage;
        function getOriginalNode(node, nodeTest) {
            if (node) {
                while (node.original !== undefined) {
                    node = node.original;
                }
            }
            return !nodeTest || nodeTest(node) ? node : undefined;
        }
        ts.getOriginalNode = getOriginalNode;
        /**
         * Gets a value indicating whether a node originated in the parse tree.
         *
         * @param node The node to test.
         */
        function isParseTreeNode(node) {
            return (node.flags & 8 /* Synthesized */) === 0;
        }
        ts.isParseTreeNode = isParseTreeNode;
        function getParseTreeNode(node, nodeTest) {
            if (node === undefined || isParseTreeNode(node)) {
                return node;
            }
            node = getOriginalNode(node);
            if (isParseTreeNode(node) && (!nodeTest || nodeTest(node))) {
                return node;
            }
            return undefined;
        }
        ts.getParseTreeNode = getParseTreeNode;
        /** Add an extra underscore to identifiers that start with two underscores to avoid issues with magic names like '__proto__' */
        function escapeLeadingUnderscores(identifier) {
            return (identifier.length >= 2 && identifier.charCodeAt(0) === 95 /* _ */ && identifier.charCodeAt(1) === 95 /* _ */ ? "_" + identifier : identifier);
        }
        ts.escapeLeadingUnderscores = escapeLeadingUnderscores;
        /**
         * Remove extra underscore from escaped identifier text content.
         *
         * @param identifier The escaped identifier text.
         * @returns The unescaped identifier text.
         */
        function unescapeLeadingUnderscores(identifier) {
            var id = identifier;
            return id.length >= 3 && id.charCodeAt(0) === 95 /* _ */ && id.charCodeAt(1) === 95 /* _ */ && id.charCodeAt(2) === 95 /* _ */ ? id.substr(1) : id;
        }
        ts.unescapeLeadingUnderscores = unescapeLeadingUnderscores;
        function idText(identifierOrPrivateName) {
            return unescapeLeadingUnderscores(identifierOrPrivateName.escapedText);
        }
        ts.idText = idText;
        function symbolName(symbol) {
            if (symbol.valueDeclaration && isPrivateIdentifierPropertyDeclaration(symbol.valueDeclaration)) {
                return idText(symbol.valueDeclaration.name);
            }
            return unescapeLeadingUnderscores(symbol.escapedName);
        }
        ts.symbolName = symbolName;
        /**
         * A JSDocTypedef tag has an _optional_ name field - if a name is not directly present, we should
         * attempt to draw the name from the node the declaration is on (as that declaration is what its' symbol
         * will be merged with)
         */
        function nameForNamelessJSDocTypedef(declaration) {
            var hostNode = declaration.parent.parent;
            if (!hostNode) {
                return undefined;
            }
            // Covers classes, functions - any named declaration host node
            if (isDeclaration(hostNode)) {
                return getDeclarationIdentifier(hostNode);
            }
            // Covers remaining cases (returning undefined if none match).
            switch (hostNode.kind) {
                case 225 /* VariableStatement */:
                    if (hostNode.declarationList && hostNode.declarationList.declarations[0]) {
                        return getDeclarationIdentifier(hostNode.declarationList.declarations[0]);
                    }
                    break;
                case 226 /* ExpressionStatement */:
                    var expr = hostNode.expression;
                    if (expr.kind === 209 /* BinaryExpression */ && expr.operatorToken.kind === 62 /* EqualsToken */) {
                        expr = expr.left;
                    }
                    switch (expr.kind) {
                        case 194 /* PropertyAccessExpression */:
                            return expr.name;
                        case 195 /* ElementAccessExpression */:
                            var arg = expr.argumentExpression;
                            if (isIdentifier(arg)) {
                                return arg;
                            }
                    }
                    break;
                case 200 /* ParenthesizedExpression */: {
                    return getDeclarationIdentifier(hostNode.expression);
                }
                case 238 /* LabeledStatement */: {
                    if (isDeclaration(hostNode.statement) || isExpression(hostNode.statement)) {
                        return getDeclarationIdentifier(hostNode.statement);
                    }
                    break;
                }
            }
        }
        function getDeclarationIdentifier(node) {
            var name = getNameOfDeclaration(node);
            return name && isIdentifier(name) ? name : undefined;
        }
        /** @internal */
        function nodeHasName(statement, name) {
            if (isNamedDeclaration(statement) && isIdentifier(statement.name) && idText(statement.name) === idText(name)) {
                return true;
            }
            if (isVariableStatement(statement) && ts.some(statement.declarationList.declarations, function (d) { return nodeHasName(d, name); })) {
                return true;
            }
            return false;
        }
        ts.nodeHasName = nodeHasName;
        function getNameOfJSDocTypedef(declaration) {
            return declaration.name || nameForNamelessJSDocTypedef(declaration);
        }
        ts.getNameOfJSDocTypedef = getNameOfJSDocTypedef;
        /** @internal */
        function isNamedDeclaration(node) {
            return !!node.name; // A 'name' property should always be a DeclarationName.
        }
        ts.isNamedDeclaration = isNamedDeclaration;
        /** @internal */
        function getNonAssignedNameOfDeclaration(declaration) {
            switch (declaration.kind) {
                case 75 /* Identifier */:
                    return declaration;
                case 322 /* JSDocPropertyTag */:
                case 316 /* JSDocParameterTag */: {
                    var name = declaration.name;
                    if (name.kind === 153 /* QualifiedName */) {
                        return name.right;
                    }
                    break;
                }
                case 196 /* CallExpression */:
                case 209 /* BinaryExpression */: {
                    var expr_1 = declaration;
                    switch (ts.getAssignmentDeclarationKind(expr_1)) {
                        case 1 /* ExportsProperty */:
                        case 4 /* ThisProperty */:
                        case 5 /* Property */:
                        case 3 /* PrototypeProperty */:
                            return ts.getElementOrPropertyAccessArgumentExpressionOrName(expr_1.left);
                        case 7 /* ObjectDefinePropertyValue */:
                        case 8 /* ObjectDefinePropertyExports */:
                        case 9 /* ObjectDefinePrototypeProperty */:
                            return expr_1.arguments[1];
                        default:
                            return undefined;
                    }
                }
                case 321 /* JSDocTypedefTag */:
                    return getNameOfJSDocTypedef(declaration);
                case 315 /* JSDocEnumTag */:
                    return nameForNamelessJSDocTypedef(declaration);
                case 259 /* ExportAssignment */: {
                    var expression = declaration.expression;
                    return isIdentifier(expression) ? expression : undefined;
                }
                case 195 /* ElementAccessExpression */:
                    var expr = declaration;
                    if (ts.isBindableStaticElementAccessExpression(expr)) {
                        return expr.argumentExpression;
                    }
            }
            return declaration.name;
        }
        ts.getNonAssignedNameOfDeclaration = getNonAssignedNameOfDeclaration;
        function getNameOfDeclaration(declaration) {
            if (declaration === undefined)
                return undefined;
            return getNonAssignedNameOfDeclaration(declaration) ||
                (isFunctionExpression(declaration) || isClassExpression(declaration) ? getAssignedName(declaration) : undefined);
        }
        ts.getNameOfDeclaration = getNameOfDeclaration;
        function getAssignedName(node) {
            if (!node.parent) {
                return undefined;
            }
            else if (isPropertyAssignment(node.parent) || isBindingElement(node.parent)) {
                return node.parent.name;
            }
            else if (isBinaryExpression(node.parent) && node === node.parent.right) {
                if (isIdentifier(node.parent.left)) {
                    return node.parent.left;
                }
                else if (ts.isAccessExpression(node.parent.left)) {
                    return ts.getElementOrPropertyAccessArgumentExpressionOrName(node.parent.left);
                }
            }
            else if (isVariableDeclaration(node.parent) && isIdentifier(node.parent.name)) {
                return node.parent.name;
            }
        }
        /**
         * Gets the JSDoc parameter tags for the node if present.
         *
         * @remarks Returns any JSDoc param tag whose name matches the provided
         * parameter, whether a param tag on a containing function
         * expression, or a param tag on a variable declaration whose
         * initializer is the containing function. The tags closest to the
         * node are returned first, so in the previous example, the param
         * tag on the containing function expression would be first.
         *
         * For binding patterns, parameter tags are matched by position.
         */
        function getJSDocParameterTags(param) {
            if (param.name) {
                if (isIdentifier(param.name)) {
                    var name_1 = param.name.escapedText;
                    return getJSDocTags(param.parent).filter(function (tag) { return isJSDocParameterTag(tag) && isIdentifier(tag.name) && tag.name.escapedText === name_1; });
                }
                else {
                    var i = param.parent.parameters.indexOf(param);
                    ts.Debug.assert(i > -1, "Parameters should always be in their parents' parameter list");
                    var paramTags = getJSDocTags(param.parent).filter(isJSDocParameterTag);
                    if (i < paramTags.length) {
                        return [paramTags[i]];
                    }
                }
            }
            // return empty array for: out-of-order binding patterns and JSDoc function syntax, which has un-named parameters
            return ts.emptyArray;
        }
        ts.getJSDocParameterTags = getJSDocParameterTags;
        /**
         * Gets the JSDoc type parameter tags for the node if present.
         *
         * @remarks Returns any JSDoc template tag whose names match the provided
         * parameter, whether a template tag on a containing function
         * expression, or a template tag on a variable declaration whose
         * initializer is the containing function. The tags closest to the
         * node are returned first, so in the previous example, the template
         * tag on the containing function expression would be first.
         */
        function getJSDocTypeParameterTags(param) {
            var name = param.name.escapedText;
            return getJSDocTags(param.parent).filter(function (tag) {
                return isJSDocTemplateTag(tag) && tag.typeParameters.some(function (tp) { return tp.name.escapedText === name; });
            });
        }
        ts.getJSDocTypeParameterTags = getJSDocTypeParameterTags;
        /**
         * Return true if the node has JSDoc parameter tags.
         *
         * @remarks Includes parameter tags that are not directly on the node,
         * for example on a variable declaration whose initializer is a function expression.
         */
        function hasJSDocParameterTags(node) {
            return !!getFirstJSDocTag(node, isJSDocParameterTag);
        }
        ts.hasJSDocParameterTags = hasJSDocParameterTags;
        /** Gets the JSDoc augments tag for the node if present */
        function getJSDocAugmentsTag(node) {
            return getFirstJSDocTag(node, isJSDocAugmentsTag);
        }
        ts.getJSDocAugmentsTag = getJSDocAugmentsTag;
        /** Gets the JSDoc class tag for the node if present */
        function getJSDocClassTag(node) {
            return getFirstJSDocTag(node, isJSDocClassTag);
        }
        ts.getJSDocClassTag = getJSDocClassTag;
        /** Gets the JSDoc public tag for the node if present */
        function getJSDocPublicTag(node) {
            return getFirstJSDocTag(node, isJSDocPublicTag);
        }
        ts.getJSDocPublicTag = getJSDocPublicTag;
        /** Gets the JSDoc private tag for the node if present */
        function getJSDocPrivateTag(node) {
            return getFirstJSDocTag(node, isJSDocPrivateTag);
        }
        ts.getJSDocPrivateTag = getJSDocPrivateTag;
        /** Gets the JSDoc protected tag for the node if present */
        function getJSDocProtectedTag(node) {
            return getFirstJSDocTag(node, isJSDocProtectedTag);
        }
        ts.getJSDocProtectedTag = getJSDocProtectedTag;
        /** Gets the JSDoc protected tag for the node if present */
        function getJSDocReadonlyTag(node) {
            return getFirstJSDocTag(node, isJSDocReadonlyTag);
        }
        ts.getJSDocReadonlyTag = getJSDocReadonlyTag;
        /** Gets the JSDoc enum tag for the node if present */
        function getJSDocEnumTag(node) {
            return getFirstJSDocTag(node, isJSDocEnumTag);
        }
        ts.getJSDocEnumTag = getJSDocEnumTag;
        /** Gets the JSDoc this tag for the node if present */
        function getJSDocThisTag(node) {
            return getFirstJSDocTag(node, isJSDocThisTag);
        }
        ts.getJSDocThisTag = getJSDocThisTag;
        /** Gets the JSDoc return tag for the node if present */
        function getJSDocReturnTag(node) {
            return getFirstJSDocTag(node, isJSDocReturnTag);
        }
        ts.getJSDocReturnTag = getJSDocReturnTag;
        /** Gets the JSDoc template tag for the node if present */
        function getJSDocTemplateTag(node) {
            return getFirstJSDocTag(node, isJSDocTemplateTag);
        }
        ts.getJSDocTemplateTag = getJSDocTemplateTag;
        /** Gets the JSDoc type tag for the node if present and valid */
        function getJSDocTypeTag(node) {
            // We should have already issued an error if there were multiple type jsdocs, so just use the first one.
            var tag = getFirstJSDocTag(node, isJSDocTypeTag);
            if (tag && tag.typeExpression && tag.typeExpression.type) {
                return tag;
            }
            return undefined;
        }
        ts.getJSDocTypeTag = getJSDocTypeTag;
        /**
         * Gets the type node for the node if provided via JSDoc.
         *
         * @remarks The search includes any JSDoc param tag that relates
         * to the provided parameter, for example a type tag on the
         * parameter itself, or a param tag on a containing function
         * expression, or a param tag on a variable declaration whose
         * initializer is the containing function. The tags closest to the
         * node are examined first, so in the previous example, the type
         * tag directly on the node would be returned.
         */
        function getJSDocType(node) {
            var tag = getFirstJSDocTag(node, isJSDocTypeTag);
            if (!tag && isParameter(node)) {
                tag = ts.find(getJSDocParameterTags(node), function (tag) { return !!tag.typeExpression; });
            }
            return tag && tag.typeExpression && tag.typeExpression.type;
        }
        ts.getJSDocType = getJSDocType;
        /**
         * Gets the return type node for the node if provided via JSDoc return tag or type tag.
         *
         * @remarks `getJSDocReturnTag` just gets the whole JSDoc tag. This function
         * gets the type from inside the braces, after the fat arrow, etc.
         */
        function getJSDocReturnType(node) {
            var returnTag = getJSDocReturnTag(node);
            if (returnTag && returnTag.typeExpression) {
                return returnTag.typeExpression.type;
            }
            var typeTag = getJSDocTypeTag(node);
            if (typeTag && typeTag.typeExpression) {
                var type = typeTag.typeExpression.type;
                if (isTypeLiteralNode(type)) {
                    var sig = ts.find(type.members, isCallSignatureDeclaration);
                    return sig && sig.type;
                }
                if (isFunctionTypeNode(type)) {
                    return type.type;
                }
            }
        }
        ts.getJSDocReturnType = getJSDocReturnType;
        /** Get all JSDoc tags related to a node, including those on parent nodes. */
        function getJSDocTags(node) {
            var tags = node.jsDocCache;
            // If cache is 'null', that means we did the work of searching for JSDoc tags and came up with nothing.
            if (tags === undefined) {
                var comments = ts.getJSDocCommentsAndTags(node);
                ts.Debug.assert(comments.length < 2 || comments[0] !== comments[1]);
                node.jsDocCache = tags = ts.flatMap(comments, function (j) { return isJSDoc(j) ? j.tags : j; });
            }
            return tags;
        }
        ts.getJSDocTags = getJSDocTags;
        /** Get the first JSDoc tag of a specified kind, or undefined if not present. */
        function getFirstJSDocTag(node, predicate) {
            return ts.find(getJSDocTags(node), predicate);
        }
        /** Gets all JSDoc tags of a specified kind, or undefined if not present. */
        function getAllJSDocTagsOfKind(node, kind) {
            return getJSDocTags(node).filter(function (doc) { return doc.kind === kind; });
        }
        ts.getAllJSDocTagsOfKind = getAllJSDocTagsOfKind;
        /**
         * Gets the effective type parameters. If the node was parsed in a
         * JavaScript file, gets the type parameters from the `@template` tag from JSDoc.
         */
        function getEffectiveTypeParameterDeclarations(node) {
            if (isJSDocSignature(node)) {
                return ts.emptyArray;
            }
            if (ts.isJSDocTypeAlias(node)) {
                ts.Debug.assert(node.parent.kind === 303 /* JSDocComment */);
                return ts.flatMap(node.parent.tags, function (tag) { return isJSDocTemplateTag(tag) ? tag.typeParameters : undefined; });
            }
            if (node.typeParameters) {
                return node.typeParameters;
            }
            if (ts.isInJSFile(node)) {
                var decls = ts.getJSDocTypeParameterDeclarations(node);
                if (decls.length) {
                    return decls;
                }
                var typeTag = getJSDocType(node);
                if (typeTag && isFunctionTypeNode(typeTag) && typeTag.typeParameters) {
                    return typeTag.typeParameters;
                }
            }
            return ts.emptyArray;
        }
        ts.getEffectiveTypeParameterDeclarations = getEffectiveTypeParameterDeclarations;
        function getEffectiveConstraintOfTypeParameter(node) {
            return node.constraint ? node.constraint :
                isJSDocTemplateTag(node.parent) && node === node.parent.typeParameters[0] ? node.parent.constraint :
                    undefined;
        }
        ts.getEffectiveConstraintOfTypeParameter = getEffectiveConstraintOfTypeParameter;
        // #region
        // Simple node tests of the form `node.kind === SyntaxKind.Foo`.
        // Literals
        function isNumericLiteral(node) {
            return node.kind === 8 /* NumericLiteral */;
        }
        ts.isNumericLiteral = isNumericLiteral;
        function isBigIntLiteral(node) {
            return node.kind === 9 /* BigIntLiteral */;
        }
        ts.isBigIntLiteral = isBigIntLiteral;
        function isStringLiteral(node) {
            return node.kind === 10 /* StringLiteral */;
        }
        ts.isStringLiteral = isStringLiteral;
        function isJsxText(node) {
            return node.kind === 11 /* JsxText */;
        }
        ts.isJsxText = isJsxText;
        function isRegularExpressionLiteral(node) {
            return node.kind === 13 /* RegularExpressionLiteral */;
        }
        ts.isRegularExpressionLiteral = isRegularExpressionLiteral;
        function isNoSubstitutionTemplateLiteral(node) {
            return node.kind === 14 /* NoSubstitutionTemplateLiteral */;
        }
        ts.isNoSubstitutionTemplateLiteral = isNoSubstitutionTemplateLiteral;
        // Pseudo-literals
        function isTemplateHead(node) {
            return node.kind === 15 /* TemplateHead */;
        }
        ts.isTemplateHead = isTemplateHead;
        function isTemplateMiddle(node) {
            return node.kind === 16 /* TemplateMiddle */;
        }
        ts.isTemplateMiddle = isTemplateMiddle;
        function isTemplateTail(node) {
            return node.kind === 17 /* TemplateTail */;
        }
        ts.isTemplateTail = isTemplateTail;
        function isIdentifier(node) {
            return node.kind === 75 /* Identifier */;
        }
        ts.isIdentifier = isIdentifier;
        // Names
        function isQualifiedName(node) {
            return node.kind === 153 /* QualifiedName */;
        }
        ts.isQualifiedName = isQualifiedName;
        function isComputedPropertyName(node) {
            return node.kind === 154 /* ComputedPropertyName */;
        }
        ts.isComputedPropertyName = isComputedPropertyName;
        function isPrivateIdentifier(node) {
            return node.kind === 76 /* PrivateIdentifier */;
        }
        ts.isPrivateIdentifier = isPrivateIdentifier;
        function isIdentifierOrPrivateIdentifier(node) {
            return node.kind === 75 /* Identifier */ || node.kind === 76 /* PrivateIdentifier */;
        }
        ts.isIdentifierOrPrivateIdentifier = isIdentifierOrPrivateIdentifier;
        // Signature elements
        function isTypeParameterDeclaration(node) {
            return node.kind === 155 /* TypeParameter */;
        }
        ts.isTypeParameterDeclaration = isTypeParameterDeclaration;
        function isParameter(node) {
            return node.kind === 156 /* Parameter */;
        }
        ts.isParameter = isParameter;
        function isDecorator(node) {
            return node.kind === 157 /* Decorator */;
        }
        ts.isDecorator = isDecorator;
        // TypeMember
        function isPropertySignature(node) {
            return node.kind === 158 /* PropertySignature */;
        }
        ts.isPropertySignature = isPropertySignature;
        function isPropertyDeclaration(node) {
            return node.kind === 159 /* PropertyDeclaration */;
        }
        ts.isPropertyDeclaration = isPropertyDeclaration;
        function isMethodSignature(node) {
            return node.kind === 160 /* MethodSignature */;
        }
        ts.isMethodSignature = isMethodSignature;
        function isMethodDeclaration(node) {
            return node.kind === 161 /* MethodDeclaration */;
        }
        ts.isMethodDeclaration = isMethodDeclaration;
        function isConstructorDeclaration(node) {
            return node.kind === 162 /* Constructor */;
        }
        ts.isConstructorDeclaration = isConstructorDeclaration;
        function isGetAccessorDeclaration(node) {
            return node.kind === 163 /* GetAccessor */;
        }
        ts.isGetAccessorDeclaration = isGetAccessorDeclaration;
        function isSetAccessorDeclaration(node) {
            return node.kind === 164 /* SetAccessor */;
        }
        ts.isSetAccessorDeclaration = isSetAccessorDeclaration;
        function isCallSignatureDeclaration(node) {
            return node.kind === 165 /* CallSignature */;
        }
        ts.isCallSignatureDeclaration = isCallSignatureDeclaration;
        function isConstructSignatureDeclaration(node) {
            return node.kind === 166 /* ConstructSignature */;
        }
        ts.isConstructSignatureDeclaration = isConstructSignatureDeclaration;
        function isIndexSignatureDeclaration(node) {
            return node.kind === 167 /* IndexSignature */;
        }
        ts.isIndexSignatureDeclaration = isIndexSignatureDeclaration;
        /* @internal */
        function isGetOrSetAccessorDeclaration(node) {
            return node.kind === 164 /* SetAccessor */ || node.kind === 163 /* GetAccessor */;
        }
        ts.isGetOrSetAccessorDeclaration = isGetOrSetAccessorDeclaration;
        // Type
        function isTypePredicateNode(node) {
            return node.kind === 168 /* TypePredicate */;
        }
        ts.isTypePredicateNode = isTypePredicateNode;
        function isTypeReferenceNode(node) {
            return node.kind === 169 /* TypeReference */;
        }
        ts.isTypeReferenceNode = isTypeReferenceNode;
        function isFunctionTypeNode(node) {
            return node.kind === 170 /* FunctionType */;
        }
        ts.isFunctionTypeNode = isFunctionTypeNode;
        function isConstructorTypeNode(node) {
            return node.kind === 171 /* ConstructorType */;
        }
        ts.isConstructorTypeNode = isConstructorTypeNode;
        function isTypeQueryNode(node) {
            return node.kind === 172 /* TypeQuery */;
        }
        ts.isTypeQueryNode = isTypeQueryNode;
        function isTypeLiteralNode(node) {
            return node.kind === 173 /* TypeLiteral */;
        }
        ts.isTypeLiteralNode = isTypeLiteralNode;
        function isArrayTypeNode(node) {
            return node.kind === 174 /* ArrayType */;
        }
        ts.isArrayTypeNode = isArrayTypeNode;
        function isTupleTypeNode(node) {
            return node.kind === 175 /* TupleType */;
        }
        ts.isTupleTypeNode = isTupleTypeNode;
        function isUnionTypeNode(node) {
            return node.kind === 178 /* UnionType */;
        }
        ts.isUnionTypeNode = isUnionTypeNode;
        function isIntersectionTypeNode(node) {
            return node.kind === 179 /* IntersectionType */;
        }
        ts.isIntersectionTypeNode = isIntersectionTypeNode;
        function isConditionalTypeNode(node) {
            return node.kind === 180 /* ConditionalType */;
        }
        ts.isConditionalTypeNode = isConditionalTypeNode;
        function isInferTypeNode(node) {
            return node.kind === 181 /* InferType */;
        }
        ts.isInferTypeNode = isInferTypeNode;
        function isParenthesizedTypeNode(node) {
            return node.kind === 182 /* ParenthesizedType */;
        }
        ts.isParenthesizedTypeNode = isParenthesizedTypeNode;
        function isThisTypeNode(node) {
            return node.kind === 183 /* ThisType */;
        }
        ts.isThisTypeNode = isThisTypeNode;
        function isTypeOperatorNode(node) {
            return node.kind === 184 /* TypeOperator */;
        }
        ts.isTypeOperatorNode = isTypeOperatorNode;
        function isIndexedAccessTypeNode(node) {
            return node.kind === 185 /* IndexedAccessType */;
        }
        ts.isIndexedAccessTypeNode = isIndexedAccessTypeNode;
        function isMappedTypeNode(node) {
            return node.kind === 186 /* MappedType */;
        }
        ts.isMappedTypeNode = isMappedTypeNode;
        function isLiteralTypeNode(node) {
            return node.kind === 187 /* LiteralType */;
        }
        ts.isLiteralTypeNode = isLiteralTypeNode;
        function isImportTypeNode(node) {
            return node.kind === 188 /* ImportType */;
        }
        ts.isImportTypeNode = isImportTypeNode;
        // Binding patterns
        function isObjectBindingPattern(node) {
            return node.kind === 189 /* ObjectBindingPattern */;
        }
        ts.isObjectBindingPattern = isObjectBindingPattern;
        function isArrayBindingPattern(node) {
            return node.kind === 190 /* ArrayBindingPattern */;
        }
        ts.isArrayBindingPattern = isArrayBindingPattern;
        function isBindingElement(node) {
            return node.kind === 191 /* BindingElement */;
        }
        ts.isBindingElement = isBindingElement;
        // Expression
        function isArrayLiteralExpression(node) {
            return node.kind === 192 /* ArrayLiteralExpression */;
        }
        ts.isArrayLiteralExpression = isArrayLiteralExpression;
        function isObjectLiteralExpression(node) {
            return node.kind === 193 /* ObjectLiteralExpression */;
        }
        ts.isObjectLiteralExpression = isObjectLiteralExpression;
        function isPropertyAccessExpression(node) {
            return node.kind === 194 /* PropertyAccessExpression */;
        }
        ts.isPropertyAccessExpression = isPropertyAccessExpression;
        function isPropertyAccessChain(node) {
            return isPropertyAccessExpression(node) && !!(node.flags & 32 /* OptionalChain */);
        }
        ts.isPropertyAccessChain = isPropertyAccessChain;
        function isElementAccessExpression(node) {
            return node.kind === 195 /* ElementAccessExpression */;
        }
        ts.isElementAccessExpression = isElementAccessExpression;
        function isElementAccessChain(node) {
            return isElementAccessExpression(node) && !!(node.flags & 32 /* OptionalChain */);
        }
        ts.isElementAccessChain = isElementAccessChain;
        function isCallExpression(node) {
            return node.kind === 196 /* CallExpression */;
        }
        ts.isCallExpression = isCallExpression;
        function isCallChain(node) {
            return isCallExpression(node) && !!(node.flags & 32 /* OptionalChain */);
        }
        ts.isCallChain = isCallChain;
        function isOptionalChain(node) {
            var kind = node.kind;
            return !!(node.flags & 32 /* OptionalChain */) &&
                (kind === 194 /* PropertyAccessExpression */
                    || kind === 195 /* ElementAccessExpression */
                    || kind === 196 /* CallExpression */);
        }
        ts.isOptionalChain = isOptionalChain;
        /* @internal */
        function isOptionalChainRoot(node) {
            return isOptionalChain(node) && !!node.questionDotToken;
        }
        ts.isOptionalChainRoot = isOptionalChainRoot;
        /**
         * Determines whether a node is the expression preceding an optional chain (i.e. `a` in `a?.b`).
         */
        /* @internal */
        function isExpressionOfOptionalChainRoot(node) {
            return isOptionalChainRoot(node.parent) && node.parent.expression === node;
        }
        ts.isExpressionOfOptionalChainRoot = isExpressionOfOptionalChainRoot;
        /**
         * Determines whether a node is the outermost `OptionalChain` in an ECMAScript `OptionalExpression`:
         *
         * 1. For `a?.b.c`, the outermost chain is `a?.b.c` (`c` is the end of the chain starting at `a?.`)
         * 2. For `(a?.b.c).d`, the outermost chain is `a?.b.c` (`c` is the end of the chain starting at `a?.` since parens end the chain)
         * 3. For `a?.b.c?.d`, both `a?.b.c` and `a?.b.c?.d` are outermost (`c` is the end of the chain starting at `a?.`, and `d` is
         *   the end of the chain starting at `c?.`)
         * 4. For `a?.(b?.c).d`, both `b?.c` and `a?.(b?.c)d` are outermost (`c` is the end of the chain starting at `b`, and `d` is
         *   the end of the chain starting at `a?.`)
         */
        /* @internal */
        function isOutermostOptionalChain(node) {
            return !isOptionalChain(node.parent) // cases 1 and 2
                || isOptionalChainRoot(node.parent) // case 3
                || node !== node.parent.expression; // case 4
        }
        ts.isOutermostOptionalChain = isOutermostOptionalChain;
        function isNullishCoalesce(node) {
            return node.kind === 209 /* BinaryExpression */ && node.operatorToken.kind === 60 /* QuestionQuestionToken */;
        }
        ts.isNullishCoalesce = isNullishCoalesce;
        function isNewExpression(node) {
            return node.kind === 197 /* NewExpression */;
        }
        ts.isNewExpression = isNewExpression;
        function isTaggedTemplateExpression(node) {
            return node.kind === 198 /* TaggedTemplateExpression */;
        }
        ts.isTaggedTemplateExpression = isTaggedTemplateExpression;
        function isTypeAssertion(node) {
            return node.kind === 199 /* TypeAssertionExpression */;
        }
        ts.isTypeAssertion = isTypeAssertion;
        function isConstTypeReference(node) {
            return isTypeReferenceNode(node) && isIdentifier(node.typeName) &&
                node.typeName.escapedText === "const" && !node.typeArguments;
        }
        ts.isConstTypeReference = isConstTypeReference;
        function isParenthesizedExpression(node) {
            return node.kind === 200 /* ParenthesizedExpression */;
        }
        ts.isParenthesizedExpression = isParenthesizedExpression;
        function skipPartiallyEmittedExpressions(node) {
            while (node.kind === 325 /* PartiallyEmittedExpression */) {
                node = node.expression;
            }
            return node;
        }
        ts.skipPartiallyEmittedExpressions = skipPartiallyEmittedExpressions;
        function isFunctionExpression(node) {
            return node.kind === 201 /* FunctionExpression */;
        }
        ts.isFunctionExpression = isFunctionExpression;
        function isArrowFunction(node) {
            return node.kind === 202 /* ArrowFunction */;
        }
        ts.isArrowFunction = isArrowFunction;
        function isDeleteExpression(node) {
            return node.kind === 203 /* DeleteExpression */;
        }
        ts.isDeleteExpression = isDeleteExpression;
        function isTypeOfExpression(node) {
            return node.kind === 204 /* TypeOfExpression */;
        }
        ts.isTypeOfExpression = isTypeOfExpression;
        function isVoidExpression(node) {
            return node.kind === 205 /* VoidExpression */;
        }
        ts.isVoidExpression = isVoidExpression;
        function isAwaitExpression(node) {
            return node.kind === 206 /* AwaitExpression */;
        }
        ts.isAwaitExpression = isAwaitExpression;
        function isPrefixUnaryExpression(node) {
            return node.kind === 207 /* PrefixUnaryExpression */;
        }
        ts.isPrefixUnaryExpression = isPrefixUnaryExpression;
        function isPostfixUnaryExpression(node) {
            return node.kind === 208 /* PostfixUnaryExpression */;
        }
        ts.isPostfixUnaryExpression = isPostfixUnaryExpression;
        function isBinaryExpression(node) {
            return node.kind === 209 /* BinaryExpression */;
        }
        ts.isBinaryExpression = isBinaryExpression;
        function isConditionalExpression(node) {
            return node.kind === 210 /* ConditionalExpression */;
        }
        ts.isConditionalExpression = isConditionalExpression;
        function isTemplateExpression(node) {
            return node.kind === 211 /* TemplateExpression */;
        }
        ts.isTemplateExpression = isTemplateExpression;
        function isYieldExpression(node) {
            return node.kind === 212 /* YieldExpression */;
        }
        ts.isYieldExpression = isYieldExpression;
        function isSpreadElement(node) {
            return node.kind === 213 /* SpreadElement */;
        }
        ts.isSpreadElement = isSpreadElement;
        function isClassExpression(node) {
            return node.kind === 214 /* ClassExpression */;
        }
        ts.isClassExpression = isClassExpression;
        function isOmittedExpression(node) {
            return node.kind === 215 /* OmittedExpression */;
        }
        ts.isOmittedExpression = isOmittedExpression;
        function isExpressionWithTypeArguments(node) {
            return node.kind === 216 /* ExpressionWithTypeArguments */;
        }
        ts.isExpressionWithTypeArguments = isExpressionWithTypeArguments;
        function isAsExpression(node) {
            return node.kind === 217 /* AsExpression */;
        }
        ts.isAsExpression = isAsExpression;
        function isNonNullExpression(node) {
            return node.kind === 218 /* NonNullExpression */;
        }
        ts.isNonNullExpression = isNonNullExpression;
        function isMetaProperty(node) {
            return node.kind === 219 /* MetaProperty */;
        }
        ts.isMetaProperty = isMetaProperty;
        // Misc
        function isTemplateSpan(node) {
            return node.kind === 221 /* TemplateSpan */;
        }
        ts.isTemplateSpan = isTemplateSpan;
        function isSemicolonClassElement(node) {
            return node.kind === 222 /* SemicolonClassElement */;
        }
        ts.isSemicolonClassElement = isSemicolonClassElement;
        // Block
        function isBlock(node) {
            return node.kind === 223 /* Block */;
        }
        ts.isBlock = isBlock;
        function isVariableStatement(node) {
            return node.kind === 225 /* VariableStatement */;
        }
        ts.isVariableStatement = isVariableStatement;
        function isEmptyStatement(node) {
            return node.kind === 224 /* EmptyStatement */;
        }
        ts.isEmptyStatement = isEmptyStatement;
        function isExpressionStatement(node) {
            return node.kind === 226 /* ExpressionStatement */;
        }
        ts.isExpressionStatement = isExpressionStatement;
        function isIfStatement(node) {
            return node.kind === 227 /* IfStatement */;
        }
        ts.isIfStatement = isIfStatement;
        function isDoStatement(node) {
            return node.kind === 228 /* DoStatement */;
        }
        ts.isDoStatement = isDoStatement;
        function isWhileStatement(node) {
            return node.kind === 229 /* WhileStatement */;
        }
        ts.isWhileStatement = isWhileStatement;
        function isForStatement(node) {
            return node.kind === 230 /* ForStatement */;
        }
        ts.isForStatement = isForStatement;
        function isForInStatement(node) {
            return node.kind === 231 /* ForInStatement */;
        }
        ts.isForInStatement = isForInStatement;
        function isForOfStatement(node) {
            return node.kind === 232 /* ForOfStatement */;
        }
        ts.isForOfStatement = isForOfStatement;
        function isContinueStatement(node) {
            return node.kind === 233 /* ContinueStatement */;
        }
        ts.isContinueStatement = isContinueStatement;
        function isBreakStatement(node) {
            return node.kind === 234 /* BreakStatement */;
        }
        ts.isBreakStatement = isBreakStatement;
        function isBreakOrContinueStatement(node) {
            return node.kind === 234 /* BreakStatement */ || node.kind === 233 /* ContinueStatement */;
        }
        ts.isBreakOrContinueStatement = isBreakOrContinueStatement;
        function isReturnStatement(node) {
            return node.kind === 235 /* ReturnStatement */;
        }
        ts.isReturnStatement = isReturnStatement;
        function isWithStatement(node) {
            return node.kind === 236 /* WithStatement */;
        }
        ts.isWithStatement = isWithStatement;
        function isSwitchStatement(node) {
            return node.kind === 237 /* SwitchStatement */;
        }
        ts.isSwitchStatement = isSwitchStatement;
        function isLabeledStatement(node) {
            return node.kind === 238 /* LabeledStatement */;
        }
        ts.isLabeledStatement = isLabeledStatement;
        function isThrowStatement(node) {
            return node.kind === 239 /* ThrowStatement */;
        }
        ts.isThrowStatement = isThrowStatement;
        function isTryStatement(node) {
            return node.kind === 240 /* TryStatement */;
        }
        ts.isTryStatement = isTryStatement;
        function isDebuggerStatement(node) {
            return node.kind === 241 /* DebuggerStatement */;
        }
        ts.isDebuggerStatement = isDebuggerStatement;
        function isVariableDeclaration(node) {
            return node.kind === 242 /* VariableDeclaration */;
        }
        ts.isVariableDeclaration = isVariableDeclaration;
        function isVariableDeclarationList(node) {
            return node.kind === 243 /* VariableDeclarationList */;
        }
        ts.isVariableDeclarationList = isVariableDeclarationList;
        function isFunctionDeclaration(node) {
            return node.kind === 244 /* FunctionDeclaration */;
        }
        ts.isFunctionDeclaration = isFunctionDeclaration;
        function isClassDeclaration(node) {
            return node.kind === 245 /* ClassDeclaration */;
        }
        ts.isClassDeclaration = isClassDeclaration;
        function isInterfaceDeclaration(node) {
            return node.kind === 246 /* InterfaceDeclaration */;
        }
        ts.isInterfaceDeclaration = isInterfaceDeclaration;
        function isTypeAliasDeclaration(node) {
            return node.kind === 247 /* TypeAliasDeclaration */;
        }
        ts.isTypeAliasDeclaration = isTypeAliasDeclaration;
        function isEnumDeclaration(node) {
            return node.kind === 248 /* EnumDeclaration */;
        }
        ts.isEnumDeclaration = isEnumDeclaration;
        function isModuleDeclaration(node) {
            return node.kind === 249 /* ModuleDeclaration */;
        }
        ts.isModuleDeclaration = isModuleDeclaration;
        function isModuleBlock(node) {
            return node.kind === 250 /* ModuleBlock */;
        }
        ts.isModuleBlock = isModuleBlock;
        function isCaseBlock(node) {
            return node.kind === 251 /* CaseBlock */;
        }
        ts.isCaseBlock = isCaseBlock;
        function isNamespaceExportDeclaration(node) {
            return node.kind === 252 /* NamespaceExportDeclaration */;
        }
        ts.isNamespaceExportDeclaration = isNamespaceExportDeclaration;
        function isImportEqualsDeclaration(node) {
            return node.kind === 253 /* ImportEqualsDeclaration */;
        }
        ts.isImportEqualsDeclaration = isImportEqualsDeclaration;
        function isImportDeclaration(node) {
            return node.kind === 254 /* ImportDeclaration */;
        }
        ts.isImportDeclaration = isImportDeclaration;
        function isImportClause(node) {
            return node.kind === 255 /* ImportClause */;
        }
        ts.isImportClause = isImportClause;
        function isNamespaceImport(node) {
            return node.kind === 256 /* NamespaceImport */;
        }
        ts.isNamespaceImport = isNamespaceImport;
        function isNamespaceExport(node) {
            return node.kind === 262 /* NamespaceExport */;
        }
        ts.isNamespaceExport = isNamespaceExport;
        function isNamedExportBindings(node) {
            return node.kind === 262 /* NamespaceExport */ || node.kind === 261 /* NamedExports */;
        }
        ts.isNamedExportBindings = isNamedExportBindings;
        function isNamedImports(node) {
            return node.kind === 257 /* NamedImports */;
        }
        ts.isNamedImports = isNamedImports;
        function isImportSpecifier(node) {
            return node.kind === 258 /* ImportSpecifier */;
        }
        ts.isImportSpecifier = isImportSpecifier;
        function isExportAssignment(node) {
            return node.kind === 259 /* ExportAssignment */;
        }
        ts.isExportAssignment = isExportAssignment;
        function isExportDeclaration(node) {
            return node.kind === 260 /* ExportDeclaration */;
        }
        ts.isExportDeclaration = isExportDeclaration;
        function isNamedExports(node) {
            return node.kind === 261 /* NamedExports */;
        }
        ts.isNamedExports = isNamedExports;
        function isExportSpecifier(node) {
            return node.kind === 263 /* ExportSpecifier */;
        }
        ts.isExportSpecifier = isExportSpecifier;
        function isMissingDeclaration(node) {
            return node.kind === 264 /* MissingDeclaration */;
        }
        ts.isMissingDeclaration = isMissingDeclaration;
        // Module References
        function isExternalModuleReference(node) {
            return node.kind === 265 /* ExternalModuleReference */;
        }
        ts.isExternalModuleReference = isExternalModuleReference;
        // JSX
        function isJsxElement(node) {
            return node.kind === 266 /* JsxElement */;
        }
        ts.isJsxElement = isJsxElement;
        function isJsxSelfClosingElement(node) {
            return node.kind === 267 /* JsxSelfClosingElement */;
        }
        ts.isJsxSelfClosingElement = isJsxSelfClosingElement;
        function isJsxOpeningElement(node) {
            return node.kind === 268 /* JsxOpeningElement */;
        }
        ts.isJsxOpeningElement = isJsxOpeningElement;
        function isJsxClosingElement(node) {
            return node.kind === 269 /* JsxClosingElement */;
        }
        ts.isJsxClosingElement = isJsxClosingElement;
        function isJsxFragment(node) {
            return node.kind === 270 /* JsxFragment */;
        }
        ts.isJsxFragment = isJsxFragment;
        function isJsxOpeningFragment(node) {
            return node.kind === 271 /* JsxOpeningFragment */;
        }
        ts.isJsxOpeningFragment = isJsxOpeningFragment;
        function isJsxClosingFragment(node) {
            return node.kind === 272 /* JsxClosingFragment */;
        }
        ts.isJsxClosingFragment = isJsxClosingFragment;
        function isJsxAttribute(node) {
            return node.kind === 273 /* JsxAttribute */;
        }
        ts.isJsxAttribute = isJsxAttribute;
        function isJsxAttributes(node) {
            return node.kind === 274 /* JsxAttributes */;
        }
        ts.isJsxAttributes = isJsxAttributes;
        function isJsxSpreadAttribute(node) {
            return node.kind === 275 /* JsxSpreadAttribute */;
        }
        ts.isJsxSpreadAttribute = isJsxSpreadAttribute;
        function isJsxExpression(node) {
            return node.kind === 276 /* JsxExpression */;
        }
        ts.isJsxExpression = isJsxExpression;
        // Clauses
        function isCaseClause(node) {
            return node.kind === 277 /* CaseClause */;
        }
        ts.isCaseClause = isCaseClause;
        function isDefaultClause(node) {
            return node.kind === 278 /* DefaultClause */;
        }
        ts.isDefaultClause = isDefaultClause;
        function isHeritageClause(node) {
            return node.kind === 279 /* HeritageClause */;
        }
        ts.isHeritageClause = isHeritageClause;
        function isCatchClause(node) {
            return node.kind === 280 /* CatchClause */;
        }
        ts.isCatchClause = isCatchClause;
        // Property assignments
        function isPropertyAssignment(node) {
            return node.kind === 281 /* PropertyAssignment */;
        }
        ts.isPropertyAssignment = isPropertyAssignment;
        function isShorthandPropertyAssignment(node) {
            return node.kind === 282 /* ShorthandPropertyAssignment */;
        }
        ts.isShorthandPropertyAssignment = isShorthandPropertyAssignment;
        function isSpreadAssignment(node) {
            return node.kind === 283 /* SpreadAssignment */;
        }
        ts.isSpreadAssignment = isSpreadAssignment;
        // Enum
        function isEnumMember(node) {
            return node.kind === 284 /* EnumMember */;
        }
        ts.isEnumMember = isEnumMember;
        // Top-level nodes
        function isSourceFile(node) {
            return node.kind === 290 /* SourceFile */;
        }
        ts.isSourceFile = isSourceFile;
        function isBundle(node) {
            return node.kind === 291 /* Bundle */;
        }
        ts.isBundle = isBundle;
        function isUnparsedSource(node) {
            return node.kind === 292 /* UnparsedSource */;
        }
        ts.isUnparsedSource = isUnparsedSource;
        function isUnparsedPrepend(node) {
            return node.kind === 286 /* UnparsedPrepend */;
        }
        ts.isUnparsedPrepend = isUnparsedPrepend;
        function isUnparsedTextLike(node) {
            switch (node.kind) {
                case 287 /* UnparsedText */:
                case 288 /* UnparsedInternalText */:
                    return true;
                default:
                    return false;
            }
        }
        ts.isUnparsedTextLike = isUnparsedTextLike;
        function isUnparsedNode(node) {
            return isUnparsedTextLike(node) ||
                node.kind === 285 /* UnparsedPrologue */ ||
                node.kind === 289 /* UnparsedSyntheticReference */;
        }
        ts.isUnparsedNode = isUnparsedNode;
        // JSDoc
        function isJSDocTypeExpression(node) {
            return node.kind === 294 /* JSDocTypeExpression */;
        }
        ts.isJSDocTypeExpression = isJSDocTypeExpression;
        function isJSDocAllType(node) {
            return node.kind === 295 /* JSDocAllType */;
        }
        ts.isJSDocAllType = isJSDocAllType;
        function isJSDocUnknownType(node) {
            return node.kind === 296 /* JSDocUnknownType */;
        }
        ts.isJSDocUnknownType = isJSDocUnknownType;
        function isJSDocNullableType(node) {
            return node.kind === 297 /* JSDocNullableType */;
        }
        ts.isJSDocNullableType = isJSDocNullableType;
        function isJSDocNonNullableType(node) {
            return node.kind === 298 /* JSDocNonNullableType */;
        }
        ts.isJSDocNonNullableType = isJSDocNonNullableType;
        function isJSDocOptionalType(node) {
            return node.kind === 299 /* JSDocOptionalType */;
        }
        ts.isJSDocOptionalType = isJSDocOptionalType;
        function isJSDocFunctionType(node) {
            return node.kind === 300 /* JSDocFunctionType */;
        }
        ts.isJSDocFunctionType = isJSDocFunctionType;
        function isJSDocVariadicType(node) {
            return node.kind === 301 /* JSDocVariadicType */;
        }
        ts.isJSDocVariadicType = isJSDocVariadicType;
        function isJSDoc(node) {
            return node.kind === 303 /* JSDocComment */;
        }
        ts.isJSDoc = isJSDoc;
        function isJSDocAuthorTag(node) {
            return node.kind === 308 /* JSDocAuthorTag */;
        }
        ts.isJSDocAuthorTag = isJSDocAuthorTag;
        function isJSDocAugmentsTag(node) {
            return node.kind === 307 /* JSDocAugmentsTag */;
        }
        ts.isJSDocAugmentsTag = isJSDocAugmentsTag;
        function isJSDocClassTag(node) {
            return node.kind === 309 /* JSDocClassTag */;
        }
        ts.isJSDocClassTag = isJSDocClassTag;
        function isJSDocPublicTag(node) {
            return node.kind === 310 /* JSDocPublicTag */;
        }
        ts.isJSDocPublicTag = isJSDocPublicTag;
        function isJSDocPrivateTag(node) {
            return node.kind === 311 /* JSDocPrivateTag */;
        }
        ts.isJSDocPrivateTag = isJSDocPrivateTag;
        function isJSDocProtectedTag(node) {
            return node.kind === 312 /* JSDocProtectedTag */;
        }
        ts.isJSDocProtectedTag = isJSDocProtectedTag;
        function isJSDocReadonlyTag(node) {
            return node.kind === 313 /* JSDocReadonlyTag */;
        }
        ts.isJSDocReadonlyTag = isJSDocReadonlyTag;
        function isJSDocEnumTag(node) {
            return node.kind === 315 /* JSDocEnumTag */;
        }
        ts.isJSDocEnumTag = isJSDocEnumTag;
        function isJSDocThisTag(node) {
            return node.kind === 318 /* JSDocThisTag */;
        }
        ts.isJSDocThisTag = isJSDocThisTag;
        function isJSDocParameterTag(node) {
            return node.kind === 316 /* JSDocParameterTag */;
        }
        ts.isJSDocParameterTag = isJSDocParameterTag;
        function isJSDocReturnTag(node) {
            return node.kind === 317 /* JSDocReturnTag */;
        }
        ts.isJSDocReturnTag = isJSDocReturnTag;
        function isJSDocTypeTag(node) {
            return node.kind === 319 /* JSDocTypeTag */;
        }
        ts.isJSDocTypeTag = isJSDocTypeTag;
        function isJSDocTemplateTag(node) {
            return node.kind === 320 /* JSDocTemplateTag */;
        }
        ts.isJSDocTemplateTag = isJSDocTemplateTag;
        function isJSDocTypedefTag(node) {
            return node.kind === 321 /* JSDocTypedefTag */;
        }
        ts.isJSDocTypedefTag = isJSDocTypedefTag;
        function isJSDocPropertyTag(node) {
            return node.kind === 322 /* JSDocPropertyTag */;
        }
        ts.isJSDocPropertyTag = isJSDocPropertyTag;
        function isJSDocPropertyLikeTag(node) {
            return node.kind === 322 /* JSDocPropertyTag */ || node.kind === 316 /* JSDocParameterTag */;
        }
        ts.isJSDocPropertyLikeTag = isJSDocPropertyLikeTag;
        function isJSDocTypeLiteral(node) {
            return node.kind === 304 /* JSDocTypeLiteral */;
        }
        ts.isJSDocTypeLiteral = isJSDocTypeLiteral;
        function isJSDocCallbackTag(node) {
            return node.kind === 314 /* JSDocCallbackTag */;
        }
        ts.isJSDocCallbackTag = isJSDocCallbackTag;
        function isJSDocSignature(node) {
            return node.kind === 305 /* JSDocSignature */;
        }
        ts.isJSDocSignature = isJSDocSignature;
        // #endregion
        // #region
        // Node tests
        //
        // All node tests in the following list should *not* reference parent pointers so that
        // they may be used with transformations.
        /* @internal */
        function isSyntaxList(n) {
            return n.kind === 323 /* SyntaxList */;
        }
        ts.isSyntaxList = isSyntaxList;
        /* @internal */
        function isNode(node) {
            return isNodeKind(node.kind);
        }
        ts.isNode = isNode;
        /* @internal */
        function isNodeKind(kind) {
            return kind >= 153 /* FirstNode */;
        }
        ts.isNodeKind = isNodeKind;
        /**
         * True if node is of some token syntax kind.
         * For example, this is true for an IfKeyword but not for an IfStatement.
         * Literals are considered tokens, except TemplateLiteral, but does include TemplateHead/Middle/Tail.
         */
        function isToken(n) {
            return n.kind >= 0 /* FirstToken */ && n.kind <= 152 /* LastToken */;
        }
        ts.isToken = isToken;
        // Node Arrays
        /* @internal */
        function isNodeArray(array) {
            return array.hasOwnProperty("pos") && array.hasOwnProperty("end");
        }
        ts.isNodeArray = isNodeArray;
        // Literals
        /* @internal */
        function isLiteralKind(kind) {
            return 8 /* FirstLiteralToken */ <= kind && kind <= 14 /* LastLiteralToken */;
        }
        ts.isLiteralKind = isLiteralKind;
        function isLiteralExpression(node) {
            return isLiteralKind(node.kind);
        }
        ts.isLiteralExpression = isLiteralExpression;
        // Pseudo-literals
        /* @internal */
        function isTemplateLiteralKind(kind) {
            return 14 /* FirstTemplateToken */ <= kind && kind <= 17 /* LastTemplateToken */;
        }
        ts.isTemplateLiteralKind = isTemplateLiteralKind;
        function isTemplateLiteralToken(node) {
            return isTemplateLiteralKind(node.kind);
        }
        ts.isTemplateLiteralToken = isTemplateLiteralToken;
        function isTemplateMiddleOrTemplateTail(node) {
            var kind = node.kind;
            return kind === 16 /* TemplateMiddle */
                || kind === 17 /* TemplateTail */;
        }
        ts.isTemplateMiddleOrTemplateTail = isTemplateMiddleOrTemplateTail;
        function isImportOrExportSpecifier(node) {
            return isImportSpecifier(node) || isExportSpecifier(node);
        }
        ts.isImportOrExportSpecifier = isImportOrExportSpecifier;
        function isTypeOnlyImportOrExportDeclaration(node) {
            switch (node.kind) {
                case 258 /* ImportSpecifier */:
                case 263 /* ExportSpecifier */:
                    return node.parent.parent.isTypeOnly;
                case 256 /* NamespaceImport */:
                    return node.parent.isTypeOnly;
                case 255 /* ImportClause */:
                    return node.isTypeOnly;
                default:
                    return false;
            }
        }
        ts.isTypeOnlyImportOrExportDeclaration = isTypeOnlyImportOrExportDeclaration;
        function isStringTextContainingNode(node) {
            return node.kind === 10 /* StringLiteral */ || isTemplateLiteralKind(node.kind);
        }
        ts.isStringTextContainingNode = isStringTextContainingNode;
        // Identifiers
        /* @internal */
        function isGeneratedIdentifier(node) {
            return isIdentifier(node) && (node.autoGenerateFlags & 7 /* KindMask */) > 0 /* None */;
        }
        ts.isGeneratedIdentifier = isGeneratedIdentifier;
        // Private Identifiers
        /*@internal*/
        function isPrivateIdentifierPropertyDeclaration(node) {
            return isPropertyDeclaration(node) && isPrivateIdentifier(node.name);
        }
        ts.isPrivateIdentifierPropertyDeclaration = isPrivateIdentifierPropertyDeclaration;
        /*@internal*/
        function isPrivateIdentifierPropertyAccessExpression(node) {
            return isPropertyAccessExpression(node) && isPrivateIdentifier(node.name);
        }
        ts.isPrivateIdentifierPropertyAccessExpression = isPrivateIdentifierPropertyAccessExpression;
        // Keywords
        /* @internal */
        function isModifierKind(token) {
            switch (token) {
                case 122 /* AbstractKeyword */:
                case 126 /* AsyncKeyword */:
                case 81 /* ConstKeyword */:
                case 130 /* DeclareKeyword */:
                case 84 /* DefaultKeyword */:
                case 89 /* ExportKeyword */:
                case 119 /* PublicKeyword */:
                case 117 /* PrivateKeyword */:
                case 118 /* ProtectedKeyword */:
                case 138 /* ReadonlyKeyword */:
                case 120 /* StaticKeyword */:
                    return true;
            }
            return false;
        }
        ts.isModifierKind = isModifierKind;
        /* @internal */
        function isParameterPropertyModifier(kind) {
            return !!(ts.modifierToFlag(kind) & 92 /* ParameterPropertyModifier */);
        }
        ts.isParameterPropertyModifier = isParameterPropertyModifier;
        /* @internal */
        function isClassMemberModifier(idToken) {
            return isParameterPropertyModifier(idToken) || idToken === 120 /* StaticKeyword */;
        }
        ts.isClassMemberModifier = isClassMemberModifier;
        function isModifier(node) {
            return isModifierKind(node.kind);
        }
        ts.isModifier = isModifier;
        function isEntityName(node) {
            var kind = node.kind;
            return kind === 153 /* QualifiedName */
                || kind === 75 /* Identifier */;
        }
        ts.isEntityName = isEntityName;
        function isPropertyName(node) {
            var kind = node.kind;
            return kind === 75 /* Identifier */
                || kind === 10 /* StringLiteral */
                || kind === 8 /* NumericLiteral */
                || kind === 154 /* ComputedPropertyName */;
        }
        ts.isPropertyName = isPropertyName;
        function isBindingName(node) {
            var kind = node.kind;
            return kind === 75 /* Identifier */
                || kind === 189 /* ObjectBindingPattern */
                || kind === 190 /* ArrayBindingPattern */;
        }
        ts.isBindingName = isBindingName;
        // Functions
        function isFunctionLike(node) {
            return node && isFunctionLikeKind(node.kind);
        }
        ts.isFunctionLike = isFunctionLike;
        /* @internal */
        function isFunctionLikeDeclaration(node) {
            return node && isFunctionLikeDeclarationKind(node.kind);
        }
        ts.isFunctionLikeDeclaration = isFunctionLikeDeclaration;
        function isFunctionLikeDeclarationKind(kind) {
            switch (kind) {
                case 244 /* FunctionDeclaration */:
                case 161 /* MethodDeclaration */:
                case 162 /* Constructor */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 201 /* FunctionExpression */:
                case 202 /* ArrowFunction */:
                    return true;
                default:
                    return false;
            }
        }
        /* @internal */
        function isFunctionLikeKind(kind) {
            switch (kind) {
                case 160 /* MethodSignature */:
                case 165 /* CallSignature */:
                case 305 /* JSDocSignature */:
                case 166 /* ConstructSignature */:
                case 167 /* IndexSignature */:
                case 170 /* FunctionType */:
                case 300 /* JSDocFunctionType */:
                case 171 /* ConstructorType */:
                    return true;
                default:
                    return isFunctionLikeDeclarationKind(kind);
            }
        }
        ts.isFunctionLikeKind = isFunctionLikeKind;
        /* @internal */
        function isFunctionOrModuleBlock(node) {
            return isSourceFile(node) || isModuleBlock(node) || isBlock(node) && isFunctionLike(node.parent);
        }
        ts.isFunctionOrModuleBlock = isFunctionOrModuleBlock;
        // Classes
        function isClassElement(node) {
            var kind = node.kind;
            return kind === 162 /* Constructor */
                || kind === 159 /* PropertyDeclaration */
                || kind === 161 /* MethodDeclaration */
                || kind === 163 /* GetAccessor */
                || kind === 164 /* SetAccessor */
                || kind === 167 /* IndexSignature */
                || kind === 222 /* SemicolonClassElement */;
        }
        ts.isClassElement = isClassElement;
        function isClassLike(node) {
            return node && (node.kind === 245 /* ClassDeclaration */ || node.kind === 214 /* ClassExpression */);
        }
        ts.isClassLike = isClassLike;
        function isAccessor(node) {
            return node && (node.kind === 163 /* GetAccessor */ || node.kind === 164 /* SetAccessor */);
        }
        ts.isAccessor = isAccessor;
        /* @internal */
        function isMethodOrAccessor(node) {
            switch (node.kind) {
                case 161 /* MethodDeclaration */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                    return true;
                default:
                    return false;
            }
        }
        ts.isMethodOrAccessor = isMethodOrAccessor;
        // Type members
        function isTypeElement(node) {
            var kind = node.kind;
            return kind === 166 /* ConstructSignature */
                || kind === 165 /* CallSignature */
                || kind === 158 /* PropertySignature */
                || kind === 160 /* MethodSignature */
                || kind === 167 /* IndexSignature */;
        }
        ts.isTypeElement = isTypeElement;
        function isClassOrTypeElement(node) {
            return isTypeElement(node) || isClassElement(node);
        }
        ts.isClassOrTypeElement = isClassOrTypeElement;
        function isObjectLiteralElementLike(node) {
            var kind = node.kind;
            return kind === 281 /* PropertyAssignment */
                || kind === 282 /* ShorthandPropertyAssignment */
                || kind === 283 /* SpreadAssignment */
                || kind === 161 /* MethodDeclaration */
                || kind === 163 /* GetAccessor */
                || kind === 164 /* SetAccessor */;
        }
        ts.isObjectLiteralElementLike = isObjectLiteralElementLike;
        // Type
        /**
         * Node test that determines whether a node is a valid type node.
         * This differs from the `isPartOfTypeNode` function which determines whether a node is *part*
         * of a TypeNode.
         */
        function isTypeNode(node) {
            return ts.isTypeNodeKind(node.kind);
        }
        ts.isTypeNode = isTypeNode;
        function isFunctionOrConstructorTypeNode(node) {
            switch (node.kind) {
                case 170 /* FunctionType */:
                case 171 /* ConstructorType */:
                    return true;
            }
            return false;
        }
        ts.isFunctionOrConstructorTypeNode = isFunctionOrConstructorTypeNode;
        // Binding patterns
        /* @internal */
        function isBindingPattern(node) {
            if (node) {
                var kind = node.kind;
                return kind === 190 /* ArrayBindingPattern */
                    || kind === 189 /* ObjectBindingPattern */;
            }
            return false;
        }
        ts.isBindingPattern = isBindingPattern;
        /* @internal */
        function isAssignmentPattern(node) {
            var kind = node.kind;
            return kind === 192 /* ArrayLiteralExpression */
                || kind === 193 /* ObjectLiteralExpression */;
        }
        ts.isAssignmentPattern = isAssignmentPattern;
        /* @internal */
        function isArrayBindingElement(node) {
            var kind = node.kind;
            return kind === 191 /* BindingElement */
                || kind === 215 /* OmittedExpression */;
        }
        ts.isArrayBindingElement = isArrayBindingElement;
        /**
         * Determines whether the BindingOrAssignmentElement is a BindingElement-like declaration
         */
        /* @internal */
        function isDeclarationBindingElement(bindingElement) {
            switch (bindingElement.kind) {
                case 242 /* VariableDeclaration */:
                case 156 /* Parameter */:
                case 191 /* BindingElement */:
                    return true;
            }
            return false;
        }
        ts.isDeclarationBindingElement = isDeclarationBindingElement;
        /**
         * Determines whether a node is a BindingOrAssignmentPattern
         */
        /* @internal */
        function isBindingOrAssignmentPattern(node) {
            return isObjectBindingOrAssignmentPattern(node)
                || isArrayBindingOrAssignmentPattern(node);
        }
        ts.isBindingOrAssignmentPattern = isBindingOrAssignmentPattern;
        /**
         * Determines whether a node is an ObjectBindingOrAssignmentPattern
         */
        /* @internal */
        function isObjectBindingOrAssignmentPattern(node) {
            switch (node.kind) {
                case 189 /* ObjectBindingPattern */:
                case 193 /* ObjectLiteralExpression */:
                    return true;
            }
            return false;
        }
        ts.isObjectBindingOrAssignmentPattern = isObjectBindingOrAssignmentPattern;
        /**
         * Determines whether a node is an ArrayBindingOrAssignmentPattern
         */
        /* @internal */
        function isArrayBindingOrAssignmentPattern(node) {
            switch (node.kind) {
                case 190 /* ArrayBindingPattern */:
                case 192 /* ArrayLiteralExpression */:
                    return true;
            }
            return false;
        }
        ts.isArrayBindingOrAssignmentPattern = isArrayBindingOrAssignmentPattern;
        /* @internal */
        function isPropertyAccessOrQualifiedNameOrImportTypeNode(node) {
            var kind = node.kind;
            return kind === 194 /* PropertyAccessExpression */
                || kind === 153 /* QualifiedName */
                || kind === 188 /* ImportType */;
        }
        ts.isPropertyAccessOrQualifiedNameOrImportTypeNode = isPropertyAccessOrQualifiedNameOrImportTypeNode;
        // Expression
        function isPropertyAccessOrQualifiedName(node) {
            var kind = node.kind;
            return kind === 194 /* PropertyAccessExpression */
                || kind === 153 /* QualifiedName */;
        }
        ts.isPropertyAccessOrQualifiedName = isPropertyAccessOrQualifiedName;
        function isCallLikeExpression(node) {
            switch (node.kind) {
                case 268 /* JsxOpeningElement */:
                case 267 /* JsxSelfClosingElement */:
                case 196 /* CallExpression */:
                case 197 /* NewExpression */:
                case 198 /* TaggedTemplateExpression */:
                case 157 /* Decorator */:
                    return true;
                default:
                    return false;
            }
        }
        ts.isCallLikeExpression = isCallLikeExpression;
        function isCallOrNewExpression(node) {
            return node.kind === 196 /* CallExpression */ || node.kind === 197 /* NewExpression */;
        }
        ts.isCallOrNewExpression = isCallOrNewExpression;
        function isTemplateLiteral(node) {
            var kind = node.kind;
            return kind === 211 /* TemplateExpression */
                || kind === 14 /* NoSubstitutionTemplateLiteral */;
        }
        ts.isTemplateLiteral = isTemplateLiteral;
        /* @internal */
        function isLeftHandSideExpression(node) {
            return isLeftHandSideExpressionKind(skipPartiallyEmittedExpressions(node).kind);
        }
        ts.isLeftHandSideExpression = isLeftHandSideExpression;
        function isLeftHandSideExpressionKind(kind) {
            switch (kind) {
                case 194 /* PropertyAccessExpression */:
                case 195 /* ElementAccessExpression */:
                case 197 /* NewExpression */:
                case 196 /* CallExpression */:
                case 266 /* JsxElement */:
                case 267 /* JsxSelfClosingElement */:
                case 270 /* JsxFragment */:
                case 198 /* TaggedTemplateExpression */:
                case 192 /* ArrayLiteralExpression */:
                case 200 /* ParenthesizedExpression */:
                case 193 /* ObjectLiteralExpression */:
                case 214 /* ClassExpression */:
                case 201 /* FunctionExpression */:
                case 75 /* Identifier */:
                case 13 /* RegularExpressionLiteral */:
                case 8 /* NumericLiteral */:
                case 9 /* BigIntLiteral */:
                case 10 /* StringLiteral */:
                case 14 /* NoSubstitutionTemplateLiteral */:
                case 211 /* TemplateExpression */:
                case 91 /* FalseKeyword */:
                case 100 /* NullKeyword */:
                case 104 /* ThisKeyword */:
                case 106 /* TrueKeyword */:
                case 102 /* SuperKeyword */:
                case 218 /* NonNullExpression */:
                case 219 /* MetaProperty */:
                case 96 /* ImportKeyword */: // technically this is only an Expression if it's in a CallExpression
                    return true;
                default:
                    return false;
            }
        }
        /* @internal */
        function isUnaryExpression(node) {
            return isUnaryExpressionKind(skipPartiallyEmittedExpressions(node).kind);
        }
        ts.isUnaryExpression = isUnaryExpression;
        function isUnaryExpressionKind(kind) {
            switch (kind) {
                case 207 /* PrefixUnaryExpression */:
                case 208 /* PostfixUnaryExpression */:
                case 203 /* DeleteExpression */:
                case 204 /* TypeOfExpression */:
                case 205 /* VoidExpression */:
                case 206 /* AwaitExpression */:
                case 199 /* TypeAssertionExpression */:
                    return true;
                default:
                    return isLeftHandSideExpressionKind(kind);
            }
        }
        /* @internal */
        function isUnaryExpressionWithWrite(expr) {
            switch (expr.kind) {
                case 208 /* PostfixUnaryExpression */:
                    return true;
                case 207 /* PrefixUnaryExpression */:
                    return expr.operator === 45 /* PlusPlusToken */ ||
                        expr.operator === 46 /* MinusMinusToken */;
                default:
                    return false;
            }
        }
        ts.isUnaryExpressionWithWrite = isUnaryExpressionWithWrite;
        /* @internal */
        /**
         * Determines whether a node is an expression based only on its kind.
         * Use `isExpressionNode` if not in transforms.
         */
        function isExpression(node) {
            return isExpressionKind(skipPartiallyEmittedExpressions(node).kind);
        }
        ts.isExpression = isExpression;
        function isExpressionKind(kind) {
            switch (kind) {
                case 210 /* ConditionalExpression */:
                case 212 /* YieldExpression */:
                case 202 /* ArrowFunction */:
                case 209 /* BinaryExpression */:
                case 213 /* SpreadElement */:
                case 217 /* AsExpression */:
                case 215 /* OmittedExpression */:
                case 326 /* CommaListExpression */:
                case 325 /* PartiallyEmittedExpression */:
                    return true;
                default:
                    return isUnaryExpressionKind(kind);
            }
        }
        function isAssertionExpression(node) {
            var kind = node.kind;
            return kind === 199 /* TypeAssertionExpression */
                || kind === 217 /* AsExpression */;
        }
        ts.isAssertionExpression = isAssertionExpression;
        /* @internal */
        function isPartiallyEmittedExpression(node) {
            return node.kind === 325 /* PartiallyEmittedExpression */;
        }
        ts.isPartiallyEmittedExpression = isPartiallyEmittedExpression;
        /* @internal */
        function isNotEmittedStatement(node) {
            return node.kind === 324 /* NotEmittedStatement */;
        }
        ts.isNotEmittedStatement = isNotEmittedStatement;
        /* @internal */
        function isSyntheticReference(node) {
            return node.kind === 329 /* SyntheticReferenceExpression */;
        }
        ts.isSyntheticReference = isSyntheticReference;
        /* @internal */
        function isNotEmittedOrPartiallyEmittedNode(node) {
            return isNotEmittedStatement(node)
                || isPartiallyEmittedExpression(node);
        }
        ts.isNotEmittedOrPartiallyEmittedNode = isNotEmittedOrPartiallyEmittedNode;
        function isIterationStatement(node, lookInLabeledStatements) {
            switch (node.kind) {
                case 230 /* ForStatement */:
                case 231 /* ForInStatement */:
                case 232 /* ForOfStatement */:
                case 228 /* DoStatement */:
                case 229 /* WhileStatement */:
                    return true;
                case 238 /* LabeledStatement */:
                    return lookInLabeledStatements && isIterationStatement(node.statement, lookInLabeledStatements);
            }
            return false;
        }
        ts.isIterationStatement = isIterationStatement;
        /* @internal */
        function isScopeMarker(node) {
            return isExportAssignment(node) || isExportDeclaration(node);
        }
        ts.isScopeMarker = isScopeMarker;
        /* @internal */
        function hasScopeMarker(statements) {
            return ts.some(statements, isScopeMarker);
        }
        ts.hasScopeMarker = hasScopeMarker;
        /* @internal */
        function needsScopeMarker(result) {
            return !ts.isAnyImportOrReExport(result) && !isExportAssignment(result) && !ts.hasModifier(result, 1 /* Export */) && !ts.isAmbientModule(result);
        }
        ts.needsScopeMarker = needsScopeMarker;
        /* @internal */
        function isExternalModuleIndicator(result) {
            // Exported top-level member indicates moduleness
            return ts.isAnyImportOrReExport(result) || isExportAssignment(result) || ts.hasModifier(result, 1 /* Export */);
        }
        ts.isExternalModuleIndicator = isExternalModuleIndicator;
        /* @internal */
        function isForInOrOfStatement(node) {
            return node.kind === 231 /* ForInStatement */ || node.kind === 232 /* ForOfStatement */;
        }
        ts.isForInOrOfStatement = isForInOrOfStatement;
        // Element
        /* @internal */
        function isConciseBody(node) {
            return isBlock(node)
                || isExpression(node);
        }
        ts.isConciseBody = isConciseBody;
        /* @internal */
        function isFunctionBody(node) {
            return isBlock(node);
        }
        ts.isFunctionBody = isFunctionBody;
        /* @internal */
        function isForInitializer(node) {
            return isVariableDeclarationList(node)
                || isExpression(node);
        }
        ts.isForInitializer = isForInitializer;
        /* @internal */
        function isModuleBody(node) {
            var kind = node.kind;
            return kind === 250 /* ModuleBlock */
                || kind === 249 /* ModuleDeclaration */
                || kind === 75 /* Identifier */;
        }
        ts.isModuleBody = isModuleBody;
        /* @internal */
        function isNamespaceBody(node) {
            var kind = node.kind;
            return kind === 250 /* ModuleBlock */
                || kind === 249 /* ModuleDeclaration */;
        }
        ts.isNamespaceBody = isNamespaceBody;
        /* @internal */
        function isJSDocNamespaceBody(node) {
            var kind = node.kind;
            return kind === 75 /* Identifier */
                || kind === 249 /* ModuleDeclaration */;
        }
        ts.isJSDocNamespaceBody = isJSDocNamespaceBody;
        /* @internal */
        function isNamedImportBindings(node) {
            var kind = node.kind;
            return kind === 257 /* NamedImports */
                || kind === 256 /* NamespaceImport */;
        }
        ts.isNamedImportBindings = isNamedImportBindings;
        /* @internal */
        function isModuleOrEnumDeclaration(node) {
            return node.kind === 249 /* ModuleDeclaration */ || node.kind === 248 /* EnumDeclaration */;
        }
        ts.isModuleOrEnumDeclaration = isModuleOrEnumDeclaration;
        function isDeclarationKind(kind) {
            return kind === 202 /* ArrowFunction */
                || kind === 191 /* BindingElement */
                || kind === 245 /* ClassDeclaration */
                || kind === 214 /* ClassExpression */
                || kind === 162 /* Constructor */
                || kind === 248 /* EnumDeclaration */
                || kind === 284 /* EnumMember */
                || kind === 263 /* ExportSpecifier */
                || kind === 244 /* FunctionDeclaration */
                || kind === 201 /* FunctionExpression */
                || kind === 163 /* GetAccessor */
                || kind === 255 /* ImportClause */
                || kind === 253 /* ImportEqualsDeclaration */
                || kind === 258 /* ImportSpecifier */
                || kind === 246 /* InterfaceDeclaration */
                || kind === 273 /* JsxAttribute */
                || kind === 161 /* MethodDeclaration */
                || kind === 160 /* MethodSignature */
                || kind === 249 /* ModuleDeclaration */
                || kind === 252 /* NamespaceExportDeclaration */
                || kind === 256 /* NamespaceImport */
                || kind === 262 /* NamespaceExport */
                || kind === 156 /* Parameter */
                || kind === 281 /* PropertyAssignment */
                || kind === 159 /* PropertyDeclaration */
                || kind === 158 /* PropertySignature */
                || kind === 164 /* SetAccessor */
                || kind === 282 /* ShorthandPropertyAssignment */
                || kind === 247 /* TypeAliasDeclaration */
                || kind === 155 /* TypeParameter */
                || kind === 242 /* VariableDeclaration */
                || kind === 321 /* JSDocTypedefTag */
                || kind === 314 /* JSDocCallbackTag */
                || kind === 322 /* JSDocPropertyTag */;
        }
        function isDeclarationStatementKind(kind) {
            return kind === 244 /* FunctionDeclaration */
                || kind === 264 /* MissingDeclaration */
                || kind === 245 /* ClassDeclaration */
                || kind === 246 /* InterfaceDeclaration */
                || kind === 247 /* TypeAliasDeclaration */
                || kind === 248 /* EnumDeclaration */
                || kind === 249 /* ModuleDeclaration */
                || kind === 254 /* ImportDeclaration */
                || kind === 253 /* ImportEqualsDeclaration */
                || kind === 260 /* ExportDeclaration */
                || kind === 259 /* ExportAssignment */
                || kind === 252 /* NamespaceExportDeclaration */;
        }
        function isStatementKindButNotDeclarationKind(kind) {
            return kind === 234 /* BreakStatement */
                || kind === 233 /* ContinueStatement */
                || kind === 241 /* DebuggerStatement */
                || kind === 228 /* DoStatement */
                || kind === 226 /* ExpressionStatement */
                || kind === 224 /* EmptyStatement */
                || kind === 231 /* ForInStatement */
                || kind === 232 /* ForOfStatement */
                || kind === 230 /* ForStatement */
                || kind === 227 /* IfStatement */
                || kind === 238 /* LabeledStatement */
                || kind === 235 /* ReturnStatement */
                || kind === 237 /* SwitchStatement */
                || kind === 239 /* ThrowStatement */
                || kind === 240 /* TryStatement */
                || kind === 225 /* VariableStatement */
                || kind === 229 /* WhileStatement */
                || kind === 236 /* WithStatement */
                || kind === 324 /* NotEmittedStatement */
                || kind === 328 /* EndOfDeclarationMarker */
                || kind === 327 /* MergeDeclarationMarker */;
        }
        /* @internal */
        function isDeclaration(node) {
            if (node.kind === 155 /* TypeParameter */) {
                return (node.parent && node.parent.kind !== 320 /* JSDocTemplateTag */) || ts.isInJSFile(node);
            }
            return isDeclarationKind(node.kind);
        }
        ts.isDeclaration = isDeclaration;
        /* @internal */
        function isDeclarationStatement(node) {
            return isDeclarationStatementKind(node.kind);
        }
        ts.isDeclarationStatement = isDeclarationStatement;
        /**
         * Determines whether the node is a statement that is not also a declaration
         */
        /* @internal */
        function isStatementButNotDeclaration(node) {
            return isStatementKindButNotDeclarationKind(node.kind);
        }
        ts.isStatementButNotDeclaration = isStatementButNotDeclaration;
        /* @internal */
        function isStatement(node) {
            var kind = node.kind;
            return isStatementKindButNotDeclarationKind(kind)
                || isDeclarationStatementKind(kind)
                || isBlockStatement(node);
        }
        ts.isStatement = isStatement;
        function isBlockStatement(node) {
            if (node.kind !== 223 /* Block */)
                return false;
            if (node.parent !== undefined) {
                if (node.parent.kind === 240 /* TryStatement */ || node.parent.kind === 280 /* CatchClause */) {
                    return false;
                }
            }
            return !ts.isFunctionBlock(node);
        }
        // Module references
        /* @internal */
        function isModuleReference(node) {
            var kind = node.kind;
            return kind === 265 /* ExternalModuleReference */
                || kind === 153 /* QualifiedName */
                || kind === 75 /* Identifier */;
        }
        ts.isModuleReference = isModuleReference;
        // JSX
        /* @internal */
        function isJsxTagNameExpression(node) {
            var kind = node.kind;
            return kind === 104 /* ThisKeyword */
                || kind === 75 /* Identifier */
                || kind === 194 /* PropertyAccessExpression */;
        }
        ts.isJsxTagNameExpression = isJsxTagNameExpression;
        /* @internal */
        function isJsxChild(node) {
            var kind = node.kind;
            return kind === 266 /* JsxElement */
                || kind === 276 /* JsxExpression */
                || kind === 267 /* JsxSelfClosingElement */
                || kind === 11 /* JsxText */
                || kind === 270 /* JsxFragment */;
        }
        ts.isJsxChild = isJsxChild;
        /* @internal */
        function isJsxAttributeLike(node) {
            var kind = node.kind;
            return kind === 273 /* JsxAttribute */
                || kind === 275 /* JsxSpreadAttribute */;
        }
        ts.isJsxAttributeLike = isJsxAttributeLike;
        /* @internal */
        function isStringLiteralOrJsxExpression(node) {
            var kind = node.kind;
            return kind === 10 /* StringLiteral */
                || kind === 276 /* JsxExpression */;
        }
        ts.isStringLiteralOrJsxExpression = isStringLiteralOrJsxExpression;
        function isJsxOpeningLikeElement(node) {
            var kind = node.kind;
            return kind === 268 /* JsxOpeningElement */
                || kind === 267 /* JsxSelfClosingElement */;
        }
        ts.isJsxOpeningLikeElement = isJsxOpeningLikeElement;
        // Clauses
        function isCaseOrDefaultClause(node) {
            var kind = node.kind;
            return kind === 277 /* CaseClause */
                || kind === 278 /* DefaultClause */;
        }
        ts.isCaseOrDefaultClause = isCaseOrDefaultClause;
        // JSDoc
        /** True if node is of some JSDoc syntax kind. */
        /* @internal */
        function isJSDocNode(node) {
            return node.kind >= 294 /* FirstJSDocNode */ && node.kind <= 322 /* LastJSDocNode */;
        }
        ts.isJSDocNode = isJSDocNode;
        /** True if node is of a kind that may contain comment text. */
        function isJSDocCommentContainingNode(node) {
            return node.kind === 303 /* JSDocComment */ || isJSDocTag(node) || isJSDocTypeLiteral(node) || isJSDocSignature(node);
        }
        ts.isJSDocCommentContainingNode = isJSDocCommentContainingNode;
        // TODO: determine what this does before making it public.
        /* @internal */
        function isJSDocTag(node) {
            return node.kind >= 306 /* FirstJSDocTagNode */ && node.kind <= 322 /* LastJSDocTagNode */;
        }
        ts.isJSDocTag = isJSDocTag;
        function isSetAccessor(node) {
            return node.kind === 164 /* SetAccessor */;
        }
        ts.isSetAccessor = isSetAccessor;
        function isGetAccessor(node) {
            return node.kind === 163 /* GetAccessor */;
        }
        ts.isGetAccessor = isGetAccessor;
        /** True if has jsdoc nodes attached to it. */
        /* @internal */
        // TODO: GH#19856 Would like to return `node is Node & { jsDoc: JSDoc[] }` but it causes long compile times
        function hasJSDocNodes(node) {
            var jsDoc = node.jsDoc;
            return !!jsDoc && jsDoc.length > 0;
        }
        ts.hasJSDocNodes = hasJSDocNodes;
        /** True if has type node attached to it. */
        /* @internal */
        function hasType(node) {
            return !!node.type;
        }
        ts.hasType = hasType;
        /** True if has initializer node attached to it. */
        /* @internal */
        function hasInitializer(node) {
            return !!node.initializer;
        }
        ts.hasInitializer = hasInitializer;
        /** True if has initializer node attached to it. */
        /* @internal */
        function hasOnlyExpressionInitializer(node) {
            return hasInitializer(node) && !isForStatement(node) && !isForInStatement(node) && !isForOfStatement(node) && !isJsxAttribute(node);
        }
        ts.hasOnlyExpressionInitializer = hasOnlyExpressionInitializer;
        function isObjectLiteralElement(node) {
            return node.kind === 273 /* JsxAttribute */ || node.kind === 275 /* JsxSpreadAttribute */ || isObjectLiteralElementLike(node);
        }
        ts.isObjectLiteralElement = isObjectLiteralElement;
        /* @internal */
        function isTypeReferenceType(node) {
            return node.kind === 169 /* TypeReference */ || node.kind === 216 /* ExpressionWithTypeArguments */;
        }
        ts.isTypeReferenceType = isTypeReferenceType;
        var MAX_SMI_X86 = 1073741823;
        /* @internal */
        function guessIndentation(lines) {
            var indentation = MAX_SMI_X86;
            for (var _i = 0, lines_1 = lines; _i < lines_1.length; _i++) {
                var line = lines_1[_i];
                if (!line.length) {
                    continue;
                }
                var i = 0;
                for (; i < line.length && i < indentation; i++) {
                    if (!ts.isWhiteSpaceLike(line.charCodeAt(i))) {
                        break;
                    }
                }
                if (i < indentation) {
                    indentation = i;
                }
                if (indentation === 0) {
                    return 0;
                }
            }
            return indentation === MAX_SMI_X86 ? undefined : indentation;
        }
        ts.guessIndentation = guessIndentation;
        function isStringLiteralLike(node) {
            return node.kind === 10 /* StringLiteral */ || node.kind === 14 /* NoSubstitutionTemplateLiteral */;
        }
        ts.isStringLiteralLike = isStringLiteralLike;
        // #endregion
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        ts.resolvingEmptyArray = [];
        ts.emptyMap = ts.createMap();
        ts.emptyUnderscoreEscapedMap = ts.emptyMap;
        ts.externalHelpersModuleNameText = "tslib";
        ts.defaultMaximumTruncationLength = 160;
        function getDeclarationOfKind(symbol, kind) {
            var declarations = symbol.declarations;
            if (declarations) {
                for (var _i = 0, declarations_1 = declarations; _i < declarations_1.length; _i++) {
                    var declaration = declarations_1[_i];
                    if (declaration.kind === kind) {
                        return declaration;
                    }
                }
            }
            return undefined;
        }
        ts.getDeclarationOfKind = getDeclarationOfKind;
        /** Create a new escaped identifier map. */
        function createUnderscoreEscapedMap() {
            return new ts.Map();
        }
        ts.createUnderscoreEscapedMap = createUnderscoreEscapedMap;
        function hasEntries(map) {
            return !!map && !!map.size;
        }
        ts.hasEntries = hasEntries;
        function createSymbolTable(symbols) {
            var result = ts.createMap();
            if (symbols) {
                for (var _i = 0, symbols_1 = symbols; _i < symbols_1.length; _i++) {
                    var symbol = symbols_1[_i];
                    result.set(symbol.escapedName, symbol);
                }
            }
            return result;
        }
        ts.createSymbolTable = createSymbolTable;
        function isTransientSymbol(symbol) {
            return (symbol.flags & 33554432 /* Transient */) !== 0;
        }
        ts.isTransientSymbol = isTransientSymbol;
        var stringWriter = createSingleLineStringWriter();
        function createSingleLineStringWriter() {
            var str = "";
            var writeText = function (text) { return str += text; };
            return {
                getText: function () { return str; },
                write: writeText,
                rawWrite: writeText,
                writeKeyword: writeText,
                writeOperator: writeText,
                writePunctuation: writeText,
                writeSpace: writeText,
                writeStringLiteral: writeText,
                writeLiteral: writeText,
                writeParameter: writeText,
                writeProperty: writeText,
                writeSymbol: function (s, _) { return writeText(s); },
                writeTrailingSemicolon: writeText,
                writeComment: writeText,
                getTextPos: function () { return str.length; },
                getLine: function () { return 0; },
                getColumn: function () { return 0; },
                getIndent: function () { return 0; },
                isAtStartOfLine: function () { return false; },
                hasTrailingComment: function () { return false; },
                hasTrailingWhitespace: function () { return !!str.length && ts.isWhiteSpaceLike(str.charCodeAt(str.length - 1)); },
                // Completely ignore indentation for string writers.  And map newlines to
                // a single space.
                writeLine: function () { return str += " "; },
                increaseIndent: ts.noop,
                decreaseIndent: ts.noop,
                clear: function () { return str = ""; },
                trackSymbol: ts.noop,
                reportInaccessibleThisError: ts.noop,
                reportInaccessibleUniqueSymbolError: ts.noop,
                reportPrivateInBaseOfClassExpression: ts.noop,
            };
        }
        function changesAffectModuleResolution(oldOptions, newOptions) {
            return oldOptions.configFilePath !== newOptions.configFilePath ||
                optionsHaveModuleResolutionChanges(oldOptions, newOptions);
        }
        ts.changesAffectModuleResolution = changesAffectModuleResolution;
        function optionsHaveModuleResolutionChanges(oldOptions, newOptions) {
            return ts.moduleResolutionOptionDeclarations.some(function (o) {
                return !isJsonEqual(getCompilerOptionValue(oldOptions, o), getCompilerOptionValue(newOptions, o));
            });
        }
        ts.optionsHaveModuleResolutionChanges = optionsHaveModuleResolutionChanges;
        function findAncestor(node, callback) {
            while (node) {
                var result = callback(node);
                if (result === "quit") {
                    return undefined;
                }
                else if (result) {
                    return node;
                }
                node = node.parent;
            }
            return undefined;
        }
        ts.findAncestor = findAncestor;
        function forEachAncestor(node, callback) {
            while (true) {
                var res = callback(node);
                if (res === "quit")
                    return undefined;
                if (res !== undefined)
                    return res;
                if (ts.isSourceFile(node))
                    return undefined;
                node = node.parent;
            }
        }
        ts.forEachAncestor = forEachAncestor;
        function forEachEntry(map, callback) {
            var iterator = map.entries();
            for (var iterResult = iterator.next(); !iterResult.done; iterResult = iterator.next()) {
                var _a = iterResult.value, key = _a[0], value = _a[1];
                var result = callback(value, key);
                if (result) {
                    return result;
                }
            }
            return undefined;
        }
        ts.forEachEntry = forEachEntry;
        function forEachKey(map, callback) {
            var iterator = map.keys();
            for (var iterResult = iterator.next(); !iterResult.done; iterResult = iterator.next()) {
                var result = callback(iterResult.value);
                if (result) {
                    return result;
                }
            }
            return undefined;
        }
        ts.forEachKey = forEachKey;
        function copyEntries(source, target) {
            source.forEach(function (value, key) {
                target.set(key, value);
            });
        }
        ts.copyEntries = copyEntries;
        function arrayToSet(array, makeKey) {
            return ts.arrayToMap(array, makeKey || (function (s) { return s; }), ts.returnTrue);
        }
        ts.arrayToSet = arrayToSet;
        function cloneMap(map) {
            var clone = ts.createMap();
            copyEntries(map, clone);
            return clone;
        }
        ts.cloneMap = cloneMap;
        function usingSingleLineStringWriter(action) {
            var oldString = stringWriter.getText();
            try {
                action(stringWriter);
                return stringWriter.getText();
            }
            finally {
                stringWriter.clear();
                stringWriter.writeKeyword(oldString);
            }
        }
        ts.usingSingleLineStringWriter = usingSingleLineStringWriter;
        function getFullWidth(node) {
            return node.end - node.pos;
        }
        ts.getFullWidth = getFullWidth;
        function getResolvedModule(sourceFile, moduleNameText) {
            return sourceFile && sourceFile.resolvedModules && sourceFile.resolvedModules.get(moduleNameText);
        }
        ts.getResolvedModule = getResolvedModule;
        function setResolvedModule(sourceFile, moduleNameText, resolvedModule) {
            if (!sourceFile.resolvedModules) {
                sourceFile.resolvedModules = ts.createMap();
            }
            sourceFile.resolvedModules.set(moduleNameText, resolvedModule);
        }
        ts.setResolvedModule = setResolvedModule;
        function setResolvedTypeReferenceDirective(sourceFile, typeReferenceDirectiveName, resolvedTypeReferenceDirective) {
            if (!sourceFile.resolvedTypeReferenceDirectiveNames) {
                sourceFile.resolvedTypeReferenceDirectiveNames = ts.createMap();
            }
            sourceFile.resolvedTypeReferenceDirectiveNames.set(typeReferenceDirectiveName, resolvedTypeReferenceDirective);
        }
        ts.setResolvedTypeReferenceDirective = setResolvedTypeReferenceDirective;
        function projectReferenceIsEqualTo(oldRef, newRef) {
            return oldRef.path === newRef.path &&
                !oldRef.prepend === !newRef.prepend &&
                !oldRef.circular === !newRef.circular;
        }
        ts.projectReferenceIsEqualTo = projectReferenceIsEqualTo;
        function moduleResolutionIsEqualTo(oldResolution, newResolution) {
            return oldResolution.isExternalLibraryImport === newResolution.isExternalLibraryImport &&
                oldResolution.extension === newResolution.extension &&
                oldResolution.resolvedFileName === newResolution.resolvedFileName &&
                oldResolution.originalPath === newResolution.originalPath &&
                packageIdIsEqual(oldResolution.packageId, newResolution.packageId);
        }
        ts.moduleResolutionIsEqualTo = moduleResolutionIsEqualTo;
        function packageIdIsEqual(a, b) {
            return a === b || !!a && !!b && a.name === b.name && a.subModuleName === b.subModuleName && a.version === b.version;
        }
        function packageIdToString(_a) {
            var name = _a.name, subModuleName = _a.subModuleName, version = _a.version;
            var fullName = subModuleName ? name + "/" + subModuleName : name;
            return fullName + "@" + version;
        }
        ts.packageIdToString = packageIdToString;
        function typeDirectiveIsEqualTo(oldResolution, newResolution) {
            return oldResolution.resolvedFileName === newResolution.resolvedFileName && oldResolution.primary === newResolution.primary;
        }
        ts.typeDirectiveIsEqualTo = typeDirectiveIsEqualTo;
        function hasChangesInResolutions(names, newResolutions, oldResolutions, comparer) {
            ts.Debug.assert(names.length === newResolutions.length);
            for (var i = 0; i < names.length; i++) {
                var newResolution = newResolutions[i];
                var oldResolution = oldResolutions && oldResolutions.get(names[i]);
                var changed = oldResolution
                    ? !newResolution || !comparer(oldResolution, newResolution)
                    : newResolution;
                if (changed) {
                    return true;
                }
            }
            return false;
        }
        ts.hasChangesInResolutions = hasChangesInResolutions;
        // Returns true if this node contains a parse error anywhere underneath it.
        function containsParseError(node) {
            aggregateChildData(node);
            return (node.flags & 262144 /* ThisNodeOrAnySubNodesHasError */) !== 0;
        }
        ts.containsParseError = containsParseError;
        function aggregateChildData(node) {
            if (!(node.flags & 524288 /* HasAggregatedChildData */)) {
                // A node is considered to contain a parse error if:
                //  a) the parser explicitly marked that it had an error
                //  b) any of it's children reported that it had an error.
                var thisNodeOrAnySubNodesHasError = ((node.flags & 65536 /* ThisNodeHasError */) !== 0) ||
                    ts.forEachChild(node, containsParseError);
                // If so, mark ourselves accordingly.
                if (thisNodeOrAnySubNodesHasError) {
                    node.flags |= 262144 /* ThisNodeOrAnySubNodesHasError */;
                }
                // Also mark that we've propagated the child information to this node.  This way we can
                // always consult the bit directly on this node without needing to check its children
                // again.
                node.flags |= 524288 /* HasAggregatedChildData */;
            }
        }
        function getSourceFileOfNode(node) {
            while (node && node.kind !== 290 /* SourceFile */) {
                node = node.parent;
            }
            return node;
        }
        ts.getSourceFileOfNode = getSourceFileOfNode;
        function isStatementWithLocals(node) {
            switch (node.kind) {
                case 223 /* Block */:
                case 251 /* CaseBlock */:
                case 230 /* ForStatement */:
                case 231 /* ForInStatement */:
                case 232 /* ForOfStatement */:
                    return true;
            }
            return false;
        }
        ts.isStatementWithLocals = isStatementWithLocals;
        function getStartPositionOfLine(line, sourceFile) {
            ts.Debug.assert(line >= 0);
            return ts.getLineStarts(sourceFile)[line];
        }
        ts.getStartPositionOfLine = getStartPositionOfLine;
        // This is a useful function for debugging purposes.
        function nodePosToString(node) {
            var file = getSourceFileOfNode(node);
            var loc = ts.getLineAndCharacterOfPosition(file, node.pos);
            return file.fileName + "(" + (loc.line + 1) + "," + (loc.character + 1) + ")";
        }
        ts.nodePosToString = nodePosToString;
        function getEndLinePosition(line, sourceFile) {
            ts.Debug.assert(line >= 0);
            var lineStarts = ts.getLineStarts(sourceFile);
            var lineIndex = line;
            var sourceText = sourceFile.text;
            if (lineIndex + 1 === lineStarts.length) {
                // last line - return EOF
                return sourceText.length - 1;
            }
            else {
                // current line start
                var start = lineStarts[lineIndex];
                // take the start position of the next line - 1 = it should be some line break
                var pos = lineStarts[lineIndex + 1] - 1;
                ts.Debug.assert(ts.isLineBreak(sourceText.charCodeAt(pos)));
                // walk backwards skipping line breaks, stop the the beginning of current line.
                // i.e:
                // <some text>
                // $ <- end of line for this position should match the start position
                while (start <= pos && ts.isLineBreak(sourceText.charCodeAt(pos))) {
                    pos--;
                }
                return pos;
            }
        }
        ts.getEndLinePosition = getEndLinePosition;
        /**
         * Returns a value indicating whether a name is unique globally or within the current file.
         * Note: This does not consider whether a name appears as a free identifier or not, so at the expression `x.y` this includes both `x` and `y`.
         */
        function isFileLevelUniqueName(sourceFile, name, hasGlobalName) {
            return !(hasGlobalName && hasGlobalName(name)) && !sourceFile.identifiers.has(name);
        }
        ts.isFileLevelUniqueName = isFileLevelUniqueName;
        // Returns true if this node is missing from the actual source code. A 'missing' node is different
        // from 'undefined/defined'. When a node is undefined (which can happen for optional nodes
        // in the tree), it is definitely missing. However, a node may be defined, but still be
        // missing.  This happens whenever the parser knows it needs to parse something, but can't
        // get anything in the source code that it expects at that location. For example:
        //
        //          let a: ;
        //
        // Here, the Type in the Type-Annotation is not-optional (as there is a colon in the source
        // code). So the parser will attempt to parse out a type, and will create an actual node.
        // However, this node will be 'missing' in the sense that no actual source-code/tokens are
        // contained within it.
        function nodeIsMissing(node) {
            if (node === undefined) {
                return true;
            }
            return node.pos === node.end && node.pos >= 0 && node.kind !== 1 /* EndOfFileToken */;
        }
        ts.nodeIsMissing = nodeIsMissing;
        function nodeIsPresent(node) {
            return !nodeIsMissing(node);
        }
        ts.nodeIsPresent = nodeIsPresent;
        function insertStatementsAfterPrologue(to, from, isPrologueDirective) {
            if (from === undefined || from.length === 0)
                return to;
            var statementIndex = 0;
            // skip all prologue directives to insert at the correct position
            for (; statementIndex < to.length; ++statementIndex) {
                if (!isPrologueDirective(to[statementIndex])) {
                    break;
                }
            }
            to.splice.apply(to, __spreadArrays([statementIndex, 0], from));
            return to;
        }
        function insertStatementAfterPrologue(to, statement, isPrologueDirective) {
            if (statement === undefined)
                return to;
            var statementIndex = 0;
            // skip all prologue directives to insert at the correct position
            for (; statementIndex < to.length; ++statementIndex) {
                if (!isPrologueDirective(to[statementIndex])) {
                    break;
                }
            }
            to.splice(statementIndex, 0, statement);
            return to;
        }
        function isAnyPrologueDirective(node) {
            return isPrologueDirective(node) || !!(getEmitFlags(node) & 1048576 /* CustomPrologue */);
        }
        /**
         * Prepends statements to an array while taking care of prologue directives.
         */
        function insertStatementsAfterStandardPrologue(to, from) {
            return insertStatementsAfterPrologue(to, from, isPrologueDirective);
        }
        ts.insertStatementsAfterStandardPrologue = insertStatementsAfterStandardPrologue;
        function insertStatementsAfterCustomPrologue(to, from) {
            return insertStatementsAfterPrologue(to, from, isAnyPrologueDirective);
        }
        ts.insertStatementsAfterCustomPrologue = insertStatementsAfterCustomPrologue;
        /**
         * Prepends statements to an array while taking care of prologue directives.
         */
        function insertStatementAfterStandardPrologue(to, statement) {
            return insertStatementAfterPrologue(to, statement, isPrologueDirective);
        }
        ts.insertStatementAfterStandardPrologue = insertStatementAfterStandardPrologue;
        function insertStatementAfterCustomPrologue(to, statement) {
            return insertStatementAfterPrologue(to, statement, isAnyPrologueDirective);
        }
        ts.insertStatementAfterCustomPrologue = insertStatementAfterCustomPrologue;
        /**
         * Determine if the given comment is a triple-slash
         *
         * @return true if the comment is a triple-slash comment else false
         */
        function isRecognizedTripleSlashComment(text, commentPos, commentEnd) {
            // Verify this is /// comment, but do the regexp match only when we first can find /// in the comment text
            // so that we don't end up computing comment string and doing match for all // comments
            if (text.charCodeAt(commentPos + 1) === 47 /* slash */ &&
                commentPos + 2 < commentEnd &&
                text.charCodeAt(commentPos + 2) === 47 /* slash */) {
                var textSubStr = text.substring(commentPos, commentEnd);
                return textSubStr.match(ts.fullTripleSlashReferencePathRegEx) ||
                    textSubStr.match(ts.fullTripleSlashAMDReferencePathRegEx) ||
                    textSubStr.match(fullTripleSlashReferenceTypeReferenceDirectiveRegEx) ||
                    textSubStr.match(defaultLibReferenceRegEx) ?
                    true : false;
            }
            return false;
        }
        ts.isRecognizedTripleSlashComment = isRecognizedTripleSlashComment;
        function isPinnedComment(text, start) {
            return text.charCodeAt(start + 1) === 42 /* asterisk */ &&
                text.charCodeAt(start + 2) === 33 /* exclamation */;
        }
        ts.isPinnedComment = isPinnedComment;
        function getTokenPosOfNode(node, sourceFile, includeJsDoc) {
            // With nodes that have no width (i.e. 'Missing' nodes), we actually *don't*
            // want to skip trivia because this will launch us forward to the next token.
            if (nodeIsMissing(node)) {
                return node.pos;
            }
            if (ts.isJSDocNode(node)) {
                return ts.skipTrivia((sourceFile || getSourceFileOfNode(node)).text, node.pos, /*stopAfterLineBreak*/ false, /*stopAtComments*/ true);
            }
            if (includeJsDoc && ts.hasJSDocNodes(node)) {
                return getTokenPosOfNode(node.jsDoc[0]);
            }
            // For a syntax list, it is possible that one of its children has JSDocComment nodes, while
            // the syntax list itself considers them as normal trivia. Therefore if we simply skip
            // trivia for the list, we may have skipped the JSDocComment as well. So we should process its
            // first child to determine the actual position of its first token.
            if (node.kind === 323 /* SyntaxList */ && node._children.length > 0) {
                return getTokenPosOfNode(node._children[0], sourceFile, includeJsDoc);
            }
            return ts.skipTrivia((sourceFile || getSourceFileOfNode(node)).text, node.pos);
        }
        ts.getTokenPosOfNode = getTokenPosOfNode;
        function getNonDecoratorTokenPosOfNode(node, sourceFile) {
            if (nodeIsMissing(node) || !node.decorators) {
                return getTokenPosOfNode(node, sourceFile);
            }
            return ts.skipTrivia((sourceFile || getSourceFileOfNode(node)).text, node.decorators.end);
        }
        ts.getNonDecoratorTokenPosOfNode = getNonDecoratorTokenPosOfNode;
        function getSourceTextOfNodeFromSourceFile(sourceFile, node, includeTrivia) {
            if (includeTrivia === void 0) {
                includeTrivia = false;
            }
            return getTextOfNodeFromSourceText(sourceFile.text, node, includeTrivia);
        }
        ts.getSourceTextOfNodeFromSourceFile = getSourceTextOfNodeFromSourceFile;
        function isJSDocTypeExpressionOrChild(node) {
            return !!findAncestor(node, ts.isJSDocTypeExpression);
        }
        function getTextOfNodeFromSourceText(sourceText, node, includeTrivia) {
            if (includeTrivia === void 0) {
                includeTrivia = false;
            }
            if (nodeIsMissing(node)) {
                return "";
            }
            var text = sourceText.substring(includeTrivia ? node.pos : ts.skipTrivia(sourceText, node.pos), node.end);
            if (isJSDocTypeExpressionOrChild(node)) {
                // strip space + asterisk at line start
                text = text.replace(/(^|\r?\n|\r)\s*\*\s*/g, "$1");
            }
            return text;
        }
        ts.getTextOfNodeFromSourceText = getTextOfNodeFromSourceText;
        function getTextOfNode(node, includeTrivia) {
            if (includeTrivia === void 0) {
                includeTrivia = false;
            }
            return getSourceTextOfNodeFromSourceFile(getSourceFileOfNode(node), node, includeTrivia);
        }
        ts.getTextOfNode = getTextOfNode;
        function getPos(range) {
            return range.pos;
        }
        /**
         * Note: it is expected that the `nodeArray` and the `node` are within the same file.
         * For example, searching for a `SourceFile` in a `SourceFile[]` wouldn't work.
         */
        function indexOfNode(nodeArray, node) {
            return ts.binarySearch(nodeArray, node, getPos, ts.compareValues);
        }
        ts.indexOfNode = indexOfNode;
        /**
         * Gets flags that control emit behavior of a node.
         */
        function getEmitFlags(node) {
            var emitNode = node.emitNode;
            return emitNode && emitNode.flags || 0;
        }
        ts.getEmitFlags = getEmitFlags;
        function getLiteralText(node, sourceFile, neverAsciiEscape, jsxAttributeEscape) {
            // If we don't need to downlevel and we can reach the original source text using
            // the node's parent reference, then simply get the text as it was originally written.
            if (!nodeIsSynthesized(node) && node.parent && !((ts.isNumericLiteral(node) && node.numericLiteralFlags & 512 /* ContainsSeparator */) ||
                ts.isBigIntLiteral(node))) {
                return getSourceTextOfNodeFromSourceFile(sourceFile, node);
            }
            // If we can't reach the original source text, use the canonical form if it's a number,
            // or a (possibly escaped) quoted form of the original text if it's string-like.
            switch (node.kind) {
                case 10 /* StringLiteral */: {
                    var escapeText = jsxAttributeEscape ? escapeJsxAttributeString :
                        neverAsciiEscape || (getEmitFlags(node) & 16777216 /* NoAsciiEscaping */) ? escapeString :
                            escapeNonAsciiString;
                    if (node.singleQuote) {
                        return "'" + escapeText(node.text, 39 /* singleQuote */) + "'";
                    }
                    else {
                        return '"' + escapeText(node.text, 34 /* doubleQuote */) + '"';
                    }
                }
                case 14 /* NoSubstitutionTemplateLiteral */:
                case 15 /* TemplateHead */:
                case 16 /* TemplateMiddle */:
                case 17 /* TemplateTail */: {
                    // If a NoSubstitutionTemplateLiteral appears to have a substitution in it, the original text
                    // had to include a backslash: `not \${a} substitution`.
                    var escapeText = neverAsciiEscape || (getEmitFlags(node) & 16777216 /* NoAsciiEscaping */) ? escapeString :
                        escapeNonAsciiString;
                    var rawText = node.rawText || escapeTemplateSubstitution(escapeText(node.text, 96 /* backtick */));
                    switch (node.kind) {
                        case 14 /* NoSubstitutionTemplateLiteral */:
                            return "`" + rawText + "`";
                        case 15 /* TemplateHead */:
                            return "`" + rawText + "${";
                        case 16 /* TemplateMiddle */:
                            return "}" + rawText + "${";
                        case 17 /* TemplateTail */:
                            return "}" + rawText + "`";
                    }
                    break;
                }
                case 8 /* NumericLiteral */:
                case 9 /* BigIntLiteral */:
                case 13 /* RegularExpressionLiteral */:
                    return node.text;
            }
            return ts.Debug.fail("Literal kind '" + node.kind + "' not accounted for.");
        }
        ts.getLiteralText = getLiteralText;
        function getTextOfConstantValue(value) {
            return ts.isString(value) ? '"' + escapeNonAsciiString(value) + '"' : "" + value;
        }
        ts.getTextOfConstantValue = getTextOfConstantValue;
        // Make an identifier from an external module name by extracting the string after the last "/" and replacing
        // all non-alphanumeric characters with underscores
        function makeIdentifierFromModuleName(moduleName) {
            return ts.getBaseFileName(moduleName).replace(/^(\d)/, "_$1").replace(/\W/g, "_");
        }
        ts.makeIdentifierFromModuleName = makeIdentifierFromModuleName;
        function isBlockOrCatchScoped(declaration) {
            return (ts.getCombinedNodeFlags(declaration) & 3 /* BlockScoped */) !== 0 ||
                isCatchClauseVariableDeclarationOrBindingElement(declaration);
        }
        ts.isBlockOrCatchScoped = isBlockOrCatchScoped;
        function isCatchClauseVariableDeclarationOrBindingElement(declaration) {
            var node = getRootDeclaration(declaration);
            return node.kind === 242 /* VariableDeclaration */ && node.parent.kind === 280 /* CatchClause */;
        }
        ts.isCatchClauseVariableDeclarationOrBindingElement = isCatchClauseVariableDeclarationOrBindingElement;
        function isAmbientModule(node) {
            return ts.isModuleDeclaration(node) && (node.name.kind === 10 /* StringLiteral */ || isGlobalScopeAugmentation(node));
        }
        ts.isAmbientModule = isAmbientModule;
        function isModuleWithStringLiteralName(node) {
            return ts.isModuleDeclaration(node) && node.name.kind === 10 /* StringLiteral */;
        }
        ts.isModuleWithStringLiteralName = isModuleWithStringLiteralName;
        function isNonGlobalAmbientModule(node) {
            return ts.isModuleDeclaration(node) && ts.isStringLiteral(node.name);
        }
        ts.isNonGlobalAmbientModule = isNonGlobalAmbientModule;
        /**
         * An effective module (namespace) declaration is either
         * 1. An actual declaration: namespace X { ... }
         * 2. A Javascript declaration, which is:
         *    An identifier in a nested property access expression: Y in `X.Y.Z = { ... }`
         */
        function isEffectiveModuleDeclaration(node) {
            return ts.isModuleDeclaration(node) || ts.isIdentifier(node);
        }
        ts.isEffectiveModuleDeclaration = isEffectiveModuleDeclaration;
        /** Given a symbol for a module, checks that it is a shorthand ambient module. */
        function isShorthandAmbientModuleSymbol(moduleSymbol) {
            return isShorthandAmbientModule(moduleSymbol.valueDeclaration);
        }
        ts.isShorthandAmbientModuleSymbol = isShorthandAmbientModuleSymbol;
        function isShorthandAmbientModule(node) {
            // The only kind of module that can be missing a body is a shorthand ambient module.
            return node && node.kind === 249 /* ModuleDeclaration */ && (!node.body);
        }
        function isBlockScopedContainerTopLevel(node) {
            return node.kind === 290 /* SourceFile */ ||
                node.kind === 249 /* ModuleDeclaration */ ||
                ts.isFunctionLike(node);
        }
        ts.isBlockScopedContainerTopLevel = isBlockScopedContainerTopLevel;
        function isGlobalScopeAugmentation(module) {
            return !!(module.flags & 1024 /* GlobalAugmentation */);
        }
        ts.isGlobalScopeAugmentation = isGlobalScopeAugmentation;
        function isExternalModuleAugmentation(node) {
            return isAmbientModule(node) && isModuleAugmentationExternal(node);
        }
        ts.isExternalModuleAugmentation = isExternalModuleAugmentation;
        function isModuleAugmentationExternal(node) {
            // external module augmentation is a ambient module declaration that is either:
            // - defined in the top level scope and source file is an external module
            // - defined inside ambient module declaration located in the top level scope and source file not an external module
            switch (node.parent.kind) {
                case 290 /* SourceFile */:
                    return ts.isExternalModule(node.parent);
                case 250 /* ModuleBlock */:
                    return isAmbientModule(node.parent.parent) && ts.isSourceFile(node.parent.parent.parent) && !ts.isExternalModule(node.parent.parent.parent);
            }
            return false;
        }
        ts.isModuleAugmentationExternal = isModuleAugmentationExternal;
        function getNonAugmentationDeclaration(symbol) {
            return ts.find(symbol.declarations, function (d) { return !isExternalModuleAugmentation(d) && !(ts.isModuleDeclaration(d) && isGlobalScopeAugmentation(d)); });
        }
        ts.getNonAugmentationDeclaration = getNonAugmentationDeclaration;
        function isEffectiveExternalModule(node, compilerOptions) {
            return ts.isExternalModule(node) || compilerOptions.isolatedModules || ((getEmitModuleKind(compilerOptions) === ts.ModuleKind.CommonJS) && !!node.commonJsModuleIndicator);
        }
        ts.isEffectiveExternalModule = isEffectiveExternalModule;
        /**
         * Returns whether the source file will be treated as if it were in strict mode at runtime.
         */
        function isEffectiveStrictModeSourceFile(node, compilerOptions) {
            // We can only verify strict mode for JS/TS files
            switch (node.scriptKind) {
                case 1 /* JS */:
                case 3 /* TS */:
                case 2 /* JSX */:
                case 4 /* TSX */:
                    break;
                default:
                    return false;
            }
            // Strict mode does not matter for declaration files.
            if (node.isDeclarationFile) {
                return false;
            }
            // If `alwaysStrict` is set, then treat the file as strict.
            if (getStrictOptionValue(compilerOptions, "alwaysStrict")) {
                return true;
            }
            // Starting with a "use strict" directive indicates the file is strict.
            if (ts.startsWithUseStrict(node.statements)) {
                return true;
            }
            if (ts.isExternalModule(node) || compilerOptions.isolatedModules) {
                // ECMAScript Modules are always strict.
                if (getEmitModuleKind(compilerOptions) >= ts.ModuleKind.ES2015) {
                    return true;
                }
                // Other modules are strict unless otherwise specified.
                return !compilerOptions.noImplicitUseStrict;
            }
            return false;
        }
        ts.isEffectiveStrictModeSourceFile = isEffectiveStrictModeSourceFile;
        function isBlockScope(node, parentNode) {
            switch (node.kind) {
                case 290 /* SourceFile */:
                case 251 /* CaseBlock */:
                case 280 /* CatchClause */:
                case 249 /* ModuleDeclaration */:
                case 230 /* ForStatement */:
                case 231 /* ForInStatement */:
                case 232 /* ForOfStatement */:
                case 162 /* Constructor */:
                case 161 /* MethodDeclaration */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 244 /* FunctionDeclaration */:
                case 201 /* FunctionExpression */:
                case 202 /* ArrowFunction */:
                    return true;
                case 223 /* Block */:
                    // function block is not considered block-scope container
                    // see comment in binder.ts: bind(...), case for SyntaxKind.Block
                    return !ts.isFunctionLike(parentNode);
            }
            return false;
        }
        ts.isBlockScope = isBlockScope;
        function isDeclarationWithTypeParameters(node) {
            switch (node.kind) {
                case 314 /* JSDocCallbackTag */:
                case 321 /* JSDocTypedefTag */:
                case 305 /* JSDocSignature */:
                    return true;
                default:
                    ts.assertType(node);
                    return isDeclarationWithTypeParameterChildren(node);
            }
        }
        ts.isDeclarationWithTypeParameters = isDeclarationWithTypeParameters;
        function isDeclarationWithTypeParameterChildren(node) {
            switch (node.kind) {
                case 165 /* CallSignature */:
                case 166 /* ConstructSignature */:
                case 160 /* MethodSignature */:
                case 167 /* IndexSignature */:
                case 170 /* FunctionType */:
                case 171 /* ConstructorType */:
                case 300 /* JSDocFunctionType */:
                case 245 /* ClassDeclaration */:
                case 214 /* ClassExpression */:
                case 246 /* InterfaceDeclaration */:
                case 247 /* TypeAliasDeclaration */:
                case 320 /* JSDocTemplateTag */:
                case 244 /* FunctionDeclaration */:
                case 161 /* MethodDeclaration */:
                case 162 /* Constructor */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 201 /* FunctionExpression */:
                case 202 /* ArrowFunction */:
                    return true;
                default:
                    ts.assertType(node);
                    return false;
            }
        }
        ts.isDeclarationWithTypeParameterChildren = isDeclarationWithTypeParameterChildren;
        function isAnyImportSyntax(node) {
            switch (node.kind) {
                case 254 /* ImportDeclaration */:
                case 253 /* ImportEqualsDeclaration */:
                    return true;
                default:
                    return false;
            }
        }
        ts.isAnyImportSyntax = isAnyImportSyntax;
        function isLateVisibilityPaintedStatement(node) {
            switch (node.kind) {
                case 254 /* ImportDeclaration */:
                case 253 /* ImportEqualsDeclaration */:
                case 225 /* VariableStatement */:
                case 245 /* ClassDeclaration */:
                case 244 /* FunctionDeclaration */:
                case 249 /* ModuleDeclaration */:
                case 247 /* TypeAliasDeclaration */:
                case 246 /* InterfaceDeclaration */:
                case 248 /* EnumDeclaration */:
                    return true;
                default:
                    return false;
            }
        }
        ts.isLateVisibilityPaintedStatement = isLateVisibilityPaintedStatement;
        function isAnyImportOrReExport(node) {
            return isAnyImportSyntax(node) || ts.isExportDeclaration(node);
        }
        ts.isAnyImportOrReExport = isAnyImportOrReExport;
        // Gets the nearest enclosing block scope container that has the provided node
        // as a descendant, that is not the provided node.
        function getEnclosingBlockScopeContainer(node) {
            return findAncestor(node.parent, function (current) { return isBlockScope(current, current.parent); });
        }
        ts.getEnclosingBlockScopeContainer = getEnclosingBlockScopeContainer;
        // Return display name of an identifier
        // Computed property names will just be emitted as "[<expr>]", where <expr> is the source
        // text of the expression in the computed property.
        function declarationNameToString(name) {
            return !name || getFullWidth(name) === 0 ? "(Missing)" : getTextOfNode(name);
        }
        ts.declarationNameToString = declarationNameToString;
        function getNameFromIndexInfo(info) {
            return info.declaration ? declarationNameToString(info.declaration.parameters[0].name) : undefined;
        }
        ts.getNameFromIndexInfo = getNameFromIndexInfo;
        function getTextOfPropertyName(name) {
            switch (name.kind) {
                case 75 /* Identifier */:
                case 76 /* PrivateIdentifier */:
                    return name.escapedText;
                case 10 /* StringLiteral */:
                case 8 /* NumericLiteral */:
                case 14 /* NoSubstitutionTemplateLiteral */:
                    return ts.escapeLeadingUnderscores(name.text);
                case 154 /* ComputedPropertyName */:
                    if (isStringOrNumericLiteralLike(name.expression))
                        return ts.escapeLeadingUnderscores(name.expression.text);
                    return ts.Debug.fail("Text of property name cannot be read from non-literal-valued ComputedPropertyNames");
                default:
                    return ts.Debug.assertNever(name);
            }
        }
        ts.getTextOfPropertyName = getTextOfPropertyName;
        function entityNameToString(name) {
            switch (name.kind) {
                case 75 /* Identifier */:
                    return getFullWidth(name) === 0 ? ts.idText(name) : getTextOfNode(name);
                case 153 /* QualifiedName */:
                    return entityNameToString(name.left) + "." + entityNameToString(name.right);
                case 194 /* PropertyAccessExpression */:
                    if (ts.isIdentifier(name.name)) {
                        return entityNameToString(name.expression) + "." + entityNameToString(name.name);
                    }
                    else {
                        return ts.Debug.assertNever(name.name);
                    }
                default:
                    return ts.Debug.assertNever(name);
            }
        }
        ts.entityNameToString = entityNameToString;
        function createDiagnosticForNode(node, message, arg0, arg1, arg2, arg3) {
            var sourceFile = getSourceFileOfNode(node);
            return createDiagnosticForNodeInSourceFile(sourceFile, node, message, arg0, arg1, arg2, arg3);
        }
        ts.createDiagnosticForNode = createDiagnosticForNode;
        function createDiagnosticForNodeArray(sourceFile, nodes, message, arg0, arg1, arg2, arg3) {
            var start = ts.skipTrivia(sourceFile.text, nodes.pos);
            return createFileDiagnostic(sourceFile, start, nodes.end - start, message, arg0, arg1, arg2, arg3);
        }
        ts.createDiagnosticForNodeArray = createDiagnosticForNodeArray;
        function createDiagnosticForNodeInSourceFile(sourceFile, node, message, arg0, arg1, arg2, arg3) {
            var span = getErrorSpanForNode(sourceFile, node);
            return createFileDiagnostic(sourceFile, span.start, span.length, message, arg0, arg1, arg2, arg3);
        }
        ts.createDiagnosticForNodeInSourceFile = createDiagnosticForNodeInSourceFile;
        function createDiagnosticForNodeFromMessageChain(node, messageChain, relatedInformation) {
            var sourceFile = getSourceFileOfNode(node);
            var span = getErrorSpanForNode(sourceFile, node);
            return {
                file: sourceFile,
                start: span.start,
                length: span.length,
                code: messageChain.code,
                category: messageChain.category,
                messageText: messageChain.next ? messageChain : messageChain.messageText,
                relatedInformation: relatedInformation
            };
        }
        ts.createDiagnosticForNodeFromMessageChain = createDiagnosticForNodeFromMessageChain;
        function getSpanOfTokenAtPosition(sourceFile, pos) {
            var scanner = ts.createScanner(sourceFile.languageVersion, /*skipTrivia*/ true, sourceFile.languageVariant, sourceFile.text, /*onError:*/ undefined, pos);
            scanner.scan();
            var start = scanner.getTokenPos();
            return ts.createTextSpanFromBounds(start, scanner.getTextPos());
        }
        ts.getSpanOfTokenAtPosition = getSpanOfTokenAtPosition;
        function getErrorSpanForArrowFunction(sourceFile, node) {
            var pos = ts.skipTrivia(sourceFile.text, node.pos);
            if (node.body && node.body.kind === 223 /* Block */) {
                var startLine = ts.getLineAndCharacterOfPosition(sourceFile, node.body.pos).line;
                var endLine = ts.getLineAndCharacterOfPosition(sourceFile, node.body.end).line;
                if (startLine < endLine) {
                    // The arrow function spans multiple lines,
                    // make the error span be the first line, inclusive.
                    return ts.createTextSpan(pos, getEndLinePosition(startLine, sourceFile) - pos + 1);
                }
            }
            return ts.createTextSpanFromBounds(pos, node.end);
        }
        function getErrorSpanForNode(sourceFile, node) {
            var errorNode = node;
            switch (node.kind) {
                case 290 /* SourceFile */:
                    var pos_1 = ts.skipTrivia(sourceFile.text, 0, /*stopAfterLineBreak*/ false);
                    if (pos_1 === sourceFile.text.length) {
                        // file is empty - return span for the beginning of the file
                        return ts.createTextSpan(0, 0);
                    }
                    return getSpanOfTokenAtPosition(sourceFile, pos_1);
                // This list is a work in progress. Add missing node kinds to improve their error
                // spans.
                case 242 /* VariableDeclaration */:
                case 191 /* BindingElement */:
                case 245 /* ClassDeclaration */:
                case 214 /* ClassExpression */:
                case 246 /* InterfaceDeclaration */:
                case 249 /* ModuleDeclaration */:
                case 248 /* EnumDeclaration */:
                case 284 /* EnumMember */:
                case 244 /* FunctionDeclaration */:
                case 201 /* FunctionExpression */:
                case 161 /* MethodDeclaration */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 247 /* TypeAliasDeclaration */:
                case 159 /* PropertyDeclaration */:
                case 158 /* PropertySignature */:
                    errorNode = node.name;
                    break;
                case 202 /* ArrowFunction */:
                    return getErrorSpanForArrowFunction(sourceFile, node);
                case 277 /* CaseClause */:
                case 278 /* DefaultClause */:
                    var start = ts.skipTrivia(sourceFile.text, node.pos);
                    var end = node.statements.length > 0 ? node.statements[0].pos : node.end;
                    return ts.createTextSpanFromBounds(start, end);
            }
            if (errorNode === undefined) {
                // If we don't have a better node, then just set the error on the first token of
                // construct.
                return getSpanOfTokenAtPosition(sourceFile, node.pos);
            }
            ts.Debug.assert(!ts.isJSDoc(errorNode));
            var isMissing = nodeIsMissing(errorNode);
            var pos = isMissing || ts.isJsxText(node)
                ? errorNode.pos
                : ts.skipTrivia(sourceFile.text, errorNode.pos);
            // These asserts should all be satisfied for a properly constructed `errorNode`.
            if (isMissing) {
                ts.Debug.assert(pos === errorNode.pos, "This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809");
                ts.Debug.assert(pos === errorNode.end, "This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809");
            }
            else {
                ts.Debug.assert(pos >= errorNode.pos, "This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809");
                ts.Debug.assert(pos <= errorNode.end, "This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809");
            }
            return ts.createTextSpanFromBounds(pos, errorNode.end);
        }
        ts.getErrorSpanForNode = getErrorSpanForNode;
        function isExternalOrCommonJsModule(file) {
            return (file.externalModuleIndicator || file.commonJsModuleIndicator) !== undefined;
        }
        ts.isExternalOrCommonJsModule = isExternalOrCommonJsModule;
        function isJsonSourceFile(file) {
            return file.scriptKind === 6 /* JSON */;
        }
        ts.isJsonSourceFile = isJsonSourceFile;
        function isEnumConst(node) {
            return !!(ts.getCombinedModifierFlags(node) & 2048 /* Const */);
        }
        ts.isEnumConst = isEnumConst;
        function isDeclarationReadonly(declaration) {
            return !!(ts.getCombinedModifierFlags(declaration) & 64 /* Readonly */ && !ts.isParameterPropertyDeclaration(declaration, declaration.parent));
        }
        ts.isDeclarationReadonly = isDeclarationReadonly;
        function isVarConst(node) {
            return !!(ts.getCombinedNodeFlags(node) & 2 /* Const */);
        }
        ts.isVarConst = isVarConst;
        function isLet(node) {
            return !!(ts.getCombinedNodeFlags(node) & 1 /* Let */);
        }
        ts.isLet = isLet;
        function isSuperCall(n) {
            return n.kind === 196 /* CallExpression */ && n.expression.kind === 102 /* SuperKeyword */;
        }
        ts.isSuperCall = isSuperCall;
        function isImportCall(n) {
            return n.kind === 196 /* CallExpression */ && n.expression.kind === 96 /* ImportKeyword */;
        }
        ts.isImportCall = isImportCall;
        function isImportMeta(n) {
            return ts.isMetaProperty(n)
                && n.keywordToken === 96 /* ImportKeyword */
                && n.name.escapedText === "meta";
        }
        ts.isImportMeta = isImportMeta;
        function isLiteralImportTypeNode(n) {
            return ts.isImportTypeNode(n) && ts.isLiteralTypeNode(n.argument) && ts.isStringLiteral(n.argument.literal);
        }
        ts.isLiteralImportTypeNode = isLiteralImportTypeNode;
        function isPrologueDirective(node) {
            return node.kind === 226 /* ExpressionStatement */
                && node.expression.kind === 10 /* StringLiteral */;
        }
        ts.isPrologueDirective = isPrologueDirective;
        function getLeadingCommentRangesOfNode(node, sourceFileOfNode) {
            return node.kind !== 11 /* JsxText */ ? ts.getLeadingCommentRanges(sourceFileOfNode.text, node.pos) : undefined;
        }
        ts.getLeadingCommentRangesOfNode = getLeadingCommentRangesOfNode;
        function getJSDocCommentRanges(node, text) {
            var commentRanges = (node.kind === 156 /* Parameter */ ||
                node.kind === 155 /* TypeParameter */ ||
                node.kind === 201 /* FunctionExpression */ ||
                node.kind === 202 /* ArrowFunction */ ||
                node.kind === 200 /* ParenthesizedExpression */) ?
                ts.concatenate(ts.getTrailingCommentRanges(text, node.pos), ts.getLeadingCommentRanges(text, node.pos)) :
                ts.getLeadingCommentRanges(text, node.pos);
            // True if the comment starts with '/**' but not if it is '/**/'
            return ts.filter(commentRanges, function (comment) {
                return text.charCodeAt(comment.pos + 1) === 42 /* asterisk */ &&
                    text.charCodeAt(comment.pos + 2) === 42 /* asterisk */ &&
                    text.charCodeAt(comment.pos + 3) !== 47 /* slash */;
            });
        }
        ts.getJSDocCommentRanges = getJSDocCommentRanges;
        ts.fullTripleSlashReferencePathRegEx = /^(\/\/\/\s*<reference\s+path\s*=\s*)('|")(.+?)\2.*?\/>/;
        var fullTripleSlashReferenceTypeReferenceDirectiveRegEx = /^(\/\/\/\s*<reference\s+types\s*=\s*)('|")(.+?)\2.*?\/>/;
        ts.fullTripleSlashAMDReferencePathRegEx = /^(\/\/\/\s*<amd-dependency\s+path\s*=\s*)('|")(.+?)\2.*?\/>/;
        var defaultLibReferenceRegEx = /^(\/\/\/\s*<reference\s+no-default-lib\s*=\s*)('|")(.+?)\2\s*\/>/;
        function isPartOfTypeNode(node) {
            if (168 /* FirstTypeNode */ <= node.kind && node.kind <= 188 /* LastTypeNode */) {
                return true;
            }
            switch (node.kind) {
                case 125 /* AnyKeyword */:
                case 148 /* UnknownKeyword */:
                case 140 /* NumberKeyword */:
                case 151 /* BigIntKeyword */:
                case 143 /* StringKeyword */:
                case 128 /* BooleanKeyword */:
                case 144 /* SymbolKeyword */:
                case 141 /* ObjectKeyword */:
                case 146 /* UndefinedKeyword */:
                case 137 /* NeverKeyword */:
                    return true;
                case 110 /* VoidKeyword */:
                    return node.parent.kind !== 205 /* VoidExpression */;
                case 216 /* ExpressionWithTypeArguments */:
                    return !isExpressionWithTypeArgumentsInClassExtendsClause(node);
                case 155 /* TypeParameter */:
                    return node.parent.kind === 186 /* MappedType */ || node.parent.kind === 181 /* InferType */;
                // Identifiers and qualified names may be type nodes, depending on their context. Climb
                // above them to find the lowest container
                case 75 /* Identifier */:
                    // If the identifier is the RHS of a qualified name, then it's a type iff its parent is.
                    if (node.parent.kind === 153 /* QualifiedName */ && node.parent.right === node) {
                        node = node.parent;
                    }
                    else if (node.parent.kind === 194 /* PropertyAccessExpression */ && node.parent.name === node) {
                        node = node.parent;
                    }
                    // At this point, node is either a qualified name or an identifier
                    ts.Debug.assert(node.kind === 75 /* Identifier */ || node.kind === 153 /* QualifiedName */ || node.kind === 194 /* PropertyAccessExpression */, "'node' was expected to be a qualified name, identifier or property access in 'isPartOfTypeNode'.");
                // falls through
                case 153 /* QualifiedName */:
                case 194 /* PropertyAccessExpression */:
                case 104 /* ThisKeyword */: {
                    var parent = node.parent;
                    if (parent.kind === 172 /* TypeQuery */) {
                        return false;
                    }
                    if (parent.kind === 188 /* ImportType */) {
                        return !parent.isTypeOf;
                    }
                    // Do not recursively call isPartOfTypeNode on the parent. In the example:
                    //
                    //     let a: A.B.C;
                    //
                    // Calling isPartOfTypeNode would consider the qualified name A.B a type node.
                    // Only C and A.B.C are type nodes.
                    if (168 /* FirstTypeNode */ <= parent.kind && parent.kind <= 188 /* LastTypeNode */) {
                        return true;
                    }
                    switch (parent.kind) {
                        case 216 /* ExpressionWithTypeArguments */:
                            return !isExpressionWithTypeArgumentsInClassExtendsClause(parent);
                        case 155 /* TypeParameter */:
                            return node === parent.constraint;
                        case 320 /* JSDocTemplateTag */:
                            return node === parent.constraint;
                        case 159 /* PropertyDeclaration */:
                        case 158 /* PropertySignature */:
                        case 156 /* Parameter */:
                        case 242 /* VariableDeclaration */:
                            return node === parent.type;
                        case 244 /* FunctionDeclaration */:
                        case 201 /* FunctionExpression */:
                        case 202 /* ArrowFunction */:
                        case 162 /* Constructor */:
                        case 161 /* MethodDeclaration */:
                        case 160 /* MethodSignature */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            return node === parent.type;
                        case 165 /* CallSignature */:
                        case 166 /* ConstructSignature */:
                        case 167 /* IndexSignature */:
                            return node === parent.type;
                        case 199 /* TypeAssertionExpression */:
                            return node === parent.type;
                        case 196 /* CallExpression */:
                        case 197 /* NewExpression */:
                            return ts.contains(parent.typeArguments, node);
                        case 198 /* TaggedTemplateExpression */:
                            // TODO (drosen): TaggedTemplateExpressions may eventually support type arguments.
                            return false;
                    }
                }
            }
            return false;
        }
        ts.isPartOfTypeNode = isPartOfTypeNode;
        function isChildOfNodeWithKind(node, kind) {
            while (node) {
                if (node.kind === kind) {
                    return true;
                }
                node = node.parent;
            }
            return false;
        }
        ts.isChildOfNodeWithKind = isChildOfNodeWithKind;
        // Warning: This has the same semantics as the forEach family of functions,
        //          in that traversal terminates in the event that 'visitor' supplies a truthy value.
        function forEachReturnStatement(body, visitor) {
            return traverse(body);
            function traverse(node) {
                switch (node.kind) {
                    case 235 /* ReturnStatement */:
                        return visitor(node);
                    case 251 /* CaseBlock */:
                    case 223 /* Block */:
                    case 227 /* IfStatement */:
                    case 228 /* DoStatement */:
                    case 229 /* WhileStatement */:
                    case 230 /* ForStatement */:
                    case 231 /* ForInStatement */:
                    case 232 /* ForOfStatement */:
                    case 236 /* WithStatement */:
                    case 237 /* SwitchStatement */:
                    case 277 /* CaseClause */:
                    case 278 /* DefaultClause */:
                    case 238 /* LabeledStatement */:
                    case 240 /* TryStatement */:
                    case 280 /* CatchClause */:
                        return ts.forEachChild(node, traverse);
                }
            }
        }
        ts.forEachReturnStatement = forEachReturnStatement;
        function forEachYieldExpression(body, visitor) {
            return traverse(body);
            function traverse(node) {
                switch (node.kind) {
                    case 212 /* YieldExpression */:
                        visitor(node);
                        var operand = node.expression;
                        if (operand) {
                            traverse(operand);
                        }
                        return;
                    case 248 /* EnumDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                    case 249 /* ModuleDeclaration */:
                    case 247 /* TypeAliasDeclaration */:
                        // These are not allowed inside a generator now, but eventually they may be allowed
                        // as local types. Regardless, skip them to avoid the work.
                        return;
                    default:
                        if (ts.isFunctionLike(node)) {
                            if (node.name && node.name.kind === 154 /* ComputedPropertyName */) {
                                // Note that we will not include methods/accessors of a class because they would require
                                // first descending into the class. This is by design.
                                traverse(node.name.expression);
                                return;
                            }
                        }
                        else if (!isPartOfTypeNode(node)) {
                            // This is the general case, which should include mostly expressions and statements.
                            // Also includes NodeArrays.
                            ts.forEachChild(node, traverse);
                        }
                }
            }
        }
        ts.forEachYieldExpression = forEachYieldExpression;
        /**
         * Gets the most likely element type for a TypeNode. This is not an exhaustive test
         * as it assumes a rest argument can only be an array type (either T[], or Array<T>).
         *
         * @param node The type node.
         */
        function getRestParameterElementType(node) {
            if (node && node.kind === 174 /* ArrayType */) {
                return node.elementType;
            }
            else if (node && node.kind === 169 /* TypeReference */) {
                return ts.singleOrUndefined(node.typeArguments);
            }
            else {
                return undefined;
            }
        }
        ts.getRestParameterElementType = getRestParameterElementType;
        function getMembersOfDeclaration(node) {
            switch (node.kind) {
                case 246 /* InterfaceDeclaration */:
                case 245 /* ClassDeclaration */:
                case 214 /* ClassExpression */:
                case 173 /* TypeLiteral */:
                    return node.members;
                case 193 /* ObjectLiteralExpression */:
                    return node.properties;
            }
        }
        ts.getMembersOfDeclaration = getMembersOfDeclaration;
        function isVariableLike(node) {
            if (node) {
                switch (node.kind) {
                    case 191 /* BindingElement */:
                    case 284 /* EnumMember */:
                    case 156 /* Parameter */:
                    case 281 /* PropertyAssignment */:
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                    case 282 /* ShorthandPropertyAssignment */:
                    case 242 /* VariableDeclaration */:
                        return true;
                }
            }
            return false;
        }
        ts.isVariableLike = isVariableLike;
        function isVariableLikeOrAccessor(node) {
            return isVariableLike(node) || ts.isAccessor(node);
        }
        ts.isVariableLikeOrAccessor = isVariableLikeOrAccessor;
        function isVariableDeclarationInVariableStatement(node) {
            return node.parent.kind === 243 /* VariableDeclarationList */
                && node.parent.parent.kind === 225 /* VariableStatement */;
        }
        ts.isVariableDeclarationInVariableStatement = isVariableDeclarationInVariableStatement;
        function isValidESSymbolDeclaration(node) {
            return ts.isVariableDeclaration(node) ? isVarConst(node) && ts.isIdentifier(node.name) && isVariableDeclarationInVariableStatement(node) :
                ts.isPropertyDeclaration(node) ? hasReadonlyModifier(node) && hasStaticModifier(node) :
                    ts.isPropertySignature(node) && hasReadonlyModifier(node);
        }
        ts.isValidESSymbolDeclaration = isValidESSymbolDeclaration;
        function introducesArgumentsExoticObject(node) {
            switch (node.kind) {
                case 161 /* MethodDeclaration */:
                case 160 /* MethodSignature */:
                case 162 /* Constructor */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 244 /* FunctionDeclaration */:
                case 201 /* FunctionExpression */:
                    return true;
            }
            return false;
        }
        ts.introducesArgumentsExoticObject = introducesArgumentsExoticObject;
        function unwrapInnermostStatementOfLabel(node, beforeUnwrapLabelCallback) {
            while (true) {
                if (beforeUnwrapLabelCallback) {
                    beforeUnwrapLabelCallback(node);
                }
                if (node.statement.kind !== 238 /* LabeledStatement */) {
                    return node.statement;
                }
                node = node.statement;
            }
        }
        ts.unwrapInnermostStatementOfLabel = unwrapInnermostStatementOfLabel;
        function isFunctionBlock(node) {
            return node && node.kind === 223 /* Block */ && ts.isFunctionLike(node.parent);
        }
        ts.isFunctionBlock = isFunctionBlock;
        function isObjectLiteralMethod(node) {
            return node && node.kind === 161 /* MethodDeclaration */ && node.parent.kind === 193 /* ObjectLiteralExpression */;
        }
        ts.isObjectLiteralMethod = isObjectLiteralMethod;
        function isObjectLiteralOrClassExpressionMethod(node) {
            return node.kind === 161 /* MethodDeclaration */ &&
                (node.parent.kind === 193 /* ObjectLiteralExpression */ ||
                    node.parent.kind === 214 /* ClassExpression */);
        }
        ts.isObjectLiteralOrClassExpressionMethod = isObjectLiteralOrClassExpressionMethod;
        function isIdentifierTypePredicate(predicate) {
            return predicate && predicate.kind === 1 /* Identifier */;
        }
        ts.isIdentifierTypePredicate = isIdentifierTypePredicate;
        function isThisTypePredicate(predicate) {
            return predicate && predicate.kind === 0 /* This */;
        }
        ts.isThisTypePredicate = isThisTypePredicate;
        function getPropertyAssignment(objectLiteral, key, key2) {
            return objectLiteral.properties.filter(function (property) {
                if (property.kind === 281 /* PropertyAssignment */) {
                    var propName = getTextOfPropertyName(property.name);
                    return key === propName || (!!key2 && key2 === propName);
                }
                return false;
            });
        }
        ts.getPropertyAssignment = getPropertyAssignment;
        function getTsConfigObjectLiteralExpression(tsConfigSourceFile) {
            if (tsConfigSourceFile && tsConfigSourceFile.statements.length) {
                var expression = tsConfigSourceFile.statements[0].expression;
                return ts.tryCast(expression, ts.isObjectLiteralExpression);
            }
        }
        ts.getTsConfigObjectLiteralExpression = getTsConfigObjectLiteralExpression;
        function getTsConfigPropArrayElementValue(tsConfigSourceFile, propKey, elementValue) {
            return ts.firstDefined(getTsConfigPropArray(tsConfigSourceFile, propKey), function (property) {
                return ts.isArrayLiteralExpression(property.initializer) ?
                    ts.find(property.initializer.elements, function (element) { return ts.isStringLiteral(element) && element.text === elementValue; }) :
                    undefined;
            });
        }
        ts.getTsConfigPropArrayElementValue = getTsConfigPropArrayElementValue;
        function getTsConfigPropArray(tsConfigSourceFile, propKey) {
            var jsonObjectLiteral = getTsConfigObjectLiteralExpression(tsConfigSourceFile);
            return jsonObjectLiteral ? getPropertyAssignment(jsonObjectLiteral, propKey) : ts.emptyArray;
        }
        ts.getTsConfigPropArray = getTsConfigPropArray;
        function getContainingFunction(node) {
            return findAncestor(node.parent, ts.isFunctionLike);
        }
        ts.getContainingFunction = getContainingFunction;
        function getContainingFunctionDeclaration(node) {
            return findAncestor(node.parent, ts.isFunctionLikeDeclaration);
        }
        ts.getContainingFunctionDeclaration = getContainingFunctionDeclaration;
        function getContainingClass(node) {
            return findAncestor(node.parent, ts.isClassLike);
        }
        ts.getContainingClass = getContainingClass;
        function getThisContainer(node, includeArrowFunctions) {
            ts.Debug.assert(node.kind !== 290 /* SourceFile */);
            while (true) {
                node = node.parent;
                if (!node) {
                    return ts.Debug.fail(); // If we never pass in a SourceFile, this should be unreachable, since we'll stop when we reach that.
                }
                switch (node.kind) {
                    case 154 /* ComputedPropertyName */:
                        // If the grandparent node is an object literal (as opposed to a class),
                        // then the computed property is not a 'this' container.
                        // A computed property name in a class needs to be a this container
                        // so that we can error on it.
                        if (ts.isClassLike(node.parent.parent)) {
                            return node;
                        }
                        // If this is a computed property, then the parent should not
                        // make it a this container. The parent might be a property
                        // in an object literal, like a method or accessor. But in order for
                        // such a parent to be a this container, the reference must be in
                        // the *body* of the container.
                        node = node.parent;
                        break;
                    case 157 /* Decorator */:
                        // Decorators are always applied outside of the body of a class or method.
                        if (node.parent.kind === 156 /* Parameter */ && ts.isClassElement(node.parent.parent)) {
                            // If the decorator's parent is a Parameter, we resolve the this container from
                            // the grandparent class declaration.
                            node = node.parent.parent;
                        }
                        else if (ts.isClassElement(node.parent)) {
                            // If the decorator's parent is a class element, we resolve the 'this' container
                            // from the parent class declaration.
                            node = node.parent;
                        }
                        break;
                    case 202 /* ArrowFunction */:
                        if (!includeArrowFunctions) {
                            continue;
                        }
                    // falls through
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 249 /* ModuleDeclaration */:
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                    case 162 /* Constructor */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                    case 165 /* CallSignature */:
                    case 166 /* ConstructSignature */:
                    case 167 /* IndexSignature */:
                    case 248 /* EnumDeclaration */:
                    case 290 /* SourceFile */:
                        return node;
                }
            }
        }
        ts.getThisContainer = getThisContainer;
        function getNewTargetContainer(node) {
            var container = getThisContainer(node, /*includeArrowFunctions*/ false);
            if (container) {
                switch (container.kind) {
                    case 162 /* Constructor */:
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                        return container;
                }
            }
            return undefined;
        }
        ts.getNewTargetContainer = getNewTargetContainer;
        /**
         * Given an super call/property node, returns the closest node where
         * - a super call/property access is legal in the node and not legal in the parent node the node.
         *   i.e. super call is legal in constructor but not legal in the class body.
         * - the container is an arrow function (so caller might need to call getSuperContainer again in case it needs to climb higher)
         * - a super call/property is definitely illegal in the container (but might be legal in some subnode)
         *   i.e. super property access is illegal in function declaration but can be legal in the statement list
         */
        function getSuperContainer(node, stopOnFunctions) {
            while (true) {
                node = node.parent;
                if (!node) {
                    return node;
                }
                switch (node.kind) {
                    case 154 /* ComputedPropertyName */:
                        node = node.parent;
                        break;
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                        if (!stopOnFunctions) {
                            continue;
                        }
                    // falls through
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                    case 162 /* Constructor */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        return node;
                    case 157 /* Decorator */:
                        // Decorators are always applied outside of the body of a class or method.
                        if (node.parent.kind === 156 /* Parameter */ && ts.isClassElement(node.parent.parent)) {
                            // If the decorator's parent is a Parameter, we resolve the this container from
                            // the grandparent class declaration.
                            node = node.parent.parent;
                        }
                        else if (ts.isClassElement(node.parent)) {
                            // If the decorator's parent is a class element, we resolve the 'this' container
                            // from the parent class declaration.
                            node = node.parent;
                        }
                        break;
                }
            }
        }
        ts.getSuperContainer = getSuperContainer;
        function getImmediatelyInvokedFunctionExpression(func) {
            if (func.kind === 201 /* FunctionExpression */ || func.kind === 202 /* ArrowFunction */) {
                var prev = func;
                var parent = func.parent;
                while (parent.kind === 200 /* ParenthesizedExpression */) {
                    prev = parent;
                    parent = parent.parent;
                }
                if (parent.kind === 196 /* CallExpression */ && parent.expression === prev) {
                    return parent;
                }
            }
        }
        ts.getImmediatelyInvokedFunctionExpression = getImmediatelyInvokedFunctionExpression;
        function isSuperOrSuperProperty(node) {
            return node.kind === 102 /* SuperKeyword */
                || isSuperProperty(node);
        }
        ts.isSuperOrSuperProperty = isSuperOrSuperProperty;
        /**
         * Determines whether a node is a property or element access expression for `super`.
         */
        function isSuperProperty(node) {
            var kind = node.kind;
            return (kind === 194 /* PropertyAccessExpression */ || kind === 195 /* ElementAccessExpression */)
                && node.expression.kind === 102 /* SuperKeyword */;
        }
        ts.isSuperProperty = isSuperProperty;
        /**
         * Determines whether a node is a property or element access expression for `this`.
         */
        function isThisProperty(node) {
            var kind = node.kind;
            return (kind === 194 /* PropertyAccessExpression */ || kind === 195 /* ElementAccessExpression */)
                && node.expression.kind === 104 /* ThisKeyword */;
        }
        ts.isThisProperty = isThisProperty;
        function getEntityNameFromTypeNode(node) {
            switch (node.kind) {
                case 169 /* TypeReference */:
                    return node.typeName;
                case 216 /* ExpressionWithTypeArguments */:
                    return isEntityNameExpression(node.expression)
                        ? node.expression
                        : undefined;
                case 75 /* Identifier */:
                case 153 /* QualifiedName */:
                    return node;
            }
            return undefined;
        }
        ts.getEntityNameFromTypeNode = getEntityNameFromTypeNode;
        function getInvokedExpression(node) {
            switch (node.kind) {
                case 198 /* TaggedTemplateExpression */:
                    return node.tag;
                case 268 /* JsxOpeningElement */:
                case 267 /* JsxSelfClosingElement */:
                    return node.tagName;
                default:
                    return node.expression;
            }
        }
        ts.getInvokedExpression = getInvokedExpression;
        function nodeCanBeDecorated(node, parent, grandparent) {
            // private names cannot be used with decorators yet
            if (ts.isNamedDeclaration(node) && ts.isPrivateIdentifier(node.name)) {
                return false;
            }
            switch (node.kind) {
                case 245 /* ClassDeclaration */:
                    // classes are valid targets
                    return true;
                case 159 /* PropertyDeclaration */:
                    // property declarations are valid if their parent is a class declaration.
                    return parent.kind === 245 /* ClassDeclaration */;
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 161 /* MethodDeclaration */:
                    // if this method has a body and its parent is a class declaration, this is a valid target.
                    return node.body !== undefined
                        && parent.kind === 245 /* ClassDeclaration */;
                case 156 /* Parameter */:
                    // if the parameter's parent has a body and its grandparent is a class declaration, this is a valid target;
                    return parent.body !== undefined
                        && (parent.kind === 162 /* Constructor */
                            || parent.kind === 161 /* MethodDeclaration */
                            || parent.kind === 164 /* SetAccessor */)
                        && grandparent.kind === 245 /* ClassDeclaration */;
            }
            return false;
        }
        ts.nodeCanBeDecorated = nodeCanBeDecorated;
        function nodeIsDecorated(node, parent, grandparent) {
            return node.decorators !== undefined
                && nodeCanBeDecorated(node, parent, grandparent); // TODO: GH#18217
        }
        ts.nodeIsDecorated = nodeIsDecorated;
        function nodeOrChildIsDecorated(node, parent, grandparent) {
            return nodeIsDecorated(node, parent, grandparent) || childIsDecorated(node, parent); // TODO: GH#18217
        }
        ts.nodeOrChildIsDecorated = nodeOrChildIsDecorated;
        function childIsDecorated(node, parent) {
            switch (node.kind) {
                case 245 /* ClassDeclaration */:
                    return ts.some(node.members, function (m) { return nodeOrChildIsDecorated(m, node, parent); }); // TODO: GH#18217
                case 161 /* MethodDeclaration */:
                case 164 /* SetAccessor */:
                    return ts.some(node.parameters, function (p) { return nodeIsDecorated(p, node, parent); }); // TODO: GH#18217
                default:
                    return false;
            }
        }
        ts.childIsDecorated = childIsDecorated;
        function isJSXTagName(node) {
            var parent = node.parent;
            if (parent.kind === 268 /* JsxOpeningElement */ ||
                parent.kind === 267 /* JsxSelfClosingElement */ ||
                parent.kind === 269 /* JsxClosingElement */) {
                return parent.tagName === node;
            }
            return false;
        }
        ts.isJSXTagName = isJSXTagName;
        function isExpressionNode(node) {
            switch (node.kind) {
                case 102 /* SuperKeyword */:
                case 100 /* NullKeyword */:
                case 106 /* TrueKeyword */:
                case 91 /* FalseKeyword */:
                case 13 /* RegularExpressionLiteral */:
                case 192 /* ArrayLiteralExpression */:
                case 193 /* ObjectLiteralExpression */:
                case 194 /* PropertyAccessExpression */:
                case 195 /* ElementAccessExpression */:
                case 196 /* CallExpression */:
                case 197 /* NewExpression */:
                case 198 /* TaggedTemplateExpression */:
                case 217 /* AsExpression */:
                case 199 /* TypeAssertionExpression */:
                case 218 /* NonNullExpression */:
                case 200 /* ParenthesizedExpression */:
                case 201 /* FunctionExpression */:
                case 214 /* ClassExpression */:
                case 202 /* ArrowFunction */:
                case 205 /* VoidExpression */:
                case 203 /* DeleteExpression */:
                case 204 /* TypeOfExpression */:
                case 207 /* PrefixUnaryExpression */:
                case 208 /* PostfixUnaryExpression */:
                case 209 /* BinaryExpression */:
                case 210 /* ConditionalExpression */:
                case 213 /* SpreadElement */:
                case 211 /* TemplateExpression */:
                case 215 /* OmittedExpression */:
                case 266 /* JsxElement */:
                case 267 /* JsxSelfClosingElement */:
                case 270 /* JsxFragment */:
                case 212 /* YieldExpression */:
                case 206 /* AwaitExpression */:
                case 219 /* MetaProperty */:
                    return true;
                case 153 /* QualifiedName */:
                    while (node.parent.kind === 153 /* QualifiedName */) {
                        node = node.parent;
                    }
                    return node.parent.kind === 172 /* TypeQuery */ || isJSXTagName(node);
                case 75 /* Identifier */:
                    if (node.parent.kind === 172 /* TypeQuery */ || isJSXTagName(node)) {
                        return true;
                    }
                // falls through
                case 8 /* NumericLiteral */:
                case 9 /* BigIntLiteral */:
                case 10 /* StringLiteral */:
                case 14 /* NoSubstitutionTemplateLiteral */:
                case 104 /* ThisKeyword */:
                    return isInExpressionContext(node);
                default:
                    return false;
            }
        }
        ts.isExpressionNode = isExpressionNode;
        function isInExpressionContext(node) {
            var parent = node.parent;
            switch (parent.kind) {
                case 242 /* VariableDeclaration */:
                case 156 /* Parameter */:
                case 159 /* PropertyDeclaration */:
                case 158 /* PropertySignature */:
                case 284 /* EnumMember */:
                case 281 /* PropertyAssignment */:
                case 191 /* BindingElement */:
                    return parent.initializer === node;
                case 226 /* ExpressionStatement */:
                case 227 /* IfStatement */:
                case 228 /* DoStatement */:
                case 229 /* WhileStatement */:
                case 235 /* ReturnStatement */:
                case 236 /* WithStatement */:
                case 237 /* SwitchStatement */:
                case 277 /* CaseClause */:
                case 239 /* ThrowStatement */:
                    return parent.expression === node;
                case 230 /* ForStatement */:
                    var forStatement = parent;
                    return (forStatement.initializer === node && forStatement.initializer.kind !== 243 /* VariableDeclarationList */) ||
                        forStatement.condition === node ||
                        forStatement.incrementor === node;
                case 231 /* ForInStatement */:
                case 232 /* ForOfStatement */:
                    var forInStatement = parent;
                    return (forInStatement.initializer === node && forInStatement.initializer.kind !== 243 /* VariableDeclarationList */) ||
                        forInStatement.expression === node;
                case 199 /* TypeAssertionExpression */:
                case 217 /* AsExpression */:
                    return node === parent.expression;
                case 221 /* TemplateSpan */:
                    return node === parent.expression;
                case 154 /* ComputedPropertyName */:
                    return node === parent.expression;
                case 157 /* Decorator */:
                case 276 /* JsxExpression */:
                case 275 /* JsxSpreadAttribute */:
                case 283 /* SpreadAssignment */:
                    return true;
                case 216 /* ExpressionWithTypeArguments */:
                    return parent.expression === node && isExpressionWithTypeArgumentsInClassExtendsClause(parent);
                case 282 /* ShorthandPropertyAssignment */:
                    return parent.objectAssignmentInitializer === node;
                default:
                    return isExpressionNode(parent);
            }
        }
        ts.isInExpressionContext = isInExpressionContext;
        function isPartOfTypeQuery(node) {
            while (node.kind === 153 /* QualifiedName */ || node.kind === 75 /* Identifier */) {
                node = node.parent;
            }
            return node.kind === 172 /* TypeQuery */;
        }
        ts.isPartOfTypeQuery = isPartOfTypeQuery;
        function isExternalModuleImportEqualsDeclaration(node) {
            return node.kind === 253 /* ImportEqualsDeclaration */ && node.moduleReference.kind === 265 /* ExternalModuleReference */;
        }
        ts.isExternalModuleImportEqualsDeclaration = isExternalModuleImportEqualsDeclaration;
        function getExternalModuleImportEqualsDeclarationExpression(node) {
            ts.Debug.assert(isExternalModuleImportEqualsDeclaration(node));
            return node.moduleReference.expression;
        }
        ts.getExternalModuleImportEqualsDeclarationExpression = getExternalModuleImportEqualsDeclarationExpression;
        function isInternalModuleImportEqualsDeclaration(node) {
            return node.kind === 253 /* ImportEqualsDeclaration */ && node.moduleReference.kind !== 265 /* ExternalModuleReference */;
        }
        ts.isInternalModuleImportEqualsDeclaration = isInternalModuleImportEqualsDeclaration;
        function isSourceFileJS(file) {
            return isInJSFile(file);
        }
        ts.isSourceFileJS = isSourceFileJS;
        function isSourceFileNotJS(file) {
            return !isInJSFile(file);
        }
        ts.isSourceFileNotJS = isSourceFileNotJS;
        function isInJSFile(node) {
            return !!node && !!(node.flags & 131072 /* JavaScriptFile */);
        }
        ts.isInJSFile = isInJSFile;
        function isInJsonFile(node) {
            return !!node && !!(node.flags & 33554432 /* JsonFile */);
        }
        ts.isInJsonFile = isInJsonFile;
        function isSourceFileNotJson(file) {
            return !isJsonSourceFile(file);
        }
        ts.isSourceFileNotJson = isSourceFileNotJson;
        function isInJSDoc(node) {
            return !!node && !!(node.flags & 4194304 /* JSDoc */);
        }
        ts.isInJSDoc = isInJSDoc;
        function isJSDocIndexSignature(node) {
            return ts.isTypeReferenceNode(node) &&
                ts.isIdentifier(node.typeName) &&
                node.typeName.escapedText === "Object" &&
                node.typeArguments && node.typeArguments.length === 2 &&
                (node.typeArguments[0].kind === 143 /* StringKeyword */ || node.typeArguments[0].kind === 140 /* NumberKeyword */);
        }
        ts.isJSDocIndexSignature = isJSDocIndexSignature;
        function isRequireCall(callExpression, requireStringLiteralLikeArgument) {
            if (callExpression.kind !== 196 /* CallExpression */) {
                return false;
            }
            var _a = callExpression, expression = _a.expression, args = _a.arguments;
            if (expression.kind !== 75 /* Identifier */ || expression.escapedText !== "require") {
                return false;
            }
            if (args.length !== 1) {
                return false;
            }
            var arg = args[0];
            return !requireStringLiteralLikeArgument || ts.isStringLiteralLike(arg);
        }
        ts.isRequireCall = isRequireCall;
        function isSingleOrDoubleQuote(charCode) {
            return charCode === 39 /* singleQuote */ || charCode === 34 /* doubleQuote */;
        }
        ts.isSingleOrDoubleQuote = isSingleOrDoubleQuote;
        function isStringDoubleQuoted(str, sourceFile) {
            return getSourceTextOfNodeFromSourceFile(sourceFile, str).charCodeAt(0) === 34 /* doubleQuote */;
        }
        ts.isStringDoubleQuoted = isStringDoubleQuoted;
        function getDeclarationOfExpando(node) {
            if (!node.parent) {
                return undefined;
            }
            var name;
            var decl;
            if (ts.isVariableDeclaration(node.parent) && node.parent.initializer === node) {
                if (!isInJSFile(node) && !isVarConst(node.parent)) {
                    return undefined;
                }
                name = node.parent.name;
                decl = node.parent;
            }
            else if (ts.isBinaryExpression(node.parent)) {
                var parentNode = node.parent;
                var parentNodeOperator = node.parent.operatorToken.kind;
                if (parentNodeOperator === 62 /* EqualsToken */ && parentNode.right === node) {
                    name = parentNode.left;
                    decl = name;
                }
                else if (parentNodeOperator === 56 /* BarBarToken */ || parentNodeOperator === 60 /* QuestionQuestionToken */) {
                    if (ts.isVariableDeclaration(parentNode.parent) && parentNode.parent.initializer === parentNode) {
                        name = parentNode.parent.name;
                        decl = parentNode.parent;
                    }
                    else if (ts.isBinaryExpression(parentNode.parent) && parentNode.parent.operatorToken.kind === 62 /* EqualsToken */ && parentNode.parent.right === parentNode) {
                        name = parentNode.parent.left;
                        decl = name;
                    }
                    if (!name || !isBindableStaticNameExpression(name) || !isSameEntityName(name, parentNode.left)) {
                        return undefined;
                    }
                }
            }
            if (!name || !getExpandoInitializer(node, isPrototypeAccess(name))) {
                return undefined;
            }
            return decl;
        }
        ts.getDeclarationOfExpando = getDeclarationOfExpando;
        function isAssignmentDeclaration(decl) {
            return ts.isBinaryExpression(decl) || isAccessExpression(decl) || ts.isIdentifier(decl) || ts.isCallExpression(decl);
        }
        ts.isAssignmentDeclaration = isAssignmentDeclaration;
        /** Get the initializer, taking into account defaulted Javascript initializers */
        function getEffectiveInitializer(node) {
            if (isInJSFile(node) && node.initializer &&
                ts.isBinaryExpression(node.initializer) &&
                (node.initializer.operatorToken.kind === 56 /* BarBarToken */ || node.initializer.operatorToken.kind === 60 /* QuestionQuestionToken */) &&
                node.name && isEntityNameExpression(node.name) && isSameEntityName(node.name, node.initializer.left)) {
                return node.initializer.right;
            }
            return node.initializer;
        }
        ts.getEffectiveInitializer = getEffectiveInitializer;
        /** Get the declaration initializer when it is container-like (See getExpandoInitializer). */
        function getDeclaredExpandoInitializer(node) {
            var init = getEffectiveInitializer(node);
            return init && getExpandoInitializer(init, isPrototypeAccess(node.name));
        }
        ts.getDeclaredExpandoInitializer = getDeclaredExpandoInitializer;
        function hasExpandoValueProperty(node, isPrototypeAssignment) {
            return ts.forEach(node.properties, function (p) {
                return ts.isPropertyAssignment(p) &&
                    ts.isIdentifier(p.name) &&
                    p.name.escapedText === "value" &&
                    p.initializer &&
                    getExpandoInitializer(p.initializer, isPrototypeAssignment);
            });
        }
        /**
         * Get the assignment 'initializer' -- the righthand side-- when the initializer is container-like (See getExpandoInitializer).
         * We treat the right hand side of assignments with container-like initalizers as declarations.
         */
        function getAssignedExpandoInitializer(node) {
            if (node && node.parent && ts.isBinaryExpression(node.parent) && node.parent.operatorToken.kind === 62 /* EqualsToken */) {
                var isPrototypeAssignment = isPrototypeAccess(node.parent.left);
                return getExpandoInitializer(node.parent.right, isPrototypeAssignment) ||
                    getDefaultedExpandoInitializer(node.parent.left, node.parent.right, isPrototypeAssignment);
            }
            if (node && ts.isCallExpression(node) && isBindableObjectDefinePropertyCall(node)) {
                var result = hasExpandoValueProperty(node.arguments[2], node.arguments[1].text === "prototype");
                if (result) {
                    return result;
                }
            }
        }
        ts.getAssignedExpandoInitializer = getAssignedExpandoInitializer;
        /**
         * Recognized expando initializers are:
         * 1. (function() {})() -- IIFEs
         * 2. function() { } -- Function expressions
         * 3. class { } -- Class expressions
         * 4. {} -- Empty object literals
         * 5. { ... } -- Non-empty object literals, when used to initialize a prototype, like `C.prototype = { m() { } }`
         *
         * This function returns the provided initializer, or undefined if it is not valid.
         */
        function getExpandoInitializer(initializer, isPrototypeAssignment) {
            if (ts.isCallExpression(initializer)) {
                var e = skipParentheses(initializer.expression);
                return e.kind === 201 /* FunctionExpression */ || e.kind === 202 /* ArrowFunction */ ? initializer : undefined;
            }
            if (initializer.kind === 201 /* FunctionExpression */ ||
                initializer.kind === 214 /* ClassExpression */ ||
                initializer.kind === 202 /* ArrowFunction */) {
                return initializer;
            }
            if (ts.isObjectLiteralExpression(initializer) && (initializer.properties.length === 0 || isPrototypeAssignment)) {
                return initializer;
            }
        }
        ts.getExpandoInitializer = getExpandoInitializer;
        /**
         * A defaulted expando initializer matches the pattern
         * `Lhs = Lhs || ExpandoInitializer`
         * or `var Lhs = Lhs || ExpandoInitializer`
         *
         * The second Lhs is required to be the same as the first except that it may be prefixed with
         * 'window.', 'global.' or 'self.' The second Lhs is otherwise ignored by the binder and checker.
         */
        function getDefaultedExpandoInitializer(name, initializer, isPrototypeAssignment) {
            var e = ts.isBinaryExpression(initializer)
                && (initializer.operatorToken.kind === 56 /* BarBarToken */ || initializer.operatorToken.kind === 60 /* QuestionQuestionToken */)
                && getExpandoInitializer(initializer.right, isPrototypeAssignment);
            if (e && isSameEntityName(name, initializer.left)) {
                return e;
            }
        }
        function isDefaultedExpandoInitializer(node) {
            var name = ts.isVariableDeclaration(node.parent) ? node.parent.name :
                ts.isBinaryExpression(node.parent) && node.parent.operatorToken.kind === 62 /* EqualsToken */ ? node.parent.left :
                    undefined;
            return name && getExpandoInitializer(node.right, isPrototypeAccess(name)) && isEntityNameExpression(name) && isSameEntityName(name, node.left);
        }
        ts.isDefaultedExpandoInitializer = isDefaultedExpandoInitializer;
        /** Given an expando initializer, return its declaration name, or the left-hand side of the assignment if it's part of an assignment declaration. */
        function getNameOfExpando(node) {
            if (ts.isBinaryExpression(node.parent)) {
                var parent = ((node.parent.operatorToken.kind === 56 /* BarBarToken */ || node.parent.operatorToken.kind === 60 /* QuestionQuestionToken */) && ts.isBinaryExpression(node.parent.parent)) ? node.parent.parent : node.parent;
                if (parent.operatorToken.kind === 62 /* EqualsToken */ && ts.isIdentifier(parent.left)) {
                    return parent.left;
                }
            }
            else if (ts.isVariableDeclaration(node.parent)) {
                return node.parent.name;
            }
        }
        ts.getNameOfExpando = getNameOfExpando;
        /**
         * Is the 'declared' name the same as the one in the initializer?
         * @return true for identical entity names, as well as ones where the initializer is prefixed with
         * 'window', 'self' or 'global'. For example:
         *
         * var my = my || {}
         * var min = window.min || {}
         * my.app = self.my.app || class { }
         */
        function isSameEntityName(name, initializer) {
            if (isPropertyNameLiteral(name) && isPropertyNameLiteral(initializer)) {
                return getTextOfIdentifierOrLiteral(name) === getTextOfIdentifierOrLiteral(name);
            }
            if (ts.isIdentifier(name) && isLiteralLikeAccess(initializer) &&
                (initializer.expression.kind === 104 /* ThisKeyword */ ||
                    ts.isIdentifier(initializer.expression) &&
                        (initializer.expression.escapedText === "window" ||
                            initializer.expression.escapedText === "self" ||
                            initializer.expression.escapedText === "global"))) {
                var nameOrArgument = getNameOrArgument(initializer);
                if (ts.isPrivateIdentifier(nameOrArgument)) {
                    ts.Debug.fail("Unexpected PrivateIdentifier in name expression with literal-like access.");
                }
                return isSameEntityName(name, nameOrArgument);
            }
            if (isLiteralLikeAccess(name) && isLiteralLikeAccess(initializer)) {
                return getElementOrPropertyAccessName(name) === getElementOrPropertyAccessName(initializer)
                    && isSameEntityName(name.expression, initializer.expression);
            }
            return false;
        }
        function getRightMostAssignedExpression(node) {
            while (isAssignmentExpression(node, /*excludeCompoundAssignments*/ true)) {
                node = node.right;
            }
            return node;
        }
        ts.getRightMostAssignedExpression = getRightMostAssignedExpression;
        function isExportsIdentifier(node) {
            return ts.isIdentifier(node) && node.escapedText === "exports";
        }
        ts.isExportsIdentifier = isExportsIdentifier;
        function isModuleExportsAccessExpression(node) {
            return (ts.isPropertyAccessExpression(node) || isLiteralLikeElementAccess(node))
                && ts.isIdentifier(node.expression)
                && node.expression.escapedText === "module"
                && getElementOrPropertyAccessName(node) === "exports";
        }
        ts.isModuleExportsAccessExpression = isModuleExportsAccessExpression;
        /// Given a BinaryExpression, returns SpecialPropertyAssignmentKind for the various kinds of property
        /// assignments we treat as special in the binder
        function getAssignmentDeclarationKind(expr) {
            var special = getAssignmentDeclarationKindWorker(expr);
            return special === 5 /* Property */ || isInJSFile(expr) ? special : 0 /* None */;
        }
        ts.getAssignmentDeclarationKind = getAssignmentDeclarationKind;
        function isBindableObjectDefinePropertyCall(expr) {
            return ts.length(expr.arguments) === 3 &&
                ts.isPropertyAccessExpression(expr.expression) &&
                ts.isIdentifier(expr.expression.expression) &&
                ts.idText(expr.expression.expression) === "Object" &&
                ts.idText(expr.expression.name) === "defineProperty" &&
                isStringOrNumericLiteralLike(expr.arguments[1]) &&
                isBindableStaticNameExpression(expr.arguments[0], /*excludeThisKeyword*/ true);
        }
        ts.isBindableObjectDefinePropertyCall = isBindableObjectDefinePropertyCall;
        /** x.y OR x[0] */
        function isLiteralLikeAccess(node) {
            return ts.isPropertyAccessExpression(node) || isLiteralLikeElementAccess(node);
        }
        ts.isLiteralLikeAccess = isLiteralLikeAccess;
        /** x[0] OR x['a'] OR x[Symbol.y] */
        function isLiteralLikeElementAccess(node) {
            return ts.isElementAccessExpression(node) && (isStringOrNumericLiteralLike(node.argumentExpression) ||
                isWellKnownSymbolSyntactically(node.argumentExpression));
        }
        ts.isLiteralLikeElementAccess = isLiteralLikeElementAccess;
        /** Any series of property and element accesses. */
        function isBindableStaticAccessExpression(node, excludeThisKeyword) {
            return ts.isPropertyAccessExpression(node) && (!excludeThisKeyword && node.expression.kind === 104 /* ThisKeyword */ || isBindableStaticNameExpression(node.expression, /*excludeThisKeyword*/ true))
                || isBindableStaticElementAccessExpression(node, excludeThisKeyword);
        }
        ts.isBindableStaticAccessExpression = isBindableStaticAccessExpression;
        /** Any series of property and element accesses, ending in a literal element access */
        function isBindableStaticElementAccessExpression(node, excludeThisKeyword) {
            return isLiteralLikeElementAccess(node)
                && ((!excludeThisKeyword && node.expression.kind === 104 /* ThisKeyword */) ||
                    isEntityNameExpression(node.expression) ||
                    isBindableStaticAccessExpression(node.expression, /*excludeThisKeyword*/ true));
        }
        ts.isBindableStaticElementAccessExpression = isBindableStaticElementAccessExpression;
        function isBindableStaticNameExpression(node, excludeThisKeyword) {
            return isEntityNameExpression(node) || isBindableStaticAccessExpression(node, excludeThisKeyword);
        }
        ts.isBindableStaticNameExpression = isBindableStaticNameExpression;
        function getNameOrArgument(expr) {
            if (ts.isPropertyAccessExpression(expr)) {
                return expr.name;
            }
            return expr.argumentExpression;
        }
        ts.getNameOrArgument = getNameOrArgument;
        function getAssignmentDeclarationKindWorker(expr) {
            if (ts.isCallExpression(expr)) {
                if (!isBindableObjectDefinePropertyCall(expr)) {
                    return 0 /* None */;
                }
                var entityName = expr.arguments[0];
                if (isExportsIdentifier(entityName) || isModuleExportsAccessExpression(entityName)) {
                    return 8 /* ObjectDefinePropertyExports */;
                }
                if (isBindableStaticAccessExpression(entityName) && getElementOrPropertyAccessName(entityName) === "prototype") {
                    return 9 /* ObjectDefinePrototypeProperty */;
                }
                return 7 /* ObjectDefinePropertyValue */;
            }
            if (expr.operatorToken.kind !== 62 /* EqualsToken */ || !isAccessExpression(expr.left)) {
                return 0 /* None */;
            }
            if (isBindableStaticNameExpression(expr.left.expression, /*excludeThisKeyword*/ true) && getElementOrPropertyAccessName(expr.left) === "prototype" && ts.isObjectLiteralExpression(getInitializerOfBinaryExpression(expr))) {
                // F.prototype = { ... }
                return 6 /* Prototype */;
            }
            return getAssignmentDeclarationPropertyAccessKind(expr.left);
        }
        /**
         * Does not handle signed numeric names like `a[+0]` - handling those would require handling prefix unary expressions
         * throughout late binding handling as well, which is awkward (but ultimately probably doable if there is demand)
         */
        /* @internal */
        function getElementOrPropertyAccessArgumentExpressionOrName(node) {
            if (ts.isPropertyAccessExpression(node)) {
                return node.name;
            }
            var arg = skipParentheses(node.argumentExpression);
            if (ts.isNumericLiteral(arg) || ts.isStringLiteralLike(arg)) {
                return arg;
            }
            return node;
        }
        ts.getElementOrPropertyAccessArgumentExpressionOrName = getElementOrPropertyAccessArgumentExpressionOrName;
        function getElementOrPropertyAccessName(node) {
            var name = getElementOrPropertyAccessArgumentExpressionOrName(node);
            if (name) {
                if (ts.isIdentifier(name)) {
                    return name.escapedText;
                }
                if (ts.isStringLiteralLike(name) || ts.isNumericLiteral(name)) {
                    return ts.escapeLeadingUnderscores(name.text);
                }
            }
            if (ts.isElementAccessExpression(node) && isWellKnownSymbolSyntactically(node.argumentExpression)) {
                return getPropertyNameForKnownSymbolName(ts.idText(node.argumentExpression.name));
            }
            return undefined;
        }
        ts.getElementOrPropertyAccessName = getElementOrPropertyAccessName;
        function getAssignmentDeclarationPropertyAccessKind(lhs) {
            if (lhs.expression.kind === 104 /* ThisKeyword */) {
                return 4 /* ThisProperty */;
            }
            else if (isModuleExportsAccessExpression(lhs)) {
                // module.exports = expr
                return 2 /* ModuleExports */;
            }
            else if (isBindableStaticNameExpression(lhs.expression, /*excludeThisKeyword*/ true)) {
                if (isPrototypeAccess(lhs.expression)) {
                    // F.G....prototype.x = expr
                    return 3 /* PrototypeProperty */;
                }
                var nextToLast = lhs;
                while (!ts.isIdentifier(nextToLast.expression)) {
                    nextToLast = nextToLast.expression;
                }
                var id = nextToLast.expression;
                if ((id.escapedText === "exports" ||
                    id.escapedText === "module" && getElementOrPropertyAccessName(nextToLast) === "exports") &&
                    // ExportsProperty does not support binding with computed names
                    isBindableStaticAccessExpression(lhs)) {
                    // exports.name = expr OR module.exports.name = expr OR exports["name"] = expr ...
                    return 1 /* ExportsProperty */;
                }
                if (isBindableStaticNameExpression(lhs, /*excludeThisKeyword*/ true) || (ts.isElementAccessExpression(lhs) && isDynamicName(lhs) && lhs.expression.kind !== 104 /* ThisKeyword */)) {
                    // F.G...x = expr
                    return 5 /* Property */;
                }
            }
            return 0 /* None */;
        }
        ts.getAssignmentDeclarationPropertyAccessKind = getAssignmentDeclarationPropertyAccessKind;
        function getInitializerOfBinaryExpression(expr) {
            while (ts.isBinaryExpression(expr.right)) {
                expr = expr.right;
            }
            return expr.right;
        }
        ts.getInitializerOfBinaryExpression = getInitializerOfBinaryExpression;
        function isPrototypePropertyAssignment(node) {
            return ts.isBinaryExpression(node) && getAssignmentDeclarationKind(node) === 3 /* PrototypeProperty */;
        }
        ts.isPrototypePropertyAssignment = isPrototypePropertyAssignment;
        function isSpecialPropertyDeclaration(expr) {
            return isInJSFile(expr) &&
                expr.parent && expr.parent.kind === 226 /* ExpressionStatement */ &&
                (!ts.isElementAccessExpression(expr) || isLiteralLikeElementAccess(expr)) &&
                !!ts.getJSDocTypeTag(expr.parent);
        }
        ts.isSpecialPropertyDeclaration = isSpecialPropertyDeclaration;
        function isFunctionSymbol(symbol) {
            if (!symbol || !symbol.valueDeclaration) {
                return false;
            }
            var decl = symbol.valueDeclaration;
            return decl.kind === 244 /* FunctionDeclaration */ || ts.isVariableDeclaration(decl) && decl.initializer && ts.isFunctionLike(decl.initializer);
        }
        ts.isFunctionSymbol = isFunctionSymbol;
        function importFromModuleSpecifier(node) {
            return tryGetImportFromModuleSpecifier(node) || ts.Debug.failBadSyntaxKind(node.parent);
        }
        ts.importFromModuleSpecifier = importFromModuleSpecifier;
        function tryGetImportFromModuleSpecifier(node) {
            switch (node.parent.kind) {
                case 254 /* ImportDeclaration */:
                case 260 /* ExportDeclaration */:
                    return node.parent;
                case 265 /* ExternalModuleReference */:
                    return node.parent.parent;
                case 196 /* CallExpression */:
                    return isImportCall(node.parent) || isRequireCall(node.parent, /*checkArg*/ false) ? node.parent : undefined;
                case 187 /* LiteralType */:
                    ts.Debug.assert(ts.isStringLiteral(node));
                    return ts.tryCast(node.parent.parent, ts.isImportTypeNode);
                default:
                    return undefined;
            }
        }
        ts.tryGetImportFromModuleSpecifier = tryGetImportFromModuleSpecifier;
        function getExternalModuleName(node) {
            switch (node.kind) {
                case 254 /* ImportDeclaration */:
                case 260 /* ExportDeclaration */:
                    return node.moduleSpecifier;
                case 253 /* ImportEqualsDeclaration */:
                    return node.moduleReference.kind === 265 /* ExternalModuleReference */ ? node.moduleReference.expression : undefined;
                case 188 /* ImportType */:
                    return isLiteralImportTypeNode(node) ? node.argument.literal : undefined;
                default:
                    return ts.Debug.assertNever(node);
            }
        }
        ts.getExternalModuleName = getExternalModuleName;
        function getNamespaceDeclarationNode(node) {
            switch (node.kind) {
                case 254 /* ImportDeclaration */:
                    return node.importClause && ts.tryCast(node.importClause.namedBindings, ts.isNamespaceImport);
                case 253 /* ImportEqualsDeclaration */:
                    return node;
                case 260 /* ExportDeclaration */:
                    return node.exportClause && ts.tryCast(node.exportClause, ts.isNamespaceExport);
                default:
                    return ts.Debug.assertNever(node);
            }
        }
        ts.getNamespaceDeclarationNode = getNamespaceDeclarationNode;
        function isDefaultImport(node) {
            return node.kind === 254 /* ImportDeclaration */ && !!node.importClause && !!node.importClause.name;
        }
        ts.isDefaultImport = isDefaultImport;
        function forEachImportClauseDeclaration(node, action) {
            if (node.name) {
                var result = action(node);
                if (result)
                    return result;
            }
            if (node.namedBindings) {
                var result = ts.isNamespaceImport(node.namedBindings)
                    ? action(node.namedBindings)
                    : ts.forEach(node.namedBindings.elements, action);
                if (result)
                    return result;
            }
        }
        ts.forEachImportClauseDeclaration = forEachImportClauseDeclaration;
        function hasQuestionToken(node) {
            if (node) {
                switch (node.kind) {
                    case 156 /* Parameter */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                    case 282 /* ShorthandPropertyAssignment */:
                    case 281 /* PropertyAssignment */:
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                        return node.questionToken !== undefined;
                }
            }
            return false;
        }
        ts.hasQuestionToken = hasQuestionToken;
        function isJSDocConstructSignature(node) {
            var param = ts.isJSDocFunctionType(node) ? ts.firstOrUndefined(node.parameters) : undefined;
            var name = ts.tryCast(param && param.name, ts.isIdentifier);
            return !!name && name.escapedText === "new";
        }
        ts.isJSDocConstructSignature = isJSDocConstructSignature;
        function isJSDocTypeAlias(node) {
            return node.kind === 321 /* JSDocTypedefTag */ || node.kind === 314 /* JSDocCallbackTag */ || node.kind === 315 /* JSDocEnumTag */;
        }
        ts.isJSDocTypeAlias = isJSDocTypeAlias;
        function isTypeAlias(node) {
            return isJSDocTypeAlias(node) || ts.isTypeAliasDeclaration(node);
        }
        ts.isTypeAlias = isTypeAlias;
        function getSourceOfAssignment(node) {
            return ts.isExpressionStatement(node) &&
                ts.isBinaryExpression(node.expression) &&
                node.expression.operatorToken.kind === 62 /* EqualsToken */
                ? getRightMostAssignedExpression(node.expression)
                : undefined;
        }
        function getSourceOfDefaultedAssignment(node) {
            return ts.isExpressionStatement(node) &&
                ts.isBinaryExpression(node.expression) &&
                getAssignmentDeclarationKind(node.expression) !== 0 /* None */ &&
                ts.isBinaryExpression(node.expression.right) &&
                (node.expression.right.operatorToken.kind === 56 /* BarBarToken */ || node.expression.right.operatorToken.kind === 60 /* QuestionQuestionToken */)
                ? node.expression.right.right
                : undefined;
        }
        function getSingleInitializerOfVariableStatementOrPropertyDeclaration(node) {
            switch (node.kind) {
                case 225 /* VariableStatement */:
                    var v = getSingleVariableOfVariableStatement(node);
                    return v && v.initializer;
                case 159 /* PropertyDeclaration */:
                    return node.initializer;
                case 281 /* PropertyAssignment */:
                    return node.initializer;
            }
        }
        ts.getSingleInitializerOfVariableStatementOrPropertyDeclaration = getSingleInitializerOfVariableStatementOrPropertyDeclaration;
        function getSingleVariableOfVariableStatement(node) {
            return ts.isVariableStatement(node) ? ts.firstOrUndefined(node.declarationList.declarations) : undefined;
        }
        function getNestedModuleDeclaration(node) {
            return ts.isModuleDeclaration(node) &&
                node.body &&
                node.body.kind === 249 /* ModuleDeclaration */
                ? node.body
                : undefined;
        }
        function getJSDocCommentsAndTags(hostNode) {
            var result;
            // Pull parameter comments from declaring function as well
            if (isVariableLike(hostNode) && ts.hasInitializer(hostNode) && ts.hasJSDocNodes(hostNode.initializer)) {
                result = ts.append(result, ts.last(hostNode.initializer.jsDoc));
            }
            var node = hostNode;
            while (node && node.parent) {
                if (ts.hasJSDocNodes(node)) {
                    result = ts.append(result, ts.last(node.jsDoc));
                }
                if (node.kind === 156 /* Parameter */) {
                    result = ts.addRange(result, ts.getJSDocParameterTags(node));
                    break;
                }
                if (node.kind === 155 /* TypeParameter */) {
                    result = ts.addRange(result, ts.getJSDocTypeParameterTags(node));
                    break;
                }
                node = getNextJSDocCommentLocation(node);
            }
            return result || ts.emptyArray;
        }
        ts.getJSDocCommentsAndTags = getJSDocCommentsAndTags;
        function getNextJSDocCommentLocation(node) {
            var parent = node.parent;
            if (parent.kind === 281 /* PropertyAssignment */ ||
                parent.kind === 259 /* ExportAssignment */ ||
                parent.kind === 159 /* PropertyDeclaration */ ||
                parent.kind === 226 /* ExpressionStatement */ && node.kind === 194 /* PropertyAccessExpression */ ||
                getNestedModuleDeclaration(parent) ||
                ts.isBinaryExpression(node) && node.operatorToken.kind === 62 /* EqualsToken */) {
                return parent;
            }
            // Try to recognize this pattern when node is initializer of variable declaration and JSDoc comments are on containing variable statement.
            // /**
            //   * @param {number} name
            //   * @returns {number}
            //   */
            // var x = function(name) { return name.length; }
            else if (parent.parent &&
                (getSingleVariableOfVariableStatement(parent.parent) === node ||
                    ts.isBinaryExpression(parent) && parent.operatorToken.kind === 62 /* EqualsToken */)) {
                return parent.parent;
            }
            else if (parent.parent && parent.parent.parent &&
                (getSingleVariableOfVariableStatement(parent.parent.parent) ||
                    getSingleInitializerOfVariableStatementOrPropertyDeclaration(parent.parent.parent) === node ||
                    getSourceOfDefaultedAssignment(parent.parent.parent))) {
                return parent.parent.parent;
            }
        }
        /** Does the opposite of `getJSDocParameterTags`: given a JSDoc parameter, finds the parameter corresponding to it. */
        function getParameterSymbolFromJSDoc(node) {
            if (node.symbol) {
                return node.symbol;
            }
            if (!ts.isIdentifier(node.name)) {
                return undefined;
            }
            var name = node.name.escapedText;
            var decl = getHostSignatureFromJSDoc(node);
            if (!decl) {
                return undefined;
            }
            var parameter = ts.find(decl.parameters, function (p) { return p.name.kind === 75 /* Identifier */ && p.name.escapedText === name; });
            return parameter && parameter.symbol;
        }
        ts.getParameterSymbolFromJSDoc = getParameterSymbolFromJSDoc;
        function getHostSignatureFromJSDoc(node) {
            return getHostSignatureFromJSDocHost(getJSDocHost(node));
        }
        ts.getHostSignatureFromJSDoc = getHostSignatureFromJSDoc;
        function getHostSignatureFromJSDocHost(host) {
            var decl = getSourceOfDefaultedAssignment(host) ||
                getSourceOfAssignment(host) ||
                getSingleInitializerOfVariableStatementOrPropertyDeclaration(host) ||
                getSingleVariableOfVariableStatement(host) ||
                getNestedModuleDeclaration(host) ||
                host;
            return decl && ts.isFunctionLike(decl) ? decl : undefined;
        }
        ts.getHostSignatureFromJSDocHost = getHostSignatureFromJSDocHost;
        function getJSDocHost(node) {
            return ts.Debug.assertDefined(findAncestor(node.parent, ts.isJSDoc)).parent;
        }
        ts.getJSDocHost = getJSDocHost;
        function getTypeParameterFromJsDoc(node) {
            var name = node.name.escapedText;
            var typeParameters = node.parent.parent.parent.typeParameters;
            return typeParameters && ts.find(typeParameters, function (p) { return p.name.escapedText === name; });
        }
        ts.getTypeParameterFromJsDoc = getTypeParameterFromJsDoc;
        function hasRestParameter(s) {
            var last = ts.lastOrUndefined(s.parameters);
            return !!last && isRestParameter(last);
        }
        ts.hasRestParameter = hasRestParameter;
        function isRestParameter(node) {
            var type = ts.isJSDocParameterTag(node) ? (node.typeExpression && node.typeExpression.type) : node.type;
            return node.dotDotDotToken !== undefined || !!type && type.kind === 301 /* JSDocVariadicType */;
        }
        ts.isRestParameter = isRestParameter;
        function hasTypeArguments(node) {
            return !!node.typeArguments;
        }
        ts.hasTypeArguments = hasTypeArguments;
        var AssignmentKind;
        (function (AssignmentKind) {
            AssignmentKind[AssignmentKind["None"] = 0] = "None";
            AssignmentKind[AssignmentKind["Definite"] = 1] = "Definite";
            AssignmentKind[AssignmentKind["Compound"] = 2] = "Compound";
        })(AssignmentKind = ts.AssignmentKind || (ts.AssignmentKind = {}));
        function getAssignmentTargetKind(node) {
            var parent = node.parent;
            while (true) {
                switch (parent.kind) {
                    case 209 /* BinaryExpression */:
                        var binaryOperator = parent.operatorToken.kind;
                        return isAssignmentOperator(binaryOperator) && parent.left === node ?
                            binaryOperator === 62 /* EqualsToken */ ? 1 /* Definite */ : 2 /* Compound */ :
                            0 /* None */;
                    case 207 /* PrefixUnaryExpression */:
                    case 208 /* PostfixUnaryExpression */:
                        var unaryOperator = parent.operator;
                        return unaryOperator === 45 /* PlusPlusToken */ || unaryOperator === 46 /* MinusMinusToken */ ? 2 /* Compound */ : 0 /* None */;
                    case 231 /* ForInStatement */:
                    case 232 /* ForOfStatement */:
                        return parent.initializer === node ? 1 /* Definite */ : 0 /* None */;
                    case 200 /* ParenthesizedExpression */:
                    case 192 /* ArrayLiteralExpression */:
                    case 213 /* SpreadElement */:
                    case 218 /* NonNullExpression */:
                        node = parent;
                        break;
                    case 282 /* ShorthandPropertyAssignment */:
                        if (parent.name !== node) {
                            return 0 /* None */;
                        }
                        node = parent.parent;
                        break;
                    case 281 /* PropertyAssignment */:
                        if (parent.name === node) {
                            return 0 /* None */;
                        }
                        node = parent.parent;
                        break;
                    default:
                        return 0 /* None */;
                }
                parent = node.parent;
            }
        }
        ts.getAssignmentTargetKind = getAssignmentTargetKind;
        // A node is an assignment target if it is on the left hand side of an '=' token, if it is parented by a property
        // assignment in an object literal that is an assignment target, or if it is parented by an array literal that is
        // an assignment target. Examples include 'a = xxx', '{ p: a } = xxx', '[{ a }] = xxx'.
        // (Note that `p` is not a target in the above examples, only `a`.)
        function isAssignmentTarget(node) {
            return getAssignmentTargetKind(node) !== 0 /* None */;
        }
        ts.isAssignmentTarget = isAssignmentTarget;
        /**
         * Indicates whether a node could contain a `var` VariableDeclarationList that contributes to
         * the same `var` declaration scope as the node's parent.
         */
        function isNodeWithPossibleHoistedDeclaration(node) {
            switch (node.kind) {
                case 223 /* Block */:
                case 225 /* VariableStatement */:
                case 236 /* WithStatement */:
                case 227 /* IfStatement */:
                case 237 /* SwitchStatement */:
                case 251 /* CaseBlock */:
                case 277 /* CaseClause */:
                case 278 /* DefaultClause */:
                case 238 /* LabeledStatement */:
                case 230 /* ForStatement */:
                case 231 /* ForInStatement */:
                case 232 /* ForOfStatement */:
                case 228 /* DoStatement */:
                case 229 /* WhileStatement */:
                case 240 /* TryStatement */:
                case 280 /* CatchClause */:
                    return true;
            }
            return false;
        }
        ts.isNodeWithPossibleHoistedDeclaration = isNodeWithPossibleHoistedDeclaration;
        function isValueSignatureDeclaration(node) {
            return ts.isFunctionExpression(node) || ts.isArrowFunction(node) || ts.isMethodOrAccessor(node) || ts.isFunctionDeclaration(node) || ts.isConstructorDeclaration(node);
        }
        ts.isValueSignatureDeclaration = isValueSignatureDeclaration;
        function walkUp(node, kind) {
            while (node && node.kind === kind) {
                node = node.parent;
            }
            return node;
        }
        function walkUpParenthesizedTypes(node) {
            return walkUp(node, 182 /* ParenthesizedType */);
        }
        ts.walkUpParenthesizedTypes = walkUpParenthesizedTypes;
        function walkUpParenthesizedExpressions(node) {
            return walkUp(node, 200 /* ParenthesizedExpression */);
        }
        ts.walkUpParenthesizedExpressions = walkUpParenthesizedExpressions;
        function skipParentheses(node) {
            while (node.kind === 200 /* ParenthesizedExpression */) {
                node = node.expression;
            }
            return node;
        }
        ts.skipParentheses = skipParentheses;
        function skipParenthesesUp(node) {
            while (node.kind === 200 /* ParenthesizedExpression */) {
                node = node.parent;
            }
            return node;
        }
        // a node is delete target iff. it is PropertyAccessExpression/ElementAccessExpression with parentheses skipped
        function isDeleteTarget(node) {
            if (node.kind !== 194 /* PropertyAccessExpression */ && node.kind !== 195 /* ElementAccessExpression */) {
                return false;
            }
            node = walkUpParenthesizedExpressions(node.parent);
            return node && node.kind === 203 /* DeleteExpression */;
        }
        ts.isDeleteTarget = isDeleteTarget;
        function isNodeDescendantOf(node, ancestor) {
            while (node) {
                if (node === ancestor)
                    return true;
                node = node.parent;
            }
            return false;
        }
        ts.isNodeDescendantOf = isNodeDescendantOf;
        // True if `name` is the name of a declaration node
        function isDeclarationName(name) {
            return !ts.isSourceFile(name) && !ts.isBindingPattern(name) && ts.isDeclaration(name.parent) && name.parent.name === name;
        }
        ts.isDeclarationName = isDeclarationName;
        // See GH#16030
        function getDeclarationFromName(name) {
            var parent = name.parent;
            switch (name.kind) {
                case 10 /* StringLiteral */:
                case 14 /* NoSubstitutionTemplateLiteral */:
                case 8 /* NumericLiteral */:
                    if (ts.isComputedPropertyName(parent))
                        return parent.parent;
                // falls through
                case 75 /* Identifier */:
                    if (ts.isDeclaration(parent)) {
                        return parent.name === name ? parent : undefined;
                    }
                    else if (ts.isQualifiedName(parent)) {
                        var tag = parent.parent;
                        return ts.isJSDocParameterTag(tag) && tag.name === parent ? tag : undefined;
                    }
                    else {
                        var binExp = parent.parent;
                        return ts.isBinaryExpression(binExp) &&
                            getAssignmentDeclarationKind(binExp) !== 0 /* None */ &&
                            (binExp.left.symbol || binExp.symbol) &&
                            ts.getNameOfDeclaration(binExp) === name
                            ? binExp
                            : undefined;
                    }
                case 76 /* PrivateIdentifier */:
                    return ts.isDeclaration(parent) && parent.name === name ? parent : undefined;
                default:
                    return undefined;
            }
        }
        ts.getDeclarationFromName = getDeclarationFromName;
        function isLiteralComputedPropertyDeclarationName(node) {
            return isStringOrNumericLiteralLike(node) &&
                node.parent.kind === 154 /* ComputedPropertyName */ &&
                ts.isDeclaration(node.parent.parent);
        }
        ts.isLiteralComputedPropertyDeclarationName = isLiteralComputedPropertyDeclarationName;
        // Return true if the given identifier is classified as an IdentifierName
        function isIdentifierName(node) {
            var parent = node.parent;
            switch (parent.kind) {
                case 159 /* PropertyDeclaration */:
                case 158 /* PropertySignature */:
                case 161 /* MethodDeclaration */:
                case 160 /* MethodSignature */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 284 /* EnumMember */:
                case 281 /* PropertyAssignment */:
                case 194 /* PropertyAccessExpression */:
                    // Name in member declaration or property name in property access
                    return parent.name === node;
                case 153 /* QualifiedName */:
                    // Name on right hand side of dot in a type query or type reference
                    if (parent.right === node) {
                        while (parent.kind === 153 /* QualifiedName */) {
                            parent = parent.parent;
                        }
                        return parent.kind === 172 /* TypeQuery */ || parent.kind === 169 /* TypeReference */;
                    }
                    return false;
                case 191 /* BindingElement */:
                case 258 /* ImportSpecifier */:
                    // Property name in binding element or import specifier
                    return parent.propertyName === node;
                case 263 /* ExportSpecifier */:
                case 273 /* JsxAttribute */:
                    // Any name in an export specifier or JSX Attribute
                    return true;
            }
            return false;
        }
        ts.isIdentifierName = isIdentifierName;
        // An alias symbol is created by one of the following declarations:
        // import <symbol> = ...
        // import <symbol> from ...
        // import * as <symbol> from ...
        // import { x as <symbol> } from ...
        // export { x as <symbol> } from ...
        // export * as ns <symbol> from ...
        // export = <EntityNameExpression>
        // export default <EntityNameExpression>
        // module.exports = <EntityNameExpression>
        // {<Identifier>}
        // {name: <EntityNameExpression>}
        function isAliasSymbolDeclaration(node) {
            return node.kind === 253 /* ImportEqualsDeclaration */ ||
                node.kind === 252 /* NamespaceExportDeclaration */ ||
                node.kind === 255 /* ImportClause */ && !!node.name ||
                node.kind === 256 /* NamespaceImport */ ||
                node.kind === 262 /* NamespaceExport */ ||
                node.kind === 258 /* ImportSpecifier */ ||
                node.kind === 263 /* ExportSpecifier */ ||
                node.kind === 259 /* ExportAssignment */ && exportAssignmentIsAlias(node) ||
                ts.isBinaryExpression(node) && getAssignmentDeclarationKind(node) === 2 /* ModuleExports */ && exportAssignmentIsAlias(node) ||
                ts.isPropertyAccessExpression(node) && ts.isBinaryExpression(node.parent) && node.parent.left === node && node.parent.operatorToken.kind === 62 /* EqualsToken */ && isAliasableExpression(node.parent.right) ||
                node.kind === 282 /* ShorthandPropertyAssignment */ ||
                node.kind === 281 /* PropertyAssignment */ && isAliasableExpression(node.initializer);
        }
        ts.isAliasSymbolDeclaration = isAliasSymbolDeclaration;
        function getAliasDeclarationFromName(node) {
            switch (node.parent.kind) {
                case 255 /* ImportClause */:
                case 258 /* ImportSpecifier */:
                case 256 /* NamespaceImport */:
                case 263 /* ExportSpecifier */:
                case 259 /* ExportAssignment */:
                case 253 /* ImportEqualsDeclaration */:
                    return node.parent;
                case 153 /* QualifiedName */:
                    do {
                        node = node.parent;
                    } while (node.parent.kind === 153 /* QualifiedName */);
                    return getAliasDeclarationFromName(node);
            }
        }
        ts.getAliasDeclarationFromName = getAliasDeclarationFromName;
        function isAliasableExpression(e) {
            return isEntityNameExpression(e) || ts.isClassExpression(e);
        }
        ts.isAliasableExpression = isAliasableExpression;
        function exportAssignmentIsAlias(node) {
            var e = getExportAssignmentExpression(node);
            return isAliasableExpression(e);
        }
        ts.exportAssignmentIsAlias = exportAssignmentIsAlias;
        function getExportAssignmentExpression(node) {
            return ts.isExportAssignment(node) ? node.expression : node.right;
        }
        ts.getExportAssignmentExpression = getExportAssignmentExpression;
        function getPropertyAssignmentAliasLikeExpression(node) {
            return node.kind === 282 /* ShorthandPropertyAssignment */ ? node.name : node.kind === 281 /* PropertyAssignment */ ? node.initializer :
                node.parent.right;
        }
        ts.getPropertyAssignmentAliasLikeExpression = getPropertyAssignmentAliasLikeExpression;
        function getEffectiveBaseTypeNode(node) {
            var baseType = getClassExtendsHeritageElement(node);
            if (baseType && isInJSFile(node)) {
                // Prefer an @augments tag because it may have type parameters.
                var tag = ts.getJSDocAugmentsTag(node);
                if (tag) {
                    return tag.class;
                }
            }
            return baseType;
        }
        ts.getEffectiveBaseTypeNode = getEffectiveBaseTypeNode;
        function getClassExtendsHeritageElement(node) {
            var heritageClause = getHeritageClause(node.heritageClauses, 90 /* ExtendsKeyword */);
            return heritageClause && heritageClause.types.length > 0 ? heritageClause.types[0] : undefined;
        }
        ts.getClassExtendsHeritageElement = getClassExtendsHeritageElement;
        function getClassImplementsHeritageClauseElements(node) {
            var heritageClause = getHeritageClause(node.heritageClauses, 113 /* ImplementsKeyword */);
            return heritageClause ? heritageClause.types : undefined;
        }
        ts.getClassImplementsHeritageClauseElements = getClassImplementsHeritageClauseElements;
        /** Returns the node in an `extends` or `implements` clause of a class or interface. */
        function getAllSuperTypeNodes(node) {
            return ts.isInterfaceDeclaration(node) ? getInterfaceBaseTypeNodes(node) || ts.emptyArray :
                ts.isClassLike(node) ? ts.concatenate(ts.singleElementArray(getEffectiveBaseTypeNode(node)), getClassImplementsHeritageClauseElements(node)) || ts.emptyArray :
                    ts.emptyArray;
        }
        ts.getAllSuperTypeNodes = getAllSuperTypeNodes;
        function getInterfaceBaseTypeNodes(node) {
            var heritageClause = getHeritageClause(node.heritageClauses, 90 /* ExtendsKeyword */);
            return heritageClause ? heritageClause.types : undefined;
        }
        ts.getInterfaceBaseTypeNodes = getInterfaceBaseTypeNodes;
        function getHeritageClause(clauses, kind) {
            if (clauses) {
                for (var _i = 0, clauses_1 = clauses; _i < clauses_1.length; _i++) {
                    var clause = clauses_1[_i];
                    if (clause.token === kind) {
                        return clause;
                    }
                }
            }
            return undefined;
        }
        ts.getHeritageClause = getHeritageClause;
        function getAncestor(node, kind) {
            while (node) {
                if (node.kind === kind) {
                    return node;
                }
                node = node.parent;
            }
            return undefined;
        }
        ts.getAncestor = getAncestor;
        function isKeyword(token) {
            return 77 /* FirstKeyword */ <= token && token <= 152 /* LastKeyword */;
        }
        ts.isKeyword = isKeyword;
        function isContextualKeyword(token) {
            return 122 /* FirstContextualKeyword */ <= token && token <= 152 /* LastContextualKeyword */;
        }
        ts.isContextualKeyword = isContextualKeyword;
        function isNonContextualKeyword(token) {
            return isKeyword(token) && !isContextualKeyword(token);
        }
        ts.isNonContextualKeyword = isNonContextualKeyword;
        function isFutureReservedKeyword(token) {
            return 113 /* FirstFutureReservedWord */ <= token && token <= 121 /* LastFutureReservedWord */;
        }
        ts.isFutureReservedKeyword = isFutureReservedKeyword;
        function isStringANonContextualKeyword(name) {
            var token = ts.stringToToken(name);
            return token !== undefined && isNonContextualKeyword(token);
        }
        ts.isStringANonContextualKeyword = isStringANonContextualKeyword;
        function isStringAKeyword(name) {
            var token = ts.stringToToken(name);
            return token !== undefined && isKeyword(token);
        }
        ts.isStringAKeyword = isStringAKeyword;
        function isIdentifierANonContextualKeyword(_a) {
            var originalKeywordKind = _a.originalKeywordKind;
            return !!originalKeywordKind && !isContextualKeyword(originalKeywordKind);
        }
        ts.isIdentifierANonContextualKeyword = isIdentifierANonContextualKeyword;
        function isTrivia(token) {
            return 2 /* FirstTriviaToken */ <= token && token <= 7 /* LastTriviaToken */;
        }
        ts.isTrivia = isTrivia;
        var FunctionFlags;
        (function (FunctionFlags) {
            FunctionFlags[FunctionFlags["Normal"] = 0] = "Normal";
            FunctionFlags[FunctionFlags["Generator"] = 1] = "Generator";
            FunctionFlags[FunctionFlags["Async"] = 2] = "Async";
            FunctionFlags[FunctionFlags["Invalid"] = 4] = "Invalid";
            FunctionFlags[FunctionFlags["AsyncGenerator"] = 3] = "AsyncGenerator";
        })(FunctionFlags = ts.FunctionFlags || (ts.FunctionFlags = {}));
        function getFunctionFlags(node) {
            if (!node) {
                return 4 /* Invalid */;
            }
            var flags = 0 /* Normal */;
            switch (node.kind) {
                case 244 /* FunctionDeclaration */:
                case 201 /* FunctionExpression */:
                case 161 /* MethodDeclaration */:
                    if (node.asteriskToken) {
                        flags |= 1 /* Generator */;
                    }
                // falls through
                case 202 /* ArrowFunction */:
                    if (hasModifier(node, 256 /* Async */)) {
                        flags |= 2 /* Async */;
                    }
                    break;
            }
            if (!node.body) {
                flags |= 4 /* Invalid */;
            }
            return flags;
        }
        ts.getFunctionFlags = getFunctionFlags;
        function isAsyncFunction(node) {
            switch (node.kind) {
                case 244 /* FunctionDeclaration */:
                case 201 /* FunctionExpression */:
                case 202 /* ArrowFunction */:
                case 161 /* MethodDeclaration */:
                    return node.body !== undefined
                        && node.asteriskToken === undefined
                        && hasModifier(node, 256 /* Async */);
            }
            return false;
        }
        ts.isAsyncFunction = isAsyncFunction;
        function isStringOrNumericLiteralLike(node) {
            return ts.isStringLiteralLike(node) || ts.isNumericLiteral(node);
        }
        ts.isStringOrNumericLiteralLike = isStringOrNumericLiteralLike;
        function isSignedNumericLiteral(node) {
            return ts.isPrefixUnaryExpression(node) && (node.operator === 39 /* PlusToken */ || node.operator === 40 /* MinusToken */) && ts.isNumericLiteral(node.operand);
        }
        ts.isSignedNumericLiteral = isSignedNumericLiteral;
        /**
         * A declaration has a dynamic name if all of the following are true:
         *   1. The declaration has a computed property name.
         *   2. The computed name is *not* expressed as a StringLiteral.
         *   3. The computed name is *not* expressed as a NumericLiteral.
         *   4. The computed name is *not* expressed as a PlusToken or MinusToken
         *      immediately followed by a NumericLiteral.
         *   5. The computed name is *not* expressed as `Symbol.<name>`, where `<name>`
         *      is a property of the Symbol constructor that denotes a built-in
         *      Symbol.
         */
        function hasDynamicName(declaration) {
            var name = ts.getNameOfDeclaration(declaration);
            return !!name && isDynamicName(name);
        }
        ts.hasDynamicName = hasDynamicName;
        function isDynamicName(name) {
            if (!(name.kind === 154 /* ComputedPropertyName */ || name.kind === 195 /* ElementAccessExpression */)) {
                return false;
            }
            var expr = ts.isElementAccessExpression(name) ? name.argumentExpression : name.expression;
            return !isStringOrNumericLiteralLike(expr) &&
                !isSignedNumericLiteral(expr) &&
                !isWellKnownSymbolSyntactically(expr);
        }
        ts.isDynamicName = isDynamicName;
        /**
         * Checks if the expression is of the form:
         *    Symbol.name
         * where Symbol is literally the word "Symbol", and name is any identifierName
         */
        function isWellKnownSymbolSyntactically(node) {
            return ts.isPropertyAccessExpression(node) && isESSymbolIdentifier(node.expression);
        }
        ts.isWellKnownSymbolSyntactically = isWellKnownSymbolSyntactically;
        function getPropertyNameForPropertyNameNode(name) {
            switch (name.kind) {
                case 75 /* Identifier */:
                case 76 /* PrivateIdentifier */:
                    return name.escapedText;
                case 10 /* StringLiteral */:
                case 8 /* NumericLiteral */:
                    return ts.escapeLeadingUnderscores(name.text);
                case 154 /* ComputedPropertyName */:
                    var nameExpression = name.expression;
                    if (isWellKnownSymbolSyntactically(nameExpression)) {
                        return getPropertyNameForKnownSymbolName(ts.idText(nameExpression.name));
                    }
                    else if (isStringOrNumericLiteralLike(nameExpression)) {
                        return ts.escapeLeadingUnderscores(nameExpression.text);
                    }
                    return undefined;
                default:
                    return ts.Debug.assertNever(name);
            }
        }
        ts.getPropertyNameForPropertyNameNode = getPropertyNameForPropertyNameNode;
        function isPropertyNameLiteral(node) {
            switch (node.kind) {
                case 75 /* Identifier */:
                case 10 /* StringLiteral */:
                case 14 /* NoSubstitutionTemplateLiteral */:
                case 8 /* NumericLiteral */:
                    return true;
                default:
                    return false;
            }
        }
        ts.isPropertyNameLiteral = isPropertyNameLiteral;
        function getTextOfIdentifierOrLiteral(node) {
            return ts.isIdentifierOrPrivateIdentifier(node) ? ts.idText(node) : node.text;
        }
        ts.getTextOfIdentifierOrLiteral = getTextOfIdentifierOrLiteral;
        function getEscapedTextOfIdentifierOrLiteral(node) {
            return ts.isIdentifierOrPrivateIdentifier(node) ? node.escapedText : ts.escapeLeadingUnderscores(node.text);
        }
        ts.getEscapedTextOfIdentifierOrLiteral = getEscapedTextOfIdentifierOrLiteral;
        function getPropertyNameForUniqueESSymbol(symbol) {
            return "__@" + ts.getSymbolId(symbol) + "@" + symbol.escapedName;
        }
        ts.getPropertyNameForUniqueESSymbol = getPropertyNameForUniqueESSymbol;
        function getPropertyNameForKnownSymbolName(symbolName) {
            return "__@" + symbolName;
        }
        ts.getPropertyNameForKnownSymbolName = getPropertyNameForKnownSymbolName;
        function getSymbolNameForPrivateIdentifier(containingClassSymbol, description) {
            return "__#" + ts.getSymbolId(containingClassSymbol) + "@" + description;
        }
        ts.getSymbolNameForPrivateIdentifier = getSymbolNameForPrivateIdentifier;
        function isKnownSymbol(symbol) {
            return ts.startsWith(symbol.escapedName, "__@");
        }
        ts.isKnownSymbol = isKnownSymbol;
        /**
         * Includes the word "Symbol" with unicode escapes
         */
        function isESSymbolIdentifier(node) {
            return node.kind === 75 /* Identifier */ && node.escapedText === "Symbol";
        }
        ts.isESSymbolIdentifier = isESSymbolIdentifier;
        function isPushOrUnshiftIdentifier(node) {
            return node.escapedText === "push" || node.escapedText === "unshift";
        }
        ts.isPushOrUnshiftIdentifier = isPushOrUnshiftIdentifier;
        function isParameterDeclaration(node) {
            var root = getRootDeclaration(node);
            return root.kind === 156 /* Parameter */;
        }
        ts.isParameterDeclaration = isParameterDeclaration;
        function getRootDeclaration(node) {
            while (node.kind === 191 /* BindingElement */) {
                node = node.parent.parent;
            }
            return node;
        }
        ts.getRootDeclaration = getRootDeclaration;
        function nodeStartsNewLexicalEnvironment(node) {
            var kind = node.kind;
            return kind === 162 /* Constructor */
                || kind === 201 /* FunctionExpression */
                || kind === 244 /* FunctionDeclaration */
                || kind === 202 /* ArrowFunction */
                || kind === 161 /* MethodDeclaration */
                || kind === 163 /* GetAccessor */
                || kind === 164 /* SetAccessor */
                || kind === 249 /* ModuleDeclaration */
                || kind === 290 /* SourceFile */;
        }
        ts.nodeStartsNewLexicalEnvironment = nodeStartsNewLexicalEnvironment;
        function nodeIsSynthesized(range) {
            return positionIsSynthesized(range.pos)
                || positionIsSynthesized(range.end);
        }
        ts.nodeIsSynthesized = nodeIsSynthesized;
        function getOriginalSourceFile(sourceFile) {
            return ts.getParseTreeNode(sourceFile, ts.isSourceFile) || sourceFile;
        }
        ts.getOriginalSourceFile = getOriginalSourceFile;
        var Associativity;
        (function (Associativity) {
            Associativity[Associativity["Left"] = 0] = "Left";
            Associativity[Associativity["Right"] = 1] = "Right";
        })(Associativity = ts.Associativity || (ts.Associativity = {}));
        function getExpressionAssociativity(expression) {
            var operator = getOperator(expression);
            var hasArguments = expression.kind === 197 /* NewExpression */ && expression.arguments !== undefined;
            return getOperatorAssociativity(expression.kind, operator, hasArguments);
        }
        ts.getExpressionAssociativity = getExpressionAssociativity;
        function getOperatorAssociativity(kind, operator, hasArguments) {
            switch (kind) {
                case 197 /* NewExpression */:
                    return hasArguments ? 0 /* Left */ : 1 /* Right */;
                case 207 /* PrefixUnaryExpression */:
                case 204 /* TypeOfExpression */:
                case 205 /* VoidExpression */:
                case 203 /* DeleteExpression */:
                case 206 /* AwaitExpression */:
                case 210 /* ConditionalExpression */:
                case 212 /* YieldExpression */:
                    return 1 /* Right */;
                case 209 /* BinaryExpression */:
                    switch (operator) {
                        case 42 /* AsteriskAsteriskToken */:
                        case 62 /* EqualsToken */:
                        case 63 /* PlusEqualsToken */:
                        case 64 /* MinusEqualsToken */:
                        case 66 /* AsteriskAsteriskEqualsToken */:
                        case 65 /* AsteriskEqualsToken */:
                        case 67 /* SlashEqualsToken */:
                        case 68 /* PercentEqualsToken */:
                        case 69 /* LessThanLessThanEqualsToken */:
                        case 70 /* GreaterThanGreaterThanEqualsToken */:
                        case 71 /* GreaterThanGreaterThanGreaterThanEqualsToken */:
                        case 72 /* AmpersandEqualsToken */:
                        case 74 /* CaretEqualsToken */:
                        case 73 /* BarEqualsToken */:
                            return 1 /* Right */;
                    }
            }
            return 0 /* Left */;
        }
        ts.getOperatorAssociativity = getOperatorAssociativity;
        function getExpressionPrecedence(expression) {
            var operator = getOperator(expression);
            var hasArguments = expression.kind === 197 /* NewExpression */ && expression.arguments !== undefined;
            return getOperatorPrecedence(expression.kind, operator, hasArguments);
        }
        ts.getExpressionPrecedence = getExpressionPrecedence;
        function getOperator(expression) {
            if (expression.kind === 209 /* BinaryExpression */) {
                return expression.operatorToken.kind;
            }
            else if (expression.kind === 207 /* PrefixUnaryExpression */ || expression.kind === 208 /* PostfixUnaryExpression */) {
                return expression.operator;
            }
            else {
                return expression.kind;
            }
        }
        ts.getOperator = getOperator;
        function getOperatorPrecedence(nodeKind, operatorKind, hasArguments) {
            switch (nodeKind) {
                case 326 /* CommaListExpression */:
                    return 0;
                case 213 /* SpreadElement */:
                    return 1;
                case 212 /* YieldExpression */:
                    return 2;
                case 210 /* ConditionalExpression */:
                    return 4;
                case 209 /* BinaryExpression */:
                    switch (operatorKind) {
                        case 27 /* CommaToken */:
                            return 0;
                        case 62 /* EqualsToken */:
                        case 63 /* PlusEqualsToken */:
                        case 64 /* MinusEqualsToken */:
                        case 66 /* AsteriskAsteriskEqualsToken */:
                        case 65 /* AsteriskEqualsToken */:
                        case 67 /* SlashEqualsToken */:
                        case 68 /* PercentEqualsToken */:
                        case 69 /* LessThanLessThanEqualsToken */:
                        case 70 /* GreaterThanGreaterThanEqualsToken */:
                        case 71 /* GreaterThanGreaterThanGreaterThanEqualsToken */:
                        case 72 /* AmpersandEqualsToken */:
                        case 74 /* CaretEqualsToken */:
                        case 73 /* BarEqualsToken */:
                            return 3;
                        default:
                            return getBinaryOperatorPrecedence(operatorKind);
                    }
                case 207 /* PrefixUnaryExpression */:
                case 204 /* TypeOfExpression */:
                case 205 /* VoidExpression */:
                case 203 /* DeleteExpression */:
                case 206 /* AwaitExpression */:
                    return 16;
                case 208 /* PostfixUnaryExpression */:
                    return 17;
                case 196 /* CallExpression */:
                    return 18;
                case 197 /* NewExpression */:
                    return hasArguments ? 19 : 18;
                case 198 /* TaggedTemplateExpression */:
                case 194 /* PropertyAccessExpression */:
                case 195 /* ElementAccessExpression */:
                    return 19;
                case 104 /* ThisKeyword */:
                case 102 /* SuperKeyword */:
                case 75 /* Identifier */:
                case 100 /* NullKeyword */:
                case 106 /* TrueKeyword */:
                case 91 /* FalseKeyword */:
                case 8 /* NumericLiteral */:
                case 9 /* BigIntLiteral */:
                case 10 /* StringLiteral */:
                case 192 /* ArrayLiteralExpression */:
                case 193 /* ObjectLiteralExpression */:
                case 201 /* FunctionExpression */:
                case 202 /* ArrowFunction */:
                case 214 /* ClassExpression */:
                case 266 /* JsxElement */:
                case 267 /* JsxSelfClosingElement */:
                case 270 /* JsxFragment */:
                case 13 /* RegularExpressionLiteral */:
                case 14 /* NoSubstitutionTemplateLiteral */:
                case 211 /* TemplateExpression */:
                case 200 /* ParenthesizedExpression */:
                case 215 /* OmittedExpression */:
                    return 20;
                default:
                    return -1;
            }
        }
        ts.getOperatorPrecedence = getOperatorPrecedence;
        function getBinaryOperatorPrecedence(kind) {
            switch (kind) {
                case 60 /* QuestionQuestionToken */:
                    return 4;
                case 56 /* BarBarToken */:
                    return 5;
                case 55 /* AmpersandAmpersandToken */:
                    return 6;
                case 51 /* BarToken */:
                    return 7;
                case 52 /* CaretToken */:
                    return 8;
                case 50 /* AmpersandToken */:
                    return 9;
                case 34 /* EqualsEqualsToken */:
                case 35 /* ExclamationEqualsToken */:
                case 36 /* EqualsEqualsEqualsToken */:
                case 37 /* ExclamationEqualsEqualsToken */:
                    return 10;
                case 29 /* LessThanToken */:
                case 31 /* GreaterThanToken */:
                case 32 /* LessThanEqualsToken */:
                case 33 /* GreaterThanEqualsToken */:
                case 98 /* InstanceOfKeyword */:
                case 97 /* InKeyword */:
                case 123 /* AsKeyword */:
                    return 11;
                case 47 /* LessThanLessThanToken */:
                case 48 /* GreaterThanGreaterThanToken */:
                case 49 /* GreaterThanGreaterThanGreaterThanToken */:
                    return 12;
                case 39 /* PlusToken */:
                case 40 /* MinusToken */:
                    return 13;
                case 41 /* AsteriskToken */:
                case 43 /* SlashToken */:
                case 44 /* PercentToken */:
                    return 14;
                case 42 /* AsteriskAsteriskToken */:
                    return 15;
            }
            // -1 is lower than all other precedences.  Returning it will cause binary expression
            // parsing to stop.
            return -1;
        }
        ts.getBinaryOperatorPrecedence = getBinaryOperatorPrecedence;
        function createDiagnosticCollection() {
            var nonFileDiagnostics = []; // See GH#19873
            var filesWithDiagnostics = [];
            var fileDiagnostics = ts.createMap();
            var hasReadNonFileDiagnostics = false;
            return {
                add: add,
                lookup: lookup,
                getGlobalDiagnostics: getGlobalDiagnostics,
                getDiagnostics: getDiagnostics,
                reattachFileDiagnostics: reattachFileDiagnostics
            };
            function reattachFileDiagnostics(newFile) {
                ts.forEach(fileDiagnostics.get(newFile.fileName), function (diagnostic) { return diagnostic.file = newFile; });
            }
            function lookup(diagnostic) {
                var diagnostics;
                if (diagnostic.file) {
                    diagnostics = fileDiagnostics.get(diagnostic.file.fileName);
                }
                else {
                    diagnostics = nonFileDiagnostics;
                }
                if (!diagnostics) {
                    return undefined;
                }
                var result = ts.binarySearch(diagnostics, diagnostic, ts.identity, compareDiagnosticsSkipRelatedInformation);
                if (result >= 0) {
                    return diagnostics[result];
                }
                return undefined;
            }
            function add(diagnostic) {
                var diagnostics;
                if (diagnostic.file) {
                    diagnostics = fileDiagnostics.get(diagnostic.file.fileName);
                    if (!diagnostics) {
                        diagnostics = []; // See GH#19873
                        fileDiagnostics.set(diagnostic.file.fileName, diagnostics);
                        ts.insertSorted(filesWithDiagnostics, diagnostic.file.fileName, ts.compareStringsCaseSensitive);
                    }
                }
                else {
                    // If we've already read the non-file diagnostics, do not modify the existing array.
                    if (hasReadNonFileDiagnostics) {
                        hasReadNonFileDiagnostics = false;
                        nonFileDiagnostics = nonFileDiagnostics.slice();
                    }
                    diagnostics = nonFileDiagnostics;
                }
                ts.insertSorted(diagnostics, diagnostic, compareDiagnostics);
            }
            function getGlobalDiagnostics() {
                hasReadNonFileDiagnostics = true;
                return nonFileDiagnostics;
            }
            function getDiagnostics(fileName) {
                if (fileName) {
                    return fileDiagnostics.get(fileName) || [];
                }
                var fileDiags = ts.flatMapToMutable(filesWithDiagnostics, function (f) { return fileDiagnostics.get(f); });
                if (!nonFileDiagnostics.length) {
                    return fileDiags;
                }
                fileDiags.unshift.apply(fileDiags, nonFileDiagnostics);
                return fileDiags;
            }
        }
        ts.createDiagnosticCollection = createDiagnosticCollection;
        var templateSubstitutionRegExp = /\$\{/g;
        function escapeTemplateSubstitution(str) {
            return str.replace(templateSubstitutionRegExp, "\\${");
        }
        // This consists of the first 19 unprintable ASCII characters, canonical escapes, lineSeparator,
        // paragraphSeparator, and nextLine. The latter three are just desirable to suppress new lines in
        // the language service. These characters should be escaped when printing, and if any characters are added,
        // the map below must be updated. Note that this regexp *does not* include the 'delete' character.
        // There is no reason for this other than that JSON.stringify does not handle it either.
        var doubleQuoteEscapedCharsRegExp = /[\\\"\u0000-\u001f\t\v\f\b\r\n\u2028\u2029\u0085]/g;
        var singleQuoteEscapedCharsRegExp = /[\\\'\u0000-\u001f\t\v\f\b\r\n\u2028\u2029\u0085]/g;
        // Template strings should be preserved as much as possible
        var backtickQuoteEscapedCharsRegExp = /[\\\`]/g;
        var escapedCharsMap = ts.createMapFromTemplate({
            "\t": "\\t",
            "\v": "\\v",
            "\f": "\\f",
            "\b": "\\b",
            "\r": "\\r",
            "\n": "\\n",
            "\\": "\\\\",
            "\"": "\\\"",
            "\'": "\\\'",
            "\`": "\\\`",
            "\u2028": "\\u2028",
            "\u2029": "\\u2029",
            "\u0085": "\\u0085" // nextLine
        });
        function encodeUtf16EscapeSequence(charCode) {
            var hexCharCode = charCode.toString(16).toUpperCase();
            var paddedHexCode = ("0000" + hexCharCode).slice(-4);
            return "\\u" + paddedHexCode;
        }
        function getReplacement(c, offset, input) {
            if (c.charCodeAt(0) === 0 /* nullCharacter */) {
                var lookAhead = input.charCodeAt(offset + c.length);
                if (lookAhead >= 48 /* _0 */ && lookAhead <= 57 /* _9 */) {
                    // If the null character is followed by digits, print as a hex escape to prevent the result from parsing as an octal (which is forbidden in strict mode)
                    return "\\x00";
                }
                // Otherwise, keep printing a literal \0 for the null character
                return "\\0";
            }
            return escapedCharsMap.get(c) || encodeUtf16EscapeSequence(c.charCodeAt(0));
        }
        /**
         * Based heavily on the abstract 'Quote'/'QuoteJSONString' operation from ECMA-262 (24.3.2.2),
         * but augmented for a few select characters (e.g. lineSeparator, paragraphSeparator, nextLine)
         * Note that this doesn't actually wrap the input in double quotes.
         */
        function escapeString(s, quoteChar) {
            var escapedCharsRegExp = quoteChar === 96 /* backtick */ ? backtickQuoteEscapedCharsRegExp :
                quoteChar === 39 /* singleQuote */ ? singleQuoteEscapedCharsRegExp :
                    doubleQuoteEscapedCharsRegExp;
            return s.replace(escapedCharsRegExp, getReplacement);
        }
        ts.escapeString = escapeString;
        var nonAsciiCharacters = /[^\u0000-\u007F]/g;
        function escapeNonAsciiString(s, quoteChar) {
            s = escapeString(s, quoteChar);
            // Replace non-ASCII characters with '\uNNNN' escapes if any exist.
            // Otherwise just return the original string.
            return nonAsciiCharacters.test(s) ?
                s.replace(nonAsciiCharacters, function (c) { return encodeUtf16EscapeSequence(c.charCodeAt(0)); }) :
                s;
        }
        ts.escapeNonAsciiString = escapeNonAsciiString;
        // This consists of the first 19 unprintable ASCII characters, JSX canonical escapes, lineSeparator,
        // paragraphSeparator, and nextLine. The latter three are just desirable to suppress new lines in
        // the language service. These characters should be escaped when printing, and if any characters are added,
        // the map below must be updated.
        var jsxDoubleQuoteEscapedCharsRegExp = /[\"\u0000-\u001f\u2028\u2029\u0085]/g;
        var jsxSingleQuoteEscapedCharsRegExp = /[\'\u0000-\u001f\u2028\u2029\u0085]/g;
        var jsxEscapedCharsMap = ts.createMapFromTemplate({
            "\"": "&quot;",
            "\'": "&apos;"
        });
        function encodeJsxCharacterEntity(charCode) {
            var hexCharCode = charCode.toString(16).toUpperCase();
            return "&#x" + hexCharCode + ";";
        }
        function getJsxAttributeStringReplacement(c) {
            if (c.charCodeAt(0) === 0 /* nullCharacter */) {
                return "&#0;";
            }
            return jsxEscapedCharsMap.get(c) || encodeJsxCharacterEntity(c.charCodeAt(0));
        }
        function escapeJsxAttributeString(s, quoteChar) {
            var escapedCharsRegExp = quoteChar === 39 /* singleQuote */ ? jsxSingleQuoteEscapedCharsRegExp :
                jsxDoubleQuoteEscapedCharsRegExp;
            return s.replace(escapedCharsRegExp, getJsxAttributeStringReplacement);
        }
        ts.escapeJsxAttributeString = escapeJsxAttributeString;
        /**
         * Strip off existed surrounding single quotes, double quotes, or backticks from a given string
         *
         * @return non-quoted string
         */
        function stripQuotes(name) {
            var length = name.length;
            if (length >= 2 && name.charCodeAt(0) === name.charCodeAt(length - 1) && isQuoteOrBacktick(name.charCodeAt(0))) {
                return name.substring(1, length - 1);
            }
            return name;
        }
        ts.stripQuotes = stripQuotes;
        function isQuoteOrBacktick(charCode) {
            return charCode === 39 /* singleQuote */ ||
                charCode === 34 /* doubleQuote */ ||
                charCode === 96 /* backtick */;
        }
        function isIntrinsicJsxName(name) {
            var ch = name.charCodeAt(0);
            return (ch >= 97 /* a */ && ch <= 122 /* z */) || ts.stringContains(name, "-");
        }
        ts.isIntrinsicJsxName = isIntrinsicJsxName;
        var indentStrings = ["", "    "];
        function getIndentString(level) {
            if (indentStrings[level] === undefined) {
                indentStrings[level] = getIndentString(level - 1) + indentStrings[1];
            }
            return indentStrings[level];
        }
        ts.getIndentString = getIndentString;
        function getIndentSize() {
            return indentStrings[1].length;
        }
        ts.getIndentSize = getIndentSize;
        function createTextWriter(newLine) {
            var output;
            var indent;
            var lineStart;
            var lineCount;
            var linePos;
            var hasTrailingComment = false;
            function updateLineCountAndPosFor(s) {
                var lineStartsOfS = ts.computeLineStarts(s);
                if (lineStartsOfS.length > 1) {
                    lineCount = lineCount + lineStartsOfS.length - 1;
                    linePos = output.length - s.length + ts.last(lineStartsOfS);
                    lineStart = (linePos - output.length) === 0;
                }
                else {
                    lineStart = false;
                }
            }
            function writeText(s) {
                if (s && s.length) {
                    if (lineStart) {
                        s = getIndentString(indent) + s;
                        lineStart = false;
                    }
                    output += s;
                    updateLineCountAndPosFor(s);
                }
            }
            function write(s) {
                if (s)
                    hasTrailingComment = false;
                writeText(s);
            }
            function writeComment(s) {
                if (s)
                    hasTrailingComment = true;
                writeText(s);
            }
            function reset() {
                output = "";
                indent = 0;
                lineStart = true;
                lineCount = 0;
                linePos = 0;
                hasTrailingComment = false;
            }
            function rawWrite(s) {
                if (s !== undefined) {
                    output += s;
                    updateLineCountAndPosFor(s);
                    hasTrailingComment = false;
                }
            }
            function writeLiteral(s) {
                if (s && s.length) {
                    write(s);
                }
            }
            function writeLine() {
                if (!lineStart) {
                    output += newLine;
                    lineCount++;
                    linePos = output.length;
                    lineStart = true;
                    hasTrailingComment = false;
                }
            }
            function getTextPosWithWriteLine() {
                return lineStart ? output.length : (output.length + newLine.length);
            }
            reset();
            return {
                write: write,
                rawWrite: rawWrite,
                writeLiteral: writeLiteral,
                writeLine: writeLine,
                increaseIndent: function () { indent++; },
                decreaseIndent: function () { indent--; },
                getIndent: function () { return indent; },
                getTextPos: function () { return output.length; },
                getLine: function () { return lineCount; },
                getColumn: function () { return lineStart ? indent * getIndentSize() : output.length - linePos; },
                getText: function () { return output; },
                isAtStartOfLine: function () { return lineStart; },
                hasTrailingComment: function () { return hasTrailingComment; },
                hasTrailingWhitespace: function () { return !!output.length && ts.isWhiteSpaceLike(output.charCodeAt(output.length - 1)); },
                clear: reset,
                reportInaccessibleThisError: ts.noop,
                reportPrivateInBaseOfClassExpression: ts.noop,
                reportInaccessibleUniqueSymbolError: ts.noop,
                trackSymbol: ts.noop,
                writeKeyword: write,
                writeOperator: write,
                writeParameter: write,
                writeProperty: write,
                writePunctuation: write,
                writeSpace: write,
                writeStringLiteral: write,
                writeSymbol: function (s, _) { return write(s); },
                writeTrailingSemicolon: write,
                writeComment: writeComment,
                getTextPosWithWriteLine: getTextPosWithWriteLine
            };
        }
        ts.createTextWriter = createTextWriter;
        function getTrailingSemicolonDeferringWriter(writer) {
            var pendingTrailingSemicolon = false;
            function commitPendingTrailingSemicolon() {
                if (pendingTrailingSemicolon) {
                    writer.writeTrailingSemicolon(";");
                    pendingTrailingSemicolon = false;
                }
            }
            return __assign(__assign({}, writer), { writeTrailingSemicolon: function () {
                    pendingTrailingSemicolon = true;
                },
                writeLiteral: function (s) {
                    commitPendingTrailingSemicolon();
                    writer.writeLiteral(s);
                },
                writeStringLiteral: function (s) {
                    commitPendingTrailingSemicolon();
                    writer.writeStringLiteral(s);
                },
                writeSymbol: function (s, sym) {
                    commitPendingTrailingSemicolon();
                    writer.writeSymbol(s, sym);
                },
                writePunctuation: function (s) {
                    commitPendingTrailingSemicolon();
                    writer.writePunctuation(s);
                },
                writeKeyword: function (s) {
                    commitPendingTrailingSemicolon();
                    writer.writeKeyword(s);
                },
                writeOperator: function (s) {
                    commitPendingTrailingSemicolon();
                    writer.writeOperator(s);
                },
                writeParameter: function (s) {
                    commitPendingTrailingSemicolon();
                    writer.writeParameter(s);
                },
                writeSpace: function (s) {
                    commitPendingTrailingSemicolon();
                    writer.writeSpace(s);
                },
                writeProperty: function (s) {
                    commitPendingTrailingSemicolon();
                    writer.writeProperty(s);
                },
                writeComment: function (s) {
                    commitPendingTrailingSemicolon();
                    writer.writeComment(s);
                },
                writeLine: function () {
                    commitPendingTrailingSemicolon();
                    writer.writeLine();
                },
                increaseIndent: function () {
                    commitPendingTrailingSemicolon();
                    writer.increaseIndent();
                },
                decreaseIndent: function () {
                    commitPendingTrailingSemicolon();
                    writer.decreaseIndent();
                } });
        }
        ts.getTrailingSemicolonDeferringWriter = getTrailingSemicolonDeferringWriter;
        function getResolvedExternalModuleName(host, file, referenceFile) {
            return file.moduleName || getExternalModuleNameFromPath(host, file.fileName, referenceFile && referenceFile.fileName);
        }
        ts.getResolvedExternalModuleName = getResolvedExternalModuleName;
        function getExternalModuleNameFromDeclaration(host, resolver, declaration) {
            var file = resolver.getExternalModuleFileFromDeclaration(declaration);
            if (!file || file.isDeclarationFile) {
                return undefined;
            }
            return getResolvedExternalModuleName(host, file);
        }
        ts.getExternalModuleNameFromDeclaration = getExternalModuleNameFromDeclaration;
        /**
         * Resolves a local path to a path which is absolute to the base of the emit
         */
        function getExternalModuleNameFromPath(host, fileName, referencePath) {
            var getCanonicalFileName = function (f) { return host.getCanonicalFileName(f); };
            var dir = ts.toPath(referencePath ? ts.getDirectoryPath(referencePath) : host.getCommonSourceDirectory(), host.getCurrentDirectory(), getCanonicalFileName);
            var filePath = ts.getNormalizedAbsolutePath(fileName, host.getCurrentDirectory());
            var relativePath = ts.getRelativePathToDirectoryOrUrl(dir, filePath, dir, getCanonicalFileName, /*isAbsolutePathAnUrl*/ false);
            var extensionless = removeFileExtension(relativePath);
            return referencePath ? ts.ensurePathIsNonModuleName(extensionless) : extensionless;
        }
        ts.getExternalModuleNameFromPath = getExternalModuleNameFromPath;
        function getOwnEmitOutputFilePath(fileName, host, extension) {
            var compilerOptions = host.getCompilerOptions();
            var emitOutputFilePathWithoutExtension;
            if (compilerOptions.outDir) {
                emitOutputFilePathWithoutExtension = removeFileExtension(getSourceFilePathInNewDir(fileName, host, compilerOptions.outDir));
            }
            else {
                emitOutputFilePathWithoutExtension = removeFileExtension(fileName);
            }
            return emitOutputFilePathWithoutExtension + extension;
        }
        ts.getOwnEmitOutputFilePath = getOwnEmitOutputFilePath;
        function getDeclarationEmitOutputFilePath(fileName, host) {
            return getDeclarationEmitOutputFilePathWorker(fileName, host.getCompilerOptions(), host.getCurrentDirectory(), host.getCommonSourceDirectory(), function (f) { return host.getCanonicalFileName(f); });
        }
        ts.getDeclarationEmitOutputFilePath = getDeclarationEmitOutputFilePath;
        function getDeclarationEmitOutputFilePathWorker(fileName, options, currentDirectory, commonSourceDirectory, getCanonicalFileName) {
            var outputDir = options.declarationDir || options.outDir; // Prefer declaration folder if specified
            var path = outputDir
                ? getSourceFilePathInNewDirWorker(fileName, outputDir, currentDirectory, commonSourceDirectory, getCanonicalFileName)
                : fileName;
            return removeFileExtension(path) + ".d.ts" /* Dts */;
        }
        ts.getDeclarationEmitOutputFilePathWorker = getDeclarationEmitOutputFilePathWorker;
        /**
         * Gets the source files that are expected to have an emit output.
         *
         * Originally part of `forEachExpectedEmitFile`, this functionality was extracted to support
         * transformations.
         *
         * @param host An EmitHost.
         * @param targetSourceFile An optional target source file to emit.
         */
        function getSourceFilesToEmit(host, targetSourceFile, forceDtsEmit) {
            var options = host.getCompilerOptions();
            if (options.outFile || options.out) {
                var moduleKind = getEmitModuleKind(options);
                var moduleEmitEnabled_1 = options.emitDeclarationOnly || moduleKind === ts.ModuleKind.AMD || moduleKind === ts.ModuleKind.System;
                // Can emit only sources that are not declaration file and are either non module code or module with --module or --target es6 specified
                return ts.filter(host.getSourceFiles(), function (sourceFile) {
                    return (moduleEmitEnabled_1 || !ts.isExternalModule(sourceFile)) &&
                        sourceFileMayBeEmitted(sourceFile, host, forceDtsEmit);
                });
            }
            else {
                var sourceFiles = targetSourceFile === undefined ? host.getSourceFiles() : [targetSourceFile];
                return ts.filter(sourceFiles, function (sourceFile) { return sourceFileMayBeEmitted(sourceFile, host, forceDtsEmit); });
            }
        }
        ts.getSourceFilesToEmit = getSourceFilesToEmit;
        /** Don't call this for `--outFile`, just for `--outDir` or plain emit. `--outFile` needs additional checks. */
        function sourceFileMayBeEmitted(sourceFile, host, forceDtsEmit) {
            var options = host.getCompilerOptions();
            return !(options.noEmitForJsFiles && isSourceFileJS(sourceFile)) &&
                !sourceFile.isDeclarationFile &&
                !host.isSourceFileFromExternalLibrary(sourceFile) &&
                !(isJsonSourceFile(sourceFile) && host.getResolvedProjectReferenceToRedirect(sourceFile.fileName)) &&
                (forceDtsEmit || !host.isSourceOfProjectReferenceRedirect(sourceFile.fileName));
        }
        ts.sourceFileMayBeEmitted = sourceFileMayBeEmitted;
        function getSourceFilePathInNewDir(fileName, host, newDirPath) {
            return getSourceFilePathInNewDirWorker(fileName, newDirPath, host.getCurrentDirectory(), host.getCommonSourceDirectory(), function (f) { return host.getCanonicalFileName(f); });
        }
        ts.getSourceFilePathInNewDir = getSourceFilePathInNewDir;
        function getSourceFilePathInNewDirWorker(fileName, newDirPath, currentDirectory, commonSourceDirectory, getCanonicalFileName) {
            var sourceFilePath = ts.getNormalizedAbsolutePath(fileName, currentDirectory);
            var isSourceFileInCommonSourceDirectory = getCanonicalFileName(sourceFilePath).indexOf(getCanonicalFileName(commonSourceDirectory)) === 0;
            sourceFilePath = isSourceFileInCommonSourceDirectory ? sourceFilePath.substring(commonSourceDirectory.length) : sourceFilePath;
            return ts.combinePaths(newDirPath, sourceFilePath);
        }
        ts.getSourceFilePathInNewDirWorker = getSourceFilePathInNewDirWorker;
        function writeFile(host, diagnostics, fileName, data, writeByteOrderMark, sourceFiles) {
            host.writeFile(fileName, data, writeByteOrderMark, function (hostErrorMessage) {
                diagnostics.add(createCompilerDiagnostic(ts.Diagnostics.Could_not_write_file_0_Colon_1, fileName, hostErrorMessage));
            }, sourceFiles);
        }
        ts.writeFile = writeFile;
        function ensureDirectoriesExist(directoryPath, createDirectory, directoryExists) {
            if (directoryPath.length > ts.getRootLength(directoryPath) && !directoryExists(directoryPath)) {
                var parentDirectory = ts.getDirectoryPath(directoryPath);
                ensureDirectoriesExist(parentDirectory, createDirectory, directoryExists);
                createDirectory(directoryPath);
            }
        }
        function writeFileEnsuringDirectories(path, data, writeByteOrderMark, writeFile, createDirectory, directoryExists) {
            // PERF: Checking for directory existence is expensive.  Instead, assume the directory exists
            // and fall back to creating it if the file write fails.
            try {
                writeFile(path, data, writeByteOrderMark);
            }
            catch (_a) {
                ensureDirectoriesExist(ts.getDirectoryPath(ts.normalizePath(path)), createDirectory, directoryExists);
                writeFile(path, data, writeByteOrderMark);
            }
        }
        ts.writeFileEnsuringDirectories = writeFileEnsuringDirectories;
        function getLineOfLocalPosition(currentSourceFile, pos) {
            return ts.getLineAndCharacterOfPosition(currentSourceFile, pos).line;
        }
        ts.getLineOfLocalPosition = getLineOfLocalPosition;
        function getLineOfLocalPositionFromLineMap(lineMap, pos) {
            return ts.computeLineAndCharacterOfPosition(lineMap, pos).line;
        }
        ts.getLineOfLocalPositionFromLineMap = getLineOfLocalPositionFromLineMap;
        function getFirstConstructorWithBody(node) {
            return ts.find(node.members, function (member) { return ts.isConstructorDeclaration(member) && nodeIsPresent(member.body); });
        }
        ts.getFirstConstructorWithBody = getFirstConstructorWithBody;
        function getSetAccessorValueParameter(accessor) {
            if (accessor && accessor.parameters.length > 0) {
                var hasThis = accessor.parameters.length === 2 && parameterIsThisKeyword(accessor.parameters[0]);
                return accessor.parameters[hasThis ? 1 : 0];
            }
        }
        ts.getSetAccessorValueParameter = getSetAccessorValueParameter;
        /** Get the type annotation for the value parameter. */
        function getSetAccessorTypeAnnotationNode(accessor) {
            var parameter = getSetAccessorValueParameter(accessor);
            return parameter && parameter.type;
        }
        ts.getSetAccessorTypeAnnotationNode = getSetAccessorTypeAnnotationNode;
        function getThisParameter(signature) {
            // callback tags do not currently support this parameters
            if (signature.parameters.length && !ts.isJSDocSignature(signature)) {
                var thisParameter = signature.parameters[0];
                if (parameterIsThisKeyword(thisParameter)) {
                    return thisParameter;
                }
            }
        }
        ts.getThisParameter = getThisParameter;
        function parameterIsThisKeyword(parameter) {
            return isThisIdentifier(parameter.name);
        }
        ts.parameterIsThisKeyword = parameterIsThisKeyword;
        function isThisIdentifier(node) {
            return !!node && node.kind === 75 /* Identifier */ && identifierIsThisKeyword(node);
        }
        ts.isThisIdentifier = isThisIdentifier;
        function identifierIsThisKeyword(id) {
            return id.originalKeywordKind === 104 /* ThisKeyword */;
        }
        ts.identifierIsThisKeyword = identifierIsThisKeyword;
        function getAllAccessorDeclarations(declarations, accessor) {
            // TODO: GH#18217
            var firstAccessor;
            var secondAccessor;
            var getAccessor;
            var setAccessor;
            if (hasDynamicName(accessor)) {
                firstAccessor = accessor;
                if (accessor.kind === 163 /* GetAccessor */) {
                    getAccessor = accessor;
                }
                else if (accessor.kind === 164 /* SetAccessor */) {
                    setAccessor = accessor;
                }
                else {
                    ts.Debug.fail("Accessor has wrong kind");
                }
            }
            else {
                ts.forEach(declarations, function (member) {
                    if (ts.isAccessor(member)
                        && hasModifier(member, 32 /* Static */) === hasModifier(accessor, 32 /* Static */)) {
                        var memberName = getPropertyNameForPropertyNameNode(member.name);
                        var accessorName = getPropertyNameForPropertyNameNode(accessor.name);
                        if (memberName === accessorName) {
                            if (!firstAccessor) {
                                firstAccessor = member;
                            }
                            else if (!secondAccessor) {
                                secondAccessor = member;
                            }
                            if (member.kind === 163 /* GetAccessor */ && !getAccessor) {
                                getAccessor = member;
                            }
                            if (member.kind === 164 /* SetAccessor */ && !setAccessor) {
                                setAccessor = member;
                            }
                        }
                    }
                });
            }
            return {
                firstAccessor: firstAccessor,
                secondAccessor: secondAccessor,
                getAccessor: getAccessor,
                setAccessor: setAccessor
            };
        }
        ts.getAllAccessorDeclarations = getAllAccessorDeclarations;
        /**
         * Gets the effective type annotation of a variable, parameter, or property. If the node was
         * parsed in a JavaScript file, gets the type annotation from JSDoc.  Also gets the type of
         * functions only the JSDoc case.
         */
        function getEffectiveTypeAnnotationNode(node) {
            if (!isInJSFile(node) && ts.isFunctionDeclaration(node))
                return undefined;
            var type = node.type;
            if (type || !isInJSFile(node))
                return type;
            return ts.isJSDocPropertyLikeTag(node) ? node.typeExpression && node.typeExpression.type : ts.getJSDocType(node);
        }
        ts.getEffectiveTypeAnnotationNode = getEffectiveTypeAnnotationNode;
        function getTypeAnnotationNode(node) {
            return node.type;
        }
        ts.getTypeAnnotationNode = getTypeAnnotationNode;
        /**
         * Gets the effective return type annotation of a signature. If the node was parsed in a
         * JavaScript file, gets the return type annotation from JSDoc.
         */
        function getEffectiveReturnTypeNode(node) {
            return ts.isJSDocSignature(node) ?
                node.type && node.type.typeExpression && node.type.typeExpression.type :
                node.type || (isInJSFile(node) ? ts.getJSDocReturnType(node) : undefined);
        }
        ts.getEffectiveReturnTypeNode = getEffectiveReturnTypeNode;
        function getJSDocTypeParameterDeclarations(node) {
            return ts.flatMap(ts.getJSDocTags(node), function (tag) { return isNonTypeAliasTemplate(tag) ? tag.typeParameters : undefined; });
        }
        ts.getJSDocTypeParameterDeclarations = getJSDocTypeParameterDeclarations;
        /** template tags are only available when a typedef isn't already using them */
        function isNonTypeAliasTemplate(tag) {
            return ts.isJSDocTemplateTag(tag) && !(tag.parent.kind === 303 /* JSDocComment */ && tag.parent.tags.some(isJSDocTypeAlias));
        }
        /**
         * Gets the effective type annotation of the value parameter of a set accessor. If the node
         * was parsed in a JavaScript file, gets the type annotation from JSDoc.
         */
        function getEffectiveSetAccessorTypeAnnotationNode(node) {
            var parameter = getSetAccessorValueParameter(node);
            return parameter && getEffectiveTypeAnnotationNode(parameter);
        }
        ts.getEffectiveSetAccessorTypeAnnotationNode = getEffectiveSetAccessorTypeAnnotationNode;
        function emitNewLineBeforeLeadingComments(lineMap, writer, node, leadingComments) {
            emitNewLineBeforeLeadingCommentsOfPosition(lineMap, writer, node.pos, leadingComments);
        }
        ts.emitNewLineBeforeLeadingComments = emitNewLineBeforeLeadingComments;
        function emitNewLineBeforeLeadingCommentsOfPosition(lineMap, writer, pos, leadingComments) {
            // If the leading comments start on different line than the start of node, write new line
            if (leadingComments && leadingComments.length && pos !== leadingComments[0].pos &&
                getLineOfLocalPositionFromLineMap(lineMap, pos) !== getLineOfLocalPositionFromLineMap(lineMap, leadingComments[0].pos)) {
                writer.writeLine();
            }
        }
        ts.emitNewLineBeforeLeadingCommentsOfPosition = emitNewLineBeforeLeadingCommentsOfPosition;
        function emitNewLineBeforeLeadingCommentOfPosition(lineMap, writer, pos, commentPos) {
            // If the leading comments start on different line than the start of node, write new line
            if (pos !== commentPos &&
                getLineOfLocalPositionFromLineMap(lineMap, pos) !== getLineOfLocalPositionFromLineMap(lineMap, commentPos)) {
                writer.writeLine();
            }
        }
        ts.emitNewLineBeforeLeadingCommentOfPosition = emitNewLineBeforeLeadingCommentOfPosition;
        function emitComments(text, lineMap, writer, comments, leadingSeparator, trailingSeparator, newLine, writeComment) {
            if (comments && comments.length > 0) {
                if (leadingSeparator) {
                    writer.writeSpace(" ");
                }
                var emitInterveningSeparator = false;
                for (var _i = 0, comments_1 = comments; _i < comments_1.length; _i++) {
                    var comment = comments_1[_i];
                    if (emitInterveningSeparator) {
                        writer.writeSpace(" ");
                        emitInterveningSeparator = false;
                    }
                    writeComment(text, lineMap, writer, comment.pos, comment.end, newLine);
                    if (comment.hasTrailingNewLine) {
                        writer.writeLine();
                    }
                    else {
                        emitInterveningSeparator = true;
                    }
                }
                if (emitInterveningSeparator && trailingSeparator) {
                    writer.writeSpace(" ");
                }
            }
        }
        ts.emitComments = emitComments;
        /**
         * Detached comment is a comment at the top of file or function body that is separated from
         * the next statement by space.
         */
        function emitDetachedComments(text, lineMap, writer, writeComment, node, newLine, removeComments) {
            var leadingComments;
            var currentDetachedCommentInfo;
            if (removeComments) {
                // removeComments is true, only reserve pinned comment at the top of file
                // For example:
                //      /*! Pinned Comment */
                //
                //      var x = 10;
                if (node.pos === 0) {
                    leadingComments = ts.filter(ts.getLeadingCommentRanges(text, node.pos), isPinnedCommentLocal);
                }
            }
            else {
                // removeComments is false, just get detached as normal and bypass the process to filter comment
                leadingComments = ts.getLeadingCommentRanges(text, node.pos);
            }
            if (leadingComments) {
                var detachedComments = [];
                var lastComment = void 0;
                for (var _i = 0, leadingComments_1 = leadingComments; _i < leadingComments_1.length; _i++) {
                    var comment = leadingComments_1[_i];
                    if (lastComment) {
                        var lastCommentLine = getLineOfLocalPositionFromLineMap(lineMap, lastComment.end);
                        var commentLine = getLineOfLocalPositionFromLineMap(lineMap, comment.pos);
                        if (commentLine >= lastCommentLine + 2) {
                            // There was a blank line between the last comment and this comment.  This
                            // comment is not part of the copyright comments.  Return what we have so
                            // far.
                            break;
                        }
                    }
                    detachedComments.push(comment);
                    lastComment = comment;
                }
                if (detachedComments.length) {
                    // All comments look like they could have been part of the copyright header.  Make
                    // sure there is at least one blank line between it and the node.  If not, it's not
                    // a copyright header.
                    var lastCommentLine = getLineOfLocalPositionFromLineMap(lineMap, ts.last(detachedComments).end);
                    var nodeLine = getLineOfLocalPositionFromLineMap(lineMap, ts.skipTrivia(text, node.pos));
                    if (nodeLine >= lastCommentLine + 2) {
                        // Valid detachedComments
                        emitNewLineBeforeLeadingComments(lineMap, writer, node, leadingComments);
                        emitComments(text, lineMap, writer, detachedComments, /*leadingSeparator*/ false, /*trailingSeparator*/ true, newLine, writeComment);
                        currentDetachedCommentInfo = { nodePos: node.pos, detachedCommentEndPos: ts.last(detachedComments).end };
                    }
                }
            }
            return currentDetachedCommentInfo;
            function isPinnedCommentLocal(comment) {
                return isPinnedComment(text, comment.pos);
            }
        }
        ts.emitDetachedComments = emitDetachedComments;
        function writeCommentRange(text, lineMap, writer, commentPos, commentEnd, newLine) {
            if (text.charCodeAt(commentPos + 1) === 42 /* asterisk */) {
                var firstCommentLineAndCharacter = ts.computeLineAndCharacterOfPosition(lineMap, commentPos);
                var lineCount = lineMap.length;
                var firstCommentLineIndent = void 0;
                for (var pos = commentPos, currentLine = firstCommentLineAndCharacter.line; pos < commentEnd; currentLine++) {
                    var nextLineStart = (currentLine + 1) === lineCount
                        ? text.length + 1
                        : lineMap[currentLine + 1];
                    if (pos !== commentPos) {
                        // If we are not emitting first line, we need to write the spaces to adjust the alignment
                        if (firstCommentLineIndent === undefined) {
                            firstCommentLineIndent = calculateIndent(text, lineMap[firstCommentLineAndCharacter.line], commentPos);
                        }
                        // These are number of spaces writer is going to write at current indent
                        var currentWriterIndentSpacing = writer.getIndent() * getIndentSize();
                        // Number of spaces we want to be writing
                        // eg: Assume writer indent
                        // module m {
                        //         /* starts at character 9 this is line 1
                        //    * starts at character pos 4 line                        --1  = 8 - 8 + 3
                        //   More left indented comment */                            --2  = 8 - 8 + 2
                        //     class c { }
                        // }
                        // module m {
                        //     /* this is line 1 -- Assume current writer indent 8
                        //      * line                                                --3 = 8 - 4 + 5
                        //            More right indented comment */                  --4 = 8 - 4 + 11
                        //     class c { }
                        // }
                        var spacesToEmit = currentWriterIndentSpacing - firstCommentLineIndent + calculateIndent(text, pos, nextLineStart);
                        if (spacesToEmit > 0) {
                            var numberOfSingleSpacesToEmit = spacesToEmit % getIndentSize();
                            var indentSizeSpaceString = getIndentString((spacesToEmit - numberOfSingleSpacesToEmit) / getIndentSize());
                            // Write indent size string ( in eg 1: = "", 2: "" , 3: string with 8 spaces 4: string with 12 spaces
                            writer.rawWrite(indentSizeSpaceString);
                            // Emit the single spaces (in eg: 1: 3 spaces, 2: 2 spaces, 3: 1 space, 4: 3 spaces)
                            while (numberOfSingleSpacesToEmit) {
                                writer.rawWrite(" ");
                                numberOfSingleSpacesToEmit--;
                            }
                        }
                        else {
                            // No spaces to emit write empty string
                            writer.rawWrite("");
                        }
                    }
                    // Write the comment line text
                    writeTrimmedCurrentLine(text, commentEnd, writer, newLine, pos, nextLineStart);
                    pos = nextLineStart;
                }
            }
            else {
                // Single line comment of style //....
                writer.writeComment(text.substring(commentPos, commentEnd));
            }
        }
        ts.writeCommentRange = writeCommentRange;
        function writeTrimmedCurrentLine(text, commentEnd, writer, newLine, pos, nextLineStart) {
            var end = Math.min(commentEnd, nextLineStart - 1);
            var currentLineText = text.substring(pos, end).replace(/^\s+|\s+$/g, "");
            if (currentLineText) {
                // trimmed forward and ending spaces text
                writer.writeComment(currentLineText);
                if (end !== commentEnd) {
                    writer.writeLine();
                }
            }
            else {
                // Empty string - make sure we write empty line
                writer.rawWrite(newLine);
            }
        }
        function calculateIndent(text, pos, end) {
            var currentLineIndent = 0;
            for (; pos < end && ts.isWhiteSpaceSingleLine(text.charCodeAt(pos)); pos++) {
                if (text.charCodeAt(pos) === 9 /* tab */) {
                    // Tabs = TabSize = indent size and go to next tabStop
                    currentLineIndent += getIndentSize() - (currentLineIndent % getIndentSize());
                }
                else {
                    // Single space
                    currentLineIndent++;
                }
            }
            return currentLineIndent;
        }
        function hasModifiers(node) {
            return getModifierFlags(node) !== 0 /* None */;
        }
        ts.hasModifiers = hasModifiers;
        function hasModifier(node, flags) {
            return !!getSelectedModifierFlags(node, flags);
        }
        ts.hasModifier = hasModifier;
        function hasStaticModifier(node) {
            return hasModifier(node, 32 /* Static */);
        }
        ts.hasStaticModifier = hasStaticModifier;
        function hasReadonlyModifier(node) {
            return hasModifier(node, 64 /* Readonly */);
        }
        ts.hasReadonlyModifier = hasReadonlyModifier;
        function getSelectedModifierFlags(node, flags) {
            return getModifierFlags(node) & flags;
        }
        ts.getSelectedModifierFlags = getSelectedModifierFlags;
        function getModifierFlags(node) {
            if (node.modifierFlagsCache & 536870912 /* HasComputedFlags */) {
                return node.modifierFlagsCache & ~536870912 /* HasComputedFlags */;
            }
            var flags = getModifierFlagsNoCache(node);
            node.modifierFlagsCache = flags | 536870912 /* HasComputedFlags */;
            return flags;
        }
        ts.getModifierFlags = getModifierFlags;
        function getModifierFlagsNoCache(node) {
            var flags = 0 /* None */;
            if (node.modifiers) {
                for (var _i = 0, _a = node.modifiers; _i < _a.length; _i++) {
                    var modifier = _a[_i];
                    flags |= modifierToFlag(modifier.kind);
                }
            }
            if (isInJSFile(node) && !!node.parent) {
                // getModifierFlagsNoCache should only be called when parent pointers are set,
                // or when !(node.flags & NodeFlags.Synthesized) && node.kind !== SyntaxKind.SourceFile)
                var tags = (ts.getJSDocPublicTag(node) ? 4 /* Public */ : 0 /* None */)
                    | (ts.getJSDocPrivateTag(node) ? 8 /* Private */ : 0 /* None */)
                    | (ts.getJSDocProtectedTag(node) ? 16 /* Protected */ : 0 /* None */)
                    | (ts.getJSDocReadonlyTag(node) ? 64 /* Readonly */ : 0 /* None */);
                flags |= tags;
            }
            if (node.flags & 4 /* NestedNamespace */ || (node.kind === 75 /* Identifier */ && node.isInJSDocNamespace)) {
                flags |= 1 /* Export */;
            }
            return flags;
        }
        ts.getModifierFlagsNoCache = getModifierFlagsNoCache;
        function modifierToFlag(token) {
            switch (token) {
                case 120 /* StaticKeyword */: return 32 /* Static */;
                case 119 /* PublicKeyword */: return 4 /* Public */;
                case 118 /* ProtectedKeyword */: return 16 /* Protected */;
                case 117 /* PrivateKeyword */: return 8 /* Private */;
                case 122 /* AbstractKeyword */: return 128 /* Abstract */;
                case 89 /* ExportKeyword */: return 1 /* Export */;
                case 130 /* DeclareKeyword */: return 2 /* Ambient */;
                case 81 /* ConstKeyword */: return 2048 /* Const */;
                case 84 /* DefaultKeyword */: return 512 /* Default */;
                case 126 /* AsyncKeyword */: return 256 /* Async */;
                case 138 /* ReadonlyKeyword */: return 64 /* Readonly */;
            }
            return 0 /* None */;
        }
        ts.modifierToFlag = modifierToFlag;
        function isLogicalOperator(token) {
            return token === 56 /* BarBarToken */
                || token === 55 /* AmpersandAmpersandToken */
                || token === 53 /* ExclamationToken */;
        }
        ts.isLogicalOperator = isLogicalOperator;
        function isAssignmentOperator(token) {
            return token >= 62 /* FirstAssignment */ && token <= 74 /* LastAssignment */;
        }
        ts.isAssignmentOperator = isAssignmentOperator;
        /** Get `C` given `N` if `N` is in the position `class C extends N` where `N` is an ExpressionWithTypeArguments. */
        function tryGetClassExtendingExpressionWithTypeArguments(node) {
            var cls = tryGetClassImplementingOrExtendingExpressionWithTypeArguments(node);
            return cls && !cls.isImplements ? cls.class : undefined;
        }
        ts.tryGetClassExtendingExpressionWithTypeArguments = tryGetClassExtendingExpressionWithTypeArguments;
        function tryGetClassImplementingOrExtendingExpressionWithTypeArguments(node) {
            return ts.isExpressionWithTypeArguments(node)
                && ts.isHeritageClause(node.parent)
                && ts.isClassLike(node.parent.parent)
                ? { class: node.parent.parent, isImplements: node.parent.token === 113 /* ImplementsKeyword */ }
                : undefined;
        }
        ts.tryGetClassImplementingOrExtendingExpressionWithTypeArguments = tryGetClassImplementingOrExtendingExpressionWithTypeArguments;
        function isAssignmentExpression(node, excludeCompoundAssignment) {
            return ts.isBinaryExpression(node)
                && (excludeCompoundAssignment
                    ? node.operatorToken.kind === 62 /* EqualsToken */
                    : isAssignmentOperator(node.operatorToken.kind))
                && ts.isLeftHandSideExpression(node.left);
        }
        ts.isAssignmentExpression = isAssignmentExpression;
        function isDestructuringAssignment(node) {
            if (isAssignmentExpression(node, /*excludeCompoundAssignment*/ true)) {
                var kind = node.left.kind;
                return kind === 193 /* ObjectLiteralExpression */
                    || kind === 192 /* ArrayLiteralExpression */;
            }
            return false;
        }
        ts.isDestructuringAssignment = isDestructuringAssignment;
        function isExpressionWithTypeArgumentsInClassExtendsClause(node) {
            return tryGetClassExtendingExpressionWithTypeArguments(node) !== undefined;
        }
        ts.isExpressionWithTypeArgumentsInClassExtendsClause = isExpressionWithTypeArgumentsInClassExtendsClause;
        function isEntityNameExpression(node) {
            return node.kind === 75 /* Identifier */ || isPropertyAccessEntityNameExpression(node);
        }
        ts.isEntityNameExpression = isEntityNameExpression;
        function getFirstIdentifier(node) {
            switch (node.kind) {
                case 75 /* Identifier */:
                    return node;
                case 153 /* QualifiedName */:
                    do {
                        node = node.left;
                    } while (node.kind !== 75 /* Identifier */);
                    return node;
                case 194 /* PropertyAccessExpression */:
                    do {
                        node = node.expression;
                    } while (node.kind !== 75 /* Identifier */);
                    return node;
            }
        }
        ts.getFirstIdentifier = getFirstIdentifier;
        function isDottedName(node) {
            return node.kind === 75 /* Identifier */ || node.kind === 104 /* ThisKeyword */ || node.kind === 102 /* SuperKeyword */ ||
                node.kind === 194 /* PropertyAccessExpression */ && isDottedName(node.expression) ||
                node.kind === 200 /* ParenthesizedExpression */ && isDottedName(node.expression);
        }
        ts.isDottedName = isDottedName;
        function isPropertyAccessEntityNameExpression(node) {
            return ts.isPropertyAccessExpression(node) && isEntityNameExpression(node.expression);
        }
        ts.isPropertyAccessEntityNameExpression = isPropertyAccessEntityNameExpression;
        function tryGetPropertyAccessOrIdentifierToString(expr) {
            if (ts.isPropertyAccessExpression(expr)) {
                var baseStr = tryGetPropertyAccessOrIdentifierToString(expr.expression);
                if (baseStr !== undefined) {
                    return baseStr + "." + expr.name;
                }
            }
            else if (ts.isIdentifier(expr)) {
                return ts.unescapeLeadingUnderscores(expr.escapedText);
            }
            return undefined;
        }
        ts.tryGetPropertyAccessOrIdentifierToString = tryGetPropertyAccessOrIdentifierToString;
        function isPrototypeAccess(node) {
            return isBindableStaticAccessExpression(node) && getElementOrPropertyAccessName(node) === "prototype";
        }
        ts.isPrototypeAccess = isPrototypeAccess;
        function isRightSideOfQualifiedNameOrPropertyAccess(node) {
            return (node.parent.kind === 153 /* QualifiedName */ && node.parent.right === node) ||
                (node.parent.kind === 194 /* PropertyAccessExpression */ && node.parent.name === node);
        }
        ts.isRightSideOfQualifiedNameOrPropertyAccess = isRightSideOfQualifiedNameOrPropertyAccess;
        function isEmptyObjectLiteral(expression) {
            return expression.kind === 193 /* ObjectLiteralExpression */ &&
                expression.properties.length === 0;
        }
        ts.isEmptyObjectLiteral = isEmptyObjectLiteral;
        function isEmptyArrayLiteral(expression) {
            return expression.kind === 192 /* ArrayLiteralExpression */ &&
                expression.elements.length === 0;
        }
        ts.isEmptyArrayLiteral = isEmptyArrayLiteral;
        function getLocalSymbolForExportDefault(symbol) {
            return isExportDefaultSymbol(symbol) ? symbol.declarations[0].localSymbol : undefined;
        }
        ts.getLocalSymbolForExportDefault = getLocalSymbolForExportDefault;
        function isExportDefaultSymbol(symbol) {
            return symbol && ts.length(symbol.declarations) > 0 && hasModifier(symbol.declarations[0], 512 /* Default */);
        }
        /** Return ".ts", ".d.ts", or ".tsx", if that is the extension. */
        function tryExtractTSExtension(fileName) {
            return ts.find(ts.supportedTSExtensionsForExtractExtension, function (extension) { return ts.fileExtensionIs(fileName, extension); });
        }
        ts.tryExtractTSExtension = tryExtractTSExtension;
        /**
         * Replace each instance of non-ascii characters by one, two, three, or four escape sequences
         * representing the UTF-8 encoding of the character, and return the expanded char code list.
         */
        function getExpandedCharCodes(input) {
            var output = [];
            var length = input.length;
            for (var i = 0; i < length; i++) {
                var charCode = input.charCodeAt(i);
                // handle utf8
                if (charCode < 0x80) {
                    output.push(charCode);
                }
                else if (charCode < 0x800) {
                    output.push((charCode >> 6) | 192);
                    output.push((charCode & 63) | 128);
                }
                else if (charCode < 0x10000) {
                    output.push((charCode >> 12) | 224);
                    output.push(((charCode >> 6) & 63) | 128);
                    output.push((charCode & 63) | 128);
                }
                else if (charCode < 0x20000) {
                    output.push((charCode >> 18) | 240);
                    output.push(((charCode >> 12) & 63) | 128);
                    output.push(((charCode >> 6) & 63) | 128);
                    output.push((charCode & 63) | 128);
                }
                else {
                    ts.Debug.assert(false, "Unexpected code point");
                }
            }
            return output;
        }
        var base64Digits = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
        /**
         * Converts a string to a base-64 encoded ASCII string.
         */
        function convertToBase64(input) {
            var result = "";
            var charCodes = getExpandedCharCodes(input);
            var i = 0;
            var length = charCodes.length;
            var byte1, byte2, byte3, byte4;
            while (i < length) {
                // Convert every 6-bits in the input 3 character points
                // into a base64 digit
                byte1 = charCodes[i] >> 2;
                byte2 = (charCodes[i] & 3) << 4 | charCodes[i + 1] >> 4;
                byte3 = (charCodes[i + 1] & 15) << 2 | charCodes[i + 2] >> 6;
                byte4 = charCodes[i + 2] & 63;
                // We are out of characters in the input, set the extra
                // digits to 64 (padding character).
                if (i + 1 >= length) {
                    byte3 = byte4 = 64;
                }
                else if (i + 2 >= length) {
                    byte4 = 64;
                }
                // Write to the output
                result += base64Digits.charAt(byte1) + base64Digits.charAt(byte2) + base64Digits.charAt(byte3) + base64Digits.charAt(byte4);
                i += 3;
            }
            return result;
        }
        ts.convertToBase64 = convertToBase64;
        function getStringFromExpandedCharCodes(codes) {
            var output = "";
            var i = 0;
            var length = codes.length;
            while (i < length) {
                var charCode = codes[i];
                if (charCode < 0x80) {
                    output += String.fromCharCode(charCode);
                    i++;
                }
                else if ((charCode & 192) === 192) {
                    var value = charCode & 63;
                    i++;
                    var nextCode = codes[i];
                    while ((nextCode & 192) === 128) {
                        value = (value << 6) | (nextCode & 63);
                        i++;
                        nextCode = codes[i];
                    }
                    // `value` may be greater than 10FFFF (the maximum unicode codepoint) - JS will just make this into an invalid character for us
                    output += String.fromCharCode(value);
                }
                else {
                    // We don't want to kill the process when decoding fails (due to a following char byte not
                    // following a leading char), so we just print the (bad) value
                    output += String.fromCharCode(charCode);
                    i++;
                }
            }
            return output;
        }
        function base64encode(host, input) {
            if (host && host.base64encode) {
                return host.base64encode(input);
            }
            return convertToBase64(input);
        }
        ts.base64encode = base64encode;
        function base64decode(host, input) {
            if (host && host.base64decode) {
                return host.base64decode(input);
            }
            var length = input.length;
            var expandedCharCodes = [];
            var i = 0;
            while (i < length) {
                // Stop decoding once padding characters are present
                if (input.charCodeAt(i) === base64Digits.charCodeAt(64)) {
                    break;
                }
                // convert 4 input digits into three characters, ignoring padding characters at the end
                var ch1 = base64Digits.indexOf(input[i]);
                var ch2 = base64Digits.indexOf(input[i + 1]);
                var ch3 = base64Digits.indexOf(input[i + 2]);
                var ch4 = base64Digits.indexOf(input[i + 3]);
                var code1 = ((ch1 & 63) << 2) | ((ch2 >> 4) & 3);
                var code2 = ((ch2 & 15) << 4) | ((ch3 >> 2) & 15);
                var code3 = ((ch3 & 3) << 6) | (ch4 & 63);
                if (code2 === 0 && ch3 !== 0) { // code2 decoded to zero, but ch3 was padding - elide code2 and code3
                    expandedCharCodes.push(code1);
                }
                else if (code3 === 0 && ch4 !== 0) { // code3 decoded to zero, but ch4 was padding, elide code3
                    expandedCharCodes.push(code1, code2);
                }
                else {
                    expandedCharCodes.push(code1, code2, code3);
                }
                i += 4;
            }
            return getStringFromExpandedCharCodes(expandedCharCodes);
        }
        ts.base64decode = base64decode;
        function readJson(path, host) {
            try {
                var jsonText = host.readFile(path);
                if (!jsonText)
                    return {};
                var result = ts.parseConfigFileTextToJson(path, jsonText);
                if (result.error) {
                    return {};
                }
                return result.config;
            }
            catch (e) {
                // gracefully handle if readFile fails or returns not JSON
                return {};
            }
        }
        ts.readJson = readJson;
        function directoryProbablyExists(directoryName, host) {
            // if host does not support 'directoryExists' assume that directory will exist
            return !host.directoryExists || host.directoryExists(directoryName);
        }
        ts.directoryProbablyExists = directoryProbablyExists;
        var carriageReturnLineFeed = "\r\n";
        var lineFeed = "\n";
        function getNewLineCharacter(options, getNewLine) {
            switch (options.newLine) {
                case 0 /* CarriageReturnLineFeed */:
                    return carriageReturnLineFeed;
                case 1 /* LineFeed */:
                    return lineFeed;
            }
            return getNewLine ? getNewLine() : ts.sys ? ts.sys.newLine : carriageReturnLineFeed;
        }
        ts.getNewLineCharacter = getNewLineCharacter;
        /**
         * Creates a new TextRange from the provided pos and end.
         *
         * @param pos The start position.
         * @param end The end position.
         */
        function createRange(pos, end) {
            if (end === void 0) {
                end = pos;
            }
            ts.Debug.assert(end >= pos || end === -1);
            return { pos: pos, end: end };
        }
        ts.createRange = createRange;
        /**
         * Creates a new TextRange from a provided range with a new end position.
         *
         * @param range A TextRange.
         * @param end The new end position.
         */
        function moveRangeEnd(range, end) {
            return createRange(range.pos, end);
        }
        ts.moveRangeEnd = moveRangeEnd;
        /**
         * Creates a new TextRange from a provided range with a new start position.
         *
         * @param range A TextRange.
         * @param pos The new Start position.
         */
        function moveRangePos(range, pos) {
            return createRange(pos, range.end);
        }
        ts.moveRangePos = moveRangePos;
        /**
         * Moves the start position of a range past any decorators.
         */
        function moveRangePastDecorators(node) {
            return node.decorators && node.decorators.length > 0
                ? moveRangePos(node, node.decorators.end)
                : node;
        }
        ts.moveRangePastDecorators = moveRangePastDecorators;
        /**
         * Moves the start position of a range past any decorators or modifiers.
         */
        function moveRangePastModifiers(node) {
            return node.modifiers && node.modifiers.length > 0
                ? moveRangePos(node, node.modifiers.end)
                : moveRangePastDecorators(node);
        }
        ts.moveRangePastModifiers = moveRangePastModifiers;
        /**
         * Determines whether a TextRange has the same start and end positions.
         *
         * @param range A TextRange.
         */
        function isCollapsedRange(range) {
            return range.pos === range.end;
        }
        ts.isCollapsedRange = isCollapsedRange;
        /**
         * Creates a new TextRange for a token at the provides start position.
         *
         * @param pos The start position.
         * @param token The token.
         */
        function createTokenRange(pos, token) {
            return createRange(pos, pos + ts.tokenToString(token).length);
        }
        ts.createTokenRange = createTokenRange;
        function rangeIsOnSingleLine(range, sourceFile) {
            return rangeStartIsOnSameLineAsRangeEnd(range, range, sourceFile);
        }
        ts.rangeIsOnSingleLine = rangeIsOnSingleLine;
        function rangeStartPositionsAreOnSameLine(range1, range2, sourceFile) {
            return positionsAreOnSameLine(getStartPositionOfRange(range1, sourceFile), getStartPositionOfRange(range2, sourceFile), sourceFile);
        }
        ts.rangeStartPositionsAreOnSameLine = rangeStartPositionsAreOnSameLine;
        function rangeEndPositionsAreOnSameLine(range1, range2, sourceFile) {
            return positionsAreOnSameLine(range1.end, range2.end, sourceFile);
        }
        ts.rangeEndPositionsAreOnSameLine = rangeEndPositionsAreOnSameLine;
        function rangeStartIsOnSameLineAsRangeEnd(range1, range2, sourceFile) {
            return positionsAreOnSameLine(getStartPositionOfRange(range1, sourceFile), range2.end, sourceFile);
        }
        ts.rangeStartIsOnSameLineAsRangeEnd = rangeStartIsOnSameLineAsRangeEnd;
        function rangeEndIsOnSameLineAsRangeStart(range1, range2, sourceFile) {
            return positionsAreOnSameLine(range1.end, getStartPositionOfRange(range2, sourceFile), sourceFile);
        }
        ts.rangeEndIsOnSameLineAsRangeStart = rangeEndIsOnSameLineAsRangeStart;
        function isNodeArrayMultiLine(list, sourceFile) {
            return !positionsAreOnSameLine(list.pos, list.end, sourceFile);
        }
        ts.isNodeArrayMultiLine = isNodeArrayMultiLine;
        function positionsAreOnSameLine(pos1, pos2, sourceFile) {
            return pos1 === pos2 ||
                getLineOfLocalPosition(sourceFile, pos1) === getLineOfLocalPosition(sourceFile, pos2);
        }
        ts.positionsAreOnSameLine = positionsAreOnSameLine;
        function getStartPositionOfRange(range, sourceFile) {
            return positionIsSynthesized(range.pos) ? -1 : ts.skipTrivia(sourceFile.text, range.pos);
        }
        ts.getStartPositionOfRange = getStartPositionOfRange;
        /**
         * Determines whether a name was originally the declaration name of an enum or namespace
         * declaration.
         */
        function isDeclarationNameOfEnumOrNamespace(node) {
            var parseNode = ts.getParseTreeNode(node);
            if (parseNode) {
                switch (parseNode.parent.kind) {
                    case 248 /* EnumDeclaration */:
                    case 249 /* ModuleDeclaration */:
                        return parseNode === parseNode.parent.name;
                }
            }
            return false;
        }
        ts.isDeclarationNameOfEnumOrNamespace = isDeclarationNameOfEnumOrNamespace;
        function getInitializedVariables(node) {
            return ts.filter(node.declarations, isInitializedVariable);
        }
        ts.getInitializedVariables = getInitializedVariables;
        function isInitializedVariable(node) {
            return node.initializer !== undefined;
        }
        function isWatchSet(options) {
            // Firefox has Object.prototype.watch
            return options.watch && options.hasOwnProperty("watch");
        }
        ts.isWatchSet = isWatchSet;
        function closeFileWatcher(watcher) {
            watcher.close();
        }
        ts.closeFileWatcher = closeFileWatcher;
        function getCheckFlags(symbol) {
            return symbol.flags & 33554432 /* Transient */ ? symbol.checkFlags : 0;
        }
        ts.getCheckFlags = getCheckFlags;
        function getDeclarationModifierFlagsFromSymbol(s) {
            if (s.valueDeclaration) {
                var flags = ts.getCombinedModifierFlags(s.valueDeclaration);
                return s.parent && s.parent.flags & 32 /* Class */ ? flags : flags & ~28 /* AccessibilityModifier */;
            }
            if (getCheckFlags(s) & 6 /* Synthetic */) {
                var checkFlags = s.checkFlags;
                var accessModifier = checkFlags & 1024 /* ContainsPrivate */ ? 8 /* Private */ :
                    checkFlags & 256 /* ContainsPublic */ ? 4 /* Public */ :
                        16 /* Protected */;
                var staticModifier = checkFlags & 2048 /* ContainsStatic */ ? 32 /* Static */ : 0;
                return accessModifier | staticModifier;
            }
            if (s.flags & 4194304 /* Prototype */) {
                return 4 /* Public */ | 32 /* Static */;
            }
            return 0;
        }
        ts.getDeclarationModifierFlagsFromSymbol = getDeclarationModifierFlagsFromSymbol;
        function skipAlias(symbol, checker) {
            return symbol.flags & 2097152 /* Alias */ ? checker.getAliasedSymbol(symbol) : symbol;
        }
        ts.skipAlias = skipAlias;
        /** See comment on `declareModuleMember` in `binder.ts`. */
        function getCombinedLocalAndExportSymbolFlags(symbol) {
            return symbol.exportSymbol ? symbol.exportSymbol.flags | symbol.flags : symbol.flags;
        }
        ts.getCombinedLocalAndExportSymbolFlags = getCombinedLocalAndExportSymbolFlags;
        function isWriteOnlyAccess(node) {
            return accessKind(node) === 1 /* Write */;
        }
        ts.isWriteOnlyAccess = isWriteOnlyAccess;
        function isWriteAccess(node) {
            return accessKind(node) !== 0 /* Read */;
        }
        ts.isWriteAccess = isWriteAccess;
        var AccessKind;
        (function (AccessKind) {
            /** Only reads from a variable. */
            AccessKind[AccessKind["Read"] = 0] = "Read";
            /** Only writes to a variable without using the result. E.g.: `x++;`. */
            AccessKind[AccessKind["Write"] = 1] = "Write";
            /** Writes to a variable and uses the result as an expression. E.g.: `f(x++);`. */
            AccessKind[AccessKind["ReadWrite"] = 2] = "ReadWrite";
        })(AccessKind || (AccessKind = {}));
        function accessKind(node) {
            var parent = node.parent;
            if (!parent)
                return 0 /* Read */;
            switch (parent.kind) {
                case 200 /* ParenthesizedExpression */:
                    return accessKind(parent);
                case 208 /* PostfixUnaryExpression */:
                case 207 /* PrefixUnaryExpression */:
                    var operator = parent.operator;
                    return operator === 45 /* PlusPlusToken */ || operator === 46 /* MinusMinusToken */ ? writeOrReadWrite() : 0 /* Read */;
                case 209 /* BinaryExpression */:
                    var _a = parent, left = _a.left, operatorToken = _a.operatorToken;
                    return left === node && isAssignmentOperator(operatorToken.kind) ?
                        operatorToken.kind === 62 /* EqualsToken */ ? 1 /* Write */ : writeOrReadWrite()
                        : 0 /* Read */;
                case 194 /* PropertyAccessExpression */:
                    return parent.name !== node ? 0 /* Read */ : accessKind(parent);
                case 281 /* PropertyAssignment */: {
                    var parentAccess = accessKind(parent.parent);
                    // In `({ x: varname }) = { x: 1 }`, the left `x` is a read, the right `x` is a write.
                    return node === parent.name ? reverseAccessKind(parentAccess) : parentAccess;
                }
                case 282 /* ShorthandPropertyAssignment */:
                    // Assume it's the local variable being accessed, since we don't check public properties for --noUnusedLocals.
                    return node === parent.objectAssignmentInitializer ? 0 /* Read */ : accessKind(parent.parent);
                case 192 /* ArrayLiteralExpression */:
                    return accessKind(parent);
                default:
                    return 0 /* Read */;
            }
            function writeOrReadWrite() {
                // If grandparent is not an ExpressionStatement, this is used as an expression in addition to having a side effect.
                return parent.parent && skipParenthesesUp(parent.parent).kind === 226 /* ExpressionStatement */ ? 1 /* Write */ : 2 /* ReadWrite */;
            }
        }
        function reverseAccessKind(a) {
            switch (a) {
                case 0 /* Read */:
                    return 1 /* Write */;
                case 1 /* Write */:
                    return 0 /* Read */;
                case 2 /* ReadWrite */:
                    return 2 /* ReadWrite */;
                default:
                    return ts.Debug.assertNever(a);
            }
        }
        function compareDataObjects(dst, src) {
            if (!dst || !src || Object.keys(dst).length !== Object.keys(src).length) {
                return false;
            }
            for (var e in dst) {
                if (typeof dst[e] === "object") {
                    if (!compareDataObjects(dst[e], src[e])) {
                        return false;
                    }
                }
                else if (typeof dst[e] !== "function") {
                    if (dst[e] !== src[e]) {
                        return false;
                    }
                }
            }
            return true;
        }
        ts.compareDataObjects = compareDataObjects;
        /**
         * clears already present map by calling onDeleteExistingValue callback before deleting that key/value
         */
        function clearMap(map, onDeleteValue) {
            // Remove all
            map.forEach(onDeleteValue);
            map.clear();
        }
        ts.clearMap = clearMap;
        /**
         * Mutates the map with newMap such that keys in map will be same as newMap.
         */
        function mutateMapSkippingNewValues(map, newMap, options) {
            var onDeleteValue = options.onDeleteValue, onExistingValue = options.onExistingValue;
            // Needs update
            map.forEach(function (existingValue, key) {
                var valueInNewMap = newMap.get(key);
                // Not present any more in new map, remove it
                if (valueInNewMap === undefined) {
                    map.delete(key);
                    onDeleteValue(existingValue, key);
                }
                // If present notify about existing values
                else if (onExistingValue) {
                    onExistingValue(existingValue, valueInNewMap, key);
                }
            });
        }
        ts.mutateMapSkippingNewValues = mutateMapSkippingNewValues;
        /**
         * Mutates the map with newMap such that keys in map will be same as newMap.
         */
        function mutateMap(map, newMap, options) {
            // Needs update
            mutateMapSkippingNewValues(map, newMap, options);
            var createNewValue = options.createNewValue;
            // Add new values that are not already present
            newMap.forEach(function (valueInNewMap, key) {
                if (!map.has(key)) {
                    // New values
                    map.set(key, createNewValue(key, valueInNewMap));
                }
            });
        }
        ts.mutateMap = mutateMap;
        // Return true if the given type is the constructor type for an abstract class
        function isAbstractConstructorType(type) {
            return !!(getObjectFlags(type) & 16 /* Anonymous */) && !!type.symbol && isAbstractConstructorSymbol(type.symbol);
        }
        ts.isAbstractConstructorType = isAbstractConstructorType;
        function isAbstractConstructorSymbol(symbol) {
            if (symbol.flags & 32 /* Class */) {
                var declaration = getClassLikeDeclarationOfSymbol(symbol);
                return !!declaration && hasModifier(declaration, 128 /* Abstract */);
            }
            return false;
        }
        ts.isAbstractConstructorSymbol = isAbstractConstructorSymbol;
        function getClassLikeDeclarationOfSymbol(symbol) {
            return ts.find(symbol.declarations, ts.isClassLike);
        }
        ts.getClassLikeDeclarationOfSymbol = getClassLikeDeclarationOfSymbol;
        function getObjectFlags(type) {
            return type.flags & 3899393 /* ObjectFlagsType */ ? type.objectFlags : 0;
        }
        ts.getObjectFlags = getObjectFlags;
        function typeHasCallOrConstructSignatures(type, checker) {
            return checker.getSignaturesOfType(type, 0 /* Call */).length !== 0 || checker.getSignaturesOfType(type, 1 /* Construct */).length !== 0;
        }
        ts.typeHasCallOrConstructSignatures = typeHasCallOrConstructSignatures;
        function forSomeAncestorDirectory(directory, callback) {
            return !!ts.forEachAncestorDirectory(directory, function (d) { return callback(d) ? true : undefined; });
        }
        ts.forSomeAncestorDirectory = forSomeAncestorDirectory;
        function isUMDExportSymbol(symbol) {
            return !!symbol && !!symbol.declarations && !!symbol.declarations[0] && ts.isNamespaceExportDeclaration(symbol.declarations[0]);
        }
        ts.isUMDExportSymbol = isUMDExportSymbol;
        function showModuleSpecifier(_a) {
            var moduleSpecifier = _a.moduleSpecifier;
            return ts.isStringLiteral(moduleSpecifier) ? moduleSpecifier.text : getTextOfNode(moduleSpecifier);
        }
        ts.showModuleSpecifier = showModuleSpecifier;
        function getLastChild(node) {
            var lastChild;
            ts.forEachChild(node, function (child) {
                if (nodeIsPresent(child))
                    lastChild = child;
            }, function (children) {
                // As an optimization, jump straight to the end of the list.
                for (var i = children.length - 1; i >= 0; i--) {
                    if (nodeIsPresent(children[i])) {
                        lastChild = children[i];
                        break;
                    }
                }
            });
            return lastChild;
        }
        ts.getLastChild = getLastChild;
        function addToSeen(seen, key, value) {
            if (value === void 0) {
                value = true;
            }
            key = String(key);
            if (seen.has(key)) {
                return false;
            }
            seen.set(key, value);
            return true;
        }
        ts.addToSeen = addToSeen;
        function isObjectTypeDeclaration(node) {
            return ts.isClassLike(node) || ts.isInterfaceDeclaration(node) || ts.isTypeLiteralNode(node);
        }
        ts.isObjectTypeDeclaration = isObjectTypeDeclaration;
        function isTypeNodeKind(kind) {
            return (kind >= 168 /* FirstTypeNode */ && kind <= 188 /* LastTypeNode */)
                || kind === 125 /* AnyKeyword */
                || kind === 148 /* UnknownKeyword */
                || kind === 140 /* NumberKeyword */
                || kind === 151 /* BigIntKeyword */
                || kind === 141 /* ObjectKeyword */
                || kind === 128 /* BooleanKeyword */
                || kind === 143 /* StringKeyword */
                || kind === 144 /* SymbolKeyword */
                || kind === 104 /* ThisKeyword */
                || kind === 110 /* VoidKeyword */
                || kind === 146 /* UndefinedKeyword */
                || kind === 100 /* NullKeyword */
                || kind === 137 /* NeverKeyword */
                || kind === 216 /* ExpressionWithTypeArguments */
                || kind === 295 /* JSDocAllType */
                || kind === 296 /* JSDocUnknownType */
                || kind === 297 /* JSDocNullableType */
                || kind === 298 /* JSDocNonNullableType */
                || kind === 299 /* JSDocOptionalType */
                || kind === 300 /* JSDocFunctionType */
                || kind === 301 /* JSDocVariadicType */;
        }
        ts.isTypeNodeKind = isTypeNodeKind;
        function isAccessExpression(node) {
            return node.kind === 194 /* PropertyAccessExpression */ || node.kind === 195 /* ElementAccessExpression */;
        }
        ts.isAccessExpression = isAccessExpression;
        function isBundleFileTextLike(section) {
            switch (section.kind) {
                case "text" /* Text */:
                case "internal" /* Internal */:
                    return true;
                default:
                    return false;
            }
        }
        ts.isBundleFileTextLike = isBundleFileTextLike;
        function getDotOrQuestionDotToken(node) {
            return node.questionDotToken || ts.createNode(24 /* DotToken */, node.expression.end, node.name.pos);
        }
        ts.getDotOrQuestionDotToken = getDotOrQuestionDotToken;
        function isNamedImportsOrExports(node) {
            return node.kind === 257 /* NamedImports */ || node.kind === 261 /* NamedExports */;
        }
        ts.isNamedImportsOrExports = isNamedImportsOrExports;
        function Symbol(flags, name) {
            this.flags = flags;
            this.escapedName = name;
            this.declarations = undefined;
            this.valueDeclaration = undefined;
            this.id = undefined;
            this.mergeId = undefined;
            this.parent = undefined;
        }
        function Type(checker, flags) {
            this.flags = flags;
            if (ts.Debug.isDebugging) {
                this.checker = checker;
            }
        }
        function Signature(checker, flags) {
            this.flags = flags;
            if (ts.Debug.isDebugging) {
                this.checker = checker;
            }
        }
        function Node(kind, pos, end) {
            this.pos = pos;
            this.end = end;
            this.kind = kind;
            this.id = 0;
            this.flags = 0 /* None */;
            this.modifierFlagsCache = 0 /* None */;
            this.transformFlags = 0 /* None */;
            this.parent = undefined;
            this.original = undefined;
        }
        function SourceMapSource(fileName, text, skipTrivia) {
            this.fileName = fileName;
            this.text = text;
            this.skipTrivia = skipTrivia || (function (pos) { return pos; });
        }
        // eslint-disable-next-line prefer-const
        ts.objectAllocator = {
            getNodeConstructor: function () { return Node; },
            getTokenConstructor: function () { return Node; },
            getIdentifierConstructor: function () { return Node; },
            getPrivateIdentifierConstructor: function () { return Node; },
            getSourceFileConstructor: function () { return Node; },
            getSymbolConstructor: function () { return Symbol; },
            getTypeConstructor: function () { return Type; },
            getSignatureConstructor: function () { return Signature; },
            getSourceMapSourceConstructor: function () { return SourceMapSource; },
        };
        function setObjectAllocator(alloc) {
            ts.objectAllocator = alloc;
        }
        ts.setObjectAllocator = setObjectAllocator;
        function formatStringFromArgs(text, args, baseIndex) {
            if (baseIndex === void 0) {
                baseIndex = 0;
            }
            return text.replace(/{(\d+)}/g, function (_match, index) { return "" + ts.Debug.assertDefined(args[+index + baseIndex]); });
        }
        ts.formatStringFromArgs = formatStringFromArgs;
        /* @internal */
        function setLocalizedDiagnosticMessages(messages) {
            ts.localizedDiagnosticMessages = messages;
        }
        ts.setLocalizedDiagnosticMessages = setLocalizedDiagnosticMessages;
        function getLocaleSpecificMessage(message) {
            return ts.localizedDiagnosticMessages && ts.localizedDiagnosticMessages[message.key] || message.message;
        }
        ts.getLocaleSpecificMessage = getLocaleSpecificMessage;
        function createFileDiagnostic(file, start, length, message) {
            ts.Debug.assertGreaterThanOrEqual(start, 0);
            ts.Debug.assertGreaterThanOrEqual(length, 0);
            if (file) {
                ts.Debug.assertLessThanOrEqual(start, file.text.length);
                ts.Debug.assertLessThanOrEqual(start + length, file.text.length);
            }
            var text = getLocaleSpecificMessage(message);
            if (arguments.length > 4) {
                text = formatStringFromArgs(text, arguments, 4);
            }
            return {
                file: file,
                start: start,
                length: length,
                messageText: text,
                category: message.category,
                code: message.code,
                reportsUnnecessary: message.reportsUnnecessary,
            };
        }
        ts.createFileDiagnostic = createFileDiagnostic;
        function formatMessage(_dummy, message) {
            var text = getLocaleSpecificMessage(message);
            if (arguments.length > 2) {
                text = formatStringFromArgs(text, arguments, 2);
            }
            return text;
        }
        ts.formatMessage = formatMessage;
        function createCompilerDiagnostic(message) {
            var text = getLocaleSpecificMessage(message);
            if (arguments.length > 1) {
                text = formatStringFromArgs(text, arguments, 1);
            }
            return {
                file: undefined,
                start: undefined,
                length: undefined,
                messageText: text,
                category: message.category,
                code: message.code,
                reportsUnnecessary: message.reportsUnnecessary,
            };
        }
        ts.createCompilerDiagnostic = createCompilerDiagnostic;
        function createCompilerDiagnosticFromMessageChain(chain) {
            return {
                file: undefined,
                start: undefined,
                length: undefined,
                code: chain.code,
                category: chain.category,
                messageText: chain.next ? chain : chain.messageText,
            };
        }
        ts.createCompilerDiagnosticFromMessageChain = createCompilerDiagnosticFromMessageChain;
        function chainDiagnosticMessages(details, message) {
            var text = getLocaleSpecificMessage(message);
            if (arguments.length > 2) {
                text = formatStringFromArgs(text, arguments, 2);
            }
            return {
                messageText: text,
                category: message.category,
                code: message.code,
                next: details === undefined || Array.isArray(details) ? details : [details]
            };
        }
        ts.chainDiagnosticMessages = chainDiagnosticMessages;
        function concatenateDiagnosticMessageChains(headChain, tailChain) {
            var lastChain = headChain;
            while (lastChain.next) {
                lastChain = lastChain.next[0];
            }
            lastChain.next = [tailChain];
        }
        ts.concatenateDiagnosticMessageChains = concatenateDiagnosticMessageChains;
        function getDiagnosticFilePath(diagnostic) {
            return diagnostic.file ? diagnostic.file.path : undefined;
        }
        function compareDiagnostics(d1, d2) {
            return compareDiagnosticsSkipRelatedInformation(d1, d2) ||
                compareRelatedInformation(d1, d2) ||
                0 /* EqualTo */;
        }
        ts.compareDiagnostics = compareDiagnostics;
        function compareDiagnosticsSkipRelatedInformation(d1, d2) {
            return ts.compareStringsCaseSensitive(getDiagnosticFilePath(d1), getDiagnosticFilePath(d2)) ||
                ts.compareValues(d1.start, d2.start) ||
                ts.compareValues(d1.length, d2.length) ||
                ts.compareValues(d1.code, d2.code) ||
                compareMessageText(d1.messageText, d2.messageText) ||
                0 /* EqualTo */;
        }
        ts.compareDiagnosticsSkipRelatedInformation = compareDiagnosticsSkipRelatedInformation;
        function compareRelatedInformation(d1, d2) {
            if (!d1.relatedInformation && !d2.relatedInformation) {
                return 0 /* EqualTo */;
            }
            if (d1.relatedInformation && d2.relatedInformation) {
                return ts.compareValues(d1.relatedInformation.length, d2.relatedInformation.length) || ts.forEach(d1.relatedInformation, function (d1i, index) {
                    var d2i = d2.relatedInformation[index];
                    return compareDiagnostics(d1i, d2i); // EqualTo is 0, so falsy, and will cause the next item to be compared
                }) || 0 /* EqualTo */;
            }
            return d1.relatedInformation ? -1 /* LessThan */ : 1 /* GreaterThan */;
        }
        function compareMessageText(t1, t2) {
            if (typeof t1 === "string" && typeof t2 === "string") {
                return ts.compareStringsCaseSensitive(t1, t2);
            }
            else if (typeof t1 === "string") {
                return -1 /* LessThan */;
            }
            else if (typeof t2 === "string") {
                return 1 /* GreaterThan */;
            }
            var res = ts.compareStringsCaseSensitive(t1.messageText, t2.messageText);
            if (res) {
                return res;
            }
            if (!t1.next && !t2.next) {
                return 0 /* EqualTo */;
            }
            if (!t1.next) {
                return -1 /* LessThan */;
            }
            if (!t2.next) {
                return 1 /* GreaterThan */;
            }
            var len = Math.min(t1.next.length, t2.next.length);
            for (var i = 0; i < len; i++) {
                res = compareMessageText(t1.next[i], t2.next[i]);
                if (res) {
                    return res;
                }
            }
            if (t1.next.length < t2.next.length) {
                return -1 /* LessThan */;
            }
            else if (t1.next.length > t2.next.length) {
                return 1 /* GreaterThan */;
            }
            return 0 /* EqualTo */;
        }
        function getEmitScriptTarget(compilerOptions) {
            return compilerOptions.target || 0 /* ES3 */;
        }
        ts.getEmitScriptTarget = getEmitScriptTarget;
        function getEmitModuleKind(compilerOptions) {
            return typeof compilerOptions.module === "number" ?
                compilerOptions.module :
                getEmitScriptTarget(compilerOptions) >= 2 /* ES2015 */ ? ts.ModuleKind.ES2015 : ts.ModuleKind.CommonJS;
        }
        ts.getEmitModuleKind = getEmitModuleKind;
        function getEmitModuleResolutionKind(compilerOptions) {
            var moduleResolution = compilerOptions.moduleResolution;
            if (moduleResolution === undefined) {
                moduleResolution = getEmitModuleKind(compilerOptions) === ts.ModuleKind.CommonJS ? ts.ModuleResolutionKind.NodeJs : ts.ModuleResolutionKind.Classic;
            }
            return moduleResolution;
        }
        ts.getEmitModuleResolutionKind = getEmitModuleResolutionKind;
        function hasJsonModuleEmitEnabled(options) {
            switch (getEmitModuleKind(options)) {
                case ts.ModuleKind.CommonJS:
                case ts.ModuleKind.AMD:
                case ts.ModuleKind.ES2015:
                case ts.ModuleKind.ES2020:
                case ts.ModuleKind.ESNext:
                    return true;
                default:
                    return false;
            }
        }
        ts.hasJsonModuleEmitEnabled = hasJsonModuleEmitEnabled;
        function unreachableCodeIsError(options) {
            return options.allowUnreachableCode === false;
        }
        ts.unreachableCodeIsError = unreachableCodeIsError;
        function unusedLabelIsError(options) {
            return options.allowUnusedLabels === false;
        }
        ts.unusedLabelIsError = unusedLabelIsError;
        function getAreDeclarationMapsEnabled(options) {
            return !!(getEmitDeclarations(options) && options.declarationMap);
        }
        ts.getAreDeclarationMapsEnabled = getAreDeclarationMapsEnabled;
        function getAllowSyntheticDefaultImports(compilerOptions) {
            var moduleKind = getEmitModuleKind(compilerOptions);
            return compilerOptions.allowSyntheticDefaultImports !== undefined
                ? compilerOptions.allowSyntheticDefaultImports
                : compilerOptions.esModuleInterop ||
                    moduleKind === ts.ModuleKind.System;
        }
        ts.getAllowSyntheticDefaultImports = getAllowSyntheticDefaultImports;
        function getEmitDeclarations(compilerOptions) {
            return !!(compilerOptions.declaration || compilerOptions.composite);
        }
        ts.getEmitDeclarations = getEmitDeclarations;
        function isIncrementalCompilation(options) {
            return !!(options.incremental || options.composite);
        }
        ts.isIncrementalCompilation = isIncrementalCompilation;
        function getStrictOptionValue(compilerOptions, flag) {
            return compilerOptions[flag] === undefined ? !!compilerOptions.strict : !!compilerOptions[flag];
        }
        ts.getStrictOptionValue = getStrictOptionValue;
        function compilerOptionsAffectSemanticDiagnostics(newOptions, oldOptions) {
            return oldOptions !== newOptions &&
                ts.semanticDiagnosticsOptionDeclarations.some(function (option) { return !isJsonEqual(getCompilerOptionValue(oldOptions, option), getCompilerOptionValue(newOptions, option)); });
        }
        ts.compilerOptionsAffectSemanticDiagnostics = compilerOptionsAffectSemanticDiagnostics;
        function compilerOptionsAffectEmit(newOptions, oldOptions) {
            return oldOptions !== newOptions &&
                ts.affectsEmitOptionDeclarations.some(function (option) { return !isJsonEqual(getCompilerOptionValue(oldOptions, option), getCompilerOptionValue(newOptions, option)); });
        }
        ts.compilerOptionsAffectEmit = compilerOptionsAffectEmit;
        function getCompilerOptionValue(options, option) {
            return option.strictFlag ? getStrictOptionValue(options, option.name) : options[option.name];
        }
        ts.getCompilerOptionValue = getCompilerOptionValue;
        function hasZeroOrOneAsteriskCharacter(str) {
            var seenAsterisk = false;
            for (var i = 0; i < str.length; i++) {
                if (str.charCodeAt(i) === 42 /* asterisk */) {
                    if (!seenAsterisk) {
                        seenAsterisk = true;
                    }
                    else {
                        // have already seen asterisk
                        return false;
                    }
                }
            }
            return true;
        }
        ts.hasZeroOrOneAsteriskCharacter = hasZeroOrOneAsteriskCharacter;
        function discoverProbableSymlinks(files, getCanonicalFileName, cwd) {
            var result = ts.createMap();
            var symlinks = ts.flatten(ts.mapDefined(files, function (sf) {
                return sf.resolvedModules && ts.compact(ts.arrayFrom(ts.mapIterator(sf.resolvedModules.values(), function (res) {
                    return res && res.originalPath && res.resolvedFileName !== res.originalPath ? [res.resolvedFileName, res.originalPath] : undefined;
                })));
            }));
            for (var _i = 0, symlinks_1 = symlinks; _i < symlinks_1.length; _i++) {
                var _a = symlinks_1[_i], resolvedPath = _a[0], originalPath = _a[1];
                var _b = guessDirectorySymlink(resolvedPath, originalPath, cwd, getCanonicalFileName), commonResolved = _b[0], commonOriginal = _b[1];
                result.set(commonOriginal, commonResolved);
            }
            return result;
        }
        ts.discoverProbableSymlinks = discoverProbableSymlinks;
        function guessDirectorySymlink(a, b, cwd, getCanonicalFileName) {
            var aParts = ts.getPathComponents(ts.toPath(a, cwd, getCanonicalFileName));
            var bParts = ts.getPathComponents(ts.toPath(b, cwd, getCanonicalFileName));
            while (!isNodeModulesOrScopedPackageDirectory(aParts[aParts.length - 2], getCanonicalFileName) &&
                !isNodeModulesOrScopedPackageDirectory(bParts[bParts.length - 2], getCanonicalFileName) &&
                getCanonicalFileName(aParts[aParts.length - 1]) === getCanonicalFileName(bParts[bParts.length - 1])) {
                aParts.pop();
                bParts.pop();
            }
            return [ts.getPathFromPathComponents(aParts), ts.getPathFromPathComponents(bParts)];
        }
        // KLUDGE: Don't assume one 'node_modules' links to another. More likely a single directory inside the node_modules is the symlink.
        // ALso, don't assume that an `@foo` directory is linked. More likely the contents of that are linked.
        function isNodeModulesOrScopedPackageDirectory(s, getCanonicalFileName) {
            return getCanonicalFileName(s) === "node_modules" || ts.startsWith(s, "@");
        }
        function stripLeadingDirectorySeparator(s) {
            return ts.isAnyDirectorySeparator(s.charCodeAt(0)) ? s.slice(1) : undefined;
        }
        function tryRemoveDirectoryPrefix(path, dirPath, getCanonicalFileName) {
            var withoutPrefix = ts.tryRemovePrefix(path, dirPath, getCanonicalFileName);
            return withoutPrefix === undefined ? undefined : stripLeadingDirectorySeparator(withoutPrefix);
        }
        ts.tryRemoveDirectoryPrefix = tryRemoveDirectoryPrefix;
        // Reserved characters, forces escaping of any non-word (or digit), non-whitespace character.
        // It may be inefficient (we could just match (/[-[\]{}()*+?.,\\^$|#\s]/g), but this is future
        // proof.
        var reservedCharacterPattern = /[^\w\s\/]/g;
        function regExpEscape(text) {
            return text.replace(reservedCharacterPattern, escapeRegExpCharacter);
        }
        ts.regExpEscape = regExpEscape;
        function escapeRegExpCharacter(match) {
            return "\\" + match;
        }
        var wildcardCharCodes = [42 /* asterisk */, 63 /* question */];
        ts.commonPackageFolders = ["node_modules", "bower_components", "jspm_packages"];
        var implicitExcludePathRegexPattern = "(?!(" + ts.commonPackageFolders.join("|") + ")(/|$))";
        var filesMatcher = {
            /**
             * Matches any single directory segment unless it is the last segment and a .min.js file
             * Breakdown:
             *  [^./]                   # matches everything up to the first . character (excluding directory separators)
             *  (\\.(?!min\\.js$))?     # matches . characters but not if they are part of the .min.js file extension
             */
            singleAsteriskRegexFragment: "([^./]|(\\.(?!min\\.js$))?)*",
            /**
             * Regex for the ** wildcard. Matches any number of subdirectories. When used for including
             * files or directories, does not match subdirectories that start with a . character
             */
            doubleAsteriskRegexFragment: "(/" + implicitExcludePathRegexPattern + "[^/.][^/]*)*?",
            replaceWildcardCharacter: function (match) { return replaceWildcardCharacter(match, filesMatcher.singleAsteriskRegexFragment); }
        };
        var directoriesMatcher = {
            singleAsteriskRegexFragment: "[^/]*",
            /**
             * Regex for the ** wildcard. Matches any number of subdirectories. When used for including
             * files or directories, does not match subdirectories that start with a . character
             */
            doubleAsteriskRegexFragment: "(/" + implicitExcludePathRegexPattern + "[^/.][^/]*)*?",
            replaceWildcardCharacter: function (match) { return replaceWildcardCharacter(match, directoriesMatcher.singleAsteriskRegexFragment); }
        };
        var excludeMatcher = {
            singleAsteriskRegexFragment: "[^/]*",
            doubleAsteriskRegexFragment: "(/.+?)?",
            replaceWildcardCharacter: function (match) { return replaceWildcardCharacter(match, excludeMatcher.singleAsteriskRegexFragment); }
        };
        var wildcardMatchers = {
            files: filesMatcher,
            directories: directoriesMatcher,
            exclude: excludeMatcher
        };
        function getRegularExpressionForWildcard(specs, basePath, usage) {
            var patterns = getRegularExpressionsForWildcards(specs, basePath, usage);
            if (!patterns || !patterns.length) {
                return undefined;
            }
            var pattern = patterns.map(function (pattern) { return "(" + pattern + ")"; }).join("|");
            // If excluding, match "foo/bar/baz...", but if including, only allow "foo".
            var terminator = usage === "exclude" ? "($|/)" : "$";
            return "^(" + pattern + ")" + terminator;
        }
        ts.getRegularExpressionForWildcard = getRegularExpressionForWildcard;
        function getRegularExpressionsForWildcards(specs, basePath, usage) {
            if (specs === undefined || specs.length === 0) {
                return undefined;
            }
            return ts.flatMap(specs, function (spec) {
                return spec && getSubPatternFromSpec(spec, basePath, usage, wildcardMatchers[usage]);
            });
        }
        ts.getRegularExpressionsForWildcards = getRegularExpressionsForWildcards;
        /**
         * An "includes" path "foo" is implicitly a glob "foo/** /*" (without the space) if its last component has no extension,
         * and does not contain any glob characters itself.
         */
        function isImplicitGlob(lastPathComponent) {
            return !/[.*?]/.test(lastPathComponent);
        }
        ts.isImplicitGlob = isImplicitGlob;
        function getSubPatternFromSpec(spec, basePath, usage, _a) {
            var singleAsteriskRegexFragment = _a.singleAsteriskRegexFragment, doubleAsteriskRegexFragment = _a.doubleAsteriskRegexFragment, replaceWildcardCharacter = _a.replaceWildcardCharacter;
            var subpattern = "";
            var hasWrittenComponent = false;
            var components = ts.getNormalizedPathComponents(spec, basePath);
            var lastComponent = ts.last(components);
            if (usage !== "exclude" && lastComponent === "**") {
                return undefined;
            }
            // getNormalizedPathComponents includes the separator for the root component.
            // We need to remove to create our regex correctly.
            components[0] = ts.removeTrailingDirectorySeparator(components[0]);
            if (isImplicitGlob(lastComponent)) {
                components.push("**", "*");
            }
            var optionalCount = 0;
            for (var _i = 0, components_1 = components; _i < components_1.length; _i++) {
                var component = components_1[_i];
                if (component === "**") {
                    subpattern += doubleAsteriskRegexFragment;
                }
                else {
                    if (usage === "directories") {
                        subpattern += "(";
                        optionalCount++;
                    }
                    if (hasWrittenComponent) {
                        subpattern += ts.directorySeparator;
                    }
                    if (usage !== "exclude") {
                        var componentPattern = "";
                        // The * and ? wildcards should not match directories or files that start with . if they
                        // appear first in a component. Dotted directories and files can be included explicitly
                        // like so: **/.*/.*
                        if (component.charCodeAt(0) === 42 /* asterisk */) {
                            componentPattern += "([^./]" + singleAsteriskRegexFragment + ")?";
                            component = component.substr(1);
                        }
                        else if (component.charCodeAt(0) === 63 /* question */) {
                            componentPattern += "[^./]";
                            component = component.substr(1);
                        }
                        componentPattern += component.replace(reservedCharacterPattern, replaceWildcardCharacter);
                        // Patterns should not include subfolders like node_modules unless they are
                        // explicitly included as part of the path.
                        //
                        // As an optimization, if the component pattern is the same as the component,
                        // then there definitely were no wildcard characters and we do not need to
                        // add the exclusion pattern.
                        if (componentPattern !== component) {
                            subpattern += implicitExcludePathRegexPattern;
                        }
                        subpattern += componentPattern;
                    }
                    else {
                        subpattern += component.replace(reservedCharacterPattern, replaceWildcardCharacter);
                    }
                }
                hasWrittenComponent = true;
            }
            while (optionalCount > 0) {
                subpattern += ")?";
                optionalCount--;
            }
            return subpattern;
        }
        function replaceWildcardCharacter(match, singleAsteriskRegexFragment) {
            return match === "*" ? singleAsteriskRegexFragment : match === "?" ? "[^/]" : "\\" + match;
        }
        /** @param path directory of the tsconfig.json */
        function getFileMatcherPatterns(path, excludes, includes, useCaseSensitiveFileNames, currentDirectory) {
            path = ts.normalizePath(path);
            currentDirectory = ts.normalizePath(currentDirectory);
            var absolutePath = ts.combinePaths(currentDirectory, path);
            return {
                includeFilePatterns: ts.map(getRegularExpressionsForWildcards(includes, absolutePath, "files"), function (pattern) { return "^" + pattern + "$"; }),
                includeFilePattern: getRegularExpressionForWildcard(includes, absolutePath, "files"),
                includeDirectoryPattern: getRegularExpressionForWildcard(includes, absolutePath, "directories"),
                excludePattern: getRegularExpressionForWildcard(excludes, absolutePath, "exclude"),
                basePaths: getBasePaths(path, includes, useCaseSensitiveFileNames)
            };
        }
        ts.getFileMatcherPatterns = getFileMatcherPatterns;
        function getRegexFromPattern(pattern, useCaseSensitiveFileNames) {
            return new RegExp(pattern, useCaseSensitiveFileNames ? "" : "i");
        }
        ts.getRegexFromPattern = getRegexFromPattern;
        /** @param path directory of the tsconfig.json */
        function matchFiles(path, extensions, excludes, includes, useCaseSensitiveFileNames, currentDirectory, depth, getFileSystemEntries, realpath) {
            path = ts.normalizePath(path);
            currentDirectory = ts.normalizePath(currentDirectory);
            var patterns = getFileMatcherPatterns(path, excludes, includes, useCaseSensitiveFileNames, currentDirectory);
            var includeFileRegexes = patterns.includeFilePatterns && patterns.includeFilePatterns.map(function (pattern) { return getRegexFromPattern(pattern, useCaseSensitiveFileNames); });
            var includeDirectoryRegex = patterns.includeDirectoryPattern && getRegexFromPattern(patterns.includeDirectoryPattern, useCaseSensitiveFileNames);
            var excludeRegex = patterns.excludePattern && getRegexFromPattern(patterns.excludePattern, useCaseSensitiveFileNames);
            // Associate an array of results with each include regex. This keeps results in order of the "include" order.
            // If there are no "includes", then just put everything in results[0].
            var results = includeFileRegexes ? includeFileRegexes.map(function () { return []; }) : [[]];
            var visited = ts.createMap();
            var toCanonical = ts.createGetCanonicalFileName(useCaseSensitiveFileNames);
            for (var _i = 0, _a = patterns.basePaths; _i < _a.length; _i++) {
                var basePath = _a[_i];
                visitDirectory(basePath, ts.combinePaths(currentDirectory, basePath), depth);
            }
            return ts.flatten(results);
            function visitDirectory(path, absolutePath, depth) {
                var canonicalPath = toCanonical(realpath(absolutePath));
                if (visited.has(canonicalPath))
                    return;
                visited.set(canonicalPath, true);
                var _a = getFileSystemEntries(path), files = _a.files, directories = _a.directories;
                var _loop_1 = function (current) {
                    var name = ts.combinePaths(path, current);
                    var absoluteName = ts.combinePaths(absolutePath, current);
                    if (extensions && !ts.fileExtensionIsOneOf(name, extensions))
                        return "continue";
                    if (excludeRegex && excludeRegex.test(absoluteName))
                        return "continue";
                    if (!includeFileRegexes) {
                        results[0].push(name);
                    }
                    else {
                        var includeIndex = ts.findIndex(includeFileRegexes, function (re) { return re.test(absoluteName); });
                        if (includeIndex !== -1) {
                            results[includeIndex].push(name);
                        }
                    }
                };
                for (var _i = 0, _b = ts.sort(files, ts.compareStringsCaseSensitive); _i < _b.length; _i++) {
                    var current = _b[_i];
                    _loop_1(current);
                }
                if (depth !== undefined) {
                    depth--;
                    if (depth === 0) {
                        return;
                    }
                }
                for (var _c = 0, _d = ts.sort(directories, ts.compareStringsCaseSensitive); _c < _d.length; _c++) {
                    var current = _d[_c];
                    var name = ts.combinePaths(path, current);
                    var absoluteName = ts.combinePaths(absolutePath, current);
                    if ((!includeDirectoryRegex || includeDirectoryRegex.test(absoluteName)) &&
                        (!excludeRegex || !excludeRegex.test(absoluteName))) {
                        visitDirectory(name, absoluteName, depth);
                    }
                }
            }
        }
        ts.matchFiles = matchFiles;
        /**
         * Computes the unique non-wildcard base paths amongst the provided include patterns.
         */
        function getBasePaths(path, includes, useCaseSensitiveFileNames) {
            // Storage for our results in the form of literal paths (e.g. the paths as written by the user).
            var basePaths = [path];
            if (includes) {
                // Storage for literal base paths amongst the include patterns.
                var includeBasePaths = [];
                for (var _i = 0, includes_1 = includes; _i < includes_1.length; _i++) {
                    var include = includes_1[_i];
                    // We also need to check the relative paths by converting them to absolute and normalizing
                    // in case they escape the base path (e.g "..\somedirectory")
                    var absolute = ts.isRootedDiskPath(include) ? include : ts.normalizePath(ts.combinePaths(path, include));
                    // Append the literal and canonical candidate base paths.
                    includeBasePaths.push(getIncludeBasePath(absolute));
                }
                // Sort the offsets array using either the literal or canonical path representations.
                includeBasePaths.sort(ts.getStringComparer(!useCaseSensitiveFileNames));
                var _loop_2 = function (includeBasePath) {
                    if (ts.every(basePaths, function (basePath) { return !ts.containsPath(basePath, includeBasePath, path, !useCaseSensitiveFileNames); })) {
                        basePaths.push(includeBasePath);
                    }
                };
                // Iterate over each include base path and include unique base paths that are not a
                // subpath of an existing base path
                for (var _a = 0, includeBasePaths_1 = includeBasePaths; _a < includeBasePaths_1.length; _a++) {
                    var includeBasePath = includeBasePaths_1[_a];
                    _loop_2(includeBasePath);
                }
            }
            return basePaths;
        }
        function getIncludeBasePath(absolute) {
            var wildcardOffset = ts.indexOfAnyCharCode(absolute, wildcardCharCodes);
            if (wildcardOffset < 0) {
                // No "*" or "?" in the path
                return !ts.hasExtension(absolute)
                    ? absolute
                    : ts.removeTrailingDirectorySeparator(ts.getDirectoryPath(absolute));
            }
            return absolute.substring(0, absolute.lastIndexOf(ts.directorySeparator, wildcardOffset));
        }
        function ensureScriptKind(fileName, scriptKind) {
            // Using scriptKind as a condition handles both:
            // - 'scriptKind' is unspecified and thus it is `undefined`
            // - 'scriptKind' is set and it is `Unknown` (0)
            // If the 'scriptKind' is 'undefined' or 'Unknown' then we attempt
            // to get the ScriptKind from the file name. If it cannot be resolved
            // from the file name then the default 'TS' script kind is returned.
            return scriptKind || getScriptKindFromFileName(fileName) || 3 /* TS */;
        }
        ts.ensureScriptKind = ensureScriptKind;
        function getScriptKindFromFileName(fileName) {
            var ext = fileName.substr(fileName.lastIndexOf("."));
            switch (ext.toLowerCase()) {
                case ".js" /* Js */:
                    return 1 /* JS */;
                case ".jsx" /* Jsx */:
                    return 2 /* JSX */;
                case ".ts" /* Ts */:
                    return 3 /* TS */;
                case ".tsx" /* Tsx */:
                    return 4 /* TSX */;
                case ".json" /* Json */:
                    return 6 /* JSON */;
                default:
                    return 0 /* Unknown */;
            }
        }
        ts.getScriptKindFromFileName = getScriptKindFromFileName;
        /**
         *  List of supported extensions in order of file resolution precedence.
         */
        ts.supportedTSExtensions = [".ts" /* Ts */, ".tsx" /* Tsx */, ".d.ts" /* Dts */];
        ts.supportedTSExtensionsWithJson = [".ts" /* Ts */, ".tsx" /* Tsx */, ".d.ts" /* Dts */, ".json" /* Json */];
        /** Must have ".d.ts" first because if ".ts" goes first, that will be detected as the extension instead of ".d.ts". */
        ts.supportedTSExtensionsForExtractExtension = [".d.ts" /* Dts */, ".ts" /* Ts */, ".tsx" /* Tsx */];
        ts.supportedJSExtensions = [".js" /* Js */, ".jsx" /* Jsx */];
        ts.supportedJSAndJsonExtensions = [".js" /* Js */, ".jsx" /* Jsx */, ".json" /* Json */];
        var allSupportedExtensions = __spreadArrays(ts.supportedTSExtensions, ts.supportedJSExtensions);
        var allSupportedExtensionsWithJson = __spreadArrays(ts.supportedTSExtensions, ts.supportedJSExtensions, [".json" /* Json */]);
        function getSupportedExtensions(options, extraFileExtensions) {
            var needJsExtensions = options && options.allowJs;
            if (!extraFileExtensions || extraFileExtensions.length === 0) {
                return needJsExtensions ? allSupportedExtensions : ts.supportedTSExtensions;
            }
            var extensions = __spreadArrays(needJsExtensions ? allSupportedExtensions : ts.supportedTSExtensions, ts.mapDefined(extraFileExtensions, function (x) { return x.scriptKind === 7 /* Deferred */ || needJsExtensions && isJSLike(x.scriptKind) ? x.extension : undefined; }));
            return ts.deduplicate(extensions, ts.equateStringsCaseSensitive, ts.compareStringsCaseSensitive);
        }
        ts.getSupportedExtensions = getSupportedExtensions;
        function getSuppoertedExtensionsWithJsonIfResolveJsonModule(options, supportedExtensions) {
            if (!options || !options.resolveJsonModule) {
                return supportedExtensions;
            }
            if (supportedExtensions === allSupportedExtensions) {
                return allSupportedExtensionsWithJson;
            }
            if (supportedExtensions === ts.supportedTSExtensions) {
                return ts.supportedTSExtensionsWithJson;
            }
            return __spreadArrays(supportedExtensions, [".json" /* Json */]);
        }
        ts.getSuppoertedExtensionsWithJsonIfResolveJsonModule = getSuppoertedExtensionsWithJsonIfResolveJsonModule;
        function isJSLike(scriptKind) {
            return scriptKind === 1 /* JS */ || scriptKind === 2 /* JSX */;
        }
        function hasJSFileExtension(fileName) {
            return ts.some(ts.supportedJSExtensions, function (extension) { return ts.fileExtensionIs(fileName, extension); });
        }
        ts.hasJSFileExtension = hasJSFileExtension;
        function hasTSFileExtension(fileName) {
            return ts.some(ts.supportedTSExtensions, function (extension) { return ts.fileExtensionIs(fileName, extension); });
        }
        ts.hasTSFileExtension = hasTSFileExtension;
        function isSupportedSourceFileName(fileName, compilerOptions, extraFileExtensions) {
            if (!fileName) {
                return false;
            }
            var supportedExtensions = getSupportedExtensions(compilerOptions, extraFileExtensions);
            for (var _i = 0, _a = getSuppoertedExtensionsWithJsonIfResolveJsonModule(compilerOptions, supportedExtensions); _i < _a.length; _i++) {
                var extension = _a[_i];
                if (ts.fileExtensionIs(fileName, extension)) {
                    return true;
                }
            }
            return false;
        }
        ts.isSupportedSourceFileName = isSupportedSourceFileName;
        /**
         * Extension boundaries by priority. Lower numbers indicate higher priorities, and are
         * aligned to the offset of the highest priority extension in the
         * allSupportedExtensions array.
         */
        var ExtensionPriority;
        (function (ExtensionPriority) {
            ExtensionPriority[ExtensionPriority["TypeScriptFiles"] = 0] = "TypeScriptFiles";
            ExtensionPriority[ExtensionPriority["DeclarationAndJavaScriptFiles"] = 2] = "DeclarationAndJavaScriptFiles";
            ExtensionPriority[ExtensionPriority["Highest"] = 0] = "Highest";
            ExtensionPriority[ExtensionPriority["Lowest"] = 2] = "Lowest";
        })(ExtensionPriority = ts.ExtensionPriority || (ts.ExtensionPriority = {}));
        function getExtensionPriority(path, supportedExtensions) {
            for (var i = supportedExtensions.length - 1; i >= 0; i--) {
                if (ts.fileExtensionIs(path, supportedExtensions[i])) {
                    return adjustExtensionPriority(i, supportedExtensions);
                }
            }
            // If its not in the list of supported extensions, this is likely a
            // TypeScript file with a non-ts extension
            return 0 /* Highest */;
        }
        ts.getExtensionPriority = getExtensionPriority;
        /**
         * Adjusts an extension priority to be the highest priority within the same range.
         */
        function adjustExtensionPriority(extensionPriority, supportedExtensions) {
            if (extensionPriority < 2 /* DeclarationAndJavaScriptFiles */) {
                return 0 /* TypeScriptFiles */;
            }
            else if (extensionPriority < supportedExtensions.length) {
                return 2 /* DeclarationAndJavaScriptFiles */;
            }
            else {
                return supportedExtensions.length;
            }
        }
        ts.adjustExtensionPriority = adjustExtensionPriority;
        /**
         * Gets the next lowest extension priority for a given priority.
         */
        function getNextLowestExtensionPriority(extensionPriority, supportedExtensions) {
            if (extensionPriority < 2 /* DeclarationAndJavaScriptFiles */) {
                return 2 /* DeclarationAndJavaScriptFiles */;
            }
            else {
                return supportedExtensions.length;
            }
        }
        ts.getNextLowestExtensionPriority = getNextLowestExtensionPriority;
        var extensionsToRemove = [".d.ts" /* Dts */, ".ts" /* Ts */, ".js" /* Js */, ".tsx" /* Tsx */, ".jsx" /* Jsx */, ".json" /* Json */];
        function removeFileExtension(path) {
            for (var _i = 0, extensionsToRemove_1 = extensionsToRemove; _i < extensionsToRemove_1.length; _i++) {
                var ext = extensionsToRemove_1[_i];
                var extensionless = tryRemoveExtension(path, ext);
                if (extensionless !== undefined) {
                    return extensionless;
                }
            }
            return path;
        }
        ts.removeFileExtension = removeFileExtension;
        function tryRemoveExtension(path, extension) {
            return ts.fileExtensionIs(path, extension) ? removeExtension(path, extension) : undefined;
        }
        ts.tryRemoveExtension = tryRemoveExtension;
        function removeExtension(path, extension) {
            return path.substring(0, path.length - extension.length);
        }
        ts.removeExtension = removeExtension;
        function changeExtension(path, newExtension) {
            return ts.changeAnyExtension(path, newExtension, extensionsToRemove, /*ignoreCase*/ false);
        }
        ts.changeExtension = changeExtension;
        function tryParsePattern(pattern) {
            // This should be verified outside of here and a proper error thrown.
            ts.Debug.assert(hasZeroOrOneAsteriskCharacter(pattern));
            var indexOfStar = pattern.indexOf("*");
            return indexOfStar === -1 ? undefined : {
                prefix: pattern.substr(0, indexOfStar),
                suffix: pattern.substr(indexOfStar + 1)
            };
        }
        ts.tryParsePattern = tryParsePattern;
        function positionIsSynthesized(pos) {
            // This is a fast way of testing the following conditions:
            //  pos === undefined || pos === null || isNaN(pos) || pos < 0;
            return !(pos >= 0);
        }
        ts.positionIsSynthesized = positionIsSynthesized;
        /** True if an extension is one of the supported TypeScript extensions. */
        function extensionIsTS(ext) {
            return ext === ".ts" /* Ts */ || ext === ".tsx" /* Tsx */ || ext === ".d.ts" /* Dts */;
        }
        ts.extensionIsTS = extensionIsTS;
        function resolutionExtensionIsTSOrJson(ext) {
            return extensionIsTS(ext) || ext === ".json" /* Json */;
        }
        ts.resolutionExtensionIsTSOrJson = resolutionExtensionIsTSOrJson;
        /**
         * Gets the extension from a path.
         * Path must have a valid extension.
         */
        function extensionFromPath(path) {
            var ext = tryGetExtensionFromPath(path);
            return ext !== undefined ? ext : ts.Debug.fail("File " + path + " has unknown extension.");
        }
        ts.extensionFromPath = extensionFromPath;
        function isAnySupportedFileExtension(path) {
            return tryGetExtensionFromPath(path) !== undefined;
        }
        ts.isAnySupportedFileExtension = isAnySupportedFileExtension;
        function tryGetExtensionFromPath(path) {
            return ts.find(extensionsToRemove, function (e) { return ts.fileExtensionIs(path, e); });
        }
        ts.tryGetExtensionFromPath = tryGetExtensionFromPath;
        function isCheckJsEnabledForFile(sourceFile, compilerOptions) {
            return sourceFile.checkJsDirective ? sourceFile.checkJsDirective.enabled : compilerOptions.checkJs;
        }
        ts.isCheckJsEnabledForFile = isCheckJsEnabledForFile;
        ts.emptyFileSystemEntries = {
            files: ts.emptyArray,
            directories: ts.emptyArray
        };
        /**
         * patternStrings contains both pattern strings (containing "*") and regular strings.
         * Return an exact match if possible, or a pattern match, or undefined.
         * (These are verified by verifyCompilerOptions to have 0 or 1 "*" characters.)
         */
        function matchPatternOrExact(patternStrings, candidate) {
            var patterns = [];
            for (var _i = 0, patternStrings_1 = patternStrings; _i < patternStrings_1.length; _i++) {
                var patternString = patternStrings_1[_i];
                if (!hasZeroOrOneAsteriskCharacter(patternString))
                    continue;
                var pattern = tryParsePattern(patternString);
                if (pattern) {
                    patterns.push(pattern);
                }
                else if (patternString === candidate) {
                    // pattern was matched as is - no need to search further
                    return patternString;
                }
            }
            return ts.findBestPatternMatch(patterns, function (_) { return _; }, candidate);
        }
        ts.matchPatternOrExact = matchPatternOrExact;
        function sliceAfter(arr, value) {
            var index = arr.indexOf(value);
            ts.Debug.assert(index !== -1);
            return arr.slice(index);
        }
        ts.sliceAfter = sliceAfter;
        function addRelatedInfo(diagnostic) {
            var _a;
            var relatedInformation = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                relatedInformation[_i - 1] = arguments[_i];
            }
            if (!relatedInformation.length) {
                return diagnostic;
            }
            if (!diagnostic.relatedInformation) {
                diagnostic.relatedInformation = [];
            }
            (_a = diagnostic.relatedInformation).push.apply(_a, relatedInformation);
            return diagnostic;
        }
        ts.addRelatedInfo = addRelatedInfo;
        function minAndMax(arr, getValue) {
            ts.Debug.assert(arr.length !== 0);
            var min = getValue(arr[0]);
            var max = min;
            for (var i = 1; i < arr.length; i++) {
                var value = getValue(arr[i]);
                if (value < min) {
                    min = value;
                }
                else if (value > max) {
                    max = value;
                }
            }
            return { min: min, max: max };
        }
        ts.minAndMax = minAndMax;
        var NodeSet = /** @class */ (function () {
            function NodeSet() {
                this.map = ts.createMap();
            }
            NodeSet.prototype.add = function (node) {
                this.map.set(String(ts.getNodeId(node)), node);
            };
            NodeSet.prototype.tryAdd = function (node) {
                if (this.has(node))
                    return false;
                this.add(node);
                return true;
            };
            NodeSet.prototype.has = function (node) {
                return this.map.has(String(ts.getNodeId(node)));
            };
            NodeSet.prototype.forEach = function (cb) {
                this.map.forEach(cb);
            };
            NodeSet.prototype.some = function (pred) {
                return forEachEntry(this.map, pred) || false;
            };
            return NodeSet;
        }());
        ts.NodeSet = NodeSet;
        var NodeMap = /** @class */ (function () {
            function NodeMap() {
                this.map = ts.createMap();
            }
            NodeMap.prototype.get = function (node) {
                var res = this.map.get(String(ts.getNodeId(node)));
                return res && res.value;
            };
            NodeMap.prototype.getOrUpdate = function (node, setValue) {
                var res = this.get(node);
                if (res)
                    return res;
                var value = setValue();
                this.set(node, value);
                return value;
            };
            NodeMap.prototype.set = function (node, value) {
                this.map.set(String(ts.getNodeId(node)), { node: node, value: value });
            };
            NodeMap.prototype.has = function (node) {
                return this.map.has(String(ts.getNodeId(node)));
            };
            NodeMap.prototype.forEach = function (cb) {
                this.map.forEach(function (_a) {
                    var node = _a.node, value = _a.value;
                    return cb(value, node);
                });
            };
            return NodeMap;
        }());
        ts.NodeMap = NodeMap;
        function rangeOfNode(node) {
            return { pos: getTokenPosOfNode(node), end: node.end };
        }
        ts.rangeOfNode = rangeOfNode;
        function rangeOfTypeParameters(typeParameters) {
            // Include the `<>`
            return { pos: typeParameters.pos - 1, end: typeParameters.end + 1 };
        }
        ts.rangeOfTypeParameters = rangeOfTypeParameters;
        function skipTypeChecking(sourceFile, options, host) {
            // If skipLibCheck is enabled, skip reporting errors if file is a declaration file.
            // If skipDefaultLibCheck is enabled, skip reporting errors if file contains a
            // '/// <reference no-default-lib="true"/>' directive.
            return (options.skipLibCheck && sourceFile.isDeclarationFile ||
                options.skipDefaultLibCheck && sourceFile.hasNoDefaultLib) ||
                host.isSourceOfProjectReferenceRedirect(sourceFile.fileName);
        }
        ts.skipTypeChecking = skipTypeChecking;
        function isJsonEqual(a, b) {
            // eslint-disable-next-line no-null/no-null
            return a === b || typeof a === "object" && a !== null && typeof b === "object" && b !== null && ts.equalOwnProperties(a, b, isJsonEqual);
        }
        ts.isJsonEqual = isJsonEqual;
        function getOrUpdate(map, key, getDefault) {
            var got = map.get(key);
            if (got === undefined) {
                var value = getDefault();
                map.set(key, value);
                return value;
            }
            else {
                return got;
            }
        }
        ts.getOrUpdate = getOrUpdate;
        /**
         * Converts a bigint literal string, e.g. `0x1234n`,
         * to its decimal string representation, e.g. `4660`.
         */
        function parsePseudoBigInt(stringValue) {
            var log2Base;
            switch (stringValue.charCodeAt(1)) { // "x" in "0x123"
                case 98 /* b */:
                case 66 /* B */: // 0b or 0B
                    log2Base = 1;
                    break;
                case 111 /* o */:
                case 79 /* O */: // 0o or 0O
                    log2Base = 3;
                    break;
                case 120 /* x */:
                case 88 /* X */: // 0x or 0X
                    log2Base = 4;
                    break;
                default: // already in decimal; omit trailing "n"
                    var nIndex = stringValue.length - 1;
                    // Skip leading 0s
                    var nonZeroStart = 0;
                    while (stringValue.charCodeAt(nonZeroStart) === 48 /* _0 */) {
                        nonZeroStart++;
                    }
                    return stringValue.slice(nonZeroStart, nIndex) || "0";
            }
            // Omit leading "0b", "0o", or "0x", and trailing "n"
            var startIndex = 2, endIndex = stringValue.length - 1;
            var bitsNeeded = (endIndex - startIndex) * log2Base;
            // Stores the value specified by the string as a LE array of 16-bit integers
            // using Uint16 instead of Uint32 so combining steps can use bitwise operators
            var segments = new Uint16Array((bitsNeeded >>> 4) + (bitsNeeded & 15 ? 1 : 0));
            // Add the digits, one at a time
            for (var i = endIndex - 1, bitOffset = 0; i >= startIndex; i--, bitOffset += log2Base) {
                var segment = bitOffset >>> 4;
                var digitChar = stringValue.charCodeAt(i);
                // Find character range: 0-9 < A-F < a-f
                var digit = digitChar <= 57 /* _9 */
                    ? digitChar - 48 /* _0 */
                    : 10 + digitChar -
                        (digitChar <= 70 /* F */ ? 65 /* A */ : 97 /* a */);
                var shiftedDigit = digit << (bitOffset & 15);
                segments[segment] |= shiftedDigit;
                var residual = shiftedDigit >>> 16;
                if (residual)
                    segments[segment + 1] |= residual; // overflows segment
            }
            // Repeatedly divide segments by 10 and add remainder to base10Value
            var base10Value = "";
            var firstNonzeroSegment = segments.length - 1;
            var segmentsRemaining = true;
            while (segmentsRemaining) {
                var mod10 = 0;
                segmentsRemaining = false;
                for (var segment = firstNonzeroSegment; segment >= 0; segment--) {
                    var newSegment = mod10 << 16 | segments[segment];
                    var segmentValue = (newSegment / 10) | 0;
                    segments[segment] = segmentValue;
                    mod10 = newSegment - segmentValue * 10;
                    if (segmentValue && !segmentsRemaining) {
                        firstNonzeroSegment = segment;
                        segmentsRemaining = true;
                    }
                }
                base10Value = mod10 + base10Value;
            }
            return base10Value;
        }
        ts.parsePseudoBigInt = parsePseudoBigInt;
        function pseudoBigIntToString(_a) {
            var negative = _a.negative, base10Value = _a.base10Value;
            return (negative && base10Value !== "0" ? "-" : "") + base10Value;
        }
        ts.pseudoBigIntToString = pseudoBigIntToString;
        function isValidTypeOnlyAliasUseSite(useSite) {
            return !!(useSite.flags & 8388608 /* Ambient */)
                || isPartOfTypeQuery(useSite)
                || isFirstIdentifierOfNonEmittingHeritageClause(useSite)
                || isPartOfPossiblyValidTypeOrAbstractComputedPropertyName(useSite)
                || !isExpressionNode(useSite);
        }
        ts.isValidTypeOnlyAliasUseSite = isValidTypeOnlyAliasUseSite;
        function typeOnlyDeclarationIsExport(typeOnlyDeclaration) {
            return typeOnlyDeclaration.kind === 263 /* ExportSpecifier */;
        }
        ts.typeOnlyDeclarationIsExport = typeOnlyDeclarationIsExport;
        function isPartOfPossiblyValidTypeOrAbstractComputedPropertyName(node) {
            while (node.kind === 75 /* Identifier */ || node.kind === 194 /* PropertyAccessExpression */) {
                node = node.parent;
            }
            if (node.kind !== 154 /* ComputedPropertyName */) {
                return false;
            }
            if (hasModifier(node.parent, 128 /* Abstract */)) {
                return true;
            }
            var containerKind = node.parent.parent.kind;
            return containerKind === 246 /* InterfaceDeclaration */ || containerKind === 173 /* TypeLiteral */;
        }
        /** Returns true for the first identifier of 1) an `implements` clause, and 2) an `extends` clause of an interface. */
        function isFirstIdentifierOfNonEmittingHeritageClause(node) {
            var _a, _b;
            // Number of parents to climb from identifier is 2 for `implements I`, 3 for `implements x.I`
            var heritageClause = (_a = ts.tryCast(node.parent.parent, ts.isHeritageClause)) !== null && _a !== void 0 ? _a : ts.tryCast((_b = node.parent.parent) === null || _b === void 0 ? void 0 : _b.parent, ts.isHeritageClause);
            return (heritageClause === null || heritageClause === void 0 ? void 0 : heritageClause.token) === 113 /* ImplementsKeyword */ || (heritageClause === null || heritageClause === void 0 ? void 0 : heritageClause.parent.kind) === 246 /* InterfaceDeclaration */;
        }
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        var SignatureFlags;
        (function (SignatureFlags) {
            SignatureFlags[SignatureFlags["None"] = 0] = "None";
            SignatureFlags[SignatureFlags["Yield"] = 1] = "Yield";
            SignatureFlags[SignatureFlags["Await"] = 2] = "Await";
            SignatureFlags[SignatureFlags["Type"] = 4] = "Type";
            SignatureFlags[SignatureFlags["IgnoreMissingOpenBrace"] = 16] = "IgnoreMissingOpenBrace";
            SignatureFlags[SignatureFlags["JSDoc"] = 32] = "JSDoc";
        })(SignatureFlags || (SignatureFlags = {}));
        var NodeConstructor;
        var TokenConstructor;
        var IdentifierConstructor;
        var PrivateIdentifierConstructor;
        var SourceFileConstructor;
        function createNode(kind, pos, end) {
            if (kind === 290 /* SourceFile */) {
                return new (SourceFileConstructor || (SourceFileConstructor = ts.objectAllocator.getSourceFileConstructor()))(kind, pos, end);
            }
            else if (kind === 75 /* Identifier */) {
                return new (IdentifierConstructor || (IdentifierConstructor = ts.objectAllocator.getIdentifierConstructor()))(kind, pos, end);
            }
            else if (kind === 76 /* PrivateIdentifier */) {
                return new (PrivateIdentifierConstructor || (PrivateIdentifierConstructor = ts.objectAllocator.getPrivateIdentifierConstructor()))(kind, pos, end);
            }
            else if (!ts.isNodeKind(kind)) {
                return new (TokenConstructor || (TokenConstructor = ts.objectAllocator.getTokenConstructor()))(kind, pos, end);
            }
            else {
                return new (NodeConstructor || (NodeConstructor = ts.objectAllocator.getNodeConstructor()))(kind, pos, end);
            }
        }
        ts.createNode = createNode;
        function visitNode(cbNode, node) {
            return node && cbNode(node);
        }
        function visitNodes(cbNode, cbNodes, nodes) {
            if (nodes) {
                if (cbNodes) {
                    return cbNodes(nodes);
                }
                for (var _i = 0, nodes_1 = nodes; _i < nodes_1.length; _i++) {
                    var node = nodes_1[_i];
                    var result = cbNode(node);
                    if (result) {
                        return result;
                    }
                }
            }
        }
        /*@internal*/
        function isJSDocLikeText(text, start) {
            return text.charCodeAt(start + 1) === 42 /* asterisk */ &&
                text.charCodeAt(start + 2) === 42 /* asterisk */ &&
                text.charCodeAt(start + 3) !== 47 /* slash */;
        }
        ts.isJSDocLikeText = isJSDocLikeText;
        /**
         * Invokes a callback for each child of the given node. The 'cbNode' callback is invoked for all child nodes
         * stored in properties. If a 'cbNodes' callback is specified, it is invoked for embedded arrays; otherwise,
         * embedded arrays are flattened and the 'cbNode' callback is invoked for each element. If a callback returns
         * a truthy value, iteration stops and that value is returned. Otherwise, undefined is returned.
         *
         * @param node a given node to visit its children
         * @param cbNode a callback to be invoked for all child nodes
         * @param cbNodes a callback to be invoked for embedded array
         *
         * @remarks `forEachChild` must visit the children of a node in the order
         * that they appear in the source code. The language service depends on this property to locate nodes by position.
         */
        function forEachChild(node, cbNode, cbNodes) {
            if (!node || node.kind <= 152 /* LastToken */) {
                return;
            }
            switch (node.kind) {
                case 153 /* QualifiedName */:
                    return visitNode(cbNode, node.left) ||
                        visitNode(cbNode, node.right);
                case 155 /* TypeParameter */:
                    return visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.constraint) ||
                        visitNode(cbNode, node.default) ||
                        visitNode(cbNode, node.expression);
                case 282 /* ShorthandPropertyAssignment */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.questionToken) ||
                        visitNode(cbNode, node.exclamationToken) ||
                        visitNode(cbNode, node.equalsToken) ||
                        visitNode(cbNode, node.objectAssignmentInitializer);
                case 283 /* SpreadAssignment */:
                    return visitNode(cbNode, node.expression);
                case 156 /* Parameter */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.dotDotDotToken) ||
                        visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.questionToken) ||
                        visitNode(cbNode, node.type) ||
                        visitNode(cbNode, node.initializer);
                case 159 /* PropertyDeclaration */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.questionToken) ||
                        visitNode(cbNode, node.exclamationToken) ||
                        visitNode(cbNode, node.type) ||
                        visitNode(cbNode, node.initializer);
                case 158 /* PropertySignature */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.questionToken) ||
                        visitNode(cbNode, node.type) ||
                        visitNode(cbNode, node.initializer);
                case 281 /* PropertyAssignment */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.questionToken) ||
                        visitNode(cbNode, node.initializer);
                case 242 /* VariableDeclaration */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.exclamationToken) ||
                        visitNode(cbNode, node.type) ||
                        visitNode(cbNode, node.initializer);
                case 191 /* BindingElement */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.dotDotDotToken) ||
                        visitNode(cbNode, node.propertyName) ||
                        visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.initializer);
                case 170 /* FunctionType */:
                case 171 /* ConstructorType */:
                case 165 /* CallSignature */:
                case 166 /* ConstructSignature */:
                case 167 /* IndexSignature */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNodes(cbNode, cbNodes, node.typeParameters) ||
                        visitNodes(cbNode, cbNodes, node.parameters) ||
                        visitNode(cbNode, node.type);
                case 161 /* MethodDeclaration */:
                case 160 /* MethodSignature */:
                case 162 /* Constructor */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 201 /* FunctionExpression */:
                case 244 /* FunctionDeclaration */:
                case 202 /* ArrowFunction */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.asteriskToken) ||
                        visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.questionToken) ||
                        visitNode(cbNode, node.exclamationToken) ||
                        visitNodes(cbNode, cbNodes, node.typeParameters) ||
                        visitNodes(cbNode, cbNodes, node.parameters) ||
                        visitNode(cbNode, node.type) ||
                        visitNode(cbNode, node.equalsGreaterThanToken) ||
                        visitNode(cbNode, node.body);
                case 169 /* TypeReference */:
                    return visitNode(cbNode, node.typeName) ||
                        visitNodes(cbNode, cbNodes, node.typeArguments);
                case 168 /* TypePredicate */:
                    return visitNode(cbNode, node.assertsModifier) ||
                        visitNode(cbNode, node.parameterName) ||
                        visitNode(cbNode, node.type);
                case 172 /* TypeQuery */:
                    return visitNode(cbNode, node.exprName);
                case 173 /* TypeLiteral */:
                    return visitNodes(cbNode, cbNodes, node.members);
                case 174 /* ArrayType */:
                    return visitNode(cbNode, node.elementType);
                case 175 /* TupleType */:
                    return visitNodes(cbNode, cbNodes, node.elementTypes);
                case 178 /* UnionType */:
                case 179 /* IntersectionType */:
                    return visitNodes(cbNode, cbNodes, node.types);
                case 180 /* ConditionalType */:
                    return visitNode(cbNode, node.checkType) ||
                        visitNode(cbNode, node.extendsType) ||
                        visitNode(cbNode, node.trueType) ||
                        visitNode(cbNode, node.falseType);
                case 181 /* InferType */:
                    return visitNode(cbNode, node.typeParameter);
                case 188 /* ImportType */:
                    return visitNode(cbNode, node.argument) ||
                        visitNode(cbNode, node.qualifier) ||
                        visitNodes(cbNode, cbNodes, node.typeArguments);
                case 182 /* ParenthesizedType */:
                case 184 /* TypeOperator */:
                    return visitNode(cbNode, node.type);
                case 185 /* IndexedAccessType */:
                    return visitNode(cbNode, node.objectType) ||
                        visitNode(cbNode, node.indexType);
                case 186 /* MappedType */:
                    return visitNode(cbNode, node.readonlyToken) ||
                        visitNode(cbNode, node.typeParameter) ||
                        visitNode(cbNode, node.questionToken) ||
                        visitNode(cbNode, node.type);
                case 187 /* LiteralType */:
                    return visitNode(cbNode, node.literal);
                case 189 /* ObjectBindingPattern */:
                case 190 /* ArrayBindingPattern */:
                    return visitNodes(cbNode, cbNodes, node.elements);
                case 192 /* ArrayLiteralExpression */:
                    return visitNodes(cbNode, cbNodes, node.elements);
                case 193 /* ObjectLiteralExpression */:
                    return visitNodes(cbNode, cbNodes, node.properties);
                case 194 /* PropertyAccessExpression */:
                    return visitNode(cbNode, node.expression) ||
                        visitNode(cbNode, node.questionDotToken) ||
                        visitNode(cbNode, node.name);
                case 195 /* ElementAccessExpression */:
                    return visitNode(cbNode, node.expression) ||
                        visitNode(cbNode, node.questionDotToken) ||
                        visitNode(cbNode, node.argumentExpression);
                case 196 /* CallExpression */:
                case 197 /* NewExpression */:
                    return visitNode(cbNode, node.expression) ||
                        visitNode(cbNode, node.questionDotToken) ||
                        visitNodes(cbNode, cbNodes, node.typeArguments) ||
                        visitNodes(cbNode, cbNodes, node.arguments);
                case 198 /* TaggedTemplateExpression */:
                    return visitNode(cbNode, node.tag) ||
                        visitNode(cbNode, node.questionDotToken) ||
                        visitNodes(cbNode, cbNodes, node.typeArguments) ||
                        visitNode(cbNode, node.template);
                case 199 /* TypeAssertionExpression */:
                    return visitNode(cbNode, node.type) ||
                        visitNode(cbNode, node.expression);
                case 200 /* ParenthesizedExpression */:
                    return visitNode(cbNode, node.expression);
                case 203 /* DeleteExpression */:
                    return visitNode(cbNode, node.expression);
                case 204 /* TypeOfExpression */:
                    return visitNode(cbNode, node.expression);
                case 205 /* VoidExpression */:
                    return visitNode(cbNode, node.expression);
                case 207 /* PrefixUnaryExpression */:
                    return visitNode(cbNode, node.operand);
                case 212 /* YieldExpression */:
                    return visitNode(cbNode, node.asteriskToken) ||
                        visitNode(cbNode, node.expression);
                case 206 /* AwaitExpression */:
                    return visitNode(cbNode, node.expression);
                case 208 /* PostfixUnaryExpression */:
                    return visitNode(cbNode, node.operand);
                case 209 /* BinaryExpression */:
                    return visitNode(cbNode, node.left) ||
                        visitNode(cbNode, node.operatorToken) ||
                        visitNode(cbNode, node.right);
                case 217 /* AsExpression */:
                    return visitNode(cbNode, node.expression) ||
                        visitNode(cbNode, node.type);
                case 218 /* NonNullExpression */:
                    return visitNode(cbNode, node.expression);
                case 219 /* MetaProperty */:
                    return visitNode(cbNode, node.name);
                case 210 /* ConditionalExpression */:
                    return visitNode(cbNode, node.condition) ||
                        visitNode(cbNode, node.questionToken) ||
                        visitNode(cbNode, node.whenTrue) ||
                        visitNode(cbNode, node.colonToken) ||
                        visitNode(cbNode, node.whenFalse);
                case 213 /* SpreadElement */:
                    return visitNode(cbNode, node.expression);
                case 223 /* Block */:
                case 250 /* ModuleBlock */:
                    return visitNodes(cbNode, cbNodes, node.statements);
                case 290 /* SourceFile */:
                    return visitNodes(cbNode, cbNodes, node.statements) ||
                        visitNode(cbNode, node.endOfFileToken);
                case 225 /* VariableStatement */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.declarationList);
                case 243 /* VariableDeclarationList */:
                    return visitNodes(cbNode, cbNodes, node.declarations);
                case 226 /* ExpressionStatement */:
                    return visitNode(cbNode, node.expression);
                case 227 /* IfStatement */:
                    return visitNode(cbNode, node.expression) ||
                        visitNode(cbNode, node.thenStatement) ||
                        visitNode(cbNode, node.elseStatement);
                case 228 /* DoStatement */:
                    return visitNode(cbNode, node.statement) ||
                        visitNode(cbNode, node.expression);
                case 229 /* WhileStatement */:
                    return visitNode(cbNode, node.expression) ||
                        visitNode(cbNode, node.statement);
                case 230 /* ForStatement */:
                    return visitNode(cbNode, node.initializer) ||
                        visitNode(cbNode, node.condition) ||
                        visitNode(cbNode, node.incrementor) ||
                        visitNode(cbNode, node.statement);
                case 231 /* ForInStatement */:
                    return visitNode(cbNode, node.initializer) ||
                        visitNode(cbNode, node.expression) ||
                        visitNode(cbNode, node.statement);
                case 232 /* ForOfStatement */:
                    return visitNode(cbNode, node.awaitModifier) ||
                        visitNode(cbNode, node.initializer) ||
                        visitNode(cbNode, node.expression) ||
                        visitNode(cbNode, node.statement);
                case 233 /* ContinueStatement */:
                case 234 /* BreakStatement */:
                    return visitNode(cbNode, node.label);
                case 235 /* ReturnStatement */:
                    return visitNode(cbNode, node.expression);
                case 236 /* WithStatement */:
                    return visitNode(cbNode, node.expression) ||
                        visitNode(cbNode, node.statement);
                case 237 /* SwitchStatement */:
                    return visitNode(cbNode, node.expression) ||
                        visitNode(cbNode, node.caseBlock);
                case 251 /* CaseBlock */:
                    return visitNodes(cbNode, cbNodes, node.clauses);
                case 277 /* CaseClause */:
                    return visitNode(cbNode, node.expression) ||
                        visitNodes(cbNode, cbNodes, node.statements);
                case 278 /* DefaultClause */:
                    return visitNodes(cbNode, cbNodes, node.statements);
                case 238 /* LabeledStatement */:
                    return visitNode(cbNode, node.label) ||
                        visitNode(cbNode, node.statement);
                case 239 /* ThrowStatement */:
                    return visitNode(cbNode, node.expression);
                case 240 /* TryStatement */:
                    return visitNode(cbNode, node.tryBlock) ||
                        visitNode(cbNode, node.catchClause) ||
                        visitNode(cbNode, node.finallyBlock);
                case 280 /* CatchClause */:
                    return visitNode(cbNode, node.variableDeclaration) ||
                        visitNode(cbNode, node.block);
                case 157 /* Decorator */:
                    return visitNode(cbNode, node.expression);
                case 245 /* ClassDeclaration */:
                case 214 /* ClassExpression */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.name) ||
                        visitNodes(cbNode, cbNodes, node.typeParameters) ||
                        visitNodes(cbNode, cbNodes, node.heritageClauses) ||
                        visitNodes(cbNode, cbNodes, node.members);
                case 246 /* InterfaceDeclaration */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.name) ||
                        visitNodes(cbNode, cbNodes, node.typeParameters) ||
                        visitNodes(cbNode, cbNodes, node.heritageClauses) ||
                        visitNodes(cbNode, cbNodes, node.members);
                case 247 /* TypeAliasDeclaration */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.name) ||
                        visitNodes(cbNode, cbNodes, node.typeParameters) ||
                        visitNode(cbNode, node.type);
                case 248 /* EnumDeclaration */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.name) ||
                        visitNodes(cbNode, cbNodes, node.members);
                case 284 /* EnumMember */:
                    return visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.initializer);
                case 249 /* ModuleDeclaration */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.body);
                case 253 /* ImportEqualsDeclaration */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.moduleReference);
                case 254 /* ImportDeclaration */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.importClause) ||
                        visitNode(cbNode, node.moduleSpecifier);
                case 255 /* ImportClause */:
                    return visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.namedBindings);
                case 252 /* NamespaceExportDeclaration */:
                    return visitNode(cbNode, node.name);
                case 256 /* NamespaceImport */:
                    return visitNode(cbNode, node.name);
                case 262 /* NamespaceExport */:
                    return visitNode(cbNode, node.name);
                case 257 /* NamedImports */:
                case 261 /* NamedExports */:
                    return visitNodes(cbNode, cbNodes, node.elements);
                case 260 /* ExportDeclaration */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.exportClause) ||
                        visitNode(cbNode, node.moduleSpecifier);
                case 258 /* ImportSpecifier */:
                case 263 /* ExportSpecifier */:
                    return visitNode(cbNode, node.propertyName) ||
                        visitNode(cbNode, node.name);
                case 259 /* ExportAssignment */:
                    return visitNodes(cbNode, cbNodes, node.decorators) ||
                        visitNodes(cbNode, cbNodes, node.modifiers) ||
                        visitNode(cbNode, node.expression);
                case 211 /* TemplateExpression */:
                    return visitNode(cbNode, node.head) || visitNodes(cbNode, cbNodes, node.templateSpans);
                case 221 /* TemplateSpan */:
                    return visitNode(cbNode, node.expression) || visitNode(cbNode, node.literal);
                case 154 /* ComputedPropertyName */:
                    return visitNode(cbNode, node.expression);
                case 279 /* HeritageClause */:
                    return visitNodes(cbNode, cbNodes, node.types);
                case 216 /* ExpressionWithTypeArguments */:
                    return visitNode(cbNode, node.expression) ||
                        visitNodes(cbNode, cbNodes, node.typeArguments);
                case 265 /* ExternalModuleReference */:
                    return visitNode(cbNode, node.expression);
                case 264 /* MissingDeclaration */:
                    return visitNodes(cbNode, cbNodes, node.decorators);
                case 326 /* CommaListExpression */:
                    return visitNodes(cbNode, cbNodes, node.elements);
                case 266 /* JsxElement */:
                    return visitNode(cbNode, node.openingElement) ||
                        visitNodes(cbNode, cbNodes, node.children) ||
                        visitNode(cbNode, node.closingElement);
                case 270 /* JsxFragment */:
                    return visitNode(cbNode, node.openingFragment) ||
                        visitNodes(cbNode, cbNodes, node.children) ||
                        visitNode(cbNode, node.closingFragment);
                case 267 /* JsxSelfClosingElement */:
                case 268 /* JsxOpeningElement */:
                    return visitNode(cbNode, node.tagName) ||
                        visitNodes(cbNode, cbNodes, node.typeArguments) ||
                        visitNode(cbNode, node.attributes);
                case 274 /* JsxAttributes */:
                    return visitNodes(cbNode, cbNodes, node.properties);
                case 273 /* JsxAttribute */:
                    return visitNode(cbNode, node.name) ||
                        visitNode(cbNode, node.initializer);
                case 275 /* JsxSpreadAttribute */:
                    return visitNode(cbNode, node.expression);
                case 276 /* JsxExpression */:
                    return visitNode(cbNode, node.dotDotDotToken) ||
                        visitNode(cbNode, node.expression);
                case 269 /* JsxClosingElement */:
                    return visitNode(cbNode, node.tagName);
                case 176 /* OptionalType */:
                case 177 /* RestType */:
                case 294 /* JSDocTypeExpression */:
                case 298 /* JSDocNonNullableType */:
                case 297 /* JSDocNullableType */:
                case 299 /* JSDocOptionalType */:
                case 301 /* JSDocVariadicType */:
                    return visitNode(cbNode, node.type);
                case 300 /* JSDocFunctionType */:
                    return visitNodes(cbNode, cbNodes, node.parameters) ||
                        visitNode(cbNode, node.type);
                case 303 /* JSDocComment */:
                    return visitNodes(cbNode, cbNodes, node.tags);
                case 316 /* JSDocParameterTag */:
                case 322 /* JSDocPropertyTag */:
                    return visitNode(cbNode, node.tagName) ||
                        (node.isNameFirst
                            ? visitNode(cbNode, node.name) ||
                                visitNode(cbNode, node.typeExpression)
                            : visitNode(cbNode, node.typeExpression) ||
                                visitNode(cbNode, node.name));
                case 308 /* JSDocAuthorTag */:
                    return visitNode(cbNode, node.tagName);
                case 307 /* JSDocAugmentsTag */:
                    return visitNode(cbNode, node.tagName) ||
                        visitNode(cbNode, node.class);
                case 320 /* JSDocTemplateTag */:
                    return visitNode(cbNode, node.tagName) ||
                        visitNode(cbNode, node.constraint) ||
                        visitNodes(cbNode, cbNodes, node.typeParameters);
                case 321 /* JSDocTypedefTag */:
                    return visitNode(cbNode, node.tagName) ||
                        (node.typeExpression &&
                            node.typeExpression.kind === 294 /* JSDocTypeExpression */
                            ? visitNode(cbNode, node.typeExpression) ||
                                visitNode(cbNode, node.fullName)
                            : visitNode(cbNode, node.fullName) ||
                                visitNode(cbNode, node.typeExpression));
                case 314 /* JSDocCallbackTag */:
                    return visitNode(cbNode, node.tagName) ||
                        visitNode(cbNode, node.fullName) ||
                        visitNode(cbNode, node.typeExpression);
                case 317 /* JSDocReturnTag */:
                case 319 /* JSDocTypeTag */:
                case 318 /* JSDocThisTag */:
                case 315 /* JSDocEnumTag */:
                    return visitNode(cbNode, node.tagName) ||
                        visitNode(cbNode, node.typeExpression);
                case 305 /* JSDocSignature */:
                    return ts.forEach(node.typeParameters, cbNode) ||
                        ts.forEach(node.parameters, cbNode) ||
                        visitNode(cbNode, node.type);
                case 304 /* JSDocTypeLiteral */:
                    return ts.forEach(node.jsDocPropertyTags, cbNode);
                case 306 /* JSDocTag */:
                case 309 /* JSDocClassTag */:
                case 310 /* JSDocPublicTag */:
                case 311 /* JSDocPrivateTag */:
                case 312 /* JSDocProtectedTag */:
                case 313 /* JSDocReadonlyTag */:
                    return visitNode(cbNode, node.tagName);
                case 325 /* PartiallyEmittedExpression */:
                    return visitNode(cbNode, node.expression);
            }
        }
        ts.forEachChild = forEachChild;
        function createSourceFile(fileName, sourceText, languageVersion, setParentNodes, scriptKind) {
            if (setParentNodes === void 0) {
                setParentNodes = false;
            }
            ts.performance.mark("beforeParse");
            var result;
            ts.perfLogger.logStartParseSourceFile(fileName);
            if (languageVersion === 100 /* JSON */) {
                result = Parser.parseSourceFile(fileName, sourceText, languageVersion, /*syntaxCursor*/ undefined, setParentNodes, 6 /* JSON */);
            }
            else {
                result = Parser.parseSourceFile(fileName, sourceText, languageVersion, /*syntaxCursor*/ undefined, setParentNodes, scriptKind);
            }
            ts.perfLogger.logStopParseSourceFile();
            ts.performance.mark("afterParse");
            ts.performance.measure("Parse", "beforeParse", "afterParse");
            return result;
        }
        ts.createSourceFile = createSourceFile;
        function parseIsolatedEntityName(text, languageVersion) {
            return Parser.parseIsolatedEntityName(text, languageVersion);
        }
        ts.parseIsolatedEntityName = parseIsolatedEntityName;
        /**
         * Parse json text into SyntaxTree and return node and parse errors if any
         * @param fileName
         * @param sourceText
         */
        function parseJsonText(fileName, sourceText) {
            return Parser.parseJsonText(fileName, sourceText);
        }
        ts.parseJsonText = parseJsonText;
        // See also `isExternalOrCommonJsModule` in utilities.ts
        function isExternalModule(file) {
            return file.externalModuleIndicator !== undefined;
        }
        ts.isExternalModule = isExternalModule;
        // Produces a new SourceFile for the 'newText' provided. The 'textChangeRange' parameter
        // indicates what changed between the 'text' that this SourceFile has and the 'newText'.
        // The SourceFile will be created with the compiler attempting to reuse as many nodes from
        // this file as possible.
        //
        // Note: this function mutates nodes from this SourceFile. That means any existing nodes
        // from this SourceFile that are being held onto may change as a result (including
        // becoming detached from any SourceFile).  It is recommended that this SourceFile not
        // be used once 'update' is called on it.
        function updateSourceFile(sourceFile, newText, textChangeRange, aggressiveChecks) {
            if (aggressiveChecks === void 0) {
                aggressiveChecks = false;
            }
            var newSourceFile = IncrementalParser.updateSourceFile(sourceFile, newText, textChangeRange, aggressiveChecks);
            // Because new source file node is created, it may not have the flag PossiblyContainDynamicImport. This is the case if there is no new edit to add dynamic import.
            // We will manually port the flag to the new source file.
            newSourceFile.flags |= (sourceFile.flags & 3145728 /* PermanentlySetIncrementalFlags */);
            return newSourceFile;
        }
        ts.updateSourceFile = updateSourceFile;
        /* @internal */
        function parseIsolatedJSDocComment(content, start, length) {
            var result = Parser.JSDocParser.parseIsolatedJSDocComment(content, start, length);
            if (result && result.jsDoc) {
                // because the jsDocComment was parsed out of the source file, it might
                // not be covered by the fixupParentReferences.
                Parser.fixupParentReferences(result.jsDoc);
            }
            return result;
        }
        ts.parseIsolatedJSDocComment = parseIsolatedJSDocComment;
        /* @internal */
        // Exposed only for testing.
        function parseJSDocTypeExpressionForTests(content, start, length) {
            return Parser.JSDocParser.parseJSDocTypeExpressionForTests(content, start, length);
        }
        ts.parseJSDocTypeExpressionForTests = parseJSDocTypeExpressionForTests;
        // Implement the parser as a singleton module.  We do this for perf reasons because creating
        // parser instances can actually be expensive enough to impact us on projects with many source
        // files.
        var Parser;
        (function (Parser) {
            // Share a single scanner across all calls to parse a source file.  This helps speed things
            // up by avoiding the cost of creating/compiling scanners over and over again.
            var scanner = ts.createScanner(99 /* Latest */, /*skipTrivia*/ true);
            var disallowInAndDecoratorContext = 4096 /* DisallowInContext */ | 16384 /* DecoratorContext */;
            // capture constructors in 'initializeState' to avoid null checks
            var NodeConstructor;
            var TokenConstructor;
            var IdentifierConstructor;
            var PrivateIdentifierConstructor;
            var SourceFileConstructor;
            var sourceFile;
            var parseDiagnostics;
            var syntaxCursor;
            var currentToken;
            var sourceText;
            var nodeCount;
            var identifiers;
            var privateIdentifiers;
            var identifierCount;
            var parsingContext;
            var notParenthesizedArrow;
            // Flags that dictate what parsing context we're in.  For example:
            // Whether or not we are in strict parsing mode.  All that changes in strict parsing mode is
            // that some tokens that would be considered identifiers may be considered keywords.
            //
            // When adding more parser context flags, consider which is the more common case that the
            // flag will be in.  This should be the 'false' state for that flag.  The reason for this is
            // that we don't store data in our nodes unless the value is in the *non-default* state.  So,
            // for example, more often than code 'allows-in' (or doesn't 'disallow-in').  We opt for
            // 'disallow-in' set to 'false'.  Otherwise, if we had 'allowsIn' set to 'true', then almost
            // all nodes would need extra state on them to store this info.
            //
            // Note: 'allowIn' and 'allowYield' track 1:1 with the [in] and [yield] concepts in the ES6
            // grammar specification.
            //
            // An important thing about these context concepts.  By default they are effectively inherited
            // while parsing through every grammar production.  i.e. if you don't change them, then when
            // you parse a sub-production, it will have the same context values as the parent production.
            // This is great most of the time.  After all, consider all the 'expression' grammar productions
            // and how nearly all of them pass along the 'in' and 'yield' context values:
            //
            // EqualityExpression[In, Yield] :
            //      RelationalExpression[?In, ?Yield]
            //      EqualityExpression[?In, ?Yield] == RelationalExpression[?In, ?Yield]
            //      EqualityExpression[?In, ?Yield] != RelationalExpression[?In, ?Yield]
            //      EqualityExpression[?In, ?Yield] === RelationalExpression[?In, ?Yield]
            //      EqualityExpression[?In, ?Yield] !== RelationalExpression[?In, ?Yield]
            //
            // Where you have to be careful is then understanding what the points are in the grammar
            // where the values are *not* passed along.  For example:
            //
            // SingleNameBinding[Yield,GeneratorParameter]
            //      [+GeneratorParameter]BindingIdentifier[Yield] Initializer[In]opt
            //      [~GeneratorParameter]BindingIdentifier[?Yield]Initializer[In, ?Yield]opt
            //
            // Here this is saying that if the GeneratorParameter context flag is set, that we should
            // explicitly set the 'yield' context flag to false before calling into the BindingIdentifier
            // and we should explicitly unset the 'yield' context flag before calling into the Initializer.
            // production.  Conversely, if the GeneratorParameter context flag is not set, then we
            // should leave the 'yield' context flag alone.
            //
            // Getting this all correct is tricky and requires careful reading of the grammar to
            // understand when these values should be changed versus when they should be inherited.
            //
            // Note: it should not be necessary to save/restore these flags during speculative/lookahead
            // parsing.  These context flags are naturally stored and restored through normal recursive
            // descent parsing and unwinding.
            var contextFlags;
            // Whether or not we've had a parse error since creating the last AST node.  If we have
            // encountered an error, it will be stored on the next AST node we create.  Parse errors
            // can be broken down into three categories:
            //
            // 1) An error that occurred during scanning.  For example, an unterminated literal, or a
            //    character that was completely not understood.
            //
            // 2) A token was expected, but was not present.  This type of error is commonly produced
            //    by the 'parseExpected' function.
            //
            // 3) A token was present that no parsing function was able to consume.  This type of error
            //    only occurs in the 'abortParsingListOrMoveToNextToken' function when the parser
            //    decides to skip the token.
            //
            // In all of these cases, we want to mark the next node as having had an error before it.
            // With this mark, we can know in incremental settings if this node can be reused, or if
            // we have to reparse it.  If we don't keep this information around, we may just reuse the
            // node.  in that event we would then not produce the same errors as we did before, causing
            // significant confusion problems.
            //
            // Note: it is necessary that this value be saved/restored during speculative/lookahead
            // parsing.  During lookahead parsing, we will often create a node.  That node will have
            // this value attached, and then this value will be set back to 'false'.  If we decide to
            // rewind, we must get back to the same value we had prior to the lookahead.
            //
            // Note: any errors at the end of the file that do not precede a regular node, should get
            // attached to the EOF token.
            var parseErrorBeforeNextFinishedNode = false;
            function parseSourceFile(fileName, sourceText, languageVersion, syntaxCursor, setParentNodes, scriptKind) {
                if (setParentNodes === void 0) {
                    setParentNodes = false;
                }
                scriptKind = ts.ensureScriptKind(fileName, scriptKind);
                if (scriptKind === 6 /* JSON */) {
                    var result_2 = parseJsonText(fileName, sourceText, languageVersion, syntaxCursor, setParentNodes);
                    ts.convertToObjectWorker(result_2, result_2.parseDiagnostics, /*returnValue*/ false, /*knownRootOptions*/ undefined, /*jsonConversionNotifier*/ undefined);
                    result_2.referencedFiles = ts.emptyArray;
                    result_2.typeReferenceDirectives = ts.emptyArray;
                    result_2.libReferenceDirectives = ts.emptyArray;
                    result_2.amdDependencies = ts.emptyArray;
                    result_2.hasNoDefaultLib = false;
                    result_2.pragmas = ts.emptyMap;
                    return result_2;
                }
                initializeState(sourceText, languageVersion, syntaxCursor, scriptKind);
                var result = parseSourceFileWorker(fileName, languageVersion, setParentNodes, scriptKind);
                clearState();
                return result;
            }
            Parser.parseSourceFile = parseSourceFile;
            function parseIsolatedEntityName(content, languageVersion) {
                // Choice of `isDeclarationFile` should be arbitrary
                initializeState(content, languageVersion, /*syntaxCursor*/ undefined, 1 /* JS */);
                // Prime the scanner.
                nextToken();
                var entityName = parseEntityName(/*allowReservedWords*/ true);
                var isInvalid = token() === 1 /* EndOfFileToken */ && !parseDiagnostics.length;
                clearState();
                return isInvalid ? entityName : undefined;
            }
            Parser.parseIsolatedEntityName = parseIsolatedEntityName;
            function parseJsonText(fileName, sourceText, languageVersion, syntaxCursor, setParentNodes) {
                if (languageVersion === void 0) {
                    languageVersion = 2 /* ES2015 */;
                }
                initializeState(sourceText, languageVersion, syntaxCursor, 6 /* JSON */);
                // Set source file so that errors will be reported with this file name
                sourceFile = createSourceFile(fileName, 2 /* ES2015 */, 6 /* JSON */, /*isDeclaration*/ false);
                sourceFile.flags = contextFlags;
                // Prime the scanner.
                nextToken();
                var pos = getNodePos();
                if (token() === 1 /* EndOfFileToken */) {
                    sourceFile.statements = createNodeArray([], pos, pos);
                    sourceFile.endOfFileToken = parseTokenNode();
                }
                else {
                    var statement = createNode(226 /* ExpressionStatement */);
                    switch (token()) {
                        case 22 /* OpenBracketToken */:
                            statement.expression = parseArrayLiteralExpression();
                            break;
                        case 106 /* TrueKeyword */:
                        case 91 /* FalseKeyword */:
                        case 100 /* NullKeyword */:
                            statement.expression = parseTokenNode();
                            break;
                        case 40 /* MinusToken */:
                            if (lookAhead(function () { return nextToken() === 8 /* NumericLiteral */ && nextToken() !== 58 /* ColonToken */; })) {
                                statement.expression = parsePrefixUnaryExpression();
                            }
                            else {
                                statement.expression = parseObjectLiteralExpression();
                            }
                            break;
                        case 8 /* NumericLiteral */:
                        case 10 /* StringLiteral */:
                            if (lookAhead(function () { return nextToken() !== 58 /* ColonToken */; })) {
                                statement.expression = parseLiteralNode();
                                break;
                            }
                        // falls through
                        default:
                            statement.expression = parseObjectLiteralExpression();
                            break;
                    }
                    finishNode(statement);
                    sourceFile.statements = createNodeArray([statement], pos);
                    sourceFile.endOfFileToken = parseExpectedToken(1 /* EndOfFileToken */, ts.Diagnostics.Unexpected_token);
                }
                if (setParentNodes) {
                    fixupParentReferences(sourceFile);
                }
                sourceFile.nodeCount = nodeCount;
                sourceFile.identifierCount = identifierCount;
                sourceFile.identifiers = identifiers;
                sourceFile.parseDiagnostics = parseDiagnostics;
                var result = sourceFile;
                clearState();
                return result;
            }
            Parser.parseJsonText = parseJsonText;
            function getLanguageVariant(scriptKind) {
                // .tsx and .jsx files are treated as jsx language variant.
                return scriptKind === 4 /* TSX */ || scriptKind === 2 /* JSX */ || scriptKind === 1 /* JS */ || scriptKind === 6 /* JSON */ ? 1 /* JSX */ : 0 /* Standard */;
            }
            function initializeState(_sourceText, languageVersion, _syntaxCursor, scriptKind) {
                NodeConstructor = ts.objectAllocator.getNodeConstructor();
                TokenConstructor = ts.objectAllocator.getTokenConstructor();
                IdentifierConstructor = ts.objectAllocator.getIdentifierConstructor();
                PrivateIdentifierConstructor = ts.objectAllocator.getPrivateIdentifierConstructor();
                SourceFileConstructor = ts.objectAllocator.getSourceFileConstructor();
                sourceText = _sourceText;
                syntaxCursor = _syntaxCursor;
                parseDiagnostics = [];
                parsingContext = 0;
                identifiers = ts.createMap();
                privateIdentifiers = ts.createMap();
                identifierCount = 0;
                nodeCount = 0;
                switch (scriptKind) {
                    case 1 /* JS */:
                    case 2 /* JSX */:
                        contextFlags = 131072 /* JavaScriptFile */;
                        break;
                    case 6 /* JSON */:
                        contextFlags = 131072 /* JavaScriptFile */ | 33554432 /* JsonFile */;
                        break;
                    default:
                        contextFlags = 0 /* None */;
                        break;
                }
                parseErrorBeforeNextFinishedNode = false;
                // Initialize and prime the scanner before parsing the source elements.
                scanner.setText(sourceText);
                scanner.setOnError(scanError);
                scanner.setScriptTarget(languageVersion);
                scanner.setLanguageVariant(getLanguageVariant(scriptKind));
            }
            function clearState() {
                // Clear out the text the scanner is pointing at, so it doesn't keep anything alive unnecessarily.
                scanner.setText("");
                scanner.setOnError(undefined);
                // Clear any data.  We don't want to accidentally hold onto it for too long.
                parseDiagnostics = undefined;
                sourceFile = undefined;
                identifiers = undefined;
                syntaxCursor = undefined;
                sourceText = undefined;
                notParenthesizedArrow = undefined;
            }
            function parseSourceFileWorker(fileName, languageVersion, setParentNodes, scriptKind) {
                var isDeclarationFile = isDeclarationFileName(fileName);
                if (isDeclarationFile) {
                    contextFlags |= 8388608 /* Ambient */;
                }
                sourceFile = createSourceFile(fileName, languageVersion, scriptKind, isDeclarationFile);
                sourceFile.flags = contextFlags;
                // Prime the scanner.
                nextToken();
                // A member of ReadonlyArray<T> isn't assignable to a member of T[] (and prevents a direct cast) - but this is where we set up those members so they can be readonly in the future
                processCommentPragmas(sourceFile, sourceText);
                processPragmasIntoFields(sourceFile, reportPragmaDiagnostic);
                sourceFile.statements = parseList(0 /* SourceElements */, parseStatement);
                ts.Debug.assert(token() === 1 /* EndOfFileToken */);
                sourceFile.endOfFileToken = addJSDocComment(parseTokenNode());
                setExternalModuleIndicator(sourceFile);
                sourceFile.nodeCount = nodeCount;
                sourceFile.identifierCount = identifierCount;
                sourceFile.identifiers = identifiers;
                sourceFile.parseDiagnostics = parseDiagnostics;
                if (setParentNodes) {
                    fixupParentReferences(sourceFile);
                }
                return sourceFile;
                function reportPragmaDiagnostic(pos, end, diagnostic) {
                    parseDiagnostics.push(ts.createFileDiagnostic(sourceFile, pos, end, diagnostic));
                }
            }
            function addJSDocComment(node) {
                ts.Debug.assert(!node.jsDoc); // Should only be called once per node
                var jsDoc = ts.mapDefined(ts.getJSDocCommentRanges(node, sourceFile.text), function (comment) { return JSDocParser.parseJSDocComment(node, comment.pos, comment.end - comment.pos); });
                if (jsDoc.length)
                    node.jsDoc = jsDoc;
                return node;
            }
            function fixupParentReferences(rootNode) {
                // normally parent references are set during binding. However, for clients that only need
                // a syntax tree, and no semantic features, then the binding process is an unnecessary
                // overhead.  This functions allows us to set all the parents, without all the expense of
                // binding.
                var stack = [rootNode];
                while (stack.length) {
                    var parent = stack.pop();
                    bindParentToChildren(parent, gatherChildren(parent));
                }
                return;
                function gatherChildren(node) {
                    var children = [];
                    forEachChild(node, function (n) { children.unshift(n); }); // By using a stack above and `unshift` here, we emulate a depth-first preorder traversal
                    return children;
                }
                function bindParentToChildren(parent, children) {
                    for (var _i = 0, children_1 = children; _i < children_1.length; _i++) {
                        var child = children_1[_i];
                        if (child.parent === parent)
                            continue; // already bound, assume subtree is bound
                        child.parent = parent;
                        stack.push(child);
                        if (ts.hasJSDocNodes(child)) {
                            for (var _a = 0, _b = child.jsDoc; _a < _b.length; _a++) {
                                var jsDoc = _b[_a];
                                jsDoc.parent = child;
                                stack.push(jsDoc);
                            }
                        }
                    }
                }
            }
            Parser.fixupParentReferences = fixupParentReferences;
            function createSourceFile(fileName, languageVersion, scriptKind, isDeclarationFile) {
                // code from createNode is inlined here so createNode won't have to deal with special case of creating source files
                // this is quite rare comparing to other nodes and createNode should be as fast as possible
                var sourceFile = new SourceFileConstructor(290 /* SourceFile */, /*pos*/ 0, /* end */ sourceText.length);
                nodeCount++;
                sourceFile.text = sourceText;
                sourceFile.bindDiagnostics = [];
                sourceFile.bindSuggestionDiagnostics = undefined;
                sourceFile.languageVersion = languageVersion;
                sourceFile.fileName = ts.normalizePath(fileName);
                sourceFile.languageVariant = getLanguageVariant(scriptKind);
                sourceFile.isDeclarationFile = isDeclarationFile;
                sourceFile.scriptKind = scriptKind;
                return sourceFile;
            }
            function setContextFlag(val, flag) {
                if (val) {
                    contextFlags |= flag;
                }
                else {
                    contextFlags &= ~flag;
                }
            }
            function setDisallowInContext(val) {
                setContextFlag(val, 4096 /* DisallowInContext */);
            }
            function setYieldContext(val) {
                setContextFlag(val, 8192 /* YieldContext */);
            }
            function setDecoratorContext(val) {
                setContextFlag(val, 16384 /* DecoratorContext */);
            }
            function setAwaitContext(val) {
                setContextFlag(val, 32768 /* AwaitContext */);
            }
            function doOutsideOfContext(context, func) {
                // contextFlagsToClear will contain only the context flags that are
                // currently set that we need to temporarily clear
                // We don't just blindly reset to the previous flags to ensure
                // that we do not mutate cached flags for the incremental
                // parser (ThisNodeHasError, ThisNodeOrAnySubNodesHasError, and
                // HasAggregatedChildData).
                var contextFlagsToClear = context & contextFlags;
                if (contextFlagsToClear) {
                    // clear the requested context flags
                    setContextFlag(/*val*/ false, contextFlagsToClear);
                    var result = func();
                    // restore the context flags we just cleared
                    setContextFlag(/*val*/ true, contextFlagsToClear);
                    return result;
                }
                // no need to do anything special as we are not in any of the requested contexts
                return func();
            }
            function doInsideOfContext(context, func) {
                // contextFlagsToSet will contain only the context flags that
                // are not currently set that we need to temporarily enable.
                // We don't just blindly reset to the previous flags to ensure
                // that we do not mutate cached flags for the incremental
                // parser (ThisNodeHasError, ThisNodeOrAnySubNodesHasError, and
                // HasAggregatedChildData).
                var contextFlagsToSet = context & ~contextFlags;
                if (contextFlagsToSet) {
                    // set the requested context flags
                    setContextFlag(/*val*/ true, contextFlagsToSet);
                    var result = func();
                    // reset the context flags we just set
                    setContextFlag(/*val*/ false, contextFlagsToSet);
                    return result;
                }
                // no need to do anything special as we are already in all of the requested contexts
                return func();
            }
            function allowInAnd(func) {
                return doOutsideOfContext(4096 /* DisallowInContext */, func);
            }
            function disallowInAnd(func) {
                return doInsideOfContext(4096 /* DisallowInContext */, func);
            }
            function doInYieldContext(func) {
                return doInsideOfContext(8192 /* YieldContext */, func);
            }
            function doInDecoratorContext(func) {
                return doInsideOfContext(16384 /* DecoratorContext */, func);
            }
            function doInAwaitContext(func) {
                return doInsideOfContext(32768 /* AwaitContext */, func);
            }
            function doOutsideOfAwaitContext(func) {
                return doOutsideOfContext(32768 /* AwaitContext */, func);
            }
            function doInYieldAndAwaitContext(func) {
                return doInsideOfContext(8192 /* YieldContext */ | 32768 /* AwaitContext */, func);
            }
            function doOutsideOfYieldAndAwaitContext(func) {
                return doOutsideOfContext(8192 /* YieldContext */ | 32768 /* AwaitContext */, func);
            }
            function inContext(flags) {
                return (contextFlags & flags) !== 0;
            }
            function inYieldContext() {
                return inContext(8192 /* YieldContext */);
            }
            function inDisallowInContext() {
                return inContext(4096 /* DisallowInContext */);
            }
            function inDecoratorContext() {
                return inContext(16384 /* DecoratorContext */);
            }
            function inAwaitContext() {
                return inContext(32768 /* AwaitContext */);
            }
            function parseErrorAtCurrentToken(message, arg0) {
                parseErrorAt(scanner.getTokenPos(), scanner.getTextPos(), message, arg0);
            }
            function parseErrorAtPosition(start, length, message, arg0) {
                // Don't report another error if it would just be at the same position as the last error.
                var lastError = ts.lastOrUndefined(parseDiagnostics);
                if (!lastError || start !== lastError.start) {
                    parseDiagnostics.push(ts.createFileDiagnostic(sourceFile, start, length, message, arg0));
                }
                // Mark that we've encountered an error.  We'll set an appropriate bit on the next
                // node we finish so that it can't be reused incrementally.
                parseErrorBeforeNextFinishedNode = true;
            }
            function parseErrorAt(start, end, message, arg0) {
                parseErrorAtPosition(start, end - start, message, arg0);
            }
            function parseErrorAtRange(range, message, arg0) {
                parseErrorAt(range.pos, range.end, message, arg0);
            }
            function scanError(message, length) {
                parseErrorAtPosition(scanner.getTextPos(), length, message);
            }
            function getNodePos() {
                return scanner.getStartPos();
            }
            // Use this function to access the current token instead of reading the currentToken
            // variable. Since function results aren't narrowed in control flow analysis, this ensures
            // that the type checker doesn't make wrong assumptions about the type of the current
            // token (e.g. a call to nextToken() changes the current token but the checker doesn't
            // reason about this side effect).  Mainstream VMs inline simple functions like this, so
            // there is no performance penalty.
            function token() {
                return currentToken;
            }
            function nextTokenWithoutCheck() {
                return currentToken = scanner.scan();
            }
            function nextToken() {
                // if the keyword had an escape
                if (ts.isKeyword(currentToken) && (scanner.hasUnicodeEscape() || scanner.hasExtendedUnicodeEscape())) {
                    // issue a parse error for the escape
                    parseErrorAt(scanner.getTokenPos(), scanner.getTextPos(), ts.Diagnostics.Keywords_cannot_contain_escape_characters);
                }
                return nextTokenWithoutCheck();
            }
            function nextTokenJSDoc() {
                return currentToken = scanner.scanJsDocToken();
            }
            function reScanGreaterToken() {
                return currentToken = scanner.reScanGreaterToken();
            }
            function reScanSlashToken() {
                return currentToken = scanner.reScanSlashToken();
            }
            function reScanTemplateToken() {
                return currentToken = scanner.reScanTemplateToken();
            }
            function reScanLessThanToken() {
                return currentToken = scanner.reScanLessThanToken();
            }
            function scanJsxIdentifier() {
                return currentToken = scanner.scanJsxIdentifier();
            }
            function scanJsxText() {
                return currentToken = scanner.scanJsxToken();
            }
            function scanJsxAttributeValue() {
                return currentToken = scanner.scanJsxAttributeValue();
            }
            function speculationHelper(callback, isLookAhead) {
                // Keep track of the state we'll need to rollback to if lookahead fails (or if the
                // caller asked us to always reset our state).
                var saveToken = currentToken;
                var saveParseDiagnosticsLength = parseDiagnostics.length;
                var saveParseErrorBeforeNextFinishedNode = parseErrorBeforeNextFinishedNode;
                // Note: it is not actually necessary to save/restore the context flags here.  That's
                // because the saving/restoring of these flags happens naturally through the recursive
                // descent nature of our parser.  However, we still store this here just so we can
                // assert that invariant holds.
                var saveContextFlags = contextFlags;
                // If we're only looking ahead, then tell the scanner to only lookahead as well.
                // Otherwise, if we're actually speculatively parsing, then tell the scanner to do the
                // same.
                var result = isLookAhead
                    ? scanner.lookAhead(callback)
                    : scanner.tryScan(callback);
                ts.Debug.assert(saveContextFlags === contextFlags);
                // If our callback returned something 'falsy' or we're just looking ahead,
                // then unconditionally restore us to where we were.
                if (!result || isLookAhead) {
                    currentToken = saveToken;
                    parseDiagnostics.length = saveParseDiagnosticsLength;
                    parseErrorBeforeNextFinishedNode = saveParseErrorBeforeNextFinishedNode;
                }
                return result;
            }
            /** Invokes the provided callback then unconditionally restores the parser to the state it
             * was in immediately prior to invoking the callback.  The result of invoking the callback
             * is returned from this function.
             */
            function lookAhead(callback) {
                return speculationHelper(callback, /*isLookAhead*/ true);
            }
            /** Invokes the provided callback.  If the callback returns something falsy, then it restores
             * the parser to the state it was in immediately prior to invoking the callback.  If the
             * callback returns something truthy, then the parser state is not rolled back.  The result
             * of invoking the callback is returned from this function.
             */
            function tryParse(callback) {
                return speculationHelper(callback, /*isLookAhead*/ false);
            }
            // Ignore strict mode flag because we will report an error in type checker instead.
            function isIdentifier() {
                if (token() === 75 /* Identifier */) {
                    return true;
                }
                // If we have a 'yield' keyword, and we're in the [yield] context, then 'yield' is
                // considered a keyword and is not an identifier.
                if (token() === 121 /* YieldKeyword */ && inYieldContext()) {
                    return false;
                }
                // If we have a 'await' keyword, and we're in the [Await] context, then 'await' is
                // considered a keyword and is not an identifier.
                if (token() === 127 /* AwaitKeyword */ && inAwaitContext()) {
                    return false;
                }
                return token() > 112 /* LastReservedWord */;
            }
            function parseExpected(kind, diagnosticMessage, shouldAdvance) {
                if (shouldAdvance === void 0) {
                    shouldAdvance = true;
                }
                if (token() === kind) {
                    if (shouldAdvance) {
                        nextToken();
                    }
                    return true;
                }
                // Report specific message if provided with one.  Otherwise, report generic fallback message.
                if (diagnosticMessage) {
                    parseErrorAtCurrentToken(diagnosticMessage);
                }
                else {
                    parseErrorAtCurrentToken(ts.Diagnostics._0_expected, ts.tokenToString(kind));
                }
                return false;
            }
            function parseExpectedJSDoc(kind) {
                if (token() === kind) {
                    nextTokenJSDoc();
                    return true;
                }
                parseErrorAtCurrentToken(ts.Diagnostics._0_expected, ts.tokenToString(kind));
                return false;
            }
            function parseOptional(t) {
                if (token() === t) {
                    nextToken();
                    return true;
                }
                return false;
            }
            function parseOptionalToken(t) {
                if (token() === t) {
                    return parseTokenNode();
                }
                return undefined;
            }
            function parseOptionalTokenJSDoc(t) {
                if (token() === t) {
                    return parseTokenNodeJSDoc();
                }
                return undefined;
            }
            function parseExpectedToken(t, diagnosticMessage, arg0) {
                return parseOptionalToken(t) ||
                    createMissingNode(t, /*reportAtCurrentPosition*/ false, diagnosticMessage || ts.Diagnostics._0_expected, arg0 || ts.tokenToString(t));
            }
            function parseExpectedTokenJSDoc(t) {
                return parseOptionalTokenJSDoc(t) ||
                    createMissingNode(t, /*reportAtCurrentPosition*/ false, ts.Diagnostics._0_expected, ts.tokenToString(t));
            }
            function parseTokenNode() {
                var node = createNode(token());
                nextToken();
                return finishNode(node);
            }
            function parseTokenNodeJSDoc() {
                var node = createNode(token());
                nextTokenJSDoc();
                return finishNode(node);
            }
            function canParseSemicolon() {
                // If there's a real semicolon, then we can always parse it out.
                if (token() === 26 /* SemicolonToken */) {
                    return true;
                }
                // We can parse out an optional semicolon in ASI cases in the following cases.
                return token() === 19 /* CloseBraceToken */ || token() === 1 /* EndOfFileToken */ || scanner.hasPrecedingLineBreak();
            }
            function parseSemicolon() {
                if (canParseSemicolon()) {
                    if (token() === 26 /* SemicolonToken */) {
                        // consume the semicolon if it was explicitly provided.
                        nextToken();
                    }
                    return true;
                }
                else {
                    return parseExpected(26 /* SemicolonToken */);
                }
            }
            function createNode(kind, pos) {
                nodeCount++;
                var p = pos >= 0 ? pos : scanner.getStartPos();
                return ts.isNodeKind(kind) || kind === 0 /* Unknown */ ? new NodeConstructor(kind, p, p) :
                    kind === 75 /* Identifier */ ? new IdentifierConstructor(kind, p, p) :
                        kind === 76 /* PrivateIdentifier */ ? new PrivateIdentifierConstructor(kind, p, p) :
                            new TokenConstructor(kind, p, p);
            }
            function createNodeWithJSDoc(kind, pos) {
                var node = createNode(kind, pos);
                if (scanner.getTokenFlags() & 2 /* PrecedingJSDocComment */) {
                    addJSDocComment(node);
                }
                return node;
            }
            function createNodeArray(elements, pos, end) {
                // Since the element list of a node array is typically created by starting with an empty array and
                // repeatedly calling push(), the list may not have the optimal memory layout. We invoke slice() for
                // small arrays (1 to 4 elements) to give the VM a chance to allocate an optimal representation.
                var length = elements.length;
                var array = (length >= 1 && length <= 4 ? elements.slice() : elements);
                array.pos = pos;
                array.end = end === undefined ? scanner.getStartPos() : end;
                return array;
            }
            function finishNode(node, end) {
                node.end = end === undefined ? scanner.getStartPos() : end;
                if (contextFlags) {
                    node.flags |= contextFlags;
                }
                // Keep track on the node if we encountered an error while parsing it.  If we did, then
                // we cannot reuse the node incrementally.  Once we've marked this node, clear out the
                // flag so that we don't mark any subsequent nodes.
                if (parseErrorBeforeNextFinishedNode) {
                    parseErrorBeforeNextFinishedNode = false;
                    node.flags |= 65536 /* ThisNodeHasError */;
                }
                return node;
            }
            function createMissingNode(kind, reportAtCurrentPosition, diagnosticMessage, arg0) {
                if (reportAtCurrentPosition) {
                    parseErrorAtPosition(scanner.getStartPos(), 0, diagnosticMessage, arg0);
                }
                else if (diagnosticMessage) {
                    parseErrorAtCurrentToken(diagnosticMessage, arg0);
                }
                var result = createNode(kind);
                if (kind === 75 /* Identifier */) {
                    result.escapedText = "";
                }
                else if (ts.isLiteralKind(kind) || ts.isTemplateLiteralKind(kind)) {
                    result.text = "";
                }
                return finishNode(result);
            }
            function internIdentifier(text) {
                var identifier = identifiers.get(text);
                if (identifier === undefined) {
                    identifiers.set(text, identifier = text);
                }
                return identifier;
            }
            // An identifier that starts with two underscores has an extra underscore character prepended to it to avoid issues
            // with magic property names like '__proto__'. The 'identifiers' object is used to share a single string instance for
            // each identifier in order to reduce memory consumption.
            function createIdentifier(isIdentifier, diagnosticMessage, privateIdentifierDiagnosticMessage) {
                identifierCount++;
                if (isIdentifier) {
                    var node = createNode(75 /* Identifier */);
                    // Store original token kind if it is not just an Identifier so we can report appropriate error later in type checker
                    if (token() !== 75 /* Identifier */) {
                        node.originalKeywordKind = token();
                    }
                    node.escapedText = ts.escapeLeadingUnderscores(internIdentifier(scanner.getTokenValue()));
                    nextTokenWithoutCheck();
                    return finishNode(node);
                }
                if (token() === 76 /* PrivateIdentifier */) {
                    parseErrorAtCurrentToken(privateIdentifierDiagnosticMessage || ts.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies);
                    return createIdentifier(/*isIdentifier*/ true);
                }
                // Only for end of file because the error gets reported incorrectly on embedded script tags.
                var reportAtCurrentPosition = token() === 1 /* EndOfFileToken */;
                var isReservedWord = scanner.isReservedWord();
                var msgArg = scanner.getTokenText();
                var defaultMessage = isReservedWord ?
                    ts.Diagnostics.Identifier_expected_0_is_a_reserved_word_that_cannot_be_used_here :
                    ts.Diagnostics.Identifier_expected;
                return createMissingNode(75 /* Identifier */, reportAtCurrentPosition, diagnosticMessage || defaultMessage, msgArg);
            }
            function parseIdentifier(diagnosticMessage, privateIdentifierDiagnosticMessage) {
                return createIdentifier(isIdentifier(), diagnosticMessage, privateIdentifierDiagnosticMessage);
            }
            function parseIdentifierName(diagnosticMessage) {
                return createIdentifier(ts.tokenIsIdentifierOrKeyword(token()), diagnosticMessage);
            }
            function isLiteralPropertyName() {
                return ts.tokenIsIdentifierOrKeyword(token()) ||
                    token() === 10 /* StringLiteral */ ||
                    token() === 8 /* NumericLiteral */;
            }
            function parsePropertyNameWorker(allowComputedPropertyNames) {
                if (token() === 10 /* StringLiteral */ || token() === 8 /* NumericLiteral */) {
                    var node = parseLiteralNode();
                    node.text = internIdentifier(node.text);
                    return node;
                }
                if (allowComputedPropertyNames && token() === 22 /* OpenBracketToken */) {
                    return parseComputedPropertyName();
                }
                if (token() === 76 /* PrivateIdentifier */) {
                    return parsePrivateIdentifier();
                }
                return parseIdentifierName();
            }
            function parsePropertyName() {
                return parsePropertyNameWorker(/*allowComputedPropertyNames*/ true);
            }
            function parseComputedPropertyName() {
                // PropertyName [Yield]:
                //      LiteralPropertyName
                //      ComputedPropertyName[?Yield]
                var node = createNode(154 /* ComputedPropertyName */);
                parseExpected(22 /* OpenBracketToken */);
                // We parse any expression (including a comma expression). But the grammar
                // says that only an assignment expression is allowed, so the grammar checker
                // will error if it sees a comma expression.
                node.expression = allowInAnd(parseExpression);
                parseExpected(23 /* CloseBracketToken */);
                return finishNode(node);
            }
            function internPrivateIdentifier(text) {
                var privateIdentifier = privateIdentifiers.get(text);
                if (privateIdentifier === undefined) {
                    privateIdentifiers.set(text, privateIdentifier = text);
                }
                return privateIdentifier;
            }
            function parsePrivateIdentifier() {
                var node = createNode(76 /* PrivateIdentifier */);
                node.escapedText = ts.escapeLeadingUnderscores(internPrivateIdentifier(scanner.getTokenText()));
                nextToken();
                return finishNode(node);
            }
            function parseContextualModifier(t) {
                return token() === t && tryParse(nextTokenCanFollowModifier);
            }
            function nextTokenIsOnSameLineAndCanFollowModifier() {
                nextToken();
                if (scanner.hasPrecedingLineBreak()) {
                    return false;
                }
                return canFollowModifier();
            }
            function nextTokenCanFollowModifier() {
                switch (token()) {
                    case 81 /* ConstKeyword */:
                        // 'const' is only a modifier if followed by 'enum'.
                        return nextToken() === 88 /* EnumKeyword */;
                    case 89 /* ExportKeyword */:
                        nextToken();
                        if (token() === 84 /* DefaultKeyword */) {
                            return lookAhead(nextTokenCanFollowDefaultKeyword);
                        }
                        if (token() === 145 /* TypeKeyword */) {
                            return lookAhead(nextTokenCanFollowExportModifier);
                        }
                        return canFollowExportModifier();
                    case 84 /* DefaultKeyword */:
                        return nextTokenCanFollowDefaultKeyword();
                    case 120 /* StaticKeyword */:
                    case 131 /* GetKeyword */:
                    case 142 /* SetKeyword */:
                        nextToken();
                        return canFollowModifier();
                    default:
                        return nextTokenIsOnSameLineAndCanFollowModifier();
                }
            }
            function canFollowExportModifier() {
                return token() !== 41 /* AsteriskToken */
                    && token() !== 123 /* AsKeyword */
                    && token() !== 18 /* OpenBraceToken */
                    && canFollowModifier();
            }
            function nextTokenCanFollowExportModifier() {
                nextToken();
                return canFollowExportModifier();
            }
            function parseAnyContextualModifier() {
                return ts.isModifierKind(token()) && tryParse(nextTokenCanFollowModifier);
            }
            function canFollowModifier() {
                return token() === 22 /* OpenBracketToken */
                    || token() === 18 /* OpenBraceToken */
                    || token() === 41 /* AsteriskToken */
                    || token() === 25 /* DotDotDotToken */
                    || isLiteralPropertyName();
            }
            function nextTokenCanFollowDefaultKeyword() {
                nextToken();
                return token() === 80 /* ClassKeyword */ || token() === 94 /* FunctionKeyword */ ||
                    token() === 114 /* InterfaceKeyword */ ||
                    (token() === 122 /* AbstractKeyword */ && lookAhead(nextTokenIsClassKeywordOnSameLine)) ||
                    (token() === 126 /* AsyncKeyword */ && lookAhead(nextTokenIsFunctionKeywordOnSameLine));
            }
            // True if positioned at the start of a list element
            function isListElement(parsingContext, inErrorRecovery) {
                var node = currentNode(parsingContext);
                if (node) {
                    return true;
                }
                switch (parsingContext) {
                    case 0 /* SourceElements */:
                    case 1 /* BlockStatements */:
                    case 3 /* SwitchClauseStatements */:
                        // If we're in error recovery, then we don't want to treat ';' as an empty statement.
                        // The problem is that ';' can show up in far too many contexts, and if we see one
                        // and assume it's a statement, then we may bail out inappropriately from whatever
                        // we're parsing.  For example, if we have a semicolon in the middle of a class, then
                        // we really don't want to assume the class is over and we're on a statement in the
                        // outer module.  We just want to consume and move on.
                        return !(token() === 26 /* SemicolonToken */ && inErrorRecovery) && isStartOfStatement();
                    case 2 /* SwitchClauses */:
                        return token() === 78 /* CaseKeyword */ || token() === 84 /* DefaultKeyword */;
                    case 4 /* TypeMembers */:
                        return lookAhead(isTypeMemberStart);
                    case 5 /* ClassMembers */:
                        // We allow semicolons as class elements (as specified by ES6) as long as we're
                        // not in error recovery.  If we're in error recovery, we don't want an errant
                        // semicolon to be treated as a class member (since they're almost always used
                        // for statements.
                        return lookAhead(isClassMemberStart) || (token() === 26 /* SemicolonToken */ && !inErrorRecovery);
                    case 6 /* EnumMembers */:
                        // Include open bracket computed properties. This technically also lets in indexers,
                        // which would be a candidate for improved error reporting.
                        return token() === 22 /* OpenBracketToken */ || isLiteralPropertyName();
                    case 12 /* ObjectLiteralMembers */:
                        switch (token()) {
                            case 22 /* OpenBracketToken */:
                            case 41 /* AsteriskToken */:
                            case 25 /* DotDotDotToken */:
                            case 24 /* DotToken */: // Not an object literal member, but don't want to close the object (see `tests/cases/fourslash/completionsDotInObjectLiteral.ts`)
                                return true;
                            default:
                                return isLiteralPropertyName();
                        }
                    case 18 /* RestProperties */:
                        return isLiteralPropertyName();
                    case 9 /* ObjectBindingElements */:
                        return token() === 22 /* OpenBracketToken */ || token() === 25 /* DotDotDotToken */ || isLiteralPropertyName();
                    case 7 /* HeritageClauseElement */:
                        // If we see `{ ... }` then only consume it as an expression if it is followed by `,` or `{`
                        // That way we won't consume the body of a class in its heritage clause.
                        if (token() === 18 /* OpenBraceToken */) {
                            return lookAhead(isValidHeritageClauseObjectLiteral);
                        }
                        if (!inErrorRecovery) {
                            return isStartOfLeftHandSideExpression() && !isHeritageClauseExtendsOrImplementsKeyword();
                        }
                        else {
                            // If we're in error recovery we tighten up what we're willing to match.
                            // That way we don't treat something like "this" as a valid heritage clause
                            // element during recovery.
                            return isIdentifier() && !isHeritageClauseExtendsOrImplementsKeyword();
                        }
                    case 8 /* VariableDeclarations */:
                        return isIdentifierOrPrivateIdentifierOrPattern();
                    case 10 /* ArrayBindingElements */:
                        return token() === 27 /* CommaToken */ || token() === 25 /* DotDotDotToken */ || isIdentifierOrPrivateIdentifierOrPattern();
                    case 19 /* TypeParameters */:
                        return isIdentifier();
                    case 15 /* ArrayLiteralMembers */:
                        switch (token()) {
                            case 27 /* CommaToken */:
                            case 24 /* DotToken */: // Not an array literal member, but don't want to close the array (see `tests/cases/fourslash/completionsDotInArrayLiteralInObjectLiteral.ts`)
                                return true;
                        }
                    // falls through
                    case 11 /* ArgumentExpressions */:
                        return token() === 25 /* DotDotDotToken */ || isStartOfExpression();
                    case 16 /* Parameters */:
                        return isStartOfParameter(/*isJSDocParameter*/ false);
                    case 17 /* JSDocParameters */:
                        return isStartOfParameter(/*isJSDocParameter*/ true);
                    case 20 /* TypeArguments */:
                    case 21 /* TupleElementTypes */:
                        return token() === 27 /* CommaToken */ || isStartOfType();
                    case 22 /* HeritageClauses */:
                        return isHeritageClause();
                    case 23 /* ImportOrExportSpecifiers */:
                        return ts.tokenIsIdentifierOrKeyword(token());
                    case 13 /* JsxAttributes */:
                        return ts.tokenIsIdentifierOrKeyword(token()) || token() === 18 /* OpenBraceToken */;
                    case 14 /* JsxChildren */:
                        return true;
                }
                return ts.Debug.fail("Non-exhaustive case in 'isListElement'.");
            }
            function isValidHeritageClauseObjectLiteral() {
                ts.Debug.assert(token() === 18 /* OpenBraceToken */);
                if (nextToken() === 19 /* CloseBraceToken */) {
                    // if we see "extends {}" then only treat the {} as what we're extending (and not
                    // the class body) if we have:
                    //
                    //      extends {} {
                    //      extends {},
                    //      extends {} extends
                    //      extends {} implements
                    var next = nextToken();
                    return next === 27 /* CommaToken */ || next === 18 /* OpenBraceToken */ || next === 90 /* ExtendsKeyword */ || next === 113 /* ImplementsKeyword */;
                }
                return true;
            }
            function nextTokenIsIdentifier() {
                nextToken();
                return isIdentifier();
            }
            function nextTokenIsIdentifierOrKeyword() {
                nextToken();
                return ts.tokenIsIdentifierOrKeyword(token());
            }
            function nextTokenIsIdentifierOrKeywordOrGreaterThan() {
                nextToken();
                return ts.tokenIsIdentifierOrKeywordOrGreaterThan(token());
            }
            function isHeritageClauseExtendsOrImplementsKeyword() {
                if (token() === 113 /* ImplementsKeyword */ ||
                    token() === 90 /* ExtendsKeyword */) {
                    return lookAhead(nextTokenIsStartOfExpression);
                }
                return false;
            }
            function nextTokenIsStartOfExpression() {
                nextToken();
                return isStartOfExpression();
            }
            function nextTokenIsStartOfType() {
                nextToken();
                return isStartOfType();
            }
            // True if positioned at a list terminator
            function isListTerminator(kind) {
                if (token() === 1 /* EndOfFileToken */) {
                    // Being at the end of the file ends all lists.
                    return true;
                }
                switch (kind) {
                    case 1 /* BlockStatements */:
                    case 2 /* SwitchClauses */:
                    case 4 /* TypeMembers */:
                    case 5 /* ClassMembers */:
                    case 6 /* EnumMembers */:
                    case 12 /* ObjectLiteralMembers */:
                    case 9 /* ObjectBindingElements */:
                    case 23 /* ImportOrExportSpecifiers */:
                        return token() === 19 /* CloseBraceToken */;
                    case 3 /* SwitchClauseStatements */:
                        return token() === 19 /* CloseBraceToken */ || token() === 78 /* CaseKeyword */ || token() === 84 /* DefaultKeyword */;
                    case 7 /* HeritageClauseElement */:
                        return token() === 18 /* OpenBraceToken */ || token() === 90 /* ExtendsKeyword */ || token() === 113 /* ImplementsKeyword */;
                    case 8 /* VariableDeclarations */:
                        return isVariableDeclaratorListTerminator();
                    case 19 /* TypeParameters */:
                        // Tokens other than '>' are here for better error recovery
                        return token() === 31 /* GreaterThanToken */ || token() === 20 /* OpenParenToken */ || token() === 18 /* OpenBraceToken */ || token() === 90 /* ExtendsKeyword */ || token() === 113 /* ImplementsKeyword */;
                    case 11 /* ArgumentExpressions */:
                        // Tokens other than ')' are here for better error recovery
                        return token() === 21 /* CloseParenToken */ || token() === 26 /* SemicolonToken */;
                    case 15 /* ArrayLiteralMembers */:
                    case 21 /* TupleElementTypes */:
                    case 10 /* ArrayBindingElements */:
                        return token() === 23 /* CloseBracketToken */;
                    case 17 /* JSDocParameters */:
                    case 16 /* Parameters */:
                    case 18 /* RestProperties */:
                        // Tokens other than ')' and ']' (the latter for index signatures) are here for better error recovery
                        return token() === 21 /* CloseParenToken */ || token() === 23 /* CloseBracketToken */ /*|| token === SyntaxKind.OpenBraceToken*/;
                    case 20 /* TypeArguments */:
                        // All other tokens should cause the type-argument to terminate except comma token
                        return token() !== 27 /* CommaToken */;
                    case 22 /* HeritageClauses */:
                        return token() === 18 /* OpenBraceToken */ || token() === 19 /* CloseBraceToken */;
                    case 13 /* JsxAttributes */:
                        return token() === 31 /* GreaterThanToken */ || token() === 43 /* SlashToken */;
                    case 14 /* JsxChildren */:
                        return token() === 29 /* LessThanToken */ && lookAhead(nextTokenIsSlash);
                    default:
                        return false;
                }
            }
            function isVariableDeclaratorListTerminator() {
                // If we can consume a semicolon (either explicitly, or with ASI), then consider us done
                // with parsing the list of variable declarators.
                if (canParseSemicolon()) {
                    return true;
                }
                // in the case where we're parsing the variable declarator of a 'for-in' statement, we
                // are done if we see an 'in' keyword in front of us. Same with for-of
                if (isInOrOfKeyword(token())) {
                    return true;
                }
                // ERROR RECOVERY TWEAK:
                // For better error recovery, if we see an '=>' then we just stop immediately.  We've got an
                // arrow function here and it's going to be very unlikely that we'll resynchronize and get
                // another variable declaration.
                if (token() === 38 /* EqualsGreaterThanToken */) {
                    return true;
                }
                // Keep trying to parse out variable declarators.
                return false;
            }
            // True if positioned at element or terminator of the current list or any enclosing list
            function isInSomeParsingContext() {
                for (var kind = 0; kind < 24 /* Count */; kind++) {
                    if (parsingContext & (1 << kind)) {
                        if (isListElement(kind, /*inErrorRecovery*/ true) || isListTerminator(kind)) {
                            return true;
                        }
                    }
                }
                return false;
            }
            // Parses a list of elements
            function parseList(kind, parseElement) {
                var saveParsingContext = parsingContext;
                parsingContext |= 1 << kind;
                var list = [];
                var listPos = getNodePos();
                while (!isListTerminator(kind)) {
                    if (isListElement(kind, /*inErrorRecovery*/ false)) {
                        var element = parseListElement(kind, parseElement);
                        list.push(element);
                        continue;
                    }
                    if (abortParsingListOrMoveToNextToken(kind)) {
                        break;
                    }
                }
                parsingContext = saveParsingContext;
                return createNodeArray(list, listPos);
            }
            function parseListElement(parsingContext, parseElement) {
                var node = currentNode(parsingContext);
                if (node) {
                    return consumeNode(node);
                }
                return parseElement();
            }
            function currentNode(parsingContext) {
                // If we don't have a cursor or the parsing context isn't reusable, there's nothing to reuse.
                //
                // If there is an outstanding parse error that we've encountered, but not attached to
                // some node, then we cannot get a node from the old source tree.  This is because we
                // want to mark the next node we encounter as being unusable.
                //
                // Note: This may be too conservative.  Perhaps we could reuse the node and set the bit
                // on it (or its leftmost child) as having the error.  For now though, being conservative
                // is nice and likely won't ever affect perf.
                if (!syntaxCursor || !isReusableParsingContext(parsingContext) || parseErrorBeforeNextFinishedNode) {
                    return undefined;
                }
                var node = syntaxCursor.currentNode(scanner.getStartPos());
                // Can't reuse a missing node.
                // Can't reuse a node that intersected the change range.
                // Can't reuse a node that contains a parse error.  This is necessary so that we
                // produce the same set of errors again.
                if (ts.nodeIsMissing(node) || node.intersectsChange || ts.containsParseError(node)) {
                    return undefined;
                }
                // We can only reuse a node if it was parsed under the same strict mode that we're
                // currently in.  i.e. if we originally parsed a node in non-strict mode, but then
                // the user added 'using strict' at the top of the file, then we can't use that node
                // again as the presence of strict mode may cause us to parse the tokens in the file
                // differently.
                //
                // Note: we *can* reuse tokens when the strict mode changes.  That's because tokens
                // are unaffected by strict mode.  It's just the parser will decide what to do with it
                // differently depending on what mode it is in.
                //
                // This also applies to all our other context flags as well.
                var nodeContextFlags = node.flags & 25358336 /* ContextFlags */;
                if (nodeContextFlags !== contextFlags) {
                    return undefined;
                }
                // Ok, we have a node that looks like it could be reused.  Now verify that it is valid
                // in the current list parsing context that we're currently at.
                if (!canReuseNode(node, parsingContext)) {
                    return undefined;
                }
                if (node.jsDocCache) {
                    // jsDocCache may include tags from parent nodes, which might have been modified.
                    node.jsDocCache = undefined;
                }
                return node;
            }
            function consumeNode(node) {
                // Move the scanner so it is after the node we just consumed.
                scanner.setTextPos(node.end);
                nextToken();
                return node;
            }
            function isReusableParsingContext(parsingContext) {
                switch (parsingContext) {
                    case 5 /* ClassMembers */:
                    case 2 /* SwitchClauses */:
                    case 0 /* SourceElements */:
                    case 1 /* BlockStatements */:
                    case 3 /* SwitchClauseStatements */:
                    case 6 /* EnumMembers */:
                    case 4 /* TypeMembers */:
                    case 8 /* VariableDeclarations */:
                    case 17 /* JSDocParameters */:
                    case 16 /* Parameters */:
                        return true;
                }
                return false;
            }
            function canReuseNode(node, parsingContext) {
                switch (parsingContext) {
                    case 5 /* ClassMembers */:
                        return isReusableClassMember(node);
                    case 2 /* SwitchClauses */:
                        return isReusableSwitchClause(node);
                    case 0 /* SourceElements */:
                    case 1 /* BlockStatements */:
                    case 3 /* SwitchClauseStatements */:
                        return isReusableStatement(node);
                    case 6 /* EnumMembers */:
                        return isReusableEnumMember(node);
                    case 4 /* TypeMembers */:
                        return isReusableTypeMember(node);
                    case 8 /* VariableDeclarations */:
                        return isReusableVariableDeclaration(node);
                    case 17 /* JSDocParameters */:
                    case 16 /* Parameters */:
                        return isReusableParameter(node);
                    // Any other lists we do not care about reusing nodes in.  But feel free to add if
                    // you can do so safely.  Danger areas involve nodes that may involve speculative
                    // parsing.  If speculative parsing is involved with the node, then the range the
                    // parser reached while looking ahead might be in the edited range (see the example
                    // in canReuseVariableDeclaratorNode for a good case of this).
                    // case ParsingContext.HeritageClauses:
                    // This would probably be safe to reuse.  There is no speculative parsing with
                    // heritage clauses.
                    // case ParsingContext.TypeParameters:
                    // This would probably be safe to reuse.  There is no speculative parsing with
                    // type parameters.  Note that that's because type *parameters* only occur in
                    // unambiguous *type* contexts.  While type *arguments* occur in very ambiguous
                    // *expression* contexts.
                    // case ParsingContext.TupleElementTypes:
                    // This would probably be safe to reuse.  There is no speculative parsing with
                    // tuple types.
                    // Technically, type argument list types are probably safe to reuse.  While
                    // speculative parsing is involved with them (since type argument lists are only
                    // produced from speculative parsing a < as a type argument list), we only have
                    // the types because speculative parsing succeeded.  Thus, the lookahead never
                    // went past the end of the list and rewound.
                    // case ParsingContext.TypeArguments:
                    // Note: these are almost certainly not safe to ever reuse.  Expressions commonly
                    // need a large amount of lookahead, and we should not reuse them as they may
                    // have actually intersected the edit.
                    // case ParsingContext.ArgumentExpressions:
                    // This is not safe to reuse for the same reason as the 'AssignmentExpression'
                    // cases.  i.e. a property assignment may end with an expression, and thus might
                    // have lookahead far beyond it's old node.
                    // case ParsingContext.ObjectLiteralMembers:
                    // This is probably not safe to reuse.  There can be speculative parsing with
                    // type names in a heritage clause.  There can be generic names in the type
                    // name list, and there can be left hand side expressions (which can have type
                    // arguments.)
                    // case ParsingContext.HeritageClauseElement:
                    // Perhaps safe to reuse, but it's unlikely we'd see more than a dozen attributes
                    // on any given element. Same for children.
                    // case ParsingContext.JsxAttributes:
                    // case ParsingContext.JsxChildren:
                }
                return false;
            }
            function isReusableClassMember(node) {
                if (node) {
                    switch (node.kind) {
                        case 162 /* Constructor */:
                        case 167 /* IndexSignature */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                        case 159 /* PropertyDeclaration */:
                        case 222 /* SemicolonClassElement */:
                            return true;
                        case 161 /* MethodDeclaration */:
                            // Method declarations are not necessarily reusable.  An object-literal
                            // may have a method calls "constructor(...)" and we must reparse that
                            // into an actual .ConstructorDeclaration.
                            var methodDeclaration = node;
                            var nameIsConstructor = methodDeclaration.name.kind === 75 /* Identifier */ &&
                                methodDeclaration.name.originalKeywordKind === 129 /* ConstructorKeyword */;
                            return !nameIsConstructor;
                    }
                }
                return false;
            }
            function isReusableSwitchClause(node) {
                if (node) {
                    switch (node.kind) {
                        case 277 /* CaseClause */:
                        case 278 /* DefaultClause */:
                            return true;
                    }
                }
                return false;
            }
            function isReusableStatement(node) {
                if (node) {
                    switch (node.kind) {
                        case 244 /* FunctionDeclaration */:
                        case 225 /* VariableStatement */:
                        case 223 /* Block */:
                        case 227 /* IfStatement */:
                        case 226 /* ExpressionStatement */:
                        case 239 /* ThrowStatement */:
                        case 235 /* ReturnStatement */:
                        case 237 /* SwitchStatement */:
                        case 234 /* BreakStatement */:
                        case 233 /* ContinueStatement */:
                        case 231 /* ForInStatement */:
                        case 232 /* ForOfStatement */:
                        case 230 /* ForStatement */:
                        case 229 /* WhileStatement */:
                        case 236 /* WithStatement */:
                        case 224 /* EmptyStatement */:
                        case 240 /* TryStatement */:
                        case 238 /* LabeledStatement */:
                        case 228 /* DoStatement */:
                        case 241 /* DebuggerStatement */:
                        case 254 /* ImportDeclaration */:
                        case 253 /* ImportEqualsDeclaration */:
                        case 260 /* ExportDeclaration */:
                        case 259 /* ExportAssignment */:
                        case 249 /* ModuleDeclaration */:
                        case 245 /* ClassDeclaration */:
                        case 246 /* InterfaceDeclaration */:
                        case 248 /* EnumDeclaration */:
                        case 247 /* TypeAliasDeclaration */:
                            return true;
                    }
                }
                return false;
            }
            function isReusableEnumMember(node) {
                return node.kind === 284 /* EnumMember */;
            }
            function isReusableTypeMember(node) {
                if (node) {
                    switch (node.kind) {
                        case 166 /* ConstructSignature */:
                        case 160 /* MethodSignature */:
                        case 167 /* IndexSignature */:
                        case 158 /* PropertySignature */:
                        case 165 /* CallSignature */:
                            return true;
                    }
                }
                return false;
            }
            function isReusableVariableDeclaration(node) {
                if (node.kind !== 242 /* VariableDeclaration */) {
                    return false;
                }
                // Very subtle incremental parsing bug.  Consider the following code:
                //
                //      let v = new List < A, B
                //
                // This is actually legal code.  It's a list of variable declarators "v = new List<A"
                // on one side and "B" on the other. If you then change that to:
                //
                //      let v = new List < A, B >()
                //
                // then we have a problem.  "v = new List<A" doesn't intersect the change range, so we
                // start reparsing at "B" and we completely fail to handle this properly.
                //
                // In order to prevent this, we do not allow a variable declarator to be reused if it
                // has an initializer.
                var variableDeclarator = node;
                return variableDeclarator.initializer === undefined;
            }
            function isReusableParameter(node) {
                if (node.kind !== 156 /* Parameter */) {
                    return false;
                }
                // See the comment in isReusableVariableDeclaration for why we do this.
                var parameter = node;
                return parameter.initializer === undefined;
            }
            // Returns true if we should abort parsing.
            function abortParsingListOrMoveToNextToken(kind) {
                parseErrorAtCurrentToken(parsingContextErrors(kind));
                if (isInSomeParsingContext()) {
                    return true;
                }
                nextToken();
                return false;
            }
            function parsingContextErrors(context) {
                switch (context) {
                    case 0 /* SourceElements */: return ts.Diagnostics.Declaration_or_statement_expected;
                    case 1 /* BlockStatements */: return ts.Diagnostics.Declaration_or_statement_expected;
                    case 2 /* SwitchClauses */: return ts.Diagnostics.case_or_default_expected;
                    case 3 /* SwitchClauseStatements */: return ts.Diagnostics.Statement_expected;
                    case 18 /* RestProperties */: // fallthrough
                    case 4 /* TypeMembers */: return ts.Diagnostics.Property_or_signature_expected;
                    case 5 /* ClassMembers */: return ts.Diagnostics.Unexpected_token_A_constructor_method_accessor_or_property_was_expected;
                    case 6 /* EnumMembers */: return ts.Diagnostics.Enum_member_expected;
                    case 7 /* HeritageClauseElement */: return ts.Diagnostics.Expression_expected;
                    case 8 /* VariableDeclarations */: return ts.Diagnostics.Variable_declaration_expected;
                    case 9 /* ObjectBindingElements */: return ts.Diagnostics.Property_destructuring_pattern_expected;
                    case 10 /* ArrayBindingElements */: return ts.Diagnostics.Array_element_destructuring_pattern_expected;
                    case 11 /* ArgumentExpressions */: return ts.Diagnostics.Argument_expression_expected;
                    case 12 /* ObjectLiteralMembers */: return ts.Diagnostics.Property_assignment_expected;
                    case 15 /* ArrayLiteralMembers */: return ts.Diagnostics.Expression_or_comma_expected;
                    case 17 /* JSDocParameters */: return ts.Diagnostics.Parameter_declaration_expected;
                    case 16 /* Parameters */: return ts.Diagnostics.Parameter_declaration_expected;
                    case 19 /* TypeParameters */: return ts.Diagnostics.Type_parameter_declaration_expected;
                    case 20 /* TypeArguments */: return ts.Diagnostics.Type_argument_expected;
                    case 21 /* TupleElementTypes */: return ts.Diagnostics.Type_expected;
                    case 22 /* HeritageClauses */: return ts.Diagnostics.Unexpected_token_expected;
                    case 23 /* ImportOrExportSpecifiers */: return ts.Diagnostics.Identifier_expected;
                    case 13 /* JsxAttributes */: return ts.Diagnostics.Identifier_expected;
                    case 14 /* JsxChildren */: return ts.Diagnostics.Identifier_expected;
                    default: return undefined; // TODO: GH#18217 `default: Debug.assertNever(context);`
                }
            }
            // Parses a comma-delimited list of elements
            function parseDelimitedList(kind, parseElement, considerSemicolonAsDelimiter) {
                var saveParsingContext = parsingContext;
                parsingContext |= 1 << kind;
                var list = [];
                var listPos = getNodePos();
                var commaStart = -1; // Meaning the previous token was not a comma
                while (true) {
                    if (isListElement(kind, /*inErrorRecovery*/ false)) {
                        var startPos = scanner.getStartPos();
                        list.push(parseListElement(kind, parseElement));
                        commaStart = scanner.getTokenPos();
                        if (parseOptional(27 /* CommaToken */)) {
                            // No need to check for a zero length node since we know we parsed a comma
                            continue;
                        }
                        commaStart = -1; // Back to the state where the last token was not a comma
                        if (isListTerminator(kind)) {
                            break;
                        }
                        // We didn't get a comma, and the list wasn't terminated, explicitly parse
                        // out a comma so we give a good error message.
                        parseExpected(27 /* CommaToken */, getExpectedCommaDiagnostic(kind));
                        // If the token was a semicolon, and the caller allows that, then skip it and
                        // continue.  This ensures we get back on track and don't result in tons of
                        // parse errors.  For example, this can happen when people do things like use
                        // a semicolon to delimit object literal members.   Note: we'll have already
                        // reported an error when we called parseExpected above.
                        if (considerSemicolonAsDelimiter && token() === 26 /* SemicolonToken */ && !scanner.hasPrecedingLineBreak()) {
                            nextToken();
                        }
                        if (startPos === scanner.getStartPos()) {
                            // What we're parsing isn't actually remotely recognizable as a element and we've consumed no tokens whatsoever
                            // Consume a token to advance the parser in some way and avoid an infinite loop
                            // This can happen when we're speculatively parsing parenthesized expressions which we think may be arrow functions,
                            // or when a modifier keyword which is disallowed as a parameter name (ie, `static` in strict mode) is supplied
                            nextToken();
                        }
                        continue;
                    }
                    if (isListTerminator(kind)) {
                        break;
                    }
                    if (abortParsingListOrMoveToNextToken(kind)) {
                        break;
                    }
                }
                parsingContext = saveParsingContext;
                var result = createNodeArray(list, listPos);
                // Recording the trailing comma is deliberately done after the previous
                // loop, and not just if we see a list terminator. This is because the list
                // may have ended incorrectly, but it is still important to know if there
                // was a trailing comma.
                // Check if the last token was a comma.
                if (commaStart >= 0) {
                    // Always preserve a trailing comma by marking it on the NodeArray
                    result.hasTrailingComma = true;
                }
                return result;
            }
            function getExpectedCommaDiagnostic(kind) {
                return kind === 6 /* EnumMembers */ ? ts.Diagnostics.An_enum_member_name_must_be_followed_by_a_or : undefined;
            }
            function createMissingList() {
                var list = createNodeArray([], getNodePos());
                list.isMissingList = true;
                return list;
            }
            function isMissingList(arr) {
                return !!arr.isMissingList;
            }
            function parseBracketedList(kind, parseElement, open, close) {
                if (parseExpected(open)) {
                    var result = parseDelimitedList(kind, parseElement);
                    parseExpected(close);
                    return result;
                }
                return createMissingList();
            }
            function parseEntityName(allowReservedWords, diagnosticMessage) {
                var entity = allowReservedWords ? parseIdentifierName(diagnosticMessage) : parseIdentifier(diagnosticMessage);
                var dotPos = scanner.getStartPos();
                while (parseOptional(24 /* DotToken */)) {
                    if (token() === 29 /* LessThanToken */) {
                        // the entity is part of a JSDoc-style generic, so record the trailing dot for later error reporting
                        entity.jsdocDotPos = dotPos;
                        break;
                    }
                    dotPos = scanner.getStartPos();
                    entity = createQualifiedName(entity, parseRightSideOfDot(allowReservedWords, /* allowPrivateIdentifiers */ false));
                }
                return entity;
            }
            function createQualifiedName(entity, name) {
                var node = createNode(153 /* QualifiedName */, entity.pos);
                node.left = entity;
                node.right = name;
                return finishNode(node);
            }
            function parseRightSideOfDot(allowIdentifierNames, allowPrivateIdentifiers) {
                // Technically a keyword is valid here as all identifiers and keywords are identifier names.
                // However, often we'll encounter this in error situations when the identifier or keyword
                // is actually starting another valid construct.
                //
                // So, we check for the following specific case:
                //
                //      name.
                //      identifierOrKeyword identifierNameOrKeyword
                //
                // Note: the newlines are important here.  For example, if that above code
                // were rewritten into:
                //
                //      name.identifierOrKeyword
                //      identifierNameOrKeyword
                //
                // Then we would consider it valid.  That's because ASI would take effect and
                // the code would be implicitly: "name.identifierOrKeyword; identifierNameOrKeyword".
                // In the first case though, ASI will not take effect because there is not a
                // line terminator after the identifier or keyword.
                if (scanner.hasPrecedingLineBreak() && ts.tokenIsIdentifierOrKeyword(token())) {
                    var matchesPattern = lookAhead(nextTokenIsIdentifierOrKeywordOnSameLine);
                    if (matchesPattern) {
                        // Report that we need an identifier.  However, report it right after the dot,
                        // and not on the next token.  This is because the next token might actually
                        // be an identifier and the error would be quite confusing.
                        return createMissingNode(75 /* Identifier */, /*reportAtCurrentPosition*/ true, ts.Diagnostics.Identifier_expected);
                    }
                }
                if (token() === 76 /* PrivateIdentifier */) {
                    var node = parsePrivateIdentifier();
                    return allowPrivateIdentifiers ? node : createMissingNode(75 /* Identifier */, /*reportAtCurrentPosition*/ true, ts.Diagnostics.Identifier_expected);
                }
                return allowIdentifierNames ? parseIdentifierName() : parseIdentifier();
            }
            function parseTemplateExpression() {
                var template = createNode(211 /* TemplateExpression */);
                template.head = parseTemplateHead();
                ts.Debug.assert(template.head.kind === 15 /* TemplateHead */, "Template head has wrong token kind");
                var list = [];
                var listPos = getNodePos();
                do {
                    list.push(parseTemplateSpan());
                } while (ts.last(list).literal.kind === 16 /* TemplateMiddle */);
                template.templateSpans = createNodeArray(list, listPos);
                return finishNode(template);
            }
            function parseTemplateSpan() {
                var span = createNode(221 /* TemplateSpan */);
                span.expression = allowInAnd(parseExpression);
                var literal;
                if (token() === 19 /* CloseBraceToken */) {
                    reScanTemplateToken();
                    literal = parseTemplateMiddleOrTemplateTail();
                }
                else {
                    literal = parseExpectedToken(17 /* TemplateTail */, ts.Diagnostics._0_expected, ts.tokenToString(19 /* CloseBraceToken */));
                }
                span.literal = literal;
                return finishNode(span);
            }
            function parseLiteralNode() {
                return parseLiteralLikeNode(token());
            }
            function parseTemplateHead() {
                var fragment = parseLiteralLikeNode(token());
                ts.Debug.assert(fragment.kind === 15 /* TemplateHead */, "Template head has wrong token kind");
                return fragment;
            }
            function parseTemplateMiddleOrTemplateTail() {
                var fragment = parseLiteralLikeNode(token());
                ts.Debug.assert(fragment.kind === 16 /* TemplateMiddle */ || fragment.kind === 17 /* TemplateTail */, "Template fragment has wrong token kind");
                return fragment;
            }
            function parseLiteralLikeNode(kind) {
                var node = createNode(kind);
                node.text = scanner.getTokenValue();
                switch (kind) {
                    case 14 /* NoSubstitutionTemplateLiteral */:
                    case 15 /* TemplateHead */:
                    case 16 /* TemplateMiddle */:
                    case 17 /* TemplateTail */:
                        var isLast = kind === 14 /* NoSubstitutionTemplateLiteral */ || kind === 17 /* TemplateTail */;
                        var tokenText = scanner.getTokenText();
                        node.rawText = tokenText.substring(1, tokenText.length - (scanner.isUnterminated() ? 0 : isLast ? 1 : 2));
                        break;
                }
                if (scanner.hasExtendedUnicodeEscape()) {
                    node.hasExtendedUnicodeEscape = true;
                }
                if (scanner.isUnterminated()) {
                    node.isUnterminated = true;
                }
                // Octal literals are not allowed in strict mode or ES5
                // Note that theoretically the following condition would hold true literals like 009,
                // which is not octal.But because of how the scanner separates the tokens, we would
                // never get a token like this. Instead, we would get 00 and 9 as two separate tokens.
                // We also do not need to check for negatives because any prefix operator would be part of a
                // parent unary expression.
                if (node.kind === 8 /* NumericLiteral */) {
                    node.numericLiteralFlags = scanner.getTokenFlags() & 1008 /* NumericLiteralFlags */;
                }
                nextToken();
                finishNode(node);
                return node;
            }
            // TYPES
            function parseTypeReference() {
                var node = createNode(169 /* TypeReference */);
                node.typeName = parseEntityName(/*allowReservedWords*/ true, ts.Diagnostics.Type_expected);
                if (!scanner.hasPrecedingLineBreak() && reScanLessThanToken() === 29 /* LessThanToken */) {
                    node.typeArguments = parseBracketedList(20 /* TypeArguments */, parseType, 29 /* LessThanToken */, 31 /* GreaterThanToken */);
                }
                return finishNode(node);
            }
            // If true, we should abort parsing an error function.
            function typeHasArrowFunctionBlockingParseError(node) {
                switch (node.kind) {
                    case 169 /* TypeReference */:
                        return ts.nodeIsMissing(node.typeName);
                    case 170 /* FunctionType */:
                    case 171 /* ConstructorType */: {
                        var _a = node, parameters = _a.parameters, type = _a.type;
                        return isMissingList(parameters) || typeHasArrowFunctionBlockingParseError(type);
                    }
                    case 182 /* ParenthesizedType */:
                        return typeHasArrowFunctionBlockingParseError(node.type);
                    default:
                        return false;
                }
            }
            function parseThisTypePredicate(lhs) {
                nextToken();
                var node = createNode(168 /* TypePredicate */, lhs.pos);
                node.parameterName = lhs;
                node.type = parseType();
                return finishNode(node);
            }
            function parseThisTypeNode() {
                var node = createNode(183 /* ThisType */);
                nextToken();
                return finishNode(node);
            }
            function parseJSDocAllType(postFixEquals) {
                var result = createNode(295 /* JSDocAllType */);
                if (postFixEquals) {
                    return createPostfixType(299 /* JSDocOptionalType */, result);
                }
                else {
                    nextToken();
                }
                return finishNode(result);
            }
            function parseJSDocNonNullableType() {
                var result = createNode(298 /* JSDocNonNullableType */);
                nextToken();
                result.type = parseNonArrayType();
                return finishNode(result);
            }
            function parseJSDocUnknownOrNullableType() {
                var pos = scanner.getStartPos();
                // skip the ?
                nextToken();
                // Need to lookahead to decide if this is a nullable or unknown type.
                // Here are cases where we'll pick the unknown type:
                //
                //      Foo(?,
                //      { a: ? }
                //      Foo(?)
                //      Foo<?>
                //      Foo(?=
                //      (?|
                if (token() === 27 /* CommaToken */ ||
                    token() === 19 /* CloseBraceToken */ ||
                    token() === 21 /* CloseParenToken */ ||
                    token() === 31 /* GreaterThanToken */ ||
                    token() === 62 /* EqualsToken */ ||
                    token() === 51 /* BarToken */) {
                    var result = createNode(296 /* JSDocUnknownType */, pos);
                    return finishNode(result);
                }
                else {
                    var result = createNode(297 /* JSDocNullableType */, pos);
                    result.type = parseType();
                    return finishNode(result);
                }
            }
            function parseJSDocFunctionType() {
                if (lookAhead(nextTokenIsOpenParen)) {
                    var result = createNodeWithJSDoc(300 /* JSDocFunctionType */);
                    nextToken();
                    fillSignature(58 /* ColonToken */, 4 /* Type */ | 32 /* JSDoc */, result);
                    return finishNode(result);
                }
                var node = createNode(169 /* TypeReference */);
                node.typeName = parseIdentifierName();
                return finishNode(node);
            }
            function parseJSDocParameter() {
                var parameter = createNode(156 /* Parameter */);
                if (token() === 104 /* ThisKeyword */ || token() === 99 /* NewKeyword */) {
                    parameter.name = parseIdentifierName();
                    parseExpected(58 /* ColonToken */);
                }
                parameter.type = parseJSDocType();
                return finishNode(parameter);
            }
            function parseJSDocType() {
                scanner.setInJSDocType(true);
                var moduleSpecifier = parseOptionalToken(135 /* ModuleKeyword */);
                if (moduleSpecifier) {
                    var moduleTag = createNode(302 /* JSDocNamepathType */, moduleSpecifier.pos);
                    terminate: while (true) {
                        switch (token()) {
                            case 19 /* CloseBraceToken */:
                            case 1 /* EndOfFileToken */:
                            case 27 /* CommaToken */:
                            case 5 /* WhitespaceTrivia */:
                                break terminate;
                            default:
                                nextTokenJSDoc();
                        }
                    }
                    scanner.setInJSDocType(false);
                    return finishNode(moduleTag);
                }
                var dotdotdot = parseOptionalToken(25 /* DotDotDotToken */);
                var type = parseTypeOrTypePredicate();
                scanner.setInJSDocType(false);
                if (dotdotdot) {
                    var variadic = createNode(301 /* JSDocVariadicType */, dotdotdot.pos);
                    variadic.type = type;
                    type = finishNode(variadic);
                }
                if (token() === 62 /* EqualsToken */) {
                    return createPostfixType(299 /* JSDocOptionalType */, type);
                }
                return type;
            }
            function parseTypeQuery() {
                var node = createNode(172 /* TypeQuery */);
                parseExpected(108 /* TypeOfKeyword */);
                node.exprName = parseEntityName(/*allowReservedWords*/ true);
                return finishNode(node);
            }
            function parseTypeParameter() {
                var node = createNode(155 /* TypeParameter */);
                node.name = parseIdentifier();
                if (parseOptional(90 /* ExtendsKeyword */)) {
                    // It's not uncommon for people to write improper constraints to a generic.  If the
                    // user writes a constraint that is an expression and not an actual type, then parse
                    // it out as an expression (so we can recover well), but report that a type is needed
                    // instead.
                    if (isStartOfType() || !isStartOfExpression()) {
                        node.constraint = parseType();
                    }
                    else {
                        // It was not a type, and it looked like an expression.  Parse out an expression
                        // here so we recover well.  Note: it is important that we call parseUnaryExpression
                        // and not parseExpression here.  If the user has:
                        //
                        //      <T extends "">
                        //
                        // We do *not* want to consume the `>` as we're consuming the expression for "".
                        node.expression = parseUnaryExpressionOrHigher();
                    }
                }
                if (parseOptional(62 /* EqualsToken */)) {
                    node.default = parseType();
                }
                return finishNode(node);
            }
            function parseTypeParameters() {
                if (token() === 29 /* LessThanToken */) {
                    return parseBracketedList(19 /* TypeParameters */, parseTypeParameter, 29 /* LessThanToken */, 31 /* GreaterThanToken */);
                }
            }
            function parseParameterType() {
                if (parseOptional(58 /* ColonToken */)) {
                    return parseType();
                }
                return undefined;
            }
            function isStartOfParameter(isJSDocParameter) {
                return token() === 25 /* DotDotDotToken */ ||
                    isIdentifierOrPrivateIdentifierOrPattern() ||
                    ts.isModifierKind(token()) ||
                    token() === 59 /* AtToken */ ||
                    isStartOfType(/*inStartOfParameter*/ !isJSDocParameter);
            }
            function parseParameter() {
                var node = createNodeWithJSDoc(156 /* Parameter */);
                if (token() === 104 /* ThisKeyword */) {
                    node.name = createIdentifier(/*isIdentifier*/ true);
                    node.type = parseParameterType();
                    return finishNode(node);
                }
                node.decorators = parseDecorators();
                node.modifiers = parseModifiers();
                node.dotDotDotToken = parseOptionalToken(25 /* DotDotDotToken */);
                // FormalParameter [Yield,Await]:
                //      BindingElement[?Yield,?Await]
                node.name = parseIdentifierOrPattern(ts.Diagnostics.Private_identifiers_cannot_be_used_as_parameters);
                if (ts.getFullWidth(node.name) === 0 && !node.modifiers && ts.isModifierKind(token())) {
                    // in cases like
                    // 'use strict'
                    // function foo(static)
                    // isParameter('static') === true, because of isModifier('static')
                    // however 'static' is not a legal identifier in a strict mode.
                    // so result of this function will be ParameterDeclaration (flags = 0, name = missing, type = undefined, initializer = undefined)
                    // and current token will not change => parsing of the enclosing parameter list will last till the end of time (or OOM)
                    // to avoid this we'll advance cursor to the next token.
                    nextToken();
                }
                node.questionToken = parseOptionalToken(57 /* QuestionToken */);
                node.type = parseParameterType();
                node.initializer = parseInitializer();
                return finishNode(node);
            }
            /**
             * Note: If returnToken is EqualsGreaterThanToken, `signature.type` will always be defined.
             * @returns If return type parsing succeeds
             */
            function fillSignature(returnToken, flags, signature) {
                if (!(flags & 32 /* JSDoc */)) {
                    signature.typeParameters = parseTypeParameters();
                }
                var parametersParsedSuccessfully = parseParameterList(signature, flags);
                if (shouldParseReturnType(returnToken, !!(flags & 4 /* Type */))) {
                    signature.type = parseTypeOrTypePredicate();
                    if (typeHasArrowFunctionBlockingParseError(signature.type))
                        return false;
                }
                return parametersParsedSuccessfully;
            }
            function shouldParseReturnType(returnToken, isType) {
                if (returnToken === 38 /* EqualsGreaterThanToken */) {
                    parseExpected(returnToken);
                    return true;
                }
                else if (parseOptional(58 /* ColonToken */)) {
                    return true;
                }
                else if (isType && token() === 38 /* EqualsGreaterThanToken */) {
                    // This is easy to get backward, especially in type contexts, so parse the type anyway
                    parseErrorAtCurrentToken(ts.Diagnostics._0_expected, ts.tokenToString(58 /* ColonToken */));
                    nextToken();
                    return true;
                }
                return false;
            }
            // Returns true on success.
            function parseParameterList(signature, flags) {
                // FormalParameters [Yield,Await]: (modified)
                //      [empty]
                //      FormalParameterList[?Yield,Await]
                //
                // FormalParameter[Yield,Await]: (modified)
                //      BindingElement[?Yield,Await]
                //
                // BindingElement [Yield,Await]: (modified)
                //      SingleNameBinding[?Yield,?Await]
                //      BindingPattern[?Yield,?Await]Initializer [In, ?Yield,?Await] opt
                //
                // SingleNameBinding [Yield,Await]:
                //      BindingIdentifier[?Yield,?Await]Initializer [In, ?Yield,?Await] opt
                if (!parseExpected(20 /* OpenParenToken */)) {
                    signature.parameters = createMissingList();
                    return false;
                }
                var savedYieldContext = inYieldContext();
                var savedAwaitContext = inAwaitContext();
                setYieldContext(!!(flags & 1 /* Yield */));
                setAwaitContext(!!(flags & 2 /* Await */));
                signature.parameters = flags & 32 /* JSDoc */ ?
                    parseDelimitedList(17 /* JSDocParameters */, parseJSDocParameter) :
                    parseDelimitedList(16 /* Parameters */, parseParameter);
                setYieldContext(savedYieldContext);
                setAwaitContext(savedAwaitContext);
                return parseExpected(21 /* CloseParenToken */);
            }
            function parseTypeMemberSemicolon() {
                // We allow type members to be separated by commas or (possibly ASI) semicolons.
                // First check if it was a comma.  If so, we're done with the member.
                if (parseOptional(27 /* CommaToken */)) {
                    return;
                }
                // Didn't have a comma.  We must have a (possible ASI) semicolon.
                parseSemicolon();
            }
            function parseSignatureMember(kind) {
                var node = createNodeWithJSDoc(kind);
                if (kind === 166 /* ConstructSignature */) {
                    parseExpected(99 /* NewKeyword */);
                }
                fillSignature(58 /* ColonToken */, 4 /* Type */, node);
                parseTypeMemberSemicolon();
                return finishNode(node);
            }
            function isIndexSignature() {
                return token() === 22 /* OpenBracketToken */ && lookAhead(isUnambiguouslyIndexSignature);
            }
            function isUnambiguouslyIndexSignature() {
                // The only allowed sequence is:
                //
                //   [id:
                //
                // However, for error recovery, we also check the following cases:
                //
                //   [...
                //   [id,
                //   [id?,
                //   [id?:
                //   [id?]
                //   [public id
                //   [private id
                //   [protected id
                //   []
                //
                nextToken();
                if (token() === 25 /* DotDotDotToken */ || token() === 23 /* CloseBracketToken */) {
                    return true;
                }
                if (ts.isModifierKind(token())) {
                    nextToken();
                    if (isIdentifier()) {
                        return true;
                    }
                }
                else if (!isIdentifier()) {
                    return false;
                }
                else {
                    // Skip the identifier
                    nextToken();
                }
                // A colon signifies a well formed indexer
                // A comma should be a badly formed indexer because comma expressions are not allowed
                // in computed properties.
                if (token() === 58 /* ColonToken */ || token() === 27 /* CommaToken */) {
                    return true;
                }
                // Question mark could be an indexer with an optional property,
                // or it could be a conditional expression in a computed property.
                if (token() !== 57 /* QuestionToken */) {
                    return false;
                }
                // If any of the following tokens are after the question mark, it cannot
                // be a conditional expression, so treat it as an indexer.
                nextToken();
                return token() === 58 /* ColonToken */ || token() === 27 /* CommaToken */ || token() === 23 /* CloseBracketToken */;
            }
            function parseIndexSignatureDeclaration(node) {
                node.kind = 167 /* IndexSignature */;
                node.parameters = parseBracketedList(16 /* Parameters */, parseParameter, 22 /* OpenBracketToken */, 23 /* CloseBracketToken */);
                node.type = parseTypeAnnotation();
                parseTypeMemberSemicolon();
                return finishNode(node);
            }
            function parsePropertyOrMethodSignature(node) {
                node.name = parsePropertyName();
                node.questionToken = parseOptionalToken(57 /* QuestionToken */);
                if (token() === 20 /* OpenParenToken */ || token() === 29 /* LessThanToken */) {
                    node.kind = 160 /* MethodSignature */;
                    // Method signatures don't exist in expression contexts.  So they have neither
                    // [Yield] nor [Await]
                    fillSignature(58 /* ColonToken */, 4 /* Type */, node);
                }
                else {
                    node.kind = 158 /* PropertySignature */;
                    node.type = parseTypeAnnotation();
                    if (token() === 62 /* EqualsToken */) {
                        // Although type literal properties cannot not have initializers, we attempt
                        // to parse an initializer so we can report in the checker that an interface
                        // property or type literal property cannot have an initializer.
                        node.initializer = parseInitializer();
                    }
                }
                parseTypeMemberSemicolon();
                return finishNode(node);
            }
            function isTypeMemberStart() {
                // Return true if we have the start of a signature member
                if (token() === 20 /* OpenParenToken */ || token() === 29 /* LessThanToken */) {
                    return true;
                }
                var idToken = false;
                // Eat up all modifiers, but hold on to the last one in case it is actually an identifier
                while (ts.isModifierKind(token())) {
                    idToken = true;
                    nextToken();
                }
                // Index signatures and computed property names are type members
                if (token() === 22 /* OpenBracketToken */) {
                    return true;
                }
                // Try to get the first property-like token following all modifiers
                if (isLiteralPropertyName()) {
                    idToken = true;
                    nextToken();
                }
                // If we were able to get any potential identifier, check that it is
                // the start of a member declaration
                if (idToken) {
                    return token() === 20 /* OpenParenToken */ ||
                        token() === 29 /* LessThanToken */ ||
                        token() === 57 /* QuestionToken */ ||
                        token() === 58 /* ColonToken */ ||
                        token() === 27 /* CommaToken */ ||
                        canParseSemicolon();
                }
                return false;
            }
            function parseTypeMember() {
                if (token() === 20 /* OpenParenToken */ || token() === 29 /* LessThanToken */) {
                    return parseSignatureMember(165 /* CallSignature */);
                }
                if (token() === 99 /* NewKeyword */ && lookAhead(nextTokenIsOpenParenOrLessThan)) {
                    return parseSignatureMember(166 /* ConstructSignature */);
                }
                var node = createNodeWithJSDoc(0 /* Unknown */);
                node.modifiers = parseModifiers();
                if (isIndexSignature()) {
                    return parseIndexSignatureDeclaration(node);
                }
                return parsePropertyOrMethodSignature(node);
            }
            function nextTokenIsOpenParenOrLessThan() {
                nextToken();
                return token() === 20 /* OpenParenToken */ || token() === 29 /* LessThanToken */;
            }
            function nextTokenIsDot() {
                return nextToken() === 24 /* DotToken */;
            }
            function nextTokenIsOpenParenOrLessThanOrDot() {
                switch (nextToken()) {
                    case 20 /* OpenParenToken */:
                    case 29 /* LessThanToken */:
                    case 24 /* DotToken */:
                        return true;
                }
                return false;
            }
            function parseTypeLiteral() {
                var node = createNode(173 /* TypeLiteral */);
                node.members = parseObjectTypeMembers();
                return finishNode(node);
            }
            function parseObjectTypeMembers() {
                var members;
                if (parseExpected(18 /* OpenBraceToken */)) {
                    members = parseList(4 /* TypeMembers */, parseTypeMember);
                    parseExpected(19 /* CloseBraceToken */);
                }
                else {
                    members = createMissingList();
                }
                return members;
            }
            function isStartOfMappedType() {
                nextToken();
                if (token() === 39 /* PlusToken */ || token() === 40 /* MinusToken */) {
                    return nextToken() === 138 /* ReadonlyKeyword */;
                }
                if (token() === 138 /* ReadonlyKeyword */) {
                    nextToken();
                }
                return token() === 22 /* OpenBracketToken */ && nextTokenIsIdentifier() && nextToken() === 97 /* InKeyword */;
            }
            function parseMappedTypeParameter() {
                var node = createNode(155 /* TypeParameter */);
                node.name = parseIdentifier();
                parseExpected(97 /* InKeyword */);
                node.constraint = parseType();
                return finishNode(node);
            }
            function parseMappedType() {
                var node = createNode(186 /* MappedType */);
                parseExpected(18 /* OpenBraceToken */);
                if (token() === 138 /* ReadonlyKeyword */ || token() === 39 /* PlusToken */ || token() === 40 /* MinusToken */) {
                    node.readonlyToken = parseTokenNode();
                    if (node.readonlyToken.kind !== 138 /* ReadonlyKeyword */) {
                        parseExpectedToken(138 /* ReadonlyKeyword */);
                    }
                }
                parseExpected(22 /* OpenBracketToken */);
                node.typeParameter = parseMappedTypeParameter();
                parseExpected(23 /* CloseBracketToken */);
                if (token() === 57 /* QuestionToken */ || token() === 39 /* PlusToken */ || token() === 40 /* MinusToken */) {
                    node.questionToken = parseTokenNode();
                    if (node.questionToken.kind !== 57 /* QuestionToken */) {
                        parseExpectedToken(57 /* QuestionToken */);
                    }
                }
                node.type = parseTypeAnnotation();
                parseSemicolon();
                parseExpected(19 /* CloseBraceToken */);
                return finishNode(node);
            }
            function parseTupleElementType() {
                var pos = getNodePos();
                if (parseOptional(25 /* DotDotDotToken */)) {
                    var node = createNode(177 /* RestType */, pos);
                    node.type = parseType();
                    return finishNode(node);
                }
                var type = parseType();
                if (!(contextFlags & 4194304 /* JSDoc */) && type.kind === 297 /* JSDocNullableType */ && type.pos === type.type.pos) {
                    type.kind = 176 /* OptionalType */;
                }
                return type;
            }
            function parseTupleType() {
                var node = createNode(175 /* TupleType */);
                node.elementTypes = parseBracketedList(21 /* TupleElementTypes */, parseTupleElementType, 22 /* OpenBracketToken */, 23 /* CloseBracketToken */);
                return finishNode(node);
            }
            function parseParenthesizedType() {
                var node = createNode(182 /* ParenthesizedType */);
                parseExpected(20 /* OpenParenToken */);
                node.type = parseType();
                parseExpected(21 /* CloseParenToken */);
                return finishNode(node);
            }
            function parseFunctionOrConstructorType() {
                var pos = getNodePos();
                var kind = parseOptional(99 /* NewKeyword */) ? 171 /* ConstructorType */ : 170 /* FunctionType */;
                var node = createNodeWithJSDoc(kind, pos);
                fillSignature(38 /* EqualsGreaterThanToken */, 4 /* Type */, node);
                return finishNode(node);
            }
            function parseKeywordAndNoDot() {
                var node = parseTokenNode();
                return token() === 24 /* DotToken */ ? undefined : node;
            }
            function parseLiteralTypeNode(negative) {
                var node = createNode(187 /* LiteralType */);
                var unaryMinusExpression;
                if (negative) {
                    unaryMinusExpression = createNode(207 /* PrefixUnaryExpression */);
                    unaryMinusExpression.operator = 40 /* MinusToken */;
                    nextToken();
                }
                var expression = token() === 106 /* TrueKeyword */ || token() === 91 /* FalseKeyword */
                    ? parseTokenNode()
                    : parseLiteralLikeNode(token());
                if (negative) {
                    unaryMinusExpression.operand = expression;
                    finishNode(unaryMinusExpression);
                    expression = unaryMinusExpression;
                }
                node.literal = expression;
                return finishNode(node);
            }
            function isStartOfTypeOfImportType() {
                nextToken();
                return token() === 96 /* ImportKeyword */;
            }
            function parseImportType() {
                sourceFile.flags |= 1048576 /* PossiblyContainsDynamicImport */;
                var node = createNode(188 /* ImportType */);
                if (parseOptional(108 /* TypeOfKeyword */)) {
                    node.isTypeOf = true;
                }
                parseExpected(96 /* ImportKeyword */);
                parseExpected(20 /* OpenParenToken */);
                node.argument = parseType();
                parseExpected(21 /* CloseParenToken */);
                if (parseOptional(24 /* DotToken */)) {
                    node.qualifier = parseEntityName(/*allowReservedWords*/ true, ts.Diagnostics.Type_expected);
                }
                if (!scanner.hasPrecedingLineBreak() && reScanLessThanToken() === 29 /* LessThanToken */) {
                    node.typeArguments = parseBracketedList(20 /* TypeArguments */, parseType, 29 /* LessThanToken */, 31 /* GreaterThanToken */);
                }
                return finishNode(node);
            }
            function nextTokenIsNumericOrBigIntLiteral() {
                nextToken();
                return token() === 8 /* NumericLiteral */ || token() === 9 /* BigIntLiteral */;
            }
            function parseNonArrayType() {
                switch (token()) {
                    case 125 /* AnyKeyword */:
                    case 148 /* UnknownKeyword */:
                    case 143 /* StringKeyword */:
                    case 140 /* NumberKeyword */:
                    case 151 /* BigIntKeyword */:
                    case 144 /* SymbolKeyword */:
                    case 128 /* BooleanKeyword */:
                    case 146 /* UndefinedKeyword */:
                    case 137 /* NeverKeyword */:
                    case 141 /* ObjectKeyword */:
                        // If these are followed by a dot, then parse these out as a dotted type reference instead.
                        return tryParse(parseKeywordAndNoDot) || parseTypeReference();
                    case 41 /* AsteriskToken */:
                        return parseJSDocAllType(/*postfixEquals*/ false);
                    case 65 /* AsteriskEqualsToken */:
                        return parseJSDocAllType(/*postfixEquals*/ true);
                    case 60 /* QuestionQuestionToken */:
                        // If there is '??', consider that is prefix '?' in JSDoc type.
                        scanner.reScanQuestionToken();
                    // falls through
                    case 57 /* QuestionToken */:
                        return parseJSDocUnknownOrNullableType();
                    case 94 /* FunctionKeyword */:
                        return parseJSDocFunctionType();
                    case 53 /* ExclamationToken */:
                        return parseJSDocNonNullableType();
                    case 14 /* NoSubstitutionTemplateLiteral */:
                    case 10 /* StringLiteral */:
                    case 8 /* NumericLiteral */:
                    case 9 /* BigIntLiteral */:
                    case 106 /* TrueKeyword */:
                    case 91 /* FalseKeyword */:
                        return parseLiteralTypeNode();
                    case 40 /* MinusToken */:
                        return lookAhead(nextTokenIsNumericOrBigIntLiteral) ? parseLiteralTypeNode(/*negative*/ true) : parseTypeReference();
                    case 110 /* VoidKeyword */:
                    case 100 /* NullKeyword */:
                        return parseTokenNode();
                    case 104 /* ThisKeyword */: {
                        var thisKeyword = parseThisTypeNode();
                        if (token() === 133 /* IsKeyword */ && !scanner.hasPrecedingLineBreak()) {
                            return parseThisTypePredicate(thisKeyword);
                        }
                        else {
                            return thisKeyword;
                        }
                    }
                    case 108 /* TypeOfKeyword */:
                        return lookAhead(isStartOfTypeOfImportType) ? parseImportType() : parseTypeQuery();
                    case 18 /* OpenBraceToken */:
                        return lookAhead(isStartOfMappedType) ? parseMappedType() : parseTypeLiteral();
                    case 22 /* OpenBracketToken */:
                        return parseTupleType();
                    case 20 /* OpenParenToken */:
                        return parseParenthesizedType();
                    case 96 /* ImportKeyword */:
                        return parseImportType();
                    case 124 /* AssertsKeyword */:
                        return lookAhead(nextTokenIsIdentifierOrKeywordOnSameLine) ? parseAssertsTypePredicate() : parseTypeReference();
                    default:
                        return parseTypeReference();
                }
            }
            function isStartOfType(inStartOfParameter) {
                switch (token()) {
                    case 125 /* AnyKeyword */:
                    case 148 /* UnknownKeyword */:
                    case 143 /* StringKeyword */:
                    case 140 /* NumberKeyword */:
                    case 151 /* BigIntKeyword */:
                    case 128 /* BooleanKeyword */:
                    case 138 /* ReadonlyKeyword */:
                    case 144 /* SymbolKeyword */:
                    case 147 /* UniqueKeyword */:
                    case 110 /* VoidKeyword */:
                    case 146 /* UndefinedKeyword */:
                    case 100 /* NullKeyword */:
                    case 104 /* ThisKeyword */:
                    case 108 /* TypeOfKeyword */:
                    case 137 /* NeverKeyword */:
                    case 18 /* OpenBraceToken */:
                    case 22 /* OpenBracketToken */:
                    case 29 /* LessThanToken */:
                    case 51 /* BarToken */:
                    case 50 /* AmpersandToken */:
                    case 99 /* NewKeyword */:
                    case 10 /* StringLiteral */:
                    case 8 /* NumericLiteral */:
                    case 9 /* BigIntLiteral */:
                    case 106 /* TrueKeyword */:
                    case 91 /* FalseKeyword */:
                    case 141 /* ObjectKeyword */:
                    case 41 /* AsteriskToken */:
                    case 57 /* QuestionToken */:
                    case 53 /* ExclamationToken */:
                    case 25 /* DotDotDotToken */:
                    case 132 /* InferKeyword */:
                    case 96 /* ImportKeyword */:
                    case 124 /* AssertsKeyword */:
                        return true;
                    case 94 /* FunctionKeyword */:
                        return !inStartOfParameter;
                    case 40 /* MinusToken */:
                        return !inStartOfParameter && lookAhead(nextTokenIsNumericOrBigIntLiteral);
                    case 20 /* OpenParenToken */:
                        // Only consider '(' the start of a type if followed by ')', '...', an identifier, a modifier,
                        // or something that starts a type. We don't want to consider things like '(1)' a type.
                        return !inStartOfParameter && lookAhead(isStartOfParenthesizedOrFunctionType);
                    default:
                        return isIdentifier();
                }
            }
            function isStartOfParenthesizedOrFunctionType() {
                nextToken();
                return token() === 21 /* CloseParenToken */ || isStartOfParameter(/*isJSDocParameter*/ false) || isStartOfType();
            }
            function parsePostfixTypeOrHigher() {
                var type = parseNonArrayType();
                while (!scanner.hasPrecedingLineBreak()) {
                    switch (token()) {
                        case 53 /* ExclamationToken */:
                            type = createPostfixType(298 /* JSDocNonNullableType */, type);
                            break;
                        case 57 /* QuestionToken */:
                            // If not in JSDoc and next token is start of a type we have a conditional type
                            if (!(contextFlags & 4194304 /* JSDoc */) && lookAhead(nextTokenIsStartOfType)) {
                                return type;
                            }
                            type = createPostfixType(297 /* JSDocNullableType */, type);
                            break;
                        case 22 /* OpenBracketToken */:
                            parseExpected(22 /* OpenBracketToken */);
                            if (isStartOfType()) {
                                var node = createNode(185 /* IndexedAccessType */, type.pos);
                                node.objectType = type;
                                node.indexType = parseType();
                                parseExpected(23 /* CloseBracketToken */);
                                type = finishNode(node);
                            }
                            else {
                                var node = createNode(174 /* ArrayType */, type.pos);
                                node.elementType = type;
                                parseExpected(23 /* CloseBracketToken */);
                                type = finishNode(node);
                            }
                            break;
                        default:
                            return type;
                    }
                }
                return type;
            }
            function createPostfixType(kind, type) {
                nextToken();
                var postfix = createNode(kind, type.pos);
                postfix.type = type;
                return finishNode(postfix);
            }
            function parseTypeOperator(operator) {
                var node = createNode(184 /* TypeOperator */);
                parseExpected(operator);
                node.operator = operator;
                node.type = parseTypeOperatorOrHigher();
                return finishNode(node);
            }
            function parseInferType() {
                var node = createNode(181 /* InferType */);
                parseExpected(132 /* InferKeyword */);
                var typeParameter = createNode(155 /* TypeParameter */);
                typeParameter.name = parseIdentifier();
                node.typeParameter = finishNode(typeParameter);
                return finishNode(node);
            }
            function parseTypeOperatorOrHigher() {
                var operator = token();
                switch (operator) {
                    case 134 /* KeyOfKeyword */:
                    case 147 /* UniqueKeyword */:
                    case 138 /* ReadonlyKeyword */:
                        return parseTypeOperator(operator);
                    case 132 /* InferKeyword */:
                        return parseInferType();
                }
                return parsePostfixTypeOrHigher();
            }
            function parseUnionOrIntersectionType(kind, parseConstituentType, operator) {
                var start = scanner.getStartPos();
                var hasLeadingOperator = parseOptional(operator);
                var type = parseConstituentType();
                if (token() === operator || hasLeadingOperator) {
                    var types = [type];
                    while (parseOptional(operator)) {
                        types.push(parseConstituentType());
                    }
                    var node = createNode(kind, start);
                    node.types = createNodeArray(types, start);
                    type = finishNode(node);
                }
                return type;
            }
            function parseIntersectionTypeOrHigher() {
                return parseUnionOrIntersectionType(179 /* IntersectionType */, parseTypeOperatorOrHigher, 50 /* AmpersandToken */);
            }
            function parseUnionTypeOrHigher() {
                return parseUnionOrIntersectionType(178 /* UnionType */, parseIntersectionTypeOrHigher, 51 /* BarToken */);
            }
            function isStartOfFunctionType() {
                if (token() === 29 /* LessThanToken */) {
                    return true;
                }
                return token() === 20 /* OpenParenToken */ && lookAhead(isUnambiguouslyStartOfFunctionType);
            }
            function skipParameterStart() {
                if (ts.isModifierKind(token())) {
                    // Skip modifiers
                    parseModifiers();
                }
                if (isIdentifier() || token() === 104 /* ThisKeyword */) {
                    nextToken();
                    return true;
                }
                if (token() === 22 /* OpenBracketToken */ || token() === 18 /* OpenBraceToken */) {
                    // Return true if we can parse an array or object binding pattern with no errors
                    var previousErrorCount = parseDiagnostics.length;
                    parseIdentifierOrPattern();
                    return previousErrorCount === parseDiagnostics.length;
                }
                return false;
            }
            function isUnambiguouslyStartOfFunctionType() {
                nextToken();
                if (token() === 21 /* CloseParenToken */ || token() === 25 /* DotDotDotToken */) {
                    // ( )
                    // ( ...
                    return true;
                }
                if (skipParameterStart()) {
                    // We successfully skipped modifiers (if any) and an identifier or binding pattern,
                    // now see if we have something that indicates a parameter declaration
                    if (token() === 58 /* ColonToken */ || token() === 27 /* CommaToken */ ||
                        token() === 57 /* QuestionToken */ || token() === 62 /* EqualsToken */) {
                        // ( xxx :
                        // ( xxx ,
                        // ( xxx ?
                        // ( xxx =
                        return true;
                    }
                    if (token() === 21 /* CloseParenToken */) {
                        nextToken();
                        if (token() === 38 /* EqualsGreaterThanToken */) {
                            // ( xxx ) =>
                            return true;
                        }
                    }
                }
                return false;
            }
            function parseTypeOrTypePredicate() {
                var typePredicateVariable = isIdentifier() && tryParse(parseTypePredicatePrefix);
                var type = parseType();
                if (typePredicateVariable) {
                    var node = createNode(168 /* TypePredicate */, typePredicateVariable.pos);
                    node.assertsModifier = undefined;
                    node.parameterName = typePredicateVariable;
                    node.type = type;
                    return finishNode(node);
                }
                else {
                    return type;
                }
            }
            function parseTypePredicatePrefix() {
                var id = parseIdentifier();
                if (token() === 133 /* IsKeyword */ && !scanner.hasPrecedingLineBreak()) {
                    nextToken();
                    return id;
                }
            }
            function parseAssertsTypePredicate() {
                var node = createNode(168 /* TypePredicate */);
                node.assertsModifier = parseExpectedToken(124 /* AssertsKeyword */);
                node.parameterName = token() === 104 /* ThisKeyword */ ? parseThisTypeNode() : parseIdentifier();
                node.type = parseOptional(133 /* IsKeyword */) ? parseType() : undefined;
                return finishNode(node);
            }
            function parseType() {
                // The rules about 'yield' only apply to actual code/expression contexts.  They don't
                // apply to 'type' contexts.  So we disable these parameters here before moving on.
                return doOutsideOfContext(40960 /* TypeExcludesFlags */, parseTypeWorker);
            }
            function parseTypeWorker(noConditionalTypes) {
                if (isStartOfFunctionType() || token() === 99 /* NewKeyword */) {
                    return parseFunctionOrConstructorType();
                }
                var type = parseUnionTypeOrHigher();
                if (!noConditionalTypes && !scanner.hasPrecedingLineBreak() && parseOptional(90 /* ExtendsKeyword */)) {
                    var node = createNode(180 /* ConditionalType */, type.pos);
                    node.checkType = type;
                    // The type following 'extends' is not permitted to be another conditional type
                    node.extendsType = parseTypeWorker(/*noConditionalTypes*/ true);
                    parseExpected(57 /* QuestionToken */);
                    node.trueType = parseTypeWorker();
                    parseExpected(58 /* ColonToken */);
                    node.falseType = parseTypeWorker();
                    return finishNode(node);
                }
                return type;
            }
            function parseTypeAnnotation() {
                return parseOptional(58 /* ColonToken */) ? parseType() : undefined;
            }
            // EXPRESSIONS
            function isStartOfLeftHandSideExpression() {
                switch (token()) {
                    case 104 /* ThisKeyword */:
                    case 102 /* SuperKeyword */:
                    case 100 /* NullKeyword */:
                    case 106 /* TrueKeyword */:
                    case 91 /* FalseKeyword */:
                    case 8 /* NumericLiteral */:
                    case 9 /* BigIntLiteral */:
                    case 10 /* StringLiteral */:
                    case 14 /* NoSubstitutionTemplateLiteral */:
                    case 15 /* TemplateHead */:
                    case 20 /* OpenParenToken */:
                    case 22 /* OpenBracketToken */:
                    case 18 /* OpenBraceToken */:
                    case 94 /* FunctionKeyword */:
                    case 80 /* ClassKeyword */:
                    case 99 /* NewKeyword */:
                    case 43 /* SlashToken */:
                    case 67 /* SlashEqualsToken */:
                    case 75 /* Identifier */:
                        return true;
                    case 96 /* ImportKeyword */:
                        return lookAhead(nextTokenIsOpenParenOrLessThanOrDot);
                    default:
                        return isIdentifier();
                }
            }
            function isStartOfExpression() {
                if (isStartOfLeftHandSideExpression()) {
                    return true;
                }
                switch (token()) {
                    case 39 /* PlusToken */:
                    case 40 /* MinusToken */:
                    case 54 /* TildeToken */:
                    case 53 /* ExclamationToken */:
                    case 85 /* DeleteKeyword */:
                    case 108 /* TypeOfKeyword */:
                    case 110 /* VoidKeyword */:
                    case 45 /* PlusPlusToken */:
                    case 46 /* MinusMinusToken */:
                    case 29 /* LessThanToken */:
                    case 127 /* AwaitKeyword */:
                    case 121 /* YieldKeyword */:
                    case 76 /* PrivateIdentifier */:
                        // Yield/await always starts an expression.  Either it is an identifier (in which case
                        // it is definitely an expression).  Or it's a keyword (either because we're in
                        // a generator or async function, or in strict mode (or both)) and it started a yield or await expression.
                        return true;
                    default:
                        // Error tolerance.  If we see the start of some binary operator, we consider
                        // that the start of an expression.  That way we'll parse out a missing identifier,
                        // give a good message about an identifier being missing, and then consume the
                        // rest of the binary expression.
                        if (isBinaryOperator()) {
                            return true;
                        }
                        return isIdentifier();
                }
            }
            function isStartOfExpressionStatement() {
                // As per the grammar, none of '{' or 'function' or 'class' can start an expression statement.
                return token() !== 18 /* OpenBraceToken */ &&
                    token() !== 94 /* FunctionKeyword */ &&
                    token() !== 80 /* ClassKeyword */ &&
                    token() !== 59 /* AtToken */ &&
                    isStartOfExpression();
            }
            function parseExpression() {
                // Expression[in]:
                //      AssignmentExpression[in]
                //      Expression[in] , AssignmentExpression[in]
                // clear the decorator context when parsing Expression, as it should be unambiguous when parsing a decorator
                var saveDecoratorContext = inDecoratorContext();
                if (saveDecoratorContext) {
                    setDecoratorContext(/*val*/ false);
                }
                var expr = parseAssignmentExpressionOrHigher();
                var operatorToken;
                while ((operatorToken = parseOptionalToken(27 /* CommaToken */))) {
                    expr = makeBinaryExpression(expr, operatorToken, parseAssignmentExpressionOrHigher());
                }
                if (saveDecoratorContext) {
                    setDecoratorContext(/*val*/ true);
                }
                return expr;
            }
            function parseInitializer() {
                return parseOptional(62 /* EqualsToken */) ? parseAssignmentExpressionOrHigher() : undefined;
            }
            function parseAssignmentExpressionOrHigher() {
                //  AssignmentExpression[in,yield]:
                //      1) ConditionalExpression[?in,?yield]
                //      2) LeftHandSideExpression = AssignmentExpression[?in,?yield]
                //      3) LeftHandSideExpression AssignmentOperator AssignmentExpression[?in,?yield]
                //      4) ArrowFunctionExpression[?in,?yield]
                //      5) AsyncArrowFunctionExpression[in,yield,await]
                //      6) [+Yield] YieldExpression[?In]
                //
                // Note: for ease of implementation we treat productions '2' and '3' as the same thing.
                // (i.e. they're both BinaryExpressions with an assignment operator in it).
                // First, do the simple check if we have a YieldExpression (production '6').
                if (isYieldExpression()) {
                    return parseYieldExpression();
                }
                // Then, check if we have an arrow function (production '4' and '5') that starts with a parenthesized
                // parameter list or is an async arrow function.
                // AsyncArrowFunctionExpression:
                //      1) async[no LineTerminator here]AsyncArrowBindingIdentifier[?Yield][no LineTerminator here]=>AsyncConciseBody[?In]
                //      2) CoverCallExpressionAndAsyncArrowHead[?Yield, ?Await][no LineTerminator here]=>AsyncConciseBody[?In]
                // Production (1) of AsyncArrowFunctionExpression is parsed in "tryParseAsyncSimpleArrowFunctionExpression".
                // And production (2) is parsed in "tryParseParenthesizedArrowFunctionExpression".
                //
                // If we do successfully parse arrow-function, we must *not* recurse for productions 1, 2 or 3. An ArrowFunction is
                // not a LeftHandSideExpression, nor does it start a ConditionalExpression.  So we are done
                // with AssignmentExpression if we see one.
                var arrowExpression = tryParseParenthesizedArrowFunctionExpression() || tryParseAsyncSimpleArrowFunctionExpression();
                if (arrowExpression) {
                    return arrowExpression;
                }
                // Now try to see if we're in production '1', '2' or '3'.  A conditional expression can
                // start with a LogicalOrExpression, while the assignment productions can only start with
                // LeftHandSideExpressions.
                //
                // So, first, we try to just parse out a BinaryExpression.  If we get something that is a
                // LeftHandSide or higher, then we can try to parse out the assignment expression part.
                // Otherwise, we try to parse out the conditional expression bit.  We want to allow any
                // binary expression here, so we pass in the 'lowest' precedence here so that it matches
                // and consumes anything.
                var expr = parseBinaryExpressionOrHigher(/*precedence*/ 0);
                // To avoid a look-ahead, we did not handle the case of an arrow function with a single un-parenthesized
                // parameter ('x => ...') above. We handle it here by checking if the parsed expression was a single
                // identifier and the current token is an arrow.
                if (expr.kind === 75 /* Identifier */ && token() === 38 /* EqualsGreaterThanToken */) {
                    return parseSimpleArrowFunctionExpression(expr);
                }
                // Now see if we might be in cases '2' or '3'.
                // If the expression was a LHS expression, and we have an assignment operator, then
                // we're in '2' or '3'. Consume the assignment and return.
                //
                // Note: we call reScanGreaterToken so that we get an appropriately merged token
                // for cases like `> > =` becoming `>>=`
                if (ts.isLeftHandSideExpression(expr) && ts.isAssignmentOperator(reScanGreaterToken())) {
                    return makeBinaryExpression(expr, parseTokenNode(), parseAssignmentExpressionOrHigher());
                }
                // It wasn't an assignment or a lambda.  This is a conditional expression:
                return parseConditionalExpressionRest(expr);
            }
            function isYieldExpression() {
                if (token() === 121 /* YieldKeyword */) {
                    // If we have a 'yield' keyword, and this is a context where yield expressions are
                    // allowed, then definitely parse out a yield expression.
                    if (inYieldContext()) {
                        return true;
                    }
                    // We're in a context where 'yield expr' is not allowed.  However, if we can
                    // definitely tell that the user was trying to parse a 'yield expr' and not
                    // just a normal expr that start with a 'yield' identifier, then parse out
                    // a 'yield expr'.  We can then report an error later that they are only
                    // allowed in generator expressions.
                    //
                    // for example, if we see 'yield(foo)', then we'll have to treat that as an
                    // invocation expression of something called 'yield'.  However, if we have
                    // 'yield foo' then that is not legal as a normal expression, so we can
                    // definitely recognize this as a yield expression.
                    //
                    // for now we just check if the next token is an identifier.  More heuristics
                    // can be added here later as necessary.  We just need to make sure that we
                    // don't accidentally consume something legal.
                    return lookAhead(nextTokenIsIdentifierOrKeywordOrLiteralOnSameLine);
                }
                return false;
            }
            function nextTokenIsIdentifierOnSameLine() {
                nextToken();
                return !scanner.hasPrecedingLineBreak() && isIdentifier();
            }
            function parseYieldExpression() {
                var node = createNode(212 /* YieldExpression */);
                // YieldExpression[In] :
                //      yield
                //      yield [no LineTerminator here] [Lexical goal InputElementRegExp]AssignmentExpression[?In, Yield]
                //      yield [no LineTerminator here] * [Lexical goal InputElementRegExp]AssignmentExpression[?In, Yield]
                nextToken();
                if (!scanner.hasPrecedingLineBreak() &&
                    (token() === 41 /* AsteriskToken */ || isStartOfExpression())) {
                    node.asteriskToken = parseOptionalToken(41 /* AsteriskToken */);
                    node.expression = parseAssignmentExpressionOrHigher();
                    return finishNode(node);
                }
                else {
                    // if the next token is not on the same line as yield.  or we don't have an '*' or
                    // the start of an expression, then this is just a simple "yield" expression.
                    return finishNode(node);
                }
            }
            function parseSimpleArrowFunctionExpression(identifier, asyncModifier) {
                ts.Debug.assert(token() === 38 /* EqualsGreaterThanToken */, "parseSimpleArrowFunctionExpression should only have been called if we had a =>");
                var node;
                if (asyncModifier) {
                    node = createNode(202 /* ArrowFunction */, asyncModifier.pos);
                    node.modifiers = asyncModifier;
                }
                else {
                    node = createNode(202 /* ArrowFunction */, identifier.pos);
                }
                var parameter = createNode(156 /* Parameter */, identifier.pos);
                parameter.name = identifier;
                finishNode(parameter);
                node.parameters = createNodeArray([parameter], parameter.pos, parameter.end);
                node.equalsGreaterThanToken = parseExpectedToken(38 /* EqualsGreaterThanToken */);
                node.body = parseArrowFunctionExpressionBody(/*isAsync*/ !!asyncModifier);
                return addJSDocComment(finishNode(node));
            }
            function tryParseParenthesizedArrowFunctionExpression() {
                var triState = isParenthesizedArrowFunctionExpression();
                if (triState === 0 /* False */) {
                    // It's definitely not a parenthesized arrow function expression.
                    return undefined;
                }
                // If we definitely have an arrow function, then we can just parse one, not requiring a
                // following => or { token. Otherwise, we *might* have an arrow function.  Try to parse
                // it out, but don't allow any ambiguity, and return 'undefined' if this could be an
                // expression instead.
                var arrowFunction = triState === 1 /* True */
                    ? parseParenthesizedArrowFunctionExpressionHead(/*allowAmbiguity*/ true)
                    : tryParse(parsePossibleParenthesizedArrowFunctionExpressionHead);
                if (!arrowFunction) {
                    // Didn't appear to actually be a parenthesized arrow function.  Just bail out.
                    return undefined;
                }
                var isAsync = hasModifierOfKind(arrowFunction, 126 /* AsyncKeyword */);
                // If we have an arrow, then try to parse the body. Even if not, try to parse if we
                // have an opening brace, just in case we're in an error state.
                var lastToken = token();
                arrowFunction.equalsGreaterThanToken = parseExpectedToken(38 /* EqualsGreaterThanToken */);
                arrowFunction.body = (lastToken === 38 /* EqualsGreaterThanToken */ || lastToken === 18 /* OpenBraceToken */)
                    ? parseArrowFunctionExpressionBody(isAsync)
                    : parseIdentifier();
                return finishNode(arrowFunction);
            }
            //  True        -> We definitely expect a parenthesized arrow function here.
            //  False       -> There *cannot* be a parenthesized arrow function here.
            //  Unknown     -> There *might* be a parenthesized arrow function here.
            //                 Speculatively look ahead to be sure, and rollback if not.
            function isParenthesizedArrowFunctionExpression() {
                if (token() === 20 /* OpenParenToken */ || token() === 29 /* LessThanToken */ || token() === 126 /* AsyncKeyword */) {
                    return lookAhead(isParenthesizedArrowFunctionExpressionWorker);
                }
                if (token() === 38 /* EqualsGreaterThanToken */) {
                    // ERROR RECOVERY TWEAK:
                    // If we see a standalone => try to parse it as an arrow function expression as that's
                    // likely what the user intended to write.
                    return 1 /* True */;
                }
                // Definitely not a parenthesized arrow function.
                return 0 /* False */;
            }
            function isParenthesizedArrowFunctionExpressionWorker() {
                if (token() === 126 /* AsyncKeyword */) {
                    nextToken();
                    if (scanner.hasPrecedingLineBreak()) {
                        return 0 /* False */;
                    }
                    if (token() !== 20 /* OpenParenToken */ && token() !== 29 /* LessThanToken */) {
                        return 0 /* False */;
                    }
                }
                var first = token();
                var second = nextToken();
                if (first === 20 /* OpenParenToken */) {
                    if (second === 21 /* CloseParenToken */) {
                        // Simple cases: "() =>", "(): ", and "() {".
                        // This is an arrow function with no parameters.
                        // The last one is not actually an arrow function,
                        // but this is probably what the user intended.
                        var third = nextToken();
                        switch (third) {
                            case 38 /* EqualsGreaterThanToken */:
                            case 58 /* ColonToken */:
                            case 18 /* OpenBraceToken */:
                                return 1 /* True */;
                            default:
                                return 0 /* False */;
                        }
                    }
                    // If encounter "([" or "({", this could be the start of a binding pattern.
                    // Examples:
                    //      ([ x ]) => { }
                    //      ({ x }) => { }
                    //      ([ x ])
                    //      ({ x })
                    if (second === 22 /* OpenBracketToken */ || second === 18 /* OpenBraceToken */) {
                        return 2 /* Unknown */;
                    }
                    // Simple case: "(..."
                    // This is an arrow function with a rest parameter.
                    if (second === 25 /* DotDotDotToken */) {
                        return 1 /* True */;
                    }
                    // Check for "(xxx yyy", where xxx is a modifier and yyy is an identifier. This
                    // isn't actually allowed, but we want to treat it as a lambda so we can provide
                    // a good error message.
                    if (ts.isModifierKind(second) && second !== 126 /* AsyncKeyword */ && lookAhead(nextTokenIsIdentifier)) {
                        return 1 /* True */;
                    }
                    // If we had "(" followed by something that's not an identifier,
                    // then this definitely doesn't look like a lambda.  "this" is not
                    // valid, but we want to parse it and then give a semantic error.
                    if (!isIdentifier() && second !== 104 /* ThisKeyword */) {
                        return 0 /* False */;
                    }
                    switch (nextToken()) {
                        case 58 /* ColonToken */:
                            // If we have something like "(a:", then we must have a
                            // type-annotated parameter in an arrow function expression.
                            return 1 /* True */;
                        case 57 /* QuestionToken */:
                            nextToken();
                            // If we have "(a?:" or "(a?," or "(a?=" or "(a?)" then it is definitely a lambda.
                            if (token() === 58 /* ColonToken */ || token() === 27 /* CommaToken */ || token() === 62 /* EqualsToken */ || token() === 21 /* CloseParenToken */) {
                                return 1 /* True */;
                            }
                            // Otherwise it is definitely not a lambda.
                            return 0 /* False */;
                        case 27 /* CommaToken */:
                        case 62 /* EqualsToken */:
                        case 21 /* CloseParenToken */:
                            // If we have "(a," or "(a=" or "(a)" this *could* be an arrow function
                            return 2 /* Unknown */;
                    }
                    // It is definitely not an arrow function
                    return 0 /* False */;
                }
                else {
                    ts.Debug.assert(first === 29 /* LessThanToken */);
                    // If we have "<" not followed by an identifier,
                    // then this definitely is not an arrow function.
                    if (!isIdentifier()) {
                        return 0 /* False */;
                    }
                    // JSX overrides
                    if (sourceFile.languageVariant === 1 /* JSX */) {
                        var isArrowFunctionInJsx = lookAhead(function () {
                            var third = nextToken();
                            if (third === 90 /* ExtendsKeyword */) {
                                var fourth = nextToken();
                                switch (fourth) {
                                    case 62 /* EqualsToken */:
                                    case 31 /* GreaterThanToken */:
                                        return false;
                                    default:
                                        return true;
                                }
                            }
                            else if (third === 27 /* CommaToken */) {
                                return true;
                            }
                            return false;
                        });
                        if (isArrowFunctionInJsx) {
                            return 1 /* True */;
                        }
                        return 0 /* False */;
                    }
                    // This *could* be a parenthesized arrow function.
                    return 2 /* Unknown */;
                }
            }
            function parsePossibleParenthesizedArrowFunctionExpressionHead() {
                var tokenPos = scanner.getTokenPos();
                if (notParenthesizedArrow && notParenthesizedArrow.has(tokenPos.toString())) {
                    return undefined;
                }
                var result = parseParenthesizedArrowFunctionExpressionHead(/*allowAmbiguity*/ false);
                if (!result) {
                    (notParenthesizedArrow || (notParenthesizedArrow = ts.createMap())).set(tokenPos.toString(), true);
                }
                return result;
            }
            function tryParseAsyncSimpleArrowFunctionExpression() {
                // We do a check here so that we won't be doing unnecessarily call to "lookAhead"
                if (token() === 126 /* AsyncKeyword */) {
                    if (lookAhead(isUnParenthesizedAsyncArrowFunctionWorker) === 1 /* True */) {
                        var asyncModifier = parseModifiersForArrowFunction();
                        var expr = parseBinaryExpressionOrHigher(/*precedence*/ 0);
                        return parseSimpleArrowFunctionExpression(expr, asyncModifier);
                    }
                }
                return undefined;
            }
            function isUnParenthesizedAsyncArrowFunctionWorker() {
                // AsyncArrowFunctionExpression:
                //      1) async[no LineTerminator here]AsyncArrowBindingIdentifier[?Yield][no LineTerminator here]=>AsyncConciseBody[?In]
                //      2) CoverCallExpressionAndAsyncArrowHead[?Yield, ?Await][no LineTerminator here]=>AsyncConciseBody[?In]
                if (token() === 126 /* AsyncKeyword */) {
                    nextToken();
                    // If the "async" is followed by "=>" token then it is not a beginning of an async arrow-function
                    // but instead a simple arrow-function which will be parsed inside "parseAssignmentExpressionOrHigher"
                    if (scanner.hasPrecedingLineBreak() || token() === 38 /* EqualsGreaterThanToken */) {
                        return 0 /* False */;
                    }
                    // Check for un-parenthesized AsyncArrowFunction
                    var expr = parseBinaryExpressionOrHigher(/*precedence*/ 0);
                    if (!scanner.hasPrecedingLineBreak() && expr.kind === 75 /* Identifier */ && token() === 38 /* EqualsGreaterThanToken */) {
                        return 1 /* True */;
                    }
                }
                return 0 /* False */;
            }
            function parseParenthesizedArrowFunctionExpressionHead(allowAmbiguity) {
                var node = createNodeWithJSDoc(202 /* ArrowFunction */);
                node.modifiers = parseModifiersForArrowFunction();
                var isAsync = hasModifierOfKind(node, 126 /* AsyncKeyword */) ? 2 /* Await */ : 0 /* None */;
                // Arrow functions are never generators.
                //
                // If we're speculatively parsing a signature for a parenthesized arrow function, then
                // we have to have a complete parameter list.  Otherwise we might see something like
                // a => (b => c)
                // And think that "(b =>" was actually a parenthesized arrow function with a missing
                // close paren.
                if (!fillSignature(58 /* ColonToken */, isAsync, node) && !allowAmbiguity) {
                    return undefined;
                }
                // Parsing a signature isn't enough.
                // Parenthesized arrow signatures often look like other valid expressions.
                // For instance:
                //  - "(x = 10)" is an assignment expression parsed as a signature with a default parameter value.
                //  - "(x,y)" is a comma expression parsed as a signature with two parameters.
                //  - "a ? (b): c" will have "(b):" parsed as a signature with a return type annotation.
                //  - "a ? (b): function() {}" will too, since function() is a valid JSDoc function type.
                //
                // So we need just a bit of lookahead to ensure that it can only be a signature.
                var hasJSDocFunctionType = node.type && ts.isJSDocFunctionType(node.type);
                if (!allowAmbiguity && token() !== 38 /* EqualsGreaterThanToken */ && (hasJSDocFunctionType || token() !== 18 /* OpenBraceToken */)) {
                    // Returning undefined here will cause our caller to rewind to where we started from.
                    return undefined;
                }
                return node;
            }
            function parseArrowFunctionExpressionBody(isAsync) {
                if (token() === 18 /* OpenBraceToken */) {
                    return parseFunctionBlock(isAsync ? 2 /* Await */ : 0 /* None */);
                }
                if (token() !== 26 /* SemicolonToken */ &&
                    token() !== 94 /* FunctionKeyword */ &&
                    token() !== 80 /* ClassKeyword */ &&
                    isStartOfStatement() &&
                    !isStartOfExpressionStatement()) {
                    // Check if we got a plain statement (i.e. no expression-statements, no function/class expressions/declarations)
                    //
                    // Here we try to recover from a potential error situation in the case where the
                    // user meant to supply a block. For example, if the user wrote:
                    //
                    //  a =>
                    //      let v = 0;
                    //  }
                    //
                    // they may be missing an open brace.  Check to see if that's the case so we can
                    // try to recover better.  If we don't do this, then the next close curly we see may end
                    // up preemptively closing the containing construct.
                    //
                    // Note: even when 'IgnoreMissingOpenBrace' is passed, parseBody will still error.
                    return parseFunctionBlock(16 /* IgnoreMissingOpenBrace */ | (isAsync ? 2 /* Await */ : 0 /* None */));
                }
                return isAsync
                    ? doInAwaitContext(parseAssignmentExpressionOrHigher)
                    : doOutsideOfAwaitContext(parseAssignmentExpressionOrHigher);
            }
            function parseConditionalExpressionRest(leftOperand) {
                // Note: we are passed in an expression which was produced from parseBinaryExpressionOrHigher.
                var questionToken = parseOptionalToken(57 /* QuestionToken */);
                if (!questionToken) {
                    return leftOperand;
                }
                // Note: we explicitly 'allowIn' in the whenTrue part of the condition expression, and
                // we do not that for the 'whenFalse' part.
                var node = createNode(210 /* ConditionalExpression */, leftOperand.pos);
                node.condition = leftOperand;
                node.questionToken = questionToken;
                node.whenTrue = doOutsideOfContext(disallowInAndDecoratorContext, parseAssignmentExpressionOrHigher);
                node.colonToken = parseExpectedToken(58 /* ColonToken */);
                node.whenFalse = ts.nodeIsPresent(node.colonToken)
                    ? parseAssignmentExpressionOrHigher()
                    : createMissingNode(75 /* Identifier */, /*reportAtCurrentPosition*/ false, ts.Diagnostics._0_expected, ts.tokenToString(58 /* ColonToken */));
                return finishNode(node);
            }
            function parseBinaryExpressionOrHigher(precedence) {
                var leftOperand = parseUnaryExpressionOrHigher();
                return parseBinaryExpressionRest(precedence, leftOperand);
            }
            function isInOrOfKeyword(t) {
                return t === 97 /* InKeyword */ || t === 152 /* OfKeyword */;
            }
            function parseBinaryExpressionRest(precedence, leftOperand) {
                while (true) {
                    // We either have a binary operator here, or we're finished.  We call
                    // reScanGreaterToken so that we merge token sequences like > and = into >=
                    reScanGreaterToken();
                    var newPrecedence = ts.getBinaryOperatorPrecedence(token());
                    // Check the precedence to see if we should "take" this operator
                    // - For left associative operator (all operator but **), consume the operator,
                    //   recursively call the function below, and parse binaryExpression as a rightOperand
                    //   of the caller if the new precedence of the operator is greater then or equal to the current precedence.
                    //   For example:
                    //      a - b - c;
                    //            ^token; leftOperand = b. Return b to the caller as a rightOperand
                    //      a * b - c
                    //            ^token; leftOperand = b. Return b to the caller as a rightOperand
                    //      a - b * c;
                    //            ^token; leftOperand = b. Return b * c to the caller as a rightOperand
                    // - For right associative operator (**), consume the operator, recursively call the function
                    //   and parse binaryExpression as a rightOperand of the caller if the new precedence of
                    //   the operator is strictly grater than the current precedence
                    //   For example:
                    //      a ** b ** c;
                    //             ^^token; leftOperand = b. Return b ** c to the caller as a rightOperand
                    //      a - b ** c;
                    //            ^^token; leftOperand = b. Return b ** c to the caller as a rightOperand
                    //      a ** b - c
                    //             ^token; leftOperand = b. Return b to the caller as a rightOperand
                    var consumeCurrentOperator = token() === 42 /* AsteriskAsteriskToken */ ?
                        newPrecedence >= precedence :
                        newPrecedence > precedence;
                    if (!consumeCurrentOperator) {
                        break;
                    }
                    if (token() === 97 /* InKeyword */ && inDisallowInContext()) {
                        break;
                    }
                    if (token() === 123 /* AsKeyword */) {
                        // Make sure we *do* perform ASI for constructs like this:
                        //    var x = foo
                        //    as (Bar)
                        // This should be parsed as an initialized variable, followed
                        // by a function call to 'as' with the argument 'Bar'
                        if (scanner.hasPrecedingLineBreak()) {
                            break;
                        }
                        else {
                            nextToken();
                            leftOperand = makeAsExpression(leftOperand, parseType());
                        }
                    }
                    else {
                        leftOperand = makeBinaryExpression(leftOperand, parseTokenNode(), parseBinaryExpressionOrHigher(newPrecedence));
                    }
                }
                return leftOperand;
            }
            function isBinaryOperator() {
                if (inDisallowInContext() && token() === 97 /* InKeyword */) {
                    return false;
                }
                return ts.getBinaryOperatorPrecedence(token()) > 0;
            }
            function makeBinaryExpression(left, operatorToken, right) {
                var node = createNode(209 /* BinaryExpression */, left.pos);
                node.left = left;
                node.operatorToken = operatorToken;
                node.right = right;
                return finishNode(node);
            }
            function makeAsExpression(left, right) {
                var node = createNode(217 /* AsExpression */, left.pos);
                node.expression = left;
                node.type = right;
                return finishNode(node);
            }
            function parsePrefixUnaryExpression() {
                var node = createNode(207 /* PrefixUnaryExpression */);
                node.operator = token();
                nextToken();
                node.operand = parseSimpleUnaryExpression();
                return finishNode(node);
            }
            function parseDeleteExpression() {
                var node = createNode(203 /* DeleteExpression */);
                nextToken();
                node.expression = parseSimpleUnaryExpression();
                return finishNode(node);
            }
            function parseTypeOfExpression() {
                var node = createNode(204 /* TypeOfExpression */);
                nextToken();
                node.expression = parseSimpleUnaryExpression();
                return finishNode(node);
            }
            function parseVoidExpression() {
                var node = createNode(205 /* VoidExpression */);
                nextToken();
                node.expression = parseSimpleUnaryExpression();
                return finishNode(node);
            }
            function isAwaitExpression() {
                if (token() === 127 /* AwaitKeyword */) {
                    if (inAwaitContext()) {
                        return true;
                    }
                    // here we are using similar heuristics as 'isYieldExpression'
                    return lookAhead(nextTokenIsIdentifierOrKeywordOrLiteralOnSameLine);
                }
                return false;
            }
            function parseAwaitExpression() {
                var node = createNode(206 /* AwaitExpression */);
                nextToken();
                node.expression = parseSimpleUnaryExpression();
                return finishNode(node);
            }
            /**
             * Parse ES7 exponential expression and await expression
             *
             * ES7 ExponentiationExpression:
             *      1) UnaryExpression[?Yield]
             *      2) UpdateExpression[?Yield] ** ExponentiationExpression[?Yield]
             *
             */
            function parseUnaryExpressionOrHigher() {
                /**
                 * ES7 UpdateExpression:
                 *      1) LeftHandSideExpression[?Yield]
                 *      2) LeftHandSideExpression[?Yield][no LineTerminator here]++
                 *      3) LeftHandSideExpression[?Yield][no LineTerminator here]--
                 *      4) ++UnaryExpression[?Yield]
                 *      5) --UnaryExpression[?Yield]
                 */
                if (isUpdateExpression()) {
                    var updateExpression = parseUpdateExpression();
                    return token() === 42 /* AsteriskAsteriskToken */ ?
                        parseBinaryExpressionRest(ts.getBinaryOperatorPrecedence(token()), updateExpression) :
                        updateExpression;
                }
                /**
                 * ES7 UnaryExpression:
                 *      1) UpdateExpression[?yield]
                 *      2) delete UpdateExpression[?yield]
                 *      3) void UpdateExpression[?yield]
                 *      4) typeof UpdateExpression[?yield]
                 *      5) + UpdateExpression[?yield]
                 *      6) - UpdateExpression[?yield]
                 *      7) ~ UpdateExpression[?yield]
                 *      8) ! UpdateExpression[?yield]
                 */
                var unaryOperator = token();
                var simpleUnaryExpression = parseSimpleUnaryExpression();
                if (token() === 42 /* AsteriskAsteriskToken */) {
                    var pos = ts.skipTrivia(sourceText, simpleUnaryExpression.pos);
                    var end = simpleUnaryExpression.end;
                    if (simpleUnaryExpression.kind === 199 /* TypeAssertionExpression */) {
                        parseErrorAt(pos, end, ts.Diagnostics.A_type_assertion_expression_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses);
                    }
                    else {
                        parseErrorAt(pos, end, ts.Diagnostics.An_unary_expression_with_the_0_operator_is_not_allowed_in_the_left_hand_side_of_an_exponentiation_expression_Consider_enclosing_the_expression_in_parentheses, ts.tokenToString(unaryOperator));
                    }
                }
                return simpleUnaryExpression;
            }
            /**
             * Parse ES7 simple-unary expression or higher:
             *
             * ES7 UnaryExpression:
             *      1) UpdateExpression[?yield]
             *      2) delete UnaryExpression[?yield]
             *      3) void UnaryExpression[?yield]
             *      4) typeof UnaryExpression[?yield]
             *      5) + UnaryExpression[?yield]
             *      6) - UnaryExpression[?yield]
             *      7) ~ UnaryExpression[?yield]
             *      8) ! UnaryExpression[?yield]
             *      9) [+Await] await UnaryExpression[?yield]
             */
            function parseSimpleUnaryExpression() {
                switch (token()) {
                    case 39 /* PlusToken */:
                    case 40 /* MinusToken */:
                    case 54 /* TildeToken */:
                    case 53 /* ExclamationToken */:
                        return parsePrefixUnaryExpression();
                    case 85 /* DeleteKeyword */:
                        return parseDeleteExpression();
                    case 108 /* TypeOfKeyword */:
                        return parseTypeOfExpression();
                    case 110 /* VoidKeyword */:
                        return parseVoidExpression();
                    case 29 /* LessThanToken */:
                        // This is modified UnaryExpression grammar in TypeScript
                        //  UnaryExpression (modified):
                        //      < type > UnaryExpression
                        return parseTypeAssertion();
                    case 127 /* AwaitKeyword */:
                        if (isAwaitExpression()) {
                            return parseAwaitExpression();
                        }
                    // falls through
                    default:
                        return parseUpdateExpression();
                }
            }
            /**
             * Check if the current token can possibly be an ES7 increment expression.
             *
             * ES7 UpdateExpression:
             *      LeftHandSideExpression[?Yield]
             *      LeftHandSideExpression[?Yield][no LineTerminator here]++
             *      LeftHandSideExpression[?Yield][no LineTerminator here]--
             *      ++LeftHandSideExpression[?Yield]
             *      --LeftHandSideExpression[?Yield]
             */
            function isUpdateExpression() {
                // This function is called inside parseUnaryExpression to decide
                // whether to call parseSimpleUnaryExpression or call parseUpdateExpression directly
                switch (token()) {
                    case 39 /* PlusToken */:
                    case 40 /* MinusToken */:
                    case 54 /* TildeToken */:
                    case 53 /* ExclamationToken */:
                    case 85 /* DeleteKeyword */:
                    case 108 /* TypeOfKeyword */:
                    case 110 /* VoidKeyword */:
                    case 127 /* AwaitKeyword */:
                        return false;
                    case 29 /* LessThanToken */:
                        // If we are not in JSX context, we are parsing TypeAssertion which is an UnaryExpression
                        if (sourceFile.languageVariant !== 1 /* JSX */) {
                            return false;
                        }
                    // We are in JSX context and the token is part of JSXElement.
                    // falls through
                    default:
                        return true;
                }
            }
            /**
             * Parse ES7 UpdateExpression. UpdateExpression is used instead of ES6's PostFixExpression.
             *
             * ES7 UpdateExpression[yield]:
             *      1) LeftHandSideExpression[?yield]
             *      2) LeftHandSideExpression[?yield] [[no LineTerminator here]]++
             *      3) LeftHandSideExpression[?yield] [[no LineTerminator here]]--
             *      4) ++LeftHandSideExpression[?yield]
             *      5) --LeftHandSideExpression[?yield]
             * In TypeScript (2), (3) are parsed as PostfixUnaryExpression. (4), (5) are parsed as PrefixUnaryExpression
             */
            function parseUpdateExpression() {
                if (token() === 45 /* PlusPlusToken */ || token() === 46 /* MinusMinusToken */) {
                    var node = createNode(207 /* PrefixUnaryExpression */);
                    node.operator = token();
                    nextToken();
                    node.operand = parseLeftHandSideExpressionOrHigher();
                    return finishNode(node);
                }
                else if (sourceFile.languageVariant === 1 /* JSX */ && token() === 29 /* LessThanToken */ && lookAhead(nextTokenIsIdentifierOrKeywordOrGreaterThan)) {
                    // JSXElement is part of primaryExpression
                    return parseJsxElementOrSelfClosingElementOrFragment(/*inExpressionContext*/ true);
                }
                var expression = parseLeftHandSideExpressionOrHigher();
                ts.Debug.assert(ts.isLeftHandSideExpression(expression));
                if ((token() === 45 /* PlusPlusToken */ || token() === 46 /* MinusMinusToken */) && !scanner.hasPrecedingLineBreak()) {
                    var node = createNode(208 /* PostfixUnaryExpression */, expression.pos);
                    node.operand = expression;
                    node.operator = token();
                    nextToken();
                    return finishNode(node);
                }
                return expression;
            }
            function parseLeftHandSideExpressionOrHigher() {
                // Original Ecma:
                // LeftHandSideExpression: See 11.2
                //      NewExpression
                //      CallExpression
                //
                // Our simplification:
                //
                // LeftHandSideExpression: See 11.2
                //      MemberExpression
                //      CallExpression
                //
                // See comment in parseMemberExpressionOrHigher on how we replaced NewExpression with
                // MemberExpression to make our lives easier.
                //
                // to best understand the below code, it's important to see how CallExpression expands
                // out into its own productions:
                //
                // CallExpression:
                //      MemberExpression Arguments
                //      CallExpression Arguments
                //      CallExpression[Expression]
                //      CallExpression.IdentifierName
                //      import (AssignmentExpression)
                //      super Arguments
                //      super.IdentifierName
                //
                // Because of the recursion in these calls, we need to bottom out first. There are three
                // bottom out states we can run into: 1) We see 'super' which must start either of
                // the last two CallExpression productions. 2) We see 'import' which must start import call.
                // 3)we have a MemberExpression which either completes the LeftHandSideExpression,
                // or starts the beginning of the first four CallExpression productions.
                var expression;
                if (token() === 96 /* ImportKeyword */) {
                    if (lookAhead(nextTokenIsOpenParenOrLessThan)) {
                        // We don't want to eagerly consume all import keyword as import call expression so we look ahead to find "("
                        // For example:
                        //      var foo3 = require("subfolder
                        //      import * as foo1 from "module-from-node
                        // We want this import to be a statement rather than import call expression
                        sourceFile.flags |= 1048576 /* PossiblyContainsDynamicImport */;
                        expression = parseTokenNode();
                    }
                    else if (lookAhead(nextTokenIsDot)) {
                        // This is an 'import.*' metaproperty (i.e. 'import.meta')
                        var fullStart = scanner.getStartPos();
                        nextToken(); // advance past the 'import'
                        nextToken(); // advance past the dot
                        var node = createNode(219 /* MetaProperty */, fullStart);
                        node.keywordToken = 96 /* ImportKeyword */;
                        node.name = parseIdentifierName();
                        expression = finishNode(node);
                        sourceFile.flags |= 2097152 /* PossiblyContainsImportMeta */;
                    }
                    else {
                        expression = parseMemberExpressionOrHigher();
                    }
                }
                else {
                    expression = token() === 102 /* SuperKeyword */ ? parseSuperExpression() : parseMemberExpressionOrHigher();
                }
                // Now, we *may* be complete.  However, we might have consumed the start of a
                // CallExpression or OptionalExpression.  As such, we need to consume the rest
                // of it here to be complete.
                return parseCallExpressionRest(expression);
            }
            function parseMemberExpressionOrHigher() {
                // Note: to make our lives simpler, we decompose the NewExpression productions and
                // place ObjectCreationExpression and FunctionExpression into PrimaryExpression.
                // like so:
                //
                //   PrimaryExpression : See 11.1
                //      this
                //      Identifier
                //      Literal
                //      ArrayLiteral
                //      ObjectLiteral
                //      (Expression)
                //      FunctionExpression
                //      new MemberExpression Arguments?
                //
                //   MemberExpression : See 11.2
                //      PrimaryExpression
                //      MemberExpression[Expression]
                //      MemberExpression.IdentifierName
                //
                //   CallExpression : See 11.2
                //      MemberExpression
                //      CallExpression Arguments
                //      CallExpression[Expression]
                //      CallExpression.IdentifierName
                //
                // Technically this is ambiguous.  i.e. CallExpression defines:
                //
                //   CallExpression:
                //      CallExpression Arguments
                //
                // If you see: "new Foo()"
                //
                // Then that could be treated as a single ObjectCreationExpression, or it could be
                // treated as the invocation of "new Foo".  We disambiguate that in code (to match
                // the original grammar) by making sure that if we see an ObjectCreationExpression
                // we always consume arguments if they are there. So we treat "new Foo()" as an
                // object creation only, and not at all as an invocation.  Another way to think
                // about this is that for every "new" that we see, we will consume an argument list if
                // it is there as part of the *associated* object creation node.  Any additional
                // argument lists we see, will become invocation expressions.
                //
                // Because there are no other places in the grammar now that refer to FunctionExpression
                // or ObjectCreationExpression, it is safe to push down into the PrimaryExpression
                // production.
                //
                // Because CallExpression and MemberExpression are left recursive, we need to bottom out
                // of the recursion immediately.  So we parse out a primary expression to start with.
                var expression = parsePrimaryExpression();
                return parseMemberExpressionRest(expression, /*allowOptionalChain*/ true);
            }
            function parseSuperExpression() {
                var expression = parseTokenNode();
                if (token() === 29 /* LessThanToken */) {
                    var startPos = getNodePos();
                    var typeArguments = tryParse(parseTypeArgumentsInExpression);
                    if (typeArguments !== undefined) {
                        parseErrorAt(startPos, getNodePos(), ts.Diagnostics.super_may_not_use_type_arguments);
                    }
                }
                if (token() === 20 /* OpenParenToken */ || token() === 24 /* DotToken */ || token() === 22 /* OpenBracketToken */) {
                    return expression;
                }
                // If we have seen "super" it must be followed by '(' or '.'.
                // If it wasn't then just try to parse out a '.' and report an error.
                var node = createNode(194 /* PropertyAccessExpression */, expression.pos);
                node.expression = expression;
                parseExpectedToken(24 /* DotToken */, ts.Diagnostics.super_must_be_followed_by_an_argument_list_or_member_access);
                // private names will never work with `super` (`super.#foo`), but that's a semantic error, not syntactic
                node.name = parseRightSideOfDot(/*allowIdentifierNames*/ true, /*allowPrivateIdentifiers*/ true);
                return finishNode(node);
            }
            function parseJsxElementOrSelfClosingElementOrFragment(inExpressionContext) {
                var opening = parseJsxOpeningOrSelfClosingElementOrOpeningFragment(inExpressionContext);
                var result;
                if (opening.kind === 268 /* JsxOpeningElement */) {
                    var node = createNode(266 /* JsxElement */, opening.pos);
                    node.openingElement = opening;
                    node.children = parseJsxChildren(node.openingElement);
                    node.closingElement = parseJsxClosingElement(inExpressionContext);
                    if (!tagNamesAreEquivalent(node.openingElement.tagName, node.closingElement.tagName)) {
                        parseErrorAtRange(node.closingElement, ts.Diagnostics.Expected_corresponding_JSX_closing_tag_for_0, ts.getTextOfNodeFromSourceText(sourceText, node.openingElement.tagName));
                    }
                    result = finishNode(node);
                }
                else if (opening.kind === 271 /* JsxOpeningFragment */) {
                    var node = createNode(270 /* JsxFragment */, opening.pos);
                    node.openingFragment = opening;
                    node.children = parseJsxChildren(node.openingFragment);
                    node.closingFragment = parseJsxClosingFragment(inExpressionContext);
                    result = finishNode(node);
                }
                else {
                    ts.Debug.assert(opening.kind === 267 /* JsxSelfClosingElement */);
                    // Nothing else to do for self-closing elements
                    result = opening;
                }
                // If the user writes the invalid code '<div></div><div></div>' in an expression context (i.e. not wrapped in
                // an enclosing tag), we'll naively try to parse   ^ this as a 'less than' operator and the remainder of the tag
                // as garbage, which will cause the formatter to badly mangle the JSX. Perform a speculative parse of a JSX
                // element if we see a < token so that we can wrap it in a synthetic binary expression so the formatter
                // does less damage and we can report a better error.
                // Since JSX elements are invalid < operands anyway, this lookahead parse will only occur in error scenarios
                // of one sort or another.
                if (inExpressionContext && token() === 29 /* LessThanToken */) {
                    var invalidElement = tryParse(function () { return parseJsxElementOrSelfClosingElementOrFragment(/*inExpressionContext*/ true); });
                    if (invalidElement) {
                        parseErrorAtCurrentToken(ts.Diagnostics.JSX_expressions_must_have_one_parent_element);
                        var badNode = createNode(209 /* BinaryExpression */, result.pos);
                        badNode.end = invalidElement.end;
                        badNode.left = result;
                        badNode.right = invalidElement;
                        badNode.operatorToken = createMissingNode(27 /* CommaToken */, /*reportAtCurrentPosition*/ false);
                        badNode.operatorToken.pos = badNode.operatorToken.end = badNode.right.pos;
                        return badNode;
                    }
                }
                return result;
            }
            function parseJsxText() {
                var node = createNode(11 /* JsxText */);
                node.text = scanner.getTokenValue();
                node.containsOnlyTriviaWhiteSpaces = currentToken === 12 /* JsxTextAllWhiteSpaces */;
                currentToken = scanner.scanJsxToken();
                return finishNode(node);
            }
            function parseJsxChild(openingTag, token) {
                switch (token) {
                    case 1 /* EndOfFileToken */:
                        // If we hit EOF, issue the error at the tag that lacks the closing element
                        // rather than at the end of the file (which is useless)
                        if (ts.isJsxOpeningFragment(openingTag)) {
                            parseErrorAtRange(openingTag, ts.Diagnostics.JSX_fragment_has_no_corresponding_closing_tag);
                        }
                        else {
                            parseErrorAtRange(openingTag.tagName, ts.Diagnostics.JSX_element_0_has_no_corresponding_closing_tag, ts.getTextOfNodeFromSourceText(sourceText, openingTag.tagName));
                        }
                        return undefined;
                    case 30 /* LessThanSlashToken */:
                    case 7 /* ConflictMarkerTrivia */:
                        return undefined;
                    case 11 /* JsxText */:
                    case 12 /* JsxTextAllWhiteSpaces */:
                        return parseJsxText();
                    case 18 /* OpenBraceToken */:
                        return parseJsxExpression(/*inExpressionContext*/ false);
                    case 29 /* LessThanToken */:
                        return parseJsxElementOrSelfClosingElementOrFragment(/*inExpressionContext*/ false);
                    default:
                        return ts.Debug.assertNever(token);
                }
            }
            function parseJsxChildren(openingTag) {
                var list = [];
                var listPos = getNodePos();
                var saveParsingContext = parsingContext;
                parsingContext |= 1 << 14 /* JsxChildren */;
                while (true) {
                    var child = parseJsxChild(openingTag, currentToken = scanner.reScanJsxToken());
                    if (!child)
                        break;
                    list.push(child);
                }
                parsingContext = saveParsingContext;
                return createNodeArray(list, listPos);
            }
            function parseJsxAttributes() {
                var jsxAttributes = createNode(274 /* JsxAttributes */);
                jsxAttributes.properties = parseList(13 /* JsxAttributes */, parseJsxAttribute);
                return finishNode(jsxAttributes);
            }
            function parseJsxOpeningOrSelfClosingElementOrOpeningFragment(inExpressionContext) {
                var fullStart = scanner.getStartPos();
                parseExpected(29 /* LessThanToken */);
                if (token() === 31 /* GreaterThanToken */) {
                    // See below for explanation of scanJsxText
                    var node_1 = createNode(271 /* JsxOpeningFragment */, fullStart);
                    scanJsxText();
                    return finishNode(node_1);
                }
                var tagName = parseJsxElementName();
                var typeArguments = tryParseTypeArguments();
                var attributes = parseJsxAttributes();
                var node;
                if (token() === 31 /* GreaterThanToken */) {
                    // Closing tag, so scan the immediately-following text with the JSX scanning instead
                    // of regular scanning to avoid treating illegal characters (e.g. '#') as immediate
                    // scanning errors
                    node = createNode(268 /* JsxOpeningElement */, fullStart);
                    scanJsxText();
                }
                else {
                    parseExpected(43 /* SlashToken */);
                    if (inExpressionContext) {
                        parseExpected(31 /* GreaterThanToken */);
                    }
                    else {
                        parseExpected(31 /* GreaterThanToken */, /*diagnostic*/ undefined, /*shouldAdvance*/ false);
                        scanJsxText();
                    }
                    node = createNode(267 /* JsxSelfClosingElement */, fullStart);
                }
                node.tagName = tagName;
                node.typeArguments = typeArguments;
                node.attributes = attributes;
                return finishNode(node);
            }
            function parseJsxElementName() {
                scanJsxIdentifier();
                // JsxElement can have name in the form of
                //      propertyAccessExpression
                //      primaryExpression in the form of an identifier and "this" keyword
                // We can't just simply use parseLeftHandSideExpressionOrHigher because then we will start consider class,function etc as a keyword
                // We only want to consider "this" as a primaryExpression
                var expression = token() === 104 /* ThisKeyword */ ?
                    parseTokenNode() : parseIdentifierName();
                while (parseOptional(24 /* DotToken */)) {
                    var propertyAccess = createNode(194 /* PropertyAccessExpression */, expression.pos);
                    propertyAccess.expression = expression;
                    propertyAccess.name = parseRightSideOfDot(/*allowIdentifierNames*/ true, /*allowPrivateIdentifiers*/ false);
                    expression = finishNode(propertyAccess);
                }
                return expression;
            }
            function parseJsxExpression(inExpressionContext) {
                var node = createNode(276 /* JsxExpression */);
                if (!parseExpected(18 /* OpenBraceToken */)) {
                    return undefined;
                }
                if (token() !== 19 /* CloseBraceToken */) {
                    node.dotDotDotToken = parseOptionalToken(25 /* DotDotDotToken */);
                    // Only an AssignmentExpression is valid here per the JSX spec,
                    // but we can unambiguously parse a comma sequence and provide
                    // a better error message in grammar checking.
                    node.expression = parseExpression();
                }
                if (inExpressionContext) {
                    parseExpected(19 /* CloseBraceToken */);
                }
                else {
                    if (parseExpected(19 /* CloseBraceToken */, /*message*/ undefined, /*shouldAdvance*/ false)) {
                        scanJsxText();
                    }
                }
                return finishNode(node);
            }
            function parseJsxAttribute() {
                if (token() === 18 /* OpenBraceToken */) {
                    return parseJsxSpreadAttribute();
                }
                scanJsxIdentifier();
                var node = createNode(273 /* JsxAttribute */);
                node.name = parseIdentifierName();
                if (token() === 62 /* EqualsToken */) {
                    switch (scanJsxAttributeValue()) {
                        case 10 /* StringLiteral */:
                            node.initializer = parseLiteralNode();
                            break;
                        default:
                            node.initializer = parseJsxExpression(/*inExpressionContext*/ true);
                            break;
                    }
                }
                return finishNode(node);
            }
            function parseJsxSpreadAttribute() {
                var node = createNode(275 /* JsxSpreadAttribute */);
                parseExpected(18 /* OpenBraceToken */);
                parseExpected(25 /* DotDotDotToken */);
                node.expression = parseExpression();
                parseExpected(19 /* CloseBraceToken */);
                return finishNode(node);
            }
            function parseJsxClosingElement(inExpressionContext) {
                var node = createNode(269 /* JsxClosingElement */);
                parseExpected(30 /* LessThanSlashToken */);
                node.tagName = parseJsxElementName();
                if (inExpressionContext) {
                    parseExpected(31 /* GreaterThanToken */);
                }
                else {
                    parseExpected(31 /* GreaterThanToken */, /*diagnostic*/ undefined, /*shouldAdvance*/ false);
                    scanJsxText();
                }
                return finishNode(node);
            }
            function parseJsxClosingFragment(inExpressionContext) {
                var node = createNode(272 /* JsxClosingFragment */);
                parseExpected(30 /* LessThanSlashToken */);
                if (ts.tokenIsIdentifierOrKeyword(token())) {
                    parseErrorAtRange(parseJsxElementName(), ts.Diagnostics.Expected_corresponding_closing_tag_for_JSX_fragment);
                }
                if (inExpressionContext) {
                    parseExpected(31 /* GreaterThanToken */);
                }
                else {
                    parseExpected(31 /* GreaterThanToken */, /*diagnostic*/ undefined, /*shouldAdvance*/ false);
                    scanJsxText();
                }
                return finishNode(node);
            }
            function parseTypeAssertion() {
                var node = createNode(199 /* TypeAssertionExpression */);
                parseExpected(29 /* LessThanToken */);
                node.type = parseType();
                parseExpected(31 /* GreaterThanToken */);
                node.expression = parseSimpleUnaryExpression();
                return finishNode(node);
            }
            function nextTokenIsIdentifierOrKeywordOrOpenBracketOrTemplate() {
                nextToken();
                return ts.tokenIsIdentifierOrKeyword(token())
                    || token() === 22 /* OpenBracketToken */
                    || isTemplateStartOfTaggedTemplate();
            }
            function isStartOfOptionalPropertyOrElementAccessChain() {
                return token() === 28 /* QuestionDotToken */
                    && lookAhead(nextTokenIsIdentifierOrKeywordOrOpenBracketOrTemplate);
            }
            function parsePropertyAccessExpressionRest(expression, questionDotToken) {
                var propertyAccess = createNode(194 /* PropertyAccessExpression */, expression.pos);
                propertyAccess.expression = expression;
                propertyAccess.questionDotToken = questionDotToken;
                propertyAccess.name = parseRightSideOfDot(/*allowIdentifierNames*/ true, /*allowPrivateIdentifiers*/ true);
                if (questionDotToken || expression.flags & 32 /* OptionalChain */) {
                    propertyAccess.flags |= 32 /* OptionalChain */;
                    if (ts.isPrivateIdentifier(propertyAccess.name)) {
                        parseErrorAtRange(propertyAccess.name, ts.Diagnostics.An_optional_chain_cannot_contain_private_identifiers);
                    }
                }
                return finishNode(propertyAccess);
            }
            function parseElementAccessExpressionRest(expression, questionDotToken) {
                var indexedAccess = createNode(195 /* ElementAccessExpression */, expression.pos);
                indexedAccess.expression = expression;
                indexedAccess.questionDotToken = questionDotToken;
                if (token() === 23 /* CloseBracketToken */) {
                    indexedAccess.argumentExpression = createMissingNode(75 /* Identifier */, /*reportAtCurrentPosition*/ true, ts.Diagnostics.An_element_access_expression_should_take_an_argument);
                }
                else {
                    var argument = allowInAnd(parseExpression);
                    if (ts.isStringOrNumericLiteralLike(argument)) {
                        argument.text = internIdentifier(argument.text);
                    }
                    indexedAccess.argumentExpression = argument;
                }
                parseExpected(23 /* CloseBracketToken */);
                if (questionDotToken || expression.flags & 32 /* OptionalChain */) {
                    indexedAccess.flags |= 32 /* OptionalChain */;
                }
                return finishNode(indexedAccess);
            }
            function parseMemberExpressionRest(expression, allowOptionalChain) {
                while (true) {
                    var questionDotToken = void 0;
                    var isPropertyAccess = false;
                    if (allowOptionalChain && isStartOfOptionalPropertyOrElementAccessChain()) {
                        questionDotToken = parseExpectedToken(28 /* QuestionDotToken */);
                        isPropertyAccess = ts.tokenIsIdentifierOrKeyword(token());
                    }
                    else {
                        isPropertyAccess = parseOptional(24 /* DotToken */);
                    }
                    if (isPropertyAccess) {
                        expression = parsePropertyAccessExpressionRest(expression, questionDotToken);
                        continue;
                    }
                    if (!questionDotToken && token() === 53 /* ExclamationToken */ && !scanner.hasPrecedingLineBreak()) {
                        nextToken();
                        var nonNullExpression = createNode(218 /* NonNullExpression */, expression.pos);
                        nonNullExpression.expression = expression;
                        expression = finishNode(nonNullExpression);
                        continue;
                    }
                    // when in the [Decorator] context, we do not parse ElementAccess as it could be part of a ComputedPropertyName
                    if ((questionDotToken || !inDecoratorContext()) && parseOptional(22 /* OpenBracketToken */)) {
                        expression = parseElementAccessExpressionRest(expression, questionDotToken);
                        continue;
                    }
                    if (isTemplateStartOfTaggedTemplate()) {
                        expression = parseTaggedTemplateRest(expression, questionDotToken, /*typeArguments*/ undefined);
                        continue;
                    }
                    return expression;
                }
            }
            function isTemplateStartOfTaggedTemplate() {
                return token() === 14 /* NoSubstitutionTemplateLiteral */ || token() === 15 /* TemplateHead */;
            }
            function parseTaggedTemplateRest(tag, questionDotToken, typeArguments) {
                var tagExpression = createNode(198 /* TaggedTemplateExpression */, tag.pos);
                tagExpression.tag = tag;
                tagExpression.questionDotToken = questionDotToken;
                tagExpression.typeArguments = typeArguments;
                tagExpression.template = token() === 14 /* NoSubstitutionTemplateLiteral */
                    ? parseLiteralNode()
                    : parseTemplateExpression();
                if (questionDotToken || tag.flags & 32 /* OptionalChain */) {
                    tagExpression.flags |= 32 /* OptionalChain */;
                }
                return finishNode(tagExpression);
            }
            function parseCallExpressionRest(expression) {
                while (true) {
                    expression = parseMemberExpressionRest(expression, /*allowOptionalChain*/ true);
                    var questionDotToken = parseOptionalToken(28 /* QuestionDotToken */);
                    // handle 'foo<<T>()'
                    if (token() === 29 /* LessThanToken */ || token() === 47 /* LessThanLessThanToken */) {
                        // See if this is the start of a generic invocation.  If so, consume it and
                        // keep checking for postfix expressions.  Otherwise, it's just a '<' that's
                        // part of an arithmetic expression.  Break out so we consume it higher in the
                        // stack.
                        var typeArguments = tryParse(parseTypeArgumentsInExpression);
                        if (typeArguments) {
                            if (isTemplateStartOfTaggedTemplate()) {
                                expression = parseTaggedTemplateRest(expression, questionDotToken, typeArguments);
                                continue;
                            }
                            var callExpr = createNode(196 /* CallExpression */, expression.pos);
                            callExpr.expression = expression;
                            callExpr.questionDotToken = questionDotToken;
                            callExpr.typeArguments = typeArguments;
                            callExpr.arguments = parseArgumentList();
                            if (questionDotToken || expression.flags & 32 /* OptionalChain */) {
                                callExpr.flags |= 32 /* OptionalChain */;
                            }
                            expression = finishNode(callExpr);
                            continue;
                        }
                    }
                    else if (token() === 20 /* OpenParenToken */) {
                        var callExpr = createNode(196 /* CallExpression */, expression.pos);
                        callExpr.expression = expression;
                        callExpr.questionDotToken = questionDotToken;
                        callExpr.arguments = parseArgumentList();
                        if (questionDotToken || expression.flags & 32 /* OptionalChain */) {
                            callExpr.flags |= 32 /* OptionalChain */;
                        }
                        expression = finishNode(callExpr);
                        continue;
                    }
                    if (questionDotToken) {
                        // We failed to parse anything, so report a missing identifier here.
                        var propertyAccess = createNode(194 /* PropertyAccessExpression */, expression.pos);
                        propertyAccess.expression = expression;
                        propertyAccess.questionDotToken = questionDotToken;
                        propertyAccess.name = createMissingNode(75 /* Identifier */, /*reportAtCurrentPosition*/ false, ts.Diagnostics.Identifier_expected);
                        propertyAccess.flags |= 32 /* OptionalChain */;
                        expression = finishNode(propertyAccess);
                    }
                    break;
                }
                return expression;
            }
            function parseArgumentList() {
                parseExpected(20 /* OpenParenToken */);
                var result = parseDelimitedList(11 /* ArgumentExpressions */, parseArgumentExpression);
                parseExpected(21 /* CloseParenToken */);
                return result;
            }
            function parseTypeArgumentsInExpression() {
                if (reScanLessThanToken() !== 29 /* LessThanToken */) {
                    return undefined;
                }
                nextToken();
                var typeArguments = parseDelimitedList(20 /* TypeArguments */, parseType);
                if (!parseExpected(31 /* GreaterThanToken */)) {
                    // If it doesn't have the closing `>` then it's definitely not an type argument list.
                    return undefined;
                }
                // If we have a '<', then only parse this as a argument list if the type arguments
                // are complete and we have an open paren.  if we don't, rewind and return nothing.
                return typeArguments && canFollowTypeArgumentsInExpression()
                    ? typeArguments
                    : undefined;
            }
            function canFollowTypeArgumentsInExpression() {
                switch (token()) {
                    case 20 /* OpenParenToken */: // foo<x>(
                    case 14 /* NoSubstitutionTemplateLiteral */: // foo<T> `...`
                    case 15 /* TemplateHead */: // foo<T> `...${100}...`
                    // these are the only tokens can legally follow a type argument
                    // list. So we definitely want to treat them as type arg lists.
                    // falls through
                    case 24 /* DotToken */: // foo<x>.
                    case 21 /* CloseParenToken */: // foo<x>)
                    case 23 /* CloseBracketToken */: // foo<x>]
                    case 58 /* ColonToken */: // foo<x>:
                    case 26 /* SemicolonToken */: // foo<x>;
                    case 57 /* QuestionToken */: // foo<x>?
                    case 34 /* EqualsEqualsToken */: // foo<x> ==
                    case 36 /* EqualsEqualsEqualsToken */: // foo<x> ===
                    case 35 /* ExclamationEqualsToken */: // foo<x> !=
                    case 37 /* ExclamationEqualsEqualsToken */: // foo<x> !==
                    case 55 /* AmpersandAmpersandToken */: // foo<x> &&
                    case 56 /* BarBarToken */: // foo<x> ||
                    case 60 /* QuestionQuestionToken */: // foo<x> ??
                    case 52 /* CaretToken */: // foo<x> ^
                    case 50 /* AmpersandToken */: // foo<x> &
                    case 51 /* BarToken */: // foo<x> |
                    case 19 /* CloseBraceToken */: // foo<x> }
                    case 1 /* EndOfFileToken */: // foo<x>
                        // these cases can't legally follow a type arg list.  However, they're not legal
                        // expressions either.  The user is probably in the middle of a generic type. So
                        // treat it as such.
                        return true;
                    case 27 /* CommaToken */: // foo<x>,
                    case 18 /* OpenBraceToken */: // foo<x> {
                    // We don't want to treat these as type arguments.  Otherwise we'll parse this
                    // as an invocation expression.  Instead, we want to parse out the expression
                    // in isolation from the type arguments.
                    // falls through
                    default:
                        // Anything else treat as an expression.
                        return false;
                }
            }
            function parsePrimaryExpression() {
                switch (token()) {
                    case 8 /* NumericLiteral */:
                    case 9 /* BigIntLiteral */:
                    case 10 /* StringLiteral */:
                    case 14 /* NoSubstitutionTemplateLiteral */:
                        return parseLiteralNode();
                    case 104 /* ThisKeyword */:
                    case 102 /* SuperKeyword */:
                    case 100 /* NullKeyword */:
                    case 106 /* TrueKeyword */:
                    case 91 /* FalseKeyword */:
                        return parseTokenNode();
                    case 20 /* OpenParenToken */:
                        return parseParenthesizedExpression();
                    case 22 /* OpenBracketToken */:
                        return parseArrayLiteralExpression();
                    case 18 /* OpenBraceToken */:
                        return parseObjectLiteralExpression();
                    case 126 /* AsyncKeyword */:
                        // Async arrow functions are parsed earlier in parseAssignmentExpressionOrHigher.
                        // If we encounter `async [no LineTerminator here] function` then this is an async
                        // function; otherwise, its an identifier.
                        if (!lookAhead(nextTokenIsFunctionKeywordOnSameLine)) {
                            break;
                        }
                        return parseFunctionExpression();
                    case 80 /* ClassKeyword */:
                        return parseClassExpression();
                    case 94 /* FunctionKeyword */:
                        return parseFunctionExpression();
                    case 99 /* NewKeyword */:
                        return parseNewExpressionOrNewDotTarget();
                    case 43 /* SlashToken */:
                    case 67 /* SlashEqualsToken */:
                        if (reScanSlashToken() === 13 /* RegularExpressionLiteral */) {
                            return parseLiteralNode();
                        }
                        break;
                    case 15 /* TemplateHead */:
                        return parseTemplateExpression();
                }
                return parseIdentifier(ts.Diagnostics.Expression_expected);
            }
            function parseParenthesizedExpression() {
                var node = createNodeWithJSDoc(200 /* ParenthesizedExpression */);
                parseExpected(20 /* OpenParenToken */);
                node.expression = allowInAnd(parseExpression);
                parseExpected(21 /* CloseParenToken */);
                return finishNode(node);
            }
            function parseSpreadElement() {
                var node = createNode(213 /* SpreadElement */);
                parseExpected(25 /* DotDotDotToken */);
                node.expression = parseAssignmentExpressionOrHigher();
                return finishNode(node);
            }
            function parseArgumentOrArrayLiteralElement() {
                return token() === 25 /* DotDotDotToken */ ? parseSpreadElement() :
                    token() === 27 /* CommaToken */ ? createNode(215 /* OmittedExpression */) :
                        parseAssignmentExpressionOrHigher();
            }
            function parseArgumentExpression() {
                return doOutsideOfContext(disallowInAndDecoratorContext, parseArgumentOrArrayLiteralElement);
            }
            function parseArrayLiteralExpression() {
                var node = createNode(192 /* ArrayLiteralExpression */);
                parseExpected(22 /* OpenBracketToken */);
                if (scanner.hasPrecedingLineBreak()) {
                    node.multiLine = true;
                }
                node.elements = parseDelimitedList(15 /* ArrayLiteralMembers */, parseArgumentOrArrayLiteralElement);
                parseExpected(23 /* CloseBracketToken */);
                return finishNode(node);
            }
            function parseObjectLiteralElement() {
                var node = createNodeWithJSDoc(0 /* Unknown */);
                if (parseOptionalToken(25 /* DotDotDotToken */)) {
                    node.kind = 283 /* SpreadAssignment */;
                    node.expression = parseAssignmentExpressionOrHigher();
                    return finishNode(node);
                }
                node.decorators = parseDecorators();
                node.modifiers = parseModifiers();
                if (parseContextualModifier(131 /* GetKeyword */)) {
                    return parseAccessorDeclaration(node, 163 /* GetAccessor */);
                }
                if (parseContextualModifier(142 /* SetKeyword */)) {
                    return parseAccessorDeclaration(node, 164 /* SetAccessor */);
                }
                var asteriskToken = parseOptionalToken(41 /* AsteriskToken */);
                var tokenIsIdentifier = isIdentifier();
                node.name = parsePropertyName();
                // Disallowing of optional property assignments and definite assignment assertion happens in the grammar checker.
                node.questionToken = parseOptionalToken(57 /* QuestionToken */);
                node.exclamationToken = parseOptionalToken(53 /* ExclamationToken */);
                if (asteriskToken || token() === 20 /* OpenParenToken */ || token() === 29 /* LessThanToken */) {
                    return parseMethodDeclaration(node, asteriskToken);
                }
                // check if it is short-hand property assignment or normal property assignment
                // NOTE: if token is EqualsToken it is interpreted as CoverInitializedName production
                // CoverInitializedName[Yield] :
                //     IdentifierReference[?Yield] Initializer[In, ?Yield]
                // this is necessary because ObjectLiteral productions are also used to cover grammar for ObjectAssignmentPattern
                var isShorthandPropertyAssignment = tokenIsIdentifier && (token() !== 58 /* ColonToken */);
                if (isShorthandPropertyAssignment) {
                    node.kind = 282 /* ShorthandPropertyAssignment */;
                    var equalsToken = parseOptionalToken(62 /* EqualsToken */);
                    if (equalsToken) {
                        node.equalsToken = equalsToken;
                        node.objectAssignmentInitializer = allowInAnd(parseAssignmentExpressionOrHigher);
                    }
                }
                else {
                    node.kind = 281 /* PropertyAssignment */;
                    parseExpected(58 /* ColonToken */);
                    node.initializer = allowInAnd(parseAssignmentExpressionOrHigher);
                }
                return finishNode(node);
            }
            function parseObjectLiteralExpression() {
                var node = createNode(193 /* ObjectLiteralExpression */);
                parseExpected(18 /* OpenBraceToken */);
                if (scanner.hasPrecedingLineBreak()) {
                    node.multiLine = true;
                }
                node.properties = parseDelimitedList(12 /* ObjectLiteralMembers */, parseObjectLiteralElement, /*considerSemicolonAsDelimiter*/ true);
                parseExpected(19 /* CloseBraceToken */);
                return finishNode(node);
            }
            function parseFunctionExpression() {
                // GeneratorExpression:
                //      function* BindingIdentifier [Yield][opt](FormalParameters[Yield]){ GeneratorBody }
                //
                // FunctionExpression:
                //      function BindingIdentifier[opt](FormalParameters){ FunctionBody }
                var saveDecoratorContext = inDecoratorContext();
                if (saveDecoratorContext) {
                    setDecoratorContext(/*val*/ false);
                }
                var node = createNodeWithJSDoc(201 /* FunctionExpression */);
                node.modifiers = parseModifiers();
                parseExpected(94 /* FunctionKeyword */);
                node.asteriskToken = parseOptionalToken(41 /* AsteriskToken */);
                var isGenerator = node.asteriskToken ? 1 /* Yield */ : 0 /* None */;
                var isAsync = hasModifierOfKind(node, 126 /* AsyncKeyword */) ? 2 /* Await */ : 0 /* None */;
                node.name =
                    isGenerator && isAsync ? doInYieldAndAwaitContext(parseOptionalIdentifier) :
                        isGenerator ? doInYieldContext(parseOptionalIdentifier) :
                            isAsync ? doInAwaitContext(parseOptionalIdentifier) :
                                parseOptionalIdentifier();
                fillSignature(58 /* ColonToken */, isGenerator | isAsync, node);
                node.body = parseFunctionBlock(isGenerator | isAsync);
                if (saveDecoratorContext) {
                    setDecoratorContext(/*val*/ true);
                }
                return finishNode(node);
            }
            function parseOptionalIdentifier() {
                return isIdentifier() ? parseIdentifier() : undefined;
            }
            function parseNewExpressionOrNewDotTarget() {
                var fullStart = scanner.getStartPos();
                parseExpected(99 /* NewKeyword */);
                if (parseOptional(24 /* DotToken */)) {
                    var node_2 = createNode(219 /* MetaProperty */, fullStart);
                    node_2.keywordToken = 99 /* NewKeyword */;
                    node_2.name = parseIdentifierName();
                    return finishNode(node_2);
                }
                var expression = parsePrimaryExpression();
                var typeArguments;
                while (true) {
                    expression = parseMemberExpressionRest(expression, /*allowOptionalChain*/ false);
                    typeArguments = tryParse(parseTypeArgumentsInExpression);
                    if (isTemplateStartOfTaggedTemplate()) {
                        ts.Debug.assert(!!typeArguments, "Expected a type argument list; all plain tagged template starts should be consumed in 'parseMemberExpressionRest'");
                        expression = parseTaggedTemplateRest(expression, /*optionalChain*/ undefined, typeArguments);
                        typeArguments = undefined;
                    }
                    break;
                }
                var node = createNode(197 /* NewExpression */, fullStart);
                node.expression = expression;
                node.typeArguments = typeArguments;
                if (node.typeArguments || token() === 20 /* OpenParenToken */) {
                    node.arguments = parseArgumentList();
                }
                return finishNode(node);
            }
            // STATEMENTS
            function parseBlock(ignoreMissingOpenBrace, diagnosticMessage) {
                var node = createNode(223 /* Block */);
                var openBracePosition = scanner.getTokenPos();
                if (parseExpected(18 /* OpenBraceToken */, diagnosticMessage) || ignoreMissingOpenBrace) {
                    if (scanner.hasPrecedingLineBreak()) {
                        node.multiLine = true;
                    }
                    node.statements = parseList(1 /* BlockStatements */, parseStatement);
                    if (!parseExpected(19 /* CloseBraceToken */)) {
                        var lastError = ts.lastOrUndefined(parseDiagnostics);
                        if (lastError && lastError.code === ts.Diagnostics._0_expected.code) {
                            ts.addRelatedInfo(lastError, ts.createFileDiagnostic(sourceFile, openBracePosition, 1, ts.Diagnostics.The_parser_expected_to_find_a_to_match_the_token_here));
                        }
                    }
                }
                else {
                    node.statements = createMissingList();
                }
                return finishNode(node);
            }
            function parseFunctionBlock(flags, diagnosticMessage) {
                var savedYieldContext = inYieldContext();
                setYieldContext(!!(flags & 1 /* Yield */));
                var savedAwaitContext = inAwaitContext();
                setAwaitContext(!!(flags & 2 /* Await */));
                // We may be in a [Decorator] context when parsing a function expression or
                // arrow function. The body of the function is not in [Decorator] context.
                var saveDecoratorContext = inDecoratorContext();
                if (saveDecoratorContext) {
                    setDecoratorContext(/*val*/ false);
                }
                var block = parseBlock(!!(flags & 16 /* IgnoreMissingOpenBrace */), diagnosticMessage);
                if (saveDecoratorContext) {
                    setDecoratorContext(/*val*/ true);
                }
                setYieldContext(savedYieldContext);
                setAwaitContext(savedAwaitContext);
                return block;
            }
            function parseEmptyStatement() {
                var node = createNode(224 /* EmptyStatement */);
                parseExpected(26 /* SemicolonToken */);
                return finishNode(node);
            }
            function parseIfStatement() {
                var node = createNode(227 /* IfStatement */);
                parseExpected(95 /* IfKeyword */);
                parseExpected(20 /* OpenParenToken */);
                node.expression = allowInAnd(parseExpression);
                parseExpected(21 /* CloseParenToken */);
                node.thenStatement = parseStatement();
                node.elseStatement = parseOptional(87 /* ElseKeyword */) ? parseStatement() : undefined;
                return finishNode(node);
            }
            function parseDoStatement() {
                var node = createNode(228 /* DoStatement */);
                parseExpected(86 /* DoKeyword */);
                node.statement = parseStatement();
                parseExpected(111 /* WhileKeyword */);
                parseExpected(20 /* OpenParenToken */);
                node.expression = allowInAnd(parseExpression);
                parseExpected(21 /* CloseParenToken */);
                // From: https://mail.mozilla.org/pipermail/es-discuss/2011-August/016188.html
                // 157 min --- All allen at wirfs-brock.com CONF --- "do{;}while(false)false" prohibited in
                // spec but allowed in consensus reality. Approved -- this is the de-facto standard whereby
                //  do;while(0)x will have a semicolon inserted before x.
                parseOptional(26 /* SemicolonToken */);
                return finishNode(node);
            }
            function parseWhileStatement() {
                var node = createNode(229 /* WhileStatement */);
                parseExpected(111 /* WhileKeyword */);
                parseExpected(20 /* OpenParenToken */);
                node.expression = allowInAnd(parseExpression);
                parseExpected(21 /* CloseParenToken */);
                node.statement = parseStatement();
                return finishNode(node);
            }
            function parseForOrForInOrForOfStatement() {
                var pos = getNodePos();
                parseExpected(93 /* ForKeyword */);
                var awaitToken = parseOptionalToken(127 /* AwaitKeyword */);
                parseExpected(20 /* OpenParenToken */);
                var initializer;
                if (token() !== 26 /* SemicolonToken */) {
                    if (token() === 109 /* VarKeyword */ || token() === 115 /* LetKeyword */ || token() === 81 /* ConstKeyword */) {
                        initializer = parseVariableDeclarationList(/*inForStatementInitializer*/ true);
                    }
                    else {
                        initializer = disallowInAnd(parseExpression);
                    }
                }
                var forOrForInOrForOfStatement;
                if (awaitToken ? parseExpected(152 /* OfKeyword */) : parseOptional(152 /* OfKeyword */)) {
                    var forOfStatement = createNode(232 /* ForOfStatement */, pos);
                    forOfStatement.awaitModifier = awaitToken;
                    forOfStatement.initializer = initializer;
                    forOfStatement.expression = allowInAnd(parseAssignmentExpressionOrHigher);
                    parseExpected(21 /* CloseParenToken */);
                    forOrForInOrForOfStatement = forOfStatement;
                }
                else if (parseOptional(97 /* InKeyword */)) {
                    var forInStatement = createNode(231 /* ForInStatement */, pos);
                    forInStatement.initializer = initializer;
                    forInStatement.expression = allowInAnd(parseExpression);
                    parseExpected(21 /* CloseParenToken */);
                    forOrForInOrForOfStatement = forInStatement;
                }
                else {
                    var forStatement = createNode(230 /* ForStatement */, pos);
                    forStatement.initializer = initializer;
                    parseExpected(26 /* SemicolonToken */);
                    if (token() !== 26 /* SemicolonToken */ && token() !== 21 /* CloseParenToken */) {
                        forStatement.condition = allowInAnd(parseExpression);
                    }
                    parseExpected(26 /* SemicolonToken */);
                    if (token() !== 21 /* CloseParenToken */) {
                        forStatement.incrementor = allowInAnd(parseExpression);
                    }
                    parseExpected(21 /* CloseParenToken */);
                    forOrForInOrForOfStatement = forStatement;
                }
                forOrForInOrForOfStatement.statement = parseStatement();
                return finishNode(forOrForInOrForOfStatement);
            }
            function parseBreakOrContinueStatement(kind) {
                var node = createNode(kind);
                parseExpected(kind === 234 /* BreakStatement */ ? 77 /* BreakKeyword */ : 82 /* ContinueKeyword */);
                if (!canParseSemicolon()) {
                    node.label = parseIdentifier();
                }
                parseSemicolon();
                return finishNode(node);
            }
            function parseReturnStatement() {
                var node = createNode(235 /* ReturnStatement */);
                parseExpected(101 /* ReturnKeyword */);
                if (!canParseSemicolon()) {
                    node.expression = allowInAnd(parseExpression);
                }
                parseSemicolon();
                return finishNode(node);
            }
            function parseWithStatement() {
                var node = createNode(236 /* WithStatement */);
                parseExpected(112 /* WithKeyword */);
                parseExpected(20 /* OpenParenToken */);
                node.expression = allowInAnd(parseExpression);
                parseExpected(21 /* CloseParenToken */);
                node.statement = doInsideOfContext(16777216 /* InWithStatement */, parseStatement);
                return finishNode(node);
            }
            function parseCaseClause() {
                var node = createNode(277 /* CaseClause */);
                parseExpected(78 /* CaseKeyword */);
                node.expression = allowInAnd(parseExpression);
                parseExpected(58 /* ColonToken */);
                node.statements = parseList(3 /* SwitchClauseStatements */, parseStatement);
                return finishNode(node);
            }
            function parseDefaultClause() {
                var node = createNode(278 /* DefaultClause */);
                parseExpected(84 /* DefaultKeyword */);
                parseExpected(58 /* ColonToken */);
                node.statements = parseList(3 /* SwitchClauseStatements */, parseStatement);
                return finishNode(node);
            }
            function parseCaseOrDefaultClause() {
                return token() === 78 /* CaseKeyword */ ? parseCaseClause() : parseDefaultClause();
            }
            function parseSwitchStatement() {
                var node = createNode(237 /* SwitchStatement */);
                parseExpected(103 /* SwitchKeyword */);
                parseExpected(20 /* OpenParenToken */);
                node.expression = allowInAnd(parseExpression);
                parseExpected(21 /* CloseParenToken */);
                var caseBlock = createNode(251 /* CaseBlock */);
                parseExpected(18 /* OpenBraceToken */);
                caseBlock.clauses = parseList(2 /* SwitchClauses */, parseCaseOrDefaultClause);
                parseExpected(19 /* CloseBraceToken */);
                node.caseBlock = finishNode(caseBlock);
                return finishNode(node);
            }
            function parseThrowStatement() {
                // ThrowStatement[Yield] :
                //      throw [no LineTerminator here]Expression[In, ?Yield];
                // Because of automatic semicolon insertion, we need to report error if this
                // throw could be terminated with a semicolon.  Note: we can't call 'parseExpression'
                // directly as that might consume an expression on the following line.
                // We just return 'undefined' in that case.  The actual error will be reported in the
                // grammar walker.
                var node = createNode(239 /* ThrowStatement */);
                parseExpected(105 /* ThrowKeyword */);
                node.expression = scanner.hasPrecedingLineBreak() ? undefined : allowInAnd(parseExpression);
                parseSemicolon();
                return finishNode(node);
            }
            // TODO: Review for error recovery
            function parseTryStatement() {
                var node = createNode(240 /* TryStatement */);
                parseExpected(107 /* TryKeyword */);
                node.tryBlock = parseBlock(/*ignoreMissingOpenBrace*/ false);
                node.catchClause = token() === 79 /* CatchKeyword */ ? parseCatchClause() : undefined;
                // If we don't have a catch clause, then we must have a finally clause.  Try to parse
                // one out no matter what.
                if (!node.catchClause || token() === 92 /* FinallyKeyword */) {
                    parseExpected(92 /* FinallyKeyword */);
                    node.finallyBlock = parseBlock(/*ignoreMissingOpenBrace*/ false);
                }
                return finishNode(node);
            }
            function parseCatchClause() {
                var result = createNode(280 /* CatchClause */);
                parseExpected(79 /* CatchKeyword */);
                if (parseOptional(20 /* OpenParenToken */)) {
                    result.variableDeclaration = parseVariableDeclaration();
                    parseExpected(21 /* CloseParenToken */);
                }
                else {
                    // Keep shape of node to avoid degrading performance.
                    result.variableDeclaration = undefined;
                }
                result.block = parseBlock(/*ignoreMissingOpenBrace*/ false);
                return finishNode(result);
            }
            function parseDebuggerStatement() {
                var node = createNode(241 /* DebuggerStatement */);
                parseExpected(83 /* DebuggerKeyword */);
                parseSemicolon();
                return finishNode(node);
            }
            function parseExpressionOrLabeledStatement() {
                // Avoiding having to do the lookahead for a labeled statement by just trying to parse
                // out an expression, seeing if it is identifier and then seeing if it is followed by
                // a colon.
                var node = createNodeWithJSDoc(0 /* Unknown */);
                var expression = allowInAnd(parseExpression);
                if (expression.kind === 75 /* Identifier */ && parseOptional(58 /* ColonToken */)) {
                    node.kind = 238 /* LabeledStatement */;
                    node.label = expression;
                    node.statement = parseStatement();
                }
                else {
                    node.kind = 226 /* ExpressionStatement */;
                    node.expression = expression;
                    parseSemicolon();
                }
                return finishNode(node);
            }
            function nextTokenIsIdentifierOrKeywordOnSameLine() {
                nextToken();
                return ts.tokenIsIdentifierOrKeyword(token()) && !scanner.hasPrecedingLineBreak();
            }
            function nextTokenIsClassKeywordOnSameLine() {
                nextToken();
                return token() === 80 /* ClassKeyword */ && !scanner.hasPrecedingLineBreak();
            }
            function nextTokenIsFunctionKeywordOnSameLine() {
                nextToken();
                return token() === 94 /* FunctionKeyword */ && !scanner.hasPrecedingLineBreak();
            }
            function nextTokenIsIdentifierOrKeywordOrLiteralOnSameLine() {
                nextToken();
                return (ts.tokenIsIdentifierOrKeyword(token()) || token() === 8 /* NumericLiteral */ || token() === 9 /* BigIntLiteral */ || token() === 10 /* StringLiteral */) && !scanner.hasPrecedingLineBreak();
            }
            function isDeclaration() {
                while (true) {
                    switch (token()) {
                        case 109 /* VarKeyword */:
                        case 115 /* LetKeyword */:
                        case 81 /* ConstKeyword */:
                        case 94 /* FunctionKeyword */:
                        case 80 /* ClassKeyword */:
                        case 88 /* EnumKeyword */:
                            return true;
                        // 'declare', 'module', 'namespace', 'interface'* and 'type' are all legal JavaScript identifiers;
                        // however, an identifier cannot be followed by another identifier on the same line. This is what we
                        // count on to parse out the respective declarations. For instance, we exploit this to say that
                        //
                        //    namespace n
                        //
                        // can be none other than the beginning of a namespace declaration, but need to respect that JavaScript sees
                        //
                        //    namespace
                        //    n
                        //
                        // as the identifier 'namespace' on one line followed by the identifier 'n' on another.
                        // We need to look one token ahead to see if it permissible to try parsing a declaration.
                        //
                        // *Note*: 'interface' is actually a strict mode reserved word. So while
                        //
                        //   "use strict"
                        //   interface
                        //   I {}
                        //
                        // could be legal, it would add complexity for very little gain.
                        case 114 /* InterfaceKeyword */:
                        case 145 /* TypeKeyword */:
                            return nextTokenIsIdentifierOnSameLine();
                        case 135 /* ModuleKeyword */:
                        case 136 /* NamespaceKeyword */:
                            return nextTokenIsIdentifierOrStringLiteralOnSameLine();
                        case 122 /* AbstractKeyword */:
                        case 126 /* AsyncKeyword */:
                        case 130 /* DeclareKeyword */:
                        case 117 /* PrivateKeyword */:
                        case 118 /* ProtectedKeyword */:
                        case 119 /* PublicKeyword */:
                        case 138 /* ReadonlyKeyword */:
                            nextToken();
                            // ASI takes effect for this modifier.
                            if (scanner.hasPrecedingLineBreak()) {
                                return false;
                            }
                            continue;
                        case 150 /* GlobalKeyword */:
                            nextToken();
                            return token() === 18 /* OpenBraceToken */ || token() === 75 /* Identifier */ || token() === 89 /* ExportKeyword */;
                        case 96 /* ImportKeyword */:
                            nextToken();
                            return token() === 10 /* StringLiteral */ || token() === 41 /* AsteriskToken */ ||
                                token() === 18 /* OpenBraceToken */ || ts.tokenIsIdentifierOrKeyword(token());
                        case 89 /* ExportKeyword */:
                            var currentToken_1 = nextToken();
                            if (currentToken_1 === 145 /* TypeKeyword */) {
                                currentToken_1 = lookAhead(nextToken);
                            }
                            if (currentToken_1 === 62 /* EqualsToken */ || currentToken_1 === 41 /* AsteriskToken */ ||
                                currentToken_1 === 18 /* OpenBraceToken */ || currentToken_1 === 84 /* DefaultKeyword */ ||
                                currentToken_1 === 123 /* AsKeyword */) {
                                return true;
                            }
                            continue;
                        case 120 /* StaticKeyword */:
                            nextToken();
                            continue;
                        default:
                            return false;
                    }
                }
            }
            function isStartOfDeclaration() {
                return lookAhead(isDeclaration);
            }
            function isStartOfStatement() {
                switch (token()) {
                    case 59 /* AtToken */:
                    case 26 /* SemicolonToken */:
                    case 18 /* OpenBraceToken */:
                    case 109 /* VarKeyword */:
                    case 115 /* LetKeyword */:
                    case 94 /* FunctionKeyword */:
                    case 80 /* ClassKeyword */:
                    case 88 /* EnumKeyword */:
                    case 95 /* IfKeyword */:
                    case 86 /* DoKeyword */:
                    case 111 /* WhileKeyword */:
                    case 93 /* ForKeyword */:
                    case 82 /* ContinueKeyword */:
                    case 77 /* BreakKeyword */:
                    case 101 /* ReturnKeyword */:
                    case 112 /* WithKeyword */:
                    case 103 /* SwitchKeyword */:
                    case 105 /* ThrowKeyword */:
                    case 107 /* TryKeyword */:
                    case 83 /* DebuggerKeyword */:
                    // 'catch' and 'finally' do not actually indicate that the code is part of a statement,
                    // however, we say they are here so that we may gracefully parse them and error later.
                    // falls through
                    case 79 /* CatchKeyword */:
                    case 92 /* FinallyKeyword */:
                        return true;
                    case 96 /* ImportKeyword */:
                        return isStartOfDeclaration() || lookAhead(nextTokenIsOpenParenOrLessThanOrDot);
                    case 81 /* ConstKeyword */:
                    case 89 /* ExportKeyword */:
                        return isStartOfDeclaration();
                    case 126 /* AsyncKeyword */:
                    case 130 /* DeclareKeyword */:
                    case 114 /* InterfaceKeyword */:
                    case 135 /* ModuleKeyword */:
                    case 136 /* NamespaceKeyword */:
                    case 145 /* TypeKeyword */:
                    case 150 /* GlobalKeyword */:
                        // When these don't start a declaration, they're an identifier in an expression statement
                        return true;
                    case 119 /* PublicKeyword */:
                    case 117 /* PrivateKeyword */:
                    case 118 /* ProtectedKeyword */:
                    case 120 /* StaticKeyword */:
                    case 138 /* ReadonlyKeyword */:
                        // When these don't start a declaration, they may be the start of a class member if an identifier
                        // immediately follows. Otherwise they're an identifier in an expression statement.
                        return isStartOfDeclaration() || !lookAhead(nextTokenIsIdentifierOrKeywordOnSameLine);
                    default:
                        return isStartOfExpression();
                }
            }
            function nextTokenIsIdentifierOrStartOfDestructuring() {
                nextToken();
                return isIdentifier() || token() === 18 /* OpenBraceToken */ || token() === 22 /* OpenBracketToken */;
            }
            function isLetDeclaration() {
                // In ES6 'let' always starts a lexical declaration if followed by an identifier or {
                // or [.
                return lookAhead(nextTokenIsIdentifierOrStartOfDestructuring);
            }
            function parseStatement() {
                switch (token()) {
                    case 26 /* SemicolonToken */:
                        return parseEmptyStatement();
                    case 18 /* OpenBraceToken */:
                        return parseBlock(/*ignoreMissingOpenBrace*/ false);
                    case 109 /* VarKeyword */:
                        return parseVariableStatement(createNodeWithJSDoc(242 /* VariableDeclaration */));
                    case 115 /* LetKeyword */:
                        if (isLetDeclaration()) {
                            return parseVariableStatement(createNodeWithJSDoc(242 /* VariableDeclaration */));
                        }
                        break;
                    case 94 /* FunctionKeyword */:
                        return parseFunctionDeclaration(createNodeWithJSDoc(244 /* FunctionDeclaration */));
                    case 80 /* ClassKeyword */:
                        return parseClassDeclaration(createNodeWithJSDoc(245 /* ClassDeclaration */));
                    case 95 /* IfKeyword */:
                        return parseIfStatement();
                    case 86 /* DoKeyword */:
                        return parseDoStatement();
                    case 111 /* WhileKeyword */:
                        return parseWhileStatement();
                    case 93 /* ForKeyword */:
                        return parseForOrForInOrForOfStatement();
                    case 82 /* ContinueKeyword */:
                        return parseBreakOrContinueStatement(233 /* ContinueStatement */);
                    case 77 /* BreakKeyword */:
                        return parseBreakOrContinueStatement(234 /* BreakStatement */);
                    case 101 /* ReturnKeyword */:
                        return parseReturnStatement();
                    case 112 /* WithKeyword */:
                        return parseWithStatement();
                    case 103 /* SwitchKeyword */:
                        return parseSwitchStatement();
                    case 105 /* ThrowKeyword */:
                        return parseThrowStatement();
                    case 107 /* TryKeyword */:
                    // Include 'catch' and 'finally' for error recovery.
                    // falls through
                    case 79 /* CatchKeyword */:
                    case 92 /* FinallyKeyword */:
                        return parseTryStatement();
                    case 83 /* DebuggerKeyword */:
                        return parseDebuggerStatement();
                    case 59 /* AtToken */:
                        return parseDeclaration();
                    case 126 /* AsyncKeyword */:
                    case 114 /* InterfaceKeyword */:
                    case 145 /* TypeKeyword */:
                    case 135 /* ModuleKeyword */:
                    case 136 /* NamespaceKeyword */:
                    case 130 /* DeclareKeyword */:
                    case 81 /* ConstKeyword */:
                    case 88 /* EnumKeyword */:
                    case 89 /* ExportKeyword */:
                    case 96 /* ImportKeyword */:
                    case 117 /* PrivateKeyword */:
                    case 118 /* ProtectedKeyword */:
                    case 119 /* PublicKeyword */:
                    case 122 /* AbstractKeyword */:
                    case 120 /* StaticKeyword */:
                    case 138 /* ReadonlyKeyword */:
                    case 150 /* GlobalKeyword */:
                        if (isStartOfDeclaration()) {
                            return parseDeclaration();
                        }
                        break;
                }
                return parseExpressionOrLabeledStatement();
            }
            function isDeclareModifier(modifier) {
                return modifier.kind === 130 /* DeclareKeyword */;
            }
            function parseDeclaration() {
                var modifiers = lookAhead(function () { return (parseDecorators(), parseModifiers()); });
                // `parseListElement` attempted to get the reused node at this position,
                // but the ambient context flag was not yet set, so the node appeared
                // not reusable in that context.
                var isAmbient = ts.some(modifiers, isDeclareModifier);
                if (isAmbient) {
                    var node_3 = tryReuseAmbientDeclaration();
                    if (node_3) {
                        return node_3;
                    }
                }
                var node = createNodeWithJSDoc(0 /* Unknown */);
                node.decorators = parseDecorators();
                node.modifiers = parseModifiers();
                if (isAmbient) {
                    for (var _i = 0, _a = node.modifiers; _i < _a.length; _i++) {
                        var m = _a[_i];
                        m.flags |= 8388608 /* Ambient */;
                    }
                    return doInsideOfContext(8388608 /* Ambient */, function () { return parseDeclarationWorker(node); });
                }
                else {
                    return parseDeclarationWorker(node);
                }
            }
            function tryReuseAmbientDeclaration() {
                return doInsideOfContext(8388608 /* Ambient */, function () {
                    var node = currentNode(parsingContext);
                    if (node) {
                        return consumeNode(node);
                    }
                });
            }
            function parseDeclarationWorker(node) {
                switch (token()) {
                    case 109 /* VarKeyword */:
                    case 115 /* LetKeyword */:
                    case 81 /* ConstKeyword */:
                        return parseVariableStatement(node);
                    case 94 /* FunctionKeyword */:
                        return parseFunctionDeclaration(node);
                    case 80 /* ClassKeyword */:
                        return parseClassDeclaration(node);
                    case 114 /* InterfaceKeyword */:
                        return parseInterfaceDeclaration(node);
                    case 145 /* TypeKeyword */:
                        return parseTypeAliasDeclaration(node);
                    case 88 /* EnumKeyword */:
                        return parseEnumDeclaration(node);
                    case 150 /* GlobalKeyword */:
                    case 135 /* ModuleKeyword */:
                    case 136 /* NamespaceKeyword */:
                        return parseModuleDeclaration(node);
                    case 96 /* ImportKeyword */:
                        return parseImportDeclarationOrImportEqualsDeclaration(node);
                    case 89 /* ExportKeyword */:
                        nextToken();
                        switch (token()) {
                            case 84 /* DefaultKeyword */:
                            case 62 /* EqualsToken */:
                                return parseExportAssignment(node);
                            case 123 /* AsKeyword */:
                                return parseNamespaceExportDeclaration(node);
                            default:
                                return parseExportDeclaration(node);
                        }
                    default:
                        if (node.decorators || node.modifiers) {
                            // We reached this point because we encountered decorators and/or modifiers and assumed a declaration
                            // would follow. For recovery and error reporting purposes, return an incomplete declaration.
                            var missing = createMissingNode(264 /* MissingDeclaration */, /*reportAtCurrentPosition*/ true, ts.Diagnostics.Declaration_expected);
                            missing.pos = node.pos;
                            missing.decorators = node.decorators;
                            missing.modifiers = node.modifiers;
                            return finishNode(missing);
                        }
                        return undefined; // TODO: GH#18217
                }
            }
            function nextTokenIsIdentifierOrStringLiteralOnSameLine() {
                nextToken();
                return !scanner.hasPrecedingLineBreak() && (isIdentifier() || token() === 10 /* StringLiteral */);
            }
            function parseFunctionBlockOrSemicolon(flags, diagnosticMessage) {
                if (token() !== 18 /* OpenBraceToken */ && canParseSemicolon()) {
                    parseSemicolon();
                    return;
                }
                return parseFunctionBlock(flags, diagnosticMessage);
            }
            // DECLARATIONS
            function parseArrayBindingElement() {
                if (token() === 27 /* CommaToken */) {
                    return createNode(215 /* OmittedExpression */);
                }
                var node = createNode(191 /* BindingElement */);
                node.dotDotDotToken = parseOptionalToken(25 /* DotDotDotToken */);
                node.name = parseIdentifierOrPattern();
                node.initializer = parseInitializer();
                return finishNode(node);
            }
            function parseObjectBindingElement() {
                var node = createNode(191 /* BindingElement */);
                node.dotDotDotToken = parseOptionalToken(25 /* DotDotDotToken */);
                var tokenIsIdentifier = isIdentifier();
                var propertyName = parsePropertyName();
                if (tokenIsIdentifier && token() !== 58 /* ColonToken */) {
                    node.name = propertyName;
                }
                else {
                    parseExpected(58 /* ColonToken */);
                    node.propertyName = propertyName;
                    node.name = parseIdentifierOrPattern();
                }
                node.initializer = parseInitializer();
                return finishNode(node);
            }
            function parseObjectBindingPattern() {
                var node = createNode(189 /* ObjectBindingPattern */);
                parseExpected(18 /* OpenBraceToken */);
                node.elements = parseDelimitedList(9 /* ObjectBindingElements */, parseObjectBindingElement);
                parseExpected(19 /* CloseBraceToken */);
                return finishNode(node);
            }
            function parseArrayBindingPattern() {
                var node = createNode(190 /* ArrayBindingPattern */);
                parseExpected(22 /* OpenBracketToken */);
                node.elements = parseDelimitedList(10 /* ArrayBindingElements */, parseArrayBindingElement);
                parseExpected(23 /* CloseBracketToken */);
                return finishNode(node);
            }
            function isIdentifierOrPrivateIdentifierOrPattern() {
                return token() === 18 /* OpenBraceToken */
                    || token() === 22 /* OpenBracketToken */
                    || token() === 76 /* PrivateIdentifier */
                    || isIdentifier();
            }
            function parseIdentifierOrPattern(privateIdentifierDiagnosticMessage) {
                if (token() === 22 /* OpenBracketToken */) {
                    return parseArrayBindingPattern();
                }
                if (token() === 18 /* OpenBraceToken */) {
                    return parseObjectBindingPattern();
                }
                return parseIdentifier(/*diagnosticMessage*/ undefined, privateIdentifierDiagnosticMessage);
            }
            function parseVariableDeclarationAllowExclamation() {
                return parseVariableDeclaration(/*allowExclamation*/ true);
            }
            function parseVariableDeclaration(allowExclamation) {
                var node = createNode(242 /* VariableDeclaration */);
                node.name = parseIdentifierOrPattern(ts.Diagnostics.Private_identifiers_are_not_allowed_in_variable_declarations);
                if (allowExclamation && node.name.kind === 75 /* Identifier */ &&
                    token() === 53 /* ExclamationToken */ && !scanner.hasPrecedingLineBreak()) {
                    node.exclamationToken = parseTokenNode();
                }
                node.type = parseTypeAnnotation();
                if (!isInOrOfKeyword(token())) {
                    node.initializer = parseInitializer();
                }
                return finishNode(node);
            }
            function parseVariableDeclarationList(inForStatementInitializer) {
                var node = createNode(243 /* VariableDeclarationList */);
                switch (token()) {
                    case 109 /* VarKeyword */:
                        break;
                    case 115 /* LetKeyword */:
                        node.flags |= 1 /* Let */;
                        break;
                    case 81 /* ConstKeyword */:
                        node.flags |= 2 /* Const */;
                        break;
                    default:
                        ts.Debug.fail();
                }
                nextToken();
                // The user may have written the following:
                //
                //    for (let of X) { }
                //
                // In this case, we want to parse an empty declaration list, and then parse 'of'
                // as a keyword. The reason this is not automatic is that 'of' is a valid identifier.
                // So we need to look ahead to determine if 'of' should be treated as a keyword in
                // this context.
                // The checker will then give an error that there is an empty declaration list.
                if (token() === 152 /* OfKeyword */ && lookAhead(canFollowContextualOfKeyword)) {
                    node.declarations = createMissingList();
                }
                else {
                    var savedDisallowIn = inDisallowInContext();
                    setDisallowInContext(inForStatementInitializer);
                    node.declarations = parseDelimitedList(8 /* VariableDeclarations */, inForStatementInitializer ? parseVariableDeclaration : parseVariableDeclarationAllowExclamation);
                    setDisallowInContext(savedDisallowIn);
                }
                return finishNode(node);
            }
            function canFollowContextualOfKeyword() {
                return nextTokenIsIdentifier() && nextToken() === 21 /* CloseParenToken */;
            }
            function parseVariableStatement(node) {
                node.kind = 225 /* VariableStatement */;
                node.declarationList = parseVariableDeclarationList(/*inForStatementInitializer*/ false);
                parseSemicolon();
                return finishNode(node);
            }
            function parseFunctionDeclaration(node) {
                node.kind = 244 /* FunctionDeclaration */;
                parseExpected(94 /* FunctionKeyword */);
                node.asteriskToken = parseOptionalToken(41 /* AsteriskToken */);
                node.name = hasModifierOfKind(node, 84 /* DefaultKeyword */) ? parseOptionalIdentifier() : parseIdentifier();
                var isGenerator = node.asteriskToken ? 1 /* Yield */ : 0 /* None */;
                var isAsync = hasModifierOfKind(node, 126 /* AsyncKeyword */) ? 2 /* Await */ : 0 /* None */;
                fillSignature(58 /* ColonToken */, isGenerator | isAsync, node);
                node.body = parseFunctionBlockOrSemicolon(isGenerator | isAsync, ts.Diagnostics.or_expected);
                return finishNode(node);
            }
            function parseConstructorName() {
                if (token() === 129 /* ConstructorKeyword */) {
                    return parseExpected(129 /* ConstructorKeyword */);
                }
                if (token() === 10 /* StringLiteral */ && lookAhead(nextToken) === 20 /* OpenParenToken */) {
                    return tryParse(function () {
                        var literalNode = parseLiteralNode();
                        return literalNode.text === "constructor" ? literalNode : undefined;
                    });
                }
            }
            function tryParseConstructorDeclaration(node) {
                return tryParse(function () {
                    if (parseConstructorName()) {
                        node.kind = 162 /* Constructor */;
                        fillSignature(58 /* ColonToken */, 0 /* None */, node);
                        node.body = parseFunctionBlockOrSemicolon(0 /* None */, ts.Diagnostics.or_expected);
                        return finishNode(node);
                    }
                });
            }
            function parseMethodDeclaration(node, asteriskToken, diagnosticMessage) {
                node.kind = 161 /* MethodDeclaration */;
                node.asteriskToken = asteriskToken;
                var isGenerator = asteriskToken ? 1 /* Yield */ : 0 /* None */;
                var isAsync = hasModifierOfKind(node, 126 /* AsyncKeyword */) ? 2 /* Await */ : 0 /* None */;
                fillSignature(58 /* ColonToken */, isGenerator | isAsync, node);
                node.body = parseFunctionBlockOrSemicolon(isGenerator | isAsync, diagnosticMessage);
                return finishNode(node);
            }
            function parsePropertyDeclaration(node) {
                node.kind = 159 /* PropertyDeclaration */;
                if (!node.questionToken && token() === 53 /* ExclamationToken */ && !scanner.hasPrecedingLineBreak()) {
                    node.exclamationToken = parseTokenNode();
                }
                node.type = parseTypeAnnotation();
                node.initializer = doOutsideOfContext(8192 /* YieldContext */ | 32768 /* AwaitContext */ | 4096 /* DisallowInContext */, parseInitializer);
                parseSemicolon();
                return finishNode(node);
            }
            function parsePropertyOrMethodDeclaration(node) {
                var asteriskToken = parseOptionalToken(41 /* AsteriskToken */);
                node.name = parsePropertyName();
                // Note: this is not legal as per the grammar.  But we allow it in the parser and
                // report an error in the grammar checker.
                node.questionToken = parseOptionalToken(57 /* QuestionToken */);
                if (asteriskToken || token() === 20 /* OpenParenToken */ || token() === 29 /* LessThanToken */) {
                    return parseMethodDeclaration(node, asteriskToken, ts.Diagnostics.or_expected);
                }
                return parsePropertyDeclaration(node);
            }
            function parseAccessorDeclaration(node, kind) {
                node.kind = kind;
                node.name = parsePropertyName();
                fillSignature(58 /* ColonToken */, 0 /* None */, node);
                node.body = parseFunctionBlockOrSemicolon(0 /* None */);
                return finishNode(node);
            }
            function isClassMemberStart() {
                var idToken;
                if (token() === 59 /* AtToken */) {
                    return true;
                }
                // Eat up all modifiers, but hold on to the last one in case it is actually an identifier.
                while (ts.isModifierKind(token())) {
                    idToken = token();
                    // If the idToken is a class modifier (protected, private, public, and static), it is
                    // certain that we are starting to parse class member. This allows better error recovery
                    // Example:
                    //      public foo() ...     // true
                    //      public @dec blah ... // true; we will then report an error later
                    //      export public ...    // true; we will then report an error later
                    if (ts.isClassMemberModifier(idToken)) {
                        return true;
                    }
                    nextToken();
                }
                if (token() === 41 /* AsteriskToken */) {
                    return true;
                }
                // Try to get the first property-like token following all modifiers.
                // This can either be an identifier or the 'get' or 'set' keywords.
                if (isLiteralPropertyName()) {
                    idToken = token();
                    nextToken();
                }
                // Index signatures and computed properties are class members; we can parse.
                if (token() === 22 /* OpenBracketToken */) {
                    return true;
                }
                // If we were able to get any potential identifier...
                if (idToken !== undefined) {
                    // If we have a non-keyword identifier, or if we have an accessor, then it's safe to parse.
                    if (!ts.isKeyword(idToken) || idToken === 142 /* SetKeyword */ || idToken === 131 /* GetKeyword */) {
                        return true;
                    }
                    // If it *is* a keyword, but not an accessor, check a little farther along
                    // to see if it should actually be parsed as a class member.
                    switch (token()) {
                        case 20 /* OpenParenToken */: // Method declaration
                        case 29 /* LessThanToken */: // Generic Method declaration
                        case 53 /* ExclamationToken */: // Non-null assertion on property name
                        case 58 /* ColonToken */: // Type Annotation for declaration
                        case 62 /* EqualsToken */: // Initializer for declaration
                        case 57 /* QuestionToken */: // Not valid, but permitted so that it gets caught later on.
                            return true;
                        default:
                            // Covers
                            //  - Semicolons     (declaration termination)
                            //  - Closing braces (end-of-class, must be declaration)
                            //  - End-of-files   (not valid, but permitted so that it gets caught later on)
                            //  - Line-breaks    (enabling *automatic semicolon insertion*)
                            return canParseSemicolon();
                    }
                }
                return false;
            }
            function parseDecorators() {
                var list;
                var listPos = getNodePos();
                while (true) {
                    var decoratorStart = getNodePos();
                    if (!parseOptional(59 /* AtToken */)) {
                        break;
                    }
                    var decorator = createNode(157 /* Decorator */, decoratorStart);
                    decorator.expression = doInDecoratorContext(parseLeftHandSideExpressionOrHigher);
                    finishNode(decorator);
                    (list || (list = [])).push(decorator);
                }
                return list && createNodeArray(list, listPos);
            }
            /*
             * There are situations in which a modifier like 'const' will appear unexpectedly, such as on a class member.
             * In those situations, if we are entirely sure that 'const' is not valid on its own (such as when ASI takes effect
             * and turns it into a standalone declaration), then it is better to parse it and report an error later.
             *
             * In such situations, 'permitInvalidConstAsModifier' should be set to true.
             */
            function parseModifiers(permitInvalidConstAsModifier) {
                var list;
                var listPos = getNodePos();
                while (true) {
                    var modifierStart = scanner.getStartPos();
                    var modifierKind = token();
                    if (token() === 81 /* ConstKeyword */ && permitInvalidConstAsModifier) {
                        // We need to ensure that any subsequent modifiers appear on the same line
                        // so that when 'const' is a standalone declaration, we don't issue an error.
                        if (!tryParse(nextTokenIsOnSameLineAndCanFollowModifier)) {
                            break;
                        }
                    }
                    else {
                        if (!parseAnyContextualModifier()) {
                            break;
                        }
                    }
                    var modifier = finishNode(createNode(modifierKind, modifierStart));
                    (list || (list = [])).push(modifier);
                }
                return list && createNodeArray(list, listPos);
            }
            function parseModifiersForArrowFunction() {
                var modifiers;
                if (token() === 126 /* AsyncKeyword */) {
                    var modifierStart = scanner.getStartPos();
                    var modifierKind = token();
                    nextToken();
                    var modifier = finishNode(createNode(modifierKind, modifierStart));
                    modifiers = createNodeArray([modifier], modifierStart);
                }
                return modifiers;
            }
            function parseClassElement() {
                if (token() === 26 /* SemicolonToken */) {
                    var result = createNode(222 /* SemicolonClassElement */);
                    nextToken();
                    return finishNode(result);
                }
                var node = createNodeWithJSDoc(0 /* Unknown */);
                node.decorators = parseDecorators();
                node.modifiers = parseModifiers(/*permitInvalidConstAsModifier*/ true);
                if (parseContextualModifier(131 /* GetKeyword */)) {
                    return parseAccessorDeclaration(node, 163 /* GetAccessor */);
                }
                if (parseContextualModifier(142 /* SetKeyword */)) {
                    return parseAccessorDeclaration(node, 164 /* SetAccessor */);
                }
                if (token() === 129 /* ConstructorKeyword */ || token() === 10 /* StringLiteral */) {
                    var constructorDeclaration = tryParseConstructorDeclaration(node);
                    if (constructorDeclaration) {
                        return constructorDeclaration;
                    }
                }
                if (isIndexSignature()) {
                    return parseIndexSignatureDeclaration(node);
                }
                // It is very important that we check this *after* checking indexers because
                // the [ token can start an index signature or a computed property name
                if (ts.tokenIsIdentifierOrKeyword(token()) ||
                    token() === 10 /* StringLiteral */ ||
                    token() === 8 /* NumericLiteral */ ||
                    token() === 41 /* AsteriskToken */ ||
                    token() === 22 /* OpenBracketToken */) {
                    var isAmbient = node.modifiers && ts.some(node.modifiers, isDeclareModifier);
                    if (isAmbient) {
                        for (var _i = 0, _a = node.modifiers; _i < _a.length; _i++) {
                            var m = _a[_i];
                            m.flags |= 8388608 /* Ambient */;
                        }
                        return doInsideOfContext(8388608 /* Ambient */, function () { return parsePropertyOrMethodDeclaration(node); });
                    }
                    else {
                        return parsePropertyOrMethodDeclaration(node);
                    }
                }
                if (node.decorators || node.modifiers) {
                    // treat this as a property declaration with a missing name.
                    node.name = createMissingNode(75 /* Identifier */, /*reportAtCurrentPosition*/ true, ts.Diagnostics.Declaration_expected);
                    return parsePropertyDeclaration(node);
                }
                // 'isClassMemberStart' should have hinted not to attempt parsing.
                return ts.Debug.fail("Should not have attempted to parse class member declaration.");
            }
            function parseClassExpression() {
                return parseClassDeclarationOrExpression(createNodeWithJSDoc(0 /* Unknown */), 214 /* ClassExpression */);
            }
            function parseClassDeclaration(node) {
                return parseClassDeclarationOrExpression(node, 245 /* ClassDeclaration */);
            }
            function parseClassDeclarationOrExpression(node, kind) {
                node.kind = kind;
                parseExpected(80 /* ClassKeyword */);
                node.name = parseNameOfClassDeclarationOrExpression();
                node.typeParameters = parseTypeParameters();
                node.heritageClauses = parseHeritageClauses();
                if (parseExpected(18 /* OpenBraceToken */)) {
                    // ClassTail[Yield,Await] : (Modified) See 14.5
                    //      ClassHeritage[?Yield,?Await]opt { ClassBody[?Yield,?Await]opt }
                    node.members = parseClassMembers();
                    parseExpected(19 /* CloseBraceToken */);
                }
                else {
                    node.members = createMissingList();
                }
                return finishNode(node);
            }
            function parseNameOfClassDeclarationOrExpression() {
                // implements is a future reserved word so
                // 'class implements' might mean either
                // - class expression with omitted name, 'implements' starts heritage clause
                // - class with name 'implements'
                // 'isImplementsClause' helps to disambiguate between these two cases
                return isIdentifier() && !isImplementsClause()
                    ? parseIdentifier()
                    : undefined;
            }
            function isImplementsClause() {
                return token() === 113 /* ImplementsKeyword */ && lookAhead(nextTokenIsIdentifierOrKeyword);
            }
            function parseHeritageClauses() {
                // ClassTail[Yield,Await] : (Modified) See 14.5
                //      ClassHeritage[?Yield,?Await]opt { ClassBody[?Yield,?Await]opt }
                if (isHeritageClause()) {
                    return parseList(22 /* HeritageClauses */, parseHeritageClause);
                }
                return undefined;
            }
            function parseHeritageClause() {
                var tok = token();
                ts.Debug.assert(tok === 90 /* ExtendsKeyword */ || tok === 113 /* ImplementsKeyword */); // isListElement() should ensure this.
                var node = createNode(279 /* HeritageClause */);
                node.token = tok;
                nextToken();
                node.types = parseDelimitedList(7 /* HeritageClauseElement */, parseExpressionWithTypeArguments);
                return finishNode(node);
            }
            function parseExpressionWithTypeArguments() {
                var node = createNode(216 /* ExpressionWithTypeArguments */);
                node.expression = parseLeftHandSideExpressionOrHigher();
                node.typeArguments = tryParseTypeArguments();
                return finishNode(node);
            }
            function tryParseTypeArguments() {
                return token() === 29 /* LessThanToken */ ?
                    parseBracketedList(20 /* TypeArguments */, parseType, 29 /* LessThanToken */, 31 /* GreaterThanToken */) : undefined;
            }
            function isHeritageClause() {
                return token() === 90 /* ExtendsKeyword */ || token() === 113 /* ImplementsKeyword */;
            }
            function parseClassMembers() {
                return parseList(5 /* ClassMembers */, parseClassElement);
            }
            function parseInterfaceDeclaration(node) {
                node.kind = 246 /* InterfaceDeclaration */;
                parseExpected(114 /* InterfaceKeyword */);
                node.name = parseIdentifier();
                node.typeParameters = parseTypeParameters();
                node.heritageClauses = parseHeritageClauses();
                node.members = parseObjectTypeMembers();
                return finishNode(node);
            }
            function parseTypeAliasDeclaration(node) {
                node.kind = 247 /* TypeAliasDeclaration */;
                parseExpected(145 /* TypeKeyword */);
                node.name = parseIdentifier();
                node.typeParameters = parseTypeParameters();
                parseExpected(62 /* EqualsToken */);
                node.type = parseType();
                parseSemicolon();
                return finishNode(node);
            }
            // In an ambient declaration, the grammar only allows integer literals as initializers.
            // In a non-ambient declaration, the grammar allows uninitialized members only in a
            // ConstantEnumMemberSection, which starts at the beginning of an enum declaration
            // or any time an integer literal initializer is encountered.
            function parseEnumMember() {
                var node = createNodeWithJSDoc(284 /* EnumMember */);
                node.name = parsePropertyName();
                node.initializer = allowInAnd(parseInitializer);
                return finishNode(node);
            }
            function parseEnumDeclaration(node) {
                node.kind = 248 /* EnumDeclaration */;
                parseExpected(88 /* EnumKeyword */);
                node.name = parseIdentifier();
                if (parseExpected(18 /* OpenBraceToken */)) {
                    node.members = doOutsideOfYieldAndAwaitContext(function () { return parseDelimitedList(6 /* EnumMembers */, parseEnumMember); });
                    parseExpected(19 /* CloseBraceToken */);
                }
                else {
                    node.members = createMissingList();
                }
                return finishNode(node);
            }
            function parseModuleBlock() {
                var node = createNode(250 /* ModuleBlock */);
                if (parseExpected(18 /* OpenBraceToken */)) {
                    node.statements = parseList(1 /* BlockStatements */, parseStatement);
                    parseExpected(19 /* CloseBraceToken */);
                }
                else {
                    node.statements = createMissingList();
                }
                return finishNode(node);
            }
            function parseModuleOrNamespaceDeclaration(node, flags) {
                node.kind = 249 /* ModuleDeclaration */;
                // If we are parsing a dotted namespace name, we want to
                // propagate the 'Namespace' flag across the names if set.
                var namespaceFlag = flags & 16 /* Namespace */;
                node.flags |= flags;
                node.name = parseIdentifier();
                node.body = parseOptional(24 /* DotToken */)
                    ? parseModuleOrNamespaceDeclaration(createNode(0 /* Unknown */), 4 /* NestedNamespace */ | namespaceFlag)
                    : parseModuleBlock();
                return finishNode(node);
            }
            function parseAmbientExternalModuleDeclaration(node) {
                node.kind = 249 /* ModuleDeclaration */;
                if (token() === 150 /* GlobalKeyword */) {
                    // parse 'global' as name of global scope augmentation
                    node.name = parseIdentifier();
                    node.flags |= 1024 /* GlobalAugmentation */;
                }
                else {
                    node.name = parseLiteralNode();
                    node.name.text = internIdentifier(node.name.text);
                }
                if (token() === 18 /* OpenBraceToken */) {
                    node.body = parseModuleBlock();
                }
                else {
                    parseSemicolon();
                }
                return finishNode(node);
            }
            function parseModuleDeclaration(node) {
                var flags = 0;
                if (token() === 150 /* GlobalKeyword */) {
                    // global augmentation
                    return parseAmbientExternalModuleDeclaration(node);
                }
                else if (parseOptional(136 /* NamespaceKeyword */)) {
                    flags |= 16 /* Namespace */;
                }
                else {
                    parseExpected(135 /* ModuleKeyword */);
                    if (token() === 10 /* StringLiteral */) {
                        return parseAmbientExternalModuleDeclaration(node);
                    }
                }
                return parseModuleOrNamespaceDeclaration(node, flags);
            }
            function isExternalModuleReference() {
                return token() === 139 /* RequireKeyword */ &&
                    lookAhead(nextTokenIsOpenParen);
            }
            function nextTokenIsOpenParen() {
                return nextToken() === 20 /* OpenParenToken */;
            }
            function nextTokenIsSlash() {
                return nextToken() === 43 /* SlashToken */;
            }
            function parseNamespaceExportDeclaration(node) {
                node.kind = 252 /* NamespaceExportDeclaration */;
                parseExpected(123 /* AsKeyword */);
                parseExpected(136 /* NamespaceKeyword */);
                node.name = parseIdentifier();
                parseSemicolon();
                return finishNode(node);
            }
            function parseImportDeclarationOrImportEqualsDeclaration(node) {
                parseExpected(96 /* ImportKeyword */);
                var afterImportPos = scanner.getStartPos();
                var identifier;
                if (isIdentifier()) {
                    identifier = parseIdentifier();
                }
                var isTypeOnly = false;
                if (token() !== 149 /* FromKeyword */ &&
                    (identifier === null || identifier === void 0 ? void 0 : identifier.escapedText) === "type" &&
                    (isIdentifier() || tokenAfterImportDefinitelyProducesImportDeclaration())) {
                    isTypeOnly = true;
                    identifier = isIdentifier() ? parseIdentifier() : undefined;
                }
                if (identifier && !tokenAfterImportedIdentifierDefinitelyProducesImportDeclaration()) {
                    return parseImportEqualsDeclaration(node, identifier, isTypeOnly);
                }
                // Import statement
                node.kind = 254 /* ImportDeclaration */;
                // ImportDeclaration:
                //  import ImportClause from ModuleSpecifier ;
                //  import ModuleSpecifier;
                if (identifier || // import id
                    token() === 41 /* AsteriskToken */ || // import *
                    token() === 18 /* OpenBraceToken */ // import {
                ) {
                    node.importClause = parseImportClause(identifier, afterImportPos, isTypeOnly);
                    parseExpected(149 /* FromKeyword */);
                }
                node.moduleSpecifier = parseModuleSpecifier();
                parseSemicolon();
                return finishNode(node);
            }
            function tokenAfterImportDefinitelyProducesImportDeclaration() {
                return token() === 41 /* AsteriskToken */ || token() === 18 /* OpenBraceToken */;
            }
            function tokenAfterImportedIdentifierDefinitelyProducesImportDeclaration() {
                // In `import id ___`, the current token decides whether to produce
                // an ImportDeclaration or ImportEqualsDeclaration.
                return token() === 27 /* CommaToken */ || token() === 149 /* FromKeyword */;
            }
            function parseImportEqualsDeclaration(node, identifier, isTypeOnly) {
                node.kind = 253 /* ImportEqualsDeclaration */;
                node.name = identifier;
                parseExpected(62 /* EqualsToken */);
                node.moduleReference = parseModuleReference();
                parseSemicolon();
                var finished = finishNode(node);
                if (isTypeOnly) {
                    parseErrorAtRange(finished, ts.Diagnostics.Only_ECMAScript_imports_may_use_import_type);
                }
                return finished;
            }
            function parseImportClause(identifier, fullStart, isTypeOnly) {
                // ImportClause:
                //  ImportedDefaultBinding
                //  NameSpaceImport
                //  NamedImports
                //  ImportedDefaultBinding, NameSpaceImport
                //  ImportedDefaultBinding, NamedImports
                var importClause = createNode(255 /* ImportClause */, fullStart);
                importClause.isTypeOnly = isTypeOnly;
                if (identifier) {
                    // ImportedDefaultBinding:
                    //  ImportedBinding
                    importClause.name = identifier;
                }
                // If there was no default import or if there is comma token after default import
                // parse namespace or named imports
                if (!importClause.name ||
                    parseOptional(27 /* CommaToken */)) {
                    importClause.namedBindings = token() === 41 /* AsteriskToken */ ? parseNamespaceImport() : parseNamedImportsOrExports(257 /* NamedImports */);
                }
                return finishNode(importClause);
            }
            function parseModuleReference() {
                return isExternalModuleReference()
                    ? parseExternalModuleReference()
                    : parseEntityName(/*allowReservedWords*/ false);
            }
            function parseExternalModuleReference() {
                var node = createNode(265 /* ExternalModuleReference */);
                parseExpected(139 /* RequireKeyword */);
                parseExpected(20 /* OpenParenToken */);
                node.expression = parseModuleSpecifier();
                parseExpected(21 /* CloseParenToken */);
                return finishNode(node);
            }
            function parseModuleSpecifier() {
                if (token() === 10 /* StringLiteral */) {
                    var result = parseLiteralNode();
                    result.text = internIdentifier(result.text);
                    return result;
                }
                else {
                    // We allow arbitrary expressions here, even though the grammar only allows string
                    // literals.  We check to ensure that it is only a string literal later in the grammar
                    // check pass.
                    return parseExpression();
                }
            }
            function parseNamespaceImport() {
                // NameSpaceImport:
                //  * as ImportedBinding
                var namespaceImport = createNode(256 /* NamespaceImport */);
                parseExpected(41 /* AsteriskToken */);
                parseExpected(123 /* AsKeyword */);
                namespaceImport.name = parseIdentifier();
                return finishNode(namespaceImport);
            }
            function parseNamedImportsOrExports(kind) {
                var node = createNode(kind);
                // NamedImports:
                //  { }
                //  { ImportsList }
                //  { ImportsList, }
                // ImportsList:
                //  ImportSpecifier
                //  ImportsList, ImportSpecifier
                node.elements = parseBracketedList(23 /* ImportOrExportSpecifiers */, kind === 257 /* NamedImports */ ? parseImportSpecifier : parseExportSpecifier, 18 /* OpenBraceToken */, 19 /* CloseBraceToken */);
                return finishNode(node);
            }
            function parseExportSpecifier() {
                return parseImportOrExportSpecifier(263 /* ExportSpecifier */);
            }
            function parseImportSpecifier() {
                return parseImportOrExportSpecifier(258 /* ImportSpecifier */);
            }
            function parseImportOrExportSpecifier(kind) {
                var node = createNode(kind);
                // ImportSpecifier:
                //   BindingIdentifier
                //   IdentifierName as BindingIdentifier
                // ExportSpecifier:
                //   IdentifierName
                //   IdentifierName as IdentifierName
                var checkIdentifierIsKeyword = ts.isKeyword(token()) && !isIdentifier();
                var checkIdentifierStart = scanner.getTokenPos();
                var checkIdentifierEnd = scanner.getTextPos();
                var identifierName = parseIdentifierName();
                if (token() === 123 /* AsKeyword */) {
                    node.propertyName = identifierName;
                    parseExpected(123 /* AsKeyword */);
                    checkIdentifierIsKeyword = ts.isKeyword(token()) && !isIdentifier();
                    checkIdentifierStart = scanner.getTokenPos();
                    checkIdentifierEnd = scanner.getTextPos();
                    node.name = parseIdentifierName();
                }
                else {
                    node.name = identifierName;
                }
                if (kind === 258 /* ImportSpecifier */ && checkIdentifierIsKeyword) {
                    parseErrorAt(checkIdentifierStart, checkIdentifierEnd, ts.Diagnostics.Identifier_expected);
                }
                return finishNode(node);
            }
            function parseNamespaceExport(pos) {
                var node = createNode(262 /* NamespaceExport */, pos);
                node.name = parseIdentifier();
                return finishNode(node);
            }
            function parseExportDeclaration(node) {
                node.kind = 260 /* ExportDeclaration */;
                node.isTypeOnly = parseOptional(145 /* TypeKeyword */);
                var namespaceExportPos = scanner.getStartPos();
                if (parseOptional(41 /* AsteriskToken */)) {
                    if (parseOptional(123 /* AsKeyword */)) {
                        node.exportClause = parseNamespaceExport(namespaceExportPos);
                    }
                    parseExpected(149 /* FromKeyword */);
                    node.moduleSpecifier = parseModuleSpecifier();
                }
                else {
                    node.exportClause = parseNamedImportsOrExports(261 /* NamedExports */);
                    // It is not uncommon to accidentally omit the 'from' keyword. Additionally, in editing scenarios,
                    // the 'from' keyword can be parsed as a named export when the export clause is unterminated (i.e. `export { from "moduleName";`)
                    // If we don't have a 'from' keyword, see if we have a string literal such that ASI won't take effect.
                    if (token() === 149 /* FromKeyword */ || (token() === 10 /* StringLiteral */ && !scanner.hasPrecedingLineBreak())) {
                        parseExpected(149 /* FromKeyword */);
                        node.moduleSpecifier = parseModuleSpecifier();
                    }
                }
                parseSemicolon();
                return finishNode(node);
            }
            function parseExportAssignment(node) {
                node.kind = 259 /* ExportAssignment */;
                if (parseOptional(62 /* EqualsToken */)) {
                    node.isExportEquals = true;
                }
                else {
                    parseExpected(84 /* DefaultKeyword */);
                }
                node.expression = parseAssignmentExpressionOrHigher();
                parseSemicolon();
                return finishNode(node);
            }
            function setExternalModuleIndicator(sourceFile) {
                // Try to use the first top-level import/export when available, then
                // fall back to looking for an 'import.meta' somewhere in the tree if necessary.
                sourceFile.externalModuleIndicator =
                    ts.forEach(sourceFile.statements, isAnExternalModuleIndicatorNode) ||
                        getImportMetaIfNecessary(sourceFile);
            }
            function isAnExternalModuleIndicatorNode(node) {
                return hasModifierOfKind(node, 89 /* ExportKeyword */)
                    || node.kind === 253 /* ImportEqualsDeclaration */ && node.moduleReference.kind === 265 /* ExternalModuleReference */
                    || node.kind === 254 /* ImportDeclaration */
                    || node.kind === 259 /* ExportAssignment */
                    || node.kind === 260 /* ExportDeclaration */ ? node : undefined;
            }
            function getImportMetaIfNecessary(sourceFile) {
                return sourceFile.flags & 2097152 /* PossiblyContainsImportMeta */ ?
                    walkTreeForExternalModuleIndicators(sourceFile) :
                    undefined;
            }
            function walkTreeForExternalModuleIndicators(node) {
                return isImportMeta(node) ? node : forEachChild(node, walkTreeForExternalModuleIndicators);
            }
            /** Do not use hasModifier inside the parser; it relies on parent pointers. Use this instead. */
            function hasModifierOfKind(node, kind) {
                return ts.some(node.modifiers, function (m) { return m.kind === kind; });
            }
            function isImportMeta(node) {
                return ts.isMetaProperty(node) && node.keywordToken === 96 /* ImportKeyword */ && node.name.escapedText === "meta";
            }
            var ParsingContext;
            (function (ParsingContext) {
                ParsingContext[ParsingContext["SourceElements"] = 0] = "SourceElements";
                ParsingContext[ParsingContext["BlockStatements"] = 1] = "BlockStatements";
                ParsingContext[ParsingContext["SwitchClauses"] = 2] = "SwitchClauses";
                ParsingContext[ParsingContext["SwitchClauseStatements"] = 3] = "SwitchClauseStatements";
                ParsingContext[ParsingContext["TypeMembers"] = 4] = "TypeMembers";
                ParsingContext[ParsingContext["ClassMembers"] = 5] = "ClassMembers";
                ParsingContext[ParsingContext["EnumMembers"] = 6] = "EnumMembers";
                ParsingContext[ParsingContext["HeritageClauseElement"] = 7] = "HeritageClauseElement";
                ParsingContext[ParsingContext["VariableDeclarations"] = 8] = "VariableDeclarations";
                ParsingContext[ParsingContext["ObjectBindingElements"] = 9] = "ObjectBindingElements";
                ParsingContext[ParsingContext["ArrayBindingElements"] = 10] = "ArrayBindingElements";
                ParsingContext[ParsingContext["ArgumentExpressions"] = 11] = "ArgumentExpressions";
                ParsingContext[ParsingContext["ObjectLiteralMembers"] = 12] = "ObjectLiteralMembers";
                ParsingContext[ParsingContext["JsxAttributes"] = 13] = "JsxAttributes";
                ParsingContext[ParsingContext["JsxChildren"] = 14] = "JsxChildren";
                ParsingContext[ParsingContext["ArrayLiteralMembers"] = 15] = "ArrayLiteralMembers";
                ParsingContext[ParsingContext["Parameters"] = 16] = "Parameters";
                ParsingContext[ParsingContext["JSDocParameters"] = 17] = "JSDocParameters";
                ParsingContext[ParsingContext["RestProperties"] = 18] = "RestProperties";
                ParsingContext[ParsingContext["TypeParameters"] = 19] = "TypeParameters";
                ParsingContext[ParsingContext["TypeArguments"] = 20] = "TypeArguments";
                ParsingContext[ParsingContext["TupleElementTypes"] = 21] = "TupleElementTypes";
                ParsingContext[ParsingContext["HeritageClauses"] = 22] = "HeritageClauses";
                ParsingContext[ParsingContext["ImportOrExportSpecifiers"] = 23] = "ImportOrExportSpecifiers";
                ParsingContext[ParsingContext["Count"] = 24] = "Count"; // Number of parsing contexts
            })(ParsingContext || (ParsingContext = {}));
            var Tristate;
            (function (Tristate) {
                Tristate[Tristate["False"] = 0] = "False";
                Tristate[Tristate["True"] = 1] = "True";
                Tristate[Tristate["Unknown"] = 2] = "Unknown";
            })(Tristate || (Tristate = {}));
            var JSDocParser;
            (function (JSDocParser) {
                function parseJSDocTypeExpressionForTests(content, start, length) {
                    initializeState(content, 99 /* Latest */, /*_syntaxCursor:*/ undefined, 1 /* JS */);
                    sourceFile = createSourceFile("file.js", 99 /* Latest */, 1 /* JS */, /*isDeclarationFile*/ false);
                    scanner.setText(content, start, length);
                    currentToken = scanner.scan();
                    var jsDocTypeExpression = parseJSDocTypeExpression();
                    var diagnostics = parseDiagnostics;
                    clearState();
                    return jsDocTypeExpression ? { jsDocTypeExpression: jsDocTypeExpression, diagnostics: diagnostics } : undefined;
                }
                JSDocParser.parseJSDocTypeExpressionForTests = parseJSDocTypeExpressionForTests;
                // Parses out a JSDoc type expression.
                function parseJSDocTypeExpression(mayOmitBraces) {
                    var result = createNode(294 /* JSDocTypeExpression */);
                    var hasBrace = (mayOmitBraces ? parseOptional : parseExpected)(18 /* OpenBraceToken */);
                    result.type = doInsideOfContext(4194304 /* JSDoc */, parseJSDocType);
                    if (!mayOmitBraces || hasBrace) {
                        parseExpectedJSDoc(19 /* CloseBraceToken */);
                    }
                    fixupParentReferences(result);
                    return finishNode(result);
                }
                JSDocParser.parseJSDocTypeExpression = parseJSDocTypeExpression;
                function parseIsolatedJSDocComment(content, start, length) {
                    initializeState(content, 99 /* Latest */, /*_syntaxCursor:*/ undefined, 1 /* JS */);
                    sourceFile = { languageVariant: 0 /* Standard */, text: content };
                    var jsDoc = doInsideOfContext(4194304 /* JSDoc */, function () { return parseJSDocCommentWorker(start, length); });
                    var diagnostics = parseDiagnostics;
                    clearState();
                    return jsDoc ? { jsDoc: jsDoc, diagnostics: diagnostics } : undefined;
                }
                JSDocParser.parseIsolatedJSDocComment = parseIsolatedJSDocComment;
                function parseJSDocComment(parent, start, length) {
                    var _a;
                    var saveToken = currentToken;
                    var saveParseDiagnosticsLength = parseDiagnostics.length;
                    var saveParseErrorBeforeNextFinishedNode = parseErrorBeforeNextFinishedNode;
                    var comment = doInsideOfContext(4194304 /* JSDoc */, function () { return parseJSDocCommentWorker(start, length); });
                    if (comment) {
                        comment.parent = parent;
                    }
                    if (contextFlags & 131072 /* JavaScriptFile */) {
                        if (!sourceFile.jsDocDiagnostics) {
                            sourceFile.jsDocDiagnostics = [];
                        }
                        (_a = sourceFile.jsDocDiagnostics).push.apply(_a, parseDiagnostics);
                    }
                    currentToken = saveToken;
                    parseDiagnostics.length = saveParseDiagnosticsLength;
                    parseErrorBeforeNextFinishedNode = saveParseErrorBeforeNextFinishedNode;
                    return comment;
                }
                JSDocParser.parseJSDocComment = parseJSDocComment;
                var JSDocState;
                (function (JSDocState) {
                    JSDocState[JSDocState["BeginningOfLine"] = 0] = "BeginningOfLine";
                    JSDocState[JSDocState["SawAsterisk"] = 1] = "SawAsterisk";
                    JSDocState[JSDocState["SavingComments"] = 2] = "SavingComments";
                    JSDocState[JSDocState["SavingBackticks"] = 3] = "SavingBackticks";
                })(JSDocState || (JSDocState = {}));
                var PropertyLikeParse;
                (function (PropertyLikeParse) {
                    PropertyLikeParse[PropertyLikeParse["Property"] = 1] = "Property";
                    PropertyLikeParse[PropertyLikeParse["Parameter"] = 2] = "Parameter";
                    PropertyLikeParse[PropertyLikeParse["CallbackParameter"] = 4] = "CallbackParameter";
                })(PropertyLikeParse || (PropertyLikeParse = {}));
                function parseJSDocCommentWorker(start, length) {
                    if (start === void 0) {
                        start = 0;
                    }
                    var content = sourceText;
                    var end = length === undefined ? content.length : start + length;
                    length = end - start;
                    ts.Debug.assert(start >= 0);
                    ts.Debug.assert(start <= end);
                    ts.Debug.assert(end <= content.length);
                    // Check for /** (JSDoc opening part)
                    if (!isJSDocLikeText(content, start)) {
                        return undefined;
                    }
                    var tags;
                    var tagsPos;
                    var tagsEnd;
                    var comments = [];
                    // + 3 for leading /**, - 5 in total for /** */
                    return scanner.scanRange(start + 3, length - 5, function () {
                        // Initially we can parse out a tag.  We also have seen a starting asterisk.
                        // This is so that /** * @type */ doesn't parse.
                        var state = 1 /* SawAsterisk */;
                        var margin;
                        // + 4 for leading '/** '
                        var indent = start - Math.max(content.lastIndexOf("\n", start), 0) + 4;
                        function pushComment(text) {
                            if (!margin) {
                                margin = indent;
                            }
                            comments.push(text);
                            indent += text.length;
                        }
                        nextTokenJSDoc();
                        while (parseOptionalJsdoc(5 /* WhitespaceTrivia */))
                            ;
                        if (parseOptionalJsdoc(4 /* NewLineTrivia */)) {
                            state = 0 /* BeginningOfLine */;
                            indent = 0;
                        }
                        loop: while (true) {
                            switch (token()) {
                                case 59 /* AtToken */:
                                    if (state === 0 /* BeginningOfLine */ || state === 1 /* SawAsterisk */) {
                                        removeTrailingWhitespace(comments);
                                        addTag(parseTag(indent));
                                        // NOTE: According to usejsdoc.org, a tag goes to end of line, except the last tag.
                                        // Real-world comments may break this rule, so "BeginningOfLine" will not be a real line beginning
                                        // for malformed examples like `/** @param {string} x @returns {number} the length */`
                                        state = 0 /* BeginningOfLine */;
                                        margin = undefined;
                                    }
                                    else {
                                        pushComment(scanner.getTokenText());
                                    }
                                    break;
                                case 4 /* NewLineTrivia */:
                                    comments.push(scanner.getTokenText());
                                    state = 0 /* BeginningOfLine */;
                                    indent = 0;
                                    break;
                                case 41 /* AsteriskToken */:
                                    var asterisk = scanner.getTokenText();
                                    if (state === 1 /* SawAsterisk */ || state === 2 /* SavingComments */) {
                                        // If we've already seen an asterisk, then we can no longer parse a tag on this line
                                        state = 2 /* SavingComments */;
                                        pushComment(asterisk);
                                    }
                                    else {
                                        // Ignore the first asterisk on a line
                                        state = 1 /* SawAsterisk */;
                                        indent += asterisk.length;
                                    }
                                    break;
                                case 5 /* WhitespaceTrivia */:
                                    // only collect whitespace if we're already saving comments or have just crossed the comment indent margin
                                    var whitespace = scanner.getTokenText();
                                    if (state === 2 /* SavingComments */) {
                                        comments.push(whitespace);
                                    }
                                    else if (margin !== undefined && indent + whitespace.length > margin) {
                                        comments.push(whitespace.slice(margin - indent - 1));
                                    }
                                    indent += whitespace.length;
                                    break;
                                case 1 /* EndOfFileToken */:
                                    break loop;
                                default:
                                    // Anything else is doc comment text. We just save it. Because it
                                    // wasn't a tag, we can no longer parse a tag on this line until we hit the next
                                    // line break.
                                    state = 2 /* SavingComments */;
                                    pushComment(scanner.getTokenText());
                                    break;
                            }
                            nextTokenJSDoc();
                        }
                        removeLeadingNewlines(comments);
                        removeTrailingWhitespace(comments);
                        return createJSDocComment();
                    });
                    function removeLeadingNewlines(comments) {
                        while (comments.length && (comments[0] === "\n" || comments[0] === "\r")) {
                            comments.shift();
                        }
                    }
                    function removeTrailingWhitespace(comments) {
                        while (comments.length && comments[comments.length - 1].trim() === "") {
                            comments.pop();
                        }
                    }
                    function createJSDocComment() {
                        var result = createNode(303 /* JSDocComment */, start);
                        result.tags = tags && createNodeArray(tags, tagsPos, tagsEnd);
                        result.comment = comments.length ? comments.join("") : undefined;
                        return finishNode(result, end);
                    }
                    function isNextNonwhitespaceTokenEndOfFile() {
                        // We must use infinite lookahead, as there could be any number of newlines :(
                        while (true) {
                            nextTokenJSDoc();
                            if (token() === 1 /* EndOfFileToken */) {
                                return true;
                            }
                            if (!(token() === 5 /* WhitespaceTrivia */ || token() === 4 /* NewLineTrivia */)) {
                                return false;
                            }
                        }
                    }
                    function skipWhitespace() {
                        if (token() === 5 /* WhitespaceTrivia */ || token() === 4 /* NewLineTrivia */) {
                            if (lookAhead(isNextNonwhitespaceTokenEndOfFile)) {
                                return; // Don't skip whitespace prior to EoF (or end of comment) - that shouldn't be included in any node's range
                            }
                        }
                        while (token() === 5 /* WhitespaceTrivia */ || token() === 4 /* NewLineTrivia */) {
                            nextTokenJSDoc();
                        }
                    }
                    function skipWhitespaceOrAsterisk() {
                        if (token() === 5 /* WhitespaceTrivia */ || token() === 4 /* NewLineTrivia */) {
                            if (lookAhead(isNextNonwhitespaceTokenEndOfFile)) {
                                return ""; // Don't skip whitespace prior to EoF (or end of comment) - that shouldn't be included in any node's range
                            }
                        }
                        var precedingLineBreak = scanner.hasPrecedingLineBreak();
                        var seenLineBreak = false;
                        var indentText = "";
                        while ((precedingLineBreak && token() === 41 /* AsteriskToken */) || token() === 5 /* WhitespaceTrivia */ || token() === 4 /* NewLineTrivia */) {
                            indentText += scanner.getTokenText();
                            if (token() === 4 /* NewLineTrivia */) {
                                precedingLineBreak = true;
                                seenLineBreak = true;
                                indentText = "";
                            }
                            else if (token() === 41 /* AsteriskToken */) {
                                precedingLineBreak = false;
                            }
                            nextTokenJSDoc();
                        }
                        return seenLineBreak ? indentText : "";
                    }
                    function parseTag(margin) {
                        ts.Debug.assert(token() === 59 /* AtToken */);
                        var start = scanner.getTokenPos();
                        nextTokenJSDoc();
                        var tagName = parseJSDocIdentifierName(/*message*/ undefined);
                        var indentText = skipWhitespaceOrAsterisk();
                        var tag;
                        switch (tagName.escapedText) {
                            case "author":
                                tag = parseAuthorTag(start, tagName, margin);
                                break;
                            case "augments":
                            case "extends":
                                tag = parseAugmentsTag(start, tagName);
                                break;
                            case "class":
                            case "constructor":
                                tag = parseSimpleTag(start, 309 /* JSDocClassTag */, tagName);
                                break;
                            case "public":
                                tag = parseSimpleTag(start, 310 /* JSDocPublicTag */, tagName);
                                break;
                            case "private":
                                tag = parseSimpleTag(start, 311 /* JSDocPrivateTag */, tagName);
                                break;
                            case "protected":
                                tag = parseSimpleTag(start, 312 /* JSDocProtectedTag */, tagName);
                                break;
                            case "readonly":
                                tag = parseSimpleTag(start, 313 /* JSDocReadonlyTag */, tagName);
                                break;
                            case "this":
                                tag = parseThisTag(start, tagName);
                                break;
                            case "enum":
                                tag = parseEnumTag(start, tagName);
                                break;
                            case "arg":
                            case "argument":
                            case "param":
                                return parseParameterOrPropertyTag(start, tagName, 2 /* Parameter */, margin);
                            case "return":
                            case "returns":
                                tag = parseReturnTag(start, tagName);
                                break;
                            case "template":
                                tag = parseTemplateTag(start, tagName);
                                break;
                            case "type":
                                tag = parseTypeTag(start, tagName);
                                break;
                            case "typedef":
                                tag = parseTypedefTag(start, tagName, margin);
                                break;
                            case "callback":
                                tag = parseCallbackTag(start, tagName, margin);
                                break;
                            default:
                                tag = parseUnknownTag(start, tagName);
                                break;
                        }
                        if (!tag.comment) {
                            // some tags, like typedef and callback, have already parsed their comments earlier
                            if (!indentText) {
                                margin += tag.end - tag.pos;
                            }
                            tag.comment = parseTagComments(margin, indentText.slice(margin));
                        }
                        return tag;
                    }
                    function parseTagComments(indent, initialMargin) {
                        var comments = [];
                        var state = 0 /* BeginningOfLine */;
                        var margin;
                        function pushComment(text) {
                            if (!margin) {
                                margin = indent;
                            }
                            comments.push(text);
                            indent += text.length;
                        }
                        if (initialMargin) {
                            // jump straight to saving comments if there is some initial indentation
                            pushComment(initialMargin);
                            state = 2 /* SavingComments */;
                        }
                        var tok = token();
                        loop: while (true) {
                            switch (tok) {
                                case 4 /* NewLineTrivia */:
                                    if (state >= 1 /* SawAsterisk */) {
                                        state = 0 /* BeginningOfLine */;
                                        // don't use pushComment here because we want to keep the margin unchanged
                                        comments.push(scanner.getTokenText());
                                    }
                                    indent = 0;
                                    break;
                                case 59 /* AtToken */:
                                    if (state === 3 /* SavingBackticks */) {
                                        comments.push(scanner.getTokenText());
                                        break;
                                    }
                                    scanner.setTextPos(scanner.getTextPos() - 1);
                                // falls through
                                case 1 /* EndOfFileToken */:
                                    // Done
                                    break loop;
                                case 5 /* WhitespaceTrivia */:
                                    if (state === 2 /* SavingComments */ || state === 3 /* SavingBackticks */) {
                                        pushComment(scanner.getTokenText());
                                    }
                                    else {
                                        var whitespace = scanner.getTokenText();
                                        // if the whitespace crosses the margin, take only the whitespace that passes the margin
                                        if (margin !== undefined && indent + whitespace.length > margin) {
                                            comments.push(whitespace.slice(margin - indent));
                                        }
                                        indent += whitespace.length;
                                    }
                                    break;
                                case 18 /* OpenBraceToken */:
                                    state = 2 /* SavingComments */;
                                    if (lookAhead(function () { return nextTokenJSDoc() === 59 /* AtToken */ && ts.tokenIsIdentifierOrKeyword(nextTokenJSDoc()) && scanner.getTokenText() === "link"; })) {
                                        pushComment(scanner.getTokenText());
                                        nextTokenJSDoc();
                                        pushComment(scanner.getTokenText());
                                        nextTokenJSDoc();
                                    }
                                    pushComment(scanner.getTokenText());
                                    break;
                                case 61 /* BacktickToken */:
                                    if (state === 3 /* SavingBackticks */) {
                                        state = 2 /* SavingComments */;
                                    }
                                    else {
                                        state = 3 /* SavingBackticks */;
                                    }
                                    pushComment(scanner.getTokenText());
                                    break;
                                case 41 /* AsteriskToken */:
                                    if (state === 0 /* BeginningOfLine */) {
                                        // leading asterisks start recording on the *next* (non-whitespace) token
                                        state = 1 /* SawAsterisk */;
                                        indent += 1;
                                        break;
                                    }
                                // record the * as a comment
                                // falls through
                                default:
                                    if (state !== 3 /* SavingBackticks */) {
                                        state = 2 /* SavingComments */; // leading identifiers start recording as well
                                    }
                                    pushComment(scanner.getTokenText());
                                    break;
                            }
                            tok = nextTokenJSDoc();
                        }
                        removeLeadingNewlines(comments);
                        removeTrailingWhitespace(comments);
                        return comments.length === 0 ? undefined : comments.join("");
                    }
                    function parseUnknownTag(start, tagName) {
                        var result = createNode(306 /* JSDocTag */, start);
                        result.tagName = tagName;
                        return finishNode(result);
                    }
                    function addTag(tag) {
                        if (!tag) {
                            return;
                        }
                        if (!tags) {
                            tags = [tag];
                            tagsPos = tag.pos;
                        }
                        else {
                            tags.push(tag);
                        }
                        tagsEnd = tag.end;
                    }
                    function tryParseTypeExpression() {
                        skipWhitespaceOrAsterisk();
                        return token() === 18 /* OpenBraceToken */ ? parseJSDocTypeExpression() : undefined;
                    }
                    function parseBracketNameInPropertyAndParamTag() {
                        // Looking for something like '[foo]', 'foo', '[foo.bar]' or 'foo.bar'
                        var isBracketed = parseOptionalJsdoc(22 /* OpenBracketToken */);
                        if (isBracketed) {
                            skipWhitespace();
                        }
                        // a markdown-quoted name: `arg` is not legal jsdoc, but occurs in the wild
                        var isBackquoted = parseOptionalJsdoc(61 /* BacktickToken */);
                        var name = parseJSDocEntityName();
                        if (isBackquoted) {
                            parseExpectedTokenJSDoc(61 /* BacktickToken */);
                        }
                        if (isBracketed) {
                            skipWhitespace();
                            // May have an optional default, e.g. '[foo = 42]'
                            if (parseOptionalToken(62 /* EqualsToken */)) {
                                parseExpression();
                            }
                            parseExpected(23 /* CloseBracketToken */);
                        }
                        return { name: name, isBracketed: isBracketed };
                    }
                    function isObjectOrObjectArrayTypeReference(node) {
                        switch (node.kind) {
                            case 141 /* ObjectKeyword */:
                                return true;
                            case 174 /* ArrayType */:
                                return isObjectOrObjectArrayTypeReference(node.elementType);
                            default:
                                return ts.isTypeReferenceNode(node) && ts.isIdentifier(node.typeName) && node.typeName.escapedText === "Object" && !node.typeArguments;
                        }
                    }
                    function parseParameterOrPropertyTag(start, tagName, target, indent) {
                        var typeExpression = tryParseTypeExpression();
                        var isNameFirst = !typeExpression;
                        skipWhitespaceOrAsterisk();
                        var _a = parseBracketNameInPropertyAndParamTag(), name = _a.name, isBracketed = _a.isBracketed;
                        skipWhitespace();
                        if (isNameFirst) {
                            typeExpression = tryParseTypeExpression();
                        }
                        var result = target === 1 /* Property */ ?
                            createNode(322 /* JSDocPropertyTag */, start) :
                            createNode(316 /* JSDocParameterTag */, start);
                        var comment = parseTagComments(indent + scanner.getStartPos() - start);
                        var nestedTypeLiteral = target !== 4 /* CallbackParameter */ && parseNestedTypeLiteral(typeExpression, name, target, indent);
                        if (nestedTypeLiteral) {
                            typeExpression = nestedTypeLiteral;
                            isNameFirst = true;
                        }
                        result.tagName = tagName;
                        result.typeExpression = typeExpression;
                        result.name = name;
                        result.isNameFirst = isNameFirst;
                        result.isBracketed = isBracketed;
                        result.comment = comment;
                        return finishNode(result);
                    }
                    function parseNestedTypeLiteral(typeExpression, name, target, indent) {
                        if (typeExpression && isObjectOrObjectArrayTypeReference(typeExpression.type)) {
                            var typeLiteralExpression = createNode(294 /* JSDocTypeExpression */, scanner.getTokenPos());
                            var child = void 0;
                            var jsdocTypeLiteral = void 0;
                            var start_3 = scanner.getStartPos();
                            var children = void 0;
                            while (child = tryParse(function () { return parseChildParameterOrPropertyTag(target, indent, name); })) {
                                if (child.kind === 316 /* JSDocParameterTag */ || child.kind === 322 /* JSDocPropertyTag */) {
                                    children = ts.append(children, child);
                                }
                            }
                            if (children) {
                                jsdocTypeLiteral = createNode(304 /* JSDocTypeLiteral */, start_3);
                                jsdocTypeLiteral.jsDocPropertyTags = children;
                                if (typeExpression.type.kind === 174 /* ArrayType */) {
                                    jsdocTypeLiteral.isArrayType = true;
                                }
                                typeLiteralExpression.type = finishNode(jsdocTypeLiteral);
                                return finishNode(typeLiteralExpression);
                            }
                        }
                    }
                    function parseReturnTag(start, tagName) {
                        if (ts.some(tags, ts.isJSDocReturnTag)) {
                            parseErrorAt(tagName.pos, scanner.getTokenPos(), ts.Diagnostics._0_tag_already_specified, tagName.escapedText);
                        }
                        var result = createNode(317 /* JSDocReturnTag */, start);
                        result.tagName = tagName;
                        result.typeExpression = tryParseTypeExpression();
                        return finishNode(result);
                    }
                    function parseTypeTag(start, tagName) {
                        if (ts.some(tags, ts.isJSDocTypeTag)) {
                            parseErrorAt(tagName.pos, scanner.getTokenPos(), ts.Diagnostics._0_tag_already_specified, tagName.escapedText);
                        }
                        var result = createNode(319 /* JSDocTypeTag */, start);
                        result.tagName = tagName;
                        result.typeExpression = parseJSDocTypeExpression(/*mayOmitBraces*/ true);
                        return finishNode(result);
                    }
                    function parseAuthorTag(start, tagName, indent) {
                        var result = createNode(308 /* JSDocAuthorTag */, start);
                        result.tagName = tagName;
                        var authorInfoWithEmail = tryParse(function () { return tryParseAuthorNameAndEmail(); });
                        if (!authorInfoWithEmail) {
                            return finishNode(result);
                        }
                        result.comment = authorInfoWithEmail;
                        if (lookAhead(function () { return nextToken() !== 4 /* NewLineTrivia */; })) {
                            var comment = parseTagComments(indent);
                            if (comment) {
                                result.comment += comment;
                            }
                        }
                        return finishNode(result);
                    }
                    function tryParseAuthorNameAndEmail() {
                        var comments = [];
                        var seenLessThan = false;
                        var seenGreaterThan = false;
                        var token = scanner.getToken();
                        loop: while (true) {
                            switch (token) {
                                case 75 /* Identifier */:
                                case 5 /* WhitespaceTrivia */:
                                case 24 /* DotToken */:
                                case 59 /* AtToken */:
                                    comments.push(scanner.getTokenText());
                                    break;
                                case 29 /* LessThanToken */:
                                    if (seenLessThan || seenGreaterThan) {
                                        return;
                                    }
                                    seenLessThan = true;
                                    comments.push(scanner.getTokenText());
                                    break;
                                case 31 /* GreaterThanToken */:
                                    if (!seenLessThan || seenGreaterThan) {
                                        return;
                                    }
                                    seenGreaterThan = true;
                                    comments.push(scanner.getTokenText());
                                    scanner.setTextPos(scanner.getTokenPos() + 1);
                                    break loop;
                                case 4 /* NewLineTrivia */:
                                case 1 /* EndOfFileToken */:
                                    break loop;
                            }
                            token = nextTokenJSDoc();
                        }
                        if (seenLessThan && seenGreaterThan) {
                            return comments.length === 0 ? undefined : comments.join("");
                        }
                    }
                    function parseAugmentsTag(start, tagName) {
                        var result = createNode(307 /* JSDocAugmentsTag */, start);
                        result.tagName = tagName;
                        result.class = parseExpressionWithTypeArgumentsForAugments();
                        return finishNode(result);
                    }
                    function parseExpressionWithTypeArgumentsForAugments() {
                        var usedBrace = parseOptional(18 /* OpenBraceToken */);
                        var node = createNode(216 /* ExpressionWithTypeArguments */);
                        node.expression = parsePropertyAccessEntityNameExpression();
                        node.typeArguments = tryParseTypeArguments();
                        var res = finishNode(node);
                        if (usedBrace) {
                            parseExpected(19 /* CloseBraceToken */);
                        }
                        return res;
                    }
                    function parsePropertyAccessEntityNameExpression() {
                        var node = parseJSDocIdentifierName();
                        while (parseOptional(24 /* DotToken */)) {
                            var prop = createNode(194 /* PropertyAccessExpression */, node.pos);
                            prop.expression = node;
                            prop.name = parseJSDocIdentifierName();
                            node = finishNode(prop);
                        }
                        return node;
                    }
                    function parseSimpleTag(start, kind, tagName) {
                        var tag = createNode(kind, start);
                        tag.tagName = tagName;
                        return finishNode(tag);
                    }
                    function parseThisTag(start, tagName) {
                        var tag = createNode(318 /* JSDocThisTag */, start);
                        tag.tagName = tagName;
                        tag.typeExpression = parseJSDocTypeExpression(/*mayOmitBraces*/ true);
                        skipWhitespace();
                        return finishNode(tag);
                    }
                    function parseEnumTag(start, tagName) {
                        var tag = createNode(315 /* JSDocEnumTag */, start);
                        tag.tagName = tagName;
                        tag.typeExpression = parseJSDocTypeExpression(/*mayOmitBraces*/ true);
                        skipWhitespace();
                        return finishNode(tag);
                    }
                    function parseTypedefTag(start, tagName, indent) {
                        var typeExpression = tryParseTypeExpression();
                        skipWhitespaceOrAsterisk();
                        var typedefTag = createNode(321 /* JSDocTypedefTag */, start);
                        typedefTag.tagName = tagName;
                        typedefTag.fullName = parseJSDocTypeNameWithNamespace();
                        typedefTag.name = getJSDocTypeAliasName(typedefTag.fullName);
                        skipWhitespace();
                        typedefTag.comment = parseTagComments(indent);
                        typedefTag.typeExpression = typeExpression;
                        var end;
                        if (!typeExpression || isObjectOrObjectArrayTypeReference(typeExpression.type)) {
                            var child = void 0;
                            var jsdocTypeLiteral = void 0;
                            var childTypeTag = void 0;
                            while (child = tryParse(function () { return parseChildPropertyTag(indent); })) {
                                if (!jsdocTypeLiteral) {
                                    jsdocTypeLiteral = createNode(304 /* JSDocTypeLiteral */, start);
                                }
                                if (child.kind === 319 /* JSDocTypeTag */) {
                                    if (childTypeTag) {
                                        break;
                                    }
                                    else {
                                        childTypeTag = child;
                                    }
                                }
                                else {
                                    jsdocTypeLiteral.jsDocPropertyTags = ts.append(jsdocTypeLiteral.jsDocPropertyTags, child);
                                }
                            }
                            if (jsdocTypeLiteral) {
                                if (typeExpression && typeExpression.type.kind === 174 /* ArrayType */) {
                                    jsdocTypeLiteral.isArrayType = true;
                                }
                                typedefTag.typeExpression = childTypeTag && childTypeTag.typeExpression && !isObjectOrObjectArrayTypeReference(childTypeTag.typeExpression.type) ?
                                    childTypeTag.typeExpression :
                                    finishNode(jsdocTypeLiteral);
                                end = typedefTag.typeExpression.end;
                            }
                        }
                        // Only include the characters between the name end and the next token if a comment was actually parsed out - otherwise it's just whitespace
                        return finishNode(typedefTag, end || typedefTag.comment !== undefined ? scanner.getStartPos() : (typedefTag.fullName || typedefTag.typeExpression || typedefTag.tagName).end);
                    }
                    function parseJSDocTypeNameWithNamespace(nested) {
                        var pos = scanner.getTokenPos();
                        if (!ts.tokenIsIdentifierOrKeyword(token())) {
                            return undefined;
                        }
                        var typeNameOrNamespaceName = parseJSDocIdentifierName();
                        if (parseOptional(24 /* DotToken */)) {
                            var jsDocNamespaceNode = createNode(249 /* ModuleDeclaration */, pos);
                            if (nested) {
                                jsDocNamespaceNode.flags |= 4 /* NestedNamespace */;
                            }
                            jsDocNamespaceNode.name = typeNameOrNamespaceName;
                            jsDocNamespaceNode.body = parseJSDocTypeNameWithNamespace(/*nested*/ true);
                            return finishNode(jsDocNamespaceNode);
                        }
                        if (nested) {
                            typeNameOrNamespaceName.isInJSDocNamespace = true;
                        }
                        return typeNameOrNamespaceName;
                    }
                    function parseCallbackTag(start, tagName, indent) {
                        var callbackTag = createNode(314 /* JSDocCallbackTag */, start);
                        callbackTag.tagName = tagName;
                        callbackTag.fullName = parseJSDocTypeNameWithNamespace();
                        callbackTag.name = getJSDocTypeAliasName(callbackTag.fullName);
                        skipWhitespace();
                        callbackTag.comment = parseTagComments(indent);
                        var child;
                        var jsdocSignature = createNode(305 /* JSDocSignature */, start);
                        jsdocSignature.parameters = [];
                        while (child = tryParse(function () { return parseChildParameterOrPropertyTag(4 /* CallbackParameter */, indent); })) {
                            jsdocSignature.parameters = ts.append(jsdocSignature.parameters, child);
                        }
                        var returnTag = tryParse(function () {
                            if (parseOptionalJsdoc(59 /* AtToken */)) {
                                var tag = parseTag(indent);
                                if (tag && tag.kind === 317 /* JSDocReturnTag */) {
                                    return tag;
                                }
                            }
                        });
                        if (returnTag) {
                            jsdocSignature.type = returnTag;
                        }
                        callbackTag.typeExpression = finishNode(jsdocSignature);
                        return finishNode(callbackTag);
                    }
                    function getJSDocTypeAliasName(fullName) {
                        if (fullName) {
                            var rightNode = fullName;
                            while (true) {
                                if (ts.isIdentifier(rightNode) || !rightNode.body) {
                                    return ts.isIdentifier(rightNode) ? rightNode : rightNode.name;
                                }
                                rightNode = rightNode.body;
                            }
                        }
                    }
                    function escapedTextsEqual(a, b) {
                        while (!ts.isIdentifier(a) || !ts.isIdentifier(b)) {
                            if (!ts.isIdentifier(a) && !ts.isIdentifier(b) && a.right.escapedText === b.right.escapedText) {
                                a = a.left;
                                b = b.left;
                            }
                            else {
                                return false;
                            }
                        }
                        return a.escapedText === b.escapedText;
                    }
                    function parseChildPropertyTag(indent) {
                        return parseChildParameterOrPropertyTag(1 /* Property */, indent);
                    }
                    function parseChildParameterOrPropertyTag(target, indent, name) {
                        var canParseTag = true;
                        var seenAsterisk = false;
                        while (true) {
                            switch (nextTokenJSDoc()) {
                                case 59 /* AtToken */:
                                    if (canParseTag) {
                                        var child = tryParseChildTag(target, indent);
                                        if (child && (child.kind === 316 /* JSDocParameterTag */ || child.kind === 322 /* JSDocPropertyTag */) &&
                                            target !== 4 /* CallbackParameter */ &&
                                            name && (ts.isIdentifier(child.name) || !escapedTextsEqual(name, child.name.left))) {
                                            return false;
                                        }
                                        return child;
                                    }
                                    seenAsterisk = false;
                                    break;
                                case 4 /* NewLineTrivia */:
                                    canParseTag = true;
                                    seenAsterisk = false;
                                    break;
                                case 41 /* AsteriskToken */:
                                    if (seenAsterisk) {
                                        canParseTag = false;
                                    }
                                    seenAsterisk = true;
                                    break;
                                case 75 /* Identifier */:
                                    canParseTag = false;
                                    break;
                                case 1 /* EndOfFileToken */:
                                    return false;
                            }
                        }
                    }
                    function tryParseChildTag(target, indent) {
                        ts.Debug.assert(token() === 59 /* AtToken */);
                        var start = scanner.getStartPos();
                        nextTokenJSDoc();
                        var tagName = parseJSDocIdentifierName();
                        skipWhitespace();
                        var t;
                        switch (tagName.escapedText) {
                            case "type":
                                return target === 1 /* Property */ && parseTypeTag(start, tagName);
                            case "prop":
                            case "property":
                                t = 1 /* Property */;
                                break;
                            case "arg":
                            case "argument":
                            case "param":
                                t = 2 /* Parameter */ | 4 /* CallbackParameter */;
                                break;
                            default:
                                return false;
                        }
                        if (!(target & t)) {
                            return false;
                        }
                        return parseParameterOrPropertyTag(start, tagName, target, indent);
                    }
                    function parseTemplateTag(start, tagName) {
                        // the template tag looks like '@template {Constraint} T,U,V'
                        var constraint;
                        if (token() === 18 /* OpenBraceToken */) {
                            constraint = parseJSDocTypeExpression();
                        }
                        var typeParameters = [];
                        var typeParametersPos = getNodePos();
                        do {
                            skipWhitespace();
                            var typeParameter = createNode(155 /* TypeParameter */);
                            typeParameter.name = parseJSDocIdentifierName(ts.Diagnostics.Unexpected_token_A_type_parameter_name_was_expected_without_curly_braces);
                            finishNode(typeParameter);
                            skipWhitespace();
                            typeParameters.push(typeParameter);
                        } while (parseOptionalJsdoc(27 /* CommaToken */));
                        var result = createNode(320 /* JSDocTemplateTag */, start);
                        result.tagName = tagName;
                        result.constraint = constraint;
                        result.typeParameters = createNodeArray(typeParameters, typeParametersPos);
                        finishNode(result);
                        return result;
                    }
                    function parseOptionalJsdoc(t) {
                        if (token() === t) {
                            nextTokenJSDoc();
                            return true;
                        }
                        return false;
                    }
                    function parseJSDocEntityName() {
                        var entity = parseJSDocIdentifierName();
                        if (parseOptional(22 /* OpenBracketToken */)) {
                            parseExpected(23 /* CloseBracketToken */);
                            // Note that y[] is accepted as an entity name, but the postfix brackets are not saved for checking.
                            // Technically usejsdoc.org requires them for specifying a property of a type equivalent to Array<{ x: ...}>
                            // but it's not worth it to enforce that restriction.
                        }
                        while (parseOptional(24 /* DotToken */)) {
                            var name = parseJSDocIdentifierName();
                            if (parseOptional(22 /* OpenBracketToken */)) {
                                parseExpected(23 /* CloseBracketToken */);
                            }
                            entity = createQualifiedName(entity, name);
                        }
                        return entity;
                    }
                    function parseJSDocIdentifierName(message) {
                        if (!ts.tokenIsIdentifierOrKeyword(token())) {
                            return createMissingNode(75 /* Identifier */, /*reportAtCurrentPosition*/ !message, message || ts.Diagnostics.Identifier_expected);
                        }
                        identifierCount++;
                        var pos = scanner.getTokenPos();
                        var end = scanner.getTextPos();
                        var result = createNode(75 /* Identifier */, pos);
                        if (token() !== 75 /* Identifier */) {
                            result.originalKeywordKind = token();
                        }
                        result.escapedText = ts.escapeLeadingUnderscores(internIdentifier(scanner.getTokenValue()));
                        finishNode(result, end);
                        nextTokenJSDoc();
                        return result;
                    }
                }
            })(JSDocParser = Parser.JSDocParser || (Parser.JSDocParser = {}));
        })(Parser || (Parser = {}));
        var IncrementalParser;
        (function (IncrementalParser) {
            function updateSourceFile(sourceFile, newText, textChangeRange, aggressiveChecks) {
                aggressiveChecks = aggressiveChecks || ts.Debug.shouldAssert(2 /* Aggressive */);
                checkChangeRange(sourceFile, newText, textChangeRange, aggressiveChecks);
                if (ts.textChangeRangeIsUnchanged(textChangeRange)) {
                    // if the text didn't change, then we can just return our current source file as-is.
                    return sourceFile;
                }
                if (sourceFile.statements.length === 0) {
                    // If we don't have any statements in the current source file, then there's no real
                    // way to incrementally parse.  So just do a full parse instead.
                    return Parser.parseSourceFile(sourceFile.fileName, newText, sourceFile.languageVersion, /*syntaxCursor*/ undefined, /*setParentNodes*/ true, sourceFile.scriptKind);
                }
                // Make sure we're not trying to incrementally update a source file more than once.  Once
                // we do an update the original source file is considered unusable from that point onwards.
                //
                // This is because we do incremental parsing in-place.  i.e. we take nodes from the old
                // tree and give them new positions and parents.  From that point on, trusting the old
                // tree at all is not possible as far too much of it may violate invariants.
                var incrementalSourceFile = sourceFile;
                ts.Debug.assert(!incrementalSourceFile.hasBeenIncrementallyParsed);
                incrementalSourceFile.hasBeenIncrementallyParsed = true;
                var oldText = sourceFile.text;
                var syntaxCursor = createSyntaxCursor(sourceFile);
                // Make the actual change larger so that we know to reparse anything whose lookahead
                // might have intersected the change.
                var changeRange = extendToAffectedRange(sourceFile, textChangeRange);
                checkChangeRange(sourceFile, newText, changeRange, aggressiveChecks);
                // Ensure that extending the affected range only moved the start of the change range
                // earlier in the file.
                ts.Debug.assert(changeRange.span.start <= textChangeRange.span.start);
                ts.Debug.assert(ts.textSpanEnd(changeRange.span) === ts.textSpanEnd(textChangeRange.span));
                ts.Debug.assert(ts.textSpanEnd(ts.textChangeRangeNewSpan(changeRange)) === ts.textSpanEnd(ts.textChangeRangeNewSpan(textChangeRange)));
                // The is the amount the nodes after the edit range need to be adjusted.  It can be
                // positive (if the edit added characters), negative (if the edit deleted characters)
                // or zero (if this was a pure overwrite with nothing added/removed).
                var delta = ts.textChangeRangeNewSpan(changeRange).length - changeRange.span.length;
                // If we added or removed characters during the edit, then we need to go and adjust all
                // the nodes after the edit.  Those nodes may move forward (if we inserted chars) or they
                // may move backward (if we deleted chars).
                //
                // Doing this helps us out in two ways.  First, it means that any nodes/tokens we want
                // to reuse are already at the appropriate position in the new text.  That way when we
                // reuse them, we don't have to figure out if they need to be adjusted.  Second, it makes
                // it very easy to determine if we can reuse a node.  If the node's position is at where
                // we are in the text, then we can reuse it.  Otherwise we can't.  If the node's position
                // is ahead of us, then we'll need to rescan tokens.  If the node's position is behind
                // us, then we'll need to skip it or crumble it as appropriate
                //
                // We will also adjust the positions of nodes that intersect the change range as well.
                // By doing this, we ensure that all the positions in the old tree are consistent, not
                // just the positions of nodes entirely before/after the change range.  By being
                // consistent, we can then easily map from positions to nodes in the old tree easily.
                //
                // Also, mark any syntax elements that intersect the changed span.  We know, up front,
                // that we cannot reuse these elements.
                updateTokenPositionsAndMarkElements(incrementalSourceFile, changeRange.span.start, ts.textSpanEnd(changeRange.span), ts.textSpanEnd(ts.textChangeRangeNewSpan(changeRange)), delta, oldText, newText, aggressiveChecks);
                // Now that we've set up our internal incremental state just proceed and parse the
                // source file in the normal fashion.  When possible the parser will retrieve and
                // reuse nodes from the old tree.
                //
                // Note: passing in 'true' for setNodeParents is very important.  When incrementally
                // parsing, we will be reusing nodes from the old tree, and placing it into new
                // parents.  If we don't set the parents now, we'll end up with an observably
                // inconsistent tree.  Setting the parents on the new tree should be very fast.  We
                // will immediately bail out of walking any subtrees when we can see that their parents
                // are already correct.
                var result = Parser.parseSourceFile(sourceFile.fileName, newText, sourceFile.languageVersion, syntaxCursor, /*setParentNodes*/ true, sourceFile.scriptKind);
                return result;
            }
            IncrementalParser.updateSourceFile = updateSourceFile;
            function moveElementEntirelyPastChangeRange(element, isArray, delta, oldText, newText, aggressiveChecks) {
                if (isArray) {
                    visitArray(element);
                }
                else {
                    visitNode(element);
                }
                return;
                function visitNode(node) {
                    var text = "";
                    if (aggressiveChecks && shouldCheckNode(node)) {
                        text = oldText.substring(node.pos, node.end);
                    }
                    // Ditch any existing LS children we may have created.  This way we can avoid
                    // moving them forward.
                    if (node._children) {
                        node._children = undefined;
                    }
                    node.pos += delta;
                    node.end += delta;
                    if (aggressiveChecks && shouldCheckNode(node)) {
                        ts.Debug.assert(text === newText.substring(node.pos, node.end));
                    }
                    forEachChild(node, visitNode, visitArray);
                    if (ts.hasJSDocNodes(node)) {
                        for (var _i = 0, _a = node.jsDoc; _i < _a.length; _i++) {
                            var jsDocComment = _a[_i];
                            visitNode(jsDocComment);
                        }
                    }
                    checkNodePositions(node, aggressiveChecks);
                }
                function visitArray(array) {
                    array._children = undefined;
                    array.pos += delta;
                    array.end += delta;
                    for (var _i = 0, array_8 = array; _i < array_8.length; _i++) {
                        var node = array_8[_i];
                        visitNode(node);
                    }
                }
            }
            function shouldCheckNode(node) {
                switch (node.kind) {
                    case 10 /* StringLiteral */:
                    case 8 /* NumericLiteral */:
                    case 75 /* Identifier */:
                        return true;
                }
                return false;
            }
            function adjustIntersectingElement(element, changeStart, changeRangeOldEnd, changeRangeNewEnd, delta) {
                ts.Debug.assert(element.end >= changeStart, "Adjusting an element that was entirely before the change range");
                ts.Debug.assert(element.pos <= changeRangeOldEnd, "Adjusting an element that was entirely after the change range");
                ts.Debug.assert(element.pos <= element.end);
                // We have an element that intersects the change range in some way.  It may have its
                // start, or its end (or both) in the changed range.  We want to adjust any part
                // that intersects such that the final tree is in a consistent state.  i.e. all
                // children have spans within the span of their parent, and all siblings are ordered
                // properly.
                // We may need to update both the 'pos' and the 'end' of the element.
                // If the 'pos' is before the start of the change, then we don't need to touch it.
                // If it isn't, then the 'pos' must be inside the change.  How we update it will
                // depend if delta is positive or negative. If delta is positive then we have
                // something like:
                //
                //  -------------------AAA-----------------
                //  -------------------BBBCCCCCCC-----------------
                //
                // In this case, we consider any node that started in the change range to still be
                // starting at the same position.
                //
                // however, if the delta is negative, then we instead have something like this:
                //
                //  -------------------XXXYYYYYYY-----------------
                //  -------------------ZZZ-----------------
                //
                // In this case, any element that started in the 'X' range will keep its position.
                // However any element that started after that will have their pos adjusted to be
                // at the end of the new range.  i.e. any node that started in the 'Y' range will
                // be adjusted to have their start at the end of the 'Z' range.
                //
                // The element will keep its position if possible.  Or Move backward to the new-end
                // if it's in the 'Y' range.
                element.pos = Math.min(element.pos, changeRangeNewEnd);
                // If the 'end' is after the change range, then we always adjust it by the delta
                // amount.  However, if the end is in the change range, then how we adjust it
                // will depend on if delta is positive or negative.  If delta is positive then we
                // have something like:
                //
                //  -------------------AAA-----------------
                //  -------------------BBBCCCCCCC-----------------
                //
                // In this case, we consider any node that ended inside the change range to keep its
                // end position.
                //
                // however, if the delta is negative, then we instead have something like this:
                //
                //  -------------------XXXYYYYYYY-----------------
                //  -------------------ZZZ-----------------
                //
                // In this case, any element that ended in the 'X' range will keep its position.
                // However any element that ended after that will have their pos adjusted to be
                // at the end of the new range.  i.e. any node that ended in the 'Y' range will
                // be adjusted to have their end at the end of the 'Z' range.
                if (element.end >= changeRangeOldEnd) {
                    // Element ends after the change range.  Always adjust the end pos.
                    element.end += delta;
                }
                else {
                    // Element ends in the change range.  The element will keep its position if
                    // possible. Or Move backward to the new-end if it's in the 'Y' range.
                    element.end = Math.min(element.end, changeRangeNewEnd);
                }
                ts.Debug.assert(element.pos <= element.end);
                if (element.parent) {
                    ts.Debug.assert(element.pos >= element.parent.pos);
                    ts.Debug.assert(element.end <= element.parent.end);
                }
            }
            function checkNodePositions(node, aggressiveChecks) {
                if (aggressiveChecks) {
                    var pos_2 = node.pos;
                    var visitNode_1 = function (child) {
                        ts.Debug.assert(child.pos >= pos_2);
                        pos_2 = child.end;
                    };
                    if (ts.hasJSDocNodes(node)) {
                        for (var _i = 0, _a = node.jsDoc; _i < _a.length; _i++) {
                            var jsDocComment = _a[_i];
                            visitNode_1(jsDocComment);
                        }
                    }
                    forEachChild(node, visitNode_1);
                    ts.Debug.assert(pos_2 <= node.end);
                }
            }
            function updateTokenPositionsAndMarkElements(sourceFile, changeStart, changeRangeOldEnd, changeRangeNewEnd, delta, oldText, newText, aggressiveChecks) {
                visitNode(sourceFile);
                return;
                function visitNode(child) {
                    ts.Debug.assert(child.pos <= child.end);
                    if (child.pos > changeRangeOldEnd) {
                        // Node is entirely past the change range.  We need to move both its pos and
                        // end, forward or backward appropriately.
                        moveElementEntirelyPastChangeRange(child, /*isArray*/ false, delta, oldText, newText, aggressiveChecks);
                        return;
                    }
                    // Check if the element intersects the change range.  If it does, then it is not
                    // reusable.  Also, we'll need to recurse to see what constituent portions we may
                    // be able to use.
                    var fullEnd = child.end;
                    if (fullEnd >= changeStart) {
                        child.intersectsChange = true;
                        child._children = undefined;
                        // Adjust the pos or end (or both) of the intersecting element accordingly.
                        adjustIntersectingElement(child, changeStart, changeRangeOldEnd, changeRangeNewEnd, delta);
                        forEachChild(child, visitNode, visitArray);
                        if (ts.hasJSDocNodes(child)) {
                            for (var _i = 0, _a = child.jsDoc; _i < _a.length; _i++) {
                                var jsDocComment = _a[_i];
                                visitNode(jsDocComment);
                            }
                        }
                        checkNodePositions(child, aggressiveChecks);
                        return;
                    }
                    // Otherwise, the node is entirely before the change range.  No need to do anything with it.
                    ts.Debug.assert(fullEnd < changeStart);
                }
                function visitArray(array) {
                    ts.Debug.assert(array.pos <= array.end);
                    if (array.pos > changeRangeOldEnd) {
                        // Array is entirely after the change range.  We need to move it, and move any of
                        // its children.
                        moveElementEntirelyPastChangeRange(array, /*isArray*/ true, delta, oldText, newText, aggressiveChecks);
                        return;
                    }
                    // Check if the element intersects the change range.  If it does, then it is not
                    // reusable.  Also, we'll need to recurse to see what constituent portions we may
                    // be able to use.
                    var fullEnd = array.end;
                    if (fullEnd >= changeStart) {
                        array.intersectsChange = true;
                        array._children = undefined;
                        // Adjust the pos or end (or both) of the intersecting array accordingly.
                        adjustIntersectingElement(array, changeStart, changeRangeOldEnd, changeRangeNewEnd, delta);
                        for (var _i = 0, array_9 = array; _i < array_9.length; _i++) {
                            var node = array_9[_i];
                            visitNode(node);
                        }
                        return;
                    }
                    // Otherwise, the array is entirely before the change range.  No need to do anything with it.
                    ts.Debug.assert(fullEnd < changeStart);
                }
            }
            function extendToAffectedRange(sourceFile, changeRange) {
                // Consider the following code:
                //      void foo() { /; }
                //
                // If the text changes with an insertion of / just before the semicolon then we end up with:
                //      void foo() { //; }
                //
                // If we were to just use the changeRange a is, then we would not rescan the { token
                // (as it does not intersect the actual original change range).  Because an edit may
                // change the token touching it, we actually need to look back *at least* one token so
                // that the prior token sees that change.
                var maxLookahead = 1;
                var start = changeRange.span.start;
                // the first iteration aligns us with the change start. subsequent iteration move us to
                // the left by maxLookahead tokens.  We only need to do this as long as we're not at the
                // start of the tree.
                for (var i = 0; start > 0 && i <= maxLookahead; i++) {
                    var nearestNode = findNearestNodeStartingBeforeOrAtPosition(sourceFile, start);
                    ts.Debug.assert(nearestNode.pos <= start);
                    var position = nearestNode.pos;
                    start = Math.max(0, position - 1);
                }
                var finalSpan = ts.createTextSpanFromBounds(start, ts.textSpanEnd(changeRange.span));
                var finalLength = changeRange.newLength + (changeRange.span.start - start);
                return ts.createTextChangeRange(finalSpan, finalLength);
            }
            function findNearestNodeStartingBeforeOrAtPosition(sourceFile, position) {
                var bestResult = sourceFile;
                var lastNodeEntirelyBeforePosition;
                forEachChild(sourceFile, visit);
                if (lastNodeEntirelyBeforePosition) {
                    var lastChildOfLastEntireNodeBeforePosition = getLastDescendant(lastNodeEntirelyBeforePosition);
                    if (lastChildOfLastEntireNodeBeforePosition.pos > bestResult.pos) {
                        bestResult = lastChildOfLastEntireNodeBeforePosition;
                    }
                }
                return bestResult;
                function getLastDescendant(node) {
                    while (true) {
                        var lastChild = ts.getLastChild(node);
                        if (lastChild) {
                            node = lastChild;
                        }
                        else {
                            return node;
                        }
                    }
                }
                function visit(child) {
                    if (ts.nodeIsMissing(child)) {
                        // Missing nodes are effectively invisible to us.  We never even consider them
                        // When trying to find the nearest node before us.
                        return;
                    }
                    // If the child intersects this position, then this node is currently the nearest
                    // node that starts before the position.
                    if (child.pos <= position) {
                        if (child.pos >= bestResult.pos) {
                            // This node starts before the position, and is closer to the position than
                            // the previous best node we found.  It is now the new best node.
                            bestResult = child;
                        }
                        // Now, the node may overlap the position, or it may end entirely before the
                        // position.  If it overlaps with the position, then either it, or one of its
                        // children must be the nearest node before the position.  So we can just
                        // recurse into this child to see if we can find something better.
                        if (position < child.end) {
                            // The nearest node is either this child, or one of the children inside
                            // of it.  We've already marked this child as the best so far.  Recurse
                            // in case one of the children is better.
                            forEachChild(child, visit);
                            // Once we look at the children of this node, then there's no need to
                            // continue any further.
                            return true;
                        }
                        else {
                            ts.Debug.assert(child.end <= position);
                            // The child ends entirely before this position.  Say you have the following
                            // (where $ is the position)
                            //
                            //      <complex expr 1> ? <complex expr 2> $ : <...> <...>
                            //
                            // We would want to find the nearest preceding node in "complex expr 2".
                            // To support that, we keep track of this node, and once we're done searching
                            // for a best node, we recurse down this node to see if we can find a good
                            // result in it.
                            //
                            // This approach allows us to quickly skip over nodes that are entirely
                            // before the position, while still allowing us to find any nodes in the
                            // last one that might be what we want.
                            lastNodeEntirelyBeforePosition = child;
                        }
                    }
                    else {
                        ts.Debug.assert(child.pos > position);
                        // We're now at a node that is entirely past the position we're searching for.
                        // This node (and all following nodes) could never contribute to the result,
                        // so just skip them by returning 'true' here.
                        return true;
                    }
                }
            }
            function checkChangeRange(sourceFile, newText, textChangeRange, aggressiveChecks) {
                var oldText = sourceFile.text;
                if (textChangeRange) {
                    ts.Debug.assert((oldText.length - textChangeRange.span.length + textChangeRange.newLength) === newText.length);
                    if (aggressiveChecks || ts.Debug.shouldAssert(3 /* VeryAggressive */)) {
                        var oldTextPrefix = oldText.substr(0, textChangeRange.span.start);
                        var newTextPrefix = newText.substr(0, textChangeRange.span.start);
                        ts.Debug.assert(oldTextPrefix === newTextPrefix);
                        var oldTextSuffix = oldText.substring(ts.textSpanEnd(textChangeRange.span), oldText.length);
                        var newTextSuffix = newText.substring(ts.textSpanEnd(ts.textChangeRangeNewSpan(textChangeRange)), newText.length);
                        ts.Debug.assert(oldTextSuffix === newTextSuffix);
                    }
                }
            }
            function createSyntaxCursor(sourceFile) {
                var currentArray = sourceFile.statements;
                var currentArrayIndex = 0;
                ts.Debug.assert(currentArrayIndex < currentArray.length);
                var current = currentArray[currentArrayIndex];
                var lastQueriedPosition = -1 /* Value */;
                return {
                    currentNode: function (position) {
                        // Only compute the current node if the position is different than the last time
                        // we were asked.  The parser commonly asks for the node at the same position
                        // twice.  Once to know if can read an appropriate list element at a certain point,
                        // and then to actually read and consume the node.
                        if (position !== lastQueriedPosition) {
                            // Much of the time the parser will need the very next node in the array that
                            // we just returned a node from.So just simply check for that case and move
                            // forward in the array instead of searching for the node again.
                            if (current && current.end === position && currentArrayIndex < (currentArray.length - 1)) {
                                currentArrayIndex++;
                                current = currentArray[currentArrayIndex];
                            }
                            // If we don't have a node, or the node we have isn't in the right position,
                            // then try to find a viable node at the position requested.
                            if (!current || current.pos !== position) {
                                findHighestListElementThatStartsAtPosition(position);
                            }
                        }
                        // Cache this query so that we don't do any extra work if the parser calls back
                        // into us.  Note: this is very common as the parser will make pairs of calls like
                        // 'isListElement -> parseListElement'.  If we were unable to find a node when
                        // called with 'isListElement', we don't want to redo the work when parseListElement
                        // is called immediately after.
                        lastQueriedPosition = position;
                        // Either we don'd have a node, or we have a node at the position being asked for.
                        ts.Debug.assert(!current || current.pos === position);
                        return current;
                    }
                };
                // Finds the highest element in the tree we can find that starts at the provided position.
                // The element must be a direct child of some node list in the tree.  This way after we
                // return it, we can easily return its next sibling in the list.
                function findHighestListElementThatStartsAtPosition(position) {
                    // Clear out any cached state about the last node we found.
                    currentArray = undefined;
                    currentArrayIndex = -1 /* Value */;
                    current = undefined;
                    // Recurse into the source file to find the highest node at this position.
                    forEachChild(sourceFile, visitNode, visitArray);
                    return;
                    function visitNode(node) {
                        if (position >= node.pos && position < node.end) {
                            // Position was within this node.  Keep searching deeper to find the node.
                            forEachChild(node, visitNode, visitArray);
                            // don't proceed any further in the search.
                            return true;
                        }
                        // position wasn't in this node, have to keep searching.
                        return false;
                    }
                    function visitArray(array) {
                        if (position >= array.pos && position < array.end) {
                            // position was in this array.  Search through this array to see if we find a
                            // viable element.
                            for (var i = 0; i < array.length; i++) {
                                var child = array[i];
                                if (child) {
                                    if (child.pos === position) {
                                        // Found the right node.  We're done.
                                        currentArray = array;
                                        currentArrayIndex = i;
                                        current = child;
                                        return true;
                                    }
                                    else {
                                        if (child.pos < position && position < child.end) {
                                            // Position in somewhere within this child.  Search in it and
                                            // stop searching in this array.
                                            forEachChild(child, visitNode, visitArray);
                                            return true;
                                        }
                                    }
                                }
                            }
                        }
                        // position wasn't in this array, have to keep searching.
                        return false;
                    }
                }
            }
            var InvalidPosition;
            (function (InvalidPosition) {
                InvalidPosition[InvalidPosition["Value"] = -1] = "Value";
            })(InvalidPosition || (InvalidPosition = {}));
        })(IncrementalParser || (IncrementalParser = {}));
        /** @internal */
        function isDeclarationFileName(fileName) {
            return ts.fileExtensionIs(fileName, ".d.ts" /* Dts */);
        }
        ts.isDeclarationFileName = isDeclarationFileName;
        /*@internal*/
        function processCommentPragmas(context, sourceText) {
            var pragmas = [];
            for (var _i = 0, _a = ts.getLeadingCommentRanges(sourceText, 0) || ts.emptyArray; _i < _a.length; _i++) {
                var range = _a[_i];
                var comment = sourceText.substring(range.pos, range.end);
                extractPragmas(pragmas, range, comment);
            }
            context.pragmas = ts.createMap();
            for (var _b = 0, pragmas_1 = pragmas; _b < pragmas_1.length; _b++) {
                var pragma = pragmas_1[_b];
                if (context.pragmas.has(pragma.name)) {
                    var currentValue = context.pragmas.get(pragma.name);
                    if (currentValue instanceof Array) {
                        currentValue.push(pragma.args);
                    }
                    else {
                        context.pragmas.set(pragma.name, [currentValue, pragma.args]);
                    }
                    continue;
                }
                context.pragmas.set(pragma.name, pragma.args);
            }
        }
        ts.processCommentPragmas = processCommentPragmas;
        /*@internal*/
        function processPragmasIntoFields(context, reportDiagnostic) {
            context.checkJsDirective = undefined;
            context.referencedFiles = [];
            context.typeReferenceDirectives = [];
            context.libReferenceDirectives = [];
            context.amdDependencies = [];
            context.hasNoDefaultLib = false;
            context.pragmas.forEach(function (entryOrList, key) {
                // TODO: The below should be strongly type-guarded and not need casts/explicit annotations, since entryOrList is related to
                // key and key is constrained to a union; but it's not (see GH#21483 for at least partial fix) :(
                switch (key) {
                    case "reference": {
                        var referencedFiles_1 = context.referencedFiles;
                        var typeReferenceDirectives_1 = context.typeReferenceDirectives;
                        var libReferenceDirectives_1 = context.libReferenceDirectives;
                        ts.forEach(ts.toArray(entryOrList), function (arg) {
                            var _a = arg.arguments, types = _a.types, lib = _a.lib, path = _a.path;
                            if (arg.arguments["no-default-lib"]) {
                                context.hasNoDefaultLib = true;
                            }
                            else if (types) {
                                typeReferenceDirectives_1.push({ pos: types.pos, end: types.end, fileName: types.value });
                            }
                            else if (lib) {
                                libReferenceDirectives_1.push({ pos: lib.pos, end: lib.end, fileName: lib.value });
                            }
                            else if (path) {
                                referencedFiles_1.push({ pos: path.pos, end: path.end, fileName: path.value });
                            }
                            else {
                                reportDiagnostic(arg.range.pos, arg.range.end - arg.range.pos, ts.Diagnostics.Invalid_reference_directive_syntax);
                            }
                        });
                        break;
                    }
                    case "amd-dependency": {
                        context.amdDependencies = ts.map(ts.toArray(entryOrList), function (x) { return ({ name: x.arguments.name, path: x.arguments.path }); });
                        break;
                    }
                    case "amd-module": {
                        if (entryOrList instanceof Array) {
                            for (var _i = 0, entryOrList_1 = entryOrList; _i < entryOrList_1.length; _i++) {
                                var entry = entryOrList_1[_i];
                                if (context.moduleName) {
                                    // TODO: It's probably fine to issue this diagnostic on all instances of the pragma
                                    reportDiagnostic(entry.range.pos, entry.range.end - entry.range.pos, ts.Diagnostics.An_AMD_module_cannot_have_multiple_name_assignments);
                                }
                                context.moduleName = entry.arguments.name;
                            }
                        }
                        else {
                            context.moduleName = entryOrList.arguments.name;
                        }
                        break;
                    }
                    case "ts-nocheck":
                    case "ts-check": {
                        // _last_ of either nocheck or check in a file is the "winner"
                        ts.forEach(ts.toArray(entryOrList), function (entry) {
                            if (!context.checkJsDirective || entry.range.pos > context.checkJsDirective.pos) {
                                context.checkJsDirective = {
                                    enabled: key === "ts-check",
                                    end: entry.range.end,
                                    pos: entry.range.pos
                                };
                            }
                        });
                        break;
                    }
                    case "jsx": return; // Accessed directly
                    default: ts.Debug.fail("Unhandled pragma kind"); // Can this be made into an assertNever in the future?
                }
            });
        }
        ts.processPragmasIntoFields = processPragmasIntoFields;
        var namedArgRegExCache = ts.createMap();
        function getNamedArgRegEx(name) {
            if (namedArgRegExCache.has(name)) {
                return namedArgRegExCache.get(name);
            }
            var result = new RegExp("(\\s" + name + "\\s*=\\s*)('|\")(.+?)\\2", "im");
            namedArgRegExCache.set(name, result);
            return result;
        }
        var tripleSlashXMLCommentStartRegEx = /^\/\/\/\s*<(\S+)\s.*?\/>/im;
        var singleLinePragmaRegEx = /^\/\/\/?\s*@(\S+)\s*(.*)\s*$/im;
        function extractPragmas(pragmas, range, text) {
            var tripleSlash = range.kind === 2 /* SingleLineCommentTrivia */ && tripleSlashXMLCommentStartRegEx.exec(text);
            if (tripleSlash) {
                var name = tripleSlash[1].toLowerCase(); // Technically unsafe cast, but we do it so the below check to make it safe typechecks
                var pragma = ts.commentPragmas[name];
                if (!pragma || !(pragma.kind & 1 /* TripleSlashXML */)) {
                    return;
                }
                if (pragma.args) {
                    var argument = {};
                    for (var _i = 0, _a = pragma.args; _i < _a.length; _i++) {
                        var arg = _a[_i];
                        var matcher = getNamedArgRegEx(arg.name);
                        var matchResult = matcher.exec(text);
                        if (!matchResult && !arg.optional) {
                            return; // Missing required argument, don't parse
                        }
                        else if (matchResult) {
                            if (arg.captureSpan) {
                                var startPos = range.pos + matchResult.index + matchResult[1].length + matchResult[2].length;
                                argument[arg.name] = {
                                    value: matchResult[3],
                                    pos: startPos,
                                    end: startPos + matchResult[3].length
                                };
                            }
                            else {
                                argument[arg.name] = matchResult[3];
                            }
                        }
                    }
                    pragmas.push({ name: name, args: { arguments: argument, range: range } });
                }
                else {
                    pragmas.push({ name: name, args: { arguments: {}, range: range } });
                }
                return;
            }
            var singleLine = range.kind === 2 /* SingleLineCommentTrivia */ && singleLinePragmaRegEx.exec(text);
            if (singleLine) {
                return addPragmaForMatch(pragmas, range, 2 /* SingleLine */, singleLine);
            }
            if (range.kind === 3 /* MultiLineCommentTrivia */) {
                var multiLinePragmaRegEx = /\s*@(\S+)\s*(.*)\s*$/gim; // Defined inline since it uses the "g" flag, which keeps a persistent index (for iterating)
                var multiLineMatch = void 0;
                while (multiLineMatch = multiLinePragmaRegEx.exec(text)) {
                    addPragmaForMatch(pragmas, range, 4 /* MultiLine */, multiLineMatch);
                }
            }
        }
        function addPragmaForMatch(pragmas, range, kind, match) {
            if (!match)
                return;
            var name = match[1].toLowerCase(); // Technically unsafe cast, but we do it so they below check to make it safe typechecks
            var pragma = ts.commentPragmas[name];
            if (!pragma || !(pragma.kind & kind)) {
                return;
            }
            var args = match[2]; // Split on spaces and match up positionally with definition
            var argument = getNamedPragmaArguments(pragma, args);
            if (argument === "fail")
                return; // Missing required argument, fail to parse it
            pragmas.push({ name: name, args: { arguments: argument, range: range } });
            return;
        }
        function getNamedPragmaArguments(pragma, text) {
            if (!text)
                return {};
            if (!pragma.args)
                return {};
            var args = text.split(/\s+/);
            var argMap = {};
            for (var i = 0; i < pragma.args.length; i++) {
                var argument = pragma.args[i];
                if (!args[i] && !argument.optional) {
                    return "fail";
                }
                if (argument.captureSpan) {
                    return ts.Debug.fail("Capture spans not yet implemented for non-xml pragmas");
                }
                argMap[argument.name] = args[i];
            }
            return argMap;
        }
        /** @internal */
        function tagNamesAreEquivalent(lhs, rhs) {
            if (lhs.kind !== rhs.kind) {
                return false;
            }
            if (lhs.kind === 75 /* Identifier */) {
                return lhs.escapedText === rhs.escapedText;
            }
            if (lhs.kind === 104 /* ThisKeyword */) {
                return true;
            }
            // If we are at this statement then we must have PropertyAccessExpression and because tag name in Jsx element can only
            // take forms of JsxTagNameExpression which includes an identifier, "this" expression, or another propertyAccessExpression
            // it is safe to case the expression property as such. See parseJsxElementName for how we parse tag name in Jsx element
            return lhs.name.escapedText === rhs.name.escapedText &&
                tagNamesAreEquivalent(lhs.expression, rhs.expression);
        }
        ts.tagNamesAreEquivalent = tagNamesAreEquivalent;
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        /* @internal */
        ts.compileOnSaveCommandLineOption = { name: "compileOnSave", type: "boolean" };
        // NOTE: The order here is important to default lib ordering as entries will have the same
        //       order in the generated program (see `getDefaultLibPriority` in program.ts). This
        //       order also affects overload resolution when a type declared in one lib is
        //       augmented in another lib.
        var libEntries = [
            // JavaScript only
            ["es5", "lib.es5.d.ts"],
            ["es6", "lib.es2015.d.ts"],
            ["es2015", "lib.es2015.d.ts"],
            ["es7", "lib.es2016.d.ts"],
            ["es2016", "lib.es2016.d.ts"],
            ["es2017", "lib.es2017.d.ts"],
            ["es2018", "lib.es2018.d.ts"],
            ["es2019", "lib.es2019.d.ts"],
            ["es2020", "lib.es2020.d.ts"],
            ["esnext", "lib.esnext.d.ts"],
            // Host only
            ["dom", "lib.dom.d.ts"],
            ["dom.iterable", "lib.dom.iterable.d.ts"],
            ["webworker", "lib.webworker.d.ts"],
            ["webworker.importscripts", "lib.webworker.importscripts.d.ts"],
            ["scripthost", "lib.scripthost.d.ts"],
            // ES2015 Or ESNext By-feature options
            ["es2015.core", "lib.es2015.core.d.ts"],
            ["es2015.collection", "lib.es2015.collection.d.ts"],
            ["es2015.generator", "lib.es2015.generator.d.ts"],
            ["es2015.iterable", "lib.es2015.iterable.d.ts"],
            ["es2015.promise", "lib.es2015.promise.d.ts"],
            ["es2015.proxy", "lib.es2015.proxy.d.ts"],
            ["es2015.reflect", "lib.es2015.reflect.d.ts"],
            ["es2015.symbol", "lib.es2015.symbol.d.ts"],
            ["es2015.symbol.wellknown", "lib.es2015.symbol.wellknown.d.ts"],
            ["es2016.array.include", "lib.es2016.array.include.d.ts"],
            ["es2017.object", "lib.es2017.object.d.ts"],
            ["es2017.sharedmemory", "lib.es2017.sharedmemory.d.ts"],
            ["es2017.string", "lib.es2017.string.d.ts"],
            ["es2017.intl", "lib.es2017.intl.d.ts"],
            ["es2017.typedarrays", "lib.es2017.typedarrays.d.ts"],
            ["es2018.asyncgenerator", "lib.es2018.asyncgenerator.d.ts"],
            ["es2018.asynciterable", "lib.es2018.asynciterable.d.ts"],
            ["es2018.intl", "lib.es2018.intl.d.ts"],
            ["es2018.promise", "lib.es2018.promise.d.ts"],
            ["es2018.regexp", "lib.es2018.regexp.d.ts"],
            ["es2019.array", "lib.es2019.array.d.ts"],
            ["es2019.object", "lib.es2019.object.d.ts"],
            ["es2019.string", "lib.es2019.string.d.ts"],
            ["es2019.symbol", "lib.es2019.symbol.d.ts"],
            ["es2020.bigint", "lib.es2020.bigint.d.ts"],
            ["es2020.promise", "lib.es2020.promise.d.ts"],
            ["es2020.string", "lib.es2020.string.d.ts"],
            ["es2020.symbol.wellknown", "lib.es2020.symbol.wellknown.d.ts"],
            ["esnext.array", "lib.es2019.array.d.ts"],
            ["esnext.symbol", "lib.es2019.symbol.d.ts"],
            ["esnext.asynciterable", "lib.es2018.asynciterable.d.ts"],
            ["esnext.intl", "lib.esnext.intl.d.ts"],
            ["esnext.bigint", "lib.es2020.bigint.d.ts"]
        ];
        /**
         * An array of supported "lib" reference file names used to determine the order for inclusion
         * when referenced, as well as for spelling suggestions. This ensures the correct ordering for
         * overload resolution when a type declared in one lib is extended by another.
         */
        /* @internal */
        ts.libs = libEntries.map(function (entry) { return entry[0]; });
        /**
         * A map of lib names to lib files. This map is used both for parsing the "lib" command line
         * option as well as for resolving lib reference directives.
         */
        /* @internal */
        ts.libMap = ts.createMapFromEntries(libEntries);
        // Watch related options
        /* @internal */
        ts.optionsForWatch = [
            {
                name: "watchFile",
                type: ts.createMapFromTemplate({
                    fixedpollinginterval: ts.WatchFileKind.FixedPollingInterval,
                    prioritypollinginterval: ts.WatchFileKind.PriorityPollingInterval,
                    dynamicprioritypolling: ts.WatchFileKind.DynamicPriorityPolling,
                    usefsevents: ts.WatchFileKind.UseFsEvents,
                    usefseventsonparentdirectory: ts.WatchFileKind.UseFsEventsOnParentDirectory,
                }),
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Specify_strategy_for_watching_file_Colon_FixedPollingInterval_default_PriorityPollingInterval_DynamicPriorityPolling_UseFsEvents_UseFsEventsOnParentDirectory,
            },
            {
                name: "watchDirectory",
                type: ts.createMapFromTemplate({
                    usefsevents: ts.WatchDirectoryKind.UseFsEvents,
                    fixedpollinginterval: ts.WatchDirectoryKind.FixedPollingInterval,
                    dynamicprioritypolling: ts.WatchDirectoryKind.DynamicPriorityPolling,
                }),
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Specify_strategy_for_watching_directory_on_platforms_that_don_t_support_recursive_watching_natively_Colon_UseFsEvents_default_FixedPollingInterval_DynamicPriorityPolling,
            },
            {
                name: "fallbackPolling",
                type: ts.createMapFromTemplate({
                    fixedinterval: ts.PollingWatchKind.FixedInterval,
                    priorityinterval: ts.PollingWatchKind.PriorityInterval,
                    dynamicpriority: ts.PollingWatchKind.DynamicPriority,
                }),
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Specify_strategy_for_creating_a_polling_watch_when_it_fails_to_create_using_file_system_events_Colon_FixedInterval_default_PriorityInterval_DynamicPriority,
            },
            {
                name: "synchronousWatchDirectory",
                type: "boolean",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Synchronously_call_callbacks_and_update_the_state_of_directory_watchers_on_platforms_that_don_t_support_recursive_watching_natively,
            },
        ];
        /* @internal */
        ts.commonOptionsWithBuild = [
            {
                name: "help",
                shortName: "h",
                type: "boolean",
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Print_this_message,
            },
            {
                name: "help",
                shortName: "?",
                type: "boolean"
            },
            {
                name: "watch",
                shortName: "w",
                type: "boolean",
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Watch_input_files,
            },
            {
                name: "preserveWatchOutput",
                type: "boolean",
                showInSimplifiedHelpView: false,
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Whether_to_keep_outdated_console_output_in_watch_mode_instead_of_clearing_the_screen,
            },
            {
                name: "listFiles",
                type: "boolean",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Print_names_of_files_part_of_the_compilation
            },
            {
                name: "listEmittedFiles",
                type: "boolean",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Print_names_of_generated_files_part_of_the_compilation
            },
            {
                name: "pretty",
                type: "boolean",
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Stylize_errors_and_messages_using_color_and_context_experimental
            },
            {
                name: "traceResolution",
                type: "boolean",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Enable_tracing_of_the_name_resolution_process
            },
            {
                name: "diagnostics",
                type: "boolean",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Show_diagnostic_information
            },
            {
                name: "extendedDiagnostics",
                type: "boolean",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Show_verbose_diagnostic_information
            },
            {
                name: "generateCpuProfile",
                type: "string",
                isFilePath: true,
                paramType: ts.Diagnostics.FILE_OR_DIRECTORY,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Generates_a_CPU_profile
            },
            {
                name: "incremental",
                shortName: "i",
                type: "boolean",
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Enable_incremental_compilation,
                transpileOptionValue: undefined
            },
            {
                name: "assumeChangesOnlyAffectDirectDependencies",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                affectsEmit: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Have_recompiles_in_incremental_and_watch_assume_that_changes_within_a_file_will_only_affect_files_directly_depending_on_it
            },
            {
                name: "locale",
                type: "string",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.The_locale_used_when_displaying_messages_to_the_user_e_g_en_us
            },
        ];
        /* @internal */
        ts.optionDeclarations = __spreadArrays(ts.commonOptionsWithBuild, [
            {
                name: "all",
                type: "boolean",
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Show_all_compiler_options,
            },
            {
                name: "version",
                shortName: "v",
                type: "boolean",
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Print_the_compiler_s_version,
            },
            {
                name: "init",
                type: "boolean",
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Initializes_a_TypeScript_project_and_creates_a_tsconfig_json_file,
            },
            {
                name: "project",
                shortName: "p",
                type: "string",
                isFilePath: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Command_line_Options,
                paramType: ts.Diagnostics.FILE_OR_DIRECTORY,
                description: ts.Diagnostics.Compile_the_project_given_the_path_to_its_configuration_file_or_to_a_folder_with_a_tsconfig_json,
            },
            {
                name: "build",
                type: "boolean",
                shortName: "b",
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Build_one_or_more_projects_and_their_dependencies_if_out_of_date
            },
            {
                name: "showConfig",
                type: "boolean",
                category: ts.Diagnostics.Command_line_Options,
                isCommandLineOnly: true,
                description: ts.Diagnostics.Print_the_final_configuration_instead_of_building
            },
            {
                name: "listFilesOnly",
                type: "boolean",
                category: ts.Diagnostics.Command_line_Options,
                affectsSemanticDiagnostics: true,
                affectsEmit: true,
                isCommandLineOnly: true,
                description: ts.Diagnostics.Print_names_of_files_that_are_part_of_the_compilation_and_then_stop_processing
            },
            // Basic
            {
                name: "target",
                shortName: "t",
                type: ts.createMapFromTemplate({
                    es3: 0 /* ES3 */,
                    es5: 1 /* ES5 */,
                    es6: 2 /* ES2015 */,
                    es2015: 2 /* ES2015 */,
                    es2016: 3 /* ES2016 */,
                    es2017: 4 /* ES2017 */,
                    es2018: 5 /* ES2018 */,
                    es2019: 6 /* ES2019 */,
                    es2020: 7 /* ES2020 */,
                    esnext: 99 /* ESNext */,
                }),
                affectsSourceFile: true,
                affectsModuleResolution: true,
                affectsEmit: true,
                paramType: ts.Diagnostics.VERSION,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Specify_ECMAScript_target_version_Colon_ES3_default_ES5_ES2015_ES2016_ES2017_ES2018_ES2019_ES2020_or_ESNEXT,
            },
            {
                name: "module",
                shortName: "m",
                type: ts.createMapFromTemplate({
                    none: ts.ModuleKind.None,
                    commonjs: ts.ModuleKind.CommonJS,
                    amd: ts.ModuleKind.AMD,
                    system: ts.ModuleKind.System,
                    umd: ts.ModuleKind.UMD,
                    es6: ts.ModuleKind.ES2015,
                    es2015: ts.ModuleKind.ES2015,
                    es2020: ts.ModuleKind.ES2020,
                    esnext: ts.ModuleKind.ESNext
                }),
                affectsModuleResolution: true,
                affectsEmit: true,
                paramType: ts.Diagnostics.KIND,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Specify_module_code_generation_Colon_none_commonjs_amd_system_umd_es2015_es2020_or_ESNext,
            },
            {
                name: "lib",
                type: "list",
                element: {
                    name: "lib",
                    type: ts.libMap
                },
                affectsModuleResolution: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Specify_library_files_to_be_included_in_the_compilation,
                transpileOptionValue: undefined
            },
            {
                name: "allowJs",
                type: "boolean",
                affectsModuleResolution: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Allow_javascript_files_to_be_compiled
            },
            {
                name: "checkJs",
                type: "boolean",
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Report_errors_in_js_files
            },
            {
                name: "jsx",
                type: ts.createMapFromTemplate({
                    "preserve": 1 /* Preserve */,
                    "react-native": 3 /* ReactNative */,
                    "react": 2 /* React */
                }),
                affectsSourceFile: true,
                paramType: ts.Diagnostics.KIND,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Specify_JSX_code_generation_Colon_preserve_react_native_or_react,
            },
            {
                name: "declaration",
                shortName: "d",
                type: "boolean",
                affectsEmit: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Generates_corresponding_d_ts_file,
                transpileOptionValue: undefined
            },
            {
                name: "declarationMap",
                type: "boolean",
                affectsEmit: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Generates_a_sourcemap_for_each_corresponding_d_ts_file,
                transpileOptionValue: undefined
            },
            {
                name: "emitDeclarationOnly",
                type: "boolean",
                affectsEmit: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Only_emit_d_ts_declaration_files,
                transpileOptionValue: undefined
            },
            {
                name: "sourceMap",
                type: "boolean",
                affectsEmit: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Generates_corresponding_map_file,
            },
            {
                name: "outFile",
                type: "string",
                affectsEmit: true,
                isFilePath: true,
                paramType: ts.Diagnostics.FILE,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Concatenate_and_emit_output_to_single_file,
                transpileOptionValue: undefined
            },
            {
                name: "outDir",
                type: "string",
                affectsEmit: true,
                isFilePath: true,
                paramType: ts.Diagnostics.DIRECTORY,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Redirect_output_structure_to_the_directory,
            },
            {
                name: "rootDir",
                type: "string",
                affectsEmit: true,
                isFilePath: true,
                paramType: ts.Diagnostics.LOCATION,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Specify_the_root_directory_of_input_files_Use_to_control_the_output_directory_structure_with_outDir,
            },
            {
                name: "composite",
                type: "boolean",
                affectsEmit: true,
                isTSConfigOnly: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Enable_project_compilation,
                transpileOptionValue: undefined
            },
            {
                name: "tsBuildInfoFile",
                type: "string",
                affectsEmit: true,
                isFilePath: true,
                paramType: ts.Diagnostics.FILE,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Specify_file_to_store_incremental_compilation_information,
                transpileOptionValue: undefined
            },
            {
                name: "removeComments",
                type: "boolean",
                affectsEmit: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Do_not_emit_comments_to_output,
            },
            {
                name: "noEmit",
                type: "boolean",
                affectsEmit: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Do_not_emit_outputs,
                transpileOptionValue: undefined
            },
            {
                name: "importHelpers",
                type: "boolean",
                affectsEmit: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Import_emit_helpers_from_tslib
            },
            {
                name: "importsNotUsedAsValues",
                type: ts.createMapFromTemplate({
                    remove: 0 /* Remove */,
                    preserve: 1 /* Preserve */,
                    error: 2 /* Error */
                }),
                affectsEmit: true,
                affectsSemanticDiagnostics: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Specify_emit_Slashchecking_behavior_for_imports_that_are_only_used_for_types
            },
            {
                name: "downlevelIteration",
                type: "boolean",
                affectsEmit: true,
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Provide_full_support_for_iterables_in_for_of_spread_and_destructuring_when_targeting_ES5_or_ES3
            },
            {
                name: "isolatedModules",
                type: "boolean",
                category: ts.Diagnostics.Basic_Options,
                description: ts.Diagnostics.Transpile_each_file_as_a_separate_module_similar_to_ts_transpileModule,
                transpileOptionValue: true
            },
            // Strict Type Checks
            {
                name: "strict",
                type: "boolean",
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Strict_Type_Checking_Options,
                description: ts.Diagnostics.Enable_all_strict_type_checking_options
            },
            {
                name: "noImplicitAny",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                strictFlag: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Strict_Type_Checking_Options,
                description: ts.Diagnostics.Raise_error_on_expressions_and_declarations_with_an_implied_any_type
            },
            {
                name: "strictNullChecks",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                strictFlag: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Strict_Type_Checking_Options,
                description: ts.Diagnostics.Enable_strict_null_checks
            },
            {
                name: "strictFunctionTypes",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                strictFlag: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Strict_Type_Checking_Options,
                description: ts.Diagnostics.Enable_strict_checking_of_function_types
            },
            {
                name: "strictBindCallApply",
                type: "boolean",
                strictFlag: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Strict_Type_Checking_Options,
                description: ts.Diagnostics.Enable_strict_bind_call_and_apply_methods_on_functions
            },
            {
                name: "strictPropertyInitialization",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                strictFlag: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Strict_Type_Checking_Options,
                description: ts.Diagnostics.Enable_strict_checking_of_property_initialization_in_classes
            },
            {
                name: "noImplicitThis",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                strictFlag: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Strict_Type_Checking_Options,
                description: ts.Diagnostics.Raise_error_on_this_expressions_with_an_implied_any_type,
            },
            {
                name: "alwaysStrict",
                type: "boolean",
                affectsSourceFile: true,
                strictFlag: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Strict_Type_Checking_Options,
                description: ts.Diagnostics.Parse_in_strict_mode_and_emit_use_strict_for_each_source_file
            },
            // Additional Checks
            {
                name: "noUnusedLocals",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Additional_Checks,
                description: ts.Diagnostics.Report_errors_on_unused_locals,
            },
            {
                name: "noUnusedParameters",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Additional_Checks,
                description: ts.Diagnostics.Report_errors_on_unused_parameters,
            },
            {
                name: "noImplicitReturns",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Additional_Checks,
                description: ts.Diagnostics.Report_error_when_not_all_code_paths_in_function_return_a_value
            },
            {
                name: "noFallthroughCasesInSwitch",
                type: "boolean",
                affectsBindDiagnostics: true,
                affectsSemanticDiagnostics: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Additional_Checks,
                description: ts.Diagnostics.Report_errors_for_fallthrough_cases_in_switch_statement
            },
            // Module Resolution
            {
                name: "moduleResolution",
                type: ts.createMapFromTemplate({
                    node: ts.ModuleResolutionKind.NodeJs,
                    classic: ts.ModuleResolutionKind.Classic,
                }),
                affectsModuleResolution: true,
                paramType: ts.Diagnostics.STRATEGY,
                category: ts.Diagnostics.Module_Resolution_Options,
                description: ts.Diagnostics.Specify_module_resolution_strategy_Colon_node_Node_js_or_classic_TypeScript_pre_1_6,
            },
            {
                name: "baseUrl",
                type: "string",
                affectsModuleResolution: true,
                isFilePath: true,
                category: ts.Diagnostics.Module_Resolution_Options,
                description: ts.Diagnostics.Base_directory_to_resolve_non_absolute_module_names
            },
            {
                // this option can only be specified in tsconfig.json
                // use type = object to copy the value as-is
                name: "paths",
                type: "object",
                affectsModuleResolution: true,
                isTSConfigOnly: true,
                category: ts.Diagnostics.Module_Resolution_Options,
                description: ts.Diagnostics.A_series_of_entries_which_re_map_imports_to_lookup_locations_relative_to_the_baseUrl,
                transpileOptionValue: undefined
            },
            {
                // this option can only be specified in tsconfig.json
                // use type = object to copy the value as-is
                name: "rootDirs",
                type: "list",
                isTSConfigOnly: true,
                element: {
                    name: "rootDirs",
                    type: "string",
                    isFilePath: true
                },
                affectsModuleResolution: true,
                category: ts.Diagnostics.Module_Resolution_Options,
                description: ts.Diagnostics.List_of_root_folders_whose_combined_content_represents_the_structure_of_the_project_at_runtime,
                transpileOptionValue: undefined
            },
            {
                name: "typeRoots",
                type: "list",
                element: {
                    name: "typeRoots",
                    type: "string",
                    isFilePath: true
                },
                affectsModuleResolution: true,
                category: ts.Diagnostics.Module_Resolution_Options,
                description: ts.Diagnostics.List_of_folders_to_include_type_definitions_from
            },
            {
                name: "types",
                type: "list",
                element: {
                    name: "types",
                    type: "string"
                },
                affectsModuleResolution: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Module_Resolution_Options,
                description: ts.Diagnostics.Type_declaration_files_to_be_included_in_compilation,
                transpileOptionValue: undefined
            },
            {
                name: "allowSyntheticDefaultImports",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                category: ts.Diagnostics.Module_Resolution_Options,
                description: ts.Diagnostics.Allow_default_imports_from_modules_with_no_default_export_This_does_not_affect_code_emit_just_typechecking
            },
            {
                name: "esModuleInterop",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                affectsEmit: true,
                showInSimplifiedHelpView: true,
                category: ts.Diagnostics.Module_Resolution_Options,
                description: ts.Diagnostics.Enables_emit_interoperability_between_CommonJS_and_ES_Modules_via_creation_of_namespace_objects_for_all_imports_Implies_allowSyntheticDefaultImports
            },
            {
                name: "preserveSymlinks",
                type: "boolean",
                category: ts.Diagnostics.Module_Resolution_Options,
                description: ts.Diagnostics.Do_not_resolve_the_real_path_of_symlinks,
            },
            {
                name: "allowUmdGlobalAccess",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                category: ts.Diagnostics.Module_Resolution_Options,
                description: ts.Diagnostics.Allow_accessing_UMD_globals_from_modules,
            },
            // Source Maps
            {
                name: "sourceRoot",
                type: "string",
                affectsEmit: true,
                paramType: ts.Diagnostics.LOCATION,
                category: ts.Diagnostics.Source_Map_Options,
                description: ts.Diagnostics.Specify_the_location_where_debugger_should_locate_TypeScript_files_instead_of_source_locations,
            },
            {
                name: "mapRoot",
                type: "string",
                affectsEmit: true,
                paramType: ts.Diagnostics.LOCATION,
                category: ts.Diagnostics.Source_Map_Options,
                description: ts.Diagnostics.Specify_the_location_where_debugger_should_locate_map_files_instead_of_generated_locations,
            },
            {
                name: "inlineSourceMap",
                type: "boolean",
                affectsEmit: true,
                category: ts.Diagnostics.Source_Map_Options,
                description: ts.Diagnostics.Emit_a_single_file_with_source_maps_instead_of_having_a_separate_file
            },
            {
                name: "inlineSources",
                type: "boolean",
                affectsEmit: true,
                category: ts.Diagnostics.Source_Map_Options,
                description: ts.Diagnostics.Emit_the_source_alongside_the_sourcemaps_within_a_single_file_requires_inlineSourceMap_or_sourceMap_to_be_set
            },
            // Experimental
            {
                name: "experimentalDecorators",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                category: ts.Diagnostics.Experimental_Options,
                description: ts.Diagnostics.Enables_experimental_support_for_ES7_decorators
            },
            {
                name: "emitDecoratorMetadata",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                affectsEmit: true,
                category: ts.Diagnostics.Experimental_Options,
                description: ts.Diagnostics.Enables_experimental_support_for_emitting_type_metadata_for_decorators
            },
            // Advanced
            {
                name: "jsxFactory",
                type: "string",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Specify_the_JSX_factory_function_to_use_when_targeting_react_JSX_emit_e_g_React_createElement_or_h
            },
            {
                name: "resolveJsonModule",
                type: "boolean",
                affectsModuleResolution: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Include_modules_imported_with_json_extension
            },
            {
                name: "out",
                type: "string",
                affectsEmit: true,
                isFilePath: false,
                // for correct behaviour, please use outFile
                category: ts.Diagnostics.Advanced_Options,
                paramType: ts.Diagnostics.FILE,
                description: ts.Diagnostics.Deprecated_Use_outFile_instead_Concatenate_and_emit_output_to_single_file,
                transpileOptionValue: undefined
            },
            {
                name: "reactNamespace",
                type: "string",
                affectsEmit: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Deprecated_Use_jsxFactory_instead_Specify_the_object_invoked_for_createElement_when_targeting_react_JSX_emit
            },
            {
                name: "skipDefaultLibCheck",
                type: "boolean",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Deprecated_Use_skipLibCheck_instead_Skip_type_checking_of_default_library_declaration_files
            },
            {
                name: "charset",
                type: "string",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.The_character_set_of_the_input_files
            },
            {
                name: "emitBOM",
                type: "boolean",
                affectsEmit: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Emit_a_UTF_8_Byte_Order_Mark_BOM_in_the_beginning_of_output_files
            },
            {
                name: "newLine",
                type: ts.createMapFromTemplate({
                    crlf: 0 /* CarriageReturnLineFeed */,
                    lf: 1 /* LineFeed */
                }),
                affectsEmit: true,
                paramType: ts.Diagnostics.NEWLINE,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Specify_the_end_of_line_sequence_to_be_used_when_emitting_files_Colon_CRLF_dos_or_LF_unix,
            },
            {
                name: "noErrorTruncation",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Do_not_truncate_error_messages
            },
            {
                name: "noLib",
                type: "boolean",
                affectsModuleResolution: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Do_not_include_the_default_library_file_lib_d_ts,
                // We are not returning a sourceFile for lib file when asked by the program,
                // so pass --noLib to avoid reporting a file not found error.
                transpileOptionValue: true
            },
            {
                name: "noResolve",
                type: "boolean",
                affectsModuleResolution: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Do_not_add_triple_slash_references_or_imported_modules_to_the_list_of_compiled_files,
                // We are not doing a full typecheck, we are not resolving the whole context,
                // so pass --noResolve to avoid reporting missing file errors.
                transpileOptionValue: true
            },
            {
                name: "stripInternal",
                type: "boolean",
                affectsEmit: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Do_not_emit_declarations_for_code_that_has_an_internal_annotation,
            },
            {
                name: "disableSizeLimit",
                type: "boolean",
                affectsSourceFile: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Disable_size_limitations_on_JavaScript_projects
            },
            {
                name: "disableSourceOfProjectReferenceRedirect",
                type: "boolean",
                isTSConfigOnly: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Disable_use_of_source_files_instead_of_declaration_files_from_referenced_projects
            },
            {
                name: "disableSolutionSearching",
                type: "boolean",
                isTSConfigOnly: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Disable_solution_searching_for_this_project
            },
            {
                name: "noImplicitUseStrict",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Do_not_emit_use_strict_directives_in_module_output
            },
            {
                name: "noEmitHelpers",
                type: "boolean",
                affectsEmit: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Do_not_generate_custom_helper_functions_like_extends_in_compiled_output
            },
            {
                name: "noEmitOnError",
                type: "boolean",
                affectsEmit: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Do_not_emit_outputs_if_any_errors_were_reported,
                transpileOptionValue: undefined
            },
            {
                name: "preserveConstEnums",
                type: "boolean",
                affectsEmit: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Do_not_erase_const_enum_declarations_in_generated_code
            },
            {
                name: "declarationDir",
                type: "string",
                affectsEmit: true,
                isFilePath: true,
                paramType: ts.Diagnostics.DIRECTORY,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Output_directory_for_generated_declaration_files,
                transpileOptionValue: undefined
            },
            {
                name: "skipLibCheck",
                type: "boolean",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Skip_type_checking_of_declaration_files,
            },
            {
                name: "allowUnusedLabels",
                type: "boolean",
                affectsBindDiagnostics: true,
                affectsSemanticDiagnostics: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Do_not_report_errors_on_unused_labels
            },
            {
                name: "allowUnreachableCode",
                type: "boolean",
                affectsBindDiagnostics: true,
                affectsSemanticDiagnostics: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Do_not_report_errors_on_unreachable_code
            },
            {
                name: "suppressExcessPropertyErrors",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Suppress_excess_property_checks_for_object_literals,
            },
            {
                name: "suppressImplicitAnyIndexErrors",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Suppress_noImplicitAny_errors_for_indexing_objects_lacking_index_signatures,
            },
            {
                name: "forceConsistentCasingInFileNames",
                type: "boolean",
                affectsModuleResolution: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Disallow_inconsistently_cased_references_to_the_same_file
            },
            {
                name: "maxNodeModuleJsDepth",
                type: "number",
                affectsModuleResolution: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.The_maximum_dependency_depth_to_search_under_node_modules_and_load_JavaScript_files
            },
            {
                name: "noStrictGenericChecks",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Disable_strict_checking_of_generic_signatures_in_function_types,
            },
            {
                name: "useDefineForClassFields",
                type: "boolean",
                affectsSemanticDiagnostics: true,
                affectsEmit: true,
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Emit_class_fields_with_Define_instead_of_Set,
            },
            {
                name: "keyofStringsOnly",
                type: "boolean",
                category: ts.Diagnostics.Advanced_Options,
                description: ts.Diagnostics.Resolve_keyof_to_string_valued_property_names_only_no_numbers_or_symbols,
            },
            {
                // A list of plugins to load in the language service
                name: "plugins",
                type: "list",
                isTSConfigOnly: true,
                element: {
                    name: "plugin",
                    type: "object"
                },
                description: ts.Diagnostics.List_of_language_service_plugins
            },
        ]);
        /* @internal */
        ts.semanticDiagnosticsOptionDeclarations = ts.optionDeclarations.filter(function (option) { return !!option.affectsSemanticDiagnostics; });
        /* @internal */
        ts.affectsEmitOptionDeclarations = ts.optionDeclarations.filter(function (option) { return !!option.affectsEmit; });
        /* @internal */
        ts.moduleResolutionOptionDeclarations = ts.optionDeclarations.filter(function (option) { return !!option.affectsModuleResolution; });
        /* @internal */
        ts.sourceFileAffectingCompilerOptions = ts.optionDeclarations.filter(function (option) {
            return !!option.affectsSourceFile || !!option.affectsModuleResolution || !!option.affectsBindDiagnostics;
        });
        /* @internal */
        ts.transpileOptionValueCompilerOptions = ts.optionDeclarations.filter(function (option) {
            return ts.hasProperty(option, "transpileOptionValue");
        });
        /* @internal */
        ts.buildOpts = __spreadArrays(ts.commonOptionsWithBuild, [
            {
                name: "verbose",
                shortName: "v",
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Enable_verbose_logging,
                type: "boolean"
            },
            {
                name: "dry",
                shortName: "d",
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Show_what_would_be_built_or_deleted_if_specified_with_clean,
                type: "boolean"
            },
            {
                name: "force",
                shortName: "f",
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Build_all_projects_including_those_that_appear_to_be_up_to_date,
                type: "boolean"
            },
            {
                name: "clean",
                category: ts.Diagnostics.Command_line_Options,
                description: ts.Diagnostics.Delete_the_outputs_of_all_projects,
                type: "boolean"
            }
        ]);
        /* @internal */
        ts.typeAcquisitionDeclarations = [
            {
                /* @deprecated typingOptions.enableAutoDiscovery
                 * Use typeAcquisition.enable instead.
                 */
                name: "enableAutoDiscovery",
                type: "boolean",
            },
            {
                name: "enable",
                type: "boolean",
            },
            {
                name: "include",
                type: "list",
                element: {
                    name: "include",
                    type: "string"
                }
            },
            {
                name: "exclude",
                type: "list",
                element: {
                    name: "exclude",
                    type: "string"
                }
            }
        ];
        /*@internal*/
        function createOptionNameMap(optionDeclarations) {
            var optionsNameMap = ts.createMap();
            var shortOptionNames = ts.createMap();
            ts.forEach(optionDeclarations, function (option) {
                optionsNameMap.set(option.name.toLowerCase(), option);
                if (option.shortName) {
                    shortOptionNames.set(option.shortName, option.name);
                }
            });
            return { optionsNameMap: optionsNameMap, shortOptionNames: shortOptionNames };
        }
        ts.createOptionNameMap = createOptionNameMap;
        var optionsNameMapCache;
        /* @internal */
        function getOptionsNameMap() {
            return optionsNameMapCache || (optionsNameMapCache = createOptionNameMap(ts.optionDeclarations));
        }
        ts.getOptionsNameMap = getOptionsNameMap;
        /* @internal */
        ts.defaultInitCompilerOptions = {
            module: ts.ModuleKind.CommonJS,
            target: 1 /* ES5 */,
            strict: true,
            esModuleInterop: true,
            forceConsistentCasingInFileNames: true
        };
        /* @internal */
        function convertEnableAutoDiscoveryToEnable(typeAcquisition) {
            // Convert deprecated typingOptions.enableAutoDiscovery to typeAcquisition.enable
            if (typeAcquisition && typeAcquisition.enableAutoDiscovery !== undefined && typeAcquisition.enable === undefined) {
                return {
                    enable: typeAcquisition.enableAutoDiscovery,
                    include: typeAcquisition.include || [],
                    exclude: typeAcquisition.exclude || []
                };
            }
            return typeAcquisition;
        }
        ts.convertEnableAutoDiscoveryToEnable = convertEnableAutoDiscoveryToEnable;
        /* @internal */
        function createCompilerDiagnosticForInvalidCustomType(opt) {
            return createDiagnosticForInvalidCustomType(opt, ts.createCompilerDiagnostic);
        }
        ts.createCompilerDiagnosticForInvalidCustomType = createCompilerDiagnosticForInvalidCustomType;
        function createDiagnosticForInvalidCustomType(opt, createDiagnostic) {
            var namesOfType = ts.arrayFrom(opt.type.keys()).map(function (key) { return "'" + key + "'"; }).join(", ");
            return createDiagnostic(ts.Diagnostics.Argument_for_0_option_must_be_Colon_1, "--" + opt.name, namesOfType);
        }
        /* @internal */
        function parseCustomTypeOption(opt, value, errors) {
            return convertJsonOptionOfCustomType(opt, trimString(value || ""), errors);
        }
        ts.parseCustomTypeOption = parseCustomTypeOption;
        /* @internal */
        function parseListTypeOption(opt, value, errors) {
            if (value === void 0) {
                value = "";
            }
            value = trimString(value);
            if (ts.startsWith(value, "-")) {
                return undefined;
            }
            if (value === "") {
                return [];
            }
            var values = value.split(",");
            switch (opt.element.type) {
                case "number":
                    return ts.map(values, parseInt);
                case "string":
                    return ts.map(values, function (v) { return v || ""; });
                default:
                    return ts.mapDefined(values, function (v) { return parseCustomTypeOption(opt.element, v, errors); });
            }
        }
        ts.parseListTypeOption = parseListTypeOption;
        function getOptionName(option) {
            return option.name;
        }
        function createUnknownOptionError(unknownOption, diagnostics, createDiagnostics, unknownOptionErrorText) {
            var possibleOption = ts.getSpellingSuggestion(unknownOption, diagnostics.optionDeclarations, getOptionName);
            return possibleOption ?
                createDiagnostics(diagnostics.unknownDidYouMeanDiagnostic, unknownOptionErrorText || unknownOption, possibleOption.name) :
                createDiagnostics(diagnostics.unknownOptionDiagnostic, unknownOptionErrorText || unknownOption);
        }
        /*@internal*/
        function parseCommandLineWorker(diagnostics, commandLine, readFile) {
            var options = {};
            var watchOptions;
            var fileNames = [];
            var errors = [];
            parseStrings(commandLine);
            return {
                options: options,
                watchOptions: watchOptions,
                fileNames: fileNames,
                errors: errors
            };
            function parseStrings(args) {
                var i = 0;
                while (i < args.length) {
                    var s = args[i];
                    i++;
                    if (s.charCodeAt(0) === 64 /* at */) {
                        parseResponseFile(s.slice(1));
                    }
                    else if (s.charCodeAt(0) === 45 /* minus */) {
                        var inputOptionName = s.slice(s.charCodeAt(1) === 45 /* minus */ ? 2 : 1);
                        var opt = getOptionDeclarationFromName(diagnostics.getOptionsNameMap, inputOptionName, /*allowShort*/ true);
                        if (opt) {
                            i = parseOptionValue(args, i, diagnostics, opt, options, errors);
                        }
                        else {
                            var watchOpt = getOptionDeclarationFromName(watchOptionsDidYouMeanDiagnostics.getOptionsNameMap, inputOptionName, /*allowShort*/ true);
                            if (watchOpt) {
                                i = parseOptionValue(args, i, watchOptionsDidYouMeanDiagnostics, watchOpt, watchOptions || (watchOptions = {}), errors);
                            }
                            else {
                                errors.push(createUnknownOptionError(inputOptionName, diagnostics, ts.createCompilerDiagnostic, s));
                            }
                        }
                    }
                    else {
                        fileNames.push(s);
                    }
                }
            }
            function parseResponseFile(fileName) {
                var text = readFile ? readFile(fileName) : ts.sys.readFile(fileName);
                if (!text) {
                    errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.File_0_not_found, fileName));
                    return;
                }
                var args = [];
                var pos = 0;
                while (true) {
                    while (pos < text.length && text.charCodeAt(pos) <= 32 /* space */)
                        pos++;
                    if (pos >= text.length)
                        break;
                    var start = pos;
                    if (text.charCodeAt(start) === 34 /* doubleQuote */) {
                        pos++;
                        while (pos < text.length && text.charCodeAt(pos) !== 34 /* doubleQuote */)
                            pos++;
                        if (pos < text.length) {
                            args.push(text.substring(start + 1, pos));
                            pos++;
                        }
                        else {
                            errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Unterminated_quoted_string_in_response_file_0, fileName));
                        }
                    }
                    else {
                        while (text.charCodeAt(pos) > 32 /* space */)
                            pos++;
                        args.push(text.substring(start, pos));
                    }
                }
                parseStrings(args);
            }
        }
        ts.parseCommandLineWorker = parseCommandLineWorker;
        function parseOptionValue(args, i, diagnostics, opt, options, errors) {
            if (opt.isTSConfigOnly) {
                var optValue = args[i];
                if (optValue === "null") {
                    options[opt.name] = undefined;
                    i++;
                }
                else if (opt.type === "boolean") {
                    if (optValue === "false") {
                        options[opt.name] = false;
                        i++;
                    }
                    else {
                        if (optValue === "true")
                            i++;
                        errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_false_or_null_on_command_line, opt.name));
                    }
                }
                else {
                    errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Option_0_can_only_be_specified_in_tsconfig_json_file_or_set_to_null_on_command_line, opt.name));
                    if (optValue && !ts.startsWith(optValue, "-"))
                        i++;
                }
            }
            else {
                // Check to see if no argument was provided (e.g. "--locale" is the last command-line argument).
                if (!args[i] && opt.type !== "boolean") {
                    errors.push(ts.createCompilerDiagnostic(diagnostics.optionTypeMismatchDiagnostic, opt.name, getCompilerOptionValueTypeString(opt)));
                }
                if (args[i] !== "null") {
                    switch (opt.type) {
                        case "number":
                            options[opt.name] = parseInt(args[i]);
                            i++;
                            break;
                        case "boolean":
                            // boolean flag has optional value true, false, others
                            var optValue = args[i];
                            options[opt.name] = optValue !== "false";
                            // consume next argument as boolean flag value
                            if (optValue === "false" || optValue === "true") {
                                i++;
                            }
                            break;
                        case "string":
                            options[opt.name] = args[i] || "";
                            i++;
                            break;
                        case "list":
                            var result = parseListTypeOption(opt, args[i], errors);
                            options[opt.name] = result || [];
                            if (result) {
                                i++;
                            }
                            break;
                        // If not a primitive, the possible types are specified in what is effectively a map of options.
                        default:
                            options[opt.name] = parseCustomTypeOption(opt, args[i], errors);
                            i++;
                            break;
                    }
                }
                else {
                    options[opt.name] = undefined;
                    i++;
                }
            }
            return i;
        }
        /*@internal*/
        ts.compilerOptionsDidYouMeanDiagnostics = {
            getOptionsNameMap: getOptionsNameMap,
            optionDeclarations: ts.optionDeclarations,
            unknownOptionDiagnostic: ts.Diagnostics.Unknown_compiler_option_0,
            unknownDidYouMeanDiagnostic: ts.Diagnostics.Unknown_compiler_option_0_Did_you_mean_1,
            optionTypeMismatchDiagnostic: ts.Diagnostics.Compiler_option_0_expects_an_argument
        };
        function parseCommandLine(commandLine, readFile) {
            return parseCommandLineWorker(ts.compilerOptionsDidYouMeanDiagnostics, commandLine, readFile);
        }
        ts.parseCommandLine = parseCommandLine;
        /** @internal */
        function getOptionFromName(optionName, allowShort) {
            return getOptionDeclarationFromName(getOptionsNameMap, optionName, allowShort);
        }
        ts.getOptionFromName = getOptionFromName;
        function getOptionDeclarationFromName(getOptionNameMap, optionName, allowShort) {
            if (allowShort === void 0) {
                allowShort = false;
            }
            optionName = optionName.toLowerCase();
            var _a = getOptionNameMap(), optionsNameMap = _a.optionsNameMap, shortOptionNames = _a.shortOptionNames;
            // Try to translate short option names to their full equivalents.
            if (allowShort) {
                var short = shortOptionNames.get(optionName);
                if (short !== undefined) {
                    optionName = short;
                }
            }
            return optionsNameMap.get(optionName);
        }
        var buildOptionsNameMapCache;
        function getBuildOptionsNameMap() {
            return buildOptionsNameMapCache || (buildOptionsNameMapCache = createOptionNameMap(ts.buildOpts));
        }
        var buildOptionsDidYouMeanDiagnostics = {
            getOptionsNameMap: getBuildOptionsNameMap,
            optionDeclarations: ts.buildOpts,
            unknownOptionDiagnostic: ts.Diagnostics.Unknown_build_option_0,
            unknownDidYouMeanDiagnostic: ts.Diagnostics.Unknown_build_option_0_Did_you_mean_1,
            optionTypeMismatchDiagnostic: ts.Diagnostics.Build_option_0_requires_a_value_of_type_1
        };
        /*@internal*/
        function parseBuildCommand(args) {
            var _a = parseCommandLineWorker(buildOptionsDidYouMeanDiagnostics, args), options = _a.options, watchOptions = _a.watchOptions, projects = _a.fileNames, errors = _a.errors;
            var buildOptions = options;
            if (projects.length === 0) {
                // tsc -b invoked with no extra arguments; act as if invoked with "tsc -b ."
                projects.push(".");
            }
            // Nonsensical combinations
            if (buildOptions.clean && buildOptions.force) {
                errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Options_0_and_1_cannot_be_combined, "clean", "force"));
            }
            if (buildOptions.clean && buildOptions.verbose) {
                errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Options_0_and_1_cannot_be_combined, "clean", "verbose"));
            }
            if (buildOptions.clean && buildOptions.watch) {
                errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Options_0_and_1_cannot_be_combined, "clean", "watch"));
            }
            if (buildOptions.watch && buildOptions.dry) {
                errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Options_0_and_1_cannot_be_combined, "watch", "dry"));
            }
            return { buildOptions: buildOptions, watchOptions: watchOptions, projects: projects, errors: errors };
        }
        ts.parseBuildCommand = parseBuildCommand;
        /* @internal */
        function getDiagnosticText(_message) {
            var _args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                _args[_i - 1] = arguments[_i];
            }
            var diagnostic = ts.createCompilerDiagnostic.apply(undefined, arguments);
            return diagnostic.messageText;
        }
        ts.getDiagnosticText = getDiagnosticText;
        /**
         * Reads the config file, reports errors if any and exits if the config file cannot be found
         */
        function getParsedCommandLineOfConfigFile(configFileName, optionsToExtend, host, extendedConfigCache, watchOptionsToExtend) {
            var configFileText;
            try {
                configFileText = host.readFile(configFileName);
            }
            catch (e) {
                var error = ts.createCompilerDiagnostic(ts.Diagnostics.Cannot_read_file_0_Colon_1, configFileName, e.message);
                host.onUnRecoverableConfigFileDiagnostic(error);
                return undefined;
            }
            if (!configFileText) {
                var error = ts.createCompilerDiagnostic(ts.Diagnostics.File_0_not_found, configFileName);
                host.onUnRecoverableConfigFileDiagnostic(error);
                return undefined;
            }
            var result = ts.parseJsonText(configFileName, configFileText);
            var cwd = host.getCurrentDirectory();
            result.path = ts.toPath(configFileName, cwd, ts.createGetCanonicalFileName(host.useCaseSensitiveFileNames));
            result.resolvedPath = result.path;
            result.originalFileName = result.fileName;
            return parseJsonSourceFileConfigFileContent(result, host, ts.getNormalizedAbsolutePath(ts.getDirectoryPath(configFileName), cwd), optionsToExtend, ts.getNormalizedAbsolutePath(configFileName, cwd), 
            /*resolutionStack*/ undefined, 
            /*extraFileExtension*/ undefined, extendedConfigCache, watchOptionsToExtend);
        }
        ts.getParsedCommandLineOfConfigFile = getParsedCommandLineOfConfigFile;
        /**
         * Read tsconfig.json file
         * @param fileName The path to the config file
         */
        function readConfigFile(fileName, readFile) {
            var textOrDiagnostic = tryReadFile(fileName, readFile);
            return ts.isString(textOrDiagnostic) ? parseConfigFileTextToJson(fileName, textOrDiagnostic) : { config: {}, error: textOrDiagnostic };
        }
        ts.readConfigFile = readConfigFile;
        /**
         * Parse the text of the tsconfig.json file
         * @param fileName The path to the config file
         * @param jsonText The text of the config file
         */
        function parseConfigFileTextToJson(fileName, jsonText) {
            var jsonSourceFile = ts.parseJsonText(fileName, jsonText);
            return {
                config: convertToObject(jsonSourceFile, jsonSourceFile.parseDiagnostics),
                error: jsonSourceFile.parseDiagnostics.length ? jsonSourceFile.parseDiagnostics[0] : undefined
            };
        }
        ts.parseConfigFileTextToJson = parseConfigFileTextToJson;
        /**
         * Read tsconfig.json file
         * @param fileName The path to the config file
         */
        function readJsonConfigFile(fileName, readFile) {
            var textOrDiagnostic = tryReadFile(fileName, readFile);
            return ts.isString(textOrDiagnostic) ? ts.parseJsonText(fileName, textOrDiagnostic) : { parseDiagnostics: [textOrDiagnostic] };
        }
        ts.readJsonConfigFile = readJsonConfigFile;
        function tryReadFile(fileName, readFile) {
            var text;
            try {
                text = readFile(fileName);
            }
            catch (e) {
                return ts.createCompilerDiagnostic(ts.Diagnostics.Cannot_read_file_0_Colon_1, fileName, e.message);
            }
            return text === undefined ? ts.createCompilerDiagnostic(ts.Diagnostics.The_specified_path_does_not_exist_Colon_0, fileName) : text;
        }
        function commandLineOptionsToMap(options) {
            return ts.arrayToMap(options, getOptionName);
        }
        var typeAcquisitionDidYouMeanDiagnostics = {
            optionDeclarations: ts.typeAcquisitionDeclarations,
            unknownOptionDiagnostic: ts.Diagnostics.Unknown_type_acquisition_option_0,
            unknownDidYouMeanDiagnostic: ts.Diagnostics.Unknown_type_acquisition_option_0_Did_you_mean_1,
        };
        var watchOptionsNameMapCache;
        function getWatchOptionsNameMap() {
            return watchOptionsNameMapCache || (watchOptionsNameMapCache = createOptionNameMap(ts.optionsForWatch));
        }
        var watchOptionsDidYouMeanDiagnostics = {
            getOptionsNameMap: getWatchOptionsNameMap,
            optionDeclarations: ts.optionsForWatch,
            unknownOptionDiagnostic: ts.Diagnostics.Unknown_watch_option_0,
            unknownDidYouMeanDiagnostic: ts.Diagnostics.Unknown_watch_option_0_Did_you_mean_1,
            optionTypeMismatchDiagnostic: ts.Diagnostics.Watch_option_0_requires_a_value_of_type_1
        };
        var commandLineCompilerOptionsMapCache;
        function getCommandLineCompilerOptionsMap() {
            return commandLineCompilerOptionsMapCache || (commandLineCompilerOptionsMapCache = commandLineOptionsToMap(ts.optionDeclarations));
        }
        var commandLineWatchOptionsMapCache;
        function getCommandLineWatchOptionsMap() {
            return commandLineWatchOptionsMapCache || (commandLineWatchOptionsMapCache = commandLineOptionsToMap(ts.optionsForWatch));
        }
        var commandLineTypeAcquisitionMapCache;
        function getCommandLineTypeAcquisitionMap() {
            return commandLineTypeAcquisitionMapCache || (commandLineTypeAcquisitionMapCache = commandLineOptionsToMap(ts.typeAcquisitionDeclarations));
        }
        var _tsconfigRootOptions;
        function getTsconfigRootOptionsMap() {
            if (_tsconfigRootOptions === undefined) {
                _tsconfigRootOptions = {
                    name: undefined,
                    type: "object",
                    elementOptions: commandLineOptionsToMap([
                        {
                            name: "compilerOptions",
                            type: "object",
                            elementOptions: getCommandLineCompilerOptionsMap(),
                            extraKeyDiagnostics: ts.compilerOptionsDidYouMeanDiagnostics,
                        },
                        {
                            name: "watchOptions",
                            type: "object",
                            elementOptions: getCommandLineWatchOptionsMap(),
                            extraKeyDiagnostics: watchOptionsDidYouMeanDiagnostics,
                        },
                        {
                            name: "typingOptions",
                            type: "object",
                            elementOptions: getCommandLineTypeAcquisitionMap(),
                            extraKeyDiagnostics: typeAcquisitionDidYouMeanDiagnostics,
                        },
                        {
                            name: "typeAcquisition",
                            type: "object",
                            elementOptions: getCommandLineTypeAcquisitionMap(),
                            extraKeyDiagnostics: typeAcquisitionDidYouMeanDiagnostics
                        },
                        {
                            name: "extends",
                            type: "string"
                        },
                        {
                            name: "references",
                            type: "list",
                            element: {
                                name: "references",
                                type: "object"
                            }
                        },
                        {
                            name: "files",
                            type: "list",
                            element: {
                                name: "files",
                                type: "string"
                            }
                        },
                        {
                            name: "include",
                            type: "list",
                            element: {
                                name: "include",
                                type: "string"
                            }
                        },
                        {
                            name: "exclude",
                            type: "list",
                            element: {
                                name: "exclude",
                                type: "string"
                            }
                        },
                        ts.compileOnSaveCommandLineOption
                    ])
                };
            }
            return _tsconfigRootOptions;
        }
        /**
         * Convert the json syntax tree into the json value
         */
        function convertToObject(sourceFile, errors) {
            return convertToObjectWorker(sourceFile, errors, /*returnValue*/ true, /*knownRootOptions*/ undefined, /*jsonConversionNotifier*/ undefined);
        }
        ts.convertToObject = convertToObject;
        /**
         * Convert the json syntax tree into the json value and report errors
         * This returns the json value (apart from checking errors) only if returnValue provided is true.
         * Otherwise it just checks the errors and returns undefined
         */
        /*@internal*/
        function convertToObjectWorker(sourceFile, errors, returnValue, knownRootOptions, jsonConversionNotifier) {
            if (!sourceFile.statements.length) {
                return returnValue ? {} : undefined;
            }
            return convertPropertyValueToJson(sourceFile.statements[0].expression, knownRootOptions);
            function isRootOptionMap(knownOptions) {
                return knownRootOptions && knownRootOptions.elementOptions === knownOptions;
            }
            function convertObjectLiteralExpressionToJson(node, knownOptions, extraKeyDiagnostics, parentOption) {
                var result = returnValue ? {} : undefined;
                var _loop_3 = function (element) {
                    if (element.kind !== 281 /* PropertyAssignment */) {
                        errors.push(ts.createDiagnosticForNodeInSourceFile(sourceFile, element, ts.Diagnostics.Property_assignment_expected));
                        return "continue";
                    }
                    if (element.questionToken) {
                        errors.push(ts.createDiagnosticForNodeInSourceFile(sourceFile, element.questionToken, ts.Diagnostics.The_0_modifier_can_only_be_used_in_TypeScript_files, "?"));
                    }
                    if (!isDoubleQuotedString(element.name)) {
                        errors.push(ts.createDiagnosticForNodeInSourceFile(sourceFile, element.name, ts.Diagnostics.String_literal_with_double_quotes_expected));
                    }
                    var textOfKey = ts.getTextOfPropertyName(element.name);
                    var keyText = textOfKey && ts.unescapeLeadingUnderscores(textOfKey);
                    var option = keyText && knownOptions ? knownOptions.get(keyText) : undefined;
                    if (keyText && extraKeyDiagnostics && !option) {
                        if (knownOptions) {
                            errors.push(createUnknownOptionError(keyText, extraKeyDiagnostics, function (message, arg0, arg1) { return ts.createDiagnosticForNodeInSourceFile(sourceFile, element.name, message, arg0, arg1); }));
                        }
                        else {
                            errors.push(ts.createDiagnosticForNodeInSourceFile(sourceFile, element.name, extraKeyDiagnostics.unknownOptionDiagnostic, keyText));
                        }
                    }
                    var value = convertPropertyValueToJson(element.initializer, option);
                    if (typeof keyText !== "undefined") {
                        if (returnValue) {
                            result[keyText] = value;
                        }
                        // Notify key value set, if user asked for it
                        if (jsonConversionNotifier &&
                            // Current callbacks are only on known parent option or if we are setting values in the root
                            (parentOption || isRootOptionMap(knownOptions))) {
                            var isValidOptionValue = isCompilerOptionsValue(option, value);
                            if (parentOption) {
                                if (isValidOptionValue) {
                                    // Notify option set in the parent if its a valid option value
                                    jsonConversionNotifier.onSetValidOptionKeyValueInParent(parentOption, option, value);
                                }
                            }
                            else if (isRootOptionMap(knownOptions)) {
                                if (isValidOptionValue) {
                                    // Notify about the valid root key value being set
                                    jsonConversionNotifier.onSetValidOptionKeyValueInRoot(keyText, element.name, value, element.initializer);
                                }
                                else if (!option) {
                                    // Notify about the unknown root key value being set
                                    jsonConversionNotifier.onSetUnknownOptionKeyValueInRoot(keyText, element.name, value, element.initializer);
                                }
                            }
                        }
                    }
                };
                for (var _i = 0, _a = node.properties; _i < _a.length; _i++) {
                    var element = _a[_i];
                    _loop_3(element);
                }
                return result;
            }
            function convertArrayLiteralExpressionToJson(elements, elementOption) {
                if (!returnValue) {
                    return elements.forEach(function (element) { return convertPropertyValueToJson(element, elementOption); });
                }
                // Filter out invalid values
                return ts.filter(elements.map(function (element) { return convertPropertyValueToJson(element, elementOption); }), function (v) { return v !== undefined; });
            }
            function convertPropertyValueToJson(valueExpression, option) {
                switch (valueExpression.kind) {
                    case 106 /* TrueKeyword */:
                        reportInvalidOptionValue(option && option.type !== "boolean");
                        return true;
                    case 91 /* FalseKeyword */:
                        reportInvalidOptionValue(option && option.type !== "boolean");
                        return false;
                    case 100 /* NullKeyword */:
                        reportInvalidOptionValue(option && option.name === "extends"); // "extends" is the only option we don't allow null/undefined for
                        return null; // eslint-disable-line no-null/no-null
                    case 10 /* StringLiteral */:
                        if (!isDoubleQuotedString(valueExpression)) {
                            errors.push(ts.createDiagnosticForNodeInSourceFile(sourceFile, valueExpression, ts.Diagnostics.String_literal_with_double_quotes_expected));
                        }
                        reportInvalidOptionValue(option && (ts.isString(option.type) && option.type !== "string"));
                        var text = valueExpression.text;
                        if (option && !ts.isString(option.type)) {
                            var customOption = option;
                            // Validate custom option type
                            if (!customOption.type.has(text.toLowerCase())) {
                                errors.push(createDiagnosticForInvalidCustomType(customOption, function (message, arg0, arg1) { return ts.createDiagnosticForNodeInSourceFile(sourceFile, valueExpression, message, arg0, arg1); }));
                            }
                        }
                        return text;
                    case 8 /* NumericLiteral */:
                        reportInvalidOptionValue(option && option.type !== "number");
                        return Number(valueExpression.text);
                    case 207 /* PrefixUnaryExpression */:
                        if (valueExpression.operator !== 40 /* MinusToken */ || valueExpression.operand.kind !== 8 /* NumericLiteral */) {
                            break; // not valid JSON syntax
                        }
                        reportInvalidOptionValue(option && option.type !== "number");
                        return -Number(valueExpression.operand.text);
                    case 193 /* ObjectLiteralExpression */:
                        reportInvalidOptionValue(option && option.type !== "object");
                        var objectLiteralExpression = valueExpression;
                        // Currently having element option declaration in the tsconfig with type "object"
                        // determines if it needs onSetValidOptionKeyValueInParent callback or not
                        // At moment there are only "compilerOptions", "typeAcquisition" and "typingOptions"
                        // that satifies it and need it to modify options set in them (for normalizing file paths)
                        // vs what we set in the json
                        // If need arises, we can modify this interface and callbacks as needed
                        if (option) {
                            var _a = option, elementOptions = _a.elementOptions, extraKeyDiagnostics = _a.extraKeyDiagnostics, optionName = _a.name;
                            return convertObjectLiteralExpressionToJson(objectLiteralExpression, elementOptions, extraKeyDiagnostics, optionName);
                        }
                        else {
                            return convertObjectLiteralExpressionToJson(objectLiteralExpression, /* knownOptions*/ undefined, 
                            /*extraKeyDiagnosticMessage */ undefined, /*parentOption*/ undefined);
                        }
                    case 192 /* ArrayLiteralExpression */:
                        reportInvalidOptionValue(option && option.type !== "list");
                        return convertArrayLiteralExpressionToJson(valueExpression.elements, option && option.element);
                }
                // Not in expected format
                if (option) {
                    reportInvalidOptionValue(/*isError*/ true);
                }
                else {
                    errors.push(ts.createDiagnosticForNodeInSourceFile(sourceFile, valueExpression, ts.Diagnostics.Property_value_can_only_be_string_literal_numeric_literal_true_false_null_object_literal_or_array_literal));
                }
                return undefined;
                function reportInvalidOptionValue(isError) {
                    if (isError) {
                        errors.push(ts.createDiagnosticForNodeInSourceFile(sourceFile, valueExpression, ts.Diagnostics.Compiler_option_0_requires_a_value_of_type_1, option.name, getCompilerOptionValueTypeString(option)));
                    }
                }
            }
            function isDoubleQuotedString(node) {
                return ts.isStringLiteral(node) && ts.isStringDoubleQuoted(node, sourceFile);
            }
        }
        ts.convertToObjectWorker = convertToObjectWorker;
        function getCompilerOptionValueTypeString(option) {
            return option.type === "list" ?
                "Array" :
                ts.isString(option.type) ? option.type : "string";
        }
        function isCompilerOptionsValue(option, value) {
            if (option) {
                if (isNullOrUndefined(value))
                    return true; // All options are undefinable/nullable
                if (option.type === "list") {
                    return ts.isArray(value);
                }
                var expectedType = ts.isString(option.type) ? option.type : "string";
                return typeof value === expectedType;
            }
            return false;
        }
        /**
         * Generate an uncommented, complete tsconfig for use with "--showConfig"
         * @param configParseResult options to be generated into tsconfig.json
         * @param configFileName name of the parsed config file - output paths will be generated relative to this
         * @param host provides current directory and case sensitivity services
         */
        /** @internal */
        function convertToTSConfig(configParseResult, configFileName, host) {
            var getCanonicalFileName = ts.createGetCanonicalFileName(host.useCaseSensitiveFileNames);
            var files = ts.map(ts.filter(configParseResult.fileNames, (!configParseResult.configFileSpecs || !configParseResult.configFileSpecs.validatedIncludeSpecs) ? function (_) { return true; } : matchesSpecs(configFileName, configParseResult.configFileSpecs.validatedIncludeSpecs, configParseResult.configFileSpecs.validatedExcludeSpecs, host)), function (f) { return ts.getRelativePathFromFile(ts.getNormalizedAbsolutePath(configFileName, host.getCurrentDirectory()), ts.getNormalizedAbsolutePath(f, host.getCurrentDirectory()), getCanonicalFileName); });
            var optionMap = serializeCompilerOptions(configParseResult.options, { configFilePath: ts.getNormalizedAbsolutePath(configFileName, host.getCurrentDirectory()), useCaseSensitiveFileNames: host.useCaseSensitiveFileNames });
            var watchOptionMap = configParseResult.watchOptions && serializeWatchOptions(configParseResult.watchOptions);
            var config = __assign(__assign({ compilerOptions: __assign(__assign({}, optionMapToObject(optionMap)), { showConfig: undefined, configFile: undefined, configFilePath: undefined, help: undefined, init: undefined, listFiles: undefined, listEmittedFiles: undefined, project: undefined, build: undefined, version: undefined }), watchOptions: watchOptionMap && optionMapToObject(watchOptionMap), references: ts.map(configParseResult.projectReferences, function (r) { return (__assign(__assign({}, r), { path: r.originalPath ? r.originalPath : "", originalPath: undefined })); }), files: ts.length(files) ? files : undefined }, (configParseResult.configFileSpecs ? {
                include: filterSameAsDefaultInclude(configParseResult.configFileSpecs.validatedIncludeSpecs),
                exclude: configParseResult.configFileSpecs.validatedExcludeSpecs
            } : {})), { compileOnSave: !!configParseResult.compileOnSave ? true : undefined });
            return config;
        }
        ts.convertToTSConfig = convertToTSConfig;
        function optionMapToObject(optionMap) {
            return __assign({}, ts.arrayFrom(optionMap.entries()).reduce(function (prev, cur) {
                var _a;
                return (__assign(__assign({}, prev), (_a = {}, _a[cur[0]] = cur[1], _a)));
            }, {}));
        }
        function filterSameAsDefaultInclude(specs) {
            if (!ts.length(specs))
                return undefined;
            if (ts.length(specs) !== 1)
                return specs;
            if (specs[0] === "**/*")
                return undefined;
            return specs;
        }
        function matchesSpecs(path, includeSpecs, excludeSpecs, host) {
            if (!includeSpecs)
                return function (_) { return true; };
            var patterns = ts.getFileMatcherPatterns(path, excludeSpecs, includeSpecs, host.useCaseSensitiveFileNames, host.getCurrentDirectory());
            var excludeRe = patterns.excludePattern && ts.getRegexFromPattern(patterns.excludePattern, host.useCaseSensitiveFileNames);
            var includeRe = patterns.includeFilePattern && ts.getRegexFromPattern(patterns.includeFilePattern, host.useCaseSensitiveFileNames);
            if (includeRe) {
                if (excludeRe) {
                    return function (path) { return !(includeRe.test(path) && !excludeRe.test(path)); };
                }
                return function (path) { return !includeRe.test(path); };
            }
            if (excludeRe) {
                return function (path) { return excludeRe.test(path); };
            }
            return function (_) { return true; };
        }
        function getCustomTypeMapOfCommandLineOption(optionDefinition) {
            if (optionDefinition.type === "string" || optionDefinition.type === "number" || optionDefinition.type === "boolean" || optionDefinition.type === "object") {
                // this is of a type CommandLineOptionOfPrimitiveType
                return undefined;
            }
            else if (optionDefinition.type === "list") {
                return getCustomTypeMapOfCommandLineOption(optionDefinition.element);
            }
            else {
                return optionDefinition.type;
            }
        }
        function getNameOfCompilerOptionValue(value, customTypeMap) {
            // There is a typeMap associated with this command-line option so use it to map value back to its name
            return ts.forEachEntry(customTypeMap, function (mapValue, key) {
                if (mapValue === value) {
                    return key;
                }
            });
        }
        function serializeCompilerOptions(options, pathOptions) {
            return serializeOptionBaseObject(options, getOptionsNameMap(), pathOptions);
        }
        function serializeWatchOptions(options) {
            return serializeOptionBaseObject(options, getWatchOptionsNameMap());
        }
        function serializeOptionBaseObject(options, _a, pathOptions) {
            var optionsNameMap = _a.optionsNameMap;
            var result = ts.createMap();
            var getCanonicalFileName = pathOptions && ts.createGetCanonicalFileName(pathOptions.useCaseSensitiveFileNames);
            var _loop_4 = function (name) {
                if (ts.hasProperty(options, name)) {
                    // tsconfig only options cannot be specified via command line,
                    // so we can assume that only types that can appear here string | number | boolean
                    if (optionsNameMap.has(name) && optionsNameMap.get(name).category === ts.Diagnostics.Command_line_Options) {
                        return "continue";
                    }
                    var value = options[name];
                    var optionDefinition = optionsNameMap.get(name.toLowerCase());
                    if (optionDefinition) {
                        var customTypeMap_1 = getCustomTypeMapOfCommandLineOption(optionDefinition);
                        if (!customTypeMap_1) {
                            // There is no map associated with this compiler option then use the value as-is
                            // This is the case if the value is expect to be string, number, boolean or list of string
                            if (pathOptions && optionDefinition.isFilePath) {
                                result.set(name, ts.getRelativePathFromFile(pathOptions.configFilePath, ts.getNormalizedAbsolutePath(value, ts.getDirectoryPath(pathOptions.configFilePath)), getCanonicalFileName));
                            }
                            else {
                                result.set(name, value);
                            }
                        }
                        else {
                            if (optionDefinition.type === "list") {
                                result.set(name, value.map(function (element) { return getNameOfCompilerOptionValue(element, customTypeMap_1); })); // TODO: GH#18217
                            }
                            else {
                                // There is a typeMap associated with this command-line option so use it to map value back to its name
                                result.set(name, getNameOfCompilerOptionValue(value, customTypeMap_1));
                            }
                        }
                    }
                }
            };
            for (var name in options) {
                _loop_4(name);
            }
            return result;
        }
        /**
         * Generate tsconfig configuration when running command line "--init"
         * @param options commandlineOptions to be generated into tsconfig.json
         * @param fileNames array of filenames to be generated into tsconfig.json
         */
        /* @internal */
        function generateTSConfig(options, fileNames, newLine) {
            var compilerOptions = ts.extend(options, ts.defaultInitCompilerOptions);
            var compilerOptionsMap = serializeCompilerOptions(compilerOptions);
            return writeConfigurations();
            function getDefaultValueForOption(option) {
                switch (option.type) {
                    case "number":
                        return 1;
                    case "boolean":
                        return true;
                    case "string":
                        return option.isFilePath ? "./" : "";
                    case "list":
                        return [];
                    case "object":
                        return {};
                    default:
                        var iterResult = option.type.keys().next();
                        if (!iterResult.done)
                            return iterResult.value;
                        return ts.Debug.fail("Expected 'option.type' to have entries.");
                }
            }
            function makePadding(paddingLength) {
                return Array(paddingLength + 1).join(" ");
            }
            function isAllowedOption(_a) {
                var category = _a.category, name = _a.name;
                // Skip options which do not have a category or have category `Command_line_Options`
                // Exclude all possible `Advanced_Options` in tsconfig.json which were NOT defined in command line
                return category !== undefined
                    && category !== ts.Diagnostics.Command_line_Options
                    && (category !== ts.Diagnostics.Advanced_Options || compilerOptionsMap.has(name));
            }
            function writeConfigurations() {
                // Filter applicable options to place in the file
                var categorizedOptions = ts.createMultiMap();
                for (var _i = 0, optionDeclarations_1 = ts.optionDeclarations; _i < optionDeclarations_1.length; _i++) {
                    var option = optionDeclarations_1[_i];
                    var category = option.category;
                    if (isAllowedOption(option)) {
                        categorizedOptions.add(ts.getLocaleSpecificMessage(category), option);
                    }
                }
                // Serialize all options and their descriptions
                var marginLength = 0;
                var seenKnownKeys = 0;
                var nameColumn = [];
                var descriptionColumn = [];
                categorizedOptions.forEach(function (options, category) {
                    if (nameColumn.length !== 0) {
                        nameColumn.push("");
                        descriptionColumn.push("");
                    }
                    nameColumn.push("/* " + category + " */");
                    descriptionColumn.push("");
                    for (var _i = 0, options_1 = options; _i < options_1.length; _i++) {
                        var option = options_1[_i];
                        var optionName = void 0;
                        if (compilerOptionsMap.has(option.name)) {
                            optionName = "\"" + option.name + "\": " + JSON.stringify(compilerOptionsMap.get(option.name)) + ((seenKnownKeys += 1) === compilerOptionsMap.size ? "" : ",");
                        }
                        else {
                            optionName = "// \"" + option.name + "\": " + JSON.stringify(getDefaultValueForOption(option)) + ",";
                        }
                        nameColumn.push(optionName);
                        descriptionColumn.push("/* " + (option.description && ts.getLocaleSpecificMessage(option.description) || option.name) + " */");
                        marginLength = Math.max(optionName.length, marginLength);
                    }
                });
                // Write the output
                var tab = makePadding(2);
                var result = [];
                result.push("{");
                result.push(tab + "\"compilerOptions\": {");
                // Print out each row, aligning all the descriptions on the same column.
                for (var i = 0; i < nameColumn.length; i++) {
                    var optionName = nameColumn[i];
                    var description = descriptionColumn[i];
                    result.push(optionName && "" + tab + tab + optionName + (description && (makePadding(marginLength - optionName.length + 2) + description)));
                }
                if (fileNames.length) {
                    result.push(tab + "},");
                    result.push(tab + "\"files\": [");
                    for (var i = 0; i < fileNames.length; i++) {
                        result.push("" + tab + tab + JSON.stringify(fileNames[i]) + (i === fileNames.length - 1 ? "" : ","));
                    }
                    result.push(tab + "]");
                }
                else {
                    result.push(tab + "}");
                }
                result.push("}");
                return result.join(newLine) + newLine;
            }
        }
        ts.generateTSConfig = generateTSConfig;
        /* @internal */
        function convertToOptionsWithAbsolutePaths(options, toAbsolutePath) {
            var result = {};
            var optionsNameMap = getOptionsNameMap().optionsNameMap;
            for (var name in options) {
                if (ts.hasProperty(options, name)) {
                    result[name] = convertToOptionValueWithAbsolutePaths(optionsNameMap.get(name.toLowerCase()), options[name], toAbsolutePath);
                }
            }
            if (result.configFilePath) {
                result.configFilePath = toAbsolutePath(result.configFilePath);
            }
            return result;
        }
        ts.convertToOptionsWithAbsolutePaths = convertToOptionsWithAbsolutePaths;
        function convertToOptionValueWithAbsolutePaths(option, value, toAbsolutePath) {
            if (option && !isNullOrUndefined(value)) {
                if (option.type === "list") {
                    var values = value;
                    if (option.element.isFilePath && values.length) {
                        return values.map(toAbsolutePath);
                    }
                }
                else if (option.isFilePath) {
                    return toAbsolutePath(value);
                }
            }
            return value;
        }
        /**
         * Parse the contents of a config file (tsconfig.json).
         * @param json The contents of the config file to parse
         * @param host Instance of ParseConfigHost used to enumerate files in folder.
         * @param basePath A root directory to resolve relative path entries in the config
         *    file to. e.g. outDir
         */
        function parseJsonConfigFileContent(json, host, basePath, existingOptions, configFileName, resolutionStack, extraFileExtensions, extendedConfigCache, existingWatchOptions) {
            return parseJsonConfigFileContentWorker(json, /*sourceFile*/ undefined, host, basePath, existingOptions, existingWatchOptions, configFileName, resolutionStack, extraFileExtensions, extendedConfigCache);
        }
        ts.parseJsonConfigFileContent = parseJsonConfigFileContent;
        /**
         * Parse the contents of a config file (tsconfig.json).
         * @param jsonNode The contents of the config file to parse
         * @param host Instance of ParseConfigHost used to enumerate files in folder.
         * @param basePath A root directory to resolve relative path entries in the config
         *    file to. e.g. outDir
         */
        function parseJsonSourceFileConfigFileContent(sourceFile, host, basePath, existingOptions, configFileName, resolutionStack, extraFileExtensions, extendedConfigCache, existingWatchOptions) {
            return parseJsonConfigFileContentWorker(/*json*/ undefined, sourceFile, host, basePath, existingOptions, existingWatchOptions, configFileName, resolutionStack, extraFileExtensions, extendedConfigCache);
        }
        ts.parseJsonSourceFileConfigFileContent = parseJsonSourceFileConfigFileContent;
        /*@internal*/
        function setConfigFileInOptions(options, configFile) {
            if (configFile) {
                Object.defineProperty(options, "configFile", { enumerable: false, writable: false, value: configFile });
            }
        }
        ts.setConfigFileInOptions = setConfigFileInOptions;
        function isNullOrUndefined(x) {
            return x === undefined || x === null; // eslint-disable-line no-null/no-null
        }
        function directoryOfCombinedPath(fileName, basePath) {
            // Use the `getNormalizedAbsolutePath` function to avoid canonicalizing the path, as it must remain noncanonical
            // until consistent casing errors are reported
            return ts.getDirectoryPath(ts.getNormalizedAbsolutePath(fileName, basePath));
        }
        /**
         * Parse the contents of a config file from json or json source file (tsconfig.json).
         * @param json The contents of the config file to parse
         * @param sourceFile sourceFile corresponding to the Json
         * @param host Instance of ParseConfigHost used to enumerate files in folder.
         * @param basePath A root directory to resolve relative path entries in the config
         *    file to. e.g. outDir
         * @param resolutionStack Only present for backwards-compatibility. Should be empty.
         */
        function parseJsonConfigFileContentWorker(json, sourceFile, host, basePath, existingOptions, existingWatchOptions, configFileName, resolutionStack, extraFileExtensions, extendedConfigCache) {
            if (existingOptions === void 0) {
                existingOptions = {};
            }
            if (resolutionStack === void 0) {
                resolutionStack = [];
            }
            if (extraFileExtensions === void 0) {
                extraFileExtensions = [];
            }
            ts.Debug.assert((json === undefined && sourceFile !== undefined) || (json !== undefined && sourceFile === undefined));
            var errors = [];
            var parsedConfig = parseConfig(json, sourceFile, host, basePath, configFileName, resolutionStack, errors, extendedConfigCache);
            var raw = parsedConfig.raw;
            var options = ts.extend(existingOptions, parsedConfig.options || {});
            var watchOptions = existingWatchOptions && parsedConfig.watchOptions ?
                ts.extend(existingWatchOptions, parsedConfig.watchOptions) :
                parsedConfig.watchOptions || existingWatchOptions;
            options.configFilePath = configFileName && ts.normalizeSlashes(configFileName);
            setConfigFileInOptions(options, sourceFile);
            var projectReferences;
            var _a = getFileNames(), fileNames = _a.fileNames, wildcardDirectories = _a.wildcardDirectories, spec = _a.spec;
            return {
                options: options,
                watchOptions: watchOptions,
                fileNames: fileNames,
                projectReferences: projectReferences,
                typeAcquisition: parsedConfig.typeAcquisition || getDefaultTypeAcquisition(),
                raw: raw,
                errors: errors,
                wildcardDirectories: wildcardDirectories,
                compileOnSave: !!raw.compileOnSave,
                configFileSpecs: spec
            };
            function getFileNames() {
                var filesSpecs;
                if (ts.hasProperty(raw, "files") && !isNullOrUndefined(raw.files)) {
                    if (ts.isArray(raw.files)) {
                        filesSpecs = raw.files;
                        var hasReferences = ts.hasProperty(raw, "references") && !isNullOrUndefined(raw.references);
                        var hasZeroOrNoReferences = !hasReferences || raw.references.length === 0;
                        var hasExtends = ts.hasProperty(raw, "extends");
                        if (filesSpecs.length === 0 && hasZeroOrNoReferences && !hasExtends) {
                            if (sourceFile) {
                                var fileName = configFileName || "tsconfig.json";
                                var diagnosticMessage = ts.Diagnostics.The_files_list_in_config_file_0_is_empty;
                                var nodeValue = ts.firstDefined(ts.getTsConfigPropArray(sourceFile, "files"), function (property) { return property.initializer; });
                                var error = nodeValue
                                    ? ts.createDiagnosticForNodeInSourceFile(sourceFile, nodeValue, diagnosticMessage, fileName)
                                    : ts.createCompilerDiagnostic(diagnosticMessage, fileName);
                                errors.push(error);
                            }
                            else {
                                createCompilerDiagnosticOnlyIfJson(ts.Diagnostics.The_files_list_in_config_file_0_is_empty, configFileName || "tsconfig.json");
                            }
                        }
                    }
                    else {
                        createCompilerDiagnosticOnlyIfJson(ts.Diagnostics.Compiler_option_0_requires_a_value_of_type_1, "files", "Array");
                    }
                }
                var includeSpecs;
                if (ts.hasProperty(raw, "include") && !isNullOrUndefined(raw.include)) {
                    if (ts.isArray(raw.include)) {
                        includeSpecs = raw.include;
                    }
                    else {
                        createCompilerDiagnosticOnlyIfJson(ts.Diagnostics.Compiler_option_0_requires_a_value_of_type_1, "include", "Array");
                    }
                }
                var excludeSpecs;
                if (ts.hasProperty(raw, "exclude") && !isNullOrUndefined(raw.exclude)) {
                    if (ts.isArray(raw.exclude)) {
                        excludeSpecs = raw.exclude;
                    }
                    else {
                        createCompilerDiagnosticOnlyIfJson(ts.Diagnostics.Compiler_option_0_requires_a_value_of_type_1, "exclude", "Array");
                    }
                }
                else if (raw.compilerOptions) {
                    var outDir = raw.compilerOptions.outDir;
                    var declarationDir = raw.compilerOptions.declarationDir;
                    if (outDir || declarationDir) {
                        excludeSpecs = [outDir, declarationDir].filter(function (d) { return !!d; });
                    }
                }
                if (filesSpecs === undefined && includeSpecs === undefined) {
                    includeSpecs = ["**/*"];
                }
                var result = matchFileNames(filesSpecs, includeSpecs, excludeSpecs, configFileName ? directoryOfCombinedPath(configFileName, basePath) : basePath, options, host, errors, extraFileExtensions, sourceFile);
                if (shouldReportNoInputFiles(result, canJsonReportNoInutFiles(raw), resolutionStack)) {
                    errors.push(getErrorForNoInputFiles(result.spec, configFileName));
                }
                if (ts.hasProperty(raw, "references") && !isNullOrUndefined(raw.references)) {
                    if (ts.isArray(raw.references)) {
                        for (var _i = 0, _a = raw.references; _i < _a.length; _i++) {
                            var ref = _a[_i];
                            if (typeof ref.path !== "string") {
                                createCompilerDiagnosticOnlyIfJson(ts.Diagnostics.Compiler_option_0_requires_a_value_of_type_1, "reference.path", "string");
                            }
                            else {
                                (projectReferences || (projectReferences = [])).push({
                                    path: ts.getNormalizedAbsolutePath(ref.path, basePath),
                                    originalPath: ref.path,
                                    prepend: ref.prepend,
                                    circular: ref.circular
                                });
                            }
                        }
                    }
                    else {
                        createCompilerDiagnosticOnlyIfJson(ts.Diagnostics.Compiler_option_0_requires_a_value_of_type_1, "references", "Array");
                    }
                }
                return result;
            }
            function createCompilerDiagnosticOnlyIfJson(message, arg0, arg1) {
                if (!sourceFile) {
                    errors.push(ts.createCompilerDiagnostic(message, arg0, arg1));
                }
            }
        }
        function isErrorNoInputFiles(error) {
            return error.code === ts.Diagnostics.No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2.code;
        }
        function getErrorForNoInputFiles(_a, configFileName) {
            var includeSpecs = _a.includeSpecs, excludeSpecs = _a.excludeSpecs;
            return ts.createCompilerDiagnostic(ts.Diagnostics.No_inputs_were_found_in_config_file_0_Specified_include_paths_were_1_and_exclude_paths_were_2, configFileName || "tsconfig.json", JSON.stringify(includeSpecs || []), JSON.stringify(excludeSpecs || []));
        }
        function shouldReportNoInputFiles(result, canJsonReportNoInutFiles, resolutionStack) {
            return result.fileNames.length === 0 && canJsonReportNoInutFiles && (!resolutionStack || resolutionStack.length === 0);
        }
        /*@internal*/
        function canJsonReportNoInutFiles(raw) {
            return !ts.hasProperty(raw, "files") && !ts.hasProperty(raw, "references");
        }
        ts.canJsonReportNoInutFiles = canJsonReportNoInutFiles;
        /*@internal*/
        function updateErrorForNoInputFiles(result, configFileName, configFileSpecs, configParseDiagnostics, canJsonReportNoInutFiles) {
            var existingErrors = configParseDiagnostics.length;
            if (shouldReportNoInputFiles(result, canJsonReportNoInutFiles)) {
                configParseDiagnostics.push(getErrorForNoInputFiles(configFileSpecs, configFileName));
            }
            else {
                ts.filterMutate(configParseDiagnostics, function (error) { return !isErrorNoInputFiles(error); });
            }
            return existingErrors !== configParseDiagnostics.length;
        }
        ts.updateErrorForNoInputFiles = updateErrorForNoInputFiles;
        function isSuccessfulParsedTsconfig(value) {
            return !!value.options;
        }
        /**
         * This *just* extracts options/include/exclude/files out of a config file.
         * It does *not* resolve the included files.
         */
        function parseConfig(json, sourceFile, host, basePath, configFileName, resolutionStack, errors, extendedConfigCache) {
            basePath = ts.normalizeSlashes(basePath);
            var resolvedPath = ts.getNormalizedAbsolutePath(configFileName || "", basePath);
            if (resolutionStack.indexOf(resolvedPath) >= 0) {
                errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Circularity_detected_while_resolving_configuration_Colon_0, __spreadArrays(resolutionStack, [resolvedPath]).join(" -> ")));
                return { raw: json || convertToObject(sourceFile, errors) };
            }
            var ownConfig = json ?
                parseOwnConfigOfJson(json, host, basePath, configFileName, errors) :
                parseOwnConfigOfJsonSourceFile(sourceFile, host, basePath, configFileName, errors);
            if (ownConfig.extendedConfigPath) {
                // copy the resolution stack so it is never reused between branches in potential diamond-problem scenarios.
                resolutionStack = resolutionStack.concat([resolvedPath]);
                var extendedConfig = getExtendedConfig(sourceFile, ownConfig.extendedConfigPath, host, basePath, resolutionStack, errors, extendedConfigCache);
                if (extendedConfig && isSuccessfulParsedTsconfig(extendedConfig)) {
                    var baseRaw_1 = extendedConfig.raw;
                    var raw_1 = ownConfig.raw;
                    var setPropertyInRawIfNotUndefined = function (propertyName) {
                        var value = raw_1[propertyName] || baseRaw_1[propertyName];
                        if (value) {
                            raw_1[propertyName] = value;
                        }
                    };
                    setPropertyInRawIfNotUndefined("include");
                    setPropertyInRawIfNotUndefined("exclude");
                    setPropertyInRawIfNotUndefined("files");
                    if (raw_1.compileOnSave === undefined) {
                        raw_1.compileOnSave = baseRaw_1.compileOnSave;
                    }
                    ownConfig.options = ts.assign({}, extendedConfig.options, ownConfig.options);
                    ownConfig.watchOptions = ownConfig.watchOptions && extendedConfig.watchOptions ?
                        ts.assign({}, extendedConfig.watchOptions, ownConfig.watchOptions) :
                        ownConfig.watchOptions || extendedConfig.watchOptions;
                    // TODO extend type typeAcquisition
                }
            }
            return ownConfig;
        }
        function parseOwnConfigOfJson(json, host, basePath, configFileName, errors) {
            if (ts.hasProperty(json, "excludes")) {
                errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Unknown_option_excludes_Did_you_mean_exclude));
            }
            var options = convertCompilerOptionsFromJsonWorker(json.compilerOptions, basePath, errors, configFileName);
            // typingOptions has been deprecated and is only supported for backward compatibility purposes.
            // It should be removed in future releases - use typeAcquisition instead.
            var typeAcquisition = convertTypeAcquisitionFromJsonWorker(json.typeAcquisition || json.typingOptions, basePath, errors, configFileName);
            var watchOptions = convertWatchOptionsFromJsonWorker(json.watchOptions, basePath, errors);
            json.compileOnSave = convertCompileOnSaveOptionFromJson(json, basePath, errors);
            var extendedConfigPath;
            if (json.extends) {
                if (!ts.isString(json.extends)) {
                    errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Compiler_option_0_requires_a_value_of_type_1, "extends", "string"));
                }
                else {
                    var newBase = configFileName ? directoryOfCombinedPath(configFileName, basePath) : basePath;
                    extendedConfigPath = getExtendsConfigPath(json.extends, host, newBase, errors, ts.createCompilerDiagnostic);
                }
            }
            return { raw: json, options: options, watchOptions: watchOptions, typeAcquisition: typeAcquisition, extendedConfigPath: extendedConfigPath };
        }
        function parseOwnConfigOfJsonSourceFile(sourceFile, host, basePath, configFileName, errors) {
            var options = getDefaultCompilerOptions(configFileName);
            var typeAcquisition, typingOptionstypeAcquisition;
            var watchOptions;
            var extendedConfigPath;
            var optionsIterator = {
                onSetValidOptionKeyValueInParent: function (parentOption, option, value) {
                    var currentOption;
                    switch (parentOption) {
                        case "compilerOptions":
                            currentOption = options;
                            break;
                        case "watchOptions":
                            currentOption = (watchOptions || (watchOptions = {}));
                            break;
                        case "typeAcquisition":
                            currentOption = (typeAcquisition || (typeAcquisition = getDefaultTypeAcquisition(configFileName)));
                            break;
                        case "typingOptions":
                            currentOption = (typingOptionstypeAcquisition || (typingOptionstypeAcquisition = getDefaultTypeAcquisition(configFileName)));
                            break;
                        default:
                            ts.Debug.fail("Unknown option");
                    }
                    currentOption[option.name] = normalizeOptionValue(option, basePath, value);
                },
                onSetValidOptionKeyValueInRoot: function (key, _keyNode, value, valueNode) {
                    switch (key) {
                        case "extends":
                            var newBase = configFileName ? directoryOfCombinedPath(configFileName, basePath) : basePath;
                            extendedConfigPath = getExtendsConfigPath(value, host, newBase, errors, function (message, arg0) {
                                return ts.createDiagnosticForNodeInSourceFile(sourceFile, valueNode, message, arg0);
                            });
                            return;
                    }
                },
                onSetUnknownOptionKeyValueInRoot: function (key, keyNode, _value, _valueNode) {
                    if (key === "excludes") {
                        errors.push(ts.createDiagnosticForNodeInSourceFile(sourceFile, keyNode, ts.Diagnostics.Unknown_option_excludes_Did_you_mean_exclude));
                    }
                }
            };
            var json = convertToObjectWorker(sourceFile, errors, /*returnValue*/ true, getTsconfigRootOptionsMap(), optionsIterator);
            if (!typeAcquisition) {
                if (typingOptionstypeAcquisition) {
                    typeAcquisition = (typingOptionstypeAcquisition.enableAutoDiscovery !== undefined) ?
                        {
                            enable: typingOptionstypeAcquisition.enableAutoDiscovery,
                            include: typingOptionstypeAcquisition.include,
                            exclude: typingOptionstypeAcquisition.exclude
                        } :
                        typingOptionstypeAcquisition;
                }
                else {
                    typeAcquisition = getDefaultTypeAcquisition(configFileName);
                }
            }
            return { raw: json, options: options, watchOptions: watchOptions, typeAcquisition: typeAcquisition, extendedConfigPath: extendedConfigPath };
        }
        function getExtendsConfigPath(extendedConfig, host, basePath, errors, createDiagnostic) {
            extendedConfig = ts.normalizeSlashes(extendedConfig);
            if (ts.isRootedDiskPath(extendedConfig) || ts.startsWith(extendedConfig, "./") || ts.startsWith(extendedConfig, "../")) {
                var extendedConfigPath = ts.getNormalizedAbsolutePath(extendedConfig, basePath);
                if (!host.fileExists(extendedConfigPath) && !ts.endsWith(extendedConfigPath, ".json" /* Json */)) {
                    extendedConfigPath = extendedConfigPath + ".json";
                    if (!host.fileExists(extendedConfigPath)) {
                        errors.push(createDiagnostic(ts.Diagnostics.File_0_not_found, extendedConfig));
                        return undefined;
                    }
                }
                return extendedConfigPath;
            }
            // If the path isn't a rooted or relative path, resolve like a module
            var resolved = ts.nodeModuleNameResolver(extendedConfig, ts.combinePaths(basePath, "tsconfig.json"), { moduleResolution: ts.ModuleResolutionKind.NodeJs }, host, /*cache*/ undefined, /*projectRefs*/ undefined, /*lookupConfig*/ true);
            if (resolved.resolvedModule) {
                return resolved.resolvedModule.resolvedFileName;
            }
            errors.push(createDiagnostic(ts.Diagnostics.File_0_not_found, extendedConfig));
            return undefined;
        }
        function getExtendedConfig(sourceFile, extendedConfigPath, host, basePath, resolutionStack, errors, extendedConfigCache) {
            var _a;
            var path = host.useCaseSensitiveFileNames ? extendedConfigPath : ts.toFileNameLowerCase(extendedConfigPath);
            var value;
            var extendedResult;
            var extendedConfig;
            if (extendedConfigCache && (value = extendedConfigCache.get(path))) {
                (extendedResult = value.extendedResult, extendedConfig = value.extendedConfig);
            }
            else {
                extendedResult = readJsonConfigFile(extendedConfigPath, function (path) { return host.readFile(path); });
                if (!extendedResult.parseDiagnostics.length) {
                    var extendedDirname = ts.getDirectoryPath(extendedConfigPath);
                    extendedConfig = parseConfig(/*json*/ undefined, extendedResult, host, extendedDirname, ts.getBaseFileName(extendedConfigPath), resolutionStack, errors, extendedConfigCache);
                    if (isSuccessfulParsedTsconfig(extendedConfig)) {
                        // Update the paths to reflect base path
                        var relativeDifference_1 = ts.convertToRelativePath(extendedDirname, basePath, ts.identity);
                        var updatePath_1 = function (path) { return ts.isRootedDiskPath(path) ? path : ts.combinePaths(relativeDifference_1, path); };
                        var mapPropertiesInRawIfNotUndefined = function (propertyName) {
                            if (raw_2[propertyName]) {
                                raw_2[propertyName] = ts.map(raw_2[propertyName], updatePath_1);
                            }
                        };
                        var raw_2 = extendedConfig.raw;
                        mapPropertiesInRawIfNotUndefined("include");
                        mapPropertiesInRawIfNotUndefined("exclude");
                        mapPropertiesInRawIfNotUndefined("files");
                    }
                }
                if (extendedConfigCache) {
                    extendedConfigCache.set(path, { extendedResult: extendedResult, extendedConfig: extendedConfig });
                }
            }
            if (sourceFile) {
                sourceFile.extendedSourceFiles = [extendedResult.fileName];
                if (extendedResult.extendedSourceFiles) {
                    (_a = sourceFile.extendedSourceFiles).push.apply(_a, extendedResult.extendedSourceFiles);
                }
            }
            if (extendedResult.parseDiagnostics.length) {
                errors.push.apply(errors, extendedResult.parseDiagnostics);
                return undefined;
            }
            return extendedConfig;
        }
        function convertCompileOnSaveOptionFromJson(jsonOption, basePath, errors) {
            if (!ts.hasProperty(jsonOption, ts.compileOnSaveCommandLineOption.name)) {
                return false;
            }
            var result = convertJsonOption(ts.compileOnSaveCommandLineOption, jsonOption.compileOnSave, basePath, errors);
            return typeof result === "boolean" && result;
        }
        function convertCompilerOptionsFromJson(jsonOptions, basePath, configFileName) {
            var errors = [];
            var options = convertCompilerOptionsFromJsonWorker(jsonOptions, basePath, errors, configFileName);
            return { options: options, errors: errors };
        }
        ts.convertCompilerOptionsFromJson = convertCompilerOptionsFromJson;
        function convertTypeAcquisitionFromJson(jsonOptions, basePath, configFileName) {
            var errors = [];
            var options = convertTypeAcquisitionFromJsonWorker(jsonOptions, basePath, errors, configFileName);
            return { options: options, errors: errors };
        }
        ts.convertTypeAcquisitionFromJson = convertTypeAcquisitionFromJson;
        function getDefaultCompilerOptions(configFileName) {
            var options = configFileName && ts.getBaseFileName(configFileName) === "jsconfig.json"
                ? { allowJs: true, maxNodeModuleJsDepth: 2, allowSyntheticDefaultImports: true, skipLibCheck: true, noEmit: true }
                : {};
            return options;
        }
        function convertCompilerOptionsFromJsonWorker(jsonOptions, basePath, errors, configFileName) {
            var options = getDefaultCompilerOptions(configFileName);
            convertOptionsFromJson(getCommandLineCompilerOptionsMap(), jsonOptions, basePath, options, ts.compilerOptionsDidYouMeanDiagnostics, errors);
            if (configFileName) {
                options.configFilePath = ts.normalizeSlashes(configFileName);
            }
            return options;
        }
        function getDefaultTypeAcquisition(configFileName) {
            return { enable: !!configFileName && ts.getBaseFileName(configFileName) === "jsconfig.json", include: [], exclude: [] };
        }
        function convertTypeAcquisitionFromJsonWorker(jsonOptions, basePath, errors, configFileName) {
            var options = getDefaultTypeAcquisition(configFileName);
            var typeAcquisition = convertEnableAutoDiscoveryToEnable(jsonOptions);
            convertOptionsFromJson(getCommandLineTypeAcquisitionMap(), typeAcquisition, basePath, options, typeAcquisitionDidYouMeanDiagnostics, errors);
            return options;
        }
        function convertWatchOptionsFromJsonWorker(jsonOptions, basePath, errors) {
            return convertOptionsFromJson(getCommandLineWatchOptionsMap(), jsonOptions, basePath, /*defaultOptions*/ undefined, watchOptionsDidYouMeanDiagnostics, errors);
        }
        function convertOptionsFromJson(optionsNameMap, jsonOptions, basePath, defaultOptions, diagnostics, errors) {
            if (!jsonOptions) {
                return;
            }
            for (var id in jsonOptions) {
                var opt = optionsNameMap.get(id);
                if (opt) {
                    (defaultOptions || (defaultOptions = {}))[opt.name] = convertJsonOption(opt, jsonOptions[id], basePath, errors);
                }
                else {
                    errors.push(createUnknownOptionError(id, diagnostics, ts.createCompilerDiagnostic));
                }
            }
            return defaultOptions;
        }
        function convertJsonOption(opt, value, basePath, errors) {
            if (isCompilerOptionsValue(opt, value)) {
                var optType = opt.type;
                if (optType === "list" && ts.isArray(value)) {
                    return convertJsonOptionOfListType(opt, value, basePath, errors);
                }
                else if (!ts.isString(optType)) {
                    return convertJsonOptionOfCustomType(opt, value, errors);
                }
                return normalizeNonListOptionValue(opt, basePath, value);
            }
            else {
                errors.push(ts.createCompilerDiagnostic(ts.Diagnostics.Compiler_option_0_requires_a_value_of_type_1, opt.name, getCompilerOptionValueTypeString(opt)));
            }
        }
        function normalizeOptionValue(option, basePath, value) {
            if (isNullOrUndefined(value))
                return undefined;
            if (option.type === "list") {
                var listOption_1 = option;
                if (listOption_1.element.isFilePath || !ts.isString(listOption_1.element.type)) {
                    return ts.filter(ts.map(value, function (v) { return normalizeOptionValue(listOption_1.element, basePath, v); }), function (v) { return !!v; });
                }
                return value;
            }
            else if (!ts.isString(option.type)) {
                return option.type.get(ts.isString(value) ? value.toLowerCase() : value);
            }
            return normalizeNonListOptionValue(option, basePath, value);
        }
        function normalizeNonListOptionValue(option, basePath, value) {
            if (option.isFilePath) {
                value = ts.getNormalizedAbsolutePath(value, basePath);
                if (value === "") {
                    value = ".";
                }
            }
            return value;
        }
        function convertJsonOptionOfCustomType(opt, value, errors) {
            if (isNullOrUndefined(value))
                return undefined;
            var key = value.toLowerCase();
            var val = opt.type.get(key);
            if (val !== undefined) {
                return val;
            }
            else {
                errors.push(createCompilerDiagnosticForInvalidCustomType(opt));
            }
        }
        function convertJsonOptionOfListType(option, values, basePath, errors) {
            return ts.filter(ts.map(values, function (v) { return convertJsonOption(option.element, v, basePath, errors); }), function (v) { return !!v; });
        }
        function trimString(s) {
            return typeof s.trim === "function" ? s.trim() : s.replace(/^[\s]+|[\s]+$/g, "");
        }
        /**
         * Tests for a path that ends in a recursive directory wildcard.
         * Matches **, \**, **\, and \**\, but not a**b.
         *
         * NOTE: used \ in place of / above to avoid issues with multiline comments.
         *
         * Breakdown:
         *  (^|\/)      # matches either the beginning of the string or a directory separator.
         *  \*\*        # matches the recursive directory wildcard "**".
         *  \/?$        # matches an optional trailing directory separator at the end of the string.
         */
        var invalidTrailingRecursionPattern = /(^|\/)\*\*\/?$/;
        /**
         * Tests for a path where .. appears after a recursive directory wildcard.
         * Matches **\..\*, **\a\..\*, and **\.., but not ..\**\*
         *
         * NOTE: used \ in place of / above to avoid issues with multiline comments.
         *
         * Breakdown:
         *  (^|\/)      # matches either the beginning of the string or a directory separator.
         *  \*\*\/      # matches a recursive directory wildcard "**" followed by a directory separator.
         *  (.*\/)?     # optionally matches any number of characters followed by a directory separator.
         *  \.\.        # matches a parent directory path component ".."
         *  ($|\/)      # matches either the end of the string or a directory separator.
         */
        var invalidDotDotAfterRecursiveWildcardPattern = /(^|\/)\*\*\/(.*\/)?\.\.($|\/)/;
        /**
         * Tests for a path containing a wildcard character in a directory component of the path.
         * Matches \*\, \?\, and \a*b\, but not \a\ or \a\*.
         *
         * NOTE: used \ in place of / above to avoid issues with multiline comments.
         *
         * Breakdown:
         *  \/          # matches a directory separator.
         *  [^/]*?      # matches any number of characters excluding directory separators (non-greedy).
         *  [*?]        # matches either a wildcard character (* or ?)
         *  [^/]*       # matches any number of characters excluding directory separators (greedy).
         *  \/          # matches a directory separator.
         */
        var watchRecursivePattern = /\/[^/]*?[*?][^/]*\//;
        /**
         * Matches the portion of a wildcard path that does not contain wildcards.
         * Matches \a of \a\*, or \a\b\c of \a\b\c\?\d.
         *
         * NOTE: used \ in place of / above to avoid issues with multiline comments.
         *
         * Breakdown:
         *  ^                   # matches the beginning of the string
         *  [^*?]*              # matches any number of non-wildcard characters
         *  (?=\/[^/]*[*?])     # lookahead that matches a directory separator followed by
         *                      # a path component that contains at least one wildcard character (* or ?).
         */
        var wildcardDirectoryPattern = /^[^*?]*(?=\/[^/]*[*?])/;
        /**
         * Expands an array of file specifications.
         *
         * @param filesSpecs The literal file names to include.
         * @param includeSpecs The wildcard file specifications to include.
         * @param excludeSpecs The wildcard file specifications to exclude.
         * @param basePath The base path for any relative file specifications.
         * @param options Compiler options.
         * @param host The host used to resolve files and directories.
         * @param errors An array for diagnostic reporting.
         */
        function matchFileNames(filesSpecs, includeSpecs, excludeSpecs, basePath, options, host, errors, extraFileExtensions, jsonSourceFile) {
            basePath = ts.normalizePath(basePath);
            var validatedIncludeSpecs, validatedExcludeSpecs;
            // The exclude spec list is converted into a regular expression, which allows us to quickly
            // test whether a file or directory should be excluded before recursively traversing the
            // file system.
            if (includeSpecs) {
                validatedIncludeSpecs = validateSpecs(includeSpecs, errors, /*allowTrailingRecursion*/ false, jsonSourceFile, "include");
            }
            if (excludeSpecs) {
                validatedExcludeSpecs = validateSpecs(excludeSpecs, errors, /*allowTrailingRecursion*/ true, jsonSourceFile, "exclude");
            }
            // Wildcard directories (provided as part of a wildcard path) are stored in a
            // file map that marks whether it was a regular wildcard match (with a `*` or `?` token),
            // or a recursive directory. This information is used by filesystem watchers to monitor for
            // new entries in these paths.
            var wildcardDirectories = getWildcardDirectories(validatedIncludeSpecs, validatedExcludeSpecs, basePath, host.useCaseSensitiveFileNames);
            var spec = { filesSpecs: filesSpecs, includeSpecs: includeSpecs, excludeSpecs: excludeSpecs, validatedIncludeSpecs: validatedIncludeSpecs, validatedExcludeSpecs: validatedExcludeSpecs, wildcardDirectories: wildcardDirectories };
            return getFileNamesFromConfigSpecs(spec, basePath, options, host, extraFileExtensions);
        }
        /**
         * Gets the file names from the provided config file specs that contain, files, include, exclude and
         * other properties needed to resolve the file names
         * @param spec The config file specs extracted with file names to include, wildcards to include/exclude and other details
         * @param basePath The base path for any relative file specifications.
         * @param options Compiler options.
         * @param host The host used to resolve files and directories.
         * @param extraFileExtensions optionaly file extra file extension information from host
         */
        /* @internal */
        function getFileNamesFromConfigSpecs(spec, basePath, options, host, extraFileExtensions) {
            if (extraFileExtensions === void 0) {
                extraFileExtensions = [];
            }
            basePath = ts.normalizePath(basePath);
            var keyMapper = ts.createGetCanonicalFileName(host.useCaseSensitiveFileNames);
            // Literal file names (provided via the "files" array in tsconfig.json) are stored in a
            // file map with a possibly case insensitive key. We use this map later when when including
            // wildcard paths.
            var literalFileMap = ts.createMap();
            // Wildcard paths (provided via the "includes" array in tsconfig.json) are stored in a
            // file map with a possibly case insensitive key. We use this map to store paths matched
            // via wildcard, and to handle extension priority.
            var wildcardFileMap = ts.createMap();
            // Wildcard paths of json files (provided via the "includes" array in tsconfig.json) are stored in a
            // file map with a possibly case insensitive key. We use this map to store paths matched
            // via wildcard of *.json kind
            var wildCardJsonFileMap = ts.createMap();
            var filesSpecs = spec.filesSpecs, validatedIncludeSpecs = spec.validatedIncludeSpecs, validatedExcludeSpecs = spec.validatedExcludeSpecs, wildcardDirectories = spec.wildcardDirectories;
            // Rather than requery this for each file and filespec, we query the supported extensions
            // once and store it on the expansion context.
            var supportedExtensions = ts.getSupportedExtensions(options, extraFileExtensions);
            var supportedExtensionsWithJsonIfResolveJsonModule = ts.getSuppoertedExtensionsWithJsonIfResolveJsonModule(options, supportedExtensions);
            // Literal files are always included verbatim. An "include" or "exclude" specification cannot
            // remove a literal file.
            if (filesSpecs) {
                for (var _i = 0, filesSpecs_1 = filesSpecs; _i < filesSpecs_1.length; _i++) {
                    var fileName = filesSpecs_1[_i];
                    var file = ts.getNormalizedAbsolutePath(fileName, basePath);
                    literalFileMap.set(keyMapper(file), file);
                }
            }
            var jsonOnlyIncludeRegexes;
            if (validatedIncludeSpecs && validatedIncludeSpecs.length > 0) {
                var _loop_5 = function (file) {
                    if (ts.fileExtensionIs(file, ".json" /* Json */)) {
                        // Valid only if *.json specified
                        if (!jsonOnlyIncludeRegexes) {
                            var includes = validatedIncludeSpecs.filter(function (s) { return ts.endsWith(s, ".json" /* Json */); });
                            var includeFilePatterns = ts.map(ts.getRegularExpressionsForWildcards(includes, basePath, "files"), function (pattern) { return "^" + pattern + "$"; });
                            jsonOnlyIncludeRegexes = includeFilePatterns ? includeFilePatterns.map(function (pattern) { return ts.getRegexFromPattern(pattern, host.useCaseSensitiveFileNames); }) : ts.emptyArray;
                        }
                        var includeIndex = ts.findIndex(jsonOnlyIncludeRegexes, function (re) { return re.test(file); });
                        if (includeIndex !== -1) {
                            var key_1 = keyMapper(file);
                            if (!literalFileMap.has(key_1) && !wildCardJsonFileMap.has(key_1)) {
                                wildCardJsonFileMap.set(key_1, file);
                            }
                        }
                        return "continue";
                    }
                    // If we have already included a literal or wildcard path with a
                    // higher priority extension, we should skip this file.
                    //
                    // This handles cases where we may encounter both <file>.ts and
                    // <file>.d.ts (or <file>.js if "allowJs" is enabled) in the same
                    // directory when they are compilation outputs.
                    if (hasFileWithHigherPriorityExtension(file, literalFileMap, wildcardFileMap, supportedExtensions, keyMapper)) {
                        return "continue";
                    }
                    // We may have included a wildcard path with a lower priority
                    // extension due to the user-defined order of entries in the
                    // "include" array. If there is a lower priority extension in the
                    // same directory, we should remove it.
                    removeWildcardFilesWithLowerPriorityExtension(file, wildcardFileMap, supportedExtensions, keyMapper);
                    var key = keyMapper(file);
                    if (!literalFileMap.has(key) && !wildcardFileMap.has(key)) {
                        wildcardFileMap.set(key, file);
                    }
                };
                for (var _a = 0, _b = host.readDirectory(basePath, supportedExtensionsWithJsonIfResolveJsonModule, validatedExcludeSpecs, validatedIncludeSpecs, /*depth*/ undefined); _a < _b.length; _a++) {
                    var file = _b[_a];
                    _loop_5(file);
                }
            }
            var literalFiles = ts.arrayFrom(literalFileMap.values());
            var wildcardFiles = ts.arrayFrom(wildcardFileMap.values());
            return {
                fileNames: literalFiles.concat(wildcardFiles, ts.arrayFrom(wildCardJsonFileMap.values())),
                wildcardDirectories: wildcardDirectories,
                spec: spec
            };
        }
        ts.getFileNamesFromConfigSpecs = getFileNamesFromConfigSpecs;
        function validateSpecs(specs, errors, allowTrailingRecursion, jsonSourceFile, specKey) {
            return specs.filter(function (spec) {
                var diag = specToDiagnostic(spec, allowTrailingRecursion);
                if (diag !== undefined) {
                    errors.push(createDiagnostic(diag, spec));
                }
                return diag === undefined;
            });
            function createDiagnostic(message, spec) {
                var element = ts.getTsConfigPropArrayElementValue(jsonSourceFile, specKey, spec);
                return element ?
                    ts.createDiagnosticForNodeInSourceFile(jsonSourceFile, element, message, spec) :
                    ts.createCompilerDiagnostic(message, spec);
            }
        }
        function specToDiagnostic(spec, allowTrailingRecursion) {
            if (!allowTrailingRecursion && invalidTrailingRecursionPattern.test(spec)) {
                return ts.Diagnostics.File_specification_cannot_end_in_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0;
            }
            else if (invalidDotDotAfterRecursiveWildcardPattern.test(spec)) {
                return ts.Diagnostics.File_specification_cannot_contain_a_parent_directory_that_appears_after_a_recursive_directory_wildcard_Asterisk_Asterisk_Colon_0;
            }
        }
        /**
         * Gets directories in a set of include patterns that should be watched for changes.
         */
        function getWildcardDirectories(include, exclude, path, useCaseSensitiveFileNames) {
            // We watch a directory recursively if it contains a wildcard anywhere in a directory segment
            // of the pattern:
            //
            //  /a/b/**/d   - Watch /a/b recursively to catch changes to any d in any subfolder recursively
            //  /a/b/*/d    - Watch /a/b recursively to catch any d in any immediate subfolder, even if a new subfolder is added
            //  /a/b        - Watch /a/b recursively to catch changes to anything in any recursive subfoler
            //
            // We watch a directory without recursion if it contains a wildcard in the file segment of
            // the pattern:
            //
            //  /a/b/*      - Watch /a/b directly to catch any new file
            //  /a/b/a?z    - Watch /a/b directly to catch any new file matching a?z
            var rawExcludeRegex = ts.getRegularExpressionForWildcard(exclude, path, "exclude");
            var excludeRegex = rawExcludeRegex && new RegExp(rawExcludeRegex, useCaseSensitiveFileNames ? "" : "i");
            var wildcardDirectories = {};
            if (include !== undefined) {
                var recursiveKeys = [];
                for (var _i = 0, include_1 = include; _i < include_1.length; _i++) {
                    var file = include_1[_i];
                    var spec = ts.normalizePath(ts.combinePaths(path, file));
                    if (excludeRegex && excludeRegex.test(spec)) {
                        continue;
                    }
                    var match = getWildcardDirectoryFromSpec(spec, useCaseSensitiveFileNames);
                    if (match) {
                        var key = match.key, flags = match.flags;
                        var existingFlags = wildcardDirectories[key];
                        if (existingFlags === undefined || existingFlags < flags) {
                            wildcardDirectories[key] = flags;
                            if (flags === 1 /* Recursive */) {
                                recursiveKeys.push(key);
                            }
                        }
                    }
                }
                // Remove any subpaths under an existing recursively watched directory.
                for (var key in wildcardDirectories) {
                    if (ts.hasProperty(wildcardDirectories, key)) {
                        for (var _a = 0, recursiveKeys_1 = recursiveKeys; _a < recursiveKeys_1.length; _a++) {
                            var recursiveKey = recursiveKeys_1[_a];
                            if (key !== recursiveKey && ts.containsPath(recursiveKey, key, path, !useCaseSensitiveFileNames)) {
                                delete wildcardDirectories[key];
                            }
                        }
                    }
                }
            }
            return wildcardDirectories;
        }
        function getWildcardDirectoryFromSpec(spec, useCaseSensitiveFileNames) {
            var match = wildcardDirectoryPattern.exec(spec);
            if (match) {
                return {
                    key: useCaseSensitiveFileNames ? match[0] : ts.toFileNameLowerCase(match[0]),
                    flags: watchRecursivePattern.test(spec) ? 1 /* Recursive */ : 0 /* None */
                };
            }
            if (ts.isImplicitGlob(spec)) {
                return { key: spec, flags: 1 /* Recursive */ };
            }
            return undefined;
        }
        /**
         * Determines whether a literal or wildcard file has already been included that has a higher
         * extension priority.
         *
         * @param file The path to the file.
         * @param extensionPriority The priority of the extension.
         * @param context The expansion context.
         */
        function hasFileWithHigherPriorityExtension(file, literalFiles, wildcardFiles, extensions, keyMapper) {
            var extensionPriority = ts.getExtensionPriority(file, extensions);
            var adjustedExtensionPriority = ts.adjustExtensionPriority(extensionPriority, extensions);
            for (var i = 0 /* Highest */; i < adjustedExtensionPriority; i++) {
                var higherPriorityExtension = extensions[i];
                var higherPriorityPath = keyMapper(ts.changeExtension(file, higherPriorityExtension));
                if (literalFiles.has(higherPriorityPath) || wildcardFiles.has(higherPriorityPath)) {
                    return true;
                }
            }
            return false;
        }
        /**
         * Removes files included via wildcard expansion with a lower extension priority that have
         * already been included.
         *
         * @param file The path to the file.
         * @param extensionPriority The priority of the extension.
         * @param context The expansion context.
         */
        function removeWildcardFilesWithLowerPriorityExtension(file, wildcardFiles, extensions, keyMapper) {
            var extensionPriority = ts.getExtensionPriority(file, extensions);
            var nextExtensionPriority = ts.getNextLowestExtensionPriority(extensionPriority, extensions);
            for (var i = nextExtensionPriority; i < extensions.length; i++) {
                var lowerPriorityExtension = extensions[i];
                var lowerPriorityPath = keyMapper(ts.changeExtension(file, lowerPriorityExtension));
                wildcardFiles.delete(lowerPriorityPath);
            }
        }
        /**
         * Produces a cleaned version of compiler options with personally identifying info (aka, paths) removed.
         * Also converts enum values back to strings.
         */
        /* @internal */
        function convertCompilerOptionsForTelemetry(opts) {
            var out = {};
            for (var key in opts) {
                if (opts.hasOwnProperty(key)) {
                    var type = getOptionFromName(key);
                    if (type !== undefined) { // Ignore unknown options
                        out[key] = getOptionValueWithEmptyStrings(opts[key], type);
                    }
                }
            }
            return out;
        }
        ts.convertCompilerOptionsForTelemetry = convertCompilerOptionsForTelemetry;
        function getOptionValueWithEmptyStrings(value, option) {
            switch (option.type) {
                case "object": // "paths". Can't get any useful information from the value since we blank out strings, so just return "".
                    return "";
                case "string": // Could be any arbitrary string -- use empty string instead.
                    return "";
                case "number": // Allow numbers, but be sure to check it's actually a number.
                    return typeof value === "number" ? value : "";
                case "boolean":
                    return typeof value === "boolean" ? value : "";
                case "list":
                    var elementType_1 = option.element;
                    return ts.isArray(value) ? value.map(function (v) { return getOptionValueWithEmptyStrings(v, elementType_1); }) : "";
                default:
                    return ts.forEachEntry(option.type, function (optionEnumValue, optionStringValue) {
                        if (optionEnumValue === value) {
                            return optionStringValue;
                        }
                    }); // TODO: GH#18217
            }
        }
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        function trace(host) {
            host.trace(ts.formatMessage.apply(undefined, arguments));
        }
        ts.trace = trace;
        /* @internal */
        function isTraceEnabled(compilerOptions, host) {
            return !!compilerOptions.traceResolution && host.trace !== undefined;
        }
        ts.isTraceEnabled = isTraceEnabled;
        function withPackageId(packageInfo, r) {
            var packageId;
            if (r && packageInfo) {
                var packageJsonContent = packageInfo.packageJsonContent;
                if (typeof packageJsonContent.name === "string" && typeof packageJsonContent.version === "string") {
                    packageId = {
                        name: packageJsonContent.name,
                        subModuleName: r.path.slice(packageInfo.packageDirectory.length + ts.directorySeparator.length),
                        version: packageJsonContent.version
                    };
                }
            }
            return r && { path: r.path, extension: r.ext, packageId: packageId };
        }
        function noPackageId(r) {
            return withPackageId(/*packageInfo*/ undefined, r);
        }
        function removeIgnoredPackageId(r) {
            if (r) {
                ts.Debug.assert(r.packageId === undefined);
                return { path: r.path, ext: r.extension };
            }
        }
        /**
         * Kinds of file that we are currently looking for.
         * Typically there is one pass with Extensions.TypeScript, then a second pass with Extensions.JavaScript.
         */
        var Extensions;
        (function (Extensions) {
            Extensions[Extensions["TypeScript"] = 0] = "TypeScript";
            Extensions[Extensions["JavaScript"] = 1] = "JavaScript";
            Extensions[Extensions["Json"] = 2] = "Json";
            Extensions[Extensions["TSConfig"] = 3] = "TSConfig";
            Extensions[Extensions["DtsOnly"] = 4] = "DtsOnly"; /** Only '.d.ts' */
        })(Extensions || (Extensions = {}));
        /** Used with `Extensions.DtsOnly` to extract the path from TypeScript results. */
        function resolvedTypeScriptOnly(resolved) {
            if (!resolved) {
                return undefined;
            }
            ts.Debug.assert(ts.extensionIsTS(resolved.extension));
            return { fileName: resolved.path, packageId: resolved.packageId };
        }
        function createResolvedModuleWithFailedLookupLocations(resolved, isExternalLibraryImport, failedLookupLocations) {
            return {
                resolvedModule: resolved && { resolvedFileName: resolved.path, originalPath: resolved.originalPath === true ? undefined : resolved.originalPath, extension: resolved.extension, isExternalLibraryImport: isExternalLibraryImport, packageId: resolved.packageId },
                failedLookupLocations: failedLookupLocations
            };
        }
        function readPackageJsonField(jsonContent, fieldName, typeOfTag, state) {
            if (!ts.hasProperty(jsonContent, fieldName)) {
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.package_json_does_not_have_a_0_field, fieldName);
                }
                return;
            }
            var value = jsonContent[fieldName];
            if (typeof value !== typeOfTag || value === null) { // eslint-disable-line no-null/no-null
                if (state.traceEnabled) {
                    // eslint-disable-next-line no-null/no-null
                    trace(state.host, ts.Diagnostics.Expected_type_of_0_field_in_package_json_to_be_1_got_2, fieldName, typeOfTag, value === null ? "null" : typeof value);
                }
                return;
            }
            return value;
        }
        function readPackageJsonPathField(jsonContent, fieldName, baseDirectory, state) {
            var fileName = readPackageJsonField(jsonContent, fieldName, "string", state);
            if (fileName === undefined) {
                return;
            }
            if (!fileName) {
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.package_json_had_a_falsy_0_field, fieldName);
                }
                return;
            }
            var path = ts.normalizePath(ts.combinePaths(baseDirectory, fileName));
            if (state.traceEnabled) {
                trace(state.host, ts.Diagnostics.package_json_has_0_field_1_that_references_2, fieldName, fileName, path);
            }
            return path;
        }
        function readPackageJsonTypesFields(jsonContent, baseDirectory, state) {
            return readPackageJsonPathField(jsonContent, "typings", baseDirectory, state)
                || readPackageJsonPathField(jsonContent, "types", baseDirectory, state);
        }
        function readPackageJsonTSConfigField(jsonContent, baseDirectory, state) {
            return readPackageJsonPathField(jsonContent, "tsconfig", baseDirectory, state);
        }
        function readPackageJsonMainField(jsonContent, baseDirectory, state) {
            return readPackageJsonPathField(jsonContent, "main", baseDirectory, state);
        }
        function readPackageJsonTypesVersionsField(jsonContent, state) {
            var typesVersions = readPackageJsonField(jsonContent, "typesVersions", "object", state);
            if (typesVersions === undefined)
                return;
            if (state.traceEnabled) {
                trace(state.host, ts.Diagnostics.package_json_has_a_typesVersions_field_with_version_specific_path_mappings);
            }
            return typesVersions;
        }
        function readPackageJsonTypesVersionPaths(jsonContent, state) {
            var typesVersions = readPackageJsonTypesVersionsField(jsonContent, state);
            if (typesVersions === undefined)
                return;
            if (state.traceEnabled) {
                for (var key in typesVersions) {
                    if (ts.hasProperty(typesVersions, key) && !ts.VersionRange.tryParse(key)) {
                        trace(state.host, ts.Diagnostics.package_json_has_a_typesVersions_entry_0_that_is_not_a_valid_semver_range, key);
                    }
                }
            }
            var result = getPackageJsonTypesVersionsPaths(typesVersions);
            if (!result) {
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.package_json_does_not_have_a_typesVersions_entry_that_matches_version_0, ts.versionMajorMinor);
                }
                return;
            }
            var bestVersionKey = result.version, bestVersionPaths = result.paths;
            if (typeof bestVersionPaths !== "object") {
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.Expected_type_of_0_field_in_package_json_to_be_1_got_2, "typesVersions['" + bestVersionKey + "']", "object", typeof bestVersionPaths);
                }
                return;
            }
            return result;
        }
        var typeScriptVersion;
        /* @internal */
        function getPackageJsonTypesVersionsPaths(typesVersions) {
            if (!typeScriptVersion)
                typeScriptVersion = new ts.Version(ts.version);
            for (var key in typesVersions) {
                if (!ts.hasProperty(typesVersions, key))
                    continue;
                var keyRange = ts.VersionRange.tryParse(key);
                if (keyRange === undefined) {
                    continue;
                }
                // return the first entry whose range matches the current compiler version.
                if (keyRange.test(typeScriptVersion)) {
                    return { version: key, paths: typesVersions[key] };
                }
            }
        }
        ts.getPackageJsonTypesVersionsPaths = getPackageJsonTypesVersionsPaths;
        function getEffectiveTypeRoots(options, host) {
            if (options.typeRoots) {
                return options.typeRoots;
            }
            var currentDirectory;
            if (options.configFilePath) {
                currentDirectory = ts.getDirectoryPath(options.configFilePath);
            }
            else if (host.getCurrentDirectory) {
                currentDirectory = host.getCurrentDirectory();
            }
            if (currentDirectory !== undefined) {
                return getDefaultTypeRoots(currentDirectory, host);
            }
        }
        ts.getEffectiveTypeRoots = getEffectiveTypeRoots;
        /**
         * Returns the path to every node_modules/@types directory from some ancestor directory.
         * Returns undefined if there are none.
         */
        function getDefaultTypeRoots(currentDirectory, host) {
            if (!host.directoryExists) {
                return [ts.combinePaths(currentDirectory, nodeModulesAtTypes)];
                // And if it doesn't exist, tough.
            }
            var typeRoots;
            ts.forEachAncestorDirectory(ts.normalizePath(currentDirectory), function (directory) {
                var atTypes = ts.combinePaths(directory, nodeModulesAtTypes);
                if (host.directoryExists(atTypes)) {
                    (typeRoots || (typeRoots = [])).push(atTypes);
                }
                return undefined;
            });
            return typeRoots;
        }
        var nodeModulesAtTypes = ts.combinePaths("node_modules", "@types");
        /**
         * @param {string | undefined} containingFile - file that contains type reference directive, can be undefined if containing file is unknown.
         * This is possible in case if resolution is performed for directives specified via 'types' parameter. In this case initial path for secondary lookups
         * is assumed to be the same as root directory of the project.
         */
        function resolveTypeReferenceDirective(typeReferenceDirectiveName, containingFile, options, host, redirectedReference) {
            var traceEnabled = isTraceEnabled(options, host);
            if (redirectedReference) {
                options = redirectedReference.commandLine.options;
            }
            var failedLookupLocations = [];
            var moduleResolutionState = { compilerOptions: options, host: host, traceEnabled: traceEnabled, failedLookupLocations: failedLookupLocations };
            var typeRoots = getEffectiveTypeRoots(options, host);
            if (traceEnabled) {
                if (containingFile === undefined) {
                    if (typeRoots === undefined) {
                        trace(host, ts.Diagnostics.Resolving_type_reference_directive_0_containing_file_not_set_root_directory_not_set, typeReferenceDirectiveName);
                    }
                    else {
                        trace(host, ts.Diagnostics.Resolving_type_reference_directive_0_containing_file_not_set_root_directory_1, typeReferenceDirectiveName, typeRoots);
                    }
                }
                else {
                    if (typeRoots === undefined) {
                        trace(host, ts.Diagnostics.Resolving_type_reference_directive_0_containing_file_1_root_directory_not_set, typeReferenceDirectiveName, containingFile);
                    }
                    else {
                        trace(host, ts.Diagnostics.Resolving_type_reference_directive_0_containing_file_1_root_directory_2, typeReferenceDirectiveName, containingFile, typeRoots);
                    }
                }
                if (redirectedReference) {
                    trace(host, ts.Diagnostics.Using_compiler_options_of_project_reference_redirect_0, redirectedReference.sourceFile.fileName);
                }
            }
            var resolved = primaryLookup();
            var primary = true;
            if (!resolved) {
                resolved = secondaryLookup();
                primary = false;
            }
            var resolvedTypeReferenceDirective;
            if (resolved) {
                var fileName = resolved.fileName, packageId = resolved.packageId;
                var resolvedFileName = options.preserveSymlinks ? fileName : realPath(fileName, host, traceEnabled);
                if (traceEnabled) {
                    if (packageId) {
                        trace(host, ts.Diagnostics.Type_reference_directive_0_was_successfully_resolved_to_1_with_Package_ID_2_primary_Colon_3, typeReferenceDirectiveName, resolvedFileName, ts.packageIdToString(packageId), primary);
                    }
                    else {
                        trace(host, ts.Diagnostics.Type_reference_directive_0_was_successfully_resolved_to_1_primary_Colon_2, typeReferenceDirectiveName, resolvedFileName, primary);
                    }
                }
                resolvedTypeReferenceDirective = { primary: primary, resolvedFileName: resolvedFileName, packageId: packageId, isExternalLibraryImport: pathContainsNodeModules(fileName) };
            }
            return { resolvedTypeReferenceDirective: resolvedTypeReferenceDirective, failedLookupLocations: failedLookupLocations };
            function primaryLookup() {
                // Check primary library paths
                if (typeRoots && typeRoots.length) {
                    if (traceEnabled) {
                        trace(host, ts.Diagnostics.Resolving_with_primary_search_path_0, typeRoots.join(", "));
                    }
                    return ts.firstDefined(typeRoots, function (typeRoot) {
                        var candidate = ts.combinePaths(typeRoot, typeReferenceDirectiveName);
                        var candidateDirectory = ts.getDirectoryPath(candidate);
                        var directoryExists = ts.directoryProbablyExists(candidateDirectory, host);
                        if (!directoryExists && traceEnabled) {
                            trace(host, ts.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it, candidateDirectory);
                        }
                        return resolvedTypeScriptOnly(loadNodeModuleFromDirectory(Extensions.DtsOnly, candidate, !directoryExists, moduleResolutionState));
                    });
                }
                else {
                    if (traceEnabled) {
                        trace(host, ts.Diagnostics.Root_directory_cannot_be_determined_skipping_primary_search_paths);
                    }
                }
            }
            function secondaryLookup() {
                var initialLocationForSecondaryLookup = containingFile && ts.getDirectoryPath(containingFile);
                if (initialLocationForSecondaryLookup !== undefined) {
                    // check secondary locations
                    if (traceEnabled) {
                        trace(host, ts.Diagnostics.Looking_up_in_node_modules_folder_initial_location_0, initialLocationForSecondaryLookup);
                    }
                    var result = void 0;
                    if (!ts.isExternalModuleNameRelative(typeReferenceDirectiveName)) {
                        var searchResult = loadModuleFromNearestNodeModulesDirectory(Extensions.DtsOnly, typeReferenceDirectiveName, initialLocationForSecondaryLookup, moduleResolutionState, /*cache*/ undefined, /*redirectedReference*/ undefined);
                        result = searchResult && searchResult.value;
                    }
                    else {
                        var candidate = ts.normalizePathAndParts(ts.combinePaths(initialLocationForSecondaryLookup, typeReferenceDirectiveName)).path;
                        result = nodeLoadModuleByRelativeName(Extensions.DtsOnly, candidate, /*onlyRecordFailures*/ false, moduleResolutionState, /*considerPackageJson*/ true);
                    }
                    var resolvedFile = resolvedTypeScriptOnly(result);
                    if (!resolvedFile && traceEnabled) {
                        trace(host, ts.Diagnostics.Type_reference_directive_0_was_not_resolved, typeReferenceDirectiveName);
                    }
                    return resolvedFile;
                }
                else {
                    if (traceEnabled) {
                        trace(host, ts.Diagnostics.Containing_file_is_not_specified_and_root_directory_cannot_be_determined_skipping_lookup_in_node_modules_folder);
                    }
                }
            }
        }
        ts.resolveTypeReferenceDirective = resolveTypeReferenceDirective;
        /**
         * Given a set of options, returns the set of type directive names
         *   that should be included for this program automatically.
         * This list could either come from the config file,
         *   or from enumerating the types root + initial secondary types lookup location.
         * More type directives might appear in the program later as a result of loading actual source files;
         *   this list is only the set of defaults that are implicitly included.
         */
        function getAutomaticTypeDirectiveNames(options, host) {
            // Use explicit type list from tsconfig.json
            if (options.types) {
                return options.types;
            }
            // Walk the primary type lookup locations
            var result = [];
            if (host.directoryExists && host.getDirectories) {
                var typeRoots = getEffectiveTypeRoots(options, host);
                if (typeRoots) {
                    for (var _i = 0, typeRoots_1 = typeRoots; _i < typeRoots_1.length; _i++) {
                        var root = typeRoots_1[_i];
                        if (host.directoryExists(root)) {
                            for (var _a = 0, _b = host.getDirectories(root); _a < _b.length; _a++) {
                                var typeDirectivePath = _b[_a];
                                var normalized = ts.normalizePath(typeDirectivePath);
                                var packageJsonPath = ts.combinePaths(root, normalized, "package.json");
                                // `types-publisher` sometimes creates packages with `"typings": null` for packages that don't provide their own types.
                                // See `createNotNeededPackageJSON` in the types-publisher` repo.
                                // eslint-disable-next-line no-null/no-null
                                var isNotNeededPackage = host.fileExists(packageJsonPath) && ts.readJson(packageJsonPath, host).typings === null;
                                if (!isNotNeededPackage) {
                                    var baseFileName = ts.getBaseFileName(normalized);
                                    // At this stage, skip results with leading dot.
                                    if (baseFileName.charCodeAt(0) !== 46 /* dot */) {
                                        // Return just the type directive names
                                        result.push(baseFileName);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            return result;
        }
        ts.getAutomaticTypeDirectiveNames = getAutomaticTypeDirectiveNames;
        function createModuleResolutionCache(currentDirectory, getCanonicalFileName, options) {
            return createModuleResolutionCacheWithMaps(createCacheWithRedirects(options), createCacheWithRedirects(options), currentDirectory, getCanonicalFileName);
        }
        ts.createModuleResolutionCache = createModuleResolutionCache;
        /*@internal*/
        function createCacheWithRedirects(options) {
            var ownMap = ts.createMap();
            var redirectsMap = ts.createMap();
            return {
                ownMap: ownMap,
                redirectsMap: redirectsMap,
                getOrCreateMapOfCacheRedirects: getOrCreateMapOfCacheRedirects,
                clear: clear,
                setOwnOptions: setOwnOptions,
                setOwnMap: setOwnMap
            };
            function setOwnOptions(newOptions) {
                options = newOptions;
            }
            function setOwnMap(newOwnMap) {
                ownMap = newOwnMap;
            }
            function getOrCreateMapOfCacheRedirects(redirectedReference) {
                if (!redirectedReference) {
                    return ownMap;
                }
                var path = redirectedReference.sourceFile.path;
                var redirects = redirectsMap.get(path);
                if (!redirects) {
                    // Reuse map if redirected reference map uses same resolution
                    redirects = !options || ts.optionsHaveModuleResolutionChanges(options, redirectedReference.commandLine.options) ? ts.createMap() : ownMap;
                    redirectsMap.set(path, redirects);
                }
                return redirects;
            }
            function clear() {
                ownMap.clear();
                redirectsMap.clear();
            }
        }
        ts.createCacheWithRedirects = createCacheWithRedirects;
        /*@internal*/
        function createModuleResolutionCacheWithMaps(directoryToModuleNameMap, moduleNameToDirectoryMap, currentDirectory, getCanonicalFileName) {
            return { getOrCreateCacheForDirectory: getOrCreateCacheForDirectory, getOrCreateCacheForModuleName: getOrCreateCacheForModuleName, directoryToModuleNameMap: directoryToModuleNameMap, moduleNameToDirectoryMap: moduleNameToDirectoryMap };
            function getOrCreateCacheForDirectory(directoryName, redirectedReference) {
                var path = ts.toPath(directoryName, currentDirectory, getCanonicalFileName);
                return getOrCreateCache(directoryToModuleNameMap, redirectedReference, path, ts.createMap);
            }
            function getOrCreateCacheForModuleName(nonRelativeModuleName, redirectedReference) {
                ts.Debug.assert(!ts.isExternalModuleNameRelative(nonRelativeModuleName));
                return getOrCreateCache(moduleNameToDirectoryMap, redirectedReference, nonRelativeModuleName, createPerModuleNameCache);
            }
            function getOrCreateCache(cacheWithRedirects, redirectedReference, key, create) {
                var cache = cacheWithRedirects.getOrCreateMapOfCacheRedirects(redirectedReference);
                var result = cache.get(key);
                if (!result) {
                    result = create();
                    cache.set(key, result);
                }
                return result;
            }
            function createPerModuleNameCache() {
                var directoryPathMap = ts.createMap();
                return { get: get, set: set };
                function get(directory) {
                    return directoryPathMap.get(ts.toPath(directory, currentDirectory, getCanonicalFileName));
                }
                /**
                 * At first this function add entry directory -> module resolution result to the table.
                 * Then it computes the set of parent folders for 'directory' that should have the same module resolution result
                 * and for every parent folder in set it adds entry: parent -> module resolution. .
                 * Lets say we first directory name: /a/b/c/d/e and resolution result is: /a/b/bar.ts.
                 * Set of parent folders that should have the same result will be:
                 * [
                 *     /a/b/c/d, /a/b/c, /a/b
                 * ]
                 * this means that request for module resolution from file in any of these folder will be immediately found in cache.
                 */
                function set(directory, result) {
                    var path = ts.toPath(directory, currentDirectory, getCanonicalFileName);
                    // if entry is already in cache do nothing
                    if (directoryPathMap.has(path)) {
                        return;
                    }
                    directoryPathMap.set(path, result);
                    var resolvedFileName = result.resolvedModule &&
                        (result.resolvedModule.originalPath || result.resolvedModule.resolvedFileName);
                    // find common prefix between directory and resolved file name
                    // this common prefix should be the shortest path that has the same resolution
                    // directory: /a/b/c/d/e
                    // resolvedFileName: /a/b/foo.d.ts
                    // commonPrefix: /a/b
                    // for failed lookups cache the result for every directory up to root
                    var commonPrefix = resolvedFileName && getCommonPrefix(path, resolvedFileName);
                    var current = path;
                    while (current !== commonPrefix) {
                        var parent = ts.getDirectoryPath(current);
                        if (parent === current || directoryPathMap.has(parent)) {
                            break;
                        }
                        directoryPathMap.set(parent, result);
                        current = parent;
                    }
                }
                function getCommonPrefix(directory, resolution) {
                    var resolutionDirectory = ts.toPath(ts.getDirectoryPath(resolution), currentDirectory, getCanonicalFileName);
                    // find first position where directory and resolution differs
                    var i = 0;
                    var limit = Math.min(directory.length, resolutionDirectory.length);
                    while (i < limit && directory.charCodeAt(i) === resolutionDirectory.charCodeAt(i)) {
                        i++;
                    }
                    if (i === directory.length && (resolutionDirectory.length === i || resolutionDirectory[i] === ts.directorySeparator)) {
                        return directory;
                    }
                    var rootLength = ts.getRootLength(directory);
                    if (i < rootLength) {
                        return undefined;
                    }
                    var sep = directory.lastIndexOf(ts.directorySeparator, i - 1);
                    if (sep === -1) {
                        return undefined;
                    }
                    return directory.substr(0, Math.max(sep, rootLength));
                }
            }
        }
        ts.createModuleResolutionCacheWithMaps = createModuleResolutionCacheWithMaps;
        function resolveModuleNameFromCache(moduleName, containingFile, cache) {
            var containingDirectory = ts.getDirectoryPath(containingFile);
            var perFolderCache = cache && cache.getOrCreateCacheForDirectory(containingDirectory);
            return perFolderCache && perFolderCache.get(moduleName);
        }
        ts.resolveModuleNameFromCache = resolveModuleNameFromCache;
        function resolveModuleName(moduleName, containingFile, compilerOptions, host, cache, redirectedReference) {
            var traceEnabled = isTraceEnabled(compilerOptions, host);
            if (redirectedReference) {
                compilerOptions = redirectedReference.commandLine.options;
            }
            if (traceEnabled) {
                trace(host, ts.Diagnostics.Resolving_module_0_from_1, moduleName, containingFile);
                if (redirectedReference) {
                    trace(host, ts.Diagnostics.Using_compiler_options_of_project_reference_redirect_0, redirectedReference.sourceFile.fileName);
                }
            }
            var containingDirectory = ts.getDirectoryPath(containingFile);
            var perFolderCache = cache && cache.getOrCreateCacheForDirectory(containingDirectory, redirectedReference);
            var result = perFolderCache && perFolderCache.get(moduleName);
            if (result) {
                if (traceEnabled) {
                    trace(host, ts.Diagnostics.Resolution_for_module_0_was_found_in_cache_from_location_1, moduleName, containingDirectory);
                }
            }
            else {
                var moduleResolution = compilerOptions.moduleResolution;
                if (moduleResolution === undefined) {
                    moduleResolution = ts.getEmitModuleKind(compilerOptions) === ts.ModuleKind.CommonJS ? ts.ModuleResolutionKind.NodeJs : ts.ModuleResolutionKind.Classic;
                    if (traceEnabled) {
                        trace(host, ts.Diagnostics.Module_resolution_kind_is_not_specified_using_0, ts.ModuleResolutionKind[moduleResolution]);
                    }
                }
                else {
                    if (traceEnabled) {
                        trace(host, ts.Diagnostics.Explicitly_specified_module_resolution_kind_Colon_0, ts.ModuleResolutionKind[moduleResolution]);
                    }
                }
                ts.perfLogger.logStartResolveModule(moduleName /* , containingFile, ModuleResolutionKind[moduleResolution]*/);
                switch (moduleResolution) {
                    case ts.ModuleResolutionKind.NodeJs:
                        result = nodeModuleNameResolver(moduleName, containingFile, compilerOptions, host, cache, redirectedReference);
                        break;
                    case ts.ModuleResolutionKind.Classic:
                        result = classicNameResolver(moduleName, containingFile, compilerOptions, host, cache, redirectedReference);
                        break;
                    default:
                        return ts.Debug.fail("Unexpected moduleResolution: " + moduleResolution);
                }
                if (result && result.resolvedModule)
                    ts.perfLogger.logInfoEvent("Module \"" + moduleName + "\" resolved to \"" + result.resolvedModule.resolvedFileName + "\"");
                ts.perfLogger.logStopResolveModule((result && result.resolvedModule) ? "" + result.resolvedModule.resolvedFileName : "null");
                if (perFolderCache) {
                    perFolderCache.set(moduleName, result);
                    if (!ts.isExternalModuleNameRelative(moduleName)) {
                        // put result in per-module name cache
                        cache.getOrCreateCacheForModuleName(moduleName, redirectedReference).set(containingDirectory, result);
                    }
                }
            }
            if (traceEnabled) {
                if (result.resolvedModule) {
                    if (result.resolvedModule.packageId) {
                        trace(host, ts.Diagnostics.Module_name_0_was_successfully_resolved_to_1_with_Package_ID_2, moduleName, result.resolvedModule.resolvedFileName, ts.packageIdToString(result.resolvedModule.packageId));
                    }
                    else {
                        trace(host, ts.Diagnostics.Module_name_0_was_successfully_resolved_to_1, moduleName, result.resolvedModule.resolvedFileName);
                    }
                }
                else {
                    trace(host, ts.Diagnostics.Module_name_0_was_not_resolved, moduleName);
                }
            }
            return result;
        }
        ts.resolveModuleName = resolveModuleName;
        /**
         * Any module resolution kind can be augmented with optional settings: 'baseUrl', 'paths' and 'rootDirs' - they are used to
         * mitigate differences between design time structure of the project and its runtime counterpart so the same import name
         * can be resolved successfully by TypeScript compiler and runtime module loader.
         * If these settings are set then loading procedure will try to use them to resolve module name and it can of failure it will
         * fallback to standard resolution routine.
         *
         * - baseUrl - this setting controls how non-relative module names are resolved. If this setting is specified then non-relative
         * names will be resolved relative to baseUrl: i.e. if baseUrl is '/a/b' then candidate location to resolve module name 'c/d' will
         * be '/a/b/c/d'
         * - paths - this setting can only be used when baseUrl is specified. allows to tune how non-relative module names
         * will be resolved based on the content of the module name.
         * Structure of 'paths' compiler options
         * 'paths': {
         *    pattern-1: [...substitutions],
         *    pattern-2: [...substitutions],
         *    ...
         *    pattern-n: [...substitutions]
         * }
         * Pattern here is a string that can contain zero or one '*' character. During module resolution module name will be matched against
         * all patterns in the list. Matching for patterns that don't contain '*' means that module name must be equal to pattern respecting the case.
         * If pattern contains '*' then to match pattern "<prefix>*<suffix>" module name must start with the <prefix> and end with <suffix>.
         * <MatchedStar> denotes part of the module name between <prefix> and <suffix>.
         * If module name can be matches with multiple patterns then pattern with the longest prefix will be picked.
         * After selecting pattern we'll use list of substitutions to get candidate locations of the module and the try to load module
         * from the candidate location.
         * Substitution is a string that can contain zero or one '*'. To get candidate location from substitution we'll pick every
         * substitution in the list and replace '*' with <MatchedStar> string. If candidate location is not rooted it
         * will be converted to absolute using baseUrl.
         * For example:
         * baseUrl: /a/b/c
         * "paths": {
         *     // match all module names
         *     "*": [
         *         "*",        // use matched name as is,
         *                     // <matched name> will be looked as /a/b/c/<matched name>
         *
         *         "folder1/*" // substitution will convert matched name to 'folder1/<matched name>',
         *                     // since it is not rooted then final candidate location will be /a/b/c/folder1/<matched name>
         *     ],
         *     // match module names that start with 'components/'
         *     "components/*": [ "/root/components/*" ] // substitution will convert /components/folder1/<matched name> to '/root/components/folder1/<matched name>',
         *                                              // it is rooted so it will be final candidate location
         * }
         *
         * 'rootDirs' allows the project to be spreaded across multiple locations and resolve modules with relative names as if
         * they were in the same location. For example lets say there are two files
         * '/local/src/content/file1.ts'
         * '/shared/components/contracts/src/content/protocols/file2.ts'
         * After bundling content of '/shared/components/contracts/src' will be merged with '/local/src' so
         * if file1 has the following import 'import {x} from "./protocols/file2"' it will be resolved successfully in runtime.
         * 'rootDirs' provides the way to tell compiler that in order to get the whole project it should behave as if content of all
         * root dirs were merged together.
         * I.e. for the example above 'rootDirs' will have two entries: [ '/local/src', '/shared/components/contracts/src' ].
         * Compiler will first convert './protocols/file2' into absolute path relative to the location of containing file:
         * '/local/src/content/protocols/file2' and try to load it - failure.
         * Then it will search 'rootDirs' looking for a longest matching prefix of this absolute path and if such prefix is found - absolute path will
         * be converted to a path relative to found rootDir entry './content/protocols/file2' (*). As a last step compiler will check all remaining
         * entries in 'rootDirs', use them to build absolute path out of (*) and try to resolve module from this location.
         */
        function tryLoadModuleUsingOptionalResolutionSettings(extensions, moduleName, containingDirectory, loader, state) {
            var resolved = tryLoadModuleUsingPathsIfEligible(extensions, moduleName, loader, state);
            if (resolved)
                return resolved.value;
            if (!ts.isExternalModuleNameRelative(moduleName)) {
                return tryLoadModuleUsingBaseUrl(extensions, moduleName, loader, state);
            }
            else {
                return tryLoadModuleUsingRootDirs(extensions, moduleName, containingDirectory, loader, state);
            }
        }
        function tryLoadModuleUsingPathsIfEligible(extensions, moduleName, loader, state) {
            var _a = state.compilerOptions, baseUrl = _a.baseUrl, paths = _a.paths;
            if (baseUrl && paths && !ts.pathIsRelative(moduleName)) {
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.baseUrl_option_is_set_to_0_using_this_value_to_resolve_non_relative_module_name_1, baseUrl, moduleName);
                    trace(state.host, ts.Diagnostics.paths_option_is_specified_looking_for_a_pattern_to_match_module_name_0, moduleName);
                }
                return tryLoadModuleUsingPaths(extensions, moduleName, baseUrl, paths, loader, /*onlyRecordFailures*/ false, state);
            }
        }
        function tryLoadModuleUsingRootDirs(extensions, moduleName, containingDirectory, loader, state) {
            if (!state.compilerOptions.rootDirs) {
                return undefined;
            }
            if (state.traceEnabled) {
                trace(state.host, ts.Diagnostics.rootDirs_option_is_set_using_it_to_resolve_relative_module_name_0, moduleName);
            }
            var candidate = ts.normalizePath(ts.combinePaths(containingDirectory, moduleName));
            var matchedRootDir;
            var matchedNormalizedPrefix;
            for (var _i = 0, _a = state.compilerOptions.rootDirs; _i < _a.length; _i++) {
                var rootDir = _a[_i];
                // rootDirs are expected to be absolute
                // in case of tsconfig.json this will happen automatically - compiler will expand relative names
                // using location of tsconfig.json as base location
                var normalizedRoot = ts.normalizePath(rootDir);
                if (!ts.endsWith(normalizedRoot, ts.directorySeparator)) {
                    normalizedRoot += ts.directorySeparator;
                }
                var isLongestMatchingPrefix = ts.startsWith(candidate, normalizedRoot) &&
                    (matchedNormalizedPrefix === undefined || matchedNormalizedPrefix.length < normalizedRoot.length);
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.Checking_if_0_is_the_longest_matching_prefix_for_1_2, normalizedRoot, candidate, isLongestMatchingPrefix);
                }
                if (isLongestMatchingPrefix) {
                    matchedNormalizedPrefix = normalizedRoot;
                    matchedRootDir = rootDir;
                }
            }
            if (matchedNormalizedPrefix) {
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.Longest_matching_prefix_for_0_is_1, candidate, matchedNormalizedPrefix);
                }
                var suffix = candidate.substr(matchedNormalizedPrefix.length);
                // first - try to load from a initial location
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.Loading_0_from_the_root_dir_1_candidate_location_2, suffix, matchedNormalizedPrefix, candidate);
                }
                var resolvedFileName = loader(extensions, candidate, !ts.directoryProbablyExists(containingDirectory, state.host), state);
                if (resolvedFileName) {
                    return resolvedFileName;
                }
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.Trying_other_entries_in_rootDirs);
                }
                // then try to resolve using remaining entries in rootDirs
                for (var _b = 0, _c = state.compilerOptions.rootDirs; _b < _c.length; _b++) {
                    var rootDir = _c[_b];
                    if (rootDir === matchedRootDir) {
                        // skip the initially matched entry
                        continue;
                    }
                    var candidate_1 = ts.combinePaths(ts.normalizePath(rootDir), suffix);
                    if (state.traceEnabled) {
                        trace(state.host, ts.Diagnostics.Loading_0_from_the_root_dir_1_candidate_location_2, suffix, rootDir, candidate_1);
                    }
                    var baseDirectory = ts.getDirectoryPath(candidate_1);
                    var resolvedFileName_1 = loader(extensions, candidate_1, !ts.directoryProbablyExists(baseDirectory, state.host), state);
                    if (resolvedFileName_1) {
                        return resolvedFileName_1;
                    }
                }
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.Module_resolution_using_rootDirs_has_failed);
                }
            }
            return undefined;
        }
        function tryLoadModuleUsingBaseUrl(extensions, moduleName, loader, state) {
            var baseUrl = state.compilerOptions.baseUrl;
            if (!baseUrl) {
                return undefined;
            }
            if (state.traceEnabled) {
                trace(state.host, ts.Diagnostics.baseUrl_option_is_set_to_0_using_this_value_to_resolve_non_relative_module_name_1, baseUrl, moduleName);
            }
            var candidate = ts.normalizePath(ts.combinePaths(baseUrl, moduleName));
            if (state.traceEnabled) {
                trace(state.host, ts.Diagnostics.Resolving_module_name_0_relative_to_base_url_1_2, moduleName, baseUrl, candidate);
            }
            return loader(extensions, candidate, !ts.directoryProbablyExists(ts.getDirectoryPath(candidate), state.host), state);
        }
        /**
         * Expose resolution logic to allow us to use Node module resolution logic from arbitrary locations.
         * No way to do this with `require()`: https://github.com/nodejs/node/issues/5963
         * Throws an error if the module can't be resolved.
         */
        /* @internal */
        function resolveJSModule(moduleName, initialDir, host) {
            var _a = tryResolveJSModuleWorker(moduleName, initialDir, host), resolvedModule = _a.resolvedModule, failedLookupLocations = _a.failedLookupLocations;
            if (!resolvedModule) {
                throw new Error("Could not resolve JS module '" + moduleName + "' starting at '" + initialDir + "'. Looked in: " + failedLookupLocations.join(", "));
            }
            return resolvedModule.resolvedFileName;
        }
        ts.resolveJSModule = resolveJSModule;
        /* @internal */
        function tryResolveJSModule(moduleName, initialDir, host) {
            var resolvedModule = tryResolveJSModuleWorker(moduleName, initialDir, host).resolvedModule;
            return resolvedModule && resolvedModule.resolvedFileName;
        }
        ts.tryResolveJSModule = tryResolveJSModule;
        var jsOnlyExtensions = [Extensions.JavaScript];
        var tsExtensions = [Extensions.TypeScript, Extensions.JavaScript];
        var tsPlusJsonExtensions = __spreadArrays(tsExtensions, [Extensions.Json]);
        var tsconfigExtensions = [Extensions.TSConfig];
        function tryResolveJSModuleWorker(moduleName, initialDir, host) {
            return nodeModuleNameResolverWorker(moduleName, initialDir, { moduleResolution: ts.ModuleResolutionKind.NodeJs, allowJs: true }, host, /*cache*/ undefined, jsOnlyExtensions, /*redirectedReferences*/ undefined);
        }
        function nodeModuleNameResolver(moduleName, containingFile, compilerOptions, host, cache, redirectedReference, lookupConfig) {
            return nodeModuleNameResolverWorker(moduleName, ts.getDirectoryPath(containingFile), compilerOptions, host, cache, lookupConfig ? tsconfigExtensions : (compilerOptions.resolveJsonModule ? tsPlusJsonExtensions : tsExtensions), redirectedReference);
        }
        ts.nodeModuleNameResolver = nodeModuleNameResolver;
        function nodeModuleNameResolverWorker(moduleName, containingDirectory, compilerOptions, host, cache, extensions, redirectedReference) {
            var traceEnabled = isTraceEnabled(compilerOptions, host);
            var failedLookupLocations = [];
            var state = { compilerOptions: compilerOptions, host: host, traceEnabled: traceEnabled, failedLookupLocations: failedLookupLocations };
            var result = ts.forEach(extensions, function (ext) { return tryResolve(ext); });
            if (result && result.value) {
                var _a = result.value, resolved = _a.resolved, isExternalLibraryImport = _a.isExternalLibraryImport;
                return createResolvedModuleWithFailedLookupLocations(resolved, isExternalLibraryImport, failedLookupLocations);
            }
            return { resolvedModule: undefined, failedLookupLocations: failedLookupLocations };
            function tryResolve(extensions) {
                var loader = function (extensions, candidate, onlyRecordFailures, state) { return nodeLoadModuleByRelativeName(extensions, candidate, onlyRecordFailures, state, /*considerPackageJson*/ true); };
                var resolved = tryLoadModuleUsingOptionalResolutionSettings(extensions, moduleName, containingDirectory, loader, state);
                if (resolved) {
                    return toSearchResult({ resolved: resolved, isExternalLibraryImport: pathContainsNodeModules(resolved.path) });
                }
                if (!ts.isExternalModuleNameRelative(moduleName)) {
                    if (traceEnabled) {
                        trace(host, ts.Diagnostics.Loading_module_0_from_node_modules_folder_target_file_type_1, moduleName, Extensions[extensions]);
                    }
                    var resolved_1 = loadModuleFromNearestNodeModulesDirectory(extensions, moduleName, containingDirectory, state, cache, redirectedReference);
                    if (!resolved_1)
                        return undefined;
                    var resolvedValue = resolved_1.value;
                    if (!compilerOptions.preserveSymlinks && resolvedValue && !resolvedValue.originalPath) {
                        var path = realPath(resolvedValue.path, host, traceEnabled);
                        var originalPath = path === resolvedValue.path ? undefined : resolvedValue.path;
                        resolvedValue = __assign(__assign({}, resolvedValue), { path: path, originalPath: originalPath });
                    }
                    // For node_modules lookups, get the real path so that multiple accesses to an `npm link`-ed module do not create duplicate files.
                    return { value: resolvedValue && { resolved: resolvedValue, isExternalLibraryImport: true } };
                }
                else {
                    var _a = ts.normalizePathAndParts(ts.combinePaths(containingDirectory, moduleName)), candidate = _a.path, parts = _a.parts;
                    var resolved_2 = nodeLoadModuleByRelativeName(extensions, candidate, /*onlyRecordFailures*/ false, state, /*considerPackageJson*/ true);
                    // Treat explicit "node_modules" import as an external library import.
                    return resolved_2 && toSearchResult({ resolved: resolved_2, isExternalLibraryImport: ts.contains(parts, "node_modules") });
                }
            }
        }
        function realPath(path, host, traceEnabled) {
            if (!host.realpath) {
                return path;
            }
            var real = ts.normalizePath(host.realpath(path));
            if (traceEnabled) {
                trace(host, ts.Diagnostics.Resolving_real_path_for_0_result_1, path, real);
            }
            ts.Debug.assert(host.fileExists(real), path + " linked to nonexistent file " + real);
            return real;
        }
        function nodeLoadModuleByRelativeName(extensions, candidate, onlyRecordFailures, state, considerPackageJson) {
            if (state.traceEnabled) {
                trace(state.host, ts.Diagnostics.Loading_module_as_file_Slash_folder_candidate_module_location_0_target_file_type_1, candidate, Extensions[extensions]);
            }
            if (!ts.hasTrailingDirectorySeparator(candidate)) {
                if (!onlyRecordFailures) {
                    var parentOfCandidate = ts.getDirectoryPath(candidate);
                    if (!ts.directoryProbablyExists(parentOfCandidate, state.host)) {
                        if (state.traceEnabled) {
                            trace(state.host, ts.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it, parentOfCandidate);
                        }
                        onlyRecordFailures = true;
                    }
                }
                var resolvedFromFile = loadModuleFromFile(extensions, candidate, onlyRecordFailures, state);
                if (resolvedFromFile) {
                    var packageDirectory = considerPackageJson ? parseNodeModuleFromPath(resolvedFromFile) : undefined;
                    var packageInfo = packageDirectory ? getPackageJsonInfo(packageDirectory, /*onlyRecordFailures*/ false, state) : undefined;
                    return withPackageId(packageInfo, resolvedFromFile);
                }
            }
            if (!onlyRecordFailures) {
                var candidateExists = ts.directoryProbablyExists(candidate, state.host);
                if (!candidateExists) {
                    if (state.traceEnabled) {
                        trace(state.host, ts.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it, candidate);
                    }
                    onlyRecordFailures = true;
                }
            }
            return loadNodeModuleFromDirectory(extensions, candidate, onlyRecordFailures, state, considerPackageJson);
        }
        /*@internal*/
        ts.nodeModulesPathPart = "/node_modules/";
        /*@internal*/
        function pathContainsNodeModules(path) {
            return ts.stringContains(path, ts.nodeModulesPathPart);
        }
        ts.pathContainsNodeModules = pathContainsNodeModules;
        /**
         * This will be called on the successfully resolved path from `loadModuleFromFile`.
         * (Not neeeded for `loadModuleFromNodeModules` as that looks up the `package.json` as part of resolution.)
         *
         * packageDirectory is the directory of the package itself.
         *   For `blah/node_modules/foo/index.d.ts` this is packageDirectory: "foo"
         *   For `/node_modules/foo/bar.d.ts` this is packageDirectory: "foo"
         *   For `/node_modules/@types/foo/bar/index.d.ts` this is packageDirectory: "@types/foo"
         *   For `/node_modules/foo/bar/index.d.ts` this is packageDirectory: "foo"
         */
        function parseNodeModuleFromPath(resolved) {
            var path = ts.normalizePath(resolved.path);
            var idx = path.lastIndexOf(ts.nodeModulesPathPart);
            if (idx === -1) {
                return undefined;
            }
            var indexAfterNodeModules = idx + ts.nodeModulesPathPart.length;
            var indexAfterPackageName = moveToNextDirectorySeparatorIfAvailable(path, indexAfterNodeModules);
            if (path.charCodeAt(indexAfterNodeModules) === 64 /* at */) {
                indexAfterPackageName = moveToNextDirectorySeparatorIfAvailable(path, indexAfterPackageName);
            }
            return path.slice(0, indexAfterPackageName);
        }
        function moveToNextDirectorySeparatorIfAvailable(path, prevSeparatorIndex) {
            var nextSeparatorIndex = path.indexOf(ts.directorySeparator, prevSeparatorIndex + 1);
            return nextSeparatorIndex === -1 ? prevSeparatorIndex : nextSeparatorIndex;
        }
        function loadModuleFromFileNoPackageId(extensions, candidate, onlyRecordFailures, state) {
            return noPackageId(loadModuleFromFile(extensions, candidate, onlyRecordFailures, state));
        }
        /**
         * @param {boolean} onlyRecordFailures - if true then function won't try to actually load files but instead record all attempts as failures. This flag is necessary
         * in cases when we know upfront that all load attempts will fail (because containing folder does not exists) however we still need to record all failed lookup locations.
         */
        function loadModuleFromFile(extensions, candidate, onlyRecordFailures, state) {
            if (extensions === Extensions.Json || extensions === Extensions.TSConfig) {
                var extensionLess = ts.tryRemoveExtension(candidate, ".json" /* Json */);
                return (extensionLess === undefined && extensions === Extensions.Json) ? undefined : tryAddingExtensions(extensionLess || candidate, extensions, onlyRecordFailures, state);
            }
            // First, try adding an extension. An import of "foo" could be matched by a file "foo.ts", or "foo.js" by "foo.js.ts"
            var resolvedByAddingExtension = tryAddingExtensions(candidate, extensions, onlyRecordFailures, state);
            if (resolvedByAddingExtension) {
                return resolvedByAddingExtension;
            }
            // If that didn't work, try stripping a ".js" or ".jsx" extension and replacing it with a TypeScript one;
            // e.g. "./foo.js" can be matched by "./foo.ts" or "./foo.d.ts"
            if (ts.hasJSFileExtension(candidate)) {
                var extensionless = ts.removeFileExtension(candidate);
                if (state.traceEnabled) {
                    var extension = candidate.substring(extensionless.length);
                    trace(state.host, ts.Diagnostics.File_name_0_has_a_1_extension_stripping_it, candidate, extension);
                }
                return tryAddingExtensions(extensionless, extensions, onlyRecordFailures, state);
            }
        }
        /** Try to return an existing file that adds one of the `extensions` to `candidate`. */
        function tryAddingExtensions(candidate, extensions, onlyRecordFailures, state) {
            if (!onlyRecordFailures) {
                // check if containing folder exists - if it doesn't then just record failures for all supported extensions without disk probing
                var directory = ts.getDirectoryPath(candidate);
                if (directory) {
                    onlyRecordFailures = !ts.directoryProbablyExists(directory, state.host);
                }
            }
            switch (extensions) {
                case Extensions.DtsOnly:
                    return tryExtension(".d.ts" /* Dts */);
                case Extensions.TypeScript:
                    return tryExtension(".ts" /* Ts */) || tryExtension(".tsx" /* Tsx */) || tryExtension(".d.ts" /* Dts */);
                case Extensions.JavaScript:
                    return tryExtension(".js" /* Js */) || tryExtension(".jsx" /* Jsx */);
                case Extensions.TSConfig:
                case Extensions.Json:
                    return tryExtension(".json" /* Json */);
            }
            function tryExtension(ext) {
                var path = tryFile(candidate + ext, onlyRecordFailures, state);
                return path === undefined ? undefined : { path: path, ext: ext };
            }
        }
        /** Return the file if it exists. */
        function tryFile(fileName, onlyRecordFailures, state) {
            if (!onlyRecordFailures) {
                if (state.host.fileExists(fileName)) {
                    if (state.traceEnabled) {
                        trace(state.host, ts.Diagnostics.File_0_exist_use_it_as_a_name_resolution_result, fileName);
                    }
                    return fileName;
                }
                else {
                    if (state.traceEnabled) {
                        trace(state.host, ts.Diagnostics.File_0_does_not_exist, fileName);
                    }
                }
            }
            state.failedLookupLocations.push(fileName);
            return undefined;
        }
        function loadNodeModuleFromDirectory(extensions, candidate, onlyRecordFailures, state, considerPackageJson) {
            if (considerPackageJson === void 0) {
                considerPackageJson = true;
            }
            var packageInfo = considerPackageJson ? getPackageJsonInfo(candidate, onlyRecordFailures, state) : undefined;
            var packageJsonContent = packageInfo && packageInfo.packageJsonContent;
            var versionPaths = packageInfo && packageInfo.versionPaths;
            return withPackageId(packageInfo, loadNodeModuleFromDirectoryWorker(extensions, candidate, onlyRecordFailures, state, packageJsonContent, versionPaths));
        }
        function getPackageJsonInfo(packageDirectory, onlyRecordFailures, state) {
            var host = state.host, traceEnabled = state.traceEnabled;
            var directoryExists = !onlyRecordFailures && ts.directoryProbablyExists(packageDirectory, host);
            var packageJsonPath = ts.combinePaths(packageDirectory, "package.json");
            if (directoryExists && host.fileExists(packageJsonPath)) {
                var packageJsonContent = ts.readJson(packageJsonPath, host);
                if (traceEnabled) {
                    trace(host, ts.Diagnostics.Found_package_json_at_0, packageJsonPath);
                }
                var versionPaths = readPackageJsonTypesVersionPaths(packageJsonContent, state);
                return { packageDirectory: packageDirectory, packageJsonContent: packageJsonContent, versionPaths: versionPaths };
            }
            else {
                if (directoryExists && traceEnabled) {
                    trace(host, ts.Diagnostics.File_0_does_not_exist, packageJsonPath);
                }
                // record package json as one of failed lookup locations - in the future if this file will appear it will invalidate resolution results
                state.failedLookupLocations.push(packageJsonPath);
            }
        }
        function loadNodeModuleFromDirectoryWorker(extensions, candidate, onlyRecordFailures, state, jsonContent, versionPaths) {
            var packageFile;
            if (jsonContent) {
                switch (extensions) {
                    case Extensions.JavaScript:
                    case Extensions.Json:
                        packageFile = readPackageJsonMainField(jsonContent, candidate, state);
                        break;
                    case Extensions.TypeScript:
                        // When resolving typescript modules, try resolving using main field as well
                        packageFile = readPackageJsonTypesFields(jsonContent, candidate, state) || readPackageJsonMainField(jsonContent, candidate, state);
                        break;
                    case Extensions.DtsOnly:
                        packageFile = readPackageJsonTypesFields(jsonContent, candidate, state);
                        break;
                    case Extensions.TSConfig:
                        packageFile = readPackageJsonTSConfigField(jsonContent, candidate, state);
                        break;
                    default:
                        return ts.Debug.assertNever(extensions);
                }
            }
            var loader = function (extensions, candidate, onlyRecordFailures, state) {
                var fromFile = tryFile(candidate, onlyRecordFailures, state);
                if (fromFile) {
                    var resolved = resolvedIfExtensionMatches(extensions, fromFile);
                    if (resolved) {
                        return noPackageId(resolved);
                    }
                    if (state.traceEnabled) {
                        trace(state.host, ts.Diagnostics.File_0_has_an_unsupported_extension_so_skipping_it, fromFile);
                    }
                }
                // Even if extensions is DtsOnly, we can still look up a .ts file as a result of package.json "types"
                var nextExtensions = extensions === Extensions.DtsOnly ? Extensions.TypeScript : extensions;
                // Don't do package.json lookup recursively, because Node.js' package lookup doesn't.
                return nodeLoadModuleByRelativeName(nextExtensions, candidate, onlyRecordFailures, state, /*considerPackageJson*/ false);
            };
            var onlyRecordFailuresForPackageFile = packageFile ? !ts.directoryProbablyExists(ts.getDirectoryPath(packageFile), state.host) : undefined;
            var onlyRecordFailuresForIndex = onlyRecordFailures || !ts.directoryProbablyExists(candidate, state.host);
            var indexPath = ts.combinePaths(candidate, extensions === Extensions.TSConfig ? "tsconfig" : "index");
            if (versionPaths && (!packageFile || ts.containsPath(candidate, packageFile))) {
                var moduleName = ts.getRelativePathFromDirectory(candidate, packageFile || indexPath, /*ignoreCase*/ false);
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_match_module_name_2, versionPaths.version, ts.version, moduleName);
                }
                var result = tryLoadModuleUsingPaths(extensions, moduleName, candidate, versionPaths.paths, loader, onlyRecordFailuresForPackageFile || onlyRecordFailuresForIndex, state);
                if (result) {
                    return removeIgnoredPackageId(result.value);
                }
            }
            // It won't have a `packageId` set, because we disabled `considerPackageJson`.
            var packageFileResult = packageFile && removeIgnoredPackageId(loader(extensions, packageFile, onlyRecordFailuresForPackageFile, state));
            if (packageFileResult)
                return packageFileResult;
            return loadModuleFromFile(extensions, indexPath, onlyRecordFailuresForIndex, state);
        }
        /** Resolve from an arbitrarily specified file. Return `undefined` if it has an unsupported extension. */
        function resolvedIfExtensionMatches(extensions, path) {
            var ext = ts.tryGetExtensionFromPath(path);
            return ext !== undefined && extensionIsOk(extensions, ext) ? { path: path, ext: ext } : undefined;
        }
        /** True if `extension` is one of the supported `extensions`. */
        function extensionIsOk(extensions, extension) {
            switch (extensions) {
                case Extensions.JavaScript:
                    return extension === ".js" /* Js */ || extension === ".jsx" /* Jsx */;
                case Extensions.TSConfig:
                case Extensions.Json:
                    return extension === ".json" /* Json */;
                case Extensions.TypeScript:
                    return extension === ".ts" /* Ts */ || extension === ".tsx" /* Tsx */ || extension === ".d.ts" /* Dts */;
                case Extensions.DtsOnly:
                    return extension === ".d.ts" /* Dts */;
            }
        }
        /* @internal */
        function parsePackageName(moduleName) {
            var idx = moduleName.indexOf(ts.directorySeparator);
            if (moduleName[0] === "@") {
                idx = moduleName.indexOf(ts.directorySeparator, idx + 1);
            }
            return idx === -1 ? { packageName: moduleName, rest: "" } : { packageName: moduleName.slice(0, idx), rest: moduleName.slice(idx + 1) };
        }
        ts.parsePackageName = parsePackageName;
        function loadModuleFromNearestNodeModulesDirectory(extensions, moduleName, directory, state, cache, redirectedReference) {
            return loadModuleFromNearestNodeModulesDirectoryWorker(extensions, moduleName, directory, state, /*typesScopeOnly*/ false, cache, redirectedReference);
        }
        function loadModuleFromNearestNodeModulesDirectoryTypesScope(moduleName, directory, state) {
            // Extensions parameter here doesn't actually matter, because typesOnly ensures we're just doing @types lookup, which is always DtsOnly.
            return loadModuleFromNearestNodeModulesDirectoryWorker(Extensions.DtsOnly, moduleName, directory, state, /*typesScopeOnly*/ true, /*cache*/ undefined, /*redirectedReference*/ undefined);
        }
        function loadModuleFromNearestNodeModulesDirectoryWorker(extensions, moduleName, directory, state, typesScopeOnly, cache, redirectedReference) {
            var perModuleNameCache = cache && cache.getOrCreateCacheForModuleName(moduleName, redirectedReference);
            return ts.forEachAncestorDirectory(ts.normalizeSlashes(directory), function (ancestorDirectory) {
                if (ts.getBaseFileName(ancestorDirectory) !== "node_modules") {
                    var resolutionFromCache = tryFindNonRelativeModuleNameInCache(perModuleNameCache, moduleName, ancestorDirectory, state);
                    if (resolutionFromCache) {
                        return resolutionFromCache;
                    }
                    return toSearchResult(loadModuleFromImmediateNodeModulesDirectory(extensions, moduleName, ancestorDirectory, state, typesScopeOnly));
                }
            });
        }
        function loadModuleFromImmediateNodeModulesDirectory(extensions, moduleName, directory, state, typesScopeOnly) {
            var nodeModulesFolder = ts.combinePaths(directory, "node_modules");
            var nodeModulesFolderExists = ts.directoryProbablyExists(nodeModulesFolder, state.host);
            if (!nodeModulesFolderExists && state.traceEnabled) {
                trace(state.host, ts.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it, nodeModulesFolder);
            }
            var packageResult = typesScopeOnly ? undefined : loadModuleFromSpecificNodeModulesDirectory(extensions, moduleName, nodeModulesFolder, nodeModulesFolderExists, state);
            if (packageResult) {
                return packageResult;
            }
            if (extensions === Extensions.TypeScript || extensions === Extensions.DtsOnly) {
                var nodeModulesAtTypes_1 = ts.combinePaths(nodeModulesFolder, "@types");
                var nodeModulesAtTypesExists = nodeModulesFolderExists;
                if (nodeModulesFolderExists && !ts.directoryProbablyExists(nodeModulesAtTypes_1, state.host)) {
                    if (state.traceEnabled) {
                        trace(state.host, ts.Diagnostics.Directory_0_does_not_exist_skipping_all_lookups_in_it, nodeModulesAtTypes_1);
                    }
                    nodeModulesAtTypesExists = false;
                }
                return loadModuleFromSpecificNodeModulesDirectory(Extensions.DtsOnly, mangleScopedPackageNameWithTrace(moduleName, state), nodeModulesAtTypes_1, nodeModulesAtTypesExists, state);
            }
        }
        function loadModuleFromSpecificNodeModulesDirectory(extensions, moduleName, nodeModulesDirectory, nodeModulesDirectoryExists, state) {
            var candidate = ts.normalizePath(ts.combinePaths(nodeModulesDirectory, moduleName));
            // First look for a nested package.json, as in `node_modules/foo/bar/package.json`.
            var packageInfo = getPackageJsonInfo(candidate, !nodeModulesDirectoryExists, state);
            if (packageInfo) {
                var fromFile = loadModuleFromFile(extensions, candidate, !nodeModulesDirectoryExists, state);
                if (fromFile) {
                    return noPackageId(fromFile);
                }
                var fromDirectory = loadNodeModuleFromDirectoryWorker(extensions, candidate, !nodeModulesDirectoryExists, state, packageInfo.packageJsonContent, packageInfo.versionPaths);
                return withPackageId(packageInfo, fromDirectory);
            }
            var loader = function (extensions, candidate, onlyRecordFailures, state) {
                var pathAndExtension = loadModuleFromFile(extensions, candidate, onlyRecordFailures, state) ||
                    loadNodeModuleFromDirectoryWorker(extensions, candidate, onlyRecordFailures, state, packageInfo && packageInfo.packageJsonContent, packageInfo && packageInfo.versionPaths);
                return withPackageId(packageInfo, pathAndExtension);
            };
            var _a = parsePackageName(moduleName), packageName = _a.packageName, rest = _a.rest;
            if (rest !== "") { // If "rest" is empty, we just did this search above.
                var packageDirectory = ts.combinePaths(nodeModulesDirectory, packageName);
                // Don't use a "types" or "main" from here because we're not loading the root, but a subdirectory -- just here for the packageId and path mappings.
                packageInfo = getPackageJsonInfo(packageDirectory, !nodeModulesDirectoryExists, state);
                if (packageInfo && packageInfo.versionPaths) {
                    if (state.traceEnabled) {
                        trace(state.host, ts.Diagnostics.package_json_has_a_typesVersions_entry_0_that_matches_compiler_version_1_looking_for_a_pattern_to_match_module_name_2, packageInfo.versionPaths.version, ts.version, rest);
                    }
                    var packageDirectoryExists = nodeModulesDirectoryExists && ts.directoryProbablyExists(packageDirectory, state.host);
                    var fromPaths = tryLoadModuleUsingPaths(extensions, rest, packageDirectory, packageInfo.versionPaths.paths, loader, !packageDirectoryExists, state);
                    if (fromPaths) {
                        return fromPaths.value;
                    }
                }
            }
            return loader(extensions, candidate, !nodeModulesDirectoryExists, state);
        }
        function tryLoadModuleUsingPaths(extensions, moduleName, baseDirectory, paths, loader, onlyRecordFailures, state) {
            var matchedPattern = ts.matchPatternOrExact(ts.getOwnKeys(paths), moduleName);
            if (matchedPattern) {
                var matchedStar_1 = ts.isString(matchedPattern) ? undefined : ts.matchedText(matchedPattern, moduleName);
                var matchedPatternText = ts.isString(matchedPattern) ? matchedPattern : ts.patternText(matchedPattern);
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.Module_name_0_matched_pattern_1, moduleName, matchedPatternText);
                }
                var resolved = ts.forEach(paths[matchedPatternText], function (subst) {
                    var path = matchedStar_1 ? subst.replace("*", matchedStar_1) : subst;
                    var candidate = ts.normalizePath(ts.combinePaths(baseDirectory, path));
                    if (state.traceEnabled) {
                        trace(state.host, ts.Diagnostics.Trying_substitution_0_candidate_module_location_Colon_1, subst, path);
                    }
                    // A path mapping may have an extension, in contrast to an import, which should omit it.
                    var extension = ts.tryGetExtensionFromPath(candidate);
                    if (extension !== undefined) {
                        var path_1 = tryFile(candidate, onlyRecordFailures, state);
                        if (path_1 !== undefined) {
                            return noPackageId({ path: path_1, ext: extension });
                        }
                    }
                    return loader(extensions, candidate, onlyRecordFailures || !ts.directoryProbablyExists(ts.getDirectoryPath(candidate), state.host), state);
                });
                return { value: resolved };
            }
        }
        /** Double underscores are used in DefinitelyTyped to delimit scoped packages. */
        var mangledScopedPackageSeparator = "__";
        /** For a scoped package, we must look in `@types/foo__bar` instead of `@types/@foo/bar`. */
        function mangleScopedPackageNameWithTrace(packageName, state) {
            var mangled = mangleScopedPackageName(packageName);
            if (state.traceEnabled && mangled !== packageName) {
                trace(state.host, ts.Diagnostics.Scoped_package_detected_looking_in_0, mangled);
            }
            return mangled;
        }
        /* @internal */
        function getTypesPackageName(packageName) {
            return "@types/" + mangleScopedPackageName(packageName);
        }
        ts.getTypesPackageName = getTypesPackageName;
        /* @internal */
        function mangleScopedPackageName(packageName) {
            if (ts.startsWith(packageName, "@")) {
                var replaceSlash = packageName.replace(ts.directorySeparator, mangledScopedPackageSeparator);
                if (replaceSlash !== packageName) {
                    return replaceSlash.slice(1); // Take off the "@"
                }
            }
            return packageName;
        }
        ts.mangleScopedPackageName = mangleScopedPackageName;
        /* @internal */
        function getPackageNameFromTypesPackageName(mangledName) {
            var withoutAtTypePrefix = ts.removePrefix(mangledName, "@types/");
            if (withoutAtTypePrefix !== mangledName) {
                return unmangleScopedPackageName(withoutAtTypePrefix);
            }
            return mangledName;
        }
        ts.getPackageNameFromTypesPackageName = getPackageNameFromTypesPackageName;
        /* @internal */
        function unmangleScopedPackageName(typesPackageName) {
            return ts.stringContains(typesPackageName, mangledScopedPackageSeparator) ?
                "@" + typesPackageName.replace(mangledScopedPackageSeparator, ts.directorySeparator) :
                typesPackageName;
        }
        ts.unmangleScopedPackageName = unmangleScopedPackageName;
        function tryFindNonRelativeModuleNameInCache(cache, moduleName, containingDirectory, state) {
            var _a;
            var result = cache && cache.get(containingDirectory);
            if (result) {
                if (state.traceEnabled) {
                    trace(state.host, ts.Diagnostics.Resolution_for_module_0_was_found_in_cache_from_location_1, moduleName, containingDirectory);
                }
                (_a = state.failedLookupLocations).push.apply(_a, result.failedLookupLocations);
                return { value: result.resolvedModule && { path: result.resolvedModule.resolvedFileName, originalPath: result.resolvedModule.originalPath || true, extension: result.resolvedModule.extension, packageId: result.resolvedModule.packageId } };
            }
        }
        function classicNameResolver(moduleName, containingFile, compilerOptions, host, cache, redirectedReference) {
            var traceEnabled = isTraceEnabled(compilerOptions, host);
            var failedLookupLocations = [];
            var state = { compilerOptions: compilerOptions, host: host, traceEnabled: traceEnabled, failedLookupLocations: failedLookupLocations };
            var containingDirectory = ts.getDirectoryPath(containingFile);
            var resolved = tryResolve(Extensions.TypeScript) || tryResolve(Extensions.JavaScript);
            // No originalPath because classic resolution doesn't resolve realPath
            return createResolvedModuleWithFailedLookupLocations(resolved && resolved.value, /*isExternalLibraryImport*/ false, failedLookupLocations);
            function tryResolve(extensions) {
                var resolvedUsingSettings = tryLoadModuleUsingOptionalResolutionSettings(extensions, moduleName, containingDirectory, loadModuleFromFileNoPackageId, state);
                if (resolvedUsingSettings) {
                    return { value: resolvedUsingSettings };
                }
                if (!ts.isExternalModuleNameRelative(moduleName)) {
                    var perModuleNameCache_1 = cache && cache.getOrCreateCacheForModuleName(moduleName, redirectedReference);
                    // Climb up parent directories looking for a module.
                    var resolved_3 = ts.forEachAncestorDirectory(containingDirectory, function (directory) {
                        var resolutionFromCache = tryFindNonRelativeModuleNameInCache(perModuleNameCache_1, moduleName, directory, state);
                        if (resolutionFromCache) {
                            return resolutionFromCache;
                        }
                        var searchName = ts.normalizePath(ts.combinePaths(directory, moduleName));
                        return toSearchResult(loadModuleFromFileNoPackageId(extensions, searchName, /*onlyRecordFailures*/ false, state));
                    });
                    if (resolved_3) {
                        return resolved_3;
                    }
                    if (extensions === Extensions.TypeScript) {
                        // If we didn't find the file normally, look it up in @types.
                        return loadModuleFromNearestNodeModulesDirectoryTypesScope(moduleName, containingDirectory, state);
                    }
                }
                else {
                    var candidate = ts.normalizePath(ts.combinePaths(containingDirectory, moduleName));
                    return toSearchResult(loadModuleFromFileNoPackageId(extensions, candidate, /*onlyRecordFailures*/ false, state));
                }
            }
        }
        ts.classicNameResolver = classicNameResolver;
        /**
         * A host may load a module from a global cache of typings.
         * This is the minumum code needed to expose that functionality; the rest is in the host.
         */
        /* @internal */
        function loadModuleFromGlobalCache(moduleName, projectName, compilerOptions, host, globalCache) {
            var traceEnabled = isTraceEnabled(compilerOptions, host);
            if (traceEnabled) {
                trace(host, ts.Diagnostics.Auto_discovery_for_typings_is_enabled_in_project_0_Running_extra_resolution_pass_for_module_1_using_cache_location_2, projectName, moduleName, globalCache);
            }
            var failedLookupLocations = [];
            var state = { compilerOptions: compilerOptions, host: host, traceEnabled: traceEnabled, failedLookupLocations: failedLookupLocations };
            var resolved = loadModuleFromImmediateNodeModulesDirectory(Extensions.DtsOnly, moduleName, globalCache, state, /*typesScopeOnly*/ false);
            return createResolvedModuleWithFailedLookupLocations(resolved, /*isExternalLibraryImport*/ true, failedLookupLocations);
        }
        ts.loadModuleFromGlobalCache = loadModuleFromGlobalCache;
        /**
         * Wraps value to SearchResult.
         * @returns undefined if value is undefined or { value } otherwise
         */
        function toSearchResult(value) {
            return value !== undefined ? { value: value } : undefined;
        }
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var ModuleInstanceState;
        (function (ModuleInstanceState) {
            ModuleInstanceState[ModuleInstanceState["NonInstantiated"] = 0] = "NonInstantiated";
            ModuleInstanceState[ModuleInstanceState["Instantiated"] = 1] = "Instantiated";
            ModuleInstanceState[ModuleInstanceState["ConstEnumOnly"] = 2] = "ConstEnumOnly";
        })(ModuleInstanceState = ts.ModuleInstanceState || (ts.ModuleInstanceState = {}));
        function getModuleInstanceState(node, visited) {
            if (node.body && !node.body.parent) {
                // getModuleInstanceStateForAliasTarget needs to walk up the parent chain, so parent pointers must be set on this tree already
                setParentPointers(node, node.body);
            }
            return node.body ? getModuleInstanceStateCached(node.body, visited) : 1 /* Instantiated */;
        }
        ts.getModuleInstanceState = getModuleInstanceState;
        function getModuleInstanceStateCached(node, visited) {
            if (visited === void 0) {
                visited = ts.createMap();
            }
            var nodeId = "" + ts.getNodeId(node);
            if (visited.has(nodeId)) {
                return visited.get(nodeId) || 0 /* NonInstantiated */;
            }
            visited.set(nodeId, undefined);
            var result = getModuleInstanceStateWorker(node, visited);
            visited.set(nodeId, result);
            return result;
        }
        function getModuleInstanceStateWorker(node, visited) {
            // A module is uninstantiated if it contains only
            switch (node.kind) {
                // 1. interface declarations, type alias declarations
                case 246 /* InterfaceDeclaration */:
                case 247 /* TypeAliasDeclaration */:
                    return 0 /* NonInstantiated */;
                // 2. const enum declarations
                case 248 /* EnumDeclaration */:
                    if (ts.isEnumConst(node)) {
                        return 2 /* ConstEnumOnly */;
                    }
                    break;
                // 3. non-exported import declarations
                case 254 /* ImportDeclaration */:
                case 253 /* ImportEqualsDeclaration */:
                    if (!(ts.hasModifier(node, 1 /* Export */))) {
                        return 0 /* NonInstantiated */;
                    }
                    break;
                // 4. Export alias declarations pointing at only uninstantiated modules or things uninstantiated modules contain
                case 260 /* ExportDeclaration */:
                    var exportDeclaration = node;
                    if (!exportDeclaration.moduleSpecifier && exportDeclaration.exportClause && exportDeclaration.exportClause.kind === 261 /* NamedExports */) {
                        var state = 0 /* NonInstantiated */;
                        for (var _i = 0, _a = exportDeclaration.exportClause.elements; _i < _a.length; _i++) {
                            var specifier = _a[_i];
                            var specifierState = getModuleInstanceStateForAliasTarget(specifier, visited);
                            if (specifierState > state) {
                                state = specifierState;
                            }
                            if (state === 1 /* Instantiated */) {
                                return state;
                            }
                        }
                        return state;
                    }
                    break;
                // 5. other uninstantiated module declarations.
                case 250 /* ModuleBlock */: {
                    var state_1 = 0 /* NonInstantiated */;
                    ts.forEachChild(node, function (n) {
                        var childState = getModuleInstanceStateCached(n, visited);
                        switch (childState) {
                            case 0 /* NonInstantiated */:
                                // child is non-instantiated - continue searching
                                return;
                            case 2 /* ConstEnumOnly */:
                                // child is const enum only - record state and continue searching
                                state_1 = 2 /* ConstEnumOnly */;
                                return;
                            case 1 /* Instantiated */:
                                // child is instantiated - record state and stop
                                state_1 = 1 /* Instantiated */;
                                return true;
                            default:
                                ts.Debug.assertNever(childState);
                        }
                    });
                    return state_1;
                }
                case 249 /* ModuleDeclaration */:
                    return getModuleInstanceState(node, visited);
                case 75 /* Identifier */:
                    // Only jsdoc typedef definition can exist in jsdoc namespace, and it should
                    // be considered the same as type alias
                    if (node.isInJSDocNamespace) {
                        return 0 /* NonInstantiated */;
                    }
            }
            return 1 /* Instantiated */;
        }
        function getModuleInstanceStateForAliasTarget(specifier, visited) {
            var name = specifier.propertyName || specifier.name;
            var p = specifier.parent;
            while (p) {
                if (ts.isBlock(p) || ts.isModuleBlock(p) || ts.isSourceFile(p)) {
                    var statements = p.statements;
                    var found = void 0;
                    for (var _i = 0, statements_1 = statements; _i < statements_1.length; _i++) {
                        var statement = statements_1[_i];
                        if (ts.nodeHasName(statement, name)) {
                            if (!statement.parent) {
                                setParentPointers(p, statement);
                            }
                            var state = getModuleInstanceStateCached(statement, visited);
                            if (found === undefined || state > found) {
                                found = state;
                            }
                            if (found === 1 /* Instantiated */) {
                                return found;
                            }
                        }
                    }
                    if (found !== undefined) {
                        return found;
                    }
                }
                p = p.parent;
            }
            return 1 /* Instantiated */; // Couldn't locate, assume could refer to a value
        }
        var ContainerFlags;
        (function (ContainerFlags) {
            // The current node is not a container, and no container manipulation should happen before
            // recursing into it.
            ContainerFlags[ContainerFlags["None"] = 0] = "None";
            // The current node is a container.  It should be set as the current container (and block-
            // container) before recursing into it.  The current node does not have locals.  Examples:
            //
            //      Classes, ObjectLiterals, TypeLiterals, Interfaces...
            ContainerFlags[ContainerFlags["IsContainer"] = 1] = "IsContainer";
            // The current node is a block-scoped-container.  It should be set as the current block-
            // container before recursing into it.  Examples:
            //
            //      Blocks (when not parented by functions), Catch clauses, For/For-in/For-of statements...
            ContainerFlags[ContainerFlags["IsBlockScopedContainer"] = 2] = "IsBlockScopedContainer";
            // The current node is the container of a control flow path. The current control flow should
            // be saved and restored, and a new control flow initialized within the container.
            ContainerFlags[ContainerFlags["IsControlFlowContainer"] = 4] = "IsControlFlowContainer";
            ContainerFlags[ContainerFlags["IsFunctionLike"] = 8] = "IsFunctionLike";
            ContainerFlags[ContainerFlags["IsFunctionExpression"] = 16] = "IsFunctionExpression";
            ContainerFlags[ContainerFlags["HasLocals"] = 32] = "HasLocals";
            ContainerFlags[ContainerFlags["IsInterface"] = 64] = "IsInterface";
            ContainerFlags[ContainerFlags["IsObjectLiteralOrClassExpressionMethod"] = 128] = "IsObjectLiteralOrClassExpressionMethod";
        })(ContainerFlags || (ContainerFlags = {}));
        function initFlowNode(node) {
            ts.Debug.attachFlowNodeDebugInfo(node);
            return node;
        }
        var binder = createBinder();
        function bindSourceFile(file, options) {
            ts.performance.mark("beforeBind");
            ts.perfLogger.logStartBindFile("" + file.fileName);
            binder(file, options);
            ts.perfLogger.logStopBindFile();
            ts.performance.mark("afterBind");
            ts.performance.measure("Bind", "beforeBind", "afterBind");
        }
        ts.bindSourceFile = bindSourceFile;
        function createBinder() {
            var file;
            var options;
            var languageVersion;
            var parent;
            var container;
            var thisParentContainer; // Container one level up
            var blockScopeContainer;
            var lastContainer;
            var delayedTypeAliases;
            var seenThisKeyword;
            // state used by control flow analysis
            var currentFlow;
            var currentBreakTarget;
            var currentContinueTarget;
            var currentReturnTarget;
            var currentTrueTarget;
            var currentFalseTarget;
            var currentExceptionTarget;
            var preSwitchCaseFlow;
            var activeLabelList;
            var hasExplicitReturn;
            // state used for emit helpers
            var emitFlags;
            // If this file is an external module, then it is automatically in strict-mode according to
            // ES6.  If it is not an external module, then we'll determine if it is in strict mode or
            // not depending on if we see "use strict" in certain places or if we hit a class/namespace
            // or if compiler options contain alwaysStrict.
            var inStrictMode;
            var symbolCount = 0;
            var Symbol;
            var classifiableNames;
            var unreachableFlow = { flags: 1 /* Unreachable */ };
            var reportedUnreachableFlow = { flags: 1 /* Unreachable */ };
            // state used to aggregate transform flags during bind.
            var subtreeTransformFlags = 0 /* None */;
            var skipTransformFlagAggregation;
            /**
             * Inside the binder, we may create a diagnostic for an as-yet unbound node (with potentially no parent pointers, implying no accessible source file)
             * If so, the node _must_ be in the current file (as that's the only way anything could have traversed to it to yield it as the error node)
             * This version of `createDiagnosticForNode` uses the binder's context to account for this, and always yields correct diagnostics even in these situations.
             */
            function createDiagnosticForNode(node, message, arg0, arg1, arg2) {
                return ts.createDiagnosticForNodeInSourceFile(ts.getSourceFileOfNode(node) || file, node, message, arg0, arg1, arg2);
            }
            function bindSourceFile(f, opts) {
                file = f;
                options = opts;
                languageVersion = ts.getEmitScriptTarget(options);
                inStrictMode = bindInStrictMode(file, opts);
                classifiableNames = ts.createUnderscoreEscapedMap();
                symbolCount = 0;
                skipTransformFlagAggregation = file.isDeclarationFile;
                Symbol = ts.objectAllocator.getSymbolConstructor();
                // Attach debugging information if necessary
                ts.Debug.attachFlowNodeDebugInfo(unreachableFlow);
                ts.Debug.attachFlowNodeDebugInfo(reportedUnreachableFlow);
                if (!file.locals) {
                    bind(file);
                    file.symbolCount = symbolCount;
                    file.classifiableNames = classifiableNames;
                    delayedBindJSDocTypedefTag();
                }
                file = undefined;
                options = undefined;
                languageVersion = undefined;
                parent = undefined;
                container = undefined;
                thisParentContainer = undefined;
                blockScopeContainer = undefined;
                lastContainer = undefined;
                delayedTypeAliases = undefined;
                seenThisKeyword = false;
                currentFlow = undefined;
                currentBreakTarget = undefined;
                currentContinueTarget = undefined;
                currentReturnTarget = undefined;
                currentTrueTarget = undefined;
                currentFalseTarget = undefined;
                currentExceptionTarget = undefined;
                activeLabelList = undefined;
                hasExplicitReturn = false;
                emitFlags = 0 /* None */;
                subtreeTransformFlags = 0 /* None */;
            }
            return bindSourceFile;
            function bindInStrictMode(file, opts) {
                if (ts.getStrictOptionValue(opts, "alwaysStrict") && !file.isDeclarationFile) {
                    // bind in strict mode source files with alwaysStrict option
                    return true;
                }
                else {
                    return !!file.externalModuleIndicator;
                }
            }
            function createSymbol(flags, name) {
                symbolCount++;
                return new Symbol(flags, name);
            }
            function addDeclarationToSymbol(symbol, node, symbolFlags) {
                symbol.flags |= symbolFlags;
                node.symbol = symbol;
                symbol.declarations = ts.appendIfUnique(symbol.declarations, node);
                if (symbolFlags & (32 /* Class */ | 384 /* Enum */ | 1536 /* Module */ | 3 /* Variable */) && !symbol.exports) {
                    symbol.exports = ts.createSymbolTable();
                }
                if (symbolFlags & (32 /* Class */ | 64 /* Interface */ | 2048 /* TypeLiteral */ | 4096 /* ObjectLiteral */) && !symbol.members) {
                    symbol.members = ts.createSymbolTable();
                }
                // On merge of const enum module with class or function, reset const enum only flag (namespaces will already recalculate)
                if (symbol.constEnumOnlyModule && (symbol.flags & (16 /* Function */ | 32 /* Class */ | 256 /* RegularEnum */))) {
                    symbol.constEnumOnlyModule = false;
                }
                if (symbolFlags & 111551 /* Value */) {
                    setValueDeclaration(symbol, node);
                }
            }
            function setValueDeclaration(symbol, node) {
                var valueDeclaration = symbol.valueDeclaration;
                if (!valueDeclaration ||
                    (ts.isAssignmentDeclaration(valueDeclaration) && !ts.isAssignmentDeclaration(node)) ||
                    (valueDeclaration.kind !== node.kind && ts.isEffectiveModuleDeclaration(valueDeclaration))) {
                    // other kinds of value declarations take precedence over modules and assignment declarations
                    symbol.valueDeclaration = node;
                }
            }
            // Should not be called on a declaration with a computed property name,
            // unless it is a well known Symbol.
            function getDeclarationName(node) {
                if (node.kind === 259 /* ExportAssignment */) {
                    return node.isExportEquals ? "export=" /* ExportEquals */ : "default" /* Default */;
                }
                var name = ts.getNameOfDeclaration(node);
                if (name) {
                    if (ts.isAmbientModule(node)) {
                        var moduleName = ts.getTextOfIdentifierOrLiteral(name);
                        return (ts.isGlobalScopeAugmentation(node) ? "__global" : "\"" + moduleName + "\"");
                    }
                    if (name.kind === 154 /* ComputedPropertyName */) {
                        var nameExpression = name.expression;
                        // treat computed property names where expression is string/numeric literal as just string/numeric literal
                        if (ts.isStringOrNumericLiteralLike(nameExpression)) {
                            return ts.escapeLeadingUnderscores(nameExpression.text);
                        }
                        if (ts.isSignedNumericLiteral(nameExpression)) {
                            return ts.tokenToString(nameExpression.operator) + nameExpression.operand.text;
                        }
                        ts.Debug.assert(ts.isWellKnownSymbolSyntactically(nameExpression));
                        return ts.getPropertyNameForKnownSymbolName(ts.idText(nameExpression.name));
                    }
                    if (ts.isWellKnownSymbolSyntactically(name)) {
                        return ts.getPropertyNameForKnownSymbolName(ts.idText(name.name));
                    }
                    if (ts.isPrivateIdentifier(name)) {
                        // containingClass exists because private names only allowed inside classes
                        var containingClass = ts.getContainingClass(node);
                        if (!containingClass) {
                            // we can get here in cases where there is already a parse error.
                            return undefined;
                        }
                        var containingClassSymbol = containingClass.symbol;
                        return ts.getSymbolNameForPrivateIdentifier(containingClassSymbol, name.escapedText);
                    }
                    return ts.isPropertyNameLiteral(name) ? ts.getEscapedTextOfIdentifierOrLiteral(name) : undefined;
                }
                switch (node.kind) {
                    case 162 /* Constructor */:
                        return "__constructor" /* Constructor */;
                    case 170 /* FunctionType */:
                    case 165 /* CallSignature */:
                    case 305 /* JSDocSignature */:
                        return "__call" /* Call */;
                    case 171 /* ConstructorType */:
                    case 166 /* ConstructSignature */:
                        return "__new" /* New */;
                    case 167 /* IndexSignature */:
                        return "__index" /* Index */;
                    case 260 /* ExportDeclaration */:
                        return "__export" /* ExportStar */;
                    case 290 /* SourceFile */:
                        // json file should behave as
                        // module.exports = ...
                        return "export=" /* ExportEquals */;
                    case 209 /* BinaryExpression */:
                        if (ts.getAssignmentDeclarationKind(node) === 2 /* ModuleExports */) {
                            // module.exports = ...
                            return "export=" /* ExportEquals */;
                        }
                        ts.Debug.fail("Unknown binary declaration kind");
                        break;
                    case 300 /* JSDocFunctionType */:
                        return (ts.isJSDocConstructSignature(node) ? "__new" /* New */ : "__call" /* Call */);
                    case 156 /* Parameter */:
                        // Parameters with names are handled at the top of this function.  Parameters
                        // without names can only come from JSDocFunctionTypes.
                        ts.Debug.assert(node.parent.kind === 300 /* JSDocFunctionType */, "Impossible parameter parent kind", function () { return "parent is: " + (ts.SyntaxKind ? ts.SyntaxKind[node.parent.kind] : node.parent.kind) + ", expected JSDocFunctionType"; });
                        var functionType = node.parent;
                        var index = functionType.parameters.indexOf(node);
                        return "arg" + index;
                }
            }
            function getDisplayName(node) {
                return ts.isNamedDeclaration(node) ? ts.declarationNameToString(node.name) : ts.unescapeLeadingUnderscores(ts.Debug.assertDefined(getDeclarationName(node)));
            }
            /**
             * Declares a Symbol for the node and adds it to symbols. Reports errors for conflicting identifier names.
             * @param symbolTable - The symbol table which node will be added to.
             * @param parent - node's parent declaration.
             * @param node - The declaration to be added to the symbol table
             * @param includes - The SymbolFlags that node has in addition to its declaration type (eg: export, ambient, etc.)
             * @param excludes - The flags which node cannot be declared alongside in a symbol table. Used to report forbidden declarations.
             */
            function declareSymbol(symbolTable, parent, node, includes, excludes, isReplaceableByMethod) {
                ts.Debug.assert(!ts.hasDynamicName(node));
                var isDefaultExport = ts.hasModifier(node, 512 /* Default */);
                // The exported symbol for an export default function/class node is always named "default"
                var name = isDefaultExport && parent ? "default" /* Default */ : getDeclarationName(node);
                var symbol;
                if (name === undefined) {
                    symbol = createSymbol(0 /* None */, "__missing" /* Missing */);
                }
                else {
                    // Check and see if the symbol table already has a symbol with this name.  If not,
                    // create a new symbol with this name and add it to the table.  Note that we don't
                    // give the new symbol any flags *yet*.  This ensures that it will not conflict
                    // with the 'excludes' flags we pass in.
                    //
                    // If we do get an existing symbol, see if it conflicts with the new symbol we're
                    // creating.  For example, a 'var' symbol and a 'class' symbol will conflict within
                    // the same symbol table.  If we have a conflict, report the issue on each
                    // declaration we have for this symbol, and then create a new symbol for this
                    // declaration.
                    //
                    // Note that when properties declared in Javascript constructors
                    // (marked by isReplaceableByMethod) conflict with another symbol, the property loses.
                    // Always. This allows the common Javascript pattern of overwriting a prototype method
                    // with an bound instance method of the same type: `this.method = this.method.bind(this)`
                    //
                    // If we created a new symbol, either because we didn't have a symbol with this name
                    // in the symbol table, or we conflicted with an existing symbol, then just add this
                    // node as the sole declaration of the new symbol.
                    //
                    // Otherwise, we'll be merging into a compatible existing symbol (for example when
                    // you have multiple 'vars' with the same name in the same container).  In this case
                    // just add this node into the declarations list of the symbol.
                    symbol = symbolTable.get(name);
                    if (includes & 2885600 /* Classifiable */) {
                        classifiableNames.set(name, true);
                    }
                    if (!symbol) {
                        symbolTable.set(name, symbol = createSymbol(0 /* None */, name));
                        if (isReplaceableByMethod)
                            symbol.isReplaceableByMethod = true;
                    }
                    else if (isReplaceableByMethod && !symbol.isReplaceableByMethod) {
                        // A symbol already exists, so don't add this as a declaration.
                        return symbol;
                    }
                    else if (symbol.flags & excludes) {
                        if (symbol.isReplaceableByMethod) {
                            // Javascript constructor-declared symbols can be discarded in favor of
                            // prototype symbols like methods.
                            symbolTable.set(name, symbol = createSymbol(0 /* None */, name));
                        }
                        else if (!(includes & 3 /* Variable */ && symbol.flags & 67108864 /* Assignment */)) {
                            // Assignment declarations are allowed to merge with variables, no matter what other flags they have.
                            if (ts.isNamedDeclaration(node)) {
                                node.name.parent = node;
                            }
                            // Report errors every position with duplicate declaration
                            // Report errors on previous encountered declarations
                            var message_1 = symbol.flags & 2 /* BlockScopedVariable */
                                ? ts.Diagnostics.Cannot_redeclare_block_scoped_variable_0
                                : ts.Diagnostics.Duplicate_identifier_0;
                            var messageNeedsName_1 = true;
                            if (symbol.flags & 384 /* Enum */ || includes & 384 /* Enum */) {
                                message_1 = ts.Diagnostics.Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations;
                                messageNeedsName_1 = false;
                            }
                            var multipleDefaultExports_1 = false;
                            if (ts.length(symbol.declarations)) {
                                // If the current node is a default export of some sort, then check if
                                // there are any other default exports that we need to error on.
                                // We'll know whether we have other default exports depending on if `symbol` already has a declaration list set.
                                if (isDefaultExport) {
                                    message_1 = ts.Diagnostics.A_module_cannot_have_multiple_default_exports;
                                    messageNeedsName_1 = false;
                                    multipleDefaultExports_1 = true;
                                }
                                else {
                                    // This is to properly report an error in the case "export default { }" is after export default of class declaration or function declaration.
                                    // Error on multiple export default in the following case:
                                    // 1. multiple export default of class declaration or function declaration by checking NodeFlags.Default
                                    // 2. multiple export default of export assignment. This one doesn't have NodeFlags.Default on (as export default doesn't considered as modifiers)
                                    if (symbol.declarations && symbol.declarations.length &&
                                        (node.kind === 259 /* ExportAssignment */ && !node.isExportEquals)) {
                                        message_1 = ts.Diagnostics.A_module_cannot_have_multiple_default_exports;
                                        messageNeedsName_1 = false;
                                        multipleDefaultExports_1 = true;
                                    }
                                }
                            }
                            var relatedInformation_1 = [];
                            if (ts.isTypeAliasDeclaration(node) && ts.nodeIsMissing(node.type) && ts.hasModifier(node, 1 /* Export */) && symbol.flags & (2097152 /* Alias */ | 788968 /* Type */ | 1920 /* Namespace */)) {
                                // export type T; - may have meant export type { T }?
                                relatedInformation_1.push(createDiagnosticForNode(node, ts.Diagnostics.Did_you_mean_0, "export type { " + ts.unescapeLeadingUnderscores(node.name.escapedText) + " }"));
                            }
                            var declarationName_1 = ts.getNameOfDeclaration(node) || node;
                            ts.forEach(symbol.declarations, function (declaration, index) {
                                var decl = ts.getNameOfDeclaration(declaration) || declaration;
                                var diag = createDiagnosticForNode(decl, message_1, messageNeedsName_1 ? getDisplayName(declaration) : undefined);
                                file.bindDiagnostics.push(multipleDefaultExports_1 ? ts.addRelatedInfo(diag, createDiagnosticForNode(declarationName_1, index === 0 ? ts.Diagnostics.Another_export_default_is_here : ts.Diagnostics.and_here)) : diag);
                                if (multipleDefaultExports_1) {
                                    relatedInformation_1.push(createDiagnosticForNode(decl, ts.Diagnostics.The_first_export_default_is_here));
                                }
                            });
                            var diag = createDiagnosticForNode(declarationName_1, message_1, messageNeedsName_1 ? getDisplayName(node) : undefined);
                            file.bindDiagnostics.push(ts.addRelatedInfo.apply(void 0, __spreadArrays([diag], relatedInformation_1)));
                            symbol = createSymbol(0 /* None */, name);
                        }
                    }
                }
                addDeclarationToSymbol(symbol, node, includes);
                if (symbol.parent) {
                    ts.Debug.assert(symbol.parent === parent, "Existing symbol parent should match new one");
                }
                else {
                    symbol.parent = parent;
                }
                return symbol;
            }
            function declareModuleMember(node, symbolFlags, symbolExcludes) {
                var hasExportModifier = ts.getCombinedModifierFlags(node) & 1 /* Export */;
                if (symbolFlags & 2097152 /* Alias */) {
                    if (node.kind === 263 /* ExportSpecifier */ || (node.kind === 253 /* ImportEqualsDeclaration */ && hasExportModifier)) {
                        return declareSymbol(container.symbol.exports, container.symbol, node, symbolFlags, symbolExcludes);
                    }
                    else {
                        return declareSymbol(container.locals, /*parent*/ undefined, node, symbolFlags, symbolExcludes);
                    }
                }
                else {
                    // Exported module members are given 2 symbols: A local symbol that is classified with an ExportValue flag,
                    // and an associated export symbol with all the correct flags set on it. There are 2 main reasons:
                    //
                    //   1. We treat locals and exports of the same name as mutually exclusive within a container.
                    //      That means the binder will issue a Duplicate Identifier error if you mix locals and exports
                    //      with the same name in the same container.
                    //      TODO: Make this a more specific error and decouple it from the exclusion logic.
                    //   2. When we checkIdentifier in the checker, we set its resolved symbol to the local symbol,
                    //      but return the export symbol (by calling getExportSymbolOfValueSymbolIfExported). That way
                    //      when the emitter comes back to it, it knows not to qualify the name if it was found in a containing scope.
                    // NOTE: Nested ambient modules always should go to to 'locals' table to prevent their automatic merge
                    //       during global merging in the checker. Why? The only case when ambient module is permitted inside another module is module augmentation
                    //       and this case is specially handled. Module augmentations should only be merged with original module definition
                    //       and should never be merged directly with other augmentation, and the latter case would be possible if automatic merge is allowed.
                    if (ts.isJSDocTypeAlias(node))
                        ts.Debug.assert(ts.isInJSFile(node)); // We shouldn't add symbols for JSDoc nodes if not in a JS file.
                    if ((!ts.isAmbientModule(node) && (hasExportModifier || container.flags & 64 /* ExportContext */)) || ts.isJSDocTypeAlias(node)) {
                        if (!container.locals || (ts.hasModifier(node, 512 /* Default */) && !getDeclarationName(node))) {
                            return declareSymbol(container.symbol.exports, container.symbol, node, symbolFlags, symbolExcludes); // No local symbol for an unnamed default!
                        }
                        var exportKind = symbolFlags & 111551 /* Value */ ? 1048576 /* ExportValue */ : 0;
                        var local = declareSymbol(container.locals, /*parent*/ undefined, node, exportKind, symbolExcludes);
                        local.exportSymbol = declareSymbol(container.symbol.exports, container.symbol, node, symbolFlags, symbolExcludes);
                        node.localSymbol = local;
                        return local;
                    }
                    else {
                        return declareSymbol(container.locals, /*parent*/ undefined, node, symbolFlags, symbolExcludes);
                    }
                }
            }
            // All container nodes are kept on a linked list in declaration order. This list is used by
            // the getLocalNameOfContainer function in the type checker to validate that the local name
            // used for a container is unique.
            function bindContainer(node, containerFlags) {
                // Before we recurse into a node's children, we first save the existing parent, container
                // and block-container.  Then after we pop out of processing the children, we restore
                // these saved values.
                var saveContainer = container;
                var saveThisParentContainer = thisParentContainer;
                var savedBlockScopeContainer = blockScopeContainer;
                // Depending on what kind of node this is, we may have to adjust the current container
                // and block-container.   If the current node is a container, then it is automatically
                // considered the current block-container as well.  Also, for containers that we know
                // may contain locals, we eagerly initialize the .locals field. We do this because
                // it's highly likely that the .locals will be needed to place some child in (for example,
                // a parameter, or variable declaration).
                //
                // However, we do not proactively create the .locals for block-containers because it's
                // totally normal and common for block-containers to never actually have a block-scoped
                // variable in them.  We don't want to end up allocating an object for every 'block' we
                // run into when most of them won't be necessary.
                //
                // Finally, if this is a block-container, then we clear out any existing .locals object
                // it may contain within it.  This happens in incremental scenarios.  Because we can be
                // reusing a node from a previous compilation, that node may have had 'locals' created
                // for it.  We must clear this so we don't accidentally move any stale data forward from
                // a previous compilation.
                if (containerFlags & 1 /* IsContainer */) {
                    if (node.kind !== 202 /* ArrowFunction */) {
                        thisParentContainer = container;
                    }
                    container = blockScopeContainer = node;
                    if (containerFlags & 32 /* HasLocals */) {
                        container.locals = ts.createSymbolTable();
                    }
                    addToContainerChain(container);
                }
                else if (containerFlags & 2 /* IsBlockScopedContainer */) {
                    blockScopeContainer = node;
                    blockScopeContainer.locals = undefined;
                }
                if (containerFlags & 4 /* IsControlFlowContainer */) {
                    var saveCurrentFlow = currentFlow;
                    var saveBreakTarget = currentBreakTarget;
                    var saveContinueTarget = currentContinueTarget;
                    var saveReturnTarget = currentReturnTarget;
                    var saveExceptionTarget = currentExceptionTarget;
                    var saveActiveLabelList = activeLabelList;
                    var saveHasExplicitReturn = hasExplicitReturn;
                    var isIIFE = containerFlags & 16 /* IsFunctionExpression */ && !ts.hasModifier(node, 256 /* Async */) &&
                        !node.asteriskToken && !!ts.getImmediatelyInvokedFunctionExpression(node);
                    // A non-async, non-generator IIFE is considered part of the containing control flow. Return statements behave
                    // similarly to break statements that exit to a label just past the statement body.
                    if (!isIIFE) {
                        currentFlow = initFlowNode({ flags: 2 /* Start */ });
                        if (containerFlags & (16 /* IsFunctionExpression */ | 128 /* IsObjectLiteralOrClassExpressionMethod */)) {
                            currentFlow.node = node;
                        }
                    }
                    // We create a return control flow graph for IIFEs and constructors. For constructors
                    // we use the return control flow graph in strict property initialization checks.
                    currentReturnTarget = isIIFE || node.kind === 162 /* Constructor */ ? createBranchLabel() : undefined;
                    currentExceptionTarget = undefined;
                    currentBreakTarget = undefined;
                    currentContinueTarget = undefined;
                    activeLabelList = undefined;
                    hasExplicitReturn = false;
                    bindChildren(node);
                    // Reset all reachability check related flags on node (for incremental scenarios)
                    node.flags &= ~2816 /* ReachabilityAndEmitFlags */;
                    if (!(currentFlow.flags & 1 /* Unreachable */) && containerFlags & 8 /* IsFunctionLike */ && ts.nodeIsPresent(node.body)) {
                        node.flags |= 256 /* HasImplicitReturn */;
                        if (hasExplicitReturn)
                            node.flags |= 512 /* HasExplicitReturn */;
                        node.endFlowNode = currentFlow;
                    }
                    if (node.kind === 290 /* SourceFile */) {
                        node.flags |= emitFlags;
                    }
                    if (currentReturnTarget) {
                        addAntecedent(currentReturnTarget, currentFlow);
                        currentFlow = finishFlowLabel(currentReturnTarget);
                        if (node.kind === 162 /* Constructor */) {
                            node.returnFlowNode = currentFlow;
                        }
                    }
                    if (!isIIFE) {
                        currentFlow = saveCurrentFlow;
                    }
                    currentBreakTarget = saveBreakTarget;
                    currentContinueTarget = saveContinueTarget;
                    currentReturnTarget = saveReturnTarget;
                    currentExceptionTarget = saveExceptionTarget;
                    activeLabelList = saveActiveLabelList;
                    hasExplicitReturn = saveHasExplicitReturn;
                }
                else if (containerFlags & 64 /* IsInterface */) {
                    seenThisKeyword = false;
                    bindChildren(node);
                    node.flags = seenThisKeyword ? node.flags | 128 /* ContainsThis */ : node.flags & ~128 /* ContainsThis */;
                }
                else {
                    bindChildren(node);
                }
                container = saveContainer;
                thisParentContainer = saveThisParentContainer;
                blockScopeContainer = savedBlockScopeContainer;
            }
            function bindChildren(node) {
                if (skipTransformFlagAggregation) {
                    bindChildrenWorker(node);
                }
                else if (node.transformFlags & 536870912 /* HasComputedFlags */) {
                    skipTransformFlagAggregation = true;
                    bindChildrenWorker(node);
                    skipTransformFlagAggregation = false;
                    subtreeTransformFlags |= node.transformFlags & ~getTransformFlagsSubtreeExclusions(node.kind);
                }
                else {
                    var savedSubtreeTransformFlags = subtreeTransformFlags;
                    subtreeTransformFlags = 0;
                    bindChildrenWorker(node);
                    subtreeTransformFlags = savedSubtreeTransformFlags | computeTransformFlagsForNode(node, subtreeTransformFlags);
                }
            }
            function bindEachFunctionsFirst(nodes) {
                bindEach(nodes, function (n) { return n.kind === 244 /* FunctionDeclaration */ ? bind(n) : undefined; });
                bindEach(nodes, function (n) { return n.kind !== 244 /* FunctionDeclaration */ ? bind(n) : undefined; });
            }
            function bindEach(nodes, bindFunction) {
                if (bindFunction === void 0) {
                    bindFunction = bind;
                }
                if (nodes === undefined) {
                    return;
                }
                if (skipTransformFlagAggregation) {
                    ts.forEach(nodes, bindFunction);
                }
                else {
                    var savedSubtreeTransformFlags = subtreeTransformFlags;
                    subtreeTransformFlags = 0 /* None */;
                    var nodeArrayFlags = 0 /* None */;
                    for (var _i = 0, nodes_2 = nodes; _i < nodes_2.length; _i++) {
                        var node = nodes_2[_i];
                        bindFunction(node);
                        nodeArrayFlags |= node.transformFlags & ~536870912 /* HasComputedFlags */;
                    }
                    nodes.transformFlags = nodeArrayFlags | 536870912 /* HasComputedFlags */;
                    subtreeTransformFlags |= savedSubtreeTransformFlags;
                }
            }
            function bindEachChild(node) {
                ts.forEachChild(node, bind, bindEach);
            }
            function bindChildrenWorker(node) {
                if (checkUnreachable(node)) {
                    bindEachChild(node);
                    bindJSDoc(node);
                    return;
                }
                if (node.kind >= 225 /* FirstStatement */ && node.kind <= 241 /* LastStatement */ && !options.allowUnreachableCode) {
                    node.flowNode = currentFlow;
                }
                switch (node.kind) {
                    case 229 /* WhileStatement */:
                        bindWhileStatement(node);
                        break;
                    case 228 /* DoStatement */:
                        bindDoStatement(node);
                        break;
                    case 230 /* ForStatement */:
                        bindForStatement(node);
                        break;
                    case 231 /* ForInStatement */:
                    case 232 /* ForOfStatement */:
                        bindForInOrForOfStatement(node);
                        break;
                    case 227 /* IfStatement */:
                        bindIfStatement(node);
                        break;
                    case 235 /* ReturnStatement */:
                    case 239 /* ThrowStatement */:
                        bindReturnOrThrow(node);
                        break;
                    case 234 /* BreakStatement */:
                    case 233 /* ContinueStatement */:
                        bindBreakOrContinueStatement(node);
                        break;
                    case 240 /* TryStatement */:
                        bindTryStatement(node);
                        break;
                    case 237 /* SwitchStatement */:
                        bindSwitchStatement(node);
                        break;
                    case 251 /* CaseBlock */:
                        bindCaseBlock(node);
                        break;
                    case 277 /* CaseClause */:
                        bindCaseClause(node);
                        break;
                    case 226 /* ExpressionStatement */:
                        bindExpressionStatement(node);
                        break;
                    case 238 /* LabeledStatement */:
                        bindLabeledStatement(node);
                        break;
                    case 207 /* PrefixUnaryExpression */:
                        bindPrefixUnaryExpressionFlow(node);
                        break;
                    case 208 /* PostfixUnaryExpression */:
                        bindPostfixUnaryExpressionFlow(node);
                        break;
                    case 209 /* BinaryExpression */:
                        bindBinaryExpressionFlow(node);
                        break;
                    case 203 /* DeleteExpression */:
                        bindDeleteExpressionFlow(node);
                        break;
                    case 210 /* ConditionalExpression */:
                        bindConditionalExpressionFlow(node);
                        break;
                    case 242 /* VariableDeclaration */:
                        bindVariableDeclarationFlow(node);
                        break;
                    case 194 /* PropertyAccessExpression */:
                    case 195 /* ElementAccessExpression */:
                        bindAccessExpressionFlow(node);
                        break;
                    case 196 /* CallExpression */:
                        bindCallExpressionFlow(node);
                        break;
                    case 321 /* JSDocTypedefTag */:
                    case 314 /* JSDocCallbackTag */:
                    case 315 /* JSDocEnumTag */:
                        bindJSDocTypeAlias(node);
                        break;
                    // In source files and blocks, bind functions first to match hoisting that occurs at runtime
                    case 290 /* SourceFile */: {
                        bindEachFunctionsFirst(node.statements);
                        bind(node.endOfFileToken);
                        break;
                    }
                    case 223 /* Block */:
                    case 250 /* ModuleBlock */:
                        bindEachFunctionsFirst(node.statements);
                        break;
                    default:
                        bindEachChild(node);
                        break;
                }
                bindJSDoc(node);
            }
            function isNarrowingExpression(expr) {
                switch (expr.kind) {
                    case 75 /* Identifier */:
                    case 104 /* ThisKeyword */:
                    case 194 /* PropertyAccessExpression */:
                    case 195 /* ElementAccessExpression */:
                        return containsNarrowableReference(expr);
                    case 196 /* CallExpression */:
                        return hasNarrowableArgument(expr);
                    case 200 /* ParenthesizedExpression */:
                        return isNarrowingExpression(expr.expression);
                    case 209 /* BinaryExpression */:
                        return isNarrowingBinaryExpression(expr);
                    case 207 /* PrefixUnaryExpression */:
                        return expr.operator === 53 /* ExclamationToken */ && isNarrowingExpression(expr.operand);
                    case 204 /* TypeOfExpression */:
                        return isNarrowingExpression(expr.expression);
                }
                return false;
            }
            function isNarrowableReference(expr) {
                return expr.kind === 75 /* Identifier */ || expr.kind === 104 /* ThisKeyword */ || expr.kind === 102 /* SuperKeyword */ ||
                    (ts.isPropertyAccessExpression(expr) || ts.isNonNullExpression(expr) || ts.isParenthesizedExpression(expr)) && isNarrowableReference(expr.expression) ||
                    ts.isElementAccessExpression(expr) && ts.isStringOrNumericLiteralLike(expr.argumentExpression) && isNarrowableReference(expr.expression);
            }
            function containsNarrowableReference(expr) {
                return isNarrowableReference(expr) || ts.isOptionalChain(expr) && containsNarrowableReference(expr.expression);
            }
            function hasNarrowableArgument(expr) {
                if (expr.arguments) {
                    for (var _i = 0, _a = expr.arguments; _i < _a.length; _i++) {
                        var argument = _a[_i];
                        if (containsNarrowableReference(argument)) {
                            return true;
                        }
                    }
                }
                if (expr.expression.kind === 194 /* PropertyAccessExpression */ &&
                    containsNarrowableReference(expr.expression.expression)) {
                    return true;
                }
                return false;
            }
            function isNarrowingTypeofOperands(expr1, expr2) {
                return ts.isTypeOfExpression(expr1) && isNarrowableOperand(expr1.expression) && ts.isStringLiteralLike(expr2);
            }
            function isNarrowableInOperands(left, right) {
                return ts.isStringLiteralLike(left) && isNarrowingExpression(right);
            }
            function isNarrowingBinaryExpression(expr) {
                switch (expr.operatorToken.kind) {
                    case 62 /* EqualsToken */:
                        return containsNarrowableReference(expr.left);
                    case 34 /* EqualsEqualsToken */:
                    case 35 /* ExclamationEqualsToken */:
                    case 36 /* EqualsEqualsEqualsToken */:
                    case 37 /* ExclamationEqualsEqualsToken */:
                        return isNarrowableOperand(expr.left) || isNarrowableOperand(expr.right) ||
                            isNarrowingTypeofOperands(expr.right, expr.left) || isNarrowingTypeofOperands(expr.left, expr.right);
                    case 98 /* InstanceOfKeyword */:
                        return isNarrowableOperand(expr.left);
                    case 97 /* InKeyword */:
                        return isNarrowableInOperands(expr.left, expr.right);
                    case 27 /* CommaToken */:
                        return isNarrowingExpression(expr.right);
                }
                return false;
            }
            function isNarrowableOperand(expr) {
                switch (expr.kind) {
                    case 200 /* ParenthesizedExpression */:
                        return isNarrowableOperand(expr.expression);
                    case 209 /* BinaryExpression */:
                        switch (expr.operatorToken.kind) {
                            case 62 /* EqualsToken */:
                                return isNarrowableOperand(expr.left);
                            case 27 /* CommaToken */:
                                return isNarrowableOperand(expr.right);
                        }
                }
                return containsNarrowableReference(expr);
            }
            function createBranchLabel() {
                return initFlowNode({ flags: 4 /* BranchLabel */, antecedents: undefined });
            }
            function createLoopLabel() {
                return initFlowNode({ flags: 8 /* LoopLabel */, antecedents: undefined });
            }
            function setFlowNodeReferenced(flow) {
                // On first reference we set the Referenced flag, thereafter we set the Shared flag
                flow.flags |= flow.flags & 1024 /* Referenced */ ? 2048 /* Shared */ : 1024 /* Referenced */;
            }
            function addAntecedent(label, antecedent) {
                if (!(antecedent.flags & 1 /* Unreachable */) && !ts.contains(label.antecedents, antecedent)) {
                    (label.antecedents || (label.antecedents = [])).push(antecedent);
                    setFlowNodeReferenced(antecedent);
                }
            }
            function createFlowCondition(flags, antecedent, expression) {
                if (antecedent.flags & 1 /* Unreachable */) {
                    return antecedent;
                }
                if (!expression) {
                    return flags & 32 /* TrueCondition */ ? antecedent : unreachableFlow;
                }
                if ((expression.kind === 106 /* TrueKeyword */ && flags & 64 /* FalseCondition */ ||
                    expression.kind === 91 /* FalseKeyword */ && flags & 32 /* TrueCondition */) &&
                    !ts.isExpressionOfOptionalChainRoot(expression) && !ts.isNullishCoalesce(expression.parent)) {
                    return unreachableFlow;
                }
                if (!isNarrowingExpression(expression)) {
                    return antecedent;
                }
                setFlowNodeReferenced(antecedent);
                return initFlowNode({ flags: flags, antecedent: antecedent, node: expression });
            }
            function createFlowSwitchClause(antecedent, switchStatement, clauseStart, clauseEnd) {
                setFlowNodeReferenced(antecedent);
                return initFlowNode({ flags: 128 /* SwitchClause */, antecedent: antecedent, switchStatement: switchStatement, clauseStart: clauseStart, clauseEnd: clauseEnd });
            }
            function createFlowMutation(flags, antecedent, node) {
                setFlowNodeReferenced(antecedent);
                var result = initFlowNode({ flags: flags, antecedent: antecedent, node: node });
                if (currentExceptionTarget) {
                    addAntecedent(currentExceptionTarget, result);
                }
                return result;
            }
            function createFlowCall(antecedent, node) {
                setFlowNodeReferenced(antecedent);
                return initFlowNode({ flags: 512 /* Call */, antecedent: antecedent, node: node });
            }
            function finishFlowLabel(flow) {
                var antecedents = flow.antecedents;
                if (!antecedents) {
                    return unreachableFlow;
                }
                if (antecedents.length === 1) {
                    return antecedents[0];
                }
                return flow;
            }
            function isStatementCondition(node) {
                var parent = node.parent;
                switch (parent.kind) {
                    case 227 /* IfStatement */:
                    case 229 /* WhileStatement */:
                    case 228 /* DoStatement */:
                        return parent.expression === node;
                    case 230 /* ForStatement */:
                    case 210 /* ConditionalExpression */:
                        return parent.condition === node;
                }
                return false;
            }
            function isLogicalExpression(node) {
                while (true) {
                    if (node.kind === 200 /* ParenthesizedExpression */) {
                        node = node.expression;
                    }
                    else if (node.kind === 207 /* PrefixUnaryExpression */ && node.operator === 53 /* ExclamationToken */) {
                        node = node.operand;
                    }
                    else {
                        return node.kind === 209 /* BinaryExpression */ && (node.operatorToken.kind === 55 /* AmpersandAmpersandToken */ ||
                            node.operatorToken.kind === 56 /* BarBarToken */ ||
                            node.operatorToken.kind === 60 /* QuestionQuestionToken */);
                    }
                }
            }
            function isTopLevelLogicalExpression(node) {
                while (ts.isParenthesizedExpression(node.parent) ||
                    ts.isPrefixUnaryExpression(node.parent) && node.parent.operator === 53 /* ExclamationToken */) {
                    node = node.parent;
                }
                return !isStatementCondition(node) &&
                    !isLogicalExpression(node.parent) &&
                    !(ts.isOptionalChain(node.parent) && node.parent.expression === node);
            }
            function doWithConditionalBranches(action, value, trueTarget, falseTarget) {
                var savedTrueTarget = currentTrueTarget;
                var savedFalseTarget = currentFalseTarget;
                currentTrueTarget = trueTarget;
                currentFalseTarget = falseTarget;
                action(value);
                currentTrueTarget = savedTrueTarget;
                currentFalseTarget = savedFalseTarget;
            }
            function bindCondition(node, trueTarget, falseTarget) {
                doWithConditionalBranches(bind, node, trueTarget, falseTarget);
                if (!node || !isLogicalExpression(node) && !(ts.isOptionalChain(node) && ts.isOutermostOptionalChain(node))) {
                    addAntecedent(trueTarget, createFlowCondition(32 /* TrueCondition */, currentFlow, node));
                    addAntecedent(falseTarget, createFlowCondition(64 /* FalseCondition */, currentFlow, node));
                }
            }
            function bindIterativeStatement(node, breakTarget, continueTarget) {
                var saveBreakTarget = currentBreakTarget;
                var saveContinueTarget = currentContinueTarget;
                currentBreakTarget = breakTarget;
                currentContinueTarget = continueTarget;
                bind(node);
                currentBreakTarget = saveBreakTarget;
                currentContinueTarget = saveContinueTarget;
            }
            function setContinueTarget(node, target) {
                var label = activeLabelList;
                while (label && node.parent.kind === 238 /* LabeledStatement */) {
                    label.continueTarget = target;
                    label = label.next;
                    node = node.parent;
                }
                return target;
            }
            function bindWhileStatement(node) {
                var preWhileLabel = setContinueTarget(node, createLoopLabel());
                var preBodyLabel = createBranchLabel();
                var postWhileLabel = createBranchLabel();
                addAntecedent(preWhileLabel, currentFlow);
                currentFlow = preWhileLabel;
                bindCondition(node.expression, preBodyLabel, postWhileLabel);
                currentFlow = finishFlowLabel(preBodyLabel);
                bindIterativeStatement(node.statement, postWhileLabel, preWhileLabel);
                addAntecedent(preWhileLabel, currentFlow);
                currentFlow = finishFlowLabel(postWhileLabel);
            }
            function bindDoStatement(node) {
                var preDoLabel = createLoopLabel();
                var preConditionLabel = setContinueTarget(node, createBranchLabel());
                var postDoLabel = createBranchLabel();
                addAntecedent(preDoLabel, currentFlow);
                currentFlow = preDoLabel;
                bindIterativeStatement(node.statement, postDoLabel, preConditionLabel);
                addAntecedent(preConditionLabel, currentFlow);
                currentFlow = finishFlowLabel(preConditionLabel);
                bindCondition(node.expression, preDoLabel, postDoLabel);
                currentFlow = finishFlowLabel(postDoLabel);
            }
            function bindForStatement(node) {
                var preLoopLabel = setContinueTarget(node, createLoopLabel());
                var preBodyLabel = createBranchLabel();
                var postLoopLabel = createBranchLabel();
                bind(node.initializer);
                addAntecedent(preLoopLabel, currentFlow);
                currentFlow = preLoopLabel;
                bindCondition(node.condition, preBodyLabel, postLoopLabel);
                currentFlow = finishFlowLabel(preBodyLabel);
                bindIterativeStatement(node.statement, postLoopLabel, preLoopLabel);
                bind(node.incrementor);
                addAntecedent(preLoopLabel, currentFlow);
                currentFlow = finishFlowLabel(postLoopLabel);
            }
            function bindForInOrForOfStatement(node) {
                var preLoopLabel = setContinueTarget(node, createLoopLabel());
                var postLoopLabel = createBranchLabel();
                bind(node.expression);
                addAntecedent(preLoopLabel, currentFlow);
                currentFlow = preLoopLabel;
                if (node.kind === 232 /* ForOfStatement */) {
                    bind(node.awaitModifier);
                }
                addAntecedent(postLoopLabel, currentFlow);
                bind(node.initializer);
                if (node.initializer.kind !== 243 /* VariableDeclarationList */) {
                    bindAssignmentTargetFlow(node.initializer);
                }
                bindIterativeStatement(node.statement, postLoopLabel, preLoopLabel);
                addAntecedent(preLoopLabel, currentFlow);
                currentFlow = finishFlowLabel(postLoopLabel);
            }
            function bindIfStatement(node) {
                var thenLabel = createBranchLabel();
                var elseLabel = createBranchLabel();
                var postIfLabel = createBranchLabel();
                bindCondition(node.expression, thenLabel, elseLabel);
                currentFlow = finishFlowLabel(thenLabel);
                bind(node.thenStatement);
                addAntecedent(postIfLabel, currentFlow);
                currentFlow = finishFlowLabel(elseLabel);
                bind(node.elseStatement);
                addAntecedent(postIfLabel, currentFlow);
                currentFlow = finishFlowLabel(postIfLabel);
            }
            function bindReturnOrThrow(node) {
                bind(node.expression);
                if (node.kind === 235 /* ReturnStatement */) {
                    hasExplicitReturn = true;
                    if (currentReturnTarget) {
                        addAntecedent(currentReturnTarget, currentFlow);
                    }
                }
                currentFlow = unreachableFlow;
            }
            function findActiveLabel(name) {
                for (var label = activeLabelList; label; label = label.next) {
                    if (label.name === name) {
                        return label;
                    }
                }
                return undefined;
            }
            function bindBreakOrContinueFlow(node, breakTarget, continueTarget) {
                var flowLabel = node.kind === 234 /* BreakStatement */ ? breakTarget : continueTarget;
                if (flowLabel) {
                    addAntecedent(flowLabel, currentFlow);
                    currentFlow = unreachableFlow;
                }
            }
            function bindBreakOrContinueStatement(node) {
                bind(node.label);
                if (node.label) {
                    var activeLabel = findActiveLabel(node.label.escapedText);
                    if (activeLabel) {
                        activeLabel.referenced = true;
                        bindBreakOrContinueFlow(node, activeLabel.breakTarget, activeLabel.continueTarget);
                    }
                }
                else {
                    bindBreakOrContinueFlow(node, currentBreakTarget, currentContinueTarget);
                }
            }
            function bindTryStatement(node) {
                var preFinallyLabel = createBranchLabel();
                // We conservatively assume that *any* code in the try block can cause an exception, but we only need
                // to track code that causes mutations (because only mutations widen the possible control flow type of
                // a variable). The currentExceptionTarget is the target label for control flows that result from
                // exceptions. We add all mutation flow nodes as antecedents of this label such that we can analyze them
                // as possible antecedents of the start of catch or finally blocks. Furthermore, we add the current
                // control flow to represent exceptions that occur before any mutations.
                var saveReturnTarget = currentReturnTarget;
                var saveExceptionTarget = currentExceptionTarget;
                currentReturnTarget = createBranchLabel();
                currentExceptionTarget = node.catchClause ? createBranchLabel() : currentReturnTarget;
                addAntecedent(currentExceptionTarget, currentFlow);
                bind(node.tryBlock);
                addAntecedent(preFinallyLabel, currentFlow);
                var flowAfterTry = currentFlow;
                var flowAfterCatch = unreachableFlow;
                if (node.catchClause) {
                    // Start of catch clause is the target of exceptions from try block.
                    currentFlow = finishFlowLabel(currentExceptionTarget);
                    // The currentExceptionTarget now represents control flows from exceptions in the catch clause.
                    // Effectively, in a try-catch-finally, if an exception occurs in the try block, the catch block
                    // acts like a second try block.
                    currentExceptionTarget = currentReturnTarget;
                    addAntecedent(currentExceptionTarget, currentFlow);
                    bind(node.catchClause);
                    addAntecedent(preFinallyLabel, currentFlow);
                    flowAfterCatch = currentFlow;
                }
                var exceptionTarget = finishFlowLabel(currentExceptionTarget);
                currentReturnTarget = saveReturnTarget;
                currentExceptionTarget = saveExceptionTarget;
                if (node.finallyBlock) {
                    // Possible ways control can reach the finally block:
                    // 1) Normal completion of try block of a try-finally or try-catch-finally
                    // 2) Normal completion of catch block (following exception in try block) of a try-catch-finally
                    // 3) Return in try or catch block of a try-finally or try-catch-finally
                    // 4) Exception in try block of a try-finally
                    // 5) Exception in catch block of a try-catch-finally
                    // When analyzing a control flow graph that starts inside a finally block we want to consider all
                    // five possibilities above. However, when analyzing a control flow graph that starts outside (past)
                    // the finally block, we only want to consider the first two (if we're past a finally block then it
                    // must have completed normally). To make this possible, we inject two extra nodes into the control
                    // flow graph: An after-finally with an antecedent of the control flow at the end of the finally
                    // block, and a pre-finally with an antecedent that represents all exceptional control flows. The
                    // 'lock' property of the pre-finally references the after-finally, and the after-finally has a
                    // boolean 'locked' property that we set to true when analyzing a control flow that contained the
                    // the after-finally node. When the lock associated with a pre-finally is locked, the antecedent of
                    // the pre-finally (i.e. the exceptional control flows) are skipped.
                    var preFinallyFlow = initFlowNode({ flags: 4096 /* PreFinally */, antecedent: exceptionTarget, lock: {} });
                    addAntecedent(preFinallyLabel, preFinallyFlow);
                    currentFlow = finishFlowLabel(preFinallyLabel);
                    bind(node.finallyBlock);
                    // If the end of the finally block is reachable, but the end of the try and catch blocks are not,
                    // convert the current flow to unreachable. For example, 'try { return 1; } finally { ... }' should
                    // result in an unreachable current control flow.
                    if (!(currentFlow.flags & 1 /* Unreachable */)) {
                        if ((flowAfterTry.flags & 1 /* Unreachable */) && (flowAfterCatch.flags & 1 /* Unreachable */)) {
                            currentFlow = flowAfterTry === reportedUnreachableFlow || flowAfterCatch === reportedUnreachableFlow
                                ? reportedUnreachableFlow
                                : unreachableFlow;
                        }
                    }
                    if (!(currentFlow.flags & 1 /* Unreachable */)) {
                        var afterFinallyFlow = initFlowNode({ flags: 8192 /* AfterFinally */, antecedent: currentFlow });
                        preFinallyFlow.lock = afterFinallyFlow;
                        currentFlow = afterFinallyFlow;
                    }
                }
                else {
                    currentFlow = finishFlowLabel(preFinallyLabel);
                }
            }
            function bindSwitchStatement(node) {
                var postSwitchLabel = createBranchLabel();
                bind(node.expression);
                var saveBreakTarget = currentBreakTarget;
                var savePreSwitchCaseFlow = preSwitchCaseFlow;
                currentBreakTarget = postSwitchLabel;
                preSwitchCaseFlow = currentFlow;
                bind(node.caseBlock);
                addAntecedent(postSwitchLabel, currentFlow);
                var hasDefault = ts.forEach(node.caseBlock.clauses, function (c) { return c.kind === 278 /* DefaultClause */; });
                // We mark a switch statement as possibly exhaustive if it has no default clause and if all
                // case clauses have unreachable end points (e.g. they all return). Note, we no longer need
                // this property in control flow analysis, it's there only for backwards compatibility.
                node.possiblyExhaustive = !hasDefault && !postSwitchLabel.antecedents;
                if (!hasDefault) {
                    addAntecedent(postSwitchLabel, createFlowSwitchClause(preSwitchCaseFlow, node, 0, 0));
                }
                currentBreakTarget = saveBreakTarget;
                preSwitchCaseFlow = savePreSwitchCaseFlow;
                currentFlow = finishFlowLabel(postSwitchLabel);
            }
            function bindCaseBlock(node) {
                var savedSubtreeTransformFlags = subtreeTransformFlags;
                subtreeTransformFlags = 0;
                var clauses = node.clauses;
                var isNarrowingSwitch = isNarrowingExpression(node.parent.expression);
                var fallthroughFlow = unreachableFlow;
                for (var i = 0; i < clauses.length; i++) {
                    var clauseStart = i;
                    while (!clauses[i].statements.length && i + 1 < clauses.length) {
                        bind(clauses[i]);
                        i++;
                    }
                    var preCaseLabel = createBranchLabel();
                    addAntecedent(preCaseLabel, isNarrowingSwitch ? createFlowSwitchClause(preSwitchCaseFlow, node.parent, clauseStart, i + 1) : preSwitchCaseFlow);
                    addAntecedent(preCaseLabel, fallthroughFlow);
                    currentFlow = finishFlowLabel(preCaseLabel);
                    var clause = clauses[i];
                    bind(clause);
                    fallthroughFlow = currentFlow;
                    if (!(currentFlow.flags & 1 /* Unreachable */) && i !== clauses.length - 1 && options.noFallthroughCasesInSwitch) {
                        clause.fallthroughFlowNode = currentFlow;
                    }
                }
                clauses.transformFlags = subtreeTransformFlags | 536870912 /* HasComputedFlags */;
                subtreeTransformFlags |= savedSubtreeTransformFlags;
            }
            function bindCaseClause(node) {
                var saveCurrentFlow = currentFlow;
                currentFlow = preSwitchCaseFlow;
                bind(node.expression);
                currentFlow = saveCurrentFlow;
                bindEach(node.statements);
            }
            function bindExpressionStatement(node) {
                bind(node.expression);
                // A top level call expression with a dotted function name and at least one argument
                // is potentially an assertion and is therefore included in the control flow.
                if (node.expression.kind === 196 /* CallExpression */) {
                    var call = node.expression;
                    if (ts.isDottedName(call.expression)) {
                        currentFlow = createFlowCall(currentFlow, call);
                    }
                }
            }
            function bindLabeledStatement(node) {
                var postStatementLabel = createBranchLabel();
                activeLabelList = {
                    next: activeLabelList,
                    name: node.label.escapedText,
                    breakTarget: postStatementLabel,
                    continueTarget: undefined,
                    referenced: false
                };
                bind(node.label);
                bind(node.statement);
                if (!activeLabelList.referenced && !options.allowUnusedLabels) {
                    errorOrSuggestionOnNode(ts.unusedLabelIsError(options), node.label, ts.Diagnostics.Unused_label);
                }
                activeLabelList = activeLabelList.next;
                addAntecedent(postStatementLabel, currentFlow);
                currentFlow = finishFlowLabel(postStatementLabel);
            }
            function bindDestructuringTargetFlow(node) {
                if (node.kind === 209 /* BinaryExpression */ && node.operatorToken.kind === 62 /* EqualsToken */) {
                    bindAssignmentTargetFlow(node.left);
                }
                else {
                    bindAssignmentTargetFlow(node);
                }
            }
            function bindAssignmentTargetFlow(node) {
                if (isNarrowableReference(node)) {
                    currentFlow = createFlowMutation(16 /* Assignment */, currentFlow, node);
                }
                else if (node.kind === 192 /* ArrayLiteralExpression */) {
                    for (var _i = 0, _a = node.elements; _i < _a.length; _i++) {
                        var e = _a[_i];
                        if (e.kind === 213 /* SpreadElement */) {
                            bindAssignmentTargetFlow(e.expression);
                        }
                        else {
                            bindDestructuringTargetFlow(e);
                        }
                    }
                }
                else if (node.kind === 193 /* ObjectLiteralExpression */) {
                    for (var _b = 0, _c = node.properties; _b < _c.length; _b++) {
                        var p = _c[_b];
                        if (p.kind === 281 /* PropertyAssignment */) {
                            bindDestructuringTargetFlow(p.initializer);
                        }
                        else if (p.kind === 282 /* ShorthandPropertyAssignment */) {
                            bindAssignmentTargetFlow(p.name);
                        }
                        else if (p.kind === 283 /* SpreadAssignment */) {
                            bindAssignmentTargetFlow(p.expression);
                        }
                    }
                }
            }
            function bindLogicalExpression(node, trueTarget, falseTarget) {
                var preRightLabel = createBranchLabel();
                if (node.operatorToken.kind === 55 /* AmpersandAmpersandToken */) {
                    bindCondition(node.left, preRightLabel, falseTarget);
                }
                else {
                    bindCondition(node.left, trueTarget, preRightLabel);
                }
                currentFlow = finishFlowLabel(preRightLabel);
                bind(node.operatorToken);
                bindCondition(node.right, trueTarget, falseTarget);
            }
            function bindPrefixUnaryExpressionFlow(node) {
                if (node.operator === 53 /* ExclamationToken */) {
                    var saveTrueTarget = currentTrueTarget;
                    currentTrueTarget = currentFalseTarget;
                    currentFalseTarget = saveTrueTarget;
                    bindEachChild(node);
                    currentFalseTarget = currentTrueTarget;
                    currentTrueTarget = saveTrueTarget;
                }
                else {
                    bindEachChild(node);
                    if (node.operator === 45 /* PlusPlusToken */ || node.operator === 46 /* MinusMinusToken */) {
                        bindAssignmentTargetFlow(node.operand);
                    }
                }
            }
            function bindPostfixUnaryExpressionFlow(node) {
                bindEachChild(node);
                if (node.operator === 45 /* PlusPlusToken */ || node.operator === 46 /* MinusMinusToken */) {
                    bindAssignmentTargetFlow(node.operand);
                }
            }
            var BindBinaryExpressionFlowState;
            (function (BindBinaryExpressionFlowState) {
                BindBinaryExpressionFlowState[BindBinaryExpressionFlowState["BindThenBindChildren"] = 0] = "BindThenBindChildren";
                BindBinaryExpressionFlowState[BindBinaryExpressionFlowState["MaybeBindLeft"] = 1] = "MaybeBindLeft";
                BindBinaryExpressionFlowState[BindBinaryExpressionFlowState["BindToken"] = 2] = "BindToken";
                BindBinaryExpressionFlowState[BindBinaryExpressionFlowState["BindRight"] = 3] = "BindRight";
                BindBinaryExpressionFlowState[BindBinaryExpressionFlowState["FinishBind"] = 4] = "FinishBind";
            })(BindBinaryExpressionFlowState || (BindBinaryExpressionFlowState = {}));
            function bindBinaryExpressionFlow(node) {
                var workStacks = {
                    expr: [node],
                    state: [1 /* MaybeBindLeft */],
                    inStrictMode: [undefined],
                    parent: [undefined],
                    subtreeFlags: [undefined]
                };
                var stackIndex = 0;
                while (stackIndex >= 0) {
                    node = workStacks.expr[stackIndex];
                    switch (workStacks.state[stackIndex]) {
                        case 0 /* BindThenBindChildren */: {
                            // This state is used only when recuring, to emulate the work that `bind` does before
                            // reaching `bindChildren`. A normal call to `bindBinaryExpressionFlow` will already have done this work.
                            node.parent = parent;
                            var saveInStrictMode = inStrictMode;
                            bindWorker(node);
                            var saveParent = parent;
                            parent = node;
                            var subtreeFlagsState = void 0;
                            // While this next part does the work of `bindChildren` before it descends into `bindChildrenWorker`
                            // and uses `subtreeFlagsState` to queue up the work that needs to be done once the node is bound.
                            if (skipTransformFlagAggregation) {
                                // do nothing extra
                            }
                            else if (node.transformFlags & 536870912 /* HasComputedFlags */) {
                                skipTransformFlagAggregation = true;
                                subtreeFlagsState = -1;
                            }
                            else {
                                var savedSubtreeTransformFlags = subtreeTransformFlags;
                                subtreeTransformFlags = 0;
                                subtreeFlagsState = savedSubtreeTransformFlags;
                            }
                            advanceState(1 /* MaybeBindLeft */, saveInStrictMode, saveParent, subtreeFlagsState);
                            break;
                        }
                        case 1 /* MaybeBindLeft */: {
                            var operator = node.operatorToken.kind;
                            // TODO: bindLogicalExpression is recursive - if we want to handle deeply nested `&&` expressions
                            // we'll need to handle the `bindLogicalExpression` scenarios in this state machine, too
                            // For now, though, since the common cases are chained `+`, leaving it recursive is fine
                            if (operator === 55 /* AmpersandAmpersandToken */ || operator === 56 /* BarBarToken */ || operator === 60 /* QuestionQuestionToken */) {
                                if (isTopLevelLogicalExpression(node)) {
                                    var postExpressionLabel = createBranchLabel();
                                    bindLogicalExpression(node, postExpressionLabel, postExpressionLabel);
                                    currentFlow = finishFlowLabel(postExpressionLabel);
                                }
                                else {
                                    bindLogicalExpression(node, currentTrueTarget, currentFalseTarget);
                                }
                                completeNode();
                            }
                            else {
                                advanceState(2 /* BindToken */);
                                maybeBind(node.left);
                            }
                            break;
                        }
                        case 2 /* BindToken */: {
                            advanceState(3 /* BindRight */);
                            maybeBind(node.operatorToken);
                            break;
                        }
                        case 3 /* BindRight */: {
                            advanceState(4 /* FinishBind */);
                            maybeBind(node.right);
                            break;
                        }
                        case 4 /* FinishBind */: {
                            var operator = node.operatorToken.kind;
                            if (ts.isAssignmentOperator(operator) && !ts.isAssignmentTarget(node)) {
                                bindAssignmentTargetFlow(node.left);
                                if (operator === 62 /* EqualsToken */ && node.left.kind === 195 /* ElementAccessExpression */) {
                                    var elementAccess = node.left;
                                    if (isNarrowableOperand(elementAccess.expression)) {
                                        currentFlow = createFlowMutation(256 /* ArrayMutation */, currentFlow, node);
                                    }
                                }
                            }
                            completeNode();
                            break;
                        }
                        default: return ts.Debug.fail("Invalid state " + workStacks.state[stackIndex] + " for bindBinaryExpressionFlow");
                    }
                }
                /**
                 * Note that `advanceState` sets the _current_ head state, and that `maybeBind` potentially pushes on a new
                 * head state; so `advanceState` must be called before any `maybeBind` during a state's execution.
                 */
                function advanceState(state, isInStrictMode, parent, subtreeFlags) {
                    workStacks.state[stackIndex] = state;
                    if (isInStrictMode !== undefined) {
                        workStacks.inStrictMode[stackIndex] = isInStrictMode;
                    }
                    if (parent !== undefined) {
                        workStacks.parent[stackIndex] = parent;
                    }
                    if (subtreeFlags !== undefined) {
                        workStacks.subtreeFlags[stackIndex] = subtreeFlags;
                    }
                }
                function completeNode() {
                    if (workStacks.inStrictMode[stackIndex] !== undefined) {
                        if (workStacks.subtreeFlags[stackIndex] === -1) {
                            skipTransformFlagAggregation = false;
                            subtreeTransformFlags |= node.transformFlags & ~getTransformFlagsSubtreeExclusions(node.kind);
                        }
                        else if (workStacks.subtreeFlags[stackIndex] !== undefined) {
                            subtreeTransformFlags = workStacks.subtreeFlags[stackIndex] | computeTransformFlagsForNode(node, subtreeTransformFlags);
                        }
                        inStrictMode = workStacks.inStrictMode[stackIndex];
                        parent = workStacks.parent[stackIndex];
                    }
                    stackIndex--;
                }
                /**
                 * If `node` is a BinaryExpression, adds it to the local work stack, otherwise recursively binds it
                 */
                function maybeBind(node) {
                    if (node && ts.isBinaryExpression(node)) {
                        stackIndex++;
                        workStacks.expr[stackIndex] = node;
                        workStacks.state[stackIndex] = 0 /* BindThenBindChildren */;
                        workStacks.inStrictMode[stackIndex] = undefined;
                        workStacks.parent[stackIndex] = undefined;
                        workStacks.subtreeFlags[stackIndex] = undefined;
                    }
                    else {
                        bind(node);
                    }
                }
            }
            function bindDeleteExpressionFlow(node) {
                bindEachChild(node);
                if (node.expression.kind === 194 /* PropertyAccessExpression */) {
                    bindAssignmentTargetFlow(node.expression);
                }
            }
            function bindConditionalExpressionFlow(node) {
                var trueLabel = createBranchLabel();
                var falseLabel = createBranchLabel();
                var postExpressionLabel = createBranchLabel();
                bindCondition(node.condition, trueLabel, falseLabel);
                currentFlow = finishFlowLabel(trueLabel);
                bind(node.questionToken);
                bind(node.whenTrue);
                addAntecedent(postExpressionLabel, currentFlow);
                currentFlow = finishFlowLabel(falseLabel);
                bind(node.colonToken);
                bind(node.whenFalse);
                addAntecedent(postExpressionLabel, currentFlow);
                currentFlow = finishFlowLabel(postExpressionLabel);
            }
            function bindInitializedVariableFlow(node) {
                var name = !ts.isOmittedExpression(node) ? node.name : undefined;
                if (ts.isBindingPattern(name)) {
                    for (var _i = 0, _a = name.elements; _i < _a.length; _i++) {
                        var child = _a[_i];
                        bindInitializedVariableFlow(child);
                    }
                }
                else {
                    currentFlow = createFlowMutation(16 /* Assignment */, currentFlow, node);
                }
            }
            function bindVariableDeclarationFlow(node) {
                bindEachChild(node);
                if (node.initializer || ts.isForInOrOfStatement(node.parent.parent)) {
                    bindInitializedVariableFlow(node);
                }
            }
            function bindJSDocTypeAlias(node) {
                node.tagName.parent = node;
                if (node.kind !== 315 /* JSDocEnumTag */ && node.fullName) {
                    setParentPointers(node, node.fullName);
                }
            }
            function bindJSDocClassTag(node) {
                bindEachChild(node);
                var host = ts.getHostSignatureFromJSDoc(node);
                if (host && host.kind !== 161 /* MethodDeclaration */) {
                    addDeclarationToSymbol(host.symbol, host, 32 /* Class */);
                }
            }
            function bindOptionalExpression(node, trueTarget, falseTarget) {
                doWithConditionalBranches(bind, node, trueTarget, falseTarget);
                if (!ts.isOptionalChain(node) || ts.isOutermostOptionalChain(node)) {
                    addAntecedent(trueTarget, createFlowCondition(32 /* TrueCondition */, currentFlow, node));
                    addAntecedent(falseTarget, createFlowCondition(64 /* FalseCondition */, currentFlow, node));
                }
            }
            function bindOptionalChainRest(node) {
                bind(node.questionDotToken);
                switch (node.kind) {
                    case 194 /* PropertyAccessExpression */:
                        bind(node.name);
                        break;
                    case 195 /* ElementAccessExpression */:
                        bind(node.argumentExpression);
                        break;
                    case 196 /* CallExpression */:
                        bindEach(node.typeArguments);
                        bindEach(node.arguments);
                        break;
                }
            }
            function bindOptionalChain(node, trueTarget, falseTarget) {
                // For an optional chain, we emulate the behavior of a logical expression:
                //
                // a?.b         -> a && a.b
                // a?.b.c       -> a && a.b.c
                // a?.b?.c      -> a && a.b && a.b.c
                // a?.[x = 1]   -> a && a[x = 1]
                //
                // To do this we descend through the chain until we reach the root of a chain (the expression with a `?.`)
                // and build it's CFA graph as if it were the first condition (`a && ...`). Then we bind the rest
                // of the node as part of the "true" branch, and continue to do so as we ascend back up to the outermost
                // chain node. We then treat the entire node as the right side of the expression.
                var preChainLabel = node.questionDotToken ? createBranchLabel() : undefined;
                bindOptionalExpression(node.expression, preChainLabel || trueTarget, falseTarget);
                if (preChainLabel) {
                    currentFlow = finishFlowLabel(preChainLabel);
                }
                doWithConditionalBranches(bindOptionalChainRest, node, trueTarget, falseTarget);
                if (ts.isOutermostOptionalChain(node)) {
                    addAntecedent(trueTarget, createFlowCondition(32 /* TrueCondition */, currentFlow, node));
                    addAntecedent(falseTarget, createFlowCondition(64 /* FalseCondition */, currentFlow, node));
                }
            }
            function bindOptionalChainFlow(node) {
                if (isTopLevelLogicalExpression(node)) {
                    var postExpressionLabel = createBranchLabel();
                    bindOptionalChain(node, postExpressionLabel, postExpressionLabel);
                    currentFlow = finishFlowLabel(postExpressionLabel);
                }
                else {
                    bindOptionalChain(node, currentTrueTarget, currentFalseTarget);
                }
            }
            function bindAccessExpressionFlow(node) {
                if (ts.isOptionalChain(node)) {
                    bindOptionalChainFlow(node);
                }
                else {
                    bindEachChild(node);
                }
            }
            function bindCallExpressionFlow(node) {
                if (ts.isOptionalChain(node)) {
                    bindOptionalChainFlow(node);
                }
                else {
                    // If the target of the call expression is a function expression or arrow function we have
                    // an immediately invoked function expression (IIFE). Initialize the flowNode property to
                    // the current control flow (which includes evaluation of the IIFE arguments).
                    var expr = ts.skipParentheses(node.expression);
                    if (expr.kind === 201 /* FunctionExpression */ || expr.kind === 202 /* ArrowFunction */) {
                        bindEach(node.typeArguments);
                        bindEach(node.arguments);
                        bind(node.expression);
                    }
                    else {
                        bindEachChild(node);
                    }
                }
                if (node.expression.kind === 194 /* PropertyAccessExpression */) {
                    var propertyAccess = node.expression;
                    if (ts.isIdentifier(propertyAccess.name) && isNarrowableOperand(propertyAccess.expression) && ts.isPushOrUnshiftIdentifier(propertyAccess.name)) {
                        currentFlow = createFlowMutation(256 /* ArrayMutation */, currentFlow, node);
                    }
                }
            }
            function getContainerFlags(node) {
                switch (node.kind) {
                    case 214 /* ClassExpression */:
                    case 245 /* ClassDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 193 /* ObjectLiteralExpression */:
                    case 173 /* TypeLiteral */:
                    case 304 /* JSDocTypeLiteral */:
                    case 274 /* JsxAttributes */:
                        return 1 /* IsContainer */;
                    case 246 /* InterfaceDeclaration */:
                        return 1 /* IsContainer */ | 64 /* IsInterface */;
                    case 249 /* ModuleDeclaration */:
                    case 247 /* TypeAliasDeclaration */:
                    case 186 /* MappedType */:
                        return 1 /* IsContainer */ | 32 /* HasLocals */;
                    case 290 /* SourceFile */:
                        return 1 /* IsContainer */ | 4 /* IsControlFlowContainer */ | 32 /* HasLocals */;
                    case 161 /* MethodDeclaration */:
                        if (ts.isObjectLiteralOrClassExpressionMethod(node)) {
                            return 1 /* IsContainer */ | 4 /* IsControlFlowContainer */ | 32 /* HasLocals */ | 8 /* IsFunctionLike */ | 128 /* IsObjectLiteralOrClassExpressionMethod */;
                        }
                    // falls through
                    case 162 /* Constructor */:
                    case 244 /* FunctionDeclaration */:
                    case 160 /* MethodSignature */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                    case 165 /* CallSignature */:
                    case 305 /* JSDocSignature */:
                    case 300 /* JSDocFunctionType */:
                    case 170 /* FunctionType */:
                    case 166 /* ConstructSignature */:
                    case 167 /* IndexSignature */:
                    case 171 /* ConstructorType */:
                        return 1 /* IsContainer */ | 4 /* IsControlFlowContainer */ | 32 /* HasLocals */ | 8 /* IsFunctionLike */;
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                        return 1 /* IsContainer */ | 4 /* IsControlFlowContainer */ | 32 /* HasLocals */ | 8 /* IsFunctionLike */ | 16 /* IsFunctionExpression */;
                    case 250 /* ModuleBlock */:
                        return 4 /* IsControlFlowContainer */;
                    case 159 /* PropertyDeclaration */:
                        return node.initializer ? 4 /* IsControlFlowContainer */ : 0;
                    case 280 /* CatchClause */:
                    case 230 /* ForStatement */:
                    case 231 /* ForInStatement */:
                    case 232 /* ForOfStatement */:
                    case 251 /* CaseBlock */:
                        return 2 /* IsBlockScopedContainer */;
                    case 223 /* Block */:
                        // do not treat blocks directly inside a function as a block-scoped-container.
                        // Locals that reside in this block should go to the function locals. Otherwise 'x'
                        // would not appear to be a redeclaration of a block scoped local in the following
                        // example:
                        //
                        //      function foo() {
                        //          var x;
                        //          let x;
                        //      }
                        //
                        // If we placed 'var x' into the function locals and 'let x' into the locals of
                        // the block, then there would be no collision.
                        //
                        // By not creating a new block-scoped-container here, we ensure that both 'var x'
                        // and 'let x' go into the Function-container's locals, and we do get a collision
                        // conflict.
                        return ts.isFunctionLike(node.parent) ? 0 /* None */ : 2 /* IsBlockScopedContainer */;
                }
                return 0 /* None */;
            }
            function addToContainerChain(next) {
                if (lastContainer) {
                    lastContainer.nextContainer = next;
                }
                lastContainer = next;
            }
            function declareSymbolAndAddToSymbolTable(node, symbolFlags, symbolExcludes) {
                switch (container.kind) {
                    // Modules, source files, and classes need specialized handling for how their
                    // members are declared (for example, a member of a class will go into a specific
                    // symbol table depending on if it is static or not). We defer to specialized
                    // handlers to take care of declaring these child members.
                    case 249 /* ModuleDeclaration */:
                        return declareModuleMember(node, symbolFlags, symbolExcludes);
                    case 290 /* SourceFile */:
                        return declareSourceFileMember(node, symbolFlags, symbolExcludes);
                    case 214 /* ClassExpression */:
                    case 245 /* ClassDeclaration */:
                        return declareClassMember(node, symbolFlags, symbolExcludes);
                    case 248 /* EnumDeclaration */:
                        return declareSymbol(container.symbol.exports, container.symbol, node, symbolFlags, symbolExcludes);
                    case 173 /* TypeLiteral */:
                    case 304 /* JSDocTypeLiteral */:
                    case 193 /* ObjectLiteralExpression */:
                    case 246 /* InterfaceDeclaration */:
                    case 274 /* JsxAttributes */:
                        // Interface/Object-types always have their children added to the 'members' of
                        // their container. They are only accessible through an instance of their
                        // container, and are never in scope otherwise (even inside the body of the
                        // object / type / interface declaring them). An exception is type parameters,
                        // which are in scope without qualification (similar to 'locals').
                        return declareSymbol(container.symbol.members, container.symbol, node, symbolFlags, symbolExcludes);
                    case 170 /* FunctionType */:
                    case 171 /* ConstructorType */:
                    case 165 /* CallSignature */:
                    case 166 /* ConstructSignature */:
                    case 305 /* JSDocSignature */:
                    case 167 /* IndexSignature */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                    case 162 /* Constructor */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                    case 300 /* JSDocFunctionType */:
                    case 321 /* JSDocTypedefTag */:
                    case 314 /* JSDocCallbackTag */:
                    case 247 /* TypeAliasDeclaration */:
                    case 186 /* MappedType */:
                        // All the children of these container types are never visible through another
                        // symbol (i.e. through another symbol's 'exports' or 'members').  Instead,
                        // they're only accessed 'lexically' (i.e. from code that exists underneath
                        // their container in the tree). To accomplish this, we simply add their declared
                        // symbol to the 'locals' of the container.  These symbols can then be found as
                        // the type checker walks up the containers, checking them for matching names.
                        return declareSymbol(container.locals, /*parent*/ undefined, node, symbolFlags, symbolExcludes);
                }
            }
            function declareClassMember(node, symbolFlags, symbolExcludes) {
                return ts.hasModifier(node, 32 /* Static */)
                    ? declareSymbol(container.symbol.exports, container.symbol, node, symbolFlags, symbolExcludes)
                    : declareSymbol(container.symbol.members, container.symbol, node, symbolFlags, symbolExcludes);
            }
            function declareSourceFileMember(node, symbolFlags, symbolExcludes) {
                return ts.isExternalModule(file)
                    ? declareModuleMember(node, symbolFlags, symbolExcludes)
                    : declareSymbol(file.locals, /*parent*/ undefined, node, symbolFlags, symbolExcludes);
            }
            function hasExportDeclarations(node) {
                var body = ts.isSourceFile(node) ? node : ts.tryCast(node.body, ts.isModuleBlock);
                return !!body && body.statements.some(function (s) { return ts.isExportDeclaration(s) || ts.isExportAssignment(s); });
            }
            function setExportContextFlag(node) {
                // A declaration source file or ambient module declaration that contains no export declarations (but possibly regular
                // declarations with export modifiers) is an export context in which declarations are implicitly exported.
                if (node.flags & 8388608 /* Ambient */ && !hasExportDeclarations(node)) {
                    node.flags |= 64 /* ExportContext */;
                }
                else {
                    node.flags &= ~64 /* ExportContext */;
                }
            }
            function bindModuleDeclaration(node) {
                setExportContextFlag(node);
                if (ts.isAmbientModule(node)) {
                    if (ts.hasModifier(node, 1 /* Export */)) {
                        errorOnFirstToken(node, ts.Diagnostics.export_modifier_cannot_be_applied_to_ambient_modules_and_module_augmentations_since_they_are_always_visible);
                    }
                    if (ts.isModuleAugmentationExternal(node)) {
                        declareModuleSymbol(node);
                    }
                    else {
                        var pattern = void 0;
                        if (node.name.kind === 10 /* StringLiteral */) {
                            var text = node.name.text;
                            if (ts.hasZeroOrOneAsteriskCharacter(text)) {
                                pattern = ts.tryParsePattern(text);
                            }
                            else {
                                errorOnFirstToken(node.name, ts.Diagnostics.Pattern_0_can_have_at_most_one_Asterisk_character, text);
                            }
                        }
                        var symbol = declareSymbolAndAddToSymbolTable(node, 512 /* ValueModule */, 110735 /* ValueModuleExcludes */);
                        file.patternAmbientModules = ts.append(file.patternAmbientModules, pattern && { pattern: pattern, symbol: symbol });
                    }
                }
                else {
                    var state = declareModuleSymbol(node);
                    if (state !== 0 /* NonInstantiated */) {
                        var symbol = node.symbol;
                        // if module was already merged with some function, class or non-const enum, treat it as non-const-enum-only
                        symbol.constEnumOnlyModule = (!(symbol.flags & (16 /* Function */ | 32 /* Class */ | 256 /* RegularEnum */)))
                            // Current must be `const enum` only
                            && state === 2 /* ConstEnumOnly */
                            // Can't have been set to 'false' in a previous merged symbol. ('undefined' OK)
                            && symbol.constEnumOnlyModule !== false;
                    }
                }
            }
            function declareModuleSymbol(node) {
                var state = getModuleInstanceState(node);
                var instantiated = state !== 0 /* NonInstantiated */;
                declareSymbolAndAddToSymbolTable(node, instantiated ? 512 /* ValueModule */ : 1024 /* NamespaceModule */, instantiated ? 110735 /* ValueModuleExcludes */ : 0 /* NamespaceModuleExcludes */);
                return state;
            }
            function bindFunctionOrConstructorType(node) {
                // For a given function symbol "<...>(...) => T" we want to generate a symbol identical
                // to the one we would get for: { <...>(...): T }
                //
                // We do that by making an anonymous type literal symbol, and then setting the function
                // symbol as its sole member. To the rest of the system, this symbol will be indistinguishable
                // from an actual type literal symbol you would have gotten had you used the long form.
                var symbol = createSymbol(131072 /* Signature */, getDeclarationName(node)); // TODO: GH#18217
                addDeclarationToSymbol(symbol, node, 131072 /* Signature */);
                var typeLiteralSymbol = createSymbol(2048 /* TypeLiteral */, "__type" /* Type */);
                addDeclarationToSymbol(typeLiteralSymbol, node, 2048 /* TypeLiteral */);
                typeLiteralSymbol.members = ts.createSymbolTable();
                typeLiteralSymbol.members.set(symbol.escapedName, symbol);
            }
            function bindObjectLiteralExpression(node) {
                var ElementKind;
                (function (ElementKind) {
                    ElementKind[ElementKind["Property"] = 1] = "Property";
                    ElementKind[ElementKind["Accessor"] = 2] = "Accessor";
                })(ElementKind || (ElementKind = {}));
                if (inStrictMode && !ts.isAssignmentTarget(node)) {
                    var seen = ts.createUnderscoreEscapedMap();
                    for (var _i = 0, _a = node.properties; _i < _a.length; _i++) {
                        var prop = _a[_i];
                        if (prop.kind === 283 /* SpreadAssignment */ || prop.name.kind !== 75 /* Identifier */) {
                            continue;
                        }
                        var identifier = prop.name;
                        // ECMA-262 11.1.5 Object Initializer
                        // If previous is not undefined then throw a SyntaxError exception if any of the following conditions are true
                        // a.This production is contained in strict code and IsDataDescriptor(previous) is true and
                        // IsDataDescriptor(propId.descriptor) is true.
                        //    b.IsDataDescriptor(previous) is true and IsAccessorDescriptor(propId.descriptor) is true.
                        //    c.IsAccessorDescriptor(previous) is true and IsDataDescriptor(propId.descriptor) is true.
                        //    d.IsAccessorDescriptor(previous) is true and IsAccessorDescriptor(propId.descriptor) is true
                        // and either both previous and propId.descriptor have[[Get]] fields or both previous and propId.descriptor have[[Set]] fields
                        var currentKind = prop.kind === 281 /* PropertyAssignment */ || prop.kind === 282 /* ShorthandPropertyAssignment */ || prop.kind === 161 /* MethodDeclaration */
                            ? 1 /* Property */
                            : 2 /* Accessor */;
                        var existingKind = seen.get(identifier.escapedText);
                        if (!existingKind) {
                            seen.set(identifier.escapedText, currentKind);
                            continue;
                        }
                        if (currentKind === 1 /* Property */ && existingKind === 1 /* Property */) {
                            var span = ts.getErrorSpanForNode(file, identifier);
                            file.bindDiagnostics.push(ts.createFileDiagnostic(file, span.start, span.length, ts.Diagnostics.An_object_literal_cannot_have_multiple_properties_with_the_same_name_in_strict_mode));
                        }
                    }
                }
                return bindAnonymousDeclaration(node, 4096 /* ObjectLiteral */, "__object" /* Object */);
            }
            function bindJsxAttributes(node) {
                return bindAnonymousDeclaration(node, 4096 /* ObjectLiteral */, "__jsxAttributes" /* JSXAttributes */);
            }
            function bindJsxAttribute(node, symbolFlags, symbolExcludes) {
                return declareSymbolAndAddToSymbolTable(node, symbolFlags, symbolExcludes);
            }
            function bindAnonymousDeclaration(node, symbolFlags, name) {
                var symbol = createSymbol(symbolFlags, name);
                if (symbolFlags & (8 /* EnumMember */ | 106500 /* ClassMember */)) {
                    symbol.parent = container.symbol;
                }
                addDeclarationToSymbol(symbol, node, symbolFlags);
                return symbol;
            }
            function bindBlockScopedDeclaration(node, symbolFlags, symbolExcludes) {
                switch (blockScopeContainer.kind) {
                    case 249 /* ModuleDeclaration */:
                        declareModuleMember(node, symbolFlags, symbolExcludes);
                        break;
                    case 290 /* SourceFile */:
                        if (ts.isExternalOrCommonJsModule(container)) {
                            declareModuleMember(node, symbolFlags, symbolExcludes);
                            break;
                        }
                    // falls through
                    default:
                        if (!blockScopeContainer.locals) {
                            blockScopeContainer.locals = ts.createSymbolTable();
                            addToContainerChain(blockScopeContainer);
                        }
                        declareSymbol(blockScopeContainer.locals, /*parent*/ undefined, node, symbolFlags, symbolExcludes);
                }
            }
            function delayedBindJSDocTypedefTag() {
                if (!delayedTypeAliases) {
                    return;
                }
                var saveContainer = container;
                var saveLastContainer = lastContainer;
                var saveBlockScopeContainer = blockScopeContainer;
                var saveParent = parent;
                var saveCurrentFlow = currentFlow;
                for (var _i = 0, delayedTypeAliases_1 = delayedTypeAliases; _i < delayedTypeAliases_1.length; _i++) {
                    var typeAlias = delayedTypeAliases_1[_i];
                    var host = ts.getJSDocHost(typeAlias);
                    container = ts.findAncestor(host.parent, function (n) { return !!(getContainerFlags(n) & 1 /* IsContainer */); }) || file;
                    blockScopeContainer = ts.getEnclosingBlockScopeContainer(host) || file;
                    currentFlow = initFlowNode({ flags: 2 /* Start */ });
                    parent = typeAlias;
                    bind(typeAlias.typeExpression);
                    var declName = ts.getNameOfDeclaration(typeAlias);
                    if ((ts.isJSDocEnumTag(typeAlias) || !typeAlias.fullName) && declName && ts.isPropertyAccessEntityNameExpression(declName.parent)) {
                        // typedef anchored to an A.B.C assignment - we need to bind into B's namespace under name C
                        var isTopLevel = isTopLevelNamespaceAssignment(declName.parent);
                        if (isTopLevel) {
                            bindPotentiallyMissingNamespaces(file.symbol, declName.parent, isTopLevel, !!ts.findAncestor(declName, function (d) { return ts.isPropertyAccessExpression(d) && d.name.escapedText === "prototype"; }), /*containerIsClass*/ false);
                            var oldContainer = container;
                            switch (ts.getAssignmentDeclarationPropertyAccessKind(declName.parent)) {
                                case 1 /* ExportsProperty */:
                                case 2 /* ModuleExports */:
                                    if (!ts.isExternalOrCommonJsModule(file)) {
                                        container = undefined;
                                    }
                                    else {
                                        container = file;
                                    }
                                    break;
                                case 4 /* ThisProperty */:
                                    container = declName.parent.expression;
                                    break;
                                case 3 /* PrototypeProperty */:
                                    container = declName.parent.expression.name;
                                    break;
                                case 5 /* Property */:
                                    container = isExportsOrModuleExportsOrAlias(file, declName.parent.expression) ? file
                                        : ts.isPropertyAccessExpression(declName.parent.expression) ? declName.parent.expression.name
                                            : declName.parent.expression;
                                    break;
                                case 0 /* None */:
                                    return ts.Debug.fail("Shouldn't have detected typedef or enum on non-assignment declaration");
                            }
                            if (container) {
                                declareModuleMember(typeAlias, 524288 /* TypeAlias */, 788968 /* TypeAliasExcludes */);
                            }
                            container = oldContainer;
                        }
                    }
                    else if (ts.isJSDocEnumTag(typeAlias) || !typeAlias.fullName || typeAlias.fullName.kind === 75 /* Identifier */) {
                        parent = typeAlias.parent;
                        bindBlockScopedDeclaration(typeAlias, 524288 /* TypeAlias */, 788968 /* TypeAliasExcludes */);
                    }
                    else {
                        bind(typeAlias.fullName);
                    }
                }
                container = saveContainer;
                lastContainer = saveLastContainer;
                blockScopeContainer = saveBlockScopeContainer;
                parent = saveParent;
                currentFlow = saveCurrentFlow;
            }
            // The binder visits every node in the syntax tree so it is a convenient place to perform a single localized
            // check for reserved words used as identifiers in strict mode code.
            function checkStrictModeIdentifier(node) {
                if (inStrictMode &&
                    node.originalKeywordKind >= 113 /* FirstFutureReservedWord */ &&
                    node.originalKeywordKind <= 121 /* LastFutureReservedWord */ &&
                    !ts.isIdentifierName(node) &&
                    !(node.flags & 8388608 /* Ambient */) &&
                    !(node.flags & 4194304 /* JSDoc */)) {
                    // Report error only if there are no parse errors in file
                    if (!file.parseDiagnostics.length) {
                        file.bindDiagnostics.push(createDiagnosticForNode(node, getStrictModeIdentifierMessage(node), ts.declarationNameToString(node)));
                    }
                }
            }
            function getStrictModeIdentifierMessage(node) {
                // Provide specialized messages to help the user understand why we think they're in
                // strict mode.
                if (ts.getContainingClass(node)) {
                    return ts.Diagnostics.Identifier_expected_0_is_a_reserved_word_in_strict_mode_Class_definitions_are_automatically_in_strict_mode;
                }
                if (file.externalModuleIndicator) {
                    return ts.Diagnostics.Identifier_expected_0_is_a_reserved_word_in_strict_mode_Modules_are_automatically_in_strict_mode;
                }
                return ts.Diagnostics.Identifier_expected_0_is_a_reserved_word_in_strict_mode;
            }
            // The binder visits every node, so this is a good place to check for
            // the reserved private name (there is only one)
            function checkPrivateIdentifier(node) {
                if (node.escapedText === "#constructor") {
                    // Report error only if there are no parse errors in file
                    if (!file.parseDiagnostics.length) {
                        file.bindDiagnostics.push(createDiagnosticForNode(node, ts.Diagnostics.constructor_is_a_reserved_word, ts.declarationNameToString(node)));
                    }
                }
            }
            function checkStrictModeBinaryExpression(node) {
                if (inStrictMode && ts.isLeftHandSideExpression(node.left) && ts.isAssignmentOperator(node.operatorToken.kind)) {
                    // ECMA 262 (Annex C) The identifier eval or arguments may not appear as the LeftHandSideExpression of an
                    // Assignment operator(11.13) or of a PostfixExpression(11.3)
                    checkStrictModeEvalOrArguments(node, node.left);
                }
            }
            function checkStrictModeCatchClause(node) {
                // It is a SyntaxError if a TryStatement with a Catch occurs within strict code and the Identifier of the
                // Catch production is eval or arguments
                if (inStrictMode && node.variableDeclaration) {
                    checkStrictModeEvalOrArguments(node, node.variableDeclaration.name);
                }
            }
            function checkStrictModeDeleteExpression(node) {
                // Grammar checking
                if (inStrictMode && node.expression.kind === 75 /* Identifier */) {
                    // When a delete operator occurs within strict mode code, a SyntaxError is thrown if its
                    // UnaryExpression is a direct reference to a variable, function argument, or function name
                    var span = ts.getErrorSpanForNode(file, node.expression);
                    file.bindDiagnostics.push(ts.createFileDiagnostic(file, span.start, span.length, ts.Diagnostics.delete_cannot_be_called_on_an_identifier_in_strict_mode));
                }
            }
            function isEvalOrArgumentsIdentifier(node) {
                return ts.isIdentifier(node) && (node.escapedText === "eval" || node.escapedText === "arguments");
            }
            function checkStrictModeEvalOrArguments(contextNode, name) {
                if (name && name.kind === 75 /* Identifier */) {
                    var identifier = name;
                    if (isEvalOrArgumentsIdentifier(identifier)) {
                        // We check first if the name is inside class declaration or class expression; if so give explicit message
                        // otherwise report generic error message.
                        var span = ts.getErrorSpanForNode(file, name);
                        file.bindDiagnostics.push(ts.createFileDiagnostic(file, span.start, span.length, getStrictModeEvalOrArgumentsMessage(contextNode), ts.idText(identifier)));
                    }
                }
            }
            function getStrictModeEvalOrArgumentsMessage(node) {
                // Provide specialized messages to help the user understand why we think they're in
                // strict mode.
                if (ts.getContainingClass(node)) {
                    return ts.Diagnostics.Invalid_use_of_0_Class_definitions_are_automatically_in_strict_mode;
                }
                if (file.externalModuleIndicator) {
                    return ts.Diagnostics.Invalid_use_of_0_Modules_are_automatically_in_strict_mode;
                }
                return ts.Diagnostics.Invalid_use_of_0_in_strict_mode;
            }
            function checkStrictModeFunctionName(node) {
                if (inStrictMode) {
                    // It is a SyntaxError if the identifier eval or arguments appears within a FormalParameterList of a strict mode FunctionDeclaration or FunctionExpression (13.1))
                    checkStrictModeEvalOrArguments(node, node.name);
                }
            }
            function getStrictModeBlockScopeFunctionDeclarationMessage(node) {
                // Provide specialized messages to help the user understand why we think they're in
                // strict mode.
                if (ts.getContainingClass(node)) {
                    return ts.Diagnostics.Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Class_definitions_are_automatically_in_strict_mode;
                }
                if (file.externalModuleIndicator) {
                    return ts.Diagnostics.Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5_Modules_are_automatically_in_strict_mode;
                }
                return ts.Diagnostics.Function_declarations_are_not_allowed_inside_blocks_in_strict_mode_when_targeting_ES3_or_ES5;
            }
            function checkStrictModeFunctionDeclaration(node) {
                if (languageVersion < 2 /* ES2015 */) {
                    // Report error if function is not top level function declaration
                    if (blockScopeContainer.kind !== 290 /* SourceFile */ &&
                        blockScopeContainer.kind !== 249 /* ModuleDeclaration */ &&
                        !ts.isFunctionLike(blockScopeContainer)) {
                        // We check first if the name is inside class declaration or class expression; if so give explicit message
                        // otherwise report generic error message.
                        var errorSpan = ts.getErrorSpanForNode(file, node);
                        file.bindDiagnostics.push(ts.createFileDiagnostic(file, errorSpan.start, errorSpan.length, getStrictModeBlockScopeFunctionDeclarationMessage(node)));
                    }
                }
            }
            function checkStrictModeNumericLiteral(node) {
                if (inStrictMode && node.numericLiteralFlags & 32 /* Octal */) {
                    file.bindDiagnostics.push(createDiagnosticForNode(node, ts.Diagnostics.Octal_literals_are_not_allowed_in_strict_mode));
                }
            }
            function checkStrictModePostfixUnaryExpression(node) {
                // Grammar checking
                // The identifier eval or arguments may not appear as the LeftHandSideExpression of an
                // Assignment operator(11.13) or of a PostfixExpression(11.3) or as the UnaryExpression
                // operated upon by a Prefix Increment(11.4.4) or a Prefix Decrement(11.4.5) operator.
                if (inStrictMode) {
                    checkStrictModeEvalOrArguments(node, node.operand);
                }
            }
            function checkStrictModePrefixUnaryExpression(node) {
                // Grammar checking
                if (inStrictMode) {
                    if (node.operator === 45 /* PlusPlusToken */ || node.operator === 46 /* MinusMinusToken */) {
                        checkStrictModeEvalOrArguments(node, node.operand);
                    }
                }
            }
            function checkStrictModeWithStatement(node) {
                // Grammar checking for withStatement
                if (inStrictMode) {
                    errorOnFirstToken(node, ts.Diagnostics.with_statements_are_not_allowed_in_strict_mode);
                }
            }
            function checkStrictModeLabeledStatement(node) {
                // Grammar checking for labeledStatement
                if (inStrictMode && options.target >= 2 /* ES2015 */) {
                    if (ts.isDeclarationStatement(node.statement) || ts.isVariableStatement(node.statement)) {
                        errorOnFirstToken(node.label, ts.Diagnostics.A_label_is_not_allowed_here);
                    }
                }
            }
            function errorOnFirstToken(node, message, arg0, arg1, arg2) {
                var span = ts.getSpanOfTokenAtPosition(file, node.pos);
                file.bindDiagnostics.push(ts.createFileDiagnostic(file, span.start, span.length, message, arg0, arg1, arg2));
            }
            function errorOrSuggestionOnNode(isError, node, message) {
                errorOrSuggestionOnRange(isError, node, node, message);
            }
            function errorOrSuggestionOnRange(isError, startNode, endNode, message) {
                addErrorOrSuggestionDiagnostic(isError, { pos: ts.getTokenPosOfNode(startNode, file), end: endNode.end }, message);
            }
            function addErrorOrSuggestionDiagnostic(isError, range, message) {
                var diag = ts.createFileDiagnostic(file, range.pos, range.end - range.pos, message);
                if (isError) {
                    file.bindDiagnostics.push(diag);
                }
                else {
                    file.bindSuggestionDiagnostics = ts.append(file.bindSuggestionDiagnostics, __assign(__assign({}, diag), { category: ts.DiagnosticCategory.Suggestion }));
                }
            }
            function bind(node) {
                if (!node) {
                    return;
                }
                node.parent = parent;
                var saveInStrictMode = inStrictMode;
                // Even though in the AST the jsdoc @typedef node belongs to the current node,
                // its symbol might be in the same scope with the current node's symbol. Consider:
                //
                //     /** @typedef {string | number} MyType */
                //     function foo();
                //
                // Here the current node is "foo", which is a container, but the scope of "MyType" should
                // not be inside "foo". Therefore we always bind @typedef before bind the parent node,
                // and skip binding this tag later when binding all the other jsdoc tags.
                // First we bind declaration nodes to a symbol if possible. We'll both create a symbol
                // and then potentially add the symbol to an appropriate symbol table. Possible
                // destination symbol tables are:
                //
                //  1) The 'exports' table of the current container's symbol.
                //  2) The 'members' table of the current container's symbol.
                //  3) The 'locals' table of the current container.
                //
                // However, not all symbols will end up in any of these tables. 'Anonymous' symbols
                // (like TypeLiterals for example) will not be put in any table.
                bindWorker(node);
                // Then we recurse into the children of the node to bind them as well. For certain
                // symbols we do specialized work when we recurse. For example, we'll keep track of
                // the current 'container' node when it changes. This helps us know which symbol table
                // a local should go into for example. Since terminal nodes are known not to have
                // children, as an optimization we don't process those.
                if (node.kind > 152 /* LastToken */) {
                    var saveParent = parent;
                    parent = node;
                    var containerFlags = getContainerFlags(node);
                    if (containerFlags === 0 /* None */) {
                        bindChildren(node);
                    }
                    else {
                        bindContainer(node, containerFlags);
                    }
                    parent = saveParent;
                }
                else if (!skipTransformFlagAggregation && (node.transformFlags & 536870912 /* HasComputedFlags */) === 0) {
                    subtreeTransformFlags |= computeTransformFlagsForNode(node, 0);
                    var saveParent = parent;
                    if (node.kind === 1 /* EndOfFileToken */)
                        parent = node;
                    bindJSDoc(node);
                    parent = saveParent;
                }
                inStrictMode = saveInStrictMode;
            }
            function bindJSDoc(node) {
                if (ts.hasJSDocNodes(node)) {
                    if (ts.isInJSFile(node)) {
                        for (var _i = 0, _a = node.jsDoc; _i < _a.length; _i++) {
                            var j = _a[_i];
                            bind(j);
                        }
                    }
                    else {
                        for (var _b = 0, _c = node.jsDoc; _b < _c.length; _b++) {
                            var j = _c[_b];
                            setParentPointers(node, j);
                        }
                    }
                }
            }
            function updateStrictModeStatementList(statements) {
                if (!inStrictMode) {
                    for (var _i = 0, statements_2 = statements; _i < statements_2.length; _i++) {
                        var statement = statements_2[_i];
                        if (!ts.isPrologueDirective(statement)) {
                            return;
                        }
                        if (isUseStrictPrologueDirective(statement)) {
                            inStrictMode = true;
                            return;
                        }
                    }
                }
            }
            /// Should be called only on prologue directives (isPrologueDirective(node) should be true)
            function isUseStrictPrologueDirective(node) {
                var nodeText = ts.getSourceTextOfNodeFromSourceFile(file, node.expression);
                // Note: the node text must be exactly "use strict" or 'use strict'.  It is not ok for the
                // string to contain unicode escapes (as per ES5).
                return nodeText === '"use strict"' || nodeText === "'use strict'";
            }
            function bindWorker(node) {
                switch (node.kind) {
                    /* Strict mode checks */
                    case 75 /* Identifier */:
                        // for typedef type names with namespaces, bind the new jsdoc type symbol here
                        // because it requires all containing namespaces to be in effect, namely the
                        // current "blockScopeContainer" needs to be set to its immediate namespace parent.
                        if (node.isInJSDocNamespace) {
                            var parentNode = node.parent;
                            while (parentNode && !ts.isJSDocTypeAlias(parentNode)) {
                                parentNode = parentNode.parent;
                            }
                            bindBlockScopedDeclaration(parentNode, 524288 /* TypeAlias */, 788968 /* TypeAliasExcludes */);
                            break;
                        }
                    // falls through
                    case 104 /* ThisKeyword */:
                        if (currentFlow && (ts.isExpression(node) || parent.kind === 282 /* ShorthandPropertyAssignment */)) {
                            node.flowNode = currentFlow;
                        }
                        return checkStrictModeIdentifier(node);
                    case 76 /* PrivateIdentifier */:
                        return checkPrivateIdentifier(node);
                    case 194 /* PropertyAccessExpression */:
                    case 195 /* ElementAccessExpression */:
                        var expr = node;
                        if (currentFlow && isNarrowableReference(expr)) {
                            expr.flowNode = currentFlow;
                        }
                        if (ts.isSpecialPropertyDeclaration(expr)) {
                            bindSpecialPropertyDeclaration(expr);
                        }
                        if (ts.isInJSFile(expr) &&
                            file.commonJsModuleIndicator &&
                            ts.isModuleExportsAccessExpression(expr) &&
                            !lookupSymbolForNameWorker(blockScopeContainer, "module")) {
                            declareSymbol(file.locals, /*parent*/ undefined, expr.expression, 1 /* FunctionScopedVariable */ | 134217728 /* ModuleExports */, 111550 /* FunctionScopedVariableExcludes */);
                        }
                        break;
                    case 209 /* BinaryExpression */:
                        var specialKind = ts.getAssignmentDeclarationKind(node);
                        switch (specialKind) {
                            case 1 /* ExportsProperty */:
                                bindExportsPropertyAssignment(node);
                                break;
                            case 2 /* ModuleExports */:
                                bindModuleExportsAssignment(node);
                                break;
                            case 3 /* PrototypeProperty */:
                                bindPrototypePropertyAssignment(node.left, node);
                                break;
                            case 6 /* Prototype */:
                                bindPrototypeAssignment(node);
                                break;
                            case 4 /* ThisProperty */:
                                bindThisPropertyAssignment(node);
                                break;
                            case 5 /* Property */:
                                bindSpecialPropertyAssignment(node);
                                break;
                            case 0 /* None */:
                                // Nothing to do
                                break;
                            default:
                                ts.Debug.fail("Unknown binary expression special property assignment kind");
                        }
                        return checkStrictModeBinaryExpression(node);
                    case 280 /* CatchClause */:
                        return checkStrictModeCatchClause(node);
                    case 203 /* DeleteExpression */:
                        return checkStrictModeDeleteExpression(node);
                    case 8 /* NumericLiteral */:
                        return checkStrictModeNumericLiteral(node);
                    case 208 /* PostfixUnaryExpression */:
                        return checkStrictModePostfixUnaryExpression(node);
                    case 207 /* PrefixUnaryExpression */:
                        return checkStrictModePrefixUnaryExpression(node);
                    case 236 /* WithStatement */:
                        return checkStrictModeWithStatement(node);
                    case 238 /* LabeledStatement */:
                        return checkStrictModeLabeledStatement(node);
                    case 183 /* ThisType */:
                        seenThisKeyword = true;
                        return;
                    case 168 /* TypePredicate */:
                        break; // Binding the children will handle everything
                    case 155 /* TypeParameter */:
                        return bindTypeParameter(node);
                    case 156 /* Parameter */:
                        return bindParameter(node);
                    case 242 /* VariableDeclaration */:
                        return bindVariableDeclarationOrBindingElement(node);
                    case 191 /* BindingElement */:
                        node.flowNode = currentFlow;
                        return bindVariableDeclarationOrBindingElement(node);
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                        return bindPropertyWorker(node);
                    case 281 /* PropertyAssignment */:
                    case 282 /* ShorthandPropertyAssignment */:
                        return bindPropertyOrMethodOrAccessor(node, 4 /* Property */, 0 /* PropertyExcludes */);
                    case 284 /* EnumMember */:
                        return bindPropertyOrMethodOrAccessor(node, 8 /* EnumMember */, 900095 /* EnumMemberExcludes */);
                    case 165 /* CallSignature */:
                    case 166 /* ConstructSignature */:
                    case 167 /* IndexSignature */:
                        return declareSymbolAndAddToSymbolTable(node, 131072 /* Signature */, 0 /* None */);
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                        // If this is an ObjectLiteralExpression method, then it sits in the same space
                        // as other properties in the object literal.  So we use SymbolFlags.PropertyExcludes
                        // so that it will conflict with any other object literal members with the same
                        // name.
                        return bindPropertyOrMethodOrAccessor(node, 8192 /* Method */ | (node.questionToken ? 16777216 /* Optional */ : 0 /* None */), ts.isObjectLiteralMethod(node) ? 0 /* PropertyExcludes */ : 103359 /* MethodExcludes */);
                    case 244 /* FunctionDeclaration */:
                        return bindFunctionDeclaration(node);
                    case 162 /* Constructor */:
                        return declareSymbolAndAddToSymbolTable(node, 16384 /* Constructor */, /*symbolExcludes:*/ 0 /* None */);
                    case 163 /* GetAccessor */:
                        return bindPropertyOrMethodOrAccessor(node, 32768 /* GetAccessor */, 46015 /* GetAccessorExcludes */);
                    case 164 /* SetAccessor */:
                        return bindPropertyOrMethodOrAccessor(node, 65536 /* SetAccessor */, 78783 /* SetAccessorExcludes */);
                    case 170 /* FunctionType */:
                    case 300 /* JSDocFunctionType */:
                    case 305 /* JSDocSignature */:
                    case 171 /* ConstructorType */:
                        return bindFunctionOrConstructorType(node);
                    case 173 /* TypeLiteral */:
                    case 304 /* JSDocTypeLiteral */:
                    case 186 /* MappedType */:
                        return bindAnonymousTypeWorker(node);
                    case 309 /* JSDocClassTag */:
                        return bindJSDocClassTag(node);
                    case 193 /* ObjectLiteralExpression */:
                        return bindObjectLiteralExpression(node);
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                        return bindFunctionExpression(node);
                    case 196 /* CallExpression */:
                        var assignmentKind = ts.getAssignmentDeclarationKind(node);
                        switch (assignmentKind) {
                            case 7 /* ObjectDefinePropertyValue */:
                                return bindObjectDefinePropertyAssignment(node);
                            case 8 /* ObjectDefinePropertyExports */:
                                return bindObjectDefinePropertyExport(node);
                            case 9 /* ObjectDefinePrototypeProperty */:
                                return bindObjectDefinePrototypeProperty(node);
                            case 0 /* None */:
                                break; // Nothing to do
                            default:
                                return ts.Debug.fail("Unknown call expression assignment declaration kind");
                        }
                        if (ts.isInJSFile(node)) {
                            bindCallExpression(node);
                        }
                        break;
                    // Members of classes, interfaces, and modules
                    case 214 /* ClassExpression */:
                    case 245 /* ClassDeclaration */:
                        // All classes are automatically in strict mode in ES6.
                        inStrictMode = true;
                        return bindClassLikeDeclaration(node);
                    case 246 /* InterfaceDeclaration */:
                        return bindBlockScopedDeclaration(node, 64 /* Interface */, 788872 /* InterfaceExcludes */);
                    case 247 /* TypeAliasDeclaration */:
                        return bindBlockScopedDeclaration(node, 524288 /* TypeAlias */, 788968 /* TypeAliasExcludes */);
                    case 248 /* EnumDeclaration */:
                        return bindEnumDeclaration(node);
                    case 249 /* ModuleDeclaration */:
                        return bindModuleDeclaration(node);
                    // Jsx-attributes
                    case 274 /* JsxAttributes */:
                        return bindJsxAttributes(node);
                    case 273 /* JsxAttribute */:
                        return bindJsxAttribute(node, 4 /* Property */, 0 /* PropertyExcludes */);
                    // Imports and exports
                    case 253 /* ImportEqualsDeclaration */:
                    case 256 /* NamespaceImport */:
                    case 258 /* ImportSpecifier */:
                    case 263 /* ExportSpecifier */:
                        return declareSymbolAndAddToSymbolTable(node, 2097152 /* Alias */, 2097152 /* AliasExcludes */);
                    case 252 /* NamespaceExportDeclaration */:
                        return bindNamespaceExportDeclaration(node);
                    case 255 /* ImportClause */:
                        return bindImportClause(node);
                    case 260 /* ExportDeclaration */:
                        return bindExportDeclaration(node);
                    case 259 /* ExportAssignment */:
                        return bindExportAssignment(node);
                    case 290 /* SourceFile */:
                        updateStrictModeStatementList(node.statements);
                        return bindSourceFileIfExternalModule();
                    case 223 /* Block */:
                        if (!ts.isFunctionLike(node.parent)) {
                            return;
                        }
                    // falls through
                    case 250 /* ModuleBlock */:
                        return updateStrictModeStatementList(node.statements);
                    case 316 /* JSDocParameterTag */:
                        if (node.parent.kind === 305 /* JSDocSignature */) {
                            return bindParameter(node);
                        }
                        if (node.parent.kind !== 304 /* JSDocTypeLiteral */) {
                            break;
                        }
                    // falls through
                    case 322 /* JSDocPropertyTag */:
                        var propTag = node;
                        var flags = propTag.isBracketed || propTag.typeExpression && propTag.typeExpression.type.kind === 299 /* JSDocOptionalType */ ?
                            4 /* Property */ | 16777216 /* Optional */ :
                            4 /* Property */;
                        return declareSymbolAndAddToSymbolTable(propTag, flags, 0 /* PropertyExcludes */);
                    case 321 /* JSDocTypedefTag */:
                    case 314 /* JSDocCallbackTag */:
                    case 315 /* JSDocEnumTag */:
                        return (delayedTypeAliases || (delayedTypeAliases = [])).push(node);
                }
            }
            function bindPropertyWorker(node) {
                return bindPropertyOrMethodOrAccessor(node, 4 /* Property */ | (node.questionToken ? 16777216 /* Optional */ : 0 /* None */), 0 /* PropertyExcludes */);
            }
            function bindAnonymousTypeWorker(node) {
                return bindAnonymousDeclaration(node, 2048 /* TypeLiteral */, "__type" /* Type */);
            }
            function bindSourceFileIfExternalModule() {
                setExportContextFlag(file);
                if (ts.isExternalModule(file)) {
                    bindSourceFileAsExternalModule();
                }
                else if (ts.isJsonSourceFile(file)) {
                    bindSourceFileAsExternalModule();
                    // Create symbol equivalent for the module.exports = {}
                    var originalSymbol = file.symbol;
                    declareSymbol(file.symbol.exports, file.symbol, file, 4 /* Property */, 67108863 /* All */);
                    file.symbol = originalSymbol;
                }
            }
            function bindSourceFileAsExternalModule() {
                bindAnonymousDeclaration(file, 512 /* ValueModule */, "\"" + ts.removeFileExtension(file.fileName) + "\"");
            }
            function bindExportAssignment(node) {
                if (!container.symbol || !container.symbol.exports) {
                    // Export assignment in some sort of block construct
                    bindAnonymousDeclaration(node, 2097152 /* Alias */, getDeclarationName(node));
                }
                else {
                    var flags = ts.exportAssignmentIsAlias(node)
                        // An export default clause with an EntityNameExpression or a class expression exports all meanings of that identifier or expression;
                        ? 2097152 /* Alias */
                        // An export default clause with any other expression exports a value
                        : 4 /* Property */;
                    // If there is an `export default x;` alias declaration, can't `export default` anything else.
                    // (In contrast, you can still have `export default function f() {}` and `export default interface I {}`.)
                    var symbol = declareSymbol(container.symbol.exports, container.symbol, node, flags, 67108863 /* All */);
                    if (node.isExportEquals) {
                        // Will be an error later, since the module already has other exports. Just make sure this has a valueDeclaration set.
                        setValueDeclaration(symbol, node);
                    }
                }
            }
            function bindNamespaceExportDeclaration(node) {
                if (node.modifiers && node.modifiers.length) {
                    file.bindDiagnostics.push(createDiagnosticForNode(node, ts.Diagnostics.Modifiers_cannot_appear_here));
                }
                var diag = !ts.isSourceFile(node.parent) ? ts.Diagnostics.Global_module_exports_may_only_appear_at_top_level
                    : !ts.isExternalModule(node.parent) ? ts.Diagnostics.Global_module_exports_may_only_appear_in_module_files
                        : !node.parent.isDeclarationFile ? ts.Diagnostics.Global_module_exports_may_only_appear_in_declaration_files
                            : undefined;
                if (diag) {
                    file.bindDiagnostics.push(createDiagnosticForNode(node, diag));
                }
                else {
                    file.symbol.globalExports = file.symbol.globalExports || ts.createSymbolTable();
                    declareSymbol(file.symbol.globalExports, file.symbol, node, 2097152 /* Alias */, 2097152 /* AliasExcludes */);
                }
            }
            function bindExportDeclaration(node) {
                if (!container.symbol || !container.symbol.exports) {
                    // Export * in some sort of block construct
                    bindAnonymousDeclaration(node, 8388608 /* ExportStar */, getDeclarationName(node));
                }
                else if (!node.exportClause) {
                    // All export * declarations are collected in an __export symbol
                    declareSymbol(container.symbol.exports, container.symbol, node, 8388608 /* ExportStar */, 0 /* None */);
                }
                else if (ts.isNamespaceExport(node.exportClause)) {
                    // declareSymbol walks up parents to find name text, parent _must_ be set
                    // but won't be set by the normal binder walk until `bindChildren` later on.
                    node.exportClause.parent = node;
                    declareSymbol(container.symbol.exports, container.symbol, node.exportClause, 2097152 /* Alias */, 2097152 /* AliasExcludes */);
                }
            }
            function bindImportClause(node) {
                if (node.name) {
                    declareSymbolAndAddToSymbolTable(node, 2097152 /* Alias */, 2097152 /* AliasExcludes */);
                }
            }
            function setCommonJsModuleIndicator(node) {
                if (file.externalModuleIndicator) {
                    return false;
                }
                if (!file.commonJsModuleIndicator) {
                    file.commonJsModuleIndicator = node;
                    bindSourceFileAsExternalModule();
                }
                return true;
            }
            function bindObjectDefinePropertyExport(node) {
                if (!setCommonJsModuleIndicator(node)) {
                    return;
                }
                var symbol = forEachIdentifierInEntityName(node.arguments[0], /*parent*/ undefined, function (id, symbol) {
                    if (symbol) {
                        addDeclarationToSymbol(symbol, id, 1536 /* Module */ | 67108864 /* Assignment */);
                    }
                    return symbol;
                });
                if (symbol) {
                    var flags = 4 /* Property */ | 1048576 /* ExportValue */;
                    declareSymbol(symbol.exports, symbol, node, flags, 0 /* None */);
                }
            }
            function bindExportsPropertyAssignment(node) {
                // When we create a property via 'exports.foo = bar', the 'exports.foo' property access
                // expression is the declaration
                if (!setCommonJsModuleIndicator(node)) {
                    return;
                }
                var symbol = forEachIdentifierInEntityName(node.left.expression, /*parent*/ undefined, function (id, symbol) {
                    if (symbol) {
                        addDeclarationToSymbol(symbol, id, 1536 /* Module */ | 67108864 /* Assignment */);
                    }
                    return symbol;
                });
                if (symbol) {
                    var flags = ts.isClassExpression(node.right) ?
                        4 /* Property */ | 1048576 /* ExportValue */ | 32 /* Class */ :
                        4 /* Property */ | 1048576 /* ExportValue */;
                    declareSymbol(symbol.exports, symbol, node.left, flags, 0 /* None */);
                }
            }
            function bindModuleExportsAssignment(node) {
                // A common practice in node modules is to set 'export = module.exports = {}', this ensures that 'exports'
                // is still pointing to 'module.exports'.
                // We do not want to consider this as 'export=' since a module can have only one of these.
                // Similarly we do not want to treat 'module.exports = exports' as an 'export='.
                if (!setCommonJsModuleIndicator(node)) {
                    return;
                }
                var assignedExpression = ts.getRightMostAssignedExpression(node.right);
                if (ts.isEmptyObjectLiteral(assignedExpression) || container === file && isExportsOrModuleExportsOrAlias(file, assignedExpression)) {
                    return;
                }
                // 'module.exports = expr' assignment
                var flags = ts.exportAssignmentIsAlias(node)
                    ? 2097152 /* Alias */
                    : 4 /* Property */ | 1048576 /* ExportValue */ | 512 /* ValueModule */;
                var symbol = declareSymbol(file.symbol.exports, file.symbol, node, flags | 67108864 /* Assignment */, 0 /* None */);
                setValueDeclaration(symbol, node);
            }
            function bindThisPropertyAssignment(node) {
                ts.Debug.assert(ts.isInJSFile(node));
                // private identifiers *must* be declared (even in JS files)
                var hasPrivateIdentifier = (ts.isBinaryExpression(node) && ts.isPropertyAccessExpression(node.left) && ts.isPrivateIdentifier(node.left.name))
                    || (ts.isPropertyAccessExpression(node) && ts.isPrivateIdentifier(node.name));
                if (hasPrivateIdentifier) {
                    return;
                }
                var thisContainer = ts.getThisContainer(node, /*includeArrowFunctions*/ false);
                switch (thisContainer.kind) {
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                        var constructorSymbol = thisContainer.symbol;
                        // For `f.prototype.m = function() { this.x = 0; }`, `this.x = 0` should modify `f`'s members, not the function expression.
                        if (ts.isBinaryExpression(thisContainer.parent) && thisContainer.parent.operatorToken.kind === 62 /* EqualsToken */) {
                            var l = thisContainer.parent.left;
                            if (ts.isBindableStaticAccessExpression(l) && ts.isPrototypeAccess(l.expression)) {
                                constructorSymbol = lookupSymbolForPropertyAccess(l.expression.expression, thisParentContainer);
                            }
                        }
                        if (constructorSymbol && constructorSymbol.valueDeclaration) {
                            // Declare a 'member' if the container is an ES5 class or ES6 constructor
                            constructorSymbol.members = constructorSymbol.members || ts.createSymbolTable();
                            // It's acceptable for multiple 'this' assignments of the same identifier to occur
                            if (ts.hasDynamicName(node)) {
                                bindDynamicallyNamedThisPropertyAssignment(node, constructorSymbol);
                            }
                            else {
                                declareSymbol(constructorSymbol.members, constructorSymbol, node, 4 /* Property */ | 67108864 /* Assignment */, 0 /* PropertyExcludes */ & ~4 /* Property */);
                            }
                            addDeclarationToSymbol(constructorSymbol, constructorSymbol.valueDeclaration, 32 /* Class */);
                        }
                        break;
                    case 162 /* Constructor */:
                    case 159 /* PropertyDeclaration */:
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        // this.foo assignment in a JavaScript class
                        // Bind this property to the containing class
                        var containingClass = thisContainer.parent;
                        var symbolTable = ts.hasModifier(thisContainer, 32 /* Static */) ? containingClass.symbol.exports : containingClass.symbol.members;
                        if (ts.hasDynamicName(node)) {
                            bindDynamicallyNamedThisPropertyAssignment(node, containingClass.symbol);
                        }
                        else {
                            declareSymbol(symbolTable, containingClass.symbol, node, 4 /* Property */ | 67108864 /* Assignment */, 0 /* None */, /*isReplaceableByMethod*/ true);
                        }
                        break;
                    case 290 /* SourceFile */:
                        // this.property = assignment in a source file -- declare symbol in exports for a module, in locals for a script
                        if (ts.hasDynamicName(node)) {
                            break;
                        }
                        else if (thisContainer.commonJsModuleIndicator) {
                            declareSymbol(thisContainer.symbol.exports, thisContainer.symbol, node, 4 /* Property */ | 1048576 /* ExportValue */, 0 /* None */);
                        }
                        else {
                            declareSymbolAndAddToSymbolTable(node, 1 /* FunctionScopedVariable */, 111550 /* FunctionScopedVariableExcludes */);
                        }
                        break;
                    default:
                        ts.Debug.failBadSyntaxKind(thisContainer);
                }
            }
            function bindDynamicallyNamedThisPropertyAssignment(node, symbol) {
                bindAnonymousDeclaration(node, 4 /* Property */, "__computed" /* Computed */);
                addLateBoundAssignmentDeclarationToSymbol(node, symbol);
            }
            function addLateBoundAssignmentDeclarationToSymbol(node, symbol) {
                if (symbol) {
                    var members = symbol.assignmentDeclarationMembers || (symbol.assignmentDeclarationMembers = ts.createMap());
                    members.set("" + ts.getNodeId(node), node);
                }
            }
            function bindSpecialPropertyDeclaration(node) {
                if (node.expression.kind === 104 /* ThisKeyword */) {
                    bindThisPropertyAssignment(node);
                }
                else if (ts.isBindableStaticAccessExpression(node) && node.parent.parent.kind === 290 /* SourceFile */) {
                    if (ts.isPrototypeAccess(node.expression)) {
                        bindPrototypePropertyAssignment(node, node.parent);
                    }
                    else {
                        bindStaticPropertyAssignment(node);
                    }
                }
            }
            /** For `x.prototype = { p, ... }`, declare members p,... if `x` is function/class/{}, or not declared. */
            function bindPrototypeAssignment(node) {
                node.left.parent = node;
                node.right.parent = node;
                bindPropertyAssignment(node.left.expression, node.left, /*isPrototypeProperty*/ false, /*containerIsClass*/ true);
            }
            function bindObjectDefinePrototypeProperty(node) {
                var namespaceSymbol = lookupSymbolForPropertyAccess(node.arguments[0].expression);
                if (namespaceSymbol && namespaceSymbol.valueDeclaration) {
                    // Ensure the namespace symbol becomes class-like
                    addDeclarationToSymbol(namespaceSymbol, namespaceSymbol.valueDeclaration, 32 /* Class */);
                }
                bindPotentiallyNewExpandoMemberToNamespace(node, namespaceSymbol, /*isPrototypeProperty*/ true);
            }
            /**
             * For `x.prototype.y = z`, declare a member `y` on `x` if `x` is a function or class, or not declared.
             * Note that jsdoc preceding an ExpressionStatement like `x.prototype.y;` is also treated as a declaration.
             */
            function bindPrototypePropertyAssignment(lhs, parent) {
                // Look up the function in the local scope, since prototype assignments should
                // follow the function declaration
                var classPrototype = lhs.expression;
                var constructorFunction = classPrototype.expression;
                // Fix up parent pointers since we're going to use these nodes before we bind into them
                lhs.parent = parent;
                constructorFunction.parent = classPrototype;
                classPrototype.parent = lhs;
                bindPropertyAssignment(constructorFunction, lhs, /*isPrototypeProperty*/ true, /*containerIsClass*/ true);
            }
            function bindObjectDefinePropertyAssignment(node) {
                var namespaceSymbol = lookupSymbolForPropertyAccess(node.arguments[0]);
                var isToplevel = node.parent.parent.kind === 290 /* SourceFile */;
                namespaceSymbol = bindPotentiallyMissingNamespaces(namespaceSymbol, node.arguments[0], isToplevel, /*isPrototypeProperty*/ false, /*containerIsClass*/ false);
                bindPotentiallyNewExpandoMemberToNamespace(node, namespaceSymbol, /*isPrototypeProperty*/ false);
            }
            function bindSpecialPropertyAssignment(node) {
                // Class declarations in Typescript do not allow property declarations
                var parentSymbol = lookupSymbolForPropertyAccess(node.left.expression);
                if (!ts.isInJSFile(node) && !ts.isFunctionSymbol(parentSymbol)) {
                    return;
                }
                // Fix up parent pointers since we're going to use these nodes before we bind into them
                node.left.parent = node;
                node.right.parent = node;
                if (ts.isIdentifier(node.left.expression) && container === file && isExportsOrModuleExportsOrAlias(file, node.left.expression)) {
                    // This can be an alias for the 'exports' or 'module.exports' names, e.g.
                    //    var util = module.exports;
                    //    util.property = function ...
                    bindExportsPropertyAssignment(node);
                }
                else {
                    if (ts.hasDynamicName(node)) {
                        bindAnonymousDeclaration(node, 4 /* Property */ | 67108864 /* Assignment */, "__computed" /* Computed */);
                        var sym = bindPotentiallyMissingNamespaces(parentSymbol, node.left.expression, isTopLevelNamespaceAssignment(node.left), /*isPrototype*/ false, /*containerIsClass*/ false);
                        addLateBoundAssignmentDeclarationToSymbol(node, sym);
                    }
                    else {
                        bindStaticPropertyAssignment(ts.cast(node.left, ts.isBindableStaticAccessExpression));
                    }
                }
            }
            /**
             * For nodes like `x.y = z`, declare a member 'y' on 'x' if x is a function (or IIFE) or class or {}, or not declared.
             * Also works for expression statements preceded by JSDoc, like / ** @type number * / x.y;
             */
            function bindStaticPropertyAssignment(node) {
                node.expression.parent = node;
                bindPropertyAssignment(node.expression, node, /*isPrototypeProperty*/ false, /*containerIsClass*/ false);
            }
            function bindPotentiallyMissingNamespaces(namespaceSymbol, entityName, isToplevel, isPrototypeProperty, containerIsClass) {
                if (isToplevel && !isPrototypeProperty) {
                    // make symbols or add declarations for intermediate containers
                    var flags_1 = 1536 /* Module */ | 67108864 /* Assignment */;
                    var excludeFlags_1 = 110735 /* ValueModuleExcludes */ & ~67108864 /* Assignment */;
                    namespaceSymbol = forEachIdentifierInEntityName(entityName, namespaceSymbol, function (id, symbol, parent) {
                        if (symbol) {
                            addDeclarationToSymbol(symbol, id, flags_1);
                            return symbol;
                        }
                        else {
                            var table = parent ? parent.exports :
                                file.jsGlobalAugmentations || (file.jsGlobalAugmentations = ts.createSymbolTable());
                            return declareSymbol(table, parent, id, flags_1, excludeFlags_1);
                        }
                    });
                }
                if (containerIsClass && namespaceSymbol && namespaceSymbol.valueDeclaration) {
                    addDeclarationToSymbol(namespaceSymbol, namespaceSymbol.valueDeclaration, 32 /* Class */);
                }
                return namespaceSymbol;
            }
            function bindPotentiallyNewExpandoMemberToNamespace(declaration, namespaceSymbol, isPrototypeProperty) {
                if (!namespaceSymbol || !isExpandoSymbol(namespaceSymbol)) {
                    return;
                }
                // Set up the members collection if it doesn't exist already
                var symbolTable = isPrototypeProperty ?
                    (namespaceSymbol.members || (namespaceSymbol.members = ts.createSymbolTable())) :
                    (namespaceSymbol.exports || (namespaceSymbol.exports = ts.createSymbolTable()));
                var includes = 0 /* None */;
                var excludes = 0 /* None */;
                // Method-like
                if (ts.isFunctionLikeDeclaration(ts.getAssignedExpandoInitializer(declaration))) {
                    includes = 8192 /* Method */;
                    excludes = 103359 /* MethodExcludes */;
                }
                // Maybe accessor-like
                else if (ts.isCallExpression(declaration) && ts.isBindableObjectDefinePropertyCall(declaration)) {
                    if (ts.some(declaration.arguments[2].properties, function (p) {
                        var id = ts.getNameOfDeclaration(p);
                        return !!id && ts.isIdentifier(id) && ts.idText(id) === "set";
                    })) {
                        // We mix in `SymbolFLags.Property` so in the checker `getTypeOfVariableParameterOrProperty` is used for this
                        // symbol, instead of `getTypeOfAccessor` (which will assert as there is no real accessor declaration)
                        includes |= 65536 /* SetAccessor */ | 4 /* Property */;
                        excludes |= 78783 /* SetAccessorExcludes */;
                    }
                    if (ts.some(declaration.arguments[2].properties, function (p) {
                        var id = ts.getNameOfDeclaration(p);
                        return !!id && ts.isIdentifier(id) && ts.idText(id) === "get";
                    })) {
                        includes |= 32768 /* GetAccessor */ | 4 /* Property */;
                        excludes |= 46015 /* GetAccessorExcludes */;
                    }
                }
                if (includes === 0 /* None */) {
                    includes = 4 /* Property */;
                    excludes = 0 /* PropertyExcludes */;
                }
                declareSymbol(symbolTable, namespaceSymbol, declaration, includes | 67108864 /* Assignment */, excludes & ~67108864 /* Assignment */);
            }
            function isTopLevelNamespaceAssignment(propertyAccess) {
                return ts.isBinaryExpression(propertyAccess.parent)
                    ? getParentOfBinaryExpression(propertyAccess.parent).parent.kind === 290 /* SourceFile */
                    : propertyAccess.parent.parent.kind === 290 /* SourceFile */;
            }
            function bindPropertyAssignment(name, propertyAccess, isPrototypeProperty, containerIsClass) {
                var namespaceSymbol = lookupSymbolForPropertyAccess(name);
                var isToplevel = isTopLevelNamespaceAssignment(propertyAccess);
                namespaceSymbol = bindPotentiallyMissingNamespaces(namespaceSymbol, propertyAccess.expression, isToplevel, isPrototypeProperty, containerIsClass);
                bindPotentiallyNewExpandoMemberToNamespace(propertyAccess, namespaceSymbol, isPrototypeProperty);
            }
            /**
             * Javascript expando values are:
             * - Functions
             * - classes
             * - namespaces
             * - variables initialized with function expressions
             * -                       with class expressions
             * -                       with empty object literals
             * -                       with non-empty object literals if assigned to the prototype property
             */
            function isExpandoSymbol(symbol) {
                if (symbol.flags & (16 /* Function */ | 32 /* Class */ | 1024 /* NamespaceModule */)) {
                    return true;
                }
                var node = symbol.valueDeclaration;
                if (node && ts.isCallExpression(node)) {
                    return !!ts.getAssignedExpandoInitializer(node);
                }
                var init = !node ? undefined :
                    ts.isVariableDeclaration(node) ? node.initializer :
                        ts.isBinaryExpression(node) ? node.right :
                            ts.isPropertyAccessExpression(node) && ts.isBinaryExpression(node.parent) ? node.parent.right :
                                undefined;
                init = init && ts.getRightMostAssignedExpression(init);
                if (init) {
                    var isPrototypeAssignment = ts.isPrototypeAccess(ts.isVariableDeclaration(node) ? node.name : ts.isBinaryExpression(node) ? node.left : node);
                    return !!ts.getExpandoInitializer(ts.isBinaryExpression(init) && (init.operatorToken.kind === 56 /* BarBarToken */ || init.operatorToken.kind === 60 /* QuestionQuestionToken */) ? init.right : init, isPrototypeAssignment);
                }
                return false;
            }
            function getParentOfBinaryExpression(expr) {
                while (ts.isBinaryExpression(expr.parent)) {
                    expr = expr.parent;
                }
                return expr.parent;
            }
            function lookupSymbolForPropertyAccess(node, lookupContainer) {
                if (lookupContainer === void 0) {
                    lookupContainer = container;
                }
                if (ts.isIdentifier(node)) {
                    return lookupSymbolForNameWorker(lookupContainer, node.escapedText);
                }
                else {
                    var symbol = lookupSymbolForPropertyAccess(node.expression);
                    return symbol && symbol.exports && symbol.exports.get(ts.getElementOrPropertyAccessName(node));
                }
            }
            function forEachIdentifierInEntityName(e, parent, action) {
                if (isExportsOrModuleExportsOrAlias(file, e)) {
                    return file.symbol;
                }
                else if (ts.isIdentifier(e)) {
                    return action(e, lookupSymbolForPropertyAccess(e), parent);
                }
                else {
                    var s = forEachIdentifierInEntityName(e.expression, parent, action);
                    var name = ts.getNameOrArgument(e);
                    // unreachable
                    if (ts.isPrivateIdentifier(name)) {
                        ts.Debug.fail("unexpected PrivateIdentifier");
                    }
                    return action(name, s && s.exports && s.exports.get(ts.getElementOrPropertyAccessName(e)), s);
                }
            }
            function bindCallExpression(node) {
                // We're only inspecting call expressions to detect CommonJS modules, so we can skip
                // this check if we've already seen the module indicator
                if (!file.commonJsModuleIndicator && ts.isRequireCall(node, /*checkArgumentIsStringLiteralLike*/ false)) {
                    setCommonJsModuleIndicator(node);
                }
            }
            function bindClassLikeDeclaration(node) {
                if (node.kind === 245 /* ClassDeclaration */) {
                    bindBlockScopedDeclaration(node, 32 /* Class */, 899503 /* ClassExcludes */);
                }
                else {
                    var bindingName = node.name ? node.name.escapedText : "__class" /* Class */;
                    bindAnonymousDeclaration(node, 32 /* Class */, bindingName);
                    // Add name of class expression into the map for semantic classifier
                    if (node.name) {
                        classifiableNames.set(node.name.escapedText, true);
                    }
                }
                var symbol = node.symbol;
                // TypeScript 1.0 spec (April 2014): 8.4
                // Every class automatically contains a static property member named 'prototype', the
                // type of which is an instantiation of the class type with type Any supplied as a type
                // argument for each type parameter. It is an error to explicitly declare a static
                // property member with the name 'prototype'.
                //
                // Note: we check for this here because this class may be merging into a module.  The
                // module might have an exported variable called 'prototype'.  We can't allow that as
                // that would clash with the built-in 'prototype' for the class.
                var prototypeSymbol = createSymbol(4 /* Property */ | 4194304 /* Prototype */, "prototype");
                var symbolExport = symbol.exports.get(prototypeSymbol.escapedName);
                if (symbolExport) {
                    if (node.name) {
                        node.name.parent = node;
                    }
                    file.bindDiagnostics.push(createDiagnosticForNode(symbolExport.declarations[0], ts.Diagnostics.Duplicate_identifier_0, ts.symbolName(prototypeSymbol)));
                }
                symbol.exports.set(prototypeSymbol.escapedName, prototypeSymbol);
                prototypeSymbol.parent = symbol;
            }
            function bindEnumDeclaration(node) {
                return ts.isEnumConst(node)
                    ? bindBlockScopedDeclaration(node, 128 /* ConstEnum */, 899967 /* ConstEnumExcludes */)
                    : bindBlockScopedDeclaration(node, 256 /* RegularEnum */, 899327 /* RegularEnumExcludes */);
            }
            function bindVariableDeclarationOrBindingElement(node) {
                if (inStrictMode) {
                    checkStrictModeEvalOrArguments(node, node.name);
                }
                if (!ts.isBindingPattern(node.name)) {
                    if (ts.isBlockOrCatchScoped(node)) {
                        bindBlockScopedDeclaration(node, 2 /* BlockScopedVariable */, 111551 /* BlockScopedVariableExcludes */);
                    }
                    else if (ts.isParameterDeclaration(node)) {
                        // It is safe to walk up parent chain to find whether the node is a destructuring parameter declaration
                        // because its parent chain has already been set up, since parents are set before descending into children.
                        //
                        // If node is a binding element in parameter declaration, we need to use ParameterExcludes.
                        // Using ParameterExcludes flag allows the compiler to report an error on duplicate identifiers in Parameter Declaration
                        // For example:
                        //      function foo([a,a]) {} // Duplicate Identifier error
                        //      function bar(a,a) {}   // Duplicate Identifier error, parameter declaration in this case is handled in bindParameter
                        //                             // which correctly set excluded symbols
                        declareSymbolAndAddToSymbolTable(node, 1 /* FunctionScopedVariable */, 111551 /* ParameterExcludes */);
                    }
                    else {
                        declareSymbolAndAddToSymbolTable(node, 1 /* FunctionScopedVariable */, 111550 /* FunctionScopedVariableExcludes */);
                    }
                }
            }
            function bindParameter(node) {
                if (node.kind === 316 /* JSDocParameterTag */ && container.kind !== 305 /* JSDocSignature */) {
                    return;
                }
                if (inStrictMode && !(node.flags & 8388608 /* Ambient */)) {
                    // It is a SyntaxError if the identifier eval or arguments appears within a FormalParameterList of a
                    // strict mode FunctionLikeDeclaration or FunctionExpression(13.1)
                    checkStrictModeEvalOrArguments(node, node.name);
                }
                if (ts.isBindingPattern(node.name)) {
                    bindAnonymousDeclaration(node, 1 /* FunctionScopedVariable */, "__" + node.parent.parameters.indexOf(node));
                }
                else {
                    declareSymbolAndAddToSymbolTable(node, 1 /* FunctionScopedVariable */, 111551 /* ParameterExcludes */);
                }
                // If this is a property-parameter, then also declare the property symbol into the
                // containing class.
                if (ts.isParameterPropertyDeclaration(node, node.parent)) {
                    var classDeclaration = node.parent.parent;
                    declareSymbol(classDeclaration.symbol.members, classDeclaration.symbol, node, 4 /* Property */ | (node.questionToken ? 16777216 /* Optional */ : 0 /* None */), 0 /* PropertyExcludes */);
                }
            }
            function bindFunctionDeclaration(node) {
                if (!file.isDeclarationFile && !(node.flags & 8388608 /* Ambient */)) {
                    if (ts.isAsyncFunction(node)) {
                        emitFlags |= 2048 /* HasAsyncFunctions */;
                    }
                }
                checkStrictModeFunctionName(node);
                if (inStrictMode) {
                    checkStrictModeFunctionDeclaration(node);
                    bindBlockScopedDeclaration(node, 16 /* Function */, 110991 /* FunctionExcludes */);
                }
                else {
                    declareSymbolAndAddToSymbolTable(node, 16 /* Function */, 110991 /* FunctionExcludes */);
                }
            }
            function bindFunctionExpression(node) {
                if (!file.isDeclarationFile && !(node.flags & 8388608 /* Ambient */)) {
                    if (ts.isAsyncFunction(node)) {
                        emitFlags |= 2048 /* HasAsyncFunctions */;
                    }
                }
                if (currentFlow) {
                    node.flowNode = currentFlow;
                }
                checkStrictModeFunctionName(node);
                var bindingName = node.name ? node.name.escapedText : "__function" /* Function */;
                return bindAnonymousDeclaration(node, 16 /* Function */, bindingName);
            }
            function bindPropertyOrMethodOrAccessor(node, symbolFlags, symbolExcludes) {
                if (!file.isDeclarationFile && !(node.flags & 8388608 /* Ambient */) && ts.isAsyncFunction(node)) {
                    emitFlags |= 2048 /* HasAsyncFunctions */;
                }
                if (currentFlow && ts.isObjectLiteralOrClassExpressionMethod(node)) {
                    node.flowNode = currentFlow;
                }
                return ts.hasDynamicName(node)
                    ? bindAnonymousDeclaration(node, symbolFlags, "__computed" /* Computed */)
                    : declareSymbolAndAddToSymbolTable(node, symbolFlags, symbolExcludes);
            }
            function getInferTypeContainer(node) {
                var extendsType = ts.findAncestor(node, function (n) { return n.parent && ts.isConditionalTypeNode(n.parent) && n.parent.extendsType === n; });
                return extendsType && extendsType.parent;
            }
            function bindTypeParameter(node) {
                if (ts.isJSDocTemplateTag(node.parent)) {
                    var container_1 = ts.find(node.parent.parent.tags, ts.isJSDocTypeAlias) || ts.getHostSignatureFromJSDoc(node.parent); // TODO: GH#18217
                    if (container_1) {
                        if (!container_1.locals) {
                            container_1.locals = ts.createSymbolTable();
                        }
                        declareSymbol(container_1.locals, /*parent*/ undefined, node, 262144 /* TypeParameter */, 526824 /* TypeParameterExcludes */);
                    }
                    else {
                        declareSymbolAndAddToSymbolTable(node, 262144 /* TypeParameter */, 526824 /* TypeParameterExcludes */);
                    }
                }
                else if (node.parent.kind === 181 /* InferType */) {
                    var container_2 = getInferTypeContainer(node.parent);
                    if (container_2) {
                        if (!container_2.locals) {
                            container_2.locals = ts.createSymbolTable();
                        }
                        declareSymbol(container_2.locals, /*parent*/ undefined, node, 262144 /* TypeParameter */, 526824 /* TypeParameterExcludes */);
                    }
                    else {
                        bindAnonymousDeclaration(node, 262144 /* TypeParameter */, getDeclarationName(node)); // TODO: GH#18217
                    }
                }
                else {
                    declareSymbolAndAddToSymbolTable(node, 262144 /* TypeParameter */, 526824 /* TypeParameterExcludes */);
                }
            }
            // reachability checks
            function shouldReportErrorOnModuleDeclaration(node) {
                var instanceState = getModuleInstanceState(node);
                return instanceState === 1 /* Instantiated */ || (instanceState === 2 /* ConstEnumOnly */ && !!options.preserveConstEnums);
            }
            function checkUnreachable(node) {
                if (!(currentFlow.flags & 1 /* Unreachable */)) {
                    return false;
                }
                if (currentFlow === unreachableFlow) {
                    var reportError = 
                    // report error on all statements except empty ones
                    (ts.isStatementButNotDeclaration(node) && node.kind !== 224 /* EmptyStatement */) ||
                        // report error on class declarations
                        node.kind === 245 /* ClassDeclaration */ ||
                        // report error on instantiated modules or const-enums only modules if preserveConstEnums is set
                        (node.kind === 249 /* ModuleDeclaration */ && shouldReportErrorOnModuleDeclaration(node));
                    if (reportError) {
                        currentFlow = reportedUnreachableFlow;
                        if (!options.allowUnreachableCode) {
                            // unreachable code is reported if
                            // - user has explicitly asked about it AND
                            // - statement is in not ambient context (statements in ambient context is already an error
                            //   so we should not report extras) AND
                            //   - node is not variable statement OR
                            //   - node is block scoped variable statement OR
                            //   - node is not block scoped variable statement and at least one variable declaration has initializer
                            //   Rationale: we don't want to report errors on non-initialized var's since they are hoisted
                            //   On the other side we do want to report errors on non-initialized 'lets' because of TDZ
                            var isError_1 = ts.unreachableCodeIsError(options) &&
                                !(node.flags & 8388608 /* Ambient */) &&
                                (!ts.isVariableStatement(node) ||
                                    !!(ts.getCombinedNodeFlags(node.declarationList) & 3 /* BlockScoped */) ||
                                    node.declarationList.declarations.some(function (d) { return !!d.initializer; }));
                            eachUnreachableRange(node, function (start, end) { return errorOrSuggestionOnRange(isError_1, start, end, ts.Diagnostics.Unreachable_code_detected); });
                        }
                    }
                }
                return true;
            }
        }
        function eachUnreachableRange(node, cb) {
            if (ts.isStatement(node) && isExecutableStatement(node) && ts.isBlock(node.parent)) {
                var statements = node.parent.statements;
                var slice_1 = ts.sliceAfter(statements, node);
                ts.getRangesWhere(slice_1, isExecutableStatement, function (start, afterEnd) { return cb(slice_1[start], slice_1[afterEnd - 1]); });
            }
            else {
                cb(node, node);
            }
        }
        // As opposed to a pure declaration like an `interface`
        function isExecutableStatement(s) {
            // Don't remove statements that can validly be used before they appear.
            return !ts.isFunctionDeclaration(s) && !isPurelyTypeDeclaration(s) && !ts.isEnumDeclaration(s) &&
                // `var x;` may declare a variable used above
                !(ts.isVariableStatement(s) && !(ts.getCombinedNodeFlags(s) & (1 /* Let */ | 2 /* Const */)) && s.declarationList.declarations.some(function (d) { return !d.initializer; }));
        }
        function isPurelyTypeDeclaration(s) {
            switch (s.kind) {
                case 246 /* InterfaceDeclaration */:
                case 247 /* TypeAliasDeclaration */:
                    return true;
                case 249 /* ModuleDeclaration */:
                    return getModuleInstanceState(s) !== 1 /* Instantiated */;
                case 248 /* EnumDeclaration */:
                    return ts.hasModifier(s, 2048 /* Const */);
                default:
                    return false;
            }
        }
        function isExportsOrModuleExportsOrAlias(sourceFile, node) {
            var i = 0;
            var q = [node];
            while (q.length && i < 100) {
                i++;
                node = q.shift();
                if (ts.isExportsIdentifier(node) || ts.isModuleExportsAccessExpression(node)) {
                    return true;
                }
                else if (ts.isIdentifier(node)) {
                    var symbol = lookupSymbolForNameWorker(sourceFile, node.escapedText);
                    if (!!symbol && !!symbol.valueDeclaration && ts.isVariableDeclaration(symbol.valueDeclaration) && !!symbol.valueDeclaration.initializer) {
                        var init = symbol.valueDeclaration.initializer;
                        q.push(init);
                        if (ts.isAssignmentExpression(init, /*excludeCompoundAssignment*/ true)) {
                            q.push(init.left);
                            q.push(init.right);
                        }
                    }
                }
            }
            return false;
        }
        ts.isExportsOrModuleExportsOrAlias = isExportsOrModuleExportsOrAlias;
        function lookupSymbolForNameWorker(container, name) {
            var local = container.locals && container.locals.get(name);
            if (local) {
                return local.exportSymbol || local;
            }
            if (ts.isSourceFile(container) && container.jsGlobalAugmentations && container.jsGlobalAugmentations.has(name)) {
                return container.jsGlobalAugmentations.get(name);
            }
            return container.symbol && container.symbol.exports && container.symbol.exports.get(name);
        }
        /**
         * Computes the transform flags for a node, given the transform flags of its subtree
         *
         * @param node The node to analyze
         * @param subtreeFlags Transform flags computed for this node's subtree
         */
        function computeTransformFlagsForNode(node, subtreeFlags) {
            var kind = node.kind;
            switch (kind) {
                case 196 /* CallExpression */:
                    return computeCallExpression(node, subtreeFlags);
                case 197 /* NewExpression */:
                    return computeNewExpression(node, subtreeFlags);
                case 249 /* ModuleDeclaration */:
                    return computeModuleDeclaration(node, subtreeFlags);
                case 200 /* ParenthesizedExpression */:
                    return computeParenthesizedExpression(node, subtreeFlags);
                case 209 /* BinaryExpression */:
                    return computeBinaryExpression(node, subtreeFlags);
                case 226 /* ExpressionStatement */:
                    return computeExpressionStatement(node, subtreeFlags);
                case 156 /* Parameter */:
                    return computeParameter(node, subtreeFlags);
                case 202 /* ArrowFunction */:
                    return computeArrowFunction(node, subtreeFlags);
                case 201 /* FunctionExpression */:
                    return computeFunctionExpression(node, subtreeFlags);
                case 244 /* FunctionDeclaration */:
                    return computeFunctionDeclaration(node, subtreeFlags);
                case 242 /* VariableDeclaration */:
                    return computeVariableDeclaration(node, subtreeFlags);
                case 243 /* VariableDeclarationList */:
                    return computeVariableDeclarationList(node, subtreeFlags);
                case 225 /* VariableStatement */:
                    return computeVariableStatement(node, subtreeFlags);
                case 238 /* LabeledStatement */:
                    return computeLabeledStatement(node, subtreeFlags);
                case 245 /* ClassDeclaration */:
                    return computeClassDeclaration(node, subtreeFlags);
                case 214 /* ClassExpression */:
                    return computeClassExpression(node, subtreeFlags);
                case 279 /* HeritageClause */:
                    return computeHeritageClause(node, subtreeFlags);
                case 280 /* CatchClause */:
                    return computeCatchClause(node, subtreeFlags);
                case 216 /* ExpressionWithTypeArguments */:
                    return computeExpressionWithTypeArguments(node, subtreeFlags);
                case 162 /* Constructor */:
                    return computeConstructor(node, subtreeFlags);
                case 159 /* PropertyDeclaration */:
                    return computePropertyDeclaration(node, subtreeFlags);
                case 161 /* MethodDeclaration */:
                    return computeMethod(node, subtreeFlags);
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                    return computeAccessor(node, subtreeFlags);
                case 253 /* ImportEqualsDeclaration */:
                    return computeImportEquals(node, subtreeFlags);
                case 194 /* PropertyAccessExpression */:
                    return computePropertyAccess(node, subtreeFlags);
                case 195 /* ElementAccessExpression */:
                    return computeElementAccess(node, subtreeFlags);
                default:
                    return computeOther(node, kind, subtreeFlags);
            }
        }
        ts.computeTransformFlagsForNode = computeTransformFlagsForNode;
        function computeCallExpression(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            var callee = ts.skipOuterExpressions(node.expression);
            var expression = node.expression;
            if (node.flags & 32 /* OptionalChain */) {
                transformFlags |= 8 /* ContainsES2020 */;
            }
            if (node.typeArguments) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            if (subtreeFlags & 8192 /* ContainsRestOrSpread */ || ts.isSuperOrSuperProperty(callee)) {
                // If the this node contains a SpreadExpression, or is a super call, then it is an ES6
                // node.
                transformFlags |= 256 /* AssertES2015 */;
                if (ts.isSuperProperty(callee)) {
                    transformFlags |= 4096 /* ContainsLexicalThis */;
                }
            }
            if (expression.kind === 96 /* ImportKeyword */) {
                transformFlags |= 2097152 /* ContainsDynamicImport */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536879104 /* ArrayLiteralOrCallOrNewExcludes */;
        }
        function computeNewExpression(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            if (node.typeArguments) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            if (subtreeFlags & 8192 /* ContainsRestOrSpread */) {
                // If the this node contains a SpreadElementExpression then it is an ES6
                // node.
                transformFlags |= 256 /* AssertES2015 */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536879104 /* ArrayLiteralOrCallOrNewExcludes */;
        }
        function computeBinaryExpression(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            var operatorTokenKind = node.operatorToken.kind;
            var leftKind = node.left.kind;
            if (operatorTokenKind === 60 /* QuestionQuestionToken */) {
                transformFlags |= 8 /* AssertES2020 */;
            }
            else if (operatorTokenKind === 62 /* EqualsToken */ && leftKind === 193 /* ObjectLiteralExpression */) {
                // Destructuring object assignments with are ES2015 syntax
                // and possibly ES2018 if they contain rest
                transformFlags |= 32 /* AssertES2018 */ | 256 /* AssertES2015 */ | 1024 /* AssertDestructuringAssignment */;
            }
            else if (operatorTokenKind === 62 /* EqualsToken */ && leftKind === 192 /* ArrayLiteralExpression */) {
                // Destructuring assignments are ES2015 syntax.
                transformFlags |= 256 /* AssertES2015 */ | 1024 /* AssertDestructuringAssignment */;
            }
            else if (operatorTokenKind === 42 /* AsteriskAsteriskToken */
                || operatorTokenKind === 66 /* AsteriskAsteriskEqualsToken */) {
                // Exponentiation is ES2016 syntax.
                transformFlags |= 128 /* AssertES2016 */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* NodeExcludes */;
        }
        function computeParameter(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            var name = node.name;
            var initializer = node.initializer;
            var dotDotDotToken = node.dotDotDotToken;
            // The '?' token, type annotations, decorators, and 'this' parameters are TypeSCript
            // syntax.
            if (node.questionToken
                || node.type
                || (subtreeFlags & 2048 /* ContainsTypeScriptClassSyntax */ && ts.some(node.decorators))
                || ts.isThisIdentifier(name)) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            // If a parameter has an accessibility modifier, then it is TypeScript syntax.
            if (ts.hasModifier(node, 92 /* ParameterPropertyModifier */)) {
                transformFlags |= 1 /* AssertTypeScript */ | 2048 /* ContainsTypeScriptClassSyntax */;
            }
            // parameters with object rest destructuring are ES2018 syntax
            if (subtreeFlags & 16384 /* ContainsObjectRestOrSpread */) {
                transformFlags |= 32 /* AssertES2018 */;
            }
            // If a parameter has an initializer, a binding pattern or a dotDotDot token, then
            // it is ES6 syntax and its container must emit default value assignments or parameter destructuring downlevel.
            if (subtreeFlags & 131072 /* ContainsBindingPattern */ || initializer || dotDotDotToken) {
                transformFlags |= 256 /* AssertES2015 */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* ParameterExcludes */;
        }
        function computeParenthesizedExpression(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            var expression = node.expression;
            var expressionKind = expression.kind;
            // If the node is synthesized, it means the emitter put the parentheses there,
            // not the user. If we didn't want them, the emitter would not have put them
            // there.
            if (expressionKind === 217 /* AsExpression */
                || expressionKind === 199 /* TypeAssertionExpression */) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* OuterExpressionExcludes */;
        }
        function computeClassDeclaration(node, subtreeFlags) {
            var transformFlags;
            if (ts.hasModifier(node, 2 /* Ambient */)) {
                // An ambient declaration is TypeScript syntax.
                transformFlags = 1 /* AssertTypeScript */;
            }
            else {
                // A ClassDeclaration is ES6 syntax.
                transformFlags = subtreeFlags | 256 /* AssertES2015 */;
                // A class with a parameter property assignment or decorator is TypeScript syntax.
                // An exported declaration may be TypeScript syntax, but is handled by the visitor
                // for a namespace declaration.
                if ((subtreeFlags & 2048 /* ContainsTypeScriptClassSyntax */)
                    || node.typeParameters) {
                    transformFlags |= 1 /* AssertTypeScript */;
                }
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536905728 /* ClassExcludes */;
        }
        function computeClassExpression(node, subtreeFlags) {
            // A ClassExpression is ES6 syntax.
            var transformFlags = subtreeFlags | 256 /* AssertES2015 */;
            // A class with a parameter property assignment or decorator is TypeScript syntax.
            if (subtreeFlags & 2048 /* ContainsTypeScriptClassSyntax */
                || node.typeParameters) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536905728 /* ClassExcludes */;
        }
        function computeHeritageClause(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            switch (node.token) {
                case 90 /* ExtendsKeyword */:
                    // An `extends` HeritageClause is ES6 syntax.
                    transformFlags |= 256 /* AssertES2015 */;
                    break;
                case 113 /* ImplementsKeyword */:
                    // An `implements` HeritageClause is TypeScript syntax.
                    transformFlags |= 1 /* AssertTypeScript */;
                    break;
                default:
                    ts.Debug.fail("Unexpected token for heritage clause");
                    break;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* NodeExcludes */;
        }
        function computeCatchClause(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            if (!node.variableDeclaration) {
                transformFlags |= 16 /* AssertES2019 */;
            }
            else if (ts.isBindingPattern(node.variableDeclaration.name)) {
                transformFlags |= 256 /* AssertES2015 */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536887296 /* CatchClauseExcludes */;
        }
        function computeExpressionWithTypeArguments(node, subtreeFlags) {
            // An ExpressionWithTypeArguments is ES6 syntax, as it is used in the
            // extends clause of a class.
            var transformFlags = subtreeFlags | 256 /* AssertES2015 */;
            // If an ExpressionWithTypeArguments contains type arguments, then it
            // is TypeScript syntax.
            if (node.typeArguments) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* NodeExcludes */;
        }
        function computeConstructor(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            // TypeScript-specific modifiers and overloads are TypeScript syntax
            if (ts.hasModifier(node, 2270 /* TypeScriptModifier */)
                || !node.body) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            // function declarations with object rest destructuring are ES2018 syntax
            if (subtreeFlags & 16384 /* ContainsObjectRestOrSpread */) {
                transformFlags |= 32 /* AssertES2018 */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~538923008 /* ConstructorExcludes */;
        }
        function computeMethod(node, subtreeFlags) {
            // A MethodDeclaration is ES6 syntax.
            var transformFlags = subtreeFlags | 256 /* AssertES2015 */;
            // Decorators, TypeScript-specific modifiers, type parameters, type annotations, and
            // overloads are TypeScript syntax.
            if (node.decorators
                || ts.hasModifier(node, 2270 /* TypeScriptModifier */)
                || node.typeParameters
                || node.type
                || !node.body
                || node.questionToken) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            // function declarations with object rest destructuring are ES2018 syntax
            if (subtreeFlags & 16384 /* ContainsObjectRestOrSpread */) {
                transformFlags |= 32 /* AssertES2018 */;
            }
            // An async method declaration is ES2017 syntax.
            if (ts.hasModifier(node, 256 /* Async */)) {
                transformFlags |= node.asteriskToken ? 32 /* AssertES2018 */ : 64 /* AssertES2017 */;
            }
            if (node.asteriskToken) {
                transformFlags |= 512 /* AssertGenerator */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return propagatePropertyNameFlags(node.name, transformFlags & ~538923008 /* MethodOrAccessorExcludes */);
        }
        function computeAccessor(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            // Decorators, TypeScript-specific modifiers, type annotations, and overloads are
            // TypeScript syntax.
            if (node.decorators
                || ts.hasModifier(node, 2270 /* TypeScriptModifier */)
                || node.type
                || !node.body) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            // function declarations with object rest destructuring are ES2018 syntax
            if (subtreeFlags & 16384 /* ContainsObjectRestOrSpread */) {
                transformFlags |= 32 /* AssertES2018 */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return propagatePropertyNameFlags(node.name, transformFlags & ~538923008 /* MethodOrAccessorExcludes */);
        }
        function computePropertyDeclaration(node, subtreeFlags) {
            var transformFlags = subtreeFlags | 4194304 /* ContainsClassFields */;
            // Decorators, TypeScript-specific modifiers, and type annotations are TypeScript syntax.
            if (ts.some(node.decorators) || ts.hasModifier(node, 2270 /* TypeScriptModifier */) || node.type || node.questionToken || node.exclamationToken) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            // Hoisted variables related to class properties should live within the TypeScript class wrapper.
            if (ts.isComputedPropertyName(node.name) || (ts.hasStaticModifier(node) && node.initializer)) {
                transformFlags |= 2048 /* ContainsTypeScriptClassSyntax */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return propagatePropertyNameFlags(node.name, transformFlags & ~536875008 /* PropertyExcludes */);
        }
        function computeFunctionDeclaration(node, subtreeFlags) {
            var transformFlags;
            var modifierFlags = ts.getModifierFlags(node);
            var body = node.body;
            if (!body || (modifierFlags & 2 /* Ambient */)) {
                // An ambient declaration is TypeScript syntax.
                // A FunctionDeclaration without a body is an overload and is TypeScript syntax.
                transformFlags = 1 /* AssertTypeScript */;
            }
            else {
                transformFlags = subtreeFlags | 1048576 /* ContainsHoistedDeclarationOrCompletion */;
                // TypeScript-specific modifiers, type parameters, and type annotations are TypeScript
                // syntax.
                if (modifierFlags & 2270 /* TypeScriptModifier */
                    || node.typeParameters
                    || node.type) {
                    transformFlags |= 1 /* AssertTypeScript */;
                }
                // An async function declaration is ES2017 syntax.
                if (modifierFlags & 256 /* Async */) {
                    transformFlags |= node.asteriskToken ? 32 /* AssertES2018 */ : 64 /* AssertES2017 */;
                }
                // function declarations with object rest destructuring are ES2018 syntax
                if (subtreeFlags & 16384 /* ContainsObjectRestOrSpread */) {
                    transformFlags |= 32 /* AssertES2018 */;
                }
                // If a FunctionDeclaration is generator function and is the body of a
                // transformed async function, then this node can be transformed to a
                // down-level generator.
                // Currently we do not support transforming any other generator functions
                // down level.
                if (node.asteriskToken) {
                    transformFlags |= 512 /* AssertGenerator */;
                }
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~538925056 /* FunctionExcludes */;
        }
        function computeFunctionExpression(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            // TypeScript-specific modifiers, type parameters, and type annotations are TypeScript
            // syntax.
            if (ts.hasModifier(node, 2270 /* TypeScriptModifier */)
                || node.typeParameters
                || node.type) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            // An async function expression is ES2017 syntax.
            if (ts.hasModifier(node, 256 /* Async */)) {
                transformFlags |= node.asteriskToken ? 32 /* AssertES2018 */ : 64 /* AssertES2017 */;
            }
            // function expressions with object rest destructuring are ES2018 syntax
            if (subtreeFlags & 16384 /* ContainsObjectRestOrSpread */) {
                transformFlags |= 32 /* AssertES2018 */;
            }
            // If a FunctionExpression is generator function and is the body of a
            // transformed async function, then this node can be transformed to a
            // down-level generator.
            if (node.asteriskToken) {
                transformFlags |= 512 /* AssertGenerator */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~538925056 /* FunctionExcludes */;
        }
        function computeArrowFunction(node, subtreeFlags) {
            // An ArrowFunction is ES6 syntax, and excludes markers that should not escape the scope of an ArrowFunction.
            var transformFlags = subtreeFlags | 256 /* AssertES2015 */;
            // TypeScript-specific modifiers, type parameters, and type annotations are TypeScript
            // syntax.
            if (ts.hasModifier(node, 2270 /* TypeScriptModifier */)
                || node.typeParameters
                || node.type) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            // An async arrow function is ES2017 syntax.
            if (ts.hasModifier(node, 256 /* Async */)) {
                transformFlags |= 64 /* AssertES2017 */;
            }
            // arrow functions with object rest destructuring are ES2018 syntax
            if (subtreeFlags & 16384 /* ContainsObjectRestOrSpread */) {
                transformFlags |= 32 /* AssertES2018 */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~538920960 /* ArrowFunctionExcludes */;
        }
        function computePropertyAccess(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            if (node.flags & 32 /* OptionalChain */) {
                transformFlags |= 8 /* ContainsES2020 */;
            }
            // If a PropertyAccessExpression starts with a super keyword, then it is
            // ES6 syntax, and requires a lexical `this` binding.
            if (node.expression.kind === 102 /* SuperKeyword */) {
                // super inside of an async function requires hoisting the super access (ES2017).
                // same for super inside of an async generator, which is ES2018.
                transformFlags |= 64 /* ContainsES2017 */ | 32 /* ContainsES2018 */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* PropertyAccessExcludes */;
        }
        function computeElementAccess(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            if (node.flags & 32 /* OptionalChain */) {
                transformFlags |= 8 /* ContainsES2020 */;
            }
            // If an ElementAccessExpression starts with a super keyword, then it is
            // ES6 syntax, and requires a lexical `this` binding.
            if (node.expression.kind === 102 /* SuperKeyword */) {
                // super inside of an async function requires hoisting the super access (ES2017).
                // same for super inside of an async generator, which is ES2018.
                transformFlags |= 64 /* ContainsES2017 */ | 32 /* ContainsES2018 */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* PropertyAccessExcludes */;
        }
        function computeVariableDeclaration(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            transformFlags |= 256 /* AssertES2015 */ | 131072 /* ContainsBindingPattern */; // TODO(rbuckton): Why are these set unconditionally?
            // A VariableDeclaration containing ObjectRest is ES2018 syntax
            if (subtreeFlags & 16384 /* ContainsObjectRestOrSpread */) {
                transformFlags |= 32 /* AssertES2018 */;
            }
            // Type annotations are TypeScript syntax.
            if (node.type || node.exclamationToken) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* NodeExcludes */;
        }
        function computeVariableStatement(node, subtreeFlags) {
            var transformFlags;
            var declarationListTransformFlags = node.declarationList.transformFlags;
            // An ambient declaration is TypeScript syntax.
            if (ts.hasModifier(node, 2 /* Ambient */)) {
                transformFlags = 1 /* AssertTypeScript */;
            }
            else {
                transformFlags = subtreeFlags;
                if (declarationListTransformFlags & 131072 /* ContainsBindingPattern */) {
                    transformFlags |= 256 /* AssertES2015 */;
                }
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* NodeExcludes */;
        }
        function computeLabeledStatement(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            // A labeled statement containing a block scoped binding *may* need to be transformed from ES6.
            if (subtreeFlags & 65536 /* ContainsBlockScopedBinding */
                && ts.isIterationStatement(node, /*lookInLabeledStatements*/ true)) {
                transformFlags |= 256 /* AssertES2015 */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* NodeExcludes */;
        }
        function computeImportEquals(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            // An ImportEqualsDeclaration with a namespace reference is TypeScript.
            if (!ts.isExternalModuleImportEqualsDeclaration(node)) {
                transformFlags |= 1 /* AssertTypeScript */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* NodeExcludes */;
        }
        function computeExpressionStatement(node, subtreeFlags) {
            var transformFlags = subtreeFlags;
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~536870912 /* NodeExcludes */;
        }
        function computeModuleDeclaration(node, subtreeFlags) {
            var transformFlags = 1 /* AssertTypeScript */;
            var modifierFlags = ts.getModifierFlags(node);
            if ((modifierFlags & 2 /* Ambient */) === 0) {
                transformFlags |= subtreeFlags;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~537991168 /* ModuleExcludes */;
        }
        function computeVariableDeclarationList(node, subtreeFlags) {
            var transformFlags = subtreeFlags | 1048576 /* ContainsHoistedDeclarationOrCompletion */;
            if (subtreeFlags & 131072 /* ContainsBindingPattern */) {
                transformFlags |= 256 /* AssertES2015 */;
            }
            // If a VariableDeclarationList is `let` or `const`, then it is ES6 syntax.
            if (node.flags & 3 /* BlockScoped */) {
                transformFlags |= 256 /* AssertES2015 */ | 65536 /* ContainsBlockScopedBinding */;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~537018368 /* VariableDeclarationListExcludes */;
        }
        function computeOther(node, kind, subtreeFlags) {
            // Mark transformations needed for each node
            var transformFlags = subtreeFlags;
            var excludeFlags = 536870912 /* NodeExcludes */;
            switch (kind) {
                case 126 /* AsyncKeyword */:
                    // async is ES2017 syntax, but may be ES2018 syntax (for async generators)
                    transformFlags |= 32 /* AssertES2018 */ | 64 /* AssertES2017 */;
                    break;
                case 206 /* AwaitExpression */:
                    // await is ES2017 syntax, but may be ES2018 syntax (for async generators)
                    transformFlags |= 32 /* AssertES2018 */ | 64 /* AssertES2017 */ | 524288 /* ContainsAwait */;
                    break;
                case 199 /* TypeAssertionExpression */:
                case 217 /* AsExpression */:
                case 325 /* PartiallyEmittedExpression */:
                    // These nodes are TypeScript syntax.
                    transformFlags |= 1 /* AssertTypeScript */;
                    excludeFlags = 536870912 /* OuterExpressionExcludes */;
                    break;
                case 119 /* PublicKeyword */:
                case 117 /* PrivateKeyword */:
                case 118 /* ProtectedKeyword */:
                case 122 /* AbstractKeyword */:
                case 130 /* DeclareKeyword */:
                case 81 /* ConstKeyword */:
                case 248 /* EnumDeclaration */:
                case 284 /* EnumMember */:
                case 218 /* NonNullExpression */:
                case 138 /* ReadonlyKeyword */:
                    // These nodes are TypeScript syntax.
                    transformFlags |= 1 /* AssertTypeScript */;
                    break;
                case 266 /* JsxElement */:
                case 267 /* JsxSelfClosingElement */:
                case 268 /* JsxOpeningElement */:
                case 11 /* JsxText */:
                case 269 /* JsxClosingElement */:
                case 270 /* JsxFragment */:
                case 271 /* JsxOpeningFragment */:
                case 272 /* JsxClosingFragment */:
                case 273 /* JsxAttribute */:
                case 274 /* JsxAttributes */:
                case 275 /* JsxSpreadAttribute */:
                case 276 /* JsxExpression */:
                    // These nodes are Jsx syntax.
                    transformFlags |= 2 /* AssertJsx */;
                    break;
                case 14 /* NoSubstitutionTemplateLiteral */:
                case 15 /* TemplateHead */:
                case 16 /* TemplateMiddle */:
                case 17 /* TemplateTail */:
                case 211 /* TemplateExpression */:
                case 198 /* TaggedTemplateExpression */:
                case 282 /* ShorthandPropertyAssignment */:
                case 120 /* StaticKeyword */:
                case 219 /* MetaProperty */:
                    // These nodes are ES6 syntax.
                    transformFlags |= 256 /* AssertES2015 */;
                    break;
                case 10 /* StringLiteral */:
                    if (node.hasExtendedUnicodeEscape) {
                        transformFlags |= 256 /* AssertES2015 */;
                    }
                    break;
                case 8 /* NumericLiteral */:
                    if (node.numericLiteralFlags & 384 /* BinaryOrOctalSpecifier */) {
                        transformFlags |= 256 /* AssertES2015 */;
                    }
                    break;
                case 9 /* BigIntLiteral */:
                    transformFlags |= 4 /* AssertESNext */;
                    break;
                case 232 /* ForOfStatement */:
                    // This node is either ES2015 syntax or ES2017 syntax (if it is a for-await-of).
                    if (node.awaitModifier) {
                        transformFlags |= 32 /* AssertES2018 */;
                    }
                    transformFlags |= 256 /* AssertES2015 */;
                    break;
                case 212 /* YieldExpression */:
                    // This node is either ES2015 syntax (in a generator) or ES2017 syntax (in an async
                    // generator).
                    transformFlags |= 32 /* AssertES2018 */ | 256 /* AssertES2015 */ | 262144 /* ContainsYield */;
                    break;
                case 125 /* AnyKeyword */:
                case 140 /* NumberKeyword */:
                case 151 /* BigIntKeyword */:
                case 137 /* NeverKeyword */:
                case 141 /* ObjectKeyword */:
                case 143 /* StringKeyword */:
                case 128 /* BooleanKeyword */:
                case 144 /* SymbolKeyword */:
                case 110 /* VoidKeyword */:
                case 155 /* TypeParameter */:
                case 158 /* PropertySignature */:
                case 160 /* MethodSignature */:
                case 165 /* CallSignature */:
                case 166 /* ConstructSignature */:
                case 167 /* IndexSignature */:
                case 168 /* TypePredicate */:
                case 169 /* TypeReference */:
                case 170 /* FunctionType */:
                case 171 /* ConstructorType */:
                case 172 /* TypeQuery */:
                case 173 /* TypeLiteral */:
                case 174 /* ArrayType */:
                case 175 /* TupleType */:
                case 176 /* OptionalType */:
                case 177 /* RestType */:
                case 178 /* UnionType */:
                case 179 /* IntersectionType */:
                case 180 /* ConditionalType */:
                case 181 /* InferType */:
                case 182 /* ParenthesizedType */:
                case 246 /* InterfaceDeclaration */:
                case 247 /* TypeAliasDeclaration */:
                case 183 /* ThisType */:
                case 184 /* TypeOperator */:
                case 185 /* IndexedAccessType */:
                case 186 /* MappedType */:
                case 187 /* LiteralType */:
                case 252 /* NamespaceExportDeclaration */:
                    // Types and signatures are TypeScript syntax, and exclude all other facts.
                    transformFlags = 1 /* AssertTypeScript */;
                    excludeFlags = -2 /* TypeExcludes */;
                    break;
                case 154 /* ComputedPropertyName */:
                    // Even though computed property names are ES6, we don't treat them as such.
                    // This is so that they can flow through PropertyName transforms unaffected.
                    // Instead, we mark the container as ES6, so that it can properly handle the transform.
                    transformFlags |= 32768 /* ContainsComputedPropertyName */;
                    break;
                case 213 /* SpreadElement */:
                    transformFlags |= 256 /* AssertES2015 */ | 8192 /* ContainsRestOrSpread */;
                    break;
                case 283 /* SpreadAssignment */:
                    transformFlags |= 32 /* AssertES2018 */ | 16384 /* ContainsObjectRestOrSpread */;
                    break;
                case 102 /* SuperKeyword */:
                    // This node is ES6 syntax.
                    transformFlags |= 256 /* AssertES2015 */;
                    excludeFlags = 536870912 /* OuterExpressionExcludes */; // must be set to persist `Super`
                    break;
                case 104 /* ThisKeyword */:
                    // Mark this node and its ancestors as containing a lexical `this` keyword.
                    transformFlags |= 4096 /* ContainsLexicalThis */;
                    break;
                case 189 /* ObjectBindingPattern */:
                    transformFlags |= 256 /* AssertES2015 */ | 131072 /* ContainsBindingPattern */;
                    if (subtreeFlags & 8192 /* ContainsRestOrSpread */) {
                        transformFlags |= 32 /* AssertES2018 */ | 16384 /* ContainsObjectRestOrSpread */;
                    }
                    excludeFlags = 536879104 /* BindingPatternExcludes */;
                    break;
                case 190 /* ArrayBindingPattern */:
                    transformFlags |= 256 /* AssertES2015 */ | 131072 /* ContainsBindingPattern */;
                    excludeFlags = 536879104 /* BindingPatternExcludes */;
                    break;
                case 191 /* BindingElement */:
                    transformFlags |= 256 /* AssertES2015 */;
                    if (node.dotDotDotToken) {
                        transformFlags |= 8192 /* ContainsRestOrSpread */;
                    }
                    break;
                case 157 /* Decorator */:
                    // This node is TypeScript syntax, and marks its container as also being TypeScript syntax.
                    transformFlags |= 1 /* AssertTypeScript */ | 2048 /* ContainsTypeScriptClassSyntax */;
                    break;
                case 193 /* ObjectLiteralExpression */:
                    excludeFlags = 536922112 /* ObjectLiteralExcludes */;
                    if (subtreeFlags & 32768 /* ContainsComputedPropertyName */) {
                        // If an ObjectLiteralExpression contains a ComputedPropertyName, then it
                        // is an ES6 node.
                        transformFlags |= 256 /* AssertES2015 */;
                    }
                    if (subtreeFlags & 16384 /* ContainsObjectRestOrSpread */) {
                        // If an ObjectLiteralExpression contains a spread element, then it
                        // is an ES2018 node.
                        transformFlags |= 32 /* AssertES2018 */;
                    }
                    break;
                case 192 /* ArrayLiteralExpression */:
                    excludeFlags = 536879104 /* ArrayLiteralOrCallOrNewExcludes */;
                    break;
                case 228 /* DoStatement */:
                case 229 /* WhileStatement */:
                case 230 /* ForStatement */:
                case 231 /* ForInStatement */:
                    // A loop containing a block scoped binding *may* need to be transformed from ES6.
                    if (subtreeFlags & 65536 /* ContainsBlockScopedBinding */) {
                        transformFlags |= 256 /* AssertES2015 */;
                    }
                    break;
                case 290 /* SourceFile */:
                    break;
                case 262 /* NamespaceExport */:
                    transformFlags |= 4 /* AssertESNext */;
                    break;
                case 235 /* ReturnStatement */:
                    // Return statements may require an `await` in ES2018.
                    transformFlags |= 1048576 /* ContainsHoistedDeclarationOrCompletion */ | 32 /* AssertES2018 */;
                    break;
                case 233 /* ContinueStatement */:
                case 234 /* BreakStatement */:
                    transformFlags |= 1048576 /* ContainsHoistedDeclarationOrCompletion */;
                    break;
                case 76 /* PrivateIdentifier */:
                    transformFlags |= 4194304 /* ContainsClassFields */;
                    break;
            }
            node.transformFlags = transformFlags | 536870912 /* HasComputedFlags */;
            return transformFlags & ~excludeFlags;
        }
        function propagatePropertyNameFlags(node, transformFlags) {
            return transformFlags | (node.transformFlags & 4096 /* PropertyNamePropagatingFlags */);
        }
        /**
         * Gets the transform flags to exclude when unioning the transform flags of a subtree.
         *
         * NOTE: This needs to be kept up-to-date with the exclusions used in `computeTransformFlagsForNode`.
         *       For performance reasons, `computeTransformFlagsForNode` uses local constant values rather
         *       than calling this function.
         */
        function getTransformFlagsSubtreeExclusions(kind) {
            if (kind >= 168 /* FirstTypeNode */ && kind <= 188 /* LastTypeNode */) {
                return -2 /* TypeExcludes */;
            }
            switch (kind) {
                case 196 /* CallExpression */:
                case 197 /* NewExpression */:
                case 192 /* ArrayLiteralExpression */:
                    return 536879104 /* ArrayLiteralOrCallOrNewExcludes */;
                case 249 /* ModuleDeclaration */:
                    return 537991168 /* ModuleExcludes */;
                case 156 /* Parameter */:
                    return 536870912 /* ParameterExcludes */;
                case 202 /* ArrowFunction */:
                    return 538920960 /* ArrowFunctionExcludes */;
                case 201 /* FunctionExpression */:
                case 244 /* FunctionDeclaration */:
                    return 538925056 /* FunctionExcludes */;
                case 243 /* VariableDeclarationList */:
                    return 537018368 /* VariableDeclarationListExcludes */;
                case 245 /* ClassDeclaration */:
                case 214 /* ClassExpression */:
                    return 536905728 /* ClassExcludes */;
                case 162 /* Constructor */:
                    return 538923008 /* ConstructorExcludes */;
                case 161 /* MethodDeclaration */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                    return 538923008 /* MethodOrAccessorExcludes */;
                case 125 /* AnyKeyword */:
                case 140 /* NumberKeyword */:
                case 151 /* BigIntKeyword */:
                case 137 /* NeverKeyword */:
                case 143 /* StringKeyword */:
                case 141 /* ObjectKeyword */:
                case 128 /* BooleanKeyword */:
                case 144 /* SymbolKeyword */:
                case 110 /* VoidKeyword */:
                case 155 /* TypeParameter */:
                case 158 /* PropertySignature */:
                case 160 /* MethodSignature */:
                case 165 /* CallSignature */:
                case 166 /* ConstructSignature */:
                case 167 /* IndexSignature */:
                case 246 /* InterfaceDeclaration */:
                case 247 /* TypeAliasDeclaration */:
                    return -2 /* TypeExcludes */;
                case 193 /* ObjectLiteralExpression */:
                    return 536922112 /* ObjectLiteralExcludes */;
                case 280 /* CatchClause */:
                    return 536887296 /* CatchClauseExcludes */;
                case 189 /* ObjectBindingPattern */:
                case 190 /* ArrayBindingPattern */:
                    return 536879104 /* BindingPatternExcludes */;
                case 199 /* TypeAssertionExpression */:
                case 217 /* AsExpression */:
                case 325 /* PartiallyEmittedExpression */:
                case 200 /* ParenthesizedExpression */:
                case 102 /* SuperKeyword */:
                    return 536870912 /* OuterExpressionExcludes */;
                case 194 /* PropertyAccessExpression */:
                case 195 /* ElementAccessExpression */:
                    return 536870912 /* PropertyAccessExcludes */;
                default:
                    return 536870912 /* NodeExcludes */;
            }
        }
        ts.getTransformFlagsSubtreeExclusions = getTransformFlagsSubtreeExclusions;
        /**
         * "Binds" JSDoc nodes in TypeScript code.
         * Since we will never create symbols for JSDoc, we just set parent pointers instead.
         */
        function setParentPointers(parent, child) {
            child.parent = parent;
            ts.forEachChild(child, function (grandchild) { return setParentPointers(child, grandchild); });
        }
    })(ts || (ts = {}));
    /** @internal */
    var ts;
    (function (ts) {
        function createGetSymbolWalker(getRestTypeOfSignature, getTypePredicateOfSignature, getReturnTypeOfSignature, getBaseTypes, resolveStructuredTypeMembers, getTypeOfSymbol, getResolvedSymbol, getIndexTypeOfStructuredType, getConstraintOfTypeParameter, getFirstIdentifier, getTypeArguments) {
            return getSymbolWalker;
            function getSymbolWalker(accept) {
                if (accept === void 0) {
                    accept = function () { return true; };
                }
                var visitedTypes = []; // Sparse array from id to type
                var visitedSymbols = []; // Sparse array from id to symbol
                return {
                    walkType: function (type) {
                        try {
                            visitType(type);
                            return { visitedTypes: ts.getOwnValues(visitedTypes), visitedSymbols: ts.getOwnValues(visitedSymbols) };
                        }
                        finally {
                            ts.clear(visitedTypes);
                            ts.clear(visitedSymbols);
                        }
                    },
                    walkSymbol: function (symbol) {
                        try {
                            visitSymbol(symbol);
                            return { visitedTypes: ts.getOwnValues(visitedTypes), visitedSymbols: ts.getOwnValues(visitedSymbols) };
                        }
                        finally {
                            ts.clear(visitedTypes);
                            ts.clear(visitedSymbols);
                        }
                    },
                };
                function visitType(type) {
                    if (!type) {
                        return;
                    }
                    if (visitedTypes[type.id]) {
                        return;
                    }
                    visitedTypes[type.id] = type;
                    // Reuse visitSymbol to visit the type's symbol,
                    //  but be sure to bail on recuring into the type if accept declines the symbol.
                    var shouldBail = visitSymbol(type.symbol);
                    if (shouldBail)
                        return;
                    // Visit the type's related types, if any
                    if (type.flags & 524288 /* Object */) {
                        var objectType = type;
                        var objectFlags = objectType.objectFlags;
                        if (objectFlags & 4 /* Reference */) {
                            visitTypeReference(type);
                        }
                        if (objectFlags & 32 /* Mapped */) {
                            visitMappedType(type);
                        }
                        if (objectFlags & (1 /* Class */ | 2 /* Interface */)) {
                            visitInterfaceType(type);
                        }
                        if (objectFlags & (8 /* Tuple */ | 16 /* Anonymous */)) {
                            visitObjectType(objectType);
                        }
                    }
                    if (type.flags & 262144 /* TypeParameter */) {
                        visitTypeParameter(type);
                    }
                    if (type.flags & 3145728 /* UnionOrIntersection */) {
                        visitUnionOrIntersectionType(type);
                    }
                    if (type.flags & 4194304 /* Index */) {
                        visitIndexType(type);
                    }
                    if (type.flags & 8388608 /* IndexedAccess */) {
                        visitIndexedAccessType(type);
                    }
                }
                function visitTypeReference(type) {
                    visitType(type.target);
                    ts.forEach(getTypeArguments(type), visitType);
                }
                function visitTypeParameter(type) {
                    visitType(getConstraintOfTypeParameter(type));
                }
                function visitUnionOrIntersectionType(type) {
                    ts.forEach(type.types, visitType);
                }
                function visitIndexType(type) {
                    visitType(type.type);
                }
                function visitIndexedAccessType(type) {
                    visitType(type.objectType);
                    visitType(type.indexType);
                    visitType(type.constraint);
                }
                function visitMappedType(type) {
                    visitType(type.typeParameter);
                    visitType(type.constraintType);
                    visitType(type.templateType);
                    visitType(type.modifiersType);
                }
                function visitSignature(signature) {
                    var typePredicate = getTypePredicateOfSignature(signature);
                    if (typePredicate) {
                        visitType(typePredicate.type);
                    }
                    ts.forEach(signature.typeParameters, visitType);
                    for (var _i = 0, _a = signature.parameters; _i < _a.length; _i++) {
                        var parameter = _a[_i];
                        visitSymbol(parameter);
                    }
                    visitType(getRestTypeOfSignature(signature));
                    visitType(getReturnTypeOfSignature(signature));
                }
                function visitInterfaceType(interfaceT) {
                    visitObjectType(interfaceT);
                    ts.forEach(interfaceT.typeParameters, visitType);
                    ts.forEach(getBaseTypes(interfaceT), visitType);
                    visitType(interfaceT.thisType);
                }
                function visitObjectType(type) {
                    var stringIndexType = getIndexTypeOfStructuredType(type, 0 /* String */);
                    visitType(stringIndexType);
                    var numberIndexType = getIndexTypeOfStructuredType(type, 1 /* Number */);
                    visitType(numberIndexType);
                    // The two checks above *should* have already resolved the type (if needed), so this should be cached
                    var resolved = resolveStructuredTypeMembers(type);
                    for (var _i = 0, _a = resolved.callSignatures; _i < _a.length; _i++) {
                        var signature = _a[_i];
                        visitSignature(signature);
                    }
                    for (var _b = 0, _c = resolved.constructSignatures; _b < _c.length; _b++) {
                        var signature = _c[_b];
                        visitSignature(signature);
                    }
                    for (var _d = 0, _e = resolved.properties; _d < _e.length; _d++) {
                        var p = _e[_d];
                        visitSymbol(p);
                    }
                }
                function visitSymbol(symbol) {
                    if (!symbol) {
                        return false;
                    }
                    var symbolId = ts.getSymbolId(symbol);
                    if (visitedSymbols[symbolId]) {
                        return false;
                    }
                    visitedSymbols[symbolId] = symbol;
                    if (!accept(symbol)) {
                        return true;
                    }
                    var t = getTypeOfSymbol(symbol);
                    visitType(t); // Should handle members on classes and such
                    if (symbol.exports) {
                        symbol.exports.forEach(visitSymbol);
                    }
                    ts.forEach(symbol.declarations, function (d) {
                        // Type queries are too far resolved when we just visit the symbol's type
                        //  (their type resolved directly to the member deeply referenced)
                        // So to get the intervening symbols, we need to check if there's a type
                        // query node on any of the symbol's declarations and get symbols there
                        if (d.type && d.type.kind === 172 /* TypeQuery */) {
                            var query = d.type;
                            var entity = getResolvedSymbol(getFirstIdentifier(query.exprName));
                            visitSymbol(entity);
                        }
                    });
                    return false;
                }
            }
        }
        ts.createGetSymbolWalker = createGetSymbolWalker;
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var ambientModuleSymbolRegex = /^".+"$/;
        var anon = "(anonymous)";
        var nextSymbolId = 1;
        var nextNodeId = 1;
        var nextMergeId = 1;
        var nextFlowId = 1;
        var IterationUse;
        (function (IterationUse) {
            IterationUse[IterationUse["AllowsSyncIterablesFlag"] = 1] = "AllowsSyncIterablesFlag";
            IterationUse[IterationUse["AllowsAsyncIterablesFlag"] = 2] = "AllowsAsyncIterablesFlag";
            IterationUse[IterationUse["AllowsStringInputFlag"] = 4] = "AllowsStringInputFlag";
            IterationUse[IterationUse["ForOfFlag"] = 8] = "ForOfFlag";
            IterationUse[IterationUse["YieldStarFlag"] = 16] = "YieldStarFlag";
            IterationUse[IterationUse["SpreadFlag"] = 32] = "SpreadFlag";
            IterationUse[IterationUse["DestructuringFlag"] = 64] = "DestructuringFlag";
            // Spread, Destructuring, Array element assignment
            IterationUse[IterationUse["Element"] = 1] = "Element";
            IterationUse[IterationUse["Spread"] = 33] = "Spread";
            IterationUse[IterationUse["Destructuring"] = 65] = "Destructuring";
            IterationUse[IterationUse["ForOf"] = 13] = "ForOf";
            IterationUse[IterationUse["ForAwaitOf"] = 15] = "ForAwaitOf";
            IterationUse[IterationUse["YieldStar"] = 17] = "YieldStar";
            IterationUse[IterationUse["AsyncYieldStar"] = 19] = "AsyncYieldStar";
            IterationUse[IterationUse["GeneratorReturnType"] = 1] = "GeneratorReturnType";
            IterationUse[IterationUse["AsyncGeneratorReturnType"] = 2] = "AsyncGeneratorReturnType";
        })(IterationUse || (IterationUse = {}));
        var IterationTypeKind;
        (function (IterationTypeKind) {
            IterationTypeKind[IterationTypeKind["Yield"] = 0] = "Yield";
            IterationTypeKind[IterationTypeKind["Return"] = 1] = "Return";
            IterationTypeKind[IterationTypeKind["Next"] = 2] = "Next";
        })(IterationTypeKind || (IterationTypeKind = {}));
        var WideningKind;
        (function (WideningKind) {
            WideningKind[WideningKind["Normal"] = 0] = "Normal";
            WideningKind[WideningKind["GeneratorYield"] = 1] = "GeneratorYield";
        })(WideningKind || (WideningKind = {}));
        var TypeFacts;
        (function (TypeFacts) {
            TypeFacts[TypeFacts["None"] = 0] = "None";
            TypeFacts[TypeFacts["TypeofEQString"] = 1] = "TypeofEQString";
            TypeFacts[TypeFacts["TypeofEQNumber"] = 2] = "TypeofEQNumber";
            TypeFacts[TypeFacts["TypeofEQBigInt"] = 4] = "TypeofEQBigInt";
            TypeFacts[TypeFacts["TypeofEQBoolean"] = 8] = "TypeofEQBoolean";
            TypeFacts[TypeFacts["TypeofEQSymbol"] = 16] = "TypeofEQSymbol";
            TypeFacts[TypeFacts["TypeofEQObject"] = 32] = "TypeofEQObject";
            TypeFacts[TypeFacts["TypeofEQFunction"] = 64] = "TypeofEQFunction";
            TypeFacts[TypeFacts["TypeofEQHostObject"] = 128] = "TypeofEQHostObject";
            TypeFacts[TypeFacts["TypeofNEString"] = 256] = "TypeofNEString";
            TypeFacts[TypeFacts["TypeofNENumber"] = 512] = "TypeofNENumber";
            TypeFacts[TypeFacts["TypeofNEBigInt"] = 1024] = "TypeofNEBigInt";
            TypeFacts[TypeFacts["TypeofNEBoolean"] = 2048] = "TypeofNEBoolean";
            TypeFacts[TypeFacts["TypeofNESymbol"] = 4096] = "TypeofNESymbol";
            TypeFacts[TypeFacts["TypeofNEObject"] = 8192] = "TypeofNEObject";
            TypeFacts[TypeFacts["TypeofNEFunction"] = 16384] = "TypeofNEFunction";
            TypeFacts[TypeFacts["TypeofNEHostObject"] = 32768] = "TypeofNEHostObject";
            TypeFacts[TypeFacts["EQUndefined"] = 65536] = "EQUndefined";
            TypeFacts[TypeFacts["EQNull"] = 131072] = "EQNull";
            TypeFacts[TypeFacts["EQUndefinedOrNull"] = 262144] = "EQUndefinedOrNull";
            TypeFacts[TypeFacts["NEUndefined"] = 524288] = "NEUndefined";
            TypeFacts[TypeFacts["NENull"] = 1048576] = "NENull";
            TypeFacts[TypeFacts["NEUndefinedOrNull"] = 2097152] = "NEUndefinedOrNull";
            TypeFacts[TypeFacts["Truthy"] = 4194304] = "Truthy";
            TypeFacts[TypeFacts["Falsy"] = 8388608] = "Falsy";
            TypeFacts[TypeFacts["All"] = 16777215] = "All";
            // The following members encode facts about particular kinds of types for use in the getTypeFacts function.
            // The presence of a particular fact means that the given test is true for some (and possibly all) values
            // of that kind of type.
            TypeFacts[TypeFacts["BaseStringStrictFacts"] = 3735041] = "BaseStringStrictFacts";
            TypeFacts[TypeFacts["BaseStringFacts"] = 12582401] = "BaseStringFacts";
            TypeFacts[TypeFacts["StringStrictFacts"] = 16317953] = "StringStrictFacts";
            TypeFacts[TypeFacts["StringFacts"] = 16776705] = "StringFacts";
            TypeFacts[TypeFacts["EmptyStringStrictFacts"] = 12123649] = "EmptyStringStrictFacts";
            TypeFacts[TypeFacts["EmptyStringFacts"] = 12582401] = "EmptyStringFacts";
            TypeFacts[TypeFacts["NonEmptyStringStrictFacts"] = 7929345] = "NonEmptyStringStrictFacts";
            TypeFacts[TypeFacts["NonEmptyStringFacts"] = 16776705] = "NonEmptyStringFacts";
            TypeFacts[TypeFacts["BaseNumberStrictFacts"] = 3734786] = "BaseNumberStrictFacts";
            TypeFacts[TypeFacts["BaseNumberFacts"] = 12582146] = "BaseNumberFacts";
            TypeFacts[TypeFacts["NumberStrictFacts"] = 16317698] = "NumberStrictFacts";
            TypeFacts[TypeFacts["NumberFacts"] = 16776450] = "NumberFacts";
            TypeFacts[TypeFacts["ZeroNumberStrictFacts"] = 12123394] = "ZeroNumberStrictFacts";
            TypeFacts[TypeFacts["ZeroNumberFacts"] = 12582146] = "ZeroNumberFacts";
            TypeFacts[TypeFacts["NonZeroNumberStrictFacts"] = 7929090] = "NonZeroNumberStrictFacts";
            TypeFacts[TypeFacts["NonZeroNumberFacts"] = 16776450] = "NonZeroNumberFacts";
            TypeFacts[TypeFacts["BaseBigIntStrictFacts"] = 3734276] = "BaseBigIntStrictFacts";
            TypeFacts[TypeFacts["BaseBigIntFacts"] = 12581636] = "BaseBigIntFacts";
            TypeFacts[TypeFacts["BigIntStrictFacts"] = 16317188] = "BigIntStrictFacts";
            TypeFacts[TypeFacts["BigIntFacts"] = 16775940] = "BigIntFacts";
            TypeFacts[TypeFacts["ZeroBigIntStrictFacts"] = 12122884] = "ZeroBigIntStrictFacts";
            TypeFacts[TypeFacts["ZeroBigIntFacts"] = 12581636] = "ZeroBigIntFacts";
            TypeFacts[TypeFacts["NonZeroBigIntStrictFacts"] = 7928580] = "NonZeroBigIntStrictFacts";
            TypeFacts[TypeFacts["NonZeroBigIntFacts"] = 16775940] = "NonZeroBigIntFacts";
            TypeFacts[TypeFacts["BaseBooleanStrictFacts"] = 3733256] = "BaseBooleanStrictFacts";
            TypeFacts[TypeFacts["BaseBooleanFacts"] = 12580616] = "BaseBooleanFacts";
            TypeFacts[TypeFacts["BooleanStrictFacts"] = 16316168] = "BooleanStrictFacts";
            TypeFacts[TypeFacts["BooleanFacts"] = 16774920] = "BooleanFacts";
            TypeFacts[TypeFacts["FalseStrictFacts"] = 12121864] = "FalseStrictFacts";
            TypeFacts[TypeFacts["FalseFacts"] = 12580616] = "FalseFacts";
            TypeFacts[TypeFacts["TrueStrictFacts"] = 7927560] = "TrueStrictFacts";
            TypeFacts[TypeFacts["TrueFacts"] = 16774920] = "TrueFacts";
            TypeFacts[TypeFacts["SymbolStrictFacts"] = 7925520] = "SymbolStrictFacts";
            TypeFacts[TypeFacts["SymbolFacts"] = 16772880] = "SymbolFacts";
            TypeFacts[TypeFacts["ObjectStrictFacts"] = 7888800] = "ObjectStrictFacts";
            TypeFacts[TypeFacts["ObjectFacts"] = 16736160] = "ObjectFacts";
            TypeFacts[TypeFacts["FunctionStrictFacts"] = 7880640] = "FunctionStrictFacts";
            TypeFacts[TypeFacts["FunctionFacts"] = 16728000] = "FunctionFacts";
            TypeFacts[TypeFacts["UndefinedFacts"] = 9830144] = "UndefinedFacts";
            TypeFacts[TypeFacts["NullFacts"] = 9363232] = "NullFacts";
            TypeFacts[TypeFacts["EmptyObjectStrictFacts"] = 16318463] = "EmptyObjectStrictFacts";
            TypeFacts[TypeFacts["EmptyObjectFacts"] = 16777215] = "EmptyObjectFacts";
        })(TypeFacts || (TypeFacts = {}));
        var typeofEQFacts = ts.createMapFromTemplate({
            string: 1 /* TypeofEQString */,
            number: 2 /* TypeofEQNumber */,
            bigint: 4 /* TypeofEQBigInt */,
            boolean: 8 /* TypeofEQBoolean */,
            symbol: 16 /* TypeofEQSymbol */,
            undefined: 65536 /* EQUndefined */,
            object: 32 /* TypeofEQObject */,
            function: 64 /* TypeofEQFunction */
        });
        var typeofNEFacts = ts.createMapFromTemplate({
            string: 256 /* TypeofNEString */,
            number: 512 /* TypeofNENumber */,
            bigint: 1024 /* TypeofNEBigInt */,
            boolean: 2048 /* TypeofNEBoolean */,
            symbol: 4096 /* TypeofNESymbol */,
            undefined: 524288 /* NEUndefined */,
            object: 8192 /* TypeofNEObject */,
            function: 16384 /* TypeofNEFunction */
        });
        var TypeSystemPropertyName;
        (function (TypeSystemPropertyName) {
            TypeSystemPropertyName[TypeSystemPropertyName["Type"] = 0] = "Type";
            TypeSystemPropertyName[TypeSystemPropertyName["ResolvedBaseConstructorType"] = 1] = "ResolvedBaseConstructorType";
            TypeSystemPropertyName[TypeSystemPropertyName["DeclaredType"] = 2] = "DeclaredType";
            TypeSystemPropertyName[TypeSystemPropertyName["ResolvedReturnType"] = 3] = "ResolvedReturnType";
            TypeSystemPropertyName[TypeSystemPropertyName["ImmediateBaseConstraint"] = 4] = "ImmediateBaseConstraint";
            TypeSystemPropertyName[TypeSystemPropertyName["EnumTagType"] = 5] = "EnumTagType";
            TypeSystemPropertyName[TypeSystemPropertyName["ResolvedTypeArguments"] = 6] = "ResolvedTypeArguments";
        })(TypeSystemPropertyName || (TypeSystemPropertyName = {}));
        var CheckMode;
        (function (CheckMode) {
            CheckMode[CheckMode["Normal"] = 0] = "Normal";
            CheckMode[CheckMode["Contextual"] = 1] = "Contextual";
            CheckMode[CheckMode["Inferential"] = 2] = "Inferential";
            CheckMode[CheckMode["SkipContextSensitive"] = 4] = "SkipContextSensitive";
            CheckMode[CheckMode["SkipGenericFunctions"] = 8] = "SkipGenericFunctions";
            CheckMode[CheckMode["IsForSignatureHelp"] = 16] = "IsForSignatureHelp";
        })(CheckMode || (CheckMode = {}));
        var AccessFlags;
        (function (AccessFlags) {
            AccessFlags[AccessFlags["None"] = 0] = "None";
            AccessFlags[AccessFlags["NoIndexSignatures"] = 1] = "NoIndexSignatures";
            AccessFlags[AccessFlags["Writing"] = 2] = "Writing";
            AccessFlags[AccessFlags["CacheSymbol"] = 4] = "CacheSymbol";
            AccessFlags[AccessFlags["NoTupleBoundsCheck"] = 8] = "NoTupleBoundsCheck";
        })(AccessFlags || (AccessFlags = {}));
        var SignatureCheckMode;
        (function (SignatureCheckMode) {
            SignatureCheckMode[SignatureCheckMode["BivariantCallback"] = 1] = "BivariantCallback";
            SignatureCheckMode[SignatureCheckMode["StrictCallback"] = 2] = "StrictCallback";
            SignatureCheckMode[SignatureCheckMode["IgnoreReturnTypes"] = 4] = "IgnoreReturnTypes";
            SignatureCheckMode[SignatureCheckMode["StrictArity"] = 8] = "StrictArity";
            SignatureCheckMode[SignatureCheckMode["Callback"] = 3] = "Callback";
        })(SignatureCheckMode || (SignatureCheckMode = {}));
        var IntersectionState;
        (function (IntersectionState) {
            IntersectionState[IntersectionState["None"] = 0] = "None";
            IntersectionState[IntersectionState["Source"] = 1] = "Source";
            IntersectionState[IntersectionState["Target"] = 2] = "Target";
        })(IntersectionState || (IntersectionState = {}));
        var MappedTypeModifiers;
        (function (MappedTypeModifiers) {
            MappedTypeModifiers[MappedTypeModifiers["IncludeReadonly"] = 1] = "IncludeReadonly";
            MappedTypeModifiers[MappedTypeModifiers["ExcludeReadonly"] = 2] = "ExcludeReadonly";
            MappedTypeModifiers[MappedTypeModifiers["IncludeOptional"] = 4] = "IncludeOptional";
            MappedTypeModifiers[MappedTypeModifiers["ExcludeOptional"] = 8] = "ExcludeOptional";
        })(MappedTypeModifiers || (MappedTypeModifiers = {}));
        var ExpandingFlags;
        (function (ExpandingFlags) {
            ExpandingFlags[ExpandingFlags["None"] = 0] = "None";
            ExpandingFlags[ExpandingFlags["Source"] = 1] = "Source";
            ExpandingFlags[ExpandingFlags["Target"] = 2] = "Target";
            ExpandingFlags[ExpandingFlags["Both"] = 3] = "Both";
        })(ExpandingFlags || (ExpandingFlags = {}));
        var MembersOrExportsResolutionKind;
        (function (MembersOrExportsResolutionKind) {
            MembersOrExportsResolutionKind["resolvedExports"] = "resolvedExports";
            MembersOrExportsResolutionKind["resolvedMembers"] = "resolvedMembers";
        })(MembersOrExportsResolutionKind || (MembersOrExportsResolutionKind = {}));
        var UnusedKind;
        (function (UnusedKind) {
            UnusedKind[UnusedKind["Local"] = 0] = "Local";
            UnusedKind[UnusedKind["Parameter"] = 1] = "Parameter";
        })(UnusedKind || (UnusedKind = {}));
        var isNotOverloadAndNotAccessor = ts.and(isNotOverload, isNotAccessor);
        var DeclarationMeaning;
        (function (DeclarationMeaning) {
            DeclarationMeaning[DeclarationMeaning["GetAccessor"] = 1] = "GetAccessor";
            DeclarationMeaning[DeclarationMeaning["SetAccessor"] = 2] = "SetAccessor";
            DeclarationMeaning[DeclarationMeaning["PropertyAssignment"] = 4] = "PropertyAssignment";
            DeclarationMeaning[DeclarationMeaning["Method"] = 8] = "Method";
            DeclarationMeaning[DeclarationMeaning["GetOrSetAccessor"] = 3] = "GetOrSetAccessor";
            DeclarationMeaning[DeclarationMeaning["PropertyAssignmentOrMethod"] = 12] = "PropertyAssignmentOrMethod";
        })(DeclarationMeaning || (DeclarationMeaning = {}));
        var DeclarationSpaces;
        (function (DeclarationSpaces) {
            DeclarationSpaces[DeclarationSpaces["None"] = 0] = "None";
            DeclarationSpaces[DeclarationSpaces["ExportValue"] = 1] = "ExportValue";
            DeclarationSpaces[DeclarationSpaces["ExportType"] = 2] = "ExportType";
            DeclarationSpaces[DeclarationSpaces["ExportNamespace"] = 4] = "ExportNamespace";
        })(DeclarationSpaces || (DeclarationSpaces = {}));
        function getNodeId(node) {
            if (!node.id) {
                node.id = nextNodeId;
                nextNodeId++;
            }
            return node.id;
        }
        ts.getNodeId = getNodeId;
        function getSymbolId(symbol) {
            if (!symbol.id) {
                symbol.id = nextSymbolId;
                nextSymbolId++;
            }
            return symbol.id;
        }
        ts.getSymbolId = getSymbolId;
        function isInstantiatedModule(node, preserveConstEnums) {
            var moduleState = ts.getModuleInstanceState(node);
            return moduleState === 1 /* Instantiated */ ||
                (preserveConstEnums && moduleState === 2 /* ConstEnumOnly */);
        }
        ts.isInstantiatedModule = isInstantiatedModule;
        function createTypeChecker(host, produceDiagnostics) {
            var getPackagesSet = ts.memoize(function () {
                var set = ts.createMap();
                host.getSourceFiles().forEach(function (sf) {
                    if (!sf.resolvedModules)
                        return;
                    ts.forEachEntry(sf.resolvedModules, function (r) {
                        if (r && r.packageId)
                            set.set(r.packageId.name, true);
                    });
                });
                return set;
            });
            // Cancellation that controls whether or not we can cancel in the middle of type checking.
            // In general cancelling is *not* safe for the type checker.  We might be in the middle of
            // computing something, and we will leave our internals in an inconsistent state.  Callers
            // who set the cancellation token should catch if a cancellation exception occurs, and
            // should throw away and create a new TypeChecker.
            //
            // Currently we only support setting the cancellation token when getting diagnostics.  This
            // is because diagnostics can be quite expensive, and we want to allow hosts to bail out if
            // they no longer need the information (for example, if the user started editing again).
            var cancellationToken;
            var requestedExternalEmitHelpers;
            var externalHelpersModule;
            var Symbol = ts.objectAllocator.getSymbolConstructor();
            var Type = ts.objectAllocator.getTypeConstructor();
            var Signature = ts.objectAllocator.getSignatureConstructor();
            var typeCount = 0;
            var symbolCount = 0;
            var enumCount = 0;
            var instantiationCount = 0;
            var instantiationDepth = 0;
            var constraintDepth = 0;
            var currentNode;
            var emptySymbols = ts.createSymbolTable();
            var identityMapper = ts.identity;
            var arrayVariances = [1 /* Covariant */];
            var compilerOptions = host.getCompilerOptions();
            var languageVersion = ts.getEmitScriptTarget(compilerOptions);
            var moduleKind = ts.getEmitModuleKind(compilerOptions);
            var allowSyntheticDefaultImports = ts.getAllowSyntheticDefaultImports(compilerOptions);
            var strictNullChecks = ts.getStrictOptionValue(compilerOptions, "strictNullChecks");
            var strictFunctionTypes = ts.getStrictOptionValue(compilerOptions, "strictFunctionTypes");
            var strictBindCallApply = ts.getStrictOptionValue(compilerOptions, "strictBindCallApply");
            var strictPropertyInitialization = ts.getStrictOptionValue(compilerOptions, "strictPropertyInitialization");
            var noImplicitAny = ts.getStrictOptionValue(compilerOptions, "noImplicitAny");
            var noImplicitThis = ts.getStrictOptionValue(compilerOptions, "noImplicitThis");
            var keyofStringsOnly = !!compilerOptions.keyofStringsOnly;
            var freshObjectLiteralFlag = compilerOptions.suppressExcessPropertyErrors ? 0 : 32768 /* FreshLiteral */;
            var emitResolver = createResolver();
            var nodeBuilder = createNodeBuilder();
            var globals = ts.createSymbolTable();
            var undefinedSymbol = createSymbol(4 /* Property */, "undefined");
            undefinedSymbol.declarations = [];
            var globalThisSymbol = createSymbol(1536 /* Module */, "globalThis", 8 /* Readonly */);
            globalThisSymbol.exports = globals;
            globalThisSymbol.declarations = [];
            globals.set(globalThisSymbol.escapedName, globalThisSymbol);
            var argumentsSymbol = createSymbol(4 /* Property */, "arguments");
            var requireSymbol = createSymbol(4 /* Property */, "require");
            /** This will be set during calls to `getResolvedSignature` where services determines an apparent number of arguments greater than what is actually provided. */
            var apparentArgumentCount;
            // for public members that accept a Node or one of its subtypes, we must guard against
            // synthetic nodes created during transformations by calling `getParseTreeNode`.
            // for most of these, we perform the guard only on `checker` to avoid any possible
            // extra cost of calling `getParseTreeNode` when calling these functions from inside the
            // checker.
            var checker = {
                getNodeCount: function () { return ts.sum(host.getSourceFiles(), "nodeCount"); },
                getIdentifierCount: function () { return ts.sum(host.getSourceFiles(), "identifierCount"); },
                getSymbolCount: function () { return ts.sum(host.getSourceFiles(), "symbolCount") + symbolCount; },
                getTypeCount: function () { return typeCount; },
                getRelationCacheSizes: function () {
                    return ({
                        assignable: assignableRelation.size,
                        identity: identityRelation.size,
                        subtype: subtypeRelation.size,
                        strictSubtype: strictSubtypeRelation.size,
                    });
                },
                isUndefinedSymbol: function (symbol) { return symbol === undefinedSymbol; },
                isArgumentsSymbol: function (symbol) { return symbol === argumentsSymbol; },
                isUnknownSymbol: function (symbol) { return symbol === unknownSymbol; },
                getMergedSymbol: getMergedSymbol,
                getDiagnostics: getDiagnostics,
                getGlobalDiagnostics: getGlobalDiagnostics,
                getTypeOfSymbolAtLocation: function (symbol, location) {
                    location = ts.getParseTreeNode(location);
                    return location ? getTypeOfSymbolAtLocation(symbol, location) : errorType;
                },
                getSymbolsOfParameterPropertyDeclaration: function (parameterIn, parameterName) {
                    var parameter = ts.getParseTreeNode(parameterIn, ts.isParameter);
                    if (parameter === undefined)
                        return ts.Debug.fail("Cannot get symbols of a synthetic parameter that cannot be resolved to a parse-tree node.");
                    return getSymbolsOfParameterPropertyDeclaration(parameter, ts.escapeLeadingUnderscores(parameterName));
                },
                getDeclaredTypeOfSymbol: getDeclaredTypeOfSymbol,
                getPropertiesOfType: getPropertiesOfType,
                getPropertyOfType: function (type, name) { return getPropertyOfType(type, ts.escapeLeadingUnderscores(name)); },
                getPrivateIdentifierPropertyOfType: function (leftType, name, location) {
                    var node = ts.getParseTreeNode(location);
                    if (!node) {
                        return undefined;
                    }
                    var propName = ts.escapeLeadingUnderscores(name);
                    var lexicallyScopedIdentifier = lookupSymbolForPrivateIdentifierDeclaration(propName, node);
                    return lexicallyScopedIdentifier ? getPrivateIdentifierPropertyOfType(leftType, lexicallyScopedIdentifier) : undefined;
                },
                getTypeOfPropertyOfType: function (type, name) { return getTypeOfPropertyOfType(type, ts.escapeLeadingUnderscores(name)); },
                getIndexInfoOfType: getIndexInfoOfType,
                getSignaturesOfType: getSignaturesOfType,
                getIndexTypeOfType: getIndexTypeOfType,
                getBaseTypes: getBaseTypes,
                getBaseTypeOfLiteralType: getBaseTypeOfLiteralType,
                getWidenedType: getWidenedType,
                getTypeFromTypeNode: function (nodeIn) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isTypeNode);
                    return node ? getTypeFromTypeNode(node) : errorType;
                },
                getParameterType: getTypeAtPosition,
                getPromisedTypeOfPromise: getPromisedTypeOfPromise,
                getReturnTypeOfSignature: getReturnTypeOfSignature,
                isNullableType: isNullableType,
                getNullableType: getNullableType,
                getNonNullableType: getNonNullableType,
                getNonOptionalType: removeOptionalTypeMarker,
                getTypeArguments: getTypeArguments,
                typeToTypeNode: nodeBuilder.typeToTypeNode,
                indexInfoToIndexSignatureDeclaration: nodeBuilder.indexInfoToIndexSignatureDeclaration,
                signatureToSignatureDeclaration: nodeBuilder.signatureToSignatureDeclaration,
                symbolToEntityName: nodeBuilder.symbolToEntityName,
                symbolToExpression: nodeBuilder.symbolToExpression,
                symbolToTypeParameterDeclarations: nodeBuilder.symbolToTypeParameterDeclarations,
                symbolToParameterDeclaration: nodeBuilder.symbolToParameterDeclaration,
                typeParameterToDeclaration: nodeBuilder.typeParameterToDeclaration,
                getSymbolsInScope: function (location, meaning) {
                    location = ts.getParseTreeNode(location);
                    return location ? getSymbolsInScope(location, meaning) : [];
                },
                getSymbolAtLocation: function (node) {
                    node = ts.getParseTreeNode(node);
                    return node ? getSymbolAtLocation(node) : undefined;
                },
                getShorthandAssignmentValueSymbol: function (node) {
                    node = ts.getParseTreeNode(node);
                    return node ? getShorthandAssignmentValueSymbol(node) : undefined;
                },
                getExportSpecifierLocalTargetSymbol: function (nodeIn) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isExportSpecifier);
                    return node ? getExportSpecifierLocalTargetSymbol(node) : undefined;
                },
                getExportSymbolOfSymbol: function (symbol) {
                    return getMergedSymbol(symbol.exportSymbol || symbol);
                },
                getTypeAtLocation: function (node) {
                    node = ts.getParseTreeNode(node);
                    return node ? getTypeOfNode(node) : errorType;
                },
                getTypeOfAssignmentPattern: function (nodeIn) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isAssignmentPattern);
                    return node && getTypeOfAssignmentPattern(node) || errorType;
                },
                getPropertySymbolOfDestructuringAssignment: function (locationIn) {
                    var location = ts.getParseTreeNode(locationIn, ts.isIdentifier);
                    return location ? getPropertySymbolOfDestructuringAssignment(location) : undefined;
                },
                signatureToString: function (signature, enclosingDeclaration, flags, kind) {
                    return signatureToString(signature, ts.getParseTreeNode(enclosingDeclaration), flags, kind);
                },
                typeToString: function (type, enclosingDeclaration, flags) {
                    return typeToString(type, ts.getParseTreeNode(enclosingDeclaration), flags);
                },
                symbolToString: function (symbol, enclosingDeclaration, meaning, flags) {
                    return symbolToString(symbol, ts.getParseTreeNode(enclosingDeclaration), meaning, flags);
                },
                typePredicateToString: function (predicate, enclosingDeclaration, flags) {
                    return typePredicateToString(predicate, ts.getParseTreeNode(enclosingDeclaration), flags);
                },
                writeSignature: function (signature, enclosingDeclaration, flags, kind, writer) {
                    return signatureToString(signature, ts.getParseTreeNode(enclosingDeclaration), flags, kind, writer);
                },
                writeType: function (type, enclosingDeclaration, flags, writer) {
                    return typeToString(type, ts.getParseTreeNode(enclosingDeclaration), flags, writer);
                },
                writeSymbol: function (symbol, enclosingDeclaration, meaning, flags, writer) {
                    return symbolToString(symbol, ts.getParseTreeNode(enclosingDeclaration), meaning, flags, writer);
                },
                writeTypePredicate: function (predicate, enclosingDeclaration, flags, writer) {
                    return typePredicateToString(predicate, ts.getParseTreeNode(enclosingDeclaration), flags, writer);
                },
                getAugmentedPropertiesOfType: getAugmentedPropertiesOfType,
                getRootSymbols: getRootSymbols,
                getContextualType: function (nodeIn, contextFlags) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isExpression);
                    if (!node) {
                        return undefined;
                    }
                    var containingCall = ts.findAncestor(node, ts.isCallLikeExpression);
                    var containingCallResolvedSignature = containingCall && getNodeLinks(containingCall).resolvedSignature;
                    if (contextFlags & 4 /* Completions */ && containingCall) {
                        var toMarkSkip = node;
                        do {
                            getNodeLinks(toMarkSkip).skipDirectInference = true;
                            toMarkSkip = toMarkSkip.parent;
                        } while (toMarkSkip && toMarkSkip !== containingCall);
                        getNodeLinks(containingCall).resolvedSignature = undefined;
                    }
                    var result = getContextualType(node, contextFlags);
                    if (contextFlags & 4 /* Completions */ && containingCall) {
                        var toMarkSkip = node;
                        do {
                            getNodeLinks(toMarkSkip).skipDirectInference = undefined;
                            toMarkSkip = toMarkSkip.parent;
                        } while (toMarkSkip && toMarkSkip !== containingCall);
                        getNodeLinks(containingCall).resolvedSignature = containingCallResolvedSignature;
                    }
                    return result;
                },
                getContextualTypeForObjectLiteralElement: function (nodeIn) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isObjectLiteralElementLike);
                    return node ? getContextualTypeForObjectLiteralElement(node) : undefined;
                },
                getContextualTypeForArgumentAtIndex: function (nodeIn, argIndex) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isCallLikeExpression);
                    return node && getContextualTypeForArgumentAtIndex(node, argIndex);
                },
                getContextualTypeForJsxAttribute: function (nodeIn) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isJsxAttributeLike);
                    return node && getContextualTypeForJsxAttribute(node);
                },
                isContextSensitive: isContextSensitive,
                getFullyQualifiedName: getFullyQualifiedName,
                getResolvedSignature: function (node, candidatesOutArray, argumentCount) {
                    return getResolvedSignatureWorker(node, candidatesOutArray, argumentCount, 0 /* Normal */);
                },
                getResolvedSignatureForSignatureHelp: function (node, candidatesOutArray, argumentCount) {
                    return getResolvedSignatureWorker(node, candidatesOutArray, argumentCount, 16 /* IsForSignatureHelp */);
                },
                getExpandedParameters: getExpandedParameters,
                hasEffectiveRestParameter: hasEffectiveRestParameter,
                getConstantValue: function (nodeIn) {
                    var node = ts.getParseTreeNode(nodeIn, canHaveConstantValue);
                    return node ? getConstantValue(node) : undefined;
                },
                isValidPropertyAccess: function (nodeIn, propertyName) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isPropertyAccessOrQualifiedNameOrImportTypeNode);
                    return !!node && isValidPropertyAccess(node, ts.escapeLeadingUnderscores(propertyName));
                },
                isValidPropertyAccessForCompletions: function (nodeIn, type, property) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isPropertyAccessExpression);
                    return !!node && isValidPropertyAccessForCompletions(node, type, property);
                },
                getSignatureFromDeclaration: function (declarationIn) {
                    var declaration = ts.getParseTreeNode(declarationIn, ts.isFunctionLike);
                    return declaration ? getSignatureFromDeclaration(declaration) : undefined;
                },
                isImplementationOfOverload: function (node) {
                    var parsed = ts.getParseTreeNode(node, ts.isFunctionLike);
                    return parsed ? isImplementationOfOverload(parsed) : undefined;
                },
                getImmediateAliasedSymbol: getImmediateAliasedSymbol,
                getAliasedSymbol: resolveAlias,
                getEmitResolver: getEmitResolver,
                getExportsOfModule: getExportsOfModuleAsArray,
                getExportsAndPropertiesOfModule: getExportsAndPropertiesOfModule,
                getSymbolWalker: ts.createGetSymbolWalker(getRestTypeOfSignature, getTypePredicateOfSignature, getReturnTypeOfSignature, getBaseTypes, resolveStructuredTypeMembers, getTypeOfSymbol, getResolvedSymbol, getIndexTypeOfStructuredType, getConstraintOfTypeParameter, ts.getFirstIdentifier, getTypeArguments),
                getAmbientModules: getAmbientModules,
                getJsxIntrinsicTagNamesAt: getJsxIntrinsicTagNamesAt,
                isOptionalParameter: function (nodeIn) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isParameter);
                    return node ? isOptionalParameter(node) : false;
                },
                tryGetMemberInModuleExports: function (name, symbol) { return tryGetMemberInModuleExports(ts.escapeLeadingUnderscores(name), symbol); },
                tryGetMemberInModuleExportsAndProperties: function (name, symbol) { return tryGetMemberInModuleExportsAndProperties(ts.escapeLeadingUnderscores(name), symbol); },
                tryFindAmbientModuleWithoutAugmentations: function (moduleName) {
                    // we deliberately exclude augmentations
                    // since we are only interested in declarations of the module itself
                    return tryFindAmbientModule(moduleName, /*withAugmentations*/ false);
                },
                getApparentType: getApparentType,
                getUnionType: getUnionType,
                isTypeAssignableTo: function (source, target) {
                    return isTypeAssignableTo(source, target);
                },
                createAnonymousType: createAnonymousType,
                createSignature: createSignature,
                createSymbol: createSymbol,
                createIndexInfo: createIndexInfo,
                getAnyType: function () { return anyType; },
                getStringType: function () { return stringType; },
                getNumberType: function () { return numberType; },
                createPromiseType: createPromiseType,
                createArrayType: createArrayType,
                getElementTypeOfArrayType: getElementTypeOfArrayType,
                getBooleanType: function () { return booleanType; },
                getFalseType: function (fresh) { return fresh ? falseType : regularFalseType; },
                getTrueType: function (fresh) { return fresh ? trueType : regularTrueType; },
                getVoidType: function () { return voidType; },
                getUndefinedType: function () { return undefinedType; },
                getNullType: function () { return nullType; },
                getESSymbolType: function () { return esSymbolType; },
                getNeverType: function () { return neverType; },
                getOptionalType: function () { return optionalType; },
                isSymbolAccessible: isSymbolAccessible,
                isArrayType: isArrayType,
                isTupleType: isTupleType,
                isArrayLikeType: isArrayLikeType,
                isTypeInvalidDueToUnionDiscriminant: isTypeInvalidDueToUnionDiscriminant,
                getAllPossiblePropertiesOfTypes: getAllPossiblePropertiesOfTypes,
                getSuggestedSymbolForNonexistentProperty: getSuggestedSymbolForNonexistentProperty,
                getSuggestionForNonexistentProperty: getSuggestionForNonexistentProperty,
                getSuggestedSymbolForNonexistentSymbol: function (location, name, meaning) { return getSuggestedSymbolForNonexistentSymbol(location, ts.escapeLeadingUnderscores(name), meaning); },
                getSuggestionForNonexistentSymbol: function (location, name, meaning) { return getSuggestionForNonexistentSymbol(location, ts.escapeLeadingUnderscores(name), meaning); },
                getSuggestedSymbolForNonexistentModule: getSuggestedSymbolForNonexistentModule,
                getSuggestionForNonexistentExport: getSuggestionForNonexistentExport,
                getBaseConstraintOfType: getBaseConstraintOfType,
                getDefaultFromTypeParameter: function (type) { return type && type.flags & 262144 /* TypeParameter */ ? getDefaultFromTypeParameter(type) : undefined; },
                resolveName: function (name, location, meaning, excludeGlobals) {
                    return resolveName(location, ts.escapeLeadingUnderscores(name), meaning, /*nameNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ false, excludeGlobals);
                },
                getJsxNamespace: function (n) { return ts.unescapeLeadingUnderscores(getJsxNamespace(n)); },
                getAccessibleSymbolChain: getAccessibleSymbolChain,
                getTypePredicateOfSignature: getTypePredicateOfSignature,
                resolveExternalModuleSymbol: resolveExternalModuleSymbol,
                tryGetThisTypeAt: function (node, includeGlobalThis) {
                    node = ts.getParseTreeNode(node);
                    return node && tryGetThisTypeAt(node, includeGlobalThis);
                },
                getTypeArgumentConstraint: function (nodeIn) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isTypeNode);
                    return node && getTypeArgumentConstraint(node);
                },
                getSuggestionDiagnostics: function (file, ct) {
                    if (ts.skipTypeChecking(file, compilerOptions, host)) {
                        return ts.emptyArray;
                    }
                    var diagnostics;
                    try {
                        // Record the cancellation token so it can be checked later on during checkSourceElement.
                        // Do this in a finally block so we can ensure that it gets reset back to nothing after
                        // this call is done.
                        cancellationToken = ct;
                        // Ensure file is type checked
                        checkSourceFile(file);
                        ts.Debug.assert(!!(getNodeLinks(file).flags & 1 /* TypeChecked */));
                        diagnostics = ts.addRange(diagnostics, suggestionDiagnostics.getDiagnostics(file.fileName));
                        checkUnusedIdentifiers(getPotentiallyUnusedIdentifiers(file), function (containingNode, kind, diag) {
                            if (!ts.containsParseError(containingNode) && !unusedIsError(kind, !!(containingNode.flags & 8388608 /* Ambient */))) {
                                (diagnostics || (diagnostics = [])).push(__assign(__assign({}, diag), { category: ts.DiagnosticCategory.Suggestion }));
                            }
                        });
                        return diagnostics || ts.emptyArray;
                    }
                    finally {
                        cancellationToken = undefined;
                    }
                },
                runWithCancellationToken: function (token, callback) {
                    try {
                        cancellationToken = token;
                        return callback(checker);
                    }
                    finally {
                        cancellationToken = undefined;
                    }
                },
                getLocalTypeParametersOfClassOrInterfaceOrTypeAlias: getLocalTypeParametersOfClassOrInterfaceOrTypeAlias,
                isDeclarationVisible: isDeclarationVisible,
            };
            function getResolvedSignatureWorker(nodeIn, candidatesOutArray, argumentCount, checkMode) {
                var node = ts.getParseTreeNode(nodeIn, ts.isCallLikeExpression);
                apparentArgumentCount = argumentCount;
                var res = node ? getResolvedSignature(node, candidatesOutArray, checkMode) : undefined;
                apparentArgumentCount = undefined;
                return res;
            }
            var tupleTypes = ts.createMap();
            var unionTypes = ts.createMap();
            var intersectionTypes = ts.createMap();
            var literalTypes = ts.createMap();
            var indexedAccessTypes = ts.createMap();
            var substitutionTypes = ts.createMap();
            var evolvingArrayTypes = [];
            var undefinedProperties = ts.createMap();
            var unknownSymbol = createSymbol(4 /* Property */, "unknown");
            var resolvingSymbol = createSymbol(0, "__resolving__" /* Resolving */);
            var anyType = createIntrinsicType(1 /* Any */, "any");
            var autoType = createIntrinsicType(1 /* Any */, "any");
            var wildcardType = createIntrinsicType(1 /* Any */, "any");
            var errorType = createIntrinsicType(1 /* Any */, "error");
            var nonInferrableAnyType = createIntrinsicType(1 /* Any */, "any", 524288 /* ContainsWideningType */);
            var unknownType = createIntrinsicType(2 /* Unknown */, "unknown");
            var undefinedType = createIntrinsicType(32768 /* Undefined */, "undefined");
            var undefinedWideningType = strictNullChecks ? undefinedType : createIntrinsicType(32768 /* Undefined */, "undefined", 524288 /* ContainsWideningType */);
            var optionalType = createIntrinsicType(32768 /* Undefined */, "undefined");
            var nullType = createIntrinsicType(65536 /* Null */, "null");
            var nullWideningType = strictNullChecks ? nullType : createIntrinsicType(65536 /* Null */, "null", 524288 /* ContainsWideningType */);
            var stringType = createIntrinsicType(4 /* String */, "string");
            var numberType = createIntrinsicType(8 /* Number */, "number");
            var bigintType = createIntrinsicType(64 /* BigInt */, "bigint");
            var falseType = createIntrinsicType(512 /* BooleanLiteral */, "false");
            var regularFalseType = createIntrinsicType(512 /* BooleanLiteral */, "false");
            var trueType = createIntrinsicType(512 /* BooleanLiteral */, "true");
            var regularTrueType = createIntrinsicType(512 /* BooleanLiteral */, "true");
            trueType.regularType = regularTrueType;
            trueType.freshType = trueType;
            regularTrueType.regularType = regularTrueType;
            regularTrueType.freshType = trueType;
            falseType.regularType = regularFalseType;
            falseType.freshType = falseType;
            regularFalseType.regularType = regularFalseType;
            regularFalseType.freshType = falseType;
            var booleanType = createBooleanType([regularFalseType, regularTrueType]);
            // Also mark all combinations of fresh/regular booleans as "Boolean" so they print as `boolean` instead of `true | false`
            // (The union is cached, so simply doing the marking here is sufficient)
            createBooleanType([regularFalseType, trueType]);
            createBooleanType([falseType, regularTrueType]);
            createBooleanType([falseType, trueType]);
            var esSymbolType = createIntrinsicType(4096 /* ESSymbol */, "symbol");
            var voidType = createIntrinsicType(16384 /* Void */, "void");
            var neverType = createIntrinsicType(131072 /* Never */, "never");
            var silentNeverType = createIntrinsicType(131072 /* Never */, "never");
            var nonInferrableType = createIntrinsicType(131072 /* Never */, "never", 2097152 /* NonInferrableType */);
            var implicitNeverType = createIntrinsicType(131072 /* Never */, "never");
            var unreachableNeverType = createIntrinsicType(131072 /* Never */, "never");
            var nonPrimitiveType = createIntrinsicType(67108864 /* NonPrimitive */, "object");
            var stringNumberSymbolType = getUnionType([stringType, numberType, esSymbolType]);
            var keyofConstraintType = keyofStringsOnly ? stringType : stringNumberSymbolType;
            var numberOrBigIntType = getUnionType([numberType, bigintType]);
            var emptyObjectType = createAnonymousType(undefined, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
            var emptyJsxObjectType = createAnonymousType(undefined, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
            emptyJsxObjectType.objectFlags |= 4096 /* JsxAttributes */;
            var emptyTypeLiteralSymbol = createSymbol(2048 /* TypeLiteral */, "__type" /* Type */);
            emptyTypeLiteralSymbol.members = ts.createSymbolTable();
            var emptyTypeLiteralType = createAnonymousType(emptyTypeLiteralSymbol, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
            var emptyGenericType = createAnonymousType(undefined, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
            emptyGenericType.instantiations = ts.createMap();
            var anyFunctionType = createAnonymousType(undefined, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
            // The anyFunctionType contains the anyFunctionType by definition. The flag is further propagated
            // in getPropagatingFlagsOfTypes, and it is checked in inferFromTypes.
            anyFunctionType.objectFlags |= 2097152 /* NonInferrableType */;
            var noConstraintType = createAnonymousType(undefined, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
            var circularConstraintType = createAnonymousType(undefined, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
            var resolvingDefaultType = createAnonymousType(undefined, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
            var markerSuperType = createTypeParameter();
            var markerSubType = createTypeParameter();
            markerSubType.constraint = markerSuperType;
            var markerOtherType = createTypeParameter();
            var noTypePredicate = createTypePredicate(1 /* Identifier */, "<<unresolved>>", 0, anyType);
            var anySignature = createSignature(undefined, undefined, undefined, ts.emptyArray, anyType, /*resolvedTypePredicate*/ undefined, 0, 0 /* None */);
            var unknownSignature = createSignature(undefined, undefined, undefined, ts.emptyArray, errorType, /*resolvedTypePredicate*/ undefined, 0, 0 /* None */);
            var resolvingSignature = createSignature(undefined, undefined, undefined, ts.emptyArray, anyType, /*resolvedTypePredicate*/ undefined, 0, 0 /* None */);
            var silentNeverSignature = createSignature(undefined, undefined, undefined, ts.emptyArray, silentNeverType, /*resolvedTypePredicate*/ undefined, 0, 0 /* None */);
            var enumNumberIndexInfo = createIndexInfo(stringType, /*isReadonly*/ true);
            var iterationTypesCache = ts.createMap(); // cache for common IterationTypes instances
            var noIterationTypes = {
                get yieldType() { return ts.Debug.fail("Not supported"); },
                get returnType() { return ts.Debug.fail("Not supported"); },
                get nextType() { return ts.Debug.fail("Not supported"); },
            };
            var anyIterationTypes = createIterationTypes(anyType, anyType, anyType);
            var anyIterationTypesExceptNext = createIterationTypes(anyType, anyType, unknownType);
            var defaultIterationTypes = createIterationTypes(neverType, anyType, undefinedType); // default iteration types for `Iterator`.
            var asyncIterationTypesResolver = {
                iterableCacheKey: "iterationTypesOfAsyncIterable",
                iteratorCacheKey: "iterationTypesOfAsyncIterator",
                iteratorSymbolName: "asyncIterator",
                getGlobalIteratorType: getGlobalAsyncIteratorType,
                getGlobalIterableType: getGlobalAsyncIterableType,
                getGlobalIterableIteratorType: getGlobalAsyncIterableIteratorType,
                getGlobalGeneratorType: getGlobalAsyncGeneratorType,
                resolveIterationType: getAwaitedType,
                mustHaveANextMethodDiagnostic: ts.Diagnostics.An_async_iterator_must_have_a_next_method,
                mustBeAMethodDiagnostic: ts.Diagnostics.The_0_property_of_an_async_iterator_must_be_a_method,
                mustHaveAValueDiagnostic: ts.Diagnostics.The_type_returned_by_the_0_method_of_an_async_iterator_must_be_a_promise_for_a_type_with_a_value_property,
            };
            var syncIterationTypesResolver = {
                iterableCacheKey: "iterationTypesOfIterable",
                iteratorCacheKey: "iterationTypesOfIterator",
                iteratorSymbolName: "iterator",
                getGlobalIteratorType: getGlobalIteratorType,
                getGlobalIterableType: getGlobalIterableType,
                getGlobalIterableIteratorType: getGlobalIterableIteratorType,
                getGlobalGeneratorType: getGlobalGeneratorType,
                resolveIterationType: function (type, _errorNode) { return type; },
                mustHaveANextMethodDiagnostic: ts.Diagnostics.An_iterator_must_have_a_next_method,
                mustBeAMethodDiagnostic: ts.Diagnostics.The_0_property_of_an_iterator_must_be_a_method,
                mustHaveAValueDiagnostic: ts.Diagnostics.The_type_returned_by_the_0_method_of_an_iterator_must_have_a_value_property,
            };
            /** Key is "/path/to/a.ts|/path/to/b.ts". */
            var amalgamatedDuplicates;
            var reverseMappedCache = ts.createMap();
            var ambientModulesCache;
            /**
             * List of every ambient module with a "*" wildcard.
             * Unlike other ambient modules, these can't be stored in `globals` because symbol tables only deal with exact matches.
             * This is only used if there is no exact match.
             */
            var patternAmbientModules;
            var patternAmbientModuleAugmentations;
            var globalObjectType;
            var globalFunctionType;
            var globalCallableFunctionType;
            var globalNewableFunctionType;
            var globalArrayType;
            var globalReadonlyArrayType;
            var globalStringType;
            var globalNumberType;
            var globalBooleanType;
            var globalRegExpType;
            var globalThisType;
            var anyArrayType;
            var autoArrayType;
            var anyReadonlyArrayType;
            var deferredGlobalNonNullableTypeAlias;
            // The library files are only loaded when the feature is used.
            // This allows users to just specify library files they want to used through --lib
            // and they will not get an error from not having unrelated library files
            var deferredGlobalESSymbolConstructorSymbol;
            var deferredGlobalESSymbolType;
            var deferredGlobalTypedPropertyDescriptorType;
            var deferredGlobalPromiseType;
            var deferredGlobalPromiseLikeType;
            var deferredGlobalPromiseConstructorSymbol;
            var deferredGlobalPromiseConstructorLikeType;
            var deferredGlobalIterableType;
            var deferredGlobalIteratorType;
            var deferredGlobalIterableIteratorType;
            var deferredGlobalGeneratorType;
            var deferredGlobalIteratorYieldResultType;
            var deferredGlobalIteratorReturnResultType;
            var deferredGlobalAsyncIterableType;
            var deferredGlobalAsyncIteratorType;
            var deferredGlobalAsyncIterableIteratorType;
            var deferredGlobalAsyncGeneratorType;
            var deferredGlobalTemplateStringsArrayType;
            var deferredGlobalImportMetaType;
            var deferredGlobalExtractSymbol;
            var deferredGlobalOmitSymbol;
            var deferredGlobalBigIntType;
            var allPotentiallyUnusedIdentifiers = ts.createMap(); // key is file name
            var flowLoopStart = 0;
            var flowLoopCount = 0;
            var sharedFlowCount = 0;
            var flowAnalysisDisabled = false;
            var flowInvocationCount = 0;
            var lastFlowNode;
            var lastFlowNodeReachable;
            var flowTypeCache;
            var emptyStringType = getLiteralType("");
            var zeroType = getLiteralType(0);
            var zeroBigIntType = getLiteralType({ negative: false, base10Value: "0" });
            var resolutionTargets = [];
            var resolutionResults = [];
            var resolutionPropertyNames = [];
            var suggestionCount = 0;
            var maximumSuggestionCount = 10;
            var mergedSymbols = [];
            var symbolLinks = [];
            var nodeLinks = [];
            var flowLoopCaches = [];
            var flowLoopNodes = [];
            var flowLoopKeys = [];
            var flowLoopTypes = [];
            var sharedFlowNodes = [];
            var sharedFlowTypes = [];
            var flowNodeReachable = [];
            var potentialThisCollisions = [];
            var potentialNewTargetCollisions = [];
            var potentialWeakMapCollisions = [];
            var awaitedTypeStack = [];
            var diagnostics = ts.createDiagnosticCollection();
            var suggestionDiagnostics = ts.createDiagnosticCollection();
            var typeofTypesByName = ts.createMapFromTemplate({
                string: stringType,
                number: numberType,
                bigint: bigintType,
                boolean: booleanType,
                symbol: esSymbolType,
                undefined: undefinedType
            });
            var typeofType = createTypeofType();
            var _jsxNamespace;
            var _jsxFactoryEntity;
            var outofbandVarianceMarkerHandler;
            var subtypeRelation = ts.createMap();
            var strictSubtypeRelation = ts.createMap();
            var assignableRelation = ts.createMap();
            var comparableRelation = ts.createMap();
            var identityRelation = ts.createMap();
            var enumRelation = ts.createMap();
            var builtinGlobals = ts.createSymbolTable();
            builtinGlobals.set(undefinedSymbol.escapedName, undefinedSymbol);
            initializeTypeChecker();
            return checker;
            function getJsxNamespace(location) {
                if (location) {
                    var file = ts.getSourceFileOfNode(location);
                    if (file) {
                        if (file.localJsxNamespace) {
                            return file.localJsxNamespace;
                        }
                        var jsxPragma = file.pragmas.get("jsx");
                        if (jsxPragma) {
                            var chosenpragma = ts.isArray(jsxPragma) ? jsxPragma[0] : jsxPragma;
                            file.localJsxFactory = ts.parseIsolatedEntityName(chosenpragma.arguments.factory, languageVersion);
                            if (file.localJsxFactory) {
                                return file.localJsxNamespace = ts.getFirstIdentifier(file.localJsxFactory).escapedText;
                            }
                        }
                    }
                }
                if (!_jsxNamespace) {
                    _jsxNamespace = "React";
                    if (compilerOptions.jsxFactory) {
                        _jsxFactoryEntity = ts.parseIsolatedEntityName(compilerOptions.jsxFactory, languageVersion);
                        if (_jsxFactoryEntity) {
                            _jsxNamespace = ts.getFirstIdentifier(_jsxFactoryEntity).escapedText;
                        }
                    }
                    else if (compilerOptions.reactNamespace) {
                        _jsxNamespace = ts.escapeLeadingUnderscores(compilerOptions.reactNamespace);
                    }
                }
                return _jsxNamespace;
            }
            function getEmitResolver(sourceFile, cancellationToken) {
                // Ensure we have all the type information in place for this file so that all the
                // emitter questions of this resolver will return the right information.
                getDiagnostics(sourceFile, cancellationToken);
                return emitResolver;
            }
            function lookupOrIssueError(location, message, arg0, arg1, arg2, arg3) {
                var diagnostic = location
                    ? ts.createDiagnosticForNode(location, message, arg0, arg1, arg2, arg3)
                    : ts.createCompilerDiagnostic(message, arg0, arg1, arg2, arg3);
                var existing = diagnostics.lookup(diagnostic);
                if (existing) {
                    return existing;
                }
                else {
                    diagnostics.add(diagnostic);
                    return diagnostic;
                }
            }
            function error(location, message, arg0, arg1, arg2, arg3) {
                var diagnostic = location
                    ? ts.createDiagnosticForNode(location, message, arg0, arg1, arg2, arg3)
                    : ts.createCompilerDiagnostic(message, arg0, arg1, arg2, arg3);
                diagnostics.add(diagnostic);
                return diagnostic;
            }
            function addErrorOrSuggestion(isError, diagnostic) {
                if (isError) {
                    diagnostics.add(diagnostic);
                }
                else {
                    suggestionDiagnostics.add(__assign(__assign({}, diagnostic), { category: ts.DiagnosticCategory.Suggestion }));
                }
            }
            function errorOrSuggestion(isError, location, message, arg0, arg1, arg2, arg3) {
                addErrorOrSuggestion(isError, "message" in message ? ts.createDiagnosticForNode(location, message, arg0, arg1, arg2, arg3) : ts.createDiagnosticForNodeFromMessageChain(location, message)); // eslint-disable-line no-in-operator
            }
            function errorAndMaybeSuggestAwait(location, maybeMissingAwait, message, arg0, arg1, arg2, arg3) {
                var diagnostic = error(location, message, arg0, arg1, arg2, arg3);
                if (maybeMissingAwait) {
                    var related = ts.createDiagnosticForNode(location, ts.Diagnostics.Did_you_forget_to_use_await);
                    ts.addRelatedInfo(diagnostic, related);
                }
                return diagnostic;
            }
            function createSymbol(flags, name, checkFlags) {
                symbolCount++;
                var symbol = (new Symbol(flags | 33554432 /* Transient */, name));
                symbol.checkFlags = checkFlags || 0;
                return symbol;
            }
            function getExcludedSymbolFlags(flags) {
                var result = 0;
                if (flags & 2 /* BlockScopedVariable */)
                    result |= 111551 /* BlockScopedVariableExcludes */;
                if (flags & 1 /* FunctionScopedVariable */)
                    result |= 111550 /* FunctionScopedVariableExcludes */;
                if (flags & 4 /* Property */)
                    result |= 0 /* PropertyExcludes */;
                if (flags & 8 /* EnumMember */)
                    result |= 900095 /* EnumMemberExcludes */;
                if (flags & 16 /* Function */)
                    result |= 110991 /* FunctionExcludes */;
                if (flags & 32 /* Class */)
                    result |= 899503 /* ClassExcludes */;
                if (flags & 64 /* Interface */)
                    result |= 788872 /* InterfaceExcludes */;
                if (flags & 256 /* RegularEnum */)
                    result |= 899327 /* RegularEnumExcludes */;
                if (flags & 128 /* ConstEnum */)
                    result |= 899967 /* ConstEnumExcludes */;
                if (flags & 512 /* ValueModule */)
                    result |= 110735 /* ValueModuleExcludes */;
                if (flags & 8192 /* Method */)
                    result |= 103359 /* MethodExcludes */;
                if (flags & 32768 /* GetAccessor */)
                    result |= 46015 /* GetAccessorExcludes */;
                if (flags & 65536 /* SetAccessor */)
                    result |= 78783 /* SetAccessorExcludes */;
                if (flags & 262144 /* TypeParameter */)
                    result |= 526824 /* TypeParameterExcludes */;
                if (flags & 524288 /* TypeAlias */)
                    result |= 788968 /* TypeAliasExcludes */;
                if (flags & 2097152 /* Alias */)
                    result |= 2097152 /* AliasExcludes */;
                return result;
            }
            function recordMergedSymbol(target, source) {
                if (!source.mergeId) {
                    source.mergeId = nextMergeId;
                    nextMergeId++;
                }
                mergedSymbols[source.mergeId] = target;
            }
            function cloneSymbol(symbol) {
                var result = createSymbol(symbol.flags, symbol.escapedName);
                result.declarations = symbol.declarations ? symbol.declarations.slice() : [];
                result.parent = symbol.parent;
                if (symbol.valueDeclaration)
                    result.valueDeclaration = symbol.valueDeclaration;
                if (symbol.constEnumOnlyModule)
                    result.constEnumOnlyModule = true;
                if (symbol.members)
                    result.members = ts.cloneMap(symbol.members);
                if (symbol.exports)
                    result.exports = ts.cloneMap(symbol.exports);
                recordMergedSymbol(result, symbol);
                return result;
            }
            /**
             * Note: if target is transient, then it is mutable, and mergeSymbol with both mutate and return it.
             * If target is not transient, mergeSymbol will produce a transient clone, mutate that and return it.
             */
            function mergeSymbol(target, source, unidirectional) {
                if (unidirectional === void 0) {
                    unidirectional = false;
                }
                if (!(target.flags & getExcludedSymbolFlags(source.flags)) ||
                    (source.flags | target.flags) & 67108864 /* Assignment */) {
                    if (source === target) {
                        // This can happen when an export assigned namespace exports something also erroneously exported at the top level
                        // See `declarationFileNoCrashOnExtraExportModifier` for an example
                        return target;
                    }
                    if (!(target.flags & 33554432 /* Transient */)) {
                        var resolvedTarget = resolveSymbol(target);
                        if (resolvedTarget === unknownSymbol) {
                            return source;
                        }
                        target = cloneSymbol(resolvedTarget);
                    }
                    // Javascript static-property-assignment declarations always merge, even though they are also values
                    if (source.flags & 512 /* ValueModule */ && target.flags & 512 /* ValueModule */ && target.constEnumOnlyModule && !source.constEnumOnlyModule) {
                        // reset flag when merging instantiated module into value module that has only const enums
                        target.constEnumOnlyModule = false;
                    }
                    target.flags |= source.flags;
                    if (source.valueDeclaration &&
                        (!target.valueDeclaration ||
                            ts.isAssignmentDeclaration(target.valueDeclaration) && !ts.isAssignmentDeclaration(source.valueDeclaration) ||
                            ts.isEffectiveModuleDeclaration(target.valueDeclaration) && !ts.isEffectiveModuleDeclaration(source.valueDeclaration))) {
                        // other kinds of value declarations take precedence over modules and assignment declarations
                        target.valueDeclaration = source.valueDeclaration;
                    }
                    ts.addRange(target.declarations, source.declarations);
                    if (source.members) {
                        if (!target.members)
                            target.members = ts.createSymbolTable();
                        mergeSymbolTable(target.members, source.members, unidirectional);
                    }
                    if (source.exports) {
                        if (!target.exports)
                            target.exports = ts.createSymbolTable();
                        mergeSymbolTable(target.exports, source.exports, unidirectional);
                    }
                    if (!unidirectional) {
                        recordMergedSymbol(target, source);
                    }
                }
                else if (target.flags & 1024 /* NamespaceModule */) {
                    // Do not report an error when merging `var globalThis` with the built-in `globalThis`,
                    // as we will already report a "Declaration name conflicts..." error, and this error
                    // won't make much sense.
                    if (target !== globalThisSymbol) {
                        error(ts.getNameOfDeclaration(source.declarations[0]), ts.Diagnostics.Cannot_augment_module_0_with_value_exports_because_it_resolves_to_a_non_module_entity, symbolToString(target));
                    }
                }
                else { // error
                    var isEitherEnum = !!(target.flags & 384 /* Enum */ || source.flags & 384 /* Enum */);
                    var isEitherBlockScoped_1 = !!(target.flags & 2 /* BlockScopedVariable */ || source.flags & 2 /* BlockScopedVariable */);
                    var message = isEitherEnum
                        ? ts.Diagnostics.Enum_declarations_can_only_merge_with_namespace_or_other_enum_declarations
                        : isEitherBlockScoped_1
                            ? ts.Diagnostics.Cannot_redeclare_block_scoped_variable_0
                            : ts.Diagnostics.Duplicate_identifier_0;
                    var sourceSymbolFile = source.declarations && ts.getSourceFileOfNode(source.declarations[0]);
                    var targetSymbolFile = target.declarations && ts.getSourceFileOfNode(target.declarations[0]);
                    var symbolName_1 = symbolToString(source);
                    // Collect top-level duplicate identifier errors into one mapping, so we can then merge their diagnostics if there are a bunch
                    if (sourceSymbolFile && targetSymbolFile && amalgamatedDuplicates && !isEitherEnum && sourceSymbolFile !== targetSymbolFile) {
                        var firstFile_1 = ts.comparePaths(sourceSymbolFile.path, targetSymbolFile.path) === -1 /* LessThan */ ? sourceSymbolFile : targetSymbolFile;
                        var secondFile_1 = firstFile_1 === sourceSymbolFile ? targetSymbolFile : sourceSymbolFile;
                        var filesDuplicates = ts.getOrUpdate(amalgamatedDuplicates, firstFile_1.path + "|" + secondFile_1.path, function () {
                            return ({ firstFile: firstFile_1, secondFile: secondFile_1, conflictingSymbols: ts.createMap() });
                        });
                        var conflictingSymbolInfo = ts.getOrUpdate(filesDuplicates.conflictingSymbols, symbolName_1, function () {
                            return ({ isBlockScoped: isEitherBlockScoped_1, firstFileLocations: [], secondFileLocations: [] });
                        });
                        addDuplicateLocations(conflictingSymbolInfo.firstFileLocations, source);
                        addDuplicateLocations(conflictingSymbolInfo.secondFileLocations, target);
                    }
                    else {
                        addDuplicateDeclarationErrorsForSymbols(source, message, symbolName_1, target);
                        addDuplicateDeclarationErrorsForSymbols(target, message, symbolName_1, source);
                    }
                }
                return target;
                function addDuplicateLocations(locs, symbol) {
                    for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        ts.pushIfUnique(locs, (ts.getExpandoInitializer(decl, /*isPrototypeAssignment*/ false) ? ts.getNameOfExpando(decl) : ts.getNameOfDeclaration(decl)) || decl);
                    }
                }
            }
            function addDuplicateDeclarationErrorsForSymbols(target, message, symbolName, source) {
                ts.forEach(target.declarations, function (node) {
                    var errorNode = (ts.getExpandoInitializer(node, /*isPrototypeAssignment*/ false) ? ts.getNameOfExpando(node) : ts.getNameOfDeclaration(node)) || node;
                    addDuplicateDeclarationError(errorNode, message, symbolName, source.declarations);
                });
            }
            function addDuplicateDeclarationError(errorNode, message, symbolName, relatedNodes) {
                var err = lookupOrIssueError(errorNode, message, symbolName);
                for (var _i = 0, _a = relatedNodes || ts.emptyArray; _i < _a.length; _i++) {
                    var relatedNode = _a[_i];
                    err.relatedInformation = err.relatedInformation || [];
                    if (ts.length(err.relatedInformation) >= 5)
                        continue;
                    ts.addRelatedInfo(err, !ts.length(err.relatedInformation) ? ts.createDiagnosticForNode(relatedNode, ts.Diagnostics._0_was_also_declared_here, symbolName) : ts.createDiagnosticForNode(relatedNode, ts.Diagnostics.and_here));
                }
            }
            function combineSymbolTables(first, second) {
                if (!ts.hasEntries(first))
                    return second;
                if (!ts.hasEntries(second))
                    return first;
                var combined = ts.createSymbolTable();
                mergeSymbolTable(combined, first);
                mergeSymbolTable(combined, second);
                return combined;
            }
            function mergeSymbolTable(target, source, unidirectional) {
                if (unidirectional === void 0) {
                    unidirectional = false;
                }
                source.forEach(function (sourceSymbol, id) {
                    var targetSymbol = target.get(id);
                    target.set(id, targetSymbol ? mergeSymbol(targetSymbol, sourceSymbol, unidirectional) : sourceSymbol);
                });
            }
            function mergeModuleAugmentation(moduleName) {
                var moduleAugmentation = moduleName.parent;
                if (moduleAugmentation.symbol.declarations[0] !== moduleAugmentation) {
                    // this is a combined symbol for multiple augmentations within the same file.
                    // its symbol already has accumulated information for all declarations
                    // so we need to add it just once - do the work only for first declaration
                    ts.Debug.assert(moduleAugmentation.symbol.declarations.length > 1);
                    return;
                }
                if (ts.isGlobalScopeAugmentation(moduleAugmentation)) {
                    mergeSymbolTable(globals, moduleAugmentation.symbol.exports);
                }
                else {
                    // find a module that about to be augmented
                    // do not validate names of augmentations that are defined in ambient context
                    var moduleNotFoundError = !(moduleName.parent.parent.flags & 8388608 /* Ambient */)
                        ? ts.Diagnostics.Invalid_module_name_in_augmentation_module_0_cannot_be_found
                        : undefined;
                    var mainModule_1 = resolveExternalModuleNameWorker(moduleName, moduleName, moduleNotFoundError, /*isForAugmentation*/ true);
                    if (!mainModule_1) {
                        return;
                    }
                    // obtain item referenced by 'export='
                    mainModule_1 = resolveExternalModuleSymbol(mainModule_1);
                    if (mainModule_1.flags & 1920 /* Namespace */) {
                        // If we're merging an augmentation to a pattern ambient module, we want to
                        // perform the merge unidirectionally from the augmentation ('a.foo') to
                        // the pattern ('*.foo'), so that 'getMergedSymbol()' on a.foo gives you
                        // all the exports both from the pattern and from the augmentation, but
                        // 'getMergedSymbol()' on *.foo only gives you exports from *.foo.
                        if (ts.some(patternAmbientModules, function (module) { return mainModule_1 === module.symbol; })) {
                            var merged = mergeSymbol(moduleAugmentation.symbol, mainModule_1, /*unidirectional*/ true);
                            if (!patternAmbientModuleAugmentations) {
                                patternAmbientModuleAugmentations = ts.createMap();
                            }
                            // moduleName will be a StringLiteral since this is not `declare global`.
                            patternAmbientModuleAugmentations.set(moduleName.text, merged);
                        }
                        else {
                            mergeSymbol(mainModule_1, moduleAugmentation.symbol);
                        }
                    }
                    else {
                        // moduleName will be a StringLiteral since this is not `declare global`.
                        error(moduleName, ts.Diagnostics.Cannot_augment_module_0_because_it_resolves_to_a_non_module_entity, moduleName.text);
                    }
                }
            }
            function addToSymbolTable(target, source, message) {
                source.forEach(function (sourceSymbol, id) {
                    var targetSymbol = target.get(id);
                    if (targetSymbol) {
                        // Error on redeclarations
                        ts.forEach(targetSymbol.declarations, addDeclarationDiagnostic(ts.unescapeLeadingUnderscores(id), message));
                    }
                    else {
                        target.set(id, sourceSymbol);
                    }
                });
                function addDeclarationDiagnostic(id, message) {
                    return function (declaration) { return diagnostics.add(ts.createDiagnosticForNode(declaration, message, id)); };
                }
            }
            function getSymbolLinks(symbol) {
                if (symbol.flags & 33554432 /* Transient */)
                    return symbol;
                var id = getSymbolId(symbol);
                return symbolLinks[id] || (symbolLinks[id] = {});
            }
            function getNodeLinks(node) {
                var nodeId = getNodeId(node);
                return nodeLinks[nodeId] || (nodeLinks[nodeId] = { flags: 0 });
            }
            function isGlobalSourceFile(node) {
                return node.kind === 290 /* SourceFile */ && !ts.isExternalOrCommonJsModule(node);
            }
            function getSymbol(symbols, name, meaning) {
                if (meaning) {
                    var symbol = symbols.get(name);
                    if (symbol) {
                        ts.Debug.assert((ts.getCheckFlags(symbol) & 1 /* Instantiated */) === 0, "Should never get an instantiated symbol here.");
                        if (symbol.flags & meaning) {
                            return symbol;
                        }
                        if (symbol.flags & 2097152 /* Alias */) {
                            var target = resolveAlias(symbol);
                            // Unknown symbol means an error occurred in alias resolution, treat it as positive answer to avoid cascading errors
                            if (target === unknownSymbol || target.flags & meaning) {
                                return symbol;
                            }
                        }
                    }
                }
                // return undefined if we can't find a symbol.
            }
            /**
             * Get symbols that represent parameter-property-declaration as parameter and as property declaration
             * @param parameter a parameterDeclaration node
             * @param parameterName a name of the parameter to get the symbols for.
             * @return a tuple of two symbols
             */
            function getSymbolsOfParameterPropertyDeclaration(parameter, parameterName) {
                var constructorDeclaration = parameter.parent;
                var classDeclaration = parameter.parent.parent;
                var parameterSymbol = getSymbol(constructorDeclaration.locals, parameterName, 111551 /* Value */);
                var propertySymbol = getSymbol(getMembersOfSymbol(classDeclaration.symbol), parameterName, 111551 /* Value */);
                if (parameterSymbol && propertySymbol) {
                    return [parameterSymbol, propertySymbol];
                }
                return ts.Debug.fail("There should exist two symbols, one as property declaration and one as parameter declaration");
            }
            function isBlockScopedNameDeclaredBeforeUse(declaration, usage) {
                var declarationFile = ts.getSourceFileOfNode(declaration);
                var useFile = ts.getSourceFileOfNode(usage);
                if (declarationFile !== useFile) {
                    if ((moduleKind && (declarationFile.externalModuleIndicator || useFile.externalModuleIndicator)) ||
                        (!compilerOptions.outFile && !compilerOptions.out) ||
                        isInTypeQuery(usage) ||
                        declaration.flags & 8388608 /* Ambient */) {
                        // nodes are in different files and order cannot be determined
                        return true;
                    }
                    // declaration is after usage
                    // can be legal if usage is deferred (i.e. inside function or in initializer of instance property)
                    if (isUsedInFunctionOrInstanceProperty(usage, declaration)) {
                        return true;
                    }
                    var sourceFiles = host.getSourceFiles();
                    return sourceFiles.indexOf(declarationFile) <= sourceFiles.indexOf(useFile);
                }
                if (declaration.pos <= usage.pos) {
                    // declaration is before usage
                    if (declaration.kind === 191 /* BindingElement */) {
                        // still might be illegal if declaration and usage are both binding elements (eg var [a = b, b = b] = [1, 2])
                        var errorBindingElement = ts.getAncestor(usage, 191 /* BindingElement */);
                        if (errorBindingElement) {
                            return ts.findAncestor(errorBindingElement, ts.isBindingElement) !== ts.findAncestor(declaration, ts.isBindingElement) ||
                                declaration.pos < errorBindingElement.pos;
                        }
                        // or it might be illegal if usage happens before parent variable is declared (eg var [a] = a)
                        return isBlockScopedNameDeclaredBeforeUse(ts.getAncestor(declaration, 242 /* VariableDeclaration */), usage);
                    }
                    else if (declaration.kind === 242 /* VariableDeclaration */) {
                        // still might be illegal if usage is in the initializer of the variable declaration (eg var a = a)
                        return !isImmediatelyUsedInInitializerOfBlockScopedVariable(declaration, usage);
                    }
                    else if (ts.isClassDeclaration(declaration)) {
                        // still might be illegal if the usage is within a computed property name in the class (eg class A { static p = "a"; [A.p]() {} })
                        return !ts.findAncestor(usage, function (n) { return ts.isComputedPropertyName(n) && n.parent.parent === declaration; });
                    }
                    else if (ts.isPropertyDeclaration(declaration)) {
                        // still might be illegal if a self-referencing property initializer (eg private x = this.x)
                        return !isPropertyImmediatelyReferencedWithinDeclaration(declaration, usage, /*stopAtAnyPropertyDeclaration*/ false);
                    }
                    else if (ts.isParameterPropertyDeclaration(declaration, declaration.parent)) {
                        var container_3 = ts.getEnclosingBlockScopeContainer(declaration.parent);
                        // foo = this.bar is illegal in esnext+useDefineForClassFields when bar is a parameter property
                        return !(compilerOptions.target === 99 /* ESNext */ && !!compilerOptions.useDefineForClassFields
                            && ts.getContainingClass(declaration) === ts.getContainingClass(usage)
                            && isUsedInFunctionOrInstanceProperty(usage, declaration, container_3));
                    }
                    return true;
                }
                // declaration is after usage, but it can still be legal if usage is deferred:
                // 1. inside an export specifier
                // 2. inside a function
                // 3. inside an instance property initializer, a reference to a non-instance property
                //    (except when target: "esnext" and useDefineForClassFields: true and the reference is to a parameter property)
                // 4. inside a static property initializer, a reference to a static method in the same class
                // 5. inside a TS export= declaration (since we will move the export statement during emit to avoid TDZ)
                // or if usage is in a type context:
                // 1. inside a type query (typeof in type position)
                // 2. inside a jsdoc comment
                if (usage.parent.kind === 263 /* ExportSpecifier */ || (usage.parent.kind === 259 /* ExportAssignment */ && usage.parent.isExportEquals)) {
                    // export specifiers do not use the variable, they only make it available for use
                    return true;
                }
                // When resolving symbols for exports, the `usage` location passed in can be the export site directly
                if (usage.kind === 259 /* ExportAssignment */ && usage.isExportEquals) {
                    return true;
                }
                var container = ts.getEnclosingBlockScopeContainer(declaration);
                if (!!(usage.flags & 4194304 /* JSDoc */) || isInTypeQuery(usage)) {
                    return true;
                }
                if (isUsedInFunctionOrInstanceProperty(usage, declaration, container)) {
                    if (compilerOptions.target === 99 /* ESNext */ && !!compilerOptions.useDefineForClassFields) {
                        return (ts.isPropertyDeclaration(declaration) || ts.isParameterPropertyDeclaration(declaration, declaration.parent)) &&
                            !isPropertyImmediatelyReferencedWithinDeclaration(declaration, usage, /*stopAtAnyPropertyDeclaration*/ true);
                    }
                    else {
                        return true;
                    }
                }
                return false;
                function isImmediatelyUsedInInitializerOfBlockScopedVariable(declaration, usage) {
                    var container = ts.getEnclosingBlockScopeContainer(declaration);
                    switch (declaration.parent.parent.kind) {
                        case 225 /* VariableStatement */:
                        case 230 /* ForStatement */:
                        case 232 /* ForOfStatement */:
                            // variable statement/for/for-of statement case,
                            // use site should not be inside variable declaration (initializer of declaration or binding element)
                            if (isSameScopeDescendentOf(usage, declaration, container)) {
                                return true;
                            }
                            break;
                    }
                    // ForIn/ForOf case - use site should not be used in expression part
                    var grandparent = declaration.parent.parent;
                    return ts.isForInOrOfStatement(grandparent) && isSameScopeDescendentOf(usage, grandparent.expression, container);
                }
                function isUsedInFunctionOrInstanceProperty(usage, declaration, container) {
                    return !!ts.findAncestor(usage, function (current) {
                        if (current === container) {
                            return "quit";
                        }
                        if (ts.isFunctionLike(current)) {
                            return true;
                        }
                        var initializerOfProperty = current.parent &&
                            current.parent.kind === 159 /* PropertyDeclaration */ &&
                            current.parent.initializer === current;
                        if (initializerOfProperty) {
                            if (ts.hasModifier(current.parent, 32 /* Static */)) {
                                if (declaration.kind === 161 /* MethodDeclaration */) {
                                    return true;
                                }
                            }
                            else {
                                var isDeclarationInstanceProperty = declaration.kind === 159 /* PropertyDeclaration */ && !ts.hasModifier(declaration, 32 /* Static */);
                                if (!isDeclarationInstanceProperty || ts.getContainingClass(usage) !== ts.getContainingClass(declaration)) {
                                    return true;
                                }
                            }
                        }
                        return false;
                    });
                }
                /** stopAtAnyPropertyDeclaration is used for detecting ES-standard class field use-before-def errors */
                function isPropertyImmediatelyReferencedWithinDeclaration(declaration, usage, stopAtAnyPropertyDeclaration) {
                    // always legal if usage is after declaration
                    if (usage.end > declaration.end) {
                        return false;
                    }
                    // still might be legal if usage is deferred (e.g. x: any = () => this.x)
                    // otherwise illegal if immediately referenced within the declaration (e.g. x: any = this.x)
                    var ancestorChangingReferenceScope = ts.findAncestor(usage, function (node) {
                        if (node === declaration) {
                            return "quit";
                        }
                        switch (node.kind) {
                            case 202 /* ArrowFunction */:
                                return true;
                            case 159 /* PropertyDeclaration */:
                                // even when stopping at any property declaration, they need to come from the same class
                                return stopAtAnyPropertyDeclaration &&
                                    (ts.isPropertyDeclaration(declaration) && node.parent === declaration.parent
                                        || ts.isParameterPropertyDeclaration(declaration, declaration.parent) && node.parent === declaration.parent.parent)
                                    ? "quit" : true;
                            case 223 /* Block */:
                                switch (node.parent.kind) {
                                    case 163 /* GetAccessor */:
                                    case 161 /* MethodDeclaration */:
                                    case 164 /* SetAccessor */:
                                        return true;
                                    default:
                                        return false;
                                }
                            default:
                                return false;
                        }
                    });
                    return ancestorChangingReferenceScope === undefined;
                }
            }
            /**
             * Resolve a given name for a given meaning at a given location. An error is reported if the name was not found and
             * the nameNotFoundMessage argument is not undefined. Returns the resolved symbol, or undefined if no symbol with
             * the given name can be found.
             *
             * @param isUse If true, this will count towards --noUnusedLocals / --noUnusedParameters.
             */
            function resolveName(location, name, meaning, nameNotFoundMessage, nameArg, isUse, excludeGlobals, suggestedNameNotFoundMessage) {
                if (excludeGlobals === void 0) {
                    excludeGlobals = false;
                }
                return resolveNameHelper(location, name, meaning, nameNotFoundMessage, nameArg, isUse, excludeGlobals, getSymbol, suggestedNameNotFoundMessage);
            }
            function resolveNameHelper(location, name, meaning, nameNotFoundMessage, nameArg, isUse, excludeGlobals, lookup, suggestedNameNotFoundMessage) {
                var originalLocation = location; // needed for did-you-mean error reporting, which gathers candidates starting from the original location
                var result;
                var lastLocation;
                var lastSelfReferenceLocation;
                var propertyWithInvalidInitializer;
                var associatedDeclarationForContainingInitializer;
                var withinDeferredContext = false;
                var errorLocation = location;
                var grandparent;
                var isInExternalModule = false;
                loop: while (location) {
                    // Locals of a source file are not in scope (because they get merged into the global symbol table)
                    if (location.locals && !isGlobalSourceFile(location)) {
                        if (result = lookup(location.locals, name, meaning)) {
                            var useResult = true;
                            if (ts.isFunctionLike(location) && lastLocation && lastLocation !== location.body) {
                                // symbol lookup restrictions for function-like declarations
                                // - Type parameters of a function are in scope in the entire function declaration, including the parameter
                                //   list and return type. However, local types are only in scope in the function body.
                                // - parameters are only in the scope of function body
                                // This restriction does not apply to JSDoc comment types because they are parented
                                // at a higher level than type parameters would normally be
                                if (meaning & result.flags & 788968 /* Type */ && lastLocation.kind !== 303 /* JSDocComment */) {
                                    useResult = result.flags & 262144 /* TypeParameter */
                                        // type parameters are visible in parameter list, return type and type parameter list
                                        ? lastLocation === location.type ||
                                            lastLocation.kind === 156 /* Parameter */ ||
                                            lastLocation.kind === 155 /* TypeParameter */
                                        // local types not visible outside the function body
                                        : false;
                                }
                                if (meaning & result.flags & 3 /* Variable */) {
                                    // expression inside parameter will lookup as normal variable scope when targeting es2015+
                                    var functionLocation = location;
                                    if (compilerOptions.target && compilerOptions.target >= 2 /* ES2015 */ && ts.isParameter(lastLocation) &&
                                        functionLocation.body && result.valueDeclaration.pos >= functionLocation.body.pos && result.valueDeclaration.end <= functionLocation.body.end) {
                                        useResult = false;
                                    }
                                    else if (result.flags & 1 /* FunctionScopedVariable */) {
                                        // parameters are visible only inside function body, parameter list and return type
                                        // technically for parameter list case here we might mix parameters and variables declared in function,
                                        // however it is detected separately when checking initializers of parameters
                                        // to make sure that they reference no variables declared after them.
                                        useResult =
                                            lastLocation.kind === 156 /* Parameter */ ||
                                                (lastLocation === location.type &&
                                                    !!ts.findAncestor(result.valueDeclaration, ts.isParameter));
                                    }
                                }
                            }
                            else if (location.kind === 180 /* ConditionalType */) {
                                // A type parameter declared using 'infer T' in a conditional type is visible only in
                                // the true branch of the conditional type.
                                useResult = lastLocation === location.trueType;
                            }
                            if (useResult) {
                                break loop;
                            }
                            else {
                                result = undefined;
                            }
                        }
                    }
                    withinDeferredContext = withinDeferredContext || getIsDeferredContext(location, lastLocation);
                    switch (location.kind) {
                        case 290 /* SourceFile */:
                            if (!ts.isExternalOrCommonJsModule(location))
                                break;
                            isInExternalModule = true;
                        // falls through
                        case 249 /* ModuleDeclaration */:
                            var moduleExports = getSymbolOfNode(location).exports || emptySymbols;
                            if (location.kind === 290 /* SourceFile */ || (ts.isModuleDeclaration(location) && location.flags & 8388608 /* Ambient */ && !ts.isGlobalScopeAugmentation(location))) {
                                // It's an external module. First see if the module has an export default and if the local
                                // name of that export default matches.
                                if (result = moduleExports.get("default" /* Default */)) {
                                    var localSymbol = ts.getLocalSymbolForExportDefault(result);
                                    if (localSymbol && (result.flags & meaning) && localSymbol.escapedName === name) {
                                        break loop;
                                    }
                                    result = undefined;
                                }
                                // Because of module/namespace merging, a module's exports are in scope,
                                // yet we never want to treat an export specifier as putting a member in scope.
                                // Therefore, if the name we find is purely an export specifier, it is not actually considered in scope.
                                // Two things to note about this:
                                //     1. We have to check this without calling getSymbol. The problem with calling getSymbol
                                //        on an export specifier is that it might find the export specifier itself, and try to
                                //        resolve it as an alias. This will cause the checker to consider the export specifier
                                //        a circular alias reference when it might not be.
                                //     2. We check === SymbolFlags.Alias in order to check that the symbol is *purely*
                                //        an alias. If we used &, we'd be throwing out symbols that have non alias aspects,
                                //        which is not the desired behavior.
                                var moduleExport = moduleExports.get(name);
                                if (moduleExport &&
                                    moduleExport.flags === 2097152 /* Alias */ &&
                                    (ts.getDeclarationOfKind(moduleExport, 263 /* ExportSpecifier */) || ts.getDeclarationOfKind(moduleExport, 262 /* NamespaceExport */))) {
                                    break;
                                }
                            }
                            // ES6 exports are also visible locally (except for 'default'), but commonjs exports are not (except typedefs)
                            if (name !== "default" /* Default */ && (result = lookup(moduleExports, name, meaning & 2623475 /* ModuleMember */))) {
                                if (ts.isSourceFile(location) && location.commonJsModuleIndicator && !result.declarations.some(ts.isJSDocTypeAlias)) {
                                    result = undefined;
                                }
                                else {
                                    break loop;
                                }
                            }
                            break;
                        case 248 /* EnumDeclaration */:
                            if (result = lookup(getSymbolOfNode(location).exports, name, meaning & 8 /* EnumMember */)) {
                                break loop;
                            }
                            break;
                        case 159 /* PropertyDeclaration */:
                            // TypeScript 1.0 spec (April 2014): 8.4.1
                            // Initializer expressions for instance member variables are evaluated in the scope
                            // of the class constructor body but are not permitted to reference parameters or
                            // local variables of the constructor. This effectively means that entities from outer scopes
                            // by the same name as a constructor parameter or local variable are inaccessible
                            // in initializer expressions for instance member variables.
                            if (!ts.hasModifier(location, 32 /* Static */)) {
                                var ctor = findConstructorDeclaration(location.parent);
                                if (ctor && ctor.locals) {
                                    if (lookup(ctor.locals, name, meaning & 111551 /* Value */)) {
                                        // Remember the property node, it will be used later to report appropriate error
                                        propertyWithInvalidInitializer = location;
                                    }
                                }
                            }
                            break;
                        case 245 /* ClassDeclaration */:
                        case 214 /* ClassExpression */:
                        case 246 /* InterfaceDeclaration */:
                            // The below is used to lookup type parameters within a class or interface, as they are added to the class/interface locals
                            // These can never be latebound, so the symbol's raw members are sufficient. `getMembersOfNode` cannot be used, as it would
                            // trigger resolving late-bound names, which we may already be in the process of doing while we're here!
                            if (result = lookup(getSymbolOfNode(location).members || emptySymbols, name, meaning & 788968 /* Type */)) {
                                if (!isTypeParameterSymbolDeclaredInContainer(result, location)) {
                                    // ignore type parameters not declared in this container
                                    result = undefined;
                                    break;
                                }
                                if (lastLocation && ts.hasModifier(lastLocation, 32 /* Static */)) {
                                    // TypeScript 1.0 spec (April 2014): 3.4.1
                                    // The scope of a type parameter extends over the entire declaration with which the type
                                    // parameter list is associated, with the exception of static member declarations in classes.
                                    error(errorLocation, ts.Diagnostics.Static_members_cannot_reference_class_type_parameters);
                                    return undefined;
                                }
                                break loop;
                            }
                            if (location.kind === 214 /* ClassExpression */ && meaning & 32 /* Class */) {
                                var className = location.name;
                                if (className && name === className.escapedText) {
                                    result = location.symbol;
                                    break loop;
                                }
                            }
                            break;
                        case 216 /* ExpressionWithTypeArguments */:
                            // The type parameters of a class are not in scope in the base class expression.
                            if (lastLocation === location.expression && location.parent.token === 90 /* ExtendsKeyword */) {
                                var container = location.parent.parent;
                                if (ts.isClassLike(container) && (result = lookup(getSymbolOfNode(container).members, name, meaning & 788968 /* Type */))) {
                                    if (nameNotFoundMessage) {
                                        error(errorLocation, ts.Diagnostics.Base_class_expressions_cannot_reference_class_type_parameters);
                                    }
                                    return undefined;
                                }
                            }
                            break;
                        // It is not legal to reference a class's own type parameters from a computed property name that
                        // belongs to the class. For example:
                        //
                        //   function foo<T>() { return '' }
                        //   class C<T> { // <-- Class's own type parameter T
                        //       [foo<T>()]() { } // <-- Reference to T from class's own computed property
                        //   }
                        //
                        case 154 /* ComputedPropertyName */:
                            grandparent = location.parent.parent;
                            if (ts.isClassLike(grandparent) || grandparent.kind === 246 /* InterfaceDeclaration */) {
                                // A reference to this grandparent's type parameters would be an error
                                if (result = lookup(getSymbolOfNode(grandparent).members, name, meaning & 788968 /* Type */)) {
                                    error(errorLocation, ts.Diagnostics.A_computed_property_name_cannot_reference_a_type_parameter_from_its_containing_type);
                                    return undefined;
                                }
                            }
                            break;
                        case 202 /* ArrowFunction */:
                            // when targeting ES6 or higher there is no 'arguments' in an arrow function
                            // for lower compile targets the resolved symbol is used to emit an error
                            if (compilerOptions.target >= 2 /* ES2015 */) {
                                break;
                            }
                        // falls through
                        case 161 /* MethodDeclaration */:
                        case 162 /* Constructor */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                        case 244 /* FunctionDeclaration */:
                            if (meaning & 3 /* Variable */ && name === "arguments") {
                                result = argumentsSymbol;
                                break loop;
                            }
                            break;
                        case 201 /* FunctionExpression */:
                            if (meaning & 3 /* Variable */ && name === "arguments") {
                                result = argumentsSymbol;
                                break loop;
                            }
                            if (meaning & 16 /* Function */) {
                                var functionName = location.name;
                                if (functionName && name === functionName.escapedText) {
                                    result = location.symbol;
                                    break loop;
                                }
                            }
                            break;
                        case 157 /* Decorator */:
                            // Decorators are resolved at the class declaration. Resolving at the parameter
                            // or member would result in looking up locals in the method.
                            //
                            //   function y() {}
                            //   class C {
                            //       method(@y x, y) {} // <-- decorator y should be resolved at the class declaration, not the parameter.
                            //   }
                            //
                            if (location.parent && location.parent.kind === 156 /* Parameter */) {
                                location = location.parent;
                            }
                            //
                            //   function y() {}
                            //   class C {
                            //       @y method(x, y) {} // <-- decorator y should be resolved at the class declaration, not the method.
                            //   }
                            //
                            // class Decorators are resolved outside of the class to avoid referencing type parameters of that class.
                            //
                            //   type T = number;
                            //   declare function y(x: T): any;
                            //   @param(1 as T) // <-- T should resolve to the type alias outside of class C
                            //   class C<T> {}
                            if (location.parent && (ts.isClassElement(location.parent) || location.parent.kind === 245 /* ClassDeclaration */)) {
                                location = location.parent;
                            }
                            break;
                        case 321 /* JSDocTypedefTag */:
                        case 314 /* JSDocCallbackTag */:
                        case 315 /* JSDocEnumTag */:
                            // js type aliases do not resolve names from their host, so skip past it
                            location = ts.getJSDocHost(location);
                            break;
                        case 156 /* Parameter */:
                            if (lastLocation && lastLocation === location.initializer) {
                                associatedDeclarationForContainingInitializer = location;
                            }
                            break;
                        case 191 /* BindingElement */:
                            if (lastLocation && lastLocation === location.initializer) {
                                var root = ts.getRootDeclaration(location);
                                if (root.kind === 156 /* Parameter */) {
                                    associatedDeclarationForContainingInitializer = location;
                                }
                            }
                            break;
                    }
                    if (isSelfReferenceLocation(location)) {
                        lastSelfReferenceLocation = location;
                    }
                    lastLocation = location;
                    location = location.parent;
                }
                // We just climbed up parents looking for the name, meaning that we started in a descendant node of `lastLocation`.
                // If `result === lastSelfReferenceLocation.symbol`, that means that we are somewhere inside `lastSelfReferenceLocation` looking up a name, and resolving to `lastLocation` itself.
                // That means that this is a self-reference of `lastLocation`, and shouldn't count this when considering whether `lastLocation` is used.
                if (isUse && result && (!lastSelfReferenceLocation || result !== lastSelfReferenceLocation.symbol)) {
                    result.isReferenced |= meaning;
                }
                if (!result) {
                    if (lastLocation) {
                        ts.Debug.assert(lastLocation.kind === 290 /* SourceFile */);
                        if (lastLocation.commonJsModuleIndicator && name === "exports" && meaning & lastLocation.symbol.flags) {
                            return lastLocation.symbol;
                        }
                    }
                    if (!excludeGlobals) {
                        result = lookup(globals, name, meaning);
                    }
                }
                if (!result) {
                    if (originalLocation && ts.isInJSFile(originalLocation) && originalLocation.parent) {
                        if (ts.isRequireCall(originalLocation.parent, /*checkArgumentIsStringLiteralLike*/ false)) {
                            return requireSymbol;
                        }
                    }
                }
                if (!result) {
                    if (nameNotFoundMessage) {
                        if (!errorLocation ||
                            !checkAndReportErrorForMissingPrefix(errorLocation, name, nameArg) && // TODO: GH#18217
                                !checkAndReportErrorForExtendingInterface(errorLocation) &&
                                !checkAndReportErrorForUsingTypeAsNamespace(errorLocation, name, meaning) &&
                                !checkAndReportErrorForExportingPrimitiveType(errorLocation, name) &&
                                !checkAndReportErrorForUsingTypeAsValue(errorLocation, name, meaning) &&
                                !checkAndReportErrorForUsingNamespaceModuleAsValue(errorLocation, name, meaning) &&
                                !checkAndReportErrorForUsingValueAsType(errorLocation, name, meaning)) {
                            var suggestion = void 0;
                            if (suggestedNameNotFoundMessage && suggestionCount < maximumSuggestionCount) {
                                suggestion = getSuggestedSymbolForNonexistentSymbol(originalLocation, name, meaning);
                                if (suggestion) {
                                    var suggestionName = symbolToString(suggestion);
                                    var diagnostic = error(errorLocation, suggestedNameNotFoundMessage, diagnosticName(nameArg), suggestionName);
                                    if (suggestion.valueDeclaration) {
                                        ts.addRelatedInfo(diagnostic, ts.createDiagnosticForNode(suggestion.valueDeclaration, ts.Diagnostics._0_is_declared_here, suggestionName));
                                    }
                                }
                            }
                            if (!suggestion) {
                                error(errorLocation, nameNotFoundMessage, diagnosticName(nameArg));
                            }
                            suggestionCount++;
                        }
                    }
                    return undefined;
                }
                // Perform extra checks only if error reporting was requested
                if (nameNotFoundMessage) {
                    if (propertyWithInvalidInitializer && !(compilerOptions.target === 99 /* ESNext */ && compilerOptions.useDefineForClassFields)) {
                        // We have a match, but the reference occurred within a property initializer and the identifier also binds
                        // to a local variable in the constructor where the code will be emitted. Note that this is actually allowed
                        // with ESNext+useDefineForClassFields because the scope semantics are different.
                        var propertyName = propertyWithInvalidInitializer.name;
                        error(errorLocation, ts.Diagnostics.Initializer_of_instance_member_variable_0_cannot_reference_identifier_1_declared_in_the_constructor, ts.declarationNameToString(propertyName), diagnosticName(nameArg));
                        return undefined;
                    }
                    // Only check for block-scoped variable if we have an error location and are looking for the
                    // name with variable meaning
                    //      For example,
                    //          declare module foo {
                    //              interface bar {}
                    //          }
                    //      const foo/*1*/: foo/*2*/.bar;
                    // The foo at /*1*/ and /*2*/ will share same symbol with two meanings:
                    // block-scoped variable and namespace module. However, only when we
                    // try to resolve name in /*1*/ which is used in variable position,
                    // we want to check for block-scoped
                    if (errorLocation &&
                        (meaning & 2 /* BlockScopedVariable */ ||
                            ((meaning & 32 /* Class */ || meaning & 384 /* Enum */) && (meaning & 111551 /* Value */) === 111551 /* Value */))) {
                        var exportOrLocalSymbol = getExportSymbolOfValueSymbolIfExported(result);
                        if (exportOrLocalSymbol.flags & 2 /* BlockScopedVariable */ || exportOrLocalSymbol.flags & 32 /* Class */ || exportOrLocalSymbol.flags & 384 /* Enum */) {
                            checkResolvedBlockScopedVariable(exportOrLocalSymbol, errorLocation);
                        }
                    }
                    // If we're in an external module, we can't reference value symbols created from UMD export declarations
                    if (result && isInExternalModule && (meaning & 111551 /* Value */) === 111551 /* Value */ && !(originalLocation.flags & 4194304 /* JSDoc */)) {
                        var merged = getMergedSymbol(result);
                        if (ts.length(merged.declarations) && ts.every(merged.declarations, function (d) { return ts.isNamespaceExportDeclaration(d) || ts.isSourceFile(d) && !!d.symbol.globalExports; })) {
                            errorOrSuggestion(!compilerOptions.allowUmdGlobalAccess, errorLocation, ts.Diagnostics._0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead, ts.unescapeLeadingUnderscores(name));
                        }
                    }
                    // If we're in a parameter initializer, we can't reference the values of the parameter whose initializer we're within or parameters to the right
                    if (result && associatedDeclarationForContainingInitializer && !withinDeferredContext && (meaning & 111551 /* Value */) === 111551 /* Value */) {
                        var candidate = getMergedSymbol(getLateBoundSymbol(result));
                        var root = ts.getRootDeclaration(associatedDeclarationForContainingInitializer);
                        // A parameter initializer or binding pattern initializer within a parameter cannot refer to itself
                        if (candidate === getSymbolOfNode(associatedDeclarationForContainingInitializer)) {
                            error(errorLocation, ts.Diagnostics.Parameter_0_cannot_be_referenced_in_its_initializer, ts.declarationNameToString(associatedDeclarationForContainingInitializer.name));
                        }
                        // And it cannot refer to any declarations which come after it
                        else if (candidate.valueDeclaration && candidate.valueDeclaration.pos > associatedDeclarationForContainingInitializer.pos && root.parent.locals && lookup(root.parent.locals, candidate.escapedName, meaning) === candidate) {
                            error(errorLocation, ts.Diagnostics.Initializer_of_parameter_0_cannot_reference_identifier_1_declared_after_it, ts.declarationNameToString(associatedDeclarationForContainingInitializer.name), ts.declarationNameToString(errorLocation));
                        }
                    }
                    if (result && errorLocation && meaning & 111551 /* Value */ && result.flags & 2097152 /* Alias */) {
                        checkSymbolUsageInExpressionContext(result, name, errorLocation);
                    }
                }
                return result;
            }
            function checkSymbolUsageInExpressionContext(symbol, name, useSite) {
                if (!ts.isValidTypeOnlyAliasUseSite(useSite)) {
                    var typeOnlyDeclaration = getTypeOnlyAliasDeclaration(symbol);
                    if (typeOnlyDeclaration) {
                        var isExport = ts.typeOnlyDeclarationIsExport(typeOnlyDeclaration);
                        var message = isExport
                            ? ts.Diagnostics._0_cannot_be_used_as_a_value_because_it_was_exported_using_export_type
                            : ts.Diagnostics._0_cannot_be_used_as_a_value_because_it_was_imported_using_import_type;
                        var relatedMessage = isExport
                            ? ts.Diagnostics._0_was_exported_here
                            : ts.Diagnostics._0_was_imported_here;
                        var unescapedName = ts.unescapeLeadingUnderscores(name);
                        ts.addRelatedInfo(error(useSite, message, unescapedName), ts.createDiagnosticForNode(typeOnlyDeclaration, relatedMessage, unescapedName));
                    }
                }
            }
            function getIsDeferredContext(location, lastLocation) {
                if (location.kind !== 202 /* ArrowFunction */ && location.kind !== 201 /* FunctionExpression */) {
                    // initializers in instance property declaration of class like entities are executed in constructor and thus deferred
                    return ts.isTypeQueryNode(location) || ((ts.isFunctionLikeDeclaration(location) ||
                        (location.kind === 159 /* PropertyDeclaration */ && !ts.hasModifier(location, 32 /* Static */))) && (!lastLocation || lastLocation !== location.name)); // A name is evaluated within the enclosing scope - so it shouldn't count as deferred
                }
                if (lastLocation && lastLocation === location.name) {
                    return false;
                }
                // generator functions and async functions are not inlined in control flow when immediately invoked
                if (location.asteriskToken || ts.hasModifier(location, 256 /* Async */)) {
                    return true;
                }
                return !ts.getImmediatelyInvokedFunctionExpression(location);
            }
            function isSelfReferenceLocation(node) {
                switch (node.kind) {
                    case 244 /* FunctionDeclaration */:
                    case 245 /* ClassDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 247 /* TypeAliasDeclaration */:
                    case 249 /* ModuleDeclaration */: // For `namespace N { N; }`
                        return true;
                    default:
                        return false;
                }
            }
            function diagnosticName(nameArg) {
                return ts.isString(nameArg) ? ts.unescapeLeadingUnderscores(nameArg) : ts.declarationNameToString(nameArg);
            }
            function isTypeParameterSymbolDeclaredInContainer(symbol, container) {
                for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                    var decl = _a[_i];
                    if (decl.kind === 155 /* TypeParameter */) {
                        var parent = ts.isJSDocTemplateTag(decl.parent) ? ts.getJSDocHost(decl.parent) : decl.parent;
                        if (parent === container) {
                            return !(ts.isJSDocTemplateTag(decl.parent) && ts.find(decl.parent.parent.tags, ts.isJSDocTypeAlias)); // TODO: GH#18217
                        }
                    }
                }
                return false;
            }
            function checkAndReportErrorForMissingPrefix(errorLocation, name, nameArg) {
                if (!ts.isIdentifier(errorLocation) || errorLocation.escapedText !== name || isTypeReferenceIdentifier(errorLocation) || isInTypeQuery(errorLocation)) {
                    return false;
                }
                var container = ts.getThisContainer(errorLocation, /*includeArrowFunctions*/ false);
                var location = container;
                while (location) {
                    if (ts.isClassLike(location.parent)) {
                        var classSymbol = getSymbolOfNode(location.parent);
                        if (!classSymbol) {
                            break;
                        }
                        // Check to see if a static member exists.
                        var constructorType = getTypeOfSymbol(classSymbol);
                        if (getPropertyOfType(constructorType, name)) {
                            error(errorLocation, ts.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0, diagnosticName(nameArg), symbolToString(classSymbol));
                            return true;
                        }
                        // No static member is present.
                        // Check if we're in an instance method and look for a relevant instance member.
                        if (location === container && !ts.hasModifier(location, 32 /* Static */)) {
                            var instanceType = getDeclaredTypeOfSymbol(classSymbol).thisType; // TODO: GH#18217
                            if (getPropertyOfType(instanceType, name)) {
                                error(errorLocation, ts.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0, diagnosticName(nameArg));
                                return true;
                            }
                        }
                    }
                    location = location.parent;
                }
                return false;
            }
            function checkAndReportErrorForExtendingInterface(errorLocation) {
                var expression = getEntityNameForExtendingInterface(errorLocation);
                if (expression && resolveEntityName(expression, 64 /* Interface */, /*ignoreErrors*/ true)) {
                    error(errorLocation, ts.Diagnostics.Cannot_extend_an_interface_0_Did_you_mean_implements, ts.getTextOfNode(expression));
                    return true;
                }
                return false;
            }
            /**
             * Climbs up parents to an ExpressionWithTypeArguments, and returns its expression,
             * but returns undefined if that expression is not an EntityNameExpression.
             */
            function getEntityNameForExtendingInterface(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                    case 194 /* PropertyAccessExpression */:
                        return node.parent ? getEntityNameForExtendingInterface(node.parent) : undefined;
                    case 216 /* ExpressionWithTypeArguments */:
                        if (ts.isEntityNameExpression(node.expression)) {
                            return node.expression;
                        }
                    // falls through
                    default:
                        return undefined;
                }
            }
            function checkAndReportErrorForUsingTypeAsNamespace(errorLocation, name, meaning) {
                var namespaceMeaning = 1920 /* Namespace */ | (ts.isInJSFile(errorLocation) ? 111551 /* Value */ : 0);
                if (meaning === namespaceMeaning) {
                    var symbol = resolveSymbol(resolveName(errorLocation, name, 788968 /* Type */ & ~namespaceMeaning, /*nameNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ false));
                    var parent = errorLocation.parent;
                    if (symbol) {
                        if (ts.isQualifiedName(parent)) {
                            ts.Debug.assert(parent.left === errorLocation, "Should only be resolving left side of qualified name as a namespace");
                            var propName = parent.right.escapedText;
                            var propType = getPropertyOfType(getDeclaredTypeOfSymbol(symbol), propName);
                            if (propType) {
                                error(parent, ts.Diagnostics.Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_property_1_in_0_with_0_1, ts.unescapeLeadingUnderscores(name), ts.unescapeLeadingUnderscores(propName));
                                return true;
                            }
                        }
                        error(errorLocation, ts.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_namespace_here, ts.unescapeLeadingUnderscores(name));
                        return true;
                    }
                }
                return false;
            }
            function checkAndReportErrorForUsingValueAsType(errorLocation, name, meaning) {
                if (meaning & (788968 /* Type */ & ~1920 /* Namespace */)) {
                    var symbol = resolveSymbol(resolveName(errorLocation, name, ~788968 /* Type */ & 111551 /* Value */, /*nameNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ false));
                    if (symbol && !(symbol.flags & 1920 /* Namespace */)) {
                        error(errorLocation, ts.Diagnostics._0_refers_to_a_value_but_is_being_used_as_a_type_here, ts.unescapeLeadingUnderscores(name));
                        return true;
                    }
                }
                return false;
            }
            function isPrimitiveTypeName(name) {
                return name === "any" || name === "string" || name === "number" || name === "boolean" || name === "never" || name === "unknown";
            }
            function checkAndReportErrorForExportingPrimitiveType(errorLocation, name) {
                if (isPrimitiveTypeName(name) && errorLocation.parent.kind === 263 /* ExportSpecifier */) {
                    error(errorLocation, ts.Diagnostics.Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module, name);
                    return true;
                }
                return false;
            }
            function checkAndReportErrorForUsingTypeAsValue(errorLocation, name, meaning) {
                if (meaning & (111551 /* Value */ & ~1024 /* NamespaceModule */)) {
                    if (isPrimitiveTypeName(name)) {
                        error(errorLocation, ts.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here, ts.unescapeLeadingUnderscores(name));
                        return true;
                    }
                    var symbol = resolveSymbol(resolveName(errorLocation, name, 788968 /* Type */ & ~111551 /* Value */, /*nameNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ false));
                    if (symbol && !(symbol.flags & 1024 /* NamespaceModule */)) {
                        var message = isES2015OrLaterConstructorName(name)
                            ? ts.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_es2015_or_later
                            : ts.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here;
                        error(errorLocation, message, ts.unescapeLeadingUnderscores(name));
                        return true;
                    }
                }
                return false;
            }
            function isES2015OrLaterConstructorName(n) {
                switch (n) {
                    case "Promise":
                    case "Symbol":
                    case "Map":
                    case "WeakMap":
                    case "Set":
                    case "WeakSet":
                        return true;
                }
                return false;
            }
            function checkAndReportErrorForUsingNamespaceModuleAsValue(errorLocation, name, meaning) {
                if (meaning & (111551 /* Value */ & ~1024 /* NamespaceModule */ & ~788968 /* Type */)) {
                    var symbol = resolveSymbol(resolveName(errorLocation, name, 1024 /* NamespaceModule */ & ~111551 /* Value */, /*nameNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ false));
                    if (symbol) {
                        error(errorLocation, ts.Diagnostics.Cannot_use_namespace_0_as_a_value, ts.unescapeLeadingUnderscores(name));
                        return true;
                    }
                }
                else if (meaning & (788968 /* Type */ & ~1024 /* NamespaceModule */ & ~111551 /* Value */)) {
                    var symbol = resolveSymbol(resolveName(errorLocation, name, (512 /* ValueModule */ | 1024 /* NamespaceModule */) & ~788968 /* Type */, /*nameNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ false));
                    if (symbol) {
                        error(errorLocation, ts.Diagnostics.Cannot_use_namespace_0_as_a_type, ts.unescapeLeadingUnderscores(name));
                        return true;
                    }
                }
                return false;
            }
            function checkResolvedBlockScopedVariable(result, errorLocation) {
                ts.Debug.assert(!!(result.flags & 2 /* BlockScopedVariable */ || result.flags & 32 /* Class */ || result.flags & 384 /* Enum */));
                if (result.flags & (16 /* Function */ | 1 /* FunctionScopedVariable */ | 67108864 /* Assignment */) && result.flags & 32 /* Class */) {
                    // constructor functions aren't block scoped
                    return;
                }
                // Block-scoped variables cannot be used before their definition
                var declaration = ts.find(result.declarations, function (d) { return ts.isBlockOrCatchScoped(d) || ts.isClassLike(d) || (d.kind === 248 /* EnumDeclaration */); });
                if (declaration === undefined)
                    return ts.Debug.fail("checkResolvedBlockScopedVariable could not find block-scoped declaration");
                if (!(declaration.flags & 8388608 /* Ambient */) && !isBlockScopedNameDeclaredBeforeUse(declaration, errorLocation)) {
                    var diagnosticMessage = void 0;
                    var declarationName = ts.declarationNameToString(ts.getNameOfDeclaration(declaration));
                    if (result.flags & 2 /* BlockScopedVariable */) {
                        diagnosticMessage = error(errorLocation, ts.Diagnostics.Block_scoped_variable_0_used_before_its_declaration, declarationName);
                    }
                    else if (result.flags & 32 /* Class */) {
                        diagnosticMessage = error(errorLocation, ts.Diagnostics.Class_0_used_before_its_declaration, declarationName);
                    }
                    else if (result.flags & 256 /* RegularEnum */) {
                        diagnosticMessage = error(errorLocation, ts.Diagnostics.Enum_0_used_before_its_declaration, declarationName);
                    }
                    else {
                        ts.Debug.assert(!!(result.flags & 128 /* ConstEnum */));
                        if (compilerOptions.preserveConstEnums) {
                            diagnosticMessage = error(errorLocation, ts.Diagnostics.Class_0_used_before_its_declaration, declarationName);
                        }
                    }
                    if (diagnosticMessage) {
                        ts.addRelatedInfo(diagnosticMessage, ts.createDiagnosticForNode(declaration, ts.Diagnostics._0_is_declared_here, declarationName));
                    }
                }
            }
            /* Starting from 'initial' node walk up the parent chain until 'stopAt' node is reached.
             * If at any point current node is equal to 'parent' node - return true.
             * Return false if 'stopAt' node is reached or isFunctionLike(current) === true.
             */
            function isSameScopeDescendentOf(initial, parent, stopAt) {
                return !!parent && !!ts.findAncestor(initial, function (n) { return n === stopAt || ts.isFunctionLike(n) ? "quit" : n === parent; });
            }
            function getAnyImportSyntax(node) {
                switch (node.kind) {
                    case 253 /* ImportEqualsDeclaration */:
                        return node;
                    case 255 /* ImportClause */:
                        return node.parent;
                    case 256 /* NamespaceImport */:
                        return node.parent.parent;
                    case 258 /* ImportSpecifier */:
                        return node.parent.parent.parent;
                    default:
                        return undefined;
                }
            }
            function getDeclarationOfAliasSymbol(symbol) {
                return ts.find(symbol.declarations, isAliasSymbolDeclaration);
            }
            /**
             * An alias symbol is created by one of the following declarations:
             * import <symbol> = ...
             * import <symbol> from ...
             * import * as <symbol> from ...
             * import { x as <symbol> } from ...
             * export { x as <symbol> } from ...
             * export * as ns <symbol> from ...
             * export = <EntityNameExpression>
             * export default <EntityNameExpression>
             * module.exports = <EntityNameExpression>
             * {<Identifier>}
             * {name: <EntityNameExpression>}
             */
            function isAliasSymbolDeclaration(node) {
                return node.kind === 253 /* ImportEqualsDeclaration */ ||
                    node.kind === 252 /* NamespaceExportDeclaration */ ||
                    node.kind === 255 /* ImportClause */ && !!node.name ||
                    node.kind === 256 /* NamespaceImport */ ||
                    node.kind === 262 /* NamespaceExport */ ||
                    node.kind === 258 /* ImportSpecifier */ ||
                    node.kind === 263 /* ExportSpecifier */ ||
                    node.kind === 259 /* ExportAssignment */ && ts.exportAssignmentIsAlias(node) ||
                    ts.isBinaryExpression(node) && ts.getAssignmentDeclarationKind(node) === 2 /* ModuleExports */ && ts.exportAssignmentIsAlias(node) ||
                    ts.isPropertyAccessExpression(node)
                        && ts.isBinaryExpression(node.parent)
                        && node.parent.left === node
                        && node.parent.operatorToken.kind === 62 /* EqualsToken */
                        && isAliasableOrJsExpression(node.parent.right) ||
                    node.kind === 282 /* ShorthandPropertyAssignment */ ||
                    node.kind === 281 /* PropertyAssignment */ && isAliasableOrJsExpression(node.initializer);
            }
            function isAliasableOrJsExpression(e) {
                return ts.isAliasableExpression(e) || ts.isFunctionExpression(e) && isJSConstructor(e);
            }
            function getTargetOfImportEqualsDeclaration(node, dontResolveAlias) {
                if (node.moduleReference.kind === 265 /* ExternalModuleReference */) {
                    var immediate = resolveExternalModuleName(node, ts.getExternalModuleImportEqualsDeclarationExpression(node));
                    var resolved_4 = resolveExternalModuleSymbol(immediate);
                    markSymbolOfAliasDeclarationIfTypeOnly(node, immediate, resolved_4, /*overwriteEmpty*/ false);
                    return resolved_4;
                }
                var resolved = getSymbolOfPartOfRightHandSideOfImportEquals(node.moduleReference, dontResolveAlias);
                checkAndReportErrorForResolvingImportAliasToTypeOnlySymbol(node, resolved);
                return resolved;
            }
            function checkAndReportErrorForResolvingImportAliasToTypeOnlySymbol(node, resolved) {
                if (markSymbolOfAliasDeclarationIfTypeOnly(node, /*immediateTarget*/ undefined, resolved, /*overwriteEmpty*/ false)) {
                    var typeOnlyDeclaration = getTypeOnlyAliasDeclaration(getSymbolOfNode(node));
                    var isExport = ts.typeOnlyDeclarationIsExport(typeOnlyDeclaration);
                    var message = isExport
                        ? ts.Diagnostics.An_import_alias_cannot_reference_a_declaration_that_was_exported_using_export_type
                        : ts.Diagnostics.An_import_alias_cannot_reference_a_declaration_that_was_imported_using_import_type;
                    var relatedMessage = isExport
                        ? ts.Diagnostics._0_was_exported_here
                        : ts.Diagnostics._0_was_imported_here;
                    // Non-null assertion is safe because the optionality comes from ImportClause,
                    // but if an ImportClause was the typeOnlyDeclaration, it had to have a `name`.
                    var name = ts.unescapeLeadingUnderscores(typeOnlyDeclaration.name.escapedText);
                    ts.addRelatedInfo(error(node.moduleReference, message), ts.createDiagnosticForNode(typeOnlyDeclaration, relatedMessage, name));
                }
            }
            function resolveExportByName(moduleSymbol, name, sourceNode, dontResolveAlias) {
                var exportValue = moduleSymbol.exports.get("export=" /* ExportEquals */);
                if (exportValue) {
                    return getPropertyOfType(getTypeOfSymbol(exportValue), name);
                }
                var exportSymbol = moduleSymbol.exports.get(name);
                var resolved = resolveSymbol(exportSymbol, dontResolveAlias);
                markSymbolOfAliasDeclarationIfTypeOnly(sourceNode, exportSymbol, resolved, /*overwriteEmpty*/ false);
                return resolved;
            }
            function isSyntacticDefault(node) {
                return ((ts.isExportAssignment(node) && !node.isExportEquals) || ts.hasModifier(node, 512 /* Default */) || ts.isExportSpecifier(node));
            }
            function canHaveSyntheticDefault(file, moduleSymbol, dontResolveAlias) {
                if (!allowSyntheticDefaultImports) {
                    return false;
                }
                // Declaration files (and ambient modules)
                if (!file || file.isDeclarationFile) {
                    // Definitely cannot have a synthetic default if they have a syntactic default member specified
                    var defaultExportSymbol = resolveExportByName(moduleSymbol, "default" /* Default */, /*sourceNode*/ undefined, /*dontResolveAlias*/ true); // Dont resolve alias because we want the immediately exported symbol's declaration
                    if (defaultExportSymbol && ts.some(defaultExportSymbol.declarations, isSyntacticDefault)) {
                        return false;
                    }
                    // It _might_ still be incorrect to assume there is no __esModule marker on the import at runtime, even if there is no `default` member
                    // So we check a bit more,
                    if (resolveExportByName(moduleSymbol, ts.escapeLeadingUnderscores("__esModule"), /*sourceNode*/ undefined, dontResolveAlias)) {
                        // If there is an `__esModule` specified in the declaration (meaning someone explicitly added it or wrote it in their code),
                        // it definitely is a module and does not have a synthetic default
                        return false;
                    }
                    // There are _many_ declaration files not written with esmodules in mind that still get compiled into a format with __esModule set
                    // Meaning there may be no default at runtime - however to be on the permissive side, we allow access to a synthetic default member
                    // as there is no marker to indicate if the accompanying JS has `__esModule` or not, or is even native esm
                    return true;
                }
                // TypeScript files never have a synthetic default (as they are always emitted with an __esModule marker) _unless_ they contain an export= statement
                if (!ts.isSourceFileJS(file)) {
                    return hasExportAssignmentSymbol(moduleSymbol);
                }
                // JS files have a synthetic default if they do not contain ES2015+ module syntax (export = is not valid in js) _and_ do not have an __esModule marker
                return !file.externalModuleIndicator && !resolveExportByName(moduleSymbol, ts.escapeLeadingUnderscores("__esModule"), /*sourceNode*/ undefined, dontResolveAlias);
            }
            function getTargetOfImportClause(node, dontResolveAlias) {
                var moduleSymbol = resolveExternalModuleName(node, node.parent.moduleSpecifier);
                if (moduleSymbol) {
                    var exportDefaultSymbol = void 0;
                    if (ts.isShorthandAmbientModuleSymbol(moduleSymbol)) {
                        exportDefaultSymbol = moduleSymbol;
                    }
                    else {
                        exportDefaultSymbol = resolveExportByName(moduleSymbol, "default" /* Default */, node, dontResolveAlias);
                    }
                    var file = ts.find(moduleSymbol.declarations, ts.isSourceFile);
                    var hasSyntheticDefault = canHaveSyntheticDefault(file, moduleSymbol, dontResolveAlias);
                    if (!exportDefaultSymbol && !hasSyntheticDefault) {
                        if (hasExportAssignmentSymbol(moduleSymbol)) {
                            var compilerOptionName = moduleKind >= ts.ModuleKind.ES2015 ? "allowSyntheticDefaultImports" : "esModuleInterop";
                            var exportEqualsSymbol = moduleSymbol.exports.get("export=" /* ExportEquals */);
                            var exportAssignment = exportEqualsSymbol.valueDeclaration;
                            var err = error(node.name, ts.Diagnostics.Module_0_can_only_be_default_imported_using_the_1_flag, symbolToString(moduleSymbol), compilerOptionName);
                            ts.addRelatedInfo(err, ts.createDiagnosticForNode(exportAssignment, ts.Diagnostics.This_module_is_declared_with_using_export_and_can_only_be_used_with_a_default_import_when_using_the_0_flag, compilerOptionName));
                        }
                        else {
                            if (moduleSymbol.exports && moduleSymbol.exports.has(node.symbol.escapedName)) {
                                error(node.name, ts.Diagnostics.Module_0_has_no_default_export_Did_you_mean_to_use_import_1_from_0_instead, symbolToString(moduleSymbol), symbolToString(node.symbol));
                            }
                            else {
                                error(node.name, ts.Diagnostics.Module_0_has_no_default_export, symbolToString(moduleSymbol));
                            }
                        }
                    }
                    else if (hasSyntheticDefault) {
                        // per emit behavior, a synthetic default overrides a "real" .default member if `__esModule` is not present
                        var resolved = resolveExternalModuleSymbol(moduleSymbol, dontResolveAlias) || resolveSymbol(moduleSymbol, dontResolveAlias);
                        markSymbolOfAliasDeclarationIfTypeOnly(node, moduleSymbol, resolved, /*overwriteTypeOnly*/ false);
                        return resolved;
                    }
                    markSymbolOfAliasDeclarationIfTypeOnly(node, exportDefaultSymbol, /*finalTarget*/ undefined, /*overwriteTypeOnly*/ false);
                    return exportDefaultSymbol;
                }
            }
            function getTargetOfNamespaceImport(node, dontResolveAlias) {
                var moduleSpecifier = node.parent.parent.moduleSpecifier;
                var immediate = resolveExternalModuleName(node, moduleSpecifier);
                var resolved = resolveESModuleSymbol(immediate, moduleSpecifier, dontResolveAlias, /*suppressUsageError*/ false);
                markSymbolOfAliasDeclarationIfTypeOnly(node, immediate, resolved, /*overwriteEmpty*/ false);
                return resolved;
            }
            function getTargetOfNamespaceExport(node, dontResolveAlias) {
                var moduleSpecifier = node.parent.moduleSpecifier;
                var immediate = moduleSpecifier && resolveExternalModuleName(node, moduleSpecifier);
                var resolved = moduleSpecifier && resolveESModuleSymbol(immediate, moduleSpecifier, dontResolveAlias, /*suppressUsageError*/ false);
                markSymbolOfAliasDeclarationIfTypeOnly(node, immediate, resolved, /*overwriteEmpty*/ false);
                return resolved;
            }
            // This function creates a synthetic symbol that combines the value side of one symbol with the
            // type/namespace side of another symbol. Consider this example:
            //
            //   declare module graphics {
            //       interface Point {
            //           x: number;
            //           y: number;
            //       }
            //   }
            //   declare var graphics: {
            //       Point: new (x: number, y: number) => graphics.Point;
            //   }
            //   declare module "graphics" {
            //       export = graphics;
            //   }
            //
            // An 'import { Point } from "graphics"' needs to create a symbol that combines the value side 'Point'
            // property with the type/namespace side interface 'Point'.
            function combineValueAndTypeSymbols(valueSymbol, typeSymbol) {
                if (valueSymbol === unknownSymbol && typeSymbol === unknownSymbol) {
                    return unknownSymbol;
                }
                if (valueSymbol.flags & (788968 /* Type */ | 1920 /* Namespace */)) {
                    return valueSymbol;
                }
                var result = createSymbol(valueSymbol.flags | typeSymbol.flags, valueSymbol.escapedName);
                result.declarations = ts.deduplicate(ts.concatenate(valueSymbol.declarations, typeSymbol.declarations), ts.equateValues);
                result.parent = valueSymbol.parent || typeSymbol.parent;
                if (valueSymbol.valueDeclaration)
                    result.valueDeclaration = valueSymbol.valueDeclaration;
                if (typeSymbol.members)
                    result.members = ts.cloneMap(typeSymbol.members);
                if (valueSymbol.exports)
                    result.exports = ts.cloneMap(valueSymbol.exports);
                return result;
            }
            function getExportOfModule(symbol, specifier, dontResolveAlias) {
                var _a;
                if (symbol.flags & 1536 /* Module */) {
                    var name = ((_a = specifier.propertyName) !== null && _a !== void 0 ? _a : specifier.name).escapedText;
                    var exportSymbol = getExportsOfSymbol(symbol).get(name);
                    var resolved = resolveSymbol(exportSymbol, dontResolveAlias);
                    markSymbolOfAliasDeclarationIfTypeOnly(specifier, exportSymbol, resolved, /*overwriteEmpty*/ false);
                    return resolved;
                }
            }
            function getPropertyOfVariable(symbol, name) {
                if (symbol.flags & 3 /* Variable */) {
                    var typeAnnotation = symbol.valueDeclaration.type;
                    if (typeAnnotation) {
                        return resolveSymbol(getPropertyOfType(getTypeFromTypeNode(typeAnnotation), name));
                    }
                }
            }
            function getExternalModuleMember(node, specifier, dontResolveAlias) {
                if (dontResolveAlias === void 0) {
                    dontResolveAlias = false;
                }
                var _a;
                var moduleSymbol = resolveExternalModuleName(node, node.moduleSpecifier); // TODO: GH#18217
                var name = specifier.propertyName || specifier.name;
                var suppressInteropError = name.escapedText === "default" /* Default */ && !!(compilerOptions.allowSyntheticDefaultImports || compilerOptions.esModuleInterop);
                var targetSymbol = resolveESModuleSymbol(moduleSymbol, node.moduleSpecifier, dontResolveAlias, suppressInteropError);
                if (targetSymbol) {
                    if (name.escapedText) {
                        if (ts.isShorthandAmbientModuleSymbol(moduleSymbol)) {
                            return moduleSymbol;
                        }
                        var symbolFromVariable = void 0;
                        // First check if module was specified with "export=". If so, get the member from the resolved type
                        if (moduleSymbol && moduleSymbol.exports && moduleSymbol.exports.get("export=" /* ExportEquals */)) {
                            symbolFromVariable = getPropertyOfType(getTypeOfSymbol(targetSymbol), name.escapedText);
                        }
                        else {
                            symbolFromVariable = getPropertyOfVariable(targetSymbol, name.escapedText);
                        }
                        // if symbolFromVariable is export - get its final target
                        symbolFromVariable = resolveSymbol(symbolFromVariable, dontResolveAlias);
                        var symbolFromModule = getExportOfModule(targetSymbol, specifier, dontResolveAlias);
                        if (symbolFromModule === undefined && name.escapedText === "default" /* Default */) {
                            var file = ts.find(moduleSymbol.declarations, ts.isSourceFile);
                            if (canHaveSyntheticDefault(file, moduleSymbol, dontResolveAlias)) {
                                symbolFromModule = resolveExternalModuleSymbol(moduleSymbol, dontResolveAlias) || resolveSymbol(moduleSymbol, dontResolveAlias);
                            }
                        }
                        var symbol = symbolFromModule && symbolFromVariable && symbolFromModule !== symbolFromVariable ?
                            combineValueAndTypeSymbols(symbolFromVariable, symbolFromModule) :
                            symbolFromModule || symbolFromVariable;
                        if (!symbol) {
                            var moduleName = getFullyQualifiedName(moduleSymbol, node);
                            var declarationName = ts.declarationNameToString(name);
                            var suggestion = getSuggestedSymbolForNonexistentModule(name, targetSymbol);
                            if (suggestion !== undefined) {
                                var suggestionName = symbolToString(suggestion);
                                var diagnostic = error(name, ts.Diagnostics.Module_0_has_no_exported_member_1_Did_you_mean_2, moduleName, declarationName, suggestionName);
                                if (suggestion.valueDeclaration) {
                                    ts.addRelatedInfo(diagnostic, ts.createDiagnosticForNode(suggestion.valueDeclaration, ts.Diagnostics._0_is_declared_here, suggestionName));
                                }
                            }
                            else {
                                if ((_a = moduleSymbol.exports) === null || _a === void 0 ? void 0 : _a.has("default" /* Default */)) {
                                    error(name, ts.Diagnostics.Module_0_has_no_exported_member_1_Did_you_mean_to_use_import_1_from_0_instead, moduleName, declarationName);
                                }
                                else {
                                    reportNonExportedMember(name, declarationName, moduleSymbol, moduleName);
                                }
                            }
                        }
                        return symbol;
                    }
                }
            }
            function reportNonExportedMember(name, declarationName, moduleSymbol, moduleName) {
                var _a;
                var localSymbol = (_a = moduleSymbol.valueDeclaration.locals) === null || _a === void 0 ? void 0 : _a.get(name.escapedText);
                var exports = moduleSymbol.exports;
                if (localSymbol) {
                    var exportedSymbol = exports && !exports.has("export=" /* ExportEquals */)
                        ? ts.find(symbolsToArray(exports), function (symbol) { return !!getSymbolIfSameReference(symbol, localSymbol); })
                        : undefined;
                    var diagnostic = exportedSymbol
                        ? error(name, ts.Diagnostics.Module_0_declares_1_locally_but_it_is_exported_as_2, moduleName, declarationName, symbolToString(exportedSymbol))
                        : error(name, ts.Diagnostics.Module_0_declares_1_locally_but_it_is_not_exported, moduleName, declarationName);
                    ts.addRelatedInfo.apply(void 0, __spreadArrays([diagnostic], ts.map(localSymbol.declarations, function (decl, index) {
                        return ts.createDiagnosticForNode(decl, index === 0 ? ts.Diagnostics._0_is_declared_here : ts.Diagnostics.and_here, declarationName);
                    })));
                }
                else {
                    error(name, ts.Diagnostics.Module_0_has_no_exported_member_1, moduleName, declarationName);
                }
            }
            function getTargetOfImportSpecifier(node, dontResolveAlias) {
                var resolved = getExternalModuleMember(node.parent.parent.parent, node, dontResolveAlias);
                markSymbolOfAliasDeclarationIfTypeOnly(node, /*immediateTarget*/ undefined, resolved, /*overwriteEmpty*/ false);
                return resolved;
            }
            function getTargetOfNamespaceExportDeclaration(node, dontResolveAlias) {
                var resolved = resolveExternalModuleSymbol(node.parent.symbol, dontResolveAlias);
                markSymbolOfAliasDeclarationIfTypeOnly(node, /*immediateTarget*/ undefined, resolved, /*overwriteEmpty*/ false);
                return resolved;
            }
            function getTargetOfExportSpecifier(node, meaning, dontResolveAlias) {
                var resolved = node.parent.parent.moduleSpecifier ?
                    getExternalModuleMember(node.parent.parent, node, dontResolveAlias) :
                    resolveEntityName(node.propertyName || node.name, meaning, /*ignoreErrors*/ false, dontResolveAlias);
                markSymbolOfAliasDeclarationIfTypeOnly(node, /*immediateTarget*/ undefined, resolved, /*overwriteEmpty*/ false);
                return resolved;
            }
            function getTargetOfExportAssignment(node, dontResolveAlias) {
                var expression = (ts.isExportAssignment(node) ? node.expression : node.right);
                var resolved = getTargetOfAliasLikeExpression(expression, dontResolveAlias);
                markSymbolOfAliasDeclarationIfTypeOnly(node, /*immediateTarget*/ undefined, resolved, /*overwriteEmpty*/ false);
                return resolved;
            }
            function getTargetOfAliasLikeExpression(expression, dontResolveAlias) {
                if (ts.isClassExpression(expression)) {
                    return checkExpressionCached(expression).symbol;
                }
                if (!ts.isEntityName(expression) && !ts.isEntityNameExpression(expression)) {
                    return undefined;
                }
                var aliasLike = resolveEntityName(expression, 111551 /* Value */ | 788968 /* Type */ | 1920 /* Namespace */, /*ignoreErrors*/ true, dontResolveAlias);
                if (aliasLike) {
                    return aliasLike;
                }
                checkExpressionCached(expression);
                return getNodeLinks(expression).resolvedSymbol;
            }
            function getTargetOfPropertyAssignment(node, dontRecursivelyResolve) {
                var expression = node.initializer;
                return getTargetOfAliasLikeExpression(expression, dontRecursivelyResolve);
            }
            function getTargetOfPropertyAccessExpression(node, dontRecursivelyResolve) {
                if (!(ts.isBinaryExpression(node.parent) && node.parent.left === node && node.parent.operatorToken.kind === 62 /* EqualsToken */)) {
                    return undefined;
                }
                return getTargetOfAliasLikeExpression(node.parent.right, dontRecursivelyResolve);
            }
            function getTargetOfAliasDeclaration(node, dontRecursivelyResolve) {
                if (dontRecursivelyResolve === void 0) {
                    dontRecursivelyResolve = false;
                }
                switch (node.kind) {
                    case 253 /* ImportEqualsDeclaration */:
                        return getTargetOfImportEqualsDeclaration(node, dontRecursivelyResolve);
                    case 255 /* ImportClause */:
                        return getTargetOfImportClause(node, dontRecursivelyResolve);
                    case 256 /* NamespaceImport */:
                        return getTargetOfNamespaceImport(node, dontRecursivelyResolve);
                    case 262 /* NamespaceExport */:
                        return getTargetOfNamespaceExport(node, dontRecursivelyResolve);
                    case 258 /* ImportSpecifier */:
                        return getTargetOfImportSpecifier(node, dontRecursivelyResolve);
                    case 263 /* ExportSpecifier */:
                        return getTargetOfExportSpecifier(node, 111551 /* Value */ | 788968 /* Type */ | 1920 /* Namespace */, dontRecursivelyResolve);
                    case 259 /* ExportAssignment */:
                    case 209 /* BinaryExpression */:
                        return getTargetOfExportAssignment(node, dontRecursivelyResolve);
                    case 252 /* NamespaceExportDeclaration */:
                        return getTargetOfNamespaceExportDeclaration(node, dontRecursivelyResolve);
                    case 282 /* ShorthandPropertyAssignment */:
                        return resolveEntityName(node.name, 111551 /* Value */ | 788968 /* Type */ | 1920 /* Namespace */, /*ignoreErrors*/ true, dontRecursivelyResolve);
                    case 281 /* PropertyAssignment */:
                        return getTargetOfPropertyAssignment(node, dontRecursivelyResolve);
                    case 194 /* PropertyAccessExpression */:
                        return getTargetOfPropertyAccessExpression(node, dontRecursivelyResolve);
                    default:
                        return ts.Debug.fail();
                }
            }
            /**
             * Indicates that a symbol is an alias that does not merge with a local declaration.
             * OR Is a JSContainer which may merge an alias with a local declaration
             */
            function isNonLocalAlias(symbol, excludes) {
                if (excludes === void 0) {
                    excludes = 111551 /* Value */ | 788968 /* Type */ | 1920 /* Namespace */;
                }
                if (!symbol)
                    return false;
                return (symbol.flags & (2097152 /* Alias */ | excludes)) === 2097152 /* Alias */ || !!(symbol.flags & 2097152 /* Alias */ && symbol.flags & 67108864 /* Assignment */);
            }
            function resolveSymbol(symbol, dontResolveAlias) {
                return !dontResolveAlias && isNonLocalAlias(symbol) ? resolveAlias(symbol) : symbol;
            }
            function resolveAlias(symbol) {
                ts.Debug.assert((symbol.flags & 2097152 /* Alias */) !== 0, "Should only get Alias here.");
                var links = getSymbolLinks(symbol);
                if (!links.target) {
                    links.target = resolvingSymbol;
                    var node = getDeclarationOfAliasSymbol(symbol);
                    if (!node)
                        return ts.Debug.fail();
                    var target = getTargetOfAliasDeclaration(node);
                    if (links.target === resolvingSymbol) {
                        links.target = target || unknownSymbol;
                    }
                    else {
                        error(node, ts.Diagnostics.Circular_definition_of_import_alias_0, symbolToString(symbol));
                    }
                }
                else if (links.target === resolvingSymbol) {
                    links.target = unknownSymbol;
                }
                return links.target;
            }
            /**
             * Marks a symbol as type-only if its declaration is syntactically type-only.
             * If it is not itself marked type-only, but resolves to a type-only alias
             * somewhere in its resolution chain, save a reference to the type-only alias declaration
             * so the alias _not_ marked type-only can be identified as _transitively_ type-only.
             *
             * This function is called on each alias declaration that could be type-only or resolve to
             * another type-only alias during `resolveAlias`, so that later, when an alias is used in a
             * JS-emitting expression, we can quickly determine if that symbol is effectively type-only
             * and issue an error if so.
             *
             * @param aliasDeclaration The alias declaration not marked as type-only
             * has already been marked as not resolving to a type-only alias. Used when recursively resolving qualified
             * names of import aliases, e.g. `import C = a.b.C`. If namespace `a` is not found to be type-only, the
             * import declaration will initially be marked as not resolving to a type-only symbol. But, namespace `b`
             * must still be checked for a type-only marker, overwriting the previous negative result if found.
             * @param immediateTarget The symbol to which the alias declaration immediately resolves
             * @param finalTarget The symbol to which the alias declaration ultimately resolves
             * @param overwriteEmpty Checks `resolvesToSymbol` for type-only declarations even if `aliasDeclaration`
             */
            function markSymbolOfAliasDeclarationIfTypeOnly(aliasDeclaration, immediateTarget, finalTarget, overwriteEmpty) {
                if (!aliasDeclaration)
                    return false;
                // If the declaration itself is type-only, mark it and return.
                // No need to check what it resolves to.
                var sourceSymbol = getSymbolOfNode(aliasDeclaration);
                if (ts.isTypeOnlyImportOrExportDeclaration(aliasDeclaration)) {
                    var links_1 = getSymbolLinks(sourceSymbol);
                    links_1.typeOnlyDeclaration = aliasDeclaration;
                    return true;
                }
                var links = getSymbolLinks(sourceSymbol);
                return markSymbolOfAliasDeclarationIfTypeOnlyWorker(links, immediateTarget, overwriteEmpty)
                    || markSymbolOfAliasDeclarationIfTypeOnlyWorker(links, finalTarget, overwriteEmpty);
            }
            function markSymbolOfAliasDeclarationIfTypeOnlyWorker(aliasDeclarationLinks, target, overwriteEmpty) {
                var _a, _b, _c;
                if (target && (aliasDeclarationLinks.typeOnlyDeclaration === undefined || overwriteEmpty && aliasDeclarationLinks.typeOnlyDeclaration === false)) {
                    var exportSymbol = (_b = (_a = target.exports) === null || _a === void 0 ? void 0 : _a.get("export=" /* ExportEquals */)) !== null && _b !== void 0 ? _b : target;
                    var typeOnly = exportSymbol.declarations && ts.find(exportSymbol.declarations, ts.isTypeOnlyImportOrExportDeclaration);
                    aliasDeclarationLinks.typeOnlyDeclaration = (_c = typeOnly !== null && typeOnly !== void 0 ? typeOnly : getSymbolLinks(exportSymbol).typeOnlyDeclaration) !== null && _c !== void 0 ? _c : false;
                }
                return !!aliasDeclarationLinks.typeOnlyDeclaration;
            }
            /** Indicates that a symbol directly or indirectly resolves to a type-only import or export. */
            function getTypeOnlyAliasDeclaration(symbol) {
                if (!(symbol.flags & 2097152 /* Alias */)) {
                    return undefined;
                }
                var links = getSymbolLinks(symbol);
                return links.typeOnlyDeclaration || undefined;
            }
            function markExportAsReferenced(node) {
                var symbol = getSymbolOfNode(node);
                var target = resolveAlias(symbol);
                if (target) {
                    var markAlias = target === unknownSymbol ||
                        ((target.flags & 111551 /* Value */) && !isConstEnumOrConstEnumOnlyModule(target) && !getTypeOnlyAliasDeclaration(symbol));
                    if (markAlias) {
                        markAliasSymbolAsReferenced(symbol);
                    }
                }
            }
            // When an alias symbol is referenced, we need to mark the entity it references as referenced and in turn repeat that until
            // we reach a non-alias or an exported entity (which is always considered referenced). We do this by checking the target of
            // the alias as an expression (which recursively takes us back here if the target references another alias).
            function markAliasSymbolAsReferenced(symbol) {
                var links = getSymbolLinks(symbol);
                if (!links.referenced) {
                    links.referenced = true;
                    var node = getDeclarationOfAliasSymbol(symbol);
                    if (!node)
                        return ts.Debug.fail();
                    // We defer checking of the reference of an `import =` until the import itself is referenced,
                    // This way a chain of imports can be elided if ultimately the final input is only used in a type
                    // position.
                    if (ts.isInternalModuleImportEqualsDeclaration(node)) {
                        var target = resolveSymbol(symbol);
                        if (target === unknownSymbol || target.flags & 111551 /* Value */) {
                            // import foo = <symbol>
                            checkExpressionCached(node.moduleReference);
                        }
                    }
                }
            }
            // Aliases that resolve to const enums are not marked as referenced because they are not emitted,
            // but their usage in value positions must be tracked to determine if the import can be type-only.
            function markConstEnumAliasAsReferenced(symbol) {
                var links = getSymbolLinks(symbol);
                if (!links.constEnumReferenced) {
                    links.constEnumReferenced = true;
                }
            }
            // This function is only for imports with entity names
            function getSymbolOfPartOfRightHandSideOfImportEquals(entityName, dontResolveAlias) {
                // There are three things we might try to look for. In the following examples,
                // the search term is enclosed in |...|:
                //
                //     import a = |b|; // Namespace
                //     import a = |b.c|; // Value, type, namespace
                //     import a = |b.c|.d; // Namespace
                if (entityName.kind === 75 /* Identifier */ && ts.isRightSideOfQualifiedNameOrPropertyAccess(entityName)) {
                    entityName = entityName.parent;
                }
                // Check for case 1 and 3 in the above example
                if (entityName.kind === 75 /* Identifier */ || entityName.parent.kind === 153 /* QualifiedName */) {
                    return resolveEntityName(entityName, 1920 /* Namespace */, /*ignoreErrors*/ false, dontResolveAlias);
                }
                else {
                    // Case 2 in above example
                    // entityName.kind could be a QualifiedName or a Missing identifier
                    ts.Debug.assert(entityName.parent.kind === 253 /* ImportEqualsDeclaration */);
                    return resolveEntityName(entityName, 111551 /* Value */ | 788968 /* Type */ | 1920 /* Namespace */, /*ignoreErrors*/ false, dontResolveAlias);
                }
            }
            function getFullyQualifiedName(symbol, containingLocation) {
                return symbol.parent ? getFullyQualifiedName(symbol.parent, containingLocation) + "." + symbolToString(symbol) : symbolToString(symbol, containingLocation, /*meaning*/ undefined, 16 /* DoNotIncludeSymbolChain */ | 4 /* AllowAnyNodeKind */);
            }
            /**
             * Resolves a qualified name and any involved aliases.
             */
            function resolveEntityName(name, meaning, ignoreErrors, dontResolveAlias, location) {
                if (ts.nodeIsMissing(name)) {
                    return undefined;
                }
                var namespaceMeaning = 1920 /* Namespace */ | (ts.isInJSFile(name) ? meaning & 111551 /* Value */ : 0);
                var symbol;
                if (name.kind === 75 /* Identifier */) {
                    var message = meaning === namespaceMeaning ? ts.Diagnostics.Cannot_find_namespace_0 : getCannotFindNameDiagnosticForName(ts.getFirstIdentifier(name));
                    var symbolFromJSPrototype = ts.isInJSFile(name) ? resolveEntityNameFromAssignmentDeclaration(name, meaning) : undefined;
                    symbol = resolveName(location || name, name.escapedText, meaning, ignoreErrors || symbolFromJSPrototype ? undefined : message, name, /*isUse*/ true);
                    if (!symbol) {
                        return symbolFromJSPrototype;
                    }
                }
                else if (name.kind === 153 /* QualifiedName */ || name.kind === 194 /* PropertyAccessExpression */) {
                    var left = name.kind === 153 /* QualifiedName */ ? name.left : name.expression;
                    var right = name.kind === 153 /* QualifiedName */ ? name.right : name.name;
                    var namespace = resolveEntityName(left, namespaceMeaning, ignoreErrors, /*dontResolveAlias*/ false, location);
                    if (!namespace || ts.nodeIsMissing(right)) {
                        return undefined;
                    }
                    else if (namespace === unknownSymbol) {
                        return namespace;
                    }
                    if (ts.isInJSFile(name)) {
                        if (namespace.valueDeclaration &&
                            ts.isVariableDeclaration(namespace.valueDeclaration) &&
                            namespace.valueDeclaration.initializer &&
                            isCommonJsRequire(namespace.valueDeclaration.initializer)) {
                            var moduleName = namespace.valueDeclaration.initializer.arguments[0];
                            var moduleSym = resolveExternalModuleName(moduleName, moduleName);
                            if (moduleSym) {
                                var resolvedModuleSymbol = resolveExternalModuleSymbol(moduleSym);
                                if (resolvedModuleSymbol) {
                                    namespace = resolvedModuleSymbol;
                                }
                            }
                        }
                    }
                    symbol = getSymbol(getExportsOfSymbol(namespace), right.escapedText, meaning);
                    if (!symbol) {
                        if (!ignoreErrors) {
                            error(right, ts.Diagnostics.Namespace_0_has_no_exported_member_1, getFullyQualifiedName(namespace), ts.declarationNameToString(right));
                        }
                        return undefined;
                    }
                }
                else {
                    throw ts.Debug.assertNever(name, "Unknown entity name kind.");
                }
                ts.Debug.assert((ts.getCheckFlags(symbol) & 1 /* Instantiated */) === 0, "Should never get an instantiated symbol here.");
                if (ts.isEntityName(name) && (symbol.flags & 2097152 /* Alias */ || name.parent.kind === 259 /* ExportAssignment */)) {
                    markSymbolOfAliasDeclarationIfTypeOnly(ts.getAliasDeclarationFromName(name), symbol, /*finalTarget*/ undefined, /*overwriteEmpty*/ true);
                }
                return (symbol.flags & meaning) || dontResolveAlias ? symbol : resolveAlias(symbol);
            }
            /**
             * 1. For prototype-property methods like `A.prototype.m = function () ...`, try to resolve names in the scope of `A` too.
             * Note that prototype-property assignment to locations outside the current file (eg globals) doesn't work, so
             * name resolution won't work either.
             * 2. For property assignments like `{ x: function f () { } }`, try to resolve names in the scope of `f` too.
             */
            function resolveEntityNameFromAssignmentDeclaration(name, meaning) {
                if (isJSDocTypeReference(name.parent)) {
                    var secondaryLocation = getAssignmentDeclarationLocation(name.parent);
                    if (secondaryLocation) {
                        return resolveName(secondaryLocation, name.escapedText, meaning, /*nameNotFoundMessage*/ undefined, name, /*isUse*/ true);
                    }
                }
            }
            function getAssignmentDeclarationLocation(node) {
                var typeAlias = ts.findAncestor(node, function (node) { return !(ts.isJSDocNode(node) || node.flags & 4194304 /* JSDoc */) ? "quit" : ts.isJSDocTypeAlias(node); });
                if (typeAlias) {
                    return;
                }
                var host = ts.getJSDocHost(node);
                if (ts.isExpressionStatement(host) &&
                    ts.isBinaryExpression(host.expression) &&
                    ts.getAssignmentDeclarationKind(host.expression) === 3 /* PrototypeProperty */) {
                    // X.prototype.m = /** @param {K} p */ function () { } <-- look for K on X's declaration
                    var symbol = getSymbolOfNode(host.expression.left);
                    if (symbol) {
                        return getDeclarationOfJSPrototypeContainer(symbol);
                    }
                }
                if ((ts.isObjectLiteralMethod(host) || ts.isPropertyAssignment(host)) &&
                    ts.isBinaryExpression(host.parent.parent) &&
                    ts.getAssignmentDeclarationKind(host.parent.parent) === 6 /* Prototype */) {
                    // X.prototype = { /** @param {K} p */m() { } } <-- look for K on X's declaration
                    var symbol = getSymbolOfNode(host.parent.parent.left);
                    if (symbol) {
                        return getDeclarationOfJSPrototypeContainer(symbol);
                    }
                }
                var sig = ts.getHostSignatureFromJSDocHost(host);
                if (sig) {
                    var symbol = getSymbolOfNode(sig);
                    return symbol && symbol.valueDeclaration;
                }
            }
            function getDeclarationOfJSPrototypeContainer(symbol) {
                var decl = symbol.parent.valueDeclaration;
                if (!decl) {
                    return undefined;
                }
                var initializer = ts.isAssignmentDeclaration(decl) ? ts.getAssignedExpandoInitializer(decl) :
                    ts.hasOnlyExpressionInitializer(decl) ? ts.getDeclaredExpandoInitializer(decl) :
                        undefined;
                return initializer || decl;
            }
            /**
             * Get the real symbol of a declaration with an expando initializer.
             *
             * Normally, declarations have an associated symbol, but when a declaration has an expando
             * initializer, the expando's symbol is the one that has all the members merged into it.
             */
            function getExpandoSymbol(symbol) {
                var decl = symbol.valueDeclaration;
                if (!decl || !ts.isInJSFile(decl) || symbol.flags & 524288 /* TypeAlias */ || ts.getExpandoInitializer(decl, /*isPrototypeAssignment*/ false)) {
                    return undefined;
                }
                var init = ts.isVariableDeclaration(decl) ? ts.getDeclaredExpandoInitializer(decl) : ts.getAssignedExpandoInitializer(decl);
                if (init) {
                    var initSymbol = getSymbolOfNode(init);
                    if (initSymbol) {
                        return mergeJSSymbols(initSymbol, symbol);
                    }
                }
            }
            function resolveExternalModuleName(location, moduleReferenceExpression, ignoreErrors) {
                return resolveExternalModuleNameWorker(location, moduleReferenceExpression, ignoreErrors ? undefined : ts.Diagnostics.Cannot_find_module_0);
            }
            function resolveExternalModuleNameWorker(location, moduleReferenceExpression, moduleNotFoundError, isForAugmentation) {
                if (isForAugmentation === void 0) {
                    isForAugmentation = false;
                }
                return ts.isStringLiteralLike(moduleReferenceExpression)
                    ? resolveExternalModule(location, moduleReferenceExpression.text, moduleNotFoundError, moduleReferenceExpression, isForAugmentation)
                    : undefined;
            }
            function resolveExternalModule(location, moduleReference, moduleNotFoundError, errorNode, isForAugmentation) {
                if (isForAugmentation === void 0) {
                    isForAugmentation = false;
                }
                if (ts.startsWith(moduleReference, "@types/")) {
                    var diag = ts.Diagnostics.Cannot_import_type_declaration_files_Consider_importing_0_instead_of_1;
                    var withoutAtTypePrefix = ts.removePrefix(moduleReference, "@types/");
                    error(errorNode, diag, withoutAtTypePrefix, moduleReference);
                }
                var ambientModule = tryFindAmbientModule(moduleReference, /*withAugmentations*/ true);
                if (ambientModule) {
                    return ambientModule;
                }
                var currentSourceFile = ts.getSourceFileOfNode(location);
                var resolvedModule = ts.getResolvedModule(currentSourceFile, moduleReference); // TODO: GH#18217
                var resolutionDiagnostic = resolvedModule && ts.getResolutionDiagnostic(compilerOptions, resolvedModule);
                var sourceFile = resolvedModule && !resolutionDiagnostic && host.getSourceFile(resolvedModule.resolvedFileName);
                if (sourceFile) {
                    if (sourceFile.symbol) {
                        if (resolvedModule.isExternalLibraryImport && !ts.resolutionExtensionIsTSOrJson(resolvedModule.extension)) {
                            errorOnImplicitAnyModule(/*isError*/ false, errorNode, resolvedModule, moduleReference);
                        }
                        // merged symbol is module declaration symbol combined with all augmentations
                        return getMergedSymbol(sourceFile.symbol);
                    }
                    if (moduleNotFoundError) {
                        // report errors only if it was requested
                        error(errorNode, ts.Diagnostics.File_0_is_not_a_module, sourceFile.fileName);
                    }
                    return undefined;
                }
                if (patternAmbientModules) {
                    var pattern = ts.findBestPatternMatch(patternAmbientModules, function (_) { return _.pattern; }, moduleReference);
                    if (pattern) {
                        // If the module reference matched a pattern ambient module ('*.foo') but there's also a
                        // module augmentation by the specific name requested ('a.foo'), we store the merged symbol
                        // by the augmentation name ('a.foo'), because asking for *.foo should not give you exports
                        // from a.foo.
                        var augmentation = patternAmbientModuleAugmentations && patternAmbientModuleAugmentations.get(moduleReference);
                        if (augmentation) {
                            return getMergedSymbol(augmentation);
                        }
                        return getMergedSymbol(pattern.symbol);
                    }
                }
                // May be an untyped module. If so, ignore resolutionDiagnostic.
                if (resolvedModule && !ts.resolutionExtensionIsTSOrJson(resolvedModule.extension) && resolutionDiagnostic === undefined || resolutionDiagnostic === ts.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type) {
                    if (isForAugmentation) {
                        var diag = ts.Diagnostics.Invalid_module_name_in_augmentation_Module_0_resolves_to_an_untyped_module_at_1_which_cannot_be_augmented;
                        error(errorNode, diag, moduleReference, resolvedModule.resolvedFileName);
                    }
                    else {
                        errorOnImplicitAnyModule(/*isError*/ noImplicitAny && !!moduleNotFoundError, errorNode, resolvedModule, moduleReference);
                    }
                    // Failed imports and untyped modules are both treated in an untyped manner; only difference is whether we give a diagnostic first.
                    return undefined;
                }
                if (moduleNotFoundError) {
                    // See if this was possibly a projectReference redirect
                    if (resolvedModule) {
                        var redirect = host.getProjectReferenceRedirect(resolvedModule.resolvedFileName);
                        if (redirect) {
                            error(errorNode, ts.Diagnostics.Output_file_0_has_not_been_built_from_source_file_1, redirect, resolvedModule.resolvedFileName);
                            return undefined;
                        }
                    }
                    if (resolutionDiagnostic) {
                        error(errorNode, resolutionDiagnostic, moduleReference, resolvedModule.resolvedFileName);
                    }
                    else {
                        var tsExtension = ts.tryExtractTSExtension(moduleReference);
                        if (tsExtension) {
                            var diag = ts.Diagnostics.An_import_path_cannot_end_with_a_0_extension_Consider_importing_1_instead;
                            error(errorNode, diag, tsExtension, ts.removeExtension(moduleReference, tsExtension));
                        }
                        else if (!compilerOptions.resolveJsonModule &&
                            ts.fileExtensionIs(moduleReference, ".json" /* Json */) &&
                            ts.getEmitModuleResolutionKind(compilerOptions) === ts.ModuleResolutionKind.NodeJs &&
                            ts.hasJsonModuleEmitEnabled(compilerOptions)) {
                            error(errorNode, ts.Diagnostics.Cannot_find_module_0_Consider_using_resolveJsonModule_to_import_module_with_json_extension, moduleReference);
                        }
                        else {
                            error(errorNode, moduleNotFoundError, moduleReference);
                        }
                    }
                }
                return undefined;
            }
            function errorOnImplicitAnyModule(isError, errorNode, _a, moduleReference) {
                var packageId = _a.packageId, resolvedFileName = _a.resolvedFileName;
                var errorInfo = !ts.isExternalModuleNameRelative(moduleReference) && packageId
                    ? typesPackageExists(packageId.name)
                        ? ts.chainDiagnosticMessages(
                        /*details*/ undefined, ts.Diagnostics.If_the_0_package_actually_exposes_this_module_consider_sending_a_pull_request_to_amend_https_Colon_Slash_Slashgithub_com_SlashDefinitelyTyped_SlashDefinitelyTyped_Slashtree_Slashmaster_Slashtypes_Slash_1, packageId.name, ts.mangleScopedPackageName(packageId.name))
                        : ts.chainDiagnosticMessages(
                        /*details*/ undefined, ts.Diagnostics.Try_npm_install_types_Slash_1_if_it_exists_or_add_a_new_declaration_d_ts_file_containing_declare_module_0, moduleReference, ts.mangleScopedPackageName(packageId.name))
                    : undefined;
                errorOrSuggestion(isError, errorNode, ts.chainDiagnosticMessages(errorInfo, ts.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type, moduleReference, resolvedFileName));
            }
            function typesPackageExists(packageName) {
                return getPackagesSet().has(ts.getTypesPackageName(packageName));
            }
            function resolveExternalModuleSymbol(moduleSymbol, dontResolveAlias) {
                if (moduleSymbol) {
                    var exportEquals = resolveSymbol(moduleSymbol.exports.get("export=" /* ExportEquals */), dontResolveAlias);
                    var exported = getCommonJsExportEquals(getMergedSymbol(exportEquals), getMergedSymbol(moduleSymbol));
                    return getMergedSymbol(exported) || moduleSymbol;
                }
                return undefined;
            }
            function getCommonJsExportEquals(exported, moduleSymbol) {
                if (!exported || exported === unknownSymbol || exported === moduleSymbol || moduleSymbol.exports.size === 1 || exported.flags & 2097152 /* Alias */) {
                    return exported;
                }
                var links = getSymbolLinks(exported);
                if (links.cjsExportMerged) {
                    return links.cjsExportMerged;
                }
                var merged = exported.flags & 33554432 /* Transient */ ? exported : cloneSymbol(exported);
                merged.flags = merged.flags | 512 /* ValueModule */;
                if (merged.exports === undefined) {
                    merged.exports = ts.createSymbolTable();
                }
                moduleSymbol.exports.forEach(function (s, name) {
                    if (name === "export=" /* ExportEquals */)
                        return;
                    merged.exports.set(name, merged.exports.has(name) ? mergeSymbol(merged.exports.get(name), s) : s);
                });
                getSymbolLinks(merged).cjsExportMerged = merged;
                return links.cjsExportMerged = merged;
            }
            // An external module with an 'export =' declaration may be referenced as an ES6 module provided the 'export ='
            // references a symbol that is at least declared as a module or a variable. The target of the 'export =' may
            // combine other declarations with the module or variable (e.g. a class/module, function/module, interface/variable).
            function resolveESModuleSymbol(moduleSymbol, referencingLocation, dontResolveAlias, suppressInteropError) {
                var symbol = resolveExternalModuleSymbol(moduleSymbol, dontResolveAlias);
                if (!dontResolveAlias && symbol) {
                    if (!suppressInteropError && !(symbol.flags & (1536 /* Module */ | 3 /* Variable */)) && !ts.getDeclarationOfKind(symbol, 290 /* SourceFile */)) {
                        var compilerOptionName = moduleKind >= ts.ModuleKind.ES2015
                            ? "allowSyntheticDefaultImports"
                            : "esModuleInterop";
                        error(referencingLocation, ts.Diagnostics.This_module_can_only_be_referenced_with_ECMAScript_imports_Slashexports_by_turning_on_the_0_flag_and_referencing_its_default_export, compilerOptionName);
                        return symbol;
                    }
                    if (compilerOptions.esModuleInterop) {
                        var referenceParent = referencingLocation.parent;
                        if ((ts.isImportDeclaration(referenceParent) && ts.getNamespaceDeclarationNode(referenceParent)) ||
                            ts.isImportCall(referenceParent)) {
                            var type = getTypeOfSymbol(symbol);
                            var sigs = getSignaturesOfStructuredType(type, 0 /* Call */);
                            if (!sigs || !sigs.length) {
                                sigs = getSignaturesOfStructuredType(type, 1 /* Construct */);
                            }
                            if (sigs && sigs.length) {
                                var moduleType = getTypeWithSyntheticDefaultImportType(type, symbol, moduleSymbol);
                                // Create a new symbol which has the module's type less the call and construct signatures
                                var result = createSymbol(symbol.flags, symbol.escapedName);
                                result.declarations = symbol.declarations ? symbol.declarations.slice() : [];
                                result.parent = symbol.parent;
                                result.target = symbol;
                                result.originatingImport = referenceParent;
                                if (symbol.valueDeclaration)
                                    result.valueDeclaration = symbol.valueDeclaration;
                                if (symbol.constEnumOnlyModule)
                                    result.constEnumOnlyModule = true;
                                if (symbol.members)
                                    result.members = ts.cloneMap(symbol.members);
                                if (symbol.exports)
                                    result.exports = ts.cloneMap(symbol.exports);
                                var resolvedModuleType = resolveStructuredTypeMembers(moduleType); // Should already be resolved from the signature checks above
                                result.type = createAnonymousType(result, resolvedModuleType.members, ts.emptyArray, ts.emptyArray, resolvedModuleType.stringIndexInfo, resolvedModuleType.numberIndexInfo);
                                return result;
                            }
                        }
                    }
                }
                return symbol;
            }
            function hasExportAssignmentSymbol(moduleSymbol) {
                return moduleSymbol.exports.get("export=" /* ExportEquals */) !== undefined;
            }
            function getExportsOfModuleAsArray(moduleSymbol) {
                return symbolsToArray(getExportsOfModule(moduleSymbol));
            }
            function getExportsAndPropertiesOfModule(moduleSymbol) {
                var exports = getExportsOfModuleAsArray(moduleSymbol);
                var exportEquals = resolveExternalModuleSymbol(moduleSymbol);
                if (exportEquals !== moduleSymbol) {
                    ts.addRange(exports, getPropertiesOfType(getTypeOfSymbol(exportEquals)));
                }
                return exports;
            }
            function tryGetMemberInModuleExports(memberName, moduleSymbol) {
                var symbolTable = getExportsOfModule(moduleSymbol);
                if (symbolTable) {
                    return symbolTable.get(memberName);
                }
            }
            function tryGetMemberInModuleExportsAndProperties(memberName, moduleSymbol) {
                var symbol = tryGetMemberInModuleExports(memberName, moduleSymbol);
                if (symbol) {
                    return symbol;
                }
                var exportEquals = resolveExternalModuleSymbol(moduleSymbol);
                if (exportEquals === moduleSymbol) {
                    return undefined;
                }
                var type = getTypeOfSymbol(exportEquals);
                return type.flags & 131068 /* Primitive */ ||
                    ts.getObjectFlags(type) & 1 /* Class */ ||
                    isArrayOrTupleLikeType(type)
                    ? undefined
                    : getPropertyOfType(type, memberName);
            }
            function getExportsOfSymbol(symbol) {
                return symbol.flags & 6256 /* LateBindingContainer */ ? getResolvedMembersOrExportsOfSymbol(symbol, "resolvedExports" /* resolvedExports */) :
                    symbol.flags & 1536 /* Module */ ? getExportsOfModule(symbol) :
                        symbol.exports || emptySymbols;
            }
            function getExportsOfModule(moduleSymbol) {
                var links = getSymbolLinks(moduleSymbol);
                return links.resolvedExports || (links.resolvedExports = getExportsOfModuleWorker(moduleSymbol));
            }
            /**
             * Extends one symbol table with another while collecting information on name collisions for error message generation into the `lookupTable` argument
             * Not passing `lookupTable` and `exportNode` disables this collection, and just extends the tables
             */
            function extendExportSymbols(target, source, lookupTable, exportNode) {
                if (!source)
                    return;
                source.forEach(function (sourceSymbol, id) {
                    if (id === "default" /* Default */)
                        return;
                    var targetSymbol = target.get(id);
                    if (!targetSymbol) {
                        target.set(id, sourceSymbol);
                        if (lookupTable && exportNode) {
                            lookupTable.set(id, {
                                specifierText: ts.getTextOfNode(exportNode.moduleSpecifier)
                            });
                        }
                    }
                    else if (lookupTable && exportNode && targetSymbol && resolveSymbol(targetSymbol) !== resolveSymbol(sourceSymbol)) {
                        var collisionTracker = lookupTable.get(id);
                        if (!collisionTracker.exportsWithDuplicate) {
                            collisionTracker.exportsWithDuplicate = [exportNode];
                        }
                        else {
                            collisionTracker.exportsWithDuplicate.push(exportNode);
                        }
                    }
                });
            }
            function getExportsOfModuleWorker(moduleSymbol) {
                var visitedSymbols = [];
                // A module defined by an 'export=' consists of one export that needs to be resolved
                moduleSymbol = resolveExternalModuleSymbol(moduleSymbol);
                return visit(moduleSymbol) || emptySymbols;
                // The ES6 spec permits export * declarations in a module to circularly reference the module itself. For example,
                // module 'a' can 'export * from "b"' and 'b' can 'export * from "a"' without error.
                function visit(symbol) {
                    if (!(symbol && symbol.exports && ts.pushIfUnique(visitedSymbols, symbol))) {
                        return;
                    }
                    var symbols = ts.cloneMap(symbol.exports);
                    // All export * declarations are collected in an __export symbol by the binder
                    var exportStars = symbol.exports.get("__export" /* ExportStar */);
                    if (exportStars) {
                        var nestedSymbols = ts.createSymbolTable();
                        var lookupTable_1 = ts.createMap();
                        for (var _i = 0, _a = exportStars.declarations; _i < _a.length; _i++) {
                            var node = _a[_i];
                            var resolvedModule = resolveExternalModuleName(node, node.moduleSpecifier);
                            var exportedSymbols = visit(resolvedModule);
                            extendExportSymbols(nestedSymbols, exportedSymbols, lookupTable_1, node);
                        }
                        lookupTable_1.forEach(function (_a, id) {
                            var exportsWithDuplicate = _a.exportsWithDuplicate;
                            // It's not an error if the file with multiple `export *`s with duplicate names exports a member with that name itself
                            if (id === "export=" || !(exportsWithDuplicate && exportsWithDuplicate.length) || symbols.has(id)) {
                                return;
                            }
                            for (var _i = 0, exportsWithDuplicate_1 = exportsWithDuplicate; _i < exportsWithDuplicate_1.length; _i++) {
                                var node = exportsWithDuplicate_1[_i];
                                diagnostics.add(ts.createDiagnosticForNode(node, ts.Diagnostics.Module_0_has_already_exported_a_member_named_1_Consider_explicitly_re_exporting_to_resolve_the_ambiguity, lookupTable_1.get(id).specifierText, ts.unescapeLeadingUnderscores(id)));
                            }
                        });
                        extendExportSymbols(symbols, nestedSymbols);
                    }
                    return symbols;
                }
            }
            function getMergedSymbol(symbol) {
                var merged;
                return symbol && symbol.mergeId && (merged = mergedSymbols[symbol.mergeId]) ? merged : symbol;
            }
            function getSymbolOfNode(node) {
                return getMergedSymbol(node.symbol && getLateBoundSymbol(node.symbol));
            }
            function getParentOfSymbol(symbol) {
                return getMergedSymbol(symbol.parent && getLateBoundSymbol(symbol.parent));
            }
            function getAlternativeContainingModules(symbol, enclosingDeclaration) {
                var containingFile = ts.getSourceFileOfNode(enclosingDeclaration);
                var id = "" + getNodeId(containingFile);
                var links = getSymbolLinks(symbol);
                var results;
                if (links.extendedContainersByFile && (results = links.extendedContainersByFile.get(id))) {
                    return results;
                }
                if (containingFile && containingFile.imports) {
                    // Try to make an import using an import already in the enclosing file, if possible
                    for (var _i = 0, _a = containingFile.imports; _i < _a.length; _i++) {
                        var importRef = _a[_i];
                        if (ts.nodeIsSynthesized(importRef))
                            continue; // Synthetic names can't be resolved by `resolveExternalModuleName` - they'll cause a debug assert if they error
                        var resolvedModule = resolveExternalModuleName(enclosingDeclaration, importRef, /*ignoreErrors*/ true);
                        if (!resolvedModule)
                            continue;
                        var ref = getAliasForSymbolInContainer(resolvedModule, symbol);
                        if (!ref)
                            continue;
                        results = ts.append(results, resolvedModule);
                    }
                    if (ts.length(results)) {
                        (links.extendedContainersByFile || (links.extendedContainersByFile = ts.createMap())).set(id, results);
                        return results;
                    }
                }
                if (links.extendedContainers) {
                    return links.extendedContainers;
                }
                // No results from files already being imported by this file - expand search (expensive, but not location-specific, so cached)
                var otherFiles = host.getSourceFiles();
                for (var _b = 0, otherFiles_1 = otherFiles; _b < otherFiles_1.length; _b++) {
                    var file = otherFiles_1[_b];
                    if (!ts.isExternalModule(file))
                        continue;
                    var sym = getSymbolOfNode(file);
                    var ref = getAliasForSymbolInContainer(sym, symbol);
                    if (!ref)
                        continue;
                    results = ts.append(results, sym);
                }
                return links.extendedContainers = results || ts.emptyArray;
            }
            /**
             * Attempts to find the symbol corresponding to the container a symbol is in - usually this
             * is just its' `.parent`, but for locals, this value is `undefined`
             */
            function getContainersOfSymbol(symbol, enclosingDeclaration) {
                var container = getParentOfSymbol(symbol);
                // Type parameters end up in the `members` lists but are not externally visible
                if (container && !(symbol.flags & 262144 /* TypeParameter */)) {
                    var additionalContainers = ts.mapDefined(container.declarations, fileSymbolIfFileSymbolExportEqualsContainer);
                    var reexportContainers = enclosingDeclaration && getAlternativeContainingModules(symbol, enclosingDeclaration);
                    if (enclosingDeclaration && getAccessibleSymbolChain(container, enclosingDeclaration, 1920 /* Namespace */, /*externalOnly*/ false)) {
                        return ts.concatenate(ts.concatenate([container], additionalContainers), reexportContainers); // This order expresses a preference for the real container if it is in scope
                    }
                    var res = ts.append(additionalContainers, container);
                    return ts.concatenate(res, reexportContainers);
                }
                var candidates = ts.mapDefined(symbol.declarations, function (d) {
                    if (!ts.isAmbientModule(d) && d.parent && hasNonGlobalAugmentationExternalModuleSymbol(d.parent)) {
                        return getSymbolOfNode(d.parent);
                    }
                    if (ts.isClassExpression(d) && ts.isBinaryExpression(d.parent) && d.parent.operatorToken.kind === 62 /* EqualsToken */ && ts.isAccessExpression(d.parent.left) && ts.isEntityNameExpression(d.parent.left.expression)) {
                        if (ts.isModuleExportsAccessExpression(d.parent.left) || ts.isExportsIdentifier(d.parent.left.expression)) {
                            return getSymbolOfNode(ts.getSourceFileOfNode(d));
                        }
                        checkExpressionCached(d.parent.left.expression);
                        return getNodeLinks(d.parent.left.expression).resolvedSymbol;
                    }
                });
                if (!ts.length(candidates)) {
                    return undefined;
                }
                return ts.mapDefined(candidates, function (candidate) { return getAliasForSymbolInContainer(candidate, symbol) ? candidate : undefined; });
                function fileSymbolIfFileSymbolExportEqualsContainer(d) {
                    var fileSymbol = getExternalModuleContainer(d);
                    var exported = fileSymbol && fileSymbol.exports && fileSymbol.exports.get("export=" /* ExportEquals */);
                    return exported && container && getSymbolIfSameReference(exported, container) ? fileSymbol : undefined;
                }
            }
            function getAliasForSymbolInContainer(container, symbol) {
                if (container === getParentOfSymbol(symbol)) {
                    // fast path, `symbol` is either already the alias or isn't aliased
                    return symbol;
                }
                // Check if container is a thing with an `export=` which points directly at `symbol`, and if so, return
                // the container itself as the alias for the symbol
                var exportEquals = container.exports && container.exports.get("export=" /* ExportEquals */);
                if (exportEquals && getSymbolIfSameReference(exportEquals, symbol)) {
                    return container;
                }
                var exports = getExportsOfSymbol(container);
                var quick = exports.get(symbol.escapedName);
                if (quick && getSymbolIfSameReference(quick, symbol)) {
                    return quick;
                }
                return ts.forEachEntry(exports, function (exported) {
                    if (getSymbolIfSameReference(exported, symbol)) {
                        return exported;
                    }
                });
            }
            /**
             * Checks if two symbols, through aliasing and/or merging, refer to the same thing
             */
            function getSymbolIfSameReference(s1, s2) {
                if (getMergedSymbol(resolveSymbol(getMergedSymbol(s1))) === getMergedSymbol(resolveSymbol(getMergedSymbol(s2)))) {
                    return s1;
                }
            }
            function getExportSymbolOfValueSymbolIfExported(symbol) {
                return getMergedSymbol(symbol && (symbol.flags & 1048576 /* ExportValue */) !== 0 ? symbol.exportSymbol : symbol);
            }
            function symbolIsValue(symbol) {
                return !!(symbol.flags & 111551 /* Value */ || symbol.flags & 2097152 /* Alias */ && resolveAlias(symbol).flags & 111551 /* Value */ && !getTypeOnlyAliasDeclaration(symbol));
            }
            function findConstructorDeclaration(node) {
                var members = node.members;
                for (var _i = 0, members_3 = members; _i < members_3.length; _i++) {
                    var member = members_3[_i];
                    if (member.kind === 162 /* Constructor */ && ts.nodeIsPresent(member.body)) {
                        return member;
                    }
                }
            }
            function createType(flags) {
                var result = new Type(checker, flags);
                typeCount++;
                result.id = typeCount;
                return result;
            }
            function createIntrinsicType(kind, intrinsicName, objectFlags) {
                if (objectFlags === void 0) {
                    objectFlags = 0;
                }
                var type = createType(kind);
                type.intrinsicName = intrinsicName;
                type.objectFlags = objectFlags;
                return type;
            }
            function createBooleanType(trueFalseTypes) {
                var type = getUnionType(trueFalseTypes);
                type.flags |= 16 /* Boolean */;
                type.intrinsicName = "boolean";
                return type;
            }
            function createObjectType(objectFlags, symbol) {
                var type = createType(524288 /* Object */);
                type.objectFlags = objectFlags;
                type.symbol = symbol;
                type.members = undefined;
                type.properties = undefined;
                type.callSignatures = undefined;
                type.constructSignatures = undefined;
                type.stringIndexInfo = undefined;
                type.numberIndexInfo = undefined;
                return type;
            }
            function createTypeofType() {
                return getUnionType(ts.arrayFrom(typeofEQFacts.keys(), getLiteralType));
            }
            function createTypeParameter(symbol) {
                var type = createType(262144 /* TypeParameter */);
                if (symbol)
                    type.symbol = symbol;
                return type;
            }
            // A reserved member name starts with two underscores, but the third character cannot be an underscore,
            // @, or #. A third underscore indicates an escaped form of an identifier that started
            // with at least two underscores. The @ character indicates that the name is denoted by a well known ES
            // Symbol instance and the # character indicates that the name is a PrivateIdentifier.
            function isReservedMemberName(name) {
                return name.charCodeAt(0) === 95 /* _ */ &&
                    name.charCodeAt(1) === 95 /* _ */ &&
                    name.charCodeAt(2) !== 95 /* _ */ &&
                    name.charCodeAt(2) !== 64 /* at */ &&
                    name.charCodeAt(2) !== 35 /* hash */;
            }
            function getNamedMembers(members) {
                var result;
                members.forEach(function (symbol, id) {
                    if (!isReservedMemberName(id) && symbolIsValue(symbol)) {
                        (result || (result = [])).push(symbol);
                    }
                });
                return result || ts.emptyArray;
            }
            function setStructuredTypeMembers(type, members, callSignatures, constructSignatures, stringIndexInfo, numberIndexInfo) {
                type.members = members;
                type.properties = members === emptySymbols ? ts.emptyArray : getNamedMembers(members);
                type.callSignatures = callSignatures;
                type.constructSignatures = constructSignatures;
                type.stringIndexInfo = stringIndexInfo;
                type.numberIndexInfo = numberIndexInfo;
                return type;
            }
            function createAnonymousType(symbol, members, callSignatures, constructSignatures, stringIndexInfo, numberIndexInfo) {
                return setStructuredTypeMembers(createObjectType(16 /* Anonymous */, symbol), members, callSignatures, constructSignatures, stringIndexInfo, numberIndexInfo);
            }
            function forEachSymbolTableInScope(enclosingDeclaration, callback) {
                var result;
                var _loop_6 = function (location) {
                    // Locals of a source file are not in scope (because they get merged into the global symbol table)
                    if (location.locals && !isGlobalSourceFile(location)) {
                        if (result = callback(location.locals)) {
                            return { value: result };
                        }
                    }
                    switch (location.kind) {
                        case 290 /* SourceFile */:
                            if (!ts.isExternalOrCommonJsModule(location)) {
                                break;
                            }
                        // falls through
                        case 249 /* ModuleDeclaration */:
                            var sym = getSymbolOfNode(location);
                            // `sym` may not have exports if this module declaration is backed by the symbol for a `const` that's being rewritten
                            // into a namespace - in such cases, it's best to just let the namespace appear empty (the const members couldn't have referred
                            // to one another anyway)
                            if (result = callback((sym === null || sym === void 0 ? void 0 : sym.exports) || emptySymbols)) {
                                return { value: result };
                            }
                            break;
                        case 245 /* ClassDeclaration */:
                        case 214 /* ClassExpression */:
                        case 246 /* InterfaceDeclaration */:
                            // Type parameters are bound into `members` lists so they can merge across declarations
                            // This is troublesome, since in all other respects, they behave like locals :cries:
                            // TODO: the below is shared with similar code in `resolveName` - in fact, rephrasing all this symbol
                            // lookup logic in terms of `resolveName` would be nice
                            // The below is used to lookup type parameters within a class or interface, as they are added to the class/interface locals
                            // These can never be latebound, so the symbol's raw members are sufficient. `getMembersOfNode` cannot be used, as it would
                            // trigger resolving late-bound names, which we may already be in the process of doing while we're here!
                            var table_1;
                            // TODO: Should this filtered table be cached in some way?
                            (getSymbolOfNode(location).members || emptySymbols).forEach(function (memberSymbol, key) {
                                if (memberSymbol.flags & (788968 /* Type */ & ~67108864 /* Assignment */)) {
                                    (table_1 || (table_1 = ts.createSymbolTable())).set(key, memberSymbol);
                                }
                            });
                            if (table_1 && (result = callback(table_1))) {
                                return { value: result };
                            }
                            break;
                    }
                };
                for (var location = enclosingDeclaration; location; location = location.parent) {
                    var state_2 = _loop_6(location);
                    if (typeof state_2 === "object")
                        return state_2.value;
                }
                return callback(globals);
            }
            function getQualifiedLeftMeaning(rightMeaning) {
                // If we are looking in value space, the parent meaning is value, other wise it is namespace
                return rightMeaning === 111551 /* Value */ ? 111551 /* Value */ : 1920 /* Namespace */;
            }
            function getAccessibleSymbolChain(symbol, enclosingDeclaration, meaning, useOnlyExternalAliasing, visitedSymbolTablesMap) {
                if (visitedSymbolTablesMap === void 0) {
                    visitedSymbolTablesMap = ts.createMap();
                }
                if (!(symbol && !isPropertyOrMethodDeclarationSymbol(symbol))) {
                    return undefined;
                }
                var id = "" + getSymbolId(symbol);
                var visitedSymbolTables = visitedSymbolTablesMap.get(id);
                if (!visitedSymbolTables) {
                    visitedSymbolTablesMap.set(id, visitedSymbolTables = []);
                }
                return forEachSymbolTableInScope(enclosingDeclaration, getAccessibleSymbolChainFromSymbolTable);
                /**
                 * @param {ignoreQualification} boolean Set when a symbol is being looked for through the exports of another symbol (meaning we have a route to qualify it already)
                 */
                function getAccessibleSymbolChainFromSymbolTable(symbols, ignoreQualification) {
                    if (!ts.pushIfUnique(visitedSymbolTables, symbols)) {
                        return undefined;
                    }
                    var result = trySymbolTable(symbols, ignoreQualification);
                    visitedSymbolTables.pop();
                    return result;
                }
                function canQualifySymbol(symbolFromSymbolTable, meaning) {
                    // If the symbol is equivalent and doesn't need further qualification, this symbol is accessible
                    return !needsQualification(symbolFromSymbolTable, enclosingDeclaration, meaning) ||
                        // If symbol needs qualification, make sure that parent is accessible, if it is then this symbol is accessible too
                        !!getAccessibleSymbolChain(symbolFromSymbolTable.parent, enclosingDeclaration, getQualifiedLeftMeaning(meaning), useOnlyExternalAliasing, visitedSymbolTablesMap);
                }
                function isAccessible(symbolFromSymbolTable, resolvedAliasSymbol, ignoreQualification) {
                    return (symbol === (resolvedAliasSymbol || symbolFromSymbolTable) || getMergedSymbol(symbol) === getMergedSymbol(resolvedAliasSymbol || symbolFromSymbolTable)) &&
                        // if the symbolFromSymbolTable is not external module (it could be if it was determined as ambient external module and would be in globals table)
                        // and if symbolFromSymbolTable or alias resolution matches the symbol,
                        // check the symbol can be qualified, it is only then this symbol is accessible
                        !ts.some(symbolFromSymbolTable.declarations, hasNonGlobalAugmentationExternalModuleSymbol) &&
                        (ignoreQualification || canQualifySymbol(getMergedSymbol(symbolFromSymbolTable), meaning));
                }
                function trySymbolTable(symbols, ignoreQualification) {
                    // If symbol is directly available by its name in the symbol table
                    if (isAccessible(symbols.get(symbol.escapedName), /*resolvedAliasSymbol*/ undefined, ignoreQualification)) {
                        return [symbol];
                    }
                    // Check if symbol is any of the aliases in scope
                    var result = ts.forEachEntry(symbols, function (symbolFromSymbolTable) {
                        if (symbolFromSymbolTable.flags & 2097152 /* Alias */
                            && symbolFromSymbolTable.escapedName !== "export=" /* ExportEquals */
                            && symbolFromSymbolTable.escapedName !== "default" /* Default */
                            && !(ts.isUMDExportSymbol(symbolFromSymbolTable) && enclosingDeclaration && ts.isExternalModule(ts.getSourceFileOfNode(enclosingDeclaration)))
                            // If `!useOnlyExternalAliasing`, we can use any type of alias to get the name
                            && (!useOnlyExternalAliasing || ts.some(symbolFromSymbolTable.declarations, ts.isExternalModuleImportEqualsDeclaration))
                            // While exports are generally considered to be in scope, export-specifier declared symbols are _not_
                            // See similar comment in `resolveName` for details
                            && (ignoreQualification || !ts.getDeclarationOfKind(symbolFromSymbolTable, 263 /* ExportSpecifier */))) {
                            var resolvedImportedSymbol = resolveAlias(symbolFromSymbolTable);
                            var candidate = getCandidateListForSymbol(symbolFromSymbolTable, resolvedImportedSymbol, ignoreQualification);
                            if (candidate) {
                                return candidate;
                            }
                        }
                        if (symbolFromSymbolTable.escapedName === symbol.escapedName && symbolFromSymbolTable.exportSymbol) {
                            if (isAccessible(getMergedSymbol(symbolFromSymbolTable.exportSymbol), /*aliasSymbol*/ undefined, ignoreQualification)) {
                                return [symbol];
                            }
                        }
                    });
                    // If there's no result and we're looking at the global symbol table, treat `globalThis` like an alias and try to lookup thru that
                    return result || (symbols === globals ? getCandidateListForSymbol(globalThisSymbol, globalThisSymbol, ignoreQualification) : undefined);
                }
                function getCandidateListForSymbol(symbolFromSymbolTable, resolvedImportedSymbol, ignoreQualification) {
                    if (isAccessible(symbolFromSymbolTable, resolvedImportedSymbol, ignoreQualification)) {
                        return [symbolFromSymbolTable];
                    }
                    // Look in the exported members, if we can find accessibleSymbolChain, symbol is accessible using this chain
                    // but only if the symbolFromSymbolTable can be qualified
                    var candidateTable = getExportsOfSymbol(resolvedImportedSymbol);
                    var accessibleSymbolsFromExports = candidateTable && getAccessibleSymbolChainFromSymbolTable(candidateTable, /*ignoreQualification*/ true);
                    if (accessibleSymbolsFromExports && canQualifySymbol(symbolFromSymbolTable, getQualifiedLeftMeaning(meaning))) {
                        return [symbolFromSymbolTable].concat(accessibleSymbolsFromExports);
                    }
                }
            }
            function needsQualification(symbol, enclosingDeclaration, meaning) {
                var qualify = false;
                forEachSymbolTableInScope(enclosingDeclaration, function (symbolTable) {
                    // If symbol of this name is not available in the symbol table we are ok
                    var symbolFromSymbolTable = getMergedSymbol(symbolTable.get(symbol.escapedName));
                    if (!symbolFromSymbolTable) {
                        // Continue to the next symbol table
                        return false;
                    }
                    // If the symbol with this name is present it should refer to the symbol
                    if (symbolFromSymbolTable === symbol) {
                        // No need to qualify
                        return true;
                    }
                    // Qualify if the symbol from symbol table has same meaning as expected
                    symbolFromSymbolTable = (symbolFromSymbolTable.flags & 2097152 /* Alias */ && !ts.getDeclarationOfKind(symbolFromSymbolTable, 263 /* ExportSpecifier */)) ? resolveAlias(symbolFromSymbolTable) : symbolFromSymbolTable;
                    if (symbolFromSymbolTable.flags & meaning) {
                        qualify = true;
                        return true;
                    }
                    // Continue to the next symbol table
                    return false;
                });
                return qualify;
            }
            function isPropertyOrMethodDeclarationSymbol(symbol) {
                if (symbol.declarations && symbol.declarations.length) {
                    for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                        var declaration = _a[_i];
                        switch (declaration.kind) {
                            case 159 /* PropertyDeclaration */:
                            case 161 /* MethodDeclaration */:
                            case 163 /* GetAccessor */:
                            case 164 /* SetAccessor */:
                                continue;
                            default:
                                return false;
                        }
                    }
                    return true;
                }
                return false;
            }
            function isTypeSymbolAccessible(typeSymbol, enclosingDeclaration) {
                var access = isSymbolAccessible(typeSymbol, enclosingDeclaration, 788968 /* Type */, /*shouldComputeAliasesToMakeVisible*/ false);
                return access.accessibility === 0 /* Accessible */;
            }
            function isValueSymbolAccessible(typeSymbol, enclosingDeclaration) {
                var access = isSymbolAccessible(typeSymbol, enclosingDeclaration, 111551 /* Value */, /*shouldComputeAliasesToMakeVisible*/ false);
                return access.accessibility === 0 /* Accessible */;
            }
            function isAnySymbolAccessible(symbols, enclosingDeclaration, initialSymbol, meaning, shouldComputeAliasesToMakeVisible) {
                if (!ts.length(symbols))
                    return;
                var hadAccessibleChain;
                var earlyModuleBail = false;
                for (var _i = 0, _a = symbols; _i < _a.length; _i++) {
                    var symbol = _a[_i];
                    // Symbol is accessible if it by itself is accessible
                    var accessibleSymbolChain = getAccessibleSymbolChain(symbol, enclosingDeclaration, meaning, /*useOnlyExternalAliasing*/ false);
                    if (accessibleSymbolChain) {
                        hadAccessibleChain = symbol;
                        var hasAccessibleDeclarations = hasVisibleDeclarations(accessibleSymbolChain[0], shouldComputeAliasesToMakeVisible);
                        if (hasAccessibleDeclarations) {
                            return hasAccessibleDeclarations;
                        }
                    }
                    else {
                        if (ts.some(symbol.declarations, hasNonGlobalAugmentationExternalModuleSymbol)) {
                            if (shouldComputeAliasesToMakeVisible) {
                                earlyModuleBail = true;
                                // Generally speaking, we want to use the aliases that already exist to refer to a module, if present
                                // In order to do so, we need to find those aliases in order to retain them in declaration emit; so
                                // if we are in declaration emit, we cannot use the fast path for module visibility until we've exhausted
                                // all other visibility options (in order to capture the possible aliases used to reference the module)
                                continue;
                            }
                            // Any meaning of a module symbol is always accessible via an `import` type
                            return {
                                accessibility: 0 /* Accessible */
                            };
                        }
                    }
                    // If we haven't got the accessible symbol, it doesn't mean the symbol is actually inaccessible.
                    // It could be a qualified symbol and hence verify the path
                    // e.g.:
                    // module m {
                    //     export class c {
                    //     }
                    // }
                    // const x: typeof m.c
                    // In the above example when we start with checking if typeof m.c symbol is accessible,
                    // we are going to see if c can be accessed in scope directly.
                    // But it can't, hence the accessible is going to be undefined, but that doesn't mean m.c is inaccessible
                    // It is accessible if the parent m is accessible because then m.c can be accessed through qualification
                    var containers = getContainersOfSymbol(symbol, enclosingDeclaration);
                    // If we're trying to reference some object literal in, eg `var a = { x: 1 }`, the symbol for the literal, `__object`, is distinct
                    // from the symbol of the declaration it is being assigned to. Since we can use the declaration to refer to the literal, however,
                    // we'd like to make that connection here - potentially causing us to paint the declaration's visibility, and therefore the literal.
                    var firstDecl = ts.first(symbol.declarations);
                    if (!ts.length(containers) && meaning & 111551 /* Value */ && firstDecl && ts.isObjectLiteralExpression(firstDecl)) {
                        if (firstDecl.parent && ts.isVariableDeclaration(firstDecl.parent) && firstDecl === firstDecl.parent.initializer) {
                            containers = [getSymbolOfNode(firstDecl.parent)];
                        }
                    }
                    var parentResult = isAnySymbolAccessible(containers, enclosingDeclaration, initialSymbol, initialSymbol === symbol ? getQualifiedLeftMeaning(meaning) : meaning, shouldComputeAliasesToMakeVisible);
                    if (parentResult) {
                        return parentResult;
                    }
                }
                if (earlyModuleBail) {
                    return {
                        accessibility: 0 /* Accessible */
                    };
                }
                if (hadAccessibleChain) {
                    return {
                        accessibility: 1 /* NotAccessible */,
                        errorSymbolName: symbolToString(initialSymbol, enclosingDeclaration, meaning),
                        errorModuleName: hadAccessibleChain !== initialSymbol ? symbolToString(hadAccessibleChain, enclosingDeclaration, 1920 /* Namespace */) : undefined,
                    };
                }
            }
            /**
             * Check if the given symbol in given enclosing declaration is accessible and mark all associated alias to be visible if requested
             *
             * @param symbol a Symbol to check if accessible
             * @param enclosingDeclaration a Node containing reference to the symbol
             * @param meaning a SymbolFlags to check if such meaning of the symbol is accessible
             * @param shouldComputeAliasToMakeVisible a boolean value to indicate whether to return aliases to be mark visible in case the symbol is accessible
             */
            function isSymbolAccessible(symbol, enclosingDeclaration, meaning, shouldComputeAliasesToMakeVisible) {
                if (symbol && enclosingDeclaration) {
                    var result = isAnySymbolAccessible([symbol], enclosingDeclaration, symbol, meaning, shouldComputeAliasesToMakeVisible);
                    if (result) {
                        return result;
                    }
                    // This could be a symbol that is not exported in the external module
                    // or it could be a symbol from different external module that is not aliased and hence cannot be named
                    var symbolExternalModule = ts.forEach(symbol.declarations, getExternalModuleContainer);
                    if (symbolExternalModule) {
                        var enclosingExternalModule = getExternalModuleContainer(enclosingDeclaration);
                        if (symbolExternalModule !== enclosingExternalModule) {
                            // name from different external module that is not visible
                            return {
                                accessibility: 2 /* CannotBeNamed */,
                                errorSymbolName: symbolToString(symbol, enclosingDeclaration, meaning),
                                errorModuleName: symbolToString(symbolExternalModule)
                            };
                        }
                    }
                    // Just a local name that is not accessible
                    return {
                        accessibility: 1 /* NotAccessible */,
                        errorSymbolName: symbolToString(symbol, enclosingDeclaration, meaning),
                    };
                }
                return { accessibility: 0 /* Accessible */ };
            }
            function getExternalModuleContainer(declaration) {
                var node = ts.findAncestor(declaration, hasExternalModuleSymbol);
                return node && getSymbolOfNode(node);
            }
            function hasExternalModuleSymbol(declaration) {
                return ts.isAmbientModule(declaration) || (declaration.kind === 290 /* SourceFile */ && ts.isExternalOrCommonJsModule(declaration));
            }
            function hasNonGlobalAugmentationExternalModuleSymbol(declaration) {
                return ts.isModuleWithStringLiteralName(declaration) || (declaration.kind === 290 /* SourceFile */ && ts.isExternalOrCommonJsModule(declaration));
            }
            function hasVisibleDeclarations(symbol, shouldComputeAliasToMakeVisible) {
                var aliasesToMakeVisible;
                if (!ts.every(ts.filter(symbol.declarations, function (d) { return d.kind !== 75 /* Identifier */; }), getIsDeclarationVisible)) {
                    return undefined;
                }
                return { accessibility: 0 /* Accessible */, aliasesToMakeVisible: aliasesToMakeVisible };
                function getIsDeclarationVisible(declaration) {
                    if (!isDeclarationVisible(declaration)) {
                        // Mark the unexported alias as visible if its parent is visible
                        // because these kind of aliases can be used to name types in declaration file
                        var anyImportSyntax = getAnyImportSyntax(declaration);
                        if (anyImportSyntax &&
                            !ts.hasModifier(anyImportSyntax, 1 /* Export */) && // import clause without export
                            isDeclarationVisible(anyImportSyntax.parent)) {
                            return addVisibleAlias(declaration, anyImportSyntax);
                        }
                        else if (ts.isVariableDeclaration(declaration) && ts.isVariableStatement(declaration.parent.parent) &&
                            !ts.hasModifier(declaration.parent.parent, 1 /* Export */) && // unexported variable statement
                            isDeclarationVisible(declaration.parent.parent.parent)) {
                            return addVisibleAlias(declaration, declaration.parent.parent);
                        }
                        else if (ts.isLateVisibilityPaintedStatement(declaration) // unexported top-level statement
                            && !ts.hasModifier(declaration, 1 /* Export */)
                            && isDeclarationVisible(declaration.parent)) {
                            return addVisibleAlias(declaration, declaration);
                        }
                        // Declaration is not visible
                        return false;
                    }
                    return true;
                }
                function addVisibleAlias(declaration, aliasingStatement) {
                    // In function "buildTypeDisplay" where we decide whether to write type-alias or serialize types,
                    // we want to just check if type- alias is accessible or not but we don't care about emitting those alias at that time
                    // since we will do the emitting later in trackSymbol.
                    if (shouldComputeAliasToMakeVisible) {
                        getNodeLinks(declaration).isVisible = true;
                        aliasesToMakeVisible = ts.appendIfUnique(aliasesToMakeVisible, aliasingStatement);
                    }
                    return true;
                }
            }
            function isEntityNameVisible(entityName, enclosingDeclaration) {
                // get symbol of the first identifier of the entityName
                var meaning;
                if (entityName.parent.kind === 172 /* TypeQuery */ ||
                    ts.isExpressionWithTypeArgumentsInClassExtendsClause(entityName.parent) ||
                    entityName.parent.kind === 154 /* ComputedPropertyName */) {
                    // Typeof value
                    meaning = 111551 /* Value */ | 1048576 /* ExportValue */;
                }
                else if (entityName.kind === 153 /* QualifiedName */ || entityName.kind === 194 /* PropertyAccessExpression */ ||
                    entityName.parent.kind === 253 /* ImportEqualsDeclaration */) {
                    // Left identifier from type reference or TypeAlias
                    // Entity name of the import declaration
                    meaning = 1920 /* Namespace */;
                }
                else {
                    // Type Reference or TypeAlias entity = Identifier
                    meaning = 788968 /* Type */;
                }
                var firstIdentifier = ts.getFirstIdentifier(entityName);
                var symbol = resolveName(enclosingDeclaration, firstIdentifier.escapedText, meaning, /*nodeNotFoundErrorMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ false);
                // Verify if the symbol is accessible
                return (symbol && hasVisibleDeclarations(symbol, /*shouldComputeAliasToMakeVisible*/ true)) || {
                    accessibility: 1 /* NotAccessible */,
                    errorSymbolName: ts.getTextOfNode(firstIdentifier),
                    errorNode: firstIdentifier
                };
            }
            function symbolToString(symbol, enclosingDeclaration, meaning, flags, writer) {
                if (flags === void 0) {
                    flags = 4 /* AllowAnyNodeKind */;
                }
                var nodeFlags = 70221824 /* IgnoreErrors */;
                if (flags & 2 /* UseOnlyExternalAliasing */) {
                    nodeFlags |= 128 /* UseOnlyExternalAliasing */;
                }
                if (flags & 1 /* WriteTypeParametersOrArguments */) {
                    nodeFlags |= 512 /* WriteTypeParametersInQualifiedName */;
                }
                if (flags & 8 /* UseAliasDefinedOutsideCurrentScope */) {
                    nodeFlags |= 16384 /* UseAliasDefinedOutsideCurrentScope */;
                }
                if (flags & 16 /* DoNotIncludeSymbolChain */) {
                    nodeFlags |= 134217728 /* DoNotIncludeSymbolChain */;
                }
                var builder = flags & 4 /* AllowAnyNodeKind */ ? nodeBuilder.symbolToExpression : nodeBuilder.symbolToEntityName;
                return writer ? symbolToStringWorker(writer).getText() : ts.usingSingleLineStringWriter(symbolToStringWorker);
                function symbolToStringWorker(writer) {
                    var entity = builder(symbol, meaning, enclosingDeclaration, nodeFlags); // TODO: GH#18217
                    var printer = ts.createPrinter({ removeComments: true });
                    var sourceFile = enclosingDeclaration && ts.getSourceFileOfNode(enclosingDeclaration);
                    printer.writeNode(4 /* Unspecified */, entity, /*sourceFile*/ sourceFile, writer);
                    return writer;
                }
            }
            function signatureToString(signature, enclosingDeclaration, flags, kind, writer) {
                if (flags === void 0) {
                    flags = 0 /* None */;
                }
                return writer ? signatureToStringWorker(writer).getText() : ts.usingSingleLineStringWriter(signatureToStringWorker);
                function signatureToStringWorker(writer) {
                    var sigOutput;
                    if (flags & 262144 /* WriteArrowStyleSignature */) {
                        sigOutput = kind === 1 /* Construct */ ? 171 /* ConstructorType */ : 170 /* FunctionType */;
                    }
                    else {
                        sigOutput = kind === 1 /* Construct */ ? 166 /* ConstructSignature */ : 165 /* CallSignature */;
                    }
                    var sig = nodeBuilder.signatureToSignatureDeclaration(signature, sigOutput, enclosingDeclaration, toNodeBuilderFlags(flags) | 70221824 /* IgnoreErrors */ | 512 /* WriteTypeParametersInQualifiedName */);
                    var printer = ts.createPrinter({ removeComments: true, omitTrailingSemicolon: true });
                    var sourceFile = enclosingDeclaration && ts.getSourceFileOfNode(enclosingDeclaration);
                    printer.writeNode(4 /* Unspecified */, sig, /*sourceFile*/ sourceFile, ts.getTrailingSemicolonDeferringWriter(writer)); // TODO: GH#18217
                    return writer;
                }
            }
            function typeToString(type, enclosingDeclaration, flags, writer) {
                if (flags === void 0) {
                    flags = 1048576 /* AllowUniqueESSymbolType */ | 16384 /* UseAliasDefinedOutsideCurrentScope */;
                }
                if (writer === void 0) {
                    writer = ts.createTextWriter("");
                }
                var noTruncation = compilerOptions.noErrorTruncation || flags & 1 /* NoTruncation */;
                var typeNode = nodeBuilder.typeToTypeNode(type, enclosingDeclaration, toNodeBuilderFlags(flags) | 70221824 /* IgnoreErrors */ | (noTruncation ? 1 /* NoTruncation */ : 0), writer);
                if (typeNode === undefined)
                    return ts.Debug.fail("should always get typenode");
                var options = { removeComments: true };
                var printer = ts.createPrinter(options);
                var sourceFile = enclosingDeclaration && ts.getSourceFileOfNode(enclosingDeclaration);
                printer.writeNode(4 /* Unspecified */, typeNode, /*sourceFile*/ sourceFile, writer);
                var result = writer.getText();
                var maxLength = noTruncation ? undefined : ts.defaultMaximumTruncationLength * 2;
                if (maxLength && result && result.length >= maxLength) {
                    return result.substr(0, maxLength - "...".length) + "...";
                }
                return result;
            }
            function getTypeNamesForErrorDisplay(left, right) {
                var leftStr = symbolValueDeclarationIsContextSensitive(left.symbol) ? typeToString(left, left.symbol.valueDeclaration) : typeToString(left);
                var rightStr = symbolValueDeclarationIsContextSensitive(right.symbol) ? typeToString(right, right.symbol.valueDeclaration) : typeToString(right);
                if (leftStr === rightStr) {
                    leftStr = typeToString(left, /*enclosingDeclaration*/ undefined, 64 /* UseFullyQualifiedType */);
                    rightStr = typeToString(right, /*enclosingDeclaration*/ undefined, 64 /* UseFullyQualifiedType */);
                }
                return [leftStr, rightStr];
            }
            function symbolValueDeclarationIsContextSensitive(symbol) {
                return symbol && symbol.valueDeclaration && ts.isExpression(symbol.valueDeclaration) && !isContextSensitive(symbol.valueDeclaration);
            }
            function toNodeBuilderFlags(flags) {
                if (flags === void 0) {
                    flags = 0 /* None */;
                }
                return flags & 277904747 /* NodeBuilderFlagsMask */;
            }
            function createNodeBuilder() {
                return {
                    typeToTypeNode: function (type, enclosingDeclaration, flags, tracker) {
                        return withContext(enclosingDeclaration, flags, tracker, function (context) { return typeToTypeNodeHelper(type, context); });
                    },
                    indexInfoToIndexSignatureDeclaration: function (indexInfo, kind, enclosingDeclaration, flags, tracker) {
                        return withContext(enclosingDeclaration, flags, tracker, function (context) { return indexInfoToIndexSignatureDeclarationHelper(indexInfo, kind, context); });
                    },
                    signatureToSignatureDeclaration: function (signature, kind, enclosingDeclaration, flags, tracker) {
                        return withContext(enclosingDeclaration, flags, tracker, function (context) { return signatureToSignatureDeclarationHelper(signature, kind, context); });
                    },
                    symbolToEntityName: function (symbol, meaning, enclosingDeclaration, flags, tracker) {
                        return withContext(enclosingDeclaration, flags, tracker, function (context) { return symbolToName(symbol, context, meaning, /*expectsIdentifier*/ false); });
                    },
                    symbolToExpression: function (symbol, meaning, enclosingDeclaration, flags, tracker) {
                        return withContext(enclosingDeclaration, flags, tracker, function (context) { return symbolToExpression(symbol, context, meaning); });
                    },
                    symbolToTypeParameterDeclarations: function (symbol, enclosingDeclaration, flags, tracker) {
                        return withContext(enclosingDeclaration, flags, tracker, function (context) { return typeParametersToTypeParameterDeclarations(symbol, context); });
                    },
                    symbolToParameterDeclaration: function (symbol, enclosingDeclaration, flags, tracker) {
                        return withContext(enclosingDeclaration, flags, tracker, function (context) { return symbolToParameterDeclaration(symbol, context); });
                    },
                    typeParameterToDeclaration: function (parameter, enclosingDeclaration, flags, tracker) {
                        return withContext(enclosingDeclaration, flags, tracker, function (context) { return typeParameterToDeclaration(parameter, context); });
                    },
                    symbolTableToDeclarationStatements: function (symbolTable, enclosingDeclaration, flags, tracker, bundled) {
                        return withContext(enclosingDeclaration, flags, tracker, function (context) { return symbolTableToDeclarationStatements(symbolTable, context, bundled); });
                    },
                };
                function withContext(enclosingDeclaration, flags, tracker, cb) {
                    ts.Debug.assert(enclosingDeclaration === undefined || (enclosingDeclaration.flags & 8 /* Synthesized */) === 0);
                    var context = {
                        enclosingDeclaration: enclosingDeclaration,
                        flags: flags || 0 /* None */,
                        // If no full tracker is provided, fake up a dummy one with a basic limited-functionality moduleResolverHost
                        tracker: tracker && tracker.trackSymbol ? tracker : { trackSymbol: ts.noop, moduleResolverHost: flags & 134217728 /* DoNotIncludeSymbolChain */ ? {
                                getCommonSourceDirectory: host.getCommonSourceDirectory ? function () { return host.getCommonSourceDirectory(); } : function () { return ""; },
                                getSourceFiles: function () { return host.getSourceFiles(); },
                                getCurrentDirectory: ts.maybeBind(host, host.getCurrentDirectory),
                                getProbableSymlinks: ts.maybeBind(host, host.getProbableSymlinks),
                            } : undefined },
                        encounteredError: false,
                        visitedTypes: undefined,
                        symbolDepth: undefined,
                        inferTypeParameters: undefined,
                        approximateLength: 0
                    };
                    var resultingNode = cb(context);
                    return context.encounteredError ? undefined : resultingNode;
                }
                function checkTruncationLength(context) {
                    if (context.truncating)
                        return context.truncating;
                    return context.truncating = !(context.flags & 1 /* NoTruncation */) && context.approximateLength > ts.defaultMaximumTruncationLength;
                }
                function typeToTypeNodeHelper(type, context) {
                    if (cancellationToken && cancellationToken.throwIfCancellationRequested) {
                        cancellationToken.throwIfCancellationRequested();
                    }
                    var inTypeAlias = context.flags & 8388608 /* InTypeAlias */;
                    context.flags &= ~8388608 /* InTypeAlias */;
                    if (!type) {
                        context.encounteredError = true;
                        return undefined; // TODO: GH#18217
                    }
                    if (type.flags & 1 /* Any */) {
                        context.approximateLength += 3;
                        return ts.createKeywordTypeNode(125 /* AnyKeyword */);
                    }
                    if (type.flags & 2 /* Unknown */) {
                        return ts.createKeywordTypeNode(148 /* UnknownKeyword */);
                    }
                    if (type.flags & 4 /* String */) {
                        context.approximateLength += 6;
                        return ts.createKeywordTypeNode(143 /* StringKeyword */);
                    }
                    if (type.flags & 8 /* Number */) {
                        context.approximateLength += 6;
                        return ts.createKeywordTypeNode(140 /* NumberKeyword */);
                    }
                    if (type.flags & 64 /* BigInt */) {
                        context.approximateLength += 6;
                        return ts.createKeywordTypeNode(151 /* BigIntKeyword */);
                    }
                    if (type.flags & 16 /* Boolean */) {
                        context.approximateLength += 7;
                        return ts.createKeywordTypeNode(128 /* BooleanKeyword */);
                    }
                    if (type.flags & 1024 /* EnumLiteral */ && !(type.flags & 1048576 /* Union */)) {
                        var parentSymbol = getParentOfSymbol(type.symbol);
                        var parentName = symbolToTypeNode(parentSymbol, context, 788968 /* Type */);
                        var enumLiteralName = getDeclaredTypeOfSymbol(parentSymbol) === type
                            ? parentName
                            : appendReferenceToType(parentName, ts.createTypeReferenceNode(ts.symbolName(type.symbol), /*typeArguments*/ undefined));
                        return enumLiteralName;
                    }
                    if (type.flags & 1056 /* EnumLike */) {
                        return symbolToTypeNode(type.symbol, context, 788968 /* Type */);
                    }
                    if (type.flags & 128 /* StringLiteral */) {
                        context.approximateLength += (type.value.length + 2);
                        return ts.createLiteralTypeNode(ts.setEmitFlags(ts.createLiteral(type.value, !!(context.flags & 268435456 /* UseSingleQuotesForStringLiteralType */)), 16777216 /* NoAsciiEscaping */));
                    }
                    if (type.flags & 256 /* NumberLiteral */) {
                        var value = type.value;
                        context.approximateLength += ("" + value).length;
                        return ts.createLiteralTypeNode(value < 0 ? ts.createPrefix(40 /* MinusToken */, ts.createLiteral(-value)) : ts.createLiteral(value));
                    }
                    if (type.flags & 2048 /* BigIntLiteral */) {
                        context.approximateLength += (ts.pseudoBigIntToString(type.value).length) + 1;
                        return ts.createLiteralTypeNode((ts.createLiteral(type.value)));
                    }
                    if (type.flags & 512 /* BooleanLiteral */) {
                        context.approximateLength += type.intrinsicName.length;
                        return type.intrinsicName === "true" ? ts.createTrue() : ts.createFalse();
                    }
                    if (type.flags & 8192 /* UniqueESSymbol */) {
                        if (!(context.flags & 1048576 /* AllowUniqueESSymbolType */)) {
                            if (isValueSymbolAccessible(type.symbol, context.enclosingDeclaration)) {
                                context.approximateLength += 6;
                                return symbolToTypeNode(type.symbol, context, 111551 /* Value */);
                            }
                            if (context.tracker.reportInaccessibleUniqueSymbolError) {
                                context.tracker.reportInaccessibleUniqueSymbolError();
                            }
                        }
                        context.approximateLength += 13;
                        return ts.createTypeOperatorNode(147 /* UniqueKeyword */, ts.createKeywordTypeNode(144 /* SymbolKeyword */));
                    }
                    if (type.flags & 16384 /* Void */) {
                        context.approximateLength += 4;
                        return ts.createKeywordTypeNode(110 /* VoidKeyword */);
                    }
                    if (type.flags & 32768 /* Undefined */) {
                        context.approximateLength += 9;
                        return ts.createKeywordTypeNode(146 /* UndefinedKeyword */);
                    }
                    if (type.flags & 65536 /* Null */) {
                        context.approximateLength += 4;
                        return ts.createKeywordTypeNode(100 /* NullKeyword */);
                    }
                    if (type.flags & 131072 /* Never */) {
                        context.approximateLength += 5;
                        return ts.createKeywordTypeNode(137 /* NeverKeyword */);
                    }
                    if (type.flags & 4096 /* ESSymbol */) {
                        context.approximateLength += 6;
                        return ts.createKeywordTypeNode(144 /* SymbolKeyword */);
                    }
                    if (type.flags & 67108864 /* NonPrimitive */) {
                        context.approximateLength += 6;
                        return ts.createKeywordTypeNode(141 /* ObjectKeyword */);
                    }
                    if (isThisTypeParameter(type)) {
                        if (context.flags & 4194304 /* InObjectTypeLiteral */) {
                            if (!context.encounteredError && !(context.flags & 32768 /* AllowThisInObjectLiteral */)) {
                                context.encounteredError = true;
                            }
                            if (context.tracker.reportInaccessibleThisError) {
                                context.tracker.reportInaccessibleThisError();
                            }
                        }
                        context.approximateLength += 4;
                        return ts.createThis();
                    }
                    if (!inTypeAlias && type.aliasSymbol && (context.flags & 16384 /* UseAliasDefinedOutsideCurrentScope */ || isTypeSymbolAccessible(type.aliasSymbol, context.enclosingDeclaration))) {
                        var typeArgumentNodes = mapToTypeNodes(type.aliasTypeArguments, context);
                        if (isReservedMemberName(type.aliasSymbol.escapedName) && !(type.aliasSymbol.flags & 32 /* Class */))
                            return ts.createTypeReferenceNode(ts.createIdentifier(""), typeArgumentNodes);
                        return symbolToTypeNode(type.aliasSymbol, context, 788968 /* Type */, typeArgumentNodes);
                    }
                    var objectFlags = ts.getObjectFlags(type);
                    if (objectFlags & 4 /* Reference */) {
                        ts.Debug.assert(!!(type.flags & 524288 /* Object */));
                        return type.node ? visitAndTransformType(type, typeReferenceToTypeNode) : typeReferenceToTypeNode(type);
                    }
                    if (type.flags & 262144 /* TypeParameter */ || objectFlags & 3 /* ClassOrInterface */) {
                        if (type.flags & 262144 /* TypeParameter */ && ts.contains(context.inferTypeParameters, type)) {
                            context.approximateLength += (ts.symbolName(type.symbol).length + 6);
                            return ts.createInferTypeNode(typeParameterToDeclarationWithConstraint(type, context, /*constraintNode*/ undefined));
                        }
                        if (context.flags & 4 /* GenerateNamesForShadowedTypeParams */ &&
                            type.flags & 262144 /* TypeParameter */ &&
                            !isTypeSymbolAccessible(type.symbol, context.enclosingDeclaration)) {
                            var name = typeParameterToName(type, context);
                            context.approximateLength += ts.idText(name).length;
                            return ts.createTypeReferenceNode(ts.createIdentifier(ts.idText(name)), /*typeArguments*/ undefined);
                        }
                        // Ignore constraint/default when creating a usage (as opposed to declaration) of a type parameter.
                        return type.symbol
                            ? symbolToTypeNode(type.symbol, context, 788968 /* Type */)
                            : ts.createTypeReferenceNode(ts.createIdentifier("?"), /*typeArguments*/ undefined);
                    }
                    if (type.flags & (1048576 /* Union */ | 2097152 /* Intersection */)) {
                        var types = type.flags & 1048576 /* Union */ ? formatUnionTypes(type.types) : type.types;
                        if (ts.length(types) === 1) {
                            return typeToTypeNodeHelper(types[0], context);
                        }
                        var typeNodes = mapToTypeNodes(types, context, /*isBareList*/ true);
                        if (typeNodes && typeNodes.length > 0) {
                            var unionOrIntersectionTypeNode = ts.createUnionOrIntersectionTypeNode(type.flags & 1048576 /* Union */ ? 178 /* UnionType */ : 179 /* IntersectionType */, typeNodes);
                            return unionOrIntersectionTypeNode;
                        }
                        else {
                            if (!context.encounteredError && !(context.flags & 262144 /* AllowEmptyUnionOrIntersection */)) {
                                context.encounteredError = true;
                            }
                            return undefined; // TODO: GH#18217
                        }
                    }
                    if (objectFlags & (16 /* Anonymous */ | 32 /* Mapped */)) {
                        ts.Debug.assert(!!(type.flags & 524288 /* Object */));
                        // The type is an object literal type.
                        return createAnonymousTypeNode(type);
                    }
                    if (type.flags & 4194304 /* Index */) {
                        var indexedType = type.type;
                        context.approximateLength += 6;
                        var indexTypeNode = typeToTypeNodeHelper(indexedType, context);
                        return ts.createTypeOperatorNode(indexTypeNode);
                    }
                    if (type.flags & 8388608 /* IndexedAccess */) {
                        var objectTypeNode = typeToTypeNodeHelper(type.objectType, context);
                        var indexTypeNode = typeToTypeNodeHelper(type.indexType, context);
                        context.approximateLength += 2;
                        return ts.createIndexedAccessTypeNode(objectTypeNode, indexTypeNode);
                    }
                    if (type.flags & 16777216 /* Conditional */) {
                        var checkTypeNode = typeToTypeNodeHelper(type.checkType, context);
                        var saveInferTypeParameters = context.inferTypeParameters;
                        context.inferTypeParameters = type.root.inferTypeParameters;
                        var extendsTypeNode = typeToTypeNodeHelper(type.extendsType, context);
                        context.inferTypeParameters = saveInferTypeParameters;
                        var trueTypeNode = typeToTypeNodeHelper(getTrueTypeFromConditionalType(type), context);
                        var falseTypeNode = typeToTypeNodeHelper(getFalseTypeFromConditionalType(type), context);
                        context.approximateLength += 15;
                        return ts.createConditionalTypeNode(checkTypeNode, extendsTypeNode, trueTypeNode, falseTypeNode);
                    }
                    if (type.flags & 33554432 /* Substitution */) {
                        return typeToTypeNodeHelper(type.typeVariable, context);
                    }
                    return ts.Debug.fail("Should be unreachable.");
                    function createMappedTypeNodeFromType(type) {
                        ts.Debug.assert(!!(type.flags & 524288 /* Object */));
                        var readonlyToken = type.declaration.readonlyToken ? ts.createToken(type.declaration.readonlyToken.kind) : undefined;
                        var questionToken = type.declaration.questionToken ? ts.createToken(type.declaration.questionToken.kind) : undefined;
                        var appropriateConstraintTypeNode;
                        if (isMappedTypeWithKeyofConstraintDeclaration(type)) {
                            // We have a { [P in keyof T]: X }
                            // We do this to ensure we retain the toplevel keyof-ness of the type which may be lost due to keyof distribution during `getConstraintTypeFromMappedType`
                            appropriateConstraintTypeNode = ts.createTypeOperatorNode(typeToTypeNodeHelper(getModifiersTypeFromMappedType(type), context));
                        }
                        else {
                            appropriateConstraintTypeNode = typeToTypeNodeHelper(getConstraintTypeFromMappedType(type), context);
                        }
                        var typeParameterNode = typeParameterToDeclarationWithConstraint(getTypeParameterFromMappedType(type), context, appropriateConstraintTypeNode);
                        var templateTypeNode = typeToTypeNodeHelper(getTemplateTypeFromMappedType(type), context);
                        var mappedTypeNode = ts.createMappedTypeNode(readonlyToken, typeParameterNode, questionToken, templateTypeNode);
                        context.approximateLength += 10;
                        return ts.setEmitFlags(mappedTypeNode, 1 /* SingleLine */);
                    }
                    function createAnonymousTypeNode(type) {
                        var typeId = "" + type.id;
                        var symbol = type.symbol;
                        if (symbol) {
                            if (isJSConstructor(symbol.valueDeclaration)) {
                                // Instance and static types share the same symbol; only add 'typeof' for the static side.
                                var isInstanceType = type === getDeclaredTypeOfClassOrInterface(symbol) ? 788968 /* Type */ : 111551 /* Value */;
                                return symbolToTypeNode(symbol, context, isInstanceType);
                            }
                            // Always use 'typeof T' for type of class, enum, and module objects
                            else if (symbol.flags & 32 /* Class */ && !getBaseTypeVariableOfClass(symbol) && !(symbol.valueDeclaration.kind === 214 /* ClassExpression */ && context.flags & 2048 /* WriteClassExpressionAsTypeLiteral */) ||
                                symbol.flags & (384 /* Enum */ | 512 /* ValueModule */) ||
                                shouldWriteTypeOfFunctionSymbol()) {
                                return symbolToTypeNode(symbol, context, 111551 /* Value */);
                            }
                            else if (context.visitedTypes && context.visitedTypes.has(typeId)) {
                                // If type is an anonymous type literal in a type alias declaration, use type alias name
                                var typeAlias = getTypeAliasForTypeLiteral(type);
                                if (typeAlias) {
                                    // The specified symbol flags need to be reinterpreted as type flags
                                    return symbolToTypeNode(typeAlias, context, 788968 /* Type */);
                                }
                                else {
                                    return createElidedInformationPlaceholder(context);
                                }
                            }
                            else {
                                return visitAndTransformType(type, createTypeNodeFromObjectType);
                            }
                        }
                        else {
                            // Anonymous types without a symbol are never circular.
                            return createTypeNodeFromObjectType(type);
                        }
                        function shouldWriteTypeOfFunctionSymbol() {
                            var isStaticMethodSymbol = !!(symbol.flags & 8192 /* Method */) && // typeof static method
                                ts.some(symbol.declarations, function (declaration) { return ts.hasModifier(declaration, 32 /* Static */); });
                            var isNonLocalFunctionSymbol = !!(symbol.flags & 16 /* Function */) &&
                                (symbol.parent || // is exported function symbol
                                    ts.forEach(symbol.declarations, function (declaration) {
                                        return declaration.parent.kind === 290 /* SourceFile */ || declaration.parent.kind === 250 /* ModuleBlock */;
                                    }));
                            if (isStaticMethodSymbol || isNonLocalFunctionSymbol) {
                                // typeof is allowed only for static/non local functions
                                return (!!(context.flags & 4096 /* UseTypeOfFunction */) || (context.visitedTypes && context.visitedTypes.has(typeId))) && // it is type of the symbol uses itself recursively
                                    (!(context.flags & 8 /* UseStructuralFallback */) || isValueSymbolAccessible(symbol, context.enclosingDeclaration)); // And the build is going to succeed without visibility error or there is no structural fallback allowed
                            }
                        }
                    }
                    function visitAndTransformType(type, transform) {
                        var typeId = "" + type.id;
                        var isConstructorObject = ts.getObjectFlags(type) & 16 /* Anonymous */ && type.symbol && type.symbol.flags & 32 /* Class */;
                        var id = ts.getObjectFlags(type) & 4 /* Reference */ && type.node ? "N" + getNodeId(type.node) :
                            type.symbol ? (isConstructorObject ? "+" : "") + getSymbolId(type.symbol) :
                                undefined;
                        // Since instantiations of the same anonymous type have the same symbol, tracking symbols instead
                        // of types allows us to catch circular references to instantiations of the same anonymous type
                        if (!context.visitedTypes) {
                            context.visitedTypes = ts.createMap();
                        }
                        if (id && !context.symbolDepth) {
                            context.symbolDepth = ts.createMap();
                        }
                        var depth;
                        if (id) {
                            depth = context.symbolDepth.get(id) || 0;
                            if (depth > 10) {
                                return createElidedInformationPlaceholder(context);
                            }
                            context.symbolDepth.set(id, depth + 1);
                        }
                        context.visitedTypes.set(typeId, true);
                        var result = transform(type);
                        context.visitedTypes.delete(typeId);
                        if (id) {
                            context.symbolDepth.set(id, depth);
                        }
                        return result;
                    }
                    function createTypeNodeFromObjectType(type) {
                        if (isGenericMappedType(type)) {
                            return createMappedTypeNodeFromType(type);
                        }
                        var resolved = resolveStructuredTypeMembers(type);
                        if (!resolved.properties.length && !resolved.stringIndexInfo && !resolved.numberIndexInfo) {
                            if (!resolved.callSignatures.length && !resolved.constructSignatures.length) {
                                context.approximateLength += 2;
                                return ts.setEmitFlags(ts.createTypeLiteralNode(/*members*/ undefined), 1 /* SingleLine */);
                            }
                            if (resolved.callSignatures.length === 1 && !resolved.constructSignatures.length) {
                                var signature = resolved.callSignatures[0];
                                var signatureNode = signatureToSignatureDeclarationHelper(signature, 170 /* FunctionType */, context);
                                return signatureNode;
                            }
                            if (resolved.constructSignatures.length === 1 && !resolved.callSignatures.length) {
                                var signature = resolved.constructSignatures[0];
                                var signatureNode = signatureToSignatureDeclarationHelper(signature, 171 /* ConstructorType */, context);
                                return signatureNode;
                            }
                        }
                        var savedFlags = context.flags;
                        context.flags |= 4194304 /* InObjectTypeLiteral */;
                        var members = createTypeNodesFromResolvedType(resolved);
                        context.flags = savedFlags;
                        var typeLiteralNode = ts.createTypeLiteralNode(members);
                        context.approximateLength += 2;
                        return ts.setEmitFlags(typeLiteralNode, (context.flags & 1024 /* MultilineObjectLiterals */) ? 0 : 1 /* SingleLine */);
                    }
                    function typeReferenceToTypeNode(type) {
                        var typeArguments = getTypeArguments(type);
                        if (type.target === globalArrayType || type.target === globalReadonlyArrayType) {
                            if (context.flags & 2 /* WriteArrayAsGenericType */) {
                                var typeArgumentNode = typeToTypeNodeHelper(typeArguments[0], context);
                                return ts.createTypeReferenceNode(type.target === globalArrayType ? "Array" : "ReadonlyArray", [typeArgumentNode]);
                            }
                            var elementType = typeToTypeNodeHelper(typeArguments[0], context);
                            var arrayType = ts.createArrayTypeNode(elementType);
                            return type.target === globalArrayType ? arrayType : ts.createTypeOperatorNode(138 /* ReadonlyKeyword */, arrayType);
                        }
                        else if (type.target.objectFlags & 8 /* Tuple */) {
                            if (typeArguments.length > 0) {
                                var arity = getTypeReferenceArity(type);
                                var tupleConstituentNodes = mapToTypeNodes(typeArguments.slice(0, arity), context);
                                var hasRestElement = type.target.hasRestElement;
                                if (tupleConstituentNodes) {
                                    for (var i = type.target.minLength; i < Math.min(arity, tupleConstituentNodes.length); i++) {
                                        tupleConstituentNodes[i] = hasRestElement && i === arity - 1 ?
                                            ts.createRestTypeNode(ts.createArrayTypeNode(tupleConstituentNodes[i])) :
                                            ts.createOptionalTypeNode(tupleConstituentNodes[i]);
                                    }
                                    var tupleTypeNode = ts.createTupleTypeNode(tupleConstituentNodes);
                                    return type.target.readonly ? ts.createTypeOperatorNode(138 /* ReadonlyKeyword */, tupleTypeNode) : tupleTypeNode;
                                }
                            }
                            if (context.encounteredError || (context.flags & 524288 /* AllowEmptyTuple */)) {
                                var tupleTypeNode = ts.createTupleTypeNode([]);
                                return type.target.readonly ? ts.createTypeOperatorNode(138 /* ReadonlyKeyword */, tupleTypeNode) : tupleTypeNode;
                            }
                            context.encounteredError = true;
                            return undefined; // TODO: GH#18217
                        }
                        else if (context.flags & 2048 /* WriteClassExpressionAsTypeLiteral */ &&
                            type.symbol.valueDeclaration &&
                            ts.isClassLike(type.symbol.valueDeclaration) &&
                            !isValueSymbolAccessible(type.symbol, context.enclosingDeclaration)) {
                            return createAnonymousTypeNode(type);
                        }
                        else {
                            var outerTypeParameters = type.target.outerTypeParameters;
                            var i = 0;
                            var resultType = void 0;
                            if (outerTypeParameters) {
                                var length_1 = outerTypeParameters.length;
                                while (i < length_1) {
                                    // Find group of type arguments for type parameters with the same declaring container.
                                    var start = i;
                                    var parent = getParentSymbolOfTypeParameter(outerTypeParameters[i]);
                                    do {
                                        i++;
                                    } while (i < length_1 && getParentSymbolOfTypeParameter(outerTypeParameters[i]) === parent);
                                    // When type parameters are their own type arguments for the whole group (i.e. we have
                                    // the default outer type arguments), we don't show the group.
                                    if (!ts.rangeEquals(outerTypeParameters, typeArguments, start, i)) {
                                        var typeArgumentSlice = mapToTypeNodes(typeArguments.slice(start, i), context);
                                        var flags_2 = context.flags;
                                        context.flags |= 16 /* ForbidIndexedAccessSymbolReferences */;
                                        var ref = symbolToTypeNode(parent, context, 788968 /* Type */, typeArgumentSlice);
                                        context.flags = flags_2;
                                        resultType = !resultType ? ref : appendReferenceToType(resultType, ref);
                                    }
                                }
                            }
                            var typeArgumentNodes = void 0;
                            if (typeArguments.length > 0) {
                                var typeParameterCount = (type.target.typeParameters || ts.emptyArray).length;
                                typeArgumentNodes = mapToTypeNodes(typeArguments.slice(i, typeParameterCount), context);
                            }
                            var flags = context.flags;
                            context.flags |= 16 /* ForbidIndexedAccessSymbolReferences */;
                            var finalRef = symbolToTypeNode(type.symbol, context, 788968 /* Type */, typeArgumentNodes);
                            context.flags = flags;
                            return !resultType ? finalRef : appendReferenceToType(resultType, finalRef);
                        }
                    }
                    function appendReferenceToType(root, ref) {
                        if (ts.isImportTypeNode(root)) {
                            // first shift type arguments
                            var innerParams = root.typeArguments;
                            if (root.qualifier) {
                                (ts.isIdentifier(root.qualifier) ? root.qualifier : root.qualifier.right).typeArguments = innerParams;
                            }
                            root.typeArguments = ref.typeArguments;
                            // then move qualifiers
                            var ids = getAccessStack(ref);
                            for (var _i = 0, ids_1 = ids; _i < ids_1.length; _i++) {
                                var id = ids_1[_i];
                                root.qualifier = root.qualifier ? ts.createQualifiedName(root.qualifier, id) : id;
                            }
                            return root;
                        }
                        else {
                            // first shift type arguments
                            var innerParams = root.typeArguments;
                            (ts.isIdentifier(root.typeName) ? root.typeName : root.typeName.right).typeArguments = innerParams;
                            root.typeArguments = ref.typeArguments;
                            // then move qualifiers
                            var ids = getAccessStack(ref);
                            for (var _a = 0, ids_2 = ids; _a < ids_2.length; _a++) {
                                var id = ids_2[_a];
                                root.typeName = ts.createQualifiedName(root.typeName, id);
                            }
                            return root;
                        }
                    }
                    function getAccessStack(ref) {
                        var state = ref.typeName;
                        var ids = [];
                        while (!ts.isIdentifier(state)) {
                            ids.unshift(state.right);
                            state = state.left;
                        }
                        ids.unshift(state);
                        return ids;
                    }
                    function createTypeNodesFromResolvedType(resolvedType) {
                        if (checkTruncationLength(context)) {
                            return [ts.createPropertySignature(/*modifiers*/ undefined, "...", /*questionToken*/ undefined, /*type*/ undefined, /*initializer*/ undefined)];
                        }
                        var typeElements = [];
                        for (var _i = 0, _a = resolvedType.callSignatures; _i < _a.length; _i++) {
                            var signature = _a[_i];
                            typeElements.push(signatureToSignatureDeclarationHelper(signature, 165 /* CallSignature */, context));
                        }
                        for (var _b = 0, _c = resolvedType.constructSignatures; _b < _c.length; _b++) {
                            var signature = _c[_b];
                            typeElements.push(signatureToSignatureDeclarationHelper(signature, 166 /* ConstructSignature */, context));
                        }
                        if (resolvedType.stringIndexInfo) {
                            var indexSignature = void 0;
                            if (resolvedType.objectFlags & 2048 /* ReverseMapped */) {
                                indexSignature = indexInfoToIndexSignatureDeclarationHelper(createIndexInfo(anyType, resolvedType.stringIndexInfo.isReadonly, resolvedType.stringIndexInfo.declaration), 0 /* String */, context);
                                indexSignature.type = createElidedInformationPlaceholder(context);
                            }
                            else {
                                indexSignature = indexInfoToIndexSignatureDeclarationHelper(resolvedType.stringIndexInfo, 0 /* String */, context);
                            }
                            typeElements.push(indexSignature);
                        }
                        if (resolvedType.numberIndexInfo) {
                            typeElements.push(indexInfoToIndexSignatureDeclarationHelper(resolvedType.numberIndexInfo, 1 /* Number */, context));
                        }
                        var properties = resolvedType.properties;
                        if (!properties) {
                            return typeElements;
                        }
                        var i = 0;
                        for (var _d = 0, properties_1 = properties; _d < properties_1.length; _d++) {
                            var propertySymbol = properties_1[_d];
                            i++;
                            if (context.flags & 2048 /* WriteClassExpressionAsTypeLiteral */) {
                                if (propertySymbol.flags & 4194304 /* Prototype */) {
                                    continue;
                                }
                                if (ts.getDeclarationModifierFlagsFromSymbol(propertySymbol) & (8 /* Private */ | 16 /* Protected */) && context.tracker.reportPrivateInBaseOfClassExpression) {
                                    context.tracker.reportPrivateInBaseOfClassExpression(ts.unescapeLeadingUnderscores(propertySymbol.escapedName));
                                }
                            }
                            if (checkTruncationLength(context) && (i + 2 < properties.length - 1)) {
                                typeElements.push(ts.createPropertySignature(/*modifiers*/ undefined, "... " + (properties.length - i) + " more ...", /*questionToken*/ undefined, /*type*/ undefined, /*initializer*/ undefined));
                                addPropertyToElementList(properties[properties.length - 1], context, typeElements);
                                break;
                            }
                            addPropertyToElementList(propertySymbol, context, typeElements);
                        }
                        return typeElements.length ? typeElements : undefined;
                    }
                }
                function createElidedInformationPlaceholder(context) {
                    context.approximateLength += 3;
                    if (!(context.flags & 1 /* NoTruncation */)) {
                        return ts.createTypeReferenceNode(ts.createIdentifier("..."), /*typeArguments*/ undefined);
                    }
                    return ts.createKeywordTypeNode(125 /* AnyKeyword */);
                }
                function addPropertyToElementList(propertySymbol, context, typeElements) {
                    var propertyIsReverseMapped = !!(ts.getCheckFlags(propertySymbol) & 8192 /* ReverseMapped */);
                    var propertyType = propertyIsReverseMapped && context.flags & 33554432 /* InReverseMappedType */ ?
                        anyType : getTypeOfSymbol(propertySymbol);
                    var saveEnclosingDeclaration = context.enclosingDeclaration;
                    context.enclosingDeclaration = undefined;
                    if (context.tracker.trackSymbol && ts.getCheckFlags(propertySymbol) & 4096 /* Late */) {
                        var decl = ts.first(propertySymbol.declarations);
                        if (hasLateBindableName(decl)) {
                            if (ts.isBinaryExpression(decl)) {
                                var name = ts.getNameOfDeclaration(decl);
                                if (name && ts.isElementAccessExpression(name) && ts.isPropertyAccessEntityNameExpression(name.argumentExpression)) {
                                    trackComputedName(name.argumentExpression, saveEnclosingDeclaration, context);
                                }
                            }
                            else {
                                trackComputedName(decl.name.expression, saveEnclosingDeclaration, context);
                            }
                        }
                    }
                    context.enclosingDeclaration = saveEnclosingDeclaration;
                    var propertyName = getPropertyNameNodeForSymbol(propertySymbol, context);
                    context.approximateLength += (ts.symbolName(propertySymbol).length + 1);
                    var optionalToken = propertySymbol.flags & 16777216 /* Optional */ ? ts.createToken(57 /* QuestionToken */) : undefined;
                    if (propertySymbol.flags & (16 /* Function */ | 8192 /* Method */) && !getPropertiesOfObjectType(propertyType).length && !isReadonlySymbol(propertySymbol)) {
                        var signatures = getSignaturesOfType(filterType(propertyType, function (t) { return !(t.flags & 32768 /* Undefined */); }), 0 /* Call */);
                        for (var _i = 0, signatures_1 = signatures; _i < signatures_1.length; _i++) {
                            var signature = signatures_1[_i];
                            var methodDeclaration = signatureToSignatureDeclarationHelper(signature, 160 /* MethodSignature */, context);
                            methodDeclaration.name = propertyName;
                            methodDeclaration.questionToken = optionalToken;
                            typeElements.push(preserveCommentsOn(methodDeclaration));
                        }
                    }
                    else {
                        var savedFlags = context.flags;
                        context.flags |= propertyIsReverseMapped ? 33554432 /* InReverseMappedType */ : 0;
                        var propertyTypeNode = void 0;
                        if (propertyIsReverseMapped && !!(savedFlags & 33554432 /* InReverseMappedType */)) {
                            propertyTypeNode = createElidedInformationPlaceholder(context);
                        }
                        else {
                            propertyTypeNode = propertyType ? typeToTypeNodeHelper(propertyType, context) : ts.createKeywordTypeNode(125 /* AnyKeyword */);
                        }
                        context.flags = savedFlags;
                        var modifiers = isReadonlySymbol(propertySymbol) ? [ts.createToken(138 /* ReadonlyKeyword */)] : undefined;
                        if (modifiers) {
                            context.approximateLength += 9;
                        }
                        var propertySignature = ts.createPropertySignature(modifiers, propertyName, optionalToken, propertyTypeNode, 
                        /*initializer*/ undefined);
                        typeElements.push(preserveCommentsOn(propertySignature));
                    }
                    function preserveCommentsOn(node) {
                        if (ts.some(propertySymbol.declarations, function (d) { return d.kind === 322 /* JSDocPropertyTag */; })) {
                            var d = ts.find(propertySymbol.declarations, function (d) { return d.kind === 322 /* JSDocPropertyTag */; });
                            var commentText = d.comment;
                            if (commentText) {
                                ts.setSyntheticLeadingComments(node, [{ kind: 3 /* MultiLineCommentTrivia */, text: "*\n * " + commentText.replace(/\n/g, "\n * ") + "\n ", pos: -1, end: -1, hasTrailingNewLine: true }]);
                            }
                        }
                        else if (propertySymbol.valueDeclaration) {
                            // Copy comments to node for declaration emit
                            ts.setCommentRange(node, propertySymbol.valueDeclaration);
                        }
                        return node;
                    }
                }
                function mapToTypeNodes(types, context, isBareList) {
                    if (ts.some(types)) {
                        if (checkTruncationLength(context)) {
                            if (!isBareList) {
                                return [ts.createTypeReferenceNode("...", /*typeArguments*/ undefined)];
                            }
                            else if (types.length > 2) {
                                return [
                                    typeToTypeNodeHelper(types[0], context),
                                    ts.createTypeReferenceNode("... " + (types.length - 2) + " more ...", /*typeArguments*/ undefined),
                                    typeToTypeNodeHelper(types[types.length - 1], context)
                                ];
                            }
                        }
                        var result = [];
                        var i = 0;
                        for (var _i = 0, types_1 = types; _i < types_1.length; _i++) {
                            var type = types_1[_i];
                            i++;
                            if (checkTruncationLength(context) && (i + 2 < types.length - 1)) {
                                result.push(ts.createTypeReferenceNode("... " + (types.length - i) + " more ...", /*typeArguments*/ undefined));
                                var typeNode_1 = typeToTypeNodeHelper(types[types.length - 1], context);
                                if (typeNode_1) {
                                    result.push(typeNode_1);
                                }
                                break;
                            }
                            context.approximateLength += 2; // Account for whitespace + separator
                            var typeNode = typeToTypeNodeHelper(type, context);
                            if (typeNode) {
                                result.push(typeNode);
                            }
                        }
                        return result;
                    }
                }
                function indexInfoToIndexSignatureDeclarationHelper(indexInfo, kind, context) {
                    var name = ts.getNameFromIndexInfo(indexInfo) || "x";
                    var indexerTypeNode = ts.createKeywordTypeNode(kind === 0 /* String */ ? 143 /* StringKeyword */ : 140 /* NumberKeyword */);
                    var indexingParameter = ts.createParameter(
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, 
                    /*dotDotDotToken*/ undefined, name, 
                    /*questionToken*/ undefined, indexerTypeNode, 
                    /*initializer*/ undefined);
                    var typeNode = typeToTypeNodeHelper(indexInfo.type || anyType, context);
                    if (!indexInfo.type && !(context.flags & 2097152 /* AllowEmptyIndexInfoType */)) {
                        context.encounteredError = true;
                    }
                    context.approximateLength += (name.length + 4);
                    return ts.createIndexSignature(
                    /*decorators*/ undefined, indexInfo.isReadonly ? [ts.createToken(138 /* ReadonlyKeyword */)] : undefined, [indexingParameter], typeNode);
                }
                function signatureToSignatureDeclarationHelper(signature, kind, context) {
                    var suppressAny = context.flags & 256 /* SuppressAnyReturnType */;
                    if (suppressAny)
                        context.flags &= ~256 /* SuppressAnyReturnType */; // suppress only toplevel `any`s
                    var typeParameters;
                    var typeArguments;
                    if (context.flags & 32 /* WriteTypeArgumentsOfSignature */ && signature.target && signature.mapper && signature.target.typeParameters) {
                        typeArguments = signature.target.typeParameters.map(function (parameter) { return typeToTypeNodeHelper(instantiateType(parameter, signature.mapper), context); });
                    }
                    else {
                        typeParameters = signature.typeParameters && signature.typeParameters.map(function (parameter) { return typeParameterToDeclaration(parameter, context); });
                    }
                    var parameters = getExpandedParameters(signature).map(function (parameter) { return symbolToParameterDeclaration(parameter, context, kind === 162 /* Constructor */); });
                    if (signature.thisParameter) {
                        var thisParameter = symbolToParameterDeclaration(signature.thisParameter, context);
                        parameters.unshift(thisParameter);
                    }
                    var returnTypeNode;
                    var typePredicate = getTypePredicateOfSignature(signature);
                    if (typePredicate) {
                        var assertsModifier = typePredicate.kind === 2 /* AssertsThis */ || typePredicate.kind === 3 /* AssertsIdentifier */ ?
                            ts.createToken(124 /* AssertsKeyword */) :
                            undefined;
                        var parameterName = typePredicate.kind === 1 /* Identifier */ || typePredicate.kind === 3 /* AssertsIdentifier */ ?
                            ts.setEmitFlags(ts.createIdentifier(typePredicate.parameterName), 16777216 /* NoAsciiEscaping */) :
                            ts.createThisTypeNode();
                        var typeNode = typePredicate.type && typeToTypeNodeHelper(typePredicate.type, context);
                        returnTypeNode = ts.createTypePredicateNodeWithModifier(assertsModifier, parameterName, typeNode);
                    }
                    else {
                        var returnType = getReturnTypeOfSignature(signature);
                        if (returnType && !(suppressAny && isTypeAny(returnType))) {
                            returnTypeNode = typeToTypeNodeHelper(returnType, context);
                        }
                        else if (!suppressAny) {
                            returnTypeNode = ts.createKeywordTypeNode(125 /* AnyKeyword */);
                        }
                    }
                    context.approximateLength += 3; // Usually a signature contributes a few more characters than this, but 3 is the minimum
                    return ts.createSignatureDeclaration(kind, typeParameters, parameters, returnTypeNode, typeArguments);
                }
                function typeParameterToDeclarationWithConstraint(type, context, constraintNode) {
                    var savedContextFlags = context.flags;
                    context.flags &= ~512 /* WriteTypeParametersInQualifiedName */; // Avoids potential infinite loop when building for a claimspace with a generic
                    var name = typeParameterToName(type, context);
                    var defaultParameter = getDefaultFromTypeParameter(type);
                    var defaultParameterNode = defaultParameter && typeToTypeNodeHelper(defaultParameter, context);
                    context.flags = savedContextFlags;
                    return ts.createTypeParameterDeclaration(name, constraintNode, defaultParameterNode);
                }
                function typeParameterToDeclaration(type, context, constraint) {
                    if (constraint === void 0) {
                        constraint = getConstraintOfTypeParameter(type);
                    }
                    var constraintNode = constraint && typeToTypeNodeHelper(constraint, context);
                    return typeParameterToDeclarationWithConstraint(type, context, constraintNode);
                }
                function symbolToParameterDeclaration(parameterSymbol, context, preserveModifierFlags) {
                    var parameterDeclaration = ts.getDeclarationOfKind(parameterSymbol, 156 /* Parameter */);
                    if (!parameterDeclaration && !ts.isTransientSymbol(parameterSymbol)) {
                        parameterDeclaration = ts.getDeclarationOfKind(parameterSymbol, 316 /* JSDocParameterTag */);
                    }
                    var parameterType = getTypeOfSymbol(parameterSymbol);
                    if (parameterDeclaration && isRequiredInitializedParameter(parameterDeclaration)) {
                        parameterType = getOptionalType(parameterType);
                    }
                    var parameterTypeNode = typeToTypeNodeHelper(parameterType, context);
                    var modifiers = !(context.flags & 8192 /* OmitParameterModifiers */) && preserveModifierFlags && parameterDeclaration && parameterDeclaration.modifiers ? parameterDeclaration.modifiers.map(ts.getSynthesizedClone) : undefined;
                    var isRest = parameterDeclaration && ts.isRestParameter(parameterDeclaration) || ts.getCheckFlags(parameterSymbol) & 32768 /* RestParameter */;
                    var dotDotDotToken = isRest ? ts.createToken(25 /* DotDotDotToken */) : undefined;
                    var name = parameterDeclaration ? parameterDeclaration.name ?
                        parameterDeclaration.name.kind === 75 /* Identifier */ ? ts.setEmitFlags(ts.getSynthesizedClone(parameterDeclaration.name), 16777216 /* NoAsciiEscaping */) :
                            parameterDeclaration.name.kind === 153 /* QualifiedName */ ? ts.setEmitFlags(ts.getSynthesizedClone(parameterDeclaration.name.right), 16777216 /* NoAsciiEscaping */) :
                                cloneBindingName(parameterDeclaration.name) :
                        ts.symbolName(parameterSymbol) :
                        ts.symbolName(parameterSymbol);
                    var isOptional = parameterDeclaration && isOptionalParameter(parameterDeclaration) || ts.getCheckFlags(parameterSymbol) & 16384 /* OptionalParameter */;
                    var questionToken = isOptional ? ts.createToken(57 /* QuestionToken */) : undefined;
                    var parameterNode = ts.createParameter(
                    /*decorators*/ undefined, modifiers, dotDotDotToken, name, questionToken, parameterTypeNode, 
                    /*initializer*/ undefined);
                    context.approximateLength += ts.symbolName(parameterSymbol).length + 3;
                    return parameterNode;
                    function cloneBindingName(node) {
                        return elideInitializerAndSetEmitFlags(node);
                        function elideInitializerAndSetEmitFlags(node) {
                            if (context.tracker.trackSymbol && ts.isComputedPropertyName(node) && isLateBindableName(node)) {
                                trackComputedName(node.expression, context.enclosingDeclaration, context);
                            }
                            var visited = ts.visitEachChild(node, elideInitializerAndSetEmitFlags, ts.nullTransformationContext, /*nodesVisitor*/ undefined, elideInitializerAndSetEmitFlags);
                            var clone = ts.nodeIsSynthesized(visited) ? visited : ts.getSynthesizedClone(visited);
                            if (clone.kind === 191 /* BindingElement */) {
                                clone.initializer = undefined;
                            }
                            return ts.setEmitFlags(clone, 1 /* SingleLine */ | 16777216 /* NoAsciiEscaping */);
                        }
                    }
                }
                function trackComputedName(accessExpression, enclosingDeclaration, context) {
                    if (!context.tracker.trackSymbol)
                        return;
                    // get symbol of the first identifier of the entityName
                    var firstIdentifier = ts.getFirstIdentifier(accessExpression);
                    var name = resolveName(firstIdentifier, firstIdentifier.escapedText, 111551 /* Value */ | 1048576 /* ExportValue */, /*nodeNotFoundErrorMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ true);
                    if (name) {
                        context.tracker.trackSymbol(name, enclosingDeclaration, 111551 /* Value */);
                    }
                }
                function lookupSymbolChain(symbol, context, meaning, yieldModuleSymbol) {
                    context.tracker.trackSymbol(symbol, context.enclosingDeclaration, meaning); // TODO: GH#18217
                    return lookupSymbolChainWorker(symbol, context, meaning, yieldModuleSymbol);
                }
                function lookupSymbolChainWorker(symbol, context, meaning, yieldModuleSymbol) {
                    // Try to get qualified name if the symbol is not a type parameter and there is an enclosing declaration.
                    var chain;
                    var isTypeParameter = symbol.flags & 262144 /* TypeParameter */;
                    if (!isTypeParameter && (context.enclosingDeclaration || context.flags & 64 /* UseFullyQualifiedType */) && !(context.flags & 134217728 /* DoNotIncludeSymbolChain */)) {
                        chain = ts.Debug.assertDefined(getSymbolChain(symbol, meaning, /*endOfChain*/ true));
                        ts.Debug.assert(chain && chain.length > 0);
                    }
                    else {
                        chain = [symbol];
                    }
                    return chain;
                    /** @param endOfChain Set to false for recursive calls; non-recursive calls should always output something. */
                    function getSymbolChain(symbol, meaning, endOfChain) {
                        var accessibleSymbolChain = getAccessibleSymbolChain(symbol, context.enclosingDeclaration, meaning, !!(context.flags & 128 /* UseOnlyExternalAliasing */));
                        var parentSpecifiers;
                        if (!accessibleSymbolChain ||
                            needsQualification(accessibleSymbolChain[0], context.enclosingDeclaration, accessibleSymbolChain.length === 1 ? meaning : getQualifiedLeftMeaning(meaning))) {
                            // Go up and add our parent.
                            var parents_1 = getContainersOfSymbol(accessibleSymbolChain ? accessibleSymbolChain[0] : symbol, context.enclosingDeclaration);
                            if (ts.length(parents_1)) {
                                parentSpecifiers = parents_1.map(function (symbol) {
                                    return ts.some(symbol.declarations, hasNonGlobalAugmentationExternalModuleSymbol)
                                        ? getSpecifierForModuleSymbol(symbol, context)
                                        : undefined;
                                });
                                var indices = parents_1.map(function (_, i) { return i; });
                                indices.sort(sortByBestName);
                                var sortedParents = indices.map(function (i) { return parents_1[i]; });
                                for (var _i = 0, sortedParents_1 = sortedParents; _i < sortedParents_1.length; _i++) {
                                    var parent = sortedParents_1[_i];
                                    var parentChain = getSymbolChain(parent, getQualifiedLeftMeaning(meaning), /*endOfChain*/ false);
                                    if (parentChain) {
                                        if (parent.exports && parent.exports.get("export=" /* ExportEquals */) &&
                                            getSymbolIfSameReference(parent.exports.get("export=" /* ExportEquals */), symbol)) {
                                            // parentChain root _is_ symbol - symbol is a module export=, so it kinda looks like it's own parent
                                            // No need to lookup an alias for the symbol in itself
                                            accessibleSymbolChain = parentChain;
                                            break;
                                        }
                                        accessibleSymbolChain = parentChain.concat(accessibleSymbolChain || [getAliasForSymbolInContainer(parent, symbol) || symbol]);
                                        break;
                                    }
                                }
                            }
                        }
                        if (accessibleSymbolChain) {
                            return accessibleSymbolChain;
                        }
                        if (
                        // If this is the last part of outputting the symbol, always output. The cases apply only to parent symbols.
                        endOfChain ||
                            // If a parent symbol is an anonymous type, don't write it.
                            !(symbol.flags & (2048 /* TypeLiteral */ | 4096 /* ObjectLiteral */))) {
                            // If a parent symbol is an external module, don't write it. (We prefer just `x` vs `"foo/bar".x`.)
                            if (!endOfChain && !yieldModuleSymbol && !!ts.forEach(symbol.declarations, hasNonGlobalAugmentationExternalModuleSymbol)) {
                                return;
                            }
                            return [symbol];
                        }
                        function sortByBestName(a, b) {
                            var specifierA = parentSpecifiers[a];
                            var specifierB = parentSpecifiers[b];
                            if (specifierA && specifierB) {
                                var isBRelative = ts.pathIsRelative(specifierB);
                                if (ts.pathIsRelative(specifierA) === isBRelative) {
                                    // Both relative or both non-relative, sort by number of parts
                                    return ts.moduleSpecifiers.countPathComponents(specifierA) - ts.moduleSpecifiers.countPathComponents(specifierB);
                                }
                                if (isBRelative) {
                                    // A is non-relative, B is relative: prefer A
                                    return -1;
                                }
                                // A is relative, B is non-relative: prefer B
                                return 1;
                            }
                            return 0;
                        }
                    }
                }
                function typeParametersToTypeParameterDeclarations(symbol, context) {
                    var typeParameterNodes;
                    var targetSymbol = getTargetSymbol(symbol);
                    if (targetSymbol.flags & (32 /* Class */ | 64 /* Interface */ | 524288 /* TypeAlias */)) {
                        typeParameterNodes = ts.createNodeArray(ts.map(getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(symbol), function (tp) { return typeParameterToDeclaration(tp, context); }));
                    }
                    return typeParameterNodes;
                }
                function lookupTypeParameterNodes(chain, index, context) {
                    ts.Debug.assert(chain && 0 <= index && index < chain.length);
                    var symbol = chain[index];
                    var symbolId = "" + getSymbolId(symbol);
                    if (context.typeParameterSymbolList && context.typeParameterSymbolList.get(symbolId)) {
                        return undefined;
                    }
                    (context.typeParameterSymbolList || (context.typeParameterSymbolList = ts.createMap())).set(symbolId, true);
                    var typeParameterNodes;
                    if (context.flags & 512 /* WriteTypeParametersInQualifiedName */ && index < (chain.length - 1)) {
                        var parentSymbol = symbol;
                        var nextSymbol = chain[index + 1];
                        if (ts.getCheckFlags(nextSymbol) & 1 /* Instantiated */) {
                            var params = getTypeParametersOfClassOrInterface(parentSymbol.flags & 2097152 /* Alias */ ? resolveAlias(parentSymbol) : parentSymbol);
                            typeParameterNodes = mapToTypeNodes(ts.map(params, nextSymbol.mapper), context);
                        }
                        else {
                            typeParameterNodes = typeParametersToTypeParameterDeclarations(symbol, context);
                        }
                    }
                    return typeParameterNodes;
                }
                /**
                 * Given A[B][C][D], finds A[B]
                 */
                function getTopmostIndexedAccessType(top) {
                    if (ts.isIndexedAccessTypeNode(top.objectType)) {
                        return getTopmostIndexedAccessType(top.objectType);
                    }
                    return top;
                }
                function getSpecifierForModuleSymbol(symbol, context) {
                    var file = ts.getDeclarationOfKind(symbol, 290 /* SourceFile */);
                    if (file && file.moduleName !== undefined) {
                        // Use the amd name if it is available
                        return file.moduleName;
                    }
                    if (!file) {
                        if (context.tracker.trackReferencedAmbientModule) {
                            var ambientDecls = ts.filter(symbol.declarations, ts.isAmbientModule);
                            if (ts.length(ambientDecls)) {
                                for (var _i = 0, ambientDecls_1 = ambientDecls; _i < ambientDecls_1.length; _i++) {
                                    var decl = ambientDecls_1[_i];
                                    context.tracker.trackReferencedAmbientModule(decl, symbol);
                                }
                            }
                        }
                        if (ambientModuleSymbolRegex.test(symbol.escapedName)) {
                            return symbol.escapedName.substring(1, symbol.escapedName.length - 1);
                        }
                    }
                    if (!context.enclosingDeclaration || !context.tracker.moduleResolverHost) {
                        // If there's no context declaration, we can't lookup a non-ambient specifier, so we just use the symbol name
                        if (ambientModuleSymbolRegex.test(symbol.escapedName)) {
                            return symbol.escapedName.substring(1, symbol.escapedName.length - 1);
                        }
                        return ts.getSourceFileOfNode(ts.getNonAugmentationDeclaration(symbol)).fileName; // A resolver may not be provided for baselines and errors - in those cases we use the fileName in full
                    }
                    var contextFile = ts.getSourceFileOfNode(ts.getOriginalNode(context.enclosingDeclaration));
                    var links = getSymbolLinks(symbol);
                    var specifier = links.specifierCache && links.specifierCache.get(contextFile.path);
                    if (!specifier) {
                        var isBundle_1 = (compilerOptions.out || compilerOptions.outFile);
                        // For declaration bundles, we need to generate absolute paths relative to the common source dir for imports,
                        // just like how the declaration emitter does for the ambient module declarations - we can easily accomplish this
                        // using the `baseUrl` compiler option (which we would otherwise never use in declaration emit) and a non-relative
                        // specifier preference
                        var moduleResolverHost = context.tracker.moduleResolverHost;
                        var specifierCompilerOptions = isBundle_1 ? __assign(__assign({}, compilerOptions), { baseUrl: moduleResolverHost.getCommonSourceDirectory() }) : compilerOptions;
                        specifier = ts.first(ts.moduleSpecifiers.getModuleSpecifiers(symbol, specifierCompilerOptions, contextFile, moduleResolverHost, host.getSourceFiles(), { importModuleSpecifierPreference: isBundle_1 ? "non-relative" : "relative" }, host.redirectTargetsMap));
                        links.specifierCache = links.specifierCache || ts.createMap();
                        links.specifierCache.set(contextFile.path, specifier);
                    }
                    return specifier;
                }
                function symbolToTypeNode(symbol, context, meaning, overrideTypeArguments) {
                    var chain = lookupSymbolChain(symbol, context, meaning, !(context.flags & 16384 /* UseAliasDefinedOutsideCurrentScope */)); // If we're using aliases outside the current scope, dont bother with the module
                    var isTypeOf = meaning === 111551 /* Value */;
                    if (ts.some(chain[0].declarations, hasNonGlobalAugmentationExternalModuleSymbol)) {
                        // module is root, must use `ImportTypeNode`
                        var nonRootParts = chain.length > 1 ? createAccessFromSymbolChain(chain, chain.length - 1, 1) : undefined;
                        var typeParameterNodes = overrideTypeArguments || lookupTypeParameterNodes(chain, 0, context);
                        var specifier = getSpecifierForModuleSymbol(chain[0], context);
                        if (!(context.flags & 67108864 /* AllowNodeModulesRelativePaths */) && ts.getEmitModuleResolutionKind(compilerOptions) === ts.ModuleResolutionKind.NodeJs && specifier.indexOf("/node_modules/") >= 0) {
                            // If ultimately we can only name the symbol with a reference that dives into a `node_modules` folder, we should error
                            // since declaration files with these kinds of references are liable to fail when published :(
                            context.encounteredError = true;
                            if (context.tracker.reportLikelyUnsafeImportRequiredError) {
                                context.tracker.reportLikelyUnsafeImportRequiredError(specifier);
                            }
                        }
                        var lit = ts.createLiteralTypeNode(ts.createLiteral(specifier));
                        if (context.tracker.trackExternalModuleSymbolOfImportTypeNode)
                            context.tracker.trackExternalModuleSymbolOfImportTypeNode(chain[0]);
                        context.approximateLength += specifier.length + 10; // specifier + import("")
                        if (!nonRootParts || ts.isEntityName(nonRootParts)) {
                            if (nonRootParts) {
                                var lastId = ts.isIdentifier(nonRootParts) ? nonRootParts : nonRootParts.right;
                                lastId.typeArguments = undefined;
                            }
                            return ts.createImportTypeNode(lit, nonRootParts, typeParameterNodes, isTypeOf);
                        }
                        else {
                            var splitNode = getTopmostIndexedAccessType(nonRootParts);
                            var qualifier = splitNode.objectType.typeName;
                            return ts.createIndexedAccessTypeNode(ts.createImportTypeNode(lit, qualifier, typeParameterNodes, isTypeOf), splitNode.indexType);
                        }
                    }
                    var entityName = createAccessFromSymbolChain(chain, chain.length - 1, 0);
                    if (ts.isIndexedAccessTypeNode(entityName)) {
                        return entityName; // Indexed accesses can never be `typeof`
                    }
                    if (isTypeOf) {
                        return ts.createTypeQueryNode(entityName);
                    }
                    else {
                        var lastId = ts.isIdentifier(entityName) ? entityName : entityName.right;
                        var lastTypeArgs = lastId.typeArguments;
                        lastId.typeArguments = undefined;
                        return ts.createTypeReferenceNode(entityName, lastTypeArgs);
                    }
                    function createAccessFromSymbolChain(chain, index, stopper) {
                        var typeParameterNodes = index === (chain.length - 1) ? overrideTypeArguments : lookupTypeParameterNodes(chain, index, context);
                        var symbol = chain[index];
                        var parent = chain[index - 1];
                        var symbolName;
                        if (index === 0) {
                            context.flags |= 16777216 /* InInitialEntityName */;
                            symbolName = getNameOfSymbolAsWritten(symbol, context);
                            context.approximateLength += (symbolName ? symbolName.length : 0) + 1;
                            context.flags ^= 16777216 /* InInitialEntityName */;
                        }
                        else {
                            if (parent && getExportsOfSymbol(parent)) {
                                var exports_1 = getExportsOfSymbol(parent);
                                ts.forEachEntry(exports_1, function (ex, name) {
                                    if (getSymbolIfSameReference(ex, symbol) && !isLateBoundName(name) && name !== "export=" /* ExportEquals */) {
                                        symbolName = ts.unescapeLeadingUnderscores(name);
                                        return true;
                                    }
                                });
                            }
                        }
                        if (!symbolName) {
                            symbolName = getNameOfSymbolAsWritten(symbol, context);
                        }
                        context.approximateLength += symbolName.length + 1;
                        if (!(context.flags & 16 /* ForbidIndexedAccessSymbolReferences */) && parent &&
                            getMembersOfSymbol(parent) && getMembersOfSymbol(parent).get(symbol.escapedName) &&
                            getSymbolIfSameReference(getMembersOfSymbol(parent).get(symbol.escapedName), symbol)) {
                            // Should use an indexed access
                            var LHS = createAccessFromSymbolChain(chain, index - 1, stopper);
                            if (ts.isIndexedAccessTypeNode(LHS)) {
                                return ts.createIndexedAccessTypeNode(LHS, ts.createLiteralTypeNode(ts.createLiteral(symbolName)));
                            }
                            else {
                                return ts.createIndexedAccessTypeNode(ts.createTypeReferenceNode(LHS, typeParameterNodes), ts.createLiteralTypeNode(ts.createLiteral(symbolName)));
                            }
                        }
                        var identifier = ts.setEmitFlags(ts.createIdentifier(symbolName, typeParameterNodes), 16777216 /* NoAsciiEscaping */);
                        identifier.symbol = symbol;
                        if (index > stopper) {
                            var LHS = createAccessFromSymbolChain(chain, index - 1, stopper);
                            if (!ts.isEntityName(LHS)) {
                                return ts.Debug.fail("Impossible construct - an export of an indexed access cannot be reachable");
                            }
                            return ts.createQualifiedName(LHS, identifier);
                        }
                        return identifier;
                    }
                }
                function typeParameterShadowsNameInScope(escapedName, context) {
                    return !!resolveName(context.enclosingDeclaration, escapedName, 788968 /* Type */, /*nameNotFoundArg*/ undefined, escapedName, /*isUse*/ false);
                }
                function typeParameterToName(type, context) {
                    if (context.flags & 4 /* GenerateNamesForShadowedTypeParams */ && context.typeParameterNames) {
                        var cached = context.typeParameterNames.get("" + getTypeId(type));
                        if (cached) {
                            return cached;
                        }
                    }
                    var result = symbolToName(type.symbol, context, 788968 /* Type */, /*expectsIdentifier*/ true);
                    if (!(result.kind & 75 /* Identifier */)) {
                        return ts.createIdentifier("(Missing type parameter)");
                    }
                    if (context.flags & 4 /* GenerateNamesForShadowedTypeParams */) {
                        var rawtext = result.escapedText;
                        var i = 0;
                        var text = rawtext;
                        while ((context.typeParameterNamesByText && context.typeParameterNamesByText.get(text)) || typeParameterShadowsNameInScope(text, context)) {
                            i++;
                            text = rawtext + "_" + i;
                        }
                        if (text !== rawtext) {
                            result = ts.createIdentifier(text, result.typeArguments);
                        }
                        (context.typeParameterNames || (context.typeParameterNames = ts.createMap())).set("" + getTypeId(type), result);
                        (context.typeParameterNamesByText || (context.typeParameterNamesByText = ts.createMap())).set(result.escapedText, true);
                    }
                    return result;
                }
                function symbolToName(symbol, context, meaning, expectsIdentifier) {
                    var chain = lookupSymbolChain(symbol, context, meaning);
                    if (expectsIdentifier && chain.length !== 1
                        && !context.encounteredError
                        && !(context.flags & 65536 /* AllowQualifedNameInPlaceOfIdentifier */)) {
                        context.encounteredError = true;
                    }
                    return createEntityNameFromSymbolChain(chain, chain.length - 1);
                    function createEntityNameFromSymbolChain(chain, index) {
                        var typeParameterNodes = lookupTypeParameterNodes(chain, index, context);
                        var symbol = chain[index];
                        if (index === 0) {
                            context.flags |= 16777216 /* InInitialEntityName */;
                        }
                        var symbolName = getNameOfSymbolAsWritten(symbol, context);
                        if (index === 0) {
                            context.flags ^= 16777216 /* InInitialEntityName */;
                        }
                        var identifier = ts.setEmitFlags(ts.createIdentifier(symbolName, typeParameterNodes), 16777216 /* NoAsciiEscaping */);
                        identifier.symbol = symbol;
                        return index > 0 ? ts.createQualifiedName(createEntityNameFromSymbolChain(chain, index - 1), identifier) : identifier;
                    }
                }
                function symbolToExpression(symbol, context, meaning) {
                    var chain = lookupSymbolChain(symbol, context, meaning);
                    return createExpressionFromSymbolChain(chain, chain.length - 1);
                    function createExpressionFromSymbolChain(chain, index) {
                        var typeParameterNodes = lookupTypeParameterNodes(chain, index, context);
                        var symbol = chain[index];
                        if (index === 0) {
                            context.flags |= 16777216 /* InInitialEntityName */;
                        }
                        var symbolName = getNameOfSymbolAsWritten(symbol, context);
                        if (index === 0) {
                            context.flags ^= 16777216 /* InInitialEntityName */;
                        }
                        var firstChar = symbolName.charCodeAt(0);
                        if (ts.isSingleOrDoubleQuote(firstChar) && ts.some(symbol.declarations, hasNonGlobalAugmentationExternalModuleSymbol)) {
                            return ts.createLiteral(getSpecifierForModuleSymbol(symbol, context));
                        }
                        var canUsePropertyAccess = firstChar === 35 /* hash */ ?
                            symbolName.length > 1 && ts.isIdentifierStart(symbolName.charCodeAt(1), languageVersion) :
                            ts.isIdentifierStart(firstChar, languageVersion);
                        if (index === 0 || canUsePropertyAccess) {
                            var identifier = ts.setEmitFlags(ts.createIdentifier(symbolName, typeParameterNodes), 16777216 /* NoAsciiEscaping */);
                            identifier.symbol = symbol;
                            return index > 0 ? ts.createPropertyAccess(createExpressionFromSymbolChain(chain, index - 1), identifier) : identifier;
                        }
                        else {
                            if (firstChar === 91 /* openBracket */) {
                                symbolName = symbolName.substring(1, symbolName.length - 1);
                                firstChar = symbolName.charCodeAt(0);
                            }
                            var expression = void 0;
                            if (ts.isSingleOrDoubleQuote(firstChar)) {
                                expression = ts.createLiteral(symbolName.substring(1, symbolName.length - 1).replace(/\\./g, function (s) { return s.substring(1); }));
                                expression.singleQuote = firstChar === 39 /* singleQuote */;
                            }
                            else if (("" + +symbolName) === symbolName) {
                                expression = ts.createLiteral(+symbolName);
                            }
                            if (!expression) {
                                expression = ts.setEmitFlags(ts.createIdentifier(symbolName, typeParameterNodes), 16777216 /* NoAsciiEscaping */);
                                expression.symbol = symbol;
                            }
                            return ts.createElementAccess(createExpressionFromSymbolChain(chain, index - 1), expression);
                        }
                    }
                }
                function isSingleQuotedStringNamed(d) {
                    var name = ts.getNameOfDeclaration(d);
                    if (name && ts.isStringLiteral(name) && (name.singleQuote ||
                        (!ts.nodeIsSynthesized(name) && ts.startsWith(ts.getTextOfNode(name, /*includeTrivia*/ false), "'")))) {
                        return true;
                    }
                    return false;
                }
                function getPropertyNameNodeForSymbol(symbol, context) {
                    var singleQuote = !!ts.length(symbol.declarations) && ts.every(symbol.declarations, isSingleQuotedStringNamed);
                    var fromNameType = getPropertyNameNodeForSymbolFromNameType(symbol, context, singleQuote);
                    if (fromNameType) {
                        return fromNameType;
                    }
                    if (ts.isKnownSymbol(symbol)) {
                        return ts.createComputedPropertyName(ts.createPropertyAccess(ts.createIdentifier("Symbol"), symbol.escapedName.substr(3)));
                    }
                    var rawName = ts.unescapeLeadingUnderscores(symbol.escapedName);
                    return createPropertyNameNodeForIdentifierOrLiteral(rawName, singleQuote);
                }
                // See getNameForSymbolFromNameType for a stringy equivalent
                function getPropertyNameNodeForSymbolFromNameType(symbol, context, singleQuote) {
                    var nameType = getSymbolLinks(symbol).nameType;
                    if (nameType) {
                        if (nameType.flags & 384 /* StringOrNumberLiteral */) {
                            var name = "" + nameType.value;
                            if (!ts.isIdentifierText(name, compilerOptions.target) && !isNumericLiteralName(name)) {
                                return ts.createLiteral(name, !!singleQuote);
                            }
                            if (isNumericLiteralName(name) && ts.startsWith(name, "-")) {
                                return ts.createComputedPropertyName(ts.createLiteral(+name));
                            }
                            return createPropertyNameNodeForIdentifierOrLiteral(name);
                        }
                        if (nameType.flags & 8192 /* UniqueESSymbol */) {
                            return ts.createComputedPropertyName(symbolToExpression(nameType.symbol, context, 111551 /* Value */));
                        }
                    }
                }
                function createPropertyNameNodeForIdentifierOrLiteral(name, singleQuote) {
                    return ts.isIdentifierText(name, compilerOptions.target) ? ts.createIdentifier(name) : ts.createLiteral(isNumericLiteralName(name) ? +name : name, !!singleQuote);
                }
                function cloneNodeBuilderContext(context) {
                    var initial = __assign({}, context);
                    // Make type parameters created within this context not consume the name outside this context
                    // The symbol serializer ends up creating many sibling scopes that all need "separate" contexts when
                    // it comes to naming things - within a normal `typeToTypeNode` call, the node builder only ever descends
                    // through the type tree, so the only cases where we could have used distinct sibling scopes was when there
                    // were multiple generic overloads with similar generated type parameter names
                    // The effect:
                    // When we write out
                    // export const x: <T>(x: T) => T
                    // export const y: <T>(x: T) => T
                    // we write it out like that, rather than as
                    // export const x: <T>(x: T) => T
                    // export const y: <T_1>(x: T_1) => T_1
                    if (initial.typeParameterNames) {
                        initial.typeParameterNames = ts.cloneMap(initial.typeParameterNames);
                    }
                    if (initial.typeParameterNamesByText) {
                        initial.typeParameterNamesByText = ts.cloneMap(initial.typeParameterNamesByText);
                    }
                    if (initial.typeParameterSymbolList) {
                        initial.typeParameterSymbolList = ts.cloneMap(initial.typeParameterSymbolList);
                    }
                    return initial;
                }
                function symbolTableToDeclarationStatements(symbolTable, context, bundled) {
                    var serializePropertySymbolForClass = makeSerializePropertySymbol(ts.createProperty, 161 /* MethodDeclaration */, /*useAcessors*/ true);
                    var serializePropertySymbolForInterfaceWorker = makeSerializePropertySymbol(function (_decorators, mods, name, question, type, initializer) { return ts.createPropertySignature(mods, name, question, type, initializer); }, 160 /* MethodSignature */, /*useAcessors*/ false);
                    // TODO: Use `setOriginalNode` on original declaration names where possible so these declarations see some kind of
                    // declaration mapping
                    // We save the enclosing declaration off here so it's not adjusted by well-meaning declaration
                    // emit codepaths which want to apply more specific contexts (so we can still refer to the root real declaration
                    // we're trying to emit from later on)
                    var enclosingDeclaration = context.enclosingDeclaration;
                    var results = [];
                    var visitedSymbols = ts.createMap();
                    var deferredPrivates;
                    var oldcontext = context;
                    context = __assign(__assign({}, oldcontext), { usedSymbolNames: ts.createMap(), remappedSymbolNames: ts.createMap(), tracker: __assign(__assign({}, oldcontext.tracker), { trackSymbol: function (sym, decl, meaning) {
                                var accessibleResult = isSymbolAccessible(sym, decl, meaning, /*computeALiases*/ false);
                                if (accessibleResult.accessibility === 0 /* Accessible */) {
                                    // Lookup the root symbol of the chain of refs we'll use to access it and serialize it
                                    var chain = lookupSymbolChainWorker(sym, context, meaning);
                                    if (!(sym.flags & 4 /* Property */)) {
                                        includePrivateSymbol(chain[0]);
                                    }
                                }
                                else if (oldcontext.tracker && oldcontext.tracker.trackSymbol) {
                                    oldcontext.tracker.trackSymbol(sym, decl, meaning);
                                }
                            } }) });
                    if (oldcontext.usedSymbolNames) {
                        oldcontext.usedSymbolNames.forEach(function (_, name) {
                            context.usedSymbolNames.set(name, true);
                        });
                    }
                    ts.forEachEntry(symbolTable, function (symbol, name) {
                        var baseName = ts.unescapeLeadingUnderscores(name);
                        void getInternalSymbolName(symbol, baseName); // Called to cache values into `usedSymbolNames` and `remappedSymbolNames`
                    });
                    var addingDeclare = !bundled;
                    var exportEquals = symbolTable.get("export=" /* ExportEquals */);
                    if (exportEquals && symbolTable.size > 1 && exportEquals.flags & 2097152 /* Alias */) {
                        symbolTable = ts.createSymbolTable();
                        // Remove extraneous elements from root symbol table (they'll be mixed back in when the target of the `export=` is looked up)
                        symbolTable.set("export=" /* ExportEquals */, exportEquals);
                    }
                    visitSymbolTable(symbolTable);
                    return mergeRedundantStatements(results);
                    function isIdentifierAndNotUndefined(node) {
                        return !!node && node.kind === 75 /* Identifier */;
                    }
                    function getNamesOfDeclaration(statement) {
                        if (ts.isVariableStatement(statement)) {
                            return ts.filter(ts.map(statement.declarationList.declarations, ts.getNameOfDeclaration), isIdentifierAndNotUndefined);
                        }
                        return ts.filter([ts.getNameOfDeclaration(statement)], isIdentifierAndNotUndefined);
                    }
                    function flattenExportAssignedNamespace(statements) {
                        var exportAssignment = ts.find(statements, ts.isExportAssignment);
                        var ns = ts.find(statements, ts.isModuleDeclaration);
                        if (ns && exportAssignment && exportAssignment.isExportEquals &&
                            ts.isIdentifier(exportAssignment.expression) && ts.isIdentifier(ns.name) && ts.idText(ns.name) === ts.idText(exportAssignment.expression) &&
                            ns.body && ts.isModuleBlock(ns.body)) {
                            // Pass 0: Correct situations where a module has both an `export = ns` and multiple top-level exports by stripping the export modifiers from
                            //  the top-level exports and exporting them in the targeted ns, as can occur when a js file has both typedefs and `module.export` assignments
                            var excessExports = ts.filter(statements, function (s) { return !!(ts.getModifierFlags(s) & 1 /* Export */); });
                            if (ts.length(excessExports)) {
                                ns.body.statements = ts.createNodeArray(__spreadArrays(ns.body.statements, [ts.createExportDeclaration(
                                    /*decorators*/ undefined, 
                                    /*modifiers*/ undefined, ts.createNamedExports(ts.map(ts.flatMap(excessExports, function (e) { return getNamesOfDeclaration(e); }), function (id) { return ts.createExportSpecifier(/*alias*/ undefined, id); })), 
                                    /*moduleSpecifier*/ undefined)]));
                            }
                            // Pass 1: Flatten `export namespace _exports {} export = _exports;` so long as the `export=` only points at a single namespace declaration
                            if (!ts.find(statements, function (s) { return s !== ns && ts.nodeHasName(s, ns.name); })) {
                                results = [];
                                ts.forEach(ns.body.statements, function (s) {
                                    addResult(s, 0 /* None */); // Recalculates the ambient (and export, if applicable from above) flag
                                });
                                statements = __spreadArrays(ts.filter(statements, function (s) { return s !== ns && s !== exportAssignment; }), results);
                            }
                        }
                        return statements;
                    }
                    function mergeExportDeclarations(statements) {
                        // Pass 2: Combine all `export {}` declarations
                        var exports = ts.filter(statements, function (d) { return ts.isExportDeclaration(d) && !d.moduleSpecifier && !!d.exportClause && ts.isNamedExports(d.exportClause); });
                        if (ts.length(exports) > 1) {
                            var nonExports = ts.filter(statements, function (d) { return !ts.isExportDeclaration(d) || !!d.moduleSpecifier || !d.exportClause; });
                            statements = __spreadArrays(nonExports, [ts.createExportDeclaration(
                                /*decorators*/ undefined, 
                                /*modifiers*/ undefined, ts.createNamedExports(ts.flatMap(exports, function (e) { return ts.cast(e.exportClause, ts.isNamedExports).elements; })), 
                                /*moduleSpecifier*/ undefined)]);
                        }
                        // Pass 2b: Also combine all `export {} from "..."` declarations as needed
                        var reexports = ts.filter(statements, function (d) { return ts.isExportDeclaration(d) && !!d.moduleSpecifier && !!d.exportClause && ts.isNamedExports(d.exportClause); });
                        if (ts.length(reexports) > 1) {
                            var groups = ts.group(reexports, function (decl) { return ts.isStringLiteral(decl.moduleSpecifier) ? ">" + decl.moduleSpecifier.text : ">"; });
                            if (groups.length !== reexports.length) {
                                var _loop_7 = function (group_1) {
                                    if (group_1.length > 1) {
                                        // remove group members from statements and then merge group members and add back to statements
                                        statements = __spreadArrays(ts.filter(statements, function (s) { return group_1.indexOf(s) === -1; }), [
                                            ts.createExportDeclaration(
                                            /*decorators*/ undefined, 
                                            /*modifiers*/ undefined, ts.createNamedExports(ts.flatMap(group_1, function (e) { return ts.cast(e.exportClause, ts.isNamedExports).elements; })), group_1[0].moduleSpecifier)
                                        ]);
                                    }
                                };
                                for (var _i = 0, groups_1 = groups; _i < groups_1.length; _i++) {
                                    var group_1 = groups_1[_i];
                                    _loop_7(group_1);
                                }
                            }
                        }
                        return statements;
                    }
                    function inlineExportModifiers(statements) {
                        // Pass 3: Move all `export {}`'s to `export` modifiers where possible
                        var exportDecl = ts.find(statements, function (d) { return ts.isExportDeclaration(d) && !d.moduleSpecifier && !!d.exportClause; });
                        if (exportDecl && exportDecl.exportClause && ts.isNamedExports(exportDecl.exportClause)) {
                            var replacements = ts.mapDefined(exportDecl.exportClause.elements, function (e) {
                                if (!e.propertyName) {
                                    // export {name} - look thru `statements` for `name`, and if all results can take an `export` modifier, do so and filter it
                                    var associated = ts.filter(statements, function (s) { return ts.nodeHasName(s, e.name); });
                                    if (ts.length(associated) && ts.every(associated, canHaveExportModifier)) {
                                        ts.forEach(associated, addExportModifier);
                                        return undefined;
                                    }
                                }
                                return e;
                            });
                            if (!ts.length(replacements)) {
                                // all clauses removed, filter the export declaration
                                statements = ts.filter(statements, function (s) { return s !== exportDecl; });
                            }
                            else {
                                // some items filtered, others not - update the export declaration
                                // (mutating because why not, we're building a whole new tree here anyway)
                                exportDecl.exportClause.elements = ts.createNodeArray(replacements);
                            }
                        }
                        return statements;
                    }
                    function mergeRedundantStatements(statements) {
                        statements = flattenExportAssignedNamespace(statements);
                        statements = mergeExportDeclarations(statements);
                        statements = inlineExportModifiers(statements);
                        // Not a cleanup, but as a final step: If there is a mix of `export` and non-`export` declarations, but no `export =` or `export {}` add a `export {};` so
                        // declaration privacy is respected.
                        if (enclosingDeclaration &&
                            ((ts.isSourceFile(enclosingDeclaration) && ts.isExternalOrCommonJsModule(enclosingDeclaration)) || ts.isModuleDeclaration(enclosingDeclaration)) &&
                            (!ts.some(statements, ts.isExternalModuleIndicator) || (!ts.hasScopeMarker(statements) && ts.some(statements, ts.needsScopeMarker)))) {
                            statements.push(ts.createEmptyExports());
                        }
                        return statements;
                    }
                    function canHaveExportModifier(node) {
                        return ts.isEnumDeclaration(node) ||
                            ts.isVariableStatement(node) ||
                            ts.isFunctionDeclaration(node) ||
                            ts.isClassDeclaration(node) ||
                            (ts.isModuleDeclaration(node) && !ts.isExternalModuleAugmentation(node) && !ts.isGlobalScopeAugmentation(node)) ||
                            ts.isInterfaceDeclaration(node) ||
                            isTypeDeclaration(node);
                    }
                    function addExportModifier(statement) {
                        var flags = (ts.getModifierFlags(statement) | 1 /* Export */) & ~2 /* Ambient */;
                        statement.modifiers = ts.createNodeArray(ts.createModifiersFromModifierFlags(flags));
                        statement.modifierFlagsCache = 0;
                    }
                    function visitSymbolTable(symbolTable, suppressNewPrivateContext, propertyAsAlias) {
                        var oldDeferredPrivates = deferredPrivates;
                        if (!suppressNewPrivateContext) {
                            deferredPrivates = ts.createMap();
                        }
                        symbolTable.forEach(function (symbol) {
                            serializeSymbol(symbol, /*isPrivate*/ false, !!propertyAsAlias);
                        });
                        if (!suppressNewPrivateContext) {
                            // deferredPrivates will be filled up by visiting the symbol table
                            // And will continue to iterate as elements are added while visited `deferredPrivates`
                            // (As that's how a map iterator is defined to work)
                            deferredPrivates.forEach(function (symbol) {
                                serializeSymbol(symbol, /*isPrivate*/ true, !!propertyAsAlias);
                            });
                        }
                        deferredPrivates = oldDeferredPrivates;
                    }
                    function serializeSymbol(symbol, isPrivate, propertyAsAlias) {
                        // cache visited list based on merged symbol, since we want to use the unmerged top-level symbol, but
                        // still skip reserializing it if we encounter the merged product later on
                        var visitedSym = getMergedSymbol(symbol);
                        if (visitedSymbols.has("" + getSymbolId(visitedSym))) {
                            return; // Already printed
                        }
                        visitedSymbols.set("" + getSymbolId(visitedSym), true);
                        // Only actually serialize symbols within the correct enclosing declaration, otherwise do nothing with the out-of-context symbol
                        var skipMembershipCheck = !isPrivate; // We only call this on exported symbols when we know they're in the correct scope
                        if (skipMembershipCheck || (!!ts.length(symbol.declarations) && ts.some(symbol.declarations, function (d) { return !!ts.findAncestor(d, function (n) { return n === enclosingDeclaration; }); }))) {
                            var oldContext = context;
                            context = cloneNodeBuilderContext(context);
                            var result = serializeSymbolWorker(symbol, isPrivate, propertyAsAlias);
                            context = oldContext;
                            return result;
                        }
                    }
                    // Synthesize declarations for a symbol - might be an Interface, a Class, a Namespace, a Type, a Variable (const, let, or var), an Alias
                    // or a merge of some number of those.
                    // An interesting challenge is ensuring that when classes merge with namespaces and interfaces, is keeping
                    // each symbol in only one of the representations
                    // Also, synthesizing a default export of some kind
                    // If it's an alias: emit `export default ref`
                    // If it's a property: emit `export default _default` with a `_default` prop
                    // If it's a class/interface/function: emit a class/interface/function with a `default` modifier
                    // These forms can merge, eg (`export default 12; export default interface A {}`)
                    function serializeSymbolWorker(symbol, isPrivate, propertyAsAlias) {
                        var symbolName = ts.unescapeLeadingUnderscores(symbol.escapedName);
                        var isDefault = symbol.escapedName === "default" /* Default */;
                        if (ts.isStringANonContextualKeyword(symbolName) && !isDefault) {
                            // Oh no. We cannot use this symbol's name as it's name... It's likely some jsdoc had an invalid name like `export` or `default` :(
                            context.encounteredError = true;
                            // TODO: Issue error via symbol tracker?
                            return; // If we need to emit a private with a keyword name, we're done for, since something else will try to refer to it by that name
                        }
                        var needsPostExportDefault = isDefault && !!(symbol.flags & -113 /* ExportDoesNotSupportDefaultModifier */
                            || (symbol.flags & 16 /* Function */ && ts.length(getPropertiesOfType(getTypeOfSymbol(symbol))))) && !(symbol.flags & 2097152 /* Alias */); // An alias symbol should preclude needing to make an alias ourselves
                        if (needsPostExportDefault) {
                            isPrivate = true;
                        }
                        var modifierFlags = (!isPrivate ? 1 /* Export */ : 0) | (isDefault && !needsPostExportDefault ? 512 /* Default */ : 0);
                        var isConstMergedWithNS = symbol.flags & 1536 /* Module */ &&
                            symbol.flags & (2 /* BlockScopedVariable */ | 1 /* FunctionScopedVariable */ | 4 /* Property */) &&
                            symbol.escapedName !== "export=" /* ExportEquals */;
                        var isConstMergedWithNSPrintableAsSignatureMerge = isConstMergedWithNS && isTypeRepresentableAsFunctionNamespaceMerge(getTypeOfSymbol(symbol), symbol);
                        if (symbol.flags & (16 /* Function */ | 8192 /* Method */) || isConstMergedWithNSPrintableAsSignatureMerge) {
                            serializeAsFunctionNamespaceMerge(getTypeOfSymbol(symbol), symbol, getInternalSymbolName(symbol, symbolName), modifierFlags);
                        }
                        if (symbol.flags & 524288 /* TypeAlias */) {
                            serializeTypeAlias(symbol, symbolName, modifierFlags);
                        }
                        // Need to skip over export= symbols below - json source files get a single `Property` flagged
                        // symbol of name `export=` which needs to be handled like an alias. It's not great, but it is what it is.
                        if (symbol.flags & (2 /* BlockScopedVariable */ | 1 /* FunctionScopedVariable */ | 4 /* Property */)
                            && symbol.escapedName !== "export=" /* ExportEquals */
                            && !(symbol.flags & 4194304 /* Prototype */)
                            && !(symbol.flags & 32 /* Class */)
                            && !isConstMergedWithNSPrintableAsSignatureMerge) {
                            serializeVariableOrProperty(symbol, symbolName, isPrivate, needsPostExportDefault, propertyAsAlias, modifierFlags);
                        }
                        if (symbol.flags & 384 /* Enum */) {
                            serializeEnum(symbol, symbolName, modifierFlags);
                        }
                        if (symbol.flags & 32 /* Class */) {
                            if (symbol.flags & 4 /* Property */) {
                                // Looks like a `module.exports.Sub = class {}` - if we serialize `symbol` as a class, the result will have no members,
                                // since the classiness is actually from the target of the effective alias the symbol is. yes. A BlockScopedVariable|Class|Property
                                // _really_ acts like an Alias, and none of a BlockScopedVariable, Class, or Property. This is the travesty of JS binding today.
                                serializeAsAlias(symbol, getInternalSymbolName(symbol, symbolName), modifierFlags);
                            }
                            else {
                                serializeAsClass(symbol, getInternalSymbolName(symbol, symbolName), modifierFlags);
                            }
                        }
                        if ((symbol.flags & (512 /* ValueModule */ | 1024 /* NamespaceModule */) && (!isConstMergedWithNS || isTypeOnlyNamespace(symbol))) || isConstMergedWithNSPrintableAsSignatureMerge) {
                            serializeModule(symbol, symbolName, modifierFlags);
                        }
                        if (symbol.flags & 64 /* Interface */) {
                            serializeInterface(symbol, symbolName, modifierFlags);
                        }
                        if (symbol.flags & 2097152 /* Alias */) {
                            serializeAsAlias(symbol, getInternalSymbolName(symbol, symbolName), modifierFlags);
                        }
                        if (symbol.flags & 4 /* Property */ && symbol.escapedName === "export=" /* ExportEquals */) {
                            serializeMaybeAliasAssignment(symbol);
                        }
                        if (symbol.flags & 8388608 /* ExportStar */) {
                            // synthesize export * from "moduleReference"
                            // Straightforward - only one thing to do - make an export declaration
                            for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                                var node = _a[_i];
                                var resolvedModule = resolveExternalModuleName(node, node.moduleSpecifier);
                                if (!resolvedModule)
                                    continue;
                                addResult(ts.createExportDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, /*exportClause*/ undefined, ts.createLiteral(getSpecifierForModuleSymbol(resolvedModule, context))), 0 /* None */);
                            }
                        }
                        if (needsPostExportDefault) {
                            addResult(ts.createExportAssignment(/*decorators*/ undefined, /*modifiers*/ undefined, /*isExportAssignment*/ false, ts.createIdentifier(getInternalSymbolName(symbol, symbolName))), 0 /* None */);
                        }
                    }
                    function includePrivateSymbol(symbol) {
                        if (ts.some(symbol.declarations, ts.isParameterDeclaration))
                            return;
                        ts.Debug.assertDefined(deferredPrivates);
                        getUnusedName(ts.unescapeLeadingUnderscores(symbol.escapedName), symbol); // Call to cache unique name for symbol
                        deferredPrivates.set("" + getSymbolId(symbol), symbol);
                    }
                    function isExportingScope(enclosingDeclaration) {
                        return ((ts.isSourceFile(enclosingDeclaration) && (ts.isExternalOrCommonJsModule(enclosingDeclaration) || ts.isJsonSourceFile(enclosingDeclaration))) ||
                            (ts.isAmbientModule(enclosingDeclaration) && !ts.isGlobalScopeAugmentation(enclosingDeclaration)));
                    }
                    // Prepends a `declare` and/or `export` modifier if the context requires it, and then adds `node` to `result` and returns `node`
                    // Note: This _mutates_ `node` without using `updateNode` - the assumption being that all nodes should be manufactured fresh by the node builder
                    function addResult(node, additionalModifierFlags) {
                        var newModifierFlags = 0 /* None */;
                        if (additionalModifierFlags & 1 /* Export */ &&
                            enclosingDeclaration &&
                            isExportingScope(enclosingDeclaration) &&
                            canHaveExportModifier(node)) {
                            // Classes, namespaces, variables, functions, interfaces, and types should all be `export`ed in a module context if not private
                            newModifierFlags |= 1 /* Export */;
                        }
                        if (addingDeclare && !(newModifierFlags & 1 /* Export */) &&
                            (!enclosingDeclaration || !(enclosingDeclaration.flags & 8388608 /* Ambient */)) &&
                            (ts.isEnumDeclaration(node) || ts.isVariableStatement(node) || ts.isFunctionDeclaration(node) || ts.isClassDeclaration(node) || ts.isModuleDeclaration(node))) {
                            // Classes, namespaces, variables, enums, and functions all need `declare` modifiers to be valid in a declaration file top-level scope
                            newModifierFlags |= 2 /* Ambient */;
                        }
                        if ((additionalModifierFlags & 512 /* Default */) && (ts.isClassDeclaration(node) || ts.isInterfaceDeclaration(node) || ts.isFunctionDeclaration(node))) {
                            newModifierFlags |= 512 /* Default */;
                        }
                        if (newModifierFlags) {
                            node.modifiers = ts.createNodeArray(ts.createModifiersFromModifierFlags(newModifierFlags | ts.getModifierFlags(node)));
                            node.modifierFlagsCache = 0; // Reset computed flags cache
                        }
                        results.push(node);
                    }
                    function serializeTypeAlias(symbol, symbolName, modifierFlags) {
                        var aliasType = getDeclaredTypeOfTypeAlias(symbol);
                        var typeParams = getSymbolLinks(symbol).typeParameters;
                        var typeParamDecls = ts.map(typeParams, function (p) { return typeParameterToDeclaration(p, context); });
                        var jsdocAliasDecl = ts.find(symbol.declarations, ts.isJSDocTypeAlias);
                        var commentText = jsdocAliasDecl ? jsdocAliasDecl.comment || jsdocAliasDecl.parent.comment : undefined;
                        var oldFlags = context.flags;
                        context.flags |= 8388608 /* InTypeAlias */;
                        addResult(ts.setSyntheticLeadingComments(ts.createTypeAliasDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, getInternalSymbolName(symbol, symbolName), typeParamDecls, typeToTypeNodeHelper(aliasType, context)), !commentText ? [] : [{ kind: 3 /* MultiLineCommentTrivia */, text: "*\n * " + commentText.replace(/\n/g, "\n * ") + "\n ", pos: -1, end: -1, hasTrailingNewLine: true }]), modifierFlags);
                        context.flags = oldFlags;
                    }
                    function serializeInterface(symbol, symbolName, modifierFlags) {
                        var interfaceType = getDeclaredTypeOfClassOrInterface(symbol);
                        var localParams = getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(symbol);
                        var typeParamDecls = ts.map(localParams, function (p) { return typeParameterToDeclaration(p, context); });
                        var baseTypes = getBaseTypes(interfaceType);
                        var baseType = ts.length(baseTypes) ? getIntersectionType(baseTypes) : undefined;
                        var members = ts.flatMap(getPropertiesOfType(interfaceType), function (p) { return serializePropertySymbolForInterface(p, baseType); });
                        var callSignatures = serializeSignatures(0 /* Call */, interfaceType, baseType, 165 /* CallSignature */);
                        var constructSignatures = serializeSignatures(1 /* Construct */, interfaceType, baseType, 166 /* ConstructSignature */);
                        var indexSignatures = serializeIndexSignatures(interfaceType, baseType);
                        var heritageClauses = !ts.length(baseTypes) ? undefined : [ts.createHeritageClause(90 /* ExtendsKeyword */, ts.mapDefined(baseTypes, function (b) { return trySerializeAsTypeReference(b); }))];
                        addResult(ts.createInterfaceDeclaration(
                        /*decorators*/ undefined, 
                        /*modifiers*/ undefined, getInternalSymbolName(symbol, symbolName), typeParamDecls, heritageClauses, __spreadArrays(indexSignatures, constructSignatures, callSignatures, members)), modifierFlags);
                    }
                    function getNamespaceMembersForSerialization(symbol) {
                        return !symbol.exports ? [] : ts.filter(ts.arrayFrom((symbol.exports).values()), function (p) { return !((p.flags & 4194304 /* Prototype */) || (p.escapedName === "prototype")); });
                    }
                    function isTypeOnlyNamespace(symbol) {
                        return ts.every(getNamespaceMembersForSerialization(symbol), function (m) { return !(resolveSymbol(m).flags & 111551 /* Value */); });
                    }
                    function serializeModule(symbol, symbolName, modifierFlags) {
                        var members = getNamespaceMembersForSerialization(symbol);
                        // Split NS members up by declaration - members whose parent symbol is the ns symbol vs those whose is not (but were added in later via merging)
                        var locationMap = ts.arrayToMultiMap(members, function (m) { return m.parent && m.parent === symbol ? "real" : "merged"; });
                        var realMembers = locationMap.get("real") || ts.emptyArray;
                        var mergedMembers = locationMap.get("merged") || ts.emptyArray;
                        // TODO: `suppressNewPrivateContext` is questionable -we need to simply be emitting privates in whatever scope they were declared in, rather
                        // than whatever scope we traverse to them in. That's a bit of a complex rewrite, since we're not _actually_ tracking privates at all in advance,
                        // so we don't even have placeholders to fill in.
                        if (ts.length(realMembers)) {
                            var localName = getInternalSymbolName(symbol, symbolName);
                            serializeAsNamespaceDeclaration(realMembers, localName, modifierFlags, !!(symbol.flags & (16 /* Function */ | 67108864 /* Assignment */)));
                        }
                        if (ts.length(mergedMembers)) {
                            var localName = getInternalSymbolName(symbol, symbolName);
                            var nsBody = ts.createModuleBlock([ts.createExportDeclaration(
                                /*decorators*/ undefined, 
                                /*modifiers*/ undefined, ts.createNamedExports(ts.map(ts.filter(mergedMembers, function (n) { return n.escapedName !== "export=" /* ExportEquals */; }), function (s) {
                                    var name = ts.unescapeLeadingUnderscores(s.escapedName);
                                    var localName = getInternalSymbolName(s, name);
                                    var aliasDecl = s.declarations && getDeclarationOfAliasSymbol(s);
                                    var target = aliasDecl && getTargetOfAliasDeclaration(aliasDecl, /*dontRecursivelyResolve*/ true);
                                    includePrivateSymbol(target || s);
                                    var targetName = target ? getInternalSymbolName(target, ts.unescapeLeadingUnderscores(target.escapedName)) : localName;
                                    return ts.createExportSpecifier(name === targetName ? undefined : targetName, name);
                                })))]);
                            addResult(ts.createModuleDeclaration(
                            /*decorators*/ undefined, 
                            /*modifiers*/ undefined, ts.createIdentifier(localName), nsBody, 16 /* Namespace */), 0 /* None */);
                        }
                    }
                    function serializeEnum(symbol, symbolName, modifierFlags) {
                        addResult(ts.createEnumDeclaration(
                        /*decorators*/ undefined, ts.createModifiersFromModifierFlags(isConstEnumSymbol(symbol) ? 2048 /* Const */ : 0), getInternalSymbolName(symbol, symbolName), ts.map(ts.filter(getPropertiesOfType(getTypeOfSymbol(symbol)), function (p) { return !!(p.flags & 8 /* EnumMember */); }), function (p) {
                            // TODO: Handle computed names
                            // I hate that to get the initialized value we need to walk back to the declarations here; but there's no
                            // other way to get the possible const value of an enum member that I'm aware of, as the value is cached
                            // _on the declaration_, not on the declaration's symbol...
                            var initializedValue = p.declarations && p.declarations[0] && ts.isEnumMember(p.declarations[0]) && getConstantValue(p.declarations[0]);
                            return ts.createEnumMember(ts.unescapeLeadingUnderscores(p.escapedName), initializedValue === undefined ? undefined : ts.createLiteral(initializedValue));
                        })), modifierFlags);
                    }
                    function serializeVariableOrProperty(symbol, symbolName, isPrivate, needsPostExportDefault, propertyAsAlias, modifierFlags) {
                        if (propertyAsAlias) {
                            serializeMaybeAliasAssignment(symbol);
                        }
                        else {
                            var type = getTypeOfSymbol(symbol);
                            var localName = getInternalSymbolName(symbol, symbolName);
                            if (!(symbol.flags & 16 /* Function */) && isTypeRepresentableAsFunctionNamespaceMerge(type, symbol)) {
                                // If the type looks like a function declaration + ns could represent it, and it's type is sourced locally, rewrite it into a function declaration + ns
                                serializeAsFunctionNamespaceMerge(type, symbol, localName, modifierFlags);
                            }
                            else {
                                // A Class + Property merge is made for a `module.exports.Member = class {}`, and it doesn't serialize well as either a class _or_ a property symbol - in fact, _it behaves like an alias!_
                                // `var` is `FunctionScopedVariable`, `const` and `let` are `BlockScopedVariable`, and `module.exports.thing =` is `Property`
                                var flags = !(symbol.flags & 2 /* BlockScopedVariable */) ? undefined
                                    : isConstVariable(symbol) ? 2 /* Const */
                                        : 1 /* Let */;
                                var name = (needsPostExportDefault || !(symbol.flags & 4 /* Property */)) ? localName : getUnusedName(localName, symbol);
                                var textRange = symbol.declarations && ts.find(symbol.declarations, function (d) { return ts.isVariableDeclaration(d); });
                                if (textRange && ts.isVariableDeclarationList(textRange.parent) && textRange.parent.declarations.length === 1) {
                                    textRange = textRange.parent.parent;
                                }
                                var statement = ts.setTextRange(ts.createVariableStatement(/*modifiers*/ undefined, ts.createVariableDeclarationList([
                                    ts.createVariableDeclaration(name, serializeTypeForDeclaration(type, symbol))
                                ], flags)), textRange);
                                addResult(statement, name !== localName ? modifierFlags & ~1 /* Export */ : modifierFlags);
                                if (name !== localName && !isPrivate) {
                                    // We rename the variable declaration we generate for Property symbols since they may have a name which
                                    // conflicts with a local declaration. For example, given input:
                                    // ```
                                    // function g() {}
                                    // module.exports.g = g
                                    // ```
                                    // In such a situation, we have a local variable named `g`, and a separate exported variable named `g`.
                                    // Naively, we would emit
                                    // ```
                                    // function g() {}
                                    // export const g: typeof g;
                                    // ```
                                    // That's obviously incorrect - the `g` in the type annotation needs to refer to the local `g`, but
                                    // the export declaration shadows it.
                                    // To work around that, we instead write
                                    // ```
                                    // function g() {}
                                    // const g_1: typeof g;
                                    // export { g_1 as g };
                                    // ```
                                    // To create an export named `g` that does _not_ shadow the local `g`
                                    addResult(ts.createExportDeclaration(
                                    /*decorators*/ undefined, 
                                    /*modifiers*/ undefined, ts.createNamedExports([ts.createExportSpecifier(name, localName)])), 0 /* None */);
                                }
                            }
                        }
                    }
                    function serializeAsFunctionNamespaceMerge(type, symbol, localName, modifierFlags) {
                        var signatures = getSignaturesOfType(type, 0 /* Call */);
                        for (var _i = 0, signatures_2 = signatures; _i < signatures_2.length; _i++) {
                            var sig = signatures_2[_i];
                            // Each overload becomes a separate function declaration, in order
                            var decl = signatureToSignatureDeclarationHelper(sig, 244 /* FunctionDeclaration */, context);
                            decl.name = ts.createIdentifier(localName);
                            addResult(ts.setTextRange(decl, sig.declaration), modifierFlags);
                        }
                        // Module symbol emit will take care of module-y members, provided it has exports
                        if (!(symbol.flags & (512 /* ValueModule */ | 1024 /* NamespaceModule */) && !!symbol.exports && !!symbol.exports.size)) {
                            var props = ts.filter(getPropertiesOfType(type), function (p) { return !((p.flags & 4194304 /* Prototype */) || (p.escapedName === "prototype")); });
                            serializeAsNamespaceDeclaration(props, localName, modifierFlags, /*suppressNewPrivateContext*/ true);
                        }
                    }
                    function serializeAsNamespaceDeclaration(props, localName, modifierFlags, suppressNewPrivateContext) {
                        if (ts.length(props)) {
                            var localVsRemoteMap = ts.arrayToMultiMap(props, function (p) {
                                return !ts.length(p.declarations) || ts.some(p.declarations, function (d) {
                                    return ts.getSourceFileOfNode(d) === ts.getSourceFileOfNode(context.enclosingDeclaration);
                                }) ? "local" : "remote";
                            });
                            var localProps = localVsRemoteMap.get("local") || ts.emptyArray;
                            // handle remote props first - we need to make an `import` declaration that points at the module containing each remote
                            // prop in the outermost scope (TODO: a namespace within a namespace would need to be appropriately handled by this)
                            // Example:
                            // import Foo_1 = require("./exporter");
                            // export namespace ns {
                            //     import Foo = Foo_1.Foo;
                            //     export { Foo };
                            //     export const c: number;
                            // }
                            // This is needed because in JS, statements like `const x = require("./f")` support both type and value lookup, even if they're
                            // normally just value lookup (so it functions kinda like an alias even when it's not an alias)
                            // _Usually_, we'll simply print the top-level as an alias instead of a `var` in such situations, however is is theoretically
                            // possible to encounter a situation where a type has members from both the current file and other files - in those situations,
                            // emit akin to the above would be needed.
                            // Add a namespace
                            var fakespace = ts.createModuleDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, ts.createIdentifier(localName), ts.createModuleBlock([]), 16 /* Namespace */);
                            fakespace.flags ^= 8 /* Synthesized */; // unset synthesized so it is usable as an enclosing declaration
                            fakespace.parent = enclosingDeclaration;
                            fakespace.locals = ts.createSymbolTable(props);
                            fakespace.symbol = props[0].parent;
                            var oldResults = results;
                            results = [];
                            var oldAddingDeclare = addingDeclare;
                            addingDeclare = false;
                            var subcontext = __assign(__assign({}, context), { enclosingDeclaration: fakespace });
                            var oldContext = context;
                            context = subcontext;
                            // TODO: implement handling for the localVsRemoteMap.get("remote") - should be difficult to trigger (see comment above), as only interesting cross-file js merges should make this possible
                            visitSymbolTable(ts.createSymbolTable(localProps), suppressNewPrivateContext, /*propertyAsAlias*/ true);
                            context = oldContext;
                            addingDeclare = oldAddingDeclare;
                            var declarations = results;
                            results = oldResults;
                            fakespace.flags ^= 8 /* Synthesized */; // reset synthesized
                            fakespace.parent = undefined;
                            fakespace.locals = undefined;
                            fakespace.symbol = undefined;
                            fakespace.body = ts.createModuleBlock(declarations);
                            addResult(fakespace, modifierFlags); // namespaces can never be default exported
                        }
                    }
                    function serializeAsClass(symbol, localName, modifierFlags) {
                        var localParams = getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(symbol);
                        var typeParamDecls = ts.map(localParams, function (p) { return typeParameterToDeclaration(p, context); });
                        var classType = getDeclaredTypeOfClassOrInterface(symbol);
                        var baseTypes = getBaseTypes(classType);
                        var staticType = getTypeOfSymbol(symbol);
                        var staticBaseType = getBaseConstructorTypeOfClass(staticType);
                        var heritageClauses = !ts.length(baseTypes) ? undefined : [ts.createHeritageClause(90 /* ExtendsKeyword */, ts.map(baseTypes, function (b) { return serializeBaseType(b, staticBaseType, localName); }))];
                        var symbolProps = getPropertiesOfType(classType);
                        var publicSymbolProps = ts.filter(symbolProps, function (s) {
                            var valueDecl = s.valueDeclaration;
                            ts.Debug.assertDefined(valueDecl);
                            return !(ts.isNamedDeclaration(valueDecl) && ts.isPrivateIdentifier(valueDecl.name));
                        });
                        var hasPrivateIdentifier = ts.some(symbolProps, function (s) {
                            var valueDecl = s.valueDeclaration;
                            ts.Debug.assertDefined(valueDecl);
                            return ts.isNamedDeclaration(valueDecl) && ts.isPrivateIdentifier(valueDecl.name);
                        });
                        // Boil down all private properties into a single one.
                        var privateProperties = hasPrivateIdentifier ?
                            [ts.createProperty(
                                /*decorators*/ undefined, 
                                /*modifiers*/ undefined, ts.createPrivateIdentifier("#private"), 
                                /*questionOrExclamationToken*/ undefined, 
                                /*type*/ undefined, 
                                /*initializer*/ undefined)] :
                            ts.emptyArray;
                        var publicProperties = ts.flatMap(publicSymbolProps, function (p) { return serializePropertySymbolForClass(p, /*isStatic*/ false, baseTypes[0]); });
                        // Consider static members empty if symbol also has function or module meaning - function namespacey emit will handle statics
                        var staticMembers = symbol.flags & (16 /* Function */ | 512 /* ValueModule */)
                            ? []
                            : ts.flatMap(ts.filter(getPropertiesOfType(staticType), function (p) { return !(p.flags & 4194304 /* Prototype */) && p.escapedName !== "prototype"; }), function (p) { return serializePropertySymbolForClass(p, /*isStatic*/ true, staticBaseType); });
                        var constructors = serializeSignatures(1 /* Construct */, staticType, baseTypes[0], 162 /* Constructor */);
                        for (var _i = 0, constructors_1 = constructors; _i < constructors_1.length; _i++) {
                            var c = constructors_1[_i];
                            // A constructor's return type and type parameters are supposed to be controlled by the enclosing class declaration
                            // `signatureToSignatureDeclarationHelper` appends them regardless, so for now we delete them here
                            c.type = undefined;
                            c.typeParameters = undefined;
                        }
                        var indexSignatures = serializeIndexSignatures(classType, baseTypes[0]);
                        addResult(ts.setTextRange(ts.createClassDeclaration(
                        /*decorators*/ undefined, 
                        /*modifiers*/ undefined, localName, typeParamDecls, heritageClauses, __spreadArrays(indexSignatures, staticMembers, constructors, publicProperties, privateProperties)), symbol.declarations && ts.filter(symbol.declarations, function (d) { return ts.isClassDeclaration(d) || ts.isClassExpression(d); })[0]), modifierFlags);
                    }
                    function serializeAsAlias(symbol, localName, modifierFlags) {
                        // synthesize an alias, eg `export { symbolName as Name }`
                        // need to mark the alias `symbol` points at
                        // as something we need to serialize as a private declaration as well
                        var node = getDeclarationOfAliasSymbol(symbol);
                        if (!node)
                            return ts.Debug.fail();
                        var target = getMergedSymbol(getTargetOfAliasDeclaration(node, /*dontRecursivelyResolve*/ true));
                        if (!target) {
                            return;
                        }
                        var verbatimTargetName = ts.unescapeLeadingUnderscores(target.escapedName);
                        if (verbatimTargetName === "export=" /* ExportEquals */ && (compilerOptions.esModuleInterop || compilerOptions.allowSyntheticDefaultImports)) {
                            // target refers to an `export=` symbol that was hoisted into a synthetic default - rename here to match
                            verbatimTargetName = "default" /* Default */;
                        }
                        var targetName = getInternalSymbolName(target, verbatimTargetName);
                        includePrivateSymbol(target); // the target may be within the same scope - attempt to serialize it first
                        switch (node.kind) {
                            case 253 /* ImportEqualsDeclaration */:
                                // Could be a local `import localName = ns.member` or
                                // an external `import localName = require("whatever")`
                                var isLocalImport = !(target.flags & 512 /* ValueModule */);
                                addResult(ts.createImportEqualsDeclaration(
                                /*decorators*/ undefined, 
                                /*modifiers*/ undefined, ts.createIdentifier(localName), isLocalImport
                                    ? symbolToName(target, context, 67108863 /* All */, /*expectsIdentifier*/ false)
                                    : ts.createExternalModuleReference(ts.createLiteral(getSpecifierForModuleSymbol(symbol, context)))), isLocalImport ? modifierFlags : 0 /* None */);
                                break;
                            case 252 /* NamespaceExportDeclaration */:
                                // export as namespace foo
                                // TODO: Not part of a file's local or export symbol tables
                                // Is bound into file.symbol.globalExports instead, which we don't currently traverse
                                addResult(ts.createNamespaceExportDeclaration(ts.idText(node.name)), 0 /* None */);
                                break;
                            case 255 /* ImportClause */:
                                addResult(ts.createImportDeclaration(
                                /*decorators*/ undefined, 
                                /*modifiers*/ undefined, ts.createImportClause(ts.createIdentifier(localName), /*namedBindings*/ undefined), 
                                // We use `target.parent || target` below as `target.parent` is unset when the target is a module which has been export assigned
                                // And then made into a default by the `esModuleInterop` or `allowSyntheticDefaultImports` flag
                                // In such cases, the `target` refers to the module itself already
                                ts.createLiteral(getSpecifierForModuleSymbol(target.parent || target, context))), 0 /* None */);
                                break;
                            case 256 /* NamespaceImport */:
                                addResult(ts.createImportDeclaration(
                                /*decorators*/ undefined, 
                                /*modifiers*/ undefined, ts.createImportClause(/*importClause*/ undefined, ts.createNamespaceImport(ts.createIdentifier(localName))), ts.createLiteral(getSpecifierForModuleSymbol(target, context))), 0 /* None */);
                                break;
                            case 262 /* NamespaceExport */:
                                addResult(ts.createExportDeclaration(
                                /*decorators*/ undefined, 
                                /*modifiers*/ undefined, ts.createNamespaceExport(ts.createIdentifier(localName)), ts.createLiteral(getSpecifierForModuleSymbol(target, context))), 0 /* None */);
                                break;
                            case 258 /* ImportSpecifier */:
                                addResult(ts.createImportDeclaration(
                                /*decorators*/ undefined, 
                                /*modifiers*/ undefined, ts.createImportClause(/*importClause*/ undefined, ts.createNamedImports([
                                    ts.createImportSpecifier(localName !== verbatimTargetName ? ts.createIdentifier(verbatimTargetName) : undefined, ts.createIdentifier(localName))
                                ])), ts.createLiteral(getSpecifierForModuleSymbol(target.parent || target, context))), 0 /* None */);
                                break;
                            case 263 /* ExportSpecifier */:
                                // does not use localName because the symbol name in this case refers to the name in the exports table,
                                // which we must exactly preserve
                                var specifier = node.parent.parent.moduleSpecifier;
                                // targetName is only used when the target is local, as otherwise the target is an alias that points at
                                // another file
                                serializeExportSpecifier(ts.unescapeLeadingUnderscores(symbol.escapedName), specifier ? verbatimTargetName : targetName, specifier && ts.isStringLiteralLike(specifier) ? ts.createLiteral(specifier.text) : undefined);
                                break;
                            case 259 /* ExportAssignment */:
                                serializeMaybeAliasAssignment(symbol);
                                break;
                            case 209 /* BinaryExpression */:
                            case 194 /* PropertyAccessExpression */:
                                // Could be best encoded as though an export specifier or as though an export assignment
                                // If name is default or export=, do an export assignment
                                // Otherwise do an export specifier
                                if (symbol.escapedName === "default" /* Default */ || symbol.escapedName === "export=" /* ExportEquals */) {
                                    serializeMaybeAliasAssignment(symbol);
                                }
                                else {
                                    serializeExportSpecifier(localName, targetName);
                                }
                                break;
                            default:
                                return ts.Debug.failBadSyntaxKind(node, "Unhandled alias declaration kind in symbol serializer!");
                        }
                    }
                    function serializeExportSpecifier(localName, targetName, specifier) {
                        addResult(ts.createExportDeclaration(
                        /*decorators*/ undefined, 
                        /*modifiers*/ undefined, ts.createNamedExports([ts.createExportSpecifier(localName !== targetName ? targetName : undefined, localName)]), specifier), 0 /* None */);
                    }
                    function serializeMaybeAliasAssignment(symbol) {
                        if (symbol.flags & 4194304 /* Prototype */) {
                            return;
                        }
                        var name = ts.unescapeLeadingUnderscores(symbol.escapedName);
                        var isExportEquals = name === "export=" /* ExportEquals */;
                        var isDefault = name === "default" /* Default */;
                        var isExportAssignment = isExportEquals || isDefault;
                        // synthesize export = ref
                        // ref should refer to either be a locally scoped symbol which we need to emit, or
                        // a reference to another namespace/module which we may need to emit an `import` statement for
                        var aliasDecl = symbol.declarations && getDeclarationOfAliasSymbol(symbol);
                        // serialize what the alias points to, preserve the declaration's initializer
                        var target = aliasDecl && getTargetOfAliasDeclaration(aliasDecl, /*dontRecursivelyResolve*/ true);
                        // If the target resolves and resolves to a thing defined in this file, emit as an alias, otherwise emit as a const
                        if (target && ts.length(target.declarations) && ts.some(target.declarations, function (d) { return ts.getSourceFileOfNode(d) === ts.getSourceFileOfNode(enclosingDeclaration); })) {
                            // In case `target` refers to a namespace member, look at the declaration and serialize the leftmost symbol in it
                            // eg, `namespace A { export class B {} }; exports = A.B;`
                            // Technically, this is all that's required in the case where the assignment is an entity name expression
                            var expr = isExportAssignment ? ts.getExportAssignmentExpression(aliasDecl) : ts.getPropertyAssignmentAliasLikeExpression(aliasDecl);
                            var first_1 = ts.isEntityNameExpression(expr) ? getFirstNonModuleExportsIdentifier(expr) : undefined;
                            var referenced = first_1 && resolveEntityName(first_1, 67108863 /* All */, /*ignoreErrors*/ true, /*dontResolveAlias*/ true, enclosingDeclaration);
                            if (referenced || target) {
                                includePrivateSymbol(referenced || target);
                            }
                            // We disable the context's symbol traker for the duration of this name serialization
                            // as, by virtue of being here, the name is required to print something, and we don't want to
                            // issue a visibility error on it. Only anonymous classes that an alias points at _would_ issue
                            // a visibility error here (as they're not visible within any scope), but we want to hoist them
                            // into the containing scope anyway, so we want to skip the visibility checks.
                            var oldTrack = context.tracker.trackSymbol;
                            context.tracker.trackSymbol = ts.noop;
                            if (isExportAssignment) {
                                results.push(ts.createExportAssignment(
                                /*decorators*/ undefined, 
                                /*modifiers*/ undefined, isExportEquals, symbolToExpression(target, context, 67108863 /* All */)));
                            }
                            else {
                                if (first_1 === expr) {
                                    // serialize as `export {target as name}`
                                    serializeExportSpecifier(name, ts.idText(first_1));
                                }
                                else if (ts.isClassExpression(expr)) {
                                    serializeExportSpecifier(name, getInternalSymbolName(target, ts.symbolName(target)));
                                }
                                else {
                                    // serialize as `import _Ref = t.arg.et; export { _Ref as name }`
                                    var varName = getUnusedName(name, symbol);
                                    addResult(ts.createImportEqualsDeclaration(
                                    /*decorators*/ undefined, 
                                    /*modifiers*/ undefined, ts.createIdentifier(varName), symbolToName(target, context, 67108863 /* All */, /*expectsIdentifier*/ false)), 0 /* None */);
                                    serializeExportSpecifier(name, varName);
                                }
                            }
                            context.tracker.trackSymbol = oldTrack;
                        }
                        else {
                            // serialize as an anonymous property declaration
                            var varName = getUnusedName(name, symbol);
                            // We have to use `getWidenedType` here since the object within a json file is unwidened within the file
                            // (Unwidened types can only exist in expression contexts and should never be serialized)
                            var typeToSerialize = getWidenedType(getTypeOfSymbol(symbol));
                            if (isTypeRepresentableAsFunctionNamespaceMerge(typeToSerialize, symbol)) {
                                // If there are no index signatures and `typeToSerialize` is an object type, emit as a namespace instead of a const
                                serializeAsFunctionNamespaceMerge(typeToSerialize, symbol, varName, isExportAssignment ? 0 /* None */ : 1 /* Export */);
                            }
                            else {
                                var statement = ts.createVariableStatement(/*modifiers*/ undefined, ts.createVariableDeclarationList([
                                    ts.createVariableDeclaration(varName, serializeTypeForDeclaration(typeToSerialize, symbol))
                                ], 2 /* Const */));
                                addResult(statement, name === varName ? 1 /* Export */ : 0 /* None */);
                            }
                            if (isExportAssignment) {
                                results.push(ts.createExportAssignment(
                                /*decorators*/ undefined, 
                                /*modifiers*/ undefined, isExportEquals, ts.createIdentifier(varName)));
                            }
                            else if (name !== varName) {
                                serializeExportSpecifier(name, varName);
                            }
                        }
                    }
                    function isTypeRepresentableAsFunctionNamespaceMerge(typeToSerialize, hostSymbol) {
                        // Only object types which are not constructable, or indexable, whose members all come from the
                        // context source file, and whose property names are all valid identifiers and not late-bound, _and_
                        // whose input is not type annotated (if the input symbol has an annotation we can reuse, we should prefer it)
                        var ctxSrc = ts.getSourceFileOfNode(context.enclosingDeclaration);
                        return ts.getObjectFlags(typeToSerialize) & (16 /* Anonymous */ | 32 /* Mapped */) &&
                            !getIndexInfoOfType(typeToSerialize, 0 /* String */) &&
                            !getIndexInfoOfType(typeToSerialize, 1 /* Number */) &&
                            !!(ts.length(getPropertiesOfType(typeToSerialize)) || ts.length(getSignaturesOfType(typeToSerialize, 0 /* Call */))) &&
                            !ts.length(getSignaturesOfType(typeToSerialize, 1 /* Construct */)) && // TODO: could probably serialize as function + ns + class, now that that's OK
                            !getDeclarationWithTypeAnnotation(hostSymbol) &&
                            !(typeToSerialize.symbol && ts.some(typeToSerialize.symbol.declarations, function (d) { return ts.getSourceFileOfNode(d) !== ctxSrc; })) &&
                            !ts.some(getPropertiesOfType(typeToSerialize), function (p) { return isLateBoundName(p.escapedName); }) &&
                            !ts.some(getPropertiesOfType(typeToSerialize), function (p) { return ts.some(p.declarations, function (d) { return ts.getSourceFileOfNode(d) !== ctxSrc; }); }) &&
                            ts.every(getPropertiesOfType(typeToSerialize), function (p) { return ts.isIdentifierText(ts.symbolName(p), languageVersion) && !ts.isStringAKeyword(ts.symbolName(p)); });
                    }
                    function makeSerializePropertySymbol(createProperty, methodKind, useAccessors) {
                        return function serializePropertySymbol(p, isStatic, baseType) {
                            var modifierFlags = ts.getDeclarationModifierFlagsFromSymbol(p);
                            var isPrivate = !!(modifierFlags & 8 /* Private */);
                            if (isStatic && (p.flags & (788968 /* Type */ | 1920 /* Namespace */ | 2097152 /* Alias */))) {
                                // Only value-only-meaning symbols can be correctly encoded as class statics, type/namespace/alias meaning symbols
                                // need to be merged namespace members
                                return [];
                            }
                            if (p.flags & 4194304 /* Prototype */ || (baseType && getPropertyOfType(baseType, p.escapedName)
                                && isReadonlySymbol(getPropertyOfType(baseType, p.escapedName)) === isReadonlySymbol(p)
                                && (p.flags & 16777216 /* Optional */) === (getPropertyOfType(baseType, p.escapedName).flags & 16777216 /* Optional */)
                                && isTypeIdenticalTo(getTypeOfSymbol(p), getTypeOfPropertyOfType(baseType, p.escapedName)))) {
                                return [];
                            }
                            var flag = (modifierFlags & ~256 /* Async */) | (isStatic ? 32 /* Static */ : 0);
                            var name = getPropertyNameNodeForSymbol(p, context);
                            var firstPropertyLikeDecl = ts.find(p.declarations, ts.or(ts.isPropertyDeclaration, ts.isAccessor, ts.isVariableDeclaration, ts.isPropertySignature, ts.isBinaryExpression, ts.isPropertyAccessExpression));
                            if (p.flags & 98304 /* Accessor */ && useAccessors) {
                                var result = [];
                                if (p.flags & 65536 /* SetAccessor */) {
                                    result.push(ts.setTextRange(ts.createSetAccessor(
                                    /*decorators*/ undefined, ts.createModifiersFromModifierFlags(flag), name, [ts.createParameter(
                                        /*decorators*/ undefined, 
                                        /*modifiers*/ undefined, 
                                        /*dotDotDotToken*/ undefined, "arg", 
                                        /*questionToken*/ undefined, isPrivate ? undefined : serializeTypeForDeclaration(getTypeOfSymbol(p), p))], 
                                    /*body*/ undefined), ts.find(p.declarations, ts.isSetAccessor) || firstPropertyLikeDecl));
                                }
                                if (p.flags & 32768 /* GetAccessor */) {
                                    var isPrivate_1 = modifierFlags & 8 /* Private */;
                                    result.push(ts.setTextRange(ts.createGetAccessor(
                                    /*decorators*/ undefined, ts.createModifiersFromModifierFlags(flag), name, [], isPrivate_1 ? undefined : serializeTypeForDeclaration(getTypeOfSymbol(p), p), 
                                    /*body*/ undefined), ts.find(p.declarations, ts.isGetAccessor) || firstPropertyLikeDecl));
                                }
                                return result;
                            }
                            // This is an else/if as accessors and properties can't merge in TS, but might in JS
                            // If this happens, we assume the accessor takes priority, as it imposes more constraints
                            else if (p.flags & (4 /* Property */ | 3 /* Variable */)) {
                                return ts.setTextRange(createProperty(
                                /*decorators*/ undefined, ts.createModifiersFromModifierFlags((isReadonlySymbol(p) ? 64 /* Readonly */ : 0) | flag), name, p.flags & 16777216 /* Optional */ ? ts.createToken(57 /* QuestionToken */) : undefined, isPrivate ? undefined : serializeTypeForDeclaration(getTypeOfSymbol(p), p), 
                                // TODO: https://github.com/microsoft/TypeScript/pull/32372#discussion_r328386357
                                // interface members can't have initializers, however class members _can_
                                /*initializer*/ undefined), ts.find(p.declarations, ts.or(ts.isPropertyDeclaration, ts.isVariableDeclaration)) || firstPropertyLikeDecl);
                            }
                            if (p.flags & (8192 /* Method */ | 16 /* Function */)) {
                                var type = getTypeOfSymbol(p);
                                var signatures = getSignaturesOfType(type, 0 /* Call */);
                                if (flag & 8 /* Private */) {
                                    return ts.setTextRange(createProperty(
                                    /*decorators*/ undefined, ts.createModifiersFromModifierFlags((isReadonlySymbol(p) ? 64 /* Readonly */ : 0) | flag), name, p.flags & 16777216 /* Optional */ ? ts.createToken(57 /* QuestionToken */) : undefined, 
                                    /*type*/ undefined, 
                                    /*initializer*/ undefined), ts.find(p.declarations, ts.isFunctionLikeDeclaration) || signatures[0] && signatures[0].declaration || p.declarations[0]);
                                }
                                var results_1 = [];
                                for (var _i = 0, signatures_3 = signatures; _i < signatures_3.length; _i++) {
                                    var sig = signatures_3[_i];
                                    // Each overload becomes a separate method declaration, in order
                                    var decl = signatureToSignatureDeclarationHelper(sig, methodKind, context);
                                    decl.name = name; // TODO: Clone
                                    if (flag) {
                                        decl.modifiers = ts.createNodeArray(ts.createModifiersFromModifierFlags(flag));
                                    }
                                    if (p.flags & 16777216 /* Optional */) {
                                        decl.questionToken = ts.createToken(57 /* QuestionToken */);
                                    }
                                    results_1.push(ts.setTextRange(decl, sig.declaration));
                                }
                                return results_1;
                            }
                            // The `Constructor`'s symbol isn't in the class's properties lists, obviously, since it's a signature on the static
                            return ts.Debug.fail("Unhandled class member kind! " + (p.__debugFlags || p.flags));
                        };
                    }
                    function serializePropertySymbolForInterface(p, baseType) {
                        return serializePropertySymbolForInterfaceWorker(p, /*isStatic*/ false, baseType);
                    }
                    function getDeclarationWithTypeAnnotation(symbol) {
                        return symbol.declarations && ts.find(symbol.declarations, function (s) { return !!ts.getEffectiveTypeAnnotationNode(s) && !!ts.findAncestor(s, function (n) { return n === enclosingDeclaration; }); });
                    }
                    /**
                     * Unlike `typeToTypeNodeHelper`, this handles setting up the `AllowUniqueESSymbolType` flag
                     * so a `unique symbol` is returned when appropriate for the input symbol, rather than `typeof sym`
                     */
                    function serializeTypeForDeclaration(type, symbol) {
                        var declWithExistingAnnotation = getDeclarationWithTypeAnnotation(symbol);
                        if (declWithExistingAnnotation && !ts.isFunctionLikeDeclaration(declWithExistingAnnotation)) {
                            // try to reuse the existing annotation
                            var existing = ts.getEffectiveTypeAnnotationNode(declWithExistingAnnotation);
                            var transformed = ts.visitNode(existing, visitExistingNodeTreeSymbols);
                            return transformed === existing ? ts.getMutableClone(existing) : transformed;
                        }
                        var oldFlags = context.flags;
                        if (type.flags & 8192 /* UniqueESSymbol */ &&
                            type.symbol === symbol) {
                            context.flags |= 1048576 /* AllowUniqueESSymbolType */;
                        }
                        var result = typeToTypeNodeHelper(type, context);
                        context.flags = oldFlags;
                        return result;
                        function visitExistingNodeTreeSymbols(node) {
                            if (ts.isJSDocAllType(node)) {
                                return ts.createKeywordTypeNode(125 /* AnyKeyword */);
                            }
                            if (ts.isJSDocUnknownType(node)) {
                                return ts.createKeywordTypeNode(148 /* UnknownKeyword */);
                            }
                            if (ts.isJSDocNullableType(node)) {
                                return ts.createUnionTypeNode([ts.visitNode(node.type, visitExistingNodeTreeSymbols), ts.createKeywordTypeNode(100 /* NullKeyword */)]);
                            }
                            if (ts.isJSDocOptionalType(node)) {
                                return ts.createUnionTypeNode([ts.visitNode(node.type, visitExistingNodeTreeSymbols), ts.createKeywordTypeNode(146 /* UndefinedKeyword */)]);
                            }
                            if (ts.isJSDocNonNullableType(node)) {
                                return ts.visitNode(node.type, visitExistingNodeTreeSymbols);
                            }
                            if ((ts.isExpressionWithTypeArguments(node) || ts.isTypeReferenceNode(node)) && ts.isJSDocIndexSignature(node)) {
                                return ts.createTypeLiteralNode([ts.createIndexSignature(
                                    /*decorators*/ undefined, 
                                    /*modifiers*/ undefined, [ts.createParameter(
                                        /*decorators*/ undefined, 
                                        /*modifiers*/ undefined, 
                                        /*dotdotdotToken*/ undefined, "x", 
                                        /*questionToken*/ undefined, ts.visitNode(node.typeArguments[0], visitExistingNodeTreeSymbols))], ts.visitNode(node.typeArguments[1], visitExistingNodeTreeSymbols))]);
                            }
                            if (ts.isJSDocFunctionType(node)) {
                                if (ts.isJSDocConstructSignature(node)) {
                                    var newTypeNode_1;
                                    return ts.createConstructorTypeNode(ts.visitNodes(node.typeParameters, visitExistingNodeTreeSymbols), ts.mapDefined(node.parameters, function (p, i) {
                                        return p.name && ts.isIdentifier(p.name) && p.name.escapedText === "new" ? (newTypeNode_1 = p.type, undefined) : ts.createParameter(
                                        /*decorators*/ undefined, 
                                        /*modifiers*/ undefined, p.dotDotDotToken, p.name || p.dotDotDotToken ? "args" : "arg" + i, p.questionToken, ts.visitNode(p.type, visitExistingNodeTreeSymbols), 
                                        /*initializer*/ undefined);
                                    }), ts.visitNode(newTypeNode_1 || node.type, visitExistingNodeTreeSymbols));
                                }
                                else {
                                    return ts.createFunctionTypeNode(ts.visitNodes(node.typeParameters, visitExistingNodeTreeSymbols), ts.map(node.parameters, function (p, i) {
                                        return ts.createParameter(
                                        /*decorators*/ undefined, 
                                        /*modifiers*/ undefined, p.dotDotDotToken, p.name || p.dotDotDotToken ? "args" : "arg" + i, p.questionToken, ts.visitNode(p.type, visitExistingNodeTreeSymbols), 
                                        /*initializer*/ undefined);
                                    }), ts.visitNode(node.type, visitExistingNodeTreeSymbols));
                                }
                            }
                            if (ts.isLiteralImportTypeNode(node)) {
                                return ts.updateImportTypeNode(node, ts.updateLiteralTypeNode(node.argument, rewriteModuleSpecifier(node, node.argument.literal)), node.qualifier, ts.visitNodes(node.typeArguments, visitExistingNodeTreeSymbols, ts.isTypeNode), node.isTypeOf);
                            }
                            if (ts.isEntityName(node) || ts.isEntityNameExpression(node)) {
                                var leftmost = ts.getFirstIdentifier(node);
                                var sym = resolveEntityName(leftmost, 67108863 /* All */, /*ignoreErrors*/ true, /*dontResolveALias*/ true);
                                if (sym) {
                                    includePrivateSymbol(sym);
                                    if (ts.isIdentifier(node) && sym.flags & 262144 /* TypeParameter */) {
                                        var name = typeParameterToName(getDeclaredTypeOfSymbol(sym), context);
                                        if (ts.idText(name) !== ts.idText(node)) {
                                            return name;
                                        }
                                        return node;
                                    }
                                }
                            }
                            return ts.visitEachChild(node, visitExistingNodeTreeSymbols, ts.nullTransformationContext);
                        }
                        function rewriteModuleSpecifier(parent, lit) {
                            if (bundled) {
                                if (context.tracker && context.tracker.moduleResolverHost) {
                                    var targetFile = getExternalModuleFileFromDeclaration(parent);
                                    if (targetFile) {
                                        var getCanonicalFileName = ts.createGetCanonicalFileName(!!host.useCaseSensitiveFileNames);
                                        var resolverHost = {
                                            getCanonicalFileName: getCanonicalFileName,
                                            getCurrentDirectory: context.tracker.moduleResolverHost.getCurrentDirectory ? function () { return context.tracker.moduleResolverHost.getCurrentDirectory(); } : function () { return ""; },
                                            getCommonSourceDirectory: function () { return context.tracker.moduleResolverHost.getCommonSourceDirectory(); }
                                        };
                                        var newName = ts.getResolvedExternalModuleName(resolverHost, targetFile);
                                        return ts.createLiteral(newName);
                                    }
                                }
                            }
                            else {
                                if (context.tracker && context.tracker.trackExternalModuleSymbolOfImportTypeNode) {
                                    var moduleSym = resolveExternalModuleNameWorker(lit, lit, /*moduleNotFoundError*/ undefined);
                                    if (moduleSym) {
                                        context.tracker.trackExternalModuleSymbolOfImportTypeNode(moduleSym);
                                    }
                                }
                            }
                            return lit;
                        }
                    }
                    function serializeSignatures(kind, input, baseType, outputKind) {
                        var signatures = getSignaturesOfType(input, kind);
                        if (kind === 1 /* Construct */) {
                            if (!baseType && ts.every(signatures, function (s) { return ts.length(s.parameters) === 0; })) {
                                return []; // No base type, every constructor is empty - elide the extraneous `constructor()`
                            }
                            if (baseType) {
                                // If there is a base type, if every signature in the class is identical to a signature in the baseType, elide all the declarations
                                var baseSigs = getSignaturesOfType(baseType, 1 /* Construct */);
                                if (!ts.length(baseSigs) && ts.every(signatures, function (s) { return ts.length(s.parameters) === 0; })) {
                                    return []; // Base had no explicit signatures, if all our signatures are also implicit, return an empty list
                                }
                                if (baseSigs.length === signatures.length) {
                                    var failed = false;
                                    for (var i = 0; i < baseSigs.length; i++) {
                                        if (!compareSignaturesIdentical(signatures[i], baseSigs[i], /*partialMatch*/ false, /*ignoreThisTypes*/ false, /*ignoreReturnTypes*/ true, compareTypesIdentical)) {
                                            failed = true;
                                            break;
                                        }
                                    }
                                    if (!failed) {
                                        return []; // Every signature was identical - elide constructor list as it is inherited
                                    }
                                }
                            }
                            var privateProtected = 0;
                            for (var _i = 0, signatures_4 = signatures; _i < signatures_4.length; _i++) {
                                var s = signatures_4[_i];
                                if (s.declaration) {
                                    privateProtected |= ts.getSelectedModifierFlags(s.declaration, 8 /* Private */ | 16 /* Protected */);
                                }
                            }
                            if (privateProtected) {
                                return [ts.setTextRange(ts.createConstructor(
                                    /*decorators*/ undefined, ts.createModifiersFromModifierFlags(privateProtected), 
                                    /*parameters*/ [], 
                                    /*body*/ undefined), signatures[0].declaration)];
                            }
                        }
                        var results = [];
                        for (var _a = 0, signatures_5 = signatures; _a < signatures_5.length; _a++) {
                            var sig = signatures_5[_a];
                            // Each overload becomes a separate constructor declaration, in order
                            var decl = signatureToSignatureDeclarationHelper(sig, outputKind, context);
                            results.push(ts.setTextRange(decl, sig.declaration));
                        }
                        return results;
                    }
                    function serializeIndexSignatures(input, baseType) {
                        var results = [];
                        for (var _i = 0, _a = [0 /* String */, 1 /* Number */]; _i < _a.length; _i++) {
                            var type = _a[_i];
                            var info = getIndexInfoOfType(input, type);
                            if (info) {
                                if (baseType) {
                                    var baseInfo = getIndexInfoOfType(baseType, type);
                                    if (baseInfo) {
                                        if (isTypeIdenticalTo(info.type, baseInfo.type)) {
                                            continue; // elide identical index signatures
                                        }
                                    }
                                }
                                results.push(indexInfoToIndexSignatureDeclarationHelper(info, type, context));
                            }
                        }
                        return results;
                    }
                    function serializeBaseType(t, staticType, rootName) {
                        var ref = trySerializeAsTypeReference(t);
                        if (ref) {
                            return ref;
                        }
                        var tempName = getUnusedName(rootName + "_base");
                        var statement = ts.createVariableStatement(/*modifiers*/ undefined, ts.createVariableDeclarationList([
                            ts.createVariableDeclaration(tempName, typeToTypeNodeHelper(staticType, context))
                        ], 2 /* Const */));
                        addResult(statement, 0 /* None */);
                        return ts.createExpressionWithTypeArguments(/*typeArgs*/ undefined, ts.createIdentifier(tempName));
                    }
                    function trySerializeAsTypeReference(t) {
                        var typeArgs;
                        var reference;
                        // We don't use `isValueSymbolAccessible` below. since that considers alternative containers (like modules)
                        // which we can't write out in a syntactically valid way as an expression
                        if (t.target && getAccessibleSymbolChain(t.target.symbol, enclosingDeclaration, 111551 /* Value */, /*useOnlyExternalAliasing*/ false)) {
                            typeArgs = ts.map(getTypeArguments(t), function (t) { return typeToTypeNodeHelper(t, context); });
                            reference = symbolToExpression(t.target.symbol, context, 788968 /* Type */);
                        }
                        else if (t.symbol && getAccessibleSymbolChain(t.symbol, enclosingDeclaration, 111551 /* Value */, /*useOnlyExternalAliasing*/ false)) {
                            reference = symbolToExpression(t.symbol, context, 788968 /* Type */);
                        }
                        if (reference) {
                            return ts.createExpressionWithTypeArguments(typeArgs, reference);
                        }
                    }
                    function getUnusedName(input, symbol) {
                        if (symbol) {
                            if (context.remappedSymbolNames.has("" + getSymbolId(symbol))) {
                                return context.remappedSymbolNames.get("" + getSymbolId(symbol));
                            }
                        }
                        if (symbol) {
                            input = getNameCandidateWorker(symbol, input);
                        }
                        var i = 0;
                        var original = input;
                        while (context.usedSymbolNames.has(input)) {
                            i++;
                            input = original + "_" + i;
                        }
                        context.usedSymbolNames.set(input, true);
                        if (symbol) {
                            context.remappedSymbolNames.set("" + getSymbolId(symbol), input);
                        }
                        return input;
                    }
                    function getNameCandidateWorker(symbol, localName) {
                        if (localName === "default" /* Default */ || localName === "__class" /* Class */ || localName === "__function" /* Function */) {
                            var flags = context.flags;
                            context.flags |= 16777216 /* InInitialEntityName */;
                            var nameCandidate = getNameOfSymbolAsWritten(symbol, context);
                            context.flags = flags;
                            localName = nameCandidate.length > 0 && ts.isSingleOrDoubleQuote(nameCandidate.charCodeAt(0)) ? ts.stripQuotes(nameCandidate) : nameCandidate;
                        }
                        if (localName === "default" /* Default */) {
                            localName = "_default";
                        }
                        else if (localName === "export=" /* ExportEquals */) {
                            localName = "_exports";
                        }
                        localName = ts.isIdentifierText(localName, languageVersion) && !ts.isStringANonContextualKeyword(localName) ? localName : "_" + localName.replace(/[^a-zA-Z0-9]/g, "_");
                        return localName;
                    }
                    function getInternalSymbolName(symbol, localName) {
                        if (context.remappedSymbolNames.has("" + getSymbolId(symbol))) {
                            return context.remappedSymbolNames.get("" + getSymbolId(symbol));
                        }
                        localName = getNameCandidateWorker(symbol, localName);
                        // The result of this is going to be used as the symbol's name - lock it in, so `getUnusedName` will also pick it up
                        context.remappedSymbolNames.set("" + getSymbolId(symbol), localName);
                        return localName;
                    }
                }
            }
            function typePredicateToString(typePredicate, enclosingDeclaration, flags, writer) {
                if (flags === void 0) {
                    flags = 16384 /* UseAliasDefinedOutsideCurrentScope */;
                }
                return writer ? typePredicateToStringWorker(writer).getText() : ts.usingSingleLineStringWriter(typePredicateToStringWorker);
                function typePredicateToStringWorker(writer) {
                    var predicate = ts.createTypePredicateNodeWithModifier(typePredicate.kind === 2 /* AssertsThis */ || typePredicate.kind === 3 /* AssertsIdentifier */ ? ts.createToken(124 /* AssertsKeyword */) : undefined, typePredicate.kind === 1 /* Identifier */ || typePredicate.kind === 3 /* AssertsIdentifier */ ? ts.createIdentifier(typePredicate.parameterName) : ts.createThisTypeNode(), typePredicate.type && nodeBuilder.typeToTypeNode(typePredicate.type, enclosingDeclaration, toNodeBuilderFlags(flags) | 70221824 /* IgnoreErrors */ | 512 /* WriteTypeParametersInQualifiedName */) // TODO: GH#18217
                    );
                    var printer = ts.createPrinter({ removeComments: true });
                    var sourceFile = enclosingDeclaration && ts.getSourceFileOfNode(enclosingDeclaration);
                    printer.writeNode(4 /* Unspecified */, predicate, /*sourceFile*/ sourceFile, writer);
                    return writer;
                }
            }
            function formatUnionTypes(types) {
                var result = [];
                var flags = 0;
                for (var i = 0; i < types.length; i++) {
                    var t = types[i];
                    flags |= t.flags;
                    if (!(t.flags & 98304 /* Nullable */)) {
                        if (t.flags & (512 /* BooleanLiteral */ | 1024 /* EnumLiteral */)) {
                            var baseType = t.flags & 512 /* BooleanLiteral */ ? booleanType : getBaseTypeOfEnumLiteralType(t);
                            if (baseType.flags & 1048576 /* Union */) {
                                var count = baseType.types.length;
                                if (i + count <= types.length && getRegularTypeOfLiteralType(types[i + count - 1]) === getRegularTypeOfLiteralType(baseType.types[count - 1])) {
                                    result.push(baseType);
                                    i += count - 1;
                                    continue;
                                }
                            }
                        }
                        result.push(t);
                    }
                }
                if (flags & 65536 /* Null */)
                    result.push(nullType);
                if (flags & 32768 /* Undefined */)
                    result.push(undefinedType);
                return result || types;
            }
            function visibilityToString(flags) {
                if (flags === 8 /* Private */) {
                    return "private";
                }
                if (flags === 16 /* Protected */) {
                    return "protected";
                }
                return "public";
            }
            function getTypeAliasForTypeLiteral(type) {
                if (type.symbol && type.symbol.flags & 2048 /* TypeLiteral */) {
                    var node = ts.findAncestor(type.symbol.declarations[0].parent, function (n) { return n.kind !== 182 /* ParenthesizedType */; });
                    if (node.kind === 247 /* TypeAliasDeclaration */) {
                        return getSymbolOfNode(node);
                    }
                }
                return undefined;
            }
            function isTopLevelInExternalModuleAugmentation(node) {
                return node && node.parent &&
                    node.parent.kind === 250 /* ModuleBlock */ &&
                    ts.isExternalModuleAugmentation(node.parent.parent);
            }
            function isDefaultBindingContext(location) {
                return location.kind === 290 /* SourceFile */ || ts.isAmbientModule(location);
            }
            function getNameOfSymbolFromNameType(symbol, context) {
                var nameType = getSymbolLinks(symbol).nameType;
                if (nameType) {
                    if (nameType.flags & 384 /* StringOrNumberLiteral */) {
                        var name = "" + nameType.value;
                        if (!ts.isIdentifierText(name, compilerOptions.target) && !isNumericLiteralName(name)) {
                            return "\"" + ts.escapeString(name, 34 /* doubleQuote */) + "\"";
                        }
                        if (isNumericLiteralName(name) && ts.startsWith(name, "-")) {
                            return "[" + name + "]";
                        }
                        return name;
                    }
                    if (nameType.flags & 8192 /* UniqueESSymbol */) {
                        return "[" + getNameOfSymbolAsWritten(nameType.symbol, context) + "]";
                    }
                }
            }
            /**
             * Gets a human-readable name for a symbol.
             * Should *not* be used for the right-hand side of a `.` -- use `symbolName(symbol)` for that instead.
             *
             * Unlike `symbolName(symbol)`, this will include quotes if the name is from a string literal.
             * It will also use a representation of a number as written instead of a decimal form, e.g. `0o11` instead of `9`.
             */
            function getNameOfSymbolAsWritten(symbol, context) {
                if (context && symbol.escapedName === "default" /* Default */ && !(context.flags & 16384 /* UseAliasDefinedOutsideCurrentScope */) &&
                    // If it's not the first part of an entity name, it must print as `default`
                    (!(context.flags & 16777216 /* InInitialEntityName */) ||
                        // if the symbol is synthesized, it will only be referenced externally it must print as `default`
                        !symbol.declarations ||
                        // if not in the same binding context (source file, module declaration), it must print as `default`
                        (context.enclosingDeclaration && ts.findAncestor(symbol.declarations[0], isDefaultBindingContext) !== ts.findAncestor(context.enclosingDeclaration, isDefaultBindingContext)))) {
                    return "default";
                }
                if (symbol.declarations && symbol.declarations.length) {
                    var declaration = ts.firstDefined(symbol.declarations, function (d) { return ts.getNameOfDeclaration(d) ? d : undefined; }); // Try using a declaration with a name, first
                    var name_2 = declaration && ts.getNameOfDeclaration(declaration);
                    if (declaration && name_2) {
                        if (ts.isCallExpression(declaration) && ts.isBindableObjectDefinePropertyCall(declaration)) {
                            return ts.symbolName(symbol);
                        }
                        if (ts.isComputedPropertyName(name_2) && !(ts.getCheckFlags(symbol) & 4096 /* Late */)) {
                            var nameType = getSymbolLinks(symbol).nameType;
                            if (nameType && nameType.flags & 384 /* StringOrNumberLiteral */) {
                                // Computed property name isn't late bound, but has a well-known name type - use name type to generate a symbol name
                                var result = getNameOfSymbolFromNameType(symbol, context);
                                if (result !== undefined) {
                                    return result;
                                }
                            }
                        }
                        return ts.declarationNameToString(name_2);
                    }
                    if (!declaration) {
                        declaration = symbol.declarations[0]; // Declaration may be nameless, but we'll try anyway
                    }
                    if (declaration.parent && declaration.parent.kind === 242 /* VariableDeclaration */) {
                        return ts.declarationNameToString(declaration.parent.name);
                    }
                    switch (declaration.kind) {
                        case 214 /* ClassExpression */:
                        case 201 /* FunctionExpression */:
                        case 202 /* ArrowFunction */:
                            if (context && !context.encounteredError && !(context.flags & 131072 /* AllowAnonymousIdentifier */)) {
                                context.encounteredError = true;
                            }
                            return declaration.kind === 214 /* ClassExpression */ ? "(Anonymous class)" : "(Anonymous function)";
                    }
                }
                var name = getNameOfSymbolFromNameType(symbol, context);
                return name !== undefined ? name : ts.symbolName(symbol);
            }
            function isDeclarationVisible(node) {
                if (node) {
                    var links = getNodeLinks(node);
                    if (links.isVisible === undefined) {
                        links.isVisible = !!determineIfDeclarationIsVisible();
                    }
                    return links.isVisible;
                }
                return false;
                function determineIfDeclarationIsVisible() {
                    switch (node.kind) {
                        case 314 /* JSDocCallbackTag */:
                        case 321 /* JSDocTypedefTag */:
                        case 315 /* JSDocEnumTag */:
                            // Top-level jsdoc type aliases are considered exported
                            // First parent is comment node, second is hosting declaration or token; we only care about those tokens or declarations whose parent is a source file
                            return !!(node.parent && node.parent.parent && node.parent.parent.parent && ts.isSourceFile(node.parent.parent.parent));
                        case 191 /* BindingElement */:
                            return isDeclarationVisible(node.parent.parent);
                        case 242 /* VariableDeclaration */:
                            if (ts.isBindingPattern(node.name) &&
                                !node.name.elements.length) {
                                // If the binding pattern is empty, this variable declaration is not visible
                                return false;
                            }
                        // falls through
                        case 249 /* ModuleDeclaration */:
                        case 245 /* ClassDeclaration */:
                        case 246 /* InterfaceDeclaration */:
                        case 247 /* TypeAliasDeclaration */:
                        case 244 /* FunctionDeclaration */:
                        case 248 /* EnumDeclaration */:
                        case 253 /* ImportEqualsDeclaration */:
                            // external module augmentation is always visible
                            if (ts.isExternalModuleAugmentation(node)) {
                                return true;
                            }
                            var parent = getDeclarationContainer(node);
                            // If the node is not exported or it is not ambient module element (except import declaration)
                            if (!(ts.getCombinedModifierFlags(node) & 1 /* Export */) &&
                                !(node.kind !== 253 /* ImportEqualsDeclaration */ && parent.kind !== 290 /* SourceFile */ && parent.flags & 8388608 /* Ambient */)) {
                                return isGlobalSourceFile(parent);
                            }
                            // Exported members/ambient module elements (exception import declaration) are visible if parent is visible
                            return isDeclarationVisible(parent);
                        case 159 /* PropertyDeclaration */:
                        case 158 /* PropertySignature */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                        case 161 /* MethodDeclaration */:
                        case 160 /* MethodSignature */:
                            if (ts.hasModifier(node, 8 /* Private */ | 16 /* Protected */)) {
                                // Private/protected properties/methods are not visible
                                return false;
                            }
                        // Public properties/methods are visible if its parents are visible, so:
                        // falls through
                        case 162 /* Constructor */:
                        case 166 /* ConstructSignature */:
                        case 165 /* CallSignature */:
                        case 167 /* IndexSignature */:
                        case 156 /* Parameter */:
                        case 250 /* ModuleBlock */:
                        case 170 /* FunctionType */:
                        case 171 /* ConstructorType */:
                        case 173 /* TypeLiteral */:
                        case 169 /* TypeReference */:
                        case 174 /* ArrayType */:
                        case 175 /* TupleType */:
                        case 178 /* UnionType */:
                        case 179 /* IntersectionType */:
                        case 182 /* ParenthesizedType */:
                            return isDeclarationVisible(node.parent);
                        // Default binding, import specifier and namespace import is visible
                        // only on demand so by default it is not visible
                        case 255 /* ImportClause */:
                        case 256 /* NamespaceImport */:
                        case 258 /* ImportSpecifier */:
                            return false;
                        // Type parameters are always visible
                        case 155 /* TypeParameter */:
                        // Source file and namespace export are always visible
                        // falls through
                        case 290 /* SourceFile */:
                        case 252 /* NamespaceExportDeclaration */:
                            return true;
                        // Export assignments do not create name bindings outside the module
                        case 259 /* ExportAssignment */:
                            return false;
                        default:
                            return false;
                    }
                }
            }
            function collectLinkedAliases(node, setVisibility) {
                var exportSymbol;
                if (node.parent && node.parent.kind === 259 /* ExportAssignment */) {
                    exportSymbol = resolveName(node, node.escapedText, 111551 /* Value */ | 788968 /* Type */ | 1920 /* Namespace */ | 2097152 /* Alias */, /*nameNotFoundMessage*/ undefined, node, /*isUse*/ false);
                }
                else if (node.parent.kind === 263 /* ExportSpecifier */) {
                    exportSymbol = getTargetOfExportSpecifier(node.parent, 111551 /* Value */ | 788968 /* Type */ | 1920 /* Namespace */ | 2097152 /* Alias */);
                }
                var result;
                var visited;
                if (exportSymbol) {
                    visited = ts.createMap();
                    visited.set("" + getSymbolId(exportSymbol), true);
                    buildVisibleNodeList(exportSymbol.declarations);
                }
                return result;
                function buildVisibleNodeList(declarations) {
                    ts.forEach(declarations, function (declaration) {
                        var resultNode = getAnyImportSyntax(declaration) || declaration;
                        if (setVisibility) {
                            getNodeLinks(declaration).isVisible = true;
                        }
                        else {
                            result = result || [];
                            ts.pushIfUnique(result, resultNode);
                        }
                        if (ts.isInternalModuleImportEqualsDeclaration(declaration)) {
                            // Add the referenced top container visible
                            var internalModuleReference = declaration.moduleReference;
                            var firstIdentifier = ts.getFirstIdentifier(internalModuleReference);
                            var importSymbol = resolveName(declaration, firstIdentifier.escapedText, 111551 /* Value */ | 788968 /* Type */ | 1920 /* Namespace */, undefined, undefined, /*isUse*/ false);
                            var id = importSymbol && "" + getSymbolId(importSymbol);
                            if (importSymbol && !visited.has(id)) {
                                visited.set(id, true);
                                buildVisibleNodeList(importSymbol.declarations);
                            }
                        }
                    });
                }
            }
            /**
             * Push an entry on the type resolution stack. If an entry with the given target and the given property name
             * is already on the stack, and no entries in between already have a type, then a circularity has occurred.
             * In this case, the result values of the existing entry and all entries pushed after it are changed to false,
             * and the value false is returned. Otherwise, the new entry is just pushed onto the stack, and true is returned.
             * In order to see if the same query has already been done before, the target object and the propertyName both
             * must match the one passed in.
             *
             * @param target The symbol, type, or signature whose type is being queried
             * @param propertyName The property name that should be used to query the target for its type
             */
            function pushTypeResolution(target, propertyName) {
                var resolutionCycleStartIndex = findResolutionCycleStartIndex(target, propertyName);
                if (resolutionCycleStartIndex >= 0) {
                    // A cycle was found
                    var length_2 = resolutionTargets.length;
                    for (var i = resolutionCycleStartIndex; i < length_2; i++) {
                        resolutionResults[i] = false;
                    }
                    return false;
                }
                resolutionTargets.push(target);
                resolutionResults.push(/*items*/ true);
                resolutionPropertyNames.push(propertyName);
                return true;
            }
            function findResolutionCycleStartIndex(target, propertyName) {
                for (var i = resolutionTargets.length - 1; i >= 0; i--) {
                    if (hasType(resolutionTargets[i], resolutionPropertyNames[i])) {
                        return -1;
                    }
                    if (resolutionTargets[i] === target && resolutionPropertyNames[i] === propertyName) {
                        return i;
                    }
                }
                return -1;
            }
            function hasType(target, propertyName) {
                switch (propertyName) {
                    case 0 /* Type */:
                        return !!getSymbolLinks(target).type;
                    case 5 /* EnumTagType */:
                        return !!(getNodeLinks(target).resolvedEnumType);
                    case 2 /* DeclaredType */:
                        return !!getSymbolLinks(target).declaredType;
                    case 1 /* ResolvedBaseConstructorType */:
                        return !!target.resolvedBaseConstructorType;
                    case 3 /* ResolvedReturnType */:
                        return !!target.resolvedReturnType;
                    case 4 /* ImmediateBaseConstraint */:
                        return !!target.immediateBaseConstraint;
                    case 6 /* ResolvedTypeArguments */:
                        return !!target.resolvedTypeArguments;
                }
                return ts.Debug.assertNever(propertyName);
            }
            /**
             * Pop an entry from the type resolution stack and return its associated result value. The result value will
             * be true if no circularities were detected, or false if a circularity was found.
             */
            function popTypeResolution() {
                resolutionTargets.pop();
                resolutionPropertyNames.pop();
                return resolutionResults.pop();
            }
            function getDeclarationContainer(node) {
                return ts.findAncestor(ts.getRootDeclaration(node), function (node) {
                    switch (node.kind) {
                        case 242 /* VariableDeclaration */:
                        case 243 /* VariableDeclarationList */:
                        case 258 /* ImportSpecifier */:
                        case 257 /* NamedImports */:
                        case 256 /* NamespaceImport */:
                        case 255 /* ImportClause */:
                            return false;
                        default:
                            return true;
                    }
                }).parent;
            }
            function getTypeOfPrototypeProperty(prototype) {
                // TypeScript 1.0 spec (April 2014): 8.4
                // Every class automatically contains a static property member named 'prototype',
                // the type of which is an instantiation of the class type with type Any supplied as a type argument for each type parameter.
                // It is an error to explicitly declare a static property member with the name 'prototype'.
                var classType = getDeclaredTypeOfSymbol(getParentOfSymbol(prototype));
                return classType.typeParameters ? createTypeReference(classType, ts.map(classType.typeParameters, function (_) { return anyType; })) : classType;
            }
            // Return the type of the given property in the given type, or undefined if no such property exists
            function getTypeOfPropertyOfType(type, name) {
                var prop = getPropertyOfType(type, name);
                return prop ? getTypeOfSymbol(prop) : undefined;
            }
            function getTypeOfPropertyOrIndexSignature(type, name) {
                return getTypeOfPropertyOfType(type, name) || isNumericLiteralName(name) && getIndexTypeOfType(type, 1 /* Number */) || getIndexTypeOfType(type, 0 /* String */) || unknownType;
            }
            function isTypeAny(type) {
                return type && (type.flags & 1 /* Any */) !== 0;
            }
            // Return the type of a binding element parent. We check SymbolLinks first to see if a type has been
            // assigned by contextual typing.
            function getTypeForBindingElementParent(node) {
                var symbol = getSymbolOfNode(node);
                return symbol && getSymbolLinks(symbol).type || getTypeForVariableLikeDeclaration(node, /*includeOptionality*/ false);
            }
            function isComputedNonLiteralName(name) {
                return name.kind === 154 /* ComputedPropertyName */ && !ts.isStringOrNumericLiteralLike(name.expression);
            }
            function getRestType(source, properties, symbol) {
                source = filterType(source, function (t) { return !(t.flags & 98304 /* Nullable */); });
                if (source.flags & 131072 /* Never */) {
                    return emptyObjectType;
                }
                if (source.flags & 1048576 /* Union */) {
                    return mapType(source, function (t) { return getRestType(t, properties, symbol); });
                }
                var omitKeyType = getUnionType(ts.map(properties, getLiteralTypeFromPropertyName));
                if (isGenericObjectType(source) || isGenericIndexType(omitKeyType)) {
                    if (omitKeyType.flags & 131072 /* Never */) {
                        return source;
                    }
                    var omitTypeAlias = getGlobalOmitSymbol();
                    if (!omitTypeAlias) {
                        return errorType;
                    }
                    return getTypeAliasInstantiation(omitTypeAlias, [source, omitKeyType]);
                }
                var members = ts.createSymbolTable();
                for (var _i = 0, _a = getPropertiesOfType(source); _i < _a.length; _i++) {
                    var prop = _a[_i];
                    if (!isTypeAssignableTo(getLiteralTypeFromProperty(prop, 8576 /* StringOrNumberLiteralOrUnique */), omitKeyType)
                        && !(ts.getDeclarationModifierFlagsFromSymbol(prop) & (8 /* Private */ | 16 /* Protected */))
                        && isSpreadableProperty(prop)) {
                        members.set(prop.escapedName, getSpreadSymbol(prop, /*readonly*/ false));
                    }
                }
                var stringIndexInfo = getIndexInfoOfType(source, 0 /* String */);
                var numberIndexInfo = getIndexInfoOfType(source, 1 /* Number */);
                var result = createAnonymousType(symbol, members, ts.emptyArray, ts.emptyArray, stringIndexInfo, numberIndexInfo);
                result.objectFlags |= 131072 /* ObjectRestType */;
                return result;
            }
            // Determine the control flow type associated with a destructuring declaration or assignment. The following
            // forms of destructuring are possible:
            //   let { x } = obj;  // BindingElement
            //   let [ x ] = obj;  // BindingElement
            //   { x } = obj;      // ShorthandPropertyAssignment
            //   { x: v } = obj;   // PropertyAssignment
            //   [ x ] = obj;      // Expression
            // We construct a synthetic element access expression corresponding to 'obj.x' such that the control
            // flow analyzer doesn't have to handle all the different syntactic forms.
            function getFlowTypeOfDestructuring(node, declaredType) {
                var reference = getSyntheticElementAccess(node);
                return reference ? getFlowTypeOfReference(reference, declaredType) : declaredType;
            }
            function getSyntheticElementAccess(node) {
                var parentAccess = getParentElementAccess(node);
                if (parentAccess && parentAccess.flowNode) {
                    var propName = getDestructuringPropertyName(node);
                    if (propName) {
                        var result = ts.createNode(195 /* ElementAccessExpression */, node.pos, node.end);
                        result.parent = node;
                        result.expression = parentAccess;
                        var literal = ts.createNode(10 /* StringLiteral */, node.pos, node.end);
                        literal.parent = result;
                        literal.text = propName;
                        result.argumentExpression = literal;
                        result.flowNode = parentAccess.flowNode;
                        return result;
                    }
                }
            }
            function getParentElementAccess(node) {
                var ancestor = node.parent.parent;
                switch (ancestor.kind) {
                    case 191 /* BindingElement */:
                    case 281 /* PropertyAssignment */:
                        return getSyntheticElementAccess(ancestor);
                    case 192 /* ArrayLiteralExpression */:
                        return getSyntheticElementAccess(node.parent);
                    case 242 /* VariableDeclaration */:
                        return ancestor.initializer;
                    case 209 /* BinaryExpression */:
                        return ancestor.right;
                }
            }
            function getDestructuringPropertyName(node) {
                var parent = node.parent;
                if (node.kind === 191 /* BindingElement */ && parent.kind === 189 /* ObjectBindingPattern */) {
                    return getLiteralPropertyNameText(node.propertyName || node.name);
                }
                if (node.kind === 281 /* PropertyAssignment */ || node.kind === 282 /* ShorthandPropertyAssignment */) {
                    return getLiteralPropertyNameText(node.name);
                }
                return "" + parent.elements.indexOf(node);
            }
            function getLiteralPropertyNameText(name) {
                var type = getLiteralTypeFromPropertyName(name);
                return type.flags & (128 /* StringLiteral */ | 256 /* NumberLiteral */) ? "" + type.value : undefined;
            }
            /** Return the inferred type for a binding element */
            function getTypeForBindingElement(declaration) {
                var pattern = declaration.parent;
                var parentType = getTypeForBindingElementParent(pattern.parent);
                // If no type or an any type was inferred for parent, infer that for the binding element
                if (!parentType || isTypeAny(parentType)) {
                    return parentType;
                }
                // Relax null check on ambient destructuring parameters, since the parameters have no implementation and are just documentation
                if (strictNullChecks && declaration.flags & 8388608 /* Ambient */ && ts.isParameterDeclaration(declaration)) {
                    parentType = getNonNullableType(parentType);
                }
                var type;
                if (pattern.kind === 189 /* ObjectBindingPattern */) {
                    if (declaration.dotDotDotToken) {
                        if (parentType.flags & 2 /* Unknown */ || !isValidSpreadType(parentType)) {
                            error(declaration, ts.Diagnostics.Rest_types_may_only_be_created_from_object_types);
                            return errorType;
                        }
                        var literalMembers = [];
                        for (var _i = 0, _a = pattern.elements; _i < _a.length; _i++) {
                            var element = _a[_i];
                            if (!element.dotDotDotToken) {
                                literalMembers.push(element.propertyName || element.name);
                            }
                        }
                        type = getRestType(parentType, literalMembers, declaration.symbol);
                    }
                    else {
                        // Use explicitly specified property name ({ p: xxx } form), or otherwise the implied name ({ p } form)
                        var name = declaration.propertyName || declaration.name;
                        var indexType = getLiteralTypeFromPropertyName(name);
                        var declaredType = getConstraintForLocation(getIndexedAccessType(parentType, indexType, name), declaration.name);
                        type = getFlowTypeOfDestructuring(declaration, declaredType);
                    }
                }
                else {
                    // This elementType will be used if the specific property corresponding to this index is not
                    // present (aka the tuple element property). This call also checks that the parentType is in
                    // fact an iterable or array (depending on target language).
                    var elementType = checkIteratedTypeOrElementType(65 /* Destructuring */, parentType, undefinedType, pattern);
                    var index_1 = pattern.elements.indexOf(declaration);
                    if (declaration.dotDotDotToken) {
                        // If the parent is a tuple type, the rest element has a tuple type of the
                        // remaining tuple element types. Otherwise, the rest element has an array type with same
                        // element type as the parent type.
                        type = everyType(parentType, isTupleType) ?
                            mapType(parentType, function (t) { return sliceTupleType(t, index_1); }) :
                            createArrayType(elementType);
                    }
                    else if (isArrayLikeType(parentType)) {
                        var indexType = getLiteralType(index_1);
                        var accessFlags = hasDefaultValue(declaration) ? 8 /* NoTupleBoundsCheck */ : 0;
                        var declaredType = getConstraintForLocation(getIndexedAccessTypeOrUndefined(parentType, indexType, declaration.name, accessFlags) || errorType, declaration.name);
                        type = getFlowTypeOfDestructuring(declaration, declaredType);
                    }
                    else {
                        type = elementType;
                    }
                }
                if (!declaration.initializer) {
                    return type;
                }
                if (ts.getEffectiveTypeAnnotationNode(ts.walkUpBindingElementsAndPatterns(declaration))) {
                    // In strict null checking mode, if a default value of a non-undefined type is specified, remove
                    // undefined from the final type.
                    return strictNullChecks && !(getFalsyFlags(checkDeclarationInitializer(declaration)) & 32768 /* Undefined */) ?
                        getTypeWithFacts(type, 524288 /* NEUndefined */) :
                        type;
                }
                return widenTypeInferredFromInitializer(declaration, getUnionType([getTypeWithFacts(type, 524288 /* NEUndefined */), checkDeclarationInitializer(declaration)], 2 /* Subtype */));
            }
            function getTypeForDeclarationFromJSDocComment(declaration) {
                var jsdocType = ts.getJSDocType(declaration);
                if (jsdocType) {
                    return getTypeFromTypeNode(jsdocType);
                }
                return undefined;
            }
            function isNullOrUndefined(node) {
                var expr = ts.skipParentheses(node);
                return expr.kind === 100 /* NullKeyword */ || expr.kind === 75 /* Identifier */ && getResolvedSymbol(expr) === undefinedSymbol;
            }
            function isEmptyArrayLiteral(node) {
                var expr = ts.skipParentheses(node);
                return expr.kind === 192 /* ArrayLiteralExpression */ && expr.elements.length === 0;
            }
            function addOptionality(type, optional) {
                if (optional === void 0) {
                    optional = true;
                }
                return strictNullChecks && optional ? getOptionalType(type) : type;
            }
            // Return the inferred type for a variable, parameter, or property declaration
            function getTypeForVariableLikeDeclaration(declaration, includeOptionality) {
                // A variable declared in a for..in statement is of type string, or of type keyof T when the
                // right hand expression is of a type parameter type.
                if (ts.isVariableDeclaration(declaration) && declaration.parent.parent.kind === 231 /* ForInStatement */) {
                    var indexType = getIndexType(getNonNullableTypeIfNeeded(checkExpression(declaration.parent.parent.expression)));
                    return indexType.flags & (262144 /* TypeParameter */ | 4194304 /* Index */) ? getExtractStringType(indexType) : stringType;
                }
                if (ts.isVariableDeclaration(declaration) && declaration.parent.parent.kind === 232 /* ForOfStatement */) {
                    // checkRightHandSideOfForOf will return undefined if the for-of expression type was
                    // missing properties/signatures required to get its iteratedType (like
                    // [Symbol.iterator] or next). This may be because we accessed properties from anyType,
                    // or it may have led to an error inside getElementTypeOfIterable.
                    var forOfStatement = declaration.parent.parent;
                    return checkRightHandSideOfForOf(forOfStatement.expression, forOfStatement.awaitModifier) || anyType;
                }
                if (ts.isBindingPattern(declaration.parent)) {
                    return getTypeForBindingElement(declaration);
                }
                var isOptional = includeOptionality && (ts.isParameter(declaration) && isJSDocOptionalParameter(declaration)
                    || !ts.isBindingElement(declaration) && !ts.isVariableDeclaration(declaration) && !!declaration.questionToken);
                // Use type from type annotation if one is present
                var declaredType = tryGetTypeFromEffectiveTypeNode(declaration);
                if (declaredType) {
                    return addOptionality(declaredType, isOptional);
                }
                if ((noImplicitAny || ts.isInJSFile(declaration)) &&
                    declaration.kind === 242 /* VariableDeclaration */ && !ts.isBindingPattern(declaration.name) &&
                    !(ts.getCombinedModifierFlags(declaration) & 1 /* Export */) && !(declaration.flags & 8388608 /* Ambient */)) {
                    // If --noImplicitAny is on or the declaration is in a Javascript file,
                    // use control flow tracked 'any' type for non-ambient, non-exported var or let variables with no
                    // initializer or a 'null' or 'undefined' initializer.
                    if (!(ts.getCombinedNodeFlags(declaration) & 2 /* Const */) && (!declaration.initializer || isNullOrUndefined(declaration.initializer))) {
                        return autoType;
                    }
                    // Use control flow tracked 'any[]' type for non-ambient, non-exported variables with an empty array
                    // literal initializer.
                    if (declaration.initializer && isEmptyArrayLiteral(declaration.initializer)) {
                        return autoArrayType;
                    }
                }
                if (declaration.kind === 156 /* Parameter */) {
                    var func = declaration.parent;
                    // For a parameter of a set accessor, use the type of the get accessor if one is present
                    if (func.kind === 164 /* SetAccessor */ && !hasNonBindableDynamicName(func)) {
                        var getter = ts.getDeclarationOfKind(getSymbolOfNode(declaration.parent), 163 /* GetAccessor */);
                        if (getter) {
                            var getterSignature = getSignatureFromDeclaration(getter);
                            var thisParameter = getAccessorThisParameter(func);
                            if (thisParameter && declaration === thisParameter) {
                                // Use the type from the *getter*
                                ts.Debug.assert(!thisParameter.type);
                                return getTypeOfSymbol(getterSignature.thisParameter);
                            }
                            return getReturnTypeOfSignature(getterSignature);
                        }
                    }
                    if (ts.isInJSFile(declaration)) {
                        var typeTag = ts.getJSDocType(func);
                        if (typeTag && ts.isFunctionTypeNode(typeTag)) {
                            return getTypeAtPosition(getSignatureFromDeclaration(typeTag), func.parameters.indexOf(declaration));
                        }
                    }
                    // Use contextual parameter type if one is available
                    var type = declaration.symbol.escapedName === "this" /* This */ ? getContextualThisParameterType(func) : getContextuallyTypedParameterType(declaration);
                    if (type) {
                        return addOptionality(type, isOptional);
                    }
                }
                else if (ts.isInJSFile(declaration)) {
                    var containerObjectType = getJSContainerObjectType(declaration, getSymbolOfNode(declaration), ts.getDeclaredExpandoInitializer(declaration));
                    if (containerObjectType) {
                        return containerObjectType;
                    }
                }
                // Use the type of the initializer expression if one is present and the declaration is
                // not a parameter of a contextually typed function
                if (declaration.initializer) {
                    var type = widenTypeInferredFromInitializer(declaration, checkDeclarationInitializer(declaration));
                    return addOptionality(type, isOptional);
                }
                if (ts.isJsxAttribute(declaration)) {
                    // if JSX attribute doesn't have initializer, by default the attribute will have boolean value of true.
                    // I.e <Elem attr /> is sugar for <Elem attr={true} />
                    return trueType;
                }
                // If the declaration specifies a binding pattern and is not a parameter of a contextually
                // typed function, use the type implied by the binding pattern
                if (ts.isBindingPattern(declaration.name)) {
                    return getTypeFromBindingPattern(declaration.name, /*includePatternInType*/ false, /*reportErrors*/ true);
                }
                // No type specified and nothing can be inferred
                return undefined;
            }
            function getWidenedTypeForAssignmentDeclaration(symbol, resolvedSymbol) {
                // function/class/{} initializers are themselves containers, so they won't merge in the same way as other initializers
                var container = ts.getAssignedExpandoInitializer(symbol.valueDeclaration);
                if (container) {
                    var tag = ts.getJSDocTypeTag(container);
                    if (tag && tag.typeExpression) {
                        return getTypeFromTypeNode(tag.typeExpression);
                    }
                    var containerObjectType = getJSContainerObjectType(symbol.valueDeclaration, symbol, container);
                    return containerObjectType || getWidenedLiteralType(checkExpressionCached(container));
                }
                var definedInConstructor = false;
                var definedInMethod = false;
                var jsdocType;
                var types;
                for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                    var declaration = _a[_i];
                    var expression = (ts.isBinaryExpression(declaration) || ts.isCallExpression(declaration)) ? declaration :
                        ts.isAccessExpression(declaration) ? ts.isBinaryExpression(declaration.parent) ? declaration.parent : declaration :
                            undefined;
                    if (!expression) {
                        continue; // Non-assignment declaration merged in (eg, an Identifier to mark the thing as a namespace) - skip over it and pull type info from elsewhere
                    }
                    var kind = ts.isAccessExpression(expression)
                        ? ts.getAssignmentDeclarationPropertyAccessKind(expression)
                        : ts.getAssignmentDeclarationKind(expression);
                    if (kind === 4 /* ThisProperty */) {
                        if (isDeclarationInConstructor(expression)) {
                            definedInConstructor = true;
                        }
                        else {
                            definedInMethod = true;
                        }
                    }
                    if (!ts.isCallExpression(expression)) {
                        jsdocType = getAnnotatedTypeForAssignmentDeclaration(jsdocType, expression, symbol, declaration);
                    }
                    if (!jsdocType) {
                        (types || (types = [])).push((ts.isBinaryExpression(expression) || ts.isCallExpression(expression)) ? getInitializerTypeFromAssignmentDeclaration(symbol, resolvedSymbol, expression, kind) : neverType);
                    }
                }
                var type = jsdocType;
                if (!type) {
                    if (!ts.length(types)) {
                        return errorType; // No types from any declarations :(
                    }
                    var constructorTypes = definedInConstructor ? getConstructorDefinedThisAssignmentTypes(types, symbol.declarations) : undefined;
                    // use only the constructor types unless they were only assigned null | undefined (including widening variants)
                    if (definedInMethod) {
                        var propType = getTypeOfAssignmentDeclarationPropertyOfBaseType(symbol);
                        if (propType) {
                            (constructorTypes || (constructorTypes = [])).push(propType);
                            definedInConstructor = true;
                        }
                    }
                    var sourceTypes = ts.some(constructorTypes, function (t) { return !!(t.flags & ~98304 /* Nullable */); }) ? constructorTypes : types; // TODO: GH#18217
                    type = getUnionType(sourceTypes, 2 /* Subtype */);
                }
                var widened = getWidenedType(addOptionality(type, definedInMethod && !definedInConstructor));
                if (filterType(widened, function (t) { return !!(t.flags & ~98304 /* Nullable */); }) === neverType) {
                    reportImplicitAny(symbol.valueDeclaration, anyType);
                    return anyType;
                }
                return widened;
            }
            function getJSContainerObjectType(decl, symbol, init) {
                if (!ts.isInJSFile(decl) || !init || !ts.isObjectLiteralExpression(init) || init.properties.length) {
                    return undefined;
                }
                var exports = ts.createSymbolTable();
                while (ts.isBinaryExpression(decl) || ts.isPropertyAccessExpression(decl)) {
                    var s_2 = getSymbolOfNode(decl);
                    if (s_2 && ts.hasEntries(s_2.exports)) {
                        mergeSymbolTable(exports, s_2.exports);
                    }
                    decl = ts.isBinaryExpression(decl) ? decl.parent : decl.parent.parent;
                }
                var s = getSymbolOfNode(decl);
                if (s && ts.hasEntries(s.exports)) {
                    mergeSymbolTable(exports, s.exports);
                }
                var type = createAnonymousType(symbol, exports, ts.emptyArray, ts.emptyArray, undefined, undefined);
                type.objectFlags |= 16384 /* JSLiteral */;
                return type;
            }
            function getAnnotatedTypeForAssignmentDeclaration(declaredType, expression, symbol, declaration) {
                var typeNode = ts.getEffectiveTypeAnnotationNode(expression.parent);
                if (typeNode) {
                    var type = getWidenedType(getTypeFromTypeNode(typeNode));
                    if (!declaredType) {
                        return type;
                    }
                    else if (declaredType !== errorType && type !== errorType && !isTypeIdenticalTo(declaredType, type)) {
                        errorNextVariableOrPropertyDeclarationMustHaveSameType(/*firstDeclaration*/ undefined, declaredType, declaration, type);
                    }
                }
                if (symbol.parent) {
                    var typeNode_2 = ts.getEffectiveTypeAnnotationNode(symbol.parent.valueDeclaration);
                    if (typeNode_2) {
                        return getTypeOfPropertyOfType(getTypeFromTypeNode(typeNode_2), symbol.escapedName);
                    }
                }
                return declaredType;
            }
            /** If we don't have an explicit JSDoc type, get the type from the initializer. */
            function getInitializerTypeFromAssignmentDeclaration(symbol, resolvedSymbol, expression, kind) {
                if (ts.isCallExpression(expression)) {
                    if (resolvedSymbol) {
                        return getTypeOfSymbol(resolvedSymbol); // This shouldn't happen except under some hopefully forbidden merges of export assignments and object define assignments
                    }
                    var objectLitType = checkExpressionCached(expression.arguments[2]);
                    var valueType = getTypeOfPropertyOfType(objectLitType, "value");
                    if (valueType) {
                        return valueType;
                    }
                    var getFunc = getTypeOfPropertyOfType(objectLitType, "get");
                    if (getFunc) {
                        var getSig = getSingleCallSignature(getFunc);
                        if (getSig) {
                            return getReturnTypeOfSignature(getSig);
                        }
                    }
                    var setFunc = getTypeOfPropertyOfType(objectLitType, "set");
                    if (setFunc) {
                        var setSig = getSingleCallSignature(setFunc);
                        if (setSig) {
                            return getTypeOfFirstParameterOfSignature(setSig);
                        }
                    }
                    return anyType;
                }
                var type = resolvedSymbol ? getTypeOfSymbol(resolvedSymbol) : getWidenedLiteralType(checkExpressionCached(expression.right));
                if (type.flags & 524288 /* Object */ &&
                    kind === 2 /* ModuleExports */ &&
                    symbol.escapedName === "export=" /* ExportEquals */) {
                    var exportedType_1 = resolveStructuredTypeMembers(type);
                    var members_4 = ts.createSymbolTable();
                    ts.copyEntries(exportedType_1.members, members_4);
                    if (resolvedSymbol && !resolvedSymbol.exports) {
                        resolvedSymbol.exports = ts.createSymbolTable();
                    }
                    (resolvedSymbol || symbol).exports.forEach(function (s, name) {
                        if (members_4.has(name)) {
                            var exportedMember = exportedType_1.members.get(name);
                            var union = createSymbol(s.flags | exportedMember.flags, name);
                            union.type = getUnionType([getTypeOfSymbol(s), getTypeOfSymbol(exportedMember)]);
                            members_4.set(name, union);
                        }
                        else {
                            members_4.set(name, s);
                        }
                    });
                    var result = createAnonymousType(exportedType_1.symbol, members_4, exportedType_1.callSignatures, exportedType_1.constructSignatures, exportedType_1.stringIndexInfo, exportedType_1.numberIndexInfo);
                    result.objectFlags |= (ts.getObjectFlags(type) & 16384 /* JSLiteral */); // Propagate JSLiteral flag
                    return result;
                }
                if (isEmptyArrayLiteralType(type)) {
                    reportImplicitAny(expression, anyArrayType);
                    return anyArrayType;
                }
                return type;
            }
            function isDeclarationInConstructor(expression) {
                var thisContainer = ts.getThisContainer(expression, /*includeArrowFunctions*/ false);
                // Properties defined in a constructor (or base constructor, or javascript constructor function) don't get undefined added.
                // Function expressions that are assigned to the prototype count as methods.
                return thisContainer.kind === 162 /* Constructor */ ||
                    thisContainer.kind === 244 /* FunctionDeclaration */ ||
                    (thisContainer.kind === 201 /* FunctionExpression */ && !ts.isPrototypePropertyAssignment(thisContainer.parent));
            }
            function getConstructorDefinedThisAssignmentTypes(types, declarations) {
                ts.Debug.assert(types.length === declarations.length);
                return types.filter(function (_, i) {
                    var declaration = declarations[i];
                    var expression = ts.isBinaryExpression(declaration) ? declaration :
                        ts.isBinaryExpression(declaration.parent) ? declaration.parent : undefined;
                    return expression && isDeclarationInConstructor(expression);
                });
            }
            /** check for definition in base class if any declaration is in a class */
            function getTypeOfAssignmentDeclarationPropertyOfBaseType(property) {
                var parentDeclaration = ts.forEach(property.declarations, function (d) {
                    var parent = ts.getThisContainer(d, /*includeArrowFunctions*/ false).parent;
                    return ts.isClassLike(parent) && parent;
                });
                if (parentDeclaration) {
                    var classType = getDeclaredTypeOfSymbol(getSymbolOfNode(parentDeclaration));
                    var baseClassType = classType && getBaseTypes(classType)[0];
                    if (baseClassType) {
                        return getTypeOfPropertyOfType(baseClassType, property.escapedName);
                    }
                }
            }
            // Return the type implied by a binding pattern element. This is the type of the initializer of the element if
            // one is present. Otherwise, if the element is itself a binding pattern, it is the type implied by the binding
            // pattern. Otherwise, it is the type any.
            function getTypeFromBindingElement(element, includePatternInType, reportErrors) {
                if (element.initializer) {
                    // The type implied by a binding pattern is independent of context, so we check the initializer with no
                    // contextual type or, if the element itself is a binding pattern, with the type implied by that binding
                    // pattern.
                    var contextualType = ts.isBindingPattern(element.name) ? getTypeFromBindingPattern(element.name, /*includePatternInType*/ true, /*reportErrors*/ false) : unknownType;
                    return addOptionality(widenTypeInferredFromInitializer(element, checkDeclarationInitializer(element, contextualType)));
                }
                if (ts.isBindingPattern(element.name)) {
                    return getTypeFromBindingPattern(element.name, includePatternInType, reportErrors);
                }
                if (reportErrors && !declarationBelongsToPrivateAmbientMember(element)) {
                    reportImplicitAny(element, anyType);
                }
                // When we're including the pattern in the type (an indication we're obtaining a contextual type), we
                // use the non-inferrable any type. Inference will never directly infer this type, but it is possible
                // to infer a type that contains it, e.g. for a binding pattern like [foo] or { foo }. In such cases,
                // widening of the binding pattern type substitutes a regular any for the non-inferrable any.
                return includePatternInType ? nonInferrableAnyType : anyType;
            }
            // Return the type implied by an object binding pattern
            function getTypeFromObjectBindingPattern(pattern, includePatternInType, reportErrors) {
                var members = ts.createSymbolTable();
                var stringIndexInfo;
                var objectFlags = 128 /* ObjectLiteral */ | 1048576 /* ContainsObjectOrArrayLiteral */;
                ts.forEach(pattern.elements, function (e) {
                    var name = e.propertyName || e.name;
                    if (e.dotDotDotToken) {
                        stringIndexInfo = createIndexInfo(anyType, /*isReadonly*/ false);
                        return;
                    }
                    var exprType = getLiteralTypeFromPropertyName(name);
                    if (!isTypeUsableAsPropertyName(exprType)) {
                        // do not include computed properties in the implied type
                        objectFlags |= 512 /* ObjectLiteralPatternWithComputedProperties */;
                        return;
                    }
                    var text = getPropertyNameFromType(exprType);
                    var flags = 4 /* Property */ | (e.initializer ? 16777216 /* Optional */ : 0);
                    var symbol = createSymbol(flags, text);
                    symbol.type = getTypeFromBindingElement(e, includePatternInType, reportErrors);
                    symbol.bindingElement = e;
                    members.set(symbol.escapedName, symbol);
                });
                var result = createAnonymousType(undefined, members, ts.emptyArray, ts.emptyArray, stringIndexInfo, undefined);
                result.objectFlags |= objectFlags;
                if (includePatternInType) {
                    result.pattern = pattern;
                    result.objectFlags |= 1048576 /* ContainsObjectOrArrayLiteral */;
                }
                return result;
            }
            // Return the type implied by an array binding pattern
            function getTypeFromArrayBindingPattern(pattern, includePatternInType, reportErrors) {
                var elements = pattern.elements;
                var lastElement = ts.lastOrUndefined(elements);
                var hasRestElement = !!(lastElement && lastElement.kind === 191 /* BindingElement */ && lastElement.dotDotDotToken);
                if (elements.length === 0 || elements.length === 1 && hasRestElement) {
                    return languageVersion >= 2 /* ES2015 */ ? createIterableType(anyType) : anyArrayType;
                }
                var elementTypes = ts.map(elements, function (e) { return ts.isOmittedExpression(e) ? anyType : getTypeFromBindingElement(e, includePatternInType, reportErrors); });
                var minLength = ts.findLastIndex(elements, function (e) { return !ts.isOmittedExpression(e) && !hasDefaultValue(e); }, elements.length - (hasRestElement ? 2 : 1)) + 1;
                var result = createTupleType(elementTypes, minLength, hasRestElement);
                if (includePatternInType) {
                    result = cloneTypeReference(result);
                    result.pattern = pattern;
                    result.objectFlags |= 1048576 /* ContainsObjectOrArrayLiteral */;
                }
                return result;
            }
            // Return the type implied by a binding pattern. This is the type implied purely by the binding pattern itself
            // and without regard to its context (i.e. without regard any type annotation or initializer associated with the
            // declaration in which the binding pattern is contained). For example, the implied type of [x, y] is [any, any]
            // and the implied type of { x, y: z = 1 } is { x: any; y: number; }. The type implied by a binding pattern is
            // used as the contextual type of an initializer associated with the binding pattern. Also, for a destructuring
            // parameter with no type annotation or initializer, the type implied by the binding pattern becomes the type of
            // the parameter.
            function getTypeFromBindingPattern(pattern, includePatternInType, reportErrors) {
                if (includePatternInType === void 0) {
                    includePatternInType = false;
                }
                if (reportErrors === void 0) {
                    reportErrors = false;
                }
                return pattern.kind === 189 /* ObjectBindingPattern */
                    ? getTypeFromObjectBindingPattern(pattern, includePatternInType, reportErrors)
                    : getTypeFromArrayBindingPattern(pattern, includePatternInType, reportErrors);
            }
            // Return the type associated with a variable, parameter, or property declaration. In the simple case this is the type
            // specified in a type annotation or inferred from an initializer. However, in the case of a destructuring declaration it
            // is a bit more involved. For example:
            //
            //   var [x, s = ""] = [1, "one"];
            //
            // Here, the array literal [1, "one"] is contextually typed by the type [any, string], which is the implied type of the
            // binding pattern [x, s = ""]. Because the contextual type is a tuple type, the resulting type of [1, "one"] is the
            // tuple type [number, string]. Thus, the type inferred for 'x' is number and the type inferred for 's' is string.
            function getWidenedTypeForVariableLikeDeclaration(declaration, reportErrors) {
                return widenTypeForVariableLikeDeclaration(getTypeForVariableLikeDeclaration(declaration, /*includeOptionality*/ true), declaration, reportErrors);
            }
            function widenTypeForVariableLikeDeclaration(type, declaration, reportErrors) {
                if (type) {
                    if (reportErrors) {
                        reportErrorsFromWidening(declaration, type);
                    }
                    // always widen a 'unique symbol' type if the type was created for a different declaration.
                    if (type.flags & 8192 /* UniqueESSymbol */ && (ts.isBindingElement(declaration) || !declaration.type) && type.symbol !== getSymbolOfNode(declaration)) {
                        type = esSymbolType;
                    }
                    return getWidenedType(type);
                }
                // Rest parameters default to type any[], other parameters default to type any
                type = ts.isParameter(declaration) && declaration.dotDotDotToken ? anyArrayType : anyType;
                // Report implicit any errors unless this is a private property within an ambient declaration
                if (reportErrors) {
                    if (!declarationBelongsToPrivateAmbientMember(declaration)) {
                        reportImplicitAny(declaration, type);
                    }
                }
                return type;
            }
            function declarationBelongsToPrivateAmbientMember(declaration) {
                var root = ts.getRootDeclaration(declaration);
                var memberDeclaration = root.kind === 156 /* Parameter */ ? root.parent : root;
                return isPrivateWithinAmbient(memberDeclaration);
            }
            function tryGetTypeFromEffectiveTypeNode(declaration) {
                var typeNode = ts.getEffectiveTypeAnnotationNode(declaration);
                if (typeNode) {
                    return getTypeFromTypeNode(typeNode);
                }
            }
            function getTypeOfVariableOrParameterOrProperty(symbol) {
                var links = getSymbolLinks(symbol);
                if (!links.type) {
                    var type = getTypeOfVariableOrParameterOrPropertyWorker(symbol);
                    // For a contextually typed parameter it is possible that a type has already
                    // been assigned (in assignTypeToParameterAndFixTypeParameters), and we want
                    // to preserve this type.
                    if (!links.type) {
                        links.type = type;
                    }
                }
                return links.type;
            }
            function getTypeOfVariableOrParameterOrPropertyWorker(symbol) {
                // Handle prototype property
                if (symbol.flags & 4194304 /* Prototype */) {
                    return getTypeOfPrototypeProperty(symbol);
                }
                // CommonsJS require and module both have type any.
                if (symbol === requireSymbol) {
                    return anyType;
                }
                if (symbol.flags & 134217728 /* ModuleExports */) {
                    var fileSymbol = getSymbolOfNode(ts.getSourceFileOfNode(symbol.valueDeclaration));
                    var members = ts.createSymbolTable();
                    members.set("exports", fileSymbol);
                    return createAnonymousType(symbol, members, ts.emptyArray, ts.emptyArray, undefined, undefined);
                }
                // Handle catch clause variables
                var declaration = symbol.valueDeclaration;
                if (ts.isCatchClauseVariableDeclarationOrBindingElement(declaration)) {
                    return anyType;
                }
                // Handle export default expressions
                if (ts.isSourceFile(declaration) && ts.isJsonSourceFile(declaration)) {
                    if (!declaration.statements.length) {
                        return emptyObjectType;
                    }
                    return getWidenedType(getWidenedLiteralType(checkExpression(declaration.statements[0].expression)));
                }
                // Handle variable, parameter or property
                if (!pushTypeResolution(symbol, 0 /* Type */)) {
                    // Symbol is property of some kind that is merged with something - should use `getTypeOfFuncClassEnumModule` and not `getTypeOfVariableOrParameterOrProperty`
                    if (symbol.flags & 512 /* ValueModule */ && !(symbol.flags & 67108864 /* Assignment */)) {
                        return getTypeOfFuncClassEnumModule(symbol);
                    }
                    return reportCircularityError(symbol);
                }
                var type;
                if (declaration.kind === 259 /* ExportAssignment */) {
                    type = widenTypeForVariableLikeDeclaration(checkExpressionCached(declaration.expression), declaration);
                }
                else if (ts.isBinaryExpression(declaration) ||
                    (ts.isInJSFile(declaration) &&
                        (ts.isCallExpression(declaration) || (ts.isPropertyAccessExpression(declaration) || ts.isBindableStaticElementAccessExpression(declaration)) && ts.isBinaryExpression(declaration.parent)))) {
                    type = getWidenedTypeForAssignmentDeclaration(symbol);
                }
                else if (ts.isJSDocPropertyLikeTag(declaration)
                    || ts.isPropertyAccessExpression(declaration)
                    || ts.isElementAccessExpression(declaration)
                    || ts.isIdentifier(declaration)
                    || ts.isStringLiteralLike(declaration)
                    || ts.isNumericLiteral(declaration)
                    || ts.isClassDeclaration(declaration)
                    || ts.isFunctionDeclaration(declaration)
                    || (ts.isMethodDeclaration(declaration) && !ts.isObjectLiteralMethod(declaration))
                    || ts.isMethodSignature(declaration)
                    || ts.isSourceFile(declaration)) {
                    // Symbol is property of some kind that is merged with something - should use `getTypeOfFuncClassEnumModule` and not `getTypeOfVariableOrParameterOrProperty`
                    if (symbol.flags & (16 /* Function */ | 8192 /* Method */ | 32 /* Class */ | 384 /* Enum */ | 512 /* ValueModule */)) {
                        return getTypeOfFuncClassEnumModule(symbol);
                    }
                    type = ts.isBinaryExpression(declaration.parent) ?
                        getWidenedTypeForAssignmentDeclaration(symbol) :
                        tryGetTypeFromEffectiveTypeNode(declaration) || anyType;
                }
                else if (ts.isPropertyAssignment(declaration)) {
                    type = tryGetTypeFromEffectiveTypeNode(declaration) || checkPropertyAssignment(declaration);
                }
                else if (ts.isJsxAttribute(declaration)) {
                    type = tryGetTypeFromEffectiveTypeNode(declaration) || checkJsxAttribute(declaration);
                }
                else if (ts.isShorthandPropertyAssignment(declaration)) {
                    type = tryGetTypeFromEffectiveTypeNode(declaration) || checkExpressionForMutableLocation(declaration.name, 0 /* Normal */);
                }
                else if (ts.isObjectLiteralMethod(declaration)) {
                    type = tryGetTypeFromEffectiveTypeNode(declaration) || checkObjectLiteralMethod(declaration, 0 /* Normal */);
                }
                else if (ts.isParameter(declaration)
                    || ts.isPropertyDeclaration(declaration)
                    || ts.isPropertySignature(declaration)
                    || ts.isVariableDeclaration(declaration)
                    || ts.isBindingElement(declaration)) {
                    type = getWidenedTypeForVariableLikeDeclaration(declaration, /*includeOptionality*/ true);
                }
                // getTypeOfSymbol dispatches some JS merges incorrectly because their symbol flags are not mutually exclusive.
                // Re-dispatch based on valueDeclaration.kind instead.
                else if (ts.isEnumDeclaration(declaration)) {
                    type = getTypeOfFuncClassEnumModule(symbol);
                }
                else if (ts.isEnumMember(declaration)) {
                    type = getTypeOfEnumMember(symbol);
                }
                else if (ts.isAccessor(declaration)) {
                    type = resolveTypeOfAccessors(symbol);
                }
                else {
                    return ts.Debug.fail("Unhandled declaration kind! " + ts.Debug.formatSyntaxKind(declaration.kind) + " for " + ts.Debug.formatSymbol(symbol));
                }
                if (!popTypeResolution()) {
                    // Symbol is property of some kind that is merged with something - should use `getTypeOfFuncClassEnumModule` and not `getTypeOfVariableOrParameterOrProperty`
                    if (symbol.flags & 512 /* ValueModule */ && !(symbol.flags & 67108864 /* Assignment */)) {
                        return getTypeOfFuncClassEnumModule(symbol);
                    }
                    return reportCircularityError(symbol);
                }
                return type;
            }
            function getAnnotatedAccessorTypeNode(accessor) {
                if (accessor) {
                    if (accessor.kind === 163 /* GetAccessor */) {
                        var getterTypeAnnotation = ts.getEffectiveReturnTypeNode(accessor);
                        return getterTypeAnnotation;
                    }
                    else {
                        var setterTypeAnnotation = ts.getEffectiveSetAccessorTypeAnnotationNode(accessor);
                        return setterTypeAnnotation;
                    }
                }
                return undefined;
            }
            function getAnnotatedAccessorType(accessor) {
                var node = getAnnotatedAccessorTypeNode(accessor);
                return node && getTypeFromTypeNode(node);
            }
            function getAnnotatedAccessorThisParameter(accessor) {
                var parameter = getAccessorThisParameter(accessor);
                return parameter && parameter.symbol;
            }
            function getThisTypeOfDeclaration(declaration) {
                return getThisTypeOfSignature(getSignatureFromDeclaration(declaration));
            }
            function getTypeOfAccessors(symbol) {
                var links = getSymbolLinks(symbol);
                return links.type || (links.type = getTypeOfAccessorsWorker(symbol));
            }
            function getTypeOfAccessorsWorker(symbol) {
                if (!pushTypeResolution(symbol, 0 /* Type */)) {
                    return errorType;
                }
                var type = resolveTypeOfAccessors(symbol);
                if (!popTypeResolution()) {
                    type = anyType;
                    if (noImplicitAny) {
                        var getter = ts.getDeclarationOfKind(symbol, 163 /* GetAccessor */);
                        error(getter, ts.Diagnostics._0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions, symbolToString(symbol));
                    }
                }
                return type;
            }
            function resolveTypeOfAccessors(symbol) {
                var getter = ts.getDeclarationOfKind(symbol, 163 /* GetAccessor */);
                var setter = ts.getDeclarationOfKind(symbol, 164 /* SetAccessor */);
                if (getter && ts.isInJSFile(getter)) {
                    var jsDocType = getTypeForDeclarationFromJSDocComment(getter);
                    if (jsDocType) {
                        return jsDocType;
                    }
                }
                // First try to see if the user specified a return type on the get-accessor.
                var getterReturnType = getAnnotatedAccessorType(getter);
                if (getterReturnType) {
                    return getterReturnType;
                }
                else {
                    // If the user didn't specify a return type, try to use the set-accessor's parameter type.
                    var setterParameterType = getAnnotatedAccessorType(setter);
                    if (setterParameterType) {
                        return setterParameterType;
                    }
                    else {
                        // If there are no specified types, try to infer it from the body of the get accessor if it exists.
                        if (getter && getter.body) {
                            return getReturnTypeFromBody(getter);
                        }
                        // Otherwise, fall back to 'any'.
                        else {
                            if (setter) {
                                if (!isPrivateWithinAmbient(setter)) {
                                    errorOrSuggestion(noImplicitAny, setter, ts.Diagnostics.Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation, symbolToString(symbol));
                                }
                            }
                            else {
                                ts.Debug.assert(!!getter, "there must exist a getter as we are current checking either setter or getter in this function");
                                if (!isPrivateWithinAmbient(getter)) {
                                    errorOrSuggestion(noImplicitAny, getter, ts.Diagnostics.Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation, symbolToString(symbol));
                                }
                            }
                            return anyType;
                        }
                    }
                }
            }
            function getBaseTypeVariableOfClass(symbol) {
                var baseConstructorType = getBaseConstructorTypeOfClass(getDeclaredTypeOfClassOrInterface(symbol));
                return baseConstructorType.flags & 8650752 /* TypeVariable */ ? baseConstructorType :
                    baseConstructorType.flags & 2097152 /* Intersection */ ? ts.find(baseConstructorType.types, function (t) { return !!(t.flags & 8650752 /* TypeVariable */); }) :
                        undefined;
            }
            function getTypeOfFuncClassEnumModule(symbol) {
                var links = getSymbolLinks(symbol);
                var originalLinks = links;
                if (!links.type) {
                    var jsDeclaration = ts.getDeclarationOfExpando(symbol.valueDeclaration);
                    if (jsDeclaration) {
                        var merged = mergeJSSymbols(symbol, getSymbolOfNode(jsDeclaration));
                        if (merged) {
                            // note:we overwrite links because we just cloned the symbol
                            symbol = links = merged;
                        }
                    }
                    originalLinks.type = links.type = getTypeOfFuncClassEnumModuleWorker(symbol);
                }
                return links.type;
            }
            function getTypeOfFuncClassEnumModuleWorker(symbol) {
                var declaration = symbol.valueDeclaration;
                if (symbol.flags & 1536 /* Module */ && ts.isShorthandAmbientModuleSymbol(symbol)) {
                    return anyType;
                }
                else if (declaration.kind === 209 /* BinaryExpression */ ||
                    ts.isAccessExpression(declaration) &&
                        declaration.parent.kind === 209 /* BinaryExpression */) {
                    return getWidenedTypeForAssignmentDeclaration(symbol);
                }
                else if (symbol.flags & 512 /* ValueModule */ && declaration && ts.isSourceFile(declaration) && declaration.commonJsModuleIndicator) {
                    var resolvedModule = resolveExternalModuleSymbol(symbol);
                    if (resolvedModule !== symbol) {
                        if (!pushTypeResolution(symbol, 0 /* Type */)) {
                            return errorType;
                        }
                        var exportEquals = getMergedSymbol(symbol.exports.get("export=" /* ExportEquals */));
                        var type_1 = getWidenedTypeForAssignmentDeclaration(exportEquals, exportEquals === resolvedModule ? undefined : resolvedModule);
                        if (!popTypeResolution()) {
                            return reportCircularityError(symbol);
                        }
                        return type_1;
                    }
                }
                var type = createObjectType(16 /* Anonymous */, symbol);
                if (symbol.flags & 32 /* Class */) {
                    var baseTypeVariable = getBaseTypeVariableOfClass(symbol);
                    return baseTypeVariable ? getIntersectionType([type, baseTypeVariable]) : type;
                }
                else {
                    return strictNullChecks && symbol.flags & 16777216 /* Optional */ ? getOptionalType(type) : type;
                }
            }
            function getTypeOfEnumMember(symbol) {
                var links = getSymbolLinks(symbol);
                return links.type || (links.type = getDeclaredTypeOfEnumMember(symbol));
            }
            function getTypeOfAlias(symbol) {
                var links = getSymbolLinks(symbol);
                if (!links.type) {
                    var targetSymbol = resolveAlias(symbol);
                    // It only makes sense to get the type of a value symbol. If the result of resolving
                    // the alias is not a value, then it has no type. To get the type associated with a
                    // type symbol, call getDeclaredTypeOfSymbol.
                    // This check is important because without it, a call to getTypeOfSymbol could end
                    // up recursively calling getTypeOfAlias, causing a stack overflow.
                    links.type = targetSymbol.flags & 111551 /* Value */
                        ? getTypeOfSymbol(targetSymbol)
                        : errorType;
                }
                return links.type;
            }
            function getTypeOfInstantiatedSymbol(symbol) {
                var links = getSymbolLinks(symbol);
                if (!links.type) {
                    if (!pushTypeResolution(symbol, 0 /* Type */)) {
                        return links.type = errorType;
                    }
                    var type = instantiateType(getTypeOfSymbol(links.target), links.mapper);
                    if (!popTypeResolution()) {
                        type = reportCircularityError(symbol);
                    }
                    links.type = type;
                }
                return links.type;
            }
            function reportCircularityError(symbol) {
                var declaration = symbol.valueDeclaration;
                // Check if variable has type annotation that circularly references the variable itself
                if (ts.getEffectiveTypeAnnotationNode(declaration)) {
                    error(symbol.valueDeclaration, ts.Diagnostics._0_is_referenced_directly_or_indirectly_in_its_own_type_annotation, symbolToString(symbol));
                    return errorType;
                }
                // Check if variable has initializer that circularly references the variable itself
                if (noImplicitAny && (declaration.kind !== 156 /* Parameter */ || declaration.initializer)) {
                    error(symbol.valueDeclaration, ts.Diagnostics._0_implicitly_has_type_any_because_it_does_not_have_a_type_annotation_and_is_referenced_directly_or_indirectly_in_its_own_initializer, symbolToString(symbol));
                }
                // Circularities could also result from parameters in function expressions that end up
                // having themselves as contextual types following type argument inference. In those cases
                // we have already reported an implicit any error so we don't report anything here.
                return anyType;
            }
            function getTypeOfSymbolWithDeferredType(symbol) {
                var links = getSymbolLinks(symbol);
                if (!links.type) {
                    ts.Debug.assertDefined(links.deferralParent);
                    ts.Debug.assertDefined(links.deferralConstituents);
                    links.type = links.deferralParent.flags & 1048576 /* Union */ ? getUnionType(links.deferralConstituents) : getIntersectionType(links.deferralConstituents);
                }
                return links.type;
            }
            function getTypeOfSymbol(symbol) {
                if (ts.getCheckFlags(symbol) & 65536 /* DeferredType */) {
                    return getTypeOfSymbolWithDeferredType(symbol);
                }
                if (ts.getCheckFlags(symbol) & 1 /* Instantiated */) {
                    return getTypeOfInstantiatedSymbol(symbol);
                }
                if (ts.getCheckFlags(symbol) & 8192 /* ReverseMapped */) {
                    return getTypeOfReverseMappedSymbol(symbol);
                }
                if (symbol.flags & (3 /* Variable */ | 4 /* Property */)) {
                    return getTypeOfVariableOrParameterOrProperty(symbol);
                }
                if (symbol.flags & (16 /* Function */ | 8192 /* Method */ | 32 /* Class */ | 384 /* Enum */ | 512 /* ValueModule */)) {
                    return getTypeOfFuncClassEnumModule(symbol);
                }
                if (symbol.flags & 8 /* EnumMember */) {
                    return getTypeOfEnumMember(symbol);
                }
                if (symbol.flags & 98304 /* Accessor */) {
                    return getTypeOfAccessors(symbol);
                }
                if (symbol.flags & 2097152 /* Alias */) {
                    return getTypeOfAlias(symbol);
                }
                return errorType;
            }
            function isReferenceToType(type, target) {
                return type !== undefined
                    && target !== undefined
                    && (ts.getObjectFlags(type) & 4 /* Reference */) !== 0
                    && type.target === target;
            }
            function getTargetType(type) {
                return ts.getObjectFlags(type) & 4 /* Reference */ ? type.target : type;
            }
            // TODO: GH#18217 If `checkBase` is undefined, we should not call this because this will always return false.
            function hasBaseType(type, checkBase) {
                return check(type);
                function check(type) {
                    if (ts.getObjectFlags(type) & (3 /* ClassOrInterface */ | 4 /* Reference */)) {
                        var target = getTargetType(type);
                        return target === checkBase || ts.some(getBaseTypes(target), check);
                    }
                    else if (type.flags & 2097152 /* Intersection */) {
                        return ts.some(type.types, check);
                    }
                    return false;
                }
            }
            // Appends the type parameters given by a list of declarations to a set of type parameters and returns the resulting set.
            // The function allocates a new array if the input type parameter set is undefined, but otherwise it modifies the set
            // in-place and returns the same array.
            function appendTypeParameters(typeParameters, declarations) {
                for (var _i = 0, declarations_2 = declarations; _i < declarations_2.length; _i++) {
                    var declaration = declarations_2[_i];
                    typeParameters = ts.appendIfUnique(typeParameters, getDeclaredTypeOfTypeParameter(getSymbolOfNode(declaration)));
                }
                return typeParameters;
            }
            // Return the outer type parameters of a node or undefined if the node has no outer type parameters.
            function getOuterTypeParameters(node, includeThisTypes) {
                while (true) {
                    node = node.parent; // TODO: GH#18217 Use SourceFile kind check instead
                    if (node && ts.isBinaryExpression(node)) {
                        // prototype assignments get the outer type parameters of their constructor function
                        var assignmentKind = ts.getAssignmentDeclarationKind(node);
                        if (assignmentKind === 6 /* Prototype */ || assignmentKind === 3 /* PrototypeProperty */) {
                            var symbol = getSymbolOfNode(node.left);
                            if (symbol && symbol.parent && !ts.findAncestor(symbol.parent.valueDeclaration, function (d) { return node === d; })) {
                                node = symbol.parent.valueDeclaration;
                            }
                        }
                    }
                    if (!node) {
                        return undefined;
                    }
                    switch (node.kind) {
                        case 245 /* ClassDeclaration */:
                        case 214 /* ClassExpression */:
                        case 246 /* InterfaceDeclaration */:
                        case 165 /* CallSignature */:
                        case 166 /* ConstructSignature */:
                        case 160 /* MethodSignature */:
                        case 170 /* FunctionType */:
                        case 171 /* ConstructorType */:
                        case 300 /* JSDocFunctionType */:
                        case 244 /* FunctionDeclaration */:
                        case 161 /* MethodDeclaration */:
                        case 201 /* FunctionExpression */:
                        case 202 /* ArrowFunction */:
                        case 247 /* TypeAliasDeclaration */:
                        case 320 /* JSDocTemplateTag */:
                        case 321 /* JSDocTypedefTag */:
                        case 315 /* JSDocEnumTag */:
                        case 314 /* JSDocCallbackTag */:
                        case 186 /* MappedType */:
                        case 180 /* ConditionalType */:
                            var outerTypeParameters = getOuterTypeParameters(node, includeThisTypes);
                            if (node.kind === 186 /* MappedType */) {
                                return ts.append(outerTypeParameters, getDeclaredTypeOfTypeParameter(getSymbolOfNode(node.typeParameter)));
                            }
                            else if (node.kind === 180 /* ConditionalType */) {
                                return ts.concatenate(outerTypeParameters, getInferTypeParameters(node));
                            }
                            var outerAndOwnTypeParameters = appendTypeParameters(outerTypeParameters, ts.getEffectiveTypeParameterDeclarations(node));
                            var thisType = includeThisTypes &&
                                (node.kind === 245 /* ClassDeclaration */ || node.kind === 214 /* ClassExpression */ || node.kind === 246 /* InterfaceDeclaration */ || isJSConstructor(node)) &&
                                getDeclaredTypeOfClassOrInterface(getSymbolOfNode(node)).thisType;
                            return thisType ? ts.append(outerAndOwnTypeParameters, thisType) : outerAndOwnTypeParameters;
                    }
                }
            }
            // The outer type parameters are those defined by enclosing generic classes, methods, or functions.
            function getOuterTypeParametersOfClassOrInterface(symbol) {
                var declaration = symbol.flags & 32 /* Class */ ? symbol.valueDeclaration : ts.getDeclarationOfKind(symbol, 246 /* InterfaceDeclaration */);
                ts.Debug.assert(!!declaration, "Class was missing valueDeclaration -OR- non-class had no interface declarations");
                return getOuterTypeParameters(declaration);
            }
            // The local type parameters are the combined set of type parameters from all declarations of the class,
            // interface, or type alias.
            function getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(symbol) {
                var result;
                for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                    var node = _a[_i];
                    if (node.kind === 246 /* InterfaceDeclaration */ ||
                        node.kind === 245 /* ClassDeclaration */ ||
                        node.kind === 214 /* ClassExpression */ ||
                        isJSConstructor(node) ||
                        ts.isTypeAlias(node)) {
                        var declaration = node;
                        result = appendTypeParameters(result, ts.getEffectiveTypeParameterDeclarations(declaration));
                    }
                }
                return result;
            }
            // The full set of type parameters for a generic class or interface type consists of its outer type parameters plus
            // its locally declared type parameters.
            function getTypeParametersOfClassOrInterface(symbol) {
                return ts.concatenate(getOuterTypeParametersOfClassOrInterface(symbol), getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(symbol));
            }
            // A type is a mixin constructor if it has a single construct signature taking no type parameters and a single
            // rest parameter of type any[].
            function isMixinConstructorType(type) {
                var signatures = getSignaturesOfType(type, 1 /* Construct */);
                if (signatures.length === 1) {
                    var s = signatures[0];
                    return !s.typeParameters && s.parameters.length === 1 && signatureHasRestParameter(s) && getElementTypeOfArrayType(getTypeOfParameter(s.parameters[0])) === anyType;
                }
                return false;
            }
            function isConstructorType(type) {
                if (getSignaturesOfType(type, 1 /* Construct */).length > 0) {
                    return true;
                }
                if (type.flags & 8650752 /* TypeVariable */) {
                    var constraint = getBaseConstraintOfType(type);
                    return !!constraint && isMixinConstructorType(constraint);
                }
                return false;
            }
            function getBaseTypeNodeOfClass(type) {
                return ts.getEffectiveBaseTypeNode(type.symbol.valueDeclaration);
            }
            function getConstructorsForTypeArguments(type, typeArgumentNodes, location) {
                var typeArgCount = ts.length(typeArgumentNodes);
                var isJavascript = ts.isInJSFile(location);
                return ts.filter(getSignaturesOfType(type, 1 /* Construct */), function (sig) { return (isJavascript || typeArgCount >= getMinTypeArgumentCount(sig.typeParameters)) && typeArgCount <= ts.length(sig.typeParameters); });
            }
            function getInstantiatedConstructorsForTypeArguments(type, typeArgumentNodes, location) {
                var signatures = getConstructorsForTypeArguments(type, typeArgumentNodes, location);
                var typeArguments = ts.map(typeArgumentNodes, getTypeFromTypeNode);
                return ts.sameMap(signatures, function (sig) { return ts.some(sig.typeParameters) ? getSignatureInstantiation(sig, typeArguments, ts.isInJSFile(location)) : sig; });
            }
            /**
             * The base constructor of a class can resolve to
             * * undefinedType if the class has no extends clause,
             * * unknownType if an error occurred during resolution of the extends expression,
             * * nullType if the extends expression is the null value,
             * * anyType if the extends expression has type any, or
             * * an object type with at least one construct signature.
             */
            function getBaseConstructorTypeOfClass(type) {
                if (!type.resolvedBaseConstructorType) {
                    var decl = type.symbol.valueDeclaration;
                    var extended = ts.getEffectiveBaseTypeNode(decl);
                    var baseTypeNode = getBaseTypeNodeOfClass(type);
                    if (!baseTypeNode) {
                        return type.resolvedBaseConstructorType = undefinedType;
                    }
                    if (!pushTypeResolution(type, 1 /* ResolvedBaseConstructorType */)) {
                        return errorType;
                    }
                    var baseConstructorType = checkExpression(baseTypeNode.expression);
                    if (extended && baseTypeNode !== extended) {
                        ts.Debug.assert(!extended.typeArguments); // Because this is in a JS file, and baseTypeNode is in an @extends tag
                        checkExpression(extended.expression);
                    }
                    if (baseConstructorType.flags & (524288 /* Object */ | 2097152 /* Intersection */)) {
                        // Resolving the members of a class requires us to resolve the base class of that class.
                        // We force resolution here such that we catch circularities now.
                        resolveStructuredTypeMembers(baseConstructorType);
                    }
                    if (!popTypeResolution()) {
                        error(type.symbol.valueDeclaration, ts.Diagnostics._0_is_referenced_directly_or_indirectly_in_its_own_base_expression, symbolToString(type.symbol));
                        return type.resolvedBaseConstructorType = errorType;
                    }
                    if (!(baseConstructorType.flags & 1 /* Any */) && baseConstructorType !== nullWideningType && !isConstructorType(baseConstructorType)) {
                        var err = error(baseTypeNode.expression, ts.Diagnostics.Type_0_is_not_a_constructor_function_type, typeToString(baseConstructorType));
                        if (baseConstructorType.flags & 262144 /* TypeParameter */) {
                            var constraint = getConstraintFromTypeParameter(baseConstructorType);
                            var ctorReturn = unknownType;
                            if (constraint) {
                                var ctorSig = getSignaturesOfType(constraint, 1 /* Construct */);
                                if (ctorSig[0]) {
                                    ctorReturn = getReturnTypeOfSignature(ctorSig[0]);
                                }
                            }
                            ts.addRelatedInfo(err, ts.createDiagnosticForNode(baseConstructorType.symbol.declarations[0], ts.Diagnostics.Did_you_mean_for_0_to_be_constrained_to_type_new_args_Colon_any_1, symbolToString(baseConstructorType.symbol), typeToString(ctorReturn)));
                        }
                        return type.resolvedBaseConstructorType = errorType;
                    }
                    type.resolvedBaseConstructorType = baseConstructorType;
                }
                return type.resolvedBaseConstructorType;
            }
            function getBaseTypes(type) {
                if (!type.resolvedBaseTypes) {
                    if (type.objectFlags & 8 /* Tuple */) {
                        type.resolvedBaseTypes = [createArrayType(getUnionType(type.typeParameters || ts.emptyArray), type.readonly)];
                    }
                    else if (type.symbol.flags & (32 /* Class */ | 64 /* Interface */)) {
                        if (type.symbol.flags & 32 /* Class */) {
                            resolveBaseTypesOfClass(type);
                        }
                        if (type.symbol.flags & 64 /* Interface */) {
                            resolveBaseTypesOfInterface(type);
                        }
                    }
                    else {
                        ts.Debug.fail("type must be class or interface");
                    }
                }
                return type.resolvedBaseTypes;
            }
            function resolveBaseTypesOfClass(type) {
                type.resolvedBaseTypes = ts.resolvingEmptyArray;
                var baseConstructorType = getApparentType(getBaseConstructorTypeOfClass(type));
                if (!(baseConstructorType.flags & (524288 /* Object */ | 2097152 /* Intersection */ | 1 /* Any */))) {
                    return type.resolvedBaseTypes = ts.emptyArray;
                }
                var baseTypeNode = getBaseTypeNodeOfClass(type);
                var baseType;
                var originalBaseType = baseConstructorType.symbol ? getDeclaredTypeOfSymbol(baseConstructorType.symbol) : undefined;
                if (baseConstructorType.symbol && baseConstructorType.symbol.flags & 32 /* Class */ &&
                    areAllOuterTypeParametersApplied(originalBaseType)) {
                    // When base constructor type is a class with no captured type arguments we know that the constructors all have the same type parameters as the
                    // class and all return the instance type of the class. There is no need for further checks and we can apply the
                    // type arguments in the same manner as a type reference to get the same error reporting experience.
                    baseType = getTypeFromClassOrInterfaceReference(baseTypeNode, baseConstructorType.symbol);
                }
                else if (baseConstructorType.flags & 1 /* Any */) {
                    baseType = baseConstructorType;
                }
                else {
                    // The class derives from a "class-like" constructor function, check that we have at least one construct signature
                    // with a matching number of type parameters and use the return type of the first instantiated signature. Elsewhere
                    // we check that all instantiated signatures return the same type.
                    var constructors = getInstantiatedConstructorsForTypeArguments(baseConstructorType, baseTypeNode.typeArguments, baseTypeNode);
                    if (!constructors.length) {
                        error(baseTypeNode.expression, ts.Diagnostics.No_base_constructor_has_the_specified_number_of_type_arguments);
                        return type.resolvedBaseTypes = ts.emptyArray;
                    }
                    baseType = getReturnTypeOfSignature(constructors[0]);
                }
                if (baseType === errorType) {
                    return type.resolvedBaseTypes = ts.emptyArray;
                }
                if (!isValidBaseType(baseType)) {
                    error(baseTypeNode.expression, ts.Diagnostics.Base_constructor_return_type_0_is_not_an_object_type_or_intersection_of_object_types_with_statically_known_members, typeToString(baseType));
                    return type.resolvedBaseTypes = ts.emptyArray;
                }
                if (type === baseType || hasBaseType(baseType, type)) {
                    error(type.symbol.valueDeclaration, ts.Diagnostics.Type_0_recursively_references_itself_as_a_base_type, typeToString(type, /*enclosingDeclaration*/ undefined, 2 /* WriteArrayAsGenericType */));
                    return type.resolvedBaseTypes = ts.emptyArray;
                }
                if (type.resolvedBaseTypes === ts.resolvingEmptyArray) {
                    // Circular reference, likely through instantiation of default parameters
                    // (otherwise there'd be an error from hasBaseType) - this is fine, but `.members` should be reset
                    // as `getIndexedAccessType` via `instantiateType` via `getTypeFromClassOrInterfaceReference` forces a
                    // partial instantiation of the members without the base types fully resolved
                    type.members = undefined;
                }
                return type.resolvedBaseTypes = [baseType];
            }
            function areAllOuterTypeParametersApplied(type) {
                // An unapplied type parameter has its symbol still the same as the matching argument symbol.
                // Since parameters are applied outer-to-inner, only the last outer parameter needs to be checked.
                var outerTypeParameters = type.outerTypeParameters;
                if (outerTypeParameters) {
                    var last_1 = outerTypeParameters.length - 1;
                    var typeArguments = getTypeArguments(type);
                    return outerTypeParameters[last_1].symbol !== typeArguments[last_1].symbol;
                }
                return true;
            }
            // A valid base type is `any`, an object type or intersection of object types.
            function isValidBaseType(type) {
                if (type.flags & 262144 /* TypeParameter */) {
                    var constraint = getBaseConstraintOfType(type);
                    if (constraint) {
                        return isValidBaseType(constraint);
                    }
                }
                // TODO: Given that we allow type parmeters here now, is this `!isGenericMappedType(type)` check really needed?
                // There's no reason a `T` should be allowed while a `Readonly<T>` should not.
                return !!(type.flags & (524288 /* Object */ | 67108864 /* NonPrimitive */ | 1 /* Any */)) && !isGenericMappedType(type) ||
                    !!(type.flags & 2097152 /* Intersection */) && ts.every(type.types, isValidBaseType);
            }
            function resolveBaseTypesOfInterface(type) {
                type.resolvedBaseTypes = type.resolvedBaseTypes || ts.emptyArray;
                for (var _i = 0, _a = type.symbol.declarations; _i < _a.length; _i++) {
                    var declaration = _a[_i];
                    if (declaration.kind === 246 /* InterfaceDeclaration */ && ts.getInterfaceBaseTypeNodes(declaration)) {
                        for (var _b = 0, _c = ts.getInterfaceBaseTypeNodes(declaration); _b < _c.length; _b++) {
                            var node = _c[_b];
                            var baseType = getTypeFromTypeNode(node);
                            if (baseType !== errorType) {
                                if (isValidBaseType(baseType)) {
                                    if (type !== baseType && !hasBaseType(baseType, type)) {
                                        if (type.resolvedBaseTypes === ts.emptyArray) {
                                            type.resolvedBaseTypes = [baseType];
                                        }
                                        else {
                                            type.resolvedBaseTypes.push(baseType);
                                        }
                                    }
                                    else {
                                        error(declaration, ts.Diagnostics.Type_0_recursively_references_itself_as_a_base_type, typeToString(type, /*enclosingDeclaration*/ undefined, 2 /* WriteArrayAsGenericType */));
                                    }
                                }
                                else {
                                    error(node, ts.Diagnostics.An_interface_can_only_extend_an_object_type_or_intersection_of_object_types_with_statically_known_members);
                                }
                            }
                        }
                    }
                }
            }
            /**
             * Returns true if the interface given by the symbol is free of "this" references.
             *
             * Specifically, the result is true if the interface itself contains no references
             * to "this" in its body, if all base types are interfaces,
             * and if none of the base interfaces have a "this" type.
             */
            function isThislessInterface(symbol) {
                for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                    var declaration = _a[_i];
                    if (declaration.kind === 246 /* InterfaceDeclaration */) {
                        if (declaration.flags & 128 /* ContainsThis */) {
                            return false;
                        }
                        var baseTypeNodes = ts.getInterfaceBaseTypeNodes(declaration);
                        if (baseTypeNodes) {
                            for (var _b = 0, baseTypeNodes_1 = baseTypeNodes; _b < baseTypeNodes_1.length; _b++) {
                                var node = baseTypeNodes_1[_b];
                                if (ts.isEntityNameExpression(node.expression)) {
                                    var baseSymbol = resolveEntityName(node.expression, 788968 /* Type */, /*ignoreErrors*/ true);
                                    if (!baseSymbol || !(baseSymbol.flags & 64 /* Interface */) || getDeclaredTypeOfClassOrInterface(baseSymbol).thisType) {
                                        return false;
                                    }
                                }
                            }
                        }
                    }
                }
                return true;
            }
            function getDeclaredTypeOfClassOrInterface(symbol) {
                var links = getSymbolLinks(symbol);
                var originalLinks = links;
                if (!links.declaredType) {
                    var kind = symbol.flags & 32 /* Class */ ? 1 /* Class */ : 2 /* Interface */;
                    var merged = mergeJSSymbols(symbol, getAssignedClassSymbol(symbol.valueDeclaration));
                    if (merged) {
                        // note:we overwrite links because we just cloned the symbol
                        symbol = links = merged;
                    }
                    var type = originalLinks.declaredType = links.declaredType = createObjectType(kind, symbol);
                    var outerTypeParameters = getOuterTypeParametersOfClassOrInterface(symbol);
                    var localTypeParameters = getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(symbol);
                    // A class or interface is generic if it has type parameters or a "this" type. We always give classes a "this" type
                    // because it is not feasible to analyze all members to determine if the "this" type escapes the class (in particular,
                    // property types inferred from initializers and method return types inferred from return statements are very hard
                    // to exhaustively analyze). We give interfaces a "this" type if we can't definitely determine that they are free of
                    // "this" references.
                    if (outerTypeParameters || localTypeParameters || kind === 1 /* Class */ || !isThislessInterface(symbol)) {
                        type.objectFlags |= 4 /* Reference */;
                        type.typeParameters = ts.concatenate(outerTypeParameters, localTypeParameters);
                        type.outerTypeParameters = outerTypeParameters;
                        type.localTypeParameters = localTypeParameters;
                        type.instantiations = ts.createMap();
                        type.instantiations.set(getTypeListId(type.typeParameters), type);
                        type.target = type;
                        type.resolvedTypeArguments = type.typeParameters;
                        type.thisType = createTypeParameter(symbol);
                        type.thisType.isThisType = true;
                        type.thisType.constraint = type;
                    }
                }
                return links.declaredType;
            }
            function getDeclaredTypeOfTypeAlias(symbol) {
                var links = getSymbolLinks(symbol);
                if (!links.declaredType) {
                    // Note that we use the links object as the target here because the symbol object is used as the unique
                    // identity for resolution of the 'type' property in SymbolLinks.
                    if (!pushTypeResolution(symbol, 2 /* DeclaredType */)) {
                        return errorType;
                    }
                    var declaration = ts.Debug.assertDefined(ts.find(symbol.declarations, ts.isTypeAlias), "Type alias symbol with no valid declaration found");
                    var typeNode = ts.isJSDocTypeAlias(declaration) ? declaration.typeExpression : declaration.type;
                    // If typeNode is missing, we will error in checkJSDocTypedefTag.
                    var type = typeNode ? getTypeFromTypeNode(typeNode) : errorType;
                    if (popTypeResolution()) {
                        var typeParameters = getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(symbol);
                        if (typeParameters) {
                            // Initialize the instantiation cache for generic type aliases. The declared type corresponds to
                            // an instantiation of the type alias with the type parameters supplied as type arguments.
                            links.typeParameters = typeParameters;
                            links.instantiations = ts.createMap();
                            links.instantiations.set(getTypeListId(typeParameters), type);
                        }
                    }
                    else {
                        type = errorType;
                        error(ts.isNamedDeclaration(declaration) ? declaration.name : declaration || declaration, ts.Diagnostics.Type_alias_0_circularly_references_itself, symbolToString(symbol));
                    }
                    links.declaredType = type;
                }
                return links.declaredType;
            }
            function isStringConcatExpression(expr) {
                if (ts.isStringLiteralLike(expr)) {
                    return true;
                }
                else if (expr.kind === 209 /* BinaryExpression */) {
                    return isStringConcatExpression(expr.left) && isStringConcatExpression(expr.right);
                }
                return false;
            }
            function isLiteralEnumMember(member) {
                var expr = member.initializer;
                if (!expr) {
                    return !(member.flags & 8388608 /* Ambient */);
                }
                switch (expr.kind) {
                    case 10 /* StringLiteral */:
                    case 8 /* NumericLiteral */:
                    case 14 /* NoSubstitutionTemplateLiteral */:
                        return true;
                    case 207 /* PrefixUnaryExpression */:
                        return expr.operator === 40 /* MinusToken */ &&
                            expr.operand.kind === 8 /* NumericLiteral */;
                    case 75 /* Identifier */:
                        return ts.nodeIsMissing(expr) || !!getSymbolOfNode(member.parent).exports.get(expr.escapedText);
                    case 209 /* BinaryExpression */:
                        return isStringConcatExpression(expr);
                    default:
                        return false;
                }
            }
            function getEnumKind(symbol) {
                var links = getSymbolLinks(symbol);
                if (links.enumKind !== undefined) {
                    return links.enumKind;
                }
                var hasNonLiteralMember = false;
                for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                    var declaration = _a[_i];
                    if (declaration.kind === 248 /* EnumDeclaration */) {
                        for (var _b = 0, _c = declaration.members; _b < _c.length; _b++) {
                            var member = _c[_b];
                            if (member.initializer && ts.isStringLiteralLike(member.initializer)) {
                                return links.enumKind = 1 /* Literal */;
                            }
                            if (!isLiteralEnumMember(member)) {
                                hasNonLiteralMember = true;
                            }
                        }
                    }
                }
                return links.enumKind = hasNonLiteralMember ? 0 /* Numeric */ : 1 /* Literal */;
            }
            function getBaseTypeOfEnumLiteralType(type) {
                return type.flags & 1024 /* EnumLiteral */ && !(type.flags & 1048576 /* Union */) ? getDeclaredTypeOfSymbol(getParentOfSymbol(type.symbol)) : type;
            }
            function getDeclaredTypeOfEnum(symbol) {
                var links = getSymbolLinks(symbol);
                if (links.declaredType) {
                    return links.declaredType;
                }
                if (getEnumKind(symbol) === 1 /* Literal */) {
                    enumCount++;
                    var memberTypeList = [];
                    for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                        var declaration = _a[_i];
                        if (declaration.kind === 248 /* EnumDeclaration */) {
                            for (var _b = 0, _c = declaration.members; _b < _c.length; _b++) {
                                var member = _c[_b];
                                var value = getEnumMemberValue(member);
                                var memberType = getFreshTypeOfLiteralType(getLiteralType(value !== undefined ? value : 0, enumCount, getSymbolOfNode(member)));
                                getSymbolLinks(getSymbolOfNode(member)).declaredType = memberType;
                                memberTypeList.push(getRegularTypeOfLiteralType(memberType));
                            }
                        }
                    }
                    if (memberTypeList.length) {
                        var enumType_1 = getUnionType(memberTypeList, 1 /* Literal */, symbol, /*aliasTypeArguments*/ undefined);
                        if (enumType_1.flags & 1048576 /* Union */) {
                            enumType_1.flags |= 1024 /* EnumLiteral */;
                            enumType_1.symbol = symbol;
                        }
                        return links.declaredType = enumType_1;
                    }
                }
                var enumType = createType(32 /* Enum */);
                enumType.symbol = symbol;
                return links.declaredType = enumType;
            }
            function getDeclaredTypeOfEnumMember(symbol) {
                var links = getSymbolLinks(symbol);
                if (!links.declaredType) {
                    var enumType = getDeclaredTypeOfEnum(getParentOfSymbol(symbol));
                    if (!links.declaredType) {
                        links.declaredType = enumType;
                    }
                }
                return links.declaredType;
            }
            function getDeclaredTypeOfTypeParameter(symbol) {
                var links = getSymbolLinks(symbol);
                return links.declaredType || (links.declaredType = createTypeParameter(symbol));
            }
            function getDeclaredTypeOfAlias(symbol) {
                var links = getSymbolLinks(symbol);
                return links.declaredType || (links.declaredType = getDeclaredTypeOfSymbol(resolveAlias(symbol)));
            }
            function getDeclaredTypeOfSymbol(symbol) {
                return tryGetDeclaredTypeOfSymbol(symbol) || errorType;
            }
            function tryGetDeclaredTypeOfSymbol(symbol) {
                if (symbol.flags & (32 /* Class */ | 64 /* Interface */)) {
                    return getDeclaredTypeOfClassOrInterface(symbol);
                }
                if (symbol.flags & 524288 /* TypeAlias */) {
                    return getDeclaredTypeOfTypeAlias(symbol);
                }
                if (symbol.flags & 262144 /* TypeParameter */) {
                    return getDeclaredTypeOfTypeParameter(symbol);
                }
                if (symbol.flags & 384 /* Enum */) {
                    return getDeclaredTypeOfEnum(symbol);
                }
                if (symbol.flags & 8 /* EnumMember */) {
                    return getDeclaredTypeOfEnumMember(symbol);
                }
                if (symbol.flags & 2097152 /* Alias */) {
                    return getDeclaredTypeOfAlias(symbol);
                }
                return undefined;
            }
            /**
             * A type is free of this references if it's the any, string, number, boolean, symbol, or void keyword, a string
             * literal type, an array with an element type that is free of this references, or a type reference that is
             * free of this references.
             */
            function isThislessType(node) {
                switch (node.kind) {
                    case 125 /* AnyKeyword */:
                    case 148 /* UnknownKeyword */:
                    case 143 /* StringKeyword */:
                    case 140 /* NumberKeyword */:
                    case 151 /* BigIntKeyword */:
                    case 128 /* BooleanKeyword */:
                    case 144 /* SymbolKeyword */:
                    case 141 /* ObjectKeyword */:
                    case 110 /* VoidKeyword */:
                    case 146 /* UndefinedKeyword */:
                    case 100 /* NullKeyword */:
                    case 137 /* NeverKeyword */:
                    case 187 /* LiteralType */:
                        return true;
                    case 174 /* ArrayType */:
                        return isThislessType(node.elementType);
                    case 169 /* TypeReference */:
                        return !node.typeArguments || node.typeArguments.every(isThislessType);
                }
                return false;
            }
            /** A type parameter is thisless if its constraint is thisless, or if it has no constraint. */
            function isThislessTypeParameter(node) {
                var constraint = ts.getEffectiveConstraintOfTypeParameter(node);
                return !constraint || isThislessType(constraint);
            }
            /**
             * A variable-like declaration is free of this references if it has a type annotation
             * that is thisless, or if it has no type annotation and no initializer (and is thus of type any).
             */
            function isThislessVariableLikeDeclaration(node) {
                var typeNode = ts.getEffectiveTypeAnnotationNode(node);
                return typeNode ? isThislessType(typeNode) : !ts.hasInitializer(node);
            }
            /**
             * A function-like declaration is considered free of `this` references if it has a return type
             * annotation that is free of this references and if each parameter is thisless and if
             * each type parameter (if present) is thisless.
             */
            function isThislessFunctionLikeDeclaration(node) {
                var returnType = ts.getEffectiveReturnTypeNode(node);
                var typeParameters = ts.getEffectiveTypeParameterDeclarations(node);
                return (node.kind === 162 /* Constructor */ || (!!returnType && isThislessType(returnType))) &&
                    node.parameters.every(isThislessVariableLikeDeclaration) &&
                    typeParameters.every(isThislessTypeParameter);
            }
            /**
             * Returns true if the class or interface member given by the symbol is free of "this" references. The
             * function may return false for symbols that are actually free of "this" references because it is not
             * feasible to perform a complete analysis in all cases. In particular, property members with types
             * inferred from their initializers and function members with inferred return types are conservatively
             * assumed not to be free of "this" references.
             */
            function isThisless(symbol) {
                if (symbol.declarations && symbol.declarations.length === 1) {
                    var declaration = symbol.declarations[0];
                    if (declaration) {
                        switch (declaration.kind) {
                            case 159 /* PropertyDeclaration */:
                            case 158 /* PropertySignature */:
                                return isThislessVariableLikeDeclaration(declaration);
                            case 161 /* MethodDeclaration */:
                            case 160 /* MethodSignature */:
                            case 162 /* Constructor */:
                            case 163 /* GetAccessor */:
                            case 164 /* SetAccessor */:
                                return isThislessFunctionLikeDeclaration(declaration);
                        }
                    }
                }
                return false;
            }
            // The mappingThisOnly flag indicates that the only type parameter being mapped is "this". When the flag is true,
            // we check symbols to see if we can quickly conclude they are free of "this" references, thus needing no instantiation.
            function createInstantiatedSymbolTable(symbols, mapper, mappingThisOnly) {
                var result = ts.createSymbolTable();
                for (var _i = 0, symbols_2 = symbols; _i < symbols_2.length; _i++) {
                    var symbol = symbols_2[_i];
                    result.set(symbol.escapedName, mappingThisOnly && isThisless(symbol) ? symbol : instantiateSymbol(symbol, mapper));
                }
                return result;
            }
            function addInheritedMembers(symbols, baseSymbols) {
                for (var _i = 0, baseSymbols_1 = baseSymbols; _i < baseSymbols_1.length; _i++) {
                    var s = baseSymbols_1[_i];
                    if (!symbols.has(s.escapedName) && !isStaticPrivateIdentifierProperty(s)) {
                        symbols.set(s.escapedName, s);
                    }
                }
            }
            function isStaticPrivateIdentifierProperty(s) {
                return !!s.valueDeclaration && ts.isPrivateIdentifierPropertyDeclaration(s.valueDeclaration) && ts.hasModifier(s.valueDeclaration, 32 /* Static */);
            }
            function resolveDeclaredMembers(type) {
                if (!type.declaredProperties) {
                    var symbol = type.symbol;
                    var members = getMembersOfSymbol(symbol);
                    type.declaredProperties = getNamedMembers(members);
                    // Start with signatures at empty array in case of recursive types
                    type.declaredCallSignatures = ts.emptyArray;
                    type.declaredConstructSignatures = ts.emptyArray;
                    type.declaredCallSignatures = getSignaturesOfSymbol(members.get("__call" /* Call */));
                    type.declaredConstructSignatures = getSignaturesOfSymbol(members.get("__new" /* New */));
                    type.declaredStringIndexInfo = getIndexInfoOfSymbol(symbol, 0 /* String */);
                    type.declaredNumberIndexInfo = getIndexInfoOfSymbol(symbol, 1 /* Number */);
                }
                return type;
            }
            /**
             * Indicates whether a type can be used as a property name.
             */
            function isTypeUsableAsPropertyName(type) {
                return !!(type.flags & 8576 /* StringOrNumberLiteralOrUnique */);
            }
            /**
             * Indicates whether a declaration name is definitely late-bindable.
             * A declaration name is only late-bindable if:
             * - It is a `ComputedPropertyName`.
             * - Its expression is an `Identifier` or either a `PropertyAccessExpression` an
             * `ElementAccessExpression` consisting only of these same three types of nodes.
             * - The type of its expression is a string or numeric literal type, or is a `unique symbol` type.
             */
            function isLateBindableName(node) {
                if (!ts.isComputedPropertyName(node) && !ts.isElementAccessExpression(node)) {
                    return false;
                }
                var expr = ts.isComputedPropertyName(node) ? node.expression : node.argumentExpression;
                return ts.isEntityNameExpression(expr)
                    && isTypeUsableAsPropertyName(ts.isComputedPropertyName(node) ? checkComputedPropertyName(node) : checkExpressionCached(expr));
            }
            function isLateBoundName(name) {
                return name.charCodeAt(0) === 95 /* _ */ &&
                    name.charCodeAt(1) === 95 /* _ */ &&
                    name.charCodeAt(2) === 64 /* at */;
            }
            /**
             * Indicates whether a declaration has a late-bindable dynamic name.
             */
            function hasLateBindableName(node) {
                var name = ts.getNameOfDeclaration(node);
                return !!name && isLateBindableName(name);
            }
            /**
             * Indicates whether a declaration has a dynamic name that cannot be late-bound.
             */
            function hasNonBindableDynamicName(node) {
                return ts.hasDynamicName(node) && !hasLateBindableName(node);
            }
            /**
             * Indicates whether a declaration name is a dynamic name that cannot be late-bound.
             */
            function isNonBindableDynamicName(node) {
                return ts.isDynamicName(node) && !isLateBindableName(node);
            }
            /**
             * Gets the symbolic name for a member from its type.
             */
            function getPropertyNameFromType(type) {
                if (type.flags & 8192 /* UniqueESSymbol */) {
                    return type.escapedName;
                }
                if (type.flags & (128 /* StringLiteral */ | 256 /* NumberLiteral */)) {
                    return ts.escapeLeadingUnderscores("" + type.value);
                }
                return ts.Debug.fail();
            }
            /**
             * Adds a declaration to a late-bound dynamic member. This performs the same function for
             * late-bound members that `addDeclarationToSymbol` in binder.ts performs for early-bound
             * members.
             */
            function addDeclarationToLateBoundSymbol(symbol, member, symbolFlags) {
                ts.Debug.assert(!!(ts.getCheckFlags(symbol) & 4096 /* Late */), "Expected a late-bound symbol.");
                symbol.flags |= symbolFlags;
                getSymbolLinks(member.symbol).lateSymbol = symbol;
                if (!symbol.declarations) {
                    symbol.declarations = [member];
                }
                else {
                    symbol.declarations.push(member);
                }
                if (symbolFlags & 111551 /* Value */) {
                    if (!symbol.valueDeclaration || symbol.valueDeclaration.kind !== member.kind) {
                        symbol.valueDeclaration = member;
                    }
                }
            }
            /**
             * Performs late-binding of a dynamic member. This performs the same function for
             * late-bound members that `declareSymbol` in binder.ts performs for early-bound
             * members.
             *
             * If a symbol is a dynamic name from a computed property, we perform an additional "late"
             * binding phase to attempt to resolve the name for the symbol from the type of the computed
             * property's expression. If the type of the expression is a string-literal, numeric-literal,
             * or unique symbol type, we can use that type as the name of the symbol.
             *
             * For example, given:
             *
             *   const x = Symbol();
             *
             *   interface I {
             *     [x]: number;
             *   }
             *
             * The binder gives the property `[x]: number` a special symbol with the name "__computed".
             * In the late-binding phase we can type-check the expression `x` and see that it has a
             * unique symbol type which we can then use as the name of the member. This allows users
             * to define custom symbols that can be used in the members of an object type.
             *
             * @param parent The containing symbol for the member.
             * @param earlySymbols The early-bound symbols of the parent.
             * @param lateSymbols The late-bound symbols of the parent.
             * @param decl The member to bind.
             */
            function lateBindMember(parent, earlySymbols, lateSymbols, decl) {
                ts.Debug.assert(!!decl.symbol, "The member is expected to have a symbol.");
                var links = getNodeLinks(decl);
                if (!links.resolvedSymbol) {
                    // In the event we attempt to resolve the late-bound name of this member recursively,
                    // fall back to the early-bound name of this member.
                    links.resolvedSymbol = decl.symbol;
                    var declName = ts.isBinaryExpression(decl) ? decl.left : decl.name;
                    var type = ts.isElementAccessExpression(declName) ? checkExpressionCached(declName.argumentExpression) : checkComputedPropertyName(declName);
                    if (isTypeUsableAsPropertyName(type)) {
                        var memberName = getPropertyNameFromType(type);
                        var symbolFlags = decl.symbol.flags;
                        // Get or add a late-bound symbol for the member. This allows us to merge late-bound accessor declarations.
                        var lateSymbol = lateSymbols.get(memberName);
                        if (!lateSymbol)
                            lateSymbols.set(memberName, lateSymbol = createSymbol(0 /* None */, memberName, 4096 /* Late */));
                        // Report an error if a late-bound member has the same name as an early-bound member,
                        // or if we have another early-bound symbol declaration with the same name and
                        // conflicting flags.
                        var earlySymbol = earlySymbols && earlySymbols.get(memberName);
                        if (lateSymbol.flags & getExcludedSymbolFlags(symbolFlags) || earlySymbol) {
                            // If we have an existing early-bound member, combine its declarations so that we can
                            // report an error at each declaration.
                            var declarations = earlySymbol ? ts.concatenate(earlySymbol.declarations, lateSymbol.declarations) : lateSymbol.declarations;
                            var name_3 = !(type.flags & 8192 /* UniqueESSymbol */) && ts.unescapeLeadingUnderscores(memberName) || ts.declarationNameToString(declName);
                            ts.forEach(declarations, function (declaration) { return error(ts.getNameOfDeclaration(declaration) || declaration, ts.Diagnostics.Property_0_was_also_declared_here, name_3); });
                            error(declName || decl, ts.Diagnostics.Duplicate_property_0, name_3);
                            lateSymbol = createSymbol(0 /* None */, memberName, 4096 /* Late */);
                        }
                        lateSymbol.nameType = type;
                        addDeclarationToLateBoundSymbol(lateSymbol, decl, symbolFlags);
                        if (lateSymbol.parent) {
                            ts.Debug.assert(lateSymbol.parent === parent, "Existing symbol parent should match new one");
                        }
                        else {
                            lateSymbol.parent = parent;
                        }
                        return links.resolvedSymbol = lateSymbol;
                    }
                }
                return links.resolvedSymbol;
            }
            function getResolvedMembersOrExportsOfSymbol(symbol, resolutionKind) {
                var links = getSymbolLinks(symbol);
                if (!links[resolutionKind]) {
                    var isStatic = resolutionKind === "resolvedExports" /* resolvedExports */;
                    var earlySymbols = !isStatic ? symbol.members :
                        symbol.flags & 1536 /* Module */ ? getExportsOfModuleWorker(symbol) :
                            symbol.exports;
                    // In the event we recursively resolve the members/exports of the symbol, we
                    // set the initial value of resolvedMembers/resolvedExports to the early-bound
                    // members/exports of the symbol.
                    links[resolutionKind] = earlySymbols || emptySymbols;
                    // fill in any as-yet-unresolved late-bound members.
                    var lateSymbols = ts.createSymbolTable();
                    for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        var members = ts.getMembersOfDeclaration(decl);
                        if (members) {
                            for (var _b = 0, members_5 = members; _b < members_5.length; _b++) {
                                var member = members_5[_b];
                                if (isStatic === ts.hasStaticModifier(member) && hasLateBindableName(member)) {
                                    lateBindMember(symbol, earlySymbols, lateSymbols, member);
                                }
                            }
                        }
                    }
                    var assignments = symbol.assignmentDeclarationMembers;
                    if (assignments) {
                        var decls = ts.arrayFrom(assignments.values());
                        for (var _c = 0, decls_1 = decls; _c < decls_1.length; _c++) {
                            var member = decls_1[_c];
                            var assignmentKind = ts.getAssignmentDeclarationKind(member);
                            var isInstanceMember = assignmentKind === 3 /* PrototypeProperty */
                                || assignmentKind === 4 /* ThisProperty */
                                || assignmentKind === 9 /* ObjectDefinePrototypeProperty */
                                || assignmentKind === 6 /* Prototype */; // A straight `Prototype` assignment probably can never have a computed name
                            if (isStatic === !isInstanceMember && hasLateBindableName(member)) {
                                lateBindMember(symbol, earlySymbols, lateSymbols, member);
                            }
                        }
                    }
                    links[resolutionKind] = combineSymbolTables(earlySymbols, lateSymbols) || emptySymbols;
                }
                return links[resolutionKind];
            }
            /**
             * Gets a SymbolTable containing both the early- and late-bound members of a symbol.
             *
             * For a description of late-binding, see `lateBindMember`.
             */
            function getMembersOfSymbol(symbol) {
                return symbol.flags & 6256 /* LateBindingContainer */
                    ? getResolvedMembersOrExportsOfSymbol(symbol, "resolvedMembers" /* resolvedMembers */)
                    : symbol.members || emptySymbols;
            }
            /**
             * If a symbol is the dynamic name of the member of an object type, get the late-bound
             * symbol of the member.
             *
             * For a description of late-binding, see `lateBindMember`.
             */
            function getLateBoundSymbol(symbol) {
                if (symbol.flags & 106500 /* ClassMember */ && symbol.escapedName === "__computed" /* Computed */) {
                    var links = getSymbolLinks(symbol);
                    if (!links.lateSymbol && ts.some(symbol.declarations, hasLateBindableName)) {
                        // force late binding of members/exports. This will set the late-bound symbol
                        var parent = getMergedSymbol(symbol.parent);
                        if (ts.some(symbol.declarations, ts.hasStaticModifier)) {
                            getExportsOfSymbol(parent);
                        }
                        else {
                            getMembersOfSymbol(parent);
                        }
                    }
                    return links.lateSymbol || (links.lateSymbol = symbol);
                }
                return symbol;
            }
            function getTypeWithThisArgument(type, thisArgument, needApparentType) {
                if (ts.getObjectFlags(type) & 4 /* Reference */) {
                    var target = type.target;
                    var typeArguments = getTypeArguments(type);
                    if (ts.length(target.typeParameters) === ts.length(typeArguments)) {
                        var ref = createTypeReference(target, ts.concatenate(typeArguments, [thisArgument || target.thisType]));
                        return needApparentType ? getApparentType(ref) : ref;
                    }
                }
                else if (type.flags & 2097152 /* Intersection */) {
                    return getIntersectionType(ts.map(type.types, function (t) { return getTypeWithThisArgument(t, thisArgument, needApparentType); }));
                }
                return needApparentType ? getApparentType(type) : type;
            }
            function resolveObjectTypeMembers(type, source, typeParameters, typeArguments) {
                var mapper;
                var members;
                var callSignatures;
                var constructSignatures;
                var stringIndexInfo;
                var numberIndexInfo;
                if (ts.rangeEquals(typeParameters, typeArguments, 0, typeParameters.length)) {
                    mapper = identityMapper;
                    members = source.symbol ? getMembersOfSymbol(source.symbol) : ts.createSymbolTable(source.declaredProperties);
                    callSignatures = source.declaredCallSignatures;
                    constructSignatures = source.declaredConstructSignatures;
                    stringIndexInfo = source.declaredStringIndexInfo;
                    numberIndexInfo = source.declaredNumberIndexInfo;
                }
                else {
                    mapper = createTypeMapper(typeParameters, typeArguments);
                    members = createInstantiatedSymbolTable(source.declaredProperties, mapper, /*mappingThisOnly*/ typeParameters.length === 1);
                    callSignatures = instantiateSignatures(source.declaredCallSignatures, mapper);
                    constructSignatures = instantiateSignatures(source.declaredConstructSignatures, mapper);
                    stringIndexInfo = instantiateIndexInfo(source.declaredStringIndexInfo, mapper);
                    numberIndexInfo = instantiateIndexInfo(source.declaredNumberIndexInfo, mapper);
                }
                var baseTypes = getBaseTypes(source);
                if (baseTypes.length) {
                    if (source.symbol && members === getMembersOfSymbol(source.symbol)) {
                        members = ts.createSymbolTable(source.declaredProperties);
                    }
                    setStructuredTypeMembers(type, members, callSignatures, constructSignatures, stringIndexInfo, numberIndexInfo);
                    var thisArgument = ts.lastOrUndefined(typeArguments);
                    for (var _i = 0, baseTypes_1 = baseTypes; _i < baseTypes_1.length; _i++) {
                        var baseType = baseTypes_1[_i];
                        var instantiatedBaseType = thisArgument ? getTypeWithThisArgument(instantiateType(baseType, mapper), thisArgument) : baseType;
                        addInheritedMembers(members, getPropertiesOfType(instantiatedBaseType));
                        callSignatures = ts.concatenate(callSignatures, getSignaturesOfType(instantiatedBaseType, 0 /* Call */));
                        constructSignatures = ts.concatenate(constructSignatures, getSignaturesOfType(instantiatedBaseType, 1 /* Construct */));
                        if (!stringIndexInfo) {
                            stringIndexInfo = instantiatedBaseType === anyType ?
                                createIndexInfo(anyType, /*isReadonly*/ false) :
                                getIndexInfoOfType(instantiatedBaseType, 0 /* String */);
                        }
                        numberIndexInfo = numberIndexInfo || getIndexInfoOfType(instantiatedBaseType, 1 /* Number */);
                    }
                }
                setStructuredTypeMembers(type, members, callSignatures, constructSignatures, stringIndexInfo, numberIndexInfo);
            }
            function resolveClassOrInterfaceMembers(type) {
                resolveObjectTypeMembers(type, resolveDeclaredMembers(type), ts.emptyArray, ts.emptyArray);
            }
            function resolveTypeReferenceMembers(type) {
                var source = resolveDeclaredMembers(type.target);
                var typeParameters = ts.concatenate(source.typeParameters, [source.thisType]);
                var typeArguments = getTypeArguments(type);
                var paddedTypeArguments = typeArguments.length === typeParameters.length ? typeArguments : ts.concatenate(typeArguments, [type]);
                resolveObjectTypeMembers(type, source, typeParameters, paddedTypeArguments);
            }
            function createSignature(declaration, typeParameters, thisParameter, parameters, resolvedReturnType, resolvedTypePredicate, minArgumentCount, flags) {
                var sig = new Signature(checker, flags);
                sig.declaration = declaration;
                sig.typeParameters = typeParameters;
                sig.parameters = parameters;
                sig.thisParameter = thisParameter;
                sig.resolvedReturnType = resolvedReturnType;
                sig.resolvedTypePredicate = resolvedTypePredicate;
                sig.minArgumentCount = minArgumentCount;
                sig.target = undefined;
                sig.mapper = undefined;
                sig.unionSignatures = undefined;
                return sig;
            }
            function cloneSignature(sig) {
                var result = createSignature(sig.declaration, sig.typeParameters, sig.thisParameter, sig.parameters, /*resolvedReturnType*/ undefined, 
                /*resolvedTypePredicate*/ undefined, sig.minArgumentCount, sig.flags & 3 /* PropagatingFlags */);
                result.target = sig.target;
                result.mapper = sig.mapper;
                result.unionSignatures = sig.unionSignatures;
                return result;
            }
            function createUnionSignature(signature, unionSignatures) {
                var result = cloneSignature(signature);
                result.unionSignatures = unionSignatures;
                result.target = undefined;
                result.mapper = undefined;
                return result;
            }
            function getOptionalCallSignature(signature, callChainFlags) {
                if ((signature.flags & 12 /* CallChainFlags */) === callChainFlags) {
                    return signature;
                }
                if (!signature.optionalCallSignatureCache) {
                    signature.optionalCallSignatureCache = {};
                }
                var key = callChainFlags === 4 /* IsInnerCallChain */ ? "inner" : "outer";
                return signature.optionalCallSignatureCache[key]
                    || (signature.optionalCallSignatureCache[key] = createOptionalCallSignature(signature, callChainFlags));
            }
            function createOptionalCallSignature(signature, callChainFlags) {
                ts.Debug.assert(callChainFlags === 4 /* IsInnerCallChain */ || callChainFlags === 8 /* IsOuterCallChain */, "An optional call signature can either be for an inner call chain or an outer call chain, but not both.");
                var result = cloneSignature(signature);
                result.flags |= callChainFlags;
                return result;
            }
            function getExpandedParameters(sig) {
                if (signatureHasRestParameter(sig)) {
                    var restIndex_1 = sig.parameters.length - 1;
                    var restParameter = sig.parameters[restIndex_1];
                    var restType = getTypeOfSymbol(restParameter);
                    if (isTupleType(restType)) {
                        var elementTypes = getTypeArguments(restType);
                        var minLength_1 = restType.target.minLength;
                        var tupleRestIndex_1 = restType.target.hasRestElement ? elementTypes.length - 1 : -1;
                        var restParams = ts.map(elementTypes, function (t, i) {
                            var name = getParameterNameAtPosition(sig, restIndex_1 + i);
                            var checkFlags = i === tupleRestIndex_1 ? 32768 /* RestParameter */ :
                                i >= minLength_1 ? 16384 /* OptionalParameter */ : 0;
                            var symbol = createSymbol(1 /* FunctionScopedVariable */, name, checkFlags);
                            symbol.type = i === tupleRestIndex_1 ? createArrayType(t) : t;
                            return symbol;
                        });
                        return ts.concatenate(sig.parameters.slice(0, restIndex_1), restParams);
                    }
                }
                return sig.parameters;
            }
            function getDefaultConstructSignatures(classType) {
                var baseConstructorType = getBaseConstructorTypeOfClass(classType);
                var baseSignatures = getSignaturesOfType(baseConstructorType, 1 /* Construct */);
                if (baseSignatures.length === 0) {
                    return [createSignature(undefined, classType.localTypeParameters, undefined, ts.emptyArray, classType, /*resolvedTypePredicate*/ undefined, 0, 0 /* None */)];
                }
                var baseTypeNode = getBaseTypeNodeOfClass(classType);
                var isJavaScript = ts.isInJSFile(baseTypeNode);
                var typeArguments = typeArgumentsFromTypeReferenceNode(baseTypeNode);
                var typeArgCount = ts.length(typeArguments);
                var result = [];
                for (var _i = 0, baseSignatures_1 = baseSignatures; _i < baseSignatures_1.length; _i++) {
                    var baseSig = baseSignatures_1[_i];
                    var minTypeArgumentCount = getMinTypeArgumentCount(baseSig.typeParameters);
                    var typeParamCount = ts.length(baseSig.typeParameters);
                    if (isJavaScript || typeArgCount >= minTypeArgumentCount && typeArgCount <= typeParamCount) {
                        var sig = typeParamCount ? createSignatureInstantiation(baseSig, fillMissingTypeArguments(typeArguments, baseSig.typeParameters, minTypeArgumentCount, isJavaScript)) : cloneSignature(baseSig);
                        sig.typeParameters = classType.localTypeParameters;
                        sig.resolvedReturnType = classType;
                        result.push(sig);
                    }
                }
                return result;
            }
            function findMatchingSignature(signatureList, signature, partialMatch, ignoreThisTypes, ignoreReturnTypes) {
                for (var _i = 0, signatureList_1 = signatureList; _i < signatureList_1.length; _i++) {
                    var s = signatureList_1[_i];
                    if (compareSignaturesIdentical(s, signature, partialMatch, ignoreThisTypes, ignoreReturnTypes, partialMatch ? compareTypesSubtypeOf : compareTypesIdentical)) {
                        return s;
                    }
                }
            }
            function findMatchingSignatures(signatureLists, signature, listIndex) {
                if (signature.typeParameters) {
                    // We require an exact match for generic signatures, so we only return signatures from the first
                    // signature list and only if they have exact matches in the other signature lists.
                    if (listIndex > 0) {
                        return undefined;
                    }
                    for (var i = 1; i < signatureLists.length; i++) {
                        if (!findMatchingSignature(signatureLists[i], signature, /*partialMatch*/ false, /*ignoreThisTypes*/ false, /*ignoreReturnTypes*/ false)) {
                            return undefined;
                        }
                    }
                    return [signature];
                }
                var result;
                for (var i = 0; i < signatureLists.length; i++) {
                    // Allow matching non-generic signatures to have excess parameters and different return types.
                    // Prefer matching this types if possible.
                    var match = i === listIndex ? signature : findMatchingSignature(signatureLists[i], signature, /*partialMatch*/ true, /*ignoreThisTypes*/ false, /*ignoreReturnTypes*/ true);
                    if (!match) {
                        return undefined;
                    }
                    result = ts.appendIfUnique(result, match);
                }
                return result;
            }
            // The signatures of a union type are those signatures that are present in each of the constituent types.
            // Generic signatures must match exactly, but non-generic signatures are allowed to have extra optional
            // parameters and may differ in return types. When signatures differ in return types, the resulting return
            // type is the union of the constituent return types.
            function getUnionSignatures(signatureLists) {
                var result;
                var indexWithLengthOverOne;
                for (var i = 0; i < signatureLists.length; i++) {
                    if (signatureLists[i].length === 0)
                        return ts.emptyArray;
                    if (signatureLists[i].length > 1) {
                        indexWithLengthOverOne = indexWithLengthOverOne === undefined ? i : -1; // -1 is a signal there are multiple overload sets
                    }
                    for (var _i = 0, _a = signatureLists[i]; _i < _a.length; _i++) {
                        var signature = _a[_i];
                        // Only process signatures with parameter lists that aren't already in the result list
                        if (!result || !findMatchingSignature(result, signature, /*partialMatch*/ false, /*ignoreThisTypes*/ false, /*ignoreReturnTypes*/ true)) {
                            var unionSignatures = findMatchingSignatures(signatureLists, signature, i);
                            if (unionSignatures) {
                                var s = signature;
                                // Union the result types when more than one signature matches
                                if (unionSignatures.length > 1) {
                                    var thisParameter = signature.thisParameter;
                                    var firstThisParameterOfUnionSignatures = ts.forEach(unionSignatures, function (sig) { return sig.thisParameter; });
                                    if (firstThisParameterOfUnionSignatures) {
                                        var thisType = getIntersectionType(ts.mapDefined(unionSignatures, function (sig) { return sig.thisParameter && getTypeOfSymbol(sig.thisParameter); }));
                                        thisParameter = createSymbolWithType(firstThisParameterOfUnionSignatures, thisType);
                                    }
                                    s = createUnionSignature(signature, unionSignatures);
                                    s.thisParameter = thisParameter;
                                }
                                (result || (result = [])).push(s);
                            }
                        }
                    }
                }
                if (!ts.length(result) && indexWithLengthOverOne !== -1) {
                    // No sufficiently similar signature existed to subsume all the other signatures in the union - time to see if we can make a single
                    // signature that handles all over them. We only do this when there are overloads in only one constituent.
                    // (Overloads are conditional in nature and having overloads in multiple constituents would necessitate making a power set of
                    // signatures from the type, whose ordering would be non-obvious)
                    var masterList = signatureLists[indexWithLengthOverOne !== undefined ? indexWithLengthOverOne : 0];
                    var results = masterList.slice();
                    var _loop_8 = function (signatures) {
                        if (signatures !== masterList) {
                            var signature_1 = signatures[0];
                            ts.Debug.assert(!!signature_1, "getUnionSignatures bails early on empty signature lists and should not have empty lists on second pass");
                            results = signature_1.typeParameters && ts.some(results, function (s) { return !!s.typeParameters; }) ? undefined : ts.map(results, function (sig) { return combineSignaturesOfUnionMembers(sig, signature_1); });
                            if (!results) {
                                return "break";
                            }
                        }
                    };
                    for (var _b = 0, signatureLists_1 = signatureLists; _b < signatureLists_1.length; _b++) {
                        var signatures = signatureLists_1[_b];
                        var state_3 = _loop_8(signatures);
                        if (state_3 === "break")
                            break;
                    }
                    result = results;
                }
                return result || ts.emptyArray;
            }
            function combineUnionThisParam(left, right) {
                if (!left || !right) {
                    return left || right;
                }
                // A signature `this` type might be a read or a write position... It's very possible that it should be invariant
                // and we should refuse to merge signatures if there are `this` types and they do not match. However, so as to be
                // permissive when calling, for now, we'll intersect the `this` types just like we do for param types in union signatures.
                var thisType = getIntersectionType([getTypeOfSymbol(left), getTypeOfSymbol(right)]);
                return createSymbolWithType(left, thisType);
            }
            function combineUnionParameters(left, right) {
                var leftCount = getParameterCount(left);
                var rightCount = getParameterCount(right);
                var longest = leftCount >= rightCount ? left : right;
                var shorter = longest === left ? right : left;
                var longestCount = longest === left ? leftCount : rightCount;
                var eitherHasEffectiveRest = (hasEffectiveRestParameter(left) || hasEffectiveRestParameter(right));
                var needsExtraRestElement = eitherHasEffectiveRest && !hasEffectiveRestParameter(longest);
                var params = new Array(longestCount + (needsExtraRestElement ? 1 : 0));
                for (var i = 0; i < longestCount; i++) {
                    var longestParamType = tryGetTypeAtPosition(longest, i);
                    var shorterParamType = tryGetTypeAtPosition(shorter, i) || unknownType;
                    var unionParamType = getIntersectionType([longestParamType, shorterParamType]);
                    var isRestParam = eitherHasEffectiveRest && !needsExtraRestElement && i === (longestCount - 1);
                    var isOptional = i >= getMinArgumentCount(longest) && i >= getMinArgumentCount(shorter);
                    var leftName = i >= leftCount ? undefined : getParameterNameAtPosition(left, i);
                    var rightName = i >= rightCount ? undefined : getParameterNameAtPosition(right, i);
                    var paramName = leftName === rightName ? leftName :
                        !leftName ? rightName :
                            !rightName ? leftName :
                                undefined;
                    var paramSymbol = createSymbol(1 /* FunctionScopedVariable */ | (isOptional && !isRestParam ? 16777216 /* Optional */ : 0), paramName || "arg" + i);
                    paramSymbol.type = isRestParam ? createArrayType(unionParamType) : unionParamType;
                    params[i] = paramSymbol;
                }
                if (needsExtraRestElement) {
                    var restParamSymbol = createSymbol(1 /* FunctionScopedVariable */, "args");
                    restParamSymbol.type = createArrayType(getTypeAtPosition(shorter, longestCount));
                    params[longestCount] = restParamSymbol;
                }
                return params;
            }
            function combineSignaturesOfUnionMembers(left, right) {
                var declaration = left.declaration;
                var params = combineUnionParameters(left, right);
                var thisParam = combineUnionThisParam(left.thisParameter, right.thisParameter);
                var minArgCount = Math.max(left.minArgumentCount, right.minArgumentCount);
                var result = createSignature(declaration, left.typeParameters || right.typeParameters, thisParam, params, 
                /*resolvedReturnType*/ undefined, 
                /*resolvedTypePredicate*/ undefined, minArgCount, (left.flags | right.flags) & 3 /* PropagatingFlags */);
                result.unionSignatures = ts.concatenate(left.unionSignatures || [left], [right]);
                return result;
            }
            function getUnionIndexInfo(types, kind) {
                var indexTypes = [];
                var isAnyReadonly = false;
                for (var _i = 0, types_2 = types; _i < types_2.length; _i++) {
                    var type = types_2[_i];
                    var indexInfo = getIndexInfoOfType(type, kind);
                    if (!indexInfo) {
                        return undefined;
                    }
                    indexTypes.push(indexInfo.type);
                    isAnyReadonly = isAnyReadonly || indexInfo.isReadonly;
                }
                return createIndexInfo(getUnionType(indexTypes, 2 /* Subtype */), isAnyReadonly);
            }
            function resolveUnionTypeMembers(type) {
                // The members and properties collections are empty for union types. To get all properties of a union
                // type use getPropertiesOfType (only the language service uses this).
                var callSignatures = getUnionSignatures(ts.map(type.types, function (t) { return t === globalFunctionType ? [unknownSignature] : getSignaturesOfType(t, 0 /* Call */); }));
                var constructSignatures = getUnionSignatures(ts.map(type.types, function (t) { return getSignaturesOfType(t, 1 /* Construct */); }));
                var stringIndexInfo = getUnionIndexInfo(type.types, 0 /* String */);
                var numberIndexInfo = getUnionIndexInfo(type.types, 1 /* Number */);
                setStructuredTypeMembers(type, emptySymbols, callSignatures, constructSignatures, stringIndexInfo, numberIndexInfo);
            }
            function intersectTypes(type1, type2) {
                return !type1 ? type2 : !type2 ? type1 : getIntersectionType([type1, type2]);
            }
            function intersectIndexInfos(info1, info2) {
                return !info1 ? info2 : !info2 ? info1 : createIndexInfo(getIntersectionType([info1.type, info2.type]), info1.isReadonly && info2.isReadonly);
            }
            function unionSpreadIndexInfos(info1, info2) {
                return info1 && info2 && createIndexInfo(getUnionType([info1.type, info2.type]), info1.isReadonly || info2.isReadonly);
            }
            function findMixins(types) {
                var constructorTypeCount = ts.countWhere(types, function (t) { return getSignaturesOfType(t, 1 /* Construct */).length > 0; });
                var mixinFlags = ts.map(types, isMixinConstructorType);
                if (constructorTypeCount > 0 && constructorTypeCount === ts.countWhere(mixinFlags, function (b) { return b; })) {
                    var firstMixinIndex = mixinFlags.indexOf(/*searchElement*/ true);
                    mixinFlags[firstMixinIndex] = false;
                }
                return mixinFlags;
            }
            function includeMixinType(type, types, mixinFlags, index) {
                var mixedTypes = [];
                for (var i = 0; i < types.length; i++) {
                    if (i === index) {
                        mixedTypes.push(type);
                    }
                    else if (mixinFlags[i]) {
                        mixedTypes.push(getReturnTypeOfSignature(getSignaturesOfType(types[i], 1 /* Construct */)[0]));
                    }
                }
                return getIntersectionType(mixedTypes);
            }
            function resolveIntersectionTypeMembers(type) {
                // The members and properties collections are empty for intersection types. To get all properties of an
                // intersection type use getPropertiesOfType (only the language service uses this).
                var callSignatures;
                var constructSignatures;
                var stringIndexInfo;
                var numberIndexInfo;
                var types = type.types;
                var mixinFlags = findMixins(types);
                var mixinCount = ts.countWhere(mixinFlags, function (b) { return b; });
                var _loop_9 = function (i) {
                    var t = type.types[i];
                    // When an intersection type contains mixin constructor types, the construct signatures from
                    // those types are discarded and their return types are mixed into the return types of all
                    // other construct signatures in the intersection type. For example, the intersection type
                    // '{ new(...args: any[]) => A } & { new(s: string) => B }' has a single construct signature
                    // 'new(s: string) => A & B'.
                    if (!mixinFlags[i]) {
                        var signatures = getSignaturesOfType(t, 1 /* Construct */);
                        if (signatures.length && mixinCount > 0) {
                            signatures = ts.map(signatures, function (s) {
                                var clone = cloneSignature(s);
                                clone.resolvedReturnType = includeMixinType(getReturnTypeOfSignature(s), types, mixinFlags, i);
                                return clone;
                            });
                        }
                        constructSignatures = appendSignatures(constructSignatures, signatures);
                    }
                    callSignatures = appendSignatures(callSignatures, getSignaturesOfType(t, 0 /* Call */));
                    stringIndexInfo = intersectIndexInfos(stringIndexInfo, getIndexInfoOfType(t, 0 /* String */));
                    numberIndexInfo = intersectIndexInfos(numberIndexInfo, getIndexInfoOfType(t, 1 /* Number */));
                };
                for (var i = 0; i < types.length; i++) {
                    _loop_9(i);
                }
                setStructuredTypeMembers(type, emptySymbols, callSignatures || ts.emptyArray, constructSignatures || ts.emptyArray, stringIndexInfo, numberIndexInfo);
            }
            function appendSignatures(signatures, newSignatures) {
                var _loop_10 = function (sig) {
                    if (!signatures || ts.every(signatures, function (s) { return !compareSignaturesIdentical(s, sig, /*partialMatch*/ false, /*ignoreThisTypes*/ false, /*ignoreReturnTypes*/ false, compareTypesIdentical); })) {
                        signatures = ts.append(signatures, sig);
                    }
                };
                for (var _i = 0, newSignatures_1 = newSignatures; _i < newSignatures_1.length; _i++) {
                    var sig = newSignatures_1[_i];
                    _loop_10(sig);
                }
                return signatures;
            }
            /**
             * Converts an AnonymousType to a ResolvedType.
             */
            function resolveAnonymousTypeMembers(type) {
                var symbol = getMergedSymbol(type.symbol);
                if (type.target) {
                    setStructuredTypeMembers(type, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
                    var members = createInstantiatedSymbolTable(getPropertiesOfObjectType(type.target), type.mapper, /*mappingThisOnly*/ false);
                    var callSignatures = instantiateSignatures(getSignaturesOfType(type.target, 0 /* Call */), type.mapper);
                    var constructSignatures = instantiateSignatures(getSignaturesOfType(type.target, 1 /* Construct */), type.mapper);
                    var stringIndexInfo = instantiateIndexInfo(getIndexInfoOfType(type.target, 0 /* String */), type.mapper);
                    var numberIndexInfo = instantiateIndexInfo(getIndexInfoOfType(type.target, 1 /* Number */), type.mapper);
                    setStructuredTypeMembers(type, members, callSignatures, constructSignatures, stringIndexInfo, numberIndexInfo);
                }
                else if (symbol.flags & 2048 /* TypeLiteral */) {
                    setStructuredTypeMembers(type, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
                    var members = getMembersOfSymbol(symbol);
                    var callSignatures = getSignaturesOfSymbol(members.get("__call" /* Call */));
                    var constructSignatures = getSignaturesOfSymbol(members.get("__new" /* New */));
                    var stringIndexInfo = getIndexInfoOfSymbol(symbol, 0 /* String */);
                    var numberIndexInfo = getIndexInfoOfSymbol(symbol, 1 /* Number */);
                    setStructuredTypeMembers(type, members, callSignatures, constructSignatures, stringIndexInfo, numberIndexInfo);
                }
                else {
                    // Combinations of function, class, enum and module
                    var members = emptySymbols;
                    var stringIndexInfo = void 0;
                    if (symbol.exports) {
                        members = getExportsOfSymbol(symbol);
                        if (symbol === globalThisSymbol) {
                            var varsOnly_1 = ts.createMap();
                            members.forEach(function (p) {
                                if (!(p.flags & 418 /* BlockScoped */)) {
                                    varsOnly_1.set(p.escapedName, p);
                                }
                            });
                            members = varsOnly_1;
                        }
                    }
                    setStructuredTypeMembers(type, members, ts.emptyArray, ts.emptyArray, undefined, undefined);
                    if (symbol.flags & 32 /* Class */) {
                        var classType = getDeclaredTypeOfClassOrInterface(symbol);
                        var baseConstructorType = getBaseConstructorTypeOfClass(classType);
                        if (baseConstructorType.flags & (524288 /* Object */ | 2097152 /* Intersection */ | 8650752 /* TypeVariable */)) {
                            members = ts.createSymbolTable(getNamedMembers(members));
                            addInheritedMembers(members, getPropertiesOfType(baseConstructorType));
                        }
                        else if (baseConstructorType === anyType) {
                            stringIndexInfo = createIndexInfo(anyType, /*isReadonly*/ false);
                        }
                    }
                    var numberIndexInfo = symbol.flags & 384 /* Enum */ && (getDeclaredTypeOfSymbol(symbol).flags & 32 /* Enum */ ||
                        ts.some(type.properties, function (prop) { return !!(getTypeOfSymbol(prop).flags & 296 /* NumberLike */); })) ? enumNumberIndexInfo : undefined;
                    setStructuredTypeMembers(type, members, ts.emptyArray, ts.emptyArray, stringIndexInfo, numberIndexInfo);
                    // We resolve the members before computing the signatures because a signature may use
                    // typeof with a qualified name expression that circularly references the type we are
                    // in the process of resolving (see issue #6072). The temporarily empty signature list
                    // will never be observed because a qualified name can't reference signatures.
                    if (symbol.flags & (16 /* Function */ | 8192 /* Method */)) {
                        type.callSignatures = getSignaturesOfSymbol(symbol);
                    }
                    // And likewise for construct signatures for classes
                    if (symbol.flags & 32 /* Class */) {
                        var classType_1 = getDeclaredTypeOfClassOrInterface(symbol);
                        var constructSignatures = symbol.members ? getSignaturesOfSymbol(symbol.members.get("__constructor" /* Constructor */)) : ts.emptyArray;
                        if (symbol.flags & 16 /* Function */) {
                            constructSignatures = ts.addRange(constructSignatures.slice(), ts.mapDefined(type.callSignatures, function (sig) {
                                return isJSConstructor(sig.declaration) ?
                                    createSignature(sig.declaration, sig.typeParameters, sig.thisParameter, sig.parameters, classType_1, /*resolvedTypePredicate*/ undefined, sig.minArgumentCount, sig.flags & 3 /* PropagatingFlags */) :
                                    undefined;
                            }));
                        }
                        if (!constructSignatures.length) {
                            constructSignatures = getDefaultConstructSignatures(classType_1);
                        }
                        type.constructSignatures = constructSignatures;
                    }
                }
            }
            function resolveReverseMappedTypeMembers(type) {
                var indexInfo = getIndexInfoOfType(type.source, 0 /* String */);
                var modifiers = getMappedTypeModifiers(type.mappedType);
                var readonlyMask = modifiers & 1 /* IncludeReadonly */ ? false : true;
                var optionalMask = modifiers & 4 /* IncludeOptional */ ? 0 : 16777216 /* Optional */;
                var stringIndexInfo = indexInfo && createIndexInfo(inferReverseMappedType(indexInfo.type, type.mappedType, type.constraintType), readonlyMask && indexInfo.isReadonly);
                var members = ts.createSymbolTable();
                for (var _i = 0, _a = getPropertiesOfType(type.source); _i < _a.length; _i++) {
                    var prop = _a[_i];
                    var checkFlags = 8192 /* ReverseMapped */ | (readonlyMask && isReadonlySymbol(prop) ? 8 /* Readonly */ : 0);
                    var inferredProp = createSymbol(4 /* Property */ | prop.flags & optionalMask, prop.escapedName, checkFlags);
                    inferredProp.declarations = prop.declarations;
                    inferredProp.nameType = getSymbolLinks(prop).nameType;
                    inferredProp.propertyType = getTypeOfSymbol(prop);
                    inferredProp.mappedType = type.mappedType;
                    inferredProp.constraintType = type.constraintType;
                    members.set(prop.escapedName, inferredProp);
                }
                setStructuredTypeMembers(type, members, ts.emptyArray, ts.emptyArray, stringIndexInfo, undefined);
            }
            // Return the lower bound of the key type in a mapped type. Intuitively, the lower
            // bound includes those keys that are known to always be present, for example because
            // because of constraints on type parameters (e.g. 'keyof T' for a constrained T).
            function getLowerBoundOfKeyType(type) {
                if (type.flags & (1 /* Any */ | 131068 /* Primitive */)) {
                    return type;
                }
                if (type.flags & 4194304 /* Index */) {
                    return getIndexType(getApparentType(type.type));
                }
                if (type.flags & 16777216 /* Conditional */) {
                    if (type.root.isDistributive) {
                        var checkType = type.checkType;
                        var constraint = getLowerBoundOfKeyType(checkType);
                        if (constraint !== checkType) {
                            var mapper = makeUnaryTypeMapper(type.root.checkType, constraint);
                            return getConditionalTypeInstantiation(type, combineTypeMappers(mapper, type.mapper));
                        }
                    }
                    return type;
                }
                if (type.flags & 1048576 /* Union */) {
                    return getUnionType(ts.sameMap(type.types, getLowerBoundOfKeyType));
                }
                if (type.flags & 2097152 /* Intersection */) {
                    return getIntersectionType(ts.sameMap(type.types, getLowerBoundOfKeyType));
                }
                return neverType;
            }
            /** Resolve the members of a mapped type { [P in K]: T } */
            function resolveMappedTypeMembers(type) {
                var members = ts.createSymbolTable();
                var stringIndexInfo;
                var numberIndexInfo;
                // Resolve upfront such that recursive references see an empty object type.
                setStructuredTypeMembers(type, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
                // In { [P in K]: T }, we refer to P as the type parameter type, K as the constraint type,
                // and T as the template type.
                var typeParameter = getTypeParameterFromMappedType(type);
                var constraintType = getConstraintTypeFromMappedType(type);
                var templateType = getTemplateTypeFromMappedType(type.target || type);
                var modifiersType = getApparentType(getModifiersTypeFromMappedType(type)); // The 'T' in 'keyof T'
                var templateModifiers = getMappedTypeModifiers(type);
                var include = keyofStringsOnly ? 128 /* StringLiteral */ : 8576 /* StringOrNumberLiteralOrUnique */;
                if (isMappedTypeWithKeyofConstraintDeclaration(type)) {
                    // We have a { [P in keyof T]: X }
                    for (var _i = 0, _a = getPropertiesOfType(modifiersType); _i < _a.length; _i++) {
                        var prop = _a[_i];
                        addMemberForKeyType(getLiteralTypeFromProperty(prop, include));
                    }
                    if (modifiersType.flags & 1 /* Any */ || getIndexInfoOfType(modifiersType, 0 /* String */)) {
                        addMemberForKeyType(stringType);
                    }
                    if (!keyofStringsOnly && getIndexInfoOfType(modifiersType, 1 /* Number */)) {
                        addMemberForKeyType(numberType);
                    }
                }
                else {
                    forEachType(getLowerBoundOfKeyType(constraintType), addMemberForKeyType);
                }
                setStructuredTypeMembers(type, members, ts.emptyArray, ts.emptyArray, stringIndexInfo, numberIndexInfo);
                function addMemberForKeyType(t) {
                    // Create a mapper from T to the current iteration type constituent. Then, if the
                    // mapped type is itself an instantiated type, combine the iteration mapper with the
                    // instantiation mapper.
                    var templateMapper = combineTypeMappers(type.mapper, createTypeMapper([typeParameter], [t]));
                    var propType = instantiateType(templateType, templateMapper);
                    // If the current iteration type constituent is a string literal type, create a property.
                    // Otherwise, for type string create a string index signature.
                    if (isTypeUsableAsPropertyName(t)) {
                        var propName = getPropertyNameFromType(t);
                        var modifiersProp = getPropertyOfType(modifiersType, propName);
                        var isOptional = !!(templateModifiers & 4 /* IncludeOptional */ ||
                            !(templateModifiers & 8 /* ExcludeOptional */) && modifiersProp && modifiersProp.flags & 16777216 /* Optional */);
                        var isReadonly = !!(templateModifiers & 1 /* IncludeReadonly */ ||
                            !(templateModifiers & 2 /* ExcludeReadonly */) && modifiersProp && isReadonlySymbol(modifiersProp));
                        var prop = createSymbol(4 /* Property */ | (isOptional ? 16777216 /* Optional */ : 0), propName, isReadonly ? 8 /* Readonly */ : 0);
                        // When creating an optional property in strictNullChecks mode, if 'undefined' isn't assignable to the
                        // type, we include 'undefined' in the type. Similarly, when creating a non-optional property in strictNullChecks
                        // mode, if the underlying property is optional we remove 'undefined' from the type.
                        prop.type = strictNullChecks && isOptional && !isTypeAssignableTo(undefinedType, propType) ? getOptionalType(propType) :
                            strictNullChecks && !isOptional && modifiersProp && modifiersProp.flags & 16777216 /* Optional */ ? getTypeWithFacts(propType, 524288 /* NEUndefined */) :
                                propType;
                        if (modifiersProp) {
                            prop.syntheticOrigin = modifiersProp;
                            prop.declarations = modifiersProp.declarations;
                        }
                        prop.nameType = t;
                        members.set(propName, prop);
                    }
                    else if (t.flags & (1 /* Any */ | 4 /* String */)) {
                        stringIndexInfo = createIndexInfo(propType, !!(templateModifiers & 1 /* IncludeReadonly */));
                    }
                    else if (t.flags & (8 /* Number */ | 32 /* Enum */)) {
                        numberIndexInfo = createIndexInfo(numberIndexInfo ? getUnionType([numberIndexInfo.type, propType]) : propType, !!(templateModifiers & 1 /* IncludeReadonly */));
                    }
                }
            }
            function getTypeParameterFromMappedType(type) {
                return type.typeParameter ||
                    (type.typeParameter = getDeclaredTypeOfTypeParameter(getSymbolOfNode(type.declaration.typeParameter)));
            }
            function getConstraintTypeFromMappedType(type) {
                return type.constraintType ||
                    (type.constraintType = getConstraintOfTypeParameter(getTypeParameterFromMappedType(type)) || errorType);
            }
            function getTemplateTypeFromMappedType(type) {
                return type.templateType ||
                    (type.templateType = type.declaration.type ?
                        instantiateType(addOptionality(getTypeFromTypeNode(type.declaration.type), !!(getMappedTypeModifiers(type) & 4 /* IncludeOptional */)), type.mapper || identityMapper) :
                        errorType);
            }
            function getConstraintDeclarationForMappedType(type) {
                return ts.getEffectiveConstraintOfTypeParameter(type.declaration.typeParameter);
            }
            function isMappedTypeWithKeyofConstraintDeclaration(type) {
                var constraintDeclaration = getConstraintDeclarationForMappedType(type); // TODO: GH#18217
                return constraintDeclaration.kind === 184 /* TypeOperator */ &&
                    constraintDeclaration.operator === 134 /* KeyOfKeyword */;
            }
            function getModifiersTypeFromMappedType(type) {
                if (!type.modifiersType) {
                    if (isMappedTypeWithKeyofConstraintDeclaration(type)) {
                        // If the constraint declaration is a 'keyof T' node, the modifiers type is T. We check
                        // AST nodes here because, when T is a non-generic type, the logic below eagerly resolves
                        // 'keyof T' to a literal union type and we can't recover T from that type.
                        type.modifiersType = instantiateType(getTypeFromTypeNode(getConstraintDeclarationForMappedType(type).type), type.mapper || identityMapper);
                    }
                    else {
                        // Otherwise, get the declared constraint type, and if the constraint type is a type parameter,
                        // get the constraint of that type parameter. If the resulting type is an indexed type 'keyof T',
                        // the modifiers type is T. Otherwise, the modifiers type is unknown.
                        var declaredType = getTypeFromMappedTypeNode(type.declaration);
                        var constraint = getConstraintTypeFromMappedType(declaredType);
                        var extendedConstraint = constraint && constraint.flags & 262144 /* TypeParameter */ ? getConstraintOfTypeParameter(constraint) : constraint;
                        type.modifiersType = extendedConstraint && extendedConstraint.flags & 4194304 /* Index */ ? instantiateType(extendedConstraint.type, type.mapper || identityMapper) : unknownType;
                    }
                }
                return type.modifiersType;
            }
            function getMappedTypeModifiers(type) {
                var declaration = type.declaration;
                return (declaration.readonlyToken ? declaration.readonlyToken.kind === 40 /* MinusToken */ ? 2 /* ExcludeReadonly */ : 1 /* IncludeReadonly */ : 0) |
                    (declaration.questionToken ? declaration.questionToken.kind === 40 /* MinusToken */ ? 8 /* ExcludeOptional */ : 4 /* IncludeOptional */ : 0);
            }
            function getMappedTypeOptionality(type) {
                var modifiers = getMappedTypeModifiers(type);
                return modifiers & 8 /* ExcludeOptional */ ? -1 : modifiers & 4 /* IncludeOptional */ ? 1 : 0;
            }
            function getCombinedMappedTypeOptionality(type) {
                var optionality = getMappedTypeOptionality(type);
                var modifiersType = getModifiersTypeFromMappedType(type);
                return optionality || (isGenericMappedType(modifiersType) ? getMappedTypeOptionality(modifiersType) : 0);
            }
            function isPartialMappedType(type) {
                return !!(ts.getObjectFlags(type) & 32 /* Mapped */ && getMappedTypeModifiers(type) & 4 /* IncludeOptional */);
            }
            function isGenericMappedType(type) {
                return !!(ts.getObjectFlags(type) & 32 /* Mapped */) && isGenericIndexType(getConstraintTypeFromMappedType(type));
            }
            function resolveStructuredTypeMembers(type) {
                if (!type.members) {
                    if (type.flags & 524288 /* Object */) {
                        if (type.objectFlags & 4 /* Reference */) {
                            resolveTypeReferenceMembers(type);
                        }
                        else if (type.objectFlags & 3 /* ClassOrInterface */) {
                            resolveClassOrInterfaceMembers(type);
                        }
                        else if (type.objectFlags & 2048 /* ReverseMapped */) {
                            resolveReverseMappedTypeMembers(type);
                        }
                        else if (type.objectFlags & 16 /* Anonymous */) {
                            resolveAnonymousTypeMembers(type);
                        }
                        else if (type.objectFlags & 32 /* Mapped */) {
                            resolveMappedTypeMembers(type);
                        }
                    }
                    else if (type.flags & 1048576 /* Union */) {
                        resolveUnionTypeMembers(type);
                    }
                    else if (type.flags & 2097152 /* Intersection */) {
                        resolveIntersectionTypeMembers(type);
                    }
                }
                return type;
            }
            /** Return properties of an object type or an empty array for other types */
            function getPropertiesOfObjectType(type) {
                if (type.flags & 524288 /* Object */) {
                    return resolveStructuredTypeMembers(type).properties;
                }
                return ts.emptyArray;
            }
            /** If the given type is an object type and that type has a property by the given name,
             * return the symbol for that property. Otherwise return undefined.
             */
            function getPropertyOfObjectType(type, name) {
                if (type.flags & 524288 /* Object */) {
                    var resolved = resolveStructuredTypeMembers(type);
                    var symbol = resolved.members.get(name);
                    if (symbol && symbolIsValue(symbol)) {
                        return symbol;
                    }
                }
            }
            function getPropertiesOfUnionOrIntersectionType(type) {
                if (!type.resolvedProperties) {
                    var members = ts.createSymbolTable();
                    for (var _i = 0, _a = type.types; _i < _a.length; _i++) {
                        var current = _a[_i];
                        for (var _b = 0, _c = getPropertiesOfType(current); _b < _c.length; _b++) {
                            var prop = _c[_b];
                            if (!members.has(prop.escapedName)) {
                                var combinedProp = getPropertyOfUnionOrIntersectionType(type, prop.escapedName);
                                if (combinedProp) {
                                    members.set(prop.escapedName, combinedProp);
                                }
                            }
                        }
                        // The properties of a union type are those that are present in all constituent types, so
                        // we only need to check the properties of the first type
                        if (type.flags & 1048576 /* Union */) {
                            break;
                        }
                    }
                    type.resolvedProperties = getNamedMembers(members);
                }
                return type.resolvedProperties;
            }
            function getPropertiesOfType(type) {
                type = getApparentType(type);
                return type.flags & 3145728 /* UnionOrIntersection */ ?
                    getPropertiesOfUnionOrIntersectionType(type) :
                    getPropertiesOfObjectType(type);
            }
            function isTypeInvalidDueToUnionDiscriminant(contextualType, obj) {
                var list = obj.properties;
                return list.some(function (property) {
                    var nameType = property.name && getLiteralTypeFromPropertyName(property.name);
                    var name = nameType && isTypeUsableAsPropertyName(nameType) ? getPropertyNameFromType(nameType) : undefined;
                    var expected = name === undefined ? undefined : getTypeOfPropertyOfType(contextualType, name);
                    return !!expected && isLiteralType(expected) && !isTypeAssignableTo(getTypeOfNode(property), expected);
                });
            }
            function getAllPossiblePropertiesOfTypes(types) {
                var unionType = getUnionType(types);
                if (!(unionType.flags & 1048576 /* Union */)) {
                    return getAugmentedPropertiesOfType(unionType);
                }
                var props = ts.createSymbolTable();
                for (var _i = 0, types_3 = types; _i < types_3.length; _i++) {
                    var memberType = types_3[_i];
                    for (var _a = 0, _b = getAugmentedPropertiesOfType(memberType); _a < _b.length; _a++) {
                        var escapedName = _b[_a].escapedName;
                        if (!props.has(escapedName)) {
                            var prop = createUnionOrIntersectionProperty(unionType, escapedName);
                            // May be undefined if the property is private
                            if (prop)
                                props.set(escapedName, prop);
                        }
                    }
                }
                return ts.arrayFrom(props.values());
            }
            function getConstraintOfType(type) {
                return type.flags & 262144 /* TypeParameter */ ? getConstraintOfTypeParameter(type) :
                    type.flags & 8388608 /* IndexedAccess */ ? getConstraintOfIndexedAccess(type) :
                        type.flags & 16777216 /* Conditional */ ? getConstraintOfConditionalType(type) :
                            getBaseConstraintOfType(type);
            }
            function getConstraintOfTypeParameter(typeParameter) {
                return hasNonCircularBaseConstraint(typeParameter) ? getConstraintFromTypeParameter(typeParameter) : undefined;
            }
            function getConstraintOfIndexedAccess(type) {
                return hasNonCircularBaseConstraint(type) ? getConstraintFromIndexedAccess(type) : undefined;
            }
            function getSimplifiedTypeOrConstraint(type) {
                var simplified = getSimplifiedType(type, /*writing*/ false);
                return simplified !== type ? simplified : getConstraintOfType(type);
            }
            function getConstraintFromIndexedAccess(type) {
                var indexConstraint = getSimplifiedTypeOrConstraint(type.indexType);
                if (indexConstraint && indexConstraint !== type.indexType) {
                    var indexedAccess = getIndexedAccessTypeOrUndefined(type.objectType, indexConstraint);
                    if (indexedAccess) {
                        return indexedAccess;
                    }
                }
                var objectConstraint = getSimplifiedTypeOrConstraint(type.objectType);
                if (objectConstraint && objectConstraint !== type.objectType) {
                    return getIndexedAccessTypeOrUndefined(objectConstraint, type.indexType);
                }
                return undefined;
            }
            function getDefaultConstraintOfConditionalType(type) {
                if (!type.resolvedDefaultConstraint) {
                    // An `any` branch of a conditional type would normally be viral - specifically, without special handling here,
                    // a conditional type with a single branch of type `any` would be assignable to anything, since it's constraint would simplify to
                    // just `any`. This result is _usually_ unwanted - so instead here we elide an `any` branch from the constraint type,
                    // in effect treating `any` like `never` rather than `unknown` in this location.
                    var trueConstraint = getInferredTrueTypeFromConditionalType(type);
                    var falseConstraint = getFalseTypeFromConditionalType(type);
                    type.resolvedDefaultConstraint = isTypeAny(trueConstraint) ? falseConstraint : isTypeAny(falseConstraint) ? trueConstraint : getUnionType([trueConstraint, falseConstraint]);
                }
                return type.resolvedDefaultConstraint;
            }
            function getConstraintOfDistributiveConditionalType(type) {
                // Check if we have a conditional type of the form 'T extends U ? X : Y', where T is a constrained
                // type parameter. If so, create an instantiation of the conditional type where T is replaced
                // with its constraint. We do this because if the constraint is a union type it will be distributed
                // over the conditional type and possibly reduced. For example, 'T extends undefined ? never : T'
                // removes 'undefined' from T.
                // We skip returning a distributive constraint for a restrictive instantiation of a conditional type
                // as the constraint for all type params (check type included) have been replace with `unknown`, which
                // is going to produce even more false positive/negative results than the distribute constraint already does.
                // Please note: the distributive constraint is a kludge for emulating what a negated type could to do filter
                // a union - once negated types exist and are applied to the conditional false branch, this "constraint"
                // likely doesn't need to exist.
                if (type.root.isDistributive && type.restrictiveInstantiation !== type) {
                    var simplified = getSimplifiedType(type.checkType, /*writing*/ false);
                    var constraint = simplified === type.checkType ? getConstraintOfType(simplified) : simplified;
                    if (constraint && constraint !== type.checkType) {
                        var mapper = makeUnaryTypeMapper(type.root.checkType, constraint);
                        var instantiated = getConditionalTypeInstantiation(type, combineTypeMappers(mapper, type.mapper));
                        if (!(instantiated.flags & 131072 /* Never */)) {
                            return instantiated;
                        }
                    }
                }
                return undefined;
            }
            function getConstraintFromConditionalType(type) {
                return getConstraintOfDistributiveConditionalType(type) || getDefaultConstraintOfConditionalType(type);
            }
            function getConstraintOfConditionalType(type) {
                return hasNonCircularBaseConstraint(type) ? getConstraintFromConditionalType(type) : undefined;
            }
            function getEffectiveConstraintOfIntersection(types, targetIsUnion) {
                var constraints;
                var hasDisjointDomainType = false;
                for (var _i = 0, types_4 = types; _i < types_4.length; _i++) {
                    var t = types_4[_i];
                    if (t.flags & 63176704 /* Instantiable */) {
                        // We keep following constraints as long as we have an instantiable type that is known
                        // not to be circular or infinite (hence we stop on index access types).
                        var constraint = getConstraintOfType(t);
                        while (constraint && constraint.flags & (262144 /* TypeParameter */ | 4194304 /* Index */ | 16777216 /* Conditional */)) {
                            constraint = getConstraintOfType(constraint);
                        }
                        if (constraint) {
                            constraints = ts.append(constraints, constraint);
                            if (targetIsUnion) {
                                constraints = ts.append(constraints, t);
                            }
                        }
                    }
                    else if (t.flags & 67238908 /* DisjointDomains */) {
                        hasDisjointDomainType = true;
                    }
                }
                // If the target is a union type or if we are intersecting with types belonging to one of the
                // disjoint domains, we may end up producing a constraint that hasn't been examined before.
                if (constraints && (targetIsUnion || hasDisjointDomainType)) {
                    if (hasDisjointDomainType) {
                        // We add any types belong to one of the disjoint domains because they might cause the final
                        // intersection operation to reduce the union constraints.
                        for (var _a = 0, types_5 = types; _a < types_5.length; _a++) {
                            var t = types_5[_a];
                            if (t.flags & 67238908 /* DisjointDomains */) {
                                constraints = ts.append(constraints, t);
                            }
                        }
                    }
                    return getIntersectionType(constraints);
                }
                return undefined;
            }
            function getBaseConstraintOfType(type) {
                if (type.flags & (58982400 /* InstantiableNonPrimitive */ | 3145728 /* UnionOrIntersection */)) {
                    var constraint = getResolvedBaseConstraint(type);
                    return constraint !== noConstraintType && constraint !== circularConstraintType ? constraint : undefined;
                }
                return type.flags & 4194304 /* Index */ ? keyofConstraintType : undefined;
            }
            /**
             * This is similar to `getBaseConstraintOfType` except it returns the input type if there's no base constraint, instead of `undefined`
             * It also doesn't map indexes to `string`, as where this is used this would be unneeded (and likely undesirable)
             */
            function getBaseConstraintOrType(type) {
                return getBaseConstraintOfType(type) || type;
            }
            function hasNonCircularBaseConstraint(type) {
                return getResolvedBaseConstraint(type) !== circularConstraintType;
            }
            /**
             * Return the resolved base constraint of a type variable. The noConstraintType singleton is returned if the
             * type variable has no constraint, and the circularConstraintType singleton is returned if the constraint
             * circularly references the type variable.
             */
            function getResolvedBaseConstraint(type) {
                var nonTerminating = false;
                return type.resolvedBaseConstraint ||
                    (type.resolvedBaseConstraint = getTypeWithThisArgument(getImmediateBaseConstraint(type), type));
                function getImmediateBaseConstraint(t) {
                    if (!t.immediateBaseConstraint) {
                        if (!pushTypeResolution(t, 4 /* ImmediateBaseConstraint */)) {
                            return circularConstraintType;
                        }
                        if (constraintDepth >= 50) {
                            // We have reached 50 recursive invocations of getImmediateBaseConstraint and there is a
                            // very high likelihood we're dealing with an infinite generic type that perpetually generates
                            // new type identities as we descend into it. We stop the recursion here and mark this type
                            // and the outer types as having circular constraints.
                            error(currentNode, ts.Diagnostics.Type_instantiation_is_excessively_deep_and_possibly_infinite);
                            nonTerminating = true;
                            return t.immediateBaseConstraint = noConstraintType;
                        }
                        constraintDepth++;
                        var result = computeBaseConstraint(getSimplifiedType(t, /*writing*/ false));
                        constraintDepth--;
                        if (!popTypeResolution()) {
                            if (t.flags & 262144 /* TypeParameter */) {
                                var errorNode = getConstraintDeclaration(t);
                                if (errorNode) {
                                    var diagnostic = error(errorNode, ts.Diagnostics.Type_parameter_0_has_a_circular_constraint, typeToString(t));
                                    if (currentNode && !ts.isNodeDescendantOf(errorNode, currentNode) && !ts.isNodeDescendantOf(currentNode, errorNode)) {
                                        ts.addRelatedInfo(diagnostic, ts.createDiagnosticForNode(currentNode, ts.Diagnostics.Circularity_originates_in_type_at_this_location));
                                    }
                                }
                            }
                            result = circularConstraintType;
                        }
                        if (nonTerminating) {
                            result = circularConstraintType;
                        }
                        t.immediateBaseConstraint = result || noConstraintType;
                    }
                    return t.immediateBaseConstraint;
                }
                function getBaseConstraint(t) {
                    var c = getImmediateBaseConstraint(t);
                    return c !== noConstraintType && c !== circularConstraintType ? c : undefined;
                }
                function computeBaseConstraint(t) {
                    if (t.flags & 262144 /* TypeParameter */) {
                        var constraint = getConstraintFromTypeParameter(t);
                        return t.isThisType || !constraint ?
                            constraint :
                            getBaseConstraint(constraint);
                    }
                    if (t.flags & 3145728 /* UnionOrIntersection */) {
                        var types = t.types;
                        var baseTypes = [];
                        for (var _i = 0, types_6 = types; _i < types_6.length; _i++) {
                            var type_2 = types_6[_i];
                            var baseType = getBaseConstraint(type_2);
                            if (baseType) {
                                baseTypes.push(baseType);
                            }
                        }
                        return t.flags & 1048576 /* Union */ && baseTypes.length === types.length ? getUnionType(baseTypes) :
                            t.flags & 2097152 /* Intersection */ && baseTypes.length ? getIntersectionType(baseTypes) :
                                undefined;
                    }
                    if (t.flags & 4194304 /* Index */) {
                        return keyofConstraintType;
                    }
                    if (t.flags & 8388608 /* IndexedAccess */) {
                        var baseObjectType = getBaseConstraint(t.objectType);
                        var baseIndexType = getBaseConstraint(t.indexType);
                        var baseIndexedAccess = baseObjectType && baseIndexType && getIndexedAccessTypeOrUndefined(baseObjectType, baseIndexType);
                        return baseIndexedAccess && getBaseConstraint(baseIndexedAccess);
                    }
                    if (t.flags & 16777216 /* Conditional */) {
                        var constraint = getConstraintFromConditionalType(t);
                        constraintDepth++; // Penalize repeating conditional types (this captures the recursion within getConstraintFromConditionalType and carries it forward)
                        var result = constraint && getBaseConstraint(constraint);
                        constraintDepth--;
                        return result;
                    }
                    if (t.flags & 33554432 /* Substitution */) {
                        return getBaseConstraint(t.substitute);
                    }
                    return t;
                }
            }
            function getApparentTypeOfIntersectionType(type) {
                return type.resolvedApparentType || (type.resolvedApparentType = getTypeWithThisArgument(type, type, /*apparentType*/ true));
            }
            function getResolvedTypeParameterDefault(typeParameter) {
                if (!typeParameter.default) {
                    if (typeParameter.target) {
                        var targetDefault = getResolvedTypeParameterDefault(typeParameter.target);
                        typeParameter.default = targetDefault ? instantiateType(targetDefault, typeParameter.mapper) : noConstraintType;
                    }
                    else {
                        // To block recursion, set the initial value to the resolvingDefaultType.
                        typeParameter.default = resolvingDefaultType;
                        var defaultDeclaration = typeParameter.symbol && ts.forEach(typeParameter.symbol.declarations, function (decl) { return ts.isTypeParameterDeclaration(decl) && decl.default; });
                        var defaultType = defaultDeclaration ? getTypeFromTypeNode(defaultDeclaration) : noConstraintType;
                        if (typeParameter.default === resolvingDefaultType) {
                            // If we have not been called recursively, set the correct default type.
                            typeParameter.default = defaultType;
                        }
                    }
                }
                else if (typeParameter.default === resolvingDefaultType) {
                    // If we are called recursively for this type parameter, mark the default as circular.
                    typeParameter.default = circularConstraintType;
                }
                return typeParameter.default;
            }
            /**
             * Gets the default type for a type parameter.
             *
             * If the type parameter is the result of an instantiation, this gets the instantiated
             * default type of its target. If the type parameter has no default type or the default is
             * circular, `undefined` is returned.
             */
            function getDefaultFromTypeParameter(typeParameter) {
                var defaultType = getResolvedTypeParameterDefault(typeParameter);
                return defaultType !== noConstraintType && defaultType !== circularConstraintType ? defaultType : undefined;
            }
            function hasNonCircularTypeParameterDefault(typeParameter) {
                return getResolvedTypeParameterDefault(typeParameter) !== circularConstraintType;
            }
            /**
             * Indicates whether the declaration of a typeParameter has a default type.
             */
            function hasTypeParameterDefault(typeParameter) {
                return !!(typeParameter.symbol && ts.forEach(typeParameter.symbol.declarations, function (decl) { return ts.isTypeParameterDeclaration(decl) && decl.default; }));
            }
            function getApparentTypeOfMappedType(type) {
                return type.resolvedApparentType || (type.resolvedApparentType = getResolvedApparentTypeOfMappedType(type));
            }
            function getResolvedApparentTypeOfMappedType(type) {
                var typeVariable = getHomomorphicTypeVariable(type);
                if (typeVariable) {
                    var constraint = getConstraintOfTypeParameter(typeVariable);
                    if (constraint && (isArrayType(constraint) || isTupleType(constraint))) {
                        var mapper = makeUnaryTypeMapper(typeVariable, constraint);
                        return instantiateType(type, combineTypeMappers(mapper, type.mapper));
                    }
                }
                return type;
            }
            /**
             * For a type parameter, return the base constraint of the type parameter. For the string, number,
             * boolean, and symbol primitive types, return the corresponding object types. Otherwise return the
             * type itself. Note that the apparent type of a union type is the union type itself.
             */
            function getApparentType(type) {
                var t = type.flags & 63176704 /* Instantiable */ ? getBaseConstraintOfType(type) || unknownType : type;
                return ts.getObjectFlags(t) & 32 /* Mapped */ ? getApparentTypeOfMappedType(t) :
                    t.flags & 2097152 /* Intersection */ ? getApparentTypeOfIntersectionType(t) :
                        t.flags & 132 /* StringLike */ ? globalStringType :
                            t.flags & 296 /* NumberLike */ ? globalNumberType :
                                t.flags & 2112 /* BigIntLike */ ? getGlobalBigIntType(/*reportErrors*/ languageVersion >= 99 /* ESNext */) :
                                    t.flags & 528 /* BooleanLike */ ? globalBooleanType :
                                        t.flags & 12288 /* ESSymbolLike */ ? getGlobalESSymbolType(/*reportErrors*/ languageVersion >= 2 /* ES2015 */) :
                                            t.flags & 67108864 /* NonPrimitive */ ? emptyObjectType :
                                                t.flags & 4194304 /* Index */ ? keyofConstraintType :
                                                    t.flags & 2 /* Unknown */ && !strictNullChecks ? emptyObjectType :
                                                        t;
            }
            function createUnionOrIntersectionProperty(containingType, name) {
                var propSet = ts.createMap();
                var indexTypes;
                var isUnion = containingType.flags & 1048576 /* Union */;
                var excludeModifiers = isUnion ? 24 /* NonPublicAccessibilityModifier */ : 0;
                // Flags we want to propagate to the result if they exist in all source symbols
                var optionalFlag = isUnion ? 0 /* None */ : 16777216 /* Optional */;
                var syntheticFlag = 4 /* SyntheticMethod */;
                var checkFlags = 0;
                for (var _i = 0, _a = containingType.types; _i < _a.length; _i++) {
                    var current = _a[_i];
                    var type = getApparentType(current);
                    if (type !== errorType) {
                        var prop = getPropertyOfType(type, name);
                        var modifiers = prop ? ts.getDeclarationModifierFlagsFromSymbol(prop) : 0;
                        if (prop && !(modifiers & excludeModifiers)) {
                            if (isUnion) {
                                optionalFlag |= (prop.flags & 16777216 /* Optional */);
                            }
                            else {
                                optionalFlag &= prop.flags;
                            }
                            var id = "" + getSymbolId(prop);
                            if (!propSet.has(id)) {
                                propSet.set(id, prop);
                            }
                            checkFlags |= (isReadonlySymbol(prop) ? 8 /* Readonly */ : 0) |
                                (!(modifiers & 24 /* NonPublicAccessibilityModifier */) ? 256 /* ContainsPublic */ : 0) |
                                (modifiers & 16 /* Protected */ ? 512 /* ContainsProtected */ : 0) |
                                (modifiers & 8 /* Private */ ? 1024 /* ContainsPrivate */ : 0) |
                                (modifiers & 32 /* Static */ ? 2048 /* ContainsStatic */ : 0);
                            if (!isPrototypeProperty(prop)) {
                                syntheticFlag = 2 /* SyntheticProperty */;
                            }
                        }
                        else if (isUnion) {
                            var indexInfo = !isLateBoundName(name) && (isNumericLiteralName(name) && getIndexInfoOfType(type, 1 /* Number */) || getIndexInfoOfType(type, 0 /* String */));
                            if (indexInfo) {
                                checkFlags |= 32 /* WritePartial */ | (indexInfo.isReadonly ? 8 /* Readonly */ : 0);
                                indexTypes = ts.append(indexTypes, isTupleType(type) ? getRestTypeOfTupleType(type) || undefinedType : indexInfo.type);
                            }
                            else if (isObjectLiteralType(type)) {
                                checkFlags |= 32 /* WritePartial */;
                                indexTypes = ts.append(indexTypes, undefinedType);
                            }
                            else {
                                checkFlags |= 16 /* ReadPartial */;
                            }
                        }
                    }
                }
                if (!propSet.size) {
                    return undefined;
                }
                var props = ts.arrayFrom(propSet.values());
                if (props.length === 1 && !(checkFlags & 16 /* ReadPartial */) && !indexTypes) {
                    return props[0];
                }
                var declarations;
                var firstType;
                var nameType;
                var propTypes = [];
                var firstValueDeclaration;
                var hasNonUniformValueDeclaration = false;
                for (var _b = 0, props_1 = props; _b < props_1.length; _b++) {
                    var prop = props_1[_b];
                    if (!firstValueDeclaration) {
                        firstValueDeclaration = prop.valueDeclaration;
                    }
                    else if (prop.valueDeclaration !== firstValueDeclaration) {
                        hasNonUniformValueDeclaration = true;
                    }
                    declarations = ts.addRange(declarations, prop.declarations);
                    var type = getTypeOfSymbol(prop);
                    if (!firstType) {
                        firstType = type;
                        nameType = getSymbolLinks(prop).nameType;
                    }
                    else if (type !== firstType) {
                        checkFlags |= 64 /* HasNonUniformType */;
                    }
                    if (isLiteralType(type)) {
                        checkFlags |= 128 /* HasLiteralType */;
                    }
                    propTypes.push(type);
                }
                ts.addRange(propTypes, indexTypes);
                var result = createSymbol(4 /* Property */ | optionalFlag, name, syntheticFlag | checkFlags);
                result.containingType = containingType;
                if (!hasNonUniformValueDeclaration && firstValueDeclaration) {
                    result.valueDeclaration = firstValueDeclaration;
                    // Inherit information about parent type.
                    if (firstValueDeclaration.symbol.parent) {
                        result.parent = firstValueDeclaration.symbol.parent;
                    }
                }
                result.declarations = declarations;
                result.nameType = nameType;
                if (propTypes.length > 2) {
                    // When `propTypes` has the potential to explode in size when normalized, defer normalization until absolutely needed
                    result.checkFlags |= 65536 /* DeferredType */;
                    result.deferralParent = containingType;
                    result.deferralConstituents = propTypes;
                }
                else {
                    result.type = isUnion ? getUnionType(propTypes) : getIntersectionType(propTypes);
                }
                return result;
            }
            // Return the symbol for a given property in a union or intersection type, or undefined if the property
            // does not exist in any constituent type. Note that the returned property may only be present in some
            // constituents, in which case the isPartial flag is set when the containing type is union type. We need
            // these partial properties when identifying discriminant properties, but otherwise they are filtered out
            // and do not appear to be present in the union type.
            function getUnionOrIntersectionProperty(type, name) {
                var properties = type.propertyCache || (type.propertyCache = ts.createSymbolTable());
                var property = properties.get(name);
                if (!property) {
                    property = createUnionOrIntersectionProperty(type, name);
                    if (property) {
                        properties.set(name, property);
                    }
                }
                return property;
            }
            function getPropertyOfUnionOrIntersectionType(type, name) {
                var property = getUnionOrIntersectionProperty(type, name);
                // We need to filter out partial properties in union types
                return property && !(ts.getCheckFlags(property) & 16 /* ReadPartial */) ? property : undefined;
            }
            /**
             * Return the symbol for the property with the given name in the given type. Creates synthetic union properties when
             * necessary, maps primitive types and type parameters are to their apparent types, and augments with properties from
             * Object and Function as appropriate.
             *
             * @param type a type to look up property from
             * @param name a name of property to look up in a given type
             */
            function getPropertyOfType(type, name) {
                type = getApparentType(type);
                if (type.flags & 524288 /* Object */) {
                    var resolved = resolveStructuredTypeMembers(type);
                    var symbol = resolved.members.get(name);
                    if (symbol && symbolIsValue(symbol)) {
                        return symbol;
                    }
                    var functionType = resolved === anyFunctionType ? globalFunctionType :
                        resolved.callSignatures.length ? globalCallableFunctionType :
                            resolved.constructSignatures.length ? globalNewableFunctionType :
                                undefined;
                    if (functionType) {
                        var symbol_1 = getPropertyOfObjectType(functionType, name);
                        if (symbol_1) {
                            return symbol_1;
                        }
                    }
                    return getPropertyOfObjectType(globalObjectType, name);
                }
                if (type.flags & 3145728 /* UnionOrIntersection */) {
                    return getPropertyOfUnionOrIntersectionType(type, name);
                }
                return undefined;
            }
            function getSignaturesOfStructuredType(type, kind) {
                if (type.flags & 3670016 /* StructuredType */) {
                    var resolved = resolveStructuredTypeMembers(type);
                    return kind === 0 /* Call */ ? resolved.callSignatures : resolved.constructSignatures;
                }
                return ts.emptyArray;
            }
            /**
             * Return the signatures of the given kind in the given type. Creates synthetic union signatures when necessary and
             * maps primitive types and type parameters are to their apparent types.
             */
            function getSignaturesOfType(type, kind) {
                return getSignaturesOfStructuredType(getApparentType(type), kind);
            }
            function getIndexInfoOfStructuredType(type, kind) {
                if (type.flags & 3670016 /* StructuredType */) {
                    var resolved = resolveStructuredTypeMembers(type);
                    return kind === 0 /* String */ ? resolved.stringIndexInfo : resolved.numberIndexInfo;
                }
            }
            function getIndexTypeOfStructuredType(type, kind) {
                var info = getIndexInfoOfStructuredType(type, kind);
                return info && info.type;
            }
            // Return the indexing info of the given kind in the given type. Creates synthetic union index types when necessary and
            // maps primitive types and type parameters are to their apparent types.
            function getIndexInfoOfType(type, kind) {
                return getIndexInfoOfStructuredType(getApparentType(type), kind);
            }
            // Return the index type of the given kind in the given type. Creates synthetic union index types when necessary and
            // maps primitive types and type parameters are to their apparent types.
            function getIndexTypeOfType(type, kind) {
                return getIndexTypeOfStructuredType(getApparentType(type), kind);
            }
            function getImplicitIndexTypeOfType(type, kind) {
                if (isObjectTypeWithInferableIndex(type)) {
                    var propTypes = [];
                    for (var _i = 0, _a = getPropertiesOfType(type); _i < _a.length; _i++) {
                        var prop = _a[_i];
                        if (kind === 0 /* String */ || isNumericLiteralName(prop.escapedName)) {
                            propTypes.push(getTypeOfSymbol(prop));
                        }
                    }
                    if (kind === 0 /* String */) {
                        ts.append(propTypes, getIndexTypeOfType(type, 1 /* Number */));
                    }
                    if (propTypes.length) {
                        return getUnionType(propTypes, 2 /* Subtype */);
                    }
                }
                return undefined;
            }
            // Return list of type parameters with duplicates removed (duplicate identifier errors are generated in the actual
            // type checking functions).
            function getTypeParametersFromDeclaration(declaration) {
                var result;
                for (var _i = 0, _a = ts.getEffectiveTypeParameterDeclarations(declaration); _i < _a.length; _i++) {
                    var node = _a[_i];
                    result = ts.appendIfUnique(result, getDeclaredTypeOfTypeParameter(node.symbol));
                }
                return result;
            }
            function symbolsToArray(symbols) {
                var result = [];
                symbols.forEach(function (symbol, id) {
                    if (!isReservedMemberName(id)) {
                        result.push(symbol);
                    }
                });
                return result;
            }
            function isJSDocOptionalParameter(node) {
                return ts.isInJSFile(node) && (
                // node.type should only be a JSDocOptionalType when node is a parameter of a JSDocFunctionType
                node.type && node.type.kind === 299 /* JSDocOptionalType */
                    || ts.getJSDocParameterTags(node).some(function (_a) {
                        var isBracketed = _a.isBracketed, typeExpression = _a.typeExpression;
                        return isBracketed || !!typeExpression && typeExpression.type.kind === 299 /* JSDocOptionalType */;
                    }));
            }
            function tryFindAmbientModule(moduleName, withAugmentations) {
                if (ts.isExternalModuleNameRelative(moduleName)) {
                    return undefined;
                }
                var symbol = getSymbol(globals, '"' + moduleName + '"', 512 /* ValueModule */);
                // merged symbol is module declaration symbol combined with all augmentations
                return symbol && withAugmentations ? getMergedSymbol(symbol) : symbol;
            }
            function isOptionalParameter(node) {
                if (ts.hasQuestionToken(node) || isOptionalJSDocParameterTag(node) || isJSDocOptionalParameter(node)) {
                    return true;
                }
                if (node.initializer) {
                    var signature = getSignatureFromDeclaration(node.parent);
                    var parameterIndex = node.parent.parameters.indexOf(node);
                    ts.Debug.assert(parameterIndex >= 0);
                    return parameterIndex >= getMinArgumentCount(signature);
                }
                var iife = ts.getImmediatelyInvokedFunctionExpression(node.parent);
                if (iife) {
                    return !node.type &&
                        !node.dotDotDotToken &&
                        node.parent.parameters.indexOf(node) >= iife.arguments.length;
                }
                return false;
            }
            function isOptionalJSDocParameterTag(node) {
                if (!ts.isJSDocParameterTag(node)) {
                    return false;
                }
                var isBracketed = node.isBracketed, typeExpression = node.typeExpression;
                return isBracketed || !!typeExpression && typeExpression.type.kind === 299 /* JSDocOptionalType */;
            }
            function createTypePredicate(kind, parameterName, parameterIndex, type) {
                return { kind: kind, parameterName: parameterName, parameterIndex: parameterIndex, type: type };
            }
            /**
             * Gets the minimum number of type arguments needed to satisfy all non-optional type
             * parameters.
             */
            function getMinTypeArgumentCount(typeParameters) {
                var minTypeArgumentCount = 0;
                if (typeParameters) {
                    for (var i = 0; i < typeParameters.length; i++) {
                        if (!hasTypeParameterDefault(typeParameters[i])) {
                            minTypeArgumentCount = i + 1;
                        }
                    }
                }
                return minTypeArgumentCount;
            }
            function fillMissingTypeArguments(typeArguments, typeParameters, minTypeArgumentCount, isJavaScriptImplicitAny) {
                var numTypeParameters = ts.length(typeParameters);
                if (!numTypeParameters) {
                    return [];
                }
                var numTypeArguments = ts.length(typeArguments);
                if (isJavaScriptImplicitAny || (numTypeArguments >= minTypeArgumentCount && numTypeArguments <= numTypeParameters)) {
                    var result = typeArguments ? typeArguments.slice() : [];
                    // Map invalid forward references in default types to the error type
                    for (var i = numTypeArguments; i < numTypeParameters; i++) {
                        result[i] = errorType;
                    }
                    var baseDefaultType = getDefaultTypeArgumentType(isJavaScriptImplicitAny);
                    for (var i = numTypeArguments; i < numTypeParameters; i++) {
                        var defaultType = getDefaultFromTypeParameter(typeParameters[i]);
                        if (isJavaScriptImplicitAny && defaultType && (isTypeIdenticalTo(defaultType, unknownType) || isTypeIdenticalTo(defaultType, emptyObjectType))) {
                            defaultType = anyType;
                        }
                        result[i] = defaultType ? instantiateType(defaultType, createTypeMapper(typeParameters, result)) : baseDefaultType;
                    }
                    result.length = typeParameters.length;
                    return result;
                }
                return typeArguments && typeArguments.slice();
            }
            function getSignatureFromDeclaration(declaration) {
                var links = getNodeLinks(declaration);
                if (!links.resolvedSignature) {
                    var parameters = [];
                    var flags = 0 /* None */;
                    var minArgumentCount = 0;
                    var thisParameter = void 0;
                    var hasThisParameter = false;
                    var iife = ts.getImmediatelyInvokedFunctionExpression(declaration);
                    var isJSConstructSignature = ts.isJSDocConstructSignature(declaration);
                    var isUntypedSignatureInJSFile = !iife &&
                        ts.isInJSFile(declaration) &&
                        ts.isValueSignatureDeclaration(declaration) &&
                        !ts.hasJSDocParameterTags(declaration) &&
                        !ts.getJSDocType(declaration);
                    // If this is a JSDoc construct signature, then skip the first parameter in the
                    // parameter list.  The first parameter represents the return type of the construct
                    // signature.
                    for (var i = isJSConstructSignature ? 1 : 0; i < declaration.parameters.length; i++) {
                        var param = declaration.parameters[i];
                        var paramSymbol = param.symbol;
                        var type = ts.isJSDocParameterTag(param) ? (param.typeExpression && param.typeExpression.type) : param.type;
                        // Include parameter symbol instead of property symbol in the signature
                        if (paramSymbol && !!(paramSymbol.flags & 4 /* Property */) && !ts.isBindingPattern(param.name)) {
                            var resolvedSymbol = resolveName(param, paramSymbol.escapedName, 111551 /* Value */, undefined, undefined, /*isUse*/ false);
                            paramSymbol = resolvedSymbol;
                        }
                        if (i === 0 && paramSymbol.escapedName === "this" /* This */) {
                            hasThisParameter = true;
                            thisParameter = param.symbol;
                        }
                        else {
                            parameters.push(paramSymbol);
                        }
                        if (type && type.kind === 187 /* LiteralType */) {
                            flags |= 2 /* HasLiteralTypes */;
                        }
                        // Record a new minimum argument count if this is not an optional parameter
                        var isOptionalParameter_1 = isOptionalJSDocParameterTag(param) ||
                            param.initializer || param.questionToken || param.dotDotDotToken ||
                            iife && parameters.length > iife.arguments.length && !type ||
                            isUntypedSignatureInJSFile ||
                            isJSDocOptionalParameter(param);
                        if (!isOptionalParameter_1) {
                            minArgumentCount = parameters.length;
                        }
                    }
                    // If only one accessor includes a this-type annotation, the other behaves as if it had the same type annotation
                    if ((declaration.kind === 163 /* GetAccessor */ || declaration.kind === 164 /* SetAccessor */) &&
                        !hasNonBindableDynamicName(declaration) &&
                        (!hasThisParameter || !thisParameter)) {
                        var otherKind = declaration.kind === 163 /* GetAccessor */ ? 164 /* SetAccessor */ : 163 /* GetAccessor */;
                        var other = ts.getDeclarationOfKind(getSymbolOfNode(declaration), otherKind);
                        if (other) {
                            thisParameter = getAnnotatedAccessorThisParameter(other);
                        }
                    }
                    var classType = declaration.kind === 162 /* Constructor */ ?
                        getDeclaredTypeOfClassOrInterface(getMergedSymbol(declaration.parent.symbol))
                        : undefined;
                    var typeParameters = classType ? classType.localTypeParameters : getTypeParametersFromDeclaration(declaration);
                    if (ts.hasRestParameter(declaration) || ts.isInJSFile(declaration) && maybeAddJsSyntheticRestParameter(declaration, parameters)) {
                        flags |= 1 /* HasRestParameter */;
                    }
                    links.resolvedSignature = createSignature(declaration, typeParameters, thisParameter, parameters, 
                    /*resolvedReturnType*/ undefined, /*resolvedTypePredicate*/ undefined, minArgumentCount, flags);
                }
                return links.resolvedSignature;
            }
            /**
             * A JS function gets a synthetic rest parameter if it references `arguments` AND:
             * 1. It has no parameters but at least one `@param` with a type that starts with `...`
             * OR
             * 2. It has at least one parameter, and the last parameter has a matching `@param` with a type that starts with `...`
             */
            function maybeAddJsSyntheticRestParameter(declaration, parameters) {
                if (ts.isJSDocSignature(declaration) || !containsArgumentsReference(declaration)) {
                    return false;
                }
                var lastParam = ts.lastOrUndefined(declaration.parameters);
                var lastParamTags = lastParam ? ts.getJSDocParameterTags(lastParam) : ts.getJSDocTags(declaration).filter(ts.isJSDocParameterTag);
                var lastParamVariadicType = ts.firstDefined(lastParamTags, function (p) {
                    return p.typeExpression && ts.isJSDocVariadicType(p.typeExpression.type) ? p.typeExpression.type : undefined;
                });
                var syntheticArgsSymbol = createSymbol(3 /* Variable */, "args", 32768 /* RestParameter */);
                syntheticArgsSymbol.type = lastParamVariadicType ? createArrayType(getTypeFromTypeNode(lastParamVariadicType.type)) : anyArrayType;
                if (lastParamVariadicType) {
                    // Replace the last parameter with a rest parameter.
                    parameters.pop();
                }
                parameters.push(syntheticArgsSymbol);
                return true;
            }
            function getSignatureOfTypeTag(node) {
                var typeTag = ts.isInJSFile(node) ? ts.getJSDocTypeTag(node) : undefined;
                var signature = typeTag && typeTag.typeExpression && getSingleCallSignature(getTypeFromTypeNode(typeTag.typeExpression));
                return signature && getErasedSignature(signature);
            }
            function getReturnTypeOfTypeTag(node) {
                var signature = getSignatureOfTypeTag(node);
                return signature && getReturnTypeOfSignature(signature);
            }
            function containsArgumentsReference(declaration) {
                var links = getNodeLinks(declaration);
                if (links.containsArgumentsReference === undefined) {
                    if (links.flags & 8192 /* CaptureArguments */) {
                        links.containsArgumentsReference = true;
                    }
                    else {
                        links.containsArgumentsReference = traverse(declaration.body);
                    }
                }
                return links.containsArgumentsReference;
                function traverse(node) {
                    if (!node)
                        return false;
                    switch (node.kind) {
                        case 75 /* Identifier */:
                            return node.escapedText === "arguments" && ts.isExpressionNode(node);
                        case 159 /* PropertyDeclaration */:
                        case 161 /* MethodDeclaration */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            return node.name.kind === 154 /* ComputedPropertyName */
                                && traverse(node.name);
                        default:
                            return !ts.nodeStartsNewLexicalEnvironment(node) && !ts.isPartOfTypeNode(node) && !!ts.forEachChild(node, traverse);
                    }
                }
            }
            function getSignaturesOfSymbol(symbol) {
                if (!symbol)
                    return ts.emptyArray;
                var result = [];
                for (var i = 0; i < symbol.declarations.length; i++) {
                    var decl = symbol.declarations[i];
                    if (!ts.isFunctionLike(decl))
                        continue;
                    // Don't include signature if node is the implementation of an overloaded function. A node is considered
                    // an implementation node if it has a body and the previous node is of the same kind and immediately
                    // precedes the implementation node (i.e. has the same parent and ends where the implementation starts).
                    if (i > 0 && decl.body) {
                        var previous = symbol.declarations[i - 1];
                        if (decl.parent === previous.parent && decl.kind === previous.kind && decl.pos === previous.end) {
                            continue;
                        }
                    }
                    result.push(getSignatureFromDeclaration(decl));
                }
                return result;
            }
            function resolveExternalModuleTypeByLiteral(name) {
                var moduleSym = resolveExternalModuleName(name, name);
                if (moduleSym) {
                    var resolvedModuleSymbol = resolveExternalModuleSymbol(moduleSym);
                    if (resolvedModuleSymbol) {
                        return getTypeOfSymbol(resolvedModuleSymbol);
                    }
                }
                return anyType;
            }
            function getThisTypeOfSignature(signature) {
                if (signature.thisParameter) {
                    return getTypeOfSymbol(signature.thisParameter);
                }
            }
            function getTypePredicateOfSignature(signature) {
                if (!signature.resolvedTypePredicate) {
                    if (signature.target) {
                        var targetTypePredicate = getTypePredicateOfSignature(signature.target);
                        signature.resolvedTypePredicate = targetTypePredicate ? instantiateTypePredicate(targetTypePredicate, signature.mapper) : noTypePredicate;
                    }
                    else if (signature.unionSignatures) {
                        signature.resolvedTypePredicate = getUnionTypePredicate(signature.unionSignatures) || noTypePredicate;
                    }
                    else {
                        var type = signature.declaration && ts.getEffectiveReturnTypeNode(signature.declaration);
                        var jsdocPredicate = void 0;
                        if (!type && ts.isInJSFile(signature.declaration)) {
                            var jsdocSignature = getSignatureOfTypeTag(signature.declaration);
                            if (jsdocSignature && signature !== jsdocSignature) {
                                jsdocPredicate = getTypePredicateOfSignature(jsdocSignature);
                            }
                        }
                        signature.resolvedTypePredicate = type && ts.isTypePredicateNode(type) ?
                            createTypePredicateFromTypePredicateNode(type, signature) :
                            jsdocPredicate || noTypePredicate;
                    }
                    ts.Debug.assert(!!signature.resolvedTypePredicate);
                }
                return signature.resolvedTypePredicate === noTypePredicate ? undefined : signature.resolvedTypePredicate;
            }
            function createTypePredicateFromTypePredicateNode(node, signature) {
                var parameterName = node.parameterName;
                var type = node.type && getTypeFromTypeNode(node.type);
                return parameterName.kind === 183 /* ThisType */ ?
                    createTypePredicate(node.assertsModifier ? 2 /* AssertsThis */ : 0 /* This */, /*parameterName*/ undefined, /*parameterIndex*/ undefined, type) :
                    createTypePredicate(node.assertsModifier ? 3 /* AssertsIdentifier */ : 1 /* Identifier */, parameterName.escapedText, ts.findIndex(signature.parameters, function (p) { return p.escapedName === parameterName.escapedText; }), type);
            }
            function getReturnTypeOfSignature(signature) {
                if (!signature.resolvedReturnType) {
                    if (!pushTypeResolution(signature, 3 /* ResolvedReturnType */)) {
                        return errorType;
                    }
                    var type = signature.target ? instantiateType(getReturnTypeOfSignature(signature.target), signature.mapper) :
                        signature.unionSignatures ? getUnionType(ts.map(signature.unionSignatures, getReturnTypeOfSignature), 2 /* Subtype */) :
                            getReturnTypeFromAnnotation(signature.declaration) ||
                                (ts.nodeIsMissing(signature.declaration.body) ? anyType : getReturnTypeFromBody(signature.declaration));
                    if (signature.flags & 4 /* IsInnerCallChain */) {
                        type = addOptionalTypeMarker(type);
                    }
                    else if (signature.flags & 8 /* IsOuterCallChain */) {
                        type = getOptionalType(type);
                    }
                    if (!popTypeResolution()) {
                        if (signature.declaration) {
                            var typeNode = ts.getEffectiveReturnTypeNode(signature.declaration);
                            if (typeNode) {
                                error(typeNode, ts.Diagnostics.Return_type_annotation_circularly_references_itself);
                            }
                            else if (noImplicitAny) {
                                var declaration = signature.declaration;
                                var name = ts.getNameOfDeclaration(declaration);
                                if (name) {
                                    error(name, ts.Diagnostics._0_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions, ts.declarationNameToString(name));
                                }
                                else {
                                    error(declaration, ts.Diagnostics.Function_implicitly_has_return_type_any_because_it_does_not_have_a_return_type_annotation_and_is_referenced_directly_or_indirectly_in_one_of_its_return_expressions);
                                }
                            }
                        }
                        type = anyType;
                    }
                    signature.resolvedReturnType = type;
                }
                return signature.resolvedReturnType;
            }
            function getReturnTypeFromAnnotation(declaration) {
                if (declaration.kind === 162 /* Constructor */) {
                    return getDeclaredTypeOfClassOrInterface(getMergedSymbol(declaration.parent.symbol));
                }
                if (ts.isJSDocConstructSignature(declaration)) {
                    return getTypeFromTypeNode(declaration.parameters[0].type); // TODO: GH#18217
                }
                var typeNode = ts.getEffectiveReturnTypeNode(declaration);
                if (typeNode) {
                    return getTypeFromTypeNode(typeNode);
                }
                if (declaration.kind === 163 /* GetAccessor */ && !hasNonBindableDynamicName(declaration)) {
                    var jsDocType = ts.isInJSFile(declaration) && getTypeForDeclarationFromJSDocComment(declaration);
                    if (jsDocType) {
                        return jsDocType;
                    }
                    var setter = ts.getDeclarationOfKind(getSymbolOfNode(declaration), 164 /* SetAccessor */);
                    var setterType = getAnnotatedAccessorType(setter);
                    if (setterType) {
                        return setterType;
                    }
                }
                return getReturnTypeOfTypeTag(declaration);
            }
            function isResolvingReturnTypeOfSignature(signature) {
                return !signature.resolvedReturnType && findResolutionCycleStartIndex(signature, 3 /* ResolvedReturnType */) >= 0;
            }
            function getRestTypeOfSignature(signature) {
                return tryGetRestTypeOfSignature(signature) || anyType;
            }
            function tryGetRestTypeOfSignature(signature) {
                if (signatureHasRestParameter(signature)) {
                    var sigRestType = getTypeOfSymbol(signature.parameters[signature.parameters.length - 1]);
                    var restType = isTupleType(sigRestType) ? getRestTypeOfTupleType(sigRestType) : sigRestType;
                    return restType && getIndexTypeOfType(restType, 1 /* Number */);
                }
                return undefined;
            }
            function getSignatureInstantiation(signature, typeArguments, isJavascript, inferredTypeParameters) {
                var instantiatedSignature = getSignatureInstantiationWithoutFillingInTypeArguments(signature, fillMissingTypeArguments(typeArguments, signature.typeParameters, getMinTypeArgumentCount(signature.typeParameters), isJavascript));
                if (inferredTypeParameters) {
                    var returnSignature = getSingleCallOrConstructSignature(getReturnTypeOfSignature(instantiatedSignature));
                    if (returnSignature) {
                        var newReturnSignature = cloneSignature(returnSignature);
                        newReturnSignature.typeParameters = inferredTypeParameters;
                        var newInstantiatedSignature = cloneSignature(instantiatedSignature);
                        newInstantiatedSignature.resolvedReturnType = getOrCreateTypeFromSignature(newReturnSignature);
                        return newInstantiatedSignature;
                    }
                }
                return instantiatedSignature;
            }
            function getSignatureInstantiationWithoutFillingInTypeArguments(signature, typeArguments) {
                var instantiations = signature.instantiations || (signature.instantiations = ts.createMap());
                var id = getTypeListId(typeArguments);
                var instantiation = instantiations.get(id);
                if (!instantiation) {
                    instantiations.set(id, instantiation = createSignatureInstantiation(signature, typeArguments));
                }
                return instantiation;
            }
            function createSignatureInstantiation(signature, typeArguments) {
                return instantiateSignature(signature, createSignatureTypeMapper(signature, typeArguments), /*eraseTypeParameters*/ true);
            }
            function createSignatureTypeMapper(signature, typeArguments) {
                return createTypeMapper(signature.typeParameters, typeArguments);
            }
            function getErasedSignature(signature) {
                return signature.typeParameters ?
                    signature.erasedSignatureCache || (signature.erasedSignatureCache = createErasedSignature(signature)) :
                    signature;
            }
            function createErasedSignature(signature) {
                // Create an instantiation of the signature where all type arguments are the any type.
                return instantiateSignature(signature, createTypeEraser(signature.typeParameters), /*eraseTypeParameters*/ true);
            }
            function getCanonicalSignature(signature) {
                return signature.typeParameters ?
                    signature.canonicalSignatureCache || (signature.canonicalSignatureCache = createCanonicalSignature(signature)) :
                    signature;
            }
            function createCanonicalSignature(signature) {
                // Create an instantiation of the signature where each unconstrained type parameter is replaced with
                // its original. When a generic class or interface is instantiated, each generic method in the class or
                // interface is instantiated with a fresh set of cloned type parameters (which we need to handle scenarios
                // where different generations of the same type parameter are in scope). This leads to a lot of new type
                // identities, and potentially a lot of work comparing those identities, so here we create an instantiation
                // that uses the original type identities for all unconstrained type parameters.
                return getSignatureInstantiation(signature, ts.map(signature.typeParameters, function (tp) { return tp.target && !getConstraintOfTypeParameter(tp.target) ? tp.target : tp; }), ts.isInJSFile(signature.declaration));
            }
            function getBaseSignature(signature) {
                var typeParameters = signature.typeParameters;
                if (typeParameters) {
                    var typeEraser_1 = createTypeEraser(typeParameters);
                    var baseConstraints = ts.map(typeParameters, function (tp) { return instantiateType(getBaseConstraintOfType(tp), typeEraser_1) || unknownType; });
                    return instantiateSignature(signature, createTypeMapper(typeParameters, baseConstraints), /*eraseTypeParameters*/ true);
                }
                return signature;
            }
            function getOrCreateTypeFromSignature(signature) {
                // There are two ways to declare a construct signature, one is by declaring a class constructor
                // using the constructor keyword, and the other is declaring a bare construct signature in an
                // object type literal or interface (using the new keyword). Each way of declaring a constructor
                // will result in a different declaration kind.
                if (!signature.isolatedSignatureType) {
                    var kind = signature.declaration ? signature.declaration.kind : 0 /* Unknown */;
                    var isConstructor = kind === 162 /* Constructor */ || kind === 166 /* ConstructSignature */ || kind === 171 /* ConstructorType */;
                    var type = createObjectType(16 /* Anonymous */);
                    type.members = emptySymbols;
                    type.properties = ts.emptyArray;
                    type.callSignatures = !isConstructor ? [signature] : ts.emptyArray;
                    type.constructSignatures = isConstructor ? [signature] : ts.emptyArray;
                    signature.isolatedSignatureType = type;
                }
                return signature.isolatedSignatureType;
            }
            function getIndexSymbol(symbol) {
                return symbol.members.get("__index" /* Index */);
            }
            function getIndexDeclarationOfSymbol(symbol, kind) {
                var syntaxKind = kind === 1 /* Number */ ? 140 /* NumberKeyword */ : 143 /* StringKeyword */;
                var indexSymbol = getIndexSymbol(symbol);
                if (indexSymbol) {
                    for (var _i = 0, _a = indexSymbol.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        var node = ts.cast(decl, ts.isIndexSignatureDeclaration);
                        if (node.parameters.length === 1) {
                            var parameter = node.parameters[0];
                            if (parameter.type && parameter.type.kind === syntaxKind) {
                                return node;
                            }
                        }
                    }
                }
                return undefined;
            }
            function createIndexInfo(type, isReadonly, declaration) {
                return { type: type, isReadonly: isReadonly, declaration: declaration };
            }
            function getIndexInfoOfSymbol(symbol, kind) {
                var declaration = getIndexDeclarationOfSymbol(symbol, kind);
                if (declaration) {
                    return createIndexInfo(declaration.type ? getTypeFromTypeNode(declaration.type) : anyType, ts.hasModifier(declaration, 64 /* Readonly */), declaration);
                }
                return undefined;
            }
            function getConstraintDeclaration(type) {
                return ts.mapDefined(ts.filter(type.symbol && type.symbol.declarations, ts.isTypeParameterDeclaration), ts.getEffectiveConstraintOfTypeParameter)[0];
            }
            function getInferredTypeParameterConstraint(typeParameter) {
                var inferences;
                if (typeParameter.symbol) {
                    for (var _i = 0, _a = typeParameter.symbol.declarations; _i < _a.length; _i++) {
                        var declaration = _a[_i];
                        if (declaration.parent.kind === 181 /* InferType */) {
                            // When an 'infer T' declaration is immediately contained in a type reference node
                            // (such as 'Foo<infer T>'), T's constraint is inferred from the constraint of the
                            // corresponding type parameter in 'Foo'. When multiple 'infer T' declarations are
                            // present, we form an intersection of the inferred constraint types.
                            var grandParent = declaration.parent.parent;
                            if (grandParent.kind === 169 /* TypeReference */) {
                                var typeReference = grandParent;
                                var typeParameters = getTypeParametersForTypeReference(typeReference);
                                if (typeParameters) {
                                    var index = typeReference.typeArguments.indexOf(declaration.parent);
                                    if (index < typeParameters.length) {
                                        var declaredConstraint = getConstraintOfTypeParameter(typeParameters[index]);
                                        if (declaredConstraint) {
                                            // Type parameter constraints can reference other type parameters so
                                            // constraints need to be instantiated. If instantiation produces the
                                            // type parameter itself, we discard that inference. For example, in
                                            //   type Foo<T extends string, U extends T> = [T, U];
                                            //   type Bar<T> = T extends Foo<infer X, infer X> ? Foo<X, X> : T;
                                            // the instantiated constraint for U is X, so we discard that inference.
                                            var mapper = createTypeMapper(typeParameters, getEffectiveTypeArguments(typeReference, typeParameters));
                                            var constraint = instantiateType(declaredConstraint, mapper);
                                            if (constraint !== typeParameter) {
                                                inferences = ts.append(inferences, constraint);
                                            }
                                        }
                                    }
                                }
                            }
                            // When an 'infer T' declaration is immediately contained in a rest parameter
                            // declaration, we infer an 'unknown[]' constraint.
                            else if (grandParent.kind === 156 /* Parameter */ && grandParent.dotDotDotToken) {
                                inferences = ts.append(inferences, createArrayType(unknownType));
                            }
                        }
                    }
                }
                return inferences && getIntersectionType(inferences);
            }
            /** This is a worker function. Use getConstraintOfTypeParameter which guards against circular constraints. */
            function getConstraintFromTypeParameter(typeParameter) {
                if (!typeParameter.constraint) {
                    if (typeParameter.target) {
                        var targetConstraint = getConstraintOfTypeParameter(typeParameter.target);
                        typeParameter.constraint = targetConstraint ? instantiateType(targetConstraint, typeParameter.mapper) : noConstraintType;
                    }
                    else {
                        var constraintDeclaration = getConstraintDeclaration(typeParameter);
                        typeParameter.constraint = constraintDeclaration ? getTypeFromTypeNode(constraintDeclaration) :
                            getInferredTypeParameterConstraint(typeParameter) || noConstraintType;
                    }
                }
                return typeParameter.constraint === noConstraintType ? undefined : typeParameter.constraint;
            }
            function getParentSymbolOfTypeParameter(typeParameter) {
                var tp = ts.getDeclarationOfKind(typeParameter.symbol, 155 /* TypeParameter */);
                var host = ts.isJSDocTemplateTag(tp.parent) ? ts.getHostSignatureFromJSDoc(tp.parent) : tp.parent;
                return host && getSymbolOfNode(host);
            }
            function getTypeListId(types) {
                var result = "";
                if (types) {
                    var length_3 = types.length;
                    var i = 0;
                    while (i < length_3) {
                        var startId = types[i].id;
                        var count = 1;
                        while (i + count < length_3 && types[i + count].id === startId + count) {
                            count++;
                        }
                        if (result.length) {
                            result += ",";
                        }
                        result += startId;
                        if (count > 1) {
                            result += ":" + count;
                        }
                        i += count;
                    }
                }
                return result;
            }
            // This function is used to propagate certain flags when creating new object type references and union types.
            // It is only necessary to do so if a constituent type might be the undefined type, the null type, the type
            // of an object literal or the anyFunctionType. This is because there are operations in the type checker
            // that care about the presence of such types at arbitrary depth in a containing type.
            function getPropagatingFlagsOfTypes(types, excludeKinds) {
                var result = 0;
                for (var _i = 0, types_7 = types; _i < types_7.length; _i++) {
                    var type = types_7[_i];
                    if (!(type.flags & excludeKinds)) {
                        result |= ts.getObjectFlags(type);
                    }
                }
                return result & 3670016 /* PropagatingFlags */;
            }
            function createTypeReference(target, typeArguments) {
                var id = getTypeListId(typeArguments);
                var type = target.instantiations.get(id);
                if (!type) {
                    type = createObjectType(4 /* Reference */, target.symbol);
                    target.instantiations.set(id, type);
                    type.objectFlags |= typeArguments ? getPropagatingFlagsOfTypes(typeArguments, /*excludeKinds*/ 0) : 0;
                    type.target = target;
                    type.resolvedTypeArguments = typeArguments;
                }
                return type;
            }
            function cloneTypeReference(source) {
                var type = createType(source.flags);
                type.symbol = source.symbol;
                type.objectFlags = source.objectFlags;
                type.target = source.target;
                type.resolvedTypeArguments = source.resolvedTypeArguments;
                return type;
            }
            function createDeferredTypeReference(target, node, mapper) {
                var aliasSymbol = getAliasSymbolForTypeNode(node);
                var aliasTypeArguments = getTypeArgumentsForAliasSymbol(aliasSymbol);
                var type = createObjectType(4 /* Reference */, target.symbol);
                type.target = target;
                type.node = node;
                type.mapper = mapper;
                type.aliasSymbol = aliasSymbol;
                type.aliasTypeArguments = mapper ? instantiateTypes(aliasTypeArguments, mapper) : aliasTypeArguments;
                return type;
            }
            function getTypeArguments(type) {
                var _a, _b;
                if (!type.resolvedTypeArguments) {
                    if (!pushTypeResolution(type, 6 /* ResolvedTypeArguments */)) {
                        return ((_a = type.target.localTypeParameters) === null || _a === void 0 ? void 0 : _a.map(function () { return errorType; })) || ts.emptyArray;
                    }
                    var node = type.node;
                    var typeArguments = !node ? ts.emptyArray :
                        node.kind === 169 /* TypeReference */ ? ts.concatenate(type.target.outerTypeParameters, getEffectiveTypeArguments(node, type.target.localTypeParameters)) :
                            node.kind === 174 /* ArrayType */ ? [getTypeFromTypeNode(node.elementType)] :
                                ts.map(node.elementTypes, getTypeFromTypeNode);
                    if (popTypeResolution()) {
                        type.resolvedTypeArguments = type.mapper ? instantiateTypes(typeArguments, type.mapper) : typeArguments;
                    }
                    else {
                        type.resolvedTypeArguments = ((_b = type.target.localTypeParameters) === null || _b === void 0 ? void 0 : _b.map(function () { return errorType; })) || ts.emptyArray;
                        error(type.node || currentNode, type.target.symbol
                            ? ts.Diagnostics.Type_arguments_for_0_circularly_reference_themselves
                            : ts.Diagnostics.Tuple_type_arguments_circularly_reference_themselves, type.target.symbol && symbolToString(type.target.symbol));
                    }
                }
                return type.resolvedTypeArguments;
            }
            function getTypeReferenceArity(type) {
                return ts.length(type.target.typeParameters);
            }
            /**
             * Get type from type-reference that reference to class or interface
             */
            function getTypeFromClassOrInterfaceReference(node, symbol) {
                var type = getDeclaredTypeOfSymbol(getMergedSymbol(symbol));
                var typeParameters = type.localTypeParameters;
                if (typeParameters) {
                    var numTypeArguments = ts.length(node.typeArguments);
                    var minTypeArgumentCount = getMinTypeArgumentCount(typeParameters);
                    var isJs = ts.isInJSFile(node);
                    var isJsImplicitAny = !noImplicitAny && isJs;
                    if (!isJsImplicitAny && (numTypeArguments < minTypeArgumentCount || numTypeArguments > typeParameters.length)) {
                        var missingAugmentsTag = isJs && ts.isExpressionWithTypeArguments(node) && !ts.isJSDocAugmentsTag(node.parent);
                        var diag = minTypeArgumentCount === typeParameters.length ?
                            missingAugmentsTag ?
                                ts.Diagnostics.Expected_0_type_arguments_provide_these_with_an_extends_tag :
                                ts.Diagnostics.Generic_type_0_requires_1_type_argument_s :
                            missingAugmentsTag ?
                                ts.Diagnostics.Expected_0_1_type_arguments_provide_these_with_an_extends_tag :
                                ts.Diagnostics.Generic_type_0_requires_between_1_and_2_type_arguments;
                        var typeStr = typeToString(type, /*enclosingDeclaration*/ undefined, 2 /* WriteArrayAsGenericType */);
                        error(node, diag, typeStr, minTypeArgumentCount, typeParameters.length);
                        if (!isJs) {
                            // TODO: Adopt same permissive behavior in TS as in JS to reduce follow-on editing experience failures (requires editing fillMissingTypeArguments)
                            return errorType;
                        }
                    }
                    if (node.kind === 169 /* TypeReference */ && isAliasedType(node)) {
                        return createDeferredTypeReference(type, node, /*mapper*/ undefined);
                    }
                    // In a type reference, the outer type parameters of the referenced class or interface are automatically
                    // supplied as type arguments and the type reference only specifies arguments for the local type parameters
                    // of the class or interface.
                    var typeArguments = ts.concatenate(type.outerTypeParameters, fillMissingTypeArguments(typeArgumentsFromTypeReferenceNode(node), typeParameters, minTypeArgumentCount, isJs));
                    return createTypeReference(type, typeArguments);
                }
                return checkNoTypeArguments(node, symbol) ? type : errorType;
            }
            function getTypeAliasInstantiation(symbol, typeArguments) {
                var type = getDeclaredTypeOfSymbol(symbol);
                var links = getSymbolLinks(symbol);
                var typeParameters = links.typeParameters;
                var id = getTypeListId(typeArguments);
                var instantiation = links.instantiations.get(id);
                if (!instantiation) {
                    links.instantiations.set(id, instantiation = instantiateType(type, createTypeMapper(typeParameters, fillMissingTypeArguments(typeArguments, typeParameters, getMinTypeArgumentCount(typeParameters), ts.isInJSFile(symbol.valueDeclaration)))));
                }
                return instantiation;
            }
            /**
             * Get type from reference to type alias. When a type alias is generic, the declared type of the type alias may include
             * references to the type parameters of the alias. We replace those with the actual type arguments by instantiating the
             * declared type. Instantiations are cached using the type identities of the type arguments as the key.
             */
            function getTypeFromTypeAliasReference(node, symbol) {
                var type = getDeclaredTypeOfSymbol(symbol);
                var typeParameters = getSymbolLinks(symbol).typeParameters;
                if (typeParameters) {
                    var numTypeArguments = ts.length(node.typeArguments);
                    var minTypeArgumentCount = getMinTypeArgumentCount(typeParameters);
                    if (numTypeArguments < minTypeArgumentCount || numTypeArguments > typeParameters.length) {
                        error(node, minTypeArgumentCount === typeParameters.length ?
                            ts.Diagnostics.Generic_type_0_requires_1_type_argument_s :
                            ts.Diagnostics.Generic_type_0_requires_between_1_and_2_type_arguments, symbolToString(symbol), minTypeArgumentCount, typeParameters.length);
                        return errorType;
                    }
                    return getTypeAliasInstantiation(symbol, typeArgumentsFromTypeReferenceNode(node));
                }
                return checkNoTypeArguments(node, symbol) ? type : errorType;
            }
            function getTypeReferenceName(node) {
                switch (node.kind) {
                    case 169 /* TypeReference */:
                        return node.typeName;
                    case 216 /* ExpressionWithTypeArguments */:
                        // We only support expressions that are simple qualified names. For other
                        // expressions this produces undefined.
                        var expr = node.expression;
                        if (ts.isEntityNameExpression(expr)) {
                            return expr;
                        }
                    // fall through;
                }
                return undefined;
            }
            function resolveTypeReferenceName(typeReferenceName, meaning, ignoreErrors) {
                if (!typeReferenceName) {
                    return unknownSymbol;
                }
                return resolveEntityName(typeReferenceName, meaning, ignoreErrors) || unknownSymbol;
            }
            function getTypeReferenceType(node, symbol) {
                if (symbol === unknownSymbol) {
                    return errorType;
                }
                symbol = getExpandoSymbol(symbol) || symbol;
                if (symbol.flags & (32 /* Class */ | 64 /* Interface */)) {
                    return getTypeFromClassOrInterfaceReference(node, symbol);
                }
                if (symbol.flags & 524288 /* TypeAlias */) {
                    return getTypeFromTypeAliasReference(node, symbol);
                }
                // Get type from reference to named type that cannot be generic (enum or type parameter)
                var res = tryGetDeclaredTypeOfSymbol(symbol);
                if (res) {
                    return checkNoTypeArguments(node, symbol) ?
                        res.flags & 262144 /* TypeParameter */ ? getConstrainedTypeVariable(res, node) : getRegularTypeOfLiteralType(res) :
                        errorType;
                }
                if (symbol.flags & 111551 /* Value */ && isJSDocTypeReference(node)) {
                    var jsdocType = getTypeFromJSDocValueReference(node, symbol);
                    if (jsdocType) {
                        return jsdocType;
                    }
                    else {
                        // Resolve the type reference as a Type for the purpose of reporting errors.
                        resolveTypeReferenceName(getTypeReferenceName(node), 788968 /* Type */);
                        return getTypeOfSymbol(symbol);
                    }
                }
                return errorType;
            }
            /**
             * A JSdoc TypeReference may be to a value, but resolve it as a type anyway.
             * Note: If the value is imported from commonjs, it should really be an alias,
             * but this function's special-case code fakes alias resolution as well.
             */
            function getTypeFromJSDocValueReference(node, symbol) {
                var links = getNodeLinks(node);
                if (!links.resolvedJSDocType) {
                    var valueType = getTypeOfSymbol(symbol);
                    var typeType = valueType;
                    if (symbol.valueDeclaration) {
                        var decl = ts.getRootDeclaration(symbol.valueDeclaration);
                        var isRequireAlias = false;
                        if (ts.isVariableDeclaration(decl) && decl.initializer) {
                            var expr = decl.initializer;
                            // skip past entity names, eg `require("x").a.b.c`
                            while (ts.isPropertyAccessExpression(expr)) {
                                expr = expr.expression;
                            }
                            isRequireAlias = ts.isCallExpression(expr) && ts.isRequireCall(expr, /*requireStringLiteralLikeArgument*/ true) && !!valueType.symbol;
                        }
                        var isImportTypeWithQualifier = node.kind === 188 /* ImportType */ && node.qualifier;
                        // valueType might not have a symbol, eg, {import('./b').STRING_LITERAL}
                        if (valueType.symbol && (isRequireAlias || isImportTypeWithQualifier)) {
                            typeType = getTypeReferenceType(node, valueType.symbol);
                        }
                    }
                    links.resolvedJSDocType = typeType;
                }
                return links.resolvedJSDocType;
            }
            function getSubstitutionType(typeVariable, substitute) {
                if (substitute.flags & 3 /* AnyOrUnknown */ || substitute === typeVariable) {
                    return typeVariable;
                }
                var id = getTypeId(typeVariable) + ">" + getTypeId(substitute);
                var cached = substitutionTypes.get(id);
                if (cached) {
                    return cached;
                }
                var result = createType(33554432 /* Substitution */);
                result.typeVariable = typeVariable;
                result.substitute = substitute;
                substitutionTypes.set(id, result);
                return result;
            }
            function isUnaryTupleTypeNode(node) {
                return node.kind === 175 /* TupleType */ && node.elementTypes.length === 1;
            }
            function getImpliedConstraint(typeVariable, checkNode, extendsNode) {
                return isUnaryTupleTypeNode(checkNode) && isUnaryTupleTypeNode(extendsNode) ? getImpliedConstraint(typeVariable, checkNode.elementTypes[0], extendsNode.elementTypes[0]) :
                    getActualTypeVariable(getTypeFromTypeNode(checkNode)) === typeVariable ? getTypeFromTypeNode(extendsNode) :
                        undefined;
            }
            function getConstrainedTypeVariable(typeVariable, node) {
                var constraints;
                while (node && !ts.isStatement(node) && node.kind !== 303 /* JSDocComment */) {
                    var parent = node.parent;
                    if (parent.kind === 180 /* ConditionalType */ && node === parent.trueType) {
                        var constraint = getImpliedConstraint(typeVariable, parent.checkType, parent.extendsType);
                        if (constraint) {
                            constraints = ts.append(constraints, constraint);
                        }
                    }
                    node = parent;
                }
                return constraints ? getSubstitutionType(typeVariable, getIntersectionType(ts.append(constraints, typeVariable))) : typeVariable;
            }
            function isJSDocTypeReference(node) {
                return !!(node.flags & 4194304 /* JSDoc */) && (node.kind === 169 /* TypeReference */ || node.kind === 188 /* ImportType */);
            }
            function checkNoTypeArguments(node, symbol) {
                if (node.typeArguments) {
                    error(node, ts.Diagnostics.Type_0_is_not_generic, symbol ? symbolToString(symbol) : node.typeName ? ts.declarationNameToString(node.typeName) : anon);
                    return false;
                }
                return true;
            }
            function getIntendedTypeFromJSDocTypeReference(node) {
                if (ts.isIdentifier(node.typeName)) {
                    var typeArgs = node.typeArguments;
                    switch (node.typeName.escapedText) {
                        case "String":
                            checkNoTypeArguments(node);
                            return stringType;
                        case "Number":
                            checkNoTypeArguments(node);
                            return numberType;
                        case "Boolean":
                            checkNoTypeArguments(node);
                            return booleanType;
                        case "Void":
                            checkNoTypeArguments(node);
                            return voidType;
                        case "Undefined":
                            checkNoTypeArguments(node);
                            return undefinedType;
                        case "Null":
                            checkNoTypeArguments(node);
                            return nullType;
                        case "Function":
                        case "function":
                            checkNoTypeArguments(node);
                            return globalFunctionType;
                        case "array":
                            return (!typeArgs || !typeArgs.length) && !noImplicitAny ? anyArrayType : undefined;
                        case "promise":
                            return (!typeArgs || !typeArgs.length) && !noImplicitAny ? createPromiseType(anyType) : undefined;
                        case "Object":
                            if (typeArgs && typeArgs.length === 2) {
                                if (ts.isJSDocIndexSignature(node)) {
                                    var indexed = getTypeFromTypeNode(typeArgs[0]);
                                    var target = getTypeFromTypeNode(typeArgs[1]);
                                    var index = createIndexInfo(target, /*isReadonly*/ false);
                                    return createAnonymousType(undefined, emptySymbols, ts.emptyArray, ts.emptyArray, indexed === stringType ? index : undefined, indexed === numberType ? index : undefined);
                                }
                                return anyType;
                            }
                            checkNoTypeArguments(node);
                            return !noImplicitAny ? anyType : undefined;
                    }
                }
            }
            function getTypeFromJSDocNullableTypeNode(node) {
                var type = getTypeFromTypeNode(node.type);
                return strictNullChecks ? getNullableType(type, 65536 /* Null */) : type;
            }
            function getTypeFromTypeReference(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    var symbol = void 0;
                    var type = void 0;
                    var meaning = 788968 /* Type */;
                    if (isJSDocTypeReference(node)) {
                        type = getIntendedTypeFromJSDocTypeReference(node);
                        if (!type) {
                            symbol = resolveTypeReferenceName(getTypeReferenceName(node), meaning, /*ignoreErrors*/ true);
                            if (symbol === unknownSymbol) {
                                symbol = resolveTypeReferenceName(getTypeReferenceName(node), meaning | 111551 /* Value */);
                            }
                            else {
                                resolveTypeReferenceName(getTypeReferenceName(node), meaning); // Resolve again to mark errors, if any
                            }
                            type = getTypeReferenceType(node, symbol);
                        }
                    }
                    if (!type) {
                        symbol = resolveTypeReferenceName(getTypeReferenceName(node), meaning);
                        type = getTypeReferenceType(node, symbol);
                    }
                    // Cache both the resolved symbol and the resolved type. The resolved symbol is needed when we check the
                    // type reference in checkTypeReferenceNode.
                    links.resolvedSymbol = symbol;
                    links.resolvedType = type;
                }
                return links.resolvedType;
            }
            function typeArgumentsFromTypeReferenceNode(node) {
                return ts.map(node.typeArguments, getTypeFromTypeNode);
            }
            function getTypeFromTypeQueryNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    // TypeScript 1.0 spec (April 2014): 3.6.3
                    // The expression is processed as an identifier expression (section 4.3)
                    // or property access expression(section 4.10),
                    // the widened type(section 3.9) of which becomes the result.
                    links.resolvedType = getRegularTypeOfLiteralType(getWidenedType(checkExpression(node.exprName)));
                }
                return links.resolvedType;
            }
            function getTypeOfGlobalSymbol(symbol, arity) {
                function getTypeDeclaration(symbol) {
                    var declarations = symbol.declarations;
                    for (var _i = 0, declarations_3 = declarations; _i < declarations_3.length; _i++) {
                        var declaration = declarations_3[_i];
                        switch (declaration.kind) {
                            case 245 /* ClassDeclaration */:
                            case 246 /* InterfaceDeclaration */:
                            case 248 /* EnumDeclaration */:
                                return declaration;
                        }
                    }
                }
                if (!symbol) {
                    return arity ? emptyGenericType : emptyObjectType;
                }
                var type = getDeclaredTypeOfSymbol(symbol);
                if (!(type.flags & 524288 /* Object */)) {
                    error(getTypeDeclaration(symbol), ts.Diagnostics.Global_type_0_must_be_a_class_or_interface_type, ts.symbolName(symbol));
                    return arity ? emptyGenericType : emptyObjectType;
                }
                if (ts.length(type.typeParameters) !== arity) {
                    error(getTypeDeclaration(symbol), ts.Diagnostics.Global_type_0_must_have_1_type_parameter_s, ts.symbolName(symbol), arity);
                    return arity ? emptyGenericType : emptyObjectType;
                }
                return type;
            }
            function getGlobalValueSymbol(name, reportErrors) {
                return getGlobalSymbol(name, 111551 /* Value */, reportErrors ? ts.Diagnostics.Cannot_find_global_value_0 : undefined);
            }
            function getGlobalTypeSymbol(name, reportErrors) {
                return getGlobalSymbol(name, 788968 /* Type */, reportErrors ? ts.Diagnostics.Cannot_find_global_type_0 : undefined);
            }
            function getGlobalSymbol(name, meaning, diagnostic) {
                // Don't track references for global symbols anyway, so value if `isReference` is arbitrary
                return resolveName(undefined, name, meaning, diagnostic, name, /*isUse*/ false);
            }
            function getGlobalType(name, arity, reportErrors) {
                var symbol = getGlobalTypeSymbol(name, reportErrors);
                return symbol || reportErrors ? getTypeOfGlobalSymbol(symbol, arity) : undefined;
            }
            function getGlobalTypedPropertyDescriptorType() {
                return deferredGlobalTypedPropertyDescriptorType || (deferredGlobalTypedPropertyDescriptorType = getGlobalType("TypedPropertyDescriptor", /*arity*/ 1, /*reportErrors*/ true)) || emptyGenericType;
            }
            function getGlobalTemplateStringsArrayType() {
                return deferredGlobalTemplateStringsArrayType || (deferredGlobalTemplateStringsArrayType = getGlobalType("TemplateStringsArray", /*arity*/ 0, /*reportErrors*/ true)) || emptyObjectType;
            }
            function getGlobalImportMetaType() {
                return deferredGlobalImportMetaType || (deferredGlobalImportMetaType = getGlobalType("ImportMeta", /*arity*/ 0, /*reportErrors*/ true)) || emptyObjectType;
            }
            function getGlobalESSymbolConstructorSymbol(reportErrors) {
                return deferredGlobalESSymbolConstructorSymbol || (deferredGlobalESSymbolConstructorSymbol = getGlobalValueSymbol("Symbol", reportErrors));
            }
            function getGlobalESSymbolType(reportErrors) {
                return deferredGlobalESSymbolType || (deferredGlobalESSymbolType = getGlobalType("Symbol", /*arity*/ 0, reportErrors)) || emptyObjectType;
            }
            function getGlobalPromiseType(reportErrors) {
                return deferredGlobalPromiseType || (deferredGlobalPromiseType = getGlobalType("Promise", /*arity*/ 1, reportErrors)) || emptyGenericType;
            }
            function getGlobalPromiseLikeType(reportErrors) {
                return deferredGlobalPromiseLikeType || (deferredGlobalPromiseLikeType = getGlobalType("PromiseLike", /*arity*/ 1, reportErrors)) || emptyGenericType;
            }
            function getGlobalPromiseConstructorSymbol(reportErrors) {
                return deferredGlobalPromiseConstructorSymbol || (deferredGlobalPromiseConstructorSymbol = getGlobalValueSymbol("Promise", reportErrors));
            }
            function getGlobalPromiseConstructorLikeType(reportErrors) {
                return deferredGlobalPromiseConstructorLikeType || (deferredGlobalPromiseConstructorLikeType = getGlobalType("PromiseConstructorLike", /*arity*/ 0, reportErrors)) || emptyObjectType;
            }
            function getGlobalAsyncIterableType(reportErrors) {
                return deferredGlobalAsyncIterableType || (deferredGlobalAsyncIterableType = getGlobalType("AsyncIterable", /*arity*/ 1, reportErrors)) || emptyGenericType;
            }
            function getGlobalAsyncIteratorType(reportErrors) {
                return deferredGlobalAsyncIteratorType || (deferredGlobalAsyncIteratorType = getGlobalType("AsyncIterator", /*arity*/ 3, reportErrors)) || emptyGenericType;
            }
            function getGlobalAsyncIterableIteratorType(reportErrors) {
                return deferredGlobalAsyncIterableIteratorType || (deferredGlobalAsyncIterableIteratorType = getGlobalType("AsyncIterableIterator", /*arity*/ 1, reportErrors)) || emptyGenericType;
            }
            function getGlobalAsyncGeneratorType(reportErrors) {
                return deferredGlobalAsyncGeneratorType || (deferredGlobalAsyncGeneratorType = getGlobalType("AsyncGenerator", /*arity*/ 3, reportErrors)) || emptyGenericType;
            }
            function getGlobalIterableType(reportErrors) {
                return deferredGlobalIterableType || (deferredGlobalIterableType = getGlobalType("Iterable", /*arity*/ 1, reportErrors)) || emptyGenericType;
            }
            function getGlobalIteratorType(reportErrors) {
                return deferredGlobalIteratorType || (deferredGlobalIteratorType = getGlobalType("Iterator", /*arity*/ 3, reportErrors)) || emptyGenericType;
            }
            function getGlobalIterableIteratorType(reportErrors) {
                return deferredGlobalIterableIteratorType || (deferredGlobalIterableIteratorType = getGlobalType("IterableIterator", /*arity*/ 1, reportErrors)) || emptyGenericType;
            }
            function getGlobalGeneratorType(reportErrors) {
                return deferredGlobalGeneratorType || (deferredGlobalGeneratorType = getGlobalType("Generator", /*arity*/ 3, reportErrors)) || emptyGenericType;
            }
            function getGlobalIteratorYieldResultType(reportErrors) {
                return deferredGlobalIteratorYieldResultType || (deferredGlobalIteratorYieldResultType = getGlobalType("IteratorYieldResult", /*arity*/ 1, reportErrors)) || emptyGenericType;
            }
            function getGlobalIteratorReturnResultType(reportErrors) {
                return deferredGlobalIteratorReturnResultType || (deferredGlobalIteratorReturnResultType = getGlobalType("IteratorReturnResult", /*arity*/ 1, reportErrors)) || emptyGenericType;
            }
            function getGlobalTypeOrUndefined(name, arity) {
                if (arity === void 0) {
                    arity = 0;
                }
                var symbol = getGlobalSymbol(name, 788968 /* Type */, /*diagnostic*/ undefined);
                return symbol && getTypeOfGlobalSymbol(symbol, arity);
            }
            function getGlobalExtractSymbol() {
                return deferredGlobalExtractSymbol || (deferredGlobalExtractSymbol = getGlobalSymbol("Extract", 524288 /* TypeAlias */, ts.Diagnostics.Cannot_find_global_type_0)); // TODO: GH#18217
            }
            function getGlobalOmitSymbol() {
                return deferredGlobalOmitSymbol || (deferredGlobalOmitSymbol = getGlobalSymbol("Omit", 524288 /* TypeAlias */, ts.Diagnostics.Cannot_find_global_type_0)); // TODO: GH#18217
            }
            function getGlobalBigIntType(reportErrors) {
                return deferredGlobalBigIntType || (deferredGlobalBigIntType = getGlobalType("BigInt", /*arity*/ 0, reportErrors)) || emptyObjectType;
            }
            /**
             * Instantiates a global type that is generic with some element type, and returns that instantiation.
             */
            function createTypeFromGenericGlobalType(genericGlobalType, typeArguments) {
                return genericGlobalType !== emptyGenericType ? createTypeReference(genericGlobalType, typeArguments) : emptyObjectType;
            }
            function createTypedPropertyDescriptorType(propertyType) {
                return createTypeFromGenericGlobalType(getGlobalTypedPropertyDescriptorType(), [propertyType]);
            }
            function createIterableType(iteratedType) {
                return createTypeFromGenericGlobalType(getGlobalIterableType(/*reportErrors*/ true), [iteratedType]);
            }
            function createArrayType(elementType, readonly) {
                return createTypeFromGenericGlobalType(readonly ? globalReadonlyArrayType : globalArrayType, [elementType]);
            }
            function getArrayOrTupleTargetType(node) {
                var readonly = isReadonlyTypeOperator(node.parent);
                if (node.kind === 174 /* ArrayType */ || node.elementTypes.length === 1 && node.elementTypes[0].kind === 177 /* RestType */) {
                    return readonly ? globalReadonlyArrayType : globalArrayType;
                }
                var lastElement = ts.lastOrUndefined(node.elementTypes);
                var restElement = lastElement && lastElement.kind === 177 /* RestType */ ? lastElement : undefined;
                var minLength = ts.findLastIndex(node.elementTypes, function (n) { return n.kind !== 176 /* OptionalType */ && n !== restElement; }) + 1;
                return getTupleTypeOfArity(node.elementTypes.length, minLength, !!restElement, readonly, /*associatedNames*/ undefined);
            }
            // Return true when the given node is transitively contained in type constructs that eagerly
            // resolve their constituent types. We include SyntaxKind.TypeReference because type arguments
            // of type aliases are eagerly resolved.
            function isAliasedType(node) {
                var parent = node.parent;
                switch (parent.kind) {
                    case 182 /* ParenthesizedType */:
                    case 169 /* TypeReference */:
                    case 178 /* UnionType */:
                    case 179 /* IntersectionType */:
                    case 185 /* IndexedAccessType */:
                    case 180 /* ConditionalType */:
                    case 184 /* TypeOperator */:
                        return isAliasedType(parent);
                    case 247 /* TypeAliasDeclaration */:
                        return true;
                }
                return false;
            }
            function getTypeFromArrayOrTupleTypeNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    var target = getArrayOrTupleTargetType(node);
                    if (target === emptyGenericType) {
                        links.resolvedType = emptyObjectType;
                    }
                    else if (isAliasedType(node)) {
                        links.resolvedType = node.kind === 175 /* TupleType */ && node.elementTypes.length === 0 ? target :
                            createDeferredTypeReference(target, node, /*mapper*/ undefined);
                    }
                    else {
                        var elementTypes = node.kind === 174 /* ArrayType */ ? [getTypeFromTypeNode(node.elementType)] : ts.map(node.elementTypes, getTypeFromTypeNode);
                        links.resolvedType = createTypeReference(target, elementTypes);
                    }
                }
                return links.resolvedType;
            }
            function isReadonlyTypeOperator(node) {
                return ts.isTypeOperatorNode(node) && node.operator === 138 /* ReadonlyKeyword */;
            }
            // We represent tuple types as type references to synthesized generic interface types created by
            // this function. The types are of the form:
            //
            //   interface Tuple<T0, T1, T2, ...> extends Array<T0 | T1 | T2 | ...> { 0: T0, 1: T1, 2: T2, ... }
            //
            // Note that the generic type created by this function has no symbol associated with it. The same
            // is true for each of the synthesized type parameters.
            function createTupleTypeOfArity(arity, minLength, hasRestElement, readonly, associatedNames) {
                var typeParameters;
                var properties = [];
                var maxLength = hasRestElement ? arity - 1 : arity;
                if (arity) {
                    typeParameters = new Array(arity);
                    for (var i = 0; i < arity; i++) {
                        var typeParameter = typeParameters[i] = createTypeParameter();
                        if (i < maxLength) {
                            var property = createSymbol(4 /* Property */ | (i >= minLength ? 16777216 /* Optional */ : 0), "" + i, readonly ? 8 /* Readonly */ : 0);
                            property.type = typeParameter;
                            properties.push(property);
                        }
                    }
                }
                var literalTypes = [];
                for (var i = minLength; i <= maxLength; i++)
                    literalTypes.push(getLiteralType(i));
                var lengthSymbol = createSymbol(4 /* Property */, "length");
                lengthSymbol.type = hasRestElement ? numberType : getUnionType(literalTypes);
                properties.push(lengthSymbol);
                var type = createObjectType(8 /* Tuple */ | 4 /* Reference */);
                type.typeParameters = typeParameters;
                type.outerTypeParameters = undefined;
                type.localTypeParameters = typeParameters;
                type.instantiations = ts.createMap();
                type.instantiations.set(getTypeListId(type.typeParameters), type);
                type.target = type;
                type.resolvedTypeArguments = type.typeParameters;
                type.thisType = createTypeParameter();
                type.thisType.isThisType = true;
                type.thisType.constraint = type;
                type.declaredProperties = properties;
                type.declaredCallSignatures = ts.emptyArray;
                type.declaredConstructSignatures = ts.emptyArray;
                type.declaredStringIndexInfo = undefined;
                type.declaredNumberIndexInfo = undefined;
                type.minLength = minLength;
                type.hasRestElement = hasRestElement;
                type.readonly = readonly;
                type.associatedNames = associatedNames;
                return type;
            }
            function getTupleTypeOfArity(arity, minLength, hasRestElement, readonly, associatedNames) {
                var key = arity + (hasRestElement ? "+" : ",") + minLength + (readonly ? "R" : "") + (associatedNames && associatedNames.length ? "," + associatedNames.join(",") : "");
                var type = tupleTypes.get(key);
                if (!type) {
                    tupleTypes.set(key, type = createTupleTypeOfArity(arity, minLength, hasRestElement, readonly, associatedNames));
                }
                return type;
            }
            function createTupleType(elementTypes, minLength, hasRestElement, readonly, associatedNames) {
                if (minLength === void 0) {
                    minLength = elementTypes.length;
                }
                if (hasRestElement === void 0) {
                    hasRestElement = false;
                }
                if (readonly === void 0) {
                    readonly = false;
                }
                var arity = elementTypes.length;
                if (arity === 1 && hasRestElement) {
                    return createArrayType(elementTypes[0], readonly);
                }
                var tupleType = getTupleTypeOfArity(arity, minLength, arity > 0 && hasRestElement, readonly, associatedNames);
                return elementTypes.length ? createTypeReference(tupleType, elementTypes) : tupleType;
            }
            function sliceTupleType(type, index) {
                var tuple = type.target;
                if (tuple.hasRestElement) {
                    // don't slice off rest element
                    index = Math.min(index, getTypeReferenceArity(type) - 1);
                }
                return createTupleType(getTypeArguments(type).slice(index), Math.max(0, tuple.minLength - index), tuple.hasRestElement, tuple.readonly, tuple.associatedNames && tuple.associatedNames.slice(index));
            }
            function getTypeFromOptionalTypeNode(node) {
                var type = getTypeFromTypeNode(node.type);
                return strictNullChecks ? getOptionalType(type) : type;
            }
            function getTypeId(type) {
                return type.id;
            }
            function containsType(types, type) {
                return ts.binarySearch(types, type, getTypeId, ts.compareValues) >= 0;
            }
            function insertType(types, type) {
                var index = ts.binarySearch(types, type, getTypeId, ts.compareValues);
                if (index < 0) {
                    types.splice(~index, 0, type);
                    return true;
                }
                return false;
            }
            function addTypeToUnion(typeSet, includes, type) {
                var flags = type.flags;
                if (flags & 1048576 /* Union */) {
                    return addTypesToUnion(typeSet, includes, type.types);
                }
                // We ignore 'never' types in unions
                if (!(flags & 131072 /* Never */)) {
                    includes |= flags & 68943871 /* IncludesMask */;
                    if (flags & 66846720 /* StructuredOrInstantiable */)
                        includes |= 262144 /* IncludesStructuredOrInstantiable */;
                    if (type === wildcardType)
                        includes |= 4194304 /* IncludesWildcard */;
                    if (!strictNullChecks && flags & 98304 /* Nullable */) {
                        if (!(ts.getObjectFlags(type) & 524288 /* ContainsWideningType */))
                            includes |= 2097152 /* IncludesNonWideningType */;
                    }
                    else {
                        var len = typeSet.length;
                        var index = len && type.id > typeSet[len - 1].id ? ~len : ts.binarySearch(typeSet, type, getTypeId, ts.compareValues);
                        if (index < 0) {
                            typeSet.splice(~index, 0, type);
                        }
                    }
                }
                return includes;
            }
            // Add the given types to the given type set. Order is preserved, duplicates are removed,
            // and nested types of the given kind are flattened into the set.
            function addTypesToUnion(typeSet, includes, types) {
                for (var _i = 0, types_8 = types; _i < types_8.length; _i++) {
                    var type = types_8[_i];
                    includes = addTypeToUnion(typeSet, includes, type);
                }
                return includes;
            }
            function isSetOfLiteralsFromSameEnum(types) {
                var first = types[0];
                if (first.flags & 1024 /* EnumLiteral */) {
                    var firstEnum = getParentOfSymbol(first.symbol);
                    for (var i = 1; i < types.length; i++) {
                        var other = types[i];
                        if (!(other.flags & 1024 /* EnumLiteral */) || (firstEnum !== getParentOfSymbol(other.symbol))) {
                            return false;
                        }
                    }
                    return true;
                }
                return false;
            }
            function removeSubtypes(types, primitivesOnly) {
                var len = types.length;
                if (len === 0 || isSetOfLiteralsFromSameEnum(types)) {
                    return true;
                }
                var i = len;
                var count = 0;
                while (i > 0) {
                    i--;
                    var source = types[i];
                    for (var _i = 0, types_9 = types; _i < types_9.length; _i++) {
                        var target = types_9[_i];
                        if (source !== target) {
                            if (count === 100000) {
                                // After 100000 subtype checks we estimate the remaining amount of work by assuming the
                                // same ratio of checks per element. If the estimated number of remaining type checks is
                                // greater than an upper limit we deem the union type too complex to represent. The
                                // upper limit is 25M for unions of primitives only, and 1M otherwise. This for example
                                // caps union types at 5000 unique literal types and 1000 unique object types.
                                var estimatedCount = (count / (len - i)) * len;
                                if (estimatedCount > (primitivesOnly ? 25000000 : 1000000)) {
                                    error(currentNode, ts.Diagnostics.Expression_produces_a_union_type_that_is_too_complex_to_represent);
                                    return false;
                                }
                            }
                            count++;
                            if (isTypeRelatedTo(source, target, strictSubtypeRelation) && (!(ts.getObjectFlags(getTargetType(source)) & 1 /* Class */) ||
                                !(ts.getObjectFlags(getTargetType(target)) & 1 /* Class */) ||
                                isTypeDerivedFrom(source, target))) {
                                ts.orderedRemoveItemAt(types, i);
                                break;
                            }
                        }
                    }
                }
                return true;
            }
            function removeRedundantLiteralTypes(types, includes) {
                var i = types.length;
                while (i > 0) {
                    i--;
                    var t = types[i];
                    var remove = t.flags & 128 /* StringLiteral */ && includes & 4 /* String */ ||
                        t.flags & 256 /* NumberLiteral */ && includes & 8 /* Number */ ||
                        t.flags & 2048 /* BigIntLiteral */ && includes & 64 /* BigInt */ ||
                        t.flags & 8192 /* UniqueESSymbol */ && includes & 4096 /* ESSymbol */ ||
                        isFreshLiteralType(t) && containsType(types, t.regularType);
                    if (remove) {
                        ts.orderedRemoveItemAt(types, i);
                    }
                }
            }
            // We sort and deduplicate the constituent types based on object identity. If the subtypeReduction
            // flag is specified we also reduce the constituent type set to only include types that aren't subtypes
            // of other types. Subtype reduction is expensive for large union types and is possible only when union
            // types are known not to circularly reference themselves (as is the case with union types created by
            // expression constructs such as array literals and the || and ?: operators). Named types can
            // circularly reference themselves and therefore cannot be subtype reduced during their declaration.
            // For example, "type Item = string | (() => Item" is a named type that circularly references itself.
            function getUnionType(types, unionReduction, aliasSymbol, aliasTypeArguments) {
                if (unionReduction === void 0) {
                    unionReduction = 1 /* Literal */;
                }
                if (types.length === 0) {
                    return neverType;
                }
                if (types.length === 1) {
                    return types[0];
                }
                var typeSet = [];
                var includes = addTypesToUnion(typeSet, 0, types);
                if (unionReduction !== 0 /* None */) {
                    if (includes & 3 /* AnyOrUnknown */) {
                        return includes & 1 /* Any */ ? includes & 4194304 /* IncludesWildcard */ ? wildcardType : anyType : unknownType;
                    }
                    switch (unionReduction) {
                        case 1 /* Literal */:
                            if (includes & (2944 /* Literal */ | 8192 /* UniqueESSymbol */)) {
                                removeRedundantLiteralTypes(typeSet, includes);
                            }
                            break;
                        case 2 /* Subtype */:
                            if (!removeSubtypes(typeSet, !(includes & 262144 /* IncludesStructuredOrInstantiable */))) {
                                return errorType;
                            }
                            break;
                    }
                    if (typeSet.length === 0) {
                        return includes & 65536 /* Null */ ? includes & 2097152 /* IncludesNonWideningType */ ? nullType : nullWideningType :
                            includes & 32768 /* Undefined */ ? includes & 2097152 /* IncludesNonWideningType */ ? undefinedType : undefinedWideningType :
                                neverType;
                    }
                }
                return getUnionTypeFromSortedList(typeSet, includes & 66994211 /* NotPrimitiveUnion */ ? 0 : 262144 /* PrimitiveUnion */, aliasSymbol, aliasTypeArguments);
            }
            function getUnionTypePredicate(signatures) {
                var first;
                var types = [];
                for (var _i = 0, signatures_6 = signatures; _i < signatures_6.length; _i++) {
                    var sig = signatures_6[_i];
                    var pred = getTypePredicateOfSignature(sig);
                    if (!pred || pred.kind === 2 /* AssertsThis */ || pred.kind === 3 /* AssertsIdentifier */) {
                        continue;
                    }
                    if (first) {
                        if (!typePredicateKindsMatch(first, pred)) {
                            // No common type predicate.
                            return undefined;
                        }
                    }
                    else {
                        first = pred;
                    }
                    types.push(pred.type);
                }
                if (!first) {
                    // No union signatures had a type predicate.
                    return undefined;
                }
                var unionType = getUnionType(types);
                return createTypePredicate(first.kind, first.parameterName, first.parameterIndex, unionType);
            }
            function typePredicateKindsMatch(a, b) {
                return a.kind === b.kind && a.parameterIndex === b.parameterIndex;
            }
            // This function assumes the constituent type list is sorted and deduplicated.
            function getUnionTypeFromSortedList(types, objectFlags, aliasSymbol, aliasTypeArguments) {
                if (types.length === 0) {
                    return neverType;
                }
                if (types.length === 1) {
                    return types[0];
                }
                var id = getTypeListId(types);
                var type = unionTypes.get(id);
                if (!type) {
                    type = createType(1048576 /* Union */);
                    unionTypes.set(id, type);
                    type.objectFlags = objectFlags | getPropagatingFlagsOfTypes(types, /*excludeKinds*/ 98304 /* Nullable */);
                    type.types = types;
                    /*
                    Note: This is the alias symbol (or lack thereof) that we see when we first encounter this union type.
                    For aliases of identical unions, eg `type T = A | B; type U = A | B`, the symbol of the first alias encountered is the aliasSymbol.
                    (In the language service, the order may depend on the order in which a user takes actions, such as hovering over symbols.)
                    It's important that we create equivalent union types only once, so that's an unfortunate side effect.
                    */
                    type.aliasSymbol = aliasSymbol;
                    type.aliasTypeArguments = aliasTypeArguments;
                }
                return type;
            }
            function getTypeFromUnionTypeNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    var aliasSymbol = getAliasSymbolForTypeNode(node);
                    links.resolvedType = getUnionType(ts.map(node.types, getTypeFromTypeNode), 1 /* Literal */, aliasSymbol, getTypeArgumentsForAliasSymbol(aliasSymbol));
                }
                return links.resolvedType;
            }
            function addTypeToIntersection(typeSet, includes, type) {
                var flags = type.flags;
                if (flags & 2097152 /* Intersection */) {
                    return addTypesToIntersection(typeSet, includes, type.types);
                }
                if (isEmptyAnonymousObjectType(type)) {
                    if (!(includes & 8388608 /* IncludesEmptyObject */)) {
                        includes |= 8388608 /* IncludesEmptyObject */;
                        typeSet.set(type.id.toString(), type);
                    }
                }
                else {
                    if (flags & 3 /* AnyOrUnknown */) {
                        if (type === wildcardType)
                            includes |= 4194304 /* IncludesWildcard */;
                    }
                    else if ((strictNullChecks || !(flags & 98304 /* Nullable */)) && !typeSet.has(type.id.toString())) {
                        if (type.flags & 109440 /* Unit */ && includes & 109440 /* Unit */) {
                            // We have seen two distinct unit types which means we should reduce to an
                            // empty intersection. Adding TypeFlags.NonPrimitive causes that to happen.
                            includes |= 67108864 /* NonPrimitive */;
                        }
                        typeSet.set(type.id.toString(), type);
                    }
                    includes |= flags & 68943871 /* IncludesMask */;
                }
                return includes;
            }
            // Add the given types to the given type set. Order is preserved, freshness is removed from literal
            // types, duplicates are removed, and nested types of the given kind are flattened into the set.
            function addTypesToIntersection(typeSet, includes, types) {
                for (var _i = 0, types_10 = types; _i < types_10.length; _i++) {
                    var type = types_10[_i];
                    includes = addTypeToIntersection(typeSet, includes, getRegularTypeOfLiteralType(type));
                }
                return includes;
            }
            function removeRedundantPrimitiveTypes(types, includes) {
                var i = types.length;
                while (i > 0) {
                    i--;
                    var t = types[i];
                    var remove = t.flags & 4 /* String */ && includes & 128 /* StringLiteral */ ||
                        t.flags & 8 /* Number */ && includes & 256 /* NumberLiteral */ ||
                        t.flags & 64 /* BigInt */ && includes & 2048 /* BigIntLiteral */ ||
                        t.flags & 4096 /* ESSymbol */ && includes & 8192 /* UniqueESSymbol */;
                    if (remove) {
                        ts.orderedRemoveItemAt(types, i);
                    }
                }
            }
            // Check that the given type has a match in every union. A given type is matched by
            // an identical type, and a literal type is additionally matched by its corresponding
            // primitive type.
            function eachUnionContains(unionTypes, type) {
                for (var _i = 0, unionTypes_1 = unionTypes; _i < unionTypes_1.length; _i++) {
                    var u = unionTypes_1[_i];
                    if (!containsType(u.types, type)) {
                        var primitive = type.flags & 128 /* StringLiteral */ ? stringType :
                            type.flags & 256 /* NumberLiteral */ ? numberType :
                                type.flags & 2048 /* BigIntLiteral */ ? bigintType :
                                    type.flags & 8192 /* UniqueESSymbol */ ? esSymbolType :
                                        undefined;
                        if (!primitive || !containsType(u.types, primitive)) {
                            return false;
                        }
                    }
                }
                return true;
            }
            function extractIrreducible(types, flag) {
                if (ts.every(types, function (t) { return !!(t.flags & 1048576 /* Union */) && ts.some(t.types, function (tt) { return !!(tt.flags & flag); }); })) {
                    for (var i = 0; i < types.length; i++) {
                        types[i] = filterType(types[i], function (t) { return !(t.flags & flag); });
                    }
                    return true;
                }
                return false;
            }
            // If the given list of types contains more than one union of primitive types, replace the
            // first with a union containing an intersection of those primitive types, then remove the
            // other unions and return true. Otherwise, do nothing and return false.
            function intersectUnionsOfPrimitiveTypes(types) {
                var unionTypes;
                var index = ts.findIndex(types, function (t) { return !!(ts.getObjectFlags(t) & 262144 /* PrimitiveUnion */); });
                if (index < 0) {
                    return false;
                }
                var i = index + 1;
                // Remove all but the first union of primitive types and collect them in
                // the unionTypes array.
                while (i < types.length) {
                    var t = types[i];
                    if (ts.getObjectFlags(t) & 262144 /* PrimitiveUnion */) {
                        (unionTypes || (unionTypes = [types[index]])).push(t);
                        ts.orderedRemoveItemAt(types, i);
                    }
                    else {
                        i++;
                    }
                }
                // Return false if there was only one union of primitive types
                if (!unionTypes) {
                    return false;
                }
                // We have more than one union of primitive types, now intersect them. For each
                // type in each union we check if the type is matched in every union and if so
                // we include it in the result.
                var checked = [];
                var result = [];
                for (var _i = 0, unionTypes_2 = unionTypes; _i < unionTypes_2.length; _i++) {
                    var u = unionTypes_2[_i];
                    for (var _a = 0, _b = u.types; _a < _b.length; _a++) {
                        var t = _b[_a];
                        if (insertType(checked, t)) {
                            if (eachUnionContains(unionTypes, t)) {
                                insertType(result, t);
                            }
                        }
                    }
                }
                // Finally replace the first union with the result
                types[index] = getUnionTypeFromSortedList(result, 262144 /* PrimitiveUnion */);
                return true;
            }
            function createIntersectionType(types, aliasSymbol, aliasTypeArguments) {
                var result = createType(2097152 /* Intersection */);
                result.objectFlags = getPropagatingFlagsOfTypes(types, /*excludeKinds*/ 98304 /* Nullable */);
                result.types = types;
                result.aliasSymbol = aliasSymbol; // See comment in `getUnionTypeFromSortedList`.
                result.aliasTypeArguments = aliasTypeArguments;
                return result;
            }
            // We normalize combinations of intersection and union types based on the distributive property of the '&'
            // operator. Specifically, because X & (A | B) is equivalent to X & A | X & B, we can transform intersection
            // types with union type constituents into equivalent union types with intersection type constituents and
            // effectively ensure that union types are always at the top level in type representations.
            //
            // We do not perform structural deduplication on intersection types. Intersection types are created only by the &
            // type operator and we can't reduce those because we want to support recursive intersection types. For example,
            // a type alias of the form "type List<T> = T & { next: List<T> }" cannot be reduced during its declaration.
            // Also, unlike union types, the order of the constituent types is preserved in order that overload resolution
            // for intersections of types with signatures can be deterministic.
            function getIntersectionType(types, aliasSymbol, aliasTypeArguments) {
                var typeMembershipMap = ts.createMap();
                var includes = addTypesToIntersection(typeMembershipMap, 0, types);
                var typeSet = ts.arrayFrom(typeMembershipMap.values());
                // An intersection type is considered empty if it contains
                // the type never, or
                // more than one unit type or,
                // an object type and a nullable type (null or undefined), or
                // a string-like type and a type known to be non-string-like, or
                // a number-like type and a type known to be non-number-like, or
                // a symbol-like type and a type known to be non-symbol-like, or
                // a void-like type and a type known to be non-void-like, or
                // a non-primitive type and a type known to be primitive.
                if (includes & 131072 /* Never */ ||
                    strictNullChecks && includes & 98304 /* Nullable */ && includes & (524288 /* Object */ | 67108864 /* NonPrimitive */ | 8388608 /* IncludesEmptyObject */) ||
                    includes & 67108864 /* NonPrimitive */ && includes & (67238908 /* DisjointDomains */ & ~67108864 /* NonPrimitive */) ||
                    includes & 132 /* StringLike */ && includes & (67238908 /* DisjointDomains */ & ~132 /* StringLike */) ||
                    includes & 296 /* NumberLike */ && includes & (67238908 /* DisjointDomains */ & ~296 /* NumberLike */) ||
                    includes & 2112 /* BigIntLike */ && includes & (67238908 /* DisjointDomains */ & ~2112 /* BigIntLike */) ||
                    includes & 12288 /* ESSymbolLike */ && includes & (67238908 /* DisjointDomains */ & ~12288 /* ESSymbolLike */) ||
                    includes & 49152 /* VoidLike */ && includes & (67238908 /* DisjointDomains */ & ~49152 /* VoidLike */)) {
                    return neverType;
                }
                if (includes & 1 /* Any */) {
                    return includes & 4194304 /* IncludesWildcard */ ? wildcardType : anyType;
                }
                if (!strictNullChecks && includes & 98304 /* Nullable */) {
                    return includes & 32768 /* Undefined */ ? undefinedType : nullType;
                }
                if (includes & 4 /* String */ && includes & 128 /* StringLiteral */ ||
                    includes & 8 /* Number */ && includes & 256 /* NumberLiteral */ ||
                    includes & 64 /* BigInt */ && includes & 2048 /* BigIntLiteral */ ||
                    includes & 4096 /* ESSymbol */ && includes & 8192 /* UniqueESSymbol */) {
                    removeRedundantPrimitiveTypes(typeSet, includes);
                }
                if (includes & 8388608 /* IncludesEmptyObject */ && includes & 524288 /* Object */) {
                    ts.orderedRemoveItemAt(typeSet, ts.findIndex(typeSet, isEmptyAnonymousObjectType));
                }
                if (typeSet.length === 0) {
                    return unknownType;
                }
                if (typeSet.length === 1) {
                    return typeSet[0];
                }
                var id = getTypeListId(typeSet);
                var result = intersectionTypes.get(id);
                if (!result) {
                    if (includes & 1048576 /* Union */) {
                        if (intersectUnionsOfPrimitiveTypes(typeSet)) {
                            // When the intersection creates a reduced set (which might mean that *all* union types have
                            // disappeared), we restart the operation to get a new set of combined flags. Once we have
                            // reduced we'll never reduce again, so this occurs at most once.
                            result = getIntersectionType(typeSet, aliasSymbol, aliasTypeArguments);
                        }
                        else if (extractIrreducible(typeSet, 32768 /* Undefined */)) {
                            result = getUnionType([getIntersectionType(typeSet), undefinedType], 1 /* Literal */, aliasSymbol, aliasTypeArguments);
                        }
                        else if (extractIrreducible(typeSet, 65536 /* Null */)) {
                            result = getUnionType([getIntersectionType(typeSet), nullType], 1 /* Literal */, aliasSymbol, aliasTypeArguments);
                        }
                        else {
                            // We are attempting to construct a type of the form X & (A | B) & Y. Transform this into a type of
                            // the form X & A & Y | X & B & Y and recursively reduce until no union type constituents remain.
                            // If the estimated size of the resulting union type exceeds 100000 constituents, report an error.
                            var size = ts.reduceLeft(typeSet, function (n, t) { return n * (t.flags & 1048576 /* Union */ ? t.types.length : 1); }, 1);
                            if (size >= 100000) {
                                error(currentNode, ts.Diagnostics.Expression_produces_a_union_type_that_is_too_complex_to_represent);
                                return errorType;
                            }
                            var unionIndex_1 = ts.findIndex(typeSet, function (t) { return (t.flags & 1048576 /* Union */) !== 0; });
                            var unionType = typeSet[unionIndex_1];
                            result = getUnionType(ts.map(unionType.types, function (t) { return getIntersectionType(ts.replaceElement(typeSet, unionIndex_1, t)); }), 1 /* Literal */, aliasSymbol, aliasTypeArguments);
                        }
                    }
                    else {
                        result = createIntersectionType(typeSet, aliasSymbol, aliasTypeArguments);
                    }
                    intersectionTypes.set(id, result);
                }
                return result;
            }
            function getTypeFromIntersectionTypeNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    var aliasSymbol = getAliasSymbolForTypeNode(node);
                    links.resolvedType = getIntersectionType(ts.map(node.types, getTypeFromTypeNode), aliasSymbol, getTypeArgumentsForAliasSymbol(aliasSymbol));
                }
                return links.resolvedType;
            }
            function createIndexType(type, stringsOnly) {
                var result = createType(4194304 /* Index */);
                result.type = type;
                result.stringsOnly = stringsOnly;
                return result;
            }
            function getIndexTypeForGenericType(type, stringsOnly) {
                return stringsOnly ?
                    type.resolvedStringIndexType || (type.resolvedStringIndexType = createIndexType(type, /*stringsOnly*/ true)) :
                    type.resolvedIndexType || (type.resolvedIndexType = createIndexType(type, /*stringsOnly*/ false));
            }
            function getLiteralTypeFromPropertyName(name) {
                if (ts.isPrivateIdentifier(name)) {
                    return neverType;
                }
                return ts.isIdentifier(name) ? getLiteralType(ts.unescapeLeadingUnderscores(name.escapedText)) :
                    getRegularTypeOfLiteralType(ts.isComputedPropertyName(name) ? checkComputedPropertyName(name) : checkExpression(name));
            }
            function getBigIntLiteralType(node) {
                return getLiteralType({
                    negative: false,
                    base10Value: ts.parsePseudoBigInt(node.text)
                });
            }
            function getLiteralTypeFromProperty(prop, include) {
                if (!(ts.getDeclarationModifierFlagsFromSymbol(prop) & 24 /* NonPublicAccessibilityModifier */)) {
                    var type = getSymbolLinks(getLateBoundSymbol(prop)).nameType;
                    if (!type && !ts.isKnownSymbol(prop)) {
                        if (prop.escapedName === "default" /* Default */) {
                            type = getLiteralType("default");
                        }
                        else {
                            var name = prop.valueDeclaration && ts.getNameOfDeclaration(prop.valueDeclaration);
                            type = name && getLiteralTypeFromPropertyName(name) || getLiteralType(ts.symbolName(prop));
                        }
                    }
                    if (type && type.flags & include) {
                        return type;
                    }
                }
                return neverType;
            }
            function getLiteralTypeFromProperties(type, include) {
                return getUnionType(ts.map(getPropertiesOfType(type), function (p) { return getLiteralTypeFromProperty(p, include); }));
            }
            function getNonEnumNumberIndexInfo(type) {
                var numberIndexInfo = getIndexInfoOfType(type, 1 /* Number */);
                return numberIndexInfo !== enumNumberIndexInfo ? numberIndexInfo : undefined;
            }
            function getIndexType(type, stringsOnly, noIndexSignatures) {
                if (stringsOnly === void 0) {
                    stringsOnly = keyofStringsOnly;
                }
                return type.flags & 1048576 /* Union */ ? getIntersectionType(ts.map(type.types, function (t) { return getIndexType(t, stringsOnly, noIndexSignatures); })) :
                    type.flags & 2097152 /* Intersection */ ? getUnionType(ts.map(type.types, function (t) { return getIndexType(t, stringsOnly, noIndexSignatures); })) :
                        maybeTypeOfKind(type, 58982400 /* InstantiableNonPrimitive */) ? getIndexTypeForGenericType(type, stringsOnly) :
                            ts.getObjectFlags(type) & 32 /* Mapped */ ? filterType(getConstraintTypeFromMappedType(type), function (t) { return !(noIndexSignatures && t.flags & (1 /* Any */ | 4 /* String */)); }) :
                                type === wildcardType ? wildcardType :
                                    type.flags & 2 /* Unknown */ ? neverType :
                                        type.flags & (1 /* Any */ | 131072 /* Never */) ? keyofConstraintType :
                                            stringsOnly ? !noIndexSignatures && getIndexInfoOfType(type, 0 /* String */) ? stringType : getLiteralTypeFromProperties(type, 128 /* StringLiteral */) :
                                                !noIndexSignatures && getIndexInfoOfType(type, 0 /* String */) ? getUnionType([stringType, numberType, getLiteralTypeFromProperties(type, 8192 /* UniqueESSymbol */)]) :
                                                    getNonEnumNumberIndexInfo(type) ? getUnionType([numberType, getLiteralTypeFromProperties(type, 128 /* StringLiteral */ | 8192 /* UniqueESSymbol */)]) :
                                                        getLiteralTypeFromProperties(type, 8576 /* StringOrNumberLiteralOrUnique */);
            }
            function getExtractStringType(type) {
                if (keyofStringsOnly) {
                    return type;
                }
                var extractTypeAlias = getGlobalExtractSymbol();
                return extractTypeAlias ? getTypeAliasInstantiation(extractTypeAlias, [type, stringType]) : stringType;
            }
            function getIndexTypeOrString(type) {
                var indexType = getExtractStringType(getIndexType(type));
                return indexType.flags & 131072 /* Never */ ? stringType : indexType;
            }
            function getTypeFromTypeOperatorNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    switch (node.operator) {
                        case 134 /* KeyOfKeyword */:
                            links.resolvedType = getIndexType(getTypeFromTypeNode(node.type));
                            break;
                        case 147 /* UniqueKeyword */:
                            links.resolvedType = node.type.kind === 144 /* SymbolKeyword */
                                ? getESSymbolLikeTypeForNode(ts.walkUpParenthesizedTypes(node.parent))
                                : errorType;
                            break;
                        case 138 /* ReadonlyKeyword */:
                            links.resolvedType = getTypeFromTypeNode(node.type);
                            break;
                        default:
                            throw ts.Debug.assertNever(node.operator);
                    }
                }
                return links.resolvedType;
            }
            function createIndexedAccessType(objectType, indexType) {
                var type = createType(8388608 /* IndexedAccess */);
                type.objectType = objectType;
                type.indexType = indexType;
                return type;
            }
            /**
             * Returns if a type is or consists of a JSLiteral object type
             * In addition to objects which are directly literals,
             * * unions where every element is a jsliteral
             * * intersections where at least one element is a jsliteral
             * * and instantiable types constrained to a jsliteral
             * Should all count as literals and not print errors on access or assignment of possibly existing properties.
             * This mirrors the behavior of the index signature propagation, to which this behaves similarly (but doesn't affect assignability or inference).
             */
            function isJSLiteralType(type) {
                if (noImplicitAny) {
                    return false; // Flag is meaningless under `noImplicitAny` mode
                }
                if (ts.getObjectFlags(type) & 16384 /* JSLiteral */) {
                    return true;
                }
                if (type.flags & 1048576 /* Union */) {
                    return ts.every(type.types, isJSLiteralType);
                }
                if (type.flags & 2097152 /* Intersection */) {
                    return ts.some(type.types, isJSLiteralType);
                }
                if (type.flags & 63176704 /* Instantiable */) {
                    return isJSLiteralType(getResolvedBaseConstraint(type));
                }
                return false;
            }
            function getPropertyNameFromIndex(indexType, accessNode) {
                var accessExpression = accessNode && accessNode.kind === 195 /* ElementAccessExpression */ ? accessNode : undefined;
                return isTypeUsableAsPropertyName(indexType) ?
                    getPropertyNameFromType(indexType) :
                    accessExpression && checkThatExpressionIsProperSymbolReference(accessExpression.argumentExpression, indexType, /*reportError*/ false) ?
                        ts.getPropertyNameForKnownSymbolName(ts.idText(accessExpression.argumentExpression.name)) :
                        accessNode && ts.isPropertyName(accessNode) ?
                            // late bound names are handled in the first branch, so here we only need to handle normal names
                            ts.getPropertyNameForPropertyNameNode(accessNode) :
                            undefined;
            }
            function getPropertyTypeForIndexType(originalObjectType, objectType, indexType, fullIndexType, suppressNoImplicitAnyError, accessNode, accessFlags) {
                var accessExpression = accessNode && accessNode.kind === 195 /* ElementAccessExpression */ ? accessNode : undefined;
                var propName = accessNode && ts.isPrivateIdentifier(accessNode) ? undefined : getPropertyNameFromIndex(indexType, accessNode);
                if (propName !== undefined) {
                    var prop = getPropertyOfType(objectType, propName);
                    if (prop) {
                        if (accessExpression) {
                            markPropertyAsReferenced(prop, accessExpression, /*isThisAccess*/ accessExpression.expression.kind === 104 /* ThisKeyword */);
                            if (isAssignmentToReadonlyEntity(accessExpression, prop, ts.getAssignmentTargetKind(accessExpression))) {
                                error(accessExpression.argumentExpression, ts.Diagnostics.Cannot_assign_to_0_because_it_is_a_read_only_property, symbolToString(prop));
                                return undefined;
                            }
                            if (accessFlags & 4 /* CacheSymbol */) {
                                getNodeLinks(accessNode).resolvedSymbol = prop;
                            }
                        }
                        var propType = getTypeOfSymbol(prop);
                        return accessExpression && ts.getAssignmentTargetKind(accessExpression) !== 1 /* Definite */ ?
                            getFlowTypeOfReference(accessExpression, propType) :
                            propType;
                    }
                    if (everyType(objectType, isTupleType) && isNumericLiteralName(propName) && +propName >= 0) {
                        if (accessNode && everyType(objectType, function (t) { return !t.target.hasRestElement; }) && !(accessFlags & 8 /* NoTupleBoundsCheck */)) {
                            var indexNode = getIndexNodeForAccessExpression(accessNode);
                            if (isTupleType(objectType)) {
                                error(indexNode, ts.Diagnostics.Tuple_type_0_of_length_1_has_no_element_at_index_2, typeToString(objectType), getTypeReferenceArity(objectType), ts.unescapeLeadingUnderscores(propName));
                            }
                            else {
                                error(indexNode, ts.Diagnostics.Property_0_does_not_exist_on_type_1, ts.unescapeLeadingUnderscores(propName), typeToString(objectType));
                            }
                        }
                        errorIfWritingToReadonlyIndex(getIndexInfoOfType(objectType, 1 /* Number */));
                        return mapType(objectType, function (t) { return getRestTypeOfTupleType(t) || undefinedType; });
                    }
                }
                if (!(indexType.flags & 98304 /* Nullable */) && isTypeAssignableToKind(indexType, 132 /* StringLike */ | 296 /* NumberLike */ | 12288 /* ESSymbolLike */)) {
                    if (objectType.flags & (1 /* Any */ | 131072 /* Never */)) {
                        return objectType;
                    }
                    var stringIndexInfo = getIndexInfoOfType(objectType, 0 /* String */);
                    var indexInfo = isTypeAssignableToKind(indexType, 296 /* NumberLike */) && getIndexInfoOfType(objectType, 1 /* Number */) || stringIndexInfo;
                    if (indexInfo) {
                        if (accessFlags & 1 /* NoIndexSignatures */ && indexInfo === stringIndexInfo) {
                            if (accessExpression) {
                                error(accessExpression, ts.Diagnostics.Type_0_cannot_be_used_to_index_type_1, typeToString(indexType), typeToString(originalObjectType));
                            }
                            return undefined;
                        }
                        if (accessNode && !isTypeAssignableToKind(indexType, 4 /* String */ | 8 /* Number */)) {
                            var indexNode = getIndexNodeForAccessExpression(accessNode);
                            error(indexNode, ts.Diagnostics.Type_0_cannot_be_used_as_an_index_type, typeToString(indexType));
                            return indexInfo.type;
                        }
                        errorIfWritingToReadonlyIndex(indexInfo);
                        return indexInfo.type;
                    }
                    if (indexType.flags & 131072 /* Never */) {
                        return neverType;
                    }
                    if (isJSLiteralType(objectType)) {
                        return anyType;
                    }
                    if (accessExpression && !isConstEnumObjectType(objectType)) {
                        if (objectType.symbol === globalThisSymbol && propName !== undefined && globalThisSymbol.exports.has(propName) && (globalThisSymbol.exports.get(propName).flags & 418 /* BlockScoped */)) {
                            error(accessExpression, ts.Diagnostics.Property_0_does_not_exist_on_type_1, ts.unescapeLeadingUnderscores(propName), typeToString(objectType));
                        }
                        else if (noImplicitAny && !compilerOptions.suppressImplicitAnyIndexErrors && !suppressNoImplicitAnyError) {
                            if (propName !== undefined && typeHasStaticProperty(propName, objectType)) {
                                error(accessExpression, ts.Diagnostics.Property_0_is_a_static_member_of_type_1, propName, typeToString(objectType));
                            }
                            else if (getIndexTypeOfType(objectType, 1 /* Number */)) {
                                error(accessExpression.argumentExpression, ts.Diagnostics.Element_implicitly_has_an_any_type_because_index_expression_is_not_of_type_number);
                            }
                            else {
                                var suggestion = void 0;
                                if (propName !== undefined && (suggestion = getSuggestionForNonexistentProperty(propName, objectType))) {
                                    if (suggestion !== undefined) {
                                        error(accessExpression.argumentExpression, ts.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2, propName, typeToString(objectType), suggestion);
                                    }
                                }
                                else {
                                    var suggestion_1 = getSuggestionForNonexistentIndexSignature(objectType, accessExpression, indexType);
                                    if (suggestion_1 !== undefined) {
                                        error(accessExpression, ts.Diagnostics.Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature_Did_you_mean_to_call_1, typeToString(objectType), suggestion_1);
                                    }
                                    else {
                                        var errorInfo = void 0;
                                        if (indexType.flags & 1024 /* EnumLiteral */) {
                                            errorInfo = ts.chainDiagnosticMessages(/* details */ undefined, ts.Diagnostics.Property_0_does_not_exist_on_type_1, "[" + typeToString(indexType) + "]", typeToString(objectType));
                                        }
                                        else if (indexType.flags & 8192 /* UniqueESSymbol */) {
                                            var symbolName_2 = getFullyQualifiedName(indexType.symbol, accessExpression);
                                            errorInfo = ts.chainDiagnosticMessages(/* details */ undefined, ts.Diagnostics.Property_0_does_not_exist_on_type_1, "[" + symbolName_2 + "]", typeToString(objectType));
                                        }
                                        else if (indexType.flags & 128 /* StringLiteral */) {
                                            errorInfo = ts.chainDiagnosticMessages(/* details */ undefined, ts.Diagnostics.Property_0_does_not_exist_on_type_1, indexType.value, typeToString(objectType));
                                        }
                                        else if (indexType.flags & 256 /* NumberLiteral */) {
                                            errorInfo = ts.chainDiagnosticMessages(/* details */ undefined, ts.Diagnostics.Property_0_does_not_exist_on_type_1, indexType.value, typeToString(objectType));
                                        }
                                        else if (indexType.flags & (8 /* Number */ | 4 /* String */)) {
                                            errorInfo = ts.chainDiagnosticMessages(/* details */ undefined, ts.Diagnostics.No_index_signature_with_a_parameter_of_type_0_was_found_on_type_1, typeToString(indexType), typeToString(objectType));
                                        }
                                        errorInfo = ts.chainDiagnosticMessages(errorInfo, ts.Diagnostics.Element_implicitly_has_an_any_type_because_expression_of_type_0_can_t_be_used_to_index_type_1, typeToString(fullIndexType), typeToString(objectType));
                                        diagnostics.add(ts.createDiagnosticForNodeFromMessageChain(accessExpression, errorInfo));
                                    }
                                }
                            }
                        }
                        return undefined;
                    }
                }
                if (isJSLiteralType(objectType)) {
                    return anyType;
                }
                if (accessNode) {
                    var indexNode = getIndexNodeForAccessExpression(accessNode);
                    if (indexType.flags & (128 /* StringLiteral */ | 256 /* NumberLiteral */)) {
                        error(indexNode, ts.Diagnostics.Property_0_does_not_exist_on_type_1, "" + indexType.value, typeToString(objectType));
                    }
                    else if (indexType.flags & (4 /* String */ | 8 /* Number */)) {
                        error(indexNode, ts.Diagnostics.Type_0_has_no_matching_index_signature_for_type_1, typeToString(objectType), typeToString(indexType));
                    }
                    else {
                        error(indexNode, ts.Diagnostics.Type_0_cannot_be_used_as_an_index_type, typeToString(indexType));
                    }
                }
                if (isTypeAny(indexType)) {
                    return indexType;
                }
                return undefined;
                function errorIfWritingToReadonlyIndex(indexInfo) {
                    if (indexInfo && indexInfo.isReadonly && accessExpression && (ts.isAssignmentTarget(accessExpression) || ts.isDeleteTarget(accessExpression))) {
                        error(accessExpression, ts.Diagnostics.Index_signature_in_type_0_only_permits_reading, typeToString(objectType));
                    }
                }
            }
            function getIndexNodeForAccessExpression(accessNode) {
                return accessNode.kind === 195 /* ElementAccessExpression */ ? accessNode.argumentExpression :
                    accessNode.kind === 185 /* IndexedAccessType */ ? accessNode.indexType :
                        accessNode.kind === 154 /* ComputedPropertyName */ ? accessNode.expression :
                            accessNode;
            }
            function isGenericObjectType(type) {
                return maybeTypeOfKind(type, 58982400 /* InstantiableNonPrimitive */ | 131072 /* GenericMappedType */);
            }
            function isGenericIndexType(type) {
                return maybeTypeOfKind(type, 58982400 /* InstantiableNonPrimitive */ | 4194304 /* Index */);
            }
            function isThisTypeParameter(type) {
                return !!(type.flags & 262144 /* TypeParameter */ && type.isThisType);
            }
            function getSimplifiedType(type, writing) {
                return type.flags & 8388608 /* IndexedAccess */ ? getSimplifiedIndexedAccessType(type, writing) :
                    type.flags & 16777216 /* Conditional */ ? getSimplifiedConditionalType(type, writing) :
                        type;
            }
            function distributeIndexOverObjectType(objectType, indexType, writing) {
                // (T | U)[K] -> T[K] | U[K] (reading)
                // (T | U)[K] -> T[K] & U[K] (writing)
                // (T & U)[K] -> T[K] & U[K]
                if (objectType.flags & 3145728 /* UnionOrIntersection */) {
                    var types = ts.map(objectType.types, function (t) { return getSimplifiedType(getIndexedAccessType(t, indexType), writing); });
                    return objectType.flags & 2097152 /* Intersection */ || writing ? getIntersectionType(types) : getUnionType(types);
                }
            }
            function distributeObjectOverIndexType(objectType, indexType, writing) {
                // T[A | B] -> T[A] | T[B] (reading)
                // T[A | B] -> T[A] & T[B] (writing)
                if (indexType.flags & 1048576 /* Union */) {
                    var types = ts.map(indexType.types, function (t) { return getSimplifiedType(getIndexedAccessType(objectType, t), writing); });
                    return writing ? getIntersectionType(types) : getUnionType(types);
                }
            }
            function unwrapSubstitution(type) {
                if (type.flags & 33554432 /* Substitution */) {
                    return type.substitute;
                }
                return type;
            }
            // Transform an indexed access to a simpler form, if possible. Return the simpler form, or return
            // the type itself if no transformation is possible. The writing flag indicates that the type is
            // the target of an assignment.
            function getSimplifiedIndexedAccessType(type, writing) {
                var cache = writing ? "simplifiedForWriting" : "simplifiedForReading";
                if (type[cache]) {
                    return type[cache] === circularConstraintType ? type : type[cache];
                }
                type[cache] = circularConstraintType;
                // We recursively simplify the object type as it may in turn be an indexed access type. For example, with
                // '{ [P in T]: { [Q in U]: number } }[T][U]' we want to first simplify the inner indexed access type.
                var objectType = unwrapSubstitution(getSimplifiedType(type.objectType, writing));
                var indexType = getSimplifiedType(type.indexType, writing);
                // T[A | B] -> T[A] | T[B] (reading)
                // T[A | B] -> T[A] & T[B] (writing)
                var distributedOverIndex = distributeObjectOverIndexType(objectType, indexType, writing);
                if (distributedOverIndex) {
                    return type[cache] = distributedOverIndex;
                }
                // Only do the inner distributions if the index can no longer be instantiated to cause index distribution again
                if (!(indexType.flags & 63176704 /* Instantiable */)) {
                    // (T | U)[K] -> T[K] | U[K] (reading)
                    // (T | U)[K] -> T[K] & U[K] (writing)
                    // (T & U)[K] -> T[K] & U[K]
                    var distributedOverObject = distributeIndexOverObjectType(objectType, indexType, writing);
                    if (distributedOverObject) {
                        return type[cache] = distributedOverObject;
                    }
                }
                // So ultimately (reading):
                // ((A & B) | C)[K1 | K2] -> ((A & B) | C)[K1] | ((A & B) | C)[K2] -> (A & B)[K1] | C[K1] | (A & B)[K2] | C[K2] -> (A[K1] & B[K1]) | C[K1] | (A[K2] & B[K2]) | C[K2]
                // If the object type is a mapped type { [P in K]: E }, where K is generic, instantiate E using a mapper
                // that substitutes the index type for P. For example, for an index access { [P in K]: Box<T[P]> }[X], we
                // construct the type Box<T[X]>.
                if (isGenericMappedType(objectType)) {
                    return type[cache] = mapType(substituteIndexedMappedType(objectType, type.indexType), function (t) { return getSimplifiedType(t, writing); });
                }
                return type[cache] = type;
            }
            function getSimplifiedConditionalType(type, writing) {
                var checkType = type.checkType;
                var extendsType = type.extendsType;
                var trueType = getTrueTypeFromConditionalType(type);
                var falseType = getFalseTypeFromConditionalType(type);
                // Simplifications for types of the form `T extends U ? T : never` and `T extends U ? never : T`.
                if (falseType.flags & 131072 /* Never */ && getActualTypeVariable(trueType) === getActualTypeVariable(checkType)) {
                    if (checkType.flags & 1 /* Any */ || isTypeAssignableTo(getRestrictiveInstantiation(checkType), getRestrictiveInstantiation(extendsType))) { // Always true
                        return getSimplifiedType(trueType, writing);
                    }
                    else if (isIntersectionEmpty(checkType, extendsType)) { // Always false
                        return neverType;
                    }
                }
                else if (trueType.flags & 131072 /* Never */ && getActualTypeVariable(falseType) === getActualTypeVariable(checkType)) {
                    if (!(checkType.flags & 1 /* Any */) && isTypeAssignableTo(getRestrictiveInstantiation(checkType), getRestrictiveInstantiation(extendsType))) { // Always true
                        return neverType;
                    }
                    else if (checkType.flags & 1 /* Any */ || isIntersectionEmpty(checkType, extendsType)) { // Always false
                        return getSimplifiedType(falseType, writing);
                    }
                }
                return type;
            }
            /**
             * Invokes union simplification logic to determine if an intersection is considered empty as a union constituent
             */
            function isIntersectionEmpty(type1, type2) {
                return !!(getUnionType([intersectTypes(type1, type2), neverType]).flags & 131072 /* Never */);
            }
            function substituteIndexedMappedType(objectType, index) {
                var mapper = createTypeMapper([getTypeParameterFromMappedType(objectType)], [index]);
                var templateMapper = combineTypeMappers(objectType.mapper, mapper);
                return instantiateType(getTemplateTypeFromMappedType(objectType), templateMapper);
            }
            function getIndexedAccessType(objectType, indexType, accessNode) {
                return getIndexedAccessTypeOrUndefined(objectType, indexType, accessNode, 0 /* None */) || (accessNode ? errorType : unknownType);
            }
            function getIndexedAccessTypeOrUndefined(objectType, indexType, accessNode, accessFlags) {
                if (accessFlags === void 0) {
                    accessFlags = 0 /* None */;
                }
                if (objectType === wildcardType || indexType === wildcardType) {
                    return wildcardType;
                }
                // If the object type has a string index signature and no other members we know that the result will
                // always be the type of that index signature and we can simplify accordingly.
                if (isStringIndexSignatureOnlyType(objectType) && !(indexType.flags & 98304 /* Nullable */) && isTypeAssignableToKind(indexType, 4 /* String */ | 8 /* Number */)) {
                    indexType = stringType;
                }
                // If the index type is generic, or if the object type is generic and doesn't originate in an expression,
                // we are performing a higher-order index access where we cannot meaningfully access the properties of the
                // object type. Note that for a generic T and a non-generic K, we eagerly resolve T[K] if it originates in
                // an expression. This is to preserve backwards compatibility. For example, an element access 'this["foo"]'
                // has always been resolved eagerly using the constraint type of 'this' at the given location.
                if (isGenericIndexType(indexType) || !(accessNode && accessNode.kind !== 185 /* IndexedAccessType */) && isGenericObjectType(objectType)) {
                    if (objectType.flags & 3 /* AnyOrUnknown */) {
                        return objectType;
                    }
                    // Defer the operation by creating an indexed access type.
                    var id = objectType.id + "," + indexType.id;
                    var type = indexedAccessTypes.get(id);
                    if (!type) {
                        indexedAccessTypes.set(id, type = createIndexedAccessType(objectType, indexType));
                    }
                    return type;
                }
                // In the following we resolve T[K] to the type of the property in T selected by K.
                // We treat boolean as different from other unions to improve errors;
                // skipping straight to getPropertyTypeForIndexType gives errors with 'boolean' instead of 'true'.
                var apparentObjectType = getApparentType(objectType);
                if (indexType.flags & 1048576 /* Union */ && !(indexType.flags & 16 /* Boolean */)) {
                    var propTypes = [];
                    var wasMissingProp = false;
                    for (var _i = 0, _a = indexType.types; _i < _a.length; _i++) {
                        var t = _a[_i];
                        var propType = getPropertyTypeForIndexType(objectType, apparentObjectType, t, indexType, wasMissingProp, accessNode, accessFlags);
                        if (propType) {
                            propTypes.push(propType);
                        }
                        else if (!accessNode) {
                            // If there's no error node, we can immeditely stop, since error reporting is off
                            return undefined;
                        }
                        else {
                            // Otherwise we set a flag and return at the end of the loop so we still mark all errors
                            wasMissingProp = true;
                        }
                    }
                    if (wasMissingProp) {
                        return undefined;
                    }
                    return accessFlags & 2 /* Writing */ ? getIntersectionType(propTypes) : getUnionType(propTypes);
                }
                return getPropertyTypeForIndexType(objectType, apparentObjectType, indexType, indexType, /* supressNoImplicitAnyError */ false, accessNode, accessFlags | 4 /* CacheSymbol */);
            }
            function getTypeFromIndexedAccessTypeNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    var objectType = getTypeFromTypeNode(node.objectType);
                    var indexType = getTypeFromTypeNode(node.indexType);
                    var resolved = getIndexedAccessType(objectType, indexType, node);
                    links.resolvedType = resolved.flags & 8388608 /* IndexedAccess */ &&
                        resolved.objectType === objectType &&
                        resolved.indexType === indexType ?
                        getConstrainedTypeVariable(resolved, node) : resolved;
                }
                return links.resolvedType;
            }
            function getTypeFromMappedTypeNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    var type = createObjectType(32 /* Mapped */, node.symbol);
                    type.declaration = node;
                    type.aliasSymbol = getAliasSymbolForTypeNode(node);
                    type.aliasTypeArguments = getTypeArgumentsForAliasSymbol(type.aliasSymbol);
                    links.resolvedType = type;
                    // Eagerly resolve the constraint type which forces an error if the constraint type circularly
                    // references itself through one or more type aliases.
                    getConstraintTypeFromMappedType(type);
                }
                return links.resolvedType;
            }
            function getActualTypeVariable(type) {
                if (type.flags & 33554432 /* Substitution */) {
                    return type.typeVariable;
                }
                if (type.flags & 8388608 /* IndexedAccess */ && (type.objectType.flags & 33554432 /* Substitution */ ||
                    type.indexType.flags & 33554432 /* Substitution */)) {
                    return getIndexedAccessType(getActualTypeVariable(type.objectType), getActualTypeVariable(type.indexType));
                }
                return type;
            }
            function getConditionalType(root, mapper) {
                var checkType = instantiateType(root.checkType, mapper);
                var extendsType = instantiateType(root.extendsType, mapper);
                if (checkType === wildcardType || extendsType === wildcardType) {
                    return wildcardType;
                }
                var checkTypeInstantiable = maybeTypeOfKind(checkType, 63176704 /* Instantiable */ | 131072 /* GenericMappedType */);
                var combinedMapper;
                if (root.inferTypeParameters) {
                    var context = createInferenceContext(root.inferTypeParameters, /*signature*/ undefined, 0 /* None */);
                    // We skip inference of the possible `infer` types unles the `extendsType` _is_ an infer type
                    // if it was, it's trivial to say that extendsType = checkType, however such a pattern is used to
                    // "reset" the type being build up during constraint calculation and avoid making an apparently "infinite" constraint
                    // so in those cases we refain from performing inference and retain the uninfered type parameter
                    if (!checkTypeInstantiable || !ts.some(root.inferTypeParameters, function (t) { return t === extendsType; })) {
                        // We don't want inferences from constraints as they may cause us to eagerly resolve the
                        // conditional type instead of deferring resolution. Also, we always want strict function
                        // types rules (i.e. proper contravariance) for inferences.
                        inferTypes(context.inferences, checkType, extendsType, 128 /* NoConstraints */ | 256 /* AlwaysStrict */);
                    }
                    combinedMapper = combineTypeMappers(mapper, context.mapper);
                }
                // Instantiate the extends type including inferences for 'infer T' type parameters
                var inferredExtendsType = combinedMapper ? instantiateType(root.extendsType, combinedMapper) : extendsType;
                // We attempt to resolve the conditional type only when the check and extends types are non-generic
                if (!checkTypeInstantiable && !maybeTypeOfKind(inferredExtendsType, 63176704 /* Instantiable */ | 131072 /* GenericMappedType */)) {
                    if (inferredExtendsType.flags & 3 /* AnyOrUnknown */) {
                        return instantiateType(root.trueType, combinedMapper || mapper);
                    }
                    // Return union of trueType and falseType for 'any' since it matches anything
                    if (checkType.flags & 1 /* Any */) {
                        return getUnionType([instantiateType(root.trueType, combinedMapper || mapper), instantiateType(root.falseType, mapper)]);
                    }
                    // Return falseType for a definitely false extends check. We check an instantiations of the two
                    // types with type parameters mapped to the wildcard type, the most permissive instantiations
                    // possible (the wildcard type is assignable to and from all types). If those are not related,
                    // then no instantiations will be and we can just return the false branch type.
                    if (!isTypeAssignableTo(getPermissiveInstantiation(checkType), getPermissiveInstantiation(inferredExtendsType))) {
                        return instantiateType(root.falseType, mapper);
                    }
                    // Return trueType for a definitely true extends check. We check instantiations of the two
                    // types with type parameters mapped to their restrictive form, i.e. a form of the type parameter
                    // that has no constraint. This ensures that, for example, the type
                    //   type Foo<T extends { x: any }> = T extends { x: string } ? string : number
                    // doesn't immediately resolve to 'string' instead of being deferred.
                    if (isTypeAssignableTo(getRestrictiveInstantiation(checkType), getRestrictiveInstantiation(inferredExtendsType))) {
                        return instantiateType(root.trueType, combinedMapper || mapper);
                    }
                }
                // Return a deferred type for a check that is neither definitely true nor definitely false
                var erasedCheckType = getActualTypeVariable(checkType);
                var result = createType(16777216 /* Conditional */);
                result.root = root;
                result.checkType = erasedCheckType;
                result.extendsType = extendsType;
                result.mapper = mapper;
                result.combinedMapper = combinedMapper;
                result.aliasSymbol = root.aliasSymbol;
                result.aliasTypeArguments = instantiateTypes(root.aliasTypeArguments, mapper); // TODO: GH#18217
                return result;
            }
            function getTrueTypeFromConditionalType(type) {
                return type.resolvedTrueType || (type.resolvedTrueType = instantiateType(type.root.trueType, type.mapper));
            }
            function getFalseTypeFromConditionalType(type) {
                return type.resolvedFalseType || (type.resolvedFalseType = instantiateType(type.root.falseType, type.mapper));
            }
            function getInferredTrueTypeFromConditionalType(type) {
                return type.resolvedInferredTrueType || (type.resolvedInferredTrueType = type.combinedMapper ? instantiateType(type.root.trueType, type.combinedMapper) : getTrueTypeFromConditionalType(type));
            }
            function getInferTypeParameters(node) {
                var result;
                if (node.locals) {
                    node.locals.forEach(function (symbol) {
                        if (symbol.flags & 262144 /* TypeParameter */) {
                            result = ts.append(result, getDeclaredTypeOfSymbol(symbol));
                        }
                    });
                }
                return result;
            }
            function getTypeFromConditionalTypeNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    var checkType = getTypeFromTypeNode(node.checkType);
                    var aliasSymbol = getAliasSymbolForTypeNode(node);
                    var aliasTypeArguments = getTypeArgumentsForAliasSymbol(aliasSymbol);
                    var allOuterTypeParameters = getOuterTypeParameters(node, /*includeThisTypes*/ true);
                    var outerTypeParameters = aliasTypeArguments ? allOuterTypeParameters : ts.filter(allOuterTypeParameters, function (tp) { return isTypeParameterPossiblyReferenced(tp, node); });
                    var root = {
                        node: node,
                        checkType: checkType,
                        extendsType: getTypeFromTypeNode(node.extendsType),
                        trueType: getTypeFromTypeNode(node.trueType),
                        falseType: getTypeFromTypeNode(node.falseType),
                        isDistributive: !!(checkType.flags & 262144 /* TypeParameter */),
                        inferTypeParameters: getInferTypeParameters(node),
                        outerTypeParameters: outerTypeParameters,
                        instantiations: undefined,
                        aliasSymbol: aliasSymbol,
                        aliasTypeArguments: aliasTypeArguments
                    };
                    links.resolvedType = getConditionalType(root, /*mapper*/ undefined);
                    if (outerTypeParameters) {
                        root.instantiations = ts.createMap();
                        root.instantiations.set(getTypeListId(outerTypeParameters), links.resolvedType);
                    }
                }
                return links.resolvedType;
            }
            function getTypeFromInferTypeNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    links.resolvedType = getDeclaredTypeOfTypeParameter(getSymbolOfNode(node.typeParameter));
                }
                return links.resolvedType;
            }
            function getIdentifierChain(node) {
                if (ts.isIdentifier(node)) {
                    return [node];
                }
                else {
                    return ts.append(getIdentifierChain(node.left), node.right);
                }
            }
            function getTypeFromImportTypeNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    if (node.isTypeOf && node.typeArguments) { // Only the non-typeof form can make use of type arguments
                        error(node, ts.Diagnostics.Type_arguments_cannot_be_used_here);
                        links.resolvedSymbol = unknownSymbol;
                        return links.resolvedType = errorType;
                    }
                    if (!ts.isLiteralImportTypeNode(node)) {
                        error(node.argument, ts.Diagnostics.String_literal_expected);
                        links.resolvedSymbol = unknownSymbol;
                        return links.resolvedType = errorType;
                    }
                    var targetMeaning = node.isTypeOf ? 111551 /* Value */ : node.flags & 4194304 /* JSDoc */ ? 111551 /* Value */ | 788968 /* Type */ : 788968 /* Type */;
                    // TODO: Future work: support unions/generics/whatever via a deferred import-type
                    var innerModuleSymbol = resolveExternalModuleName(node, node.argument.literal);
                    if (!innerModuleSymbol) {
                        links.resolvedSymbol = unknownSymbol;
                        return links.resolvedType = errorType;
                    }
                    var moduleSymbol = resolveExternalModuleSymbol(innerModuleSymbol, /*dontResolveAlias*/ false);
                    if (!ts.nodeIsMissing(node.qualifier)) {
                        var nameStack = getIdentifierChain(node.qualifier);
                        var currentNamespace = moduleSymbol;
                        var current = void 0;
                        while (current = nameStack.shift()) {
                            var meaning = nameStack.length ? 1920 /* Namespace */ : targetMeaning;
                            var next = getSymbol(getExportsOfSymbol(getMergedSymbol(resolveSymbol(currentNamespace))), current.escapedText, meaning);
                            if (!next) {
                                error(current, ts.Diagnostics.Namespace_0_has_no_exported_member_1, getFullyQualifiedName(currentNamespace), ts.declarationNameToString(current));
                                return links.resolvedType = errorType;
                            }
                            getNodeLinks(current).resolvedSymbol = next;
                            getNodeLinks(current.parent).resolvedSymbol = next;
                            currentNamespace = next;
                        }
                        links.resolvedType = resolveImportSymbolType(node, links, currentNamespace, targetMeaning);
                    }
                    else {
                        if (moduleSymbol.flags & targetMeaning) {
                            links.resolvedType = resolveImportSymbolType(node, links, moduleSymbol, targetMeaning);
                        }
                        else {
                            var errorMessage = targetMeaning === 111551 /* Value */
                                ? ts.Diagnostics.Module_0_does_not_refer_to_a_value_but_is_used_as_a_value_here
                                : ts.Diagnostics.Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0;
                            error(node, errorMessage, node.argument.literal.text);
                            links.resolvedSymbol = unknownSymbol;
                            links.resolvedType = errorType;
                        }
                    }
                }
                return links.resolvedType;
            }
            function resolveImportSymbolType(node, links, symbol, meaning) {
                var resolvedSymbol = resolveSymbol(symbol);
                links.resolvedSymbol = resolvedSymbol;
                if (meaning === 111551 /* Value */) {
                    return getTypeOfSymbol(symbol); // intentionally doesn't use resolved symbol so type is cached as expected on the alias
                }
                else {
                    return getTypeReferenceType(node, resolvedSymbol); // getTypeReferenceType doesn't handle aliases - it must get the resolved symbol
                }
            }
            function getTypeFromTypeLiteralOrFunctionOrConstructorTypeNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    // Deferred resolution of members is handled by resolveObjectTypeMembers
                    var aliasSymbol = getAliasSymbolForTypeNode(node);
                    if (getMembersOfSymbol(node.symbol).size === 0 && !aliasSymbol) {
                        links.resolvedType = emptyTypeLiteralType;
                    }
                    else {
                        var type = createObjectType(16 /* Anonymous */, node.symbol);
                        type.aliasSymbol = aliasSymbol;
                        type.aliasTypeArguments = getTypeArgumentsForAliasSymbol(aliasSymbol);
                        if (ts.isJSDocTypeLiteral(node) && node.isArrayType) {
                            type = createArrayType(type);
                        }
                        links.resolvedType = type;
                    }
                }
                return links.resolvedType;
            }
            function getAliasSymbolForTypeNode(node) {
                var host = node.parent;
                while (ts.isParenthesizedTypeNode(host) || ts.isTypeOperatorNode(host) && host.operator === 138 /* ReadonlyKeyword */) {
                    host = host.parent;
                }
                return ts.isTypeAlias(host) ? getSymbolOfNode(host) : undefined;
            }
            function getTypeArgumentsForAliasSymbol(symbol) {
                return symbol ? getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(symbol) : undefined;
            }
            function isNonGenericObjectType(type) {
                return !!(type.flags & 524288 /* Object */) && !isGenericMappedType(type);
            }
            function isEmptyObjectTypeOrSpreadsIntoEmptyObject(type) {
                return isEmptyObjectType(type) || !!(type.flags & (65536 /* Null */ | 32768 /* Undefined */ | 528 /* BooleanLike */ | 296 /* NumberLike */ | 2112 /* BigIntLike */ | 132 /* StringLike */ | 1056 /* EnumLike */ | 67108864 /* NonPrimitive */ | 4194304 /* Index */));
            }
            function isSinglePropertyAnonymousObjectType(type) {
                return !!(type.flags & 524288 /* Object */) &&
                    !!(ts.getObjectFlags(type) & 16 /* Anonymous */) &&
                    (ts.length(getPropertiesOfType(type)) === 1 || ts.every(getPropertiesOfType(type), function (p) { return !!(p.flags & 16777216 /* Optional */); }));
            }
            function tryMergeUnionOfObjectTypeAndEmptyObject(type, readonly) {
                if (type.types.length === 2) {
                    var firstType = type.types[0];
                    var secondType = type.types[1];
                    if (ts.every(type.types, isEmptyObjectTypeOrSpreadsIntoEmptyObject)) {
                        return isEmptyObjectType(firstType) ? firstType : isEmptyObjectType(secondType) ? secondType : emptyObjectType;
                    }
                    if (isEmptyObjectTypeOrSpreadsIntoEmptyObject(firstType) && isSinglePropertyAnonymousObjectType(secondType)) {
                        return getAnonymousPartialType(secondType);
                    }
                    if (isEmptyObjectTypeOrSpreadsIntoEmptyObject(secondType) && isSinglePropertyAnonymousObjectType(firstType)) {
                        return getAnonymousPartialType(firstType);
                    }
                }
                function getAnonymousPartialType(type) {
                    // gets the type as if it had been spread, but where everything in the spread is made optional
                    var members = ts.createSymbolTable();
                    for (var _i = 0, _a = getPropertiesOfType(type); _i < _a.length; _i++) {
                        var prop = _a[_i];
                        if (ts.getDeclarationModifierFlagsFromSymbol(prop) & (8 /* Private */ | 16 /* Protected */)) {
                            // do nothing, skip privates
                        }
                        else if (isSpreadableProperty(prop)) {
                            var isSetonlyAccessor = prop.flags & 65536 /* SetAccessor */ && !(prop.flags & 32768 /* GetAccessor */);
                            var flags = 4 /* Property */ | 16777216 /* Optional */;
                            var result = createSymbol(flags, prop.escapedName, readonly ? 8 /* Readonly */ : 0);
                            result.type = isSetonlyAccessor ? undefinedType : getTypeOfSymbol(prop);
                            result.declarations = prop.declarations;
                            result.nameType = getSymbolLinks(prop).nameType;
                            result.syntheticOrigin = prop;
                            members.set(prop.escapedName, result);
                        }
                    }
                    var spread = createAnonymousType(type.symbol, members, ts.emptyArray, ts.emptyArray, getIndexInfoOfType(type, 0 /* String */), getIndexInfoOfType(type, 1 /* Number */));
                    spread.objectFlags |= 128 /* ObjectLiteral */ | 1048576 /* ContainsObjectOrArrayLiteral */;
                    return spread;
                }
            }
            /**
             * Since the source of spread types are object literals, which are not binary,
             * this function should be called in a left folding style, with left = previous result of getSpreadType
             * and right = the new element to be spread.
             */
            function getSpreadType(left, right, symbol, objectFlags, readonly) {
                if (left.flags & 1 /* Any */ || right.flags & 1 /* Any */) {
                    return anyType;
                }
                if (left.flags & 2 /* Unknown */ || right.flags & 2 /* Unknown */) {
                    return unknownType;
                }
                if (left.flags & 131072 /* Never */) {
                    return right;
                }
                if (right.flags & 131072 /* Never */) {
                    return left;
                }
                if (left.flags & 1048576 /* Union */) {
                    var merged = tryMergeUnionOfObjectTypeAndEmptyObject(left, readonly);
                    if (merged) {
                        return getSpreadType(merged, right, symbol, objectFlags, readonly);
                    }
                    return mapType(left, function (t) { return getSpreadType(t, right, symbol, objectFlags, readonly); });
                }
                if (right.flags & 1048576 /* Union */) {
                    var merged = tryMergeUnionOfObjectTypeAndEmptyObject(right, readonly);
                    if (merged) {
                        return getSpreadType(left, merged, symbol, objectFlags, readonly);
                    }
                    return mapType(right, function (t) { return getSpreadType(left, t, symbol, objectFlags, readonly); });
                }
                if (right.flags & (528 /* BooleanLike */ | 296 /* NumberLike */ | 2112 /* BigIntLike */ | 132 /* StringLike */ | 1056 /* EnumLike */ | 67108864 /* NonPrimitive */ | 4194304 /* Index */)) {
                    return left;
                }
                if (isGenericObjectType(left) || isGenericObjectType(right)) {
                    if (isEmptyObjectType(left)) {
                        return right;
                    }
                    // When the left type is an intersection, we may need to merge the last constituent of the
                    // intersection with the right type. For example when the left type is 'T & { a: string }'
                    // and the right type is '{ b: string }' we produce 'T & { a: string, b: string }'.
                    if (left.flags & 2097152 /* Intersection */) {
                        var types = left.types;
                        var lastLeft = types[types.length - 1];
                        if (isNonGenericObjectType(lastLeft) && isNonGenericObjectType(right)) {
                            return getIntersectionType(ts.concatenate(types.slice(0, types.length - 1), [getSpreadType(lastLeft, right, symbol, objectFlags, readonly)]));
                        }
                    }
                    return getIntersectionType([left, right]);
                }
                var members = ts.createSymbolTable();
                var skippedPrivateMembers = ts.createUnderscoreEscapedMap();
                var stringIndexInfo;
                var numberIndexInfo;
                if (left === emptyObjectType) {
                    // for the first spread element, left === emptyObjectType, so take the right's string indexer
                    stringIndexInfo = getIndexInfoOfType(right, 0 /* String */);
                    numberIndexInfo = getIndexInfoOfType(right, 1 /* Number */);
                }
                else {
                    stringIndexInfo = unionSpreadIndexInfos(getIndexInfoOfType(left, 0 /* String */), getIndexInfoOfType(right, 0 /* String */));
                    numberIndexInfo = unionSpreadIndexInfos(getIndexInfoOfType(left, 1 /* Number */), getIndexInfoOfType(right, 1 /* Number */));
                }
                for (var _i = 0, _a = getPropertiesOfType(right); _i < _a.length; _i++) {
                    var rightProp = _a[_i];
                    if (ts.getDeclarationModifierFlagsFromSymbol(rightProp) & (8 /* Private */ | 16 /* Protected */)) {
                        skippedPrivateMembers.set(rightProp.escapedName, true);
                    }
                    else if (isSpreadableProperty(rightProp)) {
                        members.set(rightProp.escapedName, getSpreadSymbol(rightProp, readonly));
                    }
                }
                for (var _b = 0, _c = getPropertiesOfType(left); _b < _c.length; _b++) {
                    var leftProp = _c[_b];
                    if (skippedPrivateMembers.has(leftProp.escapedName) || !isSpreadableProperty(leftProp)) {
                        continue;
                    }
                    if (members.has(leftProp.escapedName)) {
                        var rightProp = members.get(leftProp.escapedName);
                        var rightType = getTypeOfSymbol(rightProp);
                        if (rightProp.flags & 16777216 /* Optional */) {
                            var declarations = ts.concatenate(leftProp.declarations, rightProp.declarations);
                            var flags = 4 /* Property */ | (leftProp.flags & 16777216 /* Optional */);
                            var result = createSymbol(flags, leftProp.escapedName);
                            result.type = getUnionType([getTypeOfSymbol(leftProp), getTypeWithFacts(rightType, 524288 /* NEUndefined */)]);
                            result.leftSpread = leftProp;
                            result.rightSpread = rightProp;
                            result.declarations = declarations;
                            result.nameType = getSymbolLinks(leftProp).nameType;
                            members.set(leftProp.escapedName, result);
                        }
                    }
                    else {
                        members.set(leftProp.escapedName, getSpreadSymbol(leftProp, readonly));
                    }
                }
                var spread = createAnonymousType(symbol, members, ts.emptyArray, ts.emptyArray, getIndexInfoWithReadonly(stringIndexInfo, readonly), getIndexInfoWithReadonly(numberIndexInfo, readonly));
                spread.objectFlags |= 128 /* ObjectLiteral */ | 1048576 /* ContainsObjectOrArrayLiteral */ | 1024 /* ContainsSpread */ | objectFlags;
                return spread;
            }
            /** We approximate own properties as non-methods plus methods that are inside the object literal */
            function isSpreadableProperty(prop) {
                return !ts.some(prop.declarations, ts.isPrivateIdentifierPropertyDeclaration) &&
                    (!(prop.flags & (8192 /* Method */ | 32768 /* GetAccessor */ | 65536 /* SetAccessor */)) ||
                        !prop.declarations.some(function (decl) { return ts.isClassLike(decl.parent); }));
            }
            function getSpreadSymbol(prop, readonly) {
                var isSetonlyAccessor = prop.flags & 65536 /* SetAccessor */ && !(prop.flags & 32768 /* GetAccessor */);
                if (!isSetonlyAccessor && readonly === isReadonlySymbol(prop)) {
                    return prop;
                }
                var flags = 4 /* Property */ | (prop.flags & 16777216 /* Optional */);
                var result = createSymbol(flags, prop.escapedName, readonly ? 8 /* Readonly */ : 0);
                result.type = isSetonlyAccessor ? undefinedType : getTypeOfSymbol(prop);
                result.declarations = prop.declarations;
                result.nameType = getSymbolLinks(prop).nameType;
                result.syntheticOrigin = prop;
                return result;
            }
            function getIndexInfoWithReadonly(info, readonly) {
                return info && info.isReadonly !== readonly ? createIndexInfo(info.type, readonly, info.declaration) : info;
            }
            function createLiteralType(flags, value, symbol) {
                var type = createType(flags);
                type.symbol = symbol;
                type.value = value;
                return type;
            }
            function getFreshTypeOfLiteralType(type) {
                if (type.flags & 2944 /* Literal */) {
                    if (!type.freshType) {
                        var freshType = createLiteralType(type.flags, type.value, type.symbol);
                        freshType.regularType = type;
                        freshType.freshType = freshType;
                        type.freshType = freshType;
                    }
                    return type.freshType;
                }
                return type;
            }
            function getRegularTypeOfLiteralType(type) {
                return type.flags & 2944 /* Literal */ ? type.regularType :
                    type.flags & 1048576 /* Union */ ? getUnionType(ts.sameMap(type.types, getRegularTypeOfLiteralType)) :
                        type;
            }
            function isFreshLiteralType(type) {
                return !!(type.flags & 2944 /* Literal */) && type.freshType === type;
            }
            function getLiteralType(value, enumId, symbol) {
                // We store all literal types in a single map with keys of the form '#NNN' and '@SSS',
                // where NNN is the text representation of a numeric literal and SSS are the characters
                // of a string literal. For literal enum members we use 'EEE#NNN' and 'EEE@SSS', where
                // EEE is a unique id for the containing enum type.
                var qualifier = typeof value === "number" ? "#" : typeof value === "string" ? "@" : "n";
                var key = (enumId ? enumId : "") + qualifier + (typeof value === "object" ? ts.pseudoBigIntToString(value) : value);
                var type = literalTypes.get(key);
                if (!type) {
                    var flags = (typeof value === "number" ? 256 /* NumberLiteral */ :
                        typeof value === "string" ? 128 /* StringLiteral */ : 2048 /* BigIntLiteral */) |
                        (enumId ? 1024 /* EnumLiteral */ : 0);
                    literalTypes.set(key, type = createLiteralType(flags, value, symbol));
                    type.regularType = type;
                }
                return type;
            }
            function getTypeFromLiteralTypeNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    links.resolvedType = getRegularTypeOfLiteralType(checkExpression(node.literal));
                }
                return links.resolvedType;
            }
            function createUniqueESSymbolType(symbol) {
                var type = createType(8192 /* UniqueESSymbol */);
                type.symbol = symbol;
                type.escapedName = "__@" + type.symbol.escapedName + "@" + getSymbolId(type.symbol);
                return type;
            }
            function getESSymbolLikeTypeForNode(node) {
                if (ts.isValidESSymbolDeclaration(node)) {
                    var symbol = getSymbolOfNode(node);
                    var links = getSymbolLinks(symbol);
                    return links.uniqueESSymbolType || (links.uniqueESSymbolType = createUniqueESSymbolType(symbol));
                }
                return esSymbolType;
            }
            function getThisType(node) {
                var container = ts.getThisContainer(node, /*includeArrowFunctions*/ false);
                var parent = container && container.parent;
                if (parent && (ts.isClassLike(parent) || parent.kind === 246 /* InterfaceDeclaration */)) {
                    if (!ts.hasModifier(container, 32 /* Static */) &&
                        (!ts.isConstructorDeclaration(container) || ts.isNodeDescendantOf(node, container.body))) {
                        return getDeclaredTypeOfClassOrInterface(getSymbolOfNode(parent)).thisType;
                    }
                }
                // inside x.prototype = { ... }
                if (parent && ts.isObjectLiteralExpression(parent) && ts.isBinaryExpression(parent.parent) && ts.getAssignmentDeclarationKind(parent.parent) === 6 /* Prototype */) {
                    return getDeclaredTypeOfClassOrInterface(getSymbolOfNode(parent.parent.left).parent).thisType;
                }
                // /** @return {this} */
                // x.prototype.m = function() { ... }
                var host = node.flags & 4194304 /* JSDoc */ ? ts.getHostSignatureFromJSDoc(node) : undefined;
                if (host && ts.isFunctionExpression(host) && ts.isBinaryExpression(host.parent) && ts.getAssignmentDeclarationKind(host.parent) === 3 /* PrototypeProperty */) {
                    return getDeclaredTypeOfClassOrInterface(getSymbolOfNode(host.parent.left).parent).thisType;
                }
                // inside constructor function C() { ... }
                if (isJSConstructor(container) && ts.isNodeDescendantOf(node, container.body)) {
                    return getDeclaredTypeOfClassOrInterface(getSymbolOfNode(container)).thisType;
                }
                error(node, ts.Diagnostics.A_this_type_is_available_only_in_a_non_static_member_of_a_class_or_interface);
                return errorType;
            }
            function getTypeFromThisTypeNode(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    links.resolvedType = getThisType(node);
                }
                return links.resolvedType;
            }
            function getTypeFromTypeNode(node) {
                switch (node.kind) {
                    case 125 /* AnyKeyword */:
                    case 295 /* JSDocAllType */:
                    case 296 /* JSDocUnknownType */:
                        return anyType;
                    case 148 /* UnknownKeyword */:
                        return unknownType;
                    case 143 /* StringKeyword */:
                        return stringType;
                    case 140 /* NumberKeyword */:
                        return numberType;
                    case 151 /* BigIntKeyword */:
                        return bigintType;
                    case 128 /* BooleanKeyword */:
                        return booleanType;
                    case 144 /* SymbolKeyword */:
                        return esSymbolType;
                    case 110 /* VoidKeyword */:
                        return voidType;
                    case 146 /* UndefinedKeyword */:
                        return undefinedType;
                    case 100 /* NullKeyword */:
                        return nullType;
                    case 137 /* NeverKeyword */:
                        return neverType;
                    case 141 /* ObjectKeyword */:
                        return node.flags & 131072 /* JavaScriptFile */ && !noImplicitAny ? anyType : nonPrimitiveType;
                    case 183 /* ThisType */:
                    case 104 /* ThisKeyword */:
                        return getTypeFromThisTypeNode(node);
                    case 187 /* LiteralType */:
                        return getTypeFromLiteralTypeNode(node);
                    case 169 /* TypeReference */:
                        return getTypeFromTypeReference(node);
                    case 168 /* TypePredicate */:
                        return node.assertsModifier ? voidType : booleanType;
                    case 216 /* ExpressionWithTypeArguments */:
                        return getTypeFromTypeReference(node);
                    case 172 /* TypeQuery */:
                        return getTypeFromTypeQueryNode(node);
                    case 174 /* ArrayType */:
                    case 175 /* TupleType */:
                        return getTypeFromArrayOrTupleTypeNode(node);
                    case 176 /* OptionalType */:
                        return getTypeFromOptionalTypeNode(node);
                    case 178 /* UnionType */:
                        return getTypeFromUnionTypeNode(node);
                    case 179 /* IntersectionType */:
                        return getTypeFromIntersectionTypeNode(node);
                    case 297 /* JSDocNullableType */:
                        return getTypeFromJSDocNullableTypeNode(node);
                    case 299 /* JSDocOptionalType */:
                        return addOptionality(getTypeFromTypeNode(node.type));
                    case 182 /* ParenthesizedType */:
                    case 298 /* JSDocNonNullableType */:
                    case 294 /* JSDocTypeExpression */:
                        return getTypeFromTypeNode(node.type);
                    case 177 /* RestType */:
                        return getElementTypeOfArrayType(getTypeFromTypeNode(node.type)) || errorType;
                    case 301 /* JSDocVariadicType */:
                        return getTypeFromJSDocVariadicType(node);
                    case 170 /* FunctionType */:
                    case 171 /* ConstructorType */:
                    case 173 /* TypeLiteral */:
                    case 304 /* JSDocTypeLiteral */:
                    case 300 /* JSDocFunctionType */:
                    case 305 /* JSDocSignature */:
                        return getTypeFromTypeLiteralOrFunctionOrConstructorTypeNode(node);
                    case 184 /* TypeOperator */:
                        return getTypeFromTypeOperatorNode(node);
                    case 185 /* IndexedAccessType */:
                        return getTypeFromIndexedAccessTypeNode(node);
                    case 186 /* MappedType */:
                        return getTypeFromMappedTypeNode(node);
                    case 180 /* ConditionalType */:
                        return getTypeFromConditionalTypeNode(node);
                    case 181 /* InferType */:
                        return getTypeFromInferTypeNode(node);
                    case 188 /* ImportType */:
                        return getTypeFromImportTypeNode(node);
                    // This function assumes that an identifier or qualified name is a type expression
                    // Callers should first ensure this by calling isTypeNode
                    case 75 /* Identifier */:
                    case 153 /* QualifiedName */:
                        var symbol = getSymbolAtLocation(node);
                        return symbol ? getDeclaredTypeOfSymbol(symbol) : errorType;
                    default:
                        return errorType;
                }
            }
            function instantiateList(items, mapper, instantiator) {
                if (items && items.length) {
                    for (var i = 0; i < items.length; i++) {
                        var item = items[i];
                        var mapped = instantiator(item, mapper);
                        if (item !== mapped) {
                            var result = i === 0 ? [] : items.slice(0, i);
                            result.push(mapped);
                            for (i++; i < items.length; i++) {
                                result.push(instantiator(items[i], mapper));
                            }
                            return result;
                        }
                    }
                }
                return items;
            }
            function instantiateTypes(types, mapper) {
                return instantiateList(types, mapper, instantiateType);
            }
            function instantiateSignatures(signatures, mapper) {
                return instantiateList(signatures, mapper, instantiateSignature);
            }
            function makeUnaryTypeMapper(source, target) {
                return function (t) { return t === source ? target : t; };
            }
            function makeBinaryTypeMapper(source1, target1, source2, target2) {
                return function (t) { return t === source1 ? target1 : t === source2 ? target2 : t; };
            }
            function makeArrayTypeMapper(sources, targets) {
                return function (t) {
                    for (var i = 0; i < sources.length; i++) {
                        if (t === sources[i]) {
                            return targets ? targets[i] : anyType;
                        }
                    }
                    return t;
                };
            }
            function createTypeMapper(sources, targets) {
                ts.Debug.assert(targets === undefined || sources.length === targets.length);
                return sources.length === 1 ? makeUnaryTypeMapper(sources[0], targets ? targets[0] : anyType) :
                    sources.length === 2 ? makeBinaryTypeMapper(sources[0], targets ? targets[0] : anyType, sources[1], targets ? targets[1] : anyType) :
                        makeArrayTypeMapper(sources, targets);
            }
            function createTypeEraser(sources) {
                return createTypeMapper(sources, /*targets*/ undefined);
            }
            /**
             * Maps forward-references to later types parameters to the empty object type.
             * This is used during inference when instantiating type parameter defaults.
             */
            function createBackreferenceMapper(context, index) {
                return function (t) { return ts.findIndex(context.inferences, function (info) { return info.typeParameter === t; }) >= index ? unknownType : t; };
            }
            function combineTypeMappers(mapper1, mapper2) {
                if (!mapper1)
                    return mapper2;
                if (!mapper2)
                    return mapper1;
                return function (t) { return instantiateType(mapper1(t), mapper2); };
            }
            function createReplacementMapper(source, target, baseMapper) {
                return function (t) { return t === source ? target : baseMapper(t); };
            }
            function permissiveMapper(type) {
                return type.flags & 262144 /* TypeParameter */ ? wildcardType : type;
            }
            function getRestrictiveTypeParameter(tp) {
                return tp.constraint === unknownType ? tp : tp.restrictiveInstantiation || (tp.restrictiveInstantiation = createTypeParameter(tp.symbol),
                    tp.restrictiveInstantiation.constraint = unknownType,
                    tp.restrictiveInstantiation);
            }
            function restrictiveMapper(type) {
                return type.flags & 262144 /* TypeParameter */ ? getRestrictiveTypeParameter(type) : type;
            }
            function cloneTypeParameter(typeParameter) {
                var result = createTypeParameter(typeParameter.symbol);
                result.target = typeParameter;
                return result;
            }
            function instantiateTypePredicate(predicate, mapper) {
                return createTypePredicate(predicate.kind, predicate.parameterName, predicate.parameterIndex, instantiateType(predicate.type, mapper));
            }
            function instantiateSignature(signature, mapper, eraseTypeParameters) {
                var freshTypeParameters;
                if (signature.typeParameters && !eraseTypeParameters) {
                    // First create a fresh set of type parameters, then include a mapping from the old to the
                    // new type parameters in the mapper function. Finally store this mapper in the new type
                    // parameters such that we can use it when instantiating constraints.
                    freshTypeParameters = ts.map(signature.typeParameters, cloneTypeParameter);
                    mapper = combineTypeMappers(createTypeMapper(signature.typeParameters, freshTypeParameters), mapper);
                    for (var _i = 0, freshTypeParameters_1 = freshTypeParameters; _i < freshTypeParameters_1.length; _i++) {
                        var tp = freshTypeParameters_1[_i];
                        tp.mapper = mapper;
                    }
                }
                // Don't compute resolvedReturnType and resolvedTypePredicate now,
                // because using `mapper` now could trigger inferences to become fixed. (See `createInferenceContext`.)
                // See GH#17600.
                var result = createSignature(signature.declaration, freshTypeParameters, signature.thisParameter && instantiateSymbol(signature.thisParameter, mapper), instantiateList(signature.parameters, mapper, instantiateSymbol), 
                /*resolvedReturnType*/ undefined, 
                /*resolvedTypePredicate*/ undefined, signature.minArgumentCount, signature.flags & 3 /* PropagatingFlags */);
                result.target = signature;
                result.mapper = mapper;
                return result;
            }
            function instantiateSymbol(symbol, mapper) {
                var links = getSymbolLinks(symbol);
                if (links.type && !maybeTypeOfKind(links.type, 524288 /* Object */ | 63176704 /* Instantiable */)) {
                    // If the type of the symbol is already resolved, and if that type could not possibly
                    // be affected by instantiation, simply return the symbol itself.
                    return symbol;
                }
                if (ts.getCheckFlags(symbol) & 1 /* Instantiated */) {
                    // If symbol being instantiated is itself a instantiation, fetch the original target and combine the
                    // type mappers. This ensures that original type identities are properly preserved and that aliases
                    // always reference a non-aliases.
                    symbol = links.target;
                    mapper = combineTypeMappers(links.mapper, mapper);
                }
                // Keep the flags from the symbol we're instantiating.  Mark that is instantiated, and
                // also transient so that we can just store data on it directly.
                var result = createSymbol(symbol.flags, symbol.escapedName, 1 /* Instantiated */ | ts.getCheckFlags(symbol) & (8 /* Readonly */ | 4096 /* Late */ | 16384 /* OptionalParameter */ | 32768 /* RestParameter */));
                result.declarations = symbol.declarations;
                result.parent = symbol.parent;
                result.target = symbol;
                result.mapper = mapper;
                if (symbol.valueDeclaration) {
                    result.valueDeclaration = symbol.valueDeclaration;
                }
                if (links.nameType) {
                    result.nameType = links.nameType;
                }
                return result;
            }
            function getObjectTypeInstantiation(type, mapper) {
                var target = type.objectFlags & 64 /* Instantiated */ ? type.target : type;
                var node = type.objectFlags & 4 /* Reference */ ? type.node : type.symbol.declarations[0];
                var links = getNodeLinks(node);
                var typeParameters = links.outerTypeParameters;
                if (!typeParameters) {
                    // The first time an anonymous type is instantiated we compute and store a list of the type
                    // parameters that are in scope (and therefore potentially referenced). For type literals that
                    // aren't the right hand side of a generic type alias declaration we optimize by reducing the
                    // set of type parameters to those that are possibly referenced in the literal.
                    var declaration_1 = node;
                    if (ts.isInJSFile(declaration_1)) {
                        var paramTag = ts.findAncestor(declaration_1, ts.isJSDocParameterTag);
                        if (paramTag) {
                            var paramSymbol = ts.getParameterSymbolFromJSDoc(paramTag);
                            if (paramSymbol) {
                                declaration_1 = paramSymbol.valueDeclaration;
                            }
                        }
                    }
                    var outerTypeParameters = getOuterTypeParameters(declaration_1, /*includeThisTypes*/ true);
                    if (isJSConstructor(declaration_1)) {
                        var templateTagParameters = getTypeParametersFromDeclaration(declaration_1);
                        outerTypeParameters = ts.addRange(outerTypeParameters, templateTagParameters);
                    }
                    typeParameters = outerTypeParameters || ts.emptyArray;
                    typeParameters = (target.objectFlags & 4 /* Reference */ || target.symbol.flags & 2048 /* TypeLiteral */) && !target.aliasTypeArguments ?
                        ts.filter(typeParameters, function (tp) { return isTypeParameterPossiblyReferenced(tp, declaration_1); }) :
                        typeParameters;
                    links.outerTypeParameters = typeParameters;
                    if (typeParameters.length) {
                        links.instantiations = ts.createMap();
                        links.instantiations.set(getTypeListId(typeParameters), target);
                    }
                }
                if (typeParameters.length) {
                    // We are instantiating an anonymous type that has one or more type parameters in scope. Apply the
                    // mapper to the type parameters to produce the effective list of type arguments, and compute the
                    // instantiation cache key from the type IDs of the type arguments.
                    var typeArguments = ts.map(typeParameters, combineTypeMappers(type.mapper, mapper));
                    var id = getTypeListId(typeArguments);
                    var result = links.instantiations.get(id);
                    if (!result) {
                        var newMapper = createTypeMapper(typeParameters, typeArguments);
                        result = target.objectFlags & 4 /* Reference */ ? createDeferredTypeReference(type.target, type.node, newMapper) :
                            target.objectFlags & 32 /* Mapped */ ? instantiateMappedType(target, newMapper) :
                                instantiateAnonymousType(target, newMapper);
                        links.instantiations.set(id, result);
                    }
                    return result;
                }
                return type;
            }
            function maybeTypeParameterReference(node) {
                return !(node.kind === 153 /* QualifiedName */ ||
                    node.parent.kind === 169 /* TypeReference */ && node.parent.typeArguments && node === node.parent.typeName ||
                    node.parent.kind === 188 /* ImportType */ && node.parent.typeArguments && node === node.parent.qualifier);
            }
            function isTypeParameterPossiblyReferenced(tp, node) {
                // If the type parameter doesn't have exactly one declaration, if there are invening statement blocks
                // between the node and the type parameter declaration, if the node contains actual references to the
                // type parameter, or if the node contains type queries, we consider the type parameter possibly referenced.
                if (tp.symbol && tp.symbol.declarations && tp.symbol.declarations.length === 1) {
                    var container = tp.symbol.declarations[0].parent;
                    for (var n = node; n !== container; n = n.parent) {
                        if (!n || n.kind === 223 /* Block */ || n.kind === 180 /* ConditionalType */ && ts.forEachChild(n.extendsType, containsReference)) {
                            return true;
                        }
                    }
                    return !!ts.forEachChild(node, containsReference);
                }
                return true;
                function containsReference(node) {
                    switch (node.kind) {
                        case 183 /* ThisType */:
                            return !!tp.isThisType;
                        case 75 /* Identifier */:
                            return !tp.isThisType && ts.isPartOfTypeNode(node) && maybeTypeParameterReference(node) &&
                                getTypeFromTypeNode(node) === tp;
                        case 172 /* TypeQuery */:
                            return true;
                    }
                    return !!ts.forEachChild(node, containsReference);
                }
            }
            function getHomomorphicTypeVariable(type) {
                var constraintType = getConstraintTypeFromMappedType(type);
                if (constraintType.flags & 4194304 /* Index */) {
                    var typeVariable = getActualTypeVariable(constraintType.type);
                    if (typeVariable.flags & 262144 /* TypeParameter */) {
                        return typeVariable;
                    }
                }
                return undefined;
            }
            function instantiateMappedType(type, mapper) {
                // For a homomorphic mapped type { [P in keyof T]: X }, where T is some type variable, the mapping
                // operation depends on T as follows:
                // * If T is a primitive type no mapping is performed and the result is simply T.
                // * If T is a union type we distribute the mapped type over the union.
                // * If T is an array we map to an array where the element type has been transformed.
                // * If T is a tuple we map to a tuple where the element types have been transformed.
                // * Otherwise we map to an object type where the type of each property has been transformed.
                // For example, when T is instantiated to a union type A | B, we produce { [P in keyof A]: X } |
                // { [P in keyof B]: X }, and when when T is instantiated to a union type A | undefined, we produce
                // { [P in keyof A]: X } | undefined.
                var typeVariable = getHomomorphicTypeVariable(type);
                if (typeVariable) {
                    var mappedTypeVariable = instantiateType(typeVariable, mapper);
                    if (typeVariable !== mappedTypeVariable) {
                        return mapType(mappedTypeVariable, function (t) {
                            if (t.flags & (3 /* AnyOrUnknown */ | 58982400 /* InstantiableNonPrimitive */ | 524288 /* Object */ | 2097152 /* Intersection */) && t !== wildcardType && t !== errorType) {
                                var replacementMapper = createReplacementMapper(typeVariable, t, mapper);
                                return isArrayType(t) ? instantiateMappedArrayType(t, type, replacementMapper) :
                                    isTupleType(t) ? instantiateMappedTupleType(t, type, replacementMapper) :
                                        instantiateAnonymousType(type, replacementMapper);
                            }
                            return t;
                        });
                    }
                }
                return instantiateAnonymousType(type, mapper);
            }
            function getModifiedReadonlyState(state, modifiers) {
                return modifiers & 1 /* IncludeReadonly */ ? true : modifiers & 2 /* ExcludeReadonly */ ? false : state;
            }
            function instantiateMappedArrayType(arrayType, mappedType, mapper) {
                var elementType = instantiateMappedTypeTemplate(mappedType, numberType, /*isOptional*/ true, mapper);
                return elementType === errorType ? errorType :
                    createArrayType(elementType, getModifiedReadonlyState(isReadonlyArrayType(arrayType), getMappedTypeModifiers(mappedType)));
            }
            function instantiateMappedTupleType(tupleType, mappedType, mapper) {
                var minLength = tupleType.target.minLength;
                var elementTypes = ts.map(getTypeArguments(tupleType), function (_, i) {
                    return instantiateMappedTypeTemplate(mappedType, getLiteralType("" + i), i >= minLength, mapper);
                });
                var modifiers = getMappedTypeModifiers(mappedType);
                var newMinLength = modifiers & 4 /* IncludeOptional */ ? 0 :
                    modifiers & 8 /* ExcludeOptional */ ? getTypeReferenceArity(tupleType) - (tupleType.target.hasRestElement ? 1 : 0) :
                        minLength;
                var newReadonly = getModifiedReadonlyState(tupleType.target.readonly, modifiers);
                return ts.contains(elementTypes, errorType) ? errorType :
                    createTupleType(elementTypes, newMinLength, tupleType.target.hasRestElement, newReadonly, tupleType.target.associatedNames);
            }
            function instantiateMappedTypeTemplate(type, key, isOptional, mapper) {
                var templateMapper = combineTypeMappers(mapper, createTypeMapper([getTypeParameterFromMappedType(type)], [key]));
                var propType = instantiateType(getTemplateTypeFromMappedType(type.target || type), templateMapper);
                var modifiers = getMappedTypeModifiers(type);
                return strictNullChecks && modifiers & 4 /* IncludeOptional */ && !isTypeAssignableTo(undefinedType, propType) ? getOptionalType(propType) :
                    strictNullChecks && modifiers & 8 /* ExcludeOptional */ && isOptional ? getTypeWithFacts(propType, 524288 /* NEUndefined */) :
                        propType;
            }
            function instantiateAnonymousType(type, mapper) {
                var result = createObjectType(type.objectFlags | 64 /* Instantiated */, type.symbol);
                if (type.objectFlags & 32 /* Mapped */) {
                    result.declaration = type.declaration;
                    // C.f. instantiateSignature
                    var origTypeParameter = getTypeParameterFromMappedType(type);
                    var freshTypeParameter = cloneTypeParameter(origTypeParameter);
                    result.typeParameter = freshTypeParameter;
                    mapper = combineTypeMappers(makeUnaryTypeMapper(origTypeParameter, freshTypeParameter), mapper);
                    freshTypeParameter.mapper = mapper;
                }
                result.target = type;
                result.mapper = mapper;
                result.aliasSymbol = type.aliasSymbol;
                result.aliasTypeArguments = instantiateTypes(type.aliasTypeArguments, mapper);
                return result;
            }
            function getConditionalTypeInstantiation(type, mapper) {
                var root = type.root;
                if (root.outerTypeParameters) {
                    // We are instantiating a conditional type that has one or more type parameters in scope. Apply the
                    // mapper to the type parameters to produce the effective list of type arguments, and compute the
                    // instantiation cache key from the type IDs of the type arguments.
                    var typeArguments = ts.map(root.outerTypeParameters, mapper);
                    var id = getTypeListId(typeArguments);
                    var result = root.instantiations.get(id);
                    if (!result) {
                        var newMapper = createTypeMapper(root.outerTypeParameters, typeArguments);
                        result = instantiateConditionalType(root, newMapper);
                        root.instantiations.set(id, result);
                    }
                    return result;
                }
                return type;
            }
            function instantiateConditionalType(root, mapper) {
                // Check if we have a conditional type where the check type is a naked type parameter. If so,
                // the conditional type is distributive over union types and when T is instantiated to a union
                // type A | B, we produce (A extends U ? X : Y) | (B extends U ? X : Y).
                if (root.isDistributive) {
                    var checkType_1 = root.checkType;
                    var instantiatedType = mapper(checkType_1);
                    if (checkType_1 !== instantiatedType && instantiatedType.flags & (1048576 /* Union */ | 131072 /* Never */)) {
                        return mapType(instantiatedType, function (t) { return getConditionalType(root, createReplacementMapper(checkType_1, t, mapper)); });
                    }
                }
                return getConditionalType(root, mapper);
            }
            function instantiateType(type, mapper) {
                if (!type || !mapper || mapper === identityMapper) {
                    return type;
                }
                if (instantiationDepth === 50 || instantiationCount >= 5000000) {
                    // We have reached 50 recursive type instantiations and there is a very high likelyhood we're dealing
                    // with a combination of infinite generic types that perpetually generate new type identities. We stop
                    // the recursion here by yielding the error type.
                    error(currentNode, ts.Diagnostics.Type_instantiation_is_excessively_deep_and_possibly_infinite);
                    return errorType;
                }
                instantiationCount++;
                instantiationDepth++;
                var result = instantiateTypeWorker(type, mapper);
                instantiationDepth--;
                return result;
            }
            function instantiateTypeWorker(type, mapper) {
                var flags = type.flags;
                if (flags & 262144 /* TypeParameter */) {
                    return mapper(type);
                }
                if (flags & 524288 /* Object */) {
                    var objectFlags = type.objectFlags;
                    if (objectFlags & 16 /* Anonymous */) {
                        // If the anonymous type originates in a declaration of a function, method, class, or
                        // interface, in an object type literal, or in an object literal expression, we may need
                        // to instantiate the type because it might reference a type parameter.
                        return couldContainTypeVariables(type) ?
                            getObjectTypeInstantiation(type, mapper) : type;
                    }
                    if (objectFlags & 32 /* Mapped */) {
                        return getObjectTypeInstantiation(type, mapper);
                    }
                    if (objectFlags & 4 /* Reference */) {
                        if (type.node) {
                            return getObjectTypeInstantiation(type, mapper);
                        }
                        var resolvedTypeArguments = type.resolvedTypeArguments;
                        var newTypeArguments = instantiateTypes(resolvedTypeArguments, mapper);
                        return newTypeArguments !== resolvedTypeArguments ? createTypeReference(type.target, newTypeArguments) : type;
                    }
                    return type;
                }
                if (flags & 1048576 /* Union */ && !(flags & 131068 /* Primitive */)) {
                    var types = type.types;
                    var newTypes = instantiateTypes(types, mapper);
                    return newTypes !== types ? getUnionType(newTypes, 1 /* Literal */, type.aliasSymbol, instantiateTypes(type.aliasTypeArguments, mapper)) : type;
                }
                if (flags & 2097152 /* Intersection */) {
                    var types = type.types;
                    var newTypes = instantiateTypes(types, mapper);
                    return newTypes !== types ? getIntersectionType(newTypes, type.aliasSymbol, instantiateTypes(type.aliasTypeArguments, mapper)) : type;
                }
                if (flags & 4194304 /* Index */) {
                    return getIndexType(instantiateType(type.type, mapper));
                }
                if (flags & 8388608 /* IndexedAccess */) {
                    return getIndexedAccessType(instantiateType(type.objectType, mapper), instantiateType(type.indexType, mapper));
                }
                if (flags & 16777216 /* Conditional */) {
                    return getConditionalTypeInstantiation(type, combineTypeMappers(type.mapper, mapper));
                }
                if (flags & 33554432 /* Substitution */) {
                    var maybeVariable = instantiateType(type.typeVariable, mapper);
                    if (maybeVariable.flags & 8650752 /* TypeVariable */) {
                        return getSubstitutionType(maybeVariable, instantiateType(type.substitute, mapper));
                    }
                    else {
                        var sub = instantiateType(type.substitute, mapper);
                        if (sub.flags & 3 /* AnyOrUnknown */ || isTypeAssignableTo(getRestrictiveInstantiation(maybeVariable), getRestrictiveInstantiation(sub))) {
                            return maybeVariable;
                        }
                        return sub;
                    }
                }
                return type;
            }
            function getPermissiveInstantiation(type) {
                return type.flags & (131068 /* Primitive */ | 3 /* AnyOrUnknown */ | 131072 /* Never */) ? type :
                    type.permissiveInstantiation || (type.permissiveInstantiation = instantiateType(type, permissiveMapper));
            }
            function getRestrictiveInstantiation(type) {
                if (type.flags & (131068 /* Primitive */ | 3 /* AnyOrUnknown */ | 131072 /* Never */)) {
                    return type;
                }
                if (type.restrictiveInstantiation) {
                    return type.restrictiveInstantiation;
                }
                type.restrictiveInstantiation = instantiateType(type, restrictiveMapper);
                // We set the following so we don't attempt to set the restrictive instance of a restrictive instance
                // which is redundant - we'll produce new type identities, but all type params have already been mapped.
                // This also gives us a way to detect restrictive instances upon comparisons and _disable_ the "distributeive constraint"
                // assignability check for them, which is distinctly unsafe, as once you have a restrctive instance, all the type parameters
                // are constrained to `unknown` and produce tons of false positives/negatives!
                type.restrictiveInstantiation.restrictiveInstantiation = type.restrictiveInstantiation;
                return type.restrictiveInstantiation;
            }
            function instantiateIndexInfo(info, mapper) {
                return info && createIndexInfo(instantiateType(info.type, mapper), info.isReadonly, info.declaration);
            }
            // Returns true if the given expression contains (at any level of nesting) a function or arrow expression
            // that is subject to contextual typing.
            function isContextSensitive(node) {
                ts.Debug.assert(node.kind !== 161 /* MethodDeclaration */ || ts.isObjectLiteralMethod(node));
                switch (node.kind) {
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                    case 161 /* MethodDeclaration */:
                    case 244 /* FunctionDeclaration */: // Function declarations can have context when annotated with a jsdoc @type
                        return isContextSensitiveFunctionLikeDeclaration(node);
                    case 193 /* ObjectLiteralExpression */:
                        return ts.some(node.properties, isContextSensitive);
                    case 192 /* ArrayLiteralExpression */:
                        return ts.some(node.elements, isContextSensitive);
                    case 210 /* ConditionalExpression */:
                        return isContextSensitive(node.whenTrue) ||
                            isContextSensitive(node.whenFalse);
                    case 209 /* BinaryExpression */:
                        return (node.operatorToken.kind === 56 /* BarBarToken */ || node.operatorToken.kind === 60 /* QuestionQuestionToken */) &&
                            (isContextSensitive(node.left) || isContextSensitive(node.right));
                    case 281 /* PropertyAssignment */:
                        return isContextSensitive(node.initializer);
                    case 200 /* ParenthesizedExpression */:
                        return isContextSensitive(node.expression);
                    case 274 /* JsxAttributes */:
                        return ts.some(node.properties, isContextSensitive) || ts.isJsxOpeningElement(node.parent) && ts.some(node.parent.parent.children, isContextSensitive);
                    case 273 /* JsxAttribute */: {
                        // If there is no initializer, JSX attribute has a boolean value of true which is not context sensitive.
                        var initializer = node.initializer;
                        return !!initializer && isContextSensitive(initializer);
                    }
                    case 276 /* JsxExpression */: {
                        // It is possible to that node.expression is undefined (e.g <div x={} />)
                        var expression = node.expression;
                        return !!expression && isContextSensitive(expression);
                    }
                }
                return false;
            }
            function isContextSensitiveFunctionLikeDeclaration(node) {
                return (!ts.isFunctionDeclaration(node) || ts.isInJSFile(node) && !!getTypeForDeclarationFromJSDocComment(node)) &&
                    (hasContextSensitiveParameters(node) || hasContextSensitiveReturnExpression(node));
            }
            function hasContextSensitiveParameters(node) {
                // Functions with type parameters are not context sensitive.
                if (!node.typeParameters) {
                    // Functions with any parameters that lack type annotations are context sensitive.
                    if (ts.some(node.parameters, function (p) { return !ts.getEffectiveTypeAnnotationNode(p); })) {
                        return true;
                    }
                    if (node.kind !== 202 /* ArrowFunction */) {
                        // If the first parameter is not an explicit 'this' parameter, then the function has
                        // an implicit 'this' parameter which is subject to contextual typing.
                        var parameter = ts.firstOrUndefined(node.parameters);
                        if (!(parameter && ts.parameterIsThisKeyword(parameter))) {
                            return true;
                        }
                    }
                }
                return false;
            }
            function hasContextSensitiveReturnExpression(node) {
                // TODO(anhans): A block should be context-sensitive if it has a context-sensitive return value.
                return !ts.getEffectiveReturnTypeNode(node) && !!node.body && node.body.kind !== 223 /* Block */ && isContextSensitive(node.body);
            }
            function isContextSensitiveFunctionOrObjectLiteralMethod(func) {
                return (ts.isInJSFile(func) && ts.isFunctionDeclaration(func) || isFunctionExpressionOrArrowFunction(func) || ts.isObjectLiteralMethod(func)) &&
                    isContextSensitiveFunctionLikeDeclaration(func);
            }
            function getTypeWithoutSignatures(type) {
                if (type.flags & 524288 /* Object */) {
                    var resolved = resolveStructuredTypeMembers(type);
                    if (resolved.constructSignatures.length || resolved.callSignatures.length) {
                        var result = createObjectType(16 /* Anonymous */, type.symbol);
                        result.members = resolved.members;
                        result.properties = resolved.properties;
                        result.callSignatures = ts.emptyArray;
                        result.constructSignatures = ts.emptyArray;
                        return result;
                    }
                }
                else if (type.flags & 2097152 /* Intersection */) {
                    return getIntersectionType(ts.map(type.types, getTypeWithoutSignatures));
                }
                return type;
            }
            // TYPE CHECKING
            function isTypeIdenticalTo(source, target) {
                return isTypeRelatedTo(source, target, identityRelation);
            }
            function compareTypesIdentical(source, target) {
                return isTypeRelatedTo(source, target, identityRelation) ? -1 /* True */ : 0 /* False */;
            }
            function compareTypesAssignable(source, target) {
                return isTypeRelatedTo(source, target, assignableRelation) ? -1 /* True */ : 0 /* False */;
            }
            function compareTypesSubtypeOf(source, target) {
                return isTypeRelatedTo(source, target, subtypeRelation) ? -1 /* True */ : 0 /* False */;
            }
            function isTypeSubtypeOf(source, target) {
                return isTypeRelatedTo(source, target, subtypeRelation);
            }
            function isTypeAssignableTo(source, target) {
                return isTypeRelatedTo(source, target, assignableRelation);
            }
            // An object type S is considered to be derived from an object type T if
            // S is a union type and every constituent of S is derived from T,
            // T is a union type and S is derived from at least one constituent of T, or
            // S is a type variable with a base constraint that is derived from T,
            // T is one of the global types Object and Function and S is a subtype of T, or
            // T occurs directly or indirectly in an 'extends' clause of S.
            // Note that this check ignores type parameters and only considers the
            // inheritance hierarchy.
            function isTypeDerivedFrom(source, target) {
                return source.flags & 1048576 /* Union */ ? ts.every(source.types, function (t) { return isTypeDerivedFrom(t, target); }) :
                    target.flags & 1048576 /* Union */ ? ts.some(target.types, function (t) { return isTypeDerivedFrom(source, t); }) :
                        source.flags & 58982400 /* InstantiableNonPrimitive */ ? isTypeDerivedFrom(getBaseConstraintOfType(source) || unknownType, target) :
                            target === globalObjectType ? !!(source.flags & (524288 /* Object */ | 67108864 /* NonPrimitive */)) :
                                target === globalFunctionType ? !!(source.flags & 524288 /* Object */) && isFunctionObjectType(source) :
                                    hasBaseType(source, getTargetType(target));
            }
            /**
             * This is *not* a bi-directional relationship.
             * If one needs to check both directions for comparability, use a second call to this function or 'checkTypeComparableTo'.
             *
             * A type S is comparable to a type T if some (but not necessarily all) of the possible values of S are also possible values of T.
             * It is used to check following cases:
             *   - the types of the left and right sides of equality/inequality operators (`===`, `!==`, `==`, `!=`).
             *   - the types of `case` clause expressions and their respective `switch` expressions.
             *   - the type of an expression in a type assertion with the type being asserted.
             */
            function isTypeComparableTo(source, target) {
                return isTypeRelatedTo(source, target, comparableRelation);
            }
            function areTypesComparable(type1, type2) {
                return isTypeComparableTo(type1, type2) || isTypeComparableTo(type2, type1);
            }
            function checkTypeAssignableTo(source, target, errorNode, headMessage, containingMessageChain, errorOutputObject) {
                return checkTypeRelatedTo(source, target, assignableRelation, errorNode, headMessage, containingMessageChain, errorOutputObject);
            }
            /**
             * Like `checkTypeAssignableTo`, but if it would issue an error, instead performs structural comparisons of the types using the given expression node to
             * attempt to issue more specific errors on, for example, specific object literal properties or tuple members.
             */
            function checkTypeAssignableToAndOptionallyElaborate(source, target, errorNode, expr, headMessage, containingMessageChain) {
                return checkTypeRelatedToAndOptionallyElaborate(source, target, assignableRelation, errorNode, expr, headMessage, containingMessageChain, /*errorOutputContainer*/ undefined);
            }
            function checkTypeRelatedToAndOptionallyElaborate(source, target, relation, errorNode, expr, headMessage, containingMessageChain, errorOutputContainer) {
                if (isTypeRelatedTo(source, target, relation))
                    return true;
                if (!errorNode || !elaborateError(expr, source, target, relation, headMessage, containingMessageChain, errorOutputContainer)) {
                    return checkTypeRelatedTo(source, target, relation, errorNode, headMessage, containingMessageChain, errorOutputContainer);
                }
                return false;
            }
            function isOrHasGenericConditional(type) {
                return !!(type.flags & 16777216 /* Conditional */ || (type.flags & 2097152 /* Intersection */ && ts.some(type.types, isOrHasGenericConditional)));
            }
            function elaborateError(node, source, target, relation, headMessage, containingMessageChain, errorOutputContainer) {
                if (!node || isOrHasGenericConditional(target))
                    return false;
                if (!checkTypeRelatedTo(source, target, relation, /*errorNode*/ undefined)
                    && elaborateDidYouMeanToCallOrConstruct(node, source, target, relation, headMessage, containingMessageChain, errorOutputContainer)) {
                    return true;
                }
                switch (node.kind) {
                    case 276 /* JsxExpression */:
                    case 200 /* ParenthesizedExpression */:
                        return elaborateError(node.expression, source, target, relation, headMessage, containingMessageChain, errorOutputContainer);
                    case 209 /* BinaryExpression */:
                        switch (node.operatorToken.kind) {
                            case 62 /* EqualsToken */:
                            case 27 /* CommaToken */:
                                return elaborateError(node.right, source, target, relation, headMessage, containingMessageChain, errorOutputContainer);
                        }
                        break;
                    case 193 /* ObjectLiteralExpression */:
                        return elaborateObjectLiteral(node, source, target, relation, containingMessageChain, errorOutputContainer);
                    case 192 /* ArrayLiteralExpression */:
                        return elaborateArrayLiteral(node, source, target, relation, containingMessageChain, errorOutputContainer);
                    case 274 /* JsxAttributes */:
                        return elaborateJsxComponents(node, source, target, relation, containingMessageChain, errorOutputContainer);
                    case 202 /* ArrowFunction */:
                        return elaborateArrowFunction(node, source, target, relation, containingMessageChain, errorOutputContainer);
                }
                return false;
            }
            function elaborateDidYouMeanToCallOrConstruct(node, source, target, relation, headMessage, containingMessageChain, errorOutputContainer) {
                var callSignatures = getSignaturesOfType(source, 0 /* Call */);
                var constructSignatures = getSignaturesOfType(source, 1 /* Construct */);
                for (var _i = 0, _a = [constructSignatures, callSignatures]; _i < _a.length; _i++) {
                    var signatures = _a[_i];
                    if (ts.some(signatures, function (s) {
                        var returnType = getReturnTypeOfSignature(s);
                        return !(returnType.flags & (1 /* Any */ | 131072 /* Never */)) && checkTypeRelatedTo(returnType, target, relation, /*errorNode*/ undefined);
                    })) {
                        var resultObj = errorOutputContainer || {};
                        checkTypeAssignableTo(source, target, node, headMessage, containingMessageChain, resultObj);
                        var diagnostic = resultObj.errors[resultObj.errors.length - 1];
                        ts.addRelatedInfo(diagnostic, ts.createDiagnosticForNode(node, signatures === constructSignatures ? ts.Diagnostics.Did_you_mean_to_use_new_with_this_expression : ts.Diagnostics.Did_you_mean_to_call_this_expression));
                        return true;
                    }
                }
                return false;
            }
            function elaborateArrowFunction(node, source, target, relation, containingMessageChain, errorOutputContainer) {
                // Don't elaborate blocks
                if (ts.isBlock(node.body)) {
                    return false;
                }
                // Or functions with annotated parameter types
                if (ts.some(node.parameters, ts.hasType)) {
                    return false;
                }
                var sourceSig = getSingleCallSignature(source);
                if (!sourceSig) {
                    return false;
                }
                var targetSignatures = getSignaturesOfType(target, 0 /* Call */);
                if (!ts.length(targetSignatures)) {
                    return false;
                }
                var returnExpression = node.body;
                var sourceReturn = getReturnTypeOfSignature(sourceSig);
                var targetReturn = getUnionType(ts.map(targetSignatures, getReturnTypeOfSignature));
                if (!checkTypeRelatedTo(sourceReturn, targetReturn, relation, /*errorNode*/ undefined)) {
                    var elaborated = returnExpression && elaborateError(returnExpression, sourceReturn, targetReturn, relation, /*headMessage*/ undefined, containingMessageChain, errorOutputContainer);
                    if (elaborated) {
                        return elaborated;
                    }
                    var resultObj = errorOutputContainer || {};
                    checkTypeRelatedTo(sourceReturn, targetReturn, relation, returnExpression, /*message*/ undefined, containingMessageChain, resultObj);
                    if (resultObj.errors) {
                        if (target.symbol && ts.length(target.symbol.declarations)) {
                            ts.addRelatedInfo(resultObj.errors[resultObj.errors.length - 1], ts.createDiagnosticForNode(target.symbol.declarations[0], ts.Diagnostics.The_expected_type_comes_from_the_return_type_of_this_signature));
                        }
                        return true;
                    }
                }
                return false;
            }
            function getBestMatchIndexedAccessTypeOrUndefined(source, target, nameType) {
                var idx = getIndexedAccessTypeOrUndefined(target, nameType);
                if (idx) {
                    return idx;
                }
                if (target.flags & 1048576 /* Union */) {
                    var best = getBestMatchingType(source, target);
                    if (best) {
                        return getIndexedAccessTypeOrUndefined(best, nameType);
                    }
                }
            }
            /**
             * For every element returned from the iterator, checks that element to issue an error on a property of that element's type
             * If that element would issue an error, we first attempt to dive into that element's inner expression and issue a more specific error by recuring into `elaborateError`
             * Otherwise, we issue an error on _every_ element which fail the assignability check
             */
            function elaborateElementwise(iterator, source, target, relation, containingMessageChain, errorOutputContainer) {
                // Assignability failure - check each prop individually, and if that fails, fall back on the bad error span
                var reportedError = false;
                for (var status = iterator.next(); !status.done; status = iterator.next()) {
                    var _a = status.value, prop = _a.errorNode, next = _a.innerExpression, nameType = _a.nameType, errorMessage = _a.errorMessage;
                    var targetPropType = getBestMatchIndexedAccessTypeOrUndefined(source, target, nameType);
                    if (!targetPropType || targetPropType.flags & 8388608 /* IndexedAccess */)
                        continue; // Don't elaborate on indexes on generic variables
                    var sourcePropType = getIndexedAccessTypeOrUndefined(source, nameType);
                    if (sourcePropType && !checkTypeRelatedTo(sourcePropType, targetPropType, relation, /*errorNode*/ undefined)) {
                        var elaborated = next && elaborateError(next, sourcePropType, targetPropType, relation, /*headMessage*/ undefined, containingMessageChain, errorOutputContainer);
                        if (elaborated) {
                            reportedError = true;
                        }
                        else {
                            // Issue error on the prop itself, since the prop couldn't elaborate the error
                            var resultObj = errorOutputContainer || {};
                            // Use the expression type, if available
                            var specificSource = next ? checkExpressionForMutableLocation(next, 0 /* Normal */, sourcePropType) : sourcePropType;
                            var result = checkTypeRelatedTo(specificSource, targetPropType, relation, prop, errorMessage, containingMessageChain, resultObj);
                            if (result && specificSource !== sourcePropType) {
                                // If for whatever reason the expression type doesn't yield an error, make sure we still issue an error on the sourcePropType
                                checkTypeRelatedTo(sourcePropType, targetPropType, relation, prop, errorMessage, containingMessageChain, resultObj);
                            }
                            if (resultObj.errors) {
                                var reportedDiag = resultObj.errors[resultObj.errors.length - 1];
                                var propertyName = isTypeUsableAsPropertyName(nameType) ? getPropertyNameFromType(nameType) : undefined;
                                var targetProp = propertyName !== undefined ? getPropertyOfType(target, propertyName) : undefined;
                                var issuedElaboration = false;
                                if (!targetProp) {
                                    var indexInfo = isTypeAssignableToKind(nameType, 296 /* NumberLike */) && getIndexInfoOfType(target, 1 /* Number */) ||
                                        getIndexInfoOfType(target, 0 /* String */) ||
                                        undefined;
                                    if (indexInfo && indexInfo.declaration && !ts.getSourceFileOfNode(indexInfo.declaration).hasNoDefaultLib) {
                                        issuedElaboration = true;
                                        ts.addRelatedInfo(reportedDiag, ts.createDiagnosticForNode(indexInfo.declaration, ts.Diagnostics.The_expected_type_comes_from_this_index_signature));
                                    }
                                }
                                if (!issuedElaboration && (targetProp && ts.length(targetProp.declarations) || target.symbol && ts.length(target.symbol.declarations))) {
                                    var targetNode = targetProp && ts.length(targetProp.declarations) ? targetProp.declarations[0] : target.symbol.declarations[0];
                                    if (!ts.getSourceFileOfNode(targetNode).hasNoDefaultLib) {
                                        ts.addRelatedInfo(reportedDiag, ts.createDiagnosticForNode(targetNode, ts.Diagnostics.The_expected_type_comes_from_property_0_which_is_declared_here_on_type_1, propertyName && !(nameType.flags & 8192 /* UniqueESSymbol */) ? ts.unescapeLeadingUnderscores(propertyName) : typeToString(nameType), typeToString(target)));
                                    }
                                }
                            }
                            reportedError = true;
                        }
                    }
                }
                return reportedError;
            }
            function generateJsxAttributes(node) {
                var _i, _a, prop;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            if (!ts.length(node.properties))
                                return [2 /*return*/];
                            _i = 0, _a = node.properties;
                            _b.label = 1;
                        case 1:
                            if (!(_i < _a.length))
                                return [3 /*break*/, 4];
                            prop = _a[_i];
                            if (ts.isJsxSpreadAttribute(prop))
                                return [3 /*break*/, 3];
                            return [4 /*yield*/, { errorNode: prop.name, innerExpression: prop.initializer, nameType: getLiteralType(ts.idText(prop.name)) }];
                        case 2:
                            _b.sent();
                            _b.label = 3;
                        case 3:
                            _i++;
                            return [3 /*break*/, 1];
                        case 4: return [2 /*return*/];
                    }
                });
            }
            function generateJsxChildren(node, getInvalidTextDiagnostic) {
                var memberOffset, i, child, nameType, elem;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            if (!ts.length(node.children))
                                return [2 /*return*/];
                            memberOffset = 0;
                            i = 0;
                            _a.label = 1;
                        case 1:
                            if (!(i < node.children.length))
                                return [3 /*break*/, 5];
                            child = node.children[i];
                            nameType = getLiteralType(i - memberOffset);
                            elem = getElaborationElementForJsxChild(child, nameType, getInvalidTextDiagnostic);
                            if (!elem)
                                return [3 /*break*/, 3];
                            return [4 /*yield*/, elem];
                        case 2:
                            _a.sent();
                            return [3 /*break*/, 4];
                        case 3:
                            memberOffset++;
                            _a.label = 4;
                        case 4:
                            i++;
                            return [3 /*break*/, 1];
                        case 5: return [2 /*return*/];
                    }
                });
            }
            function getElaborationElementForJsxChild(child, nameType, getInvalidTextDiagnostic) {
                switch (child.kind) {
                    case 276 /* JsxExpression */:
                        // child is of the type of the expression
                        return { errorNode: child, innerExpression: child.expression, nameType: nameType };
                    case 11 /* JsxText */:
                        if (child.containsOnlyTriviaWhiteSpaces) {
                            break; // Whitespace only jsx text isn't real jsx text
                        }
                        // child is a string
                        return { errorNode: child, innerExpression: undefined, nameType: nameType, errorMessage: getInvalidTextDiagnostic() };
                    case 266 /* JsxElement */:
                    case 267 /* JsxSelfClosingElement */:
                    case 270 /* JsxFragment */:
                        // child is of type JSX.Element
                        return { errorNode: child, innerExpression: child, nameType: nameType };
                    default:
                        return ts.Debug.assertNever(child, "Found invalid jsx child");
                }
            }
            function getSemanticJsxChildren(children) {
                return ts.filter(children, function (i) { return !ts.isJsxText(i) || !i.containsOnlyTriviaWhiteSpaces; });
            }
            function elaborateJsxComponents(node, source, target, relation, containingMessageChain, errorOutputContainer) {
                var result = elaborateElementwise(generateJsxAttributes(node), source, target, relation, containingMessageChain, errorOutputContainer);
                var invalidTextDiagnostic;
                if (ts.isJsxOpeningElement(node.parent) && ts.isJsxElement(node.parent.parent)) {
                    var containingElement = node.parent.parent;
                    var childPropName = getJsxElementChildrenPropertyName(getJsxNamespaceAt(node));
                    var childrenPropName = childPropName === undefined ? "children" : ts.unescapeLeadingUnderscores(childPropName);
                    var childrenNameType = getLiteralType(childrenPropName);
                    var childrenTargetType = getIndexedAccessType(target, childrenNameType);
                    var validChildren = getSemanticJsxChildren(containingElement.children);
                    if (!ts.length(validChildren)) {
                        return result;
                    }
                    var moreThanOneRealChildren = ts.length(validChildren) > 1;
                    var arrayLikeTargetParts = filterType(childrenTargetType, isArrayOrTupleLikeType);
                    var nonArrayLikeTargetParts = filterType(childrenTargetType, function (t) { return !isArrayOrTupleLikeType(t); });
                    if (moreThanOneRealChildren) {
                        if (arrayLikeTargetParts !== neverType) {
                            var realSource = createTupleType(checkJsxChildren(containingElement, 0 /* Normal */));
                            var children = generateJsxChildren(containingElement, getInvalidTextualChildDiagnostic);
                            result = elaborateElementwise(children, realSource, arrayLikeTargetParts, relation, containingMessageChain, errorOutputContainer) || result;
                        }
                        else if (!isTypeRelatedTo(getIndexedAccessType(source, childrenNameType), childrenTargetType, relation)) {
                            // arity mismatch
                            result = true;
                            var diag = error(containingElement.openingElement.tagName, ts.Diagnostics.This_JSX_tag_s_0_prop_expects_a_single_child_of_type_1_but_multiple_children_were_provided, childrenPropName, typeToString(childrenTargetType));
                            if (errorOutputContainer && errorOutputContainer.skipLogging) {
                                (errorOutputContainer.errors || (errorOutputContainer.errors = [])).push(diag);
                            }
                        }
                    }
                    else {
                        if (nonArrayLikeTargetParts !== neverType) {
                            var child = validChildren[0];
                            var elem_1 = getElaborationElementForJsxChild(child, childrenNameType, getInvalidTextualChildDiagnostic);
                            if (elem_1) {
                                result = elaborateElementwise((function () {
                                    return __generator(this, function (_a) {
                                        switch (_a.label) {
                                            case 0: return [4 /*yield*/, elem_1];
                                            case 1:
                                                _a.sent();
                                                return [2 /*return*/];
                                        }
                                    });
                                })(), source, target, relation, containingMessageChain, errorOutputContainer) || result;
                            }
                        }
                        else if (!isTypeRelatedTo(getIndexedAccessType(source, childrenNameType), childrenTargetType, relation)) {
                            // arity mismatch
                            result = true;
                            var diag = error(containingElement.openingElement.tagName, ts.Diagnostics.This_JSX_tag_s_0_prop_expects_type_1_which_requires_multiple_children_but_only_a_single_child_was_provided, childrenPropName, typeToString(childrenTargetType));
                            if (errorOutputContainer && errorOutputContainer.skipLogging) {
                                (errorOutputContainer.errors || (errorOutputContainer.errors = [])).push(diag);
                            }
                        }
                    }
                }
                return result;
                function getInvalidTextualChildDiagnostic() {
                    if (!invalidTextDiagnostic) {
                        var tagNameText = ts.getTextOfNode(node.parent.tagName);
                        var childPropName = getJsxElementChildrenPropertyName(getJsxNamespaceAt(node));
                        var childrenPropName = childPropName === undefined ? "children" : ts.unescapeLeadingUnderscores(childPropName);
                        var childrenTargetType = getIndexedAccessType(target, getLiteralType(childrenPropName));
                        var diagnostic = ts.Diagnostics._0_components_don_t_accept_text_as_child_elements_Text_in_JSX_has_the_type_string_but_the_expected_type_of_1_is_2;
                        invalidTextDiagnostic = __assign(__assign({}, diagnostic), { key: "!!ALREADY FORMATTED!!", message: ts.formatMessage(/*_dummy*/ undefined, diagnostic, tagNameText, childrenPropName, typeToString(childrenTargetType)) });
                    }
                    return invalidTextDiagnostic;
                }
            }
            function generateLimitedTupleElements(node, target) {
                var len, i, elem, nameType;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            len = ts.length(node.elements);
                            if (!len)
                                return [2 /*return*/];
                            i = 0;
                            _a.label = 1;
                        case 1:
                            if (!(i < len))
                                return [3 /*break*/, 4];
                            // Skip elements which do not exist in the target - a length error on the tuple overall is likely better than an error on a mismatched index signature
                            if (isTupleLikeType(target) && !getPropertyOfType(target, ("" + i)))
                                return [3 /*break*/, 3];
                            elem = node.elements[i];
                            if (ts.isOmittedExpression(elem))
                                return [3 /*break*/, 3];
                            nameType = getLiteralType(i);
                            return [4 /*yield*/, { errorNode: elem, innerExpression: elem, nameType: nameType }];
                        case 2:
                            _a.sent();
                            _a.label = 3;
                        case 3:
                            i++;
                            return [3 /*break*/, 1];
                        case 4: return [2 /*return*/];
                    }
                });
            }
            function elaborateArrayLiteral(node, source, target, relation, containingMessageChain, errorOutputContainer) {
                if (target.flags & 131068 /* Primitive */)
                    return false;
                if (isTupleLikeType(source)) {
                    return elaborateElementwise(generateLimitedTupleElements(node, target), source, target, relation, containingMessageChain, errorOutputContainer);
                }
                // recreate a tuple from the elements, if possible
                var tupleizedType = checkArrayLiteral(node, 1 /* Contextual */, /*forceTuple*/ true);
                if (isTupleLikeType(tupleizedType)) {
                    return elaborateElementwise(generateLimitedTupleElements(node, target), tupleizedType, target, relation, containingMessageChain, errorOutputContainer);
                }
                return false;
            }
            function generateObjectLiteralElements(node) {
                var _i, _a, prop, type, _b;
                return __generator(this, function (_c) {
                    switch (_c.label) {
                        case 0:
                            if (!ts.length(node.properties))
                                return [2 /*return*/];
                            _i = 0, _a = node.properties;
                            _c.label = 1;
                        case 1:
                            if (!(_i < _a.length))
                                return [3 /*break*/, 8];
                            prop = _a[_i];
                            if (ts.isSpreadAssignment(prop))
                                return [3 /*break*/, 7];
                            type = getLiteralTypeFromProperty(getSymbolOfNode(prop), 8576 /* StringOrNumberLiteralOrUnique */);
                            if (!type || (type.flags & 131072 /* Never */)) {
                                return [3 /*break*/, 7];
                            }
                            _b = prop.kind;
                            switch (_b) {
                                case 164 /* SetAccessor */: return [3 /*break*/, 2];
                                case 163 /* GetAccessor */: return [3 /*break*/, 2];
                                case 161 /* MethodDeclaration */: return [3 /*break*/, 2];
                                case 282 /* ShorthandPropertyAssignment */: return [3 /*break*/, 2];
                                case 281 /* PropertyAssignment */: return [3 /*break*/, 4];
                            }
                            return [3 /*break*/, 6];
                        case 2: return [4 /*yield*/, { errorNode: prop.name, innerExpression: undefined, nameType: type }];
                        case 3:
                            _c.sent();
                            return [3 /*break*/, 7];
                        case 4: return [4 /*yield*/, { errorNode: prop.name, innerExpression: prop.initializer, nameType: type, errorMessage: isComputedNonLiteralName(prop.name) ? ts.Diagnostics.Type_of_computed_property_s_value_is_0_which_is_not_assignable_to_type_1 : undefined }];
                        case 5:
                            _c.sent();
                            return [3 /*break*/, 7];
                        case 6:
                            ts.Debug.assertNever(prop);
                            _c.label = 7;
                        case 7:
                            _i++;
                            return [3 /*break*/, 1];
                        case 8: return [2 /*return*/];
                    }
                });
            }
            function elaborateObjectLiteral(node, source, target, relation, containingMessageChain, errorOutputContainer) {
                if (target.flags & 131068 /* Primitive */)
                    return false;
                return elaborateElementwise(generateObjectLiteralElements(node), source, target, relation, containingMessageChain, errorOutputContainer);
            }
            /**
             * This is *not* a bi-directional relationship.
             * If one needs to check both directions for comparability, use a second call to this function or 'isTypeComparableTo'.
             */
            function checkTypeComparableTo(source, target, errorNode, headMessage, containingMessageChain) {
                return checkTypeRelatedTo(source, target, comparableRelation, errorNode, headMessage, containingMessageChain);
            }
            function isSignatureAssignableTo(source, target, ignoreReturnTypes) {
                return compareSignaturesRelated(source, target, ignoreReturnTypes ? 4 /* IgnoreReturnTypes */ : 0, /*reportErrors*/ false, 
                /*errorReporter*/ undefined, /*errorReporter*/ undefined, compareTypesAssignable, /*reportUnreliableMarkers*/ undefined) !== 0 /* False */;
            }
            /**
             * Returns true if `s` is `(...args: any[]) => any` or `(this: any, ...args: any[]) => any`
             */
            function isAnySignature(s) {
                return !s.typeParameters && (!s.thisParameter || isTypeAny(getTypeOfParameter(s.thisParameter))) && s.parameters.length === 1 &&
                    signatureHasRestParameter(s) && (getTypeOfParameter(s.parameters[0]) === anyArrayType || isTypeAny(getTypeOfParameter(s.parameters[0]))) &&
                    isTypeAny(getReturnTypeOfSignature(s));
            }
            /**
             * See signatureRelatedTo, compareSignaturesIdentical
             */
            function compareSignaturesRelated(source, target, checkMode, reportErrors, errorReporter, incompatibleErrorReporter, compareTypes, reportUnreliableMarkers) {
                // TODO (drosen): De-duplicate code between related functions.
                if (source === target) {
                    return -1 /* True */;
                }
                if (isAnySignature(target)) {
                    return -1 /* True */;
                }
                var targetCount = getParameterCount(target);
                var sourceHasMoreParameters = !hasEffectiveRestParameter(target) &&
                    (checkMode & 8 /* StrictArity */ ? hasEffectiveRestParameter(source) || getParameterCount(source) > targetCount : getMinArgumentCount(source) > targetCount);
                if (sourceHasMoreParameters) {
                    return 0 /* False */;
                }
                if (source.typeParameters && source.typeParameters !== target.typeParameters) {
                    target = getCanonicalSignature(target);
                    source = instantiateSignatureInContextOf(source, target, /*inferenceContext*/ undefined, compareTypes);
                }
                var sourceCount = getParameterCount(source);
                var sourceRestType = getNonArrayRestType(source);
                var targetRestType = getNonArrayRestType(target);
                if (sourceRestType || targetRestType) {
                    void instantiateType(sourceRestType || targetRestType, reportUnreliableMarkers);
                }
                if (sourceRestType && targetRestType && sourceCount !== targetCount) {
                    // We're not able to relate misaligned complex rest parameters
                    return 0 /* False */;
                }
                var kind = target.declaration ? target.declaration.kind : 0 /* Unknown */;
                var strictVariance = !(checkMode & 3 /* Callback */) && strictFunctionTypes && kind !== 161 /* MethodDeclaration */ &&
                    kind !== 160 /* MethodSignature */ && kind !== 162 /* Constructor */;
                var result = -1 /* True */;
                var sourceThisType = getThisTypeOfSignature(source);
                if (sourceThisType && sourceThisType !== voidType) {
                    var targetThisType = getThisTypeOfSignature(target);
                    if (targetThisType) {
                        // void sources are assignable to anything.
                        var related = !strictVariance && compareTypes(sourceThisType, targetThisType, /*reportErrors*/ false)
                            || compareTypes(targetThisType, sourceThisType, reportErrors);
                        if (!related) {
                            if (reportErrors) {
                                errorReporter(ts.Diagnostics.The_this_types_of_each_signature_are_incompatible);
                            }
                            return 0 /* False */;
                        }
                        result &= related;
                    }
                }
                var paramCount = sourceRestType || targetRestType ? Math.min(sourceCount, targetCount) : Math.max(sourceCount, targetCount);
                var restIndex = sourceRestType || targetRestType ? paramCount - 1 : -1;
                for (var i = 0; i < paramCount; i++) {
                    var sourceType = i === restIndex ? getRestTypeAtPosition(source, i) : getTypeAtPosition(source, i);
                    var targetType = i === restIndex ? getRestTypeAtPosition(target, i) : getTypeAtPosition(target, i);
                    // In order to ensure that any generic type Foo<T> is at least co-variant with respect to T no matter
                    // how Foo uses T, we need to relate parameters bi-variantly (given that parameters are input positions,
                    // they naturally relate only contra-variantly). However, if the source and target parameters both have
                    // function types with a single call signature, we know we are relating two callback parameters. In
                    // that case it is sufficient to only relate the parameters of the signatures co-variantly because,
                    // similar to return values, callback parameters are output positions. This means that a Promise<T>,
                    // where T is used only in callback parameter positions, will be co-variant (as opposed to bi-variant)
                    // with respect to T.
                    var sourceSig = checkMode & 3 /* Callback */ ? undefined : getSingleCallSignature(getNonNullableType(sourceType));
                    var targetSig = checkMode & 3 /* Callback */ ? undefined : getSingleCallSignature(getNonNullableType(targetType));
                    var callbacks = sourceSig && targetSig && !getTypePredicateOfSignature(sourceSig) && !getTypePredicateOfSignature(targetSig) &&
                        (getFalsyFlags(sourceType) & 98304 /* Nullable */) === (getFalsyFlags(targetType) & 98304 /* Nullable */);
                    var related = callbacks ?
                        compareSignaturesRelated(targetSig, sourceSig, (checkMode & 8 /* StrictArity */) | (strictVariance ? 2 /* StrictCallback */ : 1 /* BivariantCallback */), reportErrors, errorReporter, incompatibleErrorReporter, compareTypes, reportUnreliableMarkers) :
                        !(checkMode & 3 /* Callback */) && !strictVariance && compareTypes(sourceType, targetType, /*reportErrors*/ false) || compareTypes(targetType, sourceType, reportErrors);
                    // With strict arity, (x: number | undefined) => void is a subtype of (x?: number | undefined) => void
                    if (related && checkMode & 8 /* StrictArity */ && i >= getMinArgumentCount(source) && i < getMinArgumentCount(target) && compareTypes(sourceType, targetType, /*reportErrors*/ false)) {
                        related = 0 /* False */;
                    }
                    if (!related) {
                        if (reportErrors) {
                            errorReporter(ts.Diagnostics.Types_of_parameters_0_and_1_are_incompatible, ts.unescapeLeadingUnderscores(getParameterNameAtPosition(source, i)), ts.unescapeLeadingUnderscores(getParameterNameAtPosition(target, i)));
                        }
                        return 0 /* False */;
                    }
                    result &= related;
                }
                if (!(checkMode & 4 /* IgnoreReturnTypes */)) {
                    // If a signature resolution is already in-flight, skip issuing a circularity error
                    // here and just use the `any` type directly
                    var targetReturnType = isResolvingReturnTypeOfSignature(target) ? anyType
                        : target.declaration && isJSConstructor(target.declaration) ? getDeclaredTypeOfClassOrInterface(getMergedSymbol(target.declaration.symbol))
                            : getReturnTypeOfSignature(target);
                    if (targetReturnType === voidType) {
                        return result;
                    }
                    var sourceReturnType = isResolvingReturnTypeOfSignature(source) ? anyType
                        : source.declaration && isJSConstructor(source.declaration) ? getDeclaredTypeOfClassOrInterface(getMergedSymbol(source.declaration.symbol))
                            : getReturnTypeOfSignature(source);
                    // The following block preserves behavior forbidding boolean returning functions from being assignable to type guard returning functions
                    var targetTypePredicate = getTypePredicateOfSignature(target);
                    if (targetTypePredicate) {
                        var sourceTypePredicate = getTypePredicateOfSignature(source);
                        if (sourceTypePredicate) {
                            result &= compareTypePredicateRelatedTo(sourceTypePredicate, targetTypePredicate, reportErrors, errorReporter, compareTypes);
                        }
                        else if (ts.isIdentifierTypePredicate(targetTypePredicate)) {
                            if (reportErrors) {
                                errorReporter(ts.Diagnostics.Signature_0_must_be_a_type_predicate, signatureToString(source));
                            }
                            return 0 /* False */;
                        }
                    }
                    else {
                        // When relating callback signatures, we still need to relate return types bi-variantly as otherwise
                        // the containing type wouldn't be co-variant. For example, interface Foo<T> { add(cb: () => T): void }
                        // wouldn't be co-variant for T without this rule.
                        result &= checkMode & 1 /* BivariantCallback */ && compareTypes(targetReturnType, sourceReturnType, /*reportErrors*/ false) ||
                            compareTypes(sourceReturnType, targetReturnType, reportErrors);
                        if (!result && reportErrors && incompatibleErrorReporter) {
                            incompatibleErrorReporter(sourceReturnType, targetReturnType);
                        }
                    }
                }
                return result;
            }
            function compareTypePredicateRelatedTo(source, target, reportErrors, errorReporter, compareTypes) {
                if (source.kind !== target.kind) {
                    if (reportErrors) {
                        errorReporter(ts.Diagnostics.A_this_based_type_guard_is_not_compatible_with_a_parameter_based_type_guard);
                        errorReporter(ts.Diagnostics.Type_predicate_0_is_not_assignable_to_1, typePredicateToString(source), typePredicateToString(target));
                    }
                    return 0 /* False */;
                }
                if (source.kind === 1 /* Identifier */ || source.kind === 3 /* AssertsIdentifier */) {
                    if (source.parameterIndex !== target.parameterIndex) {
                        if (reportErrors) {
                            errorReporter(ts.Diagnostics.Parameter_0_is_not_in_the_same_position_as_parameter_1, source.parameterName, target.parameterName);
                            errorReporter(ts.Diagnostics.Type_predicate_0_is_not_assignable_to_1, typePredicateToString(source), typePredicateToString(target));
                        }
                        return 0 /* False */;
                    }
                }
                var related = source.type === target.type ? -1 /* True */ :
                    source.type && target.type ? compareTypes(source.type, target.type, reportErrors) :
                        0 /* False */;
                if (related === 0 /* False */ && reportErrors) {
                    errorReporter(ts.Diagnostics.Type_predicate_0_is_not_assignable_to_1, typePredicateToString(source), typePredicateToString(target));
                }
                return related;
            }
            function isImplementationCompatibleWithOverload(implementation, overload) {
                var erasedSource = getErasedSignature(implementation);
                var erasedTarget = getErasedSignature(overload);
                // First see if the return types are compatible in either direction.
                var sourceReturnType = getReturnTypeOfSignature(erasedSource);
                var targetReturnType = getReturnTypeOfSignature(erasedTarget);
                if (targetReturnType === voidType
                    || isTypeRelatedTo(targetReturnType, sourceReturnType, assignableRelation)
                    || isTypeRelatedTo(sourceReturnType, targetReturnType, assignableRelation)) {
                    return isSignatureAssignableTo(erasedSource, erasedTarget, /*ignoreReturnTypes*/ true);
                }
                return false;
            }
            function isEmptyResolvedType(t) {
                return t !== anyFunctionType &&
                    t.properties.length === 0 &&
                    t.callSignatures.length === 0 &&
                    t.constructSignatures.length === 0 &&
                    !t.stringIndexInfo &&
                    !t.numberIndexInfo;
            }
            function isEmptyObjectType(type) {
                return type.flags & 524288 /* Object */ ? !isGenericMappedType(type) && isEmptyResolvedType(resolveStructuredTypeMembers(type)) :
                    type.flags & 67108864 /* NonPrimitive */ ? true :
                        type.flags & 1048576 /* Union */ ? ts.some(type.types, isEmptyObjectType) :
                            type.flags & 2097152 /* Intersection */ ? ts.every(type.types, isEmptyObjectType) :
                                false;
            }
            function isEmptyAnonymousObjectType(type) {
                return !!(ts.getObjectFlags(type) & 16 /* Anonymous */) && isEmptyObjectType(type);
            }
            function isStringIndexSignatureOnlyType(type) {
                return type.flags & 524288 /* Object */ && !isGenericMappedType(type) && getPropertiesOfType(type).length === 0 && getIndexInfoOfType(type, 0 /* String */) && !getIndexInfoOfType(type, 1 /* Number */) ||
                    type.flags & 3145728 /* UnionOrIntersection */ && ts.every(type.types, isStringIndexSignatureOnlyType) ||
                    false;
            }
            function isEnumTypeRelatedTo(sourceSymbol, targetSymbol, errorReporter) {
                if (sourceSymbol === targetSymbol) {
                    return true;
                }
                var id = getSymbolId(sourceSymbol) + "," + getSymbolId(targetSymbol);
                var entry = enumRelation.get(id);
                if (entry !== undefined && !(!(entry & 4 /* Reported */) && entry & 2 /* Failed */ && errorReporter)) {
                    return !!(entry & 1 /* Succeeded */);
                }
                if (sourceSymbol.escapedName !== targetSymbol.escapedName || !(sourceSymbol.flags & 256 /* RegularEnum */) || !(targetSymbol.flags & 256 /* RegularEnum */)) {
                    enumRelation.set(id, 2 /* Failed */ | 4 /* Reported */);
                    return false;
                }
                var targetEnumType = getTypeOfSymbol(targetSymbol);
                for (var _i = 0, _a = getPropertiesOfType(getTypeOfSymbol(sourceSymbol)); _i < _a.length; _i++) {
                    var property = _a[_i];
                    if (property.flags & 8 /* EnumMember */) {
                        var targetProperty = getPropertyOfType(targetEnumType, property.escapedName);
                        if (!targetProperty || !(targetProperty.flags & 8 /* EnumMember */)) {
                            if (errorReporter) {
                                errorReporter(ts.Diagnostics.Property_0_is_missing_in_type_1, ts.symbolName(property), typeToString(getDeclaredTypeOfSymbol(targetSymbol), /*enclosingDeclaration*/ undefined, 64 /* UseFullyQualifiedType */));
                                enumRelation.set(id, 2 /* Failed */ | 4 /* Reported */);
                            }
                            else {
                                enumRelation.set(id, 2 /* Failed */);
                            }
                            return false;
                        }
                    }
                }
                enumRelation.set(id, 1 /* Succeeded */);
                return true;
            }
            function isSimpleTypeRelatedTo(source, target, relation, errorReporter) {
                var s = source.flags;
                var t = target.flags;
                if (t & 3 /* AnyOrUnknown */ || s & 131072 /* Never */ || source === wildcardType)
                    return true;
                if (t & 131072 /* Never */)
                    return false;
                if (s & 132 /* StringLike */ && t & 4 /* String */)
                    return true;
                if (s & 128 /* StringLiteral */ && s & 1024 /* EnumLiteral */ &&
                    t & 128 /* StringLiteral */ && !(t & 1024 /* EnumLiteral */) &&
                    source.value === target.value)
                    return true;
                if (s & 296 /* NumberLike */ && t & 8 /* Number */)
                    return true;
                if (s & 256 /* NumberLiteral */ && s & 1024 /* EnumLiteral */ &&
                    t & 256 /* NumberLiteral */ && !(t & 1024 /* EnumLiteral */) &&
                    source.value === target.value)
                    return true;
                if (s & 2112 /* BigIntLike */ && t & 64 /* BigInt */)
                    return true;
                if (s & 528 /* BooleanLike */ && t & 16 /* Boolean */)
                    return true;
                if (s & 12288 /* ESSymbolLike */ && t & 4096 /* ESSymbol */)
                    return true;
                if (s & 32 /* Enum */ && t & 32 /* Enum */ && isEnumTypeRelatedTo(source.symbol, target.symbol, errorReporter))
                    return true;
                if (s & 1024 /* EnumLiteral */ && t & 1024 /* EnumLiteral */) {
                    if (s & 1048576 /* Union */ && t & 1048576 /* Union */ && isEnumTypeRelatedTo(source.symbol, target.symbol, errorReporter))
                        return true;
                    if (s & 2944 /* Literal */ && t & 2944 /* Literal */ &&
                        source.value === target.value &&
                        isEnumTypeRelatedTo(getParentOfSymbol(source.symbol), getParentOfSymbol(target.symbol), errorReporter))
                        return true;
                }
                if (s & 32768 /* Undefined */ && (!strictNullChecks || t & (32768 /* Undefined */ | 16384 /* Void */)))
                    return true;
                if (s & 65536 /* Null */ && (!strictNullChecks || t & 65536 /* Null */))
                    return true;
                if (s & 524288 /* Object */ && t & 67108864 /* NonPrimitive */)
                    return true;
                if (relation === assignableRelation || relation === comparableRelation) {
                    if (s & 1 /* Any */)
                        return true;
                    // Type number or any numeric literal type is assignable to any numeric enum type or any
                    // numeric enum literal type. This rule exists for backwards compatibility reasons because
                    // bit-flag enum types sometimes look like literal enum types with numeric literal values.
                    if (s & (8 /* Number */ | 256 /* NumberLiteral */) && !(s & 1024 /* EnumLiteral */) && (t & 32 /* Enum */ || t & 256 /* NumberLiteral */ && t & 1024 /* EnumLiteral */))
                        return true;
                }
                return false;
            }
            function isTypeRelatedTo(source, target, relation) {
                if (isFreshLiteralType(source)) {
                    source = source.regularType;
                }
                if (isFreshLiteralType(target)) {
                    target = target.regularType;
                }
                if (source === target ||
                    relation === comparableRelation && !(target.flags & 131072 /* Never */) && isSimpleTypeRelatedTo(target, source, relation) ||
                    relation !== identityRelation && isSimpleTypeRelatedTo(source, target, relation)) {
                    return true;
                }
                if (source.flags & 524288 /* Object */ && target.flags & 524288 /* Object */) {
                    var related = relation.get(getRelationKey(source, target, 0 /* None */, relation));
                    if (related !== undefined) {
                        return !!(related & 1 /* Succeeded */);
                    }
                }
                if (source.flags & 66846720 /* StructuredOrInstantiable */ || target.flags & 66846720 /* StructuredOrInstantiable */) {
                    return checkTypeRelatedTo(source, target, relation, /*errorNode*/ undefined);
                }
                return false;
            }
            function isIgnoredJsxProperty(source, sourceProp) {
                return ts.getObjectFlags(source) & 4096 /* JsxAttributes */ && !isUnhyphenatedJsxName(sourceProp.escapedName);
            }
            function getNormalizedType(type, writing) {
                return isFreshLiteralType(type) ? type.regularType :
                    ts.getObjectFlags(type) & 4 /* Reference */ && type.node ? createTypeReference(type.target, getTypeArguments(type)) :
                        type.flags & 33554432 /* Substitution */ ? writing ? type.typeVariable : type.substitute :
                            type.flags & 25165824 /* Simplifiable */ ? getSimplifiedType(type, writing) :
                                type;
            }
            /**
             * Checks if 'source' is related to 'target' (e.g.: is a assignable to).
             * @param source The left-hand-side of the relation.
             * @param target The right-hand-side of the relation.
             * @param relation The relation considered. One of 'identityRelation', 'subtypeRelation', 'assignableRelation', or 'comparableRelation'.
             * Used as both to determine which checks are performed and as a cache of previously computed results.
             * @param errorNode The suggested node upon which all errors will be reported, if defined. This may or may not be the actual node used.
             * @param headMessage If the error chain should be prepended by a head message, then headMessage will be used.
             * @param containingMessageChain A chain of errors to prepend any new errors found.
             * @param errorOutputContainer Return the diagnostic. Do not log if 'skipLogging' is truthy.
             */
            function checkTypeRelatedTo(source, target, relation, errorNode, headMessage, containingMessageChain, errorOutputContainer) {
                var errorInfo;
                var relatedInfo;
                var maybeKeys;
                var sourceStack;
                var targetStack;
                var maybeCount = 0;
                var depth = 0;
                var expandingFlags = 0 /* None */;
                var overflow = false;
                var overrideNextErrorInfo = 0; // How many `reportRelationError` calls should be skipped in the elaboration pyramid
                var lastSkippedInfo;
                var incompatibleStack = [];
                ts.Debug.assert(relation !== identityRelation || !errorNode, "no error reporting in identity checking");
                var result = isRelatedTo(source, target, /*reportErrors*/ !!errorNode, headMessage);
                if (incompatibleStack.length) {
                    reportIncompatibleStack();
                }
                if (overflow) {
                    var diag = error(errorNode, ts.Diagnostics.Excessive_stack_depth_comparing_types_0_and_1, typeToString(source), typeToString(target));
                    if (errorOutputContainer) {
                        (errorOutputContainer.errors || (errorOutputContainer.errors = [])).push(diag);
                    }
                }
                else if (errorInfo) {
                    if (containingMessageChain) {
                        var chain = containingMessageChain();
                        if (chain) {
                            ts.concatenateDiagnosticMessageChains(chain, errorInfo);
                            errorInfo = chain;
                        }
                    }
                    var relatedInformation = void 0;
                    // Check if we should issue an extra diagnostic to produce a quickfix for a slightly incorrect import statement
                    if (headMessage && errorNode && !result && source.symbol) {
                        var links = getSymbolLinks(source.symbol);
                        if (links.originatingImport && !ts.isImportCall(links.originatingImport)) {
                            var helpfulRetry = checkTypeRelatedTo(getTypeOfSymbol(links.target), target, relation, /*errorNode*/ undefined);
                            if (helpfulRetry) {
                                // Likely an incorrect import. Issue a helpful diagnostic to produce a quickfix to change the import
                                var diag_1 = ts.createDiagnosticForNode(links.originatingImport, ts.Diagnostics.Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cause_a_failure_at_runtime_Consider_using_a_default_import_or_import_require_here_instead);
                                relatedInformation = ts.append(relatedInformation, diag_1); // Cause the error to appear with the error that triggered it
                            }
                        }
                    }
                    var diag = ts.createDiagnosticForNodeFromMessageChain(errorNode, errorInfo, relatedInformation);
                    if (relatedInfo) {
                        ts.addRelatedInfo.apply(void 0, __spreadArrays([diag], relatedInfo));
                    }
                    if (errorOutputContainer) {
                        (errorOutputContainer.errors || (errorOutputContainer.errors = [])).push(diag);
                    }
                    if (!errorOutputContainer || !errorOutputContainer.skipLogging) {
                        diagnostics.add(diag);
                    }
                }
                if (errorNode && errorOutputContainer && errorOutputContainer.skipLogging && result === 0 /* False */) {
                    ts.Debug.assert(!!errorOutputContainer.errors, "missed opportunity to interact with error.");
                }
                return result !== 0 /* False */;
                function resetErrorInfo(saved) {
                    errorInfo = saved.errorInfo;
                    lastSkippedInfo = saved.lastSkippedInfo;
                    incompatibleStack = saved.incompatibleStack;
                    overrideNextErrorInfo = saved.overrideNextErrorInfo;
                    relatedInfo = saved.relatedInfo;
                }
                function captureErrorCalculationState() {
                    return {
                        errorInfo: errorInfo,
                        lastSkippedInfo: lastSkippedInfo,
                        incompatibleStack: incompatibleStack.slice(),
                        overrideNextErrorInfo: overrideNextErrorInfo,
                        relatedInfo: !relatedInfo ? undefined : relatedInfo.slice()
                    };
                }
                function reportIncompatibleError(message, arg0, arg1, arg2, arg3) {
                    overrideNextErrorInfo++; // Suppress the next relation error
                    lastSkippedInfo = undefined; // Reset skipped info cache
                    incompatibleStack.push([message, arg0, arg1, arg2, arg3]);
                }
                function reportIncompatibleStack() {
                    var stack = incompatibleStack;
                    incompatibleStack = [];
                    var info = lastSkippedInfo;
                    lastSkippedInfo = undefined;
                    if (stack.length === 1) {
                        reportError.apply(void 0, stack[0]);
                        if (info) {
                            // Actually do the last relation error
                            reportRelationError.apply(void 0, __spreadArrays([/*headMessage*/ undefined], info));
                        }
                        return;
                    }
                    // The first error will be the innermost, while the last will be the outermost - so by popping off the end,
                    // we can build from left to right
                    var path = "";
                    var secondaryRootErrors = [];
                    while (stack.length) {
                        var _a = stack.pop(), msg = _a[0], args = _a.slice(1);
                        switch (msg.code) {
                            case ts.Diagnostics.Types_of_property_0_are_incompatible.code: {
                                // Parenthesize a `new` if there is one
                                if (path.indexOf("new ") === 0) {
                                    path = "(" + path + ")";
                                }
                                var str = "" + args[0];
                                // If leading, just print back the arg (irrespective of if it's a valid identifier)
                                if (path.length === 0) {
                                    path = "" + str;
                                }
                                // Otherwise write a dotted name if possible
                                else if (ts.isIdentifierText(str, compilerOptions.target)) {
                                    path = path + "." + str;
                                }
                                // Failing that, check if the name is already a computed name
                                else if (str[0] === "[" && str[str.length - 1] === "]") {
                                    path = "" + path + str;
                                }
                                // And finally write out a computed name as a last resort
                                else {
                                    path = path + "[" + str + "]";
                                }
                                break;
                            }
                            case ts.Diagnostics.Call_signature_return_types_0_and_1_are_incompatible.code:
                            case ts.Diagnostics.Construct_signature_return_types_0_and_1_are_incompatible.code:
                            case ts.Diagnostics.Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code:
                            case ts.Diagnostics.Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code: {
                                if (path.length === 0) {
                                    // Don't flatten signature compatability errors at the start of a chain - instead prefer
                                    // to unify (the with no arguments bit is excessive for printback) and print them back
                                    var mappedMsg = msg;
                                    if (msg.code === ts.Diagnostics.Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code) {
                                        mappedMsg = ts.Diagnostics.Call_signature_return_types_0_and_1_are_incompatible;
                                    }
                                    else if (msg.code === ts.Diagnostics.Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code) {
                                        mappedMsg = ts.Diagnostics.Construct_signature_return_types_0_and_1_are_incompatible;
                                    }
                                    secondaryRootErrors.unshift([mappedMsg, args[0], args[1]]);
                                }
                                else {
                                    var prefix = (msg.code === ts.Diagnostics.Construct_signature_return_types_0_and_1_are_incompatible.code ||
                                        msg.code === ts.Diagnostics.Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code)
                                        ? "new "
                                        : "";
                                    var params = (msg.code === ts.Diagnostics.Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code ||
                                        msg.code === ts.Diagnostics.Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1.code)
                                        ? ""
                                        : "...";
                                    path = "" + prefix + path + "(" + params + ")";
                                }
                                break;
                            }
                            default:
                                return ts.Debug.fail("Unhandled Diagnostic: " + msg.code);
                        }
                    }
                    if (path) {
                        reportError(path[path.length - 1] === ")"
                            ? ts.Diagnostics.The_types_returned_by_0_are_incompatible_between_these_types
                            : ts.Diagnostics.The_types_of_0_are_incompatible_between_these_types, path);
                    }
                    else {
                        // Remove the innermost secondary error as it will duplicate the error already reported by `reportRelationError` on entry
                        secondaryRootErrors.shift();
                    }
                    for (var _i = 0, secondaryRootErrors_1 = secondaryRootErrors; _i < secondaryRootErrors_1.length; _i++) {
                        var _b = secondaryRootErrors_1[_i], msg = _b[0], args = _b.slice(1);
                        var originalValue = msg.elidedInCompatabilityPyramid;
                        msg.elidedInCompatabilityPyramid = false; // Teporarily override elision to ensure error is reported
                        reportError.apply(void 0, __spreadArrays([msg], args));
                        msg.elidedInCompatabilityPyramid = originalValue;
                    }
                    if (info) {
                        // Actually do the last relation error
                        reportRelationError.apply(void 0, __spreadArrays([/*headMessage*/ undefined], info));
                    }
                }
                function reportError(message, arg0, arg1, arg2, arg3) {
                    ts.Debug.assert(!!errorNode);
                    if (incompatibleStack.length)
                        reportIncompatibleStack();
                    if (message.elidedInCompatabilityPyramid)
                        return;
                    errorInfo = ts.chainDiagnosticMessages(errorInfo, message, arg0, arg1, arg2, arg3);
                }
                function associateRelatedInfo(info) {
                    ts.Debug.assert(!!errorInfo);
                    if (!relatedInfo) {
                        relatedInfo = [info];
                    }
                    else {
                        relatedInfo.push(info);
                    }
                }
                function reportRelationError(message, source, target) {
                    if (incompatibleStack.length)
                        reportIncompatibleStack();
                    var _a = getTypeNamesForErrorDisplay(source, target), sourceType = _a[0], targetType = _a[1];
                    if (target.flags & 262144 /* TypeParameter */ && target.immediateBaseConstraint !== undefined && isTypeAssignableTo(source, target.immediateBaseConstraint)) {
                        reportError(ts.Diagnostics._0_is_assignable_to_the_constraint_of_type_1_but_1_could_be_instantiated_with_a_different_subtype_of_constraint_2, sourceType, targetType, typeToString(target.immediateBaseConstraint));
                    }
                    if (!message) {
                        if (relation === comparableRelation) {
                            message = ts.Diagnostics.Type_0_is_not_comparable_to_type_1;
                        }
                        else if (sourceType === targetType) {
                            message = ts.Diagnostics.Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated;
                        }
                        else {
                            message = ts.Diagnostics.Type_0_is_not_assignable_to_type_1;
                        }
                    }
                    reportError(message, sourceType, targetType);
                }
                function tryElaborateErrorsForPrimitivesAndObjects(source, target) {
                    var sourceType = symbolValueDeclarationIsContextSensitive(source.symbol) ? typeToString(source, source.symbol.valueDeclaration) : typeToString(source);
                    var targetType = symbolValueDeclarationIsContextSensitive(target.symbol) ? typeToString(target, target.symbol.valueDeclaration) : typeToString(target);
                    if ((globalStringType === source && stringType === target) ||
                        (globalNumberType === source && numberType === target) ||
                        (globalBooleanType === source && booleanType === target) ||
                        (getGlobalESSymbolType(/*reportErrors*/ false) === source && esSymbolType === target)) {
                        reportError(ts.Diagnostics._0_is_a_primitive_but_1_is_a_wrapper_object_Prefer_using_0_when_possible, targetType, sourceType);
                    }
                }
                /**
                 * Try and elaborate array and tuple errors. Returns false
                 * if we have found an elaboration, or we should ignore
                 * any other elaborations when relating the `source` and
                 * `target` types.
                 */
                function tryElaborateArrayLikeErrors(source, target, reportErrors) {
                    /**
                     * The spec for elaboration is:
                     * - If the source is a readonly tuple and the target is a mutable array or tuple, elaborate on mutability and skip property elaborations.
                     * - If the source is a tuple then skip property elaborations if the target is an array or tuple.
                     * - If the source is a readonly array and the target is a mutable array or tuple, elaborate on mutability and skip property elaborations.
                     * - If the source an array then skip property elaborations if the target is a tuple.
                     */
                    if (isTupleType(source)) {
                        if (source.target.readonly && isMutableArrayOrTuple(target)) {
                            if (reportErrors) {
                                reportError(ts.Diagnostics.The_type_0_is_readonly_and_cannot_be_assigned_to_the_mutable_type_1, typeToString(source), typeToString(target));
                            }
                            return false;
                        }
                        return isTupleType(target) || isArrayType(target);
                    }
                    if (isReadonlyArrayType(source) && isMutableArrayOrTuple(target)) {
                        if (reportErrors) {
                            reportError(ts.Diagnostics.The_type_0_is_readonly_and_cannot_be_assigned_to_the_mutable_type_1, typeToString(source), typeToString(target));
                        }
                        return false;
                    }
                    if (isTupleType(target)) {
                        return isArrayType(source);
                    }
                    return true;
                }
                /**
                 * Compare two types and return
                 * * Ternary.True if they are related with no assumptions,
                 * * Ternary.Maybe if they are related with assumptions of other relationships, or
                 * * Ternary.False if they are not related.
                 */
                function isRelatedTo(originalSource, originalTarget, reportErrors, headMessage, intersectionState) {
                    if (reportErrors === void 0) {
                        reportErrors = false;
                    }
                    if (intersectionState === void 0) {
                        intersectionState = 0 /* None */;
                    }
                    // Normalize the source and target types: Turn fresh literal types into regular literal types,
                    // turn deferred type references into regular type references, simplify indexed access and
                    // conditional types, and resolve substitution types to either the substitution (on the source
                    // side) or the type variable (on the target side).
                    var source = getNormalizedType(originalSource, /*writing*/ false);
                    var target = getNormalizedType(originalTarget, /*writing*/ true);
                    // Try to see if we're relating something like `Foo` -> `Bar | null | undefined`.
                    // If so, reporting the `null` and `undefined` in the type is hardly useful.
                    // First, see if we're even relating an object type to a union.
                    // Then see if the target is stripped down to a single non-union type.
                    // Note
                    //  * We actually want to remove null and undefined naively here (rather than using getNonNullableType),
                    //    since we don't want to end up with a worse error like "`Foo` is not assignable to `NonNullable<T>`"
                    //    when dealing with generics.
                    //  * We also don't deal with primitive source types, since we already halt elaboration below.
                    if (target.flags & 1048576 /* Union */ && source.flags & 524288 /* Object */ &&
                        target.types.length <= 3 && maybeTypeOfKind(target, 98304 /* Nullable */)) {
                        var nullStrippedTarget = extractTypesOfKind(target, ~98304 /* Nullable */);
                        if (!(nullStrippedTarget.flags & (1048576 /* Union */ | 131072 /* Never */))) {
                            target = nullStrippedTarget;
                        }
                    }
                    // both types are the same - covers 'they are the same primitive type or both are Any' or the same type parameter cases
                    if (source === target)
                        return -1 /* True */;
                    if (relation === identityRelation) {
                        return isIdenticalTo(source, target);
                    }
                    if (relation === comparableRelation && !(target.flags & 131072 /* Never */) && isSimpleTypeRelatedTo(target, source, relation) ||
                        isSimpleTypeRelatedTo(source, target, relation, reportErrors ? reportError : undefined))
                        return -1 /* True */;
                    var isComparingJsxAttributes = !!(ts.getObjectFlags(source) & 4096 /* JsxAttributes */);
                    var isPerformingExcessPropertyChecks = !(intersectionState & 2 /* Target */) && (isObjectLiteralType(source) && ts.getObjectFlags(source) & 32768 /* FreshLiteral */);
                    if (isPerformingExcessPropertyChecks) {
                        if (hasExcessProperties(source, target, reportErrors)) {
                            if (reportErrors) {
                                reportRelationError(headMessage, source, target);
                            }
                            return 0 /* False */;
                        }
                    }
                    var isPerformingCommonPropertyChecks = relation !== comparableRelation && !(intersectionState & 2 /* Target */) &&
                        source.flags & (131068 /* Primitive */ | 524288 /* Object */ | 2097152 /* Intersection */) && source !== globalObjectType &&
                        target.flags & (524288 /* Object */ | 2097152 /* Intersection */) && isWeakType(target) &&
                        (getPropertiesOfType(source).length > 0 || typeHasCallOrConstructSignatures(source));
                    if (isPerformingCommonPropertyChecks && !hasCommonProperties(source, target, isComparingJsxAttributes)) {
                        if (reportErrors) {
                            var calls = getSignaturesOfType(source, 0 /* Call */);
                            var constructs = getSignaturesOfType(source, 1 /* Construct */);
                            if (calls.length > 0 && isRelatedTo(getReturnTypeOfSignature(calls[0]), target, /*reportErrors*/ false) ||
                                constructs.length > 0 && isRelatedTo(getReturnTypeOfSignature(constructs[0]), target, /*reportErrors*/ false)) {
                                reportError(ts.Diagnostics.Value_of_type_0_has_no_properties_in_common_with_type_1_Did_you_mean_to_call_it, typeToString(source), typeToString(target));
                            }
                            else {
                                reportError(ts.Diagnostics.Type_0_has_no_properties_in_common_with_type_1, typeToString(source), typeToString(target));
                            }
                        }
                        return 0 /* False */;
                    }
                    var result = 0 /* False */;
                    var saveErrorInfo = captureErrorCalculationState();
                    // Note that these checks are specifically ordered to produce correct results. In particular,
                    // we need to deconstruct unions before intersections (because unions are always at the top),
                    // and we need to handle "each" relations before "some" relations for the same kind of type.
                    if (source.flags & 1048576 /* Union */) {
                        result = relation === comparableRelation ?
                            someTypeRelatedToType(source, target, reportErrors && !(source.flags & 131068 /* Primitive */), intersectionState) :
                            eachTypeRelatedToType(source, target, reportErrors && !(source.flags & 131068 /* Primitive */));
                    }
                    else {
                        if (target.flags & 1048576 /* Union */) {
                            result = typeRelatedToSomeType(getRegularTypeOfObjectLiteral(source), target, reportErrors && !(source.flags & 131068 /* Primitive */) && !(target.flags & 131068 /* Primitive */));
                        }
                        else if (target.flags & 2097152 /* Intersection */) {
                            result = typeRelatedToEachType(getRegularTypeOfObjectLiteral(source), target, reportErrors, 2 /* Target */);
                            if (result && (isPerformingExcessPropertyChecks || isPerformingCommonPropertyChecks)) {
                                // Validate against excess props using the original `source`
                                if (!propertiesRelatedTo(source, target, reportErrors, /*excludedProperties*/ undefined, 0 /* None */)) {
                                    return 0 /* False */;
                                }
                            }
                        }
                        else if (source.flags & 2097152 /* Intersection */) {
                            // Check to see if any constituents of the intersection are immediately related to the target.
                            //
                            // Don't report errors though. Checking whether a constituent is related to the source is not actually
                            // useful and leads to some confusing error messages. Instead it is better to let the below checks
                            // take care of this, or to not elaborate at all. For instance,
                            //
                            //    - For an object type (such as 'C = A & B'), users are usually more interested in structural errors.
                            //
                            //    - For a union type (such as '(A | B) = (C & D)'), it's better to hold onto the whole intersection
                            //          than to report that 'D' is not assignable to 'A' or 'B'.
                            //
                            //    - For a primitive type or type parameter (such as 'number = A & B') there is no point in
                            //          breaking the intersection apart.
                            result = someTypeRelatedToType(source, target, /*reportErrors*/ false, 1 /* Source */);
                        }
                        if (!result && (source.flags & 66846720 /* StructuredOrInstantiable */ || target.flags & 66846720 /* StructuredOrInstantiable */)) {
                            if (result = recursiveTypeRelatedTo(source, target, reportErrors, intersectionState)) {
                                resetErrorInfo(saveErrorInfo);
                            }
                        }
                    }
                    if (!result && source.flags & (2097152 /* Intersection */ | 262144 /* TypeParameter */)) {
                        // The combined constraint of an intersection type is the intersection of the constraints of
                        // the constituents. When an intersection type contains instantiable types with union type
                        // constraints, there are situations where we need to examine the combined constraint. One is
                        // when the target is a union type. Another is when the intersection contains types belonging
                        // to one of the disjoint domains. For example, given type variables T and U, each with the
                        // constraint 'string | number', the combined constraint of 'T & U' is 'string | number' and
                        // we need to check this constraint against a union on the target side. Also, given a type
                        // variable V constrained to 'string | number', 'V & number' has a combined constraint of
                        // 'string & number | number & number' which reduces to just 'number'.
                        // This also handles type parameters, as a type parameter with a union constraint compared against a union
                        // needs to have its constraint hoisted into an intersection with said type parameter, this way
                        // the type param can be compared with itself in the target (with the influence of its constraint to match other parts)
                        // For example, if `T extends 1 | 2` and `U extends 2 | 3` and we compare `T & U` to `T & U & (1 | 2 | 3)`
                        var constraint = getEffectiveConstraintOfIntersection(source.flags & 2097152 /* Intersection */ ? source.types : [source], !!(target.flags & 1048576 /* Union */));
                        if (constraint && (source.flags & 2097152 /* Intersection */ || target.flags & 1048576 /* Union */)) {
                            if (everyType(constraint, function (c) { return c !== source; })) { // Skip comparison if expansion contains the source itself
                                // TODO: Stack errors so we get a pyramid for the "normal" comparison above, _and_ a second for this
                                if (result = isRelatedTo(constraint, target, /*reportErrors*/ false, /*headMessage*/ undefined, intersectionState)) {
                                    resetErrorInfo(saveErrorInfo);
                                }
                            }
                        }
                    }
                    if (!result && reportErrors) {
                        source = originalSource.aliasSymbol ? originalSource : source;
                        target = originalTarget.aliasSymbol ? originalTarget : target;
                        var maybeSuppress = overrideNextErrorInfo > 0;
                        if (maybeSuppress) {
                            overrideNextErrorInfo--;
                        }
                        if (source.flags & 524288 /* Object */ && target.flags & 524288 /* Object */) {
                            var currentError = errorInfo;
                            tryElaborateArrayLikeErrors(source, target, reportErrors);
                            if (errorInfo !== currentError) {
                                maybeSuppress = !!errorInfo;
                            }
                        }
                        if (source.flags & 524288 /* Object */ && target.flags & 131068 /* Primitive */) {
                            tryElaborateErrorsForPrimitivesAndObjects(source, target);
                        }
                        else if (source.symbol && source.flags & 524288 /* Object */ && globalObjectType === source) {
                            reportError(ts.Diagnostics.The_Object_type_is_assignable_to_very_few_other_types_Did_you_mean_to_use_the_any_type_instead);
                        }
                        else if (isComparingJsxAttributes && target.flags & 2097152 /* Intersection */) {
                            var targetTypes = target.types;
                            var intrinsicAttributes = getJsxType(JsxNames.IntrinsicAttributes, errorNode);
                            var intrinsicClassAttributes = getJsxType(JsxNames.IntrinsicClassAttributes, errorNode);
                            if (intrinsicAttributes !== errorType && intrinsicClassAttributes !== errorType &&
                                (ts.contains(targetTypes, intrinsicAttributes) || ts.contains(targetTypes, intrinsicClassAttributes))) {
                                // do not report top error
                                return result;
                            }
                        }
                        if (!headMessage && maybeSuppress) {
                            lastSkippedInfo = [source, target];
                            // Used by, eg, missing property checking to replace the top-level message with a more informative one
                            return result;
                        }
                        reportRelationError(headMessage, source, target);
                    }
                    return result;
                }
                function isIdenticalTo(source, target) {
                    var result;
                    var flags = source.flags & target.flags;
                    if (flags & 524288 /* Object */ || flags & 8388608 /* IndexedAccess */ || flags & 16777216 /* Conditional */ || flags & 4194304 /* Index */ || flags & 33554432 /* Substitution */) {
                        return recursiveTypeRelatedTo(source, target, /*reportErrors*/ false, 0 /* None */);
                    }
                    if (flags & (1048576 /* Union */ | 2097152 /* Intersection */)) {
                        if (result = eachTypeRelatedToSomeType(source, target)) {
                            if (result &= eachTypeRelatedToSomeType(target, source)) {
                                return result;
                            }
                        }
                    }
                    return 0 /* False */;
                }
                function getTypeOfPropertyInTypes(types, name) {
                    var appendPropType = function (propTypes, type) {
                        type = getApparentType(type);
                        var prop = type.flags & 3145728 /* UnionOrIntersection */ ? getPropertyOfUnionOrIntersectionType(type, name) : getPropertyOfObjectType(type, name);
                        var propType = prop && getTypeOfSymbol(prop) || isNumericLiteralName(name) && getIndexTypeOfType(type, 1 /* Number */) || getIndexTypeOfType(type, 0 /* String */) || undefinedType;
                        return ts.append(propTypes, propType);
                    };
                    return getUnionType(ts.reduceLeft(types, appendPropType, /*initial*/ undefined) || ts.emptyArray);
                }
                function hasExcessProperties(source, target, reportErrors) {
                    if (!isExcessPropertyCheckTarget(target) || !noImplicitAny && ts.getObjectFlags(target) & 16384 /* JSLiteral */) {
                        return false; // Disable excess property checks on JS literals to simulate having an implicit "index signature" - but only outside of noImplicitAny
                    }
                    var isComparingJsxAttributes = !!(ts.getObjectFlags(source) & 4096 /* JsxAttributes */);
                    if ((relation === assignableRelation || relation === comparableRelation) &&
                        (isTypeSubsetOf(globalObjectType, target) || (!isComparingJsxAttributes && isEmptyObjectType(target)))) {
                        return false;
                    }
                    var reducedTarget = target;
                    var checkTypes;
                    if (target.flags & 1048576 /* Union */) {
                        reducedTarget = findMatchingDiscriminantType(source, target, isRelatedTo) || filterPrimitivesIfContainsNonPrimitive(target);
                        checkTypes = reducedTarget.flags & 1048576 /* Union */ ? reducedTarget.types : [reducedTarget];
                    }
                    var _loop_11 = function (prop) {
                        if (shouldCheckAsExcessProperty(prop, source.symbol) && !isIgnoredJsxProperty(source, prop)) {
                            if (!isKnownProperty(reducedTarget, prop.escapedName, isComparingJsxAttributes)) {
                                if (reportErrors) {
                                    // Report error in terms of object types in the target as those are the only ones
                                    // we check in isKnownProperty.
                                    var errorTarget = filterType(reducedTarget, isExcessPropertyCheckTarget);
                                    // We know *exactly* where things went wrong when comparing the types.
                                    // Use this property as the error node as this will be more helpful in
                                    // reasoning about what went wrong.
                                    if (!errorNode)
                                        return { value: ts.Debug.fail() };
                                    if (ts.isJsxAttributes(errorNode) || ts.isJsxOpeningLikeElement(errorNode) || ts.isJsxOpeningLikeElement(errorNode.parent)) {
                                        // JsxAttributes has an object-literal flag and undergo same type-assignablity check as normal object-literal.
                                        // However, using an object-literal error message will be very confusing to the users so we give different a message.
                                        // TODO: Spelling suggestions for excess jsx attributes (needs new diagnostic messages)
                                        if (prop.valueDeclaration && ts.isJsxAttribute(prop.valueDeclaration) && ts.getSourceFileOfNode(errorNode) === ts.getSourceFileOfNode(prop.valueDeclaration.name)) {
                                            // Note that extraneous children (as in `<NoChild>extra</NoChild>`) don't pass this check,
                                            // since `children` is a SyntaxKind.PropertySignature instead of a SyntaxKind.JsxAttribute.
                                            errorNode = prop.valueDeclaration.name;
                                        }
                                        reportError(ts.Diagnostics.Property_0_does_not_exist_on_type_1, symbolToString(prop), typeToString(errorTarget));
                                    }
                                    else {
                                        // use the property's value declaration if the property is assigned inside the literal itself
                                        var objectLiteralDeclaration_1 = source.symbol && ts.firstOrUndefined(source.symbol.declarations);
                                        var suggestion = void 0;
                                        if (prop.valueDeclaration && ts.findAncestor(prop.valueDeclaration, function (d) { return d === objectLiteralDeclaration_1; }) && ts.getSourceFileOfNode(objectLiteralDeclaration_1) === ts.getSourceFileOfNode(errorNode)) {
                                            var propDeclaration = prop.valueDeclaration;
                                            ts.Debug.assertNode(propDeclaration, ts.isObjectLiteralElementLike);
                                            errorNode = propDeclaration;
                                            var name = propDeclaration.name;
                                            if (ts.isIdentifier(name)) {
                                                suggestion = getSuggestionForNonexistentProperty(name, errorTarget);
                                            }
                                        }
                                        if (suggestion !== undefined) {
                                            reportError(ts.Diagnostics.Object_literal_may_only_specify_known_properties_but_0_does_not_exist_in_type_1_Did_you_mean_to_write_2, symbolToString(prop), typeToString(errorTarget), suggestion);
                                        }
                                        else {
                                            reportError(ts.Diagnostics.Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1, symbolToString(prop), typeToString(errorTarget));
                                        }
                                    }
                                }
                                return { value: true };
                            }
                            if (checkTypes && !isRelatedTo(getTypeOfSymbol(prop), getTypeOfPropertyInTypes(checkTypes, prop.escapedName), reportErrors)) {
                                if (reportErrors) {
                                    reportIncompatibleError(ts.Diagnostics.Types_of_property_0_are_incompatible, symbolToString(prop));
                                }
                                return { value: true };
                            }
                        }
                    };
                    for (var _i = 0, _a = getPropertiesOfType(source); _i < _a.length; _i++) {
                        var prop = _a[_i];
                        var state_4 = _loop_11(prop);
                        if (typeof state_4 === "object")
                            return state_4.value;
                    }
                    return false;
                }
                function shouldCheckAsExcessProperty(prop, container) {
                    return prop.valueDeclaration && container.valueDeclaration && prop.valueDeclaration.parent === container.valueDeclaration;
                }
                function eachTypeRelatedToSomeType(source, target) {
                    var result = -1 /* True */;
                    var sourceTypes = source.types;
                    for (var _i = 0, sourceTypes_1 = sourceTypes; _i < sourceTypes_1.length; _i++) {
                        var sourceType = sourceTypes_1[_i];
                        var related = typeRelatedToSomeType(sourceType, target, /*reportErrors*/ false);
                        if (!related) {
                            return 0 /* False */;
                        }
                        result &= related;
                    }
                    return result;
                }
                function typeRelatedToSomeType(source, target, reportErrors) {
                    var targetTypes = target.types;
                    if (target.flags & 1048576 /* Union */ && containsType(targetTypes, source)) {
                        return -1 /* True */;
                    }
                    for (var _i = 0, targetTypes_1 = targetTypes; _i < targetTypes_1.length; _i++) {
                        var type = targetTypes_1[_i];
                        var related = isRelatedTo(source, type, /*reportErrors*/ false);
                        if (related) {
                            return related;
                        }
                    }
                    if (reportErrors) {
                        var bestMatchingType = getBestMatchingType(source, target, isRelatedTo);
                        isRelatedTo(source, bestMatchingType || targetTypes[targetTypes.length - 1], /*reportErrors*/ true);
                    }
                    return 0 /* False */;
                }
                function typeRelatedToEachType(source, target, reportErrors, intersectionState) {
                    var result = -1 /* True */;
                    var targetTypes = target.types;
                    for (var _i = 0, targetTypes_2 = targetTypes; _i < targetTypes_2.length; _i++) {
                        var targetType = targetTypes_2[_i];
                        var related = isRelatedTo(source, targetType, reportErrors, /*headMessage*/ undefined, intersectionState);
                        if (!related) {
                            return 0 /* False */;
                        }
                        result &= related;
                    }
                    return result;
                }
                function someTypeRelatedToType(source, target, reportErrors, intersectionState) {
                    var sourceTypes = source.types;
                    if (source.flags & 1048576 /* Union */ && containsType(sourceTypes, target)) {
                        return -1 /* True */;
                    }
                    var len = sourceTypes.length;
                    for (var i = 0; i < len; i++) {
                        var related = isRelatedTo(sourceTypes[i], target, reportErrors && i === len - 1, /*headMessage*/ undefined, intersectionState);
                        if (related) {
                            return related;
                        }
                    }
                    return 0 /* False */;
                }
                function eachTypeRelatedToType(source, target, reportErrors) {
                    var result = -1 /* True */;
                    var sourceTypes = source.types;
                    for (var _i = 0, sourceTypes_2 = sourceTypes; _i < sourceTypes_2.length; _i++) {
                        var sourceType = sourceTypes_2[_i];
                        var related = isRelatedTo(sourceType, target, reportErrors);
                        if (!related) {
                            return 0 /* False */;
                        }
                        result &= related;
                    }
                    return result;
                }
                function typeArgumentsRelatedTo(sources, targets, variances, reportErrors, intersectionState) {
                    if (sources === void 0) {
                        sources = ts.emptyArray;
                    }
                    if (targets === void 0) {
                        targets = ts.emptyArray;
                    }
                    if (variances === void 0) {
                        variances = ts.emptyArray;
                    }
                    if (sources.length !== targets.length && relation === identityRelation) {
                        return 0 /* False */;
                    }
                    var length = sources.length <= targets.length ? sources.length : targets.length;
                    var result = -1 /* True */;
                    for (var i = 0; i < length; i++) {
                        // When variance information isn't available we default to covariance. This happens
                        // in the process of computing variance information for recursive types and when
                        // comparing 'this' type arguments.
                        var varianceFlags = i < variances.length ? variances[i] : 1 /* Covariant */;
                        var variance = varianceFlags & 7 /* VarianceMask */;
                        // We ignore arguments for independent type parameters (because they're never witnessed).
                        if (variance !== 4 /* Independent */) {
                            var s = sources[i];
                            var t = targets[i];
                            var related = -1 /* True */;
                            if (varianceFlags & 8 /* Unmeasurable */) {
                                // Even an `Unmeasurable` variance works out without a structural check if the source and target are _identical_.
                                // We can't simply assume invariance, because `Unmeasurable` marks nonlinear relations, for example, a relation tained by
                                // the `-?` modifier in a mapped type (where, no matter how the inputs are related, the outputs still might not be)
                                related = relation === identityRelation ? isRelatedTo(s, t, /*reportErrors*/ false) : compareTypesIdentical(s, t);
                            }
                            else if (variance === 1 /* Covariant */) {
                                related = isRelatedTo(s, t, reportErrors, /*headMessage*/ undefined, intersectionState);
                            }
                            else if (variance === 2 /* Contravariant */) {
                                related = isRelatedTo(t, s, reportErrors, /*headMessage*/ undefined, intersectionState);
                            }
                            else if (variance === 3 /* Bivariant */) {
                                // In the bivariant case we first compare contravariantly without reporting
                                // errors. Then, if that doesn't succeed, we compare covariantly with error
                                // reporting. Thus, error elaboration will be based on the the covariant check,
                                // which is generally easier to reason about.
                                related = isRelatedTo(t, s, /*reportErrors*/ false);
                                if (!related) {
                                    related = isRelatedTo(s, t, reportErrors, /*headMessage*/ undefined, intersectionState);
                                }
                            }
                            else {
                                // In the invariant case we first compare covariantly, and only when that
                                // succeeds do we proceed to compare contravariantly. Thus, error elaboration
                                // will typically be based on the covariant check.
                                related = isRelatedTo(s, t, reportErrors, /*headMessage*/ undefined, intersectionState);
                                if (related) {
                                    related &= isRelatedTo(t, s, reportErrors, /*headMessage*/ undefined, intersectionState);
                                }
                            }
                            if (!related) {
                                return 0 /* False */;
                            }
                            result &= related;
                        }
                    }
                    return result;
                }
                // Determine if possibly recursive types are related. First, check if the result is already available in the global cache.
                // Second, check if we have already started a comparison of the given two types in which case we assume the result to be true.
                // Third, check if both types are part of deeply nested chains of generic type instantiations and if so assume the types are
                // equal and infinitely expanding. Fourth, if we have reached a depth of 100 nested comparisons, assume we have runaway recursion
                // and issue an error. Otherwise, actually compare the structure of the two types.
                function recursiveTypeRelatedTo(source, target, reportErrors, intersectionState) {
                    if (overflow) {
                        return 0 /* False */;
                    }
                    var id = getRelationKey(source, target, intersectionState, relation);
                    var entry = relation.get(id);
                    if (entry !== undefined) {
                        if (reportErrors && entry & 2 /* Failed */ && !(entry & 4 /* Reported */)) {
                            // We are elaborating errors and the cached result is an unreported failure. The result will be reported
                            // as a failure, and should be updated as a reported failure by the bottom of this function.
                        }
                        else {
                            if (outofbandVarianceMarkerHandler) {
                                // We're in the middle of variance checking - integrate any unmeasurable/unreliable flags from this cached component
                                var saved = entry & 24 /* ReportsMask */;
                                if (saved & 8 /* ReportsUnmeasurable */) {
                                    instantiateType(source, reportUnmeasurableMarkers);
                                }
                                if (saved & 16 /* ReportsUnreliable */) {
                                    instantiateType(source, reportUnreliableMarkers);
                                }
                            }
                            return entry & 1 /* Succeeded */ ? -1 /* True */ : 0 /* False */;
                        }
                    }
                    if (!maybeKeys) {
                        maybeKeys = [];
                        sourceStack = [];
                        targetStack = [];
                    }
                    else {
                        for (var i = 0; i < maybeCount; i++) {
                            // If source and target are already being compared, consider them related with assumptions
                            if (id === maybeKeys[i]) {
                                return 1 /* Maybe */;
                            }
                        }
                        if (depth === 100) {
                            overflow = true;
                            return 0 /* False */;
                        }
                    }
                    var maybeStart = maybeCount;
                    maybeKeys[maybeCount] = id;
                    maybeCount++;
                    sourceStack[depth] = source;
                    targetStack[depth] = target;
                    depth++;
                    var saveExpandingFlags = expandingFlags;
                    if (!(expandingFlags & 1 /* Source */) && isDeeplyNestedType(source, sourceStack, depth))
                        expandingFlags |= 1 /* Source */;
                    if (!(expandingFlags & 2 /* Target */) && isDeeplyNestedType(target, targetStack, depth))
                        expandingFlags |= 2 /* Target */;
                    var originalHandler;
                    var propagatingVarianceFlags = 0;
                    if (outofbandVarianceMarkerHandler) {
                        originalHandler = outofbandVarianceMarkerHandler;
                        outofbandVarianceMarkerHandler = function (onlyUnreliable) {
                            propagatingVarianceFlags |= onlyUnreliable ? 16 /* ReportsUnreliable */ : 8 /* ReportsUnmeasurable */;
                            return originalHandler(onlyUnreliable);
                        };
                    }
                    var result = expandingFlags !== 3 /* Both */ ? structuredTypeRelatedTo(source, target, reportErrors, intersectionState) : 1 /* Maybe */;
                    if (outofbandVarianceMarkerHandler) {
                        outofbandVarianceMarkerHandler = originalHandler;
                    }
                    expandingFlags = saveExpandingFlags;
                    depth--;
                    if (result) {
                        if (result === -1 /* True */ || depth === 0) {
                            // If result is definitely true, record all maybe keys as having succeeded
                            for (var i = maybeStart; i < maybeCount; i++) {
                                relation.set(maybeKeys[i], 1 /* Succeeded */ | propagatingVarianceFlags);
                            }
                            maybeCount = maybeStart;
                        }
                    }
                    else {
                        // A false result goes straight into global cache (when something is false under
                        // assumptions it will also be false without assumptions)
                        relation.set(id, (reportErrors ? 4 /* Reported */ : 0) | 2 /* Failed */ | propagatingVarianceFlags);
                        maybeCount = maybeStart;
                    }
                    return result;
                }
                function structuredTypeRelatedTo(source, target, reportErrors, intersectionState) {
                    var flags = source.flags & target.flags;
                    if (relation === identityRelation && !(flags & 524288 /* Object */)) {
                        if (flags & 4194304 /* Index */) {
                            return isRelatedTo(source.type, target.type, /*reportErrors*/ false);
                        }
                        var result_3 = 0 /* False */;
                        if (flags & 8388608 /* IndexedAccess */) {
                            if (result_3 = isRelatedTo(source.objectType, target.objectType, /*reportErrors*/ false)) {
                                if (result_3 &= isRelatedTo(source.indexType, target.indexType, /*reportErrors*/ false)) {
                                    return result_3;
                                }
                            }
                        }
                        if (flags & 16777216 /* Conditional */) {
                            if (source.root.isDistributive === target.root.isDistributive) {
                                if (result_3 = isRelatedTo(source.checkType, target.checkType, /*reportErrors*/ false)) {
                                    if (result_3 &= isRelatedTo(source.extendsType, target.extendsType, /*reportErrors*/ false)) {
                                        if (result_3 &= isRelatedTo(getTrueTypeFromConditionalType(source), getTrueTypeFromConditionalType(target), /*reportErrors*/ false)) {
                                            if (result_3 &= isRelatedTo(getFalseTypeFromConditionalType(source), getFalseTypeFromConditionalType(target), /*reportErrors*/ false)) {
                                                return result_3;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (flags & 33554432 /* Substitution */) {
                            return isRelatedTo(source.substitute, target.substitute, /*reportErrors*/ false);
                        }
                        return 0 /* False */;
                    }
                    var result;
                    var originalErrorInfo;
                    var varianceCheckFailed = false;
                    var saveErrorInfo = captureErrorCalculationState();
                    // We limit alias variance probing to only object and conditional types since their alias behavior
                    // is more predictable than other, interned types, which may or may not have an alias depending on
                    // the order in which things were checked.
                    if (source.flags & (524288 /* Object */ | 16777216 /* Conditional */) && source.aliasSymbol &&
                        source.aliasTypeArguments && source.aliasSymbol === target.aliasSymbol &&
                        !(source.aliasTypeArgumentsContainsMarker || target.aliasTypeArgumentsContainsMarker)) {
                        var variances = getAliasVariances(source.aliasSymbol);
                        if (variances === ts.emptyArray) {
                            return 1 /* Maybe */;
                        }
                        var varianceResult = relateVariances(source.aliasTypeArguments, target.aliasTypeArguments, variances, intersectionState);
                        if (varianceResult !== undefined) {
                            return varianceResult;
                        }
                    }
                    if (target.flags & 262144 /* TypeParameter */) {
                        // A source type { [P in Q]: X } is related to a target type T if keyof T is related to Q and X is related to T[Q].
                        if (ts.getObjectFlags(source) & 32 /* Mapped */ && isRelatedTo(getIndexType(target), getConstraintTypeFromMappedType(source))) {
                            if (!(getMappedTypeModifiers(source) & 4 /* IncludeOptional */)) {
                                var templateType = getTemplateTypeFromMappedType(source);
                                var indexedAccessType = getIndexedAccessType(target, getTypeParameterFromMappedType(source));
                                if (result = isRelatedTo(templateType, indexedAccessType, reportErrors)) {
                                    return result;
                                }
                            }
                        }
                    }
                    else if (target.flags & 4194304 /* Index */) {
                        // A keyof S is related to a keyof T if T is related to S.
                        if (source.flags & 4194304 /* Index */) {
                            if (result = isRelatedTo(target.type, source.type, /*reportErrors*/ false)) {
                                return result;
                            }
                        }
                        // A type S is assignable to keyof T if S is assignable to keyof C, where C is the
                        // simplified form of T or, if T doesn't simplify, the constraint of T.
                        var constraint = getSimplifiedTypeOrConstraint(target.type);
                        if (constraint) {
                            // We require Ternary.True here such that circular constraints don't cause
                            // false positives. For example, given 'T extends { [K in keyof T]: string }',
                            // 'keyof T' has itself as its constraint and produces a Ternary.Maybe when
                            // related to other types.
                            if (isRelatedTo(source, getIndexType(constraint, target.stringsOnly), reportErrors) === -1 /* True */) {
                                return -1 /* True */;
                            }
                        }
                    }
                    else if (target.flags & 8388608 /* IndexedAccess */) {
                        // A type S is related to a type T[K] if S is related to C, where C is the base
                        // constraint of T[K] for writing.
                        if (relation !== identityRelation) {
                            var objectType = target.objectType;
                            var indexType = target.indexType;
                            var baseObjectType = getBaseConstraintOfType(objectType) || objectType;
                            var baseIndexType = getBaseConstraintOfType(indexType) || indexType;
                            if (!isGenericObjectType(baseObjectType) && !isGenericIndexType(baseIndexType)) {
                                var accessFlags = 2 /* Writing */ | (baseObjectType !== objectType ? 1 /* NoIndexSignatures */ : 0);
                                var constraint = getIndexedAccessTypeOrUndefined(baseObjectType, baseIndexType, /*accessNode*/ undefined, accessFlags);
                                if (constraint && (result = isRelatedTo(source, constraint, reportErrors))) {
                                    return result;
                                }
                            }
                        }
                    }
                    else if (isGenericMappedType(target)) {
                        // A source type T is related to a target type { [P in X]: T[P] }
                        var template = getTemplateTypeFromMappedType(target);
                        var modifiers = getMappedTypeModifiers(target);
                        if (!(modifiers & 8 /* ExcludeOptional */)) {
                            if (template.flags & 8388608 /* IndexedAccess */ && template.objectType === source &&
                                template.indexType === getTypeParameterFromMappedType(target)) {
                                return -1 /* True */;
                            }
                            if (!isGenericMappedType(source)) {
                                var targetConstraint = getConstraintTypeFromMappedType(target);
                                var sourceKeys = getIndexType(source, /*stringsOnly*/ undefined, /*noIndexSignatures*/ true);
                                var includeOptional = modifiers & 4 /* IncludeOptional */;
                                var filteredByApplicability = includeOptional ? intersectTypes(targetConstraint, sourceKeys) : undefined;
                                // A source type T is related to a target type { [P in Q]: X } if Q is related to keyof T and T[Q] is related to X.
                                // A source type T is related to a target type { [P in Q]?: X } if some constituent Q' of Q is related to keyof T and T[Q'] is related to X.
                                if (includeOptional
                                    ? !(filteredByApplicability.flags & 131072 /* Never */)
                                    : isRelatedTo(targetConstraint, sourceKeys)) {
                                    var typeParameter = getTypeParameterFromMappedType(target);
                                    var indexingType = filteredByApplicability ? getIntersectionType([filteredByApplicability, typeParameter]) : typeParameter;
                                    var indexedAccessType = getIndexedAccessType(source, indexingType);
                                    var templateType = getTemplateTypeFromMappedType(target);
                                    if (result = isRelatedTo(indexedAccessType, templateType, reportErrors)) {
                                        return result;
                                    }
                                }
                                originalErrorInfo = errorInfo;
                                resetErrorInfo(saveErrorInfo);
                            }
                        }
                    }
                    if (source.flags & 8650752 /* TypeVariable */) {
                        if (source.flags & 8388608 /* IndexedAccess */ && target.flags & 8388608 /* IndexedAccess */) {
                            // A type S[K] is related to a type T[J] if S is related to T and K is related to J.
                            if (result = isRelatedTo(source.objectType, target.objectType, reportErrors)) {
                                result &= isRelatedTo(source.indexType, target.indexType, reportErrors);
                            }
                            if (result) {
                                resetErrorInfo(saveErrorInfo);
                                return result;
                            }
                        }
                        else {
                            var constraint = getConstraintOfType(source);
                            if (!constraint || (source.flags & 262144 /* TypeParameter */ && constraint.flags & 1 /* Any */)) {
                                // A type variable with no constraint is not related to the non-primitive object type.
                                if (result = isRelatedTo(emptyObjectType, extractTypesOfKind(target, ~67108864 /* NonPrimitive */))) {
                                    resetErrorInfo(saveErrorInfo);
                                    return result;
                                }
                            }
                            // hi-speed no-this-instantiation check (less accurate, but avoids costly `this`-instantiation when the constraint will suffice), see #28231 for report on why this is needed
                            else if (result = isRelatedTo(constraint, target, /*reportErrors*/ false, /*headMessage*/ undefined, intersectionState)) {
                                resetErrorInfo(saveErrorInfo);
                                return result;
                            }
                            // slower, fuller, this-instantiated check (necessary when comparing raw `this` types from base classes), see `subclassWithPolymorphicThisIsAssignable.ts` test for example
                            else if (result = isRelatedTo(getTypeWithThisArgument(constraint, source), target, reportErrors, /*headMessage*/ undefined, intersectionState)) {
                                resetErrorInfo(saveErrorInfo);
                                return result;
                            }
                        }
                    }
                    else if (source.flags & 4194304 /* Index */) {
                        if (result = isRelatedTo(keyofConstraintType, target, reportErrors)) {
                            resetErrorInfo(saveErrorInfo);
                            return result;
                        }
                    }
                    else if (source.flags & 16777216 /* Conditional */) {
                        if (target.flags & 16777216 /* Conditional */) {
                            // Two conditional types 'T1 extends U1 ? X1 : Y1' and 'T2 extends U2 ? X2 : Y2' are related if
                            // one of T1 and T2 is related to the other, U1 and U2 are identical types, X1 is related to X2,
                            // and Y1 is related to Y2.
                            var sourceParams = source.root.inferTypeParameters;
                            var sourceExtends = source.extendsType;
                            var mapper = void 0;
                            if (sourceParams) {
                                // If the source has infer type parameters, we instantiate them in the context of the target
                                var ctx = createInferenceContext(sourceParams, /*signature*/ undefined, 0 /* None */, isRelatedTo);
                                inferTypes(ctx.inferences, target.extendsType, sourceExtends, 128 /* NoConstraints */ | 256 /* AlwaysStrict */);
                                sourceExtends = instantiateType(sourceExtends, ctx.mapper);
                                mapper = ctx.mapper;
                            }
                            if (isTypeIdenticalTo(sourceExtends, target.extendsType) &&
                                (isRelatedTo(source.checkType, target.checkType) || isRelatedTo(target.checkType, source.checkType))) {
                                if (result = isRelatedTo(instantiateType(getTrueTypeFromConditionalType(source), mapper), getTrueTypeFromConditionalType(target), reportErrors)) {
                                    result &= isRelatedTo(getFalseTypeFromConditionalType(source), getFalseTypeFromConditionalType(target), reportErrors);
                                }
                                if (result) {
                                    resetErrorInfo(saveErrorInfo);
                                    return result;
                                }
                            }
                        }
                        else {
                            var distributiveConstraint = getConstraintOfDistributiveConditionalType(source);
                            if (distributiveConstraint) {
                                if (result = isRelatedTo(distributiveConstraint, target, reportErrors)) {
                                    resetErrorInfo(saveErrorInfo);
                                    return result;
                                }
                            }
                            var defaultConstraint = getDefaultConstraintOfConditionalType(source);
                            if (defaultConstraint) {
                                if (result = isRelatedTo(defaultConstraint, target, reportErrors)) {
                                    resetErrorInfo(saveErrorInfo);
                                    return result;
                                }
                            }
                        }
                    }
                    else {
                        // An empty object type is related to any mapped type that includes a '?' modifier.
                        if (relation !== subtypeRelation && relation !== strictSubtypeRelation && isPartialMappedType(target) && isEmptyObjectType(source)) {
                            return -1 /* True */;
                        }
                        if (isGenericMappedType(target)) {
                            if (isGenericMappedType(source)) {
                                if (result = mappedTypeRelatedTo(source, target, reportErrors)) {
                                    resetErrorInfo(saveErrorInfo);
                                    return result;
                                }
                            }
                            return 0 /* False */;
                        }
                        var sourceIsPrimitive = !!(source.flags & 131068 /* Primitive */);
                        if (relation !== identityRelation) {
                            source = getApparentType(source);
                        }
                        else if (isGenericMappedType(source)) {
                            return 0 /* False */;
                        }
                        if (ts.getObjectFlags(source) & 4 /* Reference */ && ts.getObjectFlags(target) & 4 /* Reference */ && source.target === target.target &&
                            !(ts.getObjectFlags(source) & 8192 /* MarkerType */ || ts.getObjectFlags(target) & 8192 /* MarkerType */)) {
                            // We have type references to the same generic type, and the type references are not marker
                            // type references (which are intended by be compared structurally). Obtain the variance
                            // information for the type parameters and relate the type arguments accordingly.
                            var variances = getVariances(source.target);
                            // We return Ternary.Maybe for a recursive invocation of getVariances (signalled by emptyArray). This
                            // effectively means we measure variance only from type parameter occurrences that aren't nested in
                            // recursive instantiations of the generic type.
                            if (variances === ts.emptyArray) {
                                return 1 /* Maybe */;
                            }
                            var varianceResult = relateVariances(getTypeArguments(source), getTypeArguments(target), variances, intersectionState);
                            if (varianceResult !== undefined) {
                                return varianceResult;
                            }
                        }
                        else if (isReadonlyArrayType(target) ? isArrayType(source) || isTupleType(source) : isArrayType(target) && isTupleType(source) && !source.target.readonly) {
                            if (relation !== identityRelation) {
                                return isRelatedTo(getIndexTypeOfType(source, 1 /* Number */) || anyType, getIndexTypeOfType(target, 1 /* Number */) || anyType, reportErrors);
                            }
                            else {
                                // By flags alone, we know that the `target` is a readonly array while the source is a normal array or tuple
                                // or `target` is an array and source is a tuple - in both cases the types cannot be identical, by construction
                                return 0 /* False */;
                            }
                        }
                        // Consider a fresh empty object literal type "closed" under the subtype relationship - this way `{} <- {[idx: string]: any} <- fresh({})`
                        // and not `{} <- fresh({}) <- {[idx: string]: any}`
                        else if ((relation === subtypeRelation || relation === strictSubtypeRelation) && isEmptyObjectType(target) && ts.getObjectFlags(target) & 32768 /* FreshLiteral */ && !isEmptyObjectType(source)) {
                            return 0 /* False */;
                        }
                        // Even if relationship doesn't hold for unions, intersections, or generic type references,
                        // it may hold in a structural comparison.
                        // In a check of the form X = A & B, we will have previously checked if A relates to X or B relates
                        // to X. Failing both of those we want to check if the aggregation of A and B's members structurally
                        // relates to X. Thus, we include intersection types on the source side here.
                        if (source.flags & (524288 /* Object */ | 2097152 /* Intersection */) && target.flags & 524288 /* Object */) {
                            // Report structural errors only if we haven't reported any errors yet
                            var reportStructuralErrors = reportErrors && errorInfo === saveErrorInfo.errorInfo && !sourceIsPrimitive;
                            result = propertiesRelatedTo(source, target, reportStructuralErrors, /*excludedProperties*/ undefined, intersectionState);
                            if (result) {
                                result &= signaturesRelatedTo(source, target, 0 /* Call */, reportStructuralErrors);
                                if (result) {
                                    result &= signaturesRelatedTo(source, target, 1 /* Construct */, reportStructuralErrors);
                                    if (result) {
                                        result &= indexTypesRelatedTo(source, target, 0 /* String */, sourceIsPrimitive, reportStructuralErrors, intersectionState);
                                        if (result) {
                                            result &= indexTypesRelatedTo(source, target, 1 /* Number */, sourceIsPrimitive, reportStructuralErrors, intersectionState);
                                        }
                                    }
                                }
                            }
                            if (varianceCheckFailed && result) {
                                errorInfo = originalErrorInfo || errorInfo || saveErrorInfo.errorInfo; // Use variance error (there is no structural one) and return false
                            }
                            else if (result) {
                                return result;
                            }
                        }
                        // If S is an object type and T is a discriminated union, S may be related to T if
                        // there exists a constituent of T for every combination of the discriminants of S
                        // with respect to T. We do not report errors here, as we will use the existing
                        // error result from checking each constituent of the union.
                        if (source.flags & (524288 /* Object */ | 2097152 /* Intersection */) && target.flags & 1048576 /* Union */) {
                            var objectOnlyTarget = extractTypesOfKind(target, 524288 /* Object */ | 2097152 /* Intersection */ | 33554432 /* Substitution */);
                            if (objectOnlyTarget.flags & 1048576 /* Union */) {
                                var result_4 = typeRelatedToDiscriminatedType(source, objectOnlyTarget);
                                if (result_4) {
                                    return result_4;
                                }
                            }
                        }
                    }
                    return 0 /* False */;
                    function relateVariances(sourceTypeArguments, targetTypeArguments, variances, intersectionState) {
                        if (result = typeArgumentsRelatedTo(sourceTypeArguments, targetTypeArguments, variances, reportErrors, intersectionState)) {
                            return result;
                        }
                        if (ts.some(variances, function (v) { return !!(v & 24 /* AllowsStructuralFallback */); })) {
                            // If some type parameter was `Unmeasurable` or `Unreliable`, and we couldn't pass by assuming it was identical, then we
                            // have to allow a structural fallback check
                            // We elide the variance-based error elaborations, since those might not be too helpful, since we'll potentially
                            // be assuming identity of the type parameter.
                            originalErrorInfo = undefined;
                            resetErrorInfo(saveErrorInfo);
                            return undefined;
                        }
                        var allowStructuralFallback = targetTypeArguments && hasCovariantVoidArgument(targetTypeArguments, variances);
                        varianceCheckFailed = !allowStructuralFallback;
                        // The type arguments did not relate appropriately, but it may be because we have no variance
                        // information (in which case typeArgumentsRelatedTo defaulted to covariance for all type
                        // arguments). It might also be the case that the target type has a 'void' type argument for
                        // a covariant type parameter that is only used in return positions within the generic type
                        // (in which case any type argument is permitted on the source side). In those cases we proceed
                        // with a structural comparison. Otherwise, we know for certain the instantiations aren't
                        // related and we can return here.
                        if (variances !== ts.emptyArray && !allowStructuralFallback) {
                            // In some cases generic types that are covariant in regular type checking mode become
                            // invariant in --strictFunctionTypes mode because one or more type parameters are used in
                            // both co- and contravariant positions. In order to make it easier to diagnose *why* such
                            // types are invariant, if any of the type parameters are invariant we reset the reported
                            // errors and instead force a structural comparison (which will include elaborations that
                            // reveal the reason).
                            // We can switch on `reportErrors` here, since varianceCheckFailed guarantees we return `False`,
                            // we can return `False` early here to skip calculating the structural error message we don't need.
                            if (varianceCheckFailed && !(reportErrors && ts.some(variances, function (v) { return (v & 7 /* VarianceMask */) === 0 /* Invariant */; }))) {
                                return 0 /* False */;
                            }
                            // We remember the original error information so we can restore it in case the structural
                            // comparison unexpectedly succeeds. This can happen when the structural comparison result
                            // is a Ternary.Maybe for example caused by the recursion depth limiter.
                            originalErrorInfo = errorInfo;
                            resetErrorInfo(saveErrorInfo);
                        }
                    }
                }
                function reportUnmeasurableMarkers(p) {
                    if (outofbandVarianceMarkerHandler && (p === markerSuperType || p === markerSubType || p === markerOtherType)) {
                        outofbandVarianceMarkerHandler(/*onlyUnreliable*/ false);
                    }
                    return p;
                }
                function reportUnreliableMarkers(p) {
                    if (outofbandVarianceMarkerHandler && (p === markerSuperType || p === markerSubType || p === markerOtherType)) {
                        outofbandVarianceMarkerHandler(/*onlyUnreliable*/ true);
                    }
                    return p;
                }
                // A type [P in S]: X is related to a type [Q in T]: Y if T is related to S and X' is
                // related to Y, where X' is an instantiation of X in which P is replaced with Q. Notice
                // that S and T are contra-variant whereas X and Y are co-variant.
                function mappedTypeRelatedTo(source, target, reportErrors) {
                    var modifiersRelated = relation === comparableRelation || (relation === identityRelation ? getMappedTypeModifiers(source) === getMappedTypeModifiers(target) :
                        getCombinedMappedTypeOptionality(source) <= getCombinedMappedTypeOptionality(target));
                    if (modifiersRelated) {
                        var result_5;
                        var targetConstraint = getConstraintTypeFromMappedType(target);
                        var sourceConstraint = instantiateType(getConstraintTypeFromMappedType(source), getCombinedMappedTypeOptionality(source) < 0 ? reportUnmeasurableMarkers : reportUnreliableMarkers);
                        if (result_5 = isRelatedTo(targetConstraint, sourceConstraint, reportErrors)) {
                            var mapper = createTypeMapper([getTypeParameterFromMappedType(source)], [getTypeParameterFromMappedType(target)]);
                            return result_5 & isRelatedTo(instantiateType(getTemplateTypeFromMappedType(source), mapper), getTemplateTypeFromMappedType(target), reportErrors);
                        }
                    }
                    return 0 /* False */;
                }
                function typeRelatedToDiscriminatedType(source, target) {
                    // 1. Generate the combinations of discriminant properties & types 'source' can satisfy.
                    //    a. If the number of combinations is above a set limit, the comparison is too complex.
                    // 2. Filter 'target' to the subset of types whose discriminants exist in the matrix.
                    //    a. If 'target' does not satisfy all discriminants in the matrix, 'source' is not related.
                    // 3. For each type in the filtered 'target', determine if all non-discriminant properties of
                    //    'target' are related to a property in 'source'.
                    //
                    // NOTE: See ~/tests/cases/conformance/types/typeRelationships/assignmentCompatibility/assignmentCompatWithDiscriminatedUnion.ts
                    //       for examples.
                    var sourceProperties = getPropertiesOfType(source);
                    var sourcePropertiesFiltered = findDiscriminantProperties(sourceProperties, target);
                    if (!sourcePropertiesFiltered)
                        return 0 /* False */;
                    // Though we could compute the number of combinations as we generate
                    // the matrix, this would incur additional memory overhead due to
                    // array allocations. To reduce this overhead, we first compute
                    // the number of combinations to ensure we will not surpass our
                    // fixed limit before incurring the cost of any allocations:
                    var numCombinations = 1;
                    for (var _i = 0, sourcePropertiesFiltered_1 = sourcePropertiesFiltered; _i < sourcePropertiesFiltered_1.length; _i++) {
                        var sourceProperty = sourcePropertiesFiltered_1[_i];
                        numCombinations *= countTypes(getTypeOfSymbol(sourceProperty));
                        if (numCombinations > 25) {
                            // We've reached the complexity limit.
                            return 0 /* False */;
                        }
                    }
                    // Compute the set of types for each discriminant property.
                    var sourceDiscriminantTypes = new Array(sourcePropertiesFiltered.length);
                    var excludedProperties = ts.createUnderscoreEscapedMap();
                    for (var i = 0; i < sourcePropertiesFiltered.length; i++) {
                        var sourceProperty = sourcePropertiesFiltered[i];
                        var sourcePropertyType = getTypeOfSymbol(sourceProperty);
                        sourceDiscriminantTypes[i] = sourcePropertyType.flags & 1048576 /* Union */
                            ? sourcePropertyType.types
                            : [sourcePropertyType];
                        excludedProperties.set(sourceProperty.escapedName, true);
                    }
                    // Match each combination of the cartesian product of discriminant properties to one or more
                    // constituents of 'target'. If any combination does not have a match then 'source' is not relatable.
                    var discriminantCombinations = ts.cartesianProduct(sourceDiscriminantTypes);
                    var matchingTypes = [];
                    var _loop_12 = function (combination) {
                        var hasMatch = false;
                        outer: for (var _i = 0, _a = target.types; _i < _a.length; _i++) {
                            var type = _a[_i];
                            var _loop_13 = function (i) {
                                var sourceProperty = sourcePropertiesFiltered[i];
                                var targetProperty = getPropertyOfType(type, sourceProperty.escapedName);
                                if (!targetProperty)
                                    return "continue-outer";
                                if (sourceProperty === targetProperty)
                                    return "continue";
                                // We compare the source property to the target in the context of a single discriminant type.
                                var related = propertyRelatedTo(source, target, sourceProperty, targetProperty, function (_) { return combination[i]; }, /*reportErrors*/ false, 0 /* None */);
                                // If the target property could not be found, or if the properties were not related,
                                // then this constituent is not a match.
                                if (!related) {
                                    return "continue-outer";
                                }
                            };
                            for (var i = 0; i < sourcePropertiesFiltered.length; i++) {
                                var state_6 = _loop_13(i);
                                switch (state_6) {
                                    case "continue-outer": continue outer;
                                }
                            }
                            ts.pushIfUnique(matchingTypes, type, ts.equateValues);
                            hasMatch = true;
                        }
                        if (!hasMatch) {
                            return { value: 0 /* False */ };
                        }
                    };
                    for (var _a = 0, discriminantCombinations_1 = discriminantCombinations; _a < discriminantCombinations_1.length; _a++) {
                        var combination = discriminantCombinations_1[_a];
                        var state_5 = _loop_12(combination);
                        if (typeof state_5 === "object")
                            return state_5.value;
                    }
                    // Compare the remaining non-discriminant properties of each match.
                    var result = -1 /* True */;
                    for (var _b = 0, matchingTypes_1 = matchingTypes; _b < matchingTypes_1.length; _b++) {
                        var type = matchingTypes_1[_b];
                        result &= propertiesRelatedTo(source, type, /*reportErrors*/ false, excludedProperties, 0 /* None */);
                        if (result) {
                            result &= signaturesRelatedTo(source, type, 0 /* Call */, /*reportStructuralErrors*/ false);
                            if (result) {
                                result &= signaturesRelatedTo(source, type, 1 /* Construct */, /*reportStructuralErrors*/ false);
                                if (result) {
                                    result &= indexTypesRelatedTo(source, type, 0 /* String */, /*sourceIsPrimitive*/ false, /*reportStructuralErrors*/ false, 0 /* None */);
                                    if (result) {
                                        result &= indexTypesRelatedTo(source, type, 1 /* Number */, /*sourceIsPrimitive*/ false, /*reportStructuralErrors*/ false, 0 /* None */);
                                    }
                                }
                            }
                        }
                        if (!result) {
                            return result;
                        }
                    }
                    return result;
                }
                function excludeProperties(properties, excludedProperties) {
                    if (!excludedProperties || properties.length === 0)
                        return properties;
                    var result;
                    for (var i = 0; i < properties.length; i++) {
                        if (!excludedProperties.has(properties[i].escapedName)) {
                            if (result) {
                                result.push(properties[i]);
                            }
                        }
                        else if (!result) {
                            result = properties.slice(0, i);
                        }
                    }
                    return result || properties;
                }
                function isPropertySymbolTypeRelated(sourceProp, targetProp, getTypeOfSourceProperty, reportErrors, intersectionState) {
                    var targetIsOptional = strictNullChecks && !!(ts.getCheckFlags(targetProp) & 48 /* Partial */);
                    var source = getTypeOfSourceProperty(sourceProp);
                    if (ts.getCheckFlags(targetProp) & 65536 /* DeferredType */ && !getSymbolLinks(targetProp).type) {
                        // Rather than resolving (and normalizing) the type, relate constituent-by-constituent without performing normalization or seconadary passes
                        var links = getSymbolLinks(targetProp);
                        ts.Debug.assertDefined(links.deferralParent);
                        ts.Debug.assertDefined(links.deferralConstituents);
                        var unionParent = !!(links.deferralParent.flags & 1048576 /* Union */);
                        var result_6 = unionParent ? 0 /* False */ : -1 /* True */;
                        var targetTypes = links.deferralConstituents;
                        for (var _i = 0, targetTypes_3 = targetTypes; _i < targetTypes_3.length; _i++) {
                            var targetType = targetTypes_3[_i];
                            var related = isRelatedTo(source, targetType, /*reportErrors*/ false, /*headMessage*/ undefined, unionParent ? 0 : 2 /* Target */);
                            if (!unionParent) {
                                if (!related) {
                                    // Can't assign to a target individually - have to fallback to assigning to the _whole_ intersection (which forces normalization)
                                    return isRelatedTo(source, addOptionality(getTypeOfSymbol(targetProp), targetIsOptional), reportErrors);
                                }
                                result_6 &= related;
                            }
                            else {
                                if (related) {
                                    return related;
                                }
                            }
                        }
                        if (unionParent && !result_6 && targetIsOptional) {
                            result_6 = isRelatedTo(source, undefinedType);
                        }
                        if (unionParent && !result_6 && reportErrors) {
                            // The easiest way to get the right errors here is to un-defer (which may be costly)
                            // If it turns out this is too costly too often, we can replicate the error handling logic within
                            // typeRelatedToSomeType without the discriminatable type branch (as that requires a manifest union
                            // type on which to hand discriminable properties, which we are expressly trying to avoid here)
                            return isRelatedTo(source, addOptionality(getTypeOfSymbol(targetProp), targetIsOptional), reportErrors);
                        }
                        return result_6;
                    }
                    else {
                        return isRelatedTo(source, addOptionality(getTypeOfSymbol(targetProp), targetIsOptional), reportErrors, /*headMessage*/ undefined, intersectionState);
                    }
                }
                function propertyRelatedTo(source, target, sourceProp, targetProp, getTypeOfSourceProperty, reportErrors, intersectionState) {
                    var sourcePropFlags = ts.getDeclarationModifierFlagsFromSymbol(sourceProp);
                    var targetPropFlags = ts.getDeclarationModifierFlagsFromSymbol(targetProp);
                    if (sourcePropFlags & 8 /* Private */ || targetPropFlags & 8 /* Private */) {
                        var hasDifferingDeclarations = sourceProp.valueDeclaration !== targetProp.valueDeclaration;
                        if (ts.getCheckFlags(sourceProp) & 1024 /* ContainsPrivate */ && hasDifferingDeclarations) {
                            if (reportErrors) {
                                reportError(ts.Diagnostics.Property_0_has_conflicting_declarations_and_is_inaccessible_in_type_1, symbolToString(sourceProp), typeToString(source));
                            }
                            return 0 /* False */;
                        }
                        if (hasDifferingDeclarations) {
                            if (reportErrors) {
                                if (sourcePropFlags & 8 /* Private */ && targetPropFlags & 8 /* Private */) {
                                    reportError(ts.Diagnostics.Types_have_separate_declarations_of_a_private_property_0, symbolToString(targetProp));
                                }
                                else {
                                    reportError(ts.Diagnostics.Property_0_is_private_in_type_1_but_not_in_type_2, symbolToString(targetProp), typeToString(sourcePropFlags & 8 /* Private */ ? source : target), typeToString(sourcePropFlags & 8 /* Private */ ? target : source));
                                }
                            }
                            return 0 /* False */;
                        }
                    }
                    else if (targetPropFlags & 16 /* Protected */) {
                        if (!isValidOverrideOf(sourceProp, targetProp)) {
                            if (reportErrors) {
                                reportError(ts.Diagnostics.Property_0_is_protected_but_type_1_is_not_a_class_derived_from_2, symbolToString(targetProp), typeToString(getDeclaringClass(sourceProp) || source), typeToString(getDeclaringClass(targetProp) || target));
                            }
                            return 0 /* False */;
                        }
                    }
                    else if (sourcePropFlags & 16 /* Protected */) {
                        if (reportErrors) {
                            reportError(ts.Diagnostics.Property_0_is_protected_in_type_1_but_public_in_type_2, symbolToString(targetProp), typeToString(source), typeToString(target));
                        }
                        return 0 /* False */;
                    }
                    // If the target comes from a partial union prop, allow `undefined` in the target type
                    var related = isPropertySymbolTypeRelated(sourceProp, targetProp, getTypeOfSourceProperty, reportErrors, intersectionState);
                    if (!related) {
                        if (reportErrors) {
                            reportIncompatibleError(ts.Diagnostics.Types_of_property_0_are_incompatible, symbolToString(targetProp));
                        }
                        return 0 /* False */;
                    }
                    // When checking for comparability, be more lenient with optional properties.
                    if (relation !== comparableRelation && sourceProp.flags & 16777216 /* Optional */ && !(targetProp.flags & 16777216 /* Optional */)) {
                        // TypeScript 1.0 spec (April 2014): 3.8.3
                        // S is a subtype of a type T, and T is a supertype of S if ...
                        // S' and T are object types and, for each member M in T..
                        // M is a property and S' contains a property N where
                        // if M is a required property, N is also a required property
                        // (M - property in T)
                        // (N - property in S)
                        if (reportErrors) {
                            reportError(ts.Diagnostics.Property_0_is_optional_in_type_1_but_required_in_type_2, symbolToString(targetProp), typeToString(source), typeToString(target));
                        }
                        return 0 /* False */;
                    }
                    return related;
                }
                function reportUnmatchedProperty(source, target, unmatchedProperty, requireOptionalProperties) {
                    var shouldSkipElaboration = false;
                    // give specific error in case where private names have the same description
                    if (unmatchedProperty.valueDeclaration
                        && ts.isNamedDeclaration(unmatchedProperty.valueDeclaration)
                        && ts.isPrivateIdentifier(unmatchedProperty.valueDeclaration.name)
                        && source.symbol
                        && source.symbol.flags & 32 /* Class */) {
                        var privateIdentifierDescription = unmatchedProperty.valueDeclaration.name.escapedText;
                        var symbolTableKey = ts.getSymbolNameForPrivateIdentifier(source.symbol, privateIdentifierDescription);
                        if (symbolTableKey && getPropertyOfType(source, symbolTableKey)) {
                            var sourceName = ts.getDeclarationName(source.symbol.valueDeclaration);
                            var targetName = ts.getDeclarationName(target.symbol.valueDeclaration);
                            reportError(ts.Diagnostics.Property_0_in_type_1_refers_to_a_different_member_that_cannot_be_accessed_from_within_type_2, diagnosticName(privateIdentifierDescription), diagnosticName(sourceName.escapedText === "" ? anon : sourceName), diagnosticName(targetName.escapedText === "" ? anon : targetName));
                            return;
                        }
                    }
                    var props = ts.arrayFrom(getUnmatchedProperties(source, target, requireOptionalProperties, /*matchDiscriminantProperties*/ false));
                    if (!headMessage || (headMessage.code !== ts.Diagnostics.Class_0_incorrectly_implements_interface_1.code &&
                        headMessage.code !== ts.Diagnostics.Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclass.code)) {
                        shouldSkipElaboration = true; // Retain top-level error for interface implementing issues, otherwise omit it
                    }
                    if (props.length === 1) {
                        var propName = symbolToString(unmatchedProperty);
                        reportError.apply(void 0, __spreadArrays([ts.Diagnostics.Property_0_is_missing_in_type_1_but_required_in_type_2, propName], getTypeNamesForErrorDisplay(source, target)));
                        if (ts.length(unmatchedProperty.declarations)) {
                            associateRelatedInfo(ts.createDiagnosticForNode(unmatchedProperty.declarations[0], ts.Diagnostics._0_is_declared_here, propName));
                        }
                        if (shouldSkipElaboration && errorInfo) {
                            overrideNextErrorInfo++;
                        }
                    }
                    else if (tryElaborateArrayLikeErrors(source, target, /*reportErrors*/ false)) {
                        if (props.length > 5) { // arbitrary cutoff for too-long list form
                            reportError(ts.Diagnostics.Type_0_is_missing_the_following_properties_from_type_1_Colon_2_and_3_more, typeToString(source), typeToString(target), ts.map(props.slice(0, 4), function (p) { return symbolToString(p); }).join(", "), props.length - 4);
                        }
                        else {
                            reportError(ts.Diagnostics.Type_0_is_missing_the_following_properties_from_type_1_Colon_2, typeToString(source), typeToString(target), ts.map(props, function (p) { return symbolToString(p); }).join(", "));
                        }
                        if (shouldSkipElaboration && errorInfo) {
                            overrideNextErrorInfo++;
                        }
                    }
                    // No array like or unmatched property error - just issue top level error (errorInfo = undefined)
                }
                function propertiesRelatedTo(source, target, reportErrors, excludedProperties, intersectionState) {
                    if (relation === identityRelation) {
                        return propertiesIdenticalTo(source, target, excludedProperties);
                    }
                    var requireOptionalProperties = (relation === subtypeRelation || relation === strictSubtypeRelation) && !isObjectLiteralType(source) && !isEmptyArrayLiteralType(source) && !isTupleType(source);
                    var unmatchedProperty = getUnmatchedProperty(source, target, requireOptionalProperties, /*matchDiscriminantProperties*/ false);
                    if (unmatchedProperty) {
                        if (reportErrors) {
                            reportUnmatchedProperty(source, target, unmatchedProperty, requireOptionalProperties);
                        }
                        return 0 /* False */;
                    }
                    if (isObjectLiteralType(target)) {
                        for (var _i = 0, _a = excludeProperties(getPropertiesOfType(source), excludedProperties); _i < _a.length; _i++) {
                            var sourceProp = _a[_i];
                            if (!getPropertyOfObjectType(target, sourceProp.escapedName)) {
                                var sourceType = getTypeOfSymbol(sourceProp);
                                if (!(sourceType === undefinedType || sourceType === undefinedWideningType || sourceType === optionalType)) {
                                    if (reportErrors) {
                                        reportError(ts.Diagnostics.Property_0_does_not_exist_on_type_1, symbolToString(sourceProp), typeToString(target));
                                    }
                                    return 0 /* False */;
                                }
                            }
                        }
                    }
                    var result = -1 /* True */;
                    if (isTupleType(target)) {
                        var targetRestType = getRestTypeOfTupleType(target);
                        if (targetRestType) {
                            if (!isTupleType(source)) {
                                return 0 /* False */;
                            }
                            var sourceRestType = getRestTypeOfTupleType(source);
                            if (sourceRestType && !isRelatedTo(sourceRestType, targetRestType, reportErrors)) {
                                if (reportErrors) {
                                    reportError(ts.Diagnostics.Rest_signatures_are_incompatible);
                                }
                                return 0 /* False */;
                            }
                            var targetCount = getTypeReferenceArity(target) - 1;
                            var sourceCount = getTypeReferenceArity(source) - (sourceRestType ? 1 : 0);
                            var sourceTypeArguments = getTypeArguments(source);
                            for (var i = targetCount; i < sourceCount; i++) {
                                var related = isRelatedTo(sourceTypeArguments[i], targetRestType, reportErrors);
                                if (!related) {
                                    if (reportErrors) {
                                        reportError(ts.Diagnostics.Property_0_is_incompatible_with_rest_element_type, "" + i);
                                    }
                                    return 0 /* False */;
                                }
                                result &= related;
                            }
                        }
                    }
                    // We only call this for union target types when we're attempting to do excess property checking - in those cases, we want to get _all possible props_
                    // from the target union, across all members
                    var properties = getPropertiesOfType(target);
                    var numericNamesOnly = isTupleType(source) && isTupleType(target);
                    for (var _b = 0, _c = excludeProperties(properties, excludedProperties); _b < _c.length; _b++) {
                        var targetProp = _c[_b];
                        var name = targetProp.escapedName;
                        if (!(targetProp.flags & 4194304 /* Prototype */) && (!numericNamesOnly || isNumericLiteralName(name) || name === "length")) {
                            var sourceProp = getPropertyOfType(source, name);
                            if (sourceProp && sourceProp !== targetProp) {
                                var related = propertyRelatedTo(source, target, sourceProp, targetProp, getTypeOfSymbol, reportErrors, intersectionState);
                                if (!related) {
                                    return 0 /* False */;
                                }
                                result &= related;
                            }
                        }
                    }
                    return result;
                }
                function propertiesIdenticalTo(source, target, excludedProperties) {
                    if (!(source.flags & 524288 /* Object */ && target.flags & 524288 /* Object */)) {
                        return 0 /* False */;
                    }
                    var sourceProperties = excludeProperties(getPropertiesOfObjectType(source), excludedProperties);
                    var targetProperties = excludeProperties(getPropertiesOfObjectType(target), excludedProperties);
                    if (sourceProperties.length !== targetProperties.length) {
                        return 0 /* False */;
                    }
                    var result = -1 /* True */;
                    for (var _i = 0, sourceProperties_1 = sourceProperties; _i < sourceProperties_1.length; _i++) {
                        var sourceProp = sourceProperties_1[_i];
                        var targetProp = getPropertyOfObjectType(target, sourceProp.escapedName);
                        if (!targetProp) {
                            return 0 /* False */;
                        }
                        var related = compareProperties(sourceProp, targetProp, isRelatedTo);
                        if (!related) {
                            return 0 /* False */;
                        }
                        result &= related;
                    }
                    return result;
                }
                function signaturesRelatedTo(source, target, kind, reportErrors) {
                    if (relation === identityRelation) {
                        return signaturesIdenticalTo(source, target, kind);
                    }
                    if (target === anyFunctionType || source === anyFunctionType) {
                        return -1 /* True */;
                    }
                    var sourceIsJSConstructor = source.symbol && isJSConstructor(source.symbol.valueDeclaration);
                    var targetIsJSConstructor = target.symbol && isJSConstructor(target.symbol.valueDeclaration);
                    var sourceSignatures = getSignaturesOfType(source, (sourceIsJSConstructor && kind === 1 /* Construct */) ?
                        0 /* Call */ : kind);
                    var targetSignatures = getSignaturesOfType(target, (targetIsJSConstructor && kind === 1 /* Construct */) ?
                        0 /* Call */ : kind);
                    if (kind === 1 /* Construct */ && sourceSignatures.length && targetSignatures.length) {
                        if (ts.isAbstractConstructorType(source) && !ts.isAbstractConstructorType(target)) {
                            // An abstract constructor type is not assignable to a non-abstract constructor type
                            // as it would otherwise be possible to new an abstract class. Note that the assignability
                            // check we perform for an extends clause excludes construct signatures from the target,
                            // so this check never proceeds.
                            if (reportErrors) {
                                reportError(ts.Diagnostics.Cannot_assign_an_abstract_constructor_type_to_a_non_abstract_constructor_type);
                            }
                            return 0 /* False */;
                        }
                        if (!constructorVisibilitiesAreCompatible(sourceSignatures[0], targetSignatures[0], reportErrors)) {
                            return 0 /* False */;
                        }
                    }
                    var result = -1 /* True */;
                    var saveErrorInfo = captureErrorCalculationState();
                    var incompatibleReporter = kind === 1 /* Construct */ ? reportIncompatibleConstructSignatureReturn : reportIncompatibleCallSignatureReturn;
                    if (ts.getObjectFlags(source) & 64 /* Instantiated */ && ts.getObjectFlags(target) & 64 /* Instantiated */ && source.symbol === target.symbol) {
                        // We have instantiations of the same anonymous type (which typically will be the type of a
                        // method). Simply do a pairwise comparison of the signatures in the two signature lists instead
                        // of the much more expensive N * M comparison matrix we explore below. We erase type parameters
                        // as they are known to always be the same.
                        for (var i = 0; i < targetSignatures.length; i++) {
                            var related = signatureRelatedTo(sourceSignatures[i], targetSignatures[i], /*erase*/ true, reportErrors, incompatibleReporter(sourceSignatures[i], targetSignatures[i]));
                            if (!related) {
                                return 0 /* False */;
                            }
                            result &= related;
                        }
                    }
                    else if (sourceSignatures.length === 1 && targetSignatures.length === 1) {
                        // For simple functions (functions with a single signature) we only erase type parameters for
                        // the comparable relation. Otherwise, if the source signature is generic, we instantiate it
                        // in the context of the target signature before checking the relationship. Ideally we'd do
                        // this regardless of the number of signatures, but the potential costs are prohibitive due
                        // to the quadratic nature of the logic below.
                        var eraseGenerics = relation === comparableRelation || !!compilerOptions.noStrictGenericChecks;
                        result = signatureRelatedTo(sourceSignatures[0], targetSignatures[0], eraseGenerics, reportErrors, incompatibleReporter(sourceSignatures[0], targetSignatures[0]));
                    }
                    else {
                        outer: for (var _i = 0, targetSignatures_1 = targetSignatures; _i < targetSignatures_1.length; _i++) {
                            var t = targetSignatures_1[_i];
                            // Only elaborate errors from the first failure
                            var shouldElaborateErrors = reportErrors;
                            for (var _a = 0, sourceSignatures_1 = sourceSignatures; _a < sourceSignatures_1.length; _a++) {
                                var s = sourceSignatures_1[_a];
                                var related = signatureRelatedTo(s, t, /*erase*/ true, shouldElaborateErrors, incompatibleReporter(s, t));
                                if (related) {
                                    result &= related;
                                    resetErrorInfo(saveErrorInfo);
                                    continue outer;
                                }
                                shouldElaborateErrors = false;
                            }
                            if (shouldElaborateErrors) {
                                reportError(ts.Diagnostics.Type_0_provides_no_match_for_the_signature_1, typeToString(source), signatureToString(t, /*enclosingDeclaration*/ undefined, /*flags*/ undefined, kind));
                            }
                            return 0 /* False */;
                        }
                    }
                    return result;
                }
                function reportIncompatibleCallSignatureReturn(siga, sigb) {
                    if (siga.parameters.length === 0 && sigb.parameters.length === 0) {
                        return function (source, target) { return reportIncompatibleError(ts.Diagnostics.Call_signatures_with_no_arguments_have_incompatible_return_types_0_and_1, typeToString(source), typeToString(target)); };
                    }
                    return function (source, target) { return reportIncompatibleError(ts.Diagnostics.Call_signature_return_types_0_and_1_are_incompatible, typeToString(source), typeToString(target)); };
                }
                function reportIncompatibleConstructSignatureReturn(siga, sigb) {
                    if (siga.parameters.length === 0 && sigb.parameters.length === 0) {
                        return function (source, target) { return reportIncompatibleError(ts.Diagnostics.Construct_signatures_with_no_arguments_have_incompatible_return_types_0_and_1, typeToString(source), typeToString(target)); };
                    }
                    return function (source, target) { return reportIncompatibleError(ts.Diagnostics.Construct_signature_return_types_0_and_1_are_incompatible, typeToString(source), typeToString(target)); };
                }
                /**
                 * See signatureAssignableTo, compareSignaturesIdentical
                 */
                function signatureRelatedTo(source, target, erase, reportErrors, incompatibleReporter) {
                    return compareSignaturesRelated(erase ? getErasedSignature(source) : source, erase ? getErasedSignature(target) : target, relation === strictSubtypeRelation ? 8 /* StrictArity */ : 0, reportErrors, reportError, incompatibleReporter, isRelatedTo, reportUnreliableMarkers);
                }
                function signaturesIdenticalTo(source, target, kind) {
                    var sourceSignatures = getSignaturesOfType(source, kind);
                    var targetSignatures = getSignaturesOfType(target, kind);
                    if (sourceSignatures.length !== targetSignatures.length) {
                        return 0 /* False */;
                    }
                    var result = -1 /* True */;
                    for (var i = 0; i < sourceSignatures.length; i++) {
                        var related = compareSignaturesIdentical(sourceSignatures[i], targetSignatures[i], /*partialMatch*/ false, /*ignoreThisTypes*/ false, /*ignoreReturnTypes*/ false, isRelatedTo);
                        if (!related) {
                            return 0 /* False */;
                        }
                        result &= related;
                    }
                    return result;
                }
                function eachPropertyRelatedTo(source, target, kind, reportErrors) {
                    var result = -1 /* True */;
                    var props = source.flags & 2097152 /* Intersection */ ? getPropertiesOfUnionOrIntersectionType(source) : getPropertiesOfObjectType(source);
                    for (var _i = 0, props_2 = props; _i < props_2.length; _i++) {
                        var prop = props_2[_i];
                        // Skip over ignored JSX and symbol-named members
                        if (isIgnoredJsxProperty(source, prop)) {
                            continue;
                        }
                        var nameType = getSymbolLinks(prop).nameType;
                        if (nameType && nameType.flags & 8192 /* UniqueESSymbol */) {
                            continue;
                        }
                        if (kind === 0 /* String */ || isNumericLiteralName(prop.escapedName)) {
                            var related = isRelatedTo(getTypeOfSymbol(prop), target, reportErrors);
                            if (!related) {
                                if (reportErrors) {
                                    reportError(ts.Diagnostics.Property_0_is_incompatible_with_index_signature, symbolToString(prop));
                                }
                                return 0 /* False */;
                            }
                            result &= related;
                        }
                    }
                    return result;
                }
                function indexTypeRelatedTo(sourceType, targetType, reportErrors) {
                    var related = isRelatedTo(sourceType, targetType, reportErrors);
                    if (!related && reportErrors) {
                        reportError(ts.Diagnostics.Index_signatures_are_incompatible);
                    }
                    return related;
                }
                function indexTypesRelatedTo(source, target, kind, sourceIsPrimitive, reportErrors, intersectionState) {
                    if (relation === identityRelation) {
                        return indexTypesIdenticalTo(source, target, kind);
                    }
                    var targetType = getIndexTypeOfType(target, kind);
                    if (!targetType || targetType.flags & 1 /* Any */ && !sourceIsPrimitive) {
                        // Index signature of type any permits assignment from everything but primitives
                        return -1 /* True */;
                    }
                    if (isGenericMappedType(source)) {
                        // A generic mapped type { [P in K]: T } is related to an index signature { [x: string]: U }
                        // if T is related to U.
                        return kind === 0 /* String */ ? isRelatedTo(getTemplateTypeFromMappedType(source), targetType, reportErrors) : 0 /* False */;
                    }
                    var indexType = getIndexTypeOfType(source, kind) || kind === 1 /* Number */ && getIndexTypeOfType(source, 0 /* String */);
                    if (indexType) {
                        return indexTypeRelatedTo(indexType, targetType, reportErrors);
                    }
                    if (!(intersectionState & 1 /* Source */) && isObjectTypeWithInferableIndex(source)) {
                        // Intersection constituents are never considered to have an inferred index signature
                        var related = eachPropertyRelatedTo(source, targetType, kind, reportErrors);
                        if (related && kind === 0 /* String */) {
                            var numberIndexType = getIndexTypeOfType(source, 1 /* Number */);
                            if (numberIndexType) {
                                related &= indexTypeRelatedTo(numberIndexType, targetType, reportErrors);
                            }
                        }
                        return related;
                    }
                    if (reportErrors) {
                        reportError(ts.Diagnostics.Index_signature_is_missing_in_type_0, typeToString(source));
                    }
                    return 0 /* False */;
                }
                function indexTypesIdenticalTo(source, target, indexKind) {
                    var targetInfo = getIndexInfoOfType(target, indexKind);
                    var sourceInfo = getIndexInfoOfType(source, indexKind);
                    if (!sourceInfo && !targetInfo) {
                        return -1 /* True */;
                    }
                    if (sourceInfo && targetInfo && sourceInfo.isReadonly === targetInfo.isReadonly) {
                        return isRelatedTo(sourceInfo.type, targetInfo.type);
                    }
                    return 0 /* False */;
                }
                function constructorVisibilitiesAreCompatible(sourceSignature, targetSignature, reportErrors) {
                    if (!sourceSignature.declaration || !targetSignature.declaration) {
                        return true;
                    }
                    var sourceAccessibility = ts.getSelectedModifierFlags(sourceSignature.declaration, 24 /* NonPublicAccessibilityModifier */);
                    var targetAccessibility = ts.getSelectedModifierFlags(targetSignature.declaration, 24 /* NonPublicAccessibilityModifier */);
                    // A public, protected and private signature is assignable to a private signature.
                    if (targetAccessibility === 8 /* Private */) {
                        return true;
                    }
                    // A public and protected signature is assignable to a protected signature.
                    if (targetAccessibility === 16 /* Protected */ && sourceAccessibility !== 8 /* Private */) {
                        return true;
                    }
                    // Only a public signature is assignable to public signature.
                    if (targetAccessibility !== 16 /* Protected */ && !sourceAccessibility) {
                        return true;
                    }
                    if (reportErrors) {
                        reportError(ts.Diagnostics.Cannot_assign_a_0_constructor_type_to_a_1_constructor_type, visibilityToString(sourceAccessibility), visibilityToString(targetAccessibility));
                    }
                    return false;
                }
            }
            function getBestMatchingType(source, target, isRelatedTo) {
                if (isRelatedTo === void 0) {
                    isRelatedTo = compareTypesAssignable;
                }
                return findMatchingDiscriminantType(source, target, isRelatedTo) ||
                    findMatchingTypeReferenceOrTypeAliasReference(source, target) ||
                    findBestTypeForObjectLiteral(source, target) ||
                    findBestTypeForInvokable(source, target) ||
                    findMostOverlappyType(source, target);
            }
            function discriminateTypeByDiscriminableItems(target, discriminators, related, defaultValue) {
                // undefined=unknown, true=discriminated, false=not discriminated
                // The state of each type progresses from left to right. Discriminated types stop at 'true'.
                var discriminable = target.types.map(function (_) { return undefined; });
                for (var _i = 0, discriminators_1 = discriminators; _i < discriminators_1.length; _i++) {
                    var _a = discriminators_1[_i], getDiscriminatingType = _a[0], propertyName = _a[1];
                    var i = 0;
                    for (var _b = 0, _c = target.types; _b < _c.length; _b++) {
                        var type = _c[_b];
                        var targetType = getTypeOfPropertyOfType(type, propertyName);
                        if (targetType && related(getDiscriminatingType(), targetType)) {
                            discriminable[i] = discriminable[i] === undefined ? true : discriminable[i];
                        }
                        else {
                            discriminable[i] = false;
                        }
                        i++;
                    }
                }
                var match = discriminable.indexOf(/*searchElement*/ true);
                // make sure exactly 1 matches before returning it
                return match === -1 || discriminable.indexOf(/*searchElement*/ true, match + 1) !== -1 ? defaultValue : target.types[match];
            }
            /**
             * A type is 'weak' if it is an object type with at least one optional property
             * and no required properties, call/construct signatures or index signatures
             */
            function isWeakType(type) {
                if (type.flags & 524288 /* Object */) {
                    var resolved = resolveStructuredTypeMembers(type);
                    return resolved.callSignatures.length === 0 && resolved.constructSignatures.length === 0 &&
                        !resolved.stringIndexInfo && !resolved.numberIndexInfo &&
                        resolved.properties.length > 0 &&
                        ts.every(resolved.properties, function (p) { return !!(p.flags & 16777216 /* Optional */); });
                }
                if (type.flags & 2097152 /* Intersection */) {
                    return ts.every(type.types, isWeakType);
                }
                return false;
            }
            function hasCommonProperties(source, target, isComparingJsxAttributes) {
                for (var _i = 0, _a = getPropertiesOfType(source); _i < _a.length; _i++) {
                    var prop = _a[_i];
                    if (isKnownProperty(target, prop.escapedName, isComparingJsxAttributes)) {
                        return true;
                    }
                }
                return false;
            }
            // Return a type reference where the source type parameter is replaced with the target marker
            // type, and flag the result as a marker type reference.
            function getMarkerTypeReference(type, source, target) {
                var result = createTypeReference(type, ts.map(type.typeParameters, function (t) { return t === source ? target : t; }));
                result.objectFlags |= 8192 /* MarkerType */;
                return result;
            }
            function getAliasVariances(symbol) {
                var links = getSymbolLinks(symbol);
                return getVariancesWorker(links.typeParameters, links, function (_links, param, marker) {
                    var type = getTypeAliasInstantiation(symbol, instantiateTypes(links.typeParameters, makeUnaryTypeMapper(param, marker)));
                    type.aliasTypeArgumentsContainsMarker = true;
                    return type;
                });
            }
            // Return an array containing the variance of each type parameter. The variance is effectively
            // a digest of the type comparisons that occur for each type argument when instantiations of the
            // generic type are structurally compared. We infer the variance information by comparing
            // instantiations of the generic type for type arguments with known relations. The function
            // returns the emptyArray singleton when invoked recursively for the given generic type.
            function getVariancesWorker(typeParameters, cache, createMarkerType) {
                if (typeParameters === void 0) {
                    typeParameters = ts.emptyArray;
                }
                var variances = cache.variances;
                if (!variances) {
                    // The emptyArray singleton is used to signal a recursive invocation.
                    cache.variances = ts.emptyArray;
                    variances = [];
                    var _loop_14 = function (tp) {
                        var unmeasurable = false;
                        var unreliable = false;
                        var oldHandler = outofbandVarianceMarkerHandler;
                        outofbandVarianceMarkerHandler = function (onlyUnreliable) { return onlyUnreliable ? unreliable = true : unmeasurable = true; };
                        // We first compare instantiations where the type parameter is replaced with
                        // marker types that have a known subtype relationship. From this we can infer
                        // invariance, covariance, contravariance or bivariance.
                        var typeWithSuper = createMarkerType(cache, tp, markerSuperType);
                        var typeWithSub = createMarkerType(cache, tp, markerSubType);
                        var variance = (isTypeAssignableTo(typeWithSub, typeWithSuper) ? 1 /* Covariant */ : 0) |
                            (isTypeAssignableTo(typeWithSuper, typeWithSub) ? 2 /* Contravariant */ : 0);
                        // If the instantiations appear to be related bivariantly it may be because the
                        // type parameter is independent (i.e. it isn't witnessed anywhere in the generic
                        // type). To determine this we compare instantiations where the type parameter is
                        // replaced with marker types that are known to be unrelated.
                        if (variance === 3 /* Bivariant */ && isTypeAssignableTo(createMarkerType(cache, tp, markerOtherType), typeWithSuper)) {
                            variance = 4 /* Independent */;
                        }
                        outofbandVarianceMarkerHandler = oldHandler;
                        if (unmeasurable || unreliable) {
                            if (unmeasurable) {
                                variance |= 8 /* Unmeasurable */;
                            }
                            if (unreliable) {
                                variance |= 16 /* Unreliable */;
                            }
                        }
                        variances.push(variance);
                    };
                    for (var _i = 0, typeParameters_1 = typeParameters; _i < typeParameters_1.length; _i++) {
                        var tp = typeParameters_1[_i];
                        _loop_14(tp);
                    }
                    cache.variances = variances;
                }
                return variances;
            }
            function getVariances(type) {
                // Arrays and tuples are known to be covariant, no need to spend time computing this.
                if (type === globalArrayType || type === globalReadonlyArrayType || type.objectFlags & 8 /* Tuple */) {
                    return arrayVariances;
                }
                return getVariancesWorker(type.typeParameters, type, getMarkerTypeReference);
            }
            // Return true if the given type reference has a 'void' type argument for a covariant type parameter.
            // See comment at call in recursiveTypeRelatedTo for when this case matters.
            function hasCovariantVoidArgument(typeArguments, variances) {
                for (var i = 0; i < variances.length; i++) {
                    if ((variances[i] & 7 /* VarianceMask */) === 1 /* Covariant */ && typeArguments[i].flags & 16384 /* Void */) {
                        return true;
                    }
                }
                return false;
            }
            function isUnconstrainedTypeParameter(type) {
                return type.flags & 262144 /* TypeParameter */ && !getConstraintOfTypeParameter(type);
            }
            function isNonDeferredTypeReference(type) {
                return !!(ts.getObjectFlags(type) & 4 /* Reference */) && !type.node;
            }
            function isTypeReferenceWithGenericArguments(type) {
                return isNonDeferredTypeReference(type) && ts.some(getTypeArguments(type), function (t) { return isUnconstrainedTypeParameter(t) || isTypeReferenceWithGenericArguments(t); });
            }
            /**
             * getTypeReferenceId(A<T, number, U>) returns "111=0-12=1"
             *   where A.id=111 and number.id=12
             */
            function getTypeReferenceId(type, typeParameters, depth) {
                if (depth === void 0) {
                    depth = 0;
                }
                var result = "" + type.target.id;
                for (var _i = 0, _a = getTypeArguments(type); _i < _a.length; _i++) {
                    var t = _a[_i];
                    if (isUnconstrainedTypeParameter(t)) {
                        var index = typeParameters.indexOf(t);
                        if (index < 0) {
                            index = typeParameters.length;
                            typeParameters.push(t);
                        }
                        result += "=" + index;
                    }
                    else if (depth < 4 && isTypeReferenceWithGenericArguments(t)) {
                        result += "<" + getTypeReferenceId(t, typeParameters, depth + 1) + ">";
                    }
                    else {
                        result += "-" + t.id;
                    }
                }
                return result;
            }
            /**
             * To improve caching, the relation key for two generic types uses the target's id plus ids of the type parameters.
             * For other cases, the types ids are used.
             */
            function getRelationKey(source, target, intersectionState, relation) {
                if (relation === identityRelation && source.id > target.id) {
                    var temp = source;
                    source = target;
                    target = temp;
                }
                var postFix = intersectionState ? ":" + intersectionState : "";
                if (isTypeReferenceWithGenericArguments(source) && isTypeReferenceWithGenericArguments(target)) {
                    var typeParameters = [];
                    return getTypeReferenceId(source, typeParameters) + "," + getTypeReferenceId(target, typeParameters) + postFix;
                }
                return source.id + "," + target.id + postFix;
            }
            // Invoke the callback for each underlying property symbol of the given symbol and return the first
            // value that isn't undefined.
            function forEachProperty(prop, callback) {
                if (ts.getCheckFlags(prop) & 6 /* Synthetic */) {
                    for (var _i = 0, _a = prop.containingType.types; _i < _a.length; _i++) {
                        var t = _a[_i];
                        var p = getPropertyOfType(t, prop.escapedName);
                        var result = p && forEachProperty(p, callback);
                        if (result) {
                            return result;
                        }
                    }
                    return undefined;
                }
                return callback(prop);
            }
            // Return the declaring class type of a property or undefined if property not declared in class
            function getDeclaringClass(prop) {
                return prop.parent && prop.parent.flags & 32 /* Class */ ? getDeclaredTypeOfSymbol(getParentOfSymbol(prop)) : undefined;
            }
            // Return true if some underlying source property is declared in a class that derives
            // from the given base class.
            function isPropertyInClassDerivedFrom(prop, baseClass) {
                return forEachProperty(prop, function (sp) {
                    var sourceClass = getDeclaringClass(sp);
                    return sourceClass ? hasBaseType(sourceClass, baseClass) : false;
                });
            }
            // Return true if source property is a valid override of protected parts of target property.
            function isValidOverrideOf(sourceProp, targetProp) {
                return !forEachProperty(targetProp, function (tp) {
                    return ts.getDeclarationModifierFlagsFromSymbol(tp) & 16 /* Protected */ ?
                        !isPropertyInClassDerivedFrom(sourceProp, getDeclaringClass(tp)) : false;
                });
            }
            // Return true if the given class derives from each of the declaring classes of the protected
            // constituents of the given property.
            function isClassDerivedFromDeclaringClasses(checkClass, prop) {
                return forEachProperty(prop, function (p) {
                    return ts.getDeclarationModifierFlagsFromSymbol(p) & 16 /* Protected */ ?
                        !hasBaseType(checkClass, getDeclaringClass(p)) : false;
                }) ? undefined : checkClass;
            }
            // Return true if the given type is deeply nested. We consider this to be the case when structural type comparisons
            // for 5 or more occurrences or instantiations of the type have been recorded on the given stack. It is possible,
            // though highly unlikely, for this test to be true in a situation where a chain of instantiations is not infinitely
            // expanding. Effectively, we will generate a false positive when two types are structurally equal to at least 5
            // levels, but unequal at some level beyond that.
            // In addition, this will also detect when an indexed access has been chained off of 5 or more times (which is essentially
            // the dual of the structural comparison), and likewise mark the type as deeply nested, potentially adding false positives
            // for finite but deeply expanding indexed accesses (eg, for `Q[P1][P2][P3][P4][P5]`).
            function isDeeplyNestedType(type, stack, depth) {
                // We track all object types that have an associated symbol (representing the origin of the type)
                if (depth >= 5 && type.flags & 524288 /* Object */ && !isObjectOrArrayLiteralType(type)) {
                    var symbol = type.symbol;
                    if (symbol) {
                        var count = 0;
                        for (var i = 0; i < depth; i++) {
                            var t = stack[i];
                            if (t.flags & 524288 /* Object */ && t.symbol === symbol) {
                                count++;
                                if (count >= 5)
                                    return true;
                            }
                        }
                    }
                }
                if (depth >= 5 && type.flags & 8388608 /* IndexedAccess */) {
                    var root = getRootObjectTypeFromIndexedAccessChain(type);
                    var count = 0;
                    for (var i = 0; i < depth; i++) {
                        var t = stack[i];
                        if (getRootObjectTypeFromIndexedAccessChain(t) === root) {
                            count++;
                            if (count >= 5)
                                return true;
                        }
                    }
                }
                return false;
            }
            /**
             * Gets the leftmost object type in a chain of indexed accesses, eg, in A[P][Q], returns A
             */
            function getRootObjectTypeFromIndexedAccessChain(type) {
                var t = type;
                while (t.flags & 8388608 /* IndexedAccess */) {
                    t = t.objectType;
                }
                return t;
            }
            function isPropertyIdenticalTo(sourceProp, targetProp) {
                return compareProperties(sourceProp, targetProp, compareTypesIdentical) !== 0 /* False */;
            }
            function compareProperties(sourceProp, targetProp, compareTypes) {
                // Two members are considered identical when
                // - they are public properties with identical names, optionality, and types,
                // - they are private or protected properties originating in the same declaration and having identical types
                if (sourceProp === targetProp) {
                    return -1 /* True */;
                }
                var sourcePropAccessibility = ts.getDeclarationModifierFlagsFromSymbol(sourceProp) & 24 /* NonPublicAccessibilityModifier */;
                var targetPropAccessibility = ts.getDeclarationModifierFlagsFromSymbol(targetProp) & 24 /* NonPublicAccessibilityModifier */;
                if (sourcePropAccessibility !== targetPropAccessibility) {
                    return 0 /* False */;
                }
                if (sourcePropAccessibility) {
                    if (getTargetSymbol(sourceProp) !== getTargetSymbol(targetProp)) {
                        return 0 /* False */;
                    }
                }
                else {
                    if ((sourceProp.flags & 16777216 /* Optional */) !== (targetProp.flags & 16777216 /* Optional */)) {
                        return 0 /* False */;
                    }
                }
                if (isReadonlySymbol(sourceProp) !== isReadonlySymbol(targetProp)) {
                    return 0 /* False */;
                }
                return compareTypes(getTypeOfSymbol(sourceProp), getTypeOfSymbol(targetProp));
            }
            function isMatchingSignature(source, target, partialMatch) {
                var sourceParameterCount = getParameterCount(source);
                var targetParameterCount = getParameterCount(target);
                var sourceMinArgumentCount = getMinArgumentCount(source);
                var targetMinArgumentCount = getMinArgumentCount(target);
                var sourceHasRestParameter = hasEffectiveRestParameter(source);
                var targetHasRestParameter = hasEffectiveRestParameter(target);
                // A source signature matches a target signature if the two signatures have the same number of required,
                // optional, and rest parameters.
                if (sourceParameterCount === targetParameterCount &&
                    sourceMinArgumentCount === targetMinArgumentCount &&
                    sourceHasRestParameter === targetHasRestParameter) {
                    return true;
                }
                // A source signature partially matches a target signature if the target signature has no fewer required
                // parameters
                if (partialMatch && sourceMinArgumentCount <= targetMinArgumentCount) {
                    return true;
                }
                return false;
            }
            /**
             * See signatureRelatedTo, compareSignaturesIdentical
             */
            function compareSignaturesIdentical(source, target, partialMatch, ignoreThisTypes, ignoreReturnTypes, compareTypes) {
                // TODO (drosen): De-duplicate code between related functions.
                if (source === target) {
                    return -1 /* True */;
                }
                if (!(isMatchingSignature(source, target, partialMatch))) {
                    return 0 /* False */;
                }
                // Check that the two signatures have the same number of type parameters.
                if (ts.length(source.typeParameters) !== ts.length(target.typeParameters)) {
                    return 0 /* False */;
                }
                // Check that type parameter constraints and defaults match. If they do, instantiate the source
                // signature with the type parameters of the target signature and continue the comparison.
                if (target.typeParameters) {
                    var mapper = createTypeMapper(source.typeParameters, target.typeParameters);
                    for (var i = 0; i < target.typeParameters.length; i++) {
                        var s = source.typeParameters[i];
                        var t = target.typeParameters[i];
                        if (!(s === t || compareTypes(instantiateType(getConstraintFromTypeParameter(s), mapper) || unknownType, getConstraintFromTypeParameter(t) || unknownType) &&
                            compareTypes(instantiateType(getDefaultFromTypeParameter(s), mapper) || unknownType, getDefaultFromTypeParameter(t) || unknownType))) {
                            return 0 /* False */;
                        }
                    }
                    source = instantiateSignature(source, mapper, /*eraseTypeParameters*/ true);
                }
                var result = -1 /* True */;
                if (!ignoreThisTypes) {
                    var sourceThisType = getThisTypeOfSignature(source);
                    if (sourceThisType) {
                        var targetThisType = getThisTypeOfSignature(target);
                        if (targetThisType) {
                            var related = compareTypes(sourceThisType, targetThisType);
                            if (!related) {
                                return 0 /* False */;
                            }
                            result &= related;
                        }
                    }
                }
                var targetLen = getParameterCount(target);
                for (var i = 0; i < targetLen; i++) {
                    var s = getTypeAtPosition(source, i);
                    var t = getTypeAtPosition(target, i);
                    var related = compareTypes(t, s);
                    if (!related) {
                        return 0 /* False */;
                    }
                    result &= related;
                }
                if (!ignoreReturnTypes) {
                    var sourceTypePredicate = getTypePredicateOfSignature(source);
                    var targetTypePredicate = getTypePredicateOfSignature(target);
                    result &= sourceTypePredicate || targetTypePredicate ?
                        compareTypePredicatesIdentical(sourceTypePredicate, targetTypePredicate, compareTypes) :
                        compareTypes(getReturnTypeOfSignature(source), getReturnTypeOfSignature(target));
                }
                return result;
            }
            function compareTypePredicatesIdentical(source, target, compareTypes) {
                return !(source && target && typePredicateKindsMatch(source, target)) ? 0 /* False */ :
                    source.type === target.type ? -1 /* True */ :
                        source.type && target.type ? compareTypes(source.type, target.type) :
                            0 /* False */;
            }
            function literalTypesWithSameBaseType(types) {
                var commonBaseType;
                for (var _i = 0, types_11 = types; _i < types_11.length; _i++) {
                    var t = types_11[_i];
                    var baseType = getBaseTypeOfLiteralType(t);
                    if (!commonBaseType) {
                        commonBaseType = baseType;
                    }
                    if (baseType === t || baseType !== commonBaseType) {
                        return false;
                    }
                }
                return true;
            }
            // When the candidate types are all literal types with the same base type, return a union
            // of those literal types. Otherwise, return the leftmost type for which no type to the
            // right is a supertype.
            function getSupertypeOrUnion(types) {
                return literalTypesWithSameBaseType(types) ?
                    getUnionType(types) :
                    ts.reduceLeft(types, function (s, t) { return isTypeSubtypeOf(s, t) ? t : s; });
            }
            function getCommonSupertype(types) {
                if (!strictNullChecks) {
                    return getSupertypeOrUnion(types);
                }
                var primaryTypes = ts.filter(types, function (t) { return !(t.flags & 98304 /* Nullable */); });
                return primaryTypes.length ?
                    getNullableType(getSupertypeOrUnion(primaryTypes), getFalsyFlagsOfTypes(types) & 98304 /* Nullable */) :
                    getUnionType(types, 2 /* Subtype */);
            }
            // Return the leftmost type for which no type to the right is a subtype.
            function getCommonSubtype(types) {
                return ts.reduceLeft(types, function (s, t) { return isTypeSubtypeOf(t, s) ? t : s; });
            }
            function isArrayType(type) {
                return !!(ts.getObjectFlags(type) & 4 /* Reference */) && (type.target === globalArrayType || type.target === globalReadonlyArrayType);
            }
            function isReadonlyArrayType(type) {
                return !!(ts.getObjectFlags(type) & 4 /* Reference */) && type.target === globalReadonlyArrayType;
            }
            function isMutableArrayOrTuple(type) {
                return isArrayType(type) && !isReadonlyArrayType(type) || isTupleType(type) && !type.target.readonly;
            }
            function getElementTypeOfArrayType(type) {
                return isArrayType(type) ? getTypeArguments(type)[0] : undefined;
            }
            function isArrayLikeType(type) {
                // A type is array-like if it is a reference to the global Array or global ReadonlyArray type,
                // or if it is not the undefined or null type and if it is assignable to ReadonlyArray<any>
                return isArrayType(type) || !(type.flags & 98304 /* Nullable */) && isTypeAssignableTo(type, anyReadonlyArrayType);
            }
            function isEmptyArrayLiteralType(type) {
                var elementType = isArrayType(type) ? getTypeArguments(type)[0] : undefined;
                return elementType === undefinedWideningType || elementType === implicitNeverType;
            }
            function isTupleLikeType(type) {
                return isTupleType(type) || !!getPropertyOfType(type, "0");
            }
            function isArrayOrTupleLikeType(type) {
                return isArrayLikeType(type) || isTupleLikeType(type);
            }
            function getTupleElementType(type, index) {
                var propType = getTypeOfPropertyOfType(type, "" + index);
                if (propType) {
                    return propType;
                }
                if (everyType(type, isTupleType)) {
                    return mapType(type, function (t) { return getRestTypeOfTupleType(t) || undefinedType; });
                }
                return undefined;
            }
            function isNeitherUnitTypeNorNever(type) {
                return !(type.flags & (109440 /* Unit */ | 131072 /* Never */));
            }
            function isUnitType(type) {
                return !!(type.flags & 109440 /* Unit */);
            }
            function isLiteralType(type) {
                return type.flags & 16 /* Boolean */ ? true :
                    type.flags & 1048576 /* Union */ ? type.flags & 1024 /* EnumLiteral */ ? true : ts.every(type.types, isUnitType) :
                        isUnitType(type);
            }
            function getBaseTypeOfLiteralType(type) {
                return type.flags & 1024 /* EnumLiteral */ ? getBaseTypeOfEnumLiteralType(type) :
                    type.flags & 128 /* StringLiteral */ ? stringType :
                        type.flags & 256 /* NumberLiteral */ ? numberType :
                            type.flags & 2048 /* BigIntLiteral */ ? bigintType :
                                type.flags & 512 /* BooleanLiteral */ ? booleanType :
                                    type.flags & 1048576 /* Union */ ? getUnionType(ts.sameMap(type.types, getBaseTypeOfLiteralType)) :
                                        type;
            }
            function getWidenedLiteralType(type) {
                return type.flags & 1024 /* EnumLiteral */ && isFreshLiteralType(type) ? getBaseTypeOfEnumLiteralType(type) :
                    type.flags & 128 /* StringLiteral */ && isFreshLiteralType(type) ? stringType :
                        type.flags & 256 /* NumberLiteral */ && isFreshLiteralType(type) ? numberType :
                            type.flags & 2048 /* BigIntLiteral */ && isFreshLiteralType(type) ? bigintType :
                                type.flags & 512 /* BooleanLiteral */ && isFreshLiteralType(type) ? booleanType :
                                    type.flags & 1048576 /* Union */ ? getUnionType(ts.sameMap(type.types, getWidenedLiteralType)) :
                                        type;
            }
            function getWidenedUniqueESSymbolType(type) {
                return type.flags & 8192 /* UniqueESSymbol */ ? esSymbolType :
                    type.flags & 1048576 /* Union */ ? getUnionType(ts.sameMap(type.types, getWidenedUniqueESSymbolType)) :
                        type;
            }
            function getWidenedLiteralLikeTypeForContextualType(type, contextualType) {
                if (!isLiteralOfContextualType(type, contextualType)) {
                    type = getWidenedUniqueESSymbolType(getWidenedLiteralType(type));
                }
                return type;
            }
            function getWidenedLiteralLikeTypeForContextualReturnTypeIfNeeded(type, contextualSignatureReturnType, isAsync) {
                if (type && isUnitType(type)) {
                    var contextualType = !contextualSignatureReturnType ? undefined :
                        isAsync ? getPromisedTypeOfPromise(contextualSignatureReturnType) :
                            contextualSignatureReturnType;
                    type = getWidenedLiteralLikeTypeForContextualType(type, contextualType);
                }
                return type;
            }
            function getWidenedLiteralLikeTypeForContextualIterationTypeIfNeeded(type, contextualSignatureReturnType, kind, isAsyncGenerator) {
                if (type && isUnitType(type)) {
                    var contextualType = !contextualSignatureReturnType ? undefined :
                        getIterationTypeOfGeneratorFunctionReturnType(kind, contextualSignatureReturnType, isAsyncGenerator);
                    type = getWidenedLiteralLikeTypeForContextualType(type, contextualType);
                }
                return type;
            }
            /**
             * Check if a Type was written as a tuple type literal.
             * Prefer using isTupleLikeType() unless the use of `elementTypes` is required.
             */
            function isTupleType(type) {
                return !!(ts.getObjectFlags(type) & 4 /* Reference */ && type.target.objectFlags & 8 /* Tuple */);
            }
            function getRestTypeOfTupleType(type) {
                return type.target.hasRestElement ? getTypeArguments(type)[type.target.typeParameters.length - 1] : undefined;
            }
            function getRestArrayTypeOfTupleType(type) {
                var restType = getRestTypeOfTupleType(type);
                return restType && createArrayType(restType);
            }
            function getLengthOfTupleType(type) {
                return getTypeReferenceArity(type) - (type.target.hasRestElement ? 1 : 0);
            }
            function isZeroBigInt(_a) {
                var value = _a.value;
                return value.base10Value === "0";
            }
            function getFalsyFlagsOfTypes(types) {
                var result = 0;
                for (var _i = 0, types_12 = types; _i < types_12.length; _i++) {
                    var t = types_12[_i];
                    result |= getFalsyFlags(t);
                }
                return result;
            }
            // Returns the String, Number, Boolean, StringLiteral, NumberLiteral, BooleanLiteral, Void, Undefined, or Null
            // flags for the string, number, boolean, "", 0, false, void, undefined, or null types respectively. Returns
            // no flags for all other types (including non-falsy literal types).
            function getFalsyFlags(type) {
                return type.flags & 1048576 /* Union */ ? getFalsyFlagsOfTypes(type.types) :
                    type.flags & 128 /* StringLiteral */ ? type.value === "" ? 128 /* StringLiteral */ : 0 :
                        type.flags & 256 /* NumberLiteral */ ? type.value === 0 ? 256 /* NumberLiteral */ : 0 :
                            type.flags & 2048 /* BigIntLiteral */ ? isZeroBigInt(type) ? 2048 /* BigIntLiteral */ : 0 :
                                type.flags & 512 /* BooleanLiteral */ ? (type === falseType || type === regularFalseType) ? 512 /* BooleanLiteral */ : 0 :
                                    type.flags & 117724 /* PossiblyFalsy */;
            }
            function removeDefinitelyFalsyTypes(type) {
                return getFalsyFlags(type) & 117632 /* DefinitelyFalsy */ ?
                    filterType(type, function (t) { return !(getFalsyFlags(t) & 117632 /* DefinitelyFalsy */); }) :
                    type;
            }
            function extractDefinitelyFalsyTypes(type) {
                return mapType(type, getDefinitelyFalsyPartOfType);
            }
            function getDefinitelyFalsyPartOfType(type) {
                return type.flags & 4 /* String */ ? emptyStringType :
                    type.flags & 8 /* Number */ ? zeroType :
                        type.flags & 64 /* BigInt */ ? zeroBigIntType :
                            type === regularFalseType ||
                                type === falseType ||
                                type.flags & (16384 /* Void */ | 32768 /* Undefined */ | 65536 /* Null */) ||
                                type.flags & 128 /* StringLiteral */ && type.value === "" ||
                                type.flags & 256 /* NumberLiteral */ && type.value === 0 ||
                                type.flags & 2048 /* BigIntLiteral */ && isZeroBigInt(type) ? type :
                                neverType;
            }
            /**
             * Add undefined or null or both to a type if they are missing.
             * @param type - type to add undefined and/or null to if not present
             * @param flags - Either TypeFlags.Undefined or TypeFlags.Null, or both
             */
            function getNullableType(type, flags) {
                var missing = (flags & ~type.flags) & (32768 /* Undefined */ | 65536 /* Null */);
                return missing === 0 ? type :
                    missing === 32768 /* Undefined */ ? getUnionType([type, undefinedType]) :
                        missing === 65536 /* Null */ ? getUnionType([type, nullType]) :
                            getUnionType([type, undefinedType, nullType]);
            }
            function getOptionalType(type) {
                ts.Debug.assert(strictNullChecks);
                return type.flags & 32768 /* Undefined */ ? type : getUnionType([type, undefinedType]);
            }
            function getGlobalNonNullableTypeInstantiation(type) {
                if (!deferredGlobalNonNullableTypeAlias) {
                    deferredGlobalNonNullableTypeAlias = getGlobalSymbol("NonNullable", 524288 /* TypeAlias */, /*diagnostic*/ undefined) || unknownSymbol;
                }
                // Use NonNullable global type alias if available to improve quick info/declaration emit
                if (deferredGlobalNonNullableTypeAlias !== unknownSymbol) {
                    return getTypeAliasInstantiation(deferredGlobalNonNullableTypeAlias, [type]);
                }
                return getTypeWithFacts(type, 2097152 /* NEUndefinedOrNull */); // Type alias unavailable, fall back to non-higher-order behavior
            }
            function getNonNullableType(type) {
                return strictNullChecks ? getGlobalNonNullableTypeInstantiation(type) : type;
            }
            function addOptionalTypeMarker(type) {
                return strictNullChecks ? getUnionType([type, optionalType]) : type;
            }
            function isNotOptionalTypeMarker(type) {
                return type !== optionalType;
            }
            function removeOptionalTypeMarker(type) {
                return strictNullChecks ? filterType(type, isNotOptionalTypeMarker) : type;
            }
            function propagateOptionalTypeMarker(type, node, wasOptional) {
                return wasOptional ? ts.isOutermostOptionalChain(node) ? getOptionalType(type) : addOptionalTypeMarker(type) : type;
            }
            function getOptionalExpressionType(exprType, expression) {
                return ts.isExpressionOfOptionalChainRoot(expression) ? getNonNullableType(exprType) :
                    ts.isOptionalChain(expression) ? removeOptionalTypeMarker(exprType) :
                        exprType;
            }
            /**
             * Is source potentially coercible to target type under `==`.
             * Assumes that `source` is a constituent of a union, hence
             * the boolean literal flag on the LHS, but not on the RHS.
             *
             * This does not fully replicate the semantics of `==`. The
             * intention is to catch cases that are clearly not right.
             *
             * Comparing (string | number) to number should not remove the
             * string element.
             *
             * Comparing (string | number) to 1 will remove the string
             * element, though this is not sound. This is a pragmatic
             * choice.
             *
             * @see narrowTypeByEquality
             *
             * @param source
             * @param target
             */
            function isCoercibleUnderDoubleEquals(source, target) {
                return ((source.flags & (8 /* Number */ | 4 /* String */ | 512 /* BooleanLiteral */)) !== 0)
                    && ((target.flags & (8 /* Number */ | 4 /* String */ | 16 /* Boolean */)) !== 0);
            }
            /**
             * Return true if type was inferred from an object literal, written as an object type literal, or is the shape of a module
             * with no call or construct signatures.
             */
            function isObjectTypeWithInferableIndex(type) {
                return type.flags & 2097152 /* Intersection */ ? ts.every(type.types, isObjectTypeWithInferableIndex) :
                    !!(type.symbol && (type.symbol.flags & (4096 /* ObjectLiteral */ | 2048 /* TypeLiteral */ | 384 /* Enum */ | 512 /* ValueModule */)) !== 0 &&
                        !typeHasCallOrConstructSignatures(type)) || !!(ts.getObjectFlags(type) & 2048 /* ReverseMapped */ && isObjectTypeWithInferableIndex(type.source));
            }
            function createSymbolWithType(source, type) {
                var symbol = createSymbol(source.flags, source.escapedName, ts.getCheckFlags(source) & 8 /* Readonly */);
                symbol.declarations = source.declarations;
                symbol.parent = source.parent;
                symbol.type = type;
                symbol.target = source;
                if (source.valueDeclaration) {
                    symbol.valueDeclaration = source.valueDeclaration;
                }
                var nameType = getSymbolLinks(source).nameType;
                if (nameType) {
                    symbol.nameType = nameType;
                }
                return symbol;
            }
            function transformTypeOfMembers(type, f) {
                var members = ts.createSymbolTable();
                for (var _i = 0, _a = getPropertiesOfObjectType(type); _i < _a.length; _i++) {
                    var property = _a[_i];
                    var original = getTypeOfSymbol(property);
                    var updated = f(original);
                    members.set(property.escapedName, updated === original ? property : createSymbolWithType(property, updated));
                }
                return members;
            }
            /**
             * If the the provided object literal is subject to the excess properties check,
             * create a new that is exempt. Recursively mark object literal members as exempt.
             * Leave signatures alone since they are not subject to the check.
             */
            function getRegularTypeOfObjectLiteral(type) {
                if (!(isObjectLiteralType(type) && ts.getObjectFlags(type) & 32768 /* FreshLiteral */)) {
                    return type;
                }
                var regularType = type.regularType;
                if (regularType) {
                    return regularType;
                }
                var resolved = type;
                var members = transformTypeOfMembers(type, getRegularTypeOfObjectLiteral);
                var regularNew = createAnonymousType(resolved.symbol, members, resolved.callSignatures, resolved.constructSignatures, resolved.stringIndexInfo, resolved.numberIndexInfo);
                regularNew.flags = resolved.flags;
                regularNew.objectFlags |= resolved.objectFlags & ~32768 /* FreshLiteral */;
                type.regularType = regularNew;
                return regularNew;
            }
            function createWideningContext(parent, propertyName, siblings) {
                return { parent: parent, propertyName: propertyName, siblings: siblings, resolvedProperties: undefined };
            }
            function getSiblingsOfContext(context) {
                if (!context.siblings) {
                    var siblings_1 = [];
                    for (var _i = 0, _a = getSiblingsOfContext(context.parent); _i < _a.length; _i++) {
                        var type = _a[_i];
                        if (isObjectLiteralType(type)) {
                            var prop = getPropertyOfObjectType(type, context.propertyName);
                            if (prop) {
                                forEachType(getTypeOfSymbol(prop), function (t) {
                                    siblings_1.push(t);
                                });
                            }
                        }
                    }
                    context.siblings = siblings_1;
                }
                return context.siblings;
            }
            function getPropertiesOfContext(context) {
                if (!context.resolvedProperties) {
                    var names = ts.createMap();
                    for (var _i = 0, _a = getSiblingsOfContext(context); _i < _a.length; _i++) {
                        var t = _a[_i];
                        if (isObjectLiteralType(t) && !(ts.getObjectFlags(t) & 1024 /* ContainsSpread */)) {
                            for (var _b = 0, _c = getPropertiesOfType(t); _b < _c.length; _b++) {
                                var prop = _c[_b];
                                names.set(prop.escapedName, prop);
                            }
                        }
                    }
                    context.resolvedProperties = ts.arrayFrom(names.values());
                }
                return context.resolvedProperties;
            }
            function getWidenedProperty(prop, context) {
                if (!(prop.flags & 4 /* Property */)) {
                    // Since get accessors already widen their return value there is no need to
                    // widen accessor based properties here.
                    return prop;
                }
                var original = getTypeOfSymbol(prop);
                var propContext = context && createWideningContext(context, prop.escapedName, /*siblings*/ undefined);
                var widened = getWidenedTypeWithContext(original, propContext);
                return widened === original ? prop : createSymbolWithType(prop, widened);
            }
            function getUndefinedProperty(prop) {
                var cached = undefinedProperties.get(prop.escapedName);
                if (cached) {
                    return cached;
                }
                var result = createSymbolWithType(prop, undefinedType);
                result.flags |= 16777216 /* Optional */;
                undefinedProperties.set(prop.escapedName, result);
                return result;
            }
            function getWidenedTypeOfObjectLiteral(type, context) {
                var members = ts.createSymbolTable();
                for (var _i = 0, _a = getPropertiesOfObjectType(type); _i < _a.length; _i++) {
                    var prop = _a[_i];
                    members.set(prop.escapedName, getWidenedProperty(prop, context));
                }
                if (context) {
                    for (var _b = 0, _c = getPropertiesOfContext(context); _b < _c.length; _b++) {
                        var prop = _c[_b];
                        if (!members.has(prop.escapedName)) {
                            members.set(prop.escapedName, getUndefinedProperty(prop));
                        }
                    }
                }
                var stringIndexInfo = getIndexInfoOfType(type, 0 /* String */);
                var numberIndexInfo = getIndexInfoOfType(type, 1 /* Number */);
                var result = createAnonymousType(type.symbol, members, ts.emptyArray, ts.emptyArray, stringIndexInfo && createIndexInfo(getWidenedType(stringIndexInfo.type), stringIndexInfo.isReadonly), numberIndexInfo && createIndexInfo(getWidenedType(numberIndexInfo.type), numberIndexInfo.isReadonly));
                result.objectFlags |= (ts.getObjectFlags(type) & (16384 /* JSLiteral */ | 2097152 /* NonInferrableType */)); // Retain js literal flag through widening
                return result;
            }
            function getWidenedType(type) {
                return getWidenedTypeWithContext(type, /*context*/ undefined);
            }
            function getWidenedTypeWithContext(type, context) {
                if (ts.getObjectFlags(type) & 1572864 /* RequiresWidening */) {
                    if (context === undefined && type.widened) {
                        return type.widened;
                    }
                    var result = void 0;
                    if (type.flags & (1 /* Any */ | 98304 /* Nullable */)) {
                        result = anyType;
                    }
                    else if (isObjectLiteralType(type)) {
                        result = getWidenedTypeOfObjectLiteral(type, context);
                    }
                    else if (type.flags & 1048576 /* Union */) {
                        var unionContext_1 = context || createWideningContext(/*parent*/ undefined, /*propertyName*/ undefined, type.types);
                        var widenedTypes = ts.sameMap(type.types, function (t) { return t.flags & 98304 /* Nullable */ ? t : getWidenedTypeWithContext(t, unionContext_1); });
                        // Widening an empty object literal transitions from a highly restrictive type to
                        // a highly inclusive one. For that reason we perform subtype reduction here if the
                        // union includes empty object types (e.g. reducing {} | string to just {}).
                        result = getUnionType(widenedTypes, ts.some(widenedTypes, isEmptyObjectType) ? 2 /* Subtype */ : 1 /* Literal */);
                    }
                    else if (type.flags & 2097152 /* Intersection */) {
                        result = getIntersectionType(ts.sameMap(type.types, getWidenedType));
                    }
                    else if (isArrayType(type) || isTupleType(type)) {
                        result = createTypeReference(type.target, ts.sameMap(getTypeArguments(type), getWidenedType));
                    }
                    if (result && context === undefined) {
                        type.widened = result;
                    }
                    return result || type;
                }
                return type;
            }
            /**
             * Reports implicit any errors that occur as a result of widening 'null' and 'undefined'
             * to 'any'. A call to reportWideningErrorsInType is normally accompanied by a call to
             * getWidenedType. But in some cases getWidenedType is called without reporting errors
             * (type argument inference is an example).
             *
             * The return value indicates whether an error was in fact reported. The particular circumstances
             * are on a best effort basis. Currently, if the null or undefined that causes widening is inside
             * an object literal property (arbitrarily deeply), this function reports an error. If no error is
             * reported, reportImplicitAnyError is a suitable fallback to report a general error.
             */
            function reportWideningErrorsInType(type) {
                var errorReported = false;
                if (ts.getObjectFlags(type) & 524288 /* ContainsWideningType */) {
                    if (type.flags & 1048576 /* Union */) {
                        if (ts.some(type.types, isEmptyObjectType)) {
                            errorReported = true;
                        }
                        else {
                            for (var _i = 0, _a = type.types; _i < _a.length; _i++) {
                                var t = _a[_i];
                                if (reportWideningErrorsInType(t)) {
                                    errorReported = true;
                                }
                            }
                        }
                    }
                    if (isArrayType(type) || isTupleType(type)) {
                        for (var _b = 0, _c = getTypeArguments(type); _b < _c.length; _b++) {
                            var t = _c[_b];
                            if (reportWideningErrorsInType(t)) {
                                errorReported = true;
                            }
                        }
                    }
                    if (isObjectLiteralType(type)) {
                        for (var _d = 0, _e = getPropertiesOfObjectType(type); _d < _e.length; _d++) {
                            var p = _e[_d];
                            var t = getTypeOfSymbol(p);
                            if (ts.getObjectFlags(t) & 524288 /* ContainsWideningType */) {
                                if (!reportWideningErrorsInType(t)) {
                                    error(p.valueDeclaration, ts.Diagnostics.Object_literal_s_property_0_implicitly_has_an_1_type, symbolToString(p), typeToString(getWidenedType(t)));
                                }
                                errorReported = true;
                            }
                        }
                    }
                }
                return errorReported;
            }
            function reportImplicitAny(declaration, type, wideningKind) {
                var typeAsString = typeToString(getWidenedType(type));
                if (ts.isInJSFile(declaration) && !ts.isCheckJsEnabledForFile(ts.getSourceFileOfNode(declaration), compilerOptions)) {
                    // Only report implicit any errors/suggestions in TS and ts-check JS files
                    return;
                }
                var diagnostic;
                switch (declaration.kind) {
                    case 209 /* BinaryExpression */:
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                        diagnostic = noImplicitAny ? ts.Diagnostics.Member_0_implicitly_has_an_1_type : ts.Diagnostics.Member_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage;
                        break;
                    case 156 /* Parameter */:
                        var param = declaration;
                        if (ts.isIdentifier(param.name) &&
                            (ts.isCallSignatureDeclaration(param.parent) || ts.isMethodSignature(param.parent) || ts.isFunctionTypeNode(param.parent)) &&
                            param.parent.parameters.indexOf(param) > -1 &&
                            (resolveName(param, param.name.escapedText, 788968 /* Type */, undefined, param.name.escapedText, /*isUse*/ true) ||
                                param.name.originalKeywordKind && ts.isTypeNodeKind(param.name.originalKeywordKind))) {
                            var newName = "arg" + param.parent.parameters.indexOf(param);
                            errorOrSuggestion(noImplicitAny, declaration, ts.Diagnostics.Parameter_has_a_name_but_no_type_Did_you_mean_0_Colon_1, newName, ts.declarationNameToString(param.name));
                            return;
                        }
                        diagnostic = declaration.dotDotDotToken ?
                            noImplicitAny ? ts.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type : ts.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage :
                            noImplicitAny ? ts.Diagnostics.Parameter_0_implicitly_has_an_1_type : ts.Diagnostics.Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage;
                        break;
                    case 191 /* BindingElement */:
                        diagnostic = ts.Diagnostics.Binding_element_0_implicitly_has_an_1_type;
                        if (!noImplicitAny) {
                            // Don't issue a suggestion for binding elements since the codefix doesn't yet support them.
                            return;
                        }
                        break;
                    case 300 /* JSDocFunctionType */:
                        error(declaration, ts.Diagnostics.Function_type_which_lacks_return_type_annotation_implicitly_has_an_0_return_type, typeAsString);
                        return;
                    case 244 /* FunctionDeclaration */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                        if (noImplicitAny && !declaration.name) {
                            if (wideningKind === 1 /* GeneratorYield */) {
                                error(declaration, ts.Diagnostics.Generator_implicitly_has_yield_type_0_because_it_does_not_yield_any_values_Consider_supplying_a_return_type_annotation, typeAsString);
                            }
                            else {
                                error(declaration, ts.Diagnostics.Function_expression_which_lacks_return_type_annotation_implicitly_has_an_0_return_type, typeAsString);
                            }
                            return;
                        }
                        diagnostic = !noImplicitAny ? ts.Diagnostics._0_implicitly_has_an_1_return_type_but_a_better_type_may_be_inferred_from_usage :
                            wideningKind === 1 /* GeneratorYield */ ? ts.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_yield_type :
                                ts.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type;
                        break;
                    case 186 /* MappedType */:
                        if (noImplicitAny) {
                            error(declaration, ts.Diagnostics.Mapped_object_type_implicitly_has_an_any_template_type);
                        }
                        return;
                    default:
                        diagnostic = noImplicitAny ? ts.Diagnostics.Variable_0_implicitly_has_an_1_type : ts.Diagnostics.Variable_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage;
                }
                errorOrSuggestion(noImplicitAny, declaration, diagnostic, ts.declarationNameToString(ts.getNameOfDeclaration(declaration)), typeAsString);
            }
            function reportErrorsFromWidening(declaration, type, wideningKind) {
                if (produceDiagnostics && noImplicitAny && ts.getObjectFlags(type) & 524288 /* ContainsWideningType */) {
                    // Report implicit any error within type if possible, otherwise report error on declaration
                    if (!reportWideningErrorsInType(type)) {
                        reportImplicitAny(declaration, type, wideningKind);
                    }
                }
            }
            function applyToParameterTypes(source, target, callback) {
                var sourceCount = getParameterCount(source);
                var targetCount = getParameterCount(target);
                var sourceRestType = getEffectiveRestType(source);
                var targetRestType = getEffectiveRestType(target);
                var targetNonRestCount = targetRestType ? targetCount - 1 : targetCount;
                var paramCount = sourceRestType ? targetNonRestCount : Math.min(sourceCount, targetNonRestCount);
                var sourceThisType = getThisTypeOfSignature(source);
                if (sourceThisType) {
                    var targetThisType = getThisTypeOfSignature(target);
                    if (targetThisType) {
                        callback(sourceThisType, targetThisType);
                    }
                }
                for (var i = 0; i < paramCount; i++) {
                    callback(getTypeAtPosition(source, i), getTypeAtPosition(target, i));
                }
                if (targetRestType) {
                    callback(getRestTypeAtPosition(source, paramCount), targetRestType);
                }
            }
            function applyToReturnTypes(source, target, callback) {
                var sourceTypePredicate = getTypePredicateOfSignature(source);
                var targetTypePredicate = getTypePredicateOfSignature(target);
                if (sourceTypePredicate && targetTypePredicate && typePredicateKindsMatch(sourceTypePredicate, targetTypePredicate) && sourceTypePredicate.type && targetTypePredicate.type) {
                    callback(sourceTypePredicate.type, targetTypePredicate.type);
                }
                else {
                    callback(getReturnTypeOfSignature(source), getReturnTypeOfSignature(target));
                }
            }
            function createInferenceContext(typeParameters, signature, flags, compareTypes) {
                return createInferenceContextWorker(typeParameters.map(createInferenceInfo), signature, flags, compareTypes || compareTypesAssignable);
            }
            function cloneInferenceContext(context, extraFlags) {
                if (extraFlags === void 0) {
                    extraFlags = 0;
                }
                return context && createInferenceContextWorker(ts.map(context.inferences, cloneInferenceInfo), context.signature, context.flags | extraFlags, context.compareTypes);
            }
            function createInferenceContextWorker(inferences, signature, flags, compareTypes) {
                var context = {
                    inferences: inferences,
                    signature: signature,
                    flags: flags,
                    compareTypes: compareTypes,
                    mapper: function (t) { return mapToInferredType(context, t, /*fix*/ true); },
                    nonFixingMapper: function (t) { return mapToInferredType(context, t, /*fix*/ false); },
                };
                return context;
            }
            function mapToInferredType(context, t, fix) {
                var inferences = context.inferences;
                for (var i = 0; i < inferences.length; i++) {
                    var inference = inferences[i];
                    if (t === inference.typeParameter) {
                        if (fix && !inference.isFixed) {
                            clearCachedInferences(inferences);
                            inference.isFixed = true;
                        }
                        return getInferredType(context, i);
                    }
                }
                return t;
            }
            function clearCachedInferences(inferences) {
                for (var _i = 0, inferences_1 = inferences; _i < inferences_1.length; _i++) {
                    var inference = inferences_1[_i];
                    if (!inference.isFixed) {
                        inference.inferredType = undefined;
                    }
                }
            }
            function createInferenceInfo(typeParameter) {
                return {
                    typeParameter: typeParameter,
                    candidates: undefined,
                    contraCandidates: undefined,
                    inferredType: undefined,
                    priority: undefined,
                    topLevel: true,
                    isFixed: false
                };
            }
            function cloneInferenceInfo(inference) {
                return {
                    typeParameter: inference.typeParameter,
                    candidates: inference.candidates && inference.candidates.slice(),
                    contraCandidates: inference.contraCandidates && inference.contraCandidates.slice(),
                    inferredType: inference.inferredType,
                    priority: inference.priority,
                    topLevel: inference.topLevel,
                    isFixed: inference.isFixed
                };
            }
            function cloneInferredPartOfContext(context) {
                var inferences = ts.filter(context.inferences, hasInferenceCandidates);
                return inferences.length ?
                    createInferenceContextWorker(ts.map(inferences, cloneInferenceInfo), context.signature, context.flags, context.compareTypes) :
                    undefined;
            }
            function getMapperFromContext(context) {
                return context && context.mapper;
            }
            // Return true if the given type could possibly reference a type parameter for which
            // we perform type inference (i.e. a type parameter of a generic function). We cache
            // results for union and intersection types for performance reasons.
            function couldContainTypeVariables(type) {
                var objectFlags = ts.getObjectFlags(type);
                return !!(type.flags & 63176704 /* Instantiable */ ||
                    objectFlags & 4 /* Reference */ && (type.node || ts.forEach(getTypeArguments(type), couldContainTypeVariables)) ||
                    objectFlags & 16 /* Anonymous */ && type.symbol && type.symbol.flags & (16 /* Function */ | 8192 /* Method */ | 32 /* Class */ | 2048 /* TypeLiteral */ | 4096 /* ObjectLiteral */) && type.symbol.declarations ||
                    objectFlags & (32 /* Mapped */ | 131072 /* ObjectRestType */) ||
                    type.flags & 3145728 /* UnionOrIntersection */ && !(type.flags & 1024 /* EnumLiteral */) && couldUnionOrIntersectionContainTypeVariables(type));
            }
            function couldUnionOrIntersectionContainTypeVariables(type) {
                if (type.couldContainTypeVariables === undefined) {
                    type.couldContainTypeVariables = ts.some(type.types, couldContainTypeVariables);
                }
                return type.couldContainTypeVariables;
            }
            function isTypeParameterAtTopLevel(type, typeParameter) {
                return !!(type === typeParameter ||
                    type.flags & 3145728 /* UnionOrIntersection */ && ts.some(type.types, function (t) { return isTypeParameterAtTopLevel(t, typeParameter); }) ||
                    type.flags & 16777216 /* Conditional */ && (isTypeParameterAtTopLevel(getTrueTypeFromConditionalType(type), typeParameter) ||
                        isTypeParameterAtTopLevel(getFalseTypeFromConditionalType(type), typeParameter)));
            }
            /** Create an object with properties named in the string literal type. Every property has type `any` */
            function createEmptyObjectTypeFromStringLiteral(type) {
                var members = ts.createSymbolTable();
                forEachType(type, function (t) {
                    if (!(t.flags & 128 /* StringLiteral */)) {
                        return;
                    }
                    var name = ts.escapeLeadingUnderscores(t.value);
                    var literalProp = createSymbol(4 /* Property */, name);
                    literalProp.type = anyType;
                    if (t.symbol) {
                        literalProp.declarations = t.symbol.declarations;
                        literalProp.valueDeclaration = t.symbol.valueDeclaration;
                    }
                    members.set(name, literalProp);
                });
                var indexInfo = type.flags & 4 /* String */ ? createIndexInfo(emptyObjectType, /*isReadonly*/ false) : undefined;
                return createAnonymousType(undefined, members, ts.emptyArray, ts.emptyArray, indexInfo, undefined);
            }
            /**
             * Infer a suitable input type for a homomorphic mapped type { [P in keyof T]: X }. We construct
             * an object type with the same set of properties as the source type, where the type of each
             * property is computed by inferring from the source property type to X for the type
             * variable T[P] (i.e. we treat the type T[P] as the type variable we're inferring for).
             */
            function inferTypeForHomomorphicMappedType(source, target, constraint) {
                var key = source.id + "," + target.id + "," + constraint.id;
                if (reverseMappedCache.has(key)) {
                    return reverseMappedCache.get(key);
                }
                reverseMappedCache.set(key, undefined);
                var type = createReverseMappedType(source, target, constraint);
                reverseMappedCache.set(key, type);
                return type;
            }
            // We consider a type to be partially inferable if it isn't marked non-inferable or if it is
            // an object literal type with at least one property of an inferable type. For example, an object
            // literal { a: 123, b: x => true } is marked non-inferable because it contains a context sensitive
            // arrow function, but is considered partially inferable because property 'a' has an inferable type.
            function isPartiallyInferableType(type) {
                return !(ts.getObjectFlags(type) & 2097152 /* NonInferrableType */) ||
                    isObjectLiteralType(type) && ts.some(getPropertiesOfType(type), function (prop) { return isPartiallyInferableType(getTypeOfSymbol(prop)); });
            }
            function createReverseMappedType(source, target, constraint) {
                // We consider a source type reverse mappable if it has a string index signature or if
                // it has one or more properties and is of a partially inferable type.
                if (!(getIndexInfoOfType(source, 0 /* String */) || getPropertiesOfType(source).length !== 0 && isPartiallyInferableType(source))) {
                    return undefined;
                }
                // For arrays and tuples we infer new arrays and tuples where the reverse mapping has been
                // applied to the element type(s).
                if (isArrayType(source)) {
                    return createArrayType(inferReverseMappedType(getTypeArguments(source)[0], target, constraint), isReadonlyArrayType(source));
                }
                if (isTupleType(source)) {
                    var elementTypes = ts.map(getTypeArguments(source), function (t) { return inferReverseMappedType(t, target, constraint); });
                    var minLength = getMappedTypeModifiers(target) & 4 /* IncludeOptional */ ?
                        getTypeReferenceArity(source) - (source.target.hasRestElement ? 1 : 0) : source.target.minLength;
                    return createTupleType(elementTypes, minLength, source.target.hasRestElement, source.target.readonly, source.target.associatedNames);
                }
                // For all other object types we infer a new object type where the reverse mapping has been
                // applied to the type of each property.
                var reversed = createObjectType(2048 /* ReverseMapped */ | 16 /* Anonymous */, /*symbol*/ undefined);
                reversed.source = source;
                reversed.mappedType = target;
                reversed.constraintType = constraint;
                return reversed;
            }
            function getTypeOfReverseMappedSymbol(symbol) {
                return inferReverseMappedType(symbol.propertyType, symbol.mappedType, symbol.constraintType);
            }
            function inferReverseMappedType(sourceType, target, constraint) {
                var typeParameter = getIndexedAccessType(constraint.type, getTypeParameterFromMappedType(target));
                var templateType = getTemplateTypeFromMappedType(target);
                var inference = createInferenceInfo(typeParameter);
                inferTypes([inference], sourceType, templateType);
                return getTypeFromInference(inference) || unknownType;
            }
            function getUnmatchedProperties(source, target, requireOptionalProperties, matchDiscriminantProperties) {
                var properties, _i, properties_2, targetProp, sourceProp, targetType, sourceType;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            properties = getPropertiesOfType(target);
                            _i = 0, properties_2 = properties;
                            _a.label = 1;
                        case 1:
                            if (!(_i < properties_2.length))
                                return [3 /*break*/, 6];
                            targetProp = properties_2[_i];
                            // TODO: remove this when we support static private identifier fields and find other solutions to get privateNamesAndStaticFields test to pass
                            if (isStaticPrivateIdentifierProperty(targetProp)) {
                                return [3 /*break*/, 5];
                            }
                            if (!(requireOptionalProperties || !(targetProp.flags & 16777216 /* Optional */ || ts.getCheckFlags(targetProp) & 48 /* Partial */)))
                                return [3 /*break*/, 5];
                            sourceProp = getPropertyOfType(source, targetProp.escapedName);
                            if (!!sourceProp)
                                return [3 /*break*/, 3];
                            return [4 /*yield*/, targetProp];
                        case 2:
                            _a.sent();
                            return [3 /*break*/, 5];
                        case 3:
                            if (!matchDiscriminantProperties)
                                return [3 /*break*/, 5];
                            targetType = getTypeOfSymbol(targetProp);
                            if (!(targetType.flags & 109440 /* Unit */))
                                return [3 /*break*/, 5];
                            sourceType = getTypeOfSymbol(sourceProp);
                            if (!!(sourceType.flags & 1 /* Any */ || getRegularTypeOfLiteralType(sourceType) === getRegularTypeOfLiteralType(targetType)))
                                return [3 /*break*/, 5];
                            return [4 /*yield*/, targetProp];
                        case 4:
                            _a.sent();
                            _a.label = 5;
                        case 5:
                            _i++;
                            return [3 /*break*/, 1];
                        case 6: return [2 /*return*/];
                    }
                });
            }
            function getUnmatchedProperty(source, target, requireOptionalProperties, matchDiscriminantProperties) {
                var result = getUnmatchedProperties(source, target, requireOptionalProperties, matchDiscriminantProperties).next();
                if (!result.done)
                    return result.value;
            }
            function tupleTypesDefinitelyUnrelated(source, target) {
                return target.target.minLength > source.target.minLength ||
                    !getRestTypeOfTupleType(target) && (!!getRestTypeOfTupleType(source) || getLengthOfTupleType(target) < getLengthOfTupleType(source));
            }
            function typesDefinitelyUnrelated(source, target) {
                // Two tuple types with incompatible arities are definitely unrelated.
                // Two object types that each have a property that is unmatched in the other are definitely unrelated.
                return isTupleType(source) && isTupleType(target) && tupleTypesDefinitelyUnrelated(source, target) ||
                    !!getUnmatchedProperty(source, target, /*requireOptionalProperties*/ false, /*matchDiscriminantProperties*/ true) &&
                        !!getUnmatchedProperty(target, source, /*requireOptionalProperties*/ false, /*matchDiscriminantProperties*/ true);
            }
            function getTypeFromInference(inference) {
                return inference.candidates ? getUnionType(inference.candidates, 2 /* Subtype */) :
                    inference.contraCandidates ? getIntersectionType(inference.contraCandidates) :
                        undefined;
            }
            function hasSkipDirectInferenceFlag(node) {
                return !!getNodeLinks(node).skipDirectInference;
            }
            function isFromInferenceBlockedSource(type) {
                return !!(type.symbol && ts.some(type.symbol.declarations, hasSkipDirectInferenceFlag));
            }
            function inferTypes(inferences, originalSource, originalTarget, priority, contravariant) {
                if (priority === void 0) {
                    priority = 0;
                }
                if (contravariant === void 0) {
                    contravariant = false;
                }
                var symbolStack;
                var visited;
                var bivariant = false;
                var propagationType;
                var inferencePriority = 512 /* MaxValue */;
                var allowComplexConstraintInference = true;
                inferFromTypes(originalSource, originalTarget);
                function inferFromTypes(source, target) {
                    if (!couldContainTypeVariables(target)) {
                        return;
                    }
                    if (source === wildcardType) {
                        // We are inferring from an 'any' type. We want to infer this type for every type parameter
                        // referenced in the target type, so we record it as the propagation type and infer from the
                        // target to itself. Then, as we find candidates we substitute the propagation type.
                        var savePropagationType = propagationType;
                        propagationType = source;
                        inferFromTypes(target, target);
                        propagationType = savePropagationType;
                        return;
                    }
                    if (source.aliasSymbol && source.aliasTypeArguments && source.aliasSymbol === target.aliasSymbol) {
                        // Source and target are types originating in the same generic type alias declaration.
                        // Simply infer from source type arguments to target type arguments.
                        inferFromTypeArguments(source.aliasTypeArguments, target.aliasTypeArguments, getAliasVariances(source.aliasSymbol));
                        return;
                    }
                    if (source === target && source.flags & 3145728 /* UnionOrIntersection */) {
                        // When source and target are the same union or intersection type, just relate each constituent
                        // type to itself.
                        for (var _i = 0, _a = source.types; _i < _a.length; _i++) {
                            var t = _a[_i];
                            inferFromTypes(t, t);
                        }
                        return;
                    }
                    if (target.flags & 1048576 /* Union */) {
                        // First, infer between identically matching source and target constituents and remove the
                        // matching types.
                        var _b = inferFromMatchingTypes(source.flags & 1048576 /* Union */ ? source.types : [source], target.types, isTypeOrBaseIdenticalTo), tempSources = _b[0], tempTargets = _b[1];
                        // Next, infer between closely matching source and target constituents and remove
                        // the matching types. Types closely match when they are instantiations of the same
                        // object type or instantiations of the same type alias.
                        var _c = inferFromMatchingTypes(tempSources, tempTargets, isTypeCloselyMatchedBy), sources = _c[0], targets = _c[1];
                        if (targets.length === 0) {
                            return;
                        }
                        target = getUnionType(targets);
                        if (sources.length === 0) {
                            // All source constituents have been matched and there is nothing further to infer from.
                            // However, simply making no inferences is undesirable because it could ultimately mean
                            // inferring a type parameter constraint. Instead, make a lower priority inference from
                            // the full source to whatever remains in the target. For example, when inferring from
                            // string to 'string | T', make a lower priority inference of string for T.
                            inferWithPriority(source, target, 1 /* NakedTypeVariable */);
                            return;
                        }
                        source = getUnionType(sources);
                    }
                    else if (target.flags & 2097152 /* Intersection */ && ts.some(target.types, function (t) { return !!getInferenceInfoForType(t) || (isGenericMappedType(t) && !!getInferenceInfoForType(getHomomorphicTypeVariable(t) || neverType)); })) {
                        // We reduce intersection types only when they contain naked type parameters. For example, when
                        // inferring from 'string[] & { extra: any }' to 'string[] & T' we want to remove string[] and
                        // infer { extra: any } for T. But when inferring to 'string[] & Iterable<T>' we want to keep the
                        // string[] on the source side and infer string for T.
                        // Likewise, we consider a homomorphic mapped type constrainted to the target type parameter as similar to a "naked type variable"
                        // in such scenarios.
                        if (!(source.flags & 1048576 /* Union */)) {
                            // Infer between identically matching source and target constituents and remove the matching types.
                            var _d = inferFromMatchingTypes(source.flags & 2097152 /* Intersection */ ? source.types : [source], target.types, isTypeIdenticalTo), sources = _d[0], targets = _d[1];
                            if (sources.length === 0 || targets.length === 0) {
                                return;
                            }
                            source = getIntersectionType(sources);
                            target = getIntersectionType(targets);
                        }
                    }
                    else if (target.flags & (8388608 /* IndexedAccess */ | 33554432 /* Substitution */)) {
                        target = getActualTypeVariable(target);
                    }
                    if (target.flags & 8650752 /* TypeVariable */) {
                        // If target is a type parameter, make an inference, unless the source type contains
                        // the anyFunctionType (the wildcard type that's used to avoid contextually typing functions).
                        // Because the anyFunctionType is internal, it should not be exposed to the user by adding
                        // it as an inference candidate. Hopefully, a better candidate will come along that does
                        // not contain anyFunctionType when we come back to this argument for its second round
                        // of inference. Also, we exclude inferences for silentNeverType (which is used as a wildcard
                        // when constructing types from type parameters that had no inference candidates).
                        if (ts.getObjectFlags(source) & 2097152 /* NonInferrableType */ || source === nonInferrableAnyType || source === silentNeverType ||
                            (priority & 32 /* ReturnType */ && (source === autoType || source === autoArrayType)) || isFromInferenceBlockedSource(source)) {
                            return;
                        }
                        var inference = getInferenceInfoForType(target);
                        if (inference) {
                            if (!inference.isFixed) {
                                if (inference.priority === undefined || priority < inference.priority) {
                                    inference.candidates = undefined;
                                    inference.contraCandidates = undefined;
                                    inference.topLevel = true;
                                    inference.priority = priority;
                                }
                                if (priority === inference.priority) {
                                    var candidate = propagationType || source;
                                    // We make contravariant inferences only if we are in a pure contravariant position,
                                    // i.e. only if we have not descended into a bivariant position.
                                    if (contravariant && !bivariant) {
                                        if (!ts.contains(inference.contraCandidates, candidate)) {
                                            inference.contraCandidates = ts.append(inference.contraCandidates, candidate);
                                            clearCachedInferences(inferences);
                                        }
                                    }
                                    else if (!ts.contains(inference.candidates, candidate)) {
                                        inference.candidates = ts.append(inference.candidates, candidate);
                                        clearCachedInferences(inferences);
                                    }
                                }
                                if (!(priority & 32 /* ReturnType */) && target.flags & 262144 /* TypeParameter */ && inference.topLevel && !isTypeParameterAtTopLevel(originalTarget, target)) {
                                    inference.topLevel = false;
                                    clearCachedInferences(inferences);
                                }
                            }
                            inferencePriority = Math.min(inferencePriority, priority);
                            return;
                        }
                        else {
                            // Infer to the simplified version of an indexed access, if possible, to (hopefully) expose more bare type parameters to the inference engine
                            var simplified = getSimplifiedType(target, /*writing*/ false);
                            if (simplified !== target) {
                                invokeOnce(source, simplified, inferFromTypes);
                            }
                            else if (target.flags & 8388608 /* IndexedAccess */) {
                                var indexType = getSimplifiedType(target.indexType, /*writing*/ false);
                                // Generally simplifications of instantiable indexes are avoided to keep relationship checking correct, however if our target is an access, we can consider
                                // that key of that access to be "instantiated", since we're looking to find the infernce goal in any way we can.
                                if (indexType.flags & 63176704 /* Instantiable */) {
                                    var simplified_1 = distributeIndexOverObjectType(getSimplifiedType(target.objectType, /*writing*/ false), indexType, /*writing*/ false);
                                    if (simplified_1 && simplified_1 !== target) {
                                        invokeOnce(source, simplified_1, inferFromTypes);
                                    }
                                }
                            }
                        }
                    }
                    if (ts.getObjectFlags(source) & 4 /* Reference */ && ts.getObjectFlags(target) & 4 /* Reference */ && (source.target === target.target || isArrayType(source) && isArrayType(target)) &&
                        !(source.node && target.node)) {
                        // If source and target are references to the same generic type, infer from type arguments
                        inferFromTypeArguments(getTypeArguments(source), getTypeArguments(target), getVariances(source.target));
                    }
                    else if (source.flags & 4194304 /* Index */ && target.flags & 4194304 /* Index */) {
                        contravariant = !contravariant;
                        inferFromTypes(source.type, target.type);
                        contravariant = !contravariant;
                    }
                    else if ((isLiteralType(source) || source.flags & 4 /* String */) && target.flags & 4194304 /* Index */) {
                        var empty = createEmptyObjectTypeFromStringLiteral(source);
                        contravariant = !contravariant;
                        inferWithPriority(empty, target.type, 64 /* LiteralKeyof */);
                        contravariant = !contravariant;
                    }
                    else if (source.flags & 8388608 /* IndexedAccess */ && target.flags & 8388608 /* IndexedAccess */) {
                        inferFromTypes(source.objectType, target.objectType);
                        inferFromTypes(source.indexType, target.indexType);
                    }
                    else if (source.flags & 16777216 /* Conditional */ && target.flags & 16777216 /* Conditional */) {
                        inferFromTypes(source.checkType, target.checkType);
                        inferFromTypes(source.extendsType, target.extendsType);
                        inferFromTypes(getTrueTypeFromConditionalType(source), getTrueTypeFromConditionalType(target));
                        inferFromTypes(getFalseTypeFromConditionalType(source), getFalseTypeFromConditionalType(target));
                    }
                    else if (target.flags & 16777216 /* Conditional */) {
                        var savePriority = priority;
                        priority |= contravariant ? 16 /* ContravariantConditional */ : 0;
                        var targetTypes = [getTrueTypeFromConditionalType(target), getFalseTypeFromConditionalType(target)];
                        inferToMultipleTypes(source, targetTypes, target.flags);
                        priority = savePriority;
                    }
                    else if (target.flags & 3145728 /* UnionOrIntersection */) {
                        inferToMultipleTypes(source, target.types, target.flags);
                    }
                    else if (source.flags & 1048576 /* Union */) {
                        // Source is a union or intersection type, infer from each constituent type
                        var sourceTypes = source.types;
                        for (var _e = 0, sourceTypes_3 = sourceTypes; _e < sourceTypes_3.length; _e++) {
                            var sourceType = sourceTypes_3[_e];
                            inferFromTypes(sourceType, target);
                        }
                    }
                    else {
                        if (!(priority & 128 /* NoConstraints */ && source.flags & (2097152 /* Intersection */ | 63176704 /* Instantiable */))) {
                            var apparentSource = getApparentType(source);
                            // getApparentType can return _any_ type, since an indexed access or conditional may simplify to any other type.
                            // If that occurs and it doesn't simplify to an object or intersection, we'll need to restart `inferFromTypes`
                            // with the simplified source.
                            if (apparentSource !== source && allowComplexConstraintInference && !(apparentSource.flags & (524288 /* Object */ | 2097152 /* Intersection */))) {
                                // TODO: The `allowComplexConstraintInference` flag is a hack! This forbids inference from complex constraints within constraints!
                                // This isn't required algorithmically, but rather is used to lower the memory burden caused by performing inference
                                // that is _too good_ in projects with complicated constraints (eg, fp-ts). In such cases, if we did not limit ourselves
                                // here, we might produce more valid inferences for types, causing us to do more checks and perform more instantiations
                                // (in addition to the extra stack depth here) which, in turn, can push the already close process over its limit.
                                // TL;DR: If we ever become generally more memory efficient (or our resource budget ever increases), we should just
                                // remove this `allowComplexConstraintInference` flag.
                                allowComplexConstraintInference = false;
                                return inferFromTypes(apparentSource, target);
                            }
                            source = apparentSource;
                        }
                        if (source.flags & (524288 /* Object */ | 2097152 /* Intersection */)) {
                            invokeOnce(source, target, inferFromObjectTypes);
                        }
                    }
                    if (source.flags & 25165824 /* Simplifiable */) {
                        var simplified = getSimplifiedType(source, contravariant);
                        if (simplified !== source) {
                            inferFromTypes(simplified, target);
                        }
                    }
                }
                function inferWithPriority(source, target, newPriority) {
                    var savePriority = priority;
                    priority |= newPriority;
                    inferFromTypes(source, target);
                    priority = savePriority;
                }
                function invokeOnce(source, target, action) {
                    var key = source.id + "," + target.id;
                    var status = visited && visited.get(key);
                    if (status !== undefined) {
                        inferencePriority = Math.min(inferencePriority, status);
                        return;
                    }
                    (visited || (visited = ts.createMap())).set(key, -1 /* Circularity */);
                    var saveInferencePriority = inferencePriority;
                    inferencePriority = 512 /* MaxValue */;
                    action(source, target);
                    visited.set(key, inferencePriority);
                    inferencePriority = Math.min(inferencePriority, saveInferencePriority);
                }
                function inferFromMatchingTypes(sources, targets, matches) {
                    var matchedSources;
                    var matchedTargets;
                    for (var _i = 0, targets_1 = targets; _i < targets_1.length; _i++) {
                        var t = targets_1[_i];
                        for (var _a = 0, sources_1 = sources; _a < sources_1.length; _a++) {
                            var s = sources_1[_a];
                            if (matches(s, t)) {
                                inferFromTypes(s, t);
                                matchedSources = ts.appendIfUnique(matchedSources, s);
                                matchedTargets = ts.appendIfUnique(matchedTargets, t);
                            }
                        }
                    }
                    return [
                        matchedSources ? ts.filter(sources, function (t) { return !ts.contains(matchedSources, t); }) : sources,
                        matchedTargets ? ts.filter(targets, function (t) { return !ts.contains(matchedTargets, t); }) : targets,
                    ];
                }
                function inferFromTypeArguments(sourceTypes, targetTypes, variances) {
                    var count = sourceTypes.length < targetTypes.length ? sourceTypes.length : targetTypes.length;
                    for (var i = 0; i < count; i++) {
                        if (i < variances.length && (variances[i] & 7 /* VarianceMask */) === 2 /* Contravariant */) {
                            inferFromContravariantTypes(sourceTypes[i], targetTypes[i]);
                        }
                        else {
                            inferFromTypes(sourceTypes[i], targetTypes[i]);
                        }
                    }
                }
                function inferFromContravariantTypes(source, target) {
                    if (strictFunctionTypes || priority & 256 /* AlwaysStrict */) {
                        contravariant = !contravariant;
                        inferFromTypes(source, target);
                        contravariant = !contravariant;
                    }
                    else {
                        inferFromTypes(source, target);
                    }
                }
                function getInferenceInfoForType(type) {
                    if (type.flags & 8650752 /* TypeVariable */) {
                        for (var _i = 0, inferences_2 = inferences; _i < inferences_2.length; _i++) {
                            var inference = inferences_2[_i];
                            if (type === inference.typeParameter) {
                                return inference;
                            }
                        }
                    }
                    return undefined;
                }
                function getSingleTypeVariableFromIntersectionTypes(types) {
                    var typeVariable;
                    for (var _i = 0, types_13 = types; _i < types_13.length; _i++) {
                        var type = types_13[_i];
                        var t = type.flags & 2097152 /* Intersection */ && ts.find(type.types, function (t) { return !!getInferenceInfoForType(t); });
                        if (!t || typeVariable && t !== typeVariable) {
                            return undefined;
                        }
                        typeVariable = t;
                    }
                    return typeVariable;
                }
                function inferToMultipleTypes(source, targets, targetFlags) {
                    var typeVariableCount = 0;
                    if (targetFlags & 1048576 /* Union */) {
                        var nakedTypeVariable = void 0;
                        var sources = source.flags & 1048576 /* Union */ ? source.types : [source];
                        var matched_1 = new Array(sources.length);
                        var inferenceCircularity = false;
                        // First infer to types that are not naked type variables. For each source type we
                        // track whether inferences were made from that particular type to some target with
                        // equal priority (i.e. of equal quality) to what we would infer for a naked type
                        // parameter.
                        for (var _i = 0, targets_2 = targets; _i < targets_2.length; _i++) {
                            var t = targets_2[_i];
                            if (getInferenceInfoForType(t)) {
                                nakedTypeVariable = t;
                                typeVariableCount++;
                            }
                            else {
                                for (var i = 0; i < sources.length; i++) {
                                    var saveInferencePriority = inferencePriority;
                                    inferencePriority = 512 /* MaxValue */;
                                    inferFromTypes(sources[i], t);
                                    if (inferencePriority === priority)
                                        matched_1[i] = true;
                                    inferenceCircularity = inferenceCircularity || inferencePriority === -1 /* Circularity */;
                                    inferencePriority = Math.min(inferencePriority, saveInferencePriority);
                                }
                            }
                        }
                        if (typeVariableCount === 0) {
                            // If every target is an intersection of types containing a single naked type variable,
                            // make a lower priority inference to that type variable. This handles inferring from
                            // 'A | B' to 'T & (X | Y)' where we want to infer 'A | B' for T.
                            var intersectionTypeVariable = getSingleTypeVariableFromIntersectionTypes(targets);
                            if (intersectionTypeVariable) {
                                inferWithPriority(source, intersectionTypeVariable, 1 /* NakedTypeVariable */);
                            }
                            return;
                        }
                        // If the target has a single naked type variable and no inference circularities were
                        // encountered above (meaning we explored the types fully), create a union of the source
                        // types from which no inferences have been made so far and infer from that union to the
                        // naked type variable.
                        if (typeVariableCount === 1 && !inferenceCircularity) {
                            var unmatched = ts.flatMap(sources, function (s, i) { return matched_1[i] ? undefined : s; });
                            if (unmatched.length) {
                                inferFromTypes(getUnionType(unmatched), nakedTypeVariable);
                                return;
                            }
                        }
                    }
                    else {
                        // We infer from types that are not naked type variables first so that inferences we
                        // make from nested naked type variables and given slightly higher priority by virtue
                        // of being first in the candidates array.
                        for (var _a = 0, targets_3 = targets; _a < targets_3.length; _a++) {
                            var t = targets_3[_a];
                            if (getInferenceInfoForType(t)) {
                                typeVariableCount++;
                            }
                            else {
                                inferFromTypes(source, t);
                            }
                        }
                    }
                    // Inferences directly to naked type variables are given lower priority as they are
                    // less specific. For example, when inferring from Promise<string> to T | Promise<T>,
                    // we want to infer string for T, not Promise<string> | string. For intersection types
                    // we only infer to single naked type variables.
                    if (targetFlags & 2097152 /* Intersection */ ? typeVariableCount === 1 : typeVariableCount > 0) {
                        for (var _b = 0, targets_4 = targets; _b < targets_4.length; _b++) {
                            var t = targets_4[_b];
                            if (getInferenceInfoForType(t)) {
                                inferWithPriority(source, t, 1 /* NakedTypeVariable */);
                            }
                        }
                    }
                }
                function inferToMappedType(source, target, constraintType) {
                    if (constraintType.flags & 1048576 /* Union */) {
                        var result = false;
                        for (var _i = 0, _a = constraintType.types; _i < _a.length; _i++) {
                            var type = _a[_i];
                            result = inferToMappedType(source, target, type) || result;
                        }
                        return result;
                    }
                    if (constraintType.flags & 4194304 /* Index */) {
                        // We're inferring from some source type S to a homomorphic mapped type { [P in keyof T]: X },
                        // where T is a type variable. Use inferTypeForHomomorphicMappedType to infer a suitable source
                        // type and then make a secondary inference from that type to T. We make a secondary inference
                        // such that direct inferences to T get priority over inferences to Partial<T>, for example.
                        var inference = getInferenceInfoForType(constraintType.type);
                        if (inference && !inference.isFixed && !isFromInferenceBlockedSource(source)) {
                            var inferredType = inferTypeForHomomorphicMappedType(source, target, constraintType);
                            if (inferredType) {
                                // We assign a lower priority to inferences made from types containing non-inferrable
                                // types because we may only have a partial result (i.e. we may have failed to make
                                // reverse inferences for some properties).
                                inferWithPriority(inferredType, inference.typeParameter, ts.getObjectFlags(source) & 2097152 /* NonInferrableType */ ?
                                    4 /* PartialHomomorphicMappedType */ :
                                    2 /* HomomorphicMappedType */);
                            }
                        }
                        return true;
                    }
                    if (constraintType.flags & 262144 /* TypeParameter */) {
                        // We're inferring from some source type S to a mapped type { [P in K]: X }, where K is a type
                        // parameter. First infer from 'keyof S' to K.
                        inferWithPriority(getIndexType(source), constraintType, 8 /* MappedTypeConstraint */);
                        // If K is constrained to a type C, also infer to C. Thus, for a mapped type { [P in K]: X },
                        // where K extends keyof T, we make the same inferences as for a homomorphic mapped type
                        // { [P in keyof T]: X }. This enables us to make meaningful inferences when the target is a
                        // Pick<T, K>.
                        var extendedConstraint = getConstraintOfType(constraintType);
                        if (extendedConstraint && inferToMappedType(source, target, extendedConstraint)) {
                            return true;
                        }
                        // If no inferences can be made to K's constraint, infer from a union of the property types
                        // in the source to the template type X.
                        var propTypes = ts.map(getPropertiesOfType(source), getTypeOfSymbol);
                        var stringIndexType = getIndexTypeOfType(source, 0 /* String */);
                        var numberIndexInfo = getNonEnumNumberIndexInfo(source);
                        var numberIndexType = numberIndexInfo && numberIndexInfo.type;
                        inferFromTypes(getUnionType(ts.append(ts.append(propTypes, stringIndexType), numberIndexType)), getTemplateTypeFromMappedType(target));
                        return true;
                    }
                    return false;
                }
                function inferFromObjectTypes(source, target) {
                    // If we are already processing another target type with the same associated symbol (such as
                    // an instantiation of the same generic type), we do not explore this target as it would yield
                    // no further inferences. We exclude the static side of classes from this check since it shares
                    // its symbol with the instance side which would lead to false positives.
                    var isNonConstructorObject = target.flags & 524288 /* Object */ &&
                        !(ts.getObjectFlags(target) & 16 /* Anonymous */ && target.symbol && target.symbol.flags & 32 /* Class */);
                    var symbol = isNonConstructorObject ? target.symbol : undefined;
                    if (symbol) {
                        if (ts.contains(symbolStack, symbol)) {
                            inferencePriority = -1 /* Circularity */;
                            return;
                        }
                        (symbolStack || (symbolStack = [])).push(symbol);
                        inferFromObjectTypesWorker(source, target);
                        symbolStack.pop();
                    }
                    else {
                        inferFromObjectTypesWorker(source, target);
                    }
                }
                function inferFromObjectTypesWorker(source, target) {
                    if (ts.getObjectFlags(source) & 4 /* Reference */ && ts.getObjectFlags(target) & 4 /* Reference */ && (source.target === target.target || isArrayType(source) && isArrayType(target))) {
                        // If source and target are references to the same generic type, infer from type arguments
                        inferFromTypeArguments(getTypeArguments(source), getTypeArguments(target), getVariances(source.target));
                        return;
                    }
                    if (isGenericMappedType(source) && isGenericMappedType(target)) {
                        // The source and target types are generic types { [P in S]: X } and { [P in T]: Y }, so we infer
                        // from S to T and from X to Y.
                        inferFromTypes(getConstraintTypeFromMappedType(source), getConstraintTypeFromMappedType(target));
                        inferFromTypes(getTemplateTypeFromMappedType(source), getTemplateTypeFromMappedType(target));
                    }
                    if (ts.getObjectFlags(target) & 32 /* Mapped */) {
                        var constraintType = getConstraintTypeFromMappedType(target);
                        if (inferToMappedType(source, target, constraintType)) {
                            return;
                        }
                    }
                    // Infer from the members of source and target only if the two types are possibly related
                    if (!typesDefinitelyUnrelated(source, target)) {
                        inferFromProperties(source, target);
                        inferFromSignatures(source, target, 0 /* Call */);
                        inferFromSignatures(source, target, 1 /* Construct */);
                        inferFromIndexTypes(source, target);
                    }
                }
                function inferFromProperties(source, target) {
                    if (isArrayType(source) || isTupleType(source)) {
                        if (isTupleType(target)) {
                            var sourceLength = isTupleType(source) ? getLengthOfTupleType(source) : 0;
                            var targetLength = getLengthOfTupleType(target);
                            var sourceRestType = isTupleType(source) ? getRestTypeOfTupleType(source) : getElementTypeOfArrayType(source);
                            var targetRestType = getRestTypeOfTupleType(target);
                            var fixedLength = targetLength < sourceLength || sourceRestType ? targetLength : sourceLength;
                            for (var i = 0; i < fixedLength; i++) {
                                inferFromTypes(i < sourceLength ? getTypeArguments(source)[i] : sourceRestType, getTypeArguments(target)[i]);
                            }
                            if (targetRestType) {
                                var types = fixedLength < sourceLength ? getTypeArguments(source).slice(fixedLength, sourceLength) : [];
                                if (sourceRestType) {
                                    types.push(sourceRestType);
                                }
                                if (types.length) {
                                    inferFromTypes(getUnionType(types), targetRestType);
                                }
                            }
                            return;
                        }
                        if (isArrayType(target)) {
                            inferFromIndexTypes(source, target);
                            return;
                        }
                    }
                    var properties = getPropertiesOfObjectType(target);
                    for (var _i = 0, properties_3 = properties; _i < properties_3.length; _i++) {
                        var targetProp = properties_3[_i];
                        var sourceProp = getPropertyOfType(source, targetProp.escapedName);
                        if (sourceProp) {
                            inferFromTypes(getTypeOfSymbol(sourceProp), getTypeOfSymbol(targetProp));
                        }
                    }
                }
                function inferFromSignatures(source, target, kind) {
                    var sourceSignatures = getSignaturesOfType(source, kind);
                    var targetSignatures = getSignaturesOfType(target, kind);
                    var sourceLen = sourceSignatures.length;
                    var targetLen = targetSignatures.length;
                    var len = sourceLen < targetLen ? sourceLen : targetLen;
                    var skipParameters = !!(ts.getObjectFlags(source) & 2097152 /* NonInferrableType */);
                    for (var i = 0; i < len; i++) {
                        inferFromSignature(getBaseSignature(sourceSignatures[sourceLen - len + i]), getBaseSignature(targetSignatures[targetLen - len + i]), skipParameters);
                    }
                }
                function inferFromSignature(source, target, skipParameters) {
                    if (!skipParameters) {
                        var saveBivariant = bivariant;
                        var kind = target.declaration ? target.declaration.kind : 0 /* Unknown */;
                        // Once we descend into a bivariant signature we remain bivariant for all nested inferences
                        bivariant = bivariant || kind === 161 /* MethodDeclaration */ || kind === 160 /* MethodSignature */ || kind === 162 /* Constructor */;
                        applyToParameterTypes(source, target, inferFromContravariantTypes);
                        bivariant = saveBivariant;
                    }
                    applyToReturnTypes(source, target, inferFromTypes);
                }
                function inferFromIndexTypes(source, target) {
                    var targetStringIndexType = getIndexTypeOfType(target, 0 /* String */);
                    if (targetStringIndexType) {
                        var sourceIndexType = getIndexTypeOfType(source, 0 /* String */) ||
                            getImplicitIndexTypeOfType(source, 0 /* String */);
                        if (sourceIndexType) {
                            inferFromTypes(sourceIndexType, targetStringIndexType);
                        }
                    }
                    var targetNumberIndexType = getIndexTypeOfType(target, 1 /* Number */);
                    if (targetNumberIndexType) {
                        var sourceIndexType = getIndexTypeOfType(source, 1 /* Number */) ||
                            getIndexTypeOfType(source, 0 /* String */) ||
                            getImplicitIndexTypeOfType(source, 1 /* Number */);
                        if (sourceIndexType) {
                            inferFromTypes(sourceIndexType, targetNumberIndexType);
                        }
                    }
                }
            }
            function isTypeOrBaseIdenticalTo(s, t) {
                return isTypeIdenticalTo(s, t) || !!(s.flags & (128 /* StringLiteral */ | 256 /* NumberLiteral */)) && isTypeIdenticalTo(getBaseTypeOfLiteralType(s), t);
            }
            function isTypeCloselyMatchedBy(s, t) {
                return !!(s.flags & 524288 /* Object */ && t.flags & 524288 /* Object */ && s.symbol && s.symbol === t.symbol ||
                    s.aliasSymbol && s.aliasTypeArguments && s.aliasSymbol === t.aliasSymbol);
            }
            function hasPrimitiveConstraint(type) {
                var constraint = getConstraintOfTypeParameter(type);
                return !!constraint && maybeTypeOfKind(constraint.flags & 16777216 /* Conditional */ ? getDefaultConstraintOfConditionalType(constraint) : constraint, 131068 /* Primitive */ | 4194304 /* Index */);
            }
            function isObjectLiteralType(type) {
                return !!(ts.getObjectFlags(type) & 128 /* ObjectLiteral */);
            }
            function isObjectOrArrayLiteralType(type) {
                return !!(ts.getObjectFlags(type) & (128 /* ObjectLiteral */ | 65536 /* ArrayLiteral */));
            }
            function unionObjectAndArrayLiteralCandidates(candidates) {
                if (candidates.length > 1) {
                    var objectLiterals = ts.filter(candidates, isObjectOrArrayLiteralType);
                    if (objectLiterals.length) {
                        var literalsType = getUnionType(objectLiterals, 2 /* Subtype */);
                        return ts.concatenate(ts.filter(candidates, function (t) { return !isObjectOrArrayLiteralType(t); }), [literalsType]);
                    }
                }
                return candidates;
            }
            function getContravariantInference(inference) {
                return inference.priority & 104 /* PriorityImpliesCombination */ ? getIntersectionType(inference.contraCandidates) : getCommonSubtype(inference.contraCandidates);
            }
            function getCovariantInference(inference, signature) {
                // Extract all object and array literal types and replace them with a single widened and normalized type.
                var candidates = unionObjectAndArrayLiteralCandidates(inference.candidates);
                // We widen inferred literal types if
                // all inferences were made to top-level occurrences of the type parameter, and
                // the type parameter has no constraint or its constraint includes no primitive or literal types, and
                // the type parameter was fixed during inference or does not occur at top-level in the return type.
                var primitiveConstraint = hasPrimitiveConstraint(inference.typeParameter);
                var widenLiteralTypes = !primitiveConstraint && inference.topLevel &&
                    (inference.isFixed || !isTypeParameterAtTopLevel(getReturnTypeOfSignature(signature), inference.typeParameter));
                var baseCandidates = primitiveConstraint ? ts.sameMap(candidates, getRegularTypeOfLiteralType) :
                    widenLiteralTypes ? ts.sameMap(candidates, getWidenedLiteralType) :
                        candidates;
                // If all inferences were made from a position that implies a combined result, infer a union type.
                // Otherwise, infer a common supertype.
                var unwidenedType = inference.priority & 104 /* PriorityImpliesCombination */ ?
                    getUnionType(baseCandidates, 2 /* Subtype */) :
                    getCommonSupertype(baseCandidates);
                return getWidenedType(unwidenedType);
            }
            function getInferredType(context, index) {
                var inference = context.inferences[index];
                if (!inference.inferredType) {
                    var inferredType = void 0;
                    var signature = context.signature;
                    if (signature) {
                        var inferredCovariantType = inference.candidates ? getCovariantInference(inference, signature) : undefined;
                        if (inference.contraCandidates) {
                            var inferredContravariantType = getContravariantInference(inference);
                            // If we have both co- and contra-variant inferences, we prefer the contra-variant inference
                            // unless the co-variant inference is a subtype and not 'never'.
                            inferredType = inferredCovariantType && !(inferredCovariantType.flags & 131072 /* Never */) &&
                                isTypeSubtypeOf(inferredCovariantType, inferredContravariantType) ?
                                inferredCovariantType : inferredContravariantType;
                        }
                        else if (inferredCovariantType) {
                            inferredType = inferredCovariantType;
                        }
                        else if (context.flags & 1 /* NoDefault */) {
                            // We use silentNeverType as the wildcard that signals no inferences.
                            inferredType = silentNeverType;
                        }
                        else {
                            // Infer either the default or the empty object type when no inferences were
                            // made. It is important to remember that in this case, inference still
                            // succeeds, meaning there is no error for not having inference candidates. An
                            // inference error only occurs when there are *conflicting* candidates, i.e.
                            // candidates with no common supertype.
                            var defaultType = getDefaultFromTypeParameter(inference.typeParameter);
                            if (defaultType) {
                                // Instantiate the default type. Any forward reference to a type
                                // parameter should be instantiated to the empty object type.
                                inferredType = instantiateType(defaultType, combineTypeMappers(createBackreferenceMapper(context, index), context.nonFixingMapper));
                            }
                        }
                    }
                    else {
                        inferredType = getTypeFromInference(inference);
                    }
                    inference.inferredType = inferredType || getDefaultTypeArgumentType(!!(context.flags & 2 /* AnyDefault */));
                    var constraint = getConstraintOfTypeParameter(inference.typeParameter);
                    if (constraint) {
                        var instantiatedConstraint = instantiateType(constraint, context.nonFixingMapper);
                        if (!inferredType || !context.compareTypes(inferredType, getTypeWithThisArgument(instantiatedConstraint, inferredType))) {
                            inference.inferredType = inferredType = instantiatedConstraint;
                        }
                    }
                }
                return inference.inferredType;
            }
            function getDefaultTypeArgumentType(isInJavaScriptFile) {
                return isInJavaScriptFile ? anyType : unknownType;
            }
            function getInferredTypes(context) {
                var result = [];
                for (var i = 0; i < context.inferences.length; i++) {
                    result.push(getInferredType(context, i));
                }
                return result;
            }
            // EXPRESSION TYPE CHECKING
            function getCannotFindNameDiagnosticForName(node) {
                switch (node.escapedText) {
                    case "document":
                    case "console":
                        return ts.Diagnostics.Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_include_dom;
                    case "$":
                        return compilerOptions.types
                            ? ts.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_types_Slashjquery_and_then_add_jquery_to_the_types_field_in_your_tsconfig
                            : ts.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_jQuery_Try_npm_i_types_Slashjquery;
                    case "describe":
                    case "suite":
                    case "it":
                    case "test":
                        return compilerOptions.types
                            ? ts.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_types_Slashjest_or_npm_i_types_Slashmocha_and_then_add_jest_or_mocha_to_the_types_field_in_your_tsconfig
                            : ts.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_a_test_runner_Try_npm_i_types_Slashjest_or_npm_i_types_Slashmocha;
                    case "process":
                    case "require":
                    case "Buffer":
                    case "module":
                        return compilerOptions.types
                            ? ts.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_types_Slashnode_and_then_add_node_to_the_types_field_in_your_tsconfig
                            : ts.Diagnostics.Cannot_find_name_0_Do_you_need_to_install_type_definitions_for_node_Try_npm_i_types_Slashnode;
                    case "Map":
                    case "Set":
                    case "Promise":
                    case "Symbol":
                    case "WeakMap":
                    case "WeakSet":
                    case "Iterator":
                    case "AsyncIterator":
                        return ts.Diagnostics.Cannot_find_name_0_Do_you_need_to_change_your_target_library_Try_changing_the_lib_compiler_option_to_es2015_or_later;
                    default:
                        if (node.parent.kind === 282 /* ShorthandPropertyAssignment */) {
                            return ts.Diagnostics.No_value_exists_in_scope_for_the_shorthand_property_0_Either_declare_one_or_provide_an_initializer;
                        }
                        else {
                            return ts.Diagnostics.Cannot_find_name_0;
                        }
                }
            }
            function getResolvedSymbol(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedSymbol) {
                    links.resolvedSymbol = !ts.nodeIsMissing(node) &&
                        resolveName(node, node.escapedText, 111551 /* Value */ | 1048576 /* ExportValue */, getCannotFindNameDiagnosticForName(node), node, !ts.isWriteOnlyAccess(node), 
                        /*excludeGlobals*/ false, ts.Diagnostics.Cannot_find_name_0_Did_you_mean_1) || unknownSymbol;
                }
                return links.resolvedSymbol;
            }
            function isInTypeQuery(node) {
                // TypeScript 1.0 spec (April 2014): 3.6.3
                // A type query consists of the keyword typeof followed by an expression.
                // The expression is restricted to a single identifier or a sequence of identifiers separated by periods
                return !!ts.findAncestor(node, function (n) { return n.kind === 172 /* TypeQuery */ ? true : n.kind === 75 /* Identifier */ || n.kind === 153 /* QualifiedName */ ? false : "quit"; });
            }
            // Return the flow cache key for a "dotted name" (i.e. a sequence of identifiers
            // separated by dots). The key consists of the id of the symbol referenced by the
            // leftmost identifier followed by zero or more property names separated by dots.
            // The result is undefined if the reference isn't a dotted name. We prefix nodes
            // occurring in an apparent type position with '@' because the control flow type
            // of such nodes may be based on the apparent type instead of the declared type.
            function getFlowCacheKey(node, declaredType, initialType, flowContainer) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                        var symbol = getResolvedSymbol(node);
                        return symbol !== unknownSymbol ? (flowContainer ? getNodeId(flowContainer) : "-1") + "|" + getTypeId(declaredType) + "|" + getTypeId(initialType) + "|" + (isConstraintPosition(node) ? "@" : "") + getSymbolId(symbol) : undefined;
                    case 104 /* ThisKeyword */:
                        return "0";
                    case 218 /* NonNullExpression */:
                    case 200 /* ParenthesizedExpression */:
                        return getFlowCacheKey(node.expression, declaredType, initialType, flowContainer);
                    case 194 /* PropertyAccessExpression */:
                    case 195 /* ElementAccessExpression */:
                        var propName = getAccessedPropertyName(node);
                        if (propName !== undefined) {
                            var key = getFlowCacheKey(node.expression, declaredType, initialType, flowContainer);
                            return key && key + "." + propName;
                        }
                }
                return undefined;
            }
            function isMatchingReference(source, target) {
                switch (target.kind) {
                    case 200 /* ParenthesizedExpression */:
                    case 218 /* NonNullExpression */:
                        return isMatchingReference(source, target.expression);
                }
                switch (source.kind) {
                    case 75 /* Identifier */:
                        return target.kind === 75 /* Identifier */ && getResolvedSymbol(source) === getResolvedSymbol(target) ||
                            (target.kind === 242 /* VariableDeclaration */ || target.kind === 191 /* BindingElement */) &&
                                getExportSymbolOfValueSymbolIfExported(getResolvedSymbol(source)) === getSymbolOfNode(target);
                    case 104 /* ThisKeyword */:
                        return target.kind === 104 /* ThisKeyword */;
                    case 102 /* SuperKeyword */:
                        return target.kind === 102 /* SuperKeyword */;
                    case 218 /* NonNullExpression */:
                    case 200 /* ParenthesizedExpression */:
                        return isMatchingReference(source.expression, target);
                    case 194 /* PropertyAccessExpression */:
                    case 195 /* ElementAccessExpression */:
                        return ts.isAccessExpression(target) &&
                            getAccessedPropertyName(source) === getAccessedPropertyName(target) &&
                            isMatchingReference(source.expression, target.expression);
                }
                return false;
            }
            function getAccessedPropertyName(access) {
                return access.kind === 194 /* PropertyAccessExpression */ ? access.name.escapedText :
                    ts.isStringOrNumericLiteralLike(access.argumentExpression) ? ts.escapeLeadingUnderscores(access.argumentExpression.text) :
                        undefined;
            }
            function containsMatchingReference(source, target) {
                while (ts.isAccessExpression(source)) {
                    source = source.expression;
                    if (isMatchingReference(source, target)) {
                        return true;
                    }
                }
                return false;
            }
            function optionalChainContainsReference(source, target) {
                while (ts.isOptionalChain(source)) {
                    source = source.expression;
                    if (isMatchingReference(source, target)) {
                        return true;
                    }
                }
                return false;
            }
            function isDiscriminantProperty(type, name) {
                if (type && type.flags & 1048576 /* Union */) {
                    var prop = getUnionOrIntersectionProperty(type, name);
                    if (prop && ts.getCheckFlags(prop) & 2 /* SyntheticProperty */) {
                        if (prop.isDiscriminantProperty === undefined) {
                            prop.isDiscriminantProperty =
                                (prop.checkFlags & 192 /* Discriminant */) === 192 /* Discriminant */ &&
                                    !maybeTypeOfKind(getTypeOfSymbol(prop), 63176704 /* Instantiable */);
                        }
                        return !!prop.isDiscriminantProperty;
                    }
                }
                return false;
            }
            function isSyntheticThisPropertyAccess(expr) {
                return ts.isAccessExpression(expr) && expr.expression.kind === 104 /* ThisKeyword */ && !!(expr.expression.flags & 8 /* Synthesized */);
            }
            function findDiscriminantProperties(sourceProperties, target) {
                var result;
                for (var _i = 0, sourceProperties_2 = sourceProperties; _i < sourceProperties_2.length; _i++) {
                    var sourceProperty = sourceProperties_2[_i];
                    if (isDiscriminantProperty(target, sourceProperty.escapedName)) {
                        if (result) {
                            result.push(sourceProperty);
                            continue;
                        }
                        result = [sourceProperty];
                    }
                }
                return result;
            }
            function isOrContainsMatchingReference(source, target) {
                return isMatchingReference(source, target) || containsMatchingReference(source, target);
            }
            function hasMatchingArgument(callExpression, reference) {
                if (callExpression.arguments) {
                    for (var _i = 0, _a = callExpression.arguments; _i < _a.length; _i++) {
                        var argument = _a[_i];
                        if (isOrContainsMatchingReference(reference, argument)) {
                            return true;
                        }
                    }
                }
                if (callExpression.expression.kind === 194 /* PropertyAccessExpression */ &&
                    isOrContainsMatchingReference(reference, callExpression.expression.expression)) {
                    return true;
                }
                return false;
            }
            function getFlowNodeId(flow) {
                if (!flow.id || flow.id < 0) {
                    flow.id = nextFlowId;
                    nextFlowId++;
                }
                return flow.id;
            }
            function typeMaybeAssignableTo(source, target) {
                if (!(source.flags & 1048576 /* Union */)) {
                    return isTypeAssignableTo(source, target);
                }
                for (var _i = 0, _a = source.types; _i < _a.length; _i++) {
                    var t = _a[_i];
                    if (isTypeAssignableTo(t, target)) {
                        return true;
                    }
                }
                return false;
            }
            // Remove those constituent types of declaredType to which no constituent type of assignedType is assignable.
            // For example, when a variable of type number | string | boolean is assigned a value of type number | boolean,
            // we remove type string.
            function getAssignmentReducedType(declaredType, assignedType) {
                if (declaredType !== assignedType) {
                    if (assignedType.flags & 131072 /* Never */) {
                        return assignedType;
                    }
                    var reducedType = filterType(declaredType, function (t) { return typeMaybeAssignableTo(assignedType, t); });
                    if (assignedType.flags & 512 /* BooleanLiteral */ && isFreshLiteralType(assignedType)) {
                        reducedType = mapType(reducedType, getFreshTypeOfLiteralType); // Ensure that if the assignment is a fresh type, that we narrow to fresh types
                    }
                    // Our crude heuristic produces an invalid result in some cases: see GH#26130.
                    // For now, when that happens, we give up and don't narrow at all.  (This also
                    // means we'll never narrow for erroneous assignments where the assigned type
                    // is not assignable to the declared type.)
                    if (isTypeAssignableTo(assignedType, reducedType)) {
                        return reducedType;
                    }
                }
                return declaredType;
            }
            function getTypeFactsOfTypes(types) {
                var result = 0 /* None */;
                for (var _i = 0, types_14 = types; _i < types_14.length; _i++) {
                    var t = types_14[_i];
                    result |= getTypeFacts(t);
                }
                return result;
            }
            function isFunctionObjectType(type) {
                // We do a quick check for a "bind" property before performing the more expensive subtype
                // check. This gives us a quicker out in the common case where an object type is not a function.
                var resolved = resolveStructuredTypeMembers(type);
                return !!(resolved.callSignatures.length || resolved.constructSignatures.length ||
                    resolved.members.get("bind") && isTypeSubtypeOf(type, globalFunctionType));
            }
            function getTypeFacts(type) {
                var flags = type.flags;
                if (flags & 4 /* String */) {
                    return strictNullChecks ? 16317953 /* StringStrictFacts */ : 16776705 /* StringFacts */;
                }
                if (flags & 128 /* StringLiteral */) {
                    var isEmpty = type.value === "";
                    return strictNullChecks ?
                        isEmpty ? 12123649 /* EmptyStringStrictFacts */ : 7929345 /* NonEmptyStringStrictFacts */ :
                        isEmpty ? 12582401 /* EmptyStringFacts */ : 16776705 /* NonEmptyStringFacts */;
                }
                if (flags & (8 /* Number */ | 32 /* Enum */)) {
                    return strictNullChecks ? 16317698 /* NumberStrictFacts */ : 16776450 /* NumberFacts */;
                }
                if (flags & 256 /* NumberLiteral */) {
                    var isZero = type.value === 0;
                    return strictNullChecks ?
                        isZero ? 12123394 /* ZeroNumberStrictFacts */ : 7929090 /* NonZeroNumberStrictFacts */ :
                        isZero ? 12582146 /* ZeroNumberFacts */ : 16776450 /* NonZeroNumberFacts */;
                }
                if (flags & 64 /* BigInt */) {
                    return strictNullChecks ? 16317188 /* BigIntStrictFacts */ : 16775940 /* BigIntFacts */;
                }
                if (flags & 2048 /* BigIntLiteral */) {
                    var isZero = isZeroBigInt(type);
                    return strictNullChecks ?
                        isZero ? 12122884 /* ZeroBigIntStrictFacts */ : 7928580 /* NonZeroBigIntStrictFacts */ :
                        isZero ? 12581636 /* ZeroBigIntFacts */ : 16775940 /* NonZeroBigIntFacts */;
                }
                if (flags & 16 /* Boolean */) {
                    return strictNullChecks ? 16316168 /* BooleanStrictFacts */ : 16774920 /* BooleanFacts */;
                }
                if (flags & 528 /* BooleanLike */) {
                    return strictNullChecks ?
                        (type === falseType || type === regularFalseType) ? 12121864 /* FalseStrictFacts */ : 7927560 /* TrueStrictFacts */ :
                        (type === falseType || type === regularFalseType) ? 12580616 /* FalseFacts */ : 16774920 /* TrueFacts */;
                }
                if (flags & 524288 /* Object */) {
                    return ts.getObjectFlags(type) & 16 /* Anonymous */ && isEmptyObjectType(type) ?
                        strictNullChecks ? 16318463 /* EmptyObjectStrictFacts */ : 16777215 /* EmptyObjectFacts */ :
                        isFunctionObjectType(type) ?
                            strictNullChecks ? 7880640 /* FunctionStrictFacts */ : 16728000 /* FunctionFacts */ :
                            strictNullChecks ? 7888800 /* ObjectStrictFacts */ : 16736160 /* ObjectFacts */;
                }
                if (flags & (16384 /* Void */ | 32768 /* Undefined */)) {
                    return 9830144 /* UndefinedFacts */;
                }
                if (flags & 65536 /* Null */) {
                    return 9363232 /* NullFacts */;
                }
                if (flags & 12288 /* ESSymbolLike */) {
                    return strictNullChecks ? 7925520 /* SymbolStrictFacts */ : 16772880 /* SymbolFacts */;
                }
                if (flags & 67108864 /* NonPrimitive */) {
                    return strictNullChecks ? 7888800 /* ObjectStrictFacts */ : 16736160 /* ObjectFacts */;
                }
                if (flags & 63176704 /* Instantiable */) {
                    return getTypeFacts(getBaseConstraintOfType(type) || unknownType);
                }
                if (flags & 3145728 /* UnionOrIntersection */) {
                    return getTypeFactsOfTypes(type.types);
                }
                return 16777215 /* All */;
            }
            function getTypeWithFacts(type, include) {
                return filterType(type, function (t) { return (getTypeFacts(t) & include) !== 0; });
            }
            function getTypeWithDefault(type, defaultExpression) {
                if (defaultExpression) {
                    var defaultType = getTypeOfExpression(defaultExpression);
                    return getUnionType([getTypeWithFacts(type, 524288 /* NEUndefined */), defaultType]);
                }
                return type;
            }
            function getTypeOfDestructuredProperty(type, name) {
                var nameType = getLiteralTypeFromPropertyName(name);
                if (!isTypeUsableAsPropertyName(nameType))
                    return errorType;
                var text = getPropertyNameFromType(nameType);
                return getConstraintForLocation(getTypeOfPropertyOfType(type, text), name) ||
                    isNumericLiteralName(text) && getIndexTypeOfType(type, 1 /* Number */) ||
                    getIndexTypeOfType(type, 0 /* String */) ||
                    errorType;
            }
            function getTypeOfDestructuredArrayElement(type, index) {
                return everyType(type, isTupleLikeType) && getTupleElementType(type, index) ||
                    checkIteratedTypeOrElementType(65 /* Destructuring */, type, undefinedType, /*errorNode*/ undefined) ||
                    errorType;
            }
            function getTypeOfDestructuredSpreadExpression(type) {
                return createArrayType(checkIteratedTypeOrElementType(65 /* Destructuring */, type, undefinedType, /*errorNode*/ undefined) || errorType);
            }
            function getAssignedTypeOfBinaryExpression(node) {
                var isDestructuringDefaultAssignment = node.parent.kind === 192 /* ArrayLiteralExpression */ && isDestructuringAssignmentTarget(node.parent) ||
                    node.parent.kind === 281 /* PropertyAssignment */ && isDestructuringAssignmentTarget(node.parent.parent);
                return isDestructuringDefaultAssignment ?
                    getTypeWithDefault(getAssignedType(node), node.right) :
                    getTypeOfExpression(node.right);
            }
            function isDestructuringAssignmentTarget(parent) {
                return parent.parent.kind === 209 /* BinaryExpression */ && parent.parent.left === parent ||
                    parent.parent.kind === 232 /* ForOfStatement */ && parent.parent.initializer === parent;
            }
            function getAssignedTypeOfArrayLiteralElement(node, element) {
                return getTypeOfDestructuredArrayElement(getAssignedType(node), node.elements.indexOf(element));
            }
            function getAssignedTypeOfSpreadExpression(node) {
                return getTypeOfDestructuredSpreadExpression(getAssignedType(node.parent));
            }
            function getAssignedTypeOfPropertyAssignment(node) {
                return getTypeOfDestructuredProperty(getAssignedType(node.parent), node.name);
            }
            function getAssignedTypeOfShorthandPropertyAssignment(node) {
                return getTypeWithDefault(getAssignedTypeOfPropertyAssignment(node), node.objectAssignmentInitializer);
            }
            function getAssignedType(node) {
                var parent = node.parent;
                switch (parent.kind) {
                    case 231 /* ForInStatement */:
                        return stringType;
                    case 232 /* ForOfStatement */:
                        return checkRightHandSideOfForOf(parent.expression, parent.awaitModifier) || errorType;
                    case 209 /* BinaryExpression */:
                        return getAssignedTypeOfBinaryExpression(parent);
                    case 203 /* DeleteExpression */:
                        return undefinedType;
                    case 192 /* ArrayLiteralExpression */:
                        return getAssignedTypeOfArrayLiteralElement(parent, node);
                    case 213 /* SpreadElement */:
                        return getAssignedTypeOfSpreadExpression(parent);
                    case 281 /* PropertyAssignment */:
                        return getAssignedTypeOfPropertyAssignment(parent);
                    case 282 /* ShorthandPropertyAssignment */:
                        return getAssignedTypeOfShorthandPropertyAssignment(parent);
                }
                return errorType;
            }
            function getInitialTypeOfBindingElement(node) {
                var pattern = node.parent;
                var parentType = getInitialType(pattern.parent);
                var type = pattern.kind === 189 /* ObjectBindingPattern */ ?
                    getTypeOfDestructuredProperty(parentType, node.propertyName || node.name) :
                    !node.dotDotDotToken ?
                        getTypeOfDestructuredArrayElement(parentType, pattern.elements.indexOf(node)) :
                        getTypeOfDestructuredSpreadExpression(parentType);
                return getTypeWithDefault(type, node.initializer);
            }
            function getTypeOfInitializer(node) {
                // Return the cached type if one is available. If the type of the variable was inferred
                // from its initializer, we'll already have cached the type. Otherwise we compute it now
                // without caching such that transient types are reflected.
                var links = getNodeLinks(node);
                return links.resolvedType || getTypeOfExpression(node);
            }
            function getInitialTypeOfVariableDeclaration(node) {
                if (node.initializer) {
                    return getTypeOfInitializer(node.initializer);
                }
                if (node.parent.parent.kind === 231 /* ForInStatement */) {
                    return stringType;
                }
                if (node.parent.parent.kind === 232 /* ForOfStatement */) {
                    return checkRightHandSideOfForOf(node.parent.parent.expression, node.parent.parent.awaitModifier) || errorType;
                }
                return errorType;
            }
            function getInitialType(node) {
                return node.kind === 242 /* VariableDeclaration */ ?
                    getInitialTypeOfVariableDeclaration(node) :
                    getInitialTypeOfBindingElement(node);
            }
            function isEmptyArrayAssignment(node) {
                return node.kind === 242 /* VariableDeclaration */ && node.initializer &&
                    isEmptyArrayLiteral(node.initializer) ||
                    node.kind !== 191 /* BindingElement */ && node.parent.kind === 209 /* BinaryExpression */ &&
                        isEmptyArrayLiteral(node.parent.right);
            }
            function getReferenceCandidate(node) {
                switch (node.kind) {
                    case 200 /* ParenthesizedExpression */:
                        return getReferenceCandidate(node.expression);
                    case 209 /* BinaryExpression */:
                        switch (node.operatorToken.kind) {
                            case 62 /* EqualsToken */:
                                return getReferenceCandidate(node.left);
                            case 27 /* CommaToken */:
                                return getReferenceCandidate(node.right);
                        }
                }
                return node;
            }
            function getReferenceRoot(node) {
                var parent = node.parent;
                return parent.kind === 200 /* ParenthesizedExpression */ ||
                    parent.kind === 209 /* BinaryExpression */ && parent.operatorToken.kind === 62 /* EqualsToken */ && parent.left === node ||
                    parent.kind === 209 /* BinaryExpression */ && parent.operatorToken.kind === 27 /* CommaToken */ && parent.right === node ?
                    getReferenceRoot(parent) : node;
            }
            function getTypeOfSwitchClause(clause) {
                if (clause.kind === 277 /* CaseClause */) {
                    return getRegularTypeOfLiteralType(getTypeOfExpression(clause.expression));
                }
                return neverType;
            }
            function getSwitchClauseTypes(switchStatement) {
                var links = getNodeLinks(switchStatement);
                if (!links.switchTypes) {
                    links.switchTypes = [];
                    for (var _i = 0, _a = switchStatement.caseBlock.clauses; _i < _a.length; _i++) {
                        var clause = _a[_i];
                        links.switchTypes.push(getTypeOfSwitchClause(clause));
                    }
                }
                return links.switchTypes;
            }
            // Get the types from all cases in a switch on `typeof`. An
            // `undefined` element denotes an explicit `default` clause.
            function getSwitchClauseTypeOfWitnesses(switchStatement) {
                var witnesses = [];
                for (var _i = 0, _a = switchStatement.caseBlock.clauses; _i < _a.length; _i++) {
                    var clause = _a[_i];
                    if (clause.kind === 277 /* CaseClause */) {
                        if (ts.isStringLiteralLike(clause.expression)) {
                            witnesses.push(clause.expression.text);
                            continue;
                        }
                        return ts.emptyArray;
                    }
                    witnesses.push(/*explicitDefaultStatement*/ undefined);
                }
                return witnesses;
            }
            function eachTypeContainedIn(source, types) {
                return source.flags & 1048576 /* Union */ ? !ts.forEach(source.types, function (t) { return !ts.contains(types, t); }) : ts.contains(types, source);
            }
            function isTypeSubsetOf(source, target) {
                return source === target || target.flags & 1048576 /* Union */ && isTypeSubsetOfUnion(source, target);
            }
            function isTypeSubsetOfUnion(source, target) {
                if (source.flags & 1048576 /* Union */) {
                    for (var _i = 0, _a = source.types; _i < _a.length; _i++) {
                        var t = _a[_i];
                        if (!containsType(target.types, t)) {
                            return false;
                        }
                    }
                    return true;
                }
                if (source.flags & 1024 /* EnumLiteral */ && getBaseTypeOfEnumLiteralType(source) === target) {
                    return true;
                }
                return containsType(target.types, source);
            }
            function forEachType(type, f) {
                return type.flags & 1048576 /* Union */ ? ts.forEach(type.types, f) : f(type);
            }
            function everyType(type, f) {
                return type.flags & 1048576 /* Union */ ? ts.every(type.types, f) : f(type);
            }
            function filterType(type, f) {
                if (type.flags & 1048576 /* Union */) {
                    var types = type.types;
                    var filtered = ts.filter(types, f);
                    return filtered === types ? type : getUnionTypeFromSortedList(filtered, type.objectFlags);
                }
                return f(type) ? type : neverType;
            }
            function countTypes(type) {
                return type.flags & 1048576 /* Union */ ? type.types.length : 1;
            }
            function mapType(type, mapper, noReductions) {
                if (type.flags & 131072 /* Never */) {
                    return type;
                }
                if (!(type.flags & 1048576 /* Union */)) {
                    return mapper(type);
                }
                var mappedTypes;
                for (var _i = 0, _a = type.types; _i < _a.length; _i++) {
                    var t = _a[_i];
                    var mapped = mapper(t);
                    if (mapped) {
                        if (!mappedTypes) {
                            mappedTypes = [mapped];
                        }
                        else {
                            mappedTypes.push(mapped);
                        }
                    }
                }
                return mappedTypes && getUnionType(mappedTypes, noReductions ? 0 /* None */ : 1 /* Literal */);
            }
            function extractTypesOfKind(type, kind) {
                return filterType(type, function (t) { return (t.flags & kind) !== 0; });
            }
            // Return a new type in which occurrences of the string and number primitive types in
            // typeWithPrimitives have been replaced with occurrences of string literals and numeric
            // literals in typeWithLiterals, respectively.
            function replacePrimitivesWithLiterals(typeWithPrimitives, typeWithLiterals) {
                if (isTypeSubsetOf(stringType, typeWithPrimitives) && maybeTypeOfKind(typeWithLiterals, 128 /* StringLiteral */) ||
                    isTypeSubsetOf(numberType, typeWithPrimitives) && maybeTypeOfKind(typeWithLiterals, 256 /* NumberLiteral */) ||
                    isTypeSubsetOf(bigintType, typeWithPrimitives) && maybeTypeOfKind(typeWithLiterals, 2048 /* BigIntLiteral */)) {
                    return mapType(typeWithPrimitives, function (t) {
                        return t.flags & 4 /* String */ ? extractTypesOfKind(typeWithLiterals, 4 /* String */ | 128 /* StringLiteral */) :
                            t.flags & 8 /* Number */ ? extractTypesOfKind(typeWithLiterals, 8 /* Number */ | 256 /* NumberLiteral */) :
                                t.flags & 64 /* BigInt */ ? extractTypesOfKind(typeWithLiterals, 64 /* BigInt */ | 2048 /* BigIntLiteral */) : t;
                    });
                }
                return typeWithPrimitives;
            }
            function isIncomplete(flowType) {
                return flowType.flags === 0;
            }
            function getTypeFromFlowType(flowType) {
                return flowType.flags === 0 ? flowType.type : flowType;
            }
            function createFlowType(type, incomplete) {
                return incomplete ? { flags: 0, type: type } : type;
            }
            // An evolving array type tracks the element types that have so far been seen in an
            // 'x.push(value)' or 'x[n] = value' operation along the control flow graph. Evolving
            // array types are ultimately converted into manifest array types (using getFinalArrayType)
            // and never escape the getFlowTypeOfReference function.
            function createEvolvingArrayType(elementType) {
                var result = createObjectType(256 /* EvolvingArray */);
                result.elementType = elementType;
                return result;
            }
            function getEvolvingArrayType(elementType) {
                return evolvingArrayTypes[elementType.id] || (evolvingArrayTypes[elementType.id] = createEvolvingArrayType(elementType));
            }
            // When adding evolving array element types we do not perform subtype reduction. Instead,
            // we defer subtype reduction until the evolving array type is finalized into a manifest
            // array type.
            function addEvolvingArrayElementType(evolvingArrayType, node) {
                var elementType = getBaseTypeOfLiteralType(getContextFreeTypeOfExpression(node));
                return isTypeSubsetOf(elementType, evolvingArrayType.elementType) ? evolvingArrayType : getEvolvingArrayType(getUnionType([evolvingArrayType.elementType, elementType]));
            }
            function createFinalArrayType(elementType) {
                return elementType.flags & 131072 /* Never */ ?
                    autoArrayType :
                    createArrayType(elementType.flags & 1048576 /* Union */ ?
                        getUnionType(elementType.types, 2 /* Subtype */) :
                        elementType);
            }
            // We perform subtype reduction upon obtaining the final array type from an evolving array type.
            function getFinalArrayType(evolvingArrayType) {
                return evolvingArrayType.finalArrayType || (evolvingArrayType.finalArrayType = createFinalArrayType(evolvingArrayType.elementType));
            }
            function finalizeEvolvingArrayType(type) {
                return ts.getObjectFlags(type) & 256 /* EvolvingArray */ ? getFinalArrayType(type) : type;
            }
            function getElementTypeOfEvolvingArrayType(type) {
                return ts.getObjectFlags(type) & 256 /* EvolvingArray */ ? type.elementType : neverType;
            }
            function isEvolvingArrayTypeList(types) {
                var hasEvolvingArrayType = false;
                for (var _i = 0, types_15 = types; _i < types_15.length; _i++) {
                    var t = types_15[_i];
                    if (!(t.flags & 131072 /* Never */)) {
                        if (!(ts.getObjectFlags(t) & 256 /* EvolvingArray */)) {
                            return false;
                        }
                        hasEvolvingArrayType = true;
                    }
                }
                return hasEvolvingArrayType;
            }
            // At flow control branch or loop junctions, if the type along every antecedent code path
            // is an evolving array type, we construct a combined evolving array type. Otherwise we
            // finalize all evolving array types.
            function getUnionOrEvolvingArrayType(types, subtypeReduction) {
                return isEvolvingArrayTypeList(types) ?
                    getEvolvingArrayType(getUnionType(ts.map(types, getElementTypeOfEvolvingArrayType))) :
                    getUnionType(ts.sameMap(types, finalizeEvolvingArrayType), subtypeReduction);
            }
            // Return true if the given node is 'x' in an 'x.length', x.push(value)', 'x.unshift(value)' or
            // 'x[n] = value' operation, where 'n' is an expression of type any, undefined, or a number-like type.
            function isEvolvingArrayOperationTarget(node) {
                var root = getReferenceRoot(node);
                var parent = root.parent;
                var isLengthPushOrUnshift = ts.isPropertyAccessExpression(parent) && (parent.name.escapedText === "length" ||
                    parent.parent.kind === 196 /* CallExpression */
                        && ts.isIdentifier(parent.name)
                        && ts.isPushOrUnshiftIdentifier(parent.name));
                var isElementAssignment = parent.kind === 195 /* ElementAccessExpression */ &&
                    parent.expression === root &&
                    parent.parent.kind === 209 /* BinaryExpression */ &&
                    parent.parent.operatorToken.kind === 62 /* EqualsToken */ &&
                    parent.parent.left === parent &&
                    !ts.isAssignmentTarget(parent.parent) &&
                    isTypeAssignableToKind(getTypeOfExpression(parent.argumentExpression), 296 /* NumberLike */);
                return isLengthPushOrUnshift || isElementAssignment;
            }
            function isDeclarationWithExplicitTypeAnnotation(declaration) {
                return !!(declaration && (declaration.kind === 242 /* VariableDeclaration */ || declaration.kind === 156 /* Parameter */ ||
                    declaration.kind === 159 /* PropertyDeclaration */ || declaration.kind === 158 /* PropertySignature */) &&
                    ts.getEffectiveTypeAnnotationNode(declaration));
            }
            function getExplicitTypeOfSymbol(symbol, diagnostic) {
                if (symbol.flags & (16 /* Function */ | 8192 /* Method */ | 32 /* Class */ | 512 /* ValueModule */)) {
                    return getTypeOfSymbol(symbol);
                }
                if (symbol.flags & (3 /* Variable */ | 4 /* Property */)) {
                    if (isDeclarationWithExplicitTypeAnnotation(symbol.valueDeclaration)) {
                        return getTypeOfSymbol(symbol);
                    }
                    if (diagnostic && symbol.valueDeclaration) {
                        ts.addRelatedInfo(diagnostic, ts.createDiagnosticForNode(symbol.valueDeclaration, ts.Diagnostics._0_needs_an_explicit_type_annotation, symbolToString(symbol)));
                    }
                }
            }
            // We require the dotted function name in an assertion expression to be comprised of identifiers
            // that reference function, method, class or value module symbols; or variable, property or
            // parameter symbols with declarations that have explicit type annotations. Such references are
            // resolvable with no possibility of triggering circularities in control flow analysis.
            function getTypeOfDottedName(node, diagnostic) {
                if (!(node.flags & 16777216 /* InWithStatement */)) {
                    switch (node.kind) {
                        case 75 /* Identifier */:
                            var symbol = getExportSymbolOfValueSymbolIfExported(getResolvedSymbol(node));
                            return getExplicitTypeOfSymbol(symbol.flags & 2097152 /* Alias */ ? resolveAlias(symbol) : symbol, diagnostic);
                        case 104 /* ThisKeyword */:
                            return getExplicitThisType(node);
                        case 102 /* SuperKeyword */:
                            return checkSuperExpression(node);
                        case 194 /* PropertyAccessExpression */:
                            var type = getTypeOfDottedName(node.expression, diagnostic);
                            var prop = type && getPropertyOfType(type, node.name.escapedText);
                            return prop && getExplicitTypeOfSymbol(prop, diagnostic);
                        case 200 /* ParenthesizedExpression */:
                            return getTypeOfDottedName(node.expression, diagnostic);
                    }
                }
            }
            function getEffectsSignature(node) {
                var links = getNodeLinks(node);
                var signature = links.effectsSignature;
                if (signature === undefined) {
                    // A call expression parented by an expression statement is a potential assertion. Other call
                    // expressions are potential type predicate function calls. In order to avoid triggering
                    // circularities in control flow analysis, we use getTypeOfDottedName when resolving the call
                    // target expression of an assertion.
                    var funcType = void 0;
                    if (node.parent.kind === 226 /* ExpressionStatement */) {
                        funcType = getTypeOfDottedName(node.expression, /*diagnostic*/ undefined);
                    }
                    else if (node.expression.kind !== 102 /* SuperKeyword */) {
                        if (ts.isOptionalChain(node)) {
                            funcType = checkNonNullType(getOptionalExpressionType(checkExpression(node.expression), node.expression), node.expression);
                        }
                        else {
                            funcType = checkNonNullExpression(node.expression);
                        }
                    }
                    var signatures = getSignaturesOfType(funcType && getApparentType(funcType) || unknownType, 0 /* Call */);
                    var candidate = signatures.length === 1 && !signatures[0].typeParameters ? signatures[0] :
                        ts.some(signatures, hasTypePredicateOrNeverReturnType) ? getResolvedSignature(node) :
                            undefined;
                    signature = links.effectsSignature = candidate && hasTypePredicateOrNeverReturnType(candidate) ? candidate : unknownSignature;
                }
                return signature === unknownSignature ? undefined : signature;
            }
            function hasTypePredicateOrNeverReturnType(signature) {
                return !!(getTypePredicateOfSignature(signature) ||
                    signature.declaration && (getReturnTypeFromAnnotation(signature.declaration) || unknownType).flags & 131072 /* Never */);
            }
            function getTypePredicateArgument(predicate, callExpression) {
                if (predicate.kind === 1 /* Identifier */ || predicate.kind === 3 /* AssertsIdentifier */) {
                    return callExpression.arguments[predicate.parameterIndex];
                }
                var invokedExpression = ts.skipParentheses(callExpression.expression);
                return ts.isAccessExpression(invokedExpression) ? ts.skipParentheses(invokedExpression.expression) : undefined;
            }
            function reportFlowControlError(node) {
                var block = ts.findAncestor(node, ts.isFunctionOrModuleBlock);
                var sourceFile = ts.getSourceFileOfNode(node);
                var span = ts.getSpanOfTokenAtPosition(sourceFile, block.statements.pos);
                diagnostics.add(ts.createFileDiagnostic(sourceFile, span.start, span.length, ts.Diagnostics.The_containing_function_or_module_body_is_too_large_for_control_flow_analysis));
            }
            function isReachableFlowNode(flow) {
                var result = isReachableFlowNodeWorker(flow, /*skipCacheCheck*/ false);
                lastFlowNode = flow;
                lastFlowNodeReachable = result;
                return result;
            }
            function isUnlockedReachableFlowNode(flow) {
                return !(flow.flags & 4096 /* PreFinally */ && flow.lock.locked) && isReachableFlowNodeWorker(flow, /*skipCacheCheck*/ false);
            }
            function isFalseExpression(expr) {
                var node = ts.skipParentheses(expr);
                return node.kind === 91 /* FalseKeyword */ || node.kind === 209 /* BinaryExpression */ && (node.operatorToken.kind === 55 /* AmpersandAmpersandToken */ && (isFalseExpression(node.left) || isFalseExpression(node.right)) ||
                    node.operatorToken.kind === 56 /* BarBarToken */ && isFalseExpression(node.left) && isFalseExpression(node.right));
            }
            function isReachableFlowNodeWorker(flow, noCacheCheck) {
                while (true) {
                    if (flow === lastFlowNode) {
                        return lastFlowNodeReachable;
                    }
                    var flags = flow.flags;
                    if (flags & 2048 /* Shared */) {
                        if (!noCacheCheck) {
                            var id = getFlowNodeId(flow);
                            var reachable = flowNodeReachable[id];
                            return reachable !== undefined ? reachable : (flowNodeReachable[id] = isReachableFlowNodeWorker(flow, /*skipCacheCheck*/ true));
                        }
                        noCacheCheck = false;
                    }
                    if (flags & (16 /* Assignment */ | 96 /* Condition */ | 256 /* ArrayMutation */ | 4096 /* PreFinally */)) {
                        flow = flow.antecedent;
                    }
                    else if (flags & 512 /* Call */) {
                        var signature = getEffectsSignature(flow.node);
                        if (signature) {
                            var predicate = getTypePredicateOfSignature(signature);
                            if (predicate && predicate.kind === 3 /* AssertsIdentifier */) {
                                var predicateArgument = flow.node.arguments[predicate.parameterIndex];
                                if (predicateArgument && isFalseExpression(predicateArgument)) {
                                    return false;
                                }
                            }
                            if (getReturnTypeOfSignature(signature).flags & 131072 /* Never */) {
                                return false;
                            }
                        }
                        flow = flow.antecedent;
                    }
                    else if (flags & 4 /* BranchLabel */) {
                        // A branching point is reachable if any branch is reachable.
                        return ts.some(flow.antecedents, isUnlockedReachableFlowNode);
                    }
                    else if (flags & 8 /* LoopLabel */) {
                        // A loop is reachable if the control flow path that leads to the top is reachable.
                        flow = flow.antecedents[0];
                    }
                    else if (flags & 128 /* SwitchClause */) {
                        // The control flow path representing an unmatched value in a switch statement with
                        // no default clause is unreachable if the switch statement is exhaustive.
                        if (flow.clauseStart === flow.clauseEnd && isExhaustiveSwitchStatement(flow.switchStatement)) {
                            return false;
                        }
                        flow = flow.antecedent;
                    }
                    else if (flags & 8192 /* AfterFinally */) {
                        // Cache is unreliable once we start locking nodes
                        lastFlowNode = undefined;
                        flow.locked = true;
                        var result = isReachableFlowNodeWorker(flow.antecedent, /*skipCacheCheck*/ false);
                        flow.locked = false;
                        return result;
                    }
                    else {
                        return !(flags & 1 /* Unreachable */);
                    }
                }
            }
            function getFlowTypeOfReference(reference, declaredType, initialType, flowContainer, couldBeUninitialized) {
                if (initialType === void 0) {
                    initialType = declaredType;
                }
                var key;
                var keySet = false;
                var flowDepth = 0;
                if (flowAnalysisDisabled) {
                    return errorType;
                }
                if (!reference.flowNode || !couldBeUninitialized && !(declaredType.flags & 133970943 /* Narrowable */)) {
                    return declaredType;
                }
                flowInvocationCount++;
                var sharedFlowStart = sharedFlowCount;
                var evolvedType = getTypeFromFlowType(getTypeAtFlowNode(reference.flowNode));
                sharedFlowCount = sharedFlowStart;
                // When the reference is 'x' in an 'x.length', 'x.push(value)', 'x.unshift(value)' or x[n] = value' operation,
                // we give type 'any[]' to 'x' instead of using the type determined by control flow analysis such that operations
                // on empty arrays are possible without implicit any errors and new element types can be inferred without
                // type mismatch errors.
                var resultType = ts.getObjectFlags(evolvedType) & 256 /* EvolvingArray */ && isEvolvingArrayOperationTarget(reference) ? autoArrayType : finalizeEvolvingArrayType(evolvedType);
                if (resultType === unreachableNeverType || reference.parent && reference.parent.kind === 218 /* NonNullExpression */ && getTypeWithFacts(resultType, 2097152 /* NEUndefinedOrNull */).flags & 131072 /* Never */) {
                    return declaredType;
                }
                return resultType;
                function getOrSetCacheKey() {
                    if (keySet) {
                        return key;
                    }
                    keySet = true;
                    return key = getFlowCacheKey(reference, declaredType, initialType, flowContainer);
                }
                function getTypeAtFlowNode(flow) {
                    if (flowDepth === 2000) {
                        // We have made 2000 recursive invocations. To avoid overflowing the call stack we report an error
                        // and disable further control flow analysis in the containing function or module body.
                        flowAnalysisDisabled = true;
                        reportFlowControlError(reference);
                        return errorType;
                    }
                    flowDepth++;
                    while (true) {
                        var flags = flow.flags;
                        if (flags & 2048 /* Shared */) {
                            // We cache results of flow type resolution for shared nodes that were previously visited in
                            // the same getFlowTypeOfReference invocation. A node is considered shared when it is the
                            // antecedent of more than one node.
                            for (var i = sharedFlowStart; i < sharedFlowCount; i++) {
                                if (sharedFlowNodes[i] === flow) {
                                    flowDepth--;
                                    return sharedFlowTypes[i];
                                }
                            }
                        }
                        var type = void 0;
                        if (flags & 8192 /* AfterFinally */) {
                            // block flow edge: finally -> pre-try (for larger explanation check comment in binder.ts - bindTryStatement
                            flow.locked = true;
                            type = getTypeAtFlowNode(flow.antecedent);
                            flow.locked = false;
                        }
                        else if (flags & 4096 /* PreFinally */) {
                            // locked pre-finally flows are filtered out in getTypeAtFlowBranchLabel
                            // so here just redirect to antecedent
                            flow = flow.antecedent;
                            continue;
                        }
                        else if (flags & 16 /* Assignment */) {
                            type = getTypeAtFlowAssignment(flow);
                            if (!type) {
                                flow = flow.antecedent;
                                continue;
                            }
                        }
                        else if (flags & 512 /* Call */) {
                            type = getTypeAtFlowCall(flow);
                            if (!type) {
                                flow = flow.antecedent;
                                continue;
                            }
                        }
                        else if (flags & 96 /* Condition */) {
                            type = getTypeAtFlowCondition(flow);
                        }
                        else if (flags & 128 /* SwitchClause */) {
                            type = getTypeAtSwitchClause(flow);
                        }
                        else if (flags & 12 /* Label */) {
                            if (flow.antecedents.length === 1) {
                                flow = flow.antecedents[0];
                                continue;
                            }
                            type = flags & 4 /* BranchLabel */ ?
                                getTypeAtFlowBranchLabel(flow) :
                                getTypeAtFlowLoopLabel(flow);
                        }
                        else if (flags & 256 /* ArrayMutation */) {
                            type = getTypeAtFlowArrayMutation(flow);
                            if (!type) {
                                flow = flow.antecedent;
                                continue;
                            }
                        }
                        else if (flags & 2 /* Start */) {
                            // Check if we should continue with the control flow of the containing function.
                            var container = flow.node;
                            if (container && container !== flowContainer &&
                                reference.kind !== 194 /* PropertyAccessExpression */ &&
                                reference.kind !== 195 /* ElementAccessExpression */ &&
                                reference.kind !== 104 /* ThisKeyword */) {
                                flow = container.flowNode;
                                continue;
                            }
                            // At the top of the flow we have the initial type.
                            type = initialType;
                        }
                        else {
                            // Unreachable code errors are reported in the binding phase. Here we
                            // simply return the non-auto declared type to reduce follow-on errors.
                            type = convertAutoToAny(declaredType);
                        }
                        if (flags & 2048 /* Shared */) {
                            // Record visited node and the associated type in the cache.
                            sharedFlowNodes[sharedFlowCount] = flow;
                            sharedFlowTypes[sharedFlowCount] = type;
                            sharedFlowCount++;
                        }
                        flowDepth--;
                        return type;
                    }
                }
                function getInitialOrAssignedType(flow) {
                    var node = flow.node;
                    return getConstraintForLocation(node.kind === 242 /* VariableDeclaration */ || node.kind === 191 /* BindingElement */ ?
                        getInitialType(node) :
                        getAssignedType(node), reference);
                }
                function getTypeAtFlowAssignment(flow) {
                    var node = flow.node;
                    // Assignments only narrow the computed type if the declared type is a union type. Thus, we
                    // only need to evaluate the assigned type if the declared type is a union type.
                    if (isMatchingReference(reference, node)) {
                        if (!isReachableFlowNode(flow)) {
                            return unreachableNeverType;
                        }
                        if (ts.getAssignmentTargetKind(node) === 2 /* Compound */) {
                            var flowType = getTypeAtFlowNode(flow.antecedent);
                            return createFlowType(getBaseTypeOfLiteralType(getTypeFromFlowType(flowType)), isIncomplete(flowType));
                        }
                        if (declaredType === autoType || declaredType === autoArrayType) {
                            if (isEmptyArrayAssignment(node)) {
                                return getEvolvingArrayType(neverType);
                            }
                            var assignedType = getBaseTypeOfLiteralType(getInitialOrAssignedType(flow));
                            return isTypeAssignableTo(assignedType, declaredType) ? assignedType : anyArrayType;
                        }
                        if (declaredType.flags & 1048576 /* Union */) {
                            return getAssignmentReducedType(declaredType, getInitialOrAssignedType(flow));
                        }
                        return declaredType;
                    }
                    // We didn't have a direct match. However, if the reference is a dotted name, this
                    // may be an assignment to a left hand part of the reference. For example, for a
                    // reference 'x.y.z', we may be at an assignment to 'x.y' or 'x'. In that case,
                    // return the declared type.
                    if (containsMatchingReference(reference, node)) {
                        if (!isReachableFlowNode(flow)) {
                            return unreachableNeverType;
                        }
                        // A matching dotted name might also be an expando property on a function *expression*,
                        // in which case we continue control flow analysis back to the function's declaration
                        if (ts.isVariableDeclaration(node) && (ts.isInJSFile(node) || ts.isVarConst(node))) {
                            var init = ts.getDeclaredExpandoInitializer(node);
                            if (init && (init.kind === 201 /* FunctionExpression */ || init.kind === 202 /* ArrowFunction */)) {
                                return getTypeAtFlowNode(flow.antecedent);
                            }
                        }
                        return declaredType;
                    }
                    // for (const _ in ref) acts as a nonnull on ref
                    if (ts.isVariableDeclaration(node) && node.parent.parent.kind === 231 /* ForInStatement */ && isMatchingReference(reference, node.parent.parent.expression)) {
                        return getNonNullableTypeIfNeeded(getTypeFromFlowType(getTypeAtFlowNode(flow.antecedent)));
                    }
                    // Assignment doesn't affect reference
                    return undefined;
                }
                function narrowTypeByAssertion(type, expr) {
                    var node = ts.skipParentheses(expr);
                    if (node.kind === 91 /* FalseKeyword */) {
                        return unreachableNeverType;
                    }
                    if (node.kind === 209 /* BinaryExpression */) {
                        if (node.operatorToken.kind === 55 /* AmpersandAmpersandToken */) {
                            return narrowTypeByAssertion(narrowTypeByAssertion(type, node.left), node.right);
                        }
                        if (node.operatorToken.kind === 56 /* BarBarToken */) {
                            return getUnionType([narrowTypeByAssertion(type, node.left), narrowTypeByAssertion(type, node.right)]);
                        }
                    }
                    return narrowType(type, node, /*assumeTrue*/ true);
                }
                function getTypeAtFlowCall(flow) {
                    var signature = getEffectsSignature(flow.node);
                    if (signature) {
                        var predicate = getTypePredicateOfSignature(signature);
                        if (predicate && (predicate.kind === 2 /* AssertsThis */ || predicate.kind === 3 /* AssertsIdentifier */)) {
                            var flowType = getTypeAtFlowNode(flow.antecedent);
                            var type = getTypeFromFlowType(flowType);
                            var narrowedType = predicate.type ? narrowTypeByTypePredicate(type, predicate, flow.node, /*assumeTrue*/ true) :
                                predicate.kind === 3 /* AssertsIdentifier */ && predicate.parameterIndex >= 0 && predicate.parameterIndex < flow.node.arguments.length ? narrowTypeByAssertion(type, flow.node.arguments[predicate.parameterIndex]) :
                                    type;
                            return narrowedType === type ? flowType : createFlowType(narrowedType, isIncomplete(flowType));
                        }
                        if (getReturnTypeOfSignature(signature).flags & 131072 /* Never */) {
                            return unreachableNeverType;
                        }
                    }
                    return undefined;
                }
                function getTypeAtFlowArrayMutation(flow) {
                    if (declaredType === autoType || declaredType === autoArrayType) {
                        var node = flow.node;
                        var expr = node.kind === 196 /* CallExpression */ ?
                            node.expression.expression :
                            node.left.expression;
                        if (isMatchingReference(reference, getReferenceCandidate(expr))) {
                            var flowType = getTypeAtFlowNode(flow.antecedent);
                            var type = getTypeFromFlowType(flowType);
                            if (ts.getObjectFlags(type) & 256 /* EvolvingArray */) {
                                var evolvedType_1 = type;
                                if (node.kind === 196 /* CallExpression */) {
                                    for (var _i = 0, _a = node.arguments; _i < _a.length; _i++) {
                                        var arg = _a[_i];
                                        evolvedType_1 = addEvolvingArrayElementType(evolvedType_1, arg);
                                    }
                                }
                                else {
                                    // We must get the context free expression type so as to not recur in an uncached fashion on the LHS (which causes exponential blowup in compile time)
                                    var indexType = getContextFreeTypeOfExpression(node.left.argumentExpression);
                                    if (isTypeAssignableToKind(indexType, 296 /* NumberLike */)) {
                                        evolvedType_1 = addEvolvingArrayElementType(evolvedType_1, node.right);
                                    }
                                }
                                return evolvedType_1 === type ? flowType : createFlowType(evolvedType_1, isIncomplete(flowType));
                            }
                            return flowType;
                        }
                    }
                    return undefined;
                }
                function getTypeAtFlowCondition(flow) {
                    var flowType = getTypeAtFlowNode(flow.antecedent);
                    var type = getTypeFromFlowType(flowType);
                    if (type.flags & 131072 /* Never */) {
                        return flowType;
                    }
                    // If we have an antecedent type (meaning we're reachable in some way), we first
                    // attempt to narrow the antecedent type. If that produces the never type, and if
                    // the antecedent type is incomplete (i.e. a transient type in a loop), then we
                    // take the type guard as an indication that control *could* reach here once we
                    // have the complete type. We proceed by switching to the silent never type which
                    // doesn't report errors when operators are applied to it. Note that this is the
                    // *only* place a silent never type is ever generated.
                    var assumeTrue = (flow.flags & 32 /* TrueCondition */) !== 0;
                    var nonEvolvingType = finalizeEvolvingArrayType(type);
                    var narrowedType = narrowType(nonEvolvingType, flow.node, assumeTrue);
                    if (narrowedType === nonEvolvingType) {
                        return flowType;
                    }
                    var incomplete = isIncomplete(flowType);
                    var resultType = incomplete && narrowedType.flags & 131072 /* Never */ ? silentNeverType : narrowedType;
                    return createFlowType(resultType, incomplete);
                }
                function getTypeAtSwitchClause(flow) {
                    var expr = flow.switchStatement.expression;
                    var flowType = getTypeAtFlowNode(flow.antecedent);
                    var type = getTypeFromFlowType(flowType);
                    if (isMatchingReference(reference, expr)) {
                        type = narrowTypeBySwitchOnDiscriminant(type, flow.switchStatement, flow.clauseStart, flow.clauseEnd);
                    }
                    else if (expr.kind === 204 /* TypeOfExpression */ && isMatchingReference(reference, expr.expression)) {
                        type = narrowBySwitchOnTypeOf(type, flow.switchStatement, flow.clauseStart, flow.clauseEnd);
                    }
                    else {
                        if (strictNullChecks) {
                            if (optionalChainContainsReference(expr, reference)) {
                                type = narrowTypeBySwitchOptionalChainContainment(type, flow.switchStatement, flow.clauseStart, flow.clauseEnd, function (t) { return !(t.flags & (32768 /* Undefined */ | 131072 /* Never */)); });
                            }
                            else if (expr.kind === 204 /* TypeOfExpression */ && optionalChainContainsReference(expr.expression, reference)) {
                                type = narrowTypeBySwitchOptionalChainContainment(type, flow.switchStatement, flow.clauseStart, flow.clauseEnd, function (t) { return !(t.flags & 131072 /* Never */ || t.flags & 128 /* StringLiteral */ && t.value === "undefined"); });
                            }
                        }
                        if (isMatchingReferenceDiscriminant(expr, type)) {
                            type = narrowTypeByDiscriminant(type, expr, function (t) { return narrowTypeBySwitchOnDiscriminant(t, flow.switchStatement, flow.clauseStart, flow.clauseEnd); });
                        }
                    }
                    return createFlowType(type, isIncomplete(flowType));
                }
                function getTypeAtFlowBranchLabel(flow) {
                    var antecedentTypes = [];
                    var subtypeReduction = false;
                    var seenIncomplete = false;
                    var bypassFlow;
                    for (var _i = 0, _a = flow.antecedents; _i < _a.length; _i++) {
                        var antecedent = _a[_i];
                        if (antecedent.flags & 4096 /* PreFinally */ && antecedent.lock.locked) {
                            // if flow correspond to branch from pre-try to finally and this branch is locked - this means that
                            // we initially have started following the flow outside the finally block.
                            // in this case we should ignore this branch.
                            continue;
                        }
                        if (!bypassFlow && antecedent.flags & 128 /* SwitchClause */ && antecedent.clauseStart === antecedent.clauseEnd) {
                            // The antecedent is the bypass branch of a potentially exhaustive switch statement.
                            bypassFlow = antecedent;
                            continue;
                        }
                        var flowType = getTypeAtFlowNode(antecedent);
                        var type = getTypeFromFlowType(flowType);
                        // If the type at a particular antecedent path is the declared type and the
                        // reference is known to always be assigned (i.e. when declared and initial types
                        // are the same), there is no reason to process more antecedents since the only
                        // possible outcome is subtypes that will be removed in the final union type anyway.
                        if (type === declaredType && declaredType === initialType) {
                            return type;
                        }
                        ts.pushIfUnique(antecedentTypes, type);
                        // If an antecedent type is not a subset of the declared type, we need to perform
                        // subtype reduction. This happens when a "foreign" type is injected into the control
                        // flow using the instanceof operator or a user defined type predicate.
                        if (!isTypeSubsetOf(type, declaredType)) {
                            subtypeReduction = true;
                        }
                        if (isIncomplete(flowType)) {
                            seenIncomplete = true;
                        }
                    }
                    if (bypassFlow) {
                        var flowType = getTypeAtFlowNode(bypassFlow);
                        var type = getTypeFromFlowType(flowType);
                        // If the bypass flow contributes a type we haven't seen yet and the switch statement
                        // isn't exhaustive, process the bypass flow type. Since exhaustiveness checks increase
                        // the risk of circularities, we only want to perform them when they make a difference.
                        if (!ts.contains(antecedentTypes, type) && !isExhaustiveSwitchStatement(bypassFlow.switchStatement)) {
                            if (type === declaredType && declaredType === initialType) {
                                return type;
                            }
                            antecedentTypes.push(type);
                            if (!isTypeSubsetOf(type, declaredType)) {
                                subtypeReduction = true;
                            }
                            if (isIncomplete(flowType)) {
                                seenIncomplete = true;
                            }
                        }
                    }
                    return createFlowType(getUnionOrEvolvingArrayType(antecedentTypes, subtypeReduction ? 2 /* Subtype */ : 1 /* Literal */), seenIncomplete);
                }
                function getTypeAtFlowLoopLabel(flow) {
                    // If we have previously computed the control flow type for the reference at
                    // this flow loop junction, return the cached type.
                    var id = getFlowNodeId(flow);
                    var cache = flowLoopCaches[id] || (flowLoopCaches[id] = ts.createMap());
                    var key = getOrSetCacheKey();
                    if (!key) {
                        // No cache key is generated when binding patterns are in unnarrowable situations
                        return declaredType;
                    }
                    var cached = cache.get(key);
                    if (cached) {
                        return cached;
                    }
                    // If this flow loop junction and reference are already being processed, return
                    // the union of the types computed for each branch so far, marked as incomplete.
                    // It is possible to see an empty array in cases where loops are nested and the
                    // back edge of the outer loop reaches an inner loop that is already being analyzed.
                    // In such cases we restart the analysis of the inner loop, which will then see
                    // a non-empty in-process array for the outer loop and eventually terminate because
                    // the first antecedent of a loop junction is always the non-looping control flow
                    // path that leads to the top.
                    for (var i = flowLoopStart; i < flowLoopCount; i++) {
                        if (flowLoopNodes[i] === flow && flowLoopKeys[i] === key && flowLoopTypes[i].length) {
                            return createFlowType(getUnionOrEvolvingArrayType(flowLoopTypes[i], 1 /* Literal */), /*incomplete*/ true);
                        }
                    }
                    // Add the flow loop junction and reference to the in-process stack and analyze
                    // each antecedent code path.
                    var antecedentTypes = [];
                    var subtypeReduction = false;
                    var firstAntecedentType;
                    for (var _i = 0, _a = flow.antecedents; _i < _a.length; _i++) {
                        var antecedent = _a[_i];
                        var flowType = void 0;
                        if (!firstAntecedentType) {
                            // The first antecedent of a loop junction is always the non-looping control
                            // flow path that leads to the top.
                            flowType = firstAntecedentType = getTypeAtFlowNode(antecedent);
                        }
                        else {
                            // All but the first antecedent are the looping control flow paths that lead
                            // back to the loop junction. We track these on the flow loop stack.
                            flowLoopNodes[flowLoopCount] = flow;
                            flowLoopKeys[flowLoopCount] = key;
                            flowLoopTypes[flowLoopCount] = antecedentTypes;
                            flowLoopCount++;
                            var saveFlowTypeCache = flowTypeCache;
                            flowTypeCache = undefined;
                            flowType = getTypeAtFlowNode(antecedent);
                            flowTypeCache = saveFlowTypeCache;
                            flowLoopCount--;
                            // If we see a value appear in the cache it is a sign that control flow analysis
                            // was restarted and completed by checkExpressionCached. We can simply pick up
                            // the resulting type and bail out.
                            var cached_1 = cache.get(key);
                            if (cached_1) {
                                return cached_1;
                            }
                        }
                        var type = getTypeFromFlowType(flowType);
                        ts.pushIfUnique(antecedentTypes, type);
                        // If an antecedent type is not a subset of the declared type, we need to perform
                        // subtype reduction. This happens when a "foreign" type is injected into the control
                        // flow using the instanceof operator or a user defined type predicate.
                        if (!isTypeSubsetOf(type, declaredType)) {
                            subtypeReduction = true;
                        }
                        // If the type at a particular antecedent path is the declared type there is no
                        // reason to process more antecedents since the only possible outcome is subtypes
                        // that will be removed in the final union type anyway.
                        if (type === declaredType) {
                            break;
                        }
                    }
                    // The result is incomplete if the first antecedent (the non-looping control flow path)
                    // is incomplete.
                    var result = getUnionOrEvolvingArrayType(antecedentTypes, subtypeReduction ? 2 /* Subtype */ : 1 /* Literal */);
                    if (isIncomplete(firstAntecedentType)) {
                        return createFlowType(result, /*incomplete*/ true);
                    }
                    cache.set(key, result);
                    return result;
                }
                function isMatchingReferenceDiscriminant(expr, computedType) {
                    if (!(computedType.flags & 1048576 /* Union */) || !ts.isAccessExpression(expr)) {
                        return false;
                    }
                    var name = getAccessedPropertyName(expr);
                    if (name === undefined) {
                        return false;
                    }
                    return isMatchingReference(reference, expr.expression) && isDiscriminantProperty(computedType, name);
                }
                function narrowTypeByDiscriminant(type, access, narrowType) {
                    var propName = getAccessedPropertyName(access);
                    if (propName === undefined) {
                        return type;
                    }
                    var propType = getTypeOfPropertyOfType(type, propName);
                    if (!propType) {
                        return type;
                    }
                    var narrowedPropType = narrowType(propType);
                    return filterType(type, function (t) {
                        var discriminantType = getTypeOfPropertyOrIndexSignature(t, propName);
                        return !(discriminantType.flags & 131072 /* Never */) && isTypeComparableTo(discriminantType, narrowedPropType);
                    });
                }
                function narrowTypeByTruthiness(type, expr, assumeTrue) {
                    if (isMatchingReference(reference, expr)) {
                        return getTypeWithFacts(type, assumeTrue ? 4194304 /* Truthy */ : 8388608 /* Falsy */);
                    }
                    if (strictNullChecks && assumeTrue && optionalChainContainsReference(expr, reference)) {
                        type = getTypeWithFacts(type, 2097152 /* NEUndefinedOrNull */);
                    }
                    if (isMatchingReferenceDiscriminant(expr, declaredType)) {
                        return narrowTypeByDiscriminant(type, expr, function (t) { return getTypeWithFacts(t, assumeTrue ? 4194304 /* Truthy */ : 8388608 /* Falsy */); });
                    }
                    return type;
                }
                function isTypePresencePossible(type, propName, assumeTrue) {
                    if (getIndexInfoOfType(type, 0 /* String */)) {
                        return true;
                    }
                    var prop = getPropertyOfType(type, propName);
                    if (prop) {
                        return prop.flags & 16777216 /* Optional */ ? true : assumeTrue;
                    }
                    return !assumeTrue;
                }
                function narrowByInKeyword(type, literal, assumeTrue) {
                    if (type.flags & (1048576 /* Union */ | 524288 /* Object */) || isThisTypeParameter(type)) {
                        var propName_1 = ts.escapeLeadingUnderscores(literal.text);
                        return filterType(type, function (t) { return isTypePresencePossible(t, propName_1, assumeTrue); });
                    }
                    return type;
                }
                function narrowTypeByBinaryExpression(type, expr, assumeTrue) {
                    switch (expr.operatorToken.kind) {
                        case 62 /* EqualsToken */:
                            return narrowTypeByTruthiness(narrowType(type, expr.right, assumeTrue), expr.left, assumeTrue);
                        case 34 /* EqualsEqualsToken */:
                        case 35 /* ExclamationEqualsToken */:
                        case 36 /* EqualsEqualsEqualsToken */:
                        case 37 /* ExclamationEqualsEqualsToken */:
                            var operator_1 = expr.operatorToken.kind;
                            var left_1 = getReferenceCandidate(expr.left);
                            var right_1 = getReferenceCandidate(expr.right);
                            if (left_1.kind === 204 /* TypeOfExpression */ && ts.isStringLiteralLike(right_1)) {
                                return narrowTypeByTypeof(type, left_1, operator_1, right_1, assumeTrue);
                            }
                            if (right_1.kind === 204 /* TypeOfExpression */ && ts.isStringLiteralLike(left_1)) {
                                return narrowTypeByTypeof(type, right_1, operator_1, left_1, assumeTrue);
                            }
                            if (isMatchingReference(reference, left_1)) {
                                return narrowTypeByEquality(type, operator_1, right_1, assumeTrue);
                            }
                            if (isMatchingReference(reference, right_1)) {
                                return narrowTypeByEquality(type, operator_1, left_1, assumeTrue);
                            }
                            if (strictNullChecks) {
                                if (optionalChainContainsReference(left_1, reference)) {
                                    type = narrowTypeByOptionalChainContainment(type, operator_1, right_1, assumeTrue);
                                }
                                else if (optionalChainContainsReference(right_1, reference)) {
                                    type = narrowTypeByOptionalChainContainment(type, operator_1, left_1, assumeTrue);
                                }
                            }
                            if (isMatchingReferenceDiscriminant(left_1, declaredType)) {
                                return narrowTypeByDiscriminant(type, left_1, function (t) { return narrowTypeByEquality(t, operator_1, right_1, assumeTrue); });
                            }
                            if (isMatchingReferenceDiscriminant(right_1, declaredType)) {
                                return narrowTypeByDiscriminant(type, right_1, function (t) { return narrowTypeByEquality(t, operator_1, left_1, assumeTrue); });
                            }
                            break;
                        case 98 /* InstanceOfKeyword */:
                            return narrowTypeByInstanceof(type, expr, assumeTrue);
                        case 97 /* InKeyword */:
                            var target = getReferenceCandidate(expr.right);
                            if (ts.isStringLiteralLike(expr.left) && isMatchingReference(reference, target)) {
                                return narrowByInKeyword(type, expr.left, assumeTrue);
                            }
                            break;
                        case 27 /* CommaToken */:
                            return narrowType(type, expr.right, assumeTrue);
                    }
                    return type;
                }
                function narrowTypeByOptionalChainContainment(type, operator, value, assumeTrue) {
                    // We are in a branch of obj?.foo === value (or any one of the other equality operators). We narrow obj as follows:
                    // When operator is === and type of value excludes undefined, null and undefined is removed from type of obj in true branch.
                    // When operator is !== and type of value excludes undefined, null and undefined is removed from type of obj in false branch.
                    // When operator is == and type of value excludes null and undefined, null and undefined is removed from type of obj in true branch.
                    // When operator is != and type of value excludes null and undefined, null and undefined is removed from type of obj in false branch.
                    // When operator is === and type of value is undefined, null and undefined is removed from type of obj in false branch.
                    // When operator is !== and type of value is undefined, null and undefined is removed from type of obj in true branch.
                    // When operator is == and type of value is null or undefined, null and undefined is removed from type of obj in false branch.
                    // When operator is != and type of value is null or undefined, null and undefined is removed from type of obj in true branch.
                    var equalsOperator = operator === 34 /* EqualsEqualsToken */ || operator === 36 /* EqualsEqualsEqualsToken */;
                    var nullableFlags = operator === 34 /* EqualsEqualsToken */ || operator === 35 /* ExclamationEqualsToken */ ? 98304 /* Nullable */ : 32768 /* Undefined */;
                    var valueType = getTypeOfExpression(value);
                    // Note that we include any and unknown in the exclusion test because their domain includes null and undefined.
                    var removeNullable = equalsOperator !== assumeTrue && everyType(valueType, function (t) { return !!(t.flags & nullableFlags); }) ||
                        equalsOperator === assumeTrue && everyType(valueType, function (t) { return !(t.flags & (3 /* AnyOrUnknown */ | nullableFlags)); });
                    return removeNullable ? getTypeWithFacts(type, 2097152 /* NEUndefinedOrNull */) : type;
                }
                function narrowTypeByEquality(type, operator, value, assumeTrue) {
                    if (type.flags & 1 /* Any */) {
                        return type;
                    }
                    if (operator === 35 /* ExclamationEqualsToken */ || operator === 37 /* ExclamationEqualsEqualsToken */) {
                        assumeTrue = !assumeTrue;
                    }
                    var valueType = getTypeOfExpression(value);
                    if ((type.flags & 2 /* Unknown */) && assumeTrue && (operator === 36 /* EqualsEqualsEqualsToken */ || operator === 37 /* ExclamationEqualsEqualsToken */)) {
                        if (valueType.flags & (131068 /* Primitive */ | 67108864 /* NonPrimitive */)) {
                            return valueType;
                        }
                        if (valueType.flags & 524288 /* Object */) {
                            return nonPrimitiveType;
                        }
                        return type;
                    }
                    if (valueType.flags & 98304 /* Nullable */) {
                        if (!strictNullChecks) {
                            return type;
                        }
                        var doubleEquals = operator === 34 /* EqualsEqualsToken */ || operator === 35 /* ExclamationEqualsToken */;
                        var facts = doubleEquals ?
                            assumeTrue ? 262144 /* EQUndefinedOrNull */ : 2097152 /* NEUndefinedOrNull */ :
                            valueType.flags & 65536 /* Null */ ?
                                assumeTrue ? 131072 /* EQNull */ : 1048576 /* NENull */ :
                                assumeTrue ? 65536 /* EQUndefined */ : 524288 /* NEUndefined */;
                        return getTypeWithFacts(type, facts);
                    }
                    if (type.flags & 67637251 /* NotUnionOrUnit */) {
                        return type;
                    }
                    if (assumeTrue) {
                        var filterFn = operator === 34 /* EqualsEqualsToken */ ?
                            (function (t) { return areTypesComparable(t, valueType) || isCoercibleUnderDoubleEquals(t, valueType); }) :
                            function (t) { return areTypesComparable(t, valueType); };
                        var narrowedType = filterType(type, filterFn);
                        return narrowedType.flags & 131072 /* Never */ ? type : replacePrimitivesWithLiterals(narrowedType, valueType);
                    }
                    if (isUnitType(valueType)) {
                        var regularType_1 = getRegularTypeOfLiteralType(valueType);
                        return filterType(type, function (t) { return getRegularTypeOfLiteralType(t) !== regularType_1; });
                    }
                    return type;
                }
                function narrowTypeByTypeof(type, typeOfExpr, operator, literal, assumeTrue) {
                    // We have '==', '!=', '===', or !==' operator with 'typeof xxx' and string literal operands
                    if (operator === 35 /* ExclamationEqualsToken */ || operator === 37 /* ExclamationEqualsEqualsToken */) {
                        assumeTrue = !assumeTrue;
                    }
                    var target = getReferenceCandidate(typeOfExpr.expression);
                    if (!isMatchingReference(reference, target)) {
                        if (strictNullChecks && optionalChainContainsReference(target, reference) && assumeTrue === (literal.text !== "undefined")) {
                            return getTypeWithFacts(type, 2097152 /* NEUndefinedOrNull */);
                        }
                        // For a reference of the form 'x.y', a 'typeof x === ...' type guard resets the
                        // narrowed type of 'y' to its declared type.
                        if (containsMatchingReference(reference, target)) {
                            return declaredType;
                        }
                        return type;
                    }
                    if (type.flags & 1 /* Any */ && literal.text === "function") {
                        return type;
                    }
                    var facts = assumeTrue ?
                        typeofEQFacts.get(literal.text) || 128 /* TypeofEQHostObject */ :
                        typeofNEFacts.get(literal.text) || 32768 /* TypeofNEHostObject */;
                    return getTypeWithFacts(assumeTrue ? mapType(type, narrowTypeForTypeof) : type, facts);
                    function narrowTypeForTypeof(type) {
                        if (type.flags & 2 /* Unknown */ && literal.text === "object") {
                            return getUnionType([nonPrimitiveType, nullType]);
                        }
                        // We narrow a non-union type to an exact primitive type if the non-union type
                        // is a supertype of that primitive type. For example, type 'any' can be narrowed
                        // to one of the primitive types.
                        var targetType = literal.text === "function" ? globalFunctionType : typeofTypesByName.get(literal.text);
                        if (targetType) {
                            if (isTypeSubtypeOf(type, targetType)) {
                                return type;
                            }
                            if (isTypeSubtypeOf(targetType, type)) {
                                return targetType;
                            }
                            if (type.flags & 63176704 /* Instantiable */) {
                                var constraint = getBaseConstraintOfType(type) || anyType;
                                if (isTypeSubtypeOf(targetType, constraint)) {
                                    return getIntersectionType([type, targetType]);
                                }
                            }
                        }
                        return type;
                    }
                }
                function narrowTypeBySwitchOptionalChainContainment(type, switchStatement, clauseStart, clauseEnd, clauseCheck) {
                    var everyClauseChecks = clauseStart !== clauseEnd && ts.every(getSwitchClauseTypes(switchStatement).slice(clauseStart, clauseEnd), clauseCheck);
                    return everyClauseChecks ? getTypeWithFacts(type, 2097152 /* NEUndefinedOrNull */) : type;
                }
                function narrowTypeBySwitchOnDiscriminant(type, switchStatement, clauseStart, clauseEnd) {
                    // We only narrow if all case expressions specify
                    // values with unit types, except for the case where
                    // `type` is unknown. In this instance we map object
                    // types to the nonPrimitive type and narrow with that.
                    var switchTypes = getSwitchClauseTypes(switchStatement);
                    if (!switchTypes.length) {
                        return type;
                    }
                    var clauseTypes = switchTypes.slice(clauseStart, clauseEnd);
                    var hasDefaultClause = clauseStart === clauseEnd || ts.contains(clauseTypes, neverType);
                    if ((type.flags & 2 /* Unknown */) && !hasDefaultClause) {
                        var groundClauseTypes = void 0;
                        for (var i = 0; i < clauseTypes.length; i += 1) {
                            var t = clauseTypes[i];
                            if (t.flags & (131068 /* Primitive */ | 67108864 /* NonPrimitive */)) {
                                if (groundClauseTypes !== undefined) {
                                    groundClauseTypes.push(t);
                                }
                            }
                            else if (t.flags & 524288 /* Object */) {
                                if (groundClauseTypes === undefined) {
                                    groundClauseTypes = clauseTypes.slice(0, i);
                                }
                                groundClauseTypes.push(nonPrimitiveType);
                            }
                            else {
                                return type;
                            }
                        }
                        return getUnionType(groundClauseTypes === undefined ? clauseTypes : groundClauseTypes);
                    }
                    var discriminantType = getUnionType(clauseTypes);
                    var caseType = discriminantType.flags & 131072 /* Never */ ? neverType :
                        replacePrimitivesWithLiterals(filterType(type, function (t) { return areTypesComparable(discriminantType, t); }), discriminantType);
                    if (!hasDefaultClause) {
                        return caseType;
                    }
                    var defaultType = filterType(type, function (t) { return !(isUnitType(t) && ts.contains(switchTypes, getRegularTypeOfLiteralType(t))); });
                    return caseType.flags & 131072 /* Never */ ? defaultType : getUnionType([caseType, defaultType]);
                }
                function getImpliedTypeFromTypeofCase(type, text) {
                    switch (text) {
                        case "function":
                            return type.flags & 1 /* Any */ ? type : globalFunctionType;
                        case "object":
                            return type.flags & 2 /* Unknown */ ? getUnionType([nonPrimitiveType, nullType]) : type;
                        default:
                            return typeofTypesByName.get(text) || type;
                    }
                }
                function narrowTypeForTypeofSwitch(candidate) {
                    return function (type) {
                        if (isTypeSubtypeOf(candidate, type)) {
                            return candidate;
                        }
                        if (type.flags & 63176704 /* Instantiable */) {
                            var constraint = getBaseConstraintOfType(type) || anyType;
                            if (isTypeSubtypeOf(candidate, constraint)) {
                                return getIntersectionType([type, candidate]);
                            }
                        }
                        return type;
                    };
                }
                function narrowBySwitchOnTypeOf(type, switchStatement, clauseStart, clauseEnd) {
                    var switchWitnesses = getSwitchClauseTypeOfWitnesses(switchStatement);
                    if (!switchWitnesses.length) {
                        return type;
                    }
                    //  Equal start and end denotes implicit fallthrough; undefined marks explicit default clause
                    var defaultCaseLocation = ts.findIndex(switchWitnesses, function (elem) { return elem === undefined; });
                    var hasDefaultClause = clauseStart === clauseEnd || (defaultCaseLocation >= clauseStart && defaultCaseLocation < clauseEnd);
                    var clauseWitnesses;
                    var switchFacts;
                    if (defaultCaseLocation > -1) {
                        // We no longer need the undefined denoting an
                        // explicit default case. Remove the undefined and
                        // fix-up clauseStart and clauseEnd.  This means
                        // that we don't have to worry about undefined
                        // in the witness array.
                        var witnesses = switchWitnesses.filter(function (witness) { return witness !== undefined; });
                        // The adjusted clause start and end after removing the `default` statement.
                        var fixedClauseStart = defaultCaseLocation < clauseStart ? clauseStart - 1 : clauseStart;
                        var fixedClauseEnd = defaultCaseLocation < clauseEnd ? clauseEnd - 1 : clauseEnd;
                        clauseWitnesses = witnesses.slice(fixedClauseStart, fixedClauseEnd);
                        switchFacts = getFactsFromTypeofSwitch(fixedClauseStart, fixedClauseEnd, witnesses, hasDefaultClause);
                    }
                    else {
                        clauseWitnesses = switchWitnesses.slice(clauseStart, clauseEnd);
                        switchFacts = getFactsFromTypeofSwitch(clauseStart, clauseEnd, switchWitnesses, hasDefaultClause);
                    }
                    if (hasDefaultClause) {
                        return filterType(type, function (t) { return (getTypeFacts(t) & switchFacts) === switchFacts; });
                    }
                    /*
                      The implied type is the raw type suggested by a
                      value being caught in this clause.
    
                      When the clause contains a default case we ignore
                      the implied type and try to narrow using any facts
                      we can learn: see `switchFacts`.
    
                      Example:
                      switch (typeof x) {
                          case 'number':
                          case 'string': break;
                          default: break;
                          case 'number':
                          case 'boolean': break
                      }
    
                      In the first clause (case `number` and `string`) the
                      implied type is number | string.
    
                      In the default clause we de not compute an implied type.
    
                      In the third clause (case `number` and `boolean`)
                      the naive implied type is number | boolean, however
                      we use the type facts to narrow the implied type to
                      boolean. We know that number cannot be selected
                      because it is caught in the first clause.
                    */
                    var impliedType = getTypeWithFacts(getUnionType(clauseWitnesses.map(function (text) { return getImpliedTypeFromTypeofCase(type, text); })), switchFacts);
                    if (impliedType.flags & 1048576 /* Union */) {
                        impliedType = getAssignmentReducedType(impliedType, getBaseConstraintOrType(type));
                    }
                    return getTypeWithFacts(mapType(type, narrowTypeForTypeofSwitch(impliedType)), switchFacts);
                }
                function narrowTypeByInstanceof(type, expr, assumeTrue) {
                    var left = getReferenceCandidate(expr.left);
                    if (!isMatchingReference(reference, left)) {
                        if (assumeTrue && strictNullChecks && optionalChainContainsReference(left, reference)) {
                            return getTypeWithFacts(type, 2097152 /* NEUndefinedOrNull */);
                        }
                        // For a reference of the form 'x.y', an 'x instanceof T' type guard resets the
                        // narrowed type of 'y' to its declared type. We do this because preceding 'x.y'
                        // references might reference a different 'y' property. However, we make an exception
                        // for property accesses where x is a synthetic 'this' expression, indicating that we
                        // were called from isPropertyInitializedInConstructor. Without this exception,
                        // initializations of 'this' properties that occur before a 'this instanceof XXX'
                        // check would not be considered.
                        if (containsMatchingReference(reference, left) && !isSyntheticThisPropertyAccess(reference)) {
                            return declaredType;
                        }
                        return type;
                    }
                    // Check that right operand is a function type with a prototype property
                    var rightType = getTypeOfExpression(expr.right);
                    if (!isTypeDerivedFrom(rightType, globalFunctionType)) {
                        return type;
                    }
                    var targetType;
                    var prototypeProperty = getPropertyOfType(rightType, "prototype");
                    if (prototypeProperty) {
                        // Target type is type of the prototype property
                        var prototypePropertyType = getTypeOfSymbol(prototypeProperty);
                        if (!isTypeAny(prototypePropertyType)) {
                            targetType = prototypePropertyType;
                        }
                    }
                    // Don't narrow from 'any' if the target type is exactly 'Object' or 'Function'
                    if (isTypeAny(type) && (targetType === globalObjectType || targetType === globalFunctionType)) {
                        return type;
                    }
                    if (!targetType) {
                        var constructSignatures = getSignaturesOfType(rightType, 1 /* Construct */);
                        targetType = constructSignatures.length ?
                            getUnionType(ts.map(constructSignatures, function (signature) { return getReturnTypeOfSignature(getErasedSignature(signature)); })) :
                            emptyObjectType;
                    }
                    return getNarrowedType(type, targetType, assumeTrue, isTypeDerivedFrom);
                }
                function getNarrowedType(type, candidate, assumeTrue, isRelated) {
                    if (!assumeTrue) {
                        return filterType(type, function (t) { return !isRelated(t, candidate); });
                    }
                    // If the current type is a union type, remove all constituents that couldn't be instances of
                    // the candidate type. If one or more constituents remain, return a union of those.
                    if (type.flags & 1048576 /* Union */) {
                        var assignableType = filterType(type, function (t) { return isRelated(t, candidate); });
                        if (!(assignableType.flags & 131072 /* Never */)) {
                            return assignableType;
                        }
                    }
                    // If the candidate type is a subtype of the target type, narrow to the candidate type.
                    // Otherwise, if the target type is assignable to the candidate type, keep the target type.
                    // Otherwise, if the candidate type is assignable to the target type, narrow to the candidate
                    // type. Otherwise, the types are completely unrelated, so narrow to an intersection of the
                    // two types.
                    return isTypeSubtypeOf(candidate, type) ? candidate :
                        isTypeAssignableTo(type, candidate) ? type :
                            isTypeAssignableTo(candidate, type) ? candidate :
                                getIntersectionType([type, candidate]);
                }
                function narrowTypeByCallExpression(type, callExpression, assumeTrue) {
                    if (hasMatchingArgument(callExpression, reference)) {
                        var signature = assumeTrue || !ts.isCallChain(callExpression) ? getEffectsSignature(callExpression) : undefined;
                        var predicate = signature && getTypePredicateOfSignature(signature);
                        if (predicate && (predicate.kind === 0 /* This */ || predicate.kind === 1 /* Identifier */)) {
                            return narrowTypeByTypePredicate(type, predicate, callExpression, assumeTrue);
                        }
                    }
                    return type;
                }
                function narrowTypeByTypePredicate(type, predicate, callExpression, assumeTrue) {
                    // Don't narrow from 'any' if the predicate type is exactly 'Object' or 'Function'
                    if (predicate.type && !(isTypeAny(type) && (predicate.type === globalObjectType || predicate.type === globalFunctionType))) {
                        var predicateArgument = getTypePredicateArgument(predicate, callExpression);
                        if (predicateArgument) {
                            if (isMatchingReference(reference, predicateArgument)) {
                                return getNarrowedType(type, predicate.type, assumeTrue, isTypeSubtypeOf);
                            }
                            if (strictNullChecks && assumeTrue && optionalChainContainsReference(predicateArgument, reference) &&
                                !(getTypeFacts(predicate.type) & 65536 /* EQUndefined */)) {
                                return getTypeWithFacts(type, 2097152 /* NEUndefinedOrNull */);
                            }
                            if (containsMatchingReference(reference, predicateArgument)) {
                                return declaredType;
                            }
                        }
                    }
                    return type;
                }
                // Narrow the given type based on the given expression having the assumed boolean value. The returned type
                // will be a subtype or the same type as the argument.
                function narrowType(type, expr, assumeTrue) {
                    // for `a?.b`, we emulate a synthetic `a !== null && a !== undefined` condition for `a`
                    if (ts.isExpressionOfOptionalChainRoot(expr) ||
                        ts.isBinaryExpression(expr.parent) && expr.parent.operatorToken.kind === 60 /* QuestionQuestionToken */ && expr.parent.left === expr) {
                        return narrowTypeByOptionality(type, expr, assumeTrue);
                    }
                    switch (expr.kind) {
                        case 75 /* Identifier */:
                        case 104 /* ThisKeyword */:
                        case 102 /* SuperKeyword */:
                        case 194 /* PropertyAccessExpression */:
                        case 195 /* ElementAccessExpression */:
                            return narrowTypeByTruthiness(type, expr, assumeTrue);
                        case 196 /* CallExpression */:
                            return narrowTypeByCallExpression(type, expr, assumeTrue);
                        case 200 /* ParenthesizedExpression */:
                            return narrowType(type, expr.expression, assumeTrue);
                        case 209 /* BinaryExpression */:
                            return narrowTypeByBinaryExpression(type, expr, assumeTrue);
                        case 207 /* PrefixUnaryExpression */:
                            if (expr.operator === 53 /* ExclamationToken */) {
                                return narrowType(type, expr.operand, !assumeTrue);
                            }
                            break;
                    }
                    return type;
                }
                function narrowTypeByOptionality(type, expr, assumePresent) {
                    if (isMatchingReference(reference, expr)) {
                        return getTypeWithFacts(type, assumePresent ? 2097152 /* NEUndefinedOrNull */ : 262144 /* EQUndefinedOrNull */);
                    }
                    if (isMatchingReferenceDiscriminant(expr, declaredType)) {
                        return narrowTypeByDiscriminant(type, expr, function (t) { return getTypeWithFacts(t, assumePresent ? 2097152 /* NEUndefinedOrNull */ : 262144 /* EQUndefinedOrNull */); });
                    }
                    return type;
                }
            }
            function getTypeOfSymbolAtLocation(symbol, location) {
                symbol = symbol.exportSymbol || symbol;
                // If we have an identifier or a property access at the given location, if the location is
                // an dotted name expression, and if the location is not an assignment target, obtain the type
                // of the expression (which will reflect control flow analysis). If the expression indeed
                // resolved to the given symbol, return the narrowed type.
                if (location.kind === 75 /* Identifier */) {
                    if (ts.isRightSideOfQualifiedNameOrPropertyAccess(location)) {
                        location = location.parent;
                    }
                    if (ts.isExpressionNode(location) && !ts.isAssignmentTarget(location)) {
                        var type = getTypeOfExpression(location);
                        if (getExportSymbolOfValueSymbolIfExported(getNodeLinks(location).resolvedSymbol) === symbol) {
                            return type;
                        }
                    }
                }
                // The location isn't a reference to the given symbol, meaning we're being asked
                // a hypothetical question of what type the symbol would have if there was a reference
                // to it at the given location. Since we have no control flow information for the
                // hypothetical reference (control flow information is created and attached by the
                // binder), we simply return the declared type of the symbol.
                return getTypeOfSymbol(symbol);
            }
            function getControlFlowContainer(node) {
                return ts.findAncestor(node.parent, function (node) {
                    return ts.isFunctionLike(node) && !ts.getImmediatelyInvokedFunctionExpression(node) ||
                        node.kind === 250 /* ModuleBlock */ ||
                        node.kind === 290 /* SourceFile */ ||
                        node.kind === 159 /* PropertyDeclaration */;
                });
            }
            // Check if a parameter is assigned anywhere within its declaring function.
            function isParameterAssigned(symbol) {
                var func = ts.getRootDeclaration(symbol.valueDeclaration).parent;
                var links = getNodeLinks(func);
                if (!(links.flags & 8388608 /* AssignmentsMarked */)) {
                    links.flags |= 8388608 /* AssignmentsMarked */;
                    if (!hasParentWithAssignmentsMarked(func)) {
                        markParameterAssignments(func);
                    }
                }
                return symbol.isAssigned || false;
            }
            function hasParentWithAssignmentsMarked(node) {
                return !!ts.findAncestor(node.parent, function (node) { return ts.isFunctionLike(node) && !!(getNodeLinks(node).flags & 8388608 /* AssignmentsMarked */); });
            }
            function markParameterAssignments(node) {
                if (node.kind === 75 /* Identifier */) {
                    if (ts.isAssignmentTarget(node)) {
                        var symbol = getResolvedSymbol(node);
                        if (symbol.valueDeclaration && ts.getRootDeclaration(symbol.valueDeclaration).kind === 156 /* Parameter */) {
                            symbol.isAssigned = true;
                        }
                    }
                }
                else {
                    ts.forEachChild(node, markParameterAssignments);
                }
            }
            function isConstVariable(symbol) {
                return symbol.flags & 3 /* Variable */ && (getDeclarationNodeFlagsFromSymbol(symbol) & 2 /* Const */) !== 0 && getTypeOfSymbol(symbol) !== autoArrayType;
            }
            /** remove undefined from the annotated type of a parameter when there is an initializer (that doesn't include undefined) */
            function removeOptionalityFromDeclaredType(declaredType, declaration) {
                var annotationIncludesUndefined = strictNullChecks &&
                    declaration.kind === 156 /* Parameter */ &&
                    declaration.initializer &&
                    getFalsyFlags(declaredType) & 32768 /* Undefined */ &&
                    !(getFalsyFlags(checkExpression(declaration.initializer)) & 32768 /* Undefined */);
                return annotationIncludesUndefined ? getTypeWithFacts(declaredType, 524288 /* NEUndefined */) : declaredType;
            }
            function isConstraintPosition(node) {
                var parent = node.parent;
                return parent.kind === 194 /* PropertyAccessExpression */ ||
                    parent.kind === 196 /* CallExpression */ && parent.expression === node ||
                    parent.kind === 195 /* ElementAccessExpression */ && parent.expression === node ||
                    parent.kind === 191 /* BindingElement */ && parent.name === node && !!parent.initializer;
            }
            function typeHasNullableConstraint(type) {
                return type.flags & 58982400 /* InstantiableNonPrimitive */ && maybeTypeOfKind(getBaseConstraintOfType(type) || unknownType, 98304 /* Nullable */);
            }
            function getConstraintForLocation(type, node) {
                // When a node is the left hand expression of a property access, element access, or call expression,
                // and the type of the node includes type variables with constraints that are nullable, we fetch the
                // apparent type of the node *before* performing control flow analysis such that narrowings apply to
                // the constraint type.
                if (type && isConstraintPosition(node) && forEachType(type, typeHasNullableConstraint)) {
                    return mapType(getWidenedType(type), getBaseConstraintOrType);
                }
                return type;
            }
            function isExportOrExportExpression(location) {
                return !!ts.findAncestor(location, function (e) { return e.parent && ts.isExportAssignment(e.parent) && e.parent.expression === e && ts.isEntityNameExpression(e); });
            }
            function markAliasReferenced(symbol, location) {
                if (isNonLocalAlias(symbol, /*excludes*/ 111551 /* Value */) && !isInTypeQuery(location) && !getTypeOnlyAliasDeclaration(symbol)) {
                    if (compilerOptions.preserveConstEnums && isExportOrExportExpression(location) || !isConstEnumOrConstEnumOnlyModule(resolveAlias(symbol))) {
                        markAliasSymbolAsReferenced(symbol);
                    }
                    else {
                        markConstEnumAliasAsReferenced(symbol);
                    }
                }
            }
            function checkIdentifier(node) {
                var symbol = getResolvedSymbol(node);
                if (symbol === unknownSymbol) {
                    return errorType;
                }
                // As noted in ECMAScript 6 language spec, arrow functions never have an arguments objects.
                // Although in down-level emit of arrow function, we emit it using function expression which means that
                // arguments objects will be bound to the inner object; emitting arrow function natively in ES6, arguments objects
                // will be bound to non-arrow function that contain this arrow function. This results in inconsistent behavior.
                // To avoid that we will give an error to users if they use arguments objects in arrow function so that they
                // can explicitly bound arguments objects
                if (symbol === argumentsSymbol) {
                    var container = ts.getContainingFunction(node);
                    if (languageVersion < 2 /* ES2015 */) {
                        if (container.kind === 202 /* ArrowFunction */) {
                            error(node, ts.Diagnostics.The_arguments_object_cannot_be_referenced_in_an_arrow_function_in_ES3_and_ES5_Consider_using_a_standard_function_expression);
                        }
                        else if (ts.hasModifier(container, 256 /* Async */)) {
                            error(node, ts.Diagnostics.The_arguments_object_cannot_be_referenced_in_an_async_function_or_method_in_ES3_and_ES5_Consider_using_a_standard_function_or_method);
                        }
                    }
                    getNodeLinks(container).flags |= 8192 /* CaptureArguments */;
                    return getTypeOfSymbol(symbol);
                }
                // We should only mark aliases as referenced if there isn't a local value declaration
                // for the symbol. Also, don't mark any property access expression LHS - checkPropertyAccessExpression will handle that
                if (!(node.parent && ts.isPropertyAccessExpression(node.parent) && node.parent.expression === node)) {
                    markAliasReferenced(symbol, node);
                }
                var localOrExportSymbol = getExportSymbolOfValueSymbolIfExported(symbol);
                var declaration = localOrExportSymbol.valueDeclaration;
                if (localOrExportSymbol.flags & 32 /* Class */) {
                    // Due to the emit for class decorators, any reference to the class from inside of the class body
                    // must instead be rewritten to point to a temporary variable to avoid issues with the double-bind
                    // behavior of class names in ES6.
                    if (declaration.kind === 245 /* ClassDeclaration */
                        && ts.nodeIsDecorated(declaration)) {
                        var container = ts.getContainingClass(node);
                        while (container !== undefined) {
                            if (container === declaration && container.name !== node) {
                                getNodeLinks(declaration).flags |= 16777216 /* ClassWithConstructorReference */;
                                getNodeLinks(node).flags |= 33554432 /* ConstructorReferenceInClass */;
                                break;
                            }
                            container = ts.getContainingClass(container);
                        }
                    }
                    else if (declaration.kind === 214 /* ClassExpression */) {
                        // When we emit a class expression with static members that contain a reference
                        // to the constructor in the initializer, we will need to substitute that
                        // binding with an alias as the class name is not in scope.
                        var container = ts.getThisContainer(node, /*includeArrowFunctions*/ false);
                        while (container.kind !== 290 /* SourceFile */) {
                            if (container.parent === declaration) {
                                if (container.kind === 159 /* PropertyDeclaration */ && ts.hasModifier(container, 32 /* Static */)) {
                                    getNodeLinks(declaration).flags |= 16777216 /* ClassWithConstructorReference */;
                                    getNodeLinks(node).flags |= 33554432 /* ConstructorReferenceInClass */;
                                }
                                break;
                            }
                            container = ts.getThisContainer(container, /*includeArrowFunctions*/ false);
                        }
                    }
                }
                checkNestedBlockScopedBinding(node, symbol);
                var type = getConstraintForLocation(getTypeOfSymbol(localOrExportSymbol), node);
                var assignmentKind = ts.getAssignmentTargetKind(node);
                if (assignmentKind) {
                    if (!(localOrExportSymbol.flags & 3 /* Variable */) &&
                        !(ts.isInJSFile(node) && localOrExportSymbol.flags & 512 /* ValueModule */)) {
                        error(node, ts.Diagnostics.Cannot_assign_to_0_because_it_is_not_a_variable, symbolToString(symbol));
                        return errorType;
                    }
                    if (isReadonlySymbol(localOrExportSymbol)) {
                        if (localOrExportSymbol.flags & 3 /* Variable */) {
                            error(node, ts.Diagnostics.Cannot_assign_to_0_because_it_is_a_constant, symbolToString(symbol));
                        }
                        else {
                            error(node, ts.Diagnostics.Cannot_assign_to_0_because_it_is_a_read_only_property, symbolToString(symbol));
                        }
                        return errorType;
                    }
                }
                var isAlias = localOrExportSymbol.flags & 2097152 /* Alias */;
                // We only narrow variables and parameters occurring in a non-assignment position. For all other
                // entities we simply return the declared type.
                if (localOrExportSymbol.flags & 3 /* Variable */) {
                    if (assignmentKind === 1 /* Definite */) {
                        return type;
                    }
                }
                else if (isAlias) {
                    declaration = ts.find(symbol.declarations, isSomeImportDeclaration);
                }
                else {
                    return type;
                }
                if (!declaration) {
                    return type;
                }
                // The declaration container is the innermost function that encloses the declaration of the variable
                // or parameter. The flow container is the innermost function starting with which we analyze the control
                // flow graph to determine the control flow based type.
                var isParameter = ts.getRootDeclaration(declaration).kind === 156 /* Parameter */;
                var declarationContainer = getControlFlowContainer(declaration);
                var flowContainer = getControlFlowContainer(node);
                var isOuterVariable = flowContainer !== declarationContainer;
                var isSpreadDestructuringAssignmentTarget = node.parent && node.parent.parent && ts.isSpreadAssignment(node.parent) && isDestructuringAssignmentTarget(node.parent.parent);
                var isModuleExports = symbol.flags & 134217728 /* ModuleExports */;
                // When the control flow originates in a function expression or arrow function and we are referencing
                // a const variable or parameter from an outer function, we extend the origin of the control flow
                // analysis to include the immediately enclosing function.
                while (flowContainer !== declarationContainer && (flowContainer.kind === 201 /* FunctionExpression */ ||
                    flowContainer.kind === 202 /* ArrowFunction */ || ts.isObjectLiteralOrClassExpressionMethod(flowContainer)) &&
                    (isConstVariable(localOrExportSymbol) || isParameter && !isParameterAssigned(localOrExportSymbol))) {
                    flowContainer = getControlFlowContainer(flowContainer);
                }
                // We only look for uninitialized variables in strict null checking mode, and only when we can analyze
                // the entire control flow graph from the variable's declaration (i.e. when the flow container and
                // declaration container are the same).
                var assumeInitialized = isParameter || isAlias || isOuterVariable || isSpreadDestructuringAssignmentTarget || isModuleExports || ts.isBindingElement(declaration) ||
                    type !== autoType && type !== autoArrayType && (!strictNullChecks || (type.flags & (3 /* AnyOrUnknown */ | 16384 /* Void */)) !== 0 ||
                        isInTypeQuery(node) || node.parent.kind === 263 /* ExportSpecifier */) ||
                    node.parent.kind === 218 /* NonNullExpression */ ||
                    declaration.kind === 242 /* VariableDeclaration */ && declaration.exclamationToken ||
                    declaration.flags & 8388608 /* Ambient */;
                var initialType = assumeInitialized ? (isParameter ? removeOptionalityFromDeclaredType(type, declaration) : type) :
                    type === autoType || type === autoArrayType ? undefinedType :
                        getOptionalType(type);
                var flowType = getFlowTypeOfReference(node, type, initialType, flowContainer, !assumeInitialized);
                // A variable is considered uninitialized when it is possible to analyze the entire control flow graph
                // from declaration to use, and when the variable's declared type doesn't include undefined but the
                // control flow based type does include undefined.
                if (!isEvolvingArrayOperationTarget(node) && (type === autoType || type === autoArrayType)) {
                    if (flowType === autoType || flowType === autoArrayType) {
                        if (noImplicitAny) {
                            error(ts.getNameOfDeclaration(declaration), ts.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined, symbolToString(symbol), typeToString(flowType));
                            error(node, ts.Diagnostics.Variable_0_implicitly_has_an_1_type, symbolToString(symbol), typeToString(flowType));
                        }
                        return convertAutoToAny(flowType);
                    }
                }
                else if (!assumeInitialized && !(getFalsyFlags(type) & 32768 /* Undefined */) && getFalsyFlags(flowType) & 32768 /* Undefined */) {
                    error(node, ts.Diagnostics.Variable_0_is_used_before_being_assigned, symbolToString(symbol));
                    // Return the declared type to reduce follow-on errors
                    return type;
                }
                return assignmentKind ? getBaseTypeOfLiteralType(flowType) : flowType;
            }
            function isInsideFunction(node, threshold) {
                return !!ts.findAncestor(node, function (n) { return n === threshold ? "quit" : ts.isFunctionLike(n); });
            }
            function getPartOfForStatementContainingNode(node, container) {
                return ts.findAncestor(node, function (n) { return n === container ? "quit" : n === container.initializer || n === container.condition || n === container.incrementor || n === container.statement; });
            }
            function checkNestedBlockScopedBinding(node, symbol) {
                if (languageVersion >= 2 /* ES2015 */ ||
                    (symbol.flags & (2 /* BlockScopedVariable */ | 32 /* Class */)) === 0 ||
                    ts.isSourceFile(symbol.valueDeclaration) ||
                    symbol.valueDeclaration.parent.kind === 280 /* CatchClause */) {
                    return;
                }
                // 1. walk from the use site up to the declaration and check
                // if there is anything function like between declaration and use-site (is binding/class is captured in function).
                // 2. walk from the declaration up to the boundary of lexical environment and check
                // if there is an iteration statement in between declaration and boundary (is binding/class declared inside iteration statement)
                var container = ts.getEnclosingBlockScopeContainer(symbol.valueDeclaration);
                var usedInFunction = isInsideFunction(node.parent, container);
                var current = container;
                var containedInIterationStatement = false;
                while (current && !ts.nodeStartsNewLexicalEnvironment(current)) {
                    if (ts.isIterationStatement(current, /*lookInLabeledStatements*/ false)) {
                        containedInIterationStatement = true;
                        break;
                    }
                    current = current.parent;
                }
                if (containedInIterationStatement) {
                    if (usedInFunction) {
                        // mark iteration statement as containing block-scoped binding captured in some function
                        var capturesBlockScopeBindingInLoopBody = true;
                        if (ts.isForStatement(container)) {
                            var varDeclList = ts.getAncestor(symbol.valueDeclaration, 243 /* VariableDeclarationList */);
                            if (varDeclList && varDeclList.parent === container) {
                                var part = getPartOfForStatementContainingNode(node.parent, container);
                                if (part) {
                                    var links = getNodeLinks(part);
                                    links.flags |= 131072 /* ContainsCapturedBlockScopeBinding */;
                                    var capturedBindings = links.capturedBlockScopeBindings || (links.capturedBlockScopeBindings = []);
                                    ts.pushIfUnique(capturedBindings, symbol);
                                    if (part === container.initializer) {
                                        capturesBlockScopeBindingInLoopBody = false; // Initializer is outside of loop body
                                    }
                                }
                            }
                        }
                        if (capturesBlockScopeBindingInLoopBody) {
                            getNodeLinks(current).flags |= 65536 /* LoopWithCapturedBlockScopedBinding */;
                        }
                    }
                    // mark variables that are declared in loop initializer and reassigned inside the body of ForStatement.
                    // if body of ForStatement will be converted to function then we'll need a extra machinery to propagate reassigned values back.
                    if (ts.isForStatement(container)) {
                        var varDeclList = ts.getAncestor(symbol.valueDeclaration, 243 /* VariableDeclarationList */);
                        if (varDeclList && varDeclList.parent === container && isAssignedInBodyOfForStatement(node, container)) {
                            getNodeLinks(symbol.valueDeclaration).flags |= 4194304 /* NeedsLoopOutParameter */;
                        }
                    }
                    // set 'declared inside loop' bit on the block-scoped binding
                    getNodeLinks(symbol.valueDeclaration).flags |= 524288 /* BlockScopedBindingInLoop */;
                }
                if (usedInFunction) {
                    getNodeLinks(symbol.valueDeclaration).flags |= 262144 /* CapturedBlockScopedBinding */;
                }
            }
            function isBindingCapturedByNode(node, decl) {
                var links = getNodeLinks(node);
                return !!links && ts.contains(links.capturedBlockScopeBindings, getSymbolOfNode(decl));
            }
            function isAssignedInBodyOfForStatement(node, container) {
                // skip parenthesized nodes
                var current = node;
                while (current.parent.kind === 200 /* ParenthesizedExpression */) {
                    current = current.parent;
                }
                // check if node is used as LHS in some assignment expression
                var isAssigned = false;
                if (ts.isAssignmentTarget(current)) {
                    isAssigned = true;
                }
                else if ((current.parent.kind === 207 /* PrefixUnaryExpression */ || current.parent.kind === 208 /* PostfixUnaryExpression */)) {
                    var expr = current.parent;
                    isAssigned = expr.operator === 45 /* PlusPlusToken */ || expr.operator === 46 /* MinusMinusToken */;
                }
                if (!isAssigned) {
                    return false;
                }
                // at this point we know that node is the target of assignment
                // now check that modification happens inside the statement part of the ForStatement
                return !!ts.findAncestor(current, function (n) { return n === container ? "quit" : n === container.statement; });
            }
            function captureLexicalThis(node, container) {
                getNodeLinks(node).flags |= 2 /* LexicalThis */;
                if (container.kind === 159 /* PropertyDeclaration */ || container.kind === 162 /* Constructor */) {
                    var classNode = container.parent;
                    getNodeLinks(classNode).flags |= 4 /* CaptureThis */;
                }
                else {
                    getNodeLinks(container).flags |= 4 /* CaptureThis */;
                }
            }
            function findFirstSuperCall(n) {
                if (ts.isSuperCall(n)) {
                    return n;
                }
                else if (ts.isFunctionLike(n)) {
                    return undefined;
                }
                return ts.forEachChild(n, findFirstSuperCall);
            }
            /**
             * Return a cached result if super-statement is already found.
             * Otherwise, find a super statement in a given constructor function and cache the result in the node-links of the constructor
             *
             * @param constructor constructor-function to look for super statement
             */
            function getSuperCallInConstructor(constructor) {
                var links = getNodeLinks(constructor);
                // Only trying to find super-call if we haven't yet tried to find one.  Once we try, we will record the result
                if (links.hasSuperCall === undefined) {
                    links.superCall = findFirstSuperCall(constructor.body);
                    links.hasSuperCall = links.superCall ? true : false;
                }
                return links.superCall;
            }
            /**
             * Check if the given class-declaration extends null then return true.
             * Otherwise, return false
             * @param classDecl a class declaration to check if it extends null
             */
            function classDeclarationExtendsNull(classDecl) {
                var classSymbol = getSymbolOfNode(classDecl);
                var classInstanceType = getDeclaredTypeOfSymbol(classSymbol);
                var baseConstructorType = getBaseConstructorTypeOfClass(classInstanceType);
                return baseConstructorType === nullWideningType;
            }
            function checkThisBeforeSuper(node, container, diagnosticMessage) {
                var containingClassDecl = container.parent;
                var baseTypeNode = ts.getClassExtendsHeritageElement(containingClassDecl);
                // If a containing class does not have extends clause or the class extends null
                // skip checking whether super statement is called before "this" accessing.
                if (baseTypeNode && !classDeclarationExtendsNull(containingClassDecl)) {
                    var superCall = getSuperCallInConstructor(container);
                    // We should give an error in the following cases:
                    //      - No super-call
                    //      - "this" is accessing before super-call.
                    //          i.e super(this)
                    //              this.x; super();
                    // We want to make sure that super-call is done before accessing "this" so that
                    // "this" is not accessed as a parameter of the super-call.
                    if (!superCall || superCall.end > node.pos) {
                        // In ES6, super inside constructor of class-declaration has to precede "this" accessing
                        error(node, diagnosticMessage);
                    }
                }
            }
            function checkThisExpression(node) {
                // Stop at the first arrow function so that we can
                // tell whether 'this' needs to be captured.
                var container = ts.getThisContainer(node, /* includeArrowFunctions */ true);
                var capturedByArrowFunction = false;
                if (container.kind === 162 /* Constructor */) {
                    checkThisBeforeSuper(node, container, ts.Diagnostics.super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class);
                }
                // Now skip arrow functions to get the "real" owner of 'this'.
                if (container.kind === 202 /* ArrowFunction */) {
                    container = ts.getThisContainer(container, /* includeArrowFunctions */ false);
                    capturedByArrowFunction = true;
                }
                switch (container.kind) {
                    case 249 /* ModuleDeclaration */:
                        error(node, ts.Diagnostics.this_cannot_be_referenced_in_a_module_or_namespace_body);
                        // do not return here so in case if lexical this is captured - it will be reflected in flags on NodeLinks
                        break;
                    case 248 /* EnumDeclaration */:
                        error(node, ts.Diagnostics.this_cannot_be_referenced_in_current_location);
                        // do not return here so in case if lexical this is captured - it will be reflected in flags on NodeLinks
                        break;
                    case 162 /* Constructor */:
                        if (isInConstructorArgumentInitializer(node, container)) {
                            error(node, ts.Diagnostics.this_cannot_be_referenced_in_constructor_arguments);
                            // do not return here so in case if lexical this is captured - it will be reflected in flags on NodeLinks
                        }
                        break;
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                        if (ts.hasModifier(container, 32 /* Static */)) {
                            error(node, ts.Diagnostics.this_cannot_be_referenced_in_a_static_property_initializer);
                            // do not return here so in case if lexical this is captured - it will be reflected in flags on NodeLinks
                        }
                        break;
                    case 154 /* ComputedPropertyName */:
                        error(node, ts.Diagnostics.this_cannot_be_referenced_in_a_computed_property_name);
                        break;
                }
                // When targeting es6, mark that we'll need to capture `this` in its lexically bound scope.
                if (capturedByArrowFunction && languageVersion < 2 /* ES2015 */) {
                    captureLexicalThis(node, container);
                }
                var type = tryGetThisTypeAt(node, /*includeGlobalThis*/ true, container);
                if (noImplicitThis) {
                    var globalThisType_1 = getTypeOfSymbol(globalThisSymbol);
                    if (type === globalThisType_1 && capturedByArrowFunction) {
                        error(node, ts.Diagnostics.The_containing_arrow_function_captures_the_global_value_of_this);
                    }
                    else if (!type) {
                        // With noImplicitThis, functions may not reference 'this' if it has type 'any'
                        var diag = error(node, ts.Diagnostics.this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation);
                        if (!ts.isSourceFile(container)) {
                            var outsideThis = tryGetThisTypeAt(container);
                            if (outsideThis && outsideThis !== globalThisType_1) {
                                ts.addRelatedInfo(diag, ts.createDiagnosticForNode(container, ts.Diagnostics.An_outer_value_of_this_is_shadowed_by_this_container));
                            }
                        }
                    }
                }
                return type || anyType;
            }
            function tryGetThisTypeAt(node, includeGlobalThis, container) {
                if (includeGlobalThis === void 0) {
                    includeGlobalThis = true;
                }
                if (container === void 0) {
                    container = ts.getThisContainer(node, /*includeArrowFunctions*/ false);
                }
                var isInJS = ts.isInJSFile(node);
                if (ts.isFunctionLike(container) &&
                    (!isInParameterInitializerBeforeContainingFunction(node) || ts.getThisParameter(container))) {
                    // Note: a parameter initializer should refer to class-this unless function-this is explicitly annotated.
                    // If this is a function in a JS file, it might be a class method.
                    var className = getClassNameFromPrototypeMethod(container);
                    if (isInJS && className) {
                        var classSymbol = checkExpression(className).symbol;
                        if (classSymbol && classSymbol.members && (classSymbol.flags & 16 /* Function */)) {
                            var classType = getDeclaredTypeOfSymbol(classSymbol).thisType;
                            return getFlowTypeOfReference(node, classType);
                        }
                    }
                    // Check if it's a constructor definition, can be either a variable decl or function decl
                    // i.e.
                    //   * /** @constructor */ function [name]() { ... }
                    //   * /** @constructor */ var x = function() { ... }
                    else if (isInJS &&
                        (container.kind === 201 /* FunctionExpression */ || container.kind === 244 /* FunctionDeclaration */) &&
                        ts.getJSDocClassTag(container)) {
                        var classType = getDeclaredTypeOfSymbol(getMergedSymbol(container.symbol)).thisType;
                        return getFlowTypeOfReference(node, classType);
                    }
                    var thisType = getThisTypeOfDeclaration(container) || getContextualThisParameterType(container);
                    if (thisType) {
                        return getFlowTypeOfReference(node, thisType);
                    }
                }
                if (ts.isClassLike(container.parent)) {
                    var symbol = getSymbolOfNode(container.parent);
                    var type = ts.hasModifier(container, 32 /* Static */) ? getTypeOfSymbol(symbol) : getDeclaredTypeOfSymbol(symbol).thisType;
                    return getFlowTypeOfReference(node, type);
                }
                if (isInJS) {
                    var type = getTypeForThisExpressionFromJSDoc(container);
                    if (type && type !== errorType) {
                        return getFlowTypeOfReference(node, type);
                    }
                }
                if (ts.isSourceFile(container)) {
                    // look up in the source file's locals or exports
                    if (container.commonJsModuleIndicator) {
                        var fileSymbol = getSymbolOfNode(container);
                        return fileSymbol && getTypeOfSymbol(fileSymbol);
                    }
                    else if (includeGlobalThis) {
                        return getTypeOfSymbol(globalThisSymbol);
                    }
                }
            }
            function getExplicitThisType(node) {
                var container = ts.getThisContainer(node, /*includeArrowFunctions*/ false);
                if (ts.isFunctionLike(container)) {
                    var signature = getSignatureFromDeclaration(container);
                    if (signature.thisParameter) {
                        return getExplicitTypeOfSymbol(signature.thisParameter);
                    }
                }
                if (ts.isClassLike(container.parent)) {
                    var symbol = getSymbolOfNode(container.parent);
                    return ts.hasModifier(container, 32 /* Static */) ? getTypeOfSymbol(symbol) : getDeclaredTypeOfSymbol(symbol).thisType;
                }
            }
            function getClassNameFromPrototypeMethod(container) {
                // Check if it's the RHS of a x.prototype.y = function [name]() { .... }
                if (container.kind === 201 /* FunctionExpression */ &&
                    ts.isBinaryExpression(container.parent) &&
                    ts.getAssignmentDeclarationKind(container.parent) === 3 /* PrototypeProperty */) {
                    // Get the 'x' of 'x.prototype.y = container'
                    return container.parent // x.prototype.y = container
                        .left // x.prototype.y
                        .expression // x.prototype
                        .expression; // x
                }
                // x.prototype = { method() { } }
                else if (container.kind === 161 /* MethodDeclaration */ &&
                    container.parent.kind === 193 /* ObjectLiteralExpression */ &&
                    ts.isBinaryExpression(container.parent.parent) &&
                    ts.getAssignmentDeclarationKind(container.parent.parent) === 6 /* Prototype */) {
                    return container.parent.parent.left.expression;
                }
                // x.prototype = { method: function() { } }
                else if (container.kind === 201 /* FunctionExpression */ &&
                    container.parent.kind === 281 /* PropertyAssignment */ &&
                    container.parent.parent.kind === 193 /* ObjectLiteralExpression */ &&
                    ts.isBinaryExpression(container.parent.parent.parent) &&
                    ts.getAssignmentDeclarationKind(container.parent.parent.parent) === 6 /* Prototype */) {
                    return container.parent.parent.parent.left.expression;
                }
                // Object.defineProperty(x, "method", { value: function() { } });
                // Object.defineProperty(x, "method", { set: (x: () => void) => void });
                // Object.defineProperty(x, "method", { get: () => function() { }) });
                else if (container.kind === 201 /* FunctionExpression */ &&
                    ts.isPropertyAssignment(container.parent) &&
                    ts.isIdentifier(container.parent.name) &&
                    (container.parent.name.escapedText === "value" || container.parent.name.escapedText === "get" || container.parent.name.escapedText === "set") &&
                    ts.isObjectLiteralExpression(container.parent.parent) &&
                    ts.isCallExpression(container.parent.parent.parent) &&
                    container.parent.parent.parent.arguments[2] === container.parent.parent &&
                    ts.getAssignmentDeclarationKind(container.parent.parent.parent) === 9 /* ObjectDefinePrototypeProperty */) {
                    return container.parent.parent.parent.arguments[0].expression;
                }
                // Object.defineProperty(x, "method", { value() { } });
                // Object.defineProperty(x, "method", { set(x: () => void) {} });
                // Object.defineProperty(x, "method", { get() { return () => {} } });
                else if (ts.isMethodDeclaration(container) &&
                    ts.isIdentifier(container.name) &&
                    (container.name.escapedText === "value" || container.name.escapedText === "get" || container.name.escapedText === "set") &&
                    ts.isObjectLiteralExpression(container.parent) &&
                    ts.isCallExpression(container.parent.parent) &&
                    container.parent.parent.arguments[2] === container.parent &&
                    ts.getAssignmentDeclarationKind(container.parent.parent) === 9 /* ObjectDefinePrototypeProperty */) {
                    return container.parent.parent.arguments[0].expression;
                }
            }
            function getTypeForThisExpressionFromJSDoc(node) {
                var jsdocType = ts.getJSDocType(node);
                if (jsdocType && jsdocType.kind === 300 /* JSDocFunctionType */) {
                    var jsDocFunctionType = jsdocType;
                    if (jsDocFunctionType.parameters.length > 0 &&
                        jsDocFunctionType.parameters[0].name &&
                        jsDocFunctionType.parameters[0].name.escapedText === "this" /* This */) {
                        return getTypeFromTypeNode(jsDocFunctionType.parameters[0].type);
                    }
                }
                var thisTag = ts.getJSDocThisTag(node);
                if (thisTag && thisTag.typeExpression) {
                    return getTypeFromTypeNode(thisTag.typeExpression);
                }
            }
            function isInConstructorArgumentInitializer(node, constructorDecl) {
                return !!ts.findAncestor(node, function (n) { return ts.isFunctionLikeDeclaration(n) ? "quit" : n.kind === 156 /* Parameter */ && n.parent === constructorDecl; });
            }
            function checkSuperExpression(node) {
                var isCallExpression = node.parent.kind === 196 /* CallExpression */ && node.parent.expression === node;
                var container = ts.getSuperContainer(node, /*stopOnFunctions*/ true);
                var needToCaptureLexicalThis = false;
                // adjust the container reference in case if super is used inside arrow functions with arbitrarily deep nesting
                if (!isCallExpression) {
                    while (container && container.kind === 202 /* ArrowFunction */) {
                        container = ts.getSuperContainer(container, /*stopOnFunctions*/ true);
                        needToCaptureLexicalThis = languageVersion < 2 /* ES2015 */;
                    }
                }
                var canUseSuperExpression = isLegalUsageOfSuperExpression(container);
                var nodeCheckFlag = 0;
                if (!canUseSuperExpression) {
                    // issue more specific error if super is used in computed property name
                    // class A { foo() { return "1" }}
                    // class B {
                    //     [super.foo()]() {}
                    // }
                    var current = ts.findAncestor(node, function (n) { return n === container ? "quit" : n.kind === 154 /* ComputedPropertyName */; });
                    if (current && current.kind === 154 /* ComputedPropertyName */) {
                        error(node, ts.Diagnostics.super_cannot_be_referenced_in_a_computed_property_name);
                    }
                    else if (isCallExpression) {
                        error(node, ts.Diagnostics.Super_calls_are_not_permitted_outside_constructors_or_in_nested_functions_inside_constructors);
                    }
                    else if (!container || !container.parent || !(ts.isClassLike(container.parent) || container.parent.kind === 193 /* ObjectLiteralExpression */)) {
                        error(node, ts.Diagnostics.super_can_only_be_referenced_in_members_of_derived_classes_or_object_literal_expressions);
                    }
                    else {
                        error(node, ts.Diagnostics.super_property_access_is_permitted_only_in_a_constructor_member_function_or_member_accessor_of_a_derived_class);
                    }
                    return errorType;
                }
                if (!isCallExpression && container.kind === 162 /* Constructor */) {
                    checkThisBeforeSuper(node, container, ts.Diagnostics.super_must_be_called_before_accessing_a_property_of_super_in_the_constructor_of_a_derived_class);
                }
                if (ts.hasModifier(container, 32 /* Static */) || isCallExpression) {
                    nodeCheckFlag = 512 /* SuperStatic */;
                }
                else {
                    nodeCheckFlag = 256 /* SuperInstance */;
                }
                getNodeLinks(node).flags |= nodeCheckFlag;
                // Due to how we emit async functions, we need to specialize the emit for an async method that contains a `super` reference.
                // This is due to the fact that we emit the body of an async function inside of a generator function. As generator
                // functions cannot reference `super`, we emit a helper inside of the method body, but outside of the generator. This helper
                // uses an arrow function, which is permitted to reference `super`.
                //
                // There are two primary ways we can access `super` from within an async method. The first is getting the value of a property
                // or indexed access on super, either as part of a right-hand-side expression or call expression. The second is when setting the value
                // of a property or indexed access, either as part of an assignment expression or destructuring assignment.
                //
                // The simplest case is reading a value, in which case we will emit something like the following:
                //
                //  // ts
                //  ...
                //  async asyncMethod() {
                //    let x = await super.asyncMethod();
                //    return x;
                //  }
                //  ...
                //
                //  // js
                //  ...
                //  asyncMethod() {
                //      const _super = Object.create(null, {
                //        asyncMethod: { get: () => super.asyncMethod },
                //      });
                //      return __awaiter(this, arguments, Promise, function *() {
                //          let x = yield _super.asyncMethod.call(this);
                //          return x;
                //      });
                //  }
                //  ...
                //
                // The more complex case is when we wish to assign a value, especially as part of a destructuring assignment. As both cases
                // are legal in ES6, but also likely less frequent, we only emit setters if there is an assignment:
                //
                //  // ts
                //  ...
                //  async asyncMethod(ar: Promise<any[]>) {
                //      [super.a, super.b] = await ar;
                //  }
                //  ...
                //
                //  // js
                //  ...
                //  asyncMethod(ar) {
                //      const _super = Object.create(null, {
                //        a: { get: () => super.a, set: (v) => super.a = v },
                //        b: { get: () => super.b, set: (v) => super.b = v }
                //      };
                //      return __awaiter(this, arguments, Promise, function *() {
                //          [_super.a, _super.b] = yield ar;
                //      });
                //  }
                //  ...
                //
                // Creating an object that has getter and setters instead of just an accessor function is required for destructuring assignments
                // as a call expression cannot be used as the target of a destructuring assignment while a property access can.
                //
                // For element access expressions (`super[x]`), we emit a generic helper that forwards the element access in both situations.
                if (container.kind === 161 /* MethodDeclaration */ && ts.hasModifier(container, 256 /* Async */)) {
                    if (ts.isSuperProperty(node.parent) && ts.isAssignmentTarget(node.parent)) {
                        getNodeLinks(container).flags |= 4096 /* AsyncMethodWithSuperBinding */;
                    }
                    else {
                        getNodeLinks(container).flags |= 2048 /* AsyncMethodWithSuper */;
                    }
                }
                if (needToCaptureLexicalThis) {
                    // call expressions are allowed only in constructors so they should always capture correct 'this'
                    // super property access expressions can also appear in arrow functions -
                    // in this case they should also use correct lexical this
                    captureLexicalThis(node.parent, container);
                }
                if (container.parent.kind === 193 /* ObjectLiteralExpression */) {
                    if (languageVersion < 2 /* ES2015 */) {
                        error(node, ts.Diagnostics.super_is_only_allowed_in_members_of_object_literal_expressions_when_option_target_is_ES2015_or_higher);
                        return errorType;
                    }
                    else {
                        // for object literal assume that type of 'super' is 'any'
                        return anyType;
                    }
                }
                // at this point the only legal case for parent is ClassLikeDeclaration
                var classLikeDeclaration = container.parent;
                if (!ts.getClassExtendsHeritageElement(classLikeDeclaration)) {
                    error(node, ts.Diagnostics.super_can_only_be_referenced_in_a_derived_class);
                    return errorType;
                }
                var classType = getDeclaredTypeOfSymbol(getSymbolOfNode(classLikeDeclaration));
                var baseClassType = classType && getBaseTypes(classType)[0];
                if (!baseClassType) {
                    return errorType;
                }
                if (container.kind === 162 /* Constructor */ && isInConstructorArgumentInitializer(node, container)) {
                    // issue custom error message for super property access in constructor arguments (to be aligned with old compiler)
                    error(node, ts.Diagnostics.super_cannot_be_referenced_in_constructor_arguments);
                    return errorType;
                }
                return nodeCheckFlag === 512 /* SuperStatic */
                    ? getBaseConstructorTypeOfClass(classType)
                    : getTypeWithThisArgument(baseClassType, classType.thisType);
                function isLegalUsageOfSuperExpression(container) {
                    if (!container) {
                        return false;
                    }
                    if (isCallExpression) {
                        // TS 1.0 SPEC (April 2014): 4.8.1
                        // Super calls are only permitted in constructors of derived classes
                        return container.kind === 162 /* Constructor */;
                    }
                    else {
                        // TS 1.0 SPEC (April 2014)
                        // 'super' property access is allowed
                        // - In a constructor, instance member function, instance member accessor, or instance member variable initializer where this references a derived class instance
                        // - In a static member function or static member accessor
                        // topmost container must be something that is directly nested in the class declaration\object literal expression
                        if (ts.isClassLike(container.parent) || container.parent.kind === 193 /* ObjectLiteralExpression */) {
                            if (ts.hasModifier(container, 32 /* Static */)) {
                                return container.kind === 161 /* MethodDeclaration */ ||
                                    container.kind === 160 /* MethodSignature */ ||
                                    container.kind === 163 /* GetAccessor */ ||
                                    container.kind === 164 /* SetAccessor */;
                            }
                            else {
                                return container.kind === 161 /* MethodDeclaration */ ||
                                    container.kind === 160 /* MethodSignature */ ||
                                    container.kind === 163 /* GetAccessor */ ||
                                    container.kind === 164 /* SetAccessor */ ||
                                    container.kind === 159 /* PropertyDeclaration */ ||
                                    container.kind === 158 /* PropertySignature */ ||
                                    container.kind === 162 /* Constructor */;
                            }
                        }
                    }
                    return false;
                }
            }
            function getContainingObjectLiteral(func) {
                return (func.kind === 161 /* MethodDeclaration */ ||
                    func.kind === 163 /* GetAccessor */ ||
                    func.kind === 164 /* SetAccessor */) && func.parent.kind === 193 /* ObjectLiteralExpression */ ? func.parent :
                    func.kind === 201 /* FunctionExpression */ && func.parent.kind === 281 /* PropertyAssignment */ ? func.parent.parent :
                        undefined;
            }
            function getThisTypeArgument(type) {
                return ts.getObjectFlags(type) & 4 /* Reference */ && type.target === globalThisType ? getTypeArguments(type)[0] : undefined;
            }
            function getThisTypeFromContextualType(type) {
                return mapType(type, function (t) {
                    return t.flags & 2097152 /* Intersection */ ? ts.forEach(t.types, getThisTypeArgument) : getThisTypeArgument(t);
                });
            }
            function getContextualThisParameterType(func) {
                if (func.kind === 202 /* ArrowFunction */) {
                    return undefined;
                }
                if (isContextSensitiveFunctionOrObjectLiteralMethod(func)) {
                    var contextualSignature = getContextualSignature(func);
                    if (contextualSignature) {
                        var thisParameter = contextualSignature.thisParameter;
                        if (thisParameter) {
                            return getTypeOfSymbol(thisParameter);
                        }
                    }
                }
                var inJs = ts.isInJSFile(func);
                if (noImplicitThis || inJs) {
                    var containingLiteral = getContainingObjectLiteral(func);
                    if (containingLiteral) {
                        // We have an object literal method. Check if the containing object literal has a contextual type
                        // that includes a ThisType<T>. If so, T is the contextual type for 'this'. We continue looking in
                        // any directly enclosing object literals.
                        var contextualType = getApparentTypeOfContextualType(containingLiteral);
                        var literal = containingLiteral;
                        var type = contextualType;
                        while (type) {
                            var thisType = getThisTypeFromContextualType(type);
                            if (thisType) {
                                return instantiateType(thisType, getMapperFromContext(getInferenceContext(containingLiteral)));
                            }
                            if (literal.parent.kind !== 281 /* PropertyAssignment */) {
                                break;
                            }
                            literal = literal.parent.parent;
                            type = getApparentTypeOfContextualType(literal);
                        }
                        // There was no contextual ThisType<T> for the containing object literal, so the contextual type
                        // for 'this' is the non-null form of the contextual type for the containing object literal or
                        // the type of the object literal itself.
                        return getWidenedType(contextualType ? getNonNullableType(contextualType) : checkExpressionCached(containingLiteral));
                    }
                    // In an assignment of the form 'obj.xxx = function(...)' or 'obj[xxx] = function(...)', the
                    // contextual type for 'this' is 'obj'.
                    var parent = ts.walkUpParenthesizedExpressions(func.parent);
                    if (parent.kind === 209 /* BinaryExpression */ && parent.operatorToken.kind === 62 /* EqualsToken */) {
                        var target = parent.left;
                        if (ts.isAccessExpression(target)) {
                            var expression = target.expression;
                            // Don't contextually type `this` as `exports` in `exports.Point = function(x, y) { this.x = x; this.y = y; }`
                            if (inJs && ts.isIdentifier(expression)) {
                                var sourceFile = ts.getSourceFileOfNode(parent);
                                if (sourceFile.commonJsModuleIndicator && getResolvedSymbol(expression) === sourceFile.symbol) {
                                    return undefined;
                                }
                            }
                            return getWidenedType(checkExpressionCached(expression));
                        }
                    }
                }
                return undefined;
            }
            // Return contextual type of parameter or undefined if no contextual type is available
            function getContextuallyTypedParameterType(parameter) {
                var func = parameter.parent;
                if (!isContextSensitiveFunctionOrObjectLiteralMethod(func)) {
                    return undefined;
                }
                var iife = ts.getImmediatelyInvokedFunctionExpression(func);
                if (iife && iife.arguments) {
                    var args = getEffectiveCallArguments(iife);
                    var indexOfParameter = func.parameters.indexOf(parameter);
                    if (parameter.dotDotDotToken) {
                        return getSpreadArgumentType(args, indexOfParameter, args.length, anyType, /*context*/ undefined);
                    }
                    var links = getNodeLinks(iife);
                    var cached = links.resolvedSignature;
                    links.resolvedSignature = anySignature;
                    var type = indexOfParameter < args.length ?
                        getWidenedLiteralType(checkExpression(args[indexOfParameter])) :
                        parameter.initializer ? undefined : undefinedWideningType;
                    links.resolvedSignature = cached;
                    return type;
                }
                var contextualSignature = getContextualSignature(func);
                if (contextualSignature) {
                    var index = func.parameters.indexOf(parameter) - (ts.getThisParameter(func) ? 1 : 0);
                    return parameter.dotDotDotToken && ts.lastOrUndefined(func.parameters) === parameter ?
                        getRestTypeAtPosition(contextualSignature, index) :
                        tryGetTypeAtPosition(contextualSignature, index);
                }
            }
            function getContextualTypeForVariableLikeDeclaration(declaration) {
                var typeNode = ts.getEffectiveTypeAnnotationNode(declaration);
                if (typeNode) {
                    return getTypeFromTypeNode(typeNode);
                }
                switch (declaration.kind) {
                    case 156 /* Parameter */:
                        return getContextuallyTypedParameterType(declaration);
                    case 191 /* BindingElement */:
                        return getContextualTypeForBindingElement(declaration);
                    // By default, do nothing and return undefined - only parameters and binding elements have context implied by a parent
                }
            }
            function getContextualTypeForBindingElement(declaration) {
                var parent = declaration.parent.parent;
                var name = declaration.propertyName || declaration.name;
                var parentType = getContextualTypeForVariableLikeDeclaration(parent) ||
                    parent.kind !== 191 /* BindingElement */ && parent.initializer && checkDeclarationInitializer(parent);
                if (parentType && !ts.isBindingPattern(name) && !isComputedNonLiteralName(name)) {
                    var nameType = getLiteralTypeFromPropertyName(name);
                    if (isTypeUsableAsPropertyName(nameType)) {
                        var text = getPropertyNameFromType(nameType);
                        return getTypeOfPropertyOfType(parentType, text);
                    }
                }
            }
            // In a variable, parameter or property declaration with a type annotation,
            //   the contextual type of an initializer expression is the type of the variable, parameter or property.
            // Otherwise, in a parameter declaration of a contextually typed function expression,
            //   the contextual type of an initializer expression is the contextual type of the parameter.
            // Otherwise, in a variable or parameter declaration with a binding pattern name,
            //   the contextual type of an initializer expression is the type implied by the binding pattern.
            // Otherwise, in a binding pattern inside a variable or parameter declaration,
            //   the contextual type of an initializer expression is the type annotation of the containing declaration, if present.
            function getContextualTypeForInitializerExpression(node) {
                var declaration = node.parent;
                if (ts.hasInitializer(declaration) && node === declaration.initializer) {
                    var result = getContextualTypeForVariableLikeDeclaration(declaration);
                    if (result) {
                        return result;
                    }
                    if (ts.isBindingPattern(declaration.name)) { // This is less a contextual type and more an implied shape - in some cases, this may be undesirable
                        return getTypeFromBindingPattern(declaration.name, /*includePatternInType*/ true, /*reportErrors*/ false);
                    }
                }
                return undefined;
            }
            function getContextualTypeForReturnExpression(node) {
                var func = ts.getContainingFunction(node);
                if (func) {
                    var functionFlags = ts.getFunctionFlags(func);
                    if (functionFlags & 1 /* Generator */) { // AsyncGenerator function or Generator function
                        return undefined;
                    }
                    var contextualReturnType = getContextualReturnType(func);
                    if (contextualReturnType) {
                        if (functionFlags & 2 /* Async */) { // Async function
                            var contextualAwaitedType = getAwaitedTypeOfPromise(contextualReturnType);
                            return contextualAwaitedType && getUnionType([contextualAwaitedType, createPromiseLikeType(contextualAwaitedType)]);
                        }
                        return contextualReturnType; // Regular function
                    }
                }
                return undefined;
            }
            function getContextualTypeForAwaitOperand(node) {
                var contextualType = getContextualType(node);
                if (contextualType) {
                    var contextualAwaitedType = getAwaitedType(contextualType);
                    return contextualAwaitedType && getUnionType([contextualAwaitedType, createPromiseLikeType(contextualAwaitedType)]);
                }
                return undefined;
            }
            function getContextualTypeForYieldOperand(node) {
                var func = ts.getContainingFunction(node);
                if (func) {
                    var functionFlags = ts.getFunctionFlags(func);
                    var contextualReturnType = getContextualReturnType(func);
                    if (contextualReturnType) {
                        return node.asteriskToken
                            ? contextualReturnType
                            : getIterationTypeOfGeneratorFunctionReturnType(0 /* Yield */, contextualReturnType, (functionFlags & 2 /* Async */) !== 0);
                    }
                }
                return undefined;
            }
            function isInParameterInitializerBeforeContainingFunction(node) {
                var inBindingInitializer = false;
                while (node.parent && !ts.isFunctionLike(node.parent)) {
                    if (ts.isParameter(node.parent) && (inBindingInitializer || node.parent.initializer === node)) {
                        return true;
                    }
                    if (ts.isBindingElement(node.parent) && node.parent.initializer === node) {
                        inBindingInitializer = true;
                    }
                    node = node.parent;
                }
                return false;
            }
            function getContextualIterationType(kind, functionDecl) {
                var isAsync = !!(ts.getFunctionFlags(functionDecl) & 2 /* Async */);
                var contextualReturnType = getContextualReturnType(functionDecl);
                if (contextualReturnType) {
                    return getIterationTypeOfGeneratorFunctionReturnType(kind, contextualReturnType, isAsync)
                        || undefined;
                }
                return undefined;
            }
            function getContextualReturnType(functionDecl) {
                // If the containing function has a return type annotation, is a constructor, or is a get accessor whose
                // corresponding set accessor has a type annotation, return statements in the function are contextually typed
                var returnType = getReturnTypeFromAnnotation(functionDecl);
                if (returnType) {
                    return returnType;
                }
                // Otherwise, if the containing function is contextually typed by a function type with exactly one call signature
                // and that call signature is non-generic, return statements are contextually typed by the return type of the signature
                var signature = getContextualSignatureForFunctionLikeDeclaration(functionDecl);
                if (signature && !isResolvingReturnTypeOfSignature(signature)) {
                    return getReturnTypeOfSignature(signature);
                }
                return undefined;
            }
            // In a typed function call, an argument or substitution expression is contextually typed by the type of the corresponding parameter.
            function getContextualTypeForArgument(callTarget, arg) {
                var args = getEffectiveCallArguments(callTarget);
                var argIndex = args.indexOf(arg); // -1 for e.g. the expression of a CallExpression, or the tag of a TaggedTemplateExpression
                return argIndex === -1 ? undefined : getContextualTypeForArgumentAtIndex(callTarget, argIndex);
            }
            function getContextualTypeForArgumentAtIndex(callTarget, argIndex) {
                // If we're already in the process of resolving the given signature, don't resolve again as
                // that could cause infinite recursion. Instead, return anySignature.
                var signature = getNodeLinks(callTarget).resolvedSignature === resolvingSignature ? resolvingSignature : getResolvedSignature(callTarget);
                if (ts.isJsxOpeningLikeElement(callTarget) && argIndex === 0) {
                    return getEffectiveFirstArgumentForJsxSignature(signature, callTarget);
                }
                return getTypeAtPosition(signature, argIndex);
            }
            function getContextualTypeForSubstitutionExpression(template, substitutionExpression) {
                if (template.parent.kind === 198 /* TaggedTemplateExpression */) {
                    return getContextualTypeForArgument(template.parent, substitutionExpression);
                }
                return undefined;
            }
            function getContextualTypeForBinaryOperand(node, contextFlags) {
                var binaryExpression = node.parent;
                var left = binaryExpression.left, operatorToken = binaryExpression.operatorToken, right = binaryExpression.right;
                switch (operatorToken.kind) {
                    case 62 /* EqualsToken */:
                        if (node !== right) {
                            return undefined;
                        }
                        var contextSensitive = getIsContextSensitiveAssignmentOrContextType(binaryExpression);
                        if (!contextSensitive) {
                            return undefined;
                        }
                        return contextSensitive === true ? getTypeOfExpression(left) : contextSensitive;
                    case 56 /* BarBarToken */:
                    case 60 /* QuestionQuestionToken */:
                        // When an || expression has a contextual type, the operands are contextually typed by that type, except
                        // when that type originates in a binding pattern, the right operand is contextually typed by the type of
                        // the left operand. When an || expression has no contextual type, the right operand is contextually typed
                        // by the type of the left operand, except for the special case of Javascript declarations of the form
                        // `namespace.prop = namespace.prop || {}`.
                        var type = getContextualType(binaryExpression, contextFlags);
                        return node === right && (type && type.pattern || !type && !ts.isDefaultedExpandoInitializer(binaryExpression)) ?
                            getTypeOfExpression(left) : type;
                    case 55 /* AmpersandAmpersandToken */:
                    case 27 /* CommaToken */:
                        return node === right ? getContextualType(binaryExpression, contextFlags) : undefined;
                    default:
                        return undefined;
                }
            }
            // In an assignment expression, the right operand is contextually typed by the type of the left operand.
            // Don't do this for assignment declarations unless there is a type tag on the assignment, to avoid circularity from checking the right operand.
            function getIsContextSensitiveAssignmentOrContextType(binaryExpression) {
                var kind = ts.getAssignmentDeclarationKind(binaryExpression);
                switch (kind) {
                    case 0 /* None */:
                        return true;
                    case 5 /* Property */:
                    case 1 /* ExportsProperty */:
                    case 6 /* Prototype */:
                    case 3 /* PrototypeProperty */:
                        // If `binaryExpression.left` was assigned a symbol, then this is a new declaration; otherwise it is an assignment to an existing declaration.
                        // See `bindStaticPropertyAssignment` in `binder.ts`.
                        if (!binaryExpression.left.symbol) {
                            return true;
                        }
                        else {
                            var decl = binaryExpression.left.symbol.valueDeclaration;
                            if (!decl) {
                                return false;
                            }
                            var lhs = ts.cast(binaryExpression.left, ts.isAccessExpression);
                            var overallAnnotation = ts.getEffectiveTypeAnnotationNode(decl);
                            if (overallAnnotation) {
                                return getTypeFromTypeNode(overallAnnotation);
                            }
                            else if (ts.isIdentifier(lhs.expression)) {
                                var id = lhs.expression;
                                var parentSymbol = resolveName(id, id.escapedText, 111551 /* Value */, undefined, id.escapedText, /*isUse*/ true);
                                if (parentSymbol) {
                                    var annotated = ts.getEffectiveTypeAnnotationNode(parentSymbol.valueDeclaration);
                                    if (annotated) {
                                        var nameStr_1 = ts.getElementOrPropertyAccessName(lhs);
                                        if (nameStr_1 !== undefined) {
                                            var type = getTypeOfPropertyOfContextualType(getTypeFromTypeNode(annotated), nameStr_1);
                                            return type || false;
                                        }
                                    }
                                    return false;
                                }
                            }
                            return !ts.isInJSFile(decl);
                        }
                    case 2 /* ModuleExports */:
                    case 4 /* ThisProperty */:
                        if (!binaryExpression.symbol)
                            return true;
                        if (binaryExpression.symbol.valueDeclaration) {
                            var annotated = ts.getEffectiveTypeAnnotationNode(binaryExpression.symbol.valueDeclaration);
                            if (annotated) {
                                var type = getTypeFromTypeNode(annotated);
                                if (type) {
                                    return type;
                                }
                            }
                        }
                        if (kind === 2 /* ModuleExports */)
                            return false;
                        var thisAccess = ts.cast(binaryExpression.left, ts.isAccessExpression);
                        if (!ts.isObjectLiteralMethod(ts.getThisContainer(thisAccess.expression, /*includeArrowFunctions*/ false))) {
                            return false;
                        }
                        var thisType = checkThisExpression(thisAccess.expression);
                        var nameStr = ts.getElementOrPropertyAccessName(thisAccess);
                        return nameStr !== undefined && thisType && getTypeOfPropertyOfContextualType(thisType, nameStr) || false;
                    case 7 /* ObjectDefinePropertyValue */:
                    case 8 /* ObjectDefinePropertyExports */:
                    case 9 /* ObjectDefinePrototypeProperty */:
                        return ts.Debug.fail("Does not apply");
                    default:
                        return ts.Debug.assertNever(kind);
                }
            }
            function getTypeOfPropertyOfContextualType(type, name) {
                return mapType(type, function (t) {
                    if (isGenericMappedType(t)) {
                        var constraint = getConstraintTypeFromMappedType(t);
                        var constraintOfConstraint = getBaseConstraintOfType(constraint) || constraint;
                        var propertyNameType = getLiteralType(ts.unescapeLeadingUnderscores(name));
                        if (isTypeAssignableTo(propertyNameType, constraintOfConstraint)) {
                            return substituteIndexedMappedType(t, propertyNameType);
                        }
                    }
                    else if (t.flags & 3670016 /* StructuredType */) {
                        var prop = getPropertyOfType(t, name);
                        if (prop) {
                            return getTypeOfSymbol(prop);
                        }
                        if (isTupleType(t)) {
                            var restType = getRestTypeOfTupleType(t);
                            if (restType && isNumericLiteralName(name) && +name >= 0) {
                                return restType;
                            }
                        }
                        return isNumericLiteralName(name) && getIndexTypeOfContextualType(t, 1 /* Number */) ||
                            getIndexTypeOfContextualType(t, 0 /* String */);
                    }
                    return undefined;
                }, /*noReductions*/ true);
            }
            function getIndexTypeOfContextualType(type, kind) {
                return mapType(type, function (t) { return getIndexTypeOfStructuredType(t, kind); }, /*noReductions*/ true);
            }
            // In an object literal contextually typed by a type T, the contextual type of a property assignment is the type of
            // the matching property in T, if one exists. Otherwise, it is the type of the numeric index signature in T, if one
            // exists. Otherwise, it is the type of the string index signature in T, if one exists.
            function getContextualTypeForObjectLiteralMethod(node, contextFlags) {
                ts.Debug.assert(ts.isObjectLiteralMethod(node));
                if (node.flags & 16777216 /* InWithStatement */) {
                    // We cannot answer semantic questions within a with block, do not proceed any further
                    return undefined;
                }
                return getContextualTypeForObjectLiteralElement(node, contextFlags);
            }
            function getContextualTypeForObjectLiteralElement(element, contextFlags) {
                var objectLiteral = element.parent;
                var type = getApparentTypeOfContextualType(objectLiteral, contextFlags);
                if (type) {
                    if (!hasNonBindableDynamicName(element)) {
                        // For a (non-symbol) computed property, there is no reason to look up the name
                        // in the type. It will just be "__computed", which does not appear in any
                        // SymbolTable.
                        var symbolName_3 = getSymbolOfNode(element).escapedName;
                        var propertyType = getTypeOfPropertyOfContextualType(type, symbolName_3);
                        if (propertyType) {
                            return propertyType;
                        }
                    }
                    return isNumericName(element.name) && getIndexTypeOfContextualType(type, 1 /* Number */) ||
                        getIndexTypeOfContextualType(type, 0 /* String */);
                }
                return undefined;
            }
            // In an array literal contextually typed by a type T, the contextual type of an element expression at index N is
            // the type of the property with the numeric name N in T, if one exists. Otherwise, if T has a numeric index signature,
            // it is the type of the numeric index signature in T. Otherwise, in ES6 and higher, the contextual type is the iterated
            // type of T.
            function getContextualTypeForElementExpression(arrayContextualType, index) {
                return arrayContextualType && (getTypeOfPropertyOfContextualType(arrayContextualType, "" + index)
                    || getIteratedTypeOrElementType(1 /* Element */, arrayContextualType, undefinedType, /*errorNode*/ undefined, /*checkAssignability*/ false));
            }
            // In a contextually typed conditional expression, the true/false expressions are contextually typed by the same type.
            function getContextualTypeForConditionalOperand(node, contextFlags) {
                var conditional = node.parent;
                return node === conditional.whenTrue || node === conditional.whenFalse ? getContextualType(conditional, contextFlags) : undefined;
            }
            function getContextualTypeForChildJsxExpression(node, child) {
                var attributesType = getApparentTypeOfContextualType(node.openingElement.tagName);
                // JSX expression is in children of JSX Element, we will look for an "children" atttribute (we get the name from JSX.ElementAttributesProperty)
                var jsxChildrenPropertyName = getJsxElementChildrenPropertyName(getJsxNamespaceAt(node));
                if (!(attributesType && !isTypeAny(attributesType) && jsxChildrenPropertyName && jsxChildrenPropertyName !== "")) {
                    return undefined;
                }
                var realChildren = getSemanticJsxChildren(node.children);
                var childIndex = realChildren.indexOf(child);
                var childFieldType = getTypeOfPropertyOfContextualType(attributesType, jsxChildrenPropertyName);
                return childFieldType && (realChildren.length === 1 ? childFieldType : mapType(childFieldType, function (t) {
                    if (isArrayLikeType(t)) {
                        return getIndexedAccessType(t, getLiteralType(childIndex));
                    }
                    else {
                        return t;
                    }
                }, /*noReductions*/ true));
            }
            function getContextualTypeForJsxExpression(node) {
                var exprParent = node.parent;
                return ts.isJsxAttributeLike(exprParent)
                    ? getContextualType(node)
                    : ts.isJsxElement(exprParent)
                        ? getContextualTypeForChildJsxExpression(exprParent, node)
                        : undefined;
            }
            function getContextualTypeForJsxAttribute(attribute) {
                // When we trying to resolve JsxOpeningLikeElement as a stateless function element, we will already give its attributes a contextual type
                // which is a type of the parameter of the signature we are trying out.
                // If there is no contextual type (e.g. we are trying to resolve stateful component), get attributes type from resolving element's tagName
                if (ts.isJsxAttribute(attribute)) {
                    var attributesType = getApparentTypeOfContextualType(attribute.parent);
                    if (!attributesType || isTypeAny(attributesType)) {
                        return undefined;
                    }
                    return getTypeOfPropertyOfContextualType(attributesType, attribute.name.escapedText);
                }
                else {
                    return getContextualType(attribute.parent);
                }
            }
            // Return true if the given expression is possibly a discriminant value. We limit the kinds of
            // expressions we check to those that don't depend on their contextual type in order not to cause
            // recursive (and possibly infinite) invocations of getContextualType.
            function isPossiblyDiscriminantValue(node) {
                switch (node.kind) {
                    case 10 /* StringLiteral */:
                    case 8 /* NumericLiteral */:
                    case 9 /* BigIntLiteral */:
                    case 14 /* NoSubstitutionTemplateLiteral */:
                    case 106 /* TrueKeyword */:
                    case 91 /* FalseKeyword */:
                    case 100 /* NullKeyword */:
                    case 75 /* Identifier */:
                    case 146 /* UndefinedKeyword */:
                        return true;
                    case 194 /* PropertyAccessExpression */:
                    case 200 /* ParenthesizedExpression */:
                        return isPossiblyDiscriminantValue(node.expression);
                    case 276 /* JsxExpression */:
                        return !node.expression || isPossiblyDiscriminantValue(node.expression);
                }
                return false;
            }
            function discriminateContextualTypeByObjectMembers(node, contextualType) {
                return discriminateTypeByDiscriminableItems(contextualType, ts.map(ts.filter(node.properties, function (p) { return !!p.symbol && p.kind === 281 /* PropertyAssignment */ && isPossiblyDiscriminantValue(p.initializer) && isDiscriminantProperty(contextualType, p.symbol.escapedName); }), function (prop) { return [function () { return checkExpression(prop.initializer); }, prop.symbol.escapedName]; }), isTypeAssignableTo, contextualType);
            }
            function discriminateContextualTypeByJSXAttributes(node, contextualType) {
                return discriminateTypeByDiscriminableItems(contextualType, ts.map(ts.filter(node.properties, function (p) { return !!p.symbol && p.kind === 273 /* JsxAttribute */ && isDiscriminantProperty(contextualType, p.symbol.escapedName) && (!p.initializer || isPossiblyDiscriminantValue(p.initializer)); }), function (prop) { return [!prop.initializer ? (function () { return trueType; }) : (function () { return checkExpression(prop.initializer); }), prop.symbol.escapedName]; }), isTypeAssignableTo, contextualType);
            }
            // Return the contextual type for a given expression node. During overload resolution, a contextual type may temporarily
            // be "pushed" onto a node using the contextualType property.
            function getApparentTypeOfContextualType(node, contextFlags) {
                var contextualType = ts.isObjectLiteralMethod(node) ?
                    getContextualTypeForObjectLiteralMethod(node, contextFlags) :
                    getContextualType(node, contextFlags);
                var instantiatedType = instantiateContextualType(contextualType, node, contextFlags);
                if (instantiatedType && !(contextFlags && contextFlags & 2 /* NoConstraints */ && instantiatedType.flags & 8650752 /* TypeVariable */)) {
                    var apparentType = mapType(instantiatedType, getApparentType, /*noReductions*/ true);
                    if (apparentType.flags & 1048576 /* Union */) {
                        if (ts.isObjectLiteralExpression(node)) {
                            return discriminateContextualTypeByObjectMembers(node, apparentType);
                        }
                        else if (ts.isJsxAttributes(node)) {
                            return discriminateContextualTypeByJSXAttributes(node, apparentType);
                        }
                    }
                    return apparentType;
                }
            }
            // If the given contextual type contains instantiable types and if a mapper representing
            // return type inferences is available, instantiate those types using that mapper.
            function instantiateContextualType(contextualType, node, contextFlags) {
                if (contextualType && maybeTypeOfKind(contextualType, 63176704 /* Instantiable */)) {
                    var inferenceContext = getInferenceContext(node);
                    // If no inferences have been made, nothing is gained from instantiating as type parameters
                    // would just be replaced with their defaults similar to the apparent type.
                    if (inferenceContext && ts.some(inferenceContext.inferences, hasInferenceCandidates)) {
                        // For contextual signatures we incorporate all inferences made so far, e.g. from return
                        // types as well as arguments to the left in a function call.
                        if (contextFlags && contextFlags & 1 /* Signature */) {
                            return instantiateInstantiableTypes(contextualType, inferenceContext.nonFixingMapper);
                        }
                        // For other purposes (e.g. determining whether to produce literal types) we only
                        // incorporate inferences made from the return type in a function call.
                        if (inferenceContext.returnMapper) {
                            return instantiateInstantiableTypes(contextualType, inferenceContext.returnMapper);
                        }
                    }
                }
                return contextualType;
            }
            // This function is similar to instantiateType, except that (a) it only instantiates types that
            // are classified as instantiable (i.e. it doesn't instantiate object types), and (b) it performs
            // no reductions on instantiated union types.
            function instantiateInstantiableTypes(type, mapper) {
                if (type.flags & 63176704 /* Instantiable */) {
                    return instantiateType(type, mapper);
                }
                if (type.flags & 1048576 /* Union */) {
                    return getUnionType(ts.map(type.types, function (t) { return instantiateInstantiableTypes(t, mapper); }), 0 /* None */);
                }
                if (type.flags & 2097152 /* Intersection */) {
                    return getIntersectionType(ts.map(type.types, function (t) { return instantiateInstantiableTypes(t, mapper); }));
                }
                return type;
            }
            /**
             * Whoa! Do you really want to use this function?
             *
             * Unless you're trying to get the *non-apparent* type for a
             * value-literal type or you're authoring relevant portions of this algorithm,
             * you probably meant to use 'getApparentTypeOfContextualType'.
             * Otherwise this may not be very useful.
             *
             * In cases where you *are* working on this function, you should understand
             * when it is appropriate to use 'getContextualType' and 'getApparentTypeOfContextualType'.
             *
             *   - Use 'getContextualType' when you are simply going to propagate the result to the expression.
             *   - Use 'getApparentTypeOfContextualType' when you're going to need the members of the type.
             *
             * @param node the expression whose contextual type will be returned.
             * @returns the contextual type of an expression.
             */
            function getContextualType(node, contextFlags) {
                if (node.flags & 16777216 /* InWithStatement */) {
                    // We cannot answer semantic questions within a with block, do not proceed any further
                    return undefined;
                }
                if (node.contextualType) {
                    return node.contextualType;
                }
                var parent = node.parent;
                switch (parent.kind) {
                    case 242 /* VariableDeclaration */:
                    case 156 /* Parameter */:
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                    case 191 /* BindingElement */:
                        return getContextualTypeForInitializerExpression(node);
                    case 202 /* ArrowFunction */:
                    case 235 /* ReturnStatement */:
                        return getContextualTypeForReturnExpression(node);
                    case 212 /* YieldExpression */:
                        return getContextualTypeForYieldOperand(parent);
                    case 206 /* AwaitExpression */:
                        return getContextualTypeForAwaitOperand(parent);
                    case 196 /* CallExpression */:
                        if (parent.expression.kind === 96 /* ImportKeyword */) {
                            return stringType;
                        }
                    /* falls through */
                    case 197 /* NewExpression */:
                        return getContextualTypeForArgument(parent, node);
                    case 199 /* TypeAssertionExpression */:
                    case 217 /* AsExpression */:
                        return ts.isConstTypeReference(parent.type) ? undefined : getTypeFromTypeNode(parent.type);
                    case 209 /* BinaryExpression */:
                        return getContextualTypeForBinaryOperand(node, contextFlags);
                    case 281 /* PropertyAssignment */:
                    case 282 /* ShorthandPropertyAssignment */:
                        return getContextualTypeForObjectLiteralElement(parent, contextFlags);
                    case 283 /* SpreadAssignment */:
                        return getApparentTypeOfContextualType(parent.parent, contextFlags);
                    case 192 /* ArrayLiteralExpression */: {
                        var arrayLiteral = parent;
                        var type = getApparentTypeOfContextualType(arrayLiteral, contextFlags);
                        return getContextualTypeForElementExpression(type, ts.indexOfNode(arrayLiteral.elements, node));
                    }
                    case 210 /* ConditionalExpression */:
                        return getContextualTypeForConditionalOperand(node, contextFlags);
                    case 221 /* TemplateSpan */:
                        ts.Debug.assert(parent.parent.kind === 211 /* TemplateExpression */);
                        return getContextualTypeForSubstitutionExpression(parent.parent, node);
                    case 200 /* ParenthesizedExpression */: {
                        // Like in `checkParenthesizedExpression`, an `/** @type {xyz} */` comment before a parenthesized expression acts as a type cast.
                        var tag = ts.isInJSFile(parent) ? ts.getJSDocTypeTag(parent) : undefined;
                        return tag ? getTypeFromTypeNode(tag.typeExpression.type) : getContextualType(parent, contextFlags);
                    }
                    case 276 /* JsxExpression */:
                        return getContextualTypeForJsxExpression(parent);
                    case 273 /* JsxAttribute */:
                    case 275 /* JsxSpreadAttribute */:
                        return getContextualTypeForJsxAttribute(parent);
                    case 268 /* JsxOpeningElement */:
                    case 267 /* JsxSelfClosingElement */:
                        return getContextualJsxElementAttributesType(parent, contextFlags);
                }
                return undefined;
            }
            function getInferenceContext(node) {
                var ancestor = ts.findAncestor(node, function (n) { return !!n.inferenceContext; });
                return ancestor && ancestor.inferenceContext;
            }
            function getContextualJsxElementAttributesType(node, contextFlags) {
                if (ts.isJsxOpeningElement(node) && node.parent.contextualType && contextFlags !== 4 /* Completions */) {
                    // Contextually applied type is moved from attributes up to the outer jsx attributes so when walking up from the children they get hit
                    // _However_ to hit them from the _attributes_ we must look for them here; otherwise we'll used the declared type
                    // (as below) instead!
                    return node.parent.contextualType;
                }
                return getContextualTypeForArgumentAtIndex(node, 0);
            }
            function getEffectiveFirstArgumentForJsxSignature(signature, node) {
                return getJsxReferenceKind(node) !== 0 /* Component */
                    ? getJsxPropsTypeFromCallSignature(signature, node)
                    : getJsxPropsTypeFromClassType(signature, node);
            }
            function getJsxPropsTypeFromCallSignature(sig, context) {
                var propsType = getTypeOfFirstParameterOfSignatureWithFallback(sig, unknownType);
                propsType = getJsxManagedAttributesFromLocatedAttributes(context, getJsxNamespaceAt(context), propsType);
                var intrinsicAttribs = getJsxType(JsxNames.IntrinsicAttributes, context);
                if (intrinsicAttribs !== errorType) {
                    propsType = intersectTypes(intrinsicAttribs, propsType);
                }
                return propsType;
            }
            function getJsxPropsTypeForSignatureFromMember(sig, forcedLookupLocation) {
                if (sig.unionSignatures) {
                    // JSX Elements using the legacy `props`-field based lookup (eg, react class components) need to treat the `props` member as an input
                    // instead of an output position when resolving the signature. We need to go back to the input signatures of the composite signature,
                    // get the type of `props` on each return type individually, and then _intersect them_, rather than union them (as would normally occur
                    // for a union signature). It's an unfortunate quirk of looking in the output of the signature for the type we want to use for the input.
                    // The default behavior of `getTypeOfFirstParameterOfSignatureWithFallback` when no `props` member name is defined is much more sane.
                    var results = [];
                    for (var _i = 0, _a = sig.unionSignatures; _i < _a.length; _i++) {
                        var signature = _a[_i];
                        var instance = getReturnTypeOfSignature(signature);
                        if (isTypeAny(instance)) {
                            return instance;
                        }
                        var propType = getTypeOfPropertyOfType(instance, forcedLookupLocation);
                        if (!propType) {
                            return;
                        }
                        results.push(propType);
                    }
                    return getIntersectionType(results);
                }
                var instanceType = getReturnTypeOfSignature(sig);
                return isTypeAny(instanceType) ? instanceType : getTypeOfPropertyOfType(instanceType, forcedLookupLocation);
            }
            function getStaticTypeOfReferencedJsxConstructor(context) {
                if (isJsxIntrinsicIdentifier(context.tagName)) {
                    var result = getIntrinsicAttributesTypeFromJsxOpeningLikeElement(context);
                    var fakeSignature = createSignatureForJSXIntrinsic(context, result);
                    return getOrCreateTypeFromSignature(fakeSignature);
                }
                var tagType = checkExpressionCached(context.tagName);
                if (tagType.flags & 128 /* StringLiteral */) {
                    var result = getIntrinsicAttributesTypeFromStringLiteralType(tagType, context);
                    if (!result) {
                        return errorType;
                    }
                    var fakeSignature = createSignatureForJSXIntrinsic(context, result);
                    return getOrCreateTypeFromSignature(fakeSignature);
                }
                return tagType;
            }
            function getJsxManagedAttributesFromLocatedAttributes(context, ns, attributesType) {
                var managedSym = getJsxLibraryManagedAttributes(ns);
                if (managedSym) {
                    var declaredManagedType = getDeclaredTypeOfSymbol(managedSym);
                    var ctorType = getStaticTypeOfReferencedJsxConstructor(context);
                    if (ts.length(declaredManagedType.typeParameters) >= 2) {
                        var args = fillMissingTypeArguments([ctorType, attributesType], declaredManagedType.typeParameters, 2, ts.isInJSFile(context));
                        return createTypeReference(declaredManagedType, args);
                    }
                    else if (ts.length(declaredManagedType.aliasTypeArguments) >= 2) {
                        var args = fillMissingTypeArguments([ctorType, attributesType], declaredManagedType.aliasTypeArguments, 2, ts.isInJSFile(context));
                        return getTypeAliasInstantiation(declaredManagedType.aliasSymbol, args);
                    }
                }
                return attributesType;
            }
            function getJsxPropsTypeFromClassType(sig, context) {
                var ns = getJsxNamespaceAt(context);
                var forcedLookupLocation = getJsxElementPropertiesName(ns);
                var attributesType = forcedLookupLocation === undefined
                    // If there is no type ElementAttributesProperty, return the type of the first parameter of the signature, which should be the props type
                    ? getTypeOfFirstParameterOfSignatureWithFallback(sig, unknownType)
                    : forcedLookupLocation === ""
                        // If there is no e.g. 'props' member in ElementAttributesProperty, use the element class type instead
                        ? getReturnTypeOfSignature(sig)
                        // Otherwise get the type of the property on the signature return type
                        : getJsxPropsTypeForSignatureFromMember(sig, forcedLookupLocation);
                if (!attributesType) {
                    // There is no property named 'props' on this instance type
                    if (!!forcedLookupLocation && !!ts.length(context.attributes.properties)) {
                        error(context, ts.Diagnostics.JSX_element_class_does_not_support_attributes_because_it_does_not_have_a_0_property, ts.unescapeLeadingUnderscores(forcedLookupLocation));
                    }
                    return unknownType;
                }
                attributesType = getJsxManagedAttributesFromLocatedAttributes(context, ns, attributesType);
                if (isTypeAny(attributesType)) {
                    // Props is of type 'any' or unknown
                    return attributesType;
                }
                else {
                    // Normal case -- add in IntrinsicClassElements<T> and IntrinsicElements
                    var apparentAttributesType = attributesType;
                    var intrinsicClassAttribs = getJsxType(JsxNames.IntrinsicClassAttributes, context);
                    if (intrinsicClassAttribs !== errorType) {
                        var typeParams = getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(intrinsicClassAttribs.symbol);
                        var hostClassType = getReturnTypeOfSignature(sig);
                        apparentAttributesType = intersectTypes(typeParams
                            ? createTypeReference(intrinsicClassAttribs, fillMissingTypeArguments([hostClassType], typeParams, getMinTypeArgumentCount(typeParams), ts.isInJSFile(context)))
                            : intrinsicClassAttribs, apparentAttributesType);
                    }
                    var intrinsicAttribs = getJsxType(JsxNames.IntrinsicAttributes, context);
                    if (intrinsicAttribs !== errorType) {
                        apparentAttributesType = intersectTypes(intrinsicAttribs, apparentAttributesType);
                    }
                    return apparentAttributesType;
                }
            }
            // If the given type is an object or union type with a single signature, and if that signature has at
            // least as many parameters as the given function, return the signature. Otherwise return undefined.
            function getContextualCallSignature(type, node) {
                var signatures = getSignaturesOfType(type, 0 /* Call */);
                if (signatures.length === 1) {
                    var signature = signatures[0];
                    if (!isAritySmaller(signature, node)) {
                        return signature;
                    }
                }
            }
            /** If the contextual signature has fewer parameters than the function expression, do not use it */
            function isAritySmaller(signature, target) {
                var targetParameterCount = 0;
                for (; targetParameterCount < target.parameters.length; targetParameterCount++) {
                    var param = target.parameters[targetParameterCount];
                    if (param.initializer || param.questionToken || param.dotDotDotToken || isJSDocOptionalParameter(param)) {
                        break;
                    }
                }
                if (target.parameters.length && ts.parameterIsThisKeyword(target.parameters[0])) {
                    targetParameterCount--;
                }
                return !hasEffectiveRestParameter(signature) && getParameterCount(signature) < targetParameterCount;
            }
            function isFunctionExpressionOrArrowFunction(node) {
                return node.kind === 201 /* FunctionExpression */ || node.kind === 202 /* ArrowFunction */;
            }
            function getContextualSignatureForFunctionLikeDeclaration(node) {
                // Only function expressions, arrow functions, and object literal methods are contextually typed.
                return isFunctionExpressionOrArrowFunction(node) || ts.isObjectLiteralMethod(node)
                    ? getContextualSignature(node)
                    : undefined;
            }
            // Return the contextual signature for a given expression node. A contextual type provides a
            // contextual signature if it has a single call signature and if that call signature is non-generic.
            // If the contextual type is a union type, get the signature from each type possible and if they are
            // all identical ignoring their return type, the result is same signature but with return type as
            // union type of return types from these signatures
            function getContextualSignature(node) {
                ts.Debug.assert(node.kind !== 161 /* MethodDeclaration */ || ts.isObjectLiteralMethod(node));
                var typeTagSignature = getSignatureOfTypeTag(node);
                if (typeTagSignature) {
                    return typeTagSignature;
                }
                var type = getApparentTypeOfContextualType(node, 1 /* Signature */);
                if (!type) {
                    return undefined;
                }
                if (!(type.flags & 1048576 /* Union */)) {
                    return getContextualCallSignature(type, node);
                }
                var signatureList;
                var types = type.types;
                for (var _i = 0, types_16 = types; _i < types_16.length; _i++) {
                    var current = types_16[_i];
                    var signature = getContextualCallSignature(current, node);
                    if (signature) {
                        if (!signatureList) {
                            // This signature will contribute to contextual union signature
                            signatureList = [signature];
                        }
                        else if (!compareSignaturesIdentical(signatureList[0], signature, /*partialMatch*/ false, /*ignoreThisTypes*/ true, /*ignoreReturnTypes*/ true, compareTypesIdentical)) {
                            // Signatures aren't identical, do not use
                            return undefined;
                        }
                        else {
                            // Use this signature for contextual union signature
                            signatureList.push(signature);
                        }
                    }
                }
                // Result is union of signatures collected (return type is union of return types of this signature set)
                if (signatureList) {
                    return signatureList.length === 1 ? signatureList[0] : createUnionSignature(signatureList[0], signatureList);
                }
            }
            function checkSpreadExpression(node, checkMode) {
                if (languageVersion < 2 /* ES2015 */) {
                    checkExternalEmitHelpers(node, compilerOptions.downlevelIteration ? 1536 /* SpreadIncludes */ : 2048 /* SpreadArrays */);
                }
                var arrayOrIterableType = checkExpression(node.expression, checkMode);
                return checkIteratedTypeOrElementType(33 /* Spread */, arrayOrIterableType, undefinedType, node.expression);
            }
            function hasDefaultValue(node) {
                return (node.kind === 191 /* BindingElement */ && !!node.initializer) ||
                    (node.kind === 209 /* BinaryExpression */ && node.operatorToken.kind === 62 /* EqualsToken */);
            }
            function checkArrayLiteral(node, checkMode, forceTuple) {
                var elements = node.elements;
                var elementCount = elements.length;
                var hasNonEndingSpreadElement = false;
                var elementTypes = [];
                var inDestructuringPattern = ts.isAssignmentTarget(node);
                var contextualType = getApparentTypeOfContextualType(node);
                var inConstContext = isConstContext(node);
                for (var index = 0; index < elementCount; index++) {
                    var e = elements[index];
                    if (inDestructuringPattern && e.kind === 213 /* SpreadElement */) {
                        // Given the following situation:
                        //    var c: {};
                        //    [...c] = ["", 0];
                        //
                        // c is represented in the tree as a spread element in an array literal.
                        // But c really functions as a rest element, and its purpose is to provide
                        // a contextual type for the right hand side of the assignment. Therefore,
                        // instead of calling checkExpression on "...c", which will give an error
                        // if c is not iterable/array-like, we need to act as if we are trying to
                        // get the contextual element type from it. So we do something similar to
                        // getContextualTypeForElementExpression, which will crucially not error
                        // if there is no index type / iterated type.
                        var restArrayType = checkExpression(e.expression, checkMode, forceTuple);
                        var restElementType = getIndexTypeOfType(restArrayType, 1 /* Number */) ||
                            getIteratedTypeOrElementType(65 /* Destructuring */, restArrayType, undefinedType, /*errorNode*/ undefined, /*checkAssignability*/ false);
                        if (restElementType) {
                            elementTypes.push(restElementType);
                        }
                    }
                    else {
                        var elementContextualType = getContextualTypeForElementExpression(contextualType, index);
                        var type = checkExpressionForMutableLocation(e, checkMode, elementContextualType, forceTuple);
                        elementTypes.push(type);
                    }
                    if (index < elementCount - 1 && e.kind === 213 /* SpreadElement */) {
                        hasNonEndingSpreadElement = true;
                    }
                }
                if (!hasNonEndingSpreadElement) {
                    var hasRestElement = elementCount > 0 && elements[elementCount - 1].kind === 213 /* SpreadElement */;
                    var minLength = elementCount - (hasRestElement ? 1 : 0);
                    // If array literal is actually a destructuring pattern, mark it as an implied type. We do this such
                    // that we get the same behavior for "var [x, y] = []" and "[x, y] = []".
                    var tupleResult = void 0;
                    if (inDestructuringPattern && minLength > 0) {
                        var type = cloneTypeReference(createTupleType(elementTypes, minLength, hasRestElement));
                        type.pattern = node;
                        return type;
                    }
                    else if (tupleResult = getArrayLiteralTupleTypeIfApplicable(elementTypes, contextualType, hasRestElement, elementCount, inConstContext)) {
                        return createArrayLiteralType(tupleResult);
                    }
                    else if (forceTuple) {
                        return createArrayLiteralType(createTupleType(elementTypes, minLength, hasRestElement));
                    }
                }
                return createArrayLiteralType(createArrayType(elementTypes.length ?
                    getUnionType(elementTypes, 2 /* Subtype */) :
                    strictNullChecks ? implicitNeverType : undefinedWideningType, inConstContext));
            }
            function createArrayLiteralType(type) {
                if (!(ts.getObjectFlags(type) & 4 /* Reference */)) {
                    return type;
                }
                var literalType = type.literalType;
                if (!literalType) {
                    literalType = type.literalType = cloneTypeReference(type);
                    literalType.objectFlags |= 65536 /* ArrayLiteral */ | 1048576 /* ContainsObjectOrArrayLiteral */;
                }
                return literalType;
            }
            function getArrayLiteralTupleTypeIfApplicable(elementTypes, contextualType, hasRestElement, elementCount, readonly) {
                if (elementCount === void 0) {
                    elementCount = elementTypes.length;
                }
                if (readonly === void 0) {
                    readonly = false;
                }
                // Infer a tuple type when the contextual type is or contains a tuple-like type
                if (readonly || (contextualType && forEachType(contextualType, isTupleLikeType))) {
                    return createTupleType(elementTypes, elementCount - (hasRestElement ? 1 : 0), hasRestElement, readonly);
                }
            }
            function isNumericName(name) {
                switch (name.kind) {
                    case 154 /* ComputedPropertyName */:
                        return isNumericComputedName(name);
                    case 75 /* Identifier */:
                        return isNumericLiteralName(name.escapedText);
                    case 8 /* NumericLiteral */:
                    case 10 /* StringLiteral */:
                        return isNumericLiteralName(name.text);
                    default:
                        return false;
                }
            }
            function isNumericComputedName(name) {
                // It seems odd to consider an expression of type Any to result in a numeric name,
                // but this behavior is consistent with checkIndexedAccess
                return isTypeAssignableToKind(checkComputedPropertyName(name), 296 /* NumberLike */);
            }
            function isInfinityOrNaNString(name) {
                return name === "Infinity" || name === "-Infinity" || name === "NaN";
            }
            function isNumericLiteralName(name) {
                // The intent of numeric names is that
                //     - they are names with text in a numeric form, and that
                //     - setting properties/indexing with them is always equivalent to doing so with the numeric literal 'numLit',
                //         acquired by applying the abstract 'ToNumber' operation on the name's text.
                //
                // The subtlety is in the latter portion, as we cannot reliably say that anything that looks like a numeric literal is a numeric name.
                // In fact, it is the case that the text of the name must be equal to 'ToString(numLit)' for this to hold.
                //
                // Consider the property name '"0xF00D"'. When one indexes with '0xF00D', they are actually indexing with the value of 'ToString(0xF00D)'
                // according to the ECMAScript specification, so it is actually as if the user indexed with the string '"61453"'.
                // Thus, the text of all numeric literals equivalent to '61543' such as '0xF00D', '0xf00D', '0170015', etc. are not valid numeric names
                // because their 'ToString' representation is not equal to their original text.
                // This is motivated by ECMA-262 sections 9.3.1, 9.8.1, 11.1.5, and 11.2.1.
                //
                // Here, we test whether 'ToString(ToNumber(name))' is exactly equal to 'name'.
                // The '+' prefix operator is equivalent here to applying the abstract ToNumber operation.
                // Applying the 'toString()' method on a number gives us the abstract ToString operation on a number.
                //
                // Note that this accepts the values 'Infinity', '-Infinity', and 'NaN', and that this is intentional.
                // This is desired behavior, because when indexing with them as numeric entities, you are indexing
                // with the strings '"Infinity"', '"-Infinity"', and '"NaN"' respectively.
                return (+name).toString() === name;
            }
            function checkComputedPropertyName(node) {
                var links = getNodeLinks(node.expression);
                if (!links.resolvedType) {
                    links.resolvedType = checkExpression(node.expression);
                    // This will allow types number, string, symbol or any. It will also allow enums, the unknown
                    // type, and any union of these types (like string | number).
                    if (links.resolvedType.flags & 98304 /* Nullable */ ||
                        !isTypeAssignableToKind(links.resolvedType, 132 /* StringLike */ | 296 /* NumberLike */ | 12288 /* ESSymbolLike */) &&
                            !isTypeAssignableTo(links.resolvedType, stringNumberSymbolType)) {
                        error(node, ts.Diagnostics.A_computed_property_name_must_be_of_type_string_number_symbol_or_any);
                    }
                    else {
                        checkThatExpressionIsProperSymbolReference(node.expression, links.resolvedType, /*reportError*/ true);
                    }
                }
                return links.resolvedType;
            }
            function getObjectLiteralIndexInfo(node, offset, properties, kind) {
                var propTypes = [];
                for (var i = 0; i < properties.length; i++) {
                    if (kind === 0 /* String */ || isNumericName(node.properties[i + offset].name)) {
                        propTypes.push(getTypeOfSymbol(properties[i]));
                    }
                }
                var unionType = propTypes.length ? getUnionType(propTypes, 2 /* Subtype */) : undefinedType;
                return createIndexInfo(unionType, isConstContext(node));
            }
            function getImmediateAliasedSymbol(symbol) {
                ts.Debug.assert((symbol.flags & 2097152 /* Alias */) !== 0, "Should only get Alias here.");
                var links = getSymbolLinks(symbol);
                if (!links.immediateTarget) {
                    var node = getDeclarationOfAliasSymbol(symbol);
                    if (!node)
                        return ts.Debug.fail();
                    links.immediateTarget = getTargetOfAliasDeclaration(node, /*dontRecursivelyResolve*/ true);
                }
                return links.immediateTarget;
            }
            function checkObjectLiteral(node, checkMode) {
                var inDestructuringPattern = ts.isAssignmentTarget(node);
                // Grammar checking
                checkGrammarObjectLiteralExpression(node, inDestructuringPattern);
                var propertiesTable;
                var propertiesArray = [];
                var spread = emptyObjectType;
                var contextualType = getApparentTypeOfContextualType(node);
                var contextualTypeHasPattern = contextualType && contextualType.pattern &&
                    (contextualType.pattern.kind === 189 /* ObjectBindingPattern */ || contextualType.pattern.kind === 193 /* ObjectLiteralExpression */);
                var inConstContext = isConstContext(node);
                var checkFlags = inConstContext ? 8 /* Readonly */ : 0;
                var isInJavascript = ts.isInJSFile(node) && !ts.isInJsonFile(node);
                var enumTag = ts.getJSDocEnumTag(node);
                var isJSObjectLiteral = !contextualType && isInJavascript && !enumTag;
                var objectFlags = freshObjectLiteralFlag;
                var patternWithComputedProperties = false;
                var hasComputedStringProperty = false;
                var hasComputedNumberProperty = false;
                propertiesTable = ts.createSymbolTable();
                var offset = 0;
                for (var i = 0; i < node.properties.length; i++) {
                    var memberDecl = node.properties[i];
                    var member = getSymbolOfNode(memberDecl);
                    var computedNameType = memberDecl.name && memberDecl.name.kind === 154 /* ComputedPropertyName */ && !ts.isWellKnownSymbolSyntactically(memberDecl.name.expression) ?
                        checkComputedPropertyName(memberDecl.name) : undefined;
                    if (memberDecl.kind === 281 /* PropertyAssignment */ ||
                        memberDecl.kind === 282 /* ShorthandPropertyAssignment */ ||
                        ts.isObjectLiteralMethod(memberDecl)) {
                        var type = memberDecl.kind === 281 /* PropertyAssignment */ ? checkPropertyAssignment(memberDecl, checkMode) :
                            memberDecl.kind === 282 /* ShorthandPropertyAssignment */ ? checkExpressionForMutableLocation(memberDecl.name, checkMode) :
                                checkObjectLiteralMethod(memberDecl, checkMode);
                        if (isInJavascript) {
                            var jsDocType = getTypeForDeclarationFromJSDocComment(memberDecl);
                            if (jsDocType) {
                                checkTypeAssignableTo(type, jsDocType, memberDecl);
                                type = jsDocType;
                            }
                            else if (enumTag && enumTag.typeExpression) {
                                checkTypeAssignableTo(type, getTypeFromTypeNode(enumTag.typeExpression), memberDecl);
                            }
                        }
                        objectFlags |= ts.getObjectFlags(type) & 3670016 /* PropagatingFlags */;
                        var nameType = computedNameType && isTypeUsableAsPropertyName(computedNameType) ? computedNameType : undefined;
                        var prop = nameType ?
                            createSymbol(4 /* Property */ | member.flags, getPropertyNameFromType(nameType), checkFlags | 4096 /* Late */) :
                            createSymbol(4 /* Property */ | member.flags, member.escapedName, checkFlags);
                        if (nameType) {
                            prop.nameType = nameType;
                        }
                        if (inDestructuringPattern) {
                            // If object literal is an assignment pattern and if the assignment pattern specifies a default value
                            // for the property, make the property optional.
                            var isOptional = (memberDecl.kind === 281 /* PropertyAssignment */ && hasDefaultValue(memberDecl.initializer)) ||
                                (memberDecl.kind === 282 /* ShorthandPropertyAssignment */ && memberDecl.objectAssignmentInitializer);
                            if (isOptional) {
                                prop.flags |= 16777216 /* Optional */;
                            }
                        }
                        else if (contextualTypeHasPattern && !(ts.getObjectFlags(contextualType) & 512 /* ObjectLiteralPatternWithComputedProperties */)) {
                            // If object literal is contextually typed by the implied type of a binding pattern, and if the
                            // binding pattern specifies a default value for the property, make the property optional.
                            var impliedProp = getPropertyOfType(contextualType, member.escapedName);
                            if (impliedProp) {
                                prop.flags |= impliedProp.flags & 16777216 /* Optional */;
                            }
                            else if (!compilerOptions.suppressExcessPropertyErrors && !getIndexInfoOfType(contextualType, 0 /* String */)) {
                                error(memberDecl.name, ts.Diagnostics.Object_literal_may_only_specify_known_properties_and_0_does_not_exist_in_type_1, symbolToString(member), typeToString(contextualType));
                            }
                        }
                        prop.declarations = member.declarations;
                        prop.parent = member.parent;
                        if (member.valueDeclaration) {
                            prop.valueDeclaration = member.valueDeclaration;
                        }
                        prop.type = type;
                        prop.target = member;
                        member = prop;
                    }
                    else if (memberDecl.kind === 283 /* SpreadAssignment */) {
                        if (languageVersion < 2 /* ES2015 */) {
                            checkExternalEmitHelpers(memberDecl, 2 /* Assign */);
                        }
                        if (propertiesArray.length > 0) {
                            spread = getSpreadType(spread, createObjectLiteralType(), node.symbol, objectFlags, inConstContext);
                            propertiesArray = [];
                            propertiesTable = ts.createSymbolTable();
                            hasComputedStringProperty = false;
                            hasComputedNumberProperty = false;
                        }
                        var type = checkExpression(memberDecl.expression);
                        if (!isValidSpreadType(type)) {
                            error(memberDecl, ts.Diagnostics.Spread_types_may_only_be_created_from_object_types);
                            return errorType;
                        }
                        spread = getSpreadType(spread, type, node.symbol, objectFlags, inConstContext);
                        offset = i + 1;
                        continue;
                    }
                    else {
                        // TypeScript 1.0 spec (April 2014)
                        // A get accessor declaration is processed in the same manner as
                        // an ordinary function declaration(section 6.1) with no parameters.
                        // A set accessor declaration is processed in the same manner
                        // as an ordinary function declaration with a single parameter and a Void return type.
                        ts.Debug.assert(memberDecl.kind === 163 /* GetAccessor */ || memberDecl.kind === 164 /* SetAccessor */);
                        checkNodeDeferred(memberDecl);
                    }
                    if (computedNameType && !(computedNameType.flags & 8576 /* StringOrNumberLiteralOrUnique */)) {
                        if (isTypeAssignableTo(computedNameType, stringNumberSymbolType)) {
                            if (isTypeAssignableTo(computedNameType, numberType)) {
                                hasComputedNumberProperty = true;
                            }
                            else {
                                hasComputedStringProperty = true;
                            }
                            if (inDestructuringPattern) {
                                patternWithComputedProperties = true;
                            }
                        }
                    }
                    else {
                        propertiesTable.set(member.escapedName, member);
                    }
                    propertiesArray.push(member);
                }
                // If object literal is contextually typed by the implied type of a binding pattern, augment the result
                // type with those properties for which the binding pattern specifies a default value.
                if (contextualTypeHasPattern) {
                    for (var _i = 0, _a = getPropertiesOfType(contextualType); _i < _a.length; _i++) {
                        var prop = _a[_i];
                        if (!propertiesTable.get(prop.escapedName) && !(spread && getPropertyOfType(spread, prop.escapedName))) {
                            if (!(prop.flags & 16777216 /* Optional */)) {
                                error(prop.valueDeclaration || prop.bindingElement, ts.Diagnostics.Initializer_provides_no_value_for_this_binding_element_and_the_binding_element_has_no_default_value);
                            }
                            propertiesTable.set(prop.escapedName, prop);
                            propertiesArray.push(prop);
                        }
                    }
                }
                if (spread !== emptyObjectType) {
                    if (propertiesArray.length > 0) {
                        spread = getSpreadType(spread, createObjectLiteralType(), node.symbol, objectFlags, inConstContext);
                        propertiesArray = [];
                        propertiesTable = ts.createSymbolTable();
                        hasComputedStringProperty = false;
                        hasComputedNumberProperty = false;
                    }
                    // remap the raw emptyObjectType fed in at the top into a fresh empty object literal type, unique to this use site
                    return mapType(spread, function (t) { return t === emptyObjectType ? createObjectLiteralType() : t; });
                }
                return createObjectLiteralType();
                function createObjectLiteralType() {
                    var stringIndexInfo = hasComputedStringProperty ? getObjectLiteralIndexInfo(node, offset, propertiesArray, 0 /* String */) : undefined;
                    var numberIndexInfo = hasComputedNumberProperty ? getObjectLiteralIndexInfo(node, offset, propertiesArray, 1 /* Number */) : undefined;
                    var result = createAnonymousType(node.symbol, propertiesTable, ts.emptyArray, ts.emptyArray, stringIndexInfo, numberIndexInfo);
                    result.objectFlags |= objectFlags | 128 /* ObjectLiteral */ | 1048576 /* ContainsObjectOrArrayLiteral */;
                    if (isJSObjectLiteral) {
                        result.objectFlags |= 16384 /* JSLiteral */;
                    }
                    if (patternWithComputedProperties) {
                        result.objectFlags |= 512 /* ObjectLiteralPatternWithComputedProperties */;
                    }
                    if (inDestructuringPattern) {
                        result.pattern = node;
                    }
                    return result;
                }
            }
            function isValidSpreadType(type) {
                if (type.flags & 63176704 /* Instantiable */) {
                    var constraint = getBaseConstraintOfType(type);
                    if (constraint !== undefined) {
                        return isValidSpreadType(constraint);
                    }
                }
                return !!(type.flags & (1 /* Any */ | 67108864 /* NonPrimitive */ | 524288 /* Object */ | 58982400 /* InstantiableNonPrimitive */) ||
                    getFalsyFlags(type) & 117632 /* DefinitelyFalsy */ && isValidSpreadType(removeDefinitelyFalsyTypes(type)) ||
                    type.flags & 3145728 /* UnionOrIntersection */ && ts.every(type.types, isValidSpreadType));
            }
            function checkJsxSelfClosingElementDeferred(node) {
                checkJsxOpeningLikeElementOrOpeningFragment(node);
            }
            function checkJsxSelfClosingElement(node, _checkMode) {
                checkNodeDeferred(node);
                return getJsxElementTypeAt(node) || anyType;
            }
            function checkJsxElementDeferred(node) {
                // Check attributes
                checkJsxOpeningLikeElementOrOpeningFragment(node.openingElement);
                // Perform resolution on the closing tag so that rename/go to definition/etc work
                if (isJsxIntrinsicIdentifier(node.closingElement.tagName)) {
                    getIntrinsicTagSymbol(node.closingElement);
                }
                else {
                    checkExpression(node.closingElement.tagName);
                }
                checkJsxChildren(node);
            }
            function checkJsxElement(node, _checkMode) {
                checkNodeDeferred(node);
                return getJsxElementTypeAt(node) || anyType;
            }
            function checkJsxFragment(node) {
                checkJsxOpeningLikeElementOrOpeningFragment(node.openingFragment);
                if (compilerOptions.jsx === 2 /* React */ && (compilerOptions.jsxFactory || ts.getSourceFileOfNode(node).pragmas.has("jsx"))) {
                    error(node, compilerOptions.jsxFactory
                        ? ts.Diagnostics.JSX_fragment_is_not_supported_when_using_jsxFactory
                        : ts.Diagnostics.JSX_fragment_is_not_supported_when_using_an_inline_JSX_factory_pragma);
                }
                checkJsxChildren(node);
                return getJsxElementTypeAt(node) || anyType;
            }
            /**
             * Returns true iff the JSX element name would be a valid JS identifier, ignoring restrictions about keywords not being identifiers
             */
            function isUnhyphenatedJsxName(name) {
                // - is the only character supported in JSX attribute names that isn't valid in JavaScript identifiers
                return !ts.stringContains(name, "-");
            }
            /**
             * Returns true iff React would emit this tag name as a string rather than an identifier or qualified name
             */
            function isJsxIntrinsicIdentifier(tagName) {
                return tagName.kind === 75 /* Identifier */ && ts.isIntrinsicJsxName(tagName.escapedText);
            }
            function checkJsxAttribute(node, checkMode) {
                return node.initializer
                    ? checkExpressionForMutableLocation(node.initializer, checkMode)
                    : trueType; // <Elem attr /> is sugar for <Elem attr={true} />
            }
            /**
             * Get attributes type of the JSX opening-like element. The result is from resolving "attributes" property of the opening-like element.
             *
             * @param openingLikeElement a JSX opening-like element
             * @param filter a function to remove attributes that will not participate in checking whether attributes are assignable
             * @return an anonymous type (similar to the one returned by checkObjectLiteral) in which its properties are attributes property.
             * @remarks Because this function calls getSpreadType, it needs to use the same checks as checkObjectLiteral,
             * which also calls getSpreadType.
             */
            function createJsxAttributesTypeFromAttributesProperty(openingLikeElement, checkMode) {
                var attributes = openingLikeElement.attributes;
                var attributesTable = ts.createSymbolTable();
                var spread = emptyJsxObjectType;
                var hasSpreadAnyType = false;
                var typeToIntersect;
                var explicitlySpecifyChildrenAttribute = false;
                var objectFlags = 4096 /* JsxAttributes */;
                var jsxChildrenPropertyName = getJsxElementChildrenPropertyName(getJsxNamespaceAt(openingLikeElement));
                for (var _i = 0, _a = attributes.properties; _i < _a.length; _i++) {
                    var attributeDecl = _a[_i];
                    var member = attributeDecl.symbol;
                    if (ts.isJsxAttribute(attributeDecl)) {
                        var exprType = checkJsxAttribute(attributeDecl, checkMode);
                        objectFlags |= ts.getObjectFlags(exprType) & 3670016 /* PropagatingFlags */;
                        var attributeSymbol = createSymbol(4 /* Property */ | 33554432 /* Transient */ | member.flags, member.escapedName);
                        attributeSymbol.declarations = member.declarations;
                        attributeSymbol.parent = member.parent;
                        if (member.valueDeclaration) {
                            attributeSymbol.valueDeclaration = member.valueDeclaration;
                        }
                        attributeSymbol.type = exprType;
                        attributeSymbol.target = member;
                        attributesTable.set(attributeSymbol.escapedName, attributeSymbol);
                        if (attributeDecl.name.escapedText === jsxChildrenPropertyName) {
                            explicitlySpecifyChildrenAttribute = true;
                        }
                    }
                    else {
                        ts.Debug.assert(attributeDecl.kind === 275 /* JsxSpreadAttribute */);
                        if (attributesTable.size > 0) {
                            spread = getSpreadType(spread, createJsxAttributesType(), attributes.symbol, objectFlags, /*readonly*/ false);
                            attributesTable = ts.createSymbolTable();
                        }
                        var exprType = checkExpressionCached(attributeDecl.expression, checkMode);
                        if (isTypeAny(exprType)) {
                            hasSpreadAnyType = true;
                        }
                        if (isValidSpreadType(exprType)) {
                            spread = getSpreadType(spread, exprType, attributes.symbol, objectFlags, /*readonly*/ false);
                        }
                        else {
                            typeToIntersect = typeToIntersect ? getIntersectionType([typeToIntersect, exprType]) : exprType;
                        }
                    }
                }
                if (!hasSpreadAnyType) {
                    if (attributesTable.size > 0) {
                        spread = getSpreadType(spread, createJsxAttributesType(), attributes.symbol, objectFlags, /*readonly*/ false);
                    }
                }
                // Handle children attribute
                var parent = openingLikeElement.parent.kind === 266 /* JsxElement */ ? openingLikeElement.parent : undefined;
                // We have to check that openingElement of the parent is the one we are visiting as this may not be true for selfClosingElement
                if (parent && parent.openingElement === openingLikeElement && parent.children.length > 0) {
                    var childrenTypes = checkJsxChildren(parent, checkMode);
                    if (!hasSpreadAnyType && jsxChildrenPropertyName && jsxChildrenPropertyName !== "") {
                        // Error if there is a attribute named "children" explicitly specified and children element.
                        // This is because children element will overwrite the value from attributes.
                        // Note: we will not warn "children" attribute overwritten if "children" attribute is specified in object spread.
                        if (explicitlySpecifyChildrenAttribute) {
                            error(attributes, ts.Diagnostics._0_are_specified_twice_The_attribute_named_0_will_be_overwritten, ts.unescapeLeadingUnderscores(jsxChildrenPropertyName));
                        }
                        var contextualType = getApparentTypeOfContextualType(openingLikeElement.attributes);
                        var childrenContextualType = contextualType && getTypeOfPropertyOfContextualType(contextualType, jsxChildrenPropertyName);
                        // If there are children in the body of JSX element, create dummy attribute "children" with the union of children types so that it will pass the attribute checking process
                        var childrenPropSymbol = createSymbol(4 /* Property */ | 33554432 /* Transient */, jsxChildrenPropertyName);
                        childrenPropSymbol.type = childrenTypes.length === 1 ?
                            childrenTypes[0] :
                            (getArrayLiteralTupleTypeIfApplicable(childrenTypes, childrenContextualType, /*hasRestElement*/ false) || createArrayType(getUnionType(childrenTypes)));
                        // Fake up a property declaration for the children
                        childrenPropSymbol.valueDeclaration = ts.createPropertySignature(/*modifiers*/ undefined, ts.unescapeLeadingUnderscores(jsxChildrenPropertyName), /*questionToken*/ undefined, /*type*/ undefined, /*initializer*/ undefined);
                        childrenPropSymbol.valueDeclaration.parent = attributes;
                        childrenPropSymbol.valueDeclaration.symbol = childrenPropSymbol;
                        var childPropMap = ts.createSymbolTable();
                        childPropMap.set(jsxChildrenPropertyName, childrenPropSymbol);
                        spread = getSpreadType(spread, createAnonymousType(attributes.symbol, childPropMap, ts.emptyArray, ts.emptyArray, /*stringIndexInfo*/ undefined, /*numberIndexInfo*/ undefined), attributes.symbol, objectFlags, /*readonly*/ false);
                    }
                }
                if (hasSpreadAnyType) {
                    return anyType;
                }
                if (typeToIntersect && spread !== emptyJsxObjectType) {
                    return getIntersectionType([typeToIntersect, spread]);
                }
                return typeToIntersect || (spread === emptyJsxObjectType ? createJsxAttributesType() : spread);
                /**
                 * Create anonymous type from given attributes symbol table.
                 * @param symbol a symbol of JsxAttributes containing attributes corresponding to attributesTable
                 * @param attributesTable a symbol table of attributes property
                 */
                function createJsxAttributesType() {
                    objectFlags |= freshObjectLiteralFlag;
                    var result = createAnonymousType(attributes.symbol, attributesTable, ts.emptyArray, ts.emptyArray, /*stringIndexInfo*/ undefined, /*numberIndexInfo*/ undefined);
                    result.objectFlags |= objectFlags | 128 /* ObjectLiteral */ | 1048576 /* ContainsObjectOrArrayLiteral */;
                    return result;
                }
            }
            function checkJsxChildren(node, checkMode) {
                var childrenTypes = [];
                for (var _i = 0, _a = node.children; _i < _a.length; _i++) {
                    var child = _a[_i];
                    // In React, JSX text that contains only whitespaces will be ignored so we don't want to type-check that
                    // because then type of children property will have constituent of string type.
                    if (child.kind === 11 /* JsxText */) {
                        if (!child.containsOnlyTriviaWhiteSpaces) {
                            childrenTypes.push(stringType);
                        }
                    }
                    else {
                        childrenTypes.push(checkExpressionForMutableLocation(child, checkMode));
                    }
                }
                return childrenTypes;
            }
            /**
             * Check attributes property of opening-like element. This function is called during chooseOverload to get call signature of a JSX opening-like element.
             * (See "checkApplicableSignatureForJsxOpeningLikeElement" for how the function is used)
             * @param node a JSXAttributes to be resolved of its type
             */
            function checkJsxAttributes(node, checkMode) {
                return createJsxAttributesTypeFromAttributesProperty(node.parent, checkMode);
            }
            function getJsxType(name, location) {
                var namespace = getJsxNamespaceAt(location);
                var exports = namespace && getExportsOfSymbol(namespace);
                var typeSymbol = exports && getSymbol(exports, name, 788968 /* Type */);
                return typeSymbol ? getDeclaredTypeOfSymbol(typeSymbol) : errorType;
            }
            /**
             * Looks up an intrinsic tag name and returns a symbol that either points to an intrinsic
             * property (in which case nodeLinks.jsxFlags will be IntrinsicNamedElement) or an intrinsic
             * string index signature (in which case nodeLinks.jsxFlags will be IntrinsicIndexedElement).
             * May also return unknownSymbol if both of these lookups fail.
             */
            function getIntrinsicTagSymbol(node) {
                var links = getNodeLinks(node);
                if (!links.resolvedSymbol) {
                    var intrinsicElementsType = getJsxType(JsxNames.IntrinsicElements, node);
                    if (intrinsicElementsType !== errorType) {
                        // Property case
                        if (!ts.isIdentifier(node.tagName))
                            return ts.Debug.fail();
                        var intrinsicProp = getPropertyOfType(intrinsicElementsType, node.tagName.escapedText);
                        if (intrinsicProp) {
                            links.jsxFlags |= 1 /* IntrinsicNamedElement */;
                            return links.resolvedSymbol = intrinsicProp;
                        }
                        // Intrinsic string indexer case
                        var indexSignatureType = getIndexTypeOfType(intrinsicElementsType, 0 /* String */);
                        if (indexSignatureType) {
                            links.jsxFlags |= 2 /* IntrinsicIndexedElement */;
                            return links.resolvedSymbol = intrinsicElementsType.symbol;
                        }
                        // Wasn't found
                        error(node, ts.Diagnostics.Property_0_does_not_exist_on_type_1, ts.idText(node.tagName), "JSX." + JsxNames.IntrinsicElements);
                        return links.resolvedSymbol = unknownSymbol;
                    }
                    else {
                        if (noImplicitAny) {
                            error(node, ts.Diagnostics.JSX_element_implicitly_has_type_any_because_no_interface_JSX_0_exists, ts.unescapeLeadingUnderscores(JsxNames.IntrinsicElements));
                        }
                        return links.resolvedSymbol = unknownSymbol;
                    }
                }
                return links.resolvedSymbol;
            }
            function getJsxNamespaceAt(location) {
                var links = location && getNodeLinks(location);
                if (links && links.jsxNamespace) {
                    return links.jsxNamespace;
                }
                if (!links || links.jsxNamespace !== false) {
                    var namespaceName = getJsxNamespace(location);
                    var resolvedNamespace = resolveName(location, namespaceName, 1920 /* Namespace */, /*diagnosticMessage*/ undefined, namespaceName, /*isUse*/ false);
                    if (resolvedNamespace) {
                        var candidate = resolveSymbol(getSymbol(getExportsOfSymbol(resolveSymbol(resolvedNamespace)), JsxNames.JSX, 1920 /* Namespace */));
                        if (candidate) {
                            if (links) {
                                links.jsxNamespace = candidate;
                            }
                            return candidate;
                        }
                        if (links) {
                            links.jsxNamespace = false;
                        }
                    }
                }
                // JSX global fallback
                return getGlobalSymbol(JsxNames.JSX, 1920 /* Namespace */, /*diagnosticMessage*/ undefined); // TODO: GH#18217
            }
            /**
             * Look into JSX namespace and then look for container with matching name as nameOfAttribPropContainer.
             * Get a single property from that container if existed. Report an error if there are more than one property.
             *
             * @param nameOfAttribPropContainer a string of value JsxNames.ElementAttributesPropertyNameContainer or JsxNames.ElementChildrenAttributeNameContainer
             *          if other string is given or the container doesn't exist, return undefined.
             */
            function getNameFromJsxElementAttributesContainer(nameOfAttribPropContainer, jsxNamespace) {
                // JSX.ElementAttributesProperty | JSX.ElementChildrenAttribute [symbol]
                var jsxElementAttribPropInterfaceSym = jsxNamespace && getSymbol(jsxNamespace.exports, nameOfAttribPropContainer, 788968 /* Type */);
                // JSX.ElementAttributesProperty | JSX.ElementChildrenAttribute [type]
                var jsxElementAttribPropInterfaceType = jsxElementAttribPropInterfaceSym && getDeclaredTypeOfSymbol(jsxElementAttribPropInterfaceSym);
                // The properties of JSX.ElementAttributesProperty | JSX.ElementChildrenAttribute
                var propertiesOfJsxElementAttribPropInterface = jsxElementAttribPropInterfaceType && getPropertiesOfType(jsxElementAttribPropInterfaceType);
                if (propertiesOfJsxElementAttribPropInterface) {
                    // Element Attributes has zero properties, so the element attributes type will be the class instance type
                    if (propertiesOfJsxElementAttribPropInterface.length === 0) {
                        return "";
                    }
                    // Element Attributes has one property, so the element attributes type will be the type of the corresponding
                    // property of the class instance type
                    else if (propertiesOfJsxElementAttribPropInterface.length === 1) {
                        return propertiesOfJsxElementAttribPropInterface[0].escapedName;
                    }
                    else if (propertiesOfJsxElementAttribPropInterface.length > 1) {
                        // More than one property on ElementAttributesProperty is an error
                        error(jsxElementAttribPropInterfaceSym.declarations[0], ts.Diagnostics.The_global_type_JSX_0_may_not_have_more_than_one_property, ts.unescapeLeadingUnderscores(nameOfAttribPropContainer));
                    }
                }
                return undefined;
            }
            function getJsxLibraryManagedAttributes(jsxNamespace) {
                // JSX.LibraryManagedAttributes [symbol]
                return jsxNamespace && getSymbol(jsxNamespace.exports, JsxNames.LibraryManagedAttributes, 788968 /* Type */);
            }
            /// e.g. "props" for React.d.ts,
            /// or 'undefined' if ElementAttributesProperty doesn't exist (which means all
            ///     non-intrinsic elements' attributes type is 'any'),
            /// or '' if it has 0 properties (which means every
            ///     non-intrinsic elements' attributes type is the element instance type)
            function getJsxElementPropertiesName(jsxNamespace) {
                return getNameFromJsxElementAttributesContainer(JsxNames.ElementAttributesPropertyNameContainer, jsxNamespace);
            }
            function getJsxElementChildrenPropertyName(jsxNamespace) {
                return getNameFromJsxElementAttributesContainer(JsxNames.ElementChildrenAttributeNameContainer, jsxNamespace);
            }
            function getUninstantiatedJsxSignaturesOfType(elementType, caller) {
                if (elementType.flags & 4 /* String */) {
                    return [anySignature];
                }
                else if (elementType.flags & 128 /* StringLiteral */) {
                    var intrinsicType = getIntrinsicAttributesTypeFromStringLiteralType(elementType, caller);
                    if (!intrinsicType) {
                        error(caller, ts.Diagnostics.Property_0_does_not_exist_on_type_1, elementType.value, "JSX." + JsxNames.IntrinsicElements);
                        return ts.emptyArray;
                    }
                    else {
                        var fakeSignature = createSignatureForJSXIntrinsic(caller, intrinsicType);
                        return [fakeSignature];
                    }
                }
                var apparentElemType = getApparentType(elementType);
                // Resolve the signatures, preferring constructor
                var signatures = getSignaturesOfType(apparentElemType, 1 /* Construct */);
                if (signatures.length === 0) {
                    // No construct signatures, try call signatures
                    signatures = getSignaturesOfType(apparentElemType, 0 /* Call */);
                }
                if (signatures.length === 0 && apparentElemType.flags & 1048576 /* Union */) {
                    // If each member has some combination of new/call signatures; make a union signature list for those
                    signatures = getUnionSignatures(ts.map(apparentElemType.types, function (t) { return getUninstantiatedJsxSignaturesOfType(t, caller); }));
                }
                return signatures;
            }
            function getIntrinsicAttributesTypeFromStringLiteralType(type, location) {
                // If the elemType is a stringLiteral type, we can then provide a check to make sure that the string literal type is one of the Jsx intrinsic element type
                // For example:
                //      var CustomTag: "h1" = "h1";
                //      <CustomTag> Hello World </CustomTag>
                var intrinsicElementsType = getJsxType(JsxNames.IntrinsicElements, location);
                if (intrinsicElementsType !== errorType) {
                    var stringLiteralTypeName = type.value;
                    var intrinsicProp = getPropertyOfType(intrinsicElementsType, ts.escapeLeadingUnderscores(stringLiteralTypeName));
                    if (intrinsicProp) {
                        return getTypeOfSymbol(intrinsicProp);
                    }
                    var indexSignatureType = getIndexTypeOfType(intrinsicElementsType, 0 /* String */);
                    if (indexSignatureType) {
                        return indexSignatureType;
                    }
                    return undefined;
                }
                // If we need to report an error, we already done so here. So just return any to prevent any more error downstream
                return anyType;
            }
            function checkJsxReturnAssignableToAppropriateBound(refKind, elemInstanceType, openingLikeElement) {
                if (refKind === 1 /* Function */) {
                    var sfcReturnConstraint = getJsxStatelessElementTypeAt(openingLikeElement);
                    if (sfcReturnConstraint) {
                        checkTypeRelatedTo(elemInstanceType, sfcReturnConstraint, assignableRelation, openingLikeElement, ts.Diagnostics.JSX_element_type_0_is_not_a_constructor_function_for_JSX_elements);
                    }
                }
                else if (refKind === 0 /* Component */) {
                    var classConstraint = getJsxElementClassTypeAt(openingLikeElement);
                    if (classConstraint) {
                        // Issue an error if this return type isn't assignable to JSX.ElementClass or JSX.Element, failing that
                        checkTypeRelatedTo(elemInstanceType, classConstraint, assignableRelation, openingLikeElement, ts.Diagnostics.JSX_element_type_0_is_not_a_constructor_function_for_JSX_elements);
                    }
                }
                else { // Mixed
                    var sfcReturnConstraint = getJsxStatelessElementTypeAt(openingLikeElement);
                    var classConstraint = getJsxElementClassTypeAt(openingLikeElement);
                    if (!sfcReturnConstraint || !classConstraint) {
                        return;
                    }
                    var combined = getUnionType([sfcReturnConstraint, classConstraint]);
                    checkTypeRelatedTo(elemInstanceType, combined, assignableRelation, openingLikeElement, ts.Diagnostics.JSX_element_type_0_is_not_a_constructor_function_for_JSX_elements);
                }
            }
            /**
             * Get attributes type of the given intrinsic opening-like Jsx element by resolving the tag name.
             * The function is intended to be called from a function which has checked that the opening element is an intrinsic element.
             * @param node an intrinsic JSX opening-like element
             */
            function getIntrinsicAttributesTypeFromJsxOpeningLikeElement(node) {
                ts.Debug.assert(isJsxIntrinsicIdentifier(node.tagName));
                var links = getNodeLinks(node);
                if (!links.resolvedJsxElementAttributesType) {
                    var symbol = getIntrinsicTagSymbol(node);
                    if (links.jsxFlags & 1 /* IntrinsicNamedElement */) {
                        return links.resolvedJsxElementAttributesType = getTypeOfSymbol(symbol);
                    }
                    else if (links.jsxFlags & 2 /* IntrinsicIndexedElement */) {
                        return links.resolvedJsxElementAttributesType =
                            getIndexTypeOfType(getDeclaredTypeOfSymbol(symbol), 0 /* String */);
                    }
                    else {
                        return links.resolvedJsxElementAttributesType = errorType;
                    }
                }
                return links.resolvedJsxElementAttributesType;
            }
            function getJsxElementClassTypeAt(location) {
                var type = getJsxType(JsxNames.ElementClass, location);
                if (type === errorType)
                    return undefined;
                return type;
            }
            function getJsxElementTypeAt(location) {
                return getJsxType(JsxNames.Element, location);
            }
            function getJsxStatelessElementTypeAt(location) {
                var jsxElementType = getJsxElementTypeAt(location);
                if (jsxElementType) {
                    return getUnionType([jsxElementType, nullType]);
                }
            }
            /**
             * Returns all the properties of the Jsx.IntrinsicElements interface
             */
            function getJsxIntrinsicTagNamesAt(location) {
                var intrinsics = getJsxType(JsxNames.IntrinsicElements, location);
                return intrinsics ? getPropertiesOfType(intrinsics) : ts.emptyArray;
            }
            function checkJsxPreconditions(errorNode) {
                // Preconditions for using JSX
                if ((compilerOptions.jsx || 0 /* None */) === 0 /* None */) {
                    error(errorNode, ts.Diagnostics.Cannot_use_JSX_unless_the_jsx_flag_is_provided);
                }
                if (getJsxElementTypeAt(errorNode) === undefined) {
                    if (noImplicitAny) {
                        error(errorNode, ts.Diagnostics.JSX_element_implicitly_has_type_any_because_the_global_type_JSX_Element_does_not_exist);
                    }
                }
            }
            function checkJsxOpeningLikeElementOrOpeningFragment(node) {
                var isNodeOpeningLikeElement = ts.isJsxOpeningLikeElement(node);
                if (isNodeOpeningLikeElement) {
                    checkGrammarJsxElement(node);
                }
                checkJsxPreconditions(node);
                // The reactNamespace/jsxFactory's root symbol should be marked as 'used' so we don't incorrectly elide its import.
                // And if there is no reactNamespace/jsxFactory's symbol in scope when targeting React emit, we should issue an error.
                var reactRefErr = diagnostics && compilerOptions.jsx === 2 /* React */ ? ts.Diagnostics.Cannot_find_name_0 : undefined;
                var reactNamespace = getJsxNamespace(node);
                var reactLocation = isNodeOpeningLikeElement ? node.tagName : node;
                var reactSym = resolveName(reactLocation, reactNamespace, 111551 /* Value */, reactRefErr, reactNamespace, /*isUse*/ true);
                if (reactSym) {
                    // Mark local symbol as referenced here because it might not have been marked
                    // if jsx emit was not react as there wont be error being emitted
                    reactSym.isReferenced = 67108863 /* All */;
                    // If react symbol is alias, mark it as refereced
                    if (reactSym.flags & 2097152 /* Alias */ && !getTypeOnlyAliasDeclaration(reactSym)) {
                        markAliasSymbolAsReferenced(reactSym);
                    }
                }
                if (isNodeOpeningLikeElement) {
                    var sig = getResolvedSignature(node);
                    checkJsxReturnAssignableToAppropriateBound(getJsxReferenceKind(node), getReturnTypeOfSignature(sig), node);
                }
            }
            /**
             * Check if a property with the given name is known anywhere in the given type. In an object type, a property
             * is considered known if
             * 1. the object type is empty and the check is for assignability, or
             * 2. if the object type has index signatures, or
             * 3. if the property is actually declared in the object type
             *    (this means that 'toString', for example, is not usually a known property).
             * 4. In a union or intersection type,
             *    a property is considered known if it is known in any constituent type.
             * @param targetType a type to search a given name in
             * @param name a property name to search
             * @param isComparingJsxAttributes a boolean flag indicating whether we are searching in JsxAttributesType
             */
            function isKnownProperty(targetType, name, isComparingJsxAttributes) {
                if (targetType.flags & 524288 /* Object */) {
                    var resolved = resolveStructuredTypeMembers(targetType);
                    if (resolved.stringIndexInfo ||
                        resolved.numberIndexInfo && isNumericLiteralName(name) ||
                        getPropertyOfObjectType(targetType, name) ||
                        isComparingJsxAttributes && !isUnhyphenatedJsxName(name)) {
                        // For JSXAttributes, if the attribute has a hyphenated name, consider that the attribute to be known.
                        return true;
                    }
                }
                else if (targetType.flags & 3145728 /* UnionOrIntersection */ && isExcessPropertyCheckTarget(targetType)) {
                    for (var _i = 0, _a = targetType.types; _i < _a.length; _i++) {
                        var t = _a[_i];
                        if (isKnownProperty(t, name, isComparingJsxAttributes)) {
                            return true;
                        }
                    }
                }
                return false;
            }
            function isExcessPropertyCheckTarget(type) {
                return !!(type.flags & 524288 /* Object */ && !(ts.getObjectFlags(type) & 512 /* ObjectLiteralPatternWithComputedProperties */) ||
                    type.flags & 67108864 /* NonPrimitive */ ||
                    type.flags & 1048576 /* Union */ && ts.some(type.types, isExcessPropertyCheckTarget) ||
                    type.flags & 2097152 /* Intersection */ && ts.every(type.types, isExcessPropertyCheckTarget));
            }
            function checkJsxExpression(node, checkMode) {
                checkGrammarJsxExpression(node);
                if (node.expression) {
                    var type = checkExpression(node.expression, checkMode);
                    if (node.dotDotDotToken && type !== anyType && !isArrayType(type)) {
                        error(node, ts.Diagnostics.JSX_spread_child_must_be_an_array_type);
                    }
                    return type;
                }
                else {
                    return errorType;
                }
            }
            function getDeclarationNodeFlagsFromSymbol(s) {
                return s.valueDeclaration ? ts.getCombinedNodeFlags(s.valueDeclaration) : 0;
            }
            /**
             * Return whether this symbol is a member of a prototype somewhere
             * Note that this is not tracked well within the compiler, so the answer may be incorrect.
             */
            function isPrototypeProperty(symbol) {
                if (symbol.flags & 8192 /* Method */ || ts.getCheckFlags(symbol) & 4 /* SyntheticMethod */) {
                    return true;
                }
                if (ts.isInJSFile(symbol.valueDeclaration)) {
                    var parent = symbol.valueDeclaration.parent;
                    return parent && ts.isBinaryExpression(parent) &&
                        ts.getAssignmentDeclarationKind(parent) === 3 /* PrototypeProperty */;
                }
            }
            /**
             * Check whether the requested property access is valid.
             * Returns true if node is a valid property access, and false otherwise.
             * @param node The node to be checked.
             * @param isSuper True if the access is from `super.`.
             * @param type The type of the object whose property is being accessed. (Not the type of the property.)
             * @param prop The symbol for the property being accessed.
             */
            function checkPropertyAccessibility(node, isSuper, type, prop) {
                var flags = ts.getDeclarationModifierFlagsFromSymbol(prop);
                var errorNode = node.kind === 153 /* QualifiedName */ ? node.right : node.kind === 188 /* ImportType */ ? node : node.name;
                if (ts.getCheckFlags(prop) & 1024 /* ContainsPrivate */) {
                    // Synthetic property with private constituent property
                    error(errorNode, ts.Diagnostics.Property_0_has_conflicting_declarations_and_is_inaccessible_in_type_1, symbolToString(prop), typeToString(type));
                    return false;
                }
                if (isSuper) {
                    // TS 1.0 spec (April 2014): 4.8.2
                    // - In a constructor, instance member function, instance member accessor, or
                    //   instance member variable initializer where this references a derived class instance,
                    //   a super property access is permitted and must specify a public instance member function of the base class.
                    // - In a static member function or static member accessor
                    //   where this references the constructor function object of a derived class,
                    //   a super property access is permitted and must specify a public static member function of the base class.
                    if (languageVersion < 2 /* ES2015 */) {
                        if (symbolHasNonMethodDeclaration(prop)) {
                            error(errorNode, ts.Diagnostics.Only_public_and_protected_methods_of_the_base_class_are_accessible_via_the_super_keyword);
                            return false;
                        }
                    }
                    if (flags & 128 /* Abstract */) {
                        // A method cannot be accessed in a super property access if the method is abstract.
                        // This error could mask a private property access error. But, a member
                        // cannot simultaneously be private and abstract, so this will trigger an
                        // additional error elsewhere.
                        error(errorNode, ts.Diagnostics.Abstract_method_0_in_class_1_cannot_be_accessed_via_super_expression, symbolToString(prop), typeToString(getDeclaringClass(prop)));
                        return false;
                    }
                }
                // Referencing abstract properties within their own constructors is not allowed
                if ((flags & 128 /* Abstract */) && ts.isThisProperty(node) && symbolHasNonMethodDeclaration(prop)) {
                    var declaringClassDeclaration = ts.getClassLikeDeclarationOfSymbol(getParentOfSymbol(prop));
                    if (declaringClassDeclaration && isNodeUsedDuringClassInitialization(node)) {
                        error(errorNode, ts.Diagnostics.Abstract_property_0_in_class_1_cannot_be_accessed_in_the_constructor, symbolToString(prop), ts.getTextOfIdentifierOrLiteral(declaringClassDeclaration.name)); // TODO: GH#18217
                        return false;
                    }
                }
                if (ts.isPropertyAccessExpression(node) && ts.isPrivateIdentifier(node.name)) {
                    if (!ts.getContainingClass(node)) {
                        error(errorNode, ts.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies);
                        return false;
                    }
                    return true;
                }
                // Public properties are otherwise accessible.
                if (!(flags & 24 /* NonPublicAccessibilityModifier */)) {
                    return true;
                }
                // Property is known to be private or protected at this point
                // Private property is accessible if the property is within the declaring class
                if (flags & 8 /* Private */) {
                    var declaringClassDeclaration = ts.getClassLikeDeclarationOfSymbol(getParentOfSymbol(prop));
                    if (!isNodeWithinClass(node, declaringClassDeclaration)) {
                        error(errorNode, ts.Diagnostics.Property_0_is_private_and_only_accessible_within_class_1, symbolToString(prop), typeToString(getDeclaringClass(prop)));
                        return false;
                    }
                    return true;
                }
                // Property is known to be protected at this point
                // All protected properties of a supertype are accessible in a super access
                if (isSuper) {
                    return true;
                }
                // Find the first enclosing class that has the declaring classes of the protected constituents
                // of the property as base classes
                var enclosingClass = forEachEnclosingClass(node, function (enclosingDeclaration) {
                    var enclosingClass = getDeclaredTypeOfSymbol(getSymbolOfNode(enclosingDeclaration));
                    return isClassDerivedFromDeclaringClasses(enclosingClass, prop) ? enclosingClass : undefined;
                });
                // A protected property is accessible if the property is within the declaring class or classes derived from it
                if (!enclosingClass) {
                    // allow PropertyAccessibility if context is in function with this parameter
                    // static member access is disallow
                    var thisParameter = void 0;
                    if (flags & 32 /* Static */ || !(thisParameter = getThisParameterFromNodeContext(node)) || !thisParameter.type) {
                        error(errorNode, ts.Diagnostics.Property_0_is_protected_and_only_accessible_within_class_1_and_its_subclasses, symbolToString(prop), typeToString(getDeclaringClass(prop) || type));
                        return false;
                    }
                    var thisType = getTypeFromTypeNode(thisParameter.type);
                    enclosingClass = ((thisType.flags & 262144 /* TypeParameter */) ? getConstraintOfTypeParameter(thisType) : thisType);
                }
                // No further restrictions for static properties
                if (flags & 32 /* Static */) {
                    return true;
                }
                if (type.flags & 262144 /* TypeParameter */) {
                    // get the original type -- represented as the type constraint of the 'this' type
                    type = type.isThisType ? getConstraintOfTypeParameter(type) : getBaseConstraintOfType(type); // TODO: GH#18217 Use a different variable that's allowed to be undefined
                }
                if (!type || !hasBaseType(type, enclosingClass)) {
                    error(errorNode, ts.Diagnostics.Property_0_is_protected_and_only_accessible_through_an_instance_of_class_1, symbolToString(prop), typeToString(enclosingClass));
                    return false;
                }
                return true;
            }
            function getThisParameterFromNodeContext(node) {
                var thisContainer = ts.getThisContainer(node, /* includeArrowFunctions */ false);
                return thisContainer && ts.isFunctionLike(thisContainer) ? ts.getThisParameter(thisContainer) : undefined;
            }
            function symbolHasNonMethodDeclaration(symbol) {
                return !!forEachProperty(symbol, function (prop) { return !(prop.flags & 8192 /* Method */); });
            }
            function checkNonNullExpression(node) {
                return checkNonNullType(checkExpression(node), node);
            }
            function isNullableType(type) {
                return !!((strictNullChecks ? getFalsyFlags(type) : type.flags) & 98304 /* Nullable */);
            }
            function getNonNullableTypeIfNeeded(type) {
                return isNullableType(type) ? getNonNullableType(type) : type;
            }
            function reportObjectPossiblyNullOrUndefinedError(node, flags) {
                error(node, flags & 32768 /* Undefined */ ? flags & 65536 /* Null */ ?
                    ts.Diagnostics.Object_is_possibly_null_or_undefined :
                    ts.Diagnostics.Object_is_possibly_undefined :
                    ts.Diagnostics.Object_is_possibly_null);
            }
            function reportCannotInvokePossiblyNullOrUndefinedError(node, flags) {
                error(node, flags & 32768 /* Undefined */ ? flags & 65536 /* Null */ ?
                    ts.Diagnostics.Cannot_invoke_an_object_which_is_possibly_null_or_undefined :
                    ts.Diagnostics.Cannot_invoke_an_object_which_is_possibly_undefined :
                    ts.Diagnostics.Cannot_invoke_an_object_which_is_possibly_null);
            }
            function checkNonNullTypeWithReporter(type, node, reportError) {
                if (strictNullChecks && type.flags & 2 /* Unknown */) {
                    error(node, ts.Diagnostics.Object_is_of_type_unknown);
                    return errorType;
                }
                var kind = (strictNullChecks ? getFalsyFlags(type) : type.flags) & 98304 /* Nullable */;
                if (kind) {
                    reportError(node, kind);
                    var t = getNonNullableType(type);
                    return t.flags & (98304 /* Nullable */ | 131072 /* Never */) ? errorType : t;
                }
                return type;
            }
            function checkNonNullType(type, node) {
                return checkNonNullTypeWithReporter(type, node, reportObjectPossiblyNullOrUndefinedError);
            }
            function checkNonNullNonVoidType(type, node) {
                var nonNullType = checkNonNullType(type, node);
                if (nonNullType !== errorType && nonNullType.flags & 16384 /* Void */) {
                    error(node, ts.Diagnostics.Object_is_possibly_undefined);
                }
                return nonNullType;
            }
            function checkPropertyAccessExpression(node) {
                return node.flags & 32 /* OptionalChain */ ? checkPropertyAccessChain(node) :
                    checkPropertyAccessExpressionOrQualifiedName(node, node.expression, checkNonNullExpression(node.expression), node.name);
            }
            function checkPropertyAccessChain(node) {
                var leftType = checkExpression(node.expression);
                var nonOptionalType = getOptionalExpressionType(leftType, node.expression);
                return propagateOptionalTypeMarker(checkPropertyAccessExpressionOrQualifiedName(node, node.expression, checkNonNullType(nonOptionalType, node.expression), node.name), node, nonOptionalType !== leftType);
            }
            function checkQualifiedName(node) {
                return checkPropertyAccessExpressionOrQualifiedName(node, node.left, checkNonNullExpression(node.left), node.right);
            }
            function isMethodAccessForCall(node) {
                while (node.parent.kind === 200 /* ParenthesizedExpression */) {
                    node = node.parent;
                }
                return ts.isCallOrNewExpression(node.parent) && node.parent.expression === node;
            }
            // Lookup the private identifier lexically.
            function lookupSymbolForPrivateIdentifierDeclaration(propName, location) {
                for (var containingClass = ts.getContainingClass(location); !!containingClass; containingClass = ts.getContainingClass(containingClass)) {
                    var symbol = containingClass.symbol;
                    var name = ts.getSymbolNameForPrivateIdentifier(symbol, propName);
                    var prop = (symbol.members && symbol.members.get(name)) || (symbol.exports && symbol.exports.get(name));
                    if (prop) {
                        return prop;
                    }
                }
            }
            function getPrivateIdentifierPropertyOfType(leftType, lexicallyScopedIdentifier) {
                return getPropertyOfType(leftType, lexicallyScopedIdentifier.escapedName);
            }
            function checkPrivateIdentifierPropertyAccess(leftType, right, lexicallyScopedIdentifier) {
                // Either the identifier could not be looked up in the lexical scope OR the lexically scoped identifier did not exist on the type.
                // Find a private identifier with the same description on the type.
                var propertyOnType;
                var properties = getPropertiesOfType(leftType);
                if (properties) {
                    ts.forEach(properties, function (symbol) {
                        var decl = symbol.valueDeclaration;
                        if (decl && ts.isNamedDeclaration(decl) && ts.isPrivateIdentifier(decl.name) && decl.name.escapedText === right.escapedText) {
                            propertyOnType = symbol;
                            return true;
                        }
                    });
                }
                var diagName = diagnosticName(right);
                if (propertyOnType) {
                    var typeValueDecl = propertyOnType.valueDeclaration;
                    var typeClass_1 = ts.getContainingClass(typeValueDecl);
                    ts.Debug.assert(!!typeClass_1);
                    // We found a private identifier property with the same description.
                    // Either:
                    // - There is a lexically scoped private identifier AND it shadows the one we found on the type.
                    // - It is an attempt to access the private identifier outside of the class.
                    if (lexicallyScopedIdentifier) {
                        var lexicalValueDecl = lexicallyScopedIdentifier.valueDeclaration;
                        var lexicalClass = ts.getContainingClass(lexicalValueDecl);
                        ts.Debug.assert(!!lexicalClass);
                        if (ts.findAncestor(lexicalClass, function (n) { return typeClass_1 === n; })) {
                            var diagnostic = error(right, ts.Diagnostics.The_property_0_cannot_be_accessed_on_type_1_within_this_class_because_it_is_shadowed_by_another_private_identifier_with_the_same_spelling, diagName, typeToString(leftType));
                            ts.addRelatedInfo(diagnostic, ts.createDiagnosticForNode(lexicalValueDecl, ts.Diagnostics.The_shadowing_declaration_of_0_is_defined_here, diagName), ts.createDiagnosticForNode(typeValueDecl, ts.Diagnostics.The_declaration_of_0_that_you_probably_intended_to_use_is_defined_here, diagName));
                            return true;
                        }
                    }
                    error(right, ts.Diagnostics.Property_0_is_not_accessible_outside_class_1_because_it_has_a_private_identifier, diagName, diagnosticName(typeClass_1.name || anon));
                    return true;
                }
                return false;
            }
            function checkPropertyAccessExpressionOrQualifiedName(node, left, leftType, right) {
                var parentSymbol = getNodeLinks(left).resolvedSymbol;
                var assignmentKind = ts.getAssignmentTargetKind(node);
                var apparentType = getApparentType(assignmentKind !== 0 /* None */ || isMethodAccessForCall(node) ? getWidenedType(leftType) : leftType);
                if (ts.isPrivateIdentifier(right)) {
                    checkExternalEmitHelpers(node, 262144 /* ClassPrivateFieldGet */);
                }
                var isAnyLike = isTypeAny(apparentType) || apparentType === silentNeverType;
                var prop;
                if (ts.isPrivateIdentifier(right)) {
                    var lexicallyScopedSymbol = lookupSymbolForPrivateIdentifierDeclaration(right.escapedText, right);
                    if (isAnyLike) {
                        if (lexicallyScopedSymbol) {
                            return apparentType;
                        }
                        if (!ts.getContainingClass(right)) {
                            grammarErrorOnNode(right, ts.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies);
                            return anyType;
                        }
                    }
                    prop = lexicallyScopedSymbol ? getPrivateIdentifierPropertyOfType(leftType, lexicallyScopedSymbol) : undefined;
                    // Check for private-identifier-specific shadowing and lexical-scoping errors.
                    if (!prop && checkPrivateIdentifierPropertyAccess(leftType, right, lexicallyScopedSymbol)) {
                        return errorType;
                    }
                }
                else {
                    if (isAnyLike) {
                        if (ts.isIdentifier(left) && parentSymbol) {
                            markAliasReferenced(parentSymbol, node);
                        }
                        return apparentType;
                    }
                    prop = getPropertyOfType(apparentType, right.escapedText);
                }
                if (ts.isIdentifier(left) && parentSymbol && !(prop && isConstEnumOrConstEnumOnlyModule(prop))) {
                    markAliasReferenced(parentSymbol, node);
                }
                var propType;
                if (!prop) {
                    var indexInfo = !ts.isPrivateIdentifier(right) && (assignmentKind === 0 /* None */ || !isGenericObjectType(leftType) || isThisTypeParameter(leftType)) ? getIndexInfoOfType(apparentType, 0 /* String */) : undefined;
                    if (!(indexInfo && indexInfo.type)) {
                        if (isJSLiteralType(leftType)) {
                            return anyType;
                        }
                        if (leftType.symbol === globalThisSymbol) {
                            if (globalThisSymbol.exports.has(right.escapedText) && (globalThisSymbol.exports.get(right.escapedText).flags & 418 /* BlockScoped */)) {
                                error(right, ts.Diagnostics.Property_0_does_not_exist_on_type_1, ts.unescapeLeadingUnderscores(right.escapedText), typeToString(leftType));
                            }
                            else if (noImplicitAny) {
                                error(right, ts.Diagnostics.Element_implicitly_has_an_any_type_because_type_0_has_no_index_signature, typeToString(leftType));
                            }
                            return anyType;
                        }
                        if (right.escapedText && !checkAndReportErrorForExtendingInterface(node)) {
                            reportNonexistentProperty(right, isThisTypeParameter(leftType) ? apparentType : leftType);
                        }
                        return errorType;
                    }
                    if (indexInfo.isReadonly && (ts.isAssignmentTarget(node) || ts.isDeleteTarget(node))) {
                        error(node, ts.Diagnostics.Index_signature_in_type_0_only_permits_reading, typeToString(apparentType));
                    }
                    propType = indexInfo.type;
                }
                else {
                    checkPropertyNotUsedBeforeDeclaration(prop, node, right);
                    markPropertyAsReferenced(prop, node, left.kind === 104 /* ThisKeyword */);
                    getNodeLinks(node).resolvedSymbol = prop;
                    checkPropertyAccessibility(node, left.kind === 102 /* SuperKeyword */, apparentType, prop);
                    if (isAssignmentToReadonlyEntity(node, prop, assignmentKind)) {
                        error(right, ts.Diagnostics.Cannot_assign_to_0_because_it_is_a_read_only_property, ts.idText(right));
                        return errorType;
                    }
                    propType = getConstraintForLocation(getTypeOfSymbol(prop), node);
                }
                return getFlowTypeOfAccessExpression(node, prop, propType, right);
            }
            function getFlowTypeOfAccessExpression(node, prop, propType, errorNode) {
                // Only compute control flow type if this is a property access expression that isn't an
                // assignment target, and the referenced property was declared as a variable, property,
                // accessor, or optional method.
                var assignmentKind = ts.getAssignmentTargetKind(node);
                if (!ts.isAccessExpression(node) ||
                    assignmentKind === 1 /* Definite */ ||
                    prop && !(prop.flags & (3 /* Variable */ | 4 /* Property */ | 98304 /* Accessor */)) && !(prop.flags & 8192 /* Method */ && propType.flags & 1048576 /* Union */)) {
                    return propType;
                }
                // If strict null checks and strict property initialization checks are enabled, if we have
                // a this.xxx property access, if the property is an instance property without an initializer,
                // and if we are in a constructor of the same class as the property declaration, assume that
                // the property is uninitialized at the top of the control flow.
                var assumeUninitialized = false;
                if (strictNullChecks && strictPropertyInitialization && node.expression.kind === 104 /* ThisKeyword */) {
                    var declaration = prop && prop.valueDeclaration;
                    if (declaration && isInstancePropertyWithoutInitializer(declaration)) {
                        var flowContainer = getControlFlowContainer(node);
                        if (flowContainer.kind === 162 /* Constructor */ && flowContainer.parent === declaration.parent && !(declaration.flags & 8388608 /* Ambient */)) {
                            assumeUninitialized = true;
                        }
                    }
                }
                else if (strictNullChecks && prop && prop.valueDeclaration &&
                    ts.isPropertyAccessExpression(prop.valueDeclaration) &&
                    ts.getAssignmentDeclarationPropertyAccessKind(prop.valueDeclaration) &&
                    getControlFlowContainer(node) === getControlFlowContainer(prop.valueDeclaration)) {
                    assumeUninitialized = true;
                }
                var flowType = getFlowTypeOfReference(node, propType, assumeUninitialized ? getOptionalType(propType) : propType);
                if (assumeUninitialized && !(getFalsyFlags(propType) & 32768 /* Undefined */) && getFalsyFlags(flowType) & 32768 /* Undefined */) {
                    error(errorNode, ts.Diagnostics.Property_0_is_used_before_being_assigned, symbolToString(prop)); // TODO: GH#18217
                    // Return the declared type to reduce follow-on errors
                    return propType;
                }
                return assignmentKind ? getBaseTypeOfLiteralType(flowType) : flowType;
            }
            function checkPropertyNotUsedBeforeDeclaration(prop, node, right) {
                var valueDeclaration = prop.valueDeclaration;
                if (!valueDeclaration || ts.getSourceFileOfNode(node).isDeclarationFile) {
                    return;
                }
                var diagnosticMessage;
                var declarationName = ts.idText(right);
                if (isInPropertyInitializer(node)
                    && !(ts.isAccessExpression(node) && ts.isAccessExpression(node.expression))
                    && !isBlockScopedNameDeclaredBeforeUse(valueDeclaration, right)
                    && !isPropertyDeclaredInAncestorClass(prop)) {
                    diagnosticMessage = error(right, ts.Diagnostics.Property_0_is_used_before_its_initialization, declarationName);
                }
                else if (valueDeclaration.kind === 245 /* ClassDeclaration */ &&
                    node.parent.kind !== 169 /* TypeReference */ &&
                    !(valueDeclaration.flags & 8388608 /* Ambient */) &&
                    !isBlockScopedNameDeclaredBeforeUse(valueDeclaration, right)) {
                    diagnosticMessage = error(right, ts.Diagnostics.Class_0_used_before_its_declaration, declarationName);
                }
                if (diagnosticMessage) {
                    ts.addRelatedInfo(diagnosticMessage, ts.createDiagnosticForNode(valueDeclaration, ts.Diagnostics._0_is_declared_here, declarationName));
                }
            }
            function isInPropertyInitializer(node) {
                return !!ts.findAncestor(node, function (node) {
                    switch (node.kind) {
                        case 159 /* PropertyDeclaration */:
                            return true;
                        case 281 /* PropertyAssignment */:
                        case 161 /* MethodDeclaration */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                        case 283 /* SpreadAssignment */:
                        case 154 /* ComputedPropertyName */:
                        case 221 /* TemplateSpan */:
                        case 276 /* JsxExpression */:
                        case 273 /* JsxAttribute */:
                        case 274 /* JsxAttributes */:
                        case 275 /* JsxSpreadAttribute */:
                        case 268 /* JsxOpeningElement */:
                        case 216 /* ExpressionWithTypeArguments */:
                        case 279 /* HeritageClause */:
                            return false;
                        default:
                            return ts.isExpressionNode(node) ? false : "quit";
                    }
                });
            }
            /**
             * It's possible that "prop.valueDeclaration" is a local declaration, but the property was also declared in a superclass.
             * In that case we won't consider it used before its declaration, because it gets its value from the superclass' declaration.
             */
            function isPropertyDeclaredInAncestorClass(prop) {
                if (!(prop.parent.flags & 32 /* Class */)) {
                    return false;
                }
                var classType = getTypeOfSymbol(prop.parent);
                while (true) {
                    classType = classType.symbol && getSuperClass(classType);
                    if (!classType) {
                        return false;
                    }
                    var superProperty = getPropertyOfType(classType, prop.escapedName);
                    if (superProperty && superProperty.valueDeclaration) {
                        return true;
                    }
                }
            }
            function getSuperClass(classType) {
                var x = getBaseTypes(classType);
                if (x.length === 0) {
                    return undefined;
                }
                return getIntersectionType(x);
            }
            function reportNonexistentProperty(propNode, containingType) {
                var errorInfo;
                var relatedInfo;
                if (!ts.isPrivateIdentifier(propNode) && containingType.flags & 1048576 /* Union */ && !(containingType.flags & 131068 /* Primitive */)) {
                    for (var _i = 0, _a = containingType.types; _i < _a.length; _i++) {
                        var subtype = _a[_i];
                        if (!getPropertyOfType(subtype, propNode.escapedText) && !getIndexInfoOfType(subtype, 0 /* String */)) {
                            errorInfo = ts.chainDiagnosticMessages(errorInfo, ts.Diagnostics.Property_0_does_not_exist_on_type_1, ts.declarationNameToString(propNode), typeToString(subtype));
                            break;
                        }
                    }
                }
                if (typeHasStaticProperty(propNode.escapedText, containingType)) {
                    errorInfo = ts.chainDiagnosticMessages(errorInfo, ts.Diagnostics.Property_0_is_a_static_member_of_type_1, ts.declarationNameToString(propNode), typeToString(containingType));
                }
                else {
                    var promisedType = getPromisedTypeOfPromise(containingType);
                    if (promisedType && getPropertyOfType(promisedType, propNode.escapedText)) {
                        errorInfo = ts.chainDiagnosticMessages(errorInfo, ts.Diagnostics.Property_0_does_not_exist_on_type_1, ts.declarationNameToString(propNode), typeToString(containingType));
                        relatedInfo = ts.createDiagnosticForNode(propNode, ts.Diagnostics.Did_you_forget_to_use_await);
                    }
                    else {
                        var suggestion = getSuggestedSymbolForNonexistentProperty(propNode, containingType);
                        if (suggestion !== undefined) {
                            var suggestedName = ts.symbolName(suggestion);
                            errorInfo = ts.chainDiagnosticMessages(errorInfo, ts.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2, ts.declarationNameToString(propNode), typeToString(containingType), suggestedName);
                            relatedInfo = suggestion.valueDeclaration && ts.createDiagnosticForNode(suggestion.valueDeclaration, ts.Diagnostics._0_is_declared_here, suggestedName);
                        }
                        else {
                            errorInfo = ts.chainDiagnosticMessages(errorInfo, ts.Diagnostics.Property_0_does_not_exist_on_type_1, ts.declarationNameToString(propNode), typeToString(containingType));
                        }
                    }
                }
                var resultDiagnostic = ts.createDiagnosticForNodeFromMessageChain(propNode, errorInfo);
                if (relatedInfo) {
                    ts.addRelatedInfo(resultDiagnostic, relatedInfo);
                }
                diagnostics.add(resultDiagnostic);
            }
            function typeHasStaticProperty(propName, containingType) {
                var prop = containingType.symbol && getPropertyOfType(getTypeOfSymbol(containingType.symbol), propName);
                return prop !== undefined && prop.valueDeclaration && ts.hasModifier(prop.valueDeclaration, 32 /* Static */);
            }
            function getSuggestedSymbolForNonexistentProperty(name, containingType) {
                return getSpellingSuggestionForName(ts.isString(name) ? name : ts.idText(name), getPropertiesOfType(containingType), 111551 /* Value */);
            }
            function getSuggestionForNonexistentProperty(name, containingType) {
                var suggestion = getSuggestedSymbolForNonexistentProperty(name, containingType);
                return suggestion && ts.symbolName(suggestion);
            }
            function getSuggestedSymbolForNonexistentSymbol(location, outerName, meaning) {
                ts.Debug.assert(outerName !== undefined, "outername should always be defined");
                var result = resolveNameHelper(location, outerName, meaning, /*nameNotFoundMessage*/ undefined, outerName, /*isUse*/ false, /*excludeGlobals*/ false, function (symbols, name, meaning) {
                    ts.Debug.assertEqual(outerName, name, "name should equal outerName");
                    var symbol = getSymbol(symbols, name, meaning);
                    // Sometimes the symbol is found when location is a return type of a function: `typeof x` and `x` is declared in the body of the function
                    // So the table *contains* `x` but `x` isn't actually in scope.
                    // However, resolveNameHelper will continue and call this callback again, so we'll eventually get a correct suggestion.
                    return symbol || getSpellingSuggestionForName(ts.unescapeLeadingUnderscores(name), ts.arrayFrom(symbols.values()), meaning);
                });
                return result;
            }
            function getSuggestionForNonexistentSymbol(location, outerName, meaning) {
                var symbolResult = getSuggestedSymbolForNonexistentSymbol(location, outerName, meaning);
                return symbolResult && ts.symbolName(symbolResult);
            }
            function getSuggestedSymbolForNonexistentModule(name, targetModule) {
                return targetModule.exports && getSpellingSuggestionForName(ts.idText(name), getExportsOfModuleAsArray(targetModule), 2623475 /* ModuleMember */);
            }
            function getSuggestionForNonexistentExport(name, targetModule) {
                var suggestion = getSuggestedSymbolForNonexistentModule(name, targetModule);
                return suggestion && ts.symbolName(suggestion);
            }
            function getSuggestionForNonexistentIndexSignature(objectType, expr, keyedType) {
                // check if object type has setter or getter
                function hasProp(name) {
                    var prop = getPropertyOfObjectType(objectType, name);
                    if (prop) {
                        var s = getSingleCallSignature(getTypeOfSymbol(prop));
                        return !!s && getMinArgumentCount(s) >= 1 && isTypeAssignableTo(keyedType, getTypeAtPosition(s, 0));
                    }
                    return false;
                }
                ;
                var suggestedMethod = ts.isAssignmentTarget(expr) ? "set" : "get";
                if (!hasProp(suggestedMethod)) {
                    return undefined;
                }
                var suggestion = ts.tryGetPropertyAccessOrIdentifierToString(expr.expression);
                if (suggestion === undefined) {
                    suggestion = suggestedMethod;
                }
                else {
                    suggestion += "." + suggestedMethod;
                }
                return suggestion;
            }
            /**
             * Given a name and a list of symbols whose names are *not* equal to the name, return a spelling suggestion if there is one that is close enough.
             * Names less than length 3 only check for case-insensitive equality, not levenshtein distance.
             *
             * If there is a candidate that's the same except for case, return that.
             * If there is a candidate that's within one edit of the name, return that.
             * Otherwise, return the candidate with the smallest Levenshtein distance,
             *    except for candidates:
             *      * With no name
             *      * Whose meaning doesn't match the `meaning` parameter.
             *      * Whose length differs from the target name by more than 0.34 of the length of the name.
             *      * Whose levenshtein distance is more than 0.4 of the length of the name
             *        (0.4 allows 1 substitution/transposition for every 5 characters,
             *         and 1 insertion/deletion at 3 characters)
             */
            function getSpellingSuggestionForName(name, symbols, meaning) {
                return ts.getSpellingSuggestion(name, symbols, getCandidateName);
                function getCandidateName(candidate) {
                    var candidateName = ts.symbolName(candidate);
                    return !ts.startsWith(candidateName, "\"") && candidate.flags & meaning ? candidateName : undefined;
                }
            }
            function markPropertyAsReferenced(prop, nodeForCheckWriteOnly, isThisAccess) {
                var valueDeclaration = prop && (prop.flags & 106500 /* ClassMember */) && prop.valueDeclaration;
                if (!valueDeclaration) {
                    return;
                }
                var hasPrivateModifier = ts.hasModifier(valueDeclaration, 8 /* Private */);
                var hasPrivateIdentifier = ts.isNamedDeclaration(prop.valueDeclaration) && ts.isPrivateIdentifier(prop.valueDeclaration.name);
                if (!hasPrivateModifier && !hasPrivateIdentifier) {
                    return;
                }
                if (nodeForCheckWriteOnly && ts.isWriteOnlyAccess(nodeForCheckWriteOnly) && !(prop.flags & 65536 /* SetAccessor */ && !(prop.flags & 32768 /* GetAccessor */))) {
                    return;
                }
                if (isThisAccess) {
                    // Find any FunctionLikeDeclaration because those create a new 'this' binding. But this should only matter for methods (or getters/setters).
                    var containingMethod = ts.findAncestor(nodeForCheckWriteOnly, ts.isFunctionLikeDeclaration);
                    if (containingMethod && containingMethod.symbol === prop) {
                        return;
                    }
                }
                (ts.getCheckFlags(prop) & 1 /* Instantiated */ ? getSymbolLinks(prop).target : prop).isReferenced = 67108863 /* All */;
            }
            function isValidPropertyAccess(node, propertyName) {
                switch (node.kind) {
                    case 194 /* PropertyAccessExpression */:
                        return isValidPropertyAccessWithType(node, node.expression.kind === 102 /* SuperKeyword */, propertyName, getWidenedType(checkExpression(node.expression)));
                    case 153 /* QualifiedName */:
                        return isValidPropertyAccessWithType(node, /*isSuper*/ false, propertyName, getWidenedType(checkExpression(node.left)));
                    case 188 /* ImportType */:
                        return isValidPropertyAccessWithType(node, /*isSuper*/ false, propertyName, getTypeFromTypeNode(node));
                }
            }
            function isValidPropertyAccessForCompletions(node, type, property) {
                return isValidPropertyAccessWithType(node, node.kind === 194 /* PropertyAccessExpression */ && node.expression.kind === 102 /* SuperKeyword */, property.escapedName, type);
                // Previously we validated the 'this' type of methods but this adversely affected performance. See #31377 for more context.
            }
            function isValidPropertyAccessWithType(node, isSuper, propertyName, type) {
                if (type === errorType || isTypeAny(type)) {
                    return true;
                }
                var prop = getPropertyOfType(type, propertyName);
                if (prop) {
                    if (ts.isPropertyAccessExpression(node) && prop.valueDeclaration && ts.isPrivateIdentifierPropertyDeclaration(prop.valueDeclaration)) {
                        var declClass_1 = ts.getContainingClass(prop.valueDeclaration);
                        return !ts.isOptionalChain(node) && !!ts.findAncestor(node, function (parent) { return parent === declClass_1; });
                    }
                    return checkPropertyAccessibility(node, isSuper, type, prop);
                }
                // In js files properties of unions are allowed in completion
                return ts.isInJSFile(node) && (type.flags & 1048576 /* Union */) !== 0 && type.types.some(function (elementType) { return isValidPropertyAccessWithType(node, isSuper, propertyName, elementType); });
            }
            /**
             * Return the symbol of the for-in variable declared or referenced by the given for-in statement.
             */
            function getForInVariableSymbol(node) {
                var initializer = node.initializer;
                if (initializer.kind === 243 /* VariableDeclarationList */) {
                    var variable = initializer.declarations[0];
                    if (variable && !ts.isBindingPattern(variable.name)) {
                        return getSymbolOfNode(variable);
                    }
                }
                else if (initializer.kind === 75 /* Identifier */) {
                    return getResolvedSymbol(initializer);
                }
                return undefined;
            }
            /**
             * Return true if the given type is considered to have numeric property names.
             */
            function hasNumericPropertyNames(type) {
                return getIndexTypeOfType(type, 1 /* Number */) && !getIndexTypeOfType(type, 0 /* String */);
            }
            /**
             * Return true if given node is an expression consisting of an identifier (possibly parenthesized)
             * that references a for-in variable for an object with numeric property names.
             */
            function isForInVariableForNumericPropertyNames(expr) {
                var e = ts.skipParentheses(expr);
                if (e.kind === 75 /* Identifier */) {
                    var symbol = getResolvedSymbol(e);
                    if (symbol.flags & 3 /* Variable */) {
                        var child = expr;
                        var node = expr.parent;
                        while (node) {
                            if (node.kind === 231 /* ForInStatement */ &&
                                child === node.statement &&
                                getForInVariableSymbol(node) === symbol &&
                                hasNumericPropertyNames(getTypeOfExpression(node.expression))) {
                                return true;
                            }
                            child = node;
                            node = node.parent;
                        }
                    }
                }
                return false;
            }
            function checkIndexedAccess(node) {
                return node.flags & 32 /* OptionalChain */ ? checkElementAccessChain(node) :
                    checkElementAccessExpression(node, checkNonNullExpression(node.expression));
            }
            function checkElementAccessChain(node) {
                var exprType = checkExpression(node.expression);
                var nonOptionalType = getOptionalExpressionType(exprType, node.expression);
                return propagateOptionalTypeMarker(checkElementAccessExpression(node, checkNonNullType(nonOptionalType, node.expression)), node, nonOptionalType !== exprType);
            }
            function checkElementAccessExpression(node, exprType) {
                var objectType = ts.getAssignmentTargetKind(node) !== 0 /* None */ || isMethodAccessForCall(node) ? getWidenedType(exprType) : exprType;
                var indexExpression = node.argumentExpression;
                var indexType = checkExpression(indexExpression);
                if (objectType === errorType || objectType === silentNeverType) {
                    return objectType;
                }
                if (isConstEnumObjectType(objectType) && !ts.isStringLiteralLike(indexExpression)) {
                    error(indexExpression, ts.Diagnostics.A_const_enum_member_can_only_be_accessed_using_a_string_literal);
                    return errorType;
                }
                var effectiveIndexType = isForInVariableForNumericPropertyNames(indexExpression) ? numberType : indexType;
                var accessFlags = ts.isAssignmentTarget(node) ?
                    2 /* Writing */ | (isGenericObjectType(objectType) && !isThisTypeParameter(objectType) ? 1 /* NoIndexSignatures */ : 0) :
                    0 /* None */;
                var indexedAccessType = getIndexedAccessTypeOrUndefined(objectType, effectiveIndexType, node, accessFlags) || errorType;
                return checkIndexedAccessIndexType(getFlowTypeOfAccessExpression(node, indexedAccessType.symbol, indexedAccessType, indexExpression), node);
            }
            function checkThatExpressionIsProperSymbolReference(expression, expressionType, reportError) {
                if (expressionType === errorType) {
                    // There is already an error, so no need to report one.
                    return false;
                }
                if (!ts.isWellKnownSymbolSyntactically(expression)) {
                    return false;
                }
                // Make sure the property type is the primitive symbol type
                if ((expressionType.flags & 12288 /* ESSymbolLike */) === 0) {
                    if (reportError) {
                        error(expression, ts.Diagnostics.A_computed_property_name_of_the_form_0_must_be_of_type_symbol, ts.getTextOfNode(expression));
                    }
                    return false;
                }
                // The name is Symbol.<someName>, so make sure Symbol actually resolves to the
                // global Symbol object
                var leftHandSide = expression.expression;
                var leftHandSideSymbol = getResolvedSymbol(leftHandSide);
                if (!leftHandSideSymbol) {
                    return false;
                }
                var globalESSymbol = getGlobalESSymbolConstructorSymbol(/*reportErrors*/ true);
                if (!globalESSymbol) {
                    // Already errored when we tried to look up the symbol
                    return false;
                }
                if (leftHandSideSymbol !== globalESSymbol) {
                    if (reportError) {
                        error(leftHandSide, ts.Diagnostics.Symbol_reference_does_not_refer_to_the_global_Symbol_constructor_object);
                    }
                    return false;
                }
                return true;
            }
            function callLikeExpressionMayHaveTypeArguments(node) {
                return ts.isCallOrNewExpression(node) || ts.isTaggedTemplateExpression(node) || ts.isJsxOpeningLikeElement(node);
            }
            function resolveUntypedCall(node) {
                if (callLikeExpressionMayHaveTypeArguments(node)) {
                    // Check type arguments even though we will give an error that untyped calls may not accept type arguments.
                    // This gets us diagnostics for the type arguments and marks them as referenced.
                    ts.forEach(node.typeArguments, checkSourceElement);
                }
                if (node.kind === 198 /* TaggedTemplateExpression */) {
                    checkExpression(node.template);
                }
                else if (ts.isJsxOpeningLikeElement(node)) {
                    checkExpression(node.attributes);
                }
                else if (node.kind !== 157 /* Decorator */) {
                    ts.forEach(node.arguments, function (argument) {
                        checkExpression(argument);
                    });
                }
                return anySignature;
            }
            function resolveErrorCall(node) {
                resolveUntypedCall(node);
                return unknownSignature;
            }
            // Re-order candidate signatures into the result array. Assumes the result array to be empty.
            // The candidate list orders groups in reverse, but within a group signatures are kept in declaration order
            // A nit here is that we reorder only signatures that belong to the same symbol,
            // so order how inherited signatures are processed is still preserved.
            // interface A { (x: string): void }
            // interface B extends A { (x: 'foo'): string }
            // const b: B;
            // b('foo') // <- here overloads should be processed as [(x:'foo'): string, (x: string): void]
            function reorderCandidates(signatures, result, callChainFlags) {
                var lastParent;
                var lastSymbol;
                var cutoffIndex = 0;
                var index;
                var specializedIndex = -1;
                var spliceIndex;
                ts.Debug.assert(!result.length);
                for (var _i = 0, signatures_7 = signatures; _i < signatures_7.length; _i++) {
                    var signature = signatures_7[_i];
                    var symbol = signature.declaration && getSymbolOfNode(signature.declaration);
                    var parent = signature.declaration && signature.declaration.parent;
                    if (!lastSymbol || symbol === lastSymbol) {
                        if (lastParent && parent === lastParent) {
                            index = index + 1;
                        }
                        else {
                            lastParent = parent;
                            index = cutoffIndex;
                        }
                    }
                    else {
                        // current declaration belongs to a different symbol
                        // set cutoffIndex so re-orderings in the future won't change result set from 0 to cutoffIndex
                        index = cutoffIndex = result.length;
                        lastParent = parent;
                    }
                    lastSymbol = symbol;
                    // specialized signatures always need to be placed before non-specialized signatures regardless
                    // of the cutoff position; see GH#1133
                    if (signatureHasLiteralTypes(signature)) {
                        specializedIndex++;
                        spliceIndex = specializedIndex;
                        // The cutoff index always needs to be greater than or equal to the specialized signature index
                        // in order to prevent non-specialized signatures from being added before a specialized
                        // signature.
                        cutoffIndex++;
                    }
                    else {
                        spliceIndex = index;
                    }
                    result.splice(spliceIndex, 0, callChainFlags ? getOptionalCallSignature(signature, callChainFlags) : signature);
                }
            }
            function isSpreadArgument(arg) {
                return !!arg && (arg.kind === 213 /* SpreadElement */ || arg.kind === 220 /* SyntheticExpression */ && arg.isSpread);
            }
            function getSpreadArgumentIndex(args) {
                return ts.findIndex(args, isSpreadArgument);
            }
            function acceptsVoid(t) {
                return !!(t.flags & 16384 /* Void */);
            }
            function hasCorrectArity(node, args, signature, signatureHelpTrailingComma) {
                if (signatureHelpTrailingComma === void 0) {
                    signatureHelpTrailingComma = false;
                }
                var argCount;
                var callIsIncomplete = false; // In incomplete call we want to be lenient when we have too few arguments
                var effectiveParameterCount = getParameterCount(signature);
                var effectiveMinimumArguments = getMinArgumentCount(signature);
                if (node.kind === 198 /* TaggedTemplateExpression */) {
                    argCount = args.length;
                    if (node.template.kind === 211 /* TemplateExpression */) {
                        // If a tagged template expression lacks a tail literal, the call is incomplete.
                        // Specifically, a template only can end in a TemplateTail or a Missing literal.
                        var lastSpan = ts.last(node.template.templateSpans); // we should always have at least one span.
                        callIsIncomplete = ts.nodeIsMissing(lastSpan.literal) || !!lastSpan.literal.isUnterminated;
                    }
                    else {
                        // If the template didn't end in a backtick, or its beginning occurred right prior to EOF,
                        // then this might actually turn out to be a TemplateHead in the future;
                        // so we consider the call to be incomplete.
                        var templateLiteral = node.template;
                        ts.Debug.assert(templateLiteral.kind === 14 /* NoSubstitutionTemplateLiteral */);
                        callIsIncomplete = !!templateLiteral.isUnterminated;
                    }
                }
                else if (node.kind === 157 /* Decorator */) {
                    argCount = getDecoratorArgumentCount(node, signature);
                }
                else if (ts.isJsxOpeningLikeElement(node)) {
                    callIsIncomplete = node.attributes.end === node.end;
                    if (callIsIncomplete) {
                        return true;
                    }
                    argCount = effectiveMinimumArguments === 0 ? args.length : 1;
                    effectiveParameterCount = args.length === 0 ? effectiveParameterCount : 1; // class may have argumentless ctor functions - still resolve ctor and compare vs props member type
                    effectiveMinimumArguments = Math.min(effectiveMinimumArguments, 1); // sfc may specify context argument - handled by framework and not typechecked
                }
                else {
                    if (!node.arguments) {
                        // This only happens when we have something of the form: 'new C'
                        ts.Debug.assert(node.kind === 197 /* NewExpression */);
                        return getMinArgumentCount(signature) === 0;
                    }
                    argCount = signatureHelpTrailingComma ? args.length + 1 : args.length;
                    // If we are missing the close parenthesis, the call is incomplete.
                    callIsIncomplete = node.arguments.end === node.end;
                    // If a spread argument is present, check that it corresponds to a rest parameter or at least that it's in the valid range.
                    var spreadArgIndex = getSpreadArgumentIndex(args);
                    if (spreadArgIndex >= 0) {
                        return spreadArgIndex >= getMinArgumentCount(signature) && (hasEffectiveRestParameter(signature) || spreadArgIndex < getParameterCount(signature));
                    }
                }
                // Too many arguments implies incorrect arity.
                if (!hasEffectiveRestParameter(signature) && argCount > effectiveParameterCount) {
                    return false;
                }
                // If the call is incomplete, we should skip the lower bound check.
                // JSX signatures can have extra parameters provided by the library which we don't check
                if (callIsIncomplete || argCount >= effectiveMinimumArguments) {
                    return true;
                }
                for (var i = argCount; i < effectiveMinimumArguments; i++) {
                    var type = getTypeAtPosition(signature, i);
                    if (filterType(type, acceptsVoid).flags & 131072 /* Never */) {
                        return false;
                    }
                }
                return true;
            }
            function hasCorrectTypeArgumentArity(signature, typeArguments) {
                // If the user supplied type arguments, but the number of type arguments does not match
                // the declared number of type parameters, the call has an incorrect arity.
                var numTypeParameters = ts.length(signature.typeParameters);
                var minTypeArgumentCount = getMinTypeArgumentCount(signature.typeParameters);
                return !ts.some(typeArguments) ||
                    (typeArguments.length >= minTypeArgumentCount && typeArguments.length <= numTypeParameters);
            }
            // If type has a single call signature and no other members, return that signature. Otherwise, return undefined.
            function getSingleCallSignature(type) {
                return getSingleSignature(type, 0 /* Call */, /*allowMembers*/ false);
            }
            function getSingleCallOrConstructSignature(type) {
                return getSingleSignature(type, 0 /* Call */, /*allowMembers*/ false) ||
                    getSingleSignature(type, 1 /* Construct */, /*allowMembers*/ false);
            }
            function getSingleSignature(type, kind, allowMembers) {
                if (type.flags & 524288 /* Object */) {
                    var resolved = resolveStructuredTypeMembers(type);
                    if (allowMembers || resolved.properties.length === 0 && !resolved.stringIndexInfo && !resolved.numberIndexInfo) {
                        if (kind === 0 /* Call */ && resolved.callSignatures.length === 1 && resolved.constructSignatures.length === 0) {
                            return resolved.callSignatures[0];
                        }
                        if (kind === 1 /* Construct */ && resolved.constructSignatures.length === 1 && resolved.callSignatures.length === 0) {
                            return resolved.constructSignatures[0];
                        }
                    }
                }
                return undefined;
            }
            // Instantiate a generic signature in the context of a non-generic signature (section 3.8.5 in TypeScript spec)
            function instantiateSignatureInContextOf(signature, contextualSignature, inferenceContext, compareTypes) {
                var context = createInferenceContext(signature.typeParameters, signature, 0 /* None */, compareTypes);
                // We clone the inferenceContext to avoid fixing. For example, when the source signature is <T>(x: T) => T[] and
                // the contextual signature is (...args: A) => B, we want to infer the element type of A's constraint (say 'any')
                // for T but leave it possible to later infer '[any]' back to A.
                var restType = getEffectiveRestType(contextualSignature);
                var mapper = inferenceContext && (restType && restType.flags & 262144 /* TypeParameter */ ? inferenceContext.nonFixingMapper : inferenceContext.mapper);
                var sourceSignature = mapper ? instantiateSignature(contextualSignature, mapper) : contextualSignature;
                applyToParameterTypes(sourceSignature, signature, function (source, target) {
                    // Type parameters from outer context referenced by source type are fixed by instantiation of the source type
                    inferTypes(context.inferences, source, target);
                });
                if (!inferenceContext) {
                    applyToReturnTypes(contextualSignature, signature, function (source, target) {
                        inferTypes(context.inferences, source, target, 32 /* ReturnType */);
                    });
                }
                return getSignatureInstantiation(signature, getInferredTypes(context), ts.isInJSFile(contextualSignature.declaration));
            }
            function inferJsxTypeArguments(node, signature, checkMode, context) {
                var paramType = getEffectiveFirstArgumentForJsxSignature(signature, node);
                var checkAttrType = checkExpressionWithContextualType(node.attributes, paramType, context, checkMode);
                inferTypes(context.inferences, checkAttrType, paramType);
                return getInferredTypes(context);
            }
            function inferTypeArguments(node, signature, args, checkMode, context) {
                if (ts.isJsxOpeningLikeElement(node)) {
                    return inferJsxTypeArguments(node, signature, checkMode, context);
                }
                // If a contextual type is available, infer from that type to the return type of the call expression. For
                // example, given a 'function wrap<T, U>(cb: (x: T) => U): (x: T) => U' and a call expression
                // 'let f: (x: string) => number = wrap(s => s.length)', we infer from the declared type of 'f' to the
                // return type of 'wrap'.
                if (node.kind !== 157 /* Decorator */) {
                    var contextualType = getContextualType(node);
                    if (contextualType) {
                        // We clone the inference context to avoid disturbing a resolution in progress for an
                        // outer call expression. Effectively we just want a snapshot of whatever has been
                        // inferred for any outer call expression so far.
                        var outerContext = getInferenceContext(node);
                        var outerMapper = getMapperFromContext(cloneInferenceContext(outerContext, 1 /* NoDefault */));
                        var instantiatedType = instantiateType(contextualType, outerMapper);
                        // If the contextual type is a generic function type with a single call signature, we
                        // instantiate the type with its own type parameters and type arguments. This ensures that
                        // the type parameters are not erased to type any during type inference such that they can
                        // be inferred as actual types from the contextual type. For example:
                        //   declare function arrayMap<T, U>(f: (x: T) => U): (a: T[]) => U[];
                        //   const boxElements: <A>(a: A[]) => { value: A }[] = arrayMap(value => ({ value }));
                        // Above, the type of the 'value' parameter is inferred to be 'A'.
                        var contextualSignature = getSingleCallSignature(instantiatedType);
                        var inferenceSourceType = contextualSignature && contextualSignature.typeParameters ?
                            getOrCreateTypeFromSignature(getSignatureInstantiationWithoutFillingInTypeArguments(contextualSignature, contextualSignature.typeParameters)) :
                            instantiatedType;
                        var inferenceTargetType = getReturnTypeOfSignature(signature);
                        // Inferences made from return types have lower priority than all other inferences.
                        inferTypes(context.inferences, inferenceSourceType, inferenceTargetType, 32 /* ReturnType */);
                        // Create a type mapper for instantiating generic contextual types using the inferences made
                        // from the return type. We need a separate inference pass here because (a) instantiation of
                        // the source type uses the outer context's return mapper (which excludes inferences made from
                        // outer arguments), and (b) we don't want any further inferences going into this context.
                        var returnContext = createInferenceContext(signature.typeParameters, signature, context.flags);
                        var returnSourceType = instantiateType(contextualType, outerContext && outerContext.returnMapper);
                        inferTypes(returnContext.inferences, returnSourceType, inferenceTargetType);
                        context.returnMapper = ts.some(returnContext.inferences, hasInferenceCandidates) ? getMapperFromContext(cloneInferredPartOfContext(returnContext)) : undefined;
                    }
                }
                var thisType = getThisTypeOfSignature(signature);
                if (thisType) {
                    var thisArgumentNode = getThisArgumentOfCall(node);
                    var thisArgumentType = thisArgumentNode ? checkExpression(thisArgumentNode) : voidType;
                    inferTypes(context.inferences, thisArgumentType, thisType);
                }
                var restType = getNonArrayRestType(signature);
                var argCount = restType ? Math.min(getParameterCount(signature) - 1, args.length) : args.length;
                for (var i = 0; i < argCount; i++) {
                    var arg = args[i];
                    if (arg.kind !== 215 /* OmittedExpression */) {
                        var paramType = getTypeAtPosition(signature, i);
                        var argType = checkExpressionWithContextualType(arg, paramType, context, checkMode);
                        inferTypes(context.inferences, argType, paramType);
                    }
                }
                if (restType) {
                    var spreadType = getSpreadArgumentType(args, argCount, args.length, restType, context);
                    inferTypes(context.inferences, spreadType, restType);
                }
                return getInferredTypes(context);
            }
            function getArrayifiedType(type) {
                return type.flags & 1048576 /* Union */ ? mapType(type, getArrayifiedType) :
                    type.flags & (1 /* Any */ | 63176704 /* Instantiable */) || isMutableArrayOrTuple(type) ? type :
                        isTupleType(type) ? createTupleType(getTypeArguments(type), type.target.minLength, type.target.hasRestElement, /*readonly*/ false, type.target.associatedNames) :
                            createArrayType(getIndexedAccessType(type, numberType));
            }
            function getSpreadArgumentType(args, index, argCount, restType, context) {
                if (index >= argCount - 1) {
                    var arg = args[argCount - 1];
                    if (isSpreadArgument(arg)) {
                        // We are inferring from a spread expression in the last argument position, i.e. both the parameter
                        // and the argument are ...x forms.
                        return arg.kind === 220 /* SyntheticExpression */ ?
                            createArrayType(arg.type) :
                            getArrayifiedType(checkExpressionWithContextualType(arg.expression, restType, context, 0 /* Normal */));
                    }
                }
                var types = [];
                var spreadIndex = -1;
                for (var i = index; i < argCount; i++) {
                    var contextualType = getIndexedAccessType(restType, getLiteralType(i - index));
                    var argType = checkExpressionWithContextualType(args[i], contextualType, context, 0 /* Normal */);
                    if (spreadIndex < 0 && isSpreadArgument(args[i])) {
                        spreadIndex = i - index;
                    }
                    var hasPrimitiveContextualType = maybeTypeOfKind(contextualType, 131068 /* Primitive */ | 4194304 /* Index */);
                    types.push(hasPrimitiveContextualType ? getRegularTypeOfLiteralType(argType) : getWidenedLiteralType(argType));
                }
                return spreadIndex < 0 ?
                    createTupleType(types) :
                    createTupleType(ts.append(types.slice(0, spreadIndex), getUnionType(types.slice(spreadIndex))), spreadIndex, /*hasRestElement*/ true);
            }
            function checkTypeArguments(signature, typeArgumentNodes, reportErrors, headMessage) {
                var isJavascript = ts.isInJSFile(signature.declaration);
                var typeParameters = signature.typeParameters;
                var typeArgumentTypes = fillMissingTypeArguments(ts.map(typeArgumentNodes, getTypeFromTypeNode), typeParameters, getMinTypeArgumentCount(typeParameters), isJavascript);
                var mapper;
                for (var i = 0; i < typeArgumentNodes.length; i++) {
                    ts.Debug.assert(typeParameters[i] !== undefined, "Should not call checkTypeArguments with too many type arguments");
                    var constraint = getConstraintOfTypeParameter(typeParameters[i]);
                    if (constraint) {
                        var errorInfo = reportErrors && headMessage ? (function () { return ts.chainDiagnosticMessages(/*details*/ undefined, ts.Diagnostics.Type_0_does_not_satisfy_the_constraint_1); }) : undefined;
                        var typeArgumentHeadMessage = headMessage || ts.Diagnostics.Type_0_does_not_satisfy_the_constraint_1;
                        if (!mapper) {
                            mapper = createTypeMapper(typeParameters, typeArgumentTypes);
                        }
                        var typeArgument = typeArgumentTypes[i];
                        if (!checkTypeAssignableTo(typeArgument, getTypeWithThisArgument(instantiateType(constraint, mapper), typeArgument), reportErrors ? typeArgumentNodes[i] : undefined, typeArgumentHeadMessage, errorInfo)) {
                            return undefined;
                        }
                    }
                }
                return typeArgumentTypes;
            }
            function getJsxReferenceKind(node) {
                if (isJsxIntrinsicIdentifier(node.tagName)) {
                    return 2 /* Mixed */;
                }
                var tagType = getApparentType(checkExpression(node.tagName));
                if (ts.length(getSignaturesOfType(tagType, 1 /* Construct */))) {
                    return 0 /* Component */;
                }
                if (ts.length(getSignaturesOfType(tagType, 0 /* Call */))) {
                    return 1 /* Function */;
                }
                return 2 /* Mixed */;
            }
            /**
             * Check if the given signature can possibly be a signature called by the JSX opening-like element.
             * @param node a JSX opening-like element we are trying to figure its call signature
             * @param signature a candidate signature we are trying whether it is a call signature
             * @param relation a relationship to check parameter and argument type
             */
            function checkApplicableSignatureForJsxOpeningLikeElement(node, signature, relation, checkMode, reportErrors, containingMessageChain, errorOutputContainer) {
                // Stateless function components can have maximum of three arguments: "props", "context", and "updater".
                // However "context" and "updater" are implicit and can't be specify by users. Only the first parameter, props,
                // can be specified by users through attributes property.
                var paramType = getEffectiveFirstArgumentForJsxSignature(signature, node);
                var attributesType = checkExpressionWithContextualType(node.attributes, paramType, /*inferenceContext*/ undefined, checkMode);
                return checkTypeRelatedToAndOptionallyElaborate(attributesType, paramType, relation, reportErrors ? node.tagName : undefined, node.attributes, 
                /*headMessage*/ undefined, containingMessageChain, errorOutputContainer);
            }
            function getSignatureApplicabilityError(node, args, signature, relation, checkMode, reportErrors, containingMessageChain) {
                var errorOutputContainer = { errors: undefined, skipLogging: true };
                if (ts.isJsxOpeningLikeElement(node)) {
                    if (!checkApplicableSignatureForJsxOpeningLikeElement(node, signature, relation, checkMode, reportErrors, containingMessageChain, errorOutputContainer)) {
                        ts.Debug.assert(!reportErrors || !!errorOutputContainer.errors, "jsx should have errors when reporting errors");
                        return errorOutputContainer.errors || ts.emptyArray;
                    }
                    return undefined;
                }
                var thisType = getThisTypeOfSignature(signature);
                if (thisType && thisType !== voidType && node.kind !== 197 /* NewExpression */) {
                    // If the called expression is not of the form `x.f` or `x["f"]`, then sourceType = voidType
                    // If the signature's 'this' type is voidType, then the check is skipped -- anything is compatible.
                    // If the expression is a new expression, then the check is skipped.
                    var thisArgumentNode = getThisArgumentOfCall(node);
                    var thisArgumentType = void 0;
                    if (thisArgumentNode) {
                        thisArgumentType = checkExpression(thisArgumentNode);
                        if (ts.isOptionalChainRoot(thisArgumentNode.parent)) {
                            thisArgumentType = getNonNullableType(thisArgumentType);
                        }
                        else if (ts.isOptionalChain(thisArgumentNode.parent)) {
                            thisArgumentType = removeOptionalTypeMarker(thisArgumentType);
                        }
                    }
                    else {
                        thisArgumentType = voidType;
                    }
                    var errorNode = reportErrors ? (thisArgumentNode || node) : undefined;
                    var headMessage_1 = ts.Diagnostics.The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1;
                    if (!checkTypeRelatedTo(thisArgumentType, thisType, relation, errorNode, headMessage_1, containingMessageChain, errorOutputContainer)) {
                        ts.Debug.assert(!reportErrors || !!errorOutputContainer.errors, "this parameter should have errors when reporting errors");
                        return errorOutputContainer.errors || ts.emptyArray;
                    }
                }
                var headMessage = ts.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1;
                var restType = getNonArrayRestType(signature);
                var argCount = restType ? Math.min(getParameterCount(signature) - 1, args.length) : args.length;
                for (var i = 0; i < argCount; i++) {
                    var arg = args[i];
                    if (arg.kind !== 215 /* OmittedExpression */) {
                        var paramType = getTypeAtPosition(signature, i);
                        var argType = checkExpressionWithContextualType(arg, paramType, /*inferenceContext*/ undefined, checkMode);
                        // If one or more arguments are still excluded (as indicated by CheckMode.SkipContextSensitive),
                        // we obtain the regular type of any object literal arguments because we may not have inferred complete
                        // parameter types yet and therefore excess property checks may yield false positives (see #17041).
                        var checkArgType = checkMode & 4 /* SkipContextSensitive */ ? getRegularTypeOfObjectLiteral(argType) : argType;
                        if (!checkTypeRelatedToAndOptionallyElaborate(checkArgType, paramType, relation, reportErrors ? arg : undefined, arg, headMessage, containingMessageChain, errorOutputContainer)) {
                            ts.Debug.assert(!reportErrors || !!errorOutputContainer.errors, "parameter should have errors when reporting errors");
                            maybeAddMissingAwaitInfo(arg, checkArgType, paramType);
                            return errorOutputContainer.errors || ts.emptyArray;
                        }
                    }
                }
                if (restType) {
                    var spreadType = getSpreadArgumentType(args, argCount, args.length, restType, /*context*/ undefined);
                    var errorNode = reportErrors ? argCount < args.length ? args[argCount] : node : undefined;
                    if (!checkTypeRelatedTo(spreadType, restType, relation, errorNode, headMessage, /*containingMessageChain*/ undefined, errorOutputContainer)) {
                        ts.Debug.assert(!reportErrors || !!errorOutputContainer.errors, "rest parameter should have errors when reporting errors");
                        maybeAddMissingAwaitInfo(errorNode, spreadType, restType);
                        return errorOutputContainer.errors || ts.emptyArray;
                    }
                }
                return undefined;
                function maybeAddMissingAwaitInfo(errorNode, source, target) {
                    if (errorNode && reportErrors && errorOutputContainer.errors && errorOutputContainer.errors.length) {
                        // Bail if target is Promise-like---something else is wrong
                        if (getAwaitedTypeOfPromise(target)) {
                            return;
                        }
                        var awaitedTypeOfSource = getAwaitedTypeOfPromise(source);
                        if (awaitedTypeOfSource && isTypeRelatedTo(awaitedTypeOfSource, target, relation)) {
                            ts.addRelatedInfo(errorOutputContainer.errors[0], ts.createDiagnosticForNode(errorNode, ts.Diagnostics.Did_you_forget_to_use_await));
                        }
                    }
                }
            }
            /**
             * Returns the this argument in calls like x.f(...) and x[f](...). Undefined otherwise.
             */
            function getThisArgumentOfCall(node) {
                if (node.kind === 196 /* CallExpression */) {
                    var callee = ts.skipOuterExpressions(node.expression);
                    if (ts.isAccessExpression(callee)) {
                        return callee.expression;
                    }
                }
            }
            function createSyntheticExpression(parent, type, isSpread) {
                var result = ts.createNode(220 /* SyntheticExpression */, parent.pos, parent.end);
                result.parent = parent;
                result.type = type;
                result.isSpread = isSpread || false;
                return result;
            }
            /**
             * Returns the effective arguments for an expression that works like a function invocation.
             */
            function getEffectiveCallArguments(node) {
                if (node.kind === 198 /* TaggedTemplateExpression */) {
                    var template = node.template;
                    var args_3 = [createSyntheticExpression(template, getGlobalTemplateStringsArrayType())];
                    if (template.kind === 211 /* TemplateExpression */) {
                        ts.forEach(template.templateSpans, function (span) {
                            args_3.push(span.expression);
                        });
                    }
                    return args_3;
                }
                if (node.kind === 157 /* Decorator */) {
                    return getEffectiveDecoratorArguments(node);
                }
                if (ts.isJsxOpeningLikeElement(node)) {
                    return node.attributes.properties.length > 0 || (ts.isJsxOpeningElement(node) && node.parent.children.length > 0) ? [node.attributes] : ts.emptyArray;
                }
                var args = node.arguments || ts.emptyArray;
                var length = args.length;
                if (length && isSpreadArgument(args[length - 1]) && getSpreadArgumentIndex(args) === length - 1) {
                    // We have a spread argument in the last position and no other spread arguments. If the type
                    // of the argument is a tuple type, spread the tuple elements into the argument list. We can
                    // call checkExpressionCached because spread expressions never have a contextual type.
                    var spreadArgument_1 = args[length - 1];
                    var type = flowLoopCount ? checkExpression(spreadArgument_1.expression) : checkExpressionCached(spreadArgument_1.expression);
                    if (isTupleType(type)) {
                        var typeArguments = getTypeArguments(type);
                        var restIndex_2 = type.target.hasRestElement ? typeArguments.length - 1 : -1;
                        var syntheticArgs = ts.map(typeArguments, function (t, i) { return createSyntheticExpression(spreadArgument_1, t, /*isSpread*/ i === restIndex_2); });
                        return ts.concatenate(args.slice(0, length - 1), syntheticArgs);
                    }
                }
                return args;
            }
            /**
             * Returns the synthetic argument list for a decorator invocation.
             */
            function getEffectiveDecoratorArguments(node) {
                var parent = node.parent;
                var expr = node.expression;
                switch (parent.kind) {
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                        // For a class decorator, the `target` is the type of the class (e.g. the
                        // "static" or "constructor" side of the class).
                        return [
                            createSyntheticExpression(expr, getTypeOfSymbol(getSymbolOfNode(parent)))
                        ];
                    case 156 /* Parameter */:
                        // A parameter declaration decorator will have three arguments (see
                        // `ParameterDecorator` in core.d.ts).
                        var func = parent.parent;
                        return [
                            createSyntheticExpression(expr, parent.parent.kind === 162 /* Constructor */ ? getTypeOfSymbol(getSymbolOfNode(func)) : errorType),
                            createSyntheticExpression(expr, anyType),
                            createSyntheticExpression(expr, numberType)
                        ];
                    case 159 /* PropertyDeclaration */:
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        // A method or accessor declaration decorator will have two or three arguments (see
                        // `PropertyDecorator` and `MethodDecorator` in core.d.ts). If we are emitting decorators
                        // for ES3, we will only pass two arguments.
                        var hasPropDesc = parent.kind !== 159 /* PropertyDeclaration */ && languageVersion !== 0 /* ES3 */;
                        return [
                            createSyntheticExpression(expr, getParentTypeOfClassElement(parent)),
                            createSyntheticExpression(expr, getClassElementPropertyKeyType(parent)),
                            createSyntheticExpression(expr, hasPropDesc ? createTypedPropertyDescriptorType(getTypeOfNode(parent)) : anyType)
                        ];
                }
                return ts.Debug.fail();
            }
            /**
             * Returns the argument count for a decorator node that works like a function invocation.
             */
            function getDecoratorArgumentCount(node, signature) {
                switch (node.parent.kind) {
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                        return 1;
                    case 159 /* PropertyDeclaration */:
                        return 2;
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        // For ES3 or decorators with only two parameters we supply only two arguments
                        return languageVersion === 0 /* ES3 */ || signature.parameters.length <= 2 ? 2 : 3;
                    case 156 /* Parameter */:
                        return 3;
                    default:
                        return ts.Debug.fail();
                }
            }
            function getDiagnosticSpanForCallNode(node, doNotIncludeArguments) {
                var start;
                var length;
                var sourceFile = ts.getSourceFileOfNode(node);
                if (ts.isPropertyAccessExpression(node.expression)) {
                    var nameSpan = ts.getErrorSpanForNode(sourceFile, node.expression.name);
                    start = nameSpan.start;
                    length = doNotIncludeArguments ? nameSpan.length : node.end - start;
                }
                else {
                    var expressionSpan = ts.getErrorSpanForNode(sourceFile, node.expression);
                    start = expressionSpan.start;
                    length = doNotIncludeArguments ? expressionSpan.length : node.end - start;
                }
                return { start: start, length: length, sourceFile: sourceFile };
            }
            function getDiagnosticForCallNode(node, message, arg0, arg1, arg2, arg3) {
                if (ts.isCallExpression(node)) {
                    var _a = getDiagnosticSpanForCallNode(node), sourceFile = _a.sourceFile, start = _a.start, length_4 = _a.length;
                    return ts.createFileDiagnostic(sourceFile, start, length_4, message, arg0, arg1, arg2, arg3);
                }
                else {
                    return ts.createDiagnosticForNode(node, message, arg0, arg1, arg2, arg3);
                }
            }
            function getArgumentArityError(node, signatures, args) {
                var min = Number.POSITIVE_INFINITY;
                var max = Number.NEGATIVE_INFINITY;
                var belowArgCount = Number.NEGATIVE_INFINITY;
                var aboveArgCount = Number.POSITIVE_INFINITY;
                var argCount = args.length;
                var closestSignature;
                for (var _i = 0, signatures_8 = signatures; _i < signatures_8.length; _i++) {
                    var sig = signatures_8[_i];
                    var minCount = getMinArgumentCount(sig);
                    var maxCount = getParameterCount(sig);
                    if (minCount < argCount && minCount > belowArgCount)
                        belowArgCount = minCount;
                    if (argCount < maxCount && maxCount < aboveArgCount)
                        aboveArgCount = maxCount;
                    if (minCount < min) {
                        min = minCount;
                        closestSignature = sig;
                    }
                    max = Math.max(max, maxCount);
                }
                var hasRestParameter = ts.some(signatures, hasEffectiveRestParameter);
                var paramRange = hasRestParameter ? min :
                    min < max ? min + "-" + max :
                        min;
                var hasSpreadArgument = getSpreadArgumentIndex(args) > -1;
                if (argCount <= max && hasSpreadArgument) {
                    argCount--;
                }
                var spanArray;
                var related;
                var error = hasRestParameter || hasSpreadArgument ? hasRestParameter && hasSpreadArgument ? ts.Diagnostics.Expected_at_least_0_arguments_but_got_1_or_more :
                    hasRestParameter ? ts.Diagnostics.Expected_at_least_0_arguments_but_got_1 :
                        ts.Diagnostics.Expected_0_arguments_but_got_1_or_more : ts.Diagnostics.Expected_0_arguments_but_got_1;
                if (closestSignature && getMinArgumentCount(closestSignature) > argCount && closestSignature.declaration) {
                    var paramDecl = closestSignature.declaration.parameters[closestSignature.thisParameter ? argCount + 1 : argCount];
                    if (paramDecl) {
                        related = ts.createDiagnosticForNode(paramDecl, ts.isBindingPattern(paramDecl.name) ? ts.Diagnostics.An_argument_matching_this_binding_pattern_was_not_provided : ts.Diagnostics.An_argument_for_0_was_not_provided, !paramDecl.name ? argCount : !ts.isBindingPattern(paramDecl.name) ? ts.idText(ts.getFirstIdentifier(paramDecl.name)) : undefined);
                    }
                }
                if (min < argCount && argCount < max) {
                    return getDiagnosticForCallNode(node, ts.Diagnostics.No_overload_expects_0_arguments_but_overloads_do_exist_that_expect_either_1_or_2_arguments, argCount, belowArgCount, aboveArgCount);
                }
                if (!hasSpreadArgument && argCount < min) {
                    var diagnostic_1 = getDiagnosticForCallNode(node, error, paramRange, argCount);
                    return related ? ts.addRelatedInfo(diagnostic_1, related) : diagnostic_1;
                }
                if (hasRestParameter || hasSpreadArgument) {
                    spanArray = ts.createNodeArray(args);
                    if (hasSpreadArgument && argCount) {
                        var nextArg = ts.elementAt(args, getSpreadArgumentIndex(args) + 1) || undefined;
                        spanArray = ts.createNodeArray(args.slice(max > argCount && nextArg ? args.indexOf(nextArg) : Math.min(max, args.length - 1)));
                    }
                }
                else {
                    spanArray = ts.createNodeArray(args.slice(max));
                }
                spanArray.pos = ts.first(spanArray).pos;
                spanArray.end = ts.last(spanArray).end;
                if (spanArray.end === spanArray.pos) {
                    spanArray.end++;
                }
                var diagnostic = ts.createDiagnosticForNodeArray(ts.getSourceFileOfNode(node), spanArray, error, paramRange, argCount);
                return related ? ts.addRelatedInfo(diagnostic, related) : diagnostic;
            }
            function getTypeArgumentArityError(node, signatures, typeArguments) {
                var argCount = typeArguments.length;
                // No overloads exist
                if (signatures.length === 1) {
                    var sig = signatures[0];
                    var min_1 = getMinTypeArgumentCount(sig.typeParameters);
                    var max = ts.length(sig.typeParameters);
                    return ts.createDiagnosticForNodeArray(ts.getSourceFileOfNode(node), typeArguments, ts.Diagnostics.Expected_0_type_arguments_but_got_1, min_1 < max ? min_1 + "-" + max : min_1, argCount);
                }
                // Overloads exist
                var belowArgCount = -Infinity;
                var aboveArgCount = Infinity;
                for (var _i = 0, signatures_9 = signatures; _i < signatures_9.length; _i++) {
                    var sig = signatures_9[_i];
                    var min_2 = getMinTypeArgumentCount(sig.typeParameters);
                    var max = ts.length(sig.typeParameters);
                    if (min_2 > argCount) {
                        aboveArgCount = Math.min(aboveArgCount, min_2);
                    }
                    else if (max < argCount) {
                        belowArgCount = Math.max(belowArgCount, max);
                    }
                }
                if (belowArgCount !== -Infinity && aboveArgCount !== Infinity) {
                    return ts.createDiagnosticForNodeArray(ts.getSourceFileOfNode(node), typeArguments, ts.Diagnostics.No_overload_expects_0_type_arguments_but_overloads_do_exist_that_expect_either_1_or_2_type_arguments, argCount, belowArgCount, aboveArgCount);
                }
                return ts.createDiagnosticForNodeArray(ts.getSourceFileOfNode(node), typeArguments, ts.Diagnostics.Expected_0_type_arguments_but_got_1, belowArgCount === -Infinity ? aboveArgCount : belowArgCount, argCount);
            }
            function resolveCall(node, signatures, candidatesOutArray, checkMode, callChainFlags, fallbackError) {
                var isTaggedTemplate = node.kind === 198 /* TaggedTemplateExpression */;
                var isDecorator = node.kind === 157 /* Decorator */;
                var isJsxOpeningOrSelfClosingElement = ts.isJsxOpeningLikeElement(node);
                var reportErrors = !candidatesOutArray;
                var typeArguments;
                if (!isDecorator) {
                    typeArguments = node.typeArguments;
                    // We already perform checking on the type arguments on the class declaration itself.
                    if (isTaggedTemplate || isJsxOpeningOrSelfClosingElement || node.expression.kind !== 102 /* SuperKeyword */) {
                        ts.forEach(typeArguments, checkSourceElement);
                    }
                }
                var candidates = candidatesOutArray || [];
                // reorderCandidates fills up the candidates array directly
                reorderCandidates(signatures, candidates, callChainFlags);
                if (!candidates.length) {
                    if (reportErrors) {
                        diagnostics.add(getDiagnosticForCallNode(node, ts.Diagnostics.Call_target_does_not_contain_any_signatures));
                    }
                    return resolveErrorCall(node);
                }
                var args = getEffectiveCallArguments(node);
                // The excludeArgument array contains true for each context sensitive argument (an argument
                // is context sensitive it is susceptible to a one-time permanent contextual typing).
                //
                // The idea is that we will perform type argument inference & assignability checking once
                // without using the susceptible parameters that are functions, and once more for those
                // parameters, contextually typing each as we go along.
                //
                // For a tagged template, then the first argument be 'undefined' if necessary because it
                // represents a TemplateStringsArray.
                //
                // For a decorator, no arguments are susceptible to contextual typing due to the fact
                // decorators are applied to a declaration by the emitter, and not to an expression.
                var isSingleNonGenericCandidate = candidates.length === 1 && !candidates[0].typeParameters;
                var argCheckMode = !isDecorator && !isSingleNonGenericCandidate && ts.some(args, isContextSensitive) ? 4 /* SkipContextSensitive */ : 0 /* Normal */;
                // The following variables are captured and modified by calls to chooseOverload.
                // If overload resolution or type argument inference fails, we want to report the
                // best error possible. The best error is one which says that an argument was not
                // assignable to a parameter. This implies that everything else about the overload
                // was fine. So if there is any overload that is only incorrect because of an
                // argument, we will report an error on that one.
                //
                //     function foo(s: string): void;
                //     function foo(n: number): void; // Report argument error on this overload
                //     function foo(): void;
                //     foo(true);
                //
                // If none of the overloads even made it that far, there are two possibilities.
                // There was a problem with type arguments for some overload, in which case
                // report an error on that. Or none of the overloads even had correct arity,
                // in which case give an arity error.
                //
                //     function foo<T extends string>(x: T): void; // Report type argument error
                //     function foo(): void;
                //     foo<number>(0);
                //
                var candidatesForArgumentError;
                var candidateForArgumentArityError;
                var candidateForTypeArgumentError;
                var result;
                // If we are in signature help, a trailing comma indicates that we intend to provide another argument,
                // so we will only accept overloads with arity at least 1 higher than the current number of provided arguments.
                var signatureHelpTrailingComma = !!(checkMode & 16 /* IsForSignatureHelp */) && node.kind === 196 /* CallExpression */ && node.arguments.hasTrailingComma;
                // Section 4.12.1:
                // if the candidate list contains one or more signatures for which the type of each argument
                // expression is a subtype of each corresponding parameter type, the return type of the first
                // of those signatures becomes the return type of the function call.
                // Otherwise, the return type of the first signature in the candidate list becomes the return
                // type of the function call.
                //
                // Whether the call is an error is determined by assignability of the arguments. The subtype pass
                // is just important for choosing the best signature. So in the case where there is only one
                // signature, the subtype pass is useless. So skipping it is an optimization.
                if (candidates.length > 1) {
                    result = chooseOverload(candidates, subtypeRelation, signatureHelpTrailingComma);
                }
                if (!result) {
                    result = chooseOverload(candidates, assignableRelation, signatureHelpTrailingComma);
                }
                if (result) {
                    return result;
                }
                // No signatures were applicable. Now report errors based on the last applicable signature with
                // no arguments excluded from assignability checks.
                // If candidate is undefined, it means that no candidates had a suitable arity. In that case,
                // skip the checkApplicableSignature check.
                if (reportErrors) {
                    if (candidatesForArgumentError) {
                        if (candidatesForArgumentError.length === 1 || candidatesForArgumentError.length > 3) {
                            var last_2 = candidatesForArgumentError[candidatesForArgumentError.length - 1];
                            var chain_1;
                            if (candidatesForArgumentError.length > 3) {
                                chain_1 = ts.chainDiagnosticMessages(chain_1, ts.Diagnostics.The_last_overload_gave_the_following_error);
                                chain_1 = ts.chainDiagnosticMessages(chain_1, ts.Diagnostics.No_overload_matches_this_call);
                            }
                            var diags = getSignatureApplicabilityError(node, args, last_2, assignableRelation, 0 /* Normal */, /*reportErrors*/ true, function () { return chain_1; });
                            if (diags) {
                                for (var _i = 0, diags_1 = diags; _i < diags_1.length; _i++) {
                                    var d = diags_1[_i];
                                    if (last_2.declaration && candidatesForArgumentError.length > 3) {
                                        ts.addRelatedInfo(d, ts.createDiagnosticForNode(last_2.declaration, ts.Diagnostics.The_last_overload_is_declared_here));
                                    }
                                    diagnostics.add(d);
                                }
                            }
                            else {
                                ts.Debug.fail("No error for last overload signature");
                            }
                        }
                        else {
                            var allDiagnostics = [];
                            var max = 0;
                            var min_3 = Number.MAX_VALUE;
                            var minIndex = 0;
                            var i_1 = 0;
                            var _loop_15 = function (c) {
                                var chain_2 = function () { return ts.chainDiagnosticMessages(/*details*/ undefined, ts.Diagnostics.Overload_0_of_1_2_gave_the_following_error, i_1 + 1, candidates.length, signatureToString(c)); };
                                var diags_2 = getSignatureApplicabilityError(node, args, c, assignableRelation, 0 /* Normal */, /*reportErrors*/ true, chain_2);
                                if (diags_2) {
                                    if (diags_2.length <= min_3) {
                                        min_3 = diags_2.length;
                                        minIndex = i_1;
                                    }
                                    max = Math.max(max, diags_2.length);
                                    allDiagnostics.push(diags_2);
                                }
                                else {
                                    ts.Debug.fail("No error for 3 or fewer overload signatures");
                                }
                                i_1++;
                            };
                            for (var _a = 0, candidatesForArgumentError_1 = candidatesForArgumentError; _a < candidatesForArgumentError_1.length; _a++) {
                                var c = candidatesForArgumentError_1[_a];
                                _loop_15(c);
                            }
                            var diags_3 = max > 1 ? allDiagnostics[minIndex] : ts.flatten(allDiagnostics);
                            ts.Debug.assert(diags_3.length > 0, "No errors reported for 3 or fewer overload signatures");
                            var chain = ts.chainDiagnosticMessages(ts.map(diags_3, function (d) { return typeof d.messageText === "string" ? d : d.messageText; }), ts.Diagnostics.No_overload_matches_this_call);
                            var related = ts.flatMap(diags_3, function (d) { return d.relatedInformation; });
                            if (ts.every(diags_3, function (d) { return d.start === diags_3[0].start && d.length === diags_3[0].length && d.file === diags_3[0].file; })) {
                                var _b = diags_3[0], file = _b.file, start = _b.start, length_5 = _b.length;
                                diagnostics.add({ file: file, start: start, length: length_5, code: chain.code, category: chain.category, messageText: chain, relatedInformation: related });
                            }
                            else {
                                diagnostics.add(ts.createDiagnosticForNodeFromMessageChain(node, chain, related));
                            }
                        }
                    }
                    else if (candidateForArgumentArityError) {
                        diagnostics.add(getArgumentArityError(node, [candidateForArgumentArityError], args));
                    }
                    else if (candidateForTypeArgumentError) {
                        checkTypeArguments(candidateForTypeArgumentError, node.typeArguments, /*reportErrors*/ true, fallbackError);
                    }
                    else {
                        var signaturesWithCorrectTypeArgumentArity = ts.filter(signatures, function (s) { return hasCorrectTypeArgumentArity(s, typeArguments); });
                        if (signaturesWithCorrectTypeArgumentArity.length === 0) {
                            diagnostics.add(getTypeArgumentArityError(node, signatures, typeArguments));
                        }
                        else if (!isDecorator) {
                            diagnostics.add(getArgumentArityError(node, signaturesWithCorrectTypeArgumentArity, args));
                        }
                        else if (fallbackError) {
                            diagnostics.add(getDiagnosticForCallNode(node, fallbackError));
                        }
                    }
                }
                return produceDiagnostics || !args ? resolveErrorCall(node) : getCandidateForOverloadFailure(node, candidates, args, !!candidatesOutArray);
                function chooseOverload(candidates, relation, signatureHelpTrailingComma) {
                    if (signatureHelpTrailingComma === void 0) {
                        signatureHelpTrailingComma = false;
                    }
                    candidatesForArgumentError = undefined;
                    candidateForArgumentArityError = undefined;
                    candidateForTypeArgumentError = undefined;
                    if (isSingleNonGenericCandidate) {
                        var candidate = candidates[0];
                        if (ts.some(typeArguments) || !hasCorrectArity(node, args, candidate, signatureHelpTrailingComma)) {
                            return undefined;
                        }
                        if (getSignatureApplicabilityError(node, args, candidate, relation, 0 /* Normal */, /*reportErrors*/ false, /*containingMessageChain*/ undefined)) {
                            candidatesForArgumentError = [candidate];
                            return undefined;
                        }
                        return candidate;
                    }
                    for (var candidateIndex = 0; candidateIndex < candidates.length; candidateIndex++) {
                        var candidate = candidates[candidateIndex];
                        if (!hasCorrectTypeArgumentArity(candidate, typeArguments) || !hasCorrectArity(node, args, candidate, signatureHelpTrailingComma)) {
                            continue;
                        }
                        var checkCandidate = void 0;
                        var inferenceContext = void 0;
                        if (candidate.typeParameters) {
                            var typeArgumentTypes = void 0;
                            if (ts.some(typeArguments)) {
                                typeArgumentTypes = checkTypeArguments(candidate, typeArguments, /*reportErrors*/ false);
                                if (!typeArgumentTypes) {
                                    candidateForTypeArgumentError = candidate;
                                    continue;
                                }
                            }
                            else {
                                inferenceContext = createInferenceContext(candidate.typeParameters, candidate, /*flags*/ ts.isInJSFile(node) ? 2 /* AnyDefault */ : 0 /* None */);
                                typeArgumentTypes = inferTypeArguments(node, candidate, args, argCheckMode | 8 /* SkipGenericFunctions */, inferenceContext);
                                argCheckMode |= inferenceContext.flags & 4 /* SkippedGenericFunction */ ? 8 /* SkipGenericFunctions */ : 0 /* Normal */;
                            }
                            checkCandidate = getSignatureInstantiation(candidate, typeArgumentTypes, ts.isInJSFile(candidate.declaration), inferenceContext && inferenceContext.inferredTypeParameters);
                            // If the original signature has a generic rest type, instantiation may produce a
                            // signature with different arity and we need to perform another arity check.
                            if (getNonArrayRestType(candidate) && !hasCorrectArity(node, args, checkCandidate, signatureHelpTrailingComma)) {
                                candidateForArgumentArityError = checkCandidate;
                                continue;
                            }
                        }
                        else {
                            checkCandidate = candidate;
                        }
                        if (getSignatureApplicabilityError(node, args, checkCandidate, relation, argCheckMode, /*reportErrors*/ false, /*containingMessageChain*/ undefined)) {
                            // Give preference to error candidates that have no rest parameters (as they are more specific)
                            (candidatesForArgumentError || (candidatesForArgumentError = [])).push(checkCandidate);
                            continue;
                        }
                        if (argCheckMode) {
                            // If one or more context sensitive arguments were excluded, we start including
                            // them now (and keeping do so for any subsequent candidates) and perform a second
                            // round of type inference and applicability checking for this particular candidate.
                            argCheckMode = 0 /* Normal */;
                            if (inferenceContext) {
                                var typeArgumentTypes = inferTypeArguments(node, candidate, args, argCheckMode, inferenceContext);
                                checkCandidate = getSignatureInstantiation(candidate, typeArgumentTypes, ts.isInJSFile(candidate.declaration), inferenceContext && inferenceContext.inferredTypeParameters);
                                // If the original signature has a generic rest type, instantiation may produce a
                                // signature with different arity and we need to perform another arity check.
                                if (getNonArrayRestType(candidate) && !hasCorrectArity(node, args, checkCandidate, signatureHelpTrailingComma)) {
                                    candidateForArgumentArityError = checkCandidate;
                                    continue;
                                }
                            }
                            if (getSignatureApplicabilityError(node, args, checkCandidate, relation, argCheckMode, /*reportErrors*/ false, /*containingMessageChain*/ undefined)) {
                                // Give preference to error candidates that have no rest parameters (as they are more specific)
                                (candidatesForArgumentError || (candidatesForArgumentError = [])).push(checkCandidate);
                                continue;
                            }
                        }
                        candidates[candidateIndex] = checkCandidate;
                        return checkCandidate;
                    }
                    return undefined;
                }
            }
            // No signature was applicable. We have already reported the errors for the invalid signature.
            // If this is a type resolution session, e.g. Language Service, try to get better information than anySignature.
            function getCandidateForOverloadFailure(node, candidates, args, hasCandidatesOutArray) {
                ts.Debug.assert(candidates.length > 0); // Else should not have called this.
                // Normally we will combine overloads. Skip this if they have type parameters since that's hard to combine.
                // Don't do this if there is a `candidatesOutArray`,
                // because then we want the chosen best candidate to be one of the overloads, not a combination.
                return hasCandidatesOutArray || candidates.length === 1 || candidates.some(function (c) { return !!c.typeParameters; })
                    ? pickLongestCandidateSignature(node, candidates, args)
                    : createUnionOfSignaturesForOverloadFailure(candidates);
            }
            function createUnionOfSignaturesForOverloadFailure(candidates) {
                var thisParameters = ts.mapDefined(candidates, function (c) { return c.thisParameter; });
                var thisParameter;
                if (thisParameters.length) {
                    thisParameter = createCombinedSymbolFromTypes(thisParameters, thisParameters.map(getTypeOfParameter));
                }
                var _a = ts.minAndMax(candidates, getNumNonRestParameters), minArgumentCount = _a.min, maxNonRestParam = _a.max;
                var parameters = [];
                var _loop_16 = function (i) {
                    var symbols = ts.mapDefined(candidates, function (s) {
                        return signatureHasRestParameter(s) ?
                            i < s.parameters.length - 1 ? s.parameters[i] : ts.last(s.parameters) :
                            i < s.parameters.length ? s.parameters[i] : undefined;
                    });
                    ts.Debug.assert(symbols.length !== 0);
                    parameters.push(createCombinedSymbolFromTypes(symbols, ts.mapDefined(candidates, function (candidate) { return tryGetTypeAtPosition(candidate, i); })));
                };
                for (var i = 0; i < maxNonRestParam; i++) {
                    _loop_16(i);
                }
                var restParameterSymbols = ts.mapDefined(candidates, function (c) { return signatureHasRestParameter(c) ? ts.last(c.parameters) : undefined; });
                var flags = 0 /* None */;
                if (restParameterSymbols.length !== 0) {
                    var type = createArrayType(getUnionType(ts.mapDefined(candidates, tryGetRestTypeOfSignature), 2 /* Subtype */));
                    parameters.push(createCombinedSymbolForOverloadFailure(restParameterSymbols, type));
                    flags |= 1 /* HasRestParameter */;
                }
                if (candidates.some(signatureHasLiteralTypes)) {
                    flags |= 2 /* HasLiteralTypes */;
                }
                return createSignature(candidates[0].declaration, 
                /*typeParameters*/ undefined, // Before calling this we tested for `!candidates.some(c => !!c.typeParameters)`.
                thisParameter, parameters, 
                /*resolvedReturnType*/ getIntersectionType(candidates.map(getReturnTypeOfSignature)), 
                /*typePredicate*/ undefined, minArgumentCount, flags);
            }
            function getNumNonRestParameters(signature) {
                var numParams = signature.parameters.length;
                return signatureHasRestParameter(signature) ? numParams - 1 : numParams;
            }
            function createCombinedSymbolFromTypes(sources, types) {
                return createCombinedSymbolForOverloadFailure(sources, getUnionType(types, 2 /* Subtype */));
            }
            function createCombinedSymbolForOverloadFailure(sources, type) {
                // This function is currently only used for erroneous overloads, so it's good enough to just use the first source.
                return createSymbolWithType(ts.first(sources), type);
            }
            function pickLongestCandidateSignature(node, candidates, args) {
                // Pick the longest signature. This way we can get a contextual type for cases like:
                //     declare function f(a: { xa: number; xb: number; }, b: number);
                //     f({ |
                // Also, use explicitly-supplied type arguments if they are provided, so we can get a contextual signature in cases like:
                //     declare function f<T>(k: keyof T);
                //     f<Foo>("
                var bestIndex = getLongestCandidateIndex(candidates, apparentArgumentCount === undefined ? args.length : apparentArgumentCount);
                var candidate = candidates[bestIndex];
                var typeParameters = candidate.typeParameters;
                if (!typeParameters) {
                    return candidate;
                }
                var typeArgumentNodes = callLikeExpressionMayHaveTypeArguments(node) ? node.typeArguments : undefined;
                var instantiated = typeArgumentNodes
                    ? createSignatureInstantiation(candidate, getTypeArgumentsFromNodes(typeArgumentNodes, typeParameters, ts.isInJSFile(node)))
                    : inferSignatureInstantiationForOverloadFailure(node, typeParameters, candidate, args);
                candidates[bestIndex] = instantiated;
                return instantiated;
            }
            function getTypeArgumentsFromNodes(typeArgumentNodes, typeParameters, isJs) {
                var typeArguments = typeArgumentNodes.map(getTypeOfNode);
                while (typeArguments.length > typeParameters.length) {
                    typeArguments.pop();
                }
                while (typeArguments.length < typeParameters.length) {
                    typeArguments.push(getConstraintOfTypeParameter(typeParameters[typeArguments.length]) || getDefaultTypeArgumentType(isJs));
                }
                return typeArguments;
            }
            function inferSignatureInstantiationForOverloadFailure(node, typeParameters, candidate, args) {
                var inferenceContext = createInferenceContext(typeParameters, candidate, /*flags*/ ts.isInJSFile(node) ? 2 /* AnyDefault */ : 0 /* None */);
                var typeArgumentTypes = inferTypeArguments(node, candidate, args, 4 /* SkipContextSensitive */ | 8 /* SkipGenericFunctions */, inferenceContext);
                return createSignatureInstantiation(candidate, typeArgumentTypes);
            }
            function getLongestCandidateIndex(candidates, argsCount) {
                var maxParamsIndex = -1;
                var maxParams = -1;
                for (var i = 0; i < candidates.length; i++) {
                    var candidate = candidates[i];
                    var paramCount = getParameterCount(candidate);
                    if (hasEffectiveRestParameter(candidate) || paramCount >= argsCount) {
                        return i;
                    }
                    if (paramCount > maxParams) {
                        maxParams = paramCount;
                        maxParamsIndex = i;
                    }
                }
                return maxParamsIndex;
            }
            function resolveCallExpression(node, candidatesOutArray, checkMode) {
                if (node.expression.kind === 102 /* SuperKeyword */) {
                    var superType = checkSuperExpression(node.expression);
                    if (isTypeAny(superType)) {
                        for (var _i = 0, _a = node.arguments; _i < _a.length; _i++) {
                            var arg = _a[_i];
                            checkExpression(arg); // Still visit arguments so they get marked for visibility, etc
                        }
                        return anySignature;
                    }
                    if (superType !== errorType) {
                        // In super call, the candidate signatures are the matching arity signatures of the base constructor function instantiated
                        // with the type arguments specified in the extends clause.
                        var baseTypeNode = ts.getEffectiveBaseTypeNode(ts.getContainingClass(node));
                        if (baseTypeNode) {
                            var baseConstructors = getInstantiatedConstructorsForTypeArguments(superType, baseTypeNode.typeArguments, baseTypeNode);
                            return resolveCall(node, baseConstructors, candidatesOutArray, checkMode, 0 /* None */);
                        }
                    }
                    return resolveUntypedCall(node);
                }
                var callChainFlags;
                var funcType = checkExpression(node.expression);
                if (ts.isCallChain(node)) {
                    var nonOptionalType = getOptionalExpressionType(funcType, node.expression);
                    callChainFlags = nonOptionalType === funcType ? 0 /* None */ :
                        ts.isOutermostOptionalChain(node) ? 8 /* IsOuterCallChain */ :
                            4 /* IsInnerCallChain */;
                    funcType = nonOptionalType;
                }
                else {
                    callChainFlags = 0 /* None */;
                }
                funcType = checkNonNullTypeWithReporter(funcType, node.expression, reportCannotInvokePossiblyNullOrUndefinedError);
                if (funcType === silentNeverType) {
                    return silentNeverSignature;
                }
                var apparentType = getApparentType(funcType);
                if (apparentType === errorType) {
                    // Another error has already been reported
                    return resolveErrorCall(node);
                }
                // Technically, this signatures list may be incomplete. We are taking the apparent type,
                // but we are not including call signatures that may have been added to the Object or
                // Function interface, since they have none by default. This is a bit of a leap of faith
                // that the user will not add any.
                var callSignatures = getSignaturesOfType(apparentType, 0 /* Call */);
                var numConstructSignatures = getSignaturesOfType(apparentType, 1 /* Construct */).length;
                // TS 1.0 Spec: 4.12
                // In an untyped function call no TypeArgs are permitted, Args can be any argument list, no contextual
                // types are provided for the argument expressions, and the result is always of type Any.
                if (isUntypedFunctionCall(funcType, apparentType, callSignatures.length, numConstructSignatures)) {
                    // The unknownType indicates that an error already occurred (and was reported).  No
                    // need to report another error in this case.
                    if (funcType !== errorType && node.typeArguments) {
                        error(node, ts.Diagnostics.Untyped_function_calls_may_not_accept_type_arguments);
                    }
                    return resolveUntypedCall(node);
                }
                // If FuncExpr's apparent type(section 3.8.1) is a function type, the call is a typed function call.
                // TypeScript employs overload resolution in typed function calls in order to support functions
                // with multiple call signatures.
                if (!callSignatures.length) {
                    if (numConstructSignatures) {
                        error(node, ts.Diagnostics.Value_of_type_0_is_not_callable_Did_you_mean_to_include_new, typeToString(funcType));
                    }
                    else {
                        var relatedInformation = void 0;
                        if (node.arguments.length === 1) {
                            var text = ts.getSourceFileOfNode(node).text;
                            if (ts.isLineBreak(text.charCodeAt(ts.skipTrivia(text, node.expression.end, /* stopAfterLineBreak */ true) - 1))) {
                                relatedInformation = ts.createDiagnosticForNode(node.expression, ts.Diagnostics.It_is_highly_likely_that_you_are_missing_a_semicolon);
                            }
                        }
                        invocationError(node.expression, apparentType, 0 /* Call */, relatedInformation);
                    }
                    return resolveErrorCall(node);
                }
                // When a call to a generic function is an argument to an outer call to a generic function for which
                // inference is in process, we have a choice to make. If the inner call relies on inferences made from
                // its contextual type to its return type, deferring the inner call processing allows the best possible
                // contextual type to accumulate. But if the outer call relies on inferences made from the return type of
                // the inner call, the inner call should be processed early. There's no sure way to know which choice is
                // right (only a full unification algorithm can determine that), so we resort to the following heuristic:
                // If no type arguments are specified in the inner call and at least one call signature is generic and
                // returns a function type, we choose to defer processing. This narrowly permits function composition
                // operators to flow inferences through return types, but otherwise processes calls right away. We
                // use the resolvingSignature singleton to indicate that we deferred processing. This result will be
                // propagated out and eventually turned into nonInferrableType (a type that is assignable to anything and
                // from which we never make inferences).
                if (checkMode & 8 /* SkipGenericFunctions */ && !node.typeArguments && callSignatures.some(isGenericFunctionReturningFunction)) {
                    skippedGenericFunction(node, checkMode);
                    return resolvingSignature;
                }
                // If the function is explicitly marked with `@class`, then it must be constructed.
                if (callSignatures.some(function (sig) { return ts.isInJSFile(sig.declaration) && !!ts.getJSDocClassTag(sig.declaration); })) {
                    error(node, ts.Diagnostics.Value_of_type_0_is_not_callable_Did_you_mean_to_include_new, typeToString(funcType));
                    return resolveErrorCall(node);
                }
                return resolveCall(node, callSignatures, candidatesOutArray, checkMode, callChainFlags);
            }
            function isGenericFunctionReturningFunction(signature) {
                return !!(signature.typeParameters && isFunctionType(getReturnTypeOfSignature(signature)));
            }
            /**
             * TS 1.0 spec: 4.12
             * If FuncExpr is of type Any, or of an object type that has no call or construct signatures
             * but is a subtype of the Function interface, the call is an untyped function call.
             */
            function isUntypedFunctionCall(funcType, apparentFuncType, numCallSignatures, numConstructSignatures) {
                // We exclude union types because we may have a union of function types that happen to have no common signatures.
                return isTypeAny(funcType) || isTypeAny(apparentFuncType) && !!(funcType.flags & 262144 /* TypeParameter */) ||
                    !numCallSignatures && !numConstructSignatures && !(apparentFuncType.flags & (1048576 /* Union */ | 131072 /* Never */)) && isTypeAssignableTo(funcType, globalFunctionType);
            }
            function resolveNewExpression(node, candidatesOutArray, checkMode) {
                if (node.arguments && languageVersion < 1 /* ES5 */) {
                    var spreadIndex = getSpreadArgumentIndex(node.arguments);
                    if (spreadIndex >= 0) {
                        error(node.arguments[spreadIndex], ts.Diagnostics.Spread_operator_in_new_expressions_is_only_available_when_targeting_ECMAScript_5_and_higher);
                    }
                }
                var expressionType = checkNonNullExpression(node.expression);
                if (expressionType === silentNeverType) {
                    return silentNeverSignature;
                }
                // If expressionType's apparent type(section 3.8.1) is an object type with one or
                // more construct signatures, the expression is processed in the same manner as a
                // function call, but using the construct signatures as the initial set of candidate
                // signatures for overload resolution. The result type of the function call becomes
                // the result type of the operation.
                expressionType = getApparentType(expressionType);
                if (expressionType === errorType) {
                    // Another error has already been reported
                    return resolveErrorCall(node);
                }
                // TS 1.0 spec: 4.11
                // If expressionType is of type Any, Args can be any argument
                // list and the result of the operation is of type Any.
                if (isTypeAny(expressionType)) {
                    if (node.typeArguments) {
                        error(node, ts.Diagnostics.Untyped_function_calls_may_not_accept_type_arguments);
                    }
                    return resolveUntypedCall(node);
                }
                // Technically, this signatures list may be incomplete. We are taking the apparent type,
                // but we are not including construct signatures that may have been added to the Object or
                // Function interface, since they have none by default. This is a bit of a leap of faith
                // that the user will not add any.
                var constructSignatures = getSignaturesOfType(expressionType, 1 /* Construct */);
                if (constructSignatures.length) {
                    if (!isConstructorAccessible(node, constructSignatures[0])) {
                        return resolveErrorCall(node);
                    }
                    // If the expression is a class of abstract type, then it cannot be instantiated.
                    // Note, only class declarations can be declared abstract.
                    // In the case of a merged class-module or class-interface declaration,
                    // only the class declaration node will have the Abstract flag set.
                    var valueDecl = expressionType.symbol && ts.getClassLikeDeclarationOfSymbol(expressionType.symbol);
                    if (valueDecl && ts.hasModifier(valueDecl, 128 /* Abstract */)) {
                        error(node, ts.Diagnostics.Cannot_create_an_instance_of_an_abstract_class);
                        return resolveErrorCall(node);
                    }
                    return resolveCall(node, constructSignatures, candidatesOutArray, checkMode, 0 /* None */);
                }
                // If expressionType's apparent type is an object type with no construct signatures but
                // one or more call signatures, the expression is processed as a function call. A compile-time
                // error occurs if the result of the function call is not Void. The type of the result of the
                // operation is Any. It is an error to have a Void this type.
                var callSignatures = getSignaturesOfType(expressionType, 0 /* Call */);
                if (callSignatures.length) {
                    var signature = resolveCall(node, callSignatures, candidatesOutArray, checkMode, 0 /* None */);
                    if (!noImplicitAny) {
                        if (signature.declaration && !isJSConstructor(signature.declaration) && getReturnTypeOfSignature(signature) !== voidType) {
                            error(node, ts.Diagnostics.Only_a_void_function_can_be_called_with_the_new_keyword);
                        }
                        if (getThisTypeOfSignature(signature) === voidType) {
                            error(node, ts.Diagnostics.A_function_that_is_called_with_the_new_keyword_cannot_have_a_this_type_that_is_void);
                        }
                    }
                    return signature;
                }
                invocationError(node.expression, expressionType, 1 /* Construct */);
                return resolveErrorCall(node);
            }
            function typeHasProtectedAccessibleBase(target, type) {
                var baseTypes = getBaseTypes(type);
                if (!ts.length(baseTypes)) {
                    return false;
                }
                var firstBase = baseTypes[0];
                if (firstBase.flags & 2097152 /* Intersection */) {
                    var types = firstBase.types;
                    var mixinFlags = findMixins(types);
                    var i = 0;
                    for (var _i = 0, _a = firstBase.types; _i < _a.length; _i++) {
                        var intersectionMember = _a[_i];
                        // We want to ignore mixin ctors
                        if (!mixinFlags[i]) {
                            if (ts.getObjectFlags(intersectionMember) & (1 /* Class */ | 2 /* Interface */)) {
                                if (intersectionMember.symbol === target) {
                                    return true;
                                }
                                if (typeHasProtectedAccessibleBase(target, intersectionMember)) {
                                    return true;
                                }
                            }
                        }
                        i++;
                    }
                    return false;
                }
                if (firstBase.symbol === target) {
                    return true;
                }
                return typeHasProtectedAccessibleBase(target, firstBase);
            }
            function isConstructorAccessible(node, signature) {
                if (!signature || !signature.declaration) {
                    return true;
                }
                var declaration = signature.declaration;
                var modifiers = ts.getSelectedModifierFlags(declaration, 24 /* NonPublicAccessibilityModifier */);
                // (1) Public constructors and (2) constructor functions are always accessible.
                if (!modifiers || declaration.kind !== 162 /* Constructor */) {
                    return true;
                }
                var declaringClassDeclaration = ts.getClassLikeDeclarationOfSymbol(declaration.parent.symbol);
                var declaringClass = getDeclaredTypeOfSymbol(declaration.parent.symbol);
                // A private or protected constructor can only be instantiated within its own class (or a subclass, for protected)
                if (!isNodeWithinClass(node, declaringClassDeclaration)) {
                    var containingClass = ts.getContainingClass(node);
                    if (containingClass && modifiers & 16 /* Protected */) {
                        var containingType = getTypeOfNode(containingClass);
                        if (typeHasProtectedAccessibleBase(declaration.parent.symbol, containingType)) {
                            return true;
                        }
                    }
                    if (modifiers & 8 /* Private */) {
                        error(node, ts.Diagnostics.Constructor_of_class_0_is_private_and_only_accessible_within_the_class_declaration, typeToString(declaringClass));
                    }
                    if (modifiers & 16 /* Protected */) {
                        error(node, ts.Diagnostics.Constructor_of_class_0_is_protected_and_only_accessible_within_the_class_declaration, typeToString(declaringClass));
                    }
                    return false;
                }
                return true;
            }
            function invocationErrorDetails(apparentType, kind) {
                var errorInfo;
                var isCall = kind === 0 /* Call */;
                var awaitedType = getAwaitedType(apparentType);
                var maybeMissingAwait = awaitedType && getSignaturesOfType(awaitedType, kind).length > 0;
                if (apparentType.flags & 1048576 /* Union */) {
                    var types = apparentType.types;
                    var hasSignatures = false;
                    for (var _i = 0, types_17 = types; _i < types_17.length; _i++) {
                        var constituent = types_17[_i];
                        var signatures = getSignaturesOfType(constituent, kind);
                        if (signatures.length !== 0) {
                            hasSignatures = true;
                            if (errorInfo) {
                                // Bail early if we already have an error, no chance of "No constituent of type is callable"
                                break;
                            }
                        }
                        else {
                            // Error on the first non callable constituent only
                            if (!errorInfo) {
                                errorInfo = ts.chainDiagnosticMessages(errorInfo, isCall ?
                                    ts.Diagnostics.Type_0_has_no_call_signatures :
                                    ts.Diagnostics.Type_0_has_no_construct_signatures, typeToString(constituent));
                                errorInfo = ts.chainDiagnosticMessages(errorInfo, isCall ?
                                    ts.Diagnostics.Not_all_constituents_of_type_0_are_callable :
                                    ts.Diagnostics.Not_all_constituents_of_type_0_are_constructable, typeToString(apparentType));
                            }
                            if (hasSignatures) {
                                // Bail early if we already found a siganture, no chance of "No constituent of type is callable"
                                break;
                            }
                        }
                    }
                    if (!hasSignatures) {
                        errorInfo = ts.chainDiagnosticMessages(
                        /* detials */ undefined, isCall ?
                            ts.Diagnostics.No_constituent_of_type_0_is_callable :
                            ts.Diagnostics.No_constituent_of_type_0_is_constructable, typeToString(apparentType));
                    }
                    if (!errorInfo) {
                        errorInfo = ts.chainDiagnosticMessages(errorInfo, isCall ?
                            ts.Diagnostics.Each_member_of_the_union_type_0_has_signatures_but_none_of_those_signatures_are_compatible_with_each_other :
                            ts.Diagnostics.Each_member_of_the_union_type_0_has_construct_signatures_but_none_of_those_signatures_are_compatible_with_each_other, typeToString(apparentType));
                    }
                }
                else {
                    errorInfo = ts.chainDiagnosticMessages(errorInfo, isCall ?
                        ts.Diagnostics.Type_0_has_no_call_signatures :
                        ts.Diagnostics.Type_0_has_no_construct_signatures, typeToString(apparentType));
                }
                return {
                    messageChain: ts.chainDiagnosticMessages(errorInfo, isCall ? ts.Diagnostics.This_expression_is_not_callable : ts.Diagnostics.This_expression_is_not_constructable),
                    relatedMessage: maybeMissingAwait ? ts.Diagnostics.Did_you_forget_to_use_await : undefined,
                };
            }
            function invocationError(errorTarget, apparentType, kind, relatedInformation) {
                var _a = invocationErrorDetails(apparentType, kind), messageChain = _a.messageChain, relatedInfo = _a.relatedMessage;
                var diagnostic = ts.createDiagnosticForNodeFromMessageChain(errorTarget, messageChain);
                if (relatedInfo) {
                    ts.addRelatedInfo(diagnostic, ts.createDiagnosticForNode(errorTarget, relatedInfo));
                }
                if (ts.isCallExpression(errorTarget.parent)) {
                    var _b = getDiagnosticSpanForCallNode(errorTarget.parent, /* doNotIncludeArguments */ true), start = _b.start, length_6 = _b.length;
                    diagnostic.start = start;
                    diagnostic.length = length_6;
                }
                diagnostics.add(diagnostic);
                invocationErrorRecovery(apparentType, kind, relatedInformation ? ts.addRelatedInfo(diagnostic, relatedInformation) : diagnostic);
            }
            function invocationErrorRecovery(apparentType, kind, diagnostic) {
                if (!apparentType.symbol) {
                    return;
                }
                var importNode = getSymbolLinks(apparentType.symbol).originatingImport;
                // Create a diagnostic on the originating import if possible onto which we can attach a quickfix
                //  An import call expression cannot be rewritten into another form to correct the error - the only solution is to use `.default` at the use-site
                if (importNode && !ts.isImportCall(importNode)) {
                    var sigs = getSignaturesOfType(getTypeOfSymbol(getSymbolLinks(apparentType.symbol).target), kind);
                    if (!sigs || !sigs.length)
                        return;
                    ts.addRelatedInfo(diagnostic, ts.createDiagnosticForNode(importNode, ts.Diagnostics.Type_originates_at_this_import_A_namespace_style_import_cannot_be_called_or_constructed_and_will_cause_a_failure_at_runtime_Consider_using_a_default_import_or_import_require_here_instead));
                }
            }
            function resolveTaggedTemplateExpression(node, candidatesOutArray, checkMode) {
                var tagType = checkExpression(node.tag);
                var apparentType = getApparentType(tagType);
                if (apparentType === errorType) {
                    // Another error has already been reported
                    return resolveErrorCall(node);
                }
                var callSignatures = getSignaturesOfType(apparentType, 0 /* Call */);
                var numConstructSignatures = getSignaturesOfType(apparentType, 1 /* Construct */).length;
                if (isUntypedFunctionCall(tagType, apparentType, callSignatures.length, numConstructSignatures)) {
                    return resolveUntypedCall(node);
                }
                if (!callSignatures.length) {
                    invocationError(node.tag, apparentType, 0 /* Call */);
                    return resolveErrorCall(node);
                }
                return resolveCall(node, callSignatures, candidatesOutArray, checkMode, 0 /* None */);
            }
            /**
             * Gets the localized diagnostic head message to use for errors when resolving a decorator as a call expression.
             */
            function getDiagnosticHeadMessageForDecoratorResolution(node) {
                switch (node.parent.kind) {
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                        return ts.Diagnostics.Unable_to_resolve_signature_of_class_decorator_when_called_as_an_expression;
                    case 156 /* Parameter */:
                        return ts.Diagnostics.Unable_to_resolve_signature_of_parameter_decorator_when_called_as_an_expression;
                    case 159 /* PropertyDeclaration */:
                        return ts.Diagnostics.Unable_to_resolve_signature_of_property_decorator_when_called_as_an_expression;
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        return ts.Diagnostics.Unable_to_resolve_signature_of_method_decorator_when_called_as_an_expression;
                    default:
                        return ts.Debug.fail();
                }
            }
            /**
             * Resolves a decorator as if it were a call expression.
             */
            function resolveDecorator(node, candidatesOutArray, checkMode) {
                var funcType = checkExpression(node.expression);
                var apparentType = getApparentType(funcType);
                if (apparentType === errorType) {
                    return resolveErrorCall(node);
                }
                var callSignatures = getSignaturesOfType(apparentType, 0 /* Call */);
                var numConstructSignatures = getSignaturesOfType(apparentType, 1 /* Construct */).length;
                if (isUntypedFunctionCall(funcType, apparentType, callSignatures.length, numConstructSignatures)) {
                    return resolveUntypedCall(node);
                }
                if (isPotentiallyUncalledDecorator(node, callSignatures)) {
                    var nodeStr = ts.getTextOfNode(node.expression, /*includeTrivia*/ false);
                    error(node, ts.Diagnostics._0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write_0, nodeStr);
                    return resolveErrorCall(node);
                }
                var headMessage = getDiagnosticHeadMessageForDecoratorResolution(node);
                if (!callSignatures.length) {
                    var errorDetails = invocationErrorDetails(apparentType, 0 /* Call */);
                    var messageChain = ts.chainDiagnosticMessages(errorDetails.messageChain, headMessage);
                    var diag = ts.createDiagnosticForNodeFromMessageChain(node.expression, messageChain);
                    if (errorDetails.relatedMessage) {
                        ts.addRelatedInfo(diag, ts.createDiagnosticForNode(node.expression, errorDetails.relatedMessage));
                    }
                    diagnostics.add(diag);
                    invocationErrorRecovery(apparentType, 0 /* Call */, diag);
                    return resolveErrorCall(node);
                }
                return resolveCall(node, callSignatures, candidatesOutArray, checkMode, 0 /* None */, headMessage);
            }
            function createSignatureForJSXIntrinsic(node, result) {
                var namespace = getJsxNamespaceAt(node);
                var exports = namespace && getExportsOfSymbol(namespace);
                // We fake up a SFC signature for each intrinsic, however a more specific per-element signature drawn from the JSX declaration
                // file would probably be preferable.
                var typeSymbol = exports && getSymbol(exports, JsxNames.Element, 788968 /* Type */);
                var returnNode = typeSymbol && nodeBuilder.symbolToEntityName(typeSymbol, 788968 /* Type */, node);
                var declaration = ts.createFunctionTypeNode(/*typeParameters*/ undefined, [ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotdotdot*/ undefined, "props", /*questionMark*/ undefined, nodeBuilder.typeToTypeNode(result, node))], returnNode ? ts.createTypeReferenceNode(returnNode, /*typeArguments*/ undefined) : ts.createKeywordTypeNode(125 /* AnyKeyword */));
                var parameterSymbol = createSymbol(1 /* FunctionScopedVariable */, "props");
                parameterSymbol.type = result;
                return createSignature(declaration, 
                /*typeParameters*/ undefined, 
                /*thisParameter*/ undefined, [parameterSymbol], typeSymbol ? getDeclaredTypeOfSymbol(typeSymbol) : errorType, 
                /*returnTypePredicate*/ undefined, 1, 0 /* None */);
            }
            function resolveJsxOpeningLikeElement(node, candidatesOutArray, checkMode) {
                if (isJsxIntrinsicIdentifier(node.tagName)) {
                    var result = getIntrinsicAttributesTypeFromJsxOpeningLikeElement(node);
                    var fakeSignature = createSignatureForJSXIntrinsic(node, result);
                    checkTypeAssignableToAndOptionallyElaborate(checkExpressionWithContextualType(node.attributes, getEffectiveFirstArgumentForJsxSignature(fakeSignature, node), /*mapper*/ undefined, 0 /* Normal */), result, node.tagName, node.attributes);
                    return fakeSignature;
                }
                var exprTypes = checkExpression(node.tagName);
                var apparentType = getApparentType(exprTypes);
                if (apparentType === errorType) {
                    return resolveErrorCall(node);
                }
                var signatures = getUninstantiatedJsxSignaturesOfType(exprTypes, node);
                if (isUntypedFunctionCall(exprTypes, apparentType, signatures.length, /*constructSignatures*/ 0)) {
                    return resolveUntypedCall(node);
                }
                if (signatures.length === 0) {
                    // We found no signatures at all, which is an error
                    error(node.tagName, ts.Diagnostics.JSX_element_type_0_does_not_have_any_construct_or_call_signatures, ts.getTextOfNode(node.tagName));
                    return resolveErrorCall(node);
                }
                return resolveCall(node, signatures, candidatesOutArray, checkMode, 0 /* None */);
            }
            /**
             * Sometimes, we have a decorator that could accept zero arguments,
             * but is receiving too many arguments as part of the decorator invocation.
             * In those cases, a user may have meant to *call* the expression before using it as a decorator.
             */
            function isPotentiallyUncalledDecorator(decorator, signatures) {
                return signatures.length && ts.every(signatures, function (signature) {
                    return signature.minArgumentCount === 0 &&
                        !signatureHasRestParameter(signature) &&
                        signature.parameters.length < getDecoratorArgumentCount(decorator, signature);
                });
            }
            function resolveSignature(node, candidatesOutArray, checkMode) {
                switch (node.kind) {
                    case 196 /* CallExpression */:
                        return resolveCallExpression(node, candidatesOutArray, checkMode);
                    case 197 /* NewExpression */:
                        return resolveNewExpression(node, candidatesOutArray, checkMode);
                    case 198 /* TaggedTemplateExpression */:
                        return resolveTaggedTemplateExpression(node, candidatesOutArray, checkMode);
                    case 157 /* Decorator */:
                        return resolveDecorator(node, candidatesOutArray, checkMode);
                    case 268 /* JsxOpeningElement */:
                    case 267 /* JsxSelfClosingElement */:
                        return resolveJsxOpeningLikeElement(node, candidatesOutArray, checkMode);
                }
                throw ts.Debug.assertNever(node, "Branch in 'resolveSignature' should be unreachable.");
            }
            /**
             * Resolve a signature of a given call-like expression.
             * @param node a call-like expression to try resolve a signature for
             * @param candidatesOutArray an array of signature to be filled in by the function. It is passed by signature help in the language service;
             *                           the function will fill it up with appropriate candidate signatures
             * @return a signature of the call-like expression or undefined if one can't be found
             */
            function getResolvedSignature(node, candidatesOutArray, checkMode) {
                var links = getNodeLinks(node);
                // If getResolvedSignature has already been called, we will have cached the resolvedSignature.
                // However, it is possible that either candidatesOutArray was not passed in the first time,
                // or that a different candidatesOutArray was passed in. Therefore, we need to redo the work
                // to correctly fill the candidatesOutArray.
                var cached = links.resolvedSignature;
                if (cached && cached !== resolvingSignature && !candidatesOutArray) {
                    return cached;
                }
                links.resolvedSignature = resolvingSignature;
                var result = resolveSignature(node, candidatesOutArray, checkMode || 0 /* Normal */);
                // When CheckMode.SkipGenericFunctions is set we use resolvingSignature to indicate that call
                // resolution should be deferred.
                if (result !== resolvingSignature) {
                    // If signature resolution originated in control flow type analysis (for example to compute the
                    // assigned type in a flow assignment) we don't cache the result as it may be based on temporary
                    // types from the control flow analysis.
                    links.resolvedSignature = flowLoopStart === flowLoopCount ? result : cached;
                }
                return result;
            }
            /**
             * Indicates whether a declaration can be treated as a constructor in a JavaScript
             * file.
             */
            function isJSConstructor(node) {
                if (!node || !ts.isInJSFile(node)) {
                    return false;
                }
                var func = ts.isFunctionDeclaration(node) || ts.isFunctionExpression(node) ? node :
                    ts.isVariableDeclaration(node) && node.initializer && ts.isFunctionExpression(node.initializer) ? node.initializer :
                        undefined;
                if (func) {
                    // If the node has a @class tag, treat it like a constructor.
                    if (ts.getJSDocClassTag(node))
                        return true;
                    // If the symbol of the node has members, treat it like a constructor.
                    var symbol = getSymbolOfNode(func);
                    return !!symbol && ts.hasEntries(symbol.members);
                }
                return false;
            }
            function mergeJSSymbols(target, source) {
                if (source) {
                    var links = getSymbolLinks(source);
                    if (!links.inferredClassSymbol || !links.inferredClassSymbol.has("" + getSymbolId(target))) {
                        var inferred = ts.isTransientSymbol(target) ? target : cloneSymbol(target);
                        inferred.exports = inferred.exports || ts.createSymbolTable();
                        inferred.members = inferred.members || ts.createSymbolTable();
                        inferred.flags |= source.flags & 32 /* Class */;
                        if (ts.hasEntries(source.exports)) {
                            mergeSymbolTable(inferred.exports, source.exports);
                        }
                        if (ts.hasEntries(source.members)) {
                            mergeSymbolTable(inferred.members, source.members);
                        }
                        (links.inferredClassSymbol || (links.inferredClassSymbol = ts.createMap())).set("" + getSymbolId(inferred), inferred);
                        return inferred;
                    }
                    return links.inferredClassSymbol.get("" + getSymbolId(target));
                }
            }
            function getAssignedClassSymbol(decl) {
                var assignmentSymbol = decl && decl.parent &&
                    (ts.isFunctionDeclaration(decl) && getSymbolOfNode(decl) ||
                        ts.isBinaryExpression(decl.parent) && getSymbolOfNode(decl.parent.left) ||
                        ts.isVariableDeclaration(decl.parent) && getSymbolOfNode(decl.parent));
                var prototype = assignmentSymbol && assignmentSymbol.exports && assignmentSymbol.exports.get("prototype");
                var init = prototype && prototype.valueDeclaration && getAssignedJSPrototype(prototype.valueDeclaration);
                return init ? getSymbolOfNode(init) : undefined;
            }
            function getAssignedJSPrototype(node) {
                if (!node.parent) {
                    return false;
                }
                var parent = node.parent;
                while (parent && parent.kind === 194 /* PropertyAccessExpression */) {
                    parent = parent.parent;
                }
                if (parent && ts.isBinaryExpression(parent) && ts.isPrototypeAccess(parent.left) && parent.operatorToken.kind === 62 /* EqualsToken */) {
                    var right = ts.getInitializerOfBinaryExpression(parent);
                    return ts.isObjectLiteralExpression(right) && right;
                }
            }
            /**
             * Syntactically and semantically checks a call or new expression.
             * @param node The call/new expression to be checked.
             * @returns On success, the expression's signature's return type. On failure, anyType.
             */
            function checkCallExpression(node, checkMode) {
                if (!checkGrammarTypeArguments(node, node.typeArguments))
                    checkGrammarArguments(node.arguments);
                var signature = getResolvedSignature(node, /*candidatesOutArray*/ undefined, checkMode);
                if (signature === resolvingSignature) {
                    // CheckMode.SkipGenericFunctions is enabled and this is a call to a generic function that
                    // returns a function type. We defer checking and return nonInferrableType.
                    return nonInferrableType;
                }
                if (node.expression.kind === 102 /* SuperKeyword */) {
                    return voidType;
                }
                if (node.kind === 197 /* NewExpression */) {
                    var declaration = signature.declaration;
                    if (declaration &&
                        declaration.kind !== 162 /* Constructor */ &&
                        declaration.kind !== 166 /* ConstructSignature */ &&
                        declaration.kind !== 171 /* ConstructorType */ &&
                        !ts.isJSDocConstructSignature(declaration) &&
                        !isJSConstructor(declaration)) {
                        // When resolved signature is a call signature (and not a construct signature) the result type is any
                        if (noImplicitAny) {
                            error(node, ts.Diagnostics.new_expression_whose_target_lacks_a_construct_signature_implicitly_has_an_any_type);
                        }
                        return anyType;
                    }
                }
                // In JavaScript files, calls to any identifier 'require' are treated as external module imports
                if (ts.isInJSFile(node) && isCommonJsRequire(node)) {
                    return resolveExternalModuleTypeByLiteral(node.arguments[0]);
                }
                var returnType = getReturnTypeOfSignature(signature);
                // Treat any call to the global 'Symbol' function that is part of a const variable or readonly property
                // as a fresh unique symbol literal type.
                if (returnType.flags & 12288 /* ESSymbolLike */ && isSymbolOrSymbolForCall(node)) {
                    return getESSymbolLikeTypeForNode(ts.walkUpParenthesizedExpressions(node.parent));
                }
                if (node.kind === 196 /* CallExpression */ && node.parent.kind === 226 /* ExpressionStatement */ &&
                    returnType.flags & 16384 /* Void */ && getTypePredicateOfSignature(signature)) {
                    if (!ts.isDottedName(node.expression)) {
                        error(node.expression, ts.Diagnostics.Assertions_require_the_call_target_to_be_an_identifier_or_qualified_name);
                    }
                    else if (!getEffectsSignature(node)) {
                        var diagnostic = error(node.expression, ts.Diagnostics.Assertions_require_every_name_in_the_call_target_to_be_declared_with_an_explicit_type_annotation);
                        getTypeOfDottedName(node.expression, diagnostic);
                    }
                }
                if (ts.isInJSFile(node)) {
                    var decl = ts.getDeclarationOfExpando(node);
                    if (decl) {
                        var jsSymbol = getSymbolOfNode(decl);
                        if (jsSymbol && ts.hasEntries(jsSymbol.exports)) {
                            var jsAssignmentType = createAnonymousType(jsSymbol, jsSymbol.exports, ts.emptyArray, ts.emptyArray, undefined, undefined);
                            jsAssignmentType.objectFlags |= 16384 /* JSLiteral */;
                            return getIntersectionType([returnType, jsAssignmentType]);
                        }
                    }
                }
                return returnType;
            }
            function isSymbolOrSymbolForCall(node) {
                if (!ts.isCallExpression(node))
                    return false;
                var left = node.expression;
                if (ts.isPropertyAccessExpression(left) && left.name.escapedText === "for") {
                    left = left.expression;
                }
                if (!ts.isIdentifier(left) || left.escapedText !== "Symbol") {
                    return false;
                }
                // make sure `Symbol` is the global symbol
                var globalESSymbol = getGlobalESSymbolConstructorSymbol(/*reportErrors*/ false);
                if (!globalESSymbol) {
                    return false;
                }
                return globalESSymbol === resolveName(left, "Symbol", 111551 /* Value */, /*nameNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ false);
            }
            function checkImportCallExpression(node) {
                // Check grammar of dynamic import
                if (!checkGrammarArguments(node.arguments))
                    checkGrammarImportCallExpression(node);
                if (node.arguments.length === 0) {
                    return createPromiseReturnType(node, anyType);
                }
                var specifier = node.arguments[0];
                var specifierType = checkExpressionCached(specifier);
                // Even though multiple arguments is grammatically incorrect, type-check extra arguments for completion
                for (var i = 1; i < node.arguments.length; ++i) {
                    checkExpressionCached(node.arguments[i]);
                }
                if (specifierType.flags & 32768 /* Undefined */ || specifierType.flags & 65536 /* Null */ || !isTypeAssignableTo(specifierType, stringType)) {
                    error(specifier, ts.Diagnostics.Dynamic_import_s_specifier_must_be_of_type_string_but_here_has_type_0, typeToString(specifierType));
                }
                // resolveExternalModuleName will return undefined if the moduleReferenceExpression is not a string literal
                var moduleSymbol = resolveExternalModuleName(node, specifier);
                if (moduleSymbol) {
                    var esModuleSymbol = resolveESModuleSymbol(moduleSymbol, specifier, /*dontRecursivelyResolve*/ true, /*suppressUsageError*/ false);
                    if (esModuleSymbol) {
                        return createPromiseReturnType(node, getTypeWithSyntheticDefaultImportType(getTypeOfSymbol(esModuleSymbol), esModuleSymbol, moduleSymbol));
                    }
                }
                return createPromiseReturnType(node, anyType);
            }
            function getTypeWithSyntheticDefaultImportType(type, symbol, originalSymbol) {
                if (allowSyntheticDefaultImports && type && type !== errorType) {
                    var synthType = type;
                    if (!synthType.syntheticType) {
                        var file = ts.find(originalSymbol.declarations, ts.isSourceFile);
                        var hasSyntheticDefault = canHaveSyntheticDefault(file, originalSymbol, /*dontResolveAlias*/ false);
                        if (hasSyntheticDefault) {
                            var memberTable = ts.createSymbolTable();
                            var newSymbol = createSymbol(2097152 /* Alias */, "default" /* Default */);
                            newSymbol.nameType = getLiteralType("default");
                            newSymbol.target = resolveSymbol(symbol);
                            memberTable.set("default" /* Default */, newSymbol);
                            var anonymousSymbol = createSymbol(2048 /* TypeLiteral */, "__type" /* Type */);
                            var defaultContainingObject = createAnonymousType(anonymousSymbol, memberTable, ts.emptyArray, ts.emptyArray, /*stringIndexInfo*/ undefined, /*numberIndexInfo*/ undefined);
                            anonymousSymbol.type = defaultContainingObject;
                            synthType.syntheticType = isValidSpreadType(type) ? getSpreadType(type, defaultContainingObject, anonymousSymbol, /*objectFlags*/ 0, /*readonly*/ false) : defaultContainingObject;
                        }
                        else {
                            synthType.syntheticType = type;
                        }
                    }
                    return synthType.syntheticType;
                }
                return type;
            }
            function isCommonJsRequire(node) {
                if (!ts.isRequireCall(node, /*checkArgumentIsStringLiteralLike*/ true)) {
                    return false;
                }
                // Make sure require is not a local function
                if (!ts.isIdentifier(node.expression))
                    return ts.Debug.fail();
                var resolvedRequire = resolveName(node.expression, node.expression.escapedText, 111551 /* Value */, /*nameNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ true); // TODO: GH#18217
                if (resolvedRequire === requireSymbol) {
                    return true;
                }
                // project includes symbol named 'require' - make sure that it is ambient and local non-alias
                if (resolvedRequire.flags & 2097152 /* Alias */) {
                    return false;
                }
                var targetDeclarationKind = resolvedRequire.flags & 16 /* Function */
                    ? 244 /* FunctionDeclaration */
                    : resolvedRequire.flags & 3 /* Variable */
                        ? 242 /* VariableDeclaration */
                        : 0 /* Unknown */;
                if (targetDeclarationKind !== 0 /* Unknown */) {
                    var decl = ts.getDeclarationOfKind(resolvedRequire, targetDeclarationKind);
                    // function/variable declaration should be ambient
                    return !!decl && !!(decl.flags & 8388608 /* Ambient */);
                }
                return false;
            }
            function checkTaggedTemplateExpression(node) {
                if (!checkGrammarTaggedTemplateChain(node))
                    checkGrammarTypeArguments(node, node.typeArguments);
                if (languageVersion < 2 /* ES2015 */) {
                    checkExternalEmitHelpers(node, 131072 /* MakeTemplateObject */);
                }
                return getReturnTypeOfSignature(getResolvedSignature(node));
            }
            function checkAssertion(node) {
                return checkAssertionWorker(node, node.type, node.expression);
            }
            function isValidConstAssertionArgument(node) {
                switch (node.kind) {
                    case 10 /* StringLiteral */:
                    case 14 /* NoSubstitutionTemplateLiteral */:
                    case 8 /* NumericLiteral */:
                    case 9 /* BigIntLiteral */:
                    case 106 /* TrueKeyword */:
                    case 91 /* FalseKeyword */:
                    case 192 /* ArrayLiteralExpression */:
                    case 193 /* ObjectLiteralExpression */:
                        return true;
                    case 200 /* ParenthesizedExpression */:
                        return isValidConstAssertionArgument(node.expression);
                    case 207 /* PrefixUnaryExpression */:
                        var op = node.operator;
                        var arg = node.operand;
                        return op === 40 /* MinusToken */ && (arg.kind === 8 /* NumericLiteral */ || arg.kind === 9 /* BigIntLiteral */) ||
                            op === 39 /* PlusToken */ && arg.kind === 8 /* NumericLiteral */;
                    case 194 /* PropertyAccessExpression */:
                    case 195 /* ElementAccessExpression */:
                        var expr = node.expression;
                        if (ts.isIdentifier(expr)) {
                            var symbol = getSymbolAtLocation(expr);
                            if (symbol && symbol.flags & 2097152 /* Alias */) {
                                symbol = resolveAlias(symbol);
                            }
                            return !!(symbol && (symbol.flags & 384 /* Enum */) && getEnumKind(symbol) === 1 /* Literal */);
                        }
                }
                return false;
            }
            function checkAssertionWorker(errNode, type, expression, checkMode) {
                var exprType = checkExpression(expression, checkMode);
                if (ts.isConstTypeReference(type)) {
                    if (!isValidConstAssertionArgument(expression)) {
                        error(expression, ts.Diagnostics.A_const_assertions_can_only_be_applied_to_references_to_enum_members_or_string_number_boolean_array_or_object_literals);
                    }
                    return getRegularTypeOfLiteralType(exprType);
                }
                checkSourceElement(type);
                exprType = getRegularTypeOfObjectLiteral(getBaseTypeOfLiteralType(exprType));
                var targetType = getTypeFromTypeNode(type);
                if (produceDiagnostics && targetType !== errorType) {
                    var widenedType = getWidenedType(exprType);
                    if (!isTypeComparableTo(targetType, widenedType)) {
                        checkTypeComparableTo(exprType, targetType, errNode, ts.Diagnostics.Conversion_of_type_0_to_type_1_may_be_a_mistake_because_neither_type_sufficiently_overlaps_with_the_other_If_this_was_intentional_convert_the_expression_to_unknown_first);
                    }
                }
                return targetType;
            }
            function checkNonNullAssertion(node) {
                return getNonNullableType(checkExpression(node.expression));
            }
            function checkMetaProperty(node) {
                checkGrammarMetaProperty(node);
                if (node.keywordToken === 99 /* NewKeyword */) {
                    return checkNewTargetMetaProperty(node);
                }
                if (node.keywordToken === 96 /* ImportKeyword */) {
                    return checkImportMetaProperty(node);
                }
                return ts.Debug.assertNever(node.keywordToken);
            }
            function checkNewTargetMetaProperty(node) {
                var container = ts.getNewTargetContainer(node);
                if (!container) {
                    error(node, ts.Diagnostics.Meta_property_0_is_only_allowed_in_the_body_of_a_function_declaration_function_expression_or_constructor, "new.target");
                    return errorType;
                }
                else if (container.kind === 162 /* Constructor */) {
                    var symbol = getSymbolOfNode(container.parent);
                    return getTypeOfSymbol(symbol);
                }
                else {
                    var symbol = getSymbolOfNode(container);
                    return getTypeOfSymbol(symbol);
                }
            }
            function checkImportMetaProperty(node) {
                if (moduleKind !== ts.ModuleKind.ESNext && moduleKind !== ts.ModuleKind.System) {
                    error(node, ts.Diagnostics.The_import_meta_meta_property_is_only_allowed_when_the_module_option_is_esnext_or_system);
                }
                var file = ts.getSourceFileOfNode(node);
                ts.Debug.assert(!!(file.flags & 2097152 /* PossiblyContainsImportMeta */), "Containing file is missing import meta node flag.");
                ts.Debug.assert(!!file.externalModuleIndicator, "Containing file should be a module.");
                return node.name.escapedText === "meta" ? getGlobalImportMetaType() : errorType;
            }
            function getTypeOfParameter(symbol) {
                var type = getTypeOfSymbol(symbol);
                if (strictNullChecks) {
                    var declaration = symbol.valueDeclaration;
                    if (declaration && ts.hasInitializer(declaration)) {
                        return getOptionalType(type);
                    }
                }
                return type;
            }
            function getParameterNameAtPosition(signature, pos) {
                var paramCount = signature.parameters.length - (signatureHasRestParameter(signature) ? 1 : 0);
                if (pos < paramCount) {
                    return signature.parameters[pos].escapedName;
                }
                var restParameter = signature.parameters[paramCount] || unknownSymbol;
                var restType = getTypeOfSymbol(restParameter);
                if (isTupleType(restType)) {
                    var associatedNames = restType.target.associatedNames;
                    var index = pos - paramCount;
                    return associatedNames && associatedNames[index] || restParameter.escapedName + "_" + index;
                }
                return restParameter.escapedName;
            }
            function getTypeAtPosition(signature, pos) {
                return tryGetTypeAtPosition(signature, pos) || anyType;
            }
            function tryGetTypeAtPosition(signature, pos) {
                var paramCount = signature.parameters.length - (signatureHasRestParameter(signature) ? 1 : 0);
                if (pos < paramCount) {
                    return getTypeOfParameter(signature.parameters[pos]);
                }
                if (signatureHasRestParameter(signature)) {
                    // We want to return the value undefined for an out of bounds parameter position,
                    // so we need to check bounds here before calling getIndexedAccessType (which
                    // otherwise would return the type 'undefined').
                    var restType = getTypeOfSymbol(signature.parameters[paramCount]);
                    var index = pos - paramCount;
                    if (!isTupleType(restType) || restType.target.hasRestElement || index < getTypeArguments(restType).length) {
                        return getIndexedAccessType(restType, getLiteralType(index));
                    }
                }
                return undefined;
            }
            function getRestTypeAtPosition(source, pos) {
                var paramCount = getParameterCount(source);
                var restType = getEffectiveRestType(source);
                var nonRestCount = paramCount - (restType ? 1 : 0);
                if (restType && pos === nonRestCount) {
                    return restType;
                }
                var types = [];
                var names = [];
                for (var i = pos; i < nonRestCount; i++) {
                    types.push(getTypeAtPosition(source, i));
                    names.push(getParameterNameAtPosition(source, i));
                }
                if (restType) {
                    types.push(getIndexedAccessType(restType, numberType));
                    names.push(getParameterNameAtPosition(source, nonRestCount));
                }
                var minArgumentCount = getMinArgumentCount(source);
                var minLength = minArgumentCount < pos ? 0 : minArgumentCount - pos;
                return createTupleType(types, minLength, !!restType, /*readonly*/ false, names);
            }
            function getParameterCount(signature) {
                var length = signature.parameters.length;
                if (signatureHasRestParameter(signature)) {
                    var restType = getTypeOfSymbol(signature.parameters[length - 1]);
                    if (isTupleType(restType)) {
                        return length + getTypeArguments(restType).length - 1;
                    }
                }
                return length;
            }
            function getMinArgumentCount(signature) {
                if (signatureHasRestParameter(signature)) {
                    var restType = getTypeOfSymbol(signature.parameters[signature.parameters.length - 1]);
                    if (isTupleType(restType)) {
                        var minLength = restType.target.minLength;
                        if (minLength > 0) {
                            return signature.parameters.length - 1 + minLength;
                        }
                    }
                }
                return signature.minArgumentCount;
            }
            function hasEffectiveRestParameter(signature) {
                if (signatureHasRestParameter(signature)) {
                    var restType = getTypeOfSymbol(signature.parameters[signature.parameters.length - 1]);
                    return !isTupleType(restType) || restType.target.hasRestElement;
                }
                return false;
            }
            function getEffectiveRestType(signature) {
                if (signatureHasRestParameter(signature)) {
                    var restType = getTypeOfSymbol(signature.parameters[signature.parameters.length - 1]);
                    return isTupleType(restType) ? getRestArrayTypeOfTupleType(restType) : restType;
                }
                return undefined;
            }
            function getNonArrayRestType(signature) {
                var restType = getEffectiveRestType(signature);
                return restType && !isArrayType(restType) && !isTypeAny(restType) ? restType : undefined;
            }
            function getTypeOfFirstParameterOfSignature(signature) {
                return getTypeOfFirstParameterOfSignatureWithFallback(signature, neverType);
            }
            function getTypeOfFirstParameterOfSignatureWithFallback(signature, fallbackType) {
                return signature.parameters.length > 0 ? getTypeAtPosition(signature, 0) : fallbackType;
            }
            function inferFromAnnotatedParameters(signature, context, inferenceContext) {
                var len = signature.parameters.length - (signatureHasRestParameter(signature) ? 1 : 0);
                for (var i = 0; i < len; i++) {
                    var declaration = signature.parameters[i].valueDeclaration;
                    if (declaration.type) {
                        var typeNode = ts.getEffectiveTypeAnnotationNode(declaration);
                        if (typeNode) {
                            inferTypes(inferenceContext.inferences, getTypeFromTypeNode(typeNode), getTypeAtPosition(context, i));
                        }
                    }
                }
                var restType = getEffectiveRestType(context);
                if (restType && restType.flags & 262144 /* TypeParameter */) {
                    // The contextual signature has a generic rest parameter. We first instantiate the contextual
                    // signature (without fixing type parameters) and assign types to contextually typed parameters.
                    var instantiatedContext = instantiateSignature(context, inferenceContext.nonFixingMapper);
                    assignContextualParameterTypes(signature, instantiatedContext);
                    // We then infer from a tuple type representing the parameters that correspond to the contextual
                    // rest parameter.
                    var restPos = getParameterCount(context) - 1;
                    inferTypes(inferenceContext.inferences, getRestTypeAtPosition(signature, restPos), restType);
                }
            }
            function assignContextualParameterTypes(signature, context) {
                signature.typeParameters = context.typeParameters;
                if (context.thisParameter) {
                    var parameter = signature.thisParameter;
                    if (!parameter || parameter.valueDeclaration && !parameter.valueDeclaration.type) {
                        if (!parameter) {
                            signature.thisParameter = createSymbolWithType(context.thisParameter, /*type*/ undefined);
                        }
                        assignParameterType(signature.thisParameter, getTypeOfSymbol(context.thisParameter));
                    }
                }
                var len = signature.parameters.length - (signatureHasRestParameter(signature) ? 1 : 0);
                for (var i = 0; i < len; i++) {
                    var parameter = signature.parameters[i];
                    if (!ts.getEffectiveTypeAnnotationNode(parameter.valueDeclaration)) {
                        var contextualParameterType = tryGetTypeAtPosition(context, i);
                        assignParameterType(parameter, contextualParameterType);
                    }
                }
                if (signatureHasRestParameter(signature)) {
                    // parameter might be a transient symbol generated by use of `arguments` in the function body.
                    var parameter = ts.last(signature.parameters);
                    if (ts.isTransientSymbol(parameter) || !ts.getEffectiveTypeAnnotationNode(parameter.valueDeclaration)) {
                        var contextualParameterType = getRestTypeAtPosition(context, len);
                        assignParameterType(parameter, contextualParameterType);
                    }
                }
            }
            function assignNonContextualParameterTypes(signature) {
                if (signature.thisParameter) {
                    assignParameterType(signature.thisParameter);
                }
                for (var _i = 0, _a = signature.parameters; _i < _a.length; _i++) {
                    var parameter = _a[_i];
                    assignParameterType(parameter);
                }
            }
            function assignParameterType(parameter, type) {
                var links = getSymbolLinks(parameter);
                if (!links.type) {
                    var declaration = parameter.valueDeclaration;
                    links.type = type || getWidenedTypeForVariableLikeDeclaration(declaration, /*includeOptionality*/ true);
                    if (declaration.name.kind !== 75 /* Identifier */) {
                        // if inference didn't come up with anything but unknown, fall back to the binding pattern if present.
                        if (links.type === unknownType) {
                            links.type = getTypeFromBindingPattern(declaration.name);
                        }
                        assignBindingElementTypes(declaration.name);
                    }
                }
            }
            // When contextual typing assigns a type to a parameter that contains a binding pattern, we also need to push
            // the destructured type into the contained binding elements.
            function assignBindingElementTypes(pattern) {
                for (var _i = 0, _a = pattern.elements; _i < _a.length; _i++) {
                    var element = _a[_i];
                    if (!ts.isOmittedExpression(element)) {
                        if (element.name.kind === 75 /* Identifier */) {
                            getSymbolLinks(getSymbolOfNode(element)).type = getTypeForBindingElement(element);
                        }
                        else {
                            assignBindingElementTypes(element.name);
                        }
                    }
                }
            }
            function createPromiseType(promisedType) {
                // creates a `Promise<T>` type where `T` is the promisedType argument
                var globalPromiseType = getGlobalPromiseType(/*reportErrors*/ true);
                if (globalPromiseType !== emptyGenericType) {
                    // if the promised type is itself a promise, get the underlying type; otherwise, fallback to the promised type
                    promisedType = getAwaitedType(promisedType) || unknownType;
                    return createTypeReference(globalPromiseType, [promisedType]);
                }
                return unknownType;
            }
            function createPromiseLikeType(promisedType) {
                // creates a `PromiseLike<T>` type where `T` is the promisedType argument
                var globalPromiseLikeType = getGlobalPromiseLikeType(/*reportErrors*/ true);
                if (globalPromiseLikeType !== emptyGenericType) {
                    // if the promised type is itself a promise, get the underlying type; otherwise, fallback to the promised type
                    promisedType = getAwaitedType(promisedType) || unknownType;
                    return createTypeReference(globalPromiseLikeType, [promisedType]);
                }
                return unknownType;
            }
            function createPromiseReturnType(func, promisedType) {
                var promiseType = createPromiseType(promisedType);
                if (promiseType === unknownType) {
                    error(func, ts.isImportCall(func) ?
                        ts.Diagnostics.A_dynamic_import_call_returns_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option :
                        ts.Diagnostics.An_async_function_or_method_must_return_a_Promise_Make_sure_you_have_a_declaration_for_Promise_or_include_ES2015_in_your_lib_option);
                    return errorType;
                }
                else if (!getGlobalPromiseConstructorSymbol(/*reportErrors*/ true)) {
                    error(func, ts.isImportCall(func) ?
                        ts.Diagnostics.A_dynamic_import_call_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option :
                        ts.Diagnostics.An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option);
                }
                return promiseType;
            }
            function getReturnTypeFromBody(func, checkMode) {
                if (!func.body) {
                    return errorType;
                }
                var functionFlags = ts.getFunctionFlags(func);
                var isAsync = (functionFlags & 2 /* Async */) !== 0;
                var isGenerator = (functionFlags & 1 /* Generator */) !== 0;
                var returnType;
                var yieldType;
                var nextType;
                var fallbackReturnType = voidType;
                if (func.body.kind !== 223 /* Block */) { // Async or normal arrow function
                    returnType = checkExpressionCached(func.body, checkMode && checkMode & ~8 /* SkipGenericFunctions */);
                    if (isAsync) {
                        // From within an async function you can return either a non-promise value or a promise. Any
                        // Promise/A+ compatible implementation will always assimilate any foreign promise, so the
                        // return type of the body should be unwrapped to its awaited type, which we will wrap in
                        // the native Promise<T> type later in this function.
                        returnType = checkAwaitedType(returnType, /*errorNode*/ func, ts.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member);
                    }
                }
                else if (isGenerator) { // Generator or AsyncGenerator function
                    var returnTypes = checkAndAggregateReturnExpressionTypes(func, checkMode);
                    if (!returnTypes) {
                        fallbackReturnType = neverType;
                    }
                    else if (returnTypes.length > 0) {
                        returnType = getUnionType(returnTypes, 2 /* Subtype */);
                    }
                    var _a = checkAndAggregateYieldOperandTypes(func, checkMode), yieldTypes = _a.yieldTypes, nextTypes = _a.nextTypes;
                    yieldType = ts.some(yieldTypes) ? getUnionType(yieldTypes, 2 /* Subtype */) : undefined;
                    nextType = ts.some(nextTypes) ? getIntersectionType(nextTypes) : undefined;
                }
                else { // Async or normal function
                    var types = checkAndAggregateReturnExpressionTypes(func, checkMode);
                    if (!types) {
                        // For an async function, the return type will not be never, but rather a Promise for never.
                        return functionFlags & 2 /* Async */
                            ? createPromiseReturnType(func, neverType) // Async function
                            : neverType; // Normal function
                    }
                    if (types.length === 0) {
                        // For an async function, the return type will not be void, but rather a Promise for void.
                        return functionFlags & 2 /* Async */
                            ? createPromiseReturnType(func, voidType) // Async function
                            : voidType; // Normal function
                    }
                    // Return a union of the return expression types.
                    returnType = getUnionType(types, 2 /* Subtype */);
                }
                if (returnType || yieldType || nextType) {
                    var contextualSignature = getContextualSignatureForFunctionLikeDeclaration(func);
                    if (!contextualSignature) {
                        if (yieldType)
                            reportErrorsFromWidening(func, yieldType, 1 /* GeneratorYield */);
                        if (returnType)
                            reportErrorsFromWidening(func, returnType);
                        if (nextType)
                            reportErrorsFromWidening(func, nextType);
                    }
                    if (returnType && isUnitType(returnType) ||
                        yieldType && isUnitType(yieldType) ||
                        nextType && isUnitType(nextType)) {
                        var contextualType = !contextualSignature ? undefined :
                            contextualSignature === getSignatureFromDeclaration(func) ? isGenerator ? undefined : returnType :
                                instantiateContextualType(getReturnTypeOfSignature(contextualSignature), func);
                        if (isGenerator) {
                            yieldType = getWidenedLiteralLikeTypeForContextualIterationTypeIfNeeded(yieldType, contextualType, 0 /* Yield */, isAsync);
                            returnType = getWidenedLiteralLikeTypeForContextualIterationTypeIfNeeded(returnType, contextualType, 1 /* Return */, isAsync);
                            nextType = getWidenedLiteralLikeTypeForContextualIterationTypeIfNeeded(nextType, contextualType, 2 /* Next */, isAsync);
                        }
                        else {
                            returnType = getWidenedLiteralLikeTypeForContextualReturnTypeIfNeeded(returnType, contextualType, isAsync);
                        }
                    }
                    if (yieldType)
                        yieldType = getWidenedType(yieldType);
                    if (returnType)
                        returnType = getWidenedType(returnType);
                    if (nextType)
                        nextType = getWidenedType(nextType);
                }
                if (isGenerator) {
                    return createGeneratorReturnType(yieldType || neverType, returnType || fallbackReturnType, nextType || getContextualIterationType(2 /* Next */, func) || unknownType, isAsync);
                }
                else {
                    // From within an async function you can return either a non-promise value or a promise. Any
                    // Promise/A+ compatible implementation will always assimilate any foreign promise, so the
                    // return type of the body is awaited type of the body, wrapped in a native Promise<T> type.
                    return isAsync
                        ? createPromiseType(returnType || fallbackReturnType)
                        : returnType || fallbackReturnType;
                }
            }
            function createGeneratorReturnType(yieldType, returnType, nextType, isAsyncGenerator) {
                var resolver = isAsyncGenerator ? asyncIterationTypesResolver : syncIterationTypesResolver;
                var globalGeneratorType = resolver.getGlobalGeneratorType(/*reportErrors*/ false);
                yieldType = resolver.resolveIterationType(yieldType, /*errorNode*/ undefined) || unknownType;
                returnType = resolver.resolveIterationType(returnType, /*errorNode*/ undefined) || unknownType;
                nextType = resolver.resolveIterationType(nextType, /*errorNode*/ undefined) || unknownType;
                if (globalGeneratorType === emptyGenericType) {
                    // Fall back to the global IterableIterator if returnType is assignable to the expected return iteration
                    // type of IterableIterator, and the expected next iteration type of IterableIterator is assignable to
                    // nextType.
                    var globalType = resolver.getGlobalIterableIteratorType(/*reportErrors*/ false);
                    var iterationTypes = globalType !== emptyGenericType ? getIterationTypesOfGlobalIterableType(globalType, resolver) : undefined;
                    var iterableIteratorReturnType = iterationTypes ? iterationTypes.returnType : anyType;
                    var iterableIteratorNextType = iterationTypes ? iterationTypes.nextType : undefinedType;
                    if (isTypeAssignableTo(returnType, iterableIteratorReturnType) &&
                        isTypeAssignableTo(iterableIteratorNextType, nextType)) {
                        if (globalType !== emptyGenericType) {
                            return createTypeFromGenericGlobalType(globalType, [yieldType]);
                        }
                        // The global IterableIterator type doesn't exist, so report an error
                        resolver.getGlobalIterableIteratorType(/*reportErrors*/ true);
                        return emptyObjectType;
                    }
                    // The global Generator type doesn't exist, so report an error
                    resolver.getGlobalGeneratorType(/*reportErrors*/ true);
                    return emptyObjectType;
                }
                return createTypeFromGenericGlobalType(globalGeneratorType, [yieldType, returnType, nextType]);
            }
            function checkAndAggregateYieldOperandTypes(func, checkMode) {
                var yieldTypes = [];
                var nextTypes = [];
                var isAsync = (ts.getFunctionFlags(func) & 2 /* Async */) !== 0;
                ts.forEachYieldExpression(func.body, function (yieldExpression) {
                    var yieldExpressionType = yieldExpression.expression ? checkExpression(yieldExpression.expression, checkMode) : undefinedWideningType;
                    ts.pushIfUnique(yieldTypes, getYieldedTypeOfYieldExpression(yieldExpression, yieldExpressionType, anyType, isAsync));
                    var nextType;
                    if (yieldExpression.asteriskToken) {
                        var iterationTypes = getIterationTypesOfIterable(yieldExpressionType, isAsync ? 19 /* AsyncYieldStar */ : 17 /* YieldStar */, yieldExpression.expression);
                        nextType = iterationTypes && iterationTypes.nextType;
                    }
                    else {
                        nextType = getContextualType(yieldExpression);
                    }
                    if (nextType)
                        ts.pushIfUnique(nextTypes, nextType);
                });
                return { yieldTypes: yieldTypes, nextTypes: nextTypes };
            }
            function getYieldedTypeOfYieldExpression(node, expressionType, sentType, isAsync) {
                var errorNode = node.expression || node;
                // A `yield*` expression effectively yields everything that its operand yields
                var yieldedType = node.asteriskToken ? checkIteratedTypeOrElementType(isAsync ? 19 /* AsyncYieldStar */ : 17 /* YieldStar */, expressionType, sentType, errorNode) : expressionType;
                return !isAsync ? yieldedType : getAwaitedType(yieldedType, errorNode, node.asteriskToken
                    ? ts.Diagnostics.Type_of_iterated_elements_of_a_yield_Asterisk_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member
                    : ts.Diagnostics.Type_of_yield_operand_in_an_async_generator_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member);
            }
            /**
             * Collect the TypeFacts learned from a typeof switch with
             * total clauses `witnesses`, and the active clause ranging
             * from `start` to `end`. Parameter `hasDefault` denotes
             * whether the active clause contains a default clause.
             */
            function getFactsFromTypeofSwitch(start, end, witnesses, hasDefault) {
                var facts = 0 /* None */;
                // When in the default we only collect inequality facts
                // because default is 'in theory' a set of infinite
                // equalities.
                if (hasDefault) {
                    // Value is not equal to any types after the active clause.
                    for (var i = end; i < witnesses.length; i++) {
                        facts |= typeofNEFacts.get(witnesses[i]) || 32768 /* TypeofNEHostObject */;
                    }
                    // Remove inequalities for types that appear in the
                    // active clause because they appear before other
                    // types collected so far.
                    for (var i = start; i < end; i++) {
                        facts &= ~(typeofNEFacts.get(witnesses[i]) || 0);
                    }
                    // Add inequalities for types before the active clause unconditionally.
                    for (var i = 0; i < start; i++) {
                        facts |= typeofNEFacts.get(witnesses[i]) || 32768 /* TypeofNEHostObject */;
                    }
                }
                // When in an active clause without default the set of
                // equalities is finite.
                else {
                    // Add equalities for all types in the active clause.
                    for (var i = start; i < end; i++) {
                        facts |= typeofEQFacts.get(witnesses[i]) || 128 /* TypeofEQHostObject */;
                    }
                    // Remove equalities for types that appear before the
                    // active clause.
                    for (var i = 0; i < start; i++) {
                        facts &= ~(typeofEQFacts.get(witnesses[i]) || 0);
                    }
                }
                return facts;
            }
            function isExhaustiveSwitchStatement(node) {
                var links = getNodeLinks(node);
                return links.isExhaustive !== undefined ? links.isExhaustive : (links.isExhaustive = computeExhaustiveSwitchStatement(node));
            }
            function computeExhaustiveSwitchStatement(node) {
                if (node.expression.kind === 204 /* TypeOfExpression */) {
                    var operandType = getTypeOfExpression(node.expression.expression);
                    // This cast is safe because the switch is possibly exhaustive and does not contain a default case, so there can be no undefined.
                    var witnesses = getSwitchClauseTypeOfWitnesses(node);
                    // notEqualFacts states that the type of the switched value is not equal to every type in the switch.
                    var notEqualFacts_1 = getFactsFromTypeofSwitch(0, 0, witnesses, /*hasDefault*/ true);
                    var type_3 = getBaseConstraintOfType(operandType) || operandType;
                    return !!(filterType(type_3, function (t) { return (getTypeFacts(t) & notEqualFacts_1) === notEqualFacts_1; }).flags & 131072 /* Never */);
                }
                var type = getTypeOfExpression(node.expression);
                if (!isLiteralType(type)) {
                    return false;
                }
                var switchTypes = getSwitchClauseTypes(node);
                if (!switchTypes.length || ts.some(switchTypes, isNeitherUnitTypeNorNever)) {
                    return false;
                }
                return eachTypeContainedIn(mapType(type, getRegularTypeOfLiteralType), switchTypes);
            }
            function functionHasImplicitReturn(func) {
                return func.endFlowNode && isReachableFlowNode(func.endFlowNode);
            }
            /** NOTE: Return value of `[]` means a different thing than `undefined`. `[]` means func returns `void`, `undefined` means it returns `never`. */
            function checkAndAggregateReturnExpressionTypes(func, checkMode) {
                var functionFlags = ts.getFunctionFlags(func);
                var aggregatedTypes = [];
                var hasReturnWithNoExpression = functionHasImplicitReturn(func);
                var hasReturnOfTypeNever = false;
                ts.forEachReturnStatement(func.body, function (returnStatement) {
                    var expr = returnStatement.expression;
                    if (expr) {
                        var type = checkExpressionCached(expr, checkMode && checkMode & ~8 /* SkipGenericFunctions */);
                        if (functionFlags & 2 /* Async */) {
                            // From within an async function you can return either a non-promise value or a promise. Any
                            // Promise/A+ compatible implementation will always assimilate any foreign promise, so the
                            // return type of the body should be unwrapped to its awaited type, which should be wrapped in
                            // the native Promise<T> type by the caller.
                            type = checkAwaitedType(type, func, ts.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member);
                        }
                        if (type.flags & 131072 /* Never */) {
                            hasReturnOfTypeNever = true;
                        }
                        ts.pushIfUnique(aggregatedTypes, type);
                    }
                    else {
                        hasReturnWithNoExpression = true;
                    }
                });
                if (aggregatedTypes.length === 0 && !hasReturnWithNoExpression && (hasReturnOfTypeNever || mayReturnNever(func))) {
                    return undefined;
                }
                if (strictNullChecks && aggregatedTypes.length && hasReturnWithNoExpression &&
                    !(isJSConstructor(func) && aggregatedTypes.some(function (t) { return t.symbol === func.symbol; }))) {
                    // Javascript "callable constructors", containing eg `if (!(this instanceof A)) return new A()` should not add undefined
                    ts.pushIfUnique(aggregatedTypes, undefinedType);
                }
                return aggregatedTypes;
            }
            function mayReturnNever(func) {
                switch (func.kind) {
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                        return true;
                    case 161 /* MethodDeclaration */:
                        return func.parent.kind === 193 /* ObjectLiteralExpression */;
                    default:
                        return false;
                }
            }
            /**
             * TypeScript Specification 1.0 (6.3) - July 2014
             *   An explicitly typed function whose return type isn't the Void type,
             *   the Any type, or a union type containing the Void or Any type as a constituent
             *   must have at least one return statement somewhere in its body.
             *   An exception to this rule is if the function implementation consists of a single 'throw' statement.
             *
             * @param returnType - return type of the function, can be undefined if return type is not explicitly specified
             */
            function checkAllCodePathsInNonVoidFunctionReturnOrThrow(func, returnType) {
                if (!produceDiagnostics) {
                    return;
                }
                var functionFlags = ts.getFunctionFlags(func);
                var type = returnType && getReturnOrPromisedType(returnType, functionFlags);
                // Functions with with an explicitly specified 'void' or 'any' return type don't need any return expressions.
                if (type && maybeTypeOfKind(type, 1 /* Any */ | 16384 /* Void */)) {
                    return;
                }
                // If all we have is a function signature, or an arrow function with an expression body, then there is nothing to check.
                // also if HasImplicitReturn flag is not set this means that all codepaths in function body end with return or throw
                if (func.kind === 160 /* MethodSignature */ || ts.nodeIsMissing(func.body) || func.body.kind !== 223 /* Block */ || !functionHasImplicitReturn(func)) {
                    return;
                }
                var hasExplicitReturn = func.flags & 512 /* HasExplicitReturn */;
                if (type && type.flags & 131072 /* Never */) {
                    error(ts.getEffectiveReturnTypeNode(func), ts.Diagnostics.A_function_returning_never_cannot_have_a_reachable_end_point);
                }
                else if (type && !hasExplicitReturn) {
                    // minimal check: function has syntactic return type annotation and no explicit return statements in the body
                    // this function does not conform to the specification.
                    // NOTE: having returnType !== undefined is a precondition for entering this branch so func.type will always be present
                    error(ts.getEffectiveReturnTypeNode(func), ts.Diagnostics.A_function_whose_declared_type_is_neither_void_nor_any_must_return_a_value);
                }
                else if (type && strictNullChecks && !isTypeAssignableTo(undefinedType, type)) {
                    error(ts.getEffectiveReturnTypeNode(func) || func, ts.Diagnostics.Function_lacks_ending_return_statement_and_return_type_does_not_include_undefined);
                }
                else if (compilerOptions.noImplicitReturns) {
                    if (!type) {
                        // If return type annotation is omitted check if function has any explicit return statements.
                        // If it does not have any - its inferred return type is void - don't do any checks.
                        // Otherwise get inferred return type from function body and report error only if it is not void / anytype
                        if (!hasExplicitReturn) {
                            return;
                        }
                        var inferredReturnType = getReturnTypeOfSignature(getSignatureFromDeclaration(func));
                        if (isUnwrappedReturnTypeVoidOrAny(func, inferredReturnType)) {
                            return;
                        }
                    }
                    error(ts.getEffectiveReturnTypeNode(func) || func, ts.Diagnostics.Not_all_code_paths_return_a_value);
                }
            }
            function checkFunctionExpressionOrObjectLiteralMethod(node, checkMode) {
                ts.Debug.assert(node.kind !== 161 /* MethodDeclaration */ || ts.isObjectLiteralMethod(node));
                checkNodeDeferred(node);
                // The identityMapper object is used to indicate that function expressions are wildcards
                if (checkMode && checkMode & 4 /* SkipContextSensitive */ && isContextSensitive(node)) {
                    // Skip parameters, return signature with return type that retains noncontextual parts so inferences can still be drawn in an early stage
                    if (!ts.getEffectiveReturnTypeNode(node) && !hasContextSensitiveParameters(node)) {
                        // Return plain anyFunctionType if there is no possibility we'll make inferences from the return type
                        var contextualSignature = getContextualSignature(node);
                        if (contextualSignature && couldContainTypeVariables(getReturnTypeOfSignature(contextualSignature))) {
                            var links = getNodeLinks(node);
                            if (links.contextFreeType) {
                                return links.contextFreeType;
                            }
                            var returnType = getReturnTypeFromBody(node, checkMode);
                            var returnOnlySignature = createSignature(undefined, undefined, undefined, ts.emptyArray, returnType, /*resolvedTypePredicate*/ undefined, 0, 0 /* None */);
                            var returnOnlyType = createAnonymousType(node.symbol, emptySymbols, [returnOnlySignature], ts.emptyArray, undefined, undefined);
                            returnOnlyType.objectFlags |= 2097152 /* NonInferrableType */;
                            return links.contextFreeType = returnOnlyType;
                        }
                    }
                    return anyFunctionType;
                }
                // Grammar checking
                var hasGrammarError = checkGrammarFunctionLikeDeclaration(node);
                if (!hasGrammarError && node.kind === 201 /* FunctionExpression */) {
                    checkGrammarForGenerator(node);
                }
                contextuallyCheckFunctionExpressionOrObjectLiteralMethod(node, checkMode);
                return getTypeOfSymbol(getSymbolOfNode(node));
            }
            function contextuallyCheckFunctionExpressionOrObjectLiteralMethod(node, checkMode) {
                var links = getNodeLinks(node);
                // Check if function expression is contextually typed and assign parameter types if so.
                if (!(links.flags & 1024 /* ContextChecked */)) {
                    var contextualSignature = getContextualSignature(node);
                    // If a type check is started at a function expression that is an argument of a function call, obtaining the
                    // contextual type may recursively get back to here during overload resolution of the call. If so, we will have
                    // already assigned contextual types.
                    if (!(links.flags & 1024 /* ContextChecked */)) {
                        links.flags |= 1024 /* ContextChecked */;
                        var signature = ts.firstOrUndefined(getSignaturesOfType(getTypeOfSymbol(getSymbolOfNode(node)), 0 /* Call */));
                        if (!signature) {
                            return;
                        }
                        if (isContextSensitive(node)) {
                            if (contextualSignature) {
                                var inferenceContext = getInferenceContext(node);
                                if (checkMode && checkMode & 2 /* Inferential */) {
                                    inferFromAnnotatedParameters(signature, contextualSignature, inferenceContext);
                                }
                                var instantiatedContextualSignature = inferenceContext ?
                                    instantiateSignature(contextualSignature, inferenceContext.mapper) : contextualSignature;
                                assignContextualParameterTypes(signature, instantiatedContextualSignature);
                            }
                            else {
                                // Force resolution of all parameter types such that the absence of a contextual type is consistently reflected.
                                assignNonContextualParameterTypes(signature);
                            }
                        }
                        if (contextualSignature && !getReturnTypeFromAnnotation(node) && !signature.resolvedReturnType) {
                            var returnType = getReturnTypeFromBody(node, checkMode);
                            if (!signature.resolvedReturnType) {
                                signature.resolvedReturnType = returnType;
                            }
                        }
                        checkSignatureDeclaration(node);
                    }
                }
            }
            function getReturnOrPromisedType(type, functionFlags) {
                var isGenerator = !!(functionFlags & 1 /* Generator */);
                var isAsync = !!(functionFlags & 2 /* Async */);
                return type && isGenerator ? getIterationTypeOfGeneratorFunctionReturnType(1 /* Return */, type, isAsync) || errorType :
                    type && isAsync ? getAwaitedType(type) || errorType :
                        type;
            }
            function checkFunctionExpressionOrObjectLiteralMethodDeferred(node) {
                ts.Debug.assert(node.kind !== 161 /* MethodDeclaration */ || ts.isObjectLiteralMethod(node));
                var functionFlags = ts.getFunctionFlags(node);
                var returnType = getReturnTypeFromAnnotation(node);
                checkAllCodePathsInNonVoidFunctionReturnOrThrow(node, returnType);
                if (node.body) {
                    if (!ts.getEffectiveReturnTypeNode(node)) {
                        // There are some checks that are only performed in getReturnTypeFromBody, that may produce errors
                        // we need. An example is the noImplicitAny errors resulting from widening the return expression
                        // of a function. Because checking of function expression bodies is deferred, there was never an
                        // appropriate time to do this during the main walk of the file (see the comment at the top of
                        // checkFunctionExpressionBodies). So it must be done now.
                        getReturnTypeOfSignature(getSignatureFromDeclaration(node));
                    }
                    if (node.body.kind === 223 /* Block */) {
                        checkSourceElement(node.body);
                    }
                    else {
                        // From within an async function you can return either a non-promise value or a promise. Any
                        // Promise/A+ compatible implementation will always assimilate any foreign promise, so we
                        // should not be checking assignability of a promise to the return type. Instead, we need to
                        // check assignability of the awaited type of the expression body against the promised type of
                        // its return type annotation.
                        var exprType = checkExpression(node.body);
                        var returnOrPromisedType = getReturnOrPromisedType(returnType, functionFlags);
                        if (returnOrPromisedType) {
                            if ((functionFlags & 3 /* AsyncGenerator */) === 2 /* Async */) { // Async function
                                var awaitedType = checkAwaitedType(exprType, node.body, ts.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member);
                                checkTypeAssignableToAndOptionallyElaborate(awaitedType, returnOrPromisedType, node.body, node.body);
                            }
                            else { // Normal function
                                checkTypeAssignableToAndOptionallyElaborate(exprType, returnOrPromisedType, node.body, node.body);
                            }
                        }
                    }
                }
            }
            function checkArithmeticOperandType(operand, type, diagnostic, isAwaitValid) {
                if (isAwaitValid === void 0) {
                    isAwaitValid = false;
                }
                if (!isTypeAssignableTo(type, numberOrBigIntType)) {
                    var awaitedType = isAwaitValid && getAwaitedTypeOfPromise(type);
                    errorAndMaybeSuggestAwait(operand, !!awaitedType && isTypeAssignableTo(awaitedType, numberOrBigIntType), diagnostic);
                    return false;
                }
                return true;
            }
            function isReadonlyAssignmentDeclaration(d) {
                if (!ts.isCallExpression(d)) {
                    return false;
                }
                if (!ts.isBindableObjectDefinePropertyCall(d)) {
                    return false;
                }
                var objectLitType = checkExpressionCached(d.arguments[2]);
                var valueType = getTypeOfPropertyOfType(objectLitType, "value");
                if (valueType) {
                    var writableProp = getPropertyOfType(objectLitType, "writable");
                    var writableType = writableProp && getTypeOfSymbol(writableProp);
                    if (!writableType || writableType === falseType || writableType === regularFalseType) {
                        return true;
                    }
                    // We include this definition whereupon we walk back and check the type at the declaration because
                    // The usual definition of `Object.defineProperty` will _not_ cause literal types to be preserved in the
                    // argument types, should the type be contextualized by the call itself.
                    if (writableProp && writableProp.valueDeclaration && ts.isPropertyAssignment(writableProp.valueDeclaration)) {
                        var initializer = writableProp.valueDeclaration.initializer;
                        var rawOriginalType = checkExpression(initializer);
                        if (rawOriginalType === falseType || rawOriginalType === regularFalseType) {
                            return true;
                        }
                    }
                    return false;
                }
                var setProp = getPropertyOfType(objectLitType, "set");
                return !setProp;
            }
            function isReadonlySymbol(symbol) {
                // The following symbols are considered read-only:
                // Properties with a 'readonly' modifier
                // Variables declared with 'const'
                // Get accessors without matching set accessors
                // Enum members
                // Object.defineProperty assignments with writable false or no setter
                // Unions and intersections of the above (unions and intersections eagerly set isReadonly on creation)
                return !!(ts.getCheckFlags(symbol) & 8 /* Readonly */ ||
                    symbol.flags & 4 /* Property */ && ts.getDeclarationModifierFlagsFromSymbol(symbol) & 64 /* Readonly */ ||
                    symbol.flags & 3 /* Variable */ && getDeclarationNodeFlagsFromSymbol(symbol) & 2 /* Const */ ||
                    symbol.flags & 98304 /* Accessor */ && !(symbol.flags & 65536 /* SetAccessor */) ||
                    symbol.flags & 8 /* EnumMember */ ||
                    ts.some(symbol.declarations, isReadonlyAssignmentDeclaration));
            }
            function isAssignmentToReadonlyEntity(expr, symbol, assignmentKind) {
                var _a, _b;
                if (assignmentKind === 0 /* None */) {
                    // no assigment means it doesn't matter whether the entity is readonly
                    return false;
                }
                if (isReadonlySymbol(symbol)) {
                    // Allow assignments to readonly properties within constructors of the same class declaration.
                    if (symbol.flags & 4 /* Property */ &&
                        ts.isAccessExpression(expr) &&
                        expr.expression.kind === 104 /* ThisKeyword */) {
                        // Look for if this is the constructor for the class that `symbol` is a property of.
                        var ctor = ts.getContainingFunction(expr);
                        if (!(ctor && ctor.kind === 162 /* Constructor */)) {
                            return true;
                        }
                        if (symbol.valueDeclaration) {
                            var isAssignmentDeclaration_1 = ts.isBinaryExpression(symbol.valueDeclaration);
                            var isLocalPropertyDeclaration = ctor.parent === symbol.valueDeclaration.parent;
                            var isLocalParameterProperty = ctor === symbol.valueDeclaration.parent;
                            var isLocalThisPropertyAssignment = isAssignmentDeclaration_1 && ((_a = symbol.parent) === null || _a === void 0 ? void 0 : _a.valueDeclaration) === ctor.parent;
                            var isLocalThisPropertyAssignmentConstructorFunction = isAssignmentDeclaration_1 && ((_b = symbol.parent) === null || _b === void 0 ? void 0 : _b.valueDeclaration) === ctor;
                            var isWriteableSymbol = isLocalPropertyDeclaration
                                || isLocalParameterProperty
                                || isLocalThisPropertyAssignment
                                || isLocalThisPropertyAssignmentConstructorFunction;
                            return !isWriteableSymbol;
                        }
                    }
                    return true;
                }
                if (ts.isAccessExpression(expr)) {
                    // references through namespace import should be readonly
                    var node = ts.skipParentheses(expr.expression);
                    if (node.kind === 75 /* Identifier */) {
                        var symbol_2 = getNodeLinks(node).resolvedSymbol;
                        if (symbol_2.flags & 2097152 /* Alias */) {
                            var declaration = getDeclarationOfAliasSymbol(symbol_2);
                            return !!declaration && declaration.kind === 256 /* NamespaceImport */;
                        }
                    }
                }
                return false;
            }
            function checkReferenceExpression(expr, invalidReferenceMessage, invalidOptionalChainMessage) {
                // References are combinations of identifiers, parentheses, and property accesses.
                var node = ts.skipOuterExpressions(expr, 2 /* Assertions */ | 1 /* Parentheses */);
                if (node.kind !== 75 /* Identifier */ && !ts.isAccessExpression(node)) {
                    error(expr, invalidReferenceMessage);
                    return false;
                }
                if (node.flags & 32 /* OptionalChain */) {
                    error(expr, invalidOptionalChainMessage);
                    return false;
                }
                return true;
            }
            function checkDeleteExpression(node) {
                checkExpression(node.expression);
                var expr = ts.skipParentheses(node.expression);
                if (!ts.isAccessExpression(expr)) {
                    error(expr, ts.Diagnostics.The_operand_of_a_delete_operator_must_be_a_property_reference);
                    return booleanType;
                }
                if (expr.kind === 194 /* PropertyAccessExpression */ && ts.isPrivateIdentifier(expr.name)) {
                    error(expr, ts.Diagnostics.The_operand_of_a_delete_operator_cannot_be_a_private_identifier);
                }
                var links = getNodeLinks(expr);
                var symbol = getExportSymbolOfValueSymbolIfExported(links.resolvedSymbol);
                if (symbol && isReadonlySymbol(symbol)) {
                    error(expr, ts.Diagnostics.The_operand_of_a_delete_operator_cannot_be_a_read_only_property);
                }
                return booleanType;
            }
            function checkTypeOfExpression(node) {
                checkExpression(node.expression);
                return typeofType;
            }
            function checkVoidExpression(node) {
                checkExpression(node.expression);
                return undefinedWideningType;
            }
            function isTopLevelAwait(node) {
                var container = ts.getThisContainer(node, /*includeArrowFunctions*/ true);
                return ts.isSourceFile(container);
            }
            function checkAwaitExpression(node) {
                // Grammar checking
                if (produceDiagnostics) {
                    if (!(node.flags & 32768 /* AwaitContext */)) {
                        if (isTopLevelAwait(node)) {
                            var sourceFile = ts.getSourceFileOfNode(node);
                            if (!hasParseDiagnostics(sourceFile)) {
                                var span = void 0;
                                if (!ts.isEffectiveExternalModule(sourceFile, compilerOptions)) {
                                    if (!span)
                                        span = ts.getSpanOfTokenAtPosition(sourceFile, node.pos);
                                    var diagnostic = ts.createFileDiagnostic(sourceFile, span.start, span.length, ts.Diagnostics.await_expressions_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_file_has_no_imports_or_exports_Consider_adding_an_empty_export_to_make_this_file_a_module);
                                    diagnostics.add(diagnostic);
                                }
                                if ((moduleKind !== ts.ModuleKind.ESNext && moduleKind !== ts.ModuleKind.System) || languageVersion < 4 /* ES2017 */) {
                                    span = ts.getSpanOfTokenAtPosition(sourceFile, node.pos);
                                    var diagnostic = ts.createFileDiagnostic(sourceFile, span.start, span.length, ts.Diagnostics.Top_level_await_expressions_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_the_target_option_is_set_to_es2017_or_higher);
                                    diagnostics.add(diagnostic);
                                }
                            }
                        }
                        else {
                            // use of 'await' in non-async function
                            var sourceFile = ts.getSourceFileOfNode(node);
                            if (!hasParseDiagnostics(sourceFile)) {
                                var span = ts.getSpanOfTokenAtPosition(sourceFile, node.pos);
                                var diagnostic = ts.createFileDiagnostic(sourceFile, span.start, span.length, ts.Diagnostics.await_expressions_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules);
                                var func = ts.getContainingFunction(node);
                                if (func && func.kind !== 162 /* Constructor */ && (ts.getFunctionFlags(func) & 2 /* Async */) === 0) {
                                    var relatedInfo = ts.createDiagnosticForNode(func, ts.Diagnostics.Did_you_mean_to_mark_this_function_as_async);
                                    ts.addRelatedInfo(diagnostic, relatedInfo);
                                }
                                diagnostics.add(diagnostic);
                            }
                        }
                    }
                    if (isInParameterInitializerBeforeContainingFunction(node)) {
                        error(node, ts.Diagnostics.await_expressions_cannot_be_used_in_a_parameter_initializer);
                    }
                }
                var operandType = checkExpression(node.expression);
                var awaitedType = checkAwaitedType(operandType, node, ts.Diagnostics.Type_of_await_operand_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member);
                if (awaitedType === operandType && awaitedType !== errorType && !(operandType.flags & 3 /* AnyOrUnknown */)) {
                    addErrorOrSuggestion(/*isError*/ false, ts.createDiagnosticForNode(node, ts.Diagnostics.await_has_no_effect_on_the_type_of_this_expression));
                }
                return awaitedType;
            }
            function checkPrefixUnaryExpression(node) {
                var operandType = checkExpression(node.operand);
                if (operandType === silentNeverType) {
                    return silentNeverType;
                }
                switch (node.operand.kind) {
                    case 8 /* NumericLiteral */:
                        switch (node.operator) {
                            case 40 /* MinusToken */:
                                return getFreshTypeOfLiteralType(getLiteralType(-node.operand.text));
                            case 39 /* PlusToken */:
                                return getFreshTypeOfLiteralType(getLiteralType(+node.operand.text));
                        }
                        break;
                    case 9 /* BigIntLiteral */:
                        if (node.operator === 40 /* MinusToken */) {
                            return getFreshTypeOfLiteralType(getLiteralType({
                                negative: true,
                                base10Value: ts.parsePseudoBigInt(node.operand.text)
                            }));
                        }
                }
                switch (node.operator) {
                    case 39 /* PlusToken */:
                    case 40 /* MinusToken */:
                    case 54 /* TildeToken */:
                        checkNonNullType(operandType, node.operand);
                        if (maybeTypeOfKind(operandType, 12288 /* ESSymbolLike */)) {
                            error(node.operand, ts.Diagnostics.The_0_operator_cannot_be_applied_to_type_symbol, ts.tokenToString(node.operator));
                        }
                        if (node.operator === 39 /* PlusToken */) {
                            if (maybeTypeOfKind(operandType, 2112 /* BigIntLike */)) {
                                error(node.operand, ts.Diagnostics.Operator_0_cannot_be_applied_to_type_1, ts.tokenToString(node.operator), typeToString(getBaseTypeOfLiteralType(operandType)));
                            }
                            return numberType;
                        }
                        return getUnaryResultType(operandType);
                    case 53 /* ExclamationToken */:
                        checkTruthinessExpression(node.operand);
                        var facts = getTypeFacts(operandType) & (4194304 /* Truthy */ | 8388608 /* Falsy */);
                        return facts === 4194304 /* Truthy */ ? falseType :
                            facts === 8388608 /* Falsy */ ? trueType :
                                booleanType;
                    case 45 /* PlusPlusToken */:
                    case 46 /* MinusMinusToken */:
                        var ok = checkArithmeticOperandType(node.operand, checkNonNullType(operandType, node.operand), ts.Diagnostics.An_arithmetic_operand_must_be_of_type_any_number_bigint_or_an_enum_type);
                        if (ok) {
                            // run check only if former checks succeeded to avoid reporting cascading errors
                            checkReferenceExpression(node.operand, ts.Diagnostics.The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access, ts.Diagnostics.The_operand_of_an_increment_or_decrement_operator_may_not_be_an_optional_property_access);
                        }
                        return getUnaryResultType(operandType);
                }
                return errorType;
            }
            function checkPostfixUnaryExpression(node) {
                var operandType = checkExpression(node.operand);
                if (operandType === silentNeverType) {
                    return silentNeverType;
                }
                var ok = checkArithmeticOperandType(node.operand, checkNonNullType(operandType, node.operand), ts.Diagnostics.An_arithmetic_operand_must_be_of_type_any_number_bigint_or_an_enum_type);
                if (ok) {
                    // run check only if former checks succeeded to avoid reporting cascading errors
                    checkReferenceExpression(node.operand, ts.Diagnostics.The_operand_of_an_increment_or_decrement_operator_must_be_a_variable_or_a_property_access, ts.Diagnostics.The_operand_of_an_increment_or_decrement_operator_may_not_be_an_optional_property_access);
                }
                return getUnaryResultType(operandType);
            }
            function getUnaryResultType(operandType) {
                if (maybeTypeOfKind(operandType, 2112 /* BigIntLike */)) {
                    return isTypeAssignableToKind(operandType, 3 /* AnyOrUnknown */) || maybeTypeOfKind(operandType, 296 /* NumberLike */)
                        ? numberOrBigIntType
                        : bigintType;
                }
                // If it's not a bigint type, implicit coercion will result in a number
                return numberType;
            }
            // Return true if type might be of the given kind. A union or intersection type might be of a given
            // kind if at least one constituent type is of the given kind.
            function maybeTypeOfKind(type, kind) {
                if (type.flags & kind & ~131072 /* GenericMappedType */ || kind & 131072 /* GenericMappedType */ && isGenericMappedType(type)) {
                    return true;
                }
                if (type.flags & 3145728 /* UnionOrIntersection */) {
                    var types = type.types;
                    for (var _i = 0, types_18 = types; _i < types_18.length; _i++) {
                        var t = types_18[_i];
                        if (maybeTypeOfKind(t, kind)) {
                            return true;
                        }
                    }
                }
                return false;
            }
            function isTypeAssignableToKind(source, kind, strict) {
                if (source.flags & kind) {
                    return true;
                }
                if (strict && source.flags & (3 /* AnyOrUnknown */ | 16384 /* Void */ | 32768 /* Undefined */ | 65536 /* Null */)) {
                    return false;
                }
                return !!(kind & 296 /* NumberLike */) && isTypeAssignableTo(source, numberType) ||
                    !!(kind & 2112 /* BigIntLike */) && isTypeAssignableTo(source, bigintType) ||
                    !!(kind & 132 /* StringLike */) && isTypeAssignableTo(source, stringType) ||
                    !!(kind & 528 /* BooleanLike */) && isTypeAssignableTo(source, booleanType) ||
                    !!(kind & 16384 /* Void */) && isTypeAssignableTo(source, voidType) ||
                    !!(kind & 131072 /* Never */) && isTypeAssignableTo(source, neverType) ||
                    !!(kind & 65536 /* Null */) && isTypeAssignableTo(source, nullType) ||
                    !!(kind & 32768 /* Undefined */) && isTypeAssignableTo(source, undefinedType) ||
                    !!(kind & 4096 /* ESSymbol */) && isTypeAssignableTo(source, esSymbolType) ||
                    !!(kind & 67108864 /* NonPrimitive */) && isTypeAssignableTo(source, nonPrimitiveType);
            }
            function allTypesAssignableToKind(source, kind, strict) {
                return source.flags & 1048576 /* Union */ ?
                    ts.every(source.types, function (subType) { return allTypesAssignableToKind(subType, kind, strict); }) :
                    isTypeAssignableToKind(source, kind, strict);
            }
            function isConstEnumObjectType(type) {
                return !!(ts.getObjectFlags(type) & 16 /* Anonymous */) && !!type.symbol && isConstEnumSymbol(type.symbol);
            }
            function isConstEnumSymbol(symbol) {
                return (symbol.flags & 128 /* ConstEnum */) !== 0;
            }
            function checkInstanceOfExpression(left, right, leftType, rightType) {
                if (leftType === silentNeverType || rightType === silentNeverType) {
                    return silentNeverType;
                }
                // TypeScript 1.0 spec (April 2014): 4.15.4
                // The instanceof operator requires the left operand to be of type Any, an object type, or a type parameter type,
                // and the right operand to be of type Any, a subtype of the 'Function' interface type, or have a call or construct signature.
                // The result is always of the Boolean primitive type.
                // NOTE: do not raise error if leftType is unknown as related error was already reported
                if (!isTypeAny(leftType) &&
                    allTypesAssignableToKind(leftType, 131068 /* Primitive */)) {
                    error(left, ts.Diagnostics.The_left_hand_side_of_an_instanceof_expression_must_be_of_type_any_an_object_type_or_a_type_parameter);
                }
                // NOTE: do not raise error if right is unknown as related error was already reported
                if (!(isTypeAny(rightType) || typeHasCallOrConstructSignatures(rightType) || isTypeSubtypeOf(rightType, globalFunctionType))) {
                    error(right, ts.Diagnostics.The_right_hand_side_of_an_instanceof_expression_must_be_of_type_any_or_of_a_type_assignable_to_the_Function_interface_type);
                }
                return booleanType;
            }
            function checkInExpression(left, right, leftType, rightType) {
                if (leftType === silentNeverType || rightType === silentNeverType) {
                    return silentNeverType;
                }
                leftType = checkNonNullType(leftType, left);
                rightType = checkNonNullType(rightType, right);
                // TypeScript 1.0 spec (April 2014): 4.15.5
                // The in operator requires the left operand to be of type Any, the String primitive type, or the Number primitive type,
                // and the right operand to be of type Any, an object type, or a type parameter type.
                // The result is always of the Boolean primitive type.
                if (!(isTypeComparableTo(leftType, stringType) || isTypeAssignableToKind(leftType, 296 /* NumberLike */ | 12288 /* ESSymbolLike */))) {
                    error(left, ts.Diagnostics.The_left_hand_side_of_an_in_expression_must_be_of_type_any_string_number_or_symbol);
                }
                if (!allTypesAssignableToKind(rightType, 67108864 /* NonPrimitive */ | 58982400 /* InstantiableNonPrimitive */)) {
                    error(right, ts.Diagnostics.The_right_hand_side_of_an_in_expression_must_be_of_type_any_an_object_type_or_a_type_parameter);
                }
                return booleanType;
            }
            function checkObjectLiteralAssignment(node, sourceType, rightIsThis) {
                var properties = node.properties;
                if (strictNullChecks && properties.length === 0) {
                    return checkNonNullType(sourceType, node);
                }
                for (var i = 0; i < properties.length; i++) {
                    checkObjectLiteralDestructuringPropertyAssignment(node, sourceType, i, properties, rightIsThis);
                }
                return sourceType;
            }
            /** Note: If property cannot be a SpreadAssignment, then allProperties does not need to be provided */
            function checkObjectLiteralDestructuringPropertyAssignment(node, objectLiteralType, propertyIndex, allProperties, rightIsThis) {
                if (rightIsThis === void 0) {
                    rightIsThis = false;
                }
                var properties = node.properties;
                var property = properties[propertyIndex];
                if (property.kind === 281 /* PropertyAssignment */ || property.kind === 282 /* ShorthandPropertyAssignment */) {
                    var name = property.name;
                    var exprType = getLiteralTypeFromPropertyName(name);
                    if (isTypeUsableAsPropertyName(exprType)) {
                        var text = getPropertyNameFromType(exprType);
                        var prop = getPropertyOfType(objectLiteralType, text);
                        if (prop) {
                            markPropertyAsReferenced(prop, property, rightIsThis);
                            checkPropertyAccessibility(property, /*isSuper*/ false, objectLiteralType, prop);
                        }
                    }
                    var elementType = getIndexedAccessType(objectLiteralType, exprType, name);
                    var type = getFlowTypeOfDestructuring(property, elementType);
                    return checkDestructuringAssignment(property.kind === 282 /* ShorthandPropertyAssignment */ ? property : property.initializer, type);
                }
                else if (property.kind === 283 /* SpreadAssignment */) {
                    if (propertyIndex < properties.length - 1) {
                        error(property, ts.Diagnostics.A_rest_element_must_be_last_in_a_destructuring_pattern);
                    }
                    else {
                        if (languageVersion < 99 /* ESNext */) {
                            checkExternalEmitHelpers(property, 4 /* Rest */);
                        }
                        var nonRestNames = [];
                        if (allProperties) {
                            for (var _i = 0, allProperties_1 = allProperties; _i < allProperties_1.length; _i++) {
                                var otherProperty = allProperties_1[_i];
                                if (!ts.isSpreadAssignment(otherProperty)) {
                                    nonRestNames.push(otherProperty.name);
                                }
                            }
                        }
                        var type = getRestType(objectLiteralType, nonRestNames, objectLiteralType.symbol);
                        checkGrammarForDisallowedTrailingComma(allProperties, ts.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma);
                        return checkDestructuringAssignment(property.expression, type);
                    }
                }
                else {
                    error(property, ts.Diagnostics.Property_assignment_expected);
                }
            }
            function checkArrayLiteralAssignment(node, sourceType, checkMode) {
                var elements = node.elements;
                if (languageVersion < 2 /* ES2015 */ && compilerOptions.downlevelIteration) {
                    checkExternalEmitHelpers(node, 512 /* Read */);
                }
                // This elementType will be used if the specific property corresponding to this index is not
                // present (aka the tuple element property). This call also checks that the parentType is in
                // fact an iterable or array (depending on target language).
                var elementType = checkIteratedTypeOrElementType(65 /* Destructuring */, sourceType, undefinedType, node) || errorType;
                for (var i = 0; i < elements.length; i++) {
                    checkArrayLiteralDestructuringElementAssignment(node, sourceType, i, elementType, checkMode);
                }
                return sourceType;
            }
            function checkArrayLiteralDestructuringElementAssignment(node, sourceType, elementIndex, elementType, checkMode) {
                var elements = node.elements;
                var element = elements[elementIndex];
                if (element.kind !== 215 /* OmittedExpression */) {
                    if (element.kind !== 213 /* SpreadElement */) {
                        var indexType = getLiteralType(elementIndex);
                        if (isArrayLikeType(sourceType)) {
                            // We create a synthetic expression so that getIndexedAccessType doesn't get confused
                            // when the element is a SyntaxKind.ElementAccessExpression.
                            var accessFlags = hasDefaultValue(element) ? 8 /* NoTupleBoundsCheck */ : 0;
                            var elementType_2 = getIndexedAccessTypeOrUndefined(sourceType, indexType, createSyntheticExpression(element, indexType), accessFlags) || errorType;
                            var assignedType = hasDefaultValue(element) ? getTypeWithFacts(elementType_2, 524288 /* NEUndefined */) : elementType_2;
                            var type = getFlowTypeOfDestructuring(element, assignedType);
                            return checkDestructuringAssignment(element, type, checkMode);
                        }
                        return checkDestructuringAssignment(element, elementType, checkMode);
                    }
                    if (elementIndex < elements.length - 1) {
                        error(element, ts.Diagnostics.A_rest_element_must_be_last_in_a_destructuring_pattern);
                    }
                    else {
                        var restExpression = element.expression;
                        if (restExpression.kind === 209 /* BinaryExpression */ && restExpression.operatorToken.kind === 62 /* EqualsToken */) {
                            error(restExpression.operatorToken, ts.Diagnostics.A_rest_element_cannot_have_an_initializer);
                        }
                        else {
                            checkGrammarForDisallowedTrailingComma(node.elements, ts.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma);
                            var type = everyType(sourceType, isTupleType) ?
                                mapType(sourceType, function (t) { return sliceTupleType(t, elementIndex); }) :
                                createArrayType(elementType);
                            return checkDestructuringAssignment(restExpression, type, checkMode);
                        }
                    }
                }
                return undefined;
            }
            function checkDestructuringAssignment(exprOrAssignment, sourceType, checkMode, rightIsThis) {
                var target;
                if (exprOrAssignment.kind === 282 /* ShorthandPropertyAssignment */) {
                    var prop = exprOrAssignment;
                    if (prop.objectAssignmentInitializer) {
                        // In strict null checking mode, if a default value of a non-undefined type is specified, remove
                        // undefined from the final type.
                        if (strictNullChecks &&
                            !(getFalsyFlags(checkExpression(prop.objectAssignmentInitializer)) & 32768 /* Undefined */)) {
                            sourceType = getTypeWithFacts(sourceType, 524288 /* NEUndefined */);
                        }
                        checkBinaryLikeExpression(prop.name, prop.equalsToken, prop.objectAssignmentInitializer, checkMode);
                    }
                    target = exprOrAssignment.name;
                }
                else {
                    target = exprOrAssignment;
                }
                if (target.kind === 209 /* BinaryExpression */ && target.operatorToken.kind === 62 /* EqualsToken */) {
                    checkBinaryExpression(target, checkMode);
                    target = target.left;
                }
                if (target.kind === 193 /* ObjectLiteralExpression */) {
                    return checkObjectLiteralAssignment(target, sourceType, rightIsThis);
                }
                if (target.kind === 192 /* ArrayLiteralExpression */) {
                    return checkArrayLiteralAssignment(target, sourceType, checkMode);
                }
                return checkReferenceAssignment(target, sourceType, checkMode);
            }
            function checkReferenceAssignment(target, sourceType, checkMode) {
                var targetType = checkExpression(target, checkMode);
                var error = target.parent.kind === 283 /* SpreadAssignment */ ?
                    ts.Diagnostics.The_target_of_an_object_rest_assignment_must_be_a_variable_or_a_property_access :
                    ts.Diagnostics.The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access;
                var optionalError = target.parent.kind === 283 /* SpreadAssignment */ ?
                    ts.Diagnostics.The_target_of_an_object_rest_assignment_may_not_be_an_optional_property_access :
                    ts.Diagnostics.The_left_hand_side_of_an_assignment_expression_may_not_be_an_optional_property_access;
                if (checkReferenceExpression(target, error, optionalError)) {
                    checkTypeAssignableToAndOptionallyElaborate(sourceType, targetType, target, target);
                }
                if (ts.isPrivateIdentifierPropertyAccessExpression(target)) {
                    checkExternalEmitHelpers(target.parent, 524288 /* ClassPrivateFieldSet */);
                }
                return sourceType;
            }
            /**
             * This is a *shallow* check: An expression is side-effect-free if the
             * evaluation of the expression *itself* cannot produce side effects.
             * For example, x++ / 3 is side-effect free because the / operator
             * does not have side effects.
             * The intent is to "smell test" an expression for correctness in positions where
             * its value is discarded (e.g. the left side of the comma operator).
             */
            function isSideEffectFree(node) {
                node = ts.skipParentheses(node);
                switch (node.kind) {
                    case 75 /* Identifier */:
                    case 10 /* StringLiteral */:
                    case 13 /* RegularExpressionLiteral */:
                    case 198 /* TaggedTemplateExpression */:
                    case 211 /* TemplateExpression */:
                    case 14 /* NoSubstitutionTemplateLiteral */:
                    case 8 /* NumericLiteral */:
                    case 9 /* BigIntLiteral */:
                    case 106 /* TrueKeyword */:
                    case 91 /* FalseKeyword */:
                    case 100 /* NullKeyword */:
                    case 146 /* UndefinedKeyword */:
                    case 201 /* FunctionExpression */:
                    case 214 /* ClassExpression */:
                    case 202 /* ArrowFunction */:
                    case 192 /* ArrayLiteralExpression */:
                    case 193 /* ObjectLiteralExpression */:
                    case 204 /* TypeOfExpression */:
                    case 218 /* NonNullExpression */:
                    case 267 /* JsxSelfClosingElement */:
                    case 266 /* JsxElement */:
                        return true;
                    case 210 /* ConditionalExpression */:
                        return isSideEffectFree(node.whenTrue) &&
                            isSideEffectFree(node.whenFalse);
                    case 209 /* BinaryExpression */:
                        if (ts.isAssignmentOperator(node.operatorToken.kind)) {
                            return false;
                        }
                        return isSideEffectFree(node.left) &&
                            isSideEffectFree(node.right);
                    case 207 /* PrefixUnaryExpression */:
                    case 208 /* PostfixUnaryExpression */:
                        // Unary operators ~, !, +, and - have no side effects.
                        // The rest do.
                        switch (node.operator) {
                            case 53 /* ExclamationToken */:
                            case 39 /* PlusToken */:
                            case 40 /* MinusToken */:
                            case 54 /* TildeToken */:
                                return true;
                        }
                        return false;
                    // Some forms listed here for clarity
                    case 205 /* VoidExpression */: // Explicit opt-out
                    case 199 /* TypeAssertionExpression */: // Not SEF, but can produce useful type warnings
                    case 217 /* AsExpression */: // Not SEF, but can produce useful type warnings
                    default:
                        return false;
                }
            }
            function isTypeEqualityComparableTo(source, target) {
                return (target.flags & 98304 /* Nullable */) !== 0 || isTypeComparableTo(source, target);
            }
            var CheckBinaryExpressionState;
            (function (CheckBinaryExpressionState) {
                CheckBinaryExpressionState[CheckBinaryExpressionState["MaybeCheckLeft"] = 0] = "MaybeCheckLeft";
                CheckBinaryExpressionState[CheckBinaryExpressionState["CheckRight"] = 1] = "CheckRight";
                CheckBinaryExpressionState[CheckBinaryExpressionState["FinishCheck"] = 2] = "FinishCheck";
            })(CheckBinaryExpressionState || (CheckBinaryExpressionState = {}));
            function checkBinaryExpression(node, checkMode) {
                var workStacks = {
                    expr: [node],
                    state: [0 /* MaybeCheckLeft */],
                    leftType: [undefined]
                };
                var stackIndex = 0;
                var lastResult;
                while (stackIndex >= 0) {
                    node = workStacks.expr[stackIndex];
                    switch (workStacks.state[stackIndex]) {
                        case 0 /* MaybeCheckLeft */: {
                            if (ts.isInJSFile(node) && ts.getAssignedExpandoInitializer(node)) {
                                finishInvocation(checkExpression(node.right, checkMode));
                                break;
                            }
                            checkGrammarNullishCoalesceWithLogicalExpression(node);
                            var operator = node.operatorToken.kind;
                            if (operator === 62 /* EqualsToken */ && (node.left.kind === 193 /* ObjectLiteralExpression */ || node.left.kind === 192 /* ArrayLiteralExpression */)) {
                                finishInvocation(checkDestructuringAssignment(node.left, checkExpression(node.right, checkMode), checkMode, node.right.kind === 104 /* ThisKeyword */));
                                break;
                            }
                            advanceState(1 /* CheckRight */);
                            maybeCheckExpression(node.left);
                            break;
                        }
                        case 1 /* CheckRight */: {
                            var leftType = lastResult;
                            workStacks.leftType[stackIndex] = leftType;
                            var operator = node.operatorToken.kind;
                            if (operator === 55 /* AmpersandAmpersandToken */ || operator === 56 /* BarBarToken */ || operator === 60 /* QuestionQuestionToken */) {
                                checkTruthinessOfType(leftType, node.left);
                            }
                            advanceState(2 /* FinishCheck */);
                            maybeCheckExpression(node.right);
                            break;
                        }
                        case 2 /* FinishCheck */: {
                            var leftType = workStacks.leftType[stackIndex];
                            var rightType = lastResult;
                            finishInvocation(checkBinaryLikeExpressionWorker(node.left, node.operatorToken, node.right, leftType, rightType, node));
                            break;
                        }
                        default: return ts.Debug.fail("Invalid state " + workStacks.state[stackIndex] + " for checkBinaryExpression");
                    }
                }
                return lastResult;
                function finishInvocation(result) {
                    lastResult = result;
                    stackIndex--;
                }
                /**
                 * Note that `advanceState` sets the _current_ head state, and that `maybeCheckExpression` potentially pushes on a new
                 * head state; so `advanceState` must be called before any `maybeCheckExpression` during a state's execution.
                 */
                function advanceState(nextState) {
                    workStacks.state[stackIndex] = nextState;
                }
                function maybeCheckExpression(node) {
                    if (ts.isBinaryExpression(node)) {
                        stackIndex++;
                        workStacks.expr[stackIndex] = node;
                        workStacks.state[stackIndex] = 0 /* MaybeCheckLeft */;
                        workStacks.leftType[stackIndex] = undefined;
                    }
                    else {
                        lastResult = checkExpression(node, checkMode);
                    }
                }
            }
            function checkGrammarNullishCoalesceWithLogicalExpression(node) {
                var left = node.left, operatorToken = node.operatorToken, right = node.right;
                if (operatorToken.kind === 60 /* QuestionQuestionToken */) {
                    if (ts.isBinaryExpression(left) && (left.operatorToken.kind === 56 /* BarBarToken */ || left.operatorToken.kind === 55 /* AmpersandAmpersandToken */)) {
                        grammarErrorOnNode(left, ts.Diagnostics._0_and_1_operations_cannot_be_mixed_without_parentheses, ts.tokenToString(left.operatorToken.kind), ts.tokenToString(operatorToken.kind));
                    }
                    if (ts.isBinaryExpression(right) && (right.operatorToken.kind === 56 /* BarBarToken */ || right.operatorToken.kind === 55 /* AmpersandAmpersandToken */)) {
                        grammarErrorOnNode(right, ts.Diagnostics._0_and_1_operations_cannot_be_mixed_without_parentheses, ts.tokenToString(right.operatorToken.kind), ts.tokenToString(operatorToken.kind));
                    }
                }
            }
            // Note that this and `checkBinaryExpression` above should behave mostly the same, except this elides some
            // expression-wide checks and does not use a work stack to fold nested binary expressions into the same callstack frame
            function checkBinaryLikeExpression(left, operatorToken, right, checkMode, errorNode) {
                var operator = operatorToken.kind;
                if (operator === 62 /* EqualsToken */ && (left.kind === 193 /* ObjectLiteralExpression */ || left.kind === 192 /* ArrayLiteralExpression */)) {
                    return checkDestructuringAssignment(left, checkExpression(right, checkMode), checkMode, right.kind === 104 /* ThisKeyword */);
                }
                var leftType;
                if (operator === 55 /* AmpersandAmpersandToken */ || operator === 56 /* BarBarToken */ || operator === 60 /* QuestionQuestionToken */) {
                    leftType = checkTruthinessExpression(left, checkMode);
                }
                else {
                    leftType = checkExpression(left, checkMode);
                }
                var rightType = checkExpression(right, checkMode);
                return checkBinaryLikeExpressionWorker(left, operatorToken, right, leftType, rightType, errorNode);
            }
            function checkBinaryLikeExpressionWorker(left, operatorToken, right, leftType, rightType, errorNode) {
                var operator = operatorToken.kind;
                switch (operator) {
                    case 41 /* AsteriskToken */:
                    case 42 /* AsteriskAsteriskToken */:
                    case 65 /* AsteriskEqualsToken */:
                    case 66 /* AsteriskAsteriskEqualsToken */:
                    case 43 /* SlashToken */:
                    case 67 /* SlashEqualsToken */:
                    case 44 /* PercentToken */:
                    case 68 /* PercentEqualsToken */:
                    case 40 /* MinusToken */:
                    case 64 /* MinusEqualsToken */:
                    case 47 /* LessThanLessThanToken */:
                    case 69 /* LessThanLessThanEqualsToken */:
                    case 48 /* GreaterThanGreaterThanToken */:
                    case 70 /* GreaterThanGreaterThanEqualsToken */:
                    case 49 /* GreaterThanGreaterThanGreaterThanToken */:
                    case 71 /* GreaterThanGreaterThanGreaterThanEqualsToken */:
                    case 51 /* BarToken */:
                    case 73 /* BarEqualsToken */:
                    case 52 /* CaretToken */:
                    case 74 /* CaretEqualsToken */:
                    case 50 /* AmpersandToken */:
                    case 72 /* AmpersandEqualsToken */:
                        if (leftType === silentNeverType || rightType === silentNeverType) {
                            return silentNeverType;
                        }
                        leftType = checkNonNullType(leftType, left);
                        rightType = checkNonNullType(rightType, right);
                        var suggestedOperator = void 0;
                        // if a user tries to apply a bitwise operator to 2 boolean operands
                        // try and return them a helpful suggestion
                        if ((leftType.flags & 528 /* BooleanLike */) &&
                            (rightType.flags & 528 /* BooleanLike */) &&
                            (suggestedOperator = getSuggestedBooleanOperator(operatorToken.kind)) !== undefined) {
                            error(errorNode || operatorToken, ts.Diagnostics.The_0_operator_is_not_allowed_for_boolean_types_Consider_using_1_instead, ts.tokenToString(operatorToken.kind), ts.tokenToString(suggestedOperator));
                            return numberType;
                        }
                        else {
                            // otherwise just check each operand separately and report errors as normal
                            var leftOk = checkArithmeticOperandType(left, leftType, ts.Diagnostics.The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type, /*isAwaitValid*/ true);
                            var rightOk = checkArithmeticOperandType(right, rightType, ts.Diagnostics.The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type, /*isAwaitValid*/ true);
                            var resultType_1;
                            // If both are any or unknown, allow operation; assume it will resolve to number
                            if ((isTypeAssignableToKind(leftType, 3 /* AnyOrUnknown */) && isTypeAssignableToKind(rightType, 3 /* AnyOrUnknown */)) ||
                                // Or, if neither could be bigint, implicit coercion results in a number result
                                !(maybeTypeOfKind(leftType, 2112 /* BigIntLike */) || maybeTypeOfKind(rightType, 2112 /* BigIntLike */))) {
                                resultType_1 = numberType;
                            }
                            // At least one is assignable to bigint, so check that both are
                            else if (bothAreBigIntLike(leftType, rightType)) {
                                switch (operator) {
                                    case 49 /* GreaterThanGreaterThanGreaterThanToken */:
                                    case 71 /* GreaterThanGreaterThanGreaterThanEqualsToken */:
                                        reportOperatorError();
                                }
                                resultType_1 = bigintType;
                            }
                            // Exactly one of leftType/rightType is assignable to bigint
                            else {
                                reportOperatorError(bothAreBigIntLike);
                                resultType_1 = errorType;
                            }
                            if (leftOk && rightOk) {
                                checkAssignmentOperator(resultType_1);
                            }
                            return resultType_1;
                        }
                    case 39 /* PlusToken */:
                    case 63 /* PlusEqualsToken */:
                        if (leftType === silentNeverType || rightType === silentNeverType) {
                            return silentNeverType;
                        }
                        if (!isTypeAssignableToKind(leftType, 132 /* StringLike */) && !isTypeAssignableToKind(rightType, 132 /* StringLike */)) {
                            leftType = checkNonNullType(leftType, left);
                            rightType = checkNonNullType(rightType, right);
                        }
                        var resultType = void 0;
                        if (isTypeAssignableToKind(leftType, 296 /* NumberLike */, /*strict*/ true) && isTypeAssignableToKind(rightType, 296 /* NumberLike */, /*strict*/ true)) {
                            // Operands of an enum type are treated as having the primitive type Number.
                            // If both operands are of the Number primitive type, the result is of the Number primitive type.
                            resultType = numberType;
                        }
                        else if (isTypeAssignableToKind(leftType, 2112 /* BigIntLike */, /*strict*/ true) && isTypeAssignableToKind(rightType, 2112 /* BigIntLike */, /*strict*/ true)) {
                            // If both operands are of the BigInt primitive type, the result is of the BigInt primitive type.
                            resultType = bigintType;
                        }
                        else if (isTypeAssignableToKind(leftType, 132 /* StringLike */, /*strict*/ true) || isTypeAssignableToKind(rightType, 132 /* StringLike */, /*strict*/ true)) {
                            // If one or both operands are of the String primitive type, the result is of the String primitive type.
                            resultType = stringType;
                        }
                        else if (isTypeAny(leftType) || isTypeAny(rightType)) {
                            // Otherwise, the result is of type Any.
                            // NOTE: unknown type here denotes error type. Old compiler treated this case as any type so do we.
                            resultType = leftType === errorType || rightType === errorType ? errorType : anyType;
                        }
                        // Symbols are not allowed at all in arithmetic expressions
                        if (resultType && !checkForDisallowedESSymbolOperand(operator)) {
                            return resultType;
                        }
                        if (!resultType) {
                            // Types that have a reasonably good chance of being a valid operand type.
                            // If both types have an awaited type of one of these, we'll assume the user
                            // might be missing an await without doing an exhaustive check that inserting
                            // await(s) will actually be a completely valid binary expression.
                            var closeEnoughKind_1 = 296 /* NumberLike */ | 2112 /* BigIntLike */ | 132 /* StringLike */ | 3 /* AnyOrUnknown */;
                            reportOperatorError(function (left, right) {
                                return isTypeAssignableToKind(left, closeEnoughKind_1) &&
                                    isTypeAssignableToKind(right, closeEnoughKind_1);
                            });
                            return anyType;
                        }
                        if (operator === 63 /* PlusEqualsToken */) {
                            checkAssignmentOperator(resultType);
                        }
                        return resultType;
                    case 29 /* LessThanToken */:
                    case 31 /* GreaterThanToken */:
                    case 32 /* LessThanEqualsToken */:
                    case 33 /* GreaterThanEqualsToken */:
                        if (checkForDisallowedESSymbolOperand(operator)) {
                            leftType = getBaseTypeOfLiteralType(checkNonNullType(leftType, left));
                            rightType = getBaseTypeOfLiteralType(checkNonNullType(rightType, right));
                            reportOperatorErrorUnless(function (left, right) {
                                return isTypeComparableTo(left, right) || isTypeComparableTo(right, left) || (isTypeAssignableTo(left, numberOrBigIntType) && isTypeAssignableTo(right, numberOrBigIntType));
                            });
                        }
                        return booleanType;
                    case 34 /* EqualsEqualsToken */:
                    case 35 /* ExclamationEqualsToken */:
                    case 36 /* EqualsEqualsEqualsToken */:
                    case 37 /* ExclamationEqualsEqualsToken */:
                        reportOperatorErrorUnless(function (left, right) { return isTypeEqualityComparableTo(left, right) || isTypeEqualityComparableTo(right, left); });
                        return booleanType;
                    case 98 /* InstanceOfKeyword */:
                        return checkInstanceOfExpression(left, right, leftType, rightType);
                    case 97 /* InKeyword */:
                        return checkInExpression(left, right, leftType, rightType);
                    case 55 /* AmpersandAmpersandToken */:
                        return getTypeFacts(leftType) & 4194304 /* Truthy */ ?
                            getUnionType([extractDefinitelyFalsyTypes(strictNullChecks ? leftType : getBaseTypeOfLiteralType(rightType)), rightType]) :
                            leftType;
                    case 56 /* BarBarToken */:
                        return getTypeFacts(leftType) & 8388608 /* Falsy */ ?
                            getUnionType([removeDefinitelyFalsyTypes(leftType), rightType], 2 /* Subtype */) :
                            leftType;
                    case 60 /* QuestionQuestionToken */:
                        return getTypeFacts(leftType) & 262144 /* EQUndefinedOrNull */ ?
                            getUnionType([getNonNullableType(leftType), rightType], 2 /* Subtype */) :
                            leftType;
                    case 62 /* EqualsToken */:
                        var declKind = ts.isBinaryExpression(left.parent) ? ts.getAssignmentDeclarationKind(left.parent) : 0 /* None */;
                        checkAssignmentDeclaration(declKind, rightType);
                        if (isAssignmentDeclaration(declKind)) {
                            if (!(rightType.flags & 524288 /* Object */) ||
                                declKind !== 2 /* ModuleExports */ &&
                                    declKind !== 6 /* Prototype */ &&
                                    !isEmptyObjectType(rightType) &&
                                    !isFunctionObjectType(rightType) &&
                                    !(ts.getObjectFlags(rightType) & 1 /* Class */)) {
                                // don't check assignability of module.exports=, C.prototype=, or expando types because they will necessarily be incomplete
                                checkAssignmentOperator(rightType);
                            }
                            return leftType;
                        }
                        else {
                            checkAssignmentOperator(rightType);
                            return getRegularTypeOfObjectLiteral(rightType);
                        }
                    case 27 /* CommaToken */:
                        if (!compilerOptions.allowUnreachableCode && isSideEffectFree(left) && !isEvalNode(right)) {
                            error(left, ts.Diagnostics.Left_side_of_comma_operator_is_unused_and_has_no_side_effects);
                        }
                        return rightType;
                    default:
                        return ts.Debug.fail();
                }
                function bothAreBigIntLike(left, right) {
                    return isTypeAssignableToKind(left, 2112 /* BigIntLike */) && isTypeAssignableToKind(right, 2112 /* BigIntLike */);
                }
                function checkAssignmentDeclaration(kind, rightType) {
                    if (kind === 2 /* ModuleExports */) {
                        for (var _i = 0, _a = getPropertiesOfObjectType(rightType); _i < _a.length; _i++) {
                            var prop = _a[_i];
                            var propType = getTypeOfSymbol(prop);
                            if (propType.symbol && propType.symbol.flags & 32 /* Class */) {
                                var name = prop.escapedName;
                                var symbol = resolveName(prop.valueDeclaration, name, 788968 /* Type */, undefined, name, /*isUse*/ false);
                                if (symbol && symbol.declarations.some(ts.isJSDocTypedefTag)) {
                                    grammarErrorOnNode(symbol.declarations[0], ts.Diagnostics.Duplicate_identifier_0, ts.unescapeLeadingUnderscores(name));
                                    return grammarErrorOnNode(prop.valueDeclaration, ts.Diagnostics.Duplicate_identifier_0, ts.unescapeLeadingUnderscores(name));
                                }
                            }
                        }
                    }
                }
                function isEvalNode(node) {
                    return node.kind === 75 /* Identifier */ && node.escapedText === "eval";
                }
                // Return true if there was no error, false if there was an error.
                function checkForDisallowedESSymbolOperand(operator) {
                    var offendingSymbolOperand = maybeTypeOfKind(leftType, 12288 /* ESSymbolLike */) ? left :
                        maybeTypeOfKind(rightType, 12288 /* ESSymbolLike */) ? right :
                            undefined;
                    if (offendingSymbolOperand) {
                        error(offendingSymbolOperand, ts.Diagnostics.The_0_operator_cannot_be_applied_to_type_symbol, ts.tokenToString(operator));
                        return false;
                    }
                    return true;
                }
                function getSuggestedBooleanOperator(operator) {
                    switch (operator) {
                        case 51 /* BarToken */:
                        case 73 /* BarEqualsToken */:
                            return 56 /* BarBarToken */;
                        case 52 /* CaretToken */:
                        case 74 /* CaretEqualsToken */:
                            return 37 /* ExclamationEqualsEqualsToken */;
                        case 50 /* AmpersandToken */:
                        case 72 /* AmpersandEqualsToken */:
                            return 55 /* AmpersandAmpersandToken */;
                        default:
                            return undefined;
                    }
                }
                function checkAssignmentOperator(valueType) {
                    if (produceDiagnostics && ts.isAssignmentOperator(operator)) {
                        // TypeScript 1.0 spec (April 2014): 4.17
                        // An assignment of the form
                        //    VarExpr = ValueExpr
                        // requires VarExpr to be classified as a reference
                        // A compound assignment furthermore requires VarExpr to be classified as a reference (section 4.1)
                        // and the type of the non-compound operation to be assignable to the type of VarExpr.
                        if (checkReferenceExpression(left, ts.Diagnostics.The_left_hand_side_of_an_assignment_expression_must_be_a_variable_or_a_property_access, ts.Diagnostics.The_left_hand_side_of_an_assignment_expression_may_not_be_an_optional_property_access)
                            && (!ts.isIdentifier(left) || ts.unescapeLeadingUnderscores(left.escapedText) !== "exports")) {
                            // to avoid cascading errors check assignability only if 'isReference' check succeeded and no errors were reported
                            checkTypeAssignableToAndOptionallyElaborate(valueType, leftType, left, right);
                        }
                    }
                }
                function isAssignmentDeclaration(kind) {
                    switch (kind) {
                        case 2 /* ModuleExports */:
                            return true;
                        case 1 /* ExportsProperty */:
                        case 5 /* Property */:
                        case 6 /* Prototype */:
                        case 3 /* PrototypeProperty */:
                        case 4 /* ThisProperty */:
                            var symbol = getSymbolOfNode(left);
                            var init = ts.getAssignedExpandoInitializer(right);
                            return init && ts.isObjectLiteralExpression(init) &&
                                symbol && ts.hasEntries(symbol.exports);
                        default:
                            return false;
                    }
                }
                /**
                 * Returns true if an error is reported
                 */
                function reportOperatorErrorUnless(typesAreCompatible) {
                    if (!typesAreCompatible(leftType, rightType)) {
                        reportOperatorError(typesAreCompatible);
                        return true;
                    }
                    return false;
                }
                function reportOperatorError(isRelated) {
                    var _a;
                    var wouldWorkWithAwait = false;
                    var errNode = errorNode || operatorToken;
                    if (isRelated) {
                        var awaitedLeftType = getAwaitedType(leftType);
                        var awaitedRightType = getAwaitedType(rightType);
                        wouldWorkWithAwait = !(awaitedLeftType === leftType && awaitedRightType === rightType)
                            && !!(awaitedLeftType && awaitedRightType)
                            && isRelated(awaitedLeftType, awaitedRightType);
                    }
                    var effectiveLeft = leftType;
                    var effectiveRight = rightType;
                    if (!wouldWorkWithAwait && isRelated) {
                        _a = getBaseTypesIfUnrelated(leftType, rightType, isRelated), effectiveLeft = _a[0], effectiveRight = _a[1];
                    }
                    var _b = getTypeNamesForErrorDisplay(effectiveLeft, effectiveRight), leftStr = _b[0], rightStr = _b[1];
                    if (!tryGiveBetterPrimaryError(errNode, wouldWorkWithAwait, leftStr, rightStr)) {
                        errorAndMaybeSuggestAwait(errNode, wouldWorkWithAwait, ts.Diagnostics.Operator_0_cannot_be_applied_to_types_1_and_2, ts.tokenToString(operatorToken.kind), leftStr, rightStr);
                    }
                }
                function tryGiveBetterPrimaryError(errNode, maybeMissingAwait, leftStr, rightStr) {
                    var typeName;
                    switch (operatorToken.kind) {
                        case 36 /* EqualsEqualsEqualsToken */:
                        case 34 /* EqualsEqualsToken */:
                            typeName = "false";
                            break;
                        case 37 /* ExclamationEqualsEqualsToken */:
                        case 35 /* ExclamationEqualsToken */:
                            typeName = "true";
                    }
                    if (typeName) {
                        return errorAndMaybeSuggestAwait(errNode, maybeMissingAwait, ts.Diagnostics.This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap, typeName, leftStr, rightStr);
                    }
                    return undefined;
                }
            }
            function getBaseTypesIfUnrelated(leftType, rightType, isRelated) {
                var effectiveLeft = leftType;
                var effectiveRight = rightType;
                var leftBase = getBaseTypeOfLiteralType(leftType);
                var rightBase = getBaseTypeOfLiteralType(rightType);
                if (!isRelated(leftBase, rightBase)) {
                    effectiveLeft = leftBase;
                    effectiveRight = rightBase;
                }
                return [effectiveLeft, effectiveRight];
            }
            function checkYieldExpression(node) {
                // Grammar checking
                if (produceDiagnostics) {
                    if (!(node.flags & 8192 /* YieldContext */)) {
                        grammarErrorOnFirstToken(node, ts.Diagnostics.A_yield_expression_is_only_allowed_in_a_generator_body);
                    }
                    if (isInParameterInitializerBeforeContainingFunction(node)) {
                        error(node, ts.Diagnostics.yield_expressions_cannot_be_used_in_a_parameter_initializer);
                    }
                }
                var func = ts.getContainingFunction(node);
                if (!func)
                    return anyType;
                var functionFlags = ts.getFunctionFlags(func);
                if (!(functionFlags & 1 /* Generator */)) {
                    // If the user's code is syntactically correct, the func should always have a star. After all, we are in a yield context.
                    return anyType;
                }
                var isAsync = (functionFlags & 2 /* Async */) !== 0;
                if (node.asteriskToken) {
                    // Async generator functions prior to ESNext require the __await, __asyncDelegator,
                    // and __asyncValues helpers
                    if (isAsync && languageVersion < 99 /* ESNext */) {
                        checkExternalEmitHelpers(node, 53248 /* AsyncDelegatorIncludes */);
                    }
                    // Generator functions prior to ES2015 require the __values helper
                    if (!isAsync && languageVersion < 2 /* ES2015 */ && compilerOptions.downlevelIteration) {
                        checkExternalEmitHelpers(node, 256 /* Values */);
                    }
                }
                // There is no point in doing an assignability check if the function
                // has no explicit return type because the return type is directly computed
                // from the yield expressions.
                var returnType = getReturnTypeFromAnnotation(func);
                var iterationTypes = returnType && getIterationTypesOfGeneratorFunctionReturnType(returnType, isAsync);
                var signatureYieldType = iterationTypes && iterationTypes.yieldType || anyType;
                var signatureNextType = iterationTypes && iterationTypes.nextType || anyType;
                var resolvedSignatureNextType = isAsync ? getAwaitedType(signatureNextType) || anyType : signatureNextType;
                var yieldExpressionType = node.expression ? checkExpression(node.expression) : undefinedWideningType;
                var yieldedType = getYieldedTypeOfYieldExpression(node, yieldExpressionType, resolvedSignatureNextType, isAsync);
                if (returnType && yieldedType) {
                    checkTypeAssignableToAndOptionallyElaborate(yieldedType, signatureYieldType, node.expression || node, node.expression);
                }
                if (node.asteriskToken) {
                    var use = isAsync ? 19 /* AsyncYieldStar */ : 17 /* YieldStar */;
                    return getIterationTypeOfIterable(use, 1 /* Return */, yieldExpressionType, node.expression)
                        || anyType;
                }
                else if (returnType) {
                    return getIterationTypeOfGeneratorFunctionReturnType(2 /* Next */, returnType, isAsync)
                        || anyType;
                }
                return getContextualIterationType(2 /* Next */, func) || anyType;
            }
            function checkConditionalExpression(node, checkMode) {
                checkTruthinessExpression(node.condition);
                var type1 = checkExpression(node.whenTrue, checkMode);
                var type2 = checkExpression(node.whenFalse, checkMode);
                return getUnionType([type1, type2], 2 /* Subtype */);
            }
            function checkTemplateExpression(node) {
                // We just want to check each expressions, but we are unconcerned with
                // the type of each expression, as any value may be coerced into a string.
                // It is worth asking whether this is what we really want though.
                // A place where we actually *are* concerned with the expressions' types are
                // in tagged templates.
                ts.forEach(node.templateSpans, function (templateSpan) {
                    if (maybeTypeOfKind(checkExpression(templateSpan.expression), 12288 /* ESSymbolLike */)) {
                        error(templateSpan.expression, ts.Diagnostics.Implicit_conversion_of_a_symbol_to_a_string_will_fail_at_runtime_Consider_wrapping_this_expression_in_String);
                    }
                });
                return stringType;
            }
            function getContextNode(node) {
                if (node.kind === 274 /* JsxAttributes */ && !ts.isJsxSelfClosingElement(node.parent)) {
                    return node.parent.parent; // Needs to be the root JsxElement, so it encompasses the attributes _and_ the children (which are essentially part of the attributes)
                }
                return node;
            }
            function checkExpressionWithContextualType(node, contextualType, inferenceContext, checkMode) {
                var context = getContextNode(node);
                var saveContextualType = context.contextualType;
                var saveInferenceContext = context.inferenceContext;
                try {
                    context.contextualType = contextualType;
                    context.inferenceContext = inferenceContext;
                    var type = checkExpression(node, checkMode | 1 /* Contextual */ | (inferenceContext ? 2 /* Inferential */ : 0));
                    // We strip literal freshness when an appropriate contextual type is present such that contextually typed
                    // literals always preserve their literal types (otherwise they might widen during type inference). An alternative
                    // here would be to not mark contextually typed literals as fresh in the first place.
                    var result = maybeTypeOfKind(type, 2944 /* Literal */) && isLiteralOfContextualType(type, instantiateContextualType(contextualType, node)) ?
                        getRegularTypeOfLiteralType(type) : type;
                    return result;
                }
                finally {
                    // In the event our operation is canceled or some other exception occurs, reset the contextual type
                    // so that we do not accidentally hold onto an instance of the checker, as a Type created in the services layer
                    // may hold onto the checker that created it.
                    context.contextualType = saveContextualType;
                    context.inferenceContext = saveInferenceContext;
                }
            }
            function checkExpressionCached(node, checkMode) {
                var links = getNodeLinks(node);
                if (!links.resolvedType) {
                    if (checkMode && checkMode !== 0 /* Normal */) {
                        return checkExpression(node, checkMode);
                    }
                    // When computing a type that we're going to cache, we need to ignore any ongoing control flow
                    // analysis because variables may have transient types in indeterminable states. Moving flowLoopStart
                    // to the top of the stack ensures all transient types are computed from a known point.
                    var saveFlowLoopStart = flowLoopStart;
                    var saveFlowTypeCache = flowTypeCache;
                    flowLoopStart = flowLoopCount;
                    flowTypeCache = undefined;
                    links.resolvedType = checkExpression(node, checkMode);
                    flowTypeCache = saveFlowTypeCache;
                    flowLoopStart = saveFlowLoopStart;
                }
                return links.resolvedType;
            }
            function isTypeAssertion(node) {
                node = ts.skipParentheses(node);
                return node.kind === 199 /* TypeAssertionExpression */ || node.kind === 217 /* AsExpression */;
            }
            function checkDeclarationInitializer(declaration, contextualType) {
                var initializer = ts.getEffectiveInitializer(declaration);
                var type = getQuickTypeOfExpression(initializer) ||
                    (contextualType ? checkExpressionWithContextualType(initializer, contextualType, /*inferenceContext*/ undefined, 0 /* Normal */) : checkExpressionCached(initializer));
                return ts.isParameter(declaration) && declaration.name.kind === 190 /* ArrayBindingPattern */ &&
                    isTupleType(type) && !type.target.hasRestElement && getTypeReferenceArity(type) < declaration.name.elements.length ?
                    padTupleType(type, declaration.name) : type;
            }
            function padTupleType(type, pattern) {
                var patternElements = pattern.elements;
                var arity = getTypeReferenceArity(type);
                var elementTypes = arity ? getTypeArguments(type).slice() : [];
                for (var i = arity; i < patternElements.length; i++) {
                    var e = patternElements[i];
                    if (i < patternElements.length - 1 || !(e.kind === 191 /* BindingElement */ && e.dotDotDotToken)) {
                        elementTypes.push(!ts.isOmittedExpression(e) && hasDefaultValue(e) ? getTypeFromBindingElement(e, /*includePatternInType*/ false, /*reportErrors*/ false) : anyType);
                        if (!ts.isOmittedExpression(e) && !hasDefaultValue(e)) {
                            reportImplicitAny(e, anyType);
                        }
                    }
                }
                return createTupleType(elementTypes, type.target.minLength, /*hasRestElement*/ false, type.target.readonly);
            }
            function widenTypeInferredFromInitializer(declaration, type) {
                var widened = ts.getCombinedNodeFlags(declaration) & 2 /* Const */ || ts.isDeclarationReadonly(declaration) ? type : getWidenedLiteralType(type);
                if (ts.isInJSFile(declaration)) {
                    if (widened.flags & 98304 /* Nullable */) {
                        reportImplicitAny(declaration, anyType);
                        return anyType;
                    }
                    else if (isEmptyArrayLiteralType(widened)) {
                        reportImplicitAny(declaration, anyArrayType);
                        return anyArrayType;
                    }
                }
                return widened;
            }
            function isLiteralOfContextualType(candidateType, contextualType) {
                if (contextualType) {
                    if (contextualType.flags & 3145728 /* UnionOrIntersection */) {
                        var types = contextualType.types;
                        return ts.some(types, function (t) { return isLiteralOfContextualType(candidateType, t); });
                    }
                    if (contextualType.flags & 58982400 /* InstantiableNonPrimitive */) {
                        // If the contextual type is a type variable constrained to a primitive type, consider
                        // this a literal context for literals of that primitive type. For example, given a
                        // type parameter 'T extends string', infer string literal types for T.
                        var constraint = getBaseConstraintOfType(contextualType) || unknownType;
                        return maybeTypeOfKind(constraint, 4 /* String */) && maybeTypeOfKind(candidateType, 128 /* StringLiteral */) ||
                            maybeTypeOfKind(constraint, 8 /* Number */) && maybeTypeOfKind(candidateType, 256 /* NumberLiteral */) ||
                            maybeTypeOfKind(constraint, 64 /* BigInt */) && maybeTypeOfKind(candidateType, 2048 /* BigIntLiteral */) ||
                            maybeTypeOfKind(constraint, 4096 /* ESSymbol */) && maybeTypeOfKind(candidateType, 8192 /* UniqueESSymbol */) ||
                            isLiteralOfContextualType(candidateType, constraint);
                    }
                    // If the contextual type is a literal of a particular primitive type, we consider this a
                    // literal context for all literals of that primitive type.
                    return !!(contextualType.flags & (128 /* StringLiteral */ | 4194304 /* Index */) && maybeTypeOfKind(candidateType, 128 /* StringLiteral */) ||
                        contextualType.flags & 256 /* NumberLiteral */ && maybeTypeOfKind(candidateType, 256 /* NumberLiteral */) ||
                        contextualType.flags & 2048 /* BigIntLiteral */ && maybeTypeOfKind(candidateType, 2048 /* BigIntLiteral */) ||
                        contextualType.flags & 512 /* BooleanLiteral */ && maybeTypeOfKind(candidateType, 512 /* BooleanLiteral */) ||
                        contextualType.flags & 8192 /* UniqueESSymbol */ && maybeTypeOfKind(candidateType, 8192 /* UniqueESSymbol */));
                }
                return false;
            }
            function isConstContext(node) {
                var parent = node.parent;
                return ts.isAssertionExpression(parent) && ts.isConstTypeReference(parent.type) ||
                    (ts.isParenthesizedExpression(parent) || ts.isArrayLiteralExpression(parent) || ts.isSpreadElement(parent)) && isConstContext(parent) ||
                    (ts.isPropertyAssignment(parent) || ts.isShorthandPropertyAssignment(parent)) && isConstContext(parent.parent);
            }
            function checkExpressionForMutableLocation(node, checkMode, contextualType, forceTuple) {
                var type = checkExpression(node, checkMode, forceTuple);
                return isConstContext(node) ? getRegularTypeOfLiteralType(type) :
                    isTypeAssertion(node) ? type :
                        getWidenedLiteralLikeTypeForContextualType(type, instantiateContextualType(arguments.length === 2 ? getContextualType(node) : contextualType, node));
            }
            function checkPropertyAssignment(node, checkMode) {
                // Do not use hasDynamicName here, because that returns false for well known symbols.
                // We want to perform checkComputedPropertyName for all computed properties, including
                // well known symbols.
                if (node.name.kind === 154 /* ComputedPropertyName */) {
                    checkComputedPropertyName(node.name);
                }
                return checkExpressionForMutableLocation(node.initializer, checkMode);
            }
            function checkObjectLiteralMethod(node, checkMode) {
                // Grammar checking
                checkGrammarMethod(node);
                // Do not use hasDynamicName here, because that returns false for well known symbols.
                // We want to perform checkComputedPropertyName for all computed properties, including
                // well known symbols.
                if (node.name.kind === 154 /* ComputedPropertyName */) {
                    checkComputedPropertyName(node.name);
                }
                var uninstantiatedType = checkFunctionExpressionOrObjectLiteralMethod(node, checkMode);
                return instantiateTypeWithSingleGenericCallSignature(node, uninstantiatedType, checkMode);
            }
            function instantiateTypeWithSingleGenericCallSignature(node, type, checkMode) {
                if (checkMode && checkMode & (2 /* Inferential */ | 8 /* SkipGenericFunctions */)) {
                    var callSignature = getSingleSignature(type, 0 /* Call */, /*allowMembers*/ true);
                    var constructSignature = getSingleSignature(type, 1 /* Construct */, /*allowMembers*/ true);
                    var signature = callSignature || constructSignature;
                    if (signature && signature.typeParameters) {
                        var contextualType = getApparentTypeOfContextualType(node, 2 /* NoConstraints */);
                        if (contextualType) {
                            var contextualSignature = getSingleSignature(getNonNullableType(contextualType), callSignature ? 0 /* Call */ : 1 /* Construct */, /*allowMembers*/ false);
                            if (contextualSignature && !contextualSignature.typeParameters) {
                                if (checkMode & 8 /* SkipGenericFunctions */) {
                                    skippedGenericFunction(node, checkMode);
                                    return anyFunctionType;
                                }
                                var context = getInferenceContext(node);
                                // We have an expression that is an argument of a generic function for which we are performing
                                // type argument inference. The expression is of a function type with a single generic call
                                // signature and a contextual function type with a single non-generic call signature. Now check
                                // if the outer function returns a function type with a single non-generic call signature and
                                // if some of the outer function type parameters have no inferences so far. If so, we can
                                // potentially add inferred type parameters to the outer function return type.
                                var returnType = context.signature && getReturnTypeOfSignature(context.signature);
                                var returnSignature = returnType && getSingleCallOrConstructSignature(returnType);
                                if (returnSignature && !returnSignature.typeParameters && !ts.every(context.inferences, hasInferenceCandidates)) {
                                    // Instantiate the signature with its own type parameters as type arguments, possibly
                                    // renaming the type parameters to ensure they have unique names.
                                    var uniqueTypeParameters = getUniqueTypeParameters(context, signature.typeParameters);
                                    var instantiatedSignature = getSignatureInstantiationWithoutFillingInTypeArguments(signature, uniqueTypeParameters);
                                    // Infer from the parameters of the instantiated signature to the parameters of the
                                    // contextual signature starting with an empty set of inference candidates.
                                    var inferences_3 = ts.map(context.inferences, function (info) { return createInferenceInfo(info.typeParameter); });
                                    applyToParameterTypes(instantiatedSignature, contextualSignature, function (source, target) {
                                        inferTypes(inferences_3, source, target, /*priority*/ 0, /*contravariant*/ true);
                                    });
                                    if (ts.some(inferences_3, hasInferenceCandidates)) {
                                        // We have inference candidates, indicating that one or more type parameters are referenced
                                        // in the parameter types of the contextual signature. Now also infer from the return type.
                                        applyToReturnTypes(instantiatedSignature, contextualSignature, function (source, target) {
                                            inferTypes(inferences_3, source, target);
                                        });
                                        // If the type parameters for which we produced candidates do not have any inferences yet,
                                        // we adopt the new inference candidates and add the type parameters of the expression type
                                        // to the set of inferred type parameters for the outer function return type.
                                        if (!hasOverlappingInferences(context.inferences, inferences_3)) {
                                            mergeInferences(context.inferences, inferences_3);
                                            context.inferredTypeParameters = ts.concatenate(context.inferredTypeParameters, uniqueTypeParameters);
                                            return getOrCreateTypeFromSignature(instantiatedSignature);
                                        }
                                    }
                                }
                                return getOrCreateTypeFromSignature(instantiateSignatureInContextOf(signature, contextualSignature, context));
                            }
                        }
                    }
                }
                return type;
            }
            function skippedGenericFunction(node, checkMode) {
                if (checkMode & 2 /* Inferential */) {
                    // We have skipped a generic function during inferential typing. Obtain the inference context and
                    // indicate this has occurred such that we know a second pass of inference is be needed.
                    var context = getInferenceContext(node);
                    context.flags |= 4 /* SkippedGenericFunction */;
                }
            }
            function hasInferenceCandidates(info) {
                return !!(info.candidates || info.contraCandidates);
            }
            function hasOverlappingInferences(a, b) {
                for (var i = 0; i < a.length; i++) {
                    if (hasInferenceCandidates(a[i]) && hasInferenceCandidates(b[i])) {
                        return true;
                    }
                }
                return false;
            }
            function mergeInferences(target, source) {
                for (var i = 0; i < target.length; i++) {
                    if (!hasInferenceCandidates(target[i]) && hasInferenceCandidates(source[i])) {
                        target[i] = source[i];
                    }
                }
            }
            function getUniqueTypeParameters(context, typeParameters) {
                var result = [];
                var oldTypeParameters;
                var newTypeParameters;
                for (var _i = 0, typeParameters_2 = typeParameters; _i < typeParameters_2.length; _i++) {
                    var tp = typeParameters_2[_i];
                    var name = tp.symbol.escapedName;
                    if (hasTypeParameterByName(context.inferredTypeParameters, name) || hasTypeParameterByName(result, name)) {
                        var newName = getUniqueTypeParameterName(ts.concatenate(context.inferredTypeParameters, result), name);
                        var symbol = createSymbol(262144 /* TypeParameter */, newName);
                        var newTypeParameter = createTypeParameter(symbol);
                        newTypeParameter.target = tp;
                        oldTypeParameters = ts.append(oldTypeParameters, tp);
                        newTypeParameters = ts.append(newTypeParameters, newTypeParameter);
                        result.push(newTypeParameter);
                    }
                    else {
                        result.push(tp);
                    }
                }
                if (newTypeParameters) {
                    var mapper = createTypeMapper(oldTypeParameters, newTypeParameters);
                    for (var _a = 0, newTypeParameters_1 = newTypeParameters; _a < newTypeParameters_1.length; _a++) {
                        var tp = newTypeParameters_1[_a];
                        tp.mapper = mapper;
                    }
                }
                return result;
            }
            function hasTypeParameterByName(typeParameters, name) {
                return ts.some(typeParameters, function (tp) { return tp.symbol.escapedName === name; });
            }
            function getUniqueTypeParameterName(typeParameters, baseName) {
                var len = baseName.length;
                while (len > 1 && baseName.charCodeAt(len - 1) >= 48 /* _0 */ && baseName.charCodeAt(len - 1) <= 57 /* _9 */)
                    len--;
                var s = baseName.slice(0, len);
                for (var index = 1; true; index++) {
                    var augmentedName = (s + index);
                    if (!hasTypeParameterByName(typeParameters, augmentedName)) {
                        return augmentedName;
                    }
                }
            }
            function getReturnTypeOfSingleNonGenericCallSignature(funcType) {
                var signature = getSingleCallSignature(funcType);
                if (signature && !signature.typeParameters) {
                    return getReturnTypeOfSignature(signature);
                }
            }
            function getReturnTypeOfSingleNonGenericSignatureOfCallChain(expr) {
                var funcType = checkExpression(expr.expression);
                var nonOptionalType = getOptionalExpressionType(funcType, expr.expression);
                var returnType = getReturnTypeOfSingleNonGenericCallSignature(funcType);
                return returnType && propagateOptionalTypeMarker(returnType, expr, nonOptionalType !== funcType);
            }
            /**
             * Returns the type of an expression. Unlike checkExpression, this function is simply concerned
             * with computing the type and may not fully check all contained sub-expressions for errors.
             */
            function getTypeOfExpression(node) {
                // Don't bother caching types that require no flow analysis and are quick to compute.
                var quickType = getQuickTypeOfExpression(node);
                if (quickType) {
                    return quickType;
                }
                // If a type has been cached for the node, return it.
                if (node.flags & 67108864 /* TypeCached */ && flowTypeCache) {
                    var cachedType = flowTypeCache[getNodeId(node)];
                    if (cachedType) {
                        return cachedType;
                    }
                }
                var startInvocationCount = flowInvocationCount;
                var type = checkExpression(node);
                // If control flow analysis was required to determine the type, it is worth caching.
                if (flowInvocationCount !== startInvocationCount) {
                    var cache = flowTypeCache || (flowTypeCache = []);
                    cache[getNodeId(node)] = type;
                    node.flags |= 67108864 /* TypeCached */;
                }
                return type;
            }
            function getQuickTypeOfExpression(node) {
                var expr = ts.skipParentheses(node);
                // Optimize for the common case of a call to a function with a single non-generic call
                // signature where we can just fetch the return type without checking the arguments.
                if (ts.isCallExpression(expr) && expr.expression.kind !== 102 /* SuperKeyword */ && !ts.isRequireCall(expr, /*checkArgumentIsStringLiteralLike*/ true) && !isSymbolOrSymbolForCall(expr)) {
                    var type = ts.isCallChain(expr) ? getReturnTypeOfSingleNonGenericSignatureOfCallChain(expr) :
                        getReturnTypeOfSingleNonGenericCallSignature(checkNonNullExpression(expr.expression));
                    if (type) {
                        return type;
                    }
                }
                else if (ts.isAssertionExpression(expr) && !ts.isConstTypeReference(expr.type)) {
                    return getTypeFromTypeNode(expr.type);
                }
                else if (node.kind === 8 /* NumericLiteral */ || node.kind === 10 /* StringLiteral */ ||
                    node.kind === 106 /* TrueKeyword */ || node.kind === 91 /* FalseKeyword */) {
                    return checkExpression(node);
                }
                return undefined;
            }
            /**
             * Returns the type of an expression. Unlike checkExpression, this function is simply concerned
             * with computing the type and may not fully check all contained sub-expressions for errors.
             * It is intended for uses where you know there is no contextual type,
             * and requesting the contextual type might cause a circularity or other bad behaviour.
             * It sets the contextual type of the node to any before calling getTypeOfExpression.
             */
            function getContextFreeTypeOfExpression(node) {
                var links = getNodeLinks(node);
                if (links.contextFreeType) {
                    return links.contextFreeType;
                }
                var saveContextualType = node.contextualType;
                node.contextualType = anyType;
                try {
                    var type = links.contextFreeType = checkExpression(node, 4 /* SkipContextSensitive */);
                    return type;
                }
                finally {
                    // In the event our operation is canceled or some other exception occurs, reset the contextual type
                    // so that we do not accidentally hold onto an instance of the checker, as a Type created in the services layer
                    // may hold onto the checker that created it.
                    node.contextualType = saveContextualType;
                }
            }
            function checkExpression(node, checkMode, forceTuple) {
                var saveCurrentNode = currentNode;
                currentNode = node;
                instantiationCount = 0;
                var uninstantiatedType = checkExpressionWorker(node, checkMode, forceTuple);
                var type = instantiateTypeWithSingleGenericCallSignature(node, uninstantiatedType, checkMode);
                if (isConstEnumObjectType(type)) {
                    checkConstEnumAccess(node, type);
                }
                currentNode = saveCurrentNode;
                return type;
            }
            function checkConstEnumAccess(node, type) {
                // enum object type for const enums are only permitted in:
                // - 'left' in property access
                // - 'object' in indexed access
                // - target in rhs of import statement
                var ok = (node.parent.kind === 194 /* PropertyAccessExpression */ && node.parent.expression === node) ||
                    (node.parent.kind === 195 /* ElementAccessExpression */ && node.parent.expression === node) ||
                    ((node.kind === 75 /* Identifier */ || node.kind === 153 /* QualifiedName */) && isInRightSideOfImportOrExportAssignment(node) ||
                        (node.parent.kind === 172 /* TypeQuery */ && node.parent.exprName === node)) ||
                    (node.parent.kind === 263 /* ExportSpecifier */); // We allow reexporting const enums
                if (!ok) {
                    error(node, ts.Diagnostics.const_enums_can_only_be_used_in_property_or_index_access_expressions_or_the_right_hand_side_of_an_import_declaration_or_export_assignment_or_type_query);
                }
                if (compilerOptions.isolatedModules) {
                    ts.Debug.assert(!!(type.symbol.flags & 128 /* ConstEnum */));
                    var constEnumDeclaration = type.symbol.valueDeclaration;
                    if (constEnumDeclaration.flags & 8388608 /* Ambient */) {
                        error(node, ts.Diagnostics.Cannot_access_ambient_const_enums_when_the_isolatedModules_flag_is_provided);
                    }
                }
            }
            function checkParenthesizedExpression(node, checkMode) {
                var tag = ts.isInJSFile(node) ? ts.getJSDocTypeTag(node) : undefined;
                if (tag) {
                    return checkAssertionWorker(tag, tag.typeExpression.type, node.expression, checkMode);
                }
                return checkExpression(node.expression, checkMode);
            }
            function checkExpressionWorker(node, checkMode, forceTuple) {
                var kind = node.kind;
                if (cancellationToken) {
                    // Only bother checking on a few construct kinds.  We don't want to be excessively
                    // hitting the cancellation token on every node we check.
                    switch (kind) {
                        case 214 /* ClassExpression */:
                        case 201 /* FunctionExpression */:
                        case 202 /* ArrowFunction */:
                            cancellationToken.throwIfCancellationRequested();
                    }
                }
                switch (kind) {
                    case 75 /* Identifier */:
                        return checkIdentifier(node);
                    case 104 /* ThisKeyword */:
                        return checkThisExpression(node);
                    case 102 /* SuperKeyword */:
                        return checkSuperExpression(node);
                    case 100 /* NullKeyword */:
                        return nullWideningType;
                    case 14 /* NoSubstitutionTemplateLiteral */:
                    case 10 /* StringLiteral */:
                        return getFreshTypeOfLiteralType(getLiteralType(node.text));
                    case 8 /* NumericLiteral */:
                        checkGrammarNumericLiteral(node);
                        return getFreshTypeOfLiteralType(getLiteralType(+node.text));
                    case 9 /* BigIntLiteral */:
                        checkGrammarBigIntLiteral(node);
                        return getFreshTypeOfLiteralType(getBigIntLiteralType(node));
                    case 106 /* TrueKeyword */:
                        return trueType;
                    case 91 /* FalseKeyword */:
                        return falseType;
                    case 211 /* TemplateExpression */:
                        return checkTemplateExpression(node);
                    case 13 /* RegularExpressionLiteral */:
                        return globalRegExpType;
                    case 192 /* ArrayLiteralExpression */:
                        return checkArrayLiteral(node, checkMode, forceTuple);
                    case 193 /* ObjectLiteralExpression */:
                        return checkObjectLiteral(node, checkMode);
                    case 194 /* PropertyAccessExpression */:
                        return checkPropertyAccessExpression(node);
                    case 153 /* QualifiedName */:
                        return checkQualifiedName(node);
                    case 195 /* ElementAccessExpression */:
                        return checkIndexedAccess(node);
                    case 196 /* CallExpression */:
                        if (node.expression.kind === 96 /* ImportKeyword */) {
                            return checkImportCallExpression(node);
                        }
                    // falls through
                    case 197 /* NewExpression */:
                        return checkCallExpression(node, checkMode);
                    case 198 /* TaggedTemplateExpression */:
                        return checkTaggedTemplateExpression(node);
                    case 200 /* ParenthesizedExpression */:
                        return checkParenthesizedExpression(node, checkMode);
                    case 214 /* ClassExpression */:
                        return checkClassExpression(node);
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                        return checkFunctionExpressionOrObjectLiteralMethod(node, checkMode);
                    case 204 /* TypeOfExpression */:
                        return checkTypeOfExpression(node);
                    case 199 /* TypeAssertionExpression */:
                    case 217 /* AsExpression */:
                        return checkAssertion(node);
                    case 218 /* NonNullExpression */:
                        return checkNonNullAssertion(node);
                    case 219 /* MetaProperty */:
                        return checkMetaProperty(node);
                    case 203 /* DeleteExpression */:
                        return checkDeleteExpression(node);
                    case 205 /* VoidExpression */:
                        return checkVoidExpression(node);
                    case 206 /* AwaitExpression */:
                        return checkAwaitExpression(node);
                    case 207 /* PrefixUnaryExpression */:
                        return checkPrefixUnaryExpression(node);
                    case 208 /* PostfixUnaryExpression */:
                        return checkPostfixUnaryExpression(node);
                    case 209 /* BinaryExpression */:
                        return checkBinaryExpression(node, checkMode);
                    case 210 /* ConditionalExpression */:
                        return checkConditionalExpression(node, checkMode);
                    case 213 /* SpreadElement */:
                        return checkSpreadExpression(node, checkMode);
                    case 215 /* OmittedExpression */:
                        return undefinedWideningType;
                    case 212 /* YieldExpression */:
                        return checkYieldExpression(node);
                    case 220 /* SyntheticExpression */:
                        return node.type;
                    case 276 /* JsxExpression */:
                        return checkJsxExpression(node, checkMode);
                    case 266 /* JsxElement */:
                        return checkJsxElement(node, checkMode);
                    case 267 /* JsxSelfClosingElement */:
                        return checkJsxSelfClosingElement(node, checkMode);
                    case 270 /* JsxFragment */:
                        return checkJsxFragment(node);
                    case 274 /* JsxAttributes */:
                        return checkJsxAttributes(node, checkMode);
                    case 268 /* JsxOpeningElement */:
                        ts.Debug.fail("Shouldn't ever directly check a JsxOpeningElement");
                }
                return errorType;
            }
            // DECLARATION AND STATEMENT TYPE CHECKING
            function checkTypeParameter(node) {
                // Grammar Checking
                if (node.expression) {
                    grammarErrorOnFirstToken(node.expression, ts.Diagnostics.Type_expected);
                }
                checkSourceElement(node.constraint);
                checkSourceElement(node.default);
                var typeParameter = getDeclaredTypeOfTypeParameter(getSymbolOfNode(node));
                // Resolve base constraint to reveal circularity errors
                getBaseConstraintOfType(typeParameter);
                if (!hasNonCircularTypeParameterDefault(typeParameter)) {
                    error(node.default, ts.Diagnostics.Type_parameter_0_has_a_circular_default, typeToString(typeParameter));
                }
                var constraintType = getConstraintOfTypeParameter(typeParameter);
                var defaultType = getDefaultFromTypeParameter(typeParameter);
                if (constraintType && defaultType) {
                    checkTypeAssignableTo(defaultType, getTypeWithThisArgument(instantiateType(constraintType, makeUnaryTypeMapper(typeParameter, defaultType)), defaultType), node.default, ts.Diagnostics.Type_0_does_not_satisfy_the_constraint_1);
                }
                if (produceDiagnostics) {
                    checkTypeNameIsReserved(node.name, ts.Diagnostics.Type_parameter_name_cannot_be_0);
                }
            }
            function checkParameter(node) {
                // Grammar checking
                // It is a SyntaxError if the Identifier "eval" or the Identifier "arguments" occurs as the
                // Identifier in a PropertySetParameterList of a PropertyAssignment that is contained in strict code
                // or if its FunctionBody is strict code(11.1.5).
                checkGrammarDecoratorsAndModifiers(node);
                checkVariableLikeDeclaration(node);
                var func = ts.getContainingFunction(node);
                if (ts.hasModifier(node, 92 /* ParameterPropertyModifier */)) {
                    if (!(func.kind === 162 /* Constructor */ && ts.nodeIsPresent(func.body))) {
                        error(node, ts.Diagnostics.A_parameter_property_is_only_allowed_in_a_constructor_implementation);
                    }
                }
                if (node.questionToken && ts.isBindingPattern(node.name) && func.body) {
                    error(node, ts.Diagnostics.A_binding_pattern_parameter_cannot_be_optional_in_an_implementation_signature);
                }
                if (node.name && ts.isIdentifier(node.name) && (node.name.escapedText === "this" || node.name.escapedText === "new")) {
                    if (func.parameters.indexOf(node) !== 0) {
                        error(node, ts.Diagnostics.A_0_parameter_must_be_the_first_parameter, node.name.escapedText);
                    }
                    if (func.kind === 162 /* Constructor */ || func.kind === 166 /* ConstructSignature */ || func.kind === 171 /* ConstructorType */) {
                        error(node, ts.Diagnostics.A_constructor_cannot_have_a_this_parameter);
                    }
                    if (func.kind === 202 /* ArrowFunction */) {
                        error(node, ts.Diagnostics.An_arrow_function_cannot_have_a_this_parameter);
                    }
                }
                // Only check rest parameter type if it's not a binding pattern. Since binding patterns are
                // not allowed in a rest parameter, we already have an error from checkGrammarParameterList.
                if (node.dotDotDotToken && !ts.isBindingPattern(node.name) && !isTypeAssignableTo(getTypeOfSymbol(node.symbol), anyReadonlyArrayType)) {
                    error(node, ts.Diagnostics.A_rest_parameter_must_be_of_an_array_type);
                }
            }
            function checkTypePredicate(node) {
                var parent = getTypePredicateParent(node);
                if (!parent) {
                    // The parent must not be valid.
                    error(node, ts.Diagnostics.A_type_predicate_is_only_allowed_in_return_type_position_for_functions_and_methods);
                    return;
                }
                var signature = getSignatureFromDeclaration(parent);
                var typePredicate = getTypePredicateOfSignature(signature);
                if (!typePredicate) {
                    return;
                }
                checkSourceElement(node.type);
                var parameterName = node.parameterName;
                if (typePredicate.kind === 0 /* This */ || typePredicate.kind === 2 /* AssertsThis */) {
                    getTypeFromThisTypeNode(parameterName);
                }
                else {
                    if (typePredicate.parameterIndex >= 0) {
                        if (signatureHasRestParameter(signature) && typePredicate.parameterIndex === signature.parameters.length - 1) {
                            error(parameterName, ts.Diagnostics.A_type_predicate_cannot_reference_a_rest_parameter);
                        }
                        else {
                            if (typePredicate.type) {
                                var leadingError = function () { return ts.chainDiagnosticMessages(/*details*/ undefined, ts.Diagnostics.A_type_predicate_s_type_must_be_assignable_to_its_parameter_s_type); };
                                checkTypeAssignableTo(typePredicate.type, getTypeOfSymbol(signature.parameters[typePredicate.parameterIndex]), node.type, 
                                /*headMessage*/ undefined, leadingError);
                            }
                        }
                    }
                    else if (parameterName) {
                        var hasReportedError = false;
                        for (var _i = 0, _a = parent.parameters; _i < _a.length; _i++) {
                            var name = _a[_i].name;
                            if (ts.isBindingPattern(name) &&
                                checkIfTypePredicateVariableIsDeclaredInBindingPattern(name, parameterName, typePredicate.parameterName)) {
                                hasReportedError = true;
                                break;
                            }
                        }
                        if (!hasReportedError) {
                            error(node.parameterName, ts.Diagnostics.Cannot_find_parameter_0, typePredicate.parameterName);
                        }
                    }
                }
            }
            function getTypePredicateParent(node) {
                switch (node.parent.kind) {
                    case 202 /* ArrowFunction */:
                    case 165 /* CallSignature */:
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 170 /* FunctionType */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                        var parent = node.parent;
                        if (node === parent.type) {
                            return parent;
                        }
                }
            }
            function checkIfTypePredicateVariableIsDeclaredInBindingPattern(pattern, predicateVariableNode, predicateVariableName) {
                for (var _i = 0, _a = pattern.elements; _i < _a.length; _i++) {
                    var element = _a[_i];
                    if (ts.isOmittedExpression(element)) {
                        continue;
                    }
                    var name = element.name;
                    if (name.kind === 75 /* Identifier */ && name.escapedText === predicateVariableName) {
                        error(predicateVariableNode, ts.Diagnostics.A_type_predicate_cannot_reference_element_0_in_a_binding_pattern, predicateVariableName);
                        return true;
                    }
                    else if (name.kind === 190 /* ArrayBindingPattern */ || name.kind === 189 /* ObjectBindingPattern */) {
                        if (checkIfTypePredicateVariableIsDeclaredInBindingPattern(name, predicateVariableNode, predicateVariableName)) {
                            return true;
                        }
                    }
                }
            }
            function checkSignatureDeclaration(node) {
                // Grammar checking
                if (node.kind === 167 /* IndexSignature */) {
                    checkGrammarIndexSignature(node);
                }
                // TODO (yuisu): Remove this check in else-if when SyntaxKind.Construct is moved and ambient context is handled
                else if (node.kind === 170 /* FunctionType */ || node.kind === 244 /* FunctionDeclaration */ || node.kind === 171 /* ConstructorType */ ||
                    node.kind === 165 /* CallSignature */ || node.kind === 162 /* Constructor */ ||
                    node.kind === 166 /* ConstructSignature */) {
                    checkGrammarFunctionLikeDeclaration(node);
                }
                var functionFlags = ts.getFunctionFlags(node);
                if (!(functionFlags & 4 /* Invalid */)) {
                    // Async generators prior to ESNext require the __await and __asyncGenerator helpers
                    if ((functionFlags & 3 /* AsyncGenerator */) === 3 /* AsyncGenerator */ && languageVersion < 99 /* ESNext */) {
                        checkExternalEmitHelpers(node, 12288 /* AsyncGeneratorIncludes */);
                    }
                    // Async functions prior to ES2017 require the __awaiter helper
                    if ((functionFlags & 3 /* AsyncGenerator */) === 2 /* Async */ && languageVersion < 4 /* ES2017 */) {
                        checkExternalEmitHelpers(node, 64 /* Awaiter */);
                    }
                    // Generator functions, Async functions, and Async Generator functions prior to
                    // ES2015 require the __generator helper
                    if ((functionFlags & 3 /* AsyncGenerator */) !== 0 /* Normal */ && languageVersion < 2 /* ES2015 */) {
                        checkExternalEmitHelpers(node, 128 /* Generator */);
                    }
                }
                checkTypeParameters(node.typeParameters);
                ts.forEach(node.parameters, checkParameter);
                // TODO(rbuckton): Should we start checking JSDoc types?
                if (node.type) {
                    checkSourceElement(node.type);
                }
                if (produceDiagnostics) {
                    checkCollisionWithArgumentsInGeneratedCode(node);
                    var returnTypeNode = ts.getEffectiveReturnTypeNode(node);
                    if (noImplicitAny && !returnTypeNode) {
                        switch (node.kind) {
                            case 166 /* ConstructSignature */:
                                error(node, ts.Diagnostics.Construct_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type);
                                break;
                            case 165 /* CallSignature */:
                                error(node, ts.Diagnostics.Call_signature_which_lacks_return_type_annotation_implicitly_has_an_any_return_type);
                                break;
                        }
                    }
                    if (returnTypeNode) {
                        var functionFlags_1 = ts.getFunctionFlags(node);
                        if ((functionFlags_1 & (4 /* Invalid */ | 1 /* Generator */)) === 1 /* Generator */) {
                            var returnType = getTypeFromTypeNode(returnTypeNode);
                            if (returnType === voidType) {
                                error(returnTypeNode, ts.Diagnostics.A_generator_cannot_have_a_void_type_annotation);
                            }
                            else {
                                // Naively, one could check that Generator<any, any, any> is assignable to the return type annotation.
                                // However, that would not catch the error in the following case.
                                //
                                //    interface BadGenerator extends Iterable<number>, Iterator<string> { }
                                //    function* g(): BadGenerator { } // Iterable and Iterator have different types!
                                //
                                var generatorYieldType = getIterationTypeOfGeneratorFunctionReturnType(0 /* Yield */, returnType, (functionFlags_1 & 2 /* Async */) !== 0) || anyType;
                                var generatorReturnType = getIterationTypeOfGeneratorFunctionReturnType(1 /* Return */, returnType, (functionFlags_1 & 2 /* Async */) !== 0) || generatorYieldType;
                                var generatorNextType = getIterationTypeOfGeneratorFunctionReturnType(2 /* Next */, returnType, (functionFlags_1 & 2 /* Async */) !== 0) || unknownType;
                                var generatorInstantiation = createGeneratorReturnType(generatorYieldType, generatorReturnType, generatorNextType, !!(functionFlags_1 & 2 /* Async */));
                                checkTypeAssignableTo(generatorInstantiation, returnType, returnTypeNode);
                            }
                        }
                        else if ((functionFlags_1 & 3 /* AsyncGenerator */) === 2 /* Async */) {
                            checkAsyncFunctionReturnType(node, returnTypeNode);
                        }
                    }
                    if (node.kind !== 167 /* IndexSignature */ && node.kind !== 300 /* JSDocFunctionType */) {
                        registerForUnusedIdentifiersCheck(node);
                    }
                }
            }
            function checkClassForDuplicateDeclarations(node) {
                var instanceNames = ts.createUnderscoreEscapedMap();
                var staticNames = ts.createUnderscoreEscapedMap();
                // instance and static private identifiers share the same scope
                var privateIdentifiers = ts.createUnderscoreEscapedMap();
                for (var _i = 0, _a = node.members; _i < _a.length; _i++) {
                    var member = _a[_i];
                    if (member.kind === 162 /* Constructor */) {
                        for (var _b = 0, _c = member.parameters; _b < _c.length; _b++) {
                            var param = _c[_b];
                            if (ts.isParameterPropertyDeclaration(param, member) && !ts.isBindingPattern(param.name)) {
                                addName(instanceNames, param.name, param.name.escapedText, 3 /* GetOrSetAccessor */);
                            }
                        }
                    }
                    else {
                        var isStatic = ts.hasModifier(member, 32 /* Static */);
                        var name = member.name;
                        if (!name) {
                            return;
                        }
                        var names = ts.isPrivateIdentifier(name) ? privateIdentifiers :
                            isStatic ? staticNames :
                                instanceNames;
                        var memberName = name && ts.getPropertyNameForPropertyNameNode(name);
                        if (memberName) {
                            switch (member.kind) {
                                case 163 /* GetAccessor */:
                                    addName(names, name, memberName, 1 /* GetAccessor */);
                                    break;
                                case 164 /* SetAccessor */:
                                    addName(names, name, memberName, 2 /* SetAccessor */);
                                    break;
                                case 159 /* PropertyDeclaration */:
                                    addName(names, name, memberName, 3 /* GetOrSetAccessor */);
                                    break;
                                case 161 /* MethodDeclaration */:
                                    addName(names, name, memberName, 8 /* Method */);
                                    break;
                            }
                        }
                    }
                }
                function addName(names, location, name, meaning) {
                    var prev = names.get(name);
                    if (prev) {
                        if (prev & 8 /* Method */) {
                            if (meaning !== 8 /* Method */) {
                                error(location, ts.Diagnostics.Duplicate_identifier_0, ts.getTextOfNode(location));
                            }
                        }
                        else if (prev & meaning) {
                            error(location, ts.Diagnostics.Duplicate_identifier_0, ts.getTextOfNode(location));
                        }
                        else {
                            names.set(name, prev | meaning);
                        }
                    }
                    else {
                        names.set(name, meaning);
                    }
                }
            }
            /**
             * Static members being set on a constructor function may conflict with built-in properties
             * of Function. Esp. in ECMAScript 5 there are non-configurable and non-writable
             * built-in properties. This check issues a transpile error when a class has a static
             * member with the same name as a non-writable built-in property.
             *
             * @see http://www.ecma-international.org/ecma-262/5.1/#sec-15.3.3
             * @see http://www.ecma-international.org/ecma-262/5.1/#sec-15.3.5
             * @see http://www.ecma-international.org/ecma-262/6.0/#sec-properties-of-the-function-constructor
             * @see http://www.ecma-international.org/ecma-262/6.0/#sec-function-instances
             */
            function checkClassForStaticPropertyNameConflicts(node) {
                for (var _i = 0, _a = node.members; _i < _a.length; _i++) {
                    var member = _a[_i];
                    var memberNameNode = member.name;
                    var isStatic = ts.hasModifier(member, 32 /* Static */);
                    if (isStatic && memberNameNode) {
                        var memberName = ts.getPropertyNameForPropertyNameNode(memberNameNode);
                        switch (memberName) {
                            case "name":
                            case "length":
                            case "caller":
                            case "arguments":
                            case "prototype":
                                var message = ts.Diagnostics.Static_property_0_conflicts_with_built_in_property_Function_0_of_constructor_function_1;
                                var className = getNameOfSymbolAsWritten(getSymbolOfNode(node));
                                error(memberNameNode, message, memberName, className);
                                break;
                        }
                    }
                }
            }
            function checkObjectTypeForDuplicateDeclarations(node) {
                var names = ts.createMap();
                for (var _i = 0, _a = node.members; _i < _a.length; _i++) {
                    var member = _a[_i];
                    if (member.kind === 158 /* PropertySignature */) {
                        var memberName = void 0;
                        var name = member.name;
                        switch (name.kind) {
                            case 10 /* StringLiteral */:
                            case 8 /* NumericLiteral */:
                                memberName = name.text;
                                break;
                            case 75 /* Identifier */:
                                memberName = ts.idText(name);
                                break;
                            default:
                                continue;
                        }
                        if (names.get(memberName)) {
                            error(ts.getNameOfDeclaration(member.symbol.valueDeclaration), ts.Diagnostics.Duplicate_identifier_0, memberName);
                            error(member.name, ts.Diagnostics.Duplicate_identifier_0, memberName);
                        }
                        else {
                            names.set(memberName, true);
                        }
                    }
                }
            }
            function checkTypeForDuplicateIndexSignatures(node) {
                if (node.kind === 246 /* InterfaceDeclaration */) {
                    var nodeSymbol = getSymbolOfNode(node);
                    // in case of merging interface declaration it is possible that we'll enter this check procedure several times for every declaration
                    // to prevent this run check only for the first declaration of a given kind
                    if (nodeSymbol.declarations.length > 0 && nodeSymbol.declarations[0] !== node) {
                        return;
                    }
                }
                // TypeScript 1.0 spec (April 2014)
                // 3.7.4: An object type can contain at most one string index signature and one numeric index signature.
                // 8.5: A class declaration can have at most one string index member declaration and one numeric index member declaration
                var indexSymbol = getIndexSymbol(getSymbolOfNode(node));
                if (indexSymbol) {
                    var seenNumericIndexer = false;
                    var seenStringIndexer = false;
                    for (var _i = 0, _a = indexSymbol.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        var declaration = decl;
                        if (declaration.parameters.length === 1 && declaration.parameters[0].type) {
                            switch (declaration.parameters[0].type.kind) {
                                case 143 /* StringKeyword */:
                                    if (!seenStringIndexer) {
                                        seenStringIndexer = true;
                                    }
                                    else {
                                        error(declaration, ts.Diagnostics.Duplicate_string_index_signature);
                                    }
                                    break;
                                case 140 /* NumberKeyword */:
                                    if (!seenNumericIndexer) {
                                        seenNumericIndexer = true;
                                    }
                                    else {
                                        error(declaration, ts.Diagnostics.Duplicate_number_index_signature);
                                    }
                                    break;
                            }
                        }
                    }
                }
            }
            function checkPropertyDeclaration(node) {
                // Grammar checking
                if (!checkGrammarDecoratorsAndModifiers(node) && !checkGrammarProperty(node))
                    checkGrammarComputedPropertyName(node.name);
                checkVariableLikeDeclaration(node);
                // Private class fields transformation relies on WeakMaps.
                if (ts.isPrivateIdentifier(node.name) && languageVersion < 99 /* ESNext */) {
                    for (var lexicalScope = ts.getEnclosingBlockScopeContainer(node); !!lexicalScope; lexicalScope = ts.getEnclosingBlockScopeContainer(lexicalScope)) {
                        getNodeLinks(lexicalScope).flags |= 67108864 /* ContainsClassWithPrivateIdentifiers */;
                    }
                }
            }
            function checkPropertySignature(node) {
                if (ts.isPrivateIdentifier(node.name)) {
                    error(node, ts.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies);
                }
                return checkPropertyDeclaration(node);
            }
            function checkMethodDeclaration(node) {
                // Grammar checking
                if (!checkGrammarMethod(node))
                    checkGrammarComputedPropertyName(node.name);
                if (ts.isPrivateIdentifier(node.name)) {
                    error(node, ts.Diagnostics.A_method_cannot_be_named_with_a_private_identifier);
                }
                // Grammar checking for modifiers is done inside the function checkGrammarFunctionLikeDeclaration
                checkFunctionOrMethodDeclaration(node);
                // Abstract methods cannot have an implementation.
                // Extra checks are to avoid reporting multiple errors relating to the "abstractness" of the node.
                if (ts.hasModifier(node, 128 /* Abstract */) && node.kind === 161 /* MethodDeclaration */ && node.body) {
                    error(node, ts.Diagnostics.Method_0_cannot_have_an_implementation_because_it_is_marked_abstract, ts.declarationNameToString(node.name));
                }
            }
            function checkConstructorDeclaration(node) {
                // Grammar check on signature of constructor and modifier of the constructor is done in checkSignatureDeclaration function.
                checkSignatureDeclaration(node);
                // Grammar check for checking only related to constructorDeclaration
                if (!checkGrammarConstructorTypeParameters(node))
                    checkGrammarConstructorTypeAnnotation(node);
                checkSourceElement(node.body);
                var symbol = getSymbolOfNode(node);
                var firstDeclaration = ts.getDeclarationOfKind(symbol, node.kind);
                // Only type check the symbol once
                if (node === firstDeclaration) {
                    checkFunctionOrConstructorSymbol(symbol);
                }
                // exit early in the case of signature - super checks are not relevant to them
                if (ts.nodeIsMissing(node.body)) {
                    return;
                }
                if (!produceDiagnostics) {
                    return;
                }
                function isInstancePropertyWithInitializerOrPrivateIdentifierProperty(n) {
                    if (ts.isPrivateIdentifierPropertyDeclaration(n)) {
                        return true;
                    }
                    return n.kind === 159 /* PropertyDeclaration */ &&
                        !ts.hasModifier(n, 32 /* Static */) &&
                        !!n.initializer;
                }
                // TS 1.0 spec (April 2014): 8.3.2
                // Constructors of classes with no extends clause may not contain super calls, whereas
                // constructors of derived classes must contain at least one super call somewhere in their function body.
                var containingClassDecl = node.parent;
                if (ts.getClassExtendsHeritageElement(containingClassDecl)) {
                    captureLexicalThis(node.parent, containingClassDecl);
                    var classExtendsNull = classDeclarationExtendsNull(containingClassDecl);
                    var superCall = getSuperCallInConstructor(node);
                    if (superCall) {
                        if (classExtendsNull) {
                            error(superCall, ts.Diagnostics.A_constructor_cannot_contain_a_super_call_when_its_class_extends_null);
                        }
                        // The first statement in the body of a constructor (excluding prologue directives) must be a super call
                        // if both of the following are true:
                        // - The containing class is a derived class.
                        // - The constructor declares parameter properties
                        //   or the containing class declares instance member variables with initializers.
                        var superCallShouldBeFirst = ts.some(node.parent.members, isInstancePropertyWithInitializerOrPrivateIdentifierProperty) ||
                            ts.some(node.parameters, function (p) { return ts.hasModifier(p, 92 /* ParameterPropertyModifier */); });
                        // Skip past any prologue directives to find the first statement
                        // to ensure that it was a super call.
                        if (superCallShouldBeFirst) {
                            var statements = node.body.statements;
                            var superCallStatement = void 0;
                            for (var _i = 0, statements_3 = statements; _i < statements_3.length; _i++) {
                                var statement = statements_3[_i];
                                if (statement.kind === 226 /* ExpressionStatement */ && ts.isSuperCall(statement.expression)) {
                                    superCallStatement = statement;
                                    break;
                                }
                                if (!ts.isPrologueDirective(statement)) {
                                    break;
                                }
                            }
                            if (!superCallStatement) {
                                error(node, ts.Diagnostics.A_super_call_must_be_the_first_statement_in_the_constructor_when_a_class_contains_initialized_properties_parameter_properties_or_private_identifiers);
                            }
                        }
                    }
                    else if (!classExtendsNull) {
                        error(node, ts.Diagnostics.Constructors_for_derived_classes_must_contain_a_super_call);
                    }
                }
            }
            function checkAccessorDeclaration(node) {
                if (produceDiagnostics) {
                    // Grammar checking accessors
                    if (!checkGrammarFunctionLikeDeclaration(node) && !checkGrammarAccessor(node))
                        checkGrammarComputedPropertyName(node.name);
                    checkDecorators(node);
                    checkSignatureDeclaration(node);
                    if (node.kind === 163 /* GetAccessor */) {
                        if (!(node.flags & 8388608 /* Ambient */) && ts.nodeIsPresent(node.body) && (node.flags & 256 /* HasImplicitReturn */)) {
                            if (!(node.flags & 512 /* HasExplicitReturn */)) {
                                error(node.name, ts.Diagnostics.A_get_accessor_must_return_a_value);
                            }
                        }
                    }
                    // Do not use hasDynamicName here, because that returns false for well known symbols.
                    // We want to perform checkComputedPropertyName for all computed properties, including
                    // well known symbols.
                    if (node.name.kind === 154 /* ComputedPropertyName */) {
                        checkComputedPropertyName(node.name);
                    }
                    if (ts.isPrivateIdentifier(node.name)) {
                        error(node.name, ts.Diagnostics.An_accessor_cannot_be_named_with_a_private_identifier);
                    }
                    if (!hasNonBindableDynamicName(node)) {
                        // TypeScript 1.0 spec (April 2014): 8.4.3
                        // Accessors for the same member name must specify the same accessibility.
                        var otherKind = node.kind === 163 /* GetAccessor */ ? 164 /* SetAccessor */ : 163 /* GetAccessor */;
                        var otherAccessor = ts.getDeclarationOfKind(getSymbolOfNode(node), otherKind);
                        if (otherAccessor) {
                            var nodeFlags = ts.getModifierFlags(node);
                            var otherFlags = ts.getModifierFlags(otherAccessor);
                            if ((nodeFlags & 28 /* AccessibilityModifier */) !== (otherFlags & 28 /* AccessibilityModifier */)) {
                                error(node.name, ts.Diagnostics.Getter_and_setter_accessors_do_not_agree_in_visibility);
                            }
                            if ((nodeFlags & 128 /* Abstract */) !== (otherFlags & 128 /* Abstract */)) {
                                error(node.name, ts.Diagnostics.Accessors_must_both_be_abstract_or_non_abstract);
                            }
                            // TypeScript 1.0 spec (April 2014): 4.5
                            // If both accessors include type annotations, the specified types must be identical.
                            checkAccessorDeclarationTypesIdentical(node, otherAccessor, getAnnotatedAccessorType, ts.Diagnostics.get_and_set_accessor_must_have_the_same_type);
                            checkAccessorDeclarationTypesIdentical(node, otherAccessor, getThisTypeOfDeclaration, ts.Diagnostics.get_and_set_accessor_must_have_the_same_this_type);
                        }
                    }
                    var returnType = getTypeOfAccessors(getSymbolOfNode(node));
                    if (node.kind === 163 /* GetAccessor */) {
                        checkAllCodePathsInNonVoidFunctionReturnOrThrow(node, returnType);
                    }
                }
                checkSourceElement(node.body);
            }
            function checkAccessorDeclarationTypesIdentical(first, second, getAnnotatedType, message) {
                var firstType = getAnnotatedType(first);
                var secondType = getAnnotatedType(second);
                if (firstType && secondType && !isTypeIdenticalTo(firstType, secondType)) {
                    error(first, message);
                }
            }
            function checkMissingDeclaration(node) {
                checkDecorators(node);
            }
            function getEffectiveTypeArguments(node, typeParameters) {
                return fillMissingTypeArguments(ts.map(node.typeArguments, getTypeFromTypeNode), typeParameters, getMinTypeArgumentCount(typeParameters), ts.isInJSFile(node));
            }
            function checkTypeArgumentConstraints(node, typeParameters) {
                var typeArguments;
                var mapper;
                var result = true;
                for (var i = 0; i < typeParameters.length; i++) {
                    var constraint = getConstraintOfTypeParameter(typeParameters[i]);
                    if (constraint) {
                        if (!typeArguments) {
                            typeArguments = getEffectiveTypeArguments(node, typeParameters);
                            mapper = createTypeMapper(typeParameters, typeArguments);
                        }
                        result = result && checkTypeAssignableTo(typeArguments[i], instantiateType(constraint, mapper), node.typeArguments[i], ts.Diagnostics.Type_0_does_not_satisfy_the_constraint_1);
                    }
                }
                return result;
            }
            function getTypeParametersForTypeReference(node) {
                var type = getTypeFromTypeReference(node);
                if (type !== errorType) {
                    var symbol = getNodeLinks(node).resolvedSymbol;
                    if (symbol) {
                        return symbol.flags & 524288 /* TypeAlias */ && getSymbolLinks(symbol).typeParameters ||
                            (ts.getObjectFlags(type) & 4 /* Reference */ ? type.target.localTypeParameters : undefined);
                    }
                }
                return undefined;
            }
            function checkTypeReferenceNode(node) {
                checkGrammarTypeArguments(node, node.typeArguments);
                if (node.kind === 169 /* TypeReference */ && node.typeName.jsdocDotPos !== undefined && !ts.isInJSFile(node) && !ts.isInJSDoc(node)) {
                    grammarErrorAtPos(node, node.typeName.jsdocDotPos, 1, ts.Diagnostics.JSDoc_types_can_only_be_used_inside_documentation_comments);
                }
                ts.forEach(node.typeArguments, checkSourceElement);
                var type = getTypeFromTypeReference(node);
                if (type !== errorType) {
                    if (node.typeArguments && produceDiagnostics) {
                        var typeParameters = getTypeParametersForTypeReference(node);
                        if (typeParameters) {
                            checkTypeArgumentConstraints(node, typeParameters);
                        }
                    }
                    if (type.flags & 32 /* Enum */ && getNodeLinks(node).resolvedSymbol.flags & 8 /* EnumMember */) {
                        error(node, ts.Diagnostics.Enum_type_0_has_members_with_initializers_that_are_not_literals, typeToString(type));
                    }
                }
            }
            function getTypeArgumentConstraint(node) {
                var typeReferenceNode = ts.tryCast(node.parent, ts.isTypeReferenceType);
                if (!typeReferenceNode)
                    return undefined;
                var typeParameters = getTypeParametersForTypeReference(typeReferenceNode); // TODO: GH#18217
                var constraint = getConstraintOfTypeParameter(typeParameters[typeReferenceNode.typeArguments.indexOf(node)]);
                return constraint && instantiateType(constraint, createTypeMapper(typeParameters, getEffectiveTypeArguments(typeReferenceNode, typeParameters)));
            }
            function checkTypeQuery(node) {
                getTypeFromTypeQueryNode(node);
            }
            function checkTypeLiteral(node) {
                ts.forEach(node.members, checkSourceElement);
                if (produceDiagnostics) {
                    var type = getTypeFromTypeLiteralOrFunctionOrConstructorTypeNode(node);
                    checkIndexConstraints(type);
                    checkTypeForDuplicateIndexSignatures(node);
                    checkObjectTypeForDuplicateDeclarations(node);
                }
            }
            function checkArrayType(node) {
                checkSourceElement(node.elementType);
            }
            function checkTupleType(node) {
                var elementTypes = node.elementTypes;
                var seenOptionalElement = false;
                for (var i = 0; i < elementTypes.length; i++) {
                    var e = elementTypes[i];
                    if (e.kind === 177 /* RestType */) {
                        if (i !== elementTypes.length - 1) {
                            grammarErrorOnNode(e, ts.Diagnostics.A_rest_element_must_be_last_in_a_tuple_type);
                            break;
                        }
                        if (!isArrayType(getTypeFromTypeNode(e.type))) {
                            error(e, ts.Diagnostics.A_rest_element_type_must_be_an_array_type);
                        }
                    }
                    else if (e.kind === 176 /* OptionalType */) {
                        seenOptionalElement = true;
                    }
                    else if (seenOptionalElement) {
                        grammarErrorOnNode(e, ts.Diagnostics.A_required_element_cannot_follow_an_optional_element);
                        break;
                    }
                }
                ts.forEach(node.elementTypes, checkSourceElement);
            }
            function checkUnionOrIntersectionType(node) {
                ts.forEach(node.types, checkSourceElement);
            }
            function checkIndexedAccessIndexType(type, accessNode) {
                if (!(type.flags & 8388608 /* IndexedAccess */)) {
                    return type;
                }
                // Check if the index type is assignable to 'keyof T' for the object type.
                var objectType = type.objectType;
                var indexType = type.indexType;
                if (isTypeAssignableTo(indexType, getIndexType(objectType, /*stringsOnly*/ false))) {
                    if (accessNode.kind === 195 /* ElementAccessExpression */ && ts.isAssignmentTarget(accessNode) &&
                        ts.getObjectFlags(objectType) & 32 /* Mapped */ && getMappedTypeModifiers(objectType) & 1 /* IncludeReadonly */) {
                        error(accessNode, ts.Diagnostics.Index_signature_in_type_0_only_permits_reading, typeToString(objectType));
                    }
                    return type;
                }
                // Check if we're indexing with a numeric type and if either object or index types
                // is a generic type with a constraint that has a numeric index signature.
                var apparentObjectType = getApparentType(objectType);
                if (getIndexInfoOfType(apparentObjectType, 1 /* Number */) && isTypeAssignableToKind(indexType, 296 /* NumberLike */)) {
                    return type;
                }
                if (isGenericObjectType(objectType)) {
                    var propertyName_1 = getPropertyNameFromIndex(indexType, accessNode);
                    if (propertyName_1) {
                        var propertySymbol = forEachType(apparentObjectType, function (t) { return getPropertyOfType(t, propertyName_1); });
                        if (propertySymbol && ts.getDeclarationModifierFlagsFromSymbol(propertySymbol) & 24 /* NonPublicAccessibilityModifier */) {
                            error(accessNode, ts.Diagnostics.Private_or_protected_member_0_cannot_be_accessed_on_a_type_parameter, ts.unescapeLeadingUnderscores(propertyName_1));
                            return errorType;
                        }
                    }
                }
                error(accessNode, ts.Diagnostics.Type_0_cannot_be_used_to_index_type_1, typeToString(indexType), typeToString(objectType));
                return errorType;
            }
            function checkIndexedAccessType(node) {
                checkSourceElement(node.objectType);
                checkSourceElement(node.indexType);
                checkIndexedAccessIndexType(getTypeFromIndexedAccessTypeNode(node), node);
            }
            function checkMappedType(node) {
                checkSourceElement(node.typeParameter);
                checkSourceElement(node.type);
                if (!node.type) {
                    reportImplicitAny(node, anyType);
                }
                var type = getTypeFromMappedTypeNode(node);
                var constraintType = getConstraintTypeFromMappedType(type);
                checkTypeAssignableTo(constraintType, keyofConstraintType, ts.getEffectiveConstraintOfTypeParameter(node.typeParameter));
            }
            function checkThisType(node) {
                getTypeFromThisTypeNode(node);
            }
            function checkTypeOperator(node) {
                checkGrammarTypeOperatorNode(node);
                checkSourceElement(node.type);
            }
            function checkConditionalType(node) {
                ts.forEachChild(node, checkSourceElement);
            }
            function checkInferType(node) {
                if (!ts.findAncestor(node, function (n) { return n.parent && n.parent.kind === 180 /* ConditionalType */ && n.parent.extendsType === n; })) {
                    grammarErrorOnNode(node, ts.Diagnostics.infer_declarations_are_only_permitted_in_the_extends_clause_of_a_conditional_type);
                }
                checkSourceElement(node.typeParameter);
                registerForUnusedIdentifiersCheck(node);
            }
            function checkImportType(node) {
                checkSourceElement(node.argument);
                getTypeFromTypeNode(node);
            }
            function isPrivateWithinAmbient(node) {
                return (ts.hasModifier(node, 8 /* Private */) || ts.isPrivateIdentifierPropertyDeclaration(node)) && !!(node.flags & 8388608 /* Ambient */);
            }
            function getEffectiveDeclarationFlags(n, flagsToCheck) {
                var flags = ts.getCombinedModifierFlags(n);
                // children of classes (even ambient classes) should not be marked as ambient or export
                // because those flags have no useful semantics there.
                if (n.parent.kind !== 246 /* InterfaceDeclaration */ &&
                    n.parent.kind !== 245 /* ClassDeclaration */ &&
                    n.parent.kind !== 214 /* ClassExpression */ &&
                    n.flags & 8388608 /* Ambient */) {
                    if (!(flags & 2 /* Ambient */) && !(ts.isModuleBlock(n.parent) && ts.isModuleDeclaration(n.parent.parent) && ts.isGlobalScopeAugmentation(n.parent.parent))) {
                        // It is nested in an ambient context, which means it is automatically exported
                        flags |= 1 /* Export */;
                    }
                    flags |= 2 /* Ambient */;
                }
                return flags & flagsToCheck;
            }
            function checkFunctionOrConstructorSymbol(symbol) {
                if (!produceDiagnostics) {
                    return;
                }
                function getCanonicalOverload(overloads, implementation) {
                    // Consider the canonical set of flags to be the flags of the bodyDeclaration or the first declaration
                    // Error on all deviations from this canonical set of flags
                    // The caveat is that if some overloads are defined in lib.d.ts, we don't want to
                    // report the errors on those. To achieve this, we will say that the implementation is
                    // the canonical signature only if it is in the same container as the first overload
                    var implementationSharesContainerWithFirstOverload = implementation !== undefined && implementation.parent === overloads[0].parent;
                    return implementationSharesContainerWithFirstOverload ? implementation : overloads[0];
                }
                function checkFlagAgreementBetweenOverloads(overloads, implementation, flagsToCheck, someOverloadFlags, allOverloadFlags) {
                    // Error if some overloads have a flag that is not shared by all overloads. To find the
                    // deviations, we XOR someOverloadFlags with allOverloadFlags
                    var someButNotAllOverloadFlags = someOverloadFlags ^ allOverloadFlags;
                    if (someButNotAllOverloadFlags !== 0) {
                        var canonicalFlags_1 = getEffectiveDeclarationFlags(getCanonicalOverload(overloads, implementation), flagsToCheck);
                        ts.forEach(overloads, function (o) {
                            var deviation = getEffectiveDeclarationFlags(o, flagsToCheck) ^ canonicalFlags_1;
                            if (deviation & 1 /* Export */) {
                                error(ts.getNameOfDeclaration(o), ts.Diagnostics.Overload_signatures_must_all_be_exported_or_non_exported);
                            }
                            else if (deviation & 2 /* Ambient */) {
                                error(ts.getNameOfDeclaration(o), ts.Diagnostics.Overload_signatures_must_all_be_ambient_or_non_ambient);
                            }
                            else if (deviation & (8 /* Private */ | 16 /* Protected */)) {
                                error(ts.getNameOfDeclaration(o) || o, ts.Diagnostics.Overload_signatures_must_all_be_public_private_or_protected);
                            }
                            else if (deviation & 128 /* Abstract */) {
                                error(ts.getNameOfDeclaration(o), ts.Diagnostics.Overload_signatures_must_all_be_abstract_or_non_abstract);
                            }
                        });
                    }
                }
                function checkQuestionTokenAgreementBetweenOverloads(overloads, implementation, someHaveQuestionToken, allHaveQuestionToken) {
                    if (someHaveQuestionToken !== allHaveQuestionToken) {
                        var canonicalHasQuestionToken_1 = ts.hasQuestionToken(getCanonicalOverload(overloads, implementation));
                        ts.forEach(overloads, function (o) {
                            var deviation = ts.hasQuestionToken(o) !== canonicalHasQuestionToken_1;
                            if (deviation) {
                                error(ts.getNameOfDeclaration(o), ts.Diagnostics.Overload_signatures_must_all_be_optional_or_required);
                            }
                        });
                    }
                }
                var flagsToCheck = 1 /* Export */ | 2 /* Ambient */ | 8 /* Private */ | 16 /* Protected */ | 128 /* Abstract */;
                var someNodeFlags = 0 /* None */;
                var allNodeFlags = flagsToCheck;
                var someHaveQuestionToken = false;
                var allHaveQuestionToken = true;
                var hasOverloads = false;
                var bodyDeclaration;
                var lastSeenNonAmbientDeclaration;
                var previousDeclaration;
                var declarations = symbol.declarations;
                var isConstructor = (symbol.flags & 16384 /* Constructor */) !== 0;
                function reportImplementationExpectedError(node) {
                    if (node.name && ts.nodeIsMissing(node.name)) {
                        return;
                    }
                    var seen = false;
                    var subsequentNode = ts.forEachChild(node.parent, function (c) {
                        if (seen) {
                            return c;
                        }
                        else {
                            seen = c === node;
                        }
                    });
                    // We may be here because of some extra nodes between overloads that could not be parsed into a valid node.
                    // In this case the subsequent node is not really consecutive (.pos !== node.end), and we must ignore it here.
                    if (subsequentNode && subsequentNode.pos === node.end) {
                        if (subsequentNode.kind === node.kind) {
                            var errorNode_1 = subsequentNode.name || subsequentNode;
                            var subsequentName = subsequentNode.name;
                            if (node.name && subsequentName && (
                            // both are private identifiers
                            ts.isPrivateIdentifier(node.name) && ts.isPrivateIdentifier(subsequentName) && node.name.escapedText === subsequentName.escapedText ||
                                // Both are computed property names
                                // TODO: GH#17345: These are methods, so handle computed name case. (`Always allowing computed property names is *not* the correct behavior!)
                                ts.isComputedPropertyName(node.name) && ts.isComputedPropertyName(subsequentName) ||
                                // Both are literal property names that are the same.
                                ts.isPropertyNameLiteral(node.name) && ts.isPropertyNameLiteral(subsequentName) &&
                                    ts.getEscapedTextOfIdentifierOrLiteral(node.name) === ts.getEscapedTextOfIdentifierOrLiteral(subsequentName))) {
                                var reportError = (node.kind === 161 /* MethodDeclaration */ || node.kind === 160 /* MethodSignature */) &&
                                    ts.hasModifier(node, 32 /* Static */) !== ts.hasModifier(subsequentNode, 32 /* Static */);
                                // we can get here in two cases
                                // 1. mixed static and instance class members
                                // 2. something with the same name was defined before the set of overloads that prevents them from merging
                                // here we'll report error only for the first case since for second we should already report error in binder
                                if (reportError) {
                                    var diagnostic = ts.hasModifier(node, 32 /* Static */) ? ts.Diagnostics.Function_overload_must_be_static : ts.Diagnostics.Function_overload_must_not_be_static;
                                    error(errorNode_1, diagnostic);
                                }
                                return;
                            }
                            if (ts.nodeIsPresent(subsequentNode.body)) {
                                error(errorNode_1, ts.Diagnostics.Function_implementation_name_must_be_0, ts.declarationNameToString(node.name));
                                return;
                            }
                        }
                    }
                    var errorNode = node.name || node;
                    if (isConstructor) {
                        error(errorNode, ts.Diagnostics.Constructor_implementation_is_missing);
                    }
                    else {
                        // Report different errors regarding non-consecutive blocks of declarations depending on whether
                        // the node in question is abstract.
                        if (ts.hasModifier(node, 128 /* Abstract */)) {
                            error(errorNode, ts.Diagnostics.All_declarations_of_an_abstract_method_must_be_consecutive);
                        }
                        else {
                            error(errorNode, ts.Diagnostics.Function_implementation_is_missing_or_not_immediately_following_the_declaration);
                        }
                    }
                }
                var duplicateFunctionDeclaration = false;
                var multipleConstructorImplementation = false;
                var hasNonAmbientClass = false;
                for (var _i = 0, declarations_4 = declarations; _i < declarations_4.length; _i++) {
                    var current = declarations_4[_i];
                    var node = current;
                    var inAmbientContext = node.flags & 8388608 /* Ambient */;
                    var inAmbientContextOrInterface = node.parent.kind === 246 /* InterfaceDeclaration */ || node.parent.kind === 173 /* TypeLiteral */ || inAmbientContext;
                    if (inAmbientContextOrInterface) {
                        // check if declarations are consecutive only if they are non-ambient
                        // 1. ambient declarations can be interleaved
                        // i.e. this is legal
                        //     declare function foo();
                        //     declare function bar();
                        //     declare function foo();
                        // 2. mixing ambient and non-ambient declarations is a separate error that will be reported - do not want to report an extra one
                        previousDeclaration = undefined;
                    }
                    if ((node.kind === 245 /* ClassDeclaration */ || node.kind === 214 /* ClassExpression */) && !inAmbientContext) {
                        hasNonAmbientClass = true;
                    }
                    if (node.kind === 244 /* FunctionDeclaration */ || node.kind === 161 /* MethodDeclaration */ || node.kind === 160 /* MethodSignature */ || node.kind === 162 /* Constructor */) {
                        var currentNodeFlags = getEffectiveDeclarationFlags(node, flagsToCheck);
                        someNodeFlags |= currentNodeFlags;
                        allNodeFlags &= currentNodeFlags;
                        someHaveQuestionToken = someHaveQuestionToken || ts.hasQuestionToken(node);
                        allHaveQuestionToken = allHaveQuestionToken && ts.hasQuestionToken(node);
                        if (ts.nodeIsPresent(node.body) && bodyDeclaration) {
                            if (isConstructor) {
                                multipleConstructorImplementation = true;
                            }
                            else {
                                duplicateFunctionDeclaration = true;
                            }
                        }
                        else if (previousDeclaration && previousDeclaration.parent === node.parent && previousDeclaration.end !== node.pos) {
                            reportImplementationExpectedError(previousDeclaration);
                        }
                        if (ts.nodeIsPresent(node.body)) {
                            if (!bodyDeclaration) {
                                bodyDeclaration = node;
                            }
                        }
                        else {
                            hasOverloads = true;
                        }
                        previousDeclaration = node;
                        if (!inAmbientContextOrInterface) {
                            lastSeenNonAmbientDeclaration = node;
                        }
                    }
                }
                if (multipleConstructorImplementation) {
                    ts.forEach(declarations, function (declaration) {
                        error(declaration, ts.Diagnostics.Multiple_constructor_implementations_are_not_allowed);
                    });
                }
                if (duplicateFunctionDeclaration) {
                    ts.forEach(declarations, function (declaration) {
                        error(ts.getNameOfDeclaration(declaration), ts.Diagnostics.Duplicate_function_implementation);
                    });
                }
                if (hasNonAmbientClass && !isConstructor && symbol.flags & 16 /* Function */) {
                    // A non-ambient class cannot be an implementation for a non-constructor function/class merge
                    // TODO: The below just replicates our older error from when classes and functions were
                    // entirely unable to merge - a more helpful message like "Class declaration cannot implement overload list"
                    // might be warranted. :shrug:
                    ts.forEach(declarations, function (declaration) {
                        addDuplicateDeclarationError(ts.getNameOfDeclaration(declaration) || declaration, ts.Diagnostics.Duplicate_identifier_0, ts.symbolName(symbol), ts.filter(declarations, function (d) { return d !== declaration; }));
                    });
                }
                // Abstract methods can't have an implementation -- in particular, they don't need one.
                if (lastSeenNonAmbientDeclaration && !lastSeenNonAmbientDeclaration.body &&
                    !ts.hasModifier(lastSeenNonAmbientDeclaration, 128 /* Abstract */) && !lastSeenNonAmbientDeclaration.questionToken) {
                    reportImplementationExpectedError(lastSeenNonAmbientDeclaration);
                }
                if (hasOverloads) {
                    checkFlagAgreementBetweenOverloads(declarations, bodyDeclaration, flagsToCheck, someNodeFlags, allNodeFlags);
                    checkQuestionTokenAgreementBetweenOverloads(declarations, bodyDeclaration, someHaveQuestionToken, allHaveQuestionToken);
                    if (bodyDeclaration) {
                        var signatures = getSignaturesOfSymbol(symbol);
                        var bodySignature = getSignatureFromDeclaration(bodyDeclaration);
                        for (var _a = 0, signatures_10 = signatures; _a < signatures_10.length; _a++) {
                            var signature = signatures_10[_a];
                            if (!isImplementationCompatibleWithOverload(bodySignature, signature)) {
                                ts.addRelatedInfo(error(signature.declaration, ts.Diagnostics.This_overload_signature_is_not_compatible_with_its_implementation_signature), ts.createDiagnosticForNode(bodyDeclaration, ts.Diagnostics.The_implementation_signature_is_declared_here));
                                break;
                            }
                        }
                    }
                }
            }
            function checkExportsOnMergedDeclarations(node) {
                if (!produceDiagnostics) {
                    return;
                }
                // if localSymbol is defined on node then node itself is exported - check is required
                var symbol = node.localSymbol;
                if (!symbol) {
                    // local symbol is undefined => this declaration is non-exported.
                    // however symbol might contain other declarations that are exported
                    symbol = getSymbolOfNode(node);
                    if (!symbol.exportSymbol) {
                        // this is a pure local symbol (all declarations are non-exported) - no need to check anything
                        return;
                    }
                }
                // run the check only for the first declaration in the list
                if (ts.getDeclarationOfKind(symbol, node.kind) !== node) {
                    return;
                }
                var exportedDeclarationSpaces = 0 /* None */;
                var nonExportedDeclarationSpaces = 0 /* None */;
                var defaultExportedDeclarationSpaces = 0 /* None */;
                for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                    var d = _a[_i];
                    var declarationSpaces = getDeclarationSpaces(d);
                    var effectiveDeclarationFlags = getEffectiveDeclarationFlags(d, 1 /* Export */ | 512 /* Default */);
                    if (effectiveDeclarationFlags & 1 /* Export */) {
                        if (effectiveDeclarationFlags & 512 /* Default */) {
                            defaultExportedDeclarationSpaces |= declarationSpaces;
                        }
                        else {
                            exportedDeclarationSpaces |= declarationSpaces;
                        }
                    }
                    else {
                        nonExportedDeclarationSpaces |= declarationSpaces;
                    }
                }
                // Spaces for anything not declared a 'default export'.
                var nonDefaultExportedDeclarationSpaces = exportedDeclarationSpaces | nonExportedDeclarationSpaces;
                var commonDeclarationSpacesForExportsAndLocals = exportedDeclarationSpaces & nonExportedDeclarationSpaces;
                var commonDeclarationSpacesForDefaultAndNonDefault = defaultExportedDeclarationSpaces & nonDefaultExportedDeclarationSpaces;
                if (commonDeclarationSpacesForExportsAndLocals || commonDeclarationSpacesForDefaultAndNonDefault) {
                    // declaration spaces for exported and non-exported declarations intersect
                    for (var _b = 0, _c = symbol.declarations; _b < _c.length; _b++) {
                        var d = _c[_b];
                        var declarationSpaces = getDeclarationSpaces(d);
                        var name = ts.getNameOfDeclaration(d);
                        // Only error on the declarations that contributed to the intersecting spaces.
                        if (declarationSpaces & commonDeclarationSpacesForDefaultAndNonDefault) {
                            error(name, ts.Diagnostics.Merged_declaration_0_cannot_include_a_default_export_declaration_Consider_adding_a_separate_export_default_0_declaration_instead, ts.declarationNameToString(name));
                        }
                        else if (declarationSpaces & commonDeclarationSpacesForExportsAndLocals) {
                            error(name, ts.Diagnostics.Individual_declarations_in_merged_declaration_0_must_be_all_exported_or_all_local, ts.declarationNameToString(name));
                        }
                    }
                }
                function getDeclarationSpaces(decl) {
                    var d = decl;
                    switch (d.kind) {
                        case 246 /* InterfaceDeclaration */:
                        case 247 /* TypeAliasDeclaration */:
                        // A jsdoc typedef and callback are, by definition, type aliases.
                        // falls through
                        case 321 /* JSDocTypedefTag */:
                        case 314 /* JSDocCallbackTag */:
                        case 315 /* JSDocEnumTag */:
                            return 2 /* ExportType */;
                        case 249 /* ModuleDeclaration */:
                            return ts.isAmbientModule(d) || ts.getModuleInstanceState(d) !== 0 /* NonInstantiated */
                                ? 4 /* ExportNamespace */ | 1 /* ExportValue */
                                : 4 /* ExportNamespace */;
                        case 245 /* ClassDeclaration */:
                        case 248 /* EnumDeclaration */:
                        case 284 /* EnumMember */:
                            return 2 /* ExportType */ | 1 /* ExportValue */;
                        case 290 /* SourceFile */:
                            return 2 /* ExportType */ | 1 /* ExportValue */ | 4 /* ExportNamespace */;
                        case 259 /* ExportAssignment */:
                            // Export assigned entity name expressions act as aliases and should fall through, otherwise they export values
                            if (!ts.isEntityNameExpression(d.expression)) {
                                return 1 /* ExportValue */;
                            }
                            d = d.expression;
                        // The below options all declare an Alias, which is allowed to merge with other values within the importing module.
                        // falls through
                        case 253 /* ImportEqualsDeclaration */:
                        case 256 /* NamespaceImport */:
                        case 255 /* ImportClause */:
                            var result_7 = 0 /* None */;
                            var target = resolveAlias(getSymbolOfNode(d));
                            ts.forEach(target.declarations, function (d) { result_7 |= getDeclarationSpaces(d); });
                            return result_7;
                        case 242 /* VariableDeclaration */:
                        case 191 /* BindingElement */:
                        case 244 /* FunctionDeclaration */:
                        case 258 /* ImportSpecifier */: // https://github.com/Microsoft/TypeScript/pull/7591
                        case 75 /* Identifier */: // https://github.com/microsoft/TypeScript/issues/36098
                            // Identifiers are used as declarations of assignment declarations whose parents may be
                            // SyntaxKind.CallExpression - `Object.defineProperty(thing, "aField", {value: 42});`
                            // SyntaxKind.ElementAccessExpression - `thing["aField"] = 42;` or `thing["aField"];` (with a doc comment on it)
                            // or SyntaxKind.PropertyAccessExpression - `thing.aField = 42;`
                            // all of which are pretty much always values, or at least imply a value meaning.
                            // It may be apprpriate to treat these as aliases in the future.
                            return 1 /* ExportValue */;
                        default:
                            return ts.Debug.failBadSyntaxKind(d);
                    }
                }
            }
            function getAwaitedTypeOfPromise(type, errorNode, diagnosticMessage, arg0) {
                var promisedType = getPromisedTypeOfPromise(type, errorNode);
                return promisedType && getAwaitedType(promisedType, errorNode, diagnosticMessage, arg0);
            }
            /**
             * Gets the "promised type" of a promise.
             * @param type The type of the promise.
             * @remarks The "promised type" of a type is the type of the "value" parameter of the "onfulfilled" callback.
             */
            function getPromisedTypeOfPromise(promise, errorNode) {
                //
                //  { // promise
                //      then( // thenFunction
                //          onfulfilled: ( // onfulfilledParameterType
                //              value: T // valueParameterType
                //          ) => any
                //      ): any;
                //  }
                //
                if (isTypeAny(promise)) {
                    return undefined;
                }
                var typeAsPromise = promise;
                if (typeAsPromise.promisedTypeOfPromise) {
                    return typeAsPromise.promisedTypeOfPromise;
                }
                if (isReferenceToType(promise, getGlobalPromiseType(/*reportErrors*/ false))) {
                    return typeAsPromise.promisedTypeOfPromise = getTypeArguments(promise)[0];
                }
                var thenFunction = getTypeOfPropertyOfType(promise, "then"); // TODO: GH#18217
                if (isTypeAny(thenFunction)) {
                    return undefined;
                }
                var thenSignatures = thenFunction ? getSignaturesOfType(thenFunction, 0 /* Call */) : ts.emptyArray;
                if (thenSignatures.length === 0) {
                    if (errorNode) {
                        error(errorNode, ts.Diagnostics.A_promise_must_have_a_then_method);
                    }
                    return undefined;
                }
                var onfulfilledParameterType = getTypeWithFacts(getUnionType(ts.map(thenSignatures, getTypeOfFirstParameterOfSignature)), 2097152 /* NEUndefinedOrNull */);
                if (isTypeAny(onfulfilledParameterType)) {
                    return undefined;
                }
                var onfulfilledParameterSignatures = getSignaturesOfType(onfulfilledParameterType, 0 /* Call */);
                if (onfulfilledParameterSignatures.length === 0) {
                    if (errorNode) {
                        error(errorNode, ts.Diagnostics.The_first_parameter_of_the_then_method_of_a_promise_must_be_a_callback);
                    }
                    return undefined;
                }
                return typeAsPromise.promisedTypeOfPromise = getUnionType(ts.map(onfulfilledParameterSignatures, getTypeOfFirstParameterOfSignature), 2 /* Subtype */);
            }
            /**
             * Gets the "awaited type" of a type.
             * @param type The type to await.
             * @remarks The "awaited type" of an expression is its "promised type" if the expression is a
             * Promise-like type; otherwise, it is the type of the expression. This is used to reflect
             * The runtime behavior of the `await` keyword.
             */
            function checkAwaitedType(type, errorNode, diagnosticMessage, arg0) {
                var awaitedType = getAwaitedType(type, errorNode, diagnosticMessage, arg0);
                return awaitedType || errorType;
            }
            function getAwaitedType(type, errorNode, diagnosticMessage, arg0) {
                var typeAsAwaitable = type;
                if (typeAsAwaitable.awaitedTypeOfType) {
                    return typeAsAwaitable.awaitedTypeOfType;
                }
                if (isTypeAny(type)) {
                    return typeAsAwaitable.awaitedTypeOfType = type;
                }
                if (type.flags & 1048576 /* Union */) {
                    var types = void 0;
                    for (var _i = 0, _a = type.types; _i < _a.length; _i++) {
                        var constituentType = _a[_i];
                        types = ts.append(types, getAwaitedType(constituentType, errorNode, diagnosticMessage, arg0));
                    }
                    if (!types) {
                        return undefined;
                    }
                    return typeAsAwaitable.awaitedTypeOfType = getUnionType(types);
                }
                var promisedType = getPromisedTypeOfPromise(type);
                if (promisedType) {
                    if (type.id === promisedType.id || awaitedTypeStack.indexOf(promisedType.id) >= 0) {
                        // Verify that we don't have a bad actor in the form of a promise whose
                        // promised type is the same as the promise type, or a mutually recursive
                        // promise. If so, we return undefined as we cannot guess the shape. If this
                        // were the actual case in the JavaScript, this Promise would never resolve.
                        //
                        // An example of a bad actor with a singly-recursive promise type might
                        // be:
                        //
                        //  interface BadPromise {
                        //      then(
                        //          onfulfilled: (value: BadPromise) => any,
                        //          onrejected: (error: any) => any): BadPromise;
                        //  }
                        // The above interface will pass the PromiseLike check, and return a
                        // promised type of `BadPromise`. Since this is a self reference, we
                        // don't want to keep recursing ad infinitum.
                        //
                        // An example of a bad actor in the form of a mutually-recursive
                        // promise type might be:
                        //
                        //  interface BadPromiseA {
                        //      then(
                        //          onfulfilled: (value: BadPromiseB) => any,
                        //          onrejected: (error: any) => any): BadPromiseB;
                        //  }
                        //
                        //  interface BadPromiseB {
                        //      then(
                        //          onfulfilled: (value: BadPromiseA) => any,
                        //          onrejected: (error: any) => any): BadPromiseA;
                        //  }
                        //
                        if (errorNode) {
                            error(errorNode, ts.Diagnostics.Type_is_referenced_directly_or_indirectly_in_the_fulfillment_callback_of_its_own_then_method);
                        }
                        return undefined;
                    }
                    // Keep track of the type we're about to unwrap to avoid bad recursive promise types.
                    // See the comments above for more information.
                    awaitedTypeStack.push(type.id);
                    var awaitedType = getAwaitedType(promisedType, errorNode, diagnosticMessage, arg0);
                    awaitedTypeStack.pop();
                    if (!awaitedType) {
                        return undefined;
                    }
                    return typeAsAwaitable.awaitedTypeOfType = awaitedType;
                }
                // The type was not a promise, so it could not be unwrapped any further.
                // As long as the type does not have a callable "then" property, it is
                // safe to return the type; otherwise, an error will be reported in
                // the call to getNonThenableType and we will return undefined.
                //
                // An example of a non-promise "thenable" might be:
                //
                //  await { then(): void {} }
                //
                // The "thenable" does not match the minimal definition for a promise. When
                // a Promise/A+-compatible or ES6 promise tries to adopt this value, the promise
                // will never settle. We treat this as an error to help flag an early indicator
                // of a runtime problem. If the user wants to return this value from an async
                // function, they would need to wrap it in some other value. If they want it to
                // be treated as a promise, they can cast to <any>.
                var thenFunction = getTypeOfPropertyOfType(type, "then");
                if (thenFunction && getSignaturesOfType(thenFunction, 0 /* Call */).length > 0) {
                    if (errorNode) {
                        if (!diagnosticMessage)
                            return ts.Debug.fail();
                        error(errorNode, diagnosticMessage, arg0);
                    }
                    return undefined;
                }
                return typeAsAwaitable.awaitedTypeOfType = type;
            }
            /**
             * Checks the return type of an async function to ensure it is a compatible
             * Promise implementation.
             *
             * This checks that an async function has a valid Promise-compatible return type.
             * An async function has a valid Promise-compatible return type if the resolved value
             * of the return type has a construct signature that takes in an `initializer` function
             * that in turn supplies a `resolve` function as one of its arguments and results in an
             * object with a callable `then` signature.
             *
             * @param node The signature to check
             */
            function checkAsyncFunctionReturnType(node, returnTypeNode) {
                // As part of our emit for an async function, we will need to emit the entity name of
                // the return type annotation as an expression. To meet the necessary runtime semantics
                // for __awaiter, we must also check that the type of the declaration (e.g. the static
                // side or "constructor" of the promise type) is compatible `PromiseConstructorLike`.
                //
                // An example might be (from lib.es6.d.ts):
                //
                //  interface Promise<T> { ... }
                //  interface PromiseConstructor {
                //      new <T>(...): Promise<T>;
                //  }
                //  declare var Promise: PromiseConstructor;
                //
                // When an async function declares a return type annotation of `Promise<T>`, we
                // need to get the type of the `Promise` variable declaration above, which would
                // be `PromiseConstructor`.
                //
                // The same case applies to a class:
                //
                //  declare class Promise<T> {
                //      constructor(...);
                //      then<U>(...): Promise<U>;
                //  }
                //
                var returnType = getTypeFromTypeNode(returnTypeNode);
                if (languageVersion >= 2 /* ES2015 */) {
                    if (returnType === errorType) {
                        return;
                    }
                    var globalPromiseType = getGlobalPromiseType(/*reportErrors*/ true);
                    if (globalPromiseType !== emptyGenericType && !isReferenceToType(returnType, globalPromiseType)) {
                        // The promise type was not a valid type reference to the global promise type, so we
                        // report an error and return the unknown type.
                        error(returnTypeNode, ts.Diagnostics.The_return_type_of_an_async_function_or_method_must_be_the_global_Promise_T_type);
                        return;
                    }
                }
                else {
                    // Always mark the type node as referenced if it points to a value
                    markTypeNodeAsReferenced(returnTypeNode);
                    if (returnType === errorType) {
                        return;
                    }
                    var promiseConstructorName = ts.getEntityNameFromTypeNode(returnTypeNode);
                    if (promiseConstructorName === undefined) {
                        error(returnTypeNode, ts.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value, typeToString(returnType));
                        return;
                    }
                    var promiseConstructorSymbol = resolveEntityName(promiseConstructorName, 111551 /* Value */, /*ignoreErrors*/ true);
                    var promiseConstructorType = promiseConstructorSymbol ? getTypeOfSymbol(promiseConstructorSymbol) : errorType;
                    if (promiseConstructorType === errorType) {
                        if (promiseConstructorName.kind === 75 /* Identifier */ && promiseConstructorName.escapedText === "Promise" && getTargetType(returnType) === getGlobalPromiseType(/*reportErrors*/ false)) {
                            error(returnTypeNode, ts.Diagnostics.An_async_function_or_method_in_ES5_SlashES3_requires_the_Promise_constructor_Make_sure_you_have_a_declaration_for_the_Promise_constructor_or_include_ES2015_in_your_lib_option);
                        }
                        else {
                            error(returnTypeNode, ts.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value, ts.entityNameToString(promiseConstructorName));
                        }
                        return;
                    }
                    var globalPromiseConstructorLikeType = getGlobalPromiseConstructorLikeType(/*reportErrors*/ true);
                    if (globalPromiseConstructorLikeType === emptyObjectType) {
                        // If we couldn't resolve the global PromiseConstructorLike type we cannot verify
                        // compatibility with __awaiter.
                        error(returnTypeNode, ts.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value, ts.entityNameToString(promiseConstructorName));
                        return;
                    }
                    if (!checkTypeAssignableTo(promiseConstructorType, globalPromiseConstructorLikeType, returnTypeNode, ts.Diagnostics.Type_0_is_not_a_valid_async_function_return_type_in_ES5_SlashES3_because_it_does_not_refer_to_a_Promise_compatible_constructor_value)) {
                        return;
                    }
                    // Verify there is no local declaration that could collide with the promise constructor.
                    var rootName = promiseConstructorName && ts.getFirstIdentifier(promiseConstructorName);
                    var collidingSymbol = getSymbol(node.locals, rootName.escapedText, 111551 /* Value */);
                    if (collidingSymbol) {
                        error(collidingSymbol.valueDeclaration, ts.Diagnostics.Duplicate_identifier_0_Compiler_uses_declaration_1_to_support_async_functions, ts.idText(rootName), ts.entityNameToString(promiseConstructorName));
                        return;
                    }
                }
                checkAwaitedType(returnType, node, ts.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member);
            }
            /** Check a decorator */
            function checkDecorator(node) {
                var signature = getResolvedSignature(node);
                var returnType = getReturnTypeOfSignature(signature);
                if (returnType.flags & 1 /* Any */) {
                    return;
                }
                var expectedReturnType;
                var headMessage = getDiagnosticHeadMessageForDecoratorResolution(node);
                var errorInfo;
                switch (node.parent.kind) {
                    case 245 /* ClassDeclaration */:
                        var classSymbol = getSymbolOfNode(node.parent);
                        var classConstructorType = getTypeOfSymbol(classSymbol);
                        expectedReturnType = getUnionType([classConstructorType, voidType]);
                        break;
                    case 156 /* Parameter */:
                        expectedReturnType = voidType;
                        errorInfo = ts.chainDiagnosticMessages(
                        /*details*/ undefined, ts.Diagnostics.The_return_type_of_a_parameter_decorator_function_must_be_either_void_or_any);
                        break;
                    case 159 /* PropertyDeclaration */:
                        expectedReturnType = voidType;
                        errorInfo = ts.chainDiagnosticMessages(
                        /*details*/ undefined, ts.Diagnostics.The_return_type_of_a_property_decorator_function_must_be_either_void_or_any);
                        break;
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        var methodType = getTypeOfNode(node.parent);
                        var descriptorType = createTypedPropertyDescriptorType(methodType);
                        expectedReturnType = getUnionType([descriptorType, voidType]);
                        break;
                    default:
                        return ts.Debug.fail();
                }
                checkTypeAssignableTo(returnType, expectedReturnType, node, headMessage, function () { return errorInfo; });
            }
            /**
             * If a TypeNode can be resolved to a value symbol imported from an external module, it is
             * marked as referenced to prevent import elision.
             */
            function markTypeNodeAsReferenced(node) {
                markEntityNameOrEntityExpressionAsReference(node && ts.getEntityNameFromTypeNode(node));
            }
            function markEntityNameOrEntityExpressionAsReference(typeName) {
                if (!typeName)
                    return;
                var rootName = ts.getFirstIdentifier(typeName);
                var meaning = (typeName.kind === 75 /* Identifier */ ? 788968 /* Type */ : 1920 /* Namespace */) | 2097152 /* Alias */;
                var rootSymbol = resolveName(rootName, rootName.escapedText, meaning, /*nameNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isRefernce*/ true);
                if (rootSymbol
                    && rootSymbol.flags & 2097152 /* Alias */
                    && symbolIsValue(rootSymbol)
                    && !isConstEnumOrConstEnumOnlyModule(resolveAlias(rootSymbol))
                    && !getTypeOnlyAliasDeclaration(rootSymbol)) {
                    markAliasSymbolAsReferenced(rootSymbol);
                }
            }
            /**
             * This function marks the type used for metadata decorator as referenced if it is import
             * from external module.
             * This is different from markTypeNodeAsReferenced because it tries to simplify type nodes in
             * union and intersection type
             * @param node
             */
            function markDecoratorMedataDataTypeNodeAsReferenced(node) {
                var entityName = getEntityNameForDecoratorMetadata(node);
                if (entityName && ts.isEntityName(entityName)) {
                    markEntityNameOrEntityExpressionAsReference(entityName);
                }
            }
            function getEntityNameForDecoratorMetadata(node) {
                if (node) {
                    switch (node.kind) {
                        case 179 /* IntersectionType */:
                        case 178 /* UnionType */:
                            return getEntityNameForDecoratorMetadataFromTypeList(node.types);
                        case 180 /* ConditionalType */:
                            return getEntityNameForDecoratorMetadataFromTypeList([node.trueType, node.falseType]);
                        case 182 /* ParenthesizedType */:
                            return getEntityNameForDecoratorMetadata(node.type);
                        case 169 /* TypeReference */:
                            return node.typeName;
                    }
                }
            }
            function getEntityNameForDecoratorMetadataFromTypeList(types) {
                var commonEntityName;
                for (var _i = 0, types_19 = types; _i < types_19.length; _i++) {
                    var typeNode = types_19[_i];
                    while (typeNode.kind === 182 /* ParenthesizedType */) {
                        typeNode = typeNode.type; // Skip parens if need be
                    }
                    if (typeNode.kind === 137 /* NeverKeyword */) {
                        continue; // Always elide `never` from the union/intersection if possible
                    }
                    if (!strictNullChecks && (typeNode.kind === 100 /* NullKeyword */ || typeNode.kind === 146 /* UndefinedKeyword */)) {
                        continue; // Elide null and undefined from unions for metadata, just like what we did prior to the implementation of strict null checks
                    }
                    var individualEntityName = getEntityNameForDecoratorMetadata(typeNode);
                    if (!individualEntityName) {
                        // Individual is something like string number
                        // So it would be serialized to either that type or object
                        // Safe to return here
                        return undefined;
                    }
                    if (commonEntityName) {
                        // Note this is in sync with the transformation that happens for type node.
                        // Keep this in sync with serializeUnionOrIntersectionType
                        // Verify if they refer to same entity and is identifier
                        // return undefined if they dont match because we would emit object
                        if (!ts.isIdentifier(commonEntityName) ||
                            !ts.isIdentifier(individualEntityName) ||
                            commonEntityName.escapedText !== individualEntityName.escapedText) {
                            return undefined;
                        }
                    }
                    else {
                        commonEntityName = individualEntityName;
                    }
                }
                return commonEntityName;
            }
            function getParameterTypeNodeForDecoratorCheck(node) {
                var typeNode = ts.getEffectiveTypeAnnotationNode(node);
                return ts.isRestParameter(node) ? ts.getRestParameterElementType(typeNode) : typeNode;
            }
            /** Check the decorators of a node */
            function checkDecorators(node) {
                if (!node.decorators) {
                    return;
                }
                // skip this check for nodes that cannot have decorators. These should have already had an error reported by
                // checkGrammarDecorators.
                if (!ts.nodeCanBeDecorated(node, node.parent, node.parent.parent)) {
                    return;
                }
                if (!compilerOptions.experimentalDecorators) {
                    error(node, ts.Diagnostics.Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_the_experimentalDecorators_option_in_your_tsconfig_or_jsconfig_to_remove_this_warning);
                }
                var firstDecorator = node.decorators[0];
                checkExternalEmitHelpers(firstDecorator, 8 /* Decorate */);
                if (node.kind === 156 /* Parameter */) {
                    checkExternalEmitHelpers(firstDecorator, 32 /* Param */);
                }
                if (compilerOptions.emitDecoratorMetadata) {
                    checkExternalEmitHelpers(firstDecorator, 16 /* Metadata */);
                    // we only need to perform these checks if we are emitting serialized type metadata for the target of a decorator.
                    switch (node.kind) {
                        case 245 /* ClassDeclaration */:
                            var constructor = ts.getFirstConstructorWithBody(node);
                            if (constructor) {
                                for (var _i = 0, _a = constructor.parameters; _i < _a.length; _i++) {
                                    var parameter = _a[_i];
                                    markDecoratorMedataDataTypeNodeAsReferenced(getParameterTypeNodeForDecoratorCheck(parameter));
                                }
                            }
                            break;
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            var otherKind = node.kind === 163 /* GetAccessor */ ? 164 /* SetAccessor */ : 163 /* GetAccessor */;
                            var otherAccessor = ts.getDeclarationOfKind(getSymbolOfNode(node), otherKind);
                            markDecoratorMedataDataTypeNodeAsReferenced(getAnnotatedAccessorTypeNode(node) || otherAccessor && getAnnotatedAccessorTypeNode(otherAccessor));
                            break;
                        case 161 /* MethodDeclaration */:
                            for (var _b = 0, _c = node.parameters; _b < _c.length; _b++) {
                                var parameter = _c[_b];
                                markDecoratorMedataDataTypeNodeAsReferenced(getParameterTypeNodeForDecoratorCheck(parameter));
                            }
                            markDecoratorMedataDataTypeNodeAsReferenced(ts.getEffectiveReturnTypeNode(node));
                            break;
                        case 159 /* PropertyDeclaration */:
                            markDecoratorMedataDataTypeNodeAsReferenced(ts.getEffectiveTypeAnnotationNode(node));
                            break;
                        case 156 /* Parameter */:
                            markDecoratorMedataDataTypeNodeAsReferenced(getParameterTypeNodeForDecoratorCheck(node));
                            var containingSignature = node.parent;
                            for (var _d = 0, _e = containingSignature.parameters; _d < _e.length; _d++) {
                                var parameter = _e[_d];
                                markDecoratorMedataDataTypeNodeAsReferenced(getParameterTypeNodeForDecoratorCheck(parameter));
                            }
                            break;
                    }
                }
                ts.forEach(node.decorators, checkDecorator);
            }
            function checkFunctionDeclaration(node) {
                if (produceDiagnostics) {
                    checkFunctionOrMethodDeclaration(node);
                    checkGrammarForGenerator(node);
                    checkCollisionWithRequireExportsInGeneratedCode(node, node.name);
                    checkCollisionWithGlobalPromiseInGeneratedCode(node, node.name);
                }
            }
            function checkJSDocTypeAliasTag(node) {
                if (!node.typeExpression) {
                    // If the node had `@property` tags, `typeExpression` would have been set to the first property tag.
                    error(node.name, ts.Diagnostics.JSDoc_typedef_tag_should_either_have_a_type_annotation_or_be_followed_by_property_or_member_tags);
                }
                if (node.name) {
                    checkTypeNameIsReserved(node.name, ts.Diagnostics.Type_alias_name_cannot_be_0);
                }
                checkSourceElement(node.typeExpression);
            }
            function checkJSDocTemplateTag(node) {
                checkSourceElement(node.constraint);
                for (var _i = 0, _a = node.typeParameters; _i < _a.length; _i++) {
                    var tp = _a[_i];
                    checkSourceElement(tp);
                }
            }
            function checkJSDocTypeTag(node) {
                checkSourceElement(node.typeExpression);
            }
            function checkJSDocParameterTag(node) {
                checkSourceElement(node.typeExpression);
                if (!ts.getParameterSymbolFromJSDoc(node)) {
                    var decl = ts.getHostSignatureFromJSDoc(node);
                    // don't issue an error for invalid hosts -- just functions --
                    // and give a better error message when the host function mentions `arguments`
                    // but the tag doesn't have an array type
                    if (decl) {
                        var i = ts.getJSDocTags(decl).filter(ts.isJSDocParameterTag).indexOf(node);
                        if (i > -1 && i < decl.parameters.length && ts.isBindingPattern(decl.parameters[i].name)) {
                            return;
                        }
                        if (!containsArgumentsReference(decl)) {
                            if (ts.isQualifiedName(node.name)) {
                                error(node.name, ts.Diagnostics.Qualified_name_0_is_not_allowed_without_a_leading_param_object_1, ts.entityNameToString(node.name), ts.entityNameToString(node.name.left));
                            }
                            else {
                                error(node.name, ts.Diagnostics.JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name, ts.idText(node.name));
                            }
                        }
                        else if (ts.findLast(ts.getJSDocTags(decl), ts.isJSDocParameterTag) === node &&
                            node.typeExpression && node.typeExpression.type &&
                            !isArrayType(getTypeFromTypeNode(node.typeExpression.type))) {
                            error(node.name, ts.Diagnostics.JSDoc_param_tag_has_name_0_but_there_is_no_parameter_with_that_name_It_would_match_arguments_if_it_had_an_array_type, ts.idText(node.name.kind === 153 /* QualifiedName */ ? node.name.right : node.name));
                        }
                    }
                }
            }
            function checkJSDocFunctionType(node) {
                if (produceDiagnostics && !node.type && !ts.isJSDocConstructSignature(node)) {
                    reportImplicitAny(node, anyType);
                }
                checkSignatureDeclaration(node);
            }
            function checkJSDocAugmentsTag(node) {
                var classLike = ts.getJSDocHost(node);
                if (!ts.isClassDeclaration(classLike) && !ts.isClassExpression(classLike)) {
                    error(classLike, ts.Diagnostics.JSDoc_0_is_not_attached_to_a_class, ts.idText(node.tagName));
                    return;
                }
                var augmentsTags = ts.getJSDocTags(classLike).filter(ts.isJSDocAugmentsTag);
                ts.Debug.assert(augmentsTags.length > 0);
                if (augmentsTags.length > 1) {
                    error(augmentsTags[1], ts.Diagnostics.Class_declarations_cannot_have_more_than_one_augments_or_extends_tag);
                }
                var name = getIdentifierFromEntityNameExpression(node.class.expression);
                var extend = ts.getClassExtendsHeritageElement(classLike);
                if (extend) {
                    var className = getIdentifierFromEntityNameExpression(extend.expression);
                    if (className && name.escapedText !== className.escapedText) {
                        error(name, ts.Diagnostics.JSDoc_0_1_does_not_match_the_extends_2_clause, ts.idText(node.tagName), ts.idText(name), ts.idText(className));
                    }
                }
            }
            function getIdentifierFromEntityNameExpression(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                        return node;
                    case 194 /* PropertyAccessExpression */:
                        return node.name;
                    default:
                        return undefined;
                }
            }
            function checkFunctionOrMethodDeclaration(node) {
                checkDecorators(node);
                checkSignatureDeclaration(node);
                var functionFlags = ts.getFunctionFlags(node);
                // Do not use hasDynamicName here, because that returns false for well known symbols.
                // We want to perform checkComputedPropertyName for all computed properties, including
                // well known symbols.
                if (node.name && node.name.kind === 154 /* ComputedPropertyName */) {
                    // This check will account for methods in class/interface declarations,
                    // as well as accessors in classes/object literals
                    checkComputedPropertyName(node.name);
                }
                if (!hasNonBindableDynamicName(node)) {
                    // first we want to check the local symbol that contain this declaration
                    // - if node.localSymbol !== undefined - this is current declaration is exported and localSymbol points to the local symbol
                    // - if node.localSymbol === undefined - this node is non-exported so we can just pick the result of getSymbolOfNode
                    var symbol = getSymbolOfNode(node);
                    var localSymbol = node.localSymbol || symbol;
                    // Since the javascript won't do semantic analysis like typescript,
                    // if the javascript file comes before the typescript file and both contain same name functions,
                    // checkFunctionOrConstructorSymbol wouldn't be called if we didnt ignore javascript function.
                    var firstDeclaration = ts.find(localSymbol.declarations, 
                    // Get first non javascript function declaration
                    function (declaration) { return declaration.kind === node.kind && !(declaration.flags & 131072 /* JavaScriptFile */); });
                    // Only type check the symbol once
                    if (node === firstDeclaration) {
                        checkFunctionOrConstructorSymbol(localSymbol);
                    }
                    if (symbol.parent) {
                        // run check once for the first declaration
                        if (ts.getDeclarationOfKind(symbol, node.kind) === node) {
                            // run check on export symbol to check that modifiers agree across all exported declarations
                            checkFunctionOrConstructorSymbol(symbol);
                        }
                    }
                }
                var body = node.kind === 160 /* MethodSignature */ ? undefined : node.body;
                checkSourceElement(body);
                checkAllCodePathsInNonVoidFunctionReturnOrThrow(node, getReturnTypeFromAnnotation(node));
                if (produceDiagnostics && !ts.getEffectiveReturnTypeNode(node)) {
                    // Report an implicit any error if there is no body, no explicit return type, and node is not a private method
                    // in an ambient context
                    if (ts.nodeIsMissing(body) && !isPrivateWithinAmbient(node)) {
                        reportImplicitAny(node, anyType);
                    }
                    if (functionFlags & 1 /* Generator */ && ts.nodeIsPresent(body)) {
                        // A generator with a body and no type annotation can still cause errors. It can error if the
                        // yielded values have no common supertype, or it can give an implicit any error if it has no
                        // yielded values. The only way to trigger these errors is to try checking its return type.
                        getReturnTypeOfSignature(getSignatureFromDeclaration(node));
                    }
                }
                // A js function declaration can have a @type tag instead of a return type node, but that type must have a call signature
                if (ts.isInJSFile(node)) {
                    var typeTag = ts.getJSDocTypeTag(node);
                    if (typeTag && typeTag.typeExpression && !getContextualCallSignature(getTypeFromTypeNode(typeTag.typeExpression), node)) {
                        error(typeTag, ts.Diagnostics.The_type_of_a_function_declaration_must_match_the_function_s_signature);
                    }
                }
            }
            function registerForUnusedIdentifiersCheck(node) {
                // May be in a call such as getTypeOfNode that happened to call this. But potentiallyUnusedIdentifiers is only defined in the scope of `checkSourceFile`.
                if (produceDiagnostics) {
                    var sourceFile = ts.getSourceFileOfNode(node);
                    var potentiallyUnusedIdentifiers = allPotentiallyUnusedIdentifiers.get(sourceFile.path);
                    if (!potentiallyUnusedIdentifiers) {
                        potentiallyUnusedIdentifiers = [];
                        allPotentiallyUnusedIdentifiers.set(sourceFile.path, potentiallyUnusedIdentifiers);
                    }
                    // TODO: GH#22580
                    // Debug.assert(addToSeen(seenPotentiallyUnusedIdentifiers, getNodeId(node)), "Adding potentially-unused identifier twice");
                    potentiallyUnusedIdentifiers.push(node);
                }
            }
            function checkUnusedIdentifiers(potentiallyUnusedIdentifiers, addDiagnostic) {
                for (var _i = 0, potentiallyUnusedIdentifiers_1 = potentiallyUnusedIdentifiers; _i < potentiallyUnusedIdentifiers_1.length; _i++) {
                    var node = potentiallyUnusedIdentifiers_1[_i];
                    switch (node.kind) {
                        case 245 /* ClassDeclaration */:
                        case 214 /* ClassExpression */:
                            checkUnusedClassMembers(node, addDiagnostic);
                            checkUnusedTypeParameters(node, addDiagnostic);
                            break;
                        case 290 /* SourceFile */:
                        case 249 /* ModuleDeclaration */:
                        case 223 /* Block */:
                        case 251 /* CaseBlock */:
                        case 230 /* ForStatement */:
                        case 231 /* ForInStatement */:
                        case 232 /* ForOfStatement */:
                            checkUnusedLocalsAndParameters(node, addDiagnostic);
                            break;
                        case 162 /* Constructor */:
                        case 201 /* FunctionExpression */:
                        case 244 /* FunctionDeclaration */:
                        case 202 /* ArrowFunction */:
                        case 161 /* MethodDeclaration */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            if (node.body) { // Don't report unused parameters in overloads
                                checkUnusedLocalsAndParameters(node, addDiagnostic);
                            }
                            checkUnusedTypeParameters(node, addDiagnostic);
                            break;
                        case 160 /* MethodSignature */:
                        case 165 /* CallSignature */:
                        case 166 /* ConstructSignature */:
                        case 170 /* FunctionType */:
                        case 171 /* ConstructorType */:
                        case 247 /* TypeAliasDeclaration */:
                        case 246 /* InterfaceDeclaration */:
                            checkUnusedTypeParameters(node, addDiagnostic);
                            break;
                        case 181 /* InferType */:
                            checkUnusedInferTypeParameter(node, addDiagnostic);
                            break;
                        default:
                            ts.Debug.assertNever(node, "Node should not have been registered for unused identifiers check");
                    }
                }
            }
            function errorUnusedLocal(declaration, name, addDiagnostic) {
                var node = ts.getNameOfDeclaration(declaration) || declaration;
                var message = isTypeDeclaration(declaration) ? ts.Diagnostics._0_is_declared_but_never_used : ts.Diagnostics._0_is_declared_but_its_value_is_never_read;
                addDiagnostic(declaration, 0 /* Local */, ts.createDiagnosticForNode(node, message, name));
            }
            function isIdentifierThatStartsWithUnderscore(node) {
                return ts.isIdentifier(node) && ts.idText(node).charCodeAt(0) === 95 /* _ */;
            }
            function checkUnusedClassMembers(node, addDiagnostic) {
                for (var _i = 0, _a = node.members; _i < _a.length; _i++) {
                    var member = _a[_i];
                    switch (member.kind) {
                        case 161 /* MethodDeclaration */:
                        case 159 /* PropertyDeclaration */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            if (member.kind === 164 /* SetAccessor */ && member.symbol.flags & 32768 /* GetAccessor */) {
                                // Already would have reported an error on the getter.
                                break;
                            }
                            var symbol = getSymbolOfNode(member);
                            if (!symbol.isReferenced && (ts.hasModifier(member, 8 /* Private */) || ts.isNamedDeclaration(member) && ts.isPrivateIdentifier(member.name))) {
                                addDiagnostic(member, 0 /* Local */, ts.createDiagnosticForNode(member.name, ts.Diagnostics._0_is_declared_but_its_value_is_never_read, symbolToString(symbol)));
                            }
                            break;
                        case 162 /* Constructor */:
                            for (var _b = 0, _c = member.parameters; _b < _c.length; _b++) {
                                var parameter = _c[_b];
                                if (!parameter.symbol.isReferenced && ts.hasModifier(parameter, 8 /* Private */)) {
                                    addDiagnostic(parameter, 0 /* Local */, ts.createDiagnosticForNode(parameter.name, ts.Diagnostics.Property_0_is_declared_but_its_value_is_never_read, ts.symbolName(parameter.symbol)));
                                }
                            }
                            break;
                        case 167 /* IndexSignature */:
                        case 222 /* SemicolonClassElement */:
                            // Can't be private
                            break;
                        default:
                            ts.Debug.fail();
                    }
                }
            }
            function checkUnusedInferTypeParameter(node, addDiagnostic) {
                var typeParameter = node.typeParameter;
                if (isTypeParameterUnused(typeParameter)) {
                    addDiagnostic(node, 1 /* Parameter */, ts.createDiagnosticForNode(node, ts.Diagnostics._0_is_declared_but_its_value_is_never_read, ts.idText(typeParameter.name)));
                }
            }
            function checkUnusedTypeParameters(node, addDiagnostic) {
                // Only report errors on the last declaration for the type parameter container;
                // this ensures that all uses have been accounted for.
                if (ts.last(getSymbolOfNode(node).declarations) !== node)
                    return;
                var typeParameters = ts.getEffectiveTypeParameterDeclarations(node);
                var seenParentsWithEveryUnused = new ts.NodeSet();
                for (var _i = 0, typeParameters_3 = typeParameters; _i < typeParameters_3.length; _i++) {
                    var typeParameter = typeParameters_3[_i];
                    if (!isTypeParameterUnused(typeParameter))
                        continue;
                    var name = ts.idText(typeParameter.name);
                    var parent = typeParameter.parent;
                    if (parent.kind !== 181 /* InferType */ && parent.typeParameters.every(isTypeParameterUnused)) {
                        if (seenParentsWithEveryUnused.tryAdd(parent)) {
                            var range = ts.isJSDocTemplateTag(parent)
                                // Whole @template tag
                                ? ts.rangeOfNode(parent)
                                // Include the `<>` in the error message
                                : ts.rangeOfTypeParameters(parent.typeParameters);
                            var only = typeParameters.length === 1;
                            var message = only ? ts.Diagnostics._0_is_declared_but_its_value_is_never_read : ts.Diagnostics.All_type_parameters_are_unused;
                            var arg0 = only ? name : undefined;
                            addDiagnostic(typeParameter, 1 /* Parameter */, ts.createFileDiagnostic(ts.getSourceFileOfNode(parent), range.pos, range.end - range.pos, message, arg0));
                        }
                    }
                    else {
                        addDiagnostic(typeParameter, 1 /* Parameter */, ts.createDiagnosticForNode(typeParameter, ts.Diagnostics._0_is_declared_but_its_value_is_never_read, name));
                    }
                }
            }
            function isTypeParameterUnused(typeParameter) {
                return !(getMergedSymbol(typeParameter.symbol).isReferenced & 262144 /* TypeParameter */) && !isIdentifierThatStartsWithUnderscore(typeParameter.name);
            }
            function addToGroup(map, key, value, getKey) {
                var keyString = String(getKey(key));
                var group = map.get(keyString);
                if (group) {
                    group[1].push(value);
                }
                else {
                    map.set(keyString, [key, [value]]);
                }
            }
            function tryGetRootParameterDeclaration(node) {
                return ts.tryCast(ts.getRootDeclaration(node), ts.isParameter);
            }
            function checkUnusedLocalsAndParameters(nodeWithLocals, addDiagnostic) {
                // Ideally we could use the ImportClause directly as a key, but must wait until we have full ES6 maps. So must store key along with value.
                var unusedImports = ts.createMap();
                var unusedDestructures = ts.createMap();
                var unusedVariables = ts.createMap();
                nodeWithLocals.locals.forEach(function (local) {
                    // If it's purely a type parameter, ignore, will be checked in `checkUnusedTypeParameters`.
                    // If it's a type parameter merged with a parameter, check if the parameter-side is used.
                    if (local.flags & 262144 /* TypeParameter */ ? !(local.flags & 3 /* Variable */ && !(local.isReferenced & 3 /* Variable */)) : local.isReferenced || local.exportSymbol) {
                        return;
                    }
                    for (var _i = 0, _a = local.declarations; _i < _a.length; _i++) {
                        var declaration = _a[_i];
                        if (ts.isAmbientModule(declaration) ||
                            (ts.isVariableDeclaration(declaration) && ts.isForInOrOfStatement(declaration.parent.parent) || isImportedDeclaration(declaration)) && isIdentifierThatStartsWithUnderscore(declaration.name)) {
                            continue;
                        }
                        if (isImportedDeclaration(declaration)) {
                            addToGroup(unusedImports, importClauseFromImported(declaration), declaration, getNodeId);
                        }
                        else if (ts.isBindingElement(declaration) && ts.isObjectBindingPattern(declaration.parent)) {
                            // In `{ a, ...b }, `a` is considered used since it removes a property from `b`. `b` may still be unused though.
                            var lastElement = ts.last(declaration.parent.elements);
                            if (declaration === lastElement || !ts.last(declaration.parent.elements).dotDotDotToken) {
                                addToGroup(unusedDestructures, declaration.parent, declaration, getNodeId);
                            }
                        }
                        else if (ts.isVariableDeclaration(declaration)) {
                            addToGroup(unusedVariables, declaration.parent, declaration, getNodeId);
                        }
                        else {
                            var parameter = local.valueDeclaration && tryGetRootParameterDeclaration(local.valueDeclaration);
                            var name = local.valueDeclaration && ts.getNameOfDeclaration(local.valueDeclaration);
                            if (parameter && name) {
                                if (!ts.isParameterPropertyDeclaration(parameter, parameter.parent) && !ts.parameterIsThisKeyword(parameter) && !isIdentifierThatStartsWithUnderscore(name)) {
                                    addDiagnostic(parameter, 1 /* Parameter */, ts.createDiagnosticForNode(name, ts.Diagnostics._0_is_declared_but_its_value_is_never_read, ts.symbolName(local)));
                                }
                            }
                            else {
                                errorUnusedLocal(declaration, ts.symbolName(local), addDiagnostic);
                            }
                        }
                    }
                });
                unusedImports.forEach(function (_a) {
                    var importClause = _a[0], unuseds = _a[1];
                    var importDecl = importClause.parent;
                    var nDeclarations = (importClause.name ? 1 : 0) +
                        (importClause.namedBindings ?
                            (importClause.namedBindings.kind === 256 /* NamespaceImport */ ? 1 : importClause.namedBindings.elements.length)
                            : 0);
                    if (nDeclarations === unuseds.length) {
                        addDiagnostic(importDecl, 0 /* Local */, unuseds.length === 1
                            ? ts.createDiagnosticForNode(importDecl, ts.Diagnostics._0_is_declared_but_its_value_is_never_read, ts.idText(ts.first(unuseds).name))
                            : ts.createDiagnosticForNode(importDecl, ts.Diagnostics.All_imports_in_import_declaration_are_unused));
                    }
                    else {
                        for (var _i = 0, unuseds_1 = unuseds; _i < unuseds_1.length; _i++) {
                            var unused = unuseds_1[_i];
                            errorUnusedLocal(unused, ts.idText(unused.name), addDiagnostic);
                        }
                    }
                });
                unusedDestructures.forEach(function (_a) {
                    var bindingPattern = _a[0], bindingElements = _a[1];
                    var kind = tryGetRootParameterDeclaration(bindingPattern.parent) ? 1 /* Parameter */ : 0 /* Local */;
                    if (bindingPattern.elements.length === bindingElements.length) {
                        if (bindingElements.length === 1 && bindingPattern.parent.kind === 242 /* VariableDeclaration */ && bindingPattern.parent.parent.kind === 243 /* VariableDeclarationList */) {
                            addToGroup(unusedVariables, bindingPattern.parent.parent, bindingPattern.parent, getNodeId);
                        }
                        else {
                            addDiagnostic(bindingPattern, kind, bindingElements.length === 1
                                ? ts.createDiagnosticForNode(bindingPattern, ts.Diagnostics._0_is_declared_but_its_value_is_never_read, bindingNameText(ts.first(bindingElements).name))
                                : ts.createDiagnosticForNode(bindingPattern, ts.Diagnostics.All_destructured_elements_are_unused));
                        }
                    }
                    else {
                        for (var _i = 0, bindingElements_1 = bindingElements; _i < bindingElements_1.length; _i++) {
                            var e = bindingElements_1[_i];
                            addDiagnostic(e, kind, ts.createDiagnosticForNode(e, ts.Diagnostics._0_is_declared_but_its_value_is_never_read, bindingNameText(e.name)));
                        }
                    }
                });
                unusedVariables.forEach(function (_a) {
                    var declarationList = _a[0], declarations = _a[1];
                    if (declarationList.declarations.length === declarations.length) {
                        addDiagnostic(declarationList, 0 /* Local */, declarations.length === 1
                            ? ts.createDiagnosticForNode(ts.first(declarations).name, ts.Diagnostics._0_is_declared_but_its_value_is_never_read, bindingNameText(ts.first(declarations).name))
                            : ts.createDiagnosticForNode(declarationList.parent.kind === 225 /* VariableStatement */ ? declarationList.parent : declarationList, ts.Diagnostics.All_variables_are_unused));
                    }
                    else {
                        for (var _i = 0, declarations_5 = declarations; _i < declarations_5.length; _i++) {
                            var decl = declarations_5[_i];
                            addDiagnostic(decl, 0 /* Local */, ts.createDiagnosticForNode(decl, ts.Diagnostics._0_is_declared_but_its_value_is_never_read, bindingNameText(decl.name)));
                        }
                    }
                });
            }
            function bindingNameText(name) {
                switch (name.kind) {
                    case 75 /* Identifier */:
                        return ts.idText(name);
                    case 190 /* ArrayBindingPattern */:
                    case 189 /* ObjectBindingPattern */:
                        return bindingNameText(ts.cast(ts.first(name.elements), ts.isBindingElement).name);
                    default:
                        return ts.Debug.assertNever(name);
                }
            }
            function isImportedDeclaration(node) {
                return node.kind === 255 /* ImportClause */ || node.kind === 258 /* ImportSpecifier */ || node.kind === 256 /* NamespaceImport */;
            }
            function importClauseFromImported(decl) {
                return decl.kind === 255 /* ImportClause */ ? decl : decl.kind === 256 /* NamespaceImport */ ? decl.parent : decl.parent.parent;
            }
            function checkBlock(node) {
                // Grammar checking for SyntaxKind.Block
                if (node.kind === 223 /* Block */) {
                    checkGrammarStatementInAmbientContext(node);
                }
                if (ts.isFunctionOrModuleBlock(node)) {
                    var saveFlowAnalysisDisabled = flowAnalysisDisabled;
                    ts.forEach(node.statements, checkSourceElement);
                    flowAnalysisDisabled = saveFlowAnalysisDisabled;
                }
                else {
                    ts.forEach(node.statements, checkSourceElement);
                }
                if (node.locals) {
                    registerForUnusedIdentifiersCheck(node);
                }
            }
            function checkCollisionWithArgumentsInGeneratedCode(node) {
                // no rest parameters \ declaration context \ overload - no codegen impact
                if (languageVersion >= 2 /* ES2015 */ || compilerOptions.noEmit || !ts.hasRestParameter(node) || node.flags & 8388608 /* Ambient */ || ts.nodeIsMissing(node.body)) {
                    return;
                }
                ts.forEach(node.parameters, function (p) {
                    if (p.name && !ts.isBindingPattern(p.name) && p.name.escapedText === argumentsSymbol.escapedName) {
                        error(p, ts.Diagnostics.Duplicate_identifier_arguments_Compiler_uses_arguments_to_initialize_rest_parameters);
                    }
                });
            }
            function needCollisionCheckForIdentifier(node, identifier, name) {
                if (!(identifier && identifier.escapedText === name)) {
                    return false;
                }
                if (node.kind === 159 /* PropertyDeclaration */ ||
                    node.kind === 158 /* PropertySignature */ ||
                    node.kind === 161 /* MethodDeclaration */ ||
                    node.kind === 160 /* MethodSignature */ ||
                    node.kind === 163 /* GetAccessor */ ||
                    node.kind === 164 /* SetAccessor */) {
                    // it is ok to have member named '_super' or '_this' - member access is always qualified
                    return false;
                }
                if (node.flags & 8388608 /* Ambient */) {
                    // ambient context - no codegen impact
                    return false;
                }
                var root = ts.getRootDeclaration(node);
                if (root.kind === 156 /* Parameter */ && ts.nodeIsMissing(root.parent.body)) {
                    // just an overload - no codegen impact
                    return false;
                }
                return true;
            }
            // this function will run after checking the source file so 'CaptureThis' is correct for all nodes
            function checkIfThisIsCapturedInEnclosingScope(node) {
                ts.findAncestor(node, function (current) {
                    if (getNodeCheckFlags(current) & 4 /* CaptureThis */) {
                        var isDeclaration_1 = node.kind !== 75 /* Identifier */;
                        if (isDeclaration_1) {
                            error(ts.getNameOfDeclaration(node), ts.Diagnostics.Duplicate_identifier_this_Compiler_uses_variable_declaration_this_to_capture_this_reference);
                        }
                        else {
                            error(node, ts.Diagnostics.Expression_resolves_to_variable_declaration_this_that_compiler_uses_to_capture_this_reference);
                        }
                        return true;
                    }
                    return false;
                });
            }
            function checkIfNewTargetIsCapturedInEnclosingScope(node) {
                ts.findAncestor(node, function (current) {
                    if (getNodeCheckFlags(current) & 8 /* CaptureNewTarget */) {
                        var isDeclaration_2 = node.kind !== 75 /* Identifier */;
                        if (isDeclaration_2) {
                            error(ts.getNameOfDeclaration(node), ts.Diagnostics.Duplicate_identifier_newTarget_Compiler_uses_variable_declaration_newTarget_to_capture_new_target_meta_property_reference);
                        }
                        else {
                            error(node, ts.Diagnostics.Expression_resolves_to_variable_declaration_newTarget_that_compiler_uses_to_capture_new_target_meta_property_reference);
                        }
                        return true;
                    }
                    return false;
                });
            }
            function checkWeakMapCollision(node) {
                var enclosingBlockScope = ts.getEnclosingBlockScopeContainer(node);
                if (getNodeCheckFlags(enclosingBlockScope) & 67108864 /* ContainsClassWithPrivateIdentifiers */) {
                    error(node, ts.Diagnostics.Compiler_reserves_name_0_when_emitting_private_identifier_downlevel, "WeakMap");
                }
            }
            function checkCollisionWithRequireExportsInGeneratedCode(node, name) {
                // No need to check for require or exports for ES6 modules and later
                if (moduleKind >= ts.ModuleKind.ES2015 || compilerOptions.noEmit) {
                    return;
                }
                if (!needCollisionCheckForIdentifier(node, name, "require") && !needCollisionCheckForIdentifier(node, name, "exports")) {
                    return;
                }
                // Uninstantiated modules shouldnt do this check
                if (ts.isModuleDeclaration(node) && ts.getModuleInstanceState(node) !== 1 /* Instantiated */) {
                    return;
                }
                // In case of variable declaration, node.parent is variable statement so look at the variable statement's parent
                var parent = getDeclarationContainer(node);
                if (parent.kind === 290 /* SourceFile */ && ts.isExternalOrCommonJsModule(parent)) {
                    // If the declaration happens to be in external module, report error that require and exports are reserved keywords
                    error(name, ts.Diagnostics.Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module, ts.declarationNameToString(name), ts.declarationNameToString(name));
                }
            }
            function checkCollisionWithGlobalPromiseInGeneratedCode(node, name) {
                if (languageVersion >= 4 /* ES2017 */ || compilerOptions.noEmit || !needCollisionCheckForIdentifier(node, name, "Promise")) {
                    return;
                }
                // Uninstantiated modules shouldnt do this check
                if (ts.isModuleDeclaration(node) && ts.getModuleInstanceState(node) !== 1 /* Instantiated */) {
                    return;
                }
                // In case of variable declaration, node.parent is variable statement so look at the variable statement's parent
                var parent = getDeclarationContainer(node);
                if (parent.kind === 290 /* SourceFile */ && ts.isExternalOrCommonJsModule(parent) && parent.flags & 2048 /* HasAsyncFunctions */) {
                    // If the declaration happens to be in external module, report error that Promise is a reserved identifier.
                    error(name, ts.Diagnostics.Duplicate_identifier_0_Compiler_reserves_name_1_in_top_level_scope_of_a_module_containing_async_functions, ts.declarationNameToString(name), ts.declarationNameToString(name));
                }
            }
            function checkVarDeclaredNamesNotShadowed(node) {
                // - ScriptBody : StatementList
                // It is a Syntax Error if any element of the LexicallyDeclaredNames of StatementList
                // also occurs in the VarDeclaredNames of StatementList.
                // - Block : { StatementList }
                // It is a Syntax Error if any element of the LexicallyDeclaredNames of StatementList
                // also occurs in the VarDeclaredNames of StatementList.
                // Variable declarations are hoisted to the top of their function scope. They can shadow
                // block scoped declarations, which bind tighter. this will not be flagged as duplicate definition
                // by the binder as the declaration scope is different.
                // A non-initialized declaration is a no-op as the block declaration will resolve before the var
                // declaration. the problem is if the declaration has an initializer. this will act as a write to the
                // block declared value. this is fine for let, but not const.
                // Only consider declarations with initializers, uninitialized const declarations will not
                // step on a let/const variable.
                // Do not consider const and const declarations, as duplicate block-scoped declarations
                // are handled by the binder.
                // We are only looking for const declarations that step on let\const declarations from a
                // different scope. e.g.:
                //      {
                //          const x = 0; // localDeclarationSymbol obtained after name resolution will correspond to this declaration
                //          const x = 0; // symbol for this declaration will be 'symbol'
                //      }
                // skip block-scoped variables and parameters
                if ((ts.getCombinedNodeFlags(node) & 3 /* BlockScoped */) !== 0 || ts.isParameterDeclaration(node)) {
                    return;
                }
                // skip variable declarations that don't have initializers
                // NOTE: in ES6 spec initializer is required in variable declarations where name is binding pattern
                // so we'll always treat binding elements as initialized
                if (node.kind === 242 /* VariableDeclaration */ && !node.initializer) {
                    return;
                }
                var symbol = getSymbolOfNode(node);
                if (symbol.flags & 1 /* FunctionScopedVariable */) {
                    if (!ts.isIdentifier(node.name))
                        return ts.Debug.fail();
                    var localDeclarationSymbol = resolveName(node, node.name.escapedText, 3 /* Variable */, /*nodeNotFoundErrorMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ false);
                    if (localDeclarationSymbol &&
                        localDeclarationSymbol !== symbol &&
                        localDeclarationSymbol.flags & 2 /* BlockScopedVariable */) {
                        if (getDeclarationNodeFlagsFromSymbol(localDeclarationSymbol) & 3 /* BlockScoped */) {
                            var varDeclList = ts.getAncestor(localDeclarationSymbol.valueDeclaration, 243 /* VariableDeclarationList */);
                            var container = varDeclList.parent.kind === 225 /* VariableStatement */ && varDeclList.parent.parent
                                ? varDeclList.parent.parent
                                : undefined;
                            // names of block-scoped and function scoped variables can collide only
                            // if block scoped variable is defined in the function\module\source file scope (because of variable hoisting)
                            var namesShareScope = container &&
                                (container.kind === 223 /* Block */ && ts.isFunctionLike(container.parent) ||
                                    container.kind === 250 /* ModuleBlock */ ||
                                    container.kind === 249 /* ModuleDeclaration */ ||
                                    container.kind === 290 /* SourceFile */);
                            // here we know that function scoped variable is shadowed by block scoped one
                            // if they are defined in the same scope - binder has already reported redeclaration error
                            // otherwise if variable has an initializer - show error that initialization will fail
                            // since LHS will be block scoped name instead of function scoped
                            if (!namesShareScope) {
                                var name = symbolToString(localDeclarationSymbol);
                                error(node, ts.Diagnostics.Cannot_initialize_outer_scoped_variable_0_in_the_same_scope_as_block_scoped_declaration_1, name, name);
                            }
                        }
                    }
                }
            }
            function convertAutoToAny(type) {
                return type === autoType ? anyType : type === autoArrayType ? anyArrayType : type;
            }
            // Check variable, parameter, or property declaration
            function checkVariableLikeDeclaration(node) {
                checkDecorators(node);
                if (!ts.isBindingElement(node)) {
                    checkSourceElement(node.type);
                }
                // JSDoc `function(string, string): string` syntax results in parameters with no name
                if (!node.name) {
                    return;
                }
                // For a computed property, just check the initializer and exit
                // Do not use hasDynamicName here, because that returns false for well known symbols.
                // We want to perform checkComputedPropertyName for all computed properties, including
                // well known symbols.
                if (node.name.kind === 154 /* ComputedPropertyName */) {
                    checkComputedPropertyName(node.name);
                    if (node.initializer) {
                        checkExpressionCached(node.initializer);
                    }
                }
                if (node.kind === 191 /* BindingElement */) {
                    if (node.parent.kind === 189 /* ObjectBindingPattern */ && languageVersion < 99 /* ESNext */) {
                        checkExternalEmitHelpers(node, 4 /* Rest */);
                    }
                    // check computed properties inside property names of binding elements
                    if (node.propertyName && node.propertyName.kind === 154 /* ComputedPropertyName */) {
                        checkComputedPropertyName(node.propertyName);
                    }
                    // check private/protected variable access
                    var parent = node.parent.parent;
                    var parentType = getTypeForBindingElementParent(parent);
                    var name = node.propertyName || node.name;
                    if (parentType && !ts.isBindingPattern(name)) {
                        var exprType = getLiteralTypeFromPropertyName(name);
                        if (isTypeUsableAsPropertyName(exprType)) {
                            var nameText = getPropertyNameFromType(exprType);
                            var property = getPropertyOfType(parentType, nameText);
                            if (property) {
                                markPropertyAsReferenced(property, /*nodeForCheckWriteOnly*/ undefined, /*isThisAccess*/ false); // A destructuring is never a write-only reference.
                                checkPropertyAccessibility(parent, !!parent.initializer && parent.initializer.kind === 102 /* SuperKeyword */, parentType, property);
                            }
                        }
                    }
                }
                // For a binding pattern, check contained binding elements
                if (ts.isBindingPattern(node.name)) {
                    if (node.name.kind === 190 /* ArrayBindingPattern */ && languageVersion < 2 /* ES2015 */ && compilerOptions.downlevelIteration) {
                        checkExternalEmitHelpers(node, 512 /* Read */);
                    }
                    ts.forEach(node.name.elements, checkSourceElement);
                }
                // For a parameter declaration with an initializer, error and exit if the containing function doesn't have a body
                if (node.initializer && ts.getRootDeclaration(node).kind === 156 /* Parameter */ && ts.nodeIsMissing(ts.getContainingFunction(node).body)) {
                    error(node, ts.Diagnostics.A_parameter_initializer_is_only_allowed_in_a_function_or_constructor_implementation);
                    return;
                }
                // For a binding pattern, validate the initializer and exit
                if (ts.isBindingPattern(node.name)) {
                    var needCheckInitializer = node.initializer && node.parent.parent.kind !== 231 /* ForInStatement */;
                    var needCheckWidenedType = node.name.elements.length === 0;
                    if (needCheckInitializer || needCheckWidenedType) {
                        // Don't validate for-in initializer as it is already an error
                        var widenedType = getWidenedTypeForVariableLikeDeclaration(node);
                        if (needCheckInitializer) {
                            var initializerType = checkExpressionCached(node.initializer);
                            if (strictNullChecks && needCheckWidenedType) {
                                checkNonNullNonVoidType(initializerType, node);
                            }
                            else {
                                checkTypeAssignableToAndOptionallyElaborate(initializerType, getWidenedTypeForVariableLikeDeclaration(node), node, node.initializer);
                            }
                        }
                        // check the binding pattern with empty elements
                        if (needCheckWidenedType) {
                            if (ts.isArrayBindingPattern(node.name)) {
                                checkIteratedTypeOrElementType(65 /* Destructuring */, widenedType, undefinedType, node);
                            }
                            else if (strictNullChecks) {
                                checkNonNullNonVoidType(widenedType, node);
                            }
                        }
                    }
                    return;
                }
                var symbol = getSymbolOfNode(node);
                var type = convertAutoToAny(getTypeOfSymbol(symbol));
                if (node === symbol.valueDeclaration) {
                    // Node is the primary declaration of the symbol, just validate the initializer
                    // Don't validate for-in initializer as it is already an error
                    var initializer = ts.getEffectiveInitializer(node);
                    if (initializer) {
                        var isJSObjectLiteralInitializer = ts.isInJSFile(node) &&
                            ts.isObjectLiteralExpression(initializer) &&
                            (initializer.properties.length === 0 || ts.isPrototypeAccess(node.name)) &&
                            ts.hasEntries(symbol.exports);
                        if (!isJSObjectLiteralInitializer && node.parent.parent.kind !== 231 /* ForInStatement */) {
                            checkTypeAssignableToAndOptionallyElaborate(checkExpressionCached(initializer), type, node, initializer, /*headMessage*/ undefined);
                        }
                    }
                    if (symbol.declarations.length > 1) {
                        if (ts.some(symbol.declarations, function (d) { return d !== node && ts.isVariableLike(d) && !areDeclarationFlagsIdentical(d, node); })) {
                            error(node.name, ts.Diagnostics.All_declarations_of_0_must_have_identical_modifiers, ts.declarationNameToString(node.name));
                        }
                    }
                }
                else {
                    // Node is a secondary declaration, check that type is identical to primary declaration and check that
                    // initializer is consistent with type associated with the node
                    var declarationType = convertAutoToAny(getWidenedTypeForVariableLikeDeclaration(node));
                    if (type !== errorType && declarationType !== errorType &&
                        !isTypeIdenticalTo(type, declarationType) &&
                        !(symbol.flags & 67108864 /* Assignment */)) {
                        errorNextVariableOrPropertyDeclarationMustHaveSameType(symbol.valueDeclaration, type, node, declarationType);
                    }
                    if (node.initializer) {
                        checkTypeAssignableToAndOptionallyElaborate(checkExpressionCached(node.initializer), declarationType, node, node.initializer, /*headMessage*/ undefined);
                    }
                    if (!areDeclarationFlagsIdentical(node, symbol.valueDeclaration)) {
                        error(node.name, ts.Diagnostics.All_declarations_of_0_must_have_identical_modifiers, ts.declarationNameToString(node.name));
                    }
                }
                if (node.kind !== 159 /* PropertyDeclaration */ && node.kind !== 158 /* PropertySignature */) {
                    // We know we don't have a binding pattern or computed name here
                    checkExportsOnMergedDeclarations(node);
                    if (node.kind === 242 /* VariableDeclaration */ || node.kind === 191 /* BindingElement */) {
                        checkVarDeclaredNamesNotShadowed(node);
                    }
                    checkCollisionWithRequireExportsInGeneratedCode(node, node.name);
                    checkCollisionWithGlobalPromiseInGeneratedCode(node, node.name);
                    if (!compilerOptions.noEmit && languageVersion < 99 /* ESNext */ && needCollisionCheckForIdentifier(node, node.name, "WeakMap")) {
                        potentialWeakMapCollisions.push(node);
                    }
                }
            }
            function errorNextVariableOrPropertyDeclarationMustHaveSameType(firstDeclaration, firstType, nextDeclaration, nextType) {
                var nextDeclarationName = ts.getNameOfDeclaration(nextDeclaration);
                var message = nextDeclaration.kind === 159 /* PropertyDeclaration */ || nextDeclaration.kind === 158 /* PropertySignature */
                    ? ts.Diagnostics.Subsequent_property_declarations_must_have_the_same_type_Property_0_must_be_of_type_1_but_here_has_type_2
                    : ts.Diagnostics.Subsequent_variable_declarations_must_have_the_same_type_Variable_0_must_be_of_type_1_but_here_has_type_2;
                var declName = ts.declarationNameToString(nextDeclarationName);
                var err = error(nextDeclarationName, message, declName, typeToString(firstType), typeToString(nextType));
                if (firstDeclaration) {
                    ts.addRelatedInfo(err, ts.createDiagnosticForNode(firstDeclaration, ts.Diagnostics._0_was_also_declared_here, declName));
                }
            }
            function areDeclarationFlagsIdentical(left, right) {
                if ((left.kind === 156 /* Parameter */ && right.kind === 242 /* VariableDeclaration */) ||
                    (left.kind === 242 /* VariableDeclaration */ && right.kind === 156 /* Parameter */)) {
                    // Differences in optionality between parameters and variables are allowed.
                    return true;
                }
                if (ts.hasQuestionToken(left) !== ts.hasQuestionToken(right)) {
                    return false;
                }
                var interestingFlags = 8 /* Private */ |
                    16 /* Protected */ |
                    256 /* Async */ |
                    128 /* Abstract */ |
                    64 /* Readonly */ |
                    32 /* Static */;
                return ts.getSelectedModifierFlags(left, interestingFlags) === ts.getSelectedModifierFlags(right, interestingFlags);
            }
            function checkVariableDeclaration(node) {
                checkGrammarVariableDeclaration(node);
                return checkVariableLikeDeclaration(node);
            }
            function checkBindingElement(node) {
                checkGrammarBindingElement(node);
                return checkVariableLikeDeclaration(node);
            }
            function checkVariableStatement(node) {
                // Grammar checking
                if (!checkGrammarDecoratorsAndModifiers(node) && !checkGrammarVariableDeclarationList(node.declarationList))
                    checkGrammarForDisallowedLetOrConstStatement(node);
                ts.forEach(node.declarationList.declarations, checkSourceElement);
            }
            function checkExpressionStatement(node) {
                // Grammar checking
                checkGrammarStatementInAmbientContext(node);
                checkExpression(node.expression);
            }
            function checkIfStatement(node) {
                // Grammar checking
                checkGrammarStatementInAmbientContext(node);
                var type = checkTruthinessExpression(node.expression);
                checkTestingKnownTruthyCallableType(node, type);
                checkSourceElement(node.thenStatement);
                if (node.thenStatement.kind === 224 /* EmptyStatement */) {
                    error(node.thenStatement, ts.Diagnostics.The_body_of_an_if_statement_cannot_be_the_empty_statement);
                }
                checkSourceElement(node.elseStatement);
            }
            function checkTestingKnownTruthyCallableType(ifStatement, type) {
                if (!strictNullChecks) {
                    return;
                }
                var testedNode = ts.isIdentifier(ifStatement.expression)
                    ? ifStatement.expression
                    : ts.isPropertyAccessExpression(ifStatement.expression)
                        ? ifStatement.expression.name
                        : undefined;
                if (!testedNode) {
                    return;
                }
                var possiblyFalsy = getFalsyFlags(type);
                if (possiblyFalsy) {
                    return;
                }
                // While it technically should be invalid for any known-truthy value
                // to be tested, we de-scope to functions unrefenced in the block as a
                // heuristic to identify the most common bugs. There are too many
                // false positives for values sourced from type definitions without
                // strictNullChecks otherwise.
                var callSignatures = getSignaturesOfType(type, 0 /* Call */);
                if (callSignatures.length === 0) {
                    return;
                }
                var testedFunctionSymbol = getSymbolAtLocation(testedNode);
                if (!testedFunctionSymbol) {
                    return;
                }
                var functionIsUsedInBody = ts.forEachChild(ifStatement.thenStatement, function check(childNode) {
                    if (ts.isIdentifier(childNode)) {
                        var childSymbol = getSymbolAtLocation(childNode);
                        if (childSymbol && childSymbol.id === testedFunctionSymbol.id) {
                            return true;
                        }
                    }
                    return ts.forEachChild(childNode, check);
                });
                if (!functionIsUsedInBody) {
                    error(ifStatement.expression, ts.Diagnostics.This_condition_will_always_return_true_since_the_function_is_always_defined_Did_you_mean_to_call_it_instead);
                }
            }
            function checkDoStatement(node) {
                // Grammar checking
                checkGrammarStatementInAmbientContext(node);
                checkSourceElement(node.statement);
                checkTruthinessExpression(node.expression);
            }
            function checkWhileStatement(node) {
                // Grammar checking
                checkGrammarStatementInAmbientContext(node);
                checkTruthinessExpression(node.expression);
                checkSourceElement(node.statement);
            }
            function checkTruthinessOfType(type, node) {
                if (type.flags & 16384 /* Void */) {
                    error(node, ts.Diagnostics.An_expression_of_type_void_cannot_be_tested_for_truthiness);
                }
                return type;
            }
            function checkTruthinessExpression(node, checkMode) {
                return checkTruthinessOfType(checkExpression(node, checkMode), node);
            }
            function checkForStatement(node) {
                // Grammar checking
                if (!checkGrammarStatementInAmbientContext(node)) {
                    if (node.initializer && node.initializer.kind === 243 /* VariableDeclarationList */) {
                        checkGrammarVariableDeclarationList(node.initializer);
                    }
                }
                if (node.initializer) {
                    if (node.initializer.kind === 243 /* VariableDeclarationList */) {
                        ts.forEach(node.initializer.declarations, checkVariableDeclaration);
                    }
                    else {
                        checkExpression(node.initializer);
                    }
                }
                if (node.condition)
                    checkTruthinessExpression(node.condition);
                if (node.incrementor)
                    checkExpression(node.incrementor);
                checkSourceElement(node.statement);
                if (node.locals) {
                    registerForUnusedIdentifiersCheck(node);
                }
            }
            function checkForOfStatement(node) {
                checkGrammarForInOrForOfStatement(node);
                if (node.awaitModifier) {
                    var functionFlags = ts.getFunctionFlags(ts.getContainingFunction(node));
                    if ((functionFlags & (4 /* Invalid */ | 2 /* Async */)) === 2 /* Async */ && languageVersion < 99 /* ESNext */) {
                        // for..await..of in an async function or async generator function prior to ESNext requires the __asyncValues helper
                        checkExternalEmitHelpers(node, 32768 /* ForAwaitOfIncludes */);
                    }
                }
                else if (compilerOptions.downlevelIteration && languageVersion < 2 /* ES2015 */) {
                    // for..of prior to ES2015 requires the __values helper when downlevelIteration is enabled
                    checkExternalEmitHelpers(node, 256 /* ForOfIncludes */);
                }
                // Check the LHS and RHS
                // If the LHS is a declaration, just check it as a variable declaration, which will in turn check the RHS
                // via checkRightHandSideOfForOf.
                // If the LHS is an expression, check the LHS, as a destructuring assignment or as a reference.
                // Then check that the RHS is assignable to it.
                if (node.initializer.kind === 243 /* VariableDeclarationList */) {
                    checkForInOrForOfVariableDeclaration(node);
                }
                else {
                    var varExpr = node.initializer;
                    var iteratedType = checkRightHandSideOfForOf(node.expression, node.awaitModifier);
                    // There may be a destructuring assignment on the left side
                    if (varExpr.kind === 192 /* ArrayLiteralExpression */ || varExpr.kind === 193 /* ObjectLiteralExpression */) {
                        // iteratedType may be undefined. In this case, we still want to check the structure of
                        // varExpr, in particular making sure it's a valid LeftHandSideExpression. But we'd like
                        // to short circuit the type relation checking as much as possible, so we pass the unknownType.
                        checkDestructuringAssignment(varExpr, iteratedType || errorType);
                    }
                    else {
                        var leftType = checkExpression(varExpr);
                        checkReferenceExpression(varExpr, ts.Diagnostics.The_left_hand_side_of_a_for_of_statement_must_be_a_variable_or_a_property_access, ts.Diagnostics.The_left_hand_side_of_a_for_of_statement_may_not_be_an_optional_property_access);
                        // iteratedType will be undefined if the rightType was missing properties/signatures
                        // required to get its iteratedType (like [Symbol.iterator] or next). This may be
                        // because we accessed properties from anyType, or it may have led to an error inside
                        // getElementTypeOfIterable.
                        if (iteratedType) {
                            checkTypeAssignableToAndOptionallyElaborate(iteratedType, leftType, varExpr, node.expression);
                        }
                    }
                }
                checkSourceElement(node.statement);
                if (node.locals) {
                    registerForUnusedIdentifiersCheck(node);
                }
            }
            function checkForInStatement(node) {
                // Grammar checking
                checkGrammarForInOrForOfStatement(node);
                var rightType = getNonNullableTypeIfNeeded(checkExpression(node.expression));
                // TypeScript 1.0 spec (April 2014): 5.4
                // In a 'for-in' statement of the form
                // for (let VarDecl in Expr) Statement
                //   VarDecl must be a variable declaration without a type annotation that declares a variable of type Any,
                //   and Expr must be an expression of type Any, an object type, or a type parameter type.
                if (node.initializer.kind === 243 /* VariableDeclarationList */) {
                    var variable = node.initializer.declarations[0];
                    if (variable && ts.isBindingPattern(variable.name)) {
                        error(variable.name, ts.Diagnostics.The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern);
                    }
                    checkForInOrForOfVariableDeclaration(node);
                }
                else {
                    // In a 'for-in' statement of the form
                    // for (Var in Expr) Statement
                    //   Var must be an expression classified as a reference of type Any or the String primitive type,
                    //   and Expr must be an expression of type Any, an object type, or a type parameter type.
                    var varExpr = node.initializer;
                    var leftType = checkExpression(varExpr);
                    if (varExpr.kind === 192 /* ArrayLiteralExpression */ || varExpr.kind === 193 /* ObjectLiteralExpression */) {
                        error(varExpr, ts.Diagnostics.The_left_hand_side_of_a_for_in_statement_cannot_be_a_destructuring_pattern);
                    }
                    else if (!isTypeAssignableTo(getIndexTypeOrString(rightType), leftType)) {
                        error(varExpr, ts.Diagnostics.The_left_hand_side_of_a_for_in_statement_must_be_of_type_string_or_any);
                    }
                    else {
                        // run check only former check succeeded to avoid cascading errors
                        checkReferenceExpression(varExpr, ts.Diagnostics.The_left_hand_side_of_a_for_in_statement_must_be_a_variable_or_a_property_access, ts.Diagnostics.The_left_hand_side_of_a_for_in_statement_may_not_be_an_optional_property_access);
                    }
                }
                // unknownType is returned i.e. if node.expression is identifier whose name cannot be resolved
                // in this case error about missing name is already reported - do not report extra one
                if (rightType === neverType || !isTypeAssignableToKind(rightType, 67108864 /* NonPrimitive */ | 58982400 /* InstantiableNonPrimitive */)) {
                    error(node.expression, ts.Diagnostics.The_right_hand_side_of_a_for_in_statement_must_be_of_type_any_an_object_type_or_a_type_parameter_but_here_has_type_0, typeToString(rightType));
                }
                checkSourceElement(node.statement);
                if (node.locals) {
                    registerForUnusedIdentifiersCheck(node);
                }
            }
            function checkForInOrForOfVariableDeclaration(iterationStatement) {
                var variableDeclarationList = iterationStatement.initializer;
                // checkGrammarForInOrForOfStatement will check that there is exactly one declaration.
                if (variableDeclarationList.declarations.length >= 1) {
                    var decl = variableDeclarationList.declarations[0];
                    checkVariableDeclaration(decl);
                }
            }
            function checkRightHandSideOfForOf(rhsExpression, awaitModifier) {
                var expressionType = checkNonNullExpression(rhsExpression);
                var use = awaitModifier ? 15 /* ForAwaitOf */ : 13 /* ForOf */;
                return checkIteratedTypeOrElementType(use, expressionType, undefinedType, rhsExpression);
            }
            function checkIteratedTypeOrElementType(use, inputType, sentType, errorNode) {
                if (isTypeAny(inputType)) {
                    return inputType;
                }
                return getIteratedTypeOrElementType(use, inputType, sentType, errorNode, /*checkAssignability*/ true) || anyType;
            }
            /**
             * When consuming an iterable type in a for..of, spread, or iterator destructuring assignment
             * we want to get the iterated type of an iterable for ES2015 or later, or the iterated type
             * of a iterable (if defined globally) or element type of an array like for ES2015 or earlier.
             */
            function getIteratedTypeOrElementType(use, inputType, sentType, errorNode, checkAssignability) {
                var allowAsyncIterables = (use & 2 /* AllowsAsyncIterablesFlag */) !== 0;
                if (inputType === neverType) {
                    reportTypeNotIterableError(errorNode, inputType, allowAsyncIterables); // TODO: GH#18217
                    return undefined;
                }
                var uplevelIteration = languageVersion >= 2 /* ES2015 */;
                var downlevelIteration = !uplevelIteration && compilerOptions.downlevelIteration;
                // Get the iterated type of an `Iterable<T>` or `IterableIterator<T>` only in ES2015
                // or higher, when inside of an async generator or for-await-if, or when
                // downlevelIteration is requested.
                if (uplevelIteration || downlevelIteration || allowAsyncIterables) {
                    // We only report errors for an invalid iterable type in ES2015 or higher.
                    var iterationTypes = getIterationTypesOfIterable(inputType, use, uplevelIteration ? errorNode : undefined);
                    if (checkAssignability) {
                        if (iterationTypes) {
                            var diagnostic = use & 8 /* ForOfFlag */ ? ts.Diagnostics.Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_for_of_will_always_send_0 :
                                use & 32 /* SpreadFlag */ ? ts.Diagnostics.Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_spread_will_always_send_0 :
                                    use & 64 /* DestructuringFlag */ ? ts.Diagnostics.Cannot_iterate_value_because_the_next_method_of_its_iterator_expects_type_1_but_array_destructuring_will_always_send_0 :
                                        use & 16 /* YieldStarFlag */ ? ts.Diagnostics.Cannot_delegate_iteration_to_value_because_the_next_method_of_its_iterator_expects_type_1_but_the_containing_generator_will_always_send_0 :
                                            undefined;
                            if (diagnostic) {
                                checkTypeAssignableTo(sentType, iterationTypes.nextType, errorNode, diagnostic);
                            }
                        }
                    }
                    if (iterationTypes || uplevelIteration) {
                        return iterationTypes && iterationTypes.yieldType;
                    }
                }
                var arrayType = inputType;
                var reportedError = false;
                var hasStringConstituent = false;
                // If strings are permitted, remove any string-like constituents from the array type.
                // This allows us to find other non-string element types from an array unioned with
                // a string.
                if (use & 4 /* AllowsStringInputFlag */) {
                    if (arrayType.flags & 1048576 /* Union */) {
                        // After we remove all types that are StringLike, we will know if there was a string constituent
                        // based on whether the result of filter is a new array.
                        var arrayTypes = inputType.types;
                        var filteredTypes = ts.filter(arrayTypes, function (t) { return !(t.flags & 132 /* StringLike */); });
                        if (filteredTypes !== arrayTypes) {
                            arrayType = getUnionType(filteredTypes, 2 /* Subtype */);
                        }
                    }
                    else if (arrayType.flags & 132 /* StringLike */) {
                        arrayType = neverType;
                    }
                    hasStringConstituent = arrayType !== inputType;
                    if (hasStringConstituent) {
                        if (languageVersion < 1 /* ES5 */) {
                            if (errorNode) {
                                error(errorNode, ts.Diagnostics.Using_a_string_in_a_for_of_statement_is_only_supported_in_ECMAScript_5_and_higher);
                                reportedError = true;
                            }
                        }
                        // Now that we've removed all the StringLike types, if no constituents remain, then the entire
                        // arrayOrStringType was a string.
                        if (arrayType.flags & 131072 /* Never */) {
                            return stringType;
                        }
                    }
                }
                if (!isArrayLikeType(arrayType)) {
                    if (errorNode && !reportedError) {
                        // Which error we report depends on whether we allow strings or if there was a
                        // string constituent. For example, if the input type is number | string, we
                        // want to say that number is not an array type. But if the input was just
                        // number and string input is allowed, we want to say that number is not an
                        // array type or a string type.
                        var yieldType = getIterationTypeOfIterable(use, 0 /* Yield */, inputType, /*errorNode*/ undefined);
                        var _a = !(use & 4 /* AllowsStringInputFlag */) || hasStringConstituent
                            ? downlevelIteration
                                ? [ts.Diagnostics.Type_0_is_not_an_array_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator, true]
                                : yieldType
                                    ? [ts.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators, false]
                                    : [ts.Diagnostics.Type_0_is_not_an_array_type, true]
                            : downlevelIteration
                                ? [ts.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator, true]
                                : yieldType
                                    ? [ts.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators, false]
                                    : [ts.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type, true], defaultDiagnostic = _a[0], maybeMissingAwait = _a[1];
                        errorAndMaybeSuggestAwait(errorNode, maybeMissingAwait && !!getAwaitedTypeOfPromise(arrayType), defaultDiagnostic, typeToString(arrayType));
                    }
                    return hasStringConstituent ? stringType : undefined;
                }
                var arrayElementType = getIndexTypeOfType(arrayType, 1 /* Number */);
                if (hasStringConstituent && arrayElementType) {
                    // This is just an optimization for the case where arrayOrStringType is string | string[]
                    if (arrayElementType.flags & 132 /* StringLike */) {
                        return stringType;
                    }
                    return getUnionType([arrayElementType, stringType], 2 /* Subtype */);
                }
                return arrayElementType;
            }
            /**
             * Gets the requested "iteration type" from an `Iterable`-like or `AsyncIterable`-like type.
             */
            function getIterationTypeOfIterable(use, typeKind, inputType, errorNode) {
                if (isTypeAny(inputType)) {
                    return undefined;
                }
                var iterationTypes = getIterationTypesOfIterable(inputType, use, errorNode);
                return iterationTypes && iterationTypes[getIterationTypesKeyFromIterationTypeKind(typeKind)];
            }
            function createIterationTypes(yieldType, returnType, nextType) {
                // `yieldType` and `returnType` are defaulted to `neverType` they each will be combined
                // via `getUnionType` when merging iteration types. `nextType` is defined as `unknownType`
                // as it is combined via `getIntersectionType` when merging iteration types.
                if (yieldType === void 0) {
                    yieldType = neverType;
                }
                if (returnType === void 0) {
                    returnType = neverType;
                }
                if (nextType === void 0) {
                    nextType = unknownType;
                }
                // Use the cache only for intrinsic types to keep it small as they are likely to be
                // more frequently created (i.e. `Iterator<number, void, unknown>`). Iteration types
                // are also cached on the type they are requested for, so we shouldn't need to maintain
                // the cache for less-frequently used types.
                if (yieldType.flags & 67359327 /* Intrinsic */ &&
                    returnType.flags & (1 /* Any */ | 131072 /* Never */ | 2 /* Unknown */ | 16384 /* Void */ | 32768 /* Undefined */) &&
                    nextType.flags & (1 /* Any */ | 131072 /* Never */ | 2 /* Unknown */ | 16384 /* Void */ | 32768 /* Undefined */)) {
                    var id = getTypeListId([yieldType, returnType, nextType]);
                    var iterationTypes = iterationTypesCache.get(id);
                    if (!iterationTypes) {
                        iterationTypes = { yieldType: yieldType, returnType: returnType, nextType: nextType };
                        iterationTypesCache.set(id, iterationTypes);
                    }
                    return iterationTypes;
                }
                return { yieldType: yieldType, returnType: returnType, nextType: nextType };
            }
            /**
             * Combines multiple `IterationTypes` records.
             *
             * If `array` is empty or all elements are missing or are references to `noIterationTypes`,
             * then `noIterationTypes` is returned. Otherwise, an `IterationTypes` record is returned
             * for the combined iteration types.
             */
            function combineIterationTypes(array) {
                var yieldTypes;
                var returnTypes;
                var nextTypes;
                for (var _i = 0, array_10 = array; _i < array_10.length; _i++) {
                    var iterationTypes = array_10[_i];
                    if (iterationTypes === undefined || iterationTypes === noIterationTypes) {
                        continue;
                    }
                    if (iterationTypes === anyIterationTypes) {
                        return anyIterationTypes;
                    }
                    yieldTypes = ts.append(yieldTypes, iterationTypes.yieldType);
                    returnTypes = ts.append(returnTypes, iterationTypes.returnType);
                    nextTypes = ts.append(nextTypes, iterationTypes.nextType);
                }
                if (yieldTypes || returnTypes || nextTypes) {
                    return createIterationTypes(yieldTypes && getUnionType(yieldTypes), returnTypes && getUnionType(returnTypes), nextTypes && getIntersectionType(nextTypes));
                }
                return noIterationTypes;
            }
            function getCachedIterationTypes(type, cacheKey) {
                return type[cacheKey];
            }
            function setCachedIterationTypes(type, cacheKey, cachedTypes) {
                return type[cacheKey] = cachedTypes;
            }
            /**
             * Gets the *yield*, *return*, and *next* types from an `Iterable`-like or `AsyncIterable`-like type.
             *
             * At every level that involves analyzing return types of signatures, we union the return types of all the signatures.
             *
             * Another thing to note is that at any step of this process, we could run into a dead end,
             * meaning either the property is missing, or we run into the anyType. If either of these things
             * happens, we return `undefined` to signal that we could not find the iteration type. If a property
             * is missing, and the previous step did not result in `any`, then we also give an error if the
             * caller requested it. Then the caller can decide what to do in the case where there is no iterated
             * type.
             *
             * For a **for-of** statement, `yield*` (in a normal generator), spread, array
             * destructuring, or normal generator we will only ever look for a `[Symbol.iterator]()`
             * method.
             *
             * For an async generator we will only ever look at the `[Symbol.asyncIterator]()` method.
             *
             * For a **for-await-of** statement or a `yield*` in an async generator we will look for
             * the `[Symbol.asyncIterator]()` method first, and then the `[Symbol.iterator]()` method.
             */
            function getIterationTypesOfIterable(type, use, errorNode) {
                if (isTypeAny(type)) {
                    return anyIterationTypes;
                }
                if (!(type.flags & 1048576 /* Union */)) {
                    var iterationTypes_1 = getIterationTypesOfIterableWorker(type, use, errorNode);
                    if (iterationTypes_1 === noIterationTypes) {
                        if (errorNode) {
                            reportTypeNotIterableError(errorNode, type, !!(use & 2 /* AllowsAsyncIterablesFlag */));
                        }
                        return undefined;
                    }
                    return iterationTypes_1;
                }
                var cacheKey = use & 2 /* AllowsAsyncIterablesFlag */ ? "iterationTypesOfAsyncIterable" : "iterationTypesOfIterable";
                var cachedTypes = getCachedIterationTypes(type, cacheKey);
                if (cachedTypes)
                    return cachedTypes === noIterationTypes ? undefined : cachedTypes;
                var allIterationTypes;
                for (var _i = 0, _a = type.types; _i < _a.length; _i++) {
                    var constituent = _a[_i];
                    var iterationTypes_2 = getIterationTypesOfIterableWorker(constituent, use, errorNode);
                    if (iterationTypes_2 === noIterationTypes) {
                        if (errorNode) {
                            reportTypeNotIterableError(errorNode, type, !!(use & 2 /* AllowsAsyncIterablesFlag */));
                            errorNode = undefined;
                        }
                    }
                    else {
                        allIterationTypes = ts.append(allIterationTypes, iterationTypes_2);
                    }
                }
                var iterationTypes = allIterationTypes ? combineIterationTypes(allIterationTypes) : noIterationTypes;
                setCachedIterationTypes(type, cacheKey, iterationTypes);
                return iterationTypes === noIterationTypes ? undefined : iterationTypes;
            }
            function getAsyncFromSyncIterationTypes(iterationTypes, errorNode) {
                if (iterationTypes === noIterationTypes)
                    return noIterationTypes;
                if (iterationTypes === anyIterationTypes)
                    return anyIterationTypes;
                var yieldType = iterationTypes.yieldType, returnType = iterationTypes.returnType, nextType = iterationTypes.nextType;
                return createIterationTypes(getAwaitedType(yieldType, errorNode) || anyType, getAwaitedType(returnType, errorNode) || anyType, nextType);
            }
            /**
             * Gets the *yield*, *return*, and *next* types from a non-union type.
             *
             * If we are unable to find the *yield*, *return*, and *next* types, `noIterationTypes` is
             * returned to indicate to the caller that it should report an error. Otherwise, an
             * `IterationTypes` record is returned.
             *
             * NOTE: You probably don't want to call this directly and should be calling
             * `getIterationTypesOfIterable` instead.
             */
            function getIterationTypesOfIterableWorker(type, use, errorNode) {
                if (isTypeAny(type)) {
                    return anyIterationTypes;
                }
                if (use & 2 /* AllowsAsyncIterablesFlag */) {
                    var iterationTypes = getIterationTypesOfIterableCached(type, asyncIterationTypesResolver) ||
                        getIterationTypesOfIterableFast(type, asyncIterationTypesResolver);
                    if (iterationTypes) {
                        return iterationTypes;
                    }
                }
                if (use & 1 /* AllowsSyncIterablesFlag */) {
                    var iterationTypes = getIterationTypesOfIterableCached(type, syncIterationTypesResolver) ||
                        getIterationTypesOfIterableFast(type, syncIterationTypesResolver);
                    if (iterationTypes) {
                        if (use & 2 /* AllowsAsyncIterablesFlag */) {
                            // for a sync iterable in an async context, only use the cached types if they are valid.
                            if (iterationTypes !== noIterationTypes) {
                                return setCachedIterationTypes(type, "iterationTypesOfAsyncIterable", getAsyncFromSyncIterationTypes(iterationTypes, errorNode));
                            }
                        }
                        else {
                            return iterationTypes;
                        }
                    }
                }
                if (use & 2 /* AllowsAsyncIterablesFlag */) {
                    var iterationTypes = getIterationTypesOfIterableSlow(type, asyncIterationTypesResolver, errorNode);
                    if (iterationTypes !== noIterationTypes) {
                        return iterationTypes;
                    }
                }
                if (use & 1 /* AllowsSyncIterablesFlag */) {
                    var iterationTypes = getIterationTypesOfIterableSlow(type, syncIterationTypesResolver, errorNode);
                    if (iterationTypes !== noIterationTypes) {
                        if (use & 2 /* AllowsAsyncIterablesFlag */) {
                            return setCachedIterationTypes(type, "iterationTypesOfAsyncIterable", iterationTypes
                                ? getAsyncFromSyncIterationTypes(iterationTypes, errorNode)
                                : noIterationTypes);
                        }
                        else {
                            return iterationTypes;
                        }
                    }
                }
                return noIterationTypes;
            }
            /**
             * Gets the *yield*, *return*, and *next* types of an `Iterable`-like or
             * `AsyncIterable`-like type from the cache.
             *
             * NOTE: You probably don't want to call this directly and should be calling
             * `getIterationTypesOfIterable` instead.
             */
            function getIterationTypesOfIterableCached(type, resolver) {
                return getCachedIterationTypes(type, resolver.iterableCacheKey);
            }
            function getIterationTypesOfGlobalIterableType(globalType, resolver) {
                var globalIterationTypes = getIterationTypesOfIterableCached(globalType, resolver) ||
                    getIterationTypesOfIterableSlow(globalType, resolver, /*errorNode*/ undefined);
                return globalIterationTypes === noIterationTypes ? defaultIterationTypes : globalIterationTypes;
            }
            /**
             * Gets the *yield*, *return*, and *next* types of an `Iterable`-like or `AsyncIterable`-like
             * type from from common heuristics.
             *
             * If we previously analyzed this type and found no iteration types, `noIterationTypes` is
             * returned. If we found iteration types, an `IterationTypes` record is returned.
             * Otherwise, we return `undefined` to indicate to the caller it should perform a more
             * exhaustive analysis.
             *
             * NOTE: You probably don't want to call this directly and should be calling
             * `getIterationTypesOfIterable` instead.
             */
            function getIterationTypesOfIterableFast(type, resolver) {
                // As an optimization, if the type is an instantiation of one of the following global types, then
                // just grab its related type argument:
                // - `Iterable<T>` or `AsyncIterable<T>`
                // - `IterableIterator<T>` or `AsyncIterableIterator<T>`
                var globalType;
                if (isReferenceToType(type, globalType = resolver.getGlobalIterableType(/*reportErrors*/ false)) ||
                    isReferenceToType(type, globalType = resolver.getGlobalIterableIteratorType(/*reportErrors*/ false))) {
                    var yieldType = getTypeArguments(type)[0];
                    // The "return" and "next" types of `Iterable` and `IterableIterator` are defined by the
                    // iteration types of their `[Symbol.iterator]()` method. The same is true for their async cousins.
                    // While we define these as `any` and `undefined` in our libs by default, a custom lib *could* use
                    // different definitions.
                    var _a = getIterationTypesOfGlobalIterableType(globalType, resolver), returnType = _a.returnType, nextType = _a.nextType;
                    return setCachedIterationTypes(type, resolver.iterableCacheKey, createIterationTypes(yieldType, returnType, nextType));
                }
                // As an optimization, if the type is an instantiation of the following global type, then
                // just grab its related type arguments:
                // - `Generator<T, TReturn, TNext>` or `AsyncGenerator<T, TReturn, TNext>`
                if (isReferenceToType(type, resolver.getGlobalGeneratorType(/*reportErrors*/ false))) {
                    var _b = getTypeArguments(type), yieldType = _b[0], returnType = _b[1], nextType = _b[2];
                    return setCachedIterationTypes(type, resolver.iterableCacheKey, createIterationTypes(yieldType, returnType, nextType));
                }
            }
            /**
             * Gets the *yield*, *return*, and *next* types of an `Iterable`-like or `AsyncIterable`-like
             * type from its members.
             *
             * If we successfully found the *yield*, *return*, and *next* types, an `IterationTypes`
             * record is returned. Otherwise, `noIterationTypes` is returned.
             *
             * NOTE: You probably don't want to call this directly and should be calling
             * `getIterationTypesOfIterable` instead.
             */
            function getIterationTypesOfIterableSlow(type, resolver, errorNode) {
                var _a;
                var method = getPropertyOfType(type, ts.getPropertyNameForKnownSymbolName(resolver.iteratorSymbolName));
                var methodType = method && !(method.flags & 16777216 /* Optional */) ? getTypeOfSymbol(method) : undefined;
                if (isTypeAny(methodType)) {
                    return setCachedIterationTypes(type, resolver.iterableCacheKey, anyIterationTypes);
                }
                var signatures = methodType ? getSignaturesOfType(methodType, 0 /* Call */) : undefined;
                if (!ts.some(signatures)) {
                    return setCachedIterationTypes(type, resolver.iterableCacheKey, noIterationTypes);
                }
                var iteratorType = getUnionType(ts.map(signatures, getReturnTypeOfSignature), 2 /* Subtype */);
                var iterationTypes = (_a = getIterationTypesOfIterator(iteratorType, resolver, errorNode)) !== null && _a !== void 0 ? _a : noIterationTypes;
                return setCachedIterationTypes(type, resolver.iterableCacheKey, iterationTypes);
            }
            function reportTypeNotIterableError(errorNode, type, allowAsyncIterables) {
                var message = allowAsyncIterables
                    ? ts.Diagnostics.Type_0_must_have_a_Symbol_asyncIterator_method_that_returns_an_async_iterator
                    : ts.Diagnostics.Type_0_must_have_a_Symbol_iterator_method_that_returns_an_iterator;
                errorAndMaybeSuggestAwait(errorNode, !!getAwaitedTypeOfPromise(type), message, typeToString(type));
            }
            /**
             * Gets the *yield*, *return*, and *next* types from an `Iterator`-like or `AsyncIterator`-like type.
             *
             * If we successfully found the *yield*, *return*, and *next* types, an `IterationTypes`
             * record is returned. Otherwise, `undefined` is returned.
             */
            function getIterationTypesOfIterator(type, resolver, errorNode) {
                if (isTypeAny(type)) {
                    return anyIterationTypes;
                }
                var iterationTypes = getIterationTypesOfIteratorCached(type, resolver) ||
                    getIterationTypesOfIteratorFast(type, resolver) ||
                    getIterationTypesOfIteratorSlow(type, resolver, errorNode);
                return iterationTypes === noIterationTypes ? undefined : iterationTypes;
            }
            /**
             * Gets the iteration types of an `Iterator`-like or `AsyncIterator`-like type from the
             * cache.
             *
             * NOTE: You probably don't want to call this directly and should be calling
             * `getIterationTypesOfIterator` instead.
             */
            function getIterationTypesOfIteratorCached(type, resolver) {
                return getCachedIterationTypes(type, resolver.iteratorCacheKey);
            }
            /**
             * Gets the iteration types of an `Iterator`-like or `AsyncIterator`-like type from the
             * cache or from common heuristics.
             *
             * If we previously analyzed this type and found no iteration types, `noIterationTypes` is
             * returned. If we found iteration types, an `IterationTypes` record is returned.
             * Otherwise, we return `undefined` to indicate to the caller it should perform a more
             * exhaustive analysis.
             *
             * NOTE: You probably don't want to call this directly and should be calling
             * `getIterationTypesOfIterator` instead.
             */
            function getIterationTypesOfIteratorFast(type, resolver) {
                // As an optimization, if the type is an instantiation of one of the following global types,
                // then just grab its related type argument:
                // - `IterableIterator<T>` or `AsyncIterableIterator<T>`
                // - `Iterator<T, TReturn, TNext>` or `AsyncIterator<T, TReturn, TNext>`
                // - `Generator<T, TReturn, TNext>` or `AsyncGenerator<T, TReturn, TNext>`
                var globalType = resolver.getGlobalIterableIteratorType(/*reportErrors*/ false);
                if (isReferenceToType(type, globalType)) {
                    var yieldType = getTypeArguments(type)[0];
                    // The "return" and "next" types of `IterableIterator` and `AsyncIterableIterator` are defined by the
                    // iteration types of their `next`, `return`, and `throw` methods. While we define these as `any`
                    // and `undefined` in our libs by default, a custom lib *could* use different definitions.
                    var globalIterationTypes = getIterationTypesOfIteratorCached(globalType, resolver) ||
                        getIterationTypesOfIteratorSlow(globalType, resolver, /*errorNode*/ undefined);
                    var _a = globalIterationTypes === noIterationTypes ? defaultIterationTypes : globalIterationTypes, returnType = _a.returnType, nextType = _a.nextType;
                    return setCachedIterationTypes(type, resolver.iteratorCacheKey, createIterationTypes(yieldType, returnType, nextType));
                }
                if (isReferenceToType(type, resolver.getGlobalIteratorType(/*reportErrors*/ false)) ||
                    isReferenceToType(type, resolver.getGlobalGeneratorType(/*reportErrors*/ false))) {
                    var _b = getTypeArguments(type), yieldType = _b[0], returnType = _b[1], nextType = _b[2];
                    return setCachedIterationTypes(type, resolver.iteratorCacheKey, createIterationTypes(yieldType, returnType, nextType));
                }
            }
            function isIteratorResult(type, kind) {
                // From https://tc39.github.io/ecma262/#sec-iteratorresult-interface:
                // > [done] is the result status of an iterator `next` method call. If the end of the iterator was reached `done` is `true`.
                // > If the end was not reached `done` is `false` and a value is available.
                // > If a `done` property (either own or inherited) does not exist, it is consider to have the value `false`.
                var doneType = getTypeOfPropertyOfType(type, "done") || falseType;
                return isTypeAssignableTo(kind === 0 /* Yield */ ? falseType : trueType, doneType);
            }
            function isYieldIteratorResult(type) {
                return isIteratorResult(type, 0 /* Yield */);
            }
            function isReturnIteratorResult(type) {
                return isIteratorResult(type, 1 /* Return */);
            }
            /**
             * Gets the *yield* and *return* types of an `IteratorResult`-like type.
             *
             * If we are unable to determine a *yield* or a *return* type, `noIterationTypes` is
             * returned to indicate to the caller that it should handle the error. Otherwise, an
             * `IterationTypes` record is returned.
             */
            function getIterationTypesOfIteratorResult(type) {
                if (isTypeAny(type)) {
                    return anyIterationTypes;
                }
                var cachedTypes = getCachedIterationTypes(type, "iterationTypesOfIteratorResult");
                if (cachedTypes) {
                    return cachedTypes;
                }
                // As an optimization, if the type is an instantiation of one of the global `IteratorYieldResult<T>`
                // or `IteratorReturnResult<TReturn>` types, then just grab its type argument.
                if (isReferenceToType(type, getGlobalIteratorYieldResultType(/*reportErrors*/ false))) {
                    var yieldType_1 = getTypeArguments(type)[0];
                    return setCachedIterationTypes(type, "iterationTypesOfIteratorResult", createIterationTypes(yieldType_1, /*returnType*/ undefined, /*nextType*/ undefined));
                }
                if (isReferenceToType(type, getGlobalIteratorReturnResultType(/*reportErrors*/ false))) {
                    var returnType_1 = getTypeArguments(type)[0];
                    return setCachedIterationTypes(type, "iterationTypesOfIteratorResult", createIterationTypes(/*yieldType*/ undefined, returnType_1, /*nextType*/ undefined));
                }
                // Choose any constituents that can produce the requested iteration type.
                var yieldIteratorResult = filterType(type, isYieldIteratorResult);
                var yieldType = yieldIteratorResult !== neverType ? getTypeOfPropertyOfType(yieldIteratorResult, "value") : undefined;
                var returnIteratorResult = filterType(type, isReturnIteratorResult);
                var returnType = returnIteratorResult !== neverType ? getTypeOfPropertyOfType(returnIteratorResult, "value") : undefined;
                if (!yieldType && !returnType) {
                    return setCachedIterationTypes(type, "iterationTypesOfIteratorResult", noIterationTypes);
                }
                // From https://tc39.github.io/ecma262/#sec-iteratorresult-interface
                // > ... If the iterator does not have a return value, `value` is `undefined`. In that case, the
                // > `value` property may be absent from the conforming object if it does not inherit an explicit
                // > `value` property.
                return setCachedIterationTypes(type, "iterationTypesOfIteratorResult", createIterationTypes(yieldType, returnType || voidType, /*nextType*/ undefined));
            }
            /**
             * Gets the *yield*, *return*, and *next* types of a the `next()`, `return()`, or
             * `throw()` method of an `Iterator`-like or `AsyncIterator`-like type.
             *
             * If we successfully found the *yield*, *return*, and *next* types, an `IterationTypes`
             * record is returned. Otherwise, we return `undefined`.
             */
            function getIterationTypesOfMethod(type, resolver, methodName, errorNode) {
                var method = getPropertyOfType(type, methodName);
                // Ignore 'return' or 'throw' if they are missing.
                if (!method && methodName !== "next") {
                    return undefined;
                }
                var methodType = method && !(methodName === "next" && (method.flags & 16777216 /* Optional */))
                    ? methodName === "next" ? getTypeOfSymbol(method) : getTypeWithFacts(getTypeOfSymbol(method), 2097152 /* NEUndefinedOrNull */)
                    : undefined;
                if (isTypeAny(methodType)) {
                    // `return()` and `throw()` don't provide a *next* type.
                    return methodName === "next" ? anyIterationTypes : anyIterationTypesExceptNext;
                }
                // Both async and non-async iterators *must* have a `next` method.
                var methodSignatures = methodType ? getSignaturesOfType(methodType, 0 /* Call */) : ts.emptyArray;
                if (methodSignatures.length === 0) {
                    if (errorNode) {
                        var diagnostic = methodName === "next"
                            ? resolver.mustHaveANextMethodDiagnostic
                            : resolver.mustBeAMethodDiagnostic;
                        error(errorNode, diagnostic, methodName);
                    }
                    return methodName === "next" ? anyIterationTypes : undefined;
                }
                // Extract the first parameter and return type of each signature.
                var methodParameterTypes;
                var methodReturnTypes;
                for (var _i = 0, methodSignatures_1 = methodSignatures; _i < methodSignatures_1.length; _i++) {
                    var signature = methodSignatures_1[_i];
                    if (methodName !== "throw" && ts.some(signature.parameters)) {
                        methodParameterTypes = ts.append(methodParameterTypes, getTypeAtPosition(signature, 0));
                    }
                    methodReturnTypes = ts.append(methodReturnTypes, getReturnTypeOfSignature(signature));
                }
                // Resolve the *next* or *return* type from the first parameter of a `next()` or
                // `return()` method, respectively.
                var returnTypes;
                var nextType;
                if (methodName !== "throw") {
                    var methodParameterType = methodParameterTypes ? getUnionType(methodParameterTypes) : unknownType;
                    if (methodName === "next") {
                        // The value of `next(value)` is *not* awaited by async generators
                        nextType = methodParameterType;
                    }
                    else if (methodName === "return") {
                        // The value of `return(value)` *is* awaited by async generators
                        var resolvedMethodParameterType = resolver.resolveIterationType(methodParameterType, errorNode) || anyType;
                        returnTypes = ts.append(returnTypes, resolvedMethodParameterType);
                    }
                }
                // Resolve the *yield* and *return* types from the return type of the method (i.e. `IteratorResult`)
                var yieldType;
                var methodReturnType = methodReturnTypes ? getUnionType(methodReturnTypes, 2 /* Subtype */) : neverType;
                var resolvedMethodReturnType = resolver.resolveIterationType(methodReturnType, errorNode) || anyType;
                var iterationTypes = getIterationTypesOfIteratorResult(resolvedMethodReturnType);
                if (iterationTypes === noIterationTypes) {
                    if (errorNode) {
                        error(errorNode, resolver.mustHaveAValueDiagnostic, methodName);
                    }
                    yieldType = anyType;
                    returnTypes = ts.append(returnTypes, anyType);
                }
                else {
                    yieldType = iterationTypes.yieldType;
                    returnTypes = ts.append(returnTypes, iterationTypes.returnType);
                }
                return createIterationTypes(yieldType, getUnionType(returnTypes), nextType);
            }
            /**
             * Gets the *yield*, *return*, and *next* types of an `Iterator`-like or `AsyncIterator`-like
             * type from its members.
             *
             * If we successfully found the *yield*, *return*, and *next* types, an `IterationTypes`
             * record is returned. Otherwise, `noIterationTypes` is returned.
             *
             * NOTE: You probably don't want to call this directly and should be calling
             * `getIterationTypesOfIterator` instead.
             */
            function getIterationTypesOfIteratorSlow(type, resolver, errorNode) {
                var iterationTypes = combineIterationTypes([
                    getIterationTypesOfMethod(type, resolver, "next", errorNode),
                    getIterationTypesOfMethod(type, resolver, "return", errorNode),
                    getIterationTypesOfMethod(type, resolver, "throw", errorNode),
                ]);
                return setCachedIterationTypes(type, resolver.iteratorCacheKey, iterationTypes);
            }
            /**
             * Gets the requested "iteration type" from a type that is either `Iterable`-like, `Iterator`-like,
             * `IterableIterator`-like, or `Generator`-like (for a non-async generator); or `AsyncIterable`-like,
             * `AsyncIterator`-like, `AsyncIterableIterator`-like, or `AsyncGenerator`-like (for an async generator).
             */
            function getIterationTypeOfGeneratorFunctionReturnType(kind, returnType, isAsyncGenerator) {
                if (isTypeAny(returnType)) {
                    return undefined;
                }
                var iterationTypes = getIterationTypesOfGeneratorFunctionReturnType(returnType, isAsyncGenerator);
                return iterationTypes && iterationTypes[getIterationTypesKeyFromIterationTypeKind(kind)];
            }
            function getIterationTypesOfGeneratorFunctionReturnType(type, isAsyncGenerator) {
                if (isTypeAny(type)) {
                    return anyIterationTypes;
                }
                var use = isAsyncGenerator ? 2 /* AsyncGeneratorReturnType */ : 1 /* GeneratorReturnType */;
                var resolver = isAsyncGenerator ? asyncIterationTypesResolver : syncIterationTypesResolver;
                return getIterationTypesOfIterable(type, use, /*errorNode*/ undefined) ||
                    getIterationTypesOfIterator(type, resolver, /*errorNode*/ undefined);
            }
            function checkBreakOrContinueStatement(node) {
                // Grammar checking
                if (!checkGrammarStatementInAmbientContext(node))
                    checkGrammarBreakOrContinueStatement(node);
                // TODO: Check that target label is valid
            }
            function unwrapReturnType(returnType, functionFlags) {
                var isGenerator = !!(functionFlags & 1 /* Generator */);
                var isAsync = !!(functionFlags & 2 /* Async */);
                return isGenerator ? getIterationTypeOfGeneratorFunctionReturnType(1 /* Return */, returnType, isAsync) || errorType :
                    isAsync ? getPromisedTypeOfPromise(returnType) || errorType :
                        returnType;
            }
            function isUnwrappedReturnTypeVoidOrAny(func, returnType) {
                var unwrappedReturnType = unwrapReturnType(returnType, ts.getFunctionFlags(func));
                return !!unwrappedReturnType && maybeTypeOfKind(unwrappedReturnType, 16384 /* Void */ | 3 /* AnyOrUnknown */);
            }
            function checkReturnStatement(node) {
                // Grammar checking
                if (checkGrammarStatementInAmbientContext(node)) {
                    return;
                }
                var func = ts.getContainingFunction(node);
                if (!func) {
                    grammarErrorOnFirstToken(node, ts.Diagnostics.A_return_statement_can_only_be_used_within_a_function_body);
                    return;
                }
                var signature = getSignatureFromDeclaration(func);
                var returnType = getReturnTypeOfSignature(signature);
                var functionFlags = ts.getFunctionFlags(func);
                if (strictNullChecks || node.expression || returnType.flags & 131072 /* Never */) {
                    var exprType = node.expression ? checkExpressionCached(node.expression) : undefinedType;
                    if (func.kind === 164 /* SetAccessor */) {
                        if (node.expression) {
                            error(node, ts.Diagnostics.Setters_cannot_return_a_value);
                        }
                    }
                    else if (func.kind === 162 /* Constructor */) {
                        if (node.expression && !checkTypeAssignableToAndOptionallyElaborate(exprType, returnType, node, node.expression)) {
                            error(node, ts.Diagnostics.Return_type_of_constructor_signature_must_be_assignable_to_the_instance_type_of_the_class);
                        }
                    }
                    else if (getReturnTypeFromAnnotation(func)) {
                        var unwrappedReturnType = unwrapReturnType(returnType, functionFlags);
                        var unwrappedExprType = functionFlags & 2 /* Async */
                            ? checkAwaitedType(exprType, node, ts.Diagnostics.The_return_type_of_an_async_function_must_either_be_a_valid_promise_or_must_not_contain_a_callable_then_member)
                            : exprType;
                        if (unwrappedReturnType) {
                            // If the function has a return type, but promisedType is
                            // undefined, an error will be reported in checkAsyncFunctionReturnType
                            // so we don't need to report one here.
                            checkTypeAssignableToAndOptionallyElaborate(unwrappedExprType, unwrappedReturnType, node, node.expression);
                        }
                    }
                }
                else if (func.kind !== 162 /* Constructor */ && compilerOptions.noImplicitReturns && !isUnwrappedReturnTypeVoidOrAny(func, returnType)) {
                    // The function has a return type, but the return statement doesn't have an expression.
                    error(node, ts.Diagnostics.Not_all_code_paths_return_a_value);
                }
            }
            function checkWithStatement(node) {
                // Grammar checking for withStatement
                if (!checkGrammarStatementInAmbientContext(node)) {
                    if (node.flags & 32768 /* AwaitContext */) {
                        grammarErrorOnFirstToken(node, ts.Diagnostics.with_statements_are_not_allowed_in_an_async_function_block);
                    }
                }
                checkExpression(node.expression);
                var sourceFile = ts.getSourceFileOfNode(node);
                if (!hasParseDiagnostics(sourceFile)) {
                    var start = ts.getSpanOfTokenAtPosition(sourceFile, node.pos).start;
                    var end = node.statement.pos;
                    grammarErrorAtPos(sourceFile, start, end - start, ts.Diagnostics.The_with_statement_is_not_supported_All_symbols_in_a_with_block_will_have_type_any);
                }
            }
            function checkSwitchStatement(node) {
                // Grammar checking
                checkGrammarStatementInAmbientContext(node);
                var firstDefaultClause;
                var hasDuplicateDefaultClause = false;
                var expressionType = checkExpression(node.expression);
                var expressionIsLiteral = isLiteralType(expressionType);
                ts.forEach(node.caseBlock.clauses, function (clause) {
                    // Grammar check for duplicate default clauses, skip if we already report duplicate default clause
                    if (clause.kind === 278 /* DefaultClause */ && !hasDuplicateDefaultClause) {
                        if (firstDefaultClause === undefined) {
                            firstDefaultClause = clause;
                        }
                        else {
                            grammarErrorOnNode(clause, ts.Diagnostics.A_default_clause_cannot_appear_more_than_once_in_a_switch_statement);
                            hasDuplicateDefaultClause = true;
                        }
                    }
                    if (produceDiagnostics && clause.kind === 277 /* CaseClause */) {
                        // TypeScript 1.0 spec (April 2014): 5.9
                        // In a 'switch' statement, each 'case' expression must be of a type that is comparable
                        // to or from the type of the 'switch' expression.
                        var caseType = checkExpression(clause.expression);
                        var caseIsLiteral = isLiteralType(caseType);
                        var comparedExpressionType = expressionType;
                        if (!caseIsLiteral || !expressionIsLiteral) {
                            caseType = caseIsLiteral ? getBaseTypeOfLiteralType(caseType) : caseType;
                            comparedExpressionType = getBaseTypeOfLiteralType(expressionType);
                        }
                        if (!isTypeEqualityComparableTo(comparedExpressionType, caseType)) {
                            // expressionType is not comparable to caseType, try the reversed check and report errors if it fails
                            checkTypeComparableTo(caseType, comparedExpressionType, clause.expression, /*headMessage*/ undefined);
                        }
                    }
                    ts.forEach(clause.statements, checkSourceElement);
                    if (compilerOptions.noFallthroughCasesInSwitch && clause.fallthroughFlowNode && isReachableFlowNode(clause.fallthroughFlowNode)) {
                        error(clause, ts.Diagnostics.Fallthrough_case_in_switch);
                    }
                });
                if (node.caseBlock.locals) {
                    registerForUnusedIdentifiersCheck(node.caseBlock);
                }
            }
            function checkLabeledStatement(node) {
                // Grammar checking
                if (!checkGrammarStatementInAmbientContext(node)) {
                    ts.findAncestor(node.parent, function (current) {
                        if (ts.isFunctionLike(current)) {
                            return "quit";
                        }
                        if (current.kind === 238 /* LabeledStatement */ && current.label.escapedText === node.label.escapedText) {
                            grammarErrorOnNode(node.label, ts.Diagnostics.Duplicate_label_0, ts.getTextOfNode(node.label));
                            return true;
                        }
                        return false;
                    });
                }
                // ensure that label is unique
                checkSourceElement(node.statement);
            }
            function checkThrowStatement(node) {
                // Grammar checking
                if (!checkGrammarStatementInAmbientContext(node)) {
                    if (node.expression === undefined) {
                        grammarErrorAfterFirstToken(node, ts.Diagnostics.Line_break_not_permitted_here);
                    }
                }
                if (node.expression) {
                    checkExpression(node.expression);
                }
            }
            function checkTryStatement(node) {
                // Grammar checking
                checkGrammarStatementInAmbientContext(node);
                checkBlock(node.tryBlock);
                var catchClause = node.catchClause;
                if (catchClause) {
                    // Grammar checking
                    if (catchClause.variableDeclaration) {
                        if (catchClause.variableDeclaration.type) {
                            grammarErrorOnFirstToken(catchClause.variableDeclaration.type, ts.Diagnostics.Catch_clause_variable_cannot_have_a_type_annotation);
                        }
                        else if (catchClause.variableDeclaration.initializer) {
                            grammarErrorOnFirstToken(catchClause.variableDeclaration.initializer, ts.Diagnostics.Catch_clause_variable_cannot_have_an_initializer);
                        }
                        else {
                            var blockLocals_1 = catchClause.block.locals;
                            if (blockLocals_1) {
                                ts.forEachKey(catchClause.locals, function (caughtName) {
                                    var blockLocal = blockLocals_1.get(caughtName);
                                    if (blockLocal && (blockLocal.flags & 2 /* BlockScopedVariable */) !== 0) {
                                        grammarErrorOnNode(blockLocal.valueDeclaration, ts.Diagnostics.Cannot_redeclare_identifier_0_in_catch_clause, caughtName);
                                    }
                                });
                            }
                        }
                    }
                    checkBlock(catchClause.block);
                }
                if (node.finallyBlock) {
                    checkBlock(node.finallyBlock);
                }
            }
            function checkIndexConstraints(type) {
                var declaredNumberIndexer = getIndexDeclarationOfSymbol(type.symbol, 1 /* Number */);
                var declaredStringIndexer = getIndexDeclarationOfSymbol(type.symbol, 0 /* String */);
                var stringIndexType = getIndexTypeOfType(type, 0 /* String */);
                var numberIndexType = getIndexTypeOfType(type, 1 /* Number */);
                if (stringIndexType || numberIndexType) {
                    ts.forEach(getPropertiesOfObjectType(type), function (prop) {
                        var propType = getTypeOfSymbol(prop);
                        checkIndexConstraintForProperty(prop, propType, type, declaredStringIndexer, stringIndexType, 0 /* String */);
                        checkIndexConstraintForProperty(prop, propType, type, declaredNumberIndexer, numberIndexType, 1 /* Number */);
                    });
                    var classDeclaration = type.symbol.valueDeclaration;
                    if (ts.getObjectFlags(type) & 1 /* Class */ && ts.isClassLike(classDeclaration)) {
                        for (var _i = 0, _a = classDeclaration.members; _i < _a.length; _i++) {
                            var member = _a[_i];
                            // Only process instance properties with computed names here.
                            // Static properties cannot be in conflict with indexers,
                            // and properties with literal names were already checked.
                            if (!ts.hasModifier(member, 32 /* Static */) && hasNonBindableDynamicName(member)) {
                                var symbol = getSymbolOfNode(member);
                                var propType = getTypeOfSymbol(symbol);
                                checkIndexConstraintForProperty(symbol, propType, type, declaredStringIndexer, stringIndexType, 0 /* String */);
                                checkIndexConstraintForProperty(symbol, propType, type, declaredNumberIndexer, numberIndexType, 1 /* Number */);
                            }
                        }
                    }
                }
                var errorNode;
                if (stringIndexType && numberIndexType) {
                    errorNode = declaredNumberIndexer || declaredStringIndexer;
                    // condition 'errorNode === undefined' may appear if types does not declare nor string neither number indexer
                    if (!errorNode && (ts.getObjectFlags(type) & 2 /* Interface */)) {
                        var someBaseTypeHasBothIndexers = ts.forEach(getBaseTypes(type), function (base) { return getIndexTypeOfType(base, 0 /* String */) && getIndexTypeOfType(base, 1 /* Number */); });
                        errorNode = someBaseTypeHasBothIndexers ? undefined : type.symbol.declarations[0];
                    }
                }
                if (errorNode && !isTypeAssignableTo(numberIndexType, stringIndexType)) { // TODO: GH#18217
                    error(errorNode, ts.Diagnostics.Numeric_index_type_0_is_not_assignable_to_string_index_type_1, typeToString(numberIndexType), typeToString(stringIndexType));
                }
                function checkIndexConstraintForProperty(prop, propertyType, containingType, indexDeclaration, indexType, indexKind) {
                    // ESSymbol properties apply to neither string nor numeric indexers.
                    if (!indexType || ts.isKnownSymbol(prop)) {
                        return;
                    }
                    var propDeclaration = prop.valueDeclaration;
                    var name = propDeclaration && ts.getNameOfDeclaration(propDeclaration);
                    // index is numeric and property name is not valid numeric literal
                    if (indexKind === 1 /* Number */ && !(name ? isNumericName(name) : isNumericLiteralName(prop.escapedName))) {
                        return;
                    }
                    // perform property check if property or indexer is declared in 'type'
                    // this allows us to rule out cases when both property and indexer are inherited from the base class
                    var errorNode;
                    if (propDeclaration && name &&
                        (propDeclaration.kind === 209 /* BinaryExpression */ ||
                            name.kind === 154 /* ComputedPropertyName */ ||
                            prop.parent === containingType.symbol)) {
                        errorNode = propDeclaration;
                    }
                    else if (indexDeclaration) {
                        errorNode = indexDeclaration;
                    }
                    else if (ts.getObjectFlags(containingType) & 2 /* Interface */) {
                        // for interfaces property and indexer might be inherited from different bases
                        // check if any base class already has both property and indexer.
                        // check should be performed only if 'type' is the first type that brings property\indexer together
                        var someBaseClassHasBothPropertyAndIndexer = ts.forEach(getBaseTypes(containingType), function (base) { return getPropertyOfObjectType(base, prop.escapedName) && getIndexTypeOfType(base, indexKind); });
                        errorNode = someBaseClassHasBothPropertyAndIndexer ? undefined : containingType.symbol.declarations[0];
                    }
                    if (errorNode && !isTypeAssignableTo(propertyType, indexType)) {
                        var errorMessage = indexKind === 0 /* String */
                            ? ts.Diagnostics.Property_0_of_type_1_is_not_assignable_to_string_index_type_2
                            : ts.Diagnostics.Property_0_of_type_1_is_not_assignable_to_numeric_index_type_2;
                        error(errorNode, errorMessage, symbolToString(prop), typeToString(propertyType), typeToString(indexType));
                    }
                }
            }
            function checkTypeNameIsReserved(name, message) {
                // TS 1.0 spec (April 2014): 3.6.1
                // The predefined type keywords are reserved and cannot be used as names of user defined types.
                switch (name.escapedText) {
                    case "any":
                    case "unknown":
                    case "number":
                    case "bigint":
                    case "boolean":
                    case "string":
                    case "symbol":
                    case "void":
                    case "object":
                        error(name, message, name.escapedText);
                }
            }
            /**
             * The name cannot be used as 'Object' of user defined types with special target.
             */
            function checkClassNameCollisionWithObject(name) {
                if (languageVersion === 1 /* ES5 */ && name.escapedText === "Object"
                    && moduleKind < ts.ModuleKind.ES2015) {
                    error(name, ts.Diagnostics.Class_name_cannot_be_Object_when_targeting_ES5_with_module_0, ts.ModuleKind[moduleKind]); // https://github.com/Microsoft/TypeScript/issues/17494
                }
            }
            /**
             * Check each type parameter and check that type parameters have no duplicate type parameter declarations
             */
            function checkTypeParameters(typeParameterDeclarations) {
                if (typeParameterDeclarations) {
                    var seenDefault = false;
                    for (var i = 0; i < typeParameterDeclarations.length; i++) {
                        var node = typeParameterDeclarations[i];
                        checkTypeParameter(node);
                        if (produceDiagnostics) {
                            if (node.default) {
                                seenDefault = true;
                                checkTypeParametersNotReferenced(node.default, typeParameterDeclarations, i);
                            }
                            else if (seenDefault) {
                                error(node, ts.Diagnostics.Required_type_parameters_may_not_follow_optional_type_parameters);
                            }
                            for (var j = 0; j < i; j++) {
                                if (typeParameterDeclarations[j].symbol === node.symbol) {
                                    error(node.name, ts.Diagnostics.Duplicate_identifier_0, ts.declarationNameToString(node.name));
                                }
                            }
                        }
                    }
                }
            }
            /** Check that type parameter defaults only reference previously declared type parameters */
            function checkTypeParametersNotReferenced(root, typeParameters, index) {
                visit(root);
                function visit(node) {
                    if (node.kind === 169 /* TypeReference */) {
                        var type = getTypeFromTypeReference(node);
                        if (type.flags & 262144 /* TypeParameter */) {
                            for (var i = index; i < typeParameters.length; i++) {
                                if (type.symbol === getSymbolOfNode(typeParameters[i])) {
                                    error(node, ts.Diagnostics.Type_parameter_defaults_can_only_reference_previously_declared_type_parameters);
                                }
                            }
                        }
                    }
                    ts.forEachChild(node, visit);
                }
            }
            /** Check that type parameter lists are identical across multiple declarations */
            function checkTypeParameterListsIdentical(symbol) {
                if (symbol.declarations.length === 1) {
                    return;
                }
                var links = getSymbolLinks(symbol);
                if (!links.typeParametersChecked) {
                    links.typeParametersChecked = true;
                    var declarations = getClassOrInterfaceDeclarationsOfSymbol(symbol);
                    if (declarations.length <= 1) {
                        return;
                    }
                    var type = getDeclaredTypeOfSymbol(symbol);
                    if (!areTypeParametersIdentical(declarations, type.localTypeParameters)) {
                        // Report an error on every conflicting declaration.
                        var name = symbolToString(symbol);
                        for (var _i = 0, declarations_6 = declarations; _i < declarations_6.length; _i++) {
                            var declaration = declarations_6[_i];
                            error(declaration.name, ts.Diagnostics.All_declarations_of_0_must_have_identical_type_parameters, name);
                        }
                    }
                }
            }
            function areTypeParametersIdentical(declarations, targetParameters) {
                var maxTypeArgumentCount = ts.length(targetParameters);
                var minTypeArgumentCount = getMinTypeArgumentCount(targetParameters);
                for (var _i = 0, declarations_7 = declarations; _i < declarations_7.length; _i++) {
                    var declaration = declarations_7[_i];
                    // If this declaration has too few or too many type parameters, we report an error
                    var sourceParameters = ts.getEffectiveTypeParameterDeclarations(declaration);
                    var numTypeParameters = sourceParameters.length;
                    if (numTypeParameters < minTypeArgumentCount || numTypeParameters > maxTypeArgumentCount) {
                        return false;
                    }
                    for (var i = 0; i < numTypeParameters; i++) {
                        var source = sourceParameters[i];
                        var target = targetParameters[i];
                        // If the type parameter node does not have the same as the resolved type
                        // parameter at this position, we report an error.
                        if (source.name.escapedText !== target.symbol.escapedName) {
                            return false;
                        }
                        // If the type parameter node does not have an identical constraint as the resolved
                        // type parameter at this position, we report an error.
                        var constraint = ts.getEffectiveConstraintOfTypeParameter(source);
                        var sourceConstraint = constraint && getTypeFromTypeNode(constraint);
                        var targetConstraint = getConstraintOfTypeParameter(target);
                        // relax check if later interface augmentation has no constraint, it's more broad and is OK to merge with
                        // a more constrained interface (this could be generalized to a full hierarchy check, but that's maybe overkill)
                        if (sourceConstraint && targetConstraint && !isTypeIdenticalTo(sourceConstraint, targetConstraint)) {
                            return false;
                        }
                        // If the type parameter node has a default and it is not identical to the default
                        // for the type parameter at this position, we report an error.
                        var sourceDefault = source.default && getTypeFromTypeNode(source.default);
                        var targetDefault = getDefaultFromTypeParameter(target);
                        if (sourceDefault && targetDefault && !isTypeIdenticalTo(sourceDefault, targetDefault)) {
                            return false;
                        }
                    }
                }
                return true;
            }
            function checkClassExpression(node) {
                checkClassLikeDeclaration(node);
                checkNodeDeferred(node);
                return getTypeOfSymbol(getSymbolOfNode(node));
            }
            function checkClassExpressionDeferred(node) {
                ts.forEach(node.members, checkSourceElement);
                registerForUnusedIdentifiersCheck(node);
            }
            function checkClassDeclaration(node) {
                if (!node.name && !ts.hasModifier(node, 512 /* Default */)) {
                    grammarErrorOnFirstToken(node, ts.Diagnostics.A_class_declaration_without_the_default_modifier_must_have_a_name);
                }
                checkClassLikeDeclaration(node);
                ts.forEach(node.members, checkSourceElement);
                registerForUnusedIdentifiersCheck(node);
            }
            function checkClassLikeDeclaration(node) {
                checkGrammarClassLikeDeclaration(node);
                checkDecorators(node);
                if (node.name) {
                    checkTypeNameIsReserved(node.name, ts.Diagnostics.Class_name_cannot_be_0);
                    checkCollisionWithRequireExportsInGeneratedCode(node, node.name);
                    checkCollisionWithGlobalPromiseInGeneratedCode(node, node.name);
                    if (!(node.flags & 8388608 /* Ambient */)) {
                        checkClassNameCollisionWithObject(node.name);
                    }
                }
                checkTypeParameters(ts.getEffectiveTypeParameterDeclarations(node));
                checkExportsOnMergedDeclarations(node);
                var symbol = getSymbolOfNode(node);
                var type = getDeclaredTypeOfSymbol(symbol);
                var typeWithThis = getTypeWithThisArgument(type);
                var staticType = getTypeOfSymbol(symbol);
                checkTypeParameterListsIdentical(symbol);
                checkClassForDuplicateDeclarations(node);
                // Only check for reserved static identifiers on non-ambient context.
                if (!(node.flags & 8388608 /* Ambient */)) {
                    checkClassForStaticPropertyNameConflicts(node);
                }
                var baseTypeNode = ts.getEffectiveBaseTypeNode(node);
                if (baseTypeNode) {
                    ts.forEach(baseTypeNode.typeArguments, checkSourceElement);
                    if (languageVersion < 2 /* ES2015 */) {
                        checkExternalEmitHelpers(baseTypeNode.parent, 1 /* Extends */);
                    }
                    // check both @extends and extends if both are specified.
                    var extendsNode = ts.getClassExtendsHeritageElement(node);
                    if (extendsNode && extendsNode !== baseTypeNode) {
                        checkExpression(extendsNode.expression);
                    }
                    var baseTypes = getBaseTypes(type);
                    if (baseTypes.length && produceDiagnostics) {
                        var baseType_1 = baseTypes[0];
                        var baseConstructorType = getBaseConstructorTypeOfClass(type);
                        var staticBaseType = getApparentType(baseConstructorType);
                        checkBaseTypeAccessibility(staticBaseType, baseTypeNode);
                        checkSourceElement(baseTypeNode.expression);
                        if (ts.some(baseTypeNode.typeArguments)) {
                            ts.forEach(baseTypeNode.typeArguments, checkSourceElement);
                            for (var _i = 0, _a = getConstructorsForTypeArguments(staticBaseType, baseTypeNode.typeArguments, baseTypeNode); _i < _a.length; _i++) {
                                var constructor = _a[_i];
                                if (!checkTypeArgumentConstraints(baseTypeNode, constructor.typeParameters)) {
                                    break;
                                }
                            }
                        }
                        var baseWithThis = getTypeWithThisArgument(baseType_1, type.thisType);
                        if (!checkTypeAssignableTo(typeWithThis, baseWithThis, /*errorNode*/ undefined)) {
                            issueMemberSpecificError(node, typeWithThis, baseWithThis, ts.Diagnostics.Class_0_incorrectly_extends_base_class_1);
                        }
                        else {
                            // Report static side error only when instance type is assignable
                            checkTypeAssignableTo(staticType, getTypeWithoutSignatures(staticBaseType), node.name || node, ts.Diagnostics.Class_static_side_0_incorrectly_extends_base_class_static_side_1);
                        }
                        if (baseConstructorType.flags & 8650752 /* TypeVariable */ && !isMixinConstructorType(staticType)) {
                            error(node.name || node, ts.Diagnostics.A_mixin_class_must_have_a_constructor_with_a_single_rest_parameter_of_type_any);
                        }
                        if (!(staticBaseType.symbol && staticBaseType.symbol.flags & 32 /* Class */) && !(baseConstructorType.flags & 8650752 /* TypeVariable */)) {
                            // When the static base type is a "class-like" constructor function (but not actually a class), we verify
                            // that all instantiated base constructor signatures return the same type.
                            var constructors = getInstantiatedConstructorsForTypeArguments(staticBaseType, baseTypeNode.typeArguments, baseTypeNode);
                            if (ts.forEach(constructors, function (sig) { return !isJSConstructor(sig.declaration) && !isTypeIdenticalTo(getReturnTypeOfSignature(sig), baseType_1); })) {
                                error(baseTypeNode.expression, ts.Diagnostics.Base_constructors_must_all_have_the_same_return_type);
                            }
                        }
                        checkKindsOfPropertyMemberOverrides(type, baseType_1);
                    }
                }
                var implementedTypeNodes = ts.getClassImplementsHeritageClauseElements(node);
                if (implementedTypeNodes) {
                    for (var _b = 0, implementedTypeNodes_1 = implementedTypeNodes; _b < implementedTypeNodes_1.length; _b++) {
                        var typeRefNode = implementedTypeNodes_1[_b];
                        if (!ts.isEntityNameExpression(typeRefNode.expression)) {
                            error(typeRefNode.expression, ts.Diagnostics.A_class_can_only_implement_an_identifier_Slashqualified_name_with_optional_type_arguments);
                        }
                        checkTypeReferenceNode(typeRefNode);
                        if (produceDiagnostics) {
                            var t = getTypeFromTypeNode(typeRefNode);
                            if (t !== errorType) {
                                if (isValidBaseType(t)) {
                                    var genericDiag = t.symbol && t.symbol.flags & 32 /* Class */ ?
                                        ts.Diagnostics.Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclass :
                                        ts.Diagnostics.Class_0_incorrectly_implements_interface_1;
                                    var baseWithThis = getTypeWithThisArgument(t, type.thisType);
                                    if (!checkTypeAssignableTo(typeWithThis, baseWithThis, /*errorNode*/ undefined)) {
                                        issueMemberSpecificError(node, typeWithThis, baseWithThis, genericDiag);
                                    }
                                }
                                else {
                                    error(typeRefNode, ts.Diagnostics.A_class_can_only_implement_an_object_type_or_intersection_of_object_types_with_statically_known_members);
                                }
                            }
                        }
                    }
                }
                if (produceDiagnostics) {
                    checkIndexConstraints(type);
                    checkTypeForDuplicateIndexSignatures(node);
                    checkPropertyInitialization(node);
                }
            }
            function issueMemberSpecificError(node, typeWithThis, baseWithThis, broadDiag) {
                // iterate over all implemented properties and issue errors on each one which isn't compatible, rather than the class as a whole, if possible
                var issuedMemberError = false;
                var _loop_17 = function (member) {
                    if (ts.hasStaticModifier(member)) {
                        return "continue";
                    }
                    var declaredProp = member.name && getSymbolAtLocation(member.name) || getSymbolAtLocation(member);
                    if (declaredProp) {
                        var prop = getPropertyOfType(typeWithThis, declaredProp.escapedName);
                        var baseProp = getPropertyOfType(baseWithThis, declaredProp.escapedName);
                        if (prop && baseProp) {
                            var rootChain = function () {
                                return ts.chainDiagnosticMessages(
                                /*details*/ undefined, ts.Diagnostics.Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2, symbolToString(declaredProp), typeToString(typeWithThis), typeToString(baseWithThis));
                            };
                            if (!checkTypeAssignableTo(getTypeOfSymbol(prop), getTypeOfSymbol(baseProp), member.name || member, /*message*/ undefined, rootChain)) {
                                issuedMemberError = true;
                            }
                        }
                    }
                };
                for (var _i = 0, _a = node.members; _i < _a.length; _i++) {
                    var member = _a[_i];
                    _loop_17(member);
                }
                if (!issuedMemberError) {
                    // check again with diagnostics to generate a less-specific error
                    checkTypeAssignableTo(typeWithThis, baseWithThis, node.name || node, broadDiag);
                }
            }
            function checkBaseTypeAccessibility(type, node) {
                var signatures = getSignaturesOfType(type, 1 /* Construct */);
                if (signatures.length) {
                    var declaration = signatures[0].declaration;
                    if (declaration && ts.hasModifier(declaration, 8 /* Private */)) {
                        var typeClassDeclaration = ts.getClassLikeDeclarationOfSymbol(type.symbol);
                        if (!isNodeWithinClass(node, typeClassDeclaration)) {
                            error(node, ts.Diagnostics.Cannot_extend_a_class_0_Class_constructor_is_marked_as_private, getFullyQualifiedName(type.symbol));
                        }
                    }
                }
            }
            function getTargetSymbol(s) {
                // if symbol is instantiated its flags are not copied from the 'target'
                // so we'll need to get back original 'target' symbol to work with correct set of flags
                return ts.getCheckFlags(s) & 1 /* Instantiated */ ? s.target : s;
            }
            function getClassOrInterfaceDeclarationsOfSymbol(symbol) {
                return ts.filter(symbol.declarations, function (d) {
                    return d.kind === 245 /* ClassDeclaration */ || d.kind === 246 /* InterfaceDeclaration */;
                });
            }
            function checkKindsOfPropertyMemberOverrides(type, baseType) {
                // TypeScript 1.0 spec (April 2014): 8.2.3
                // A derived class inherits all members from its base class it doesn't override.
                // Inheritance means that a derived class implicitly contains all non - overridden members of the base class.
                // Both public and private property members are inherited, but only public property members can be overridden.
                // A property member in a derived class is said to override a property member in a base class
                // when the derived class property member has the same name and kind(instance or static)
                // as the base class property member.
                // The type of an overriding property member must be assignable(section 3.8.4)
                // to the type of the overridden property member, or otherwise a compile - time error occurs.
                // Base class instance member functions can be overridden by derived class instance member functions,
                // but not by other kinds of members.
                // Base class instance member variables and accessors can be overridden by
                // derived class instance member variables and accessors, but not by other kinds of members.
                // NOTE: assignability is checked in checkClassDeclaration
                var baseProperties = getPropertiesOfType(baseType);
                basePropertyCheck: for (var _i = 0, baseProperties_1 = baseProperties; _i < baseProperties_1.length; _i++) {
                    var baseProperty = baseProperties_1[_i];
                    var base = getTargetSymbol(baseProperty);
                    if (base.flags & 4194304 /* Prototype */) {
                        continue;
                    }
                    var baseSymbol = getPropertyOfObjectType(type, base.escapedName);
                    if (!baseSymbol) {
                        continue;
                    }
                    var derived = getTargetSymbol(baseSymbol);
                    var baseDeclarationFlags = ts.getDeclarationModifierFlagsFromSymbol(base);
                    ts.Debug.assert(!!derived, "derived should point to something, even if it is the base class' declaration.");
                    // In order to resolve whether the inherited method was overridden in the base class or not,
                    // we compare the Symbols obtained. Since getTargetSymbol returns the symbol on the *uninstantiated*
                    // type declaration, derived and base resolve to the same symbol even in the case of generic classes.
                    if (derived === base) {
                        // derived class inherits base without override/redeclaration
                        var derivedClassDecl = ts.getClassLikeDeclarationOfSymbol(type.symbol);
                        // It is an error to inherit an abstract member without implementing it or being declared abstract.
                        // If there is no declaration for the derived class (as in the case of class expressions),
                        // then the class cannot be declared abstract.
                        if (baseDeclarationFlags & 128 /* Abstract */ && (!derivedClassDecl || !ts.hasModifier(derivedClassDecl, 128 /* Abstract */))) {
                            // Searches other base types for a declaration that would satisfy the inherited abstract member.
                            // (The class may have more than one base type via declaration merging with an interface with the
                            // same name.)
                            for (var _a = 0, _b = getBaseTypes(type); _a < _b.length; _a++) {
                                var otherBaseType = _b[_a];
                                if (otherBaseType === baseType)
                                    continue;
                                var baseSymbol_1 = getPropertyOfObjectType(otherBaseType, base.escapedName);
                                var derivedElsewhere = baseSymbol_1 && getTargetSymbol(baseSymbol_1);
                                if (derivedElsewhere && derivedElsewhere !== base) {
                                    continue basePropertyCheck;
                                }
                            }
                            if (derivedClassDecl.kind === 214 /* ClassExpression */) {
                                error(derivedClassDecl, ts.Diagnostics.Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1, symbolToString(baseProperty), typeToString(baseType));
                            }
                            else {
                                error(derivedClassDecl, ts.Diagnostics.Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2, typeToString(type), symbolToString(baseProperty), typeToString(baseType));
                            }
                        }
                    }
                    else {
                        // derived overrides base.
                        var derivedDeclarationFlags = ts.getDeclarationModifierFlagsFromSymbol(derived);
                        if (baseDeclarationFlags & 8 /* Private */ || derivedDeclarationFlags & 8 /* Private */) {
                            // either base or derived property is private - not override, skip it
                            continue;
                        }
                        var errorMessage = void 0;
                        var basePropertyFlags = base.flags & 98308 /* PropertyOrAccessor */;
                        var derivedPropertyFlags = derived.flags & 98308 /* PropertyOrAccessor */;
                        if (basePropertyFlags && derivedPropertyFlags) {
                            // property/accessor is overridden with property/accessor
                            if (!compilerOptions.useDefineForClassFields
                                || baseDeclarationFlags & 128 /* Abstract */ && !(base.valueDeclaration && ts.isPropertyDeclaration(base.valueDeclaration) && base.valueDeclaration.initializer)
                                || base.valueDeclaration && base.valueDeclaration.parent.kind === 246 /* InterfaceDeclaration */
                                || derived.valueDeclaration && ts.isBinaryExpression(derived.valueDeclaration)) {
                                // when the base property is abstract or from an interface, base/derived flags don't need to match
                                // same when the derived property is from an assignment
                                continue;
                            }
                            var overriddenInstanceProperty = basePropertyFlags !== 4 /* Property */ && derivedPropertyFlags === 4 /* Property */;
                            var overriddenInstanceAccessor = basePropertyFlags === 4 /* Property */ && derivedPropertyFlags !== 4 /* Property */;
                            if (overriddenInstanceProperty || overriddenInstanceAccessor) {
                                var errorMessage_1 = overriddenInstanceProperty ?
                                    ts.Diagnostics._0_is_defined_as_an_accessor_in_class_1_but_is_overridden_here_in_2_as_an_instance_property :
                                    ts.Diagnostics._0_is_defined_as_a_property_in_class_1_but_is_overridden_here_in_2_as_an_accessor;
                                error(ts.getNameOfDeclaration(derived.valueDeclaration) || derived.valueDeclaration, errorMessage_1, symbolToString(base), typeToString(baseType), typeToString(type));
                            }
                            else {
                                var uninitialized = ts.find(derived.declarations, function (d) { return d.kind === 159 /* PropertyDeclaration */ && !d.initializer; });
                                if (uninitialized
                                    && !(derived.flags & 33554432 /* Transient */)
                                    && !(baseDeclarationFlags & 128 /* Abstract */)
                                    && !(derivedDeclarationFlags & 128 /* Abstract */)
                                    && !derived.declarations.some(function (d) { return !!(d.flags & 8388608 /* Ambient */); })) {
                                    var constructor = findConstructorDeclaration(ts.getClassLikeDeclarationOfSymbol(type.symbol));
                                    var propName = uninitialized.name;
                                    if (uninitialized.exclamationToken
                                        || !constructor
                                        || !ts.isIdentifier(propName)
                                        || !strictNullChecks
                                        || !isPropertyInitializedInConstructor(propName, type, constructor)) {
                                        var errorMessage_2 = ts.Diagnostics.Property_0_will_overwrite_the_base_property_in_1_If_this_is_intentional_add_an_initializer_Otherwise_add_a_declare_modifier_or_remove_the_redundant_declaration;
                                        error(ts.getNameOfDeclaration(derived.valueDeclaration) || derived.valueDeclaration, errorMessage_2, symbolToString(base), typeToString(baseType));
                                    }
                                }
                            }
                            // correct case
                            continue;
                        }
                        else if (isPrototypeProperty(base)) {
                            if (isPrototypeProperty(derived) || derived.flags & 4 /* Property */) {
                                // method is overridden with method or property -- correct case
                                continue;
                            }
                            else {
                                ts.Debug.assert(!!(derived.flags & 98304 /* Accessor */));
                                errorMessage = ts.Diagnostics.Class_0_defines_instance_member_function_1_but_extended_class_2_defines_it_as_instance_member_accessor;
                            }
                        }
                        else if (base.flags & 98304 /* Accessor */) {
                            errorMessage = ts.Diagnostics.Class_0_defines_instance_member_accessor_1_but_extended_class_2_defines_it_as_instance_member_function;
                        }
                        else {
                            errorMessage = ts.Diagnostics.Class_0_defines_instance_member_property_1_but_extended_class_2_defines_it_as_instance_member_function;
                        }
                        error(ts.getNameOfDeclaration(derived.valueDeclaration) || derived.valueDeclaration, errorMessage, typeToString(baseType), symbolToString(base), typeToString(type));
                    }
                }
            }
            function checkInheritedPropertiesAreIdentical(type, typeNode) {
                var baseTypes = getBaseTypes(type);
                if (baseTypes.length < 2) {
                    return true;
                }
                var seen = ts.createUnderscoreEscapedMap();
                ts.forEach(resolveDeclaredMembers(type).declaredProperties, function (p) { seen.set(p.escapedName, { prop: p, containingType: type }); });
                var ok = true;
                for (var _i = 0, baseTypes_2 = baseTypes; _i < baseTypes_2.length; _i++) {
                    var base = baseTypes_2[_i];
                    var properties = getPropertiesOfType(getTypeWithThisArgument(base, type.thisType));
                    for (var _a = 0, properties_4 = properties; _a < properties_4.length; _a++) {
                        var prop = properties_4[_a];
                        var existing = seen.get(prop.escapedName);
                        if (!existing) {
                            seen.set(prop.escapedName, { prop: prop, containingType: base });
                        }
                        else {
                            var isInheritedProperty = existing.containingType !== type;
                            if (isInheritedProperty && !isPropertyIdenticalTo(existing.prop, prop)) {
                                ok = false;
                                var typeName1 = typeToString(existing.containingType);
                                var typeName2 = typeToString(base);
                                var errorInfo = ts.chainDiagnosticMessages(/*details*/ undefined, ts.Diagnostics.Named_property_0_of_types_1_and_2_are_not_identical, symbolToString(prop), typeName1, typeName2);
                                errorInfo = ts.chainDiagnosticMessages(errorInfo, ts.Diagnostics.Interface_0_cannot_simultaneously_extend_types_1_and_2, typeToString(type), typeName1, typeName2);
                                diagnostics.add(ts.createDiagnosticForNodeFromMessageChain(typeNode, errorInfo));
                            }
                        }
                    }
                }
                return ok;
            }
            function checkPropertyInitialization(node) {
                if (!strictNullChecks || !strictPropertyInitialization || node.flags & 8388608 /* Ambient */) {
                    return;
                }
                var constructor = findConstructorDeclaration(node);
                for (var _i = 0, _a = node.members; _i < _a.length; _i++) {
                    var member = _a[_i];
                    if (ts.getModifierFlags(member) & 2 /* Ambient */) {
                        continue;
                    }
                    if (isInstancePropertyWithoutInitializer(member)) {
                        var propName = member.name;
                        if (ts.isIdentifier(propName) || ts.isPrivateIdentifier(propName)) {
                            var type = getTypeOfSymbol(getSymbolOfNode(member));
                            if (!(type.flags & 3 /* AnyOrUnknown */ || getFalsyFlags(type) & 32768 /* Undefined */)) {
                                if (!constructor || !isPropertyInitializedInConstructor(propName, type, constructor)) {
                                    error(member.name, ts.Diagnostics.Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor, ts.declarationNameToString(propName));
                                }
                            }
                        }
                    }
                }
            }
            function isInstancePropertyWithoutInitializer(node) {
                return node.kind === 159 /* PropertyDeclaration */ &&
                    !ts.hasModifier(node, 32 /* Static */ | 128 /* Abstract */) &&
                    !node.exclamationToken &&
                    !node.initializer;
            }
            function isPropertyInitializedInConstructor(propName, propType, constructor) {
                var reference = ts.createPropertyAccess(ts.createThis(), propName);
                reference.expression.parent = reference;
                reference.parent = constructor;
                reference.flowNode = constructor.returnFlowNode;
                var flowType = getFlowTypeOfReference(reference, propType, getOptionalType(propType));
                return !(getFalsyFlags(flowType) & 32768 /* Undefined */);
            }
            function checkInterfaceDeclaration(node) {
                // Grammar checking
                if (!checkGrammarDecoratorsAndModifiers(node))
                    checkGrammarInterfaceDeclaration(node);
                checkTypeParameters(node.typeParameters);
                if (produceDiagnostics) {
                    checkTypeNameIsReserved(node.name, ts.Diagnostics.Interface_name_cannot_be_0);
                    checkExportsOnMergedDeclarations(node);
                    var symbol = getSymbolOfNode(node);
                    checkTypeParameterListsIdentical(symbol);
                    // Only check this symbol once
                    var firstInterfaceDecl = ts.getDeclarationOfKind(symbol, 246 /* InterfaceDeclaration */);
                    if (node === firstInterfaceDecl) {
                        var type = getDeclaredTypeOfSymbol(symbol);
                        var typeWithThis = getTypeWithThisArgument(type);
                        // run subsequent checks only if first set succeeded
                        if (checkInheritedPropertiesAreIdentical(type, node.name)) {
                            for (var _i = 0, _a = getBaseTypes(type); _i < _a.length; _i++) {
                                var baseType = _a[_i];
                                checkTypeAssignableTo(typeWithThis, getTypeWithThisArgument(baseType, type.thisType), node.name, ts.Diagnostics.Interface_0_incorrectly_extends_interface_1);
                            }
                            checkIndexConstraints(type);
                        }
                    }
                    checkObjectTypeForDuplicateDeclarations(node);
                }
                ts.forEach(ts.getInterfaceBaseTypeNodes(node), function (heritageElement) {
                    if (!ts.isEntityNameExpression(heritageElement.expression)) {
                        error(heritageElement.expression, ts.Diagnostics.An_interface_can_only_extend_an_identifier_Slashqualified_name_with_optional_type_arguments);
                    }
                    checkTypeReferenceNode(heritageElement);
                });
                ts.forEach(node.members, checkSourceElement);
                if (produceDiagnostics) {
                    checkTypeForDuplicateIndexSignatures(node);
                    registerForUnusedIdentifiersCheck(node);
                }
            }
            function checkTypeAliasDeclaration(node) {
                // Grammar checking
                checkGrammarDecoratorsAndModifiers(node);
                checkTypeNameIsReserved(node.name, ts.Diagnostics.Type_alias_name_cannot_be_0);
                checkExportsOnMergedDeclarations(node);
                checkTypeParameters(node.typeParameters);
                checkSourceElement(node.type);
                registerForUnusedIdentifiersCheck(node);
            }
            function computeEnumMemberValues(node) {
                var nodeLinks = getNodeLinks(node);
                if (!(nodeLinks.flags & 16384 /* EnumValuesComputed */)) {
                    nodeLinks.flags |= 16384 /* EnumValuesComputed */;
                    var autoValue = 0;
                    for (var _i = 0, _a = node.members; _i < _a.length; _i++) {
                        var member = _a[_i];
                        var value = computeMemberValue(member, autoValue);
                        getNodeLinks(member).enumMemberValue = value;
                        autoValue = typeof value === "number" ? value + 1 : undefined;
                    }
                }
            }
            function computeMemberValue(member, autoValue) {
                if (isComputedNonLiteralName(member.name)) {
                    error(member.name, ts.Diagnostics.Computed_property_names_are_not_allowed_in_enums);
                }
                else {
                    var text = ts.getTextOfPropertyName(member.name);
                    if (isNumericLiteralName(text) && !isInfinityOrNaNString(text)) {
                        error(member.name, ts.Diagnostics.An_enum_member_cannot_have_a_numeric_name);
                    }
                }
                if (member.initializer) {
                    return computeConstantValue(member);
                }
                // In ambient non-const numeric enum declarations, enum members without initializers are
                // considered computed members (as opposed to having auto-incremented values).
                if (member.parent.flags & 8388608 /* Ambient */ && !ts.isEnumConst(member.parent) && getEnumKind(getSymbolOfNode(member.parent)) === 0 /* Numeric */) {
                    return undefined;
                }
                // If the member declaration specifies no value, the member is considered a constant enum member.
                // If the member is the first member in the enum declaration, it is assigned the value zero.
                // Otherwise, it is assigned the value of the immediately preceding member plus one, and an error
                // occurs if the immediately preceding member is not a constant enum member.
                if (autoValue !== undefined) {
                    return autoValue;
                }
                error(member.name, ts.Diagnostics.Enum_member_must_have_initializer);
                return undefined;
            }
            function computeConstantValue(member) {
                var enumKind = getEnumKind(getSymbolOfNode(member.parent));
                var isConstEnum = ts.isEnumConst(member.parent);
                var initializer = member.initializer;
                var value = enumKind === 1 /* Literal */ && !isLiteralEnumMember(member) ? undefined : evaluate(initializer);
                if (value !== undefined) {
                    if (isConstEnum && typeof value === "number" && !isFinite(value)) {
                        error(initializer, isNaN(value) ?
                            ts.Diagnostics.const_enum_member_initializer_was_evaluated_to_disallowed_value_NaN :
                            ts.Diagnostics.const_enum_member_initializer_was_evaluated_to_a_non_finite_value);
                    }
                }
                else if (enumKind === 1 /* Literal */) {
                    error(initializer, ts.Diagnostics.Computed_values_are_not_permitted_in_an_enum_with_string_valued_members);
                    return 0;
                }
                else if (isConstEnum) {
                    error(initializer, ts.Diagnostics.const_enum_member_initializers_can_only_contain_literal_values_and_other_computed_enum_values);
                }
                else if (member.parent.flags & 8388608 /* Ambient */) {
                    error(initializer, ts.Diagnostics.In_ambient_enum_declarations_member_initializer_must_be_constant_expression);
                }
                else {
                    // Only here do we need to check that the initializer is assignable to the enum type.
                    checkTypeAssignableTo(checkExpression(initializer), getDeclaredTypeOfSymbol(getSymbolOfNode(member.parent)), initializer, /*headMessage*/ undefined);
                }
                return value;
                function evaluate(expr) {
                    switch (expr.kind) {
                        case 207 /* PrefixUnaryExpression */:
                            var value_2 = evaluate(expr.operand);
                            if (typeof value_2 === "number") {
                                switch (expr.operator) {
                                    case 39 /* PlusToken */: return value_2;
                                    case 40 /* MinusToken */: return -value_2;
                                    case 54 /* TildeToken */: return ~value_2;
                                }
                            }
                            break;
                        case 209 /* BinaryExpression */:
                            var left = evaluate(expr.left);
                            var right = evaluate(expr.right);
                            if (typeof left === "number" && typeof right === "number") {
                                switch (expr.operatorToken.kind) {
                                    case 51 /* BarToken */: return left | right;
                                    case 50 /* AmpersandToken */: return left & right;
                                    case 48 /* GreaterThanGreaterThanToken */: return left >> right;
                                    case 49 /* GreaterThanGreaterThanGreaterThanToken */: return left >>> right;
                                    case 47 /* LessThanLessThanToken */: return left << right;
                                    case 52 /* CaretToken */: return left ^ right;
                                    case 41 /* AsteriskToken */: return left * right;
                                    case 43 /* SlashToken */: return left / right;
                                    case 39 /* PlusToken */: return left + right;
                                    case 40 /* MinusToken */: return left - right;
                                    case 44 /* PercentToken */: return left % right;
                                    case 42 /* AsteriskAsteriskToken */: return Math.pow(left, right);
                                }
                            }
                            else if (typeof left === "string" && typeof right === "string" && expr.operatorToken.kind === 39 /* PlusToken */) {
                                return left + right;
                            }
                            break;
                        case 10 /* StringLiteral */:
                        case 14 /* NoSubstitutionTemplateLiteral */:
                            return expr.text;
                        case 8 /* NumericLiteral */:
                            checkGrammarNumericLiteral(expr);
                            return +expr.text;
                        case 200 /* ParenthesizedExpression */:
                            return evaluate(expr.expression);
                        case 75 /* Identifier */:
                            var identifier = expr;
                            if (isInfinityOrNaNString(identifier.escapedText)) {
                                return +(identifier.escapedText);
                            }
                            return ts.nodeIsMissing(expr) ? 0 : evaluateEnumMember(expr, getSymbolOfNode(member.parent), identifier.escapedText);
                        case 195 /* ElementAccessExpression */:
                        case 194 /* PropertyAccessExpression */:
                            var ex = expr;
                            if (isConstantMemberAccess(ex)) {
                                var type = getTypeOfExpression(ex.expression);
                                if (type.symbol && type.symbol.flags & 384 /* Enum */) {
                                    var name = void 0;
                                    if (ex.kind === 194 /* PropertyAccessExpression */) {
                                        name = ex.name.escapedText;
                                    }
                                    else {
                                        name = ts.escapeLeadingUnderscores(ts.cast(ex.argumentExpression, ts.isLiteralExpression).text);
                                    }
                                    return evaluateEnumMember(expr, type.symbol, name);
                                }
                            }
                            break;
                    }
                    return undefined;
                }
                function evaluateEnumMember(expr, enumSymbol, name) {
                    var memberSymbol = enumSymbol.exports.get(name);
                    if (memberSymbol) {
                        var declaration = memberSymbol.valueDeclaration;
                        if (declaration !== member) {
                            if (isBlockScopedNameDeclaredBeforeUse(declaration, member)) {
                                return getEnumMemberValue(declaration);
                            }
                            error(expr, ts.Diagnostics.A_member_initializer_in_a_enum_declaration_cannot_reference_members_declared_after_it_including_members_defined_in_other_enums);
                            return 0;
                        }
                    }
                    return undefined;
                }
            }
            function isConstantMemberAccess(node) {
                return node.kind === 75 /* Identifier */ ||
                    node.kind === 194 /* PropertyAccessExpression */ && isConstantMemberAccess(node.expression) ||
                    node.kind === 195 /* ElementAccessExpression */ && isConstantMemberAccess(node.expression) &&
                        ts.isStringLiteralLike(node.argumentExpression);
            }
            function checkEnumDeclaration(node) {
                if (!produceDiagnostics) {
                    return;
                }
                // Grammar checking
                checkGrammarDecoratorsAndModifiers(node);
                checkTypeNameIsReserved(node.name, ts.Diagnostics.Enum_name_cannot_be_0);
                checkCollisionWithRequireExportsInGeneratedCode(node, node.name);
                checkCollisionWithGlobalPromiseInGeneratedCode(node, node.name);
                checkExportsOnMergedDeclarations(node);
                node.members.forEach(checkEnumMember);
                computeEnumMemberValues(node);
                // Spec 2014 - Section 9.3:
                // It isn't possible for one enum declaration to continue the automatic numbering sequence of another,
                // and when an enum type has multiple declarations, only one declaration is permitted to omit a value
                // for the first member.
                //
                // Only perform this check once per symbol
                var enumSymbol = getSymbolOfNode(node);
                var firstDeclaration = ts.getDeclarationOfKind(enumSymbol, node.kind);
                if (node === firstDeclaration) {
                    if (enumSymbol.declarations.length > 1) {
                        var enumIsConst_1 = ts.isEnumConst(node);
                        // check that const is placed\omitted on all enum declarations
                        ts.forEach(enumSymbol.declarations, function (decl) {
                            if (ts.isEnumDeclaration(decl) && ts.isEnumConst(decl) !== enumIsConst_1) {
                                error(ts.getNameOfDeclaration(decl), ts.Diagnostics.Enum_declarations_must_all_be_const_or_non_const);
                            }
                        });
                    }
                    var seenEnumMissingInitialInitializer_1 = false;
                    ts.forEach(enumSymbol.declarations, function (declaration) {
                        // return true if we hit a violation of the rule, false otherwise
                        if (declaration.kind !== 248 /* EnumDeclaration */) {
                            return false;
                        }
                        var enumDeclaration = declaration;
                        if (!enumDeclaration.members.length) {
                            return false;
                        }
                        var firstEnumMember = enumDeclaration.members[0];
                        if (!firstEnumMember.initializer) {
                            if (seenEnumMissingInitialInitializer_1) {
                                error(firstEnumMember.name, ts.Diagnostics.In_an_enum_with_multiple_declarations_only_one_declaration_can_omit_an_initializer_for_its_first_enum_element);
                            }
                            else {
                                seenEnumMissingInitialInitializer_1 = true;
                            }
                        }
                    });
                }
            }
            function checkEnumMember(node) {
                if (ts.isPrivateIdentifier(node.name)) {
                    error(node, ts.Diagnostics.An_enum_member_cannot_be_named_with_a_private_identifier);
                }
            }
            function getFirstNonAmbientClassOrFunctionDeclaration(symbol) {
                var declarations = symbol.declarations;
                for (var _i = 0, declarations_8 = declarations; _i < declarations_8.length; _i++) {
                    var declaration = declarations_8[_i];
                    if ((declaration.kind === 245 /* ClassDeclaration */ ||
                        (declaration.kind === 244 /* FunctionDeclaration */ && ts.nodeIsPresent(declaration.body))) &&
                        !(declaration.flags & 8388608 /* Ambient */)) {
                        return declaration;
                    }
                }
                return undefined;
            }
            function inSameLexicalScope(node1, node2) {
                var container1 = ts.getEnclosingBlockScopeContainer(node1);
                var container2 = ts.getEnclosingBlockScopeContainer(node2);
                if (isGlobalSourceFile(container1)) {
                    return isGlobalSourceFile(container2);
                }
                else if (isGlobalSourceFile(container2)) {
                    return false;
                }
                else {
                    return container1 === container2;
                }
            }
            function checkModuleDeclaration(node) {
                if (produceDiagnostics) {
                    // Grammar checking
                    var isGlobalAugmentation = ts.isGlobalScopeAugmentation(node);
                    var inAmbientContext = node.flags & 8388608 /* Ambient */;
                    if (isGlobalAugmentation && !inAmbientContext) {
                        error(node.name, ts.Diagnostics.Augmentations_for_the_global_scope_should_have_declare_modifier_unless_they_appear_in_already_ambient_context);
                    }
                    var isAmbientExternalModule = ts.isAmbientModule(node);
                    var contextErrorMessage = isAmbientExternalModule
                        ? ts.Diagnostics.An_ambient_module_declaration_is_only_allowed_at_the_top_level_in_a_file
                        : ts.Diagnostics.A_namespace_declaration_is_only_allowed_in_a_namespace_or_module;
                    if (checkGrammarModuleElementContext(node, contextErrorMessage)) {
                        // If we hit a module declaration in an illegal context, just bail out to avoid cascading errors.
                        return;
                    }
                    if (!checkGrammarDecoratorsAndModifiers(node)) {
                        if (!inAmbientContext && node.name.kind === 10 /* StringLiteral */) {
                            grammarErrorOnNode(node.name, ts.Diagnostics.Only_ambient_modules_can_use_quoted_names);
                        }
                    }
                    if (ts.isIdentifier(node.name)) {
                        checkCollisionWithRequireExportsInGeneratedCode(node, node.name);
                        checkCollisionWithGlobalPromiseInGeneratedCode(node, node.name);
                    }
                    checkExportsOnMergedDeclarations(node);
                    var symbol = getSymbolOfNode(node);
                    // The following checks only apply on a non-ambient instantiated module declaration.
                    if (symbol.flags & 512 /* ValueModule */
                        && !inAmbientContext
                        && symbol.declarations.length > 1
                        && isInstantiatedModule(node, !!compilerOptions.preserveConstEnums || !!compilerOptions.isolatedModules)) {
                        var firstNonAmbientClassOrFunc = getFirstNonAmbientClassOrFunctionDeclaration(symbol);
                        if (firstNonAmbientClassOrFunc) {
                            if (ts.getSourceFileOfNode(node) !== ts.getSourceFileOfNode(firstNonAmbientClassOrFunc)) {
                                error(node.name, ts.Diagnostics.A_namespace_declaration_cannot_be_in_a_different_file_from_a_class_or_function_with_which_it_is_merged);
                            }
                            else if (node.pos < firstNonAmbientClassOrFunc.pos) {
                                error(node.name, ts.Diagnostics.A_namespace_declaration_cannot_be_located_prior_to_a_class_or_function_with_which_it_is_merged);
                            }
                        }
                        // if the module merges with a class declaration in the same lexical scope,
                        // we need to track this to ensure the correct emit.
                        var mergedClass = ts.getDeclarationOfKind(symbol, 245 /* ClassDeclaration */);
                        if (mergedClass &&
                            inSameLexicalScope(node, mergedClass)) {
                            getNodeLinks(node).flags |= 32768 /* LexicalModuleMergesWithClass */;
                        }
                    }
                    if (isAmbientExternalModule) {
                        if (ts.isExternalModuleAugmentation(node)) {
                            // body of the augmentation should be checked for consistency only if augmentation was applied to its target (either global scope or module)
                            // otherwise we'll be swamped in cascading errors.
                            // We can detect if augmentation was applied using following rules:
                            // - augmentation for a global scope is always applied
                            // - augmentation for some external module is applied if symbol for augmentation is merged (it was combined with target module).
                            var checkBody = isGlobalAugmentation || (getSymbolOfNode(node).flags & 33554432 /* Transient */);
                            if (checkBody && node.body) {
                                for (var _i = 0, _a = node.body.statements; _i < _a.length; _i++) {
                                    var statement = _a[_i];
                                    checkModuleAugmentationElement(statement, isGlobalAugmentation);
                                }
                            }
                        }
                        else if (isGlobalSourceFile(node.parent)) {
                            if (isGlobalAugmentation) {
                                error(node.name, ts.Diagnostics.Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_declarations);
                            }
                            else if (ts.isExternalModuleNameRelative(ts.getTextOfIdentifierOrLiteral(node.name))) {
                                error(node.name, ts.Diagnostics.Ambient_module_declaration_cannot_specify_relative_module_name);
                            }
                        }
                        else {
                            if (isGlobalAugmentation) {
                                error(node.name, ts.Diagnostics.Augmentations_for_the_global_scope_can_only_be_directly_nested_in_external_modules_or_ambient_module_declarations);
                            }
                            else {
                                // Node is not an augmentation and is not located on the script level.
                                // This means that this is declaration of ambient module that is located in other module or namespace which is prohibited.
                                error(node.name, ts.Diagnostics.Ambient_modules_cannot_be_nested_in_other_modules_or_namespaces);
                            }
                        }
                    }
                }
                if (node.body) {
                    checkSourceElement(node.body);
                    if (!ts.isGlobalScopeAugmentation(node)) {
                        registerForUnusedIdentifiersCheck(node);
                    }
                }
            }
            function checkModuleAugmentationElement(node, isGlobalAugmentation) {
                switch (node.kind) {
                    case 225 /* VariableStatement */:
                        // error each individual name in variable statement instead of marking the entire variable statement
                        for (var _i = 0, _a = node.declarationList.declarations; _i < _a.length; _i++) {
                            var decl = _a[_i];
                            checkModuleAugmentationElement(decl, isGlobalAugmentation);
                        }
                        break;
                    case 259 /* ExportAssignment */:
                    case 260 /* ExportDeclaration */:
                        grammarErrorOnFirstToken(node, ts.Diagnostics.Exports_and_export_assignments_are_not_permitted_in_module_augmentations);
                        break;
                    case 253 /* ImportEqualsDeclaration */:
                    case 254 /* ImportDeclaration */:
                        grammarErrorOnFirstToken(node, ts.Diagnostics.Imports_are_not_permitted_in_module_augmentations_Consider_moving_them_to_the_enclosing_external_module);
                        break;
                    case 191 /* BindingElement */:
                    case 242 /* VariableDeclaration */:
                        var name = node.name;
                        if (ts.isBindingPattern(name)) {
                            for (var _b = 0, _c = name.elements; _b < _c.length; _b++) {
                                var el = _c[_b];
                                // mark individual names in binding pattern
                                checkModuleAugmentationElement(el, isGlobalAugmentation);
                            }
                            break;
                        }
                    // falls through
                    case 245 /* ClassDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 244 /* FunctionDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                    case 249 /* ModuleDeclaration */:
                    case 247 /* TypeAliasDeclaration */:
                        if (isGlobalAugmentation) {
                            return;
                        }
                        var symbol = getSymbolOfNode(node);
                        if (symbol) {
                            // module augmentations cannot introduce new names on the top level scope of the module
                            // this is done it two steps
                            // 1. quick check - if symbol for node is not merged - this is local symbol to this augmentation - report error
                            // 2. main check - report error if value declaration of the parent symbol is module augmentation)
                            var reportError = !(symbol.flags & 33554432 /* Transient */);
                            if (!reportError) {
                                // symbol should not originate in augmentation
                                reportError = !!symbol.parent && ts.isExternalModuleAugmentation(symbol.parent.declarations[0]);
                            }
                        }
                        break;
                }
            }
            function getFirstNonModuleExportsIdentifier(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                        return node;
                    case 153 /* QualifiedName */:
                        do {
                            node = node.left;
                        } while (node.kind !== 75 /* Identifier */);
                        return node;
                    case 194 /* PropertyAccessExpression */:
                        do {
                            if (ts.isModuleExportsAccessExpression(node.expression) && !ts.isPrivateIdentifier(node.name)) {
                                return node.name;
                            }
                            node = node.expression;
                        } while (node.kind !== 75 /* Identifier */);
                        return node;
                }
            }
            function checkExternalImportOrExportDeclaration(node) {
                var moduleName = ts.getExternalModuleName(node);
                if (!moduleName || ts.nodeIsMissing(moduleName)) {
                    // Should be a parse error.
                    return false;
                }
                if (!ts.isStringLiteral(moduleName)) {
                    error(moduleName, ts.Diagnostics.String_literal_expected);
                    return false;
                }
                var inAmbientExternalModule = node.parent.kind === 250 /* ModuleBlock */ && ts.isAmbientModule(node.parent.parent);
                if (node.parent.kind !== 290 /* SourceFile */ && !inAmbientExternalModule) {
                    error(moduleName, node.kind === 260 /* ExportDeclaration */ ?
                        ts.Diagnostics.Export_declarations_are_not_permitted_in_a_namespace :
                        ts.Diagnostics.Import_declarations_in_a_namespace_cannot_reference_a_module);
                    return false;
                }
                if (inAmbientExternalModule && ts.isExternalModuleNameRelative(moduleName.text)) {
                    // we have already reported errors on top level imports\exports in external module augmentations in checkModuleDeclaration
                    // no need to do this again.
                    if (!isTopLevelInExternalModuleAugmentation(node)) {
                        // TypeScript 1.0 spec (April 2013): 12.1.6
                        // An ExternalImportDeclaration in an AmbientExternalModuleDeclaration may reference
                        // other external modules only through top - level external module names.
                        // Relative external module names are not permitted.
                        error(node, ts.Diagnostics.Import_or_export_declaration_in_an_ambient_module_declaration_cannot_reference_module_through_relative_module_name);
                        return false;
                    }
                }
                return true;
            }
            function checkAliasSymbol(node) {
                var symbol = getSymbolOfNode(node);
                var target = resolveAlias(symbol);
                var shouldSkipWithJSExpandoTargets = symbol.flags & 67108864 /* Assignment */;
                if (!shouldSkipWithJSExpandoTargets && target !== unknownSymbol) {
                    // For external modules symbol represents local symbol for an alias.
                    // This local symbol will merge any other local declarations (excluding other aliases)
                    // and symbol.flags will contains combined representation for all merged declaration.
                    // Based on symbol.flags we can compute a set of excluded meanings (meaning that resolved alias should not have,
                    // otherwise it will conflict with some local declaration). Note that in addition to normal flags we include matching SymbolFlags.Export*
                    // in order to prevent collisions with declarations that were exported from the current module (they still contribute to local names).
                    symbol = getMergedSymbol(symbol.exportSymbol || symbol);
                    var excludedMeanings = (symbol.flags & (111551 /* Value */ | 1048576 /* ExportValue */) ? 111551 /* Value */ : 0) |
                        (symbol.flags & 788968 /* Type */ ? 788968 /* Type */ : 0) |
                        (symbol.flags & 1920 /* Namespace */ ? 1920 /* Namespace */ : 0);
                    if (target.flags & excludedMeanings) {
                        var message = node.kind === 263 /* ExportSpecifier */ ?
                            ts.Diagnostics.Export_declaration_conflicts_with_exported_declaration_of_0 :
                            ts.Diagnostics.Import_declaration_conflicts_with_local_declaration_of_0;
                        error(node, message, symbolToString(symbol));
                    }
                    // Don't allow to re-export something with no value side when `--isolatedModules` is set.
                    if (compilerOptions.isolatedModules
                        && node.kind === 263 /* ExportSpecifier */
                        && !node.parent.parent.isTypeOnly
                        && !(target.flags & 111551 /* Value */)
                        && !(node.flags & 8388608 /* Ambient */)) {
                        error(node, ts.Diagnostics.Re_exporting_a_type_when_the_isolatedModules_flag_is_provided_requires_using_export_type);
                    }
                }
            }
            function checkImportBinding(node) {
                checkCollisionWithRequireExportsInGeneratedCode(node, node.name);
                checkCollisionWithGlobalPromiseInGeneratedCode(node, node.name);
                checkAliasSymbol(node);
            }
            function checkImportDeclaration(node) {
                if (checkGrammarModuleElementContext(node, ts.Diagnostics.An_import_declaration_can_only_be_used_in_a_namespace_or_module)) {
                    // If we hit an import declaration in an illegal context, just bail out to avoid cascading errors.
                    return;
                }
                if (!checkGrammarDecoratorsAndModifiers(node) && ts.hasModifiers(node)) {
                    grammarErrorOnFirstToken(node, ts.Diagnostics.An_import_declaration_cannot_have_modifiers);
                }
                if (checkExternalImportOrExportDeclaration(node)) {
                    var importClause = node.importClause;
                    if (importClause && !checkGrammarImportClause(importClause)) {
                        if (importClause.name) {
                            checkImportBinding(importClause);
                        }
                        if (importClause.namedBindings) {
                            if (importClause.namedBindings.kind === 256 /* NamespaceImport */) {
                                checkImportBinding(importClause.namedBindings);
                            }
                            else {
                                var moduleExisted = resolveExternalModuleName(node, node.moduleSpecifier);
                                if (moduleExisted) {
                                    ts.forEach(importClause.namedBindings.elements, checkImportBinding);
                                }
                            }
                        }
                    }
                }
            }
            function checkImportEqualsDeclaration(node) {
                if (checkGrammarModuleElementContext(node, ts.Diagnostics.An_import_declaration_can_only_be_used_in_a_namespace_or_module)) {
                    // If we hit an import declaration in an illegal context, just bail out to avoid cascading errors.
                    return;
                }
                checkGrammarDecoratorsAndModifiers(node);
                if (ts.isInternalModuleImportEqualsDeclaration(node) || checkExternalImportOrExportDeclaration(node)) {
                    checkImportBinding(node);
                    if (ts.hasModifier(node, 1 /* Export */)) {
                        markExportAsReferenced(node);
                    }
                    if (node.moduleReference.kind !== 265 /* ExternalModuleReference */) {
                        var target = resolveAlias(getSymbolOfNode(node));
                        if (target !== unknownSymbol) {
                            if (target.flags & 111551 /* Value */) {
                                // Target is a value symbol, check that it is not hidden by a local declaration with the same name
                                var moduleName = ts.getFirstIdentifier(node.moduleReference);
                                if (!(resolveEntityName(moduleName, 111551 /* Value */ | 1920 /* Namespace */).flags & 1920 /* Namespace */)) {
                                    error(moduleName, ts.Diagnostics.Module_0_is_hidden_by_a_local_declaration_with_the_same_name, ts.declarationNameToString(moduleName));
                                }
                            }
                            if (target.flags & 788968 /* Type */) {
                                checkTypeNameIsReserved(node.name, ts.Diagnostics.Import_name_cannot_be_0);
                            }
                        }
                    }
                    else {
                        if (moduleKind >= ts.ModuleKind.ES2015 && !(node.flags & 8388608 /* Ambient */)) {
                            // Import equals declaration is deprecated in es6 or above
                            grammarErrorOnNode(node, ts.Diagnostics.Import_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_import_Asterisk_as_ns_from_mod_import_a_from_mod_import_d_from_mod_or_another_module_format_instead);
                        }
                    }
                }
            }
            function checkExportDeclaration(node) {
                if (checkGrammarModuleElementContext(node, ts.Diagnostics.An_export_declaration_can_only_be_used_in_a_module)) {
                    // If we hit an export in an illegal context, just bail out to avoid cascading errors.
                    return;
                }
                if (!checkGrammarDecoratorsAndModifiers(node) && ts.hasModifiers(node)) {
                    grammarErrorOnFirstToken(node, ts.Diagnostics.An_export_declaration_cannot_have_modifiers);
                }
                checkGrammarExportDeclaration(node);
                if (!node.moduleSpecifier || checkExternalImportOrExportDeclaration(node)) {
                    if (node.exportClause) {
                        // export { x, y }
                        // export { x, y } from "foo"
                        if (ts.isNamedExports(node.exportClause)) {
                            ts.forEach(node.exportClause.elements, checkExportSpecifier);
                        }
                        else if (!ts.isNamespaceExport(node.exportClause)) {
                            checkImportBinding(node.exportClause);
                        }
                        var inAmbientExternalModule = node.parent.kind === 250 /* ModuleBlock */ && ts.isAmbientModule(node.parent.parent);
                        var inAmbientNamespaceDeclaration = !inAmbientExternalModule && node.parent.kind === 250 /* ModuleBlock */ &&
                            !node.moduleSpecifier && node.flags & 8388608 /* Ambient */;
                        if (node.parent.kind !== 290 /* SourceFile */ && !inAmbientExternalModule && !inAmbientNamespaceDeclaration) {
                            error(node, ts.Diagnostics.Export_declarations_are_not_permitted_in_a_namespace);
                        }
                    }
                    else {
                        // export * from "foo"
                        var moduleSymbol = resolveExternalModuleName(node, node.moduleSpecifier);
                        if (moduleSymbol && hasExportAssignmentSymbol(moduleSymbol)) {
                            error(node.moduleSpecifier, ts.Diagnostics.Module_0_uses_export_and_cannot_be_used_with_export_Asterisk, symbolToString(moduleSymbol));
                        }
                        if (moduleKind !== ts.ModuleKind.System && moduleKind < ts.ModuleKind.ES2015) {
                            checkExternalEmitHelpers(node, 65536 /* ExportStar */);
                        }
                    }
                }
            }
            function checkGrammarExportDeclaration(node) {
                var _a;
                var isTypeOnlyExportStar = node.isTypeOnly && ((_a = node.exportClause) === null || _a === void 0 ? void 0 : _a.kind) !== 261 /* NamedExports */;
                if (isTypeOnlyExportStar) {
                    grammarErrorOnNode(node, ts.Diagnostics.Only_named_exports_may_use_export_type);
                }
                return !isTypeOnlyExportStar;
            }
            function checkGrammarModuleElementContext(node, errorMessage) {
                var isInAppropriateContext = node.parent.kind === 290 /* SourceFile */ || node.parent.kind === 250 /* ModuleBlock */ || node.parent.kind === 249 /* ModuleDeclaration */;
                if (!isInAppropriateContext) {
                    grammarErrorOnFirstToken(node, errorMessage);
                }
                return !isInAppropriateContext;
            }
            function importClauseContainsReferencedImport(importClause) {
                return ts.forEachImportClauseDeclaration(importClause, function (declaration) {
                    return !!getSymbolOfNode(declaration).isReferenced;
                });
            }
            function importClauseContainsConstEnumUsedAsValue(importClause) {
                return ts.forEachImportClauseDeclaration(importClause, function (declaration) {
                    return !!getSymbolLinks(getSymbolOfNode(declaration)).constEnumReferenced;
                });
            }
            function checkImportsForTypeOnlyConversion(sourceFile) {
                for (var _i = 0, _a = sourceFile.statements; _i < _a.length; _i++) {
                    var statement = _a[_i];
                    if (ts.isImportDeclaration(statement) &&
                        statement.importClause &&
                        !statement.importClause.isTypeOnly &&
                        importClauseContainsReferencedImport(statement.importClause) &&
                        !isReferencedAliasDeclaration(statement.importClause, /*checkChildren*/ true) &&
                        !importClauseContainsConstEnumUsedAsValue(statement.importClause)) {
                        error(statement, ts.Diagnostics.This_import_is_never_used_as_a_value_and_must_use_import_type_because_the_importsNotUsedAsValues_is_set_to_error);
                    }
                }
            }
            function checkExportSpecifier(node) {
                checkAliasSymbol(node);
                if (ts.getEmitDeclarations(compilerOptions)) {
                    collectLinkedAliases(node.propertyName || node.name, /*setVisibility*/ true);
                }
                if (!node.parent.parent.moduleSpecifier) {
                    var exportedName = node.propertyName || node.name;
                    // find immediate value referenced by exported name (SymbolFlags.Alias is set so we don't chase down aliases)
                    var symbol = resolveName(exportedName, exportedName.escapedText, 111551 /* Value */ | 788968 /* Type */ | 1920 /* Namespace */ | 2097152 /* Alias */, 
                    /*nameNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ true);
                    if (symbol && (symbol === undefinedSymbol || symbol === globalThisSymbol || isGlobalSourceFile(getDeclarationContainer(symbol.declarations[0])))) {
                        error(exportedName, ts.Diagnostics.Cannot_export_0_Only_local_declarations_can_be_exported_from_a_module, ts.idText(exportedName));
                    }
                    else {
                        markExportAsReferenced(node);
                        var target = symbol && (symbol.flags & 2097152 /* Alias */ ? resolveAlias(symbol) : symbol);
                        if (!target || target === unknownSymbol || target.flags & 111551 /* Value */) {
                            checkExpressionCached(node.propertyName || node.name);
                        }
                    }
                }
            }
            function checkExportAssignment(node) {
                if (checkGrammarModuleElementContext(node, ts.Diagnostics.An_export_assignment_can_only_be_used_in_a_module)) {
                    // If we hit an export assignment in an illegal context, just bail out to avoid cascading errors.
                    return;
                }
                var container = node.parent.kind === 290 /* SourceFile */ ? node.parent : node.parent.parent;
                if (container.kind === 249 /* ModuleDeclaration */ && !ts.isAmbientModule(container)) {
                    if (node.isExportEquals) {
                        error(node, ts.Diagnostics.An_export_assignment_cannot_be_used_in_a_namespace);
                    }
                    else {
                        error(node, ts.Diagnostics.A_default_export_can_only_be_used_in_an_ECMAScript_style_module);
                    }
                    return;
                }
                // Grammar checking
                if (!checkGrammarDecoratorsAndModifiers(node) && ts.hasModifiers(node)) {
                    grammarErrorOnFirstToken(node, ts.Diagnostics.An_export_assignment_cannot_have_modifiers);
                }
                if (node.expression.kind === 75 /* Identifier */) {
                    var id = node.expression;
                    var sym = resolveEntityName(id, 67108863 /* All */, /*ignoreErrors*/ true, /*dontResolveAlias*/ true, node);
                    if (sym) {
                        markAliasReferenced(sym, id);
                        // If not a value, we're interpreting the identifier as a type export, along the lines of (`export { Id as default }`)
                        var target = sym.flags & 2097152 /* Alias */ ? resolveAlias(sym) : sym;
                        if (target === unknownSymbol || target.flags & 111551 /* Value */) {
                            // However if it is a value, we need to check it's being used correctly
                            checkExpressionCached(node.expression);
                        }
                    }
                    if (ts.getEmitDeclarations(compilerOptions)) {
                        collectLinkedAliases(node.expression, /*setVisibility*/ true);
                    }
                }
                else {
                    checkExpressionCached(node.expression);
                }
                checkExternalModuleExports(container);
                if ((node.flags & 8388608 /* Ambient */) && !ts.isEntityNameExpression(node.expression)) {
                    grammarErrorOnNode(node.expression, ts.Diagnostics.The_expression_of_an_export_assignment_must_be_an_identifier_or_qualified_name_in_an_ambient_context);
                }
                if (node.isExportEquals && !(node.flags & 8388608 /* Ambient */)) {
                    if (moduleKind >= ts.ModuleKind.ES2015) {
                        // export assignment is not supported in es6 modules
                        grammarErrorOnNode(node, ts.Diagnostics.Export_assignment_cannot_be_used_when_targeting_ECMAScript_modules_Consider_using_export_default_or_another_module_format_instead);
                    }
                    else if (moduleKind === ts.ModuleKind.System) {
                        // system modules does not support export assignment
                        grammarErrorOnNode(node, ts.Diagnostics.Export_assignment_is_not_supported_when_module_flag_is_system);
                    }
                }
            }
            function hasExportedMembers(moduleSymbol) {
                return ts.forEachEntry(moduleSymbol.exports, function (_, id) { return id !== "export="; });
            }
            function checkExternalModuleExports(node) {
                var moduleSymbol = getSymbolOfNode(node);
                var links = getSymbolLinks(moduleSymbol);
                if (!links.exportsChecked) {
                    var exportEqualsSymbol = moduleSymbol.exports.get("export=");
                    if (exportEqualsSymbol && hasExportedMembers(moduleSymbol)) {
                        var declaration = getDeclarationOfAliasSymbol(exportEqualsSymbol) || exportEqualsSymbol.valueDeclaration;
                        if (!isTopLevelInExternalModuleAugmentation(declaration) && !ts.isInJSFile(declaration)) {
                            error(declaration, ts.Diagnostics.An_export_assignment_cannot_be_used_in_a_module_with_other_exported_elements);
                        }
                    }
                    // Checks for export * conflicts
                    var exports_2 = getExportsOfModule(moduleSymbol);
                    if (exports_2) {
                        exports_2.forEach(function (_a, id) {
                            var declarations = _a.declarations, flags = _a.flags;
                            if (id === "__export") {
                                return;
                            }
                            // ECMA262: 15.2.1.1 It is a Syntax Error if the ExportedNames of ModuleItemList contains any duplicate entries.
                            // (TS Exceptions: namespaces, function overloads, enums, and interfaces)
                            if (flags & (1920 /* Namespace */ | 64 /* Interface */ | 384 /* Enum */)) {
                                return;
                            }
                            var exportedDeclarationsCount = ts.countWhere(declarations, isNotOverloadAndNotAccessor);
                            if (flags & 524288 /* TypeAlias */ && exportedDeclarationsCount <= 2) {
                                // it is legal to merge type alias with other values
                                // so count should be either 1 (just type alias) or 2 (type alias + merged value)
                                return;
                            }
                            if (exportedDeclarationsCount > 1) {
                                for (var _i = 0, declarations_9 = declarations; _i < declarations_9.length; _i++) {
                                    var declaration = declarations_9[_i];
                                    if (isNotOverload(declaration)) {
                                        diagnostics.add(ts.createDiagnosticForNode(declaration, ts.Diagnostics.Cannot_redeclare_exported_variable_0, ts.unescapeLeadingUnderscores(id)));
                                    }
                                }
                            }
                        });
                    }
                    links.exportsChecked = true;
                }
            }
            function checkSourceElement(node) {
                if (node) {
                    var saveCurrentNode = currentNode;
                    currentNode = node;
                    instantiationCount = 0;
                    checkSourceElementWorker(node);
                    currentNode = saveCurrentNode;
                }
            }
            function checkSourceElementWorker(node) {
                if (ts.isInJSFile(node)) {
                    ts.forEach(node.jsDoc, function (_a) {
                        var tags = _a.tags;
                        return ts.forEach(tags, checkSourceElement);
                    });
                }
                var kind = node.kind;
                if (cancellationToken) {
                    // Only bother checking on a few construct kinds.  We don't want to be excessively
                    // hitting the cancellation token on every node we check.
                    switch (kind) {
                        case 249 /* ModuleDeclaration */:
                        case 245 /* ClassDeclaration */:
                        case 246 /* InterfaceDeclaration */:
                        case 244 /* FunctionDeclaration */:
                            cancellationToken.throwIfCancellationRequested();
                    }
                }
                if (kind >= 225 /* FirstStatement */ && kind <= 241 /* LastStatement */ && node.flowNode && !isReachableFlowNode(node.flowNode)) {
                    errorOrSuggestion(compilerOptions.allowUnreachableCode === false, node, ts.Diagnostics.Unreachable_code_detected);
                }
                switch (kind) {
                    case 155 /* TypeParameter */:
                        return checkTypeParameter(node);
                    case 156 /* Parameter */:
                        return checkParameter(node);
                    case 159 /* PropertyDeclaration */:
                        return checkPropertyDeclaration(node);
                    case 158 /* PropertySignature */:
                        return checkPropertySignature(node);
                    case 170 /* FunctionType */:
                    case 171 /* ConstructorType */:
                    case 165 /* CallSignature */:
                    case 166 /* ConstructSignature */:
                    case 167 /* IndexSignature */:
                        return checkSignatureDeclaration(node);
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                        return checkMethodDeclaration(node);
                    case 162 /* Constructor */:
                        return checkConstructorDeclaration(node);
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        return checkAccessorDeclaration(node);
                    case 169 /* TypeReference */:
                        return checkTypeReferenceNode(node);
                    case 168 /* TypePredicate */:
                        return checkTypePredicate(node);
                    case 172 /* TypeQuery */:
                        return checkTypeQuery(node);
                    case 173 /* TypeLiteral */:
                        return checkTypeLiteral(node);
                    case 174 /* ArrayType */:
                        return checkArrayType(node);
                    case 175 /* TupleType */:
                        return checkTupleType(node);
                    case 178 /* UnionType */:
                    case 179 /* IntersectionType */:
                        return checkUnionOrIntersectionType(node);
                    case 182 /* ParenthesizedType */:
                    case 176 /* OptionalType */:
                    case 177 /* RestType */:
                        return checkSourceElement(node.type);
                    case 183 /* ThisType */:
                        return checkThisType(node);
                    case 184 /* TypeOperator */:
                        return checkTypeOperator(node);
                    case 180 /* ConditionalType */:
                        return checkConditionalType(node);
                    case 181 /* InferType */:
                        return checkInferType(node);
                    case 188 /* ImportType */:
                        return checkImportType(node);
                    case 307 /* JSDocAugmentsTag */:
                        return checkJSDocAugmentsTag(node);
                    case 321 /* JSDocTypedefTag */:
                    case 314 /* JSDocCallbackTag */:
                    case 315 /* JSDocEnumTag */:
                        return checkJSDocTypeAliasTag(node);
                    case 320 /* JSDocTemplateTag */:
                        return checkJSDocTemplateTag(node);
                    case 319 /* JSDocTypeTag */:
                        return checkJSDocTypeTag(node);
                    case 316 /* JSDocParameterTag */:
                        return checkJSDocParameterTag(node);
                    case 300 /* JSDocFunctionType */:
                        checkJSDocFunctionType(node);
                    // falls through
                    case 298 /* JSDocNonNullableType */:
                    case 297 /* JSDocNullableType */:
                    case 295 /* JSDocAllType */:
                    case 296 /* JSDocUnknownType */:
                    case 304 /* JSDocTypeLiteral */:
                        checkJSDocTypeIsInJsFile(node);
                        ts.forEachChild(node, checkSourceElement);
                        return;
                    case 301 /* JSDocVariadicType */:
                        checkJSDocVariadicType(node);
                        return;
                    case 294 /* JSDocTypeExpression */:
                        return checkSourceElement(node.type);
                    case 185 /* IndexedAccessType */:
                        return checkIndexedAccessType(node);
                    case 186 /* MappedType */:
                        return checkMappedType(node);
                    case 244 /* FunctionDeclaration */:
                        return checkFunctionDeclaration(node);
                    case 223 /* Block */:
                    case 250 /* ModuleBlock */:
                        return checkBlock(node);
                    case 225 /* VariableStatement */:
                        return checkVariableStatement(node);
                    case 226 /* ExpressionStatement */:
                        return checkExpressionStatement(node);
                    case 227 /* IfStatement */:
                        return checkIfStatement(node);
                    case 228 /* DoStatement */:
                        return checkDoStatement(node);
                    case 229 /* WhileStatement */:
                        return checkWhileStatement(node);
                    case 230 /* ForStatement */:
                        return checkForStatement(node);
                    case 231 /* ForInStatement */:
                        return checkForInStatement(node);
                    case 232 /* ForOfStatement */:
                        return checkForOfStatement(node);
                    case 233 /* ContinueStatement */:
                    case 234 /* BreakStatement */:
                        return checkBreakOrContinueStatement(node);
                    case 235 /* ReturnStatement */:
                        return checkReturnStatement(node);
                    case 236 /* WithStatement */:
                        return checkWithStatement(node);
                    case 237 /* SwitchStatement */:
                        return checkSwitchStatement(node);
                    case 238 /* LabeledStatement */:
                        return checkLabeledStatement(node);
                    case 239 /* ThrowStatement */:
                        return checkThrowStatement(node);
                    case 240 /* TryStatement */:
                        return checkTryStatement(node);
                    case 242 /* VariableDeclaration */:
                        return checkVariableDeclaration(node);
                    case 191 /* BindingElement */:
                        return checkBindingElement(node);
                    case 245 /* ClassDeclaration */:
                        return checkClassDeclaration(node);
                    case 246 /* InterfaceDeclaration */:
                        return checkInterfaceDeclaration(node);
                    case 247 /* TypeAliasDeclaration */:
                        return checkTypeAliasDeclaration(node);
                    case 248 /* EnumDeclaration */:
                        return checkEnumDeclaration(node);
                    case 249 /* ModuleDeclaration */:
                        return checkModuleDeclaration(node);
                    case 254 /* ImportDeclaration */:
                        return checkImportDeclaration(node);
                    case 253 /* ImportEqualsDeclaration */:
                        return checkImportEqualsDeclaration(node);
                    case 260 /* ExportDeclaration */:
                        return checkExportDeclaration(node);
                    case 259 /* ExportAssignment */:
                        return checkExportAssignment(node);
                    case 224 /* EmptyStatement */:
                    case 241 /* DebuggerStatement */:
                        checkGrammarStatementInAmbientContext(node);
                        return;
                    case 264 /* MissingDeclaration */:
                        return checkMissingDeclaration(node);
                }
            }
            function checkJSDocTypeIsInJsFile(node) {
                if (!ts.isInJSFile(node)) {
                    grammarErrorOnNode(node, ts.Diagnostics.JSDoc_types_can_only_be_used_inside_documentation_comments);
                }
            }
            function checkJSDocVariadicType(node) {
                checkJSDocTypeIsInJsFile(node);
                checkSourceElement(node.type);
                // Only legal location is in the *last* parameter tag or last parameter of a JSDoc function.
                var parent = node.parent;
                if (ts.isParameter(parent) && ts.isJSDocFunctionType(parent.parent)) {
                    if (ts.last(parent.parent.parameters) !== parent) {
                        error(node, ts.Diagnostics.A_rest_parameter_must_be_last_in_a_parameter_list);
                    }
                    return;
                }
                if (!ts.isJSDocTypeExpression(parent)) {
                    error(node, ts.Diagnostics.JSDoc_may_only_appear_in_the_last_parameter_of_a_signature);
                }
                var paramTag = node.parent.parent;
                if (!ts.isJSDocParameterTag(paramTag)) {
                    error(node, ts.Diagnostics.JSDoc_may_only_appear_in_the_last_parameter_of_a_signature);
                    return;
                }
                var param = ts.getParameterSymbolFromJSDoc(paramTag);
                if (!param) {
                    // We will error in `checkJSDocParameterTag`.
                    return;
                }
                var host = ts.getHostSignatureFromJSDoc(paramTag);
                if (!host || ts.last(host.parameters).symbol !== param) {
                    error(node, ts.Diagnostics.A_rest_parameter_must_be_last_in_a_parameter_list);
                }
            }
            function getTypeFromJSDocVariadicType(node) {
                var type = getTypeFromTypeNode(node.type);
                var parent = node.parent;
                var paramTag = node.parent.parent;
                if (ts.isJSDocTypeExpression(node.parent) && ts.isJSDocParameterTag(paramTag)) {
                    // Else we will add a diagnostic, see `checkJSDocVariadicType`.
                    var host_1 = ts.getHostSignatureFromJSDoc(paramTag);
                    if (host_1) {
                        /*
                        Only return an array type if the corresponding parameter is marked as a rest parameter, or if there are no parameters.
                        So in the following situation we will not create an array type:
                            /** @param {...number} a * /
                            function f(a) {}
                        Because `a` will just be of type `number | undefined`. A synthetic `...args` will also be added, which *will* get an array type.
                        */
                        var lastParamDeclaration = ts.lastOrUndefined(host_1.parameters);
                        var symbol = ts.getParameterSymbolFromJSDoc(paramTag);
                        if (!lastParamDeclaration ||
                            symbol && lastParamDeclaration.symbol === symbol && ts.isRestParameter(lastParamDeclaration)) {
                            return createArrayType(type);
                        }
                    }
                }
                if (ts.isParameter(parent) && ts.isJSDocFunctionType(parent.parent)) {
                    return createArrayType(type);
                }
                return addOptionality(type);
            }
            // Function and class expression bodies are checked after all statements in the enclosing body. This is
            // to ensure constructs like the following are permitted:
            //     const foo = function () {
            //        const s = foo();
            //        return "hello";
            //     }
            // Here, performing a full type check of the body of the function expression whilst in the process of
            // determining the type of foo would cause foo to be given type any because of the recursive reference.
            // Delaying the type check of the body ensures foo has been assigned a type.
            function checkNodeDeferred(node) {
                var enclosingFile = ts.getSourceFileOfNode(node);
                var links = getNodeLinks(enclosingFile);
                if (!(links.flags & 1 /* TypeChecked */)) {
                    links.deferredNodes = links.deferredNodes || ts.createMap();
                    var id = "" + getNodeId(node);
                    links.deferredNodes.set(id, node);
                }
            }
            function checkDeferredNodes(context) {
                var links = getNodeLinks(context);
                if (links.deferredNodes) {
                    links.deferredNodes.forEach(checkDeferredNode);
                }
            }
            function checkDeferredNode(node) {
                var saveCurrentNode = currentNode;
                currentNode = node;
                instantiationCount = 0;
                switch (node.kind) {
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                        checkFunctionExpressionOrObjectLiteralMethodDeferred(node);
                        break;
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        checkAccessorDeclaration(node);
                        break;
                    case 214 /* ClassExpression */:
                        checkClassExpressionDeferred(node);
                        break;
                    case 267 /* JsxSelfClosingElement */:
                        checkJsxSelfClosingElementDeferred(node);
                        break;
                    case 266 /* JsxElement */:
                        checkJsxElementDeferred(node);
                        break;
                }
                currentNode = saveCurrentNode;
            }
            function checkSourceFile(node) {
                ts.performance.mark("beforeCheck");
                checkSourceFileWorker(node);
                ts.performance.mark("afterCheck");
                ts.performance.measure("Check", "beforeCheck", "afterCheck");
            }
            function unusedIsError(kind, isAmbient) {
                if (isAmbient) {
                    return false;
                }
                switch (kind) {
                    case 0 /* Local */:
                        return !!compilerOptions.noUnusedLocals;
                    case 1 /* Parameter */:
                        return !!compilerOptions.noUnusedParameters;
                    default:
                        return ts.Debug.assertNever(kind);
                }
            }
            function getPotentiallyUnusedIdentifiers(sourceFile) {
                return allPotentiallyUnusedIdentifiers.get(sourceFile.path) || ts.emptyArray;
            }
            // Fully type check a source file and collect the relevant diagnostics.
            function checkSourceFileWorker(node) {
                var links = getNodeLinks(node);
                if (!(links.flags & 1 /* TypeChecked */)) {
                    if (ts.skipTypeChecking(node, compilerOptions, host)) {
                        return;
                    }
                    // Grammar checking
                    checkGrammarSourceFile(node);
                    ts.clear(potentialThisCollisions);
                    ts.clear(potentialNewTargetCollisions);
                    ts.clear(potentialWeakMapCollisions);
                    ts.forEach(node.statements, checkSourceElement);
                    checkSourceElement(node.endOfFileToken);
                    checkDeferredNodes(node);
                    if (ts.isExternalOrCommonJsModule(node)) {
                        registerForUnusedIdentifiersCheck(node);
                    }
                    if (!node.isDeclarationFile && (compilerOptions.noUnusedLocals || compilerOptions.noUnusedParameters)) {
                        checkUnusedIdentifiers(getPotentiallyUnusedIdentifiers(node), function (containingNode, kind, diag) {
                            if (!ts.containsParseError(containingNode) && unusedIsError(kind, !!(containingNode.flags & 8388608 /* Ambient */))) {
                                diagnostics.add(diag);
                            }
                        });
                    }
                    if (compilerOptions.importsNotUsedAsValues === 2 /* Error */ &&
                        !node.isDeclarationFile &&
                        ts.isExternalModule(node)) {
                        checkImportsForTypeOnlyConversion(node);
                    }
                    if (ts.isExternalOrCommonJsModule(node)) {
                        checkExternalModuleExports(node);
                    }
                    if (potentialThisCollisions.length) {
                        ts.forEach(potentialThisCollisions, checkIfThisIsCapturedInEnclosingScope);
                        ts.clear(potentialThisCollisions);
                    }
                    if (potentialNewTargetCollisions.length) {
                        ts.forEach(potentialNewTargetCollisions, checkIfNewTargetIsCapturedInEnclosingScope);
                        ts.clear(potentialNewTargetCollisions);
                    }
                    if (potentialWeakMapCollisions.length) {
                        ts.forEach(potentialWeakMapCollisions, checkWeakMapCollision);
                        ts.clear(potentialWeakMapCollisions);
                    }
                    links.flags |= 1 /* TypeChecked */;
                }
            }
            function getDiagnostics(sourceFile, ct) {
                try {
                    // Record the cancellation token so it can be checked later on during checkSourceElement.
                    // Do this in a finally block so we can ensure that it gets reset back to nothing after
                    // this call is done.
                    cancellationToken = ct;
                    return getDiagnosticsWorker(sourceFile);
                }
                finally {
                    cancellationToken = undefined;
                }
            }
            function getDiagnosticsWorker(sourceFile) {
                throwIfNonDiagnosticsProducing();
                if (sourceFile) {
                    // Some global diagnostics are deferred until they are needed and
                    // may not be reported in the first call to getGlobalDiagnostics.
                    // We should catch these changes and report them.
                    var previousGlobalDiagnostics = diagnostics.getGlobalDiagnostics();
                    var previousGlobalDiagnosticsSize = previousGlobalDiagnostics.length;
                    checkSourceFile(sourceFile);
                    var semanticDiagnostics = diagnostics.getDiagnostics(sourceFile.fileName);
                    var currentGlobalDiagnostics = diagnostics.getGlobalDiagnostics();
                    if (currentGlobalDiagnostics !== previousGlobalDiagnostics) {
                        // If the arrays are not the same reference, new diagnostics were added.
                        var deferredGlobalDiagnostics = ts.relativeComplement(previousGlobalDiagnostics, currentGlobalDiagnostics, ts.compareDiagnostics);
                        return ts.concatenate(deferredGlobalDiagnostics, semanticDiagnostics);
                    }
                    else if (previousGlobalDiagnosticsSize === 0 && currentGlobalDiagnostics.length > 0) {
                        // If the arrays are the same reference, but the length has changed, a single
                        // new diagnostic was added as DiagnosticCollection attempts to reuse the
                        // same array.
                        return ts.concatenate(currentGlobalDiagnostics, semanticDiagnostics);
                    }
                    return semanticDiagnostics;
                }
                // Global diagnostics are always added when a file is not provided to
                // getDiagnostics
                ts.forEach(host.getSourceFiles(), checkSourceFile);
                return diagnostics.getDiagnostics();
            }
            function getGlobalDiagnostics() {
                throwIfNonDiagnosticsProducing();
                return diagnostics.getGlobalDiagnostics();
            }
            function throwIfNonDiagnosticsProducing() {
                if (!produceDiagnostics) {
                    throw new Error("Trying to get diagnostics from a type checker that does not produce them.");
                }
            }
            // Language service support
            function getSymbolsInScope(location, meaning) {
                if (location.flags & 16777216 /* InWithStatement */) {
                    // We cannot answer semantic questions within a with block, do not proceed any further
                    return [];
                }
                var symbols = ts.createSymbolTable();
                var isStatic = false;
                populateSymbols();
                symbols.delete("this" /* This */); // Not a symbol, a keyword
                return symbolsToArray(symbols);
                function populateSymbols() {
                    while (location) {
                        if (location.locals && !isGlobalSourceFile(location)) {
                            copySymbols(location.locals, meaning);
                        }
                        switch (location.kind) {
                            case 290 /* SourceFile */:
                                if (!ts.isExternalOrCommonJsModule(location))
                                    break;
                            // falls through
                            case 249 /* ModuleDeclaration */:
                                copySymbols(getSymbolOfNode(location).exports, meaning & 2623475 /* ModuleMember */);
                                break;
                            case 248 /* EnumDeclaration */:
                                copySymbols(getSymbolOfNode(location).exports, meaning & 8 /* EnumMember */);
                                break;
                            case 214 /* ClassExpression */:
                                var className = location.name;
                                if (className) {
                                    copySymbol(location.symbol, meaning);
                                }
                            // this fall-through is necessary because we would like to handle
                            // type parameter inside class expression similar to how we handle it in classDeclaration and interface Declaration.
                            // falls through
                            case 245 /* ClassDeclaration */:
                            case 246 /* InterfaceDeclaration */:
                                // If we didn't come from static member of class or interface,
                                // add the type parameters into the symbol table
                                // (type parameters of classDeclaration/classExpression and interface are in member property of the symbol.
                                // Note: that the memberFlags come from previous iteration.
                                if (!isStatic) {
                                    copySymbols(getMembersOfSymbol(getSymbolOfNode(location)), meaning & 788968 /* Type */);
                                }
                                break;
                            case 201 /* FunctionExpression */:
                                var funcName = location.name;
                                if (funcName) {
                                    copySymbol(location.symbol, meaning);
                                }
                                break;
                        }
                        if (ts.introducesArgumentsExoticObject(location)) {
                            copySymbol(argumentsSymbol, meaning);
                        }
                        isStatic = ts.hasModifier(location, 32 /* Static */);
                        location = location.parent;
                    }
                    copySymbols(globals, meaning);
                }
                /**
                 * Copy the given symbol into symbol tables if the symbol has the given meaning
                 * and it doesn't already existed in the symbol table
                 * @param key a key for storing in symbol table; if undefined, use symbol.name
                 * @param symbol the symbol to be added into symbol table
                 * @param meaning meaning of symbol to filter by before adding to symbol table
                 */
                function copySymbol(symbol, meaning) {
                    if (ts.getCombinedLocalAndExportSymbolFlags(symbol) & meaning) {
                        var id = symbol.escapedName;
                        // We will copy all symbol regardless of its reserved name because
                        // symbolsToArray will check whether the key is a reserved name and
                        // it will not copy symbol with reserved name to the array
                        if (!symbols.has(id)) {
                            symbols.set(id, symbol);
                        }
                    }
                }
                function copySymbols(source, meaning) {
                    if (meaning) {
                        source.forEach(function (symbol) {
                            copySymbol(symbol, meaning);
                        });
                    }
                }
            }
            function isTypeDeclarationName(name) {
                return name.kind === 75 /* Identifier */ &&
                    isTypeDeclaration(name.parent) &&
                    name.parent.name === name;
            }
            function isTypeDeclaration(node) {
                switch (node.kind) {
                    case 155 /* TypeParameter */:
                    case 245 /* ClassDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                    case 247 /* TypeAliasDeclaration */:
                    case 248 /* EnumDeclaration */:
                        return true;
                    case 255 /* ImportClause */:
                        return node.isTypeOnly;
                    case 258 /* ImportSpecifier */:
                    case 263 /* ExportSpecifier */:
                        return node.parent.parent.isTypeOnly;
                    default:
                        return false;
                }
            }
            // True if the given identifier is part of a type reference
            function isTypeReferenceIdentifier(node) {
                while (node.parent.kind === 153 /* QualifiedName */) {
                    node = node.parent;
                }
                return node.parent.kind === 169 /* TypeReference */;
            }
            function isHeritageClauseElementIdentifier(node) {
                while (node.parent.kind === 194 /* PropertyAccessExpression */) {
                    node = node.parent;
                }
                return node.parent.kind === 216 /* ExpressionWithTypeArguments */;
            }
            function forEachEnclosingClass(node, callback) {
                var result;
                while (true) {
                    node = ts.getContainingClass(node);
                    if (!node)
                        break;
                    if (result = callback(node))
                        break;
                }
                return result;
            }
            function isNodeUsedDuringClassInitialization(node) {
                return !!ts.findAncestor(node, function (element) {
                    if (ts.isConstructorDeclaration(element) && ts.nodeIsPresent(element.body) || ts.isPropertyDeclaration(element)) {
                        return true;
                    }
                    else if (ts.isClassLike(element) || ts.isFunctionLikeDeclaration(element)) {
                        return "quit";
                    }
                    return false;
                });
            }
            function isNodeWithinClass(node, classDeclaration) {
                return !!forEachEnclosingClass(node, function (n) { return n === classDeclaration; });
            }
            function getLeftSideOfImportEqualsOrExportAssignment(nodeOnRightSide) {
                while (nodeOnRightSide.parent.kind === 153 /* QualifiedName */) {
                    nodeOnRightSide = nodeOnRightSide.parent;
                }
                if (nodeOnRightSide.parent.kind === 253 /* ImportEqualsDeclaration */) {
                    return nodeOnRightSide.parent.moduleReference === nodeOnRightSide ? nodeOnRightSide.parent : undefined;
                }
                if (nodeOnRightSide.parent.kind === 259 /* ExportAssignment */) {
                    return nodeOnRightSide.parent.expression === nodeOnRightSide ? nodeOnRightSide.parent : undefined;
                }
                return undefined;
            }
            function isInRightSideOfImportOrExportAssignment(node) {
                return getLeftSideOfImportEqualsOrExportAssignment(node) !== undefined;
            }
            function getSpecialPropertyAssignmentSymbolFromEntityName(entityName) {
                var specialPropertyAssignmentKind = ts.getAssignmentDeclarationKind(entityName.parent.parent);
                switch (specialPropertyAssignmentKind) {
                    case 1 /* ExportsProperty */:
                    case 3 /* PrototypeProperty */:
                        return getSymbolOfNode(entityName.parent);
                    case 4 /* ThisProperty */:
                    case 2 /* ModuleExports */:
                    case 5 /* Property */:
                        return getSymbolOfNode(entityName.parent.parent);
                }
            }
            function isImportTypeQualifierPart(node) {
                var parent = node.parent;
                while (ts.isQualifiedName(parent)) {
                    node = parent;
                    parent = parent.parent;
                }
                if (parent && parent.kind === 188 /* ImportType */ && parent.qualifier === node) {
                    return parent;
                }
                return undefined;
            }
            function getSymbolOfNameOrPropertyAccessExpression(name) {
                if (ts.isDeclarationName(name)) {
                    return getSymbolOfNode(name.parent);
                }
                if (ts.isInJSFile(name) &&
                    name.parent.kind === 194 /* PropertyAccessExpression */ &&
                    name.parent === name.parent.parent.left) {
                    // Check if this is a special property assignment
                    if (!ts.isPrivateIdentifier(name)) {
                        var specialPropertyAssignmentSymbol = getSpecialPropertyAssignmentSymbolFromEntityName(name);
                        if (specialPropertyAssignmentSymbol) {
                            return specialPropertyAssignmentSymbol;
                        }
                    }
                }
                if (name.parent.kind === 259 /* ExportAssignment */ && ts.isEntityNameExpression(name)) {
                    // Even an entity name expression that doesn't resolve as an entityname may still typecheck as a property access expression
                    var success = resolveEntityName(name, 
                    /*all meanings*/ 111551 /* Value */ | 788968 /* Type */ | 1920 /* Namespace */ | 2097152 /* Alias */, /*ignoreErrors*/ true);
                    if (success && success !== unknownSymbol) {
                        return success;
                    }
                }
                else if (!ts.isPropertyAccessExpression(name) && !ts.isPrivateIdentifier(name) && isInRightSideOfImportOrExportAssignment(name)) {
                    // Since we already checked for ExportAssignment, this really could only be an Import
                    var importEqualsDeclaration = ts.getAncestor(name, 253 /* ImportEqualsDeclaration */);
                    ts.Debug.assert(importEqualsDeclaration !== undefined);
                    return getSymbolOfPartOfRightHandSideOfImportEquals(name, /*dontResolveAlias*/ true);
                }
                if (!ts.isPropertyAccessExpression(name) && !ts.isPrivateIdentifier(name)) {
                    var possibleImportNode = isImportTypeQualifierPart(name);
                    if (possibleImportNode) {
                        getTypeFromTypeNode(possibleImportNode);
                        var sym = getNodeLinks(name).resolvedSymbol;
                        return sym === unknownSymbol ? undefined : sym;
                    }
                }
                while (ts.isRightSideOfQualifiedNameOrPropertyAccess(name)) {
                    name = name.parent;
                }
                if (isHeritageClauseElementIdentifier(name)) {
                    var meaning = 0 /* None */;
                    // In an interface or class, we're definitely interested in a type.
                    if (name.parent.kind === 216 /* ExpressionWithTypeArguments */) {
                        meaning = 788968 /* Type */;
                        // In a class 'extends' clause we are also looking for a value.
                        if (ts.isExpressionWithTypeArgumentsInClassExtendsClause(name.parent)) {
                            meaning |= 111551 /* Value */;
                        }
                    }
                    else {
                        meaning = 1920 /* Namespace */;
                    }
                    meaning |= 2097152 /* Alias */;
                    var entityNameSymbol = ts.isEntityNameExpression(name) ? resolveEntityName(name, meaning) : undefined;
                    if (entityNameSymbol) {
                        return entityNameSymbol;
                    }
                }
                if (name.parent.kind === 316 /* JSDocParameterTag */) {
                    return ts.getParameterSymbolFromJSDoc(name.parent);
                }
                if (name.parent.kind === 155 /* TypeParameter */ && name.parent.parent.kind === 320 /* JSDocTemplateTag */) {
                    ts.Debug.assert(!ts.isInJSFile(name)); // Otherwise `isDeclarationName` would have been true.
                    var typeParameter = ts.getTypeParameterFromJsDoc(name.parent);
                    return typeParameter && typeParameter.symbol;
                }
                if (ts.isExpressionNode(name)) {
                    if (ts.nodeIsMissing(name)) {
                        // Missing entity name.
                        return undefined;
                    }
                    if (name.kind === 75 /* Identifier */) {
                        if (ts.isJSXTagName(name) && isJsxIntrinsicIdentifier(name)) {
                            var symbol = getIntrinsicTagSymbol(name.parent);
                            return symbol === unknownSymbol ? undefined : symbol;
                        }
                        return resolveEntityName(name, 111551 /* Value */, /*ignoreErrors*/ false, /*dontResolveAlias*/ true);
                    }
                    else if (name.kind === 194 /* PropertyAccessExpression */ || name.kind === 153 /* QualifiedName */) {
                        var links = getNodeLinks(name);
                        if (links.resolvedSymbol) {
                            return links.resolvedSymbol;
                        }
                        if (name.kind === 194 /* PropertyAccessExpression */) {
                            checkPropertyAccessExpression(name);
                        }
                        else {
                            checkQualifiedName(name);
                        }
                        return links.resolvedSymbol;
                    }
                }
                else if (isTypeReferenceIdentifier(name)) {
                    var meaning = name.parent.kind === 169 /* TypeReference */ ? 788968 /* Type */ : 1920 /* Namespace */;
                    return resolveEntityName(name, meaning, /*ignoreErrors*/ false, /*dontResolveAlias*/ true);
                }
                if (name.parent.kind === 168 /* TypePredicate */) {
                    return resolveEntityName(name, /*meaning*/ 1 /* FunctionScopedVariable */);
                }
                // Do we want to return undefined here?
                return undefined;
            }
            function getSymbolAtLocation(node) {
                if (node.kind === 290 /* SourceFile */) {
                    return ts.isExternalModule(node) ? getMergedSymbol(node.symbol) : undefined;
                }
                var parent = node.parent;
                var grandParent = parent.parent;
                if (node.flags & 16777216 /* InWithStatement */) {
                    // We cannot answer semantic questions within a with block, do not proceed any further
                    return undefined;
                }
                if (isDeclarationNameOrImportPropertyName(node)) {
                    // This is a declaration, call getSymbolOfNode
                    var parentSymbol = getSymbolOfNode(parent);
                    return ts.isImportOrExportSpecifier(node.parent) && node.parent.propertyName === node
                        ? getImmediateAliasedSymbol(parentSymbol)
                        : parentSymbol;
                }
                else if (ts.isLiteralComputedPropertyDeclarationName(node)) {
                    return getSymbolOfNode(parent.parent);
                }
                if (node.kind === 75 /* Identifier */) {
                    if (isInRightSideOfImportOrExportAssignment(node)) {
                        return getSymbolOfNameOrPropertyAccessExpression(node);
                    }
                    else if (parent.kind === 191 /* BindingElement */ &&
                        grandParent.kind === 189 /* ObjectBindingPattern */ &&
                        node === parent.propertyName) {
                        var typeOfPattern = getTypeOfNode(grandParent);
                        var propertyDeclaration = getPropertyOfType(typeOfPattern, node.escapedText);
                        if (propertyDeclaration) {
                            return propertyDeclaration;
                        }
                    }
                }
                switch (node.kind) {
                    case 75 /* Identifier */:
                    case 76 /* PrivateIdentifier */:
                    case 194 /* PropertyAccessExpression */:
                    case 153 /* QualifiedName */:
                        return getSymbolOfNameOrPropertyAccessExpression(node);
                    case 104 /* ThisKeyword */:
                        var container = ts.getThisContainer(node, /*includeArrowFunctions*/ false);
                        if (ts.isFunctionLike(container)) {
                            var sig = getSignatureFromDeclaration(container);
                            if (sig.thisParameter) {
                                return sig.thisParameter;
                            }
                        }
                        if (ts.isInExpressionContext(node)) {
                            return checkExpression(node).symbol;
                        }
                    // falls through
                    case 183 /* ThisType */:
                        return getTypeFromThisTypeNode(node).symbol;
                    case 102 /* SuperKeyword */:
                        return checkExpression(node).symbol;
                    case 129 /* ConstructorKeyword */:
                        // constructor keyword for an overload, should take us to the definition if it exist
                        var constructorDeclaration = node.parent;
                        if (constructorDeclaration && constructorDeclaration.kind === 162 /* Constructor */) {
                            return constructorDeclaration.parent.symbol;
                        }
                        return undefined;
                    case 10 /* StringLiteral */:
                    case 14 /* NoSubstitutionTemplateLiteral */:
                        // 1). import x = require("./mo/*gotToDefinitionHere*/d")
                        // 2). External module name in an import declaration
                        // 3). Dynamic import call or require in javascript
                        // 4). type A = import("./f/*gotToDefinitionHere*/oo")
                        if ((ts.isExternalModuleImportEqualsDeclaration(node.parent.parent) && ts.getExternalModuleImportEqualsDeclarationExpression(node.parent.parent) === node) ||
                            ((node.parent.kind === 254 /* ImportDeclaration */ || node.parent.kind === 260 /* ExportDeclaration */) && node.parent.moduleSpecifier === node) ||
                            ((ts.isInJSFile(node) && ts.isRequireCall(node.parent, /*checkArgumentIsStringLiteralLike*/ false)) || ts.isImportCall(node.parent)) ||
                            (ts.isLiteralTypeNode(node.parent) && ts.isLiteralImportTypeNode(node.parent.parent) && node.parent.parent.argument === node.parent)) {
                            return resolveExternalModuleName(node, node);
                        }
                        if (ts.isCallExpression(parent) && ts.isBindableObjectDefinePropertyCall(parent) && parent.arguments[1] === node) {
                            return getSymbolOfNode(parent);
                        }
                    // falls through
                    case 8 /* NumericLiteral */:
                        // index access
                        var objectType = ts.isElementAccessExpression(parent)
                            ? parent.argumentExpression === node ? getTypeOfExpression(parent.expression) : undefined
                            : ts.isLiteralTypeNode(parent) && ts.isIndexedAccessTypeNode(grandParent)
                                ? getTypeFromTypeNode(grandParent.objectType)
                                : undefined;
                        return objectType && getPropertyOfType(objectType, ts.escapeLeadingUnderscores(node.text));
                    case 84 /* DefaultKeyword */:
                    case 94 /* FunctionKeyword */:
                    case 38 /* EqualsGreaterThanToken */:
                    case 80 /* ClassKeyword */:
                        return getSymbolOfNode(node.parent);
                    case 188 /* ImportType */:
                        return ts.isLiteralImportTypeNode(node) ? getSymbolAtLocation(node.argument.literal) : undefined;
                    case 89 /* ExportKeyword */:
                        return ts.isExportAssignment(node.parent) ? ts.Debug.assertDefined(node.parent.symbol) : undefined;
                    default:
                        return undefined;
                }
            }
            function getShorthandAssignmentValueSymbol(location) {
                if (location && location.kind === 282 /* ShorthandPropertyAssignment */) {
                    return resolveEntityName(location.name, 111551 /* Value */ | 2097152 /* Alias */);
                }
                return undefined;
            }
            /** Returns the target of an export specifier without following aliases */
            function getExportSpecifierLocalTargetSymbol(node) {
                return node.parent.parent.moduleSpecifier ?
                    getExternalModuleMember(node.parent.parent, node) :
                    resolveEntityName(node.propertyName || node.name, 111551 /* Value */ | 788968 /* Type */ | 1920 /* Namespace */ | 2097152 /* Alias */);
            }
            function getTypeOfNode(node) {
                if (node.flags & 16777216 /* InWithStatement */) {
                    // We cannot answer semantic questions within a with block, do not proceed any further
                    return errorType;
                }
                var classDecl = ts.tryGetClassImplementingOrExtendingExpressionWithTypeArguments(node);
                var classType = classDecl && getDeclaredTypeOfClassOrInterface(getSymbolOfNode(classDecl.class));
                if (ts.isPartOfTypeNode(node)) {
                    var typeFromTypeNode = getTypeFromTypeNode(node);
                    return classType ? getTypeWithThisArgument(typeFromTypeNode, classType.thisType) : typeFromTypeNode;
                }
                if (ts.isExpressionNode(node)) {
                    return getRegularTypeOfExpression(node);
                }
                if (classType && !classDecl.isImplements) {
                    // A SyntaxKind.ExpressionWithTypeArguments is considered a type node, except when it occurs in the
                    // extends clause of a class. We handle that case here.
                    var baseType = ts.firstOrUndefined(getBaseTypes(classType));
                    return baseType ? getTypeWithThisArgument(baseType, classType.thisType) : errorType;
                }
                if (isTypeDeclaration(node)) {
                    // In this case, we call getSymbolOfNode instead of getSymbolAtLocation because it is a declaration
                    var symbol = getSymbolOfNode(node);
                    return getDeclaredTypeOfSymbol(symbol);
                }
                if (isTypeDeclarationName(node)) {
                    var symbol = getSymbolAtLocation(node);
                    return symbol ? getDeclaredTypeOfSymbol(symbol) : errorType;
                }
                if (ts.isDeclaration(node)) {
                    // In this case, we call getSymbolOfNode instead of getSymbolAtLocation because it is a declaration
                    var symbol = getSymbolOfNode(node);
                    return getTypeOfSymbol(symbol);
                }
                if (isDeclarationNameOrImportPropertyName(node)) {
                    var symbol = getSymbolAtLocation(node);
                    if (symbol) {
                        return getTypeOfSymbol(symbol);
                    }
                    return errorType;
                }
                if (ts.isBindingPattern(node)) {
                    return getTypeForVariableLikeDeclaration(node.parent, /*includeOptionality*/ true) || errorType;
                }
                if (isInRightSideOfImportOrExportAssignment(node)) {
                    var symbol = getSymbolAtLocation(node);
                    if (symbol) {
                        var declaredType = getDeclaredTypeOfSymbol(symbol);
                        return declaredType !== errorType ? declaredType : getTypeOfSymbol(symbol);
                    }
                }
                return errorType;
            }
            // Gets the type of object literal or array literal of destructuring assignment.
            // { a } from
            //     for ( { a } of elems) {
            //     }
            // [ a ] from
            //     [a] = [ some array ...]
            function getTypeOfAssignmentPattern(expr) {
                ts.Debug.assert(expr.kind === 193 /* ObjectLiteralExpression */ || expr.kind === 192 /* ArrayLiteralExpression */);
                // If this is from "for of"
                //     for ( { a } of elems) {
                //     }
                if (expr.parent.kind === 232 /* ForOfStatement */) {
                    var iteratedType = checkRightHandSideOfForOf(expr.parent.expression, expr.parent.awaitModifier);
                    return checkDestructuringAssignment(expr, iteratedType || errorType);
                }
                // If this is from "for" initializer
                //     for ({a } = elems[0];.....) { }
                if (expr.parent.kind === 209 /* BinaryExpression */) {
                    var iteratedType = getTypeOfExpression(expr.parent.right);
                    return checkDestructuringAssignment(expr, iteratedType || errorType);
                }
                // If this is from nested object binding pattern
                //     for ({ skills: { primary, secondary } } = multiRobot, i = 0; i < 1; i++) {
                if (expr.parent.kind === 281 /* PropertyAssignment */) {
                    var node_4 = ts.cast(expr.parent.parent, ts.isObjectLiteralExpression);
                    var typeOfParentObjectLiteral = getTypeOfAssignmentPattern(node_4) || errorType;
                    var propertyIndex = ts.indexOfNode(node_4.properties, expr.parent);
                    return checkObjectLiteralDestructuringPropertyAssignment(node_4, typeOfParentObjectLiteral, propertyIndex);
                }
                // Array literal assignment - array destructuring pattern
                var node = ts.cast(expr.parent, ts.isArrayLiteralExpression);
                //    [{ property1: p1, property2 }] = elems;
                var typeOfArrayLiteral = getTypeOfAssignmentPattern(node) || errorType;
                var elementType = checkIteratedTypeOrElementType(65 /* Destructuring */, typeOfArrayLiteral, undefinedType, expr.parent) || errorType;
                return checkArrayLiteralDestructuringElementAssignment(node, typeOfArrayLiteral, node.elements.indexOf(expr), elementType);
            }
            // Gets the property symbol corresponding to the property in destructuring assignment
            // 'property1' from
            //     for ( { property1: a } of elems) {
            //     }
            // 'property1' at location 'a' from:
            //     [a] = [ property1, property2 ]
            function getPropertySymbolOfDestructuringAssignment(location) {
                // Get the type of the object or array literal and then look for property of given name in the type
                var typeOfObjectLiteral = getTypeOfAssignmentPattern(ts.cast(location.parent.parent, ts.isAssignmentPattern));
                return typeOfObjectLiteral && getPropertyOfType(typeOfObjectLiteral, location.escapedText);
            }
            function getRegularTypeOfExpression(expr) {
                if (ts.isRightSideOfQualifiedNameOrPropertyAccess(expr)) {
                    expr = expr.parent;
                }
                return getRegularTypeOfLiteralType(getTypeOfExpression(expr));
            }
            /**
             * Gets either the static or instance type of a class element, based on
             * whether the element is declared as "static".
             */
            function getParentTypeOfClassElement(node) {
                var classSymbol = getSymbolOfNode(node.parent);
                return ts.hasModifier(node, 32 /* Static */)
                    ? getTypeOfSymbol(classSymbol)
                    : getDeclaredTypeOfSymbol(classSymbol);
            }
            function getClassElementPropertyKeyType(element) {
                var name = element.name;
                switch (name.kind) {
                    case 75 /* Identifier */:
                        return getLiteralType(ts.idText(name));
                    case 8 /* NumericLiteral */:
                    case 10 /* StringLiteral */:
                        return getLiteralType(name.text);
                    case 154 /* ComputedPropertyName */:
                        var nameType = checkComputedPropertyName(name);
                        return isTypeAssignableToKind(nameType, 12288 /* ESSymbolLike */) ? nameType : stringType;
                    default:
                        return ts.Debug.fail("Unsupported property name.");
                }
            }
            // Return the list of properties of the given type, augmented with properties from Function
            // if the type has call or construct signatures
            function getAugmentedPropertiesOfType(type) {
                type = getApparentType(type);
                var propsByName = ts.createSymbolTable(getPropertiesOfType(type));
                var functionType = getSignaturesOfType(type, 0 /* Call */).length ? globalCallableFunctionType :
                    getSignaturesOfType(type, 1 /* Construct */).length ? globalNewableFunctionType :
                        undefined;
                if (functionType) {
                    ts.forEach(getPropertiesOfType(functionType), function (p) {
                        if (!propsByName.has(p.escapedName)) {
                            propsByName.set(p.escapedName, p);
                        }
                    });
                }
                return getNamedMembers(propsByName);
            }
            function typeHasCallOrConstructSignatures(type) {
                return ts.typeHasCallOrConstructSignatures(type, checker);
            }
            function getRootSymbols(symbol) {
                var roots = getImmediateRootSymbols(symbol);
                return roots ? ts.flatMap(roots, getRootSymbols) : [symbol];
            }
            function getImmediateRootSymbols(symbol) {
                if (ts.getCheckFlags(symbol) & 6 /* Synthetic */) {
                    return ts.mapDefined(getSymbolLinks(symbol).containingType.types, function (type) { return getPropertyOfType(type, symbol.escapedName); });
                }
                else if (symbol.flags & 33554432 /* Transient */) {
                    var _a = symbol, leftSpread = _a.leftSpread, rightSpread = _a.rightSpread, syntheticOrigin = _a.syntheticOrigin;
                    return leftSpread ? [leftSpread, rightSpread]
                        : syntheticOrigin ? [syntheticOrigin]
                            : ts.singleElementArray(tryGetAliasTarget(symbol));
                }
                return undefined;
            }
            function tryGetAliasTarget(symbol) {
                var target;
                var next = symbol;
                while (next = getSymbolLinks(next).target) {
                    target = next;
                }
                return target;
            }
            // Emitter support
            function isArgumentsLocalBinding(nodeIn) {
                if (!ts.isGeneratedIdentifier(nodeIn)) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isIdentifier);
                    if (node) {
                        var isPropertyName_1 = node.parent.kind === 194 /* PropertyAccessExpression */ && node.parent.name === node;
                        return !isPropertyName_1 && getReferencedValueSymbol(node) === argumentsSymbol;
                    }
                }
                return false;
            }
            function moduleExportsSomeValue(moduleReferenceExpression) {
                var moduleSymbol = resolveExternalModuleName(moduleReferenceExpression.parent, moduleReferenceExpression);
                if (!moduleSymbol || ts.isShorthandAmbientModuleSymbol(moduleSymbol)) {
                    // If the module is not found or is shorthand, assume that it may export a value.
                    return true;
                }
                var hasExportAssignment = hasExportAssignmentSymbol(moduleSymbol);
                // if module has export assignment then 'resolveExternalModuleSymbol' will return resolved symbol for export assignment
                // otherwise it will return moduleSymbol itself
                moduleSymbol = resolveExternalModuleSymbol(moduleSymbol);
                var symbolLinks = getSymbolLinks(moduleSymbol);
                if (symbolLinks.exportsSomeValue === undefined) {
                    // for export assignments - check if resolved symbol for RHS is itself a value
                    // otherwise - check if at least one export is value
                    symbolLinks.exportsSomeValue = hasExportAssignment
                        ? !!(moduleSymbol.flags & 111551 /* Value */)
                        : ts.forEachEntry(getExportsOfModule(moduleSymbol), isValue);
                }
                return symbolLinks.exportsSomeValue;
                function isValue(s) {
                    s = resolveSymbol(s);
                    return s && !!(s.flags & 111551 /* Value */);
                }
            }
            function isNameOfModuleOrEnumDeclaration(node) {
                return ts.isModuleOrEnumDeclaration(node.parent) && node === node.parent.name;
            }
            // When resolved as an expression identifier, if the given node references an exported entity, return the declaration
            // node of the exported entity's container. Otherwise, return undefined.
            function getReferencedExportContainer(nodeIn, prefixLocals) {
                var node = ts.getParseTreeNode(nodeIn, ts.isIdentifier);
                if (node) {
                    // When resolving the export container for the name of a module or enum
                    // declaration, we need to start resolution at the declaration's container.
                    // Otherwise, we could incorrectly resolve the export container as the
                    // declaration if it contains an exported member with the same name.
                    var symbol = getReferencedValueSymbol(node, /*startInDeclarationContainer*/ isNameOfModuleOrEnumDeclaration(node));
                    if (symbol) {
                        if (symbol.flags & 1048576 /* ExportValue */) {
                            // If we reference an exported entity within the same module declaration, then whether
                            // we prefix depends on the kind of entity. SymbolFlags.ExportHasLocal encompasses all the
                            // kinds that we do NOT prefix.
                            var exportSymbol = getMergedSymbol(symbol.exportSymbol);
                            if (!prefixLocals && exportSymbol.flags & 944 /* ExportHasLocal */ && !(exportSymbol.flags & 3 /* Variable */)) {
                                return undefined;
                            }
                            symbol = exportSymbol;
                        }
                        var parentSymbol_1 = getParentOfSymbol(symbol);
                        if (parentSymbol_1) {
                            if (parentSymbol_1.flags & 512 /* ValueModule */ && parentSymbol_1.valueDeclaration.kind === 290 /* SourceFile */) {
                                var symbolFile = parentSymbol_1.valueDeclaration;
                                var referenceFile = ts.getSourceFileOfNode(node);
                                // If `node` accesses an export and that export isn't in the same file, then symbol is a namespace export, so return undefined.
                                var symbolIsUmdExport = symbolFile !== referenceFile;
                                return symbolIsUmdExport ? undefined : symbolFile;
                            }
                            return ts.findAncestor(node.parent, function (n) { return ts.isModuleOrEnumDeclaration(n) && getSymbolOfNode(n) === parentSymbol_1; });
                        }
                    }
                }
            }
            // When resolved as an expression identifier, if the given node references an import, return the declaration of
            // that import. Otherwise, return undefined.
            function getReferencedImportDeclaration(nodeIn) {
                var node = ts.getParseTreeNode(nodeIn, ts.isIdentifier);
                if (node) {
                    var symbol = getReferencedValueSymbol(node);
                    // We should only get the declaration of an alias if there isn't a local value
                    // declaration for the symbol
                    if (isNonLocalAlias(symbol, /*excludes*/ 111551 /* Value */) && !getTypeOnlyAliasDeclaration(symbol)) {
                        return getDeclarationOfAliasSymbol(symbol);
                    }
                }
                return undefined;
            }
            function isSymbolOfDestructuredElementOfCatchBinding(symbol) {
                return ts.isBindingElement(symbol.valueDeclaration)
                    && ts.walkUpBindingElementsAndPatterns(symbol.valueDeclaration).parent.kind === 280 /* CatchClause */;
            }
            function isSymbolOfDeclarationWithCollidingName(symbol) {
                if (symbol.flags & 418 /* BlockScoped */ && !ts.isSourceFile(symbol.valueDeclaration)) {
                    var links = getSymbolLinks(symbol);
                    if (links.isDeclarationWithCollidingName === undefined) {
                        var container = ts.getEnclosingBlockScopeContainer(symbol.valueDeclaration);
                        if (ts.isStatementWithLocals(container) || isSymbolOfDestructuredElementOfCatchBinding(symbol)) {
                            var nodeLinks_1 = getNodeLinks(symbol.valueDeclaration);
                            if (resolveName(container.parent, symbol.escapedName, 111551 /* Value */, /*nameNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ false)) {
                                // redeclaration - always should be renamed
                                links.isDeclarationWithCollidingName = true;
                            }
                            else if (nodeLinks_1.flags & 262144 /* CapturedBlockScopedBinding */) {
                                // binding is captured in the function
                                // should be renamed if:
                                // - binding is not top level - top level bindings never collide with anything
                                // AND
                                //   - binding is not declared in loop, should be renamed to avoid name reuse across siblings
                                //     let a, b
                                //     { let x = 1; a = () => x; }
                                //     { let x = 100; b = () => x; }
                                //     console.log(a()); // should print '1'
                                //     console.log(b()); // should print '100'
                                //     OR
                                //   - binding is declared inside loop but not in inside initializer of iteration statement or directly inside loop body
                                //     * variables from initializer are passed to rewritten loop body as parameters so they are not captured directly
                                //     * variables that are declared immediately in loop body will become top level variable after loop is rewritten and thus
                                //       they will not collide with anything
                                var isDeclaredInLoop = nodeLinks_1.flags & 524288 /* BlockScopedBindingInLoop */;
                                var inLoopInitializer = ts.isIterationStatement(container, /*lookInLabeledStatements*/ false);
                                var inLoopBodyBlock = container.kind === 223 /* Block */ && ts.isIterationStatement(container.parent, /*lookInLabeledStatements*/ false);
                                links.isDeclarationWithCollidingName = !ts.isBlockScopedContainerTopLevel(container) && (!isDeclaredInLoop || (!inLoopInitializer && !inLoopBodyBlock));
                            }
                            else {
                                links.isDeclarationWithCollidingName = false;
                            }
                        }
                    }
                    return links.isDeclarationWithCollidingName;
                }
                return false;
            }
            // When resolved as an expression identifier, if the given node references a nested block scoped entity with
            // a name that either hides an existing name or might hide it when compiled downlevel,
            // return the declaration of that entity. Otherwise, return undefined.
            function getReferencedDeclarationWithCollidingName(nodeIn) {
                if (!ts.isGeneratedIdentifier(nodeIn)) {
                    var node = ts.getParseTreeNode(nodeIn, ts.isIdentifier);
                    if (node) {
                        var symbol = getReferencedValueSymbol(node);
                        if (symbol && isSymbolOfDeclarationWithCollidingName(symbol)) {
                            return symbol.valueDeclaration;
                        }
                    }
                }
                return undefined;
            }
            // Return true if the given node is a declaration of a nested block scoped entity with a name that either hides an
            // existing name or might hide a name when compiled downlevel
            function isDeclarationWithCollidingName(nodeIn) {
                var node = ts.getParseTreeNode(nodeIn, ts.isDeclaration);
                if (node) {
                    var symbol = getSymbolOfNode(node);
                    if (symbol) {
                        return isSymbolOfDeclarationWithCollidingName(symbol);
                    }
                }
                return false;
            }
            function isValueAliasDeclaration(node) {
                switch (node.kind) {
                    case 253 /* ImportEqualsDeclaration */:
                        return isAliasResolvedToValue(getSymbolOfNode(node) || unknownSymbol);
                    case 255 /* ImportClause */:
                    case 256 /* NamespaceImport */:
                    case 258 /* ImportSpecifier */:
                    case 263 /* ExportSpecifier */:
                        var symbol = getSymbolOfNode(node) || unknownSymbol;
                        return isAliasResolvedToValue(symbol) && !getTypeOnlyAliasDeclaration(symbol);
                    case 260 /* ExportDeclaration */:
                        var exportClause = node.exportClause;
                        return !!exportClause && (ts.isNamespaceExport(exportClause) ||
                            ts.some(exportClause.elements, isValueAliasDeclaration));
                    case 259 /* ExportAssignment */:
                        return node.expression && node.expression.kind === 75 /* Identifier */ ?
                            isAliasResolvedToValue(getSymbolOfNode(node) || unknownSymbol) :
                            true;
                }
                return false;
            }
            function isTopLevelValueImportEqualsWithEntityName(nodeIn) {
                var node = ts.getParseTreeNode(nodeIn, ts.isImportEqualsDeclaration);
                if (node === undefined || node.parent.kind !== 290 /* SourceFile */ || !ts.isInternalModuleImportEqualsDeclaration(node)) {
                    // parent is not source file or it is not reference to internal module
                    return false;
                }
                var isValue = isAliasResolvedToValue(getSymbolOfNode(node));
                return isValue && node.moduleReference && !ts.nodeIsMissing(node.moduleReference);
            }
            function isAliasResolvedToValue(symbol) {
                var target = resolveAlias(symbol);
                if (target === unknownSymbol) {
                    return true;
                }
                // const enums and modules that contain only const enums are not considered values from the emit perspective
                // unless 'preserveConstEnums' option is set to true
                return !!(target.flags & 111551 /* Value */) &&
                    (compilerOptions.preserveConstEnums || !isConstEnumOrConstEnumOnlyModule(target));
            }
            function isConstEnumOrConstEnumOnlyModule(s) {
                return isConstEnumSymbol(s) || !!s.constEnumOnlyModule;
            }
            function isReferencedAliasDeclaration(node, checkChildren) {
                if (isAliasSymbolDeclaration(node)) {
                    var symbol = getSymbolOfNode(node);
                    if (symbol && getSymbolLinks(symbol).referenced) {
                        return true;
                    }
                    var target = getSymbolLinks(symbol).target; // TODO: GH#18217
                    if (target && ts.getModifierFlags(node) & 1 /* Export */ &&
                        target.flags & 111551 /* Value */ &&
                        (compilerOptions.preserveConstEnums || !isConstEnumOrConstEnumOnlyModule(target))) {
                        // An `export import ... =` of a value symbol is always considered referenced
                        return true;
                    }
                }
                if (checkChildren) {
                    return !!ts.forEachChild(node, function (node) { return isReferencedAliasDeclaration(node, checkChildren); });
                }
                return false;
            }
            function isImplementationOfOverload(node) {
                if (ts.nodeIsPresent(node.body)) {
                    if (ts.isGetAccessor(node) || ts.isSetAccessor(node))
                        return false; // Get or set accessors can never be overload implementations, but can have up to 2 signatures
                    var symbol = getSymbolOfNode(node);
                    var signaturesOfSymbol = getSignaturesOfSymbol(symbol);
                    // If this function body corresponds to function with multiple signature, it is implementation of overload
                    // e.g.: function foo(a: string): string;
                    //       function foo(a: number): number;
                    //       function foo(a: any) { // This is implementation of the overloads
                    //           return a;
                    //       }
                    return signaturesOfSymbol.length > 1 ||
                        // If there is single signature for the symbol, it is overload if that signature isn't coming from the node
                        // e.g.: function foo(a: string): string;
                        //       function foo(a: any) { // This is implementation of the overloads
                        //           return a;
                        //       }
                        (signaturesOfSymbol.length === 1 && signaturesOfSymbol[0].declaration !== node);
                }
                return false;
            }
            function isRequiredInitializedParameter(parameter) {
                return !!strictNullChecks &&
                    !isOptionalParameter(parameter) &&
                    !ts.isJSDocParameterTag(parameter) &&
                    !!parameter.initializer &&
                    !ts.hasModifier(parameter, 92 /* ParameterPropertyModifier */);
            }
            function isOptionalUninitializedParameterProperty(parameter) {
                return strictNullChecks &&
                    isOptionalParameter(parameter) &&
                    !parameter.initializer &&
                    ts.hasModifier(parameter, 92 /* ParameterPropertyModifier */);
            }
            function isExpandoFunctionDeclaration(node) {
                var declaration = ts.getParseTreeNode(node, ts.isFunctionDeclaration);
                if (!declaration) {
                    return false;
                }
                var symbol = getSymbolOfNode(declaration);
                if (!symbol || !(symbol.flags & 16 /* Function */)) {
                    return false;
                }
                return !!ts.forEachEntry(getExportsOfSymbol(symbol), function (p) { return p.flags & 111551 /* Value */ && p.valueDeclaration && ts.isPropertyAccessExpression(p.valueDeclaration); });
            }
            function getPropertiesOfContainerFunction(node) {
                var declaration = ts.getParseTreeNode(node, ts.isFunctionDeclaration);
                if (!declaration) {
                    return ts.emptyArray;
                }
                var symbol = getSymbolOfNode(declaration);
                return symbol && getPropertiesOfType(getTypeOfSymbol(symbol)) || ts.emptyArray;
            }
            function getNodeCheckFlags(node) {
                return getNodeLinks(node).flags || 0;
            }
            function getEnumMemberValue(node) {
                computeEnumMemberValues(node.parent);
                return getNodeLinks(node).enumMemberValue;
            }
            function canHaveConstantValue(node) {
                switch (node.kind) {
                    case 284 /* EnumMember */:
                    case 194 /* PropertyAccessExpression */:
                    case 195 /* ElementAccessExpression */:
                        return true;
                }
                return false;
            }
            function getConstantValue(node) {
                if (node.kind === 284 /* EnumMember */) {
                    return getEnumMemberValue(node);
                }
                var symbol = getNodeLinks(node).resolvedSymbol;
                if (symbol && (symbol.flags & 8 /* EnumMember */)) {
                    // inline property\index accesses only for const enums
                    var member = symbol.valueDeclaration;
                    if (ts.isEnumConst(member.parent)) {
                        return getEnumMemberValue(member);
                    }
                }
                return undefined;
            }
            function isFunctionType(type) {
                return !!(type.flags & 524288 /* Object */) && getSignaturesOfType(type, 0 /* Call */).length > 0;
            }
            function getTypeReferenceSerializationKind(typeNameIn, location) {
                // ensure both `typeName` and `location` are parse tree nodes.
                var typeName = ts.getParseTreeNode(typeNameIn, ts.isEntityName);
                if (!typeName)
                    return ts.TypeReferenceSerializationKind.Unknown;
                if (location) {
                    location = ts.getParseTreeNode(location);
                    if (!location)
                        return ts.TypeReferenceSerializationKind.Unknown;
                }
                // Resolve the symbol as a value to ensure the type can be reached at runtime during emit.
                var valueSymbol = resolveEntityName(typeName, 111551 /* Value */, /*ignoreErrors*/ true, /*dontResolveAlias*/ false, location);
                // Resolve the symbol as a type so that we can provide a more useful hint for the type serializer.
                var typeSymbol = resolveEntityName(typeName, 788968 /* Type */, /*ignoreErrors*/ true, /*dontResolveAlias*/ false, location);
                if (valueSymbol && valueSymbol === typeSymbol) {
                    var globalPromiseSymbol = getGlobalPromiseConstructorSymbol(/*reportErrors*/ false);
                    if (globalPromiseSymbol && valueSymbol === globalPromiseSymbol) {
                        return ts.TypeReferenceSerializationKind.Promise;
                    }
                    var constructorType = getTypeOfSymbol(valueSymbol);
                    if (constructorType && isConstructorType(constructorType)) {
                        return ts.TypeReferenceSerializationKind.TypeWithConstructSignatureAndValue;
                    }
                }
                // We might not be able to resolve type symbol so use unknown type in that case (eg error case)
                if (!typeSymbol) {
                    return ts.TypeReferenceSerializationKind.Unknown;
                }
                var type = getDeclaredTypeOfSymbol(typeSymbol);
                if (type === errorType) {
                    return ts.TypeReferenceSerializationKind.Unknown;
                }
                else if (type.flags & 3 /* AnyOrUnknown */) {
                    return ts.TypeReferenceSerializationKind.ObjectType;
                }
                else if (isTypeAssignableToKind(type, 16384 /* Void */ | 98304 /* Nullable */ | 131072 /* Never */)) {
                    return ts.TypeReferenceSerializationKind.VoidNullableOrNeverType;
                }
                else if (isTypeAssignableToKind(type, 528 /* BooleanLike */)) {
                    return ts.TypeReferenceSerializationKind.BooleanType;
                }
                else if (isTypeAssignableToKind(type, 296 /* NumberLike */)) {
                    return ts.TypeReferenceSerializationKind.NumberLikeType;
                }
                else if (isTypeAssignableToKind(type, 2112 /* BigIntLike */)) {
                    return ts.TypeReferenceSerializationKind.BigIntLikeType;
                }
                else if (isTypeAssignableToKind(type, 132 /* StringLike */)) {
                    return ts.TypeReferenceSerializationKind.StringLikeType;
                }
                else if (isTupleType(type)) {
                    return ts.TypeReferenceSerializationKind.ArrayLikeType;
                }
                else if (isTypeAssignableToKind(type, 12288 /* ESSymbolLike */)) {
                    return ts.TypeReferenceSerializationKind.ESSymbolType;
                }
                else if (isFunctionType(type)) {
                    return ts.TypeReferenceSerializationKind.TypeWithCallSignature;
                }
                else if (isArrayType(type)) {
                    return ts.TypeReferenceSerializationKind.ArrayLikeType;
                }
                else {
                    return ts.TypeReferenceSerializationKind.ObjectType;
                }
            }
            function createTypeOfDeclaration(declarationIn, enclosingDeclaration, flags, tracker, addUndefined) {
                var declaration = ts.getParseTreeNode(declarationIn, ts.isVariableLikeOrAccessor);
                if (!declaration) {
                    return ts.createToken(125 /* AnyKeyword */);
                }
                // Get type of the symbol if this is the valid symbol otherwise get type at location
                var symbol = getSymbolOfNode(declaration);
                var type = symbol && !(symbol.flags & (2048 /* TypeLiteral */ | 131072 /* Signature */))
                    ? getWidenedLiteralType(getTypeOfSymbol(symbol))
                    : errorType;
                if (type.flags & 8192 /* UniqueESSymbol */ &&
                    type.symbol === symbol) {
                    flags |= 1048576 /* AllowUniqueESSymbolType */;
                }
                if (addUndefined) {
                    type = getOptionalType(type);
                }
                return nodeBuilder.typeToTypeNode(type, enclosingDeclaration, flags | 1024 /* MultilineObjectLiterals */, tracker);
            }
            function createReturnTypeOfSignatureDeclaration(signatureDeclarationIn, enclosingDeclaration, flags, tracker) {
                var signatureDeclaration = ts.getParseTreeNode(signatureDeclarationIn, ts.isFunctionLike);
                if (!signatureDeclaration) {
                    return ts.createToken(125 /* AnyKeyword */);
                }
                var signature = getSignatureFromDeclaration(signatureDeclaration);
                return nodeBuilder.typeToTypeNode(getReturnTypeOfSignature(signature), enclosingDeclaration, flags | 1024 /* MultilineObjectLiterals */, tracker);
            }
            function createTypeOfExpression(exprIn, enclosingDeclaration, flags, tracker) {
                var expr = ts.getParseTreeNode(exprIn, ts.isExpression);
                if (!expr) {
                    return ts.createToken(125 /* AnyKeyword */);
                }
                var type = getWidenedType(getRegularTypeOfExpression(expr));
                return nodeBuilder.typeToTypeNode(type, enclosingDeclaration, flags | 1024 /* MultilineObjectLiterals */, tracker);
            }
            function hasGlobalName(name) {
                return globals.has(ts.escapeLeadingUnderscores(name));
            }
            function getReferencedValueSymbol(reference, startInDeclarationContainer) {
                var resolvedSymbol = getNodeLinks(reference).resolvedSymbol;
                if (resolvedSymbol) {
                    return resolvedSymbol;
                }
                var location = reference;
                if (startInDeclarationContainer) {
                    // When resolving the name of a declaration as a value, we need to start resolution
                    // at a point outside of the declaration.
                    var parent = reference.parent;
                    if (ts.isDeclaration(parent) && reference === parent.name) {
                        location = getDeclarationContainer(parent);
                    }
                }
                return resolveName(location, reference.escapedText, 111551 /* Value */ | 1048576 /* ExportValue */ | 2097152 /* Alias */, /*nodeNotFoundMessage*/ undefined, /*nameArg*/ undefined, /*isUse*/ true);
            }
            function getReferencedValueDeclaration(referenceIn) {
                if (!ts.isGeneratedIdentifier(referenceIn)) {
                    var reference = ts.getParseTreeNode(referenceIn, ts.isIdentifier);
                    if (reference) {
                        var symbol = getReferencedValueSymbol(reference);
                        if (symbol) {
                            return getExportSymbolOfValueSymbolIfExported(symbol).valueDeclaration;
                        }
                    }
                }
                return undefined;
            }
            function isLiteralConstDeclaration(node) {
                if (ts.isDeclarationReadonly(node) || ts.isVariableDeclaration(node) && ts.isVarConst(node)) {
                    return isFreshLiteralType(getTypeOfSymbol(getSymbolOfNode(node)));
                }
                return false;
            }
            function literalTypeToNode(type, enclosing, tracker) {
                var enumResult = type.flags & 1024 /* EnumLiteral */ ? nodeBuilder.symbolToExpression(type.symbol, 111551 /* Value */, enclosing, /*flags*/ undefined, tracker)
                    : type === trueType ? ts.createTrue() : type === falseType && ts.createFalse();
                return enumResult || ts.createLiteral(type.value);
            }
            function createLiteralConstValue(node, tracker) {
                var type = getTypeOfSymbol(getSymbolOfNode(node));
                return literalTypeToNode(type, node, tracker);
            }
            function createResolver() {
                // this variable and functions that use it are deliberately moved here from the outer scope
                // to avoid scope pollution
                var resolvedTypeReferenceDirectives = host.getResolvedTypeReferenceDirectives();
                var fileToDirective;
                if (resolvedTypeReferenceDirectives) {
                    // populate reverse mapping: file path -> type reference directive that was resolved to this file
                    fileToDirective = ts.createMap();
                    resolvedTypeReferenceDirectives.forEach(function (resolvedDirective, key) {
                        if (!resolvedDirective || !resolvedDirective.resolvedFileName) {
                            return;
                        }
                        var file = host.getSourceFile(resolvedDirective.resolvedFileName);
                        // Add the transitive closure of path references loaded by this file (as long as they are not)
                        // part of an existing type reference.
                        addReferencedFilesToTypeDirective(file, key);
                    });
                }
                return {
                    getReferencedExportContainer: getReferencedExportContainer,
                    getReferencedImportDeclaration: getReferencedImportDeclaration,
                    getReferencedDeclarationWithCollidingName: getReferencedDeclarationWithCollidingName,
                    isDeclarationWithCollidingName: isDeclarationWithCollidingName,
                    isValueAliasDeclaration: function (node) {
                        node = ts.getParseTreeNode(node);
                        // Synthesized nodes are always treated like values.
                        return node ? isValueAliasDeclaration(node) : true;
                    },
                    hasGlobalName: hasGlobalName,
                    isReferencedAliasDeclaration: function (node, checkChildren) {
                        node = ts.getParseTreeNode(node);
                        // Synthesized nodes are always treated as referenced.
                        return node ? isReferencedAliasDeclaration(node, checkChildren) : true;
                    },
                    getNodeCheckFlags: function (node) {
                        node = ts.getParseTreeNode(node);
                        return node ? getNodeCheckFlags(node) : 0;
                    },
                    isTopLevelValueImportEqualsWithEntityName: isTopLevelValueImportEqualsWithEntityName,
                    isDeclarationVisible: isDeclarationVisible,
                    isImplementationOfOverload: isImplementationOfOverload,
                    isRequiredInitializedParameter: isRequiredInitializedParameter,
                    isOptionalUninitializedParameterProperty: isOptionalUninitializedParameterProperty,
                    isExpandoFunctionDeclaration: isExpandoFunctionDeclaration,
                    getPropertiesOfContainerFunction: getPropertiesOfContainerFunction,
                    createTypeOfDeclaration: createTypeOfDeclaration,
                    createReturnTypeOfSignatureDeclaration: createReturnTypeOfSignatureDeclaration,
                    createTypeOfExpression: createTypeOfExpression,
                    createLiteralConstValue: createLiteralConstValue,
                    isSymbolAccessible: isSymbolAccessible,
                    isEntityNameVisible: isEntityNameVisible,
                    getConstantValue: function (nodeIn) {
                        var node = ts.getParseTreeNode(nodeIn, canHaveConstantValue);
                        return node ? getConstantValue(node) : undefined;
                    },
                    collectLinkedAliases: collectLinkedAliases,
                    getReferencedValueDeclaration: getReferencedValueDeclaration,
                    getTypeReferenceSerializationKind: getTypeReferenceSerializationKind,
                    isOptionalParameter: isOptionalParameter,
                    moduleExportsSomeValue: moduleExportsSomeValue,
                    isArgumentsLocalBinding: isArgumentsLocalBinding,
                    getExternalModuleFileFromDeclaration: getExternalModuleFileFromDeclaration,
                    getTypeReferenceDirectivesForEntityName: getTypeReferenceDirectivesForEntityName,
                    getTypeReferenceDirectivesForSymbol: getTypeReferenceDirectivesForSymbol,
                    isLiteralConstDeclaration: isLiteralConstDeclaration,
                    isLateBound: function (nodeIn) {
                        var node = ts.getParseTreeNode(nodeIn, ts.isDeclaration);
                        var symbol = node && getSymbolOfNode(node);
                        return !!(symbol && ts.getCheckFlags(symbol) & 4096 /* Late */);
                    },
                    getJsxFactoryEntity: function (location) { return location ? (getJsxNamespace(location), (ts.getSourceFileOfNode(location).localJsxFactory || _jsxFactoryEntity)) : _jsxFactoryEntity; },
                    getAllAccessorDeclarations: function (accessor) {
                        accessor = ts.getParseTreeNode(accessor, ts.isGetOrSetAccessorDeclaration); // TODO: GH#18217
                        var otherKind = accessor.kind === 164 /* SetAccessor */ ? 163 /* GetAccessor */ : 164 /* SetAccessor */;
                        var otherAccessor = ts.getDeclarationOfKind(getSymbolOfNode(accessor), otherKind);
                        var firstAccessor = otherAccessor && (otherAccessor.pos < accessor.pos) ? otherAccessor : accessor;
                        var secondAccessor = otherAccessor && (otherAccessor.pos < accessor.pos) ? accessor : otherAccessor;
                        var setAccessor = accessor.kind === 164 /* SetAccessor */ ? accessor : otherAccessor;
                        var getAccessor = accessor.kind === 163 /* GetAccessor */ ? accessor : otherAccessor;
                        return {
                            firstAccessor: firstAccessor,
                            secondAccessor: secondAccessor,
                            setAccessor: setAccessor,
                            getAccessor: getAccessor
                        };
                    },
                    getSymbolOfExternalModuleSpecifier: function (moduleName) { return resolveExternalModuleNameWorker(moduleName, moduleName, /*moduleNotFoundError*/ undefined); },
                    isBindingCapturedByNode: function (node, decl) {
                        var parseNode = ts.getParseTreeNode(node);
                        var parseDecl = ts.getParseTreeNode(decl);
                        return !!parseNode && !!parseDecl && (ts.isVariableDeclaration(parseDecl) || ts.isBindingElement(parseDecl)) && isBindingCapturedByNode(parseNode, parseDecl);
                    },
                    getDeclarationStatementsForSourceFile: function (node, flags, tracker, bundled) {
                        var n = ts.getParseTreeNode(node);
                        ts.Debug.assert(n && n.kind === 290 /* SourceFile */, "Non-sourcefile node passed into getDeclarationsForSourceFile");
                        var sym = getSymbolOfNode(node);
                        if (!sym) {
                            return !node.locals ? [] : nodeBuilder.symbolTableToDeclarationStatements(node.locals, node, flags, tracker, bundled);
                        }
                        return !sym.exports ? [] : nodeBuilder.symbolTableToDeclarationStatements(sym.exports, node, flags, tracker, bundled);
                    }
                };
                function isInHeritageClause(node) {
                    return node.parent && node.parent.kind === 216 /* ExpressionWithTypeArguments */ && node.parent.parent && node.parent.parent.kind === 279 /* HeritageClause */;
                }
                // defined here to avoid outer scope pollution
                function getTypeReferenceDirectivesForEntityName(node) {
                    // program does not have any files with type reference directives - bail out
                    if (!fileToDirective) {
                        return undefined;
                    }
                    // property access can only be used as values, or types when within an expression with type arguments inside a heritage clause
                    // qualified names can only be used as types\namespaces
                    // identifiers are treated as values only if they appear in type queries
                    var meaning = 788968 /* Type */ | 1920 /* Namespace */;
                    if ((node.kind === 75 /* Identifier */ && isInTypeQuery(node)) || (node.kind === 194 /* PropertyAccessExpression */ && !isInHeritageClause(node))) {
                        meaning = 111551 /* Value */ | 1048576 /* ExportValue */;
                    }
                    var symbol = resolveEntityName(node, meaning, /*ignoreErrors*/ true);
                    return symbol && symbol !== unknownSymbol ? getTypeReferenceDirectivesForSymbol(symbol, meaning) : undefined;
                }
                // defined here to avoid outer scope pollution
                function getTypeReferenceDirectivesForSymbol(symbol, meaning) {
                    // program does not have any files with type reference directives - bail out
                    if (!fileToDirective) {
                        return undefined;
                    }
                    if (!isSymbolFromTypeDeclarationFile(symbol)) {
                        return undefined;
                    }
                    // check what declarations in the symbol can contribute to the target meaning
                    var typeReferenceDirectives;
                    for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        // check meaning of the local symbol to see if declaration needs to be analyzed further
                        if (decl.symbol && decl.symbol.flags & meaning) {
                            var file = ts.getSourceFileOfNode(decl);
                            var typeReferenceDirective = fileToDirective.get(file.path);
                            if (typeReferenceDirective) {
                                (typeReferenceDirectives || (typeReferenceDirectives = [])).push(typeReferenceDirective);
                            }
                            else {
                                // found at least one entry that does not originate from type reference directive
                                return undefined;
                            }
                        }
                    }
                    return typeReferenceDirectives;
                }
                function isSymbolFromTypeDeclarationFile(symbol) {
                    // bail out if symbol does not have associated declarations (i.e. this is transient symbol created for property in binding pattern)
                    if (!symbol.declarations) {
                        return false;
                    }
                    // walk the parent chain for symbols to make sure that top level parent symbol is in the global scope
                    // external modules cannot define or contribute to type declaration files
                    var current = symbol;
                    while (true) {
                        var parent = getParentOfSymbol(current);
                        if (parent) {
                            current = parent;
                        }
                        else {
                            break;
                        }
                    }
                    if (current.valueDeclaration && current.valueDeclaration.kind === 290 /* SourceFile */ && current.flags & 512 /* ValueModule */) {
                        return false;
                    }
                    // check that at least one declaration of top level symbol originates from type declaration file
                    for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        var file = ts.getSourceFileOfNode(decl);
                        if (fileToDirective.has(file.path)) {
                            return true;
                        }
                    }
                    return false;
                }
                function addReferencedFilesToTypeDirective(file, key) {
                    if (fileToDirective.has(file.path))
                        return;
                    fileToDirective.set(file.path, key);
                    for (var _i = 0, _a = file.referencedFiles; _i < _a.length; _i++) {
                        var fileName = _a[_i].fileName;
                        var resolvedFile = ts.resolveTripleslashReference(fileName, file.originalFileName);
                        var referencedFile = host.getSourceFile(resolvedFile);
                        if (referencedFile) {
                            addReferencedFilesToTypeDirective(referencedFile, key);
                        }
                    }
                }
            }
            function getExternalModuleFileFromDeclaration(declaration) {
                var specifier = declaration.kind === 249 /* ModuleDeclaration */ ? ts.tryCast(declaration.name, ts.isStringLiteral) : ts.getExternalModuleName(declaration);
                var moduleSymbol = resolveExternalModuleNameWorker(specifier, specifier, /*moduleNotFoundError*/ undefined); // TODO: GH#18217
                if (!moduleSymbol) {
                    return undefined;
                }
                return ts.getDeclarationOfKind(moduleSymbol, 290 /* SourceFile */);
            }
            function initializeTypeChecker() {
                // Bind all source files and propagate errors
                for (var _i = 0, _a = host.getSourceFiles(); _i < _a.length; _i++) {
                    var file = _a[_i];
                    ts.bindSourceFile(file, compilerOptions);
                }
                amalgamatedDuplicates = ts.createMap();
                // Initialize global symbol table
                var augmentations;
                for (var _b = 0, _c = host.getSourceFiles(); _b < _c.length; _b++) {
                    var file = _c[_b];
                    if (file.redirectInfo) {
                        continue;
                    }
                    if (!ts.isExternalOrCommonJsModule(file)) {
                        // It is an error for a non-external-module (i.e. script) to declare its own `globalThis`.
                        // We can't use `builtinGlobals` for this due to synthetic expando-namespace generation in JS files.
                        var fileGlobalThisSymbol = file.locals.get("globalThis");
                        if (fileGlobalThisSymbol) {
                            for (var _d = 0, _e = fileGlobalThisSymbol.declarations; _d < _e.length; _d++) {
                                var declaration = _e[_d];
                                diagnostics.add(ts.createDiagnosticForNode(declaration, ts.Diagnostics.Declaration_name_conflicts_with_built_in_global_identifier_0, "globalThis"));
                            }
                        }
                        mergeSymbolTable(globals, file.locals);
                    }
                    if (file.jsGlobalAugmentations) {
                        mergeSymbolTable(globals, file.jsGlobalAugmentations);
                    }
                    if (file.patternAmbientModules && file.patternAmbientModules.length) {
                        patternAmbientModules = ts.concatenate(patternAmbientModules, file.patternAmbientModules);
                    }
                    if (file.moduleAugmentations.length) {
                        (augmentations || (augmentations = [])).push(file.moduleAugmentations);
                    }
                    if (file.symbol && file.symbol.globalExports) {
                        // Merge in UMD exports with first-in-wins semantics (see #9771)
                        var source = file.symbol.globalExports;
                        source.forEach(function (sourceSymbol, id) {
                            if (!globals.has(id)) {
                                globals.set(id, sourceSymbol);
                            }
                        });
                    }
                }
                // We do global augmentations separately from module augmentations (and before creating global types) because they
                //  1. Affect global types. We won't have the correct global types until global augmentations are merged. Also,
                //  2. Module augmentation instantiation requires creating the type of a module, which, in turn, can require
                //       checking for an export or property on the module (if export=) which, in turn, can fall back to the
                //       apparent type of the module - either globalObjectType or globalFunctionType - which wouldn't exist if we
                //       did module augmentations prior to finalizing the global types.
                if (augmentations) {
                    // merge _global_ module augmentations.
                    // this needs to be done after global symbol table is initialized to make sure that all ambient modules are indexed
                    for (var _f = 0, augmentations_1 = augmentations; _f < augmentations_1.length; _f++) {
                        var list = augmentations_1[_f];
                        for (var _g = 0, list_1 = list; _g < list_1.length; _g++) {
                            var augmentation = list_1[_g];
                            if (!ts.isGlobalScopeAugmentation(augmentation.parent))
                                continue;
                            mergeModuleAugmentation(augmentation);
                        }
                    }
                }
                // Setup global builtins
                addToSymbolTable(globals, builtinGlobals, ts.Diagnostics.Declaration_name_conflicts_with_built_in_global_identifier_0);
                getSymbolLinks(undefinedSymbol).type = undefinedWideningType;
                getSymbolLinks(argumentsSymbol).type = getGlobalType("IArguments", /*arity*/ 0, /*reportErrors*/ true);
                getSymbolLinks(unknownSymbol).type = errorType;
                getSymbolLinks(globalThisSymbol).type = createObjectType(16 /* Anonymous */, globalThisSymbol);
                // Initialize special types
                globalArrayType = getGlobalType("Array", /*arity*/ 1, /*reportErrors*/ true);
                globalObjectType = getGlobalType("Object", /*arity*/ 0, /*reportErrors*/ true);
                globalFunctionType = getGlobalType("Function", /*arity*/ 0, /*reportErrors*/ true);
                globalCallableFunctionType = strictBindCallApply && getGlobalType("CallableFunction", /*arity*/ 0, /*reportErrors*/ true) || globalFunctionType;
                globalNewableFunctionType = strictBindCallApply && getGlobalType("NewableFunction", /*arity*/ 0, /*reportErrors*/ true) || globalFunctionType;
                globalStringType = getGlobalType("String", /*arity*/ 0, /*reportErrors*/ true);
                globalNumberType = getGlobalType("Number", /*arity*/ 0, /*reportErrors*/ true);
                globalBooleanType = getGlobalType("Boolean", /*arity*/ 0, /*reportErrors*/ true);
                globalRegExpType = getGlobalType("RegExp", /*arity*/ 0, /*reportErrors*/ true);
                anyArrayType = createArrayType(anyType);
                autoArrayType = createArrayType(autoType);
                if (autoArrayType === emptyObjectType) {
                    // autoArrayType is used as a marker, so even if global Array type is not defined, it needs to be a unique type
                    autoArrayType = createAnonymousType(undefined, emptySymbols, ts.emptyArray, ts.emptyArray, undefined, undefined);
                }
                globalReadonlyArrayType = getGlobalTypeOrUndefined("ReadonlyArray", /*arity*/ 1) || globalArrayType;
                anyReadonlyArrayType = globalReadonlyArrayType ? createTypeFromGenericGlobalType(globalReadonlyArrayType, [anyType]) : anyArrayType;
                globalThisType = getGlobalTypeOrUndefined("ThisType", /*arity*/ 1);
                if (augmentations) {
                    // merge _nonglobal_ module augmentations.
                    // this needs to be done after global symbol table is initialized to make sure that all ambient modules are indexed
                    for (var _h = 0, augmentations_2 = augmentations; _h < augmentations_2.length; _h++) {
                        var list = augmentations_2[_h];
                        for (var _j = 0, list_2 = list; _j < list_2.length; _j++) {
                            var augmentation = list_2[_j];
                            if (ts.isGlobalScopeAugmentation(augmentation.parent))
                                continue;
                            mergeModuleAugmentation(augmentation);
                        }
                    }
                }
                amalgamatedDuplicates.forEach(function (_a) {
                    var firstFile = _a.firstFile, secondFile = _a.secondFile, conflictingSymbols = _a.conflictingSymbols;
                    // If not many things conflict, issue individual errors
                    if (conflictingSymbols.size < 8) {
                        conflictingSymbols.forEach(function (_a, symbolName) {
                            var isBlockScoped = _a.isBlockScoped, firstFileLocations = _a.firstFileLocations, secondFileLocations = _a.secondFileLocations;
                            var message = isBlockScoped ? ts.Diagnostics.Cannot_redeclare_block_scoped_variable_0 : ts.Diagnostics.Duplicate_identifier_0;
                            for (var _i = 0, firstFileLocations_1 = firstFileLocations; _i < firstFileLocations_1.length; _i++) {
                                var node = firstFileLocations_1[_i];
                                addDuplicateDeclarationError(node, message, symbolName, secondFileLocations);
                            }
                            for (var _b = 0, secondFileLocations_1 = secondFileLocations; _b < secondFileLocations_1.length; _b++) {
                                var node = secondFileLocations_1[_b];
                                addDuplicateDeclarationError(node, message, symbolName, firstFileLocations);
                            }
                        });
                    }
                    else {
                        // Otherwise issue top-level error since the files appear very identical in terms of what they contain
                        var list = ts.arrayFrom(conflictingSymbols.keys()).join(", ");
                        diagnostics.add(ts.addRelatedInfo(ts.createDiagnosticForNode(firstFile, ts.Diagnostics.Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0, list), ts.createDiagnosticForNode(secondFile, ts.Diagnostics.Conflicts_are_in_this_file)));
                        diagnostics.add(ts.addRelatedInfo(ts.createDiagnosticForNode(secondFile, ts.Diagnostics.Definitions_of_the_following_identifiers_conflict_with_those_in_another_file_Colon_0, list), ts.createDiagnosticForNode(firstFile, ts.Diagnostics.Conflicts_are_in_this_file)));
                    }
                });
                amalgamatedDuplicates = undefined;
            }
            function checkExternalEmitHelpers(location, helpers) {
                if ((requestedExternalEmitHelpers & helpers) !== helpers && compilerOptions.importHelpers) {
                    var sourceFile = ts.getSourceFileOfNode(location);
                    if (ts.isEffectiveExternalModule(sourceFile, compilerOptions) && !(location.flags & 8388608 /* Ambient */)) {
                        var helpersModule = resolveHelpersModule(sourceFile, location);
                        if (helpersModule !== unknownSymbol) {
                            var uncheckedHelpers = helpers & ~requestedExternalEmitHelpers;
                            for (var helper = 1 /* FirstEmitHelper */; helper <= 524288 /* LastEmitHelper */; helper <<= 1) {
                                if (uncheckedHelpers & helper) {
                                    var name = getHelperName(helper);
                                    var symbol = getSymbol(helpersModule.exports, ts.escapeLeadingUnderscores(name), 111551 /* Value */);
                                    if (!symbol) {
                                        error(location, ts.Diagnostics.This_syntax_requires_an_imported_helper_named_1_which_does_not_exist_in_0_Consider_upgrading_your_version_of_0, ts.externalHelpersModuleNameText, name);
                                    }
                                }
                            }
                        }
                        requestedExternalEmitHelpers |= helpers;
                    }
                }
            }
            function getHelperName(helper) {
                switch (helper) {
                    case 1 /* Extends */: return "__extends";
                    case 2 /* Assign */: return "__assign";
                    case 4 /* Rest */: return "__rest";
                    case 8 /* Decorate */: return "__decorate";
                    case 16 /* Metadata */: return "__metadata";
                    case 32 /* Param */: return "__param";
                    case 64 /* Awaiter */: return "__awaiter";
                    case 128 /* Generator */: return "__generator";
                    case 256 /* Values */: return "__values";
                    case 512 /* Read */: return "__read";
                    case 1024 /* Spread */: return "__spread";
                    case 2048 /* SpreadArrays */: return "__spreadArrays";
                    case 4096 /* Await */: return "__await";
                    case 8192 /* AsyncGenerator */: return "__asyncGenerator";
                    case 16384 /* AsyncDelegator */: return "__asyncDelegator";
                    case 32768 /* AsyncValues */: return "__asyncValues";
                    case 65536 /* ExportStar */: return "__exportStar";
                    case 131072 /* MakeTemplateObject */: return "__makeTemplateObject";
                    case 262144 /* ClassPrivateFieldGet */: return "__classPrivateFieldGet";
                    case 524288 /* ClassPrivateFieldSet */: return "__classPrivateFieldSet";
                    default: return ts.Debug.fail("Unrecognized helper");
                }
            }
            function resolveHelpersModule(node, errorNode) {
                if (!externalHelpersModule) {
                    externalHelpersModule = resolveExternalModule(node, ts.externalHelpersModuleNameText, ts.Diagnostics.This_syntax_requires_an_imported_helper_but_module_0_cannot_be_found, errorNode) || unknownSymbol;
                }
                return externalHelpersModule;
            }
            // GRAMMAR CHECKING
            function checkGrammarDecoratorsAndModifiers(node) {
                return checkGrammarDecorators(node) || checkGrammarModifiers(node);
            }
            function checkGrammarDecorators(node) {
                if (!node.decorators) {
                    return false;
                }
                if (!ts.nodeCanBeDecorated(node, node.parent, node.parent.parent)) {
                    if (node.kind === 161 /* MethodDeclaration */ && !ts.nodeIsPresent(node.body)) {
                        return grammarErrorOnFirstToken(node, ts.Diagnostics.A_decorator_can_only_decorate_a_method_implementation_not_an_overload);
                    }
                    else {
                        return grammarErrorOnFirstToken(node, ts.Diagnostics.Decorators_are_not_valid_here);
                    }
                }
                else if (node.kind === 163 /* GetAccessor */ || node.kind === 164 /* SetAccessor */) {
                    var accessors = ts.getAllAccessorDeclarations(node.parent.members, node);
                    if (accessors.firstAccessor.decorators && node === accessors.secondAccessor) {
                        return grammarErrorOnFirstToken(node, ts.Diagnostics.Decorators_cannot_be_applied_to_multiple_get_Slashset_accessors_of_the_same_name);
                    }
                }
                return false;
            }
            function checkGrammarModifiers(node) {
                var quickResult = reportObviousModifierErrors(node);
                if (quickResult !== undefined) {
                    return quickResult;
                }
                var lastStatic, lastDeclare, lastAsync, lastReadonly;
                var flags = 0 /* None */;
                for (var _i = 0, _a = node.modifiers; _i < _a.length; _i++) {
                    var modifier = _a[_i];
                    if (modifier.kind !== 138 /* ReadonlyKeyword */) {
                        if (node.kind === 158 /* PropertySignature */ || node.kind === 160 /* MethodSignature */) {
                            return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_appear_on_a_type_member, ts.tokenToString(modifier.kind));
                        }
                        if (node.kind === 167 /* IndexSignature */) {
                            return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_appear_on_an_index_signature, ts.tokenToString(modifier.kind));
                        }
                    }
                    switch (modifier.kind) {
                        case 81 /* ConstKeyword */:
                            if (node.kind !== 248 /* EnumDeclaration */) {
                                return grammarErrorOnNode(node, ts.Diagnostics.A_class_member_cannot_have_the_0_keyword, ts.tokenToString(81 /* ConstKeyword */));
                            }
                            break;
                        case 119 /* PublicKeyword */:
                        case 118 /* ProtectedKeyword */:
                        case 117 /* PrivateKeyword */:
                            var text = visibilityToString(ts.modifierToFlag(modifier.kind));
                            if (flags & 28 /* AccessibilityModifier */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics.Accessibility_modifier_already_seen);
                            }
                            else if (flags & 32 /* Static */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_must_precede_1_modifier, text, "static");
                            }
                            else if (flags & 64 /* Readonly */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_must_precede_1_modifier, text, "readonly");
                            }
                            else if (flags & 256 /* Async */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_must_precede_1_modifier, text, "async");
                            }
                            else if (node.parent.kind === 250 /* ModuleBlock */ || node.parent.kind === 290 /* SourceFile */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_appear_on_a_module_or_namespace_element, text);
                            }
                            else if (flags & 128 /* Abstract */) {
                                if (modifier.kind === 117 /* PrivateKeyword */) {
                                    return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_be_used_with_1_modifier, text, "abstract");
                                }
                                else {
                                    return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_must_precede_1_modifier, text, "abstract");
                                }
                            }
                            else if (ts.isPrivateIdentifierPropertyDeclaration(node)) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics.An_accessibility_modifier_cannot_be_used_with_a_private_identifier);
                            }
                            flags |= ts.modifierToFlag(modifier.kind);
                            break;
                        case 120 /* StaticKeyword */:
                            if (flags & 32 /* Static */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_already_seen, "static");
                            }
                            else if (flags & 64 /* Readonly */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_must_precede_1_modifier, "static", "readonly");
                            }
                            else if (flags & 256 /* Async */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_must_precede_1_modifier, "static", "async");
                            }
                            else if (node.parent.kind === 250 /* ModuleBlock */ || node.parent.kind === 290 /* SourceFile */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_appear_on_a_module_or_namespace_element, "static");
                            }
                            else if (node.kind === 156 /* Parameter */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_appear_on_a_parameter, "static");
                            }
                            else if (flags & 128 /* Abstract */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_be_used_with_1_modifier, "static", "abstract");
                            }
                            else if (ts.isPrivateIdentifierPropertyDeclaration(node)) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_be_used_with_a_private_identifier, "static");
                            }
                            flags |= 32 /* Static */;
                            lastStatic = modifier;
                            break;
                        case 138 /* ReadonlyKeyword */:
                            if (flags & 64 /* Readonly */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_already_seen, "readonly");
                            }
                            else if (node.kind !== 159 /* PropertyDeclaration */ && node.kind !== 158 /* PropertySignature */ && node.kind !== 167 /* IndexSignature */ && node.kind !== 156 /* Parameter */) {
                                // If node.kind === SyntaxKind.Parameter, checkParameter report an error if it's not a parameter property.
                                return grammarErrorOnNode(modifier, ts.Diagnostics.readonly_modifier_can_only_appear_on_a_property_declaration_or_index_signature);
                            }
                            flags |= 64 /* Readonly */;
                            lastReadonly = modifier;
                            break;
                        case 89 /* ExportKeyword */:
                            if (flags & 1 /* Export */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_already_seen, "export");
                            }
                            else if (flags & 2 /* Ambient */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_must_precede_1_modifier, "export", "declare");
                            }
                            else if (flags & 128 /* Abstract */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_must_precede_1_modifier, "export", "abstract");
                            }
                            else if (flags & 256 /* Async */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_must_precede_1_modifier, "export", "async");
                            }
                            else if (ts.isClassLike(node.parent)) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_appear_on_a_class_element, "export");
                            }
                            else if (node.kind === 156 /* Parameter */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_appear_on_a_parameter, "export");
                            }
                            flags |= 1 /* Export */;
                            break;
                        case 84 /* DefaultKeyword */:
                            var container = node.parent.kind === 290 /* SourceFile */ ? node.parent : node.parent.parent;
                            if (container.kind === 249 /* ModuleDeclaration */ && !ts.isAmbientModule(container)) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics.A_default_export_can_only_be_used_in_an_ECMAScript_style_module);
                            }
                            flags |= 512 /* Default */;
                            break;
                        case 130 /* DeclareKeyword */:
                            if (flags & 2 /* Ambient */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_already_seen, "declare");
                            }
                            else if (flags & 256 /* Async */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_be_used_in_an_ambient_context, "async");
                            }
                            else if (ts.isClassLike(node.parent) && !ts.isPropertyDeclaration(node)) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_appear_on_a_class_element, "declare");
                            }
                            else if (node.kind === 156 /* Parameter */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_appear_on_a_parameter, "declare");
                            }
                            else if ((node.parent.flags & 8388608 /* Ambient */) && node.parent.kind === 250 /* ModuleBlock */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics.A_declare_modifier_cannot_be_used_in_an_already_ambient_context);
                            }
                            else if (ts.isPrivateIdentifierPropertyDeclaration(node)) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_be_used_with_a_private_identifier, "declare");
                            }
                            flags |= 2 /* Ambient */;
                            lastDeclare = modifier;
                            break;
                        case 122 /* AbstractKeyword */:
                            if (flags & 128 /* Abstract */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_already_seen, "abstract");
                            }
                            if (node.kind !== 245 /* ClassDeclaration */) {
                                if (node.kind !== 161 /* MethodDeclaration */ &&
                                    node.kind !== 159 /* PropertyDeclaration */ &&
                                    node.kind !== 163 /* GetAccessor */ &&
                                    node.kind !== 164 /* SetAccessor */) {
                                    return grammarErrorOnNode(modifier, ts.Diagnostics.abstract_modifier_can_only_appear_on_a_class_method_or_property_declaration);
                                }
                                if (!(node.parent.kind === 245 /* ClassDeclaration */ && ts.hasModifier(node.parent, 128 /* Abstract */))) {
                                    return grammarErrorOnNode(modifier, ts.Diagnostics.Abstract_methods_can_only_appear_within_an_abstract_class);
                                }
                                if (flags & 32 /* Static */) {
                                    return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_be_used_with_1_modifier, "static", "abstract");
                                }
                                if (flags & 8 /* Private */) {
                                    return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_be_used_with_1_modifier, "private", "abstract");
                                }
                            }
                            if (ts.isNamedDeclaration(node) && node.name.kind === 76 /* PrivateIdentifier */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_be_used_with_a_private_identifier, "abstract");
                            }
                            flags |= 128 /* Abstract */;
                            break;
                        case 126 /* AsyncKeyword */:
                            if (flags & 256 /* Async */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_already_seen, "async");
                            }
                            else if (flags & 2 /* Ambient */ || node.parent.flags & 8388608 /* Ambient */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_be_used_in_an_ambient_context, "async");
                            }
                            else if (node.kind === 156 /* Parameter */) {
                                return grammarErrorOnNode(modifier, ts.Diagnostics._0_modifier_cannot_appear_on_a_parameter, "async");
                            }
                            flags |= 256 /* Async */;
                            lastAsync = modifier;
                            break;
                    }
                }
                if (node.kind === 162 /* Constructor */) {
                    if (flags & 32 /* Static */) {
                        return grammarErrorOnNode(lastStatic, ts.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration, "static");
                    }
                    if (flags & 128 /* Abstract */) {
                        return grammarErrorOnNode(lastStatic, ts.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration, "abstract"); // TODO: GH#18217
                    }
                    else if (flags & 256 /* Async */) {
                        return grammarErrorOnNode(lastAsync, ts.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration, "async");
                    }
                    else if (flags & 64 /* Readonly */) {
                        return grammarErrorOnNode(lastReadonly, ts.Diagnostics._0_modifier_cannot_appear_on_a_constructor_declaration, "readonly");
                    }
                    return false;
                }
                else if ((node.kind === 254 /* ImportDeclaration */ || node.kind === 253 /* ImportEqualsDeclaration */) && flags & 2 /* Ambient */) {
                    return grammarErrorOnNode(lastDeclare, ts.Diagnostics.A_0_modifier_cannot_be_used_with_an_import_declaration, "declare");
                }
                else if (node.kind === 156 /* Parameter */ && (flags & 92 /* ParameterPropertyModifier */) && ts.isBindingPattern(node.name)) {
                    return grammarErrorOnNode(node, ts.Diagnostics.A_parameter_property_may_not_be_declared_using_a_binding_pattern);
                }
                else if (node.kind === 156 /* Parameter */ && (flags & 92 /* ParameterPropertyModifier */) && node.dotDotDotToken) {
                    return grammarErrorOnNode(node, ts.Diagnostics.A_parameter_property_cannot_be_declared_using_a_rest_parameter);
                }
                if (flags & 256 /* Async */) {
                    return checkGrammarAsyncModifier(node, lastAsync);
                }
                return false;
            }
            /**
             * true | false: Early return this value from checkGrammarModifiers.
             * undefined: Need to do full checking on the modifiers.
             */
            function reportObviousModifierErrors(node) {
                return !node.modifiers
                    ? false
                    : shouldReportBadModifier(node)
                        ? grammarErrorOnFirstToken(node, ts.Diagnostics.Modifiers_cannot_appear_here)
                        : undefined;
            }
            function shouldReportBadModifier(node) {
                switch (node.kind) {
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                    case 162 /* Constructor */:
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                    case 167 /* IndexSignature */:
                    case 249 /* ModuleDeclaration */:
                    case 254 /* ImportDeclaration */:
                    case 253 /* ImportEqualsDeclaration */:
                    case 260 /* ExportDeclaration */:
                    case 259 /* ExportAssignment */:
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                    case 156 /* Parameter */:
                        return false;
                    default:
                        if (node.parent.kind === 250 /* ModuleBlock */ || node.parent.kind === 290 /* SourceFile */) {
                            return false;
                        }
                        switch (node.kind) {
                            case 244 /* FunctionDeclaration */:
                                return nodeHasAnyModifiersExcept(node, 126 /* AsyncKeyword */);
                            case 245 /* ClassDeclaration */:
                                return nodeHasAnyModifiersExcept(node, 122 /* AbstractKeyword */);
                            case 246 /* InterfaceDeclaration */:
                            case 225 /* VariableStatement */:
                            case 247 /* TypeAliasDeclaration */:
                                return true;
                            case 248 /* EnumDeclaration */:
                                return nodeHasAnyModifiersExcept(node, 81 /* ConstKeyword */);
                            default:
                                ts.Debug.fail();
                                return false;
                        }
                }
            }
            function nodeHasAnyModifiersExcept(node, allowedModifier) {
                return node.modifiers.length > 1 || node.modifiers[0].kind !== allowedModifier;
            }
            function checkGrammarAsyncModifier(node, asyncModifier) {
                switch (node.kind) {
                    case 161 /* MethodDeclaration */:
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                        return false;
                }
                return grammarErrorOnNode(asyncModifier, ts.Diagnostics._0_modifier_cannot_be_used_here, "async");
            }
            function checkGrammarForDisallowedTrailingComma(list, diag) {
                if (diag === void 0) {
                    diag = ts.Diagnostics.Trailing_comma_not_allowed;
                }
                if (list && list.hasTrailingComma) {
                    return grammarErrorAtPos(list[0], list.end - ",".length, ",".length, diag);
                }
                return false;
            }
            function checkGrammarTypeParameterList(typeParameters, file) {
                if (typeParameters && typeParameters.length === 0) {
                    var start = typeParameters.pos - "<".length;
                    var end = ts.skipTrivia(file.text, typeParameters.end) + ">".length;
                    return grammarErrorAtPos(file, start, end - start, ts.Diagnostics.Type_parameter_list_cannot_be_empty);
                }
                return false;
            }
            function checkGrammarParameterList(parameters) {
                var seenOptionalParameter = false;
                var parameterCount = parameters.length;
                for (var i = 0; i < parameterCount; i++) {
                    var parameter = parameters[i];
                    if (parameter.dotDotDotToken) {
                        if (i !== (parameterCount - 1)) {
                            return grammarErrorOnNode(parameter.dotDotDotToken, ts.Diagnostics.A_rest_parameter_must_be_last_in_a_parameter_list);
                        }
                        if (!(parameter.flags & 8388608 /* Ambient */)) { // Allow `...foo,` in ambient declarations; see GH#23070
                            checkGrammarForDisallowedTrailingComma(parameters, ts.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma);
                        }
                        if (parameter.questionToken) {
                            return grammarErrorOnNode(parameter.questionToken, ts.Diagnostics.A_rest_parameter_cannot_be_optional);
                        }
                        if (parameter.initializer) {
                            return grammarErrorOnNode(parameter.name, ts.Diagnostics.A_rest_parameter_cannot_have_an_initializer);
                        }
                    }
                    else if (parameter.questionToken) {
                        seenOptionalParameter = true;
                        if (parameter.initializer) {
                            return grammarErrorOnNode(parameter.name, ts.Diagnostics.Parameter_cannot_have_question_mark_and_initializer);
                        }
                    }
                    else if (seenOptionalParameter && !parameter.initializer) {
                        return grammarErrorOnNode(parameter.name, ts.Diagnostics.A_required_parameter_cannot_follow_an_optional_parameter);
                    }
                }
            }
            function getNonSimpleParameters(parameters) {
                return ts.filter(parameters, function (parameter) { return !!parameter.initializer || ts.isBindingPattern(parameter.name) || ts.isRestParameter(parameter); });
            }
            function checkGrammarForUseStrictSimpleParameterList(node) {
                if (languageVersion >= 3 /* ES2016 */) {
                    var useStrictDirective_1 = node.body && ts.isBlock(node.body) && ts.findUseStrictPrologue(node.body.statements);
                    if (useStrictDirective_1) {
                        var nonSimpleParameters = getNonSimpleParameters(node.parameters);
                        if (ts.length(nonSimpleParameters)) {
                            ts.forEach(nonSimpleParameters, function (parameter) {
                                ts.addRelatedInfo(error(parameter, ts.Diagnostics.This_parameter_is_not_allowed_with_use_strict_directive), ts.createDiagnosticForNode(useStrictDirective_1, ts.Diagnostics.use_strict_directive_used_here));
                            });
                            var diagnostics_1 = nonSimpleParameters.map(function (parameter, index) { return (index === 0 ? ts.createDiagnosticForNode(parameter, ts.Diagnostics.Non_simple_parameter_declared_here) : ts.createDiagnosticForNode(parameter, ts.Diagnostics.and_here)); });
                            ts.addRelatedInfo.apply(void 0, __spreadArrays([error(useStrictDirective_1, ts.Diagnostics.use_strict_directive_cannot_be_used_with_non_simple_parameter_list)], diagnostics_1));
                            return true;
                        }
                    }
                }
                return false;
            }
            function checkGrammarFunctionLikeDeclaration(node) {
                // Prevent cascading error by short-circuit
                var file = ts.getSourceFileOfNode(node);
                return checkGrammarDecoratorsAndModifiers(node) || checkGrammarTypeParameterList(node.typeParameters, file) ||
                    checkGrammarParameterList(node.parameters) || checkGrammarArrowFunction(node, file) ||
                    (ts.isFunctionLikeDeclaration(node) && checkGrammarForUseStrictSimpleParameterList(node));
            }
            function checkGrammarClassLikeDeclaration(node) {
                var file = ts.getSourceFileOfNode(node);
                return checkGrammarClassDeclarationHeritageClauses(node) || checkGrammarTypeParameterList(node.typeParameters, file);
            }
            function checkGrammarArrowFunction(node, file) {
                if (!ts.isArrowFunction(node)) {
                    return false;
                }
                var equalsGreaterThanToken = node.equalsGreaterThanToken;
                var startLine = ts.getLineAndCharacterOfPosition(file, equalsGreaterThanToken.pos).line;
                var endLine = ts.getLineAndCharacterOfPosition(file, equalsGreaterThanToken.end).line;
                return startLine !== endLine && grammarErrorOnNode(equalsGreaterThanToken, ts.Diagnostics.Line_terminator_not_permitted_before_arrow);
            }
            function checkGrammarIndexSignatureParameters(node) {
                var parameter = node.parameters[0];
                if (node.parameters.length !== 1) {
                    if (parameter) {
                        return grammarErrorOnNode(parameter.name, ts.Diagnostics.An_index_signature_must_have_exactly_one_parameter);
                    }
                    else {
                        return grammarErrorOnNode(node, ts.Diagnostics.An_index_signature_must_have_exactly_one_parameter);
                    }
                }
                if (parameter.dotDotDotToken) {
                    return grammarErrorOnNode(parameter.dotDotDotToken, ts.Diagnostics.An_index_signature_cannot_have_a_rest_parameter);
                }
                if (ts.hasModifiers(parameter)) {
                    return grammarErrorOnNode(parameter.name, ts.Diagnostics.An_index_signature_parameter_cannot_have_an_accessibility_modifier);
                }
                if (parameter.questionToken) {
                    return grammarErrorOnNode(parameter.questionToken, ts.Diagnostics.An_index_signature_parameter_cannot_have_a_question_mark);
                }
                if (parameter.initializer) {
                    return grammarErrorOnNode(parameter.name, ts.Diagnostics.An_index_signature_parameter_cannot_have_an_initializer);
                }
                if (!parameter.type) {
                    return grammarErrorOnNode(parameter.name, ts.Diagnostics.An_index_signature_parameter_must_have_a_type_annotation);
                }
                if (parameter.type.kind !== 143 /* StringKeyword */ && parameter.type.kind !== 140 /* NumberKeyword */) {
                    var type = getTypeFromTypeNode(parameter.type);
                    if (type.flags & 4 /* String */ || type.flags & 8 /* Number */) {
                        return grammarErrorOnNode(parameter.name, ts.Diagnostics.An_index_signature_parameter_type_cannot_be_a_type_alias_Consider_writing_0_Colon_1_Colon_2_instead, ts.getTextOfNode(parameter.name), typeToString(type), typeToString(node.type ? getTypeFromTypeNode(node.type) : anyType));
                    }
                    if (type.flags & 1048576 /* Union */ && allTypesAssignableToKind(type, 384 /* StringOrNumberLiteral */, /*strict*/ true)) {
                        return grammarErrorOnNode(parameter.name, ts.Diagnostics.An_index_signature_parameter_type_cannot_be_a_union_type_Consider_using_a_mapped_object_type_instead);
                    }
                    return grammarErrorOnNode(parameter.name, ts.Diagnostics.An_index_signature_parameter_type_must_be_either_string_or_number);
                }
                if (!node.type) {
                    return grammarErrorOnNode(node, ts.Diagnostics.An_index_signature_must_have_a_type_annotation);
                }
                return false;
            }
            function checkGrammarIndexSignature(node) {
                // Prevent cascading error by short-circuit
                return checkGrammarDecoratorsAndModifiers(node) || checkGrammarIndexSignatureParameters(node);
            }
            function checkGrammarForAtLeastOneTypeArgument(node, typeArguments) {
                if (typeArguments && typeArguments.length === 0) {
                    var sourceFile = ts.getSourceFileOfNode(node);
                    var start = typeArguments.pos - "<".length;
                    var end = ts.skipTrivia(sourceFile.text, typeArguments.end) + ">".length;
                    return grammarErrorAtPos(sourceFile, start, end - start, ts.Diagnostics.Type_argument_list_cannot_be_empty);
                }
                return false;
            }
            function checkGrammarTypeArguments(node, typeArguments) {
                return checkGrammarForDisallowedTrailingComma(typeArguments) ||
                    checkGrammarForAtLeastOneTypeArgument(node, typeArguments);
            }
            function checkGrammarTaggedTemplateChain(node) {
                if (node.questionDotToken || node.flags & 32 /* OptionalChain */) {
                    return grammarErrorOnNode(node.template, ts.Diagnostics.Tagged_template_expressions_are_not_permitted_in_an_optional_chain);
                }
                return false;
            }
            function checkGrammarForOmittedArgument(args) {
                if (args) {
                    for (var _i = 0, args_4 = args; _i < args_4.length; _i++) {
                        var arg = args_4[_i];
                        if (arg.kind === 215 /* OmittedExpression */) {
                            return grammarErrorAtPos(arg, arg.pos, 0, ts.Diagnostics.Argument_expression_expected);
                        }
                    }
                }
                return false;
            }
            function checkGrammarArguments(args) {
                return checkGrammarForOmittedArgument(args);
            }
            function checkGrammarHeritageClause(node) {
                var types = node.types;
                if (checkGrammarForDisallowedTrailingComma(types)) {
                    return true;
                }
                if (types && types.length === 0) {
                    var listType = ts.tokenToString(node.token);
                    return grammarErrorAtPos(node, types.pos, 0, ts.Diagnostics._0_list_cannot_be_empty, listType);
                }
                return ts.some(types, checkGrammarExpressionWithTypeArguments);
            }
            function checkGrammarExpressionWithTypeArguments(node) {
                return checkGrammarTypeArguments(node, node.typeArguments);
            }
            function checkGrammarClassDeclarationHeritageClauses(node) {
                var seenExtendsClause = false;
                var seenImplementsClause = false;
                if (!checkGrammarDecoratorsAndModifiers(node) && node.heritageClauses) {
                    for (var _i = 0, _a = node.heritageClauses; _i < _a.length; _i++) {
                        var heritageClause = _a[_i];
                        if (heritageClause.token === 90 /* ExtendsKeyword */) {
                            if (seenExtendsClause) {
                                return grammarErrorOnFirstToken(heritageClause, ts.Diagnostics.extends_clause_already_seen);
                            }
                            if (seenImplementsClause) {
                                return grammarErrorOnFirstToken(heritageClause, ts.Diagnostics.extends_clause_must_precede_implements_clause);
                            }
                            if (heritageClause.types.length > 1) {
                                return grammarErrorOnFirstToken(heritageClause.types[1], ts.Diagnostics.Classes_can_only_extend_a_single_class);
                            }
                            seenExtendsClause = true;
                        }
                        else {
                            ts.Debug.assert(heritageClause.token === 113 /* ImplementsKeyword */);
                            if (seenImplementsClause) {
                                return grammarErrorOnFirstToken(heritageClause, ts.Diagnostics.implements_clause_already_seen);
                            }
                            seenImplementsClause = true;
                        }
                        // Grammar checking heritageClause inside class declaration
                        checkGrammarHeritageClause(heritageClause);
                    }
                }
            }
            function checkGrammarInterfaceDeclaration(node) {
                var seenExtendsClause = false;
                if (node.heritageClauses) {
                    for (var _i = 0, _a = node.heritageClauses; _i < _a.length; _i++) {
                        var heritageClause = _a[_i];
                        if (heritageClause.token === 90 /* ExtendsKeyword */) {
                            if (seenExtendsClause) {
                                return grammarErrorOnFirstToken(heritageClause, ts.Diagnostics.extends_clause_already_seen);
                            }
                            seenExtendsClause = true;
                        }
                        else {
                            ts.Debug.assert(heritageClause.token === 113 /* ImplementsKeyword */);
                            return grammarErrorOnFirstToken(heritageClause, ts.Diagnostics.Interface_declaration_cannot_have_implements_clause);
                        }
                        // Grammar checking heritageClause inside class declaration
                        checkGrammarHeritageClause(heritageClause);
                    }
                }
                return false;
            }
            function checkGrammarComputedPropertyName(node) {
                // If node is not a computedPropertyName, just skip the grammar checking
                if (node.kind !== 154 /* ComputedPropertyName */) {
                    return false;
                }
                var computedPropertyName = node;
                if (computedPropertyName.expression.kind === 209 /* BinaryExpression */ && computedPropertyName.expression.operatorToken.kind === 27 /* CommaToken */) {
                    return grammarErrorOnNode(computedPropertyName.expression, ts.Diagnostics.A_comma_expression_is_not_allowed_in_a_computed_property_name);
                }
                return false;
            }
            function checkGrammarForGenerator(node) {
                if (node.asteriskToken) {
                    ts.Debug.assert(node.kind === 244 /* FunctionDeclaration */ ||
                        node.kind === 201 /* FunctionExpression */ ||
                        node.kind === 161 /* MethodDeclaration */);
                    if (node.flags & 8388608 /* Ambient */) {
                        return grammarErrorOnNode(node.asteriskToken, ts.Diagnostics.Generators_are_not_allowed_in_an_ambient_context);
                    }
                    if (!node.body) {
                        return grammarErrorOnNode(node.asteriskToken, ts.Diagnostics.An_overload_signature_cannot_be_declared_as_a_generator);
                    }
                }
            }
            function checkGrammarForInvalidQuestionMark(questionToken, message) {
                return !!questionToken && grammarErrorOnNode(questionToken, message);
            }
            function checkGrammarForInvalidExclamationToken(exclamationToken, message) {
                return !!exclamationToken && grammarErrorOnNode(exclamationToken, message);
            }
            function checkGrammarObjectLiteralExpression(node, inDestructuring) {
                var seen = ts.createUnderscoreEscapedMap();
                for (var _i = 0, _a = node.properties; _i < _a.length; _i++) {
                    var prop = _a[_i];
                    if (prop.kind === 283 /* SpreadAssignment */) {
                        if (inDestructuring) {
                            // a rest property cannot be destructured any further
                            var expression = ts.skipParentheses(prop.expression);
                            if (ts.isArrayLiteralExpression(expression) || ts.isObjectLiteralExpression(expression)) {
                                return grammarErrorOnNode(prop.expression, ts.Diagnostics.A_rest_element_cannot_contain_a_binding_pattern);
                            }
                        }
                        continue;
                    }
                    var name = prop.name;
                    if (name.kind === 154 /* ComputedPropertyName */) {
                        // If the name is not a ComputedPropertyName, the grammar checking will skip it
                        checkGrammarComputedPropertyName(name);
                    }
                    if (prop.kind === 282 /* ShorthandPropertyAssignment */ && !inDestructuring && prop.objectAssignmentInitializer) {
                        // having objectAssignmentInitializer is only valid in ObjectAssignmentPattern
                        // outside of destructuring it is a syntax error
                        return grammarErrorOnNode(prop.equalsToken, ts.Diagnostics.can_only_be_used_in_an_object_literal_property_inside_a_destructuring_assignment);
                    }
                    if (name.kind === 76 /* PrivateIdentifier */) {
                        return grammarErrorOnNode(name, ts.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies);
                    }
                    // Modifiers are never allowed on properties except for 'async' on a method declaration
                    if (prop.modifiers) {
                        // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
                        for (var _b = 0, _c = prop.modifiers; _b < _c.length; _b++) { // TODO: GH#19955
                            var mod = _c[_b];
                            if (mod.kind !== 126 /* AsyncKeyword */ || prop.kind !== 161 /* MethodDeclaration */) {
                                grammarErrorOnNode(mod, ts.Diagnostics._0_modifier_cannot_be_used_here, ts.getTextOfNode(mod));
                            }
                        }
                    }
                    // ECMA-262 11.1.5 Object Initializer
                    // If previous is not undefined then throw a SyntaxError exception if any of the following conditions are true
                    // a.This production is contained in strict code and IsDataDescriptor(previous) is true and
                    // IsDataDescriptor(propId.descriptor) is true.
                    //    b.IsDataDescriptor(previous) is true and IsAccessorDescriptor(propId.descriptor) is true.
                    //    c.IsAccessorDescriptor(previous) is true and IsDataDescriptor(propId.descriptor) is true.
                    //    d.IsAccessorDescriptor(previous) is true and IsAccessorDescriptor(propId.descriptor) is true
                    // and either both previous and propId.descriptor have[[Get]] fields or both previous and propId.descriptor have[[Set]] fields
                    var currentKind = void 0;
                    switch (prop.kind) {
                        case 282 /* ShorthandPropertyAssignment */:
                            checkGrammarForInvalidExclamationToken(prop.exclamationToken, ts.Diagnostics.A_definite_assignment_assertion_is_not_permitted_in_this_context);
                        // falls through
                        case 281 /* PropertyAssignment */:
                            // Grammar checking for computedPropertyName and shorthandPropertyAssignment
                            checkGrammarForInvalidQuestionMark(prop.questionToken, ts.Diagnostics.An_object_member_cannot_be_declared_optional);
                            if (name.kind === 8 /* NumericLiteral */) {
                                checkGrammarNumericLiteral(name);
                            }
                            currentKind = 4 /* PropertyAssignment */;
                            break;
                        case 161 /* MethodDeclaration */:
                            currentKind = 8 /* Method */;
                            break;
                        case 163 /* GetAccessor */:
                            currentKind = 1 /* GetAccessor */;
                            break;
                        case 164 /* SetAccessor */:
                            currentKind = 2 /* SetAccessor */;
                            break;
                        default:
                            throw ts.Debug.assertNever(prop, "Unexpected syntax kind:" + prop.kind);
                    }
                    if (!inDestructuring) {
                        var effectiveName = ts.getPropertyNameForPropertyNameNode(name);
                        if (effectiveName === undefined) {
                            continue;
                        }
                        var existingKind = seen.get(effectiveName);
                        if (!existingKind) {
                            seen.set(effectiveName, currentKind);
                        }
                        else {
                            if ((currentKind & 12 /* PropertyAssignmentOrMethod */) && (existingKind & 12 /* PropertyAssignmentOrMethod */)) {
                                grammarErrorOnNode(name, ts.Diagnostics.Duplicate_identifier_0, ts.getTextOfNode(name));
                            }
                            else if ((currentKind & 3 /* GetOrSetAccessor */) && (existingKind & 3 /* GetOrSetAccessor */)) {
                                if (existingKind !== 3 /* GetOrSetAccessor */ && currentKind !== existingKind) {
                                    seen.set(effectiveName, currentKind | existingKind);
                                }
                                else {
                                    return grammarErrorOnNode(name, ts.Diagnostics.An_object_literal_cannot_have_multiple_get_Slashset_accessors_with_the_same_name);
                                }
                            }
                            else {
                                return grammarErrorOnNode(name, ts.Diagnostics.An_object_literal_cannot_have_property_and_accessor_with_the_same_name);
                            }
                        }
                    }
                }
            }
            function checkGrammarJsxElement(node) {
                checkGrammarTypeArguments(node, node.typeArguments);
                var seen = ts.createUnderscoreEscapedMap();
                for (var _i = 0, _a = node.attributes.properties; _i < _a.length; _i++) {
                    var attr = _a[_i];
                    if (attr.kind === 275 /* JsxSpreadAttribute */) {
                        continue;
                    }
                    var name = attr.name, initializer = attr.initializer;
                    if (!seen.get(name.escapedText)) {
                        seen.set(name.escapedText, true);
                    }
                    else {
                        return grammarErrorOnNode(name, ts.Diagnostics.JSX_elements_cannot_have_multiple_attributes_with_the_same_name);
                    }
                    if (initializer && initializer.kind === 276 /* JsxExpression */ && !initializer.expression) {
                        return grammarErrorOnNode(initializer, ts.Diagnostics.JSX_attributes_must_only_be_assigned_a_non_empty_expression);
                    }
                }
            }
            function checkGrammarJsxExpression(node) {
                if (node.expression && ts.isCommaSequence(node.expression)) {
                    return grammarErrorOnNode(node.expression, ts.Diagnostics.JSX_expressions_may_not_use_the_comma_operator_Did_you_mean_to_write_an_array);
                }
            }
            function checkGrammarForInOrForOfStatement(forInOrOfStatement) {
                if (checkGrammarStatementInAmbientContext(forInOrOfStatement)) {
                    return true;
                }
                if (forInOrOfStatement.kind === 232 /* ForOfStatement */ && forInOrOfStatement.awaitModifier) {
                    if ((forInOrOfStatement.flags & 32768 /* AwaitContext */) === 0 /* None */) {
                        // use of 'for-await-of' in non-async function
                        var sourceFile = ts.getSourceFileOfNode(forInOrOfStatement);
                        if (!hasParseDiagnostics(sourceFile)) {
                            var diagnostic = ts.createDiagnosticForNode(forInOrOfStatement.awaitModifier, ts.Diagnostics.A_for_await_of_statement_is_only_allowed_within_an_async_function_or_async_generator);
                            var func = ts.getContainingFunction(forInOrOfStatement);
                            if (func && func.kind !== 162 /* Constructor */) {
                                ts.Debug.assert((ts.getFunctionFlags(func) & 2 /* Async */) === 0, "Enclosing function should never be an async function.");
                                var relatedInfo = ts.createDiagnosticForNode(func, ts.Diagnostics.Did_you_mean_to_mark_this_function_as_async);
                                ts.addRelatedInfo(diagnostic, relatedInfo);
                            }
                            diagnostics.add(diagnostic);
                            return true;
                        }
                        return false;
                    }
                }
                if (forInOrOfStatement.initializer.kind === 243 /* VariableDeclarationList */) {
                    var variableList = forInOrOfStatement.initializer;
                    if (!checkGrammarVariableDeclarationList(variableList)) {
                        var declarations = variableList.declarations;
                        // declarations.length can be zero if there is an error in variable declaration in for-of or for-in
                        // See http://www.ecma-international.org/ecma-262/6.0/#sec-for-in-and-for-of-statements for details
                        // For example:
                        //      var let = 10;
                        //      for (let of [1,2,3]) {} // this is invalid ES6 syntax
                        //      for (let in [1,2,3]) {} // this is invalid ES6 syntax
                        // We will then want to skip on grammar checking on variableList declaration
                        if (!declarations.length) {
                            return false;
                        }
                        if (declarations.length > 1) {
                            var diagnostic = forInOrOfStatement.kind === 231 /* ForInStatement */
                                ? ts.Diagnostics.Only_a_single_variable_declaration_is_allowed_in_a_for_in_statement
                                : ts.Diagnostics.Only_a_single_variable_declaration_is_allowed_in_a_for_of_statement;
                            return grammarErrorOnFirstToken(variableList.declarations[1], diagnostic);
                        }
                        var firstDeclaration = declarations[0];
                        if (firstDeclaration.initializer) {
                            var diagnostic = forInOrOfStatement.kind === 231 /* ForInStatement */
                                ? ts.Diagnostics.The_variable_declaration_of_a_for_in_statement_cannot_have_an_initializer
                                : ts.Diagnostics.The_variable_declaration_of_a_for_of_statement_cannot_have_an_initializer;
                            return grammarErrorOnNode(firstDeclaration.name, diagnostic);
                        }
                        if (firstDeclaration.type) {
                            var diagnostic = forInOrOfStatement.kind === 231 /* ForInStatement */
                                ? ts.Diagnostics.The_left_hand_side_of_a_for_in_statement_cannot_use_a_type_annotation
                                : ts.Diagnostics.The_left_hand_side_of_a_for_of_statement_cannot_use_a_type_annotation;
                            return grammarErrorOnNode(firstDeclaration, diagnostic);
                        }
                    }
                }
                return false;
            }
            function checkGrammarAccessor(accessor) {
                if (!(accessor.flags & 8388608 /* Ambient */)) {
                    if (languageVersion < 1 /* ES5 */) {
                        return grammarErrorOnNode(accessor.name, ts.Diagnostics.Accessors_are_only_available_when_targeting_ECMAScript_5_and_higher);
                    }
                    if (accessor.body === undefined && !ts.hasModifier(accessor, 128 /* Abstract */)) {
                        return grammarErrorAtPos(accessor, accessor.end - 1, ";".length, ts.Diagnostics._0_expected, "{");
                    }
                }
                if (accessor.body && ts.hasModifier(accessor, 128 /* Abstract */)) {
                    return grammarErrorOnNode(accessor, ts.Diagnostics.An_abstract_accessor_cannot_have_an_implementation);
                }
                if (accessor.typeParameters) {
                    return grammarErrorOnNode(accessor.name, ts.Diagnostics.An_accessor_cannot_have_type_parameters);
                }
                if (!doesAccessorHaveCorrectParameterCount(accessor)) {
                    return grammarErrorOnNode(accessor.name, accessor.kind === 163 /* GetAccessor */ ?
                        ts.Diagnostics.A_get_accessor_cannot_have_parameters :
                        ts.Diagnostics.A_set_accessor_must_have_exactly_one_parameter);
                }
                if (accessor.kind === 164 /* SetAccessor */) {
                    if (accessor.type) {
                        return grammarErrorOnNode(accessor.name, ts.Diagnostics.A_set_accessor_cannot_have_a_return_type_annotation);
                    }
                    var parameter = ts.Debug.assertDefined(ts.getSetAccessorValueParameter(accessor), "Return value does not match parameter count assertion.");
                    if (parameter.dotDotDotToken) {
                        return grammarErrorOnNode(parameter.dotDotDotToken, ts.Diagnostics.A_set_accessor_cannot_have_rest_parameter);
                    }
                    if (parameter.questionToken) {
                        return grammarErrorOnNode(parameter.questionToken, ts.Diagnostics.A_set_accessor_cannot_have_an_optional_parameter);
                    }
                    if (parameter.initializer) {
                        return grammarErrorOnNode(accessor.name, ts.Diagnostics.A_set_accessor_parameter_cannot_have_an_initializer);
                    }
                }
                return false;
            }
            /** Does the accessor have the right number of parameters?
             * A get accessor has no parameters or a single `this` parameter.
             * A set accessor has one parameter or a `this` parameter and one more parameter.
             */
            function doesAccessorHaveCorrectParameterCount(accessor) {
                return getAccessorThisParameter(accessor) || accessor.parameters.length === (accessor.kind === 163 /* GetAccessor */ ? 0 : 1);
            }
            function getAccessorThisParameter(accessor) {
                if (accessor.parameters.length === (accessor.kind === 163 /* GetAccessor */ ? 1 : 2)) {
                    return ts.getThisParameter(accessor);
                }
            }
            function checkGrammarTypeOperatorNode(node) {
                if (node.operator === 147 /* UniqueKeyword */) {
                    if (node.type.kind !== 144 /* SymbolKeyword */) {
                        return grammarErrorOnNode(node.type, ts.Diagnostics._0_expected, ts.tokenToString(144 /* SymbolKeyword */));
                    }
                    var parent = ts.walkUpParenthesizedTypes(node.parent);
                    switch (parent.kind) {
                        case 242 /* VariableDeclaration */:
                            var decl = parent;
                            if (decl.name.kind !== 75 /* Identifier */) {
                                return grammarErrorOnNode(node, ts.Diagnostics.unique_symbol_types_may_not_be_used_on_a_variable_declaration_with_a_binding_name);
                            }
                            if (!ts.isVariableDeclarationInVariableStatement(decl)) {
                                return grammarErrorOnNode(node, ts.Diagnostics.unique_symbol_types_are_only_allowed_on_variables_in_a_variable_statement);
                            }
                            if (!(decl.parent.flags & 2 /* Const */)) {
                                return grammarErrorOnNode(parent.name, ts.Diagnostics.A_variable_whose_type_is_a_unique_symbol_type_must_be_const);
                            }
                            break;
                        case 159 /* PropertyDeclaration */:
                            if (!ts.hasModifier(parent, 32 /* Static */) ||
                                !ts.hasModifier(parent, 64 /* Readonly */)) {
                                return grammarErrorOnNode(parent.name, ts.Diagnostics.A_property_of_a_class_whose_type_is_a_unique_symbol_type_must_be_both_static_and_readonly);
                            }
                            break;
                        case 158 /* PropertySignature */:
                            if (!ts.hasModifier(parent, 64 /* Readonly */)) {
                                return grammarErrorOnNode(parent.name, ts.Diagnostics.A_property_of_an_interface_or_type_literal_whose_type_is_a_unique_symbol_type_must_be_readonly);
                            }
                            break;
                        default:
                            return grammarErrorOnNode(node, ts.Diagnostics.unique_symbol_types_are_not_allowed_here);
                    }
                }
                else if (node.operator === 138 /* ReadonlyKeyword */) {
                    if (node.type.kind !== 174 /* ArrayType */ && node.type.kind !== 175 /* TupleType */) {
                        return grammarErrorOnFirstToken(node, ts.Diagnostics.readonly_type_modifier_is_only_permitted_on_array_and_tuple_literal_types, ts.tokenToString(144 /* SymbolKeyword */));
                    }
                }
            }
            function checkGrammarForInvalidDynamicName(node, message) {
                if (isNonBindableDynamicName(node)) {
                    return grammarErrorOnNode(node, message);
                }
            }
            function checkGrammarMethod(node) {
                if (checkGrammarFunctionLikeDeclaration(node)) {
                    return true;
                }
                if (node.kind === 161 /* MethodDeclaration */) {
                    if (node.parent.kind === 193 /* ObjectLiteralExpression */) {
                        // We only disallow modifier on a method declaration if it is a property of object-literal-expression
                        if (node.modifiers && !(node.modifiers.length === 1 && ts.first(node.modifiers).kind === 126 /* AsyncKeyword */)) {
                            return grammarErrorOnFirstToken(node, ts.Diagnostics.Modifiers_cannot_appear_here);
                        }
                        else if (checkGrammarForInvalidQuestionMark(node.questionToken, ts.Diagnostics.An_object_member_cannot_be_declared_optional)) {
                            return true;
                        }
                        else if (checkGrammarForInvalidExclamationToken(node.exclamationToken, ts.Diagnostics.A_definite_assignment_assertion_is_not_permitted_in_this_context)) {
                            return true;
                        }
                        else if (node.body === undefined) {
                            return grammarErrorAtPos(node, node.end - 1, ";".length, ts.Diagnostics._0_expected, "{");
                        }
                    }
                    if (checkGrammarForGenerator(node)) {
                        return true;
                    }
                }
                if (ts.isClassLike(node.parent)) {
                    // Technically, computed properties in ambient contexts is disallowed
                    // for property declarations and accessors too, not just methods.
                    // However, property declarations disallow computed names in general,
                    // and accessors are not allowed in ambient contexts in general,
                    // so this error only really matters for methods.
                    if (node.flags & 8388608 /* Ambient */) {
                        return checkGrammarForInvalidDynamicName(node.name, ts.Diagnostics.A_computed_property_name_in_an_ambient_context_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type);
                    }
                    else if (node.kind === 161 /* MethodDeclaration */ && !node.body) {
                        return checkGrammarForInvalidDynamicName(node.name, ts.Diagnostics.A_computed_property_name_in_a_method_overload_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type);
                    }
                }
                else if (node.parent.kind === 246 /* InterfaceDeclaration */) {
                    return checkGrammarForInvalidDynamicName(node.name, ts.Diagnostics.A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type);
                }
                else if (node.parent.kind === 173 /* TypeLiteral */) {
                    return checkGrammarForInvalidDynamicName(node.name, ts.Diagnostics.A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type);
                }
            }
            function checkGrammarBreakOrContinueStatement(node) {
                var current = node;
                while (current) {
                    if (ts.isFunctionLike(current)) {
                        return grammarErrorOnNode(node, ts.Diagnostics.Jump_target_cannot_cross_function_boundary);
                    }
                    switch (current.kind) {
                        case 238 /* LabeledStatement */:
                            if (node.label && current.label.escapedText === node.label.escapedText) {
                                // found matching label - verify that label usage is correct
                                // continue can only target labels that are on iteration statements
                                var isMisplacedContinueLabel = node.kind === 233 /* ContinueStatement */
                                    && !ts.isIterationStatement(current.statement, /*lookInLabeledStatement*/ true);
                                if (isMisplacedContinueLabel) {
                                    return grammarErrorOnNode(node, ts.Diagnostics.A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement);
                                }
                                return false;
                            }
                            break;
                        case 237 /* SwitchStatement */:
                            if (node.kind === 234 /* BreakStatement */ && !node.label) {
                                // unlabeled break within switch statement - ok
                                return false;
                            }
                            break;
                        default:
                            if (ts.isIterationStatement(current, /*lookInLabeledStatement*/ false) && !node.label) {
                                // unlabeled break or continue within iteration statement - ok
                                return false;
                            }
                            break;
                    }
                    current = current.parent;
                }
                if (node.label) {
                    var message = node.kind === 234 /* BreakStatement */
                        ? ts.Diagnostics.A_break_statement_can_only_jump_to_a_label_of_an_enclosing_statement
                        : ts.Diagnostics.A_continue_statement_can_only_jump_to_a_label_of_an_enclosing_iteration_statement;
                    return grammarErrorOnNode(node, message);
                }
                else {
                    var message = node.kind === 234 /* BreakStatement */
                        ? ts.Diagnostics.A_break_statement_can_only_be_used_within_an_enclosing_iteration_or_switch_statement
                        : ts.Diagnostics.A_continue_statement_can_only_be_used_within_an_enclosing_iteration_statement;
                    return grammarErrorOnNode(node, message);
                }
            }
            function checkGrammarBindingElement(node) {
                if (node.dotDotDotToken) {
                    var elements = node.parent.elements;
                    if (node !== ts.last(elements)) {
                        return grammarErrorOnNode(node, ts.Diagnostics.A_rest_element_must_be_last_in_a_destructuring_pattern);
                    }
                    checkGrammarForDisallowedTrailingComma(elements, ts.Diagnostics.A_rest_parameter_or_binding_pattern_may_not_have_a_trailing_comma);
                    if (node.propertyName) {
                        return grammarErrorOnNode(node.name, ts.Diagnostics.A_rest_element_cannot_have_a_property_name);
                    }
                    if (node.initializer) {
                        // Error on equals token which immediately precedes the initializer
                        return grammarErrorAtPos(node, node.initializer.pos - 1, 1, ts.Diagnostics.A_rest_element_cannot_have_an_initializer);
                    }
                }
            }
            function isStringOrNumberLiteralExpression(expr) {
                return ts.isStringOrNumericLiteralLike(expr) ||
                    expr.kind === 207 /* PrefixUnaryExpression */ && expr.operator === 40 /* MinusToken */ &&
                        expr.operand.kind === 8 /* NumericLiteral */;
            }
            function isBigIntLiteralExpression(expr) {
                return expr.kind === 9 /* BigIntLiteral */ ||
                    expr.kind === 207 /* PrefixUnaryExpression */ && expr.operator === 40 /* MinusToken */ &&
                        expr.operand.kind === 9 /* BigIntLiteral */;
            }
            function isSimpleLiteralEnumReference(expr) {
                if ((ts.isPropertyAccessExpression(expr) || (ts.isElementAccessExpression(expr) && isStringOrNumberLiteralExpression(expr.argumentExpression))) &&
                    ts.isEntityNameExpression(expr.expression)) {
                    return !!(checkExpressionCached(expr).flags & 1024 /* EnumLiteral */);
                }
            }
            function checkAmbientInitializer(node) {
                var initializer = node.initializer;
                if (initializer) {
                    var isInvalidInitializer = !(isStringOrNumberLiteralExpression(initializer) ||
                        isSimpleLiteralEnumReference(initializer) ||
                        initializer.kind === 106 /* TrueKeyword */ || initializer.kind === 91 /* FalseKeyword */ ||
                        isBigIntLiteralExpression(initializer));
                    var isConstOrReadonly = ts.isDeclarationReadonly(node) || ts.isVariableDeclaration(node) && ts.isVarConst(node);
                    if (isConstOrReadonly && !node.type) {
                        if (isInvalidInitializer) {
                            return grammarErrorOnNode(initializer, ts.Diagnostics.A_const_initializer_in_an_ambient_context_must_be_a_string_or_numeric_literal_or_literal_enum_reference);
                        }
                    }
                    else {
                        return grammarErrorOnNode(initializer, ts.Diagnostics.Initializers_are_not_allowed_in_ambient_contexts);
                    }
                    if (!isConstOrReadonly || isInvalidInitializer) {
                        return grammarErrorOnNode(initializer, ts.Diagnostics.Initializers_are_not_allowed_in_ambient_contexts);
                    }
                }
            }
            function checkGrammarVariableDeclaration(node) {
                if (node.parent.parent.kind !== 231 /* ForInStatement */ && node.parent.parent.kind !== 232 /* ForOfStatement */) {
                    if (node.flags & 8388608 /* Ambient */) {
                        checkAmbientInitializer(node);
                    }
                    else if (!node.initializer) {
                        if (ts.isBindingPattern(node.name) && !ts.isBindingPattern(node.parent)) {
                            return grammarErrorOnNode(node, ts.Diagnostics.A_destructuring_declaration_must_have_an_initializer);
                        }
                        if (ts.isVarConst(node)) {
                            return grammarErrorOnNode(node, ts.Diagnostics.const_declarations_must_be_initialized);
                        }
                    }
                }
                if (node.exclamationToken && (node.parent.parent.kind !== 225 /* VariableStatement */ || !node.type || node.initializer || node.flags & 8388608 /* Ambient */)) {
                    return grammarErrorOnNode(node.exclamationToken, ts.Diagnostics.Definite_assignment_assertions_can_only_be_used_along_with_a_type_annotation);
                }
                var moduleKind = ts.getEmitModuleKind(compilerOptions);
                if (moduleKind < ts.ModuleKind.ES2015 && moduleKind !== ts.ModuleKind.System && !compilerOptions.noEmit &&
                    !(node.parent.parent.flags & 8388608 /* Ambient */) && ts.hasModifier(node.parent.parent, 1 /* Export */)) {
                    checkESModuleMarker(node.name);
                }
                var checkLetConstNames = (ts.isLet(node) || ts.isVarConst(node));
                // 1. LexicalDeclaration : LetOrConst BindingList ;
                // It is a Syntax Error if the BoundNames of BindingList contains "let".
                // 2. ForDeclaration: ForDeclaration : LetOrConst ForBinding
                // It is a Syntax Error if the BoundNames of ForDeclaration contains "let".
                // It is a SyntaxError if a VariableDeclaration or VariableDeclarationNoIn occurs within strict code
                // and its Identifier is eval or arguments
                return checkLetConstNames && checkGrammarNameInLetOrConstDeclarations(node.name);
            }
            function checkESModuleMarker(name) {
                if (name.kind === 75 /* Identifier */) {
                    if (ts.idText(name) === "__esModule") {
                        return grammarErrorOnNode(name, ts.Diagnostics.Identifier_expected_esModule_is_reserved_as_an_exported_marker_when_transforming_ECMAScript_modules);
                    }
                }
                else {
                    var elements = name.elements;
                    for (var _i = 0, elements_1 = elements; _i < elements_1.length; _i++) {
                        var element = elements_1[_i];
                        if (!ts.isOmittedExpression(element)) {
                            return checkESModuleMarker(element.name);
                        }
                    }
                }
                return false;
            }
            function checkGrammarNameInLetOrConstDeclarations(name) {
                if (name.kind === 75 /* Identifier */) {
                    if (name.originalKeywordKind === 115 /* LetKeyword */) {
                        return grammarErrorOnNode(name, ts.Diagnostics.let_is_not_allowed_to_be_used_as_a_name_in_let_or_const_declarations);
                    }
                }
                else {
                    var elements = name.elements;
                    for (var _i = 0, elements_2 = elements; _i < elements_2.length; _i++) {
                        var element = elements_2[_i];
                        if (!ts.isOmittedExpression(element)) {
                            checkGrammarNameInLetOrConstDeclarations(element.name);
                        }
                    }
                }
                return false;
            }
            function checkGrammarVariableDeclarationList(declarationList) {
                var declarations = declarationList.declarations;
                if (checkGrammarForDisallowedTrailingComma(declarationList.declarations)) {
                    return true;
                }
                if (!declarationList.declarations.length) {
                    return grammarErrorAtPos(declarationList, declarations.pos, declarations.end - declarations.pos, ts.Diagnostics.Variable_declaration_list_cannot_be_empty);
                }
                return false;
            }
            function allowLetAndConstDeclarations(parent) {
                switch (parent.kind) {
                    case 227 /* IfStatement */:
                    case 228 /* DoStatement */:
                    case 229 /* WhileStatement */:
                    case 236 /* WithStatement */:
                    case 230 /* ForStatement */:
                    case 231 /* ForInStatement */:
                    case 232 /* ForOfStatement */:
                        return false;
                    case 238 /* LabeledStatement */:
                        return allowLetAndConstDeclarations(parent.parent);
                }
                return true;
            }
            function checkGrammarForDisallowedLetOrConstStatement(node) {
                if (!allowLetAndConstDeclarations(node.parent)) {
                    if (ts.isLet(node.declarationList)) {
                        return grammarErrorOnNode(node, ts.Diagnostics.let_declarations_can_only_be_declared_inside_a_block);
                    }
                    else if (ts.isVarConst(node.declarationList)) {
                        return grammarErrorOnNode(node, ts.Diagnostics.const_declarations_can_only_be_declared_inside_a_block);
                    }
                }
            }
            function checkGrammarMetaProperty(node) {
                var escapedText = node.name.escapedText;
                switch (node.keywordToken) {
                    case 99 /* NewKeyword */:
                        if (escapedText !== "target") {
                            return grammarErrorOnNode(node.name, ts.Diagnostics._0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2, node.name.escapedText, ts.tokenToString(node.keywordToken), "target");
                        }
                        break;
                    case 96 /* ImportKeyword */:
                        if (escapedText !== "meta") {
                            return grammarErrorOnNode(node.name, ts.Diagnostics._0_is_not_a_valid_meta_property_for_keyword_1_Did_you_mean_2, node.name.escapedText, ts.tokenToString(node.keywordToken), "meta");
                        }
                        break;
                }
            }
            function hasParseDiagnostics(sourceFile) {
                return sourceFile.parseDiagnostics.length > 0;
            }
            function grammarErrorOnFirstToken(node, message, arg0, arg1, arg2) {
                var sourceFile = ts.getSourceFileOfNode(node);
                if (!hasParseDiagnostics(sourceFile)) {
                    var span = ts.getSpanOfTokenAtPosition(sourceFile, node.pos);
                    diagnostics.add(ts.createFileDiagnostic(sourceFile, span.start, span.length, message, arg0, arg1, arg2));
                    return true;
                }
                return false;
            }
            function grammarErrorAtPos(nodeForSourceFile, start, length, message, arg0, arg1, arg2) {
                var sourceFile = ts.getSourceFileOfNode(nodeForSourceFile);
                if (!hasParseDiagnostics(sourceFile)) {
                    diagnostics.add(ts.createFileDiagnostic(sourceFile, start, length, message, arg0, arg1, arg2));
                    return true;
                }
                return false;
            }
            function grammarErrorOnNode(node, message, arg0, arg1, arg2) {
                var sourceFile = ts.getSourceFileOfNode(node);
                if (!hasParseDiagnostics(sourceFile)) {
                    diagnostics.add(ts.createDiagnosticForNode(node, message, arg0, arg1, arg2));
                    return true;
                }
                return false;
            }
            function checkGrammarConstructorTypeParameters(node) {
                var jsdocTypeParameters = ts.isInJSFile(node) ? ts.getJSDocTypeParameterDeclarations(node) : undefined;
                var range = node.typeParameters || jsdocTypeParameters && ts.firstOrUndefined(jsdocTypeParameters);
                if (range) {
                    var pos = range.pos === range.end ? range.pos : ts.skipTrivia(ts.getSourceFileOfNode(node).text, range.pos);
                    return grammarErrorAtPos(node, pos, range.end - pos, ts.Diagnostics.Type_parameters_cannot_appear_on_a_constructor_declaration);
                }
            }
            function checkGrammarConstructorTypeAnnotation(node) {
                var type = ts.getEffectiveReturnTypeNode(node);
                if (type) {
                    return grammarErrorOnNode(type, ts.Diagnostics.Type_annotation_cannot_appear_on_a_constructor_declaration);
                }
            }
            function checkGrammarProperty(node) {
                if (ts.isClassLike(node.parent)) {
                    if (ts.isStringLiteral(node.name) && node.name.text === "constructor") {
                        return grammarErrorOnNode(node.name, ts.Diagnostics.Classes_may_not_have_a_field_named_constructor);
                    }
                    if (checkGrammarForInvalidDynamicName(node.name, ts.Diagnostics.A_computed_property_name_in_a_class_property_declaration_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type)) {
                        return true;
                    }
                    if (languageVersion < 2 /* ES2015 */ && ts.isPrivateIdentifier(node.name)) {
                        return grammarErrorOnNode(node.name, ts.Diagnostics.Private_identifiers_are_only_available_when_targeting_ECMAScript_2015_and_higher);
                    }
                }
                else if (node.parent.kind === 246 /* InterfaceDeclaration */) {
                    if (checkGrammarForInvalidDynamicName(node.name, ts.Diagnostics.A_computed_property_name_in_an_interface_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type)) {
                        return true;
                    }
                    if (node.initializer) {
                        return grammarErrorOnNode(node.initializer, ts.Diagnostics.An_interface_property_cannot_have_an_initializer);
                    }
                }
                else if (node.parent.kind === 173 /* TypeLiteral */) {
                    if (checkGrammarForInvalidDynamicName(node.name, ts.Diagnostics.A_computed_property_name_in_a_type_literal_must_refer_to_an_expression_whose_type_is_a_literal_type_or_a_unique_symbol_type)) {
                        return true;
                    }
                    if (node.initializer) {
                        return grammarErrorOnNode(node.initializer, ts.Diagnostics.A_type_literal_property_cannot_have_an_initializer);
                    }
                }
                if (node.flags & 8388608 /* Ambient */) {
                    checkAmbientInitializer(node);
                }
                if (ts.isPropertyDeclaration(node) && node.exclamationToken && (!ts.isClassLike(node.parent) || !node.type || node.initializer ||
                    node.flags & 8388608 /* Ambient */ || ts.hasModifier(node, 32 /* Static */ | 128 /* Abstract */))) {
                    return grammarErrorOnNode(node.exclamationToken, ts.Diagnostics.A_definite_assignment_assertion_is_not_permitted_in_this_context);
                }
            }
            function checkGrammarTopLevelElementForRequiredDeclareModifier(node) {
                // A declare modifier is required for any top level .d.ts declaration except export=, export default, export as namespace
                // interfaces and imports categories:
                //
                //  DeclarationElement:
                //     ExportAssignment
                //     export_opt   InterfaceDeclaration
                //     export_opt   TypeAliasDeclaration
                //     export_opt   ImportDeclaration
                //     export_opt   ExternalImportDeclaration
                //     export_opt   AmbientDeclaration
                //
                // TODO: The spec needs to be amended to reflect this grammar.
                if (node.kind === 246 /* InterfaceDeclaration */ ||
                    node.kind === 247 /* TypeAliasDeclaration */ ||
                    node.kind === 254 /* ImportDeclaration */ ||
                    node.kind === 253 /* ImportEqualsDeclaration */ ||
                    node.kind === 260 /* ExportDeclaration */ ||
                    node.kind === 259 /* ExportAssignment */ ||
                    node.kind === 252 /* NamespaceExportDeclaration */ ||
                    ts.hasModifier(node, 2 /* Ambient */ | 1 /* Export */ | 512 /* Default */)) {
                    return false;
                }
                return grammarErrorOnFirstToken(node, ts.Diagnostics.Top_level_declarations_in_d_ts_files_must_start_with_either_a_declare_or_export_modifier);
            }
            function checkGrammarTopLevelElementsForRequiredDeclareModifier(file) {
                for (var _i = 0, _a = file.statements; _i < _a.length; _i++) {
                    var decl = _a[_i];
                    if (ts.isDeclaration(decl) || decl.kind === 225 /* VariableStatement */) {
                        if (checkGrammarTopLevelElementForRequiredDeclareModifier(decl)) {
                            return true;
                        }
                    }
                }
                return false;
            }
            function checkGrammarSourceFile(node) {
                return !!(node.flags & 8388608 /* Ambient */) && checkGrammarTopLevelElementsForRequiredDeclareModifier(node);
            }
            function checkGrammarStatementInAmbientContext(node) {
                if (node.flags & 8388608 /* Ambient */) {
                    // Find containing block which is either Block, ModuleBlock, SourceFile
                    var links = getNodeLinks(node);
                    if (!links.hasReportedStatementInAmbientContext && (ts.isFunctionLike(node.parent) || ts.isAccessor(node.parent))) {
                        return getNodeLinks(node).hasReportedStatementInAmbientContext = grammarErrorOnFirstToken(node, ts.Diagnostics.An_implementation_cannot_be_declared_in_ambient_contexts);
                    }
                    // We are either parented by another statement, or some sort of block.
                    // If we're in a block, we only want to really report an error once
                    // to prevent noisiness.  So use a bit on the block to indicate if
                    // this has already been reported, and don't report if it has.
                    //
                    if (node.parent.kind === 223 /* Block */ || node.parent.kind === 250 /* ModuleBlock */ || node.parent.kind === 290 /* SourceFile */) {
                        var links_2 = getNodeLinks(node.parent);
                        // Check if the containing block ever report this error
                        if (!links_2.hasReportedStatementInAmbientContext) {
                            return links_2.hasReportedStatementInAmbientContext = grammarErrorOnFirstToken(node, ts.Diagnostics.Statements_are_not_allowed_in_ambient_contexts);
                        }
                    }
                    else {
                        // We must be parented by a statement.  If so, there's no need
                        // to report the error as our parent will have already done it.
                        // Debug.assert(isStatement(node.parent));
                    }
                }
                return false;
            }
            function checkGrammarNumericLiteral(node) {
                // Grammar checking
                if (node.numericLiteralFlags & 32 /* Octal */) {
                    var diagnosticMessage = void 0;
                    if (languageVersion >= 1 /* ES5 */) {
                        diagnosticMessage = ts.Diagnostics.Octal_literals_are_not_available_when_targeting_ECMAScript_5_and_higher_Use_the_syntax_0;
                    }
                    else if (ts.isChildOfNodeWithKind(node, 187 /* LiteralType */)) {
                        diagnosticMessage = ts.Diagnostics.Octal_literal_types_must_use_ES2015_syntax_Use_the_syntax_0;
                    }
                    else if (ts.isChildOfNodeWithKind(node, 284 /* EnumMember */)) {
                        diagnosticMessage = ts.Diagnostics.Octal_literals_are_not_allowed_in_enums_members_initializer_Use_the_syntax_0;
                    }
                    if (diagnosticMessage) {
                        var withMinus = ts.isPrefixUnaryExpression(node.parent) && node.parent.operator === 40 /* MinusToken */;
                        var literal = (withMinus ? "-" : "") + "0o" + node.text;
                        return grammarErrorOnNode(withMinus ? node.parent : node, diagnosticMessage, literal);
                    }
                }
                // Realism (size) checking
                checkNumericLiteralValueSize(node);
                return false;
            }
            function checkNumericLiteralValueSize(node) {
                // Scientific notation (e.g. 2e54 and 1e00000000010) can't be converted to bigint
                // Literals with 15 or fewer characters aren't long enough to reach past 2^53 - 1
                // Fractional numbers (e.g. 9000000000000000.001) are inherently imprecise anyway
                if (node.numericLiteralFlags & 16 /* Scientific */ || node.text.length <= 15 || node.text.indexOf(".") !== -1) {
                    return;
                }
                // We can't rely on the runtime to accurately store and compare extremely large numeric values
                // Even for internal use, we use getTextOfNode: https://github.com/microsoft/TypeScript/issues/33298
                // Thus, if the runtime claims a too-large number is lower than Number.MAX_SAFE_INTEGER,
                // it's likely addition operations on it will fail too
                var apparentValue = +ts.getTextOfNode(node);
                if (apparentValue <= Math.pow(2, 53) - 1 && apparentValue + 1 > apparentValue) {
                    return;
                }
                addErrorOrSuggestion(/*isError*/ false, ts.createDiagnosticForNode(node, ts.Diagnostics.Numeric_literals_with_absolute_values_equal_to_2_53_or_greater_are_too_large_to_be_represented_accurately_as_integers));
            }
            function checkGrammarBigIntLiteral(node) {
                var literalType = ts.isLiteralTypeNode(node.parent) ||
                    ts.isPrefixUnaryExpression(node.parent) && ts.isLiteralTypeNode(node.parent.parent);
                if (!literalType) {
                    if (languageVersion < 7 /* ES2020 */) {
                        if (grammarErrorOnNode(node, ts.Diagnostics.BigInt_literals_are_not_available_when_targeting_lower_than_ES2020)) {
                            return true;
                        }
                    }
                }
                return false;
            }
            function grammarErrorAfterFirstToken(node, message, arg0, arg1, arg2) {
                var sourceFile = ts.getSourceFileOfNode(node);
                if (!hasParseDiagnostics(sourceFile)) {
                    var span = ts.getSpanOfTokenAtPosition(sourceFile, node.pos);
                    diagnostics.add(ts.createFileDiagnostic(sourceFile, ts.textSpanEnd(span), /*length*/ 0, message, arg0, arg1, arg2));
                    return true;
                }
                return false;
            }
            function getAmbientModules() {
                if (!ambientModulesCache) {
                    ambientModulesCache = [];
                    globals.forEach(function (global, sym) {
                        // No need to `unescapeLeadingUnderscores`, an escaped symbol is never an ambient module.
                        if (ambientModuleSymbolRegex.test(sym)) {
                            ambientModulesCache.push(global);
                        }
                    });
                }
                return ambientModulesCache;
            }
            function checkGrammarImportClause(node) {
                if (node.isTypeOnly && node.name && node.namedBindings) {
                    return grammarErrorOnNode(node, ts.Diagnostics.A_type_only_import_can_specify_a_default_import_or_named_bindings_but_not_both);
                }
                return false;
            }
            function checkGrammarImportCallExpression(node) {
                if (moduleKind === ts.ModuleKind.ES2015) {
                    return grammarErrorOnNode(node, ts.Diagnostics.Dynamic_imports_are_only_supported_when_the_module_flag_is_set_to_es2020_esnext_commonjs_amd_system_or_umd);
                }
                if (node.typeArguments) {
                    return grammarErrorOnNode(node, ts.Diagnostics.Dynamic_import_cannot_have_type_arguments);
                }
                var nodeArguments = node.arguments;
                if (nodeArguments.length !== 1) {
                    return grammarErrorOnNode(node, ts.Diagnostics.Dynamic_import_must_have_one_specifier_as_an_argument);
                }
                checkGrammarForDisallowedTrailingComma(nodeArguments);
                // see: parseArgumentOrArrayLiteralElement...we use this function which parse arguments of callExpression to parse specifier for dynamic import.
                // parseArgumentOrArrayLiteralElement allows spread element to be in an argument list which is not allowed as specifier in dynamic import.
                if (ts.isSpreadElement(nodeArguments[0])) {
                    return grammarErrorOnNode(nodeArguments[0], ts.Diagnostics.Specifier_of_dynamic_import_cannot_be_spread_element);
                }
                return false;
            }
            function findMatchingTypeReferenceOrTypeAliasReference(source, unionTarget) {
                var sourceObjectFlags = ts.getObjectFlags(source);
                if (sourceObjectFlags & (4 /* Reference */ | 16 /* Anonymous */) && unionTarget.flags & 1048576 /* Union */) {
                    return ts.find(unionTarget.types, function (target) {
                        if (target.flags & 524288 /* Object */) {
                            var overlapObjFlags = sourceObjectFlags & ts.getObjectFlags(target);
                            if (overlapObjFlags & 4 /* Reference */) {
                                return source.target === target.target;
                            }
                            if (overlapObjFlags & 16 /* Anonymous */) {
                                return !!source.aliasSymbol && source.aliasSymbol === target.aliasSymbol;
                            }
                        }
                        return false;
                    });
                }
            }
            function findBestTypeForObjectLiteral(source, unionTarget) {
                if (ts.getObjectFlags(source) & 128 /* ObjectLiteral */ && forEachType(unionTarget, isArrayLikeType)) {
                    return ts.find(unionTarget.types, function (t) { return !isArrayLikeType(t); });
                }
            }
            function findBestTypeForInvokable(source, unionTarget) {
                var signatureKind = 0 /* Call */;
                var hasSignatures = getSignaturesOfType(source, signatureKind).length > 0 ||
                    (signatureKind = 1 /* Construct */, getSignaturesOfType(source, signatureKind).length > 0);
                if (hasSignatures) {
                    return ts.find(unionTarget.types, function (t) { return getSignaturesOfType(t, signatureKind).length > 0; });
                }
            }
            function findMostOverlappyType(source, unionTarget) {
                var bestMatch;
                var matchingCount = 0;
                for (var _i = 0, _a = unionTarget.types; _i < _a.length; _i++) {
                    var target = _a[_i];
                    var overlap = getIntersectionType([getIndexType(source), getIndexType(target)]);
                    if (overlap.flags & 4194304 /* Index */) {
                        // perfect overlap of keys
                        bestMatch = target;
                        matchingCount = Infinity;
                    }
                    else if (overlap.flags & 1048576 /* Union */) {
                        // We only want to account for literal types otherwise.
                        // If we have a union of index types, it seems likely that we
                        // needed to elaborate between two generic mapped types anyway.
                        var len = ts.length(ts.filter(overlap.types, isUnitType));
                        if (len >= matchingCount) {
                            bestMatch = target;
                            matchingCount = len;
                        }
                    }
                    else if (isUnitType(overlap) && 1 >= matchingCount) {
                        bestMatch = target;
                        matchingCount = 1;
                    }
                }
                return bestMatch;
            }
            function filterPrimitivesIfContainsNonPrimitive(type) {
                if (maybeTypeOfKind(type, 67108864 /* NonPrimitive */)) {
                    var result = filterType(type, function (t) { return !(t.flags & 131068 /* Primitive */); });
                    if (!(result.flags & 131072 /* Never */)) {
                        return result;
                    }
                }
                return type;
            }
            // Keep this up-to-date with the same logic within `getApparentTypeOfContextualType`, since they should behave similarly
            function findMatchingDiscriminantType(source, target, isRelatedTo) {
                if (target.flags & 1048576 /* Union */ && source.flags & (2097152 /* Intersection */ | 524288 /* Object */)) {
                    var sourceProperties = getPropertiesOfType(source);
                    if (sourceProperties) {
                        var sourcePropertiesFiltered = findDiscriminantProperties(sourceProperties, target);
                        if (sourcePropertiesFiltered) {
                            return discriminateTypeByDiscriminableItems(target, ts.map(sourcePropertiesFiltered, function (p) { return [function () { return getTypeOfSymbol(p); }, p.escapedName]; }), isRelatedTo);
                        }
                    }
                }
                return undefined;
            }
        }
        ts.createTypeChecker = createTypeChecker;
        function isNotAccessor(declaration) {
            // Accessors check for their own matching duplicates, and in contexts where they are valid, there are already duplicate identifier checks
            return !ts.isAccessor(declaration);
        }
        function isNotOverload(declaration) {
            return (declaration.kind !== 244 /* FunctionDeclaration */ && declaration.kind !== 161 /* MethodDeclaration */) ||
                !!declaration.body;
        }
        /** Like 'isDeclarationName', but returns true for LHS of `import { x as y }` or `export { x as y }`. */
        function isDeclarationNameOrImportPropertyName(name) {
            switch (name.parent.kind) {
                case 258 /* ImportSpecifier */:
                case 263 /* ExportSpecifier */:
                    return ts.isIdentifier(name);
                default:
                    return ts.isDeclarationName(name);
            }
        }
        function isSomeImportDeclaration(decl) {
            switch (decl.kind) {
                case 255 /* ImportClause */: // For default import
                case 253 /* ImportEqualsDeclaration */:
                case 256 /* NamespaceImport */:
                case 258 /* ImportSpecifier */: // For rename import `x as y`
                    return true;
                case 75 /* Identifier */:
                    // For regular import, `decl` is an Identifier under the ImportSpecifier.
                    return decl.parent.kind === 258 /* ImportSpecifier */;
                default:
                    return false;
            }
        }
        var JsxNames;
        (function (JsxNames) {
            JsxNames.JSX = "JSX";
            JsxNames.IntrinsicElements = "IntrinsicElements";
            JsxNames.ElementClass = "ElementClass";
            JsxNames.ElementAttributesPropertyNameContainer = "ElementAttributesProperty"; // TODO: Deprecate and remove support
            JsxNames.ElementChildrenAttributeNameContainer = "ElementChildrenAttribute";
            JsxNames.Element = "Element";
            JsxNames.IntrinsicAttributes = "IntrinsicAttributes";
            JsxNames.IntrinsicClassAttributes = "IntrinsicClassAttributes";
            JsxNames.LibraryManagedAttributes = "LibraryManagedAttributes";
        })(JsxNames || (JsxNames = {}));
        function getIterationTypesKeyFromIterationTypeKind(typeKind) {
            switch (typeKind) {
                case 0 /* Yield */: return "yieldType";
                case 1 /* Return */: return "returnType";
                case 2 /* Next */: return "nextType";
            }
        }
        function signatureHasRestParameter(s) {
            return !!(s.flags & 1 /* HasRestParameter */);
        }
        ts.signatureHasRestParameter = signatureHasRestParameter;
        function signatureHasLiteralTypes(s) {
            return !!(s.flags & 2 /* HasLiteralTypes */);
        }
        ts.signatureHasLiteralTypes = signatureHasLiteralTypes;
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        function createSynthesizedNode(kind) {
            var node = ts.createNode(kind, -1, -1);
            node.flags |= 8 /* Synthesized */;
            return node;
        }
        /* @internal */
        function updateNode(updated, original) {
            if (updated !== original) {
                setOriginalNode(updated, original);
                setTextRange(updated, original);
                ts.aggregateTransformFlags(updated);
            }
            return updated;
        }
        ts.updateNode = updateNode;
        /**
         * Make `elements` into a `NodeArray<T>`. If `elements` is `undefined`, returns an empty `NodeArray<T>`.
         */
        function createNodeArray(elements, hasTrailingComma) {
            if (!elements || elements === ts.emptyArray) {
                elements = [];
            }
            else if (ts.isNodeArray(elements)) {
                return elements;
            }
            var array = elements;
            array.pos = -1;
            array.end = -1;
            array.hasTrailingComma = hasTrailingComma;
            return array;
        }
        ts.createNodeArray = createNodeArray;
        /**
         * Creates a shallow, memberwise clone of a node with no source map location.
         */
        /* @internal */
        function getSynthesizedClone(node) {
            // We don't use "clone" from core.ts here, as we need to preserve the prototype chain of
            // the original node. We also need to exclude specific properties and only include own-
            // properties (to skip members already defined on the shared prototype).
            if (node === undefined) {
                return node;
            }
            var clone = createSynthesizedNode(node.kind);
            clone.flags |= node.flags;
            setOriginalNode(clone, node);
            for (var key in node) {
                if (clone.hasOwnProperty(key) || !node.hasOwnProperty(key)) {
                    continue;
                }
                clone[key] = node[key];
            }
            return clone;
        }
        ts.getSynthesizedClone = getSynthesizedClone;
        function createLiteral(value, isSingleQuote) {
            if (typeof value === "number") {
                return createNumericLiteral(value + "");
            }
            // eslint-disable-next-line no-in-operator
            if (typeof value === "object" && "base10Value" in value) { // PseudoBigInt
                return createBigIntLiteral(ts.pseudoBigIntToString(value) + "n");
            }
            if (typeof value === "boolean") {
                return value ? createTrue() : createFalse();
            }
            if (ts.isString(value)) {
                var res = createStringLiteral(value);
                if (isSingleQuote)
                    res.singleQuote = true;
                return res;
            }
            return createLiteralFromNode(value);
        }
        ts.createLiteral = createLiteral;
        function createNumericLiteral(value, numericLiteralFlags) {
            if (numericLiteralFlags === void 0) {
                numericLiteralFlags = 0 /* None */;
            }
            var node = createSynthesizedNode(8 /* NumericLiteral */);
            node.text = value;
            node.numericLiteralFlags = numericLiteralFlags;
            return node;
        }
        ts.createNumericLiteral = createNumericLiteral;
        function createBigIntLiteral(value) {
            var node = createSynthesizedNode(9 /* BigIntLiteral */);
            node.text = value;
            return node;
        }
        ts.createBigIntLiteral = createBigIntLiteral;
        function createStringLiteral(text) {
            var node = createSynthesizedNode(10 /* StringLiteral */);
            node.text = text;
            return node;
        }
        ts.createStringLiteral = createStringLiteral;
        function createRegularExpressionLiteral(text) {
            var node = createSynthesizedNode(13 /* RegularExpressionLiteral */);
            node.text = text;
            return node;
        }
        ts.createRegularExpressionLiteral = createRegularExpressionLiteral;
        function createLiteralFromNode(sourceNode) {
            var node = createStringLiteral(ts.getTextOfIdentifierOrLiteral(sourceNode));
            node.textSourceNode = sourceNode;
            return node;
        }
        function createIdentifier(text, typeArguments) {
            var node = createSynthesizedNode(75 /* Identifier */);
            node.escapedText = ts.escapeLeadingUnderscores(text);
            node.originalKeywordKind = text ? ts.stringToToken(text) : 0 /* Unknown */;
            node.autoGenerateFlags = 0 /* None */;
            node.autoGenerateId = 0;
            if (typeArguments) {
                node.typeArguments = createNodeArray(typeArguments);
            }
            return node;
        }
        ts.createIdentifier = createIdentifier;
        function updateIdentifier(node, typeArguments) {
            return node.typeArguments !== typeArguments
                ? updateNode(createIdentifier(ts.idText(node), typeArguments), node)
                : node;
        }
        ts.updateIdentifier = updateIdentifier;
        var nextAutoGenerateId = 0;
        function createTempVariable(recordTempVariable, reservedInNestedScopes) {
            var name = createIdentifier("");
            name.autoGenerateFlags = 1 /* Auto */;
            name.autoGenerateId = nextAutoGenerateId;
            nextAutoGenerateId++;
            if (recordTempVariable) {
                recordTempVariable(name);
            }
            if (reservedInNestedScopes) {
                name.autoGenerateFlags |= 8 /* ReservedInNestedScopes */;
            }
            return name;
        }
        ts.createTempVariable = createTempVariable;
        /** Create a unique temporary variable for use in a loop. */
        function createLoopVariable() {
            var name = createIdentifier("");
            name.autoGenerateFlags = 2 /* Loop */;
            name.autoGenerateId = nextAutoGenerateId;
            nextAutoGenerateId++;
            return name;
        }
        ts.createLoopVariable = createLoopVariable;
        /** Create a unique name based on the supplied text. */
        function createUniqueName(text) {
            var name = createIdentifier(text);
            name.autoGenerateFlags = 3 /* Unique */;
            name.autoGenerateId = nextAutoGenerateId;
            nextAutoGenerateId++;
            return name;
        }
        ts.createUniqueName = createUniqueName;
        function createOptimisticUniqueName(text) {
            var name = createIdentifier(text);
            name.autoGenerateFlags = 3 /* Unique */ | 16 /* Optimistic */;
            name.autoGenerateId = nextAutoGenerateId;
            nextAutoGenerateId++;
            return name;
        }
        ts.createOptimisticUniqueName = createOptimisticUniqueName;
        /** Create a unique name based on the supplied text. This does not consider names injected by the transformer. */
        function createFileLevelUniqueName(text) {
            var name = createOptimisticUniqueName(text);
            name.autoGenerateFlags |= 32 /* FileLevel */;
            return name;
        }
        ts.createFileLevelUniqueName = createFileLevelUniqueName;
        function getGeneratedNameForNode(node, flags) {
            var name = createIdentifier(node && ts.isIdentifier(node) ? ts.idText(node) : "");
            name.autoGenerateFlags = 4 /* Node */ | flags;
            name.autoGenerateId = nextAutoGenerateId;
            name.original = node;
            nextAutoGenerateId++;
            return name;
        }
        ts.getGeneratedNameForNode = getGeneratedNameForNode;
        // Private Identifiers
        function createPrivateIdentifier(text) {
            if (text[0] !== "#") {
                ts.Debug.fail("First character of private identifier must be #: " + text);
            }
            var node = createSynthesizedNode(76 /* PrivateIdentifier */);
            node.escapedText = ts.escapeLeadingUnderscores(text);
            return node;
        }
        ts.createPrivateIdentifier = createPrivateIdentifier;
        // Punctuation
        function createToken(token) {
            return createSynthesizedNode(token);
        }
        ts.createToken = createToken;
        // Reserved words
        function createSuper() {
            return createSynthesizedNode(102 /* SuperKeyword */);
        }
        ts.createSuper = createSuper;
        function createThis() {
            return createSynthesizedNode(104 /* ThisKeyword */);
        }
        ts.createThis = createThis;
        function createNull() {
            return createSynthesizedNode(100 /* NullKeyword */);
        }
        ts.createNull = createNull;
        function createTrue() {
            return createSynthesizedNode(106 /* TrueKeyword */);
        }
        ts.createTrue = createTrue;
        function createFalse() {
            return createSynthesizedNode(91 /* FalseKeyword */);
        }
        ts.createFalse = createFalse;
        // Modifiers
        function createModifier(kind) {
            return createToken(kind);
        }
        ts.createModifier = createModifier;
        function createModifiersFromModifierFlags(flags) {
            var result = [];
            if (flags & 1 /* Export */) {
                result.push(createModifier(89 /* ExportKeyword */));
            }
            if (flags & 2 /* Ambient */) {
                result.push(createModifier(130 /* DeclareKeyword */));
            }
            if (flags & 512 /* Default */) {
                result.push(createModifier(84 /* DefaultKeyword */));
            }
            if (flags & 2048 /* Const */) {
                result.push(createModifier(81 /* ConstKeyword */));
            }
            if (flags & 4 /* Public */) {
                result.push(createModifier(119 /* PublicKeyword */));
            }
            if (flags & 8 /* Private */) {
                result.push(createModifier(117 /* PrivateKeyword */));
            }
            if (flags & 16 /* Protected */) {
                result.push(createModifier(118 /* ProtectedKeyword */));
            }
            if (flags & 128 /* Abstract */) {
                result.push(createModifier(122 /* AbstractKeyword */));
            }
            if (flags & 32 /* Static */) {
                result.push(createModifier(120 /* StaticKeyword */));
            }
            if (flags & 64 /* Readonly */) {
                result.push(createModifier(138 /* ReadonlyKeyword */));
            }
            if (flags & 256 /* Async */) {
                result.push(createModifier(126 /* AsyncKeyword */));
            }
            return result;
        }
        ts.createModifiersFromModifierFlags = createModifiersFromModifierFlags;
        // Names
        function createQualifiedName(left, right) {
            var node = createSynthesizedNode(153 /* QualifiedName */);
            node.left = left;
            node.right = asName(right);
            return node;
        }
        ts.createQualifiedName = createQualifiedName;
        function updateQualifiedName(node, left, right) {
            return node.left !== left
                || node.right !== right
                ? updateNode(createQualifiedName(left, right), node)
                : node;
        }
        ts.updateQualifiedName = updateQualifiedName;
        function parenthesizeForComputedName(expression) {
            return ts.isCommaSequence(expression)
                ? createParen(expression)
                : expression;
        }
        function createComputedPropertyName(expression) {
            var node = createSynthesizedNode(154 /* ComputedPropertyName */);
            node.expression = parenthesizeForComputedName(expression);
            return node;
        }
        ts.createComputedPropertyName = createComputedPropertyName;
        function updateComputedPropertyName(node, expression) {
            return node.expression !== expression
                ? updateNode(createComputedPropertyName(expression), node)
                : node;
        }
        ts.updateComputedPropertyName = updateComputedPropertyName;
        // Signature elements
        function createTypeParameterDeclaration(name, constraint, defaultType) {
            var node = createSynthesizedNode(155 /* TypeParameter */);
            node.name = asName(name);
            node.constraint = constraint;
            node.default = defaultType;
            return node;
        }
        ts.createTypeParameterDeclaration = createTypeParameterDeclaration;
        function updateTypeParameterDeclaration(node, name, constraint, defaultType) {
            return node.name !== name
                || node.constraint !== constraint
                || node.default !== defaultType
                ? updateNode(createTypeParameterDeclaration(name, constraint, defaultType), node)
                : node;
        }
        ts.updateTypeParameterDeclaration = updateTypeParameterDeclaration;
        function createParameter(decorators, modifiers, dotDotDotToken, name, questionToken, type, initializer) {
            var node = createSynthesizedNode(156 /* Parameter */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.dotDotDotToken = dotDotDotToken;
            node.name = asName(name);
            node.questionToken = questionToken;
            node.type = type;
            node.initializer = initializer ? ts.parenthesizeExpressionForList(initializer) : undefined;
            return node;
        }
        ts.createParameter = createParameter;
        function updateParameter(node, decorators, modifiers, dotDotDotToken, name, questionToken, type, initializer) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.dotDotDotToken !== dotDotDotToken
                || node.name !== name
                || node.questionToken !== questionToken
                || node.type !== type
                || node.initializer !== initializer
                ? updateNode(createParameter(decorators, modifiers, dotDotDotToken, name, questionToken, type, initializer), node)
                : node;
        }
        ts.updateParameter = updateParameter;
        function createDecorator(expression) {
            var node = createSynthesizedNode(157 /* Decorator */);
            node.expression = ts.parenthesizeForAccess(expression);
            return node;
        }
        ts.createDecorator = createDecorator;
        function updateDecorator(node, expression) {
            return node.expression !== expression
                ? updateNode(createDecorator(expression), node)
                : node;
        }
        ts.updateDecorator = updateDecorator;
        // Type Elements
        function createPropertySignature(modifiers, name, questionToken, type, initializer) {
            var node = createSynthesizedNode(158 /* PropertySignature */);
            node.modifiers = asNodeArray(modifiers);
            node.name = asName(name);
            node.questionToken = questionToken;
            node.type = type;
            node.initializer = initializer;
            return node;
        }
        ts.createPropertySignature = createPropertySignature;
        function updatePropertySignature(node, modifiers, name, questionToken, type, initializer) {
            return node.modifiers !== modifiers
                || node.name !== name
                || node.questionToken !== questionToken
                || node.type !== type
                || node.initializer !== initializer
                ? updateNode(createPropertySignature(modifiers, name, questionToken, type, initializer), node)
                : node;
        }
        ts.updatePropertySignature = updatePropertySignature;
        function createProperty(decorators, modifiers, name, questionOrExclamationToken, type, initializer) {
            var node = createSynthesizedNode(159 /* PropertyDeclaration */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.name = asName(name);
            node.questionToken = questionOrExclamationToken !== undefined && questionOrExclamationToken.kind === 57 /* QuestionToken */ ? questionOrExclamationToken : undefined;
            node.exclamationToken = questionOrExclamationToken !== undefined && questionOrExclamationToken.kind === 53 /* ExclamationToken */ ? questionOrExclamationToken : undefined;
            node.type = type;
            node.initializer = initializer;
            return node;
        }
        ts.createProperty = createProperty;
        function updateProperty(node, decorators, modifiers, name, questionOrExclamationToken, type, initializer) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.name !== name
                || node.questionToken !== (questionOrExclamationToken !== undefined && questionOrExclamationToken.kind === 57 /* QuestionToken */ ? questionOrExclamationToken : undefined)
                || node.exclamationToken !== (questionOrExclamationToken !== undefined && questionOrExclamationToken.kind === 53 /* ExclamationToken */ ? questionOrExclamationToken : undefined)
                || node.type !== type
                || node.initializer !== initializer
                ? updateNode(createProperty(decorators, modifiers, name, questionOrExclamationToken, type, initializer), node)
                : node;
        }
        ts.updateProperty = updateProperty;
        function createMethodSignature(typeParameters, parameters, type, name, questionToken) {
            var node = createSignatureDeclaration(160 /* MethodSignature */, typeParameters, parameters, type);
            node.name = asName(name);
            node.questionToken = questionToken;
            return node;
        }
        ts.createMethodSignature = createMethodSignature;
        function updateMethodSignature(node, typeParameters, parameters, type, name, questionToken) {
            return node.typeParameters !== typeParameters
                || node.parameters !== parameters
                || node.type !== type
                || node.name !== name
                || node.questionToken !== questionToken
                ? updateNode(createMethodSignature(typeParameters, parameters, type, name, questionToken), node)
                : node;
        }
        ts.updateMethodSignature = updateMethodSignature;
        function createMethod(decorators, modifiers, asteriskToken, name, questionToken, typeParameters, parameters, type, body) {
            var node = createSynthesizedNode(161 /* MethodDeclaration */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.asteriskToken = asteriskToken;
            node.name = asName(name);
            node.questionToken = questionToken;
            node.typeParameters = asNodeArray(typeParameters);
            node.parameters = createNodeArray(parameters);
            node.type = type;
            node.body = body;
            return node;
        }
        ts.createMethod = createMethod;
        function createMethodCall(object, methodName, argumentsList) {
            return createCall(createPropertyAccess(object, asName(methodName)), 
            /*typeArguments*/ undefined, argumentsList);
        }
        function createGlobalMethodCall(globalObjectName, methodName, argumentsList) {
            return createMethodCall(createIdentifier(globalObjectName), methodName, argumentsList);
        }
        /* @internal */
        function createObjectDefinePropertyCall(target, propertyName, attributes) {
            return createGlobalMethodCall("Object", "defineProperty", [target, asExpression(propertyName), attributes]);
        }
        ts.createObjectDefinePropertyCall = createObjectDefinePropertyCall;
        function tryAddPropertyAssignment(properties, propertyName, expression) {
            if (expression) {
                properties.push(createPropertyAssignment(propertyName, expression));
                return true;
            }
            return false;
        }
        /* @internal */
        function createPropertyDescriptor(attributes, singleLine) {
            var properties = [];
            tryAddPropertyAssignment(properties, "enumerable", asExpression(attributes.enumerable));
            tryAddPropertyAssignment(properties, "configurable", asExpression(attributes.configurable));
            var isData = tryAddPropertyAssignment(properties, "writable", asExpression(attributes.writable));
            isData = tryAddPropertyAssignment(properties, "value", attributes.value) || isData;
            var isAccessor = tryAddPropertyAssignment(properties, "get", attributes.get);
            isAccessor = tryAddPropertyAssignment(properties, "set", attributes.set) || isAccessor;
            ts.Debug.assert(!(isData && isAccessor), "A PropertyDescriptor may not be both an accessor descriptor and a data descriptor.");
            return createObjectLiteral(properties, !singleLine);
        }
        ts.createPropertyDescriptor = createPropertyDescriptor;
        function updateMethod(node, decorators, modifiers, asteriskToken, name, questionToken, typeParameters, parameters, type, body) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.asteriskToken !== asteriskToken
                || node.name !== name
                || node.questionToken !== questionToken
                || node.typeParameters !== typeParameters
                || node.parameters !== parameters
                || node.type !== type
                || node.body !== body
                ? updateNode(createMethod(decorators, modifiers, asteriskToken, name, questionToken, typeParameters, parameters, type, body), node)
                : node;
        }
        ts.updateMethod = updateMethod;
        function createConstructor(decorators, modifiers, parameters, body) {
            var node = createSynthesizedNode(162 /* Constructor */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.typeParameters = undefined;
            node.parameters = createNodeArray(parameters);
            node.type = undefined;
            node.body = body;
            return node;
        }
        ts.createConstructor = createConstructor;
        function updateConstructor(node, decorators, modifiers, parameters, body) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.parameters !== parameters
                || node.body !== body
                ? updateNode(createConstructor(decorators, modifiers, parameters, body), node)
                : node;
        }
        ts.updateConstructor = updateConstructor;
        function createGetAccessor(decorators, modifiers, name, parameters, type, body) {
            var node = createSynthesizedNode(163 /* GetAccessor */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.name = asName(name);
            node.typeParameters = undefined;
            node.parameters = createNodeArray(parameters);
            node.type = type;
            node.body = body;
            return node;
        }
        ts.createGetAccessor = createGetAccessor;
        function updateGetAccessor(node, decorators, modifiers, name, parameters, type, body) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.name !== name
                || node.parameters !== parameters
                || node.type !== type
                || node.body !== body
                ? updateNode(createGetAccessor(decorators, modifiers, name, parameters, type, body), node)
                : node;
        }
        ts.updateGetAccessor = updateGetAccessor;
        function createSetAccessor(decorators, modifiers, name, parameters, body) {
            var node = createSynthesizedNode(164 /* SetAccessor */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.name = asName(name);
            node.typeParameters = undefined;
            node.parameters = createNodeArray(parameters);
            node.body = body;
            return node;
        }
        ts.createSetAccessor = createSetAccessor;
        function updateSetAccessor(node, decorators, modifiers, name, parameters, body) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.name !== name
                || node.parameters !== parameters
                || node.body !== body
                ? updateNode(createSetAccessor(decorators, modifiers, name, parameters, body), node)
                : node;
        }
        ts.updateSetAccessor = updateSetAccessor;
        function createCallSignature(typeParameters, parameters, type) {
            return createSignatureDeclaration(165 /* CallSignature */, typeParameters, parameters, type);
        }
        ts.createCallSignature = createCallSignature;
        function updateCallSignature(node, typeParameters, parameters, type) {
            return updateSignatureDeclaration(node, typeParameters, parameters, type);
        }
        ts.updateCallSignature = updateCallSignature;
        function createConstructSignature(typeParameters, parameters, type) {
            return createSignatureDeclaration(166 /* ConstructSignature */, typeParameters, parameters, type);
        }
        ts.createConstructSignature = createConstructSignature;
        function updateConstructSignature(node, typeParameters, parameters, type) {
            return updateSignatureDeclaration(node, typeParameters, parameters, type);
        }
        ts.updateConstructSignature = updateConstructSignature;
        function createIndexSignature(decorators, modifiers, parameters, type) {
            var node = createSynthesizedNode(167 /* IndexSignature */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.parameters = createNodeArray(parameters);
            node.type = type;
            return node;
        }
        ts.createIndexSignature = createIndexSignature;
        function updateIndexSignature(node, decorators, modifiers, parameters, type) {
            return node.parameters !== parameters
                || node.type !== type
                || node.decorators !== decorators
                || node.modifiers !== modifiers
                ? updateNode(createIndexSignature(decorators, modifiers, parameters, type), node)
                : node;
        }
        ts.updateIndexSignature = updateIndexSignature;
        /* @internal */
        function createSignatureDeclaration(kind, typeParameters, parameters, type, typeArguments) {
            var node = createSynthesizedNode(kind);
            node.typeParameters = asNodeArray(typeParameters);
            node.parameters = asNodeArray(parameters);
            node.type = type;
            node.typeArguments = asNodeArray(typeArguments);
            return node;
        }
        ts.createSignatureDeclaration = createSignatureDeclaration;
        function updateSignatureDeclaration(node, typeParameters, parameters, type) {
            return node.typeParameters !== typeParameters
                || node.parameters !== parameters
                || node.type !== type
                ? updateNode(createSignatureDeclaration(node.kind, typeParameters, parameters, type), node)
                : node;
        }
        // Types
        function createKeywordTypeNode(kind) {
            return createSynthesizedNode(kind);
        }
        ts.createKeywordTypeNode = createKeywordTypeNode;
        function createTypePredicateNode(parameterName, type) {
            return createTypePredicateNodeWithModifier(/*assertsModifier*/ undefined, parameterName, type);
        }
        ts.createTypePredicateNode = createTypePredicateNode;
        function createTypePredicateNodeWithModifier(assertsModifier, parameterName, type) {
            var node = createSynthesizedNode(168 /* TypePredicate */);
            node.assertsModifier = assertsModifier;
            node.parameterName = asName(parameterName);
            node.type = type;
            return node;
        }
        ts.createTypePredicateNodeWithModifier = createTypePredicateNodeWithModifier;
        function updateTypePredicateNode(node, parameterName, type) {
            return updateTypePredicateNodeWithModifier(node, node.assertsModifier, parameterName, type);
        }
        ts.updateTypePredicateNode = updateTypePredicateNode;
        function updateTypePredicateNodeWithModifier(node, assertsModifier, parameterName, type) {
            return node.assertsModifier !== assertsModifier
                || node.parameterName !== parameterName
                || node.type !== type
                ? updateNode(createTypePredicateNodeWithModifier(assertsModifier, parameterName, type), node)
                : node;
        }
        ts.updateTypePredicateNodeWithModifier = updateTypePredicateNodeWithModifier;
        function createTypeReferenceNode(typeName, typeArguments) {
            var node = createSynthesizedNode(169 /* TypeReference */);
            node.typeName = asName(typeName);
            node.typeArguments = typeArguments && ts.parenthesizeTypeParameters(typeArguments);
            return node;
        }
        ts.createTypeReferenceNode = createTypeReferenceNode;
        function updateTypeReferenceNode(node, typeName, typeArguments) {
            return node.typeName !== typeName
                || node.typeArguments !== typeArguments
                ? updateNode(createTypeReferenceNode(typeName, typeArguments), node)
                : node;
        }
        ts.updateTypeReferenceNode = updateTypeReferenceNode;
        function createFunctionTypeNode(typeParameters, parameters, type) {
            return createSignatureDeclaration(170 /* FunctionType */, typeParameters, parameters, type);
        }
        ts.createFunctionTypeNode = createFunctionTypeNode;
        function updateFunctionTypeNode(node, typeParameters, parameters, type) {
            return updateSignatureDeclaration(node, typeParameters, parameters, type);
        }
        ts.updateFunctionTypeNode = updateFunctionTypeNode;
        function createConstructorTypeNode(typeParameters, parameters, type) {
            return createSignatureDeclaration(171 /* ConstructorType */, typeParameters, parameters, type);
        }
        ts.createConstructorTypeNode = createConstructorTypeNode;
        function updateConstructorTypeNode(node, typeParameters, parameters, type) {
            return updateSignatureDeclaration(node, typeParameters, parameters, type);
        }
        ts.updateConstructorTypeNode = updateConstructorTypeNode;
        function createTypeQueryNode(exprName) {
            var node = createSynthesizedNode(172 /* TypeQuery */);
            node.exprName = exprName;
            return node;
        }
        ts.createTypeQueryNode = createTypeQueryNode;
        function updateTypeQueryNode(node, exprName) {
            return node.exprName !== exprName
                ? updateNode(createTypeQueryNode(exprName), node)
                : node;
        }
        ts.updateTypeQueryNode = updateTypeQueryNode;
        function createTypeLiteralNode(members) {
            var node = createSynthesizedNode(173 /* TypeLiteral */);
            node.members = createNodeArray(members);
            return node;
        }
        ts.createTypeLiteralNode = createTypeLiteralNode;
        function updateTypeLiteralNode(node, members) {
            return node.members !== members
                ? updateNode(createTypeLiteralNode(members), node)
                : node;
        }
        ts.updateTypeLiteralNode = updateTypeLiteralNode;
        function createArrayTypeNode(elementType) {
            var node = createSynthesizedNode(174 /* ArrayType */);
            node.elementType = ts.parenthesizeArrayTypeMember(elementType);
            return node;
        }
        ts.createArrayTypeNode = createArrayTypeNode;
        function updateArrayTypeNode(node, elementType) {
            return node.elementType !== elementType
                ? updateNode(createArrayTypeNode(elementType), node)
                : node;
        }
        ts.updateArrayTypeNode = updateArrayTypeNode;
        function createTupleTypeNode(elementTypes) {
            var node = createSynthesizedNode(175 /* TupleType */);
            node.elementTypes = createNodeArray(elementTypes);
            return node;
        }
        ts.createTupleTypeNode = createTupleTypeNode;
        function updateTupleTypeNode(node, elementTypes) {
            return node.elementTypes !== elementTypes
                ? updateNode(createTupleTypeNode(elementTypes), node)
                : node;
        }
        ts.updateTupleTypeNode = updateTupleTypeNode;
        function createOptionalTypeNode(type) {
            var node = createSynthesizedNode(176 /* OptionalType */);
            node.type = ts.parenthesizeArrayTypeMember(type);
            return node;
        }
        ts.createOptionalTypeNode = createOptionalTypeNode;
        function updateOptionalTypeNode(node, type) {
            return node.type !== type
                ? updateNode(createOptionalTypeNode(type), node)
                : node;
        }
        ts.updateOptionalTypeNode = updateOptionalTypeNode;
        function createRestTypeNode(type) {
            var node = createSynthesizedNode(177 /* RestType */);
            node.type = type;
            return node;
        }
        ts.createRestTypeNode = createRestTypeNode;
        function updateRestTypeNode(node, type) {
            return node.type !== type
                ? updateNode(createRestTypeNode(type), node)
                : node;
        }
        ts.updateRestTypeNode = updateRestTypeNode;
        function createUnionTypeNode(types) {
            return createUnionOrIntersectionTypeNode(178 /* UnionType */, types);
        }
        ts.createUnionTypeNode = createUnionTypeNode;
        function updateUnionTypeNode(node, types) {
            return updateUnionOrIntersectionTypeNode(node, types);
        }
        ts.updateUnionTypeNode = updateUnionTypeNode;
        function createIntersectionTypeNode(types) {
            return createUnionOrIntersectionTypeNode(179 /* IntersectionType */, types);
        }
        ts.createIntersectionTypeNode = createIntersectionTypeNode;
        function updateIntersectionTypeNode(node, types) {
            return updateUnionOrIntersectionTypeNode(node, types);
        }
        ts.updateIntersectionTypeNode = updateIntersectionTypeNode;
        function createUnionOrIntersectionTypeNode(kind, types) {
            var node = createSynthesizedNode(kind);
            node.types = ts.parenthesizeElementTypeMembers(types);
            return node;
        }
        ts.createUnionOrIntersectionTypeNode = createUnionOrIntersectionTypeNode;
        function updateUnionOrIntersectionTypeNode(node, types) {
            return node.types !== types
                ? updateNode(createUnionOrIntersectionTypeNode(node.kind, types), node)
                : node;
        }
        function createConditionalTypeNode(checkType, extendsType, trueType, falseType) {
            var node = createSynthesizedNode(180 /* ConditionalType */);
            node.checkType = ts.parenthesizeConditionalTypeMember(checkType);
            node.extendsType = ts.parenthesizeConditionalTypeMember(extendsType);
            node.trueType = trueType;
            node.falseType = falseType;
            return node;
        }
        ts.createConditionalTypeNode = createConditionalTypeNode;
        function updateConditionalTypeNode(node, checkType, extendsType, trueType, falseType) {
            return node.checkType !== checkType
                || node.extendsType !== extendsType
                || node.trueType !== trueType
                || node.falseType !== falseType
                ? updateNode(createConditionalTypeNode(checkType, extendsType, trueType, falseType), node)
                : node;
        }
        ts.updateConditionalTypeNode = updateConditionalTypeNode;
        function createInferTypeNode(typeParameter) {
            var node = createSynthesizedNode(181 /* InferType */);
            node.typeParameter = typeParameter;
            return node;
        }
        ts.createInferTypeNode = createInferTypeNode;
        function updateInferTypeNode(node, typeParameter) {
            return node.typeParameter !== typeParameter
                ? updateNode(createInferTypeNode(typeParameter), node)
                : node;
        }
        ts.updateInferTypeNode = updateInferTypeNode;
        function createImportTypeNode(argument, qualifier, typeArguments, isTypeOf) {
            var node = createSynthesizedNode(188 /* ImportType */);
            node.argument = argument;
            node.qualifier = qualifier;
            node.typeArguments = ts.parenthesizeTypeParameters(typeArguments);
            node.isTypeOf = isTypeOf;
            return node;
        }
        ts.createImportTypeNode = createImportTypeNode;
        function updateImportTypeNode(node, argument, qualifier, typeArguments, isTypeOf) {
            return node.argument !== argument
                || node.qualifier !== qualifier
                || node.typeArguments !== typeArguments
                || node.isTypeOf !== isTypeOf
                ? updateNode(createImportTypeNode(argument, qualifier, typeArguments, isTypeOf), node)
                : node;
        }
        ts.updateImportTypeNode = updateImportTypeNode;
        function createParenthesizedType(type) {
            var node = createSynthesizedNode(182 /* ParenthesizedType */);
            node.type = type;
            return node;
        }
        ts.createParenthesizedType = createParenthesizedType;
        function updateParenthesizedType(node, type) {
            return node.type !== type
                ? updateNode(createParenthesizedType(type), node)
                : node;
        }
        ts.updateParenthesizedType = updateParenthesizedType;
        function createThisTypeNode() {
            return createSynthesizedNode(183 /* ThisType */);
        }
        ts.createThisTypeNode = createThisTypeNode;
        function createTypeOperatorNode(operatorOrType, type) {
            var node = createSynthesizedNode(184 /* TypeOperator */);
            node.operator = typeof operatorOrType === "number" ? operatorOrType : 134 /* KeyOfKeyword */;
            node.type = ts.parenthesizeElementTypeMember(typeof operatorOrType === "number" ? type : operatorOrType);
            return node;
        }
        ts.createTypeOperatorNode = createTypeOperatorNode;
        function updateTypeOperatorNode(node, type) {
            return node.type !== type ? updateNode(createTypeOperatorNode(node.operator, type), node) : node;
        }
        ts.updateTypeOperatorNode = updateTypeOperatorNode;
        function createIndexedAccessTypeNode(objectType, indexType) {
            var node = createSynthesizedNode(185 /* IndexedAccessType */);
            node.objectType = ts.parenthesizeElementTypeMember(objectType);
            node.indexType = indexType;
            return node;
        }
        ts.createIndexedAccessTypeNode = createIndexedAccessTypeNode;
        function updateIndexedAccessTypeNode(node, objectType, indexType) {
            return node.objectType !== objectType
                || node.indexType !== indexType
                ? updateNode(createIndexedAccessTypeNode(objectType, indexType), node)
                : node;
        }
        ts.updateIndexedAccessTypeNode = updateIndexedAccessTypeNode;
        function createMappedTypeNode(readonlyToken, typeParameter, questionToken, type) {
            var node = createSynthesizedNode(186 /* MappedType */);
            node.readonlyToken = readonlyToken;
            node.typeParameter = typeParameter;
            node.questionToken = questionToken;
            node.type = type;
            return node;
        }
        ts.createMappedTypeNode = createMappedTypeNode;
        function updateMappedTypeNode(node, readonlyToken, typeParameter, questionToken, type) {
            return node.readonlyToken !== readonlyToken
                || node.typeParameter !== typeParameter
                || node.questionToken !== questionToken
                || node.type !== type
                ? updateNode(createMappedTypeNode(readonlyToken, typeParameter, questionToken, type), node)
                : node;
        }
        ts.updateMappedTypeNode = updateMappedTypeNode;
        function createLiteralTypeNode(literal) {
            var node = createSynthesizedNode(187 /* LiteralType */);
            node.literal = literal;
            return node;
        }
        ts.createLiteralTypeNode = createLiteralTypeNode;
        function updateLiteralTypeNode(node, literal) {
            return node.literal !== literal
                ? updateNode(createLiteralTypeNode(literal), node)
                : node;
        }
        ts.updateLiteralTypeNode = updateLiteralTypeNode;
        // Binding Patterns
        function createObjectBindingPattern(elements) {
            var node = createSynthesizedNode(189 /* ObjectBindingPattern */);
            node.elements = createNodeArray(elements);
            return node;
        }
        ts.createObjectBindingPattern = createObjectBindingPattern;
        function updateObjectBindingPattern(node, elements) {
            return node.elements !== elements
                ? updateNode(createObjectBindingPattern(elements), node)
                : node;
        }
        ts.updateObjectBindingPattern = updateObjectBindingPattern;
        function createArrayBindingPattern(elements) {
            var node = createSynthesizedNode(190 /* ArrayBindingPattern */);
            node.elements = createNodeArray(elements);
            return node;
        }
        ts.createArrayBindingPattern = createArrayBindingPattern;
        function updateArrayBindingPattern(node, elements) {
            return node.elements !== elements
                ? updateNode(createArrayBindingPattern(elements), node)
                : node;
        }
        ts.updateArrayBindingPattern = updateArrayBindingPattern;
        function createBindingElement(dotDotDotToken, propertyName, name, initializer) {
            var node = createSynthesizedNode(191 /* BindingElement */);
            node.dotDotDotToken = dotDotDotToken;
            node.propertyName = asName(propertyName);
            node.name = asName(name);
            node.initializer = initializer;
            return node;
        }
        ts.createBindingElement = createBindingElement;
        function updateBindingElement(node, dotDotDotToken, propertyName, name, initializer) {
            return node.propertyName !== propertyName
                || node.dotDotDotToken !== dotDotDotToken
                || node.name !== name
                || node.initializer !== initializer
                ? updateNode(createBindingElement(dotDotDotToken, propertyName, name, initializer), node)
                : node;
        }
        ts.updateBindingElement = updateBindingElement;
        // Expression
        function createArrayLiteral(elements, multiLine) {
            var node = createSynthesizedNode(192 /* ArrayLiteralExpression */);
            node.elements = ts.parenthesizeListElements(createNodeArray(elements));
            if (multiLine)
                node.multiLine = true;
            return node;
        }
        ts.createArrayLiteral = createArrayLiteral;
        function updateArrayLiteral(node, elements) {
            return node.elements !== elements
                ? updateNode(createArrayLiteral(elements, node.multiLine), node)
                : node;
        }
        ts.updateArrayLiteral = updateArrayLiteral;
        function createObjectLiteral(properties, multiLine) {
            var node = createSynthesizedNode(193 /* ObjectLiteralExpression */);
            node.properties = createNodeArray(properties);
            if (multiLine)
                node.multiLine = true;
            return node;
        }
        ts.createObjectLiteral = createObjectLiteral;
        function updateObjectLiteral(node, properties) {
            return node.properties !== properties
                ? updateNode(createObjectLiteral(properties, node.multiLine), node)
                : node;
        }
        ts.updateObjectLiteral = updateObjectLiteral;
        function createPropertyAccess(expression, name) {
            var node = createSynthesizedNode(194 /* PropertyAccessExpression */);
            node.expression = ts.parenthesizeForAccess(expression);
            node.name = asName(name);
            setEmitFlags(node, 131072 /* NoIndentation */);
            return node;
        }
        ts.createPropertyAccess = createPropertyAccess;
        function updatePropertyAccess(node, expression, name) {
            if (ts.isOptionalChain(node) && ts.isIdentifier(node.name) && ts.isIdentifier(name)) {
                // Not sure why this cast was necessary: the previous line should already establish that node.name is an identifier
                var theNode = node;
                return updatePropertyAccessChain(theNode, expression, node.questionDotToken, name);
            }
            // Because we are updating existed propertyAccess we want to inherit its emitFlags
            // instead of using the default from createPropertyAccess
            return node.expression !== expression
                || node.name !== name
                ? updateNode(setEmitFlags(createPropertyAccess(expression, name), ts.getEmitFlags(node)), node)
                : node;
        }
        ts.updatePropertyAccess = updatePropertyAccess;
        function createPropertyAccessChain(expression, questionDotToken, name) {
            var node = createSynthesizedNode(194 /* PropertyAccessExpression */);
            node.flags |= 32 /* OptionalChain */;
            node.expression = ts.parenthesizeForAccess(expression);
            node.questionDotToken = questionDotToken;
            node.name = asName(name);
            setEmitFlags(node, 131072 /* NoIndentation */);
            return node;
        }
        ts.createPropertyAccessChain = createPropertyAccessChain;
        function updatePropertyAccessChain(node, expression, questionDotToken, name) {
            ts.Debug.assert(!!(node.flags & 32 /* OptionalChain */), "Cannot update a PropertyAccessExpression using updatePropertyAccessChain. Use updatePropertyAccess instead.");
            // Because we are updating an existing PropertyAccessChain we want to inherit its emitFlags
            // instead of using the default from createPropertyAccess
            return node.expression !== expression
                || node.questionDotToken !== questionDotToken
                || node.name !== name
                ? updateNode(setEmitFlags(createPropertyAccessChain(expression, questionDotToken, name), ts.getEmitFlags(node)), node)
                : node;
        }
        ts.updatePropertyAccessChain = updatePropertyAccessChain;
        function createElementAccess(expression, index) {
            var node = createSynthesizedNode(195 /* ElementAccessExpression */);
            node.expression = ts.parenthesizeForAccess(expression);
            node.argumentExpression = asExpression(index);
            return node;
        }
        ts.createElementAccess = createElementAccess;
        function updateElementAccess(node, expression, argumentExpression) {
            if (ts.isOptionalChain(node)) {
                return updateElementAccessChain(node, expression, node.questionDotToken, argumentExpression);
            }
            return node.expression !== expression
                || node.argumentExpression !== argumentExpression
                ? updateNode(createElementAccess(expression, argumentExpression), node)
                : node;
        }
        ts.updateElementAccess = updateElementAccess;
        function createElementAccessChain(expression, questionDotToken, index) {
            var node = createSynthesizedNode(195 /* ElementAccessExpression */);
            node.flags |= 32 /* OptionalChain */;
            node.expression = ts.parenthesizeForAccess(expression);
            node.questionDotToken = questionDotToken;
            node.argumentExpression = asExpression(index);
            return node;
        }
        ts.createElementAccessChain = createElementAccessChain;
        function updateElementAccessChain(node, expression, questionDotToken, argumentExpression) {
            ts.Debug.assert(!!(node.flags & 32 /* OptionalChain */), "Cannot update an ElementAccessExpression using updateElementAccessChain. Use updateElementAccess instead.");
            return node.expression !== expression
                || node.questionDotToken !== questionDotToken
                || node.argumentExpression !== argumentExpression
                ? updateNode(createElementAccessChain(expression, questionDotToken, argumentExpression), node)
                : node;
        }
        ts.updateElementAccessChain = updateElementAccessChain;
        function createCall(expression, typeArguments, argumentsArray) {
            var node = createSynthesizedNode(196 /* CallExpression */);
            node.expression = ts.parenthesizeForAccess(expression);
            node.typeArguments = asNodeArray(typeArguments);
            node.arguments = ts.parenthesizeListElements(createNodeArray(argumentsArray));
            return node;
        }
        ts.createCall = createCall;
        function updateCall(node, expression, typeArguments, argumentsArray) {
            if (ts.isOptionalChain(node)) {
                return updateCallChain(node, expression, node.questionDotToken, typeArguments, argumentsArray);
            }
            return node.expression !== expression
                || node.typeArguments !== typeArguments
                || node.arguments !== argumentsArray
                ? updateNode(createCall(expression, typeArguments, argumentsArray), node)
                : node;
        }
        ts.updateCall = updateCall;
        function createCallChain(expression, questionDotToken, typeArguments, argumentsArray) {
            var node = createSynthesizedNode(196 /* CallExpression */);
            node.flags |= 32 /* OptionalChain */;
            node.expression = ts.parenthesizeForAccess(expression);
            node.questionDotToken = questionDotToken;
            node.typeArguments = asNodeArray(typeArguments);
            node.arguments = ts.parenthesizeListElements(createNodeArray(argumentsArray));
            return node;
        }
        ts.createCallChain = createCallChain;
        function updateCallChain(node, expression, questionDotToken, typeArguments, argumentsArray) {
            ts.Debug.assert(!!(node.flags & 32 /* OptionalChain */), "Cannot update a CallExpression using updateCallChain. Use updateCall instead.");
            return node.expression !== expression
                || node.questionDotToken !== questionDotToken
                || node.typeArguments !== typeArguments
                || node.arguments !== argumentsArray
                ? updateNode(createCallChain(expression, questionDotToken, typeArguments, argumentsArray), node)
                : node;
        }
        ts.updateCallChain = updateCallChain;
        function createNew(expression, typeArguments, argumentsArray) {
            var node = createSynthesizedNode(197 /* NewExpression */);
            node.expression = ts.parenthesizeForNew(expression);
            node.typeArguments = asNodeArray(typeArguments);
            node.arguments = argumentsArray ? ts.parenthesizeListElements(createNodeArray(argumentsArray)) : undefined;
            return node;
        }
        ts.createNew = createNew;
        function updateNew(node, expression, typeArguments, argumentsArray) {
            return node.expression !== expression
                || node.typeArguments !== typeArguments
                || node.arguments !== argumentsArray
                ? updateNode(createNew(expression, typeArguments, argumentsArray), node)
                : node;
        }
        ts.updateNew = updateNew;
        function createTaggedTemplate(tag, typeArgumentsOrTemplate, template) {
            var node = createSynthesizedNode(198 /* TaggedTemplateExpression */);
            node.tag = ts.parenthesizeForAccess(tag);
            if (template) {
                node.typeArguments = asNodeArray(typeArgumentsOrTemplate);
                node.template = template;
            }
            else {
                node.typeArguments = undefined;
                node.template = typeArgumentsOrTemplate;
            }
            return node;
        }
        ts.createTaggedTemplate = createTaggedTemplate;
        function updateTaggedTemplate(node, tag, typeArgumentsOrTemplate, template) {
            return node.tag !== tag
                || (template
                    ? node.typeArguments !== typeArgumentsOrTemplate || node.template !== template
                    : node.typeArguments !== undefined || node.template !== typeArgumentsOrTemplate)
                ? updateNode(createTaggedTemplate(tag, typeArgumentsOrTemplate, template), node)
                : node;
        }
        ts.updateTaggedTemplate = updateTaggedTemplate;
        function createTypeAssertion(type, expression) {
            var node = createSynthesizedNode(199 /* TypeAssertionExpression */);
            node.type = type;
            node.expression = ts.parenthesizePrefixOperand(expression);
            return node;
        }
        ts.createTypeAssertion = createTypeAssertion;
        function updateTypeAssertion(node, type, expression) {
            return node.type !== type
                || node.expression !== expression
                ? updateNode(createTypeAssertion(type, expression), node)
                : node;
        }
        ts.updateTypeAssertion = updateTypeAssertion;
        function createParen(expression) {
            var node = createSynthesizedNode(200 /* ParenthesizedExpression */);
            node.expression = expression;
            return node;
        }
        ts.createParen = createParen;
        function updateParen(node, expression) {
            return node.expression !== expression
                ? updateNode(createParen(expression), node)
                : node;
        }
        ts.updateParen = updateParen;
        function createFunctionExpression(modifiers, asteriskToken, name, typeParameters, parameters, type, body) {
            var node = createSynthesizedNode(201 /* FunctionExpression */);
            node.modifiers = asNodeArray(modifiers);
            node.asteriskToken = asteriskToken;
            node.name = asName(name);
            node.typeParameters = asNodeArray(typeParameters);
            node.parameters = createNodeArray(parameters);
            node.type = type;
            node.body = body;
            return node;
        }
        ts.createFunctionExpression = createFunctionExpression;
        function updateFunctionExpression(node, modifiers, asteriskToken, name, typeParameters, parameters, type, body) {
            return node.name !== name
                || node.modifiers !== modifiers
                || node.asteriskToken !== asteriskToken
                || node.typeParameters !== typeParameters
                || node.parameters !== parameters
                || node.type !== type
                || node.body !== body
                ? updateNode(createFunctionExpression(modifiers, asteriskToken, name, typeParameters, parameters, type, body), node)
                : node;
        }
        ts.updateFunctionExpression = updateFunctionExpression;
        function createArrowFunction(modifiers, typeParameters, parameters, type, equalsGreaterThanToken, body) {
            var node = createSynthesizedNode(202 /* ArrowFunction */);
            node.modifiers = asNodeArray(modifiers);
            node.typeParameters = asNodeArray(typeParameters);
            node.parameters = createNodeArray(parameters);
            node.type = type;
            node.equalsGreaterThanToken = equalsGreaterThanToken || createToken(38 /* EqualsGreaterThanToken */);
            node.body = ts.parenthesizeConciseBody(body);
            return node;
        }
        ts.createArrowFunction = createArrowFunction;
        function updateArrowFunction(node, modifiers, typeParameters, parameters, type, equalsGreaterThanToken, body) {
            return node.modifiers !== modifiers
                || node.typeParameters !== typeParameters
                || node.parameters !== parameters
                || node.type !== type
                || node.equalsGreaterThanToken !== equalsGreaterThanToken
                || node.body !== body
                ? updateNode(createArrowFunction(modifiers, typeParameters, parameters, type, equalsGreaterThanToken, body), node)
                : node;
        }
        ts.updateArrowFunction = updateArrowFunction;
        function createDelete(expression) {
            var node = createSynthesizedNode(203 /* DeleteExpression */);
            node.expression = ts.parenthesizePrefixOperand(expression);
            return node;
        }
        ts.createDelete = createDelete;
        function updateDelete(node, expression) {
            return node.expression !== expression
                ? updateNode(createDelete(expression), node)
                : node;
        }
        ts.updateDelete = updateDelete;
        function createTypeOf(expression) {
            var node = createSynthesizedNode(204 /* TypeOfExpression */);
            node.expression = ts.parenthesizePrefixOperand(expression);
            return node;
        }
        ts.createTypeOf = createTypeOf;
        function updateTypeOf(node, expression) {
            return node.expression !== expression
                ? updateNode(createTypeOf(expression), node)
                : node;
        }
        ts.updateTypeOf = updateTypeOf;
        function createVoid(expression) {
            var node = createSynthesizedNode(205 /* VoidExpression */);
            node.expression = ts.parenthesizePrefixOperand(expression);
            return node;
        }
        ts.createVoid = createVoid;
        function updateVoid(node, expression) {
            return node.expression !== expression
                ? updateNode(createVoid(expression), node)
                : node;
        }
        ts.updateVoid = updateVoid;
        function createAwait(expression) {
            var node = createSynthesizedNode(206 /* AwaitExpression */);
            node.expression = ts.parenthesizePrefixOperand(expression);
            return node;
        }
        ts.createAwait = createAwait;
        function updateAwait(node, expression) {
            return node.expression !== expression
                ? updateNode(createAwait(expression), node)
                : node;
        }
        ts.updateAwait = updateAwait;
        function createPrefix(operator, operand) {
            var node = createSynthesizedNode(207 /* PrefixUnaryExpression */);
            node.operator = operator;
            node.operand = ts.parenthesizePrefixOperand(operand);
            return node;
        }
        ts.createPrefix = createPrefix;
        function updatePrefix(node, operand) {
            return node.operand !== operand
                ? updateNode(createPrefix(node.operator, operand), node)
                : node;
        }
        ts.updatePrefix = updatePrefix;
        function createPostfix(operand, operator) {
            var node = createSynthesizedNode(208 /* PostfixUnaryExpression */);
            node.operand = ts.parenthesizePostfixOperand(operand);
            node.operator = operator;
            return node;
        }
        ts.createPostfix = createPostfix;
        function updatePostfix(node, operand) {
            return node.operand !== operand
                ? updateNode(createPostfix(operand, node.operator), node)
                : node;
        }
        ts.updatePostfix = updatePostfix;
        function createBinary(left, operator, right) {
            var node = createSynthesizedNode(209 /* BinaryExpression */);
            var operatorToken = asToken(operator);
            var operatorKind = operatorToken.kind;
            node.left = ts.parenthesizeBinaryOperand(operatorKind, left, /*isLeftSideOfBinary*/ true, /*leftOperand*/ undefined);
            node.operatorToken = operatorToken;
            node.right = ts.parenthesizeBinaryOperand(operatorKind, right, /*isLeftSideOfBinary*/ false, node.left);
            return node;
        }
        ts.createBinary = createBinary;
        function updateBinary(node, left, right, operator) {
            return node.left !== left
                || node.right !== right
                ? updateNode(createBinary(left, operator || node.operatorToken, right), node)
                : node;
        }
        ts.updateBinary = updateBinary;
        function createConditional(condition, questionTokenOrWhenTrue, whenTrueOrWhenFalse, colonToken, whenFalse) {
            var node = createSynthesizedNode(210 /* ConditionalExpression */);
            node.condition = ts.parenthesizeForConditionalHead(condition);
            node.questionToken = whenFalse ? questionTokenOrWhenTrue : createToken(57 /* QuestionToken */);
            node.whenTrue = ts.parenthesizeSubexpressionOfConditionalExpression(whenFalse ? whenTrueOrWhenFalse : questionTokenOrWhenTrue);
            node.colonToken = whenFalse ? colonToken : createToken(58 /* ColonToken */);
            node.whenFalse = ts.parenthesizeSubexpressionOfConditionalExpression(whenFalse ? whenFalse : whenTrueOrWhenFalse);
            return node;
        }
        ts.createConditional = createConditional;
        function updateConditional(node, condition, questionToken, whenTrue, colonToken, whenFalse) {
            return node.condition !== condition
                || node.questionToken !== questionToken
                || node.whenTrue !== whenTrue
                || node.colonToken !== colonToken
                || node.whenFalse !== whenFalse
                ? updateNode(createConditional(condition, questionToken, whenTrue, colonToken, whenFalse), node)
                : node;
        }
        ts.updateConditional = updateConditional;
        function createTemplateExpression(head, templateSpans) {
            var node = createSynthesizedNode(211 /* TemplateExpression */);
            node.head = head;
            node.templateSpans = createNodeArray(templateSpans);
            return node;
        }
        ts.createTemplateExpression = createTemplateExpression;
        function updateTemplateExpression(node, head, templateSpans) {
            return node.head !== head
                || node.templateSpans !== templateSpans
                ? updateNode(createTemplateExpression(head, templateSpans), node)
                : node;
        }
        ts.updateTemplateExpression = updateTemplateExpression;
        var rawTextScanner;
        var invalidValueSentinel = {};
        function getCookedText(kind, rawText) {
            if (!rawTextScanner) {
                rawTextScanner = ts.createScanner(99 /* Latest */, /*skipTrivia*/ false, 0 /* Standard */);
            }
            switch (kind) {
                case 14 /* NoSubstitutionTemplateLiteral */:
                    rawTextScanner.setText("`" + rawText + "`");
                    break;
                case 15 /* TemplateHead */:
                    rawTextScanner.setText("`" + rawText + "${");
                    break;
                case 16 /* TemplateMiddle */:
                    rawTextScanner.setText("}" + rawText + "${");
                    break;
                case 17 /* TemplateTail */:
                    rawTextScanner.setText("}" + rawText + "`");
                    break;
            }
            var token = rawTextScanner.scan();
            if (token === 23 /* CloseBracketToken */) {
                token = rawTextScanner.reScanTemplateToken();
            }
            if (rawTextScanner.isUnterminated()) {
                rawTextScanner.setText(undefined);
                return invalidValueSentinel;
            }
            var tokenValue;
            switch (token) {
                case 14 /* NoSubstitutionTemplateLiteral */:
                case 15 /* TemplateHead */:
                case 16 /* TemplateMiddle */:
                case 17 /* TemplateTail */:
                    tokenValue = rawTextScanner.getTokenValue();
                    break;
            }
            if (rawTextScanner.scan() !== 1 /* EndOfFileToken */) {
                rawTextScanner.setText(undefined);
                return invalidValueSentinel;
            }
            rawTextScanner.setText(undefined);
            return tokenValue;
        }
        function createTemplateLiteralLikeNode(kind, text, rawText) {
            var node = createSynthesizedNode(kind);
            node.text = text;
            if (rawText === undefined || text === rawText) {
                node.rawText = rawText;
            }
            else {
                var cooked = getCookedText(kind, rawText);
                if (typeof cooked === "object") {
                    return ts.Debug.fail("Invalid raw text");
                }
                ts.Debug.assert(text === cooked, "Expected argument 'text' to be the normalized (i.e. 'cooked') version of argument 'rawText'.");
                node.rawText = rawText;
            }
            return node;
        }
        function createTemplateHead(text, rawText) {
            var node = createTemplateLiteralLikeNode(15 /* TemplateHead */, text, rawText);
            node.text = text;
            return node;
        }
        ts.createTemplateHead = createTemplateHead;
        function createTemplateMiddle(text, rawText) {
            var node = createTemplateLiteralLikeNode(16 /* TemplateMiddle */, text, rawText);
            node.text = text;
            return node;
        }
        ts.createTemplateMiddle = createTemplateMiddle;
        function createTemplateTail(text, rawText) {
            var node = createTemplateLiteralLikeNode(17 /* TemplateTail */, text, rawText);
            node.text = text;
            return node;
        }
        ts.createTemplateTail = createTemplateTail;
        function createNoSubstitutionTemplateLiteral(text, rawText) {
            var node = createTemplateLiteralLikeNode(14 /* NoSubstitutionTemplateLiteral */, text, rawText);
            return node;
        }
        ts.createNoSubstitutionTemplateLiteral = createNoSubstitutionTemplateLiteral;
        function createYield(asteriskTokenOrExpression, expression) {
            var node = createSynthesizedNode(212 /* YieldExpression */);
            node.asteriskToken = asteriskTokenOrExpression && asteriskTokenOrExpression.kind === 41 /* AsteriskToken */ ? asteriskTokenOrExpression : undefined;
            node.expression = asteriskTokenOrExpression && asteriskTokenOrExpression.kind !== 41 /* AsteriskToken */ ? asteriskTokenOrExpression : expression;
            return node;
        }
        ts.createYield = createYield;
        function updateYield(node, asteriskToken, expression) {
            return node.expression !== expression
                || node.asteriskToken !== asteriskToken
                ? updateNode(createYield(asteriskToken, expression), node)
                : node;
        }
        ts.updateYield = updateYield;
        function createSpread(expression) {
            var node = createSynthesizedNode(213 /* SpreadElement */);
            node.expression = ts.parenthesizeExpressionForList(expression);
            return node;
        }
        ts.createSpread = createSpread;
        function updateSpread(node, expression) {
            return node.expression !== expression
                ? updateNode(createSpread(expression), node)
                : node;
        }
        ts.updateSpread = updateSpread;
        function createClassExpression(modifiers, name, typeParameters, heritageClauses, members) {
            var node = createSynthesizedNode(214 /* ClassExpression */);
            node.decorators = undefined;
            node.modifiers = asNodeArray(modifiers);
            node.name = asName(name);
            node.typeParameters = asNodeArray(typeParameters);
            node.heritageClauses = asNodeArray(heritageClauses);
            node.members = createNodeArray(members);
            return node;
        }
        ts.createClassExpression = createClassExpression;
        function updateClassExpression(node, modifiers, name, typeParameters, heritageClauses, members) {
            return node.modifiers !== modifiers
                || node.name !== name
                || node.typeParameters !== typeParameters
                || node.heritageClauses !== heritageClauses
                || node.members !== members
                ? updateNode(createClassExpression(modifiers, name, typeParameters, heritageClauses, members), node)
                : node;
        }
        ts.updateClassExpression = updateClassExpression;
        function createOmittedExpression() {
            return createSynthesizedNode(215 /* OmittedExpression */);
        }
        ts.createOmittedExpression = createOmittedExpression;
        function createExpressionWithTypeArguments(typeArguments, expression) {
            var node = createSynthesizedNode(216 /* ExpressionWithTypeArguments */);
            node.expression = ts.parenthesizeForAccess(expression);
            node.typeArguments = asNodeArray(typeArguments);
            return node;
        }
        ts.createExpressionWithTypeArguments = createExpressionWithTypeArguments;
        function updateExpressionWithTypeArguments(node, typeArguments, expression) {
            return node.typeArguments !== typeArguments
                || node.expression !== expression
                ? updateNode(createExpressionWithTypeArguments(typeArguments, expression), node)
                : node;
        }
        ts.updateExpressionWithTypeArguments = updateExpressionWithTypeArguments;
        function createAsExpression(expression, type) {
            var node = createSynthesizedNode(217 /* AsExpression */);
            node.expression = expression;
            node.type = type;
            return node;
        }
        ts.createAsExpression = createAsExpression;
        function updateAsExpression(node, expression, type) {
            return node.expression !== expression
                || node.type !== type
                ? updateNode(createAsExpression(expression, type), node)
                : node;
        }
        ts.updateAsExpression = updateAsExpression;
        function createNonNullExpression(expression) {
            var node = createSynthesizedNode(218 /* NonNullExpression */);
            node.expression = ts.parenthesizeForAccess(expression);
            return node;
        }
        ts.createNonNullExpression = createNonNullExpression;
        function updateNonNullExpression(node, expression) {
            return node.expression !== expression
                ? updateNode(createNonNullExpression(expression), node)
                : node;
        }
        ts.updateNonNullExpression = updateNonNullExpression;
        function createMetaProperty(keywordToken, name) {
            var node = createSynthesizedNode(219 /* MetaProperty */);
            node.keywordToken = keywordToken;
            node.name = name;
            return node;
        }
        ts.createMetaProperty = createMetaProperty;
        function updateMetaProperty(node, name) {
            return node.name !== name
                ? updateNode(createMetaProperty(node.keywordToken, name), node)
                : node;
        }
        ts.updateMetaProperty = updateMetaProperty;
        // Misc
        function createTemplateSpan(expression, literal) {
            var node = createSynthesizedNode(221 /* TemplateSpan */);
            node.expression = expression;
            node.literal = literal;
            return node;
        }
        ts.createTemplateSpan = createTemplateSpan;
        function updateTemplateSpan(node, expression, literal) {
            return node.expression !== expression
                || node.literal !== literal
                ? updateNode(createTemplateSpan(expression, literal), node)
                : node;
        }
        ts.updateTemplateSpan = updateTemplateSpan;
        function createSemicolonClassElement() {
            return createSynthesizedNode(222 /* SemicolonClassElement */);
        }
        ts.createSemicolonClassElement = createSemicolonClassElement;
        // Element
        function createBlock(statements, multiLine) {
            var block = createSynthesizedNode(223 /* Block */);
            block.statements = createNodeArray(statements);
            if (multiLine)
                block.multiLine = multiLine;
            return block;
        }
        ts.createBlock = createBlock;
        function updateBlock(node, statements) {
            return node.statements !== statements
                ? updateNode(createBlock(statements, node.multiLine), node)
                : node;
        }
        ts.updateBlock = updateBlock;
        function createVariableStatement(modifiers, declarationList) {
            var node = createSynthesizedNode(225 /* VariableStatement */);
            node.decorators = undefined;
            node.modifiers = asNodeArray(modifiers);
            node.declarationList = ts.isArray(declarationList) ? createVariableDeclarationList(declarationList) : declarationList;
            return node;
        }
        ts.createVariableStatement = createVariableStatement;
        function updateVariableStatement(node, modifiers, declarationList) {
            return node.modifiers !== modifiers
                || node.declarationList !== declarationList
                ? updateNode(createVariableStatement(modifiers, declarationList), node)
                : node;
        }
        ts.updateVariableStatement = updateVariableStatement;
        function createEmptyStatement() {
            return createSynthesizedNode(224 /* EmptyStatement */);
        }
        ts.createEmptyStatement = createEmptyStatement;
        function createExpressionStatement(expression) {
            var node = createSynthesizedNode(226 /* ExpressionStatement */);
            node.expression = ts.parenthesizeExpressionForExpressionStatement(expression);
            return node;
        }
        ts.createExpressionStatement = createExpressionStatement;
        function updateExpressionStatement(node, expression) {
            return node.expression !== expression
                ? updateNode(createExpressionStatement(expression), node)
                : node;
        }
        ts.updateExpressionStatement = updateExpressionStatement;
        /** @deprecated Use `createExpressionStatement` instead.  */
        ts.createStatement = createExpressionStatement;
        /** @deprecated Use `updateExpressionStatement` instead.  */
        ts.updateStatement = updateExpressionStatement;
        function createIf(expression, thenStatement, elseStatement) {
            var node = createSynthesizedNode(227 /* IfStatement */);
            node.expression = expression;
            node.thenStatement = asEmbeddedStatement(thenStatement);
            node.elseStatement = asEmbeddedStatement(elseStatement);
            return node;
        }
        ts.createIf = createIf;
        function updateIf(node, expression, thenStatement, elseStatement) {
            return node.expression !== expression
                || node.thenStatement !== thenStatement
                || node.elseStatement !== elseStatement
                ? updateNode(createIf(expression, thenStatement, elseStatement), node)
                : node;
        }
        ts.updateIf = updateIf;
        function createDo(statement, expression) {
            var node = createSynthesizedNode(228 /* DoStatement */);
            node.statement = asEmbeddedStatement(statement);
            node.expression = expression;
            return node;
        }
        ts.createDo = createDo;
        function updateDo(node, statement, expression) {
            return node.statement !== statement
                || node.expression !== expression
                ? updateNode(createDo(statement, expression), node)
                : node;
        }
        ts.updateDo = updateDo;
        function createWhile(expression, statement) {
            var node = createSynthesizedNode(229 /* WhileStatement */);
            node.expression = expression;
            node.statement = asEmbeddedStatement(statement);
            return node;
        }
        ts.createWhile = createWhile;
        function updateWhile(node, expression, statement) {
            return node.expression !== expression
                || node.statement !== statement
                ? updateNode(createWhile(expression, statement), node)
                : node;
        }
        ts.updateWhile = updateWhile;
        function createFor(initializer, condition, incrementor, statement) {
            var node = createSynthesizedNode(230 /* ForStatement */);
            node.initializer = initializer;
            node.condition = condition;
            node.incrementor = incrementor;
            node.statement = asEmbeddedStatement(statement);
            return node;
        }
        ts.createFor = createFor;
        function updateFor(node, initializer, condition, incrementor, statement) {
            return node.initializer !== initializer
                || node.condition !== condition
                || node.incrementor !== incrementor
                || node.statement !== statement
                ? updateNode(createFor(initializer, condition, incrementor, statement), node)
                : node;
        }
        ts.updateFor = updateFor;
        function createForIn(initializer, expression, statement) {
            var node = createSynthesizedNode(231 /* ForInStatement */);
            node.initializer = initializer;
            node.expression = expression;
            node.statement = asEmbeddedStatement(statement);
            return node;
        }
        ts.createForIn = createForIn;
        function updateForIn(node, initializer, expression, statement) {
            return node.initializer !== initializer
                || node.expression !== expression
                || node.statement !== statement
                ? updateNode(createForIn(initializer, expression, statement), node)
                : node;
        }
        ts.updateForIn = updateForIn;
        function createForOf(awaitModifier, initializer, expression, statement) {
            var node = createSynthesizedNode(232 /* ForOfStatement */);
            node.awaitModifier = awaitModifier;
            node.initializer = initializer;
            node.expression = ts.isCommaSequence(expression) ? createParen(expression) : expression;
            node.statement = asEmbeddedStatement(statement);
            return node;
        }
        ts.createForOf = createForOf;
        function updateForOf(node, awaitModifier, initializer, expression, statement) {
            return node.awaitModifier !== awaitModifier
                || node.initializer !== initializer
                || node.expression !== expression
                || node.statement !== statement
                ? updateNode(createForOf(awaitModifier, initializer, expression, statement), node)
                : node;
        }
        ts.updateForOf = updateForOf;
        function createContinue(label) {
            var node = createSynthesizedNode(233 /* ContinueStatement */);
            node.label = asName(label);
            return node;
        }
        ts.createContinue = createContinue;
        function updateContinue(node, label) {
            return node.label !== label
                ? updateNode(createContinue(label), node)
                : node;
        }
        ts.updateContinue = updateContinue;
        function createBreak(label) {
            var node = createSynthesizedNode(234 /* BreakStatement */);
            node.label = asName(label);
            return node;
        }
        ts.createBreak = createBreak;
        function updateBreak(node, label) {
            return node.label !== label
                ? updateNode(createBreak(label), node)
                : node;
        }
        ts.updateBreak = updateBreak;
        function createReturn(expression) {
            var node = createSynthesizedNode(235 /* ReturnStatement */);
            node.expression = expression;
            return node;
        }
        ts.createReturn = createReturn;
        function updateReturn(node, expression) {
            return node.expression !== expression
                ? updateNode(createReturn(expression), node)
                : node;
        }
        ts.updateReturn = updateReturn;
        function createWith(expression, statement) {
            var node = createSynthesizedNode(236 /* WithStatement */);
            node.expression = expression;
            node.statement = asEmbeddedStatement(statement);
            return node;
        }
        ts.createWith = createWith;
        function updateWith(node, expression, statement) {
            return node.expression !== expression
                || node.statement !== statement
                ? updateNode(createWith(expression, statement), node)
                : node;
        }
        ts.updateWith = updateWith;
        function createSwitch(expression, caseBlock) {
            var node = createSynthesizedNode(237 /* SwitchStatement */);
            node.expression = ts.parenthesizeExpressionForList(expression);
            node.caseBlock = caseBlock;
            return node;
        }
        ts.createSwitch = createSwitch;
        function updateSwitch(node, expression, caseBlock) {
            return node.expression !== expression
                || node.caseBlock !== caseBlock
                ? updateNode(createSwitch(expression, caseBlock), node)
                : node;
        }
        ts.updateSwitch = updateSwitch;
        function createLabel(label, statement) {
            var node = createSynthesizedNode(238 /* LabeledStatement */);
            node.label = asName(label);
            node.statement = asEmbeddedStatement(statement);
            return node;
        }
        ts.createLabel = createLabel;
        function updateLabel(node, label, statement) {
            return node.label !== label
                || node.statement !== statement
                ? updateNode(createLabel(label, statement), node)
                : node;
        }
        ts.updateLabel = updateLabel;
        function createThrow(expression) {
            var node = createSynthesizedNode(239 /* ThrowStatement */);
            node.expression = expression;
            return node;
        }
        ts.createThrow = createThrow;
        function updateThrow(node, expression) {
            return node.expression !== expression
                ? updateNode(createThrow(expression), node)
                : node;
        }
        ts.updateThrow = updateThrow;
        function createTry(tryBlock, catchClause, finallyBlock) {
            var node = createSynthesizedNode(240 /* TryStatement */);
            node.tryBlock = tryBlock;
            node.catchClause = catchClause;
            node.finallyBlock = finallyBlock;
            return node;
        }
        ts.createTry = createTry;
        function updateTry(node, tryBlock, catchClause, finallyBlock) {
            return node.tryBlock !== tryBlock
                || node.catchClause !== catchClause
                || node.finallyBlock !== finallyBlock
                ? updateNode(createTry(tryBlock, catchClause, finallyBlock), node)
                : node;
        }
        ts.updateTry = updateTry;
        function createDebuggerStatement() {
            return createSynthesizedNode(241 /* DebuggerStatement */);
        }
        ts.createDebuggerStatement = createDebuggerStatement;
        function createVariableDeclaration(name, type, initializer) {
            /* Internally, one should probably use createTypeScriptVariableDeclaration instead and handle definite assignment assertions */
            var node = createSynthesizedNode(242 /* VariableDeclaration */);
            node.name = asName(name);
            node.type = type;
            node.initializer = initializer !== undefined ? ts.parenthesizeExpressionForList(initializer) : undefined;
            return node;
        }
        ts.createVariableDeclaration = createVariableDeclaration;
        function updateVariableDeclaration(node, name, type, initializer) {
            /* Internally, one should probably use updateTypeScriptVariableDeclaration instead and handle definite assignment assertions */
            return node.name !== name
                || node.type !== type
                || node.initializer !== initializer
                ? updateNode(createVariableDeclaration(name, type, initializer), node)
                : node;
        }
        ts.updateVariableDeclaration = updateVariableDeclaration;
        /* @internal */
        function createTypeScriptVariableDeclaration(name, exclaimationToken, type, initializer) {
            var node = createSynthesizedNode(242 /* VariableDeclaration */);
            node.name = asName(name);
            node.type = type;
            node.initializer = initializer !== undefined ? ts.parenthesizeExpressionForList(initializer) : undefined;
            node.exclamationToken = exclaimationToken;
            return node;
        }
        ts.createTypeScriptVariableDeclaration = createTypeScriptVariableDeclaration;
        /* @internal */
        function updateTypeScriptVariableDeclaration(node, name, exclaimationToken, type, initializer) {
            return node.name !== name
                || node.type !== type
                || node.initializer !== initializer
                || node.exclamationToken !== exclaimationToken
                ? updateNode(createTypeScriptVariableDeclaration(name, exclaimationToken, type, initializer), node)
                : node;
        }
        ts.updateTypeScriptVariableDeclaration = updateTypeScriptVariableDeclaration;
        function createVariableDeclarationList(declarations, flags) {
            if (flags === void 0) {
                flags = 0 /* None */;
            }
            var node = createSynthesizedNode(243 /* VariableDeclarationList */);
            node.flags |= flags & 3 /* BlockScoped */;
            node.declarations = createNodeArray(declarations);
            return node;
        }
        ts.createVariableDeclarationList = createVariableDeclarationList;
        function updateVariableDeclarationList(node, declarations) {
            return node.declarations !== declarations
                ? updateNode(createVariableDeclarationList(declarations, node.flags), node)
                : node;
        }
        ts.updateVariableDeclarationList = updateVariableDeclarationList;
        function createFunctionDeclaration(decorators, modifiers, asteriskToken, name, typeParameters, parameters, type, body) {
            var node = createSynthesizedNode(244 /* FunctionDeclaration */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.asteriskToken = asteriskToken;
            node.name = asName(name);
            node.typeParameters = asNodeArray(typeParameters);
            node.parameters = createNodeArray(parameters);
            node.type = type;
            node.body = body;
            return node;
        }
        ts.createFunctionDeclaration = createFunctionDeclaration;
        function updateFunctionDeclaration(node, decorators, modifiers, asteriskToken, name, typeParameters, parameters, type, body) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.asteriskToken !== asteriskToken
                || node.name !== name
                || node.typeParameters !== typeParameters
                || node.parameters !== parameters
                || node.type !== type
                || node.body !== body
                ? updateNode(createFunctionDeclaration(decorators, modifiers, asteriskToken, name, typeParameters, parameters, type, body), node)
                : node;
        }
        ts.updateFunctionDeclaration = updateFunctionDeclaration;
        function createClassDeclaration(decorators, modifiers, name, typeParameters, heritageClauses, members) {
            var node = createSynthesizedNode(245 /* ClassDeclaration */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.name = asName(name);
            node.typeParameters = asNodeArray(typeParameters);
            node.heritageClauses = asNodeArray(heritageClauses);
            node.members = createNodeArray(members);
            return node;
        }
        ts.createClassDeclaration = createClassDeclaration;
        function updateClassDeclaration(node, decorators, modifiers, name, typeParameters, heritageClauses, members) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.name !== name
                || node.typeParameters !== typeParameters
                || node.heritageClauses !== heritageClauses
                || node.members !== members
                ? updateNode(createClassDeclaration(decorators, modifiers, name, typeParameters, heritageClauses, members), node)
                : node;
        }
        ts.updateClassDeclaration = updateClassDeclaration;
        function createInterfaceDeclaration(decorators, modifiers, name, typeParameters, heritageClauses, members) {
            var node = createSynthesizedNode(246 /* InterfaceDeclaration */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.name = asName(name);
            node.typeParameters = asNodeArray(typeParameters);
            node.heritageClauses = asNodeArray(heritageClauses);
            node.members = createNodeArray(members);
            return node;
        }
        ts.createInterfaceDeclaration = createInterfaceDeclaration;
        function updateInterfaceDeclaration(node, decorators, modifiers, name, typeParameters, heritageClauses, members) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.name !== name
                || node.typeParameters !== typeParameters
                || node.heritageClauses !== heritageClauses
                || node.members !== members
                ? updateNode(createInterfaceDeclaration(decorators, modifiers, name, typeParameters, heritageClauses, members), node)
                : node;
        }
        ts.updateInterfaceDeclaration = updateInterfaceDeclaration;
        function createTypeAliasDeclaration(decorators, modifiers, name, typeParameters, type) {
            var node = createSynthesizedNode(247 /* TypeAliasDeclaration */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.name = asName(name);
            node.typeParameters = asNodeArray(typeParameters);
            node.type = type;
            return node;
        }
        ts.createTypeAliasDeclaration = createTypeAliasDeclaration;
        function updateTypeAliasDeclaration(node, decorators, modifiers, name, typeParameters, type) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.name !== name
                || node.typeParameters !== typeParameters
                || node.type !== type
                ? updateNode(createTypeAliasDeclaration(decorators, modifiers, name, typeParameters, type), node)
                : node;
        }
        ts.updateTypeAliasDeclaration = updateTypeAliasDeclaration;
        function createEnumDeclaration(decorators, modifiers, name, members) {
            var node = createSynthesizedNode(248 /* EnumDeclaration */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.name = asName(name);
            node.members = createNodeArray(members);
            return node;
        }
        ts.createEnumDeclaration = createEnumDeclaration;
        function updateEnumDeclaration(node, decorators, modifiers, name, members) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.name !== name
                || node.members !== members
                ? updateNode(createEnumDeclaration(decorators, modifiers, name, members), node)
                : node;
        }
        ts.updateEnumDeclaration = updateEnumDeclaration;
        function createModuleDeclaration(decorators, modifiers, name, body, flags) {
            if (flags === void 0) {
                flags = 0 /* None */;
            }
            var node = createSynthesizedNode(249 /* ModuleDeclaration */);
            node.flags |= flags & (16 /* Namespace */ | 4 /* NestedNamespace */ | 1024 /* GlobalAugmentation */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.name = name;
            node.body = body;
            return node;
        }
        ts.createModuleDeclaration = createModuleDeclaration;
        function updateModuleDeclaration(node, decorators, modifiers, name, body) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.name !== name
                || node.body !== body
                ? updateNode(createModuleDeclaration(decorators, modifiers, name, body, node.flags), node)
                : node;
        }
        ts.updateModuleDeclaration = updateModuleDeclaration;
        function createModuleBlock(statements) {
            var node = createSynthesizedNode(250 /* ModuleBlock */);
            node.statements = createNodeArray(statements);
            return node;
        }
        ts.createModuleBlock = createModuleBlock;
        function updateModuleBlock(node, statements) {
            return node.statements !== statements
                ? updateNode(createModuleBlock(statements), node)
                : node;
        }
        ts.updateModuleBlock = updateModuleBlock;
        function createCaseBlock(clauses) {
            var node = createSynthesizedNode(251 /* CaseBlock */);
            node.clauses = createNodeArray(clauses);
            return node;
        }
        ts.createCaseBlock = createCaseBlock;
        function updateCaseBlock(node, clauses) {
            return node.clauses !== clauses
                ? updateNode(createCaseBlock(clauses), node)
                : node;
        }
        ts.updateCaseBlock = updateCaseBlock;
        function createNamespaceExportDeclaration(name) {
            var node = createSynthesizedNode(252 /* NamespaceExportDeclaration */);
            node.name = asName(name);
            return node;
        }
        ts.createNamespaceExportDeclaration = createNamespaceExportDeclaration;
        function updateNamespaceExportDeclaration(node, name) {
            return node.name !== name
                ? updateNode(createNamespaceExportDeclaration(name), node)
                : node;
        }
        ts.updateNamespaceExportDeclaration = updateNamespaceExportDeclaration;
        function createImportEqualsDeclaration(decorators, modifiers, name, moduleReference) {
            var node = createSynthesizedNode(253 /* ImportEqualsDeclaration */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.name = asName(name);
            node.moduleReference = moduleReference;
            return node;
        }
        ts.createImportEqualsDeclaration = createImportEqualsDeclaration;
        function updateImportEqualsDeclaration(node, decorators, modifiers, name, moduleReference) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.name !== name
                || node.moduleReference !== moduleReference
                ? updateNode(createImportEqualsDeclaration(decorators, modifiers, name, moduleReference), node)
                : node;
        }
        ts.updateImportEqualsDeclaration = updateImportEqualsDeclaration;
        function createImportDeclaration(decorators, modifiers, importClause, moduleSpecifier) {
            var node = createSynthesizedNode(254 /* ImportDeclaration */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.importClause = importClause;
            node.moduleSpecifier = moduleSpecifier;
            return node;
        }
        ts.createImportDeclaration = createImportDeclaration;
        function updateImportDeclaration(node, decorators, modifiers, importClause, moduleSpecifier) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.importClause !== importClause
                || node.moduleSpecifier !== moduleSpecifier
                ? updateNode(createImportDeclaration(decorators, modifiers, importClause, moduleSpecifier), node)
                : node;
        }
        ts.updateImportDeclaration = updateImportDeclaration;
        function createImportClause(name, namedBindings, isTypeOnly) {
            if (isTypeOnly === void 0) {
                isTypeOnly = false;
            }
            var node = createSynthesizedNode(255 /* ImportClause */);
            node.name = name;
            node.namedBindings = namedBindings;
            node.isTypeOnly = isTypeOnly;
            return node;
        }
        ts.createImportClause = createImportClause;
        function updateImportClause(node, name, namedBindings, isTypeOnly) {
            return node.name !== name
                || node.namedBindings !== namedBindings
                || node.isTypeOnly !== isTypeOnly
                ? updateNode(createImportClause(name, namedBindings, isTypeOnly), node)
                : node;
        }
        ts.updateImportClause = updateImportClause;
        function createNamespaceImport(name) {
            var node = createSynthesizedNode(256 /* NamespaceImport */);
            node.name = name;
            return node;
        }
        ts.createNamespaceImport = createNamespaceImport;
        function createNamespaceExport(name) {
            var node = createSynthesizedNode(262 /* NamespaceExport */);
            node.name = name;
            return node;
        }
        ts.createNamespaceExport = createNamespaceExport;
        function updateNamespaceImport(node, name) {
            return node.name !== name
                ? updateNode(createNamespaceImport(name), node)
                : node;
        }
        ts.updateNamespaceImport = updateNamespaceImport;
        function updateNamespaceExport(node, name) {
            return node.name !== name
                ? updateNode(createNamespaceExport(name), node)
                : node;
        }
        ts.updateNamespaceExport = updateNamespaceExport;
        function createNamedImports(elements) {
            var node = createSynthesizedNode(257 /* NamedImports */);
            node.elements = createNodeArray(elements);
            return node;
        }
        ts.createNamedImports = createNamedImports;
        function updateNamedImports(node, elements) {
            return node.elements !== elements
                ? updateNode(createNamedImports(elements), node)
                : node;
        }
        ts.updateNamedImports = updateNamedImports;
        function createImportSpecifier(propertyName, name) {
            var node = createSynthesizedNode(258 /* ImportSpecifier */);
            node.propertyName = propertyName;
            node.name = name;
            return node;
        }
        ts.createImportSpecifier = createImportSpecifier;
        function updateImportSpecifier(node, propertyName, name) {
            return node.propertyName !== propertyName
                || node.name !== name
                ? updateNode(createImportSpecifier(propertyName, name), node)
                : node;
        }
        ts.updateImportSpecifier = updateImportSpecifier;
        function createExportAssignment(decorators, modifiers, isExportEquals, expression) {
            var node = createSynthesizedNode(259 /* ExportAssignment */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.isExportEquals = isExportEquals;
            node.expression = isExportEquals ? ts.parenthesizeBinaryOperand(62 /* EqualsToken */, expression, /*isLeftSideOfBinary*/ false, /*leftOperand*/ undefined) : ts.parenthesizeDefaultExpression(expression);
            return node;
        }
        ts.createExportAssignment = createExportAssignment;
        function updateExportAssignment(node, decorators, modifiers, expression) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.expression !== expression
                ? updateNode(createExportAssignment(decorators, modifiers, node.isExportEquals, expression), node)
                : node;
        }
        ts.updateExportAssignment = updateExportAssignment;
        function createExportDeclaration(decorators, modifiers, exportClause, moduleSpecifier, isTypeOnly) {
            if (isTypeOnly === void 0) {
                isTypeOnly = false;
            }
            var node = createSynthesizedNode(260 /* ExportDeclaration */);
            node.decorators = asNodeArray(decorators);
            node.modifiers = asNodeArray(modifiers);
            node.isTypeOnly = isTypeOnly;
            node.exportClause = exportClause;
            node.moduleSpecifier = moduleSpecifier;
            return node;
        }
        ts.createExportDeclaration = createExportDeclaration;
        function updateExportDeclaration(node, decorators, modifiers, exportClause, moduleSpecifier, isTypeOnly) {
            return node.decorators !== decorators
                || node.modifiers !== modifiers
                || node.isTypeOnly !== isTypeOnly
                || node.exportClause !== exportClause
                || node.moduleSpecifier !== moduleSpecifier
                ? updateNode(createExportDeclaration(decorators, modifiers, exportClause, moduleSpecifier, isTypeOnly), node)
                : node;
        }
        ts.updateExportDeclaration = updateExportDeclaration;
        /* @internal */
        function createEmptyExports() {
            return createExportDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, createNamedExports([]), /*moduleSpecifier*/ undefined);
        }
        ts.createEmptyExports = createEmptyExports;
        function createNamedExports(elements) {
            var node = createSynthesizedNode(261 /* NamedExports */);
            node.elements = createNodeArray(elements);
            return node;
        }
        ts.createNamedExports = createNamedExports;
        function updateNamedExports(node, elements) {
            return node.elements !== elements
                ? updateNode(createNamedExports(elements), node)
                : node;
        }
        ts.updateNamedExports = updateNamedExports;
        function createExportSpecifier(propertyName, name) {
            var node = createSynthesizedNode(263 /* ExportSpecifier */);
            node.propertyName = asName(propertyName);
            node.name = asName(name);
            return node;
        }
        ts.createExportSpecifier = createExportSpecifier;
        function updateExportSpecifier(node, propertyName, name) {
            return node.propertyName !== propertyName
                || node.name !== name
                ? updateNode(createExportSpecifier(propertyName, name), node)
                : node;
        }
        ts.updateExportSpecifier = updateExportSpecifier;
        // Module references
        function createExternalModuleReference(expression) {
            var node = createSynthesizedNode(265 /* ExternalModuleReference */);
            node.expression = expression;
            return node;
        }
        ts.createExternalModuleReference = createExternalModuleReference;
        function updateExternalModuleReference(node, expression) {
            return node.expression !== expression
                ? updateNode(createExternalModuleReference(expression), node)
                : node;
        }
        ts.updateExternalModuleReference = updateExternalModuleReference;
        // JSDoc
        /* @internal */
        function createJSDocTypeExpression(type) {
            var node = createSynthesizedNode(294 /* JSDocTypeExpression */);
            node.type = type;
            return node;
        }
        ts.createJSDocTypeExpression = createJSDocTypeExpression;
        /* @internal */
        function createJSDocTypeTag(typeExpression, comment) {
            var tag = createJSDocTag(319 /* JSDocTypeTag */, "type");
            tag.typeExpression = typeExpression;
            tag.comment = comment;
            return tag;
        }
        ts.createJSDocTypeTag = createJSDocTypeTag;
        /* @internal */
        function createJSDocReturnTag(typeExpression, comment) {
            var tag = createJSDocTag(317 /* JSDocReturnTag */, "returns");
            tag.typeExpression = typeExpression;
            tag.comment = comment;
            return tag;
        }
        ts.createJSDocReturnTag = createJSDocReturnTag;
        /** @internal */
        function createJSDocThisTag(typeExpression) {
            var tag = createJSDocTag(318 /* JSDocThisTag */, "this");
            tag.typeExpression = typeExpression;
            return tag;
        }
        ts.createJSDocThisTag = createJSDocThisTag;
        /* @internal */
        function createJSDocParamTag(name, isBracketed, typeExpression, comment) {
            var tag = createJSDocTag(316 /* JSDocParameterTag */, "param");
            tag.typeExpression = typeExpression;
            tag.name = name;
            tag.isBracketed = isBracketed;
            tag.comment = comment;
            return tag;
        }
        ts.createJSDocParamTag = createJSDocParamTag;
        /* @internal */
        function createJSDocComment(comment, tags) {
            var node = createSynthesizedNode(303 /* JSDocComment */);
            node.comment = comment;
            node.tags = tags;
            return node;
        }
        ts.createJSDocComment = createJSDocComment;
        /* @internal */
        function createJSDocTag(kind, tagName) {
            var node = createSynthesizedNode(kind);
            node.tagName = createIdentifier(tagName);
            return node;
        }
        // JSX
        function createJsxElement(openingElement, children, closingElement) {
            var node = createSynthesizedNode(266 /* JsxElement */);
            node.openingElement = openingElement;
            node.children = createNodeArray(children);
            node.closingElement = closingElement;
            return node;
        }
        ts.createJsxElement = createJsxElement;
        function updateJsxElement(node, openingElement, children, closingElement) {
            return node.openingElement !== openingElement
                || node.children !== children
                || node.closingElement !== closingElement
                ? updateNode(createJsxElement(openingElement, children, closingElement), node)
                : node;
        }
        ts.updateJsxElement = updateJsxElement;
        function createJsxSelfClosingElement(tagName, typeArguments, attributes) {
            var node = createSynthesizedNode(267 /* JsxSelfClosingElement */);
            node.tagName = tagName;
            node.typeArguments = asNodeArray(typeArguments);
            node.attributes = attributes;
            return node;
        }
        ts.createJsxSelfClosingElement = createJsxSelfClosingElement;
        function updateJsxSelfClosingElement(node, tagName, typeArguments, attributes) {
            return node.tagName !== tagName
                || node.typeArguments !== typeArguments
                || node.attributes !== attributes
                ? updateNode(createJsxSelfClosingElement(tagName, typeArguments, attributes), node)
                : node;
        }
        ts.updateJsxSelfClosingElement = updateJsxSelfClosingElement;
        function createJsxOpeningElement(tagName, typeArguments, attributes) {
            var node = createSynthesizedNode(268 /* JsxOpeningElement */);
            node.tagName = tagName;
            node.typeArguments = asNodeArray(typeArguments);
            node.attributes = attributes;
            return node;
        }
        ts.createJsxOpeningElement = createJsxOpeningElement;
        function updateJsxOpeningElement(node, tagName, typeArguments, attributes) {
            return node.tagName !== tagName
                || node.typeArguments !== typeArguments
                || node.attributes !== attributes
                ? updateNode(createJsxOpeningElement(tagName, typeArguments, attributes), node)
                : node;
        }
        ts.updateJsxOpeningElement = updateJsxOpeningElement;
        function createJsxClosingElement(tagName) {
            var node = createSynthesizedNode(269 /* JsxClosingElement */);
            node.tagName = tagName;
            return node;
        }
        ts.createJsxClosingElement = createJsxClosingElement;
        function updateJsxClosingElement(node, tagName) {
            return node.tagName !== tagName
                ? updateNode(createJsxClosingElement(tagName), node)
                : node;
        }
        ts.updateJsxClosingElement = updateJsxClosingElement;
        function createJsxFragment(openingFragment, children, closingFragment) {
            var node = createSynthesizedNode(270 /* JsxFragment */);
            node.openingFragment = openingFragment;
            node.children = createNodeArray(children);
            node.closingFragment = closingFragment;
            return node;
        }
        ts.createJsxFragment = createJsxFragment;
        function createJsxText(text, containsOnlyTriviaWhiteSpaces) {
            var node = createSynthesizedNode(11 /* JsxText */);
            node.text = text;
            node.containsOnlyTriviaWhiteSpaces = !!containsOnlyTriviaWhiteSpaces;
            return node;
        }
        ts.createJsxText = createJsxText;
        function updateJsxText(node, text, containsOnlyTriviaWhiteSpaces) {
            return node.text !== text
                || node.containsOnlyTriviaWhiteSpaces !== containsOnlyTriviaWhiteSpaces
                ? updateNode(createJsxText(text, containsOnlyTriviaWhiteSpaces), node)
                : node;
        }
        ts.updateJsxText = updateJsxText;
        function createJsxOpeningFragment() {
            return createSynthesizedNode(271 /* JsxOpeningFragment */);
        }
        ts.createJsxOpeningFragment = createJsxOpeningFragment;
        function createJsxJsxClosingFragment() {
            return createSynthesizedNode(272 /* JsxClosingFragment */);
        }
        ts.createJsxJsxClosingFragment = createJsxJsxClosingFragment;
        function updateJsxFragment(node, openingFragment, children, closingFragment) {
            return node.openingFragment !== openingFragment
                || node.children !== children
                || node.closingFragment !== closingFragment
                ? updateNode(createJsxFragment(openingFragment, children, closingFragment), node)
                : node;
        }
        ts.updateJsxFragment = updateJsxFragment;
        function createJsxAttribute(name, initializer) {
            var node = createSynthesizedNode(273 /* JsxAttribute */);
            node.name = name;
            node.initializer = initializer;
            return node;
        }
        ts.createJsxAttribute = createJsxAttribute;
        function updateJsxAttribute(node, name, initializer) {
            return node.name !== name
                || node.initializer !== initializer
                ? updateNode(createJsxAttribute(name, initializer), node)
                : node;
        }
        ts.updateJsxAttribute = updateJsxAttribute;
        function createJsxAttributes(properties) {
            var node = createSynthesizedNode(274 /* JsxAttributes */);
            node.properties = createNodeArray(properties);
            return node;
        }
        ts.createJsxAttributes = createJsxAttributes;
        function updateJsxAttributes(node, properties) {
            return node.properties !== properties
                ? updateNode(createJsxAttributes(properties), node)
                : node;
        }
        ts.updateJsxAttributes = updateJsxAttributes;
        function createJsxSpreadAttribute(expression) {
            var node = createSynthesizedNode(275 /* JsxSpreadAttribute */);
            node.expression = expression;
            return node;
        }
        ts.createJsxSpreadAttribute = createJsxSpreadAttribute;
        function updateJsxSpreadAttribute(node, expression) {
            return node.expression !== expression
                ? updateNode(createJsxSpreadAttribute(expression), node)
                : node;
        }
        ts.updateJsxSpreadAttribute = updateJsxSpreadAttribute;
        function createJsxExpression(dotDotDotToken, expression) {
            var node = createSynthesizedNode(276 /* JsxExpression */);
            node.dotDotDotToken = dotDotDotToken;
            node.expression = expression;
            return node;
        }
        ts.createJsxExpression = createJsxExpression;
        function updateJsxExpression(node, expression) {
            return node.expression !== expression
                ? updateNode(createJsxExpression(node.dotDotDotToken, expression), node)
                : node;
        }
        ts.updateJsxExpression = updateJsxExpression;
        // Clauses
        function createCaseClause(expression, statements) {
            var node = createSynthesizedNode(277 /* CaseClause */);
            node.expression = ts.parenthesizeExpressionForList(expression);
            node.statements = createNodeArray(statements);
            return node;
        }
        ts.createCaseClause = createCaseClause;
        function updateCaseClause(node, expression, statements) {
            return node.expression !== expression
                || node.statements !== statements
                ? updateNode(createCaseClause(expression, statements), node)
                : node;
        }
        ts.updateCaseClause = updateCaseClause;
        function createDefaultClause(statements) {
            var node = createSynthesizedNode(278 /* DefaultClause */);
            node.statements = createNodeArray(statements);
            return node;
        }
        ts.createDefaultClause = createDefaultClause;
        function updateDefaultClause(node, statements) {
            return node.statements !== statements
                ? updateNode(createDefaultClause(statements), node)
                : node;
        }
        ts.updateDefaultClause = updateDefaultClause;
        function createHeritageClause(token, types) {
            var node = createSynthesizedNode(279 /* HeritageClause */);
            node.token = token;
            node.types = createNodeArray(types);
            return node;
        }
        ts.createHeritageClause = createHeritageClause;
        function updateHeritageClause(node, types) {
            return node.types !== types
                ? updateNode(createHeritageClause(node.token, types), node)
                : node;
        }
        ts.updateHeritageClause = updateHeritageClause;
        function createCatchClause(variableDeclaration, block) {
            var node = createSynthesizedNode(280 /* CatchClause */);
            node.variableDeclaration = ts.isString(variableDeclaration) ? createVariableDeclaration(variableDeclaration) : variableDeclaration;
            node.block = block;
            return node;
        }
        ts.createCatchClause = createCatchClause;
        function updateCatchClause(node, variableDeclaration, block) {
            return node.variableDeclaration !== variableDeclaration
                || node.block !== block
                ? updateNode(createCatchClause(variableDeclaration, block), node)
                : node;
        }
        ts.updateCatchClause = updateCatchClause;
        // Property assignments
        function createPropertyAssignment(name, initializer) {
            var node = createSynthesizedNode(281 /* PropertyAssignment */);
            node.name = asName(name);
            node.questionToken = undefined;
            node.initializer = ts.parenthesizeExpressionForList(initializer);
            return node;
        }
        ts.createPropertyAssignment = createPropertyAssignment;
        function updatePropertyAssignment(node, name, initializer) {
            return node.name !== name
                || node.initializer !== initializer
                ? updateNode(createPropertyAssignment(name, initializer), node)
                : node;
        }
        ts.updatePropertyAssignment = updatePropertyAssignment;
        function createShorthandPropertyAssignment(name, objectAssignmentInitializer) {
            var node = createSynthesizedNode(282 /* ShorthandPropertyAssignment */);
            node.name = asName(name);
            node.objectAssignmentInitializer = objectAssignmentInitializer !== undefined ? ts.parenthesizeExpressionForList(objectAssignmentInitializer) : undefined;
            return node;
        }
        ts.createShorthandPropertyAssignment = createShorthandPropertyAssignment;
        function updateShorthandPropertyAssignment(node, name, objectAssignmentInitializer) {
            return node.name !== name
                || node.objectAssignmentInitializer !== objectAssignmentInitializer
                ? updateNode(createShorthandPropertyAssignment(name, objectAssignmentInitializer), node)
                : node;
        }
        ts.updateShorthandPropertyAssignment = updateShorthandPropertyAssignment;
        function createSpreadAssignment(expression) {
            var node = createSynthesizedNode(283 /* SpreadAssignment */);
            node.expression = ts.parenthesizeExpressionForList(expression);
            return node;
        }
        ts.createSpreadAssignment = createSpreadAssignment;
        function updateSpreadAssignment(node, expression) {
            return node.expression !== expression
                ? updateNode(createSpreadAssignment(expression), node)
                : node;
        }
        ts.updateSpreadAssignment = updateSpreadAssignment;
        // Enum
        function createEnumMember(name, initializer) {
            var node = createSynthesizedNode(284 /* EnumMember */);
            node.name = asName(name);
            node.initializer = initializer && ts.parenthesizeExpressionForList(initializer);
            return node;
        }
        ts.createEnumMember = createEnumMember;
        function updateEnumMember(node, name, initializer) {
            return node.name !== name
                || node.initializer !== initializer
                ? updateNode(createEnumMember(name, initializer), node)
                : node;
        }
        ts.updateEnumMember = updateEnumMember;
        // Top-level nodes
        function updateSourceFileNode(node, statements, isDeclarationFile, referencedFiles, typeReferences, hasNoDefaultLib, libReferences) {
            if (node.statements !== statements ||
                (isDeclarationFile !== undefined && node.isDeclarationFile !== isDeclarationFile) ||
                (referencedFiles !== undefined && node.referencedFiles !== referencedFiles) ||
                (typeReferences !== undefined && node.typeReferenceDirectives !== typeReferences) ||
                (libReferences !== undefined && node.libReferenceDirectives !== libReferences) ||
                (hasNoDefaultLib !== undefined && node.hasNoDefaultLib !== hasNoDefaultLib)) {
                var updated = createSynthesizedNode(290 /* SourceFile */);
                updated.flags |= node.flags;
                updated.statements = createNodeArray(statements);
                updated.endOfFileToken = node.endOfFileToken;
                updated.fileName = node.fileName;
                updated.path = node.path;
                updated.text = node.text;
                updated.isDeclarationFile = isDeclarationFile === undefined ? node.isDeclarationFile : isDeclarationFile;
                updated.referencedFiles = referencedFiles === undefined ? node.referencedFiles : referencedFiles;
                updated.typeReferenceDirectives = typeReferences === undefined ? node.typeReferenceDirectives : typeReferences;
                updated.hasNoDefaultLib = hasNoDefaultLib === undefined ? node.hasNoDefaultLib : hasNoDefaultLib;
                updated.libReferenceDirectives = libReferences === undefined ? node.libReferenceDirectives : libReferences;
                if (node.amdDependencies !== undefined)
                    updated.amdDependencies = node.amdDependencies;
                if (node.moduleName !== undefined)
                    updated.moduleName = node.moduleName;
                if (node.languageVariant !== undefined)
                    updated.languageVariant = node.languageVariant;
                if (node.renamedDependencies !== undefined)
                    updated.renamedDependencies = node.renamedDependencies;
                if (node.languageVersion !== undefined)
                    updated.languageVersion = node.languageVersion;
                if (node.scriptKind !== undefined)
                    updated.scriptKind = node.scriptKind;
                if (node.externalModuleIndicator !== undefined)
                    updated.externalModuleIndicator = node.externalModuleIndicator;
                if (node.commonJsModuleIndicator !== undefined)
                    updated.commonJsModuleIndicator = node.commonJsModuleIndicator;
                if (node.identifiers !== undefined)
                    updated.identifiers = node.identifiers;
                if (node.nodeCount !== undefined)
                    updated.nodeCount = node.nodeCount;
                if (node.identifierCount !== undefined)
                    updated.identifierCount = node.identifierCount;
                if (node.symbolCount !== undefined)
                    updated.symbolCount = node.symbolCount;
                if (node.parseDiagnostics !== undefined)
                    updated.parseDiagnostics = node.parseDiagnostics;
                if (node.bindDiagnostics !== undefined)
                    updated.bindDiagnostics = node.bindDiagnostics;
                if (node.bindSuggestionDiagnostics !== undefined)
                    updated.bindSuggestionDiagnostics = node.bindSuggestionDiagnostics;
                if (node.lineMap !== undefined)
                    updated.lineMap = node.lineMap;
                if (node.classifiableNames !== undefined)
                    updated.classifiableNames = node.classifiableNames;
                if (node.resolvedModules !== undefined)
                    updated.resolvedModules = node.resolvedModules;
                if (node.resolvedTypeReferenceDirectiveNames !== undefined)
                    updated.resolvedTypeReferenceDirectiveNames = node.resolvedTypeReferenceDirectiveNames;
                if (node.imports !== undefined)
                    updated.imports = node.imports;
                if (node.moduleAugmentations !== undefined)
                    updated.moduleAugmentations = node.moduleAugmentations;
                if (node.pragmas !== undefined)
                    updated.pragmas = node.pragmas;
                if (node.localJsxFactory !== undefined)
                    updated.localJsxFactory = node.localJsxFactory;
                if (node.localJsxNamespace !== undefined)
                    updated.localJsxNamespace = node.localJsxNamespace;
                return updateNode(updated, node);
            }
            return node;
        }
        ts.updateSourceFileNode = updateSourceFileNode;
        /**
         * Creates a shallow, memberwise clone of a node for mutation.
         */
        function getMutableClone(node) {
            var clone = getSynthesizedClone(node);
            clone.pos = node.pos;
            clone.end = node.end;
            clone.parent = node.parent;
            return clone;
        }
        ts.getMutableClone = getMutableClone;
        // Transformation nodes
        /**
         * Creates a synthetic statement to act as a placeholder for a not-emitted statement in
         * order to preserve comments.
         *
         * @param original The original statement.
         */
        function createNotEmittedStatement(original) {
            var node = createSynthesizedNode(324 /* NotEmittedStatement */);
            node.original = original;
            setTextRange(node, original);
            return node;
        }
        ts.createNotEmittedStatement = createNotEmittedStatement;
        /**
         * Creates a synthetic element to act as a placeholder for the end of an emitted declaration in
         * order to properly emit exports.
         */
        /* @internal */
        function createEndOfDeclarationMarker(original) {
            var node = createSynthesizedNode(328 /* EndOfDeclarationMarker */);
            node.emitNode = {};
            node.original = original;
            return node;
        }
        ts.createEndOfDeclarationMarker = createEndOfDeclarationMarker;
        /**
         * Creates a synthetic element to act as a placeholder for the beginning of a merged declaration in
         * order to properly emit exports.
         */
        /* @internal */
        function createMergeDeclarationMarker(original) {
            var node = createSynthesizedNode(327 /* MergeDeclarationMarker */);
            node.emitNode = {};
            node.original = original;
            return node;
        }
        ts.createMergeDeclarationMarker = createMergeDeclarationMarker;
        /**
         * Creates a synthetic expression to act as a placeholder for a not-emitted expression in
         * order to preserve comments or sourcemap positions.
         *
         * @param expression The inner expression to emit.
         * @param original The original outer expression.
         * @param location The location for the expression. Defaults to the positions from "original" if provided.
         */
        function createPartiallyEmittedExpression(expression, original) {
            var node = createSynthesizedNode(325 /* PartiallyEmittedExpression */);
            node.expression = expression;
            node.original = original;
            setTextRange(node, original);
            return node;
        }
        ts.createPartiallyEmittedExpression = createPartiallyEmittedExpression;
        function updatePartiallyEmittedExpression(node, expression) {
            if (node.expression !== expression) {
                return updateNode(createPartiallyEmittedExpression(expression, node.original), node);
            }
            return node;
        }
        ts.updatePartiallyEmittedExpression = updatePartiallyEmittedExpression;
        function flattenCommaElements(node) {
            if (ts.nodeIsSynthesized(node) && !ts.isParseTreeNode(node) && !node.original && !node.emitNode && !node.id) {
                if (node.kind === 326 /* CommaListExpression */) {
                    return node.elements;
                }
                if (ts.isBinaryExpression(node) && node.operatorToken.kind === 27 /* CommaToken */) {
                    return [node.left, node.right];
                }
            }
            return node;
        }
        function createCommaList(elements) {
            var node = createSynthesizedNode(326 /* CommaListExpression */);
            node.elements = createNodeArray(ts.sameFlatMap(elements, flattenCommaElements));
            return node;
        }
        ts.createCommaList = createCommaList;
        function updateCommaList(node, elements) {
            return node.elements !== elements
                ? updateNode(createCommaList(elements), node)
                : node;
        }
        ts.updateCommaList = updateCommaList;
        /* @internal */
        function createSyntheticReferenceExpression(expression, thisArg) {
            var node = createSynthesizedNode(329 /* SyntheticReferenceExpression */);
            node.expression = expression;
            node.thisArg = thisArg;
            return node;
        }
        ts.createSyntheticReferenceExpression = createSyntheticReferenceExpression;
        /* @internal */
        function updateSyntheticReferenceExpression(node, expression, thisArg) {
            return node.expression !== expression
                || node.thisArg !== thisArg
                ? updateNode(createSyntheticReferenceExpression(expression, thisArg), node)
                : node;
        }
        ts.updateSyntheticReferenceExpression = updateSyntheticReferenceExpression;
        function createBundle(sourceFiles, prepends) {
            if (prepends === void 0) {
                prepends = ts.emptyArray;
            }
            var node = ts.createNode(291 /* Bundle */);
            node.prepends = prepends;
            node.sourceFiles = sourceFiles;
            return node;
        }
        ts.createBundle = createBundle;
        var allUnscopedEmitHelpers;
        function getAllUnscopedEmitHelpers() {
            return allUnscopedEmitHelpers || (allUnscopedEmitHelpers = ts.arrayToMap([
                ts.valuesHelper,
                ts.readHelper,
                ts.spreadHelper,
                ts.spreadArraysHelper,
                ts.restHelper,
                ts.decorateHelper,
                ts.metadataHelper,
                ts.paramHelper,
                ts.awaiterHelper,
                ts.assignHelper,
                ts.awaitHelper,
                ts.asyncGeneratorHelper,
                ts.asyncDelegator,
                ts.asyncValues,
                ts.extendsHelper,
                ts.templateObjectHelper,
                ts.generatorHelper,
                ts.importStarHelper,
                ts.importDefaultHelper,
                ts.classPrivateFieldGetHelper,
                ts.classPrivateFieldSetHelper,
            ], function (helper) { return helper.name; }));
        }
        function createUnparsedSource() {
            var node = ts.createNode(292 /* UnparsedSource */);
            node.prologues = ts.emptyArray;
            node.referencedFiles = ts.emptyArray;
            node.libReferenceDirectives = ts.emptyArray;
            node.getLineAndCharacterOfPosition = function (pos) { return ts.getLineAndCharacterOfPosition(node, pos); };
            return node;
        }
        function createUnparsedSourceFile(textOrInputFiles, mapPathOrType, mapTextOrStripInternal) {
            var node = createUnparsedSource();
            var stripInternal;
            var bundleFileInfo;
            if (!ts.isString(textOrInputFiles)) {
                ts.Debug.assert(mapPathOrType === "js" || mapPathOrType === "dts");
                node.fileName = (mapPathOrType === "js" ? textOrInputFiles.javascriptPath : textOrInputFiles.declarationPath) || "";
                node.sourceMapPath = mapPathOrType === "js" ? textOrInputFiles.javascriptMapPath : textOrInputFiles.declarationMapPath;
                Object.defineProperties(node, {
                    text: { get: function () { return mapPathOrType === "js" ? textOrInputFiles.javascriptText : textOrInputFiles.declarationText; } },
                    sourceMapText: { get: function () { return mapPathOrType === "js" ? textOrInputFiles.javascriptMapText : textOrInputFiles.declarationMapText; } },
                });
                if (textOrInputFiles.buildInfo && textOrInputFiles.buildInfo.bundle) {
                    node.oldFileOfCurrentEmit = textOrInputFiles.oldFileOfCurrentEmit;
                    ts.Debug.assert(mapTextOrStripInternal === undefined || typeof mapTextOrStripInternal === "boolean");
                    stripInternal = mapTextOrStripInternal;
                    bundleFileInfo = mapPathOrType === "js" ? textOrInputFiles.buildInfo.bundle.js : textOrInputFiles.buildInfo.bundle.dts;
                    if (node.oldFileOfCurrentEmit) {
                        parseOldFileOfCurrentEmit(node, ts.Debug.assertDefined(bundleFileInfo));
                        return node;
                    }
                }
            }
            else {
                node.fileName = "";
                node.text = textOrInputFiles;
                node.sourceMapPath = mapPathOrType;
                node.sourceMapText = mapTextOrStripInternal;
            }
            ts.Debug.assert(!node.oldFileOfCurrentEmit);
            parseUnparsedSourceFile(node, bundleFileInfo, stripInternal);
            return node;
        }
        ts.createUnparsedSourceFile = createUnparsedSourceFile;
        function parseUnparsedSourceFile(node, bundleFileInfo, stripInternal) {
            var prologues;
            var helpers;
            var referencedFiles;
            var typeReferenceDirectives;
            var libReferenceDirectives;
            var texts;
            for (var _i = 0, _a = bundleFileInfo ? bundleFileInfo.sections : ts.emptyArray; _i < _a.length; _i++) {
                var section = _a[_i];
                switch (section.kind) {
                    case "prologue" /* Prologue */:
                        (prologues || (prologues = [])).push(createUnparsedNode(section, node));
                        break;
                    case "emitHelpers" /* EmitHelpers */:
                        (helpers || (helpers = [])).push(getAllUnscopedEmitHelpers().get(section.data));
                        break;
                    case "no-default-lib" /* NoDefaultLib */:
                        node.hasNoDefaultLib = true;
                        break;
                    case "reference" /* Reference */:
                        (referencedFiles || (referencedFiles = [])).push({ pos: -1, end: -1, fileName: section.data });
                        break;
                    case "type" /* Type */:
                        (typeReferenceDirectives || (typeReferenceDirectives = [])).push(section.data);
                        break;
                    case "lib" /* Lib */:
                        (libReferenceDirectives || (libReferenceDirectives = [])).push({ pos: -1, end: -1, fileName: section.data });
                        break;
                    case "prepend" /* Prepend */:
                        var prependNode = createUnparsedNode(section, node);
                        var prependTexts = void 0;
                        for (var _b = 0, _c = section.texts; _b < _c.length; _b++) {
                            var text = _c[_b];
                            if (!stripInternal || text.kind !== "internal" /* Internal */) {
                                (prependTexts || (prependTexts = [])).push(createUnparsedNode(text, node));
                            }
                        }
                        prependNode.texts = prependTexts || ts.emptyArray;
                        (texts || (texts = [])).push(prependNode);
                        break;
                    case "internal" /* Internal */:
                        if (stripInternal) {
                            if (!texts)
                                texts = [];
                            break;
                        }
                    // falls through
                    case "text" /* Text */:
                        (texts || (texts = [])).push(createUnparsedNode(section, node));
                        break;
                    default:
                        ts.Debug.assertNever(section);
                }
            }
            node.prologues = prologues || ts.emptyArray;
            node.helpers = helpers;
            node.referencedFiles = referencedFiles || ts.emptyArray;
            node.typeReferenceDirectives = typeReferenceDirectives;
            node.libReferenceDirectives = libReferenceDirectives || ts.emptyArray;
            node.texts = texts || [createUnparsedNode({ kind: "text" /* Text */, pos: 0, end: node.text.length }, node)];
        }
        function parseOldFileOfCurrentEmit(node, bundleFileInfo) {
            ts.Debug.assert(!!node.oldFileOfCurrentEmit);
            var texts;
            var syntheticReferences;
            for (var _i = 0, _a = bundleFileInfo.sections; _i < _a.length; _i++) {
                var section = _a[_i];
                switch (section.kind) {
                    case "internal" /* Internal */:
                    case "text" /* Text */:
                        (texts || (texts = [])).push(createUnparsedNode(section, node));
                        break;
                    case "no-default-lib" /* NoDefaultLib */:
                    case "reference" /* Reference */:
                    case "type" /* Type */:
                    case "lib" /* Lib */:
                        (syntheticReferences || (syntheticReferences = [])).push(createUnparsedSyntheticReference(section, node));
                        break;
                    // Ignore
                    case "prologue" /* Prologue */:
                    case "emitHelpers" /* EmitHelpers */:
                    case "prepend" /* Prepend */:
                        break;
                    default:
                        ts.Debug.assertNever(section);
                }
            }
            node.texts = texts || ts.emptyArray;
            node.helpers = ts.map(bundleFileInfo.sources && bundleFileInfo.sources.helpers, function (name) { return getAllUnscopedEmitHelpers().get(name); });
            node.syntheticReferences = syntheticReferences;
            return node;
        }
        function mapBundleFileSectionKindToSyntaxKind(kind) {
            switch (kind) {
                case "prologue" /* Prologue */: return 285 /* UnparsedPrologue */;
                case "prepend" /* Prepend */: return 286 /* UnparsedPrepend */;
                case "internal" /* Internal */: return 288 /* UnparsedInternalText */;
                case "text" /* Text */: return 287 /* UnparsedText */;
                case "emitHelpers" /* EmitHelpers */:
                case "no-default-lib" /* NoDefaultLib */:
                case "reference" /* Reference */:
                case "type" /* Type */:
                case "lib" /* Lib */:
                    return ts.Debug.fail("BundleFileSectionKind: " + kind + " not yet mapped to SyntaxKind");
                default:
                    return ts.Debug.assertNever(kind);
            }
        }
        function createUnparsedNode(section, parent) {
            var node = ts.createNode(mapBundleFileSectionKindToSyntaxKind(section.kind), section.pos, section.end);
            node.parent = parent;
            node.data = section.data;
            return node;
        }
        function createUnparsedSyntheticReference(section, parent) {
            var node = ts.createNode(289 /* UnparsedSyntheticReference */, section.pos, section.end);
            node.parent = parent;
            node.data = section.data;
            node.section = section;
            return node;
        }
        function createInputFiles(javascriptTextOrReadFileText, declarationTextOrJavascriptPath, javascriptMapPath, javascriptMapTextOrDeclarationPath, declarationMapPath, declarationMapTextOrBuildInfoPath, javascriptPath, declarationPath, buildInfoPath, buildInfo, oldFileOfCurrentEmit) {
            var node = ts.createNode(293 /* InputFiles */);
            if (!ts.isString(javascriptTextOrReadFileText)) {
                var cache_1 = ts.createMap();
                var textGetter_1 = function (path) {
                    if (path === undefined)
                        return undefined;
                    var value = cache_1.get(path);
                    if (value === undefined) {
                        value = javascriptTextOrReadFileText(path);
                        cache_1.set(path, value !== undefined ? value : false);
                    }
                    return value !== false ? value : undefined;
                };
                var definedTextGetter_1 = function (path) {
                    var result = textGetter_1(path);
                    return result !== undefined ? result : "/* Input file " + path + " was missing */\r\n";
                };
                var buildInfo_1;
                var getAndCacheBuildInfo_1 = function (getText) {
                    if (buildInfo_1 === undefined) {
                        var result = getText();
                        buildInfo_1 = result !== undefined ? ts.getBuildInfo(result) : false;
                    }
                    return buildInfo_1 || undefined;
                };
                node.javascriptPath = declarationTextOrJavascriptPath;
                node.javascriptMapPath = javascriptMapPath;
                node.declarationPath = ts.Debug.assertDefined(javascriptMapTextOrDeclarationPath);
                node.declarationMapPath = declarationMapPath;
                node.buildInfoPath = declarationMapTextOrBuildInfoPath;
                Object.defineProperties(node, {
                    javascriptText: { get: function () { return definedTextGetter_1(declarationTextOrJavascriptPath); } },
                    javascriptMapText: { get: function () { return textGetter_1(javascriptMapPath); } },
                    declarationText: { get: function () { return definedTextGetter_1(ts.Debug.assertDefined(javascriptMapTextOrDeclarationPath)); } },
                    declarationMapText: { get: function () { return textGetter_1(declarationMapPath); } },
                    buildInfo: { get: function () { return getAndCacheBuildInfo_1(function () { return textGetter_1(declarationMapTextOrBuildInfoPath); }); } }
                });
            }
            else {
                node.javascriptText = javascriptTextOrReadFileText;
                node.javascriptMapPath = javascriptMapPath;
                node.javascriptMapText = javascriptMapTextOrDeclarationPath;
                node.declarationText = declarationTextOrJavascriptPath;
                node.declarationMapPath = declarationMapPath;
                node.declarationMapText = declarationMapTextOrBuildInfoPath;
                node.javascriptPath = javascriptPath;
                node.declarationPath = declarationPath;
                node.buildInfoPath = buildInfoPath;
                node.buildInfo = buildInfo;
                node.oldFileOfCurrentEmit = oldFileOfCurrentEmit;
            }
            return node;
        }
        ts.createInputFiles = createInputFiles;
        function updateBundle(node, sourceFiles, prepends) {
            if (prepends === void 0) {
                prepends = ts.emptyArray;
            }
            if (node.sourceFiles !== sourceFiles || node.prepends !== prepends) {
                return createBundle(sourceFiles, prepends);
            }
            return node;
        }
        ts.updateBundle = updateBundle;
        function createImmediatelyInvokedFunctionExpression(statements, param, paramValue) {
            return createCall(createFunctionExpression(
            /*modifiers*/ undefined, 
            /*asteriskToken*/ undefined, 
            /*name*/ undefined, 
            /*typeParameters*/ undefined, 
            /*parameters*/ param ? [param] : [], 
            /*type*/ undefined, createBlock(statements, /*multiLine*/ true)), 
            /*typeArguments*/ undefined, 
            /*argumentsArray*/ paramValue ? [paramValue] : []);
        }
        ts.createImmediatelyInvokedFunctionExpression = createImmediatelyInvokedFunctionExpression;
        function createImmediatelyInvokedArrowFunction(statements, param, paramValue) {
            return createCall(createArrowFunction(
            /*modifiers*/ undefined, 
            /*typeParameters*/ undefined, 
            /*parameters*/ param ? [param] : [], 
            /*type*/ undefined, 
            /*equalsGreaterThanToken*/ undefined, createBlock(statements, /*multiLine*/ true)), 
            /*typeArguments*/ undefined, 
            /*argumentsArray*/ paramValue ? [paramValue] : []);
        }
        ts.createImmediatelyInvokedArrowFunction = createImmediatelyInvokedArrowFunction;
        function createComma(left, right) {
            return createBinary(left, 27 /* CommaToken */, right);
        }
        ts.createComma = createComma;
        function createLessThan(left, right) {
            return createBinary(left, 29 /* LessThanToken */, right);
        }
        ts.createLessThan = createLessThan;
        function createAssignment(left, right) {
            return createBinary(left, 62 /* EqualsToken */, right);
        }
        ts.createAssignment = createAssignment;
        function createStrictEquality(left, right) {
            return createBinary(left, 36 /* EqualsEqualsEqualsToken */, right);
        }
        ts.createStrictEquality = createStrictEquality;
        function createStrictInequality(left, right) {
            return createBinary(left, 37 /* ExclamationEqualsEqualsToken */, right);
        }
        ts.createStrictInequality = createStrictInequality;
        function createAdd(left, right) {
            return createBinary(left, 39 /* PlusToken */, right);
        }
        ts.createAdd = createAdd;
        function createSubtract(left, right) {
            return createBinary(left, 40 /* MinusToken */, right);
        }
        ts.createSubtract = createSubtract;
        function createPostfixIncrement(operand) {
            return createPostfix(operand, 45 /* PlusPlusToken */);
        }
        ts.createPostfixIncrement = createPostfixIncrement;
        function createLogicalAnd(left, right) {
            return createBinary(left, 55 /* AmpersandAmpersandToken */, right);
        }
        ts.createLogicalAnd = createLogicalAnd;
        function createLogicalOr(left, right) {
            return createBinary(left, 56 /* BarBarToken */, right);
        }
        ts.createLogicalOr = createLogicalOr;
        function createNullishCoalesce(left, right) {
            return createBinary(left, 60 /* QuestionQuestionToken */, right);
        }
        ts.createNullishCoalesce = createNullishCoalesce;
        function createLogicalNot(operand) {
            return createPrefix(53 /* ExclamationToken */, operand);
        }
        ts.createLogicalNot = createLogicalNot;
        function createVoidZero() {
            return createVoid(createLiteral(0));
        }
        ts.createVoidZero = createVoidZero;
        function createExportDefault(expression) {
            return createExportAssignment(/*decorators*/ undefined, /*modifiers*/ undefined, /*isExportEquals*/ false, expression);
        }
        ts.createExportDefault = createExportDefault;
        function createExternalModuleExport(exportName) {
            return createExportDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, createNamedExports([createExportSpecifier(/*propertyName*/ undefined, exportName)]));
        }
        ts.createExternalModuleExport = createExternalModuleExport;
        // Utilities
        function asName(name) {
            return ts.isString(name) ? createIdentifier(name) : name;
        }
        function asExpression(value) {
            return typeof value === "string" ? createStringLiteral(value) :
                typeof value === "number" ? createNumericLiteral("" + value) :
                    typeof value === "boolean" ? value ? createTrue() : createFalse() :
                        value;
        }
        function asNodeArray(array) {
            return array ? createNodeArray(array) : undefined;
        }
        function asToken(value) {
            return typeof value === "number" ? createToken(value) : value;
        }
        function asEmbeddedStatement(statement) {
            return statement && ts.isNotEmittedStatement(statement) ? setTextRange(setOriginalNode(createEmptyStatement(), statement), statement) : statement;
        }
        /**
         * Clears any EmitNode entries from parse-tree nodes.
         * @param sourceFile A source file.
         */
        function disposeEmitNodes(sourceFile) {
            // During transformation we may need to annotate a parse tree node with transient
            // transformation properties. As parse tree nodes live longer than transformation
            // nodes, we need to make sure we reclaim any memory allocated for custom ranges
            // from these nodes to ensure we do not hold onto entire subtrees just for position
            // information. We also need to reset these nodes to a pre-transformation state
            // for incremental parsing scenarios so that we do not impact later emit.
            sourceFile = ts.getSourceFileOfNode(ts.getParseTreeNode(sourceFile));
            var emitNode = sourceFile && sourceFile.emitNode;
            var annotatedNodes = emitNode && emitNode.annotatedNodes;
            if (annotatedNodes) {
                for (var _i = 0, annotatedNodes_1 = annotatedNodes; _i < annotatedNodes_1.length; _i++) {
                    var node = annotatedNodes_1[_i];
                    node.emitNode = undefined;
                }
            }
        }
        ts.disposeEmitNodes = disposeEmitNodes;
        /**
         * Associates a node with the current transformation, initializing
         * various transient transformation properties.
         */
        /* @internal */
        function getOrCreateEmitNode(node) {
            if (!node.emitNode) {
                if (ts.isParseTreeNode(node)) {
                    // To avoid holding onto transformation artifacts, we keep track of any
                    // parse tree node we are annotating. This allows us to clean them up after
                    // all transformations have completed.
                    if (node.kind === 290 /* SourceFile */) {
                        return node.emitNode = { annotatedNodes: [node] };
                    }
                    var sourceFile = ts.getSourceFileOfNode(ts.getParseTreeNode(ts.getSourceFileOfNode(node)));
                    getOrCreateEmitNode(sourceFile).annotatedNodes.push(node);
                }
                node.emitNode = {};
            }
            return node.emitNode;
        }
        ts.getOrCreateEmitNode = getOrCreateEmitNode;
        /**
         * Sets `EmitFlags.NoComments` on a node and removes any leading and trailing synthetic comments.
         * @internal
         */
        function removeAllComments(node) {
            var emitNode = getOrCreateEmitNode(node);
            emitNode.flags |= 1536 /* NoComments */;
            emitNode.leadingComments = undefined;
            emitNode.trailingComments = undefined;
            return node;
        }
        ts.removeAllComments = removeAllComments;
        function setTextRange(range, location) {
            if (location) {
                range.pos = location.pos;
                range.end = location.end;
            }
            return range;
        }
        ts.setTextRange = setTextRange;
        /**
         * Sets flags that control emit behavior of a node.
         */
        function setEmitFlags(node, emitFlags) {
            getOrCreateEmitNode(node).flags = emitFlags;
            return node;
        }
        ts.setEmitFlags = setEmitFlags;
        /**
         * Sets flags that control emit behavior of a node.
         */
        /* @internal */
        function addEmitFlags(node, emitFlags) {
            var emitNode = getOrCreateEmitNode(node);
            emitNode.flags = emitNode.flags | emitFlags;
            return node;
        }
        ts.addEmitFlags = addEmitFlags;
        /**
         * Gets a custom text range to use when emitting source maps.
         */
        function getSourceMapRange(node) {
            var emitNode = node.emitNode;
            return (emitNode && emitNode.sourceMapRange) || node;
        }
        ts.getSourceMapRange = getSourceMapRange;
        /**
         * Sets a custom text range to use when emitting source maps.
         */
        function setSourceMapRange(node, range) {
            getOrCreateEmitNode(node).sourceMapRange = range;
            return node;
        }
        ts.setSourceMapRange = setSourceMapRange;
        var SourceMapSource;
        /**
         * Create an external source map source file reference
         */
        function createSourceMapSource(fileName, text, skipTrivia) {
            return new (SourceMapSource || (SourceMapSource = ts.objectAllocator.getSourceMapSourceConstructor()))(fileName, text, skipTrivia);
        }
        ts.createSourceMapSource = createSourceMapSource;
        /**
         * Gets the TextRange to use for source maps for a token of a node.
         */
        function getTokenSourceMapRange(node, token) {
            var emitNode = node.emitNode;
            var tokenSourceMapRanges = emitNode && emitNode.tokenSourceMapRanges;
            return tokenSourceMapRanges && tokenSourceMapRanges[token];
        }
        ts.getTokenSourceMapRange = getTokenSourceMapRange;
        /**
         * Sets the TextRange to use for source maps for a token of a node.
         */
        function setTokenSourceMapRange(node, token, range) {
            var emitNode = getOrCreateEmitNode(node);
            var tokenSourceMapRanges = emitNode.tokenSourceMapRanges || (emitNode.tokenSourceMapRanges = []);
            tokenSourceMapRanges[token] = range;
            return node;
        }
        ts.setTokenSourceMapRange = setTokenSourceMapRange;
        /**
         * Gets a custom text range to use when emitting comments.
         */
        /*@internal*/
        function getStartsOnNewLine(node) {
            var emitNode = node.emitNode;
            return emitNode && emitNode.startsOnNewLine;
        }
        ts.getStartsOnNewLine = getStartsOnNewLine;
        /**
         * Sets a custom text range to use when emitting comments.
         */
        /*@internal*/
        function setStartsOnNewLine(node, newLine) {
            getOrCreateEmitNode(node).startsOnNewLine = newLine;
            return node;
        }
        ts.setStartsOnNewLine = setStartsOnNewLine;
        /**
         * Gets a custom text range to use when emitting comments.
         */
        function getCommentRange(node) {
            var emitNode = node.emitNode;
            return (emitNode && emitNode.commentRange) || node;
        }
        ts.getCommentRange = getCommentRange;
        /**
         * Sets a custom text range to use when emitting comments.
         */
        function setCommentRange(node, range) {
            getOrCreateEmitNode(node).commentRange = range;
            return node;
        }
        ts.setCommentRange = setCommentRange;
        function getSyntheticLeadingComments(node) {
            var emitNode = node.emitNode;
            return emitNode && emitNode.leadingComments;
        }
        ts.getSyntheticLeadingComments = getSyntheticLeadingComments;
        function setSyntheticLeadingComments(node, comments) {
            getOrCreateEmitNode(node).leadingComments = comments;
            return node;
        }
        ts.setSyntheticLeadingComments = setSyntheticLeadingComments;
        function addSyntheticLeadingComment(node, kind, text, hasTrailingNewLine) {
            return setSyntheticLeadingComments(node, ts.append(getSyntheticLeadingComments(node), { kind: kind, pos: -1, end: -1, hasTrailingNewLine: hasTrailingNewLine, text: text }));
        }
        ts.addSyntheticLeadingComment = addSyntheticLeadingComment;
        function getSyntheticTrailingComments(node) {
            var emitNode = node.emitNode;
            return emitNode && emitNode.trailingComments;
        }
        ts.getSyntheticTrailingComments = getSyntheticTrailingComments;
        function setSyntheticTrailingComments(node, comments) {
            getOrCreateEmitNode(node).trailingComments = comments;
            return node;
        }
        ts.setSyntheticTrailingComments = setSyntheticTrailingComments;
        function addSyntheticTrailingComment(node, kind, text, hasTrailingNewLine) {
            return setSyntheticTrailingComments(node, ts.append(getSyntheticTrailingComments(node), { kind: kind, pos: -1, end: -1, hasTrailingNewLine: hasTrailingNewLine, text: text }));
        }
        ts.addSyntheticTrailingComment = addSyntheticTrailingComment;
        function moveSyntheticComments(node, original) {
            setSyntheticLeadingComments(node, getSyntheticLeadingComments(original));
            setSyntheticTrailingComments(node, getSyntheticTrailingComments(original));
            var emit = getOrCreateEmitNode(original);
            emit.leadingComments = undefined;
            emit.trailingComments = undefined;
            return node;
        }
        ts.moveSyntheticComments = moveSyntheticComments;
        /**
         * Gets the constant value to emit for an expression.
         */
        function getConstantValue(node) {
            var emitNode = node.emitNode;
            return emitNode && emitNode.constantValue;
        }
        ts.getConstantValue = getConstantValue;
        /**
         * Sets the constant value to emit for an expression.
         */
        function setConstantValue(node, value) {
            var emitNode = getOrCreateEmitNode(node);
            emitNode.constantValue = value;
            return node;
        }
        ts.setConstantValue = setConstantValue;
        /**
         * Adds an EmitHelper to a node.
         */
        function addEmitHelper(node, helper) {
            var emitNode = getOrCreateEmitNode(node);
            emitNode.helpers = ts.append(emitNode.helpers, helper);
            return node;
        }
        ts.addEmitHelper = addEmitHelper;
        /**
         * Add EmitHelpers to a node.
         */
        function addEmitHelpers(node, helpers) {
            if (ts.some(helpers)) {
                var emitNode = getOrCreateEmitNode(node);
                for (var _i = 0, helpers_1 = helpers; _i < helpers_1.length; _i++) {
                    var helper = helpers_1[_i];
                    emitNode.helpers = ts.appendIfUnique(emitNode.helpers, helper);
                }
            }
            return node;
        }
        ts.addEmitHelpers = addEmitHelpers;
        /**
         * Removes an EmitHelper from a node.
         */
        function removeEmitHelper(node, helper) {
            var emitNode = node.emitNode;
            if (emitNode) {
                var helpers = emitNode.helpers;
                if (helpers) {
                    return ts.orderedRemoveItem(helpers, helper);
                }
            }
            return false;
        }
        ts.removeEmitHelper = removeEmitHelper;
        /**
         * Gets the EmitHelpers of a node.
         */
        function getEmitHelpers(node) {
            var emitNode = node.emitNode;
            return emitNode && emitNode.helpers;
        }
        ts.getEmitHelpers = getEmitHelpers;
        /**
         * Moves matching emit helpers from a source node to a target node.
         */
        function moveEmitHelpers(source, target, predicate) {
            var sourceEmitNode = source.emitNode;
            var sourceEmitHelpers = sourceEmitNode && sourceEmitNode.helpers;
            if (!ts.some(sourceEmitHelpers))
                return;
            var targetEmitNode = getOrCreateEmitNode(target);
            var helpersRemoved = 0;
            for (var i = 0; i < sourceEmitHelpers.length; i++) {
                var helper = sourceEmitHelpers[i];
                if (predicate(helper)) {
                    helpersRemoved++;
                    targetEmitNode.helpers = ts.appendIfUnique(targetEmitNode.helpers, helper);
                }
                else if (helpersRemoved > 0) {
                    sourceEmitHelpers[i - helpersRemoved] = helper;
                }
            }
            if (helpersRemoved > 0) {
                sourceEmitHelpers.length -= helpersRemoved;
            }
        }
        ts.moveEmitHelpers = moveEmitHelpers;
        /* @internal */
        function compareEmitHelpers(x, y) {
            if (x === y)
                return 0 /* EqualTo */;
            if (x.priority === y.priority)
                return 0 /* EqualTo */;
            if (x.priority === undefined)
                return 1 /* GreaterThan */;
            if (y.priority === undefined)
                return -1 /* LessThan */;
            return ts.compareValues(x.priority, y.priority);
        }
        ts.compareEmitHelpers = compareEmitHelpers;
        function setOriginalNode(node, original) {
            node.original = original;
            if (original) {
                var emitNode = original.emitNode;
                if (emitNode)
                    node.emitNode = mergeEmitNode(emitNode, node.emitNode);
            }
            return node;
        }
        ts.setOriginalNode = setOriginalNode;
        function mergeEmitNode(sourceEmitNode, destEmitNode) {
            var flags = sourceEmitNode.flags, leadingComments = sourceEmitNode.leadingComments, trailingComments = sourceEmitNode.trailingComments, commentRange = sourceEmitNode.commentRange, sourceMapRange = sourceEmitNode.sourceMapRange, tokenSourceMapRanges = sourceEmitNode.tokenSourceMapRanges, constantValue = sourceEmitNode.constantValue, helpers = sourceEmitNode.helpers, startsOnNewLine = sourceEmitNode.startsOnNewLine;
            if (!destEmitNode)
                destEmitNode = {};
            // We are using `.slice()` here in case `destEmitNode.leadingComments` is pushed to later.
            if (leadingComments)
                destEmitNode.leadingComments = ts.addRange(leadingComments.slice(), destEmitNode.leadingComments);
            if (trailingComments)
                destEmitNode.trailingComments = ts.addRange(trailingComments.slice(), destEmitNode.trailingComments);
            if (flags)
                destEmitNode.flags = flags;
            if (commentRange)
                destEmitNode.commentRange = commentRange;
            if (sourceMapRange)
                destEmitNode.sourceMapRange = sourceMapRange;
            if (tokenSourceMapRanges)
                destEmitNode.tokenSourceMapRanges = mergeTokenSourceMapRanges(tokenSourceMapRanges, destEmitNode.tokenSourceMapRanges);
            if (constantValue !== undefined)
                destEmitNode.constantValue = constantValue;
            if (helpers)
                destEmitNode.helpers = ts.addRange(destEmitNode.helpers, helpers);
            if (startsOnNewLine !== undefined)
                destEmitNode.startsOnNewLine = startsOnNewLine;
            return destEmitNode;
        }
        function mergeTokenSourceMapRanges(sourceRanges, destRanges) {
            if (!destRanges)
                destRanges = [];
            for (var key in sourceRanges) {
                destRanges[key] = sourceRanges[key];
            }
            return destRanges;
        }
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        ts.nullTransformationContext = {
            enableEmitNotification: ts.noop,
            enableSubstitution: ts.noop,
            endLexicalEnvironment: ts.returnUndefined,
            getCompilerOptions: ts.notImplemented,
            getEmitHost: ts.notImplemented,
            getEmitResolver: ts.notImplemented,
            hoistFunctionDeclaration: ts.noop,
            hoistVariableDeclaration: ts.noop,
            isEmitNotificationEnabled: ts.notImplemented,
            isSubstitutionEnabled: ts.notImplemented,
            onEmitNode: ts.noop,
            onSubstituteNode: ts.notImplemented,
            readEmitHelpers: ts.notImplemented,
            requestEmitHelper: ts.noop,
            resumeLexicalEnvironment: ts.noop,
            startLexicalEnvironment: ts.noop,
            suspendLexicalEnvironment: ts.noop,
            addDiagnostic: ts.noop,
        };
        function createTypeCheck(value, tag) {
            return tag === "undefined"
                ? ts.createStrictEquality(value, ts.createVoidZero())
                : ts.createStrictEquality(ts.createTypeOf(value), ts.createLiteral(tag));
        }
        ts.createTypeCheck = createTypeCheck;
        function createMemberAccessForPropertyName(target, memberName, location) {
            if (ts.isComputedPropertyName(memberName)) {
                return ts.setTextRange(ts.createElementAccess(target, memberName.expression), location);
            }
            else {
                var expression = ts.setTextRange((ts.isIdentifier(memberName) || ts.isPrivateIdentifier(memberName))
                    ? ts.createPropertyAccess(target, memberName)
                    : ts.createElementAccess(target, memberName), memberName);
                ts.getOrCreateEmitNode(expression).flags |= 64 /* NoNestedSourceMaps */;
                return expression;
            }
        }
        ts.createMemberAccessForPropertyName = createMemberAccessForPropertyName;
        function createFunctionCall(func, thisArg, argumentsList, location) {
            return ts.setTextRange(ts.createCall(ts.createPropertyAccess(func, "call"), 
            /*typeArguments*/ undefined, __spreadArrays([
                thisArg
            ], argumentsList)), location);
        }
        ts.createFunctionCall = createFunctionCall;
        function createFunctionApply(func, thisArg, argumentsExpression, location) {
            return ts.setTextRange(ts.createCall(ts.createPropertyAccess(func, "apply"), 
            /*typeArguments*/ undefined, [
                thisArg,
                argumentsExpression
            ]), location);
        }
        ts.createFunctionApply = createFunctionApply;
        function createArraySlice(array, start) {
            var argumentsList = [];
            if (start !== undefined) {
                argumentsList.push(typeof start === "number" ? ts.createLiteral(start) : start);
            }
            return ts.createCall(ts.createPropertyAccess(array, "slice"), /*typeArguments*/ undefined, argumentsList);
        }
        ts.createArraySlice = createArraySlice;
        function createArrayConcat(array, values) {
            return ts.createCall(ts.createPropertyAccess(array, "concat"), 
            /*typeArguments*/ undefined, values);
        }
        ts.createArrayConcat = createArrayConcat;
        function createMathPow(left, right, location) {
            return ts.setTextRange(ts.createCall(ts.createPropertyAccess(ts.createIdentifier("Math"), "pow"), 
            /*typeArguments*/ undefined, [left, right]), location);
        }
        ts.createMathPow = createMathPow;
        function createReactNamespace(reactNamespace, parent) {
            // To ensure the emit resolver can properly resolve the namespace, we need to
            // treat this identifier as if it were a source tree node by clearing the `Synthesized`
            // flag and setting a parent node.
            var react = ts.createIdentifier(reactNamespace || "React");
            react.flags &= ~8 /* Synthesized */;
            // Set the parent that is in parse tree
            // this makes sure that parent chain is intact for checker to traverse complete scope tree
            react.parent = ts.getParseTreeNode(parent);
            return react;
        }
        function createJsxFactoryExpressionFromEntityName(jsxFactory, parent) {
            if (ts.isQualifiedName(jsxFactory)) {
                var left = createJsxFactoryExpressionFromEntityName(jsxFactory.left, parent);
                var right = ts.createIdentifier(ts.idText(jsxFactory.right));
                right.escapedText = jsxFactory.right.escapedText;
                return ts.createPropertyAccess(left, right);
            }
            else {
                return createReactNamespace(ts.idText(jsxFactory), parent);
            }
        }
        function createJsxFactoryExpression(jsxFactoryEntity, reactNamespace, parent) {
            return jsxFactoryEntity ?
                createJsxFactoryExpressionFromEntityName(jsxFactoryEntity, parent) :
                ts.createPropertyAccess(createReactNamespace(reactNamespace, parent), "createElement");
        }
        function createExpressionForJsxElement(jsxFactoryEntity, reactNamespace, tagName, props, children, parentElement, location) {
            var argumentsList = [tagName];
            if (props) {
                argumentsList.push(props);
            }
            if (children && children.length > 0) {
                if (!props) {
                    argumentsList.push(ts.createNull());
                }
                if (children.length > 1) {
                    for (var _i = 0, children_2 = children; _i < children_2.length; _i++) {
                        var child = children_2[_i];
                        startOnNewLine(child);
                        argumentsList.push(child);
                    }
                }
                else {
                    argumentsList.push(children[0]);
                }
            }
            return ts.setTextRange(ts.createCall(createJsxFactoryExpression(jsxFactoryEntity, reactNamespace, parentElement), 
            /*typeArguments*/ undefined, argumentsList), location);
        }
        ts.createExpressionForJsxElement = createExpressionForJsxElement;
        function createExpressionForJsxFragment(jsxFactoryEntity, reactNamespace, children, parentElement, location) {
            var tagName = ts.createPropertyAccess(createReactNamespace(reactNamespace, parentElement), "Fragment");
            var argumentsList = [tagName];
            argumentsList.push(ts.createNull());
            if (children && children.length > 0) {
                if (children.length > 1) {
                    for (var _i = 0, children_3 = children; _i < children_3.length; _i++) {
                        var child = children_3[_i];
                        startOnNewLine(child);
                        argumentsList.push(child);
                    }
                }
                else {
                    argumentsList.push(children[0]);
                }
            }
            return ts.setTextRange(ts.createCall(createJsxFactoryExpression(jsxFactoryEntity, reactNamespace, parentElement), 
            /*typeArguments*/ undefined, argumentsList), location);
        }
        ts.createExpressionForJsxFragment = createExpressionForJsxFragment;
        // Helpers
        /**
         * Gets an identifier for the name of an *unscoped* emit helper.
         */
        function getUnscopedHelperName(name) {
            return ts.setEmitFlags(ts.createIdentifier(name), 4096 /* HelperName */ | 2 /* AdviseOnEmitNode */);
        }
        ts.getUnscopedHelperName = getUnscopedHelperName;
        ts.valuesHelper = {
            name: "typescript:values",
            importName: "__values",
            scoped: false,
            text: "\n            var __values = (this && this.__values) || function(o) {\n                var s = typeof Symbol === \"function\" && Symbol.iterator, m = s && o[s], i = 0;\n                if (m) return m.call(o);\n                if (o && typeof o.length === \"number\") return {\n                    next: function () {\n                        if (o && i >= o.length) o = void 0;\n                        return { value: o && o[i++], done: !o };\n                    }\n                };\n                throw new TypeError(s ? \"Object is not iterable.\" : \"Symbol.iterator is not defined.\");\n            };"
        };
        function createValuesHelper(context, expression, location) {
            context.requestEmitHelper(ts.valuesHelper);
            return ts.setTextRange(ts.createCall(getUnscopedHelperName("__values"), 
            /*typeArguments*/ undefined, [expression]), location);
        }
        ts.createValuesHelper = createValuesHelper;
        ts.readHelper = {
            name: "typescript:read",
            importName: "__read",
            scoped: false,
            text: "\n            var __read = (this && this.__read) || function (o, n) {\n                var m = typeof Symbol === \"function\" && o[Symbol.iterator];\n                if (!m) return o;\n                var i = m.call(o), r, ar = [], e;\n                try {\n                    while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);\n                }\n                catch (error) { e = { error: error }; }\n                finally {\n                    try {\n                        if (r && !r.done && (m = i[\"return\"])) m.call(i);\n                    }\n                    finally { if (e) throw e.error; }\n                }\n                return ar;\n            };"
        };
        function createReadHelper(context, iteratorRecord, count, location) {
            context.requestEmitHelper(ts.readHelper);
            return ts.setTextRange(ts.createCall(getUnscopedHelperName("__read"), 
            /*typeArguments*/ undefined, count !== undefined
                ? [iteratorRecord, ts.createLiteral(count)]
                : [iteratorRecord]), location);
        }
        ts.createReadHelper = createReadHelper;
        ts.spreadHelper = {
            name: "typescript:spread",
            importName: "__spread",
            scoped: false,
            text: "\n            var __spread = (this && this.__spread) || function () {\n                for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));\n                return ar;\n            };"
        };
        function createSpreadHelper(context, argumentList, location) {
            context.requestEmitHelper(ts.readHelper);
            context.requestEmitHelper(ts.spreadHelper);
            return ts.setTextRange(ts.createCall(getUnscopedHelperName("__spread"), 
            /*typeArguments*/ undefined, argumentList), location);
        }
        ts.createSpreadHelper = createSpreadHelper;
        ts.spreadArraysHelper = {
            name: "typescript:spreadArrays",
            importName: "__spreadArrays",
            scoped: false,
            text: "\n            var __spreadArrays = (this && this.__spreadArrays) || function () {\n                for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;\n                for (var r = Array(s), k = 0, i = 0; i < il; i++)\n                    for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)\n                        r[k] = a[j];\n                return r;\n            };"
        };
        function createSpreadArraysHelper(context, argumentList, location) {
            context.requestEmitHelper(ts.spreadArraysHelper);
            return ts.setTextRange(ts.createCall(getUnscopedHelperName("__spreadArrays"), 
            /*typeArguments*/ undefined, argumentList), location);
        }
        ts.createSpreadArraysHelper = createSpreadArraysHelper;
        // Utilities
        function createForOfBindingStatement(node, boundValue) {
            if (ts.isVariableDeclarationList(node)) {
                var firstDeclaration = ts.first(node.declarations);
                var updatedDeclaration = ts.updateVariableDeclaration(firstDeclaration, firstDeclaration.name, 
                /*typeNode*/ undefined, boundValue);
                return ts.setTextRange(ts.createVariableStatement(
                /*modifiers*/ undefined, ts.updateVariableDeclarationList(node, [updatedDeclaration])), 
                /*location*/ node);
            }
            else {
                var updatedExpression = ts.setTextRange(ts.createAssignment(node, boundValue), /*location*/ node);
                return ts.setTextRange(ts.createStatement(updatedExpression), /*location*/ node);
            }
        }
        ts.createForOfBindingStatement = createForOfBindingStatement;
        function insertLeadingStatement(dest, source) {
            if (ts.isBlock(dest)) {
                return ts.updateBlock(dest, ts.setTextRange(ts.createNodeArray(__spreadArrays([source], dest.statements)), dest.statements));
            }
            else {
                return ts.createBlock(ts.createNodeArray([dest, source]), /*multiLine*/ true);
            }
        }
        ts.insertLeadingStatement = insertLeadingStatement;
        function restoreEnclosingLabel(node, outermostLabeledStatement, afterRestoreLabelCallback) {
            if (!outermostLabeledStatement) {
                return node;
            }
            var updated = ts.updateLabel(outermostLabeledStatement, outermostLabeledStatement.label, outermostLabeledStatement.statement.kind === 238 /* LabeledStatement */
                ? restoreEnclosingLabel(node, outermostLabeledStatement.statement)
                : node);
            if (afterRestoreLabelCallback) {
                afterRestoreLabelCallback(outermostLabeledStatement);
            }
            return updated;
        }
        ts.restoreEnclosingLabel = restoreEnclosingLabel;
        function shouldBeCapturedInTempVariable(node, cacheIdentifiers) {
            var target = ts.skipParentheses(node);
            switch (target.kind) {
                case 75 /* Identifier */:
                    return cacheIdentifiers;
                case 104 /* ThisKeyword */:
                case 8 /* NumericLiteral */:
                case 9 /* BigIntLiteral */:
                case 10 /* StringLiteral */:
                    return false;
                case 192 /* ArrayLiteralExpression */:
                    var elements = target.elements;
                    if (elements.length === 0) {
                        return false;
                    }
                    return true;
                case 193 /* ObjectLiteralExpression */:
                    return target.properties.length > 0;
                default:
                    return true;
            }
        }
        function createCallBinding(expression, recordTempVariable, languageVersion, cacheIdentifiers) {
            if (cacheIdentifiers === void 0) {
                cacheIdentifiers = false;
            }
            var callee = skipOuterExpressions(expression, 7 /* All */);
            var thisArg;
            var target;
            if (ts.isSuperProperty(callee)) {
                thisArg = ts.createThis();
                target = callee;
            }
            else if (callee.kind === 102 /* SuperKeyword */) {
                thisArg = ts.createThis();
                target = languageVersion < 2 /* ES2015 */
                    ? ts.setTextRange(ts.createIdentifier("_super"), callee)
                    : callee;
            }
            else if (ts.getEmitFlags(callee) & 4096 /* HelperName */) {
                thisArg = ts.createVoidZero();
                target = parenthesizeForAccess(callee);
            }
            else {
                switch (callee.kind) {
                    case 194 /* PropertyAccessExpression */: {
                        if (shouldBeCapturedInTempVariable(callee.expression, cacheIdentifiers)) {
                            // for `a.b()` target is `(_a = a).b` and thisArg is `_a`
                            thisArg = ts.createTempVariable(recordTempVariable);
                            target = ts.createPropertyAccess(ts.setTextRange(ts.createAssignment(thisArg, callee.expression), callee.expression), callee.name);
                            ts.setTextRange(target, callee);
                        }
                        else {
                            thisArg = callee.expression;
                            target = callee;
                        }
                        break;
                    }
                    case 195 /* ElementAccessExpression */: {
                        if (shouldBeCapturedInTempVariable(callee.expression, cacheIdentifiers)) {
                            // for `a[b]()` target is `(_a = a)[b]` and thisArg is `_a`
                            thisArg = ts.createTempVariable(recordTempVariable);
                            target = ts.createElementAccess(ts.setTextRange(ts.createAssignment(thisArg, callee.expression), callee.expression), callee.argumentExpression);
                            ts.setTextRange(target, callee);
                        }
                        else {
                            thisArg = callee.expression;
                            target = callee;
                        }
                        break;
                    }
                    default: {
                        // for `a()` target is `a` and thisArg is `void 0`
                        thisArg = ts.createVoidZero();
                        target = parenthesizeForAccess(expression);
                        break;
                    }
                }
            }
            return { target: target, thisArg: thisArg };
        }
        ts.createCallBinding = createCallBinding;
        function inlineExpressions(expressions) {
            // Avoid deeply nested comma expressions as traversing them during emit can result in "Maximum call
            // stack size exceeded" errors.
            return expressions.length > 10
                ? ts.createCommaList(expressions)
                : ts.reduceLeft(expressions, ts.createComma);
        }
        ts.inlineExpressions = inlineExpressions;
        function createExpressionFromEntityName(node) {
            if (ts.isQualifiedName(node)) {
                var left = createExpressionFromEntityName(node.left);
                var right = ts.getMutableClone(node.right);
                return ts.setTextRange(ts.createPropertyAccess(left, right), node);
            }
            else {
                return ts.getMutableClone(node);
            }
        }
        ts.createExpressionFromEntityName = createExpressionFromEntityName;
        function createExpressionForPropertyName(memberName) {
            if (ts.isIdentifier(memberName)) {
                return ts.createLiteral(memberName);
            }
            else if (ts.isComputedPropertyName(memberName)) {
                return ts.getMutableClone(memberName.expression);
            }
            else {
                return ts.getMutableClone(memberName);
            }
        }
        ts.createExpressionForPropertyName = createExpressionForPropertyName;
        function createExpressionForObjectLiteralElementLike(node, property, receiver) {
            if (property.name && ts.isPrivateIdentifier(property.name)) {
                ts.Debug.failBadSyntaxKind(property.name, "Private identifiers are not allowed in object literals.");
            }
            switch (property.kind) {
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                    return createExpressionForAccessorDeclaration(node.properties, property, receiver, !!node.multiLine);
                case 281 /* PropertyAssignment */:
                    return createExpressionForPropertyAssignment(property, receiver);
                case 282 /* ShorthandPropertyAssignment */:
                    return createExpressionForShorthandPropertyAssignment(property, receiver);
                case 161 /* MethodDeclaration */:
                    return createExpressionForMethodDeclaration(property, receiver);
            }
        }
        ts.createExpressionForObjectLiteralElementLike = createExpressionForObjectLiteralElementLike;
        function createExpressionForAccessorDeclaration(properties, property, receiver, multiLine) {
            var _a = ts.getAllAccessorDeclarations(properties, property), firstAccessor = _a.firstAccessor, getAccessor = _a.getAccessor, setAccessor = _a.setAccessor;
            if (property === firstAccessor) {
                var properties_5 = [];
                if (getAccessor) {
                    var getterFunction = ts.createFunctionExpression(getAccessor.modifiers, 
                    /*asteriskToken*/ undefined, 
                    /*name*/ undefined, 
                    /*typeParameters*/ undefined, getAccessor.parameters, 
                    /*type*/ undefined, getAccessor.body // TODO: GH#18217
                    );
                    ts.setTextRange(getterFunction, getAccessor);
                    ts.setOriginalNode(getterFunction, getAccessor);
                    var getter = ts.createPropertyAssignment("get", getterFunction);
                    properties_5.push(getter);
                }
                if (setAccessor) {
                    var setterFunction = ts.createFunctionExpression(setAccessor.modifiers, 
                    /*asteriskToken*/ undefined, 
                    /*name*/ undefined, 
                    /*typeParameters*/ undefined, setAccessor.parameters, 
                    /*type*/ undefined, setAccessor.body // TODO: GH#18217
                    );
                    ts.setTextRange(setterFunction, setAccessor);
                    ts.setOriginalNode(setterFunction, setAccessor);
                    var setter = ts.createPropertyAssignment("set", setterFunction);
                    properties_5.push(setter);
                }
                properties_5.push(ts.createPropertyAssignment("enumerable", ts.createTrue()));
                properties_5.push(ts.createPropertyAssignment("configurable", ts.createTrue()));
                var expression = ts.setTextRange(ts.createCall(ts.createPropertyAccess(ts.createIdentifier("Object"), "defineProperty"), 
                /*typeArguments*/ undefined, [
                    receiver,
                    createExpressionForPropertyName(property.name),
                    ts.createObjectLiteral(properties_5, multiLine)
                ]), 
                /*location*/ firstAccessor);
                return ts.aggregateTransformFlags(expression);
            }
            return undefined;
        }
        function createExpressionForPropertyAssignment(property, receiver) {
            return ts.aggregateTransformFlags(ts.setOriginalNode(ts.setTextRange(ts.createAssignment(createMemberAccessForPropertyName(receiver, property.name, /*location*/ property.name), property.initializer), property), property));
        }
        function createExpressionForShorthandPropertyAssignment(property, receiver) {
            return ts.aggregateTransformFlags(ts.setOriginalNode(ts.setTextRange(ts.createAssignment(createMemberAccessForPropertyName(receiver, property.name, /*location*/ property.name), ts.getSynthesizedClone(property.name)), 
            /*location*/ property), 
            /*original*/ property));
        }
        function createExpressionForMethodDeclaration(method, receiver) {
            return ts.aggregateTransformFlags(ts.setOriginalNode(ts.setTextRange(ts.createAssignment(createMemberAccessForPropertyName(receiver, method.name, /*location*/ method.name), ts.setOriginalNode(ts.setTextRange(ts.createFunctionExpression(method.modifiers, method.asteriskToken, 
            /*name*/ undefined, 
            /*typeParameters*/ undefined, method.parameters, 
            /*type*/ undefined, method.body // TODO: GH#18217
            ), 
            /*location*/ method), 
            /*original*/ method)), 
            /*location*/ method), 
            /*original*/ method));
        }
        /**
         * Gets the internal name of a declaration. This is primarily used for declarations that can be
         * referred to by name in the body of an ES5 class function body. An internal name will *never*
         * be prefixed with an module or namespace export modifier like "exports." when emitted as an
         * expression. An internal name will also *never* be renamed due to a collision with a block
         * scoped variable.
         *
         * @param node The declaration.
         * @param allowComments A value indicating whether comments may be emitted for the name.
         * @param allowSourceMaps A value indicating whether source maps may be emitted for the name.
         */
        function getInternalName(node, allowComments, allowSourceMaps) {
            return getName(node, allowComments, allowSourceMaps, 16384 /* LocalName */ | 32768 /* InternalName */);
        }
        ts.getInternalName = getInternalName;
        /**
         * Gets whether an identifier should only be referred to by its internal name.
         */
        function isInternalName(node) {
            return (ts.getEmitFlags(node) & 32768 /* InternalName */) !== 0;
        }
        ts.isInternalName = isInternalName;
        /**
         * Gets the local name of a declaration. This is primarily used for declarations that can be
         * referred to by name in the declaration's immediate scope (classes, enums, namespaces). A
         * local name will *never* be prefixed with an module or namespace export modifier like
         * "exports." when emitted as an expression.
         *
         * @param node The declaration.
         * @param allowComments A value indicating whether comments may be emitted for the name.
         * @param allowSourceMaps A value indicating whether source maps may be emitted for the name.
         */
        function getLocalName(node, allowComments, allowSourceMaps) {
            return getName(node, allowComments, allowSourceMaps, 16384 /* LocalName */);
        }
        ts.getLocalName = getLocalName;
        /**
         * Gets whether an identifier should only be referred to by its local name.
         */
        function isLocalName(node) {
            return (ts.getEmitFlags(node) & 16384 /* LocalName */) !== 0;
        }
        ts.isLocalName = isLocalName;
        /**
         * Gets the export name of a declaration. This is primarily used for declarations that can be
         * referred to by name in the declaration's immediate scope (classes, enums, namespaces). An
         * export name will *always* be prefixed with an module or namespace export modifier like
         * `"exports."` when emitted as an expression if the name points to an exported symbol.
         *
         * @param node The declaration.
         * @param allowComments A value indicating whether comments may be emitted for the name.
         * @param allowSourceMaps A value indicating whether source maps may be emitted for the name.
         */
        function getExportName(node, allowComments, allowSourceMaps) {
            return getName(node, allowComments, allowSourceMaps, 8192 /* ExportName */);
        }
        ts.getExportName = getExportName;
        /**
         * Gets whether an identifier should only be referred to by its export representation if the
         * name points to an exported symbol.
         */
        function isExportName(node) {
            return (ts.getEmitFlags(node) & 8192 /* ExportName */) !== 0;
        }
        ts.isExportName = isExportName;
        /**
         * Gets the name of a declaration for use in declarations.
         *
         * @param node The declaration.
         * @param allowComments A value indicating whether comments may be emitted for the name.
         * @param allowSourceMaps A value indicating whether source maps may be emitted for the name.
         */
        function getDeclarationName(node, allowComments, allowSourceMaps) {
            return getName(node, allowComments, allowSourceMaps);
        }
        ts.getDeclarationName = getDeclarationName;
        function getName(node, allowComments, allowSourceMaps, emitFlags) {
            if (emitFlags === void 0) {
                emitFlags = 0;
            }
            var nodeName = ts.getNameOfDeclaration(node);
            if (nodeName && ts.isIdentifier(nodeName) && !ts.isGeneratedIdentifier(nodeName)) {
                var name = ts.getMutableClone(nodeName);
                emitFlags |= ts.getEmitFlags(nodeName);
                if (!allowSourceMaps)
                    emitFlags |= 48 /* NoSourceMap */;
                if (!allowComments)
                    emitFlags |= 1536 /* NoComments */;
                if (emitFlags)
                    ts.setEmitFlags(name, emitFlags);
                return name;
            }
            return ts.getGeneratedNameForNode(node);
        }
        /**
         * Gets the exported name of a declaration for use in expressions.
         *
         * An exported name will *always* be prefixed with an module or namespace export modifier like
         * "exports." if the name points to an exported symbol.
         *
         * @param ns The namespace identifier.
         * @param node The declaration.
         * @param allowComments A value indicating whether comments may be emitted for the name.
         * @param allowSourceMaps A value indicating whether source maps may be emitted for the name.
         */
        function getExternalModuleOrNamespaceExportName(ns, node, allowComments, allowSourceMaps) {
            if (ns && ts.hasModifier(node, 1 /* Export */)) {
                return getNamespaceMemberName(ns, getName(node), allowComments, allowSourceMaps);
            }
            return getExportName(node, allowComments, allowSourceMaps);
        }
        ts.getExternalModuleOrNamespaceExportName = getExternalModuleOrNamespaceExportName;
        /**
         * Gets a namespace-qualified name for use in expressions.
         *
         * @param ns The namespace identifier.
         * @param name The name.
         * @param allowComments A value indicating whether comments may be emitted for the name.
         * @param allowSourceMaps A value indicating whether source maps may be emitted for the name.
         */
        function getNamespaceMemberName(ns, name, allowComments, allowSourceMaps) {
            var qualifiedName = ts.createPropertyAccess(ns, ts.nodeIsSynthesized(name) ? name : ts.getSynthesizedClone(name));
            ts.setTextRange(qualifiedName, name);
            var emitFlags = 0;
            if (!allowSourceMaps)
                emitFlags |= 48 /* NoSourceMap */;
            if (!allowComments)
                emitFlags |= 1536 /* NoComments */;
            if (emitFlags)
                ts.setEmitFlags(qualifiedName, emitFlags);
            return qualifiedName;
        }
        ts.getNamespaceMemberName = getNamespaceMemberName;
        function convertToFunctionBody(node, multiLine) {
            return ts.isBlock(node) ? node : ts.setTextRange(ts.createBlock([ts.setTextRange(ts.createReturn(node), node)], multiLine), node);
        }
        ts.convertToFunctionBody = convertToFunctionBody;
        function convertFunctionDeclarationToExpression(node) {
            if (!node.body)
                return ts.Debug.fail();
            var updated = ts.createFunctionExpression(node.modifiers, node.asteriskToken, node.name, node.typeParameters, node.parameters, node.type, node.body);
            ts.setOriginalNode(updated, node);
            ts.setTextRange(updated, node);
            if (ts.getStartsOnNewLine(node)) {
                ts.setStartsOnNewLine(updated, /*newLine*/ true);
            }
            ts.aggregateTransformFlags(updated);
            return updated;
        }
        ts.convertFunctionDeclarationToExpression = convertFunctionDeclarationToExpression;
        function isUseStrictPrologue(node) {
            return ts.isStringLiteral(node.expression) && node.expression.text === "use strict";
        }
        /**
         * Add any necessary prologue-directives into target statement-array.
         * The function needs to be called during each transformation step.
         * This function needs to be called whenever we transform the statement
         * list of a source file, namespace, or function-like body.
         *
         * @param target: result statements array
         * @param source: origin statements array
         * @param ensureUseStrict: boolean determining whether the function need to add prologue-directives
         * @param visitor: Optional callback used to visit any custom prologue directives.
         */
        function addPrologue(target, source, ensureUseStrict, visitor) {
            var offset = addStandardPrologue(target, source, ensureUseStrict);
            return addCustomPrologue(target, source, offset, visitor);
        }
        ts.addPrologue = addPrologue;
        /**
         * Add just the standard (string-expression) prologue-directives into target statement-array.
         * The function needs to be called during each transformation step.
         * This function needs to be called whenever we transform the statement
         * list of a source file, namespace, or function-like body.
         */
        function addStandardPrologue(target, source, ensureUseStrict) {
            ts.Debug.assert(target.length === 0, "Prologue directives should be at the first statement in the target statements array");
            var foundUseStrict = false;
            var statementOffset = 0;
            var numStatements = source.length;
            while (statementOffset < numStatements) {
                var statement = source[statementOffset];
                if (ts.isPrologueDirective(statement)) {
                    if (isUseStrictPrologue(statement)) {
                        foundUseStrict = true;
                    }
                    target.push(statement);
                }
                else {
                    break;
                }
                statementOffset++;
            }
            if (ensureUseStrict && !foundUseStrict) {
                target.push(startOnNewLine(ts.createStatement(ts.createLiteral("use strict"))));
            }
            return statementOffset;
        }
        ts.addStandardPrologue = addStandardPrologue;
        function addCustomPrologue(target, source, statementOffset, visitor) {
            var numStatements = source.length;
            while (statementOffset !== undefined && statementOffset < numStatements) {
                var statement = source[statementOffset];
                if (ts.getEmitFlags(statement) & 1048576 /* CustomPrologue */) {
                    ts.append(target, visitor ? ts.visitNode(statement, visitor, ts.isStatement) : statement);
                }
                else {
                    break;
                }
                statementOffset++;
            }
            return statementOffset;
        }
        ts.addCustomPrologue = addCustomPrologue;
        function findUseStrictPrologue(statements) {
            for (var _i = 0, statements_4 = statements; _i < statements_4.length; _i++) {
                var statement = statements_4[_i];
                if (ts.isPrologueDirective(statement)) {
                    if (isUseStrictPrologue(statement)) {
                        return statement;
                    }
                }
                else {
                    break;
                }
            }
            return undefined;
        }
        ts.findUseStrictPrologue = findUseStrictPrologue;
        function startsWithUseStrict(statements) {
            var firstStatement = ts.firstOrUndefined(statements);
            return firstStatement !== undefined
                && ts.isPrologueDirective(firstStatement)
                && isUseStrictPrologue(firstStatement);
        }
        ts.startsWithUseStrict = startsWithUseStrict;
        /**
         * Ensures "use strict" directive is added
         *
         * @param statements An array of statements
         */
        function ensureUseStrict(statements) {
            var foundUseStrict = findUseStrictPrologue(statements);
            if (!foundUseStrict) {
                return ts.setTextRange(ts.createNodeArray(__spreadArrays([
                    startOnNewLine(ts.createStatement(ts.createLiteral("use strict")))
                ], statements)), statements);
            }
            return statements;
        }
        ts.ensureUseStrict = ensureUseStrict;
        /**
         * Wraps the operand to a BinaryExpression in parentheses if they are needed to preserve the intended
         * order of operations.
         *
         * @param binaryOperator The operator for the BinaryExpression.
         * @param operand The operand for the BinaryExpression.
         * @param isLeftSideOfBinary A value indicating whether the operand is the left side of the
         *                           BinaryExpression.
         */
        function parenthesizeBinaryOperand(binaryOperator, operand, isLeftSideOfBinary, leftOperand) {
            var skipped = ts.skipPartiallyEmittedExpressions(operand);
            // If the resulting expression is already parenthesized, we do not need to do any further processing.
            if (skipped.kind === 200 /* ParenthesizedExpression */) {
                return operand;
            }
            return binaryOperandNeedsParentheses(binaryOperator, operand, isLeftSideOfBinary, leftOperand)
                ? ts.createParen(operand)
                : operand;
        }
        ts.parenthesizeBinaryOperand = parenthesizeBinaryOperand;
        /**
         * Determines whether the operand to a BinaryExpression needs to be parenthesized.
         *
         * @param binaryOperator The operator for the BinaryExpression.
         * @param operand The operand for the BinaryExpression.
         * @param isLeftSideOfBinary A value indicating whether the operand is the left side of the
         *                           BinaryExpression.
         */
        function binaryOperandNeedsParentheses(binaryOperator, operand, isLeftSideOfBinary, leftOperand) {
            // If the operand has lower precedence, then it needs to be parenthesized to preserve the
            // intent of the expression. For example, if the operand is `a + b` and the operator is
            // `*`, then we need to parenthesize the operand to preserve the intended order of
            // operations: `(a + b) * x`.
            //
            // If the operand has higher precedence, then it does not need to be parenthesized. For
            // example, if the operand is `a * b` and the operator is `+`, then we do not need to
            // parenthesize to preserve the intended order of operations: `a * b + x`.
            //
            // If the operand has the same precedence, then we need to check the associativity of
            // the operator based on whether this is the left or right operand of the expression.
            //
            // For example, if `a / d` is on the right of operator `*`, we need to parenthesize
            // to preserve the intended order of operations: `x * (a / d)`
            //
            // If `a ** d` is on the left of operator `**`, we need to parenthesize to preserve
            // the intended order of operations: `(a ** b) ** c`
            var binaryOperatorPrecedence = ts.getOperatorPrecedence(209 /* BinaryExpression */, binaryOperator);
            var binaryOperatorAssociativity = ts.getOperatorAssociativity(209 /* BinaryExpression */, binaryOperator);
            var emittedOperand = ts.skipPartiallyEmittedExpressions(operand);
            if (!isLeftSideOfBinary && operand.kind === 202 /* ArrowFunction */ && binaryOperatorPrecedence > 3) {
                // We need to parenthesize arrow functions on the right side to avoid it being
                // parsed as parenthesized expression: `a && (() => {})`
                return true;
            }
            var operandPrecedence = ts.getExpressionPrecedence(emittedOperand);
            switch (ts.compareValues(operandPrecedence, binaryOperatorPrecedence)) {
                case -1 /* LessThan */:
                    // If the operand is the right side of a right-associative binary operation
                    // and is a yield expression, then we do not need parentheses.
                    if (!isLeftSideOfBinary
                        && binaryOperatorAssociativity === 1 /* Right */
                        && operand.kind === 212 /* YieldExpression */) {
                        return false;
                    }
                    return true;
                case 1 /* GreaterThan */:
                    return false;
                case 0 /* EqualTo */:
                    if (isLeftSideOfBinary) {
                        // No need to parenthesize the left operand when the binary operator is
                        // left associative:
                        //  (a*b)/x    -> a*b/x
                        //  (a**b)/x   -> a**b/x
                        //
                        // Parentheses are needed for the left operand when the binary operator is
                        // right associative:
                        //  (a/b)**x   -> (a/b)**x
                        //  (a**b)**x  -> (a**b)**x
                        return binaryOperatorAssociativity === 1 /* Right */;
                    }
                    else {
                        if (ts.isBinaryExpression(emittedOperand)
                            && emittedOperand.operatorToken.kind === binaryOperator) {
                            // No need to parenthesize the right operand when the binary operator and
                            // operand are the same and one of the following:
                            //  x*(a*b)     => x*a*b
                            //  x|(a|b)     => x|a|b
                            //  x&(a&b)     => x&a&b
                            //  x^(a^b)     => x^a^b
                            if (operatorHasAssociativeProperty(binaryOperator)) {
                                return false;
                            }
                            // No need to parenthesize the right operand when the binary operator
                            // is plus (+) if both the left and right operands consist solely of either
                            // literals of the same kind or binary plus (+) expressions for literals of
                            // the same kind (recursively).
                            //  "a"+(1+2)       => "a"+(1+2)
                            //  "a"+("b"+"c")   => "a"+"b"+"c"
                            if (binaryOperator === 39 /* PlusToken */) {
                                var leftKind = leftOperand ? getLiteralKindOfBinaryPlusOperand(leftOperand) : 0 /* Unknown */;
                                if (ts.isLiteralKind(leftKind) && leftKind === getLiteralKindOfBinaryPlusOperand(emittedOperand)) {
                                    return false;
                                }
                            }
                        }
                        // No need to parenthesize the right operand when the operand is right
                        // associative:
                        //  x/(a**b)    -> x/a**b
                        //  x**(a**b)   -> x**a**b
                        //
                        // Parentheses are needed for the right operand when the operand is left
                        // associative:
                        //  x/(a*b)     -> x/(a*b)
                        //  x**(a/b)    -> x**(a/b)
                        var operandAssociativity = ts.getExpressionAssociativity(emittedOperand);
                        return operandAssociativity === 0 /* Left */;
                    }
            }
        }
        /**
         * Determines whether a binary operator is mathematically associative.
         *
         * @param binaryOperator The binary operator.
         */
        function operatorHasAssociativeProperty(binaryOperator) {
            // The following operators are associative in JavaScript:
            //  (a*b)*c     -> a*(b*c)  -> a*b*c
            //  (a|b)|c     -> a|(b|c)  -> a|b|c
            //  (a&b)&c     -> a&(b&c)  -> a&b&c
            //  (a^b)^c     -> a^(b^c)  -> a^b^c
            //
            // While addition is associative in mathematics, JavaScript's `+` is not
            // guaranteed to be associative as it is overloaded with string concatenation.
            return binaryOperator === 41 /* AsteriskToken */
                || binaryOperator === 51 /* BarToken */
                || binaryOperator === 50 /* AmpersandToken */
                || binaryOperator === 52 /* CaretToken */;
        }
        /**
         * This function determines whether an expression consists of a homogeneous set of
         * literal expressions or binary plus expressions that all share the same literal kind.
         * It is used to determine whether the right-hand operand of a binary plus expression can be
         * emitted without parentheses.
         */
        function getLiteralKindOfBinaryPlusOperand(node) {
            node = ts.skipPartiallyEmittedExpressions(node);
            if (ts.isLiteralKind(node.kind)) {
                return node.kind;
            }
            if (node.kind === 209 /* BinaryExpression */ && node.operatorToken.kind === 39 /* PlusToken */) {
                if (node.cachedLiteralKind !== undefined) {
                    return node.cachedLiteralKind;
                }
                var leftKind = getLiteralKindOfBinaryPlusOperand(node.left);
                var literalKind = ts.isLiteralKind(leftKind) && leftKind === getLiteralKindOfBinaryPlusOperand(node.right) ? leftKind :
                    0 /* Unknown */;
                node.cachedLiteralKind = literalKind;
                return literalKind;
            }
            return 0 /* Unknown */;
        }
        function parenthesizeForConditionalHead(condition) {
            var conditionalPrecedence = ts.getOperatorPrecedence(210 /* ConditionalExpression */, 57 /* QuestionToken */);
            var emittedCondition = ts.skipPartiallyEmittedExpressions(condition);
            var conditionPrecedence = ts.getExpressionPrecedence(emittedCondition);
            if (ts.compareValues(conditionPrecedence, conditionalPrecedence) !== 1 /* GreaterThan */) {
                return ts.createParen(condition);
            }
            return condition;
        }
        ts.parenthesizeForConditionalHead = parenthesizeForConditionalHead;
        function parenthesizeSubexpressionOfConditionalExpression(e) {
            // per ES grammar both 'whenTrue' and 'whenFalse' parts of conditional expression are assignment expressions
            // so in case when comma expression is introduced as a part of previous transformations
            // if should be wrapped in parens since comma operator has the lowest precedence
            var emittedExpression = ts.skipPartiallyEmittedExpressions(e);
            return isCommaSequence(emittedExpression)
                ? ts.createParen(e)
                : e;
        }
        ts.parenthesizeSubexpressionOfConditionalExpression = parenthesizeSubexpressionOfConditionalExpression;
        /**
         *  [Per the spec](https://tc39.github.io/ecma262/#prod-ExportDeclaration), `export default` accepts _AssigmentExpression_ but
         *  has a lookahead restriction for `function`, `async function`, and `class`.
         *
         * Basically, that means we need to parenthesize in the following cases:
         *
         * - BinaryExpression of CommaToken
         * - CommaList (synthetic list of multiple comma expressions)
         * - FunctionExpression
         * - ClassExpression
         */
        function parenthesizeDefaultExpression(e) {
            var check = ts.skipPartiallyEmittedExpressions(e);
            var needsParens = isCommaSequence(check);
            if (!needsParens) {
                switch (getLeftmostExpression(check, /*stopAtCallExpression*/ false).kind) {
                    case 214 /* ClassExpression */:
                    case 201 /* FunctionExpression */:
                        needsParens = true;
                }
            }
            return needsParens ? ts.createParen(e) : e;
        }
        ts.parenthesizeDefaultExpression = parenthesizeDefaultExpression;
        /**
         * Wraps an expression in parentheses if it is needed in order to use the expression
         * as the expression of a NewExpression node.
         *
         * @param expression The Expression node.
         */
        function parenthesizeForNew(expression) {
            var leftmostExpr = getLeftmostExpression(expression, /*stopAtCallExpressions*/ true);
            switch (leftmostExpr.kind) {
                case 196 /* CallExpression */:
                    return ts.createParen(expression);
                case 197 /* NewExpression */:
                    return !leftmostExpr.arguments
                        ? ts.createParen(expression)
                        : expression;
            }
            return parenthesizeForAccess(expression);
        }
        ts.parenthesizeForNew = parenthesizeForNew;
        /**
         * Wraps an expression in parentheses if it is needed in order to use the expression for
         * property or element access.
         *
         * @param expr The expression node.
         */
        function parenthesizeForAccess(expression) {
            // isLeftHandSideExpression is almost the correct criterion for when it is not necessary
            // to parenthesize the expression before a dot. The known exception is:
            //
            //    NewExpression:
            //       new C.x        -> not the same as (new C).x
            //
            var emittedExpression = ts.skipPartiallyEmittedExpressions(expression);
            if (ts.isLeftHandSideExpression(emittedExpression)
                && (emittedExpression.kind !== 197 /* NewExpression */ || emittedExpression.arguments)) {
                return expression;
            }
            return ts.setTextRange(ts.createParen(expression), expression);
        }
        ts.parenthesizeForAccess = parenthesizeForAccess;
        function parenthesizePostfixOperand(operand) {
            return ts.isLeftHandSideExpression(operand)
                ? operand
                : ts.setTextRange(ts.createParen(operand), operand);
        }
        ts.parenthesizePostfixOperand = parenthesizePostfixOperand;
        function parenthesizePrefixOperand(operand) {
            return ts.isUnaryExpression(operand)
                ? operand
                : ts.setTextRange(ts.createParen(operand), operand);
        }
        ts.parenthesizePrefixOperand = parenthesizePrefixOperand;
        function parenthesizeListElements(elements) {
            var result;
            for (var i = 0; i < elements.length; i++) {
                var element = parenthesizeExpressionForList(elements[i]);
                if (result !== undefined || element !== elements[i]) {
                    if (result === undefined) {
                        result = elements.slice(0, i);
                    }
                    result.push(element);
                }
            }
            if (result !== undefined) {
                return ts.setTextRange(ts.createNodeArray(result, elements.hasTrailingComma), elements);
            }
            return elements;
        }
        ts.parenthesizeListElements = parenthesizeListElements;
        function parenthesizeExpressionForList(expression) {
            var emittedExpression = ts.skipPartiallyEmittedExpressions(expression);
            var expressionPrecedence = ts.getExpressionPrecedence(emittedExpression);
            var commaPrecedence = ts.getOperatorPrecedence(209 /* BinaryExpression */, 27 /* CommaToken */);
            return expressionPrecedence > commaPrecedence
                ? expression
                : ts.setTextRange(ts.createParen(expression), expression);
        }
        ts.parenthesizeExpressionForList = parenthesizeExpressionForList;
        function parenthesizeExpressionForExpressionStatement(expression) {
            var emittedExpression = ts.skipPartiallyEmittedExpressions(expression);
            if (ts.isCallExpression(emittedExpression)) {
                var callee = emittedExpression.expression;
                var kind = ts.skipPartiallyEmittedExpressions(callee).kind;
                if (kind === 201 /* FunctionExpression */ || kind === 202 /* ArrowFunction */) {
                    var mutableCall = ts.getMutableClone(emittedExpression);
                    mutableCall.expression = ts.setTextRange(ts.createParen(callee), callee);
                    return recreateOuterExpressions(expression, mutableCall, 4 /* PartiallyEmittedExpressions */);
                }
            }
            var leftmostExpressionKind = getLeftmostExpression(emittedExpression, /*stopAtCallExpressions*/ false).kind;
            if (leftmostExpressionKind === 193 /* ObjectLiteralExpression */ || leftmostExpressionKind === 201 /* FunctionExpression */) {
                return ts.setTextRange(ts.createParen(expression), expression);
            }
            return expression;
        }
        ts.parenthesizeExpressionForExpressionStatement = parenthesizeExpressionForExpressionStatement;
        function parenthesizeConditionalTypeMember(member) {
            return member.kind === 180 /* ConditionalType */ ? ts.createParenthesizedType(member) : member;
        }
        ts.parenthesizeConditionalTypeMember = parenthesizeConditionalTypeMember;
        function parenthesizeElementTypeMember(member) {
            switch (member.kind) {
                case 178 /* UnionType */:
                case 179 /* IntersectionType */:
                case 170 /* FunctionType */:
                case 171 /* ConstructorType */:
                    return ts.createParenthesizedType(member);
            }
            return parenthesizeConditionalTypeMember(member);
        }
        ts.parenthesizeElementTypeMember = parenthesizeElementTypeMember;
        function parenthesizeArrayTypeMember(member) {
            switch (member.kind) {
                case 172 /* TypeQuery */:
                case 184 /* TypeOperator */:
                case 181 /* InferType */:
                    return ts.createParenthesizedType(member);
            }
            return parenthesizeElementTypeMember(member);
        }
        ts.parenthesizeArrayTypeMember = parenthesizeArrayTypeMember;
        function parenthesizeElementTypeMembers(members) {
            return ts.createNodeArray(ts.sameMap(members, parenthesizeElementTypeMember));
        }
        ts.parenthesizeElementTypeMembers = parenthesizeElementTypeMembers;
        function parenthesizeTypeParameters(typeParameters) {
            if (ts.some(typeParameters)) {
                var params = [];
                for (var i = 0; i < typeParameters.length; ++i) {
                    var entry = typeParameters[i];
                    params.push(i === 0 && ts.isFunctionOrConstructorTypeNode(entry) && entry.typeParameters ?
                        ts.createParenthesizedType(entry) :
                        entry);
                }
                return ts.createNodeArray(params);
            }
        }
        ts.parenthesizeTypeParameters = parenthesizeTypeParameters;
        function getLeftmostExpression(node, stopAtCallExpressions) {
            while (true) {
                switch (node.kind) {
                    case 208 /* PostfixUnaryExpression */:
                        node = node.operand;
                        continue;
                    case 209 /* BinaryExpression */:
                        node = node.left;
                        continue;
                    case 210 /* ConditionalExpression */:
                        node = node.condition;
                        continue;
                    case 198 /* TaggedTemplateExpression */:
                        node = node.tag;
                        continue;
                    case 196 /* CallExpression */:
                        if (stopAtCallExpressions) {
                            return node;
                        }
                    // falls through
                    case 217 /* AsExpression */:
                    case 195 /* ElementAccessExpression */:
                    case 194 /* PropertyAccessExpression */:
                    case 218 /* NonNullExpression */:
                    case 325 /* PartiallyEmittedExpression */:
                        node = node.expression;
                        continue;
                }
                return node;
            }
        }
        ts.getLeftmostExpression = getLeftmostExpression;
        function parenthesizeConciseBody(body) {
            if (!ts.isBlock(body) && (isCommaSequence(body) || getLeftmostExpression(body, /*stopAtCallExpressions*/ false).kind === 193 /* ObjectLiteralExpression */)) {
                return ts.setTextRange(ts.createParen(body), body);
            }
            return body;
        }
        ts.parenthesizeConciseBody = parenthesizeConciseBody;
        function isCommaSequence(node) {
            return node.kind === 209 /* BinaryExpression */ && node.operatorToken.kind === 27 /* CommaToken */ ||
                node.kind === 326 /* CommaListExpression */;
        }
        ts.isCommaSequence = isCommaSequence;
        var OuterExpressionKinds;
        (function (OuterExpressionKinds) {
            OuterExpressionKinds[OuterExpressionKinds["Parentheses"] = 1] = "Parentheses";
            OuterExpressionKinds[OuterExpressionKinds["Assertions"] = 2] = "Assertions";
            OuterExpressionKinds[OuterExpressionKinds["PartiallyEmittedExpressions"] = 4] = "PartiallyEmittedExpressions";
            OuterExpressionKinds[OuterExpressionKinds["All"] = 7] = "All";
        })(OuterExpressionKinds = ts.OuterExpressionKinds || (ts.OuterExpressionKinds = {}));
        function isOuterExpression(node, kinds) {
            if (kinds === void 0) {
                kinds = 7 /* All */;
            }
            switch (node.kind) {
                case 200 /* ParenthesizedExpression */:
                    return (kinds & 1 /* Parentheses */) !== 0;
                case 199 /* TypeAssertionExpression */:
                case 217 /* AsExpression */:
                case 218 /* NonNullExpression */:
                    return (kinds & 2 /* Assertions */) !== 0;
                case 325 /* PartiallyEmittedExpression */:
                    return (kinds & 4 /* PartiallyEmittedExpressions */) !== 0;
            }
            return false;
        }
        ts.isOuterExpression = isOuterExpression;
        function skipOuterExpressions(node, kinds) {
            if (kinds === void 0) {
                kinds = 7 /* All */;
            }
            var previousNode;
            do {
                previousNode = node;
                if (kinds & 1 /* Parentheses */) {
                    node = ts.skipParentheses(node);
                }
                if (kinds & 2 /* Assertions */) {
                    node = skipAssertions(node);
                }
                if (kinds & 4 /* PartiallyEmittedExpressions */) {
                    node = ts.skipPartiallyEmittedExpressions(node);
                }
            } while (previousNode !== node);
            return node;
        }
        ts.skipOuterExpressions = skipOuterExpressions;
        function skipAssertions(node) {
            while (ts.isAssertionExpression(node) || node.kind === 218 /* NonNullExpression */) {
                node = node.expression;
            }
            return node;
        }
        ts.skipAssertions = skipAssertions;
        function updateOuterExpression(outerExpression, expression) {
            switch (outerExpression.kind) {
                case 200 /* ParenthesizedExpression */: return ts.updateParen(outerExpression, expression);
                case 199 /* TypeAssertionExpression */: return ts.updateTypeAssertion(outerExpression, outerExpression.type, expression);
                case 217 /* AsExpression */: return ts.updateAsExpression(outerExpression, expression, outerExpression.type);
                case 218 /* NonNullExpression */: return ts.updateNonNullExpression(outerExpression, expression);
                case 325 /* PartiallyEmittedExpression */: return ts.updatePartiallyEmittedExpression(outerExpression, expression);
            }
        }
        /**
         * Determines whether a node is a parenthesized expression that can be ignored when recreating outer expressions.
         *
         * A parenthesized expression can be ignored when all of the following are true:
         *
         * - It's `pos` and `end` are not -1
         * - It does not have a custom source map range
         * - It does not have a custom comment range
         * - It does not have synthetic leading or trailing comments
         *
         * If an outermost parenthesized expression is ignored, but the containing expression requires a parentheses around
         * the expression to maintain precedence, a new parenthesized expression should be created automatically when
         * the containing expression is created/updated.
         */
        function isIgnorableParen(node) {
            return node.kind === 200 /* ParenthesizedExpression */
                && ts.nodeIsSynthesized(node)
                && ts.nodeIsSynthesized(ts.getSourceMapRange(node))
                && ts.nodeIsSynthesized(ts.getCommentRange(node))
                && !ts.some(ts.getSyntheticLeadingComments(node))
                && !ts.some(ts.getSyntheticTrailingComments(node));
        }
        function recreateOuterExpressions(outerExpression, innerExpression, kinds) {
            if (kinds === void 0) {
                kinds = 7 /* All */;
            }
            if (outerExpression && isOuterExpression(outerExpression, kinds) && !isIgnorableParen(outerExpression)) {
                return updateOuterExpression(outerExpression, recreateOuterExpressions(outerExpression.expression, innerExpression));
            }
            return innerExpression;
        }
        ts.recreateOuterExpressions = recreateOuterExpressions;
        function startOnNewLine(node) {
            return ts.setStartsOnNewLine(node, /*newLine*/ true);
        }
        ts.startOnNewLine = startOnNewLine;
        function getExternalHelpersModuleName(node) {
            var parseNode = ts.getOriginalNode(node, ts.isSourceFile);
            var emitNode = parseNode && parseNode.emitNode;
            return emitNode && emitNode.externalHelpersModuleName;
        }
        ts.getExternalHelpersModuleName = getExternalHelpersModuleName;
        function hasRecordedExternalHelpers(sourceFile) {
            var parseNode = ts.getOriginalNode(sourceFile, ts.isSourceFile);
            var emitNode = parseNode && parseNode.emitNode;
            return !!emitNode && (!!emitNode.externalHelpersModuleName || !!emitNode.externalHelpers);
        }
        ts.hasRecordedExternalHelpers = hasRecordedExternalHelpers;
        function createExternalHelpersImportDeclarationIfNeeded(sourceFile, compilerOptions, hasExportStarsToExportValues, hasImportStar, hasImportDefault) {
            if (compilerOptions.importHelpers && ts.isEffectiveExternalModule(sourceFile, compilerOptions)) {
                var namedBindings = void 0;
                var moduleKind = ts.getEmitModuleKind(compilerOptions);
                if (moduleKind >= ts.ModuleKind.ES2015 && moduleKind <= ts.ModuleKind.ESNext) {
                    // use named imports
                    var helpers = ts.getEmitHelpers(sourceFile);
                    if (helpers) {
                        var helperNames = [];
                        for (var _i = 0, helpers_2 = helpers; _i < helpers_2.length; _i++) {
                            var helper = helpers_2[_i];
                            if (!helper.scoped) {
                                var importName = helper.importName;
                                if (importName) {
                                    ts.pushIfUnique(helperNames, importName);
                                }
                            }
                        }
                        if (ts.some(helperNames)) {
                            helperNames.sort(ts.compareStringsCaseSensitive);
                            // Alias the imports if the names are used somewhere in the file.
                            // NOTE: We don't need to care about global import collisions as this is a module.
                            namedBindings = ts.createNamedImports(ts.map(helperNames, function (name) {
                                return ts.isFileLevelUniqueName(sourceFile, name)
                                    ? ts.createImportSpecifier(/*propertyName*/ undefined, ts.createIdentifier(name))
                                    : ts.createImportSpecifier(ts.createIdentifier(name), getUnscopedHelperName(name));
                            }));
                            var parseNode = ts.getOriginalNode(sourceFile, ts.isSourceFile);
                            var emitNode = ts.getOrCreateEmitNode(parseNode);
                            emitNode.externalHelpers = true;
                        }
                    }
                }
                else {
                    // use a namespace import
                    var externalHelpersModuleName = getOrCreateExternalHelpersModuleNameIfNeeded(sourceFile, compilerOptions, hasExportStarsToExportValues, hasImportStar || hasImportDefault);
                    if (externalHelpersModuleName) {
                        namedBindings = ts.createNamespaceImport(externalHelpersModuleName);
                    }
                }
                if (namedBindings) {
                    var externalHelpersImportDeclaration = ts.createImportDeclaration(
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, ts.createImportClause(/*name*/ undefined, namedBindings), ts.createLiteral(ts.externalHelpersModuleNameText));
                    ts.addEmitFlags(externalHelpersImportDeclaration, 67108864 /* NeverApplyImportHelper */);
                    return externalHelpersImportDeclaration;
                }
            }
        }
        ts.createExternalHelpersImportDeclarationIfNeeded = createExternalHelpersImportDeclarationIfNeeded;
        function getOrCreateExternalHelpersModuleNameIfNeeded(node, compilerOptions, hasExportStarsToExportValues, hasImportStarOrImportDefault) {
            if (compilerOptions.importHelpers && ts.isEffectiveExternalModule(node, compilerOptions)) {
                var externalHelpersModuleName = getExternalHelpersModuleName(node);
                if (externalHelpersModuleName) {
                    return externalHelpersModuleName;
                }
                var moduleKind = ts.getEmitModuleKind(compilerOptions);
                var create = (hasExportStarsToExportValues || (compilerOptions.esModuleInterop && hasImportStarOrImportDefault))
                    && moduleKind !== ts.ModuleKind.System
                    && moduleKind < ts.ModuleKind.ES2015;
                if (!create) {
                    var helpers = ts.getEmitHelpers(node);
                    if (helpers) {
                        for (var _i = 0, helpers_3 = helpers; _i < helpers_3.length; _i++) {
                            var helper = helpers_3[_i];
                            if (!helper.scoped) {
                                create = true;
                                break;
                            }
                        }
                    }
                }
                if (create) {
                    var parseNode = ts.getOriginalNode(node, ts.isSourceFile);
                    var emitNode = ts.getOrCreateEmitNode(parseNode);
                    return emitNode.externalHelpersModuleName || (emitNode.externalHelpersModuleName = ts.createUniqueName(ts.externalHelpersModuleNameText));
                }
            }
        }
        ts.getOrCreateExternalHelpersModuleNameIfNeeded = getOrCreateExternalHelpersModuleNameIfNeeded;
        /**
         * Get the name of that target module from an import or export declaration
         */
        function getLocalNameForExternalImport(node, sourceFile) {
            var namespaceDeclaration = ts.getNamespaceDeclarationNode(node);
            if (namespaceDeclaration && !ts.isDefaultImport(node)) {
                var name = namespaceDeclaration.name;
                return ts.isGeneratedIdentifier(name) ? name : ts.createIdentifier(ts.getSourceTextOfNodeFromSourceFile(sourceFile, name) || ts.idText(name));
            }
            if (node.kind === 254 /* ImportDeclaration */ && node.importClause) {
                return ts.getGeneratedNameForNode(node);
            }
            if (node.kind === 260 /* ExportDeclaration */ && node.moduleSpecifier) {
                return ts.getGeneratedNameForNode(node);
            }
            return undefined;
        }
        ts.getLocalNameForExternalImport = getLocalNameForExternalImport;
        /**
         * Get the name of a target module from an import/export declaration as should be written in the emitted output.
         * The emitted output name can be different from the input if:
         *  1. The module has a /// <amd-module name="<new name>" />
         *  2. --out or --outFile is used, making the name relative to the rootDir
         *  3- The containing SourceFile has an entry in renamedDependencies for the import as requested by some module loaders (e.g. System).
         * Otherwise, a new StringLiteral node representing the module name will be returned.
         */
        function getExternalModuleNameLiteral(importNode, sourceFile, host, resolver, compilerOptions) {
            var moduleName = ts.getExternalModuleName(importNode); // TODO: GH#18217
            if (moduleName.kind === 10 /* StringLiteral */) {
                return tryGetModuleNameFromDeclaration(importNode, host, resolver, compilerOptions)
                    || tryRenameExternalModule(moduleName, sourceFile)
                    || ts.getSynthesizedClone(moduleName);
            }
            return undefined;
        }
        ts.getExternalModuleNameLiteral = getExternalModuleNameLiteral;
        /**
         * Some bundlers (SystemJS builder) sometimes want to rename dependencies.
         * Here we check if alternative name was provided for a given moduleName and return it if possible.
         */
        function tryRenameExternalModule(moduleName, sourceFile) {
            var rename = sourceFile.renamedDependencies && sourceFile.renamedDependencies.get(moduleName.text);
            return rename && ts.createLiteral(rename);
        }
        /**
         * Get the name of a module as should be written in the emitted output.
         * The emitted output name can be different from the input if:
         *  1. The module has a /// <amd-module name="<new name>" />
         *  2. --out or --outFile is used, making the name relative to the rootDir
         * Otherwise, a new StringLiteral node representing the module name will be returned.
         */
        function tryGetModuleNameFromFile(file, host, options) {
            if (!file) {
                return undefined;
            }
            if (file.moduleName) {
                return ts.createLiteral(file.moduleName);
            }
            if (!file.isDeclarationFile && (options.out || options.outFile)) {
                return ts.createLiteral(ts.getExternalModuleNameFromPath(host, file.fileName));
            }
            return undefined;
        }
        ts.tryGetModuleNameFromFile = tryGetModuleNameFromFile;
        function tryGetModuleNameFromDeclaration(declaration, host, resolver, compilerOptions) {
            return tryGetModuleNameFromFile(resolver.getExternalModuleFileFromDeclaration(declaration), host, compilerOptions);
        }
        /**
         * Gets the initializer of an BindingOrAssignmentElement.
         */
        function getInitializerOfBindingOrAssignmentElement(bindingElement) {
            if (ts.isDeclarationBindingElement(bindingElement)) {
                // `1` in `let { a = 1 } = ...`
                // `1` in `let { a: b = 1 } = ...`
                // `1` in `let { a: {b} = 1 } = ...`
                // `1` in `let { a: [b] = 1 } = ...`
                // `1` in `let [a = 1] = ...`
                // `1` in `let [{a} = 1] = ...`
                // `1` in `let [[a] = 1] = ...`
                return bindingElement.initializer;
            }
            if (ts.isPropertyAssignment(bindingElement)) {
                // `1` in `({ a: b = 1 } = ...)`
                // `1` in `({ a: {b} = 1 } = ...)`
                // `1` in `({ a: [b] = 1 } = ...)`
                var initializer = bindingElement.initializer;
                return ts.isAssignmentExpression(initializer, /*excludeCompoundAssignment*/ true)
                    ? initializer.right
                    : undefined;
            }
            if (ts.isShorthandPropertyAssignment(bindingElement)) {
                // `1` in `({ a = 1 } = ...)`
                return bindingElement.objectAssignmentInitializer;
            }
            if (ts.isAssignmentExpression(bindingElement, /*excludeCompoundAssignment*/ true)) {
                // `1` in `[a = 1] = ...`
                // `1` in `[{a} = 1] = ...`
                // `1` in `[[a] = 1] = ...`
                return bindingElement.right;
            }
            if (ts.isSpreadElement(bindingElement)) {
                // Recovery consistent with existing emit.
                return getInitializerOfBindingOrAssignmentElement(bindingElement.expression);
            }
        }
        ts.getInitializerOfBindingOrAssignmentElement = getInitializerOfBindingOrAssignmentElement;
        /**
         * Gets the name of an BindingOrAssignmentElement.
         */
        function getTargetOfBindingOrAssignmentElement(bindingElement) {
            if (ts.isDeclarationBindingElement(bindingElement)) {
                // `a` in `let { a } = ...`
                // `a` in `let { a = 1 } = ...`
                // `b` in `let { a: b } = ...`
                // `b` in `let { a: b = 1 } = ...`
                // `a` in `let { ...a } = ...`
                // `{b}` in `let { a: {b} } = ...`
                // `{b}` in `let { a: {b} = 1 } = ...`
                // `[b]` in `let { a: [b] } = ...`
                // `[b]` in `let { a: [b] = 1 } = ...`
                // `a` in `let [a] = ...`
                // `a` in `let [a = 1] = ...`
                // `a` in `let [...a] = ...`
                // `{a}` in `let [{a}] = ...`
                // `{a}` in `let [{a} = 1] = ...`
                // `[a]` in `let [[a]] = ...`
                // `[a]` in `let [[a] = 1] = ...`
                return bindingElement.name;
            }
            if (ts.isObjectLiteralElementLike(bindingElement)) {
                switch (bindingElement.kind) {
                    case 281 /* PropertyAssignment */:
                        // `b` in `({ a: b } = ...)`
                        // `b` in `({ a: b = 1 } = ...)`
                        // `{b}` in `({ a: {b} } = ...)`
                        // `{b}` in `({ a: {b} = 1 } = ...)`
                        // `[b]` in `({ a: [b] } = ...)`
                        // `[b]` in `({ a: [b] = 1 } = ...)`
                        // `b.c` in `({ a: b.c } = ...)`
                        // `b.c` in `({ a: b.c = 1 } = ...)`
                        // `b[0]` in `({ a: b[0] } = ...)`
                        // `b[0]` in `({ a: b[0] = 1 } = ...)`
                        return getTargetOfBindingOrAssignmentElement(bindingElement.initializer);
                    case 282 /* ShorthandPropertyAssignment */:
                        // `a` in `({ a } = ...)`
                        // `a` in `({ a = 1 } = ...)`
                        return bindingElement.name;
                    case 283 /* SpreadAssignment */:
                        // `a` in `({ ...a } = ...)`
                        return getTargetOfBindingOrAssignmentElement(bindingElement.expression);
                }
                // no target
                return undefined;
            }
            if (ts.isAssignmentExpression(bindingElement, /*excludeCompoundAssignment*/ true)) {
                // `a` in `[a = 1] = ...`
                // `{a}` in `[{a} = 1] = ...`
                // `[a]` in `[[a] = 1] = ...`
                // `a.b` in `[a.b = 1] = ...`
                // `a[0]` in `[a[0] = 1] = ...`
                return getTargetOfBindingOrAssignmentElement(bindingElement.left);
            }
            if (ts.isSpreadElement(bindingElement)) {
                // `a` in `[...a] = ...`
                return getTargetOfBindingOrAssignmentElement(bindingElement.expression);
            }
            // `a` in `[a] = ...`
            // `{a}` in `[{a}] = ...`
            // `[a]` in `[[a]] = ...`
            // `a.b` in `[a.b] = ...`
            // `a[0]` in `[a[0]] = ...`
            return bindingElement;
        }
        ts.getTargetOfBindingOrAssignmentElement = getTargetOfBindingOrAssignmentElement;
        /**
         * Determines whether an BindingOrAssignmentElement is a rest element.
         */
        function getRestIndicatorOfBindingOrAssignmentElement(bindingElement) {
            switch (bindingElement.kind) {
                case 156 /* Parameter */:
                case 191 /* BindingElement */:
                    // `...` in `let [...a] = ...`
                    return bindingElement.dotDotDotToken;
                case 213 /* SpreadElement */:
                case 283 /* SpreadAssignment */:
                    // `...` in `[...a] = ...`
                    return bindingElement;
            }
            return undefined;
        }
        ts.getRestIndicatorOfBindingOrAssignmentElement = getRestIndicatorOfBindingOrAssignmentElement;
        /**
         * Gets the property name of a BindingOrAssignmentElement
         */
        function getPropertyNameOfBindingOrAssignmentElement(bindingElement) {
            var propertyName = tryGetPropertyNameOfBindingOrAssignmentElement(bindingElement);
            ts.Debug.assert(!!propertyName || ts.isSpreadAssignment(bindingElement), "Invalid property name for binding element.");
            return propertyName;
        }
        ts.getPropertyNameOfBindingOrAssignmentElement = getPropertyNameOfBindingOrAssignmentElement;
        function tryGetPropertyNameOfBindingOrAssignmentElement(bindingElement) {
            switch (bindingElement.kind) {
                case 191 /* BindingElement */:
                    // `a` in `let { a: b } = ...`
                    // `[a]` in `let { [a]: b } = ...`
                    // `"a"` in `let { "a": b } = ...`
                    // `1` in `let { 1: b } = ...`
                    if (bindingElement.propertyName) {
                        var propertyName = bindingElement.propertyName;
                        if (ts.isPrivateIdentifier(propertyName)) {
                            return ts.Debug.failBadSyntaxKind(propertyName);
                        }
                        return ts.isComputedPropertyName(propertyName) && isStringOrNumericLiteral(propertyName.expression)
                            ? propertyName.expression
                            : propertyName;
                    }
                    break;
                case 281 /* PropertyAssignment */:
                    // `a` in `({ a: b } = ...)`
                    // `[a]` in `({ [a]: b } = ...)`
                    // `"a"` in `({ "a": b } = ...)`
                    // `1` in `({ 1: b } = ...)`
                    if (bindingElement.name) {
                        var propertyName = bindingElement.name;
                        if (ts.isPrivateIdentifier(propertyName)) {
                            return ts.Debug.failBadSyntaxKind(propertyName);
                        }
                        return ts.isComputedPropertyName(propertyName) && isStringOrNumericLiteral(propertyName.expression)
                            ? propertyName.expression
                            : propertyName;
                    }
                    break;
                case 283 /* SpreadAssignment */:
                    // `a` in `({ ...a } = ...)`
                    if (bindingElement.name && ts.isPrivateIdentifier(bindingElement.name)) {
                        return ts.Debug.failBadSyntaxKind(bindingElement.name);
                    }
                    return bindingElement.name;
            }
            var target = getTargetOfBindingOrAssignmentElement(bindingElement);
            if (target && ts.isPropertyName(target)) {
                return ts.isComputedPropertyName(target) && isStringOrNumericLiteral(target.expression)
                    ? target.expression
                    : target;
            }
        }
        ts.tryGetPropertyNameOfBindingOrAssignmentElement = tryGetPropertyNameOfBindingOrAssignmentElement;
        function isStringOrNumericLiteral(node) {
            var kind = node.kind;
            return kind === 10 /* StringLiteral */
                || kind === 8 /* NumericLiteral */;
        }
        /**
         * Gets the elements of a BindingOrAssignmentPattern
         */
        function getElementsOfBindingOrAssignmentPattern(name) {
            switch (name.kind) {
                case 189 /* ObjectBindingPattern */:
                case 190 /* ArrayBindingPattern */:
                case 192 /* ArrayLiteralExpression */:
                    // `a` in `{a}`
                    // `a` in `[a]`
                    return name.elements;
                case 193 /* ObjectLiteralExpression */:
                    // `a` in `{a}`
                    return name.properties;
            }
        }
        ts.getElementsOfBindingOrAssignmentPattern = getElementsOfBindingOrAssignmentPattern;
        function convertToArrayAssignmentElement(element) {
            if (ts.isBindingElement(element)) {
                if (element.dotDotDotToken) {
                    ts.Debug.assertNode(element.name, ts.isIdentifier);
                    return ts.setOriginalNode(ts.setTextRange(ts.createSpread(element.name), element), element);
                }
                var expression = convertToAssignmentElementTarget(element.name);
                return element.initializer
                    ? ts.setOriginalNode(ts.setTextRange(ts.createAssignment(expression, element.initializer), element), element)
                    : expression;
            }
            ts.Debug.assertNode(element, ts.isExpression);
            return element;
        }
        ts.convertToArrayAssignmentElement = convertToArrayAssignmentElement;
        function convertToObjectAssignmentElement(element) {
            if (ts.isBindingElement(element)) {
                if (element.dotDotDotToken) {
                    ts.Debug.assertNode(element.name, ts.isIdentifier);
                    return ts.setOriginalNode(ts.setTextRange(ts.createSpreadAssignment(element.name), element), element);
                }
                if (element.propertyName) {
                    var expression = convertToAssignmentElementTarget(element.name);
                    return ts.setOriginalNode(ts.setTextRange(ts.createPropertyAssignment(element.propertyName, element.initializer ? ts.createAssignment(expression, element.initializer) : expression), element), element);
                }
                ts.Debug.assertNode(element.name, ts.isIdentifier);
                return ts.setOriginalNode(ts.setTextRange(ts.createShorthandPropertyAssignment(element.name, element.initializer), element), element);
            }
            ts.Debug.assertNode(element, ts.isObjectLiteralElementLike);
            return element;
        }
        ts.convertToObjectAssignmentElement = convertToObjectAssignmentElement;
        function convertToAssignmentPattern(node) {
            switch (node.kind) {
                case 190 /* ArrayBindingPattern */:
                case 192 /* ArrayLiteralExpression */:
                    return convertToArrayAssignmentPattern(node);
                case 189 /* ObjectBindingPattern */:
                case 193 /* ObjectLiteralExpression */:
                    return convertToObjectAssignmentPattern(node);
            }
        }
        ts.convertToAssignmentPattern = convertToAssignmentPattern;
        function convertToObjectAssignmentPattern(node) {
            if (ts.isObjectBindingPattern(node)) {
                return ts.setOriginalNode(ts.setTextRange(ts.createObjectLiteral(ts.map(node.elements, convertToObjectAssignmentElement)), node), node);
            }
            ts.Debug.assertNode(node, ts.isObjectLiteralExpression);
            return node;
        }
        ts.convertToObjectAssignmentPattern = convertToObjectAssignmentPattern;
        function convertToArrayAssignmentPattern(node) {
            if (ts.isArrayBindingPattern(node)) {
                return ts.setOriginalNode(ts.setTextRange(ts.createArrayLiteral(ts.map(node.elements, convertToArrayAssignmentElement)), node), node);
            }
            ts.Debug.assertNode(node, ts.isArrayLiteralExpression);
            return node;
        }
        ts.convertToArrayAssignmentPattern = convertToArrayAssignmentPattern;
        function convertToAssignmentElementTarget(node) {
            if (ts.isBindingPattern(node)) {
                return convertToAssignmentPattern(node);
            }
            ts.Debug.assertNode(node, ts.isExpression);
            return node;
        }
        ts.convertToAssignmentElementTarget = convertToAssignmentElementTarget;
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        var isTypeNodeOrTypeParameterDeclaration = ts.or(ts.isTypeNode, ts.isTypeParameterDeclaration);
        function visitNode(node, visitor, test, lift) {
            if (node === undefined || visitor === undefined) {
                return node;
            }
            ts.aggregateTransformFlags(node);
            var visited = visitor(node);
            if (visited === node) {
                return node;
            }
            var visitedNode;
            if (visited === undefined) {
                return undefined;
            }
            else if (ts.isArray(visited)) {
                visitedNode = (lift || extractSingleNode)(visited);
            }
            else {
                visitedNode = visited;
            }
            ts.Debug.assertNode(visitedNode, test);
            ts.aggregateTransformFlags(visitedNode);
            return visitedNode;
        }
        ts.visitNode = visitNode;
        /**
         * Visits a NodeArray using the supplied visitor, possibly returning a new NodeArray in its place.
         *
         * @param nodes The NodeArray to visit.
         * @param visitor The callback used to visit a Node.
         * @param test A node test to execute for each node.
         * @param start An optional value indicating the starting offset at which to start visiting.
         * @param count An optional value indicating the maximum number of nodes to visit.
         */
        function visitNodes(nodes, visitor, test, start, count) {
            if (nodes === undefined || visitor === undefined) {
                return nodes;
            }
            var updated;
            // Ensure start and count have valid values
            var length = nodes.length;
            if (start === undefined || start < 0) {
                start = 0;
            }
            if (count === undefined || count > length - start) {
                count = length - start;
            }
            if (start > 0 || count < length) {
                // If we are not visiting all of the original nodes, we must always create a new array.
                // Since this is a fragment of a node array, we do not copy over the previous location
                // and will only copy over `hasTrailingComma` if we are including the last element.
                updated = ts.createNodeArray([], /*hasTrailingComma*/ nodes.hasTrailingComma && start + count === length);
            }
            // Visit each original node.
            for (var i = 0; i < count; i++) {
                var node = nodes[i + start];
                ts.aggregateTransformFlags(node);
                var visited = node !== undefined ? visitor(node) : undefined;
                if (updated !== undefined || visited === undefined || visited !== node) {
                    if (updated === undefined) {
                        // Ensure we have a copy of `nodes`, up to the current index.
                        updated = ts.createNodeArray(nodes.slice(0, i), nodes.hasTrailingComma);
                        ts.setTextRange(updated, nodes);
                    }
                    if (visited) {
                        if (ts.isArray(visited)) {
                            for (var _i = 0, visited_1 = visited; _i < visited_1.length; _i++) {
                                var visitedNode = visited_1[_i];
                                ts.Debug.assertNode(visitedNode, test);
                                ts.aggregateTransformFlags(visitedNode);
                                updated.push(visitedNode);
                            }
                        }
                        else {
                            ts.Debug.assertNode(visited, test);
                            ts.aggregateTransformFlags(visited);
                            updated.push(visited);
                        }
                    }
                }
            }
            return updated || nodes;
        }
        ts.visitNodes = visitNodes;
        /**
         * Starts a new lexical environment and visits a statement list, ending the lexical environment
         * and merging hoisted declarations upon completion.
         */
        function visitLexicalEnvironment(statements, visitor, context, start, ensureUseStrict) {
            context.startLexicalEnvironment();
            statements = visitNodes(statements, visitor, ts.isStatement, start);
            if (ensureUseStrict)
                statements = ts.ensureUseStrict(statements); // eslint-disable-line @typescript-eslint/no-unnecessary-qualifier
            return ts.mergeLexicalEnvironment(statements, context.endLexicalEnvironment());
        }
        ts.visitLexicalEnvironment = visitLexicalEnvironment;
        /**
         * Starts a new lexical environment and visits a parameter list, suspending the lexical
         * environment upon completion.
         */
        function visitParameterList(nodes, visitor, context, nodesVisitor) {
            if (nodesVisitor === void 0) {
                nodesVisitor = visitNodes;
            }
            context.startLexicalEnvironment();
            var updated = nodesVisitor(nodes, visitor, ts.isParameterDeclaration);
            context.suspendLexicalEnvironment();
            return updated;
        }
        ts.visitParameterList = visitParameterList;
        function visitFunctionBody(node, visitor, context) {
            context.resumeLexicalEnvironment();
            var updated = visitNode(node, visitor, ts.isConciseBody);
            var declarations = context.endLexicalEnvironment();
            if (ts.some(declarations)) {
                var block = ts.convertToFunctionBody(updated);
                var statements = ts.mergeLexicalEnvironment(block.statements, declarations);
                return ts.updateBlock(block, statements);
            }
            return updated;
        }
        ts.visitFunctionBody = visitFunctionBody;
        function visitEachChild(node, visitor, context, nodesVisitor, tokenVisitor) {
            if (nodesVisitor === void 0) {
                nodesVisitor = visitNodes;
            }
            if (node === undefined) {
                return undefined;
            }
            var kind = node.kind;
            // No need to visit nodes with no children.
            if ((kind > 0 /* FirstToken */ && kind <= 152 /* LastToken */) || kind === 183 /* ThisType */) {
                return node;
            }
            switch (kind) {
                // Names
                case 75 /* Identifier */:
                    return ts.updateIdentifier(node, nodesVisitor(node.typeArguments, visitor, isTypeNodeOrTypeParameterDeclaration));
                case 153 /* QualifiedName */:
                    return ts.updateQualifiedName(node, visitNode(node.left, visitor, ts.isEntityName), visitNode(node.right, visitor, ts.isIdentifier));
                case 154 /* ComputedPropertyName */:
                    return ts.updateComputedPropertyName(node, visitNode(node.expression, visitor, ts.isExpression));
                // Signature elements
                case 155 /* TypeParameter */:
                    return ts.updateTypeParameterDeclaration(node, visitNode(node.name, visitor, ts.isIdentifier), visitNode(node.constraint, visitor, ts.isTypeNode), visitNode(node.default, visitor, ts.isTypeNode));
                case 156 /* Parameter */:
                    return ts.updateParameter(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.dotDotDotToken, tokenVisitor, ts.isToken), visitNode(node.name, visitor, ts.isBindingName), visitNode(node.questionToken, tokenVisitor, ts.isToken), visitNode(node.type, visitor, ts.isTypeNode), visitNode(node.initializer, visitor, ts.isExpression));
                case 157 /* Decorator */:
                    return ts.updateDecorator(node, visitNode(node.expression, visitor, ts.isExpression));
                // Type elements
                case 158 /* PropertySignature */:
                    return ts.updatePropertySignature(node, nodesVisitor(node.modifiers, visitor, ts.isToken), visitNode(node.name, visitor, ts.isPropertyName), visitNode(node.questionToken, tokenVisitor, ts.isToken), visitNode(node.type, visitor, ts.isTypeNode), visitNode(node.initializer, visitor, ts.isExpression));
                case 159 /* PropertyDeclaration */:
                    return ts.updateProperty(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.name, visitor, ts.isPropertyName), 
                    // QuestionToken and ExclamationToken is uniqued in Property Declaration and the signature of 'updateProperty' is that too
                    visitNode(node.questionToken || node.exclamationToken, tokenVisitor, ts.isToken), visitNode(node.type, visitor, ts.isTypeNode), visitNode(node.initializer, visitor, ts.isExpression));
                case 160 /* MethodSignature */:
                    return ts.updateMethodSignature(node, nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), nodesVisitor(node.parameters, visitor, ts.isParameterDeclaration), visitNode(node.type, visitor, ts.isTypeNode), visitNode(node.name, visitor, ts.isPropertyName), visitNode(node.questionToken, tokenVisitor, ts.isToken));
                case 161 /* MethodDeclaration */:
                    return ts.updateMethod(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.asteriskToken, tokenVisitor, ts.isToken), visitNode(node.name, visitor, ts.isPropertyName), visitNode(node.questionToken, tokenVisitor, ts.isToken), nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), visitParameterList(node.parameters, visitor, context, nodesVisitor), visitNode(node.type, visitor, ts.isTypeNode), visitFunctionBody(node.body, visitor, context));
                case 162 /* Constructor */:
                    return ts.updateConstructor(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitParameterList(node.parameters, visitor, context, nodesVisitor), visitFunctionBody(node.body, visitor, context));
                case 163 /* GetAccessor */:
                    return ts.updateGetAccessor(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.name, visitor, ts.isPropertyName), visitParameterList(node.parameters, visitor, context, nodesVisitor), visitNode(node.type, visitor, ts.isTypeNode), visitFunctionBody(node.body, visitor, context));
                case 164 /* SetAccessor */:
                    return ts.updateSetAccessor(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.name, visitor, ts.isPropertyName), visitParameterList(node.parameters, visitor, context, nodesVisitor), visitFunctionBody(node.body, visitor, context));
                case 165 /* CallSignature */:
                    return ts.updateCallSignature(node, nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), nodesVisitor(node.parameters, visitor, ts.isParameterDeclaration), visitNode(node.type, visitor, ts.isTypeNode));
                case 166 /* ConstructSignature */:
                    return ts.updateConstructSignature(node, nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), nodesVisitor(node.parameters, visitor, ts.isParameterDeclaration), visitNode(node.type, visitor, ts.isTypeNode));
                case 167 /* IndexSignature */:
                    return ts.updateIndexSignature(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), nodesVisitor(node.parameters, visitor, ts.isParameterDeclaration), visitNode(node.type, visitor, ts.isTypeNode));
                // Types
                case 168 /* TypePredicate */:
                    return ts.updateTypePredicateNodeWithModifier(node, visitNode(node.assertsModifier, visitor), visitNode(node.parameterName, visitor), visitNode(node.type, visitor, ts.isTypeNode));
                case 169 /* TypeReference */:
                    return ts.updateTypeReferenceNode(node, visitNode(node.typeName, visitor, ts.isEntityName), nodesVisitor(node.typeArguments, visitor, ts.isTypeNode));
                case 170 /* FunctionType */:
                    return ts.updateFunctionTypeNode(node, nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), nodesVisitor(node.parameters, visitor, ts.isParameterDeclaration), visitNode(node.type, visitor, ts.isTypeNode));
                case 171 /* ConstructorType */:
                    return ts.updateConstructorTypeNode(node, nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), nodesVisitor(node.parameters, visitor, ts.isParameterDeclaration), visitNode(node.type, visitor, ts.isTypeNode));
                case 172 /* TypeQuery */:
                    return ts.updateTypeQueryNode(node, visitNode(node.exprName, visitor, ts.isEntityName));
                case 173 /* TypeLiteral */:
                    return ts.updateTypeLiteralNode(node, nodesVisitor(node.members, visitor, ts.isTypeElement));
                case 174 /* ArrayType */:
                    return ts.updateArrayTypeNode(node, visitNode(node.elementType, visitor, ts.isTypeNode));
                case 175 /* TupleType */:
                    return ts.updateTupleTypeNode(node, nodesVisitor(node.elementTypes, visitor, ts.isTypeNode));
                case 176 /* OptionalType */:
                    return ts.updateOptionalTypeNode(node, visitNode(node.type, visitor, ts.isTypeNode));
                case 177 /* RestType */:
                    return ts.updateRestTypeNode(node, visitNode(node.type, visitor, ts.isTypeNode));
                case 178 /* UnionType */:
                    return ts.updateUnionTypeNode(node, nodesVisitor(node.types, visitor, ts.isTypeNode));
                case 179 /* IntersectionType */:
                    return ts.updateIntersectionTypeNode(node, nodesVisitor(node.types, visitor, ts.isTypeNode));
                case 180 /* ConditionalType */:
                    return ts.updateConditionalTypeNode(node, visitNode(node.checkType, visitor, ts.isTypeNode), visitNode(node.extendsType, visitor, ts.isTypeNode), visitNode(node.trueType, visitor, ts.isTypeNode), visitNode(node.falseType, visitor, ts.isTypeNode));
                case 181 /* InferType */:
                    return ts.updateInferTypeNode(node, visitNode(node.typeParameter, visitor, ts.isTypeParameterDeclaration));
                case 188 /* ImportType */:
                    return ts.updateImportTypeNode(node, visitNode(node.argument, visitor, ts.isTypeNode), visitNode(node.qualifier, visitor, ts.isEntityName), visitNodes(node.typeArguments, visitor, ts.isTypeNode), node.isTypeOf);
                case 182 /* ParenthesizedType */:
                    return ts.updateParenthesizedType(node, visitNode(node.type, visitor, ts.isTypeNode));
                case 184 /* TypeOperator */:
                    return ts.updateTypeOperatorNode(node, visitNode(node.type, visitor, ts.isTypeNode));
                case 185 /* IndexedAccessType */:
                    return ts.updateIndexedAccessTypeNode(node, visitNode(node.objectType, visitor, ts.isTypeNode), visitNode(node.indexType, visitor, ts.isTypeNode));
                case 186 /* MappedType */:
                    return ts.updateMappedTypeNode(node, visitNode(node.readonlyToken, tokenVisitor, ts.isToken), visitNode(node.typeParameter, visitor, ts.isTypeParameterDeclaration), visitNode(node.questionToken, tokenVisitor, ts.isToken), visitNode(node.type, visitor, ts.isTypeNode));
                case 187 /* LiteralType */:
                    return ts.updateLiteralTypeNode(node, visitNode(node.literal, visitor, ts.isExpression));
                // Binding patterns
                case 189 /* ObjectBindingPattern */:
                    return ts.updateObjectBindingPattern(node, nodesVisitor(node.elements, visitor, ts.isBindingElement));
                case 190 /* ArrayBindingPattern */:
                    return ts.updateArrayBindingPattern(node, nodesVisitor(node.elements, visitor, ts.isArrayBindingElement));
                case 191 /* BindingElement */:
                    return ts.updateBindingElement(node, visitNode(node.dotDotDotToken, tokenVisitor, ts.isToken), visitNode(node.propertyName, visitor, ts.isPropertyName), visitNode(node.name, visitor, ts.isBindingName), visitNode(node.initializer, visitor, ts.isExpression));
                // Expression
                case 192 /* ArrayLiteralExpression */:
                    return ts.updateArrayLiteral(node, nodesVisitor(node.elements, visitor, ts.isExpression));
                case 193 /* ObjectLiteralExpression */:
                    return ts.updateObjectLiteral(node, nodesVisitor(node.properties, visitor, ts.isObjectLiteralElementLike));
                case 194 /* PropertyAccessExpression */:
                    if (node.flags & 32 /* OptionalChain */) {
                        return ts.updatePropertyAccessChain(node, visitNode(node.expression, visitor, ts.isExpression), visitNode(node.questionDotToken, visitor, ts.isToken), visitNode(node.name, visitor, ts.isIdentifier));
                    }
                    return ts.updatePropertyAccess(node, visitNode(node.expression, visitor, ts.isExpression), visitNode(node.name, visitor, ts.isIdentifierOrPrivateIdentifier));
                case 195 /* ElementAccessExpression */:
                    if (node.flags & 32 /* OptionalChain */) {
                        return ts.updateElementAccessChain(node, visitNode(node.expression, visitor, ts.isExpression), visitNode(node.questionDotToken, visitor, ts.isToken), visitNode(node.argumentExpression, visitor, ts.isExpression));
                    }
                    return ts.updateElementAccess(node, visitNode(node.expression, visitor, ts.isExpression), visitNode(node.argumentExpression, visitor, ts.isExpression));
                case 196 /* CallExpression */:
                    if (node.flags & 32 /* OptionalChain */) {
                        return ts.updateCallChain(node, visitNode(node.expression, visitor, ts.isExpression), visitNode(node.questionDotToken, visitor, ts.isToken), nodesVisitor(node.typeArguments, visitor, ts.isTypeNode), nodesVisitor(node.arguments, visitor, ts.isExpression));
                    }
                    return ts.updateCall(node, visitNode(node.expression, visitor, ts.isExpression), nodesVisitor(node.typeArguments, visitor, ts.isTypeNode), nodesVisitor(node.arguments, visitor, ts.isExpression));
                case 197 /* NewExpression */:
                    return ts.updateNew(node, visitNode(node.expression, visitor, ts.isExpression), nodesVisitor(node.typeArguments, visitor, ts.isTypeNode), nodesVisitor(node.arguments, visitor, ts.isExpression));
                case 198 /* TaggedTemplateExpression */:
                    return ts.updateTaggedTemplate(node, visitNode(node.tag, visitor, ts.isExpression), visitNodes(node.typeArguments, visitor, ts.isExpression), visitNode(node.template, visitor, ts.isTemplateLiteral));
                case 199 /* TypeAssertionExpression */:
                    return ts.updateTypeAssertion(node, visitNode(node.type, visitor, ts.isTypeNode), visitNode(node.expression, visitor, ts.isExpression));
                case 200 /* ParenthesizedExpression */:
                    return ts.updateParen(node, visitNode(node.expression, visitor, ts.isExpression));
                case 201 /* FunctionExpression */:
                    return ts.updateFunctionExpression(node, nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.asteriskToken, tokenVisitor, ts.isToken), visitNode(node.name, visitor, ts.isIdentifier), nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), visitParameterList(node.parameters, visitor, context, nodesVisitor), visitNode(node.type, visitor, ts.isTypeNode), visitFunctionBody(node.body, visitor, context));
                case 202 /* ArrowFunction */:
                    return ts.updateArrowFunction(node, nodesVisitor(node.modifiers, visitor, ts.isModifier), nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), visitParameterList(node.parameters, visitor, context, nodesVisitor), visitNode(node.type, visitor, ts.isTypeNode), visitNode(node.equalsGreaterThanToken, visitor, ts.isToken), visitFunctionBody(node.body, visitor, context));
                case 203 /* DeleteExpression */:
                    return ts.updateDelete(node, visitNode(node.expression, visitor, ts.isExpression));
                case 204 /* TypeOfExpression */:
                    return ts.updateTypeOf(node, visitNode(node.expression, visitor, ts.isExpression));
                case 205 /* VoidExpression */:
                    return ts.updateVoid(node, visitNode(node.expression, visitor, ts.isExpression));
                case 206 /* AwaitExpression */:
                    return ts.updateAwait(node, visitNode(node.expression, visitor, ts.isExpression));
                case 207 /* PrefixUnaryExpression */:
                    return ts.updatePrefix(node, visitNode(node.operand, visitor, ts.isExpression));
                case 208 /* PostfixUnaryExpression */:
                    return ts.updatePostfix(node, visitNode(node.operand, visitor, ts.isExpression));
                case 209 /* BinaryExpression */:
                    return ts.updateBinary(node, visitNode(node.left, visitor, ts.isExpression), visitNode(node.right, visitor, ts.isExpression), visitNode(node.operatorToken, visitor, ts.isToken));
                case 210 /* ConditionalExpression */:
                    return ts.updateConditional(node, visitNode(node.condition, visitor, ts.isExpression), visitNode(node.questionToken, visitor, ts.isToken), visitNode(node.whenTrue, visitor, ts.isExpression), visitNode(node.colonToken, visitor, ts.isToken), visitNode(node.whenFalse, visitor, ts.isExpression));
                case 211 /* TemplateExpression */:
                    return ts.updateTemplateExpression(node, visitNode(node.head, visitor, ts.isTemplateHead), nodesVisitor(node.templateSpans, visitor, ts.isTemplateSpan));
                case 212 /* YieldExpression */:
                    return ts.updateYield(node, visitNode(node.asteriskToken, tokenVisitor, ts.isToken), visitNode(node.expression, visitor, ts.isExpression));
                case 213 /* SpreadElement */:
                    return ts.updateSpread(node, visitNode(node.expression, visitor, ts.isExpression));
                case 214 /* ClassExpression */:
                    return ts.updateClassExpression(node, nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.name, visitor, ts.isIdentifier), nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), nodesVisitor(node.heritageClauses, visitor, ts.isHeritageClause), nodesVisitor(node.members, visitor, ts.isClassElement));
                case 216 /* ExpressionWithTypeArguments */:
                    return ts.updateExpressionWithTypeArguments(node, nodesVisitor(node.typeArguments, visitor, ts.isTypeNode), visitNode(node.expression, visitor, ts.isExpression));
                case 217 /* AsExpression */:
                    return ts.updateAsExpression(node, visitNode(node.expression, visitor, ts.isExpression), visitNode(node.type, visitor, ts.isTypeNode));
                case 218 /* NonNullExpression */:
                    return ts.updateNonNullExpression(node, visitNode(node.expression, visitor, ts.isExpression));
                case 219 /* MetaProperty */:
                    return ts.updateMetaProperty(node, visitNode(node.name, visitor, ts.isIdentifier));
                // Misc
                case 221 /* TemplateSpan */:
                    return ts.updateTemplateSpan(node, visitNode(node.expression, visitor, ts.isExpression), visitNode(node.literal, visitor, ts.isTemplateMiddleOrTemplateTail));
                // Element
                case 223 /* Block */:
                    return ts.updateBlock(node, nodesVisitor(node.statements, visitor, ts.isStatement));
                case 225 /* VariableStatement */:
                    return ts.updateVariableStatement(node, nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.declarationList, visitor, ts.isVariableDeclarationList));
                case 226 /* ExpressionStatement */:
                    return ts.updateExpressionStatement(node, visitNode(node.expression, visitor, ts.isExpression));
                case 227 /* IfStatement */:
                    return ts.updateIf(node, visitNode(node.expression, visitor, ts.isExpression), visitNode(node.thenStatement, visitor, ts.isStatement, ts.liftToBlock), visitNode(node.elseStatement, visitor, ts.isStatement, ts.liftToBlock));
                case 228 /* DoStatement */:
                    return ts.updateDo(node, visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock), visitNode(node.expression, visitor, ts.isExpression));
                case 229 /* WhileStatement */:
                    return ts.updateWhile(node, visitNode(node.expression, visitor, ts.isExpression), visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock));
                case 230 /* ForStatement */:
                    return ts.updateFor(node, visitNode(node.initializer, visitor, ts.isForInitializer), visitNode(node.condition, visitor, ts.isExpression), visitNode(node.incrementor, visitor, ts.isExpression), visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock));
                case 231 /* ForInStatement */:
                    return ts.updateForIn(node, visitNode(node.initializer, visitor, ts.isForInitializer), visitNode(node.expression, visitor, ts.isExpression), visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock));
                case 232 /* ForOfStatement */:
                    return ts.updateForOf(node, visitNode(node.awaitModifier, visitor, ts.isToken), visitNode(node.initializer, visitor, ts.isForInitializer), visitNode(node.expression, visitor, ts.isExpression), visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock));
                case 233 /* ContinueStatement */:
                    return ts.updateContinue(node, visitNode(node.label, visitor, ts.isIdentifier));
                case 234 /* BreakStatement */:
                    return ts.updateBreak(node, visitNode(node.label, visitor, ts.isIdentifier));
                case 235 /* ReturnStatement */:
                    return ts.updateReturn(node, visitNode(node.expression, visitor, ts.isExpression));
                case 236 /* WithStatement */:
                    return ts.updateWith(node, visitNode(node.expression, visitor, ts.isExpression), visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock));
                case 237 /* SwitchStatement */:
                    return ts.updateSwitch(node, visitNode(node.expression, visitor, ts.isExpression), visitNode(node.caseBlock, visitor, ts.isCaseBlock));
                case 238 /* LabeledStatement */:
                    return ts.updateLabel(node, visitNode(node.label, visitor, ts.isIdentifier), visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock));
                case 239 /* ThrowStatement */:
                    return ts.updateThrow(node, visitNode(node.expression, visitor, ts.isExpression));
                case 240 /* TryStatement */:
                    return ts.updateTry(node, visitNode(node.tryBlock, visitor, ts.isBlock), visitNode(node.catchClause, visitor, ts.isCatchClause), visitNode(node.finallyBlock, visitor, ts.isBlock));
                case 242 /* VariableDeclaration */:
                    return ts.updateTypeScriptVariableDeclaration(node, visitNode(node.name, visitor, ts.isBindingName), visitNode(node.exclamationToken, tokenVisitor, ts.isToken), visitNode(node.type, visitor, ts.isTypeNode), visitNode(node.initializer, visitor, ts.isExpression));
                case 243 /* VariableDeclarationList */:
                    return ts.updateVariableDeclarationList(node, nodesVisitor(node.declarations, visitor, ts.isVariableDeclaration));
                case 244 /* FunctionDeclaration */:
                    return ts.updateFunctionDeclaration(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.asteriskToken, tokenVisitor, ts.isToken), visitNode(node.name, visitor, ts.isIdentifier), nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), visitParameterList(node.parameters, visitor, context, nodesVisitor), visitNode(node.type, visitor, ts.isTypeNode), visitFunctionBody(node.body, visitor, context));
                case 245 /* ClassDeclaration */:
                    return ts.updateClassDeclaration(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.name, visitor, ts.isIdentifier), nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), nodesVisitor(node.heritageClauses, visitor, ts.isHeritageClause), nodesVisitor(node.members, visitor, ts.isClassElement));
                case 246 /* InterfaceDeclaration */:
                    return ts.updateInterfaceDeclaration(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.name, visitor, ts.isIdentifier), nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), nodesVisitor(node.heritageClauses, visitor, ts.isHeritageClause), nodesVisitor(node.members, visitor, ts.isTypeElement));
                case 247 /* TypeAliasDeclaration */:
                    return ts.updateTypeAliasDeclaration(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.name, visitor, ts.isIdentifier), nodesVisitor(node.typeParameters, visitor, ts.isTypeParameterDeclaration), visitNode(node.type, visitor, ts.isTypeNode));
                case 248 /* EnumDeclaration */:
                    return ts.updateEnumDeclaration(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.name, visitor, ts.isIdentifier), nodesVisitor(node.members, visitor, ts.isEnumMember));
                case 249 /* ModuleDeclaration */:
                    return ts.updateModuleDeclaration(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.name, visitor, ts.isIdentifier), visitNode(node.body, visitor, ts.isModuleBody));
                case 250 /* ModuleBlock */:
                    return ts.updateModuleBlock(node, nodesVisitor(node.statements, visitor, ts.isStatement));
                case 251 /* CaseBlock */:
                    return ts.updateCaseBlock(node, nodesVisitor(node.clauses, visitor, ts.isCaseOrDefaultClause));
                case 252 /* NamespaceExportDeclaration */:
                    return ts.updateNamespaceExportDeclaration(node, visitNode(node.name, visitor, ts.isIdentifier));
                case 253 /* ImportEqualsDeclaration */:
                    return ts.updateImportEqualsDeclaration(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.name, visitor, ts.isIdentifier), visitNode(node.moduleReference, visitor, ts.isModuleReference));
                case 254 /* ImportDeclaration */:
                    return ts.updateImportDeclaration(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.importClause, visitor, ts.isImportClause), visitNode(node.moduleSpecifier, visitor, ts.isExpression));
                case 255 /* ImportClause */:
                    return ts.updateImportClause(node, visitNode(node.name, visitor, ts.isIdentifier), visitNode(node.namedBindings, visitor, ts.isNamedImportBindings), node.isTypeOnly);
                case 256 /* NamespaceImport */:
                    return ts.updateNamespaceImport(node, visitNode(node.name, visitor, ts.isIdentifier));
                case 262 /* NamespaceExport */:
                    return ts.updateNamespaceExport(node, visitNode(node.name, visitor, ts.isIdentifier));
                case 257 /* NamedImports */:
                    return ts.updateNamedImports(node, nodesVisitor(node.elements, visitor, ts.isImportSpecifier));
                case 258 /* ImportSpecifier */:
                    return ts.updateImportSpecifier(node, visitNode(node.propertyName, visitor, ts.isIdentifier), visitNode(node.name, visitor, ts.isIdentifier));
                case 259 /* ExportAssignment */:
                    return ts.updateExportAssignment(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.expression, visitor, ts.isExpression));
                case 260 /* ExportDeclaration */:
                    return ts.updateExportDeclaration(node, nodesVisitor(node.decorators, visitor, ts.isDecorator), nodesVisitor(node.modifiers, visitor, ts.isModifier), visitNode(node.exportClause, visitor, ts.isNamedExportBindings), visitNode(node.moduleSpecifier, visitor, ts.isExpression), node.isTypeOnly);
                case 261 /* NamedExports */:
                    return ts.updateNamedExports(node, nodesVisitor(node.elements, visitor, ts.isExportSpecifier));
                case 263 /* ExportSpecifier */:
                    return ts.updateExportSpecifier(node, visitNode(node.propertyName, visitor, ts.isIdentifier), visitNode(node.name, visitor, ts.isIdentifier));
                // Module references
                case 265 /* ExternalModuleReference */:
                    return ts.updateExternalModuleReference(node, visitNode(node.expression, visitor, ts.isExpression));
                // JSX
                case 266 /* JsxElement */:
                    return ts.updateJsxElement(node, visitNode(node.openingElement, visitor, ts.isJsxOpeningElement), nodesVisitor(node.children, visitor, ts.isJsxChild), visitNode(node.closingElement, visitor, ts.isJsxClosingElement));
                case 267 /* JsxSelfClosingElement */:
                    return ts.updateJsxSelfClosingElement(node, visitNode(node.tagName, visitor, ts.isJsxTagNameExpression), nodesVisitor(node.typeArguments, visitor, ts.isTypeNode), visitNode(node.attributes, visitor, ts.isJsxAttributes));
                case 268 /* JsxOpeningElement */:
                    return ts.updateJsxOpeningElement(node, visitNode(node.tagName, visitor, ts.isJsxTagNameExpression), nodesVisitor(node.typeArguments, visitor, ts.isTypeNode), visitNode(node.attributes, visitor, ts.isJsxAttributes));
                case 269 /* JsxClosingElement */:
                    return ts.updateJsxClosingElement(node, visitNode(node.tagName, visitor, ts.isJsxTagNameExpression));
                case 270 /* JsxFragment */:
                    return ts.updateJsxFragment(node, visitNode(node.openingFragment, visitor, ts.isJsxOpeningFragment), nodesVisitor(node.children, visitor, ts.isJsxChild), visitNode(node.closingFragment, visitor, ts.isJsxClosingFragment));
                case 273 /* JsxAttribute */:
                    return ts.updateJsxAttribute(node, visitNode(node.name, visitor, ts.isIdentifier), visitNode(node.initializer, visitor, ts.isStringLiteralOrJsxExpression));
                case 274 /* JsxAttributes */:
                    return ts.updateJsxAttributes(node, nodesVisitor(node.properties, visitor, ts.isJsxAttributeLike));
                case 275 /* JsxSpreadAttribute */:
                    return ts.updateJsxSpreadAttribute(node, visitNode(node.expression, visitor, ts.isExpression));
                case 276 /* JsxExpression */:
                    return ts.updateJsxExpression(node, visitNode(node.expression, visitor, ts.isExpression));
                // Clauses
                case 277 /* CaseClause */:
                    return ts.updateCaseClause(node, visitNode(node.expression, visitor, ts.isExpression), nodesVisitor(node.statements, visitor, ts.isStatement));
                case 278 /* DefaultClause */:
                    return ts.updateDefaultClause(node, nodesVisitor(node.statements, visitor, ts.isStatement));
                case 279 /* HeritageClause */:
                    return ts.updateHeritageClause(node, nodesVisitor(node.types, visitor, ts.isExpressionWithTypeArguments));
                case 280 /* CatchClause */:
                    return ts.updateCatchClause(node, visitNode(node.variableDeclaration, visitor, ts.isVariableDeclaration), visitNode(node.block, visitor, ts.isBlock));
                // Property assignments
                case 281 /* PropertyAssignment */:
                    return ts.updatePropertyAssignment(node, visitNode(node.name, visitor, ts.isPropertyName), visitNode(node.initializer, visitor, ts.isExpression));
                case 282 /* ShorthandPropertyAssignment */:
                    return ts.updateShorthandPropertyAssignment(node, visitNode(node.name, visitor, ts.isIdentifier), visitNode(node.objectAssignmentInitializer, visitor, ts.isExpression));
                case 283 /* SpreadAssignment */:
                    return ts.updateSpreadAssignment(node, visitNode(node.expression, visitor, ts.isExpression));
                // Enum
                case 284 /* EnumMember */:
                    return ts.updateEnumMember(node, visitNode(node.name, visitor, ts.isPropertyName), visitNode(node.initializer, visitor, ts.isExpression));
                // Top-level nodes
                case 290 /* SourceFile */:
                    return ts.updateSourceFileNode(node, visitLexicalEnvironment(node.statements, visitor, context));
                // Transformation nodes
                case 325 /* PartiallyEmittedExpression */:
                    return ts.updatePartiallyEmittedExpression(node, visitNode(node.expression, visitor, ts.isExpression));
                case 326 /* CommaListExpression */:
                    return ts.updateCommaList(node, nodesVisitor(node.elements, visitor, ts.isExpression));
                default:
                    // No need to visit nodes with no children.
                    return node;
            }
        }
        ts.visitEachChild = visitEachChild;
        /**
         * Extracts the single node from a NodeArray.
         *
         * @param nodes The NodeArray.
         */
        function extractSingleNode(nodes) {
            ts.Debug.assert(nodes.length <= 1, "Too many nodes written to output.");
            return ts.singleOrUndefined(nodes);
        }
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        function reduceNode(node, f, initial) {
            return node ? f(initial, node) : initial;
        }
        function reduceNodeArray(nodes, f, initial) {
            return nodes ? f(initial, nodes) : initial;
        }
        /**
         * Similar to `reduceLeft`, performs a reduction against each child of a node.
         * NOTE: Unlike `forEachChild`, this does *not* visit every node.
         *
         * @param node The node containing the children to reduce.
         * @param initial The initial value to supply to the reduction.
         * @param f The callback function
         */
        function reduceEachChild(node, initial, cbNode, cbNodeArray) {
            if (node === undefined) {
                return initial;
            }
            var reduceNodes = cbNodeArray ? reduceNodeArray : ts.reduceLeft;
            var cbNodes = cbNodeArray || cbNode;
            var kind = node.kind;
            // No need to visit nodes with no children.
            if ((kind > 0 /* FirstToken */ && kind <= 152 /* LastToken */)) {
                return initial;
            }
            // We do not yet support types.
            if ((kind >= 168 /* TypePredicate */ && kind <= 187 /* LiteralType */)) {
                return initial;
            }
            var result = initial;
            switch (node.kind) {
                // Leaf nodes
                case 222 /* SemicolonClassElement */:
                case 224 /* EmptyStatement */:
                case 215 /* OmittedExpression */:
                case 241 /* DebuggerStatement */:
                case 324 /* NotEmittedStatement */:
                    // No need to visit nodes with no children.
                    break;
                // Names
                case 153 /* QualifiedName */:
                    result = reduceNode(node.left, cbNode, result);
                    result = reduceNode(node.right, cbNode, result);
                    break;
                case 154 /* ComputedPropertyName */:
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                // Signature elements
                case 156 /* Parameter */:
                    result = reduceNodes(node.decorators, cbNodes, result);
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.type, cbNode, result);
                    result = reduceNode(node.initializer, cbNode, result);
                    break;
                case 157 /* Decorator */:
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                // Type member
                case 158 /* PropertySignature */:
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.questionToken, cbNode, result);
                    result = reduceNode(node.type, cbNode, result);
                    result = reduceNode(node.initializer, cbNode, result);
                    break;
                case 159 /* PropertyDeclaration */:
                    result = reduceNodes(node.decorators, cbNodes, result);
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.type, cbNode, result);
                    result = reduceNode(node.initializer, cbNode, result);
                    break;
                case 161 /* MethodDeclaration */:
                    result = reduceNodes(node.decorators, cbNodes, result);
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNodes(node.typeParameters, cbNodes, result);
                    result = reduceNodes(node.parameters, cbNodes, result);
                    result = reduceNode(node.type, cbNode, result);
                    result = reduceNode(node.body, cbNode, result);
                    break;
                case 162 /* Constructor */:
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNodes(node.parameters, cbNodes, result);
                    result = reduceNode(node.body, cbNode, result);
                    break;
                case 163 /* GetAccessor */:
                    result = reduceNodes(node.decorators, cbNodes, result);
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNodes(node.parameters, cbNodes, result);
                    result = reduceNode(node.type, cbNode, result);
                    result = reduceNode(node.body, cbNode, result);
                    break;
                case 164 /* SetAccessor */:
                    result = reduceNodes(node.decorators, cbNodes, result);
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNodes(node.parameters, cbNodes, result);
                    result = reduceNode(node.body, cbNode, result);
                    break;
                // Binding patterns
                case 189 /* ObjectBindingPattern */:
                case 190 /* ArrayBindingPattern */:
                    result = reduceNodes(node.elements, cbNodes, result);
                    break;
                case 191 /* BindingElement */:
                    result = reduceNode(node.propertyName, cbNode, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.initializer, cbNode, result);
                    break;
                // Expression
                case 192 /* ArrayLiteralExpression */:
                    result = reduceNodes(node.elements, cbNodes, result);
                    break;
                case 193 /* ObjectLiteralExpression */:
                    result = reduceNodes(node.properties, cbNodes, result);
                    break;
                case 194 /* PropertyAccessExpression */:
                    result = reduceNode(node.expression, cbNode, result);
                    result = reduceNode(node.name, cbNode, result);
                    break;
                case 195 /* ElementAccessExpression */:
                    result = reduceNode(node.expression, cbNode, result);
                    result = reduceNode(node.argumentExpression, cbNode, result);
                    break;
                case 196 /* CallExpression */:
                    result = reduceNode(node.expression, cbNode, result);
                    result = reduceNodes(node.typeArguments, cbNodes, result);
                    result = reduceNodes(node.arguments, cbNodes, result);
                    break;
                case 197 /* NewExpression */:
                    result = reduceNode(node.expression, cbNode, result);
                    result = reduceNodes(node.typeArguments, cbNodes, result);
                    result = reduceNodes(node.arguments, cbNodes, result);
                    break;
                case 198 /* TaggedTemplateExpression */:
                    result = reduceNode(node.tag, cbNode, result);
                    result = reduceNodes(node.typeArguments, cbNodes, result);
                    result = reduceNode(node.template, cbNode, result);
                    break;
                case 199 /* TypeAssertionExpression */:
                    result = reduceNode(node.type, cbNode, result);
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                case 201 /* FunctionExpression */:
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNodes(node.typeParameters, cbNodes, result);
                    result = reduceNodes(node.parameters, cbNodes, result);
                    result = reduceNode(node.type, cbNode, result);
                    result = reduceNode(node.body, cbNode, result);
                    break;
                case 202 /* ArrowFunction */:
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNodes(node.typeParameters, cbNodes, result);
                    result = reduceNodes(node.parameters, cbNodes, result);
                    result = reduceNode(node.type, cbNode, result);
                    result = reduceNode(node.body, cbNode, result);
                    break;
                case 200 /* ParenthesizedExpression */:
                case 203 /* DeleteExpression */:
                case 204 /* TypeOfExpression */:
                case 205 /* VoidExpression */:
                case 206 /* AwaitExpression */:
                case 212 /* YieldExpression */:
                case 213 /* SpreadElement */:
                case 218 /* NonNullExpression */:
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                case 207 /* PrefixUnaryExpression */:
                case 208 /* PostfixUnaryExpression */:
                    result = reduceNode(node.operand, cbNode, result);
                    break;
                case 209 /* BinaryExpression */:
                    result = reduceNode(node.left, cbNode, result);
                    result = reduceNode(node.right, cbNode, result);
                    break;
                case 210 /* ConditionalExpression */:
                    result = reduceNode(node.condition, cbNode, result);
                    result = reduceNode(node.whenTrue, cbNode, result);
                    result = reduceNode(node.whenFalse, cbNode, result);
                    break;
                case 211 /* TemplateExpression */:
                    result = reduceNode(node.head, cbNode, result);
                    result = reduceNodes(node.templateSpans, cbNodes, result);
                    break;
                case 214 /* ClassExpression */:
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNodes(node.typeParameters, cbNodes, result);
                    result = reduceNodes(node.heritageClauses, cbNodes, result);
                    result = reduceNodes(node.members, cbNodes, result);
                    break;
                case 216 /* ExpressionWithTypeArguments */:
                    result = reduceNode(node.expression, cbNode, result);
                    result = reduceNodes(node.typeArguments, cbNodes, result);
                    break;
                case 217 /* AsExpression */:
                    result = reduceNode(node.expression, cbNode, result);
                    result = reduceNode(node.type, cbNode, result);
                    break;
                // Misc
                case 221 /* TemplateSpan */:
                    result = reduceNode(node.expression, cbNode, result);
                    result = reduceNode(node.literal, cbNode, result);
                    break;
                // Element
                case 223 /* Block */:
                    result = reduceNodes(node.statements, cbNodes, result);
                    break;
                case 225 /* VariableStatement */:
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.declarationList, cbNode, result);
                    break;
                case 226 /* ExpressionStatement */:
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                case 227 /* IfStatement */:
                    result = reduceNode(node.expression, cbNode, result);
                    result = reduceNode(node.thenStatement, cbNode, result);
                    result = reduceNode(node.elseStatement, cbNode, result);
                    break;
                case 228 /* DoStatement */:
                    result = reduceNode(node.statement, cbNode, result);
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                case 229 /* WhileStatement */:
                case 236 /* WithStatement */:
                    result = reduceNode(node.expression, cbNode, result);
                    result = reduceNode(node.statement, cbNode, result);
                    break;
                case 230 /* ForStatement */:
                    result = reduceNode(node.initializer, cbNode, result);
                    result = reduceNode(node.condition, cbNode, result);
                    result = reduceNode(node.incrementor, cbNode, result);
                    result = reduceNode(node.statement, cbNode, result);
                    break;
                case 231 /* ForInStatement */:
                case 232 /* ForOfStatement */:
                    result = reduceNode(node.initializer, cbNode, result);
                    result = reduceNode(node.expression, cbNode, result);
                    result = reduceNode(node.statement, cbNode, result);
                    break;
                case 235 /* ReturnStatement */:
                case 239 /* ThrowStatement */:
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                case 237 /* SwitchStatement */:
                    result = reduceNode(node.expression, cbNode, result);
                    result = reduceNode(node.caseBlock, cbNode, result);
                    break;
                case 238 /* LabeledStatement */:
                    result = reduceNode(node.label, cbNode, result);
                    result = reduceNode(node.statement, cbNode, result);
                    break;
                case 240 /* TryStatement */:
                    result = reduceNode(node.tryBlock, cbNode, result);
                    result = reduceNode(node.catchClause, cbNode, result);
                    result = reduceNode(node.finallyBlock, cbNode, result);
                    break;
                case 242 /* VariableDeclaration */:
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.type, cbNode, result);
                    result = reduceNode(node.initializer, cbNode, result);
                    break;
                case 243 /* VariableDeclarationList */:
                    result = reduceNodes(node.declarations, cbNodes, result);
                    break;
                case 244 /* FunctionDeclaration */:
                    result = reduceNodes(node.decorators, cbNodes, result);
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNodes(node.typeParameters, cbNodes, result);
                    result = reduceNodes(node.parameters, cbNodes, result);
                    result = reduceNode(node.type, cbNode, result);
                    result = reduceNode(node.body, cbNode, result);
                    break;
                case 245 /* ClassDeclaration */:
                    result = reduceNodes(node.decorators, cbNodes, result);
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNodes(node.typeParameters, cbNodes, result);
                    result = reduceNodes(node.heritageClauses, cbNodes, result);
                    result = reduceNodes(node.members, cbNodes, result);
                    break;
                case 248 /* EnumDeclaration */:
                    result = reduceNodes(node.decorators, cbNodes, result);
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNodes(node.members, cbNodes, result);
                    break;
                case 249 /* ModuleDeclaration */:
                    result = reduceNodes(node.decorators, cbNodes, result);
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.body, cbNode, result);
                    break;
                case 250 /* ModuleBlock */:
                    result = reduceNodes(node.statements, cbNodes, result);
                    break;
                case 251 /* CaseBlock */:
                    result = reduceNodes(node.clauses, cbNodes, result);
                    break;
                case 253 /* ImportEqualsDeclaration */:
                    result = reduceNodes(node.decorators, cbNodes, result);
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.moduleReference, cbNode, result);
                    break;
                case 254 /* ImportDeclaration */:
                    result = reduceNodes(node.decorators, cbNodes, result);
                    result = reduceNodes(node.modifiers, cbNodes, result);
                    result = reduceNode(node.importClause, cbNode, result);
                    result = reduceNode(node.moduleSpecifier, cbNode, result);
                    break;
                case 255 /* ImportClause */:
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.namedBindings, cbNode, result);
                    break;
                case 256 /* NamespaceImport */:
                    result = reduceNode(node.name, cbNode, result);
                    break;
                case 262 /* NamespaceExport */:
                    result = reduceNode(node.name, cbNode, result);
                    break;
                case 257 /* NamedImports */:
                case 261 /* NamedExports */:
                    result = reduceNodes(node.elements, cbNodes, result);
                    break;
                case 258 /* ImportSpecifier */:
                case 263 /* ExportSpecifier */:
                    result = reduceNode(node.propertyName, cbNode, result);
                    result = reduceNode(node.name, cbNode, result);
                    break;
                case 259 /* ExportAssignment */:
                    result = ts.reduceLeft(node.decorators, cbNode, result);
                    result = ts.reduceLeft(node.modifiers, cbNode, result);
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                case 260 /* ExportDeclaration */:
                    result = ts.reduceLeft(node.decorators, cbNode, result);
                    result = ts.reduceLeft(node.modifiers, cbNode, result);
                    result = reduceNode(node.exportClause, cbNode, result);
                    result = reduceNode(node.moduleSpecifier, cbNode, result);
                    break;
                // Module references
                case 265 /* ExternalModuleReference */:
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                // JSX
                case 266 /* JsxElement */:
                    result = reduceNode(node.openingElement, cbNode, result);
                    result = ts.reduceLeft(node.children, cbNode, result);
                    result = reduceNode(node.closingElement, cbNode, result);
                    break;
                case 270 /* JsxFragment */:
                    result = reduceNode(node.openingFragment, cbNode, result);
                    result = ts.reduceLeft(node.children, cbNode, result);
                    result = reduceNode(node.closingFragment, cbNode, result);
                    break;
                case 267 /* JsxSelfClosingElement */:
                case 268 /* JsxOpeningElement */:
                    result = reduceNode(node.tagName, cbNode, result);
                    result = reduceNodes(node.typeArguments, cbNode, result);
                    result = reduceNode(node.attributes, cbNode, result);
                    break;
                case 274 /* JsxAttributes */:
                    result = reduceNodes(node.properties, cbNodes, result);
                    break;
                case 269 /* JsxClosingElement */:
                    result = reduceNode(node.tagName, cbNode, result);
                    break;
                case 273 /* JsxAttribute */:
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.initializer, cbNode, result);
                    break;
                case 275 /* JsxSpreadAttribute */:
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                case 276 /* JsxExpression */:
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                // Clauses
                case 277 /* CaseClause */:
                    result = reduceNode(node.expression, cbNode, result);
                // falls through
                case 278 /* DefaultClause */:
                    result = reduceNodes(node.statements, cbNodes, result);
                    break;
                case 279 /* HeritageClause */:
                    result = reduceNodes(node.types, cbNodes, result);
                    break;
                case 280 /* CatchClause */:
                    result = reduceNode(node.variableDeclaration, cbNode, result);
                    result = reduceNode(node.block, cbNode, result);
                    break;
                // Property assignments
                case 281 /* PropertyAssignment */:
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.initializer, cbNode, result);
                    break;
                case 282 /* ShorthandPropertyAssignment */:
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.objectAssignmentInitializer, cbNode, result);
                    break;
                case 283 /* SpreadAssignment */:
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                // Enum
                case 284 /* EnumMember */:
                    result = reduceNode(node.name, cbNode, result);
                    result = reduceNode(node.initializer, cbNode, result);
                    break;
                // Top-level nodes
                case 290 /* SourceFile */:
                    result = reduceNodes(node.statements, cbNodes, result);
                    break;
                // Transformation nodes
                case 325 /* PartiallyEmittedExpression */:
                    result = reduceNode(node.expression, cbNode, result);
                    break;
                case 326 /* CommaListExpression */:
                    result = reduceNodes(node.elements, cbNodes, result);
                    break;
                default:
                    break;
            }
            return result;
        }
        ts.reduceEachChild = reduceEachChild;
        function mergeLexicalEnvironment(statements, declarations) {
            if (!ts.some(declarations)) {
                return statements;
            }
            return ts.isNodeArray(statements)
                ? ts.setTextRange(ts.createNodeArray(ts.insertStatementsAfterStandardPrologue(statements.slice(), declarations)), statements)
                : ts.insertStatementsAfterStandardPrologue(statements, declarations);
        }
        ts.mergeLexicalEnvironment = mergeLexicalEnvironment;
        /**
         * Lifts a NodeArray containing only Statement nodes to a block.
         *
         * @param nodes The NodeArray.
         */
        function liftToBlock(nodes) {
            ts.Debug.assert(ts.every(nodes, ts.isStatement), "Cannot lift nodes to a Block.");
            return ts.singleOrUndefined(nodes) || ts.createBlock(nodes);
        }
        ts.liftToBlock = liftToBlock;
        /**
         * Aggregates the TransformFlags for a Node and its subtree.
         */
        function aggregateTransformFlags(node) {
            aggregateTransformFlagsForNode(node);
            return node;
        }
        ts.aggregateTransformFlags = aggregateTransformFlags;
        /**
         * Aggregates the TransformFlags for a Node and its subtree. The flags for the subtree are
         * computed first, then the transform flags for the current node are computed from the subtree
         * flags and the state of the current node. Finally, the transform flags of the node are
         * returned, excluding any flags that should not be included in its parent node's subtree
         * flags.
         */
        function aggregateTransformFlagsForNode(node) {
            if (node === undefined) {
                return 0 /* None */;
            }
            if (node.transformFlags & 536870912 /* HasComputedFlags */) {
                return node.transformFlags & ~ts.getTransformFlagsSubtreeExclusions(node.kind);
            }
            var subtreeFlags = aggregateTransformFlagsForSubtree(node);
            return ts.computeTransformFlagsForNode(node, subtreeFlags);
        }
        function aggregateTransformFlagsForNodeArray(nodes) {
            if (nodes === undefined) {
                return 0 /* None */;
            }
            var subtreeFlags = 0 /* None */;
            var nodeArrayFlags = 0 /* None */;
            for (var _i = 0, nodes_3 = nodes; _i < nodes_3.length; _i++) {
                var node = nodes_3[_i];
                subtreeFlags |= aggregateTransformFlagsForNode(node);
                nodeArrayFlags |= node.transformFlags & ~536870912 /* HasComputedFlags */;
            }
            nodes.transformFlags = nodeArrayFlags | 536870912 /* HasComputedFlags */;
            return subtreeFlags;
        }
        /**
         * Aggregates the transform flags for the subtree of a node.
         */
        function aggregateTransformFlagsForSubtree(node) {
            // We do not transform ambient declarations or types, so there is no need to
            // recursively aggregate transform flags.
            if (ts.hasModifier(node, 2 /* Ambient */) || (ts.isTypeNode(node) && node.kind !== 216 /* ExpressionWithTypeArguments */)) {
                return 0 /* None */;
            }
            // Aggregate the transform flags of each child.
            return reduceEachChild(node, 0 /* None */, aggregateTransformFlagsForChildNode, aggregateTransformFlagsForChildNodes);
        }
        /**
         * Aggregates the TransformFlags of a child node with the TransformFlags of its
         * siblings.
         */
        function aggregateTransformFlagsForChildNode(transformFlags, node) {
            return transformFlags | aggregateTransformFlagsForNode(node);
        }
        function aggregateTransformFlagsForChildNodes(transformFlags, nodes) {
            return transformFlags | aggregateTransformFlagsForNodeArray(nodes);
        }
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        function createSourceMapGenerator(host, file, sourceRoot, sourcesDirectoryPath, generatorOptions) {
            var _a = generatorOptions.extendedDiagnostics
                ? ts.performance.createTimer("Source Map", "beforeSourcemap", "afterSourcemap")
                : ts.performance.nullTimer, enter = _a.enter, exit = _a.exit;
            // Current source map file and its index in the sources list
            var rawSources = [];
            var sources = [];
            var sourceToSourceIndexMap = ts.createMap();
            var sourcesContent;
            var names = [];
            var nameToNameIndexMap;
            var mappings = "";
            // Last recorded and encoded mappings
            var lastGeneratedLine = 0;
            var lastGeneratedCharacter = 0;
            var lastSourceIndex = 0;
            var lastSourceLine = 0;
            var lastSourceCharacter = 0;
            var lastNameIndex = 0;
            var hasLast = false;
            var pendingGeneratedLine = 0;
            var pendingGeneratedCharacter = 0;
            var pendingSourceIndex = 0;
            var pendingSourceLine = 0;
            var pendingSourceCharacter = 0;
            var pendingNameIndex = 0;
            var hasPending = false;
            var hasPendingSource = false;
            var hasPendingName = false;
            return {
                getSources: function () { return rawSources; },
                addSource: addSource,
                setSourceContent: setSourceContent,
                addName: addName,
                addMapping: addMapping,
                appendSourceMap: appendSourceMap,
                toJSON: toJSON,
                toString: function () { return JSON.stringify(toJSON()); }
            };
            function addSource(fileName) {
                enter();
                var source = ts.getRelativePathToDirectoryOrUrl(sourcesDirectoryPath, fileName, host.getCurrentDirectory(), host.getCanonicalFileName, 
                /*isAbsolutePathAnUrl*/ true);
                var sourceIndex = sourceToSourceIndexMap.get(source);
                if (sourceIndex === undefined) {
                    sourceIndex = sources.length;
                    sources.push(source);
                    rawSources.push(fileName);
                    sourceToSourceIndexMap.set(source, sourceIndex);
                }
                exit();
                return sourceIndex;
            }
            /* eslint-disable boolean-trivia, no-null/no-null */
            function setSourceContent(sourceIndex, content) {
                enter();
                if (content !== null) {
                    if (!sourcesContent)
                        sourcesContent = [];
                    while (sourcesContent.length < sourceIndex) {
                        sourcesContent.push(null);
                    }
                    sourcesContent[sourceIndex] = content;
                }
                exit();
            }
            /* eslint-enable boolean-trivia, no-null/no-null */
            function addName(name) {
                enter();
                if (!nameToNameIndexMap)
                    nameToNameIndexMap = ts.createMap();
                var nameIndex = nameToNameIndexMap.get(name);
                if (nameIndex === undefined) {
                    nameIndex = names.length;
                    names.push(name);
                    nameToNameIndexMap.set(name, nameIndex);
                }
                exit();
                return nameIndex;
            }
            function isNewGeneratedPosition(generatedLine, generatedCharacter) {
                return !hasPending
                    || pendingGeneratedLine !== generatedLine
                    || pendingGeneratedCharacter !== generatedCharacter;
            }
            function isBacktrackingSourcePosition(sourceIndex, sourceLine, sourceCharacter) {
                return sourceIndex !== undefined
                    && sourceLine !== undefined
                    && sourceCharacter !== undefined
                    && pendingSourceIndex === sourceIndex
                    && (pendingSourceLine > sourceLine
                        || pendingSourceLine === sourceLine && pendingSourceCharacter > sourceCharacter);
            }
            function addMapping(generatedLine, generatedCharacter, sourceIndex, sourceLine, sourceCharacter, nameIndex) {
                ts.Debug.assert(generatedLine >= pendingGeneratedLine, "generatedLine cannot backtrack");
                ts.Debug.assert(generatedCharacter >= 0, "generatedCharacter cannot be negative");
                ts.Debug.assert(sourceIndex === undefined || sourceIndex >= 0, "sourceIndex cannot be negative");
                ts.Debug.assert(sourceLine === undefined || sourceLine >= 0, "sourceLine cannot be negative");
                ts.Debug.assert(sourceCharacter === undefined || sourceCharacter >= 0, "sourceCharacter cannot be negative");
                enter();
                // If this location wasn't recorded or the location in source is going backwards, record the mapping
                if (isNewGeneratedPosition(generatedLine, generatedCharacter) ||
                    isBacktrackingSourcePosition(sourceIndex, sourceLine, sourceCharacter)) {
                    commitPendingMapping();
                    pendingGeneratedLine = generatedLine;
                    pendingGeneratedCharacter = generatedCharacter;
                    hasPendingSource = false;
                    hasPendingName = false;
                    hasPending = true;
                }
                if (sourceIndex !== undefined && sourceLine !== undefined && sourceCharacter !== undefined) {
                    pendingSourceIndex = sourceIndex;
                    pendingSourceLine = sourceLine;
                    pendingSourceCharacter = sourceCharacter;
                    hasPendingSource = true;
                    if (nameIndex !== undefined) {
                        pendingNameIndex = nameIndex;
                        hasPendingName = true;
                    }
                }
                exit();
            }
            function appendSourceMap(generatedLine, generatedCharacter, map, sourceMapPath, start, end) {
                ts.Debug.assert(generatedLine >= pendingGeneratedLine, "generatedLine cannot backtrack");
                ts.Debug.assert(generatedCharacter >= 0, "generatedCharacter cannot be negative");
                enter();
                // First, decode the old component sourcemap
                var sourceIndexToNewSourceIndexMap = [];
                var nameIndexToNewNameIndexMap;
                var mappingIterator = decodeMappings(map.mappings);
                for (var iterResult = mappingIterator.next(); !iterResult.done; iterResult = mappingIterator.next()) {
                    var raw = iterResult.value;
                    if (end && (raw.generatedLine > end.line ||
                        (raw.generatedLine === end.line && raw.generatedCharacter > end.character))) {
                        break;
                    }
                    if (start && (raw.generatedLine < start.line ||
                        (start.line === raw.generatedLine && raw.generatedCharacter < start.character))) {
                        continue;
                    }
                    // Then reencode all the updated mappings into the overall map
                    var newSourceIndex = void 0;
                    var newSourceLine = void 0;
                    var newSourceCharacter = void 0;
                    var newNameIndex = void 0;
                    if (raw.sourceIndex !== undefined) {
                        newSourceIndex = sourceIndexToNewSourceIndexMap[raw.sourceIndex];
                        if (newSourceIndex === undefined) {
                            // Apply offsets to each position and fixup source entries
                            var rawPath = map.sources[raw.sourceIndex];
                            var relativePath = map.sourceRoot ? ts.combinePaths(map.sourceRoot, rawPath) : rawPath;
                            var combinedPath = ts.combinePaths(ts.getDirectoryPath(sourceMapPath), relativePath);
                            sourceIndexToNewSourceIndexMap[raw.sourceIndex] = newSourceIndex = addSource(combinedPath);
                            if (map.sourcesContent && typeof map.sourcesContent[raw.sourceIndex] === "string") {
                                setSourceContent(newSourceIndex, map.sourcesContent[raw.sourceIndex]);
                            }
                        }
                        newSourceLine = raw.sourceLine;
                        newSourceCharacter = raw.sourceCharacter;
                        if (map.names && raw.nameIndex !== undefined) {
                            if (!nameIndexToNewNameIndexMap)
                                nameIndexToNewNameIndexMap = [];
                            newNameIndex = nameIndexToNewNameIndexMap[raw.nameIndex];
                            if (newNameIndex === undefined) {
                                nameIndexToNewNameIndexMap[raw.nameIndex] = newNameIndex = addName(map.names[raw.nameIndex]);
                            }
                        }
                    }
                    var rawGeneratedLine = raw.generatedLine - (start ? start.line : 0);
                    var newGeneratedLine = rawGeneratedLine + generatedLine;
                    var rawGeneratedCharacter = start && start.line === raw.generatedLine ? raw.generatedCharacter - start.character : raw.generatedCharacter;
                    var newGeneratedCharacter = rawGeneratedLine === 0 ? rawGeneratedCharacter + generatedCharacter : rawGeneratedCharacter;
                    addMapping(newGeneratedLine, newGeneratedCharacter, newSourceIndex, newSourceLine, newSourceCharacter, newNameIndex);
                }
                exit();
            }
            function shouldCommitMapping() {
                return !hasLast
                    || lastGeneratedLine !== pendingGeneratedLine
                    || lastGeneratedCharacter !== pendingGeneratedCharacter
                    || lastSourceIndex !== pendingSourceIndex
                    || lastSourceLine !== pendingSourceLine
                    || lastSourceCharacter !== pendingSourceCharacter
                    || lastNameIndex !== pendingNameIndex;
            }
            function commitPendingMapping() {
                if (!hasPending || !shouldCommitMapping()) {
                    return;
                }
                enter();
                // Line/Comma delimiters
                if (lastGeneratedLine < pendingGeneratedLine) {
                    // Emit line delimiters
                    do {
                        mappings += ";";
                        lastGeneratedLine++;
                        lastGeneratedCharacter = 0;
                    } while (lastGeneratedLine < pendingGeneratedLine);
                }
                else {
                    ts.Debug.assertEqual(lastGeneratedLine, pendingGeneratedLine, "generatedLine cannot backtrack");
                    // Emit comma to separate the entry
                    if (hasLast) {
                        mappings += ",";
                    }
                }
                // 1. Relative generated character
                mappings += base64VLQFormatEncode(pendingGeneratedCharacter - lastGeneratedCharacter);
                lastGeneratedCharacter = pendingGeneratedCharacter;
                if (hasPendingSource) {
                    // 2. Relative sourceIndex
                    mappings += base64VLQFormatEncode(pendingSourceIndex - lastSourceIndex);
                    lastSourceIndex = pendingSourceIndex;
                    // 3. Relative source line
                    mappings += base64VLQFormatEncode(pendingSourceLine - lastSourceLine);
                    lastSourceLine = pendingSourceLine;
                    // 4. Relative source character
                    mappings += base64VLQFormatEncode(pendingSourceCharacter - lastSourceCharacter);
                    lastSourceCharacter = pendingSourceCharacter;
                    if (hasPendingName) {
                        // 5. Relative nameIndex
                        mappings += base64VLQFormatEncode(pendingNameIndex - lastNameIndex);
                        lastNameIndex = pendingNameIndex;
                    }
                }
                hasLast = true;
                exit();
            }
            function toJSON() {
                commitPendingMapping();
                return {
                    version: 3,
                    file: file,
                    sourceRoot: sourceRoot,
                    sources: sources,
                    names: names,
                    mappings: mappings,
                    sourcesContent: sourcesContent,
                };
            }
        }
        ts.createSourceMapGenerator = createSourceMapGenerator;
        // Sometimes tools can see the following line as a source mapping url comment, so we mangle it a bit (the [M])
        var sourceMapCommentRegExp = /^\/\/[@#] source[M]appingURL=(.+)\s*$/;
        var whitespaceOrMapCommentRegExp = /^\s*(\/\/[@#] .*)?$/;
        function getLineInfo(text, lineStarts) {
            return {
                getLineCount: function () { return lineStarts.length; },
                getLineText: function (line) { return text.substring(lineStarts[line], lineStarts[line + 1]); }
            };
        }
        ts.getLineInfo = getLineInfo;
        /**
         * Tries to find the sourceMappingURL comment at the end of a file.
         */
        function tryGetSourceMappingURL(lineInfo) {
            for (var index = lineInfo.getLineCount() - 1; index >= 0; index--) {
                var line = lineInfo.getLineText(index);
                var comment = sourceMapCommentRegExp.exec(line);
                if (comment) {
                    return comment[1];
                }
                // If we see a non-whitespace/map comment-like line, break, to avoid scanning up the entire file
                else if (!line.match(whitespaceOrMapCommentRegExp)) {
                    break;
                }
            }
        }
        ts.tryGetSourceMappingURL = tryGetSourceMappingURL;
        /* eslint-disable no-null/no-null */
        function isStringOrNull(x) {
            return typeof x === "string" || x === null;
        }
        function isRawSourceMap(x) {
            return x !== null
                && typeof x === "object"
                && x.version === 3
                && typeof x.file === "string"
                && typeof x.mappings === "string"
                && ts.isArray(x.sources) && ts.every(x.sources, ts.isString)
                && (x.sourceRoot === undefined || x.sourceRoot === null || typeof x.sourceRoot === "string")
                && (x.sourcesContent === undefined || x.sourcesContent === null || ts.isArray(x.sourcesContent) && ts.every(x.sourcesContent, isStringOrNull))
                && (x.names === undefined || x.names === null || ts.isArray(x.names) && ts.every(x.names, ts.isString));
        }
        ts.isRawSourceMap = isRawSourceMap;
        /* eslint-enable no-null/no-null */
        function tryParseRawSourceMap(text) {
            try {
                var parsed = JSON.parse(text);
                if (isRawSourceMap(parsed)) {
                    return parsed;
                }
            }
            catch (_a) {
                // empty
            }
            return undefined;
        }
        ts.tryParseRawSourceMap = tryParseRawSourceMap;
        function decodeMappings(mappings) {
            var done = false;
            var pos = 0;
            var generatedLine = 0;
            var generatedCharacter = 0;
            var sourceIndex = 0;
            var sourceLine = 0;
            var sourceCharacter = 0;
            var nameIndex = 0;
            var error;
            return {
                get pos() { return pos; },
                get error() { return error; },
                get state() { return captureMapping(/*hasSource*/ true, /*hasName*/ true); },
                next: function () {
                    while (!done && pos < mappings.length) {
                        var ch = mappings.charCodeAt(pos);
                        if (ch === 59 /* semicolon */) {
                            // new line
                            generatedLine++;
                            generatedCharacter = 0;
                            pos++;
                            continue;
                        }
                        if (ch === 44 /* comma */) {
                            // Next entry is on same line - no action needed
                            pos++;
                            continue;
                        }
                        var hasSource = false;
                        var hasName = false;
                        generatedCharacter += base64VLQFormatDecode();
                        if (hasReportedError())
                            return stopIterating();
                        if (generatedCharacter < 0)
                            return setErrorAndStopIterating("Invalid generatedCharacter found");
                        if (!isSourceMappingSegmentEnd()) {
                            hasSource = true;
                            sourceIndex += base64VLQFormatDecode();
                            if (hasReportedError())
                                return stopIterating();
                            if (sourceIndex < 0)
                                return setErrorAndStopIterating("Invalid sourceIndex found");
                            if (isSourceMappingSegmentEnd())
                                return setErrorAndStopIterating("Unsupported Format: No entries after sourceIndex");
                            sourceLine += base64VLQFormatDecode();
                            if (hasReportedError())
                                return stopIterating();
                            if (sourceLine < 0)
                                return setErrorAndStopIterating("Invalid sourceLine found");
                            if (isSourceMappingSegmentEnd())
                                return setErrorAndStopIterating("Unsupported Format: No entries after sourceLine");
                            sourceCharacter += base64VLQFormatDecode();
                            if (hasReportedError())
                                return stopIterating();
                            if (sourceCharacter < 0)
                                return setErrorAndStopIterating("Invalid sourceCharacter found");
                            if (!isSourceMappingSegmentEnd()) {
                                hasName = true;
                                nameIndex += base64VLQFormatDecode();
                                if (hasReportedError())
                                    return stopIterating();
                                if (nameIndex < 0)
                                    return setErrorAndStopIterating("Invalid nameIndex found");
                                if (!isSourceMappingSegmentEnd())
                                    return setErrorAndStopIterating("Unsupported Error Format: Entries after nameIndex");
                            }
                        }
                        return { value: captureMapping(hasSource, hasName), done: done };
                    }
                    return stopIterating();
                }
            };
            function captureMapping(hasSource, hasName) {
                return {
                    generatedLine: generatedLine,
                    generatedCharacter: generatedCharacter,
                    sourceIndex: hasSource ? sourceIndex : undefined,
                    sourceLine: hasSource ? sourceLine : undefined,
                    sourceCharacter: hasSource ? sourceCharacter : undefined,
                    nameIndex: hasName ? nameIndex : undefined
                };
            }
            function stopIterating() {
                done = true;
                return { value: undefined, done: true };
            }
            function setError(message) {
                if (error === undefined) {
                    error = message;
                }
            }
            function setErrorAndStopIterating(message) {
                setError(message);
                return stopIterating();
            }
            function hasReportedError() {
                return error !== undefined;
            }
            function isSourceMappingSegmentEnd() {
                return (pos === mappings.length ||
                    mappings.charCodeAt(pos) === 44 /* comma */ ||
                    mappings.charCodeAt(pos) === 59 /* semicolon */);
            }
            function base64VLQFormatDecode() {
                var moreDigits = true;
                var shiftCount = 0;
                var value = 0;
                for (; moreDigits; pos++) {
                    if (pos >= mappings.length)
                        return setError("Error in decoding base64VLQFormatDecode, past the mapping string"), -1;
                    // 6 digit number
                    var currentByte = base64FormatDecode(mappings.charCodeAt(pos));
                    if (currentByte === -1)
                        return setError("Invalid character in VLQ"), -1;
                    // If msb is set, we still have more bits to continue
                    moreDigits = (currentByte & 32) !== 0;
                    // least significant 5 bits are the next msbs in the final value.
                    value = value | ((currentByte & 31) << shiftCount);
                    shiftCount += 5;
                }
                // Least significant bit if 1 represents negative and rest of the msb is actual absolute value
                if ((value & 1) === 0) {
                    // + number
                    value = value >> 1;
                }
                else {
                    // - number
                    value = value >> 1;
                    value = -value;
                }
                return value;
            }
        }
        ts.decodeMappings = decodeMappings;
        function sameMapping(left, right) {
            return left === right
                || left.generatedLine === right.generatedLine
                    && left.generatedCharacter === right.generatedCharacter
                    && left.sourceIndex === right.sourceIndex
                    && left.sourceLine === right.sourceLine
                    && left.sourceCharacter === right.sourceCharacter
                    && left.nameIndex === right.nameIndex;
        }
        ts.sameMapping = sameMapping;
        function isSourceMapping(mapping) {
            return mapping.sourceIndex !== undefined
                && mapping.sourceLine !== undefined
                && mapping.sourceCharacter !== undefined;
        }
        ts.isSourceMapping = isSourceMapping;
        function base64FormatEncode(value) {
            return value >= 0 && value < 26 ? 65 /* A */ + value :
                value >= 26 && value < 52 ? 97 /* a */ + value - 26 :
                    value >= 52 && value < 62 ? 48 /* _0 */ + value - 52 :
                        value === 62 ? 43 /* plus */ :
                            value === 63 ? 47 /* slash */ :
                                ts.Debug.fail(value + ": not a base64 value");
        }
        function base64FormatDecode(ch) {
            return ch >= 65 /* A */ && ch <= 90 /* Z */ ? ch - 65 /* A */ :
                ch >= 97 /* a */ && ch <= 122 /* z */ ? ch - 97 /* a */ + 26 :
                    ch >= 48 /* _0 */ && ch <= 57 /* _9 */ ? ch - 48 /* _0 */ + 52 :
                        ch === 43 /* plus */ ? 62 :
                            ch === 47 /* slash */ ? 63 :
                                -1;
        }
        function base64VLQFormatEncode(inValue) {
            // Add a new least significant bit that has the sign of the value.
            // if negative number the least significant bit that gets added to the number has value 1
            // else least significant bit value that gets added is 0
            // eg. -1 changes to binary : 01 [1] => 3
            //     +1 changes to binary : 01 [0] => 2
            if (inValue < 0) {
                inValue = ((-inValue) << 1) + 1;
            }
            else {
                inValue = inValue << 1;
            }
            // Encode 5 bits at a time starting from least significant bits
            var encodedStr = "";
            do {
                var currentDigit = inValue & 31; // 11111
                inValue = inValue >> 5;
                if (inValue > 0) {
                    // There are still more digits to decode, set the msb (6th bit)
                    currentDigit = currentDigit | 32;
                }
                encodedStr = encodedStr + String.fromCharCode(base64FormatEncode(currentDigit));
            } while (inValue > 0);
            return encodedStr;
        }
        function isSourceMappedPosition(value) {
            return value.sourceIndex !== undefined
                && value.sourcePosition !== undefined;
        }
        function sameMappedPosition(left, right) {
            return left.generatedPosition === right.generatedPosition
                && left.sourceIndex === right.sourceIndex
                && left.sourcePosition === right.sourcePosition;
        }
        function compareSourcePositions(left, right) {
            // Compares sourcePosition without comparing sourceIndex
            // since the mappings are grouped by sourceIndex
            ts.Debug.assert(left.sourceIndex === right.sourceIndex);
            return ts.compareValues(left.sourcePosition, right.sourcePosition);
        }
        function compareGeneratedPositions(left, right) {
            return ts.compareValues(left.generatedPosition, right.generatedPosition);
        }
        function getSourcePositionOfMapping(value) {
            return value.sourcePosition;
        }
        function getGeneratedPositionOfMapping(value) {
            return value.generatedPosition;
        }
        function createDocumentPositionMapper(host, map, mapPath) {
            var mapDirectory = ts.getDirectoryPath(mapPath);
            var sourceRoot = map.sourceRoot ? ts.getNormalizedAbsolutePath(map.sourceRoot, mapDirectory) : mapDirectory;
            var generatedAbsoluteFilePath = ts.getNormalizedAbsolutePath(map.file, mapDirectory);
            var generatedFile = host.getSourceFileLike(generatedAbsoluteFilePath);
            var sourceFileAbsolutePaths = map.sources.map(function (source) { return ts.getNormalizedAbsolutePath(source, sourceRoot); });
            var sourceToSourceIndexMap = ts.createMapFromEntries(sourceFileAbsolutePaths.map(function (source, i) { return [host.getCanonicalFileName(source), i]; }));
            var decodedMappings;
            var generatedMappings;
            var sourceMappings;
            return {
                getSourcePosition: getSourcePosition,
                getGeneratedPosition: getGeneratedPosition
            };
            function processMapping(mapping) {
                var generatedPosition = generatedFile !== undefined
                    ? ts.getPositionOfLineAndCharacter(generatedFile, mapping.generatedLine, mapping.generatedCharacter, /*allowEdits*/ true)
                    : -1;
                var source;
                var sourcePosition;
                if (isSourceMapping(mapping)) {
                    var sourceFile = host.getSourceFileLike(sourceFileAbsolutePaths[mapping.sourceIndex]);
                    source = map.sources[mapping.sourceIndex];
                    sourcePosition = sourceFile !== undefined
                        ? ts.getPositionOfLineAndCharacter(sourceFile, mapping.sourceLine, mapping.sourceCharacter, /*allowEdits*/ true)
                        : -1;
                }
                return {
                    generatedPosition: generatedPosition,
                    source: source,
                    sourceIndex: mapping.sourceIndex,
                    sourcePosition: sourcePosition,
                    nameIndex: mapping.nameIndex
                };
            }
            function getDecodedMappings() {
                if (decodedMappings === undefined) {
                    var decoder = decodeMappings(map.mappings);
                    var mappings = ts.arrayFrom(decoder, processMapping);
                    if (decoder.error !== undefined) {
                        if (host.log) {
                            host.log("Encountered error while decoding sourcemap: " + decoder.error);
                        }
                        decodedMappings = ts.emptyArray;
                    }
                    else {
                        decodedMappings = mappings;
                    }
                }
                return decodedMappings;
            }
            function getSourceMappings(sourceIndex) {
                if (sourceMappings === undefined) {
                    var lists = [];
                    for (var _i = 0, _a = getDecodedMappings(); _i < _a.length; _i++) {
                        var mapping = _a[_i];
                        if (!isSourceMappedPosition(mapping))
                            continue;
                        var list = lists[mapping.sourceIndex];
                        if (!list)
                            lists[mapping.sourceIndex] = list = [];
                        list.push(mapping);
                    }
                    sourceMappings = lists.map(function (list) { return ts.sortAndDeduplicate(list, compareSourcePositions, sameMappedPosition); });
                }
                return sourceMappings[sourceIndex];
            }
            function getGeneratedMappings() {
                if (generatedMappings === undefined) {
                    var list = [];
                    for (var _i = 0, _a = getDecodedMappings(); _i < _a.length; _i++) {
                        var mapping = _a[_i];
                        list.push(mapping);
                    }
                    generatedMappings = ts.sortAndDeduplicate(list, compareGeneratedPositions, sameMappedPosition);
                }
                return generatedMappings;
            }
            function getGeneratedPosition(loc) {
                var sourceIndex = sourceToSourceIndexMap.get(host.getCanonicalFileName(loc.fileName));
                if (sourceIndex === undefined)
                    return loc;
                var sourceMappings = getSourceMappings(sourceIndex);
                if (!ts.some(sourceMappings))
                    return loc;
                var targetIndex = ts.binarySearchKey(sourceMappings, loc.pos, getSourcePositionOfMapping, ts.compareValues);
                if (targetIndex < 0) {
                    // if no exact match, closest is 2's complement of result
                    targetIndex = ~targetIndex;
                }
                var mapping = sourceMappings[targetIndex];
                if (mapping === undefined || mapping.sourceIndex !== sourceIndex) {
                    return loc;
                }
                return { fileName: generatedAbsoluteFilePath, pos: mapping.generatedPosition }; // Closest pos
            }
            function getSourcePosition(loc) {
                var generatedMappings = getGeneratedMappings();
                if (!ts.some(generatedMappings))
                    return loc;
                var targetIndex = ts.binarySearchKey(generatedMappings, loc.pos, getGeneratedPositionOfMapping, ts.compareValues);
                if (targetIndex < 0) {
                    // if no exact match, closest is 2's complement of result
                    targetIndex = ~targetIndex;
                }
                var mapping = generatedMappings[targetIndex];
                if (mapping === undefined || !isSourceMappedPosition(mapping)) {
                    return loc;
                }
                return { fileName: sourceFileAbsolutePaths[mapping.sourceIndex], pos: mapping.sourcePosition }; // Closest pos
            }
        }
        ts.createDocumentPositionMapper = createDocumentPositionMapper;
        ts.identitySourceMapConsumer = {
            getSourcePosition: ts.identity,
            getGeneratedPosition: ts.identity
        };
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        function getOriginalNodeId(node) {
            node = ts.getOriginalNode(node);
            return node ? ts.getNodeId(node) : 0;
        }
        ts.getOriginalNodeId = getOriginalNodeId;
        function containsDefaultReference(node) {
            if (!node)
                return false;
            if (!ts.isNamedImports(node))
                return false;
            return ts.some(node.elements, isNamedDefaultReference);
        }
        function isNamedDefaultReference(e) {
            return e.propertyName !== undefined && e.propertyName.escapedText === "default" /* Default */;
        }
        function chainBundle(transformSourceFile) {
            return transformSourceFileOrBundle;
            function transformSourceFileOrBundle(node) {
                return node.kind === 290 /* SourceFile */ ? transformSourceFile(node) : transformBundle(node);
            }
            function transformBundle(node) {
                return ts.createBundle(ts.map(node.sourceFiles, transformSourceFile), node.prepends);
            }
        }
        ts.chainBundle = chainBundle;
        function getExportNeedsImportStarHelper(node) {
            return !!ts.getNamespaceDeclarationNode(node);
        }
        ts.getExportNeedsImportStarHelper = getExportNeedsImportStarHelper;
        function getImportNeedsImportStarHelper(node) {
            if (!!ts.getNamespaceDeclarationNode(node)) {
                return true;
            }
            var bindings = node.importClause && node.importClause.namedBindings;
            if (!bindings) {
                return false;
            }
            if (!ts.isNamedImports(bindings))
                return false;
            var defaultRefCount = 0;
            for (var _i = 0, _a = bindings.elements; _i < _a.length; _i++) {
                var binding = _a[_i];
                if (isNamedDefaultReference(binding)) {
                    defaultRefCount++;
                }
            }
            // Import star is required if there's default named refs mixed with non-default refs, or if theres non-default refs and it has a default import
            return (defaultRefCount > 0 && defaultRefCount !== bindings.elements.length) || (!!(bindings.elements.length - defaultRefCount) && ts.isDefaultImport(node));
        }
        ts.getImportNeedsImportStarHelper = getImportNeedsImportStarHelper;
        function getImportNeedsImportDefaultHelper(node) {
            // Import default is needed if there's a default import or a default ref and no other refs (meaning an import star helper wasn't requested)
            return !getImportNeedsImportStarHelper(node) && (ts.isDefaultImport(node) || (!!node.importClause && ts.isNamedImports(node.importClause.namedBindings) && containsDefaultReference(node.importClause.namedBindings))); // TODO: GH#18217
        }
        ts.getImportNeedsImportDefaultHelper = getImportNeedsImportDefaultHelper;
        function collectExternalModuleInfo(sourceFile, resolver, compilerOptions) {
            var externalImports = [];
            var exportSpecifiers = ts.createMultiMap();
            var exportedBindings = [];
            var uniqueExports = ts.createMap();
            var exportedNames;
            var hasExportDefault = false;
            var exportEquals;
            var hasExportStarsToExportValues = false;
            var hasImportStar = false;
            var hasImportDefault = false;
            for (var _i = 0, _a = sourceFile.statements; _i < _a.length; _i++) {
                var node = _a[_i];
                switch (node.kind) {
                    case 254 /* ImportDeclaration */:
                        // import "mod"
                        // import x from "mod"
                        // import * as x from "mod"
                        // import { x, y } from "mod"
                        externalImports.push(node);
                        if (!hasImportStar && getImportNeedsImportStarHelper(node)) {
                            hasImportStar = true;
                        }
                        if (!hasImportDefault && getImportNeedsImportDefaultHelper(node)) {
                            hasImportDefault = true;
                        }
                        break;
                    case 253 /* ImportEqualsDeclaration */:
                        if (node.moduleReference.kind === 265 /* ExternalModuleReference */) {
                            // import x = require("mod")
                            externalImports.push(node);
                        }
                        break;
                    case 260 /* ExportDeclaration */:
                        if (node.moduleSpecifier) {
                            if (!node.exportClause) {
                                // export * from "mod"
                                externalImports.push(node);
                                hasExportStarsToExportValues = true;
                            }
                            else {
                                // export * as ns from "mod"
                                // export { x, y } from "mod"
                                externalImports.push(node);
                            }
                        }
                        else {
                            // export { x, y }
                            for (var _b = 0, _c = ts.cast(node.exportClause, ts.isNamedExports).elements; _b < _c.length; _b++) {
                                var specifier = _c[_b];
                                if (!uniqueExports.get(ts.idText(specifier.name))) {
                                    var name = specifier.propertyName || specifier.name;
                                    exportSpecifiers.add(ts.idText(name), specifier);
                                    var decl = resolver.getReferencedImportDeclaration(name)
                                        || resolver.getReferencedValueDeclaration(name);
                                    if (decl) {
                                        multiMapSparseArrayAdd(exportedBindings, getOriginalNodeId(decl), specifier.name);
                                    }
                                    uniqueExports.set(ts.idText(specifier.name), true);
                                    exportedNames = ts.append(exportedNames, specifier.name);
                                }
                            }
                        }
                        break;
                    case 259 /* ExportAssignment */:
                        if (node.isExportEquals && !exportEquals) {
                            // export = x
                            exportEquals = node;
                        }
                        break;
                    case 225 /* VariableStatement */:
                        if (ts.hasModifier(node, 1 /* Export */)) {
                            for (var _d = 0, _e = node.declarationList.declarations; _d < _e.length; _d++) {
                                var decl = _e[_d];
                                exportedNames = collectExportedVariableInfo(decl, uniqueExports, exportedNames);
                            }
                        }
                        break;
                    case 244 /* FunctionDeclaration */:
                        if (ts.hasModifier(node, 1 /* Export */)) {
                            if (ts.hasModifier(node, 512 /* Default */)) {
                                // export default function() { }
                                if (!hasExportDefault) {
                                    multiMapSparseArrayAdd(exportedBindings, getOriginalNodeId(node), ts.getDeclarationName(node));
                                    hasExportDefault = true;
                                }
                            }
                            else {
                                // export function x() { }
                                var name = node.name;
                                if (!uniqueExports.get(ts.idText(name))) {
                                    multiMapSparseArrayAdd(exportedBindings, getOriginalNodeId(node), name);
                                    uniqueExports.set(ts.idText(name), true);
                                    exportedNames = ts.append(exportedNames, name);
                                }
                            }
                        }
                        break;
                    case 245 /* ClassDeclaration */:
                        if (ts.hasModifier(node, 1 /* Export */)) {
                            if (ts.hasModifier(node, 512 /* Default */)) {
                                // export default class { }
                                if (!hasExportDefault) {
                                    multiMapSparseArrayAdd(exportedBindings, getOriginalNodeId(node), ts.getDeclarationName(node));
                                    hasExportDefault = true;
                                }
                            }
                            else {
                                // export class x { }
                                var name = node.name;
                                if (name && !uniqueExports.get(ts.idText(name))) {
                                    multiMapSparseArrayAdd(exportedBindings, getOriginalNodeId(node), name);
                                    uniqueExports.set(ts.idText(name), true);
                                    exportedNames = ts.append(exportedNames, name);
                                }
                            }
                        }
                        break;
                }
            }
            var externalHelpersImportDeclaration = ts.createExternalHelpersImportDeclarationIfNeeded(sourceFile, compilerOptions, hasExportStarsToExportValues, hasImportStar, hasImportDefault);
            if (externalHelpersImportDeclaration) {
                externalImports.unshift(externalHelpersImportDeclaration);
            }
            return { externalImports: externalImports, exportSpecifiers: exportSpecifiers, exportEquals: exportEquals, hasExportStarsToExportValues: hasExportStarsToExportValues, exportedBindings: exportedBindings, exportedNames: exportedNames, externalHelpersImportDeclaration: externalHelpersImportDeclaration };
        }
        ts.collectExternalModuleInfo = collectExternalModuleInfo;
        function collectExportedVariableInfo(decl, uniqueExports, exportedNames) {
            if (ts.isBindingPattern(decl.name)) {
                for (var _i = 0, _a = decl.name.elements; _i < _a.length; _i++) {
                    var element = _a[_i];
                    if (!ts.isOmittedExpression(element)) {
                        exportedNames = collectExportedVariableInfo(element, uniqueExports, exportedNames);
                    }
                }
            }
            else if (!ts.isGeneratedIdentifier(decl.name)) {
                var text = ts.idText(decl.name);
                if (!uniqueExports.get(text)) {
                    uniqueExports.set(text, true);
                    exportedNames = ts.append(exportedNames, decl.name);
                }
            }
            return exportedNames;
        }
        /** Use a sparse array as a multi-map. */
        function multiMapSparseArrayAdd(map, key, value) {
            var values = map[key];
            if (values) {
                values.push(value);
            }
            else {
                map[key] = values = [value];
            }
            return values;
        }
        /**
         * Used in the module transformer to check if an expression is reasonably without sideeffect,
         *  and thus better to copy into multiple places rather than to cache in a temporary variable
         *  - this is mostly subjective beyond the requirement that the expression not be sideeffecting
         */
        function isSimpleCopiableExpression(expression) {
            return ts.isStringLiteralLike(expression) ||
                expression.kind === 8 /* NumericLiteral */ ||
                ts.isKeyword(expression.kind) ||
                ts.isIdentifier(expression);
        }
        ts.isSimpleCopiableExpression = isSimpleCopiableExpression;
        /**
         * A simple inlinable expression is an expression which can be copied into multiple locations
         * without risk of repeating any sideeffects and whose value could not possibly change between
         * any such locations
         */
        function isSimpleInlineableExpression(expression) {
            return !ts.isIdentifier(expression) && isSimpleCopiableExpression(expression) ||
                ts.isWellKnownSymbolSyntactically(expression);
        }
        ts.isSimpleInlineableExpression = isSimpleInlineableExpression;
        function isCompoundAssignment(kind) {
            return kind >= 63 /* FirstCompoundAssignment */
                && kind <= 74 /* LastCompoundAssignment */;
        }
        ts.isCompoundAssignment = isCompoundAssignment;
        function getNonAssignmentOperatorForCompoundAssignment(kind) {
            switch (kind) {
                case 63 /* PlusEqualsToken */: return 39 /* PlusToken */;
                case 64 /* MinusEqualsToken */: return 40 /* MinusToken */;
                case 65 /* AsteriskEqualsToken */: return 41 /* AsteriskToken */;
                case 66 /* AsteriskAsteriskEqualsToken */: return 42 /* AsteriskAsteriskToken */;
                case 67 /* SlashEqualsToken */: return 43 /* SlashToken */;
                case 68 /* PercentEqualsToken */: return 44 /* PercentToken */;
                case 69 /* LessThanLessThanEqualsToken */: return 47 /* LessThanLessThanToken */;
                case 70 /* GreaterThanGreaterThanEqualsToken */: return 48 /* GreaterThanGreaterThanToken */;
                case 71 /* GreaterThanGreaterThanGreaterThanEqualsToken */: return 49 /* GreaterThanGreaterThanGreaterThanToken */;
                case 72 /* AmpersandEqualsToken */: return 50 /* AmpersandToken */;
                case 73 /* BarEqualsToken */: return 51 /* BarToken */;
                case 74 /* CaretEqualsToken */: return 52 /* CaretToken */;
            }
        }
        ts.getNonAssignmentOperatorForCompoundAssignment = getNonAssignmentOperatorForCompoundAssignment;
        /**
         * Adds super call and preceding prologue directives into the list of statements.
         *
         * @param ctor The constructor node.
         * @param result The list of statements.
         * @param visitor The visitor to apply to each node added to the result array.
         * @returns index of the statement that follows super call
         */
        function addPrologueDirectivesAndInitialSuperCall(ctor, result, visitor) {
            if (ctor.body) {
                var statements = ctor.body.statements;
                // add prologue directives to the list (if any)
                var index = ts.addPrologue(result, statements, /*ensureUseStrict*/ false, visitor);
                if (index === statements.length) {
                    // list contains nothing but prologue directives (or empty) - exit
                    return index;
                }
                var superIndex = ts.findIndex(statements, function (s) { return ts.isExpressionStatement(s) && ts.isSuperCall(s.expression); }, index);
                if (superIndex > -1) {
                    for (var i = index; i <= superIndex; i++) {
                        result.push(ts.visitNode(statements[i], visitor, ts.isStatement));
                    }
                    return superIndex + 1;
                }
                return index;
            }
            return 0;
        }
        ts.addPrologueDirectivesAndInitialSuperCall = addPrologueDirectivesAndInitialSuperCall;
        /**
         * @param input Template string input strings
         * @param args Names which need to be made file-level unique
         */
        function helperString(input) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            return function (uniqueName) {
                var result = "";
                for (var i = 0; i < args.length; i++) {
                    result += input[i];
                    result += uniqueName(args[i]);
                }
                result += input[input.length - 1];
                return result;
            };
        }
        ts.helperString = helperString;
        /**
         * Gets all the static or all the instance property declarations of a class
         *
         * @param node The class node.
         * @param isStatic A value indicating whether to get properties from the static or instance side of the class.
         */
        function getProperties(node, requireInitializer, isStatic) {
            return ts.filter(node.members, function (m) { return isInitializedOrStaticProperty(m, requireInitializer, isStatic); });
        }
        ts.getProperties = getProperties;
        /**
         * Is a class element either a static or an instance property declaration with an initializer?
         *
         * @param member The class element node.
         * @param isStatic A value indicating whether the member should be a static or instance member.
         */
        function isInitializedOrStaticProperty(member, requireInitializer, isStatic) {
            return ts.isPropertyDeclaration(member)
                && (!!member.initializer || !requireInitializer)
                && ts.hasStaticModifier(member) === isStatic;
        }
        /**
         * Gets a value indicating whether a class element is either a static or an instance property declaration with an initializer.
         *
         * @param member The class element node.
         * @param isStatic A value indicating whether the member should be a static or instance member.
         */
        function isInitializedProperty(member) {
            return member.kind === 159 /* PropertyDeclaration */
                && member.initializer !== undefined;
        }
        ts.isInitializedProperty = isInitializedProperty;
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        var FlattenLevel;
        (function (FlattenLevel) {
            FlattenLevel[FlattenLevel["All"] = 0] = "All";
            FlattenLevel[FlattenLevel["ObjectRest"] = 1] = "ObjectRest";
        })(FlattenLevel = ts.FlattenLevel || (ts.FlattenLevel = {}));
        /**
         * Flattens a DestructuringAssignment or a VariableDeclaration to an expression.
         *
         * @param node The node to flatten.
         * @param visitor An optional visitor used to visit initializers.
         * @param context The transformation context.
         * @param level Indicates the extent to which flattening should occur.
         * @param needsValue An optional value indicating whether the value from the right-hand-side of
         * the destructuring assignment is needed as part of a larger expression.
         * @param createAssignmentCallback An optional callback used to create the assignment expression.
         */
        function flattenDestructuringAssignment(node, visitor, context, level, needsValue, createAssignmentCallback) {
            var location = node;
            var value;
            if (ts.isDestructuringAssignment(node)) {
                value = node.right;
                while (ts.isEmptyArrayLiteral(node.left) || ts.isEmptyObjectLiteral(node.left)) {
                    if (ts.isDestructuringAssignment(value)) {
                        location = node = value;
                        value = node.right;
                    }
                    else {
                        return ts.visitNode(value, visitor, ts.isExpression);
                    }
                }
            }
            var expressions;
            var flattenContext = {
                context: context,
                level: level,
                downlevelIteration: !!context.getCompilerOptions().downlevelIteration,
                hoistTempVariables: true,
                emitExpression: emitExpression,
                emitBindingOrAssignment: emitBindingOrAssignment,
                createArrayBindingOrAssignmentPattern: makeArrayAssignmentPattern,
                createObjectBindingOrAssignmentPattern: makeObjectAssignmentPattern,
                createArrayBindingOrAssignmentElement: makeAssignmentElement,
                visitor: visitor
            };
            if (value) {
                value = ts.visitNode(value, visitor, ts.isExpression);
                if (ts.isIdentifier(value) && bindingOrAssignmentElementAssignsToName(node, value.escapedText) ||
                    bindingOrAssignmentElementContainsNonLiteralComputedName(node)) {
                    // If the right-hand value of the assignment is also an assignment target then
                    // we need to cache the right-hand value.
                    value = ensureIdentifier(flattenContext, value, /*reuseIdentifierExpressions*/ false, location);
                }
                else if (needsValue) {
                    // If the right-hand value of the destructuring assignment needs to be preserved (as
                    // is the case when the destructuring assignment is part of a larger expression),
                    // then we need to cache the right-hand value.
                    //
                    // The source map location for the assignment should point to the entire binary
                    // expression.
                    value = ensureIdentifier(flattenContext, value, /*reuseIdentifierExpressions*/ true, location);
                }
                else if (ts.nodeIsSynthesized(node)) {
                    // Generally, the source map location for a destructuring assignment is the root
                    // expression.
                    //
                    // However, if the root expression is synthesized (as in the case
                    // of the initializer when transforming a ForOfStatement), then the source map
                    // location should point to the right-hand value of the expression.
                    location = value;
                }
            }
            flattenBindingOrAssignmentElement(flattenContext, node, value, location, /*skipInitializer*/ ts.isDestructuringAssignment(node));
            if (value && needsValue) {
                if (!ts.some(expressions)) {
                    return value;
                }
                expressions.push(value);
            }
            return ts.aggregateTransformFlags(ts.inlineExpressions(expressions)) || ts.createOmittedExpression();
            function emitExpression(expression) {
                // NOTE: this completely disables source maps, but aligns with the behavior of
                //       `emitAssignment` in the old emitter.
                ts.setEmitFlags(expression, 64 /* NoNestedSourceMaps */);
                ts.aggregateTransformFlags(expression);
                expressions = ts.append(expressions, expression);
            }
            function emitBindingOrAssignment(target, value, location, original) {
                ts.Debug.assertNode(target, createAssignmentCallback ? ts.isIdentifier : ts.isExpression);
                var expression = createAssignmentCallback
                    ? createAssignmentCallback(target, value, location)
                    : ts.setTextRange(ts.createAssignment(ts.visitNode(target, visitor, ts.isExpression), value), location);
                expression.original = original;
                emitExpression(expression);
            }
        }
        ts.flattenDestructuringAssignment = flattenDestructuringAssignment;
        function bindingOrAssignmentElementAssignsToName(element, escapedName) {
            var target = ts.getTargetOfBindingOrAssignmentElement(element); // TODO: GH#18217
            if (ts.isBindingOrAssignmentPattern(target)) {
                return bindingOrAssignmentPatternAssignsToName(target, escapedName);
            }
            else if (ts.isIdentifier(target)) {
                return target.escapedText === escapedName;
            }
            return false;
        }
        function bindingOrAssignmentPatternAssignsToName(pattern, escapedName) {
            var elements = ts.getElementsOfBindingOrAssignmentPattern(pattern);
            for (var _i = 0, elements_3 = elements; _i < elements_3.length; _i++) {
                var element = elements_3[_i];
                if (bindingOrAssignmentElementAssignsToName(element, escapedName)) {
                    return true;
                }
            }
            return false;
        }
        function bindingOrAssignmentElementContainsNonLiteralComputedName(element) {
            var propertyName = ts.tryGetPropertyNameOfBindingOrAssignmentElement(element);
            if (propertyName && ts.isComputedPropertyName(propertyName) && !ts.isLiteralExpression(propertyName.expression)) {
                return true;
            }
            var target = ts.getTargetOfBindingOrAssignmentElement(element);
            return !!target && ts.isBindingOrAssignmentPattern(target) && bindingOrAssignmentPatternContainsNonLiteralComputedName(target);
        }
        function bindingOrAssignmentPatternContainsNonLiteralComputedName(pattern) {
            return !!ts.forEach(ts.getElementsOfBindingOrAssignmentPattern(pattern), bindingOrAssignmentElementContainsNonLiteralComputedName);
        }
        /**
         * Flattens a VariableDeclaration or ParameterDeclaration to one or more variable declarations.
         *
         * @param node The node to flatten.
         * @param visitor An optional visitor used to visit initializers.
         * @param context The transformation context.
         * @param boundValue The value bound to the declaration.
         * @param skipInitializer A value indicating whether to ignore the initializer of `node`.
         * @param hoistTempVariables Indicates whether temporary variables should not be recorded in-line.
         * @param level Indicates the extent to which flattening should occur.
         */
        function flattenDestructuringBinding(node, visitor, context, level, rval, hoistTempVariables, skipInitializer) {
            if (hoistTempVariables === void 0) {
                hoistTempVariables = false;
            }
            var pendingExpressions;
            var pendingDeclarations = [];
            var declarations = [];
            var flattenContext = {
                context: context,
                level: level,
                downlevelIteration: !!context.getCompilerOptions().downlevelIteration,
                hoistTempVariables: hoistTempVariables,
                emitExpression: emitExpression,
                emitBindingOrAssignment: emitBindingOrAssignment,
                createArrayBindingOrAssignmentPattern: makeArrayBindingPattern,
                createObjectBindingOrAssignmentPattern: makeObjectBindingPattern,
                createArrayBindingOrAssignmentElement: makeBindingElement,
                visitor: visitor
            };
            if (ts.isVariableDeclaration(node)) {
                var initializer = ts.getInitializerOfBindingOrAssignmentElement(node);
                if (initializer && (ts.isIdentifier(initializer) && bindingOrAssignmentElementAssignsToName(node, initializer.escapedText) ||
                    bindingOrAssignmentElementContainsNonLiteralComputedName(node))) {
                    // If the right-hand value of the assignment is also an assignment target then
                    // we need to cache the right-hand value.
                    initializer = ensureIdentifier(flattenContext, initializer, /*reuseIdentifierExpressions*/ false, initializer);
                    node = ts.updateVariableDeclaration(node, node.name, node.type, initializer);
                }
            }
            flattenBindingOrAssignmentElement(flattenContext, node, rval, node, skipInitializer);
            if (pendingExpressions) {
                var temp = ts.createTempVariable(/*recordTempVariable*/ undefined);
                if (hoistTempVariables) {
                    var value = ts.inlineExpressions(pendingExpressions);
                    pendingExpressions = undefined;
                    emitBindingOrAssignment(temp, value, /*location*/ undefined, /*original*/ undefined);
                }
                else {
                    context.hoistVariableDeclaration(temp);
                    var pendingDeclaration = ts.last(pendingDeclarations);
                    pendingDeclaration.pendingExpressions = ts.append(pendingDeclaration.pendingExpressions, ts.createAssignment(temp, pendingDeclaration.value));
                    ts.addRange(pendingDeclaration.pendingExpressions, pendingExpressions);
                    pendingDeclaration.value = temp;
                }
            }
            for (var _i = 0, pendingDeclarations_1 = pendingDeclarations; _i < pendingDeclarations_1.length; _i++) {
                var _a = pendingDeclarations_1[_i], pendingExpressions_1 = _a.pendingExpressions, name = _a.name, value = _a.value, location = _a.location, original = _a.original;
                var variable = ts.createVariableDeclaration(name, 
                /*type*/ undefined, pendingExpressions_1 ? ts.inlineExpressions(ts.append(pendingExpressions_1, value)) : value);
                variable.original = original;
                ts.setTextRange(variable, location);
                if (ts.isIdentifier(name)) {
                    ts.setEmitFlags(variable, 64 /* NoNestedSourceMaps */);
                }
                ts.aggregateTransformFlags(variable);
                declarations.push(variable);
            }
            return declarations;
            function emitExpression(value) {
                pendingExpressions = ts.append(pendingExpressions, value);
            }
            function emitBindingOrAssignment(target, value, location, original) {
                ts.Debug.assertNode(target, ts.isBindingName);
                if (pendingExpressions) {
                    value = ts.inlineExpressions(ts.append(pendingExpressions, value));
                    pendingExpressions = undefined;
                }
                pendingDeclarations.push({ pendingExpressions: pendingExpressions, name: target, value: value, location: location, original: original });
            }
        }
        ts.flattenDestructuringBinding = flattenDestructuringBinding;
        /**
         * Flattens a BindingOrAssignmentElement into zero or more bindings or assignments.
         *
         * @param flattenContext Options used to control flattening.
         * @param element The element to flatten.
         * @param value The current RHS value to assign to the element.
         * @param location The location to use for source maps and comments.
         * @param skipInitializer An optional value indicating whether to include the initializer
         * for the element.
         */
        function flattenBindingOrAssignmentElement(flattenContext, element, value, location, skipInitializer) {
            if (!skipInitializer) {
                var initializer = ts.visitNode(ts.getInitializerOfBindingOrAssignmentElement(element), flattenContext.visitor, ts.isExpression);
                if (initializer) {
                    // Combine value and initializer
                    value = value ? createDefaultValueCheck(flattenContext, value, initializer, location) : initializer;
                }
                else if (!value) {
                    // Use 'void 0' in absence of value and initializer
                    value = ts.createVoidZero();
                }
            }
            var bindingTarget = ts.getTargetOfBindingOrAssignmentElement(element); // TODO: GH#18217
            if (ts.isObjectBindingOrAssignmentPattern(bindingTarget)) {
                flattenObjectBindingOrAssignmentPattern(flattenContext, element, bindingTarget, value, location);
            }
            else if (ts.isArrayBindingOrAssignmentPattern(bindingTarget)) {
                flattenArrayBindingOrAssignmentPattern(flattenContext, element, bindingTarget, value, location);
            }
            else {
                flattenContext.emitBindingOrAssignment(bindingTarget, value, location, /*original*/ element); // TODO: GH#18217
            }
        }
        /**
         * Flattens an ObjectBindingOrAssignmentPattern into zero or more bindings or assignments.
         *
         * @param flattenContext Options used to control flattening.
         * @param parent The parent element of the pattern.
         * @param pattern The ObjectBindingOrAssignmentPattern to flatten.
         * @param value The current RHS value to assign to the element.
         * @param location The location to use for source maps and comments.
         */
        function flattenObjectBindingOrAssignmentPattern(flattenContext, parent, pattern, value, location) {
            var elements = ts.getElementsOfBindingOrAssignmentPattern(pattern);
            var numElements = elements.length;
            if (numElements !== 1) {
                // For anything other than a single-element destructuring we need to generate a temporary
                // to ensure value is evaluated exactly once. Additionally, if we have zero elements
                // we need to emit *something* to ensure that in case a 'var' keyword was already emitted,
                // so in that case, we'll intentionally create that temporary.
                var reuseIdentifierExpressions = !ts.isDeclarationBindingElement(parent) || numElements !== 0;
                value = ensureIdentifier(flattenContext, value, reuseIdentifierExpressions, location);
            }
            var bindingElements;
            var computedTempVariables;
            for (var i = 0; i < numElements; i++) {
                var element = elements[i];
                if (!ts.getRestIndicatorOfBindingOrAssignmentElement(element)) {
                    var propertyName = ts.getPropertyNameOfBindingOrAssignmentElement(element);
                    if (flattenContext.level >= 1 /* ObjectRest */
                        && !(element.transformFlags & (8192 /* ContainsRestOrSpread */ | 16384 /* ContainsObjectRestOrSpread */))
                        && !(ts.getTargetOfBindingOrAssignmentElement(element).transformFlags & (8192 /* ContainsRestOrSpread */ | 16384 /* ContainsObjectRestOrSpread */))
                        && !ts.isComputedPropertyName(propertyName)) {
                        bindingElements = ts.append(bindingElements, ts.visitNode(element, flattenContext.visitor));
                    }
                    else {
                        if (bindingElements) {
                            flattenContext.emitBindingOrAssignment(flattenContext.createObjectBindingOrAssignmentPattern(bindingElements), value, location, pattern);
                            bindingElements = undefined;
                        }
                        var rhsValue = createDestructuringPropertyAccess(flattenContext, value, propertyName);
                        if (ts.isComputedPropertyName(propertyName)) {
                            computedTempVariables = ts.append(computedTempVariables, rhsValue.argumentExpression);
                        }
                        flattenBindingOrAssignmentElement(flattenContext, element, rhsValue, /*location*/ element);
                    }
                }
                else if (i === numElements - 1) {
                    if (bindingElements) {
                        flattenContext.emitBindingOrAssignment(flattenContext.createObjectBindingOrAssignmentPattern(bindingElements), value, location, pattern);
                        bindingElements = undefined;
                    }
                    var rhsValue = createRestCall(flattenContext.context, value, elements, computedTempVariables, pattern); // TODO: GH#18217
                    flattenBindingOrAssignmentElement(flattenContext, element, rhsValue, element);
                }
            }
            if (bindingElements) {
                flattenContext.emitBindingOrAssignment(flattenContext.createObjectBindingOrAssignmentPattern(bindingElements), value, location, pattern);
            }
        }
        /**
         * Flattens an ArrayBindingOrAssignmentPattern into zero or more bindings or assignments.
         *
         * @param flattenContext Options used to control flattening.
         * @param parent The parent element of the pattern.
         * @param pattern The ArrayBindingOrAssignmentPattern to flatten.
         * @param value The current RHS value to assign to the element.
         * @param location The location to use for source maps and comments.
         */
        function flattenArrayBindingOrAssignmentPattern(flattenContext, parent, pattern, value, location) {
            var elements = ts.getElementsOfBindingOrAssignmentPattern(pattern);
            var numElements = elements.length;
            if (flattenContext.level < 1 /* ObjectRest */ && flattenContext.downlevelIteration) {
                // Read the elements of the iterable into an array
                value = ensureIdentifier(flattenContext, ts.createReadHelper(flattenContext.context, value, numElements > 0 && ts.getRestIndicatorOfBindingOrAssignmentElement(elements[numElements - 1])
                    ? undefined
                    : numElements, location), 
                /*reuseIdentifierExpressions*/ false, location);
            }
            else if (numElements !== 1 && (flattenContext.level < 1 /* ObjectRest */ || numElements === 0)
                || ts.every(elements, ts.isOmittedExpression)) {
                // For anything other than a single-element destructuring we need to generate a temporary
                // to ensure value is evaluated exactly once. Additionally, if we have zero elements
                // we need to emit *something* to ensure that in case a 'var' keyword was already emitted,
                // so in that case, we'll intentionally create that temporary.
                // Or all the elements of the binding pattern are omitted expression such as "var [,] = [1,2]",
                // then we will create temporary variable.
                var reuseIdentifierExpressions = !ts.isDeclarationBindingElement(parent) || numElements !== 0;
                value = ensureIdentifier(flattenContext, value, reuseIdentifierExpressions, location);
            }
            var bindingElements;
            var restContainingElements;
            for (var i = 0; i < numElements; i++) {
                var element = elements[i];
                if (flattenContext.level >= 1 /* ObjectRest */) {
                    // If an array pattern contains an ObjectRest, we must cache the result so that we
                    // can perform the ObjectRest destructuring in a different declaration
                    if (element.transformFlags & 16384 /* ContainsObjectRestOrSpread */) {
                        var temp = ts.createTempVariable(/*recordTempVariable*/ undefined);
                        if (flattenContext.hoistTempVariables) {
                            flattenContext.context.hoistVariableDeclaration(temp);
                        }
                        restContainingElements = ts.append(restContainingElements, [temp, element]);
                        bindingElements = ts.append(bindingElements, flattenContext.createArrayBindingOrAssignmentElement(temp));
                    }
                    else {
                        bindingElements = ts.append(bindingElements, element);
                    }
                }
                else if (ts.isOmittedExpression(element)) {
                    continue;
                }
                else if (!ts.getRestIndicatorOfBindingOrAssignmentElement(element)) {
                    var rhsValue = ts.createElementAccess(value, i);
                    flattenBindingOrAssignmentElement(flattenContext, element, rhsValue, /*location*/ element);
                }
                else if (i === numElements - 1) {
                    var rhsValue = ts.createArraySlice(value, i);
                    flattenBindingOrAssignmentElement(flattenContext, element, rhsValue, /*location*/ element);
                }
            }
            if (bindingElements) {
                flattenContext.emitBindingOrAssignment(flattenContext.createArrayBindingOrAssignmentPattern(bindingElements), value, location, pattern);
            }
            if (restContainingElements) {
                for (var _i = 0, restContainingElements_1 = restContainingElements; _i < restContainingElements_1.length; _i++) {
                    var _a = restContainingElements_1[_i], id = _a[0], element = _a[1];
                    flattenBindingOrAssignmentElement(flattenContext, element, id, element);
                }
            }
        }
        /**
         * Creates an expression used to provide a default value if a value is `undefined` at runtime.
         *
         * @param flattenContext Options used to control flattening.
         * @param value The RHS value to test.
         * @param defaultValue The default value to use if `value` is `undefined` at runtime.
         * @param location The location to use for source maps and comments.
         */
        function createDefaultValueCheck(flattenContext, value, defaultValue, location) {
            value = ensureIdentifier(flattenContext, value, /*reuseIdentifierExpressions*/ true, location);
            return ts.createConditional(ts.createTypeCheck(value, "undefined"), defaultValue, value);
        }
        /**
         * Creates either a PropertyAccessExpression or an ElementAccessExpression for the
         * right-hand side of a transformed destructuring assignment.
         *
         * @link https://tc39.github.io/ecma262/#sec-runtime-semantics-keyeddestructuringassignmentevaluation
         *
         * @param flattenContext Options used to control flattening.
         * @param value The RHS value that is the source of the property.
         * @param propertyName The destructuring property name.
         */
        function createDestructuringPropertyAccess(flattenContext, value, propertyName) {
            if (ts.isComputedPropertyName(propertyName)) {
                var argumentExpression = ensureIdentifier(flattenContext, ts.visitNode(propertyName.expression, flattenContext.visitor), /*reuseIdentifierExpressions*/ false, /*location*/ propertyName);
                return ts.createElementAccess(value, argumentExpression);
            }
            else if (ts.isStringOrNumericLiteralLike(propertyName)) {
                var argumentExpression = ts.getSynthesizedClone(propertyName);
                argumentExpression.text = argumentExpression.text;
                return ts.createElementAccess(value, argumentExpression);
            }
            else {
                var name = ts.createIdentifier(ts.idText(propertyName));
                return ts.createPropertyAccess(value, name);
            }
        }
        /**
         * Ensures that there exists a declared identifier whose value holds the given expression.
         * This function is useful to ensure that the expression's value can be read from in subsequent expressions.
         * Unless 'reuseIdentifierExpressions' is false, 'value' will be returned if it is just an identifier.
         *
         * @param flattenContext Options used to control flattening.
         * @param value the expression whose value needs to be bound.
         * @param reuseIdentifierExpressions true if identifier expressions can simply be returned;
         * false if it is necessary to always emit an identifier.
         * @param location The location to use for source maps and comments.
         */
        function ensureIdentifier(flattenContext, value, reuseIdentifierExpressions, location) {
            if (ts.isIdentifier(value) && reuseIdentifierExpressions) {
                return value;
            }
            else {
                var temp = ts.createTempVariable(/*recordTempVariable*/ undefined);
                if (flattenContext.hoistTempVariables) {
                    flattenContext.context.hoistVariableDeclaration(temp);
                    flattenContext.emitExpression(ts.setTextRange(ts.createAssignment(temp, value), location));
                }
                else {
                    flattenContext.emitBindingOrAssignment(temp, value, location, /*original*/ undefined);
                }
                return temp;
            }
        }
        function makeArrayBindingPattern(elements) {
            ts.Debug.assertEachNode(elements, ts.isArrayBindingElement);
            return ts.createArrayBindingPattern(elements);
        }
        function makeArrayAssignmentPattern(elements) {
            return ts.createArrayLiteral(ts.map(elements, ts.convertToArrayAssignmentElement));
        }
        function makeObjectBindingPattern(elements) {
            ts.Debug.assertEachNode(elements, ts.isBindingElement);
            return ts.createObjectBindingPattern(elements);
        }
        function makeObjectAssignmentPattern(elements) {
            return ts.createObjectLiteral(ts.map(elements, ts.convertToObjectAssignmentElement));
        }
        function makeBindingElement(name) {
            return ts.createBindingElement(/*dotDotDotToken*/ undefined, /*propertyName*/ undefined, name);
        }
        function makeAssignmentElement(name) {
            return name;
        }
        ts.restHelper = {
            name: "typescript:rest",
            importName: "__rest",
            scoped: false,
            text: "\n            var __rest = (this && this.__rest) || function (s, e) {\n                var t = {};\n                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)\n                    t[p] = s[p];\n                if (s != null && typeof Object.getOwnPropertySymbols === \"function\")\n                    for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {\n                        if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))\n                            t[p[i]] = s[p[i]];\n                    }\n                return t;\n            };"
        };
        /** Given value: o, propName: p, pattern: { a, b, ...p } from the original statement
         * `{ a, b, ...p } = o`, create `p = __rest(o, ["a", "b"]);`
         */
        function createRestCall(context, value, elements, computedTempVariables, location) {
            context.requestEmitHelper(ts.restHelper);
            var propertyNames = [];
            var computedTempVariableOffset = 0;
            for (var i = 0; i < elements.length - 1; i++) {
                var propertyName = ts.getPropertyNameOfBindingOrAssignmentElement(elements[i]);
                if (propertyName) {
                    if (ts.isComputedPropertyName(propertyName)) {
                        var temp = computedTempVariables[computedTempVariableOffset];
                        computedTempVariableOffset++;
                        // typeof _tmp === "symbol" ? _tmp : _tmp + ""
                        propertyNames.push(ts.createConditional(ts.createTypeCheck(temp, "symbol"), temp, ts.createAdd(temp, ts.createLiteral(""))));
                    }
                    else {
                        propertyNames.push(ts.createLiteral(propertyName));
                    }
                }
            }
            return ts.createCall(ts.getUnscopedHelperName("__rest"), 
            /*typeArguments*/ undefined, [
                value,
                ts.setTextRange(ts.createArrayLiteral(propertyNames), location)
            ]);
        }
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        /**
         * Indicates whether to emit type metadata in the new format.
         */
        var USE_NEW_TYPE_METADATA_FORMAT = false;
        var TypeScriptSubstitutionFlags;
        (function (TypeScriptSubstitutionFlags) {
            /** Enables substitutions for decorated classes. */
            TypeScriptSubstitutionFlags[TypeScriptSubstitutionFlags["ClassAliases"] = 1] = "ClassAliases";
            /** Enables substitutions for namespace exports. */
            TypeScriptSubstitutionFlags[TypeScriptSubstitutionFlags["NamespaceExports"] = 2] = "NamespaceExports";
            /* Enables substitutions for unqualified enum members */
            TypeScriptSubstitutionFlags[TypeScriptSubstitutionFlags["NonQualifiedEnumMembers"] = 8] = "NonQualifiedEnumMembers";
        })(TypeScriptSubstitutionFlags || (TypeScriptSubstitutionFlags = {}));
        var ClassFacts;
        (function (ClassFacts) {
            ClassFacts[ClassFacts["None"] = 0] = "None";
            ClassFacts[ClassFacts["HasStaticInitializedProperties"] = 1] = "HasStaticInitializedProperties";
            ClassFacts[ClassFacts["HasConstructorDecorators"] = 2] = "HasConstructorDecorators";
            ClassFacts[ClassFacts["HasMemberDecorators"] = 4] = "HasMemberDecorators";
            ClassFacts[ClassFacts["IsExportOfNamespace"] = 8] = "IsExportOfNamespace";
            ClassFacts[ClassFacts["IsNamedExternalExport"] = 16] = "IsNamedExternalExport";
            ClassFacts[ClassFacts["IsDefaultExternalExport"] = 32] = "IsDefaultExternalExport";
            ClassFacts[ClassFacts["IsDerivedClass"] = 64] = "IsDerivedClass";
            ClassFacts[ClassFacts["UseImmediatelyInvokedFunctionExpression"] = 128] = "UseImmediatelyInvokedFunctionExpression";
            ClassFacts[ClassFacts["HasAnyDecorators"] = 6] = "HasAnyDecorators";
            ClassFacts[ClassFacts["NeedsName"] = 5] = "NeedsName";
            ClassFacts[ClassFacts["MayNeedImmediatelyInvokedFunctionExpression"] = 7] = "MayNeedImmediatelyInvokedFunctionExpression";
            ClassFacts[ClassFacts["IsExported"] = 56] = "IsExported";
        })(ClassFacts || (ClassFacts = {}));
        function transformTypeScript(context) {
            var startLexicalEnvironment = context.startLexicalEnvironment, resumeLexicalEnvironment = context.resumeLexicalEnvironment, endLexicalEnvironment = context.endLexicalEnvironment, hoistVariableDeclaration = context.hoistVariableDeclaration;
            var resolver = context.getEmitResolver();
            var compilerOptions = context.getCompilerOptions();
            var strictNullChecks = ts.getStrictOptionValue(compilerOptions, "strictNullChecks");
            var languageVersion = ts.getEmitScriptTarget(compilerOptions);
            var moduleKind = ts.getEmitModuleKind(compilerOptions);
            // Save the previous transformation hooks.
            var previousOnEmitNode = context.onEmitNode;
            var previousOnSubstituteNode = context.onSubstituteNode;
            // Set new transformation hooks.
            context.onEmitNode = onEmitNode;
            context.onSubstituteNode = onSubstituteNode;
            // Enable substitution for property/element access to emit const enum values.
            context.enableSubstitution(194 /* PropertyAccessExpression */);
            context.enableSubstitution(195 /* ElementAccessExpression */);
            // These variables contain state that changes as we descend into the tree.
            var currentSourceFile;
            var currentNamespace;
            var currentNamespaceContainerName;
            var currentLexicalScope;
            var currentNameScope;
            var currentScopeFirstDeclarationsOfName;
            var currentClassHasParameterProperties;
            /**
             * Keeps track of whether expression substitution has been enabled for specific edge cases.
             * They are persisted between each SourceFile transformation and should not be reset.
             */
            var enabledSubstitutions;
            /**
             * A map that keeps track of aliases created for classes with decorators to avoid issues
             * with the double-binding behavior of classes.
             */
            var classAliases;
            /**
             * Keeps track of whether we are within any containing namespaces when performing
             * just-in-time substitution while printing an expression identifier.
             */
            var applicableSubstitutions;
            return transformSourceFileOrBundle;
            function transformSourceFileOrBundle(node) {
                if (node.kind === 291 /* Bundle */) {
                    return transformBundle(node);
                }
                return transformSourceFile(node);
            }
            function transformBundle(node) {
                return ts.createBundle(node.sourceFiles.map(transformSourceFile), ts.mapDefined(node.prepends, function (prepend) {
                    if (prepend.kind === 293 /* InputFiles */) {
                        return ts.createUnparsedSourceFile(prepend, "js");
                    }
                    return prepend;
                }));
            }
            /**
             * Transform TypeScript-specific syntax in a SourceFile.
             *
             * @param node A SourceFile node.
             */
            function transformSourceFile(node) {
                if (node.isDeclarationFile) {
                    return node;
                }
                currentSourceFile = node;
                var visited = saveStateAndInvoke(node, visitSourceFile);
                ts.addEmitHelpers(visited, context.readEmitHelpers());
                currentSourceFile = undefined;
                return visited;
            }
            /**
             * Visits a node, saving and restoring state variables on the stack.
             *
             * @param node The node to visit.
             */
            function saveStateAndInvoke(node, f) {
                // Save state
                var savedCurrentScope = currentLexicalScope;
                var savedCurrentNameScope = currentNameScope;
                var savedCurrentScopeFirstDeclarationsOfName = currentScopeFirstDeclarationsOfName;
                var savedCurrentClassHasParameterProperties = currentClassHasParameterProperties;
                // Handle state changes before visiting a node.
                onBeforeVisitNode(node);
                var visited = f(node);
                // Restore state
                if (currentLexicalScope !== savedCurrentScope) {
                    currentScopeFirstDeclarationsOfName = savedCurrentScopeFirstDeclarationsOfName;
                }
                currentLexicalScope = savedCurrentScope;
                currentNameScope = savedCurrentNameScope;
                currentClassHasParameterProperties = savedCurrentClassHasParameterProperties;
                return visited;
            }
            /**
             * Performs actions that should always occur immediately before visiting a node.
             *
             * @param node The node to visit.
             */
            function onBeforeVisitNode(node) {
                switch (node.kind) {
                    case 290 /* SourceFile */:
                    case 251 /* CaseBlock */:
                    case 250 /* ModuleBlock */:
                    case 223 /* Block */:
                        currentLexicalScope = node;
                        currentNameScope = undefined;
                        currentScopeFirstDeclarationsOfName = undefined;
                        break;
                    case 245 /* ClassDeclaration */:
                    case 244 /* FunctionDeclaration */:
                        if (ts.hasModifier(node, 2 /* Ambient */)) {
                            break;
                        }
                        // Record these declarations provided that they have a name.
                        if (node.name) {
                            recordEmittedDeclarationInScope(node);
                        }
                        else {
                            // These nodes should always have names unless they are default-exports;
                            // however, class declaration parsing allows for undefined names, so syntactically invalid
                            // programs may also have an undefined name.
                            ts.Debug.assert(node.kind === 245 /* ClassDeclaration */ || ts.hasModifier(node, 512 /* Default */));
                        }
                        if (ts.isClassDeclaration(node)) {
                            // XXX: should probably also cover interfaces and type aliases that can have type variables?
                            currentNameScope = node;
                        }
                        break;
                }
            }
            /**
             * General-purpose node visitor.
             *
             * @param node The node to visit.
             */
            function visitor(node) {
                return saveStateAndInvoke(node, visitorWorker);
            }
            /**
             * Visits and possibly transforms any node.
             *
             * @param node The node to visit.
             */
            function visitorWorker(node) {
                if (node.transformFlags & 1 /* ContainsTypeScript */) {
                    return visitTypeScript(node);
                }
                return node;
            }
            /**
             * Specialized visitor that visits the immediate children of a SourceFile.
             *
             * @param node The node to visit.
             */
            function sourceElementVisitor(node) {
                return saveStateAndInvoke(node, sourceElementVisitorWorker);
            }
            /**
             * Specialized visitor that visits the immediate children of a SourceFile.
             *
             * @param node The node to visit.
             */
            function sourceElementVisitorWorker(node) {
                switch (node.kind) {
                    case 254 /* ImportDeclaration */:
                    case 253 /* ImportEqualsDeclaration */:
                    case 259 /* ExportAssignment */:
                    case 260 /* ExportDeclaration */:
                        return visitEllidableStatement(node);
                    default:
                        return visitorWorker(node);
                }
            }
            function visitEllidableStatement(node) {
                var parsed = ts.getParseTreeNode(node);
                if (parsed !== node) {
                    // If the node has been transformed by a `before` transformer, perform no ellision on it
                    // As the type information we would attempt to lookup to perform ellision is potentially unavailable for the synthesized nodes
                    // We do not reuse `visitorWorker`, as the ellidable statement syntax kinds are technically unrecognized by the switch-case in `visitTypeScript`,
                    // and will trigger debug failures when debug verbosity is turned up
                    if (node.transformFlags & 1 /* ContainsTypeScript */) {
                        // This node contains TypeScript, so we should visit its children.
                        return ts.visitEachChild(node, visitor, context);
                    }
                    // Otherwise, we can just return the node
                    return node;
                }
                switch (node.kind) {
                    case 254 /* ImportDeclaration */:
                        return visitImportDeclaration(node);
                    case 253 /* ImportEqualsDeclaration */:
                        return visitImportEqualsDeclaration(node);
                    case 259 /* ExportAssignment */:
                        return visitExportAssignment(node);
                    case 260 /* ExportDeclaration */:
                        return visitExportDeclaration(node);
                    default:
                        ts.Debug.fail("Unhandled ellided statement");
                }
            }
            /**
             * Specialized visitor that visits the immediate children of a namespace.
             *
             * @param node The node to visit.
             */
            function namespaceElementVisitor(node) {
                return saveStateAndInvoke(node, namespaceElementVisitorWorker);
            }
            /**
             * Specialized visitor that visits the immediate children of a namespace.
             *
             * @param node The node to visit.
             */
            function namespaceElementVisitorWorker(node) {
                if (node.kind === 260 /* ExportDeclaration */ ||
                    node.kind === 254 /* ImportDeclaration */ ||
                    node.kind === 255 /* ImportClause */ ||
                    (node.kind === 253 /* ImportEqualsDeclaration */ &&
                        node.moduleReference.kind === 265 /* ExternalModuleReference */)) {
                    // do not emit ES6 imports and exports since they are illegal inside a namespace
                    return undefined;
                }
                else if (node.transformFlags & 1 /* ContainsTypeScript */ || ts.hasModifier(node, 1 /* Export */)) {
                    return visitTypeScript(node);
                }
                return node;
            }
            /**
             * Specialized visitor that visits the immediate children of a class with TypeScript syntax.
             *
             * @param node The node to visit.
             */
            function classElementVisitor(node) {
                return saveStateAndInvoke(node, classElementVisitorWorker);
            }
            /**
             * Specialized visitor that visits the immediate children of a class with TypeScript syntax.
             *
             * @param node The node to visit.
             */
            function classElementVisitorWorker(node) {
                switch (node.kind) {
                    case 162 /* Constructor */:
                        return visitConstructor(node);
                    case 159 /* PropertyDeclaration */:
                        // Property declarations are not TypeScript syntax, but they must be visited
                        // for the decorator transformation.
                        return visitPropertyDeclaration(node);
                    case 167 /* IndexSignature */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                    case 161 /* MethodDeclaration */:
                        // Fallback to the default visit behavior.
                        return visitorWorker(node);
                    case 222 /* SemicolonClassElement */:
                        return node;
                    default:
                        return ts.Debug.failBadSyntaxKind(node);
                }
            }
            function modifierVisitor(node) {
                if (ts.modifierToFlag(node.kind) & 2270 /* TypeScriptModifier */) {
                    return undefined;
                }
                else if (currentNamespace && node.kind === 89 /* ExportKeyword */) {
                    return undefined;
                }
                return node;
            }
            /**
             * Branching visitor, visits a TypeScript syntax node.
             *
             * @param node The node to visit.
             */
            function visitTypeScript(node) {
                if (ts.isStatement(node) && ts.hasModifier(node, 2 /* Ambient */)) {
                    // TypeScript ambient declarations are elided, but some comments may be preserved.
                    // See the implementation of `getLeadingComments` in comments.ts for more details.
                    return ts.createNotEmittedStatement(node);
                }
                switch (node.kind) {
                    case 89 /* ExportKeyword */:
                    case 84 /* DefaultKeyword */:
                        // ES6 export and default modifiers are elided when inside a namespace.
                        return currentNamespace ? undefined : node;
                    case 119 /* PublicKeyword */:
                    case 117 /* PrivateKeyword */:
                    case 118 /* ProtectedKeyword */:
                    case 122 /* AbstractKeyword */:
                    case 81 /* ConstKeyword */:
                    case 130 /* DeclareKeyword */:
                    case 138 /* ReadonlyKeyword */:
                    // TypeScript accessibility and readonly modifiers are elided
                    // falls through
                    case 174 /* ArrayType */:
                    case 175 /* TupleType */:
                    case 176 /* OptionalType */:
                    case 177 /* RestType */:
                    case 173 /* TypeLiteral */:
                    case 168 /* TypePredicate */:
                    case 155 /* TypeParameter */:
                    case 125 /* AnyKeyword */:
                    case 148 /* UnknownKeyword */:
                    case 128 /* BooleanKeyword */:
                    case 143 /* StringKeyword */:
                    case 140 /* NumberKeyword */:
                    case 137 /* NeverKeyword */:
                    case 110 /* VoidKeyword */:
                    case 144 /* SymbolKeyword */:
                    case 171 /* ConstructorType */:
                    case 170 /* FunctionType */:
                    case 172 /* TypeQuery */:
                    case 169 /* TypeReference */:
                    case 178 /* UnionType */:
                    case 179 /* IntersectionType */:
                    case 180 /* ConditionalType */:
                    case 182 /* ParenthesizedType */:
                    case 183 /* ThisType */:
                    case 184 /* TypeOperator */:
                    case 185 /* IndexedAccessType */:
                    case 186 /* MappedType */:
                    case 187 /* LiteralType */:
                    // TypeScript type nodes are elided.
                    // falls through
                    case 167 /* IndexSignature */:
                    // TypeScript index signatures are elided.
                    // falls through
                    case 157 /* Decorator */:
                    // TypeScript decorators are elided. They will be emitted as part of visitClassDeclaration.
                    // falls through
                    case 247 /* TypeAliasDeclaration */:
                        // TypeScript type-only declarations are elided.
                        return undefined;
                    case 159 /* PropertyDeclaration */:
                        // TypeScript property declarations are elided. However their names are still visited, and can potentially be retained if they could have sideeffects
                        return visitPropertyDeclaration(node);
                    case 252 /* NamespaceExportDeclaration */:
                        // TypeScript namespace export declarations are elided.
                        return undefined;
                    case 162 /* Constructor */:
                        return visitConstructor(node);
                    case 246 /* InterfaceDeclaration */:
                        // TypeScript interfaces are elided, but some comments may be preserved.
                        // See the implementation of `getLeadingComments` in comments.ts for more details.
                        return ts.createNotEmittedStatement(node);
                    case 245 /* ClassDeclaration */:
                        // This may be a class declaration with TypeScript syntax extensions.
                        //
                        // TypeScript class syntax extensions include:
                        // - decorators
                        // - optional `implements` heritage clause
                        // - parameter property assignments in the constructor
                        // - index signatures
                        // - method overload signatures
                        return visitClassDeclaration(node);
                    case 214 /* ClassExpression */:
                        // This may be a class expression with TypeScript syntax extensions.
                        //
                        // TypeScript class syntax extensions include:
                        // - decorators
                        // - optional `implements` heritage clause
                        // - parameter property assignments in the constructor
                        // - index signatures
                        // - method overload signatures
                        return visitClassExpression(node);
                    case 279 /* HeritageClause */:
                        // This may be a heritage clause with TypeScript syntax extensions.
                        //
                        // TypeScript heritage clause extensions include:
                        // - `implements` clause
                        return visitHeritageClause(node);
                    case 216 /* ExpressionWithTypeArguments */:
                        // TypeScript supports type arguments on an expression in an `extends` heritage clause.
                        return visitExpressionWithTypeArguments(node);
                    case 161 /* MethodDeclaration */:
                        // TypeScript method declarations may have decorators, modifiers
                        // or type annotations.
                        return visitMethodDeclaration(node);
                    case 163 /* GetAccessor */:
                        // Get Accessors can have TypeScript modifiers, decorators, and type annotations.
                        return visitGetAccessor(node);
                    case 164 /* SetAccessor */:
                        // Set Accessors can have TypeScript modifiers and type annotations.
                        return visitSetAccessor(node);
                    case 244 /* FunctionDeclaration */:
                        // Typescript function declarations can have modifiers, decorators, and type annotations.
                        return visitFunctionDeclaration(node);
                    case 201 /* FunctionExpression */:
                        // TypeScript function expressions can have modifiers and type annotations.
                        return visitFunctionExpression(node);
                    case 202 /* ArrowFunction */:
                        // TypeScript arrow functions can have modifiers and type annotations.
                        return visitArrowFunction(node);
                    case 156 /* Parameter */:
                        // This may be a parameter declaration with TypeScript syntax extensions.
                        //
                        // TypeScript parameter declaration syntax extensions include:
                        // - decorators
                        // - accessibility modifiers
                        // - the question mark (?) token for optional parameters
                        // - type annotations
                        // - this parameters
                        return visitParameter(node);
                    case 200 /* ParenthesizedExpression */:
                        // ParenthesizedExpressions are TypeScript if their expression is a
                        // TypeAssertion or AsExpression
                        return visitParenthesizedExpression(node);
                    case 199 /* TypeAssertionExpression */:
                    case 217 /* AsExpression */:
                        // TypeScript type assertions are removed, but their subtrees are preserved.
                        return visitAssertionExpression(node);
                    case 196 /* CallExpression */:
                        return visitCallExpression(node);
                    case 197 /* NewExpression */:
                        return visitNewExpression(node);
                    case 198 /* TaggedTemplateExpression */:
                        return visitTaggedTemplateExpression(node);
                    case 218 /* NonNullExpression */:
                        // TypeScript non-null expressions are removed, but their subtrees are preserved.
                        return visitNonNullExpression(node);
                    case 248 /* EnumDeclaration */:
                        // TypeScript enum declarations do not exist in ES6 and must be rewritten.
                        return visitEnumDeclaration(node);
                    case 225 /* VariableStatement */:
                        // TypeScript namespace exports for variable statements must be transformed.
                        return visitVariableStatement(node);
                    case 242 /* VariableDeclaration */:
                        return visitVariableDeclaration(node);
                    case 249 /* ModuleDeclaration */:
                        // TypeScript namespace declarations must be transformed.
                        return visitModuleDeclaration(node);
                    case 253 /* ImportEqualsDeclaration */:
                        // TypeScript namespace or external module import.
                        return visitImportEqualsDeclaration(node);
                    default:
                        // node contains some other TypeScript syntax
                        return ts.visitEachChild(node, visitor, context);
                }
            }
            function visitSourceFile(node) {
                var alwaysStrict = ts.getStrictOptionValue(compilerOptions, "alwaysStrict") &&
                    !(ts.isExternalModule(node) && moduleKind >= ts.ModuleKind.ES2015) &&
                    !ts.isJsonSourceFile(node);
                return ts.updateSourceFileNode(node, ts.visitLexicalEnvironment(node.statements, sourceElementVisitor, context, /*start*/ 0, alwaysStrict));
            }
            /**
             * Tests whether we should emit a __decorate call for a class declaration.
             */
            function shouldEmitDecorateCallForClass(node) {
                if (node.decorators && node.decorators.length > 0) {
                    return true;
                }
                var constructor = ts.getFirstConstructorWithBody(node);
                if (constructor) {
                    return ts.forEach(constructor.parameters, shouldEmitDecorateCallForParameter);
                }
                return false;
            }
            /**
             * Tests whether we should emit a __decorate call for a parameter declaration.
             */
            function shouldEmitDecorateCallForParameter(parameter) {
                return parameter.decorators !== undefined && parameter.decorators.length > 0;
            }
            function getClassFacts(node, staticProperties) {
                var facts = 0 /* None */;
                if (ts.some(staticProperties))
                    facts |= 1 /* HasStaticInitializedProperties */;
                var extendsClauseElement = ts.getEffectiveBaseTypeNode(node);
                if (extendsClauseElement && ts.skipOuterExpressions(extendsClauseElement.expression).kind !== 100 /* NullKeyword */)
                    facts |= 64 /* IsDerivedClass */;
                if (shouldEmitDecorateCallForClass(node))
                    facts |= 2 /* HasConstructorDecorators */;
                if (ts.childIsDecorated(node))
                    facts |= 4 /* HasMemberDecorators */;
                if (isExportOfNamespace(node))
                    facts |= 8 /* IsExportOfNamespace */;
                else if (isDefaultExternalModuleExport(node))
                    facts |= 32 /* IsDefaultExternalExport */;
                else if (isNamedExternalModuleExport(node))
                    facts |= 16 /* IsNamedExternalExport */;
                if (languageVersion <= 1 /* ES5 */ && (facts & 7 /* MayNeedImmediatelyInvokedFunctionExpression */))
                    facts |= 128 /* UseImmediatelyInvokedFunctionExpression */;
                return facts;
            }
            function hasTypeScriptClassSyntax(node) {
                return !!(node.transformFlags & 2048 /* ContainsTypeScriptClassSyntax */);
            }
            function isClassLikeDeclarationWithTypeScriptSyntax(node) {
                return ts.some(node.decorators)
                    || ts.some(node.typeParameters)
                    || ts.some(node.heritageClauses, hasTypeScriptClassSyntax)
                    || ts.some(node.members, hasTypeScriptClassSyntax);
            }
            function visitClassDeclaration(node) {
                if (!isClassLikeDeclarationWithTypeScriptSyntax(node) && !(currentNamespace && ts.hasModifier(node, 1 /* Export */))) {
                    return ts.visitEachChild(node, visitor, context);
                }
                var staticProperties = ts.getProperties(node, /*requireInitializer*/ true, /*isStatic*/ true);
                var facts = getClassFacts(node, staticProperties);
                if (facts & 128 /* UseImmediatelyInvokedFunctionExpression */) {
                    context.startLexicalEnvironment();
                }
                var name = node.name || (facts & 5 /* NeedsName */ ? ts.getGeneratedNameForNode(node) : undefined);
                var classStatement = facts & 2 /* HasConstructorDecorators */
                    ? createClassDeclarationHeadWithDecorators(node, name)
                    : createClassDeclarationHeadWithoutDecorators(node, name, facts);
                var statements = [classStatement];
                // Write any decorators of the node.
                addClassElementDecorationStatements(statements, node, /*isStatic*/ false);
                addClassElementDecorationStatements(statements, node, /*isStatic*/ true);
                addConstructorDecorationStatement(statements, node);
                if (facts & 128 /* UseImmediatelyInvokedFunctionExpression */) {
                    // When we emit a TypeScript class down to ES5, we must wrap it in an IIFE so that the
                    // 'es2015' transformer can properly nest static initializers and decorators. The result
                    // looks something like:
                    //
                    //  var C = function () {
                    //      class C {
                    //      }
                    //      C.static_prop = 1;
                    //      return C;
                    //  }();
                    //
                    var closingBraceLocation = ts.createTokenRange(ts.skipTrivia(currentSourceFile.text, node.members.end), 19 /* CloseBraceToken */);
                    var localName = ts.getInternalName(node);
                    // The following partially-emitted expression exists purely to align our sourcemap
                    // emit with the original emitter.
                    var outer = ts.createPartiallyEmittedExpression(localName);
                    outer.end = closingBraceLocation.end;
                    ts.setEmitFlags(outer, 1536 /* NoComments */);
                    var statement = ts.createReturn(outer);
                    statement.pos = closingBraceLocation.pos;
                    ts.setEmitFlags(statement, 1536 /* NoComments */ | 384 /* NoTokenSourceMaps */);
                    statements.push(statement);
                    ts.insertStatementsAfterStandardPrologue(statements, context.endLexicalEnvironment());
                    var iife = ts.createImmediatelyInvokedArrowFunction(statements);
                    ts.setEmitFlags(iife, 33554432 /* TypeScriptClassWrapper */);
                    var varStatement = ts.createVariableStatement(
                    /*modifiers*/ undefined, ts.createVariableDeclarationList([
                        ts.createVariableDeclaration(ts.getLocalName(node, /*allowComments*/ false, /*allowSourceMaps*/ false), 
                        /*type*/ undefined, iife)
                    ]));
                    ts.setOriginalNode(varStatement, node);
                    ts.setCommentRange(varStatement, node);
                    ts.setSourceMapRange(varStatement, ts.moveRangePastDecorators(node));
                    ts.startOnNewLine(varStatement);
                    statements = [varStatement];
                }
                // If the class is exported as part of a TypeScript namespace, emit the namespace export.
                // Otherwise, if the class was exported at the top level and was decorated, emit an export
                // declaration or export default for the class.
                if (facts & 8 /* IsExportOfNamespace */) {
                    addExportMemberAssignment(statements, node);
                }
                else if (facts & 128 /* UseImmediatelyInvokedFunctionExpression */ || facts & 2 /* HasConstructorDecorators */) {
                    if (facts & 32 /* IsDefaultExternalExport */) {
                        statements.push(ts.createExportDefault(ts.getLocalName(node, /*allowComments*/ false, /*allowSourceMaps*/ true)));
                    }
                    else if (facts & 16 /* IsNamedExternalExport */) {
                        statements.push(ts.createExternalModuleExport(ts.getLocalName(node, /*allowComments*/ false, /*allowSourceMaps*/ true)));
                    }
                }
                if (statements.length > 1) {
                    // Add a DeclarationMarker as a marker for the end of the declaration
                    statements.push(ts.createEndOfDeclarationMarker(node));
                    ts.setEmitFlags(classStatement, ts.getEmitFlags(classStatement) | 4194304 /* HasEndOfDeclarationMarker */);
                }
                return ts.singleOrMany(statements);
            }
            /**
             * Transforms a non-decorated class declaration and appends the resulting statements.
             *
             * @param node A ClassDeclaration node.
             * @param name The name of the class.
             * @param facts Precomputed facts about the class.
             */
            function createClassDeclarationHeadWithoutDecorators(node, name, facts) {
                //  ${modifiers} class ${name} ${heritageClauses} {
                //      ${members}
                //  }
                // we do not emit modifiers on the declaration if we are emitting an IIFE
                var modifiers = !(facts & 128 /* UseImmediatelyInvokedFunctionExpression */)
                    ? ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier)
                    : undefined;
                var classDeclaration = ts.createClassDeclaration(
                /*decorators*/ undefined, modifiers, name, 
                /*typeParameters*/ undefined, ts.visitNodes(node.heritageClauses, visitor, ts.isHeritageClause), transformClassMembers(node));
                // To better align with the old emitter, we should not emit a trailing source map
                // entry if the class has static properties.
                var emitFlags = ts.getEmitFlags(node);
                if (facts & 1 /* HasStaticInitializedProperties */) {
                    emitFlags |= 32 /* NoTrailingSourceMap */;
                }
                ts.aggregateTransformFlags(classDeclaration);
                ts.setTextRange(classDeclaration, node);
                ts.setOriginalNode(classDeclaration, node);
                ts.setEmitFlags(classDeclaration, emitFlags);
                return classDeclaration;
            }
            /**
             * Transforms a decorated class declaration and appends the resulting statements. If
             * the class requires an alias to avoid issues with double-binding, the alias is returned.
             */
            function createClassDeclarationHeadWithDecorators(node, name) {
                // When we emit an ES6 class that has a class decorator, we must tailor the
                // emit to certain specific cases.
                //
                // In the simplest case, we emit the class declaration as a let declaration, and
                // evaluate decorators after the close of the class body:
                //
                //  [Example 1]
                //  ---------------------------------------------------------------------
                //  TypeScript                      | Javascript
                //  ---------------------------------------------------------------------
                //  @dec                            | let C = class C {
                //  class C {                       | }
                //  }                               | C = __decorate([dec], C);
                //  ---------------------------------------------------------------------
                //  @dec                            | let C = class C {
                //  export class C {                | }
                //  }                               | C = __decorate([dec], C);
                //                                  | export { C };
                //  ---------------------------------------------------------------------
                //
                // If a class declaration contains a reference to itself *inside* of the class body,
                // this introduces two bindings to the class: One outside of the class body, and one
                // inside of the class body. If we apply decorators as in [Example 1] above, there
                // is the possibility that the decorator `dec` will return a new value for the
                // constructor, which would result in the binding inside of the class no longer
                // pointing to the same reference as the binding outside of the class.
                //
                // As a result, we must instead rewrite all references to the class *inside* of the
                // class body to instead point to a local temporary alias for the class:
                //
                //  [Example 2]
                //  ---------------------------------------------------------------------
                //  TypeScript                      | Javascript
                //  ---------------------------------------------------------------------
                //  @dec                            | let C = C_1 = class C {
                //  class C {                       |   static x() { return C_1.y; }
                //    static x() { return C.y; }    | }
                //    static y = 1;                 | C.y = 1;
                //  }                               | C = C_1 = __decorate([dec], C);
                //                                  | var C_1;
                //  ---------------------------------------------------------------------
                //  @dec                            | let C = class C {
                //  export class C {                |   static x() { return C_1.y; }
                //    static x() { return C.y; }    | }
                //    static y = 1;                 | C.y = 1;
                //  }                               | C = C_1 = __decorate([dec], C);
                //                                  | export { C };
                //                                  | var C_1;
                //  ---------------------------------------------------------------------
                //
                // If a class declaration is the default export of a module, we instead emit
                // the export after the decorated declaration:
                //
                //  [Example 3]
                //  ---------------------------------------------------------------------
                //  TypeScript                      | Javascript
                //  ---------------------------------------------------------------------
                //  @dec                            | let default_1 = class {
                //  export default class {          | }
                //  }                               | default_1 = __decorate([dec], default_1);
                //                                  | export default default_1;
                //  ---------------------------------------------------------------------
                //  @dec                            | let C = class C {
                //  export default class C {        | }
                //  }                               | C = __decorate([dec], C);
                //                                  | export default C;
                //  ---------------------------------------------------------------------
                //
                // If the class declaration is the default export and a reference to itself
                // inside of the class body, we must emit both an alias for the class *and*
                // move the export after the declaration:
                //
                //  [Example 4]
                //  ---------------------------------------------------------------------
                //  TypeScript                      | Javascript
                //  ---------------------------------------------------------------------
                //  @dec                            | let C = class C {
                //  export default class C {        |   static x() { return C_1.y; }
                //    static x() { return C.y; }    | }
                //    static y = 1;                 | C.y = 1;
                //  }                               | C = C_1 = __decorate([dec], C);
                //                                  | export default C;
                //                                  | var C_1;
                //  ---------------------------------------------------------------------
                //
                var location = ts.moveRangePastDecorators(node);
                var classAlias = getClassAliasIfNeeded(node);
                var declName = ts.getLocalName(node, /*allowComments*/ false, /*allowSourceMaps*/ true);
                //  ... = class ${name} ${heritageClauses} {
                //      ${members}
                //  }
                var heritageClauses = ts.visitNodes(node.heritageClauses, visitor, ts.isHeritageClause);
                var members = transformClassMembers(node);
                var classExpression = ts.createClassExpression(/*modifiers*/ undefined, name, /*typeParameters*/ undefined, heritageClauses, members);
                ts.aggregateTransformFlags(classExpression);
                ts.setOriginalNode(classExpression, node);
                ts.setTextRange(classExpression, location);
                //  let ${name} = ${classExpression} where name is either declaredName if the class doesn't contain self-reference
                //                                         or decoratedClassAlias if the class contain self-reference.
                var statement = ts.createVariableStatement(
                /*modifiers*/ undefined, ts.createVariableDeclarationList([
                    ts.createVariableDeclaration(declName, 
                    /*type*/ undefined, classAlias ? ts.createAssignment(classAlias, classExpression) : classExpression)
                ], 1 /* Let */));
                ts.setOriginalNode(statement, node);
                ts.setTextRange(statement, location);
                ts.setCommentRange(statement, node);
                return statement;
            }
            function visitClassExpression(node) {
                if (!isClassLikeDeclarationWithTypeScriptSyntax(node)) {
                    return ts.visitEachChild(node, visitor, context);
                }
                var classExpression = ts.createClassExpression(
                /*modifiers*/ undefined, node.name, 
                /*typeParameters*/ undefined, ts.visitNodes(node.heritageClauses, visitor, ts.isHeritageClause), transformClassMembers(node));
                ts.aggregateTransformFlags(classExpression);
                ts.setOriginalNode(classExpression, node);
                ts.setTextRange(classExpression, node);
                return classExpression;
            }
            /**
             * Transforms the members of a class.
             *
             * @param node The current class.
             */
            function transformClassMembers(node) {
                var members = [];
                var constructor = ts.getFirstConstructorWithBody(node);
                var parametersWithPropertyAssignments = constructor &&
                    ts.filter(constructor.parameters, function (p) { return ts.isParameterPropertyDeclaration(p, constructor); });
                if (parametersWithPropertyAssignments) {
                    for (var _i = 0, parametersWithPropertyAssignments_1 = parametersWithPropertyAssignments; _i < parametersWithPropertyAssignments_1.length; _i++) {
                        var parameter = parametersWithPropertyAssignments_1[_i];
                        if (ts.isIdentifier(parameter.name)) {
                            members.push(ts.setOriginalNode(ts.aggregateTransformFlags(ts.createProperty(
                            /*decorators*/ undefined, 
                            /*modifiers*/ undefined, parameter.name, 
                            /*questionOrExclamationToken*/ undefined, 
                            /*type*/ undefined, 
                            /*initializer*/ undefined)), parameter));
                        }
                    }
                }
                ts.addRange(members, ts.visitNodes(node.members, classElementVisitor, ts.isClassElement));
                return ts.setTextRange(ts.createNodeArray(members), /*location*/ node.members);
            }
            /**
             * Gets either the static or instance members of a class that are decorated, or have
             * parameters that are decorated.
             *
             * @param node The class containing the member.
             * @param isStatic A value indicating whether to retrieve static or instance members of
             *                 the class.
             */
            function getDecoratedClassElements(node, isStatic) {
                return ts.filter(node.members, isStatic ? function (m) { return isStaticDecoratedClassElement(m, node); } : function (m) { return isInstanceDecoratedClassElement(m, node); });
            }
            /**
             * Determines whether a class member is a static member of a class that is decorated, or
             * has parameters that are decorated.
             *
             * @param member The class member.
             */
            function isStaticDecoratedClassElement(member, parent) {
                return isDecoratedClassElement(member, /*isStatic*/ true, parent);
            }
            /**
             * Determines whether a class member is an instance member of a class that is decorated,
             * or has parameters that are decorated.
             *
             * @param member The class member.
             */
            function isInstanceDecoratedClassElement(member, parent) {
                return isDecoratedClassElement(member, /*isStatic*/ false, parent);
            }
            /**
             * Determines whether a class member is either a static or an instance member of a class
             * that is decorated, or has parameters that are decorated.
             *
             * @param member The class member.
             */
            function isDecoratedClassElement(member, isStatic, parent) {
                return ts.nodeOrChildIsDecorated(member, parent)
                    && isStatic === ts.hasModifier(member, 32 /* Static */);
            }
            /**
             * Gets an array of arrays of decorators for the parameters of a function-like node.
             * The offset into the result array should correspond to the offset of the parameter.
             *
             * @param node The function-like node.
             */
            function getDecoratorsOfParameters(node) {
                var decorators;
                if (node) {
                    var parameters = node.parameters;
                    var firstParameterIsThis = parameters.length > 0 && ts.parameterIsThisKeyword(parameters[0]);
                    var firstParameterOffset = firstParameterIsThis ? 1 : 0;
                    var numParameters = firstParameterIsThis ? parameters.length - 1 : parameters.length;
                    for (var i = 0; i < numParameters; i++) {
                        var parameter = parameters[i + firstParameterOffset];
                        if (decorators || parameter.decorators) {
                            if (!decorators) {
                                decorators = new Array(numParameters);
                            }
                            decorators[i] = parameter.decorators;
                        }
                    }
                }
                return decorators;
            }
            /**
             * Gets an AllDecorators object containing the decorators for the class and the decorators for the
             * parameters of the constructor of the class.
             *
             * @param node The class node.
             */
            function getAllDecoratorsOfConstructor(node) {
                var decorators = node.decorators;
                var parameters = getDecoratorsOfParameters(ts.getFirstConstructorWithBody(node));
                if (!decorators && !parameters) {
                    return undefined;
                }
                return {
                    decorators: decorators,
                    parameters: parameters
                };
            }
            /**
             * Gets an AllDecorators object containing the decorators for the member and its parameters.
             *
             * @param node The class node that contains the member.
             * @param member The class member.
             */
            function getAllDecoratorsOfClassElement(node, member) {
                switch (member.kind) {
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        return getAllDecoratorsOfAccessors(node, member);
                    case 161 /* MethodDeclaration */:
                        return getAllDecoratorsOfMethod(member);
                    case 159 /* PropertyDeclaration */:
                        return getAllDecoratorsOfProperty(member);
                    default:
                        return undefined;
                }
            }
            /**
             * Gets an AllDecorators object containing the decorators for the accessor and its parameters.
             *
             * @param node The class node that contains the accessor.
             * @param accessor The class accessor member.
             */
            function getAllDecoratorsOfAccessors(node, accessor) {
                if (!accessor.body) {
                    return undefined;
                }
                var _a = ts.getAllAccessorDeclarations(node.members, accessor), firstAccessor = _a.firstAccessor, secondAccessor = _a.secondAccessor, setAccessor = _a.setAccessor;
                var firstAccessorWithDecorators = firstAccessor.decorators ? firstAccessor : secondAccessor && secondAccessor.decorators ? secondAccessor : undefined;
                if (!firstAccessorWithDecorators || accessor !== firstAccessorWithDecorators) {
                    return undefined;
                }
                var decorators = firstAccessorWithDecorators.decorators;
                var parameters = getDecoratorsOfParameters(setAccessor);
                if (!decorators && !parameters) {
                    return undefined;
                }
                return { decorators: decorators, parameters: parameters };
            }
            /**
             * Gets an AllDecorators object containing the decorators for the method and its parameters.
             *
             * @param method The class method member.
             */
            function getAllDecoratorsOfMethod(method) {
                if (!method.body) {
                    return undefined;
                }
                var decorators = method.decorators;
                var parameters = getDecoratorsOfParameters(method);
                if (!decorators && !parameters) {
                    return undefined;
                }
                return { decorators: decorators, parameters: parameters };
            }
            /**
             * Gets an AllDecorators object containing the decorators for the property.
             *
             * @param property The class property member.
             */
            function getAllDecoratorsOfProperty(property) {
                var decorators = property.decorators;
                if (!decorators) {
                    return undefined;
                }
                return { decorators: decorators };
            }
            /**
             * Transforms all of the decorators for a declaration into an array of expressions.
             *
             * @param node The declaration node.
             * @param allDecorators An object containing all of the decorators for the declaration.
             */
            function transformAllDecoratorsOfDeclaration(node, container, allDecorators) {
                if (!allDecorators) {
                    return undefined;
                }
                var decoratorExpressions = [];
                ts.addRange(decoratorExpressions, ts.map(allDecorators.decorators, transformDecorator));
                ts.addRange(decoratorExpressions, ts.flatMap(allDecorators.parameters, transformDecoratorsOfParameter));
                addTypeMetadata(node, container, decoratorExpressions);
                return decoratorExpressions;
            }
            /**
             * Generates statements used to apply decorators to either the static or instance members
             * of a class.
             *
             * @param node The class node.
             * @param isStatic A value indicating whether to generate statements for static or
             *                 instance members.
             */
            function addClassElementDecorationStatements(statements, node, isStatic) {
                ts.addRange(statements, ts.map(generateClassElementDecorationExpressions(node, isStatic), expressionToStatement));
            }
            /**
             * Generates expressions used to apply decorators to either the static or instance members
             * of a class.
             *
             * @param node The class node.
             * @param isStatic A value indicating whether to generate expressions for static or
             *                 instance members.
             */
            function generateClassElementDecorationExpressions(node, isStatic) {
                var members = getDecoratedClassElements(node, isStatic);
                var expressions;
                for (var _i = 0, members_6 = members; _i < members_6.length; _i++) {
                    var member = members_6[_i];
                    var expression = generateClassElementDecorationExpression(node, member);
                    if (expression) {
                        if (!expressions) {
                            expressions = [expression];
                        }
                        else {
                            expressions.push(expression);
                        }
                    }
                }
                return expressions;
            }
            /**
             * Generates an expression used to evaluate class element decorators at runtime.
             *
             * @param node The class node that contains the member.
             * @param member The class member.
             */
            function generateClassElementDecorationExpression(node, member) {
                var allDecorators = getAllDecoratorsOfClassElement(node, member);
                var decoratorExpressions = transformAllDecoratorsOfDeclaration(member, node, allDecorators);
                if (!decoratorExpressions) {
                    return undefined;
                }
                // Emit the call to __decorate. Given the following:
                //
                //   class C {
                //     @dec method(@dec2 x) {}
                //     @dec get accessor() {}
                //     @dec prop;
                //   }
                //
                // The emit for a method is:
                //
                //   __decorate([
                //       dec,
                //       __param(0, dec2),
                //       __metadata("design:type", Function),
                //       __metadata("design:paramtypes", [Object]),
                //       __metadata("design:returntype", void 0)
                //   ], C.prototype, "method", null);
                //
                // The emit for an accessor is:
                //
                //   __decorate([
                //       dec
                //   ], C.prototype, "accessor", null);
                //
                // The emit for a property is:
                //
                //   __decorate([
                //       dec
                //   ], C.prototype, "prop");
                //
                var prefix = getClassMemberPrefix(node, member);
                var memberName = getExpressionForPropertyName(member, /*generateNameForComputedPropertyName*/ true);
                var descriptor = languageVersion > 0 /* ES3 */
                    ? member.kind === 159 /* PropertyDeclaration */
                        // We emit `void 0` here to indicate to `__decorate` that it can invoke `Object.defineProperty` directly, but that it
                        // should not invoke `Object.getOwnPropertyDescriptor`.
                        ? ts.createVoidZero()
                        // We emit `null` here to indicate to `__decorate` that it can invoke `Object.getOwnPropertyDescriptor` directly.
                        // We have this extra argument here so that we can inject an explicit property descriptor at a later date.
                        : ts.createNull()
                    : undefined;
                var helper = createDecorateHelper(context, decoratorExpressions, prefix, memberName, descriptor, ts.moveRangePastDecorators(member));
                ts.setEmitFlags(helper, 1536 /* NoComments */);
                return helper;
            }
            /**
             * Generates a __decorate helper call for a class constructor.
             *
             * @param node The class node.
             */
            function addConstructorDecorationStatement(statements, node) {
                var expression = generateConstructorDecorationExpression(node);
                if (expression) {
                    statements.push(ts.setOriginalNode(ts.createExpressionStatement(expression), node));
                }
            }
            /**
             * Generates a __decorate helper call for a class constructor.
             *
             * @param node The class node.
             */
            function generateConstructorDecorationExpression(node) {
                var allDecorators = getAllDecoratorsOfConstructor(node);
                var decoratorExpressions = transformAllDecoratorsOfDeclaration(node, node, allDecorators);
                if (!decoratorExpressions) {
                    return undefined;
                }
                var classAlias = classAliases && classAliases[ts.getOriginalNodeId(node)];
                var localName = ts.getLocalName(node, /*allowComments*/ false, /*allowSourceMaps*/ true);
                var decorate = createDecorateHelper(context, decoratorExpressions, localName);
                var expression = ts.createAssignment(localName, classAlias ? ts.createAssignment(classAlias, decorate) : decorate);
                ts.setEmitFlags(expression, 1536 /* NoComments */);
                ts.setSourceMapRange(expression, ts.moveRangePastDecorators(node));
                return expression;
            }
            /**
             * Transforms a decorator into an expression.
             *
             * @param decorator The decorator node.
             */
            function transformDecorator(decorator) {
                return ts.visitNode(decorator.expression, visitor, ts.isExpression);
            }
            /**
             * Transforms the decorators of a parameter.
             *
             * @param decorators The decorators for the parameter at the provided offset.
             * @param parameterOffset The offset of the parameter.
             */
            function transformDecoratorsOfParameter(decorators, parameterOffset) {
                var expressions;
                if (decorators) {
                    expressions = [];
                    for (var _i = 0, decorators_1 = decorators; _i < decorators_1.length; _i++) {
                        var decorator = decorators_1[_i];
                        var helper = createParamHelper(context, transformDecorator(decorator), parameterOffset, 
                        /*location*/ decorator.expression);
                        ts.setEmitFlags(helper, 1536 /* NoComments */);
                        expressions.push(helper);
                    }
                }
                return expressions;
            }
            /**
             * Adds optional type metadata for a declaration.
             *
             * @param node The declaration node.
             * @param decoratorExpressions The destination array to which to add new decorator expressions.
             */
            function addTypeMetadata(node, container, decoratorExpressions) {
                if (USE_NEW_TYPE_METADATA_FORMAT) {
                    addNewTypeMetadata(node, container, decoratorExpressions);
                }
                else {
                    addOldTypeMetadata(node, container, decoratorExpressions);
                }
            }
            function addOldTypeMetadata(node, container, decoratorExpressions) {
                if (compilerOptions.emitDecoratorMetadata) {
                    if (shouldAddTypeMetadata(node)) {
                        decoratorExpressions.push(createMetadataHelper(context, "design:type", serializeTypeOfNode(node)));
                    }
                    if (shouldAddParamTypesMetadata(node)) {
                        decoratorExpressions.push(createMetadataHelper(context, "design:paramtypes", serializeParameterTypesOfNode(node, container)));
                    }
                    if (shouldAddReturnTypeMetadata(node)) {
                        decoratorExpressions.push(createMetadataHelper(context, "design:returntype", serializeReturnTypeOfNode(node)));
                    }
                }
            }
            function addNewTypeMetadata(node, container, decoratorExpressions) {
                if (compilerOptions.emitDecoratorMetadata) {
                    var properties = void 0;
                    if (shouldAddTypeMetadata(node)) {
                        (properties || (properties = [])).push(ts.createPropertyAssignment("type", ts.createArrowFunction(/*modifiers*/ undefined, /*typeParameters*/ undefined, [], /*type*/ undefined, ts.createToken(38 /* EqualsGreaterThanToken */), serializeTypeOfNode(node))));
                    }
                    if (shouldAddParamTypesMetadata(node)) {
                        (properties || (properties = [])).push(ts.createPropertyAssignment("paramTypes", ts.createArrowFunction(/*modifiers*/ undefined, /*typeParameters*/ undefined, [], /*type*/ undefined, ts.createToken(38 /* EqualsGreaterThanToken */), serializeParameterTypesOfNode(node, container))));
                    }
                    if (shouldAddReturnTypeMetadata(node)) {
                        (properties || (properties = [])).push(ts.createPropertyAssignment("returnType", ts.createArrowFunction(/*modifiers*/ undefined, /*typeParameters*/ undefined, [], /*type*/ undefined, ts.createToken(38 /* EqualsGreaterThanToken */), serializeReturnTypeOfNode(node))));
                    }
                    if (properties) {
                        decoratorExpressions.push(createMetadataHelper(context, "design:typeinfo", ts.createObjectLiteral(properties, /*multiLine*/ true)));
                    }
                }
            }
            /**
             * Determines whether to emit the "design:type" metadata based on the node's kind.
             * The caller should have already tested whether the node has decorators and whether the
             * emitDecoratorMetadata compiler option is set.
             *
             * @param node The node to test.
             */
            function shouldAddTypeMetadata(node) {
                var kind = node.kind;
                return kind === 161 /* MethodDeclaration */
                    || kind === 163 /* GetAccessor */
                    || kind === 164 /* SetAccessor */
                    || kind === 159 /* PropertyDeclaration */;
            }
            /**
             * Determines whether to emit the "design:returntype" metadata based on the node's kind.
             * The caller should have already tested whether the node has decorators and whether the
             * emitDecoratorMetadata compiler option is set.
             *
             * @param node The node to test.
             */
            function shouldAddReturnTypeMetadata(node) {
                return node.kind === 161 /* MethodDeclaration */;
            }
            /**
             * Determines whether to emit the "design:paramtypes" metadata based on the node's kind.
             * The caller should have already tested whether the node has decorators and whether the
             * emitDecoratorMetadata compiler option is set.
             *
             * @param node The node to test.
             */
            function shouldAddParamTypesMetadata(node) {
                switch (node.kind) {
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                        return ts.getFirstConstructorWithBody(node) !== undefined;
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        return true;
                }
                return false;
            }
            function getAccessorTypeNode(node) {
                var accessors = resolver.getAllAccessorDeclarations(node);
                return accessors.setAccessor && ts.getSetAccessorTypeAnnotationNode(accessors.setAccessor)
                    || accessors.getAccessor && ts.getEffectiveReturnTypeNode(accessors.getAccessor);
            }
            /**
             * Serializes the type of a node for use with decorator type metadata.
             *
             * @param node The node that should have its type serialized.
             */
            function serializeTypeOfNode(node) {
                switch (node.kind) {
                    case 159 /* PropertyDeclaration */:
                    case 156 /* Parameter */:
                        return serializeTypeNode(node.type);
                    case 164 /* SetAccessor */:
                    case 163 /* GetAccessor */:
                        return serializeTypeNode(getAccessorTypeNode(node));
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                    case 161 /* MethodDeclaration */:
                        return ts.createIdentifier("Function");
                    default:
                        return ts.createVoidZero();
                }
            }
            /**
             * Serializes the types of the parameters of a node for use with decorator type metadata.
             *
             * @param node The node that should have its parameter types serialized.
             */
            function serializeParameterTypesOfNode(node, container) {
                var valueDeclaration = ts.isClassLike(node)
                    ? ts.getFirstConstructorWithBody(node)
                    : ts.isFunctionLike(node) && ts.nodeIsPresent(node.body)
                        ? node
                        : undefined;
                var expressions = [];
                if (valueDeclaration) {
                    var parameters = getParametersOfDecoratedDeclaration(valueDeclaration, container);
                    var numParameters = parameters.length;
                    for (var i = 0; i < numParameters; i++) {
                        var parameter = parameters[i];
                        if (i === 0 && ts.isIdentifier(parameter.name) && parameter.name.escapedText === "this") {
                            continue;
                        }
                        if (parameter.dotDotDotToken) {
                            expressions.push(serializeTypeNode(ts.getRestParameterElementType(parameter.type)));
                        }
                        else {
                            expressions.push(serializeTypeOfNode(parameter));
                        }
                    }
                }
                return ts.createArrayLiteral(expressions);
            }
            function getParametersOfDecoratedDeclaration(node, container) {
                if (container && node.kind === 163 /* GetAccessor */) {
                    var setAccessor = ts.getAllAccessorDeclarations(container.members, node).setAccessor;
                    if (setAccessor) {
                        return setAccessor.parameters;
                    }
                }
                return node.parameters;
            }
            /**
             * Serializes the return type of a node for use with decorator type metadata.
             *
             * @param node The node that should have its return type serialized.
             */
            function serializeReturnTypeOfNode(node) {
                if (ts.isFunctionLike(node) && node.type) {
                    return serializeTypeNode(node.type);
                }
                else if (ts.isAsyncFunction(node)) {
                    return ts.createIdentifier("Promise");
                }
                return ts.createVoidZero();
            }
            /**
             * Serializes a type node for use with decorator type metadata.
             *
             * Types are serialized in the following fashion:
             * - Void types point to "undefined" (e.g. "void 0")
             * - Function and Constructor types point to the global "Function" constructor.
             * - Interface types with a call or construct signature types point to the global
             *   "Function" constructor.
             * - Array and Tuple types point to the global "Array" constructor.
             * - Type predicates and booleans point to the global "Boolean" constructor.
             * - String literal types and strings point to the global "String" constructor.
             * - Enum and number types point to the global "Number" constructor.
             * - Symbol types point to the global "Symbol" constructor.
             * - Type references to classes (or class-like variables) point to the constructor for the class.
             * - Anything else points to the global "Object" constructor.
             *
             * @param node The type node to serialize.
             */
            function serializeTypeNode(node) {
                if (node === undefined) {
                    return ts.createIdentifier("Object");
                }
                switch (node.kind) {
                    case 110 /* VoidKeyword */:
                    case 146 /* UndefinedKeyword */:
                    case 100 /* NullKeyword */:
                    case 137 /* NeverKeyword */:
                        return ts.createVoidZero();
                    case 182 /* ParenthesizedType */:
                        return serializeTypeNode(node.type);
                    case 170 /* FunctionType */:
                    case 171 /* ConstructorType */:
                        return ts.createIdentifier("Function");
                    case 174 /* ArrayType */:
                    case 175 /* TupleType */:
                        return ts.createIdentifier("Array");
                    case 168 /* TypePredicate */:
                    case 128 /* BooleanKeyword */:
                        return ts.createIdentifier("Boolean");
                    case 143 /* StringKeyword */:
                        return ts.createIdentifier("String");
                    case 141 /* ObjectKeyword */:
                        return ts.createIdentifier("Object");
                    case 187 /* LiteralType */:
                        switch (node.literal.kind) {
                            case 10 /* StringLiteral */:
                                return ts.createIdentifier("String");
                            case 207 /* PrefixUnaryExpression */:
                            case 8 /* NumericLiteral */:
                                return ts.createIdentifier("Number");
                            case 9 /* BigIntLiteral */:
                                return getGlobalBigIntNameWithFallback();
                            case 106 /* TrueKeyword */:
                            case 91 /* FalseKeyword */:
                                return ts.createIdentifier("Boolean");
                            default:
                                return ts.Debug.failBadSyntaxKind(node.literal);
                        }
                    case 140 /* NumberKeyword */:
                        return ts.createIdentifier("Number");
                    case 151 /* BigIntKeyword */:
                        return getGlobalBigIntNameWithFallback();
                    case 144 /* SymbolKeyword */:
                        return languageVersion < 2 /* ES2015 */
                            ? getGlobalSymbolNameWithFallback()
                            : ts.createIdentifier("Symbol");
                    case 169 /* TypeReference */:
                        return serializeTypeReferenceNode(node);
                    case 179 /* IntersectionType */:
                    case 178 /* UnionType */:
                        return serializeTypeList(node.types);
                    case 180 /* ConditionalType */:
                        return serializeTypeList([node.trueType, node.falseType]);
                    case 184 /* TypeOperator */:
                        if (node.operator === 138 /* ReadonlyKeyword */) {
                            return serializeTypeNode(node.type);
                        }
                        break;
                    case 172 /* TypeQuery */:
                    case 185 /* IndexedAccessType */:
                    case 186 /* MappedType */:
                    case 173 /* TypeLiteral */:
                    case 125 /* AnyKeyword */:
                    case 148 /* UnknownKeyword */:
                    case 183 /* ThisType */:
                    case 188 /* ImportType */:
                        break;
                    default:
                        return ts.Debug.failBadSyntaxKind(node);
                }
                return ts.createIdentifier("Object");
            }
            function serializeTypeList(types) {
                // Note when updating logic here also update getEntityNameForDecoratorMetadata
                // so that aliases can be marked as referenced
                var serializedUnion;
                for (var _i = 0, types_20 = types; _i < types_20.length; _i++) {
                    var typeNode = types_20[_i];
                    while (typeNode.kind === 182 /* ParenthesizedType */) {
                        typeNode = typeNode.type; // Skip parens if need be
                    }
                    if (typeNode.kind === 137 /* NeverKeyword */) {
                        continue; // Always elide `never` from the union/intersection if possible
                    }
                    if (!strictNullChecks && (typeNode.kind === 100 /* NullKeyword */ || typeNode.kind === 146 /* UndefinedKeyword */)) {
                        continue; // Elide null and undefined from unions for metadata, just like what we did prior to the implementation of strict null checks
                    }
                    var serializedIndividual = serializeTypeNode(typeNode);
                    if (ts.isIdentifier(serializedIndividual) && serializedIndividual.escapedText === "Object") {
                        // One of the individual is global object, return immediately
                        return serializedIndividual;
                    }
                    // If there exists union that is not void 0 expression, check if the the common type is identifier.
                    // anything more complex and we will just default to Object
                    else if (serializedUnion) {
                        // Different types
                        if (!ts.isIdentifier(serializedUnion) ||
                            !ts.isIdentifier(serializedIndividual) ||
                            serializedUnion.escapedText !== serializedIndividual.escapedText) {
                            return ts.createIdentifier("Object");
                        }
                    }
                    else {
                        // Initialize the union type
                        serializedUnion = serializedIndividual;
                    }
                }
                // If we were able to find common type, use it
                return serializedUnion || ts.createVoidZero(); // Fallback is only hit if all union constituients are null/undefined/never
            }
            /**
             * Serializes a TypeReferenceNode to an appropriate JS constructor value for use with
             * decorator type metadata.
             *
             * @param node The type reference node.
             */
            function serializeTypeReferenceNode(node) {
                var kind = resolver.getTypeReferenceSerializationKind(node.typeName, currentNameScope || currentLexicalScope);
                switch (kind) {
                    case ts.TypeReferenceSerializationKind.Unknown:
                        // From conditional type type reference that cannot be resolved is Similar to any or unknown
                        if (ts.findAncestor(node, function (n) { return n.parent && ts.isConditionalTypeNode(n.parent) && (n.parent.trueType === n || n.parent.falseType === n); })) {
                            return ts.createIdentifier("Object");
                        }
                        var serialized = serializeEntityNameAsExpressionFallback(node.typeName);
                        var temp = ts.createTempVariable(hoistVariableDeclaration);
                        return ts.createConditional(ts.createTypeCheck(ts.createAssignment(temp, serialized), "function"), temp, ts.createIdentifier("Object"));
                    case ts.TypeReferenceSerializationKind.TypeWithConstructSignatureAndValue:
                        return serializeEntityNameAsExpression(node.typeName);
                    case ts.TypeReferenceSerializationKind.VoidNullableOrNeverType:
                        return ts.createVoidZero();
                    case ts.TypeReferenceSerializationKind.BigIntLikeType:
                        return getGlobalBigIntNameWithFallback();
                    case ts.TypeReferenceSerializationKind.BooleanType:
                        return ts.createIdentifier("Boolean");
                    case ts.TypeReferenceSerializationKind.NumberLikeType:
                        return ts.createIdentifier("Number");
                    case ts.TypeReferenceSerializationKind.StringLikeType:
                        return ts.createIdentifier("String");
                    case ts.TypeReferenceSerializationKind.ArrayLikeType:
                        return ts.createIdentifier("Array");
                    case ts.TypeReferenceSerializationKind.ESSymbolType:
                        return languageVersion < 2 /* ES2015 */
                            ? getGlobalSymbolNameWithFallback()
                            : ts.createIdentifier("Symbol");
                    case ts.TypeReferenceSerializationKind.TypeWithCallSignature:
                        return ts.createIdentifier("Function");
                    case ts.TypeReferenceSerializationKind.Promise:
                        return ts.createIdentifier("Promise");
                    case ts.TypeReferenceSerializationKind.ObjectType:
                        return ts.createIdentifier("Object");
                    default:
                        return ts.Debug.assertNever(kind);
                }
            }
            function createCheckedValue(left, right) {
                return ts.createLogicalAnd(ts.createStrictInequality(ts.createTypeOf(left), ts.createLiteral("undefined")), right);
            }
            /**
             * Serializes an entity name which may not exist at runtime, but whose access shouldn't throw
             *
             * @param node The entity name to serialize.
             */
            function serializeEntityNameAsExpressionFallback(node) {
                if (node.kind === 75 /* Identifier */) {
                    // A -> typeof A !== undefined && A
                    var copied = serializeEntityNameAsExpression(node);
                    return createCheckedValue(copied, copied);
                }
                if (node.left.kind === 75 /* Identifier */) {
                    // A.B -> typeof A !== undefined && A.B
                    return createCheckedValue(serializeEntityNameAsExpression(node.left), serializeEntityNameAsExpression(node));
                }
                // A.B.C -> typeof A !== undefined && (_a = A.B) !== void 0 && _a.C
                var left = serializeEntityNameAsExpressionFallback(node.left);
                var temp = ts.createTempVariable(hoistVariableDeclaration);
                return ts.createLogicalAnd(ts.createLogicalAnd(left.left, ts.createStrictInequality(ts.createAssignment(temp, left.right), ts.createVoidZero())), ts.createPropertyAccess(temp, node.right));
            }
            /**
             * Serializes an entity name as an expression for decorator type metadata.
             *
             * @param node The entity name to serialize.
             */
            function serializeEntityNameAsExpression(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                        // Create a clone of the name with a new parent, and treat it as if it were
                        // a source tree node for the purposes of the checker.
                        var name = ts.getMutableClone(node);
                        name.flags &= ~8 /* Synthesized */;
                        name.original = undefined;
                        name.parent = ts.getParseTreeNode(currentLexicalScope); // ensure the parent is set to a parse tree node.
                        return name;
                    case 153 /* QualifiedName */:
                        return serializeQualifiedNameAsExpression(node);
                }
            }
            /**
             * Serializes an qualified name as an expression for decorator type metadata.
             *
             * @param node The qualified name to serialize.
             * @param useFallback A value indicating whether to use logical operators to test for the
             *                    qualified name at runtime.
             */
            function serializeQualifiedNameAsExpression(node) {
                return ts.createPropertyAccess(serializeEntityNameAsExpression(node.left), node.right);
            }
            /**
             * Gets an expression that points to the global "Symbol" constructor at runtime if it is
             * available.
             */
            function getGlobalSymbolNameWithFallback() {
                return ts.createConditional(ts.createTypeCheck(ts.createIdentifier("Symbol"), "function"), ts.createIdentifier("Symbol"), ts.createIdentifier("Object"));
            }
            /**
             * Gets an expression that points to the global "BigInt" constructor at runtime if it is
             * available.
             */
            function getGlobalBigIntNameWithFallback() {
                return languageVersion < 99 /* ESNext */
                    ? ts.createConditional(ts.createTypeCheck(ts.createIdentifier("BigInt"), "function"), ts.createIdentifier("BigInt"), ts.createIdentifier("Object"))
                    : ts.createIdentifier("BigInt");
            }
            /**
             * Gets an expression that represents a property name (for decorated properties or enums).
             * For a computed property, a name is generated for the node.
             *
             * @param member The member whose name should be converted into an expression.
             */
            function getExpressionForPropertyName(member, generateNameForComputedPropertyName) {
                var name = member.name;
                if (ts.isPrivateIdentifier(name)) {
                    return ts.createIdentifier("");
                }
                else if (ts.isComputedPropertyName(name)) {
                    return generateNameForComputedPropertyName && !ts.isSimpleInlineableExpression(name.expression)
                        ? ts.getGeneratedNameForNode(name)
                        : name.expression;
                }
                else if (ts.isIdentifier(name)) {
                    return ts.createLiteral(ts.idText(name));
                }
                else {
                    return ts.getSynthesizedClone(name);
                }
            }
            /**
             * Visits the property name of a class element, for use when emitting property
             * initializers. For a computed property on a node with decorators, a temporary
             * value is stored for later use.
             *
             * @param member The member whose name should be visited.
             */
            function visitPropertyNameOfClassElement(member) {
                var name = member.name;
                // Computed property names need to be transformed into a hoisted variable when they are used more than once.
                // The names are used more than once when:
                //   - the property is non-static and its initializer is moved to the constructor (when there are parameter property assignments).
                //   - the property has a decorator.
                if (ts.isComputedPropertyName(name) && ((!ts.hasStaticModifier(member) && currentClassHasParameterProperties) || ts.some(member.decorators))) {
                    var expression = ts.visitNode(name.expression, visitor, ts.isExpression);
                    var innerExpression = ts.skipPartiallyEmittedExpressions(expression);
                    if (!ts.isSimpleInlineableExpression(innerExpression)) {
                        var generatedName = ts.getGeneratedNameForNode(name);
                        hoistVariableDeclaration(generatedName);
                        return ts.updateComputedPropertyName(name, ts.createAssignment(generatedName, expression));
                    }
                }
                return ts.visitNode(name, visitor, ts.isPropertyName);
            }
            /**
             * Transforms a HeritageClause with TypeScript syntax.
             *
             * This function will only be called when one of the following conditions are met:
             * - The node is a non-`extends` heritage clause that should be elided.
             * - The node is an `extends` heritage clause that should be visited, but only allow a single type.
             *
             * @param node The HeritageClause to transform.
             */
            function visitHeritageClause(node) {
                if (node.token === 113 /* ImplementsKeyword */) {
                    // implements clauses are elided
                    return undefined;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Transforms an ExpressionWithTypeArguments with TypeScript syntax.
             *
             * This function will only be called when one of the following conditions are met:
             * - The node contains type arguments that should be elided.
             *
             * @param node The ExpressionWithTypeArguments to transform.
             */
            function visitExpressionWithTypeArguments(node) {
                return ts.updateExpressionWithTypeArguments(node, 
                /*typeArguments*/ undefined, ts.visitNode(node.expression, visitor, ts.isLeftHandSideExpression));
            }
            /**
             * Determines whether to emit a function-like declaration. We should not emit the
             * declaration if it does not have a body.
             *
             * @param node The declaration node.
             */
            function shouldEmitFunctionLikeDeclaration(node) {
                return !ts.nodeIsMissing(node.body);
            }
            function visitPropertyDeclaration(node) {
                if (node.flags & 8388608 /* Ambient */) {
                    return undefined;
                }
                var updated = ts.updateProperty(node, 
                /*decorators*/ undefined, ts.visitNodes(node.modifiers, visitor, ts.isModifier), visitPropertyNameOfClassElement(node), 
                /*questionOrExclamationToken*/ undefined, 
                /*type*/ undefined, ts.visitNode(node.initializer, visitor));
                if (updated !== node) {
                    // While we emit the source map for the node after skipping decorators and modifiers,
                    // we need to emit the comments for the original range.
                    ts.setCommentRange(updated, node);
                    ts.setSourceMapRange(updated, ts.moveRangePastDecorators(node));
                }
                return updated;
            }
            function visitConstructor(node) {
                if (!shouldEmitFunctionLikeDeclaration(node)) {
                    return undefined;
                }
                return ts.updateConstructor(node, 
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, ts.visitParameterList(node.parameters, visitor, context), transformConstructorBody(node.body, node));
            }
            function transformConstructorBody(body, constructor) {
                var parametersWithPropertyAssignments = constructor &&
                    ts.filter(constructor.parameters, function (p) { return ts.isParameterPropertyDeclaration(p, constructor); });
                if (!ts.some(parametersWithPropertyAssignments)) {
                    return ts.visitFunctionBody(body, visitor, context);
                }
                var statements = [];
                var indexOfFirstStatement = 0;
                resumeLexicalEnvironment();
                indexOfFirstStatement = ts.addPrologueDirectivesAndInitialSuperCall(constructor, statements, visitor);
                // Add parameters with property assignments. Transforms this:
                //
                //  constructor (public x, public y) {
                //  }
                //
                // Into this:
                //
                //  constructor (x, y) {
                //      this.x = x;
                //      this.y = y;
                //  }
                //
                ts.addRange(statements, ts.map(parametersWithPropertyAssignments, transformParameterWithPropertyAssignment));
                // Add the existing statements, skipping the initial super call.
                ts.addRange(statements, ts.visitNodes(body.statements, visitor, ts.isStatement, indexOfFirstStatement));
                // End the lexical environment.
                statements = ts.mergeLexicalEnvironment(statements, endLexicalEnvironment());
                var block = ts.createBlock(ts.setTextRange(ts.createNodeArray(statements), body.statements), /*multiLine*/ true);
                ts.setTextRange(block, /*location*/ body);
                ts.setOriginalNode(block, body);
                return block;
            }
            /**
             * Transforms a parameter into a property assignment statement.
             *
             * @param node The parameter declaration.
             */
            function transformParameterWithPropertyAssignment(node) {
                var name = node.name;
                if (!ts.isIdentifier(name)) {
                    return undefined;
                }
                var propertyName = ts.getMutableClone(name);
                ts.setEmitFlags(propertyName, 1536 /* NoComments */ | 48 /* NoSourceMap */);
                var localName = ts.getMutableClone(name);
                ts.setEmitFlags(localName, 1536 /* NoComments */);
                return ts.startOnNewLine(ts.removeAllComments(ts.setTextRange(ts.setOriginalNode(ts.createExpressionStatement(ts.createAssignment(ts.setTextRange(ts.createPropertyAccess(ts.createThis(), propertyName), node.name), localName)), node), ts.moveRangePos(node, -1))));
            }
            function visitMethodDeclaration(node) {
                if (!shouldEmitFunctionLikeDeclaration(node)) {
                    return undefined;
                }
                var updated = ts.updateMethod(node, 
                /*decorators*/ undefined, ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier), node.asteriskToken, visitPropertyNameOfClassElement(node), 
                /*questionToken*/ undefined, 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, ts.visitFunctionBody(node.body, visitor, context));
                if (updated !== node) {
                    // While we emit the source map for the node after skipping decorators and modifiers,
                    // we need to emit the comments for the original range.
                    ts.setCommentRange(updated, node);
                    ts.setSourceMapRange(updated, ts.moveRangePastDecorators(node));
                }
                return updated;
            }
            /**
             * Determines whether to emit an accessor declaration. We should not emit the
             * declaration if it does not have a body and is abstract.
             *
             * @param node The declaration node.
             */
            function shouldEmitAccessorDeclaration(node) {
                return !(ts.nodeIsMissing(node.body) && ts.hasModifier(node, 128 /* Abstract */));
            }
            function visitGetAccessor(node) {
                if (!shouldEmitAccessorDeclaration(node)) {
                    return undefined;
                }
                var updated = ts.updateGetAccessor(node, 
                /*decorators*/ undefined, ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier), visitPropertyNameOfClassElement(node), ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, ts.visitFunctionBody(node.body, visitor, context) || ts.createBlock([]));
                if (updated !== node) {
                    // While we emit the source map for the node after skipping decorators and modifiers,
                    // we need to emit the comments for the original range.
                    ts.setCommentRange(updated, node);
                    ts.setSourceMapRange(updated, ts.moveRangePastDecorators(node));
                }
                return updated;
            }
            function visitSetAccessor(node) {
                if (!shouldEmitAccessorDeclaration(node)) {
                    return undefined;
                }
                var updated = ts.updateSetAccessor(node, 
                /*decorators*/ undefined, ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier), visitPropertyNameOfClassElement(node), ts.visitParameterList(node.parameters, visitor, context), ts.visitFunctionBody(node.body, visitor, context) || ts.createBlock([]));
                if (updated !== node) {
                    // While we emit the source map for the node after skipping decorators and modifiers,
                    // we need to emit the comments for the original range.
                    ts.setCommentRange(updated, node);
                    ts.setSourceMapRange(updated, ts.moveRangePastDecorators(node));
                }
                return updated;
            }
            function visitFunctionDeclaration(node) {
                if (!shouldEmitFunctionLikeDeclaration(node)) {
                    return ts.createNotEmittedStatement(node);
                }
                var updated = ts.updateFunctionDeclaration(node, 
                /*decorators*/ undefined, ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier), node.asteriskToken, node.name, 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, ts.visitFunctionBody(node.body, visitor, context) || ts.createBlock([]));
                if (isExportOfNamespace(node)) {
                    var statements = [updated];
                    addExportMemberAssignment(statements, node);
                    return statements;
                }
                return updated;
            }
            function visitFunctionExpression(node) {
                if (!shouldEmitFunctionLikeDeclaration(node)) {
                    return ts.createOmittedExpression();
                }
                var updated = ts.updateFunctionExpression(node, ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier), node.asteriskToken, node.name, 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, ts.visitFunctionBody(node.body, visitor, context) || ts.createBlock([]));
                return updated;
            }
            function visitArrowFunction(node) {
                var updated = ts.updateArrowFunction(node, ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier), 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, node.equalsGreaterThanToken, ts.visitFunctionBody(node.body, visitor, context));
                return updated;
            }
            function visitParameter(node) {
                if (ts.parameterIsThisKeyword(node)) {
                    return undefined;
                }
                var updated = ts.updateParameter(node, 
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, node.dotDotDotToken, ts.visitNode(node.name, visitor, ts.isBindingName), 
                /*questionToken*/ undefined, 
                /*type*/ undefined, ts.visitNode(node.initializer, visitor, ts.isExpression));
                if (updated !== node) {
                    // While we emit the source map for the node after skipping decorators and modifiers,
                    // we need to emit the comments for the original range.
                    ts.setCommentRange(updated, node);
                    ts.setTextRange(updated, ts.moveRangePastModifiers(node));
                    ts.setSourceMapRange(updated, ts.moveRangePastModifiers(node));
                    ts.setEmitFlags(updated.name, 32 /* NoTrailingSourceMap */);
                }
                return updated;
            }
            function visitVariableStatement(node) {
                if (isExportOfNamespace(node)) {
                    var variables = ts.getInitializedVariables(node.declarationList);
                    if (variables.length === 0) {
                        // elide statement if there are no initialized variables.
                        return undefined;
                    }
                    return ts.setTextRange(ts.createExpressionStatement(ts.inlineExpressions(ts.map(variables, transformInitializedVariable))), node);
                }
                else {
                    return ts.visitEachChild(node, visitor, context);
                }
            }
            function transformInitializedVariable(node) {
                var name = node.name;
                if (ts.isBindingPattern(name)) {
                    return ts.flattenDestructuringAssignment(node, visitor, context, 0 /* All */, 
                    /*needsValue*/ false, createNamespaceExportExpression);
                }
                else {
                    return ts.setTextRange(ts.createAssignment(getNamespaceMemberNameWithSourceMapsAndWithoutComments(name), ts.visitNode(node.initializer, visitor, ts.isExpression)), 
                    /*location*/ node);
                }
            }
            function visitVariableDeclaration(node) {
                return ts.updateTypeScriptVariableDeclaration(node, ts.visitNode(node.name, visitor, ts.isBindingName), 
                /*exclaimationToken*/ undefined, 
                /*type*/ undefined, ts.visitNode(node.initializer, visitor, ts.isExpression));
            }
            function visitParenthesizedExpression(node) {
                var innerExpression = ts.skipOuterExpressions(node.expression, ~2 /* Assertions */);
                if (ts.isAssertionExpression(innerExpression)) {
                    // Make sure we consider all nested cast expressions, e.g.:
                    // (<any><number><any>-A).x;
                    var expression = ts.visitNode(node.expression, visitor, ts.isExpression);
                    // We have an expression of the form: (<Type>SubExpr). Emitting this as (SubExpr)
                    // is really not desirable. We would like to emit the subexpression as-is. Omitting
                    // the parentheses, however, could cause change in the semantics of the generated
                    // code if the casted expression has a lower precedence than the rest of the
                    // expression.
                    //
                    // To preserve comments, we return a "PartiallyEmittedExpression" here which will
                    // preserve the position information of the original expression.
                    //
                    // Due to the auto-parenthesization rules used by the visitor and factory functions
                    // we can safely elide the parentheses here, as a new synthetic
                    // ParenthesizedExpression will be inserted if we remove parentheses too
                    // aggressively.
                    // HOWEVER - if there are leading comments on the expression itself, to handle ASI
                    // correctly for return and throw, we must keep the parenthesis
                    if (ts.length(ts.getLeadingCommentRangesOfNode(expression, currentSourceFile))) {
                        return ts.updateParen(node, expression);
                    }
                    return ts.createPartiallyEmittedExpression(expression, node);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitAssertionExpression(node) {
                var expression = ts.visitNode(node.expression, visitor, ts.isExpression);
                return ts.createPartiallyEmittedExpression(expression, node);
            }
            function visitNonNullExpression(node) {
                var expression = ts.visitNode(node.expression, visitor, ts.isLeftHandSideExpression);
                return ts.createPartiallyEmittedExpression(expression, node);
            }
            function visitCallExpression(node) {
                return ts.updateCall(node, ts.visitNode(node.expression, visitor, ts.isExpression), 
                /*typeArguments*/ undefined, ts.visitNodes(node.arguments, visitor, ts.isExpression));
            }
            function visitNewExpression(node) {
                return ts.updateNew(node, ts.visitNode(node.expression, visitor, ts.isExpression), 
                /*typeArguments*/ undefined, ts.visitNodes(node.arguments, visitor, ts.isExpression));
            }
            function visitTaggedTemplateExpression(node) {
                return ts.updateTaggedTemplate(node, ts.visitNode(node.tag, visitor, ts.isExpression), 
                /*typeArguments*/ undefined, ts.visitNode(node.template, visitor, ts.isExpression));
            }
            /**
             * Determines whether to emit an enum declaration.
             *
             * @param node The enum declaration node.
             */
            function shouldEmitEnumDeclaration(node) {
                return !ts.isEnumConst(node)
                    || compilerOptions.preserveConstEnums
                    || compilerOptions.isolatedModules;
            }
            /**
             * Visits an enum declaration.
             *
             * This function will be called any time a TypeScript enum is encountered.
             *
             * @param node The enum declaration node.
             */
            function visitEnumDeclaration(node) {
                if (!shouldEmitEnumDeclaration(node)) {
                    return ts.createNotEmittedStatement(node);
                }
                var statements = [];
                // We request to be advised when the printer is about to print this node. This allows
                // us to set up the correct state for later substitutions.
                var emitFlags = 2 /* AdviseOnEmitNode */;
                // If needed, we should emit a variable declaration for the enum. If we emit
                // a leading variable declaration, we should not emit leading comments for the
                // enum body.
                var varAdded = addVarForEnumOrModuleDeclaration(statements, node);
                if (varAdded) {
                    // We should still emit the comments if we are emitting a system module.
                    if (moduleKind !== ts.ModuleKind.System || currentLexicalScope !== currentSourceFile) {
                        emitFlags |= 512 /* NoLeadingComments */;
                    }
                }
                // `parameterName` is the declaration name used inside of the enum.
                var parameterName = getNamespaceParameterName(node);
                // `containerName` is the expression used inside of the enum for assignments.
                var containerName = getNamespaceContainerName(node);
                // `exportName` is the expression used within this node's container for any exported references.
                var exportName = ts.hasModifier(node, 1 /* Export */)
                    ? ts.getExternalModuleOrNamespaceExportName(currentNamespaceContainerName, node, /*allowComments*/ false, /*allowSourceMaps*/ true)
                    : ts.getLocalName(node, /*allowComments*/ false, /*allowSourceMaps*/ true);
                //  x || (x = {})
                //  exports.x || (exports.x = {})
                var moduleArg = ts.createLogicalOr(exportName, ts.createAssignment(exportName, ts.createObjectLiteral()));
                if (hasNamespaceQualifiedExportName(node)) {
                    // `localName` is the expression used within this node's containing scope for any local references.
                    var localName = ts.getLocalName(node, /*allowComments*/ false, /*allowSourceMaps*/ true);
                    //  x = (exports.x || (exports.x = {}))
                    moduleArg = ts.createAssignment(localName, moduleArg);
                }
                //  (function (x) {
                //      x[x["y"] = 0] = "y";
                //      ...
                //  })(x || (x = {}));
                var enumStatement = ts.createExpressionStatement(ts.createCall(ts.createFunctionExpression(
                /*modifiers*/ undefined, 
                /*asteriskToken*/ undefined, 
                /*name*/ undefined, 
                /*typeParameters*/ undefined, [ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, parameterName)], 
                /*type*/ undefined, transformEnumBody(node, containerName)), 
                /*typeArguments*/ undefined, [moduleArg]));
                ts.setOriginalNode(enumStatement, node);
                if (varAdded) {
                    // If a variable was added, synthetic comments are emitted on it, not on the moduleStatement.
                    ts.setSyntheticLeadingComments(enumStatement, undefined);
                    ts.setSyntheticTrailingComments(enumStatement, undefined);
                }
                ts.setTextRange(enumStatement, node);
                ts.addEmitFlags(enumStatement, emitFlags);
                statements.push(enumStatement);
                // Add a DeclarationMarker for the enum to preserve trailing comments and mark
                // the end of the declaration.
                statements.push(ts.createEndOfDeclarationMarker(node));
                return statements;
            }
            /**
             * Transforms the body of an enum declaration.
             *
             * @param node The enum declaration node.
             */
            function transformEnumBody(node, localName) {
                var savedCurrentNamespaceLocalName = currentNamespaceContainerName;
                currentNamespaceContainerName = localName;
                var statements = [];
                startLexicalEnvironment();
                var members = ts.map(node.members, transformEnumMember);
                ts.insertStatementsAfterStandardPrologue(statements, endLexicalEnvironment());
                ts.addRange(statements, members);
                currentNamespaceContainerName = savedCurrentNamespaceLocalName;
                return ts.createBlock(ts.setTextRange(ts.createNodeArray(statements), /*location*/ node.members), 
                /*multiLine*/ true);
            }
            /**
             * Transforms an enum member into a statement.
             *
             * @param member The enum member node.
             */
            function transformEnumMember(member) {
                // enums don't support computed properties
                // we pass false as 'generateNameForComputedPropertyName' for a backward compatibility purposes
                // old emitter always generate 'expression' part of the name as-is.
                var name = getExpressionForPropertyName(member, /*generateNameForComputedPropertyName*/ false);
                var valueExpression = transformEnumMemberDeclarationValue(member);
                var innerAssignment = ts.createAssignment(ts.createElementAccess(currentNamespaceContainerName, name), valueExpression);
                var outerAssignment = valueExpression.kind === 10 /* StringLiteral */ ?
                    innerAssignment :
                    ts.createAssignment(ts.createElementAccess(currentNamespaceContainerName, innerAssignment), name);
                return ts.setTextRange(ts.createExpressionStatement(ts.setTextRange(outerAssignment, member)), member);
            }
            /**
             * Transforms the value of an enum member.
             *
             * @param member The enum member node.
             */
            function transformEnumMemberDeclarationValue(member) {
                var value = resolver.getConstantValue(member);
                if (value !== undefined) {
                    return ts.createLiteral(value);
                }
                else {
                    enableSubstitutionForNonQualifiedEnumMembers();
                    if (member.initializer) {
                        return ts.visitNode(member.initializer, visitor, ts.isExpression);
                    }
                    else {
                        return ts.createVoidZero();
                    }
                }
            }
            /**
             * Determines whether to elide a module declaration.
             *
             * @param node The module declaration node.
             */
            function shouldEmitModuleDeclaration(nodeIn) {
                var node = ts.getParseTreeNode(nodeIn, ts.isModuleDeclaration);
                if (!node) {
                    // If we can't find a parse tree node, assume the node is instantiated.
                    return true;
                }
                return ts.isInstantiatedModule(node, !!compilerOptions.preserveConstEnums || !!compilerOptions.isolatedModules);
            }
            /**
             * Determines whether an exported declaration will have a qualified export name (e.g. `f.x`
             * or `exports.x`).
             */
            function hasNamespaceQualifiedExportName(node) {
                return isExportOfNamespace(node)
                    || (isExternalModuleExport(node)
                        && moduleKind !== ts.ModuleKind.ES2015
                        && moduleKind !== ts.ModuleKind.ES2020
                        && moduleKind !== ts.ModuleKind.ESNext
                        && moduleKind !== ts.ModuleKind.System);
            }
            /**
             * Records that a declaration was emitted in the current scope, if it was the first
             * declaration for the provided symbol.
             */
            function recordEmittedDeclarationInScope(node) {
                if (!currentScopeFirstDeclarationsOfName) {
                    currentScopeFirstDeclarationsOfName = ts.createUnderscoreEscapedMap();
                }
                var name = declaredNameInScope(node);
                if (!currentScopeFirstDeclarationsOfName.has(name)) {
                    currentScopeFirstDeclarationsOfName.set(name, node);
                }
            }
            /**
             * Determines whether a declaration is the first declaration with
             * the same name emitted in the current scope.
             */
            function isFirstEmittedDeclarationInScope(node) {
                if (currentScopeFirstDeclarationsOfName) {
                    var name = declaredNameInScope(node);
                    return currentScopeFirstDeclarationsOfName.get(name) === node;
                }
                return true;
            }
            function declaredNameInScope(node) {
                ts.Debug.assertNode(node.name, ts.isIdentifier);
                return node.name.escapedText;
            }
            /**
             * Adds a leading VariableStatement for a enum or module declaration.
             */
            function addVarForEnumOrModuleDeclaration(statements, node) {
                // Emit a variable statement for the module. We emit top-level enums as a `var`
                // declaration to avoid static errors in global scripts scripts due to redeclaration.
                // enums in any other scope are emitted as a `let` declaration.
                var statement = ts.createVariableStatement(ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier), ts.createVariableDeclarationList([
                    ts.createVariableDeclaration(ts.getLocalName(node, /*allowComments*/ false, /*allowSourceMaps*/ true))
                ], currentLexicalScope.kind === 290 /* SourceFile */ ? 0 /* None */ : 1 /* Let */));
                ts.setOriginalNode(statement, node);
                recordEmittedDeclarationInScope(node);
                if (isFirstEmittedDeclarationInScope(node)) {
                    // Adjust the source map emit to match the old emitter.
                    if (node.kind === 248 /* EnumDeclaration */) {
                        ts.setSourceMapRange(statement.declarationList, node);
                    }
                    else {
                        ts.setSourceMapRange(statement, node);
                    }
                    // Trailing comments for module declaration should be emitted after the function closure
                    // instead of the variable statement:
                    //
                    //     /** Module comment*/
                    //     module m1 {
                    //         function foo4Export() {
                    //         }
                    //     } // trailing comment module
                    //
                    // Should emit:
                    //
                    //     /** Module comment*/
                    //     var m1;
                    //     (function (m1) {
                    //         function foo4Export() {
                    //         }
                    //     })(m1 || (m1 = {})); // trailing comment module
                    //
                    ts.setCommentRange(statement, node);
                    ts.addEmitFlags(statement, 1024 /* NoTrailingComments */ | 4194304 /* HasEndOfDeclarationMarker */);
                    statements.push(statement);
                    return true;
                }
                else {
                    // For an EnumDeclaration or ModuleDeclaration that merges with a preceeding
                    // declaration we do not emit a leading variable declaration. To preserve the
                    // begin/end semantics of the declararation and to properly handle exports
                    // we wrap the leading variable declaration in a `MergeDeclarationMarker`.
                    var mergeMarker = ts.createMergeDeclarationMarker(statement);
                    ts.setEmitFlags(mergeMarker, 1536 /* NoComments */ | 4194304 /* HasEndOfDeclarationMarker */);
                    statements.push(mergeMarker);
                    return false;
                }
            }
            /**
             * Visits a module declaration node.
             *
             * This function will be called any time a TypeScript namespace (ModuleDeclaration) is encountered.
             *
             * @param node The module declaration node.
             */
            function visitModuleDeclaration(node) {
                if (!shouldEmitModuleDeclaration(node)) {
                    return ts.createNotEmittedStatement(node);
                }
                ts.Debug.assertNode(node.name, ts.isIdentifier, "A TypeScript namespace should have an Identifier name.");
                enableSubstitutionForNamespaceExports();
                var statements = [];
                // We request to be advised when the printer is about to print this node. This allows
                // us to set up the correct state for later substitutions.
                var emitFlags = 2 /* AdviseOnEmitNode */;
                // If needed, we should emit a variable declaration for the module. If we emit
                // a leading variable declaration, we should not emit leading comments for the
                // module body.
                var varAdded = addVarForEnumOrModuleDeclaration(statements, node);
                if (varAdded) {
                    // We should still emit the comments if we are emitting a system module.
                    if (moduleKind !== ts.ModuleKind.System || currentLexicalScope !== currentSourceFile) {
                        emitFlags |= 512 /* NoLeadingComments */;
                    }
                }
                // `parameterName` is the declaration name used inside of the namespace.
                var parameterName = getNamespaceParameterName(node);
                // `containerName` is the expression used inside of the namespace for exports.
                var containerName = getNamespaceContainerName(node);
                // `exportName` is the expression used within this node's container for any exported references.
                var exportName = ts.hasModifier(node, 1 /* Export */)
                    ? ts.getExternalModuleOrNamespaceExportName(currentNamespaceContainerName, node, /*allowComments*/ false, /*allowSourceMaps*/ true)
                    : ts.getLocalName(node, /*allowComments*/ false, /*allowSourceMaps*/ true);
                //  x || (x = {})
                //  exports.x || (exports.x = {})
                var moduleArg = ts.createLogicalOr(exportName, ts.createAssignment(exportName, ts.createObjectLiteral()));
                if (hasNamespaceQualifiedExportName(node)) {
                    // `localName` is the expression used within this node's containing scope for any local references.
                    var localName = ts.getLocalName(node, /*allowComments*/ false, /*allowSourceMaps*/ true);
                    //  x = (exports.x || (exports.x = {}))
                    moduleArg = ts.createAssignment(localName, moduleArg);
                }
                //  (function (x_1) {
                //      x_1.y = ...;
                //  })(x || (x = {}));
                var moduleStatement = ts.createExpressionStatement(ts.createCall(ts.createFunctionExpression(
                /*modifiers*/ undefined, 
                /*asteriskToken*/ undefined, 
                /*name*/ undefined, 
                /*typeParameters*/ undefined, [ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, parameterName)], 
                /*type*/ undefined, transformModuleBody(node, containerName)), 
                /*typeArguments*/ undefined, [moduleArg]));
                ts.setOriginalNode(moduleStatement, node);
                if (varAdded) {
                    // If a variable was added, synthetic comments are emitted on it, not on the moduleStatement.
                    ts.setSyntheticLeadingComments(moduleStatement, undefined);
                    ts.setSyntheticTrailingComments(moduleStatement, undefined);
                }
                ts.setTextRange(moduleStatement, node);
                ts.addEmitFlags(moduleStatement, emitFlags);
                statements.push(moduleStatement);
                // Add a DeclarationMarker for the namespace to preserve trailing comments and mark
                // the end of the declaration.
                statements.push(ts.createEndOfDeclarationMarker(node));
                return statements;
            }
            /**
             * Transforms the body of a module declaration.
             *
             * @param node The module declaration node.
             */
            function transformModuleBody(node, namespaceLocalName) {
                var savedCurrentNamespaceContainerName = currentNamespaceContainerName;
                var savedCurrentNamespace = currentNamespace;
                var savedCurrentScopeFirstDeclarationsOfName = currentScopeFirstDeclarationsOfName;
                currentNamespaceContainerName = namespaceLocalName;
                currentNamespace = node;
                currentScopeFirstDeclarationsOfName = undefined;
                var statements = [];
                startLexicalEnvironment();
                var statementsLocation;
                var blockLocation;
                var body = node.body;
                if (body.kind === 250 /* ModuleBlock */) {
                    saveStateAndInvoke(body, function (body) { return ts.addRange(statements, ts.visitNodes(body.statements, namespaceElementVisitor, ts.isStatement)); });
                    statementsLocation = body.statements;
                    blockLocation = body;
                }
                else {
                    var result = visitModuleDeclaration(body);
                    if (result) {
                        if (ts.isArray(result)) {
                            ts.addRange(statements, result);
                        }
                        else {
                            statements.push(result);
                        }
                    }
                    var moduleBlock = getInnerMostModuleDeclarationFromDottedModule(node).body;
                    statementsLocation = ts.moveRangePos(moduleBlock.statements, -1);
                }
                ts.insertStatementsAfterStandardPrologue(statements, endLexicalEnvironment());
                currentNamespaceContainerName = savedCurrentNamespaceContainerName;
                currentNamespace = savedCurrentNamespace;
                currentScopeFirstDeclarationsOfName = savedCurrentScopeFirstDeclarationsOfName;
                var block = ts.createBlock(ts.setTextRange(ts.createNodeArray(statements), 
                /*location*/ statementsLocation), 
                /*multiLine*/ true);
                ts.setTextRange(block, blockLocation);
                // namespace hello.hi.world {
                //      function foo() {}
                //
                //      // TODO, blah
                // }
                //
                // should be emitted as
                //
                // var hello;
                // (function (hello) {
                //     var hi;
                //     (function (hi) {
                //         var world;
                //         (function (world) {
                //             function foo() { }
                //             // TODO, blah
                //         })(world = hi.world || (hi.world = {}));
                //     })(hi = hello.hi || (hello.hi = {}));
                // })(hello || (hello = {}));
                // We only want to emit comment on the namespace which contains block body itself, not the containing namespaces.
                if (body.kind !== 250 /* ModuleBlock */) {
                    ts.setEmitFlags(block, ts.getEmitFlags(block) | 1536 /* NoComments */);
                }
                return block;
            }
            function getInnerMostModuleDeclarationFromDottedModule(moduleDeclaration) {
                if (moduleDeclaration.body.kind === 249 /* ModuleDeclaration */) {
                    var recursiveInnerModule = getInnerMostModuleDeclarationFromDottedModule(moduleDeclaration.body);
                    return recursiveInnerModule || moduleDeclaration.body;
                }
            }
            /**
             * Visits an import declaration, eliding it if it is not referenced and `importsNotUsedAsValues` is not 'preserve'.
             *
             * @param node The import declaration node.
             */
            function visitImportDeclaration(node) {
                if (!node.importClause) {
                    // Do not elide a side-effect only import declaration.
                    //  import "foo";
                    return node;
                }
                if (node.importClause.isTypeOnly) {
                    // Always elide type-only imports
                    return undefined;
                }
                // Elide the declaration if the import clause was elided.
                var importClause = ts.visitNode(node.importClause, visitImportClause, ts.isImportClause);
                return importClause ||
                    compilerOptions.importsNotUsedAsValues === 1 /* Preserve */ ||
                    compilerOptions.importsNotUsedAsValues === 2 /* Error */
                    ? ts.updateImportDeclaration(node, 
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, importClause, node.moduleSpecifier)
                    : undefined;
            }
            /**
             * Visits an import clause, eliding it if it is not referenced.
             *
             * @param node The import clause node.
             */
            function visitImportClause(node) {
                if (node.isTypeOnly) {
                    return undefined;
                }
                // Elide the import clause if we elide both its name and its named bindings.
                var name = resolver.isReferencedAliasDeclaration(node) ? node.name : undefined;
                var namedBindings = ts.visitNode(node.namedBindings, visitNamedImportBindings, ts.isNamedImportBindings);
                return (name || namedBindings) ? ts.updateImportClause(node, name, namedBindings, /*isTypeOnly*/ false) : undefined;
            }
            /**
             * Visits named import bindings, eliding it if it is not referenced.
             *
             * @param node The named import bindings node.
             */
            function visitNamedImportBindings(node) {
                if (node.kind === 256 /* NamespaceImport */) {
                    // Elide a namespace import if it is not referenced.
                    return resolver.isReferencedAliasDeclaration(node) ? node : undefined;
                }
                else {
                    // Elide named imports if all of its import specifiers are elided.
                    var elements = ts.visitNodes(node.elements, visitImportSpecifier, ts.isImportSpecifier);
                    return ts.some(elements) ? ts.updateNamedImports(node, elements) : undefined;
                }
            }
            /**
             * Visits an import specifier, eliding it if it is not referenced.
             *
             * @param node The import specifier node.
             */
            function visitImportSpecifier(node) {
                // Elide an import specifier if it is not referenced.
                return resolver.isReferencedAliasDeclaration(node) ? node : undefined;
            }
            /**
             * Visits an export assignment, eliding it if it does not contain a clause that resolves
             * to a value.
             *
             * @param node The export assignment node.
             */
            function visitExportAssignment(node) {
                // Elide the export assignment if it does not reference a value.
                return resolver.isValueAliasDeclaration(node)
                    ? ts.visitEachChild(node, visitor, context)
                    : undefined;
            }
            /**
             * Visits an export declaration, eliding it if it does not contain a clause that resolves
             * to a value.
             *
             * @param node The export declaration node.
             */
            function visitExportDeclaration(node) {
                if (node.isTypeOnly) {
                    return undefined;
                }
                if (!node.exportClause) {
                    // Elide a star export if the module it references does not export a value.
                    return compilerOptions.isolatedModules || resolver.moduleExportsSomeValue(node.moduleSpecifier) ? node : undefined;
                }
                if (!resolver.isValueAliasDeclaration(node)) {
                    // Elide the export declaration if it does not export a value.
                    return undefined;
                }
                // Elide the export declaration if all of its named exports are elided.
                var exportClause = ts.visitNode(node.exportClause, visitNamedExportBindings, ts.isNamedImportBindings);
                return exportClause
                    ? ts.updateExportDeclaration(node, 
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, exportClause, node.moduleSpecifier, node.isTypeOnly)
                    : undefined;
            }
            /**
             * Visits named exports, eliding it if it does not contain an export specifier that
             * resolves to a value.
             *
             * @param node The named exports node.
             */
            function visitNamedExports(node) {
                // Elide the named exports if all of its export specifiers were elided.
                var elements = ts.visitNodes(node.elements, visitExportSpecifier, ts.isExportSpecifier);
                return ts.some(elements) ? ts.updateNamedExports(node, elements) : undefined;
            }
            function visitNamespaceExports(node) {
                return ts.updateNamespaceExport(node, ts.visitNode(node.name, visitor, ts.isIdentifier));
            }
            function visitNamedExportBindings(node) {
                return ts.isNamespaceExport(node) ? visitNamespaceExports(node) : visitNamedExports(node);
            }
            /**
             * Visits an export specifier, eliding it if it does not resolve to a value.
             *
             * @param node The export specifier node.
             */
            function visitExportSpecifier(node) {
                // Elide an export specifier if it does not reference a value.
                return resolver.isValueAliasDeclaration(node) ? node : undefined;
            }
            /**
             * Determines whether to emit an import equals declaration.
             *
             * @param node The import equals declaration node.
             */
            function shouldEmitImportEqualsDeclaration(node) {
                // preserve old compiler's behavior: emit 'var' for import declaration (even if we do not consider them referenced) when
                // - current file is not external module
                // - import declaration is top level and target is value imported by entity name
                return resolver.isReferencedAliasDeclaration(node)
                    || (!ts.isExternalModule(currentSourceFile)
                        && resolver.isTopLevelValueImportEqualsWithEntityName(node));
            }
            /**
             * Visits an import equals declaration.
             *
             * @param node The import equals declaration node.
             */
            function visitImportEqualsDeclaration(node) {
                if (ts.isExternalModuleImportEqualsDeclaration(node)) {
                    var isReferenced = resolver.isReferencedAliasDeclaration(node);
                    // If the alias is unreferenced but we want to keep the import, replace with 'import "mod"'.
                    if (!isReferenced && compilerOptions.importsNotUsedAsValues === 1 /* Preserve */) {
                        return ts.setOriginalNode(ts.setTextRange(ts.createImportDeclaration(
                        /*decorators*/ undefined, 
                        /*modifiers*/ undefined, 
                        /*importClause*/ undefined, node.moduleReference.expression), node), node);
                    }
                    return isReferenced ? ts.visitEachChild(node, visitor, context) : undefined;
                }
                if (!shouldEmitImportEqualsDeclaration(node)) {
                    return undefined;
                }
                var moduleReference = ts.createExpressionFromEntityName(node.moduleReference);
                ts.setEmitFlags(moduleReference, 1536 /* NoComments */ | 2048 /* NoNestedComments */);
                if (isNamedExternalModuleExport(node) || !isExportOfNamespace(node)) {
                    //  export var ${name} = ${moduleReference};
                    //  var ${name} = ${moduleReference};
                    return ts.setOriginalNode(ts.setTextRange(ts.createVariableStatement(ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier), ts.createVariableDeclarationList([
                        ts.setOriginalNode(ts.createVariableDeclaration(node.name, 
                        /*type*/ undefined, moduleReference), node)
                    ])), node), node);
                }
                else {
                    // exports.${name} = ${moduleReference};
                    return ts.setOriginalNode(createNamespaceExport(node.name, moduleReference, node), node);
                }
            }
            /**
             * Gets a value indicating whether the node is exported from a namespace.
             *
             * @param node The node to test.
             */
            function isExportOfNamespace(node) {
                return currentNamespace !== undefined && ts.hasModifier(node, 1 /* Export */);
            }
            /**
             * Gets a value indicating whether the node is exported from an external module.
             *
             * @param node The node to test.
             */
            function isExternalModuleExport(node) {
                return currentNamespace === undefined && ts.hasModifier(node, 1 /* Export */);
            }
            /**
             * Gets a value indicating whether the node is a named export from an external module.
             *
             * @param node The node to test.
             */
            function isNamedExternalModuleExport(node) {
                return isExternalModuleExport(node)
                    && !ts.hasModifier(node, 512 /* Default */);
            }
            /**
             * Gets a value indicating whether the node is the default export of an external module.
             *
             * @param node The node to test.
             */
            function isDefaultExternalModuleExport(node) {
                return isExternalModuleExport(node)
                    && ts.hasModifier(node, 512 /* Default */);
            }
            /**
             * Creates a statement for the provided expression. This is used in calls to `map`.
             */
            function expressionToStatement(expression) {
                return ts.createExpressionStatement(expression);
            }
            function addExportMemberAssignment(statements, node) {
                var expression = ts.createAssignment(ts.getExternalModuleOrNamespaceExportName(currentNamespaceContainerName, node, /*allowComments*/ false, /*allowSourceMaps*/ true), ts.getLocalName(node));
                ts.setSourceMapRange(expression, ts.createRange(node.name ? node.name.pos : node.pos, node.end));
                var statement = ts.createExpressionStatement(expression);
                ts.setSourceMapRange(statement, ts.createRange(-1, node.end));
                statements.push(statement);
            }
            function createNamespaceExport(exportName, exportValue, location) {
                return ts.setTextRange(ts.createExpressionStatement(ts.createAssignment(ts.getNamespaceMemberName(currentNamespaceContainerName, exportName, /*allowComments*/ false, /*allowSourceMaps*/ true), exportValue)), location);
            }
            function createNamespaceExportExpression(exportName, exportValue, location) {
                return ts.setTextRange(ts.createAssignment(getNamespaceMemberNameWithSourceMapsAndWithoutComments(exportName), exportValue), location);
            }
            function getNamespaceMemberNameWithSourceMapsAndWithoutComments(name) {
                return ts.getNamespaceMemberName(currentNamespaceContainerName, name, /*allowComments*/ false, /*allowSourceMaps*/ true);
            }
            /**
             * Gets the declaration name used inside of a namespace or enum.
             */
            function getNamespaceParameterName(node) {
                var name = ts.getGeneratedNameForNode(node);
                ts.setSourceMapRange(name, node.name);
                return name;
            }
            /**
             * Gets the expression used to refer to a namespace or enum within the body
             * of its declaration.
             */
            function getNamespaceContainerName(node) {
                return ts.getGeneratedNameForNode(node);
            }
            /**
             * Gets a local alias for a class declaration if it is a decorated class with an internal
             * reference to the static side of the class. This is necessary to avoid issues with
             * double-binding semantics for the class name.
             */
            function getClassAliasIfNeeded(node) {
                if (resolver.getNodeCheckFlags(node) & 16777216 /* ClassWithConstructorReference */) {
                    enableSubstitutionForClassAliases();
                    var classAlias = ts.createUniqueName(node.name && !ts.isGeneratedIdentifier(node.name) ? ts.idText(node.name) : "default");
                    classAliases[ts.getOriginalNodeId(node)] = classAlias;
                    hoistVariableDeclaration(classAlias);
                    return classAlias;
                }
            }
            function getClassPrototype(node) {
                return ts.createPropertyAccess(ts.getDeclarationName(node), "prototype");
            }
            function getClassMemberPrefix(node, member) {
                return ts.hasModifier(member, 32 /* Static */)
                    ? ts.getDeclarationName(node)
                    : getClassPrototype(node);
            }
            function enableSubstitutionForNonQualifiedEnumMembers() {
                if ((enabledSubstitutions & 8 /* NonQualifiedEnumMembers */) === 0) {
                    enabledSubstitutions |= 8 /* NonQualifiedEnumMembers */;
                    context.enableSubstitution(75 /* Identifier */);
                }
            }
            function enableSubstitutionForClassAliases() {
                if ((enabledSubstitutions & 1 /* ClassAliases */) === 0) {
                    enabledSubstitutions |= 1 /* ClassAliases */;
                    // We need to enable substitutions for identifiers. This allows us to
                    // substitute class names inside of a class declaration.
                    context.enableSubstitution(75 /* Identifier */);
                    // Keep track of class aliases.
                    classAliases = [];
                }
            }
            function enableSubstitutionForNamespaceExports() {
                if ((enabledSubstitutions & 2 /* NamespaceExports */) === 0) {
                    enabledSubstitutions |= 2 /* NamespaceExports */;
                    // We need to enable substitutions for identifiers and shorthand property assignments. This allows us to
                    // substitute the names of exported members of a namespace.
                    context.enableSubstitution(75 /* Identifier */);
                    context.enableSubstitution(282 /* ShorthandPropertyAssignment */);
                    // We need to be notified when entering and exiting namespaces.
                    context.enableEmitNotification(249 /* ModuleDeclaration */);
                }
            }
            function isTransformedModuleDeclaration(node) {
                return ts.getOriginalNode(node).kind === 249 /* ModuleDeclaration */;
            }
            function isTransformedEnumDeclaration(node) {
                return ts.getOriginalNode(node).kind === 248 /* EnumDeclaration */;
            }
            /**
             * Hook for node emit.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to emit.
             * @param emit A callback used to emit the node in the printer.
             */
            function onEmitNode(hint, node, emitCallback) {
                var savedApplicableSubstitutions = applicableSubstitutions;
                var savedCurrentSourceFile = currentSourceFile;
                if (ts.isSourceFile(node)) {
                    currentSourceFile = node;
                }
                if (enabledSubstitutions & 2 /* NamespaceExports */ && isTransformedModuleDeclaration(node)) {
                    applicableSubstitutions |= 2 /* NamespaceExports */;
                }
                if (enabledSubstitutions & 8 /* NonQualifiedEnumMembers */ && isTransformedEnumDeclaration(node)) {
                    applicableSubstitutions |= 8 /* NonQualifiedEnumMembers */;
                }
                previousOnEmitNode(hint, node, emitCallback);
                applicableSubstitutions = savedApplicableSubstitutions;
                currentSourceFile = savedCurrentSourceFile;
            }
            /**
             * Hooks node substitutions.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to substitute.
             */
            function onSubstituteNode(hint, node) {
                node = previousOnSubstituteNode(hint, node);
                if (hint === 1 /* Expression */) {
                    return substituteExpression(node);
                }
                else if (ts.isShorthandPropertyAssignment(node)) {
                    return substituteShorthandPropertyAssignment(node);
                }
                return node;
            }
            function substituteShorthandPropertyAssignment(node) {
                if (enabledSubstitutions & 2 /* NamespaceExports */) {
                    var name = node.name;
                    var exportedName = trySubstituteNamespaceExportedName(name);
                    if (exportedName) {
                        // A shorthand property with an assignment initializer is probably part of a
                        // destructuring assignment
                        if (node.objectAssignmentInitializer) {
                            var initializer = ts.createAssignment(exportedName, node.objectAssignmentInitializer);
                            return ts.setTextRange(ts.createPropertyAssignment(name, initializer), node);
                        }
                        return ts.setTextRange(ts.createPropertyAssignment(name, exportedName), node);
                    }
                }
                return node;
            }
            function substituteExpression(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                        return substituteExpressionIdentifier(node);
                    case 194 /* PropertyAccessExpression */:
                        return substitutePropertyAccessExpression(node);
                    case 195 /* ElementAccessExpression */:
                        return substituteElementAccessExpression(node);
                }
                return node;
            }
            function substituteExpressionIdentifier(node) {
                return trySubstituteClassAlias(node)
                    || trySubstituteNamespaceExportedName(node)
                    || node;
            }
            function trySubstituteClassAlias(node) {
                if (enabledSubstitutions & 1 /* ClassAliases */) {
                    if (resolver.getNodeCheckFlags(node) & 33554432 /* ConstructorReferenceInClass */) {
                        // Due to the emit for class decorators, any reference to the class from inside of the class body
                        // must instead be rewritten to point to a temporary variable to avoid issues with the double-bind
                        // behavior of class names in ES6.
                        // Also, when emitting statics for class expressions, we must substitute a class alias for
                        // constructor references in static property initializers.
                        var declaration = resolver.getReferencedValueDeclaration(node);
                        if (declaration) {
                            var classAlias = classAliases[declaration.id]; // TODO: GH#18217
                            if (classAlias) {
                                var clone_1 = ts.getSynthesizedClone(classAlias);
                                ts.setSourceMapRange(clone_1, node);
                                ts.setCommentRange(clone_1, node);
                                return clone_1;
                            }
                        }
                    }
                }
                return undefined;
            }
            function trySubstituteNamespaceExportedName(node) {
                // If this is explicitly a local name, do not substitute.
                if (enabledSubstitutions & applicableSubstitutions && !ts.isGeneratedIdentifier(node) && !ts.isLocalName(node)) {
                    // If we are nested within a namespace declaration, we may need to qualifiy
                    // an identifier that is exported from a merged namespace.
                    var container = resolver.getReferencedExportContainer(node, /*prefixLocals*/ false);
                    if (container && container.kind !== 290 /* SourceFile */) {
                        var substitute = (applicableSubstitutions & 2 /* NamespaceExports */ && container.kind === 249 /* ModuleDeclaration */) ||
                            (applicableSubstitutions & 8 /* NonQualifiedEnumMembers */ && container.kind === 248 /* EnumDeclaration */);
                        if (substitute) {
                            return ts.setTextRange(ts.createPropertyAccess(ts.getGeneratedNameForNode(container), node), 
                            /*location*/ node);
                        }
                    }
                }
                return undefined;
            }
            function substitutePropertyAccessExpression(node) {
                return substituteConstantValue(node);
            }
            function substituteElementAccessExpression(node) {
                return substituteConstantValue(node);
            }
            function substituteConstantValue(node) {
                var constantValue = tryGetConstEnumValue(node);
                if (constantValue !== undefined) {
                    // track the constant value on the node for the printer in needsDotDotForPropertyAccess
                    ts.setConstantValue(node, constantValue);
                    var substitute = ts.createLiteral(constantValue);
                    if (!compilerOptions.removeComments) {
                        var originalNode = ts.getOriginalNode(node, ts.isAccessExpression);
                        var propertyName = ts.isPropertyAccessExpression(originalNode)
                            ? ts.declarationNameToString(originalNode.name)
                            : ts.getTextOfNode(originalNode.argumentExpression);
                        ts.addSyntheticTrailingComment(substitute, 3 /* MultiLineCommentTrivia */, " " + propertyName + " ");
                    }
                    return substitute;
                }
                return node;
            }
            function tryGetConstEnumValue(node) {
                if (compilerOptions.isolatedModules) {
                    return undefined;
                }
                return ts.isPropertyAccessExpression(node) || ts.isElementAccessExpression(node) ? resolver.getConstantValue(node) : undefined;
            }
        }
        ts.transformTypeScript = transformTypeScript;
        function createDecorateHelper(context, decoratorExpressions, target, memberName, descriptor, location) {
            var argumentsArray = [];
            argumentsArray.push(ts.createArrayLiteral(decoratorExpressions, /*multiLine*/ true));
            argumentsArray.push(target);
            if (memberName) {
                argumentsArray.push(memberName);
                if (descriptor) {
                    argumentsArray.push(descriptor);
                }
            }
            context.requestEmitHelper(ts.decorateHelper);
            return ts.setTextRange(ts.createCall(ts.getUnscopedHelperName("__decorate"), 
            /*typeArguments*/ undefined, argumentsArray), location);
        }
        ts.decorateHelper = {
            name: "typescript:decorate",
            importName: "__decorate",
            scoped: false,
            priority: 2,
            text: "\n            var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {\n                var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;\n                if (typeof Reflect === \"object\" && typeof Reflect.decorate === \"function\") r = Reflect.decorate(decorators, target, key, desc);\n                else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;\n                return c > 3 && r && Object.defineProperty(target, key, r), r;\n            };"
        };
        function createMetadataHelper(context, metadataKey, metadataValue) {
            context.requestEmitHelper(ts.metadataHelper);
            return ts.createCall(ts.getUnscopedHelperName("__metadata"), 
            /*typeArguments*/ undefined, [
                ts.createLiteral(metadataKey),
                metadataValue
            ]);
        }
        ts.metadataHelper = {
            name: "typescript:metadata",
            importName: "__metadata",
            scoped: false,
            priority: 3,
            text: "\n            var __metadata = (this && this.__metadata) || function (k, v) {\n                if (typeof Reflect === \"object\" && typeof Reflect.metadata === \"function\") return Reflect.metadata(k, v);\n            };"
        };
        function createParamHelper(context, expression, parameterOffset, location) {
            context.requestEmitHelper(ts.paramHelper);
            return ts.setTextRange(ts.createCall(ts.getUnscopedHelperName("__param"), 
            /*typeArguments*/ undefined, [
                ts.createLiteral(parameterOffset),
                expression
            ]), location);
        }
        ts.paramHelper = {
            name: "typescript:param",
            importName: "__param",
            scoped: false,
            priority: 4,
            text: "\n            var __param = (this && this.__param) || function (paramIndex, decorator) {\n                return function (target, key) { decorator(target, key, paramIndex); }\n            };"
        };
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        var ClassPropertySubstitutionFlags;
        (function (ClassPropertySubstitutionFlags) {
            /**
             * Enables substitutions for class expressions with static fields
             * which have initializers that reference the class name.
             */
            ClassPropertySubstitutionFlags[ClassPropertySubstitutionFlags["ClassAliases"] = 1] = "ClassAliases";
        })(ClassPropertySubstitutionFlags || (ClassPropertySubstitutionFlags = {}));
        var PrivateIdentifierPlacement;
        (function (PrivateIdentifierPlacement) {
            PrivateIdentifierPlacement[PrivateIdentifierPlacement["InstanceField"] = 0] = "InstanceField";
        })(PrivateIdentifierPlacement || (PrivateIdentifierPlacement = {}));
        /**
         * Transforms ECMAScript Class Syntax.
         * TypeScript parameter property syntax is transformed in the TypeScript transformer.
         * For now, this transforms public field declarations using TypeScript class semantics,
         * where declarations are elided and initializers are transformed as assignments in the constructor.
         * When --useDefineForClassFields is on, this transforms to ECMAScript semantics, with Object.defineProperty.
         */
        function transformClassFields(context) {
            var hoistVariableDeclaration = context.hoistVariableDeclaration, endLexicalEnvironment = context.endLexicalEnvironment, resumeLexicalEnvironment = context.resumeLexicalEnvironment;
            var resolver = context.getEmitResolver();
            var compilerOptions = context.getCompilerOptions();
            var languageVersion = ts.getEmitScriptTarget(compilerOptions);
            var shouldTransformPrivateFields = languageVersion < 99 /* ESNext */;
            var previousOnSubstituteNode = context.onSubstituteNode;
            context.onSubstituteNode = onSubstituteNode;
            var enabledSubstitutions;
            var classAliases;
            /**
             * Tracks what computed name expressions originating from elided names must be inlined
             * at the next execution site, in document order
             */
            var pendingExpressions;
            /**
             * Tracks what computed name expression statements and static property initializers must be
             * emitted at the next execution site, in document order (for decorated classes).
             */
            var pendingStatements;
            var privateIdentifierEnvironmentStack = [];
            var currentPrivateIdentifierEnvironment;
            return ts.chainBundle(transformSourceFile);
            function transformSourceFile(node) {
                var options = context.getCompilerOptions();
                if (node.isDeclarationFile
                    || options.useDefineForClassFields && options.target === 99 /* ESNext */) {
                    return node;
                }
                var visited = ts.visitEachChild(node, visitor, context);
                ts.addEmitHelpers(visited, context.readEmitHelpers());
                return visited;
            }
            function visitor(node) {
                if (!(node.transformFlags & 4194304 /* ContainsClassFields */))
                    return node;
                switch (node.kind) {
                    case 214 /* ClassExpression */:
                    case 245 /* ClassDeclaration */:
                        return visitClassLike(node);
                    case 159 /* PropertyDeclaration */:
                        return visitPropertyDeclaration(node);
                    case 225 /* VariableStatement */:
                        return visitVariableStatement(node);
                    case 154 /* ComputedPropertyName */:
                        return visitComputedPropertyName(node);
                    case 194 /* PropertyAccessExpression */:
                        return visitPropertyAccessExpression(node);
                    case 207 /* PrefixUnaryExpression */:
                        return visitPrefixUnaryExpression(node);
                    case 208 /* PostfixUnaryExpression */:
                        return visitPostfixUnaryExpression(node, /*valueIsDiscarded*/ false);
                    case 196 /* CallExpression */:
                        return visitCallExpression(node);
                    case 209 /* BinaryExpression */:
                        return visitBinaryExpression(node);
                    case 76 /* PrivateIdentifier */:
                        return visitPrivateIdentifier(node);
                    case 226 /* ExpressionStatement */:
                        return visitExpressionStatement(node);
                    case 230 /* ForStatement */:
                        return visitForStatement(node);
                    case 198 /* TaggedTemplateExpression */:
                        return visitTaggedTemplateExpression(node);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitorDestructuringTarget(node) {
                switch (node.kind) {
                    case 193 /* ObjectLiteralExpression */:
                    case 192 /* ArrayLiteralExpression */:
                        return visitAssignmentPattern(node);
                    default:
                        return visitor(node);
                }
            }
            /**
             * If we visit a private name, this means it is an undeclared private name.
             * Replace it with an empty identifier to indicate a problem with the code.
             */
            function visitPrivateIdentifier(node) {
                if (!shouldTransformPrivateFields) {
                    return node;
                }
                return ts.setOriginalNode(ts.createIdentifier(""), node);
            }
            /**
             * Visits the members of a class that has fields.
             *
             * @param node The node to visit.
             */
            function classElementVisitor(node) {
                switch (node.kind) {
                    case 162 /* Constructor */:
                        // Constructors for classes using class fields are transformed in
                        // `visitClassDeclaration` or `visitClassExpression`.
                        return undefined;
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                    case 161 /* MethodDeclaration */:
                        // Visit the name of the member (if it's a computed property name).
                        return ts.visitEachChild(node, classElementVisitor, context);
                    case 159 /* PropertyDeclaration */:
                        return visitPropertyDeclaration(node);
                    case 154 /* ComputedPropertyName */:
                        return visitComputedPropertyName(node);
                    case 222 /* SemicolonClassElement */:
                        return node;
                    default:
                        return visitor(node);
                }
            }
            function visitVariableStatement(node) {
                var savedPendingStatements = pendingStatements;
                pendingStatements = [];
                var visitedNode = ts.visitEachChild(node, visitor, context);
                var statement = ts.some(pendingStatements) ? __spreadArrays([visitedNode], pendingStatements) :
                    visitedNode;
                pendingStatements = savedPendingStatements;
                return statement;
            }
            function visitComputedPropertyName(name) {
                var node = ts.visitEachChild(name, visitor, context);
                if (ts.some(pendingExpressions)) {
                    var expressions = pendingExpressions;
                    expressions.push(name.expression);
                    pendingExpressions = [];
                    node = ts.updateComputedPropertyName(node, ts.inlineExpressions(expressions));
                }
                return node;
            }
            function visitPropertyDeclaration(node) {
                ts.Debug.assert(!ts.some(node.decorators));
                if (!shouldTransformPrivateFields && ts.isPrivateIdentifier(node.name)) {
                    // Initializer is elided as the field is initialized in transformConstructor.
                    return ts.updateProperty(node, 
                    /*decorators*/ undefined, ts.visitNodes(node.modifiers, visitor, ts.isModifier), node.name, 
                    /*questionOrExclamationToken*/ undefined, 
                    /*type*/ undefined, 
                    /*initializer*/ undefined);
                }
                // Create a temporary variable to store a computed property name (if necessary).
                // If it's not inlineable, then we emit an expression after the class which assigns
                // the property name to the temporary variable.
                var expr = getPropertyNameExpressionIfNeeded(node.name, !!node.initializer || !!context.getCompilerOptions().useDefineForClassFields);
                if (expr && !ts.isSimpleInlineableExpression(expr)) {
                    (pendingExpressions || (pendingExpressions = [])).push(expr);
                }
                return undefined;
            }
            function createPrivateIdentifierAccess(info, receiver) {
                receiver = ts.visitNode(receiver, visitor, ts.isExpression);
                switch (info.placement) {
                    case 0 /* InstanceField */:
                        return createClassPrivateFieldGetHelper(context, ts.nodeIsSynthesized(receiver) ? receiver : ts.getSynthesizedClone(receiver), info.weakMapName);
                    default: return ts.Debug.fail("Unexpected private identifier placement");
                }
            }
            function visitPropertyAccessExpression(node) {
                if (shouldTransformPrivateFields && ts.isPrivateIdentifier(node.name)) {
                    var privateIdentifierInfo = accessPrivateIdentifier(node.name);
                    if (privateIdentifierInfo) {
                        return ts.setOriginalNode(createPrivateIdentifierAccess(privateIdentifierInfo, node.expression), node);
                    }
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitPrefixUnaryExpression(node) {
                if (shouldTransformPrivateFields && ts.isPrivateIdentifierPropertyAccessExpression(node.operand)) {
                    var operator = node.operator === 45 /* PlusPlusToken */ ?
                        39 /* PlusToken */ : node.operator === 46 /* MinusMinusToken */ ?
                        40 /* MinusToken */ : undefined;
                    var info = void 0;
                    if (operator && (info = accessPrivateIdentifier(node.operand.name))) {
                        var receiver = ts.visitNode(node.operand.expression, visitor, ts.isExpression);
                        var _a = createCopiableReceiverExpr(receiver), readExpression = _a.readExpression, initializeExpression = _a.initializeExpression;
                        var existingValue = ts.createPrefix(39 /* PlusToken */, createPrivateIdentifierAccess(info, readExpression));
                        return ts.setOriginalNode(createPrivateIdentifierAssignment(info, initializeExpression || readExpression, ts.createBinary(existingValue, operator, ts.createLiteral(1)), 62 /* EqualsToken */), node);
                    }
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitPostfixUnaryExpression(node, valueIsDiscarded) {
                if (shouldTransformPrivateFields && ts.isPrivateIdentifierPropertyAccessExpression(node.operand)) {
                    var operator = node.operator === 45 /* PlusPlusToken */ ?
                        39 /* PlusToken */ : node.operator === 46 /* MinusMinusToken */ ?
                        40 /* MinusToken */ : undefined;
                    var info = void 0;
                    if (operator && (info = accessPrivateIdentifier(node.operand.name))) {
                        var receiver = ts.visitNode(node.operand.expression, visitor, ts.isExpression);
                        var _a = createCopiableReceiverExpr(receiver), readExpression = _a.readExpression, initializeExpression = _a.initializeExpression;
                        var existingValue = ts.createPrefix(39 /* PlusToken */, createPrivateIdentifierAccess(info, readExpression));
                        // Create a temporary variable to store the value returned by the expression.
                        var returnValue = valueIsDiscarded ? undefined : ts.createTempVariable(hoistVariableDeclaration);
                        return ts.setOriginalNode(ts.inlineExpressions(ts.compact([
                            createPrivateIdentifierAssignment(info, initializeExpression || readExpression, ts.createBinary(returnValue ? ts.createAssignment(returnValue, existingValue) : existingValue, operator, ts.createLiteral(1)), 62 /* EqualsToken */),
                            returnValue
                        ])), node);
                    }
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitForStatement(node) {
                if (node.incrementor && ts.isPostfixUnaryExpression(node.incrementor)) {
                    return ts.updateFor(node, ts.visitNode(node.initializer, visitor, ts.isForInitializer), ts.visitNode(node.condition, visitor, ts.isExpression), visitPostfixUnaryExpression(node.incrementor, /*valueIsDiscarded*/ true), ts.visitNode(node.statement, visitor, ts.isStatement));
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitExpressionStatement(node) {
                if (ts.isPostfixUnaryExpression(node.expression)) {
                    return ts.updateExpressionStatement(node, visitPostfixUnaryExpression(node.expression, /*valueIsDiscarded*/ true));
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function createCopiableReceiverExpr(receiver) {
                var clone = ts.nodeIsSynthesized(receiver) ? receiver : ts.getSynthesizedClone(receiver);
                if (ts.isSimpleInlineableExpression(receiver)) {
                    return { readExpression: clone, initializeExpression: undefined };
                }
                var readExpression = ts.createTempVariable(hoistVariableDeclaration);
                var initializeExpression = ts.createAssignment(readExpression, clone);
                return { readExpression: readExpression, initializeExpression: initializeExpression };
            }
            function visitCallExpression(node) {
                if (shouldTransformPrivateFields && ts.isPrivateIdentifierPropertyAccessExpression(node.expression)) {
                    // Transform call expressions of private names to properly bind the `this` parameter.
                    var _a = ts.createCallBinding(node.expression, hoistVariableDeclaration, languageVersion), thisArg = _a.thisArg, target = _a.target;
                    return ts.updateCall(node, ts.createPropertyAccess(ts.visitNode(target, visitor), "call"), 
                    /*typeArguments*/ undefined, __spreadArrays([ts.visitNode(thisArg, visitor, ts.isExpression)], ts.visitNodes(node.arguments, visitor, ts.isExpression)));
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitTaggedTemplateExpression(node) {
                if (shouldTransformPrivateFields && ts.isPrivateIdentifierPropertyAccessExpression(node.tag)) {
                    // Bind the `this` correctly for tagged template literals when the tag is a private identifier property access.
                    var _a = ts.createCallBinding(node.tag, hoistVariableDeclaration, languageVersion), thisArg = _a.thisArg, target = _a.target;
                    return ts.updateTaggedTemplate(node, ts.createCall(ts.createPropertyAccess(ts.visitNode(target, visitor), "bind"), 
                    /*typeArguments*/ undefined, [ts.visitNode(thisArg, visitor, ts.isExpression)]), ts.visitNode(node.template, visitor, ts.isTemplateLiteral));
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitBinaryExpression(node) {
                if (shouldTransformPrivateFields) {
                    if (ts.isDestructuringAssignment(node)) {
                        var savedPendingExpressions = pendingExpressions;
                        pendingExpressions = undefined;
                        node = ts.updateBinary(node, ts.visitNode(node.left, visitorDestructuringTarget), ts.visitNode(node.right, visitor), node.operatorToken);
                        var expr = ts.some(pendingExpressions) ?
                            ts.inlineExpressions(ts.compact(__spreadArrays(pendingExpressions, [node]))) :
                            node;
                        pendingExpressions = savedPendingExpressions;
                        return expr;
                    }
                    if (ts.isAssignmentExpression(node) && ts.isPrivateIdentifierPropertyAccessExpression(node.left)) {
                        var info = accessPrivateIdentifier(node.left.name);
                        if (info) {
                            return ts.setOriginalNode(createPrivateIdentifierAssignment(info, node.left.expression, node.right, node.operatorToken.kind), node);
                        }
                    }
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function createPrivateIdentifierAssignment(info, receiver, right, operator) {
                switch (info.placement) {
                    case 0 /* InstanceField */: {
                        return createPrivateIdentifierInstanceFieldAssignment(info, receiver, right, operator);
                    }
                    default: return ts.Debug.fail("Unexpected private identifier placement");
                }
            }
            function createPrivateIdentifierInstanceFieldAssignment(info, receiver, right, operator) {
                receiver = ts.visitNode(receiver, visitor, ts.isExpression);
                right = ts.visitNode(right, visitor, ts.isExpression);
                if (ts.isCompoundAssignment(operator)) {
                    var _a = createCopiableReceiverExpr(receiver), readExpression = _a.readExpression, initializeExpression = _a.initializeExpression;
                    return createClassPrivateFieldSetHelper(context, initializeExpression || readExpression, info.weakMapName, ts.createBinary(createClassPrivateFieldGetHelper(context, readExpression, info.weakMapName), ts.getNonAssignmentOperatorForCompoundAssignment(operator), right));
                }
                else {
                    return createClassPrivateFieldSetHelper(context, receiver, info.weakMapName, right);
                }
            }
            /**
             * Set up the environment for a class.
             */
            function visitClassLike(node) {
                var savedPendingExpressions = pendingExpressions;
                pendingExpressions = undefined;
                if (shouldTransformPrivateFields) {
                    startPrivateIdentifierEnvironment();
                }
                var result = ts.isClassDeclaration(node) ?
                    visitClassDeclaration(node) :
                    visitClassExpression(node);
                if (shouldTransformPrivateFields) {
                    endPrivateIdentifierEnvironment();
                }
                pendingExpressions = savedPendingExpressions;
                return result;
            }
            function doesClassElementNeedTransform(node) {
                return ts.isPropertyDeclaration(node) || (shouldTransformPrivateFields && node.name && ts.isPrivateIdentifier(node.name));
            }
            function visitClassDeclaration(node) {
                if (!ts.forEach(node.members, doesClassElementNeedTransform)) {
                    return ts.visitEachChild(node, visitor, context);
                }
                var extendsClauseElement = ts.getEffectiveBaseTypeNode(node);
                var isDerivedClass = !!(extendsClauseElement && ts.skipOuterExpressions(extendsClauseElement.expression).kind !== 100 /* NullKeyword */);
                var statements = [
                    ts.updateClassDeclaration(node, 
                    /*decorators*/ undefined, node.modifiers, node.name, 
                    /*typeParameters*/ undefined, ts.visitNodes(node.heritageClauses, visitor, ts.isHeritageClause), transformClassMembers(node, isDerivedClass))
                ];
                // Write any pending expressions from elided or moved computed property names
                if (ts.some(pendingExpressions)) {
                    statements.push(ts.createExpressionStatement(ts.inlineExpressions(pendingExpressions)));
                }
                // Emit static property assignment. Because classDeclaration is lexically evaluated,
                // it is safe to emit static property assignment after classDeclaration
                // From ES6 specification:
                //      HasLexicalDeclaration (N) : Determines if the argument identifier has a binding in this environment record that was created using
                //                                  a lexical declaration such as a LexicalDeclaration or a ClassDeclaration.
                var staticProperties = ts.getProperties(node, /*requireInitializer*/ true, /*isStatic*/ true);
                if (ts.some(staticProperties)) {
                    addPropertyStatements(statements, staticProperties, ts.getInternalName(node));
                }
                return statements;
            }
            function visitClassExpression(node) {
                if (!ts.forEach(node.members, doesClassElementNeedTransform)) {
                    return ts.visitEachChild(node, visitor, context);
                }
                // If this class expression is a transformation of a decorated class declaration,
                // then we want to output the pendingExpressions as statements, not as inlined
                // expressions with the class statement.
                //
                // In this case, we use pendingStatements to produce the same output as the
                // class declaration transformation. The VariableStatement visitor will insert
                // these statements after the class expression variable statement.
                var isDecoratedClassDeclaration = ts.isClassDeclaration(ts.getOriginalNode(node));
                var staticProperties = ts.getProperties(node, /*requireInitializer*/ true, /*isStatic*/ true);
                var extendsClauseElement = ts.getEffectiveBaseTypeNode(node);
                var isDerivedClass = !!(extendsClauseElement && ts.skipOuterExpressions(extendsClauseElement.expression).kind !== 100 /* NullKeyword */);
                var classExpression = ts.updateClassExpression(node, node.modifiers, node.name, 
                /*typeParameters*/ undefined, ts.visitNodes(node.heritageClauses, visitor, ts.isHeritageClause), transformClassMembers(node, isDerivedClass));
                if (ts.some(staticProperties) || ts.some(pendingExpressions)) {
                    if (isDecoratedClassDeclaration) {
                        ts.Debug.assertDefined(pendingStatements, "Decorated classes transformed by TypeScript are expected to be within a variable declaration.");
                        // Write any pending expressions from elided or moved computed property names
                        if (pendingStatements && pendingExpressions && ts.some(pendingExpressions)) {
                            pendingStatements.push(ts.createExpressionStatement(ts.inlineExpressions(pendingExpressions)));
                        }
                        if (pendingStatements && ts.some(staticProperties)) {
                            addPropertyStatements(pendingStatements, staticProperties, ts.getInternalName(node));
                        }
                        return classExpression;
                    }
                    else {
                        var expressions = [];
                        var isClassWithConstructorReference = resolver.getNodeCheckFlags(node) & 16777216 /* ClassWithConstructorReference */;
                        var temp = ts.createTempVariable(hoistVariableDeclaration, !!isClassWithConstructorReference);
                        if (isClassWithConstructorReference) {
                            // record an alias as the class name is not in scope for statics.
                            enableSubstitutionForClassAliases();
                            var alias = ts.getSynthesizedClone(temp);
                            alias.autoGenerateFlags &= ~8 /* ReservedInNestedScopes */;
                            classAliases[ts.getOriginalNodeId(node)] = alias;
                        }
                        // To preserve the behavior of the old emitter, we explicitly indent
                        // the body of a class with static initializers.
                        ts.setEmitFlags(classExpression, 65536 /* Indented */ | ts.getEmitFlags(classExpression));
                        expressions.push(ts.startOnNewLine(ts.createAssignment(temp, classExpression)));
                        // Add any pending expressions leftover from elided or relocated computed property names
                        ts.addRange(expressions, ts.map(pendingExpressions, ts.startOnNewLine));
                        ts.addRange(expressions, generateInitializedPropertyExpressions(staticProperties, temp));
                        expressions.push(ts.startOnNewLine(temp));
                        return ts.inlineExpressions(expressions);
                    }
                }
                return classExpression;
            }
            function transformClassMembers(node, isDerivedClass) {
                if (shouldTransformPrivateFields) {
                    // Declare private names.
                    for (var _i = 0, _a = node.members; _i < _a.length; _i++) {
                        var member = _a[_i];
                        if (ts.isPrivateIdentifierPropertyDeclaration(member)) {
                            addPrivateIdentifierToEnvironment(member.name);
                        }
                    }
                }
                var members = [];
                var constructor = transformConstructor(node, isDerivedClass);
                if (constructor) {
                    members.push(constructor);
                }
                ts.addRange(members, ts.visitNodes(node.members, classElementVisitor, ts.isClassElement));
                return ts.setTextRange(ts.createNodeArray(members), /*location*/ node.members);
            }
            function isPropertyDeclarationThatRequiresConstructorStatement(member) {
                if (!ts.isPropertyDeclaration(member) || ts.hasStaticModifier(member)) {
                    return false;
                }
                if (context.getCompilerOptions().useDefineForClassFields) {
                    // If we are using define semantics and targeting ESNext or higher,
                    // then we don't need to transform any class properties.
                    return languageVersion < 99 /* ESNext */;
                }
                return ts.isInitializedProperty(member) || shouldTransformPrivateFields && ts.isPrivateIdentifierPropertyDeclaration(member);
            }
            function transformConstructor(node, isDerivedClass) {
                var constructor = ts.visitNode(ts.getFirstConstructorWithBody(node), visitor, ts.isConstructorDeclaration);
                var properties = node.members.filter(isPropertyDeclarationThatRequiresConstructorStatement);
                if (!ts.some(properties)) {
                    return constructor;
                }
                var parameters = ts.visitParameterList(constructor ? constructor.parameters : undefined, visitor, context);
                var body = transformConstructorBody(node, constructor, isDerivedClass);
                if (!body) {
                    return undefined;
                }
                return ts.startOnNewLine(ts.setOriginalNode(ts.setTextRange(ts.createConstructor(
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, parameters, body), constructor || node), constructor));
            }
            function transformConstructorBody(node, constructor, isDerivedClass) {
                var useDefineForClassFields = context.getCompilerOptions().useDefineForClassFields;
                var properties = ts.getProperties(node, /*requireInitializer*/ false, /*isStatic*/ false);
                if (!useDefineForClassFields) {
                    properties = ts.filter(properties, function (property) { return !!property.initializer || ts.isPrivateIdentifier(property.name); });
                }
                // Only generate synthetic constructor when there are property initializers to move.
                if (!constructor && !ts.some(properties)) {
                    return ts.visitFunctionBody(/*node*/ undefined, visitor, context);
                }
                resumeLexicalEnvironment();
                var indexOfFirstStatement = 0;
                var statements = [];
                if (!constructor && isDerivedClass) {
                    // Add a synthetic `super` call:
                    //
                    //  super(...arguments);
                    //
                    statements.push(ts.createExpressionStatement(ts.createCall(ts.createSuper(), 
                    /*typeArguments*/ undefined, [ts.createSpread(ts.createIdentifier("arguments"))])));
                }
                if (constructor) {
                    indexOfFirstStatement = ts.addPrologueDirectivesAndInitialSuperCall(constructor, statements, visitor);
                }
                // Add the property initializers. Transforms this:
                //
                //  public x = 1;
                //
                // Into this:
                //
                //  constructor() {
                //      this.x = 1;
                //  }
                //
                if (constructor === null || constructor === void 0 ? void 0 : constructor.body) {
                    var afterParameterProperties = ts.findIndex(constructor.body.statements, function (s) { return !ts.isParameterPropertyDeclaration(ts.getOriginalNode(s), constructor); }, indexOfFirstStatement);
                    if (afterParameterProperties === -1) {
                        afterParameterProperties = constructor.body.statements.length;
                    }
                    if (afterParameterProperties > indexOfFirstStatement) {
                        if (!useDefineForClassFields) {
                            ts.addRange(statements, ts.visitNodes(constructor.body.statements, visitor, ts.isStatement, indexOfFirstStatement, afterParameterProperties - indexOfFirstStatement));
                        }
                        indexOfFirstStatement = afterParameterProperties;
                    }
                }
                addPropertyStatements(statements, properties, ts.createThis());
                // Add existing statements, skipping the initial super call.
                if (constructor) {
                    ts.addRange(statements, ts.visitNodes(constructor.body.statements, visitor, ts.isStatement, indexOfFirstStatement));
                }
                statements = ts.mergeLexicalEnvironment(statements, endLexicalEnvironment());
                return ts.setTextRange(ts.createBlock(ts.setTextRange(ts.createNodeArray(statements), 
                /*location*/ constructor ? constructor.body.statements : node.members), 
                /*multiLine*/ true), 
                /*location*/ constructor ? constructor.body : undefined);
            }
            /**
             * Generates assignment statements for property initializers.
             *
             * @param properties An array of property declarations to transform.
             * @param receiver The receiver on which each property should be assigned.
             */
            function addPropertyStatements(statements, properties, receiver) {
                for (var _i = 0, properties_6 = properties; _i < properties_6.length; _i++) {
                    var property = properties_6[_i];
                    var expression = transformProperty(property, receiver);
                    if (!expression) {
                        continue;
                    }
                    var statement = ts.createExpressionStatement(expression);
                    ts.setSourceMapRange(statement, ts.moveRangePastModifiers(property));
                    ts.setCommentRange(statement, property);
                    ts.setOriginalNode(statement, property);
                    statements.push(statement);
                }
            }
            /**
             * Generates assignment expressions for property initializers.
             *
             * @param properties An array of property declarations to transform.
             * @param receiver The receiver on which each property should be assigned.
             */
            function generateInitializedPropertyExpressions(properties, receiver) {
                var expressions = [];
                for (var _i = 0, properties_7 = properties; _i < properties_7.length; _i++) {
                    var property = properties_7[_i];
                    var expression = transformProperty(property, receiver);
                    if (!expression) {
                        continue;
                    }
                    ts.startOnNewLine(expression);
                    ts.setSourceMapRange(expression, ts.moveRangePastModifiers(property));
                    ts.setCommentRange(expression, property);
                    ts.setOriginalNode(expression, property);
                    expressions.push(expression);
                }
                return expressions;
            }
            /**
             * Transforms a property initializer into an assignment statement.
             *
             * @param property The property declaration.
             * @param receiver The object receiving the property assignment.
             */
            function transformProperty(property, receiver) {
                // We generate a name here in order to reuse the value cached by the relocated computed name expression (which uses the same generated name)
                var emitAssignment = !context.getCompilerOptions().useDefineForClassFields;
                var propertyName = ts.isComputedPropertyName(property.name) && !ts.isSimpleInlineableExpression(property.name.expression)
                    ? ts.updateComputedPropertyName(property.name, ts.getGeneratedNameForNode(property.name))
                    : property.name;
                if (shouldTransformPrivateFields && ts.isPrivateIdentifier(propertyName)) {
                    var privateIdentifierInfo = accessPrivateIdentifier(propertyName);
                    if (privateIdentifierInfo) {
                        switch (privateIdentifierInfo.placement) {
                            case 0 /* InstanceField */: {
                                return createPrivateInstanceFieldInitializer(receiver, ts.visitNode(property.initializer, visitor, ts.isExpression), privateIdentifierInfo.weakMapName);
                            }
                        }
                    }
                    else {
                        ts.Debug.fail("Undeclared private name for property declaration.");
                    }
                }
                if (ts.isPrivateIdentifier(propertyName) && !property.initializer) {
                    return undefined;
                }
                if (ts.isPrivateIdentifier(propertyName) && !property.initializer) {
                    return undefined;
                }
                var propertyOriginalNode = ts.getOriginalNode(property);
                var initializer = property.initializer || emitAssignment ? ts.visitNode(property.initializer, visitor, ts.isExpression)
                    : ts.isParameterPropertyDeclaration(propertyOriginalNode, propertyOriginalNode.parent) && ts.isIdentifier(propertyName) ? propertyName
                        : ts.createVoidZero();
                if (emitAssignment || ts.isPrivateIdentifier(propertyName)) {
                    var memberAccess = ts.createMemberAccessForPropertyName(receiver, propertyName, /*location*/ propertyName);
                    return ts.createAssignment(memberAccess, initializer);
                }
                else {
                    var name = ts.isComputedPropertyName(propertyName) ? propertyName.expression
                        : ts.isIdentifier(propertyName) ? ts.createStringLiteral(ts.unescapeLeadingUnderscores(propertyName.escapedText))
                            : propertyName;
                    var descriptor = ts.createPropertyDescriptor({ value: initializer, configurable: true, writable: true, enumerable: true });
                    return ts.createObjectDefinePropertyCall(receiver, name, descriptor);
                }
            }
            function enableSubstitutionForClassAliases() {
                if ((enabledSubstitutions & 1 /* ClassAliases */) === 0) {
                    enabledSubstitutions |= 1 /* ClassAliases */;
                    // We need to enable substitutions for identifiers. This allows us to
                    // substitute class names inside of a class declaration.
                    context.enableSubstitution(75 /* Identifier */);
                    // Keep track of class aliases.
                    classAliases = [];
                }
            }
            /**
             * Hooks node substitutions.
             *
             * @param hint The context for the emitter.
             * @param node The node to substitute.
             */
            function onSubstituteNode(hint, node) {
                node = previousOnSubstituteNode(hint, node);
                if (hint === 1 /* Expression */) {
                    return substituteExpression(node);
                }
                return node;
            }
            function substituteExpression(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                        return substituteExpressionIdentifier(node);
                }
                return node;
            }
            function substituteExpressionIdentifier(node) {
                return trySubstituteClassAlias(node) || node;
            }
            function trySubstituteClassAlias(node) {
                if (enabledSubstitutions & 1 /* ClassAliases */) {
                    if (resolver.getNodeCheckFlags(node) & 33554432 /* ConstructorReferenceInClass */) {
                        // Due to the emit for class decorators, any reference to the class from inside of the class body
                        // must instead be rewritten to point to a temporary variable to avoid issues with the double-bind
                        // behavior of class names in ES6.
                        // Also, when emitting statics for class expressions, we must substitute a class alias for
                        // constructor references in static property initializers.
                        var declaration = resolver.getReferencedValueDeclaration(node);
                        if (declaration) {
                            var classAlias = classAliases[declaration.id]; // TODO: GH#18217
                            if (classAlias) {
                                var clone_2 = ts.getSynthesizedClone(classAlias);
                                ts.setSourceMapRange(clone_2, node);
                                ts.setCommentRange(clone_2, node);
                                return clone_2;
                            }
                        }
                    }
                }
                return undefined;
            }
            /**
             * If the name is a computed property, this function transforms it, then either returns an expression which caches the
             * value of the result or the expression itself if the value is either unused or safe to inline into multiple locations
             * @param shouldHoist Does the expression need to be reused? (ie, for an initializer or a decorator)
             */
            function getPropertyNameExpressionIfNeeded(name, shouldHoist) {
                if (ts.isComputedPropertyName(name)) {
                    var expression = ts.visitNode(name.expression, visitor, ts.isExpression);
                    var innerExpression = ts.skipPartiallyEmittedExpressions(expression);
                    var inlinable = ts.isSimpleInlineableExpression(innerExpression);
                    var alreadyTransformed = ts.isAssignmentExpression(innerExpression) && ts.isGeneratedIdentifier(innerExpression.left);
                    if (!alreadyTransformed && !inlinable && shouldHoist) {
                        var generatedName = ts.getGeneratedNameForNode(name);
                        hoistVariableDeclaration(generatedName);
                        return ts.createAssignment(generatedName, expression);
                    }
                    return (inlinable || ts.isIdentifier(innerExpression)) ? undefined : expression;
                }
            }
            function startPrivateIdentifierEnvironment() {
                privateIdentifierEnvironmentStack.push(currentPrivateIdentifierEnvironment);
                currentPrivateIdentifierEnvironment = undefined;
            }
            function endPrivateIdentifierEnvironment() {
                currentPrivateIdentifierEnvironment = privateIdentifierEnvironmentStack.pop();
            }
            function addPrivateIdentifierToEnvironment(name) {
                var text = ts.getTextOfPropertyName(name);
                var weakMapName = ts.createOptimisticUniqueName("_" + text.substring(1));
                weakMapName.autoGenerateFlags |= 8 /* ReservedInNestedScopes */;
                hoistVariableDeclaration(weakMapName);
                (currentPrivateIdentifierEnvironment || (currentPrivateIdentifierEnvironment = ts.createUnderscoreEscapedMap()))
                    .set(name.escapedText, { placement: 0 /* InstanceField */, weakMapName: weakMapName });
                (pendingExpressions || (pendingExpressions = [])).push(ts.createAssignment(weakMapName, ts.createNew(ts.createIdentifier("WeakMap"), 
                /*typeArguments*/ undefined, [])));
            }
            function accessPrivateIdentifier(name) {
                if (currentPrivateIdentifierEnvironment) {
                    var info = currentPrivateIdentifierEnvironment.get(name.escapedText);
                    if (info) {
                        return info;
                    }
                }
                for (var i = privateIdentifierEnvironmentStack.length - 1; i >= 0; --i) {
                    var env = privateIdentifierEnvironmentStack[i];
                    if (!env) {
                        continue;
                    }
                    var info = env.get(name.escapedText);
                    if (info) {
                        return info;
                    }
                }
                return undefined;
            }
            function wrapPrivateIdentifierForDestructuringTarget(node) {
                var parameter = ts.getGeneratedNameForNode(node);
                var info = accessPrivateIdentifier(node.name);
                if (!info) {
                    return ts.visitEachChild(node, visitor, context);
                }
                var receiver = node.expression;
                // We cannot copy `this` or `super` into the function because they will be bound
                // differently inside the function.
                if (ts.isThisProperty(node) || ts.isSuperProperty(node) || !ts.isSimpleCopiableExpression(node.expression)) {
                    receiver = ts.createTempVariable(hoistVariableDeclaration);
                    receiver.autoGenerateFlags |= 8 /* ReservedInNestedScopes */;
                    (pendingExpressions || (pendingExpressions = [])).push(ts.createBinary(receiver, 62 /* EqualsToken */, node.expression));
                }
                return ts.createPropertyAccess(
                // Explicit parens required because of v8 regression (https://bugs.chromium.org/p/v8/issues/detail?id=9560)
                ts.createParen(ts.createObjectLiteral([
                    ts.createSetAccessor(
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, "value", [ts.createParameter(
                        /*decorators*/ undefined, 
                        /*modifiers*/ undefined, 
                        /*dotDotDotToken*/ undefined, parameter, 
                        /*questionToken*/ undefined, 
                        /*type*/ undefined, 
                        /*initializer*/ undefined)], ts.createBlock([ts.createExpressionStatement(createPrivateIdentifierAssignment(info, receiver, parameter, 62 /* EqualsToken */))]))
                ])), "value");
            }
            function visitArrayAssignmentTarget(node) {
                var target = ts.getTargetOfBindingOrAssignmentElement(node);
                if (target && ts.isPrivateIdentifierPropertyAccessExpression(target)) {
                    var wrapped = wrapPrivateIdentifierForDestructuringTarget(target);
                    if (ts.isAssignmentExpression(node)) {
                        return ts.updateBinary(node, wrapped, ts.visitNode(node.right, visitor, ts.isExpression), node.operatorToken);
                    }
                    else if (ts.isSpreadElement(node)) {
                        return ts.updateSpread(node, wrapped);
                    }
                    else {
                        return wrapped;
                    }
                }
                return ts.visitNode(node, visitorDestructuringTarget);
            }
            function visitObjectAssignmentTarget(node) {
                if (ts.isPropertyAssignment(node)) {
                    var target = ts.getTargetOfBindingOrAssignmentElement(node);
                    if (target && ts.isPrivateIdentifierPropertyAccessExpression(target)) {
                        var initializer = ts.getInitializerOfBindingOrAssignmentElement(node);
                        var wrapped = wrapPrivateIdentifierForDestructuringTarget(target);
                        return ts.updatePropertyAssignment(node, ts.visitNode(node.name, visitor), initializer ? ts.createAssignment(wrapped, ts.visitNode(initializer, visitor)) : wrapped);
                    }
                    return ts.updatePropertyAssignment(node, ts.visitNode(node.name, visitor), ts.visitNode(node.initializer, visitorDestructuringTarget));
                }
                return ts.visitNode(node, visitor);
            }
            function visitAssignmentPattern(node) {
                if (ts.isArrayLiteralExpression(node)) {
                    // Transforms private names in destructuring assignment array bindings.
                    //
                    // Source:
                    // ([ this.#myProp ] = [ "hello" ]);
                    //
                    // Transformation:
                    // [ { set value(x) { this.#myProp = x; } }.value ] = [ "hello" ];
                    return ts.updateArrayLiteral(node, ts.visitNodes(node.elements, visitArrayAssignmentTarget, ts.isExpression));
                }
                else {
                    // Transforms private names in destructuring assignment object bindings.
                    //
                    // Source:
                    // ({ stringProperty: this.#myProp } = { stringProperty: "hello" });
                    //
                    // Transformation:
                    // ({ stringProperty: { set value(x) { this.#myProp = x; } }.value }) = { stringProperty: "hello" };
                    return ts.updateObjectLiteral(node, ts.visitNodes(node.properties, visitObjectAssignmentTarget, ts.isObjectLiteralElementLike));
                }
            }
        }
        ts.transformClassFields = transformClassFields;
        function createPrivateInstanceFieldInitializer(receiver, initializer, weakMapName) {
            return ts.createCall(ts.createPropertyAccess(weakMapName, "set"), 
            /*typeArguments*/ undefined, [receiver, initializer || ts.createVoidZero()]);
        }
        ts.classPrivateFieldGetHelper = {
            name: "typescript:classPrivateFieldGet",
            scoped: false,
            text: "\n            var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {\n                if (!privateMap.has(receiver)) {\n                    throw new TypeError(\"attempted to get private field on non-instance\");\n                }\n                return privateMap.get(receiver);\n            };"
        };
        function createClassPrivateFieldGetHelper(context, receiver, privateField) {
            context.requestEmitHelper(ts.classPrivateFieldGetHelper);
            return ts.createCall(ts.getUnscopedHelperName("__classPrivateFieldGet"), /* typeArguments */ undefined, [receiver, privateField]);
        }
        ts.classPrivateFieldSetHelper = {
            name: "typescript:classPrivateFieldSet",
            scoped: false,
            text: "\n            var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {\n                if (!privateMap.has(receiver)) {\n                    throw new TypeError(\"attempted to set private field on non-instance\");\n                }\n                privateMap.set(receiver, value);\n                return value;\n            };"
        };
        function createClassPrivateFieldSetHelper(context, receiver, privateField, value) {
            context.requestEmitHelper(ts.classPrivateFieldSetHelper);
            return ts.createCall(ts.getUnscopedHelperName("__classPrivateFieldSet"), /* typeArguments */ undefined, [receiver, privateField, value]);
        }
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        var ES2017SubstitutionFlags;
        (function (ES2017SubstitutionFlags) {
            /** Enables substitutions for async methods with `super` calls. */
            ES2017SubstitutionFlags[ES2017SubstitutionFlags["AsyncMethodsWithSuper"] = 1] = "AsyncMethodsWithSuper";
        })(ES2017SubstitutionFlags || (ES2017SubstitutionFlags = {}));
        var ContextFlags;
        (function (ContextFlags) {
            ContextFlags[ContextFlags["NonTopLevel"] = 1] = "NonTopLevel";
            ContextFlags[ContextFlags["HasLexicalThis"] = 2] = "HasLexicalThis";
        })(ContextFlags || (ContextFlags = {}));
        function transformES2017(context) {
            var resumeLexicalEnvironment = context.resumeLexicalEnvironment, endLexicalEnvironment = context.endLexicalEnvironment, hoistVariableDeclaration = context.hoistVariableDeclaration;
            var resolver = context.getEmitResolver();
            var compilerOptions = context.getCompilerOptions();
            var languageVersion = ts.getEmitScriptTarget(compilerOptions);
            /**
             * Keeps track of whether expression substitution has been enabled for specific edge cases.
             * They are persisted between each SourceFile transformation and should not be reset.
             */
            var enabledSubstitutions;
            /**
             * This keeps track of containers where `super` is valid, for use with
             * just-in-time substitution for `super` expressions inside of async methods.
             */
            var enclosingSuperContainerFlags = 0;
            var enclosingFunctionParameterNames;
            /**
             * Keeps track of property names accessed on super (`super.x`) within async functions.
             */
            var capturedSuperProperties;
            /** Whether the async function contains an element access on super (`super[x]`). */
            var hasSuperElementAccess;
            /** A set of node IDs for generated super accessors (variable statements). */
            var substitutedSuperAccessors = [];
            var contextFlags = 0;
            // Save the previous transformation hooks.
            var previousOnEmitNode = context.onEmitNode;
            var previousOnSubstituteNode = context.onSubstituteNode;
            // Set new transformation hooks.
            context.onEmitNode = onEmitNode;
            context.onSubstituteNode = onSubstituteNode;
            return ts.chainBundle(transformSourceFile);
            function transformSourceFile(node) {
                if (node.isDeclarationFile) {
                    return node;
                }
                setContextFlag(1 /* NonTopLevel */, false);
                setContextFlag(2 /* HasLexicalThis */, !ts.isEffectiveStrictModeSourceFile(node, compilerOptions));
                var visited = ts.visitEachChild(node, visitor, context);
                ts.addEmitHelpers(visited, context.readEmitHelpers());
                return visited;
            }
            function setContextFlag(flag, val) {
                contextFlags = val ? contextFlags | flag : contextFlags & ~flag;
            }
            function inContext(flags) {
                return (contextFlags & flags) !== 0;
            }
            function inTopLevelContext() {
                return !inContext(1 /* NonTopLevel */);
            }
            function inHasLexicalThisContext() {
                return inContext(2 /* HasLexicalThis */);
            }
            function doWithContext(flags, cb, value) {
                var contextFlagsToSet = flags & ~contextFlags;
                if (contextFlagsToSet) {
                    setContextFlag(contextFlagsToSet, /*val*/ true);
                    var result = cb(value);
                    setContextFlag(contextFlagsToSet, /*val*/ false);
                    return result;
                }
                return cb(value);
            }
            function visitDefault(node) {
                return ts.visitEachChild(node, visitor, context);
            }
            function visitor(node) {
                if ((node.transformFlags & 64 /* ContainsES2017 */) === 0) {
                    return node;
                }
                switch (node.kind) {
                    case 126 /* AsyncKeyword */:
                        // ES2017 async modifier should be elided for targets < ES2017
                        return undefined;
                    case 206 /* AwaitExpression */:
                        return visitAwaitExpression(node);
                    case 161 /* MethodDeclaration */:
                        return doWithContext(1 /* NonTopLevel */ | 2 /* HasLexicalThis */, visitMethodDeclaration, node);
                    case 244 /* FunctionDeclaration */:
                        return doWithContext(1 /* NonTopLevel */ | 2 /* HasLexicalThis */, visitFunctionDeclaration, node);
                    case 201 /* FunctionExpression */:
                        return doWithContext(1 /* NonTopLevel */ | 2 /* HasLexicalThis */, visitFunctionExpression, node);
                    case 202 /* ArrowFunction */:
                        return doWithContext(1 /* NonTopLevel */, visitArrowFunction, node);
                    case 194 /* PropertyAccessExpression */:
                        if (capturedSuperProperties && ts.isPropertyAccessExpression(node) && node.expression.kind === 102 /* SuperKeyword */) {
                            capturedSuperProperties.set(node.name.escapedText, true);
                        }
                        return ts.visitEachChild(node, visitor, context);
                    case 195 /* ElementAccessExpression */:
                        if (capturedSuperProperties && node.expression.kind === 102 /* SuperKeyword */) {
                            hasSuperElementAccess = true;
                        }
                        return ts.visitEachChild(node, visitor, context);
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                    case 162 /* Constructor */:
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                        return doWithContext(1 /* NonTopLevel */ | 2 /* HasLexicalThis */, visitDefault, node);
                    default:
                        return ts.visitEachChild(node, visitor, context);
                }
            }
            function asyncBodyVisitor(node) {
                if (ts.isNodeWithPossibleHoistedDeclaration(node)) {
                    switch (node.kind) {
                        case 225 /* VariableStatement */:
                            return visitVariableStatementInAsyncBody(node);
                        case 230 /* ForStatement */:
                            return visitForStatementInAsyncBody(node);
                        case 231 /* ForInStatement */:
                            return visitForInStatementInAsyncBody(node);
                        case 232 /* ForOfStatement */:
                            return visitForOfStatementInAsyncBody(node);
                        case 280 /* CatchClause */:
                            return visitCatchClauseInAsyncBody(node);
                        case 223 /* Block */:
                        case 237 /* SwitchStatement */:
                        case 251 /* CaseBlock */:
                        case 277 /* CaseClause */:
                        case 278 /* DefaultClause */:
                        case 240 /* TryStatement */:
                        case 228 /* DoStatement */:
                        case 229 /* WhileStatement */:
                        case 227 /* IfStatement */:
                        case 236 /* WithStatement */:
                        case 238 /* LabeledStatement */:
                            return ts.visitEachChild(node, asyncBodyVisitor, context);
                        default:
                            return ts.Debug.assertNever(node, "Unhandled node.");
                    }
                }
                return visitor(node);
            }
            function visitCatchClauseInAsyncBody(node) {
                var catchClauseNames = ts.createUnderscoreEscapedMap();
                recordDeclarationName(node.variableDeclaration, catchClauseNames); // TODO: GH#18217
                // names declared in a catch variable are block scoped
                var catchClauseUnshadowedNames;
                catchClauseNames.forEach(function (_, escapedName) {
                    if (enclosingFunctionParameterNames.has(escapedName)) {
                        if (!catchClauseUnshadowedNames) {
                            catchClauseUnshadowedNames = ts.cloneMap(enclosingFunctionParameterNames);
                        }
                        catchClauseUnshadowedNames.delete(escapedName);
                    }
                });
                if (catchClauseUnshadowedNames) {
                    var savedEnclosingFunctionParameterNames = enclosingFunctionParameterNames;
                    enclosingFunctionParameterNames = catchClauseUnshadowedNames;
                    var result = ts.visitEachChild(node, asyncBodyVisitor, context);
                    enclosingFunctionParameterNames = savedEnclosingFunctionParameterNames;
                    return result;
                }
                else {
                    return ts.visitEachChild(node, asyncBodyVisitor, context);
                }
            }
            function visitVariableStatementInAsyncBody(node) {
                if (isVariableDeclarationListWithCollidingName(node.declarationList)) {
                    var expression = visitVariableDeclarationListWithCollidingNames(node.declarationList, /*hasReceiver*/ false);
                    return expression ? ts.createExpressionStatement(expression) : undefined;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitForInStatementInAsyncBody(node) {
                return ts.updateForIn(node, isVariableDeclarationListWithCollidingName(node.initializer)
                    ? visitVariableDeclarationListWithCollidingNames(node.initializer, /*hasReceiver*/ true)
                    : ts.visitNode(node.initializer, visitor, ts.isForInitializer), ts.visitNode(node.expression, visitor, ts.isExpression), ts.visitNode(node.statement, asyncBodyVisitor, ts.isStatement, ts.liftToBlock));
            }
            function visitForOfStatementInAsyncBody(node) {
                return ts.updateForOf(node, ts.visitNode(node.awaitModifier, visitor, ts.isToken), isVariableDeclarationListWithCollidingName(node.initializer)
                    ? visitVariableDeclarationListWithCollidingNames(node.initializer, /*hasReceiver*/ true)
                    : ts.visitNode(node.initializer, visitor, ts.isForInitializer), ts.visitNode(node.expression, visitor, ts.isExpression), ts.visitNode(node.statement, asyncBodyVisitor, ts.isStatement, ts.liftToBlock));
            }
            function visitForStatementInAsyncBody(node) {
                var initializer = node.initializer; // TODO: GH#18217
                return ts.updateFor(node, isVariableDeclarationListWithCollidingName(initializer)
                    ? visitVariableDeclarationListWithCollidingNames(initializer, /*hasReceiver*/ false)
                    : ts.visitNode(node.initializer, visitor, ts.isForInitializer), ts.visitNode(node.condition, visitor, ts.isExpression), ts.visitNode(node.incrementor, visitor, ts.isExpression), ts.visitNode(node.statement, asyncBodyVisitor, ts.isStatement, ts.liftToBlock));
            }
            /**
             * Visits an AwaitExpression node.
             *
             * This function will be called any time a ES2017 await expression is encountered.
             *
             * @param node The node to visit.
             */
            function visitAwaitExpression(node) {
                // do not downlevel a top-level await as it is module syntax...
                if (inTopLevelContext()) {
                    return ts.visitEachChild(node, visitor, context);
                }
                return ts.setOriginalNode(ts.setTextRange(ts.createYield(
                /*asteriskToken*/ undefined, ts.visitNode(node.expression, visitor, ts.isExpression)), node), node);
            }
            /**
             * Visits a MethodDeclaration node.
             *
             * This function will be called when one of the following conditions are met:
             * - The node is marked as async
             *
             * @param node The node to visit.
             */
            function visitMethodDeclaration(node) {
                return ts.updateMethod(node, 
                /*decorators*/ undefined, ts.visitNodes(node.modifiers, visitor, ts.isModifier), node.asteriskToken, node.name, 
                /*questionToken*/ undefined, 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, ts.getFunctionFlags(node) & 2 /* Async */
                    ? transformAsyncFunctionBody(node)
                    : ts.visitFunctionBody(node.body, visitor, context));
            }
            /**
             * Visits a FunctionDeclaration node.
             *
             * This function will be called when one of the following conditions are met:
             * - The node is marked async
             *
             * @param node The node to visit.
             */
            function visitFunctionDeclaration(node) {
                return ts.updateFunctionDeclaration(node, 
                /*decorators*/ undefined, ts.visitNodes(node.modifiers, visitor, ts.isModifier), node.asteriskToken, node.name, 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, ts.getFunctionFlags(node) & 2 /* Async */
                    ? transformAsyncFunctionBody(node)
                    : ts.visitFunctionBody(node.body, visitor, context));
            }
            /**
             * Visits a FunctionExpression node.
             *
             * This function will be called when one of the following conditions are met:
             * - The node is marked async
             *
             * @param node The node to visit.
             */
            function visitFunctionExpression(node) {
                return ts.updateFunctionExpression(node, ts.visitNodes(node.modifiers, visitor, ts.isModifier), node.asteriskToken, node.name, 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, ts.getFunctionFlags(node) & 2 /* Async */
                    ? transformAsyncFunctionBody(node)
                    : ts.visitFunctionBody(node.body, visitor, context));
            }
            /**
             * Visits an ArrowFunction.
             *
             * This function will be called when one of the following conditions are met:
             * - The node is marked async
             *
             * @param node The node to visit.
             */
            function visitArrowFunction(node) {
                return ts.updateArrowFunction(node, ts.visitNodes(node.modifiers, visitor, ts.isModifier), 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, node.equalsGreaterThanToken, ts.getFunctionFlags(node) & 2 /* Async */
                    ? transformAsyncFunctionBody(node)
                    : ts.visitFunctionBody(node.body, visitor, context));
            }
            function recordDeclarationName(_a, names) {
                var name = _a.name;
                if (ts.isIdentifier(name)) {
                    names.set(name.escapedText, true);
                }
                else {
                    for (var _i = 0, _b = name.elements; _i < _b.length; _i++) {
                        var element = _b[_i];
                        if (!ts.isOmittedExpression(element)) {
                            recordDeclarationName(element, names);
                        }
                    }
                }
            }
            function isVariableDeclarationListWithCollidingName(node) {
                return !!node
                    && ts.isVariableDeclarationList(node)
                    && !(node.flags & 3 /* BlockScoped */)
                    && node.declarations.some(collidesWithParameterName);
            }
            function visitVariableDeclarationListWithCollidingNames(node, hasReceiver) {
                hoistVariableDeclarationList(node);
                var variables = ts.getInitializedVariables(node);
                if (variables.length === 0) {
                    if (hasReceiver) {
                        return ts.visitNode(ts.convertToAssignmentElementTarget(node.declarations[0].name), visitor, ts.isExpression);
                    }
                    return undefined;
                }
                return ts.inlineExpressions(ts.map(variables, transformInitializedVariable));
            }
            function hoistVariableDeclarationList(node) {
                ts.forEach(node.declarations, hoistVariable);
            }
            function hoistVariable(_a) {
                var name = _a.name;
                if (ts.isIdentifier(name)) {
                    hoistVariableDeclaration(name);
                }
                else {
                    for (var _i = 0, _b = name.elements; _i < _b.length; _i++) {
                        var element = _b[_i];
                        if (!ts.isOmittedExpression(element)) {
                            hoistVariable(element);
                        }
                    }
                }
            }
            function transformInitializedVariable(node) {
                var converted = ts.setSourceMapRange(ts.createAssignment(ts.convertToAssignmentElementTarget(node.name), node.initializer), node);
                return ts.visitNode(converted, visitor, ts.isExpression);
            }
            function collidesWithParameterName(_a) {
                var name = _a.name;
                if (ts.isIdentifier(name)) {
                    return enclosingFunctionParameterNames.has(name.escapedText);
                }
                else {
                    for (var _i = 0, _b = name.elements; _i < _b.length; _i++) {
                        var element = _b[_i];
                        if (!ts.isOmittedExpression(element) && collidesWithParameterName(element)) {
                            return true;
                        }
                    }
                }
                return false;
            }
            function transformAsyncFunctionBody(node) {
                resumeLexicalEnvironment();
                var original = ts.getOriginalNode(node, ts.isFunctionLike);
                var nodeType = original.type;
                var promiseConstructor = languageVersion < 2 /* ES2015 */ ? getPromiseConstructor(nodeType) : undefined;
                var isArrowFunction = node.kind === 202 /* ArrowFunction */;
                var hasLexicalArguments = (resolver.getNodeCheckFlags(node) & 8192 /* CaptureArguments */) !== 0;
                // An async function is emit as an outer function that calls an inner
                // generator function. To preserve lexical bindings, we pass the current
                // `this` and `arguments` objects to `__awaiter`. The generator function
                // passed to `__awaiter` is executed inside of the callback to the
                // promise constructor.
                var savedEnclosingFunctionParameterNames = enclosingFunctionParameterNames;
                enclosingFunctionParameterNames = ts.createUnderscoreEscapedMap();
                for (var _i = 0, _a = node.parameters; _i < _a.length; _i++) {
                    var parameter = _a[_i];
                    recordDeclarationName(parameter, enclosingFunctionParameterNames);
                }
                var savedCapturedSuperProperties = capturedSuperProperties;
                var savedHasSuperElementAccess = hasSuperElementAccess;
                if (!isArrowFunction) {
                    capturedSuperProperties = ts.createUnderscoreEscapedMap();
                    hasSuperElementAccess = false;
                }
                var result;
                if (!isArrowFunction) {
                    var statements = [];
                    var statementOffset = ts.addPrologue(statements, node.body.statements, /*ensureUseStrict*/ false, visitor);
                    statements.push(ts.createReturn(createAwaiterHelper(context, inHasLexicalThisContext(), hasLexicalArguments, promiseConstructor, transformAsyncFunctionBodyWorker(node.body, statementOffset))));
                    ts.insertStatementsAfterStandardPrologue(statements, endLexicalEnvironment());
                    // Minor optimization, emit `_super` helper to capture `super` access in an arrow.
                    // This step isn't needed if we eventually transform this to ES5.
                    var emitSuperHelpers = languageVersion >= 2 /* ES2015 */ && resolver.getNodeCheckFlags(node) & (4096 /* AsyncMethodWithSuperBinding */ | 2048 /* AsyncMethodWithSuper */);
                    if (emitSuperHelpers) {
                        enableSubstitutionForAsyncMethodsWithSuper();
                        if (ts.hasEntries(capturedSuperProperties)) {
                            var variableStatement = createSuperAccessVariableStatement(resolver, node, capturedSuperProperties);
                            substitutedSuperAccessors[ts.getNodeId(variableStatement)] = true;
                            ts.insertStatementsAfterStandardPrologue(statements, [variableStatement]);
                        }
                    }
                    var block = ts.createBlock(statements, /*multiLine*/ true);
                    ts.setTextRange(block, node.body);
                    if (emitSuperHelpers && hasSuperElementAccess) {
                        // Emit helpers for super element access expressions (`super[x]`).
                        if (resolver.getNodeCheckFlags(node) & 4096 /* AsyncMethodWithSuperBinding */) {
                            ts.addEmitHelper(block, ts.advancedAsyncSuperHelper);
                        }
                        else if (resolver.getNodeCheckFlags(node) & 2048 /* AsyncMethodWithSuper */) {
                            ts.addEmitHelper(block, ts.asyncSuperHelper);
                        }
                    }
                    result = block;
                }
                else {
                    var expression = createAwaiterHelper(context, inHasLexicalThisContext(), hasLexicalArguments, promiseConstructor, transformAsyncFunctionBodyWorker(node.body));
                    var declarations = endLexicalEnvironment();
                    if (ts.some(declarations)) {
                        var block = ts.convertToFunctionBody(expression);
                        result = ts.updateBlock(block, ts.setTextRange(ts.createNodeArray(ts.concatenate(declarations, block.statements)), block.statements));
                    }
                    else {
                        result = expression;
                    }
                }
                enclosingFunctionParameterNames = savedEnclosingFunctionParameterNames;
                if (!isArrowFunction) {
                    capturedSuperProperties = savedCapturedSuperProperties;
                    hasSuperElementAccess = savedHasSuperElementAccess;
                }
                return result;
            }
            function transformAsyncFunctionBodyWorker(body, start) {
                if (ts.isBlock(body)) {
                    return ts.updateBlock(body, ts.visitNodes(body.statements, asyncBodyVisitor, ts.isStatement, start));
                }
                else {
                    return ts.convertToFunctionBody(ts.visitNode(body, asyncBodyVisitor, ts.isConciseBody));
                }
            }
            function getPromiseConstructor(type) {
                var typeName = type && ts.getEntityNameFromTypeNode(type);
                if (typeName && ts.isEntityName(typeName)) {
                    var serializationKind = resolver.getTypeReferenceSerializationKind(typeName);
                    if (serializationKind === ts.TypeReferenceSerializationKind.TypeWithConstructSignatureAndValue
                        || serializationKind === ts.TypeReferenceSerializationKind.Unknown) {
                        return typeName;
                    }
                }
                return undefined;
            }
            function enableSubstitutionForAsyncMethodsWithSuper() {
                if ((enabledSubstitutions & 1 /* AsyncMethodsWithSuper */) === 0) {
                    enabledSubstitutions |= 1 /* AsyncMethodsWithSuper */;
                    // We need to enable substitutions for call, property access, and element access
                    // if we need to rewrite super calls.
                    context.enableSubstitution(196 /* CallExpression */);
                    context.enableSubstitution(194 /* PropertyAccessExpression */);
                    context.enableSubstitution(195 /* ElementAccessExpression */);
                    // We need to be notified when entering and exiting declarations that bind super.
                    context.enableEmitNotification(245 /* ClassDeclaration */);
                    context.enableEmitNotification(161 /* MethodDeclaration */);
                    context.enableEmitNotification(163 /* GetAccessor */);
                    context.enableEmitNotification(164 /* SetAccessor */);
                    context.enableEmitNotification(162 /* Constructor */);
                    // We need to be notified when entering the generated accessor arrow functions.
                    context.enableEmitNotification(225 /* VariableStatement */);
                }
            }
            /**
             * Hook for node emit.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to emit.
             * @param emit A callback used to emit the node in the printer.
             */
            function onEmitNode(hint, node, emitCallback) {
                // If we need to support substitutions for `super` in an async method,
                // we should track it here.
                if (enabledSubstitutions & 1 /* AsyncMethodsWithSuper */ && isSuperContainer(node)) {
                    var superContainerFlags = resolver.getNodeCheckFlags(node) & (2048 /* AsyncMethodWithSuper */ | 4096 /* AsyncMethodWithSuperBinding */);
                    if (superContainerFlags !== enclosingSuperContainerFlags) {
                        var savedEnclosingSuperContainerFlags = enclosingSuperContainerFlags;
                        enclosingSuperContainerFlags = superContainerFlags;
                        previousOnEmitNode(hint, node, emitCallback);
                        enclosingSuperContainerFlags = savedEnclosingSuperContainerFlags;
                        return;
                    }
                }
                // Disable substitution in the generated super accessor itself.
                else if (enabledSubstitutions && substitutedSuperAccessors[ts.getNodeId(node)]) {
                    var savedEnclosingSuperContainerFlags = enclosingSuperContainerFlags;
                    enclosingSuperContainerFlags = 0;
                    previousOnEmitNode(hint, node, emitCallback);
                    enclosingSuperContainerFlags = savedEnclosingSuperContainerFlags;
                    return;
                }
                previousOnEmitNode(hint, node, emitCallback);
            }
            /**
             * Hooks node substitutions.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to substitute.
             */
            function onSubstituteNode(hint, node) {
                node = previousOnSubstituteNode(hint, node);
                if (hint === 1 /* Expression */ && enclosingSuperContainerFlags) {
                    return substituteExpression(node);
                }
                return node;
            }
            function substituteExpression(node) {
                switch (node.kind) {
                    case 194 /* PropertyAccessExpression */:
                        return substitutePropertyAccessExpression(node);
                    case 195 /* ElementAccessExpression */:
                        return substituteElementAccessExpression(node);
                    case 196 /* CallExpression */:
                        return substituteCallExpression(node);
                }
                return node;
            }
            function substitutePropertyAccessExpression(node) {
                if (node.expression.kind === 102 /* SuperKeyword */) {
                    return ts.setTextRange(ts.createPropertyAccess(ts.createFileLevelUniqueName("_super"), node.name), node);
                }
                return node;
            }
            function substituteElementAccessExpression(node) {
                if (node.expression.kind === 102 /* SuperKeyword */) {
                    return createSuperElementAccessInAsyncMethod(node.argumentExpression, node);
                }
                return node;
            }
            function substituteCallExpression(node) {
                var expression = node.expression;
                if (ts.isSuperProperty(expression)) {
                    var argumentExpression = ts.isPropertyAccessExpression(expression)
                        ? substitutePropertyAccessExpression(expression)
                        : substituteElementAccessExpression(expression);
                    return ts.createCall(ts.createPropertyAccess(argumentExpression, "call"), 
                    /*typeArguments*/ undefined, __spreadArrays([
                        ts.createThis()
                    ], node.arguments));
                }
                return node;
            }
            function isSuperContainer(node) {
                var kind = node.kind;
                return kind === 245 /* ClassDeclaration */
                    || kind === 162 /* Constructor */
                    || kind === 161 /* MethodDeclaration */
                    || kind === 163 /* GetAccessor */
                    || kind === 164 /* SetAccessor */;
            }
            function createSuperElementAccessInAsyncMethod(argumentExpression, location) {
                if (enclosingSuperContainerFlags & 4096 /* AsyncMethodWithSuperBinding */) {
                    return ts.setTextRange(ts.createPropertyAccess(ts.createCall(ts.createFileLevelUniqueName("_superIndex"), 
                    /*typeArguments*/ undefined, [argumentExpression]), "value"), location);
                }
                else {
                    return ts.setTextRange(ts.createCall(ts.createFileLevelUniqueName("_superIndex"), 
                    /*typeArguments*/ undefined, [argumentExpression]), location);
                }
            }
        }
        ts.transformES2017 = transformES2017;
        /** Creates a variable named `_super` with accessor properties for the given property names. */
        function createSuperAccessVariableStatement(resolver, node, names) {
            // Create a variable declaration with a getter/setter (if binding) definition for each name:
            //   const _super = Object.create(null, { x: { get: () => super.x, set: (v) => super.x = v }, ... });
            var hasBinding = (resolver.getNodeCheckFlags(node) & 4096 /* AsyncMethodWithSuperBinding */) !== 0;
            var accessors = [];
            names.forEach(function (_, key) {
                var name = ts.unescapeLeadingUnderscores(key);
                var getterAndSetter = [];
                getterAndSetter.push(ts.createPropertyAssignment("get", ts.createArrowFunction(
                /* modifiers */ undefined, 
                /* typeParameters */ undefined, 
                /* parameters */ [], 
                /* type */ undefined, 
                /* equalsGreaterThanToken */ undefined, ts.setEmitFlags(ts.createPropertyAccess(ts.setEmitFlags(ts.createSuper(), 4 /* NoSubstitution */), name), 4 /* NoSubstitution */))));
                if (hasBinding) {
                    getterAndSetter.push(ts.createPropertyAssignment("set", ts.createArrowFunction(
                    /* modifiers */ undefined, 
                    /* typeParameters */ undefined, 
                    /* parameters */ [
                        ts.createParameter(
                        /* decorators */ undefined, 
                        /* modifiers */ undefined, 
                        /* dotDotDotToken */ undefined, "v", 
                        /* questionToken */ undefined, 
                        /* type */ undefined, 
                        /* initializer */ undefined)
                    ], 
                    /* type */ undefined, 
                    /* equalsGreaterThanToken */ undefined, ts.createAssignment(ts.setEmitFlags(ts.createPropertyAccess(ts.setEmitFlags(ts.createSuper(), 4 /* NoSubstitution */), name), 4 /* NoSubstitution */), ts.createIdentifier("v")))));
                }
                accessors.push(ts.createPropertyAssignment(name, ts.createObjectLiteral(getterAndSetter)));
            });
            return ts.createVariableStatement(
            /* modifiers */ undefined, ts.createVariableDeclarationList([
                ts.createVariableDeclaration(ts.createFileLevelUniqueName("_super"), 
                /* type */ undefined, ts.createCall(ts.createPropertyAccess(ts.createIdentifier("Object"), "create"), 
                /* typeArguments */ undefined, [
                    ts.createNull(),
                    ts.createObjectLiteral(accessors, /* multiline */ true)
                ]))
            ], 2 /* Const */));
        }
        ts.createSuperAccessVariableStatement = createSuperAccessVariableStatement;
        ts.awaiterHelper = {
            name: "typescript:awaiter",
            importName: "__awaiter",
            scoped: false,
            priority: 5,
            text: "\n            var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {\n                function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }\n                return new (P || (P = Promise))(function (resolve, reject) {\n                    function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }\n                    function rejected(value) { try { step(generator[\"throw\"](value)); } catch (e) { reject(e); } }\n                    function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }\n                    step((generator = generator.apply(thisArg, _arguments || [])).next());\n                });\n            };"
        };
        function createAwaiterHelper(context, hasLexicalThis, hasLexicalArguments, promiseConstructor, body) {
            context.requestEmitHelper(ts.awaiterHelper);
            var generatorFunc = ts.createFunctionExpression(
            /*modifiers*/ undefined, ts.createToken(41 /* AsteriskToken */), 
            /*name*/ undefined, 
            /*typeParameters*/ undefined, 
            /*parameters*/ [], 
            /*type*/ undefined, body);
            // Mark this node as originally an async function
            (generatorFunc.emitNode || (generatorFunc.emitNode = {})).flags |= 262144 /* AsyncFunctionBody */ | 524288 /* ReuseTempVariableScope */;
            return ts.createCall(ts.getUnscopedHelperName("__awaiter"), 
            /*typeArguments*/ undefined, [
                hasLexicalThis ? ts.createThis() : ts.createVoidZero(),
                hasLexicalArguments ? ts.createIdentifier("arguments") : ts.createVoidZero(),
                promiseConstructor ? ts.createExpressionFromEntityName(promiseConstructor) : ts.createVoidZero(),
                generatorFunc
            ]);
        }
        ts.asyncSuperHelper = {
            name: "typescript:async-super",
            scoped: true,
            text: ts.helperString(__makeTemplateObject(["\n            const ", " = name => super[name];"], ["\n            const ", " = name => super[name];"]), "_superIndex")
        };
        ts.advancedAsyncSuperHelper = {
            name: "typescript:advanced-async-super",
            scoped: true,
            text: ts.helperString(__makeTemplateObject(["\n            const ", " = (function (geti, seti) {\n                const cache = Object.create(null);\n                return name => cache[name] || (cache[name] = { get value() { return geti(name); }, set value(v) { seti(name, v); } });\n            })(name => super[name], (name, value) => super[name] = value);"], ["\n            const ", " = (function (geti, seti) {\n                const cache = Object.create(null);\n                return name => cache[name] || (cache[name] = { get value() { return geti(name); }, set value(v) { seti(name, v); } });\n            })(name => super[name], (name, value) => super[name] = value);"]), "_superIndex")
        };
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        var ESNextSubstitutionFlags;
        (function (ESNextSubstitutionFlags) {
            /** Enables substitutions for async methods with `super` calls. */
            ESNextSubstitutionFlags[ESNextSubstitutionFlags["AsyncMethodsWithSuper"] = 1] = "AsyncMethodsWithSuper";
        })(ESNextSubstitutionFlags || (ESNextSubstitutionFlags = {}));
        function transformES2018(context) {
            var resumeLexicalEnvironment = context.resumeLexicalEnvironment, endLexicalEnvironment = context.endLexicalEnvironment, hoistVariableDeclaration = context.hoistVariableDeclaration;
            var resolver = context.getEmitResolver();
            var compilerOptions = context.getCompilerOptions();
            var languageVersion = ts.getEmitScriptTarget(compilerOptions);
            var previousOnEmitNode = context.onEmitNode;
            context.onEmitNode = onEmitNode;
            var previousOnSubstituteNode = context.onSubstituteNode;
            context.onSubstituteNode = onSubstituteNode;
            var exportedVariableStatement = false;
            var enabledSubstitutions;
            var enclosingFunctionFlags;
            var enclosingSuperContainerFlags = 0;
            var hasLexicalThis;
            /** Keeps track of property names accessed on super (`super.x`) within async functions. */
            var capturedSuperProperties;
            /** Whether the async function contains an element access on super (`super[x]`). */
            var hasSuperElementAccess;
            /** A set of node IDs for generated super accessors. */
            var substitutedSuperAccessors = [];
            return ts.chainBundle(transformSourceFile);
            function transformSourceFile(node) {
                if (node.isDeclarationFile) {
                    return node;
                }
                exportedVariableStatement = false;
                hasLexicalThis = !ts.isEffectiveStrictModeSourceFile(node, compilerOptions);
                var visited = ts.visitEachChild(node, visitor, context);
                ts.addEmitHelpers(visited, context.readEmitHelpers());
                return visited;
            }
            function visitor(node) {
                return visitorWorker(node, /*noDestructuringValue*/ false);
            }
            function visitorNoDestructuringValue(node) {
                return visitorWorker(node, /*noDestructuringValue*/ true);
            }
            function visitorNoAsyncModifier(node) {
                if (node.kind === 126 /* AsyncKeyword */) {
                    return undefined;
                }
                return node;
            }
            function doWithLexicalThis(cb, value) {
                if (!hasLexicalThis) {
                    hasLexicalThis = true;
                    var result = cb(value);
                    hasLexicalThis = false;
                    return result;
                }
                return cb(value);
            }
            function visitDefault(node) {
                return ts.visitEachChild(node, visitor, context);
            }
            function visitorWorker(node, noDestructuringValue) {
                if ((node.transformFlags & 32 /* ContainsES2018 */) === 0) {
                    return node;
                }
                switch (node.kind) {
                    case 206 /* AwaitExpression */:
                        return visitAwaitExpression(node);
                    case 212 /* YieldExpression */:
                        return visitYieldExpression(node);
                    case 235 /* ReturnStatement */:
                        return visitReturnStatement(node);
                    case 238 /* LabeledStatement */:
                        return visitLabeledStatement(node);
                    case 193 /* ObjectLiteralExpression */:
                        return visitObjectLiteralExpression(node);
                    case 209 /* BinaryExpression */:
                        return visitBinaryExpression(node, noDestructuringValue);
                    case 280 /* CatchClause */:
                        return visitCatchClause(node);
                    case 225 /* VariableStatement */:
                        return visitVariableStatement(node);
                    case 242 /* VariableDeclaration */:
                        return visitVariableDeclaration(node);
                    case 232 /* ForOfStatement */:
                        return visitForOfStatement(node, /*outermostLabeledStatement*/ undefined);
                    case 230 /* ForStatement */:
                        return visitForStatement(node);
                    case 205 /* VoidExpression */:
                        return visitVoidExpression(node);
                    case 162 /* Constructor */:
                        return doWithLexicalThis(visitConstructorDeclaration, node);
                    case 161 /* MethodDeclaration */:
                        return doWithLexicalThis(visitMethodDeclaration, node);
                    case 163 /* GetAccessor */:
                        return doWithLexicalThis(visitGetAccessorDeclaration, node);
                    case 164 /* SetAccessor */:
                        return doWithLexicalThis(visitSetAccessorDeclaration, node);
                    case 244 /* FunctionDeclaration */:
                        return doWithLexicalThis(visitFunctionDeclaration, node);
                    case 201 /* FunctionExpression */:
                        return doWithLexicalThis(visitFunctionExpression, node);
                    case 202 /* ArrowFunction */:
                        return visitArrowFunction(node);
                    case 156 /* Parameter */:
                        return visitParameter(node);
                    case 226 /* ExpressionStatement */:
                        return visitExpressionStatement(node);
                    case 200 /* ParenthesizedExpression */:
                        return visitParenthesizedExpression(node, noDestructuringValue);
                    case 194 /* PropertyAccessExpression */:
                        if (capturedSuperProperties && ts.isPropertyAccessExpression(node) && node.expression.kind === 102 /* SuperKeyword */) {
                            capturedSuperProperties.set(node.name.escapedText, true);
                        }
                        return ts.visitEachChild(node, visitor, context);
                    case 195 /* ElementAccessExpression */:
                        if (capturedSuperProperties && node.expression.kind === 102 /* SuperKeyword */) {
                            hasSuperElementAccess = true;
                        }
                        return ts.visitEachChild(node, visitor, context);
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                        return doWithLexicalThis(visitDefault, node);
                    default:
                        return ts.visitEachChild(node, visitor, context);
                }
            }
            function visitAwaitExpression(node) {
                if (enclosingFunctionFlags & 2 /* Async */ && enclosingFunctionFlags & 1 /* Generator */) {
                    return ts.setOriginalNode(ts.setTextRange(ts.createYield(createAwaitHelper(context, ts.visitNode(node.expression, visitor, ts.isExpression))), 
                    /*location*/ node), node);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitYieldExpression(node) {
                if (enclosingFunctionFlags & 2 /* Async */ && enclosingFunctionFlags & 1 /* Generator */) {
                    if (node.asteriskToken) {
                        var expression = ts.visitNode(node.expression, visitor, ts.isExpression);
                        return ts.setOriginalNode(ts.setTextRange(ts.createYield(createAwaitHelper(context, ts.updateYield(node, node.asteriskToken, createAsyncDelegatorHelper(context, createAsyncValuesHelper(context, expression, expression), expression)))), node), node);
                    }
                    return ts.setOriginalNode(ts.setTextRange(ts.createYield(createDownlevelAwait(node.expression
                        ? ts.visitNode(node.expression, visitor, ts.isExpression)
                        : ts.createVoidZero())), node), node);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitReturnStatement(node) {
                if (enclosingFunctionFlags & 2 /* Async */ && enclosingFunctionFlags & 1 /* Generator */) {
                    return ts.updateReturn(node, createDownlevelAwait(node.expression ? ts.visitNode(node.expression, visitor, ts.isExpression) : ts.createVoidZero()));
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitLabeledStatement(node) {
                if (enclosingFunctionFlags & 2 /* Async */) {
                    var statement = ts.unwrapInnermostStatementOfLabel(node);
                    if (statement.kind === 232 /* ForOfStatement */ && statement.awaitModifier) {
                        return visitForOfStatement(statement, node);
                    }
                    return ts.restoreEnclosingLabel(ts.visitEachChild(statement, visitor, context), node);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function chunkObjectLiteralElements(elements) {
                var chunkObject;
                var objects = [];
                for (var _i = 0, elements_4 = elements; _i < elements_4.length; _i++) {
                    var e = elements_4[_i];
                    if (e.kind === 283 /* SpreadAssignment */) {
                        if (chunkObject) {
                            objects.push(ts.createObjectLiteral(chunkObject));
                            chunkObject = undefined;
                        }
                        var target = e.expression;
                        objects.push(ts.visitNode(target, visitor, ts.isExpression));
                    }
                    else {
                        chunkObject = ts.append(chunkObject, e.kind === 281 /* PropertyAssignment */
                            ? ts.createPropertyAssignment(e.name, ts.visitNode(e.initializer, visitor, ts.isExpression))
                            : ts.visitNode(e, visitor, ts.isObjectLiteralElementLike));
                    }
                }
                if (chunkObject) {
                    objects.push(ts.createObjectLiteral(chunkObject));
                }
                return objects;
            }
            function visitObjectLiteralExpression(node) {
                if (node.transformFlags & 16384 /* ContainsObjectRestOrSpread */) {
                    // spread elements emit like so:
                    // non-spread elements are chunked together into object literals, and then all are passed to __assign:
                    //     { a, ...o, b } => __assign(__assign({a}, o), {b});
                    // If the first element is a spread element, then the first argument to __assign is {}:
                    //     { ...o, a, b, ...o2 } => __assign(__assign(__assign({}, o), {a, b}), o2)
                    //
                    // We cannot call __assign with more than two elements, since any element could cause side effects. For
                    // example:
                    //      var k = { a: 1, b: 2 };
                    //      var o = { a: 3, ...k, b: k.a++ };
                    //      // expected: { a: 1, b: 1 }
                    // If we translate the above to `__assign({ a: 3 }, k, { b: k.a++ })`, the `k.a++` will evaluate before
                    // `k` is spread and we end up with `{ a: 2, b: 1 }`.
                    //
                    // This also occurs for spread elements, not just property assignments:
                    //      var k = { a: 1, get b() { l = { z: 9 }; return 2; } };
                    //      var l = { c: 3 };
                    //      var o = { ...k, ...l };
                    //      // expected: { a: 1, b: 2, z: 9 }
                    // If we translate the above to `__assign({}, k, l)`, the `l` will evaluate before `k` is spread and we
                    // end up with `{ a: 1, b: 2, c: 3 }`
                    var objects = chunkObjectLiteralElements(node.properties);
                    if (objects.length && objects[0].kind !== 193 /* ObjectLiteralExpression */) {
                        objects.unshift(ts.createObjectLiteral());
                    }
                    var expression = objects[0];
                    if (objects.length > 1) {
                        for (var i = 1; i < objects.length; i++) {
                            expression = createAssignHelper(context, [expression, objects[i]]);
                        }
                        return expression;
                    }
                    else {
                        return createAssignHelper(context, objects);
                    }
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitExpressionStatement(node) {
                return ts.visitEachChild(node, visitorNoDestructuringValue, context);
            }
            function visitParenthesizedExpression(node, noDestructuringValue) {
                return ts.visitEachChild(node, noDestructuringValue ? visitorNoDestructuringValue : visitor, context);
            }
            /**
             * Visits a BinaryExpression that contains a destructuring assignment.
             *
             * @param node A BinaryExpression node.
             */
            function visitBinaryExpression(node, noDestructuringValue) {
                if (ts.isDestructuringAssignment(node) && node.left.transformFlags & 16384 /* ContainsObjectRestOrSpread */) {
                    return ts.flattenDestructuringAssignment(node, visitor, context, 1 /* ObjectRest */, !noDestructuringValue);
                }
                else if (node.operatorToken.kind === 27 /* CommaToken */) {
                    return ts.updateBinary(node, ts.visitNode(node.left, visitorNoDestructuringValue, ts.isExpression), ts.visitNode(node.right, noDestructuringValue ? visitorNoDestructuringValue : visitor, ts.isExpression));
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitCatchClause(node) {
                if (node.variableDeclaration &&
                    ts.isBindingPattern(node.variableDeclaration.name) &&
                    node.variableDeclaration.name.transformFlags & 16384 /* ContainsObjectRestOrSpread */) {
                    var name = ts.getGeneratedNameForNode(node.variableDeclaration.name);
                    var updatedDecl = ts.updateVariableDeclaration(node.variableDeclaration, node.variableDeclaration.name, /*type*/ undefined, name);
                    var visitedBindings = ts.flattenDestructuringBinding(updatedDecl, visitor, context, 1 /* ObjectRest */);
                    var block = ts.visitNode(node.block, visitor, ts.isBlock);
                    if (ts.some(visitedBindings)) {
                        block = ts.updateBlock(block, __spreadArrays([
                            ts.createVariableStatement(/*modifiers*/ undefined, visitedBindings)
                        ], block.statements));
                    }
                    return ts.updateCatchClause(node, ts.updateVariableDeclaration(node.variableDeclaration, name, /*type*/ undefined, /*initializer*/ undefined), block);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitVariableStatement(node) {
                if (ts.hasModifier(node, 1 /* Export */)) {
                    var savedExportedVariableStatement = exportedVariableStatement;
                    exportedVariableStatement = true;
                    var visited = ts.visitEachChild(node, visitor, context);
                    exportedVariableStatement = savedExportedVariableStatement;
                    return visited;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Visits a VariableDeclaration node with a binding pattern.
             *
             * @param node A VariableDeclaration node.
             */
            function visitVariableDeclaration(node) {
                if (exportedVariableStatement) {
                    var savedExportedVariableStatement = exportedVariableStatement;
                    exportedVariableStatement = false;
                    var visited = visitVariableDeclarationWorker(node, /*exportedVariableStatement*/ true);
                    exportedVariableStatement = savedExportedVariableStatement;
                    return visited;
                }
                return visitVariableDeclarationWorker(node, /*exportedVariableStatement*/ false);
            }
            function visitVariableDeclarationWorker(node, exportedVariableStatement) {
                // If we are here it is because the name contains a binding pattern with a rest somewhere in it.
                if (ts.isBindingPattern(node.name) && node.name.transformFlags & 16384 /* ContainsObjectRestOrSpread */) {
                    return ts.flattenDestructuringBinding(node, visitor, context, 1 /* ObjectRest */, 
                    /*rval*/ undefined, exportedVariableStatement);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitForStatement(node) {
                return ts.updateFor(node, ts.visitNode(node.initializer, visitorNoDestructuringValue, ts.isForInitializer), ts.visitNode(node.condition, visitor, ts.isExpression), ts.visitNode(node.incrementor, visitor, ts.isExpression), ts.visitNode(node.statement, visitor, ts.isStatement));
            }
            function visitVoidExpression(node) {
                return ts.visitEachChild(node, visitorNoDestructuringValue, context);
            }
            /**
             * Visits a ForOfStatement and converts it into a ES2015-compatible ForOfStatement.
             *
             * @param node A ForOfStatement.
             */
            function visitForOfStatement(node, outermostLabeledStatement) {
                if (node.initializer.transformFlags & 16384 /* ContainsObjectRestOrSpread */) {
                    node = transformForOfStatementWithObjectRest(node);
                }
                if (node.awaitModifier) {
                    return transformForAwaitOfStatement(node, outermostLabeledStatement);
                }
                else {
                    return ts.restoreEnclosingLabel(ts.visitEachChild(node, visitor, context), outermostLabeledStatement);
                }
            }
            function transformForOfStatementWithObjectRest(node) {
                var initializerWithoutParens = ts.skipParentheses(node.initializer);
                if (ts.isVariableDeclarationList(initializerWithoutParens) || ts.isAssignmentPattern(initializerWithoutParens)) {
                    var bodyLocation = void 0;
                    var statementsLocation = void 0;
                    var temp = ts.createTempVariable(/*recordTempVariable*/ undefined);
                    var statements = [ts.createForOfBindingStatement(initializerWithoutParens, temp)];
                    if (ts.isBlock(node.statement)) {
                        ts.addRange(statements, node.statement.statements);
                        bodyLocation = node.statement;
                        statementsLocation = node.statement.statements;
                    }
                    else if (node.statement) {
                        ts.append(statements, node.statement);
                        bodyLocation = node.statement;
                        statementsLocation = node.statement;
                    }
                    return ts.updateForOf(node, node.awaitModifier, ts.setTextRange(ts.createVariableDeclarationList([
                        ts.setTextRange(ts.createVariableDeclaration(temp), node.initializer)
                    ], 1 /* Let */), node.initializer), node.expression, ts.setTextRange(ts.createBlock(ts.setTextRange(ts.createNodeArray(statements), statementsLocation), 
                    /*multiLine*/ true), bodyLocation));
                }
                return node;
            }
            function convertForOfStatementHead(node, boundValue) {
                var binding = ts.createForOfBindingStatement(node.initializer, boundValue);
                var bodyLocation;
                var statementsLocation;
                var statements = [ts.visitNode(binding, visitor, ts.isStatement)];
                var statement = ts.visitNode(node.statement, visitor, ts.isStatement);
                if (ts.isBlock(statement)) {
                    ts.addRange(statements, statement.statements);
                    bodyLocation = statement;
                    statementsLocation = statement.statements;
                }
                else {
                    statements.push(statement);
                }
                return ts.setEmitFlags(ts.setTextRange(ts.createBlock(ts.setTextRange(ts.createNodeArray(statements), statementsLocation), 
                /*multiLine*/ true), bodyLocation), 48 /* NoSourceMap */ | 384 /* NoTokenSourceMaps */);
            }
            function createDownlevelAwait(expression) {
                return enclosingFunctionFlags & 1 /* Generator */
                    ? ts.createYield(/*asteriskToken*/ undefined, createAwaitHelper(context, expression))
                    : ts.createAwait(expression);
            }
            function transformForAwaitOfStatement(node, outermostLabeledStatement) {
                var expression = ts.visitNode(node.expression, visitor, ts.isExpression);
                var iterator = ts.isIdentifier(expression) ? ts.getGeneratedNameForNode(expression) : ts.createTempVariable(/*recordTempVariable*/ undefined);
                var result = ts.isIdentifier(expression) ? ts.getGeneratedNameForNode(iterator) : ts.createTempVariable(/*recordTempVariable*/ undefined);
                var errorRecord = ts.createUniqueName("e");
                var catchVariable = ts.getGeneratedNameForNode(errorRecord);
                var returnMethod = ts.createTempVariable(/*recordTempVariable*/ undefined);
                var callValues = createAsyncValuesHelper(context, expression, /*location*/ node.expression);
                var callNext = ts.createCall(ts.createPropertyAccess(iterator, "next"), /*typeArguments*/ undefined, []);
                var getDone = ts.createPropertyAccess(result, "done");
                var getValue = ts.createPropertyAccess(result, "value");
                var callReturn = ts.createFunctionCall(returnMethod, iterator, []);
                hoistVariableDeclaration(errorRecord);
                hoistVariableDeclaration(returnMethod);
                var forStatement = ts.setEmitFlags(ts.setTextRange(ts.createFor(
                /*initializer*/ ts.setEmitFlags(ts.setTextRange(ts.createVariableDeclarationList([
                    ts.setTextRange(ts.createVariableDeclaration(iterator, /*type*/ undefined, callValues), node.expression),
                    ts.createVariableDeclaration(result)
                ]), node.expression), 2097152 /* NoHoisting */), 
                /*condition*/ ts.createComma(ts.createAssignment(result, createDownlevelAwait(callNext)), ts.createLogicalNot(getDone)), 
                /*incrementor*/ undefined, 
                /*statement*/ convertForOfStatementHead(node, getValue)), 
                /*location*/ node), 256 /* NoTokenTrailingSourceMaps */);
                return ts.createTry(ts.createBlock([
                    ts.restoreEnclosingLabel(forStatement, outermostLabeledStatement)
                ]), ts.createCatchClause(ts.createVariableDeclaration(catchVariable), ts.setEmitFlags(ts.createBlock([
                    ts.createExpressionStatement(ts.createAssignment(errorRecord, ts.createObjectLiteral([
                        ts.createPropertyAssignment("error", catchVariable)
                    ])))
                ]), 1 /* SingleLine */)), ts.createBlock([
                    ts.createTry(
                    /*tryBlock*/ ts.createBlock([
                        ts.setEmitFlags(ts.createIf(ts.createLogicalAnd(ts.createLogicalAnd(result, ts.createLogicalNot(getDone)), ts.createAssignment(returnMethod, ts.createPropertyAccess(iterator, "return"))), ts.createExpressionStatement(createDownlevelAwait(callReturn))), 1 /* SingleLine */)
                    ]), 
                    /*catchClause*/ undefined, 
                    /*finallyBlock*/ ts.setEmitFlags(ts.createBlock([
                        ts.setEmitFlags(ts.createIf(errorRecord, ts.createThrow(ts.createPropertyAccess(errorRecord, "error"))), 1 /* SingleLine */)
                    ]), 1 /* SingleLine */))
                ]));
            }
            function visitParameter(node) {
                if (node.transformFlags & 16384 /* ContainsObjectRestOrSpread */) {
                    // Binding patterns are converted into a generated name and are
                    // evaluated inside the function body.
                    return ts.updateParameter(node, 
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, node.dotDotDotToken, ts.getGeneratedNameForNode(node), 
                    /*questionToken*/ undefined, 
                    /*type*/ undefined, ts.visitNode(node.initializer, visitor, ts.isExpression));
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitConstructorDeclaration(node) {
                var savedEnclosingFunctionFlags = enclosingFunctionFlags;
                enclosingFunctionFlags = 0 /* Normal */;
                var updated = ts.updateConstructor(node, 
                /*decorators*/ undefined, node.modifiers, ts.visitParameterList(node.parameters, visitor, context), transformFunctionBody(node));
                enclosingFunctionFlags = savedEnclosingFunctionFlags;
                return updated;
            }
            function visitGetAccessorDeclaration(node) {
                var savedEnclosingFunctionFlags = enclosingFunctionFlags;
                enclosingFunctionFlags = 0 /* Normal */;
                var updated = ts.updateGetAccessor(node, 
                /*decorators*/ undefined, node.modifiers, ts.visitNode(node.name, visitor, ts.isPropertyName), ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, transformFunctionBody(node));
                enclosingFunctionFlags = savedEnclosingFunctionFlags;
                return updated;
            }
            function visitSetAccessorDeclaration(node) {
                var savedEnclosingFunctionFlags = enclosingFunctionFlags;
                enclosingFunctionFlags = 0 /* Normal */;
                var updated = ts.updateSetAccessor(node, 
                /*decorators*/ undefined, node.modifiers, ts.visitNode(node.name, visitor, ts.isPropertyName), ts.visitParameterList(node.parameters, visitor, context), transformFunctionBody(node));
                enclosingFunctionFlags = savedEnclosingFunctionFlags;
                return updated;
            }
            function visitMethodDeclaration(node) {
                var savedEnclosingFunctionFlags = enclosingFunctionFlags;
                enclosingFunctionFlags = ts.getFunctionFlags(node);
                var updated = ts.updateMethod(node, 
                /*decorators*/ undefined, enclosingFunctionFlags & 1 /* Generator */
                    ? ts.visitNodes(node.modifiers, visitorNoAsyncModifier, ts.isModifier)
                    : node.modifiers, enclosingFunctionFlags & 2 /* Async */
                    ? undefined
                    : node.asteriskToken, ts.visitNode(node.name, visitor, ts.isPropertyName), ts.visitNode(/*questionToken*/ undefined, visitor, ts.isToken), 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, enclosingFunctionFlags & 2 /* Async */ && enclosingFunctionFlags & 1 /* Generator */
                    ? transformAsyncGeneratorFunctionBody(node)
                    : transformFunctionBody(node));
                enclosingFunctionFlags = savedEnclosingFunctionFlags;
                return updated;
            }
            function visitFunctionDeclaration(node) {
                var savedEnclosingFunctionFlags = enclosingFunctionFlags;
                enclosingFunctionFlags = ts.getFunctionFlags(node);
                var updated = ts.updateFunctionDeclaration(node, 
                /*decorators*/ undefined, enclosingFunctionFlags & 1 /* Generator */
                    ? ts.visitNodes(node.modifiers, visitorNoAsyncModifier, ts.isModifier)
                    : node.modifiers, enclosingFunctionFlags & 2 /* Async */
                    ? undefined
                    : node.asteriskToken, node.name, 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, enclosingFunctionFlags & 2 /* Async */ && enclosingFunctionFlags & 1 /* Generator */
                    ? transformAsyncGeneratorFunctionBody(node)
                    : transformFunctionBody(node));
                enclosingFunctionFlags = savedEnclosingFunctionFlags;
                return updated;
            }
            function visitArrowFunction(node) {
                var savedEnclosingFunctionFlags = enclosingFunctionFlags;
                enclosingFunctionFlags = ts.getFunctionFlags(node);
                var updated = ts.updateArrowFunction(node, node.modifiers, 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, node.equalsGreaterThanToken, transformFunctionBody(node));
                enclosingFunctionFlags = savedEnclosingFunctionFlags;
                return updated;
            }
            function visitFunctionExpression(node) {
                var savedEnclosingFunctionFlags = enclosingFunctionFlags;
                enclosingFunctionFlags = ts.getFunctionFlags(node);
                var updated = ts.updateFunctionExpression(node, enclosingFunctionFlags & 1 /* Generator */
                    ? ts.visitNodes(node.modifiers, visitorNoAsyncModifier, ts.isModifier)
                    : node.modifiers, enclosingFunctionFlags & 2 /* Async */
                    ? undefined
                    : node.asteriskToken, node.name, 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, enclosingFunctionFlags & 2 /* Async */ && enclosingFunctionFlags & 1 /* Generator */
                    ? transformAsyncGeneratorFunctionBody(node)
                    : transformFunctionBody(node));
                enclosingFunctionFlags = savedEnclosingFunctionFlags;
                return updated;
            }
            function transformAsyncGeneratorFunctionBody(node) {
                resumeLexicalEnvironment();
                var statements = [];
                var statementOffset = ts.addPrologue(statements, node.body.statements, /*ensureUseStrict*/ false, visitor);
                appendObjectRestAssignmentsIfNeeded(statements, node);
                var savedCapturedSuperProperties = capturedSuperProperties;
                var savedHasSuperElementAccess = hasSuperElementAccess;
                capturedSuperProperties = ts.createUnderscoreEscapedMap();
                hasSuperElementAccess = false;
                var returnStatement = ts.createReturn(createAsyncGeneratorHelper(context, ts.createFunctionExpression(
                /*modifiers*/ undefined, ts.createToken(41 /* AsteriskToken */), node.name && ts.getGeneratedNameForNode(node.name), 
                /*typeParameters*/ undefined, 
                /*parameters*/ [], 
                /*type*/ undefined, ts.updateBlock(node.body, ts.visitLexicalEnvironment(node.body.statements, visitor, context, statementOffset))), hasLexicalThis));
                // Minor optimization, emit `_super` helper to capture `super` access in an arrow.
                // This step isn't needed if we eventually transform this to ES5.
                var emitSuperHelpers = languageVersion >= 2 /* ES2015 */ && resolver.getNodeCheckFlags(node) & (4096 /* AsyncMethodWithSuperBinding */ | 2048 /* AsyncMethodWithSuper */);
                if (emitSuperHelpers) {
                    enableSubstitutionForAsyncMethodsWithSuper();
                    var variableStatement = ts.createSuperAccessVariableStatement(resolver, node, capturedSuperProperties);
                    substitutedSuperAccessors[ts.getNodeId(variableStatement)] = true;
                    ts.insertStatementsAfterStandardPrologue(statements, [variableStatement]);
                }
                statements.push(returnStatement);
                ts.insertStatementsAfterStandardPrologue(statements, endLexicalEnvironment());
                var block = ts.updateBlock(node.body, statements);
                if (emitSuperHelpers && hasSuperElementAccess) {
                    if (resolver.getNodeCheckFlags(node) & 4096 /* AsyncMethodWithSuperBinding */) {
                        ts.addEmitHelper(block, ts.advancedAsyncSuperHelper);
                    }
                    else if (resolver.getNodeCheckFlags(node) & 2048 /* AsyncMethodWithSuper */) {
                        ts.addEmitHelper(block, ts.asyncSuperHelper);
                    }
                }
                capturedSuperProperties = savedCapturedSuperProperties;
                hasSuperElementAccess = savedHasSuperElementAccess;
                return block;
            }
            function transformFunctionBody(node) {
                resumeLexicalEnvironment();
                var statementOffset = 0;
                var statements = [];
                var body = ts.visitNode(node.body, visitor, ts.isConciseBody);
                if (ts.isBlock(body)) {
                    statementOffset = ts.addPrologue(statements, body.statements, /*ensureUseStrict*/ false, visitor);
                }
                ts.addRange(statements, appendObjectRestAssignmentsIfNeeded(/*statements*/ undefined, node));
                var leadingStatements = endLexicalEnvironment();
                if (statementOffset > 0 || ts.some(statements) || ts.some(leadingStatements)) {
                    var block = ts.convertToFunctionBody(body, /*multiLine*/ true);
                    ts.insertStatementsAfterStandardPrologue(statements, leadingStatements);
                    ts.addRange(statements, block.statements.slice(statementOffset));
                    return ts.updateBlock(block, ts.setTextRange(ts.createNodeArray(statements), block.statements));
                }
                return body;
            }
            function appendObjectRestAssignmentsIfNeeded(statements, node) {
                for (var _i = 0, _a = node.parameters; _i < _a.length; _i++) {
                    var parameter = _a[_i];
                    if (parameter.transformFlags & 16384 /* ContainsObjectRestOrSpread */) {
                        var temp = ts.getGeneratedNameForNode(parameter);
                        var declarations = ts.flattenDestructuringBinding(parameter, visitor, context, 1 /* ObjectRest */, temp, 
                        /*doNotRecordTempVariablesInLine*/ false, 
                        /*skipInitializer*/ true);
                        if (ts.some(declarations)) {
                            var statement = ts.createVariableStatement(
                            /*modifiers*/ undefined, ts.createVariableDeclarationList(declarations));
                            ts.setEmitFlags(statement, 1048576 /* CustomPrologue */);
                            statements = ts.append(statements, statement);
                        }
                    }
                }
                return statements;
            }
            function enableSubstitutionForAsyncMethodsWithSuper() {
                if ((enabledSubstitutions & 1 /* AsyncMethodsWithSuper */) === 0) {
                    enabledSubstitutions |= 1 /* AsyncMethodsWithSuper */;
                    // We need to enable substitutions for call, property access, and element access
                    // if we need to rewrite super calls.
                    context.enableSubstitution(196 /* CallExpression */);
                    context.enableSubstitution(194 /* PropertyAccessExpression */);
                    context.enableSubstitution(195 /* ElementAccessExpression */);
                    // We need to be notified when entering and exiting declarations that bind super.
                    context.enableEmitNotification(245 /* ClassDeclaration */);
                    context.enableEmitNotification(161 /* MethodDeclaration */);
                    context.enableEmitNotification(163 /* GetAccessor */);
                    context.enableEmitNotification(164 /* SetAccessor */);
                    context.enableEmitNotification(162 /* Constructor */);
                    // We need to be notified when entering the generated accessor arrow functions.
                    context.enableEmitNotification(225 /* VariableStatement */);
                }
            }
            /**
             * Called by the printer just before a node is printed.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to be printed.
             * @param emitCallback The callback used to emit the node.
             */
            function onEmitNode(hint, node, emitCallback) {
                // If we need to support substitutions for `super` in an async method,
                // we should track it here.
                if (enabledSubstitutions & 1 /* AsyncMethodsWithSuper */ && isSuperContainer(node)) {
                    var superContainerFlags = resolver.getNodeCheckFlags(node) & (2048 /* AsyncMethodWithSuper */ | 4096 /* AsyncMethodWithSuperBinding */);
                    if (superContainerFlags !== enclosingSuperContainerFlags) {
                        var savedEnclosingSuperContainerFlags = enclosingSuperContainerFlags;
                        enclosingSuperContainerFlags = superContainerFlags;
                        previousOnEmitNode(hint, node, emitCallback);
                        enclosingSuperContainerFlags = savedEnclosingSuperContainerFlags;
                        return;
                    }
                }
                // Disable substitution in the generated super accessor itself.
                else if (enabledSubstitutions && substitutedSuperAccessors[ts.getNodeId(node)]) {
                    var savedEnclosingSuperContainerFlags = enclosingSuperContainerFlags;
                    enclosingSuperContainerFlags = 0;
                    previousOnEmitNode(hint, node, emitCallback);
                    enclosingSuperContainerFlags = savedEnclosingSuperContainerFlags;
                    return;
                }
                previousOnEmitNode(hint, node, emitCallback);
            }
            /**
             * Hooks node substitutions.
             *
             * @param hint The context for the emitter.
             * @param node The node to substitute.
             */
            function onSubstituteNode(hint, node) {
                node = previousOnSubstituteNode(hint, node);
                if (hint === 1 /* Expression */ && enclosingSuperContainerFlags) {
                    return substituteExpression(node);
                }
                return node;
            }
            function substituteExpression(node) {
                switch (node.kind) {
                    case 194 /* PropertyAccessExpression */:
                        return substitutePropertyAccessExpression(node);
                    case 195 /* ElementAccessExpression */:
                        return substituteElementAccessExpression(node);
                    case 196 /* CallExpression */:
                        return substituteCallExpression(node);
                }
                return node;
            }
            function substitutePropertyAccessExpression(node) {
                if (node.expression.kind === 102 /* SuperKeyword */) {
                    return ts.setTextRange(ts.createPropertyAccess(ts.createFileLevelUniqueName("_super"), node.name), node);
                }
                return node;
            }
            function substituteElementAccessExpression(node) {
                if (node.expression.kind === 102 /* SuperKeyword */) {
                    return createSuperElementAccessInAsyncMethod(node.argumentExpression, node);
                }
                return node;
            }
            function substituteCallExpression(node) {
                var expression = node.expression;
                if (ts.isSuperProperty(expression)) {
                    var argumentExpression = ts.isPropertyAccessExpression(expression)
                        ? substitutePropertyAccessExpression(expression)
                        : substituteElementAccessExpression(expression);
                    return ts.createCall(ts.createPropertyAccess(argumentExpression, "call"), 
                    /*typeArguments*/ undefined, __spreadArrays([
                        ts.createThis()
                    ], node.arguments));
                }
                return node;
            }
            function isSuperContainer(node) {
                var kind = node.kind;
                return kind === 245 /* ClassDeclaration */
                    || kind === 162 /* Constructor */
                    || kind === 161 /* MethodDeclaration */
                    || kind === 163 /* GetAccessor */
                    || kind === 164 /* SetAccessor */;
            }
            function createSuperElementAccessInAsyncMethod(argumentExpression, location) {
                if (enclosingSuperContainerFlags & 4096 /* AsyncMethodWithSuperBinding */) {
                    return ts.setTextRange(ts.createPropertyAccess(ts.createCall(ts.createIdentifier("_superIndex"), 
                    /*typeArguments*/ undefined, [argumentExpression]), "value"), location);
                }
                else {
                    return ts.setTextRange(ts.createCall(ts.createIdentifier("_superIndex"), 
                    /*typeArguments*/ undefined, [argumentExpression]), location);
                }
            }
        }
        ts.transformES2018 = transformES2018;
        ts.assignHelper = {
            name: "typescript:assign",
            importName: "__assign",
            scoped: false,
            priority: 1,
            text: "\n            var __assign = (this && this.__assign) || function () {\n                __assign = Object.assign || function(t) {\n                    for (var s, i = 1, n = arguments.length; i < n; i++) {\n                        s = arguments[i];\n                        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))\n                            t[p] = s[p];\n                    }\n                    return t;\n                };\n                return __assign.apply(this, arguments);\n            };"
        };
        function createAssignHelper(context, attributesSegments) {
            if (context.getCompilerOptions().target >= 2 /* ES2015 */) {
                return ts.createCall(ts.createPropertyAccess(ts.createIdentifier("Object"), "assign"), /*typeArguments*/ undefined, attributesSegments);
            }
            context.requestEmitHelper(ts.assignHelper);
            return ts.createCall(ts.getUnscopedHelperName("__assign"), 
            /*typeArguments*/ undefined, attributesSegments);
        }
        ts.createAssignHelper = createAssignHelper;
        ts.awaitHelper = {
            name: "typescript:await",
            importName: "__await",
            scoped: false,
            text: "\n            var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }"
        };
        function createAwaitHelper(context, expression) {
            context.requestEmitHelper(ts.awaitHelper);
            return ts.createCall(ts.getUnscopedHelperName("__await"), /*typeArguments*/ undefined, [expression]);
        }
        ts.asyncGeneratorHelper = {
            name: "typescript:asyncGenerator",
            importName: "__asyncGenerator",
            scoped: false,
            text: "\n            var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {\n                if (!Symbol.asyncIterator) throw new TypeError(\"Symbol.asyncIterator is not defined.\");\n                var g = generator.apply(thisArg, _arguments || []), i, q = [];\n                return i = {}, verb(\"next\"), verb(\"throw\"), verb(\"return\"), i[Symbol.asyncIterator] = function () { return this; }, i;\n                function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }\n                function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }\n                function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }\n                function fulfill(value) { resume(\"next\", value); }\n                function reject(value) { resume(\"throw\", value); }\n                function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }\n            };"
        };
        function createAsyncGeneratorHelper(context, generatorFunc, hasLexicalThis) {
            context.requestEmitHelper(ts.awaitHelper);
            context.requestEmitHelper(ts.asyncGeneratorHelper);
            // Mark this node as originally an async function
            (generatorFunc.emitNode || (generatorFunc.emitNode = {})).flags |= 262144 /* AsyncFunctionBody */;
            return ts.createCall(ts.getUnscopedHelperName("__asyncGenerator"), 
            /*typeArguments*/ undefined, [
                hasLexicalThis ? ts.createThis() : ts.createVoidZero(),
                ts.createIdentifier("arguments"),
                generatorFunc
            ]);
        }
        ts.asyncDelegator = {
            name: "typescript:asyncDelegator",
            importName: "__asyncDelegator",
            scoped: false,
            text: "\n            var __asyncDelegator = (this && this.__asyncDelegator) || function (o) {\n                var i, p;\n                return i = {}, verb(\"next\"), verb(\"throw\", function (e) { throw e; }), verb(\"return\"), i[Symbol.iterator] = function () { return this; }, i;\n                function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === \"return\" } : f ? f(v) : v; } : f; }\n            };"
        };
        function createAsyncDelegatorHelper(context, expression, location) {
            context.requestEmitHelper(ts.awaitHelper);
            context.requestEmitHelper(ts.asyncDelegator);
            return ts.setTextRange(ts.createCall(ts.getUnscopedHelperName("__asyncDelegator"), 
            /*typeArguments*/ undefined, [expression]), location);
        }
        ts.asyncValues = {
            name: "typescript:asyncValues",
            importName: "__asyncValues",
            scoped: false,
            text: "\n            var __asyncValues = (this && this.__asyncValues) || function (o) {\n                if (!Symbol.asyncIterator) throw new TypeError(\"Symbol.asyncIterator is not defined.\");\n                var m = o[Symbol.asyncIterator], i;\n                return m ? m.call(o) : (o = typeof __values === \"function\" ? __values(o) : o[Symbol.iterator](), i = {}, verb(\"next\"), verb(\"throw\"), verb(\"return\"), i[Symbol.asyncIterator] = function () { return this; }, i);\n                function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }\n                function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }\n            };"
        };
        function createAsyncValuesHelper(context, expression, location) {
            context.requestEmitHelper(ts.asyncValues);
            return ts.setTextRange(ts.createCall(ts.getUnscopedHelperName("__asyncValues"), 
            /*typeArguments*/ undefined, [expression]), location);
        }
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        function transformES2019(context) {
            return ts.chainBundle(transformSourceFile);
            function transformSourceFile(node) {
                if (node.isDeclarationFile) {
                    return node;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitor(node) {
                if ((node.transformFlags & 16 /* ContainsES2019 */) === 0) {
                    return node;
                }
                switch (node.kind) {
                    case 280 /* CatchClause */:
                        return visitCatchClause(node);
                    default:
                        return ts.visitEachChild(node, visitor, context);
                }
            }
            function visitCatchClause(node) {
                if (!node.variableDeclaration) {
                    return ts.updateCatchClause(node, ts.createVariableDeclaration(ts.createTempVariable(/*recordTempVariable*/ undefined)), ts.visitNode(node.block, visitor, ts.isBlock));
                }
                return ts.visitEachChild(node, visitor, context);
            }
        }
        ts.transformES2019 = transformES2019;
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        function transformES2020(context) {
            var hoistVariableDeclaration = context.hoistVariableDeclaration;
            return ts.chainBundle(transformSourceFile);
            function transformSourceFile(node) {
                if (node.isDeclarationFile) {
                    return node;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitor(node) {
                if ((node.transformFlags & 8 /* ContainsES2020 */) === 0) {
                    return node;
                }
                switch (node.kind) {
                    case 194 /* PropertyAccessExpression */:
                    case 195 /* ElementAccessExpression */:
                    case 196 /* CallExpression */:
                        if (node.flags & 32 /* OptionalChain */) {
                            var updated = visitOptionalExpression(node, /*captureThisArg*/ false, /*isDelete*/ false);
                            ts.Debug.assertNotNode(updated, ts.isSyntheticReference);
                            return updated;
                        }
                        return ts.visitEachChild(node, visitor, context);
                    case 209 /* BinaryExpression */:
                        if (node.operatorToken.kind === 60 /* QuestionQuestionToken */) {
                            return transformNullishCoalescingExpression(node);
                        }
                        return ts.visitEachChild(node, visitor, context);
                    case 203 /* DeleteExpression */:
                        return visitDeleteExpression(node);
                    default:
                        return ts.visitEachChild(node, visitor, context);
                }
            }
            function flattenChain(chain) {
                var links = [chain];
                while (!chain.questionDotToken && !ts.isTaggedTemplateExpression(chain)) {
                    chain = ts.cast(chain.expression, ts.isOptionalChain);
                    links.unshift(chain);
                }
                return { expression: chain.expression, chain: links };
            }
            function visitNonOptionalParenthesizedExpression(node, captureThisArg, isDelete) {
                var expression = visitNonOptionalExpression(node.expression, captureThisArg, isDelete);
                if (ts.isSyntheticReference(expression)) {
                    // `(a.b)` -> { expression `((_a = a).b)`, thisArg: `_a` }
                    // `(a[b])` -> { expression `((_a = a)[b])`, thisArg: `_a` }
                    return ts.createSyntheticReferenceExpression(ts.updateParen(node, expression.expression), expression.thisArg);
                }
                return ts.updateParen(node, expression);
            }
            function visitNonOptionalPropertyOrElementAccessExpression(node, captureThisArg, isDelete) {
                if (ts.isOptionalChain(node)) {
                    // If `node` is an optional chain, then it is the outermost chain of an optional expression.
                    return visitOptionalExpression(node, captureThisArg, isDelete);
                }
                var expression = ts.visitNode(node.expression, visitor, ts.isExpression);
                ts.Debug.assertNotNode(expression, ts.isSyntheticReference);
                var thisArg;
                if (captureThisArg) {
                    if (shouldCaptureInTempVariable(expression)) {
                        thisArg = ts.createTempVariable(hoistVariableDeclaration);
                        expression = ts.createAssignment(thisArg, expression);
                    }
                    else {
                        thisArg = expression;
                    }
                }
                expression = node.kind === 194 /* PropertyAccessExpression */
                    ? ts.updatePropertyAccess(node, expression, ts.visitNode(node.name, visitor, ts.isIdentifier))
                    : ts.updateElementAccess(node, expression, ts.visitNode(node.argumentExpression, visitor, ts.isExpression));
                return thisArg ? ts.createSyntheticReferenceExpression(expression, thisArg) : expression;
            }
            function visitNonOptionalCallExpression(node, captureThisArg) {
                if (ts.isOptionalChain(node)) {
                    // If `node` is an optional chain, then it is the outermost chain of an optional expression.
                    return visitOptionalExpression(node, captureThisArg, /*isDelete*/ false);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitNonOptionalExpression(node, captureThisArg, isDelete) {
                switch (node.kind) {
                    case 200 /* ParenthesizedExpression */: return visitNonOptionalParenthesizedExpression(node, captureThisArg, isDelete);
                    case 194 /* PropertyAccessExpression */:
                    case 195 /* ElementAccessExpression */: return visitNonOptionalPropertyOrElementAccessExpression(node, captureThisArg, isDelete);
                    case 196 /* CallExpression */: return visitNonOptionalCallExpression(node, captureThisArg);
                    default: return ts.visitNode(node, visitor, ts.isExpression);
                }
            }
            function visitOptionalExpression(node, captureThisArg, isDelete) {
                var _a = flattenChain(node), expression = _a.expression, chain = _a.chain;
                var left = visitNonOptionalExpression(expression, ts.isCallChain(chain[0]), /*isDelete*/ false);
                var leftThisArg = ts.isSyntheticReference(left) ? left.thisArg : undefined;
                var leftExpression = ts.isSyntheticReference(left) ? left.expression : left;
                var capturedLeft = leftExpression;
                if (shouldCaptureInTempVariable(leftExpression)) {
                    capturedLeft = ts.createTempVariable(hoistVariableDeclaration);
                    leftExpression = ts.createAssignment(capturedLeft, leftExpression);
                }
                var rightExpression = capturedLeft;
                var thisArg;
                for (var i = 0; i < chain.length; i++) {
                    var segment = chain[i];
                    switch (segment.kind) {
                        case 194 /* PropertyAccessExpression */:
                        case 195 /* ElementAccessExpression */:
                            if (i === chain.length - 1 && captureThisArg) {
                                if (shouldCaptureInTempVariable(rightExpression)) {
                                    thisArg = ts.createTempVariable(hoistVariableDeclaration);
                                    rightExpression = ts.createAssignment(thisArg, rightExpression);
                                }
                                else {
                                    thisArg = rightExpression;
                                }
                            }
                            rightExpression = segment.kind === 194 /* PropertyAccessExpression */
                                ? ts.createPropertyAccess(rightExpression, ts.visitNode(segment.name, visitor, ts.isIdentifier))
                                : ts.createElementAccess(rightExpression, ts.visitNode(segment.argumentExpression, visitor, ts.isExpression));
                            break;
                        case 196 /* CallExpression */:
                            if (i === 0 && leftThisArg) {
                                rightExpression = ts.createFunctionCall(rightExpression, leftThisArg.kind === 102 /* SuperKeyword */ ? ts.createThis() : leftThisArg, ts.visitNodes(segment.arguments, visitor, ts.isExpression));
                            }
                            else {
                                rightExpression = ts.createCall(rightExpression, 
                                /*typeArguments*/ undefined, ts.visitNodes(segment.arguments, visitor, ts.isExpression));
                            }
                            break;
                    }
                    ts.setOriginalNode(rightExpression, segment);
                }
                var target = isDelete
                    ? ts.createConditional(createNotNullCondition(leftExpression, capturedLeft, /*invert*/ true), ts.createTrue(), ts.createDelete(rightExpression))
                    : ts.createConditional(createNotNullCondition(leftExpression, capturedLeft, /*invert*/ true), ts.createVoidZero(), rightExpression);
                return thisArg ? ts.createSyntheticReferenceExpression(target, thisArg) : target;
            }
            function createNotNullCondition(left, right, invert) {
                return ts.createBinary(ts.createBinary(left, ts.createToken(invert ? 36 /* EqualsEqualsEqualsToken */ : 37 /* ExclamationEqualsEqualsToken */), ts.createNull()), ts.createToken(invert ? 56 /* BarBarToken */ : 55 /* AmpersandAmpersandToken */), ts.createBinary(right, ts.createToken(invert ? 36 /* EqualsEqualsEqualsToken */ : 37 /* ExclamationEqualsEqualsToken */), ts.createVoidZero()));
            }
            function transformNullishCoalescingExpression(node) {
                var left = ts.visitNode(node.left, visitor, ts.isExpression);
                var right = left;
                if (shouldCaptureInTempVariable(left)) {
                    right = ts.createTempVariable(hoistVariableDeclaration);
                    left = ts.createAssignment(right, left);
                }
                return ts.createConditional(createNotNullCondition(left, right), right, ts.visitNode(node.right, visitor, ts.isExpression));
            }
            function shouldCaptureInTempVariable(expression) {
                // don't capture identifiers and `this` in a temporary variable
                // `super` cannot be captured as it's no real variable
                return !ts.isIdentifier(expression) &&
                    expression.kind !== 104 /* ThisKeyword */ &&
                    expression.kind !== 102 /* SuperKeyword */;
            }
            function visitDeleteExpression(node) {
                return ts.isOptionalChain(ts.skipParentheses(node.expression))
                    ? ts.setOriginalNode(visitNonOptionalExpression(node.expression, /*captureThisArg*/ false, /*isDelete*/ true), node)
                    : ts.updateDelete(node, ts.visitNode(node.expression, visitor, ts.isExpression));
            }
        }
        ts.transformES2020 = transformES2020;
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        function transformESNext(context) {
            return ts.chainBundle(transformSourceFile);
            function transformSourceFile(node) {
                if (node.isDeclarationFile) {
                    return node;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitor(node) {
                if ((node.transformFlags & 4 /* ContainsESNext */) === 0) {
                    return node;
                }
                switch (node.kind) {
                    default:
                        return ts.visitEachChild(node, visitor, context);
                }
            }
        }
        ts.transformESNext = transformESNext;
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        function transformJsx(context) {
            var compilerOptions = context.getCompilerOptions();
            var currentSourceFile;
            return ts.chainBundle(transformSourceFile);
            /**
             * Transform JSX-specific syntax in a SourceFile.
             *
             * @param node A SourceFile node.
             */
            function transformSourceFile(node) {
                if (node.isDeclarationFile) {
                    return node;
                }
                currentSourceFile = node;
                var visited = ts.visitEachChild(node, visitor, context);
                ts.addEmitHelpers(visited, context.readEmitHelpers());
                return visited;
            }
            function visitor(node) {
                if (node.transformFlags & 2 /* ContainsJsx */) {
                    return visitorWorker(node);
                }
                else {
                    return node;
                }
            }
            function visitorWorker(node) {
                switch (node.kind) {
                    case 266 /* JsxElement */:
                        return visitJsxElement(node, /*isChild*/ false);
                    case 267 /* JsxSelfClosingElement */:
                        return visitJsxSelfClosingElement(node, /*isChild*/ false);
                    case 270 /* JsxFragment */:
                        return visitJsxFragment(node, /*isChild*/ false);
                    case 276 /* JsxExpression */:
                        return visitJsxExpression(node);
                    default:
                        return ts.visitEachChild(node, visitor, context);
                }
            }
            function transformJsxChildToExpression(node) {
                switch (node.kind) {
                    case 11 /* JsxText */:
                        return visitJsxText(node);
                    case 276 /* JsxExpression */:
                        return visitJsxExpression(node);
                    case 266 /* JsxElement */:
                        return visitJsxElement(node, /*isChild*/ true);
                    case 267 /* JsxSelfClosingElement */:
                        return visitJsxSelfClosingElement(node, /*isChild*/ true);
                    case 270 /* JsxFragment */:
                        return visitJsxFragment(node, /*isChild*/ true);
                    default:
                        return ts.Debug.failBadSyntaxKind(node);
                }
            }
            function visitJsxElement(node, isChild) {
                return visitJsxOpeningLikeElement(node.openingElement, node.children, isChild, /*location*/ node);
            }
            function visitJsxSelfClosingElement(node, isChild) {
                return visitJsxOpeningLikeElement(node, /*children*/ undefined, isChild, /*location*/ node);
            }
            function visitJsxFragment(node, isChild) {
                return visitJsxOpeningFragment(node.openingFragment, node.children, isChild, /*location*/ node);
            }
            function visitJsxOpeningLikeElement(node, children, isChild, location) {
                var tagName = getTagName(node);
                var objectProperties;
                var attrs = node.attributes.properties;
                if (attrs.length === 0) {
                    // When there are no attributes, React wants "null"
                    objectProperties = ts.createNull();
                }
                else {
                    // Map spans of JsxAttribute nodes into object literals and spans
                    // of JsxSpreadAttribute nodes into expressions.
                    var segments = ts.flatten(ts.spanMap(attrs, ts.isJsxSpreadAttribute, function (attrs, isSpread) {
                        return isSpread
                            ? ts.map(attrs, transformJsxSpreadAttributeToExpression)
                            : ts.createObjectLiteral(ts.map(attrs, transformJsxAttributeToObjectLiteralElement));
                    }));
                    if (ts.isJsxSpreadAttribute(attrs[0])) {
                        // We must always emit at least one object literal before a spread
                        // argument.
                        segments.unshift(ts.createObjectLiteral());
                    }
                    // Either emit one big object literal (no spread attribs), or
                    // a call to the __assign helper.
                    objectProperties = ts.singleOrUndefined(segments);
                    if (!objectProperties) {
                        objectProperties = ts.createAssignHelper(context, segments);
                    }
                }
                var element = ts.createExpressionForJsxElement(context.getEmitResolver().getJsxFactoryEntity(currentSourceFile), compilerOptions.reactNamespace, // TODO: GH#18217
                tagName, objectProperties, ts.mapDefined(children, transformJsxChildToExpression), node, location);
                if (isChild) {
                    ts.startOnNewLine(element);
                }
                return element;
            }
            function visitJsxOpeningFragment(node, children, isChild, location) {
                var element = ts.createExpressionForJsxFragment(context.getEmitResolver().getJsxFactoryEntity(currentSourceFile), compilerOptions.reactNamespace, // TODO: GH#18217
                ts.mapDefined(children, transformJsxChildToExpression), node, location);
                if (isChild) {
                    ts.startOnNewLine(element);
                }
                return element;
            }
            function transformJsxSpreadAttributeToExpression(node) {
                return ts.visitNode(node.expression, visitor, ts.isExpression);
            }
            function transformJsxAttributeToObjectLiteralElement(node) {
                var name = getAttributeName(node);
                var expression = transformJsxAttributeInitializer(node.initializer);
                return ts.createPropertyAssignment(name, expression);
            }
            function transformJsxAttributeInitializer(node) {
                if (node === undefined) {
                    return ts.createTrue();
                }
                else if (node.kind === 10 /* StringLiteral */) {
                    // Always recreate the literal to escape any escape sequences or newlines which may be in the original jsx string and which
                    // Need to be escaped to be handled correctly in a normal string
                    var literal = ts.createLiteral(tryDecodeEntities(node.text) || node.text);
                    literal.singleQuote = node.singleQuote !== undefined ? node.singleQuote : !ts.isStringDoubleQuoted(node, currentSourceFile);
                    return ts.setTextRange(literal, node);
                }
                else if (node.kind === 276 /* JsxExpression */) {
                    if (node.expression === undefined) {
                        return ts.createTrue();
                    }
                    return visitJsxExpression(node);
                }
                else {
                    return ts.Debug.failBadSyntaxKind(node);
                }
            }
            function visitJsxText(node) {
                var fixed = fixupWhitespaceAndDecodeEntities(node.text);
                return fixed === undefined ? undefined : ts.createLiteral(fixed);
            }
            /**
             * JSX trims whitespace at the end and beginning of lines, except that the
             * start/end of a tag is considered a start/end of a line only if that line is
             * on the same line as the closing tag. See examples in
             * tests/cases/conformance/jsx/tsxReactEmitWhitespace.tsx
             * See also https://www.w3.org/TR/html4/struct/text.html#h-9.1 and https://www.w3.org/TR/CSS2/text.html#white-space-model
             *
             * An equivalent algorithm would be:
             * - If there is only one line, return it.
             * - If there is only whitespace (but multiple lines), return `undefined`.
             * - Split the text into lines.
             * - 'trimRight' the first line, 'trimLeft' the last line, 'trim' middle lines.
             * - Decode entities on each line (individually).
             * - Remove empty lines and join the rest with " ".
             */
            function fixupWhitespaceAndDecodeEntities(text) {
                var acc;
                // First non-whitespace character on this line.
                var firstNonWhitespace = 0;
                // Last non-whitespace character on this line.
                var lastNonWhitespace = -1;
                // These initial values are special because the first line is:
                // firstNonWhitespace = 0 to indicate that we want leading whitsepace,
                // but lastNonWhitespace = -1 as a special flag to indicate that we *don't* include the line if it's all whitespace.
                for (var i = 0; i < text.length; i++) {
                    var c = text.charCodeAt(i);
                    if (ts.isLineBreak(c)) {
                        // If we've seen any non-whitespace characters on this line, add the 'trim' of the line.
                        // (lastNonWhitespace === -1 is a special flag to detect whether the first line is all whitespace.)
                        if (firstNonWhitespace !== -1 && lastNonWhitespace !== -1) {
                            acc = addLineOfJsxText(acc, text.substr(firstNonWhitespace, lastNonWhitespace - firstNonWhitespace + 1));
                        }
                        // Reset firstNonWhitespace for the next line.
                        // Don't bother to reset lastNonWhitespace because we ignore it if firstNonWhitespace = -1.
                        firstNonWhitespace = -1;
                    }
                    else if (!ts.isWhiteSpaceSingleLine(c)) {
                        lastNonWhitespace = i;
                        if (firstNonWhitespace === -1) {
                            firstNonWhitespace = i;
                        }
                    }
                }
                return firstNonWhitespace !== -1
                    // Last line had a non-whitespace character. Emit the 'trimLeft', meaning keep trailing whitespace.
                    ? addLineOfJsxText(acc, text.substr(firstNonWhitespace))
                    // Last line was all whitespace, so ignore it
                    : acc;
            }
            function addLineOfJsxText(acc, trimmedLine) {
                // We do not escape the string here as that is handled by the printer
                // when it emits the literal. We do, however, need to decode JSX entities.
                var decoded = decodeEntities(trimmedLine);
                return acc === undefined ? decoded : acc + " " + decoded;
            }
            /**
             * Replace entities like "&nbsp;", "&#123;", and "&#xDEADBEEF;" with the characters they encode.
             * See https://en.wikipedia.org/wiki/List_of_XML_and_HTML_character_entity_references
             */
            function decodeEntities(text) {
                return text.replace(/&((#((\d+)|x([\da-fA-F]+)))|(\w+));/g, function (match, _all, _number, _digits, decimal, hex, word) {
                    if (decimal) {
                        return ts.utf16EncodeAsString(parseInt(decimal, 10));
                    }
                    else if (hex) {
                        return ts.utf16EncodeAsString(parseInt(hex, 16));
                    }
                    else {
                        var ch = entities.get(word);
                        // If this is not a valid entity, then just use `match` (replace it with itself, i.e. don't replace)
                        return ch ? ts.utf16EncodeAsString(ch) : match;
                    }
                });
            }
            /** Like `decodeEntities` but returns `undefined` if there were no entities to decode. */
            function tryDecodeEntities(text) {
                var decoded = decodeEntities(text);
                return decoded === text ? undefined : decoded;
            }
            function getTagName(node) {
                if (node.kind === 266 /* JsxElement */) {
                    return getTagName(node.openingElement);
                }
                else {
                    var name = node.tagName;
                    if (ts.isIdentifier(name) && ts.isIntrinsicJsxName(name.escapedText)) {
                        return ts.createLiteral(ts.idText(name));
                    }
                    else {
                        return ts.createExpressionFromEntityName(name);
                    }
                }
            }
            /**
             * Emit an attribute name, which is quoted if it needs to be quoted. Because
             * these emit into an object literal property name, we don't need to be worried
             * about keywords, just non-identifier characters
             */
            function getAttributeName(node) {
                var name = node.name;
                var text = ts.idText(name);
                if (/^[A-Za-z_]\w*$/.test(text)) {
                    return name;
                }
                else {
                    return ts.createLiteral(text);
                }
            }
            function visitJsxExpression(node) {
                return ts.visitNode(node.expression, visitor, ts.isExpression);
            }
        }
        ts.transformJsx = transformJsx;
        var entities = ts.createMapFromTemplate({
            quot: 0x0022,
            amp: 0x0026,
            apos: 0x0027,
            lt: 0x003C,
            gt: 0x003E,
            nbsp: 0x00A0,
            iexcl: 0x00A1,
            cent: 0x00A2,
            pound: 0x00A3,
            curren: 0x00A4,
            yen: 0x00A5,
            brvbar: 0x00A6,
            sect: 0x00A7,
            uml: 0x00A8,
            copy: 0x00A9,
            ordf: 0x00AA,
            laquo: 0x00AB,
            not: 0x00AC,
            shy: 0x00AD,
            reg: 0x00AE,
            macr: 0x00AF,
            deg: 0x00B0,
            plusmn: 0x00B1,
            sup2: 0x00B2,
            sup3: 0x00B3,
            acute: 0x00B4,
            micro: 0x00B5,
            para: 0x00B6,
            middot: 0x00B7,
            cedil: 0x00B8,
            sup1: 0x00B9,
            ordm: 0x00BA,
            raquo: 0x00BB,
            frac14: 0x00BC,
            frac12: 0x00BD,
            frac34: 0x00BE,
            iquest: 0x00BF,
            Agrave: 0x00C0,
            Aacute: 0x00C1,
            Acirc: 0x00C2,
            Atilde: 0x00C3,
            Auml: 0x00C4,
            Aring: 0x00C5,
            AElig: 0x00C6,
            Ccedil: 0x00C7,
            Egrave: 0x00C8,
            Eacute: 0x00C9,
            Ecirc: 0x00CA,
            Euml: 0x00CB,
            Igrave: 0x00CC,
            Iacute: 0x00CD,
            Icirc: 0x00CE,
            Iuml: 0x00CF,
            ETH: 0x00D0,
            Ntilde: 0x00D1,
            Ograve: 0x00D2,
            Oacute: 0x00D3,
            Ocirc: 0x00D4,
            Otilde: 0x00D5,
            Ouml: 0x00D6,
            times: 0x00D7,
            Oslash: 0x00D8,
            Ugrave: 0x00D9,
            Uacute: 0x00DA,
            Ucirc: 0x00DB,
            Uuml: 0x00DC,
            Yacute: 0x00DD,
            THORN: 0x00DE,
            szlig: 0x00DF,
            agrave: 0x00E0,
            aacute: 0x00E1,
            acirc: 0x00E2,
            atilde: 0x00E3,
            auml: 0x00E4,
            aring: 0x00E5,
            aelig: 0x00E6,
            ccedil: 0x00E7,
            egrave: 0x00E8,
            eacute: 0x00E9,
            ecirc: 0x00EA,
            euml: 0x00EB,
            igrave: 0x00EC,
            iacute: 0x00ED,
            icirc: 0x00EE,
            iuml: 0x00EF,
            eth: 0x00F0,
            ntilde: 0x00F1,
            ograve: 0x00F2,
            oacute: 0x00F3,
            ocirc: 0x00F4,
            otilde: 0x00F5,
            ouml: 0x00F6,
            divide: 0x00F7,
            oslash: 0x00F8,
            ugrave: 0x00F9,
            uacute: 0x00FA,
            ucirc: 0x00FB,
            uuml: 0x00FC,
            yacute: 0x00FD,
            thorn: 0x00FE,
            yuml: 0x00FF,
            OElig: 0x0152,
            oelig: 0x0153,
            Scaron: 0x0160,
            scaron: 0x0161,
            Yuml: 0x0178,
            fnof: 0x0192,
            circ: 0x02C6,
            tilde: 0x02DC,
            Alpha: 0x0391,
            Beta: 0x0392,
            Gamma: 0x0393,
            Delta: 0x0394,
            Epsilon: 0x0395,
            Zeta: 0x0396,
            Eta: 0x0397,
            Theta: 0x0398,
            Iota: 0x0399,
            Kappa: 0x039A,
            Lambda: 0x039B,
            Mu: 0x039C,
            Nu: 0x039D,
            Xi: 0x039E,
            Omicron: 0x039F,
            Pi: 0x03A0,
            Rho: 0x03A1,
            Sigma: 0x03A3,
            Tau: 0x03A4,
            Upsilon: 0x03A5,
            Phi: 0x03A6,
            Chi: 0x03A7,
            Psi: 0x03A8,
            Omega: 0x03A9,
            alpha: 0x03B1,
            beta: 0x03B2,
            gamma: 0x03B3,
            delta: 0x03B4,
            epsilon: 0x03B5,
            zeta: 0x03B6,
            eta: 0x03B7,
            theta: 0x03B8,
            iota: 0x03B9,
            kappa: 0x03BA,
            lambda: 0x03BB,
            mu: 0x03BC,
            nu: 0x03BD,
            xi: 0x03BE,
            omicron: 0x03BF,
            pi: 0x03C0,
            rho: 0x03C1,
            sigmaf: 0x03C2,
            sigma: 0x03C3,
            tau: 0x03C4,
            upsilon: 0x03C5,
            phi: 0x03C6,
            chi: 0x03C7,
            psi: 0x03C8,
            omega: 0x03C9,
            thetasym: 0x03D1,
            upsih: 0x03D2,
            piv: 0x03D6,
            ensp: 0x2002,
            emsp: 0x2003,
            thinsp: 0x2009,
            zwnj: 0x200C,
            zwj: 0x200D,
            lrm: 0x200E,
            rlm: 0x200F,
            ndash: 0x2013,
            mdash: 0x2014,
            lsquo: 0x2018,
            rsquo: 0x2019,
            sbquo: 0x201A,
            ldquo: 0x201C,
            rdquo: 0x201D,
            bdquo: 0x201E,
            dagger: 0x2020,
            Dagger: 0x2021,
            bull: 0x2022,
            hellip: 0x2026,
            permil: 0x2030,
            prime: 0x2032,
            Prime: 0x2033,
            lsaquo: 0x2039,
            rsaquo: 0x203A,
            oline: 0x203E,
            frasl: 0x2044,
            euro: 0x20AC,
            image: 0x2111,
            weierp: 0x2118,
            real: 0x211C,
            trade: 0x2122,
            alefsym: 0x2135,
            larr: 0x2190,
            uarr: 0x2191,
            rarr: 0x2192,
            darr: 0x2193,
            harr: 0x2194,
            crarr: 0x21B5,
            lArr: 0x21D0,
            uArr: 0x21D1,
            rArr: 0x21D2,
            dArr: 0x21D3,
            hArr: 0x21D4,
            forall: 0x2200,
            part: 0x2202,
            exist: 0x2203,
            empty: 0x2205,
            nabla: 0x2207,
            isin: 0x2208,
            notin: 0x2209,
            ni: 0x220B,
            prod: 0x220F,
            sum: 0x2211,
            minus: 0x2212,
            lowast: 0x2217,
            radic: 0x221A,
            prop: 0x221D,
            infin: 0x221E,
            ang: 0x2220,
            and: 0x2227,
            or: 0x2228,
            cap: 0x2229,
            cup: 0x222A,
            int: 0x222B,
            there4: 0x2234,
            sim: 0x223C,
            cong: 0x2245,
            asymp: 0x2248,
            ne: 0x2260,
            equiv: 0x2261,
            le: 0x2264,
            ge: 0x2265,
            sub: 0x2282,
            sup: 0x2283,
            nsub: 0x2284,
            sube: 0x2286,
            supe: 0x2287,
            oplus: 0x2295,
            otimes: 0x2297,
            perp: 0x22A5,
            sdot: 0x22C5,
            lceil: 0x2308,
            rceil: 0x2309,
            lfloor: 0x230A,
            rfloor: 0x230B,
            lang: 0x2329,
            rang: 0x232A,
            loz: 0x25CA,
            spades: 0x2660,
            clubs: 0x2663,
            hearts: 0x2665,
            diams: 0x2666
        });
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        function transformES2016(context) {
            var hoistVariableDeclaration = context.hoistVariableDeclaration;
            return ts.chainBundle(transformSourceFile);
            function transformSourceFile(node) {
                if (node.isDeclarationFile) {
                    return node;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitor(node) {
                if ((node.transformFlags & 128 /* ContainsES2016 */) === 0) {
                    return node;
                }
                switch (node.kind) {
                    case 209 /* BinaryExpression */:
                        return visitBinaryExpression(node);
                    default:
                        return ts.visitEachChild(node, visitor, context);
                }
            }
            function visitBinaryExpression(node) {
                switch (node.operatorToken.kind) {
                    case 66 /* AsteriskAsteriskEqualsToken */:
                        return visitExponentiationAssignmentExpression(node);
                    case 42 /* AsteriskAsteriskToken */:
                        return visitExponentiationExpression(node);
                    default:
                        return ts.visitEachChild(node, visitor, context);
                }
            }
            function visitExponentiationAssignmentExpression(node) {
                var target;
                var value;
                var left = ts.visitNode(node.left, visitor, ts.isExpression);
                var right = ts.visitNode(node.right, visitor, ts.isExpression);
                if (ts.isElementAccessExpression(left)) {
                    // Transforms `a[x] **= b` into `(_a = a)[_x = x] = Math.pow(_a[_x], b)`
                    var expressionTemp = ts.createTempVariable(hoistVariableDeclaration);
                    var argumentExpressionTemp = ts.createTempVariable(hoistVariableDeclaration);
                    target = ts.setTextRange(ts.createElementAccess(ts.setTextRange(ts.createAssignment(expressionTemp, left.expression), left.expression), ts.setTextRange(ts.createAssignment(argumentExpressionTemp, left.argumentExpression), left.argumentExpression)), left);
                    value = ts.setTextRange(ts.createElementAccess(expressionTemp, argumentExpressionTemp), left);
                }
                else if (ts.isPropertyAccessExpression(left)) {
                    // Transforms `a.x **= b` into `(_a = a).x = Math.pow(_a.x, b)`
                    var expressionTemp = ts.createTempVariable(hoistVariableDeclaration);
                    target = ts.setTextRange(ts.createPropertyAccess(ts.setTextRange(ts.createAssignment(expressionTemp, left.expression), left.expression), left.name), left);
                    value = ts.setTextRange(ts.createPropertyAccess(expressionTemp, left.name), left);
                }
                else {
                    // Transforms `a **= b` into `a = Math.pow(a, b)`
                    target = left;
                    value = left;
                }
                return ts.setTextRange(ts.createAssignment(target, ts.createMathPow(value, right, /*location*/ node)), node);
            }
            function visitExponentiationExpression(node) {
                // Transforms `a ** b` into `Math.pow(a, b)`
                var left = ts.visitNode(node.left, visitor, ts.isExpression);
                var right = ts.visitNode(node.right, visitor, ts.isExpression);
                return ts.createMathPow(left, right, /*location*/ node);
            }
        }
        ts.transformES2016 = transformES2016;
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        var ES2015SubstitutionFlags;
        (function (ES2015SubstitutionFlags) {
            /** Enables substitutions for captured `this` */
            ES2015SubstitutionFlags[ES2015SubstitutionFlags["CapturedThis"] = 1] = "CapturedThis";
            /** Enables substitutions for block-scoped bindings. */
            ES2015SubstitutionFlags[ES2015SubstitutionFlags["BlockScopedBindings"] = 2] = "BlockScopedBindings";
        })(ES2015SubstitutionFlags || (ES2015SubstitutionFlags = {}));
        var LoopOutParameterFlags;
        (function (LoopOutParameterFlags) {
            LoopOutParameterFlags[LoopOutParameterFlags["Body"] = 1] = "Body";
            LoopOutParameterFlags[LoopOutParameterFlags["Initializer"] = 2] = "Initializer";
        })(LoopOutParameterFlags || (LoopOutParameterFlags = {}));
        var CopyDirection;
        (function (CopyDirection) {
            CopyDirection[CopyDirection["ToOriginal"] = 0] = "ToOriginal";
            CopyDirection[CopyDirection["ToOutParameter"] = 1] = "ToOutParameter";
        })(CopyDirection || (CopyDirection = {}));
        var Jump;
        (function (Jump) {
            Jump[Jump["Break"] = 2] = "Break";
            Jump[Jump["Continue"] = 4] = "Continue";
            Jump[Jump["Return"] = 8] = "Return";
        })(Jump || (Jump = {}));
        // Facts we track as we traverse the tree
        var HierarchyFacts;
        (function (HierarchyFacts) {
            HierarchyFacts[HierarchyFacts["None"] = 0] = "None";
            //
            // Ancestor facts
            //
            HierarchyFacts[HierarchyFacts["Function"] = 1] = "Function";
            HierarchyFacts[HierarchyFacts["ArrowFunction"] = 2] = "ArrowFunction";
            HierarchyFacts[HierarchyFacts["AsyncFunctionBody"] = 4] = "AsyncFunctionBody";
            HierarchyFacts[HierarchyFacts["NonStaticClassElement"] = 8] = "NonStaticClassElement";
            HierarchyFacts[HierarchyFacts["CapturesThis"] = 16] = "CapturesThis";
            HierarchyFacts[HierarchyFacts["ExportedVariableStatement"] = 32] = "ExportedVariableStatement";
            HierarchyFacts[HierarchyFacts["TopLevel"] = 64] = "TopLevel";
            HierarchyFacts[HierarchyFacts["Block"] = 128] = "Block";
            HierarchyFacts[HierarchyFacts["IterationStatement"] = 256] = "IterationStatement";
            HierarchyFacts[HierarchyFacts["IterationStatementBlock"] = 512] = "IterationStatementBlock";
            HierarchyFacts[HierarchyFacts["IterationContainer"] = 1024] = "IterationContainer";
            HierarchyFacts[HierarchyFacts["ForStatement"] = 2048] = "ForStatement";
            HierarchyFacts[HierarchyFacts["ForInOrForOfStatement"] = 4096] = "ForInOrForOfStatement";
            HierarchyFacts[HierarchyFacts["ConstructorWithCapturedSuper"] = 8192] = "ConstructorWithCapturedSuper";
            // NOTE: do not add more ancestor flags without also updating AncestorFactsMask below.
            // NOTE: when adding a new ancestor flag, be sure to update the subtree flags below.
            //
            // Ancestor masks
            //
            HierarchyFacts[HierarchyFacts["AncestorFactsMask"] = 16383] = "AncestorFactsMask";
            // We are always in *some* kind of block scope, but only specific block-scope containers are
            // top-level or Blocks.
            HierarchyFacts[HierarchyFacts["BlockScopeIncludes"] = 0] = "BlockScopeIncludes";
            HierarchyFacts[HierarchyFacts["BlockScopeExcludes"] = 7104] = "BlockScopeExcludes";
            // A source file is a top-level block scope.
            HierarchyFacts[HierarchyFacts["SourceFileIncludes"] = 64] = "SourceFileIncludes";
            HierarchyFacts[HierarchyFacts["SourceFileExcludes"] = 8064] = "SourceFileExcludes";
            // Functions, methods, and accessors are both new lexical scopes and new block scopes.
            HierarchyFacts[HierarchyFacts["FunctionIncludes"] = 65] = "FunctionIncludes";
            HierarchyFacts[HierarchyFacts["FunctionExcludes"] = 16286] = "FunctionExcludes";
            HierarchyFacts[HierarchyFacts["AsyncFunctionBodyIncludes"] = 69] = "AsyncFunctionBodyIncludes";
            HierarchyFacts[HierarchyFacts["AsyncFunctionBodyExcludes"] = 16278] = "AsyncFunctionBodyExcludes";
            // Arrow functions are lexically scoped to their container, but are new block scopes.
            HierarchyFacts[HierarchyFacts["ArrowFunctionIncludes"] = 66] = "ArrowFunctionIncludes";
            HierarchyFacts[HierarchyFacts["ArrowFunctionExcludes"] = 15232] = "ArrowFunctionExcludes";
            // Constructors are both new lexical scopes and new block scopes. Constructors are also
            // always considered non-static members of a class.
            HierarchyFacts[HierarchyFacts["ConstructorIncludes"] = 73] = "ConstructorIncludes";
            HierarchyFacts[HierarchyFacts["ConstructorExcludes"] = 16278] = "ConstructorExcludes";
            // 'do' and 'while' statements are not block scopes. We track that the subtree is contained
            // within an IterationStatement to indicate whether the embedded statement is an
            // IterationStatementBlock.
            HierarchyFacts[HierarchyFacts["DoOrWhileStatementIncludes"] = 1280] = "DoOrWhileStatementIncludes";
            HierarchyFacts[HierarchyFacts["DoOrWhileStatementExcludes"] = 0] = "DoOrWhileStatementExcludes";
            // 'for' statements are new block scopes and have special handling for 'let' declarations.
            HierarchyFacts[HierarchyFacts["ForStatementIncludes"] = 3328] = "ForStatementIncludes";
            HierarchyFacts[HierarchyFacts["ForStatementExcludes"] = 5056] = "ForStatementExcludes";
            // 'for-in' and 'for-of' statements are new block scopes and have special handling for
            // 'let' declarations.
            HierarchyFacts[HierarchyFacts["ForInOrForOfStatementIncludes"] = 5376] = "ForInOrForOfStatementIncludes";
            HierarchyFacts[HierarchyFacts["ForInOrForOfStatementExcludes"] = 3008] = "ForInOrForOfStatementExcludes";
            // Blocks (other than function bodies) are new block scopes.
            HierarchyFacts[HierarchyFacts["BlockIncludes"] = 128] = "BlockIncludes";
            HierarchyFacts[HierarchyFacts["BlockExcludes"] = 6976] = "BlockExcludes";
            HierarchyFacts[HierarchyFacts["IterationStatementBlockIncludes"] = 512] = "IterationStatementBlockIncludes";
            HierarchyFacts[HierarchyFacts["IterationStatementBlockExcludes"] = 7104] = "IterationStatementBlockExcludes";
            //
            // Subtree facts
            //
            HierarchyFacts[HierarchyFacts["NewTarget"] = 16384] = "NewTarget";
            HierarchyFacts[HierarchyFacts["CapturedLexicalThis"] = 32768] = "CapturedLexicalThis";
            //
            // Subtree masks
            //
            HierarchyFacts[HierarchyFacts["SubtreeFactsMask"] = -16384] = "SubtreeFactsMask";
            HierarchyFacts[HierarchyFacts["ArrowFunctionSubtreeExcludes"] = 0] = "ArrowFunctionSubtreeExcludes";
            HierarchyFacts[HierarchyFacts["FunctionSubtreeExcludes"] = 49152] = "FunctionSubtreeExcludes";
        })(HierarchyFacts || (HierarchyFacts = {}));
        function transformES2015(context) {
            var startLexicalEnvironment = context.startLexicalEnvironment, resumeLexicalEnvironment = context.resumeLexicalEnvironment, endLexicalEnvironment = context.endLexicalEnvironment, hoistVariableDeclaration = context.hoistVariableDeclaration;
            var compilerOptions = context.getCompilerOptions();
            var resolver = context.getEmitResolver();
            var previousOnSubstituteNode = context.onSubstituteNode;
            var previousOnEmitNode = context.onEmitNode;
            context.onEmitNode = onEmitNode;
            context.onSubstituteNode = onSubstituteNode;
            var currentSourceFile;
            var currentText;
            var hierarchyFacts;
            var taggedTemplateStringDeclarations;
            function recordTaggedTemplateString(temp) {
                taggedTemplateStringDeclarations = ts.append(taggedTemplateStringDeclarations, ts.createVariableDeclaration(temp));
            }
            /**
             * Used to track if we are emitting body of the converted loop
             */
            var convertedLoopState;
            /**
             * Keeps track of whether substitutions have been enabled for specific cases.
             * They are persisted between each SourceFile transformation and should not
             * be reset.
             */
            var enabledSubstitutions;
            return ts.chainBundle(transformSourceFile);
            function transformSourceFile(node) {
                if (node.isDeclarationFile) {
                    return node;
                }
                currentSourceFile = node;
                currentText = node.text;
                var visited = visitSourceFile(node);
                ts.addEmitHelpers(visited, context.readEmitHelpers());
                currentSourceFile = undefined;
                currentText = undefined;
                taggedTemplateStringDeclarations = undefined;
                hierarchyFacts = 0 /* None */;
                return visited;
            }
            /**
             * Sets the `HierarchyFacts` for this node prior to visiting this node's subtree, returning the facts set prior to modification.
             * @param excludeFacts The existing `HierarchyFacts` to reset before visiting the subtree.
             * @param includeFacts The new `HierarchyFacts` to set before visiting the subtree.
             */
            function enterSubtree(excludeFacts, includeFacts) {
                var ancestorFacts = hierarchyFacts;
                hierarchyFacts = (hierarchyFacts & ~excludeFacts | includeFacts) & 16383 /* AncestorFactsMask */;
                return ancestorFacts;
            }
            /**
             * Restores the `HierarchyFacts` for this node's ancestor after visiting this node's
             * subtree, propagating specific facts from the subtree.
             * @param ancestorFacts The `HierarchyFacts` of the ancestor to restore after visiting the subtree.
             * @param excludeFacts The existing `HierarchyFacts` of the subtree that should not be propagated.
             * @param includeFacts The new `HierarchyFacts` of the subtree that should be propagated.
             */
            function exitSubtree(ancestorFacts, excludeFacts, includeFacts) {
                hierarchyFacts = (hierarchyFacts & ~excludeFacts | includeFacts) & -16384 /* SubtreeFactsMask */ | ancestorFacts;
            }
            function isReturnVoidStatementInConstructorWithCapturedSuper(node) {
                return (hierarchyFacts & 8192 /* ConstructorWithCapturedSuper */) !== 0
                    && node.kind === 235 /* ReturnStatement */
                    && !node.expression;
            }
            function shouldVisitNode(node) {
                return (node.transformFlags & 256 /* ContainsES2015 */) !== 0
                    || convertedLoopState !== undefined
                    || (hierarchyFacts & 8192 /* ConstructorWithCapturedSuper */ && (ts.isStatement(node) || (node.kind === 223 /* Block */)))
                    || (ts.isIterationStatement(node, /*lookInLabeledStatements*/ false) && shouldConvertIterationStatement(node))
                    || (ts.getEmitFlags(node) & 33554432 /* TypeScriptClassWrapper */) !== 0;
            }
            function visitor(node) {
                if (shouldVisitNode(node)) {
                    return visitJavaScript(node);
                }
                else {
                    return node;
                }
            }
            function callExpressionVisitor(node) {
                if (node.kind === 102 /* SuperKeyword */) {
                    return visitSuperKeyword(/*isExpressionOfCall*/ true);
                }
                return visitor(node);
            }
            function visitJavaScript(node) {
                switch (node.kind) {
                    case 120 /* StaticKeyword */:
                        return undefined; // elide static keyword
                    case 245 /* ClassDeclaration */:
                        return visitClassDeclaration(node);
                    case 214 /* ClassExpression */:
                        return visitClassExpression(node);
                    case 156 /* Parameter */:
                        return visitParameter(node);
                    case 244 /* FunctionDeclaration */:
                        return visitFunctionDeclaration(node);
                    case 202 /* ArrowFunction */:
                        return visitArrowFunction(node);
                    case 201 /* FunctionExpression */:
                        return visitFunctionExpression(node);
                    case 242 /* VariableDeclaration */:
                        return visitVariableDeclaration(node);
                    case 75 /* Identifier */:
                        return visitIdentifier(node);
                    case 243 /* VariableDeclarationList */:
                        return visitVariableDeclarationList(node);
                    case 237 /* SwitchStatement */:
                        return visitSwitchStatement(node);
                    case 251 /* CaseBlock */:
                        return visitCaseBlock(node);
                    case 223 /* Block */:
                        return visitBlock(node, /*isFunctionBody*/ false);
                    case 234 /* BreakStatement */:
                    case 233 /* ContinueStatement */:
                        return visitBreakOrContinueStatement(node);
                    case 238 /* LabeledStatement */:
                        return visitLabeledStatement(node);
                    case 228 /* DoStatement */:
                    case 229 /* WhileStatement */:
                        return visitDoOrWhileStatement(node, /*outermostLabeledStatement*/ undefined);
                    case 230 /* ForStatement */:
                        return visitForStatement(node, /*outermostLabeledStatement*/ undefined);
                    case 231 /* ForInStatement */:
                        return visitForInStatement(node, /*outermostLabeledStatement*/ undefined);
                    case 232 /* ForOfStatement */:
                        return visitForOfStatement(node, /*outermostLabeledStatement*/ undefined);
                    case 226 /* ExpressionStatement */:
                        return visitExpressionStatement(node);
                    case 193 /* ObjectLiteralExpression */:
                        return visitObjectLiteralExpression(node);
                    case 280 /* CatchClause */:
                        return visitCatchClause(node);
                    case 282 /* ShorthandPropertyAssignment */:
                        return visitShorthandPropertyAssignment(node);
                    case 154 /* ComputedPropertyName */:
                        return visitComputedPropertyName(node);
                    case 192 /* ArrayLiteralExpression */:
                        return visitArrayLiteralExpression(node);
                    case 196 /* CallExpression */:
                        return visitCallExpression(node);
                    case 197 /* NewExpression */:
                        return visitNewExpression(node);
                    case 200 /* ParenthesizedExpression */:
                        return visitParenthesizedExpression(node, /*needsDestructuringValue*/ true);
                    case 209 /* BinaryExpression */:
                        return visitBinaryExpression(node, /*needsDestructuringValue*/ true);
                    case 14 /* NoSubstitutionTemplateLiteral */:
                    case 15 /* TemplateHead */:
                    case 16 /* TemplateMiddle */:
                    case 17 /* TemplateTail */:
                        return visitTemplateLiteral(node);
                    case 10 /* StringLiteral */:
                        return visitStringLiteral(node);
                    case 8 /* NumericLiteral */:
                        return visitNumericLiteral(node);
                    case 198 /* TaggedTemplateExpression */:
                        return visitTaggedTemplateExpression(node);
                    case 211 /* TemplateExpression */:
                        return visitTemplateExpression(node);
                    case 212 /* YieldExpression */:
                        return visitYieldExpression(node);
                    case 213 /* SpreadElement */:
                        return visitSpreadElement(node);
                    case 102 /* SuperKeyword */:
                        return visitSuperKeyword(/*isExpressionOfCall*/ false);
                    case 104 /* ThisKeyword */:
                        return visitThisKeyword(node);
                    case 219 /* MetaProperty */:
                        return visitMetaProperty(node);
                    case 161 /* MethodDeclaration */:
                        return visitMethodDeclaration(node);
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        return visitAccessorDeclaration(node);
                    case 225 /* VariableStatement */:
                        return visitVariableStatement(node);
                    case 235 /* ReturnStatement */:
                        return visitReturnStatement(node);
                    default:
                        return ts.visitEachChild(node, visitor, context);
                }
            }
            function visitSourceFile(node) {
                var ancestorFacts = enterSubtree(8064 /* SourceFileExcludes */, 64 /* SourceFileIncludes */);
                var prologue = [];
                var statements = [];
                startLexicalEnvironment();
                var statementOffset = ts.addStandardPrologue(prologue, node.statements, /*ensureUseStrict*/ false);
                statementOffset = ts.addCustomPrologue(prologue, node.statements, statementOffset, visitor);
                ts.addRange(statements, ts.visitNodes(node.statements, visitor, ts.isStatement, statementOffset));
                if (taggedTemplateStringDeclarations) {
                    statements.push(ts.createVariableStatement(/*modifiers*/ undefined, ts.createVariableDeclarationList(taggedTemplateStringDeclarations)));
                }
                ts.mergeLexicalEnvironment(prologue, endLexicalEnvironment());
                insertCaptureThisForNodeIfNeeded(prologue, node);
                exitSubtree(ancestorFacts, 0 /* None */, 0 /* None */);
                return ts.updateSourceFileNode(node, ts.setTextRange(ts.createNodeArray(ts.concatenate(prologue, statements)), node.statements));
            }
            function visitSwitchStatement(node) {
                if (convertedLoopState !== undefined) {
                    var savedAllowedNonLabeledJumps = convertedLoopState.allowedNonLabeledJumps;
                    // for switch statement allow only non-labeled break
                    convertedLoopState.allowedNonLabeledJumps |= 2 /* Break */;
                    var result = ts.visitEachChild(node, visitor, context);
                    convertedLoopState.allowedNonLabeledJumps = savedAllowedNonLabeledJumps;
                    return result;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitCaseBlock(node) {
                var ancestorFacts = enterSubtree(7104 /* BlockScopeExcludes */, 0 /* BlockScopeIncludes */);
                var updated = ts.visitEachChild(node, visitor, context);
                exitSubtree(ancestorFacts, 0 /* None */, 0 /* None */);
                return updated;
            }
            function returnCapturedThis(node) {
                return ts.setOriginalNode(ts.createReturn(ts.createFileLevelUniqueName("_this")), node);
            }
            function visitReturnStatement(node) {
                if (convertedLoopState) {
                    convertedLoopState.nonLocalJumps |= 8 /* Return */;
                    if (isReturnVoidStatementInConstructorWithCapturedSuper(node)) {
                        node = returnCapturedThis(node);
                    }
                    return ts.createReturn(ts.createObjectLiteral([
                        ts.createPropertyAssignment(ts.createIdentifier("value"), node.expression
                            ? ts.visitNode(node.expression, visitor, ts.isExpression)
                            : ts.createVoidZero())
                    ]));
                }
                else if (isReturnVoidStatementInConstructorWithCapturedSuper(node)) {
                    return returnCapturedThis(node);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitThisKeyword(node) {
                if (hierarchyFacts & 2 /* ArrowFunction */) {
                    hierarchyFacts |= 32768 /* CapturedLexicalThis */;
                }
                if (convertedLoopState) {
                    if (hierarchyFacts & 2 /* ArrowFunction */) {
                        // if the enclosing function is an ArrowFunction then we use the captured 'this' keyword.
                        convertedLoopState.containsLexicalThis = true;
                        return node;
                    }
                    return convertedLoopState.thisName || (convertedLoopState.thisName = ts.createUniqueName("this"));
                }
                return node;
            }
            function visitIdentifier(node) {
                if (!convertedLoopState) {
                    return node;
                }
                if (ts.isGeneratedIdentifier(node)) {
                    return node;
                }
                if (node.escapedText !== "arguments" || !resolver.isArgumentsLocalBinding(node)) {
                    return node;
                }
                return convertedLoopState.argumentsName || (convertedLoopState.argumentsName = ts.createUniqueName("arguments"));
            }
            function visitBreakOrContinueStatement(node) {
                if (convertedLoopState) {
                    // check if we can emit break/continue as is
                    // it is possible if either
                    //   - break/continue is labeled and label is located inside the converted loop
                    //   - break/continue is non-labeled and located in non-converted loop/switch statement
                    var jump = node.kind === 234 /* BreakStatement */ ? 2 /* Break */ : 4 /* Continue */;
                    var canUseBreakOrContinue = (node.label && convertedLoopState.labels && convertedLoopState.labels.get(ts.idText(node.label))) ||
                        (!node.label && (convertedLoopState.allowedNonLabeledJumps & jump));
                    if (!canUseBreakOrContinue) {
                        var labelMarker = void 0;
                        var label = node.label;
                        if (!label) {
                            if (node.kind === 234 /* BreakStatement */) {
                                convertedLoopState.nonLocalJumps |= 2 /* Break */;
                                labelMarker = "break";
                            }
                            else {
                                convertedLoopState.nonLocalJumps |= 4 /* Continue */;
                                // note: return value is emitted only to simplify debugging, call to converted loop body does not do any dispatching on it.
                                labelMarker = "continue";
                            }
                        }
                        else {
                            if (node.kind === 234 /* BreakStatement */) {
                                labelMarker = "break-" + label.escapedText;
                                setLabeledJump(convertedLoopState, /*isBreak*/ true, ts.idText(label), labelMarker);
                            }
                            else {
                                labelMarker = "continue-" + label.escapedText;
                                setLabeledJump(convertedLoopState, /*isBreak*/ false, ts.idText(label), labelMarker);
                            }
                        }
                        var returnExpression = ts.createLiteral(labelMarker);
                        if (convertedLoopState.loopOutParameters.length) {
                            var outParams = convertedLoopState.loopOutParameters;
                            var expr = void 0;
                            for (var i = 0; i < outParams.length; i++) {
                                var copyExpr = copyOutParameter(outParams[i], 1 /* ToOutParameter */);
                                if (i === 0) {
                                    expr = copyExpr;
                                }
                                else {
                                    expr = ts.createBinary(expr, 27 /* CommaToken */, copyExpr);
                                }
                            }
                            returnExpression = ts.createBinary(expr, 27 /* CommaToken */, returnExpression);
                        }
                        return ts.createReturn(returnExpression);
                    }
                }
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Visits a ClassDeclaration and transforms it into a variable statement.
             *
             * @param node A ClassDeclaration node.
             */
            function visitClassDeclaration(node) {
                // [source]
                //      class C { }
                //
                // [output]
                //      var C = (function () {
                //          function C() {
                //          }
                //          return C;
                //      }());
                var variable = ts.createVariableDeclaration(ts.getLocalName(node, /*allowComments*/ true), 
                /*type*/ undefined, transformClassLikeDeclarationToExpression(node));
                ts.setOriginalNode(variable, node);
                var statements = [];
                var statement = ts.createVariableStatement(/*modifiers*/ undefined, ts.createVariableDeclarationList([variable]));
                ts.setOriginalNode(statement, node);
                ts.setTextRange(statement, node);
                ts.startOnNewLine(statement);
                statements.push(statement);
                // Add an `export default` statement for default exports (for `--target es5 --module es6`)
                if (ts.hasModifier(node, 1 /* Export */)) {
                    var exportStatement = ts.hasModifier(node, 512 /* Default */)
                        ? ts.createExportDefault(ts.getLocalName(node))
                        : ts.createExternalModuleExport(ts.getLocalName(node));
                    ts.setOriginalNode(exportStatement, statement);
                    statements.push(exportStatement);
                }
                var emitFlags = ts.getEmitFlags(node);
                if ((emitFlags & 4194304 /* HasEndOfDeclarationMarker */) === 0) {
                    // Add a DeclarationMarker as a marker for the end of the declaration
                    statements.push(ts.createEndOfDeclarationMarker(node));
                    ts.setEmitFlags(statement, emitFlags | 4194304 /* HasEndOfDeclarationMarker */);
                }
                return ts.singleOrMany(statements);
            }
            /**
             * Visits a ClassExpression and transforms it into an expression.
             *
             * @param node A ClassExpression node.
             */
            function visitClassExpression(node) {
                // [source]
                //      C = class { }
                //
                // [output]
                //      C = (function () {
                //          function class_1() {
                //          }
                //          return class_1;
                //      }())
                return transformClassLikeDeclarationToExpression(node);
            }
            /**
             * Transforms a ClassExpression or ClassDeclaration into an expression.
             *
             * @param node A ClassExpression or ClassDeclaration node.
             */
            function transformClassLikeDeclarationToExpression(node) {
                // [source]
                //      class C extends D {
                //          constructor() {}
                //          method() {}
                //          get prop() {}
                //          set prop(v) {}
                //      }
                //
                // [output]
                //      (function (_super) {
                //          __extends(C, _super);
                //          function C() {
                //          }
                //          C.prototype.method = function () {}
                //          Object.defineProperty(C.prototype, "prop", {
                //              get: function() {},
                //              set: function() {},
                //              enumerable: true,
                //              configurable: true
                //          });
                //          return C;
                //      }(D))
                if (node.name) {
                    enableSubstitutionsForBlockScopedBindings();
                }
                var extendsClauseElement = ts.getClassExtendsHeritageElement(node);
                var classFunction = ts.createFunctionExpression(
                /*modifiers*/ undefined, 
                /*asteriskToken*/ undefined, 
                /*name*/ undefined, 
                /*typeParameters*/ undefined, extendsClauseElement ? [ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, ts.createFileLevelUniqueName("_super"))] : [], 
                /*type*/ undefined, transformClassBody(node, extendsClauseElement));
                // To preserve the behavior of the old emitter, we explicitly indent
                // the body of the function here if it was requested in an earlier
                // transformation.
                ts.setEmitFlags(classFunction, (ts.getEmitFlags(node) & 65536 /* Indented */) | 524288 /* ReuseTempVariableScope */);
                // "inner" and "outer" below are added purely to preserve source map locations from
                // the old emitter
                var inner = ts.createPartiallyEmittedExpression(classFunction);
                inner.end = node.end;
                ts.setEmitFlags(inner, 1536 /* NoComments */);
                var outer = ts.createPartiallyEmittedExpression(inner);
                outer.end = ts.skipTrivia(currentText, node.pos);
                ts.setEmitFlags(outer, 1536 /* NoComments */);
                var result = ts.createParen(ts.createCall(outer, 
                /*typeArguments*/ undefined, extendsClauseElement
                    ? [ts.visitNode(extendsClauseElement.expression, visitor, ts.isExpression)]
                    : []));
                ts.addSyntheticLeadingComment(result, 3 /* MultiLineCommentTrivia */, "* @class ");
                return result;
            }
            /**
             * Transforms a ClassExpression or ClassDeclaration into a function body.
             *
             * @param node A ClassExpression or ClassDeclaration node.
             * @param extendsClauseElement The expression for the class `extends` clause.
             */
            function transformClassBody(node, extendsClauseElement) {
                var statements = [];
                startLexicalEnvironment();
                addExtendsHelperIfNeeded(statements, node, extendsClauseElement);
                addConstructor(statements, node, extendsClauseElement);
                addClassMembers(statements, node);
                // Create a synthetic text range for the return statement.
                var closingBraceLocation = ts.createTokenRange(ts.skipTrivia(currentText, node.members.end), 19 /* CloseBraceToken */);
                var localName = ts.getInternalName(node);
                // The following partially-emitted expression exists purely to align our sourcemap
                // emit with the original emitter.
                var outer = ts.createPartiallyEmittedExpression(localName);
                outer.end = closingBraceLocation.end;
                ts.setEmitFlags(outer, 1536 /* NoComments */);
                var statement = ts.createReturn(outer);
                statement.pos = closingBraceLocation.pos;
                ts.setEmitFlags(statement, 1536 /* NoComments */ | 384 /* NoTokenSourceMaps */);
                statements.push(statement);
                ts.insertStatementsAfterStandardPrologue(statements, endLexicalEnvironment());
                var block = ts.createBlock(ts.setTextRange(ts.createNodeArray(statements), /*location*/ node.members), /*multiLine*/ true);
                ts.setEmitFlags(block, 1536 /* NoComments */);
                return block;
            }
            /**
             * Adds a call to the `__extends` helper if needed for a class.
             *
             * @param statements The statements of the class body function.
             * @param node The ClassExpression or ClassDeclaration node.
             * @param extendsClauseElement The expression for the class `extends` clause.
             */
            function addExtendsHelperIfNeeded(statements, node, extendsClauseElement) {
                if (extendsClauseElement) {
                    statements.push(ts.setTextRange(ts.createExpressionStatement(createExtendsHelper(context, ts.getInternalName(node))), 
                    /*location*/ extendsClauseElement));
                }
            }
            /**
             * Adds the constructor of the class to a class body function.
             *
             * @param statements The statements of the class body function.
             * @param node The ClassExpression or ClassDeclaration node.
             * @param extendsClauseElement The expression for the class `extends` clause.
             */
            function addConstructor(statements, node, extendsClauseElement) {
                var savedConvertedLoopState = convertedLoopState;
                convertedLoopState = undefined;
                var ancestorFacts = enterSubtree(16278 /* ConstructorExcludes */, 73 /* ConstructorIncludes */);
                var constructor = ts.getFirstConstructorWithBody(node);
                var hasSynthesizedSuper = hasSynthesizedDefaultSuperCall(constructor, extendsClauseElement !== undefined);
                var constructorFunction = ts.createFunctionDeclaration(
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, 
                /*asteriskToken*/ undefined, ts.getInternalName(node), 
                /*typeParameters*/ undefined, transformConstructorParameters(constructor, hasSynthesizedSuper), 
                /*type*/ undefined, transformConstructorBody(constructor, node, extendsClauseElement, hasSynthesizedSuper));
                ts.setTextRange(constructorFunction, constructor || node);
                if (extendsClauseElement) {
                    ts.setEmitFlags(constructorFunction, 8 /* CapturesThis */);
                }
                statements.push(constructorFunction);
                exitSubtree(ancestorFacts, 49152 /* FunctionSubtreeExcludes */, 0 /* None */);
                convertedLoopState = savedConvertedLoopState;
            }
            /**
             * Transforms the parameters of the constructor declaration of a class.
             *
             * @param constructor The constructor for the class.
             * @param hasSynthesizedSuper A value indicating whether the constructor starts with a
             *                            synthesized `super` call.
             */
            function transformConstructorParameters(constructor, hasSynthesizedSuper) {
                // If the TypeScript transformer needed to synthesize a constructor for property
                // initializers, it would have also added a synthetic `...args` parameter and
                // `super` call.
                // If this is the case, we do not include the synthetic `...args` parameter and
                // will instead use the `arguments` object in ES5/3.
                return ts.visitParameterList(constructor && !hasSynthesizedSuper ? constructor.parameters : undefined, visitor, context)
                    || [];
            }
            function createDefaultConstructorBody(node, isDerivedClass) {
                // We must be here because the user didn't write a constructor
                // but we needed to call 'super(...args)' anyway as per 14.5.14 of the ES2016 spec.
                // If that's the case we can just immediately return the result of a 'super()' call.
                var statements = [];
                resumeLexicalEnvironment();
                ts.mergeLexicalEnvironment(statements, endLexicalEnvironment());
                if (isDerivedClass) {
                    // return _super !== null && _super.apply(this, arguments) || this;
                    statements.push(ts.createReturn(createDefaultSuperCallOrThis()));
                }
                var statementsArray = ts.createNodeArray(statements);
                ts.setTextRange(statementsArray, node.members);
                var block = ts.createBlock(statementsArray, /*multiLine*/ true);
                ts.setTextRange(block, node);
                ts.setEmitFlags(block, 1536 /* NoComments */);
                return block;
            }
            /**
             * Transforms the body of a constructor declaration of a class.
             *
             * @param constructor The constructor for the class.
             * @param node The node which contains the constructor.
             * @param extendsClauseElement The expression for the class `extends` clause.
             * @param hasSynthesizedSuper A value indicating whether the constructor starts with a
             *                            synthesized `super` call.
             */
            function transformConstructorBody(constructor, node, extendsClauseElement, hasSynthesizedSuper) {
                // determine whether the class is known syntactically to be a derived class (e.g. a
                // class that extends a value that is not syntactically known to be `null`).
                var isDerivedClass = !!extendsClauseElement && ts.skipOuterExpressions(extendsClauseElement.expression).kind !== 100 /* NullKeyword */;
                // When the subclass does not have a constructor, we synthesize a *default* constructor using the following
                // representation:
                //
                // ```
                // // es2015 (source)
                // class C extends Base { }
                //
                // // es5 (transformed)
                // var C = (function (_super) {
                //     function C() {
                //         return _super.apply(this, arguments) || this;
                //     }
                //     return C;
                // })(Base);
                // ```
                if (!constructor)
                    return createDefaultConstructorBody(node, isDerivedClass);
                // The prologue will contain all leading standard and custom prologue statements added by this transform
                var prologue = [];
                var statements = [];
                resumeLexicalEnvironment();
                // If a super call has already been synthesized,
                // we're going to assume that we should just transform everything after that.
                // The assumption is that no prior step in the pipeline has added any prologue directives.
                var statementOffset = 0;
                if (!hasSynthesizedSuper)
                    statementOffset = ts.addStandardPrologue(prologue, constructor.body.statements, /*ensureUseStrict*/ false);
                addDefaultValueAssignmentsIfNeeded(statements, constructor);
                addRestParameterIfNeeded(statements, constructor, hasSynthesizedSuper);
                if (!hasSynthesizedSuper)
                    statementOffset = ts.addCustomPrologue(statements, constructor.body.statements, statementOffset, visitor);
                // If the first statement is a call to `super()`, visit the statement directly
                var superCallExpression;
                if (hasSynthesizedSuper) {
                    superCallExpression = createDefaultSuperCallOrThis();
                }
                else if (isDerivedClass && statementOffset < constructor.body.statements.length) {
                    var firstStatement = constructor.body.statements[statementOffset];
                    if (ts.isExpressionStatement(firstStatement) && ts.isSuperCall(firstStatement.expression)) {
                        superCallExpression = visitImmediateSuperCallInBody(firstStatement.expression);
                    }
                }
                if (superCallExpression) {
                    hierarchyFacts |= 8192 /* ConstructorWithCapturedSuper */;
                    statementOffset++; // skip this statement, we will add it after visiting the rest of the body.
                }
                // visit the remaining statements
                ts.addRange(statements, ts.visitNodes(constructor.body.statements, visitor, ts.isStatement, /*start*/ statementOffset));
                ts.mergeLexicalEnvironment(prologue, endLexicalEnvironment());
                insertCaptureNewTargetIfNeeded(prologue, constructor, /*copyOnWrite*/ false);
                if (isDerivedClass) {
                    if (superCallExpression && statementOffset === constructor.body.statements.length && !(constructor.body.transformFlags & 4096 /* ContainsLexicalThis */)) {
                        // If the subclass constructor does *not* contain `this` and *ends* with a `super()` call, we will use the
                        // following representation:
                        //
                        // ```
                        // // es2015 (source)
                        // class C extends Base {
                        //     constructor() {
                        //         super("foo");
                        //     }
                        // }
                        //
                        // // es5 (transformed)
                        // var C = (function (_super) {
                        //     function C() {
                        //         return _super.call(this, "foo") || this;
                        //     }
                        //     return C;
                        // })(Base);
                        // ```
                        var superCall = ts.cast(ts.cast(superCallExpression, ts.isBinaryExpression).left, ts.isCallExpression);
                        var returnStatement = ts.createReturn(superCallExpression);
                        ts.setCommentRange(returnStatement, ts.getCommentRange(superCall));
                        ts.setEmitFlags(superCall, 1536 /* NoComments */);
                        statements.push(returnStatement);
                    }
                    else {
                        // Otherwise, we will use the following transformed representation for calls to `super()` in a constructor:
                        //
                        // ```
                        // // es2015 (source)
                        // class C extends Base {
                        //     constructor() {
                        //         super("foo");
                        //         this.x = 1;
                        //     }
                        // }
                        //
                        // // es5 (transformed)
                        // var C = (function (_super) {
                        //     function C() {
                        //         var _this = _super.call(this, "foo") || this;
                        //         _this.x = 1;
                        //         return _this;
                        //     }
                        //     return C;
                        // })(Base);
                        // ```
                        // Since the `super()` call was the first statement, we insert the `this` capturing call to
                        // `super()` at the top of the list of `statements` (after any pre-existing custom prologues).
                        insertCaptureThisForNode(statements, constructor, superCallExpression || createActualThis());
                        if (!isSufficientlyCoveredByReturnStatements(constructor.body)) {
                            statements.push(ts.createReturn(ts.createFileLevelUniqueName("_this")));
                        }
                    }
                }
                else {
                    // If a class is not derived from a base class or does not have a call to `super()`, `this` is only
                    // captured when necessitated by an arrow function capturing the lexical `this`:
                    //
                    // ```
                    // // es2015
                    // class C {}
                    //
                    // // es5
                    // var C = (function () {
                    //     function C() {
                    //     }
                    //     return C;
                    // })();
                    // ```
                    insertCaptureThisForNodeIfNeeded(prologue, constructor);
                }
                var block = ts.createBlock(ts.setTextRange(ts.createNodeArray(ts.concatenate(prologue, statements)), 
                /*location*/ constructor.body.statements), 
                /*multiLine*/ true);
                ts.setTextRange(block, constructor.body);
                return block;
            }
            /**
             * We want to try to avoid emitting a return statement in certain cases if a user already returned something.
             * It would generate obviously dead code, so we'll try to make things a little bit prettier
             * by doing a minimal check on whether some common patterns always explicitly return.
             */
            function isSufficientlyCoveredByReturnStatements(statement) {
                // A return statement is considered covered.
                if (statement.kind === 235 /* ReturnStatement */) {
                    return true;
                }
                // An if-statement with two covered branches is covered.
                else if (statement.kind === 227 /* IfStatement */) {
                    var ifStatement = statement;
                    if (ifStatement.elseStatement) {
                        return isSufficientlyCoveredByReturnStatements(ifStatement.thenStatement) &&
                            isSufficientlyCoveredByReturnStatements(ifStatement.elseStatement);
                    }
                }
                // A block is covered if it has a last statement which is covered.
                else if (statement.kind === 223 /* Block */) {
                    var lastStatement = ts.lastOrUndefined(statement.statements);
                    if (lastStatement && isSufficientlyCoveredByReturnStatements(lastStatement)) {
                        return true;
                    }
                }
                return false;
            }
            function createActualThis() {
                return ts.setEmitFlags(ts.createThis(), 4 /* NoSubstitution */);
            }
            function createDefaultSuperCallOrThis() {
                return ts.createLogicalOr(ts.createLogicalAnd(ts.createStrictInequality(ts.createFileLevelUniqueName("_super"), ts.createNull()), ts.createFunctionApply(ts.createFileLevelUniqueName("_super"), createActualThis(), ts.createIdentifier("arguments"))), createActualThis());
            }
            /**
             * Visits a parameter declaration.
             *
             * @param node A ParameterDeclaration node.
             */
            function visitParameter(node) {
                if (node.dotDotDotToken) {
                    // rest parameters are elided
                    return undefined;
                }
                else if (ts.isBindingPattern(node.name)) {
                    // Binding patterns are converted into a generated name and are
                    // evaluated inside the function body.
                    return ts.setOriginalNode(ts.setTextRange(ts.createParameter(
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, 
                    /*dotDotDotToken*/ undefined, ts.getGeneratedNameForNode(node), 
                    /*questionToken*/ undefined, 
                    /*type*/ undefined, 
                    /*initializer*/ undefined), 
                    /*location*/ node), 
                    /*original*/ node);
                }
                else if (node.initializer) {
                    // Initializers are elided
                    return ts.setOriginalNode(ts.setTextRange(ts.createParameter(
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, 
                    /*dotDotDotToken*/ undefined, node.name, 
                    /*questionToken*/ undefined, 
                    /*type*/ undefined, 
                    /*initializer*/ undefined), 
                    /*location*/ node), 
                    /*original*/ node);
                }
                else {
                    return node;
                }
            }
            function hasDefaultValueOrBindingPattern(node) {
                return node.initializer !== undefined
                    || ts.isBindingPattern(node.name);
            }
            /**
             * Adds statements to the body of a function-like node if it contains parameters with
             * binding patterns or initializers.
             *
             * @param statements The statements for the new function body.
             * @param node A function-like node.
             */
            function addDefaultValueAssignmentsIfNeeded(statements, node) {
                if (!ts.some(node.parameters, hasDefaultValueOrBindingPattern)) {
                    return false;
                }
                var added = false;
                for (var _i = 0, _a = node.parameters; _i < _a.length; _i++) {
                    var parameter = _a[_i];
                    var name = parameter.name, initializer = parameter.initializer, dotDotDotToken = parameter.dotDotDotToken;
                    // A rest parameter cannot have a binding pattern or an initializer,
                    // so let's just ignore it.
                    if (dotDotDotToken) {
                        continue;
                    }
                    if (ts.isBindingPattern(name)) {
                        added = insertDefaultValueAssignmentForBindingPattern(statements, parameter, name, initializer) || added;
                    }
                    else if (initializer) {
                        insertDefaultValueAssignmentForInitializer(statements, parameter, name, initializer);
                        added = true;
                    }
                }
                return added;
            }
            /**
             * Adds statements to the body of a function-like node for parameters with binding patterns
             *
             * @param statements The statements for the new function body.
             * @param parameter The parameter for the function.
             * @param name The name of the parameter.
             * @param initializer The initializer for the parameter.
             */
            function insertDefaultValueAssignmentForBindingPattern(statements, parameter, name, initializer) {
                // In cases where a binding pattern is simply '[]' or '{}',
                // we usually don't want to emit a var declaration; however, in the presence
                // of an initializer, we must emit that expression to preserve side effects.
                if (name.elements.length > 0) {
                    ts.insertStatementAfterCustomPrologue(statements, ts.setEmitFlags(ts.createVariableStatement(
                    /*modifiers*/ undefined, ts.createVariableDeclarationList(ts.flattenDestructuringBinding(parameter, visitor, context, 0 /* All */, ts.getGeneratedNameForNode(parameter)))), 1048576 /* CustomPrologue */));
                    return true;
                }
                else if (initializer) {
                    ts.insertStatementAfterCustomPrologue(statements, ts.setEmitFlags(ts.createExpressionStatement(ts.createAssignment(ts.getGeneratedNameForNode(parameter), ts.visitNode(initializer, visitor, ts.isExpression))), 1048576 /* CustomPrologue */));
                    return true;
                }
                return false;
            }
            /**
             * Adds statements to the body of a function-like node for parameters with initializers.
             *
             * @param statements The statements for the new function body.
             * @param parameter The parameter for the function.
             * @param name The name of the parameter.
             * @param initializer The initializer for the parameter.
             */
            function insertDefaultValueAssignmentForInitializer(statements, parameter, name, initializer) {
                initializer = ts.visitNode(initializer, visitor, ts.isExpression);
                var statement = ts.createIf(ts.createTypeCheck(ts.getSynthesizedClone(name), "undefined"), ts.setEmitFlags(ts.setTextRange(ts.createBlock([
                    ts.createExpressionStatement(ts.setEmitFlags(ts.setTextRange(ts.createAssignment(ts.setEmitFlags(ts.getMutableClone(name), 48 /* NoSourceMap */), ts.setEmitFlags(initializer, 48 /* NoSourceMap */ | ts.getEmitFlags(initializer) | 1536 /* NoComments */)), parameter), 1536 /* NoComments */))
                ]), parameter), 1 /* SingleLine */ | 32 /* NoTrailingSourceMap */ | 384 /* NoTokenSourceMaps */ | 1536 /* NoComments */));
                ts.startOnNewLine(statement);
                ts.setTextRange(statement, parameter);
                ts.setEmitFlags(statement, 384 /* NoTokenSourceMaps */ | 32 /* NoTrailingSourceMap */ | 1048576 /* CustomPrologue */ | 1536 /* NoComments */);
                ts.insertStatementAfterCustomPrologue(statements, statement);
            }
            /**
             * Gets a value indicating whether we need to add statements to handle a rest parameter.
             *
             * @param node A ParameterDeclaration node.
             * @param inConstructorWithSynthesizedSuper A value indicating whether the parameter is
             *                                          part of a constructor declaration with a
             *                                          synthesized call to `super`
             */
            function shouldAddRestParameter(node, inConstructorWithSynthesizedSuper) {
                return !!(node && node.dotDotDotToken && !inConstructorWithSynthesizedSuper);
            }
            /**
             * Adds statements to the body of a function-like node if it contains a rest parameter.
             *
             * @param statements The statements for the new function body.
             * @param node A function-like node.
             * @param inConstructorWithSynthesizedSuper A value indicating whether the parameter is
             *                                          part of a constructor declaration with a
             *                                          synthesized call to `super`
             */
            function addRestParameterIfNeeded(statements, node, inConstructorWithSynthesizedSuper) {
                var prologueStatements = [];
                var parameter = ts.lastOrUndefined(node.parameters);
                if (!shouldAddRestParameter(parameter, inConstructorWithSynthesizedSuper)) {
                    return false;
                }
                // `declarationName` is the name of the local declaration for the parameter.
                var declarationName = parameter.name.kind === 75 /* Identifier */ ? ts.getMutableClone(parameter.name) : ts.createTempVariable(/*recordTempVariable*/ undefined);
                ts.setEmitFlags(declarationName, 48 /* NoSourceMap */);
                // `expressionName` is the name of the parameter used in expressions.
                var expressionName = parameter.name.kind === 75 /* Identifier */ ? ts.getSynthesizedClone(parameter.name) : declarationName;
                var restIndex = node.parameters.length - 1;
                var temp = ts.createLoopVariable();
                // var param = [];
                prologueStatements.push(ts.setEmitFlags(ts.setTextRange(ts.createVariableStatement(
                /*modifiers*/ undefined, ts.createVariableDeclarationList([
                    ts.createVariableDeclaration(declarationName, 
                    /*type*/ undefined, ts.createArrayLiteral([]))
                ])), 
                /*location*/ parameter), 1048576 /* CustomPrologue */));
                // for (var _i = restIndex; _i < arguments.length; _i++) {
                //   param[_i - restIndex] = arguments[_i];
                // }
                var forStatement = ts.createFor(ts.setTextRange(ts.createVariableDeclarationList([
                    ts.createVariableDeclaration(temp, /*type*/ undefined, ts.createLiteral(restIndex))
                ]), parameter), ts.setTextRange(ts.createLessThan(temp, ts.createPropertyAccess(ts.createIdentifier("arguments"), "length")), parameter), ts.setTextRange(ts.createPostfixIncrement(temp), parameter), ts.createBlock([
                    ts.startOnNewLine(ts.setTextRange(ts.createExpressionStatement(ts.createAssignment(ts.createElementAccess(expressionName, restIndex === 0
                        ? temp
                        : ts.createSubtract(temp, ts.createLiteral(restIndex))), ts.createElementAccess(ts.createIdentifier("arguments"), temp))), 
                    /*location*/ parameter))
                ]));
                ts.setEmitFlags(forStatement, 1048576 /* CustomPrologue */);
                ts.startOnNewLine(forStatement);
                prologueStatements.push(forStatement);
                if (parameter.name.kind !== 75 /* Identifier */) {
                    // do the actual destructuring of the rest parameter if necessary
                    prologueStatements.push(ts.setEmitFlags(ts.setTextRange(ts.createVariableStatement(
                    /*modifiers*/ undefined, ts.createVariableDeclarationList(ts.flattenDestructuringBinding(parameter, visitor, context, 0 /* All */, expressionName))), parameter), 1048576 /* CustomPrologue */));
                }
                ts.insertStatementsAfterCustomPrologue(statements, prologueStatements);
                return true;
            }
            /**
             * Adds a statement to capture the `this` of a function declaration if it is needed.
             * NOTE: This must be executed *after* the subtree has been visited.
             *
             * @param statements The statements for the new function body.
             * @param node A node.
             */
            function insertCaptureThisForNodeIfNeeded(statements, node) {
                if (hierarchyFacts & 32768 /* CapturedLexicalThis */ && node.kind !== 202 /* ArrowFunction */) {
                    insertCaptureThisForNode(statements, node, ts.createThis());
                    return true;
                }
                return false;
            }
            function insertCaptureThisForNode(statements, node, initializer) {
                enableSubstitutionsForCapturedThis();
                var captureThisStatement = ts.createVariableStatement(
                /*modifiers*/ undefined, ts.createVariableDeclarationList([
                    ts.createVariableDeclaration(ts.createFileLevelUniqueName("_this"), 
                    /*type*/ undefined, initializer)
                ]));
                ts.setEmitFlags(captureThisStatement, 1536 /* NoComments */ | 1048576 /* CustomPrologue */);
                ts.setSourceMapRange(captureThisStatement, node);
                ts.insertStatementAfterCustomPrologue(statements, captureThisStatement);
            }
            function insertCaptureNewTargetIfNeeded(statements, node, copyOnWrite) {
                if (hierarchyFacts & 16384 /* NewTarget */) {
                    var newTarget = void 0;
                    switch (node.kind) {
                        case 202 /* ArrowFunction */:
                            return statements;
                        case 161 /* MethodDeclaration */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            // Methods and accessors cannot be constructors, so 'new.target' will
                            // always return 'undefined'.
                            newTarget = ts.createVoidZero();
                            break;
                        case 162 /* Constructor */:
                            // Class constructors can only be called with `new`, so `this.constructor`
                            // should be relatively safe to use.
                            newTarget = ts.createPropertyAccess(ts.setEmitFlags(ts.createThis(), 4 /* NoSubstitution */), "constructor");
                            break;
                        case 244 /* FunctionDeclaration */:
                        case 201 /* FunctionExpression */:
                            // Functions can be called or constructed, and may have a `this` due to
                            // being a member or when calling an imported function via `other_1.f()`.
                            newTarget = ts.createConditional(ts.createLogicalAnd(ts.setEmitFlags(ts.createThis(), 4 /* NoSubstitution */), ts.createBinary(ts.setEmitFlags(ts.createThis(), 4 /* NoSubstitution */), 98 /* InstanceOfKeyword */, ts.getLocalName(node))), ts.createPropertyAccess(ts.setEmitFlags(ts.createThis(), 4 /* NoSubstitution */), "constructor"), ts.createVoidZero());
                            break;
                        default:
                            return ts.Debug.failBadSyntaxKind(node);
                    }
                    var captureNewTargetStatement = ts.createVariableStatement(
                    /*modifiers*/ undefined, ts.createVariableDeclarationList([
                        ts.createVariableDeclaration(ts.createFileLevelUniqueName("_newTarget"), 
                        /*type*/ undefined, newTarget)
                    ]));
                    ts.setEmitFlags(captureNewTargetStatement, 1536 /* NoComments */ | 1048576 /* CustomPrologue */);
                    if (copyOnWrite) {
                        statements = statements.slice();
                    }
                    ts.insertStatementAfterCustomPrologue(statements, captureNewTargetStatement);
                }
                return statements;
            }
            /**
             * Adds statements to the class body function for a class to define the members of the
             * class.
             *
             * @param statements The statements for the class body function.
             * @param node The ClassExpression or ClassDeclaration node.
             */
            function addClassMembers(statements, node) {
                for (var _i = 0, _a = node.members; _i < _a.length; _i++) {
                    var member = _a[_i];
                    switch (member.kind) {
                        case 222 /* SemicolonClassElement */:
                            statements.push(transformSemicolonClassElementToStatement(member));
                            break;
                        case 161 /* MethodDeclaration */:
                            statements.push(transformClassMethodDeclarationToStatement(getClassMemberPrefix(node, member), member, node));
                            break;
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            var accessors = ts.getAllAccessorDeclarations(node.members, member);
                            if (member === accessors.firstAccessor) {
                                statements.push(transformAccessorsToStatement(getClassMemberPrefix(node, member), accessors, node));
                            }
                            break;
                        case 162 /* Constructor */:
                            // Constructors are handled in visitClassExpression/visitClassDeclaration
                            break;
                        default:
                            ts.Debug.failBadSyntaxKind(member, currentSourceFile && currentSourceFile.fileName);
                            break;
                    }
                }
            }
            /**
             * Transforms a SemicolonClassElement into a statement for a class body function.
             *
             * @param member The SemicolonClassElement node.
             */
            function transformSemicolonClassElementToStatement(member) {
                return ts.setTextRange(ts.createEmptyStatement(), member);
            }
            /**
             * Transforms a MethodDeclaration into a statement for a class body function.
             *
             * @param receiver The receiver for the member.
             * @param member The MethodDeclaration node.
             */
            function transformClassMethodDeclarationToStatement(receiver, member, container) {
                var commentRange = ts.getCommentRange(member);
                var sourceMapRange = ts.getSourceMapRange(member);
                var memberFunction = transformFunctionLikeToExpression(member, /*location*/ member, /*name*/ undefined, container);
                var propertyName = ts.visitNode(member.name, visitor, ts.isPropertyName);
                var e;
                if (!ts.isPrivateIdentifier(propertyName) && context.getCompilerOptions().useDefineForClassFields) {
                    var name = ts.isComputedPropertyName(propertyName) ? propertyName.expression
                        : ts.isIdentifier(propertyName) ? ts.createStringLiteral(ts.unescapeLeadingUnderscores(propertyName.escapedText))
                            : propertyName;
                    e = ts.createObjectDefinePropertyCall(receiver, name, ts.createPropertyDescriptor({ value: memberFunction, enumerable: false, writable: true, configurable: true }));
                }
                else {
                    var memberName = ts.createMemberAccessForPropertyName(receiver, propertyName, /*location*/ member.name);
                    e = ts.createAssignment(memberName, memberFunction);
                }
                ts.setEmitFlags(memberFunction, 1536 /* NoComments */);
                ts.setSourceMapRange(memberFunction, sourceMapRange);
                var statement = ts.setTextRange(ts.createExpressionStatement(e), /*location*/ member);
                ts.setOriginalNode(statement, member);
                ts.setCommentRange(statement, commentRange);
                // The location for the statement is used to emit comments only.
                // No source map should be emitted for this statement to align with the
                // old emitter.
                ts.setEmitFlags(statement, 48 /* NoSourceMap */);
                return statement;
            }
            /**
             * Transforms a set of related of get/set accessors into a statement for a class body function.
             *
             * @param receiver The receiver for the member.
             * @param accessors The set of related get/set accessors.
             */
            function transformAccessorsToStatement(receiver, accessors, container) {
                var statement = ts.createExpressionStatement(transformAccessorsToExpression(receiver, accessors, container, /*startsOnNewLine*/ false));
                // The location for the statement is used to emit source maps only.
                // No comments should be emitted for this statement to align with the
                // old emitter.
                ts.setEmitFlags(statement, 1536 /* NoComments */);
                ts.setSourceMapRange(statement, ts.getSourceMapRange(accessors.firstAccessor));
                return statement;
            }
            /**
             * Transforms a set of related get/set accessors into an expression for either a class
             * body function or an ObjectLiteralExpression with computed properties.
             *
             * @param receiver The receiver for the member.
             */
            function transformAccessorsToExpression(receiver, _a, container, startsOnNewLine) {
                var firstAccessor = _a.firstAccessor, getAccessor = _a.getAccessor, setAccessor = _a.setAccessor;
                // To align with source maps in the old emitter, the receiver and property name
                // arguments are both mapped contiguously to the accessor name.
                var target = ts.getMutableClone(receiver);
                ts.setEmitFlags(target, 1536 /* NoComments */ | 32 /* NoTrailingSourceMap */);
                ts.setSourceMapRange(target, firstAccessor.name);
                var visitedAccessorName = ts.visitNode(firstAccessor.name, visitor, ts.isPropertyName);
                if (ts.isPrivateIdentifier(visitedAccessorName)) {
                    return ts.Debug.failBadSyntaxKind(visitedAccessorName, "Encountered unhandled private identifier while transforming ES2015.");
                }
                var propertyName = ts.createExpressionForPropertyName(visitedAccessorName);
                ts.setEmitFlags(propertyName, 1536 /* NoComments */ | 16 /* NoLeadingSourceMap */);
                ts.setSourceMapRange(propertyName, firstAccessor.name);
                var properties = [];
                if (getAccessor) {
                    var getterFunction = transformFunctionLikeToExpression(getAccessor, /*location*/ undefined, /*name*/ undefined, container);
                    ts.setSourceMapRange(getterFunction, ts.getSourceMapRange(getAccessor));
                    ts.setEmitFlags(getterFunction, 512 /* NoLeadingComments */);
                    var getter = ts.createPropertyAssignment("get", getterFunction);
                    ts.setCommentRange(getter, ts.getCommentRange(getAccessor));
                    properties.push(getter);
                }
                if (setAccessor) {
                    var setterFunction = transformFunctionLikeToExpression(setAccessor, /*location*/ undefined, /*name*/ undefined, container);
                    ts.setSourceMapRange(setterFunction, ts.getSourceMapRange(setAccessor));
                    ts.setEmitFlags(setterFunction, 512 /* NoLeadingComments */);
                    var setter = ts.createPropertyAssignment("set", setterFunction);
                    ts.setCommentRange(setter, ts.getCommentRange(setAccessor));
                    properties.push(setter);
                }
                properties.push(ts.createPropertyAssignment("enumerable", ts.createTrue()), ts.createPropertyAssignment("configurable", ts.createTrue()));
                var call = ts.createCall(ts.createPropertyAccess(ts.createIdentifier("Object"), "defineProperty"), 
                /*typeArguments*/ undefined, [
                    target,
                    propertyName,
                    ts.createObjectLiteral(properties, /*multiLine*/ true)
                ]);
                if (startsOnNewLine) {
                    ts.startOnNewLine(call);
                }
                return call;
            }
            /**
             * Visits an ArrowFunction and transforms it into a FunctionExpression.
             *
             * @param node An ArrowFunction node.
             */
            function visitArrowFunction(node) {
                if (node.transformFlags & 4096 /* ContainsLexicalThis */) {
                    hierarchyFacts |= 32768 /* CapturedLexicalThis */;
                }
                var savedConvertedLoopState = convertedLoopState;
                convertedLoopState = undefined;
                var ancestorFacts = enterSubtree(15232 /* ArrowFunctionExcludes */, 66 /* ArrowFunctionIncludes */);
                var func = ts.createFunctionExpression(
                /*modifiers*/ undefined, 
                /*asteriskToken*/ undefined, 
                /*name*/ undefined, 
                /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                /*type*/ undefined, transformFunctionBody(node));
                ts.setTextRange(func, node);
                ts.setOriginalNode(func, node);
                ts.setEmitFlags(func, 8 /* CapturesThis */);
                if (hierarchyFacts & 32768 /* CapturedLexicalThis */) {
                    enableSubstitutionsForCapturedThis();
                }
                // If an arrow function contains
                exitSubtree(ancestorFacts, 0 /* ArrowFunctionSubtreeExcludes */, 0 /* None */);
                convertedLoopState = savedConvertedLoopState;
                return func;
            }
            /**
             * Visits a FunctionExpression node.
             *
             * @param node a FunctionExpression node.
             */
            function visitFunctionExpression(node) {
                var ancestorFacts = ts.getEmitFlags(node) & 262144 /* AsyncFunctionBody */
                    ? enterSubtree(16278 /* AsyncFunctionBodyExcludes */, 69 /* AsyncFunctionBodyIncludes */)
                    : enterSubtree(16286 /* FunctionExcludes */, 65 /* FunctionIncludes */);
                var savedConvertedLoopState = convertedLoopState;
                convertedLoopState = undefined;
                var parameters = ts.visitParameterList(node.parameters, visitor, context);
                var body = transformFunctionBody(node);
                var name = hierarchyFacts & 16384 /* NewTarget */
                    ? ts.getLocalName(node)
                    : node.name;
                exitSubtree(ancestorFacts, 49152 /* FunctionSubtreeExcludes */, 0 /* None */);
                convertedLoopState = savedConvertedLoopState;
                return ts.updateFunctionExpression(node, 
                /*modifiers*/ undefined, node.asteriskToken, name, 
                /*typeParameters*/ undefined, parameters, 
                /*type*/ undefined, body);
            }
            /**
             * Visits a FunctionDeclaration node.
             *
             * @param node a FunctionDeclaration node.
             */
            function visitFunctionDeclaration(node) {
                var savedConvertedLoopState = convertedLoopState;
                convertedLoopState = undefined;
                var ancestorFacts = enterSubtree(16286 /* FunctionExcludes */, 65 /* FunctionIncludes */);
                var parameters = ts.visitParameterList(node.parameters, visitor, context);
                var body = transformFunctionBody(node);
                var name = hierarchyFacts & 16384 /* NewTarget */
                    ? ts.getLocalName(node)
                    : node.name;
                exitSubtree(ancestorFacts, 49152 /* FunctionSubtreeExcludes */, 0 /* None */);
                convertedLoopState = savedConvertedLoopState;
                return ts.updateFunctionDeclaration(node, 
                /*decorators*/ undefined, ts.visitNodes(node.modifiers, visitor, ts.isModifier), node.asteriskToken, name, 
                /*typeParameters*/ undefined, parameters, 
                /*type*/ undefined, body);
            }
            /**
             * Transforms a function-like node into a FunctionExpression.
             *
             * @param node The function-like node to transform.
             * @param location The source-map location for the new FunctionExpression.
             * @param name The name of the new FunctionExpression.
             */
            function transformFunctionLikeToExpression(node, location, name, container) {
                var savedConvertedLoopState = convertedLoopState;
                convertedLoopState = undefined;
                var ancestorFacts = container && ts.isClassLike(container) && !ts.hasModifier(node, 32 /* Static */)
                    ? enterSubtree(16286 /* FunctionExcludes */, 65 /* FunctionIncludes */ | 8 /* NonStaticClassElement */)
                    : enterSubtree(16286 /* FunctionExcludes */, 65 /* FunctionIncludes */);
                var parameters = ts.visitParameterList(node.parameters, visitor, context);
                var body = transformFunctionBody(node);
                if (hierarchyFacts & 16384 /* NewTarget */ && !name && (node.kind === 244 /* FunctionDeclaration */ || node.kind === 201 /* FunctionExpression */)) {
                    name = ts.getGeneratedNameForNode(node);
                }
                exitSubtree(ancestorFacts, 49152 /* FunctionSubtreeExcludes */, 0 /* None */);
                convertedLoopState = savedConvertedLoopState;
                return ts.setOriginalNode(ts.setTextRange(ts.createFunctionExpression(
                /*modifiers*/ undefined, node.asteriskToken, name, 
                /*typeParameters*/ undefined, parameters, 
                /*type*/ undefined, body), location), 
                /*original*/ node);
            }
            /**
             * Transforms the body of a function-like node.
             *
             * @param node A function-like node.
             */
            function transformFunctionBody(node) {
                var multiLine = false; // indicates whether the block *must* be emitted as multiple lines
                var singleLine = false; // indicates whether the block *may* be emitted as a single line
                var statementsLocation;
                var closeBraceLocation;
                var prologue = [];
                var statements = [];
                var body = node.body;
                var statementOffset;
                resumeLexicalEnvironment();
                if (ts.isBlock(body)) {
                    // ensureUseStrict is false because no new prologue-directive should be added.
                    // addStandardPrologue will put already-existing directives at the beginning of the target statement-array
                    statementOffset = ts.addStandardPrologue(prologue, body.statements, /*ensureUseStrict*/ false);
                }
                multiLine = addDefaultValueAssignmentsIfNeeded(statements, node) || multiLine;
                multiLine = addRestParameterIfNeeded(statements, node, /*inConstructorWithSynthesizedSuper*/ false) || multiLine;
                if (ts.isBlock(body)) {
                    // addCustomPrologue puts already-existing directives at the beginning of the target statement-array
                    statementOffset = ts.addCustomPrologue(statements, body.statements, statementOffset, visitor);
                    statementsLocation = body.statements;
                    ts.addRange(statements, ts.visitNodes(body.statements, visitor, ts.isStatement, statementOffset));
                    // If the original body was a multi-line block, this must be a multi-line block.
                    if (!multiLine && body.multiLine) {
                        multiLine = true;
                    }
                }
                else {
                    ts.Debug.assert(node.kind === 202 /* ArrowFunction */);
                    // To align with the old emitter, we use a synthetic end position on the location
                    // for the statement list we synthesize when we down-level an arrow function with
                    // an expression function body. This prevents both comments and source maps from
                    // being emitted for the end position only.
                    statementsLocation = ts.moveRangeEnd(body, -1);
                    var equalsGreaterThanToken = node.equalsGreaterThanToken;
                    if (!ts.nodeIsSynthesized(equalsGreaterThanToken) && !ts.nodeIsSynthesized(body)) {
                        if (ts.rangeEndIsOnSameLineAsRangeStart(equalsGreaterThanToken, body, currentSourceFile)) {
                            singleLine = true;
                        }
                        else {
                            multiLine = true;
                        }
                    }
                    var expression = ts.visitNode(body, visitor, ts.isExpression);
                    var returnStatement = ts.createReturn(expression);
                    ts.setTextRange(returnStatement, body);
                    ts.moveSyntheticComments(returnStatement, body);
                    ts.setEmitFlags(returnStatement, 384 /* NoTokenSourceMaps */ | 32 /* NoTrailingSourceMap */ | 1024 /* NoTrailingComments */);
                    statements.push(returnStatement);
                    // To align with the source map emit for the old emitter, we set a custom
                    // source map location for the close brace.
                    closeBraceLocation = body;
                }
                ts.mergeLexicalEnvironment(prologue, endLexicalEnvironment());
                insertCaptureNewTargetIfNeeded(prologue, node, /*copyOnWrite*/ false);
                insertCaptureThisForNodeIfNeeded(prologue, node);
                // If we added any final generated statements, this must be a multi-line block
                if (ts.some(prologue)) {
                    multiLine = true;
                }
                statements.unshift.apply(statements, prologue);
                if (ts.isBlock(body) && ts.arrayIsEqualTo(statements, body.statements)) {
                    // no changes were made, preserve the tree
                    return body;
                }
                var block = ts.createBlock(ts.setTextRange(ts.createNodeArray(statements), statementsLocation), multiLine);
                ts.setTextRange(block, node.body);
                if (!multiLine && singleLine) {
                    ts.setEmitFlags(block, 1 /* SingleLine */);
                }
                if (closeBraceLocation) {
                    ts.setTokenSourceMapRange(block, 19 /* CloseBraceToken */, closeBraceLocation);
                }
                ts.setOriginalNode(block, node.body);
                return block;
            }
            function visitBlock(node, isFunctionBody) {
                if (isFunctionBody) {
                    // A function body is not a block scope.
                    return ts.visitEachChild(node, visitor, context);
                }
                var ancestorFacts = hierarchyFacts & 256 /* IterationStatement */
                    ? enterSubtree(7104 /* IterationStatementBlockExcludes */, 512 /* IterationStatementBlockIncludes */)
                    : enterSubtree(6976 /* BlockExcludes */, 128 /* BlockIncludes */);
                var updated = ts.visitEachChild(node, visitor, context);
                exitSubtree(ancestorFacts, 0 /* None */, 0 /* None */);
                return updated;
            }
            /**
             * Visits an ExpressionStatement that contains a destructuring assignment.
             *
             * @param node An ExpressionStatement node.
             */
            function visitExpressionStatement(node) {
                // If we are here it is most likely because our expression is a destructuring assignment.
                switch (node.expression.kind) {
                    case 200 /* ParenthesizedExpression */:
                        return ts.updateExpressionStatement(node, visitParenthesizedExpression(node.expression, /*needsDestructuringValue*/ false));
                    case 209 /* BinaryExpression */:
                        return ts.updateExpressionStatement(node, visitBinaryExpression(node.expression, /*needsDestructuringValue*/ false));
                }
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Visits a ParenthesizedExpression that may contain a destructuring assignment.
             *
             * @param node A ParenthesizedExpression node.
             * @param needsDestructuringValue A value indicating whether we need to hold onto the rhs
             *                                of a destructuring assignment.
             */
            function visitParenthesizedExpression(node, needsDestructuringValue) {
                // If we are here it is most likely because our expression is a destructuring assignment.
                if (!needsDestructuringValue) {
                    // By default we always emit the RHS at the end of a flattened destructuring
                    // expression. If we are in a state where we do not need the destructuring value,
                    // we pass that information along to the children that care about it.
                    switch (node.expression.kind) {
                        case 200 /* ParenthesizedExpression */:
                            return ts.updateParen(node, visitParenthesizedExpression(node.expression, /*needsDestructuringValue*/ false));
                        case 209 /* BinaryExpression */:
                            return ts.updateParen(node, visitBinaryExpression(node.expression, /*needsDestructuringValue*/ false));
                    }
                }
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Visits a BinaryExpression that contains a destructuring assignment.
             *
             * @param node A BinaryExpression node.
             * @param needsDestructuringValue A value indicating whether we need to hold onto the rhs
             *                                of a destructuring assignment.
             */
            function visitBinaryExpression(node, needsDestructuringValue) {
                // If we are here it is because this is a destructuring assignment.
                if (ts.isDestructuringAssignment(node)) {
                    return ts.flattenDestructuringAssignment(node, visitor, context, 0 /* All */, needsDestructuringValue);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitVariableStatement(node) {
                var ancestorFacts = enterSubtree(0 /* None */, ts.hasModifier(node, 1 /* Export */) ? 32 /* ExportedVariableStatement */ : 0 /* None */);
                var updated;
                if (convertedLoopState && (node.declarationList.flags & 3 /* BlockScoped */) === 0) {
                    // we are inside a converted loop - hoist variable declarations
                    var assignments = void 0;
                    for (var _i = 0, _a = node.declarationList.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        hoistVariableDeclarationDeclaredInConvertedLoop(convertedLoopState, decl);
                        if (decl.initializer) {
                            var assignment = void 0;
                            if (ts.isBindingPattern(decl.name)) {
                                assignment = ts.flattenDestructuringAssignment(decl, visitor, context, 0 /* All */);
                            }
                            else {
                                assignment = ts.createBinary(decl.name, 62 /* EqualsToken */, ts.visitNode(decl.initializer, visitor, ts.isExpression));
                                ts.setTextRange(assignment, decl);
                            }
                            assignments = ts.append(assignments, assignment);
                        }
                    }
                    if (assignments) {
                        updated = ts.setTextRange(ts.createExpressionStatement(ts.inlineExpressions(assignments)), node);
                    }
                    else {
                        // none of declarations has initializer - the entire variable statement can be deleted
                        updated = undefined;
                    }
                }
                else {
                    updated = ts.visitEachChild(node, visitor, context);
                }
                exitSubtree(ancestorFacts, 0 /* None */, 0 /* None */);
                return updated;
            }
            /**
             * Visits a VariableDeclarationList that is block scoped (e.g. `let` or `const`).
             *
             * @param node A VariableDeclarationList node.
             */
            function visitVariableDeclarationList(node) {
                if (node.flags & 3 /* BlockScoped */ || node.transformFlags & 131072 /* ContainsBindingPattern */) {
                    if (node.flags & 3 /* BlockScoped */) {
                        enableSubstitutionsForBlockScopedBindings();
                    }
                    var declarations = ts.flatMap(node.declarations, node.flags & 1 /* Let */
                        ? visitVariableDeclarationInLetDeclarationList
                        : visitVariableDeclaration);
                    var declarationList = ts.createVariableDeclarationList(declarations);
                    ts.setOriginalNode(declarationList, node);
                    ts.setTextRange(declarationList, node);
                    ts.setCommentRange(declarationList, node);
                    // If the first or last declaration is a binding pattern, we need to modify
                    // the source map range for the declaration list.
                    if (node.transformFlags & 131072 /* ContainsBindingPattern */
                        && (ts.isBindingPattern(node.declarations[0].name) || ts.isBindingPattern(ts.last(node.declarations).name))) {
                        ts.setSourceMapRange(declarationList, getRangeUnion(declarations));
                    }
                    return declarationList;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function getRangeUnion(declarations) {
                // declarations may not be sorted by position.
                // pos should be the minimum* position over all nodes (that's not -1), end should be the maximum end over all nodes.
                var pos = -1, end = -1;
                for (var _i = 0, declarations_10 = declarations; _i < declarations_10.length; _i++) {
                    var node = declarations_10[_i];
                    pos = pos === -1 ? node.pos : node.pos === -1 ? pos : Math.min(pos, node.pos);
                    end = Math.max(end, node.end);
                }
                return ts.createRange(pos, end);
            }
            /**
             * Gets a value indicating whether we should emit an explicit initializer for a variable
             * declaration in a `let` declaration list.
             *
             * @param node A VariableDeclaration node.
             */
            function shouldEmitExplicitInitializerForLetDeclaration(node) {
                // Nested let bindings might need to be initialized explicitly to preserve
                // ES6 semantic:
                //
                //  { let x = 1; }
                //  { let x; } // x here should be undefined. not 1
                //
                // Top level bindings never collide with anything and thus don't require
                // explicit initialization. As for nested let bindings there are two cases:
                //
                // - Nested let bindings that were not renamed definitely should be
                //   initialized explicitly:
                //
                //    { let x = 1; }
                //    { let x; if (some-condition) { x = 1}; if (x) { /*1*/ } }
                //
                //   Without explicit initialization code in /*1*/ can be executed even if
                //   some-condition is evaluated to false.
                //
                // - Renaming introduces fresh name that should not collide with any
                //   existing names, however renamed bindings sometimes also should be
                //   explicitly initialized. One particular case: non-captured binding
                //   declared inside loop body (but not in loop initializer):
                //
                //    let x;
                //    for (;;) {
                //        let x;
                //    }
                //
                //   In downlevel codegen inner 'x' will be renamed so it won't collide
                //   with outer 'x' however it will should be reset on every iteration as
                //   if it was declared anew.
                //
                //   * Why non-captured binding?
                //     - Because if loop contains block scoped binding captured in some
                //       function then loop body will be rewritten to have a fresh scope
                //       on every iteration so everything will just work.
                //
                //   * Why loop initializer is excluded?
                //     - Since we've introduced a fresh name it already will be undefined.
                var flags = resolver.getNodeCheckFlags(node);
                var isCapturedInFunction = flags & 262144 /* CapturedBlockScopedBinding */;
                var isDeclaredInLoop = flags & 524288 /* BlockScopedBindingInLoop */;
                var emittedAsTopLevel = (hierarchyFacts & 64 /* TopLevel */) !== 0
                    || (isCapturedInFunction
                        && isDeclaredInLoop
                        && (hierarchyFacts & 512 /* IterationStatementBlock */) !== 0);
                var emitExplicitInitializer = !emittedAsTopLevel
                    && (hierarchyFacts & 4096 /* ForInOrForOfStatement */) === 0
                    && (!resolver.isDeclarationWithCollidingName(node)
                        || (isDeclaredInLoop
                            && !isCapturedInFunction
                            && (hierarchyFacts & (2048 /* ForStatement */ | 4096 /* ForInOrForOfStatement */)) === 0));
                return emitExplicitInitializer;
            }
            /**
             * Visits a VariableDeclaration in a `let` declaration list.
             *
             * @param node A VariableDeclaration node.
             */
            function visitVariableDeclarationInLetDeclarationList(node) {
                // For binding pattern names that lack initializers there is no point to emit
                // explicit initializer since downlevel codegen for destructuring will fail
                // in the absence of initializer so all binding elements will say uninitialized
                var name = node.name;
                if (ts.isBindingPattern(name)) {
                    return visitVariableDeclaration(node);
                }
                if (!node.initializer && shouldEmitExplicitInitializerForLetDeclaration(node)) {
                    var clone_3 = ts.getMutableClone(node);
                    clone_3.initializer = ts.createVoidZero();
                    return clone_3;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Visits a VariableDeclaration node with a binding pattern.
             *
             * @param node A VariableDeclaration node.
             */
            function visitVariableDeclaration(node) {
                var ancestorFacts = enterSubtree(32 /* ExportedVariableStatement */, 0 /* None */);
                var updated;
                if (ts.isBindingPattern(node.name)) {
                    updated = ts.flattenDestructuringBinding(node, visitor, context, 0 /* All */, 
                    /*value*/ undefined, (ancestorFacts & 32 /* ExportedVariableStatement */) !== 0);
                }
                else {
                    updated = ts.visitEachChild(node, visitor, context);
                }
                exitSubtree(ancestorFacts, 0 /* None */, 0 /* None */);
                return updated;
            }
            function recordLabel(node) {
                convertedLoopState.labels.set(ts.idText(node.label), true);
            }
            function resetLabel(node) {
                convertedLoopState.labels.set(ts.idText(node.label), false);
            }
            function visitLabeledStatement(node) {
                if (convertedLoopState && !convertedLoopState.labels) {
                    convertedLoopState.labels = ts.createMap();
                }
                var statement = ts.unwrapInnermostStatementOfLabel(node, convertedLoopState && recordLabel);
                return ts.isIterationStatement(statement, /*lookInLabeledStatements*/ false)
                    ? visitIterationStatement(statement, /*outermostLabeledStatement*/ node)
                    : ts.restoreEnclosingLabel(ts.visitNode(statement, visitor, ts.isStatement, ts.liftToBlock), node, convertedLoopState && resetLabel);
            }
            function visitIterationStatement(node, outermostLabeledStatement) {
                switch (node.kind) {
                    case 228 /* DoStatement */:
                    case 229 /* WhileStatement */:
                        return visitDoOrWhileStatement(node, outermostLabeledStatement);
                    case 230 /* ForStatement */:
                        return visitForStatement(node, outermostLabeledStatement);
                    case 231 /* ForInStatement */:
                        return visitForInStatement(node, outermostLabeledStatement);
                    case 232 /* ForOfStatement */:
                        return visitForOfStatement(node, outermostLabeledStatement);
                }
            }
            function visitIterationStatementWithFacts(excludeFacts, includeFacts, node, outermostLabeledStatement, convert) {
                var ancestorFacts = enterSubtree(excludeFacts, includeFacts);
                var updated = convertIterationStatementBodyIfNecessary(node, outermostLabeledStatement, ancestorFacts, convert);
                exitSubtree(ancestorFacts, 0 /* None */, 0 /* None */);
                return updated;
            }
            function visitDoOrWhileStatement(node, outermostLabeledStatement) {
                return visitIterationStatementWithFacts(0 /* DoOrWhileStatementExcludes */, 1280 /* DoOrWhileStatementIncludes */, node, outermostLabeledStatement);
            }
            function visitForStatement(node, outermostLabeledStatement) {
                return visitIterationStatementWithFacts(5056 /* ForStatementExcludes */, 3328 /* ForStatementIncludes */, node, outermostLabeledStatement);
            }
            function visitForInStatement(node, outermostLabeledStatement) {
                return visitIterationStatementWithFacts(3008 /* ForInOrForOfStatementExcludes */, 5376 /* ForInOrForOfStatementIncludes */, node, outermostLabeledStatement);
            }
            function visitForOfStatement(node, outermostLabeledStatement) {
                return visitIterationStatementWithFacts(3008 /* ForInOrForOfStatementExcludes */, 5376 /* ForInOrForOfStatementIncludes */, node, outermostLabeledStatement, compilerOptions.downlevelIteration ? convertForOfStatementForIterable : convertForOfStatementForArray);
            }
            function convertForOfStatementHead(node, boundValue, convertedLoopBodyStatements) {
                var statements = [];
                var initializer = node.initializer;
                if (ts.isVariableDeclarationList(initializer)) {
                    if (node.initializer.flags & 3 /* BlockScoped */) {
                        enableSubstitutionsForBlockScopedBindings();
                    }
                    var firstOriginalDeclaration = ts.firstOrUndefined(initializer.declarations);
                    if (firstOriginalDeclaration && ts.isBindingPattern(firstOriginalDeclaration.name)) {
                        // This works whether the declaration is a var, let, or const.
                        // It will use rhsIterationValue _a[_i] as the initializer.
                        var declarations = ts.flattenDestructuringBinding(firstOriginalDeclaration, visitor, context, 0 /* All */, boundValue);
                        var declarationList = ts.setTextRange(ts.createVariableDeclarationList(declarations), node.initializer);
                        ts.setOriginalNode(declarationList, node.initializer);
                        // Adjust the source map range for the first declaration to align with the old
                        // emitter.
                        ts.setSourceMapRange(declarationList, ts.createRange(declarations[0].pos, ts.last(declarations).end));
                        statements.push(ts.createVariableStatement(
                        /*modifiers*/ undefined, declarationList));
                    }
                    else {
                        // The following call does not include the initializer, so we have
                        // to emit it separately.
                        statements.push(ts.setTextRange(ts.createVariableStatement(
                        /*modifiers*/ undefined, ts.setOriginalNode(ts.setTextRange(ts.createVariableDeclarationList([
                            ts.createVariableDeclaration(firstOriginalDeclaration ? firstOriginalDeclaration.name : ts.createTempVariable(/*recordTempVariable*/ undefined), 
                            /*type*/ undefined, boundValue)
                        ]), ts.moveRangePos(initializer, -1)), initializer)), ts.moveRangeEnd(initializer, -1)));
                    }
                }
                else {
                    // Initializer is an expression. Emit the expression in the body, so that it's
                    // evaluated on every iteration.
                    var assignment = ts.createAssignment(initializer, boundValue);
                    if (ts.isDestructuringAssignment(assignment)) {
                        ts.aggregateTransformFlags(assignment);
                        statements.push(ts.createExpressionStatement(visitBinaryExpression(assignment, /*needsDestructuringValue*/ false)));
                    }
                    else {
                        assignment.end = initializer.end;
                        statements.push(ts.setTextRange(ts.createExpressionStatement(ts.visitNode(assignment, visitor, ts.isExpression)), ts.moveRangeEnd(initializer, -1)));
                    }
                }
                if (convertedLoopBodyStatements) {
                    return createSyntheticBlockForConvertedStatements(ts.addRange(statements, convertedLoopBodyStatements));
                }
                else {
                    var statement = ts.visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock);
                    if (ts.isBlock(statement)) {
                        return ts.updateBlock(statement, ts.setTextRange(ts.createNodeArray(ts.concatenate(statements, statement.statements)), statement.statements));
                    }
                    else {
                        statements.push(statement);
                        return createSyntheticBlockForConvertedStatements(statements);
                    }
                }
            }
            function createSyntheticBlockForConvertedStatements(statements) {
                return ts.setEmitFlags(ts.createBlock(ts.createNodeArray(statements), 
                /*multiLine*/ true), 48 /* NoSourceMap */ | 384 /* NoTokenSourceMaps */);
            }
            function convertForOfStatementForArray(node, outermostLabeledStatement, convertedLoopBodyStatements) {
                // The following ES6 code:
                //
                //    for (let v of expr) { }
                //
                // should be emitted as
                //
                //    for (var _i = 0, _a = expr; _i < _a.length; _i++) {
                //        var v = _a[_i];
                //    }
                //
                // where _a and _i are temps emitted to capture the RHS and the counter,
                // respectively.
                // When the left hand side is an expression instead of a let declaration,
                // the "let v" is not emitted.
                // When the left hand side is a let/const, the v is renamed if there is
                // another v in scope.
                // Note that all assignments to the LHS are emitted in the body, including
                // all destructuring.
                // Note also that because an extra statement is needed to assign to the LHS,
                // for-of bodies are always emitted as blocks.
                var expression = ts.visitNode(node.expression, visitor, ts.isExpression);
                // In the case where the user wrote an identifier as the RHS, like this:
                //
                //     for (let v of arr) { }
                //
                // we don't want to emit a temporary variable for the RHS, just use it directly.
                var counter = ts.createLoopVariable();
                var rhsReference = ts.isIdentifier(expression) ? ts.getGeneratedNameForNode(expression) : ts.createTempVariable(/*recordTempVariable*/ undefined);
                // The old emitter does not emit source maps for the expression
                ts.setEmitFlags(expression, 48 /* NoSourceMap */ | ts.getEmitFlags(expression));
                var forStatement = ts.setTextRange(ts.createFor(
                /*initializer*/ ts.setEmitFlags(ts.setTextRange(ts.createVariableDeclarationList([
                    ts.setTextRange(ts.createVariableDeclaration(counter, /*type*/ undefined, ts.createLiteral(0)), ts.moveRangePos(node.expression, -1)),
                    ts.setTextRange(ts.createVariableDeclaration(rhsReference, /*type*/ undefined, expression), node.expression)
                ]), node.expression), 2097152 /* NoHoisting */), 
                /*condition*/ ts.setTextRange(ts.createLessThan(counter, ts.createPropertyAccess(rhsReference, "length")), node.expression), 
                /*incrementor*/ ts.setTextRange(ts.createPostfixIncrement(counter), node.expression), 
                /*statement*/ convertForOfStatementHead(node, ts.createElementAccess(rhsReference, counter), convertedLoopBodyStatements)), 
                /*location*/ node);
                // Disable trailing source maps for the OpenParenToken to align source map emit with the old emitter.
                ts.setEmitFlags(forStatement, 256 /* NoTokenTrailingSourceMaps */);
                ts.setTextRange(forStatement, node);
                return ts.restoreEnclosingLabel(forStatement, outermostLabeledStatement, convertedLoopState && resetLabel);
            }
            function convertForOfStatementForIterable(node, outermostLabeledStatement, convertedLoopBodyStatements, ancestorFacts) {
                var expression = ts.visitNode(node.expression, visitor, ts.isExpression);
                var iterator = ts.isIdentifier(expression) ? ts.getGeneratedNameForNode(expression) : ts.createTempVariable(/*recordTempVariable*/ undefined);
                var result = ts.isIdentifier(expression) ? ts.getGeneratedNameForNode(iterator) : ts.createTempVariable(/*recordTempVariable*/ undefined);
                var errorRecord = ts.createUniqueName("e");
                var catchVariable = ts.getGeneratedNameForNode(errorRecord);
                var returnMethod = ts.createTempVariable(/*recordTempVariable*/ undefined);
                var values = ts.createValuesHelper(context, expression, node.expression);
                var next = ts.createCall(ts.createPropertyAccess(iterator, "next"), /*typeArguments*/ undefined, []);
                hoistVariableDeclaration(errorRecord);
                hoistVariableDeclaration(returnMethod);
                // if we are enclosed in an outer loop ensure we reset 'errorRecord' per each iteration
                var initializer = ancestorFacts & 1024 /* IterationContainer */
                    ? ts.inlineExpressions([ts.createAssignment(errorRecord, ts.createVoidZero()), values])
                    : values;
                var forStatement = ts.setEmitFlags(ts.setTextRange(ts.createFor(
                /*initializer*/ ts.setEmitFlags(ts.setTextRange(ts.createVariableDeclarationList([
                    ts.setTextRange(ts.createVariableDeclaration(iterator, /*type*/ undefined, initializer), node.expression),
                    ts.createVariableDeclaration(result, /*type*/ undefined, next)
                ]), node.expression), 2097152 /* NoHoisting */), 
                /*condition*/ ts.createLogicalNot(ts.createPropertyAccess(result, "done")), 
                /*incrementor*/ ts.createAssignment(result, next), 
                /*statement*/ convertForOfStatementHead(node, ts.createPropertyAccess(result, "value"), convertedLoopBodyStatements)), 
                /*location*/ node), 256 /* NoTokenTrailingSourceMaps */);
                return ts.createTry(ts.createBlock([
                    ts.restoreEnclosingLabel(forStatement, outermostLabeledStatement, convertedLoopState && resetLabel)
                ]), ts.createCatchClause(ts.createVariableDeclaration(catchVariable), ts.setEmitFlags(ts.createBlock([
                    ts.createExpressionStatement(ts.createAssignment(errorRecord, ts.createObjectLiteral([
                        ts.createPropertyAssignment("error", catchVariable)
                    ])))
                ]), 1 /* SingleLine */)), ts.createBlock([
                    ts.createTry(
                    /*tryBlock*/ ts.createBlock([
                        ts.setEmitFlags(ts.createIf(ts.createLogicalAnd(ts.createLogicalAnd(result, ts.createLogicalNot(ts.createPropertyAccess(result, "done"))), ts.createAssignment(returnMethod, ts.createPropertyAccess(iterator, "return"))), ts.createExpressionStatement(ts.createFunctionCall(returnMethod, iterator, []))), 1 /* SingleLine */),
                    ]), 
                    /*catchClause*/ undefined, 
                    /*finallyBlock*/ ts.setEmitFlags(ts.createBlock([
                        ts.setEmitFlags(ts.createIf(errorRecord, ts.createThrow(ts.createPropertyAccess(errorRecord, "error"))), 1 /* SingleLine */)
                    ]), 1 /* SingleLine */))
                ]));
            }
            /**
             * Visits an ObjectLiteralExpression with computed property names.
             *
             * @param node An ObjectLiteralExpression node.
             */
            function visitObjectLiteralExpression(node) {
                // We are here because a ComputedPropertyName was used somewhere in the expression.
                var properties = node.properties;
                var numProperties = properties.length;
                // Find the first computed property.
                // Everything until that point can be emitted as part of the initial object literal.
                var numInitialProperties = numProperties;
                var numInitialPropertiesWithoutYield = numProperties;
                for (var i = 0; i < numProperties; i++) {
                    var property = properties[i];
                    if ((property.transformFlags & 262144 /* ContainsYield */ && hierarchyFacts & 4 /* AsyncFunctionBody */)
                        && i < numInitialPropertiesWithoutYield) {
                        numInitialPropertiesWithoutYield = i;
                    }
                    if (property.name.kind === 154 /* ComputedPropertyName */) {
                        numInitialProperties = i;
                        break;
                    }
                }
                if (numInitialProperties !== numProperties) {
                    if (numInitialPropertiesWithoutYield < numInitialProperties) {
                        numInitialProperties = numInitialPropertiesWithoutYield;
                    }
                    // For computed properties, we need to create a unique handle to the object
                    // literal so we can modify it without risking internal assignments tainting the object.
                    var temp = ts.createTempVariable(hoistVariableDeclaration);
                    // Write out the first non-computed properties, then emit the rest through indexing on the temp variable.
                    var expressions = [];
                    var assignment = ts.createAssignment(temp, ts.setEmitFlags(ts.createObjectLiteral(ts.visitNodes(properties, visitor, ts.isObjectLiteralElementLike, 0, numInitialProperties), node.multiLine), 65536 /* Indented */));
                    if (node.multiLine) {
                        ts.startOnNewLine(assignment);
                    }
                    expressions.push(assignment);
                    addObjectLiteralMembers(expressions, node, temp, numInitialProperties);
                    // We need to clone the temporary identifier so that we can write it on a
                    // new line
                    expressions.push(node.multiLine ? ts.startOnNewLine(ts.getMutableClone(temp)) : temp);
                    return ts.inlineExpressions(expressions);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function shouldConvertPartOfIterationStatement(node) {
                return (resolver.getNodeCheckFlags(node) & 131072 /* ContainsCapturedBlockScopeBinding */) !== 0;
            }
            function shouldConvertInitializerOfForStatement(node) {
                return ts.isForStatement(node) && !!node.initializer && shouldConvertPartOfIterationStatement(node.initializer);
            }
            function shouldConvertConditionOfForStatement(node) {
                return ts.isForStatement(node) && !!node.condition && shouldConvertPartOfIterationStatement(node.condition);
            }
            function shouldConvertIncrementorOfForStatement(node) {
                return ts.isForStatement(node) && !!node.incrementor && shouldConvertPartOfIterationStatement(node.incrementor);
            }
            function shouldConvertIterationStatement(node) {
                return shouldConvertBodyOfIterationStatement(node)
                    || shouldConvertInitializerOfForStatement(node);
            }
            function shouldConvertBodyOfIterationStatement(node) {
                return (resolver.getNodeCheckFlags(node) & 65536 /* LoopWithCapturedBlockScopedBinding */) !== 0;
            }
            /**
             * Records constituents of name for the given variable to be hoisted in the outer scope.
             */
            function hoistVariableDeclarationDeclaredInConvertedLoop(state, node) {
                if (!state.hoistedLocalVariables) {
                    state.hoistedLocalVariables = [];
                }
                visit(node.name);
                function visit(node) {
                    if (node.kind === 75 /* Identifier */) {
                        state.hoistedLocalVariables.push(node);
                    }
                    else {
                        for (var _i = 0, _a = node.elements; _i < _a.length; _i++) {
                            var element = _a[_i];
                            if (!ts.isOmittedExpression(element)) {
                                visit(element.name);
                            }
                        }
                    }
                }
            }
            function convertIterationStatementBodyIfNecessary(node, outermostLabeledStatement, ancestorFacts, convert) {
                if (!shouldConvertIterationStatement(node)) {
                    var saveAllowedNonLabeledJumps = void 0;
                    if (convertedLoopState) {
                        // we get here if we are trying to emit normal loop loop inside converted loop
                        // set allowedNonLabeledJumps to Break | Continue to mark that break\continue inside the loop should be emitted as is
                        saveAllowedNonLabeledJumps = convertedLoopState.allowedNonLabeledJumps;
                        convertedLoopState.allowedNonLabeledJumps = 2 /* Break */ | 4 /* Continue */;
                    }
                    var result = convert
                        ? convert(node, outermostLabeledStatement, /*convertedLoopBodyStatements*/ undefined, ancestorFacts)
                        : ts.restoreEnclosingLabel(ts.visitEachChild(node, visitor, context), outermostLabeledStatement, convertedLoopState && resetLabel);
                    if (convertedLoopState) {
                        convertedLoopState.allowedNonLabeledJumps = saveAllowedNonLabeledJumps;
                    }
                    return result;
                }
                var currentState = createConvertedLoopState(node);
                var statements = [];
                var outerConvertedLoopState = convertedLoopState;
                convertedLoopState = currentState;
                var initializerFunction = shouldConvertInitializerOfForStatement(node) ? createFunctionForInitializerOfForStatement(node, currentState) : undefined;
                var bodyFunction = shouldConvertBodyOfIterationStatement(node) ? createFunctionForBodyOfIterationStatement(node, currentState, outerConvertedLoopState) : undefined;
                convertedLoopState = outerConvertedLoopState;
                if (initializerFunction)
                    statements.push(initializerFunction.functionDeclaration);
                if (bodyFunction)
                    statements.push(bodyFunction.functionDeclaration);
                addExtraDeclarationsForConvertedLoop(statements, currentState, outerConvertedLoopState);
                if (initializerFunction) {
                    statements.push(generateCallToConvertedLoopInitializer(initializerFunction.functionName, initializerFunction.containsYield));
                }
                var loop;
                if (bodyFunction) {
                    if (convert) {
                        loop = convert(node, outermostLabeledStatement, bodyFunction.part, ancestorFacts);
                    }
                    else {
                        var clone_4 = convertIterationStatementCore(node, initializerFunction, ts.createBlock(bodyFunction.part, /*multiLine*/ true));
                        ts.aggregateTransformFlags(clone_4);
                        loop = ts.restoreEnclosingLabel(clone_4, outermostLabeledStatement, convertedLoopState && resetLabel);
                    }
                }
                else {
                    var clone_5 = convertIterationStatementCore(node, initializerFunction, ts.visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock));
                    ts.aggregateTransformFlags(clone_5);
                    loop = ts.restoreEnclosingLabel(clone_5, outermostLabeledStatement, convertedLoopState && resetLabel);
                }
                statements.push(loop);
                return statements;
            }
            function convertIterationStatementCore(node, initializerFunction, convertedLoopBody) {
                switch (node.kind) {
                    case 230 /* ForStatement */: return convertForStatement(node, initializerFunction, convertedLoopBody);
                    case 231 /* ForInStatement */: return convertForInStatement(node, convertedLoopBody);
                    case 232 /* ForOfStatement */: return convertForOfStatement(node, convertedLoopBody);
                    case 228 /* DoStatement */: return convertDoStatement(node, convertedLoopBody);
                    case 229 /* WhileStatement */: return convertWhileStatement(node, convertedLoopBody);
                    default: return ts.Debug.failBadSyntaxKind(node, "IterationStatement expected");
                }
            }
            function convertForStatement(node, initializerFunction, convertedLoopBody) {
                var shouldConvertCondition = node.condition && shouldConvertPartOfIterationStatement(node.condition);
                var shouldConvertIncrementor = shouldConvertCondition || node.incrementor && shouldConvertPartOfIterationStatement(node.incrementor);
                return ts.updateFor(node, ts.visitNode(initializerFunction ? initializerFunction.part : node.initializer, visitor, ts.isForInitializer), ts.visitNode(shouldConvertCondition ? undefined : node.condition, visitor, ts.isExpression), ts.visitNode(shouldConvertIncrementor ? undefined : node.incrementor, visitor, ts.isExpression), convertedLoopBody);
            }
            function convertForOfStatement(node, convertedLoopBody) {
                return ts.updateForOf(node, 
                /*awaitModifier*/ undefined, ts.visitNode(node.initializer, visitor, ts.isForInitializer), ts.visitNode(node.expression, visitor, ts.isExpression), convertedLoopBody);
            }
            function convertForInStatement(node, convertedLoopBody) {
                return ts.updateForIn(node, ts.visitNode(node.initializer, visitor, ts.isForInitializer), ts.visitNode(node.expression, visitor, ts.isExpression), convertedLoopBody);
            }
            function convertDoStatement(node, convertedLoopBody) {
                return ts.updateDo(node, convertedLoopBody, ts.visitNode(node.expression, visitor, ts.isExpression));
            }
            function convertWhileStatement(node, convertedLoopBody) {
                return ts.updateWhile(node, ts.visitNode(node.expression, visitor, ts.isExpression), convertedLoopBody);
            }
            function createConvertedLoopState(node) {
                var loopInitializer;
                switch (node.kind) {
                    case 230 /* ForStatement */:
                    case 231 /* ForInStatement */:
                    case 232 /* ForOfStatement */:
                        var initializer = node.initializer;
                        if (initializer && initializer.kind === 243 /* VariableDeclarationList */) {
                            loopInitializer = initializer;
                        }
                        break;
                }
                // variables that will be passed to the loop as parameters
                var loopParameters = [];
                // variables declared in the loop initializer that will be changed inside the loop
                var loopOutParameters = [];
                if (loopInitializer && (ts.getCombinedNodeFlags(loopInitializer) & 3 /* BlockScoped */)) {
                    var hasCapturedBindingsInForInitializer = shouldConvertInitializerOfForStatement(node);
                    for (var _i = 0, _a = loopInitializer.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        processLoopVariableDeclaration(node, decl, loopParameters, loopOutParameters, hasCapturedBindingsInForInitializer);
                    }
                }
                var currentState = { loopParameters: loopParameters, loopOutParameters: loopOutParameters };
                if (convertedLoopState) {
                    // convertedOuterLoopState !== undefined means that this converted loop is nested in another converted loop.
                    // if outer converted loop has already accumulated some state - pass it through
                    if (convertedLoopState.argumentsName) {
                        // outer loop has already used 'arguments' so we've already have some name to alias it
                        // use the same name in all nested loops
                        currentState.argumentsName = convertedLoopState.argumentsName;
                    }
                    if (convertedLoopState.thisName) {
                        // outer loop has already used 'this' so we've already have some name to alias it
                        // use the same name in all nested loops
                        currentState.thisName = convertedLoopState.thisName;
                    }
                    if (convertedLoopState.hoistedLocalVariables) {
                        // we've already collected some non-block scoped variable declarations in enclosing loop
                        // use the same storage in nested loop
                        currentState.hoistedLocalVariables = convertedLoopState.hoistedLocalVariables;
                    }
                }
                return currentState;
            }
            function addExtraDeclarationsForConvertedLoop(statements, state, outerState) {
                var extraVariableDeclarations;
                // propagate state from the inner loop to the outer loop if necessary
                if (state.argumentsName) {
                    // if alias for arguments is set
                    if (outerState) {
                        // pass it to outer converted loop
                        outerState.argumentsName = state.argumentsName;
                    }
                    else {
                        // this is top level converted loop and we need to create an alias for 'arguments' object
                        (extraVariableDeclarations || (extraVariableDeclarations = [])).push(ts.createVariableDeclaration(state.argumentsName, 
                        /*type*/ undefined, ts.createIdentifier("arguments")));
                    }
                }
                if (state.thisName) {
                    // if alias for this is set
                    if (outerState) {
                        // pass it to outer converted loop
                        outerState.thisName = state.thisName;
                    }
                    else {
                        // this is top level converted loop so we need to create an alias for 'this' here
                        // NOTE:
                        // if converted loops were all nested in arrow function then we'll always emit '_this' so convertedLoopState.thisName will not be set.
                        // If it is set this means that all nested loops are not nested in arrow function and it is safe to capture 'this'.
                        (extraVariableDeclarations || (extraVariableDeclarations = [])).push(ts.createVariableDeclaration(state.thisName, 
                        /*type*/ undefined, ts.createIdentifier("this")));
                    }
                }
                if (state.hoistedLocalVariables) {
                    // if hoistedLocalVariables !== undefined this means that we've possibly collected some variable declarations to be hoisted later
                    if (outerState) {
                        // pass them to outer converted loop
                        outerState.hoistedLocalVariables = state.hoistedLocalVariables;
                    }
                    else {
                        if (!extraVariableDeclarations) {
                            extraVariableDeclarations = [];
                        }
                        // hoist collected variable declarations
                        for (var _i = 0, _a = state.hoistedLocalVariables; _i < _a.length; _i++) {
                            var identifier = _a[_i];
                            extraVariableDeclarations.push(ts.createVariableDeclaration(identifier));
                        }
                    }
                }
                // add extra variables to hold out parameters if necessary
                if (state.loopOutParameters.length) {
                    if (!extraVariableDeclarations) {
                        extraVariableDeclarations = [];
                    }
                    for (var _b = 0, _c = state.loopOutParameters; _b < _c.length; _b++) {
                        var outParam = _c[_b];
                        extraVariableDeclarations.push(ts.createVariableDeclaration(outParam.outParamName));
                    }
                }
                if (state.conditionVariable) {
                    if (!extraVariableDeclarations) {
                        extraVariableDeclarations = [];
                    }
                    extraVariableDeclarations.push(ts.createVariableDeclaration(state.conditionVariable, /*type*/ undefined, ts.createFalse()));
                }
                // create variable statement to hold all introduced variable declarations
                if (extraVariableDeclarations) {
                    statements.push(ts.createVariableStatement(
                    /*modifiers*/ undefined, ts.createVariableDeclarationList(extraVariableDeclarations)));
                }
            }
            function createOutVariable(p) {
                return ts.createVariableDeclaration(p.originalName, /*type*/ undefined, p.outParamName);
            }
            /**
             * Creates a `_loop_init` function for a `ForStatement` with a block-scoped initializer
             * that is captured in a closure inside of the initializer. The `_loop_init` function is
             * used to preserve the per-iteration environment semantics of
             * [13.7.4.8 RS: ForBodyEvaluation](https://tc39.github.io/ecma262/#sec-forbodyevaluation).
             */
            function createFunctionForInitializerOfForStatement(node, currentState) {
                var functionName = ts.createUniqueName("_loop_init");
                var containsYield = (node.initializer.transformFlags & 262144 /* ContainsYield */) !== 0;
                var emitFlags = 0 /* None */;
                if (currentState.containsLexicalThis)
                    emitFlags |= 8 /* CapturesThis */;
                if (containsYield && hierarchyFacts & 4 /* AsyncFunctionBody */)
                    emitFlags |= 262144 /* AsyncFunctionBody */;
                var statements = [];
                statements.push(ts.createVariableStatement(/*modifiers*/ undefined, node.initializer));
                copyOutParameters(currentState.loopOutParameters, 2 /* Initializer */, 1 /* ToOutParameter */, statements);
                // This transforms the following ES2015 syntax:
                //
                //  for (let i = (setImmediate(() => console.log(i)), 0); i < 2; i++) {
                //      // loop body
                //  }
                //
                // Into the following ES5 syntax:
                //
                //  var _loop_init_1 = function () {
                //      var i = (setImmediate(() => console.log(i)), 0);
                //      out_i_1 = i;
                //  };
                //  var out_i_1;
                //  _loop_init_1();
                //  for (var i = out_i_1; i < 2; i++) {
                //      // loop body
                //  }
                //
                // Which prevents mutations to `i` in the per-iteration environment of the body
                // from affecting the initial value for `i` outside of the per-iteration environment.
                var functionDeclaration = ts.createVariableStatement(
                /*modifiers*/ undefined, ts.setEmitFlags(ts.createVariableDeclarationList([
                    ts.createVariableDeclaration(functionName, 
                    /*type*/ undefined, ts.setEmitFlags(ts.createFunctionExpression(
                    /*modifiers*/ undefined, containsYield ? ts.createToken(41 /* AsteriskToken */) : undefined, 
                    /*name*/ undefined, 
                    /*typeParameters*/ undefined, 
                    /*parameters*/ undefined, 
                    /*type*/ undefined, ts.visitNode(ts.createBlock(statements, /*multiLine*/ true), visitor, ts.isBlock)), emitFlags))
                ]), 2097152 /* NoHoisting */));
                var part = ts.createVariableDeclarationList(ts.map(currentState.loopOutParameters, createOutVariable));
                return { functionName: functionName, containsYield: containsYield, functionDeclaration: functionDeclaration, part: part };
            }
            /**
             * Creates a `_loop` function for an `IterationStatement` with a block-scoped initializer
             * that is captured in a closure inside of the loop body. The `_loop` function is used to
             * preserve the per-iteration environment semantics of
             * [13.7.4.8 RS: ForBodyEvaluation](https://tc39.github.io/ecma262/#sec-forbodyevaluation).
             */
            function createFunctionForBodyOfIterationStatement(node, currentState, outerState) {
                var functionName = ts.createUniqueName("_loop");
                startLexicalEnvironment();
                var statement = ts.visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock);
                var lexicalEnvironment = endLexicalEnvironment();
                var statements = [];
                if (shouldConvertConditionOfForStatement(node) || shouldConvertIncrementorOfForStatement(node)) {
                    // If a block-scoped variable declared in the initializer of `node` is captured in
                    // the condition or incrementor, we must move the condition and incrementor into
                    // the body of the for loop.
                    //
                    // This transforms the following ES2015 syntax:
                    //
                    //  for (let i = 0; setImmediate(() => console.log(i)), i < 2; setImmediate(() => console.log(i)), i++) {
                    //      // loop body
                    //  }
                    //
                    // Into the following ES5 syntax:
                    //
                    //  var _loop_1 = function (i) {
                    //      if (inc_1)
                    //          setImmediate(() => console.log(i)), i++;
                    //      else
                    //          inc_1 = true;
                    //      if (!(setImmediate(() => console.log(i)), i < 2))
                    //          return out_i_1 = i, "break";
                    //      // loop body
                    //      out_i_1 = i;
                    //  }
                    //  var out_i_1, inc_1 = false;
                    //  for (var i = 0;;) {
                    //      var state_1 = _loop_1(i);
                    //      i = out_i_1;
                    //      if (state_1 === "break")
                    //          break;
                    //  }
                    //
                    // Which prevents mutations to `i` in the per-iteration environment of the body
                    // from affecting the value of `i` in the previous per-iteration environment.
                    //
                    // Note that the incrementor of a `for` loop is evaluated in a *new* per-iteration
                    // environment that is carried over to the next iteration of the loop. As a result,
                    // we must indicate whether this is the first evaluation of the loop body so that
                    // we only evaluate the incrementor on subsequent evaluations.
                    currentState.conditionVariable = ts.createUniqueName("inc");
                    statements.push(ts.createIf(currentState.conditionVariable, ts.createStatement(ts.visitNode(node.incrementor, visitor, ts.isExpression)), ts.createStatement(ts.createAssignment(currentState.conditionVariable, ts.createTrue()))));
                    if (shouldConvertConditionOfForStatement(node)) {
                        statements.push(ts.createIf(ts.createPrefix(53 /* ExclamationToken */, ts.visitNode(node.condition, visitor, ts.isExpression)), ts.visitNode(ts.createBreak(), visitor, ts.isStatement)));
                    }
                }
                if (ts.isBlock(statement)) {
                    ts.addRange(statements, statement.statements);
                }
                else {
                    statements.push(statement);
                }
                copyOutParameters(currentState.loopOutParameters, 1 /* Body */, 1 /* ToOutParameter */, statements);
                ts.insertStatementsAfterStandardPrologue(statements, lexicalEnvironment);
                var loopBody = ts.createBlock(statements, /*multiLine*/ true);
                if (ts.isBlock(statement))
                    ts.setOriginalNode(loopBody, statement);
                var containsYield = (node.statement.transformFlags & 262144 /* ContainsYield */) !== 0;
                var emitFlags = 0;
                if (currentState.containsLexicalThis)
                    emitFlags |= 8 /* CapturesThis */;
                if (containsYield && (hierarchyFacts & 4 /* AsyncFunctionBody */) !== 0)
                    emitFlags |= 262144 /* AsyncFunctionBody */;
                // This transforms the following ES2015 syntax (in addition to other variations):
                //
                //  for (let i = 0; i < 2; i++) {
                //      setImmediate(() => console.log(i));
                //  }
                //
                // Into the following ES5 syntax:
                //
                //  var _loop_1 = function (i) {
                //      setImmediate(() => console.log(i));
                //  };
                //  for (var i = 0; i < 2; i++) {
                //      _loop_1(i);
                //  }
                var functionDeclaration = ts.createVariableStatement(
                /*modifiers*/ undefined, ts.setEmitFlags(ts.createVariableDeclarationList([
                    ts.createVariableDeclaration(functionName, 
                    /*type*/ undefined, ts.setEmitFlags(ts.createFunctionExpression(
                    /*modifiers*/ undefined, containsYield ? ts.createToken(41 /* AsteriskToken */) : undefined, 
                    /*name*/ undefined, 
                    /*typeParameters*/ undefined, currentState.loopParameters, 
                    /*type*/ undefined, loopBody), emitFlags))
                ]), 2097152 /* NoHoisting */));
                var part = generateCallToConvertedLoop(functionName, currentState, outerState, containsYield);
                return { functionName: functionName, containsYield: containsYield, functionDeclaration: functionDeclaration, part: part };
            }
            function copyOutParameter(outParam, copyDirection) {
                var source = copyDirection === 0 /* ToOriginal */ ? outParam.outParamName : outParam.originalName;
                var target = copyDirection === 0 /* ToOriginal */ ? outParam.originalName : outParam.outParamName;
                return ts.createBinary(target, 62 /* EqualsToken */, source);
            }
            function copyOutParameters(outParams, partFlags, copyDirection, statements) {
                for (var _i = 0, outParams_1 = outParams; _i < outParams_1.length; _i++) {
                    var outParam = outParams_1[_i];
                    if (outParam.flags & partFlags) {
                        statements.push(ts.createExpressionStatement(copyOutParameter(outParam, copyDirection)));
                    }
                }
            }
            function generateCallToConvertedLoopInitializer(initFunctionExpressionName, containsYield) {
                var call = ts.createCall(initFunctionExpressionName, /*typeArguments*/ undefined, []);
                var callResult = containsYield
                    ? ts.createYield(ts.createToken(41 /* AsteriskToken */), ts.setEmitFlags(call, 8388608 /* Iterator */))
                    : call;
                return ts.createStatement(callResult);
            }
            function generateCallToConvertedLoop(loopFunctionExpressionName, state, outerState, containsYield) {
                var statements = [];
                // loop is considered simple if it does not have any return statements or break\continue that transfer control outside of the loop
                // simple loops are emitted as just 'loop()';
                // NOTE: if loop uses only 'continue' it still will be emitted as simple loop
                var isSimpleLoop = !(state.nonLocalJumps & ~4 /* Continue */) &&
                    !state.labeledNonLocalBreaks &&
                    !state.labeledNonLocalContinues;
                var call = ts.createCall(loopFunctionExpressionName, /*typeArguments*/ undefined, ts.map(state.loopParameters, function (p) { return p.name; }));
                var callResult = containsYield
                    ? ts.createYield(ts.createToken(41 /* AsteriskToken */), ts.setEmitFlags(call, 8388608 /* Iterator */))
                    : call;
                if (isSimpleLoop) {
                    statements.push(ts.createExpressionStatement(callResult));
                    copyOutParameters(state.loopOutParameters, 1 /* Body */, 0 /* ToOriginal */, statements);
                }
                else {
                    var loopResultName = ts.createUniqueName("state");
                    var stateVariable = ts.createVariableStatement(
                    /*modifiers*/ undefined, ts.createVariableDeclarationList([ts.createVariableDeclaration(loopResultName, /*type*/ undefined, callResult)]));
                    statements.push(stateVariable);
                    copyOutParameters(state.loopOutParameters, 1 /* Body */, 0 /* ToOriginal */, statements);
                    if (state.nonLocalJumps & 8 /* Return */) {
                        var returnStatement = void 0;
                        if (outerState) {
                            outerState.nonLocalJumps |= 8 /* Return */;
                            returnStatement = ts.createReturn(loopResultName);
                        }
                        else {
                            returnStatement = ts.createReturn(ts.createPropertyAccess(loopResultName, "value"));
                        }
                        statements.push(ts.createIf(ts.createBinary(ts.createTypeOf(loopResultName), 36 /* EqualsEqualsEqualsToken */, ts.createLiteral("object")), returnStatement));
                    }
                    if (state.nonLocalJumps & 2 /* Break */) {
                        statements.push(ts.createIf(ts.createBinary(loopResultName, 36 /* EqualsEqualsEqualsToken */, ts.createLiteral("break")), ts.createBreak()));
                    }
                    if (state.labeledNonLocalBreaks || state.labeledNonLocalContinues) {
                        var caseClauses = [];
                        processLabeledJumps(state.labeledNonLocalBreaks, /*isBreak*/ true, loopResultName, outerState, caseClauses);
                        processLabeledJumps(state.labeledNonLocalContinues, /*isBreak*/ false, loopResultName, outerState, caseClauses);
                        statements.push(ts.createSwitch(loopResultName, ts.createCaseBlock(caseClauses)));
                    }
                }
                return statements;
            }
            function setLabeledJump(state, isBreak, labelText, labelMarker) {
                if (isBreak) {
                    if (!state.labeledNonLocalBreaks) {
                        state.labeledNonLocalBreaks = ts.createMap();
                    }
                    state.labeledNonLocalBreaks.set(labelText, labelMarker);
                }
                else {
                    if (!state.labeledNonLocalContinues) {
                        state.labeledNonLocalContinues = ts.createMap();
                    }
                    state.labeledNonLocalContinues.set(labelText, labelMarker);
                }
            }
            function processLabeledJumps(table, isBreak, loopResultName, outerLoop, caseClauses) {
                if (!table) {
                    return;
                }
                table.forEach(function (labelMarker, labelText) {
                    var statements = [];
                    // if there are no outer converted loop or outer label in question is located inside outer converted loop
                    // then emit labeled break\continue
                    // otherwise propagate pair 'label -> marker' to outer converted loop and emit 'return labelMarker' so outer loop can later decide what to do
                    if (!outerLoop || (outerLoop.labels && outerLoop.labels.get(labelText))) {
                        var label = ts.createIdentifier(labelText);
                        statements.push(isBreak ? ts.createBreak(label) : ts.createContinue(label));
                    }
                    else {
                        setLabeledJump(outerLoop, isBreak, labelText, labelMarker);
                        statements.push(ts.createReturn(loopResultName));
                    }
                    caseClauses.push(ts.createCaseClause(ts.createLiteral(labelMarker), statements));
                });
            }
            function processLoopVariableDeclaration(container, decl, loopParameters, loopOutParameters, hasCapturedBindingsInForInitializer) {
                var name = decl.name;
                if (ts.isBindingPattern(name)) {
                    for (var _i = 0, _a = name.elements; _i < _a.length; _i++) {
                        var element = _a[_i];
                        if (!ts.isOmittedExpression(element)) {
                            processLoopVariableDeclaration(container, element, loopParameters, loopOutParameters, hasCapturedBindingsInForInitializer);
                        }
                    }
                }
                else {
                    loopParameters.push(ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, name));
                    var checkFlags = resolver.getNodeCheckFlags(decl);
                    if (checkFlags & 4194304 /* NeedsLoopOutParameter */ || hasCapturedBindingsInForInitializer) {
                        var outParamName = ts.createUniqueName("out_" + ts.idText(name));
                        var flags = 0;
                        if (checkFlags & 4194304 /* NeedsLoopOutParameter */) {
                            flags |= 1 /* Body */;
                        }
                        if (ts.isForStatement(container) && container.initializer && resolver.isBindingCapturedByNode(container.initializer, decl)) {
                            flags |= 2 /* Initializer */;
                        }
                        loopOutParameters.push({ flags: flags, originalName: name, outParamName: outParamName });
                    }
                }
            }
            /**
             * Adds the members of an object literal to an array of expressions.
             *
             * @param expressions An array of expressions.
             * @param node An ObjectLiteralExpression node.
             * @param receiver The receiver for members of the ObjectLiteralExpression.
             * @param numInitialNonComputedProperties The number of initial properties without
             *                                        computed property names.
             */
            function addObjectLiteralMembers(expressions, node, receiver, start) {
                var properties = node.properties;
                var numProperties = properties.length;
                for (var i = start; i < numProperties; i++) {
                    var property = properties[i];
                    switch (property.kind) {
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            var accessors = ts.getAllAccessorDeclarations(node.properties, property);
                            if (property === accessors.firstAccessor) {
                                expressions.push(transformAccessorsToExpression(receiver, accessors, node, !!node.multiLine));
                            }
                            break;
                        case 161 /* MethodDeclaration */:
                            expressions.push(transformObjectLiteralMethodDeclarationToExpression(property, receiver, node, node.multiLine));
                            break;
                        case 281 /* PropertyAssignment */:
                            expressions.push(transformPropertyAssignmentToExpression(property, receiver, node.multiLine));
                            break;
                        case 282 /* ShorthandPropertyAssignment */:
                            expressions.push(transformShorthandPropertyAssignmentToExpression(property, receiver, node.multiLine));
                            break;
                        default:
                            ts.Debug.failBadSyntaxKind(node);
                            break;
                    }
                }
            }
            /**
             * Transforms a PropertyAssignment node into an expression.
             *
             * @param node The ObjectLiteralExpression that contains the PropertyAssignment.
             * @param property The PropertyAssignment node.
             * @param receiver The receiver for the assignment.
             */
            function transformPropertyAssignmentToExpression(property, receiver, startsOnNewLine) {
                var expression = ts.createAssignment(ts.createMemberAccessForPropertyName(receiver, ts.visitNode(property.name, visitor, ts.isPropertyName)), ts.visitNode(property.initializer, visitor, ts.isExpression));
                ts.setTextRange(expression, property);
                if (startsOnNewLine) {
                    ts.startOnNewLine(expression);
                }
                return expression;
            }
            /**
             * Transforms a ShorthandPropertyAssignment node into an expression.
             *
             * @param node The ObjectLiteralExpression that contains the ShorthandPropertyAssignment.
             * @param property The ShorthandPropertyAssignment node.
             * @param receiver The receiver for the assignment.
             */
            function transformShorthandPropertyAssignmentToExpression(property, receiver, startsOnNewLine) {
                var expression = ts.createAssignment(ts.createMemberAccessForPropertyName(receiver, ts.visitNode(property.name, visitor, ts.isPropertyName)), ts.getSynthesizedClone(property.name));
                ts.setTextRange(expression, property);
                if (startsOnNewLine) {
                    ts.startOnNewLine(expression);
                }
                return expression;
            }
            /**
             * Transforms a MethodDeclaration of an ObjectLiteralExpression into an expression.
             *
             * @param node The ObjectLiteralExpression that contains the MethodDeclaration.
             * @param method The MethodDeclaration node.
             * @param receiver The receiver for the assignment.
             */
            function transformObjectLiteralMethodDeclarationToExpression(method, receiver, container, startsOnNewLine) {
                var expression = ts.createAssignment(ts.createMemberAccessForPropertyName(receiver, ts.visitNode(method.name, visitor, ts.isPropertyName)), transformFunctionLikeToExpression(method, /*location*/ method, /*name*/ undefined, container));
                ts.setTextRange(expression, method);
                if (startsOnNewLine) {
                    ts.startOnNewLine(expression);
                }
                return expression;
            }
            function visitCatchClause(node) {
                var ancestorFacts = enterSubtree(7104 /* BlockScopeExcludes */, 0 /* BlockScopeIncludes */);
                var updated;
                ts.Debug.assert(!!node.variableDeclaration, "Catch clause variable should always be present when downleveling ES2015.");
                if (ts.isBindingPattern(node.variableDeclaration.name)) {
                    var temp = ts.createTempVariable(/*recordTempVariable*/ undefined);
                    var newVariableDeclaration = ts.createVariableDeclaration(temp);
                    ts.setTextRange(newVariableDeclaration, node.variableDeclaration);
                    var vars = ts.flattenDestructuringBinding(node.variableDeclaration, visitor, context, 0 /* All */, temp);
                    var list = ts.createVariableDeclarationList(vars);
                    ts.setTextRange(list, node.variableDeclaration);
                    var destructure = ts.createVariableStatement(/*modifiers*/ undefined, list);
                    updated = ts.updateCatchClause(node, newVariableDeclaration, addStatementToStartOfBlock(node.block, destructure));
                }
                else {
                    updated = ts.visitEachChild(node, visitor, context);
                }
                exitSubtree(ancestorFacts, 0 /* None */, 0 /* None */);
                return updated;
            }
            function addStatementToStartOfBlock(block, statement) {
                var transformedStatements = ts.visitNodes(block.statements, visitor, ts.isStatement);
                return ts.updateBlock(block, __spreadArrays([statement], transformedStatements));
            }
            /**
             * Visits a MethodDeclaration of an ObjectLiteralExpression and transforms it into a
             * PropertyAssignment.
             *
             * @param node A MethodDeclaration node.
             */
            function visitMethodDeclaration(node) {
                // We should only get here for methods on an object literal with regular identifier names.
                // Methods on classes are handled in visitClassDeclaration/visitClassExpression.
                // Methods with computed property names are handled in visitObjectLiteralExpression.
                ts.Debug.assert(!ts.isComputedPropertyName(node.name));
                var functionExpression = transformFunctionLikeToExpression(node, /*location*/ ts.moveRangePos(node, -1), /*name*/ undefined, /*container*/ undefined);
                ts.setEmitFlags(functionExpression, 512 /* NoLeadingComments */ | ts.getEmitFlags(functionExpression));
                return ts.setTextRange(ts.createPropertyAssignment(node.name, functionExpression), 
                /*location*/ node);
            }
            /**
             * Visits an AccessorDeclaration of an ObjectLiteralExpression.
             *
             * @param node An AccessorDeclaration node.
             */
            function visitAccessorDeclaration(node) {
                ts.Debug.assert(!ts.isComputedPropertyName(node.name));
                var savedConvertedLoopState = convertedLoopState;
                convertedLoopState = undefined;
                var ancestorFacts = enterSubtree(16286 /* FunctionExcludes */, 65 /* FunctionIncludes */);
                var updated;
                var parameters = ts.visitParameterList(node.parameters, visitor, context);
                var body = transformFunctionBody(node);
                if (node.kind === 163 /* GetAccessor */) {
                    updated = ts.updateGetAccessor(node, node.decorators, node.modifiers, node.name, parameters, node.type, body);
                }
                else {
                    updated = ts.updateSetAccessor(node, node.decorators, node.modifiers, node.name, parameters, body);
                }
                exitSubtree(ancestorFacts, 49152 /* FunctionSubtreeExcludes */, 0 /* None */);
                convertedLoopState = savedConvertedLoopState;
                return updated;
            }
            /**
             * Visits a ShorthandPropertyAssignment and transforms it into a PropertyAssignment.
             *
             * @param node A ShorthandPropertyAssignment node.
             */
            function visitShorthandPropertyAssignment(node) {
                return ts.setTextRange(ts.createPropertyAssignment(node.name, ts.getSynthesizedClone(node.name)), 
                /*location*/ node);
            }
            function visitComputedPropertyName(node) {
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Visits a YieldExpression node.
             *
             * @param node A YieldExpression node.
             */
            function visitYieldExpression(node) {
                // `yield` expressions are transformed using the generators transformer.
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Visits an ArrayLiteralExpression that contains a spread element.
             *
             * @param node An ArrayLiteralExpression node.
             */
            function visitArrayLiteralExpression(node) {
                if (ts.some(node.elements, ts.isSpreadElement)) {
                    // We are here because we contain a SpreadElementExpression.
                    return transformAndSpreadElements(node.elements, /*needsUniqueCopy*/ true, !!node.multiLine, /*hasTrailingComma*/ !!node.elements.hasTrailingComma);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Visits a CallExpression that contains either a spread element or `super`.
             *
             * @param node a CallExpression.
             */
            function visitCallExpression(node) {
                if (ts.getEmitFlags(node) & 33554432 /* TypeScriptClassWrapper */) {
                    return visitTypeScriptClassWrapper(node);
                }
                var expression = ts.skipOuterExpressions(node.expression);
                if (expression.kind === 102 /* SuperKeyword */ ||
                    ts.isSuperProperty(expression) ||
                    ts.some(node.arguments, ts.isSpreadElement)) {
                    return visitCallExpressionWithPotentialCapturedThisAssignment(node, /*assignToCapturedThis*/ true);
                }
                return ts.updateCall(node, ts.visitNode(node.expression, callExpressionVisitor, ts.isExpression), 
                /*typeArguments*/ undefined, ts.visitNodes(node.arguments, visitor, ts.isExpression));
            }
            function visitTypeScriptClassWrapper(node) {
                // This is a call to a class wrapper function (an IIFE) created by the 'ts' transformer.
                // The wrapper has a form similar to:
                //
                //  (function() {
                //      class C { // 1
                //      }
                //      C.x = 1; // 2
                //      return C;
                //  }())
                //
                // When we transform the class, we end up with something like this:
                //
                //  (function () {
                //      var C = (function () { // 3
                //          function C() {
                //          }
                //          return C; // 4
                //      }());
                //      C.x = 1;
                //      return C;
                //  }())
                //
                // We want to simplify the two nested IIFEs to end up with something like this:
                //
                //  (function () {
                //      function C() {
                //      }
                //      C.x = 1;
                //      return C;
                //  }())
                // We skip any outer expressions in a number of places to get to the innermost
                // expression, but we will restore them later to preserve comments and source maps.
                var body = ts.cast(ts.cast(ts.skipOuterExpressions(node.expression), ts.isArrowFunction).body, ts.isBlock);
                // The class statements are the statements generated by visiting the first statement with initializer of the
                // body (1), while all other statements are added to remainingStatements (2)
                var isVariableStatementWithInitializer = function (stmt) { return ts.isVariableStatement(stmt) && !!ts.first(stmt.declarationList.declarations).initializer; };
                var bodyStatements = ts.visitNodes(body.statements, visitor, ts.isStatement);
                var classStatements = ts.filter(bodyStatements, isVariableStatementWithInitializer);
                var remainingStatements = ts.filter(bodyStatements, function (stmt) { return !isVariableStatementWithInitializer(stmt); });
                var varStatement = ts.cast(ts.first(classStatements), ts.isVariableStatement);
                // We know there is only one variable declaration here as we verified this in an
                // earlier call to isTypeScriptClassWrapper
                var variable = varStatement.declarationList.declarations[0];
                var initializer = ts.skipOuterExpressions(variable.initializer);
                // Under certain conditions, the 'ts' transformer may introduce a class alias, which
                // we see as an assignment, for example:
                //
                //  (function () {
                //      var C_1;
                //      var C = C_1 = (function () {
                //          function C() {
                //          }
                //          C.x = function () { return C_1; }
                //          return C;
                //      }());
                //      C = C_1 = __decorate([dec], C);
                //      return C;
                //  }())
                //
                var aliasAssignment = ts.tryCast(initializer, ts.isAssignmentExpression);
                // The underlying call (3) is another IIFE that may contain a '_super' argument.
                var call = ts.cast(aliasAssignment ? ts.skipOuterExpressions(aliasAssignment.right) : initializer, ts.isCallExpression);
                var func = ts.cast(ts.skipOuterExpressions(call.expression), ts.isFunctionExpression);
                var funcStatements = func.body.statements;
                var classBodyStart = 0;
                var classBodyEnd = -1;
                var statements = [];
                if (aliasAssignment) {
                    // If we have a class alias assignment, we need to move it to the down-level constructor
                    // function we generated for the class.
                    var extendsCall = ts.tryCast(funcStatements[classBodyStart], ts.isExpressionStatement);
                    if (extendsCall) {
                        statements.push(extendsCall);
                        classBodyStart++;
                    }
                    // The next statement is the function declaration.
                    statements.push(funcStatements[classBodyStart]);
                    classBodyStart++;
                    // Add the class alias following the declaration.
                    statements.push(ts.createExpressionStatement(ts.createAssignment(aliasAssignment.left, ts.cast(variable.name, ts.isIdentifier))));
                }
                // Find the trailing 'return' statement (4)
                while (!ts.isReturnStatement(ts.elementAt(funcStatements, classBodyEnd))) {
                    classBodyEnd--;
                }
                // When we extract the statements of the inner IIFE, we exclude the 'return' statement (4)
                // as we already have one that has been introduced by the 'ts' transformer.
                ts.addRange(statements, funcStatements, classBodyStart, classBodyEnd);
                if (classBodyEnd < -1) {
                    // If there were any hoisted declarations following the return statement, we should
                    // append them.
                    ts.addRange(statements, funcStatements, classBodyEnd + 1);
                }
                // Add the remaining statements of the outer wrapper.
                ts.addRange(statements, remainingStatements);
                // The 'es2015' class transform may add an end-of-declaration marker. If so we will add it
                // after the remaining statements from the 'ts' transformer.
                ts.addRange(statements, classStatements, /*start*/ 1);
                // Recreate any outer parentheses or partially-emitted expressions to preserve source map
                // and comment locations.
                return ts.recreateOuterExpressions(node.expression, ts.recreateOuterExpressions(variable.initializer, ts.recreateOuterExpressions(aliasAssignment && aliasAssignment.right, ts.updateCall(call, ts.recreateOuterExpressions(call.expression, ts.updateFunctionExpression(func, 
                /*modifiers*/ undefined, 
                /*asteriskToken*/ undefined, 
                /*name*/ undefined, 
                /*typeParameters*/ undefined, func.parameters, 
                /*type*/ undefined, ts.updateBlock(func.body, statements))), 
                /*typeArguments*/ undefined, call.arguments))));
            }
            function visitImmediateSuperCallInBody(node) {
                return visitCallExpressionWithPotentialCapturedThisAssignment(node, /*assignToCapturedThis*/ false);
            }
            function visitCallExpressionWithPotentialCapturedThisAssignment(node, assignToCapturedThis) {
                // We are here either because SuperKeyword was used somewhere in the expression, or
                // because we contain a SpreadElementExpression.
                if (node.transformFlags & 8192 /* ContainsRestOrSpread */ ||
                    node.expression.kind === 102 /* SuperKeyword */ ||
                    ts.isSuperProperty(ts.skipOuterExpressions(node.expression))) {
                    var _a = ts.createCallBinding(node.expression, hoistVariableDeclaration), target = _a.target, thisArg = _a.thisArg;
                    if (node.expression.kind === 102 /* SuperKeyword */) {
                        ts.setEmitFlags(thisArg, 4 /* NoSubstitution */);
                    }
                    var resultingCall = void 0;
                    if (node.transformFlags & 8192 /* ContainsRestOrSpread */) {
                        // [source]
                        //      f(...a, b)
                        //      x.m(...a, b)
                        //      super(...a, b)
                        //      super.m(...a, b) // in static
                        //      super.m(...a, b) // in instance
                        //
                        // [output]
                        //      f.apply(void 0, a.concat([b]))
                        //      (_a = x).m.apply(_a, a.concat([b]))
                        //      _super.apply(this, a.concat([b]))
                        //      _super.m.apply(this, a.concat([b]))
                        //      _super.prototype.m.apply(this, a.concat([b]))
                        resultingCall = ts.createFunctionApply(ts.visitNode(target, callExpressionVisitor, ts.isExpression), node.expression.kind === 102 /* SuperKeyword */ ? thisArg : ts.visitNode(thisArg, visitor, ts.isExpression), transformAndSpreadElements(node.arguments, /*needsUniqueCopy*/ false, /*multiLine*/ false, /*hasTrailingComma*/ false));
                    }
                    else {
                        // [source]
                        //      super(a)
                        //      super.m(a) // in static
                        //      super.m(a) // in instance
                        //
                        // [output]
                        //      _super.call(this, a)
                        //      _super.m.call(this, a)
                        //      _super.prototype.m.call(this, a)
                        resultingCall = ts.createFunctionCall(ts.visitNode(target, callExpressionVisitor, ts.isExpression), node.expression.kind === 102 /* SuperKeyword */ ? thisArg : ts.visitNode(thisArg, visitor, ts.isExpression), ts.visitNodes(node.arguments, visitor, ts.isExpression), 
                        /*location*/ node);
                    }
                    if (node.expression.kind === 102 /* SuperKeyword */) {
                        var initializer = ts.createLogicalOr(resultingCall, createActualThis());
                        resultingCall = assignToCapturedThis
                            ? ts.createAssignment(ts.createFileLevelUniqueName("_this"), initializer)
                            : initializer;
                    }
                    return ts.setOriginalNode(resultingCall, node);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Visits a NewExpression that contains a spread element.
             *
             * @param node A NewExpression node.
             */
            function visitNewExpression(node) {
                if (ts.some(node.arguments, ts.isSpreadElement)) {
                    // We are here because we contain a SpreadElementExpression.
                    // [source]
                    //      new C(...a)
                    //
                    // [output]
                    //      new ((_a = C).bind.apply(_a, [void 0].concat(a)))()
                    var _a = ts.createCallBinding(ts.createPropertyAccess(node.expression, "bind"), hoistVariableDeclaration), target = _a.target, thisArg = _a.thisArg;
                    return ts.createNew(ts.createFunctionApply(ts.visitNode(target, visitor, ts.isExpression), thisArg, transformAndSpreadElements(ts.createNodeArray(__spreadArrays([ts.createVoidZero()], node.arguments)), /*needsUniqueCopy*/ false, /*multiLine*/ false, /*hasTrailingComma*/ false)), 
                    /*typeArguments*/ undefined, []);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Transforms an array of Expression nodes that contains a SpreadExpression.
             *
             * @param elements The array of Expression nodes.
             * @param needsUniqueCopy A value indicating whether to ensure that the result is a fresh array.
             * @param multiLine A value indicating whether the result should be emitted on multiple lines.
             */
            function transformAndSpreadElements(elements, needsUniqueCopy, multiLine, hasTrailingComma) {
                // [source]
                //      [a, ...b, c]
                //
                // [output (downlevelIteration)]
                //      __spread([a], b, [c])
                //
                // [output]
                //      __spreadArrays([a], b, [c])
                // Map spans of spread expressions into their expressions and spans of other
                // expressions into an array literal.
                var numElements = elements.length;
                var segments = ts.flatten(ts.spanMap(elements, partitionSpread, function (partition, visitPartition, _start, end) {
                    return visitPartition(partition, multiLine, hasTrailingComma && end === numElements);
                }));
                if (compilerOptions.downlevelIteration) {
                    if (segments.length === 1) {
                        var firstSegment = segments[0];
                        if (isCallToHelper(firstSegment, "___spread")) {
                            return segments[0];
                        }
                    }
                    return ts.createSpreadHelper(context, segments);
                }
                else {
                    if (segments.length === 1) {
                        var firstSegment = segments[0];
                        if (!needsUniqueCopy
                            || isPackedArrayLiteral(firstSegment)
                            || isCallToHelper(firstSegment, "___spreadArrays")) {
                            return segments[0];
                        }
                    }
                    return ts.createSpreadArraysHelper(context, segments);
                }
            }
            function isPackedElement(node) {
                return !ts.isOmittedExpression(node);
            }
            function isPackedArrayLiteral(node) {
                return ts.isArrayLiteralExpression(node) && ts.every(node.elements, isPackedElement);
            }
            function isCallToHelper(firstSegment, helperName) {
                return ts.isCallExpression(firstSegment)
                    && ts.isIdentifier(firstSegment.expression)
                    && (ts.getEmitFlags(firstSegment.expression) & 4096 /* HelperName */)
                    && firstSegment.expression.escapedText === helperName;
            }
            function partitionSpread(node) {
                return ts.isSpreadElement(node)
                    ? visitSpanOfSpreads
                    : visitSpanOfNonSpreads;
            }
            function visitSpanOfSpreads(chunk) {
                return ts.map(chunk, visitExpressionOfSpread);
            }
            function visitSpanOfNonSpreads(chunk, multiLine, hasTrailingComma) {
                return ts.createArrayLiteral(ts.visitNodes(ts.createNodeArray(chunk, hasTrailingComma), visitor, ts.isExpression), multiLine);
            }
            function visitSpreadElement(node) {
                return ts.visitNode(node.expression, visitor, ts.isExpression);
            }
            /**
             * Transforms the expression of a SpreadExpression node.
             *
             * @param node A SpreadExpression node.
             */
            function visitExpressionOfSpread(node) {
                return ts.visitNode(node.expression, visitor, ts.isExpression);
            }
            /**
             * Visits a template literal.
             *
             * @param node A template literal.
             */
            function visitTemplateLiteral(node) {
                return ts.setTextRange(ts.createLiteral(node.text), node);
            }
            /**
             * Visits a string literal with an extended unicode escape.
             *
             * @param node A string literal.
             */
            function visitStringLiteral(node) {
                if (node.hasExtendedUnicodeEscape) {
                    return ts.setTextRange(ts.createLiteral(node.text), node);
                }
                return node;
            }
            /**
             * Visits a binary or octal (ES6) numeric literal.
             *
             * @param node A string literal.
             */
            function visitNumericLiteral(node) {
                if (node.numericLiteralFlags & 384 /* BinaryOrOctalSpecifier */) {
                    return ts.setTextRange(ts.createNumericLiteral(node.text), node);
                }
                return node;
            }
            /**
             * Visits a TaggedTemplateExpression node.
             *
             * @param node A TaggedTemplateExpression node.
             */
            function visitTaggedTemplateExpression(node) {
                // Visit the tag expression
                var tag = ts.visitNode(node.tag, visitor, ts.isExpression);
                // Build up the template arguments and the raw and cooked strings for the template.
                // We start out with 'undefined' for the first argument and revisit later
                // to avoid walking over the template string twice and shifting all our arguments over after the fact.
                var templateArguments = [undefined];
                var cookedStrings = [];
                var rawStrings = [];
                var template = node.template;
                if (ts.isNoSubstitutionTemplateLiteral(template)) {
                    cookedStrings.push(ts.createLiteral(template.text));
                    rawStrings.push(getRawLiteral(template));
                }
                else {
                    cookedStrings.push(ts.createLiteral(template.head.text));
                    rawStrings.push(getRawLiteral(template.head));
                    for (var _i = 0, _a = template.templateSpans; _i < _a.length; _i++) {
                        var templateSpan = _a[_i];
                        cookedStrings.push(ts.createLiteral(templateSpan.literal.text));
                        rawStrings.push(getRawLiteral(templateSpan.literal));
                        templateArguments.push(ts.visitNode(templateSpan.expression, visitor, ts.isExpression));
                    }
                }
                var helperCall = createTemplateObjectHelper(context, ts.createArrayLiteral(cookedStrings), ts.createArrayLiteral(rawStrings));
                // Create a variable to cache the template object if we're in a module.
                // Do not do this in the global scope, as any variable we currently generate could conflict with
                // variables from outside of the current compilation. In the future, we can revisit this behavior.
                if (ts.isExternalModule(currentSourceFile)) {
                    var tempVar = ts.createUniqueName("templateObject");
                    recordTaggedTemplateString(tempVar);
                    templateArguments[0] = ts.createLogicalOr(tempVar, ts.createAssignment(tempVar, helperCall));
                }
                else {
                    templateArguments[0] = helperCall;
                }
                return ts.createCall(tag, /*typeArguments*/ undefined, templateArguments);
            }
            /**
             * Creates an ES5 compatible literal from an ES6 template literal.
             *
             * @param node The ES6 template literal.
             */
            function getRawLiteral(node) {
                // Find original source text, since we need to emit the raw strings of the tagged template.
                // The raw strings contain the (escaped) strings of what the user wrote.
                // Examples: `\n` is converted to "\\n", a template string with a newline to "\n".
                var text = node.rawText;
                if (text === undefined) {
                    text = ts.getSourceTextOfNodeFromSourceFile(currentSourceFile, node);
                    // text contains the original source, it will also contain quotes ("`"), dolar signs and braces ("${" and "}"),
                    // thus we need to remove those characters.
                    // First template piece starts with "`", others with "}"
                    // Last template piece ends with "`", others with "${"
                    var isLast = node.kind === 14 /* NoSubstitutionTemplateLiteral */ || node.kind === 17 /* TemplateTail */;
                    text = text.substring(1, text.length - (isLast ? 1 : 2));
                }
                // Newline normalization:
                // ES6 Spec 11.8.6.1 - Static Semantics of TV's and TRV's
                // <CR><LF> and <CR> LineTerminatorSequences are normalized to <LF> for both TV and TRV.
                text = text.replace(/\r\n?/g, "\n");
                return ts.setTextRange(ts.createLiteral(text), node);
            }
            /**
             * Visits a TemplateExpression node.
             *
             * @param node A TemplateExpression node.
             */
            function visitTemplateExpression(node) {
                var expressions = [];
                addTemplateHead(expressions, node);
                addTemplateSpans(expressions, node);
                // createAdd will check if each expression binds less closely than binary '+'.
                // If it does, it wraps the expression in parentheses. Otherwise, something like
                //    `abc${ 1 << 2 }`
                // becomes
                //    "abc" + 1 << 2 + ""
                // which is really
                //    ("abc" + 1) << (2 + "")
                // rather than
                //    "abc" + (1 << 2) + ""
                var expression = ts.reduceLeft(expressions, ts.createAdd);
                if (ts.nodeIsSynthesized(expression)) {
                    expression.pos = node.pos;
                    expression.end = node.end;
                }
                return expression;
            }
            /**
             * Gets a value indicating whether we need to include the head of a TemplateExpression.
             *
             * @param node A TemplateExpression node.
             */
            function shouldAddTemplateHead(node) {
                // If this expression has an empty head literal and the first template span has a non-empty
                // literal, then emitting the empty head literal is not necessary.
                //     `${ foo } and ${ bar }`
                // can be emitted as
                //     foo + " and " + bar
                // This is because it is only required that one of the first two operands in the emit
                // output must be a string literal, so that the other operand and all following operands
                // are forced into strings.
                //
                // If the first template span has an empty literal, then the head must still be emitted.
                //     `${ foo }${ bar }`
                // must still be emitted as
                //     "" + foo + bar
                // There is always atleast one templateSpan in this code path, since
                // NoSubstitutionTemplateLiterals are directly emitted via emitLiteral()
                ts.Debug.assert(node.templateSpans.length !== 0);
                return node.head.text.length !== 0 || node.templateSpans[0].literal.text.length === 0;
            }
            /**
             * Adds the head of a TemplateExpression to an array of expressions.
             *
             * @param expressions An array of expressions.
             * @param node A TemplateExpression node.
             */
            function addTemplateHead(expressions, node) {
                if (!shouldAddTemplateHead(node)) {
                    return;
                }
                expressions.push(ts.createLiteral(node.head.text));
            }
            /**
             * Visits and adds the template spans of a TemplateExpression to an array of expressions.
             *
             * @param expressions An array of expressions.
             * @param node A TemplateExpression node.
             */
            function addTemplateSpans(expressions, node) {
                for (var _i = 0, _a = node.templateSpans; _i < _a.length; _i++) {
                    var span = _a[_i];
                    expressions.push(ts.visitNode(span.expression, visitor, ts.isExpression));
                    // Only emit if the literal is non-empty.
                    // The binary '+' operator is left-associative, so the first string concatenation
                    // with the head will force the result up to this point to be a string.
                    // Emitting a '+ ""' has no semantic effect for middles and tails.
                    if (span.literal.text.length !== 0) {
                        expressions.push(ts.createLiteral(span.literal.text));
                    }
                }
            }
            /**
             * Visits the `super` keyword
             */
            function visitSuperKeyword(isExpressionOfCall) {
                return hierarchyFacts & 8 /* NonStaticClassElement */ && !isExpressionOfCall ? ts.createPropertyAccess(ts.createFileLevelUniqueName("_super"), "prototype") :
                    ts.createFileLevelUniqueName("_super");
            }
            function visitMetaProperty(node) {
                if (node.keywordToken === 99 /* NewKeyword */ && node.name.escapedText === "target") {
                    hierarchyFacts |= 16384 /* NewTarget */;
                    return ts.createFileLevelUniqueName("_newTarget");
                }
                return node;
            }
            /**
             * Called by the printer just before a node is printed.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to be printed.
             * @param emitCallback The callback used to emit the node.
             */
            function onEmitNode(hint, node, emitCallback) {
                if (enabledSubstitutions & 1 /* CapturedThis */ && ts.isFunctionLike(node)) {
                    // If we are tracking a captured `this`, keep track of the enclosing function.
                    var ancestorFacts = enterSubtree(16286 /* FunctionExcludes */, ts.getEmitFlags(node) & 8 /* CapturesThis */
                        ? 65 /* FunctionIncludes */ | 16 /* CapturesThis */
                        : 65 /* FunctionIncludes */);
                    previousOnEmitNode(hint, node, emitCallback);
                    exitSubtree(ancestorFacts, 0 /* None */, 0 /* None */);
                    return;
                }
                previousOnEmitNode(hint, node, emitCallback);
            }
            /**
             * Enables a more costly code path for substitutions when we determine a source file
             * contains block-scoped bindings (e.g. `let` or `const`).
             */
            function enableSubstitutionsForBlockScopedBindings() {
                if ((enabledSubstitutions & 2 /* BlockScopedBindings */) === 0) {
                    enabledSubstitutions |= 2 /* BlockScopedBindings */;
                    context.enableSubstitution(75 /* Identifier */);
                }
            }
            /**
             * Enables a more costly code path for substitutions when we determine a source file
             * contains a captured `this`.
             */
            function enableSubstitutionsForCapturedThis() {
                if ((enabledSubstitutions & 1 /* CapturedThis */) === 0) {
                    enabledSubstitutions |= 1 /* CapturedThis */;
                    context.enableSubstitution(104 /* ThisKeyword */);
                    context.enableEmitNotification(162 /* Constructor */);
                    context.enableEmitNotification(161 /* MethodDeclaration */);
                    context.enableEmitNotification(163 /* GetAccessor */);
                    context.enableEmitNotification(164 /* SetAccessor */);
                    context.enableEmitNotification(202 /* ArrowFunction */);
                    context.enableEmitNotification(201 /* FunctionExpression */);
                    context.enableEmitNotification(244 /* FunctionDeclaration */);
                }
            }
            /**
             * Hooks node substitutions.
             *
             * @param hint The context for the emitter.
             * @param node The node to substitute.
             */
            function onSubstituteNode(hint, node) {
                node = previousOnSubstituteNode(hint, node);
                if (hint === 1 /* Expression */) {
                    return substituteExpression(node);
                }
                if (ts.isIdentifier(node)) {
                    return substituteIdentifier(node);
                }
                return node;
            }
            /**
             * Hooks substitutions for non-expression identifiers.
             */
            function substituteIdentifier(node) {
                // Only substitute the identifier if we have enabled substitutions for block-scoped
                // bindings.
                if (enabledSubstitutions & 2 /* BlockScopedBindings */ && !ts.isInternalName(node)) {
                    var original = ts.getParseTreeNode(node, ts.isIdentifier);
                    if (original && isNameOfDeclarationWithCollidingName(original)) {
                        return ts.setTextRange(ts.getGeneratedNameForNode(original), node);
                    }
                }
                return node;
            }
            /**
             * Determines whether a name is the name of a declaration with a colliding name.
             * NOTE: This function expects to be called with an original source tree node.
             *
             * @param node An original source tree node.
             */
            function isNameOfDeclarationWithCollidingName(node) {
                switch (node.parent.kind) {
                    case 191 /* BindingElement */:
                    case 245 /* ClassDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 242 /* VariableDeclaration */:
                        return node.parent.name === node
                            && resolver.isDeclarationWithCollidingName(node.parent);
                }
                return false;
            }
            /**
             * Substitutes an expression.
             *
             * @param node An Expression node.
             */
            function substituteExpression(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                        return substituteExpressionIdentifier(node);
                    case 104 /* ThisKeyword */:
                        return substituteThisKeyword(node);
                }
                return node;
            }
            /**
             * Substitutes an expression identifier.
             *
             * @param node An Identifier node.
             */
            function substituteExpressionIdentifier(node) {
                if (enabledSubstitutions & 2 /* BlockScopedBindings */ && !ts.isInternalName(node)) {
                    var declaration = resolver.getReferencedDeclarationWithCollidingName(node);
                    if (declaration && !(ts.isClassLike(declaration) && isPartOfClassBody(declaration, node))) {
                        return ts.setTextRange(ts.getGeneratedNameForNode(ts.getNameOfDeclaration(declaration)), node);
                    }
                }
                return node;
            }
            function isPartOfClassBody(declaration, node) {
                var currentNode = ts.getParseTreeNode(node);
                if (!currentNode || currentNode === declaration || currentNode.end <= declaration.pos || currentNode.pos >= declaration.end) {
                    // if the node has no correlation to a parse tree node, its definitely not
                    // part of the body.
                    // if the node is outside of the document range of the declaration, its
                    // definitely not part of the body.
                    return false;
                }
                var blockScope = ts.getEnclosingBlockScopeContainer(declaration);
                while (currentNode) {
                    if (currentNode === blockScope || currentNode === declaration) {
                        // if we are in the enclosing block scope of the declaration, we are definitely
                        // not inside the class body.
                        return false;
                    }
                    if (ts.isClassElement(currentNode) && currentNode.parent === declaration) {
                        return true;
                    }
                    currentNode = currentNode.parent;
                }
                return false;
            }
            /**
             * Substitutes `this` when contained within an arrow function.
             *
             * @param node The ThisKeyword node.
             */
            function substituteThisKeyword(node) {
                if (enabledSubstitutions & 1 /* CapturedThis */
                    && hierarchyFacts & 16 /* CapturesThis */) {
                    return ts.setTextRange(ts.createFileLevelUniqueName("_this"), node);
                }
                return node;
            }
            function getClassMemberPrefix(node, member) {
                return ts.hasModifier(member, 32 /* Static */)
                    ? ts.getInternalName(node)
                    : ts.createPropertyAccess(ts.getInternalName(node), "prototype");
            }
            function hasSynthesizedDefaultSuperCall(constructor, hasExtendsClause) {
                if (!constructor || !hasExtendsClause) {
                    return false;
                }
                if (ts.some(constructor.parameters)) {
                    return false;
                }
                var statement = ts.firstOrUndefined(constructor.body.statements);
                if (!statement || !ts.nodeIsSynthesized(statement) || statement.kind !== 226 /* ExpressionStatement */) {
                    return false;
                }
                var statementExpression = statement.expression;
                if (!ts.nodeIsSynthesized(statementExpression) || statementExpression.kind !== 196 /* CallExpression */) {
                    return false;
                }
                var callTarget = statementExpression.expression;
                if (!ts.nodeIsSynthesized(callTarget) || callTarget.kind !== 102 /* SuperKeyword */) {
                    return false;
                }
                var callArgument = ts.singleOrUndefined(statementExpression.arguments);
                if (!callArgument || !ts.nodeIsSynthesized(callArgument) || callArgument.kind !== 213 /* SpreadElement */) {
                    return false;
                }
                var expression = callArgument.expression;
                return ts.isIdentifier(expression) && expression.escapedText === "arguments";
            }
        }
        ts.transformES2015 = transformES2015;
        function createExtendsHelper(context, name) {
            context.requestEmitHelper(ts.extendsHelper);
            return ts.createCall(ts.getUnscopedHelperName("__extends"), 
            /*typeArguments*/ undefined, [
                name,
                ts.createFileLevelUniqueName("_super")
            ]);
        }
        function createTemplateObjectHelper(context, cooked, raw) {
            context.requestEmitHelper(ts.templateObjectHelper);
            return ts.createCall(ts.getUnscopedHelperName("__makeTemplateObject"), 
            /*typeArguments*/ undefined, [
                cooked,
                raw
            ]);
        }
        ts.extendsHelper = {
            name: "typescript:extends",
            importName: "__extends",
            scoped: false,
            priority: 0,
            text: "\n            var __extends = (this && this.__extends) || (function () {\n                var extendStatics = function (d, b) {\n                    extendStatics = Object.setPrototypeOf ||\n                        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\n                        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };\n                    return extendStatics(d, b);\n                };\n\n                return function (d, b) {\n                    extendStatics(d, b);\n                    function __() { this.constructor = d; }\n                    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\n                };\n            })();"
        };
        ts.templateObjectHelper = {
            name: "typescript:makeTemplateObject",
            importName: "__makeTemplateObject",
            scoped: false,
            priority: 0,
            text: "\n            var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {\n                if (Object.defineProperty) { Object.defineProperty(cooked, \"raw\", { value: raw }); } else { cooked.raw = raw; }\n                return cooked;\n            };"
        };
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        /**
         * Transforms ES5 syntax into ES3 syntax.
         *
         * @param context Context and state information for the transformation.
         */
        function transformES5(context) {
            var compilerOptions = context.getCompilerOptions();
            // enable emit notification only if using --jsx preserve or react-native
            var previousOnEmitNode;
            var noSubstitution;
            if (compilerOptions.jsx === 1 /* Preserve */ || compilerOptions.jsx === 3 /* ReactNative */) {
                previousOnEmitNode = context.onEmitNode;
                context.onEmitNode = onEmitNode;
                context.enableEmitNotification(268 /* JsxOpeningElement */);
                context.enableEmitNotification(269 /* JsxClosingElement */);
                context.enableEmitNotification(267 /* JsxSelfClosingElement */);
                noSubstitution = [];
            }
            var previousOnSubstituteNode = context.onSubstituteNode;
            context.onSubstituteNode = onSubstituteNode;
            context.enableSubstitution(194 /* PropertyAccessExpression */);
            context.enableSubstitution(281 /* PropertyAssignment */);
            return ts.chainBundle(transformSourceFile);
            /**
             * Transforms an ES5 source file to ES3.
             *
             * @param node A SourceFile
             */
            function transformSourceFile(node) {
                return node;
            }
            /**
             * Called by the printer just before a node is printed.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to emit.
             * @param emitCallback A callback used to emit the node.
             */
            function onEmitNode(hint, node, emitCallback) {
                switch (node.kind) {
                    case 268 /* JsxOpeningElement */:
                    case 269 /* JsxClosingElement */:
                    case 267 /* JsxSelfClosingElement */:
                        var tagName = node.tagName;
                        noSubstitution[ts.getOriginalNodeId(tagName)] = true;
                        break;
                }
                previousOnEmitNode(hint, node, emitCallback);
            }
            /**
             * Hooks node substitutions.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to substitute.
             */
            function onSubstituteNode(hint, node) {
                if (node.id && noSubstitution && noSubstitution[node.id]) {
                    return previousOnSubstituteNode(hint, node);
                }
                node = previousOnSubstituteNode(hint, node);
                if (ts.isPropertyAccessExpression(node)) {
                    return substitutePropertyAccessExpression(node);
                }
                else if (ts.isPropertyAssignment(node)) {
                    return substitutePropertyAssignment(node);
                }
                return node;
            }
            /**
             * Substitutes a PropertyAccessExpression whose name is a reserved word.
             *
             * @param node A PropertyAccessExpression
             */
            function substitutePropertyAccessExpression(node) {
                if (ts.isPrivateIdentifier(node.name)) {
                    return node;
                }
                var literalName = trySubstituteReservedName(node.name);
                if (literalName) {
                    return ts.setTextRange(ts.createElementAccess(node.expression, literalName), node);
                }
                return node;
            }
            /**
             * Substitutes a PropertyAssignment whose name is a reserved word.
             *
             * @param node A PropertyAssignment
             */
            function substitutePropertyAssignment(node) {
                var literalName = ts.isIdentifier(node.name) && trySubstituteReservedName(node.name);
                if (literalName) {
                    return ts.updatePropertyAssignment(node, literalName, node.initializer);
                }
                return node;
            }
            /**
             * If an identifier name is a reserved word, returns a string literal for the name.
             *
             * @param name An Identifier
             */
            function trySubstituteReservedName(name) {
                var token = name.originalKeywordKind || (ts.nodeIsSynthesized(name) ? ts.stringToToken(ts.idText(name)) : undefined);
                if (token !== undefined && token >= 77 /* FirstReservedWord */ && token <= 112 /* LastReservedWord */) {
                    return ts.setTextRange(ts.createLiteral(name), name);
                }
                return undefined;
            }
        }
        ts.transformES5 = transformES5;
    })(ts || (ts = {}));
    // Transforms generator functions into a compatible ES5 representation with similar runtime
    // semantics. This is accomplished by first transforming the body of each generator
    // function into an intermediate representation that is the compiled into a JavaScript
    // switch statement.
    //
    // Many functions in this transformer will contain comments indicating the expected
    // intermediate representation. For illustrative purposes, the following intermediate
    // language is used to define this intermediate representation:
    //
    //  .nop                            - Performs no operation.
    //  .local NAME, ...                - Define local variable declarations.
    //  .mark LABEL                     - Mark the location of a label.
    //  .br LABEL                       - Jump to a label. If jumping out of a protected
    //                                    region, all .finally blocks are executed.
    //  .brtrue LABEL, (x)              - Jump to a label IIF the expression `x` is truthy.
    //                                    If jumping out of a protected region, all .finally
    //                                    blocks are executed.
    //  .brfalse LABEL, (x)             - Jump to a label IIF the expression `x` is falsey.
    //                                    If jumping out of a protected region, all .finally
    //                                    blocks are executed.
    //  .yield (x)                      - Yield the value of the optional expression `x`.
    //                                    Resume at the next label.
    //  .yieldstar (x)                  - Delegate yield to the value of the optional
    //                                    expression `x`. Resume at the next label.
    //                                    NOTE: `x` must be an Iterator, not an Iterable.
    //  .loop CONTINUE, BREAK           - Marks the beginning of a loop. Any "continue" or
    //                                    "break" abrupt completions jump to the CONTINUE or
    //                                    BREAK labels, respectively.
    //  .endloop                        - Marks the end of a loop.
    //  .with (x)                       - Marks the beginning of a WithStatement block, using
    //                                    the supplied expression.
    //  .endwith                        - Marks the end of a WithStatement.
    //  .switch                         - Marks the beginning of a SwitchStatement.
    //  .endswitch                      - Marks the end of a SwitchStatement.
    //  .labeled NAME                   - Marks the beginning of a LabeledStatement with the
    //                                    supplied name.
    //  .endlabeled                     - Marks the end of a LabeledStatement.
    //  .try TRY, CATCH, FINALLY, END   - Marks the beginning of a protected region, and the
    //                                    labels for each block.
    //  .catch (x)                      - Marks the beginning of a catch block.
    //  .finally                        - Marks the beginning of a finally block.
    //  .endfinally                     - Marks the end of a finally block.
    //  .endtry                         - Marks the end of a protected region.
    //  .throw (x)                      - Throws the value of the expression `x`.
    //  .return (x)                     - Returns the value of the expression `x`.
    //
    // In addition, the illustrative intermediate representation introduces some special
    // variables:
    //
    //  %sent%                          - Either returns the next value sent to the generator,
    //                                    returns the result of a delegated yield, or throws
    //                                    the exception sent to the generator.
    //  %error%                         - Returns the value of the current exception in a
    //                                    catch block.
    //
    // This intermediate representation is then compiled into JavaScript syntax. The resulting
    // compilation output looks something like the following:
    //
    //  function f() {
    //      var /*locals*/;
    //      /*functions*/
    //      return __generator(function (state) {
    //          switch (state.label) {
    //              /*cases per label*/
    //          }
    //      });
    //  }
    //
    // Each of the above instructions corresponds to JavaScript emit similar to the following:
    //
    //  .local NAME                   | var NAME;
    // -------------------------------|----------------------------------------------
    //  .mark LABEL                   | case LABEL:
    // -------------------------------|----------------------------------------------
    //  .br LABEL                     |     return [3 /*break*/, LABEL];
    // -------------------------------|----------------------------------------------
    //  .brtrue LABEL, (x)            |     if (x) return [3 /*break*/, LABEL];
    // -------------------------------|----------------------------------------------
    //  .brfalse LABEL, (x)           |     if (!(x)) return [3, /*break*/, LABEL];
    // -------------------------------|----------------------------------------------
    //  .yield (x)                    |     return [4 /*yield*/, x];
    //  .mark RESUME                  | case RESUME:
    //      a = %sent%;               |     a = state.sent();
    // -------------------------------|----------------------------------------------
    //  .yieldstar (x)                |     return [5 /*yield**/, x];
    //  .mark RESUME                  | case RESUME:
    //      a = %sent%;               |     a = state.sent();
    // -------------------------------|----------------------------------------------
    //  .with (_a)                    |     with (_a) {
    //      a();                      |         a();
    //                                |     }
    //                                |     state.label = LABEL;
    //  .mark LABEL                   | case LABEL:
    //                                |     with (_a) {
    //      b();                      |         b();
    //                                |     }
    //  .endwith                      |
    // -------------------------------|----------------------------------------------
    //                                | case 0:
    //                                |     state.trys = [];
    //                                | ...
    //  .try TRY, CATCH, FINALLY, END |
    //  .mark TRY                     | case TRY:
    //                                |     state.trys.push([TRY, CATCH, FINALLY, END]);
    //  .nop                          |
    //      a();                      |     a();
    //  .br END                       |     return [3 /*break*/, END];
    //  .catch (e)                    |
    //  .mark CATCH                   | case CATCH:
    //                                |     e = state.sent();
    //      b();                      |     b();
    //  .br END                       |     return [3 /*break*/, END];
    //  .finally                      |
    //  .mark FINALLY                 | case FINALLY:
    //      c();                      |     c();
    //  .endfinally                   |     return [7 /*endfinally*/];
    //  .endtry                       |
    //  .mark END                     | case END:
    /*@internal*/
    var ts;
    (function (ts) {
        var OpCode;
        (function (OpCode) {
            OpCode[OpCode["Nop"] = 0] = "Nop";
            OpCode[OpCode["Statement"] = 1] = "Statement";
            OpCode[OpCode["Assign"] = 2] = "Assign";
            OpCode[OpCode["Break"] = 3] = "Break";
            OpCode[OpCode["BreakWhenTrue"] = 4] = "BreakWhenTrue";
            OpCode[OpCode["BreakWhenFalse"] = 5] = "BreakWhenFalse";
            OpCode[OpCode["Yield"] = 6] = "Yield";
            OpCode[OpCode["YieldStar"] = 7] = "YieldStar";
            OpCode[OpCode["Return"] = 8] = "Return";
            OpCode[OpCode["Throw"] = 9] = "Throw";
            OpCode[OpCode["Endfinally"] = 10] = "Endfinally"; // Marks the end of a `finally` block
        })(OpCode || (OpCode = {}));
        // whether a generated code block is opening or closing at the current operation for a FunctionBuilder
        var BlockAction;
        (function (BlockAction) {
            BlockAction[BlockAction["Open"] = 0] = "Open";
            BlockAction[BlockAction["Close"] = 1] = "Close";
        })(BlockAction || (BlockAction = {}));
        // the kind for a generated code block in a FunctionBuilder
        var CodeBlockKind;
        (function (CodeBlockKind) {
            CodeBlockKind[CodeBlockKind["Exception"] = 0] = "Exception";
            CodeBlockKind[CodeBlockKind["With"] = 1] = "With";
            CodeBlockKind[CodeBlockKind["Switch"] = 2] = "Switch";
            CodeBlockKind[CodeBlockKind["Loop"] = 3] = "Loop";
            CodeBlockKind[CodeBlockKind["Labeled"] = 4] = "Labeled";
        })(CodeBlockKind || (CodeBlockKind = {}));
        // the state for a generated code exception block
        var ExceptionBlockState;
        (function (ExceptionBlockState) {
            ExceptionBlockState[ExceptionBlockState["Try"] = 0] = "Try";
            ExceptionBlockState[ExceptionBlockState["Catch"] = 1] = "Catch";
            ExceptionBlockState[ExceptionBlockState["Finally"] = 2] = "Finally";
            ExceptionBlockState[ExceptionBlockState["Done"] = 3] = "Done";
        })(ExceptionBlockState || (ExceptionBlockState = {}));
        // NOTE: changes to this enum should be reflected in the __generator helper.
        var Instruction;
        (function (Instruction) {
            Instruction[Instruction["Next"] = 0] = "Next";
            Instruction[Instruction["Throw"] = 1] = "Throw";
            Instruction[Instruction["Return"] = 2] = "Return";
            Instruction[Instruction["Break"] = 3] = "Break";
            Instruction[Instruction["Yield"] = 4] = "Yield";
            Instruction[Instruction["YieldStar"] = 5] = "YieldStar";
            Instruction[Instruction["Catch"] = 6] = "Catch";
            Instruction[Instruction["Endfinally"] = 7] = "Endfinally";
        })(Instruction || (Instruction = {}));
        function getInstructionName(instruction) {
            switch (instruction) {
                case 2 /* Return */: return "return";
                case 3 /* Break */: return "break";
                case 4 /* Yield */: return "yield";
                case 5 /* YieldStar */: return "yield*";
                case 7 /* Endfinally */: return "endfinally";
                default: return undefined; // TODO: GH#18217
            }
        }
        function transformGenerators(context) {
            var resumeLexicalEnvironment = context.resumeLexicalEnvironment, endLexicalEnvironment = context.endLexicalEnvironment, hoistFunctionDeclaration = context.hoistFunctionDeclaration, hoistVariableDeclaration = context.hoistVariableDeclaration;
            var compilerOptions = context.getCompilerOptions();
            var languageVersion = ts.getEmitScriptTarget(compilerOptions);
            var resolver = context.getEmitResolver();
            var previousOnSubstituteNode = context.onSubstituteNode;
            context.onSubstituteNode = onSubstituteNode;
            var renamedCatchVariables;
            var renamedCatchVariableDeclarations;
            var inGeneratorFunctionBody;
            var inStatementContainingYield;
            // The following three arrays store information about generated code blocks.
            // All three arrays are correlated by their index. This approach is used over allocating
            // objects to store the same information to avoid GC overhead.
            //
            var blocks; // Information about the code block
            var blockOffsets; // The operation offset at which a code block begins or ends
            var blockActions; // Whether the code block is opened or closed
            var blockStack; // A stack of currently open code blocks
            // Labels are used to mark locations in the code that can be the target of a Break (jump)
            // operation. These are translated into case clauses in a switch statement.
            // The following two arrays are correlated by their index. This approach is used over
            // allocating objects to store the same information to avoid GC overhead.
            //
            var labelOffsets; // The operation offset at which the label is defined.
            var labelExpressions; // The NumericLiteral nodes bound to each label.
            var nextLabelId = 1; // The next label id to use.
            // Operations store information about generated code for the function body. This
            // Includes things like statements, assignments, breaks (jumps), and yields.
            // The following three arrays are correlated by their index. This approach is used over
            // allocating objects to store the same information to avoid GC overhead.
            //
            var operations; // The operation to perform.
            var operationArguments; // The arguments to the operation.
            var operationLocations; // The source map location for the operation.
            var state; // The name of the state object used by the generator at runtime.
            // The following variables store information used by the `build` function:
            //
            var blockIndex = 0; // The index of the current block.
            var labelNumber = 0; // The current label number.
            var labelNumbers;
            var lastOperationWasAbrupt; // Indicates whether the last operation was abrupt (break/continue).
            var lastOperationWasCompletion; // Indicates whether the last operation was a completion (return/throw).
            var clauses; // The case clauses generated for labels.
            var statements; // The statements for the current label.
            var exceptionBlockStack; // A stack of containing exception blocks.
            var currentExceptionBlock; // The current exception block.
            var withBlockStack; // A stack containing `with` blocks.
            return ts.chainBundle(transformSourceFile);
            function transformSourceFile(node) {
                if (node.isDeclarationFile || (node.transformFlags & 512 /* ContainsGenerator */) === 0) {
                    return node;
                }
                var visited = ts.visitEachChild(node, visitor, context);
                ts.addEmitHelpers(visited, context.readEmitHelpers());
                return visited;
            }
            /**
             * Visits a node.
             *
             * @param node The node to visit.
             */
            function visitor(node) {
                var transformFlags = node.transformFlags;
                if (inStatementContainingYield) {
                    return visitJavaScriptInStatementContainingYield(node);
                }
                else if (inGeneratorFunctionBody) {
                    return visitJavaScriptInGeneratorFunctionBody(node);
                }
                else if (ts.isFunctionLikeDeclaration(node) && node.asteriskToken) {
                    return visitGenerator(node);
                }
                else if (transformFlags & 512 /* ContainsGenerator */) {
                    return ts.visitEachChild(node, visitor, context);
                }
                else {
                    return node;
                }
            }
            /**
             * Visits a node that is contained within a statement that contains yield.
             *
             * @param node The node to visit.
             */
            function visitJavaScriptInStatementContainingYield(node) {
                switch (node.kind) {
                    case 228 /* DoStatement */:
                        return visitDoStatement(node);
                    case 229 /* WhileStatement */:
                        return visitWhileStatement(node);
                    case 237 /* SwitchStatement */:
                        return visitSwitchStatement(node);
                    case 238 /* LabeledStatement */:
                        return visitLabeledStatement(node);
                    default:
                        return visitJavaScriptInGeneratorFunctionBody(node);
                }
            }
            /**
             * Visits a node that is contained within a generator function.
             *
             * @param node The node to visit.
             */
            function visitJavaScriptInGeneratorFunctionBody(node) {
                switch (node.kind) {
                    case 244 /* FunctionDeclaration */:
                        return visitFunctionDeclaration(node);
                    case 201 /* FunctionExpression */:
                        return visitFunctionExpression(node);
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        return visitAccessorDeclaration(node);
                    case 225 /* VariableStatement */:
                        return visitVariableStatement(node);
                    case 230 /* ForStatement */:
                        return visitForStatement(node);
                    case 231 /* ForInStatement */:
                        return visitForInStatement(node);
                    case 234 /* BreakStatement */:
                        return visitBreakStatement(node);
                    case 233 /* ContinueStatement */:
                        return visitContinueStatement(node);
                    case 235 /* ReturnStatement */:
                        return visitReturnStatement(node);
                    default:
                        if (node.transformFlags & 262144 /* ContainsYield */) {
                            return visitJavaScriptContainingYield(node);
                        }
                        else if (node.transformFlags & (512 /* ContainsGenerator */ | 1048576 /* ContainsHoistedDeclarationOrCompletion */)) {
                            return ts.visitEachChild(node, visitor, context);
                        }
                        else {
                            return node;
                        }
                }
            }
            /**
             * Visits a node that contains a YieldExpression.
             *
             * @param node The node to visit.
             */
            function visitJavaScriptContainingYield(node) {
                switch (node.kind) {
                    case 209 /* BinaryExpression */:
                        return visitBinaryExpression(node);
                    case 210 /* ConditionalExpression */:
                        return visitConditionalExpression(node);
                    case 212 /* YieldExpression */:
                        return visitYieldExpression(node);
                    case 192 /* ArrayLiteralExpression */:
                        return visitArrayLiteralExpression(node);
                    case 193 /* ObjectLiteralExpression */:
                        return visitObjectLiteralExpression(node);
                    case 195 /* ElementAccessExpression */:
                        return visitElementAccessExpression(node);
                    case 196 /* CallExpression */:
                        return visitCallExpression(node);
                    case 197 /* NewExpression */:
                        return visitNewExpression(node);
                    default:
                        return ts.visitEachChild(node, visitor, context);
                }
            }
            /**
             * Visits a generator function.
             *
             * @param node The node to visit.
             */
            function visitGenerator(node) {
                switch (node.kind) {
                    case 244 /* FunctionDeclaration */:
                        return visitFunctionDeclaration(node);
                    case 201 /* FunctionExpression */:
                        return visitFunctionExpression(node);
                    default:
                        return ts.Debug.failBadSyntaxKind(node);
                }
            }
            /**
             * Visits a function declaration.
             *
             * This will be called when one of the following conditions are met:
             * - The function declaration is a generator function.
             * - The function declaration is contained within the body of a generator function.
             *
             * @param node The node to visit.
             */
            function visitFunctionDeclaration(node) {
                // Currently, we only support generators that were originally async functions.
                if (node.asteriskToken) {
                    node = ts.setOriginalNode(ts.setTextRange(ts.createFunctionDeclaration(
                    /*decorators*/ undefined, node.modifiers, 
                    /*asteriskToken*/ undefined, node.name, 
                    /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                    /*type*/ undefined, transformGeneratorFunctionBody(node.body)), 
                    /*location*/ node), node);
                }
                else {
                    var savedInGeneratorFunctionBody = inGeneratorFunctionBody;
                    var savedInStatementContainingYield = inStatementContainingYield;
                    inGeneratorFunctionBody = false;
                    inStatementContainingYield = false;
                    node = ts.visitEachChild(node, visitor, context);
                    inGeneratorFunctionBody = savedInGeneratorFunctionBody;
                    inStatementContainingYield = savedInStatementContainingYield;
                }
                if (inGeneratorFunctionBody) {
                    // Function declarations in a generator function body are hoisted
                    // to the top of the lexical scope and elided from the current statement.
                    hoistFunctionDeclaration(node);
                    return undefined;
                }
                else {
                    return node;
                }
            }
            /**
             * Visits a function expression.
             *
             * This will be called when one of the following conditions are met:
             * - The function expression is a generator function.
             * - The function expression is contained within the body of a generator function.
             *
             * @param node The node to visit.
             */
            function visitFunctionExpression(node) {
                // Currently, we only support generators that were originally async functions.
                if (node.asteriskToken) {
                    node = ts.setOriginalNode(ts.setTextRange(ts.createFunctionExpression(
                    /*modifiers*/ undefined, 
                    /*asteriskToken*/ undefined, node.name, 
                    /*typeParameters*/ undefined, ts.visitParameterList(node.parameters, visitor, context), 
                    /*type*/ undefined, transformGeneratorFunctionBody(node.body)), 
                    /*location*/ node), node);
                }
                else {
                    var savedInGeneratorFunctionBody = inGeneratorFunctionBody;
                    var savedInStatementContainingYield = inStatementContainingYield;
                    inGeneratorFunctionBody = false;
                    inStatementContainingYield = false;
                    node = ts.visitEachChild(node, visitor, context);
                    inGeneratorFunctionBody = savedInGeneratorFunctionBody;
                    inStatementContainingYield = savedInStatementContainingYield;
                }
                return node;
            }
            /**
             * Visits a get or set accessor declaration.
             *
             * This will be called when one of the following conditions are met:
             * - The accessor is contained within the body of a generator function.
             *
             * @param node The node to visit.
             */
            function visitAccessorDeclaration(node) {
                var savedInGeneratorFunctionBody = inGeneratorFunctionBody;
                var savedInStatementContainingYield = inStatementContainingYield;
                inGeneratorFunctionBody = false;
                inStatementContainingYield = false;
                node = ts.visitEachChild(node, visitor, context);
                inGeneratorFunctionBody = savedInGeneratorFunctionBody;
                inStatementContainingYield = savedInStatementContainingYield;
                return node;
            }
            /**
             * Transforms the body of a generator function declaration.
             *
             * @param node The function body to transform.
             */
            function transformGeneratorFunctionBody(body) {
                // Save existing generator state
                var statements = [];
                var savedInGeneratorFunctionBody = inGeneratorFunctionBody;
                var savedInStatementContainingYield = inStatementContainingYield;
                var savedBlocks = blocks;
                var savedBlockOffsets = blockOffsets;
                var savedBlockActions = blockActions;
                var savedBlockStack = blockStack;
                var savedLabelOffsets = labelOffsets;
                var savedLabelExpressions = labelExpressions;
                var savedNextLabelId = nextLabelId;
                var savedOperations = operations;
                var savedOperationArguments = operationArguments;
                var savedOperationLocations = operationLocations;
                var savedState = state;
                // Initialize generator state
                inGeneratorFunctionBody = true;
                inStatementContainingYield = false;
                blocks = undefined;
                blockOffsets = undefined;
                blockActions = undefined;
                blockStack = undefined;
                labelOffsets = undefined;
                labelExpressions = undefined;
                nextLabelId = 1;
                operations = undefined;
                operationArguments = undefined;
                operationLocations = undefined;
                state = ts.createTempVariable(/*recordTempVariable*/ undefined);
                // Build the generator
                resumeLexicalEnvironment();
                var statementOffset = ts.addPrologue(statements, body.statements, /*ensureUseStrict*/ false, visitor);
                transformAndEmitStatements(body.statements, statementOffset);
                var buildResult = build();
                ts.insertStatementsAfterStandardPrologue(statements, endLexicalEnvironment());
                statements.push(ts.createReturn(buildResult));
                // Restore previous generator state
                inGeneratorFunctionBody = savedInGeneratorFunctionBody;
                inStatementContainingYield = savedInStatementContainingYield;
                blocks = savedBlocks;
                blockOffsets = savedBlockOffsets;
                blockActions = savedBlockActions;
                blockStack = savedBlockStack;
                labelOffsets = savedLabelOffsets;
                labelExpressions = savedLabelExpressions;
                nextLabelId = savedNextLabelId;
                operations = savedOperations;
                operationArguments = savedOperationArguments;
                operationLocations = savedOperationLocations;
                state = savedState;
                return ts.setTextRange(ts.createBlock(statements, body.multiLine), body);
            }
            /**
             * Visits a variable statement.
             *
             * This will be called when one of the following conditions are met:
             * - The variable statement is contained within the body of a generator function.
             *
             * @param node The node to visit.
             */
            function visitVariableStatement(node) {
                if (node.transformFlags & 262144 /* ContainsYield */) {
                    transformAndEmitVariableDeclarationList(node.declarationList);
                    return undefined;
                }
                else {
                    // Do not hoist custom prologues.
                    if (ts.getEmitFlags(node) & 1048576 /* CustomPrologue */) {
                        return node;
                    }
                    for (var _i = 0, _a = node.declarationList.declarations; _i < _a.length; _i++) {
                        var variable = _a[_i];
                        hoistVariableDeclaration(variable.name);
                    }
                    var variables = ts.getInitializedVariables(node.declarationList);
                    if (variables.length === 0) {
                        return undefined;
                    }
                    return ts.setSourceMapRange(ts.createExpressionStatement(ts.inlineExpressions(ts.map(variables, transformInitializedVariable))), node);
                }
            }
            /**
             * Visits a binary expression.
             *
             * This will be called when one of the following conditions are met:
             * - The node contains a YieldExpression.
             *
             * @param node The node to visit.
             */
            function visitBinaryExpression(node) {
                var assoc = ts.getExpressionAssociativity(node);
                switch (assoc) {
                    case 0 /* Left */:
                        return visitLeftAssociativeBinaryExpression(node);
                    case 1 /* Right */:
                        return visitRightAssociativeBinaryExpression(node);
                    default:
                        return ts.Debug.assertNever(assoc);
                }
            }
            /**
             * Visits a right-associative binary expression containing `yield`.
             *
             * @param node The node to visit.
             */
            function visitRightAssociativeBinaryExpression(node) {
                var left = node.left, right = node.right;
                if (containsYield(right)) {
                    var target = void 0;
                    switch (left.kind) {
                        case 194 /* PropertyAccessExpression */:
                            // [source]
                            //      a.b = yield;
                            //
                            // [intermediate]
                            //  .local _a
                            //      _a = a;
                            //  .yield resumeLabel
                            //  .mark resumeLabel
                            //      _a.b = %sent%;
                            target = ts.updatePropertyAccess(left, cacheExpression(ts.visitNode(left.expression, visitor, ts.isLeftHandSideExpression)), left.name);
                            break;
                        case 195 /* ElementAccessExpression */:
                            // [source]
                            //      a[b] = yield;
                            //
                            // [intermediate]
                            //  .local _a, _b
                            //      _a = a;
                            //      _b = b;
                            //  .yield resumeLabel
                            //  .mark resumeLabel
                            //      _a[_b] = %sent%;
                            target = ts.updateElementAccess(left, cacheExpression(ts.visitNode(left.expression, visitor, ts.isLeftHandSideExpression)), cacheExpression(ts.visitNode(left.argumentExpression, visitor, ts.isExpression)));
                            break;
                        default:
                            target = ts.visitNode(left, visitor, ts.isExpression);
                            break;
                    }
                    var operator = node.operatorToken.kind;
                    if (ts.isCompoundAssignment(operator)) {
                        return ts.setTextRange(ts.createAssignment(target, ts.setTextRange(ts.createBinary(cacheExpression(target), ts.getNonAssignmentOperatorForCompoundAssignment(operator), ts.visitNode(right, visitor, ts.isExpression)), node)), node);
                    }
                    else {
                        return ts.updateBinary(node, target, ts.visitNode(right, visitor, ts.isExpression));
                    }
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitLeftAssociativeBinaryExpression(node) {
                if (containsYield(node.right)) {
                    if (ts.isLogicalOperator(node.operatorToken.kind)) {
                        return visitLogicalBinaryExpression(node);
                    }
                    else if (node.operatorToken.kind === 27 /* CommaToken */) {
                        return visitCommaExpression(node);
                    }
                    // [source]
                    //      a() + (yield) + c()
                    //
                    // [intermediate]
                    //  .local _a
                    //      _a = a();
                    //  .yield resumeLabel
                    //      _a + %sent% + c()
                    var clone_6 = ts.getMutableClone(node);
                    clone_6.left = cacheExpression(ts.visitNode(node.left, visitor, ts.isExpression));
                    clone_6.right = ts.visitNode(node.right, visitor, ts.isExpression);
                    return clone_6;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Visits a logical binary expression containing `yield`.
             *
             * @param node A node to visit.
             */
            function visitLogicalBinaryExpression(node) {
                // Logical binary expressions (`&&` and `||`) are shortcutting expressions and need
                // to be transformed as such:
                //
                // [source]
                //      x = a() && yield;
                //
                // [intermediate]
                //  .local _a
                //      _a = a();
                //  .brfalse resultLabel, (_a)
                //  .yield resumeLabel
                //  .mark resumeLabel
                //      _a = %sent%;
                //  .mark resultLabel
                //      x = _a;
                //
                // [source]
                //      x = a() || yield;
                //
                // [intermediate]
                //  .local _a
                //      _a = a();
                //  .brtrue resultLabel, (_a)
                //  .yield resumeLabel
                //  .mark resumeLabel
                //      _a = %sent%;
                //  .mark resultLabel
                //      x = _a;
                var resultLabel = defineLabel();
                var resultLocal = declareLocal();
                emitAssignment(resultLocal, ts.visitNode(node.left, visitor, ts.isExpression), /*location*/ node.left);
                if (node.operatorToken.kind === 55 /* AmpersandAmpersandToken */) {
                    // Logical `&&` shortcuts when the left-hand operand is falsey.
                    emitBreakWhenFalse(resultLabel, resultLocal, /*location*/ node.left);
                }
                else {
                    // Logical `||` shortcuts when the left-hand operand is truthy.
                    emitBreakWhenTrue(resultLabel, resultLocal, /*location*/ node.left);
                }
                emitAssignment(resultLocal, ts.visitNode(node.right, visitor, ts.isExpression), /*location*/ node.right);
                markLabel(resultLabel);
                return resultLocal;
            }
            /**
             * Visits a comma expression containing `yield`.
             *
             * @param node The node to visit.
             */
            function visitCommaExpression(node) {
                // [source]
                //      x = a(), yield, b();
                //
                // [intermediate]
                //      a();
                //  .yield resumeLabel
                //  .mark resumeLabel
                //      x = %sent%, b();
                var pendingExpressions = [];
                visit(node.left);
                visit(node.right);
                return ts.inlineExpressions(pendingExpressions);
                function visit(node) {
                    if (ts.isBinaryExpression(node) && node.operatorToken.kind === 27 /* CommaToken */) {
                        visit(node.left);
                        visit(node.right);
                    }
                    else {
                        if (containsYield(node) && pendingExpressions.length > 0) {
                            emitWorker(1 /* Statement */, [ts.createExpressionStatement(ts.inlineExpressions(pendingExpressions))]);
                            pendingExpressions = [];
                        }
                        pendingExpressions.push(ts.visitNode(node, visitor, ts.isExpression));
                    }
                }
            }
            /**
             * Visits a conditional expression containing `yield`.
             *
             * @param node The node to visit.
             */
            function visitConditionalExpression(node) {
                // [source]
                //      x = a() ? yield : b();
                //
                // [intermediate]
                //  .local _a
                //  .brfalse whenFalseLabel, (a())
                //  .yield resumeLabel
                //  .mark resumeLabel
                //      _a = %sent%;
                //  .br resultLabel
                //  .mark whenFalseLabel
                //      _a = b();
                //  .mark resultLabel
                //      x = _a;
                // We only need to perform a specific transformation if a `yield` expression exists
                // in either the `whenTrue` or `whenFalse` branches.
                // A `yield` in the condition will be handled by the normal visitor.
                if (containsYield(node.whenTrue) || containsYield(node.whenFalse)) {
                    var whenFalseLabel = defineLabel();
                    var resultLabel = defineLabel();
                    var resultLocal = declareLocal();
                    emitBreakWhenFalse(whenFalseLabel, ts.visitNode(node.condition, visitor, ts.isExpression), /*location*/ node.condition);
                    emitAssignment(resultLocal, ts.visitNode(node.whenTrue, visitor, ts.isExpression), /*location*/ node.whenTrue);
                    emitBreak(resultLabel);
                    markLabel(whenFalseLabel);
                    emitAssignment(resultLocal, ts.visitNode(node.whenFalse, visitor, ts.isExpression), /*location*/ node.whenFalse);
                    markLabel(resultLabel);
                    return resultLocal;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            /**
             * Visits a `yield` expression.
             *
             * @param node The node to visit.
             */
            function visitYieldExpression(node) {
                // [source]
                //      x = yield a();
                //
                // [intermediate]
                //  .yield resumeLabel, (a())
                //  .mark resumeLabel
                //      x = %sent%;
                var resumeLabel = defineLabel();
                var expression = ts.visitNode(node.expression, visitor, ts.isExpression);
                if (node.asteriskToken) {
                    var iterator = (ts.getEmitFlags(node.expression) & 8388608 /* Iterator */) === 0
                        ? ts.createValuesHelper(context, expression, /*location*/ node)
                        : expression;
                    emitYieldStar(iterator, /*location*/ node);
                }
                else {
                    emitYield(expression, /*location*/ node);
                }
                markLabel(resumeLabel);
                return createGeneratorResume(/*location*/ node);
            }
            /**
             * Visits an ArrayLiteralExpression that contains a YieldExpression.
             *
             * @param node The node to visit.
             */
            function visitArrayLiteralExpression(node) {
                return visitElements(node.elements, /*leadingElement*/ undefined, /*location*/ undefined, node.multiLine);
            }
            /**
             * Visits an array of expressions containing one or more YieldExpression nodes
             * and returns an expression for the resulting value.
             *
             * @param elements The elements to visit.
             * @param multiLine Whether array literals created should be emitted on multiple lines.
             */
            function visitElements(elements, leadingElement, location, multiLine) {
                // [source]
                //      ar = [1, yield, 2];
                //
                // [intermediate]
                //  .local _a
                //      _a = [1];
                //  .yield resumeLabel
                //  .mark resumeLabel
                //      ar = _a.concat([%sent%, 2]);
                var numInitialElements = countInitialNodesWithoutYield(elements);
                var temp;
                if (numInitialElements > 0) {
                    temp = declareLocal();
                    var initialElements = ts.visitNodes(elements, visitor, ts.isExpression, 0, numInitialElements);
                    emitAssignment(temp, ts.createArrayLiteral(leadingElement
                        ? __spreadArrays([leadingElement], initialElements) : initialElements));
                    leadingElement = undefined;
                }
                var expressions = ts.reduceLeft(elements, reduceElement, [], numInitialElements);
                return temp
                    ? ts.createArrayConcat(temp, [ts.createArrayLiteral(expressions, multiLine)])
                    : ts.setTextRange(ts.createArrayLiteral(leadingElement ? __spreadArrays([leadingElement], expressions) : expressions, multiLine), location);
                function reduceElement(expressions, element) {
                    if (containsYield(element) && expressions.length > 0) {
                        var hasAssignedTemp = temp !== undefined;
                        if (!temp) {
                            temp = declareLocal();
                        }
                        emitAssignment(temp, hasAssignedTemp
                            ? ts.createArrayConcat(temp, [ts.createArrayLiteral(expressions, multiLine)])
                            : ts.createArrayLiteral(leadingElement ? __spreadArrays([leadingElement], expressions) : expressions, multiLine));
                        leadingElement = undefined;
                        expressions = [];
                    }
                    expressions.push(ts.visitNode(element, visitor, ts.isExpression));
                    return expressions;
                }
            }
            function visitObjectLiteralExpression(node) {
                // [source]
                //      o = {
                //          a: 1,
                //          b: yield,
                //          c: 2
                //      };
                //
                // [intermediate]
                //  .local _a
                //      _a = {
                //          a: 1
                //      };
                //  .yield resumeLabel
                //  .mark resumeLabel
                //      o = (_a.b = %sent%,
                //          _a.c = 2,
                //          _a);
                var properties = node.properties;
                var multiLine = node.multiLine;
                var numInitialProperties = countInitialNodesWithoutYield(properties);
                var temp = declareLocal();
                emitAssignment(temp, ts.createObjectLiteral(ts.visitNodes(properties, visitor, ts.isObjectLiteralElementLike, 0, numInitialProperties), multiLine));
                var expressions = ts.reduceLeft(properties, reduceProperty, [], numInitialProperties);
                expressions.push(multiLine ? ts.startOnNewLine(ts.getMutableClone(temp)) : temp);
                return ts.inlineExpressions(expressions);
                function reduceProperty(expressions, property) {
                    if (containsYield(property) && expressions.length > 0) {
                        emitStatement(ts.createExpressionStatement(ts.inlineExpressions(expressions)));
                        expressions = [];
                    }
                    var expression = ts.createExpressionForObjectLiteralElementLike(node, property, temp);
                    var visited = ts.visitNode(expression, visitor, ts.isExpression);
                    if (visited) {
                        if (multiLine) {
                            ts.startOnNewLine(visited);
                        }
                        expressions.push(visited);
                    }
                    return expressions;
                }
            }
            /**
             * Visits an ElementAccessExpression that contains a YieldExpression.
             *
             * @param node The node to visit.
             */
            function visitElementAccessExpression(node) {
                if (containsYield(node.argumentExpression)) {
                    // [source]
                    //      a = x[yield];
                    //
                    // [intermediate]
                    //  .local _a
                    //      _a = x;
                    //  .yield resumeLabel
                    //  .mark resumeLabel
                    //      a = _a[%sent%]
                    var clone_7 = ts.getMutableClone(node);
                    clone_7.expression = cacheExpression(ts.visitNode(node.expression, visitor, ts.isLeftHandSideExpression));
                    clone_7.argumentExpression = ts.visitNode(node.argumentExpression, visitor, ts.isExpression);
                    return clone_7;
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitCallExpression(node) {
                if (!ts.isImportCall(node) && ts.forEach(node.arguments, containsYield)) {
                    // [source]
                    //      a.b(1, yield, 2);
                    //
                    // [intermediate]
                    //  .local _a, _b, _c
                    //      _b = (_a = a).b;
                    //      _c = [1];
                    //  .yield resumeLabel
                    //  .mark resumeLabel
                    //      _b.apply(_a, _c.concat([%sent%, 2]));
                    var _a = ts.createCallBinding(node.expression, hoistVariableDeclaration, languageVersion, /*cacheIdentifiers*/ true), target = _a.target, thisArg = _a.thisArg;
                    return ts.setOriginalNode(ts.createFunctionApply(cacheExpression(ts.visitNode(target, visitor, ts.isLeftHandSideExpression)), thisArg, visitElements(node.arguments), 
                    /*location*/ node), node);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function visitNewExpression(node) {
                if (ts.forEach(node.arguments, containsYield)) {
                    // [source]
                    //      new a.b(1, yield, 2);
                    //
                    // [intermediate]
                    //  .local _a, _b, _c
                    //      _b = (_a = a.b).bind;
                    //      _c = [1];
                    //  .yield resumeLabel
                    //  .mark resumeLabel
                    //      new (_b.apply(_a, _c.concat([%sent%, 2])));
                    var _a = ts.createCallBinding(ts.createPropertyAccess(node.expression, "bind"), hoistVariableDeclaration), target = _a.target, thisArg = _a.thisArg;
                    return ts.setOriginalNode(ts.setTextRange(ts.createNew(ts.createFunctionApply(cacheExpression(ts.visitNode(target, visitor, ts.isExpression)), thisArg, visitElements(node.arguments, 
                    /*leadingElement*/ ts.createVoidZero())), 
                    /*typeArguments*/ undefined, []), node), node);
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function transformAndEmitStatements(statements, start) {
                if (start === void 0) {
                    start = 0;
                }
                var numStatements = statements.length;
                for (var i = start; i < numStatements; i++) {
                    transformAndEmitStatement(statements[i]);
                }
            }
            function transformAndEmitEmbeddedStatement(node) {
                if (ts.isBlock(node)) {
                    transformAndEmitStatements(node.statements);
                }
                else {
                    transformAndEmitStatement(node);
                }
            }
            function transformAndEmitStatement(node) {
                var savedInStatementContainingYield = inStatementContainingYield;
                if (!inStatementContainingYield) {
                    inStatementContainingYield = containsYield(node);
                }
                transformAndEmitStatementWorker(node);
                inStatementContainingYield = savedInStatementContainingYield;
            }
            function transformAndEmitStatementWorker(node) {
                switch (node.kind) {
                    case 223 /* Block */:
                        return transformAndEmitBlock(node);
                    case 226 /* ExpressionStatement */:
                        return transformAndEmitExpressionStatement(node);
                    case 227 /* IfStatement */:
                        return transformAndEmitIfStatement(node);
                    case 228 /* DoStatement */:
                        return transformAndEmitDoStatement(node);
                    case 229 /* WhileStatement */:
                        return transformAndEmitWhileStatement(node);
                    case 230 /* ForStatement */:
                        return transformAndEmitForStatement(node);
                    case 231 /* ForInStatement */:
                        return transformAndEmitForInStatement(node);
                    case 233 /* ContinueStatement */:
                        return transformAndEmitContinueStatement(node);
                    case 234 /* BreakStatement */:
                        return transformAndEmitBreakStatement(node);
                    case 235 /* ReturnStatement */:
                        return transformAndEmitReturnStatement(node);
                    case 236 /* WithStatement */:
                        return transformAndEmitWithStatement(node);
                    case 237 /* SwitchStatement */:
                        return transformAndEmitSwitchStatement(node);
                    case 238 /* LabeledStatement */:
                        return transformAndEmitLabeledStatement(node);
                    case 239 /* ThrowStatement */:
                        return transformAndEmitThrowStatement(node);
                    case 240 /* TryStatement */:
                        return transformAndEmitTryStatement(node);
                    default:
                        return emitStatement(ts.visitNode(node, visitor, ts.isStatement));
                }
            }
            function transformAndEmitBlock(node) {
                if (containsYield(node)) {
                    transformAndEmitStatements(node.statements);
                }
                else {
                    emitStatement(ts.visitNode(node, visitor, ts.isStatement));
                }
            }
            function transformAndEmitExpressionStatement(node) {
                emitStatement(ts.visitNode(node, visitor, ts.isStatement));
            }
            function transformAndEmitVariableDeclarationList(node) {
                for (var _i = 0, _a = node.declarations; _i < _a.length; _i++) {
                    var variable = _a[_i];
                    var name = ts.getSynthesizedClone(variable.name);
                    ts.setCommentRange(name, variable.name);
                    hoistVariableDeclaration(name);
                }
                var variables = ts.getInitializedVariables(node);
                var numVariables = variables.length;
                var variablesWritten = 0;
                var pendingExpressions = [];
                while (variablesWritten < numVariables) {
                    for (var i = variablesWritten; i < numVariables; i++) {
                        var variable = variables[i];
                        if (containsYield(variable.initializer) && pendingExpressions.length > 0) {
                            break;
                        }
                        pendingExpressions.push(transformInitializedVariable(variable));
                    }
                    if (pendingExpressions.length) {
                        emitStatement(ts.createExpressionStatement(ts.inlineExpressions(pendingExpressions)));
                        variablesWritten += pendingExpressions.length;
                        pendingExpressions = [];
                    }
                }
                return undefined;
            }
            function transformInitializedVariable(node) {
                return ts.setSourceMapRange(ts.createAssignment(ts.setSourceMapRange(ts.getSynthesizedClone(node.name), node.name), ts.visitNode(node.initializer, visitor, ts.isExpression)), node);
            }
            function transformAndEmitIfStatement(node) {
                if (containsYield(node)) {
                    // [source]
                    //      if (x)
                    //          /*thenStatement*/
                    //      else
                    //          /*elseStatement*/
                    //
                    // [intermediate]
                    //  .brfalse elseLabel, (x)
                    //      /*thenStatement*/
                    //  .br endLabel
                    //  .mark elseLabel
                    //      /*elseStatement*/
                    //  .mark endLabel
                    if (containsYield(node.thenStatement) || containsYield(node.elseStatement)) {
                        var endLabel = defineLabel();
                        var elseLabel = node.elseStatement ? defineLabel() : undefined;
                        emitBreakWhenFalse(node.elseStatement ? elseLabel : endLabel, ts.visitNode(node.expression, visitor, ts.isExpression), /*location*/ node.expression);
                        transformAndEmitEmbeddedStatement(node.thenStatement);
                        if (node.elseStatement) {
                            emitBreak(endLabel);
                            markLabel(elseLabel);
                            transformAndEmitEmbeddedStatement(node.elseStatement);
                        }
                        markLabel(endLabel);
                    }
                    else {
                        emitStatement(ts.visitNode(node, visitor, ts.isStatement));
                    }
                }
                else {
                    emitStatement(ts.visitNode(node, visitor, ts.isStatement));
                }
            }
            function transformAndEmitDoStatement(node) {
                if (containsYield(node)) {
                    // [source]
                    //      do {
                    //          /*body*/
                    //      }
                    //      while (i < 10);
                    //
                    // [intermediate]
                    //  .loop conditionLabel, endLabel
                    //  .mark loopLabel
                    //      /*body*/
                    //  .mark conditionLabel
                    //  .brtrue loopLabel, (i < 10)
                    //  .endloop
                    //  .mark endLabel
                    var conditionLabel = defineLabel();
                    var loopLabel = defineLabel();
                    beginLoopBlock(/*continueLabel*/ conditionLabel);
                    markLabel(loopLabel);
                    transformAndEmitEmbeddedStatement(node.statement);
                    markLabel(conditionLabel);
                    emitBreakWhenTrue(loopLabel, ts.visitNode(node.expression, visitor, ts.isExpression));
                    endLoopBlock();
                }
                else {
                    emitStatement(ts.visitNode(node, visitor, ts.isStatement));
                }
            }
            function visitDoStatement(node) {
                if (inStatementContainingYield) {
                    beginScriptLoopBlock();
                    node = ts.visitEachChild(node, visitor, context);
                    endLoopBlock();
                    return node;
                }
                else {
                    return ts.visitEachChild(node, visitor, context);
                }
            }
            function transformAndEmitWhileStatement(node) {
                if (containsYield(node)) {
                    // [source]
                    //      while (i < 10) {
                    //          /*body*/
                    //      }
                    //
                    // [intermediate]
                    //  .loop loopLabel, endLabel
                    //  .mark loopLabel
                    //  .brfalse endLabel, (i < 10)
                    //      /*body*/
                    //  .br loopLabel
                    //  .endloop
                    //  .mark endLabel
                    var loopLabel = defineLabel();
                    var endLabel = beginLoopBlock(loopLabel);
                    markLabel(loopLabel);
                    emitBreakWhenFalse(endLabel, ts.visitNode(node.expression, visitor, ts.isExpression));
                    transformAndEmitEmbeddedStatement(node.statement);
                    emitBreak(loopLabel);
                    endLoopBlock();
                }
                else {
                    emitStatement(ts.visitNode(node, visitor, ts.isStatement));
                }
            }
            function visitWhileStatement(node) {
                if (inStatementContainingYield) {
                    beginScriptLoopBlock();
                    node = ts.visitEachChild(node, visitor, context);
                    endLoopBlock();
                    return node;
                }
                else {
                    return ts.visitEachChild(node, visitor, context);
                }
            }
            function transformAndEmitForStatement(node) {
                if (containsYield(node)) {
                    // [source]
                    //      for (var i = 0; i < 10; i++) {
                    //          /*body*/
                    //      }
                    //
                    // [intermediate]
                    //  .local i
                    //      i = 0;
                    //  .loop incrementLabel, endLoopLabel
                    //  .mark conditionLabel
                    //  .brfalse endLoopLabel, (i < 10)
                    //      /*body*/
                    //  .mark incrementLabel
                    //      i++;
                    //  .br conditionLabel
                    //  .endloop
                    //  .mark endLoopLabel
                    var conditionLabel = defineLabel();
                    var incrementLabel = defineLabel();
                    var endLabel = beginLoopBlock(incrementLabel);
                    if (node.initializer) {
                        var initializer = node.initializer;
                        if (ts.isVariableDeclarationList(initializer)) {
                            transformAndEmitVariableDeclarationList(initializer);
                        }
                        else {
                            emitStatement(ts.setTextRange(ts.createExpressionStatement(ts.visitNode(initializer, visitor, ts.isExpression)), initializer));
                        }
                    }
                    markLabel(conditionLabel);
                    if (node.condition) {
                        emitBreakWhenFalse(endLabel, ts.visitNode(node.condition, visitor, ts.isExpression));
                    }
                    transformAndEmitEmbeddedStatement(node.statement);
                    markLabel(incrementLabel);
                    if (node.incrementor) {
                        emitStatement(ts.setTextRange(ts.createExpressionStatement(ts.visitNode(node.incrementor, visitor, ts.isExpression)), node.incrementor));
                    }
                    emitBreak(conditionLabel);
                    endLoopBlock();
                }
                else {
                    emitStatement(ts.visitNode(node, visitor, ts.isStatement));
                }
            }
            function visitForStatement(node) {
                if (inStatementContainingYield) {
                    beginScriptLoopBlock();
                }
                var initializer = node.initializer;
                if (initializer && ts.isVariableDeclarationList(initializer)) {
                    for (var _i = 0, _a = initializer.declarations; _i < _a.length; _i++) {
                        var variable = _a[_i];
                        hoistVariableDeclaration(variable.name);
                    }
                    var variables = ts.getInitializedVariables(initializer);
                    node = ts.updateFor(node, variables.length > 0
                        ? ts.inlineExpressions(ts.map(variables, transformInitializedVariable))
                        : undefined, ts.visitNode(node.condition, visitor, ts.isExpression), ts.visitNode(node.incrementor, visitor, ts.isExpression), ts.visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock));
                }
                else {
                    node = ts.visitEachChild(node, visitor, context);
                }
                if (inStatementContainingYield) {
                    endLoopBlock();
                }
                return node;
            }
            function transformAndEmitForInStatement(node) {
                // TODO(rbuckton): Source map locations
                if (containsYield(node)) {
                    // [source]
                    //      for (var p in o) {
                    //          /*body*/
                    //      }
                    //
                    // [intermediate]
                    //  .local _a, _b, _i
                    //      _a = [];
                    //      for (_b in o) _a.push(_b);
                    //      _i = 0;
                    //  .loop incrementLabel, endLoopLabel
                    //  .mark conditionLabel
                    //  .brfalse endLoopLabel, (_i < _a.length)
                    //      p = _a[_i];
                    //      /*body*/
                    //  .mark incrementLabel
                    //      _b++;
                    //  .br conditionLabel
                    //  .endloop
                    //  .mark endLoopLabel
                    var keysArray = declareLocal(); // _a
                    var key = declareLocal(); // _b
                    var keysIndex = ts.createLoopVariable(); // _i
                    var initializer = node.initializer;
                    hoistVariableDeclaration(keysIndex);
                    emitAssignment(keysArray, ts.createArrayLiteral());
                    emitStatement(ts.createForIn(key, ts.visitNode(node.expression, visitor, ts.isExpression), ts.createExpressionStatement(ts.createCall(ts.createPropertyAccess(keysArray, "push"), 
                    /*typeArguments*/ undefined, [key]))));
                    emitAssignment(keysIndex, ts.createLiteral(0));
                    var conditionLabel = defineLabel();
                    var incrementLabel = defineLabel();
                    var endLabel = beginLoopBlock(incrementLabel);
                    markLabel(conditionLabel);
                    emitBreakWhenFalse(endLabel, ts.createLessThan(keysIndex, ts.createPropertyAccess(keysArray, "length")));
                    var variable = void 0;
                    if (ts.isVariableDeclarationList(initializer)) {
                        for (var _i = 0, _a = initializer.declarations; _i < _a.length; _i++) {
                            var variable_1 = _a[_i];
                            hoistVariableDeclaration(variable_1.name);
                        }
                        variable = ts.getSynthesizedClone(initializer.declarations[0].name);
                    }
                    else {
                        variable = ts.visitNode(initializer, visitor, ts.isExpression);
                        ts.Debug.assert(ts.isLeftHandSideExpression(variable));
                    }
                    emitAssignment(variable, ts.createElementAccess(keysArray, keysIndex));
                    transformAndEmitEmbeddedStatement(node.statement);
                    markLabel(incrementLabel);
                    emitStatement(ts.createExpressionStatement(ts.createPostfixIncrement(keysIndex)));
                    emitBreak(conditionLabel);
                    endLoopBlock();
                }
                else {
                    emitStatement(ts.visitNode(node, visitor, ts.isStatement));
                }
            }
            function visitForInStatement(node) {
                // [source]
                //      for (var x in a) {
                //          /*body*/
                //      }
                //
                // [intermediate]
                //  .local x
                //  .loop
                //      for (x in a) {
                //          /*body*/
                //      }
                //  .endloop
                if (inStatementContainingYield) {
                    beginScriptLoopBlock();
                }
                var initializer = node.initializer;
                if (ts.isVariableDeclarationList(initializer)) {
                    for (var _i = 0, _a = initializer.declarations; _i < _a.length; _i++) {
                        var variable = _a[_i];
                        hoistVariableDeclaration(variable.name);
                    }
                    node = ts.updateForIn(node, initializer.declarations[0].name, ts.visitNode(node.expression, visitor, ts.isExpression), ts.visitNode(node.statement, visitor, ts.isStatement, ts.liftToBlock));
                }
                else {
                    node = ts.visitEachChild(node, visitor, context);
                }
                if (inStatementContainingYield) {
                    endLoopBlock();
                }
                return node;
            }
            function transformAndEmitContinueStatement(node) {
                var label = findContinueTarget(node.label ? ts.idText(node.label) : undefined);
                if (label > 0) {
                    emitBreak(label, /*location*/ node);
                }
                else {
                    // invalid continue without a containing loop. Leave the node as is, per #17875.
                    emitStatement(node);
                }
            }
            function visitContinueStatement(node) {
                if (inStatementContainingYield) {
                    var label = findContinueTarget(node.label && ts.idText(node.label));
                    if (label > 0) {
                        return createInlineBreak(label, /*location*/ node);
                    }
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function transformAndEmitBreakStatement(node) {
                var label = findBreakTarget(node.label ? ts.idText(node.label) : undefined);
                if (label > 0) {
                    emitBreak(label, /*location*/ node);
                }
                else {
                    // invalid break without a containing loop, switch, or labeled statement. Leave the node as is, per #17875.
                    emitStatement(node);
                }
            }
            function visitBreakStatement(node) {
                if (inStatementContainingYield) {
                    var label = findBreakTarget(node.label && ts.idText(node.label));
                    if (label > 0) {
                        return createInlineBreak(label, /*location*/ node);
                    }
                }
                return ts.visitEachChild(node, visitor, context);
            }
            function transformAndEmitReturnStatement(node) {
                emitReturn(ts.visitNode(node.expression, visitor, ts.isExpression), 
                /*location*/ node);
            }
            function visitReturnStatement(node) {
                return createInlineReturn(ts.visitNode(node.expression, visitor, ts.isExpression), 
                /*location*/ node);
            }
            function transformAndEmitWithStatement(node) {
                if (containsYield(node)) {
                    // [source]
                    //      with (x) {
                    //          /*body*/
                    //      }
                    //
                    // [intermediate]
                    //  .with (x)
                    //      /*body*/
                    //  .endwith
                    beginWithBlock(cacheExpression(ts.visitNode(node.expression, visitor, ts.isExpression)));
                    transformAndEmitEmbeddedStatement(node.statement);
                    endWithBlock();
                }
                else {
                    emitStatement(ts.visitNode(node, visitor, ts.isStatement));
                }
            }
            function transformAndEmitSwitchStatement(node) {
                if (containsYield(node.caseBlock)) {
                    // [source]
                    //      switch (x) {
                    //          case a:
                    //              /*caseStatements*/
                    //          case b:
                    //              /*caseStatements*/
                    //          default:
                    //              /*defaultStatements*/
                    //      }
                    //
                    // [intermediate]
                    //  .local _a
                    //  .switch endLabel
                    //      _a = x;
                    //      switch (_a) {
                    //          case a:
                    //  .br clauseLabels[0]
                    //      }
                    //      switch (_a) {
                    //          case b:
                    //  .br clauseLabels[1]
                    //      }
                    //  .br clauseLabels[2]
                    //  .mark clauseLabels[0]
                    //      /*caseStatements*/
                    //  .mark clauseLabels[1]
                    //      /*caseStatements*/
                    //  .mark clauseLabels[2]
                    //      /*caseStatements*/
                    //  .endswitch
                    //  .mark endLabel
                    var caseBlock = node.caseBlock;
                    var numClauses = caseBlock.clauses.length;
                    var endLabel = beginSwitchBlock();
                    var expression = cacheExpression(ts.visitNode(node.expression, visitor, ts.isExpression));
                    // Create labels for each clause and find the index of the first default clause.
                    var clauseLabels = [];
                    var defaultClauseIndex = -1;
                    for (var i = 0; i < numClauses; i++) {
                        var clause = caseBlock.clauses[i];
                        clauseLabels.push(defineLabel());
                        if (clause.kind === 278 /* DefaultClause */ && defaultClauseIndex === -1) {
                            defaultClauseIndex = i;
                        }
                    }
                    // Emit switch statements for each run of case clauses either from the first case
                    // clause or the next case clause with a `yield` in its expression, up to the next
                    // case clause with a `yield` in its expression.
                    var clausesWritten = 0;
                    var pendingClauses = [];
                    while (clausesWritten < numClauses) {
                        var defaultClausesSkipped = 0;
                        for (var i = clausesWritten; i < numClauses; i++) {
                            var clause = caseBlock.clauses[i];
                            if (clause.kind === 277 /* CaseClause */) {
                                if (containsYield(clause.expression) && pendingClauses.length > 0) {
                                    break;
                                }
                                pendingClauses.push(ts.createCaseClause(ts.visitNode(clause.expression, visitor, ts.isExpression), [
                                    createInlineBreak(clauseLabels[i], /*location*/ clause.expression)
                                ]));
                            }
                            else {
                                defaultClausesSkipped++;
                            }
                        }
                        if (pendingClauses.length) {
                            emitStatement(ts.createSwitch(expression, ts.createCaseBlock(pendingClauses)));
                            clausesWritten += pendingClauses.length;
                            pendingClauses = [];
                        }
                        if (defaultClausesSkipped > 0) {
                            clausesWritten += defaultClausesSkipped;
                            defaultClausesSkipped = 0;
                        }
                    }
                    if (defaultClauseIndex >= 0) {
                        emitBreak(clauseLabels[defaultClauseIndex]);
                    }
                    else {
                        emitBreak(endLabel);
                    }
                    for (var i = 0; i < numClauses; i++) {
                        markLabel(clauseLabels[i]);
                        transformAndEmitStatements(caseBlock.clauses[i].statements);
                    }
                    endSwitchBlock();
                }
                else {
                    emitStatement(ts.visitNode(node, visitor, ts.isStatement));
                }
            }
            function visitSwitchStatement(node) {
                if (inStatementContainingYield) {
                    beginScriptSwitchBlock();
                }
                node = ts.visitEachChild(node, visitor, context);
                if (inStatementContainingYield) {
                    endSwitchBlock();
                }
                return node;
            }
            function transformAndEmitLabeledStatement(node) {
                if (containsYield(node)) {
                    // [source]
                    //      x: {
                    //          /*body*/
                    //      }
                    //
                    // [intermediate]
                    //  .labeled "x", endLabel
                    //      /*body*/
                    //  .endlabeled
                    //  .mark endLabel
                    beginLabeledBlock(ts.idText(node.label));
                    transformAndEmitEmbeddedStatement(node.statement);
                    endLabeledBlock();
                }
                else {
                    emitStatement(ts.visitNode(node, visitor, ts.isStatement));
                }
            }
            function visitLabeledStatement(node) {
                if (inStatementContainingYield) {
                    beginScriptLabeledBlock(ts.idText(node.label));
                }
                node = ts.visitEachChild(node, visitor, context);
                if (inStatementContainingYield) {
                    endLabeledBlock();
                }
                return node;
            }
            function transformAndEmitThrowStatement(node) {
                emitThrow(ts.visitNode(node.expression, visitor, ts.isExpression), 
                /*location*/ node);
            }
            function transformAndEmitTryStatement(node) {
                if (containsYield(node)) {
                    // [source]
                    //      try {
                    //          /*tryBlock*/
                    //      }
                    //      catch (e) {
                    //          /*catchBlock*/
                    //      }
                    //      finally {
                    //          /*finallyBlock*/
                    //      }
                    //
                    // [intermediate]
                    //  .local _a
                    //  .try tryLabel, catchLabel, finallyLabel, endLabel
                    //  .mark tryLabel
                    //  .nop
                    //      /*tryBlock*/
                    //  .br endLabel
                    //  .catch
                    //  .mark catchLabel
                    //      _a = %error%;
                    //      /*catchBlock*/
                    //  .br endLabel
                    //  .finally
                    //  .mark finallyLabel
                    //      /*finallyBlock*/
                    //  .endfinally
                    //  .endtry
                    //  .mark endLabel
                    beginExceptionBlock();
                    transformAndEmitEmbeddedStatement(node.tryBlock);
                    if (node.catchClause) {
                        beginCatchBlock(node.catchClause.variableDeclaration); // TODO: GH#18217
                        transformAndEmitEmbeddedStatement(node.catchClause.block);
                    }
                    if (node.finallyBlock) {
                        beginFinallyBlock();
                        transformAndEmitEmbeddedStatement(node.finallyBlock);
                    }
                    endExceptionBlock();
                }
                else {
                    emitStatement(ts.visitEachChild(node, visitor, context));
                }
            }
            function containsYield(node) {
                return !!node && (node.transformFlags & 262144 /* ContainsYield */) !== 0;
            }
            function countInitialNodesWithoutYield(nodes) {
                var numNodes = nodes.length;
                for (var i = 0; i < numNodes; i++) {
                    if (containsYield(nodes[i])) {
                        return i;
                    }
                }
                return -1;
            }
            function onSubstituteNode(hint, node) {
                node = previousOnSubstituteNode(hint, node);
                if (hint === 1 /* Expression */) {
                    return substituteExpression(node);
                }
                return node;
            }
            function substituteExpression(node) {
                if (ts.isIdentifier(node)) {
                    return substituteExpressionIdentifier(node);
                }
                return node;
            }
            function substituteExpressionIdentifier(node) {
                if (!ts.isGeneratedIdentifier(node) && renamedCatchVariables && renamedCatchVariables.has(ts.idText(node))) {
                    var original = ts.getOriginalNode(node);
                    if (ts.isIdentifier(original) && original.parent) {
                        var declaration = resolver.getReferencedValueDeclaration(original);
                        if (declaration) {
                            var name = renamedCatchVariableDeclarations[ts.getOriginalNodeId(declaration)];
                            if (name) {
                                var clone_8 = ts.getMutableClone(name);
                                ts.setSourceMapRange(clone_8, node);
                                ts.setCommentRange(clone_8, node);
                                return clone_8;
                            }
                        }
                    }
                }
                return node;
            }
            function cacheExpression(node) {
                if (ts.isGeneratedIdentifier(node) || ts.getEmitFlags(node) & 4096 /* HelperName */) {
                    return node;
                }
                var temp = ts.createTempVariable(hoistVariableDeclaration);
                emitAssignment(temp, node, /*location*/ node);
                return temp;
            }
            function declareLocal(name) {
                var temp = name
                    ? ts.createUniqueName(name)
                    : ts.createTempVariable(/*recordTempVariable*/ undefined);
                hoistVariableDeclaration(temp);
                return temp;
            }
            /**
             * Defines a label, uses as the target of a Break operation.
             */
            function defineLabel() {
                if (!labelOffsets) {
                    labelOffsets = [];
                }
                var label = nextLabelId;
                nextLabelId++;
                labelOffsets[label] = -1;
                return label;
            }
            /**
             * Marks the current operation with the specified label.
             */
            function markLabel(label) {
                ts.Debug.assert(labelOffsets !== undefined, "No labels were defined.");
                labelOffsets[label] = operations ? operations.length : 0;
            }
            /**
             * Begins a block operation (With, Break/Continue, Try/Catch/Finally)
             *
             * @param block Information about the block.
             */
            function beginBlock(block) {
                if (!blocks) {
                    blocks = [];
                    blockActions = [];
                    blockOffsets = [];
                    blockStack = [];
                }
                var index = blockActions.length;
                blockActions[index] = 0 /* Open */;
                blockOffsets[index] = operations ? operations.length : 0;
                blocks[index] = block;
                blockStack.push(block);
                return index;
            }
            /**
             * Ends the current block operation.
             */
            function endBlock() {
                var block = peekBlock();
                if (block === undefined)
                    return ts.Debug.fail("beginBlock was never called.");
                var index = blockActions.length;
                blockActions[index] = 1 /* Close */;
                blockOffsets[index] = operations ? operations.length : 0;
                blocks[index] = block;
                blockStack.pop();
                return block;
            }
            /**
             * Gets the current open block.
             */
            function peekBlock() {
                return ts.lastOrUndefined(blockStack);
            }
            /**
             * Gets the kind of the current open block.
             */
            function peekBlockKind() {
                var block = peekBlock();
                return block && block.kind;
            }
            /**
             * Begins a code block for a generated `with` statement.
             *
             * @param expression An identifier representing expression for the `with` block.
             */
            function beginWithBlock(expression) {
                var startLabel = defineLabel();
                var endLabel = defineLabel();
                markLabel(startLabel);
                beginBlock({
                    kind: 1 /* With */,
                    expression: expression,
                    startLabel: startLabel,
                    endLabel: endLabel
                });
            }
            /**
             * Ends a code block for a generated `with` statement.
             */
            function endWithBlock() {
                ts.Debug.assert(peekBlockKind() === 1 /* With */);
                var block = endBlock();
                markLabel(block.endLabel);
            }
            /**
             * Begins a code block for a generated `try` statement.
             */
            function beginExceptionBlock() {
                var startLabel = defineLabel();
                var endLabel = defineLabel();
                markLabel(startLabel);
                beginBlock({
                    kind: 0 /* Exception */,
                    state: 0 /* Try */,
                    startLabel: startLabel,
                    endLabel: endLabel
                });
                emitNop();
                return endLabel;
            }
            /**
             * Enters the `catch` clause of a generated `try` statement.
             *
             * @param variable The catch variable.
             */
            function beginCatchBlock(variable) {
                ts.Debug.assert(peekBlockKind() === 0 /* Exception */);
                // generated identifiers should already be unique within a file
                var name;
                if (ts.isGeneratedIdentifier(variable.name)) {
                    name = variable.name;
                    hoistVariableDeclaration(variable.name);
                }
                else {
                    var text = ts.idText(variable.name);
                    name = declareLocal(text);
                    if (!renamedCatchVariables) {
                        renamedCatchVariables = ts.createMap();
                        renamedCatchVariableDeclarations = [];
                        context.enableSubstitution(75 /* Identifier */);
                    }
                    renamedCatchVariables.set(text, true);
                    renamedCatchVariableDeclarations[ts.getOriginalNodeId(variable)] = name;
                }
                var exception = peekBlock();
                ts.Debug.assert(exception.state < 1 /* Catch */);
                var endLabel = exception.endLabel;
                emitBreak(endLabel);
                var catchLabel = defineLabel();
                markLabel(catchLabel);
                exception.state = 1 /* Catch */;
                exception.catchVariable = name;
                exception.catchLabel = catchLabel;
                emitAssignment(name, ts.createCall(ts.createPropertyAccess(state, "sent"), /*typeArguments*/ undefined, []));
                emitNop();
            }
            /**
             * Enters the `finally` block of a generated `try` statement.
             */
            function beginFinallyBlock() {
                ts.Debug.assert(peekBlockKind() === 0 /* Exception */);
                var exception = peekBlock();
                ts.Debug.assert(exception.state < 2 /* Finally */);
                var endLabel = exception.endLabel;
                emitBreak(endLabel);
                var finallyLabel = defineLabel();
                markLabel(finallyLabel);
                exception.state = 2 /* Finally */;
                exception.finallyLabel = finallyLabel;
            }
            /**
             * Ends the code block for a generated `try` statement.
             */
            function endExceptionBlock() {
                ts.Debug.assert(peekBlockKind() === 0 /* Exception */);
                var exception = endBlock();
                var state = exception.state;
                if (state < 2 /* Finally */) {
                    emitBreak(exception.endLabel);
                }
                else {
                    emitEndfinally();
                }
                markLabel(exception.endLabel);
                emitNop();
                exception.state = 3 /* Done */;
            }
            /**
             * Begins a code block that supports `break` or `continue` statements that are defined in
             * the source tree and not from generated code.
             *
             * @param labelText Names from containing labeled statements.
             */
            function beginScriptLoopBlock() {
                beginBlock({
                    kind: 3 /* Loop */,
                    isScript: true,
                    breakLabel: -1,
                    continueLabel: -1
                });
            }
            /**
             * Begins a code block that supports `break` or `continue` statements that are defined in
             * generated code. Returns a label used to mark the operation to which to jump when a
             * `break` statement targets this block.
             *
             * @param continueLabel A Label used to mark the operation to which to jump when a
             *                      `continue` statement targets this block.
             */
            function beginLoopBlock(continueLabel) {
                var breakLabel = defineLabel();
                beginBlock({
                    kind: 3 /* Loop */,
                    isScript: false,
                    breakLabel: breakLabel,
                    continueLabel: continueLabel,
                });
                return breakLabel;
            }
            /**
             * Ends a code block that supports `break` or `continue` statements that are defined in
             * generated code or in the source tree.
             */
            function endLoopBlock() {
                ts.Debug.assert(peekBlockKind() === 3 /* Loop */);
                var block = endBlock();
                var breakLabel = block.breakLabel;
                if (!block.isScript) {
                    markLabel(breakLabel);
                }
            }
            /**
             * Begins a code block that supports `break` statements that are defined in the source
             * tree and not from generated code.
             *
             */
            function beginScriptSwitchBlock() {
                beginBlock({
                    kind: 2 /* Switch */,
                    isScript: true,
                    breakLabel: -1
                });
            }
            /**
             * Begins a code block that supports `break` statements that are defined in generated code.
             * Returns a label used to mark the operation to which to jump when a `break` statement
             * targets this block.
             */
            function beginSwitchBlock() {
                var breakLabel = defineLabel();
                beginBlock({
                    kind: 2 /* Switch */,
                    isScript: false,
                    breakLabel: breakLabel,
                });
                return breakLabel;
            }
            /**
             * Ends a code block that supports `break` statements that are defined in generated code.
             */
            function endSwitchBlock() {
                ts.Debug.assert(peekBlockKind() === 2 /* Switch */);
                var block = endBlock();
                var breakLabel = block.breakLabel;
                if (!block.isScript) {
                    markLabel(breakLabel);
                }
            }
            function beginScriptLabeledBlock(labelText) {
                beginBlock({
                    kind: 4 /* Labeled */,
                    isScript: true,
                    labelText: labelText,
                    breakLabel: -1
                });
            }
            function beginLabeledBlock(labelText) {
                var breakLabel = defineLabel();
                beginBlock({
                    kind: 4 /* Labeled */,
                    isScript: false,
                    labelText: labelText,
                    breakLabel: breakLabel
                });
            }
            function endLabeledBlock() {
                ts.Debug.assert(peekBlockKind() === 4 /* Labeled */);
                var block = endBlock();
                if (!block.isScript) {
                    markLabel(block.breakLabel);
                }
            }
            /**
             * Indicates whether the provided block supports `break` statements.
             *
             * @param block A code block.
             */
            function supportsUnlabeledBreak(block) {
                return block.kind === 2 /* Switch */
                    || block.kind === 3 /* Loop */;
            }
            /**
             * Indicates whether the provided block supports `break` statements with labels.
             *
             * @param block A code block.
             */
            function supportsLabeledBreakOrContinue(block) {
                return block.kind === 4 /* Labeled */;
            }
            /**
             * Indicates whether the provided block supports `continue` statements.
             *
             * @param block A code block.
             */
            function supportsUnlabeledContinue(block) {
                return block.kind === 3 /* Loop */;
            }
            function hasImmediateContainingLabeledBlock(labelText, start) {
                for (var j = start; j >= 0; j--) {
                    var containingBlock = blockStack[j];
                    if (supportsLabeledBreakOrContinue(containingBlock)) {
                        if (containingBlock.labelText === labelText) {
                            return true;
                        }
                    }
                    else {
                        break;
                    }
                }
                return false;
            }
            /**
             * Finds the label that is the target for a `break` statement.
             *
             * @param labelText An optional name of a containing labeled statement.
             */
            function findBreakTarget(labelText) {
                if (blockStack) {
                    if (labelText) {
                        for (var i = blockStack.length - 1; i >= 0; i--) {
                            var block = blockStack[i];
                            if (supportsLabeledBreakOrContinue(block) && block.labelText === labelText) {
                                return block.breakLabel;
                            }
                            else if (supportsUnlabeledBreak(block) && hasImmediateContainingLabeledBlock(labelText, i - 1)) {
                                return block.breakLabel;
                            }
                        }
                    }
                    else {
                        for (var i = blockStack.length - 1; i >= 0; i--) {
                            var block = blockStack[i];
                            if (supportsUnlabeledBreak(block)) {
                                return block.breakLabel;
                            }
                        }
                    }
                }
                return 0;
            }
            /**
             * Finds the label that is the target for a `continue` statement.
             *
             * @param labelText An optional name of a containing labeled statement.
             */
            function findContinueTarget(labelText) {
                if (blockStack) {
                    if (labelText) {
                        for (var i = blockStack.length - 1; i >= 0; i--) {
                            var block = blockStack[i];
                            if (supportsUnlabeledContinue(block) && hasImmediateContainingLabeledBlock(labelText, i - 1)) {
                                return block.continueLabel;
                            }
                        }
                    }
                    else {
                        for (var i = blockStack.length - 1; i >= 0; i--) {
                            var block = blockStack[i];
                            if (supportsUnlabeledContinue(block)) {
                                return block.continueLabel;
                            }
                        }
                    }
                }
                return 0;
            }
            /**
             * Creates an expression that can be used to indicate the value for a label.
             *
             * @param label A label.
             */
            function createLabel(label) {
                if (label !== undefined && label > 0) {
                    if (labelExpressions === undefined) {
                        labelExpressions = [];
                    }
                    var expression = ts.createLiteral(-1);
                    if (labelExpressions[label] === undefined) {
                        labelExpressions[label] = [expression];
                    }
                    else {
                        labelExpressions[label].push(expression);
                    }
                    return expression;
                }
                return ts.createOmittedExpression();
            }
            /**
             * Creates a numeric literal for the provided instruction.
             */
            function createInstruction(instruction) {
                var literal = ts.createLiteral(instruction);
                ts.addSyntheticTrailingComment(literal, 3 /* MultiLineCommentTrivia */, getInstructionName(instruction));
                return literal;
            }
            /**
             * Creates a statement that can be used indicate a Break operation to the provided label.
             *
             * @param label A label.
             * @param location An optional source map location for the statement.
             */
            function createInlineBreak(label, location) {
                ts.Debug.assertLessThan(0, label, "Invalid label");
                return ts.setTextRange(ts.createReturn(ts.createArrayLiteral([
                    createInstruction(3 /* Break */),
                    createLabel(label)
                ])), location);
            }
            /**
             * Creates a statement that can be used indicate a Return operation.
             *
             * @param expression The expression for the return statement.
             * @param location An optional source map location for the statement.
             */
            function createInlineReturn(expression, location) {
                return ts.setTextRange(ts.createReturn(ts.createArrayLiteral(expression
                    ? [createInstruction(2 /* Return */), expression]
                    : [createInstruction(2 /* Return */)])), location);
            }
            /**
             * Creates an expression that can be used to resume from a Yield operation.
             */
            function createGeneratorResume(location) {
                return ts.setTextRange(ts.createCall(ts.createPropertyAccess(state, "sent"), 
                /*typeArguments*/ undefined, []), location);
            }
            /**
             * Emits an empty instruction.
             */
            function emitNop() {
                emitWorker(0 /* Nop */);
            }
            /**
             * Emits a Statement.
             *
             * @param node A statement.
             */
            function emitStatement(node) {
                if (node) {
                    emitWorker(1 /* Statement */, [node]);
                }
                else {
                    emitNop();
                }
            }
            /**
             * Emits an Assignment operation.
             *
             * @param left The left-hand side of the assignment.
             * @param right The right-hand side of the assignment.
             * @param location An optional source map location for the assignment.
             */
            function emitAssignment(left, right, location) {
                emitWorker(2 /* Assign */, [left, right], location);
            }
            /**
             * Emits a Break operation to the specified label.
             *
             * @param label A label.
             * @param location An optional source map location for the assignment.
             */
            function emitBreak(label, location) {
                emitWorker(3 /* Break */, [label], location);
            }
            /**
             * Emits a Break operation to the specified label when a condition evaluates to a truthy
             * value at runtime.
             *
             * @param label A label.
             * @param condition The condition.
             * @param location An optional source map location for the assignment.
             */
            function emitBreakWhenTrue(label, condition, location) {
                emitWorker(4 /* BreakWhenTrue */, [label, condition], location);
            }
            /**
             * Emits a Break to the specified label when a condition evaluates to a falsey value at
             * runtime.
             *
             * @param label A label.
             * @param condition The condition.
             * @param location An optional source map location for the assignment.
             */
            function emitBreakWhenFalse(label, condition, location) {
                emitWorker(5 /* BreakWhenFalse */, [label, condition], location);
            }
            /**
             * Emits a YieldStar operation for the provided expression.
             *
             * @param expression An optional value for the yield operation.
             * @param location An optional source map location for the assignment.
             */
            function emitYieldStar(expression, location) {
                emitWorker(7 /* YieldStar */, [expression], location);
            }
            /**
             * Emits a Yield operation for the provided expression.
             *
             * @param expression An optional value for the yield operation.
             * @param location An optional source map location for the assignment.
             */
            function emitYield(expression, location) {
                emitWorker(6 /* Yield */, [expression], location);
            }
            /**
             * Emits a Return operation for the provided expression.
             *
             * @param expression An optional value for the operation.
             * @param location An optional source map location for the assignment.
             */
            function emitReturn(expression, location) {
                emitWorker(8 /* Return */, [expression], location);
            }
            /**
             * Emits a Throw operation for the provided expression.
             *
             * @param expression A value for the operation.
             * @param location An optional source map location for the assignment.
             */
            function emitThrow(expression, location) {
                emitWorker(9 /* Throw */, [expression], location);
            }
            /**
             * Emits an Endfinally operation. This is used to handle `finally` block semantics.
             */
            function emitEndfinally() {
                emitWorker(10 /* Endfinally */);
            }
            /**
             * Emits an operation.
             *
             * @param code The OpCode for the operation.
             * @param args The optional arguments for the operation.
             */
            function emitWorker(code, args, location) {
                if (operations === undefined) {
                    operations = [];
                    operationArguments = [];
                    operationLocations = [];
                }
                if (labelOffsets === undefined) {
                    // mark entry point
                    markLabel(defineLabel());
                }
                var operationIndex = operations.length;
                operations[operationIndex] = code;
                operationArguments[operationIndex] = args;
                operationLocations[operationIndex] = location;
            }
            /**
             * Builds the generator function body.
             */
            function build() {
                blockIndex = 0;
                labelNumber = 0;
                labelNumbers = undefined;
                lastOperationWasAbrupt = false;
                lastOperationWasCompletion = false;
                clauses = undefined;
                statements = undefined;
                exceptionBlockStack = undefined;
                currentExceptionBlock = undefined;
                withBlockStack = undefined;
                var buildResult = buildStatements();
                return createGeneratorHelper(context, ts.setEmitFlags(ts.createFunctionExpression(
                /*modifiers*/ undefined, 
                /*asteriskToken*/ undefined, 
                /*name*/ undefined, 
                /*typeParameters*/ undefined, [ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, state)], 
                /*type*/ undefined, ts.createBlock(buildResult, 
                /*multiLine*/ buildResult.length > 0)), 524288 /* ReuseTempVariableScope */));
            }
            /**
             * Builds the statements for the generator function body.
             */
            function buildStatements() {
                if (operations) {
                    for (var operationIndex = 0; operationIndex < operations.length; operationIndex++) {
                        writeOperation(operationIndex);
                    }
                    flushFinalLabel(operations.length);
                }
                else {
                    flushFinalLabel(0);
                }
                if (clauses) {
                    var labelExpression = ts.createPropertyAccess(state, "label");
                    var switchStatement = ts.createSwitch(labelExpression, ts.createCaseBlock(clauses));
                    return [ts.startOnNewLine(switchStatement)];
                }
                if (statements) {
                    return statements;
                }
                return [];
            }
            /**
             * Flush the current label and advance to a new label.
             */
            function flushLabel() {
                if (!statements) {
                    return;
                }
                appendLabel(/*markLabelEnd*/ !lastOperationWasAbrupt);
                lastOperationWasAbrupt = false;
                lastOperationWasCompletion = false;
                labelNumber++;
            }
            /**
             * Flush the final label of the generator function body.
             */
            function flushFinalLabel(operationIndex) {
                if (isFinalLabelReachable(operationIndex)) {
                    tryEnterLabel(operationIndex);
                    withBlockStack = undefined;
                    writeReturn(/*expression*/ undefined, /*operationLocation*/ undefined);
                }
                if (statements && clauses) {
                    appendLabel(/*markLabelEnd*/ false);
                }
                updateLabelExpressions();
            }
            /**
             * Tests whether the final label of the generator function body
             * is reachable by user code.
             */
            function isFinalLabelReachable(operationIndex) {
                // if the last operation was *not* a completion (return/throw) then
                // the final label is reachable.
                if (!lastOperationWasCompletion) {
                    return true;
                }
                // if there are no labels defined or referenced, then the final label is
                // not reachable.
                if (!labelOffsets || !labelExpressions) {
                    return false;
                }
                // if the label for this offset is referenced, then the final label
                // is reachable.
                for (var label = 0; label < labelOffsets.length; label++) {
                    if (labelOffsets[label] === operationIndex && labelExpressions[label]) {
                        return true;
                    }
                }
                return false;
            }
            /**
             * Appends a case clause for the last label and sets the new label.
             *
             * @param markLabelEnd Indicates that the transition between labels was a fall-through
             *                     from a previous case clause and the change in labels should be
             *                     reflected on the `state` object.
             */
            function appendLabel(markLabelEnd) {
                if (!clauses) {
                    clauses = [];
                }
                if (statements) {
                    if (withBlockStack) {
                        // The previous label was nested inside one or more `with` blocks, so we
                        // surround the statements in generated `with` blocks to create the same environment.
                        for (var i = withBlockStack.length - 1; i >= 0; i--) {
                            var withBlock = withBlockStack[i];
                            statements = [ts.createWith(withBlock.expression, ts.createBlock(statements))];
                        }
                    }
                    if (currentExceptionBlock) {
                        // The previous label was nested inside of an exception block, so we must
                        // indicate entry into a protected region by pushing the label numbers
                        // for each block in the protected region.
                        var startLabel = currentExceptionBlock.startLabel, catchLabel = currentExceptionBlock.catchLabel, finallyLabel = currentExceptionBlock.finallyLabel, endLabel = currentExceptionBlock.endLabel;
                        statements.unshift(ts.createExpressionStatement(ts.createCall(ts.createPropertyAccess(ts.createPropertyAccess(state, "trys"), "push"), 
                        /*typeArguments*/ undefined, [
                            ts.createArrayLiteral([
                                createLabel(startLabel),
                                createLabel(catchLabel),
                                createLabel(finallyLabel),
                                createLabel(endLabel)
                            ])
                        ])));
                        currentExceptionBlock = undefined;
                    }
                    if (markLabelEnd) {
                        // The case clause for the last label falls through to this label, so we
                        // add an assignment statement to reflect the change in labels.
                        statements.push(ts.createExpressionStatement(ts.createAssignment(ts.createPropertyAccess(state, "label"), ts.createLiteral(labelNumber + 1))));
                    }
                }
                clauses.push(ts.createCaseClause(ts.createLiteral(labelNumber), statements || []));
                statements = undefined;
            }
            /**
             * Tries to enter into a new label at the current operation index.
             */
            function tryEnterLabel(operationIndex) {
                if (!labelOffsets) {
                    return;
                }
                for (var label = 0; label < labelOffsets.length; label++) {
                    if (labelOffsets[label] === operationIndex) {
                        flushLabel();
                        if (labelNumbers === undefined) {
                            labelNumbers = [];
                        }
                        if (labelNumbers[labelNumber] === undefined) {
                            labelNumbers[labelNumber] = [label];
                        }
                        else {
                            labelNumbers[labelNumber].push(label);
                        }
                    }
                }
            }
            /**
             * Updates literal expressions for labels with actual label numbers.
             */
            function updateLabelExpressions() {
                if (labelExpressions !== undefined && labelNumbers !== undefined) {
                    for (var labelNumber_1 = 0; labelNumber_1 < labelNumbers.length; labelNumber_1++) {
                        var labels = labelNumbers[labelNumber_1];
                        if (labels !== undefined) {
                            for (var _i = 0, labels_1 = labels; _i < labels_1.length; _i++) {
                                var label = labels_1[_i];
                                var expressions = labelExpressions[label];
                                if (expressions !== undefined) {
                                    for (var _a = 0, expressions_1 = expressions; _a < expressions_1.length; _a++) {
                                        var expression = expressions_1[_a];
                                        expression.text = String(labelNumber_1);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            /**
             * Tries to enter or leave a code block.
             */
            function tryEnterOrLeaveBlock(operationIndex) {
                if (blocks) {
                    for (; blockIndex < blockActions.length && blockOffsets[blockIndex] <= operationIndex; blockIndex++) {
                        var block = blocks[blockIndex];
                        var blockAction = blockActions[blockIndex];
                        switch (block.kind) {
                            case 0 /* Exception */:
                                if (blockAction === 0 /* Open */) {
                                    if (!exceptionBlockStack) {
                                        exceptionBlockStack = [];
                                    }
                                    if (!statements) {
                                        statements = [];
                                    }
                                    exceptionBlockStack.push(currentExceptionBlock);
                                    currentExceptionBlock = block;
                                }
                                else if (blockAction === 1 /* Close */) {
                                    currentExceptionBlock = exceptionBlockStack.pop();
                                }
                                break;
                            case 1 /* With */:
                                if (blockAction === 0 /* Open */) {
                                    if (!withBlockStack) {
                                        withBlockStack = [];
                                    }
                                    withBlockStack.push(block);
                                }
                                else if (blockAction === 1 /* Close */) {
                                    withBlockStack.pop();
                                }
                                break;
                            // default: do nothing
                        }
                    }
                }
            }
            /**
             * Writes an operation as a statement to the current label's statement list.
             *
             * @param operation The OpCode of the operation
             */
            function writeOperation(operationIndex) {
                tryEnterLabel(operationIndex);
                tryEnterOrLeaveBlock(operationIndex);
                // early termination, nothing else to process in this label
                if (lastOperationWasAbrupt) {
                    return;
                }
                lastOperationWasAbrupt = false;
                lastOperationWasCompletion = false;
                var opcode = operations[operationIndex];
                if (opcode === 0 /* Nop */) {
                    return;
                }
                else if (opcode === 10 /* Endfinally */) {
                    return writeEndfinally();
                }
                var args = operationArguments[operationIndex];
                if (opcode === 1 /* Statement */) {
                    return writeStatement(args[0]);
                }
                var location = operationLocations[operationIndex];
                switch (opcode) {
                    case 2 /* Assign */:
                        return writeAssign(args[0], args[1], location);
                    case 3 /* Break */:
                        return writeBreak(args[0], location);
                    case 4 /* BreakWhenTrue */:
                        return writeBreakWhenTrue(args[0], args[1], location);
                    case 5 /* BreakWhenFalse */:
                        return writeBreakWhenFalse(args[0], args[1], location);
                    case 6 /* Yield */:
                        return writeYield(args[0], location);
                    case 7 /* YieldStar */:
                        return writeYieldStar(args[0], location);
                    case 8 /* Return */:
                        return writeReturn(args[0], location);
                    case 9 /* Throw */:
                        return writeThrow(args[0], location);
                }
            }
            /**
             * Writes a statement to the current label's statement list.
             *
             * @param statement A statement to write.
             */
            function writeStatement(statement) {
                if (statement) {
                    if (!statements) {
                        statements = [statement];
                    }
                    else {
                        statements.push(statement);
                    }
                }
            }
            /**
             * Writes an Assign operation to the current label's statement list.
             *
             * @param left The left-hand side of the assignment.
             * @param right The right-hand side of the assignment.
             * @param operationLocation The source map location for the operation.
             */
            function writeAssign(left, right, operationLocation) {
                writeStatement(ts.setTextRange(ts.createExpressionStatement(ts.createAssignment(left, right)), operationLocation));
            }
            /**
             * Writes a Throw operation to the current label's statement list.
             *
             * @param expression The value to throw.
             * @param operationLocation The source map location for the operation.
             */
            function writeThrow(expression, operationLocation) {
                lastOperationWasAbrupt = true;
                lastOperationWasCompletion = true;
                writeStatement(ts.setTextRange(ts.createThrow(expression), operationLocation));
            }
            /**
             * Writes a Return operation to the current label's statement list.
             *
             * @param expression The value to return.
             * @param operationLocation The source map location for the operation.
             */
            function writeReturn(expression, operationLocation) {
                lastOperationWasAbrupt = true;
                lastOperationWasCompletion = true;
                writeStatement(ts.setEmitFlags(ts.setTextRange(ts.createReturn(ts.createArrayLiteral(expression
                    ? [createInstruction(2 /* Return */), expression]
                    : [createInstruction(2 /* Return */)])), operationLocation), 384 /* NoTokenSourceMaps */));
            }
            /**
             * Writes a Break operation to the current label's statement list.
             *
             * @param label The label for the Break.
             * @param operationLocation The source map location for the operation.
             */
            function writeBreak(label, operationLocation) {
                lastOperationWasAbrupt = true;
                writeStatement(ts.setEmitFlags(ts.setTextRange(ts.createReturn(ts.createArrayLiteral([
                    createInstruction(3 /* Break */),
                    createLabel(label)
                ])), operationLocation), 384 /* NoTokenSourceMaps */));
            }
            /**
             * Writes a BreakWhenTrue operation to the current label's statement list.
             *
             * @param label The label for the Break.
             * @param condition The condition for the Break.
             * @param operationLocation The source map location for the operation.
             */
            function writeBreakWhenTrue(label, condition, operationLocation) {
                writeStatement(ts.setEmitFlags(ts.createIf(condition, ts.setEmitFlags(ts.setTextRange(ts.createReturn(ts.createArrayLiteral([
                    createInstruction(3 /* Break */),
                    createLabel(label)
                ])), operationLocation), 384 /* NoTokenSourceMaps */)), 1 /* SingleLine */));
            }
            /**
             * Writes a BreakWhenFalse operation to the current label's statement list.
             *
             * @param label The label for the Break.
             * @param condition The condition for the Break.
             * @param operationLocation The source map location for the operation.
             */
            function writeBreakWhenFalse(label, condition, operationLocation) {
                writeStatement(ts.setEmitFlags(ts.createIf(ts.createLogicalNot(condition), ts.setEmitFlags(ts.setTextRange(ts.createReturn(ts.createArrayLiteral([
                    createInstruction(3 /* Break */),
                    createLabel(label)
                ])), operationLocation), 384 /* NoTokenSourceMaps */)), 1 /* SingleLine */));
            }
            /**
             * Writes a Yield operation to the current label's statement list.
             *
             * @param expression The expression to yield.
             * @param operationLocation The source map location for the operation.
             */
            function writeYield(expression, operationLocation) {
                lastOperationWasAbrupt = true;
                writeStatement(ts.setEmitFlags(ts.setTextRange(ts.createReturn(ts.createArrayLiteral(expression
                    ? [createInstruction(4 /* Yield */), expression]
                    : [createInstruction(4 /* Yield */)])), operationLocation), 384 /* NoTokenSourceMaps */));
            }
            /**
             * Writes a YieldStar instruction to the current label's statement list.
             *
             * @param expression The expression to yield.
             * @param operationLocation The source map location for the operation.
             */
            function writeYieldStar(expression, operationLocation) {
                lastOperationWasAbrupt = true;
                writeStatement(ts.setEmitFlags(ts.setTextRange(ts.createReturn(ts.createArrayLiteral([
                    createInstruction(5 /* YieldStar */),
                    expression
                ])), operationLocation), 384 /* NoTokenSourceMaps */));
            }
            /**
             * Writes an Endfinally instruction to the current label's statement list.
             */
            function writeEndfinally() {
                lastOperationWasAbrupt = true;
                writeStatement(ts.createReturn(ts.createArrayLiteral([
                    createInstruction(7 /* Endfinally */)
                ])));
            }
        }
        ts.transformGenerators = transformGenerators;
        function createGeneratorHelper(context, body) {
            context.requestEmitHelper(ts.generatorHelper);
            return ts.createCall(ts.getUnscopedHelperName("__generator"), 
            /*typeArguments*/ undefined, [ts.createThis(), body]);
        }
        // The __generator helper is used by down-level transformations to emulate the runtime
        // semantics of an ES2015 generator function. When called, this helper returns an
        // object that implements the Iterator protocol, in that it has `next`, `return`, and
        // `throw` methods that step through the generator when invoked.
        //
        // parameters:
        //  @param thisArg  The value to use as the `this` binding for the transformed generator body.
        //  @param body     A function that acts as the transformed generator body.
        //
        // variables:
        //  _       Persistent state for the generator that is shared between the helper and the
        //          generator body. The state object has the following members:
        //            sent() - A method that returns or throws the current completion value.
        //            label  - The next point at which to resume evaluation of the generator body.
        //            trys   - A stack of protected regions (try/catch/finally blocks).
        //            ops    - A stack of pending instructions when inside of a finally block.
        //  f       A value indicating whether the generator is executing.
        //  y       An iterator to delegate for a yield*.
        //  t       A temporary variable that holds one of the following values (note that these
        //          cases do not overlap):
        //          - The completion value when resuming from a `yield` or `yield*`.
        //          - The error value for a catch block.
        //          - The current protected region (array of try/catch/finally/end labels).
        //          - The verb (`next`, `throw`, or `return` method) to delegate to the expression
        //            of a `yield*`.
        //          - The result of evaluating the verb delegated to the expression of a `yield*`.
        //
        // functions:
        //  verb(n)     Creates a bound callback to the `step` function for opcode `n`.
        //  step(op)    Evaluates opcodes in a generator body until execution is suspended or
        //              completed.
        //
        // The __generator helper understands a limited set of instructions:
        //  0: next(value?)     - Start or resume the generator with the specified value.
        //  1: throw(error)     - Resume the generator with an exception. If the generator is
        //                        suspended inside of one or more protected regions, evaluates
        //                        any intervening finally blocks between the current label and
        //                        the nearest catch block or function boundary. If uncaught, the
        //                        exception is thrown to the caller.
        //  2: return(value?)   - Resume the generator as if with a return. If the generator is
        //                        suspended inside of one or more protected regions, evaluates any
        //                        intervening finally blocks.
        //  3: break(label)     - Jump to the specified label. If the label is outside of the
        //                        current protected region, evaluates any intervening finally
        //                        blocks.
        //  4: yield(value?)    - Yield execution to the caller with an optional value. When
        //                        resumed, the generator will continue at the next label.
        //  5: yield*(value)    - Delegates evaluation to the supplied iterator. When
        //                        delegation completes, the generator will continue at the next
        //                        label.
        //  6: catch(error)     - Handles an exception thrown from within the generator body. If
        //                        the current label is inside of one or more protected regions,
        //                        evaluates any intervening finally blocks between the current
        //                        label and the nearest catch block or function boundary. If
        //                        uncaught, the exception is thrown to the caller.
        //  7: endfinally       - Ends a finally block, resuming the last instruction prior to
        //                        entering a finally block.
        //
        // For examples of how these are used, see the comments in ./transformers/generators.ts
        ts.generatorHelper = {
            name: "typescript:generator",
            importName: "__generator",
            scoped: false,
            priority: 6,
            text: "\n            var __generator = (this && this.__generator) || function (thisArg, body) {\n                var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;\n                return g = { next: verb(0), \"throw\": verb(1), \"return\": verb(2) }, typeof Symbol === \"function\" && (g[Symbol.iterator] = function() { return this; }), g;\n                function verb(n) { return function (v) { return step([n, v]); }; }\n                function step(op) {\n                    if (f) throw new TypeError(\"Generator is already executing.\");\n                    while (_) try {\n                        if (f = 1, y && (t = op[0] & 2 ? y[\"return\"] : op[0] ? y[\"throw\"] || ((t = y[\"return\"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;\n                        if (y = 0, t) op = [op[0] & 2, t.value];\n                        switch (op[0]) {\n                            case 0: case 1: t = op; break;\n                            case 4: _.label++; return { value: op[1], done: false };\n                            case 5: _.label++; y = op[1]; op = [0]; continue;\n                            case 7: op = _.ops.pop(); _.trys.pop(); continue;\n                            default:\n                                if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }\n                                if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }\n                                if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }\n                                if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }\n                                if (t[2]) _.ops.pop();\n                                _.trys.pop(); continue;\n                        }\n                        op = body.call(thisArg, _);\n                    } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }\n                    if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };\n                }\n            };"
        };
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        function transformModule(context) {
            function getTransformModuleDelegate(moduleKind) {
                switch (moduleKind) {
                    case ts.ModuleKind.AMD: return transformAMDModule;
                    case ts.ModuleKind.UMD: return transformUMDModule;
                    default: return transformCommonJSModule;
                }
            }
            var startLexicalEnvironment = context.startLexicalEnvironment, endLexicalEnvironment = context.endLexicalEnvironment, hoistVariableDeclaration = context.hoistVariableDeclaration;
            var compilerOptions = context.getCompilerOptions();
            var resolver = context.getEmitResolver();
            var host = context.getEmitHost();
            var languageVersion = ts.getEmitScriptTarget(compilerOptions);
            var moduleKind = ts.getEmitModuleKind(compilerOptions);
            var previousOnSubstituteNode = context.onSubstituteNode;
            var previousOnEmitNode = context.onEmitNode;
            context.onSubstituteNode = onSubstituteNode;
            context.onEmitNode = onEmitNode;
            context.enableSubstitution(75 /* Identifier */); // Substitutes expression identifiers with imported/exported symbols.
            context.enableSubstitution(209 /* BinaryExpression */); // Substitutes assignments to exported symbols.
            context.enableSubstitution(207 /* PrefixUnaryExpression */); // Substitutes updates to exported symbols.
            context.enableSubstitution(208 /* PostfixUnaryExpression */); // Substitutes updates to exported symbols.
            context.enableSubstitution(282 /* ShorthandPropertyAssignment */); // Substitutes shorthand property assignments for imported/exported symbols.
            context.enableEmitNotification(290 /* SourceFile */); // Restore state when substituting nodes in a file.
            var moduleInfoMap = []; // The ExternalModuleInfo for each file.
            var deferredExports = []; // Exports to defer until an EndOfDeclarationMarker is found.
            var currentSourceFile; // The current file.
            var currentModuleInfo; // The ExternalModuleInfo for the current file.
            var noSubstitution; // Set of nodes for which substitution rules should be ignored.
            var needUMDDynamicImportHelper;
            return ts.chainBundle(transformSourceFile);
            /**
             * Transforms the module aspects of a SourceFile.
             *
             * @param node The SourceFile node.
             */
            function transformSourceFile(node) {
                if (node.isDeclarationFile ||
                    !(ts.isEffectiveExternalModule(node, compilerOptions) ||
                        node.transformFlags & 2097152 /* ContainsDynamicImport */ ||
                        (ts.isJsonSourceFile(node) && ts.hasJsonModuleEmitEnabled(compilerOptions) && (compilerOptions.out || compilerOptions.outFile)))) {
                    return node;
                }
                currentSourceFile = node;
                currentModuleInfo = ts.collectExternalModuleInfo(node, resolver, compilerOptions);
                moduleInfoMap[ts.getOriginalNodeId(node)] = currentModuleInfo;
                // Perform the transformation.
                var transformModule = getTransformModuleDelegate(moduleKind);
                var updated = transformModule(node);
                currentSourceFile = undefined;
                currentModuleInfo = undefined;
                needUMDDynamicImportHelper = false;
                return ts.aggregateTransformFlags(updated);
            }
            function shouldEmitUnderscoreUnderscoreESModule() {
                if (!currentModuleInfo.exportEquals && ts.isExternalModule(currentSourceFile)) {
                    return true;
                }
                return false;
            }
            /**
             * Transforms a SourceFile into a CommonJS module.
             *
             * @param node The SourceFile node.
             */
            function transformCommonJSModule(node) {
                startLexicalEnvironment();
                var statements = [];
                var ensureUseStrict = ts.getStrictOptionValue(compilerOptions, "alwaysStrict") || (!compilerOptions.noImplicitUseStrict && ts.isExternalModule(currentSourceFile));
                var statementOffset = ts.addPrologue(statements, node.statements, ensureUseStrict && !ts.isJsonSourceFile(node), sourceElementVisitor);
                if (shouldEmitUnderscoreUnderscoreESModule()) {
                    ts.append(statements, createUnderscoreUnderscoreESModule());
                }
                ts.append(statements, ts.visitNode(currentModuleInfo.externalHelpersImportDeclaration, sourceElementVisitor, ts.isStatement));
                ts.addRange(statements, ts.visitNodes(node.statements, sourceElementVisitor, ts.isStatement, statementOffset));
                addExportEqualsIfNeeded(statements, /*emitAsReturn*/ false);
                ts.insertStatementsAfterStandardPrologue(statements, endLexicalEnvironment());
                var updated = ts.updateSourceFileNode(node, ts.setTextRange(ts.createNodeArray(statements), node.statements));
                if (currentModuleInfo.hasExportStarsToExportValues && !compilerOptions.importHelpers) {
                    // If we have any `export * from ...` declarations
                    // we need to inform the emitter to add the __export helper.
                    ts.addEmitHelper(updated, exportStarHelper);
                }
                ts.addEmitHelpers(updated, context.readEmitHelpers());
                return updated;
            }
            /**
             * Transforms a SourceFile into an AMD module.
             *
             * @param node The SourceFile node.
             */
            function transformAMDModule(node) {
                var define = ts.createIdentifier("define");
                var moduleName = ts.tryGetModuleNameFromFile(node, host, compilerOptions);
                var jsonSourceFile = ts.isJsonSourceFile(node) && node;
                // An AMD define function has the following shape:
                //
                //     define(id?, dependencies?, factory);
                //
                // This has the shape of the following:
                //
                //     define(name, ["module1", "module2"], function (module1Alias) { ... }
                //
                // The location of the alias in the parameter list in the factory function needs to
                // match the position of the module name in the dependency list.
                //
                // To ensure this is true in cases of modules with no aliases, e.g.:
                //
                //     import "module"
                //
                // or
                //
                //     /// <amd-dependency path= "a.css" />
                //
                // we need to add modules without alias names to the end of the dependencies list
                var _a = collectAsynchronousDependencies(node, /*includeNonAmdDependencies*/ true), aliasedModuleNames = _a.aliasedModuleNames, unaliasedModuleNames = _a.unaliasedModuleNames, importAliasNames = _a.importAliasNames;
                // Create an updated SourceFile:
                //
                //     define(moduleName?, ["module1", "module2"], function ...
                var updated = ts.updateSourceFileNode(node, ts.setTextRange(ts.createNodeArray([
                    ts.createExpressionStatement(ts.createCall(define, 
                    /*typeArguments*/ undefined, __spreadArrays((moduleName ? [moduleName] : []), [
                        // Add the dependency array argument:
                        //
                        //     ["require", "exports", module1", "module2", ...]
                        ts.createArrayLiteral(jsonSourceFile ? ts.emptyArray : __spreadArrays([
                            ts.createLiteral("require"),
                            ts.createLiteral("exports")
                        ], aliasedModuleNames, unaliasedModuleNames)),
                        // Add the module body function argument:
                        //
                        //     function (require, exports, module1, module2) ...
                        jsonSourceFile ?
                            jsonSourceFile.statements.length ? jsonSourceFile.statements[0].expression : ts.createObjectLiteral() :
                            ts.createFunctionExpression(
                            /*modifiers*/ undefined, 
                            /*asteriskToken*/ undefined, 
                            /*name*/ undefined, 
                            /*typeParameters*/ undefined, __spreadArrays([
                                ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, "require"),
                                ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, "exports")
                            ], importAliasNames), 
                            /*type*/ undefined, transformAsynchronousModuleBody(node))
                    ])))
                ]), 
                /*location*/ node.statements));
                ts.addEmitHelpers(updated, context.readEmitHelpers());
                return updated;
            }
            /**
             * Transforms a SourceFile into a UMD module.
             *
             * @param node The SourceFile node.
             */
            function transformUMDModule(node) {
                var _a = collectAsynchronousDependencies(node, /*includeNonAmdDependencies*/ false), aliasedModuleNames = _a.aliasedModuleNames, unaliasedModuleNames = _a.unaliasedModuleNames, importAliasNames = _a.importAliasNames;
                var moduleName = ts.tryGetModuleNameFromFile(node, host, compilerOptions);
                var umdHeader = ts.createFunctionExpression(
                /*modifiers*/ undefined, 
                /*asteriskToken*/ undefined, 
                /*name*/ undefined, 
                /*typeParameters*/ undefined, [ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, "factory")], 
                /*type*/ undefined, ts.setTextRange(ts.createBlock([
                    ts.createIf(ts.createLogicalAnd(ts.createTypeCheck(ts.createIdentifier("module"), "object"), ts.createTypeCheck(ts.createPropertyAccess(ts.createIdentifier("module"), "exports"), "object")), ts.createBlock([
                        ts.createVariableStatement(
                        /*modifiers*/ undefined, [
                            ts.createVariableDeclaration("v", 
                            /*type*/ undefined, ts.createCall(ts.createIdentifier("factory"), 
                            /*typeArguments*/ undefined, [
                                ts.createIdentifier("require"),
                                ts.createIdentifier("exports")
                            ]))
                        ]),
                        ts.setEmitFlags(ts.createIf(ts.createStrictInequality(ts.createIdentifier("v"), ts.createIdentifier("undefined")), ts.createExpressionStatement(ts.createAssignment(ts.createPropertyAccess(ts.createIdentifier("module"), "exports"), ts.createIdentifier("v")))), 1 /* SingleLine */)
                    ]), ts.createIf(ts.createLogicalAnd(ts.createTypeCheck(ts.createIdentifier("define"), "function"), ts.createPropertyAccess(ts.createIdentifier("define"), "amd")), ts.createBlock([
                        ts.createExpressionStatement(ts.createCall(ts.createIdentifier("define"), 
                        /*typeArguments*/ undefined, __spreadArrays((moduleName ? [moduleName] : []), [
                            ts.createArrayLiteral(__spreadArrays([
                                ts.createLiteral("require"),
                                ts.createLiteral("exports")
                            ], aliasedModuleNames, unaliasedModuleNames)),
                            ts.createIdentifier("factory")
                        ])))
                    ])))
                ], 
                /*multiLine*/ true), 
                /*location*/ undefined));
                // Create an updated SourceFile:
                //
                //  (function (factory) {
                //      if (typeof module === "object" && typeof module.exports === "object") {
                //          var v = factory(require, exports);
                //          if (v !== undefined) module.exports = v;
                //      }
                //      else if (typeof define === 'function' && define.amd) {
                //          define(["require", "exports"], factory);
                //      }
                //  })(function ...)
                var updated = ts.updateSourceFileNode(node, ts.setTextRange(ts.createNodeArray([
                    ts.createExpressionStatement(ts.createCall(umdHeader, 
                    /*typeArguments*/ undefined, [
                        // Add the module body function argument:
                        //
                        //     function (require, exports) ...
                        ts.createFunctionExpression(
                        /*modifiers*/ undefined, 
                        /*asteriskToken*/ undefined, 
                        /*name*/ undefined, 
                        /*typeParameters*/ undefined, __spreadArrays([
                            ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, "require"),
                            ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, "exports")
                        ], importAliasNames), 
                        /*type*/ undefined, transformAsynchronousModuleBody(node))
                    ]))
                ]), 
                /*location*/ node.statements));
                ts.addEmitHelpers(updated, context.readEmitHelpers());
                return updated;
            }
            /**
             * Collect the additional asynchronous dependencies for the module.
             *
             * @param node The source file.
             * @param includeNonAmdDependencies A value indicating whether to include non-AMD dependencies.
             */
            function collectAsynchronousDependencies(node, includeNonAmdDependencies) {
                // names of modules with corresponding parameter in the factory function
                var aliasedModuleNames = [];
                // names of modules with no corresponding parameters in factory function
                var unaliasedModuleNames = [];
                // names of the parameters in the factory function; these
                // parameters need to match the indexes of the corresponding
                // module names in aliasedModuleNames.
                var importAliasNames = [];
                // Fill in amd-dependency tags
                for (var _i = 0, _a = node.amdDependencies; _i < _a.length; _i++) {
                    var amdDependency = _a[_i];
                    if (amdDependency.name) {
                        aliasedModuleNames.push(ts.createLiteral(amdDependency.path));
                        importAliasNames.push(ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, amdDependency.name));
                    }
                    else {
                        unaliasedModuleNames.push(ts.createLiteral(amdDependency.path));
                    }
                }
                for (var _b = 0, _c = currentModuleInfo.externalImports; _b < _c.length; _b++) {
                    var importNode = _c[_b];
                    // Find the name of the external module
                    var externalModuleName = ts.getExternalModuleNameLiteral(importNode, currentSourceFile, host, resolver, compilerOptions);
                    // Find the name of the module alias, if there is one
                    var importAliasName = ts.getLocalNameForExternalImport(importNode, currentSourceFile);
                    // It is possible that externalModuleName is undefined if it is not string literal.
                    // This can happen in the invalid import syntax.
                    // E.g : "import * from alias from 'someLib';"
                    if (externalModuleName) {
                        if (includeNonAmdDependencies && importAliasName) {
                            // Set emitFlags on the name of the classDeclaration
                            // This is so that when printer will not substitute the identifier
                            ts.setEmitFlags(importAliasName, 4 /* NoSubstitution */);
                            aliasedModuleNames.push(externalModuleName);
                            importAliasNames.push(ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, importAliasName));
                        }
                        else {
                            unaliasedModuleNames.push(externalModuleName);
                        }
                    }
                }
                return { aliasedModuleNames: aliasedModuleNames, unaliasedModuleNames: unaliasedModuleNames, importAliasNames: importAliasNames };
            }
            function getAMDImportExpressionForImport(node) {
                if (ts.isImportEqualsDeclaration(node) || ts.isExportDeclaration(node) || !ts.getExternalModuleNameLiteral(node, currentSourceFile, host, resolver, compilerOptions)) {
                    return undefined;
                }
                var name = ts.getLocalNameForExternalImport(node, currentSourceFile); // TODO: GH#18217
                var expr = getHelperExpressionForImport(node, name);
                if (expr === name) {
                    return undefined;
                }
                return ts.createExpressionStatement(ts.createAssignment(name, expr));
            }
            /**
             * Transforms a SourceFile into an AMD or UMD module body.
             *
             * @param node The SourceFile node.
             */
            function transformAsynchronousModuleBody(node) {
                startLexicalEnvironment();
                var statements = [];
                var statementOffset = ts.addPrologue(statements, node.statements, /*ensureUseStrict*/ !compilerOptions.noImplicitUseStrict, sourceElementVisitor);
                if (shouldEmitUnderscoreUnderscoreESModule()) {
                    ts.append(statements, createUnderscoreUnderscoreESModule());
                }
                // Visit each statement of the module body.
                ts.append(statements, ts.visitNode(currentModuleInfo.externalHelpersImportDeclaration, sourceElementVisitor, ts.isStatement));
                if (moduleKind === ts.ModuleKind.AMD) {
                    ts.addRange(statements, ts.mapDefined(currentModuleInfo.externalImports, getAMDImportExpressionForImport));
                }
                ts.addRange(statements, ts.visitNodes(node.statements, sourceElementVisitor, ts.isStatement, statementOffset));
                // Append the 'export =' statement if provided.
                addExportEqualsIfNeeded(statements, /*emitAsReturn*/ true);
                // End the lexical environment for the module body
                // and merge any new lexical declarations.
                ts.insertStatementsAfterStandardPrologue(statements, endLexicalEnvironment());
                var body = ts.createBlock(statements, /*multiLine*/ true);
                if (currentModuleInfo.hasExportStarsToExportValues && !compilerOptions.importHelpers) {
                    // If we have any `export * from ...` declarations
                    // we need to inform the emitter to add the __export helper.
                    ts.addEmitHelper(body, exportStarHelper);
                }
                if (needUMDDynamicImportHelper) {
                    ts.addEmitHelper(body, dynamicImportUMDHelper);
                }
                return body;
            }
            /**
             * Adds the down-level representation of `export=` to the statement list if one exists
             * in the source file.
             *
             * @param statements The Statement list to modify.
             * @param emitAsReturn A value indicating whether to emit the `export=` statement as a
             * return statement.
             */
            function addExportEqualsIfNeeded(statements, emitAsReturn) {
                if (currentModuleInfo.exportEquals) {
                    var expressionResult = ts.visitNode(currentModuleInfo.exportEquals.expression, moduleExpressionElementVisitor);
                    if (expressionResult) {
                        if (emitAsReturn) {
                            var statement = ts.createReturn(expressionResult);
                            ts.setTextRange(statement, currentModuleInfo.exportEquals);
                            ts.setEmitFlags(statement, 384 /* NoTokenSourceMaps */ | 1536 /* NoComments */);
                            statements.push(statement);
                        }
                        else {
                            var statement = ts.createExpressionStatement(ts.createAssignment(ts.createPropertyAccess(ts.createIdentifier("module"), "exports"), expressionResult));
                            ts.setTextRange(statement, currentModuleInfo.exportEquals);
                            ts.setEmitFlags(statement, 1536 /* NoComments */);
                            statements.push(statement);
                        }
                    }
                }
            }
            //
            // Top-Level Source Element Visitors
            //
            /**
             * Visits a node at the top level of the source file.
             *
             * @param node The node to visit.
             */
            function sourceElementVisitor(node) {
                switch (node.kind) {
                    case 254 /* ImportDeclaration */:
                        return visitImportDeclaration(node);
                    case 253 /* ImportEqualsDeclaration */:
                        return visitImportEqualsDeclaration(node);
                    case 260 /* ExportDeclaration */:
                        return visitExportDeclaration(node);
                    case 259 /* ExportAssignment */:
                        return visitExportAssignment(node);
                    case 225 /* VariableStatement */:
                        return visitVariableStatement(node);
                    case 244 /* FunctionDeclaration */:
                        return visitFunctionDeclaration(node);
                    case 245 /* ClassDeclaration */:
                        return visitClassDeclaration(node);
                    case 327 /* MergeDeclarationMarker */:
                        return visitMergeDeclarationMarker(node);
                    case 328 /* EndOfDeclarationMarker */:
                        return visitEndOfDeclarationMarker(node);
                    default:
                        return ts.visitEachChild(node, moduleExpressionElementVisitor, context);
                }
            }
            function moduleExpressionElementVisitor(node) {
                // This visitor does not need to descend into the tree if there is no dynamic import or destructuring assignment,
                // as export/import statements are only transformed at the top level of a file.
                if (!(node.transformFlags & 2097152 /* ContainsDynamicImport */) && !(node.transformFlags & 1024 /* ContainsDestructuringAssignment */)) {
                    return node;
                }
                if (ts.isImportCall(node)) {
                    return visitImportCallExpression(node);
                }
                else if (ts.isDestructuringAssignment(node)) {
                    return visitDestructuringAssignment(node);
                }
                else {
                    return ts.visitEachChild(node, moduleExpressionElementVisitor, context);
                }
            }
            function destructuringNeedsFlattening(node) {
                if (ts.isObjectLiteralExpression(node)) {
                    for (var _i = 0, _a = node.properties; _i < _a.length; _i++) {
                        var elem = _a[_i];
                        switch (elem.kind) {
                            case 281 /* PropertyAssignment */:
                                if (destructuringNeedsFlattening(elem.initializer)) {
                                    return true;
                                }
                                break;
                            case 282 /* ShorthandPropertyAssignment */:
                                if (destructuringNeedsFlattening(elem.name)) {
                                    return true;
                                }
                                break;
                            case 283 /* SpreadAssignment */:
                                if (destructuringNeedsFlattening(elem.expression)) {
                                    return true;
                                }
                                break;
                            case 161 /* MethodDeclaration */:
                            case 163 /* GetAccessor */:
                            case 164 /* SetAccessor */:
                                return false;
                            default: ts.Debug.assertNever(elem, "Unhandled object member kind");
                        }
                    }
                }
                else if (ts.isArrayLiteralExpression(node)) {
                    for (var _b = 0, _c = node.elements; _b < _c.length; _b++) {
                        var elem = _c[_b];
                        if (ts.isSpreadElement(elem)) {
                            if (destructuringNeedsFlattening(elem.expression)) {
                                return true;
                            }
                        }
                        else if (destructuringNeedsFlattening(elem)) {
                            return true;
                        }
                    }
                }
                else if (ts.isIdentifier(node)) {
                    return ts.length(getExports(node)) > (ts.isExportName(node) ? 1 : 0);
                }
                return false;
            }
            function visitDestructuringAssignment(node) {
                if (destructuringNeedsFlattening(node.left)) {
                    return ts.flattenDestructuringAssignment(node, moduleExpressionElementVisitor, context, 0 /* All */, /*needsValue*/ false, createAllExportExpressions);
                }
                return ts.visitEachChild(node, moduleExpressionElementVisitor, context);
            }
            function visitImportCallExpression(node) {
                var argument = ts.visitNode(ts.firstOrUndefined(node.arguments), moduleExpressionElementVisitor);
                var containsLexicalThis = !!(node.transformFlags & 4096 /* ContainsLexicalThis */);
                switch (compilerOptions.module) {
                    case ts.ModuleKind.AMD:
                        return createImportCallExpressionAMD(argument, containsLexicalThis);
                    case ts.ModuleKind.UMD:
                        return createImportCallExpressionUMD(argument, containsLexicalThis);
                    case ts.ModuleKind.CommonJS:
                    default:
                        return createImportCallExpressionCommonJS(argument, containsLexicalThis);
                }
            }
            function createImportCallExpressionUMD(arg, containsLexicalThis) {
                // (function (factory) {
                //      ... (regular UMD)
                // }
                // })(function (require, exports, useSyncRequire) {
                //      "use strict";
                //      Object.defineProperty(exports, "__esModule", { value: true });
                //      var __syncRequire = typeof module === "object" && typeof module.exports === "object";
                //      var __resolved = new Promise(function (resolve) { resolve(); });
                //      .....
                //      __syncRequire
                //          ? __resolved.then(function () { return require(x); }) /*CommonJs Require*/
                //          : new Promise(function (_a, _b) { require([x], _a, _b); }); /*Amd Require*/
                // });
                needUMDDynamicImportHelper = true;
                if (ts.isSimpleCopiableExpression(arg)) {
                    var argClone = ts.isGeneratedIdentifier(arg) ? arg : ts.isStringLiteral(arg) ? ts.createLiteral(arg) : ts.setEmitFlags(ts.setTextRange(ts.getSynthesizedClone(arg), arg), 1536 /* NoComments */);
                    return ts.createConditional(
                    /*condition*/ ts.createIdentifier("__syncRequire"), 
                    /*whenTrue*/ createImportCallExpressionCommonJS(arg, containsLexicalThis), 
                    /*whenFalse*/ createImportCallExpressionAMD(argClone, containsLexicalThis));
                }
                else {
                    var temp = ts.createTempVariable(hoistVariableDeclaration);
                    return ts.createComma(ts.createAssignment(temp, arg), ts.createConditional(
                    /*condition*/ ts.createIdentifier("__syncRequire"), 
                    /*whenTrue*/ createImportCallExpressionCommonJS(temp, containsLexicalThis), 
                    /*whenFalse*/ createImportCallExpressionAMD(temp, containsLexicalThis)));
                }
            }
            function createImportCallExpressionAMD(arg, containsLexicalThis) {
                // improt("./blah")
                // emit as
                // define(["require", "exports", "blah"], function (require, exports) {
                //     ...
                //     new Promise(function (_a, _b) { require([x], _a, _b); }); /*Amd Require*/
                // });
                var resolve = ts.createUniqueName("resolve");
                var reject = ts.createUniqueName("reject");
                var parameters = [
                    ts.createParameter(/*decorator*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, /*name*/ resolve),
                    ts.createParameter(/*decorator*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, /*name*/ reject)
                ];
                var body = ts.createBlock([
                    ts.createExpressionStatement(ts.createCall(ts.createIdentifier("require"), 
                    /*typeArguments*/ undefined, [ts.createArrayLiteral([arg || ts.createOmittedExpression()]), resolve, reject]))
                ]);
                var func;
                if (languageVersion >= 2 /* ES2015 */) {
                    func = ts.createArrowFunction(
                    /*modifiers*/ undefined, 
                    /*typeParameters*/ undefined, parameters, 
                    /*type*/ undefined, 
                    /*equalsGreaterThanToken*/ undefined, body);
                }
                else {
                    func = ts.createFunctionExpression(
                    /*modifiers*/ undefined, 
                    /*asteriskToken*/ undefined, 
                    /*name*/ undefined, 
                    /*typeParameters*/ undefined, parameters, 
                    /*type*/ undefined, body);
                    // if there is a lexical 'this' in the import call arguments, ensure we indicate
                    // that this new function expression indicates it captures 'this' so that the
                    // es2015 transformer will properly substitute 'this' with '_this'.
                    if (containsLexicalThis) {
                        ts.setEmitFlags(func, 8 /* CapturesThis */);
                    }
                }
                var promise = ts.createNew(ts.createIdentifier("Promise"), /*typeArguments*/ undefined, [func]);
                if (compilerOptions.esModuleInterop) {
                    context.requestEmitHelper(ts.importStarHelper);
                    return ts.createCall(ts.createPropertyAccess(promise, ts.createIdentifier("then")), /*typeArguments*/ undefined, [ts.getUnscopedHelperName("__importStar")]);
                }
                return promise;
            }
            function createImportCallExpressionCommonJS(arg, containsLexicalThis) {
                // import("./blah")
                // emit as
                // Promise.resolve().then(function () { return require(x); }) /*CommonJs Require*/
                // We have to wrap require in then callback so that require is done in asynchronously
                // if we simply do require in resolve callback in Promise constructor. We will execute the loading immediately
                var promiseResolveCall = ts.createCall(ts.createPropertyAccess(ts.createIdentifier("Promise"), "resolve"), /*typeArguments*/ undefined, /*argumentsArray*/ []);
                var requireCall = ts.createCall(ts.createIdentifier("require"), /*typeArguments*/ undefined, arg ? [arg] : []);
                if (compilerOptions.esModuleInterop) {
                    context.requestEmitHelper(ts.importStarHelper);
                    requireCall = ts.createCall(ts.getUnscopedHelperName("__importStar"), /*typeArguments*/ undefined, [requireCall]);
                }
                var func;
                if (languageVersion >= 2 /* ES2015 */) {
                    func = ts.createArrowFunction(
                    /*modifiers*/ undefined, 
                    /*typeParameters*/ undefined, 
                    /*parameters*/ [], 
                    /*type*/ undefined, 
                    /*equalsGreaterThanToken*/ undefined, requireCall);
                }
                else {
                    func = ts.createFunctionExpression(
                    /*modifiers*/ undefined, 
                    /*asteriskToken*/ undefined, 
                    /*name*/ undefined, 
                    /*typeParameters*/ undefined, 
                    /*parameters*/ [], 
                    /*type*/ undefined, ts.createBlock([ts.createReturn(requireCall)]));
                    // if there is a lexical 'this' in the import call arguments, ensure we indicate
                    // that this new function expression indicates it captures 'this' so that the
                    // es2015 transformer will properly substitute 'this' with '_this'.
                    if (containsLexicalThis) {
                        ts.setEmitFlags(func, 8 /* CapturesThis */);
                    }
                }
                return ts.createCall(ts.createPropertyAccess(promiseResolveCall, "then"), /*typeArguments*/ undefined, [func]);
            }
            function getHelperExpressionForExport(node, innerExpr) {
                if (!compilerOptions.esModuleInterop || ts.getEmitFlags(node) & 67108864 /* NeverApplyImportHelper */) {
                    return innerExpr;
                }
                if (ts.getExportNeedsImportStarHelper(node)) {
                    context.requestEmitHelper(ts.importStarHelper);
                    return ts.createCall(ts.getUnscopedHelperName("__importStar"), /*typeArguments*/ undefined, [innerExpr]);
                }
                return innerExpr;
            }
            function getHelperExpressionForImport(node, innerExpr) {
                if (!compilerOptions.esModuleInterop || ts.getEmitFlags(node) & 67108864 /* NeverApplyImportHelper */) {
                    return innerExpr;
                }
                if (ts.getImportNeedsImportStarHelper(node)) {
                    context.requestEmitHelper(ts.importStarHelper);
                    return ts.createCall(ts.getUnscopedHelperName("__importStar"), /*typeArguments*/ undefined, [innerExpr]);
                }
                if (ts.getImportNeedsImportDefaultHelper(node)) {
                    context.requestEmitHelper(ts.importDefaultHelper);
                    return ts.createCall(ts.getUnscopedHelperName("__importDefault"), /*typeArguments*/ undefined, [innerExpr]);
                }
                return innerExpr;
            }
            /**
             * Visits an ImportDeclaration node.
             *
             * @param node The node to visit.
             */
            function visitImportDeclaration(node) {
                var statements;
                var namespaceDeclaration = ts.getNamespaceDeclarationNode(node);
                if (moduleKind !== ts.ModuleKind.AMD) {
                    if (!node.importClause) {
                        // import "mod";
                        return ts.setOriginalNode(ts.setTextRange(ts.createExpressionStatement(createRequireCall(node)), node), node);
                    }
                    else {
                        var variables = [];
                        if (namespaceDeclaration && !ts.isDefaultImport(node)) {
                            // import * as n from "mod";
                            variables.push(ts.createVariableDeclaration(ts.getSynthesizedClone(namespaceDeclaration.name), 
                            /*type*/ undefined, getHelperExpressionForImport(node, createRequireCall(node))));
                        }
                        else {
                            // import d from "mod";
                            // import { x, y } from "mod";
                            // import d, { x, y } from "mod";
                            // import d, * as n from "mod";
                            variables.push(ts.createVariableDeclaration(ts.getGeneratedNameForNode(node), 
                            /*type*/ undefined, getHelperExpressionForImport(node, createRequireCall(node))));
                            if (namespaceDeclaration && ts.isDefaultImport(node)) {
                                variables.push(ts.createVariableDeclaration(ts.getSynthesizedClone(namespaceDeclaration.name), 
                                /*type*/ undefined, ts.getGeneratedNameForNode(node)));
                            }
                        }
                        statements = ts.append(statements, ts.setOriginalNode(ts.setTextRange(ts.createVariableStatement(
                        /*modifiers*/ undefined, ts.createVariableDeclarationList(variables, languageVersion >= 2 /* ES2015 */ ? 2 /* Const */ : 0 /* None */)), 
                        /*location*/ node), 
                        /*original*/ node));
                    }
                }
                else if (namespaceDeclaration && ts.isDefaultImport(node)) {
                    // import d, * as n from "mod";
                    statements = ts.append(statements, ts.createVariableStatement(
                    /*modifiers*/ undefined, ts.createVariableDeclarationList([
                        ts.setOriginalNode(ts.setTextRange(ts.createVariableDeclaration(ts.getSynthesizedClone(namespaceDeclaration.name), 
                        /*type*/ undefined, ts.getGeneratedNameForNode(node)), 
                        /*location*/ node), 
                        /*original*/ node)
                    ], languageVersion >= 2 /* ES2015 */ ? 2 /* Const */ : 0 /* None */)));
                }
                if (hasAssociatedEndOfDeclarationMarker(node)) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportsOfImportDeclaration(deferredExports[id], node);
                }
                else {
                    statements = appendExportsOfImportDeclaration(statements, node);
                }
                return ts.singleOrMany(statements);
            }
            /**
             * Creates a `require()` call to import an external module.
             *
             * @param importNode The declararation to import.
             */
            function createRequireCall(importNode) {
                var moduleName = ts.getExternalModuleNameLiteral(importNode, currentSourceFile, host, resolver, compilerOptions);
                var args = [];
                if (moduleName) {
                    args.push(moduleName);
                }
                return ts.createCall(ts.createIdentifier("require"), /*typeArguments*/ undefined, args);
            }
            /**
             * Visits an ImportEqualsDeclaration node.
             *
             * @param node The node to visit.
             */
            function visitImportEqualsDeclaration(node) {
                ts.Debug.assert(ts.isExternalModuleImportEqualsDeclaration(node), "import= for internal module references should be handled in an earlier transformer.");
                var statements;
                if (moduleKind !== ts.ModuleKind.AMD) {
                    if (ts.hasModifier(node, 1 /* Export */)) {
                        statements = ts.append(statements, ts.setOriginalNode(ts.setTextRange(ts.createExpressionStatement(createExportExpression(node.name, createRequireCall(node))), node), node));
                    }
                    else {
                        statements = ts.append(statements, ts.setOriginalNode(ts.setTextRange(ts.createVariableStatement(
                        /*modifiers*/ undefined, ts.createVariableDeclarationList([
                            ts.createVariableDeclaration(ts.getSynthesizedClone(node.name), 
                            /*type*/ undefined, createRequireCall(node))
                        ], 
                        /*flags*/ languageVersion >= 2 /* ES2015 */ ? 2 /* Const */ : 0 /* None */)), node), node));
                    }
                }
                else {
                    if (ts.hasModifier(node, 1 /* Export */)) {
                        statements = ts.append(statements, ts.setOriginalNode(ts.setTextRange(ts.createExpressionStatement(createExportExpression(ts.getExportName(node), ts.getLocalName(node))), node), node));
                    }
                }
                if (hasAssociatedEndOfDeclarationMarker(node)) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportsOfImportEqualsDeclaration(deferredExports[id], node);
                }
                else {
                    statements = appendExportsOfImportEqualsDeclaration(statements, node);
                }
                return ts.singleOrMany(statements);
            }
            /**
             * Visits an ExportDeclaration node.
             *
             * @param The node to visit.
             */
            function visitExportDeclaration(node) {
                if (!node.moduleSpecifier) {
                    // Elide export declarations with no module specifier as they are handled
                    // elsewhere.
                    return undefined;
                }
                var generatedName = ts.getGeneratedNameForNode(node);
                if (node.exportClause && ts.isNamedExports(node.exportClause)) {
                    var statements = [];
                    // export { x, y } from "mod";
                    if (moduleKind !== ts.ModuleKind.AMD) {
                        statements.push(ts.setOriginalNode(ts.setTextRange(ts.createVariableStatement(
                        /*modifiers*/ undefined, ts.createVariableDeclarationList([
                            ts.createVariableDeclaration(generatedName, 
                            /*type*/ undefined, createRequireCall(node))
                        ])), 
                        /*location*/ node), 
                        /* original */ node));
                    }
                    for (var _i = 0, _a = node.exportClause.elements; _i < _a.length; _i++) {
                        var specifier = _a[_i];
                        var exportedValue = ts.createPropertyAccess(generatedName, specifier.propertyName || specifier.name);
                        statements.push(ts.setOriginalNode(ts.setTextRange(ts.createExpressionStatement(createExportExpression(ts.getExportName(specifier), exportedValue)), specifier), specifier));
                    }
                    return ts.singleOrMany(statements);
                }
                else if (node.exportClause) {
                    var statements = [];
                    // export * as ns from "mod";
                    statements.push(ts.setOriginalNode(ts.setTextRange(ts.createExpressionStatement(createExportExpression(ts.getSynthesizedClone(node.exportClause.name), moduleKind !== ts.ModuleKind.AMD ?
                        getHelperExpressionForExport(node, createRequireCall(node)) :
                        ts.createIdentifier(ts.idText(node.exportClause.name)))), node), node));
                    return ts.singleOrMany(statements);
                }
                else {
                    // export * from "mod";
                    return ts.setOriginalNode(ts.setTextRange(ts.createExpressionStatement(createExportStarHelper(context, moduleKind !== ts.ModuleKind.AMD ? createRequireCall(node) : generatedName)), node), node);
                }
            }
            /**
             * Visits an ExportAssignment node.
             *
             * @param node The node to visit.
             */
            function visitExportAssignment(node) {
                if (node.isExportEquals) {
                    return undefined;
                }
                var statements;
                var original = node.original;
                if (original && hasAssociatedEndOfDeclarationMarker(original)) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportStatement(deferredExports[id], ts.createIdentifier("default"), ts.visitNode(node.expression, moduleExpressionElementVisitor), /*location*/ node, /*allowComments*/ true);
                }
                else {
                    statements = appendExportStatement(statements, ts.createIdentifier("default"), ts.visitNode(node.expression, moduleExpressionElementVisitor), /*location*/ node, /*allowComments*/ true);
                }
                return ts.singleOrMany(statements);
            }
            /**
             * Visits a FunctionDeclaration node.
             *
             * @param node The node to visit.
             */
            function visitFunctionDeclaration(node) {
                var statements;
                if (ts.hasModifier(node, 1 /* Export */)) {
                    statements = ts.append(statements, ts.setOriginalNode(ts.setTextRange(ts.createFunctionDeclaration(
                    /*decorators*/ undefined, ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier), node.asteriskToken, ts.getDeclarationName(node, /*allowComments*/ true, /*allowSourceMaps*/ true), 
                    /*typeParameters*/ undefined, ts.visitNodes(node.parameters, moduleExpressionElementVisitor), 
                    /*type*/ undefined, ts.visitEachChild(node.body, moduleExpressionElementVisitor, context)), 
                    /*location*/ node), 
                    /*original*/ node));
                }
                else {
                    statements = ts.append(statements, ts.visitEachChild(node, moduleExpressionElementVisitor, context));
                }
                if (hasAssociatedEndOfDeclarationMarker(node)) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportsOfHoistedDeclaration(deferredExports[id], node);
                }
                else {
                    statements = appendExportsOfHoistedDeclaration(statements, node);
                }
                return ts.singleOrMany(statements);
            }
            /**
             * Visits a ClassDeclaration node.
             *
             * @param node The node to visit.
             */
            function visitClassDeclaration(node) {
                var statements;
                if (ts.hasModifier(node, 1 /* Export */)) {
                    statements = ts.append(statements, ts.setOriginalNode(ts.setTextRange(ts.createClassDeclaration(
                    /*decorators*/ undefined, ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier), ts.getDeclarationName(node, /*allowComments*/ true, /*allowSourceMaps*/ true), 
                    /*typeParameters*/ undefined, ts.visitNodes(node.heritageClauses, moduleExpressionElementVisitor), ts.visitNodes(node.members, moduleExpressionElementVisitor)), node), node));
                }
                else {
                    statements = ts.append(statements, ts.visitEachChild(node, moduleExpressionElementVisitor, context));
                }
                if (hasAssociatedEndOfDeclarationMarker(node)) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportsOfHoistedDeclaration(deferredExports[id], node);
                }
                else {
                    statements = appendExportsOfHoistedDeclaration(statements, node);
                }
                return ts.singleOrMany(statements);
            }
            /**
             * Visits a VariableStatement node.
             *
             * @param node The node to visit.
             */
            function visitVariableStatement(node) {
                var statements;
                var variables;
                var expressions;
                if (ts.hasModifier(node, 1 /* Export */)) {
                    var modifiers = void 0;
                    // If we're exporting these variables, then these just become assignments to 'exports.x'.
                    // We only want to emit assignments for variables with initializers.
                    for (var _i = 0, _a = node.declarationList.declarations; _i < _a.length; _i++) {
                        var variable = _a[_i];
                        if (ts.isIdentifier(variable.name) && ts.isLocalName(variable.name)) {
                            if (!modifiers) {
                                modifiers = ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier);
                            }
                            variables = ts.append(variables, variable);
                        }
                        else if (variable.initializer) {
                            expressions = ts.append(expressions, transformInitializedVariable(variable));
                        }
                    }
                    if (variables) {
                        statements = ts.append(statements, ts.updateVariableStatement(node, modifiers, ts.updateVariableDeclarationList(node.declarationList, variables)));
                    }
                    if (expressions) {
                        statements = ts.append(statements, ts.setOriginalNode(ts.setTextRange(ts.createExpressionStatement(ts.inlineExpressions(expressions)), node), node));
                    }
                }
                else {
                    statements = ts.append(statements, ts.visitEachChild(node, moduleExpressionElementVisitor, context));
                }
                if (hasAssociatedEndOfDeclarationMarker(node)) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportsOfVariableStatement(deferredExports[id], node);
                }
                else {
                    statements = appendExportsOfVariableStatement(statements, node);
                }
                return ts.singleOrMany(statements);
            }
            function createAllExportExpressions(name, value, location) {
                var exportedNames = getExports(name);
                if (exportedNames) {
                    // For each additional export of the declaration, apply an export assignment.
                    var expression = ts.isExportName(name) ? value : ts.createAssignment(name, value);
                    for (var _i = 0, exportedNames_1 = exportedNames; _i < exportedNames_1.length; _i++) {
                        var exportName = exportedNames_1[_i];
                        // Mark the node to prevent triggering substitution.
                        ts.setEmitFlags(expression, 4 /* NoSubstitution */);
                        expression = createExportExpression(exportName, expression, /*location*/ location);
                    }
                    return expression;
                }
                return ts.createAssignment(name, value);
            }
            /**
             * Transforms an exported variable with an initializer into an expression.
             *
             * @param node The node to transform.
             */
            function transformInitializedVariable(node) {
                if (ts.isBindingPattern(node.name)) {
                    return ts.flattenDestructuringAssignment(ts.visitNode(node, moduleExpressionElementVisitor), 
                    /*visitor*/ undefined, context, 0 /* All */, 
                    /*needsValue*/ false, createAllExportExpressions);
                }
                else {
                    return ts.createAssignment(ts.setTextRange(ts.createPropertyAccess(ts.createIdentifier("exports"), node.name), 
                    /*location*/ node.name), ts.visitNode(node.initializer, moduleExpressionElementVisitor));
                }
            }
            /**
             * Visits a MergeDeclarationMarker used as a placeholder for the beginning of a merged
             * and transformed declaration.
             *
             * @param node The node to visit.
             */
            function visitMergeDeclarationMarker(node) {
                // For an EnumDeclaration or ModuleDeclaration that merges with a preceeding
                // declaration we do not emit a leading variable declaration. To preserve the
                // begin/end semantics of the declararation and to properly handle exports
                // we wrapped the leading variable declaration in a `MergeDeclarationMarker`.
                //
                // To balance the declaration, add the exports of the elided variable
                // statement.
                if (hasAssociatedEndOfDeclarationMarker(node) && node.original.kind === 225 /* VariableStatement */) {
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportsOfVariableStatement(deferredExports[id], node.original);
                }
                return node;
            }
            /**
             * Determines whether a node has an associated EndOfDeclarationMarker.
             *
             * @param node The node to test.
             */
            function hasAssociatedEndOfDeclarationMarker(node) {
                return (ts.getEmitFlags(node) & 4194304 /* HasEndOfDeclarationMarker */) !== 0;
            }
            /**
             * Visits a DeclarationMarker used as a placeholder for the end of a transformed
             * declaration.
             *
             * @param node The node to visit.
             */
            function visitEndOfDeclarationMarker(node) {
                // For some transformations we emit an `EndOfDeclarationMarker` to mark the actual
                // end of the transformed declaration. We use this marker to emit any deferred exports
                // of the declaration.
                var id = ts.getOriginalNodeId(node);
                var statements = deferredExports[id];
                if (statements) {
                    delete deferredExports[id];
                    return ts.append(statements, node);
                }
                return node;
            }
            /**
             * Appends the exports of an ImportDeclaration to a statement list, returning the
             * statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param decl The declaration whose exports are to be recorded.
             */
            function appendExportsOfImportDeclaration(statements, decl) {
                if (currentModuleInfo.exportEquals) {
                    return statements;
                }
                var importClause = decl.importClause;
                if (!importClause) {
                    return statements;
                }
                if (importClause.name) {
                    statements = appendExportsOfDeclaration(statements, importClause);
                }
                var namedBindings = importClause.namedBindings;
                if (namedBindings) {
                    switch (namedBindings.kind) {
                        case 256 /* NamespaceImport */:
                            statements = appendExportsOfDeclaration(statements, namedBindings);
                            break;
                        case 257 /* NamedImports */:
                            for (var _i = 0, _a = namedBindings.elements; _i < _a.length; _i++) {
                                var importBinding = _a[_i];
                                statements = appendExportsOfDeclaration(statements, importBinding);
                            }
                            break;
                    }
                }
                return statements;
            }
            /**
             * Appends the exports of an ImportEqualsDeclaration to a statement list, returning the
             * statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param decl The declaration whose exports are to be recorded.
             */
            function appendExportsOfImportEqualsDeclaration(statements, decl) {
                if (currentModuleInfo.exportEquals) {
                    return statements;
                }
                return appendExportsOfDeclaration(statements, decl);
            }
            /**
             * Appends the exports of a VariableStatement to a statement list, returning the statement
             * list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param node The VariableStatement whose exports are to be recorded.
             */
            function appendExportsOfVariableStatement(statements, node) {
                if (currentModuleInfo.exportEquals) {
                    return statements;
                }
                for (var _i = 0, _a = node.declarationList.declarations; _i < _a.length; _i++) {
                    var decl = _a[_i];
                    statements = appendExportsOfBindingElement(statements, decl);
                }
                return statements;
            }
            /**
             * Appends the exports of a VariableDeclaration or BindingElement to a statement list,
             * returning the statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param decl The declaration whose exports are to be recorded.
             */
            function appendExportsOfBindingElement(statements, decl) {
                if (currentModuleInfo.exportEquals) {
                    return statements;
                }
                if (ts.isBindingPattern(decl.name)) {
                    for (var _i = 0, _a = decl.name.elements; _i < _a.length; _i++) {
                        var element = _a[_i];
                        if (!ts.isOmittedExpression(element)) {
                            statements = appendExportsOfBindingElement(statements, element);
                        }
                    }
                }
                else if (!ts.isGeneratedIdentifier(decl.name)) {
                    statements = appendExportsOfDeclaration(statements, decl);
                }
                return statements;
            }
            /**
             * Appends the exports of a ClassDeclaration or FunctionDeclaration to a statement list,
             * returning the statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param decl The declaration whose exports are to be recorded.
             */
            function appendExportsOfHoistedDeclaration(statements, decl) {
                if (currentModuleInfo.exportEquals) {
                    return statements;
                }
                if (ts.hasModifier(decl, 1 /* Export */)) {
                    var exportName = ts.hasModifier(decl, 512 /* Default */) ? ts.createIdentifier("default") : ts.getDeclarationName(decl);
                    statements = appendExportStatement(statements, exportName, ts.getLocalName(decl), /*location*/ decl);
                }
                if (decl.name) {
                    statements = appendExportsOfDeclaration(statements, decl);
                }
                return statements;
            }
            /**
             * Appends the exports of a declaration to a statement list, returning the statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param decl The declaration to export.
             */
            function appendExportsOfDeclaration(statements, decl) {
                var name = ts.getDeclarationName(decl);
                var exportSpecifiers = currentModuleInfo.exportSpecifiers.get(ts.idText(name));
                if (exportSpecifiers) {
                    for (var _i = 0, exportSpecifiers_1 = exportSpecifiers; _i < exportSpecifiers_1.length; _i++) {
                        var exportSpecifier = exportSpecifiers_1[_i];
                        statements = appendExportStatement(statements, exportSpecifier.name, name, /*location*/ exportSpecifier.name);
                    }
                }
                return statements;
            }
            /**
             * Appends the down-level representation of an export to a statement list, returning the
             * statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param exportName The name of the export.
             * @param expression The expression to export.
             * @param location The location to use for source maps and comments for the export.
             * @param allowComments Whether to allow comments on the export.
             */
            function appendExportStatement(statements, exportName, expression, location, allowComments) {
                statements = ts.append(statements, createExportStatement(exportName, expression, location, allowComments));
                return statements;
            }
            function createUnderscoreUnderscoreESModule() {
                var statement;
                if (languageVersion === 0 /* ES3 */) {
                    statement = ts.createExpressionStatement(createExportExpression(ts.createIdentifier("__esModule"), ts.createLiteral(/*value*/ true)));
                }
                else {
                    statement = ts.createExpressionStatement(ts.createCall(ts.createPropertyAccess(ts.createIdentifier("Object"), "defineProperty"), 
                    /*typeArguments*/ undefined, [
                        ts.createIdentifier("exports"),
                        ts.createLiteral("__esModule"),
                        ts.createObjectLiteral([
                            ts.createPropertyAssignment("value", ts.createLiteral(/*value*/ true))
                        ])
                    ]));
                }
                ts.setEmitFlags(statement, 1048576 /* CustomPrologue */);
                return statement;
            }
            /**
             * Creates a call to the current file's export function to export a value.
             *
             * @param name The bound name of the export.
             * @param value The exported value.
             * @param location The location to use for source maps and comments for the export.
             * @param allowComments An optional value indicating whether to emit comments for the statement.
             */
            function createExportStatement(name, value, location, allowComments) {
                var statement = ts.setTextRange(ts.createExpressionStatement(createExportExpression(name, value)), location);
                ts.startOnNewLine(statement);
                if (!allowComments) {
                    ts.setEmitFlags(statement, 1536 /* NoComments */);
                }
                return statement;
            }
            /**
             * Creates a call to the current file's export function to export a value.
             *
             * @param name The bound name of the export.
             * @param value The exported value.
             * @param location The location to use for source maps and comments for the export.
             */
            function createExportExpression(name, value, location) {
                return ts.setTextRange(ts.createAssignment(ts.createPropertyAccess(ts.createIdentifier("exports"), ts.getSynthesizedClone(name)), value), location);
            }
            //
            // Modifier Visitors
            //
            /**
             * Visit nodes to elide module-specific modifiers.
             *
             * @param node The node to visit.
             */
            function modifierVisitor(node) {
                // Elide module-specific modifiers.
                switch (node.kind) {
                    case 89 /* ExportKeyword */:
                    case 84 /* DefaultKeyword */:
                        return undefined;
                }
                return node;
            }
            //
            // Emit Notification
            //
            /**
             * Hook for node emit notifications.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to emit.
             * @param emit A callback used to emit the node in the printer.
             */
            function onEmitNode(hint, node, emitCallback) {
                if (node.kind === 290 /* SourceFile */) {
                    currentSourceFile = node;
                    currentModuleInfo = moduleInfoMap[ts.getOriginalNodeId(currentSourceFile)];
                    noSubstitution = [];
                    previousOnEmitNode(hint, node, emitCallback);
                    currentSourceFile = undefined;
                    currentModuleInfo = undefined;
                    noSubstitution = undefined;
                }
                else {
                    previousOnEmitNode(hint, node, emitCallback);
                }
            }
            //
            // Substitutions
            //
            /**
             * Hooks node substitutions.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to substitute.
             */
            function onSubstituteNode(hint, node) {
                node = previousOnSubstituteNode(hint, node);
                if (node.id && noSubstitution[node.id]) {
                    return node;
                }
                if (hint === 1 /* Expression */) {
                    return substituteExpression(node);
                }
                else if (ts.isShorthandPropertyAssignment(node)) {
                    return substituteShorthandPropertyAssignment(node);
                }
                return node;
            }
            /**
             * Substitution for a ShorthandPropertyAssignment whose declaration name is an imported
             * or exported symbol.
             *
             * @param node The node to substitute.
             */
            function substituteShorthandPropertyAssignment(node) {
                var name = node.name;
                var exportedOrImportedName = substituteExpressionIdentifier(name);
                if (exportedOrImportedName !== name) {
                    // A shorthand property with an assignment initializer is probably part of a
                    // destructuring assignment
                    if (node.objectAssignmentInitializer) {
                        var initializer = ts.createAssignment(exportedOrImportedName, node.objectAssignmentInitializer);
                        return ts.setTextRange(ts.createPropertyAssignment(name, initializer), node);
                    }
                    return ts.setTextRange(ts.createPropertyAssignment(name, exportedOrImportedName), node);
                }
                return node;
            }
            /**
             * Substitution for an Expression that may contain an imported or exported symbol.
             *
             * @param node The node to substitute.
             */
            function substituteExpression(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                        return substituteExpressionIdentifier(node);
                    case 209 /* BinaryExpression */:
                        return substituteBinaryExpression(node);
                    case 208 /* PostfixUnaryExpression */:
                    case 207 /* PrefixUnaryExpression */:
                        return substituteUnaryExpression(node);
                }
                return node;
            }
            /**
             * Substitution for an Identifier expression that may contain an imported or exported
             * symbol.
             *
             * @param node The node to substitute.
             */
            function substituteExpressionIdentifier(node) {
                if (ts.getEmitFlags(node) & 4096 /* HelperName */) {
                    var externalHelpersModuleName = ts.getExternalHelpersModuleName(currentSourceFile);
                    if (externalHelpersModuleName) {
                        return ts.createPropertyAccess(externalHelpersModuleName, node);
                    }
                    return node;
                }
                if (!ts.isGeneratedIdentifier(node) && !ts.isLocalName(node)) {
                    var exportContainer = resolver.getReferencedExportContainer(node, ts.isExportName(node));
                    if (exportContainer && exportContainer.kind === 290 /* SourceFile */) {
                        return ts.setTextRange(ts.createPropertyAccess(ts.createIdentifier("exports"), ts.getSynthesizedClone(node)), 
                        /*location*/ node);
                    }
                    var importDeclaration = resolver.getReferencedImportDeclaration(node);
                    if (importDeclaration) {
                        if (ts.isImportClause(importDeclaration)) {
                            return ts.setTextRange(ts.createPropertyAccess(ts.getGeneratedNameForNode(importDeclaration.parent), ts.createIdentifier("default")), 
                            /*location*/ node);
                        }
                        else if (ts.isImportSpecifier(importDeclaration)) {
                            var name = importDeclaration.propertyName || importDeclaration.name;
                            return ts.setTextRange(ts.createPropertyAccess(ts.getGeneratedNameForNode(importDeclaration.parent.parent.parent), ts.getSynthesizedClone(name)), 
                            /*location*/ node);
                        }
                    }
                }
                return node;
            }
            /**
             * Substitution for a BinaryExpression that may contain an imported or exported symbol.
             *
             * @param node The node to substitute.
             */
            function substituteBinaryExpression(node) {
                // When we see an assignment expression whose left-hand side is an exported symbol,
                // we should ensure all exports of that symbol are updated with the correct value.
                //
                // - We do not substitute generated identifiers for any reason.
                // - We do not substitute identifiers tagged with the LocalName flag.
                // - We do not substitute identifiers that were originally the name of an enum or
                //   namespace due to how they are transformed in TypeScript.
                // - We only substitute identifiers that are exported at the top level.
                if (ts.isAssignmentOperator(node.operatorToken.kind)
                    && ts.isIdentifier(node.left)
                    && !ts.isGeneratedIdentifier(node.left)
                    && !ts.isLocalName(node.left)
                    && !ts.isDeclarationNameOfEnumOrNamespace(node.left)) {
                    var exportedNames = getExports(node.left);
                    if (exportedNames) {
                        // For each additional export of the declaration, apply an export assignment.
                        var expression = node;
                        for (var _i = 0, exportedNames_2 = exportedNames; _i < exportedNames_2.length; _i++) {
                            var exportName = exportedNames_2[_i];
                            // Mark the node to prevent triggering this rule again.
                            noSubstitution[ts.getNodeId(expression)] = true;
                            expression = createExportExpression(exportName, expression, /*location*/ node);
                        }
                        return expression;
                    }
                }
                return node;
            }
            /**
             * Substitution for a UnaryExpression that may contain an imported or exported symbol.
             *
             * @param node The node to substitute.
             */
            function substituteUnaryExpression(node) {
                // When we see a prefix or postfix increment expression whose operand is an exported
                // symbol, we should ensure all exports of that symbol are updated with the correct
                // value.
                //
                // - We do not substitute generated identifiers for any reason.
                // - We do not substitute identifiers tagged with the LocalName flag.
                // - We do not substitute identifiers that were originally the name of an enum or
                //   namespace due to how they are transformed in TypeScript.
                // - We only substitute identifiers that are exported at the top level.
                if ((node.operator === 45 /* PlusPlusToken */ || node.operator === 46 /* MinusMinusToken */)
                    && ts.isIdentifier(node.operand)
                    && !ts.isGeneratedIdentifier(node.operand)
                    && !ts.isLocalName(node.operand)
                    && !ts.isDeclarationNameOfEnumOrNamespace(node.operand)) {
                    var exportedNames = getExports(node.operand);
                    if (exportedNames) {
                        var expression = node.kind === 208 /* PostfixUnaryExpression */
                            ? ts.setTextRange(ts.createBinary(node.operand, ts.createToken(node.operator === 45 /* PlusPlusToken */ ? 63 /* PlusEqualsToken */ : 64 /* MinusEqualsToken */), ts.createLiteral(1)), 
                            /*location*/ node)
                            : node;
                        for (var _i = 0, exportedNames_3 = exportedNames; _i < exportedNames_3.length; _i++) {
                            var exportName = exportedNames_3[_i];
                            // Mark the node to prevent triggering this rule again.
                            noSubstitution[ts.getNodeId(expression)] = true;
                            expression = createExportExpression(exportName, expression);
                        }
                        return expression;
                    }
                }
                return node;
            }
            /**
             * Gets the additional exports of a name.
             *
             * @param name The name.
             */
            function getExports(name) {
                if (!ts.isGeneratedIdentifier(name)) {
                    var valueDeclaration = resolver.getReferencedImportDeclaration(name)
                        || resolver.getReferencedValueDeclaration(name);
                    if (valueDeclaration) {
                        return currentModuleInfo
                            && currentModuleInfo.exportedBindings[ts.getOriginalNodeId(valueDeclaration)];
                    }
                }
            }
        }
        ts.transformModule = transformModule;
        // emit output for the __export helper function
        var exportStarHelper = {
            name: "typescript:export-star",
            scoped: true,
            text: "\n            function __export(m) {\n                for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];\n            }"
        };
        function createExportStarHelper(context, module) {
            var compilerOptions = context.getCompilerOptions();
            return compilerOptions.importHelpers
                ? ts.createCall(ts.getUnscopedHelperName("__exportStar"), /*typeArguments*/ undefined, [module, ts.createIdentifier("exports")])
                : ts.createCall(ts.createIdentifier("__export"), /*typeArguments*/ undefined, [module]);
        }
        // emit helper for dynamic import
        var dynamicImportUMDHelper = {
            name: "typescript:dynamicimport-sync-require",
            scoped: true,
            text: "\n            var __syncRequire = typeof module === \"object\" && typeof module.exports === \"object\";"
        };
        // emit helper for `import * as Name from "foo"`
        ts.importStarHelper = {
            name: "typescript:commonjsimportstar",
            importName: "__importStar",
            scoped: false,
            text: "\nvar __importStar = (this && this.__importStar) || function (mod) {\n    if (mod && mod.__esModule) return mod;\n    var result = {};\n    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];\n    result[\"default\"] = mod;\n    return result;\n};"
        };
        // emit helper for `import Name from "foo"`
        ts.importDefaultHelper = {
            name: "typescript:commonjsimportdefault",
            importName: "__importDefault",
            scoped: false,
            text: "\nvar __importDefault = (this && this.__importDefault) || function (mod) {\n    return (mod && mod.__esModule) ? mod : { \"default\": mod };\n};"
        };
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        function transformSystemModule(context) {
            var startLexicalEnvironment = context.startLexicalEnvironment, endLexicalEnvironment = context.endLexicalEnvironment, hoistVariableDeclaration = context.hoistVariableDeclaration;
            var compilerOptions = context.getCompilerOptions();
            var resolver = context.getEmitResolver();
            var host = context.getEmitHost();
            var previousOnSubstituteNode = context.onSubstituteNode;
            var previousOnEmitNode = context.onEmitNode;
            context.onSubstituteNode = onSubstituteNode;
            context.onEmitNode = onEmitNode;
            context.enableSubstitution(75 /* Identifier */); // Substitutes expression identifiers for imported symbols.
            context.enableSubstitution(282 /* ShorthandPropertyAssignment */); // Substitutes expression identifiers for imported symbols
            context.enableSubstitution(209 /* BinaryExpression */); // Substitutes assignments to exported symbols.
            context.enableSubstitution(207 /* PrefixUnaryExpression */); // Substitutes updates to exported symbols.
            context.enableSubstitution(208 /* PostfixUnaryExpression */); // Substitutes updates to exported symbols.
            context.enableSubstitution(219 /* MetaProperty */); // Substitutes 'import.meta'
            context.enableEmitNotification(290 /* SourceFile */); // Restore state when substituting nodes in a file.
            var moduleInfoMap = []; // The ExternalModuleInfo for each file.
            var deferredExports = []; // Exports to defer until an EndOfDeclarationMarker is found.
            var exportFunctionsMap = []; // The export function associated with a source file.
            var noSubstitutionMap = []; // Set of nodes for which substitution rules should be ignored for each file.
            var contextObjectMap = []; // The context object associated with a source file.
            var currentSourceFile; // The current file.
            var moduleInfo; // ExternalModuleInfo for the current file.
            var exportFunction; // The export function for the current file.
            var contextObject; // The context object for the current file.
            var hoistedStatements;
            var enclosingBlockScopedContainer;
            var noSubstitution; // Set of nodes for which substitution rules should be ignored.
            return ts.chainBundle(transformSourceFile);
            /**
             * Transforms the module aspects of a SourceFile.
             *
             * @param node The SourceFile node.
             */
            function transformSourceFile(node) {
                if (node.isDeclarationFile || !(ts.isEffectiveExternalModule(node, compilerOptions) || node.transformFlags & 2097152 /* ContainsDynamicImport */)) {
                    return node;
                }
                var id = ts.getOriginalNodeId(node);
                currentSourceFile = node;
                enclosingBlockScopedContainer = node;
                // System modules have the following shape:
                //
                //     System.register(['dep-1', ... 'dep-n'], function(exports) {/* module body function */})
                //
                // The parameter 'exports' here is a callback '<T>(name: string, value: T) => T' that
                // is used to publish exported values. 'exports' returns its 'value' argument so in
                // most cases expressions that mutate exported values can be rewritten as:
                //
                //     expr -> exports('name', expr)
                //
                // The only exception in this rule is postfix unary operators,
                // see comment to 'substitutePostfixUnaryExpression' for more details
                // Collect information about the external module and dependency groups.
                moduleInfo = moduleInfoMap[id] = ts.collectExternalModuleInfo(node, resolver, compilerOptions);
                // Make sure that the name of the 'exports' function does not conflict with
                // existing identifiers.
                exportFunction = ts.createUniqueName("exports");
                exportFunctionsMap[id] = exportFunction;
                contextObject = contextObjectMap[id] = ts.createUniqueName("context");
                // Add the body of the module.
                var dependencyGroups = collectDependencyGroups(moduleInfo.externalImports);
                var moduleBodyBlock = createSystemModuleBody(node, dependencyGroups);
                var moduleBodyFunction = ts.createFunctionExpression(
                /*modifiers*/ undefined, 
                /*asteriskToken*/ undefined, 
                /*name*/ undefined, 
                /*typeParameters*/ undefined, [
                    ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, exportFunction),
                    ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, contextObject)
                ], 
                /*type*/ undefined, moduleBodyBlock);
                // Write the call to `System.register`
                // Clear the emit-helpers flag for later passes since we'll have already used it in the module body
                // So the helper will be emit at the correct position instead of at the top of the source-file
                var moduleName = ts.tryGetModuleNameFromFile(node, host, compilerOptions);
                var dependencies = ts.createArrayLiteral(ts.map(dependencyGroups, function (dependencyGroup) { return dependencyGroup.name; }));
                var updated = ts.setEmitFlags(ts.updateSourceFileNode(node, ts.setTextRange(ts.createNodeArray([
                    ts.createExpressionStatement(ts.createCall(ts.createPropertyAccess(ts.createIdentifier("System"), "register"), 
                    /*typeArguments*/ undefined, moduleName
                        ? [moduleName, dependencies, moduleBodyFunction]
                        : [dependencies, moduleBodyFunction]))
                ]), node.statements)), 1024 /* NoTrailingComments */);
                if (!(compilerOptions.outFile || compilerOptions.out)) {
                    ts.moveEmitHelpers(updated, moduleBodyBlock, function (helper) { return !helper.scoped; });
                }
                if (noSubstitution) {
                    noSubstitutionMap[id] = noSubstitution;
                    noSubstitution = undefined;
                }
                currentSourceFile = undefined;
                moduleInfo = undefined;
                exportFunction = undefined;
                contextObject = undefined;
                hoistedStatements = undefined;
                enclosingBlockScopedContainer = undefined;
                return ts.aggregateTransformFlags(updated);
            }
            /**
             * Collects the dependency groups for this files imports.
             *
             * @param externalImports The imports for the file.
             */
            function collectDependencyGroups(externalImports) {
                var groupIndices = ts.createMap();
                var dependencyGroups = [];
                for (var _i = 0, externalImports_1 = externalImports; _i < externalImports_1.length; _i++) {
                    var externalImport = externalImports_1[_i];
                    var externalModuleName = ts.getExternalModuleNameLiteral(externalImport, currentSourceFile, host, resolver, compilerOptions);
                    if (externalModuleName) {
                        var text = externalModuleName.text;
                        var groupIndex = groupIndices.get(text);
                        if (groupIndex !== undefined) {
                            // deduplicate/group entries in dependency list by the dependency name
                            dependencyGroups[groupIndex].externalImports.push(externalImport);
                        }
                        else {
                            groupIndices.set(text, dependencyGroups.length);
                            dependencyGroups.push({
                                name: externalModuleName,
                                externalImports: [externalImport]
                            });
                        }
                    }
                }
                return dependencyGroups;
            }
            /**
             * Adds the statements for the module body function for the source file.
             *
             * @param node The source file for the module.
             * @param dependencyGroups The grouped dependencies of the module.
             */
            function createSystemModuleBody(node, dependencyGroups) {
                // Shape of the body in system modules:
                //
                //  function (exports) {
                //      <list of local aliases for imports>
                //      <hoisted variable declarations>
                //      <hoisted function declarations>
                //      return {
                //          setters: [
                //              <list of setter function for imports>
                //          ],
                //          execute: function() {
                //              <module statements>
                //          }
                //      }
                //      <temp declarations>
                //  }
                //
                // i.e:
                //
                //   import {x} from 'file1'
                //   var y = 1;
                //   export function foo() { return y + x(); }
                //   console.log(y);
                //
                // Will be transformed to:
                //
                //  function(exports) {
                //      function foo() { return y + file_1.x(); }
                //      exports("foo", foo);
                //      var file_1, y;
                //      return {
                //          setters: [
                //              function(v) { file_1 = v }
                //          ],
                //          execute(): function() {
                //              y = 1;
                //              console.log(y);
                //          }
                //      };
                //  }
                var statements = [];
                // We start a new lexical environment in this function body, but *not* in the
                // body of the execute function. This allows us to emit temporary declarations
                // only in the outer module body and not in the inner one.
                startLexicalEnvironment();
                // Add any prologue directives.
                var ensureUseStrict = ts.getStrictOptionValue(compilerOptions, "alwaysStrict") || (!compilerOptions.noImplicitUseStrict && ts.isExternalModule(currentSourceFile));
                var statementOffset = ts.addPrologue(statements, node.statements, ensureUseStrict, sourceElementVisitor);
                // var __moduleName = context_1 && context_1.id;
                statements.push(ts.createVariableStatement(
                /*modifiers*/ undefined, ts.createVariableDeclarationList([
                    ts.createVariableDeclaration("__moduleName", 
                    /*type*/ undefined, ts.createLogicalAnd(contextObject, ts.createPropertyAccess(contextObject, "id")))
                ])));
                // Visit the synthetic external helpers import declaration if present
                ts.visitNode(moduleInfo.externalHelpersImportDeclaration, sourceElementVisitor, ts.isStatement);
                // Visit the statements of the source file, emitting any transformations into
                // the `executeStatements` array. We do this *before* we fill the `setters` array
                // as we both emit transformations as well as aggregate some data used when creating
                // setters. This allows us to reduce the number of times we need to loop through the
                // statements of the source file.
                var executeStatements = ts.visitNodes(node.statements, sourceElementVisitor, ts.isStatement, statementOffset);
                // Emit early exports for function declarations.
                ts.addRange(statements, hoistedStatements);
                // We emit hoisted variables early to align roughly with our previous emit output.
                // Two key differences in this approach are:
                // - Temporary variables will appear at the top rather than at the bottom of the file
                ts.insertStatementsAfterStandardPrologue(statements, endLexicalEnvironment());
                var exportStarFunction = addExportStarIfNeeded(statements); // TODO: GH#18217
                var modifiers = node.transformFlags & 524288 /* ContainsAwait */ ?
                    ts.createModifiersFromModifierFlags(256 /* Async */) :
                    undefined;
                var moduleObject = ts.createObjectLiteral([
                    ts.createPropertyAssignment("setters", createSettersArray(exportStarFunction, dependencyGroups)),
                    ts.createPropertyAssignment("execute", ts.createFunctionExpression(modifiers, 
                    /*asteriskToken*/ undefined, 
                    /*name*/ undefined, 
                    /*typeParameters*/ undefined, 
                    /*parameters*/ [], 
                    /*type*/ undefined, ts.createBlock(executeStatements, /*multiLine*/ true)))
                ]);
                moduleObject.multiLine = true;
                statements.push(ts.createReturn(moduleObject));
                return ts.createBlock(statements, /*multiLine*/ true);
            }
            /**
             * Adds an exportStar function to a statement list if it is needed for the file.
             *
             * @param statements A statement list.
             */
            function addExportStarIfNeeded(statements) {
                if (!moduleInfo.hasExportStarsToExportValues) {
                    return;
                }
                // when resolving exports local exported entries/indirect exported entries in the module
                // should always win over entries with similar names that were added via star exports
                // to support this we store names of local/indirect exported entries in a set.
                // this set is used to filter names brought by star expors.
                // local names set should only be added if we have anything exported
                if (!moduleInfo.exportedNames && moduleInfo.exportSpecifiers.size === 0) {
                    // no exported declarations (export var ...) or export specifiers (export {x})
                    // check if we have any non star export declarations.
                    var hasExportDeclarationWithExportClause = false;
                    for (var _i = 0, _a = moduleInfo.externalImports; _i < _a.length; _i++) {
                        var externalImport = _a[_i];
                        if (externalImport.kind === 260 /* ExportDeclaration */ && externalImport.exportClause) {
                            hasExportDeclarationWithExportClause = true;
                            break;
                        }
                    }
                    if (!hasExportDeclarationWithExportClause) {
                        // we still need to emit exportStar helper
                        var exportStarFunction_1 = createExportStarFunction(/*localNames*/ undefined);
                        statements.push(exportStarFunction_1);
                        return exportStarFunction_1.name;
                    }
                }
                var exportedNames = [];
                if (moduleInfo.exportedNames) {
                    for (var _b = 0, _c = moduleInfo.exportedNames; _b < _c.length; _b++) {
                        var exportedLocalName = _c[_b];
                        if (exportedLocalName.escapedText === "default") {
                            continue;
                        }
                        // write name of exported declaration, i.e 'export var x...'
                        exportedNames.push(ts.createPropertyAssignment(ts.createLiteral(exportedLocalName), ts.createTrue()));
                    }
                }
                for (var _d = 0, _e = moduleInfo.externalImports; _d < _e.length; _d++) {
                    var externalImport = _e[_d];
                    if (externalImport.kind !== 260 /* ExportDeclaration */) {
                        continue;
                    }
                    if (!externalImport.exportClause) {
                        // export * from ...
                        continue;
                    }
                    if (ts.isNamedExports(externalImport.exportClause)) {
                        for (var _f = 0, _g = externalImport.exportClause.elements; _f < _g.length; _f++) {
                            var element = _g[_f];
                            // write name of indirectly exported entry, i.e. 'export {x} from ...'
                            exportedNames.push(ts.createPropertyAssignment(ts.createLiteral(ts.idText(element.name || element.propertyName)), ts.createTrue()));
                        }
                    }
                    else {
                        exportedNames.push(ts.createPropertyAssignment(ts.createLiteral(ts.idText(externalImport.exportClause.name)), ts.createTrue()));
                    }
                }
                var exportedNamesStorageRef = ts.createUniqueName("exportedNames");
                statements.push(ts.createVariableStatement(
                /*modifiers*/ undefined, ts.createVariableDeclarationList([
                    ts.createVariableDeclaration(exportedNamesStorageRef, 
                    /*type*/ undefined, ts.createObjectLiteral(exportedNames, /*multiline*/ true))
                ])));
                var exportStarFunction = createExportStarFunction(exportedNamesStorageRef);
                statements.push(exportStarFunction);
                return exportStarFunction.name;
            }
            /**
             * Creates an exportStar function for the file, with an optional set of excluded local
             * names.
             *
             * @param localNames An optional reference to an object containing a set of excluded local
             * names.
             */
            function createExportStarFunction(localNames) {
                var exportStarFunction = ts.createUniqueName("exportStar");
                var m = ts.createIdentifier("m");
                var n = ts.createIdentifier("n");
                var exports = ts.createIdentifier("exports");
                var condition = ts.createStrictInequality(n, ts.createLiteral("default"));
                if (localNames) {
                    condition = ts.createLogicalAnd(condition, ts.createLogicalNot(ts.createCall(ts.createPropertyAccess(localNames, "hasOwnProperty"), 
                    /*typeArguments*/ undefined, [n])));
                }
                return ts.createFunctionDeclaration(
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, 
                /*asteriskToken*/ undefined, exportStarFunction, 
                /*typeParameters*/ undefined, [ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, m)], 
                /*type*/ undefined, ts.createBlock([
                    ts.createVariableStatement(
                    /*modifiers*/ undefined, ts.createVariableDeclarationList([
                        ts.createVariableDeclaration(exports, 
                        /*type*/ undefined, ts.createObjectLiteral([]))
                    ])),
                    ts.createForIn(ts.createVariableDeclarationList([
                        ts.createVariableDeclaration(n, /*type*/ undefined)
                    ]), m, ts.createBlock([
                        ts.setEmitFlags(ts.createIf(condition, ts.createExpressionStatement(ts.createAssignment(ts.createElementAccess(exports, n), ts.createElementAccess(m, n)))), 1 /* SingleLine */)
                    ])),
                    ts.createExpressionStatement(ts.createCall(exportFunction, 
                    /*typeArguments*/ undefined, [exports]))
                ], /*multiline*/ true));
            }
            /**
             * Creates an array setter callbacks for each dependency group.
             *
             * @param exportStarFunction A reference to an exportStarFunction for the file.
             * @param dependencyGroups An array of grouped dependencies.
             */
            function createSettersArray(exportStarFunction, dependencyGroups) {
                var setters = [];
                for (var _i = 0, dependencyGroups_1 = dependencyGroups; _i < dependencyGroups_1.length; _i++) {
                    var group_2 = dependencyGroups_1[_i];
                    // derive a unique name for parameter from the first named entry in the group
                    var localName = ts.forEach(group_2.externalImports, function (i) { return ts.getLocalNameForExternalImport(i, currentSourceFile); });
                    var parameterName = localName ? ts.getGeneratedNameForNode(localName) : ts.createUniqueName("");
                    var statements = [];
                    for (var _a = 0, _b = group_2.externalImports; _a < _b.length; _a++) {
                        var entry = _b[_a];
                        var importVariableName = ts.getLocalNameForExternalImport(entry, currentSourceFile); // TODO: GH#18217
                        switch (entry.kind) {
                            case 254 /* ImportDeclaration */:
                                if (!entry.importClause) {
                                    // 'import "..."' case
                                    // module is imported only for side-effects, no emit required
                                    break;
                                }
                            // falls through
                            case 253 /* ImportEqualsDeclaration */:
                                ts.Debug.assert(importVariableName !== undefined);
                                // save import into the local
                                statements.push(ts.createExpressionStatement(ts.createAssignment(importVariableName, parameterName)));
                                break;
                            case 260 /* ExportDeclaration */:
                                ts.Debug.assert(importVariableName !== undefined);
                                if (entry.exportClause) {
                                    if (ts.isNamedExports(entry.exportClause)) {
                                        //  export {a, b as c} from 'foo'
                                        //
                                        // emit as:
                                        //
                                        //  exports_({
                                        //     "a": _["a"],
                                        //     "c": _["b"]
                                        //  });
                                        var properties = [];
                                        for (var _c = 0, _d = entry.exportClause.elements; _c < _d.length; _c++) {
                                            var e = _d[_c];
                                            properties.push(ts.createPropertyAssignment(ts.createLiteral(ts.idText(e.name)), ts.createElementAccess(parameterName, ts.createLiteral(ts.idText(e.propertyName || e.name)))));
                                        }
                                        statements.push(ts.createExpressionStatement(ts.createCall(exportFunction, 
                                        /*typeArguments*/ undefined, [ts.createObjectLiteral(properties, /*multiline*/ true)])));
                                    }
                                    else {
                                        statements.push(ts.createExpressionStatement(ts.createCall(exportFunction, 
                                        /*typeArguments*/ undefined, [
                                            ts.createLiteral(ts.idText(entry.exportClause.name)),
                                            parameterName
                                        ])));
                                    }
                                }
                                else {
                                    //  export * from 'foo'
                                    //
                                    // emit as:
                                    //
                                    //  exportStar(foo_1_1);
                                    statements.push(ts.createExpressionStatement(ts.createCall(exportStarFunction, 
                                    /*typeArguments*/ undefined, [parameterName])));
                                }
                                break;
                        }
                    }
                    setters.push(ts.createFunctionExpression(
                    /*modifiers*/ undefined, 
                    /*asteriskToken*/ undefined, 
                    /*name*/ undefined, 
                    /*typeParameters*/ undefined, [ts.createParameter(/*decorators*/ undefined, /*modifiers*/ undefined, /*dotDotDotToken*/ undefined, parameterName)], 
                    /*type*/ undefined, ts.createBlock(statements, /*multiLine*/ true)));
                }
                return ts.createArrayLiteral(setters, /*multiLine*/ true);
            }
            //
            // Top-level Source Element Visitors
            //
            /**
             * Visit source elements at the top-level of a module.
             *
             * @param node The node to visit.
             */
            function sourceElementVisitor(node) {
                switch (node.kind) {
                    case 254 /* ImportDeclaration */:
                        return visitImportDeclaration(node);
                    case 253 /* ImportEqualsDeclaration */:
                        return visitImportEqualsDeclaration(node);
                    case 260 /* ExportDeclaration */:
                        return visitExportDeclaration(node);
                    case 259 /* ExportAssignment */:
                        return visitExportAssignment(node);
                    default:
                        return nestedElementVisitor(node);
                }
            }
            /**
             * Visits an ImportDeclaration node.
             *
             * @param node The node to visit.
             */
            function visitImportDeclaration(node) {
                var statements;
                if (node.importClause) {
                    hoistVariableDeclaration(ts.getLocalNameForExternalImport(node, currentSourceFile)); // TODO: GH#18217
                }
                if (hasAssociatedEndOfDeclarationMarker(node)) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportsOfImportDeclaration(deferredExports[id], node);
                }
                else {
                    statements = appendExportsOfImportDeclaration(statements, node);
                }
                return ts.singleOrMany(statements);
            }
            function visitExportDeclaration(node) {
                ts.Debug.assertDefined(node);
                return undefined;
            }
            /**
             * Visits an ImportEqualsDeclaration node.
             *
             * @param node The node to visit.
             */
            function visitImportEqualsDeclaration(node) {
                ts.Debug.assert(ts.isExternalModuleImportEqualsDeclaration(node), "import= for internal module references should be handled in an earlier transformer.");
                var statements;
                hoistVariableDeclaration(ts.getLocalNameForExternalImport(node, currentSourceFile)); // TODO: GH#18217
                if (hasAssociatedEndOfDeclarationMarker(node)) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportsOfImportEqualsDeclaration(deferredExports[id], node);
                }
                else {
                    statements = appendExportsOfImportEqualsDeclaration(statements, node);
                }
                return ts.singleOrMany(statements);
            }
            /**
             * Visits an ExportAssignment node.
             *
             * @param node The node to visit.
             */
            function visitExportAssignment(node) {
                if (node.isExportEquals) {
                    // Elide `export=` as it is illegal in a SystemJS module.
                    return undefined;
                }
                var expression = ts.visitNode(node.expression, destructuringAndImportCallVisitor, ts.isExpression);
                var original = node.original;
                if (original && hasAssociatedEndOfDeclarationMarker(original)) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportStatement(deferredExports[id], ts.createIdentifier("default"), expression, /*allowComments*/ true);
                }
                else {
                    return createExportStatement(ts.createIdentifier("default"), expression, /*allowComments*/ true);
                }
            }
            /**
             * Visits a FunctionDeclaration, hoisting it to the outer module body function.
             *
             * @param node The node to visit.
             */
            function visitFunctionDeclaration(node) {
                if (ts.hasModifier(node, 1 /* Export */)) {
                    hoistedStatements = ts.append(hoistedStatements, ts.updateFunctionDeclaration(node, node.decorators, ts.visitNodes(node.modifiers, modifierVisitor, ts.isModifier), node.asteriskToken, ts.getDeclarationName(node, /*allowComments*/ true, /*allowSourceMaps*/ true), 
                    /*typeParameters*/ undefined, ts.visitNodes(node.parameters, destructuringAndImportCallVisitor, ts.isParameterDeclaration), 
                    /*type*/ undefined, ts.visitNode(node.body, destructuringAndImportCallVisitor, ts.isBlock)));
                }
                else {
                    hoistedStatements = ts.append(hoistedStatements, ts.visitEachChild(node, destructuringAndImportCallVisitor, context));
                }
                if (hasAssociatedEndOfDeclarationMarker(node)) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportsOfHoistedDeclaration(deferredExports[id], node);
                }
                else {
                    hoistedStatements = appendExportsOfHoistedDeclaration(hoistedStatements, node);
                }
                return undefined;
            }
            /**
             * Visits a ClassDeclaration, hoisting its name to the outer module body function.
             *
             * @param node The node to visit.
             */
            function visitClassDeclaration(node) {
                var statements;
                // Hoist the name of the class declaration to the outer module body function.
                var name = ts.getLocalName(node);
                hoistVariableDeclaration(name);
                // Rewrite the class declaration into an assignment of a class expression.
                statements = ts.append(statements, ts.setTextRange(ts.createExpressionStatement(ts.createAssignment(name, ts.setTextRange(ts.createClassExpression(
                /*modifiers*/ undefined, node.name, 
                /*typeParameters*/ undefined, ts.visitNodes(node.heritageClauses, destructuringAndImportCallVisitor, ts.isHeritageClause), ts.visitNodes(node.members, destructuringAndImportCallVisitor, ts.isClassElement)), node))), node));
                if (hasAssociatedEndOfDeclarationMarker(node)) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportsOfHoistedDeclaration(deferredExports[id], node);
                }
                else {
                    statements = appendExportsOfHoistedDeclaration(statements, node);
                }
                return ts.singleOrMany(statements);
            }
            /**
             * Visits a variable statement, hoisting declared names to the top-level module body.
             * Each declaration is rewritten into an assignment expression.
             *
             * @param node The node to visit.
             */
            function visitVariableStatement(node) {
                if (!shouldHoistVariableDeclarationList(node.declarationList)) {
                    return ts.visitNode(node, destructuringAndImportCallVisitor, ts.isStatement);
                }
                var expressions;
                var isExportedDeclaration = ts.hasModifier(node, 1 /* Export */);
                var isMarkedDeclaration = hasAssociatedEndOfDeclarationMarker(node);
                for (var _i = 0, _a = node.declarationList.declarations; _i < _a.length; _i++) {
                    var variable = _a[_i];
                    if (variable.initializer) {
                        expressions = ts.append(expressions, transformInitializedVariable(variable, isExportedDeclaration && !isMarkedDeclaration));
                    }
                    else {
                        hoistBindingElement(variable);
                    }
                }
                var statements;
                if (expressions) {
                    statements = ts.append(statements, ts.setTextRange(ts.createExpressionStatement(ts.inlineExpressions(expressions)), node));
                }
                if (isMarkedDeclaration) {
                    // Defer exports until we encounter an EndOfDeclarationMarker node
                    var id = ts.getOriginalNodeId(node);
                    deferredExports[id] = appendExportsOfVariableStatement(deferredExports[id], node, isExportedDeclaration);
                }
                else {
                    statements = appendExportsOfVariableStatement(statements, node, /*exportSelf*/ false);
                }
                return ts.singleOrMany(statements);
            }
            /**
             * Hoists the declared names of a VariableDeclaration or BindingElement.
             *
             * @param node The declaration to hoist.
             */
            function hoistBindingElement(node) {
                if (ts.isBindingPattern(node.name)) {
                    for (var _i = 0, _a = node.name.elements; _i < _a.length; _i++) {
                        var element = _a[_i];
                        if (!ts.isOmittedExpression(element)) {
                            hoistBindingElement(element);
                        }
                    }
                }
                else {
                    hoistVariableDeclaration(ts.getSynthesizedClone(node.name));
                }
            }
            /**
             * Determines whether a VariableDeclarationList should be hoisted.
             *
             * @param node The node to test.
             */
            function shouldHoistVariableDeclarationList(node) {
                // hoist only non-block scoped declarations or block scoped declarations parented by source file
                return (ts.getEmitFlags(node) & 2097152 /* NoHoisting */) === 0
                    && (enclosingBlockScopedContainer.kind === 290 /* SourceFile */
                        || (ts.getOriginalNode(node).flags & 3 /* BlockScoped */) === 0);
            }
            /**
             * Transform an initialized variable declaration into an expression.
             *
             * @param node The node to transform.
             * @param isExportedDeclaration A value indicating whether the variable is exported.
             */
            function transformInitializedVariable(node, isExportedDeclaration) {
                var createAssignment = isExportedDeclaration ? createExportedVariableAssignment : createNonExportedVariableAssignment;
                return ts.isBindingPattern(node.name)
                    ? ts.flattenDestructuringAssignment(node, destructuringAndImportCallVisitor, context, 0 /* All */, 
                    /*needsValue*/ false, createAssignment)
                    : node.initializer ? createAssignment(node.name, ts.visitNode(node.initializer, destructuringAndImportCallVisitor, ts.isExpression)) : node.name;
            }
            /**
             * Creates an assignment expression for an exported variable declaration.
             *
             * @param name The name of the variable.
             * @param value The value of the variable's initializer.
             * @param location The source map location for the assignment.
             */
            function createExportedVariableAssignment(name, value, location) {
                return createVariableAssignment(name, value, location, /*isExportedDeclaration*/ true);
            }
            /**
             * Creates an assignment expression for a non-exported variable declaration.
             *
             * @param name The name of the variable.
             * @param value The value of the variable's initializer.
             * @param location The source map location for the assignment.
             */
            function createNonExportedVariableAssignment(name, value, location) {
                return createVariableAssignment(name, value, location, /*isExportedDeclaration*/ false);
            }
            /**
             * Creates an assignment expression for a variable declaration.
             *
             * @param name The name of the variable.
             * @param value The value of the variable's initializer.
             * @param location The source map location for the assignment.
             * @param isExportedDeclaration A value indicating whether the variable is exported.
             */
            function createVariableAssignment(name, value, location, isExportedDeclaration) {
                hoistVariableDeclaration(ts.getSynthesizedClone(name));
                return isExportedDeclaration
                    ? createExportExpression(name, preventSubstitution(ts.setTextRange(ts.createAssignment(name, value), location)))
                    : preventSubstitution(ts.setTextRange(ts.createAssignment(name, value), location));
            }
            /**
             * Visits a MergeDeclarationMarker used as a placeholder for the beginning of a merged
             * and transformed declaration.
             *
             * @param node The node to visit.
             */
            function visitMergeDeclarationMarker(node) {
                // For an EnumDeclaration or ModuleDeclaration that merges with a preceeding
                // declaration we do not emit a leading variable declaration. To preserve the
                // begin/end semantics of the declararation and to properly handle exports
                // we wrapped the leading variable declaration in a `MergeDeclarationMarker`.
                //
                // To balance the declaration, we defer the exports of the elided variable
                // statement until we visit this declaration's `EndOfDeclarationMarker`.
                if (hasAssociatedEndOfDeclarationMarker(node) && node.original.kind === 225 /* VariableStatement */) {
                    var id = ts.getOriginalNodeId(node);
                    var isExportedDeclaration = ts.hasModifier(node.original, 1 /* Export */);
                    deferredExports[id] = appendExportsOfVariableStatement(deferredExports[id], node.original, isExportedDeclaration);
                }
                return node;
            }
            /**
             * Determines whether a node has an associated EndOfDeclarationMarker.
             *
             * @param node The node to test.
             */
            function hasAssociatedEndOfDeclarationMarker(node) {
                return (ts.getEmitFlags(node) & 4194304 /* HasEndOfDeclarationMarker */) !== 0;
            }
            /**
             * Visits a DeclarationMarker used as a placeholder for the end of a transformed
             * declaration.
             *
             * @param node The node to visit.
             */
            function visitEndOfDeclarationMarker(node) {
                // For some transformations we emit an `EndOfDeclarationMarker` to mark the actual
                // end of the transformed declaration. We use this marker to emit any deferred exports
                // of the declaration.
                var id = ts.getOriginalNodeId(node);
                var statements = deferredExports[id];
                if (statements) {
                    delete deferredExports[id];
                    return ts.append(statements, node);
                }
                else {
                    var original = ts.getOriginalNode(node);
                    if (ts.isModuleOrEnumDeclaration(original)) {
                        return ts.append(appendExportsOfDeclaration(statements, original), node);
                    }
                }
                return node;
            }
            /**
             * Appends the exports of an ImportDeclaration to a statement list, returning the
             * statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param decl The declaration whose exports are to be recorded.
             */
            function appendExportsOfImportDeclaration(statements, decl) {
                if (moduleInfo.exportEquals) {
                    return statements;
                }
                var importClause = decl.importClause;
                if (!importClause) {
                    return statements;
                }
                if (importClause.name) {
                    statements = appendExportsOfDeclaration(statements, importClause);
                }
                var namedBindings = importClause.namedBindings;
                if (namedBindings) {
                    switch (namedBindings.kind) {
                        case 256 /* NamespaceImport */:
                            statements = appendExportsOfDeclaration(statements, namedBindings);
                            break;
                        case 257 /* NamedImports */:
                            for (var _i = 0, _a = namedBindings.elements; _i < _a.length; _i++) {
                                var importBinding = _a[_i];
                                statements = appendExportsOfDeclaration(statements, importBinding);
                            }
                            break;
                    }
                }
                return statements;
            }
            /**
             * Appends the export of an ImportEqualsDeclaration to a statement list, returning the
             * statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param decl The declaration whose exports are to be recorded.
             */
            function appendExportsOfImportEqualsDeclaration(statements, decl) {
                if (moduleInfo.exportEquals) {
                    return statements;
                }
                return appendExportsOfDeclaration(statements, decl);
            }
            /**
             * Appends the exports of a VariableStatement to a statement list, returning the statement
             * list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param node The VariableStatement whose exports are to be recorded.
             * @param exportSelf A value indicating whether to also export each VariableDeclaration of
             * `nodes` declaration list.
             */
            function appendExportsOfVariableStatement(statements, node, exportSelf) {
                if (moduleInfo.exportEquals) {
                    return statements;
                }
                for (var _i = 0, _a = node.declarationList.declarations; _i < _a.length; _i++) {
                    var decl = _a[_i];
                    if (decl.initializer || exportSelf) {
                        statements = appendExportsOfBindingElement(statements, decl, exportSelf);
                    }
                }
                return statements;
            }
            /**
             * Appends the exports of a VariableDeclaration or BindingElement to a statement list,
             * returning the statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param decl The declaration whose exports are to be recorded.
             * @param exportSelf A value indicating whether to also export the declaration itself.
             */
            function appendExportsOfBindingElement(statements, decl, exportSelf) {
                if (moduleInfo.exportEquals) {
                    return statements;
                }
                if (ts.isBindingPattern(decl.name)) {
                    for (var _i = 0, _a = decl.name.elements; _i < _a.length; _i++) {
                        var element = _a[_i];
                        if (!ts.isOmittedExpression(element)) {
                            statements = appendExportsOfBindingElement(statements, element, exportSelf);
                        }
                    }
                }
                else if (!ts.isGeneratedIdentifier(decl.name)) {
                    var excludeName = void 0;
                    if (exportSelf) {
                        statements = appendExportStatement(statements, decl.name, ts.getLocalName(decl));
                        excludeName = ts.idText(decl.name);
                    }
                    statements = appendExportsOfDeclaration(statements, decl, excludeName);
                }
                return statements;
            }
            /**
             * Appends the exports of a ClassDeclaration or FunctionDeclaration to a statement list,
             * returning the statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param decl The declaration whose exports are to be recorded.
             */
            function appendExportsOfHoistedDeclaration(statements, decl) {
                if (moduleInfo.exportEquals) {
                    return statements;
                }
                var excludeName;
                if (ts.hasModifier(decl, 1 /* Export */)) {
                    var exportName = ts.hasModifier(decl, 512 /* Default */) ? ts.createLiteral("default") : decl.name;
                    statements = appendExportStatement(statements, exportName, ts.getLocalName(decl));
                    excludeName = ts.getTextOfIdentifierOrLiteral(exportName);
                }
                if (decl.name) {
                    statements = appendExportsOfDeclaration(statements, decl, excludeName);
                }
                return statements;
            }
            /**
             * Appends the exports of a declaration to a statement list, returning the statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param decl The declaration to export.
             * @param excludeName An optional name to exclude from exports.
             */
            function appendExportsOfDeclaration(statements, decl, excludeName) {
                if (moduleInfo.exportEquals) {
                    return statements;
                }
                var name = ts.getDeclarationName(decl);
                var exportSpecifiers = moduleInfo.exportSpecifiers.get(ts.idText(name));
                if (exportSpecifiers) {
                    for (var _i = 0, exportSpecifiers_2 = exportSpecifiers; _i < exportSpecifiers_2.length; _i++) {
                        var exportSpecifier = exportSpecifiers_2[_i];
                        if (exportSpecifier.name.escapedText !== excludeName) {
                            statements = appendExportStatement(statements, exportSpecifier.name, name);
                        }
                    }
                }
                return statements;
            }
            /**
             * Appends the down-level representation of an export to a statement list, returning the
             * statement list.
             *
             * @param statements A statement list to which the down-level export statements are to be
             * appended. If `statements` is `undefined`, a new array is allocated if statements are
             * appended.
             * @param exportName The name of the export.
             * @param expression The expression to export.
             * @param allowComments Whether to allow comments on the export.
             */
            function appendExportStatement(statements, exportName, expression, allowComments) {
                statements = ts.append(statements, createExportStatement(exportName, expression, allowComments));
                return statements;
            }
            /**
             * Creates a call to the current file's export function to export a value.
             *
             * @param name The bound name of the export.
             * @param value The exported value.
             * @param allowComments An optional value indicating whether to emit comments for the statement.
             */
            function createExportStatement(name, value, allowComments) {
                var statement = ts.createExpressionStatement(createExportExpression(name, value));
                ts.startOnNewLine(statement);
                if (!allowComments) {
                    ts.setEmitFlags(statement, 1536 /* NoComments */);
                }
                return statement;
            }
            /**
             * Creates a call to the current file's export function to export a value.
             *
             * @param name The bound name of the export.
             * @param value The exported value.
             */
            function createExportExpression(name, value) {
                var exportName = ts.isIdentifier(name) ? ts.createLiteral(name) : name;
                ts.setEmitFlags(value, ts.getEmitFlags(value) | 1536 /* NoComments */);
                return ts.setCommentRange(ts.createCall(exportFunction, /*typeArguments*/ undefined, [exportName, value]), value);
            }
            //
            // Top-Level or Nested Source Element Visitors
            //
            /**
             * Visit nested elements at the top-level of a module.
             *
             * @param node The node to visit.
             */
            function nestedElementVisitor(node) {
                switch (node.kind) {
                    case 225 /* VariableStatement */:
                        return visitVariableStatement(node);
                    case 244 /* FunctionDeclaration */:
                        return visitFunctionDeclaration(node);
                    case 245 /* ClassDeclaration */:
                        return visitClassDeclaration(node);
                    case 230 /* ForStatement */:
                        return visitForStatement(node);
                    case 231 /* ForInStatement */:
                        return visitForInStatement(node);
                    case 232 /* ForOfStatement */:
                        return visitForOfStatement(node);
                    case 228 /* DoStatement */:
                        return visitDoStatement(node);
                    case 229 /* WhileStatement */:
                        return visitWhileStatement(node);
                    case 238 /* LabeledStatement */:
                        return visitLabeledStatement(node);
                    case 236 /* WithStatement */:
                        return visitWithStatement(node);
                    case 237 /* SwitchStatement */:
                        return visitSwitchStatement(node);
                    case 251 /* CaseBlock */:
                        return visitCaseBlock(node);
                    case 277 /* CaseClause */:
                        return visitCaseClause(node);
                    case 278 /* DefaultClause */:
                        return visitDefaultClause(node);
                    case 240 /* TryStatement */:
                        return visitTryStatement(node);
                    case 280 /* CatchClause */:
                        return visitCatchClause(node);
                    case 223 /* Block */:
                        return visitBlock(node);
                    case 327 /* MergeDeclarationMarker */:
                        return visitMergeDeclarationMarker(node);
                    case 328 /* EndOfDeclarationMarker */:
                        return visitEndOfDeclarationMarker(node);
                    default:
                        return destructuringAndImportCallVisitor(node);
                }
            }
            /**
             * Visits the body of a ForStatement to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitForStatement(node) {
                var savedEnclosingBlockScopedContainer = enclosingBlockScopedContainer;
                enclosingBlockScopedContainer = node;
                node = ts.updateFor(node, node.initializer && visitForInitializer(node.initializer), ts.visitNode(node.condition, destructuringAndImportCallVisitor, ts.isExpression), ts.visitNode(node.incrementor, destructuringAndImportCallVisitor, ts.isExpression), ts.visitNode(node.statement, nestedElementVisitor, ts.isStatement));
                enclosingBlockScopedContainer = savedEnclosingBlockScopedContainer;
                return node;
            }
            /**
             * Visits the body of a ForInStatement to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitForInStatement(node) {
                var savedEnclosingBlockScopedContainer = enclosingBlockScopedContainer;
                enclosingBlockScopedContainer = node;
                node = ts.updateForIn(node, visitForInitializer(node.initializer), ts.visitNode(node.expression, destructuringAndImportCallVisitor, ts.isExpression), ts.visitNode(node.statement, nestedElementVisitor, ts.isStatement, ts.liftToBlock));
                enclosingBlockScopedContainer = savedEnclosingBlockScopedContainer;
                return node;
            }
            /**
             * Visits the body of a ForOfStatement to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitForOfStatement(node) {
                var savedEnclosingBlockScopedContainer = enclosingBlockScopedContainer;
                enclosingBlockScopedContainer = node;
                node = ts.updateForOf(node, node.awaitModifier, visitForInitializer(node.initializer), ts.visitNode(node.expression, destructuringAndImportCallVisitor, ts.isExpression), ts.visitNode(node.statement, nestedElementVisitor, ts.isStatement, ts.liftToBlock));
                enclosingBlockScopedContainer = savedEnclosingBlockScopedContainer;
                return node;
            }
            /**
             * Determines whether to hoist the initializer of a ForStatement, ForInStatement, or
             * ForOfStatement.
             *
             * @param node The node to test.
             */
            function shouldHoistForInitializer(node) {
                return ts.isVariableDeclarationList(node)
                    && shouldHoistVariableDeclarationList(node);
            }
            /**
             * Visits the initializer of a ForStatement, ForInStatement, or ForOfStatement
             *
             * @param node The node to visit.
             */
            function visitForInitializer(node) {
                if (shouldHoistForInitializer(node)) {
                    var expressions = void 0;
                    for (var _i = 0, _a = node.declarations; _i < _a.length; _i++) {
                        var variable = _a[_i];
                        expressions = ts.append(expressions, transformInitializedVariable(variable, /*isExportedDeclaration*/ false));
                        if (!variable.initializer) {
                            hoistBindingElement(variable);
                        }
                    }
                    return expressions ? ts.inlineExpressions(expressions) : ts.createOmittedExpression();
                }
                else {
                    return ts.visitEachChild(node, nestedElementVisitor, context);
                }
            }
            /**
             * Visits the body of a DoStatement to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitDoStatement(node) {
                return ts.updateDo(node, ts.visitNode(node.statement, nestedElementVisitor, ts.isStatement, ts.liftToBlock), ts.visitNode(node.expression, destructuringAndImportCallVisitor, ts.isExpression));
            }
            /**
             * Visits the body of a WhileStatement to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitWhileStatement(node) {
                return ts.updateWhile(node, ts.visitNode(node.expression, destructuringAndImportCallVisitor, ts.isExpression), ts.visitNode(node.statement, nestedElementVisitor, ts.isStatement, ts.liftToBlock));
            }
            /**
             * Visits the body of a LabeledStatement to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitLabeledStatement(node) {
                return ts.updateLabel(node, node.label, ts.visitNode(node.statement, nestedElementVisitor, ts.isStatement, ts.liftToBlock));
            }
            /**
             * Visits the body of a WithStatement to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitWithStatement(node) {
                return ts.updateWith(node, ts.visitNode(node.expression, destructuringAndImportCallVisitor, ts.isExpression), ts.visitNode(node.statement, nestedElementVisitor, ts.isStatement, ts.liftToBlock));
            }
            /**
             * Visits the body of a SwitchStatement to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitSwitchStatement(node) {
                return ts.updateSwitch(node, ts.visitNode(node.expression, destructuringAndImportCallVisitor, ts.isExpression), ts.visitNode(node.caseBlock, nestedElementVisitor, ts.isCaseBlock));
            }
            /**
             * Visits the body of a CaseBlock to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitCaseBlock(node) {
                var savedEnclosingBlockScopedContainer = enclosingBlockScopedContainer;
                enclosingBlockScopedContainer = node;
                node = ts.updateCaseBlock(node, ts.visitNodes(node.clauses, nestedElementVisitor, ts.isCaseOrDefaultClause));
                enclosingBlockScopedContainer = savedEnclosingBlockScopedContainer;
                return node;
            }
            /**
             * Visits the body of a CaseClause to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitCaseClause(node) {
                return ts.updateCaseClause(node, ts.visitNode(node.expression, destructuringAndImportCallVisitor, ts.isExpression), ts.visitNodes(node.statements, nestedElementVisitor, ts.isStatement));
            }
            /**
             * Visits the body of a DefaultClause to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitDefaultClause(node) {
                return ts.visitEachChild(node, nestedElementVisitor, context);
            }
            /**
             * Visits the body of a TryStatement to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitTryStatement(node) {
                return ts.visitEachChild(node, nestedElementVisitor, context);
            }
            /**
             * Visits the body of a CatchClause to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitCatchClause(node) {
                var savedEnclosingBlockScopedContainer = enclosingBlockScopedContainer;
                enclosingBlockScopedContainer = node;
                node = ts.updateCatchClause(node, node.variableDeclaration, ts.visitNode(node.block, nestedElementVisitor, ts.isBlock));
                enclosingBlockScopedContainer = savedEnclosingBlockScopedContainer;
                return node;
            }
            /**
             * Visits the body of a Block to hoist declarations.
             *
             * @param node The node to visit.
             */
            function visitBlock(node) {
                var savedEnclosingBlockScopedContainer = enclosingBlockScopedContainer;
                enclosingBlockScopedContainer = node;
                node = ts.visitEachChild(node, nestedElementVisitor, context);
                enclosingBlockScopedContainer = savedEnclosingBlockScopedContainer;
                return node;
            }
            //
            // Destructuring Assignment Visitors
            //
            /**
             * Visit nodes to flatten destructuring assignments to exported symbols.
             *
             * @param node The node to visit.
             */
            function destructuringAndImportCallVisitor(node) {
                if (ts.isDestructuringAssignment(node)) {
                    return visitDestructuringAssignment(node);
                }
                else if (ts.isImportCall(node)) {
                    return visitImportCallExpression(node);
                }
                else if ((node.transformFlags & 1024 /* ContainsDestructuringAssignment */) || (node.transformFlags & 2097152 /* ContainsDynamicImport */)) {
                    return ts.visitEachChild(node, destructuringAndImportCallVisitor, context);
                }
                else {
                    return node;
                }
            }
            function visitImportCallExpression(node) {
                // import("./blah")
                // emit as
                // System.register([], function (_export, _context) {
                //     return {
                //         setters: [],
                //         execute: () => {
                //             _context.import('./blah');
                //         }
                //     };
                // });
                return ts.createCall(ts.createPropertyAccess(contextObject, ts.createIdentifier("import")), 
                /*typeArguments*/ undefined, ts.some(node.arguments) ? [ts.visitNode(node.arguments[0], destructuringAndImportCallVisitor)] : []);
            }
            /**
             * Visits a DestructuringAssignment to flatten destructuring to exported symbols.
             *
             * @param node The node to visit.
             */
            function visitDestructuringAssignment(node) {
                if (hasExportedReferenceInDestructuringTarget(node.left)) {
                    return ts.flattenDestructuringAssignment(node, destructuringAndImportCallVisitor, context, 0 /* All */, 
                    /*needsValue*/ true);
                }
                return ts.visitEachChild(node, destructuringAndImportCallVisitor, context);
            }
            /**
             * Determines whether the target of a destructuring assigment refers to an exported symbol.
             *
             * @param node The destructuring target.
             */
            function hasExportedReferenceInDestructuringTarget(node) {
                if (ts.isAssignmentExpression(node, /*excludeCompoundAssignment*/ true)) {
                    return hasExportedReferenceInDestructuringTarget(node.left);
                }
                else if (ts.isSpreadElement(node)) {
                    return hasExportedReferenceInDestructuringTarget(node.expression);
                }
                else if (ts.isObjectLiteralExpression(node)) {
                    return ts.some(node.properties, hasExportedReferenceInDestructuringTarget);
                }
                else if (ts.isArrayLiteralExpression(node)) {
                    return ts.some(node.elements, hasExportedReferenceInDestructuringTarget);
                }
                else if (ts.isShorthandPropertyAssignment(node)) {
                    return hasExportedReferenceInDestructuringTarget(node.name);
                }
                else if (ts.isPropertyAssignment(node)) {
                    return hasExportedReferenceInDestructuringTarget(node.initializer);
                }
                else if (ts.isIdentifier(node)) {
                    var container = resolver.getReferencedExportContainer(node);
                    return container !== undefined && container.kind === 290 /* SourceFile */;
                }
                else {
                    return false;
                }
            }
            //
            // Modifier Visitors
            //
            /**
             * Visit nodes to elide module-specific modifiers.
             *
             * @param node The node to visit.
             */
            function modifierVisitor(node) {
                switch (node.kind) {
                    case 89 /* ExportKeyword */:
                    case 84 /* DefaultKeyword */:
                        return undefined;
                }
                return node;
            }
            //
            // Emit Notification
            //
            /**
             * Hook for node emit notifications.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to emit.
             * @param emitCallback A callback used to emit the node in the printer.
             */
            function onEmitNode(hint, node, emitCallback) {
                if (node.kind === 290 /* SourceFile */) {
                    var id = ts.getOriginalNodeId(node);
                    currentSourceFile = node;
                    moduleInfo = moduleInfoMap[id];
                    exportFunction = exportFunctionsMap[id];
                    noSubstitution = noSubstitutionMap[id];
                    contextObject = contextObjectMap[id];
                    if (noSubstitution) {
                        delete noSubstitutionMap[id];
                    }
                    previousOnEmitNode(hint, node, emitCallback);
                    currentSourceFile = undefined;
                    moduleInfo = undefined;
                    exportFunction = undefined;
                    contextObject = undefined;
                    noSubstitution = undefined;
                }
                else {
                    previousOnEmitNode(hint, node, emitCallback);
                }
            }
            //
            // Substitutions
            //
            /**
             * Hooks node substitutions.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to substitute.
             */
            function onSubstituteNode(hint, node) {
                node = previousOnSubstituteNode(hint, node);
                if (isSubstitutionPrevented(node)) {
                    return node;
                }
                if (hint === 1 /* Expression */) {
                    return substituteExpression(node);
                }
                else if (hint === 4 /* Unspecified */) {
                    return substituteUnspecified(node);
                }
                return node;
            }
            /**
             * Substitute the node, if necessary.
             *
             * @param node The node to substitute.
             */
            function substituteUnspecified(node) {
                switch (node.kind) {
                    case 282 /* ShorthandPropertyAssignment */:
                        return substituteShorthandPropertyAssignment(node);
                }
                return node;
            }
            /**
             * Substitution for a ShorthandPropertyAssignment whose name that may contain an imported or exported symbol.
             *
             * @param node The node to substitute.
             */
            function substituteShorthandPropertyAssignment(node) {
                var name = node.name;
                if (!ts.isGeneratedIdentifier(name) && !ts.isLocalName(name)) {
                    var importDeclaration = resolver.getReferencedImportDeclaration(name);
                    if (importDeclaration) {
                        if (ts.isImportClause(importDeclaration)) {
                            return ts.setTextRange(ts.createPropertyAssignment(ts.getSynthesizedClone(name), ts.createPropertyAccess(ts.getGeneratedNameForNode(importDeclaration.parent), ts.createIdentifier("default"))), 
                            /*location*/ node);
                        }
                        else if (ts.isImportSpecifier(importDeclaration)) {
                            return ts.setTextRange(ts.createPropertyAssignment(ts.getSynthesizedClone(name), ts.createPropertyAccess(ts.getGeneratedNameForNode(importDeclaration.parent.parent.parent), ts.getSynthesizedClone(importDeclaration.propertyName || importDeclaration.name))), 
                            /*location*/ node);
                        }
                    }
                }
                return node;
            }
            /**
             * Substitute the expression, if necessary.
             *
             * @param node The node to substitute.
             */
            function substituteExpression(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                        return substituteExpressionIdentifier(node);
                    case 209 /* BinaryExpression */:
                        return substituteBinaryExpression(node);
                    case 207 /* PrefixUnaryExpression */:
                    case 208 /* PostfixUnaryExpression */:
                        return substituteUnaryExpression(node);
                    case 219 /* MetaProperty */:
                        return substituteMetaProperty(node);
                }
                return node;
            }
            /**
             * Substitution for an Identifier expression that may contain an imported or exported symbol.
             *
             * @param node The node to substitute.
             */
            function substituteExpressionIdentifier(node) {
                if (ts.getEmitFlags(node) & 4096 /* HelperName */) {
                    var externalHelpersModuleName = ts.getExternalHelpersModuleName(currentSourceFile);
                    if (externalHelpersModuleName) {
                        return ts.createPropertyAccess(externalHelpersModuleName, node);
                    }
                    return node;
                }
                // When we see an identifier in an expression position that
                // points to an imported symbol, we should substitute a qualified
                // reference to the imported symbol if one is needed.
                //
                // - We do not substitute generated identifiers for any reason.
                // - We do not substitute identifiers tagged with the LocalName flag.
                if (!ts.isGeneratedIdentifier(node) && !ts.isLocalName(node)) {
                    var importDeclaration = resolver.getReferencedImportDeclaration(node);
                    if (importDeclaration) {
                        if (ts.isImportClause(importDeclaration)) {
                            return ts.setTextRange(ts.createPropertyAccess(ts.getGeneratedNameForNode(importDeclaration.parent), ts.createIdentifier("default")), 
                            /*location*/ node);
                        }
                        else if (ts.isImportSpecifier(importDeclaration)) {
                            return ts.setTextRange(ts.createPropertyAccess(ts.getGeneratedNameForNode(importDeclaration.parent.parent.parent), ts.getSynthesizedClone(importDeclaration.propertyName || importDeclaration.name)), 
                            /*location*/ node);
                        }
                    }
                }
                return node;
            }
            /**
             * Substitution for a BinaryExpression that may contain an imported or exported symbol.
             *
             * @param node The node to substitute.
             */
            function substituteBinaryExpression(node) {
                // When we see an assignment expression whose left-hand side is an exported symbol,
                // we should ensure all exports of that symbol are updated with the correct value.
                //
                // - We do not substitute generated identifiers for any reason.
                // - We do not substitute identifiers tagged with the LocalName flag.
                // - We do not substitute identifiers that were originally the name of an enum or
                //   namespace due to how they are transformed in TypeScript.
                // - We only substitute identifiers that are exported at the top level.
                if (ts.isAssignmentOperator(node.operatorToken.kind)
                    && ts.isIdentifier(node.left)
                    && !ts.isGeneratedIdentifier(node.left)
                    && !ts.isLocalName(node.left)
                    && !ts.isDeclarationNameOfEnumOrNamespace(node.left)) {
                    var exportedNames = getExports(node.left);
                    if (exportedNames) {
                        // For each additional export of the declaration, apply an export assignment.
                        var expression = node;
                        for (var _i = 0, exportedNames_4 = exportedNames; _i < exportedNames_4.length; _i++) {
                            var exportName = exportedNames_4[_i];
                            expression = createExportExpression(exportName, preventSubstitution(expression));
                        }
                        return expression;
                    }
                }
                return node;
            }
            /**
             * Substitution for a UnaryExpression that may contain an imported or exported symbol.
             *
             * @param node The node to substitute.
             */
            function substituteUnaryExpression(node) {
                // When we see a prefix or postfix increment expression whose operand is an exported
                // symbol, we should ensure all exports of that symbol are updated with the correct
                // value.
                //
                // - We do not substitute generated identifiers for any reason.
                // - We do not substitute identifiers tagged with the LocalName flag.
                // - We do not substitute identifiers that were originally the name of an enum or
                //   namespace due to how they are transformed in TypeScript.
                // - We only substitute identifiers that are exported at the top level.
                if ((node.operator === 45 /* PlusPlusToken */ || node.operator === 46 /* MinusMinusToken */)
                    && ts.isIdentifier(node.operand)
                    && !ts.isGeneratedIdentifier(node.operand)
                    && !ts.isLocalName(node.operand)
                    && !ts.isDeclarationNameOfEnumOrNamespace(node.operand)) {
                    var exportedNames = getExports(node.operand);
                    if (exportedNames) {
                        var expression = node.kind === 208 /* PostfixUnaryExpression */
                            ? ts.setTextRange(ts.createPrefix(node.operator, node.operand), node)
                            : node;
                        for (var _i = 0, exportedNames_5 = exportedNames; _i < exportedNames_5.length; _i++) {
                            var exportName = exportedNames_5[_i];
                            expression = createExportExpression(exportName, preventSubstitution(expression));
                        }
                        if (node.kind === 208 /* PostfixUnaryExpression */) {
                            expression = node.operator === 45 /* PlusPlusToken */
                                ? ts.createSubtract(preventSubstitution(expression), ts.createLiteral(1))
                                : ts.createAdd(preventSubstitution(expression), ts.createLiteral(1));
                        }
                        return expression;
                    }
                }
                return node;
            }
            function substituteMetaProperty(node) {
                if (ts.isImportMeta(node)) {
                    return ts.createPropertyAccess(contextObject, ts.createIdentifier("meta"));
                }
                return node;
            }
            /**
             * Gets the exports of a name.
             *
             * @param name The name.
             */
            function getExports(name) {
                var exportedNames;
                if (!ts.isGeneratedIdentifier(name)) {
                    var valueDeclaration = resolver.getReferencedImportDeclaration(name)
                        || resolver.getReferencedValueDeclaration(name);
                    if (valueDeclaration) {
                        var exportContainer = resolver.getReferencedExportContainer(name, /*prefixLocals*/ false);
                        if (exportContainer && exportContainer.kind === 290 /* SourceFile */) {
                            exportedNames = ts.append(exportedNames, ts.getDeclarationName(valueDeclaration));
                        }
                        exportedNames = ts.addRange(exportedNames, moduleInfo && moduleInfo.exportedBindings[ts.getOriginalNodeId(valueDeclaration)]);
                    }
                }
                return exportedNames;
            }
            /**
             * Prevent substitution of a node for this transformer.
             *
             * @param node The node which should not be substituted.
             */
            function preventSubstitution(node) {
                if (noSubstitution === undefined)
                    noSubstitution = [];
                noSubstitution[ts.getNodeId(node)] = true;
                return node;
            }
            /**
             * Determines whether a node should not be substituted.
             *
             * @param node The node to test.
             */
            function isSubstitutionPrevented(node) {
                return noSubstitution && node.id && noSubstitution[node.id];
            }
        }
        ts.transformSystemModule = transformSystemModule;
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        function transformECMAScriptModule(context) {
            var compilerOptions = context.getCompilerOptions();
            var previousOnEmitNode = context.onEmitNode;
            var previousOnSubstituteNode = context.onSubstituteNode;
            context.onEmitNode = onEmitNode;
            context.onSubstituteNode = onSubstituteNode;
            context.enableEmitNotification(290 /* SourceFile */);
            context.enableSubstitution(75 /* Identifier */);
            var helperNameSubstitutions;
            return ts.chainBundle(transformSourceFile);
            function transformSourceFile(node) {
                if (node.isDeclarationFile) {
                    return node;
                }
                if (ts.isExternalModule(node) || compilerOptions.isolatedModules) {
                    var externalHelpersImportDeclaration = ts.createExternalHelpersImportDeclarationIfNeeded(node, compilerOptions);
                    if (externalHelpersImportDeclaration) {
                        var statements = [];
                        var statementOffset = ts.addPrologue(statements, node.statements);
                        ts.append(statements, externalHelpersImportDeclaration);
                        ts.addRange(statements, ts.visitNodes(node.statements, visitor, ts.isStatement, statementOffset));
                        return ts.updateSourceFileNode(node, ts.setTextRange(ts.createNodeArray(statements), node.statements));
                    }
                    else {
                        return ts.visitEachChild(node, visitor, context);
                    }
                }
                return node;
            }
            function visitor(node) {
                switch (node.kind) {
                    case 253 /* ImportEqualsDeclaration */:
                        // Elide `import=` as it is not legal with --module ES6
                        return undefined;
                    case 259 /* ExportAssignment */:
                        return visitExportAssignment(node);
                    case 260 /* ExportDeclaration */:
                        var exportDecl = node;
                        return visitExportDeclaration(exportDecl);
                }
                return node;
            }
            function visitExportAssignment(node) {
                // Elide `export=` as it is not legal with --module ES6
                return node.isExportEquals ? undefined : node;
            }
            function visitExportDeclaration(node) {
                // `export * as ns` only needs to be transformed in ES2015
                if (compilerOptions.module !== undefined && compilerOptions.module > ts.ModuleKind.ES2015) {
                    return node;
                }
                // Either ill-formed or don't need to be tranformed.
                if (!node.exportClause || !ts.isNamespaceExport(node.exportClause) || !node.moduleSpecifier) {
                    return node;
                }
                var oldIdentifier = node.exportClause.name;
                var synthName = ts.getGeneratedNameForNode(oldIdentifier);
                var importDecl = ts.createImportDeclaration(
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, ts.createImportClause(/*name*/ undefined, ts.createNamespaceImport(synthName)), node.moduleSpecifier);
                ts.setOriginalNode(importDecl, node.exportClause);
                var exportDecl = ts.createExportDeclaration(
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, ts.createNamedExports([ts.createExportSpecifier(synthName, oldIdentifier)]));
                ts.setOriginalNode(exportDecl, node);
                return [importDecl, exportDecl];
            }
            //
            // Emit Notification
            //
            /**
             * Hook for node emit.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to emit.
             * @param emit A callback used to emit the node in the printer.
             */
            function onEmitNode(hint, node, emitCallback) {
                if (ts.isSourceFile(node)) {
                    if ((ts.isExternalModule(node) || compilerOptions.isolatedModules) && compilerOptions.importHelpers) {
                        helperNameSubstitutions = ts.createMap();
                    }
                    previousOnEmitNode(hint, node, emitCallback);
                    helperNameSubstitutions = undefined;
                }
                else {
                    previousOnEmitNode(hint, node, emitCallback);
                }
            }
            //
            // Substitutions
            //
            /**
             * Hooks node substitutions.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to substitute.
             */
            function onSubstituteNode(hint, node) {
                node = previousOnSubstituteNode(hint, node);
                if (helperNameSubstitutions && ts.isIdentifier(node) && ts.getEmitFlags(node) & 4096 /* HelperName */) {
                    return substituteHelperName(node);
                }
                return node;
            }
            function substituteHelperName(node) {
                var name = ts.idText(node);
                var substitution = helperNameSubstitutions.get(name);
                if (!substitution) {
                    helperNameSubstitutions.set(name, substitution = ts.createFileLevelUniqueName(name));
                }
                return substitution;
            }
        }
        ts.transformECMAScriptModule = transformECMAScriptModule;
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        function canProduceDiagnostics(node) {
            return ts.isVariableDeclaration(node) ||
                ts.isPropertyDeclaration(node) ||
                ts.isPropertySignature(node) ||
                ts.isBindingElement(node) ||
                ts.isSetAccessor(node) ||
                ts.isGetAccessor(node) ||
                ts.isConstructSignatureDeclaration(node) ||
                ts.isCallSignatureDeclaration(node) ||
                ts.isMethodDeclaration(node) ||
                ts.isMethodSignature(node) ||
                ts.isFunctionDeclaration(node) ||
                ts.isParameter(node) ||
                ts.isTypeParameterDeclaration(node) ||
                ts.isExpressionWithTypeArguments(node) ||
                ts.isImportEqualsDeclaration(node) ||
                ts.isTypeAliasDeclaration(node) ||
                ts.isConstructorDeclaration(node) ||
                ts.isIndexSignatureDeclaration(node) ||
                ts.isPropertyAccessExpression(node);
        }
        ts.canProduceDiagnostics = canProduceDiagnostics;
        function createGetSymbolAccessibilityDiagnosticForNodeName(node) {
            if (ts.isSetAccessor(node) || ts.isGetAccessor(node)) {
                return getAccessorNameVisibilityError;
            }
            else if (ts.isMethodSignature(node) || ts.isMethodDeclaration(node)) {
                return getMethodNameVisibilityError;
            }
            else {
                return createGetSymbolAccessibilityDiagnosticForNode(node);
            }
            function getAccessorNameVisibilityError(symbolAccessibilityResult) {
                var diagnosticMessage = getAccessorNameVisibilityDiagnosticMessage(symbolAccessibilityResult);
                return diagnosticMessage !== undefined ? {
                    diagnosticMessage: diagnosticMessage,
                    errorNode: node,
                    typeName: node.name
                } : undefined;
            }
            function getAccessorNameVisibilityDiagnosticMessage(symbolAccessibilityResult) {
                if (ts.hasModifier(node, 32 /* Static */)) {
                    return symbolAccessibilityResult.errorModuleName ?
                        symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                            ts.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                            ts.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2 :
                        ts.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_private_name_1;
                }
                else if (node.parent.kind === 245 /* ClassDeclaration */) {
                    return symbolAccessibilityResult.errorModuleName ?
                        symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                            ts.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                            ts.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2 :
                        ts.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_private_name_1;
                }
                else {
                    return symbolAccessibilityResult.errorModuleName ?
                        ts.Diagnostics.Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2 :
                        ts.Diagnostics.Property_0_of_exported_interface_has_or_is_using_private_name_1;
                }
            }
            function getMethodNameVisibilityError(symbolAccessibilityResult) {
                var diagnosticMessage = getMethodNameVisibilityDiagnosticMessage(symbolAccessibilityResult);
                return diagnosticMessage !== undefined ? {
                    diagnosticMessage: diagnosticMessage,
                    errorNode: node,
                    typeName: node.name
                } : undefined;
            }
            function getMethodNameVisibilityDiagnosticMessage(symbolAccessibilityResult) {
                if (ts.hasModifier(node, 32 /* Static */)) {
                    return symbolAccessibilityResult.errorModuleName ?
                        symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                            ts.Diagnostics.Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                            ts.Diagnostics.Public_static_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2 :
                        ts.Diagnostics.Public_static_method_0_of_exported_class_has_or_is_using_private_name_1;
                }
                else if (node.parent.kind === 245 /* ClassDeclaration */) {
                    return symbolAccessibilityResult.errorModuleName ?
                        symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                            ts.Diagnostics.Public_method_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                            ts.Diagnostics.Public_method_0_of_exported_class_has_or_is_using_name_1_from_private_module_2 :
                        ts.Diagnostics.Public_method_0_of_exported_class_has_or_is_using_private_name_1;
                }
                else {
                    return symbolAccessibilityResult.errorModuleName ?
                        ts.Diagnostics.Method_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2 :
                        ts.Diagnostics.Method_0_of_exported_interface_has_or_is_using_private_name_1;
                }
            }
        }
        ts.createGetSymbolAccessibilityDiagnosticForNodeName = createGetSymbolAccessibilityDiagnosticForNodeName;
        function createGetSymbolAccessibilityDiagnosticForNode(node) {
            if (ts.isVariableDeclaration(node) || ts.isPropertyDeclaration(node) || ts.isPropertySignature(node) || ts.isPropertyAccessExpression(node) || ts.isBindingElement(node) || ts.isConstructorDeclaration(node)) {
                return getVariableDeclarationTypeVisibilityError;
            }
            else if (ts.isSetAccessor(node) || ts.isGetAccessor(node)) {
                return getAccessorDeclarationTypeVisibilityError;
            }
            else if (ts.isConstructSignatureDeclaration(node) || ts.isCallSignatureDeclaration(node) || ts.isMethodDeclaration(node) || ts.isMethodSignature(node) || ts.isFunctionDeclaration(node) || ts.isIndexSignatureDeclaration(node)) {
                return getReturnTypeVisibilityError;
            }
            else if (ts.isParameter(node)) {
                if (ts.isParameterPropertyDeclaration(node, node.parent) && ts.hasModifier(node.parent, 8 /* Private */)) {
                    return getVariableDeclarationTypeVisibilityError;
                }
                return getParameterDeclarationTypeVisibilityError;
            }
            else if (ts.isTypeParameterDeclaration(node)) {
                return getTypeParameterConstraintVisibilityError;
            }
            else if (ts.isExpressionWithTypeArguments(node)) {
                return getHeritageClauseVisibilityError;
            }
            else if (ts.isImportEqualsDeclaration(node)) {
                return getImportEntityNameVisibilityError;
            }
            else if (ts.isTypeAliasDeclaration(node)) {
                return getTypeAliasDeclarationVisibilityError;
            }
            else {
                return ts.Debug.assertNever(node, "Attempted to set a declaration diagnostic context for unhandled node kind: " + ts.SyntaxKind[node.kind]);
            }
            function getVariableDeclarationTypeVisibilityDiagnosticMessage(symbolAccessibilityResult) {
                if (node.kind === 242 /* VariableDeclaration */ || node.kind === 191 /* BindingElement */) {
                    return symbolAccessibilityResult.errorModuleName ?
                        symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                            ts.Diagnostics.Exported_variable_0_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                            ts.Diagnostics.Exported_variable_0_has_or_is_using_name_1_from_private_module_2 :
                        ts.Diagnostics.Exported_variable_0_has_or_is_using_private_name_1;
                }
                // This check is to ensure we don't report error on constructor parameter property as that error would be reported during parameter emit
                // The only exception here is if the constructor was marked as private. we are not emitting the constructor parameters at all.
                else if (node.kind === 159 /* PropertyDeclaration */ || node.kind === 194 /* PropertyAccessExpression */ || node.kind === 158 /* PropertySignature */ ||
                    (node.kind === 156 /* Parameter */ && ts.hasModifier(node.parent, 8 /* Private */))) {
                    // TODO(jfreeman): Deal with computed properties in error reporting.
                    if (ts.hasModifier(node, 32 /* Static */)) {
                        return symbolAccessibilityResult.errorModuleName ?
                            symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                ts.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                                ts.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Public_static_property_0_of_exported_class_has_or_is_using_private_name_1;
                    }
                    else if (node.parent.kind === 245 /* ClassDeclaration */ || node.kind === 156 /* Parameter */) {
                        return symbolAccessibilityResult.errorModuleName ?
                            symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                ts.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                                ts.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Public_property_0_of_exported_class_has_or_is_using_private_name_1;
                    }
                    else {
                        // Interfaces cannot have types that cannot be named
                        return symbolAccessibilityResult.errorModuleName ?
                            ts.Diagnostics.Property_0_of_exported_interface_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Property_0_of_exported_interface_has_or_is_using_private_name_1;
                    }
                }
            }
            function getVariableDeclarationTypeVisibilityError(symbolAccessibilityResult) {
                var diagnosticMessage = getVariableDeclarationTypeVisibilityDiagnosticMessage(symbolAccessibilityResult);
                return diagnosticMessage !== undefined ? {
                    diagnosticMessage: diagnosticMessage,
                    errorNode: node,
                    typeName: node.name
                } : undefined;
            }
            function getAccessorDeclarationTypeVisibilityError(symbolAccessibilityResult) {
                var diagnosticMessage;
                if (node.kind === 164 /* SetAccessor */) {
                    // Getters can infer the return type from the returned expression, but setters cannot, so the
                    // "_from_external_module_1_but_cannot_be_named" case cannot occur.
                    if (ts.hasModifier(node, 32 /* Static */)) {
                        diagnosticMessage = symbolAccessibilityResult.errorModuleName ?
                            ts.Diagnostics.Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Parameter_type_of_public_static_setter_0_from_exported_class_has_or_is_using_private_name_1;
                    }
                    else {
                        diagnosticMessage = symbolAccessibilityResult.errorModuleName ?
                            ts.Diagnostics.Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Parameter_type_of_public_setter_0_from_exported_class_has_or_is_using_private_name_1;
                    }
                }
                else {
                    if (ts.hasModifier(node, 32 /* Static */)) {
                        diagnosticMessage = symbolAccessibilityResult.errorModuleName ?
                            symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                ts.Diagnostics.Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                                ts.Diagnostics.Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Return_type_of_public_static_getter_0_from_exported_class_has_or_is_using_private_name_1;
                    }
                    else {
                        diagnosticMessage = symbolAccessibilityResult.errorModuleName ?
                            symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                ts.Diagnostics.Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                                ts.Diagnostics.Return_type_of_public_getter_0_from_exported_class_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Return_type_of_public_getter_0_from_exported_class_has_or_is_using_private_name_1;
                    }
                }
                return {
                    diagnosticMessage: diagnosticMessage,
                    errorNode: node.name,
                    typeName: node.name
                };
            }
            function getReturnTypeVisibilityError(symbolAccessibilityResult) {
                var diagnosticMessage;
                switch (node.kind) {
                    case 166 /* ConstructSignature */:
                        // Interfaces cannot have return types that cannot be named
                        diagnosticMessage = symbolAccessibilityResult.errorModuleName ?
                            ts.Diagnostics.Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1 :
                            ts.Diagnostics.Return_type_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_0;
                        break;
                    case 165 /* CallSignature */:
                        // Interfaces cannot have return types that cannot be named
                        diagnosticMessage = symbolAccessibilityResult.errorModuleName ?
                            ts.Diagnostics.Return_type_of_call_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1 :
                            ts.Diagnostics.Return_type_of_call_signature_from_exported_interface_has_or_is_using_private_name_0;
                        break;
                    case 167 /* IndexSignature */:
                        // Interfaces cannot have return types that cannot be named
                        diagnosticMessage = symbolAccessibilityResult.errorModuleName ?
                            ts.Diagnostics.Return_type_of_index_signature_from_exported_interface_has_or_is_using_name_0_from_private_module_1 :
                            ts.Diagnostics.Return_type_of_index_signature_from_exported_interface_has_or_is_using_private_name_0;
                        break;
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                        if (ts.hasModifier(node, 32 /* Static */)) {
                            diagnosticMessage = symbolAccessibilityResult.errorModuleName ?
                                symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                    ts.Diagnostics.Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named :
                                    ts.Diagnostics.Return_type_of_public_static_method_from_exported_class_has_or_is_using_name_0_from_private_module_1 :
                                ts.Diagnostics.Return_type_of_public_static_method_from_exported_class_has_or_is_using_private_name_0;
                        }
                        else if (node.parent.kind === 245 /* ClassDeclaration */) {
                            diagnosticMessage = symbolAccessibilityResult.errorModuleName ?
                                symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                    ts.Diagnostics.Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named :
                                    ts.Diagnostics.Return_type_of_public_method_from_exported_class_has_or_is_using_name_0_from_private_module_1 :
                                ts.Diagnostics.Return_type_of_public_method_from_exported_class_has_or_is_using_private_name_0;
                        }
                        else {
                            // Interfaces cannot have return types that cannot be named
                            diagnosticMessage = symbolAccessibilityResult.errorModuleName ?
                                ts.Diagnostics.Return_type_of_method_from_exported_interface_has_or_is_using_name_0_from_private_module_1 :
                                ts.Diagnostics.Return_type_of_method_from_exported_interface_has_or_is_using_private_name_0;
                        }
                        break;
                    case 244 /* FunctionDeclaration */:
                        diagnosticMessage = symbolAccessibilityResult.errorModuleName ?
                            symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                ts.Diagnostics.Return_type_of_exported_function_has_or_is_using_name_0_from_external_module_1_but_cannot_be_named :
                                ts.Diagnostics.Return_type_of_exported_function_has_or_is_using_name_0_from_private_module_1 :
                            ts.Diagnostics.Return_type_of_exported_function_has_or_is_using_private_name_0;
                        break;
                    default:
                        return ts.Debug.fail("This is unknown kind for signature: " + node.kind);
                }
                return {
                    diagnosticMessage: diagnosticMessage,
                    errorNode: node.name || node
                };
            }
            function getParameterDeclarationTypeVisibilityError(symbolAccessibilityResult) {
                var diagnosticMessage = getParameterDeclarationTypeVisibilityDiagnosticMessage(symbolAccessibilityResult);
                return diagnosticMessage !== undefined ? {
                    diagnosticMessage: diagnosticMessage,
                    errorNode: node,
                    typeName: node.name
                } : undefined;
            }
            function getParameterDeclarationTypeVisibilityDiagnosticMessage(symbolAccessibilityResult) {
                switch (node.parent.kind) {
                    case 162 /* Constructor */:
                        return symbolAccessibilityResult.errorModuleName ?
                            symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                ts.Diagnostics.Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                                ts.Diagnostics.Parameter_0_of_constructor_from_exported_class_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Parameter_0_of_constructor_from_exported_class_has_or_is_using_private_name_1;
                    case 166 /* ConstructSignature */:
                    case 171 /* ConstructorType */:
                        // Interfaces cannot have parameter types that cannot be named
                        return symbolAccessibilityResult.errorModuleName ?
                            ts.Diagnostics.Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1;
                    case 165 /* CallSignature */:
                        // Interfaces cannot have parameter types that cannot be named
                        return symbolAccessibilityResult.errorModuleName ?
                            ts.Diagnostics.Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1;
                    case 167 /* IndexSignature */:
                        // Interfaces cannot have parameter types that cannot be named
                        return symbolAccessibilityResult.errorModuleName ?
                            ts.Diagnostics.Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Parameter_0_of_index_signature_from_exported_interface_has_or_is_using_private_name_1;
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                        if (ts.hasModifier(node.parent, 32 /* Static */)) {
                            return symbolAccessibilityResult.errorModuleName ?
                                symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                    ts.Diagnostics.Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                                    ts.Diagnostics.Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_name_1_from_private_module_2 :
                                ts.Diagnostics.Parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1;
                        }
                        else if (node.parent.parent.kind === 245 /* ClassDeclaration */) {
                            return symbolAccessibilityResult.errorModuleName ?
                                symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                    ts.Diagnostics.Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                                    ts.Diagnostics.Parameter_0_of_public_method_from_exported_class_has_or_is_using_name_1_from_private_module_2 :
                                ts.Diagnostics.Parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1;
                        }
                        else {
                            // Interfaces cannot have parameter types that cannot be named
                            return symbolAccessibilityResult.errorModuleName ?
                                ts.Diagnostics.Parameter_0_of_method_from_exported_interface_has_or_is_using_name_1_from_private_module_2 :
                                ts.Diagnostics.Parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1;
                        }
                    case 244 /* FunctionDeclaration */:
                    case 170 /* FunctionType */:
                        return symbolAccessibilityResult.errorModuleName ?
                            symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                ts.Diagnostics.Parameter_0_of_exported_function_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                                ts.Diagnostics.Parameter_0_of_exported_function_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Parameter_0_of_exported_function_has_or_is_using_private_name_1;
                    case 164 /* SetAccessor */:
                    case 163 /* GetAccessor */:
                        return symbolAccessibilityResult.errorModuleName ?
                            symbolAccessibilityResult.accessibility === 2 /* CannotBeNamed */ ?
                                ts.Diagnostics.Parameter_0_of_accessor_has_or_is_using_name_1_from_external_module_2_but_cannot_be_named :
                                ts.Diagnostics.Parameter_0_of_accessor_has_or_is_using_name_1_from_private_module_2 :
                            ts.Diagnostics.Parameter_0_of_accessor_has_or_is_using_private_name_1;
                    default:
                        return ts.Debug.fail("Unknown parent for parameter: " + ts.SyntaxKind[node.parent.kind]);
                }
            }
            function getTypeParameterConstraintVisibilityError() {
                // Type parameter constraints are named by user so we should always be able to name it
                var diagnosticMessage;
                switch (node.parent.kind) {
                    case 245 /* ClassDeclaration */:
                        diagnosticMessage = ts.Diagnostics.Type_parameter_0_of_exported_class_has_or_is_using_private_name_1;
                        break;
                    case 246 /* InterfaceDeclaration */:
                        diagnosticMessage = ts.Diagnostics.Type_parameter_0_of_exported_interface_has_or_is_using_private_name_1;
                        break;
                    case 186 /* MappedType */:
                        diagnosticMessage = ts.Diagnostics.Type_parameter_0_of_exported_mapped_object_type_is_using_private_name_1;
                        break;
                    case 171 /* ConstructorType */:
                    case 166 /* ConstructSignature */:
                        diagnosticMessage = ts.Diagnostics.Type_parameter_0_of_constructor_signature_from_exported_interface_has_or_is_using_private_name_1;
                        break;
                    case 165 /* CallSignature */:
                        diagnosticMessage = ts.Diagnostics.Type_parameter_0_of_call_signature_from_exported_interface_has_or_is_using_private_name_1;
                        break;
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                        if (ts.hasModifier(node.parent, 32 /* Static */)) {
                            diagnosticMessage = ts.Diagnostics.Type_parameter_0_of_public_static_method_from_exported_class_has_or_is_using_private_name_1;
                        }
                        else if (node.parent.parent.kind === 245 /* ClassDeclaration */) {
                            diagnosticMessage = ts.Diagnostics.Type_parameter_0_of_public_method_from_exported_class_has_or_is_using_private_name_1;
                        }
                        else {
                            diagnosticMessage = ts.Diagnostics.Type_parameter_0_of_method_from_exported_interface_has_or_is_using_private_name_1;
                        }
                        break;
                    case 170 /* FunctionType */:
                    case 244 /* FunctionDeclaration */:
                        diagnosticMessage = ts.Diagnostics.Type_parameter_0_of_exported_function_has_or_is_using_private_name_1;
                        break;
                    case 247 /* TypeAliasDeclaration */:
                        diagnosticMessage = ts.Diagnostics.Type_parameter_0_of_exported_type_alias_has_or_is_using_private_name_1;
                        break;
                    default:
                        return ts.Debug.fail("This is unknown parent for type parameter: " + node.parent.kind);
                }
                return {
                    diagnosticMessage: diagnosticMessage,
                    errorNode: node,
                    typeName: node.name
                };
            }
            function getHeritageClauseVisibilityError() {
                var diagnosticMessage;
                // Heritage clause is written by user so it can always be named
                if (node.parent.parent.kind === 245 /* ClassDeclaration */) {
                    // Class or Interface implemented/extended is inaccessible
                    diagnosticMessage = ts.isHeritageClause(node.parent) && node.parent.token === 113 /* ImplementsKeyword */ ?
                        ts.Diagnostics.Implements_clause_of_exported_class_0_has_or_is_using_private_name_1 :
                        ts.Diagnostics.extends_clause_of_exported_class_0_has_or_is_using_private_name_1;
                }
                else {
                    // interface is inaccessible
                    diagnosticMessage = ts.Diagnostics.extends_clause_of_exported_interface_0_has_or_is_using_private_name_1;
                }
                return {
                    diagnosticMessage: diagnosticMessage,
                    errorNode: node,
                    typeName: ts.getNameOfDeclaration(node.parent.parent)
                };
            }
            function getImportEntityNameVisibilityError() {
                return {
                    diagnosticMessage: ts.Diagnostics.Import_declaration_0_is_using_private_name_1,
                    errorNode: node,
                    typeName: node.name
                };
            }
            function getTypeAliasDeclarationVisibilityError() {
                return {
                    diagnosticMessage: ts.Diagnostics.Exported_type_alias_0_has_or_is_using_private_name_1,
                    errorNode: node.type,
                    typeName: node.name
                };
            }
        }
        ts.createGetSymbolAccessibilityDiagnosticForNode = createGetSymbolAccessibilityDiagnosticForNode;
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        function getDeclarationDiagnostics(host, resolver, file) {
            if (file && ts.isJsonSourceFile(file)) {
                return []; // No declaration diagnostics for json for now
            }
            var compilerOptions = host.getCompilerOptions();
            var result = ts.transformNodes(resolver, host, compilerOptions, file ? [file] : ts.filter(host.getSourceFiles(), ts.isSourceFileNotJson), [transformDeclarations], /*allowDtsFiles*/ false);
            return result.diagnostics;
        }
        ts.getDeclarationDiagnostics = getDeclarationDiagnostics;
        function hasInternalAnnotation(range, currentSourceFile) {
            var comment = currentSourceFile.text.substring(range.pos, range.end);
            return ts.stringContains(comment, "@internal");
        }
        function isInternalDeclaration(node, currentSourceFile) {
            var parseTreeNode = ts.getParseTreeNode(node);
            if (parseTreeNode && parseTreeNode.kind === 156 /* Parameter */) {
                var paramIdx = parseTreeNode.parent.parameters.indexOf(parseTreeNode);
                var previousSibling = paramIdx > 0 ? parseTreeNode.parent.parameters[paramIdx - 1] : undefined;
                var text = currentSourceFile.text;
                var commentRanges = previousSibling
                    ? ts.concatenate(
                    // to handle
                    // ... parameters, /* @internal */
                    // public param: string
                    ts.getTrailingCommentRanges(text, ts.skipTrivia(text, previousSibling.end + 1, /* stopAfterLineBreak */ false, /* stopAtComments */ true)), ts.getLeadingCommentRanges(text, node.pos))
                    : ts.getTrailingCommentRanges(text, ts.skipTrivia(text, node.pos, /* stopAfterLineBreak */ false, /* stopAtComments */ true));
                return commentRanges && commentRanges.length && hasInternalAnnotation(ts.last(commentRanges), currentSourceFile);
            }
            var leadingCommentRanges = parseTreeNode && ts.getLeadingCommentRangesOfNode(parseTreeNode, currentSourceFile);
            return !!ts.forEach(leadingCommentRanges, function (range) {
                return hasInternalAnnotation(range, currentSourceFile);
            });
        }
        ts.isInternalDeclaration = isInternalDeclaration;
        var declarationEmitNodeBuilderFlags = 1024 /* MultilineObjectLiterals */ |
            2048 /* WriteClassExpressionAsTypeLiteral */ |
            4096 /* UseTypeOfFunction */ |
            8 /* UseStructuralFallback */ |
            524288 /* AllowEmptyTuple */ |
            4 /* GenerateNamesForShadowedTypeParams */ |
            1 /* NoTruncation */;
        /**
         * Transforms a ts file into a .d.ts file
         * This process requires type information, which is retrieved through the emit resolver. Because of this,
         * in many places this transformer assumes it will be operating on parse tree nodes directly.
         * This means that _no transforms should be allowed to occur before this one_.
         */
        function transformDeclarations(context) {
            var throwDiagnostic = function () { return ts.Debug.fail("Diagnostic emitted without context"); };
            var getSymbolAccessibilityDiagnostic = throwDiagnostic;
            var needsDeclare = true;
            var isBundledEmit = false;
            var resultHasExternalModuleIndicator = false;
            var needsScopeFixMarker = false;
            var resultHasScopeMarker = false;
            var enclosingDeclaration;
            var necessaryTypeReferences;
            var lateMarkedStatements;
            var lateStatementReplacementMap;
            var suppressNewDiagnosticContexts;
            var exportedModulesFromDeclarationEmit;
            var host = context.getEmitHost();
            var symbolTracker = {
                trackSymbol: trackSymbol,
                reportInaccessibleThisError: reportInaccessibleThisError,
                reportInaccessibleUniqueSymbolError: reportInaccessibleUniqueSymbolError,
                reportPrivateInBaseOfClassExpression: reportPrivateInBaseOfClassExpression,
                reportLikelyUnsafeImportRequiredError: reportLikelyUnsafeImportRequiredError,
                moduleResolverHost: host,
                trackReferencedAmbientModule: trackReferencedAmbientModule,
                trackExternalModuleSymbolOfImportTypeNode: trackExternalModuleSymbolOfImportTypeNode
            };
            var errorNameNode;
            var currentSourceFile;
            var refs;
            var libs;
            var emittedImports; // must be declared in container so it can be `undefined` while transformer's first pass
            var resolver = context.getEmitResolver();
            var options = context.getCompilerOptions();
            var noResolve = options.noResolve, stripInternal = options.stripInternal;
            return transformRoot;
            function recordTypeReferenceDirectivesIfNecessary(typeReferenceDirectives) {
                if (!typeReferenceDirectives) {
                    return;
                }
                necessaryTypeReferences = necessaryTypeReferences || ts.createMap();
                for (var _i = 0, typeReferenceDirectives_2 = typeReferenceDirectives; _i < typeReferenceDirectives_2.length; _i++) {
                    var ref = typeReferenceDirectives_2[_i];
                    necessaryTypeReferences.set(ref, true);
                }
            }
            function trackReferencedAmbientModule(node, symbol) {
                // If it is visible via `// <reference types="..."/>`, then we should just use that
                var directives = resolver.getTypeReferenceDirectivesForSymbol(symbol, 67108863 /* All */);
                if (ts.length(directives)) {
                    return recordTypeReferenceDirectivesIfNecessary(directives);
                }
                // Otherwise we should emit a path-based reference
                var container = ts.getSourceFileOfNode(node);
                refs.set("" + ts.getOriginalNodeId(container), container);
            }
            function handleSymbolAccessibilityError(symbolAccessibilityResult) {
                if (symbolAccessibilityResult.accessibility === 0 /* Accessible */) {
                    // Add aliases back onto the possible imports list if they're not there so we can try them again with updated visibility info
                    if (symbolAccessibilityResult && symbolAccessibilityResult.aliasesToMakeVisible) {
                        if (!lateMarkedStatements) {
                            lateMarkedStatements = symbolAccessibilityResult.aliasesToMakeVisible;
                        }
                        else {
                            for (var _i = 0, _a = symbolAccessibilityResult.aliasesToMakeVisible; _i < _a.length; _i++) {
                                var ref = _a[_i];
                                ts.pushIfUnique(lateMarkedStatements, ref);
                            }
                        }
                    }
                    // TODO: Do all these accessibility checks inside/after the first pass in the checker when declarations are enabled, if possible
                }
                else {
                    // Report error
                    var errorInfo = getSymbolAccessibilityDiagnostic(symbolAccessibilityResult);
                    if (errorInfo) {
                        if (errorInfo.typeName) {
                            context.addDiagnostic(ts.createDiagnosticForNode(symbolAccessibilityResult.errorNode || errorInfo.errorNode, errorInfo.diagnosticMessage, ts.getTextOfNode(errorInfo.typeName), symbolAccessibilityResult.errorSymbolName, symbolAccessibilityResult.errorModuleName));
                        }
                        else {
                            context.addDiagnostic(ts.createDiagnosticForNode(symbolAccessibilityResult.errorNode || errorInfo.errorNode, errorInfo.diagnosticMessage, symbolAccessibilityResult.errorSymbolName, symbolAccessibilityResult.errorModuleName));
                        }
                    }
                }
            }
            function trackExternalModuleSymbolOfImportTypeNode(symbol) {
                if (!isBundledEmit) {
                    (exportedModulesFromDeclarationEmit || (exportedModulesFromDeclarationEmit = [])).push(symbol);
                }
            }
            function trackSymbol(symbol, enclosingDeclaration, meaning) {
                if (symbol.flags & 262144 /* TypeParameter */)
                    return;
                handleSymbolAccessibilityError(resolver.isSymbolAccessible(symbol, enclosingDeclaration, meaning, /*shouldComputeAliasesToMakeVisible*/ true));
                recordTypeReferenceDirectivesIfNecessary(resolver.getTypeReferenceDirectivesForSymbol(symbol, meaning));
            }
            function reportPrivateInBaseOfClassExpression(propertyName) {
                if (errorNameNode) {
                    context.addDiagnostic(ts.createDiagnosticForNode(errorNameNode, ts.Diagnostics.Property_0_of_exported_class_expression_may_not_be_private_or_protected, propertyName));
                }
            }
            function reportInaccessibleUniqueSymbolError() {
                if (errorNameNode) {
                    context.addDiagnostic(ts.createDiagnosticForNode(errorNameNode, ts.Diagnostics.The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary, ts.declarationNameToString(errorNameNode), "unique symbol"));
                }
            }
            function reportInaccessibleThisError() {
                if (errorNameNode) {
                    context.addDiagnostic(ts.createDiagnosticForNode(errorNameNode, ts.Diagnostics.The_inferred_type_of_0_references_an_inaccessible_1_type_A_type_annotation_is_necessary, ts.declarationNameToString(errorNameNode), "this"));
                }
            }
            function reportLikelyUnsafeImportRequiredError(specifier) {
                if (errorNameNode) {
                    context.addDiagnostic(ts.createDiagnosticForNode(errorNameNode, ts.Diagnostics.The_inferred_type_of_0_cannot_be_named_without_a_reference_to_1_This_is_likely_not_portable_A_type_annotation_is_necessary, ts.declarationNameToString(errorNameNode), specifier));
                }
            }
            function transformDeclarationsForJS(sourceFile, bundled) {
                var oldDiag = getSymbolAccessibilityDiagnostic;
                getSymbolAccessibilityDiagnostic = function (s) {
                    return ({
                        diagnosticMessage: s.errorModuleName
                            ? ts.Diagnostics.Declaration_emit_for_this_file_requires_using_private_name_0_from_module_1_An_explicit_type_annotation_may_unblock_declaration_emit
                            : ts.Diagnostics.Declaration_emit_for_this_file_requires_using_private_name_0_An_explicit_type_annotation_may_unblock_declaration_emit,
                        errorNode: s.errorNode || sourceFile
                    });
                };
                var result = resolver.getDeclarationStatementsForSourceFile(sourceFile, declarationEmitNodeBuilderFlags, symbolTracker, bundled);
                getSymbolAccessibilityDiagnostic = oldDiag;
                return result;
            }
            function transformRoot(node) {
                if (node.kind === 290 /* SourceFile */ && node.isDeclarationFile) {
                    return node;
                }
                if (node.kind === 291 /* Bundle */) {
                    isBundledEmit = true;
                    refs = ts.createMap();
                    libs = ts.createMap();
                    var hasNoDefaultLib_1 = false;
                    var bundle = ts.createBundle(ts.map(node.sourceFiles, function (sourceFile) {
                        if (sourceFile.isDeclarationFile)
                            return undefined; // Omit declaration files from bundle results, too // TODO: GH#18217
                        hasNoDefaultLib_1 = hasNoDefaultLib_1 || sourceFile.hasNoDefaultLib;
                        currentSourceFile = sourceFile;
                        enclosingDeclaration = sourceFile;
                        lateMarkedStatements = undefined;
                        suppressNewDiagnosticContexts = false;
                        lateStatementReplacementMap = ts.createMap();
                        getSymbolAccessibilityDiagnostic = throwDiagnostic;
                        needsScopeFixMarker = false;
                        resultHasScopeMarker = false;
                        collectReferences(sourceFile, refs);
                        collectLibs(sourceFile, libs);
                        if (ts.isExternalOrCommonJsModule(sourceFile) || ts.isJsonSourceFile(sourceFile)) {
                            resultHasExternalModuleIndicator = false; // unused in external module bundle emit (all external modules are within module blocks, therefore are known to be modules)
                            needsDeclare = false;
                            var statements = ts.isSourceFileJS(sourceFile) ? ts.createNodeArray(transformDeclarationsForJS(sourceFile, /*bundled*/ true)) : ts.visitNodes(sourceFile.statements, visitDeclarationStatements);
                            var newFile = ts.updateSourceFileNode(sourceFile, [ts.createModuleDeclaration([], [ts.createModifier(130 /* DeclareKeyword */)], ts.createLiteral(ts.getResolvedExternalModuleName(context.getEmitHost(), sourceFile)), ts.createModuleBlock(ts.setTextRange(ts.createNodeArray(transformAndReplaceLatePaintedStatements(statements)), sourceFile.statements)))], /*isDeclarationFile*/ true, /*referencedFiles*/ [], /*typeReferences*/ [], /*hasNoDefaultLib*/ false, /*libReferences*/ []);
                            return newFile;
                        }
                        needsDeclare = true;
                        var updated = ts.isSourceFileJS(sourceFile) ? ts.createNodeArray(transformDeclarationsForJS(sourceFile)) : ts.visitNodes(sourceFile.statements, visitDeclarationStatements);
                        return ts.updateSourceFileNode(sourceFile, transformAndReplaceLatePaintedStatements(updated), /*isDeclarationFile*/ true, /*referencedFiles*/ [], /*typeReferences*/ [], /*hasNoDefaultLib*/ false, /*libReferences*/ []);
                    }), ts.mapDefined(node.prepends, function (prepend) {
                        if (prepend.kind === 293 /* InputFiles */) {
                            var sourceFile = ts.createUnparsedSourceFile(prepend, "dts", stripInternal);
                            hasNoDefaultLib_1 = hasNoDefaultLib_1 || !!sourceFile.hasNoDefaultLib;
                            collectReferences(sourceFile, refs);
                            recordTypeReferenceDirectivesIfNecessary(sourceFile.typeReferenceDirectives);
                            collectLibs(sourceFile, libs);
                            return sourceFile;
                        }
                        return prepend;
                    }));
                    bundle.syntheticFileReferences = [];
                    bundle.syntheticTypeReferences = getFileReferencesForUsedTypeReferences();
                    bundle.syntheticLibReferences = getLibReferences();
                    bundle.hasNoDefaultLib = hasNoDefaultLib_1;
                    var outputFilePath_1 = ts.getDirectoryPath(ts.normalizeSlashes(ts.getOutputPathsFor(node, host, /*forceDtsPaths*/ true).declarationFilePath));
                    var referenceVisitor_1 = mapReferencesIntoArray(bundle.syntheticFileReferences, outputFilePath_1);
                    refs.forEach(referenceVisitor_1);
                    return bundle;
                }
                // Single source file
                needsDeclare = true;
                needsScopeFixMarker = false;
                resultHasScopeMarker = false;
                enclosingDeclaration = node;
                currentSourceFile = node;
                getSymbolAccessibilityDiagnostic = throwDiagnostic;
                isBundledEmit = false;
                resultHasExternalModuleIndicator = false;
                suppressNewDiagnosticContexts = false;
                lateMarkedStatements = undefined;
                lateStatementReplacementMap = ts.createMap();
                necessaryTypeReferences = undefined;
                refs = collectReferences(currentSourceFile, ts.createMap());
                libs = collectLibs(currentSourceFile, ts.createMap());
                var references = [];
                var outputFilePath = ts.getDirectoryPath(ts.normalizeSlashes(ts.getOutputPathsFor(node, host, /*forceDtsPaths*/ true).declarationFilePath));
                var referenceVisitor = mapReferencesIntoArray(references, outputFilePath);
                var combinedStatements;
                if (ts.isSourceFileJS(currentSourceFile)) {
                    combinedStatements = ts.createNodeArray(transformDeclarationsForJS(node));
                    refs.forEach(referenceVisitor);
                    emittedImports = ts.filter(combinedStatements, ts.isAnyImportSyntax);
                }
                else {
                    var statements = ts.visitNodes(node.statements, visitDeclarationStatements);
                    combinedStatements = ts.setTextRange(ts.createNodeArray(transformAndReplaceLatePaintedStatements(statements)), node.statements);
                    refs.forEach(referenceVisitor);
                    emittedImports = ts.filter(combinedStatements, ts.isAnyImportSyntax);
                    if (ts.isExternalModule(node) && (!resultHasExternalModuleIndicator || (needsScopeFixMarker && !resultHasScopeMarker))) {
                        combinedStatements = ts.setTextRange(ts.createNodeArray(__spreadArrays(combinedStatements, [ts.createEmptyExports()])), combinedStatements);
                    }
                }
                var updated = ts.updateSourceFileNode(node, combinedStatements, /*isDeclarationFile*/ true, references, getFileReferencesForUsedTypeReferences(), node.hasNoDefaultLib, getLibReferences());
                updated.exportedModulesFromDeclarationEmit = exportedModulesFromDeclarationEmit;
                return updated;
                function getLibReferences() {
                    return ts.map(ts.arrayFrom(libs.keys()), function (lib) { return ({ fileName: lib, pos: -1, end: -1 }); });
                }
                function getFileReferencesForUsedTypeReferences() {
                    return necessaryTypeReferences ? ts.mapDefined(ts.arrayFrom(necessaryTypeReferences.keys()), getFileReferenceForTypeName) : [];
                }
                function getFileReferenceForTypeName(typeName) {
                    // Elide type references for which we have imports
                    if (emittedImports) {
                        for (var _i = 0, emittedImports_1 = emittedImports; _i < emittedImports_1.length; _i++) {
                            var importStatement = emittedImports_1[_i];
                            if (ts.isImportEqualsDeclaration(importStatement) && ts.isExternalModuleReference(importStatement.moduleReference)) {
                                var expr = importStatement.moduleReference.expression;
                                if (ts.isStringLiteralLike(expr) && expr.text === typeName) {
                                    return undefined;
                                }
                            }
                            else if (ts.isImportDeclaration(importStatement) && ts.isStringLiteral(importStatement.moduleSpecifier) && importStatement.moduleSpecifier.text === typeName) {
                                return undefined;
                            }
                        }
                    }
                    return { fileName: typeName, pos: -1, end: -1 };
                }
                function mapReferencesIntoArray(references, outputFilePath) {
                    return function (file) {
                        var declFileName;
                        if (file.isDeclarationFile) { // Neither decl files or js should have their refs changed
                            declFileName = file.fileName;
                        }
                        else {
                            if (isBundledEmit && ts.contains(node.sourceFiles, file))
                                return; // Omit references to files which are being merged
                            var paths = ts.getOutputPathsFor(file, host, /*forceDtsPaths*/ true);
                            declFileName = paths.declarationFilePath || paths.jsFilePath || file.fileName;
                        }
                        if (declFileName) {
                            var specifier = ts.moduleSpecifiers.getModuleSpecifier(__assign(__assign({}, options), { baseUrl: options.baseUrl && ts.toPath(options.baseUrl, host.getCurrentDirectory(), host.getCanonicalFileName) }), currentSourceFile, ts.toPath(outputFilePath, host.getCurrentDirectory(), host.getCanonicalFileName), ts.toPath(declFileName, host.getCurrentDirectory(), host.getCanonicalFileName), host, host.getSourceFiles(), 
                            /*preferences*/ undefined, host.redirectTargetsMap);
                            if (!ts.pathIsRelative(specifier)) {
                                // If some compiler option/symlink/whatever allows access to the file containing the ambient module declaration
                                // via a non-relative name, emit a type reference directive to that non-relative name, rather than
                                // a relative path to the declaration file
                                recordTypeReferenceDirectivesIfNecessary([specifier]);
                                return;
                            }
                            var fileName = ts.getRelativePathToDirectoryOrUrl(outputFilePath, declFileName, host.getCurrentDirectory(), host.getCanonicalFileName, 
                            /*isAbsolutePathAnUrl*/ false);
                            if (ts.startsWith(fileName, "./") && ts.hasExtension(fileName)) {
                                fileName = fileName.substring(2);
                            }
                            // omit references to files from node_modules (npm may disambiguate module
                            // references when installing this package, making the path is unreliable).
                            if (ts.startsWith(fileName, "node_modules/") || fileName.indexOf("/node_modules/") !== -1) {
                                return;
                            }
                            references.push({ pos: -1, end: -1, fileName: fileName });
                        }
                    };
                }
            }
            function collectReferences(sourceFile, ret) {
                if (noResolve || (!ts.isUnparsedSource(sourceFile) && ts.isSourceFileJS(sourceFile)))
                    return ret;
                ts.forEach(sourceFile.referencedFiles, function (f) {
                    var elem = host.getSourceFileFromReference(sourceFile, f);
                    if (elem) {
                        ret.set("" + ts.getOriginalNodeId(elem), elem);
                    }
                });
                return ret;
            }
            function collectLibs(sourceFile, ret) {
                ts.forEach(sourceFile.libReferenceDirectives, function (ref) {
                    var lib = host.getLibFileFromReference(ref);
                    if (lib) {
                        ret.set(ts.toFileNameLowerCase(ref.fileName), true);
                    }
                });
                return ret;
            }
            function filterBindingPatternInitializers(name) {
                if (name.kind === 75 /* Identifier */) {
                    return name;
                }
                else {
                    if (name.kind === 190 /* ArrayBindingPattern */) {
                        return ts.updateArrayBindingPattern(name, ts.visitNodes(name.elements, visitBindingElement));
                    }
                    else {
                        return ts.updateObjectBindingPattern(name, ts.visitNodes(name.elements, visitBindingElement));
                    }
                }
                function visitBindingElement(elem) {
                    if (elem.kind === 215 /* OmittedExpression */) {
                        return elem;
                    }
                    return ts.updateBindingElement(elem, elem.dotDotDotToken, elem.propertyName, filterBindingPatternInitializers(elem.name), shouldPrintWithInitializer(elem) ? elem.initializer : undefined);
                }
            }
            function ensureParameter(p, modifierMask, type) {
                var oldDiag;
                if (!suppressNewDiagnosticContexts) {
                    oldDiag = getSymbolAccessibilityDiagnostic;
                    getSymbolAccessibilityDiagnostic = ts.createGetSymbolAccessibilityDiagnosticForNode(p);
                }
                var newParam = ts.updateParameter(p, 
                /*decorators*/ undefined, maskModifiers(p, modifierMask), p.dotDotDotToken, filterBindingPatternInitializers(p.name), resolver.isOptionalParameter(p) ? (p.questionToken || ts.createToken(57 /* QuestionToken */)) : undefined, ensureType(p, type || p.type, /*ignorePrivate*/ true), // Ignore private param props, since this type is going straight back into a param
                ensureNoInitializer(p));
                if (!suppressNewDiagnosticContexts) {
                    getSymbolAccessibilityDiagnostic = oldDiag;
                }
                return newParam;
            }
            function shouldPrintWithInitializer(node) {
                return canHaveLiteralInitializer(node) && resolver.isLiteralConstDeclaration(ts.getParseTreeNode(node)); // TODO: Make safe
            }
            function ensureNoInitializer(node) {
                if (shouldPrintWithInitializer(node)) {
                    return resolver.createLiteralConstValue(ts.getParseTreeNode(node), symbolTracker); // TODO: Make safe
                }
                return undefined;
            }
            function ensureType(node, type, ignorePrivate) {
                if (!ignorePrivate && ts.hasModifier(node, 8 /* Private */)) {
                    // Private nodes emit no types (except private parameter properties, whose parameter types are actually visible)
                    return;
                }
                if (shouldPrintWithInitializer(node)) {
                    // Literal const declarations will have an initializer ensured rather than a type
                    return;
                }
                var shouldUseResolverType = node.kind === 156 /* Parameter */ &&
                    (resolver.isRequiredInitializedParameter(node) ||
                        resolver.isOptionalUninitializedParameterProperty(node));
                if (type && !shouldUseResolverType) {
                    return ts.visitNode(type, visitDeclarationSubtree);
                }
                if (!ts.getParseTreeNode(node)) {
                    return type ? ts.visitNode(type, visitDeclarationSubtree) : ts.createKeywordTypeNode(125 /* AnyKeyword */);
                }
                if (node.kind === 164 /* SetAccessor */) {
                    // Set accessors with no associated type node (from it's param or get accessor return) are `any` since they are never contextually typed right now
                    // (The inferred type here will be void, but the old declaration emitter printed `any`, so this replicates that)
                    return ts.createKeywordTypeNode(125 /* AnyKeyword */);
                }
                errorNameNode = node.name;
                var oldDiag;
                if (!suppressNewDiagnosticContexts) {
                    oldDiag = getSymbolAccessibilityDiagnostic;
                    getSymbolAccessibilityDiagnostic = ts.createGetSymbolAccessibilityDiagnosticForNode(node);
                }
                if (node.kind === 242 /* VariableDeclaration */ || node.kind === 191 /* BindingElement */) {
                    return cleanup(resolver.createTypeOfDeclaration(node, enclosingDeclaration, declarationEmitNodeBuilderFlags, symbolTracker));
                }
                if (node.kind === 156 /* Parameter */
                    || node.kind === 159 /* PropertyDeclaration */
                    || node.kind === 158 /* PropertySignature */) {
                    if (!node.initializer)
                        return cleanup(resolver.createTypeOfDeclaration(node, enclosingDeclaration, declarationEmitNodeBuilderFlags, symbolTracker, shouldUseResolverType));
                    return cleanup(resolver.createTypeOfDeclaration(node, enclosingDeclaration, declarationEmitNodeBuilderFlags, symbolTracker, shouldUseResolverType) || resolver.createTypeOfExpression(node.initializer, enclosingDeclaration, declarationEmitNodeBuilderFlags, symbolTracker));
                }
                return cleanup(resolver.createReturnTypeOfSignatureDeclaration(node, enclosingDeclaration, declarationEmitNodeBuilderFlags, symbolTracker));
                function cleanup(returnValue) {
                    errorNameNode = undefined;
                    if (!suppressNewDiagnosticContexts) {
                        getSymbolAccessibilityDiagnostic = oldDiag;
                    }
                    return returnValue || ts.createKeywordTypeNode(125 /* AnyKeyword */);
                }
            }
            function isDeclarationAndNotVisible(node) {
                node = ts.getParseTreeNode(node);
                switch (node.kind) {
                    case 244 /* FunctionDeclaration */:
                    case 249 /* ModuleDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                    case 245 /* ClassDeclaration */:
                    case 247 /* TypeAliasDeclaration */:
                    case 248 /* EnumDeclaration */:
                        return !resolver.isDeclarationVisible(node);
                    // The following should be doing their own visibility checks based on filtering their members
                    case 242 /* VariableDeclaration */:
                        return !getBindingNameVisible(node);
                    case 253 /* ImportEqualsDeclaration */:
                    case 254 /* ImportDeclaration */:
                    case 260 /* ExportDeclaration */:
                    case 259 /* ExportAssignment */:
                        return false;
                }
                return false;
            }
            function getBindingNameVisible(elem) {
                if (ts.isOmittedExpression(elem)) {
                    return false;
                }
                if (ts.isBindingPattern(elem.name)) {
                    // If any child binding pattern element has been marked visible (usually by collect linked aliases), then this is visible
                    return ts.some(elem.name.elements, getBindingNameVisible);
                }
                else {
                    return resolver.isDeclarationVisible(elem);
                }
            }
            function updateParamsList(node, params, modifierMask) {
                if (ts.hasModifier(node, 8 /* Private */)) {
                    return undefined; // TODO: GH#18217
                }
                var newParams = ts.map(params, function (p) { return ensureParameter(p, modifierMask); });
                if (!newParams) {
                    return undefined; // TODO: GH#18217
                }
                return ts.createNodeArray(newParams, params.hasTrailingComma);
            }
            function updateAccessorParamsList(input, isPrivate) {
                var newParams;
                if (!isPrivate) {
                    var thisParameter = ts.getThisParameter(input);
                    if (thisParameter) {
                        newParams = [ensureParameter(thisParameter)];
                    }
                }
                if (ts.isSetAccessorDeclaration(input)) {
                    var newValueParameter = void 0;
                    if (!isPrivate) {
                        var valueParameter = ts.getSetAccessorValueParameter(input);
                        if (valueParameter) {
                            var accessorType = getTypeAnnotationFromAllAccessorDeclarations(input, resolver.getAllAccessorDeclarations(input));
                            newValueParameter = ensureParameter(valueParameter, /*modifierMask*/ undefined, accessorType);
                        }
                    }
                    if (!newValueParameter) {
                        newValueParameter = ts.createParameter(
                        /*decorators*/ undefined, 
                        /*modifiers*/ undefined, 
                        /*dotDotDotToken*/ undefined, "value");
                    }
                    newParams = ts.append(newParams, newValueParameter);
                }
                return ts.createNodeArray(newParams || ts.emptyArray);
            }
            function ensureTypeParams(node, params) {
                return ts.hasModifier(node, 8 /* Private */) ? undefined : ts.visitNodes(params, visitDeclarationSubtree);
            }
            function isEnclosingDeclaration(node) {
                return ts.isSourceFile(node)
                    || ts.isTypeAliasDeclaration(node)
                    || ts.isModuleDeclaration(node)
                    || ts.isClassDeclaration(node)
                    || ts.isInterfaceDeclaration(node)
                    || ts.isFunctionLike(node)
                    || ts.isIndexSignatureDeclaration(node)
                    || ts.isMappedTypeNode(node);
            }
            function checkEntityNameVisibility(entityName, enclosingDeclaration) {
                var visibilityResult = resolver.isEntityNameVisible(entityName, enclosingDeclaration);
                handleSymbolAccessibilityError(visibilityResult);
                recordTypeReferenceDirectivesIfNecessary(resolver.getTypeReferenceDirectivesForEntityName(entityName));
            }
            function preserveJsDoc(updated, original) {
                if (ts.hasJSDocNodes(updated) && ts.hasJSDocNodes(original)) {
                    updated.jsDoc = original.jsDoc;
                }
                return ts.setCommentRange(updated, ts.getCommentRange(original));
            }
            function rewriteModuleSpecifier(parent, input) {
                if (!input)
                    return undefined; // TODO: GH#18217
                resultHasExternalModuleIndicator = resultHasExternalModuleIndicator || (parent.kind !== 249 /* ModuleDeclaration */ && parent.kind !== 188 /* ImportType */);
                if (ts.isStringLiteralLike(input)) {
                    if (isBundledEmit) {
                        var newName = ts.getExternalModuleNameFromDeclaration(context.getEmitHost(), resolver, parent);
                        if (newName) {
                            return ts.createLiteral(newName);
                        }
                    }
                    else {
                        var symbol = resolver.getSymbolOfExternalModuleSpecifier(input);
                        if (symbol) {
                            (exportedModulesFromDeclarationEmit || (exportedModulesFromDeclarationEmit = [])).push(symbol);
                        }
                    }
                }
                return input;
            }
            function transformImportEqualsDeclaration(decl) {
                if (!resolver.isDeclarationVisible(decl))
                    return;
                if (decl.moduleReference.kind === 265 /* ExternalModuleReference */) {
                    // Rewrite external module names if necessary
                    var specifier = ts.getExternalModuleImportEqualsDeclarationExpression(decl);
                    return ts.updateImportEqualsDeclaration(decl, 
                    /*decorators*/ undefined, decl.modifiers, decl.name, ts.updateExternalModuleReference(decl.moduleReference, rewriteModuleSpecifier(decl, specifier)));
                }
                else {
                    var oldDiag = getSymbolAccessibilityDiagnostic;
                    getSymbolAccessibilityDiagnostic = ts.createGetSymbolAccessibilityDiagnosticForNode(decl);
                    checkEntityNameVisibility(decl.moduleReference, enclosingDeclaration);
                    getSymbolAccessibilityDiagnostic = oldDiag;
                    return decl;
                }
            }
            function transformImportDeclaration(decl) {
                if (!decl.importClause) {
                    // import "mod" - possibly needed for side effects? (global interface patches, module augmentations, etc)
                    return ts.updateImportDeclaration(decl, 
                    /*decorators*/ undefined, decl.modifiers, decl.importClause, rewriteModuleSpecifier(decl, decl.moduleSpecifier));
                }
                // The `importClause` visibility corresponds to the default's visibility.
                var visibleDefaultBinding = decl.importClause && decl.importClause.name && resolver.isDeclarationVisible(decl.importClause) ? decl.importClause.name : undefined;
                if (!decl.importClause.namedBindings) {
                    // No named bindings (either namespace or list), meaning the import is just default or should be elided
                    return visibleDefaultBinding && ts.updateImportDeclaration(decl, /*decorators*/ undefined, decl.modifiers, ts.updateImportClause(decl.importClause, visibleDefaultBinding, 
                    /*namedBindings*/ undefined, decl.importClause.isTypeOnly), rewriteModuleSpecifier(decl, decl.moduleSpecifier));
                }
                if (decl.importClause.namedBindings.kind === 256 /* NamespaceImport */) {
                    // Namespace import (optionally with visible default)
                    var namedBindings = resolver.isDeclarationVisible(decl.importClause.namedBindings) ? decl.importClause.namedBindings : /*namedBindings*/ undefined;
                    return visibleDefaultBinding || namedBindings ? ts.updateImportDeclaration(decl, /*decorators*/ undefined, decl.modifiers, ts.updateImportClause(decl.importClause, visibleDefaultBinding, namedBindings, decl.importClause.isTypeOnly), rewriteModuleSpecifier(decl, decl.moduleSpecifier)) : undefined;
                }
                // Named imports (optionally with visible default)
                var bindingList = ts.mapDefined(decl.importClause.namedBindings.elements, function (b) { return resolver.isDeclarationVisible(b) ? b : undefined; });
                if ((bindingList && bindingList.length) || visibleDefaultBinding) {
                    return ts.updateImportDeclaration(decl, 
                    /*decorators*/ undefined, decl.modifiers, ts.updateImportClause(decl.importClause, visibleDefaultBinding, bindingList && bindingList.length ? ts.updateNamedImports(decl.importClause.namedBindings, bindingList) : undefined, decl.importClause.isTypeOnly), rewriteModuleSpecifier(decl, decl.moduleSpecifier));
                }
                // Nothing visible
            }
            function transformAndReplaceLatePaintedStatements(statements) {
                // This is a `while` loop because `handleSymbolAccessibilityError` can see additional import aliases marked as visible during
                // error handling which must now be included in the output and themselves checked for errors.
                // For example:
                // ```
                // module A {
                //   export module Q {}
                //   import B = Q;
                //   import C = B;
                //   export import D = C;
                // }
                // ```
                // In such a scenario, only Q and D are initially visible, but we don't consider imports as private names - instead we say they if they are referenced they must
                // be recorded. So while checking D's visibility we mark C as visible, then we must check C which in turn marks B, completing the chain of
                // dependent imports and allowing a valid declaration file output. Today, this dependent alias marking only happens for internal import aliases.
                while (ts.length(lateMarkedStatements)) {
                    var i = lateMarkedStatements.shift();
                    if (!ts.isLateVisibilityPaintedStatement(i)) {
                        return ts.Debug.fail("Late replaced statement was found which is not handled by the declaration transformer!: " + (ts.SyntaxKind ? ts.SyntaxKind[i.kind] : i.kind));
                    }
                    var priorNeedsDeclare = needsDeclare;
                    needsDeclare = i.parent && ts.isSourceFile(i.parent) && !(ts.isExternalModule(i.parent) && isBundledEmit);
                    var result = transformTopLevelDeclaration(i);
                    needsDeclare = priorNeedsDeclare;
                    lateStatementReplacementMap.set("" + ts.getOriginalNodeId(i), result);
                }
                // And lastly, we need to get the final form of all those indetermine import declarations from before and add them to the output list
                // (and remove them from the set to examine for outter declarations)
                return ts.visitNodes(statements, visitLateVisibilityMarkedStatements);
                function visitLateVisibilityMarkedStatements(statement) {
                    if (ts.isLateVisibilityPaintedStatement(statement)) {
                        var key = "" + ts.getOriginalNodeId(statement);
                        if (lateStatementReplacementMap.has(key)) {
                            var result = lateStatementReplacementMap.get(key);
                            lateStatementReplacementMap.delete(key);
                            if (result) {
                                if (ts.isArray(result) ? ts.some(result, ts.needsScopeMarker) : ts.needsScopeMarker(result)) {
                                    // Top-level declarations in .d.ts files are always considered exported even without a modifier unless there's an export assignment or specifier
                                    needsScopeFixMarker = true;
                                }
                                if (ts.isSourceFile(statement.parent) && (ts.isArray(result) ? ts.some(result, ts.isExternalModuleIndicator) : ts.isExternalModuleIndicator(result))) {
                                    resultHasExternalModuleIndicator = true;
                                }
                            }
                            return result;
                        }
                    }
                    return statement;
                }
            }
            function visitDeclarationSubtree(input) {
                if (shouldStripInternal(input))
                    return;
                if (ts.isDeclaration(input)) {
                    if (isDeclarationAndNotVisible(input))
                        return;
                    if (ts.hasDynamicName(input) && !resolver.isLateBound(ts.getParseTreeNode(input))) {
                        return;
                    }
                }
                // Elide implementation signatures from overload sets
                if (ts.isFunctionLike(input) && resolver.isImplementationOfOverload(input))
                    return;
                // Elide semicolon class statements
                if (ts.isSemicolonClassElement(input))
                    return;
                var previousEnclosingDeclaration;
                if (isEnclosingDeclaration(input)) {
                    previousEnclosingDeclaration = enclosingDeclaration;
                    enclosingDeclaration = input;
                }
                var oldDiag = getSymbolAccessibilityDiagnostic;
                // Setup diagnostic-related flags before first potential `cleanup` call, otherwise
                // We'd see a TDZ violation at runtime
                var canProduceDiagnostic = ts.canProduceDiagnostics(input);
                var oldWithinObjectLiteralType = suppressNewDiagnosticContexts;
                var shouldEnterSuppressNewDiagnosticsContextContext = ((input.kind === 173 /* TypeLiteral */ || input.kind === 186 /* MappedType */) && input.parent.kind !== 247 /* TypeAliasDeclaration */);
                // Emit methods which are private as properties with no type information
                if (ts.isMethodDeclaration(input) || ts.isMethodSignature(input)) {
                    if (ts.hasModifier(input, 8 /* Private */)) {
                        if (input.symbol && input.symbol.declarations && input.symbol.declarations[0] !== input)
                            return; // Elide all but the first overload
                        return cleanup(ts.createProperty(/*decorators*/ undefined, ensureModifiers(input), input.name, /*questionToken*/ undefined, /*type*/ undefined, /*initializer*/ undefined));
                    }
                }
                if (canProduceDiagnostic && !suppressNewDiagnosticContexts) {
                    getSymbolAccessibilityDiagnostic = ts.createGetSymbolAccessibilityDiagnosticForNode(input);
                }
                if (ts.isTypeQueryNode(input)) {
                    checkEntityNameVisibility(input.exprName, enclosingDeclaration);
                }
                if (shouldEnterSuppressNewDiagnosticsContextContext) {
                    // We stop making new diagnostic contexts within object literal types. Unless it's an object type on the RHS of a type alias declaration. Then we do.
                    suppressNewDiagnosticContexts = true;
                }
                if (isProcessedComponent(input)) {
                    switch (input.kind) {
                        case 216 /* ExpressionWithTypeArguments */: {
                            if ((ts.isEntityName(input.expression) || ts.isEntityNameExpression(input.expression))) {
                                checkEntityNameVisibility(input.expression, enclosingDeclaration);
                            }
                            var node = ts.visitEachChild(input, visitDeclarationSubtree, context);
                            return cleanup(ts.updateExpressionWithTypeArguments(node, ts.parenthesizeTypeParameters(node.typeArguments), node.expression));
                        }
                        case 169 /* TypeReference */: {
                            checkEntityNameVisibility(input.typeName, enclosingDeclaration);
                            var node = ts.visitEachChild(input, visitDeclarationSubtree, context);
                            return cleanup(ts.updateTypeReferenceNode(node, node.typeName, ts.parenthesizeTypeParameters(node.typeArguments)));
                        }
                        case 166 /* ConstructSignature */:
                            return cleanup(ts.updateConstructSignature(input, ensureTypeParams(input, input.typeParameters), updateParamsList(input, input.parameters), ensureType(input, input.type)));
                        case 162 /* Constructor */: {
                            // A constructor declaration may not have a type annotation
                            var ctor = ts.createSignatureDeclaration(162 /* Constructor */, ensureTypeParams(input, input.typeParameters), updateParamsList(input, input.parameters, 0 /* None */), 
                            /*type*/ undefined);
                            ctor.modifiers = ts.createNodeArray(ensureModifiers(input));
                            return cleanup(ctor);
                        }
                        case 161 /* MethodDeclaration */: {
                            if (ts.isPrivateIdentifier(input.name)) {
                                return cleanup(/*returnValue*/ undefined);
                            }
                            var sig = ts.createSignatureDeclaration(160 /* MethodSignature */, ensureTypeParams(input, input.typeParameters), updateParamsList(input, input.parameters), ensureType(input, input.type));
                            sig.name = input.name;
                            sig.modifiers = ts.createNodeArray(ensureModifiers(input));
                            sig.questionToken = input.questionToken;
                            return cleanup(sig);
                        }
                        case 163 /* GetAccessor */: {
                            if (ts.isPrivateIdentifier(input.name)) {
                                return cleanup(/*returnValue*/ undefined);
                            }
                            var accessorType = getTypeAnnotationFromAllAccessorDeclarations(input, resolver.getAllAccessorDeclarations(input));
                            return cleanup(ts.updateGetAccessor(input, 
                            /*decorators*/ undefined, ensureModifiers(input), input.name, updateAccessorParamsList(input, ts.hasModifier(input, 8 /* Private */)), ensureType(input, accessorType), 
                            /*body*/ undefined));
                        }
                        case 164 /* SetAccessor */: {
                            if (ts.isPrivateIdentifier(input.name)) {
                                return cleanup(/*returnValue*/ undefined);
                            }
                            return cleanup(ts.updateSetAccessor(input, 
                            /*decorators*/ undefined, ensureModifiers(input), input.name, updateAccessorParamsList(input, ts.hasModifier(input, 8 /* Private */)), 
                            /*body*/ undefined));
                        }
                        case 159 /* PropertyDeclaration */:
                            if (ts.isPrivateIdentifier(input.name)) {
                                return cleanup(/*returnValue*/ undefined);
                            }
                            return cleanup(ts.updateProperty(input, 
                            /*decorators*/ undefined, ensureModifiers(input), input.name, input.questionToken, ensureType(input, input.type), ensureNoInitializer(input)));
                        case 158 /* PropertySignature */:
                            if (ts.isPrivateIdentifier(input.name)) {
                                return cleanup(/*returnValue*/ undefined);
                            }
                            return cleanup(ts.updatePropertySignature(input, ensureModifiers(input), input.name, input.questionToken, ensureType(input, input.type), ensureNoInitializer(input)));
                        case 160 /* MethodSignature */: {
                            if (ts.isPrivateIdentifier(input.name)) {
                                return cleanup(/*returnValue*/ undefined);
                            }
                            return cleanup(ts.updateMethodSignature(input, ensureTypeParams(input, input.typeParameters), updateParamsList(input, input.parameters), ensureType(input, input.type), input.name, input.questionToken));
                        }
                        case 165 /* CallSignature */: {
                            return cleanup(ts.updateCallSignature(input, ensureTypeParams(input, input.typeParameters), updateParamsList(input, input.parameters), ensureType(input, input.type)));
                        }
                        case 167 /* IndexSignature */: {
                            return cleanup(ts.updateIndexSignature(input, 
                            /*decorators*/ undefined, ensureModifiers(input), updateParamsList(input, input.parameters), ts.visitNode(input.type, visitDeclarationSubtree) || ts.createKeywordTypeNode(125 /* AnyKeyword */)));
                        }
                        case 242 /* VariableDeclaration */: {
                            if (ts.isBindingPattern(input.name)) {
                                return recreateBindingPattern(input.name);
                            }
                            shouldEnterSuppressNewDiagnosticsContextContext = true;
                            suppressNewDiagnosticContexts = true; // Variable declaration types also suppress new diagnostic contexts, provided the contexts wouldn't be made for binding pattern types
                            return cleanup(ts.updateTypeScriptVariableDeclaration(input, input.name, /*exclaimationToken*/ undefined, ensureType(input, input.type), ensureNoInitializer(input)));
                        }
                        case 155 /* TypeParameter */: {
                            if (isPrivateMethodTypeParameter(input) && (input.default || input.constraint)) {
                                return cleanup(ts.updateTypeParameterDeclaration(input, input.name, /*constraint*/ undefined, /*defaultType*/ undefined));
                            }
                            return cleanup(ts.visitEachChild(input, visitDeclarationSubtree, context));
                        }
                        case 180 /* ConditionalType */: {
                            // We have to process conditional types in a special way because for visibility purposes we need to push a new enclosingDeclaration
                            // just for the `infer` types in the true branch. It's an implicit declaration scope that only applies to _part_ of the type.
                            var checkType = ts.visitNode(input.checkType, visitDeclarationSubtree);
                            var extendsType = ts.visitNode(input.extendsType, visitDeclarationSubtree);
                            var oldEnclosingDecl = enclosingDeclaration;
                            enclosingDeclaration = input.trueType;
                            var trueType = ts.visitNode(input.trueType, visitDeclarationSubtree);
                            enclosingDeclaration = oldEnclosingDecl;
                            var falseType = ts.visitNode(input.falseType, visitDeclarationSubtree);
                            return cleanup(ts.updateConditionalTypeNode(input, checkType, extendsType, trueType, falseType));
                        }
                        case 170 /* FunctionType */: {
                            return cleanup(ts.updateFunctionTypeNode(input, ts.visitNodes(input.typeParameters, visitDeclarationSubtree), updateParamsList(input, input.parameters), ts.visitNode(input.type, visitDeclarationSubtree)));
                        }
                        case 171 /* ConstructorType */: {
                            return cleanup(ts.updateConstructorTypeNode(input, ts.visitNodes(input.typeParameters, visitDeclarationSubtree), updateParamsList(input, input.parameters), ts.visitNode(input.type, visitDeclarationSubtree)));
                        }
                        case 188 /* ImportType */: {
                            if (!ts.isLiteralImportTypeNode(input))
                                return cleanup(input);
                            return cleanup(ts.updateImportTypeNode(input, ts.updateLiteralTypeNode(input.argument, rewriteModuleSpecifier(input, input.argument.literal)), input.qualifier, ts.visitNodes(input.typeArguments, visitDeclarationSubtree, ts.isTypeNode), input.isTypeOf));
                        }
                        default: ts.Debug.assertNever(input, "Attempted to process unhandled node kind: " + ts.SyntaxKind[input.kind]);
                    }
                }
                return cleanup(ts.visitEachChild(input, visitDeclarationSubtree, context));
                function cleanup(returnValue) {
                    if (returnValue && canProduceDiagnostic && ts.hasDynamicName(input)) {
                        checkName(input);
                    }
                    if (isEnclosingDeclaration(input)) {
                        enclosingDeclaration = previousEnclosingDeclaration;
                    }
                    if (canProduceDiagnostic && !suppressNewDiagnosticContexts) {
                        getSymbolAccessibilityDiagnostic = oldDiag;
                    }
                    if (shouldEnterSuppressNewDiagnosticsContextContext) {
                        suppressNewDiagnosticContexts = oldWithinObjectLiteralType;
                    }
                    if (returnValue === input) {
                        return returnValue;
                    }
                    return returnValue && ts.setOriginalNode(preserveJsDoc(returnValue, input), input);
                }
            }
            function isPrivateMethodTypeParameter(node) {
                return node.parent.kind === 161 /* MethodDeclaration */ && ts.hasModifier(node.parent, 8 /* Private */);
            }
            function visitDeclarationStatements(input) {
                if (!isPreservedDeclarationStatement(input)) {
                    // return undefined for unmatched kinds to omit them from the tree
                    return;
                }
                if (shouldStripInternal(input))
                    return;
                switch (input.kind) {
                    case 260 /* ExportDeclaration */: {
                        if (ts.isSourceFile(input.parent)) {
                            resultHasExternalModuleIndicator = true;
                        }
                        resultHasScopeMarker = true;
                        // Always visible if the parent node isn't dropped for being not visible
                        // Rewrite external module names if necessary
                        return ts.updateExportDeclaration(input, 
                        /*decorators*/ undefined, input.modifiers, input.exportClause, rewriteModuleSpecifier(input, input.moduleSpecifier), input.isTypeOnly);
                    }
                    case 259 /* ExportAssignment */: {
                        // Always visible if the parent node isn't dropped for being not visible
                        if (ts.isSourceFile(input.parent)) {
                            resultHasExternalModuleIndicator = true;
                        }
                        resultHasScopeMarker = true;
                        if (input.expression.kind === 75 /* Identifier */) {
                            return input;
                        }
                        else {
                            var newId = ts.createOptimisticUniqueName("_default");
                            getSymbolAccessibilityDiagnostic = function () {
                                return ({
                                    diagnosticMessage: ts.Diagnostics.Default_export_of_the_module_has_or_is_using_private_name_0,
                                    errorNode: input
                                });
                            };
                            var varDecl = ts.createVariableDeclaration(newId, resolver.createTypeOfExpression(input.expression, input, declarationEmitNodeBuilderFlags, symbolTracker), /*initializer*/ undefined);
                            var statement = ts.createVariableStatement(needsDeclare ? [ts.createModifier(130 /* DeclareKeyword */)] : [], ts.createVariableDeclarationList([varDecl], 2 /* Const */));
                            return [statement, ts.updateExportAssignment(input, input.decorators, input.modifiers, newId)];
                        }
                    }
                }
                var result = transformTopLevelDeclaration(input);
                // Don't actually transform yet; just leave as original node - will be elided/swapped by late pass
                lateStatementReplacementMap.set("" + ts.getOriginalNodeId(input), result);
                return input;
            }
            function stripExportModifiers(statement) {
                if (ts.isImportEqualsDeclaration(statement) || ts.hasModifier(statement, 512 /* Default */)) {
                    // `export import` statements should remain as-is, as imports are _not_ implicitly exported in an ambient namespace
                    // Likewise, `export default` classes and the like and just be `default`, so we preserve their `export` modifiers, too
                    return statement;
                }
                var clone = ts.getMutableClone(statement);
                var modifiers = ts.createModifiersFromModifierFlags(ts.getModifierFlags(statement) & (3071 /* All */ ^ 1 /* Export */));
                clone.modifiers = modifiers.length ? ts.createNodeArray(modifiers) : undefined;
                return clone;
            }
            function transformTopLevelDeclaration(input) {
                if (shouldStripInternal(input))
                    return;
                switch (input.kind) {
                    case 253 /* ImportEqualsDeclaration */: {
                        return transformImportEqualsDeclaration(input);
                    }
                    case 254 /* ImportDeclaration */: {
                        return transformImportDeclaration(input);
                    }
                }
                if (ts.isDeclaration(input) && isDeclarationAndNotVisible(input))
                    return;
                // Elide implementation signatures from overload sets
                if (ts.isFunctionLike(input) && resolver.isImplementationOfOverload(input))
                    return;
                var previousEnclosingDeclaration;
                if (isEnclosingDeclaration(input)) {
                    previousEnclosingDeclaration = enclosingDeclaration;
                    enclosingDeclaration = input;
                }
                var canProdiceDiagnostic = ts.canProduceDiagnostics(input);
                var oldDiag = getSymbolAccessibilityDiagnostic;
                if (canProdiceDiagnostic) {
                    getSymbolAccessibilityDiagnostic = ts.createGetSymbolAccessibilityDiagnosticForNode(input);
                }
                var previousNeedsDeclare = needsDeclare;
                switch (input.kind) {
                    case 247 /* TypeAliasDeclaration */: // Type aliases get `declare`d if need be (for legacy support), but that's all
                        return cleanup(ts.updateTypeAliasDeclaration(input, 
                        /*decorators*/ undefined, ensureModifiers(input), input.name, ts.visitNodes(input.typeParameters, visitDeclarationSubtree, ts.isTypeParameterDeclaration), ts.visitNode(input.type, visitDeclarationSubtree, ts.isTypeNode)));
                    case 246 /* InterfaceDeclaration */: {
                        return cleanup(ts.updateInterfaceDeclaration(input, 
                        /*decorators*/ undefined, ensureModifiers(input), input.name, ensureTypeParams(input, input.typeParameters), transformHeritageClauses(input.heritageClauses), ts.visitNodes(input.members, visitDeclarationSubtree)));
                    }
                    case 244 /* FunctionDeclaration */: {
                        // Generators lose their generator-ness, excepting their return type
                        var clean = cleanup(ts.updateFunctionDeclaration(input, 
                        /*decorators*/ undefined, ensureModifiers(input), 
                        /*asteriskToken*/ undefined, input.name, ensureTypeParams(input, input.typeParameters), updateParamsList(input, input.parameters), ensureType(input, input.type), 
                        /*body*/ undefined));
                        if (clean && resolver.isExpandoFunctionDeclaration(input)) {
                            var props = resolver.getPropertiesOfContainerFunction(input);
                            var fakespace_1 = ts.createModuleDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, clean.name || ts.createIdentifier("_default"), ts.createModuleBlock([]), 16 /* Namespace */);
                            fakespace_1.flags ^= 8 /* Synthesized */; // unset synthesized so it is usable as an enclosing declaration
                            fakespace_1.parent = enclosingDeclaration;
                            fakespace_1.locals = ts.createSymbolTable(props);
                            fakespace_1.symbol = props[0].parent;
                            var declarations = ts.mapDefined(props, function (p) {
                                if (!ts.isPropertyAccessExpression(p.valueDeclaration)) {
                                    return undefined; // TODO GH#33569: Handle element access expressions that created late bound names (rather than silently omitting them)
                                }
                                getSymbolAccessibilityDiagnostic = ts.createGetSymbolAccessibilityDiagnosticForNode(p.valueDeclaration);
                                var type = resolver.createTypeOfDeclaration(p.valueDeclaration, fakespace_1, declarationEmitNodeBuilderFlags, symbolTracker);
                                getSymbolAccessibilityDiagnostic = oldDiag;
                                var varDecl = ts.createVariableDeclaration(ts.unescapeLeadingUnderscores(p.escapedName), type, /*initializer*/ undefined);
                                return ts.createVariableStatement(/*modifiers*/ undefined, ts.createVariableDeclarationList([varDecl]));
                            });
                            var namespaceDecl = ts.createModuleDeclaration(/*decorators*/ undefined, ensureModifiers(input), input.name, ts.createModuleBlock(declarations), 16 /* Namespace */);
                            if (!ts.hasModifier(clean, 512 /* Default */)) {
                                return [clean, namespaceDecl];
                            }
                            var modifiers = ts.createModifiersFromModifierFlags((ts.getModifierFlags(clean) & ~513 /* ExportDefault */) | 2 /* Ambient */);
                            var cleanDeclaration = ts.updateFunctionDeclaration(clean, 
                            /*decorators*/ undefined, modifiers, 
                            /*asteriskToken*/ undefined, clean.name, clean.typeParameters, clean.parameters, clean.type, 
                            /*body*/ undefined);
                            var namespaceDeclaration = ts.updateModuleDeclaration(namespaceDecl, 
                            /*decorators*/ undefined, modifiers, namespaceDecl.name, namespaceDecl.body);
                            var exportDefaultDeclaration = ts.createExportAssignment(
                            /*decorators*/ undefined, 
                            /*modifiers*/ undefined, 
                            /*isExportEquals*/ false, namespaceDecl.name);
                            if (ts.isSourceFile(input.parent)) {
                                resultHasExternalModuleIndicator = true;
                            }
                            resultHasScopeMarker = true;
                            return [cleanDeclaration, namespaceDeclaration, exportDefaultDeclaration];
                        }
                        else {
                            return clean;
                        }
                    }
                    case 249 /* ModuleDeclaration */: {
                        needsDeclare = false;
                        var inner = input.body;
                        if (inner && inner.kind === 250 /* ModuleBlock */) {
                            var oldNeedsScopeFix = needsScopeFixMarker;
                            var oldHasScopeFix = resultHasScopeMarker;
                            resultHasScopeMarker = false;
                            needsScopeFixMarker = false;
                            var statements = ts.visitNodes(inner.statements, visitDeclarationStatements);
                            var lateStatements = transformAndReplaceLatePaintedStatements(statements);
                            if (input.flags & 8388608 /* Ambient */) {
                                needsScopeFixMarker = false; // If it was `declare`'d everything is implicitly exported already, ignore late printed "privates"
                            }
                            // With the final list of statements, there are 3 possibilities:
                            // 1. There's an export assignment or export declaration in the namespace - do nothing
                            // 2. Everything is exported and there are no export assignments or export declarations - strip all export modifiers
                            // 3. Some things are exported, some are not, and there's no marker - add an empty marker
                            if (!ts.isGlobalScopeAugmentation(input) && !hasScopeMarker(lateStatements) && !resultHasScopeMarker) {
                                if (needsScopeFixMarker) {
                                    lateStatements = ts.createNodeArray(__spreadArrays(lateStatements, [ts.createEmptyExports()]));
                                }
                                else {
                                    lateStatements = ts.visitNodes(lateStatements, stripExportModifiers);
                                }
                            }
                            var body = ts.updateModuleBlock(inner, lateStatements);
                            needsDeclare = previousNeedsDeclare;
                            needsScopeFixMarker = oldNeedsScopeFix;
                            resultHasScopeMarker = oldHasScopeFix;
                            var mods = ensureModifiers(input);
                            return cleanup(ts.updateModuleDeclaration(input, 
                            /*decorators*/ undefined, mods, ts.isExternalModuleAugmentation(input) ? rewriteModuleSpecifier(input, input.name) : input.name, body));
                        }
                        else {
                            needsDeclare = previousNeedsDeclare;
                            var mods = ensureModifiers(input);
                            needsDeclare = false;
                            ts.visitNode(inner, visitDeclarationStatements);
                            // eagerly transform nested namespaces (the nesting doesn't need any elision or painting done)
                            var id = "" + ts.getOriginalNodeId(inner); // TODO: GH#18217
                            var body = lateStatementReplacementMap.get(id);
                            lateStatementReplacementMap.delete(id);
                            return cleanup(ts.updateModuleDeclaration(input, 
                            /*decorators*/ undefined, mods, input.name, body));
                        }
                    }
                    case 245 /* ClassDeclaration */: {
                        var modifiers = ts.createNodeArray(ensureModifiers(input));
                        var typeParameters = ensureTypeParams(input, input.typeParameters);
                        var ctor = ts.getFirstConstructorWithBody(input);
                        var parameterProperties = void 0;
                        if (ctor) {
                            var oldDiag_1 = getSymbolAccessibilityDiagnostic;
                            parameterProperties = ts.compact(ts.flatMap(ctor.parameters, function (param) {
                                if (!ts.hasModifier(param, 92 /* ParameterPropertyModifier */) || shouldStripInternal(param))
                                    return;
                                getSymbolAccessibilityDiagnostic = ts.createGetSymbolAccessibilityDiagnosticForNode(param);
                                if (param.name.kind === 75 /* Identifier */) {
                                    return preserveJsDoc(ts.createProperty(
                                    /*decorators*/ undefined, ensureModifiers(param), param.name, param.questionToken, ensureType(param, param.type), ensureNoInitializer(param)), param);
                                }
                                else {
                                    // Pattern - this is currently an error, but we emit declarations for it somewhat correctly
                                    return walkBindingPattern(param.name);
                                }
                                function walkBindingPattern(pattern) {
                                    var elems;
                                    for (var _i = 0, _a = pattern.elements; _i < _a.length; _i++) {
                                        var elem = _a[_i];
                                        if (ts.isOmittedExpression(elem))
                                            continue;
                                        if (ts.isBindingPattern(elem.name)) {
                                            elems = ts.concatenate(elems, walkBindingPattern(elem.name));
                                        }
                                        elems = elems || [];
                                        elems.push(ts.createProperty(
                                        /*decorators*/ undefined, ensureModifiers(param), elem.name, 
                                        /*questionToken*/ undefined, ensureType(elem, /*type*/ undefined), 
                                        /*initializer*/ undefined));
                                    }
                                    return elems;
                                }
                            }));
                            getSymbolAccessibilityDiagnostic = oldDiag_1;
                        }
                        var hasPrivateIdentifier = ts.some(input.members, function (member) { return !!member.name && ts.isPrivateIdentifier(member.name); });
                        var privateIdentifier = hasPrivateIdentifier ? [
                            ts.createProperty(
                            /*decorators*/ undefined, 
                            /*modifiers*/ undefined, ts.createPrivateIdentifier("#private"), 
                            /*questionToken*/ undefined, 
                            /*type*/ undefined, 
                            /*initializer*/ undefined)
                        ] : undefined;
                        var memberNodes = ts.concatenate(ts.concatenate(privateIdentifier, parameterProperties), ts.visitNodes(input.members, visitDeclarationSubtree));
                        var members = ts.createNodeArray(memberNodes);
                        var extendsClause_1 = ts.getEffectiveBaseTypeNode(input);
                        if (extendsClause_1 && !ts.isEntityNameExpression(extendsClause_1.expression) && extendsClause_1.expression.kind !== 100 /* NullKeyword */) {
                            // We must add a temporary declaration for the extends clause expression
                            var oldId = input.name ? ts.unescapeLeadingUnderscores(input.name.escapedText) : "default";
                            var newId_1 = ts.createOptimisticUniqueName(oldId + "_base");
                            getSymbolAccessibilityDiagnostic = function () {
                                return ({
                                    diagnosticMessage: ts.Diagnostics.extends_clause_of_exported_class_0_has_or_is_using_private_name_1,
                                    errorNode: extendsClause_1,
                                    typeName: input.name
                                });
                            };
                            var varDecl = ts.createVariableDeclaration(newId_1, resolver.createTypeOfExpression(extendsClause_1.expression, input, declarationEmitNodeBuilderFlags, symbolTracker), /*initializer*/ undefined);
                            var statement = ts.createVariableStatement(needsDeclare ? [ts.createModifier(130 /* DeclareKeyword */)] : [], ts.createVariableDeclarationList([varDecl], 2 /* Const */));
                            var heritageClauses = ts.createNodeArray(ts.map(input.heritageClauses, function (clause) {
                                if (clause.token === 90 /* ExtendsKeyword */) {
                                    var oldDiag_2 = getSymbolAccessibilityDiagnostic;
                                    getSymbolAccessibilityDiagnostic = ts.createGetSymbolAccessibilityDiagnosticForNode(clause.types[0]);
                                    var newClause = ts.updateHeritageClause(clause, ts.map(clause.types, function (t) { return ts.updateExpressionWithTypeArguments(t, ts.visitNodes(t.typeArguments, visitDeclarationSubtree), newId_1); }));
                                    getSymbolAccessibilityDiagnostic = oldDiag_2;
                                    return newClause;
                                }
                                return ts.updateHeritageClause(clause, ts.visitNodes(ts.createNodeArray(ts.filter(clause.types, function (t) { return ts.isEntityNameExpression(t.expression) || t.expression.kind === 100 /* NullKeyword */; })), visitDeclarationSubtree));
                            }));
                            return [statement, cleanup(ts.updateClassDeclaration(input, 
                                /*decorators*/ undefined, modifiers, input.name, typeParameters, heritageClauses, members))]; // TODO: GH#18217
                        }
                        else {
                            var heritageClauses = transformHeritageClauses(input.heritageClauses);
                            return cleanup(ts.updateClassDeclaration(input, 
                            /*decorators*/ undefined, modifiers, input.name, typeParameters, heritageClauses, members));
                        }
                    }
                    case 225 /* VariableStatement */: {
                        return cleanup(transformVariableStatement(input));
                    }
                    case 248 /* EnumDeclaration */: {
                        return cleanup(ts.updateEnumDeclaration(input, /*decorators*/ undefined, ts.createNodeArray(ensureModifiers(input)), input.name, ts.createNodeArray(ts.mapDefined(input.members, function (m) {
                            if (shouldStripInternal(m))
                                return;
                            // Rewrite enum values to their constants, if available
                            var constValue = resolver.getConstantValue(m);
                            return preserveJsDoc(ts.updateEnumMember(m, m.name, constValue !== undefined ? ts.createLiteral(constValue) : undefined), m);
                        }))));
                    }
                }
                // Anything left unhandled is an error, so this should be unreachable
                return ts.Debug.assertNever(input, "Unhandled top-level node in declaration emit: " + ts.SyntaxKind[input.kind]);
                function cleanup(node) {
                    if (isEnclosingDeclaration(input)) {
                        enclosingDeclaration = previousEnclosingDeclaration;
                    }
                    if (canProdiceDiagnostic) {
                        getSymbolAccessibilityDiagnostic = oldDiag;
                    }
                    if (input.kind === 249 /* ModuleDeclaration */) {
                        needsDeclare = previousNeedsDeclare;
                    }
                    if (node === input) {
                        return node;
                    }
                    return node && ts.setOriginalNode(preserveJsDoc(node, input), input);
                }
            }
            function transformVariableStatement(input) {
                if (!ts.forEach(input.declarationList.declarations, getBindingNameVisible))
                    return;
                var nodes = ts.visitNodes(input.declarationList.declarations, visitDeclarationSubtree);
                if (!ts.length(nodes))
                    return;
                return ts.updateVariableStatement(input, ts.createNodeArray(ensureModifiers(input)), ts.updateVariableDeclarationList(input.declarationList, nodes));
            }
            function recreateBindingPattern(d) {
                return ts.flatten(ts.mapDefined(d.elements, function (e) { return recreateBindingElement(e); }));
            }
            function recreateBindingElement(e) {
                if (e.kind === 215 /* OmittedExpression */) {
                    return;
                }
                if (e.name) {
                    if (!getBindingNameVisible(e))
                        return;
                    if (ts.isBindingPattern(e.name)) {
                        return recreateBindingPattern(e.name);
                    }
                    else {
                        return ts.createVariableDeclaration(e.name, ensureType(e, /*type*/ undefined), /*initializer*/ undefined);
                    }
                }
            }
            function checkName(node) {
                var oldDiag;
                if (!suppressNewDiagnosticContexts) {
                    oldDiag = getSymbolAccessibilityDiagnostic;
                    getSymbolAccessibilityDiagnostic = ts.createGetSymbolAccessibilityDiagnosticForNodeName(node);
                }
                errorNameNode = node.name;
                ts.Debug.assert(resolver.isLateBound(ts.getParseTreeNode(node))); // Should only be called with dynamic names
                var decl = node;
                var entityName = decl.name.expression;
                checkEntityNameVisibility(entityName, enclosingDeclaration);
                if (!suppressNewDiagnosticContexts) {
                    getSymbolAccessibilityDiagnostic = oldDiag;
                }
                errorNameNode = undefined;
            }
            function shouldStripInternal(node) {
                return !!stripInternal && !!node && isInternalDeclaration(node, currentSourceFile);
            }
            function isScopeMarker(node) {
                return ts.isExportAssignment(node) || ts.isExportDeclaration(node);
            }
            function hasScopeMarker(statements) {
                return ts.some(statements, isScopeMarker);
            }
            function ensureModifiers(node) {
                var currentFlags = ts.getModifierFlags(node);
                var newFlags = ensureModifierFlags(node);
                if (currentFlags === newFlags) {
                    return node.modifiers;
                }
                return ts.createModifiersFromModifierFlags(newFlags);
            }
            function ensureModifierFlags(node) {
                var mask = 3071 /* All */ ^ (4 /* Public */ | 256 /* Async */); // No async modifiers in declaration files
                var additions = (needsDeclare && !isAlwaysType(node)) ? 2 /* Ambient */ : 0 /* None */;
                var parentIsFile = node.parent.kind === 290 /* SourceFile */;
                if (!parentIsFile || (isBundledEmit && parentIsFile && ts.isExternalModule(node.parent))) {
                    mask ^= 2 /* Ambient */;
                    additions = 0 /* None */;
                }
                return maskModifierFlags(node, mask, additions);
            }
            function getTypeAnnotationFromAllAccessorDeclarations(node, accessors) {
                var accessorType = getTypeAnnotationFromAccessor(node);
                if (!accessorType && node !== accessors.firstAccessor) {
                    accessorType = getTypeAnnotationFromAccessor(accessors.firstAccessor);
                    // If we end up pulling the type from the second accessor, we also need to change the diagnostic context to get the expected error message
                    getSymbolAccessibilityDiagnostic = ts.createGetSymbolAccessibilityDiagnosticForNode(accessors.firstAccessor);
                }
                if (!accessorType && accessors.secondAccessor && node !== accessors.secondAccessor) {
                    accessorType = getTypeAnnotationFromAccessor(accessors.secondAccessor);
                    // If we end up pulling the type from the second accessor, we also need to change the diagnostic context to get the expected error message
                    getSymbolAccessibilityDiagnostic = ts.createGetSymbolAccessibilityDiagnosticForNode(accessors.secondAccessor);
                }
                return accessorType;
            }
            function transformHeritageClauses(nodes) {
                return ts.createNodeArray(ts.filter(ts.map(nodes, function (clause) {
                    return ts.updateHeritageClause(clause, ts.visitNodes(ts.createNodeArray(ts.filter(clause.types, function (t) {
                        return ts.isEntityNameExpression(t.expression) || (clause.token === 90 /* ExtendsKeyword */ && t.expression.kind === 100 /* NullKeyword */);
                    })), visitDeclarationSubtree));
                }), function (clause) { return clause.types && !!clause.types.length; }));
            }
        }
        ts.transformDeclarations = transformDeclarations;
        function isAlwaysType(node) {
            if (node.kind === 246 /* InterfaceDeclaration */) {
                return true;
            }
            return false;
        }
        // Elide "public" modifier, as it is the default
        function maskModifiers(node, modifierMask, modifierAdditions) {
            return ts.createModifiersFromModifierFlags(maskModifierFlags(node, modifierMask, modifierAdditions));
        }
        function maskModifierFlags(node, modifierMask, modifierAdditions) {
            if (modifierMask === void 0) {
                modifierMask = 3071 /* All */ ^ 4 /* Public */;
            }
            if (modifierAdditions === void 0) {
                modifierAdditions = 0 /* None */;
            }
            var flags = (ts.getModifierFlags(node) & modifierMask) | modifierAdditions;
            if (flags & 512 /* Default */ && !(flags & 1 /* Export */)) {
                // A non-exported default is a nonsequitor - we usually try to remove all export modifiers
                // from statements in ambient declarations; but a default export must retain its export modifier to be syntactically valid
                flags ^= 1 /* Export */;
            }
            if (flags & 512 /* Default */ && flags & 2 /* Ambient */) {
                flags ^= 2 /* Ambient */; // `declare` is never required alongside `default` (and would be an error if printed)
            }
            return flags;
        }
        function getTypeAnnotationFromAccessor(accessor) {
            if (accessor) {
                return accessor.kind === 163 /* GetAccessor */
                    ? accessor.type // Getter - return type
                    : accessor.parameters.length > 0
                        ? accessor.parameters[0].type // Setter parameter type
                        : undefined;
            }
        }
        function canHaveLiteralInitializer(node) {
            switch (node.kind) {
                case 159 /* PropertyDeclaration */:
                case 158 /* PropertySignature */:
                    return !ts.hasModifier(node, 8 /* Private */);
                case 156 /* Parameter */:
                case 242 /* VariableDeclaration */:
                    return true;
            }
            return false;
        }
        function isPreservedDeclarationStatement(node) {
            switch (node.kind) {
                case 244 /* FunctionDeclaration */:
                case 249 /* ModuleDeclaration */:
                case 253 /* ImportEqualsDeclaration */:
                case 246 /* InterfaceDeclaration */:
                case 245 /* ClassDeclaration */:
                case 247 /* TypeAliasDeclaration */:
                case 248 /* EnumDeclaration */:
                case 225 /* VariableStatement */:
                case 254 /* ImportDeclaration */:
                case 260 /* ExportDeclaration */:
                case 259 /* ExportAssignment */:
                    return true;
            }
            return false;
        }
        function isProcessedComponent(node) {
            switch (node.kind) {
                case 166 /* ConstructSignature */:
                case 162 /* Constructor */:
                case 161 /* MethodDeclaration */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 159 /* PropertyDeclaration */:
                case 158 /* PropertySignature */:
                case 160 /* MethodSignature */:
                case 165 /* CallSignature */:
                case 167 /* IndexSignature */:
                case 242 /* VariableDeclaration */:
                case 155 /* TypeParameter */:
                case 216 /* ExpressionWithTypeArguments */:
                case 169 /* TypeReference */:
                case 180 /* ConditionalType */:
                case 170 /* FunctionType */:
                case 171 /* ConstructorType */:
                case 188 /* ImportType */:
                    return true;
            }
            return false;
        }
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        function getModuleTransformer(moduleKind) {
            switch (moduleKind) {
                case ts.ModuleKind.ESNext:
                case ts.ModuleKind.ES2020:
                case ts.ModuleKind.ES2015:
                    return ts.transformECMAScriptModule;
                case ts.ModuleKind.System:
                    return ts.transformSystemModule;
                default:
                    return ts.transformModule;
            }
        }
        var TransformationState;
        (function (TransformationState) {
            TransformationState[TransformationState["Uninitialized"] = 0] = "Uninitialized";
            TransformationState[TransformationState["Initialized"] = 1] = "Initialized";
            TransformationState[TransformationState["Completed"] = 2] = "Completed";
            TransformationState[TransformationState["Disposed"] = 3] = "Disposed";
        })(TransformationState || (TransformationState = {}));
        var SyntaxKindFeatureFlags;
        (function (SyntaxKindFeatureFlags) {
            SyntaxKindFeatureFlags[SyntaxKindFeatureFlags["Substitution"] = 1] = "Substitution";
            SyntaxKindFeatureFlags[SyntaxKindFeatureFlags["EmitNotifications"] = 2] = "EmitNotifications";
        })(SyntaxKindFeatureFlags || (SyntaxKindFeatureFlags = {}));
        ts.noTransformers = { scriptTransformers: ts.emptyArray, declarationTransformers: ts.emptyArray };
        function getTransformers(compilerOptions, customTransformers, emitOnlyDtsFiles) {
            return {
                scriptTransformers: getScriptTransformers(compilerOptions, customTransformers, emitOnlyDtsFiles),
                declarationTransformers: getDeclarationTransformers(customTransformers),
            };
        }
        ts.getTransformers = getTransformers;
        function getScriptTransformers(compilerOptions, customTransformers, emitOnlyDtsFiles) {
            if (emitOnlyDtsFiles)
                return ts.emptyArray;
            var jsx = compilerOptions.jsx;
            var languageVersion = ts.getEmitScriptTarget(compilerOptions);
            var moduleKind = ts.getEmitModuleKind(compilerOptions);
            var transformers = [];
            ts.addRange(transformers, customTransformers && ts.map(customTransformers.before, wrapScriptTransformerFactory));
            transformers.push(ts.transformTypeScript);
            transformers.push(ts.transformClassFields);
            if (jsx === 2 /* React */) {
                transformers.push(ts.transformJsx);
            }
            if (languageVersion < 99 /* ESNext */) {
                transformers.push(ts.transformESNext);
            }
            if (languageVersion < 7 /* ES2020 */) {
                transformers.push(ts.transformES2020);
            }
            if (languageVersion < 6 /* ES2019 */) {
                transformers.push(ts.transformES2019);
            }
            if (languageVersion < 5 /* ES2018 */) {
                transformers.push(ts.transformES2018);
            }
            if (languageVersion < 4 /* ES2017 */) {
                transformers.push(ts.transformES2017);
            }
            if (languageVersion < 3 /* ES2016 */) {
                transformers.push(ts.transformES2016);
            }
            if (languageVersion < 2 /* ES2015 */) {
                transformers.push(ts.transformES2015);
                transformers.push(ts.transformGenerators);
            }
            transformers.push(getModuleTransformer(moduleKind));
            // The ES5 transformer is last so that it can substitute expressions like `exports.default`
            // for ES3.
            if (languageVersion < 1 /* ES5 */) {
                transformers.push(ts.transformES5);
            }
            ts.addRange(transformers, customTransformers && ts.map(customTransformers.after, wrapScriptTransformerFactory));
            return transformers;
        }
        function getDeclarationTransformers(customTransformers) {
            var transformers = [];
            transformers.push(ts.transformDeclarations);
            ts.addRange(transformers, customTransformers && ts.map(customTransformers.afterDeclarations, wrapDeclarationTransformerFactory));
            return transformers;
        }
        /**
         * Wrap a custom script or declaration transformer object in a `Transformer` callback with fallback support for transforming bundles.
         */
        function wrapCustomTransformer(transformer) {
            return function (node) { return ts.isBundle(node) ? transformer.transformBundle(node) : transformer.transformSourceFile(node); };
        }
        /**
         * Wrap a transformer factory that may return a custom script or declaration transformer object.
         */
        function wrapCustomTransformerFactory(transformer, handleDefault) {
            return function (context) {
                var customTransformer = transformer(context);
                return typeof customTransformer === "function"
                    ? handleDefault(customTransformer)
                    : wrapCustomTransformer(customTransformer);
            };
        }
        function wrapScriptTransformerFactory(transformer) {
            return wrapCustomTransformerFactory(transformer, ts.chainBundle);
        }
        function wrapDeclarationTransformerFactory(transformer) {
            return wrapCustomTransformerFactory(transformer, ts.identity);
        }
        function noEmitSubstitution(_hint, node) {
            return node;
        }
        ts.noEmitSubstitution = noEmitSubstitution;
        function noEmitNotification(hint, node, callback) {
            callback(hint, node);
        }
        ts.noEmitNotification = noEmitNotification;
        /**
         * Transforms an array of SourceFiles by passing them through each transformer.
         *
         * @param resolver The emit resolver provided by the checker.
         * @param host The emit host object used to interact with the file system.
         * @param options Compiler options to surface in the `TransformationContext`.
         * @param nodes An array of nodes to transform.
         * @param transforms An array of `TransformerFactory` callbacks.
         * @param allowDtsFiles A value indicating whether to allow the transformation of .d.ts files.
         */
        function transformNodes(resolver, host, options, nodes, transformers, allowDtsFiles) {
            var enabledSyntaxKindFeatures = new Array(330 /* Count */);
            var lexicalEnvironmentVariableDeclarations;
            var lexicalEnvironmentFunctionDeclarations;
            var lexicalEnvironmentVariableDeclarationsStack = [];
            var lexicalEnvironmentFunctionDeclarationsStack = [];
            var lexicalEnvironmentStackOffset = 0;
            var lexicalEnvironmentSuspended = false;
            var emitHelpers;
            var onSubstituteNode = noEmitSubstitution;
            var onEmitNode = noEmitNotification;
            var state = 0 /* Uninitialized */;
            var diagnostics = [];
            // The transformation context is provided to each transformer as part of transformer
            // initialization.
            var context = {
                getCompilerOptions: function () { return options; },
                getEmitResolver: function () { return resolver; },
                getEmitHost: function () { return host; },
                startLexicalEnvironment: startLexicalEnvironment,
                suspendLexicalEnvironment: suspendLexicalEnvironment,
                resumeLexicalEnvironment: resumeLexicalEnvironment,
                endLexicalEnvironment: endLexicalEnvironment,
                hoistVariableDeclaration: hoistVariableDeclaration,
                hoistFunctionDeclaration: hoistFunctionDeclaration,
                requestEmitHelper: requestEmitHelper,
                readEmitHelpers: readEmitHelpers,
                enableSubstitution: enableSubstitution,
                enableEmitNotification: enableEmitNotification,
                isSubstitutionEnabled: isSubstitutionEnabled,
                isEmitNotificationEnabled: isEmitNotificationEnabled,
                get onSubstituteNode() { return onSubstituteNode; },
                set onSubstituteNode(value) {
                    ts.Debug.assert(state < 1 /* Initialized */, "Cannot modify transformation hooks after initialization has completed.");
                    ts.Debug.assert(value !== undefined, "Value must not be 'undefined'");
                    onSubstituteNode = value;
                },
                get onEmitNode() { return onEmitNode; },
                set onEmitNode(value) {
                    ts.Debug.assert(state < 1 /* Initialized */, "Cannot modify transformation hooks after initialization has completed.");
                    ts.Debug.assert(value !== undefined, "Value must not be 'undefined'");
                    onEmitNode = value;
                },
                addDiagnostic: function (diag) {
                    diagnostics.push(diag);
                }
            };
            // Ensure the parse tree is clean before applying transformations
            for (var _i = 0, nodes_4 = nodes; _i < nodes_4.length; _i++) {
                var node = nodes_4[_i];
                ts.disposeEmitNodes(ts.getSourceFileOfNode(ts.getParseTreeNode(node)));
            }
            ts.performance.mark("beforeTransform");
            // Chain together and initialize each transformer.
            var transformersWithContext = transformers.map(function (t) { return t(context); });
            var transformation = function (node) {
                for (var _i = 0, transformersWithContext_1 = transformersWithContext; _i < transformersWithContext_1.length; _i++) {
                    var transform = transformersWithContext_1[_i];
                    node = transform(node);
                }
                return node;
            };
            // prevent modification of transformation hooks.
            state = 1 /* Initialized */;
            // Transform each node.
            var transformed = ts.map(nodes, allowDtsFiles ? transformation : transformRoot);
            // prevent modification of the lexical environment.
            state = 2 /* Completed */;
            ts.performance.mark("afterTransform");
            ts.performance.measure("transformTime", "beforeTransform", "afterTransform");
            return {
                transformed: transformed,
                substituteNode: substituteNode,
                emitNodeWithNotification: emitNodeWithNotification,
                isEmitNotificationEnabled: isEmitNotificationEnabled,
                dispose: dispose,
                diagnostics: diagnostics
            };
            function transformRoot(node) {
                return node && (!ts.isSourceFile(node) || !node.isDeclarationFile) ? transformation(node) : node;
            }
            /**
             * Enables expression substitutions in the pretty printer for the provided SyntaxKind.
             */
            function enableSubstitution(kind) {
                ts.Debug.assert(state < 2 /* Completed */, "Cannot modify the transformation context after transformation has completed.");
                enabledSyntaxKindFeatures[kind] |= 1 /* Substitution */;
            }
            /**
             * Determines whether expression substitutions are enabled for the provided node.
             */
            function isSubstitutionEnabled(node) {
                return (enabledSyntaxKindFeatures[node.kind] & 1 /* Substitution */) !== 0
                    && (ts.getEmitFlags(node) & 4 /* NoSubstitution */) === 0;
            }
            /**
             * Emits a node with possible substitution.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to emit.
             * @param emitCallback The callback used to emit the node or its substitute.
             */
            function substituteNode(hint, node) {
                ts.Debug.assert(state < 3 /* Disposed */, "Cannot substitute a node after the result is disposed.");
                return node && isSubstitutionEnabled(node) && onSubstituteNode(hint, node) || node;
            }
            /**
             * Enables before/after emit notifications in the pretty printer for the provided SyntaxKind.
             */
            function enableEmitNotification(kind) {
                ts.Debug.assert(state < 2 /* Completed */, "Cannot modify the transformation context after transformation has completed.");
                enabledSyntaxKindFeatures[kind] |= 2 /* EmitNotifications */;
            }
            /**
             * Determines whether before/after emit notifications should be raised in the pretty
             * printer when it emits a node.
             */
            function isEmitNotificationEnabled(node) {
                return (enabledSyntaxKindFeatures[node.kind] & 2 /* EmitNotifications */) !== 0
                    || (ts.getEmitFlags(node) & 2 /* AdviseOnEmitNode */) !== 0;
            }
            /**
             * Emits a node with possible emit notification.
             *
             * @param hint A hint as to the intended usage of the node.
             * @param node The node to emit.
             * @param emitCallback The callback used to emit the node.
             */
            function emitNodeWithNotification(hint, node, emitCallback) {
                ts.Debug.assert(state < 3 /* Disposed */, "Cannot invoke TransformationResult callbacks after the result is disposed.");
                if (node) {
                    // TODO: Remove check and unconditionally use onEmitNode when API is breakingly changed
                    // (see https://github.com/microsoft/TypeScript/pull/36248/files/5062623f39120171b98870c71344b3242eb03d23#r369766739)
                    if (isEmitNotificationEnabled(node)) {
                        onEmitNode(hint, node, emitCallback);
                    }
                    else {
                        emitCallback(hint, node);
                    }
                }
            }
            /**
             * Records a hoisted variable declaration for the provided name within a lexical environment.
             */
            function hoistVariableDeclaration(name) {
                ts.Debug.assert(state > 0 /* Uninitialized */, "Cannot modify the lexical environment during initialization.");
                ts.Debug.assert(state < 2 /* Completed */, "Cannot modify the lexical environment after transformation has completed.");
                var decl = ts.setEmitFlags(ts.createVariableDeclaration(name), 64 /* NoNestedSourceMaps */);
                if (!lexicalEnvironmentVariableDeclarations) {
                    lexicalEnvironmentVariableDeclarations = [decl];
                }
                else {
                    lexicalEnvironmentVariableDeclarations.push(decl);
                }
            }
            /**
             * Records a hoisted function declaration within a lexical environment.
             */
            function hoistFunctionDeclaration(func) {
                ts.Debug.assert(state > 0 /* Uninitialized */, "Cannot modify the lexical environment during initialization.");
                ts.Debug.assert(state < 2 /* Completed */, "Cannot modify the lexical environment after transformation has completed.");
                if (!lexicalEnvironmentFunctionDeclarations) {
                    lexicalEnvironmentFunctionDeclarations = [func];
                }
                else {
                    lexicalEnvironmentFunctionDeclarations.push(func);
                }
            }
            /**
             * Starts a new lexical environment. Any existing hoisted variable or function declarations
             * are pushed onto a stack, and the related storage variables are reset.
             */
            function startLexicalEnvironment() {
                ts.Debug.assert(state > 0 /* Uninitialized */, "Cannot modify the lexical environment during initialization.");
                ts.Debug.assert(state < 2 /* Completed */, "Cannot modify the lexical environment after transformation has completed.");
                ts.Debug.assert(!lexicalEnvironmentSuspended, "Lexical environment is suspended.");
                // Save the current lexical environment. Rather than resizing the array we adjust the
                // stack size variable. This allows us to reuse existing array slots we've
                // already allocated between transformations to avoid allocation and GC overhead during
                // transformation.
                lexicalEnvironmentVariableDeclarationsStack[lexicalEnvironmentStackOffset] = lexicalEnvironmentVariableDeclarations;
                lexicalEnvironmentFunctionDeclarationsStack[lexicalEnvironmentStackOffset] = lexicalEnvironmentFunctionDeclarations;
                lexicalEnvironmentStackOffset++;
                lexicalEnvironmentVariableDeclarations = undefined;
                lexicalEnvironmentFunctionDeclarations = undefined;
            }
            /** Suspends the current lexical environment, usually after visiting a parameter list. */
            function suspendLexicalEnvironment() {
                ts.Debug.assert(state > 0 /* Uninitialized */, "Cannot modify the lexical environment during initialization.");
                ts.Debug.assert(state < 2 /* Completed */, "Cannot modify the lexical environment after transformation has completed.");
                ts.Debug.assert(!lexicalEnvironmentSuspended, "Lexical environment is already suspended.");
                lexicalEnvironmentSuspended = true;
            }
            /** Resumes a suspended lexical environment, usually before visiting a function body. */
            function resumeLexicalEnvironment() {
                ts.Debug.assert(state > 0 /* Uninitialized */, "Cannot modify the lexical environment during initialization.");
                ts.Debug.assert(state < 2 /* Completed */, "Cannot modify the lexical environment after transformation has completed.");
                ts.Debug.assert(lexicalEnvironmentSuspended, "Lexical environment is not suspended.");
                lexicalEnvironmentSuspended = false;
            }
            /**
             * Ends a lexical environment. The previous set of hoisted declarations are restored and
             * any hoisted declarations added in this environment are returned.
             */
            function endLexicalEnvironment() {
                ts.Debug.assert(state > 0 /* Uninitialized */, "Cannot modify the lexical environment during initialization.");
                ts.Debug.assert(state < 2 /* Completed */, "Cannot modify the lexical environment after transformation has completed.");
                ts.Debug.assert(!lexicalEnvironmentSuspended, "Lexical environment is suspended.");
                var statements;
                if (lexicalEnvironmentVariableDeclarations || lexicalEnvironmentFunctionDeclarations) {
                    if (lexicalEnvironmentFunctionDeclarations) {
                        statements = __spreadArrays(lexicalEnvironmentFunctionDeclarations);
                    }
                    if (lexicalEnvironmentVariableDeclarations) {
                        var statement = ts.createVariableStatement(
                        /*modifiers*/ undefined, ts.createVariableDeclarationList(lexicalEnvironmentVariableDeclarations));
                        ts.setEmitFlags(statement, 1048576 /* CustomPrologue */);
                        if (!statements) {
                            statements = [statement];
                        }
                        else {
                            statements.push(statement);
                        }
                    }
                }
                // Restore the previous lexical environment.
                lexicalEnvironmentStackOffset--;
                lexicalEnvironmentVariableDeclarations = lexicalEnvironmentVariableDeclarationsStack[lexicalEnvironmentStackOffset];
                lexicalEnvironmentFunctionDeclarations = lexicalEnvironmentFunctionDeclarationsStack[lexicalEnvironmentStackOffset];
                if (lexicalEnvironmentStackOffset === 0) {
                    lexicalEnvironmentVariableDeclarationsStack = [];
                    lexicalEnvironmentFunctionDeclarationsStack = [];
                }
                return statements;
            }
            function requestEmitHelper(helper) {
                ts.Debug.assert(state > 0 /* Uninitialized */, "Cannot modify the transformation context during initialization.");
                ts.Debug.assert(state < 2 /* Completed */, "Cannot modify the transformation context after transformation has completed.");
                ts.Debug.assert(!helper.scoped, "Cannot request a scoped emit helper.");
                emitHelpers = ts.append(emitHelpers, helper);
            }
            function readEmitHelpers() {
                ts.Debug.assert(state > 0 /* Uninitialized */, "Cannot modify the transformation context during initialization.");
                ts.Debug.assert(state < 2 /* Completed */, "Cannot modify the transformation context after transformation has completed.");
                var helpers = emitHelpers;
                emitHelpers = undefined;
                return helpers;
            }
            function dispose() {
                if (state < 3 /* Disposed */) {
                    // Clean up emit nodes on parse tree
                    for (var _i = 0, nodes_5 = nodes; _i < nodes_5.length; _i++) {
                        var node = nodes_5[_i];
                        ts.disposeEmitNodes(ts.getSourceFileOfNode(ts.getParseTreeNode(node)));
                    }
                    // Release references to external entries for GC purposes.
                    lexicalEnvironmentVariableDeclarations = undefined;
                    lexicalEnvironmentVariableDeclarationsStack = undefined;
                    lexicalEnvironmentFunctionDeclarations = undefined;
                    lexicalEnvironmentFunctionDeclarationsStack = undefined;
                    onSubstituteNode = undefined;
                    onEmitNode = undefined;
                    emitHelpers = undefined;
                    // Prevent further use of the transformation result.
                    state = 3 /* Disposed */;
                }
            }
        }
        ts.transformNodes = transformNodes;
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        var brackets = createBracketsMap();
        var syntheticParent = { pos: -1, end: -1 };
        /*@internal*/
        function isBuildInfoFile(file) {
            return ts.fileExtensionIs(file, ".tsbuildinfo" /* TsBuildInfo */);
        }
        ts.isBuildInfoFile = isBuildInfoFile;
        /*@internal*/
        /**
         * Iterates over the source files that are expected to have an emit output.
         *
         * @param host An EmitHost.
         * @param action The action to execute.
         * @param sourceFilesOrTargetSourceFile
         *   If an array, the full list of source files to emit.
         *   Else, calls `getSourceFilesToEmit` with the (optional) target source file to determine the list of source files to emit.
         */
        function forEachEmittedFile(host, action, sourceFilesOrTargetSourceFile, forceDtsEmit, onlyBuildInfo, includeBuildInfo) {
            if (forceDtsEmit === void 0) {
                forceDtsEmit = false;
            }
            var sourceFiles = ts.isArray(sourceFilesOrTargetSourceFile) ? sourceFilesOrTargetSourceFile : ts.getSourceFilesToEmit(host, sourceFilesOrTargetSourceFile, forceDtsEmit);
            var options = host.getCompilerOptions();
            if (options.outFile || options.out) {
                var prepends = host.getPrependNodes();
                if (sourceFiles.length || prepends.length) {
                    var bundle = ts.createBundle(sourceFiles, prepends);
                    var result = action(getOutputPathsFor(bundle, host, forceDtsEmit), bundle);
                    if (result) {
                        return result;
                    }
                }
            }
            else {
                if (!onlyBuildInfo) {
                    for (var _a = 0, sourceFiles_1 = sourceFiles; _a < sourceFiles_1.length; _a++) {
                        var sourceFile = sourceFiles_1[_a];
                        var result = action(getOutputPathsFor(sourceFile, host, forceDtsEmit), sourceFile);
                        if (result) {
                            return result;
                        }
                    }
                }
                if (includeBuildInfo) {
                    var buildInfoPath = getTsBuildInfoEmitOutputFilePath(host.getCompilerOptions());
                    if (buildInfoPath)
                        return action({ buildInfoPath: buildInfoPath }, /*sourceFileOrBundle*/ undefined);
                }
            }
        }
        ts.forEachEmittedFile = forEachEmittedFile;
        function getTsBuildInfoEmitOutputFilePath(options) {
            var configFile = options.configFilePath;
            if (!ts.isIncrementalCompilation(options))
                return undefined;
            if (options.tsBuildInfoFile)
                return options.tsBuildInfoFile;
            var outPath = options.outFile || options.out;
            var buildInfoExtensionLess;
            if (outPath) {
                buildInfoExtensionLess = ts.removeFileExtension(outPath);
            }
            else {
                if (!configFile)
                    return undefined;
                var configFileExtensionLess = ts.removeFileExtension(configFile);
                buildInfoExtensionLess = options.outDir ?
                    options.rootDir ?
                        ts.resolvePath(options.outDir, ts.getRelativePathFromDirectory(options.rootDir, configFileExtensionLess, /*ignoreCase*/ true)) :
                        ts.combinePaths(options.outDir, ts.getBaseFileName(configFileExtensionLess)) :
                    configFileExtensionLess;
            }
            return buildInfoExtensionLess + ".tsbuildinfo" /* TsBuildInfo */;
        }
        ts.getTsBuildInfoEmitOutputFilePath = getTsBuildInfoEmitOutputFilePath;
        /*@internal*/
        function getOutputPathsForBundle(options, forceDtsPaths) {
            var outPath = options.outFile || options.out;
            var jsFilePath = options.emitDeclarationOnly ? undefined : outPath;
            var sourceMapFilePath = jsFilePath && getSourceMapFilePath(jsFilePath, options);
            var declarationFilePath = (forceDtsPaths || ts.getEmitDeclarations(options)) ? ts.removeFileExtension(outPath) + ".d.ts" /* Dts */ : undefined;
            var declarationMapPath = declarationFilePath && ts.getAreDeclarationMapsEnabled(options) ? declarationFilePath + ".map" : undefined;
            var buildInfoPath = getTsBuildInfoEmitOutputFilePath(options);
            return { jsFilePath: jsFilePath, sourceMapFilePath: sourceMapFilePath, declarationFilePath: declarationFilePath, declarationMapPath: declarationMapPath, buildInfoPath: buildInfoPath };
        }
        ts.getOutputPathsForBundle = getOutputPathsForBundle;
        /*@internal*/
        function getOutputPathsFor(sourceFile, host, forceDtsPaths) {
            var options = host.getCompilerOptions();
            if (sourceFile.kind === 291 /* Bundle */) {
                return getOutputPathsForBundle(options, forceDtsPaths);
            }
            else {
                var ownOutputFilePath = ts.getOwnEmitOutputFilePath(sourceFile.fileName, host, getOutputExtension(sourceFile, options));
                var isJsonFile = ts.isJsonSourceFile(sourceFile);
                // If json file emits to the same location skip writing it, if emitDeclarationOnly skip writing it
                var isJsonEmittedToSameLocation = isJsonFile &&
                    ts.comparePaths(sourceFile.fileName, ownOutputFilePath, host.getCurrentDirectory(), !host.useCaseSensitiveFileNames()) === 0 /* EqualTo */;
                var jsFilePath = options.emitDeclarationOnly || isJsonEmittedToSameLocation ? undefined : ownOutputFilePath;
                var sourceMapFilePath = !jsFilePath || ts.isJsonSourceFile(sourceFile) ? undefined : getSourceMapFilePath(jsFilePath, options);
                var declarationFilePath = (forceDtsPaths || (ts.getEmitDeclarations(options) && !isJsonFile)) ? ts.getDeclarationEmitOutputFilePath(sourceFile.fileName, host) : undefined;
                var declarationMapPath = declarationFilePath && ts.getAreDeclarationMapsEnabled(options) ? declarationFilePath + ".map" : undefined;
                return { jsFilePath: jsFilePath, sourceMapFilePath: sourceMapFilePath, declarationFilePath: declarationFilePath, declarationMapPath: declarationMapPath, buildInfoPath: undefined };
            }
        }
        ts.getOutputPathsFor = getOutputPathsFor;
        function getSourceMapFilePath(jsFilePath, options) {
            return (options.sourceMap && !options.inlineSourceMap) ? jsFilePath + ".map" : undefined;
        }
        // JavaScript files are always LanguageVariant.JSX, as JSX syntax is allowed in .js files also.
        // So for JavaScript files, '.jsx' is only emitted if the input was '.jsx', and JsxEmit.Preserve.
        // For TypeScript, the only time to emit with a '.jsx' extension, is on JSX input, and JsxEmit.Preserve
        /* @internal */
        function getOutputExtension(sourceFile, options) {
            if (ts.isJsonSourceFile(sourceFile)) {
                return ".json" /* Json */;
            }
            if (options.jsx === 1 /* Preserve */) {
                if (ts.isSourceFileJS(sourceFile)) {
                    if (ts.fileExtensionIs(sourceFile.fileName, ".jsx" /* Jsx */)) {
                        return ".jsx" /* Jsx */;
                    }
                }
                else if (sourceFile.languageVariant === 1 /* JSX */) {
                    // TypeScript source file preserving JSX syntax
                    return ".jsx" /* Jsx */;
                }
            }
            return ".js" /* Js */;
        }
        ts.getOutputExtension = getOutputExtension;
        function rootDirOfOptions(configFile) {
            return configFile.options.rootDir || ts.getDirectoryPath(ts.Debug.assertDefined(configFile.options.configFilePath));
        }
        function getOutputPathWithoutChangingExt(inputFileName, configFile, ignoreCase, outputDir) {
            return outputDir ?
                ts.resolvePath(outputDir, ts.getRelativePathFromDirectory(rootDirOfOptions(configFile), inputFileName, ignoreCase)) :
                inputFileName;
        }
        /* @internal */
        function getOutputDeclarationFileName(inputFileName, configFile, ignoreCase) {
            ts.Debug.assert(!ts.fileExtensionIs(inputFileName, ".d.ts" /* Dts */) && !ts.fileExtensionIs(inputFileName, ".json" /* Json */));
            return ts.changeExtension(getOutputPathWithoutChangingExt(inputFileName, configFile, ignoreCase, configFile.options.declarationDir || configFile.options.outDir), ".d.ts" /* Dts */);
        }
        ts.getOutputDeclarationFileName = getOutputDeclarationFileName;
        function getOutputJSFileName(inputFileName, configFile, ignoreCase) {
            if (configFile.options.emitDeclarationOnly)
                return undefined;
            var isJsonFile = ts.fileExtensionIs(inputFileName, ".json" /* Json */);
            var outputFileName = ts.changeExtension(getOutputPathWithoutChangingExt(inputFileName, configFile, ignoreCase, configFile.options.outDir), isJsonFile ?
                ".json" /* Json */ :
                ts.fileExtensionIs(inputFileName, ".tsx" /* Tsx */) && configFile.options.jsx === 1 /* Preserve */ ?
                    ".jsx" /* Jsx */ :
                    ".js" /* Js */);
            return !isJsonFile || ts.comparePaths(inputFileName, outputFileName, ts.Debug.assertDefined(configFile.options.configFilePath), ignoreCase) !== 0 /* EqualTo */ ?
                outputFileName :
                undefined;
        }
        function createAddOutput() {
            var outputs;
            return { addOutput: addOutput, getOutputs: getOutputs };
            function addOutput(path) {
                if (path) {
                    (outputs || (outputs = [])).push(path);
                }
            }
            function getOutputs() {
                return outputs || ts.emptyArray;
            }
        }
        function getSingleOutputFileNames(configFile, addOutput) {
            var _a = getOutputPathsForBundle(configFile.options, /*forceDtsPaths*/ false), jsFilePath = _a.jsFilePath, sourceMapFilePath = _a.sourceMapFilePath, declarationFilePath = _a.declarationFilePath, declarationMapPath = _a.declarationMapPath, buildInfoPath = _a.buildInfoPath;
            addOutput(jsFilePath);
            addOutput(sourceMapFilePath);
            addOutput(declarationFilePath);
            addOutput(declarationMapPath);
            addOutput(buildInfoPath);
        }
        function getOwnOutputFileNames(configFile, inputFileName, ignoreCase, addOutput) {
            if (ts.fileExtensionIs(inputFileName, ".d.ts" /* Dts */))
                return;
            var js = getOutputJSFileName(inputFileName, configFile, ignoreCase);
            addOutput(js);
            if (ts.fileExtensionIs(inputFileName, ".json" /* Json */))
                return;
            if (js && configFile.options.sourceMap) {
                addOutput(js + ".map");
            }
            if (ts.getEmitDeclarations(configFile.options)) {
                var dts = getOutputDeclarationFileName(inputFileName, configFile, ignoreCase);
                addOutput(dts);
                if (configFile.options.declarationMap) {
                    addOutput(dts + ".map");
                }
            }
        }
        /*@internal*/
        function getAllProjectOutputs(configFile, ignoreCase) {
            var _a = createAddOutput(), addOutput = _a.addOutput, getOutputs = _a.getOutputs;
            if (configFile.options.outFile || configFile.options.out) {
                getSingleOutputFileNames(configFile, addOutput);
            }
            else {
                for (var _b = 0, _c = configFile.fileNames; _b < _c.length; _b++) {
                    var inputFileName = _c[_b];
                    getOwnOutputFileNames(configFile, inputFileName, ignoreCase, addOutput);
                }
                addOutput(getTsBuildInfoEmitOutputFilePath(configFile.options));
            }
            return getOutputs();
        }
        ts.getAllProjectOutputs = getAllProjectOutputs;
        function getOutputFileNames(commandLine, inputFileName, ignoreCase) {
            inputFileName = ts.normalizePath(inputFileName);
            ts.Debug.assert(ts.contains(commandLine.fileNames, inputFileName), "Expected fileName to be present in command line");
            var _a = createAddOutput(), addOutput = _a.addOutput, getOutputs = _a.getOutputs;
            if (commandLine.options.outFile || commandLine.options.out) {
                getSingleOutputFileNames(commandLine, addOutput);
            }
            else {
                getOwnOutputFileNames(commandLine, inputFileName, ignoreCase, addOutput);
            }
            return getOutputs();
        }
        ts.getOutputFileNames = getOutputFileNames;
        /*@internal*/
        function getFirstProjectOutput(configFile, ignoreCase) {
            if (configFile.options.outFile || configFile.options.out) {
                var jsFilePath = getOutputPathsForBundle(configFile.options, /*forceDtsPaths*/ false).jsFilePath;
                return ts.Debug.assertDefined(jsFilePath, "project " + configFile.options.configFilePath + " expected to have at least one output");
            }
            for (var _a = 0, _b = configFile.fileNames; _a < _b.length; _a++) {
                var inputFileName = _b[_a];
                if (ts.fileExtensionIs(inputFileName, ".d.ts" /* Dts */))
                    continue;
                var jsFilePath = getOutputJSFileName(inputFileName, configFile, ignoreCase);
                if (jsFilePath)
                    return jsFilePath;
                if (ts.fileExtensionIs(inputFileName, ".json" /* Json */))
                    continue;
                if (ts.getEmitDeclarations(configFile.options)) {
                    return getOutputDeclarationFileName(inputFileName, configFile, ignoreCase);
                }
            }
            var buildInfoPath = getTsBuildInfoEmitOutputFilePath(configFile.options);
            if (buildInfoPath)
                return buildInfoPath;
            return ts.Debug.fail("project " + configFile.options.configFilePath + " expected to have at least one output");
        }
        ts.getFirstProjectOutput = getFirstProjectOutput;
        /*@internal*/
        // targetSourceFile is when users only want one file in entire project to be emitted. This is used in compileOnSave feature
        function emitFiles(resolver, host, targetSourceFile, _a, emitOnlyDtsFiles, onlyBuildInfo, forceDtsEmit) {
            var scriptTransformers = _a.scriptTransformers, declarationTransformers = _a.declarationTransformers;
            var compilerOptions = host.getCompilerOptions();
            var sourceMapDataList = (compilerOptions.sourceMap || compilerOptions.inlineSourceMap || ts.getAreDeclarationMapsEnabled(compilerOptions)) ? [] : undefined;
            var emittedFilesList = compilerOptions.listEmittedFiles ? [] : undefined;
            var emitterDiagnostics = ts.createDiagnosticCollection();
            var newLine = ts.getNewLineCharacter(compilerOptions, function () { return host.getNewLine(); });
            var writer = ts.createTextWriter(newLine);
            var _b = ts.performance.createTimer("printTime", "beforePrint", "afterPrint"), enter = _b.enter, exit = _b.exit;
            var bundleBuildInfo;
            var emitSkipped = false;
            var exportedModulesFromDeclarationEmit;
            // Emit each output file
            enter();
            forEachEmittedFile(host, emitSourceFileOrBundle, ts.getSourceFilesToEmit(host, targetSourceFile, forceDtsEmit), forceDtsEmit, onlyBuildInfo, !targetSourceFile);
            exit();
            return {
                emitSkipped: emitSkipped,
                diagnostics: emitterDiagnostics.getDiagnostics(),
                emittedFiles: emittedFilesList,
                sourceMaps: sourceMapDataList,
                exportedModulesFromDeclarationEmit: exportedModulesFromDeclarationEmit
            };
            function emitSourceFileOrBundle(_a, sourceFileOrBundle) {
                var jsFilePath = _a.jsFilePath, sourceMapFilePath = _a.sourceMapFilePath, declarationFilePath = _a.declarationFilePath, declarationMapPath = _a.declarationMapPath, buildInfoPath = _a.buildInfoPath;
                var buildInfoDirectory;
                if (buildInfoPath && sourceFileOrBundle && ts.isBundle(sourceFileOrBundle)) {
                    buildInfoDirectory = ts.getDirectoryPath(ts.getNormalizedAbsolutePath(buildInfoPath, host.getCurrentDirectory()));
                    bundleBuildInfo = {
                        commonSourceDirectory: relativeToBuildInfo(host.getCommonSourceDirectory()),
                        sourceFiles: sourceFileOrBundle.sourceFiles.map(function (file) { return relativeToBuildInfo(ts.getNormalizedAbsolutePath(file.fileName, host.getCurrentDirectory())); })
                    };
                }
                emitJsFileOrBundle(sourceFileOrBundle, jsFilePath, sourceMapFilePath, relativeToBuildInfo);
                emitDeclarationFileOrBundle(sourceFileOrBundle, declarationFilePath, declarationMapPath, relativeToBuildInfo);
                emitBuildInfo(bundleBuildInfo, buildInfoPath);
                if (!emitSkipped && emittedFilesList) {
                    if (!emitOnlyDtsFiles) {
                        if (jsFilePath) {
                            emittedFilesList.push(jsFilePath);
                        }
                        if (sourceMapFilePath) {
                            emittedFilesList.push(sourceMapFilePath);
                        }
                        if (buildInfoPath) {
                            emittedFilesList.push(buildInfoPath);
                        }
                    }
                    if (declarationFilePath) {
                        emittedFilesList.push(declarationFilePath);
                    }
                    if (declarationMapPath) {
                        emittedFilesList.push(declarationMapPath);
                    }
                }
                function relativeToBuildInfo(path) {
                    return ts.ensurePathIsNonModuleName(ts.getRelativePathFromDirectory(buildInfoDirectory, path, host.getCanonicalFileName));
                }
            }
            function emitBuildInfo(bundle, buildInfoPath) {
                // Write build information if applicable
                if (!buildInfoPath || targetSourceFile || emitSkipped)
                    return;
                var program = host.getProgramBuildInfo();
                if (host.isEmitBlocked(buildInfoPath) || compilerOptions.noEmit) {
                    emitSkipped = true;
                    return;
                }
                var version = ts.version; // Extracted into a const so the form is stable between namespace and module
                ts.writeFile(host, emitterDiagnostics, buildInfoPath, getBuildInfoText({ bundle: bundle, program: program, version: version }), /*writeByteOrderMark*/ false);
            }
            function emitJsFileOrBundle(sourceFileOrBundle, jsFilePath, sourceMapFilePath, relativeToBuildInfo) {
                if (!sourceFileOrBundle || emitOnlyDtsFiles || !jsFilePath) {
                    return;
                }
                // Make sure not to write js file and source map file if any of them cannot be written
                if ((jsFilePath && host.isEmitBlocked(jsFilePath)) || compilerOptions.noEmit) {
                    emitSkipped = true;
                    return;
                }
                // Transform the source files
                var transform = ts.transformNodes(resolver, host, compilerOptions, [sourceFileOrBundle], scriptTransformers, /*allowDtsFiles*/ false);
                var printerOptions = {
                    removeComments: compilerOptions.removeComments,
                    newLine: compilerOptions.newLine,
                    noEmitHelpers: compilerOptions.noEmitHelpers,
                    module: compilerOptions.module,
                    target: compilerOptions.target,
                    sourceMap: compilerOptions.sourceMap,
                    inlineSourceMap: compilerOptions.inlineSourceMap,
                    inlineSources: compilerOptions.inlineSources,
                    extendedDiagnostics: compilerOptions.extendedDiagnostics,
                    writeBundleFileInfo: !!bundleBuildInfo,
                    relativeToBuildInfo: relativeToBuildInfo
                };
                // Create a printer to print the nodes
                var printer = createPrinter(printerOptions, {
                    // resolver hooks
                    hasGlobalName: resolver.hasGlobalName,
                    // transform hooks
                    onEmitNode: transform.emitNodeWithNotification,
                    isEmitNotificationEnabled: transform.isEmitNotificationEnabled,
                    substituteNode: transform.substituteNode,
                });
                ts.Debug.assert(transform.transformed.length === 1, "Should only see one output from the transform");
                printSourceFileOrBundle(jsFilePath, sourceMapFilePath, transform.transformed[0], printer, compilerOptions);
                // Clean up emit nodes on parse tree
                transform.dispose();
                if (bundleBuildInfo)
                    bundleBuildInfo.js = printer.bundleFileInfo;
            }
            function emitDeclarationFileOrBundle(sourceFileOrBundle, declarationFilePath, declarationMapPath, relativeToBuildInfo) {
                if (!sourceFileOrBundle)
                    return;
                if (!declarationFilePath) {
                    if (emitOnlyDtsFiles || compilerOptions.emitDeclarationOnly)
                        emitSkipped = true;
                    return;
                }
                var sourceFiles = ts.isSourceFile(sourceFileOrBundle) ? [sourceFileOrBundle] : sourceFileOrBundle.sourceFiles;
                var filesForEmit = forceDtsEmit ? sourceFiles : ts.filter(sourceFiles, ts.isSourceFileNotJson);
                // Setup and perform the transformation to retrieve declarations from the input files
                var inputListOrBundle = (compilerOptions.outFile || compilerOptions.out) ? [ts.createBundle(filesForEmit, !ts.isSourceFile(sourceFileOrBundle) ? sourceFileOrBundle.prepends : undefined)] : filesForEmit;
                if (emitOnlyDtsFiles && !ts.getEmitDeclarations(compilerOptions)) {
                    // Checker wont collect the linked aliases since thats only done when declaration is enabled.
                    // Do that here when emitting only dts files
                    filesForEmit.forEach(collectLinkedAliases);
                }
                var declarationTransform = ts.transformNodes(resolver, host, compilerOptions, inputListOrBundle, declarationTransformers, /*allowDtsFiles*/ false);
                if (ts.length(declarationTransform.diagnostics)) {
                    for (var _a = 0, _b = declarationTransform.diagnostics; _a < _b.length; _a++) {
                        var diagnostic = _b[_a];
                        emitterDiagnostics.add(diagnostic);
                    }
                }
                var printerOptions = {
                    removeComments: compilerOptions.removeComments,
                    newLine: compilerOptions.newLine,
                    noEmitHelpers: true,
                    module: compilerOptions.module,
                    target: compilerOptions.target,
                    sourceMap: compilerOptions.sourceMap,
                    inlineSourceMap: compilerOptions.inlineSourceMap,
                    extendedDiagnostics: compilerOptions.extendedDiagnostics,
                    onlyPrintJsDocStyle: true,
                    writeBundleFileInfo: !!bundleBuildInfo,
                    recordInternalSection: !!bundleBuildInfo,
                    relativeToBuildInfo: relativeToBuildInfo
                };
                var declarationPrinter = createPrinter(printerOptions, {
                    // resolver hooks
                    hasGlobalName: resolver.hasGlobalName,
                    // transform hooks
                    onEmitNode: declarationTransform.emitNodeWithNotification,
                    isEmitNotificationEnabled: declarationTransform.isEmitNotificationEnabled,
                    substituteNode: declarationTransform.substituteNode,
                });
                var declBlocked = (!!declarationTransform.diagnostics && !!declarationTransform.diagnostics.length) || !!host.isEmitBlocked(declarationFilePath) || !!compilerOptions.noEmit;
                emitSkipped = emitSkipped || declBlocked;
                if (!declBlocked || forceDtsEmit) {
                    ts.Debug.assert(declarationTransform.transformed.length === 1, "Should only see one output from the decl transform");
                    printSourceFileOrBundle(declarationFilePath, declarationMapPath, declarationTransform.transformed[0], declarationPrinter, {
                        sourceMap: compilerOptions.declarationMap,
                        sourceRoot: compilerOptions.sourceRoot,
                        mapRoot: compilerOptions.mapRoot,
                        extendedDiagnostics: compilerOptions.extendedDiagnostics,
                    });
                    if (forceDtsEmit && declarationTransform.transformed[0].kind === 290 /* SourceFile */) {
                        var sourceFile = declarationTransform.transformed[0];
                        exportedModulesFromDeclarationEmit = sourceFile.exportedModulesFromDeclarationEmit;
                    }
                }
                declarationTransform.dispose();
                if (bundleBuildInfo)
                    bundleBuildInfo.dts = declarationPrinter.bundleFileInfo;
            }
            function collectLinkedAliases(node) {
                if (ts.isExportAssignment(node)) {
                    if (node.expression.kind === 75 /* Identifier */) {
                        resolver.collectLinkedAliases(node.expression, /*setVisibility*/ true);
                    }
                    return;
                }
                else if (ts.isExportSpecifier(node)) {
                    resolver.collectLinkedAliases(node.propertyName || node.name, /*setVisibility*/ true);
                    return;
                }
                ts.forEachChild(node, collectLinkedAliases);
            }
            function printSourceFileOrBundle(jsFilePath, sourceMapFilePath, sourceFileOrBundle, printer, mapOptions) {
                var bundle = sourceFileOrBundle.kind === 291 /* Bundle */ ? sourceFileOrBundle : undefined;
                var sourceFile = sourceFileOrBundle.kind === 290 /* SourceFile */ ? sourceFileOrBundle : undefined;
                var sourceFiles = bundle ? bundle.sourceFiles : [sourceFile];
                var sourceMapGenerator;
                if (shouldEmitSourceMaps(mapOptions, sourceFileOrBundle)) {
                    sourceMapGenerator = ts.createSourceMapGenerator(host, ts.getBaseFileName(ts.normalizeSlashes(jsFilePath)), getSourceRoot(mapOptions), getSourceMapDirectory(mapOptions, jsFilePath, sourceFile), mapOptions);
                }
                if (bundle) {
                    printer.writeBundle(bundle, writer, sourceMapGenerator);
                }
                else {
                    printer.writeFile(sourceFile, writer, sourceMapGenerator);
                }
                if (sourceMapGenerator) {
                    if (sourceMapDataList) {
                        sourceMapDataList.push({
                            inputSourceFileNames: sourceMapGenerator.getSources(),
                            sourceMap: sourceMapGenerator.toJSON()
                        });
                    }
                    var sourceMappingURL = getSourceMappingURL(mapOptions, sourceMapGenerator, jsFilePath, sourceMapFilePath, sourceFile);
                    if (sourceMappingURL) {
                        if (!writer.isAtStartOfLine())
                            writer.rawWrite(newLine);
                        writer.writeComment("//# " + "sourceMappingURL" + "=" + sourceMappingURL); // Tools can sometimes see this line as a source mapping url comment
                    }
                    // Write the source map
                    if (sourceMapFilePath) {
                        var sourceMap = sourceMapGenerator.toString();
                        ts.writeFile(host, emitterDiagnostics, sourceMapFilePath, sourceMap, /*writeByteOrderMark*/ false, sourceFiles);
                    }
                }
                else {
                    writer.writeLine();
                }
                // Write the output file
                ts.writeFile(host, emitterDiagnostics, jsFilePath, writer.getText(), !!compilerOptions.emitBOM, sourceFiles);
                // Reset state
                writer.clear();
            }
            function shouldEmitSourceMaps(mapOptions, sourceFileOrBundle) {
                return (mapOptions.sourceMap || mapOptions.inlineSourceMap)
                    && (sourceFileOrBundle.kind !== 290 /* SourceFile */ || !ts.fileExtensionIs(sourceFileOrBundle.fileName, ".json" /* Json */));
            }
            function getSourceRoot(mapOptions) {
                // Normalize source root and make sure it has trailing "/" so that it can be used to combine paths with the
                // relative paths of the sources list in the sourcemap
                var sourceRoot = ts.normalizeSlashes(mapOptions.sourceRoot || "");
                return sourceRoot ? ts.ensureTrailingDirectorySeparator(sourceRoot) : sourceRoot;
            }
            function getSourceMapDirectory(mapOptions, filePath, sourceFile) {
                if (mapOptions.sourceRoot)
                    return host.getCommonSourceDirectory();
                if (mapOptions.mapRoot) {
                    var sourceMapDir = ts.normalizeSlashes(mapOptions.mapRoot);
                    if (sourceFile) {
                        // For modules or multiple emit files the mapRoot will have directory structure like the sources
                        // So if src\a.ts and src\lib\b.ts are compiled together user would be moving the maps into mapRoot\a.js.map and mapRoot\lib\b.js.map
                        sourceMapDir = ts.getDirectoryPath(ts.getSourceFilePathInNewDir(sourceFile.fileName, host, sourceMapDir));
                    }
                    if (ts.getRootLength(sourceMapDir) === 0) {
                        // The relative paths are relative to the common directory
                        sourceMapDir = ts.combinePaths(host.getCommonSourceDirectory(), sourceMapDir);
                    }
                    return sourceMapDir;
                }
                return ts.getDirectoryPath(ts.normalizePath(filePath));
            }
            function getSourceMappingURL(mapOptions, sourceMapGenerator, filePath, sourceMapFilePath, sourceFile) {
                if (mapOptions.inlineSourceMap) {
                    // Encode the sourceMap into the sourceMap url
                    var sourceMapText = sourceMapGenerator.toString();
                    var base64SourceMapText = ts.base64encode(ts.sys, sourceMapText);
                    return "data:application/json;base64," + base64SourceMapText;
                }
                var sourceMapFile = ts.getBaseFileName(ts.normalizeSlashes(ts.Debug.assertDefined(sourceMapFilePath)));
                if (mapOptions.mapRoot) {
                    var sourceMapDir = ts.normalizeSlashes(mapOptions.mapRoot);
                    if (sourceFile) {
                        // For modules or multiple emit files the mapRoot will have directory structure like the sources
                        // So if src\a.ts and src\lib\b.ts are compiled together user would be moving the maps into mapRoot\a.js.map and mapRoot\lib\b.js.map
                        sourceMapDir = ts.getDirectoryPath(ts.getSourceFilePathInNewDir(sourceFile.fileName, host, sourceMapDir));
                    }
                    if (ts.getRootLength(sourceMapDir) === 0) {
                        // The relative paths are relative to the common directory
                        sourceMapDir = ts.combinePaths(host.getCommonSourceDirectory(), sourceMapDir);
                        return ts.getRelativePathToDirectoryOrUrl(ts.getDirectoryPath(ts.normalizePath(filePath)), // get the relative sourceMapDir path based on jsFilePath
                        ts.combinePaths(sourceMapDir, sourceMapFile), // this is where user expects to see sourceMap
                        host.getCurrentDirectory(), host.getCanonicalFileName, 
                        /*isAbsolutePathAnUrl*/ true);
                    }
                    else {
                        return ts.combinePaths(sourceMapDir, sourceMapFile);
                    }
                }
                return sourceMapFile;
            }
        }
        ts.emitFiles = emitFiles;
        /*@internal*/
        function getBuildInfoText(buildInfo) {
            return JSON.stringify(buildInfo, undefined, 2);
        }
        ts.getBuildInfoText = getBuildInfoText;
        /*@internal*/
        function getBuildInfo(buildInfoText) {
            return JSON.parse(buildInfoText);
        }
        ts.getBuildInfo = getBuildInfo;
        /*@internal*/
        ts.notImplementedResolver = {
            hasGlobalName: ts.notImplemented,
            getReferencedExportContainer: ts.notImplemented,
            getReferencedImportDeclaration: ts.notImplemented,
            getReferencedDeclarationWithCollidingName: ts.notImplemented,
            isDeclarationWithCollidingName: ts.notImplemented,
            isValueAliasDeclaration: ts.notImplemented,
            isReferencedAliasDeclaration: ts.notImplemented,
            isTopLevelValueImportEqualsWithEntityName: ts.notImplemented,
            getNodeCheckFlags: ts.notImplemented,
            isDeclarationVisible: ts.notImplemented,
            isLateBound: function (_node) { return false; },
            collectLinkedAliases: ts.notImplemented,
            isImplementationOfOverload: ts.notImplemented,
            isRequiredInitializedParameter: ts.notImplemented,
            isOptionalUninitializedParameterProperty: ts.notImplemented,
            isExpandoFunctionDeclaration: ts.notImplemented,
            getPropertiesOfContainerFunction: ts.notImplemented,
            createTypeOfDeclaration: ts.notImplemented,
            createReturnTypeOfSignatureDeclaration: ts.notImplemented,
            createTypeOfExpression: ts.notImplemented,
            createLiteralConstValue: ts.notImplemented,
            isSymbolAccessible: ts.notImplemented,
            isEntityNameVisible: ts.notImplemented,
            // Returns the constant value this property access resolves to: notImplemented, or 'undefined' for a non-constant
            getConstantValue: ts.notImplemented,
            getReferencedValueDeclaration: ts.notImplemented,
            getTypeReferenceSerializationKind: ts.notImplemented,
            isOptionalParameter: ts.notImplemented,
            moduleExportsSomeValue: ts.notImplemented,
            isArgumentsLocalBinding: ts.notImplemented,
            getExternalModuleFileFromDeclaration: ts.notImplemented,
            getTypeReferenceDirectivesForEntityName: ts.notImplemented,
            getTypeReferenceDirectivesForSymbol: ts.notImplemented,
            isLiteralConstDeclaration: ts.notImplemented,
            getJsxFactoryEntity: ts.notImplemented,
            getAllAccessorDeclarations: ts.notImplemented,
            getSymbolOfExternalModuleSpecifier: ts.notImplemented,
            isBindingCapturedByNode: ts.notImplemented,
            getDeclarationStatementsForSourceFile: ts.notImplemented,
        };
        function createSourceFilesFromBundleBuildInfo(bundle, buildInfoDirectory, host) {
            var sourceFiles = bundle.sourceFiles.map(function (fileName) {
                var sourceFile = ts.createNode(290 /* SourceFile */, 0, 0);
                sourceFile.fileName = ts.getRelativePathFromDirectory(host.getCurrentDirectory(), ts.getNormalizedAbsolutePath(fileName, buildInfoDirectory), !host.useCaseSensitiveFileNames());
                sourceFile.text = "";
                sourceFile.statements = ts.createNodeArray();
                return sourceFile;
            });
            var jsBundle = ts.Debug.assertDefined(bundle.js);
            ts.forEach(jsBundle.sources && jsBundle.sources.prologues, function (prologueInfo) {
                var sourceFile = sourceFiles[prologueInfo.file];
                sourceFile.text = prologueInfo.text;
                sourceFile.end = prologueInfo.text.length;
                sourceFile.statements = ts.createNodeArray(prologueInfo.directives.map(function (directive) {
                    var statement = ts.createNode(226 /* ExpressionStatement */, directive.pos, directive.end);
                    statement.expression = ts.createNode(10 /* StringLiteral */, directive.expression.pos, directive.expression.end);
                    statement.expression.text = directive.expression.text;
                    return statement;
                }));
            });
            return sourceFiles;
        }
        /*@internal*/
        function emitUsingBuildInfo(config, host, getCommandLine, customTransformers) {
            var _a = getOutputPathsForBundle(config.options, /*forceDtsPaths*/ false), buildInfoPath = _a.buildInfoPath, jsFilePath = _a.jsFilePath, sourceMapFilePath = _a.sourceMapFilePath, declarationFilePath = _a.declarationFilePath, declarationMapPath = _a.declarationMapPath;
            var buildInfoText = host.readFile(ts.Debug.assertDefined(buildInfoPath));
            if (!buildInfoText)
                return buildInfoPath;
            var jsFileText = host.readFile(ts.Debug.assertDefined(jsFilePath));
            if (!jsFileText)
                return jsFilePath;
            var sourceMapText = sourceMapFilePath && host.readFile(sourceMapFilePath);
            // error if no source map or for now if inline sourcemap
            if ((sourceMapFilePath && !sourceMapText) || config.options.inlineSourceMap)
                return sourceMapFilePath || "inline sourcemap decoding";
            // read declaration text
            var declarationText = declarationFilePath && host.readFile(declarationFilePath);
            if (declarationFilePath && !declarationText)
                return declarationFilePath;
            var declarationMapText = declarationMapPath && host.readFile(declarationMapPath);
            // error if no source map or for now if inline sourcemap
            if ((declarationMapPath && !declarationMapText) || config.options.inlineSourceMap)
                return declarationMapPath || "inline sourcemap decoding";
            var buildInfo = getBuildInfo(buildInfoText);
            if (!buildInfo.bundle || !buildInfo.bundle.js || (declarationText && !buildInfo.bundle.dts))
                return buildInfoPath;
            var buildInfoDirectory = ts.getDirectoryPath(ts.getNormalizedAbsolutePath(buildInfoPath, host.getCurrentDirectory()));
            var ownPrependInput = ts.createInputFiles(jsFileText, declarationText, sourceMapFilePath, sourceMapText, declarationMapPath, declarationMapText, jsFilePath, declarationFilePath, buildInfoPath, buildInfo, 
            /*onlyOwnText*/ true);
            var outputFiles = [];
            var prependNodes = ts.createPrependNodes(config.projectReferences, getCommandLine, function (f) { return host.readFile(f); });
            var sourceFilesForJsEmit = createSourceFilesFromBundleBuildInfo(buildInfo.bundle, buildInfoDirectory, host);
            var emitHost = {
                getPrependNodes: ts.memoize(function () { return __spreadArrays(prependNodes, [ownPrependInput]); }),
                getCanonicalFileName: host.getCanonicalFileName,
                getCommonSourceDirectory: function () { return ts.getNormalizedAbsolutePath(buildInfo.bundle.commonSourceDirectory, buildInfoDirectory); },
                getCompilerOptions: function () { return config.options; },
                getCurrentDirectory: function () { return host.getCurrentDirectory(); },
                getNewLine: function () { return host.getNewLine(); },
                getSourceFile: ts.returnUndefined,
                getSourceFileByPath: ts.returnUndefined,
                getSourceFiles: function () { return sourceFilesForJsEmit; },
                getLibFileFromReference: ts.notImplemented,
                isSourceFileFromExternalLibrary: ts.returnFalse,
                getResolvedProjectReferenceToRedirect: ts.returnUndefined,
                isSourceOfProjectReferenceRedirect: ts.returnFalse,
                writeFile: function (name, text, writeByteOrderMark) {
                    switch (name) {
                        case jsFilePath:
                            if (jsFileText === text)
                                return;
                            break;
                        case sourceMapFilePath:
                            if (sourceMapText === text)
                                return;
                            break;
                        case buildInfoPath:
                            var newBuildInfo = getBuildInfo(text);
                            newBuildInfo.program = buildInfo.program;
                            // Update sourceFileInfo
                            var _a = buildInfo.bundle, js = _a.js, dts = _a.dts, sourceFiles = _a.sourceFiles;
                            newBuildInfo.bundle.js.sources = js.sources;
                            if (dts) {
                                newBuildInfo.bundle.dts.sources = dts.sources;
                            }
                            newBuildInfo.bundle.sourceFiles = sourceFiles;
                            outputFiles.push({ name: name, text: getBuildInfoText(newBuildInfo), writeByteOrderMark: writeByteOrderMark });
                            return;
                        case declarationFilePath:
                            if (declarationText === text)
                                return;
                            break;
                        case declarationMapPath:
                            if (declarationMapText === text)
                                return;
                            break;
                        default:
                            ts.Debug.fail("Unexpected path: " + name);
                    }
                    outputFiles.push({ name: name, text: text, writeByteOrderMark: writeByteOrderMark });
                },
                isEmitBlocked: ts.returnFalse,
                readFile: function (f) { return host.readFile(f); },
                fileExists: function (f) { return host.fileExists(f); },
                directoryExists: host.directoryExists && (function (f) { return host.directoryExists(f); }),
                useCaseSensitiveFileNames: function () { return host.useCaseSensitiveFileNames(); },
                getProgramBuildInfo: ts.returnUndefined,
                getSourceFileFromReference: ts.returnUndefined,
                redirectTargetsMap: ts.createMultiMap()
            };
            emitFiles(ts.notImplementedResolver, emitHost, 
            /*targetSourceFile*/ undefined, ts.getTransformers(config.options, customTransformers));
            return outputFiles;
        }
        ts.emitUsingBuildInfo = emitUsingBuildInfo;
        var PipelinePhase;
        (function (PipelinePhase) {
            PipelinePhase[PipelinePhase["Notification"] = 0] = "Notification";
            PipelinePhase[PipelinePhase["Substitution"] = 1] = "Substitution";
            PipelinePhase[PipelinePhase["Comments"] = 2] = "Comments";
            PipelinePhase[PipelinePhase["SourceMaps"] = 3] = "SourceMaps";
            PipelinePhase[PipelinePhase["Emit"] = 4] = "Emit";
        })(PipelinePhase || (PipelinePhase = {}));
        function createPrinter(printerOptions, handlers) {
            if (printerOptions === void 0) {
                printerOptions = {};
            }
            if (handlers === void 0) {
                handlers = {};
            }
            var hasGlobalName = handlers.hasGlobalName, _a = handlers.onEmitNode, onEmitNode = _a === void 0 ? ts.noEmitNotification : _a, isEmitNotificationEnabled = handlers.isEmitNotificationEnabled, _b = handlers.substituteNode, substituteNode = _b === void 0 ? ts.noEmitSubstitution : _b, onBeforeEmitNodeArray = handlers.onBeforeEmitNodeArray, onAfterEmitNodeArray = handlers.onAfterEmitNodeArray, onBeforeEmitToken = handlers.onBeforeEmitToken, onAfterEmitToken = handlers.onAfterEmitToken;
            var extendedDiagnostics = !!printerOptions.extendedDiagnostics;
            var newLine = ts.getNewLineCharacter(printerOptions);
            var moduleKind = ts.getEmitModuleKind(printerOptions);
            var bundledHelpers = ts.createMap();
            var currentSourceFile;
            var nodeIdToGeneratedName; // Map of generated names for specific nodes.
            var autoGeneratedIdToGeneratedName; // Map of generated names for temp and loop variables.
            var generatedNames; // Set of names generated by the NameGenerator.
            var tempFlagsStack; // Stack of enclosing name generation scopes.
            var tempFlags; // TempFlags for the current name generation scope.
            var reservedNamesStack; // Stack of TempFlags reserved in enclosing name generation scopes.
            var reservedNames; // TempFlags to reserve in nested name generation scopes.
            var writer;
            var ownWriter; // Reusable `EmitTextWriter` for basic printing.
            var write = writeBase;
            var isOwnFileEmit;
            var bundleFileInfo = printerOptions.writeBundleFileInfo ? { sections: [] } : undefined;
            var relativeToBuildInfo = bundleFileInfo ? ts.Debug.assertDefined(printerOptions.relativeToBuildInfo) : undefined;
            var recordInternalSection = printerOptions.recordInternalSection;
            var sourceFileTextPos = 0;
            var sourceFileTextKind = "text" /* Text */;
            // Source Maps
            var sourceMapsDisabled = true;
            var sourceMapGenerator;
            var sourceMapSource;
            var sourceMapSourceIndex = -1;
            // Comments
            var containerPos = -1;
            var containerEnd = -1;
            var declarationListContainerEnd = -1;
            var currentLineMap;
            var detachedCommentsInfo;
            var hasWrittenComment = false;
            var commentsDisabled = !!printerOptions.removeComments;
            var lastNode;
            var lastSubstitution;
            var _c = ts.performance.createTimerIf(extendedDiagnostics, "commentTime", "beforeComment", "afterComment"), enterComment = _c.enter, exitComment = _c.exit;
            reset();
            return {
                // public API
                printNode: printNode,
                printList: printList,
                printFile: printFile,
                printBundle: printBundle,
                // internal API
                writeNode: writeNode,
                writeList: writeList,
                writeFile: writeFile,
                writeBundle: writeBundle,
                bundleFileInfo: bundleFileInfo
            };
            function printNode(hint, node, sourceFile) {
                switch (hint) {
                    case 0 /* SourceFile */:
                        ts.Debug.assert(ts.isSourceFile(node), "Expected a SourceFile node.");
                        break;
                    case 2 /* IdentifierName */:
                        ts.Debug.assert(ts.isIdentifier(node), "Expected an Identifier node.");
                        break;
                    case 1 /* Expression */:
                        ts.Debug.assert(ts.isExpression(node), "Expected an Expression node.");
                        break;
                }
                switch (node.kind) {
                    case 290 /* SourceFile */: return printFile(node);
                    case 291 /* Bundle */: return printBundle(node);
                    case 292 /* UnparsedSource */: return printUnparsedSource(node);
                }
                writeNode(hint, node, sourceFile, beginPrint());
                return endPrint();
            }
            function printList(format, nodes, sourceFile) {
                writeList(format, nodes, sourceFile, beginPrint());
                return endPrint();
            }
            function printBundle(bundle) {
                writeBundle(bundle, beginPrint(), /*sourceMapEmitter*/ undefined);
                return endPrint();
            }
            function printFile(sourceFile) {
                writeFile(sourceFile, beginPrint(), /*sourceMapEmitter*/ undefined);
                return endPrint();
            }
            function printUnparsedSource(unparsed) {
                writeUnparsedSource(unparsed, beginPrint());
                return endPrint();
            }
            function writeNode(hint, node, sourceFile, output) {
                var previousWriter = writer;
                setWriter(output, /*_sourceMapGenerator*/ undefined);
                print(hint, node, sourceFile);
                reset();
                writer = previousWriter;
            }
            function writeList(format, nodes, sourceFile, output) {
                var previousWriter = writer;
                setWriter(output, /*_sourceMapGenerator*/ undefined);
                if (sourceFile) {
                    setSourceFile(sourceFile);
                }
                emitList(syntheticParent, nodes, format);
                reset();
                writer = previousWriter;
            }
            function getTextPosWithWriteLine() {
                return writer.getTextPosWithWriteLine ? writer.getTextPosWithWriteLine() : writer.getTextPos();
            }
            function updateOrPushBundleFileTextLike(pos, end, kind) {
                var last = ts.lastOrUndefined(bundleFileInfo.sections);
                if (last && last.kind === kind) {
                    last.end = end;
                }
                else {
                    bundleFileInfo.sections.push({ pos: pos, end: end, kind: kind });
                }
            }
            function recordBundleFileInternalSectionStart(node) {
                if (recordInternalSection &&
                    bundleFileInfo &&
                    currentSourceFile &&
                    (ts.isDeclaration(node) || ts.isVariableStatement(node)) &&
                    ts.isInternalDeclaration(node, currentSourceFile) &&
                    sourceFileTextKind !== "internal" /* Internal */) {
                    var prevSourceFileTextKind = sourceFileTextKind;
                    recordBundleFileTextLikeSection(writer.getTextPos());
                    sourceFileTextPos = getTextPosWithWriteLine();
                    sourceFileTextKind = "internal" /* Internal */;
                    return prevSourceFileTextKind;
                }
                return undefined;
            }
            function recordBundleFileInternalSectionEnd(prevSourceFileTextKind) {
                if (prevSourceFileTextKind) {
                    recordBundleFileTextLikeSection(writer.getTextPos());
                    sourceFileTextPos = getTextPosWithWriteLine();
                    sourceFileTextKind = prevSourceFileTextKind;
                }
            }
            function recordBundleFileTextLikeSection(end) {
                if (sourceFileTextPos < end) {
                    updateOrPushBundleFileTextLike(sourceFileTextPos, end, sourceFileTextKind);
                    return true;
                }
                return false;
            }
            function writeBundle(bundle, output, sourceMapGenerator) {
                var _a;
                isOwnFileEmit = false;
                var previousWriter = writer;
                setWriter(output, sourceMapGenerator);
                emitShebangIfNeeded(bundle);
                emitPrologueDirectivesIfNeeded(bundle);
                emitHelpers(bundle);
                emitSyntheticTripleSlashReferencesIfNeeded(bundle);
                for (var _b = 0, _c = bundle.prepends; _b < _c.length; _b++) {
                    var prepend = _c[_b];
                    writeLine();
                    var pos = writer.getTextPos();
                    var savedSections = bundleFileInfo && bundleFileInfo.sections;
                    if (savedSections)
                        bundleFileInfo.sections = [];
                    print(4 /* Unspecified */, prepend, /*sourceFile*/ undefined);
                    if (bundleFileInfo) {
                        var newSections = bundleFileInfo.sections;
                        bundleFileInfo.sections = savedSections;
                        if (prepend.oldFileOfCurrentEmit)
                            (_a = bundleFileInfo.sections).push.apply(_a, newSections);
                        else {
                            newSections.forEach(function (section) { return ts.Debug.assert(ts.isBundleFileTextLike(section)); });
                            bundleFileInfo.sections.push({
                                pos: pos,
                                end: writer.getTextPos(),
                                kind: "prepend" /* Prepend */,
                                data: relativeToBuildInfo(prepend.fileName),
                                texts: newSections
                            });
                        }
                    }
                }
                sourceFileTextPos = getTextPosWithWriteLine();
                for (var _d = 0, _e = bundle.sourceFiles; _d < _e.length; _d++) {
                    var sourceFile = _e[_d];
                    print(0 /* SourceFile */, sourceFile, sourceFile);
                }
                if (bundleFileInfo && bundle.sourceFiles.length) {
                    var end = writer.getTextPos();
                    if (recordBundleFileTextLikeSection(end)) {
                        // Store prologues
                        var prologues = getPrologueDirectivesFromBundledSourceFiles(bundle);
                        if (prologues) {
                            if (!bundleFileInfo.sources)
                                bundleFileInfo.sources = {};
                            bundleFileInfo.sources.prologues = prologues;
                        }
                        // Store helpes
                        var helpers = getHelpersFromBundledSourceFiles(bundle);
                        if (helpers) {
                            if (!bundleFileInfo.sources)
                                bundleFileInfo.sources = {};
                            bundleFileInfo.sources.helpers = helpers;
                        }
                    }
                }
                reset();
                writer = previousWriter;
            }
            function writeUnparsedSource(unparsed, output) {
                var previousWriter = writer;
                setWriter(output, /*_sourceMapGenerator*/ undefined);
                print(4 /* Unspecified */, unparsed, /*sourceFile*/ undefined);
                reset();
                writer = previousWriter;
            }
            function writeFile(sourceFile, output, sourceMapGenerator) {
                isOwnFileEmit = true;
                var previousWriter = writer;
                setWriter(output, sourceMapGenerator);
                emitShebangIfNeeded(sourceFile);
                emitPrologueDirectivesIfNeeded(sourceFile);
                print(0 /* SourceFile */, sourceFile, sourceFile);
                reset();
                writer = previousWriter;
            }
            function beginPrint() {
                return ownWriter || (ownWriter = ts.createTextWriter(newLine));
            }
            function endPrint() {
                var text = ownWriter.getText();
                ownWriter.clear();
                return text;
            }
            function print(hint, node, sourceFile) {
                if (sourceFile) {
                    setSourceFile(sourceFile);
                }
                pipelineEmit(hint, node);
            }
            function setSourceFile(sourceFile) {
                currentSourceFile = sourceFile;
                currentLineMap = undefined;
                detachedCommentsInfo = undefined;
                if (sourceFile) {
                    setSourceMapSource(sourceFile);
                }
            }
            function setWriter(_writer, _sourceMapGenerator) {
                if (_writer && printerOptions.omitTrailingSemicolon) {
                    _writer = ts.getTrailingSemicolonDeferringWriter(_writer);
                }
                writer = _writer; // TODO: GH#18217
                sourceMapGenerator = _sourceMapGenerator;
                sourceMapsDisabled = !writer || !sourceMapGenerator;
            }
            function reset() {
                nodeIdToGeneratedName = [];
                autoGeneratedIdToGeneratedName = [];
                generatedNames = ts.createMap();
                tempFlagsStack = [];
                tempFlags = 0 /* Auto */;
                reservedNamesStack = [];
                currentSourceFile = undefined;
                currentLineMap = undefined;
                detachedCommentsInfo = undefined;
                lastNode = undefined;
                lastSubstitution = undefined;
                setWriter(/*output*/ undefined, /*_sourceMapGenerator*/ undefined);
            }
            function getCurrentLineMap() {
                return currentLineMap || (currentLineMap = ts.getLineStarts(currentSourceFile));
            }
            function emit(node) {
                if (node === undefined)
                    return;
                var prevSourceFileTextKind = recordBundleFileInternalSectionStart(node);
                var substitute = pipelineEmit(4 /* Unspecified */, node);
                recordBundleFileInternalSectionEnd(prevSourceFileTextKind);
                return substitute;
            }
            function emitIdentifierName(node) {
                if (node === undefined)
                    return;
                return pipelineEmit(2 /* IdentifierName */, node);
            }
            function emitExpression(node) {
                if (node === undefined)
                    return;
                return pipelineEmit(1 /* Expression */, node);
            }
            function emitJsxAttributeValue(node) {
                return pipelineEmit(ts.isStringLiteral(node) ? 6 /* JsxAttributeValue */ : 4 /* Unspecified */, node);
            }
            function pipelineEmit(emitHint, node) {
                var savedLastNode = lastNode;
                var savedLastSubstitution = lastSubstitution;
                lastNode = node;
                lastSubstitution = undefined;
                var pipelinePhase = getPipelinePhase(0 /* Notification */, emitHint, node);
                pipelinePhase(emitHint, node);
                ts.Debug.assert(lastNode === node);
                var substitute = lastSubstitution;
                lastNode = savedLastNode;
                lastSubstitution = savedLastSubstitution;
                return substitute || node;
            }
            function getPipelinePhase(phase, emitHint, node) {
                switch (phase) {
                    case 0 /* Notification */:
                        if (onEmitNode !== ts.noEmitNotification && (!isEmitNotificationEnabled || isEmitNotificationEnabled(node))) {
                            return pipelineEmitWithNotification;
                        }
                    // falls through
                    case 1 /* Substitution */:
                        if (substituteNode !== ts.noEmitSubstitution && (lastSubstitution = substituteNode(emitHint, node)) !== node) {
                            return pipelineEmitWithSubstitution;
                        }
                    // falls through
                    case 2 /* Comments */:
                        if (!commentsDisabled && node.kind !== 290 /* SourceFile */) {
                            return pipelineEmitWithComments;
                        }
                    // falls through
                    case 3 /* SourceMaps */:
                        if (!sourceMapsDisabled && node.kind !== 290 /* SourceFile */ && !ts.isInJsonFile(node)) {
                            return pipelineEmitWithSourceMap;
                        }
                    // falls through
                    case 4 /* Emit */:
                        return pipelineEmitWithHint;
                    default:
                        return ts.Debug.assertNever(phase);
                }
            }
            function getNextPipelinePhase(currentPhase, emitHint, node) {
                return getPipelinePhase(currentPhase + 1, emitHint, node);
            }
            function pipelineEmitWithNotification(hint, node) {
                ts.Debug.assert(lastNode === node);
                var pipelinePhase = getNextPipelinePhase(0 /* Notification */, hint, node);
                onEmitNode(hint, node, pipelinePhase);
                ts.Debug.assert(lastNode === node);
            }
            function pipelineEmitWithHint(hint, node) {
                ts.Debug.assert(lastNode === node || lastSubstitution === node);
                if (hint === 0 /* SourceFile */)
                    return emitSourceFile(ts.cast(node, ts.isSourceFile));
                if (hint === 2 /* IdentifierName */)
                    return emitIdentifier(ts.cast(node, ts.isIdentifier));
                if (hint === 6 /* JsxAttributeValue */)
                    return emitLiteral(ts.cast(node, ts.isStringLiteral), /*jsxAttributeEscape*/ true);
                if (hint === 3 /* MappedTypeParameter */)
                    return emitMappedTypeParameter(ts.cast(node, ts.isTypeParameterDeclaration));
                if (hint === 5 /* EmbeddedStatement */) {
                    ts.Debug.assertNode(node, ts.isEmptyStatement);
                    return emitEmptyStatement(/*isEmbeddedStatement*/ true);
                }
                if (hint === 4 /* Unspecified */) {
                    if (ts.isKeyword(node.kind))
                        return writeTokenNode(node, writeKeyword);
                    switch (node.kind) {
                        // Pseudo-literals
                        case 15 /* TemplateHead */:
                        case 16 /* TemplateMiddle */:
                        case 17 /* TemplateTail */:
                            return emitLiteral(node, /*jsxAttributeEscape*/ false);
                        case 292 /* UnparsedSource */:
                        case 286 /* UnparsedPrepend */:
                            return emitUnparsedSourceOrPrepend(node);
                        case 285 /* UnparsedPrologue */:
                            return writeUnparsedNode(node);
                        case 287 /* UnparsedText */:
                        case 288 /* UnparsedInternalText */:
                            return emitUnparsedTextLike(node);
                        case 289 /* UnparsedSyntheticReference */:
                            return emitUnparsedSyntheticReference(node);
                        // Identifiers
                        case 75 /* Identifier */:
                            return emitIdentifier(node);
                        // PrivateIdentifiers
                        case 76 /* PrivateIdentifier */:
                            return emitPrivateIdentifier(node);
                        // Parse tree nodes
                        // Names
                        case 153 /* QualifiedName */:
                            return emitQualifiedName(node);
                        case 154 /* ComputedPropertyName */:
                            return emitComputedPropertyName(node);
                        // Signature elements
                        case 155 /* TypeParameter */:
                            return emitTypeParameter(node);
                        case 156 /* Parameter */:
                            return emitParameter(node);
                        case 157 /* Decorator */:
                            return emitDecorator(node);
                        // Type members
                        case 158 /* PropertySignature */:
                            return emitPropertySignature(node);
                        case 159 /* PropertyDeclaration */:
                            return emitPropertyDeclaration(node);
                        case 160 /* MethodSignature */:
                            return emitMethodSignature(node);
                        case 161 /* MethodDeclaration */:
                            return emitMethodDeclaration(node);
                        case 162 /* Constructor */:
                            return emitConstructor(node);
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            return emitAccessorDeclaration(node);
                        case 165 /* CallSignature */:
                            return emitCallSignature(node);
                        case 166 /* ConstructSignature */:
                            return emitConstructSignature(node);
                        case 167 /* IndexSignature */:
                            return emitIndexSignature(node);
                        // Types
                        case 168 /* TypePredicate */:
                            return emitTypePredicate(node);
                        case 169 /* TypeReference */:
                            return emitTypeReference(node);
                        case 170 /* FunctionType */:
                            return emitFunctionType(node);
                        case 300 /* JSDocFunctionType */:
                            return emitJSDocFunctionType(node);
                        case 171 /* ConstructorType */:
                            return emitConstructorType(node);
                        case 172 /* TypeQuery */:
                            return emitTypeQuery(node);
                        case 173 /* TypeLiteral */:
                            return emitTypeLiteral(node);
                        case 174 /* ArrayType */:
                            return emitArrayType(node);
                        case 175 /* TupleType */:
                            return emitTupleType(node);
                        case 176 /* OptionalType */:
                            return emitOptionalType(node);
                        case 178 /* UnionType */:
                            return emitUnionType(node);
                        case 179 /* IntersectionType */:
                            return emitIntersectionType(node);
                        case 180 /* ConditionalType */:
                            return emitConditionalType(node);
                        case 181 /* InferType */:
                            return emitInferType(node);
                        case 182 /* ParenthesizedType */:
                            return emitParenthesizedType(node);
                        case 216 /* ExpressionWithTypeArguments */:
                            return emitExpressionWithTypeArguments(node);
                        case 183 /* ThisType */:
                            return emitThisType();
                        case 184 /* TypeOperator */:
                            return emitTypeOperator(node);
                        case 185 /* IndexedAccessType */:
                            return emitIndexedAccessType(node);
                        case 186 /* MappedType */:
                            return emitMappedType(node);
                        case 187 /* LiteralType */:
                            return emitLiteralType(node);
                        case 188 /* ImportType */:
                            return emitImportTypeNode(node);
                        case 295 /* JSDocAllType */:
                            writePunctuation("*");
                            return;
                        case 296 /* JSDocUnknownType */:
                            writePunctuation("?");
                            return;
                        case 297 /* JSDocNullableType */:
                            return emitJSDocNullableType(node);
                        case 298 /* JSDocNonNullableType */:
                            return emitJSDocNonNullableType(node);
                        case 299 /* JSDocOptionalType */:
                            return emitJSDocOptionalType(node);
                        case 177 /* RestType */:
                        case 301 /* JSDocVariadicType */:
                            return emitRestOrJSDocVariadicType(node);
                        // Binding patterns
                        case 189 /* ObjectBindingPattern */:
                            return emitObjectBindingPattern(node);
                        case 190 /* ArrayBindingPattern */:
                            return emitArrayBindingPattern(node);
                        case 191 /* BindingElement */:
                            return emitBindingElement(node);
                        // Misc
                        case 221 /* TemplateSpan */:
                            return emitTemplateSpan(node);
                        case 222 /* SemicolonClassElement */:
                            return emitSemicolonClassElement();
                        // Statements
                        case 223 /* Block */:
                            return emitBlock(node);
                        case 225 /* VariableStatement */:
                            return emitVariableStatement(node);
                        case 224 /* EmptyStatement */:
                            return emitEmptyStatement(/*isEmbeddedStatement*/ false);
                        case 226 /* ExpressionStatement */:
                            return emitExpressionStatement(node);
                        case 227 /* IfStatement */:
                            return emitIfStatement(node);
                        case 228 /* DoStatement */:
                            return emitDoStatement(node);
                        case 229 /* WhileStatement */:
                            return emitWhileStatement(node);
                        case 230 /* ForStatement */:
                            return emitForStatement(node);
                        case 231 /* ForInStatement */:
                            return emitForInStatement(node);
                        case 232 /* ForOfStatement */:
                            return emitForOfStatement(node);
                        case 233 /* ContinueStatement */:
                            return emitContinueStatement(node);
                        case 234 /* BreakStatement */:
                            return emitBreakStatement(node);
                        case 235 /* ReturnStatement */:
                            return emitReturnStatement(node);
                        case 236 /* WithStatement */:
                            return emitWithStatement(node);
                        case 237 /* SwitchStatement */:
                            return emitSwitchStatement(node);
                        case 238 /* LabeledStatement */:
                            return emitLabeledStatement(node);
                        case 239 /* ThrowStatement */:
                            return emitThrowStatement(node);
                        case 240 /* TryStatement */:
                            return emitTryStatement(node);
                        case 241 /* DebuggerStatement */:
                            return emitDebuggerStatement(node);
                        // Declarations
                        case 242 /* VariableDeclaration */:
                            return emitVariableDeclaration(node);
                        case 243 /* VariableDeclarationList */:
                            return emitVariableDeclarationList(node);
                        case 244 /* FunctionDeclaration */:
                            return emitFunctionDeclaration(node);
                        case 245 /* ClassDeclaration */:
                            return emitClassDeclaration(node);
                        case 246 /* InterfaceDeclaration */:
                            return emitInterfaceDeclaration(node);
                        case 247 /* TypeAliasDeclaration */:
                            return emitTypeAliasDeclaration(node);
                        case 248 /* EnumDeclaration */:
                            return emitEnumDeclaration(node);
                        case 249 /* ModuleDeclaration */:
                            return emitModuleDeclaration(node);
                        case 250 /* ModuleBlock */:
                            return emitModuleBlock(node);
                        case 251 /* CaseBlock */:
                            return emitCaseBlock(node);
                        case 252 /* NamespaceExportDeclaration */:
                            return emitNamespaceExportDeclaration(node);
                        case 253 /* ImportEqualsDeclaration */:
                            return emitImportEqualsDeclaration(node);
                        case 254 /* ImportDeclaration */:
                            return emitImportDeclaration(node);
                        case 255 /* ImportClause */:
                            return emitImportClause(node);
                        case 256 /* NamespaceImport */:
                            return emitNamespaceImport(node);
                        case 262 /* NamespaceExport */:
                            return emitNamespaceExport(node);
                        case 257 /* NamedImports */:
                            return emitNamedImports(node);
                        case 258 /* ImportSpecifier */:
                            return emitImportSpecifier(node);
                        case 259 /* ExportAssignment */:
                            return emitExportAssignment(node);
                        case 260 /* ExportDeclaration */:
                            return emitExportDeclaration(node);
                        case 261 /* NamedExports */:
                            return emitNamedExports(node);
                        case 263 /* ExportSpecifier */:
                            return emitExportSpecifier(node);
                        case 264 /* MissingDeclaration */:
                            return;
                        // Module references
                        case 265 /* ExternalModuleReference */:
                            return emitExternalModuleReference(node);
                        // JSX (non-expression)
                        case 11 /* JsxText */:
                            return emitJsxText(node);
                        case 268 /* JsxOpeningElement */:
                        case 271 /* JsxOpeningFragment */:
                            return emitJsxOpeningElementOrFragment(node);
                        case 269 /* JsxClosingElement */:
                        case 272 /* JsxClosingFragment */:
                            return emitJsxClosingElementOrFragment(node);
                        case 273 /* JsxAttribute */:
                            return emitJsxAttribute(node);
                        case 274 /* JsxAttributes */:
                            return emitJsxAttributes(node);
                        case 275 /* JsxSpreadAttribute */:
                            return emitJsxSpreadAttribute(node);
                        case 276 /* JsxExpression */:
                            return emitJsxExpression(node);
                        // Clauses
                        case 277 /* CaseClause */:
                            return emitCaseClause(node);
                        case 278 /* DefaultClause */:
                            return emitDefaultClause(node);
                        case 279 /* HeritageClause */:
                            return emitHeritageClause(node);
                        case 280 /* CatchClause */:
                            return emitCatchClause(node);
                        // Property assignments
                        case 281 /* PropertyAssignment */:
                            return emitPropertyAssignment(node);
                        case 282 /* ShorthandPropertyAssignment */:
                            return emitShorthandPropertyAssignment(node);
                        case 283 /* SpreadAssignment */:
                            return emitSpreadAssignment(node);
                        // Enum
                        case 284 /* EnumMember */:
                            return emitEnumMember(node);
                        // JSDoc nodes (only used in codefixes currently)
                        case 316 /* JSDocParameterTag */:
                        case 322 /* JSDocPropertyTag */:
                            return emitJSDocPropertyLikeTag(node);
                        case 317 /* JSDocReturnTag */:
                        case 319 /* JSDocTypeTag */:
                        case 318 /* JSDocThisTag */:
                        case 315 /* JSDocEnumTag */:
                            return emitJSDocSimpleTypedTag(node);
                        case 307 /* JSDocAugmentsTag */:
                            return emitJSDocAugmentsTag(node);
                        case 320 /* JSDocTemplateTag */:
                            return emitJSDocTemplateTag(node);
                        case 321 /* JSDocTypedefTag */:
                            return emitJSDocTypedefTag(node);
                        case 314 /* JSDocCallbackTag */:
                            return emitJSDocCallbackTag(node);
                        case 305 /* JSDocSignature */:
                            return emitJSDocSignature(node);
                        case 304 /* JSDocTypeLiteral */:
                            return emitJSDocTypeLiteral(node);
                        case 309 /* JSDocClassTag */:
                        case 306 /* JSDocTag */:
                            return emitJSDocSimpleTag(node);
                        case 303 /* JSDocComment */:
                            return emitJSDoc(node);
                        // Transformation nodes (ignored)
                    }
                    if (ts.isExpression(node)) {
                        hint = 1 /* Expression */;
                        if (substituteNode !== ts.noEmitSubstitution) {
                            lastSubstitution = node = substituteNode(hint, node);
                        }
                    }
                    else if (ts.isToken(node)) {
                        return writeTokenNode(node, writePunctuation);
                    }
                }
                if (hint === 1 /* Expression */) {
                    switch (node.kind) {
                        // Literals
                        case 8 /* NumericLiteral */:
                        case 9 /* BigIntLiteral */:
                            return emitNumericOrBigIntLiteral(node);
                        case 10 /* StringLiteral */:
                        case 13 /* RegularExpressionLiteral */:
                        case 14 /* NoSubstitutionTemplateLiteral */:
                            return emitLiteral(node, /*jsxAttributeEscape*/ false);
                        // Identifiers
                        case 75 /* Identifier */:
                            return emitIdentifier(node);
                        // Reserved words
                        case 91 /* FalseKeyword */:
                        case 100 /* NullKeyword */:
                        case 102 /* SuperKeyword */:
                        case 106 /* TrueKeyword */:
                        case 104 /* ThisKeyword */:
                        case 96 /* ImportKeyword */:
                            writeTokenNode(node, writeKeyword);
                            return;
                        // Expressions
                        case 192 /* ArrayLiteralExpression */:
                            return emitArrayLiteralExpression(node);
                        case 193 /* ObjectLiteralExpression */:
                            return emitObjectLiteralExpression(node);
                        case 194 /* PropertyAccessExpression */:
                            return emitPropertyAccessExpression(node);
                        case 195 /* ElementAccessExpression */:
                            return emitElementAccessExpression(node);
                        case 196 /* CallExpression */:
                            return emitCallExpression(node);
                        case 197 /* NewExpression */:
                            return emitNewExpression(node);
                        case 198 /* TaggedTemplateExpression */:
                            return emitTaggedTemplateExpression(node);
                        case 199 /* TypeAssertionExpression */:
                            return emitTypeAssertionExpression(node);
                        case 200 /* ParenthesizedExpression */:
                            return emitParenthesizedExpression(node);
                        case 201 /* FunctionExpression */:
                            return emitFunctionExpression(node);
                        case 202 /* ArrowFunction */:
                            return emitArrowFunction(node);
                        case 203 /* DeleteExpression */:
                            return emitDeleteExpression(node);
                        case 204 /* TypeOfExpression */:
                            return emitTypeOfExpression(node);
                        case 205 /* VoidExpression */:
                            return emitVoidExpression(node);
                        case 206 /* AwaitExpression */:
                            return emitAwaitExpression(node);
                        case 207 /* PrefixUnaryExpression */:
                            return emitPrefixUnaryExpression(node);
                        case 208 /* PostfixUnaryExpression */:
                            return emitPostfixUnaryExpression(node);
                        case 209 /* BinaryExpression */:
                            return emitBinaryExpression(node);
                        case 210 /* ConditionalExpression */:
                            return emitConditionalExpression(node);
                        case 211 /* TemplateExpression */:
                            return emitTemplateExpression(node);
                        case 212 /* YieldExpression */:
                            return emitYieldExpression(node);
                        case 213 /* SpreadElement */:
                            return emitSpreadExpression(node);
                        case 214 /* ClassExpression */:
                            return emitClassExpression(node);
                        case 215 /* OmittedExpression */:
                            return;
                        case 217 /* AsExpression */:
                            return emitAsExpression(node);
                        case 218 /* NonNullExpression */:
                            return emitNonNullExpression(node);
                        case 219 /* MetaProperty */:
                            return emitMetaProperty(node);
                        // JSX
                        case 266 /* JsxElement */:
                            return emitJsxElement(node);
                        case 267 /* JsxSelfClosingElement */:
                            return emitJsxSelfClosingElement(node);
                        case 270 /* JsxFragment */:
                            return emitJsxFragment(node);
                        // Transformation nodes
                        case 325 /* PartiallyEmittedExpression */:
                            return emitPartiallyEmittedExpression(node);
                        case 326 /* CommaListExpression */:
                            return emitCommaList(node);
                    }
                }
            }
            function emitMappedTypeParameter(node) {
                emit(node.name);
                writeSpace();
                writeKeyword("in");
                writeSpace();
                emit(node.constraint);
            }
            function pipelineEmitWithSubstitution(hint, node) {
                ts.Debug.assert(lastNode === node || lastSubstitution === node);
                var pipelinePhase = getNextPipelinePhase(1 /* Substitution */, hint, node);
                pipelinePhase(hint, lastSubstitution);
                ts.Debug.assert(lastNode === node || lastSubstitution === node);
            }
            function getHelpersFromBundledSourceFiles(bundle) {
                var result;
                if (moduleKind === ts.ModuleKind.None || printerOptions.noEmitHelpers) {
                    return undefined;
                }
                var bundledHelpers = ts.createMap();
                for (var _a = 0, _b = bundle.sourceFiles; _a < _b.length; _a++) {
                    var sourceFile = _b[_a];
                    var shouldSkip = ts.getExternalHelpersModuleName(sourceFile) !== undefined;
                    var helpers = getSortedEmitHelpers(sourceFile);
                    if (!helpers)
                        continue;
                    for (var _c = 0, helpers_4 = helpers; _c < helpers_4.length; _c++) {
                        var helper = helpers_4[_c];
                        if (!helper.scoped && !shouldSkip && !bundledHelpers.get(helper.name)) {
                            bundledHelpers.set(helper.name, true);
                            (result || (result = [])).push(helper.name);
                        }
                    }
                }
                return result;
            }
            function emitHelpers(node) {
                var helpersEmitted = false;
                var bundle = node.kind === 291 /* Bundle */ ? node : undefined;
                if (bundle && moduleKind === ts.ModuleKind.None) {
                    return;
                }
                var numPrepends = bundle ? bundle.prepends.length : 0;
                var numNodes = bundle ? bundle.sourceFiles.length + numPrepends : 1;
                for (var i = 0; i < numNodes; i++) {
                    var currentNode = bundle ? i < numPrepends ? bundle.prepends[i] : bundle.sourceFiles[i - numPrepends] : node;
                    var sourceFile = ts.isSourceFile(currentNode) ? currentNode : ts.isUnparsedSource(currentNode) ? undefined : currentSourceFile;
                    var shouldSkip = printerOptions.noEmitHelpers || (!!sourceFile && ts.hasRecordedExternalHelpers(sourceFile));
                    var shouldBundle = (ts.isSourceFile(currentNode) || ts.isUnparsedSource(currentNode)) && !isOwnFileEmit;
                    var helpers = ts.isUnparsedSource(currentNode) ? currentNode.helpers : getSortedEmitHelpers(currentNode);
                    if (helpers) {
                        for (var _a = 0, helpers_5 = helpers; _a < helpers_5.length; _a++) {
                            var helper = helpers_5[_a];
                            if (!helper.scoped) {
                                // Skip the helper if it can be skipped and the noEmitHelpers compiler
                                // option is set, or if it can be imported and the importHelpers compiler
                                // option is set.
                                if (shouldSkip)
                                    continue;
                                // Skip the helper if it can be bundled but hasn't already been emitted and we
                                // are emitting a bundled module.
                                if (shouldBundle) {
                                    if (bundledHelpers.get(helper.name)) {
                                        continue;
                                    }
                                    bundledHelpers.set(helper.name, true);
                                }
                            }
                            else if (bundle) {
                                // Skip the helper if it is scoped and we are emitting bundled helpers
                                continue;
                            }
                            var pos = getTextPosWithWriteLine();
                            if (typeof helper.text === "string") {
                                writeLines(helper.text);
                            }
                            else {
                                writeLines(helper.text(makeFileLevelOptimisticUniqueName));
                            }
                            if (bundleFileInfo)
                                bundleFileInfo.sections.push({ pos: pos, end: writer.getTextPos(), kind: "emitHelpers" /* EmitHelpers */, data: helper.name });
                            helpersEmitted = true;
                        }
                    }
                }
                return helpersEmitted;
            }
            function getSortedEmitHelpers(node) {
                var helpers = ts.getEmitHelpers(node);
                return helpers && ts.stableSort(helpers, ts.compareEmitHelpers);
            }
            //
            // Literals/Pseudo-literals
            //
            // SyntaxKind.NumericLiteral
            // SyntaxKind.BigIntLiteral
            function emitNumericOrBigIntLiteral(node) {
                emitLiteral(node, /*jsxAttributeEscape*/ false);
            }
            // SyntaxKind.StringLiteral
            // SyntaxKind.RegularExpressionLiteral
            // SyntaxKind.NoSubstitutionTemplateLiteral
            // SyntaxKind.TemplateHead
            // SyntaxKind.TemplateMiddle
            // SyntaxKind.TemplateTail
            function emitLiteral(node, jsxAttributeEscape) {
                var text = getLiteralTextOfNode(node, printerOptions.neverAsciiEscape, jsxAttributeEscape);
                if ((printerOptions.sourceMap || printerOptions.inlineSourceMap)
                    && (node.kind === 10 /* StringLiteral */ || ts.isTemplateLiteralKind(node.kind))) {
                    writeLiteral(text);
                }
                else {
                    // Quick info expects all literals to be called with writeStringLiteral, as there's no specific type for numberLiterals
                    writeStringLiteral(text);
                }
            }
            // SyntaxKind.UnparsedSource
            // SyntaxKind.UnparsedPrepend
            function emitUnparsedSourceOrPrepend(unparsed) {
                for (var _a = 0, _b = unparsed.texts; _a < _b.length; _a++) {
                    var text = _b[_a];
                    writeLine();
                    emit(text);
                }
            }
            // SyntaxKind.UnparsedPrologue
            // SyntaxKind.UnparsedText
            // SyntaxKind.UnparsedInternal
            // SyntaxKind.UnparsedSyntheticReference
            function writeUnparsedNode(unparsed) {
                writer.rawWrite(unparsed.parent.text.substring(unparsed.pos, unparsed.end));
            }
            // SyntaxKind.UnparsedText
            // SyntaxKind.UnparsedInternal
            function emitUnparsedTextLike(unparsed) {
                var pos = getTextPosWithWriteLine();
                writeUnparsedNode(unparsed);
                if (bundleFileInfo) {
                    updateOrPushBundleFileTextLike(pos, writer.getTextPos(), unparsed.kind === 287 /* UnparsedText */ ?
                        "text" /* Text */ :
                        "internal" /* Internal */);
                }
            }
            // SyntaxKind.UnparsedSyntheticReference
            function emitUnparsedSyntheticReference(unparsed) {
                var pos = getTextPosWithWriteLine();
                writeUnparsedNode(unparsed);
                if (bundleFileInfo) {
                    var section = ts.clone(unparsed.section);
                    section.pos = pos;
                    section.end = writer.getTextPos();
                    bundleFileInfo.sections.push(section);
                }
            }
            //
            // Identifiers
            //
            function emitIdentifier(node) {
                var writeText = node.symbol ? writeSymbol : write;
                writeText(getTextOfNode(node, /*includeTrivia*/ false), node.symbol);
                emitList(node, node.typeArguments, 53776 /* TypeParameters */); // Call emitList directly since it could be an array of TypeParameterDeclarations _or_ type arguments
            }
            //
            // Names
            //
            function emitPrivateIdentifier(node) {
                var writeText = node.symbol ? writeSymbol : write;
                writeText(getTextOfNode(node, /*includeTrivia*/ false), node.symbol);
            }
            function emitQualifiedName(node) {
                emitEntityName(node.left);
                writePunctuation(".");
                emit(node.right);
            }
            function emitEntityName(node) {
                if (node.kind === 75 /* Identifier */) {
                    emitExpression(node);
                }
                else {
                    emit(node);
                }
            }
            function emitComputedPropertyName(node) {
                writePunctuation("[");
                emitExpression(node.expression);
                writePunctuation("]");
            }
            //
            // Signature elements
            //
            function emitTypeParameter(node) {
                emit(node.name);
                if (node.constraint) {
                    writeSpace();
                    writeKeyword("extends");
                    writeSpace();
                    emit(node.constraint);
                }
                if (node.default) {
                    writeSpace();
                    writeOperator("=");
                    writeSpace();
                    emit(node.default);
                }
            }
            function emitParameter(node) {
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                emit(node.dotDotDotToken);
                emitNodeWithWriter(node.name, writeParameter);
                emit(node.questionToken);
                if (node.parent && node.parent.kind === 300 /* JSDocFunctionType */ && !node.name) {
                    emit(node.type);
                }
                else {
                    emitTypeAnnotation(node.type);
                }
                // The comment position has to fallback to any present node within the parameterdeclaration because as it turns out, the parser can make parameter declarations with _just_ an initializer.
                emitInitializer(node.initializer, node.type ? node.type.end : node.questionToken ? node.questionToken.end : node.name ? node.name.end : node.modifiers ? node.modifiers.end : node.decorators ? node.decorators.end : node.pos, node);
            }
            function emitDecorator(decorator) {
                writePunctuation("@");
                emitExpression(decorator.expression);
            }
            //
            // Type members
            //
            function emitPropertySignature(node) {
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                emitNodeWithWriter(node.name, writeProperty);
                emit(node.questionToken);
                emitTypeAnnotation(node.type);
                writeTrailingSemicolon();
            }
            function emitPropertyDeclaration(node) {
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                emit(node.name);
                emit(node.questionToken);
                emit(node.exclamationToken);
                emitTypeAnnotation(node.type);
                emitInitializer(node.initializer, node.type ? node.type.end : node.questionToken ? node.questionToken.end : node.name.end, node);
                writeTrailingSemicolon();
            }
            function emitMethodSignature(node) {
                pushNameGenerationScope(node);
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                emit(node.name);
                emit(node.questionToken);
                emitTypeParameters(node, node.typeParameters);
                emitParameters(node, node.parameters);
                emitTypeAnnotation(node.type);
                writeTrailingSemicolon();
                popNameGenerationScope(node);
            }
            function emitMethodDeclaration(node) {
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                emit(node.asteriskToken);
                emit(node.name);
                emit(node.questionToken);
                emitSignatureAndBody(node, emitSignatureHead);
            }
            function emitConstructor(node) {
                emitModifiers(node, node.modifiers);
                writeKeyword("constructor");
                emitSignatureAndBody(node, emitSignatureHead);
            }
            function emitAccessorDeclaration(node) {
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                writeKeyword(node.kind === 163 /* GetAccessor */ ? "get" : "set");
                writeSpace();
                emit(node.name);
                emitSignatureAndBody(node, emitSignatureHead);
            }
            function emitCallSignature(node) {
                pushNameGenerationScope(node);
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                emitTypeParameters(node, node.typeParameters);
                emitParameters(node, node.parameters);
                emitTypeAnnotation(node.type);
                writeTrailingSemicolon();
                popNameGenerationScope(node);
            }
            function emitConstructSignature(node) {
                pushNameGenerationScope(node);
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                writeKeyword("new");
                writeSpace();
                emitTypeParameters(node, node.typeParameters);
                emitParameters(node, node.parameters);
                emitTypeAnnotation(node.type);
                writeTrailingSemicolon();
                popNameGenerationScope(node);
            }
            function emitIndexSignature(node) {
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                emitParametersForIndexSignature(node, node.parameters);
                emitTypeAnnotation(node.type);
                writeTrailingSemicolon();
            }
            function emitSemicolonClassElement() {
                writeTrailingSemicolon();
            }
            //
            // Types
            //
            function emitTypePredicate(node) {
                if (node.assertsModifier) {
                    emit(node.assertsModifier);
                    writeSpace();
                }
                emit(node.parameterName);
                if (node.type) {
                    writeSpace();
                    writeKeyword("is");
                    writeSpace();
                    emit(node.type);
                }
            }
            function emitTypeReference(node) {
                emit(node.typeName);
                emitTypeArguments(node, node.typeArguments);
            }
            function emitFunctionType(node) {
                pushNameGenerationScope(node);
                emitTypeParameters(node, node.typeParameters);
                emitParametersForArrow(node, node.parameters);
                writeSpace();
                writePunctuation("=>");
                writeSpace();
                emit(node.type);
                popNameGenerationScope(node);
            }
            function emitJSDocFunctionType(node) {
                writeKeyword("function");
                emitParameters(node, node.parameters);
                writePunctuation(":");
                emit(node.type);
            }
            function emitJSDocNullableType(node) {
                writePunctuation("?");
                emit(node.type);
            }
            function emitJSDocNonNullableType(node) {
                writePunctuation("!");
                emit(node.type);
            }
            function emitJSDocOptionalType(node) {
                emit(node.type);
                writePunctuation("=");
            }
            function emitConstructorType(node) {
                pushNameGenerationScope(node);
                writeKeyword("new");
                writeSpace();
                emitTypeParameters(node, node.typeParameters);
                emitParameters(node, node.parameters);
                writeSpace();
                writePunctuation("=>");
                writeSpace();
                emit(node.type);
                popNameGenerationScope(node);
            }
            function emitTypeQuery(node) {
                writeKeyword("typeof");
                writeSpace();
                emit(node.exprName);
            }
            function emitTypeLiteral(node) {
                writePunctuation("{");
                var flags = ts.getEmitFlags(node) & 1 /* SingleLine */ ? 768 /* SingleLineTypeLiteralMembers */ : 32897 /* MultiLineTypeLiteralMembers */;
                emitList(node, node.members, flags | 524288 /* NoSpaceIfEmpty */);
                writePunctuation("}");
            }
            function emitArrayType(node) {
                emit(node.elementType);
                writePunctuation("[");
                writePunctuation("]");
            }
            function emitRestOrJSDocVariadicType(node) {
                writePunctuation("...");
                emit(node.type);
            }
            function emitTupleType(node) {
                writePunctuation("[");
                emitList(node, node.elementTypes, 528 /* TupleTypeElements */);
                writePunctuation("]");
            }
            function emitOptionalType(node) {
                emit(node.type);
                writePunctuation("?");
            }
            function emitUnionType(node) {
                emitList(node, node.types, 516 /* UnionTypeConstituents */);
            }
            function emitIntersectionType(node) {
                emitList(node, node.types, 520 /* IntersectionTypeConstituents */);
            }
            function emitConditionalType(node) {
                emit(node.checkType);
                writeSpace();
                writeKeyword("extends");
                writeSpace();
                emit(node.extendsType);
                writeSpace();
                writePunctuation("?");
                writeSpace();
                emit(node.trueType);
                writeSpace();
                writePunctuation(":");
                writeSpace();
                emit(node.falseType);
            }
            function emitInferType(node) {
                writeKeyword("infer");
                writeSpace();
                emit(node.typeParameter);
            }
            function emitParenthesizedType(node) {
                writePunctuation("(");
                emit(node.type);
                writePunctuation(")");
            }
            function emitThisType() {
                writeKeyword("this");
            }
            function emitTypeOperator(node) {
                writeTokenText(node.operator, writeKeyword);
                writeSpace();
                emit(node.type);
            }
            function emitIndexedAccessType(node) {
                emit(node.objectType);
                writePunctuation("[");
                emit(node.indexType);
                writePunctuation("]");
            }
            function emitMappedType(node) {
                var emitFlags = ts.getEmitFlags(node);
                writePunctuation("{");
                if (emitFlags & 1 /* SingleLine */) {
                    writeSpace();
                }
                else {
                    writeLine();
                    increaseIndent();
                }
                if (node.readonlyToken) {
                    emit(node.readonlyToken);
                    if (node.readonlyToken.kind !== 138 /* ReadonlyKeyword */) {
                        writeKeyword("readonly");
                    }
                    writeSpace();
                }
                writePunctuation("[");
                pipelineEmit(3 /* MappedTypeParameter */, node.typeParameter);
                writePunctuation("]");
                if (node.questionToken) {
                    emit(node.questionToken);
                    if (node.questionToken.kind !== 57 /* QuestionToken */) {
                        writePunctuation("?");
                    }
                }
                writePunctuation(":");
                writeSpace();
                emit(node.type);
                writeTrailingSemicolon();
                if (emitFlags & 1 /* SingleLine */) {
                    writeSpace();
                }
                else {
                    writeLine();
                    decreaseIndent();
                }
                writePunctuation("}");
            }
            function emitLiteralType(node) {
                emitExpression(node.literal);
            }
            function emitImportTypeNode(node) {
                if (node.isTypeOf) {
                    writeKeyword("typeof");
                    writeSpace();
                }
                writeKeyword("import");
                writePunctuation("(");
                emit(node.argument);
                writePunctuation(")");
                if (node.qualifier) {
                    writePunctuation(".");
                    emit(node.qualifier);
                }
                emitTypeArguments(node, node.typeArguments);
            }
            //
            // Binding patterns
            //
            function emitObjectBindingPattern(node) {
                writePunctuation("{");
                emitList(node, node.elements, 525136 /* ObjectBindingPatternElements */);
                writePunctuation("}");
            }
            function emitArrayBindingPattern(node) {
                writePunctuation("[");
                emitList(node, node.elements, 524880 /* ArrayBindingPatternElements */);
                writePunctuation("]");
            }
            function emitBindingElement(node) {
                emit(node.dotDotDotToken);
                if (node.propertyName) {
                    emit(node.propertyName);
                    writePunctuation(":");
                    writeSpace();
                }
                emit(node.name);
                emitInitializer(node.initializer, node.name.end, node);
            }
            //
            // Expressions
            //
            function emitArrayLiteralExpression(node) {
                var elements = node.elements;
                var preferNewLine = node.multiLine ? 65536 /* PreferNewLine */ : 0 /* None */;
                emitExpressionList(node, elements, 8914 /* ArrayLiteralExpressionElements */ | preferNewLine);
            }
            function emitObjectLiteralExpression(node) {
                ts.forEach(node.properties, generateMemberNames);
                var indentedFlag = ts.getEmitFlags(node) & 65536 /* Indented */;
                if (indentedFlag) {
                    increaseIndent();
                }
                var preferNewLine = node.multiLine ? 65536 /* PreferNewLine */ : 0 /* None */;
                var allowTrailingComma = currentSourceFile.languageVersion >= 1 /* ES5 */ && !ts.isJsonSourceFile(currentSourceFile) ? 64 /* AllowTrailingComma */ : 0 /* None */;
                emitList(node, node.properties, 526226 /* ObjectLiteralExpressionProperties */ | allowTrailingComma | preferNewLine);
                if (indentedFlag) {
                    decreaseIndent();
                }
            }
            function emitPropertyAccessExpression(node) {
                var expression = ts.cast(emitExpression(node.expression), ts.isExpression);
                var token = ts.getDotOrQuestionDotToken(node);
                var indentBeforeDot = needsIndentation(node, node.expression, token);
                var indentAfterDot = needsIndentation(node, token, node.name);
                increaseIndentIf(indentBeforeDot, /*writeSpaceIfNotIndenting*/ false);
                var shouldEmitDotDot = token.kind !== 28 /* QuestionDotToken */ &&
                    mayNeedDotDotForPropertyAccess(expression) &&
                    !writer.hasTrailingComment() &&
                    !writer.hasTrailingWhitespace();
                if (shouldEmitDotDot) {
                    writePunctuation(".");
                }
                emitTokenWithComment(token.kind, node.expression.end, writePunctuation, node);
                increaseIndentIf(indentAfterDot, /*writeSpaceIfNotIndenting*/ false);
                emit(node.name);
                decreaseIndentIf(indentBeforeDot, indentAfterDot);
            }
            // 1..toString is a valid property access, emit a dot after the literal
            // Also emit a dot if expression is a integer const enum value - it will appear in generated code as numeric literal
            function mayNeedDotDotForPropertyAccess(expression) {
                expression = ts.skipPartiallyEmittedExpressions(expression);
                if (ts.isNumericLiteral(expression)) {
                    // check if numeric literal is a decimal literal that was originally written with a dot
                    var text = getLiteralTextOfNode(expression, /*neverAsciiEscape*/ true, /*jsxAttributeEscape*/ false);
                    // If he number will be printed verbatim and it doesn't already contain a dot, add one
                    // if the expression doesn't have any comments that will be emitted.
                    return !expression.numericLiteralFlags && !ts.stringContains(text, ts.tokenToString(24 /* DotToken */));
                }
                else if (ts.isAccessExpression(expression)) {
                    // check if constant enum value is integer
                    var constantValue = ts.getConstantValue(expression);
                    // isFinite handles cases when constantValue is undefined
                    return typeof constantValue === "number" && isFinite(constantValue)
                        && Math.floor(constantValue) === constantValue;
                }
            }
            function emitElementAccessExpression(node) {
                emitExpression(node.expression);
                emit(node.questionDotToken);
                emitTokenWithComment(22 /* OpenBracketToken */, node.expression.end, writePunctuation, node);
                emitExpression(node.argumentExpression);
                emitTokenWithComment(23 /* CloseBracketToken */, node.argumentExpression.end, writePunctuation, node);
            }
            function emitCallExpression(node) {
                emitExpression(node.expression);
                emit(node.questionDotToken);
                emitTypeArguments(node, node.typeArguments);
                emitExpressionList(node, node.arguments, 2576 /* CallExpressionArguments */);
            }
            function emitNewExpression(node) {
                emitTokenWithComment(99 /* NewKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emitExpression(node.expression);
                emitTypeArguments(node, node.typeArguments);
                emitExpressionList(node, node.arguments, 18960 /* NewExpressionArguments */);
            }
            function emitTaggedTemplateExpression(node) {
                emitExpression(node.tag);
                emitTypeArguments(node, node.typeArguments);
                writeSpace();
                emitExpression(node.template);
            }
            function emitTypeAssertionExpression(node) {
                writePunctuation("<");
                emit(node.type);
                writePunctuation(">");
                emitExpression(node.expression);
            }
            function emitParenthesizedExpression(node) {
                var openParenPos = emitTokenWithComment(20 /* OpenParenToken */, node.pos, writePunctuation, node);
                emitExpression(node.expression);
                emitTokenWithComment(21 /* CloseParenToken */, node.expression ? node.expression.end : openParenPos, writePunctuation, node);
            }
            function emitFunctionExpression(node) {
                generateNameIfNeeded(node.name);
                emitFunctionDeclarationOrExpression(node);
            }
            function emitArrowFunction(node) {
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                emitSignatureAndBody(node, emitArrowFunctionHead);
            }
            function emitArrowFunctionHead(node) {
                emitTypeParameters(node, node.typeParameters);
                emitParametersForArrow(node, node.parameters);
                emitTypeAnnotation(node.type);
                writeSpace();
                emit(node.equalsGreaterThanToken);
            }
            function emitDeleteExpression(node) {
                emitTokenWithComment(85 /* DeleteKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emitExpression(node.expression);
            }
            function emitTypeOfExpression(node) {
                emitTokenWithComment(108 /* TypeOfKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emitExpression(node.expression);
            }
            function emitVoidExpression(node) {
                emitTokenWithComment(110 /* VoidKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emitExpression(node.expression);
            }
            function emitAwaitExpression(node) {
                emitTokenWithComment(127 /* AwaitKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emitExpression(node.expression);
            }
            function emitPrefixUnaryExpression(node) {
                writeTokenText(node.operator, writeOperator);
                if (shouldEmitWhitespaceBeforeOperand(node)) {
                    writeSpace();
                }
                emitExpression(node.operand);
            }
            function shouldEmitWhitespaceBeforeOperand(node) {
                // In some cases, we need to emit a space between the operator and the operand. One obvious case
                // is when the operator is an identifier, like delete or typeof. We also need to do this for plus
                // and minus expressions in certain cases. Specifically, consider the following two cases (parens
                // are just for clarity of exposition, and not part of the source code):
                //
                //  (+(+1))
                //  (+(++1))
                //
                // We need to emit a space in both cases. In the first case, the absence of a space will make
                // the resulting expression a prefix increment operation. And in the second, it will make the resulting
                // expression a prefix increment whose operand is a plus expression - (++(+x))
                // The same is true of minus of course.
                var operand = node.operand;
                return operand.kind === 207 /* PrefixUnaryExpression */
                    && ((node.operator === 39 /* PlusToken */ && (operand.operator === 39 /* PlusToken */ || operand.operator === 45 /* PlusPlusToken */))
                        || (node.operator === 40 /* MinusToken */ && (operand.operator === 40 /* MinusToken */ || operand.operator === 46 /* MinusMinusToken */)));
            }
            function emitPostfixUnaryExpression(node) {
                emitExpression(node.operand);
                writeTokenText(node.operator, writeOperator);
            }
            var EmitBinaryExpressionState;
            (function (EmitBinaryExpressionState) {
                EmitBinaryExpressionState[EmitBinaryExpressionState["EmitLeft"] = 0] = "EmitLeft";
                EmitBinaryExpressionState[EmitBinaryExpressionState["EmitRight"] = 1] = "EmitRight";
                EmitBinaryExpressionState[EmitBinaryExpressionState["FinishEmit"] = 2] = "FinishEmit";
            })(EmitBinaryExpressionState || (EmitBinaryExpressionState = {}));
            /**
             * emitBinaryExpression includes an embedded work stack to attempt to handle as many nested binary expressions
             * as possible without creating any additional stack frames. This can only be done when the emit pipeline does
             * not require notification/substitution/comment/sourcemap decorations.
             */
            function emitBinaryExpression(node) {
                var nodeStack = [node];
                var stateStack = [0 /* EmitLeft */];
                var stackIndex = 0;
                while (stackIndex >= 0) {
                    node = nodeStack[stackIndex];
                    switch (stateStack[stackIndex]) {
                        case 0 /* EmitLeft */: {
                            maybePipelineEmitExpression(node.left);
                            break;
                        }
                        case 1 /* EmitRight */: {
                            var isCommaOperator = node.operatorToken.kind !== 27 /* CommaToken */;
                            var indentBeforeOperator = needsIndentation(node, node.left, node.operatorToken);
                            var indentAfterOperator = needsIndentation(node, node.operatorToken, node.right);
                            increaseIndentIf(indentBeforeOperator, isCommaOperator);
                            emitLeadingCommentsOfPosition(node.operatorToken.pos);
                            writeTokenNode(node.operatorToken, node.operatorToken.kind === 97 /* InKeyword */ ? writeKeyword : writeOperator);
                            emitTrailingCommentsOfPosition(node.operatorToken.end, /*prefixSpace*/ true); // Binary operators should have a space before the comment starts
                            increaseIndentIf(indentAfterOperator, /*writeSpaceIfNotIndenting*/ true);
                            maybePipelineEmitExpression(node.right);
                            break;
                        }
                        case 2 /* FinishEmit */: {
                            var indentBeforeOperator = needsIndentation(node, node.left, node.operatorToken);
                            var indentAfterOperator = needsIndentation(node, node.operatorToken, node.right);
                            decreaseIndentIf(indentBeforeOperator, indentAfterOperator);
                            stackIndex--;
                            break;
                        }
                        default: return ts.Debug.fail("Invalid state " + stateStack[stackIndex] + " for emitBinaryExpressionWorker");
                    }
                }
                function maybePipelineEmitExpression(next) {
                    // Advance the state of this unit of work,
                    stateStack[stackIndex]++;
                    // Then actually do the work of emitting the node `next` returned by the prior state
                    // The following section should be identical to `pipelineEmit` save it assumes EmitHint.Expression and offloads
                    // binary expression handling, where possible, to the contained work queue
                    // #region trampolinePipelineEmit
                    var savedLastNode = lastNode;
                    var savedLastSubstitution = lastSubstitution;
                    lastNode = next;
                    lastSubstitution = undefined;
                    var pipelinePhase = getPipelinePhase(0 /* Notification */, 1 /* Expression */, next);
                    if (pipelinePhase === pipelineEmitWithHint && ts.isBinaryExpression(next)) {
                        // If the target pipeline phase is emit directly, and the next node's also a binary expression,
                        // skip all the intermediate indirection and push the expression directly onto the work stack
                        stackIndex++;
                        stateStack[stackIndex] = 0 /* EmitLeft */;
                        nodeStack[stackIndex] = next;
                    }
                    else {
                        pipelinePhase(1 /* Expression */, next);
                    }
                    ts.Debug.assert(lastNode === next);
                    lastNode = savedLastNode;
                    lastSubstitution = savedLastSubstitution;
                    // #endregion trampolinePipelineEmit
                }
            }
            function emitConditionalExpression(node) {
                var indentBeforeQuestion = needsIndentation(node, node.condition, node.questionToken);
                var indentAfterQuestion = needsIndentation(node, node.questionToken, node.whenTrue);
                var indentBeforeColon = needsIndentation(node, node.whenTrue, node.colonToken);
                var indentAfterColon = needsIndentation(node, node.colonToken, node.whenFalse);
                emitExpression(node.condition);
                increaseIndentIf(indentBeforeQuestion, /*writeSpaceIfNotIndenting*/ true);
                emit(node.questionToken);
                increaseIndentIf(indentAfterQuestion, /*writeSpaceIfNotIndenting*/ true);
                emitExpression(node.whenTrue);
                decreaseIndentIf(indentBeforeQuestion, indentAfterQuestion);
                increaseIndentIf(indentBeforeColon, /*writeSpaceIfNotIndenting*/ true);
                emit(node.colonToken);
                increaseIndentIf(indentAfterColon, /*writeSpaceIfNotIndenting*/ true);
                emitExpression(node.whenFalse);
                decreaseIndentIf(indentBeforeColon, indentAfterColon);
            }
            function emitTemplateExpression(node) {
                emit(node.head);
                emitList(node, node.templateSpans, 262144 /* TemplateExpressionSpans */);
            }
            function emitYieldExpression(node) {
                emitTokenWithComment(121 /* YieldKeyword */, node.pos, writeKeyword, node);
                emit(node.asteriskToken);
                emitExpressionWithLeadingSpace(node.expression);
            }
            function emitSpreadExpression(node) {
                emitTokenWithComment(25 /* DotDotDotToken */, node.pos, writePunctuation, node);
                emitExpression(node.expression);
            }
            function emitClassExpression(node) {
                generateNameIfNeeded(node.name);
                emitClassDeclarationOrExpression(node);
            }
            function emitExpressionWithTypeArguments(node) {
                emitExpression(node.expression);
                emitTypeArguments(node, node.typeArguments);
            }
            function emitAsExpression(node) {
                emitExpression(node.expression);
                if (node.type) {
                    writeSpace();
                    writeKeyword("as");
                    writeSpace();
                    emit(node.type);
                }
            }
            function emitNonNullExpression(node) {
                emitExpression(node.expression);
                writeOperator("!");
            }
            function emitMetaProperty(node) {
                writeToken(node.keywordToken, node.pos, writePunctuation);
                writePunctuation(".");
                emit(node.name);
            }
            //
            // Misc
            //
            function emitTemplateSpan(node) {
                emitExpression(node.expression);
                emit(node.literal);
            }
            //
            // Statements
            //
            function emitBlock(node) {
                emitBlockStatements(node, /*forceSingleLine*/ !node.multiLine && isEmptyBlock(node));
            }
            function emitBlockStatements(node, forceSingleLine) {
                emitTokenWithComment(18 /* OpenBraceToken */, node.pos, writePunctuation, /*contextNode*/ node);
                var format = forceSingleLine || ts.getEmitFlags(node) & 1 /* SingleLine */ ? 768 /* SingleLineBlockStatements */ : 129 /* MultiLineBlockStatements */;
                emitList(node, node.statements, format);
                emitTokenWithComment(19 /* CloseBraceToken */, node.statements.end, writePunctuation, /*contextNode*/ node, /*indentLeading*/ !!(format & 1 /* MultiLine */));
            }
            function emitVariableStatement(node) {
                emitModifiers(node, node.modifiers);
                emit(node.declarationList);
                writeTrailingSemicolon();
            }
            function emitEmptyStatement(isEmbeddedStatement) {
                // While most trailing semicolons are possibly insignificant, an embedded "empty"
                // statement is significant and cannot be elided by a trailing-semicolon-omitting writer.
                if (isEmbeddedStatement) {
                    writePunctuation(";");
                }
                else {
                    writeTrailingSemicolon();
                }
            }
            function emitExpressionStatement(node) {
                emitExpression(node.expression);
                // Emit semicolon in non json files
                // or if json file that created synthesized expression(eg.define expression statement when --out and amd code generation)
                if (!ts.isJsonSourceFile(currentSourceFile) || ts.nodeIsSynthesized(node.expression)) {
                    writeTrailingSemicolon();
                }
            }
            function emitIfStatement(node) {
                var openParenPos = emitTokenWithComment(95 /* IfKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emitTokenWithComment(20 /* OpenParenToken */, openParenPos, writePunctuation, node);
                emitExpression(node.expression);
                emitTokenWithComment(21 /* CloseParenToken */, node.expression.end, writePunctuation, node);
                emitEmbeddedStatement(node, node.thenStatement);
                if (node.elseStatement) {
                    writeLineOrSpace(node);
                    emitTokenWithComment(87 /* ElseKeyword */, node.thenStatement.end, writeKeyword, node);
                    if (node.elseStatement.kind === 227 /* IfStatement */) {
                        writeSpace();
                        emit(node.elseStatement);
                    }
                    else {
                        emitEmbeddedStatement(node, node.elseStatement);
                    }
                }
            }
            function emitWhileClause(node, startPos) {
                var openParenPos = emitTokenWithComment(111 /* WhileKeyword */, startPos, writeKeyword, node);
                writeSpace();
                emitTokenWithComment(20 /* OpenParenToken */, openParenPos, writePunctuation, node);
                emitExpression(node.expression);
                emitTokenWithComment(21 /* CloseParenToken */, node.expression.end, writePunctuation, node);
            }
            function emitDoStatement(node) {
                emitTokenWithComment(86 /* DoKeyword */, node.pos, writeKeyword, node);
                emitEmbeddedStatement(node, node.statement);
                if (ts.isBlock(node.statement)) {
                    writeSpace();
                }
                else {
                    writeLineOrSpace(node);
                }
                emitWhileClause(node, node.statement.end);
                writeTrailingSemicolon();
            }
            function emitWhileStatement(node) {
                emitWhileClause(node, node.pos);
                emitEmbeddedStatement(node, node.statement);
            }
            function emitForStatement(node) {
                var openParenPos = emitTokenWithComment(93 /* ForKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                var pos = emitTokenWithComment(20 /* OpenParenToken */, openParenPos, writePunctuation, /*contextNode*/ node);
                emitForBinding(node.initializer);
                pos = emitTokenWithComment(26 /* SemicolonToken */, node.initializer ? node.initializer.end : pos, writePunctuation, node);
                emitExpressionWithLeadingSpace(node.condition);
                pos = emitTokenWithComment(26 /* SemicolonToken */, node.condition ? node.condition.end : pos, writePunctuation, node);
                emitExpressionWithLeadingSpace(node.incrementor);
                emitTokenWithComment(21 /* CloseParenToken */, node.incrementor ? node.incrementor.end : pos, writePunctuation, node);
                emitEmbeddedStatement(node, node.statement);
            }
            function emitForInStatement(node) {
                var openParenPos = emitTokenWithComment(93 /* ForKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emitTokenWithComment(20 /* OpenParenToken */, openParenPos, writePunctuation, node);
                emitForBinding(node.initializer);
                writeSpace();
                emitTokenWithComment(97 /* InKeyword */, node.initializer.end, writeKeyword, node);
                writeSpace();
                emitExpression(node.expression);
                emitTokenWithComment(21 /* CloseParenToken */, node.expression.end, writePunctuation, node);
                emitEmbeddedStatement(node, node.statement);
            }
            function emitForOfStatement(node) {
                var openParenPos = emitTokenWithComment(93 /* ForKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emitWithTrailingSpace(node.awaitModifier);
                emitTokenWithComment(20 /* OpenParenToken */, openParenPos, writePunctuation, node);
                emitForBinding(node.initializer);
                writeSpace();
                emitTokenWithComment(152 /* OfKeyword */, node.initializer.end, writeKeyword, node);
                writeSpace();
                emitExpression(node.expression);
                emitTokenWithComment(21 /* CloseParenToken */, node.expression.end, writePunctuation, node);
                emitEmbeddedStatement(node, node.statement);
            }
            function emitForBinding(node) {
                if (node !== undefined) {
                    if (node.kind === 243 /* VariableDeclarationList */) {
                        emit(node);
                    }
                    else {
                        emitExpression(node);
                    }
                }
            }
            function emitContinueStatement(node) {
                emitTokenWithComment(82 /* ContinueKeyword */, node.pos, writeKeyword, node);
                emitWithLeadingSpace(node.label);
                writeTrailingSemicolon();
            }
            function emitBreakStatement(node) {
                emitTokenWithComment(77 /* BreakKeyword */, node.pos, writeKeyword, node);
                emitWithLeadingSpace(node.label);
                writeTrailingSemicolon();
            }
            function emitTokenWithComment(token, pos, writer, contextNode, indentLeading) {
                var node = ts.getParseTreeNode(contextNode);
                var isSimilarNode = node && node.kind === contextNode.kind;
                var startPos = pos;
                if (isSimilarNode && currentSourceFile) {
                    pos = ts.skipTrivia(currentSourceFile.text, pos);
                }
                if (emitLeadingCommentsOfPosition && isSimilarNode && contextNode.pos !== startPos) {
                    var needsIndent = indentLeading && currentSourceFile && !ts.positionsAreOnSameLine(startPos, pos, currentSourceFile);
                    if (needsIndent) {
                        increaseIndent();
                    }
                    emitLeadingCommentsOfPosition(startPos);
                    if (needsIndent) {
                        decreaseIndent();
                    }
                }
                pos = writeTokenText(token, writer, pos);
                if (emitTrailingCommentsOfPosition && isSimilarNode && contextNode.end !== pos) {
                    emitTrailingCommentsOfPosition(pos, /*prefixSpace*/ true);
                }
                return pos;
            }
            function emitReturnStatement(node) {
                emitTokenWithComment(101 /* ReturnKeyword */, node.pos, writeKeyword, /*contextNode*/ node);
                emitExpressionWithLeadingSpace(node.expression);
                writeTrailingSemicolon();
            }
            function emitWithStatement(node) {
                var openParenPos = emitTokenWithComment(112 /* WithKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emitTokenWithComment(20 /* OpenParenToken */, openParenPos, writePunctuation, node);
                emitExpression(node.expression);
                emitTokenWithComment(21 /* CloseParenToken */, node.expression.end, writePunctuation, node);
                emitEmbeddedStatement(node, node.statement);
            }
            function emitSwitchStatement(node) {
                var openParenPos = emitTokenWithComment(103 /* SwitchKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emitTokenWithComment(20 /* OpenParenToken */, openParenPos, writePunctuation, node);
                emitExpression(node.expression);
                emitTokenWithComment(21 /* CloseParenToken */, node.expression.end, writePunctuation, node);
                writeSpace();
                emit(node.caseBlock);
            }
            function emitLabeledStatement(node) {
                emit(node.label);
                emitTokenWithComment(58 /* ColonToken */, node.label.end, writePunctuation, node);
                writeSpace();
                emit(node.statement);
            }
            function emitThrowStatement(node) {
                emitTokenWithComment(105 /* ThrowKeyword */, node.pos, writeKeyword, node);
                emitExpressionWithLeadingSpace(node.expression);
                writeTrailingSemicolon();
            }
            function emitTryStatement(node) {
                emitTokenWithComment(107 /* TryKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emit(node.tryBlock);
                if (node.catchClause) {
                    writeLineOrSpace(node);
                    emit(node.catchClause);
                }
                if (node.finallyBlock) {
                    writeLineOrSpace(node);
                    emitTokenWithComment(92 /* FinallyKeyword */, (node.catchClause || node.tryBlock).end, writeKeyword, node);
                    writeSpace();
                    emit(node.finallyBlock);
                }
            }
            function emitDebuggerStatement(node) {
                writeToken(83 /* DebuggerKeyword */, node.pos, writeKeyword);
                writeTrailingSemicolon();
            }
            //
            // Declarations
            //
            function emitVariableDeclaration(node) {
                emit(node.name);
                emit(node.exclamationToken);
                emitTypeAnnotation(node.type);
                emitInitializer(node.initializer, node.type ? node.type.end : node.name.end, node);
            }
            function emitVariableDeclarationList(node) {
                writeKeyword(ts.isLet(node) ? "let" : ts.isVarConst(node) ? "const" : "var");
                writeSpace();
                emitList(node, node.declarations, 528 /* VariableDeclarationList */);
            }
            function emitFunctionDeclaration(node) {
                emitFunctionDeclarationOrExpression(node);
            }
            function emitFunctionDeclarationOrExpression(node) {
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                writeKeyword("function");
                emit(node.asteriskToken);
                writeSpace();
                emitIdentifierName(node.name);
                emitSignatureAndBody(node, emitSignatureHead);
            }
            function emitBlockCallback(_hint, body) {
                emitBlockFunctionBody(body);
            }
            function emitSignatureAndBody(node, emitSignatureHead) {
                var body = node.body;
                if (body) {
                    if (ts.isBlock(body)) {
                        var indentedFlag = ts.getEmitFlags(node) & 65536 /* Indented */;
                        if (indentedFlag) {
                            increaseIndent();
                        }
                        pushNameGenerationScope(node);
                        ts.forEach(node.parameters, generateNames);
                        generateNames(node.body);
                        emitSignatureHead(node);
                        if (onEmitNode) {
                            onEmitNode(4 /* Unspecified */, body, emitBlockCallback);
                        }
                        else {
                            emitBlockFunctionBody(body);
                        }
                        popNameGenerationScope(node);
                        if (indentedFlag) {
                            decreaseIndent();
                        }
                    }
                    else {
                        emitSignatureHead(node);
                        writeSpace();
                        emitExpression(body);
                    }
                }
                else {
                    emitSignatureHead(node);
                    writeTrailingSemicolon();
                }
            }
            function emitSignatureHead(node) {
                emitTypeParameters(node, node.typeParameters);
                emitParameters(node, node.parameters);
                emitTypeAnnotation(node.type);
            }
            function shouldEmitBlockFunctionBodyOnSingleLine(body) {
                // We must emit a function body as a single-line body in the following case:
                // * The body has NodeEmitFlags.SingleLine specified.
                // We must emit a function body as a multi-line body in the following cases:
                // * The body is explicitly marked as multi-line.
                // * A non-synthesized body's start and end position are on different lines.
                // * Any statement in the body starts on a new line.
                if (ts.getEmitFlags(body) & 1 /* SingleLine */) {
                    return true;
                }
                if (body.multiLine) {
                    return false;
                }
                if (!ts.nodeIsSynthesized(body) && !ts.rangeIsOnSingleLine(body, currentSourceFile)) {
                    return false;
                }
                if (shouldWriteLeadingLineTerminator(body, body.statements, 2 /* PreserveLines */)
                    || shouldWriteClosingLineTerminator(body, body.statements, 2 /* PreserveLines */)) {
                    return false;
                }
                var previousStatement;
                for (var _a = 0, _b = body.statements; _a < _b.length; _a++) {
                    var statement = _b[_a];
                    if (shouldWriteSeparatingLineTerminator(previousStatement, statement, 2 /* PreserveLines */)) {
                        return false;
                    }
                    previousStatement = statement;
                }
                return true;
            }
            function emitBlockFunctionBody(body) {
                writeSpace();
                writePunctuation("{");
                increaseIndent();
                var emitBlockFunctionBody = shouldEmitBlockFunctionBodyOnSingleLine(body)
                    ? emitBlockFunctionBodyOnSingleLine
                    : emitBlockFunctionBodyWorker;
                if (emitBodyWithDetachedComments) {
                    emitBodyWithDetachedComments(body, body.statements, emitBlockFunctionBody);
                }
                else {
                    emitBlockFunctionBody(body);
                }
                decreaseIndent();
                writeToken(19 /* CloseBraceToken */, body.statements.end, writePunctuation, body);
            }
            function emitBlockFunctionBodyOnSingleLine(body) {
                emitBlockFunctionBodyWorker(body, /*emitBlockFunctionBodyOnSingleLine*/ true);
            }
            function emitBlockFunctionBodyWorker(body, emitBlockFunctionBodyOnSingleLine) {
                // Emit all the prologue directives (like "use strict").
                var statementOffset = emitPrologueDirectives(body.statements);
                var pos = writer.getTextPos();
                emitHelpers(body);
                if (statementOffset === 0 && pos === writer.getTextPos() && emitBlockFunctionBodyOnSingleLine) {
                    decreaseIndent();
                    emitList(body, body.statements, 768 /* SingleLineFunctionBodyStatements */);
                    increaseIndent();
                }
                else {
                    emitList(body, body.statements, 1 /* MultiLineFunctionBodyStatements */, statementOffset);
                }
            }
            function emitClassDeclaration(node) {
                emitClassDeclarationOrExpression(node);
            }
            function emitClassDeclarationOrExpression(node) {
                ts.forEach(node.members, generateMemberNames);
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                writeKeyword("class");
                if (node.name) {
                    writeSpace();
                    emitIdentifierName(node.name);
                }
                var indentedFlag = ts.getEmitFlags(node) & 65536 /* Indented */;
                if (indentedFlag) {
                    increaseIndent();
                }
                emitTypeParameters(node, node.typeParameters);
                emitList(node, node.heritageClauses, 0 /* ClassHeritageClauses */);
                writeSpace();
                writePunctuation("{");
                emitList(node, node.members, 129 /* ClassMembers */);
                writePunctuation("}");
                if (indentedFlag) {
                    decreaseIndent();
                }
            }
            function emitInterfaceDeclaration(node) {
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                writeKeyword("interface");
                writeSpace();
                emit(node.name);
                emitTypeParameters(node, node.typeParameters);
                emitList(node, node.heritageClauses, 512 /* HeritageClauses */);
                writeSpace();
                writePunctuation("{");
                emitList(node, node.members, 129 /* InterfaceMembers */);
                writePunctuation("}");
            }
            function emitTypeAliasDeclaration(node) {
                emitDecorators(node, node.decorators);
                emitModifiers(node, node.modifiers);
                writeKeyword("type");
                writeSpace();
                emit(node.name);
                emitTypeParameters(node, node.typeParameters);
                writeSpace();
                writePunctuation("=");
                writeSpace();
                emit(node.type);
                writeTrailingSemicolon();
            }
            function emitEnumDeclaration(node) {
                emitModifiers(node, node.modifiers);
                writeKeyword("enum");
                writeSpace();
                emit(node.name);
                writeSpace();
                writePunctuation("{");
                emitList(node, node.members, 145 /* EnumMembers */);
                writePunctuation("}");
            }
            function emitModuleDeclaration(node) {
                emitModifiers(node, node.modifiers);
                if (~node.flags & 1024 /* GlobalAugmentation */) {
                    writeKeyword(node.flags & 16 /* Namespace */ ? "namespace" : "module");
                    writeSpace();
                }
                emit(node.name);
                var body = node.body;
                if (!body)
                    return writeTrailingSemicolon();
                while (body.kind === 249 /* ModuleDeclaration */) {
                    writePunctuation(".");
                    emit(body.name);
                    body = body.body;
                }
                writeSpace();
                emit(body);
            }
            function emitModuleBlock(node) {
                pushNameGenerationScope(node);
                ts.forEach(node.statements, generateNames);
                emitBlockStatements(node, /*forceSingleLine*/ isEmptyBlock(node));
                popNameGenerationScope(node);
            }
            function emitCaseBlock(node) {
                emitTokenWithComment(18 /* OpenBraceToken */, node.pos, writePunctuation, node);
                emitList(node, node.clauses, 129 /* CaseBlockClauses */);
                emitTokenWithComment(19 /* CloseBraceToken */, node.clauses.end, writePunctuation, node, /*indentLeading*/ true);
            }
            function emitImportEqualsDeclaration(node) {
                emitModifiers(node, node.modifiers);
                emitTokenWithComment(96 /* ImportKeyword */, node.modifiers ? node.modifiers.end : node.pos, writeKeyword, node);
                writeSpace();
                emit(node.name);
                writeSpace();
                emitTokenWithComment(62 /* EqualsToken */, node.name.end, writePunctuation, node);
                writeSpace();
                emitModuleReference(node.moduleReference);
                writeTrailingSemicolon();
            }
            function emitModuleReference(node) {
                if (node.kind === 75 /* Identifier */) {
                    emitExpression(node);
                }
                else {
                    emit(node);
                }
            }
            function emitImportDeclaration(node) {
                emitModifiers(node, node.modifiers);
                emitTokenWithComment(96 /* ImportKeyword */, node.modifiers ? node.modifiers.end : node.pos, writeKeyword, node);
                writeSpace();
                if (node.importClause) {
                    emit(node.importClause);
                    writeSpace();
                    emitTokenWithComment(149 /* FromKeyword */, node.importClause.end, writeKeyword, node);
                    writeSpace();
                }
                emitExpression(node.moduleSpecifier);
                writeTrailingSemicolon();
            }
            function emitImportClause(node) {
                if (node.isTypeOnly) {
                    emitTokenWithComment(145 /* TypeKeyword */, node.pos, writeKeyword, node);
                    writeSpace();
                }
                emit(node.name);
                if (node.name && node.namedBindings) {
                    emitTokenWithComment(27 /* CommaToken */, node.name.end, writePunctuation, node);
                    writeSpace();
                }
                emit(node.namedBindings);
            }
            function emitNamespaceImport(node) {
                var asPos = emitTokenWithComment(41 /* AsteriskToken */, node.pos, writePunctuation, node);
                writeSpace();
                emitTokenWithComment(123 /* AsKeyword */, asPos, writeKeyword, node);
                writeSpace();
                emit(node.name);
            }
            function emitNamedImports(node) {
                emitNamedImportsOrExports(node);
            }
            function emitImportSpecifier(node) {
                emitImportOrExportSpecifier(node);
            }
            function emitExportAssignment(node) {
                var nextPos = emitTokenWithComment(89 /* ExportKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                if (node.isExportEquals) {
                    emitTokenWithComment(62 /* EqualsToken */, nextPos, writeOperator, node);
                }
                else {
                    emitTokenWithComment(84 /* DefaultKeyword */, nextPos, writeKeyword, node);
                }
                writeSpace();
                emitExpression(node.expression);
                writeTrailingSemicolon();
            }
            function emitExportDeclaration(node) {
                var nextPos = emitTokenWithComment(89 /* ExportKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                if (node.isTypeOnly) {
                    nextPos = emitTokenWithComment(145 /* TypeKeyword */, nextPos, writeKeyword, node);
                    writeSpace();
                }
                if (node.exportClause) {
                    emit(node.exportClause);
                }
                else {
                    nextPos = emitTokenWithComment(41 /* AsteriskToken */, nextPos, writePunctuation, node);
                }
                if (node.moduleSpecifier) {
                    writeSpace();
                    var fromPos = node.exportClause ? node.exportClause.end : nextPos;
                    emitTokenWithComment(149 /* FromKeyword */, fromPos, writeKeyword, node);
                    writeSpace();
                    emitExpression(node.moduleSpecifier);
                }
                writeTrailingSemicolon();
            }
            function emitNamespaceExportDeclaration(node) {
                var nextPos = emitTokenWithComment(89 /* ExportKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                nextPos = emitTokenWithComment(123 /* AsKeyword */, nextPos, writeKeyword, node);
                writeSpace();
                nextPos = emitTokenWithComment(136 /* NamespaceKeyword */, nextPos, writeKeyword, node);
                writeSpace();
                emit(node.name);
                writeTrailingSemicolon();
            }
            function emitNamespaceExport(node) {
                var asPos = emitTokenWithComment(41 /* AsteriskToken */, node.pos, writePunctuation, node);
                writeSpace();
                emitTokenWithComment(123 /* AsKeyword */, asPos, writeKeyword, node);
                writeSpace();
                emit(node.name);
            }
            function emitNamedExports(node) {
                emitNamedImportsOrExports(node);
            }
            function emitExportSpecifier(node) {
                emitImportOrExportSpecifier(node);
            }
            function emitNamedImportsOrExports(node) {
                writePunctuation("{");
                emitList(node, node.elements, 525136 /* NamedImportsOrExportsElements */);
                writePunctuation("}");
            }
            function emitImportOrExportSpecifier(node) {
                if (node.propertyName) {
                    emit(node.propertyName);
                    writeSpace();
                    emitTokenWithComment(123 /* AsKeyword */, node.propertyName.end, writeKeyword, node);
                    writeSpace();
                }
                emit(node.name);
            }
            //
            // Module references
            //
            function emitExternalModuleReference(node) {
                writeKeyword("require");
                writePunctuation("(");
                emitExpression(node.expression);
                writePunctuation(")");
            }
            //
            // JSX
            //
            function emitJsxElement(node) {
                emit(node.openingElement);
                emitList(node, node.children, 262144 /* JsxElementOrFragmentChildren */);
                emit(node.closingElement);
            }
            function emitJsxSelfClosingElement(node) {
                writePunctuation("<");
                emitJsxTagName(node.tagName);
                emitTypeArguments(node, node.typeArguments);
                writeSpace();
                emit(node.attributes);
                writePunctuation("/>");
            }
            function emitJsxFragment(node) {
                emit(node.openingFragment);
                emitList(node, node.children, 262144 /* JsxElementOrFragmentChildren */);
                emit(node.closingFragment);
            }
            function emitJsxOpeningElementOrFragment(node) {
                writePunctuation("<");
                if (ts.isJsxOpeningElement(node)) {
                    emitJsxTagName(node.tagName);
                    emitTypeArguments(node, node.typeArguments);
                    if (node.attributes.properties && node.attributes.properties.length > 0) {
                        writeSpace();
                    }
                    emit(node.attributes);
                }
                writePunctuation(">");
            }
            function emitJsxText(node) {
                writer.writeLiteral(node.text);
            }
            function emitJsxClosingElementOrFragment(node) {
                writePunctuation("</");
                if (ts.isJsxClosingElement(node)) {
                    emitJsxTagName(node.tagName);
                }
                writePunctuation(">");
            }
            function emitJsxAttributes(node) {
                emitList(node, node.properties, 262656 /* JsxElementAttributes */);
            }
            function emitJsxAttribute(node) {
                emit(node.name);
                emitNodeWithPrefix("=", writePunctuation, node.initializer, emitJsxAttributeValue);
            }
            function emitJsxSpreadAttribute(node) {
                writePunctuation("{...");
                emitExpression(node.expression);
                writePunctuation("}");
            }
            function emitJsxExpression(node) {
                if (node.expression) {
                    writePunctuation("{");
                    emit(node.dotDotDotToken);
                    emitExpression(node.expression);
                    writePunctuation("}");
                }
            }
            function emitJsxTagName(node) {
                if (node.kind === 75 /* Identifier */) {
                    emitExpression(node);
                }
                else {
                    emit(node);
                }
            }
            //
            // Clauses
            //
            function emitCaseClause(node) {
                emitTokenWithComment(78 /* CaseKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                emitExpression(node.expression);
                emitCaseOrDefaultClauseRest(node, node.statements, node.expression.end);
            }
            function emitDefaultClause(node) {
                var pos = emitTokenWithComment(84 /* DefaultKeyword */, node.pos, writeKeyword, node);
                emitCaseOrDefaultClauseRest(node, node.statements, pos);
            }
            function emitCaseOrDefaultClauseRest(parentNode, statements, colonPos) {
                var emitAsSingleStatement = statements.length === 1 &&
                    (
                    // treat synthesized nodes as located on the same line for emit purposes
                    ts.nodeIsSynthesized(parentNode) ||
                        ts.nodeIsSynthesized(statements[0]) ||
                        ts.rangeStartPositionsAreOnSameLine(parentNode, statements[0], currentSourceFile));
                var format = 163969 /* CaseOrDefaultClauseStatements */;
                if (emitAsSingleStatement) {
                    writeToken(58 /* ColonToken */, colonPos, writePunctuation, parentNode);
                    writeSpace();
                    format &= ~(1 /* MultiLine */ | 128 /* Indented */);
                }
                else {
                    emitTokenWithComment(58 /* ColonToken */, colonPos, writePunctuation, parentNode);
                }
                emitList(parentNode, statements, format);
            }
            function emitHeritageClause(node) {
                writeSpace();
                writeTokenText(node.token, writeKeyword);
                writeSpace();
                emitList(node, node.types, 528 /* HeritageClauseTypes */);
            }
            function emitCatchClause(node) {
                var openParenPos = emitTokenWithComment(79 /* CatchKeyword */, node.pos, writeKeyword, node);
                writeSpace();
                if (node.variableDeclaration) {
                    emitTokenWithComment(20 /* OpenParenToken */, openParenPos, writePunctuation, node);
                    emit(node.variableDeclaration);
                    emitTokenWithComment(21 /* CloseParenToken */, node.variableDeclaration.end, writePunctuation, node);
                    writeSpace();
                }
                emit(node.block);
            }
            //
            // Property assignments
            //
            function emitPropertyAssignment(node) {
                emit(node.name);
                writePunctuation(":");
                writeSpace();
                // This is to ensure that we emit comment in the following case:
                //      For example:
                //          obj = {
                //              id: /*comment1*/ ()=>void
                //          }
                // "comment1" is not considered to be leading comment for node.initializer
                // but rather a trailing comment on the previous node.
                var initializer = node.initializer;
                if (emitTrailingCommentsOfPosition && (ts.getEmitFlags(initializer) & 512 /* NoLeadingComments */) === 0) {
                    var commentRange = ts.getCommentRange(initializer);
                    emitTrailingCommentsOfPosition(commentRange.pos);
                }
                emitExpression(initializer);
            }
            function emitShorthandPropertyAssignment(node) {
                emit(node.name);
                if (node.objectAssignmentInitializer) {
                    writeSpace();
                    writePunctuation("=");
                    writeSpace();
                    emitExpression(node.objectAssignmentInitializer);
                }
            }
            function emitSpreadAssignment(node) {
                if (node.expression) {
                    emitTokenWithComment(25 /* DotDotDotToken */, node.pos, writePunctuation, node);
                    emitExpression(node.expression);
                }
            }
            //
            // Enum
            //
            function emitEnumMember(node) {
                emit(node.name);
                emitInitializer(node.initializer, node.name.end, node);
            }
            //
            // JSDoc
            //
            function emitJSDoc(node) {
                write("/**");
                if (node.comment) {
                    var lines = node.comment.split(/\r\n?|\n/g);
                    for (var _a = 0, lines_2 = lines; _a < lines_2.length; _a++) {
                        var line = lines_2[_a];
                        writeLine();
                        writeSpace();
                        writePunctuation("*");
                        writeSpace();
                        write(line);
                    }
                }
                if (node.tags) {
                    if (node.tags.length === 1 && node.tags[0].kind === 319 /* JSDocTypeTag */ && !node.comment) {
                        writeSpace();
                        emit(node.tags[0]);
                    }
                    else {
                        emitList(node, node.tags, 33 /* JSDocComment */);
                    }
                }
                writeSpace();
                write("*/");
            }
            function emitJSDocSimpleTypedTag(tag) {
                emitJSDocTagName(tag.tagName);
                emitJSDocTypeExpression(tag.typeExpression);
                emitJSDocComment(tag.comment);
            }
            function emitJSDocAugmentsTag(tag) {
                emitJSDocTagName(tag.tagName);
                writeSpace();
                writePunctuation("{");
                emit(tag.class);
                writePunctuation("}");
                emitJSDocComment(tag.comment);
            }
            function emitJSDocTemplateTag(tag) {
                emitJSDocTagName(tag.tagName);
                emitJSDocTypeExpression(tag.constraint);
                writeSpace();
                emitList(tag, tag.typeParameters, 528 /* CommaListElements */);
                emitJSDocComment(tag.comment);
            }
            function emitJSDocTypedefTag(tag) {
                emitJSDocTagName(tag.tagName);
                if (tag.typeExpression) {
                    if (tag.typeExpression.kind === 294 /* JSDocTypeExpression */) {
                        emitJSDocTypeExpression(tag.typeExpression);
                    }
                    else {
                        writeSpace();
                        writePunctuation("{");
                        write("Object");
                        if (tag.typeExpression.isArrayType) {
                            writePunctuation("[");
                            writePunctuation("]");
                        }
                        writePunctuation("}");
                    }
                }
                if (tag.fullName) {
                    writeSpace();
                    emit(tag.fullName);
                }
                emitJSDocComment(tag.comment);
                if (tag.typeExpression && tag.typeExpression.kind === 304 /* JSDocTypeLiteral */) {
                    emitJSDocTypeLiteral(tag.typeExpression);
                }
            }
            function emitJSDocCallbackTag(tag) {
                emitJSDocTagName(tag.tagName);
                if (tag.name) {
                    writeSpace();
                    emit(tag.name);
                }
                emitJSDocComment(tag.comment);
                emitJSDocSignature(tag.typeExpression);
            }
            function emitJSDocSimpleTag(tag) {
                emitJSDocTagName(tag.tagName);
                emitJSDocComment(tag.comment);
            }
            function emitJSDocTypeLiteral(lit) {
                emitList(lit, ts.createNodeArray(lit.jsDocPropertyTags), 33 /* JSDocComment */);
            }
            function emitJSDocSignature(sig) {
                if (sig.typeParameters) {
                    emitList(sig, ts.createNodeArray(sig.typeParameters), 33 /* JSDocComment */);
                }
                if (sig.parameters) {
                    emitList(sig, ts.createNodeArray(sig.parameters), 33 /* JSDocComment */);
                }
                if (sig.type) {
                    writeLine();
                    writeSpace();
                    writePunctuation("*");
                    writeSpace();
                    emit(sig.type);
                }
            }
            function emitJSDocPropertyLikeTag(param) {
                emitJSDocTagName(param.tagName);
                emitJSDocTypeExpression(param.typeExpression);
                writeSpace();
                if (param.isBracketed) {
                    writePunctuation("[");
                }
                emit(param.name);
                if (param.isBracketed) {
                    writePunctuation("]");
                }
                emitJSDocComment(param.comment);
            }
            function emitJSDocTagName(tagName) {
                writePunctuation("@");
                emit(tagName);
            }
            function emitJSDocComment(comment) {
                if (comment) {
                    writeSpace();
                    write(comment);
                }
            }
            function emitJSDocTypeExpression(typeExpression) {
                if (typeExpression) {
                    writeSpace();
                    writePunctuation("{");
                    emit(typeExpression.type);
                    writePunctuation("}");
                }
            }
            //
            // Top-level nodes
            //
            function emitSourceFile(node) {
                writeLine();
                var statements = node.statements;
                if (emitBodyWithDetachedComments) {
                    // Emit detached comment if there are no prologue directives or if the first node is synthesized.
                    // The synthesized node will have no leading comment so some comments may be missed.
                    var shouldEmitDetachedComment = statements.length === 0 ||
                        !ts.isPrologueDirective(statements[0]) ||
                        ts.nodeIsSynthesized(statements[0]);
                    if (shouldEmitDetachedComment) {
                        emitBodyWithDetachedComments(node, statements, emitSourceFileWorker);
                        return;
                    }
                }
                emitSourceFileWorker(node);
            }
            function emitSyntheticTripleSlashReferencesIfNeeded(node) {
                emitTripleSlashDirectives(!!node.hasNoDefaultLib, node.syntheticFileReferences || [], node.syntheticTypeReferences || [], node.syntheticLibReferences || []);
                for (var _a = 0, _b = node.prepends; _a < _b.length; _a++) {
                    var prepend = _b[_a];
                    if (ts.isUnparsedSource(prepend) && prepend.syntheticReferences) {
                        for (var _c = 0, _d = prepend.syntheticReferences; _c < _d.length; _c++) {
                            var ref = _d[_c];
                            emit(ref);
                            writeLine();
                        }
                    }
                }
            }
            function emitTripleSlashDirectivesIfNeeded(node) {
                if (node.isDeclarationFile)
                    emitTripleSlashDirectives(node.hasNoDefaultLib, node.referencedFiles, node.typeReferenceDirectives, node.libReferenceDirectives);
            }
            function emitTripleSlashDirectives(hasNoDefaultLib, files, types, libs) {
                if (hasNoDefaultLib) {
                    var pos = writer.getTextPos();
                    writeComment("/// <reference no-default-lib=\"true\"/>");
                    if (bundleFileInfo)
                        bundleFileInfo.sections.push({ pos: pos, end: writer.getTextPos(), kind: "no-default-lib" /* NoDefaultLib */ });
                    writeLine();
                }
                if (currentSourceFile && currentSourceFile.moduleName) {
                    writeComment("/// <amd-module name=\"" + currentSourceFile.moduleName + "\" />");
                    writeLine();
                }
                if (currentSourceFile && currentSourceFile.amdDependencies) {
                    for (var _a = 0, _b = currentSourceFile.amdDependencies; _a < _b.length; _a++) {
                        var dep = _b[_a];
                        if (dep.name) {
                            writeComment("/// <amd-dependency name=\"" + dep.name + "\" path=\"" + dep.path + "\" />");
                        }
                        else {
                            writeComment("/// <amd-dependency path=\"" + dep.path + "\" />");
                        }
                        writeLine();
                    }
                }
                for (var _c = 0, files_1 = files; _c < files_1.length; _c++) {
                    var directive = files_1[_c];
                    var pos = writer.getTextPos();
                    writeComment("/// <reference path=\"" + directive.fileName + "\" />");
                    if (bundleFileInfo)
                        bundleFileInfo.sections.push({ pos: pos, end: writer.getTextPos(), kind: "reference" /* Reference */, data: directive.fileName });
                    writeLine();
                }
                for (var _d = 0, types_21 = types; _d < types_21.length; _d++) {
                    var directive = types_21[_d];
                    var pos = writer.getTextPos();
                    writeComment("/// <reference types=\"" + directive.fileName + "\" />");
                    if (bundleFileInfo)
                        bundleFileInfo.sections.push({ pos: pos, end: writer.getTextPos(), kind: "type" /* Type */, data: directive.fileName });
                    writeLine();
                }
                for (var _e = 0, libs_1 = libs; _e < libs_1.length; _e++) {
                    var directive = libs_1[_e];
                    var pos = writer.getTextPos();
                    writeComment("/// <reference lib=\"" + directive.fileName + "\" />");
                    if (bundleFileInfo)
                        bundleFileInfo.sections.push({ pos: pos, end: writer.getTextPos(), kind: "lib" /* Lib */, data: directive.fileName });
                    writeLine();
                }
            }
            function emitSourceFileWorker(node) {
                var statements = node.statements;
                pushNameGenerationScope(node);
                ts.forEach(node.statements, generateNames);
                emitHelpers(node);
                var index = ts.findIndex(statements, function (statement) { return !ts.isPrologueDirective(statement); });
                emitTripleSlashDirectivesIfNeeded(node);
                emitList(node, statements, 1 /* MultiLine */, index === -1 ? statements.length : index);
                popNameGenerationScope(node);
            }
            // Transformation nodes
            function emitPartiallyEmittedExpression(node) {
                emitExpression(node.expression);
            }
            function emitCommaList(node) {
                emitExpressionList(node, node.elements, 528 /* CommaListElements */);
            }
            /**
             * Emits any prologue directives at the start of a Statement list, returning the
             * number of prologue directives written to the output.
             */
            function emitPrologueDirectives(statements, sourceFile, seenPrologueDirectives, recordBundleFileSection) {
                var needsToSetSourceFile = !!sourceFile;
                for (var i = 0; i < statements.length; i++) {
                    var statement = statements[i];
                    if (ts.isPrologueDirective(statement)) {
                        var shouldEmitPrologueDirective = seenPrologueDirectives ? !seenPrologueDirectives.has(statement.expression.text) : true;
                        if (shouldEmitPrologueDirective) {
                            if (needsToSetSourceFile) {
                                needsToSetSourceFile = false;
                                setSourceFile(sourceFile);
                            }
                            writeLine();
                            var pos = writer.getTextPos();
                            emit(statement);
                            if (recordBundleFileSection && bundleFileInfo)
                                bundleFileInfo.sections.push({ pos: pos, end: writer.getTextPos(), kind: "prologue" /* Prologue */, data: statement.expression.text });
                            if (seenPrologueDirectives) {
                                seenPrologueDirectives.set(statement.expression.text, true);
                            }
                        }
                    }
                    else {
                        // return index of the first non prologue directive
                        return i;
                    }
                }
                return statements.length;
            }
            function emitUnparsedPrologues(prologues, seenPrologueDirectives) {
                for (var _a = 0, prologues_1 = prologues; _a < prologues_1.length; _a++) {
                    var prologue = prologues_1[_a];
                    if (!seenPrologueDirectives.has(prologue.data)) {
                        writeLine();
                        var pos = writer.getTextPos();
                        emit(prologue);
                        if (bundleFileInfo)
                            bundleFileInfo.sections.push({ pos: pos, end: writer.getTextPos(), kind: "prologue" /* Prologue */, data: prologue.data });
                        if (seenPrologueDirectives) {
                            seenPrologueDirectives.set(prologue.data, true);
                        }
                    }
                }
            }
            function emitPrologueDirectivesIfNeeded(sourceFileOrBundle) {
                if (ts.isSourceFile(sourceFileOrBundle)) {
                    emitPrologueDirectives(sourceFileOrBundle.statements, sourceFileOrBundle);
                }
                else {
                    var seenPrologueDirectives = ts.createMap();
                    for (var _a = 0, _b = sourceFileOrBundle.prepends; _a < _b.length; _a++) {
                        var prepend = _b[_a];
                        emitUnparsedPrologues(prepend.prologues, seenPrologueDirectives);
                    }
                    for (var _c = 0, _d = sourceFileOrBundle.sourceFiles; _c < _d.length; _c++) {
                        var sourceFile = _d[_c];
                        emitPrologueDirectives(sourceFile.statements, sourceFile, seenPrologueDirectives, /*recordBundleFileSection*/ true);
                    }
                    setSourceFile(undefined);
                }
            }
            function getPrologueDirectivesFromBundledSourceFiles(bundle) {
                var seenPrologueDirectives = ts.createMap();
                var prologues;
                for (var index = 0; index < bundle.sourceFiles.length; index++) {
                    var sourceFile = bundle.sourceFiles[index];
                    var directives = void 0;
                    var end = 0;
                    for (var _a = 0, _b = sourceFile.statements; _a < _b.length; _a++) {
                        var statement = _b[_a];
                        if (!ts.isPrologueDirective(statement))
                            break;
                        if (seenPrologueDirectives.has(statement.expression.text))
                            continue;
                        seenPrologueDirectives.set(statement.expression.text, true);
                        (directives || (directives = [])).push({
                            pos: statement.pos,
                            end: statement.end,
                            expression: {
                                pos: statement.expression.pos,
                                end: statement.expression.end,
                                text: statement.expression.text
                            }
                        });
                        end = end < statement.end ? statement.end : end;
                    }
                    if (directives)
                        (prologues || (prologues = [])).push({ file: index, text: sourceFile.text.substring(0, end), directives: directives });
                }
                return prologues;
            }
            function emitShebangIfNeeded(sourceFileOrBundle) {
                if (ts.isSourceFile(sourceFileOrBundle) || ts.isUnparsedSource(sourceFileOrBundle)) {
                    var shebang = ts.getShebang(sourceFileOrBundle.text);
                    if (shebang) {
                        writeComment(shebang);
                        writeLine();
                        return true;
                    }
                }
                else {
                    for (var _a = 0, _b = sourceFileOrBundle.prepends; _a < _b.length; _a++) {
                        var prepend = _b[_a];
                        ts.Debug.assertNode(prepend, ts.isUnparsedSource);
                        if (emitShebangIfNeeded(prepend)) {
                            return true;
                        }
                    }
                    for (var _c = 0, _d = sourceFileOrBundle.sourceFiles; _c < _d.length; _c++) {
                        var sourceFile = _d[_c];
                        // Emit only the first encountered shebang
                        if (emitShebangIfNeeded(sourceFile)) {
                            return true;
                        }
                    }
                }
            }
            //
            // Helpers
            //
            function emitNodeWithWriter(node, writer) {
                if (!node)
                    return;
                var savedWrite = write;
                write = writer;
                emit(node);
                write = savedWrite;
            }
            function emitModifiers(node, modifiers) {
                if (modifiers && modifiers.length) {
                    emitList(node, modifiers, 262656 /* Modifiers */);
                    writeSpace();
                }
            }
            function emitTypeAnnotation(node) {
                if (node) {
                    writePunctuation(":");
                    writeSpace();
                    emit(node);
                }
            }
            function emitInitializer(node, equalCommentStartPos, container) {
                if (node) {
                    writeSpace();
                    emitTokenWithComment(62 /* EqualsToken */, equalCommentStartPos, writeOperator, container);
                    writeSpace();
                    emitExpression(node);
                }
            }
            function emitNodeWithPrefix(prefix, prefixWriter, node, emit) {
                if (node) {
                    prefixWriter(prefix);
                    emit(node);
                }
            }
            function emitWithLeadingSpace(node) {
                if (node) {
                    writeSpace();
                    emit(node);
                }
            }
            function emitExpressionWithLeadingSpace(node) {
                if (node) {
                    writeSpace();
                    emitExpression(node);
                }
            }
            function emitWithTrailingSpace(node) {
                if (node) {
                    emit(node);
                    writeSpace();
                }
            }
            function emitEmbeddedStatement(parent, node) {
                if (ts.isBlock(node) || ts.getEmitFlags(parent) & 1 /* SingleLine */) {
                    writeSpace();
                    emit(node);
                }
                else {
                    writeLine();
                    increaseIndent();
                    if (ts.isEmptyStatement(node)) {
                        pipelineEmit(5 /* EmbeddedStatement */, node);
                    }
                    else {
                        emit(node);
                    }
                    decreaseIndent();
                }
            }
            function emitDecorators(parentNode, decorators) {
                emitList(parentNode, decorators, 49153 /* Decorators */);
            }
            function emitTypeArguments(parentNode, typeArguments) {
                emitList(parentNode, typeArguments, 53776 /* TypeArguments */);
            }
            function emitTypeParameters(parentNode, typeParameters) {
                if (ts.isFunctionLike(parentNode) && parentNode.typeArguments) { // Quick info uses type arguments in place of type parameters on instantiated signatures
                    return emitTypeArguments(parentNode, parentNode.typeArguments);
                }
                emitList(parentNode, typeParameters, 53776 /* TypeParameters */);
            }
            function emitParameters(parentNode, parameters) {
                emitList(parentNode, parameters, 2576 /* Parameters */);
            }
            function canEmitSimpleArrowHead(parentNode, parameters) {
                var parameter = ts.singleOrUndefined(parameters);
                return parameter
                    && parameter.pos === parentNode.pos // may not have parsed tokens between parent and parameter
                    && ts.isArrowFunction(parentNode) // only arrow functions may have simple arrow head
                    && !parentNode.type // arrow function may not have return type annotation
                    && !ts.some(parentNode.decorators) // parent may not have decorators
                    && !ts.some(parentNode.modifiers) // parent may not have modifiers
                    && !ts.some(parentNode.typeParameters) // parent may not have type parameters
                    && !ts.some(parameter.decorators) // parameter may not have decorators
                    && !ts.some(parameter.modifiers) // parameter may not have modifiers
                    && !parameter.dotDotDotToken // parameter may not be rest
                    && !parameter.questionToken // parameter may not be optional
                    && !parameter.type // parameter may not have a type annotation
                    && !parameter.initializer // parameter may not have an initializer
                    && ts.isIdentifier(parameter.name); // parameter name must be identifier
            }
            function emitParametersForArrow(parentNode, parameters) {
                if (canEmitSimpleArrowHead(parentNode, parameters)) {
                    emitList(parentNode, parameters, 2576 /* Parameters */ & ~2048 /* Parenthesis */);
                }
                else {
                    emitParameters(parentNode, parameters);
                }
            }
            function emitParametersForIndexSignature(parentNode, parameters) {
                emitList(parentNode, parameters, 8848 /* IndexSignatureParameters */);
            }
            function emitList(parentNode, children, format, start, count) {
                emitNodeList(emit, parentNode, children, format, start, count);
            }
            function emitExpressionList(parentNode, children, format, start, count) {
                emitNodeList(emitExpression, parentNode, children, format, start, count); // TODO: GH#18217
            }
            function writeDelimiter(format) {
                switch (format & 60 /* DelimitersMask */) {
                    case 0 /* None */:
                        break;
                    case 16 /* CommaDelimited */:
                        writePunctuation(",");
                        break;
                    case 4 /* BarDelimited */:
                        writeSpace();
                        writePunctuation("|");
                        break;
                    case 32 /* AsteriskDelimited */:
                        writeSpace();
                        writePunctuation("*");
                        writeSpace();
                        break;
                    case 8 /* AmpersandDelimited */:
                        writeSpace();
                        writePunctuation("&");
                        break;
                }
            }
            function emitNodeList(emit, parentNode, children, format, start, count) {
                if (start === void 0) {
                    start = 0;
                }
                if (count === void 0) {
                    count = children ? children.length - start : 0;
                }
                var isUndefined = children === undefined;
                if (isUndefined && format & 16384 /* OptionalIfUndefined */) {
                    return;
                }
                var isEmpty = children === undefined || start >= children.length || count === 0;
                if (isEmpty && format & 32768 /* OptionalIfEmpty */) {
                    if (onBeforeEmitNodeArray) {
                        onBeforeEmitNodeArray(children);
                    }
                    if (onAfterEmitNodeArray) {
                        onAfterEmitNodeArray(children);
                    }
                    return;
                }
                if (format & 15360 /* BracketsMask */) {
                    writePunctuation(getOpeningBracket(format));
                    if (isEmpty && !isUndefined) {
                        // TODO: GH#18217
                        emitTrailingCommentsOfPosition(children.pos, /*prefixSpace*/ true); // Emit comments within empty bracketed lists
                    }
                }
                if (onBeforeEmitNodeArray) {
                    onBeforeEmitNodeArray(children);
                }
                if (isEmpty) {
                    // Write a line terminator if the parent node was multi-line
                    if (format & 1 /* MultiLine */) {
                        writeLine();
                    }
                    else if (format & 256 /* SpaceBetweenBraces */ && !(format & 524288 /* NoSpaceIfEmpty */)) {
                        writeSpace();
                    }
                }
                else {
                    // Write the opening line terminator or leading whitespace.
                    var mayEmitInterveningComments = (format & 262144 /* NoInterveningComments */) === 0;
                    var shouldEmitInterveningComments = mayEmitInterveningComments;
                    if (shouldWriteLeadingLineTerminator(parentNode, children, format)) { // TODO: GH#18217
                        writeLine();
                        shouldEmitInterveningComments = false;
                    }
                    else if (format & 256 /* SpaceBetweenBraces */) {
                        writeSpace();
                    }
                    // Increase the indent, if requested.
                    if (format & 128 /* Indented */) {
                        increaseIndent();
                    }
                    // Emit each child.
                    var previousSibling = void 0;
                    var previousSourceFileTextKind = void 0;
                    var shouldDecreaseIndentAfterEmit = false;
                    for (var i = 0; i < count; i++) {
                        var child = children[start + i];
                        // Write the delimiter if this is not the first node.
                        if (format & 32 /* AsteriskDelimited */) {
                            // always write JSDoc in the format "\n *"
                            writeLine();
                            writeDelimiter(format);
                        }
                        else if (previousSibling) {
                            // i.e
                            //      function commentedParameters(
                            //          /* Parameter a */
                            //          a
                            //          /* End of parameter a */ -> this comment isn't considered to be trailing comment of parameter "a" due to newline
                            //          ,
                            if (format & 60 /* DelimitersMask */ && previousSibling.end !== parentNode.end) {
                                emitLeadingCommentsOfPosition(previousSibling.end);
                            }
                            writeDelimiter(format);
                            recordBundleFileInternalSectionEnd(previousSourceFileTextKind);
                            // Write either a line terminator or whitespace to separate the elements.
                            if (shouldWriteSeparatingLineTerminator(previousSibling, child, format)) {
                                // If a synthesized node in a single-line list starts on a new
                                // line, we should increase the indent.
                                if ((format & (3 /* LinesMask */ | 128 /* Indented */)) === 0 /* SingleLine */) {
                                    increaseIndent();
                                    shouldDecreaseIndentAfterEmit = true;
                                }
                                writeLine();
                                shouldEmitInterveningComments = false;
                            }
                            else if (previousSibling && format & 512 /* SpaceBetweenSiblings */) {
                                writeSpace();
                            }
                        }
                        // Emit this child.
                        previousSourceFileTextKind = recordBundleFileInternalSectionStart(child);
                        if (shouldEmitInterveningComments) {
                            if (emitTrailingCommentsOfPosition) {
                                var commentRange = ts.getCommentRange(child);
                                emitTrailingCommentsOfPosition(commentRange.pos);
                            }
                        }
                        else {
                            shouldEmitInterveningComments = mayEmitInterveningComments;
                        }
                        emit(child);
                        if (shouldDecreaseIndentAfterEmit) {
                            decreaseIndent();
                            shouldDecreaseIndentAfterEmit = false;
                        }
                        previousSibling = child;
                    }
                    // Write a trailing comma, if requested.
                    var hasTrailingComma = (format & 64 /* AllowTrailingComma */) && children.hasTrailingComma;
                    if (format & 16 /* CommaDelimited */ && hasTrailingComma) {
                        writePunctuation(",");
                    }
                    // Emit any trailing comment of the last element in the list
                    // i.e
                    //       var array = [...
                    //          2
                    //          /* end of element 2 */
                    //       ];
                    if (previousSibling && format & 60 /* DelimitersMask */ && previousSibling.end !== parentNode.end && !(ts.getEmitFlags(previousSibling) & 1024 /* NoTrailingComments */)) {
                        emitLeadingCommentsOfPosition(previousSibling.end);
                    }
                    // Decrease the indent, if requested.
                    if (format & 128 /* Indented */) {
                        decreaseIndent();
                    }
                    recordBundleFileInternalSectionEnd(previousSourceFileTextKind);
                    // Write the closing line terminator or closing whitespace.
                    if (shouldWriteClosingLineTerminator(parentNode, children, format)) {
                        writeLine();
                    }
                    else if (format & 256 /* SpaceBetweenBraces */) {
                        writeSpace();
                    }
                }
                if (onAfterEmitNodeArray) {
                    onAfterEmitNodeArray(children);
                }
                if (format & 15360 /* BracketsMask */) {
                    if (isEmpty && !isUndefined) {
                        // TODO: GH#18217
                        emitLeadingCommentsOfPosition(children.end); // Emit leading comments within empty lists
                    }
                    writePunctuation(getClosingBracket(format));
                }
            }
            // Writers
            function writeLiteral(s) {
                writer.writeLiteral(s);
            }
            function writeStringLiteral(s) {
                writer.writeStringLiteral(s);
            }
            function writeBase(s) {
                writer.write(s);
            }
            function writeSymbol(s, sym) {
                writer.writeSymbol(s, sym);
            }
            function writePunctuation(s) {
                writer.writePunctuation(s);
            }
            function writeTrailingSemicolon() {
                writer.writeTrailingSemicolon(";");
            }
            function writeKeyword(s) {
                writer.writeKeyword(s);
            }
            function writeOperator(s) {
                writer.writeOperator(s);
            }
            function writeParameter(s) {
                writer.writeParameter(s);
            }
            function writeComment(s) {
                writer.writeComment(s);
            }
            function writeSpace() {
                writer.writeSpace(" ");
            }
            function writeProperty(s) {
                writer.writeProperty(s);
            }
            function writeLine() {
                writer.writeLine();
            }
            function increaseIndent() {
                writer.increaseIndent();
            }
            function decreaseIndent() {
                writer.decreaseIndent();
            }
            function writeToken(token, pos, writer, contextNode) {
                return !sourceMapsDisabled
                    ? emitTokenWithSourceMap(contextNode, token, writer, pos, writeTokenText)
                    : writeTokenText(token, writer, pos);
            }
            function writeTokenNode(node, writer) {
                if (onBeforeEmitToken) {
                    onBeforeEmitToken(node);
                }
                writer(ts.tokenToString(node.kind));
                if (onAfterEmitToken) {
                    onAfterEmitToken(node);
                }
            }
            function writeTokenText(token, writer, pos) {
                var tokenString = ts.tokenToString(token);
                writer(tokenString);
                return pos < 0 ? pos : pos + tokenString.length;
            }
            function writeLineOrSpace(node) {
                if (ts.getEmitFlags(node) & 1 /* SingleLine */) {
                    writeSpace();
                }
                else {
                    writeLine();
                }
            }
            function writeLines(text) {
                var lines = text.split(/\r\n?|\n/g);
                var indentation = ts.guessIndentation(lines);
                for (var _a = 0, lines_3 = lines; _a < lines_3.length; _a++) {
                    var lineText = lines_3[_a];
                    var line = indentation ? lineText.slice(indentation) : lineText;
                    if (line.length) {
                        writeLine();
                        write(line);
                    }
                }
            }
            function increaseIndentIf(value, writeSpaceIfNotIndenting) {
                if (value) {
                    increaseIndent();
                    writeLine();
                }
                else if (writeSpaceIfNotIndenting) {
                    writeSpace();
                }
            }
            // Helper function to decrease the indent if we previously indented.  Allows multiple
            // previous indent values to be considered at a time.  This also allows caller to just
            // call this once, passing in all their appropriate indent values, instead of needing
            // to call this helper function multiple times.
            function decreaseIndentIf(value1, value2) {
                if (value1) {
                    decreaseIndent();
                }
                if (value2) {
                    decreaseIndent();
                }
            }
            function shouldWriteLeadingLineTerminator(parentNode, children, format) {
                if (format & 1 /* MultiLine */) {
                    return true;
                }
                if (format & 2 /* PreserveLines */) {
                    if (format & 65536 /* PreferNewLine */) {
                        return true;
                    }
                    var firstChild = children[0];
                    if (firstChild === undefined) {
                        return !ts.rangeIsOnSingleLine(parentNode, currentSourceFile);
                    }
                    else if (ts.positionIsSynthesized(parentNode.pos) || ts.nodeIsSynthesized(firstChild)) {
                        return synthesizedNodeStartsOnNewLine(firstChild, format);
                    }
                    else {
                        return !ts.rangeStartPositionsAreOnSameLine(parentNode, firstChild, currentSourceFile);
                    }
                }
                else {
                    return false;
                }
            }
            function shouldWriteSeparatingLineTerminator(previousNode, nextNode, format) {
                if (format & 1 /* MultiLine */) {
                    return true;
                }
                else if (format & 2 /* PreserveLines */) {
                    if (previousNode === undefined || nextNode === undefined) {
                        return false;
                    }
                    else if (ts.nodeIsSynthesized(previousNode) || ts.nodeIsSynthesized(nextNode)) {
                        return synthesizedNodeStartsOnNewLine(previousNode, format) || synthesizedNodeStartsOnNewLine(nextNode, format);
                    }
                    else {
                        return !ts.rangeEndIsOnSameLineAsRangeStart(previousNode, nextNode, currentSourceFile);
                    }
                }
                else {
                    return ts.getStartsOnNewLine(nextNode);
                }
            }
            function shouldWriteClosingLineTerminator(parentNode, children, format) {
                if (format & 1 /* MultiLine */) {
                    return (format & 131072 /* NoTrailingNewLine */) === 0;
                }
                else if (format & 2 /* PreserveLines */) {
                    if (format & 65536 /* PreferNewLine */) {
                        return true;
                    }
                    var lastChild = ts.lastOrUndefined(children);
                    if (lastChild === undefined) {
                        return !ts.rangeIsOnSingleLine(parentNode, currentSourceFile);
                    }
                    else if (ts.positionIsSynthesized(parentNode.pos) || ts.nodeIsSynthesized(lastChild)) {
                        return synthesizedNodeStartsOnNewLine(lastChild, format);
                    }
                    else {
                        return !ts.rangeEndPositionsAreOnSameLine(parentNode, lastChild, currentSourceFile);
                    }
                }
                else {
                    return false;
                }
            }
            function synthesizedNodeStartsOnNewLine(node, format) {
                if (ts.nodeIsSynthesized(node)) {
                    var startsOnNewLine = ts.getStartsOnNewLine(node);
                    if (startsOnNewLine === undefined) {
                        return (format & 65536 /* PreferNewLine */) !== 0;
                    }
                    return startsOnNewLine;
                }
                return (format & 65536 /* PreferNewLine */) !== 0;
            }
            function needsIndentation(parent, node1, node2) {
                if (ts.getEmitFlags(parent) & 131072 /* NoIndentation */) {
                    return false;
                }
                parent = skipSynthesizedParentheses(parent);
                node1 = skipSynthesizedParentheses(node1);
                node2 = skipSynthesizedParentheses(node2);
                // Always use a newline for synthesized code if the synthesizer desires it.
                if (ts.getStartsOnNewLine(node2)) {
                    return true;
                }
                return !ts.nodeIsSynthesized(parent)
                    && !ts.nodeIsSynthesized(node1)
                    && !ts.nodeIsSynthesized(node2)
                    && !ts.rangeEndIsOnSameLineAsRangeStart(node1, node2, currentSourceFile);
            }
            function isEmptyBlock(block) {
                return block.statements.length === 0
                    && ts.rangeEndIsOnSameLineAsRangeStart(block, block, currentSourceFile);
            }
            function skipSynthesizedParentheses(node) {
                while (node.kind === 200 /* ParenthesizedExpression */ && ts.nodeIsSynthesized(node)) {
                    node = node.expression;
                }
                return node;
            }
            function getTextOfNode(node, includeTrivia) {
                if (ts.isGeneratedIdentifier(node)) {
                    return generateName(node);
                }
                else if ((ts.isIdentifier(node) || ts.isPrivateIdentifier(node)) && (ts.nodeIsSynthesized(node) || !node.parent || !currentSourceFile || (node.parent && currentSourceFile && ts.getSourceFileOfNode(node) !== ts.getOriginalNode(currentSourceFile)))) {
                    return ts.idText(node);
                }
                else if (node.kind === 10 /* StringLiteral */ && node.textSourceNode) {
                    return getTextOfNode(node.textSourceNode, includeTrivia);
                }
                else if (ts.isLiteralExpression(node) && (ts.nodeIsSynthesized(node) || !node.parent)) {
                    return node.text;
                }
                return ts.getSourceTextOfNodeFromSourceFile(currentSourceFile, node, includeTrivia);
            }
            function getLiteralTextOfNode(node, neverAsciiEscape, jsxAttributeEscape) {
                if (node.kind === 10 /* StringLiteral */ && node.textSourceNode) {
                    var textSourceNode = node.textSourceNode;
                    if (ts.isIdentifier(textSourceNode)) {
                        return jsxAttributeEscape ? "\"" + ts.escapeJsxAttributeString(getTextOfNode(textSourceNode)) + "\"" :
                            neverAsciiEscape || (ts.getEmitFlags(node) & 16777216 /* NoAsciiEscaping */) ? "\"" + ts.escapeString(getTextOfNode(textSourceNode)) + "\"" :
                                "\"" + ts.escapeNonAsciiString(getTextOfNode(textSourceNode)) + "\"";
                    }
                    else {
                        return getLiteralTextOfNode(textSourceNode, neverAsciiEscape, jsxAttributeEscape);
                    }
                }
                return ts.getLiteralText(node, currentSourceFile, neverAsciiEscape, jsxAttributeEscape);
            }
            /**
             * Push a new name generation scope.
             */
            function pushNameGenerationScope(node) {
                if (node && ts.getEmitFlags(node) & 524288 /* ReuseTempVariableScope */) {
                    return;
                }
                tempFlagsStack.push(tempFlags);
                tempFlags = 0;
                reservedNamesStack.push(reservedNames);
            }
            /**
             * Pop the current name generation scope.
             */
            function popNameGenerationScope(node) {
                if (node && ts.getEmitFlags(node) & 524288 /* ReuseTempVariableScope */) {
                    return;
                }
                tempFlags = tempFlagsStack.pop();
                reservedNames = reservedNamesStack.pop();
            }
            function reserveNameInNestedScopes(name) {
                if (!reservedNames || reservedNames === ts.lastOrUndefined(reservedNamesStack)) {
                    reservedNames = ts.createMap();
                }
                reservedNames.set(name, true);
            }
            function generateNames(node) {
                if (!node)
                    return;
                switch (node.kind) {
                    case 223 /* Block */:
                        ts.forEach(node.statements, generateNames);
                        break;
                    case 238 /* LabeledStatement */:
                    case 236 /* WithStatement */:
                    case 228 /* DoStatement */:
                    case 229 /* WhileStatement */:
                        generateNames(node.statement);
                        break;
                    case 227 /* IfStatement */:
                        generateNames(node.thenStatement);
                        generateNames(node.elseStatement);
                        break;
                    case 230 /* ForStatement */:
                    case 232 /* ForOfStatement */:
                    case 231 /* ForInStatement */:
                        generateNames(node.initializer);
                        generateNames(node.statement);
                        break;
                    case 237 /* SwitchStatement */:
                        generateNames(node.caseBlock);
                        break;
                    case 251 /* CaseBlock */:
                        ts.forEach(node.clauses, generateNames);
                        break;
                    case 277 /* CaseClause */:
                    case 278 /* DefaultClause */:
                        ts.forEach(node.statements, generateNames);
                        break;
                    case 240 /* TryStatement */:
                        generateNames(node.tryBlock);
                        generateNames(node.catchClause);
                        generateNames(node.finallyBlock);
                        break;
                    case 280 /* CatchClause */:
                        generateNames(node.variableDeclaration);
                        generateNames(node.block);
                        break;
                    case 225 /* VariableStatement */:
                        generateNames(node.declarationList);
                        break;
                    case 243 /* VariableDeclarationList */:
                        ts.forEach(node.declarations, generateNames);
                        break;
                    case 242 /* VariableDeclaration */:
                    case 156 /* Parameter */:
                    case 191 /* BindingElement */:
                    case 245 /* ClassDeclaration */:
                        generateNameIfNeeded(node.name);
                        break;
                    case 244 /* FunctionDeclaration */:
                        generateNameIfNeeded(node.name);
                        if (ts.getEmitFlags(node) & 524288 /* ReuseTempVariableScope */) {
                            ts.forEach(node.parameters, generateNames);
                            generateNames(node.body);
                        }
                        break;
                    case 189 /* ObjectBindingPattern */:
                    case 190 /* ArrayBindingPattern */:
                        ts.forEach(node.elements, generateNames);
                        break;
                    case 254 /* ImportDeclaration */:
                        generateNames(node.importClause);
                        break;
                    case 255 /* ImportClause */:
                        generateNameIfNeeded(node.name);
                        generateNames(node.namedBindings);
                        break;
                    case 256 /* NamespaceImport */:
                        generateNameIfNeeded(node.name);
                        break;
                    case 262 /* NamespaceExport */:
                        generateNameIfNeeded(node.name);
                        break;
                    case 257 /* NamedImports */:
                        ts.forEach(node.elements, generateNames);
                        break;
                    case 258 /* ImportSpecifier */:
                        generateNameIfNeeded(node.propertyName || node.name);
                        break;
                }
            }
            function generateMemberNames(node) {
                if (!node)
                    return;
                switch (node.kind) {
                    case 281 /* PropertyAssignment */:
                    case 282 /* ShorthandPropertyAssignment */:
                    case 159 /* PropertyDeclaration */:
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        generateNameIfNeeded(node.name);
                        break;
                }
            }
            function generateNameIfNeeded(name) {
                if (name) {
                    if (ts.isGeneratedIdentifier(name)) {
                        generateName(name);
                    }
                    else if (ts.isBindingPattern(name)) {
                        generateNames(name);
                    }
                }
            }
            /**
             * Generate the text for a generated identifier.
             */
            function generateName(name) {
                if ((name.autoGenerateFlags & 7 /* KindMask */) === 4 /* Node */) {
                    // Node names generate unique names based on their original node
                    // and are cached based on that node's id.
                    return generateNameCached(getNodeForGeneratedName(name), name.autoGenerateFlags);
                }
                else {
                    // Auto, Loop, and Unique names are cached based on their unique
                    // autoGenerateId.
                    var autoGenerateId = name.autoGenerateId;
                    return autoGeneratedIdToGeneratedName[autoGenerateId] || (autoGeneratedIdToGeneratedName[autoGenerateId] = makeName(name));
                }
            }
            function generateNameCached(node, flags) {
                var nodeId = ts.getNodeId(node);
                return nodeIdToGeneratedName[nodeId] || (nodeIdToGeneratedName[nodeId] = generateNameForNode(node, flags));
            }
            /**
             * Returns a value indicating whether a name is unique globally, within the current file,
             * or within the NameGenerator.
             */
            function isUniqueName(name) {
                return isFileLevelUniqueName(name)
                    && !generatedNames.has(name)
                    && !(reservedNames && reservedNames.has(name));
            }
            /**
             * Returns a value indicating whether a name is unique globally or within the current file.
             */
            function isFileLevelUniqueName(name) {
                return currentSourceFile ? ts.isFileLevelUniqueName(currentSourceFile, name, hasGlobalName) : true;
            }
            /**
             * Returns a value indicating whether a name is unique within a container.
             */
            function isUniqueLocalName(name, container) {
                for (var node = container; ts.isNodeDescendantOf(node, container); node = node.nextContainer) {
                    if (node.locals) {
                        var local = node.locals.get(ts.escapeLeadingUnderscores(name));
                        // We conservatively include alias symbols to cover cases where they're emitted as locals
                        if (local && local.flags & (111551 /* Value */ | 1048576 /* ExportValue */ | 2097152 /* Alias */)) {
                            return false;
                        }
                    }
                }
                return true;
            }
            /**
             * Return the next available name in the pattern _a ... _z, _0, _1, ...
             * TempFlags._i or TempFlags._n may be used to express a preference for that dedicated name.
             * Note that names generated by makeTempVariableName and makeUniqueName will never conflict.
             */
            function makeTempVariableName(flags, reservedInNestedScopes) {
                if (flags && !(tempFlags & flags)) {
                    var name = flags === 268435456 /* _i */ ? "_i" : "_n";
                    if (isUniqueName(name)) {
                        tempFlags |= flags;
                        if (reservedInNestedScopes) {
                            reserveNameInNestedScopes(name);
                        }
                        return name;
                    }
                }
                while (true) {
                    var count = tempFlags & 268435455 /* CountMask */;
                    tempFlags++;
                    // Skip over 'i' and 'n'
                    if (count !== 8 && count !== 13) {
                        var name = count < 26
                            ? "_" + String.fromCharCode(97 /* a */ + count)
                            : "_" + (count - 26);
                        if (isUniqueName(name)) {
                            if (reservedInNestedScopes) {
                                reserveNameInNestedScopes(name);
                            }
                            return name;
                        }
                    }
                }
            }
            /**
             * Generate a name that is unique within the current file and doesn't conflict with any names
             * in global scope. The name is formed by adding an '_n' suffix to the specified base name,
             * where n is a positive integer. Note that names generated by makeTempVariableName and
             * makeUniqueName are guaranteed to never conflict.
             * If `optimistic` is set, the first instance will use 'baseName' verbatim instead of 'baseName_1'
             */
            function makeUniqueName(baseName, checkFn, optimistic, scoped) {
                if (checkFn === void 0) {
                    checkFn = isUniqueName;
                }
                if (optimistic) {
                    if (checkFn(baseName)) {
                        if (scoped) {
                            reserveNameInNestedScopes(baseName);
                        }
                        else {
                            generatedNames.set(baseName, true);
                        }
                        return baseName;
                    }
                }
                // Find the first unique 'name_n', where n is a positive number
                if (baseName.charCodeAt(baseName.length - 1) !== 95 /* _ */) {
                    baseName += "_";
                }
                var i = 1;
                while (true) {
                    var generatedName = baseName + i;
                    if (checkFn(generatedName)) {
                        if (scoped) {
                            reserveNameInNestedScopes(generatedName);
                        }
                        else {
                            generatedNames.set(generatedName, true);
                        }
                        return generatedName;
                    }
                    i++;
                }
            }
            function makeFileLevelOptimisticUniqueName(name) {
                return makeUniqueName(name, isFileLevelUniqueName, /*optimistic*/ true);
            }
            /**
             * Generates a unique name for a ModuleDeclaration or EnumDeclaration.
             */
            function generateNameForModuleOrEnum(node) {
                var name = getTextOfNode(node.name);
                // Use module/enum name itself if it is unique, otherwise make a unique variation
                return isUniqueLocalName(name, node) ? name : makeUniqueName(name);
            }
            /**
             * Generates a unique name for an ImportDeclaration or ExportDeclaration.
             */
            function generateNameForImportOrExportDeclaration(node) {
                var expr = ts.getExternalModuleName(node); // TODO: GH#18217
                var baseName = ts.isStringLiteral(expr) ?
                    ts.makeIdentifierFromModuleName(expr.text) : "module";
                return makeUniqueName(baseName);
            }
            /**
             * Generates a unique name for a default export.
             */
            function generateNameForExportDefault() {
                return makeUniqueName("default");
            }
            /**
             * Generates a unique name for a class expression.
             */
            function generateNameForClassExpression() {
                return makeUniqueName("class");
            }
            function generateNameForMethodOrAccessor(node) {
                if (ts.isIdentifier(node.name)) {
                    return generateNameCached(node.name);
                }
                return makeTempVariableName(0 /* Auto */);
            }
            /**
             * Generates a unique name from a node.
             */
            function generateNameForNode(node, flags) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                        return makeUniqueName(getTextOfNode(node), isUniqueName, !!(flags & 16 /* Optimistic */), !!(flags & 8 /* ReservedInNestedScopes */));
                    case 249 /* ModuleDeclaration */:
                    case 248 /* EnumDeclaration */:
                        return generateNameForModuleOrEnum(node);
                    case 254 /* ImportDeclaration */:
                    case 260 /* ExportDeclaration */:
                        return generateNameForImportOrExportDeclaration(node);
                    case 244 /* FunctionDeclaration */:
                    case 245 /* ClassDeclaration */:
                    case 259 /* ExportAssignment */:
                        return generateNameForExportDefault();
                    case 214 /* ClassExpression */:
                        return generateNameForClassExpression();
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        return generateNameForMethodOrAccessor(node);
                    case 154 /* ComputedPropertyName */:
                        return makeTempVariableName(0 /* Auto */, /*reserveInNestedScopes*/ true);
                    default:
                        return makeTempVariableName(0 /* Auto */);
                }
            }
            /**
             * Generates a unique identifier for a node.
             */
            function makeName(name) {
                switch (name.autoGenerateFlags & 7 /* KindMask */) {
                    case 1 /* Auto */:
                        return makeTempVariableName(0 /* Auto */, !!(name.autoGenerateFlags & 8 /* ReservedInNestedScopes */));
                    case 2 /* Loop */:
                        return makeTempVariableName(268435456 /* _i */, !!(name.autoGenerateFlags & 8 /* ReservedInNestedScopes */));
                    case 3 /* Unique */:
                        return makeUniqueName(ts.idText(name), (name.autoGenerateFlags & 32 /* FileLevel */) ? isFileLevelUniqueName : isUniqueName, !!(name.autoGenerateFlags & 16 /* Optimistic */), !!(name.autoGenerateFlags & 8 /* ReservedInNestedScopes */));
                }
                return ts.Debug.fail("Unsupported GeneratedIdentifierKind.");
            }
            /**
             * Gets the node from which a name should be generated.
             */
            function getNodeForGeneratedName(name) {
                var autoGenerateId = name.autoGenerateId;
                var node = name;
                var original = node.original;
                while (original) {
                    node = original;
                    // if "node" is a different generated name (having a different
                    // "autoGenerateId"), use it and stop traversing.
                    if (ts.isIdentifier(node)
                        && !!(node.autoGenerateFlags & 4 /* Node */)
                        && node.autoGenerateId !== autoGenerateId) {
                        break;
                    }
                    original = node.original;
                }
                // otherwise, return the original node for the source;
                return node;
            }
            // Comments
            function pipelineEmitWithComments(hint, node) {
                ts.Debug.assert(lastNode === node || lastSubstitution === node);
                enterComment();
                hasWrittenComment = false;
                var emitFlags = ts.getEmitFlags(node);
                var _a = ts.getCommentRange(node), pos = _a.pos, end = _a.end;
                var isEmittedNode = node.kind !== 324 /* NotEmittedStatement */;
                // We have to explicitly check that the node is JsxText because if the compilerOptions.jsx is "preserve" we will not do any transformation.
                // It is expensive to walk entire tree just to set one kind of node to have no comments.
                var skipLeadingComments = pos < 0 || (emitFlags & 512 /* NoLeadingComments */) !== 0 || node.kind === 11 /* JsxText */;
                var skipTrailingComments = end < 0 || (emitFlags & 1024 /* NoTrailingComments */) !== 0 || node.kind === 11 /* JsxText */;
                // Save current container state on the stack.
                var savedContainerPos = containerPos;
                var savedContainerEnd = containerEnd;
                var savedDeclarationListContainerEnd = declarationListContainerEnd;
                if ((pos > 0 || end > 0) && pos !== end) {
                    // Emit leading comments if the position is not synthesized and the node
                    // has not opted out from emitting leading comments.
                    if (!skipLeadingComments) {
                        emitLeadingComments(pos, isEmittedNode);
                    }
                    if (!skipLeadingComments || (pos >= 0 && (emitFlags & 512 /* NoLeadingComments */) !== 0)) {
                        // Advance the container position if comments get emitted or if they've been disabled explicitly using NoLeadingComments.
                        containerPos = pos;
                    }
                    if (!skipTrailingComments || (end >= 0 && (emitFlags & 1024 /* NoTrailingComments */) !== 0)) {
                        // As above.
                        containerEnd = end;
                        // To avoid invalid comment emit in a down-level binding pattern, we
                        // keep track of the last declaration list container's end
                        if (node.kind === 243 /* VariableDeclarationList */) {
                            declarationListContainerEnd = end;
                        }
                    }
                }
                ts.forEach(ts.getSyntheticLeadingComments(node), emitLeadingSynthesizedComment);
                exitComment();
                var pipelinePhase = getNextPipelinePhase(2 /* Comments */, hint, node);
                if (emitFlags & 2048 /* NoNestedComments */) {
                    commentsDisabled = true;
                    pipelinePhase(hint, node);
                    commentsDisabled = false;
                }
                else {
                    pipelinePhase(hint, node);
                }
                enterComment();
                ts.forEach(ts.getSyntheticTrailingComments(node), emitTrailingSynthesizedComment);
                if ((pos > 0 || end > 0) && pos !== end) {
                    // Restore previous container state.
                    containerPos = savedContainerPos;
                    containerEnd = savedContainerEnd;
                    declarationListContainerEnd = savedDeclarationListContainerEnd;
                    // Emit trailing comments if the position is not synthesized and the node
                    // has not opted out from emitting leading comments and is an emitted node.
                    if (!skipTrailingComments && isEmittedNode) {
                        emitTrailingComments(end);
                    }
                }
                exitComment();
                ts.Debug.assert(lastNode === node || lastSubstitution === node);
            }
            function emitLeadingSynthesizedComment(comment) {
                if (comment.kind === 2 /* SingleLineCommentTrivia */) {
                    writer.writeLine();
                }
                writeSynthesizedComment(comment);
                if (comment.hasTrailingNewLine || comment.kind === 2 /* SingleLineCommentTrivia */) {
                    writer.writeLine();
                }
                else {
                    writer.writeSpace(" ");
                }
            }
            function emitTrailingSynthesizedComment(comment) {
                if (!writer.isAtStartOfLine()) {
                    writer.writeSpace(" ");
                }
                writeSynthesizedComment(comment);
                if (comment.hasTrailingNewLine) {
                    writer.writeLine();
                }
            }
            function writeSynthesizedComment(comment) {
                var text = formatSynthesizedComment(comment);
                var lineMap = comment.kind === 3 /* MultiLineCommentTrivia */ ? ts.computeLineStarts(text) : undefined;
                ts.writeCommentRange(text, lineMap, writer, 0, text.length, newLine);
            }
            function formatSynthesizedComment(comment) {
                return comment.kind === 3 /* MultiLineCommentTrivia */
                    ? "/*" + comment.text + "*/"
                    : "//" + comment.text;
            }
            function emitBodyWithDetachedComments(node, detachedRange, emitCallback) {
                enterComment();
                var pos = detachedRange.pos, end = detachedRange.end;
                var emitFlags = ts.getEmitFlags(node);
                var skipLeadingComments = pos < 0 || (emitFlags & 512 /* NoLeadingComments */) !== 0;
                var skipTrailingComments = commentsDisabled || end < 0 || (emitFlags & 1024 /* NoTrailingComments */) !== 0;
                if (!skipLeadingComments) {
                    emitDetachedCommentsAndUpdateCommentsInfo(detachedRange);
                }
                exitComment();
                if (emitFlags & 2048 /* NoNestedComments */ && !commentsDisabled) {
                    commentsDisabled = true;
                    emitCallback(node);
                    commentsDisabled = false;
                }
                else {
                    emitCallback(node);
                }
                enterComment();
                if (!skipTrailingComments) {
                    emitLeadingComments(detachedRange.end, /*isEmittedNode*/ true);
                    if (hasWrittenComment && !writer.isAtStartOfLine()) {
                        writer.writeLine();
                    }
                }
                exitComment();
            }
            function emitLeadingComments(pos, isEmittedNode) {
                hasWrittenComment = false;
                if (isEmittedNode) {
                    forEachLeadingCommentToEmit(pos, emitLeadingComment);
                }
                else if (pos === 0) {
                    // If the node will not be emitted in JS, remove all the comments(normal, pinned and ///) associated with the node,
                    // unless it is a triple slash comment at the top of the file.
                    // For Example:
                    //      /// <reference-path ...>
                    //      declare var x;
                    //      /// <reference-path ...>
                    //      interface F {}
                    //  The first /// will NOT be removed while the second one will be removed even though both node will not be emitted
                    forEachLeadingCommentToEmit(pos, emitTripleSlashLeadingComment);
                }
            }
            function emitTripleSlashLeadingComment(commentPos, commentEnd, kind, hasTrailingNewLine, rangePos) {
                if (isTripleSlashComment(commentPos, commentEnd)) {
                    emitLeadingComment(commentPos, commentEnd, kind, hasTrailingNewLine, rangePos);
                }
            }
            function shouldWriteComment(text, pos) {
                if (printerOptions.onlyPrintJsDocStyle) {
                    return (ts.isJSDocLikeText(text, pos) || ts.isPinnedComment(text, pos));
                }
                return true;
            }
            function emitLeadingComment(commentPos, commentEnd, kind, hasTrailingNewLine, rangePos) {
                if (!shouldWriteComment(currentSourceFile.text, commentPos))
                    return;
                if (!hasWrittenComment) {
                    ts.emitNewLineBeforeLeadingCommentOfPosition(getCurrentLineMap(), writer, rangePos, commentPos);
                    hasWrittenComment = true;
                }
                // Leading comments are emitted at /*leading comment1 */space/*leading comment*/space
                emitPos(commentPos);
                ts.writeCommentRange(currentSourceFile.text, getCurrentLineMap(), writer, commentPos, commentEnd, newLine);
                emitPos(commentEnd);
                if (hasTrailingNewLine) {
                    writer.writeLine();
                }
                else if (kind === 3 /* MultiLineCommentTrivia */) {
                    writer.writeSpace(" ");
                }
            }
            function emitLeadingCommentsOfPosition(pos) {
                if (commentsDisabled || pos === -1) {
                    return;
                }
                emitLeadingComments(pos, /*isEmittedNode*/ true);
            }
            function emitTrailingComments(pos) {
                forEachTrailingCommentToEmit(pos, emitTrailingComment);
            }
            function emitTrailingComment(commentPos, commentEnd, _kind, hasTrailingNewLine) {
                if (!shouldWriteComment(currentSourceFile.text, commentPos))
                    return;
                // trailing comments are emitted at space/*trailing comment1 */space/*trailing comment2*/
                if (!writer.isAtStartOfLine()) {
                    writer.writeSpace(" ");
                }
                emitPos(commentPos);
                ts.writeCommentRange(currentSourceFile.text, getCurrentLineMap(), writer, commentPos, commentEnd, newLine);
                emitPos(commentEnd);
                if (hasTrailingNewLine) {
                    writer.writeLine();
                }
            }
            function emitTrailingCommentsOfPosition(pos, prefixSpace) {
                if (commentsDisabled) {
                    return;
                }
                enterComment();
                forEachTrailingCommentToEmit(pos, prefixSpace ? emitTrailingComment : emitTrailingCommentOfPosition);
                exitComment();
            }
            function emitTrailingCommentOfPosition(commentPos, commentEnd, _kind, hasTrailingNewLine) {
                // trailing comments of a position are emitted at /*trailing comment1 */space/*trailing comment*/space
                emitPos(commentPos);
                ts.writeCommentRange(currentSourceFile.text, getCurrentLineMap(), writer, commentPos, commentEnd, newLine);
                emitPos(commentEnd);
                if (hasTrailingNewLine) {
                    writer.writeLine();
                }
                else {
                    writer.writeSpace(" ");
                }
            }
            function forEachLeadingCommentToEmit(pos, cb) {
                // Emit the leading comments only if the container's pos doesn't match because the container should take care of emitting these comments
                if (currentSourceFile && (containerPos === -1 || pos !== containerPos)) {
                    if (hasDetachedComments(pos)) {
                        forEachLeadingCommentWithoutDetachedComments(cb);
                    }
                    else {
                        ts.forEachLeadingCommentRange(currentSourceFile.text, pos, cb, /*state*/ pos);
                    }
                }
            }
            function forEachTrailingCommentToEmit(end, cb) {
                // Emit the trailing comments only if the container's end doesn't match because the container should take care of emitting these comments
                if (currentSourceFile && (containerEnd === -1 || (end !== containerEnd && end !== declarationListContainerEnd))) {
                    ts.forEachTrailingCommentRange(currentSourceFile.text, end, cb);
                }
            }
            function hasDetachedComments(pos) {
                return detachedCommentsInfo !== undefined && ts.last(detachedCommentsInfo).nodePos === pos;
            }
            function forEachLeadingCommentWithoutDetachedComments(cb) {
                // get the leading comments from detachedPos
                var pos = ts.last(detachedCommentsInfo).detachedCommentEndPos;
                if (detachedCommentsInfo.length - 1) {
                    detachedCommentsInfo.pop();
                }
                else {
                    detachedCommentsInfo = undefined;
                }
                ts.forEachLeadingCommentRange(currentSourceFile.text, pos, cb, /*state*/ pos);
            }
            function emitDetachedCommentsAndUpdateCommentsInfo(range) {
                var currentDetachedCommentInfo = ts.emitDetachedComments(currentSourceFile.text, getCurrentLineMap(), writer, emitComment, range, newLine, commentsDisabled);
                if (currentDetachedCommentInfo) {
                    if (detachedCommentsInfo) {
                        detachedCommentsInfo.push(currentDetachedCommentInfo);
                    }
                    else {
                        detachedCommentsInfo = [currentDetachedCommentInfo];
                    }
                }
            }
            function emitComment(text, lineMap, writer, commentPos, commentEnd, newLine) {
                if (!shouldWriteComment(currentSourceFile.text, commentPos))
                    return;
                emitPos(commentPos);
                ts.writeCommentRange(text, lineMap, writer, commentPos, commentEnd, newLine);
                emitPos(commentEnd);
            }
            /**
             * Determine if the given comment is a triple-slash
             *
             * @return true if the comment is a triple-slash comment else false
             */
            function isTripleSlashComment(commentPos, commentEnd) {
                return ts.isRecognizedTripleSlashComment(currentSourceFile.text, commentPos, commentEnd);
            }
            // Source Maps
            function getParsedSourceMap(node) {
                if (node.parsedSourceMap === undefined && node.sourceMapText !== undefined) {
                    node.parsedSourceMap = ts.tryParseRawSourceMap(node.sourceMapText) || false;
                }
                return node.parsedSourceMap || undefined;
            }
            function pipelineEmitWithSourceMap(hint, node) {
                ts.Debug.assert(lastNode === node || lastSubstitution === node);
                var pipelinePhase = getNextPipelinePhase(3 /* SourceMaps */, hint, node);
                if (ts.isUnparsedSource(node) || ts.isUnparsedPrepend(node)) {
                    pipelinePhase(hint, node);
                }
                else if (ts.isUnparsedNode(node)) {
                    var parsed = getParsedSourceMap(node.parent);
                    if (parsed && sourceMapGenerator) {
                        sourceMapGenerator.appendSourceMap(writer.getLine(), writer.getColumn(), parsed, node.parent.sourceMapPath, node.parent.getLineAndCharacterOfPosition(node.pos), node.parent.getLineAndCharacterOfPosition(node.end));
                    }
                    pipelinePhase(hint, node);
                }
                else {
                    var _a = ts.getSourceMapRange(node), pos = _a.pos, end = _a.end, _b = _a.source, source = _b === void 0 ? sourceMapSource : _b;
                    var emitFlags = ts.getEmitFlags(node);
                    if (node.kind !== 324 /* NotEmittedStatement */
                        && (emitFlags & 16 /* NoLeadingSourceMap */) === 0
                        && pos >= 0) {
                        emitSourcePos(source, skipSourceTrivia(source, pos));
                    }
                    if (emitFlags & 64 /* NoNestedSourceMaps */) {
                        sourceMapsDisabled = true;
                        pipelinePhase(hint, node);
                        sourceMapsDisabled = false;
                    }
                    else {
                        pipelinePhase(hint, node);
                    }
                    if (node.kind !== 324 /* NotEmittedStatement */
                        && (emitFlags & 32 /* NoTrailingSourceMap */) === 0
                        && end >= 0) {
                        emitSourcePos(source, end);
                    }
                }
                ts.Debug.assert(lastNode === node || lastSubstitution === node);
            }
            /**
             * Skips trivia such as comments and white-space that can be optionally overridden by the source-map source
             */
            function skipSourceTrivia(source, pos) {
                return source.skipTrivia ? source.skipTrivia(pos) : ts.skipTrivia(source.text, pos);
            }
            /**
             * Emits a mapping.
             *
             * If the position is synthetic (undefined or a negative value), no mapping will be
             * created.
             *
             * @param pos The position.
             */
            function emitPos(pos) {
                if (sourceMapsDisabled || ts.positionIsSynthesized(pos) || isJsonSourceMapSource(sourceMapSource)) {
                    return;
                }
                var _a = ts.getLineAndCharacterOfPosition(sourceMapSource, pos), sourceLine = _a.line, sourceCharacter = _a.character;
                sourceMapGenerator.addMapping(writer.getLine(), writer.getColumn(), sourceMapSourceIndex, sourceLine, sourceCharacter, 
                /*nameIndex*/ undefined);
            }
            function emitSourcePos(source, pos) {
                if (source !== sourceMapSource) {
                    var savedSourceMapSource = sourceMapSource;
                    setSourceMapSource(source);
                    emitPos(pos);
                    setSourceMapSource(savedSourceMapSource);
                }
                else {
                    emitPos(pos);
                }
            }
            /**
             * Emits a token of a node with possible leading and trailing source maps.
             *
             * @param node The node containing the token.
             * @param token The token to emit.
             * @param tokenStartPos The start pos of the token.
             * @param emitCallback The callback used to emit the token.
             */
            function emitTokenWithSourceMap(node, token, writer, tokenPos, emitCallback) {
                if (sourceMapsDisabled || node && ts.isInJsonFile(node)) {
                    return emitCallback(token, writer, tokenPos);
                }
                var emitNode = node && node.emitNode;
                var emitFlags = emitNode && emitNode.flags || 0 /* None */;
                var range = emitNode && emitNode.tokenSourceMapRanges && emitNode.tokenSourceMapRanges[token];
                var source = range && range.source || sourceMapSource;
                tokenPos = skipSourceTrivia(source, range ? range.pos : tokenPos);
                if ((emitFlags & 128 /* NoTokenLeadingSourceMaps */) === 0 && tokenPos >= 0) {
                    emitSourcePos(source, tokenPos);
                }
                tokenPos = emitCallback(token, writer, tokenPos);
                if (range)
                    tokenPos = range.end;
                if ((emitFlags & 256 /* NoTokenTrailingSourceMaps */) === 0 && tokenPos >= 0) {
                    emitSourcePos(source, tokenPos);
                }
                return tokenPos;
            }
            function setSourceMapSource(source) {
                if (sourceMapsDisabled) {
                    return;
                }
                sourceMapSource = source;
                if (isJsonSourceMapSource(source)) {
                    return;
                }
                sourceMapSourceIndex = sourceMapGenerator.addSource(source.fileName);
                if (printerOptions.inlineSources) {
                    sourceMapGenerator.setSourceContent(sourceMapSourceIndex, source.text);
                }
            }
            function isJsonSourceMapSource(sourceFile) {
                return ts.fileExtensionIs(sourceFile.fileName, ".json" /* Json */);
            }
        }
        ts.createPrinter = createPrinter;
        function createBracketsMap() {
            var brackets = [];
            brackets[1024 /* Braces */] = ["{", "}"];
            brackets[2048 /* Parenthesis */] = ["(", ")"];
            brackets[4096 /* AngleBrackets */] = ["<", ">"];
            brackets[8192 /* SquareBrackets */] = ["[", "]"];
            return brackets;
        }
        function getOpeningBracket(format) {
            return brackets[format & 15360 /* BracketsMask */][0];
        }
        function getClosingBracket(format) {
            return brackets[format & 15360 /* BracketsMask */][1];
        }
        // Flags enum to track count of temp variables and a few dedicated names
        var TempFlags;
        (function (TempFlags) {
            TempFlags[TempFlags["Auto"] = 0] = "Auto";
            TempFlags[TempFlags["CountMask"] = 268435455] = "CountMask";
            TempFlags[TempFlags["_i"] = 268435456] = "_i";
        })(TempFlags || (TempFlags = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        function createCachedDirectoryStructureHost(host, currentDirectory, useCaseSensitiveFileNames) {
            if (!host.getDirectories || !host.readDirectory) {
                return undefined;
            }
            var cachedReadDirectoryResult = ts.createMap();
            var getCanonicalFileName = ts.createGetCanonicalFileName(useCaseSensitiveFileNames);
            return {
                useCaseSensitiveFileNames: useCaseSensitiveFileNames,
                fileExists: fileExists,
                readFile: function (path, encoding) { return host.readFile(path, encoding); },
                directoryExists: host.directoryExists && directoryExists,
                getDirectories: getDirectories,
                readDirectory: readDirectory,
                createDirectory: host.createDirectory && createDirectory,
                writeFile: host.writeFile && writeFile,
                addOrDeleteFileOrDirectory: addOrDeleteFileOrDirectory,
                addOrDeleteFile: addOrDeleteFile,
                clearCache: clearCache,
                realpath: host.realpath && realpath
            };
            function toPath(fileName) {
                return ts.toPath(fileName, currentDirectory, getCanonicalFileName);
            }
            function getCachedFileSystemEntries(rootDirPath) {
                return cachedReadDirectoryResult.get(ts.ensureTrailingDirectorySeparator(rootDirPath));
            }
            function getCachedFileSystemEntriesForBaseDir(path) {
                return getCachedFileSystemEntries(ts.getDirectoryPath(path));
            }
            function getBaseNameOfFileName(fileName) {
                return ts.getBaseFileName(ts.normalizePath(fileName));
            }
            function createCachedFileSystemEntries(rootDir, rootDirPath) {
                var resultFromHost = {
                    files: ts.map(host.readDirectory(rootDir, /*extensions*/ undefined, /*exclude*/ undefined, /*include*/ ["*.*"]), getBaseNameOfFileName) || [],
                    directories: host.getDirectories(rootDir) || []
                };
                cachedReadDirectoryResult.set(ts.ensureTrailingDirectorySeparator(rootDirPath), resultFromHost);
                return resultFromHost;
            }
            /**
             * If the readDirectory result was already cached, it returns that
             * Otherwise gets result from host and caches it.
             * The host request is done under try catch block to avoid caching incorrect result
             */
            function tryReadDirectory(rootDir, rootDirPath) {
                rootDirPath = ts.ensureTrailingDirectorySeparator(rootDirPath);
                var cachedResult = getCachedFileSystemEntries(rootDirPath);
                if (cachedResult) {
                    return cachedResult;
                }
                try {
                    return createCachedFileSystemEntries(rootDir, rootDirPath);
                }
                catch (_e) {
                    // If there is exception to read directories, dont cache the result and direct the calls to host
                    ts.Debug.assert(!cachedReadDirectoryResult.has(ts.ensureTrailingDirectorySeparator(rootDirPath)));
                    return undefined;
                }
            }
            function fileNameEqual(name1, name2) {
                return getCanonicalFileName(name1) === getCanonicalFileName(name2);
            }
            function hasEntry(entries, name) {
                return ts.some(entries, function (file) { return fileNameEqual(file, name); });
            }
            function updateFileSystemEntry(entries, baseName, isValid) {
                if (hasEntry(entries, baseName)) {
                    if (!isValid) {
                        return ts.filterMutate(entries, function (entry) { return !fileNameEqual(entry, baseName); });
                    }
                }
                else if (isValid) {
                    return entries.push(baseName);
                }
            }
            function writeFile(fileName, data, writeByteOrderMark) {
                var path = toPath(fileName);
                var result = getCachedFileSystemEntriesForBaseDir(path);
                if (result) {
                    updateFilesOfFileSystemEntry(result, getBaseNameOfFileName(fileName), /*fileExists*/ true);
                }
                return host.writeFile(fileName, data, writeByteOrderMark);
            }
            function fileExists(fileName) {
                var path = toPath(fileName);
                var result = getCachedFileSystemEntriesForBaseDir(path);
                return result && hasEntry(result.files, getBaseNameOfFileName(fileName)) ||
                    host.fileExists(fileName);
            }
            function directoryExists(dirPath) {
                var path = toPath(dirPath);
                return cachedReadDirectoryResult.has(ts.ensureTrailingDirectorySeparator(path)) || host.directoryExists(dirPath);
            }
            function createDirectory(dirPath) {
                var path = toPath(dirPath);
                var result = getCachedFileSystemEntriesForBaseDir(path);
                var baseFileName = getBaseNameOfFileName(dirPath);
                if (result) {
                    updateFileSystemEntry(result.directories, baseFileName, /*isValid*/ true);
                }
                host.createDirectory(dirPath);
            }
            function getDirectories(rootDir) {
                var rootDirPath = toPath(rootDir);
                var result = tryReadDirectory(rootDir, rootDirPath);
                if (result) {
                    return result.directories.slice();
                }
                return host.getDirectories(rootDir);
            }
            function readDirectory(rootDir, extensions, excludes, includes, depth) {
                var rootDirPath = toPath(rootDir);
                var result = tryReadDirectory(rootDir, rootDirPath);
                if (result) {
                    return ts.matchFiles(rootDir, extensions, excludes, includes, useCaseSensitiveFileNames, currentDirectory, depth, getFileSystemEntries, realpath);
                }
                return host.readDirectory(rootDir, extensions, excludes, includes, depth);
                function getFileSystemEntries(dir) {
                    var path = toPath(dir);
                    if (path === rootDirPath) {
                        return result;
                    }
                    return tryReadDirectory(dir, path) || ts.emptyFileSystemEntries;
                }
            }
            function realpath(s) {
                return host.realpath ? host.realpath(s) : s;
            }
            function addOrDeleteFileOrDirectory(fileOrDirectory, fileOrDirectoryPath) {
                var existingResult = getCachedFileSystemEntries(fileOrDirectoryPath);
                if (existingResult) {
                    // Just clear the cache for now
                    // For now just clear the cache, since this could mean that multiple level entries might need to be re-evaluated
                    clearCache();
                    return undefined;
                }
                var parentResult = getCachedFileSystemEntriesForBaseDir(fileOrDirectoryPath);
                if (!parentResult) {
                    return undefined;
                }
                // This was earlier a file (hence not in cached directory contents)
                // or we never cached the directory containing it
                if (!host.directoryExists) {
                    // Since host doesnt support directory exists, clear the cache as otherwise it might not be same
                    clearCache();
                    return undefined;
                }
                var baseName = getBaseNameOfFileName(fileOrDirectory);
                var fsQueryResult = {
                    fileExists: host.fileExists(fileOrDirectoryPath),
                    directoryExists: host.directoryExists(fileOrDirectoryPath)
                };
                if (fsQueryResult.directoryExists || hasEntry(parentResult.directories, baseName)) {
                    // Folder added or removed, clear the cache instead of updating the folder and its structure
                    clearCache();
                }
                else {
                    // No need to update the directory structure, just files
                    updateFilesOfFileSystemEntry(parentResult, baseName, fsQueryResult.fileExists);
                }
                return fsQueryResult;
            }
            function addOrDeleteFile(fileName, filePath, eventKind) {
                if (eventKind === ts.FileWatcherEventKind.Changed) {
                    return;
                }
                var parentResult = getCachedFileSystemEntriesForBaseDir(filePath);
                if (parentResult) {
                    updateFilesOfFileSystemEntry(parentResult, getBaseNameOfFileName(fileName), eventKind === ts.FileWatcherEventKind.Created);
                }
            }
            function updateFilesOfFileSystemEntry(parentResult, baseName, fileExists) {
                updateFileSystemEntry(parentResult.files, baseName, fileExists);
            }
            function clearCache() {
                cachedReadDirectoryResult.clear();
            }
        }
        ts.createCachedDirectoryStructureHost = createCachedDirectoryStructureHost;
        var ConfigFileProgramReloadLevel;
        (function (ConfigFileProgramReloadLevel) {
            ConfigFileProgramReloadLevel[ConfigFileProgramReloadLevel["None"] = 0] = "None";
            /** Update the file name list from the disk */
            ConfigFileProgramReloadLevel[ConfigFileProgramReloadLevel["Partial"] = 1] = "Partial";
            /** Reload completely by re-reading contents of config file from disk and updating program */
            ConfigFileProgramReloadLevel[ConfigFileProgramReloadLevel["Full"] = 2] = "Full";
        })(ConfigFileProgramReloadLevel = ts.ConfigFileProgramReloadLevel || (ts.ConfigFileProgramReloadLevel = {}));
        /**
         * Updates the existing missing file watches with the new set of missing files after new program is created
         */
        function updateMissingFilePathsWatch(program, missingFileWatches, createMissingFileWatch) {
            var missingFilePaths = program.getMissingFilePaths();
            var newMissingFilePathMap = ts.arrayToSet(missingFilePaths);
            // Update the missing file paths watcher
            ts.mutateMap(missingFileWatches, newMissingFilePathMap, {
                // Watch the missing files
                createNewValue: createMissingFileWatch,
                // Files that are no longer missing (e.g. because they are no longer required)
                // should no longer be watched.
                onDeleteValue: ts.closeFileWatcher
            });
        }
        ts.updateMissingFilePathsWatch = updateMissingFilePathsWatch;
        /**
         * Updates the existing wild card directory watches with the new set of wild card directories from the config file
         * after new program is created because the config file was reloaded or program was created first time from the config file
         * Note that there is no need to call this function when the program is updated with additional files without reloading config files,
         * as wildcard directories wont change unless reloading config file
         */
        function updateWatchingWildcardDirectories(existingWatchedForWildcards, wildcardDirectories, watchDirectory) {
            ts.mutateMap(existingWatchedForWildcards, wildcardDirectories, {
                // Create new watch and recursive info
                createNewValue: createWildcardDirectoryWatcher,
                // Close existing watch thats not needed any more
                onDeleteValue: closeFileWatcherOf,
                // Close existing watch that doesnt match in the flags
                onExistingValue: updateWildcardDirectoryWatcher
            });
            function createWildcardDirectoryWatcher(directory, flags) {
                // Create new watch and recursive info
                return {
                    watcher: watchDirectory(directory, flags),
                    flags: flags
                };
            }
            function updateWildcardDirectoryWatcher(existingWatcher, flags, directory) {
                // Watcher needs to be updated if the recursive flags dont match
                if (existingWatcher.flags === flags) {
                    return;
                }
                existingWatcher.watcher.close();
                existingWatchedForWildcards.set(directory, createWildcardDirectoryWatcher(directory, flags));
            }
        }
        ts.updateWatchingWildcardDirectories = updateWatchingWildcardDirectories;
        function isEmittedFileOfProgram(program, file) {
            if (!program) {
                return false;
            }
            return program.isEmittedFile(file);
        }
        ts.isEmittedFileOfProgram = isEmittedFileOfProgram;
        var WatchLogLevel;
        (function (WatchLogLevel) {
            WatchLogLevel[WatchLogLevel["None"] = 0] = "None";
            WatchLogLevel[WatchLogLevel["TriggerOnly"] = 1] = "TriggerOnly";
            WatchLogLevel[WatchLogLevel["Verbose"] = 2] = "Verbose";
        })(WatchLogLevel = ts.WatchLogLevel || (ts.WatchLogLevel = {}));
        function getWatchFactory(watchLogLevel, log, getDetailWatchInfo) {
            return getWatchFactoryWith(watchLogLevel, log, getDetailWatchInfo, watchFile, watchDirectory);
        }
        ts.getWatchFactory = getWatchFactory;
        function getWatchFactoryWith(watchLogLevel, log, getDetailWatchInfo, watchFile, watchDirectory) {
            var createFileWatcher = getCreateFileWatcher(watchLogLevel, watchFile);
            var createFilePathWatcher = watchLogLevel === WatchLogLevel.None ? watchFilePath : createFileWatcher;
            var createDirectoryWatcher = getCreateFileWatcher(watchLogLevel, watchDirectory);
            if (watchLogLevel === WatchLogLevel.Verbose && ts.sysLog === ts.noop) {
                ts.setSysLog(function (s) { return log(s); });
            }
            return {
                watchFile: function (host, file, callback, pollingInterval, options, detailInfo1, detailInfo2) {
                    return createFileWatcher(host, file, callback, pollingInterval, options, /*passThrough*/ undefined, detailInfo1, detailInfo2, watchFile, log, "FileWatcher", getDetailWatchInfo);
                },
                watchFilePath: function (host, file, callback, pollingInterval, options, path, detailInfo1, detailInfo2) {
                    return createFilePathWatcher(host, file, callback, pollingInterval, options, path, detailInfo1, detailInfo2, watchFile, log, "FileWatcher", getDetailWatchInfo);
                },
                watchDirectory: function (host, directory, callback, flags, options, detailInfo1, detailInfo2) {
                    return createDirectoryWatcher(host, directory, callback, flags, options, /*passThrough*/ undefined, detailInfo1, detailInfo2, watchDirectory, log, "DirectoryWatcher", getDetailWatchInfo);
                }
            };
        }
        function watchFile(host, file, callback, pollingInterval, options) {
            return host.watchFile(file, callback, pollingInterval, options);
        }
        function watchFilePath(host, file, callback, pollingInterval, options, path) {
            return watchFile(host, file, function (fileName, eventKind) { return callback(fileName, eventKind, path); }, pollingInterval, options);
        }
        function watchDirectory(host, directory, callback, flags, options) {
            return host.watchDirectory(directory, callback, (flags & 1 /* Recursive */) !== 0, options);
        }
        function getCreateFileWatcher(watchLogLevel, addWatch) {
            switch (watchLogLevel) {
                case WatchLogLevel.None:
                    return addWatch;
                case WatchLogLevel.TriggerOnly:
                    return createFileWatcherWithTriggerLogging;
                case WatchLogLevel.Verbose:
                    return addWatch === watchDirectory ? createDirectoryWatcherWithLogging : createFileWatcherWithLogging;
            }
        }
        function createFileWatcherWithLogging(host, file, cb, flags, options, passThrough, detailInfo1, detailInfo2, addWatch, log, watchCaption, getDetailWatchInfo) {
            log(watchCaption + ":: Added:: " + getWatchInfo(file, flags, options, detailInfo1, detailInfo2, getDetailWatchInfo));
            var watcher = createFileWatcherWithTriggerLogging(host, file, cb, flags, options, passThrough, detailInfo1, detailInfo2, addWatch, log, watchCaption, getDetailWatchInfo);
            return {
                close: function () {
                    log(watchCaption + ":: Close:: " + getWatchInfo(file, flags, options, detailInfo1, detailInfo2, getDetailWatchInfo));
                    watcher.close();
                }
            };
        }
        function createDirectoryWatcherWithLogging(host, file, cb, flags, options, passThrough, detailInfo1, detailInfo2, addWatch, log, watchCaption, getDetailWatchInfo) {
            var watchInfo = watchCaption + ":: Added:: " + getWatchInfo(file, flags, options, detailInfo1, detailInfo2, getDetailWatchInfo);
            log(watchInfo);
            var start = ts.timestamp();
            var watcher = createFileWatcherWithTriggerLogging(host, file, cb, flags, options, passThrough, detailInfo1, detailInfo2, addWatch, log, watchCaption, getDetailWatchInfo);
            var elapsed = ts.timestamp() - start;
            log("Elapsed:: " + elapsed + "ms " + watchInfo);
            return {
                close: function () {
                    var watchInfo = watchCaption + ":: Close:: " + getWatchInfo(file, flags, options, detailInfo1, detailInfo2, getDetailWatchInfo);
                    log(watchInfo);
                    var start = ts.timestamp();
                    watcher.close();
                    var elapsed = ts.timestamp() - start;
                    log("Elapsed:: " + elapsed + "ms " + watchInfo);
                }
            };
        }
        function createFileWatcherWithTriggerLogging(host, file, cb, flags, options, passThrough, detailInfo1, detailInfo2, addWatch, log, watchCaption, getDetailWatchInfo) {
            return addWatch(host, file, function (fileName, cbOptional) {
                var triggerredInfo = watchCaption + ":: Triggered with " + fileName + " " + (cbOptional !== undefined ? cbOptional : "") + ":: " + getWatchInfo(file, flags, options, detailInfo1, detailInfo2, getDetailWatchInfo);
                log(triggerredInfo);
                var start = ts.timestamp();
                cb(fileName, cbOptional, passThrough);
                var elapsed = ts.timestamp() - start;
                log("Elapsed:: " + elapsed + "ms " + triggerredInfo);
            }, flags, options);
        }
        function getFallbackOptions(options) {
            var fallbackPolling = options === null || options === void 0 ? void 0 : options.fallbackPolling;
            return {
                watchFile: fallbackPolling !== undefined ?
                    fallbackPolling :
                    ts.WatchFileKind.PriorityPollingInterval
            };
        }
        ts.getFallbackOptions = getFallbackOptions;
        function getWatchInfo(file, flags, options, detailInfo1, detailInfo2, getDetailWatchInfo) {
            return "WatchInfo: " + file + " " + flags + " " + JSON.stringify(options) + " " + (getDetailWatchInfo ? getDetailWatchInfo(detailInfo1, detailInfo2) : detailInfo2 === undefined ? detailInfo1 : detailInfo1 + " " + detailInfo2);
        }
        function closeFileWatcherOf(objWithWatcher) {
            objWithWatcher.watcher.close();
        }
        ts.closeFileWatcherOf = closeFileWatcherOf;
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        var ignoreDiagnosticCommentRegEx = /(^\s*$)|(^\s*\/\/\/?\s*(@ts-ignore)?)/;
        function findConfigFile(searchPath, fileExists, configName) {
            if (configName === void 0) {
                configName = "tsconfig.json";
            }
            return ts.forEachAncestorDirectory(searchPath, function (ancestor) {
                var fileName = ts.combinePaths(ancestor, configName);
                return fileExists(fileName) ? fileName : undefined;
            });
        }
        ts.findConfigFile = findConfigFile;
        function resolveTripleslashReference(moduleName, containingFile) {
            var basePath = ts.getDirectoryPath(containingFile);
            var referencedFileName = ts.isRootedDiskPath(moduleName) ? moduleName : ts.combinePaths(basePath, moduleName);
            return ts.normalizePath(referencedFileName);
        }
        ts.resolveTripleslashReference = resolveTripleslashReference;
        /* @internal */
        function computeCommonSourceDirectoryOfFilenames(fileNames, currentDirectory, getCanonicalFileName) {
            var commonPathComponents;
            var failed = ts.forEach(fileNames, function (sourceFile) {
                // Each file contributes into common source file path
                var sourcePathComponents = ts.getNormalizedPathComponents(sourceFile, currentDirectory);
                sourcePathComponents.pop(); // The base file name is not part of the common directory path
                if (!commonPathComponents) {
                    // first file
                    commonPathComponents = sourcePathComponents;
                    return;
                }
                var n = Math.min(commonPathComponents.length, sourcePathComponents.length);
                for (var i = 0; i < n; i++) {
                    if (getCanonicalFileName(commonPathComponents[i]) !== getCanonicalFileName(sourcePathComponents[i])) {
                        if (i === 0) {
                            // Failed to find any common path component
                            return true;
                        }
                        // New common path found that is 0 -> i-1
                        commonPathComponents.length = i;
                        break;
                    }
                }
                // If the sourcePathComponents was shorter than the commonPathComponents, truncate to the sourcePathComponents
                if (sourcePathComponents.length < commonPathComponents.length) {
                    commonPathComponents.length = sourcePathComponents.length;
                }
            });
            // A common path can not be found when paths span multiple drives on windows, for example
            if (failed) {
                return "";
            }
            if (!commonPathComponents) { // Can happen when all input files are .d.ts files
                return currentDirectory;
            }
            return ts.getPathFromPathComponents(commonPathComponents);
        }
        ts.computeCommonSourceDirectoryOfFilenames = computeCommonSourceDirectoryOfFilenames;
        function createCompilerHost(options, setParentNodes) {
            return createCompilerHostWorker(options, setParentNodes);
        }
        ts.createCompilerHost = createCompilerHost;
        /*@internal*/
        // TODO(shkamat): update this after reworking ts build API
        function createCompilerHostWorker(options, setParentNodes, system) {
            if (system === void 0) {
                system = ts.sys;
            }
            var existingDirectories = ts.createMap();
            var getCanonicalFileName = ts.createGetCanonicalFileName(system.useCaseSensitiveFileNames);
            function getSourceFile(fileName, languageVersion, onError) {
                var text;
                try {
                    ts.performance.mark("beforeIORead");
                    text = compilerHost.readFile(fileName);
                    ts.performance.mark("afterIORead");
                    ts.performance.measure("I/O Read", "beforeIORead", "afterIORead");
                }
                catch (e) {
                    if (onError) {
                        onError(e.message);
                    }
                    text = "";
                }
                return text !== undefined ? ts.createSourceFile(fileName, text, languageVersion, setParentNodes) : undefined;
            }
            function directoryExists(directoryPath) {
                if (existingDirectories.has(directoryPath)) {
                    return true;
                }
                if ((compilerHost.directoryExists || system.directoryExists)(directoryPath)) {
                    existingDirectories.set(directoryPath, true);
                    return true;
                }
                return false;
            }
            function writeFile(fileName, data, writeByteOrderMark, onError) {
                try {
                    ts.performance.mark("beforeIOWrite");
                    // NOTE: If patchWriteFileEnsuringDirectory has been called,
                    // the system.writeFile will do its own directory creation and
                    // the ensureDirectoriesExist call will always be redundant.
                    ts.writeFileEnsuringDirectories(fileName, data, writeByteOrderMark, function (path, data, writeByteOrderMark) { return writeFileWorker(path, data, writeByteOrderMark); }, function (path) { return (compilerHost.createDirectory || system.createDirectory)(path); }, function (path) { return directoryExists(path); });
                    ts.performance.mark("afterIOWrite");
                    ts.performance.measure("I/O Write", "beforeIOWrite", "afterIOWrite");
                }
                catch (e) {
                    if (onError) {
                        onError(e.message);
                    }
                }
            }
            var outputFingerprints;
            function writeFileWorker(fileName, data, writeByteOrderMark) {
                if (!ts.isWatchSet(options) || !system.createHash || !system.getModifiedTime) {
                    system.writeFile(fileName, data, writeByteOrderMark);
                    return;
                }
                if (!outputFingerprints) {
                    outputFingerprints = ts.createMap();
                }
                var hash = system.createHash(data);
                var mtimeBefore = system.getModifiedTime(fileName);
                if (mtimeBefore) {
                    var fingerprint = outputFingerprints.get(fileName);
                    // If output has not been changed, and the file has no external modification
                    if (fingerprint &&
                        fingerprint.byteOrderMark === writeByteOrderMark &&
                        fingerprint.hash === hash &&
                        fingerprint.mtime.getTime() === mtimeBefore.getTime()) {
                        return;
                    }
                }
                system.writeFile(fileName, data, writeByteOrderMark);
                var mtimeAfter = system.getModifiedTime(fileName) || ts.missingFileModifiedTime;
                outputFingerprints.set(fileName, {
                    hash: hash,
                    byteOrderMark: writeByteOrderMark,
                    mtime: mtimeAfter
                });
            }
            function getDefaultLibLocation() {
                return ts.getDirectoryPath(ts.normalizePath(system.getExecutingFilePath()));
            }
            var newLine = ts.getNewLineCharacter(options, function () { return system.newLine; });
            var realpath = system.realpath && (function (path) { return system.realpath(path); });
            var compilerHost = {
                getSourceFile: getSourceFile,
                getDefaultLibLocation: getDefaultLibLocation,
                getDefaultLibFileName: function (options) { return ts.combinePaths(getDefaultLibLocation(), ts.getDefaultLibFileName(options)); },
                writeFile: writeFile,
                getCurrentDirectory: ts.memoize(function () { return system.getCurrentDirectory(); }),
                useCaseSensitiveFileNames: function () { return system.useCaseSensitiveFileNames; },
                getCanonicalFileName: getCanonicalFileName,
                getNewLine: function () { return newLine; },
                fileExists: function (fileName) { return system.fileExists(fileName); },
                readFile: function (fileName) { return system.readFile(fileName); },
                trace: function (s) { return system.write(s + newLine); },
                directoryExists: function (directoryName) { return system.directoryExists(directoryName); },
                getEnvironmentVariable: function (name) { return system.getEnvironmentVariable ? system.getEnvironmentVariable(name) : ""; },
                getDirectories: function (path) { return system.getDirectories(path); },
                realpath: realpath,
                readDirectory: function (path, extensions, include, exclude, depth) { return system.readDirectory(path, extensions, include, exclude, depth); },
                createDirectory: function (d) { return system.createDirectory(d); },
                createHash: ts.maybeBind(system, system.createHash)
            };
            return compilerHost;
        }
        ts.createCompilerHostWorker = createCompilerHostWorker;
        /*@internal*/
        function changeCompilerHostLikeToUseCache(host, toPath, getSourceFile) {
            var originalReadFile = host.readFile;
            var originalFileExists = host.fileExists;
            var originalDirectoryExists = host.directoryExists;
            var originalCreateDirectory = host.createDirectory;
            var originalWriteFile = host.writeFile;
            var readFileCache = ts.createMap();
            var fileExistsCache = ts.createMap();
            var directoryExistsCache = ts.createMap();
            var sourceFileCache = ts.createMap();
            var readFileWithCache = function (fileName) {
                var key = toPath(fileName);
                var value = readFileCache.get(key);
                if (value !== undefined)
                    return value !== false ? value : undefined;
                return setReadFileCache(key, fileName);
            };
            var setReadFileCache = function (key, fileName) {
                var newValue = originalReadFile.call(host, fileName);
                readFileCache.set(key, newValue !== undefined ? newValue : false);
                return newValue;
            };
            host.readFile = function (fileName) {
                var key = toPath(fileName);
                var value = readFileCache.get(key);
                if (value !== undefined)
                    return value !== false ? value : undefined; // could be .d.ts from output
                // Cache json or buildInfo
                if (!ts.fileExtensionIs(fileName, ".json" /* Json */) && !ts.isBuildInfoFile(fileName)) {
                    return originalReadFile.call(host, fileName);
                }
                return setReadFileCache(key, fileName);
            };
            var getSourceFileWithCache = getSourceFile ? function (fileName, languageVersion, onError, shouldCreateNewSourceFile) {
                var key = toPath(fileName);
                var value = sourceFileCache.get(key);
                if (value)
                    return value;
                var sourceFile = getSourceFile(fileName, languageVersion, onError, shouldCreateNewSourceFile);
                if (sourceFile && (ts.isDeclarationFileName(fileName) || ts.fileExtensionIs(fileName, ".json" /* Json */))) {
                    sourceFileCache.set(key, sourceFile);
                }
                return sourceFile;
            } : undefined;
            // fileExists for any kind of extension
            host.fileExists = function (fileName) {
                var key = toPath(fileName);
                var value = fileExistsCache.get(key);
                if (value !== undefined)
                    return value;
                var newValue = originalFileExists.call(host, fileName);
                fileExistsCache.set(key, !!newValue);
                return newValue;
            };
            if (originalWriteFile) {
                host.writeFile = function (fileName, data, writeByteOrderMark, onError, sourceFiles) {
                    var key = toPath(fileName);
                    fileExistsCache.delete(key);
                    var value = readFileCache.get(key);
                    if (value !== undefined && value !== data) {
                        readFileCache.delete(key);
                        sourceFileCache.delete(key);
                    }
                    else if (getSourceFileWithCache) {
                        var sourceFile = sourceFileCache.get(key);
                        if (sourceFile && sourceFile.text !== data) {
                            sourceFileCache.delete(key);
                        }
                    }
                    originalWriteFile.call(host, fileName, data, writeByteOrderMark, onError, sourceFiles);
                };
            }
            // directoryExists
            if (originalDirectoryExists && originalCreateDirectory) {
                host.directoryExists = function (directory) {
                    var key = toPath(directory);
                    var value = directoryExistsCache.get(key);
                    if (value !== undefined)
                        return value;
                    var newValue = originalDirectoryExists.call(host, directory);
                    directoryExistsCache.set(key, !!newValue);
                    return newValue;
                };
                host.createDirectory = function (directory) {
                    var key = toPath(directory);
                    directoryExistsCache.delete(key);
                    originalCreateDirectory.call(host, directory);
                };
            }
            return {
                originalReadFile: originalReadFile,
                originalFileExists: originalFileExists,
                originalDirectoryExists: originalDirectoryExists,
                originalCreateDirectory: originalCreateDirectory,
                originalWriteFile: originalWriteFile,
                getSourceFileWithCache: getSourceFileWithCache,
                readFileWithCache: readFileWithCache
            };
        }
        ts.changeCompilerHostLikeToUseCache = changeCompilerHostLikeToUseCache;
        function getPreEmitDiagnostics(program, sourceFile, cancellationToken) {
            var diagnostics;
            diagnostics = ts.addRange(diagnostics, program.getConfigFileParsingDiagnostics());
            diagnostics = ts.addRange(diagnostics, program.getOptionsDiagnostics(cancellationToken));
            diagnostics = ts.addRange(diagnostics, program.getSyntacticDiagnostics(sourceFile, cancellationToken));
            diagnostics = ts.addRange(diagnostics, program.getGlobalDiagnostics(cancellationToken));
            diagnostics = ts.addRange(diagnostics, program.getSemanticDiagnostics(sourceFile, cancellationToken));
            if (ts.getEmitDeclarations(program.getCompilerOptions())) {
                diagnostics = ts.addRange(diagnostics, program.getDeclarationDiagnostics(sourceFile, cancellationToken));
            }
            return ts.sortAndDeduplicateDiagnostics(diagnostics || ts.emptyArray);
        }
        ts.getPreEmitDiagnostics = getPreEmitDiagnostics;
        function formatDiagnostics(diagnostics, host) {
            var output = "";
            for (var _i = 0, diagnostics_2 = diagnostics; _i < diagnostics_2.length; _i++) {
                var diagnostic = diagnostics_2[_i];
                output += formatDiagnostic(diagnostic, host);
            }
            return output;
        }
        ts.formatDiagnostics = formatDiagnostics;
        function formatDiagnostic(diagnostic, host) {
            var errorMessage = ts.diagnosticCategoryName(diagnostic) + " TS" + diagnostic.code + ": " + flattenDiagnosticMessageText(diagnostic.messageText, host.getNewLine()) + host.getNewLine();
            if (diagnostic.file) {
                var _a = ts.getLineAndCharacterOfPosition(diagnostic.file, diagnostic.start), line = _a.line, character = _a.character; // TODO: GH#18217
                var fileName = diagnostic.file.fileName;
                var relativeFileName = ts.convertToRelativePath(fileName, host.getCurrentDirectory(), function (fileName) { return host.getCanonicalFileName(fileName); });
                return relativeFileName + "(" + (line + 1) + "," + (character + 1) + "): " + errorMessage;
            }
            return errorMessage;
        }
        ts.formatDiagnostic = formatDiagnostic;
        /** @internal */
        var ForegroundColorEscapeSequences;
        (function (ForegroundColorEscapeSequences) {
            ForegroundColorEscapeSequences["Grey"] = "\u001B[90m";
            ForegroundColorEscapeSequences["Red"] = "\u001B[91m";
            ForegroundColorEscapeSequences["Yellow"] = "\u001B[93m";
            ForegroundColorEscapeSequences["Blue"] = "\u001B[94m";
            ForegroundColorEscapeSequences["Cyan"] = "\u001B[96m";
        })(ForegroundColorEscapeSequences = ts.ForegroundColorEscapeSequences || (ts.ForegroundColorEscapeSequences = {}));
        var gutterStyleSequence = "\u001b[7m";
        var gutterSeparator = " ";
        var resetEscapeSequence = "\u001b[0m";
        var ellipsis = "...";
        var halfIndent = "  ";
        var indent = "    ";
        function getCategoryFormat(category) {
            switch (category) {
                case ts.DiagnosticCategory.Error: return ForegroundColorEscapeSequences.Red;
                case ts.DiagnosticCategory.Warning: return ForegroundColorEscapeSequences.Yellow;
                case ts.DiagnosticCategory.Suggestion: return ts.Debug.fail("Should never get an Info diagnostic on the command line.");
                case ts.DiagnosticCategory.Message: return ForegroundColorEscapeSequences.Blue;
            }
        }
        /** @internal */
        function formatColorAndReset(text, formatStyle) {
            return formatStyle + text + resetEscapeSequence;
        }
        ts.formatColorAndReset = formatColorAndReset;
        function padLeft(s, length) {
            while (s.length < length) {
                s = " " + s;
            }
            return s;
        }
        function formatCodeSpan(file, start, length, indent, squiggleColor, host) {
            var _a = ts.getLineAndCharacterOfPosition(file, start), firstLine = _a.line, firstLineChar = _a.character;
            var _b = ts.getLineAndCharacterOfPosition(file, start + length), lastLine = _b.line, lastLineChar = _b.character;
            var lastLineInFile = ts.getLineAndCharacterOfPosition(file, file.text.length).line;
            var hasMoreThanFiveLines = (lastLine - firstLine) >= 4;
            var gutterWidth = (lastLine + 1 + "").length;
            if (hasMoreThanFiveLines) {
                gutterWidth = Math.max(ellipsis.length, gutterWidth);
            }
            var context = "";
            for (var i = firstLine; i <= lastLine; i++) {
                context += host.getNewLine();
                // If the error spans over 5 lines, we'll only show the first 2 and last 2 lines,
                // so we'll skip ahead to the second-to-last line.
                if (hasMoreThanFiveLines && firstLine + 1 < i && i < lastLine - 1) {
                    context += indent + formatColorAndReset(padLeft(ellipsis, gutterWidth), gutterStyleSequence) + gutterSeparator + host.getNewLine();
                    i = lastLine - 1;
                }
                var lineStart = ts.getPositionOfLineAndCharacter(file, i, 0);
                var lineEnd = i < lastLineInFile ? ts.getPositionOfLineAndCharacter(file, i + 1, 0) : file.text.length;
                var lineContent = file.text.slice(lineStart, lineEnd);
                lineContent = lineContent.replace(/\s+$/g, ""); // trim from end
                lineContent = lineContent.replace("\t", " "); // convert tabs to single spaces
                // Output the gutter and the actual contents of the line.
                context += indent + formatColorAndReset(padLeft(i + 1 + "", gutterWidth), gutterStyleSequence) + gutterSeparator;
                context += lineContent + host.getNewLine();
                // Output the gutter and the error span for the line using tildes.
                context += indent + formatColorAndReset(padLeft("", gutterWidth), gutterStyleSequence) + gutterSeparator;
                context += squiggleColor;
                if (i === firstLine) {
                    // If we're on the last line, then limit it to the last character of the last line.
                    // Otherwise, we'll just squiggle the rest of the line, giving 'slice' no end position.
                    var lastCharForLine = i === lastLine ? lastLineChar : undefined;
                    context += lineContent.slice(0, firstLineChar).replace(/\S/g, " ");
                    context += lineContent.slice(firstLineChar, lastCharForLine).replace(/./g, "~");
                }
                else if (i === lastLine) {
                    context += lineContent.slice(0, lastLineChar).replace(/./g, "~");
                }
                else {
                    // Squiggle the entire line.
                    context += lineContent.replace(/./g, "~");
                }
                context += resetEscapeSequence;
            }
            return context;
        }
        /* @internal */
        function formatLocation(file, start, host, color) {
            if (color === void 0) {
                color = formatColorAndReset;
            }
            var _a = ts.getLineAndCharacterOfPosition(file, start), firstLine = _a.line, firstLineChar = _a.character; // TODO: GH#18217
            var relativeFileName = host ? ts.convertToRelativePath(file.fileName, host.getCurrentDirectory(), function (fileName) { return host.getCanonicalFileName(fileName); }) : file.fileName;
            var output = "";
            output += color(relativeFileName, ForegroundColorEscapeSequences.Cyan);
            output += ":";
            output += color("" + (firstLine + 1), ForegroundColorEscapeSequences.Yellow);
            output += ":";
            output += color("" + (firstLineChar + 1), ForegroundColorEscapeSequences.Yellow);
            return output;
        }
        ts.formatLocation = formatLocation;
        function formatDiagnosticsWithColorAndContext(diagnostics, host) {
            var output = "";
            for (var _i = 0, diagnostics_3 = diagnostics; _i < diagnostics_3.length; _i++) {
                var diagnostic = diagnostics_3[_i];
                if (diagnostic.file) {
                    var file = diagnostic.file, start = diagnostic.start;
                    output += formatLocation(file, start, host); // TODO: GH#18217
                    output += " - ";
                }
                output += formatColorAndReset(ts.diagnosticCategoryName(diagnostic), getCategoryFormat(diagnostic.category));
                output += formatColorAndReset(" TS" + diagnostic.code + ": ", ForegroundColorEscapeSequences.Grey);
                output += flattenDiagnosticMessageText(diagnostic.messageText, host.getNewLine());
                if (diagnostic.file) {
                    output += host.getNewLine();
                    output += formatCodeSpan(diagnostic.file, diagnostic.start, diagnostic.length, "", getCategoryFormat(diagnostic.category), host); // TODO: GH#18217
                    if (diagnostic.relatedInformation) {
                        output += host.getNewLine();
                        for (var _a = 0, _b = diagnostic.relatedInformation; _a < _b.length; _a++) {
                            var _c = _b[_a], file = _c.file, start = _c.start, length_7 = _c.length, messageText = _c.messageText;
                            if (file) {
                                output += host.getNewLine();
                                output += halfIndent + formatLocation(file, start, host); // TODO: GH#18217
                                output += formatCodeSpan(file, start, length_7, indent, ForegroundColorEscapeSequences.Cyan, host); // TODO: GH#18217
                            }
                            output += host.getNewLine();
                            output += indent + flattenDiagnosticMessageText(messageText, host.getNewLine());
                        }
                    }
                }
                output += host.getNewLine();
            }
            return output;
        }
        ts.formatDiagnosticsWithColorAndContext = formatDiagnosticsWithColorAndContext;
        function flattenDiagnosticMessageText(diag, newLine, indent) {
            if (indent === void 0) {
                indent = 0;
            }
            if (ts.isString(diag)) {
                return diag;
            }
            else if (diag === undefined) {
                return "";
            }
            var result = "";
            if (indent) {
                result += newLine;
                for (var i = 0; i < indent; i++) {
                    result += "  ";
                }
            }
            result += diag.messageText;
            indent++;
            if (diag.next) {
                for (var _i = 0, _a = diag.next; _i < _a.length; _i++) {
                    var kid = _a[_i];
                    result += flattenDiagnosticMessageText(kid, newLine, indent);
                }
            }
            return result;
        }
        ts.flattenDiagnosticMessageText = flattenDiagnosticMessageText;
        /* @internal */
        function loadWithLocalCache(names, containingFile, redirectedReference, loader) {
            if (names.length === 0) {
                return [];
            }
            var resolutions = [];
            var cache = ts.createMap();
            for (var _i = 0, names_2 = names; _i < names_2.length; _i++) {
                var name = names_2[_i];
                var result = void 0;
                if (cache.has(name)) {
                    result = cache.get(name);
                }
                else {
                    cache.set(name, result = loader(name, containingFile, redirectedReference));
                }
                resolutions.push(result);
            }
            return resolutions;
        }
        ts.loadWithLocalCache = loadWithLocalCache;
        /* @internal */
        ts.inferredTypesContainingFile = "__inferred type names__.ts";
        /**
         * Determines if program structure is upto date or needs to be recreated
         */
        /* @internal */
        function isProgramUptoDate(program, rootFileNames, newOptions, getSourceVersion, fileExists, hasInvalidatedResolution, hasChangedAutomaticTypeDirectiveNames, projectReferences) {
            // If we haven't created a program yet or have changed automatic type directives, then it is not up-to-date
            if (!program || hasChangedAutomaticTypeDirectiveNames) {
                return false;
            }
            // If root file names don't match
            if (!ts.arrayIsEqualTo(program.getRootFileNames(), rootFileNames)) {
                return false;
            }
            var seenResolvedRefs;
            // If project references don't match
            if (!ts.arrayIsEqualTo(program.getProjectReferences(), projectReferences, projectReferenceUptoDate)) {
                return false;
            }
            // If any file is not up-to-date, then the whole program is not up-to-date
            if (program.getSourceFiles().some(sourceFileNotUptoDate)) {
                return false;
            }
            // If any of the missing file paths are now created
            if (program.getMissingFilePaths().some(fileExists)) {
                return false;
            }
            var currentOptions = program.getCompilerOptions();
            // If the compilation settings do no match, then the program is not up-to-date
            if (!ts.compareDataObjects(currentOptions, newOptions)) {
                return false;
            }
            // If everything matches but the text of config file is changed,
            // error locations can change for program options, so update the program
            if (currentOptions.configFile && newOptions.configFile) {
                return currentOptions.configFile.text === newOptions.configFile.text;
            }
            return true;
            function sourceFileNotUptoDate(sourceFile) {
                return !sourceFileVersionUptoDate(sourceFile) ||
                    hasInvalidatedResolution(sourceFile.path);
            }
            function sourceFileVersionUptoDate(sourceFile) {
                return sourceFile.version === getSourceVersion(sourceFile.resolvedPath);
            }
            function projectReferenceUptoDate(oldRef, newRef, index) {
                if (!ts.projectReferenceIsEqualTo(oldRef, newRef)) {
                    return false;
                }
                return resolvedProjectReferenceUptoDate(program.getResolvedProjectReferences()[index], oldRef);
            }
            function resolvedProjectReferenceUptoDate(oldResolvedRef, oldRef) {
                if (oldResolvedRef) {
                    if (ts.contains(seenResolvedRefs, oldResolvedRef)) {
                        // Assume true
                        return true;
                    }
                    // If sourceFile for the oldResolvedRef existed, check the version for uptodate
                    if (!sourceFileVersionUptoDate(oldResolvedRef.sourceFile)) {
                        return false;
                    }
                    // Add to seen before checking the referenced paths of this config file
                    (seenResolvedRefs || (seenResolvedRefs = [])).push(oldResolvedRef);
                    // If child project references are upto date, this project reference is uptodate
                    return !ts.forEach(oldResolvedRef.references, function (childResolvedRef, index) {
                        return !resolvedProjectReferenceUptoDate(childResolvedRef, oldResolvedRef.commandLine.projectReferences[index]);
                    });
                }
                // In old program, not able to resolve project reference path,
                // so if config file doesnt exist, it is uptodate.
                return !fileExists(resolveProjectReferencePath(oldRef));
            }
        }
        ts.isProgramUptoDate = isProgramUptoDate;
        function getConfigFileParsingDiagnostics(configFileParseResult) {
            return configFileParseResult.options.configFile ? __spreadArrays(configFileParseResult.options.configFile.parseDiagnostics, configFileParseResult.errors) :
                configFileParseResult.errors;
        }
        ts.getConfigFileParsingDiagnostics = getConfigFileParsingDiagnostics;
        /**
         * Determine if source file needs to be re-created even if its text hasn't changed
         */
        function shouldProgramCreateNewSourceFiles(program, newOptions) {
            if (!program)
                return false;
            // If any compiler options change, we can't reuse old source file even if version match
            // The change in options like these could result in change in syntax tree or `sourceFile.bindDiagnostics`.
            var oldOptions = program.getCompilerOptions();
            return !!ts.sourceFileAffectingCompilerOptions.some(function (option) {
                return !ts.isJsonEqual(ts.getCompilerOptionValue(oldOptions, option), ts.getCompilerOptionValue(newOptions, option));
            });
        }
        function createCreateProgramOptions(rootNames, options, host, oldProgram, configFileParsingDiagnostics) {
            return {
                rootNames: rootNames,
                options: options,
                host: host,
                oldProgram: oldProgram,
                configFileParsingDiagnostics: configFileParsingDiagnostics
            };
        }
        function createProgram(rootNamesOrOptions, _options, _host, _oldProgram, _configFileParsingDiagnostics) {
            var createProgramOptions = ts.isArray(rootNamesOrOptions) ? createCreateProgramOptions(rootNamesOrOptions, _options, _host, _oldProgram, _configFileParsingDiagnostics) : rootNamesOrOptions; // TODO: GH#18217
            var rootNames = createProgramOptions.rootNames, options = createProgramOptions.options, configFileParsingDiagnostics = createProgramOptions.configFileParsingDiagnostics, projectReferences = createProgramOptions.projectReferences;
            var oldProgram = createProgramOptions.oldProgram;
            var processingDefaultLibFiles;
            var processingOtherFiles;
            var files;
            var symlinks;
            var commonSourceDirectory;
            var diagnosticsProducingTypeChecker;
            var noDiagnosticsTypeChecker;
            var classifiableNames;
            var ambientModuleNameToUnmodifiedFileName = ts.createMap();
            // Todo:: Use this to report why file was included in --extendedDiagnostics
            var refFileMap;
            var cachedBindAndCheckDiagnosticsForFile = {};
            var cachedDeclarationDiagnosticsForFile = {};
            var resolvedTypeReferenceDirectives = ts.createMap();
            var fileProcessingDiagnostics = ts.createDiagnosticCollection();
            // The below settings are to track if a .js file should be add to the program if loaded via searching under node_modules.
            // This works as imported modules are discovered recursively in a depth first manner, specifically:
            // - For each root file, findSourceFile is called.
            // - This calls processImportedModules for each module imported in the source file.
            // - This calls resolveModuleNames, and then calls findSourceFile for each resolved module.
            // As all these operations happen - and are nested - within the createProgram call, they close over the below variables.
            // The current resolution depth is tracked by incrementing/decrementing as the depth first search progresses.
            var maxNodeModuleJsDepth = typeof options.maxNodeModuleJsDepth === "number" ? options.maxNodeModuleJsDepth : 0;
            var currentNodeModulesDepth = 0;
            // If a module has some of its imports skipped due to being at the depth limit under node_modules, then track
            // this, as it may be imported at a shallower depth later, and then it will need its skipped imports processed.
            var modulesWithElidedImports = ts.createMap();
            // Track source files that are source files found by searching under node_modules, as these shouldn't be compiled.
            var sourceFilesFoundSearchingNodeModules = ts.createMap();
            ts.performance.mark("beforeProgram");
            var host = createProgramOptions.host || createCompilerHost(options);
            var configParsingHost = parseConfigHostFromCompilerHostLike(host);
            var skipDefaultLib = options.noLib;
            var getDefaultLibraryFileName = ts.memoize(function () { return host.getDefaultLibFileName(options); });
            var defaultLibraryPath = host.getDefaultLibLocation ? host.getDefaultLibLocation() : ts.getDirectoryPath(getDefaultLibraryFileName());
            var programDiagnostics = ts.createDiagnosticCollection();
            var currentDirectory = host.getCurrentDirectory();
            var supportedExtensions = ts.getSupportedExtensions(options);
            var supportedExtensionsWithJsonIfResolveJsonModule = ts.getSuppoertedExtensionsWithJsonIfResolveJsonModule(options, supportedExtensions);
            // Map storing if there is emit blocking diagnostics for given input
            var hasEmitBlockingDiagnostics = ts.createMap();
            var _compilerOptionsObjectLiteralSyntax;
            var moduleResolutionCache;
            var resolveModuleNamesWorker;
            var hasInvalidatedResolution = host.hasInvalidatedResolution || ts.returnFalse;
            if (host.resolveModuleNames) {
                resolveModuleNamesWorker = function (moduleNames, containingFile, reusedNames, redirectedReference) {
                    return host.resolveModuleNames(ts.Debug.assertEachDefined(moduleNames), containingFile, reusedNames, redirectedReference, options).map(function (resolved) {
                        // An older host may have omitted extension, in which case we should infer it from the file extension of resolvedFileName.
                        if (!resolved || resolved.extension !== undefined) {
                            return resolved;
                        }
                        var withExtension = ts.clone(resolved);
                        withExtension.extension = ts.extensionFromPath(resolved.resolvedFileName);
                        return withExtension;
                    });
                };
            }
            else {
                moduleResolutionCache = ts.createModuleResolutionCache(currentDirectory, function (x) { return host.getCanonicalFileName(x); }, options);
                var loader_1 = function (moduleName, containingFile, redirectedReference) { return ts.resolveModuleName(moduleName, containingFile, options, host, moduleResolutionCache, redirectedReference).resolvedModule; }; // TODO: GH#18217
                resolveModuleNamesWorker = function (moduleNames, containingFile, _reusedNames, redirectedReference) { return loadWithLocalCache(ts.Debug.assertEachDefined(moduleNames), containingFile, redirectedReference, loader_1); };
            }
            var resolveTypeReferenceDirectiveNamesWorker;
            if (host.resolveTypeReferenceDirectives) {
                resolveTypeReferenceDirectiveNamesWorker = function (typeDirectiveNames, containingFile, redirectedReference) { return host.resolveTypeReferenceDirectives(ts.Debug.assertEachDefined(typeDirectiveNames), containingFile, redirectedReference, options); };
            }
            else {
                var loader_2 = function (typesRef, containingFile, redirectedReference) { return ts.resolveTypeReferenceDirective(typesRef, containingFile, options, host, redirectedReference).resolvedTypeReferenceDirective; }; // TODO: GH#18217
                resolveTypeReferenceDirectiveNamesWorker = function (typeReferenceDirectiveNames, containingFile, redirectedReference) { return loadWithLocalCache(ts.Debug.assertEachDefined(typeReferenceDirectiveNames), containingFile, redirectedReference, loader_2); };
            }
            // Map from a stringified PackageId to the source file with that id.
            // Only one source file may have a given packageId. Others become redirects (see createRedirectSourceFile).
            // `packageIdToSourceFile` is only used while building the program, while `sourceFileToPackageName` and `isSourceFileTargetOfRedirect` are kept around.
            var packageIdToSourceFile = ts.createMap();
            // Maps from a SourceFile's `.path` to the name of the package it was imported with.
            var sourceFileToPackageName = ts.createMap();
            // Key is a file name. Value is the (non-empty, or undefined) list of files that redirect to it.
            var redirectTargetsMap = ts.createMultiMap();
            /**
             * map with
             * - SourceFile if present
             * - false if sourceFile missing for source of project reference redirect
             * - undefined otherwise
             */
            var filesByName = ts.createMap();
            var missingFilePaths;
            // stores 'filename -> file association' ignoring case
            // used to track cases when two file names differ only in casing
            var filesByNameIgnoreCase = host.useCaseSensitiveFileNames() ? ts.createMap() : undefined;
            // A parallel array to projectReferences storing the results of reading in the referenced tsconfig files
            var resolvedProjectReferences;
            var projectReferenceRedirects;
            var mapFromFileToProjectReferenceRedirects;
            var mapFromToProjectReferenceRedirectSource;
            var useSourceOfProjectReferenceRedirect = !!host.useSourceOfProjectReferenceRedirect && host.useSourceOfProjectReferenceRedirect();
            var shouldCreateNewSourceFile = shouldProgramCreateNewSourceFiles(oldProgram, options);
            // We set `structuralIsReused` to `undefined` because `tryReuseStructureFromOldProgram` calls `tryReuseStructureFromOldProgram` which checks
            // `structuralIsReused`, which would be a TDZ violation if it was not set in advance to `undefined`.
            var structuralIsReused;
            structuralIsReused = tryReuseStructureFromOldProgram(); // eslint-disable-line prefer-const
            if (structuralIsReused !== 2 /* Completely */) {
                processingDefaultLibFiles = [];
                processingOtherFiles = [];
                if (projectReferences) {
                    if (!resolvedProjectReferences) {
                        resolvedProjectReferences = projectReferences.map(parseProjectReferenceConfigFile);
                    }
                    if (host.setResolvedProjectReferenceCallbacks) {
                        host.setResolvedProjectReferenceCallbacks({
                            getSourceOfProjectReferenceRedirect: getSourceOfProjectReferenceRedirect,
                            forEachResolvedProjectReference: forEachResolvedProjectReference
                        });
                    }
                    if (rootNames.length) {
                        for (var _i = 0, resolvedProjectReferences_1 = resolvedProjectReferences; _i < resolvedProjectReferences_1.length; _i++) {
                            var parsedRef = resolvedProjectReferences_1[_i];
                            if (!parsedRef)
                                continue;
                            var out = parsedRef.commandLine.options.outFile || parsedRef.commandLine.options.out;
                            if (useSourceOfProjectReferenceRedirect) {
                                if (out || ts.getEmitModuleKind(parsedRef.commandLine.options) === ts.ModuleKind.None) {
                                    for (var _a = 0, _b = parsedRef.commandLine.fileNames; _a < _b.length; _a++) {
                                        var fileName = _b[_a];
                                        processSourceFile(fileName, /*isDefaultLib*/ false, /*ignoreNoDefaultLib*/ false, /*packageId*/ undefined);
                                    }
                                }
                            }
                            else {
                                if (out) {
                                    processSourceFile(ts.changeExtension(out, ".d.ts"), /*isDefaultLib*/ false, /*ignoreNoDefaultLib*/ false, /*packageId*/ undefined);
                                }
                                else if (ts.getEmitModuleKind(parsedRef.commandLine.options) === ts.ModuleKind.None) {
                                    for (var _c = 0, _d = parsedRef.commandLine.fileNames; _c < _d.length; _c++) {
                                        var fileName = _d[_c];
                                        if (!ts.fileExtensionIs(fileName, ".d.ts" /* Dts */) && !ts.fileExtensionIs(fileName, ".json" /* Json */)) {
                                            processSourceFile(ts.getOutputDeclarationFileName(fileName, parsedRef.commandLine, !host.useCaseSensitiveFileNames()), /*isDefaultLib*/ false, /*ignoreNoDefaultLib*/ false, /*packageId*/ undefined);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                ts.forEach(rootNames, function (name) { return processRootFile(name, /*isDefaultLib*/ false, /*ignoreNoDefaultLib*/ false); });
                // load type declarations specified via 'types' argument or implicitly from types/ and node_modules/@types folders
                var typeReferences = rootNames.length ? ts.getAutomaticTypeDirectiveNames(options, host) : ts.emptyArray;
                if (typeReferences.length) {
                    // This containingFilename needs to match with the one used in managed-side
                    var containingDirectory = options.configFilePath ? ts.getDirectoryPath(options.configFilePath) : host.getCurrentDirectory();
                    var containingFilename = ts.combinePaths(containingDirectory, ts.inferredTypesContainingFile);
                    var resolutions = resolveTypeReferenceDirectiveNamesWorker(typeReferences, containingFilename);
                    for (var i = 0; i < typeReferences.length; i++) {
                        processTypeReferenceDirective(typeReferences[i], resolutions[i]);
                    }
                }
                // Do not process the default library if:
                //  - The '--noLib' flag is used.
                //  - A 'no-default-lib' reference comment is encountered in
                //      processing the root files.
                if (rootNames.length && !skipDefaultLib) {
                    // If '--lib' is not specified, include default library file according to '--target'
                    // otherwise, using options specified in '--lib' instead of '--target' default library file
                    var defaultLibraryFileName = getDefaultLibraryFileName();
                    if (!options.lib && defaultLibraryFileName) {
                        processRootFile(defaultLibraryFileName, /*isDefaultLib*/ true, /*ignoreNoDefaultLib*/ false);
                    }
                    else {
                        ts.forEach(options.lib, function (libFileName) {
                            processRootFile(ts.combinePaths(defaultLibraryPath, libFileName), /*isDefaultLib*/ true, /*ignoreNoDefaultLib*/ false);
                        });
                    }
                }
                missingFilePaths = ts.arrayFrom(ts.mapDefinedIterator(filesByName.entries(), function (_a) {
                    var path = _a[0], file = _a[1];
                    return file === undefined ? path : undefined;
                }));
                files = ts.stableSort(processingDefaultLibFiles, compareDefaultLibFiles).concat(processingOtherFiles);
                processingDefaultLibFiles = undefined;
                processingOtherFiles = undefined;
            }
            ts.Debug.assert(!!missingFilePaths);
            // Release any files we have acquired in the old program but are
            // not part of the new program.
            if (oldProgram && host.onReleaseOldSourceFile) {
                var oldSourceFiles = oldProgram.getSourceFiles();
                for (var _e = 0, oldSourceFiles_1 = oldSourceFiles; _e < oldSourceFiles_1.length; _e++) {
                    var oldSourceFile = oldSourceFiles_1[_e];
                    var newFile = getSourceFileByPath(oldSourceFile.resolvedPath);
                    if (shouldCreateNewSourceFile || !newFile ||
                        // old file wasnt redirect but new file is
                        (oldSourceFile.resolvedPath === oldSourceFile.path && newFile.resolvedPath !== oldSourceFile.path)) {
                        host.onReleaseOldSourceFile(oldSourceFile, oldProgram.getCompilerOptions(), !!getSourceFileByPath(oldSourceFile.path));
                    }
                }
                oldProgram.forEachResolvedProjectReference(function (resolvedProjectReference, resolvedProjectReferencePath) {
                    if (resolvedProjectReference && !getResolvedProjectReferenceByPath(resolvedProjectReferencePath)) {
                        host.onReleaseOldSourceFile(resolvedProjectReference.sourceFile, oldProgram.getCompilerOptions(), /*hasSourceFileByPath*/ false);
                    }
                });
            }
            // unconditionally set oldProgram to undefined to prevent it from being captured in closure
            oldProgram = undefined;
            var program = {
                getRootFileNames: function () { return rootNames; },
                getSourceFile: getSourceFile,
                getSourceFileByPath: getSourceFileByPath,
                getSourceFiles: function () { return files; },
                getMissingFilePaths: function () { return missingFilePaths; },
                getRefFileMap: function () { return refFileMap; },
                getFilesByNameMap: function () { return filesByName; },
                getCompilerOptions: function () { return options; },
                getSyntacticDiagnostics: getSyntacticDiagnostics,
                getOptionsDiagnostics: getOptionsDiagnostics,
                getGlobalDiagnostics: getGlobalDiagnostics,
                getSemanticDiagnostics: getSemanticDiagnostics,
                getSuggestionDiagnostics: getSuggestionDiagnostics,
                getDeclarationDiagnostics: getDeclarationDiagnostics,
                getBindAndCheckDiagnostics: getBindAndCheckDiagnostics,
                getProgramDiagnostics: getProgramDiagnostics,
                getTypeChecker: getTypeChecker,
                getClassifiableNames: getClassifiableNames,
                getDiagnosticsProducingTypeChecker: getDiagnosticsProducingTypeChecker,
                getCommonSourceDirectory: getCommonSourceDirectory,
                emit: emit,
                getCurrentDirectory: function () { return currentDirectory; },
                getNodeCount: function () { return getDiagnosticsProducingTypeChecker().getNodeCount(); },
                getIdentifierCount: function () { return getDiagnosticsProducingTypeChecker().getIdentifierCount(); },
                getSymbolCount: function () { return getDiagnosticsProducingTypeChecker().getSymbolCount(); },
                getTypeCount: function () { return getDiagnosticsProducingTypeChecker().getTypeCount(); },
                getRelationCacheSizes: function () { return getDiagnosticsProducingTypeChecker().getRelationCacheSizes(); },
                getFileProcessingDiagnostics: function () { return fileProcessingDiagnostics; },
                getResolvedTypeReferenceDirectives: function () { return resolvedTypeReferenceDirectives; },
                isSourceFileFromExternalLibrary: isSourceFileFromExternalLibrary,
                isSourceFileDefaultLibrary: isSourceFileDefaultLibrary,
                dropDiagnosticsProducingTypeChecker: dropDiagnosticsProducingTypeChecker,
                getSourceFileFromReference: getSourceFileFromReference,
                getLibFileFromReference: getLibFileFromReference,
                sourceFileToPackageName: sourceFileToPackageName,
                redirectTargetsMap: redirectTargetsMap,
                isEmittedFile: isEmittedFile,
                getConfigFileParsingDiagnostics: getConfigFileParsingDiagnostics,
                getResolvedModuleWithFailedLookupLocationsFromCache: getResolvedModuleWithFailedLookupLocationsFromCache,
                getProjectReferences: getProjectReferences,
                getResolvedProjectReferences: getResolvedProjectReferences,
                getProjectReferenceRedirect: getProjectReferenceRedirect,
                getResolvedProjectReferenceToRedirect: getResolvedProjectReferenceToRedirect,
                getResolvedProjectReferenceByPath: getResolvedProjectReferenceByPath,
                forEachResolvedProjectReference: forEachResolvedProjectReference,
                isSourceOfProjectReferenceRedirect: isSourceOfProjectReferenceRedirect,
                emitBuildInfo: emitBuildInfo,
                getProbableSymlinks: getProbableSymlinks
            };
            verifyCompilerOptions();
            ts.performance.mark("afterProgram");
            ts.performance.measure("Program", "beforeProgram", "afterProgram");
            return program;
            function compareDefaultLibFiles(a, b) {
                return ts.compareValues(getDefaultLibFilePriority(a), getDefaultLibFilePriority(b));
            }
            function getDefaultLibFilePriority(a) {
                if (ts.containsPath(defaultLibraryPath, a.fileName, /*ignoreCase*/ false)) {
                    var basename = ts.getBaseFileName(a.fileName);
                    if (basename === "lib.d.ts" || basename === "lib.es6.d.ts")
                        return 0;
                    var name = ts.removeSuffix(ts.removePrefix(basename, "lib."), ".d.ts");
                    var index = ts.libs.indexOf(name);
                    if (index !== -1)
                        return index + 1;
                }
                return ts.libs.length + 2;
            }
            function getResolvedModuleWithFailedLookupLocationsFromCache(moduleName, containingFile) {
                return moduleResolutionCache && ts.resolveModuleNameFromCache(moduleName, containingFile, moduleResolutionCache);
            }
            function toPath(fileName) {
                return ts.toPath(fileName, currentDirectory, getCanonicalFileName);
            }
            function getCommonSourceDirectory() {
                if (commonSourceDirectory === undefined) {
                    var emittedFiles = ts.filter(files, function (file) { return ts.sourceFileMayBeEmitted(file, program); });
                    if (options.rootDir && checkSourceFilesBelongToPath(emittedFiles, options.rootDir)) {
                        // If a rootDir is specified use it as the commonSourceDirectory
                        commonSourceDirectory = ts.getNormalizedAbsolutePath(options.rootDir, currentDirectory);
                    }
                    else if (options.composite && options.configFilePath) {
                        // Project compilations never infer their root from the input source paths
                        commonSourceDirectory = ts.getDirectoryPath(ts.normalizeSlashes(options.configFilePath));
                        checkSourceFilesBelongToPath(emittedFiles, commonSourceDirectory);
                    }
                    else {
                        commonSourceDirectory = computeCommonSourceDirectory(emittedFiles);
                    }
                    if (commonSourceDirectory && commonSourceDirectory[commonSourceDirectory.length - 1] !== ts.directorySeparator) {
                        // Make sure directory path ends with directory separator so this string can directly
                        // used to replace with "" to get the relative path of the source file and the relative path doesn't
                        // start with / making it rooted path
                        commonSourceDirectory += ts.directorySeparator;
                    }
                }
                return commonSourceDirectory;
            }
            function getClassifiableNames() {
                if (!classifiableNames) {
                    // Initialize a checker so that all our files are bound.
                    getTypeChecker();
                    classifiableNames = ts.createUnderscoreEscapedMap();
                    for (var _i = 0, files_2 = files; _i < files_2.length; _i++) {
                        var sourceFile = files_2[_i];
                        ts.copyEntries(sourceFile.classifiableNames, classifiableNames);
                    }
                }
                return classifiableNames;
            }
            function resolveModuleNamesReusingOldState(moduleNames, containingFile, file) {
                if (structuralIsReused === 0 /* Not */ && !file.ambientModuleNames.length) {
                    // If the old program state does not permit reusing resolutions and `file` does not contain locally defined ambient modules,
                    // the best we can do is fallback to the default logic.
                    return resolveModuleNamesWorker(moduleNames, containingFile, /*reusedNames*/ undefined, getResolvedProjectReferenceToRedirect(file.originalFileName));
                }
                var oldSourceFile = oldProgram && oldProgram.getSourceFile(containingFile);
                if (oldSourceFile !== file && file.resolvedModules) {
                    // `file` was created for the new program.
                    //
                    // We only set `file.resolvedModules` via work from the current function,
                    // so it is defined iff we already called the current function on `file`.
                    // That call happened no later than the creation of the `file` object,
                    // which per above occurred during the current program creation.
                    // Since we assume the filesystem does not change during program creation,
                    // it is safe to reuse resolutions from the earlier call.
                    var result_8 = [];
                    for (var _i = 0, moduleNames_1 = moduleNames; _i < moduleNames_1.length; _i++) {
                        var moduleName = moduleNames_1[_i];
                        var resolvedModule = file.resolvedModules.get(moduleName);
                        result_8.push(resolvedModule);
                    }
                    return result_8;
                }
                // At this point, we know at least one of the following hold:
                // - file has local declarations for ambient modules
                // - old program state is available
                // With this information, we can infer some module resolutions without performing resolution.
                /** An ordered list of module names for which we cannot recover the resolution. */
                var unknownModuleNames;
                /**
                 * The indexing of elements in this list matches that of `moduleNames`.
                 *
                 * Before combining results, result[i] is in one of the following states:
                 * * undefined: needs to be recomputed,
                 * * predictedToResolveToAmbientModuleMarker: known to be an ambient module.
                 * Needs to be reset to undefined before returning,
                 * * ResolvedModuleFull instance: can be reused.
                 */
                var result;
                var reusedNames;
                /** A transient placeholder used to mark predicted resolution in the result list. */
                var predictedToResolveToAmbientModuleMarker = {};
                for (var i = 0; i < moduleNames.length; i++) {
                    var moduleName = moduleNames[i];
                    // If the source file is unchanged and doesnt have invalidated resolution, reuse the module resolutions
                    if (file === oldSourceFile && !hasInvalidatedResolution(oldSourceFile.path)) {
                        var oldResolvedModule = oldSourceFile && oldSourceFile.resolvedModules.get(moduleName);
                        if (oldResolvedModule) {
                            if (ts.isTraceEnabled(options, host)) {
                                ts.trace(host, ts.Diagnostics.Reusing_resolution_of_module_0_to_file_1_from_old_program, moduleName, containingFile);
                            }
                            (result || (result = new Array(moduleNames.length)))[i] = oldResolvedModule;
                            (reusedNames || (reusedNames = [])).push(moduleName);
                            continue;
                        }
                    }
                    // We know moduleName resolves to an ambient module provided that moduleName:
                    // - is in the list of ambient modules locally declared in the current source file.
                    // - resolved to an ambient module in the old program whose declaration is in an unmodified file
                    //   (so the same module declaration will land in the new program)
                    var resolvesToAmbientModuleInNonModifiedFile = false;
                    if (ts.contains(file.ambientModuleNames, moduleName)) {
                        resolvesToAmbientModuleInNonModifiedFile = true;
                        if (ts.isTraceEnabled(options, host)) {
                            ts.trace(host, ts.Diagnostics.Module_0_was_resolved_as_locally_declared_ambient_module_in_file_1, moduleName, containingFile);
                        }
                    }
                    else {
                        resolvesToAmbientModuleInNonModifiedFile = moduleNameResolvesToAmbientModuleInNonModifiedFile(moduleName);
                    }
                    if (resolvesToAmbientModuleInNonModifiedFile) {
                        (result || (result = new Array(moduleNames.length)))[i] = predictedToResolveToAmbientModuleMarker;
                    }
                    else {
                        // Resolution failed in the old program, or resolved to an ambient module for which we can't reuse the result.
                        (unknownModuleNames || (unknownModuleNames = [])).push(moduleName);
                    }
                }
                var resolutions = unknownModuleNames && unknownModuleNames.length
                    ? resolveModuleNamesWorker(unknownModuleNames, containingFile, reusedNames, getResolvedProjectReferenceToRedirect(file.originalFileName))
                    : ts.emptyArray;
                // Combine results of resolutions and predicted results
                if (!result) {
                    // There were no unresolved/ambient resolutions.
                    ts.Debug.assert(resolutions.length === moduleNames.length);
                    return resolutions;
                }
                var j = 0;
                for (var i = 0; i < result.length; i++) {
                    if (result[i]) {
                        // `result[i]` is either a `ResolvedModuleFull` or a marker.
                        // If it is the former, we can leave it as is.
                        if (result[i] === predictedToResolveToAmbientModuleMarker) {
                            result[i] = undefined; // TODO: GH#18217
                        }
                    }
                    else {
                        result[i] = resolutions[j];
                        j++;
                    }
                }
                ts.Debug.assert(j === resolutions.length);
                return result;
                // If we change our policy of rechecking failed lookups on each program create,
                // we should adjust the value returned here.
                function moduleNameResolvesToAmbientModuleInNonModifiedFile(moduleName) {
                    var resolutionToFile = ts.getResolvedModule(oldSourceFile, moduleName);
                    var resolvedFile = resolutionToFile && oldProgram.getSourceFile(resolutionToFile.resolvedFileName);
                    if (resolutionToFile && resolvedFile) {
                        // In the old program, we resolved to an ambient module that was in the same
                        //   place as we expected to find an actual module file.
                        // We actually need to return 'false' here even though this seems like a 'true' case
                        //   because the normal module resolution algorithm will find this anyway.
                        return false;
                    }
                    // at least one of declarations should come from non-modified source file
                    var unmodifiedFile = ambientModuleNameToUnmodifiedFileName.get(moduleName);
                    if (!unmodifiedFile) {
                        return false;
                    }
                    if (ts.isTraceEnabled(options, host)) {
                        ts.trace(host, ts.Diagnostics.Module_0_was_resolved_as_ambient_module_declared_in_1_since_this_file_was_not_modified, moduleName, unmodifiedFile);
                    }
                    return true;
                }
            }
            function canReuseProjectReferences() {
                return !forEachProjectReference(oldProgram.getProjectReferences(), oldProgram.getResolvedProjectReferences(), function (oldResolvedRef, index, parent) {
                    var newRef = (parent ? parent.commandLine.projectReferences : projectReferences)[index];
                    var newResolvedRef = parseProjectReferenceConfigFile(newRef);
                    if (oldResolvedRef) {
                        // Resolved project reference has gone missing or changed
                        return !newResolvedRef || newResolvedRef.sourceFile !== oldResolvedRef.sourceFile;
                    }
                    else {
                        // A previously-unresolved reference may be resolved now
                        return newResolvedRef !== undefined;
                    }
                }, function (oldProjectReferences, parent) {
                    // If array of references is changed, we cant resue old program
                    var newReferences = parent ? getResolvedProjectReferenceByPath(parent.sourceFile.path).commandLine.projectReferences : projectReferences;
                    return !ts.arrayIsEqualTo(oldProjectReferences, newReferences, ts.projectReferenceIsEqualTo);
                });
            }
            function tryReuseStructureFromOldProgram() {
                if (!oldProgram) {
                    return 0 /* Not */;
                }
                // check properties that can affect structure of the program or module resolution strategy
                // if any of these properties has changed - structure cannot be reused
                var oldOptions = oldProgram.getCompilerOptions();
                if (ts.changesAffectModuleResolution(oldOptions, options)) {
                    return oldProgram.structureIsReused = 0 /* Not */;
                }
                ts.Debug.assert(!(oldProgram.structureIsReused & (2 /* Completely */ | 1 /* SafeModules */)));
                // there is an old program, check if we can reuse its structure
                var oldRootNames = oldProgram.getRootFileNames();
                if (!ts.arrayIsEqualTo(oldRootNames, rootNames)) {
                    return oldProgram.structureIsReused = 0 /* Not */;
                }
                if (!ts.arrayIsEqualTo(options.types, oldOptions.types)) {
                    return oldProgram.structureIsReused = 0 /* Not */;
                }
                // Check if any referenced project tsconfig files are different
                if (!canReuseProjectReferences()) {
                    return oldProgram.structureIsReused = 0 /* Not */;
                }
                if (projectReferences) {
                    resolvedProjectReferences = projectReferences.map(parseProjectReferenceConfigFile);
                    if (host.setResolvedProjectReferenceCallbacks) {
                        host.setResolvedProjectReferenceCallbacks({
                            getSourceOfProjectReferenceRedirect: getSourceOfProjectReferenceRedirect,
                            forEachResolvedProjectReference: forEachResolvedProjectReference
                        });
                    }
                }
                // check if program source files has changed in the way that can affect structure of the program
                var newSourceFiles = [];
                var modifiedSourceFiles = [];
                oldProgram.structureIsReused = 2 /* Completely */;
                // If the missing file paths are now present, it can change the progam structure,
                // and hence cant reuse the structure.
                // This is same as how we dont reuse the structure if one of the file from old program is now missing
                if (oldProgram.getMissingFilePaths().some(function (missingFilePath) { return host.fileExists(missingFilePath); })) {
                    return oldProgram.structureIsReused = 0 /* Not */;
                }
                var oldSourceFiles = oldProgram.getSourceFiles();
                var SeenPackageName;
                (function (SeenPackageName) {
                    SeenPackageName[SeenPackageName["Exists"] = 0] = "Exists";
                    SeenPackageName[SeenPackageName["Modified"] = 1] = "Modified";
                })(SeenPackageName || (SeenPackageName = {}));
                var seenPackageNames = ts.createMap();
                for (var _i = 0, oldSourceFiles_2 = oldSourceFiles; _i < oldSourceFiles_2.length; _i++) {
                    var oldSourceFile = oldSourceFiles_2[_i];
                    var newSourceFile = host.getSourceFileByPath
                        ? host.getSourceFileByPath(oldSourceFile.fileName, oldSourceFile.resolvedPath, options.target, /*onError*/ undefined, shouldCreateNewSourceFile)
                        : host.getSourceFile(oldSourceFile.fileName, options.target, /*onError*/ undefined, shouldCreateNewSourceFile); // TODO: GH#18217
                    if (!newSourceFile) {
                        return oldProgram.structureIsReused = 0 /* Not */;
                    }
                    ts.Debug.assert(!newSourceFile.redirectInfo, "Host should not return a redirect source file from `getSourceFile`");
                    var fileChanged = void 0;
                    if (oldSourceFile.redirectInfo) {
                        // We got `newSourceFile` by path, so it is actually for the unredirected file.
                        // This lets us know if the unredirected file has changed. If it has we should break the redirect.
                        if (newSourceFile !== oldSourceFile.redirectInfo.unredirected) {
                            // Underlying file has changed. Might not redirect anymore. Must rebuild program.
                            return oldProgram.structureIsReused = 0 /* Not */;
                        }
                        fileChanged = false;
                        newSourceFile = oldSourceFile; // Use the redirect.
                    }
                    else if (oldProgram.redirectTargetsMap.has(oldSourceFile.path)) {
                        // If a redirected-to source file changes, the redirect may be broken.
                        if (newSourceFile !== oldSourceFile) {
                            return oldProgram.structureIsReused = 0 /* Not */;
                        }
                        fileChanged = false;
                    }
                    else {
                        fileChanged = newSourceFile !== oldSourceFile;
                    }
                    // Since the project references havent changed, its right to set originalFileName and resolvedPath here
                    newSourceFile.path = oldSourceFile.path;
                    newSourceFile.originalFileName = oldSourceFile.originalFileName;
                    newSourceFile.resolvedPath = oldSourceFile.resolvedPath;
                    newSourceFile.fileName = oldSourceFile.fileName;
                    var packageName = oldProgram.sourceFileToPackageName.get(oldSourceFile.path);
                    if (packageName !== undefined) {
                        // If there are 2 different source files for the same package name and at least one of them changes,
                        // they might become redirects. So we must rebuild the program.
                        var prevKind = seenPackageNames.get(packageName);
                        var newKind = fileChanged ? 1 /* Modified */ : 0 /* Exists */;
                        if ((prevKind !== undefined && newKind === 1 /* Modified */) || prevKind === 1 /* Modified */) {
                            return oldProgram.structureIsReused = 0 /* Not */;
                        }
                        seenPackageNames.set(packageName, newKind);
                    }
                    if (fileChanged) {
                        // The `newSourceFile` object was created for the new program.
                        if (!ts.arrayIsEqualTo(oldSourceFile.libReferenceDirectives, newSourceFile.libReferenceDirectives, fileReferenceIsEqualTo)) {
                            // 'lib' references has changed. Matches behavior in changesAffectModuleResolution
                            return oldProgram.structureIsReused = 0 /* Not */;
                        }
                        if (oldSourceFile.hasNoDefaultLib !== newSourceFile.hasNoDefaultLib) {
                            // value of no-default-lib has changed
                            // this will affect if default library is injected into the list of files
                            oldProgram.structureIsReused = 1 /* SafeModules */;
                        }
                        // check tripleslash references
                        if (!ts.arrayIsEqualTo(oldSourceFile.referencedFiles, newSourceFile.referencedFiles, fileReferenceIsEqualTo)) {
                            // tripleslash references has changed
                            oldProgram.structureIsReused = 1 /* SafeModules */;
                        }
                        // check imports and module augmentations
                        collectExternalModuleReferences(newSourceFile);
                        if (!ts.arrayIsEqualTo(oldSourceFile.imports, newSourceFile.imports, moduleNameIsEqualTo)) {
                            // imports has changed
                            oldProgram.structureIsReused = 1 /* SafeModules */;
                        }
                        if (!ts.arrayIsEqualTo(oldSourceFile.moduleAugmentations, newSourceFile.moduleAugmentations, moduleNameIsEqualTo)) {
                            // moduleAugmentations has changed
                            oldProgram.structureIsReused = 1 /* SafeModules */;
                        }
                        if ((oldSourceFile.flags & 3145728 /* PermanentlySetIncrementalFlags */) !== (newSourceFile.flags & 3145728 /* PermanentlySetIncrementalFlags */)) {
                            // dynamicImport has changed
                            oldProgram.structureIsReused = 1 /* SafeModules */;
                        }
                        if (!ts.arrayIsEqualTo(oldSourceFile.typeReferenceDirectives, newSourceFile.typeReferenceDirectives, fileReferenceIsEqualTo)) {
                            // 'types' references has changed
                            oldProgram.structureIsReused = 1 /* SafeModules */;
                        }
                        // tentatively approve the file
                        modifiedSourceFiles.push({ oldFile: oldSourceFile, newFile: newSourceFile });
                    }
                    else if (hasInvalidatedResolution(oldSourceFile.path)) {
                        // 'module/types' references could have changed
                        oldProgram.structureIsReused = 1 /* SafeModules */;
                        // add file to the modified list so that we will resolve it later
                        modifiedSourceFiles.push({ oldFile: oldSourceFile, newFile: newSourceFile });
                    }
                    // if file has passed all checks it should be safe to reuse it
                    newSourceFiles.push(newSourceFile);
                }
                if (oldProgram.structureIsReused !== 2 /* Completely */) {
                    return oldProgram.structureIsReused;
                }
                var modifiedFiles = modifiedSourceFiles.map(function (f) { return f.oldFile; });
                for (var _a = 0, oldSourceFiles_3 = oldSourceFiles; _a < oldSourceFiles_3.length; _a++) {
                    var oldFile = oldSourceFiles_3[_a];
                    if (!ts.contains(modifiedFiles, oldFile)) {
                        for (var _b = 0, _c = oldFile.ambientModuleNames; _b < _c.length; _b++) {
                            var moduleName = _c[_b];
                            ambientModuleNameToUnmodifiedFileName.set(moduleName, oldFile.fileName);
                        }
                    }
                }
                // try to verify results of module resolution
                for (var _d = 0, modifiedSourceFiles_1 = modifiedSourceFiles; _d < modifiedSourceFiles_1.length; _d++) {
                    var _e = modifiedSourceFiles_1[_d], oldSourceFile = _e.oldFile, newSourceFile = _e.newFile;
                    var newSourceFilePath = ts.getNormalizedAbsolutePath(newSourceFile.originalFileName, currentDirectory);
                    var moduleNames = getModuleNames(newSourceFile);
                    var resolutions = resolveModuleNamesReusingOldState(moduleNames, newSourceFilePath, newSourceFile);
                    // ensure that module resolution results are still correct
                    var resolutionsChanged = ts.hasChangesInResolutions(moduleNames, resolutions, oldSourceFile.resolvedModules, ts.moduleResolutionIsEqualTo);
                    if (resolutionsChanged) {
                        oldProgram.structureIsReused = 1 /* SafeModules */;
                        newSourceFile.resolvedModules = ts.zipToMap(moduleNames, resolutions);
                    }
                    else {
                        newSourceFile.resolvedModules = oldSourceFile.resolvedModules;
                    }
                    if (resolveTypeReferenceDirectiveNamesWorker) {
                        // We lower-case all type references because npm automatically lowercases all packages. See GH#9824.
                        var typesReferenceDirectives = ts.map(newSourceFile.typeReferenceDirectives, function (ref) { return ts.toFileNameLowerCase(ref.fileName); });
                        var resolutions_1 = resolveTypeReferenceDirectiveNamesWorker(typesReferenceDirectives, newSourceFilePath, getResolvedProjectReferenceToRedirect(newSourceFile.originalFileName));
                        // ensure that types resolutions are still correct
                        var resolutionsChanged_1 = ts.hasChangesInResolutions(typesReferenceDirectives, resolutions_1, oldSourceFile.resolvedTypeReferenceDirectiveNames, ts.typeDirectiveIsEqualTo);
                        if (resolutionsChanged_1) {
                            oldProgram.structureIsReused = 1 /* SafeModules */;
                            newSourceFile.resolvedTypeReferenceDirectiveNames = ts.zipToMap(typesReferenceDirectives, resolutions_1);
                        }
                        else {
                            newSourceFile.resolvedTypeReferenceDirectiveNames = oldSourceFile.resolvedTypeReferenceDirectiveNames;
                        }
                    }
                }
                if (oldProgram.structureIsReused !== 2 /* Completely */) {
                    return oldProgram.structureIsReused;
                }
                if (host.hasChangedAutomaticTypeDirectiveNames) {
                    return oldProgram.structureIsReused = 1 /* SafeModules */;
                }
                missingFilePaths = oldProgram.getMissingFilePaths();
                refFileMap = oldProgram.getRefFileMap();
                // update fileName -> file mapping
                ts.Debug.assert(newSourceFiles.length === oldProgram.getSourceFiles().length);
                for (var _f = 0, newSourceFiles_1 = newSourceFiles; _f < newSourceFiles_1.length; _f++) {
                    var newSourceFile = newSourceFiles_1[_f];
                    filesByName.set(newSourceFile.path, newSourceFile);
                }
                var oldFilesByNameMap = oldProgram.getFilesByNameMap();
                oldFilesByNameMap.forEach(function (oldFile, path) {
                    if (!oldFile) {
                        filesByName.set(path, oldFile);
                        return;
                    }
                    if (oldFile.path === path) {
                        // Set the file as found during node modules search if it was found that way in old progra,
                        if (oldProgram.isSourceFileFromExternalLibrary(oldFile)) {
                            sourceFilesFoundSearchingNodeModules.set(oldFile.path, true);
                        }
                        return;
                    }
                    filesByName.set(path, filesByName.get(oldFile.path));
                });
                files = newSourceFiles;
                fileProcessingDiagnostics = oldProgram.getFileProcessingDiagnostics();
                for (var _g = 0, modifiedSourceFiles_2 = modifiedSourceFiles; _g < modifiedSourceFiles_2.length; _g++) {
                    var modifiedFile = modifiedSourceFiles_2[_g];
                    fileProcessingDiagnostics.reattachFileDiagnostics(modifiedFile.newFile);
                }
                resolvedTypeReferenceDirectives = oldProgram.getResolvedTypeReferenceDirectives();
                sourceFileToPackageName = oldProgram.sourceFileToPackageName;
                redirectTargetsMap = oldProgram.redirectTargetsMap;
                return oldProgram.structureIsReused = 2 /* Completely */;
            }
            function getEmitHost(writeFileCallback) {
                return __assign(__assign({ getPrependNodes: getPrependNodes,
                    getCanonicalFileName: getCanonicalFileName, getCommonSourceDirectory: program.getCommonSourceDirectory, getCompilerOptions: program.getCompilerOptions, getCurrentDirectory: function () { return currentDirectory; }, getNewLine: function () { return host.getNewLine(); }, getSourceFile: program.getSourceFile, getSourceFileByPath: program.getSourceFileByPath, getSourceFiles: program.getSourceFiles, getLibFileFromReference: program.getLibFileFromReference, isSourceFileFromExternalLibrary: isSourceFileFromExternalLibrary,
                    getResolvedProjectReferenceToRedirect: getResolvedProjectReferenceToRedirect,
                    isSourceOfProjectReferenceRedirect: isSourceOfProjectReferenceRedirect,
                    getProbableSymlinks: getProbableSymlinks, writeFile: writeFileCallback || (function (fileName, data, writeByteOrderMark, onError, sourceFiles) { return host.writeFile(fileName, data, writeByteOrderMark, onError, sourceFiles); }), isEmitBlocked: isEmitBlocked, readFile: function (f) { return host.readFile(f); }, fileExists: function (f) {
                        // Use local caches
                        var path = toPath(f);
                        if (getSourceFileByPath(path))
                            return true;
                        if (ts.contains(missingFilePaths, path))
                            return false;
                        // Before falling back to the host
                        return host.fileExists(f);
                    } }, (host.directoryExists ? { directoryExists: function (f) { return host.directoryExists(f); } } : {})), { useCaseSensitiveFileNames: function () { return host.useCaseSensitiveFileNames(); }, getProgramBuildInfo: function () { return program.getProgramBuildInfo && program.getProgramBuildInfo(); }, getSourceFileFromReference: function (file, ref) { return program.getSourceFileFromReference(file, ref); }, redirectTargetsMap: redirectTargetsMap });
            }
            function emitBuildInfo(writeFileCallback) {
                ts.Debug.assert(!options.out && !options.outFile);
                ts.performance.mark("beforeEmit");
                var emitResult = ts.emitFiles(ts.notImplementedResolver, getEmitHost(writeFileCallback), 
                /*targetSourceFile*/ undefined, 
                /*transformers*/ ts.noTransformers, 
                /*emitOnlyDtsFiles*/ false, 
                /*onlyBuildInfo*/ true);
                ts.performance.mark("afterEmit");
                ts.performance.measure("Emit", "beforeEmit", "afterEmit");
                return emitResult;
            }
            function getResolvedProjectReferences() {
                return resolvedProjectReferences;
            }
            function getProjectReferences() {
                return projectReferences;
            }
            function getPrependNodes() {
                return createPrependNodes(projectReferences, function (_ref, index) { return resolvedProjectReferences[index].commandLine; }, function (fileName) {
                    var path = toPath(fileName);
                    var sourceFile = getSourceFileByPath(path);
                    return sourceFile ? sourceFile.text : filesByName.has(path) ? undefined : host.readFile(path);
                });
            }
            function isSourceFileFromExternalLibrary(file) {
                return !!sourceFilesFoundSearchingNodeModules.get(file.path);
            }
            function isSourceFileDefaultLibrary(file) {
                if (file.hasNoDefaultLib) {
                    return true;
                }
                if (!options.noLib) {
                    return false;
                }
                // If '--lib' is not specified, include default library file according to '--target'
                // otherwise, using options specified in '--lib' instead of '--target' default library file
                var equalityComparer = host.useCaseSensitiveFileNames() ? ts.equateStringsCaseSensitive : ts.equateStringsCaseInsensitive;
                if (!options.lib) {
                    return equalityComparer(file.fileName, getDefaultLibraryFileName());
                }
                else {
                    return ts.some(options.lib, function (libFileName) { return equalityComparer(file.fileName, ts.combinePaths(defaultLibraryPath, libFileName)); });
                }
            }
            function getDiagnosticsProducingTypeChecker() {
                return diagnosticsProducingTypeChecker || (diagnosticsProducingTypeChecker = ts.createTypeChecker(program, /*produceDiagnostics:*/ true));
            }
            function dropDiagnosticsProducingTypeChecker() {
                diagnosticsProducingTypeChecker = undefined;
            }
            function getTypeChecker() {
                return noDiagnosticsTypeChecker || (noDiagnosticsTypeChecker = ts.createTypeChecker(program, /*produceDiagnostics:*/ false));
            }
            function emit(sourceFile, writeFileCallback, cancellationToken, emitOnlyDtsFiles, transformers, forceDtsEmit) {
                return runWithCancellationToken(function () { return emitWorker(program, sourceFile, writeFileCallback, cancellationToken, emitOnlyDtsFiles, transformers, forceDtsEmit); });
            }
            function isEmitBlocked(emitFileName) {
                return hasEmitBlockingDiagnostics.has(toPath(emitFileName));
            }
            function emitWorker(program, sourceFile, writeFileCallback, cancellationToken, emitOnlyDtsFiles, customTransformers, forceDtsEmit) {
                if (!forceDtsEmit) {
                    var result = handleNoEmitOptions(program, sourceFile, cancellationToken);
                    if (result)
                        return result;
                }
                // Create the emit resolver outside of the "emitTime" tracking code below.  That way
                // any cost associated with it (like type checking) are appropriate associated with
                // the type-checking counter.
                //
                // If the -out option is specified, we should not pass the source file to getEmitResolver.
                // This is because in the -out scenario all files need to be emitted, and therefore all
                // files need to be type checked. And the way to specify that all files need to be type
                // checked is to not pass the file to getEmitResolver.
                var emitResolver = getDiagnosticsProducingTypeChecker().getEmitResolver((options.outFile || options.out) ? undefined : sourceFile, cancellationToken);
                ts.performance.mark("beforeEmit");
                var emitResult = ts.emitFiles(emitResolver, getEmitHost(writeFileCallback), sourceFile, ts.getTransformers(options, customTransformers, emitOnlyDtsFiles), emitOnlyDtsFiles, 
                /*onlyBuildInfo*/ false, forceDtsEmit);
                ts.performance.mark("afterEmit");
                ts.performance.measure("Emit", "beforeEmit", "afterEmit");
                return emitResult;
            }
            function getSourceFile(fileName) {
                return getSourceFileByPath(toPath(fileName));
            }
            function getSourceFileByPath(path) {
                return filesByName.get(path) || undefined;
            }
            function getDiagnosticsHelper(sourceFile, getDiagnostics, cancellationToken) {
                if (sourceFile) {
                    return getDiagnostics(sourceFile, cancellationToken);
                }
                return ts.sortAndDeduplicateDiagnostics(ts.flatMap(program.getSourceFiles(), function (sourceFile) {
                    if (cancellationToken) {
                        cancellationToken.throwIfCancellationRequested();
                    }
                    return getDiagnostics(sourceFile, cancellationToken);
                }));
            }
            function getSyntacticDiagnostics(sourceFile, cancellationToken) {
                return getDiagnosticsHelper(sourceFile, getSyntacticDiagnosticsForFile, cancellationToken);
            }
            function getSemanticDiagnostics(sourceFile, cancellationToken) {
                return getDiagnosticsHelper(sourceFile, getSemanticDiagnosticsForFile, cancellationToken);
            }
            function getBindAndCheckDiagnostics(sourceFile, cancellationToken) {
                return getBindAndCheckDiagnosticsForFile(sourceFile, cancellationToken);
            }
            function getProgramDiagnostics(sourceFile) {
                if (ts.skipTypeChecking(sourceFile, options, program)) {
                    return ts.emptyArray;
                }
                var fileProcessingDiagnosticsInFile = fileProcessingDiagnostics.getDiagnostics(sourceFile.fileName);
                var programDiagnosticsInFile = programDiagnostics.getDiagnostics(sourceFile.fileName);
                var diagnostics;
                for (var _i = 0, _a = [fileProcessingDiagnosticsInFile, programDiagnosticsInFile]; _i < _a.length; _i++) {
                    var diags = _a[_i];
                    if (diags) {
                        for (var _b = 0, diags_4 = diags; _b < diags_4.length; _b++) {
                            var diag = diags_4[_b];
                            if (shouldReportDiagnostic(diag)) {
                                diagnostics = ts.append(diagnostics, diag);
                            }
                        }
                    }
                }
                return diagnostics || ts.emptyArray;
            }
            function getDeclarationDiagnostics(sourceFile, cancellationToken) {
                var options = program.getCompilerOptions();
                // collect diagnostics from the program only once if either no source file was specified or out/outFile is set (bundled emit)
                if (!sourceFile || options.out || options.outFile) {
                    return getDeclarationDiagnosticsWorker(sourceFile, cancellationToken);
                }
                else {
                    return getDiagnosticsHelper(sourceFile, getDeclarationDiagnosticsForFile, cancellationToken);
                }
            }
            function getSyntacticDiagnosticsForFile(sourceFile) {
                // For JavaScript files, we report semantic errors for using TypeScript-only
                // constructs from within a JavaScript file as syntactic errors.
                if (ts.isSourceFileJS(sourceFile)) {
                    if (!sourceFile.additionalSyntacticDiagnostics) {
                        sourceFile.additionalSyntacticDiagnostics = getJSSyntacticDiagnosticsForFile(sourceFile);
                    }
                    return ts.concatenate(sourceFile.additionalSyntacticDiagnostics, sourceFile.parseDiagnostics);
                }
                return sourceFile.parseDiagnostics;
            }
            function runWithCancellationToken(func) {
                try {
                    return func();
                }
                catch (e) {
                    if (e instanceof ts.OperationCanceledException) {
                        // We were canceled while performing the operation.  Because our type checker
                        // might be a bad state, we need to throw it away.
                        //
                        // Note: we are overly aggressive here.  We do not actually *have* to throw away
                        // the "noDiagnosticsTypeChecker".  However, for simplicity, i'd like to keep
                        // the lifetimes of these two TypeCheckers the same.  Also, we generally only
                        // cancel when the user has made a change anyways.  And, in that case, we (the
                        // program instance) will get thrown away anyways.  So trying to keep one of
                        // these type checkers alive doesn't serve much purpose.
                        noDiagnosticsTypeChecker = undefined;
                        diagnosticsProducingTypeChecker = undefined;
                    }
                    throw e;
                }
            }
            function getSemanticDiagnosticsForFile(sourceFile, cancellationToken) {
                return ts.concatenate(getBindAndCheckDiagnosticsForFile(sourceFile, cancellationToken), getProgramDiagnostics(sourceFile));
            }
            function getBindAndCheckDiagnosticsForFile(sourceFile, cancellationToken) {
                return getAndCacheDiagnostics(sourceFile, cancellationToken, cachedBindAndCheckDiagnosticsForFile, getBindAndCheckDiagnosticsForFileNoCache);
            }
            function getBindAndCheckDiagnosticsForFileNoCache(sourceFile, cancellationToken) {
                return runWithCancellationToken(function () {
                    if (ts.skipTypeChecking(sourceFile, options, program)) {
                        return ts.emptyArray;
                    }
                    var typeChecker = getDiagnosticsProducingTypeChecker();
                    ts.Debug.assert(!!sourceFile.bindDiagnostics);
                    var isCheckJs = ts.isCheckJsEnabledForFile(sourceFile, options);
                    var isTsNoCheck = !!sourceFile.checkJsDirective && sourceFile.checkJsDirective.enabled === false;
                    // By default, only type-check .ts, .tsx, 'Deferred' and 'External' files (external files are added by plugins)
                    var includeBindAndCheckDiagnostics = !isTsNoCheck && (sourceFile.scriptKind === 3 /* TS */ || sourceFile.scriptKind === 4 /* TSX */ ||
                        sourceFile.scriptKind === 5 /* External */ || isCheckJs || sourceFile.scriptKind === 7 /* Deferred */);
                    var bindDiagnostics = includeBindAndCheckDiagnostics ? sourceFile.bindDiagnostics : ts.emptyArray;
                    var checkDiagnostics = includeBindAndCheckDiagnostics ? typeChecker.getDiagnostics(sourceFile, cancellationToken) : ts.emptyArray;
                    var diagnostics;
                    for (var _i = 0, _a = [bindDiagnostics, checkDiagnostics, isCheckJs ? sourceFile.jsDocDiagnostics : undefined]; _i < _a.length; _i++) {
                        var diags = _a[_i];
                        if (diags) {
                            for (var _b = 0, diags_5 = diags; _b < diags_5.length; _b++) {
                                var diag = diags_5[_b];
                                if (shouldReportDiagnostic(diag)) {
                                    diagnostics = ts.append(diagnostics, diag);
                                }
                            }
                        }
                    }
                    return diagnostics || ts.emptyArray;
                });
            }
            function getSuggestionDiagnostics(sourceFile, cancellationToken) {
                return runWithCancellationToken(function () {
                    return getDiagnosticsProducingTypeChecker().getSuggestionDiagnostics(sourceFile, cancellationToken);
                });
            }
            /**
             * Skip errors if previous line start with '// @ts-ignore' comment, not counting non-empty non-comment lines
             */
            function shouldReportDiagnostic(diagnostic) {
                var file = diagnostic.file, start = diagnostic.start;
                if (file) {
                    var lineStarts = ts.getLineStarts(file);
                    var line = ts.computeLineAndCharacterOfPosition(lineStarts, start).line; // TODO: GH#18217
                    while (line > 0) {
                        var previousLineText = file.text.slice(lineStarts[line - 1], lineStarts[line]);
                        var result = ignoreDiagnosticCommentRegEx.exec(previousLineText);
                        if (!result) {
                            // non-empty line
                            return true;
                        }
                        if (result[3]) {
                            // @ts-ignore
                            return false;
                        }
                        line--;
                    }
                }
                return true;
            }
            function getJSSyntacticDiagnosticsForFile(sourceFile) {
                return runWithCancellationToken(function () {
                    var diagnostics = [];
                    var parent = sourceFile;
                    walk(sourceFile);
                    return diagnostics;
                    function walk(node) {
                        // Return directly from the case if the given node doesnt want to visit each child
                        // Otherwise break to visit each child
                        switch (parent.kind) {
                            case 156 /* Parameter */:
                            case 159 /* PropertyDeclaration */:
                            case 161 /* MethodDeclaration */:
                                if (parent.questionToken === node) {
                                    diagnostics.push(createDiagnosticForNode(node, ts.Diagnostics.The_0_modifier_can_only_be_used_in_TypeScript_files, "?"));
                                    return;
                                }
                            // falls through
                            case 160 /* MethodSignature */:
                            case 162 /* Constructor */:
                            case 163 /* GetAccessor */:
                            case 164 /* SetAccessor */:
                            case 201 /* FunctionExpression */:
                            case 244 /* FunctionDeclaration */:
                            case 202 /* ArrowFunction */:
                            case 242 /* VariableDeclaration */:
                                // type annotation
                                if (parent.type === node) {
                                    diagnostics.push(createDiagnosticForNode(node, ts.Diagnostics.Type_annotations_can_only_be_used_in_TypeScript_files));
                                    return;
                                }
                        }
                        switch (node.kind) {
                            case 255 /* ImportClause */:
                                if (node.isTypeOnly) {
                                    diagnostics.push(createDiagnosticForNode(node.parent, ts.Diagnostics._0_declarations_can_only_be_used_in_TypeScript_files, "import type"));
                                    return;
                                }
                                break;
                            case 260 /* ExportDeclaration */:
                                if (node.isTypeOnly) {
                                    diagnostics.push(createDiagnosticForNode(node, ts.Diagnostics._0_declarations_can_only_be_used_in_TypeScript_files, "export type"));
                                    return;
                                }
                                break;
                            case 253 /* ImportEqualsDeclaration */:
                                diagnostics.push(createDiagnosticForNode(node, ts.Diagnostics.import_can_only_be_used_in_TypeScript_files));
                                return;
                            case 259 /* ExportAssignment */:
                                if (node.isExportEquals) {
                                    diagnostics.push(createDiagnosticForNode(node, ts.Diagnostics.export_can_only_be_used_in_TypeScript_files));
                                    return;
                                }
                                break;
                            case 279 /* HeritageClause */:
                                var heritageClause = node;
                                if (heritageClause.token === 113 /* ImplementsKeyword */) {
                                    diagnostics.push(createDiagnosticForNode(node, ts.Diagnostics.implements_clauses_can_only_be_used_in_TypeScript_files));
                                    return;
                                }
                                break;
                            case 246 /* InterfaceDeclaration */:
                                var interfaceKeyword = ts.tokenToString(114 /* InterfaceKeyword */);
                                ts.Debug.assertDefined(interfaceKeyword);
                                diagnostics.push(createDiagnosticForNode(node, ts.Diagnostics._0_declarations_can_only_be_used_in_TypeScript_files, interfaceKeyword));
                                return;
                            case 249 /* ModuleDeclaration */:
                                var moduleKeyword = node.flags & 16 /* Namespace */ ? ts.tokenToString(136 /* NamespaceKeyword */) : ts.tokenToString(135 /* ModuleKeyword */);
                                ts.Debug.assertDefined(moduleKeyword);
                                diagnostics.push(createDiagnosticForNode(node, ts.Diagnostics._0_declarations_can_only_be_used_in_TypeScript_files, moduleKeyword));
                                return;
                            case 247 /* TypeAliasDeclaration */:
                                diagnostics.push(createDiagnosticForNode(node, ts.Diagnostics.Type_aliases_can_only_be_used_in_TypeScript_files));
                                return;
                            case 248 /* EnumDeclaration */:
                                var enumKeyword = ts.Debug.assertDefined(ts.tokenToString(88 /* EnumKeyword */));
                                diagnostics.push(createDiagnosticForNode(node, ts.Diagnostics._0_declarations_can_only_be_used_in_TypeScript_files, enumKeyword));
                                return;
                            case 218 /* NonNullExpression */:
                                diagnostics.push(createDiagnosticForNode(node, ts.Diagnostics.Non_null_assertions_can_only_be_used_in_TypeScript_files));
                                return;
                            case 217 /* AsExpression */:
                                diagnostics.push(createDiagnosticForNode(node.type, ts.Diagnostics.Type_assertion_expressions_can_only_be_used_in_TypeScript_files));
                                return;
                            case 199 /* TypeAssertionExpression */:
                                ts.Debug.fail(); // Won't parse these in a JS file anyway, as they are interpreted as JSX.
                        }
                        var prevParent = parent;
                        parent = node;
                        ts.forEachChild(node, walk, walkArray);
                        parent = prevParent;
                    }
                    function walkArray(nodes) {
                        if (parent.decorators === nodes && !options.experimentalDecorators) {
                            diagnostics.push(createDiagnosticForNode(parent, ts.Diagnostics.Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_the_experimentalDecorators_option_in_your_tsconfig_or_jsconfig_to_remove_this_warning));
                        }
                        switch (parent.kind) {
                            case 245 /* ClassDeclaration */:
                            case 214 /* ClassExpression */:
                            case 161 /* MethodDeclaration */:
                            case 162 /* Constructor */:
                            case 163 /* GetAccessor */:
                            case 164 /* SetAccessor */:
                            case 201 /* FunctionExpression */:
                            case 244 /* FunctionDeclaration */:
                            case 202 /* ArrowFunction */:
                                // Check type parameters
                                if (nodes === parent.typeParameters) {
                                    diagnostics.push(createDiagnosticForNodeArray(nodes, ts.Diagnostics.Type_parameter_declarations_can_only_be_used_in_TypeScript_files));
                                    return;
                                }
                            // falls through
                            case 225 /* VariableStatement */:
                                // Check modifiers
                                if (nodes === parent.modifiers) {
                                    return checkModifiers(parent.modifiers, parent.kind === 225 /* VariableStatement */);
                                }
                                break;
                            case 159 /* PropertyDeclaration */:
                                // Check modifiers of property declaration
                                if (nodes === parent.modifiers) {
                                    for (var _i = 0, _a = nodes; _i < _a.length; _i++) {
                                        var modifier = _a[_i];
                                        if (modifier.kind !== 120 /* StaticKeyword */) {
                                            diagnostics.push(createDiagnosticForNode(modifier, ts.Diagnostics.The_0_modifier_can_only_be_used_in_TypeScript_files, ts.tokenToString(modifier.kind)));
                                        }
                                    }
                                    return;
                                }
                                break;
                            case 156 /* Parameter */:
                                // Check modifiers of parameter declaration
                                if (nodes === parent.modifiers) {
                                    diagnostics.push(createDiagnosticForNodeArray(nodes, ts.Diagnostics.Parameter_modifiers_can_only_be_used_in_TypeScript_files));
                                    return;
                                }
                                break;
                            case 196 /* CallExpression */:
                            case 197 /* NewExpression */:
                            case 216 /* ExpressionWithTypeArguments */:
                            case 267 /* JsxSelfClosingElement */:
                            case 268 /* JsxOpeningElement */:
                            case 198 /* TaggedTemplateExpression */:
                                // Check type arguments
                                if (nodes === parent.typeArguments) {
                                    diagnostics.push(createDiagnosticForNodeArray(nodes, ts.Diagnostics.Type_arguments_can_only_be_used_in_TypeScript_files));
                                    return;
                                }
                                break;
                        }
                        for (var _b = 0, nodes_6 = nodes; _b < nodes_6.length; _b++) {
                            var node = nodes_6[_b];
                            walk(node);
                        }
                    }
                    function checkModifiers(modifiers, isConstValid) {
                        for (var _i = 0, modifiers_1 = modifiers; _i < modifiers_1.length; _i++) {
                            var modifier = modifiers_1[_i];
                            switch (modifier.kind) {
                                case 81 /* ConstKeyword */:
                                    if (isConstValid) {
                                        continue;
                                    }
                                // to report error,
                                // falls through
                                case 119 /* PublicKeyword */:
                                case 117 /* PrivateKeyword */:
                                case 118 /* ProtectedKeyword */:
                                case 138 /* ReadonlyKeyword */:
                                case 130 /* DeclareKeyword */:
                                case 122 /* AbstractKeyword */:
                                    diagnostics.push(createDiagnosticForNode(modifier, ts.Diagnostics.The_0_modifier_can_only_be_used_in_TypeScript_files, ts.tokenToString(modifier.kind)));
                                    break;
                                // These are all legal modifiers.
                                case 120 /* StaticKeyword */:
                                case 89 /* ExportKeyword */:
                                case 84 /* DefaultKeyword */:
                            }
                        }
                    }
                    function createDiagnosticForNodeArray(nodes, message, arg0, arg1, arg2) {
                        var start = nodes.pos;
                        return ts.createFileDiagnostic(sourceFile, start, nodes.end - start, message, arg0, arg1, arg2);
                    }
                    // Since these are syntactic diagnostics, parent might not have been set
                    // this means the sourceFile cannot be infered from the node
                    function createDiagnosticForNode(node, message, arg0, arg1, arg2) {
                        return ts.createDiagnosticForNodeInSourceFile(sourceFile, node, message, arg0, arg1, arg2);
                    }
                });
            }
            function getDeclarationDiagnosticsWorker(sourceFile, cancellationToken) {
                return getAndCacheDiagnostics(sourceFile, cancellationToken, cachedDeclarationDiagnosticsForFile, getDeclarationDiagnosticsForFileNoCache);
            }
            function getDeclarationDiagnosticsForFileNoCache(sourceFile, cancellationToken) {
                return runWithCancellationToken(function () {
                    var resolver = getDiagnosticsProducingTypeChecker().getEmitResolver(sourceFile, cancellationToken);
                    // Don't actually write any files since we're just getting diagnostics.
                    return ts.getDeclarationDiagnostics(getEmitHost(ts.noop), resolver, sourceFile) || ts.emptyArray;
                });
            }
            function getAndCacheDiagnostics(sourceFile, cancellationToken, cache, getDiagnostics) {
                var cachedResult = sourceFile
                    ? cache.perFile && cache.perFile.get(sourceFile.path)
                    : cache.allDiagnostics;
                if (cachedResult) {
                    return cachedResult;
                }
                var result = getDiagnostics(sourceFile, cancellationToken);
                if (sourceFile) {
                    if (!cache.perFile) {
                        cache.perFile = ts.createMap();
                    }
                    cache.perFile.set(sourceFile.path, result);
                }
                else {
                    cache.allDiagnostics = result;
                }
                return result;
            }
            function getDeclarationDiagnosticsForFile(sourceFile, cancellationToken) {
                return sourceFile.isDeclarationFile ? [] : getDeclarationDiagnosticsWorker(sourceFile, cancellationToken);
            }
            function getOptionsDiagnostics() {
                return ts.sortAndDeduplicateDiagnostics(ts.concatenate(fileProcessingDiagnostics.getGlobalDiagnostics(), ts.concatenate(programDiagnostics.getGlobalDiagnostics(), getOptionsDiagnosticsOfConfigFile())));
            }
            function getOptionsDiagnosticsOfConfigFile() {
                if (!options.configFile) {
                    return ts.emptyArray;
                }
                var diagnostics = programDiagnostics.getDiagnostics(options.configFile.fileName);
                forEachResolvedProjectReference(function (resolvedRef) {
                    if (resolvedRef) {
                        diagnostics = ts.concatenate(diagnostics, programDiagnostics.getDiagnostics(resolvedRef.sourceFile.fileName));
                    }
                });
                return diagnostics;
            }
            function getGlobalDiagnostics() {
                return rootNames.length ? ts.sortAndDeduplicateDiagnostics(getDiagnosticsProducingTypeChecker().getGlobalDiagnostics().slice()) : ts.emptyArray;
            }
            function getConfigFileParsingDiagnostics() {
                return configFileParsingDiagnostics || ts.emptyArray;
            }
            function processRootFile(fileName, isDefaultLib, ignoreNoDefaultLib) {
                processSourceFile(ts.normalizePath(fileName), isDefaultLib, ignoreNoDefaultLib, /*packageId*/ undefined);
            }
            function fileReferenceIsEqualTo(a, b) {
                return a.fileName === b.fileName;
            }
            function moduleNameIsEqualTo(a, b) {
                return a.kind === 75 /* Identifier */
                    ? b.kind === 75 /* Identifier */ && a.escapedText === b.escapedText
                    : b.kind === 10 /* StringLiteral */ && a.text === b.text;
            }
            function collectExternalModuleReferences(file) {
                if (file.imports) {
                    return;
                }
                var isJavaScriptFile = ts.isSourceFileJS(file);
                var isExternalModuleFile = ts.isExternalModule(file);
                // file.imports may not be undefined if there exists dynamic import
                var imports;
                var moduleAugmentations;
                var ambientModules;
                // If we are importing helpers, we need to add a synthetic reference to resolve the
                // helpers library.
                if (options.importHelpers
                    && (options.isolatedModules || isExternalModuleFile)
                    && !file.isDeclarationFile) {
                    // synthesize 'import "tslib"' declaration
                    var externalHelpersModuleReference = ts.createLiteral(ts.externalHelpersModuleNameText);
                    var importDecl = ts.createImportDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, /*importClause*/ undefined, externalHelpersModuleReference);
                    ts.addEmitFlags(importDecl, 67108864 /* NeverApplyImportHelper */);
                    externalHelpersModuleReference.parent = importDecl;
                    importDecl.parent = file;
                    imports = [externalHelpersModuleReference];
                }
                for (var _i = 0, _a = file.statements; _i < _a.length; _i++) {
                    var node = _a[_i];
                    collectModuleReferences(node, /*inAmbientModule*/ false);
                }
                if ((file.flags & 1048576 /* PossiblyContainsDynamicImport */) || isJavaScriptFile) {
                    collectDynamicImportOrRequireCalls(file);
                }
                file.imports = imports || ts.emptyArray;
                file.moduleAugmentations = moduleAugmentations || ts.emptyArray;
                file.ambientModuleNames = ambientModules || ts.emptyArray;
                return;
                function collectModuleReferences(node, inAmbientModule) {
                    if (ts.isAnyImportOrReExport(node)) {
                        var moduleNameExpr = ts.getExternalModuleName(node);
                        // TypeScript 1.0 spec (April 2014): 12.1.6
                        // An ExternalImportDeclaration in an AmbientExternalModuleDeclaration may reference other external modules
                        // only through top - level external module names. Relative external module names are not permitted.
                        if (moduleNameExpr && ts.isStringLiteral(moduleNameExpr) && moduleNameExpr.text && (!inAmbientModule || !ts.isExternalModuleNameRelative(moduleNameExpr.text))) {
                            imports = ts.append(imports, moduleNameExpr);
                        }
                    }
                    else if (ts.isModuleDeclaration(node)) {
                        if (ts.isAmbientModule(node) && (inAmbientModule || ts.hasModifier(node, 2 /* Ambient */) || file.isDeclarationFile)) {
                            var nameText = ts.getTextOfIdentifierOrLiteral(node.name);
                            // Ambient module declarations can be interpreted as augmentations for some existing external modules.
                            // This will happen in two cases:
                            // - if current file is external module then module augmentation is a ambient module declaration defined in the top level scope
                            // - if current file is not external module then module augmentation is an ambient module declaration with non-relative module name
                            //   immediately nested in top level ambient module declaration .
                            if (isExternalModuleFile || (inAmbientModule && !ts.isExternalModuleNameRelative(nameText))) {
                                (moduleAugmentations || (moduleAugmentations = [])).push(node.name);
                            }
                            else if (!inAmbientModule) {
                                if (file.isDeclarationFile) {
                                    // for global .d.ts files record name of ambient module
                                    (ambientModules || (ambientModules = [])).push(nameText);
                                }
                                // An AmbientExternalModuleDeclaration declares an external module.
                                // This type of declaration is permitted only in the global module.
                                // The StringLiteral must specify a top - level external module name.
                                // Relative external module names are not permitted
                                // NOTE: body of ambient module is always a module block, if it exists
                                var body = node.body;
                                if (body) {
                                    for (var _i = 0, _a = body.statements; _i < _a.length; _i++) {
                                        var statement = _a[_i];
                                        collectModuleReferences(statement, /*inAmbientModule*/ true);
                                    }
                                }
                            }
                        }
                    }
                }
                function collectDynamicImportOrRequireCalls(file) {
                    var r = /import|require/g;
                    while (r.exec(file.text) !== null) { // eslint-disable-line no-null/no-null
                        var node = getNodeAtPosition(file, r.lastIndex);
                        if (ts.isRequireCall(node, /*checkArgumentIsStringLiteralLike*/ true)) {
                            imports = ts.append(imports, node.arguments[0]);
                        }
                        // we have to check the argument list has length of 1. We will still have to process these even though we have parsing error.
                        else if (ts.isImportCall(node) && node.arguments.length === 1 && ts.isStringLiteralLike(node.arguments[0])) {
                            imports = ts.append(imports, node.arguments[0]);
                        }
                        else if (ts.isLiteralImportTypeNode(node)) {
                            imports = ts.append(imports, node.argument.literal);
                        }
                    }
                }
                /** Returns a token if position is in [start-of-leading-trivia, end), includes JSDoc only in JS files */
                function getNodeAtPosition(sourceFile, position) {
                    var current = sourceFile;
                    var getContainingChild = function (child) {
                        if (child.pos <= position && (position < child.end || (position === child.end && (child.kind === 1 /* EndOfFileToken */)))) {
                            return child;
                        }
                    };
                    while (true) {
                        var child = isJavaScriptFile && ts.hasJSDocNodes(current) && ts.forEach(current.jsDoc, getContainingChild) || ts.forEachChild(current, getContainingChild);
                        if (!child) {
                            return current;
                        }
                        current = child;
                    }
                }
            }
            function getLibFileFromReference(ref) {
                var libName = ts.toFileNameLowerCase(ref.fileName);
                var libFileName = ts.libMap.get(libName);
                if (libFileName) {
                    return getSourceFile(ts.combinePaths(defaultLibraryPath, libFileName));
                }
            }
            /** This should have similar behavior to 'processSourceFile' without diagnostics or mutation. */
            function getSourceFileFromReference(referencingFile, ref) {
                return getSourceFileFromReferenceWorker(resolveTripleslashReference(ref.fileName, referencingFile.fileName), function (fileName) { return filesByName.get(toPath(fileName)) || undefined; });
            }
            function getSourceFileFromReferenceWorker(fileName, getSourceFile, fail, refFile) {
                if (ts.hasExtension(fileName)) {
                    var canonicalFileName_1 = host.getCanonicalFileName(fileName);
                    if (!options.allowNonTsExtensions && !ts.forEach(supportedExtensionsWithJsonIfResolveJsonModule, function (extension) { return ts.fileExtensionIs(canonicalFileName_1, extension); })) {
                        if (fail) {
                            if (ts.hasJSFileExtension(canonicalFileName_1)) {
                                fail(ts.Diagnostics.File_0_is_a_JavaScript_file_Did_you_mean_to_enable_the_allowJs_option, fileName);
                            }
                            else {
                                fail(ts.Diagnostics.File_0_has_an_unsupported_extension_The_only_supported_extensions_are_1, fileName, "'" + supportedExtensions.join("', '") + "'");
                            }
                        }
                        return undefined;
                    }
                    var sourceFile = getSourceFile(fileName);
                    if (fail) {
                        if (!sourceFile) {
                            var redirect = getProjectReferenceRedirect(fileName);
                            if (redirect) {
                                fail(ts.Diagnostics.Output_file_0_has_not_been_built_from_source_file_1, redirect, fileName);
                            }
                            else {
                                fail(ts.Diagnostics.File_0_not_found, fileName);
                            }
                        }
                        else if (refFile && canonicalFileName_1 === host.getCanonicalFileName(refFile.fileName)) {
                            fail(ts.Diagnostics.A_file_cannot_have_a_reference_to_itself);
                        }
                    }
                    return sourceFile;
                }
                else {
                    var sourceFileNoExtension = options.allowNonTsExtensions && getSourceFile(fileName);
                    if (sourceFileNoExtension)
                        return sourceFileNoExtension;
                    if (fail && options.allowNonTsExtensions) {
                        fail(ts.Diagnostics.File_0_not_found, fileName);
                        return undefined;
                    }
                    var sourceFileWithAddedExtension = ts.forEach(supportedExtensions, function (extension) { return getSourceFile(fileName + extension); });
                    if (fail && !sourceFileWithAddedExtension)
                        fail(ts.Diagnostics.File_0_not_found, fileName + ".ts" /* Ts */);
                    return sourceFileWithAddedExtension;
                }
            }
            /** This has side effects through `findSourceFile`. */
            function processSourceFile(fileName, isDefaultLib, ignoreNoDefaultLib, packageId, refFile) {
                getSourceFileFromReferenceWorker(fileName, function (fileName) { return findSourceFile(fileName, toPath(fileName), isDefaultLib, ignoreNoDefaultLib, refFile, packageId); }, // TODO: GH#18217
                function (diagnostic) {
                    var args = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        args[_i - 1] = arguments[_i];
                    }
                    return fileProcessingDiagnostics.add(createRefFileDiagnostic.apply(void 0, __spreadArrays([refFile, diagnostic], args)));
                }, refFile && refFile.file);
            }
            function reportFileNamesDifferOnlyInCasingError(fileName, existingFile, refFile) {
                var refs = !refFile ? refFileMap && refFileMap.get(existingFile.path) : undefined;
                var refToReportErrorOn = refs && ts.find(refs, function (ref) { return ref.referencedFileName === existingFile.fileName; });
                fileProcessingDiagnostics.add(refToReportErrorOn ?
                    createFileDiagnosticAtReference(refToReportErrorOn, ts.Diagnostics.Already_included_file_name_0_differs_from_file_name_1_only_in_casing, existingFile.fileName, fileName) :
                    createRefFileDiagnostic(refFile, ts.Diagnostics.File_name_0_differs_from_already_included_file_name_1_only_in_casing, fileName, existingFile.fileName));
            }
            function createRedirectSourceFile(redirectTarget, unredirected, fileName, path, resolvedPath, originalFileName) {
                var redirect = Object.create(redirectTarget);
                redirect.fileName = fileName;
                redirect.path = path;
                redirect.resolvedPath = resolvedPath;
                redirect.originalFileName = originalFileName;
                redirect.redirectInfo = { redirectTarget: redirectTarget, unredirected: unredirected };
                sourceFilesFoundSearchingNodeModules.set(path, currentNodeModulesDepth > 0);
                Object.defineProperties(redirect, {
                    id: {
                        get: function () { return this.redirectInfo.redirectTarget.id; },
                        set: function (value) { this.redirectInfo.redirectTarget.id = value; },
                    },
                    symbol: {
                        get: function () { return this.redirectInfo.redirectTarget.symbol; },
                        set: function (value) { this.redirectInfo.redirectTarget.symbol = value; },
                    },
                });
                return redirect;
            }
            // Get source file from normalized fileName
            function findSourceFile(fileName, path, isDefaultLib, ignoreNoDefaultLib, refFile, packageId) {
                if (useSourceOfProjectReferenceRedirect) {
                    var source = getSourceOfProjectReferenceRedirect(fileName);
                    // If preserveSymlinks is true, module resolution wont jump the symlink
                    // but the resolved real path may be the .d.ts from project reference
                    // Note:: Currently we try the real path only if the
                    // file is from node_modules to avoid having to run real path on all file paths
                    if (!source &&
                        host.realpath &&
                        options.preserveSymlinks &&
                        ts.isDeclarationFileName(fileName) &&
                        ts.stringContains(fileName, ts.nodeModulesPathPart)) {
                        var realPath = host.realpath(fileName);
                        if (realPath !== fileName)
                            source = getSourceOfProjectReferenceRedirect(realPath);
                    }
                    if (source) {
                        var file_1 = ts.isString(source) ?
                            findSourceFile(source, toPath(source), isDefaultLib, ignoreNoDefaultLib, refFile, packageId) :
                            undefined;
                        if (file_1)
                            addFileToFilesByName(file_1, path, /*redirectedPath*/ undefined);
                        return file_1;
                    }
                }
                var originalFileName = fileName;
                if (filesByName.has(path)) {
                    var file_2 = filesByName.get(path);
                    addFileToRefFileMap(fileName, file_2 || undefined, refFile);
                    // try to check if we've already seen this file but with a different casing in path
                    // NOTE: this only makes sense for case-insensitive file systems, and only on files which are not redirected
                    if (file_2 && options.forceConsistentCasingInFileNames) {
                        var checkedName = file_2.fileName;
                        var isRedirect = toPath(checkedName) !== toPath(fileName);
                        if (isRedirect) {
                            fileName = getProjectReferenceRedirect(fileName) || fileName;
                        }
                        // Check if it differs only in drive letters its ok to ignore that error:
                        var checkedAbsolutePath = ts.getNormalizedAbsolutePathWithoutRoot(checkedName, currentDirectory);
                        var inputAbsolutePath = ts.getNormalizedAbsolutePathWithoutRoot(fileName, currentDirectory);
                        if (checkedAbsolutePath !== inputAbsolutePath) {
                            reportFileNamesDifferOnlyInCasingError(fileName, file_2, refFile);
                        }
                    }
                    // If the file was previously found via a node_modules search, but is now being processed as a root file,
                    // then everything it sucks in may also be marked incorrectly, and needs to be checked again.
                    if (file_2 && sourceFilesFoundSearchingNodeModules.get(file_2.path) && currentNodeModulesDepth === 0) {
                        sourceFilesFoundSearchingNodeModules.set(file_2.path, false);
                        if (!options.noResolve) {
                            processReferencedFiles(file_2, isDefaultLib);
                            processTypeReferenceDirectives(file_2);
                        }
                        if (!options.noLib) {
                            processLibReferenceDirectives(file_2);
                        }
                        modulesWithElidedImports.set(file_2.path, false);
                        processImportedModules(file_2);
                    }
                    // See if we need to reprocess the imports due to prior skipped imports
                    else if (file_2 && modulesWithElidedImports.get(file_2.path)) {
                        if (currentNodeModulesDepth < maxNodeModuleJsDepth) {
                            modulesWithElidedImports.set(file_2.path, false);
                            processImportedModules(file_2);
                        }
                    }
                    return file_2 || undefined;
                }
                var redirectedPath;
                if (refFile && !useSourceOfProjectReferenceRedirect) {
                    var redirectProject = getProjectReferenceRedirectProject(fileName);
                    if (redirectProject) {
                        if (redirectProject.commandLine.options.outFile || redirectProject.commandLine.options.out) {
                            // Shouldnt create many to 1 mapping file in --out scenario
                            return undefined;
                        }
                        var redirect = getProjectReferenceOutputName(redirectProject, fileName);
                        fileName = redirect;
                        // Once we start redirecting to a file, we can potentially come back to it
                        // via a back-reference from another file in the .d.ts folder. If that happens we'll
                        // end up trying to add it to the program *again* because we were tracking it via its
                        // original (un-redirected) name. So we have to map both the original path and the redirected path
                        // to the source file we're about to find/create
                        redirectedPath = toPath(redirect);
                    }
                }
                // We haven't looked for this file, do so now and cache result
                var file = host.getSourceFile(fileName, options.target, function (hostErrorMessage) { return fileProcessingDiagnostics.add(createRefFileDiagnostic(refFile, ts.Diagnostics.Cannot_read_file_0_Colon_1, fileName, hostErrorMessage)); }, shouldCreateNewSourceFile);
                if (packageId) {
                    var packageIdKey = ts.packageIdToString(packageId);
                    var fileFromPackageId = packageIdToSourceFile.get(packageIdKey);
                    if (fileFromPackageId) {
                        // Some other SourceFile already exists with this package name and version.
                        // Instead of creating a duplicate, just redirect to the existing one.
                        var dupFile = createRedirectSourceFile(fileFromPackageId, file, fileName, path, toPath(fileName), originalFileName); // TODO: GH#18217
                        redirectTargetsMap.add(fileFromPackageId.path, fileName);
                        addFileToFilesByName(dupFile, path, redirectedPath);
                        sourceFileToPackageName.set(path, packageId.name);
                        processingOtherFiles.push(dupFile);
                        return dupFile;
                    }
                    else if (file) {
                        // This is the first source file to have this packageId.
                        packageIdToSourceFile.set(packageIdKey, file);
                        sourceFileToPackageName.set(path, packageId.name);
                    }
                }
                addFileToFilesByName(file, path, redirectedPath);
                if (file) {
                    sourceFilesFoundSearchingNodeModules.set(path, currentNodeModulesDepth > 0);
                    file.fileName = fileName; // Ensure that source file has same name as what we were looking for
                    file.path = path;
                    file.resolvedPath = toPath(fileName);
                    file.originalFileName = originalFileName;
                    addFileToRefFileMap(fileName, file, refFile);
                    if (host.useCaseSensitiveFileNames()) {
                        var pathLowerCase = ts.toFileNameLowerCase(path);
                        // for case-sensitive file systems check if we've already seen some file with similar filename ignoring case
                        var existingFile = filesByNameIgnoreCase.get(pathLowerCase);
                        if (existingFile) {
                            reportFileNamesDifferOnlyInCasingError(fileName, existingFile, refFile);
                        }
                        else {
                            filesByNameIgnoreCase.set(pathLowerCase, file);
                        }
                    }
                    skipDefaultLib = skipDefaultLib || (file.hasNoDefaultLib && !ignoreNoDefaultLib);
                    if (!options.noResolve) {
                        processReferencedFiles(file, isDefaultLib);
                        processTypeReferenceDirectives(file);
                    }
                    if (!options.noLib) {
                        processLibReferenceDirectives(file);
                    }
                    // always process imported modules to record module name resolutions
                    processImportedModules(file);
                    if (isDefaultLib) {
                        processingDefaultLibFiles.push(file);
                    }
                    else {
                        processingOtherFiles.push(file);
                    }
                }
                return file;
            }
            function addFileToRefFileMap(referencedFileName, file, refFile) {
                if (refFile && file) {
                    (refFileMap || (refFileMap = ts.createMultiMap())).add(file.path, {
                        referencedFileName: referencedFileName,
                        kind: refFile.kind,
                        index: refFile.index,
                        file: refFile.file.path
                    });
                }
            }
            function addFileToFilesByName(file, path, redirectedPath) {
                if (redirectedPath) {
                    filesByName.set(redirectedPath, file);
                    filesByName.set(path, file || false);
                }
                else {
                    filesByName.set(path, file);
                }
            }
            function getProjectReferenceRedirect(fileName) {
                var referencedProject = getProjectReferenceRedirectProject(fileName);
                return referencedProject && getProjectReferenceOutputName(referencedProject, fileName);
            }
            function getProjectReferenceRedirectProject(fileName) {
                // Ignore dts or any json files
                if (!resolvedProjectReferences || !resolvedProjectReferences.length || ts.fileExtensionIs(fileName, ".d.ts" /* Dts */) || ts.fileExtensionIs(fileName, ".json" /* Json */)) {
                    return undefined;
                }
                // If this file is produced by a referenced project, we need to rewrite it to
                // look in the output folder of the referenced project rather than the input
                return getResolvedProjectReferenceToRedirect(fileName);
            }
            function getProjectReferenceOutputName(referencedProject, fileName) {
                var out = referencedProject.commandLine.options.outFile || referencedProject.commandLine.options.out;
                return out ?
                    ts.changeExtension(out, ".d.ts" /* Dts */) :
                    ts.getOutputDeclarationFileName(fileName, referencedProject.commandLine, !host.useCaseSensitiveFileNames());
            }
            /**
             * Get the referenced project if the file is input file from that reference project
             */
            function getResolvedProjectReferenceToRedirect(fileName) {
                if (mapFromFileToProjectReferenceRedirects === undefined) {
                    mapFromFileToProjectReferenceRedirects = ts.createMap();
                    forEachResolvedProjectReference(function (referencedProject, referenceProjectPath) {
                        // not input file from the referenced project, ignore
                        if (referencedProject &&
                            toPath(options.configFilePath) !== referenceProjectPath) {
                            referencedProject.commandLine.fileNames.forEach(function (f) {
                                return mapFromFileToProjectReferenceRedirects.set(toPath(f), referenceProjectPath);
                            });
                        }
                    });
                }
                var referencedProjectPath = mapFromFileToProjectReferenceRedirects.get(toPath(fileName));
                return referencedProjectPath && getResolvedProjectReferenceByPath(referencedProjectPath);
            }
            function forEachResolvedProjectReference(cb) {
                return forEachProjectReference(projectReferences, resolvedProjectReferences, function (resolvedRef, index, parent) {
                    var ref = (parent ? parent.commandLine.projectReferences : projectReferences)[index];
                    var resolvedRefPath = toPath(resolveProjectReferencePath(ref));
                    return cb(resolvedRef, resolvedRefPath);
                });
            }
            function getSourceOfProjectReferenceRedirect(file) {
                if (!ts.isDeclarationFileName(file))
                    return undefined;
                if (mapFromToProjectReferenceRedirectSource === undefined) {
                    mapFromToProjectReferenceRedirectSource = ts.createMap();
                    forEachResolvedProjectReference(function (resolvedRef) {
                        if (resolvedRef) {
                            var out = resolvedRef.commandLine.options.outFile || resolvedRef.commandLine.options.out;
                            if (out) {
                                // Dont know which source file it means so return true?
                                var outputDts = ts.changeExtension(out, ".d.ts" /* Dts */);
                                mapFromToProjectReferenceRedirectSource.set(toPath(outputDts), true);
                            }
                            else {
                                ts.forEach(resolvedRef.commandLine.fileNames, function (fileName) {
                                    if (!ts.fileExtensionIs(fileName, ".d.ts" /* Dts */) && !ts.fileExtensionIs(fileName, ".json" /* Json */)) {
                                        var outputDts = ts.getOutputDeclarationFileName(fileName, resolvedRef.commandLine, host.useCaseSensitiveFileNames());
                                        mapFromToProjectReferenceRedirectSource.set(toPath(outputDts), fileName);
                                    }
                                });
                            }
                        }
                    });
                }
                return mapFromToProjectReferenceRedirectSource.get(toPath(file));
            }
            function isSourceOfProjectReferenceRedirect(fileName) {
                return useSourceOfProjectReferenceRedirect && !!getResolvedProjectReferenceToRedirect(fileName);
            }
            function forEachProjectReference(projectReferences, resolvedProjectReferences, cbResolvedRef, cbRef) {
                var seenResolvedRefs;
                return worker(projectReferences, resolvedProjectReferences, /*parent*/ undefined, cbResolvedRef, cbRef);
                function worker(projectReferences, resolvedProjectReferences, parent, cbResolvedRef, cbRef) {
                    // Visit project references first
                    if (cbRef) {
                        var result = cbRef(projectReferences, parent);
                        if (result) {
                            return result;
                        }
                    }
                    return ts.forEach(resolvedProjectReferences, function (resolvedRef, index) {
                        if (ts.contains(seenResolvedRefs, resolvedRef)) {
                            // ignore recursives
                            return undefined;
                        }
                        var result = cbResolvedRef(resolvedRef, index, parent);
                        if (result) {
                            return result;
                        }
                        if (!resolvedRef)
                            return undefined;
                        (seenResolvedRefs || (seenResolvedRefs = [])).push(resolvedRef);
                        return worker(resolvedRef.commandLine.projectReferences, resolvedRef.references, resolvedRef, cbResolvedRef, cbRef);
                    });
                }
            }
            function getResolvedProjectReferenceByPath(projectReferencePath) {
                if (!projectReferenceRedirects) {
                    return undefined;
                }
                return projectReferenceRedirects.get(projectReferencePath) || undefined;
            }
            function processReferencedFiles(file, isDefaultLib) {
                ts.forEach(file.referencedFiles, function (ref, index) {
                    var referencedFileName = resolveTripleslashReference(ref.fileName, file.originalFileName);
                    processSourceFile(referencedFileName, isDefaultLib, 
                    /*ignoreNoDefaultLib*/ false, 
                    /*packageId*/ undefined, {
                        kind: ts.RefFileKind.ReferenceFile,
                        index: index,
                        file: file,
                        pos: ref.pos,
                        end: ref.end
                    });
                });
            }
            function processTypeReferenceDirectives(file) {
                // We lower-case all type references because npm automatically lowercases all packages. See GH#9824.
                var typeDirectives = ts.map(file.typeReferenceDirectives, function (ref) { return ts.toFileNameLowerCase(ref.fileName); });
                if (!typeDirectives) {
                    return;
                }
                var resolutions = resolveTypeReferenceDirectiveNamesWorker(typeDirectives, file.originalFileName, getResolvedProjectReferenceToRedirect(file.originalFileName));
                for (var i = 0; i < typeDirectives.length; i++) {
                    var ref = file.typeReferenceDirectives[i];
                    var resolvedTypeReferenceDirective = resolutions[i];
                    // store resolved type directive on the file
                    var fileName = ts.toFileNameLowerCase(ref.fileName);
                    ts.setResolvedTypeReferenceDirective(file, fileName, resolvedTypeReferenceDirective);
                    processTypeReferenceDirective(fileName, resolvedTypeReferenceDirective, {
                        kind: ts.RefFileKind.TypeReferenceDirective,
                        index: i,
                        file: file,
                        pos: ref.pos,
                        end: ref.end
                    });
                }
            }
            function processTypeReferenceDirective(typeReferenceDirective, resolvedTypeReferenceDirective, refFile) {
                // If we already found this library as a primary reference - nothing to do
                var previousResolution = resolvedTypeReferenceDirectives.get(typeReferenceDirective);
                if (previousResolution && previousResolution.primary) {
                    return;
                }
                var saveResolution = true;
                if (resolvedTypeReferenceDirective) {
                    if (resolvedTypeReferenceDirective.isExternalLibraryImport)
                        currentNodeModulesDepth++;
                    if (resolvedTypeReferenceDirective.primary) {
                        // resolved from the primary path
                        processSourceFile(resolvedTypeReferenceDirective.resolvedFileName, /*isDefaultLib*/ false, /*ignoreNoDefaultLib*/ false, resolvedTypeReferenceDirective.packageId, refFile); // TODO: GH#18217
                    }
                    else {
                        // If we already resolved to this file, it must have been a secondary reference. Check file contents
                        // for sameness and possibly issue an error
                        if (previousResolution) {
                            // Don't bother reading the file again if it's the same file.
                            if (resolvedTypeReferenceDirective.resolvedFileName !== previousResolution.resolvedFileName) {
                                var otherFileText = host.readFile(resolvedTypeReferenceDirective.resolvedFileName);
                                var existingFile_1 = getSourceFile(previousResolution.resolvedFileName);
                                if (otherFileText !== existingFile_1.text) {
                                    // Try looking up ref for original file
                                    var refs = !refFile ? refFileMap && refFileMap.get(existingFile_1.path) : undefined;
                                    var refToReportErrorOn = refs && ts.find(refs, function (ref) { return ref.referencedFileName === existingFile_1.fileName; });
                                    fileProcessingDiagnostics.add(refToReportErrorOn ?
                                        createFileDiagnosticAtReference(refToReportErrorOn, ts.Diagnostics.Conflicting_definitions_for_0_found_at_1_and_2_Consider_installing_a_specific_version_of_this_library_to_resolve_the_conflict, typeReferenceDirective, resolvedTypeReferenceDirective.resolvedFileName, previousResolution.resolvedFileName) :
                                        createRefFileDiagnostic(refFile, ts.Diagnostics.Conflicting_definitions_for_0_found_at_1_and_2_Consider_installing_a_specific_version_of_this_library_to_resolve_the_conflict, typeReferenceDirective, resolvedTypeReferenceDirective.resolvedFileName, previousResolution.resolvedFileName));
                                }
                            }
                            // don't overwrite previous resolution result
                            saveResolution = false;
                        }
                        else {
                            // First resolution of this library
                            processSourceFile(resolvedTypeReferenceDirective.resolvedFileName, /*isDefaultLib*/ false, /*ignoreNoDefaultLib*/ false, resolvedTypeReferenceDirective.packageId, refFile);
                        }
                    }
                    if (resolvedTypeReferenceDirective.isExternalLibraryImport)
                        currentNodeModulesDepth--;
                }
                else {
                    fileProcessingDiagnostics.add(createRefFileDiagnostic(refFile, ts.Diagnostics.Cannot_find_type_definition_file_for_0, typeReferenceDirective));
                }
                if (saveResolution) {
                    resolvedTypeReferenceDirectives.set(typeReferenceDirective, resolvedTypeReferenceDirective);
                }
            }
            function processLibReferenceDirectives(file) {
                ts.forEach(file.libReferenceDirectives, function (libReference) {
                    var libName = ts.toFileNameLowerCase(libReference.fileName);
                    var libFileName = ts.libMap.get(libName);
                    if (libFileName) {
                        // we ignore any 'no-default-lib' reference set on this file.
                        processRootFile(ts.combinePaths(defaultLibraryPath, libFileName), /*isDefaultLib*/ true, /*ignoreNoDefaultLib*/ true);
                    }
                    else {
                        var unqualifiedLibName = ts.removeSuffix(ts.removePrefix(libName, "lib."), ".d.ts");
                        var suggestion = ts.getSpellingSuggestion(unqualifiedLibName, ts.libs, ts.identity);
                        var message = suggestion ? ts.Diagnostics.Cannot_find_lib_definition_for_0_Did_you_mean_1 : ts.Diagnostics.Cannot_find_lib_definition_for_0;
                        fileProcessingDiagnostics.add(ts.createFileDiagnostic(file, libReference.pos, libReference.end - libReference.pos, message, libName, suggestion));
                    }
                });
            }
            function createRefFileDiagnostic(refFile, message) {
                var args = [];
                for (var _i = 2; _i < arguments.length; _i++) {
                    args[_i - 2] = arguments[_i];
                }
                if (!refFile) {
                    return ts.createCompilerDiagnostic.apply(void 0, __spreadArrays([message], args));
                }
                else {
                    return ts.createFileDiagnostic.apply(void 0, __spreadArrays([refFile.file, refFile.pos, refFile.end - refFile.pos, message], args));
                }
            }
            function getCanonicalFileName(fileName) {
                return host.getCanonicalFileName(fileName);
            }
            function processImportedModules(file) {
                collectExternalModuleReferences(file);
                if (file.imports.length || file.moduleAugmentations.length) {
                    // Because global augmentation doesn't have string literal name, we can check for global augmentation as such.
                    var moduleNames = getModuleNames(file);
                    var resolutions = resolveModuleNamesReusingOldState(moduleNames, ts.getNormalizedAbsolutePath(file.originalFileName, currentDirectory), file);
                    ts.Debug.assert(resolutions.length === moduleNames.length);
                    for (var i = 0; i < moduleNames.length; i++) {
                        var resolution = resolutions[i];
                        ts.setResolvedModule(file, moduleNames[i], resolution);
                        if (!resolution) {
                            continue;
                        }
                        var isFromNodeModulesSearch = resolution.isExternalLibraryImport;
                        var isJsFile = !ts.resolutionExtensionIsTSOrJson(resolution.extension);
                        var isJsFileFromNodeModules = isFromNodeModulesSearch && isJsFile;
                        var resolvedFileName = resolution.resolvedFileName;
                        if (isFromNodeModulesSearch) {
                            currentNodeModulesDepth++;
                        }
                        // add file to program only if:
                        // - resolution was successful
                        // - noResolve is falsy
                        // - module name comes from the list of imports
                        // - it's not a top level JavaScript module that exceeded the search max
                        var elideImport = isJsFileFromNodeModules && currentNodeModulesDepth > maxNodeModuleJsDepth;
                        // Don't add the file if it has a bad extension (e.g. 'tsx' if we don't have '--allowJs')
                        // This may still end up being an untyped module -- the file won't be included but imports will be allowed.
                        var shouldAddFile = resolvedFileName
                            && !getResolutionDiagnostic(options, resolution)
                            && !options.noResolve
                            && i < file.imports.length
                            && !elideImport
                            && !(isJsFile && !options.allowJs)
                            && (ts.isInJSFile(file.imports[i]) || !(file.imports[i].flags & 4194304 /* JSDoc */));
                        if (elideImport) {
                            modulesWithElidedImports.set(file.path, true);
                        }
                        else if (shouldAddFile) {
                            var path = toPath(resolvedFileName);
                            var pos = ts.skipTrivia(file.text, file.imports[i].pos);
                            findSourceFile(resolvedFileName, path, 
                            /*isDefaultLib*/ false, 
                            /*ignoreNoDefaultLib*/ false, {
                                kind: ts.RefFileKind.Import,
                                index: i,
                                file: file,
                                pos: pos,
                                end: file.imports[i].end
                            }, resolution.packageId);
                        }
                        if (isFromNodeModulesSearch) {
                            currentNodeModulesDepth--;
                        }
                    }
                }
                else {
                    // no imports - drop cached module resolutions
                    file.resolvedModules = undefined;
                }
            }
            function computeCommonSourceDirectory(sourceFiles) {
                var fileNames = ts.mapDefined(sourceFiles, function (file) { return file.isDeclarationFile ? undefined : file.fileName; });
                return computeCommonSourceDirectoryOfFilenames(fileNames, currentDirectory, getCanonicalFileName);
            }
            function checkSourceFilesBelongToPath(sourceFiles, rootDirectory) {
                var allFilesBelongToPath = true;
                var absoluteRootDirectoryPath = host.getCanonicalFileName(ts.getNormalizedAbsolutePath(rootDirectory, currentDirectory));
                var rootPaths;
                for (var _i = 0, sourceFiles_2 = sourceFiles; _i < sourceFiles_2.length; _i++) {
                    var sourceFile = sourceFiles_2[_i];
                    if (!sourceFile.isDeclarationFile) {
                        var absoluteSourceFilePath = host.getCanonicalFileName(ts.getNormalizedAbsolutePath(sourceFile.fileName, currentDirectory));
                        if (absoluteSourceFilePath.indexOf(absoluteRootDirectoryPath) !== 0) {
                            if (!rootPaths)
                                rootPaths = ts.arrayToSet(rootNames, toPath);
                            addProgramDiagnosticAtRefPath(sourceFile, rootPaths, ts.Diagnostics.File_0_is_not_under_rootDir_1_rootDir_is_expected_to_contain_all_source_files, sourceFile.fileName, rootDirectory);
                            allFilesBelongToPath = false;
                        }
                    }
                }
                return allFilesBelongToPath;
            }
            function parseProjectReferenceConfigFile(ref) {
                if (!projectReferenceRedirects) {
                    projectReferenceRedirects = ts.createMap();
                }
                // The actual filename (i.e. add "/tsconfig.json" if necessary)
                var refPath = resolveProjectReferencePath(ref);
                var sourceFilePath = toPath(refPath);
                var fromCache = projectReferenceRedirects.get(sourceFilePath);
                if (fromCache !== undefined) {
                    return fromCache || undefined;
                }
                var commandLine;
                var sourceFile;
                if (host.getParsedCommandLine) {
                    commandLine = host.getParsedCommandLine(refPath);
                    if (!commandLine) {
                        addFileToFilesByName(/*sourceFile*/ undefined, sourceFilePath, /*redirectedPath*/ undefined);
                        projectReferenceRedirects.set(sourceFilePath, false);
                        return undefined;
                    }
                    sourceFile = ts.Debug.assertDefined(commandLine.options.configFile);
                    ts.Debug.assert(!sourceFile.path || sourceFile.path === sourceFilePath);
                    addFileToFilesByName(sourceFile, sourceFilePath, /*redirectedPath*/ undefined);
                }
                else {
                    // An absolute path pointing to the containing directory of the config file
                    var basePath = ts.getNormalizedAbsolutePath(ts.getDirectoryPath(refPath), host.getCurrentDirectory());
                    sourceFile = host.getSourceFile(refPath, 100 /* JSON */);
                    addFileToFilesByName(sourceFile, sourceFilePath, /*redirectedPath*/ undefined);
                    if (sourceFile === undefined) {
                        projectReferenceRedirects.set(sourceFilePath, false);
                        return undefined;
                    }
                    commandLine = ts.parseJsonSourceFileConfigFileContent(sourceFile, configParsingHost, basePath, /*existingOptions*/ undefined, refPath);
                }
                sourceFile.fileName = refPath;
                sourceFile.path = sourceFilePath;
                sourceFile.resolvedPath = sourceFilePath;
                sourceFile.originalFileName = refPath;
                var resolvedRef = { commandLine: commandLine, sourceFile: sourceFile };
                projectReferenceRedirects.set(sourceFilePath, resolvedRef);
                if (commandLine.projectReferences) {
                    resolvedRef.references = commandLine.projectReferences.map(parseProjectReferenceConfigFile);
                }
                return resolvedRef;
            }
            function verifyCompilerOptions() {
                if (options.strictPropertyInitialization && !ts.getStrictOptionValue(options, "strictNullChecks")) {
                    createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1, "strictPropertyInitialization", "strictNullChecks");
                }
                if (options.isolatedModules) {
                    if (options.out) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_with_option_1, "out", "isolatedModules");
                    }
                    if (options.outFile) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_with_option_1, "outFile", "isolatedModules");
                    }
                }
                if (options.inlineSourceMap) {
                    if (options.sourceMap) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_with_option_1, "sourceMap", "inlineSourceMap");
                    }
                    if (options.mapRoot) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_with_option_1, "mapRoot", "inlineSourceMap");
                    }
                }
                if (options.paths && options.baseUrl === undefined) {
                    createDiagnosticForOptionName(ts.Diagnostics.Option_paths_cannot_be_used_without_specifying_baseUrl_option, "paths");
                }
                if (options.composite) {
                    if (options.declaration === false) {
                        createDiagnosticForOptionName(ts.Diagnostics.Composite_projects_may_not_disable_declaration_emit, "declaration");
                    }
                    if (options.incremental === false) {
                        createDiagnosticForOptionName(ts.Diagnostics.Composite_projects_may_not_disable_incremental_compilation, "declaration");
                    }
                }
                if (options.tsBuildInfoFile) {
                    if (!ts.isIncrementalCompilation(options)) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2, "tsBuildInfoFile", "incremental", "composite");
                    }
                }
                else if (options.incremental && !options.outFile && !options.out && !options.configFilePath) {
                    programDiagnostics.add(ts.createCompilerDiagnostic(ts.Diagnostics.Option_incremental_can_only_be_specified_using_tsconfig_emitting_to_single_file_or_when_option_tsBuildInfoFile_is_specified));
                }
                if (!options.listFilesOnly && options.noEmit && ts.isIncrementalCompilation(options)) {
                    createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_with_option_1, "noEmit", options.incremental ? "incremental" : "composite");
                }
                verifyProjectReferences();
                // List of collected files is complete; validate exhautiveness if this is a project with a file list
                if (options.composite) {
                    var rootPaths = ts.arrayToSet(rootNames, toPath);
                    for (var _i = 0, files_3 = files; _i < files_3.length; _i++) {
                        var file = files_3[_i];
                        // Ignore file that is not emitted
                        if (ts.sourceFileMayBeEmitted(file, program) && !rootPaths.has(file.path)) {
                            addProgramDiagnosticAtRefPath(file, rootPaths, ts.Diagnostics.File_0_is_not_listed_within_the_file_list_of_project_1_Projects_must_list_all_files_or_use_an_include_pattern, file.fileName, options.configFilePath || "");
                        }
                    }
                }
                if (options.paths) {
                    for (var key in options.paths) {
                        if (!ts.hasProperty(options.paths, key)) {
                            continue;
                        }
                        if (!ts.hasZeroOrOneAsteriskCharacter(key)) {
                            createDiagnosticForOptionPaths(/*onKey*/ true, key, ts.Diagnostics.Pattern_0_can_have_at_most_one_Asterisk_character, key);
                        }
                        if (ts.isArray(options.paths[key])) {
                            var len = options.paths[key].length;
                            if (len === 0) {
                                createDiagnosticForOptionPaths(/*onKey*/ false, key, ts.Diagnostics.Substitutions_for_pattern_0_shouldn_t_be_an_empty_array, key);
                            }
                            for (var i = 0; i < len; i++) {
                                var subst = options.paths[key][i];
                                var typeOfSubst = typeof subst;
                                if (typeOfSubst === "string") {
                                    if (!ts.hasZeroOrOneAsteriskCharacter(subst)) {
                                        createDiagnosticForOptionPathKeyValue(key, i, ts.Diagnostics.Substitution_0_in_pattern_1_can_have_at_most_one_Asterisk_character, subst, key);
                                    }
                                }
                                else {
                                    createDiagnosticForOptionPathKeyValue(key, i, ts.Diagnostics.Substitution_0_for_pattern_1_has_incorrect_type_expected_string_got_2, subst, key, typeOfSubst);
                                }
                            }
                        }
                        else {
                            createDiagnosticForOptionPaths(/*onKey*/ false, key, ts.Diagnostics.Substitutions_for_pattern_0_should_be_an_array, key);
                        }
                    }
                }
                if (!options.sourceMap && !options.inlineSourceMap) {
                    if (options.inlineSources) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided, "inlineSources");
                    }
                    if (options.sourceRoot) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_can_only_be_used_when_either_option_inlineSourceMap_or_option_sourceMap_is_provided, "sourceRoot");
                    }
                }
                if (options.out && options.outFile) {
                    createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_with_option_1, "out", "outFile");
                }
                if (options.mapRoot && !(options.sourceMap || options.declarationMap)) {
                    // Error to specify --mapRoot without --sourcemap
                    createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2, "mapRoot", "sourceMap", "declarationMap");
                }
                if (options.declarationDir) {
                    if (!ts.getEmitDeclarations(options)) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2, "declarationDir", "declaration", "composite");
                    }
                    if (options.out || options.outFile) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_with_option_1, "declarationDir", options.out ? "out" : "outFile");
                    }
                }
                if (options.declarationMap && !ts.getEmitDeclarations(options)) {
                    createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2, "declarationMap", "declaration", "composite");
                }
                if (options.lib && options.noLib) {
                    createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_with_option_1, "lib", "noLib");
                }
                if (options.noImplicitUseStrict && ts.getStrictOptionValue(options, "alwaysStrict")) {
                    createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_with_option_1, "noImplicitUseStrict", "alwaysStrict");
                }
                var languageVersion = options.target || 0 /* ES3 */;
                var outFile = options.outFile || options.out;
                var firstNonAmbientExternalModuleSourceFile = ts.find(files, function (f) { return ts.isExternalModule(f) && !f.isDeclarationFile; });
                if (options.isolatedModules) {
                    if (options.module === ts.ModuleKind.None && languageVersion < 2 /* ES2015 */) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_isolatedModules_can_only_be_used_when_either_option_module_is_provided_or_option_target_is_ES2015_or_higher, "isolatedModules", "target");
                    }
                    var firstNonExternalModuleSourceFile = ts.find(files, function (f) { return !ts.isExternalModule(f) && !ts.isSourceFileJS(f) && !f.isDeclarationFile && f.scriptKind !== 6 /* JSON */; });
                    if (firstNonExternalModuleSourceFile) {
                        var span = ts.getErrorSpanForNode(firstNonExternalModuleSourceFile, firstNonExternalModuleSourceFile);
                        programDiagnostics.add(ts.createFileDiagnostic(firstNonExternalModuleSourceFile, span.start, span.length, ts.Diagnostics.All_files_must_be_modules_when_the_isolatedModules_flag_is_provided));
                    }
                }
                else if (firstNonAmbientExternalModuleSourceFile && languageVersion < 2 /* ES2015 */ && options.module === ts.ModuleKind.None) {
                    // We cannot use createDiagnosticFromNode because nodes do not have parents yet
                    var span = ts.getErrorSpanForNode(firstNonAmbientExternalModuleSourceFile, firstNonAmbientExternalModuleSourceFile.externalModuleIndicator);
                    programDiagnostics.add(ts.createFileDiagnostic(firstNonAmbientExternalModuleSourceFile, span.start, span.length, ts.Diagnostics.Cannot_use_imports_exports_or_module_augmentations_when_module_is_none));
                }
                // Cannot specify module gen that isn't amd or system with --out
                if (outFile && !options.emitDeclarationOnly) {
                    if (options.module && !(options.module === ts.ModuleKind.AMD || options.module === ts.ModuleKind.System)) {
                        createDiagnosticForOptionName(ts.Diagnostics.Only_amd_and_system_modules_are_supported_alongside_0, options.out ? "out" : "outFile", "module");
                    }
                    else if (options.module === undefined && firstNonAmbientExternalModuleSourceFile) {
                        var span = ts.getErrorSpanForNode(firstNonAmbientExternalModuleSourceFile, firstNonAmbientExternalModuleSourceFile.externalModuleIndicator);
                        programDiagnostics.add(ts.createFileDiagnostic(firstNonAmbientExternalModuleSourceFile, span.start, span.length, ts.Diagnostics.Cannot_compile_modules_using_option_0_unless_the_module_flag_is_amd_or_system, options.out ? "out" : "outFile"));
                    }
                }
                if (options.resolveJsonModule) {
                    if (ts.getEmitModuleResolutionKind(options) !== ts.ModuleResolutionKind.NodeJs) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_resolveJsonModule_cannot_be_specified_without_node_module_resolution_strategy, "resolveJsonModule");
                    }
                    // Any emit other than common js, amd, es2015 or esnext is error
                    else if (!ts.hasJsonModuleEmitEnabled(options)) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_resolveJsonModule_can_only_be_specified_when_module_code_generation_is_commonjs_amd_es2015_or_esNext, "resolveJsonModule", "module");
                    }
                }
                // there has to be common source directory if user specified --outdir || --sourceRoot
                // if user specified --mapRoot, there needs to be common source directory if there would be multiple files being emitted
                if (options.outDir || // there is --outDir specified
                    options.sourceRoot || // there is --sourceRoot specified
                    options.mapRoot) { // there is --mapRoot specified
                    // Precalculate and cache the common source directory
                    var dir = getCommonSourceDirectory();
                    // If we failed to find a good common directory, but outDir is specified and at least one of our files is on a windows drive/URL/other resource, add a failure
                    if (options.outDir && dir === "" && files.some(function (file) { return ts.getRootLength(file.fileName) > 1; })) {
                        createDiagnosticForOptionName(ts.Diagnostics.Cannot_find_the_common_subdirectory_path_for_the_input_files, "outDir");
                    }
                }
                if (options.useDefineForClassFields && languageVersion === 0 /* ES3 */) {
                    createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_when_option_target_is_ES3, "useDefineForClassFields");
                }
                if (options.checkJs && !options.allowJs) {
                    programDiagnostics.add(ts.createCompilerDiagnostic(ts.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1, "checkJs", "allowJs"));
                }
                if (options.emitDeclarationOnly) {
                    if (!ts.getEmitDeclarations(options)) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1_or_option_2, "emitDeclarationOnly", "declaration", "composite");
                    }
                    if (options.noEmit) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_with_option_1, "emitDeclarationOnly", "noEmit");
                    }
                }
                if (options.emitDecoratorMetadata &&
                    !options.experimentalDecorators) {
                    createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_without_specifying_option_1, "emitDecoratorMetadata", "experimentalDecorators");
                }
                if (options.jsxFactory) {
                    if (options.reactNamespace) {
                        createDiagnosticForOptionName(ts.Diagnostics.Option_0_cannot_be_specified_with_option_1, "reactNamespace", "jsxFactory");
                    }
                    if (!ts.parseIsolatedEntityName(options.jsxFactory, languageVersion)) {
                        createOptionValueDiagnostic("jsxFactory", ts.Diagnostics.Invalid_value_for_jsxFactory_0_is_not_a_valid_identifier_or_qualified_name, options.jsxFactory);
                    }
                }
                else if (options.reactNamespace && !ts.isIdentifierText(options.reactNamespace, languageVersion)) {
                    createOptionValueDiagnostic("reactNamespace", ts.Diagnostics.Invalid_value_for_reactNamespace_0_is_not_a_valid_identifier, options.reactNamespace);
                }
                // If the emit is enabled make sure that every output file is unique and not overwriting any of the input files
                if (!options.noEmit && !options.suppressOutputPathCheck) {
                    var emitHost = getEmitHost();
                    var emitFilesSeen_1 = ts.createMap();
                    ts.forEachEmittedFile(emitHost, function (emitFileNames) {
                        if (!options.emitDeclarationOnly) {
                            verifyEmitFilePath(emitFileNames.jsFilePath, emitFilesSeen_1);
                        }
                        verifyEmitFilePath(emitFileNames.declarationFilePath, emitFilesSeen_1);
                    });
                }
                // Verify that all the emit files are unique and don't overwrite input files
                function verifyEmitFilePath(emitFileName, emitFilesSeen) {
                    if (emitFileName) {
                        var emitFilePath = toPath(emitFileName);
                        // Report error if the output overwrites input file
                        if (filesByName.has(emitFilePath)) {
                            var chain = void 0;
                            if (!options.configFilePath) {
                                // The program is from either an inferred project or an external project
                                chain = ts.chainDiagnosticMessages(/*details*/ undefined, ts.Diagnostics.Adding_a_tsconfig_json_file_will_help_organize_projects_that_contain_both_TypeScript_and_JavaScript_files_Learn_more_at_https_Colon_Slash_Slashaka_ms_Slashtsconfig);
                            }
                            chain = ts.chainDiagnosticMessages(chain, ts.Diagnostics.Cannot_write_file_0_because_it_would_overwrite_input_file, emitFileName);
                            blockEmittingOfFile(emitFileName, ts.createCompilerDiagnosticFromMessageChain(chain));
                        }
                        var emitFileKey = !host.useCaseSensitiveFileNames() ? ts.toFileNameLowerCase(emitFilePath) : emitFilePath;
                        // Report error if multiple files write into same file
                        if (emitFilesSeen.has(emitFileKey)) {
                            // Already seen the same emit file - report error
                            blockEmittingOfFile(emitFileName, ts.createCompilerDiagnostic(ts.Diagnostics.Cannot_write_file_0_because_it_would_be_overwritten_by_multiple_input_files, emitFileName));
                        }
                        else {
                            emitFilesSeen.set(emitFileKey, true);
                        }
                    }
                }
            }
            function createFileDiagnosticAtReference(refPathToReportErrorOn, message) {
                var _a, _b;
                var args = [];
                for (var _i = 2; _i < arguments.length; _i++) {
                    args[_i - 2] = arguments[_i];
                }
                var refFile = ts.Debug.assertDefined(getSourceFileByPath(refPathToReportErrorOn.file));
                var kind = refPathToReportErrorOn.kind, index = refPathToReportErrorOn.index;
                var pos, end;
                switch (kind) {
                    case ts.RefFileKind.Import:
                        pos = ts.skipTrivia(refFile.text, refFile.imports[index].pos);
                        end = refFile.imports[index].end;
                        break;
                    case ts.RefFileKind.ReferenceFile:
                        (_a = refFile.referencedFiles[index], pos = _a.pos, end = _a.end);
                        break;
                    case ts.RefFileKind.TypeReferenceDirective:
                        (_b = refFile.typeReferenceDirectives[index], pos = _b.pos, end = _b.end);
                        break;
                    default:
                        return ts.Debug.assertNever(kind);
                }
                return ts.createFileDiagnostic.apply(void 0, __spreadArrays([refFile, pos, end - pos, message], args));
            }
            function addProgramDiagnosticAtRefPath(file, rootPaths, message) {
                var args = [];
                for (var _i = 3; _i < arguments.length; _i++) {
                    args[_i - 3] = arguments[_i];
                }
                var refPaths = refFileMap && refFileMap.get(file.path);
                var refPathToReportErrorOn = ts.forEach(refPaths, function (refPath) { return rootPaths.has(refPath.file) ? refPath : undefined; }) ||
                    ts.elementAt(refPaths, 0);
                programDiagnostics.add(refPathToReportErrorOn ? createFileDiagnosticAtReference.apply(void 0, __spreadArrays([refPathToReportErrorOn, message], args)) : ts.createCompilerDiagnostic.apply(void 0, __spreadArrays([message], args)));
            }
            function verifyProjectReferences() {
                var buildInfoPath = !options.noEmit && !options.suppressOutputPathCheck ? ts.getTsBuildInfoEmitOutputFilePath(options) : undefined;
                forEachProjectReference(projectReferences, resolvedProjectReferences, function (resolvedRef, index, parent) {
                    var ref = (parent ? parent.commandLine.projectReferences : projectReferences)[index];
                    var parentFile = parent && parent.sourceFile;
                    if (!resolvedRef) {
                        createDiagnosticForReference(parentFile, index, ts.Diagnostics.File_0_not_found, ref.path);
                        return;
                    }
                    var options = resolvedRef.commandLine.options;
                    if (!options.composite) {
                        // ok to not have composite if the current program is container only
                        var inputs = parent ? parent.commandLine.fileNames : rootNames;
                        if (inputs.length) {
                            createDiagnosticForReference(parentFile, index, ts.Diagnostics.Referenced_project_0_must_have_setting_composite_Colon_true, ref.path);
                        }
                    }
                    if (ref.prepend) {
                        var out = options.outFile || options.out;
                        if (out) {
                            if (!host.fileExists(out)) {
                                createDiagnosticForReference(parentFile, index, ts.Diagnostics.Output_file_0_from_project_1_does_not_exist, out, ref.path);
                            }
                        }
                        else {
                            createDiagnosticForReference(parentFile, index, ts.Diagnostics.Cannot_prepend_project_0_because_it_does_not_have_outFile_set, ref.path);
                        }
                    }
                    if (!parent && buildInfoPath && buildInfoPath === ts.getTsBuildInfoEmitOutputFilePath(options)) {
                        createDiagnosticForReference(parentFile, index, ts.Diagnostics.Cannot_write_file_0_because_it_will_overwrite_tsbuildinfo_file_generated_by_referenced_project_1, buildInfoPath, ref.path);
                        hasEmitBlockingDiagnostics.set(toPath(buildInfoPath), true);
                    }
                });
            }
            function createDiagnosticForOptionPathKeyValue(key, valueIndex, message, arg0, arg1, arg2) {
                var needCompilerDiagnostic = true;
                var pathsSyntax = getOptionPathsSyntax();
                for (var _i = 0, pathsSyntax_1 = pathsSyntax; _i < pathsSyntax_1.length; _i++) {
                    var pathProp = pathsSyntax_1[_i];
                    if (ts.isObjectLiteralExpression(pathProp.initializer)) {
                        for (var _a = 0, _b = ts.getPropertyAssignment(pathProp.initializer, key); _a < _b.length; _a++) {
                            var keyProps = _b[_a];
                            var initializer = keyProps.initializer;
                            if (ts.isArrayLiteralExpression(initializer) && initializer.elements.length > valueIndex) {
                                programDiagnostics.add(ts.createDiagnosticForNodeInSourceFile(options.configFile, initializer.elements[valueIndex], message, arg0, arg1, arg2));
                                needCompilerDiagnostic = false;
                            }
                        }
                    }
                }
                if (needCompilerDiagnostic) {
                    programDiagnostics.add(ts.createCompilerDiagnostic(message, arg0, arg1, arg2));
                }
            }
            function createDiagnosticForOptionPaths(onKey, key, message, arg0) {
                var needCompilerDiagnostic = true;
                var pathsSyntax = getOptionPathsSyntax();
                for (var _i = 0, pathsSyntax_2 = pathsSyntax; _i < pathsSyntax_2.length; _i++) {
                    var pathProp = pathsSyntax_2[_i];
                    if (ts.isObjectLiteralExpression(pathProp.initializer) &&
                        createOptionDiagnosticInObjectLiteralSyntax(pathProp.initializer, onKey, key, /*key2*/ undefined, message, arg0)) {
                        needCompilerDiagnostic = false;
                    }
                }
                if (needCompilerDiagnostic) {
                    programDiagnostics.add(ts.createCompilerDiagnostic(message, arg0));
                }
            }
            function getOptionsSyntaxByName(name) {
                var compilerOptionsObjectLiteralSyntax = getCompilerOptionsObjectLiteralSyntax();
                if (compilerOptionsObjectLiteralSyntax) {
                    return ts.getPropertyAssignment(compilerOptionsObjectLiteralSyntax, name);
                }
                return undefined;
            }
            function getOptionPathsSyntax() {
                return getOptionsSyntaxByName("paths") || ts.emptyArray;
            }
            function createDiagnosticForOptionName(message, option1, option2, option3) {
                createDiagnosticForOption(/*onKey*/ true, option1, option2, message, option1, option2, option3);
            }
            function createOptionValueDiagnostic(option1, message, arg0) {
                createDiagnosticForOption(/*onKey*/ false, option1, /*option2*/ undefined, message, arg0);
            }
            function createDiagnosticForReference(sourceFile, index, message, arg0, arg1) {
                var referencesSyntax = ts.firstDefined(ts.getTsConfigPropArray(sourceFile || options.configFile, "references"), function (property) { return ts.isArrayLiteralExpression(property.initializer) ? property.initializer : undefined; });
                if (referencesSyntax && referencesSyntax.elements.length > index) {
                    programDiagnostics.add(ts.createDiagnosticForNodeInSourceFile(sourceFile || options.configFile, referencesSyntax.elements[index], message, arg0, arg1));
                }
                else {
                    programDiagnostics.add(ts.createCompilerDiagnostic(message, arg0, arg1));
                }
            }
            function createDiagnosticForOption(onKey, option1, option2, message, arg0, arg1, arg2) {
                var compilerOptionsObjectLiteralSyntax = getCompilerOptionsObjectLiteralSyntax();
                var needCompilerDiagnostic = !compilerOptionsObjectLiteralSyntax ||
                    !createOptionDiagnosticInObjectLiteralSyntax(compilerOptionsObjectLiteralSyntax, onKey, option1, option2, message, arg0, arg1, arg2);
                if (needCompilerDiagnostic) {
                    programDiagnostics.add(ts.createCompilerDiagnostic(message, arg0, arg1, arg2));
                }
            }
            function getCompilerOptionsObjectLiteralSyntax() {
                if (_compilerOptionsObjectLiteralSyntax === undefined) {
                    _compilerOptionsObjectLiteralSyntax = null; // eslint-disable-line no-null/no-null
                    var jsonObjectLiteral = ts.getTsConfigObjectLiteralExpression(options.configFile);
                    if (jsonObjectLiteral) {
                        for (var _i = 0, _a = ts.getPropertyAssignment(jsonObjectLiteral, "compilerOptions"); _i < _a.length; _i++) {
                            var prop = _a[_i];
                            if (ts.isObjectLiteralExpression(prop.initializer)) {
                                _compilerOptionsObjectLiteralSyntax = prop.initializer;
                                break;
                            }
                        }
                    }
                }
                return _compilerOptionsObjectLiteralSyntax;
            }
            function createOptionDiagnosticInObjectLiteralSyntax(objectLiteral, onKey, key1, key2, message, arg0, arg1, arg2) {
                var props = ts.getPropertyAssignment(objectLiteral, key1, key2);
                for (var _i = 0, props_3 = props; _i < props_3.length; _i++) {
                    var prop = props_3[_i];
                    programDiagnostics.add(ts.createDiagnosticForNodeInSourceFile(options.configFile, onKey ? prop.name : prop.initializer, message, arg0, arg1, arg2));
                }
                return !!props.length;
            }
            function blockEmittingOfFile(emitFileName, diag) {
                hasEmitBlockingDiagnostics.set(toPath(emitFileName), true);
                programDiagnostics.add(diag);
            }
            function isEmittedFile(file) {
                if (options.noEmit) {
                    return false;
                }
                // If this is source file, its not emitted file
                var filePath = toPath(file);
                if (getSourceFileByPath(filePath)) {
                    return false;
                }
                // If options have --outFile or --out just check that
                var out = options.outFile || options.out;
                if (out) {
                    return isSameFile(filePath, out) || isSameFile(filePath, ts.removeFileExtension(out) + ".d.ts" /* Dts */);
                }
                // If declarationDir is specified, return if its a file in that directory
                if (options.declarationDir && ts.containsPath(options.declarationDir, filePath, currentDirectory, !host.useCaseSensitiveFileNames())) {
                    return true;
                }
                // If --outDir, check if file is in that directory
                if (options.outDir) {
                    return ts.containsPath(options.outDir, filePath, currentDirectory, !host.useCaseSensitiveFileNames());
                }
                if (ts.fileExtensionIsOneOf(filePath, ts.supportedJSExtensions) || ts.fileExtensionIs(filePath, ".d.ts" /* Dts */)) {
                    // Otherwise just check if sourceFile with the name exists
                    var filePathWithoutExtension = ts.removeFileExtension(filePath);
                    return !!getSourceFileByPath((filePathWithoutExtension + ".ts" /* Ts */)) ||
                        !!getSourceFileByPath((filePathWithoutExtension + ".tsx" /* Tsx */));
                }
                return false;
            }
            function isSameFile(file1, file2) {
                return ts.comparePaths(file1, file2, currentDirectory, !host.useCaseSensitiveFileNames()) === 0 /* EqualTo */;
            }
            function getProbableSymlinks() {
                if (host.getSymlinks) {
                    return host.getSymlinks();
                }
                return symlinks || (symlinks = ts.discoverProbableSymlinks(files, getCanonicalFileName, host.getCurrentDirectory()));
            }
        }
        ts.createProgram = createProgram;
        /*@internal*/
        function handleNoEmitOptions(program, sourceFile, cancellationToken) {
            var options = program.getCompilerOptions();
            if (options.noEmit) {
                return { diagnostics: ts.emptyArray, sourceMaps: undefined, emittedFiles: undefined, emitSkipped: true };
            }
            // If the noEmitOnError flag is set, then check if we have any errors so far.  If so,
            // immediately bail out.  Note that we pass 'undefined' for 'sourceFile' so that we
            // get any preEmit diagnostics, not just the ones
            if (!options.noEmitOnError)
                return undefined;
            var diagnostics = __spreadArrays(program.getOptionsDiagnostics(cancellationToken), program.getSyntacticDiagnostics(sourceFile, cancellationToken), program.getGlobalDiagnostics(cancellationToken), program.getSemanticDiagnostics(sourceFile, cancellationToken));
            if (diagnostics.length === 0 && ts.getEmitDeclarations(program.getCompilerOptions())) {
                diagnostics = program.getDeclarationDiagnostics(/*sourceFile*/ undefined, cancellationToken);
            }
            return diagnostics.length > 0 ?
                { diagnostics: diagnostics, sourceMaps: undefined, emittedFiles: undefined, emitSkipped: true } :
                undefined;
        }
        ts.handleNoEmitOptions = handleNoEmitOptions;
        /* @internal */
        function parseConfigHostFromCompilerHostLike(host, directoryStructureHost) {
            if (directoryStructureHost === void 0) {
                directoryStructureHost = host;
            }
            return {
                fileExists: function (f) { return directoryStructureHost.fileExists(f); },
                readDirectory: function (root, extensions, excludes, includes, depth) {
                    ts.Debug.assertDefined(directoryStructureHost.readDirectory, "'CompilerHost.readDirectory' must be implemented to correctly process 'projectReferences'");
                    return directoryStructureHost.readDirectory(root, extensions, excludes, includes, depth);
                },
                readFile: function (f) { return directoryStructureHost.readFile(f); },
                useCaseSensitiveFileNames: host.useCaseSensitiveFileNames(),
                getCurrentDirectory: function () { return host.getCurrentDirectory(); },
                onUnRecoverableConfigFileDiagnostic: host.onUnRecoverableConfigFileDiagnostic || ts.returnUndefined,
                trace: host.trace ? function (s) { return host.trace(s); } : undefined
            };
        }
        ts.parseConfigHostFromCompilerHostLike = parseConfigHostFromCompilerHostLike;
        /* @internal */
        function createPrependNodes(projectReferences, getCommandLine, readFile) {
            if (!projectReferences)
                return ts.emptyArray;
            var nodes;
            for (var i = 0; i < projectReferences.length; i++) {
                var ref = projectReferences[i];
                var resolvedRefOpts = getCommandLine(ref, i);
                if (ref.prepend && resolvedRefOpts && resolvedRefOpts.options) {
                    var out = resolvedRefOpts.options.outFile || resolvedRefOpts.options.out;
                    // Upstream project didn't have outFile set -- skip (error will have been issued earlier)
                    if (!out)
                        continue;
                    var _a = ts.getOutputPathsForBundle(resolvedRefOpts.options, /*forceDtsPaths*/ true), jsFilePath = _a.jsFilePath, sourceMapFilePath = _a.sourceMapFilePath, declarationFilePath = _a.declarationFilePath, declarationMapPath = _a.declarationMapPath, buildInfoPath = _a.buildInfoPath;
                    var node = ts.createInputFiles(readFile, jsFilePath, sourceMapFilePath, declarationFilePath, declarationMapPath, buildInfoPath);
                    (nodes || (nodes = [])).push(node);
                }
            }
            return nodes || ts.emptyArray;
        }
        ts.createPrependNodes = createPrependNodes;
        function resolveProjectReferencePath(hostOrRef, ref) {
            var passedInRef = ref ? ref : hostOrRef;
            return ts.resolveConfigFileProjectName(passedInRef.path);
        }
        ts.resolveProjectReferencePath = resolveProjectReferencePath;
        /* @internal */
        /**
         * Returns a DiagnosticMessage if we won't include a resolved module due to its extension.
         * The DiagnosticMessage's parameters are the imported module name, and the filename it resolved to.
         * This returns a diagnostic even if the module will be an untyped module.
         */
        function getResolutionDiagnostic(options, _a) {
            var extension = _a.extension;
            switch (extension) {
                case ".ts" /* Ts */:
                case ".d.ts" /* Dts */:
                    // These are always allowed.
                    return undefined;
                case ".tsx" /* Tsx */:
                    return needJsx();
                case ".jsx" /* Jsx */:
                    return needJsx() || needAllowJs();
                case ".js" /* Js */:
                    return needAllowJs();
                case ".json" /* Json */:
                    return needResolveJsonModule();
            }
            function needJsx() {
                return options.jsx ? undefined : ts.Diagnostics.Module_0_was_resolved_to_1_but_jsx_is_not_set;
            }
            function needAllowJs() {
                return options.allowJs || !ts.getStrictOptionValue(options, "noImplicitAny") ? undefined : ts.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type;
            }
            function needResolveJsonModule() {
                return options.resolveJsonModule ? undefined : ts.Diagnostics.Module_0_was_resolved_to_1_but_resolveJsonModule_is_not_used;
            }
        }
        ts.getResolutionDiagnostic = getResolutionDiagnostic;
        function getModuleNames(_a) {
            var imports = _a.imports, moduleAugmentations = _a.moduleAugmentations;
            var res = imports.map(function (i) { return i.text; });
            for (var _i = 0, moduleAugmentations_1 = moduleAugmentations; _i < moduleAugmentations_1.length; _i++) {
                var aug = moduleAugmentations_1[_i];
                if (aug.kind === 10 /* StringLiteral */) {
                    res.push(aug.text);
                }
                // Do nothing if it's an Identifier; we don't need to do module resolution for `declare global`.
            }
            return res;
        }
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        function getFileEmitOutput(program, sourceFile, emitOnlyDtsFiles, cancellationToken, customTransformers, forceDtsEmit) {
            var outputFiles = [];
            var emitResult = program.emit(sourceFile, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers, forceDtsEmit);
            return { outputFiles: outputFiles, emitSkipped: emitResult.emitSkipped, exportedModulesFromDeclarationEmit: emitResult.exportedModulesFromDeclarationEmit };
            function writeFile(fileName, text, writeByteOrderMark) {
                outputFiles.push({ name: fileName, writeByteOrderMark: writeByteOrderMark, text: text });
            }
        }
        ts.getFileEmitOutput = getFileEmitOutput;
        var BuilderState;
        (function (BuilderState) {
            /**
             * Get the referencedFile from the imported module symbol
             */
            function getReferencedFileFromImportedModuleSymbol(symbol) {
                if (symbol.declarations && symbol.declarations[0]) {
                    var declarationSourceFile = ts.getSourceFileOfNode(symbol.declarations[0]);
                    return declarationSourceFile && declarationSourceFile.resolvedPath;
                }
            }
            /**
             * Get the referencedFile from the import name node from file
             */
            function getReferencedFileFromImportLiteral(checker, importName) {
                var symbol = checker.getSymbolAtLocation(importName);
                return symbol && getReferencedFileFromImportedModuleSymbol(symbol);
            }
            /**
             * Gets the path to reference file from file name, it could be resolvedPath if present otherwise path
             */
            function getReferencedFileFromFileName(program, fileName, sourceFileDirectory, getCanonicalFileName) {
                return ts.toPath(program.getProjectReferenceRedirect(fileName) || fileName, sourceFileDirectory, getCanonicalFileName);
            }
            /**
             * Gets the referenced files for a file from the program with values for the keys as referenced file's path to be true
             */
            function getReferencedFiles(program, sourceFile, getCanonicalFileName) {
                var referencedFiles;
                // We need to use a set here since the code can contain the same import twice,
                // but that will only be one dependency.
                // To avoid invernal conversion, the key of the referencedFiles map must be of type Path
                if (sourceFile.imports && sourceFile.imports.length > 0) {
                    var checker = program.getTypeChecker();
                    for (var _i = 0, _a = sourceFile.imports; _i < _a.length; _i++) {
                        var importName = _a[_i];
                        var declarationSourceFilePath = getReferencedFileFromImportLiteral(checker, importName);
                        if (declarationSourceFilePath) {
                            addReferencedFile(declarationSourceFilePath);
                        }
                    }
                }
                var sourceFileDirectory = ts.getDirectoryPath(sourceFile.resolvedPath);
                // Handle triple slash references
                if (sourceFile.referencedFiles && sourceFile.referencedFiles.length > 0) {
                    for (var _b = 0, _c = sourceFile.referencedFiles; _b < _c.length; _b++) {
                        var referencedFile = _c[_b];
                        var referencedPath = getReferencedFileFromFileName(program, referencedFile.fileName, sourceFileDirectory, getCanonicalFileName);
                        addReferencedFile(referencedPath);
                    }
                }
                // Handle type reference directives
                if (sourceFile.resolvedTypeReferenceDirectiveNames) {
                    sourceFile.resolvedTypeReferenceDirectiveNames.forEach(function (resolvedTypeReferenceDirective) {
                        if (!resolvedTypeReferenceDirective) {
                            return;
                        }
                        var fileName = resolvedTypeReferenceDirective.resolvedFileName; // TODO: GH#18217
                        var typeFilePath = getReferencedFileFromFileName(program, fileName, sourceFileDirectory, getCanonicalFileName);
                        addReferencedFile(typeFilePath);
                    });
                }
                // Add module augmentation as references
                if (sourceFile.moduleAugmentations.length) {
                    var checker = program.getTypeChecker();
                    for (var _d = 0, _e = sourceFile.moduleAugmentations; _d < _e.length; _d++) {
                        var moduleName = _e[_d];
                        if (!ts.isStringLiteral(moduleName)) {
                            continue;
                        }
                        var symbol = checker.getSymbolAtLocation(moduleName);
                        if (!symbol) {
                            continue;
                        }
                        // Add any file other than our own as reference
                        addReferenceFromAmbientModule(symbol);
                    }
                }
                // From ambient modules
                for (var _f = 0, _g = program.getTypeChecker().getAmbientModules(); _f < _g.length; _f++) {
                    var ambientModule = _g[_f];
                    if (ambientModule.declarations.length > 1) {
                        addReferenceFromAmbientModule(ambientModule);
                    }
                }
                return referencedFiles;
                function addReferenceFromAmbientModule(symbol) {
                    // Add any file other than our own as reference
                    for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                        var declaration = _a[_i];
                        var declarationSourceFile = ts.getSourceFileOfNode(declaration);
                        if (declarationSourceFile &&
                            declarationSourceFile !== sourceFile) {
                            addReferencedFile(declarationSourceFile.resolvedPath);
                        }
                    }
                }
                function addReferencedFile(referencedPath) {
                    if (!referencedFiles) {
                        referencedFiles = ts.createMap();
                    }
                    referencedFiles.set(referencedPath, true);
                }
            }
            /**
             * Returns true if oldState is reusable, that is the emitKind = module/non module has not changed
             */
            function canReuseOldState(newReferencedMap, oldState) {
                return oldState && !oldState.referencedMap === !newReferencedMap;
            }
            BuilderState.canReuseOldState = canReuseOldState;
            /**
             * Creates the state of file references and signature for the new program from oldState if it is safe
             */
            function create(newProgram, getCanonicalFileName, oldState) {
                var fileInfos = ts.createMap();
                var referencedMap = newProgram.getCompilerOptions().module !== ts.ModuleKind.None ? ts.createMap() : undefined;
                var exportedModulesMap = referencedMap ? ts.createMap() : undefined;
                var hasCalledUpdateShapeSignature = ts.createMap();
                var useOldState = canReuseOldState(referencedMap, oldState);
                // Create the reference map, and set the file infos
                for (var _i = 0, _a = newProgram.getSourceFiles(); _i < _a.length; _i++) {
                    var sourceFile = _a[_i];
                    var version_1 = ts.Debug.assertDefined(sourceFile.version, "Program intended to be used with Builder should have source files with versions set");
                    var oldInfo = useOldState ? oldState.fileInfos.get(sourceFile.resolvedPath) : undefined;
                    if (referencedMap) {
                        var newReferences = getReferencedFiles(newProgram, sourceFile, getCanonicalFileName);
                        if (newReferences) {
                            referencedMap.set(sourceFile.resolvedPath, newReferences);
                        }
                        // Copy old visible to outside files map
                        if (useOldState) {
                            var exportedModules = oldState.exportedModulesMap.get(sourceFile.resolvedPath);
                            if (exportedModules) {
                                exportedModulesMap.set(sourceFile.resolvedPath, exportedModules);
                            }
                        }
                    }
                    fileInfos.set(sourceFile.resolvedPath, { version: version_1, signature: oldInfo && oldInfo.signature });
                }
                return {
                    fileInfos: fileInfos,
                    referencedMap: referencedMap,
                    exportedModulesMap: exportedModulesMap,
                    hasCalledUpdateShapeSignature: hasCalledUpdateShapeSignature
                };
            }
            BuilderState.create = create;
            /**
             * Releases needed properties
             */
            function releaseCache(state) {
                state.allFilesExcludingDefaultLibraryFile = undefined;
                state.allFileNames = undefined;
            }
            BuilderState.releaseCache = releaseCache;
            /**
             * Creates a clone of the state
             */
            function clone(state) {
                var fileInfos = ts.createMap();
                state.fileInfos.forEach(function (value, key) {
                    fileInfos.set(key, __assign({}, value));
                });
                // Dont need to backup allFiles info since its cache anyway
                return {
                    fileInfos: fileInfos,
                    referencedMap: cloneMapOrUndefined(state.referencedMap),
                    exportedModulesMap: cloneMapOrUndefined(state.exportedModulesMap),
                    hasCalledUpdateShapeSignature: ts.cloneMap(state.hasCalledUpdateShapeSignature),
                };
            }
            BuilderState.clone = clone;
            /**
             * Gets the files affected by the path from the program
             */
            function getFilesAffectedBy(state, programOfThisState, path, cancellationToken, computeHash, cacheToUpdateSignature, exportedModulesMapCache) {
                // Since the operation could be cancelled, the signatures are always stored in the cache
                // They will be committed once it is safe to use them
                // eg when calling this api from tsserver, if there is no cancellation of the operation
                // In the other cases the affected files signatures are committed only after the iteration through the result is complete
                var signatureCache = cacheToUpdateSignature || ts.createMap();
                var sourceFile = programOfThisState.getSourceFileByPath(path);
                if (!sourceFile) {
                    return ts.emptyArray;
                }
                if (!updateShapeSignature(state, programOfThisState, sourceFile, signatureCache, cancellationToken, computeHash, exportedModulesMapCache)) {
                    return [sourceFile];
                }
                var result = (state.referencedMap ? getFilesAffectedByUpdatedShapeWhenModuleEmit : getFilesAffectedByUpdatedShapeWhenNonModuleEmit)(state, programOfThisState, sourceFile, signatureCache, cancellationToken, computeHash, exportedModulesMapCache);
                if (!cacheToUpdateSignature) {
                    // Commit all the signatures in the signature cache
                    updateSignaturesFromCache(state, signatureCache);
                }
                return result;
            }
            BuilderState.getFilesAffectedBy = getFilesAffectedBy;
            /**
             * Updates the signatures from the cache into state's fileinfo signatures
             * This should be called whenever it is safe to commit the state of the builder
             */
            function updateSignaturesFromCache(state, signatureCache) {
                signatureCache.forEach(function (signature, path) {
                    state.fileInfos.get(path).signature = signature;
                    state.hasCalledUpdateShapeSignature.set(path, true);
                });
            }
            BuilderState.updateSignaturesFromCache = updateSignaturesFromCache;
            /**
             * Returns if the shape of the signature has changed since last emit
             */
            function updateShapeSignature(state, programOfThisState, sourceFile, cacheToUpdateSignature, cancellationToken, computeHash, exportedModulesMapCache) {
                ts.Debug.assert(!!sourceFile);
                ts.Debug.assert(!exportedModulesMapCache || !!state.exportedModulesMap, "Compute visible to outside map only if visibleToOutsideReferencedMap present in the state");
                // If we have cached the result for this file, that means hence forth we should assume file shape is uptodate
                if (state.hasCalledUpdateShapeSignature.has(sourceFile.resolvedPath) || cacheToUpdateSignature.has(sourceFile.resolvedPath)) {
                    return false;
                }
                var info = state.fileInfos.get(sourceFile.resolvedPath);
                if (!info)
                    return ts.Debug.fail();
                var prevSignature = info.signature;
                var latestSignature;
                if (sourceFile.isDeclarationFile) {
                    latestSignature = sourceFile.version;
                    if (exportedModulesMapCache && latestSignature !== prevSignature) {
                        // All the references in this file are exported
                        var references = state.referencedMap ? state.referencedMap.get(sourceFile.resolvedPath) : undefined;
                        exportedModulesMapCache.set(sourceFile.resolvedPath, references || false);
                    }
                }
                else {
                    var emitOutput_1 = getFileEmitOutput(programOfThisState, sourceFile, 
                    /*emitOnlyDtsFiles*/ true, cancellationToken, 
                    /*customTransformers*/ undefined, 
                    /*forceDtsEmit*/ true);
                    var firstDts_1 = emitOutput_1.outputFiles &&
                        programOfThisState.getCompilerOptions().declarationMap ?
                        emitOutput_1.outputFiles.length > 1 ? emitOutput_1.outputFiles[1] : undefined :
                        emitOutput_1.outputFiles.length > 0 ? emitOutput_1.outputFiles[0] : undefined;
                    if (firstDts_1) {
                        ts.Debug.assert(ts.fileExtensionIs(firstDts_1.name, ".d.ts" /* Dts */), "File extension for signature expected to be dts", function () { return "Found: " + ts.getAnyExtensionFromPath(firstDts_1.name) + " for " + firstDts_1.name + ":: All output files: " + JSON.stringify(emitOutput_1.outputFiles.map(function (f) { return f.name; })); });
                        latestSignature = computeHash(firstDts_1.text);
                        if (exportedModulesMapCache && latestSignature !== prevSignature) {
                            updateExportedModules(sourceFile, emitOutput_1.exportedModulesFromDeclarationEmit, exportedModulesMapCache);
                        }
                    }
                    else {
                        latestSignature = prevSignature; // TODO: GH#18217
                    }
                }
                cacheToUpdateSignature.set(sourceFile.resolvedPath, latestSignature);
                return !prevSignature || latestSignature !== prevSignature;
            }
            BuilderState.updateShapeSignature = updateShapeSignature;
            /**
             * Coverts the declaration emit result into exported modules map
             */
            function updateExportedModules(sourceFile, exportedModulesFromDeclarationEmit, exportedModulesMapCache) {
                if (!exportedModulesFromDeclarationEmit) {
                    exportedModulesMapCache.set(sourceFile.resolvedPath, false);
                    return;
                }
                var exportedModules;
                exportedModulesFromDeclarationEmit.forEach(function (symbol) { return addExportedModule(getReferencedFileFromImportedModuleSymbol(symbol)); });
                exportedModulesMapCache.set(sourceFile.resolvedPath, exportedModules || false);
                function addExportedModule(exportedModulePath) {
                    if (exportedModulePath) {
                        if (!exportedModules) {
                            exportedModules = ts.createMap();
                        }
                        exportedModules.set(exportedModulePath, true);
                    }
                }
            }
            /**
             * Updates the exported modules from cache into state's exported modules map
             * This should be called whenever it is safe to commit the state of the builder
             */
            function updateExportedFilesMapFromCache(state, exportedModulesMapCache) {
                if (exportedModulesMapCache) {
                    ts.Debug.assert(!!state.exportedModulesMap);
                    exportedModulesMapCache.forEach(function (exportedModules, path) {
                        if (exportedModules) {
                            state.exportedModulesMap.set(path, exportedModules);
                        }
                        else {
                            state.exportedModulesMap.delete(path);
                        }
                    });
                }
            }
            BuilderState.updateExportedFilesMapFromCache = updateExportedFilesMapFromCache;
            /**
             * Get all the dependencies of the sourceFile
             */
            function getAllDependencies(state, programOfThisState, sourceFile) {
                var compilerOptions = programOfThisState.getCompilerOptions();
                // With --out or --outFile all outputs go into single file, all files depend on each other
                if (compilerOptions.outFile || compilerOptions.out) {
                    return getAllFileNames(state, programOfThisState);
                }
                // If this is non module emit, or its a global file, it depends on all the source files
                if (!state.referencedMap || isFileAffectingGlobalScope(sourceFile)) {
                    return getAllFileNames(state, programOfThisState);
                }
                // Get the references, traversing deep from the referenceMap
                var seenMap = ts.createMap();
                var queue = [sourceFile.resolvedPath];
                while (queue.length) {
                    var path = queue.pop();
                    if (!seenMap.has(path)) {
                        seenMap.set(path, true);
                        var references = state.referencedMap.get(path);
                        if (references) {
                            var iterator = references.keys();
                            for (var iterResult = iterator.next(); !iterResult.done; iterResult = iterator.next()) {
                                queue.push(iterResult.value);
                            }
                        }
                    }
                }
                return ts.arrayFrom(ts.mapDefinedIterator(seenMap.keys(), function (path) {
                    var file = programOfThisState.getSourceFileByPath(path);
                    return file ? file.fileName : path;
                }));
            }
            BuilderState.getAllDependencies = getAllDependencies;
            /**
             * Gets the names of all files from the program
             */
            function getAllFileNames(state, programOfThisState) {
                if (!state.allFileNames) {
                    var sourceFiles = programOfThisState.getSourceFiles();
                    state.allFileNames = sourceFiles === ts.emptyArray ? ts.emptyArray : sourceFiles.map(function (file) { return file.fileName; });
                }
                return state.allFileNames;
            }
            /**
             * Gets the files referenced by the the file path
             */
            function getReferencedByPaths(state, referencedFilePath) {
                return ts.arrayFrom(ts.mapDefinedIterator(state.referencedMap.entries(), function (_a) {
                    var filePath = _a[0], referencesInFile = _a[1];
                    return referencesInFile.has(referencedFilePath) ? filePath : undefined;
                }));
            }
            BuilderState.getReferencedByPaths = getReferencedByPaths;
            /**
             * For script files that contains only ambient external modules, although they are not actually external module files,
             * they can only be consumed via importing elements from them. Regular script files cannot consume them. Therefore,
             * there are no point to rebuild all script files if these special files have changed. However, if any statement
             * in the file is not ambient external module, we treat it as a regular script file.
             */
            function containsOnlyAmbientModules(sourceFile) {
                for (var _i = 0, _a = sourceFile.statements; _i < _a.length; _i++) {
                    var statement = _a[_i];
                    if (!ts.isModuleWithStringLiteralName(statement)) {
                        return false;
                    }
                }
                return true;
            }
            /**
             * Return true if file contains anything that augments to global scope we need to build them as if
             * they are global files as well as module
             */
            function containsGlobalScopeAugmentation(sourceFile) {
                return ts.some(sourceFile.moduleAugmentations, function (augmentation) { return ts.isGlobalScopeAugmentation(augmentation.parent); });
            }
            /**
             * Return true if the file will invalidate all files because it affectes global scope
             */
            function isFileAffectingGlobalScope(sourceFile) {
                return containsGlobalScopeAugmentation(sourceFile) ||
                    !ts.isExternalModule(sourceFile) && !containsOnlyAmbientModules(sourceFile);
            }
            /**
             * Gets all files of the program excluding the default library file
             */
            function getAllFilesExcludingDefaultLibraryFile(state, programOfThisState, firstSourceFile) {
                // Use cached result
                if (state.allFilesExcludingDefaultLibraryFile) {
                    return state.allFilesExcludingDefaultLibraryFile;
                }
                var result;
                addSourceFile(firstSourceFile);
                for (var _i = 0, _a = programOfThisState.getSourceFiles(); _i < _a.length; _i++) {
                    var sourceFile = _a[_i];
                    if (sourceFile !== firstSourceFile) {
                        addSourceFile(sourceFile);
                    }
                }
                state.allFilesExcludingDefaultLibraryFile = result || ts.emptyArray;
                return state.allFilesExcludingDefaultLibraryFile;
                function addSourceFile(sourceFile) {
                    if (!programOfThisState.isSourceFileDefaultLibrary(sourceFile)) {
                        (result || (result = [])).push(sourceFile);
                    }
                }
            }
            /**
             * When program emits non modular code, gets the files affected by the sourceFile whose shape has changed
             */
            function getFilesAffectedByUpdatedShapeWhenNonModuleEmit(state, programOfThisState, sourceFileWithUpdatedShape) {
                var compilerOptions = programOfThisState.getCompilerOptions();
                // If `--out` or `--outFile` is specified, any new emit will result in re-emitting the entire project,
                // so returning the file itself is good enough.
                if (compilerOptions && (compilerOptions.out || compilerOptions.outFile)) {
                    return [sourceFileWithUpdatedShape];
                }
                return getAllFilesExcludingDefaultLibraryFile(state, programOfThisState, sourceFileWithUpdatedShape);
            }
            /**
             * When program emits modular code, gets the files affected by the sourceFile whose shape has changed
             */
            function getFilesAffectedByUpdatedShapeWhenModuleEmit(state, programOfThisState, sourceFileWithUpdatedShape, cacheToUpdateSignature, cancellationToken, computeHash, exportedModulesMapCache) {
                if (isFileAffectingGlobalScope(sourceFileWithUpdatedShape)) {
                    return getAllFilesExcludingDefaultLibraryFile(state, programOfThisState, sourceFileWithUpdatedShape);
                }
                var compilerOptions = programOfThisState.getCompilerOptions();
                if (compilerOptions && (compilerOptions.isolatedModules || compilerOptions.out || compilerOptions.outFile)) {
                    return [sourceFileWithUpdatedShape];
                }
                // Now we need to if each file in the referencedBy list has a shape change as well.
                // Because if so, its own referencedBy files need to be saved as well to make the
                // emitting result consistent with files on disk.
                var seenFileNamesMap = ts.createMap();
                // Start with the paths this file was referenced by
                seenFileNamesMap.set(sourceFileWithUpdatedShape.resolvedPath, sourceFileWithUpdatedShape);
                var queue = getReferencedByPaths(state, sourceFileWithUpdatedShape.resolvedPath);
                while (queue.length > 0) {
                    var currentPath = queue.pop();
                    if (!seenFileNamesMap.has(currentPath)) {
                        var currentSourceFile = programOfThisState.getSourceFileByPath(currentPath);
                        seenFileNamesMap.set(currentPath, currentSourceFile);
                        if (currentSourceFile && updateShapeSignature(state, programOfThisState, currentSourceFile, cacheToUpdateSignature, cancellationToken, computeHash, exportedModulesMapCache)) { // TODO: GH#18217
                            queue.push.apply(// TODO: GH#18217
                            queue, getReferencedByPaths(state, currentSourceFile.resolvedPath));
                        }
                    }
                }
                // Return array of values that needs emit
                // Return array of values that needs emit
                return ts.arrayFrom(ts.mapDefinedIterator(seenFileNamesMap.values(), function (value) { return value; }));
            }
        })(BuilderState = ts.BuilderState || (ts.BuilderState = {}));
        function cloneMapOrUndefined(map) {
            return map ? ts.cloneMap(map) : undefined;
        }
        ts.cloneMapOrUndefined = cloneMapOrUndefined;
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        var BuilderFileEmit;
        (function (BuilderFileEmit) {
            BuilderFileEmit[BuilderFileEmit["DtsOnly"] = 0] = "DtsOnly";
            BuilderFileEmit[BuilderFileEmit["Full"] = 1] = "Full";
        })(BuilderFileEmit = ts.BuilderFileEmit || (ts.BuilderFileEmit = {}));
        function hasSameKeys(map1, map2) {
            // Has same size and every key is present in both maps
            return map1 === map2 || map1 !== undefined && map2 !== undefined && map1.size === map2.size && !ts.forEachKey(map1, function (key) { return !map2.has(key); });
        }
        /**
         * Create the state so that we can iterate on changedFiles/affected files
         */
        function createBuilderProgramState(newProgram, getCanonicalFileName, oldState) {
            var state = ts.BuilderState.create(newProgram, getCanonicalFileName, oldState);
            state.program = newProgram;
            var compilerOptions = newProgram.getCompilerOptions();
            state.compilerOptions = compilerOptions;
            // With --out or --outFile, any change affects all semantic diagnostics so no need to cache them
            if (!compilerOptions.outFile && !compilerOptions.out) {
                state.semanticDiagnosticsPerFile = ts.createMap();
            }
            state.changedFilesSet = ts.createMap();
            var useOldState = ts.BuilderState.canReuseOldState(state.referencedMap, oldState);
            var oldCompilerOptions = useOldState ? oldState.compilerOptions : undefined;
            var canCopySemanticDiagnostics = useOldState && oldState.semanticDiagnosticsPerFile && !!state.semanticDiagnosticsPerFile &&
                !ts.compilerOptionsAffectSemanticDiagnostics(compilerOptions, oldCompilerOptions);
            if (useOldState) {
                // Verify the sanity of old state
                if (!oldState.currentChangedFilePath) {
                    var affectedSignatures = oldState.currentAffectedFilesSignatures;
                    ts.Debug.assert(!oldState.affectedFiles && (!affectedSignatures || !affectedSignatures.size), "Cannot reuse if only few affected files of currentChangedFile were iterated");
                }
                var changedFilesSet = oldState.changedFilesSet;
                if (canCopySemanticDiagnostics) {
                    ts.Debug.assert(!changedFilesSet || !ts.forEachKey(changedFilesSet, function (path) { return oldState.semanticDiagnosticsPerFile.has(path); }), "Semantic diagnostics shouldnt be available for changed files");
                }
                // Copy old state's changed files set
                if (changedFilesSet) {
                    ts.copyEntries(changedFilesSet, state.changedFilesSet);
                }
                if (!compilerOptions.outFile && !compilerOptions.out && oldState.affectedFilesPendingEmit) {
                    state.affectedFilesPendingEmit = oldState.affectedFilesPendingEmit.slice();
                    state.affectedFilesPendingEmitKind = ts.cloneMapOrUndefined(oldState.affectedFilesPendingEmitKind);
                    state.affectedFilesPendingEmitIndex = oldState.affectedFilesPendingEmitIndex;
                }
            }
            // Update changed files and copy semantic diagnostics if we can
            var referencedMap = state.referencedMap;
            var oldReferencedMap = useOldState ? oldState.referencedMap : undefined;
            var copyDeclarationFileDiagnostics = canCopySemanticDiagnostics && !compilerOptions.skipLibCheck === !oldCompilerOptions.skipLibCheck;
            var copyLibFileDiagnostics = copyDeclarationFileDiagnostics && !compilerOptions.skipDefaultLibCheck === !oldCompilerOptions.skipDefaultLibCheck;
            state.fileInfos.forEach(function (info, sourceFilePath) {
                var oldInfo;
                var newReferences;
                // if not using old state, every file is changed
                if (!useOldState ||
                    // File wasnt present in old state
                    !(oldInfo = oldState.fileInfos.get(sourceFilePath)) ||
                    // versions dont match
                    oldInfo.version !== info.version ||
                    // Referenced files changed
                    !hasSameKeys(newReferences = referencedMap && referencedMap.get(sourceFilePath), oldReferencedMap && oldReferencedMap.get(sourceFilePath)) ||
                    // Referenced file was deleted in the new program
                    newReferences && ts.forEachKey(newReferences, function (path) { return !state.fileInfos.has(path) && oldState.fileInfos.has(path); })) {
                    // Register file as changed file and do not copy semantic diagnostics, since all changed files need to be re-evaluated
                    state.changedFilesSet.set(sourceFilePath, true);
                }
                else if (canCopySemanticDiagnostics) {
                    var sourceFile = newProgram.getSourceFileByPath(sourceFilePath);
                    if (sourceFile.isDeclarationFile && !copyDeclarationFileDiagnostics) {
                        return;
                    }
                    if (sourceFile.hasNoDefaultLib && !copyLibFileDiagnostics) {
                        return;
                    }
                    // Unchanged file copy diagnostics
                    var diagnostics = oldState.semanticDiagnosticsPerFile.get(sourceFilePath);
                    if (diagnostics) {
                        state.semanticDiagnosticsPerFile.set(sourceFilePath, oldState.hasReusableDiagnostic ? convertToDiagnostics(diagnostics, newProgram, getCanonicalFileName) : diagnostics);
                        if (!state.semanticDiagnosticsFromOldState) {
                            state.semanticDiagnosticsFromOldState = ts.createMap();
                        }
                        state.semanticDiagnosticsFromOldState.set(sourceFilePath, true);
                    }
                }
            });
            if (oldCompilerOptions && ts.compilerOptionsAffectEmit(compilerOptions, oldCompilerOptions)) {
                // Add all files to affectedFilesPendingEmit since emit changed
                newProgram.getSourceFiles().forEach(function (f) { return addToAffectedFilesPendingEmit(state, f.resolvedPath, 1 /* Full */); });
                ts.Debug.assert(state.seenAffectedFiles === undefined);
                state.seenAffectedFiles = ts.createMap();
            }
            state.emittedBuildInfo = !state.changedFilesSet.size && !state.affectedFilesPendingEmit;
            return state;
        }
        function convertToDiagnostics(diagnostics, newProgram, getCanonicalFileName) {
            if (!diagnostics.length)
                return ts.emptyArray;
            var buildInfoDirectory = ts.getDirectoryPath(ts.getNormalizedAbsolutePath(ts.getTsBuildInfoEmitOutputFilePath(newProgram.getCompilerOptions()), newProgram.getCurrentDirectory()));
            return diagnostics.map(function (diagnostic) {
                var result = convertToDiagnosticRelatedInformation(diagnostic, newProgram, toPath);
                result.reportsUnnecessary = diagnostic.reportsUnnecessary;
                result.source = diagnostic.source;
                var relatedInformation = diagnostic.relatedInformation;
                result.relatedInformation = relatedInformation ?
                    relatedInformation.length ?
                        relatedInformation.map(function (r) { return convertToDiagnosticRelatedInformation(r, newProgram, toPath); }) :
                        ts.emptyArray :
                    undefined;
                return result;
            });
            function toPath(path) {
                return ts.toPath(path, buildInfoDirectory, getCanonicalFileName);
            }
        }
        function convertToDiagnosticRelatedInformation(diagnostic, newProgram, toPath) {
            var file = diagnostic.file;
            return __assign(__assign({}, diagnostic), { file: file ? newProgram.getSourceFileByPath(toPath(file)) : undefined });
        }
        /**
         * Releases program and other related not needed properties
         */
        function releaseCache(state) {
            ts.BuilderState.releaseCache(state);
            state.program = undefined;
        }
        /**
         * Creates a clone of the state
         */
        function cloneBuilderProgramState(state) {
            var newState = ts.BuilderState.clone(state);
            newState.semanticDiagnosticsPerFile = ts.cloneMapOrUndefined(state.semanticDiagnosticsPerFile);
            newState.changedFilesSet = ts.cloneMap(state.changedFilesSet);
            newState.affectedFiles = state.affectedFiles;
            newState.affectedFilesIndex = state.affectedFilesIndex;
            newState.currentChangedFilePath = state.currentChangedFilePath;
            newState.currentAffectedFilesSignatures = ts.cloneMapOrUndefined(state.currentAffectedFilesSignatures);
            newState.currentAffectedFilesExportedModulesMap = ts.cloneMapOrUndefined(state.currentAffectedFilesExportedModulesMap);
            newState.seenAffectedFiles = ts.cloneMapOrUndefined(state.seenAffectedFiles);
            newState.cleanedDiagnosticsOfLibFiles = state.cleanedDiagnosticsOfLibFiles;
            newState.semanticDiagnosticsFromOldState = ts.cloneMapOrUndefined(state.semanticDiagnosticsFromOldState);
            newState.program = state.program;
            newState.compilerOptions = state.compilerOptions;
            newState.affectedFilesPendingEmit = state.affectedFilesPendingEmit && state.affectedFilesPendingEmit.slice();
            newState.affectedFilesPendingEmitKind = ts.cloneMapOrUndefined(state.affectedFilesPendingEmitKind);
            newState.affectedFilesPendingEmitIndex = state.affectedFilesPendingEmitIndex;
            newState.seenEmittedFiles = ts.cloneMapOrUndefined(state.seenEmittedFiles);
            newState.programEmitComplete = state.programEmitComplete;
            return newState;
        }
        /**
         * Verifies that source file is ok to be used in calls that arent handled by next
         */
        function assertSourceFileOkWithoutNextAffectedCall(state, sourceFile) {
            ts.Debug.assert(!sourceFile || !state.affectedFiles || state.affectedFiles[state.affectedFilesIndex - 1] !== sourceFile || !state.semanticDiagnosticsPerFile.has(sourceFile.resolvedPath));
        }
        /**
         * This function returns the next affected file to be processed.
         * Note that until doneAffected is called it would keep reporting same result
         * This is to allow the callers to be able to actually remove affected file only when the operation is complete
         * eg. if during diagnostics check cancellation token ends up cancelling the request, the affected file should be retained
         */
        function getNextAffectedFile(state, cancellationToken, computeHash) {
            while (true) {
                var affectedFiles = state.affectedFiles;
                if (affectedFiles) {
                    var seenAffectedFiles = state.seenAffectedFiles;
                    var affectedFilesIndex = state.affectedFilesIndex; // TODO: GH#18217
                    while (affectedFilesIndex < affectedFiles.length) {
                        var affectedFile = affectedFiles[affectedFilesIndex];
                        if (!seenAffectedFiles.has(affectedFile.resolvedPath)) {
                            // Set the next affected file as seen and remove the cached semantic diagnostics
                            state.affectedFilesIndex = affectedFilesIndex;
                            handleDtsMayChangeOfAffectedFile(state, affectedFile, cancellationToken, computeHash);
                            return affectedFile;
                        }
                        affectedFilesIndex++;
                    }
                    // Remove the changed file from the change set
                    state.changedFilesSet.delete(state.currentChangedFilePath);
                    state.currentChangedFilePath = undefined;
                    // Commit the changes in file signature
                    ts.BuilderState.updateSignaturesFromCache(state, state.currentAffectedFilesSignatures);
                    state.currentAffectedFilesSignatures.clear();
                    ts.BuilderState.updateExportedFilesMapFromCache(state, state.currentAffectedFilesExportedModulesMap);
                    state.affectedFiles = undefined;
                }
                // Get next changed file
                var nextKey = state.changedFilesSet.keys().next();
                if (nextKey.done) {
                    // Done
                    return undefined;
                }
                // With --out or --outFile all outputs go into single file
                // so operations are performed directly on program, return program
                var program = ts.Debug.assertDefined(state.program);
                var compilerOptions = program.getCompilerOptions();
                if (compilerOptions.outFile || compilerOptions.out) {
                    ts.Debug.assert(!state.semanticDiagnosticsPerFile);
                    return program;
                }
                // Get next batch of affected files
                state.currentAffectedFilesSignatures = state.currentAffectedFilesSignatures || ts.createMap();
                if (state.exportedModulesMap) {
                    state.currentAffectedFilesExportedModulesMap = state.currentAffectedFilesExportedModulesMap || ts.createMap();
                }
                state.affectedFiles = ts.BuilderState.getFilesAffectedBy(state, program, nextKey.value, cancellationToken, computeHash, state.currentAffectedFilesSignatures, state.currentAffectedFilesExportedModulesMap);
                state.currentChangedFilePath = nextKey.value;
                state.affectedFilesIndex = 0;
                state.seenAffectedFiles = state.seenAffectedFiles || ts.createMap();
            }
        }
        /**
         * Returns next file to be emitted from files that retrieved semantic diagnostics but did not emit yet
         */
        function getNextAffectedFilePendingEmit(state) {
            var affectedFilesPendingEmit = state.affectedFilesPendingEmit;
            if (affectedFilesPendingEmit) {
                var seenEmittedFiles = state.seenEmittedFiles || (state.seenEmittedFiles = ts.createMap());
                for (var i = state.affectedFilesPendingEmitIndex; i < affectedFilesPendingEmit.length; i++) {
                    var affectedFile = ts.Debug.assertDefined(state.program).getSourceFileByPath(affectedFilesPendingEmit[i]);
                    if (affectedFile) {
                        var seenKind = seenEmittedFiles.get(affectedFile.resolvedPath);
                        var emitKind = ts.Debug.assertDefined(ts.Debug.assertDefined(state.affectedFilesPendingEmitKind).get(affectedFile.resolvedPath));
                        if (seenKind === undefined || seenKind < emitKind) {
                            // emit this file
                            state.affectedFilesPendingEmitIndex = i;
                            return { affectedFile: affectedFile, emitKind: emitKind };
                        }
                    }
                }
                state.affectedFilesPendingEmit = undefined;
                state.affectedFilesPendingEmitKind = undefined;
                state.affectedFilesPendingEmitIndex = undefined;
            }
            return undefined;
        }
        /**
         *  Handles semantic diagnostics and dts emit for affectedFile and files, that are referencing modules that export entities from affected file
         *  This is because even though js emit doesnt change, dts emit / type used can change resulting in need for dts emit and js change
         */
        function handleDtsMayChangeOfAffectedFile(state, affectedFile, cancellationToken, computeHash) {
            removeSemanticDiagnosticsOf(state, affectedFile.resolvedPath);
            // If affected files is everything except default library, then nothing more to do
            if (state.allFilesExcludingDefaultLibraryFile === state.affectedFiles) {
                if (!state.cleanedDiagnosticsOfLibFiles) {
                    state.cleanedDiagnosticsOfLibFiles = true;
                    var program_1 = ts.Debug.assertDefined(state.program);
                    var options_2 = program_1.getCompilerOptions();
                    ts.forEach(program_1.getSourceFiles(), function (f) {
                        return program_1.isSourceFileDefaultLibrary(f) &&
                            !ts.skipTypeChecking(f, options_2, program_1) &&
                            removeSemanticDiagnosticsOf(state, f.resolvedPath);
                    });
                }
                return;
            }
            if (!state.compilerOptions.assumeChangesOnlyAffectDirectDependencies) {
                forEachReferencingModulesOfExportOfAffectedFile(state, affectedFile, function (state, path) { return handleDtsMayChangeOf(state, path, cancellationToken, computeHash); });
            }
        }
        /**
         * Handle the dts may change, so they need to be added to pending emit if dts emit is enabled,
         * Also we need to make sure signature is updated for these files
         */
        function handleDtsMayChangeOf(state, path, cancellationToken, computeHash) {
            removeSemanticDiagnosticsOf(state, path);
            if (!state.changedFilesSet.has(path)) {
                var program = ts.Debug.assertDefined(state.program);
                var sourceFile = program.getSourceFileByPath(path);
                if (sourceFile) {
                    // Even though the js emit doesnt change and we are already handling dts emit and semantic diagnostics
                    // we need to update the signature to reflect correctness of the signature(which is output d.ts emit) of this file
                    // This ensures that we dont later during incremental builds considering wrong signature.
                    // Eg where this also is needed to ensure that .tsbuildinfo generated by incremental build should be same as if it was first fresh build
                    ts.BuilderState.updateShapeSignature(state, program, sourceFile, ts.Debug.assertDefined(state.currentAffectedFilesSignatures), cancellationToken, computeHash, state.currentAffectedFilesExportedModulesMap);
                    // If not dts emit, nothing more to do
                    if (ts.getEmitDeclarations(state.compilerOptions)) {
                        addToAffectedFilesPendingEmit(state, path, 0 /* DtsOnly */);
                    }
                }
            }
            return false;
        }
        /**
         * Removes semantic diagnostics for path and
         * returns true if there are no more semantic diagnostics from the old state
         */
        function removeSemanticDiagnosticsOf(state, path) {
            if (!state.semanticDiagnosticsFromOldState) {
                return true;
            }
            state.semanticDiagnosticsFromOldState.delete(path);
            state.semanticDiagnosticsPerFile.delete(path);
            return !state.semanticDiagnosticsFromOldState.size;
        }
        function isChangedSignagure(state, path) {
            var newSignature = ts.Debug.assertDefined(state.currentAffectedFilesSignatures).get(path);
            var oldSignagure = ts.Debug.assertDefined(state.fileInfos.get(path)).signature;
            return newSignature !== oldSignagure;
        }
        /**
         * Iterate on referencing modules that export entities from affected file
         */
        function forEachReferencingModulesOfExportOfAffectedFile(state, affectedFile, fn) {
            // If there was change in signature (dts output) for the changed file,
            // then only we need to handle pending file emit
            if (!state.exportedModulesMap || !state.changedFilesSet.has(affectedFile.resolvedPath)) {
                return;
            }
            if (!isChangedSignagure(state, affectedFile.resolvedPath))
                return;
            // Since isolated modules dont change js files, files affected by change in signature is itself
            // But we need to cleanup semantic diagnostics and queue dts emit for affected files
            if (state.compilerOptions.isolatedModules) {
                var seenFileNamesMap = ts.createMap();
                seenFileNamesMap.set(affectedFile.resolvedPath, true);
                var queue = ts.BuilderState.getReferencedByPaths(state, affectedFile.resolvedPath);
                while (queue.length > 0) {
                    var currentPath = queue.pop();
                    if (!seenFileNamesMap.has(currentPath)) {
                        seenFileNamesMap.set(currentPath, true);
                        var result = fn(state, currentPath);
                        if (result && isChangedSignagure(state, currentPath)) {
                            var currentSourceFile = ts.Debug.assertDefined(state.program).getSourceFileByPath(currentPath);
                            queue.push.apply(queue, ts.BuilderState.getReferencedByPaths(state, currentSourceFile.resolvedPath));
                        }
                    }
                }
            }
            ts.Debug.assert(!!state.currentAffectedFilesExportedModulesMap);
            var seenFileAndExportsOfFile = ts.createMap();
            // Go through exported modules from cache first
            // If exported modules has path, all files referencing file exported from are affected
            if (ts.forEachEntry(state.currentAffectedFilesExportedModulesMap, function (exportedModules, exportedFromPath) {
                return exportedModules &&
                    exportedModules.has(affectedFile.resolvedPath) &&
                    forEachFilesReferencingPath(state, exportedFromPath, seenFileAndExportsOfFile, fn);
            })) {
                return;
            }
            // If exported from path is not from cache and exported modules has path, all files referencing file exported from are affected
            ts.forEachEntry(state.exportedModulesMap, function (exportedModules, exportedFromPath) {
                return !state.currentAffectedFilesExportedModulesMap.has(exportedFromPath) && // If we already iterated this through cache, ignore it
                    exportedModules.has(affectedFile.resolvedPath) &&
                    forEachFilesReferencingPath(state, exportedFromPath, seenFileAndExportsOfFile, fn);
            });
        }
        /**
         * Iterate on files referencing referencedPath
         */
        function forEachFilesReferencingPath(state, referencedPath, seenFileAndExportsOfFile, fn) {
            return ts.forEachEntry(state.referencedMap, function (referencesInFile, filePath) {
                return referencesInFile.has(referencedPath) && forEachFileAndExportsOfFile(state, filePath, seenFileAndExportsOfFile, fn);
            });
        }
        /**
         * fn on file and iterate on anything that exports this file
         */
        function forEachFileAndExportsOfFile(state, filePath, seenFileAndExportsOfFile, fn) {
            if (!ts.addToSeen(seenFileAndExportsOfFile, filePath)) {
                return false;
            }
            if (fn(state, filePath)) {
                // If there are no more diagnostics from old cache, done
                return true;
            }
            ts.Debug.assert(!!state.currentAffectedFilesExportedModulesMap);
            // Go through exported modules from cache first
            // If exported modules has path, all files referencing file exported from are affected
            if (ts.forEachEntry(state.currentAffectedFilesExportedModulesMap, function (exportedModules, exportedFromPath) {
                return exportedModules &&
                    exportedModules.has(filePath) &&
                    forEachFileAndExportsOfFile(state, exportedFromPath, seenFileAndExportsOfFile, fn);
            })) {
                return true;
            }
            // If exported from path is not from cache and exported modules has path, all files referencing file exported from are affected
            if (ts.forEachEntry(state.exportedModulesMap, function (exportedModules, exportedFromPath) {
                return !state.currentAffectedFilesExportedModulesMap.has(exportedFromPath) && // If we already iterated this through cache, ignore it
                    exportedModules.has(filePath) &&
                    forEachFileAndExportsOfFile(state, exportedFromPath, seenFileAndExportsOfFile, fn);
            })) {
                return true;
            }
            // Remove diagnostics of files that import this file (without going to exports of referencing files)
            return !!ts.forEachEntry(state.referencedMap, function (referencesInFile, referencingFilePath) {
                return referencesInFile.has(filePath) &&
                    !seenFileAndExportsOfFile.has(referencingFilePath) && // Not already removed diagnostic file
                    fn(state, referencingFilePath);
            } // Dont add to seen since this is not yet done with the export removal
            );
        }
        /**
         * This is called after completing operation on the next affected file.
         * The operations here are postponed to ensure that cancellation during the iteration is handled correctly
         */
        function doneWithAffectedFile(state, affected, emitKind, isPendingEmit, isBuildInfoEmit) {
            if (isBuildInfoEmit) {
                state.emittedBuildInfo = true;
            }
            else if (affected === state.program) {
                state.changedFilesSet.clear();
                state.programEmitComplete = true;
            }
            else {
                state.seenAffectedFiles.set(affected.resolvedPath, true);
                if (emitKind !== undefined) {
                    (state.seenEmittedFiles || (state.seenEmittedFiles = ts.createMap())).set(affected.resolvedPath, emitKind);
                }
                if (isPendingEmit) {
                    state.affectedFilesPendingEmitIndex++;
                }
                else {
                    state.affectedFilesIndex++;
                }
            }
        }
        /**
         * Returns the result with affected file
         */
        function toAffectedFileResult(state, result, affected) {
            doneWithAffectedFile(state, affected);
            return { result: result, affected: affected };
        }
        /**
         * Returns the result with affected file
         */
        function toAffectedFileEmitResult(state, result, affected, emitKind, isPendingEmit, isBuildInfoEmit) {
            doneWithAffectedFile(state, affected, emitKind, isPendingEmit, isBuildInfoEmit);
            return { result: result, affected: affected };
        }
        /**
         * Gets semantic diagnostics for the file which are
         * bindAndCheckDiagnostics (from cache) and program diagnostics
         */
        function getSemanticDiagnosticsOfFile(state, sourceFile, cancellationToken) {
            return ts.concatenate(getBinderAndCheckerDiagnosticsOfFile(state, sourceFile, cancellationToken), ts.Debug.assertDefined(state.program).getProgramDiagnostics(sourceFile));
        }
        /**
         * Gets the binder and checker diagnostics either from cache if present, or otherwise from program and caches it
         * Note that it is assumed that when asked about binder and checker diagnostics, the file has been taken out of affected files/changed file set
         */
        function getBinderAndCheckerDiagnosticsOfFile(state, sourceFile, cancellationToken) {
            var path = sourceFile.resolvedPath;
            if (state.semanticDiagnosticsPerFile) {
                var cachedDiagnostics = state.semanticDiagnosticsPerFile.get(path);
                // Report the bind and check diagnostics from the cache if we already have those diagnostics present
                if (cachedDiagnostics) {
                    return cachedDiagnostics;
                }
            }
            // Diagnostics werent cached, get them from program, and cache the result
            var diagnostics = ts.Debug.assertDefined(state.program).getBindAndCheckDiagnostics(sourceFile, cancellationToken);
            if (state.semanticDiagnosticsPerFile) {
                state.semanticDiagnosticsPerFile.set(path, diagnostics);
            }
            return diagnostics;
        }
        /**
         * Gets the program information to be emitted in buildInfo so that we can use it to create new program
         */
        function getProgramBuildInfo(state, getCanonicalFileName) {
            if (state.compilerOptions.outFile || state.compilerOptions.out)
                return undefined;
            var currentDirectory = ts.Debug.assertDefined(state.program).getCurrentDirectory();
            var buildInfoDirectory = ts.getDirectoryPath(ts.getNormalizedAbsolutePath(ts.getTsBuildInfoEmitOutputFilePath(state.compilerOptions), currentDirectory));
            var fileInfos = {};
            state.fileInfos.forEach(function (value, key) {
                var signature = state.currentAffectedFilesSignatures && state.currentAffectedFilesSignatures.get(key);
                fileInfos[relativeToBuildInfo(key)] = signature === undefined ? value : { version: value.version, signature: signature };
            });
            var result = {
                fileInfos: fileInfos,
                options: convertToReusableCompilerOptions(state.compilerOptions, relativeToBuildInfoEnsuringAbsolutePath)
            };
            if (state.referencedMap) {
                var referencedMap_1 = {};
                state.referencedMap.forEach(function (value, key) {
                    referencedMap_1[relativeToBuildInfo(key)] = ts.arrayFrom(value.keys(), relativeToBuildInfo);
                });
                result.referencedMap = referencedMap_1;
            }
            if (state.exportedModulesMap) {
                var exportedModulesMap_1 = {};
                state.exportedModulesMap.forEach(function (value, key) {
                    var newValue = state.currentAffectedFilesExportedModulesMap && state.currentAffectedFilesExportedModulesMap.get(key);
                    // Not in temporary cache, use existing value
                    if (newValue === undefined)
                        exportedModulesMap_1[relativeToBuildInfo(key)] = ts.arrayFrom(value.keys(), relativeToBuildInfo);
                    // Value in cache and has updated value map, use that
                    else if (newValue)
                        exportedModulesMap_1[relativeToBuildInfo(key)] = ts.arrayFrom(newValue.keys(), relativeToBuildInfo);
                });
                result.exportedModulesMap = exportedModulesMap_1;
            }
            if (state.semanticDiagnosticsPerFile) {
                var semanticDiagnosticsPerFile_1 = [];
                // Currently not recording actual errors since those mean no emit for tsc --build
                state.semanticDiagnosticsPerFile.forEach(function (value, key) {
                    return semanticDiagnosticsPerFile_1.push(value.length ?
                        [
                            relativeToBuildInfo(key),
                            state.hasReusableDiagnostic ?
                                value :
                                convertToReusableDiagnostics(value, relativeToBuildInfo)
                        ] :
                        relativeToBuildInfo(key));
                });
                result.semanticDiagnosticsPerFile = semanticDiagnosticsPerFile_1;
            }
            return result;
            function relativeToBuildInfoEnsuringAbsolutePath(path) {
                return relativeToBuildInfo(ts.getNormalizedAbsolutePath(path, currentDirectory));
            }
            function relativeToBuildInfo(path) {
                return ts.ensurePathIsNonModuleName(ts.getRelativePathFromDirectory(buildInfoDirectory, path, getCanonicalFileName));
            }
        }
        function convertToReusableCompilerOptions(options, relativeToBuildInfo) {
            var result = {};
            var optionsNameMap = ts.getOptionsNameMap().optionsNameMap;
            for (var name in options) {
                if (ts.hasProperty(options, name)) {
                    result[name] = convertToReusableCompilerOptionValue(optionsNameMap.get(name.toLowerCase()), options[name], relativeToBuildInfo);
                }
            }
            if (result.configFilePath) {
                result.configFilePath = relativeToBuildInfo(result.configFilePath);
            }
            return result;
        }
        function convertToReusableCompilerOptionValue(option, value, relativeToBuildInfo) {
            if (option) {
                if (option.type === "list") {
                    var values = value;
                    if (option.element.isFilePath && values.length) {
                        return values.map(relativeToBuildInfo);
                    }
                }
                else if (option.isFilePath) {
                    return relativeToBuildInfo(value);
                }
            }
            return value;
        }
        function convertToReusableDiagnostics(diagnostics, relativeToBuildInfo) {
            ts.Debug.assert(!!diagnostics.length);
            return diagnostics.map(function (diagnostic) {
                var result = convertToReusableDiagnosticRelatedInformation(diagnostic, relativeToBuildInfo);
                result.reportsUnnecessary = diagnostic.reportsUnnecessary;
                result.source = diagnostic.source;
                var relatedInformation = diagnostic.relatedInformation;
                result.relatedInformation = relatedInformation ?
                    relatedInformation.length ?
                        relatedInformation.map(function (r) { return convertToReusableDiagnosticRelatedInformation(r, relativeToBuildInfo); }) :
                        ts.emptyArray :
                    undefined;
                return result;
            });
        }
        function convertToReusableDiagnosticRelatedInformation(diagnostic, relativeToBuildInfo) {
            var file = diagnostic.file;
            return __assign(__assign({}, diagnostic), { file: file ? relativeToBuildInfo(file.resolvedPath) : undefined });
        }
        var BuilderProgramKind;
        (function (BuilderProgramKind) {
            BuilderProgramKind[BuilderProgramKind["SemanticDiagnosticsBuilderProgram"] = 0] = "SemanticDiagnosticsBuilderProgram";
            BuilderProgramKind[BuilderProgramKind["EmitAndSemanticDiagnosticsBuilderProgram"] = 1] = "EmitAndSemanticDiagnosticsBuilderProgram";
        })(BuilderProgramKind = ts.BuilderProgramKind || (ts.BuilderProgramKind = {}));
        function getBuilderCreationParameters(newProgramOrRootNames, hostOrOptions, oldProgramOrHost, configFileParsingDiagnosticsOrOldProgram, configFileParsingDiagnostics, projectReferences) {
            var host;
            var newProgram;
            var oldProgram;
            if (newProgramOrRootNames === undefined) {
                ts.Debug.assert(hostOrOptions === undefined);
                host = oldProgramOrHost;
                oldProgram = configFileParsingDiagnosticsOrOldProgram;
                ts.Debug.assert(!!oldProgram);
                newProgram = oldProgram.getProgram();
            }
            else if (ts.isArray(newProgramOrRootNames)) {
                oldProgram = configFileParsingDiagnosticsOrOldProgram;
                newProgram = ts.createProgram({
                    rootNames: newProgramOrRootNames,
                    options: hostOrOptions,
                    host: oldProgramOrHost,
                    oldProgram: oldProgram && oldProgram.getProgramOrUndefined(),
                    configFileParsingDiagnostics: configFileParsingDiagnostics,
                    projectReferences: projectReferences
                });
                host = oldProgramOrHost;
            }
            else {
                newProgram = newProgramOrRootNames;
                host = hostOrOptions;
                oldProgram = oldProgramOrHost;
                configFileParsingDiagnostics = configFileParsingDiagnosticsOrOldProgram;
            }
            return { host: host, newProgram: newProgram, oldProgram: oldProgram, configFileParsingDiagnostics: configFileParsingDiagnostics || ts.emptyArray };
        }
        ts.getBuilderCreationParameters = getBuilderCreationParameters;
        function createBuilderProgram(kind, _a) {
            var newProgram = _a.newProgram, host = _a.host, oldProgram = _a.oldProgram, configFileParsingDiagnostics = _a.configFileParsingDiagnostics;
            // Return same program if underlying program doesnt change
            var oldState = oldProgram && oldProgram.getState();
            if (oldState && newProgram === oldState.program && configFileParsingDiagnostics === newProgram.getConfigFileParsingDiagnostics()) {
                newProgram = undefined; // TODO: GH#18217
                oldState = undefined;
                return oldProgram;
            }
            /**
             * Create the canonical file name for identity
             */
            var getCanonicalFileName = ts.createGetCanonicalFileName(host.useCaseSensitiveFileNames());
            /**
             * Computing hash to for signature verification
             */
            var computeHash = host.createHash || ts.generateDjb2Hash;
            var state = createBuilderProgramState(newProgram, getCanonicalFileName, oldState);
            var backupState;
            newProgram.getProgramBuildInfo = function () { return getProgramBuildInfo(state, getCanonicalFileName); };
            // To ensure that we arent storing any references to old program or new program without state
            newProgram = undefined; // TODO: GH#18217
            oldProgram = undefined;
            oldState = undefined;
            var builderProgram = createRedirectedBuilderProgram(state, configFileParsingDiagnostics);
            builderProgram.getState = function () { return state; };
            builderProgram.backupState = function () {
                ts.Debug.assert(backupState === undefined);
                backupState = cloneBuilderProgramState(state);
            };
            builderProgram.restoreState = function () {
                state = ts.Debug.assertDefined(backupState);
                backupState = undefined;
            };
            builderProgram.getAllDependencies = function (sourceFile) { return ts.BuilderState.getAllDependencies(state, ts.Debug.assertDefined(state.program), sourceFile); };
            builderProgram.getSemanticDiagnostics = getSemanticDiagnostics;
            builderProgram.emit = emit;
            builderProgram.releaseProgram = function () {
                releaseCache(state);
                backupState = undefined;
            };
            if (kind === BuilderProgramKind.SemanticDiagnosticsBuilderProgram) {
                builderProgram.getSemanticDiagnosticsOfNextAffectedFile = getSemanticDiagnosticsOfNextAffectedFile;
            }
            else if (kind === BuilderProgramKind.EmitAndSemanticDiagnosticsBuilderProgram) {
                builderProgram.getSemanticDiagnosticsOfNextAffectedFile = getSemanticDiagnosticsOfNextAffectedFile;
                builderProgram.emitNextAffectedFile = emitNextAffectedFile;
            }
            else {
                ts.notImplemented();
            }
            return builderProgram;
            /**
             * Emits the next affected file's emit result (EmitResult and sourceFiles emitted) or returns undefined if iteration is complete
             * The first of writeFile if provided, writeFile of BuilderProgramHost if provided, writeFile of compiler host
             * in that order would be used to write the files
             */
            function emitNextAffectedFile(writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers) {
                var affected = getNextAffectedFile(state, cancellationToken, computeHash);
                var emitKind = 1 /* Full */;
                var isPendingEmitFile = false;
                if (!affected) {
                    if (!state.compilerOptions.out && !state.compilerOptions.outFile) {
                        var pendingAffectedFile = getNextAffectedFilePendingEmit(state);
                        if (!pendingAffectedFile) {
                            if (state.emittedBuildInfo) {
                                return undefined;
                            }
                            var affected_1 = ts.Debug.assertDefined(state.program);
                            return toAffectedFileEmitResult(state, 
                            // When whole program is affected, do emit only once (eg when --out or --outFile is specified)
                            // Otherwise just affected file
                            affected_1.emitBuildInfo(writeFile || ts.maybeBind(host, host.writeFile), cancellationToken), affected_1, 1 /* Full */, 
                            /*isPendingEmitFile*/ false, 
                            /*isBuildInfoEmit*/ true);
                        }
                        (affected = pendingAffectedFile.affectedFile, emitKind = pendingAffectedFile.emitKind);
                        isPendingEmitFile = true;
                    }
                    else {
                        var program = ts.Debug.assertDefined(state.program);
                        if (state.programEmitComplete)
                            return undefined;
                        affected = program;
                    }
                }
                return toAffectedFileEmitResult(state, 
                // When whole program is affected, do emit only once (eg when --out or --outFile is specified)
                // Otherwise just affected file
                ts.Debug.assertDefined(state.program).emit(affected === state.program ? undefined : affected, writeFile || ts.maybeBind(host, host.writeFile), cancellationToken, emitOnlyDtsFiles || emitKind === 0 /* DtsOnly */, customTransformers), affected, emitKind, isPendingEmitFile);
            }
            /**
             * Emits the JavaScript and declaration files.
             * When targetSource file is specified, emits the files corresponding to that source file,
             * otherwise for the whole program.
             * In case of EmitAndSemanticDiagnosticsBuilderProgram, when targetSourceFile is specified,
             * it is assumed that that file is handled from affected file list. If targetSourceFile is not specified,
             * it will only emit all the affected files instead of whole program
             *
             * The first of writeFile if provided, writeFile of BuilderProgramHost if provided, writeFile of compiler host
             * in that order would be used to write the files
             */
            function emit(targetSourceFile, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers) {
                if (kind === BuilderProgramKind.EmitAndSemanticDiagnosticsBuilderProgram) {
                    assertSourceFileOkWithoutNextAffectedCall(state, targetSourceFile);
                    var result = ts.handleNoEmitOptions(builderProgram, targetSourceFile, cancellationToken);
                    if (result)
                        return result;
                    if (!targetSourceFile) {
                        // Emit and report any errors we ran into.
                        var sourceMaps = [];
                        var emitSkipped = false;
                        var diagnostics = void 0;
                        var emittedFiles = [];
                        var affectedEmitResult = void 0;
                        while (affectedEmitResult = emitNextAffectedFile(writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers)) {
                            emitSkipped = emitSkipped || affectedEmitResult.result.emitSkipped;
                            diagnostics = ts.addRange(diagnostics, affectedEmitResult.result.diagnostics);
                            emittedFiles = ts.addRange(emittedFiles, affectedEmitResult.result.emittedFiles);
                            sourceMaps = ts.addRange(sourceMaps, affectedEmitResult.result.sourceMaps);
                        }
                        return {
                            emitSkipped: emitSkipped,
                            diagnostics: diagnostics || ts.emptyArray,
                            emittedFiles: emittedFiles,
                            sourceMaps: sourceMaps
                        };
                    }
                }
                return ts.Debug.assertDefined(state.program).emit(targetSourceFile, writeFile || ts.maybeBind(host, host.writeFile), cancellationToken, emitOnlyDtsFiles, customTransformers);
            }
            /**
             * Return the semantic diagnostics for the next affected file or undefined if iteration is complete
             * If provided ignoreSourceFile would be called before getting the diagnostics and would ignore the sourceFile if the returned value was true
             */
            function getSemanticDiagnosticsOfNextAffectedFile(cancellationToken, ignoreSourceFile) {
                while (true) {
                    var affected = getNextAffectedFile(state, cancellationToken, computeHash);
                    if (!affected) {
                        // Done
                        return undefined;
                    }
                    else if (affected === state.program) {
                        // When whole program is affected, get all semantic diagnostics (eg when --out or --outFile is specified)
                        return toAffectedFileResult(state, state.program.getSemanticDiagnostics(/*targetSourceFile*/ undefined, cancellationToken), affected);
                    }
                    // Add file to affected file pending emit to handle for later emit time
                    if (kind === BuilderProgramKind.EmitAndSemanticDiagnosticsBuilderProgram) {
                        addToAffectedFilesPendingEmit(state, affected.resolvedPath, 1 /* Full */);
                    }
                    // Get diagnostics for the affected file if its not ignored
                    if (ignoreSourceFile && ignoreSourceFile(affected)) {
                        // Get next affected file
                        doneWithAffectedFile(state, affected);
                        continue;
                    }
                    return toAffectedFileResult(state, getSemanticDiagnosticsOfFile(state, affected, cancellationToken), affected);
                }
            }
            /**
             * Gets the semantic diagnostics from the program corresponding to this state of file (if provided) or whole program
             * The semantic diagnostics are cached and managed here
             * Note that it is assumed that when asked about semantic diagnostics through this API,
             * the file has been taken out of affected files so it is safe to use cache or get from program and cache the diagnostics
             * In case of SemanticDiagnosticsBuilderProgram if the source file is not provided,
             * it will iterate through all the affected files, to ensure that cache stays valid and yet provide a way to get all semantic diagnostics
             */
            function getSemanticDiagnostics(sourceFile, cancellationToken) {
                assertSourceFileOkWithoutNextAffectedCall(state, sourceFile);
                var compilerOptions = ts.Debug.assertDefined(state.program).getCompilerOptions();
                if (compilerOptions.outFile || compilerOptions.out) {
                    ts.Debug.assert(!state.semanticDiagnosticsPerFile);
                    // We dont need to cache the diagnostics just return them from program
                    return ts.Debug.assertDefined(state.program).getSemanticDiagnostics(sourceFile, cancellationToken);
                }
                if (sourceFile) {
                    return getSemanticDiagnosticsOfFile(state, sourceFile, cancellationToken);
                }
                // When semantic builder asks for diagnostics of the whole program,
                // ensure that all the affected files are handled
                // eslint-disable-next-line no-empty
                while (getSemanticDiagnosticsOfNextAffectedFile(cancellationToken)) {
                }
                var diagnostics;
                for (var _i = 0, _a = ts.Debug.assertDefined(state.program).getSourceFiles(); _i < _a.length; _i++) {
                    var sourceFile_1 = _a[_i];
                    diagnostics = ts.addRange(diagnostics, getSemanticDiagnosticsOfFile(state, sourceFile_1, cancellationToken));
                }
                return diagnostics || ts.emptyArray;
            }
        }
        ts.createBuilderProgram = createBuilderProgram;
        function addToAffectedFilesPendingEmit(state, affectedFilePendingEmit, kind) {
            if (!state.affectedFilesPendingEmit)
                state.affectedFilesPendingEmit = [];
            if (!state.affectedFilesPendingEmitKind)
                state.affectedFilesPendingEmitKind = ts.createMap();
            var existingKind = state.affectedFilesPendingEmitKind.get(affectedFilePendingEmit);
            state.affectedFilesPendingEmit.push(affectedFilePendingEmit);
            state.affectedFilesPendingEmitKind.set(affectedFilePendingEmit, existingKind || kind);
            // affectedFilesPendingEmitIndex === undefined
            // - means the emit state.affectedFilesPendingEmit was undefined before adding current affected files
            //   so start from 0 as array would be affectedFilesPendingEmit
            // else, continue to iterate from existing index, the current set is appended to existing files
            if (state.affectedFilesPendingEmitIndex === undefined) {
                state.affectedFilesPendingEmitIndex = 0;
            }
        }
        function getMapOfReferencedSet(mapLike, toPath) {
            if (!mapLike)
                return undefined;
            var map = ts.createMap();
            // Copies keys/values from template. Note that for..in will not throw if
            // template is undefined, and instead will just exit the loop.
            for (var key in mapLike) {
                if (ts.hasProperty(mapLike, key)) {
                    map.set(toPath(key), ts.arrayToSet(mapLike[key], toPath));
                }
            }
            return map;
        }
        function createBuildProgramUsingProgramBuildInfo(program, buildInfoPath, host) {
            var buildInfoDirectory = ts.getDirectoryPath(ts.getNormalizedAbsolutePath(buildInfoPath, host.getCurrentDirectory()));
            var getCanonicalFileName = ts.createGetCanonicalFileName(host.useCaseSensitiveFileNames());
            var fileInfos = ts.createMap();
            for (var key in program.fileInfos) {
                if (ts.hasProperty(program.fileInfos, key)) {
                    fileInfos.set(toPath(key), program.fileInfos[key]);
                }
            }
            var state = {
                fileInfos: fileInfos,
                compilerOptions: ts.convertToOptionsWithAbsolutePaths(program.options, toAbsolutePath),
                referencedMap: getMapOfReferencedSet(program.referencedMap, toPath),
                exportedModulesMap: getMapOfReferencedSet(program.exportedModulesMap, toPath),
                semanticDiagnosticsPerFile: program.semanticDiagnosticsPerFile && ts.arrayToMap(program.semanticDiagnosticsPerFile, function (value) { return toPath(ts.isString(value) ? value : value[0]); }, function (value) { return ts.isString(value) ? ts.emptyArray : value[1]; }),
                hasReusableDiagnostic: true
            };
            return {
                getState: function () { return state; },
                backupState: ts.noop,
                restoreState: ts.noop,
                getProgram: ts.notImplemented,
                getProgramOrUndefined: ts.returnUndefined,
                releaseProgram: ts.noop,
                getCompilerOptions: function () { return state.compilerOptions; },
                getSourceFile: ts.notImplemented,
                getSourceFiles: ts.notImplemented,
                getOptionsDiagnostics: ts.notImplemented,
                getGlobalDiagnostics: ts.notImplemented,
                getConfigFileParsingDiagnostics: ts.notImplemented,
                getSyntacticDiagnostics: ts.notImplemented,
                getDeclarationDiagnostics: ts.notImplemented,
                getSemanticDiagnostics: ts.notImplemented,
                emit: ts.notImplemented,
                getAllDependencies: ts.notImplemented,
                getCurrentDirectory: ts.notImplemented,
                emitNextAffectedFile: ts.notImplemented,
                getSemanticDiagnosticsOfNextAffectedFile: ts.notImplemented,
                close: ts.noop,
            };
            function toPath(path) {
                return ts.toPath(path, buildInfoDirectory, getCanonicalFileName);
            }
            function toAbsolutePath(path) {
                return ts.getNormalizedAbsolutePath(path, buildInfoDirectory);
            }
        }
        ts.createBuildProgramUsingProgramBuildInfo = createBuildProgramUsingProgramBuildInfo;
        function createRedirectedBuilderProgram(state, configFileParsingDiagnostics) {
            return {
                getState: ts.notImplemented,
                backupState: ts.noop,
                restoreState: ts.noop,
                getProgram: getProgram,
                getProgramOrUndefined: function () { return state.program; },
                releaseProgram: function () { return state.program = undefined; },
                getCompilerOptions: function () { return state.compilerOptions; },
                getSourceFile: function (fileName) { return getProgram().getSourceFile(fileName); },
                getSourceFiles: function () { return getProgram().getSourceFiles(); },
                getOptionsDiagnostics: function (cancellationToken) { return getProgram().getOptionsDiagnostics(cancellationToken); },
                getGlobalDiagnostics: function (cancellationToken) { return getProgram().getGlobalDiagnostics(cancellationToken); },
                getConfigFileParsingDiagnostics: function () { return configFileParsingDiagnostics; },
                getSyntacticDiagnostics: function (sourceFile, cancellationToken) { return getProgram().getSyntacticDiagnostics(sourceFile, cancellationToken); },
                getDeclarationDiagnostics: function (sourceFile, cancellationToken) { return getProgram().getDeclarationDiagnostics(sourceFile, cancellationToken); },
                getSemanticDiagnostics: function (sourceFile, cancellationToken) { return getProgram().getSemanticDiagnostics(sourceFile, cancellationToken); },
                emit: function (sourceFile, writeFile, cancellationToken, emitOnlyDts, customTransformers) { return getProgram().emit(sourceFile, writeFile, cancellationToken, emitOnlyDts, customTransformers); },
                getAllDependencies: ts.notImplemented,
                getCurrentDirectory: function () { return getProgram().getCurrentDirectory(); },
                close: ts.noop,
            };
            function getProgram() {
                return ts.Debug.assertDefined(state.program);
            }
        }
        ts.createRedirectedBuilderProgram = createRedirectedBuilderProgram;
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        function createSemanticDiagnosticsBuilderProgram(newProgramOrRootNames, hostOrOptions, oldProgramOrHost, configFileParsingDiagnosticsOrOldProgram, configFileParsingDiagnostics, projectReferences) {
            return ts.createBuilderProgram(ts.BuilderProgramKind.SemanticDiagnosticsBuilderProgram, ts.getBuilderCreationParameters(newProgramOrRootNames, hostOrOptions, oldProgramOrHost, configFileParsingDiagnosticsOrOldProgram, configFileParsingDiagnostics, projectReferences));
        }
        ts.createSemanticDiagnosticsBuilderProgram = createSemanticDiagnosticsBuilderProgram;
        function createEmitAndSemanticDiagnosticsBuilderProgram(newProgramOrRootNames, hostOrOptions, oldProgramOrHost, configFileParsingDiagnosticsOrOldProgram, configFileParsingDiagnostics, projectReferences) {
            return ts.createBuilderProgram(ts.BuilderProgramKind.EmitAndSemanticDiagnosticsBuilderProgram, ts.getBuilderCreationParameters(newProgramOrRootNames, hostOrOptions, oldProgramOrHost, configFileParsingDiagnosticsOrOldProgram, configFileParsingDiagnostics, projectReferences));
        }
        ts.createEmitAndSemanticDiagnosticsBuilderProgram = createEmitAndSemanticDiagnosticsBuilderProgram;
        function createAbstractBuilder(newProgramOrRootNames, hostOrOptions, oldProgramOrHost, configFileParsingDiagnosticsOrOldProgram, configFileParsingDiagnostics, projectReferences) {
            var _a = ts.getBuilderCreationParameters(newProgramOrRootNames, hostOrOptions, oldProgramOrHost, configFileParsingDiagnosticsOrOldProgram, configFileParsingDiagnostics, projectReferences), newProgram = _a.newProgram, newConfigFileParsingDiagnostics = _a.configFileParsingDiagnostics;
            return ts.createRedirectedBuilderProgram({ program: newProgram, compilerOptions: newProgram.getCompilerOptions() }, newConfigFileParsingDiagnostics);
        }
        ts.createAbstractBuilder = createAbstractBuilder;
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        function removeIgnoredPath(path) {
            // Consider whole staging folder as if node_modules changed.
            if (ts.endsWith(path, "/node_modules/.staging")) {
                return ts.removeSuffix(path, "/.staging");
            }
            return ts.some(ts.ignoredPaths, function (searchPath) { return ts.stringContains(path, searchPath); }) ?
                undefined :
                path;
        }
        ts.removeIgnoredPath = removeIgnoredPath;
        /**
         * Filter out paths like
         * "/", "/user", "/user/username", "/user/username/folderAtRoot",
         * "c:/", "c:/users", "c:/users/username", "c:/users/username/folderAtRoot", "c:/folderAtRoot"
         * @param dirPath
         */
        function canWatchDirectory(dirPath) {
            var rootLength = ts.getRootLength(dirPath);
            if (dirPath.length === rootLength) {
                // Ignore "/", "c:/"
                return false;
            }
            var nextDirectorySeparator = dirPath.indexOf(ts.directorySeparator, rootLength);
            if (nextDirectorySeparator === -1) {
                // ignore "/user", "c:/users" or "c:/folderAtRoot"
                return false;
            }
            var pathPartForUserCheck = dirPath.substring(rootLength, nextDirectorySeparator + 1);
            var isNonDirectorySeparatorRoot = rootLength > 1 || dirPath.charCodeAt(0) !== 47 /* slash */;
            if (isNonDirectorySeparatorRoot &&
                dirPath.search(/[a-zA-Z]:/) !== 0 && // Non dos style paths
                pathPartForUserCheck.search(/[a-zA-z]\$\//) === 0) { // Dos style nextPart
                nextDirectorySeparator = dirPath.indexOf(ts.directorySeparator, nextDirectorySeparator + 1);
                if (nextDirectorySeparator === -1) {
                    // ignore "//vda1cs4850/c$/folderAtRoot"
                    return false;
                }
                pathPartForUserCheck = dirPath.substring(rootLength + pathPartForUserCheck.length, nextDirectorySeparator + 1);
            }
            if (isNonDirectorySeparatorRoot &&
                pathPartForUserCheck.search(/users\//i) !== 0) {
                // Paths like c:/folderAtRoot/subFolder are allowed
                return true;
            }
            for (var searchIndex = nextDirectorySeparator + 1, searchLevels = 2; searchLevels > 0; searchLevels--) {
                searchIndex = dirPath.indexOf(ts.directorySeparator, searchIndex) + 1;
                if (searchIndex === 0) {
                    // Folder isnt at expected minimum levels
                    return false;
                }
            }
            return true;
        }
        ts.canWatchDirectory = canWatchDirectory;
        ts.maxNumberOfFilesToIterateForInvalidation = 256;
        function createResolutionCache(resolutionHost, rootDirForResolution, logChangesWhenResolvingModule) {
            var filesWithChangedSetOfUnresolvedImports;
            var filesWithInvalidatedResolutions;
            var filesWithInvalidatedNonRelativeUnresolvedImports;
            var allFilesHaveInvalidatedResolution = false;
            var nonRelativeExternalModuleResolutions = ts.createMultiMap();
            var getCurrentDirectory = ts.memoize(function () { return resolutionHost.getCurrentDirectory(); }); // TODO: GH#18217
            var cachedDirectoryStructureHost = resolutionHost.getCachedDirectoryStructureHost();
            // The resolvedModuleNames and resolvedTypeReferenceDirectives are the cache of resolutions per file.
            // The key in the map is source file's path.
            // The values are Map of resolutions with key being name lookedup.
            var resolvedModuleNames = ts.createMap();
            var perDirectoryResolvedModuleNames = ts.createCacheWithRedirects();
            var nonRelativeModuleNameCache = ts.createCacheWithRedirects();
            var moduleResolutionCache = ts.createModuleResolutionCacheWithMaps(perDirectoryResolvedModuleNames, nonRelativeModuleNameCache, getCurrentDirectory(), resolutionHost.getCanonicalFileName);
            var resolvedTypeReferenceDirectives = ts.createMap();
            var perDirectoryResolvedTypeReferenceDirectives = ts.createCacheWithRedirects();
            /**
             * These are the extensions that failed lookup files will have by default,
             * any other extension of failed lookup will be store that path in custom failed lookup path
             * This helps in not having to comb through all resolutions when files are added/removed
             * Note that .d.ts file also has .d.ts extension hence will be part of default extensions
             */
            var failedLookupDefaultExtensions = [".ts" /* Ts */, ".tsx" /* Tsx */, ".js" /* Js */, ".jsx" /* Jsx */, ".json" /* Json */];
            var customFailedLookupPaths = ts.createMap();
            var directoryWatchesOfFailedLookups = ts.createMap();
            var rootDir = rootDirForResolution && ts.removeTrailingDirectorySeparator(ts.getNormalizedAbsolutePath(rootDirForResolution, getCurrentDirectory()));
            var rootPath = (rootDir && resolutionHost.toPath(rootDir)); // TODO: GH#18217
            var rootSplitLength = rootPath !== undefined ? rootPath.split(ts.directorySeparator).length : 0;
            // TypeRoot watches for the types that get added as part of getAutomaticTypeDirectiveNames
            var typeRootsWatches = ts.createMap();
            return {
                startRecordingFilesWithChangedResolutions: startRecordingFilesWithChangedResolutions,
                finishRecordingFilesWithChangedResolutions: finishRecordingFilesWithChangedResolutions,
                // perDirectoryResolvedModuleNames and perDirectoryResolvedTypeReferenceDirectives could be non empty if there was exception during program update
                // (between startCachingPerDirectoryResolution and finishCachingPerDirectoryResolution)
                startCachingPerDirectoryResolution: clearPerDirectoryResolutions,
                finishCachingPerDirectoryResolution: finishCachingPerDirectoryResolution,
                resolveModuleNames: resolveModuleNames,
                getResolvedModuleWithFailedLookupLocationsFromCache: getResolvedModuleWithFailedLookupLocationsFromCache,
                resolveTypeReferenceDirectives: resolveTypeReferenceDirectives,
                removeResolutionsFromProjectReferenceRedirects: removeResolutionsFromProjectReferenceRedirects,
                removeResolutionsOfFile: removeResolutionsOfFile,
                invalidateResolutionOfFile: invalidateResolutionOfFile,
                setFilesWithInvalidatedNonRelativeUnresolvedImports: setFilesWithInvalidatedNonRelativeUnresolvedImports,
                createHasInvalidatedResolution: createHasInvalidatedResolution,
                updateTypeRootsWatch: updateTypeRootsWatch,
                closeTypeRootsWatch: closeTypeRootsWatch,
                clear: clear
            };
            function getResolvedModule(resolution) {
                return resolution.resolvedModule;
            }
            function getResolvedTypeReferenceDirective(resolution) {
                return resolution.resolvedTypeReferenceDirective;
            }
            function isInDirectoryPath(dir, file) {
                if (dir === undefined || file.length <= dir.length) {
                    return false;
                }
                return ts.startsWith(file, dir) && file[dir.length] === ts.directorySeparator;
            }
            function clear() {
                ts.clearMap(directoryWatchesOfFailedLookups, ts.closeFileWatcherOf);
                customFailedLookupPaths.clear();
                nonRelativeExternalModuleResolutions.clear();
                closeTypeRootsWatch();
                resolvedModuleNames.clear();
                resolvedTypeReferenceDirectives.clear();
                allFilesHaveInvalidatedResolution = false;
                // perDirectoryResolvedModuleNames and perDirectoryResolvedTypeReferenceDirectives could be non empty if there was exception during program update
                // (between startCachingPerDirectoryResolution and finishCachingPerDirectoryResolution)
                clearPerDirectoryResolutions();
            }
            function startRecordingFilesWithChangedResolutions() {
                filesWithChangedSetOfUnresolvedImports = [];
            }
            function finishRecordingFilesWithChangedResolutions() {
                var collected = filesWithChangedSetOfUnresolvedImports;
                filesWithChangedSetOfUnresolvedImports = undefined;
                return collected;
            }
            function isFileWithInvalidatedNonRelativeUnresolvedImports(path) {
                if (!filesWithInvalidatedNonRelativeUnresolvedImports) {
                    return false;
                }
                // Invalidated if file has unresolved imports
                var value = filesWithInvalidatedNonRelativeUnresolvedImports.get(path);
                return !!value && !!value.length;
            }
            function createHasInvalidatedResolution(forceAllFilesAsInvalidated) {
                if (allFilesHaveInvalidatedResolution || forceAllFilesAsInvalidated) {
                    // Any file asked would have invalidated resolution
                    filesWithInvalidatedResolutions = undefined;
                    return ts.returnTrue;
                }
                var collected = filesWithInvalidatedResolutions;
                filesWithInvalidatedResolutions = undefined;
                return function (path) {
                    return (!!collected && collected.has(path)) ||
                        isFileWithInvalidatedNonRelativeUnresolvedImports(path);
                };
            }
            function clearPerDirectoryResolutions() {
                perDirectoryResolvedModuleNames.clear();
                nonRelativeModuleNameCache.clear();
                perDirectoryResolvedTypeReferenceDirectives.clear();
                nonRelativeExternalModuleResolutions.forEach(watchFailedLookupLocationOfNonRelativeModuleResolutions);
                nonRelativeExternalModuleResolutions.clear();
            }
            function finishCachingPerDirectoryResolution() {
                allFilesHaveInvalidatedResolution = false;
                filesWithInvalidatedNonRelativeUnresolvedImports = undefined;
                clearPerDirectoryResolutions();
                directoryWatchesOfFailedLookups.forEach(function (watcher, path) {
                    if (watcher.refCount === 0) {
                        directoryWatchesOfFailedLookups.delete(path);
                        watcher.watcher.close();
                    }
                });
            }
            function resolveModuleName(moduleName, containingFile, compilerOptions, host, redirectedReference) {
                var primaryResult = ts.resolveModuleName(moduleName, containingFile, compilerOptions, host, moduleResolutionCache, redirectedReference);
                // return result immediately only if global cache support is not enabled or if it is .ts, .tsx or .d.ts
                if (!resolutionHost.getGlobalCache) {
                    return primaryResult;
                }
                // otherwise try to load typings from @types
                var globalCache = resolutionHost.getGlobalCache();
                if (globalCache !== undefined && !ts.isExternalModuleNameRelative(moduleName) && !(primaryResult.resolvedModule && ts.extensionIsTS(primaryResult.resolvedModule.extension))) {
                    // create different collection of failed lookup locations for second pass
                    // if it will fail and we've already found something during the first pass - we don't want to pollute its results
                    var _a = ts.loadModuleFromGlobalCache(ts.Debug.assertDefined(resolutionHost.globalCacheResolutionModuleName)(moduleName), resolutionHost.projectName, compilerOptions, host, globalCache), resolvedModule = _a.resolvedModule, failedLookupLocations = _a.failedLookupLocations;
                    if (resolvedModule) {
                        return { resolvedModule: resolvedModule, failedLookupLocations: ts.addRange(primaryResult.failedLookupLocations, failedLookupLocations) };
                    }
                }
                // Default return the result from the first pass
                return primaryResult;
            }
            function resolveNamesWithLocalCache(names, containingFile, redirectedReference, cache, perDirectoryCacheWithRedirects, loader, getResolutionWithResolvedFileName, shouldRetryResolution, reusedNames, logChanges) {
                var path = resolutionHost.toPath(containingFile);
                var resolutionsInFile = cache.get(path) || cache.set(path, ts.createMap()).get(path);
                var dirPath = ts.getDirectoryPath(path);
                var perDirectoryCache = perDirectoryCacheWithRedirects.getOrCreateMapOfCacheRedirects(redirectedReference);
                var perDirectoryResolution = perDirectoryCache.get(dirPath);
                if (!perDirectoryResolution) {
                    perDirectoryResolution = ts.createMap();
                    perDirectoryCache.set(dirPath, perDirectoryResolution);
                }
                var resolvedModules = [];
                var compilerOptions = resolutionHost.getCompilationSettings();
                var hasInvalidatedNonRelativeUnresolvedImport = logChanges && isFileWithInvalidatedNonRelativeUnresolvedImports(path);
                // All the resolutions in this file are invalidated if this file wasnt resolved using same redirect
                var program = resolutionHost.getCurrentProgram();
                var oldRedirect = program && program.getResolvedProjectReferenceToRedirect(containingFile);
                var unmatchedRedirects = oldRedirect ?
                    !redirectedReference || redirectedReference.sourceFile.path !== oldRedirect.sourceFile.path :
                    !!redirectedReference;
                var seenNamesInFile = ts.createMap();
                for (var _i = 0, names_3 = names; _i < names_3.length; _i++) {
                    var name = names_3[_i];
                    var resolution = resolutionsInFile.get(name);
                    // Resolution is valid if it is present and not invalidated
                    if (!seenNamesInFile.has(name) &&
                        allFilesHaveInvalidatedResolution || unmatchedRedirects || !resolution || resolution.isInvalidated ||
                        // If the name is unresolved import that was invalidated, recalculate
                        (hasInvalidatedNonRelativeUnresolvedImport && !ts.isExternalModuleNameRelative(name) && shouldRetryResolution(resolution))) {
                        var existingResolution = resolution;
                        var resolutionInDirectory = perDirectoryResolution.get(name);
                        if (resolutionInDirectory) {
                            resolution = resolutionInDirectory;
                        }
                        else {
                            resolution = loader(name, containingFile, compilerOptions, resolutionHost, redirectedReference);
                            perDirectoryResolution.set(name, resolution);
                        }
                        resolutionsInFile.set(name, resolution);
                        watchFailedLookupLocationsOfExternalModuleResolutions(name, resolution);
                        if (existingResolution) {
                            stopWatchFailedLookupLocationOfResolution(existingResolution);
                        }
                        if (logChanges && filesWithChangedSetOfUnresolvedImports && !resolutionIsEqualTo(existingResolution, resolution)) {
                            filesWithChangedSetOfUnresolvedImports.push(path);
                            // reset log changes to avoid recording the same file multiple times
                            logChanges = false;
                        }
                    }
                    ts.Debug.assert(resolution !== undefined && !resolution.isInvalidated);
                    seenNamesInFile.set(name, true);
                    resolvedModules.push(getResolutionWithResolvedFileName(resolution));
                }
                // Stop watching and remove the unused name
                resolutionsInFile.forEach(function (resolution, name) {
                    if (!seenNamesInFile.has(name) && !ts.contains(reusedNames, name)) {
                        stopWatchFailedLookupLocationOfResolution(resolution);
                        resolutionsInFile.delete(name);
                    }
                });
                return resolvedModules;
                function resolutionIsEqualTo(oldResolution, newResolution) {
                    if (oldResolution === newResolution) {
                        return true;
                    }
                    if (!oldResolution || !newResolution) {
                        return false;
                    }
                    var oldResult = getResolutionWithResolvedFileName(oldResolution);
                    var newResult = getResolutionWithResolvedFileName(newResolution);
                    if (oldResult === newResult) {
                        return true;
                    }
                    if (!oldResult || !newResult) {
                        return false;
                    }
                    return oldResult.resolvedFileName === newResult.resolvedFileName;
                }
            }
            function resolveTypeReferenceDirectives(typeDirectiveNames, containingFile, redirectedReference) {
                return resolveNamesWithLocalCache(typeDirectiveNames, containingFile, redirectedReference, resolvedTypeReferenceDirectives, perDirectoryResolvedTypeReferenceDirectives, ts.resolveTypeReferenceDirective, getResolvedTypeReferenceDirective, 
                /*shouldRetryResolution*/ function (resolution) { return resolution.resolvedTypeReferenceDirective === undefined; }, 
                /*reusedNames*/ undefined, /*logChanges*/ false);
            }
            function resolveModuleNames(moduleNames, containingFile, reusedNames, redirectedReference) {
                return resolveNamesWithLocalCache(moduleNames, containingFile, redirectedReference, resolvedModuleNames, perDirectoryResolvedModuleNames, resolveModuleName, getResolvedModule, 
                /*shouldRetryResolution*/ function (resolution) { return !resolution.resolvedModule || !ts.resolutionExtensionIsTSOrJson(resolution.resolvedModule.extension); }, reusedNames, logChangesWhenResolvingModule);
            }
            function getResolvedModuleWithFailedLookupLocationsFromCache(moduleName, containingFile) {
                var cache = resolvedModuleNames.get(resolutionHost.toPath(containingFile));
                return cache && cache.get(moduleName);
            }
            function isNodeModulesDirectory(dirPath) {
                return ts.endsWith(dirPath, "/node_modules");
            }
            function isNodeModulesAtTypesDirectory(dirPath) {
                return ts.endsWith(dirPath, "/node_modules/@types");
            }
            function getDirectoryToWatchFailedLookupLocation(failedLookupLocation, failedLookupLocationPath) {
                if (isInDirectoryPath(rootPath, failedLookupLocationPath)) {
                    // Ensure failed look up is normalized path
                    failedLookupLocation = ts.isRootedDiskPath(failedLookupLocation) ? ts.normalizePath(failedLookupLocation) : ts.getNormalizedAbsolutePath(failedLookupLocation, getCurrentDirectory());
                    var failedLookupPathSplit = failedLookupLocationPath.split(ts.directorySeparator);
                    var failedLookupSplit = failedLookupLocation.split(ts.directorySeparator);
                    ts.Debug.assert(failedLookupSplit.length === failedLookupPathSplit.length, "FailedLookup: " + failedLookupLocation + " failedLookupLocationPath: " + failedLookupLocationPath);
                    if (failedLookupPathSplit.length > rootSplitLength + 1) {
                        // Instead of watching root, watch directory in root to avoid watching excluded directories not needed for module resolution
                        return {
                            dir: failedLookupSplit.slice(0, rootSplitLength + 1).join(ts.directorySeparator),
                            dirPath: failedLookupPathSplit.slice(0, rootSplitLength + 1).join(ts.directorySeparator)
                        };
                    }
                    else {
                        // Always watch root directory non recursively
                        return {
                            dir: rootDir,
                            dirPath: rootPath,
                            nonRecursive: false
                        };
                    }
                }
                return getDirectoryToWatchFromFailedLookupLocationDirectory(ts.getDirectoryPath(ts.getNormalizedAbsolutePath(failedLookupLocation, getCurrentDirectory())), ts.getDirectoryPath(failedLookupLocationPath));
            }
            function getDirectoryToWatchFromFailedLookupLocationDirectory(dir, dirPath) {
                // If directory path contains node module, get the most parent node_modules directory for watching
                while (ts.pathContainsNodeModules(dirPath)) {
                    dir = ts.getDirectoryPath(dir);
                    dirPath = ts.getDirectoryPath(dirPath);
                }
                // If the directory is node_modules use it to watch, always watch it recursively
                if (isNodeModulesDirectory(dirPath)) {
                    return canWatchDirectory(ts.getDirectoryPath(dirPath)) ? { dir: dir, dirPath: dirPath } : undefined;
                }
                var nonRecursive = true;
                // Use some ancestor of the root directory
                var subDirectoryPath, subDirectory;
                if (rootPath !== undefined) {
                    while (!isInDirectoryPath(dirPath, rootPath)) {
                        var parentPath = ts.getDirectoryPath(dirPath);
                        if (parentPath === dirPath) {
                            break;
                        }
                        nonRecursive = false;
                        subDirectoryPath = dirPath;
                        subDirectory = dir;
                        dirPath = parentPath;
                        dir = ts.getDirectoryPath(dir);
                    }
                }
                return canWatchDirectory(dirPath) ? { dir: subDirectory || dir, dirPath: subDirectoryPath || dirPath, nonRecursive: nonRecursive } : undefined;
            }
            function isPathWithDefaultFailedLookupExtension(path) {
                return ts.fileExtensionIsOneOf(path, failedLookupDefaultExtensions);
            }
            function watchFailedLookupLocationsOfExternalModuleResolutions(name, resolution) {
                // No need to set the resolution refCount
                if (resolution.failedLookupLocations && resolution.failedLookupLocations.length) {
                    if (resolution.refCount) {
                        resolution.refCount++;
                    }
                    else {
                        resolution.refCount = 1;
                        if (ts.isExternalModuleNameRelative(name)) {
                            watchFailedLookupLocationOfResolution(resolution);
                        }
                        else {
                            nonRelativeExternalModuleResolutions.add(name, resolution);
                        }
                    }
                }
            }
            function watchFailedLookupLocationOfResolution(resolution) {
                ts.Debug.assert(!!resolution.refCount);
                var failedLookupLocations = resolution.failedLookupLocations;
                var setAtRoot = false;
                for (var _i = 0, failedLookupLocations_1 = failedLookupLocations; _i < failedLookupLocations_1.length; _i++) {
                    var failedLookupLocation = failedLookupLocations_1[_i];
                    var failedLookupLocationPath = resolutionHost.toPath(failedLookupLocation);
                    var toWatch = getDirectoryToWatchFailedLookupLocation(failedLookupLocation, failedLookupLocationPath);
                    if (toWatch) {
                        var dir = toWatch.dir, dirPath = toWatch.dirPath, nonRecursive = toWatch.nonRecursive;
                        // If the failed lookup location path is not one of the supported extensions,
                        // store it in the custom path
                        if (!isPathWithDefaultFailedLookupExtension(failedLookupLocationPath)) {
                            var refCount = customFailedLookupPaths.get(failedLookupLocationPath) || 0;
                            customFailedLookupPaths.set(failedLookupLocationPath, refCount + 1);
                        }
                        if (dirPath === rootPath) {
                            ts.Debug.assert(!nonRecursive);
                            setAtRoot = true;
                        }
                        else {
                            setDirectoryWatcher(dir, dirPath, nonRecursive);
                        }
                    }
                }
                if (setAtRoot) {
                    // This is always non recursive
                    setDirectoryWatcher(rootDir, rootPath, /*nonRecursive*/ true); // TODO: GH#18217
                }
            }
            function setRefCountToUndefined(resolution) {
                resolution.refCount = undefined;
            }
            function watchFailedLookupLocationOfNonRelativeModuleResolutions(resolutions, name) {
                var program = resolutionHost.getCurrentProgram();
                var updateResolution = program && program.getTypeChecker().tryFindAmbientModuleWithoutAugmentations(name) ?
                    setRefCountToUndefined : watchFailedLookupLocationOfResolution;
                resolutions.forEach(updateResolution);
            }
            function setDirectoryWatcher(dir, dirPath, nonRecursive) {
                var dirWatcher = directoryWatchesOfFailedLookups.get(dirPath);
                if (dirWatcher) {
                    ts.Debug.assert(!!nonRecursive === !!dirWatcher.nonRecursive);
                    dirWatcher.refCount++;
                }
                else {
                    directoryWatchesOfFailedLookups.set(dirPath, { watcher: createDirectoryWatcher(dir, dirPath, nonRecursive), refCount: 1, nonRecursive: nonRecursive });
                }
            }
            function stopWatchFailedLookupLocationOfResolution(resolution) {
                if (!resolution.refCount) {
                    return;
                }
                resolution.refCount--;
                if (resolution.refCount) {
                    return;
                }
                var failedLookupLocations = resolution.failedLookupLocations;
                var removeAtRoot = false;
                for (var _i = 0, failedLookupLocations_2 = failedLookupLocations; _i < failedLookupLocations_2.length; _i++) {
                    var failedLookupLocation = failedLookupLocations_2[_i];
                    var failedLookupLocationPath = resolutionHost.toPath(failedLookupLocation);
                    var toWatch = getDirectoryToWatchFailedLookupLocation(failedLookupLocation, failedLookupLocationPath);
                    if (toWatch) {
                        var dirPath = toWatch.dirPath;
                        var refCount = customFailedLookupPaths.get(failedLookupLocationPath);
                        if (refCount) {
                            if (refCount === 1) {
                                customFailedLookupPaths.delete(failedLookupLocationPath);
                            }
                            else {
                                ts.Debug.assert(refCount > 1);
                                customFailedLookupPaths.set(failedLookupLocationPath, refCount - 1);
                            }
                        }
                        if (dirPath === rootPath) {
                            removeAtRoot = true;
                        }
                        else {
                            removeDirectoryWatcher(dirPath);
                        }
                    }
                }
                if (removeAtRoot) {
                    removeDirectoryWatcher(rootPath);
                }
            }
            function removeDirectoryWatcher(dirPath) {
                var dirWatcher = directoryWatchesOfFailedLookups.get(dirPath);
                // Do not close the watcher yet since it might be needed by other failed lookup locations.
                dirWatcher.refCount--;
            }
            function createDirectoryWatcher(directory, dirPath, nonRecursive) {
                return resolutionHost.watchDirectoryOfFailedLookupLocation(directory, function (fileOrDirectory) {
                    var fileOrDirectoryPath = resolutionHost.toPath(fileOrDirectory);
                    if (cachedDirectoryStructureHost) {
                        // Since the file existence changed, update the sourceFiles cache
                        cachedDirectoryStructureHost.addOrDeleteFileOrDirectory(fileOrDirectory, fileOrDirectoryPath);
                    }
                    if (!allFilesHaveInvalidatedResolution && invalidateResolutionOfFailedLookupLocation(fileOrDirectoryPath, dirPath === fileOrDirectoryPath)) {
                        resolutionHost.onInvalidatedResolution();
                    }
                }, nonRecursive ? 0 /* None */ : 1 /* Recursive */);
            }
            function removeResolutionsOfFileFromCache(cache, filePath) {
                // Deleted file, stop watching failed lookups for all the resolutions in the file
                var resolutions = cache.get(filePath);
                if (resolutions) {
                    resolutions.forEach(stopWatchFailedLookupLocationOfResolution);
                    cache.delete(filePath);
                }
            }
            function removeResolutionsFromProjectReferenceRedirects(filePath) {
                if (!ts.fileExtensionIs(filePath, ".json" /* Json */)) {
                    return;
                }
                var program = resolutionHost.getCurrentProgram();
                if (!program) {
                    return;
                }
                // If this file is input file for the referenced project, get it
                var resolvedProjectReference = program.getResolvedProjectReferenceByPath(filePath);
                if (!resolvedProjectReference) {
                    return;
                }
                // filePath is for the projectReference and the containing file is from this project reference, invalidate the resolution
                resolvedProjectReference.commandLine.fileNames.forEach(function (f) { return removeResolutionsOfFile(resolutionHost.toPath(f)); });
            }
            function removeResolutionsOfFile(filePath) {
                removeResolutionsOfFileFromCache(resolvedModuleNames, filePath);
                removeResolutionsOfFileFromCache(resolvedTypeReferenceDirectives, filePath);
            }
            function invalidateResolutionCache(cache, isInvalidatedResolution, getResolutionWithResolvedFileName) {
                var seen = ts.createMap();
                cache.forEach(function (resolutions, containingFilePath) {
                    var dirPath = ts.getDirectoryPath(containingFilePath);
                    var seenInDir = seen.get(dirPath);
                    if (!seenInDir) {
                        seenInDir = ts.createMap();
                        seen.set(dirPath, seenInDir);
                    }
                    resolutions.forEach(function (resolution, name) {
                        if (seenInDir.has(name)) {
                            return;
                        }
                        seenInDir.set(name, true);
                        if (!resolution.isInvalidated && isInvalidatedResolution(resolution, getResolutionWithResolvedFileName)) {
                            // Mark the file as needing re-evaluation of module resolution instead of using it blindly.
                            resolution.isInvalidated = true;
                            (filesWithInvalidatedResolutions || (filesWithInvalidatedResolutions = ts.createMap())).set(containingFilePath, true);
                            // When its a file with inferred types resolution, invalidate type reference directive resolution
                            if (containingFilePath.endsWith(ts.inferredTypesContainingFile)) {
                                resolutionHost.onChangedAutomaticTypeDirectiveNames();
                            }
                        }
                    });
                });
            }
            function hasReachedResolutionIterationLimit() {
                var maxSize = resolutionHost.maxNumberOfFilesToIterateForInvalidation || ts.maxNumberOfFilesToIterateForInvalidation;
                return resolvedModuleNames.size > maxSize || resolvedTypeReferenceDirectives.size > maxSize;
            }
            function invalidateResolutions(isInvalidatedResolution) {
                // If more than maxNumberOfFilesToIterateForInvalidation present,
                // just invalidated all files and recalculate the resolutions for files instead
                if (hasReachedResolutionIterationLimit()) {
                    allFilesHaveInvalidatedResolution = true;
                    return;
                }
                invalidateResolutionCache(resolvedModuleNames, isInvalidatedResolution, getResolvedModule);
                invalidateResolutionCache(resolvedTypeReferenceDirectives, isInvalidatedResolution, getResolvedTypeReferenceDirective);
            }
            function invalidateResolutionOfFile(filePath) {
                removeResolutionsOfFile(filePath);
                invalidateResolutions(
                // Resolution is invalidated if the resulting file name is same as the deleted file path
                function (resolution, getResolutionWithResolvedFileName) {
                    var result = getResolutionWithResolvedFileName(resolution);
                    return !!result && resolutionHost.toPath(result.resolvedFileName) === filePath; // TODO: GH#18217
                });
            }
            function setFilesWithInvalidatedNonRelativeUnresolvedImports(filesMap) {
                ts.Debug.assert(filesWithInvalidatedNonRelativeUnresolvedImports === filesMap || filesWithInvalidatedNonRelativeUnresolvedImports === undefined);
                filesWithInvalidatedNonRelativeUnresolvedImports = filesMap;
            }
            function invalidateResolutionOfFailedLookupLocation(fileOrDirectoryPath, isCreatingWatchedDirectory) {
                var isChangedFailedLookupLocation;
                if (isCreatingWatchedDirectory) {
                    // Watching directory is created
                    // Invalidate any resolution has failed lookup in this directory
                    isChangedFailedLookupLocation = function (location) { return isInDirectoryPath(fileOrDirectoryPath, resolutionHost.toPath(location)); };
                }
                else {
                    // If something to do with folder/file starting with "." in node_modules folder, skip it
                    var updatedPath = removeIgnoredPath(fileOrDirectoryPath);
                    if (!updatedPath)
                        return false;
                    fileOrDirectoryPath = updatedPath;
                    // prevent saving an open file from over-eagerly triggering invalidation
                    if (resolutionHost.fileIsOpen(fileOrDirectoryPath)) {
                        return false;
                    }
                    // Some file or directory in the watching directory is created
                    // Return early if it does not have any of the watching extension or not the custom failed lookup path
                    var dirOfFileOrDirectory = ts.getDirectoryPath(fileOrDirectoryPath);
                    if (isNodeModulesAtTypesDirectory(fileOrDirectoryPath) || isNodeModulesDirectory(fileOrDirectoryPath) ||
                        isNodeModulesAtTypesDirectory(dirOfFileOrDirectory) || isNodeModulesDirectory(dirOfFileOrDirectory)) {
                        // Invalidate any resolution from this directory
                        isChangedFailedLookupLocation = function (location) {
                            var locationPath = resolutionHost.toPath(location);
                            return locationPath === fileOrDirectoryPath || ts.startsWith(resolutionHost.toPath(location), fileOrDirectoryPath);
                        };
                    }
                    else {
                        if (!isPathWithDefaultFailedLookupExtension(fileOrDirectoryPath) && !customFailedLookupPaths.has(fileOrDirectoryPath)) {
                            return false;
                        }
                        // Ignore emits from the program
                        if (ts.isEmittedFileOfProgram(resolutionHost.getCurrentProgram(), fileOrDirectoryPath)) {
                            return false;
                        }
                        // Resolution need to be invalidated if failed lookup location is same as the file or directory getting created
                        isChangedFailedLookupLocation = function (location) { return resolutionHost.toPath(location) === fileOrDirectoryPath; };
                    }
                }
                var hasChangedFailedLookupLocation = function (resolution) { return ts.some(resolution.failedLookupLocations, isChangedFailedLookupLocation); };
                var invalidatedFilesCount = filesWithInvalidatedResolutions && filesWithInvalidatedResolutions.size;
                invalidateResolutions(
                // Resolution is invalidated if the resulting file name is same as the deleted file path
                hasChangedFailedLookupLocation);
                return allFilesHaveInvalidatedResolution || filesWithInvalidatedResolutions && filesWithInvalidatedResolutions.size !== invalidatedFilesCount;
            }
            function closeTypeRootsWatch() {
                ts.clearMap(typeRootsWatches, ts.closeFileWatcher);
            }
            function getDirectoryToWatchFailedLookupLocationFromTypeRoot(typeRoot, typeRootPath) {
                if (allFilesHaveInvalidatedResolution) {
                    return undefined;
                }
                if (isInDirectoryPath(rootPath, typeRootPath)) {
                    return rootPath;
                }
                var toWatch = getDirectoryToWatchFromFailedLookupLocationDirectory(typeRoot, typeRootPath);
                return toWatch && directoryWatchesOfFailedLookups.has(toWatch.dirPath) ? toWatch.dirPath : undefined;
            }
            function createTypeRootsWatch(typeRootPath, typeRoot) {
                // Create new watch and recursive info
                return resolutionHost.watchTypeRootsDirectory(typeRoot, function (fileOrDirectory) {
                    var fileOrDirectoryPath = resolutionHost.toPath(fileOrDirectory);
                    if (cachedDirectoryStructureHost) {
                        // Since the file existence changed, update the sourceFiles cache
                        cachedDirectoryStructureHost.addOrDeleteFileOrDirectory(fileOrDirectory, fileOrDirectoryPath);
                    }
                    // For now just recompile
                    // We could potentially store more data here about whether it was/would be really be used or not
                    // and with that determine to trigger compilation but for now this is enough
                    resolutionHost.onChangedAutomaticTypeDirectiveNames();
                    // Since directory watchers invoked are flaky, the failed lookup location events might not be triggered
                    // So handle to failed lookup locations here as well to ensure we are invalidating resolutions
                    var dirPath = getDirectoryToWatchFailedLookupLocationFromTypeRoot(typeRoot, typeRootPath);
                    if (dirPath && invalidateResolutionOfFailedLookupLocation(fileOrDirectoryPath, dirPath === fileOrDirectoryPath)) {
                        resolutionHost.onInvalidatedResolution();
                    }
                }, 1 /* Recursive */);
            }
            /**
             * Watches the types that would get added as part of getAutomaticTypeDirectiveNames
             * To be called when compiler options change
             */
            function updateTypeRootsWatch() {
                var options = resolutionHost.getCompilationSettings();
                if (options.types) {
                    // No need to do any watch since resolution cache is going to handle the failed lookups
                    // for the types added by this
                    closeTypeRootsWatch();
                    return;
                }
                // we need to assume the directories exist to ensure that we can get all the type root directories that get included
                // But filter directories that are at root level to say directory doesnt exist, so that we arent watching them
                var typeRoots = ts.getEffectiveTypeRoots(options, { directoryExists: directoryExistsForTypeRootWatch, getCurrentDirectory: getCurrentDirectory });
                if (typeRoots) {
                    ts.mutateMap(typeRootsWatches, ts.arrayToMap(typeRoots, function (tr) { return resolutionHost.toPath(tr); }), {
                        createNewValue: createTypeRootsWatch,
                        onDeleteValue: ts.closeFileWatcher
                    });
                }
                else {
                    closeTypeRootsWatch();
                }
            }
            /**
             * Use this function to return if directory exists to get type roots to watch
             * If we return directory exists then only the paths will be added to type roots
             * Hence return true for all directories except root directories which are filtered from watching
             */
            function directoryExistsForTypeRootWatch(nodeTypesDirectory) {
                var dir = ts.getDirectoryPath(ts.getDirectoryPath(nodeTypesDirectory));
                var dirPath = resolutionHost.toPath(dir);
                return dirPath === rootPath || canWatchDirectory(dirPath);
            }
        }
        ts.createResolutionCache = createResolutionCache;
    })(ts || (ts = {}));
    // Used by importFixes, getEditsForFileRename, and declaration emit to synthesize import module specifiers.
    /* @internal */
    var ts;
    (function (ts) {
        var moduleSpecifiers;
        (function (moduleSpecifiers) {
            var RelativePreference;
            (function (RelativePreference) {
                RelativePreference[RelativePreference["Relative"] = 0] = "Relative";
                RelativePreference[RelativePreference["NonRelative"] = 1] = "NonRelative";
                RelativePreference[RelativePreference["Auto"] = 2] = "Auto";
            })(RelativePreference || (RelativePreference = {}));
            // See UserPreferences#importPathEnding
            var Ending;
            (function (Ending) {
                Ending[Ending["Minimal"] = 0] = "Minimal";
                Ending[Ending["Index"] = 1] = "Index";
                Ending[Ending["JsExtension"] = 2] = "JsExtension";
            })(Ending || (Ending = {}));
            function getPreferences(_a, compilerOptions, importingSourceFile) {
                var importModuleSpecifierPreference = _a.importModuleSpecifierPreference, importModuleSpecifierEnding = _a.importModuleSpecifierEnding;
                return {
                    relativePreference: importModuleSpecifierPreference === "relative" ? 0 /* Relative */ : importModuleSpecifierPreference === "non-relative" ? 1 /* NonRelative */ : 2 /* Auto */,
                    ending: getEnding(),
                };
                function getEnding() {
                    switch (importModuleSpecifierEnding) {
                        case "minimal": return 0 /* Minimal */;
                        case "index": return 1 /* Index */;
                        case "js": return 2 /* JsExtension */;
                        default: return usesJsExtensionOnImports(importingSourceFile) ? 2 /* JsExtension */
                            : ts.getEmitModuleResolutionKind(compilerOptions) !== ts.ModuleResolutionKind.NodeJs ? 1 /* Index */ : 0 /* Minimal */;
                    }
                }
            }
            function getPreferencesForUpdate(compilerOptions, oldImportSpecifier) {
                return {
                    relativePreference: ts.isExternalModuleNameRelative(oldImportSpecifier) ? 0 /* Relative */ : 1 /* NonRelative */,
                    ending: ts.hasJSFileExtension(oldImportSpecifier) ?
                        2 /* JsExtension */ :
                        ts.getEmitModuleResolutionKind(compilerOptions) !== ts.ModuleResolutionKind.NodeJs || ts.endsWith(oldImportSpecifier, "index") ? 1 /* Index */ : 0 /* Minimal */,
                };
            }
            function updateModuleSpecifier(compilerOptions, importingSourceFileName, toFileName, host, files, redirectTargetsMap, oldImportSpecifier) {
                var res = getModuleSpecifierWorker(compilerOptions, importingSourceFileName, toFileName, host, files, redirectTargetsMap, getPreferencesForUpdate(compilerOptions, oldImportSpecifier));
                if (res === oldImportSpecifier)
                    return undefined;
                return res;
            }
            moduleSpecifiers.updateModuleSpecifier = updateModuleSpecifier;
            // Note: importingSourceFile is just for usesJsExtensionOnImports
            function getModuleSpecifier(compilerOptions, importingSourceFile, importingSourceFileName, toFileName, host, files, preferences, redirectTargetsMap) {
                if (preferences === void 0) {
                    preferences = {};
                }
                return getModuleSpecifierWorker(compilerOptions, importingSourceFileName, toFileName, host, files, redirectTargetsMap, getPreferences(preferences, compilerOptions, importingSourceFile));
            }
            moduleSpecifiers.getModuleSpecifier = getModuleSpecifier;
            function getNodeModulesPackageName(compilerOptions, importingSourceFileName, nodeModulesFileName, host, files, redirectTargetsMap) {
                var info = getInfo(importingSourceFileName, host);
                var modulePaths = getAllModulePaths(files, importingSourceFileName, nodeModulesFileName, info.getCanonicalFileName, host, redirectTargetsMap);
                return ts.firstDefined(modulePaths, function (moduleFileName) { return tryGetModuleNameAsNodeModule(moduleFileName, info, host, compilerOptions, /*packageNameOnly*/ true); });
            }
            moduleSpecifiers.getNodeModulesPackageName = getNodeModulesPackageName;
            function getModuleSpecifierWorker(compilerOptions, importingSourceFileName, toFileName, host, files, redirectTargetsMap, preferences) {
                var info = getInfo(importingSourceFileName, host);
                var modulePaths = getAllModulePaths(files, importingSourceFileName, toFileName, info.getCanonicalFileName, host, redirectTargetsMap);
                return ts.firstDefined(modulePaths, function (moduleFileName) { return tryGetModuleNameAsNodeModule(moduleFileName, info, host, compilerOptions); }) ||
                    getLocalModuleSpecifier(toFileName, info, compilerOptions, preferences);
            }
            /** Returns an import for each symlink and for the realpath. */
            function getModuleSpecifiers(moduleSymbol, compilerOptions, importingSourceFile, host, files, userPreferences, redirectTargetsMap) {
                var ambient = tryGetModuleNameFromAmbientModule(moduleSymbol);
                if (ambient)
                    return [ambient];
                var info = getInfo(importingSourceFile.path, host);
                var moduleSourceFile = ts.getSourceFileOfNode(moduleSymbol.valueDeclaration || ts.getNonAugmentationDeclaration(moduleSymbol));
                var modulePaths = getAllModulePaths(files, importingSourceFile.path, moduleSourceFile.originalFileName, info.getCanonicalFileName, host, redirectTargetsMap);
                var preferences = getPreferences(userPreferences, compilerOptions, importingSourceFile);
                var global = ts.mapDefined(modulePaths, function (moduleFileName) { return tryGetModuleNameAsNodeModule(moduleFileName, info, host, compilerOptions); });
                return global.length ? global : modulePaths.map(function (moduleFileName) { return getLocalModuleSpecifier(moduleFileName, info, compilerOptions, preferences); });
            }
            moduleSpecifiers.getModuleSpecifiers = getModuleSpecifiers;
            // importingSourceFileName is separate because getEditsForFileRename may need to specify an updated path
            function getInfo(importingSourceFileName, host) {
                var getCanonicalFileName = ts.createGetCanonicalFileName(host.useCaseSensitiveFileNames ? host.useCaseSensitiveFileNames() : true);
                var sourceDirectory = ts.getDirectoryPath(importingSourceFileName);
                return { getCanonicalFileName: getCanonicalFileName, sourceDirectory: sourceDirectory };
            }
            function getLocalModuleSpecifier(moduleFileName, _a, compilerOptions, _b) {
                var getCanonicalFileName = _a.getCanonicalFileName, sourceDirectory = _a.sourceDirectory;
                var ending = _b.ending, relativePreference = _b.relativePreference;
                var baseUrl = compilerOptions.baseUrl, paths = compilerOptions.paths, rootDirs = compilerOptions.rootDirs;
                var relativePath = rootDirs && tryGetModuleNameFromRootDirs(rootDirs, moduleFileName, sourceDirectory, getCanonicalFileName, ending, compilerOptions) ||
                    removeExtensionAndIndexPostFix(ts.ensurePathIsNonModuleName(ts.getRelativePathFromDirectory(sourceDirectory, moduleFileName, getCanonicalFileName)), ending, compilerOptions);
                if (!baseUrl || relativePreference === 0 /* Relative */) {
                    return relativePath;
                }
                var relativeToBaseUrl = getRelativePathIfInDirectory(moduleFileName, baseUrl, getCanonicalFileName);
                if (!relativeToBaseUrl) {
                    return relativePath;
                }
                var importRelativeToBaseUrl = removeExtensionAndIndexPostFix(relativeToBaseUrl, ending, compilerOptions);
                var fromPaths = paths && tryGetModuleNameFromPaths(ts.removeFileExtension(relativeToBaseUrl), importRelativeToBaseUrl, paths);
                var nonRelative = fromPaths === undefined ? importRelativeToBaseUrl : fromPaths;
                if (relativePreference === 1 /* NonRelative */) {
                    return nonRelative;
                }
                if (relativePreference !== 2 /* Auto */)
                    ts.Debug.assertNever(relativePreference);
                // Prefer a relative import over a baseUrl import if it has fewer components.
                return isPathRelativeToParent(nonRelative) || countPathComponents(relativePath) < countPathComponents(nonRelative) ? relativePath : nonRelative;
            }
            function countPathComponents(path) {
                var count = 0;
                for (var i = ts.startsWith(path, "./") ? 2 : 0; i < path.length; i++) {
                    if (path.charCodeAt(i) === 47 /* slash */)
                        count++;
                }
                return count;
            }
            moduleSpecifiers.countPathComponents = countPathComponents;
            function usesJsExtensionOnImports(_a) {
                var imports = _a.imports;
                return ts.firstDefined(imports, function (_a) {
                    var text = _a.text;
                    return ts.pathIsRelative(text) ? ts.hasJSFileExtension(text) : undefined;
                }) || false;
            }
            function numberOfDirectorySeparators(str) {
                var match = str.match(/\//g);
                return match ? match.length : 0;
            }
            function comparePathsByNumberOfDirectorySeparators(a, b) {
                return ts.compareValues(numberOfDirectorySeparators(a), numberOfDirectorySeparators(b));
            }
            /**
             * Looks for existing imports that use symlinks to this module.
             * Symlinks will be returned first so they are preferred over the real path.
             */
            function getAllModulePaths(files, importingFileName, importedFileName, getCanonicalFileName, host, redirectTargetsMap) {
                var redirects = redirectTargetsMap.get(importedFileName);
                var importedFileNames = redirects ? __spreadArrays(redirects, [importedFileName]) : [importedFileName];
                var cwd = host.getCurrentDirectory ? host.getCurrentDirectory() : "";
                var targets = importedFileNames.map(function (f) { return ts.getNormalizedAbsolutePath(f, cwd); });
                var links = host.getProbableSymlinks
                    ? host.getProbableSymlinks(files)
                    : ts.discoverProbableSymlinks(files, getCanonicalFileName, cwd);
                var result = [];
                var compareStrings = (!host.useCaseSensitiveFileNames || host.useCaseSensitiveFileNames()) ? ts.compareStringsCaseSensitive : ts.compareStringsCaseInsensitive;
                links.forEach(function (resolved, path) {
                    if (ts.startsWithDirectory(importingFileName, resolved, getCanonicalFileName)) {
                        return; // Don't want to a package to globally import from itself
                    }
                    var target = ts.find(targets, function (t) { return compareStrings(t.slice(0, resolved.length + 1), resolved + "/") === 0 /* EqualTo */; });
                    if (target === undefined)
                        return;
                    var relative = ts.getRelativePathFromDirectory(resolved, target, getCanonicalFileName);
                    var option = ts.resolvePath(path, relative);
                    if (!host.fileExists || host.fileExists(option)) {
                        result.push(option);
                    }
                });
                result.push.apply(result, targets);
                if (result.length < 2)
                    return result;
                // Sort by paths closest to importing file Name directory
                var allFileNames = ts.arrayToMap(result, ts.identity, getCanonicalFileName);
                var sortedPaths = [];
                var _loop_18 = function (directory) {
                    var directoryStart = ts.ensureTrailingDirectorySeparator(directory);
                    var pathsInDirectory;
                    allFileNames.forEach(function (canonicalFileName, fileName) {
                        if (ts.startsWith(canonicalFileName, directoryStart)) {
                            (pathsInDirectory || (pathsInDirectory = [])).push(fileName);
                            allFileNames.delete(fileName);
                        }
                    });
                    if (pathsInDirectory) {
                        if (pathsInDirectory.length > 1) {
                            pathsInDirectory.sort(comparePathsByNumberOfDirectorySeparators);
                        }
                        sortedPaths.push.apply(sortedPaths, pathsInDirectory);
                    }
                    var newDirectory = ts.getDirectoryPath(directory);
                    if (newDirectory === directory)
                        return out_directory_1 = directory, "break";
                    directory = newDirectory;
                    out_directory_1 = directory;
                };
                var out_directory_1;
                for (var directory = ts.getDirectoryPath(ts.toPath(importingFileName, cwd, getCanonicalFileName)); allFileNames.size !== 0;) {
                    var state_7 = _loop_18(directory);
                    directory = out_directory_1;
                    if (state_7 === "break")
                        break;
                }
                if (allFileNames.size) {
                    var remainingPaths = ts.arrayFrom(allFileNames.values());
                    if (remainingPaths.length > 1)
                        remainingPaths.sort(comparePathsByNumberOfDirectorySeparators);
                    sortedPaths.push.apply(sortedPaths, remainingPaths);
                }
                return sortedPaths;
            }
            function tryGetModuleNameFromAmbientModule(moduleSymbol) {
                var decl = ts.find(moduleSymbol.declarations, function (d) { return ts.isNonGlobalAmbientModule(d) && (!ts.isExternalModuleAugmentation(d) || !ts.isExternalModuleNameRelative(ts.getTextOfIdentifierOrLiteral(d.name))); });
                if (decl) {
                    return decl.name.text;
                }
            }
            function tryGetModuleNameFromPaths(relativeToBaseUrlWithIndex, relativeToBaseUrl, paths) {
                for (var key in paths) {
                    for (var _i = 0, _a = paths[key]; _i < _a.length; _i++) {
                        var patternText_1 = _a[_i];
                        var pattern = ts.removeFileExtension(ts.normalizePath(patternText_1));
                        var indexOfStar = pattern.indexOf("*");
                        if (indexOfStar !== -1) {
                            var prefix = pattern.substr(0, indexOfStar);
                            var suffix = pattern.substr(indexOfStar + 1);
                            if (relativeToBaseUrl.length >= prefix.length + suffix.length &&
                                ts.startsWith(relativeToBaseUrl, prefix) &&
                                ts.endsWith(relativeToBaseUrl, suffix) ||
                                !suffix && relativeToBaseUrl === ts.removeTrailingDirectorySeparator(prefix)) {
                                var matchedStar = relativeToBaseUrl.substr(prefix.length, relativeToBaseUrl.length - suffix.length);
                                return key.replace("*", matchedStar);
                            }
                        }
                        else if (pattern === relativeToBaseUrl || pattern === relativeToBaseUrlWithIndex) {
                            return key;
                        }
                    }
                }
            }
            function tryGetModuleNameFromRootDirs(rootDirs, moduleFileName, sourceDirectory, getCanonicalFileName, ending, compilerOptions) {
                var normalizedTargetPath = getPathRelativeToRootDirs(moduleFileName, rootDirs, getCanonicalFileName);
                if (normalizedTargetPath === undefined) {
                    return undefined;
                }
                var normalizedSourcePath = getPathRelativeToRootDirs(sourceDirectory, rootDirs, getCanonicalFileName);
                var relativePath = normalizedSourcePath !== undefined ? ts.ensurePathIsNonModuleName(ts.getRelativePathFromDirectory(normalizedSourcePath, normalizedTargetPath, getCanonicalFileName)) : normalizedTargetPath;
                return ts.getEmitModuleResolutionKind(compilerOptions) === ts.ModuleResolutionKind.NodeJs
                    ? removeExtensionAndIndexPostFix(relativePath, ending, compilerOptions)
                    : ts.removeFileExtension(relativePath);
            }
            function tryGetModuleNameAsNodeModule(moduleFileName, _a, host, options, packageNameOnly) {
                var getCanonicalFileName = _a.getCanonicalFileName, sourceDirectory = _a.sourceDirectory;
                if (!host.fileExists || !host.readFile) {
                    return undefined;
                }
                var parts = getNodeModulePathParts(moduleFileName);
                if (!parts) {
                    return undefined;
                }
                var packageJsonContent;
                var packageRootPath = moduleFileName.substring(0, parts.packageRootIndex);
                if (!packageNameOnly) {
                    var packageJsonPath = ts.combinePaths(packageRootPath, "package.json");
                    packageJsonContent = host.fileExists(packageJsonPath)
                        ? JSON.parse(host.readFile(packageJsonPath))
                        : undefined;
                    var versionPaths = packageJsonContent && packageJsonContent.typesVersions
                        ? ts.getPackageJsonTypesVersionsPaths(packageJsonContent.typesVersions)
                        : undefined;
                    if (versionPaths) {
                        var subModuleName = moduleFileName.slice(parts.packageRootIndex + 1);
                        var fromPaths = tryGetModuleNameFromPaths(ts.removeFileExtension(subModuleName), removeExtensionAndIndexPostFix(subModuleName, 0 /* Minimal */, options), versionPaths.paths);
                        if (fromPaths !== undefined) {
                            moduleFileName = ts.combinePaths(moduleFileName.slice(0, parts.packageRootIndex), fromPaths);
                        }
                    }
                }
                // Simplify the full file path to something that can be resolved by Node.
                // If the module could be imported by a directory name, use that directory's name
                var moduleSpecifier = packageNameOnly ? moduleFileName : getDirectoryOrExtensionlessFileName(moduleFileName);
                var globalTypingsCacheLocation = host.getGlobalTypingsCacheLocation && host.getGlobalTypingsCacheLocation();
                // Get a path that's relative to node_modules or the importing file's path
                // if node_modules folder is in this folder or any of its parent folders, no need to keep it.
                var pathToTopLevelNodeModules = getCanonicalFileName(moduleSpecifier.substring(0, parts.topLevelNodeModulesIndex));
                if (!(ts.startsWith(sourceDirectory, pathToTopLevelNodeModules) || globalTypingsCacheLocation && ts.startsWith(getCanonicalFileName(globalTypingsCacheLocation), pathToTopLevelNodeModules))) {
                    return undefined;
                }
                // If the module was found in @types, get the actual Node package name
                var nodeModulesDirectoryName = moduleSpecifier.substring(parts.topLevelPackageNameIndex + 1);
                var packageName = ts.getPackageNameFromTypesPackageName(nodeModulesDirectoryName);
                // For classic resolution, only allow importing from node_modules/@types, not other node_modules
                return ts.getEmitModuleResolutionKind(options) !== ts.ModuleResolutionKind.NodeJs && packageName === nodeModulesDirectoryName ? undefined : packageName;
                function getDirectoryOrExtensionlessFileName(path) {
                    // If the file is the main module, it can be imported by the package name
                    if (packageJsonContent) {
                        var mainFileRelative = packageJsonContent.typings || packageJsonContent.types || packageJsonContent.main;
                        if (ts.isString(mainFileRelative)) {
                            var mainExportFile = ts.toPath(mainFileRelative, packageRootPath, getCanonicalFileName);
                            if (ts.removeFileExtension(mainExportFile) === ts.removeFileExtension(getCanonicalFileName(path))) {
                                return packageRootPath;
                            }
                        }
                    }
                    // We still have a file name - remove the extension
                    var fullModulePathWithoutExtension = ts.removeFileExtension(path);
                    // If the file is /index, it can be imported by its directory name
                    // IFF there is not _also_ a file by the same name
                    if (getCanonicalFileName(fullModulePathWithoutExtension.substring(parts.fileNameIndex)) === "/index" && !tryGetAnyFileFromPath(host, fullModulePathWithoutExtension.substring(0, parts.fileNameIndex))) {
                        return fullModulePathWithoutExtension.substring(0, parts.fileNameIndex);
                    }
                    return fullModulePathWithoutExtension;
                }
            }
            function tryGetAnyFileFromPath(host, path) {
                if (!host.fileExists)
                    return;
                // We check all js, `node` and `json` extensions in addition to TS, since node module resolution would also choose those over the directory
                var extensions = ts.getSupportedExtensions({ allowJs: true }, [{ extension: "node", isMixedContent: false }, { extension: "json", isMixedContent: false, scriptKind: 6 /* JSON */ }]);
                for (var _i = 0, extensions_3 = extensions; _i < extensions_3.length; _i++) {
                    var e = extensions_3[_i];
                    var fullPath = path + e;
                    if (host.fileExists(fullPath)) {
                        return fullPath;
                    }
                }
            }
            function getNodeModulePathParts(fullPath) {
                // If fullPath can't be valid module file within node_modules, returns undefined.
                // Example of expected pattern: /base/path/node_modules/[@scope/otherpackage/@otherscope/node_modules/]package/[subdirectory/]file.js
                // Returns indices:                       ^            ^                                                      ^             ^
                var topLevelNodeModulesIndex = 0;
                var topLevelPackageNameIndex = 0;
                var packageRootIndex = 0;
                var fileNameIndex = 0;
                var States;
                (function (States) {
                    States[States["BeforeNodeModules"] = 0] = "BeforeNodeModules";
                    States[States["NodeModules"] = 1] = "NodeModules";
                    States[States["Scope"] = 2] = "Scope";
                    States[States["PackageContent"] = 3] = "PackageContent";
                })(States || (States = {}));
                var partStart = 0;
                var partEnd = 0;
                var state = 0 /* BeforeNodeModules */;
                while (partEnd >= 0) {
                    partStart = partEnd;
                    partEnd = fullPath.indexOf("/", partStart + 1);
                    switch (state) {
                        case 0 /* BeforeNodeModules */:
                            if (fullPath.indexOf(ts.nodeModulesPathPart, partStart) === partStart) {
                                topLevelNodeModulesIndex = partStart;
                                topLevelPackageNameIndex = partEnd;
                                state = 1 /* NodeModules */;
                            }
                            break;
                        case 1 /* NodeModules */:
                        case 2 /* Scope */:
                            if (state === 1 /* NodeModules */ && fullPath.charAt(partStart + 1) === "@") {
                                state = 2 /* Scope */;
                            }
                            else {
                                packageRootIndex = partEnd;
                                state = 3 /* PackageContent */;
                            }
                            break;
                        case 3 /* PackageContent */:
                            if (fullPath.indexOf(ts.nodeModulesPathPart, partStart) === partStart) {
                                state = 1 /* NodeModules */;
                            }
                            else {
                                state = 3 /* PackageContent */;
                            }
                            break;
                    }
                }
                fileNameIndex = partStart;
                return state > 1 /* NodeModules */ ? { topLevelNodeModulesIndex: topLevelNodeModulesIndex, topLevelPackageNameIndex: topLevelPackageNameIndex, packageRootIndex: packageRootIndex, fileNameIndex: fileNameIndex } : undefined;
            }
            function getPathRelativeToRootDirs(path, rootDirs, getCanonicalFileName) {
                return ts.firstDefined(rootDirs, function (rootDir) {
                    var relativePath = getRelativePathIfInDirectory(path, rootDir, getCanonicalFileName); // TODO: GH#18217
                    return isPathRelativeToParent(relativePath) ? undefined : relativePath;
                });
            }
            function removeExtensionAndIndexPostFix(fileName, ending, options) {
                if (ts.fileExtensionIs(fileName, ".json" /* Json */))
                    return fileName;
                var noExtension = ts.removeFileExtension(fileName);
                switch (ending) {
                    case 0 /* Minimal */:
                        return ts.removeSuffix(noExtension, "/index");
                    case 1 /* Index */:
                        return noExtension;
                    case 2 /* JsExtension */:
                        return noExtension + getJSExtensionForFile(fileName, options);
                    default:
                        return ts.Debug.assertNever(ending);
                }
            }
            function getJSExtensionForFile(fileName, options) {
                var ext = ts.extensionFromPath(fileName);
                switch (ext) {
                    case ".ts" /* Ts */:
                    case ".d.ts" /* Dts */:
                        return ".js" /* Js */;
                    case ".tsx" /* Tsx */:
                        return options.jsx === 1 /* Preserve */ ? ".jsx" /* Jsx */ : ".js" /* Js */;
                    case ".js" /* Js */:
                    case ".jsx" /* Jsx */:
                    case ".json" /* Json */:
                        return ext;
                    case ".tsbuildinfo" /* TsBuildInfo */:
                        return ts.Debug.fail("Extension " + ".tsbuildinfo" /* TsBuildInfo */ + " is unsupported:: FileName:: " + fileName);
                    default:
                        return ts.Debug.assertNever(ext);
                }
            }
            function getRelativePathIfInDirectory(path, directoryPath, getCanonicalFileName) {
                var relativePath = ts.getRelativePathToDirectoryOrUrl(directoryPath, path, directoryPath, getCanonicalFileName, /*isAbsolutePathAnUrl*/ false);
                return ts.isRootedDiskPath(relativePath) ? undefined : relativePath;
            }
            function isPathRelativeToParent(path) {
                return ts.startsWith(path, "..");
            }
        })(moduleSpecifiers = ts.moduleSpecifiers || (ts.moduleSpecifiers = {}));
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        var sysFormatDiagnosticsHost = ts.sys ? {
            getCurrentDirectory: function () { return ts.sys.getCurrentDirectory(); },
            getNewLine: function () { return ts.sys.newLine; },
            getCanonicalFileName: ts.createGetCanonicalFileName(ts.sys.useCaseSensitiveFileNames)
        } : undefined; // TODO: GH#18217
        /**
         * Create a function that reports error by writing to the system and handles the formating of the diagnostic
         */
        function createDiagnosticReporter(system, pretty) {
            var host = system === ts.sys ? sysFormatDiagnosticsHost : {
                getCurrentDirectory: function () { return system.getCurrentDirectory(); },
                getNewLine: function () { return system.newLine; },
                getCanonicalFileName: ts.createGetCanonicalFileName(system.useCaseSensitiveFileNames),
            };
            if (!pretty) {
                return function (diagnostic) { return system.write(ts.formatDiagnostic(diagnostic, host)); };
            }
            var diagnostics = new Array(1);
            return function (diagnostic) {
                diagnostics[0] = diagnostic;
                system.write(ts.formatDiagnosticsWithColorAndContext(diagnostics, host) + host.getNewLine());
                diagnostics[0] = undefined; // TODO: GH#18217
            };
        }
        ts.createDiagnosticReporter = createDiagnosticReporter;
        /**
         * @returns Whether the screen was cleared.
         */
        function clearScreenIfNotWatchingForFileChanges(system, diagnostic, options) {
            if (system.clearScreen &&
                !options.preserveWatchOutput &&
                !options.extendedDiagnostics &&
                !options.diagnostics &&
                ts.contains(ts.screenStartingMessageCodes, diagnostic.code)) {
                system.clearScreen();
                return true;
            }
            return false;
        }
        ts.screenStartingMessageCodes = [
            ts.Diagnostics.Starting_compilation_in_watch_mode.code,
            ts.Diagnostics.File_change_detected_Starting_incremental_compilation.code,
        ];
        function getPlainDiagnosticFollowingNewLines(diagnostic, newLine) {
            return ts.contains(ts.screenStartingMessageCodes, diagnostic.code)
                ? newLine + newLine
                : newLine;
        }
        /**
         * Get locale specific time based on whether we are in test mode
         */
        function getLocaleTimeString(system) {
            return !system.now ?
                new Date().toLocaleTimeString() :
                system.now().toLocaleTimeString("en-US", { timeZone: "UTC" });
        }
        ts.getLocaleTimeString = getLocaleTimeString;
        /**
         * Create a function that reports watch status by writing to the system and handles the formating of the diagnostic
         */
        function createWatchStatusReporter(system, pretty) {
            return pretty ?
                function (diagnostic, newLine, options) {
                    clearScreenIfNotWatchingForFileChanges(system, diagnostic, options);
                    var output = "[" + ts.formatColorAndReset(getLocaleTimeString(system), ts.ForegroundColorEscapeSequences.Grey) + "] ";
                    output += "" + ts.flattenDiagnosticMessageText(diagnostic.messageText, system.newLine) + (newLine + newLine);
                    system.write(output);
                } :
                function (diagnostic, newLine, options) {
                    var output = "";
                    if (!clearScreenIfNotWatchingForFileChanges(system, diagnostic, options)) {
                        output += newLine;
                    }
                    output += getLocaleTimeString(system) + " - ";
                    output += "" + ts.flattenDiagnosticMessageText(diagnostic.messageText, system.newLine) + getPlainDiagnosticFollowingNewLines(diagnostic, newLine);
                    system.write(output);
                };
        }
        ts.createWatchStatusReporter = createWatchStatusReporter;
        /** Parses config file using System interface */
        function parseConfigFileWithSystem(configFileName, optionsToExtend, watchOptionsToExtend, system, reportDiagnostic) {
            var host = system;
            host.onUnRecoverableConfigFileDiagnostic = function (diagnostic) { return reportUnrecoverableDiagnostic(system, reportDiagnostic, diagnostic); };
            var result = ts.getParsedCommandLineOfConfigFile(configFileName, optionsToExtend, host, /*extendedConfigCache*/ undefined, watchOptionsToExtend);
            host.onUnRecoverableConfigFileDiagnostic = undefined; // TODO: GH#18217
            return result;
        }
        ts.parseConfigFileWithSystem = parseConfigFileWithSystem;
        function getErrorCountForSummary(diagnostics) {
            return ts.countWhere(diagnostics, function (diagnostic) { return diagnostic.category === ts.DiagnosticCategory.Error; });
        }
        ts.getErrorCountForSummary = getErrorCountForSummary;
        function getWatchErrorSummaryDiagnosticMessage(errorCount) {
            return errorCount === 1 ?
                ts.Diagnostics.Found_1_error_Watching_for_file_changes :
                ts.Diagnostics.Found_0_errors_Watching_for_file_changes;
        }
        ts.getWatchErrorSummaryDiagnosticMessage = getWatchErrorSummaryDiagnosticMessage;
        function getErrorSummaryText(errorCount, newLine) {
            if (errorCount === 0)
                return "";
            var d = ts.createCompilerDiagnostic(errorCount === 1 ? ts.Diagnostics.Found_1_error : ts.Diagnostics.Found_0_errors, errorCount);
            return "" + newLine + ts.flattenDiagnosticMessageText(d.messageText, newLine) + newLine + newLine;
        }
        ts.getErrorSummaryText = getErrorSummaryText;
        function listFiles(program, writeFileName) {
            if (program.getCompilerOptions().listFiles || program.getCompilerOptions().listFilesOnly) {
                ts.forEach(program.getSourceFiles(), function (file) {
                    writeFileName(file.fileName);
                });
            }
        }
        ts.listFiles = listFiles;
        /**
         * Helper that emit files, report diagnostics and lists emitted and/or source files depending on compiler options
         */
        function emitFilesAndReportErrors(program, reportDiagnostic, writeFileName, reportSummary, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers) {
            var isListFilesOnly = !!program.getCompilerOptions().listFilesOnly;
            // First get and report any syntactic errors.
            var allDiagnostics = program.getConfigFileParsingDiagnostics().slice();
            var configFileParsingDiagnosticsLength = allDiagnostics.length;
            ts.addRange(allDiagnostics, program.getSyntacticDiagnostics(/*sourceFile*/ undefined, cancellationToken));
            // If we didn't have any syntactic errors, then also try getting the global and
            // semantic errors.
            if (allDiagnostics.length === configFileParsingDiagnosticsLength) {
                ts.addRange(allDiagnostics, program.getOptionsDiagnostics(cancellationToken));
                if (!isListFilesOnly) {
                    ts.addRange(allDiagnostics, program.getGlobalDiagnostics(cancellationToken));
                    if (allDiagnostics.length === configFileParsingDiagnosticsLength) {
                        ts.addRange(allDiagnostics, program.getSemanticDiagnostics(/*sourceFile*/ undefined, cancellationToken));
                    }
                }
            }
            // Emit and report any errors we ran into.
            var emitResult = isListFilesOnly
                ? { emitSkipped: true, diagnostics: ts.emptyArray }
                : program.emit(/*targetSourceFile*/ undefined, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers);
            var emittedFiles = emitResult.emittedFiles, emitDiagnostics = emitResult.diagnostics;
            ts.addRange(allDiagnostics, emitDiagnostics);
            var diagnostics = ts.sortAndDeduplicateDiagnostics(allDiagnostics);
            diagnostics.forEach(reportDiagnostic);
            if (writeFileName) {
                var currentDir_1 = program.getCurrentDirectory();
                ts.forEach(emittedFiles, function (file) {
                    var filepath = ts.getNormalizedAbsolutePath(file, currentDir_1);
                    writeFileName("TSFILE: " + filepath);
                });
                listFiles(program, writeFileName);
            }
            if (reportSummary) {
                reportSummary(getErrorCountForSummary(diagnostics));
            }
            return {
                emitResult: emitResult,
                diagnostics: diagnostics,
            };
        }
        ts.emitFilesAndReportErrors = emitFilesAndReportErrors;
        function emitFilesAndReportErrorsAndGetExitStatus(program, reportDiagnostic, writeFileName, reportSummary, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers) {
            var _a = emitFilesAndReportErrors(program, reportDiagnostic, writeFileName, reportSummary, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers), emitResult = _a.emitResult, diagnostics = _a.diagnostics;
            if (emitResult.emitSkipped && diagnostics.length > 0) {
                // If the emitter didn't emit anything, then pass that value along.
                return ts.ExitStatus.DiagnosticsPresent_OutputsSkipped;
            }
            else if (diagnostics.length > 0) {
                // The emitter emitted something, inform the caller if that happened in the presence
                // of diagnostics or not.
                return ts.ExitStatus.DiagnosticsPresent_OutputsGenerated;
            }
            return ts.ExitStatus.Success;
        }
        ts.emitFilesAndReportErrorsAndGetExitStatus = emitFilesAndReportErrorsAndGetExitStatus;
        ts.noopFileWatcher = { close: ts.noop };
        function createWatchHost(system, reportWatchStatus) {
            if (system === void 0) {
                system = ts.sys;
            }
            var onWatchStatusChange = reportWatchStatus || createWatchStatusReporter(system);
            return {
                onWatchStatusChange: onWatchStatusChange,
                watchFile: ts.maybeBind(system, system.watchFile) || (function () { return ts.noopFileWatcher; }),
                watchDirectory: ts.maybeBind(system, system.watchDirectory) || (function () { return ts.noopFileWatcher; }),
                setTimeout: ts.maybeBind(system, system.setTimeout) || ts.noop,
                clearTimeout: ts.maybeBind(system, system.clearTimeout) || ts.noop
            };
        }
        ts.createWatchHost = createWatchHost;
        ts.WatchType = {
            ConfigFile: "Config file",
            SourceFile: "Source file",
            MissingFile: "Missing file",
            WildcardDirectory: "Wild card directory",
            FailedLookupLocations: "Failed Lookup Locations",
            TypeRoots: "Type roots"
        };
        function createWatchFactory(host, options) {
            var watchLogLevel = host.trace ? options.extendedDiagnostics ? ts.WatchLogLevel.Verbose : options.diagnostics ? ts.WatchLogLevel.TriggerOnly : ts.WatchLogLevel.None : ts.WatchLogLevel.None;
            var writeLog = watchLogLevel !== ts.WatchLogLevel.None ? (function (s) { return host.trace(s); }) : ts.noop;
            var result = ts.getWatchFactory(watchLogLevel, writeLog);
            result.writeLog = writeLog;
            return result;
        }
        ts.createWatchFactory = createWatchFactory;
        function createCompilerHostFromProgramHost(host, getCompilerOptions, directoryStructureHost) {
            if (directoryStructureHost === void 0) {
                directoryStructureHost = host;
            }
            var useCaseSensitiveFileNames = host.useCaseSensitiveFileNames();
            var hostGetNewLine = ts.memoize(function () { return host.getNewLine(); });
            return {
                getSourceFile: function (fileName, languageVersion, onError) {
                    var text;
                    try {
                        ts.performance.mark("beforeIORead");
                        text = host.readFile(fileName, getCompilerOptions().charset);
                        ts.performance.mark("afterIORead");
                        ts.performance.measure("I/O Read", "beforeIORead", "afterIORead");
                    }
                    catch (e) {
                        if (onError) {
                            onError(e.message);
                        }
                        text = "";
                    }
                    return text !== undefined ? ts.createSourceFile(fileName, text, languageVersion) : undefined;
                },
                getDefaultLibLocation: ts.maybeBind(host, host.getDefaultLibLocation),
                getDefaultLibFileName: function (options) { return host.getDefaultLibFileName(options); },
                writeFile: writeFile,
                getCurrentDirectory: ts.memoize(function () { return host.getCurrentDirectory(); }),
                useCaseSensitiveFileNames: function () { return useCaseSensitiveFileNames; },
                getCanonicalFileName: ts.createGetCanonicalFileName(useCaseSensitiveFileNames),
                getNewLine: function () { return ts.getNewLineCharacter(getCompilerOptions(), hostGetNewLine); },
                fileExists: function (f) { return host.fileExists(f); },
                readFile: function (f) { return host.readFile(f); },
                trace: ts.maybeBind(host, host.trace),
                directoryExists: ts.maybeBind(directoryStructureHost, directoryStructureHost.directoryExists),
                getDirectories: ts.maybeBind(directoryStructureHost, directoryStructureHost.getDirectories),
                realpath: ts.maybeBind(host, host.realpath),
                getEnvironmentVariable: ts.maybeBind(host, host.getEnvironmentVariable) || (function () { return ""; }),
                createHash: ts.maybeBind(host, host.createHash),
                readDirectory: ts.maybeBind(host, host.readDirectory),
            };
            function writeFile(fileName, text, writeByteOrderMark, onError) {
                try {
                    ts.performance.mark("beforeIOWrite");
                    // NOTE: If patchWriteFileEnsuringDirectory has been called,
                    // the host.writeFile will do its own directory creation and
                    // the ensureDirectoriesExist call will always be redundant.
                    ts.writeFileEnsuringDirectories(fileName, text, writeByteOrderMark, function (path, data, writeByteOrderMark) { return host.writeFile(path, data, writeByteOrderMark); }, function (path) { return host.createDirectory(path); }, function (path) { return host.directoryExists(path); });
                    ts.performance.mark("afterIOWrite");
                    ts.performance.measure("I/O Write", "beforeIOWrite", "afterIOWrite");
                }
                catch (e) {
                    if (onError) {
                        onError(e.message);
                    }
                }
            }
        }
        ts.createCompilerHostFromProgramHost = createCompilerHostFromProgramHost;
        function setGetSourceFileAsHashVersioned(compilerHost, host) {
            var originalGetSourceFile = compilerHost.getSourceFile;
            var computeHash = host.createHash || ts.generateDjb2Hash;
            compilerHost.getSourceFile = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var result = originalGetSourceFile.call.apply(originalGetSourceFile, __spreadArrays([compilerHost], args));
                if (result) {
                    result.version = computeHash.call(host, result.text);
                }
                return result;
            };
        }
        ts.setGetSourceFileAsHashVersioned = setGetSourceFileAsHashVersioned;
        /**
         * Creates the watch compiler host that can be extended with config file or root file names and options host
         */
        function createProgramHost(system, createProgram) {
            var getDefaultLibLocation = ts.memoize(function () { return ts.getDirectoryPath(ts.normalizePath(system.getExecutingFilePath())); });
            var host = system;
            // TODO: `host` is unused!
            // eslint-disable-next-line no-unused-expressions
            host;
            return {
                useCaseSensitiveFileNames: function () { return system.useCaseSensitiveFileNames; },
                getNewLine: function () { return system.newLine; },
                getCurrentDirectory: ts.memoize(function () { return system.getCurrentDirectory(); }),
                getDefaultLibLocation: getDefaultLibLocation,
                getDefaultLibFileName: function (options) { return ts.combinePaths(getDefaultLibLocation(), ts.getDefaultLibFileName(options)); },
                fileExists: function (path) { return system.fileExists(path); },
                readFile: function (path, encoding) { return system.readFile(path, encoding); },
                directoryExists: function (path) { return system.directoryExists(path); },
                getDirectories: function (path) { return system.getDirectories(path); },
                readDirectory: function (path, extensions, exclude, include, depth) { return system.readDirectory(path, extensions, exclude, include, depth); },
                realpath: ts.maybeBind(system, system.realpath),
                getEnvironmentVariable: ts.maybeBind(system, system.getEnvironmentVariable),
                trace: function (s) { return system.write(s + system.newLine); },
                createDirectory: function (path) { return system.createDirectory(path); },
                writeFile: function (path, data, writeByteOrderMark) { return system.writeFile(path, data, writeByteOrderMark); },
                onCachedDirectoryStructureHostCreate: function (cacheHost) { return host = cacheHost || system; },
                createHash: ts.maybeBind(system, system.createHash),
                createProgram: createProgram || ts.createEmitAndSemanticDiagnosticsBuilderProgram
            };
        }
        ts.createProgramHost = createProgramHost;
        /**
         * Creates the watch compiler host that can be extended with config file or root file names and options host
         */
        function createWatchCompilerHost(system, createProgram, reportDiagnostic, reportWatchStatus) {
            if (system === void 0) {
                system = ts.sys;
            }
            var writeFileName = function (s) { return system.write(s + system.newLine); };
            var result = createProgramHost(system, createProgram);
            ts.copyProperties(result, createWatchHost(system, reportWatchStatus));
            result.afterProgramCreate = function (builderProgram) {
                var compilerOptions = builderProgram.getCompilerOptions();
                var newLine = ts.getNewLineCharacter(compilerOptions, function () { return system.newLine; });
                emitFilesAndReportErrors(builderProgram, reportDiagnostic, writeFileName, function (errorCount) { return result.onWatchStatusChange(ts.createCompilerDiagnostic(getWatchErrorSummaryDiagnosticMessage(errorCount), errorCount), newLine, compilerOptions, errorCount); });
            };
            return result;
        }
        /**
         * Report error and exit
         */
        function reportUnrecoverableDiagnostic(system, reportDiagnostic, diagnostic) {
            reportDiagnostic(diagnostic);
            system.exit(ts.ExitStatus.DiagnosticsPresent_OutputsSkipped);
        }
        /**
         * Creates the watch compiler host from system for config file in watch mode
         */
        function createWatchCompilerHostOfConfigFile(configFileName, optionsToExtend, watchOptionsToExtend, system, createProgram, reportDiagnostic, reportWatchStatus) {
            var diagnosticReporter = reportDiagnostic || createDiagnosticReporter(system);
            var host = createWatchCompilerHost(system, createProgram, diagnosticReporter, reportWatchStatus);
            host.onUnRecoverableConfigFileDiagnostic = function (diagnostic) { return reportUnrecoverableDiagnostic(system, diagnosticReporter, diagnostic); };
            host.configFileName = configFileName;
            host.optionsToExtend = optionsToExtend;
            host.watchOptionsToExtend = watchOptionsToExtend;
            return host;
        }
        ts.createWatchCompilerHostOfConfigFile = createWatchCompilerHostOfConfigFile;
        /**
         * Creates the watch compiler host from system for compiling root files and options in watch mode
         */
        function createWatchCompilerHostOfFilesAndCompilerOptions(rootFiles, options, watchOptions, system, createProgram, reportDiagnostic, reportWatchStatus, projectReferences) {
            var host = createWatchCompilerHost(system, createProgram, reportDiagnostic || createDiagnosticReporter(system), reportWatchStatus);
            host.rootFiles = rootFiles;
            host.options = options;
            host.watchOptions = watchOptions;
            host.projectReferences = projectReferences;
            return host;
        }
        ts.createWatchCompilerHostOfFilesAndCompilerOptions = createWatchCompilerHostOfFilesAndCompilerOptions;
        function performIncrementalCompilation(input) {
            var system = input.system || ts.sys;
            var host = input.host || (input.host = ts.createIncrementalCompilerHost(input.options, system));
            var builderProgram = ts.createIncrementalProgram(input);
            var exitStatus = emitFilesAndReportErrorsAndGetExitStatus(builderProgram, input.reportDiagnostic || createDiagnosticReporter(system), function (s) { return host.trace && host.trace(s); }, input.reportErrorSummary || input.options.pretty ? function (errorCount) { return system.write(getErrorSummaryText(errorCount, system.newLine)); } : undefined);
            if (input.afterProgramEmitAndDiagnostics)
                input.afterProgramEmitAndDiagnostics(builderProgram);
            return exitStatus;
        }
        ts.performIncrementalCompilation = performIncrementalCompilation;
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        function readBuilderProgram(compilerOptions, host) {
            if (compilerOptions.out || compilerOptions.outFile)
                return undefined;
            var buildInfoPath = ts.getTsBuildInfoEmitOutputFilePath(compilerOptions);
            if (!buildInfoPath)
                return undefined;
            var content = host.readFile(buildInfoPath);
            if (!content)
                return undefined;
            var buildInfo = ts.getBuildInfo(content);
            if (buildInfo.version !== ts.version)
                return undefined;
            if (!buildInfo.program)
                return undefined;
            return ts.createBuildProgramUsingProgramBuildInfo(buildInfo.program, buildInfoPath, host);
        }
        ts.readBuilderProgram = readBuilderProgram;
        function createIncrementalCompilerHost(options, system) {
            if (system === void 0) {
                system = ts.sys;
            }
            var host = ts.createCompilerHostWorker(options, /*setParentNodes*/ undefined, system);
            host.createHash = ts.maybeBind(system, system.createHash);
            ts.setGetSourceFileAsHashVersioned(host, system);
            ts.changeCompilerHostLikeToUseCache(host, function (fileName) { return ts.toPath(fileName, host.getCurrentDirectory(), host.getCanonicalFileName); });
            return host;
        }
        ts.createIncrementalCompilerHost = createIncrementalCompilerHost;
        function createIncrementalProgram(_a) {
            var rootNames = _a.rootNames, options = _a.options, configFileParsingDiagnostics = _a.configFileParsingDiagnostics, projectReferences = _a.projectReferences, host = _a.host, createProgram = _a.createProgram;
            host = host || createIncrementalCompilerHost(options);
            createProgram = createProgram || ts.createEmitAndSemanticDiagnosticsBuilderProgram;
            var oldProgram = readBuilderProgram(options, host);
            return createProgram(rootNames, options, host, oldProgram, configFileParsingDiagnostics, projectReferences);
        }
        ts.createIncrementalProgram = createIncrementalProgram;
        function createWatchCompilerHost(rootFilesOrConfigFileName, options, system, createProgram, reportDiagnostic, reportWatchStatus, projectReferencesOrWatchOptionsToExtend, watchOptions) {
            if (ts.isArray(rootFilesOrConfigFileName)) {
                return ts.createWatchCompilerHostOfFilesAndCompilerOptions(rootFilesOrConfigFileName, options, watchOptions, system, createProgram, reportDiagnostic, reportWatchStatus, projectReferencesOrWatchOptionsToExtend); // TODO: GH#18217
            }
            else {
                return ts.createWatchCompilerHostOfConfigFile(rootFilesOrConfigFileName, options, projectReferencesOrWatchOptionsToExtend, system, createProgram, reportDiagnostic, reportWatchStatus);
            }
        }
        ts.createWatchCompilerHost = createWatchCompilerHost;
        function createWatchProgram(host) {
            var builderProgram;
            var reloadLevel; // level to indicate if the program needs to be reloaded from config file/just filenames etc
            var missingFilesMap; // Map of file watchers for the missing files
            var watchedWildcardDirectories; // map of watchers for the wild card directories in the config file
            var timerToUpdateProgram; // timer callback to recompile the program
            var sourceFilesCache = ts.createMap(); // Cache that stores the source file and version info
            var missingFilePathsRequestedForRelease; // These paths are held temparirly so that we can remove the entry from source file cache if the file is not tracked by missing files
            var hasChangedCompilerOptions = false; // True if the compiler options have changed between compilations
            var hasChangedAutomaticTypeDirectiveNames = false; // True if the automatic type directives have changed
            var useCaseSensitiveFileNames = host.useCaseSensitiveFileNames();
            var currentDirectory = host.getCurrentDirectory();
            var configFileName = host.configFileName, _a = host.optionsToExtend, optionsToExtendForConfigFile = _a === void 0 ? {} : _a, watchOptionsToExtend = host.watchOptionsToExtend, createProgram = host.createProgram;
            var rootFileNames = host.rootFiles, compilerOptions = host.options, watchOptions = host.watchOptions, projectReferences = host.projectReferences;
            var configFileSpecs;
            var configFileParsingDiagnostics;
            var canConfigFileJsonReportNoInputFiles = false;
            var hasChangedConfigFileParsingErrors = false;
            var cachedDirectoryStructureHost = configFileName === undefined ? undefined : ts.createCachedDirectoryStructureHost(host, currentDirectory, useCaseSensitiveFileNames);
            if (cachedDirectoryStructureHost && host.onCachedDirectoryStructureHostCreate) {
                host.onCachedDirectoryStructureHostCreate(cachedDirectoryStructureHost);
            }
            var directoryStructureHost = cachedDirectoryStructureHost || host;
            var parseConfigFileHost = ts.parseConfigHostFromCompilerHostLike(host, directoryStructureHost);
            // From tsc we want to get already parsed result and hence check for rootFileNames
            var newLine = updateNewLine();
            if (configFileName && host.configFileParsingResult) {
                setConfigFileParsingResult(host.configFileParsingResult);
                newLine = updateNewLine();
            }
            reportWatchDiagnostic(ts.Diagnostics.Starting_compilation_in_watch_mode);
            if (configFileName && !host.configFileParsingResult) {
                newLine = ts.getNewLineCharacter(optionsToExtendForConfigFile, function () { return host.getNewLine(); });
                ts.Debug.assert(!rootFileNames);
                parseConfigFile();
                newLine = updateNewLine();
            }
            var _b = ts.createWatchFactory(host, compilerOptions), watchFile = _b.watchFile, watchFilePath = _b.watchFilePath, watchDirectory = _b.watchDirectory, writeLog = _b.writeLog;
            var getCanonicalFileName = ts.createGetCanonicalFileName(useCaseSensitiveFileNames);
            writeLog("Current directory: " + currentDirectory + " CaseSensitiveFileNames: " + useCaseSensitiveFileNames);
            var configFileWatcher;
            if (configFileName) {
                configFileWatcher = watchFile(host, configFileName, scheduleProgramReload, ts.PollingInterval.High, watchOptions, ts.WatchType.ConfigFile);
            }
            var compilerHost = ts.createCompilerHostFromProgramHost(host, function () { return compilerOptions; }, directoryStructureHost);
            ts.setGetSourceFileAsHashVersioned(compilerHost, host);
            // Members for CompilerHost
            var getNewSourceFile = compilerHost.getSourceFile;
            compilerHost.getSourceFile = function (fileName) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                return getVersionedSourceFileByPath.apply(void 0, __spreadArrays([fileName, toPath(fileName)], args));
            };
            compilerHost.getSourceFileByPath = getVersionedSourceFileByPath;
            compilerHost.getNewLine = function () { return newLine; };
            compilerHost.fileExists = fileExists;
            compilerHost.onReleaseOldSourceFile = onReleaseOldSourceFile;
            // Members for ResolutionCacheHost
            compilerHost.toPath = toPath;
            compilerHost.getCompilationSettings = function () { return compilerOptions; };
            compilerHost.watchDirectoryOfFailedLookupLocation = function (dir, cb, flags) { return watchDirectory(host, dir, cb, flags, watchOptions, ts.WatchType.FailedLookupLocations); };
            compilerHost.watchTypeRootsDirectory = function (dir, cb, flags) { return watchDirectory(host, dir, cb, flags, watchOptions, ts.WatchType.TypeRoots); };
            compilerHost.getCachedDirectoryStructureHost = function () { return cachedDirectoryStructureHost; };
            compilerHost.onInvalidatedResolution = scheduleProgramUpdate;
            compilerHost.onChangedAutomaticTypeDirectiveNames = function () {
                hasChangedAutomaticTypeDirectiveNames = true;
                scheduleProgramUpdate();
            };
            compilerHost.fileIsOpen = ts.returnFalse;
            compilerHost.maxNumberOfFilesToIterateForInvalidation = host.maxNumberOfFilesToIterateForInvalidation;
            compilerHost.getCurrentProgram = getCurrentProgram;
            compilerHost.writeLog = writeLog;
            // Cache for the module resolution
            var resolutionCache = ts.createResolutionCache(compilerHost, configFileName ?
                ts.getDirectoryPath(ts.getNormalizedAbsolutePath(configFileName, currentDirectory)) :
                currentDirectory, 
            /*logChangesWhenResolvingModule*/ false);
            // Resolve module using host module resolution strategy if provided otherwise use resolution cache to resolve module names
            compilerHost.resolveModuleNames = host.resolveModuleNames ?
                (function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    return host.resolveModuleNames.apply(host, args);
                }) :
                (function (moduleNames, containingFile, reusedNames, redirectedReference) { return resolutionCache.resolveModuleNames(moduleNames, containingFile, reusedNames, redirectedReference); });
            compilerHost.resolveTypeReferenceDirectives = host.resolveTypeReferenceDirectives ?
                (function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    return host.resolveTypeReferenceDirectives.apply(host, args);
                }) :
                (function (typeDirectiveNames, containingFile, redirectedReference) { return resolutionCache.resolveTypeReferenceDirectives(typeDirectiveNames, containingFile, redirectedReference); });
            var userProvidedResolution = !!host.resolveModuleNames || !!host.resolveTypeReferenceDirectives;
            builderProgram = readBuilderProgram(compilerOptions, compilerHost);
            synchronizeProgram();
            // Update the wild card directory watch
            watchConfigFileWildCardDirectories();
            return configFileName ?
                { getCurrentProgram: getCurrentBuilderProgram, getProgram: synchronizeProgram, close: close } :
                { getCurrentProgram: getCurrentBuilderProgram, getProgram: synchronizeProgram, updateRootFileNames: updateRootFileNames, close: close };
            function close() {
                resolutionCache.clear();
                ts.clearMap(sourceFilesCache, function (value) {
                    if (value && value.fileWatcher) {
                        value.fileWatcher.close();
                        value.fileWatcher = undefined;
                    }
                });
                if (configFileWatcher) {
                    configFileWatcher.close();
                    configFileWatcher = undefined;
                }
                if (watchedWildcardDirectories) {
                    ts.clearMap(watchedWildcardDirectories, ts.closeFileWatcherOf);
                    watchedWildcardDirectories = undefined;
                }
                if (missingFilesMap) {
                    ts.clearMap(missingFilesMap, ts.closeFileWatcher);
                    missingFilesMap = undefined;
                }
            }
            function getCurrentBuilderProgram() {
                return builderProgram;
            }
            function getCurrentProgram() {
                return builderProgram && builderProgram.getProgramOrUndefined();
            }
            function synchronizeProgram() {
                writeLog("Synchronizing program");
                var program = getCurrentBuilderProgram();
                if (hasChangedCompilerOptions) {
                    newLine = updateNewLine();
                    if (program && ts.changesAffectModuleResolution(program.getCompilerOptions(), compilerOptions)) {
                        resolutionCache.clear();
                    }
                }
                // All resolutions are invalid if user provided resolutions
                var hasInvalidatedResolution = resolutionCache.createHasInvalidatedResolution(userProvidedResolution);
                if (ts.isProgramUptoDate(getCurrentProgram(), rootFileNames, compilerOptions, getSourceVersion, fileExists, hasInvalidatedResolution, hasChangedAutomaticTypeDirectiveNames, projectReferences)) {
                    if (hasChangedConfigFileParsingErrors) {
                        builderProgram = createProgram(/*rootNames*/ undefined, /*options*/ undefined, compilerHost, builderProgram, configFileParsingDiagnostics, projectReferences);
                        hasChangedConfigFileParsingErrors = false;
                    }
                }
                else {
                    createNewProgram(hasInvalidatedResolution);
                }
                if (host.afterProgramCreate) {
                    host.afterProgramCreate(builderProgram);
                }
                return builderProgram;
            }
            function createNewProgram(hasInvalidatedResolution) {
                // Compile the program
                writeLog("CreatingProgramWith::");
                writeLog("  roots: " + JSON.stringify(rootFileNames));
                writeLog("  options: " + JSON.stringify(compilerOptions));
                var needsUpdateInTypeRootWatch = hasChangedCompilerOptions || !getCurrentProgram();
                hasChangedCompilerOptions = false;
                hasChangedConfigFileParsingErrors = false;
                resolutionCache.startCachingPerDirectoryResolution();
                compilerHost.hasInvalidatedResolution = hasInvalidatedResolution;
                compilerHost.hasChangedAutomaticTypeDirectiveNames = hasChangedAutomaticTypeDirectiveNames;
                builderProgram = createProgram(rootFileNames, compilerOptions, compilerHost, builderProgram, configFileParsingDiagnostics, projectReferences);
                resolutionCache.finishCachingPerDirectoryResolution();
                // Update watches
                ts.updateMissingFilePathsWatch(builderProgram.getProgram(), missingFilesMap || (missingFilesMap = ts.createMap()), watchMissingFilePath);
                if (needsUpdateInTypeRootWatch) {
                    resolutionCache.updateTypeRootsWatch();
                }
                if (missingFilePathsRequestedForRelease) {
                    // These are the paths that program creater told us as not in use any more but were missing on the disk.
                    // We didnt remove the entry for them from sourceFiles cache so that we dont have to do File IO,
                    // if there is already watcher for it (for missing files)
                    // At this point our watches were updated, hence now we know that these paths are not tracked and need to be removed
                    // so that at later time we have correct result of their presence
                    for (var _i = 0, missingFilePathsRequestedForRelease_1 = missingFilePathsRequestedForRelease; _i < missingFilePathsRequestedForRelease_1.length; _i++) {
                        var missingFilePath = missingFilePathsRequestedForRelease_1[_i];
                        if (!missingFilesMap.has(missingFilePath)) {
                            sourceFilesCache.delete(missingFilePath);
                        }
                    }
                    missingFilePathsRequestedForRelease = undefined;
                }
            }
            function updateRootFileNames(files) {
                ts.Debug.assert(!configFileName, "Cannot update root file names with config file watch mode");
                rootFileNames = files;
                scheduleProgramUpdate();
            }
            function updateNewLine() {
                return ts.getNewLineCharacter(compilerOptions || optionsToExtendForConfigFile, function () { return host.getNewLine(); });
            }
            function toPath(fileName) {
                return ts.toPath(fileName, currentDirectory, getCanonicalFileName);
            }
            function isFileMissingOnHost(hostSourceFile) {
                return typeof hostSourceFile === "boolean";
            }
            function isFilePresenceUnknownOnHost(hostSourceFile) {
                return typeof hostSourceFile.version === "boolean";
            }
            function fileExists(fileName) {
                var path = toPath(fileName);
                // If file is missing on host from cache, we can definitely say file doesnt exist
                // otherwise we need to ensure from the disk
                if (isFileMissingOnHost(sourceFilesCache.get(path))) {
                    return true;
                }
                return directoryStructureHost.fileExists(fileName);
            }
            function getVersionedSourceFileByPath(fileName, path, languageVersion, onError, shouldCreateNewSourceFile) {
                var hostSourceFile = sourceFilesCache.get(path);
                // No source file on the host
                if (isFileMissingOnHost(hostSourceFile)) {
                    return undefined;
                }
                // Create new source file if requested or the versions dont match
                if (hostSourceFile === undefined || shouldCreateNewSourceFile || isFilePresenceUnknownOnHost(hostSourceFile)) {
                    var sourceFile = getNewSourceFile(fileName, languageVersion, onError);
                    if (hostSourceFile) {
                        if (sourceFile) {
                            // Set the source file and create file watcher now that file was present on the disk
                            hostSourceFile.sourceFile = sourceFile;
                            hostSourceFile.version = sourceFile.version;
                            if (!hostSourceFile.fileWatcher) {
                                hostSourceFile.fileWatcher = watchFilePath(host, fileName, onSourceFileChange, ts.PollingInterval.Low, watchOptions, path, ts.WatchType.SourceFile);
                            }
                        }
                        else {
                            // There is no source file on host any more, close the watch, missing file paths will track it
                            if (hostSourceFile.fileWatcher) {
                                hostSourceFile.fileWatcher.close();
                            }
                            sourceFilesCache.set(path, false);
                        }
                    }
                    else {
                        if (sourceFile) {
                            var fileWatcher = watchFilePath(host, fileName, onSourceFileChange, ts.PollingInterval.Low, watchOptions, path, ts.WatchType.SourceFile);
                            sourceFilesCache.set(path, { sourceFile: sourceFile, version: sourceFile.version, fileWatcher: fileWatcher });
                        }
                        else {
                            sourceFilesCache.set(path, false);
                        }
                    }
                    return sourceFile;
                }
                return hostSourceFile.sourceFile;
            }
            function nextSourceFileVersion(path) {
                var hostSourceFile = sourceFilesCache.get(path);
                if (hostSourceFile !== undefined) {
                    if (isFileMissingOnHost(hostSourceFile)) {
                        // The next version, lets set it as presence unknown file
                        sourceFilesCache.set(path, { version: false });
                    }
                    else {
                        hostSourceFile.version = false;
                    }
                }
            }
            function getSourceVersion(path) {
                var hostSourceFile = sourceFilesCache.get(path);
                return !hostSourceFile || !hostSourceFile.version ? undefined : hostSourceFile.version;
            }
            function onReleaseOldSourceFile(oldSourceFile, _oldOptions, hasSourceFileByPath) {
                var hostSourceFileInfo = sourceFilesCache.get(oldSourceFile.resolvedPath);
                // If this is the source file thats in the cache and new program doesnt need it,
                // remove the cached entry.
                // Note we arent deleting entry if file became missing in new program or
                // there was version update and new source file was created.
                if (hostSourceFileInfo !== undefined) {
                    // record the missing file paths so they can be removed later if watchers arent tracking them
                    if (isFileMissingOnHost(hostSourceFileInfo)) {
                        (missingFilePathsRequestedForRelease || (missingFilePathsRequestedForRelease = [])).push(oldSourceFile.path);
                    }
                    else if (hostSourceFileInfo.sourceFile === oldSourceFile) {
                        if (hostSourceFileInfo.fileWatcher) {
                            hostSourceFileInfo.fileWatcher.close();
                        }
                        sourceFilesCache.delete(oldSourceFile.resolvedPath);
                        if (!hasSourceFileByPath) {
                            resolutionCache.removeResolutionsOfFile(oldSourceFile.path);
                        }
                    }
                }
            }
            function reportWatchDiagnostic(message) {
                if (host.onWatchStatusChange) {
                    host.onWatchStatusChange(ts.createCompilerDiagnostic(message), newLine, compilerOptions || optionsToExtendForConfigFile);
                }
            }
            // Upon detecting a file change, wait for 250ms and then perform a recompilation. This gives batch
            // operations (such as saving all modified files in an editor) a chance to complete before we kick
            // off a new compilation.
            function scheduleProgramUpdate() {
                if (!host.setTimeout || !host.clearTimeout) {
                    return;
                }
                if (timerToUpdateProgram) {
                    host.clearTimeout(timerToUpdateProgram);
                }
                writeLog("Scheduling update");
                timerToUpdateProgram = host.setTimeout(updateProgram, 250);
            }
            function scheduleProgramReload() {
                ts.Debug.assert(!!configFileName);
                reloadLevel = ts.ConfigFileProgramReloadLevel.Full;
                scheduleProgramUpdate();
            }
            function updateProgram() {
                timerToUpdateProgram = undefined;
                reportWatchDiagnostic(ts.Diagnostics.File_change_detected_Starting_incremental_compilation);
                switch (reloadLevel) {
                    case ts.ConfigFileProgramReloadLevel.Partial:
                        ts.perfLogger.logStartUpdateProgram("PartialConfigReload");
                        reloadFileNamesFromConfigFile();
                        break;
                    case ts.ConfigFileProgramReloadLevel.Full:
                        ts.perfLogger.logStartUpdateProgram("FullConfigReload");
                        reloadConfigFile();
                        break;
                    default:
                        ts.perfLogger.logStartUpdateProgram("SynchronizeProgram");
                        synchronizeProgram();
                        break;
                }
                ts.perfLogger.logStopUpdateProgram("Done");
            }
            function reloadFileNamesFromConfigFile() {
                writeLog("Reloading new file names and options");
                var result = ts.getFileNamesFromConfigSpecs(configFileSpecs, ts.getNormalizedAbsolutePath(ts.getDirectoryPath(configFileName), currentDirectory), compilerOptions, parseConfigFileHost);
                if (ts.updateErrorForNoInputFiles(result, ts.getNormalizedAbsolutePath(configFileName, currentDirectory), configFileSpecs, configFileParsingDiagnostics, canConfigFileJsonReportNoInputFiles)) {
                    hasChangedConfigFileParsingErrors = true;
                }
                rootFileNames = result.fileNames;
                // Update the program
                synchronizeProgram();
            }
            function reloadConfigFile() {
                writeLog("Reloading config file: " + configFileName);
                reloadLevel = ts.ConfigFileProgramReloadLevel.None;
                if (cachedDirectoryStructureHost) {
                    cachedDirectoryStructureHost.clearCache();
                }
                parseConfigFile();
                hasChangedCompilerOptions = true;
                synchronizeProgram();
                // Update the wild card directory watch
                watchConfigFileWildCardDirectories();
            }
            function parseConfigFile() {
                setConfigFileParsingResult(ts.getParsedCommandLineOfConfigFile(configFileName, optionsToExtendForConfigFile, parseConfigFileHost, /*extendedConfigCache*/ undefined, watchOptionsToExtend)); // TODO: GH#18217
            }
            function setConfigFileParsingResult(configFileParseResult) {
                rootFileNames = configFileParseResult.fileNames;
                compilerOptions = configFileParseResult.options;
                watchOptions = configFileParseResult.watchOptions;
                configFileSpecs = configFileParseResult.configFileSpecs; // TODO: GH#18217
                projectReferences = configFileParseResult.projectReferences;
                configFileParsingDiagnostics = ts.getConfigFileParsingDiagnostics(configFileParseResult).slice();
                canConfigFileJsonReportNoInputFiles = ts.canJsonReportNoInutFiles(configFileParseResult.raw);
                hasChangedConfigFileParsingErrors = true;
            }
            function onSourceFileChange(fileName, eventKind, path) {
                updateCachedSystemWithFile(fileName, path, eventKind);
                // Update the source file cache
                if (eventKind === ts.FileWatcherEventKind.Deleted && sourceFilesCache.has(path)) {
                    resolutionCache.invalidateResolutionOfFile(path);
                }
                resolutionCache.removeResolutionsFromProjectReferenceRedirects(path);
                nextSourceFileVersion(path);
                // Update the program
                scheduleProgramUpdate();
            }
            function updateCachedSystemWithFile(fileName, path, eventKind) {
                if (cachedDirectoryStructureHost) {
                    cachedDirectoryStructureHost.addOrDeleteFile(fileName, path, eventKind);
                }
            }
            function watchMissingFilePath(missingFilePath) {
                return watchFilePath(host, missingFilePath, onMissingFileChange, ts.PollingInterval.Medium, watchOptions, missingFilePath, ts.WatchType.MissingFile);
            }
            function onMissingFileChange(fileName, eventKind, missingFilePath) {
                updateCachedSystemWithFile(fileName, missingFilePath, eventKind);
                if (eventKind === ts.FileWatcherEventKind.Created && missingFilesMap.has(missingFilePath)) {
                    missingFilesMap.get(missingFilePath).close();
                    missingFilesMap.delete(missingFilePath);
                    // Delete the entry in the source files cache so that new source file is created
                    nextSourceFileVersion(missingFilePath);
                    // When a missing file is created, we should update the graph.
                    scheduleProgramUpdate();
                }
            }
            function watchConfigFileWildCardDirectories() {
                if (configFileSpecs) {
                    ts.updateWatchingWildcardDirectories(watchedWildcardDirectories || (watchedWildcardDirectories = ts.createMap()), ts.createMapFromTemplate(configFileSpecs.wildcardDirectories), watchWildcardDirectory);
                }
                else if (watchedWildcardDirectories) {
                    ts.clearMap(watchedWildcardDirectories, ts.closeFileWatcherOf);
                }
            }
            function watchWildcardDirectory(directory, flags) {
                return watchDirectory(host, directory, function (fileOrDirectory) {
                    ts.Debug.assert(!!configFileName);
                    var fileOrDirectoryPath = toPath(fileOrDirectory);
                    // Since the file existence changed, update the sourceFiles cache
                    if (cachedDirectoryStructureHost) {
                        cachedDirectoryStructureHost.addOrDeleteFileOrDirectory(fileOrDirectory, fileOrDirectoryPath);
                    }
                    nextSourceFileVersion(fileOrDirectoryPath);
                    fileOrDirectoryPath = ts.removeIgnoredPath(fileOrDirectoryPath);
                    if (!fileOrDirectoryPath)
                        return;
                    // If the the added or created file or directory is not supported file name, ignore the file
                    // But when watched directory is added/removed, we need to reload the file list
                    if (fileOrDirectoryPath !== directory && ts.hasExtension(fileOrDirectoryPath) && !ts.isSupportedSourceFileName(fileOrDirectory, compilerOptions)) {
                        writeLog("Project: " + configFileName + " Detected file add/remove of non supported extension: " + fileOrDirectory);
                        return;
                    }
                    // Reload is pending, do the reload
                    if (reloadLevel !== ts.ConfigFileProgramReloadLevel.Full) {
                        reloadLevel = ts.ConfigFileProgramReloadLevel.Partial;
                        // Schedule Update the program
                        scheduleProgramUpdate();
                    }
                }, flags, watchOptions, ts.WatchType.WildcardDirectory);
            }
        }
        ts.createWatchProgram = createWatchProgram;
    })(ts || (ts = {}));
    /*@internal*/
    var ts;
    (function (ts) {
        var UpToDateStatusType;
        (function (UpToDateStatusType) {
            UpToDateStatusType[UpToDateStatusType["Unbuildable"] = 0] = "Unbuildable";
            UpToDateStatusType[UpToDateStatusType["UpToDate"] = 1] = "UpToDate";
            /**
             * The project appears out of date because its upstream inputs are newer than its outputs,
             * but all of its outputs are actually newer than the previous identical outputs of its (.d.ts) inputs.
             * This means we can Pseudo-build (just touch timestamps), as if we had actually built this project.
             */
            UpToDateStatusType[UpToDateStatusType["UpToDateWithUpstreamTypes"] = 2] = "UpToDateWithUpstreamTypes";
            /**
             * The project appears out of date because its upstream inputs are newer than its outputs,
             * but all of its outputs are actually newer than the previous identical outputs of its (.d.ts) inputs.
             * This means we can Pseudo-build (just manipulate outputs), as if we had actually built this project.
             */
            UpToDateStatusType[UpToDateStatusType["OutOfDateWithPrepend"] = 3] = "OutOfDateWithPrepend";
            UpToDateStatusType[UpToDateStatusType["OutputMissing"] = 4] = "OutputMissing";
            UpToDateStatusType[UpToDateStatusType["OutOfDateWithSelf"] = 5] = "OutOfDateWithSelf";
            UpToDateStatusType[UpToDateStatusType["OutOfDateWithUpstream"] = 6] = "OutOfDateWithUpstream";
            UpToDateStatusType[UpToDateStatusType["UpstreamOutOfDate"] = 7] = "UpstreamOutOfDate";
            UpToDateStatusType[UpToDateStatusType["UpstreamBlocked"] = 8] = "UpstreamBlocked";
            UpToDateStatusType[UpToDateStatusType["ComputingUpstream"] = 9] = "ComputingUpstream";
            UpToDateStatusType[UpToDateStatusType["TsVersionOutputOfDate"] = 10] = "TsVersionOutputOfDate";
            /**
             * Projects with no outputs (i.e. "solution" files)
             */
            UpToDateStatusType[UpToDateStatusType["ContainerOnly"] = 11] = "ContainerOnly";
        })(UpToDateStatusType = ts.UpToDateStatusType || (ts.UpToDateStatusType = {}));
        function resolveConfigFileProjectName(project) {
            if (ts.fileExtensionIs(project, ".json" /* Json */)) {
                return project;
            }
            return ts.combinePaths(project, "tsconfig.json");
        }
        ts.resolveConfigFileProjectName = resolveConfigFileProjectName;
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        var minimumDate = new Date(-8640000000000000);
        var maximumDate = new Date(8640000000000000);
        var BuildResultFlags;
        (function (BuildResultFlags) {
            BuildResultFlags[BuildResultFlags["None"] = 0] = "None";
            /**
             * No errors of any kind occurred during build
             */
            BuildResultFlags[BuildResultFlags["Success"] = 1] = "Success";
            /**
             * None of the .d.ts files emitted by this build were
             * different from the existing files on disk
             */
            BuildResultFlags[BuildResultFlags["DeclarationOutputUnchanged"] = 2] = "DeclarationOutputUnchanged";
            BuildResultFlags[BuildResultFlags["ConfigFileErrors"] = 4] = "ConfigFileErrors";
            BuildResultFlags[BuildResultFlags["SyntaxErrors"] = 8] = "SyntaxErrors";
            BuildResultFlags[BuildResultFlags["TypeErrors"] = 16] = "TypeErrors";
            BuildResultFlags[BuildResultFlags["DeclarationEmitErrors"] = 32] = "DeclarationEmitErrors";
            BuildResultFlags[BuildResultFlags["EmitErrors"] = 64] = "EmitErrors";
            BuildResultFlags[BuildResultFlags["AnyErrors"] = 124] = "AnyErrors";
        })(BuildResultFlags || (BuildResultFlags = {}));
        function createConfigFileMap() {
            return ts.createMap();
        }
        function getOrCreateValueFromConfigFileMap(configFileMap, resolved, createT) {
            var existingValue = configFileMap.get(resolved);
            var newValue;
            if (!existingValue) {
                newValue = createT();
                configFileMap.set(resolved, newValue);
            }
            return existingValue || newValue;
        }
        function getOrCreateValueMapFromConfigFileMap(configFileMap, resolved) {
            return getOrCreateValueFromConfigFileMap(configFileMap, resolved, ts.createMap);
        }
        function newer(date1, date2) {
            return date2 > date1 ? date2 : date1;
        }
        function isDeclarationFile(fileName) {
            return ts.fileExtensionIs(fileName, ".d.ts" /* Dts */);
        }
        /*@internal*/
        function isCircularBuildOrder(buildOrder) {
            return !!buildOrder && !!buildOrder.buildOrder;
        }
        ts.isCircularBuildOrder = isCircularBuildOrder;
        /*@internal*/
        function getBuildOrderFromAnyBuildOrder(anyBuildOrder) {
            return isCircularBuildOrder(anyBuildOrder) ? anyBuildOrder.buildOrder : anyBuildOrder;
        }
        ts.getBuildOrderFromAnyBuildOrder = getBuildOrderFromAnyBuildOrder;
        /**
         * Create a function that reports watch status by writing to the system and handles the formating of the diagnostic
         */
        function createBuilderStatusReporter(system, pretty) {
            return function (diagnostic) {
                var output = pretty ? "[" + ts.formatColorAndReset(ts.getLocaleTimeString(system), ts.ForegroundColorEscapeSequences.Grey) + "] " : ts.getLocaleTimeString(system) + " - ";
                output += "" + ts.flattenDiagnosticMessageText(diagnostic.messageText, system.newLine) + (system.newLine + system.newLine);
                system.write(output);
            };
        }
        ts.createBuilderStatusReporter = createBuilderStatusReporter;
        function createSolutionBuilderHostBase(system, createProgram, reportDiagnostic, reportSolutionBuilderStatus) {
            var host = ts.createProgramHost(system, createProgram);
            host.getModifiedTime = system.getModifiedTime ? function (path) { return system.getModifiedTime(path); } : ts.returnUndefined;
            host.setModifiedTime = system.setModifiedTime ? function (path, date) { return system.setModifiedTime(path, date); } : ts.noop;
            host.deleteFile = system.deleteFile ? function (path) { return system.deleteFile(path); } : ts.noop;
            host.reportDiagnostic = reportDiagnostic || ts.createDiagnosticReporter(system);
            host.reportSolutionBuilderStatus = reportSolutionBuilderStatus || createBuilderStatusReporter(system);
            host.now = ts.maybeBind(system, system.now); // For testing
            return host;
        }
        function createSolutionBuilderHost(system, createProgram, reportDiagnostic, reportSolutionBuilderStatus, reportErrorSummary) {
            if (system === void 0) {
                system = ts.sys;
            }
            var host = createSolutionBuilderHostBase(system, createProgram, reportDiagnostic, reportSolutionBuilderStatus);
            host.reportErrorSummary = reportErrorSummary;
            return host;
        }
        ts.createSolutionBuilderHost = createSolutionBuilderHost;
        function createSolutionBuilderWithWatchHost(system, createProgram, reportDiagnostic, reportSolutionBuilderStatus, reportWatchStatus) {
            if (system === void 0) {
                system = ts.sys;
            }
            var host = createSolutionBuilderHostBase(system, createProgram, reportDiagnostic, reportSolutionBuilderStatus);
            var watchHost = ts.createWatchHost(system, reportWatchStatus);
            ts.copyProperties(host, watchHost);
            return host;
        }
        ts.createSolutionBuilderWithWatchHost = createSolutionBuilderWithWatchHost;
        function getCompilerOptionsOfBuildOptions(buildOptions) {
            var result = {};
            ts.commonOptionsWithBuild.forEach(function (option) {
                if (ts.hasProperty(buildOptions, option.name))
                    result[option.name] = buildOptions[option.name];
            });
            return result;
        }
        function createSolutionBuilder(host, rootNames, defaultOptions) {
            return createSolutionBuilderWorker(/*watch*/ false, host, rootNames, defaultOptions);
        }
        ts.createSolutionBuilder = createSolutionBuilder;
        function createSolutionBuilderWithWatch(host, rootNames, defaultOptions, baseWatchOptions) {
            return createSolutionBuilderWorker(/*watch*/ true, host, rootNames, defaultOptions, baseWatchOptions);
        }
        ts.createSolutionBuilderWithWatch = createSolutionBuilderWithWatch;
        function createSolutionBuilderState(watch, hostOrHostWithWatch, rootNames, options, baseWatchOptions) {
            var host = hostOrHostWithWatch;
            var hostWithWatch = hostOrHostWithWatch;
            var currentDirectory = host.getCurrentDirectory();
            var getCanonicalFileName = ts.createGetCanonicalFileName(host.useCaseSensitiveFileNames());
            // State of the solution
            var baseCompilerOptions = getCompilerOptionsOfBuildOptions(options);
            var compilerHost = ts.createCompilerHostFromProgramHost(host, function () { return state.projectCompilerOptions; });
            ts.setGetSourceFileAsHashVersioned(compilerHost, host);
            compilerHost.getParsedCommandLine = function (fileName) { return parseConfigFile(state, fileName, toResolvedConfigFilePath(state, fileName)); };
            compilerHost.resolveModuleNames = ts.maybeBind(host, host.resolveModuleNames);
            compilerHost.resolveTypeReferenceDirectives = ts.maybeBind(host, host.resolveTypeReferenceDirectives);
            var moduleResolutionCache = !compilerHost.resolveModuleNames ? ts.createModuleResolutionCache(currentDirectory, getCanonicalFileName) : undefined;
            if (!compilerHost.resolveModuleNames) {
                var loader_3 = function (moduleName, containingFile, redirectedReference) { return ts.resolveModuleName(moduleName, containingFile, state.projectCompilerOptions, compilerHost, moduleResolutionCache, redirectedReference).resolvedModule; };
                compilerHost.resolveModuleNames = function (moduleNames, containingFile, _reusedNames, redirectedReference) {
                    return ts.loadWithLocalCache(ts.Debug.assertEachDefined(moduleNames), containingFile, redirectedReference, loader_3);
                };
            }
            var _a = ts.createWatchFactory(hostWithWatch, options), watchFile = _a.watchFile, watchFilePath = _a.watchFilePath, watchDirectory = _a.watchDirectory, writeLog = _a.writeLog;
            var state = {
                host: host,
                hostWithWatch: hostWithWatch,
                currentDirectory: currentDirectory,
                getCanonicalFileName: getCanonicalFileName,
                parseConfigFileHost: ts.parseConfigHostFromCompilerHostLike(host),
                writeFileName: host.trace ? function (s) { return host.trace(s); } : undefined,
                // State of solution
                options: options,
                baseCompilerOptions: baseCompilerOptions,
                rootNames: rootNames,
                baseWatchOptions: baseWatchOptions,
                resolvedConfigFilePaths: ts.createMap(),
                configFileCache: createConfigFileMap(),
                projectStatus: createConfigFileMap(),
                buildInfoChecked: createConfigFileMap(),
                extendedConfigCache: ts.createMap(),
                builderPrograms: createConfigFileMap(),
                diagnostics: createConfigFileMap(),
                projectPendingBuild: createConfigFileMap(),
                projectErrorsReported: createConfigFileMap(),
                compilerHost: compilerHost,
                moduleResolutionCache: moduleResolutionCache,
                // Mutable state
                buildOrder: undefined,
                readFileWithCache: function (f) { return host.readFile(f); },
                projectCompilerOptions: baseCompilerOptions,
                cache: undefined,
                allProjectBuildPending: true,
                needsSummary: true,
                watchAllProjectsPending: watch,
                currentInvalidatedProject: undefined,
                // Watch state
                watch: watch,
                allWatchedWildcardDirectories: createConfigFileMap(),
                allWatchedInputFiles: createConfigFileMap(),
                allWatchedConfigFiles: createConfigFileMap(),
                timerToBuildInvalidatedProject: undefined,
                reportFileChangeDetected: false,
                watchFile: watchFile,
                watchFilePath: watchFilePath,
                watchDirectory: watchDirectory,
                writeLog: writeLog,
            };
            return state;
        }
        function toPath(state, fileName) {
            return ts.toPath(fileName, state.currentDirectory, state.getCanonicalFileName);
        }
        function toResolvedConfigFilePath(state, fileName) {
            var resolvedConfigFilePaths = state.resolvedConfigFilePaths;
            var path = resolvedConfigFilePaths.get(fileName);
            if (path !== undefined)
                return path;
            var resolvedPath = toPath(state, fileName);
            resolvedConfigFilePaths.set(fileName, resolvedPath);
            return resolvedPath;
        }
        function isParsedCommandLine(entry) {
            return !!entry.options;
        }
        function parseConfigFile(state, configFileName, configFilePath) {
            var configFileCache = state.configFileCache;
            var value = configFileCache.get(configFilePath);
            if (value) {
                return isParsedCommandLine(value) ? value : undefined;
            }
            var diagnostic;
            var parseConfigFileHost = state.parseConfigFileHost, baseCompilerOptions = state.baseCompilerOptions, baseWatchOptions = state.baseWatchOptions, extendedConfigCache = state.extendedConfigCache, host = state.host;
            var parsed;
            if (host.getParsedCommandLine) {
                parsed = host.getParsedCommandLine(configFileName);
                if (!parsed)
                    diagnostic = ts.createCompilerDiagnostic(ts.Diagnostics.File_0_not_found, configFileName);
            }
            else {
                parseConfigFileHost.onUnRecoverableConfigFileDiagnostic = function (d) { return diagnostic = d; };
                parsed = ts.getParsedCommandLineOfConfigFile(configFileName, baseCompilerOptions, parseConfigFileHost, extendedConfigCache, baseWatchOptions);
                parseConfigFileHost.onUnRecoverableConfigFileDiagnostic = ts.noop;
            }
            configFileCache.set(configFilePath, parsed || diagnostic);
            return parsed;
        }
        function resolveProjectName(state, name) {
            return ts.resolveConfigFileProjectName(ts.resolvePath(state.currentDirectory, name));
        }
        function createBuildOrder(state, roots) {
            var temporaryMarks = ts.createMap();
            var permanentMarks = ts.createMap();
            var circularityReportStack = [];
            var buildOrder;
            var circularDiagnostics;
            for (var _i = 0, roots_1 = roots; _i < roots_1.length; _i++) {
                var root = roots_1[_i];
                visit(root);
            }
            return circularDiagnostics ?
                { buildOrder: buildOrder || ts.emptyArray, circularDiagnostics: circularDiagnostics } :
                buildOrder || ts.emptyArray;
            function visit(configFileName, inCircularContext) {
                var projPath = toResolvedConfigFilePath(state, configFileName);
                // Already visited
                if (permanentMarks.has(projPath))
                    return;
                // Circular
                if (temporaryMarks.has(projPath)) {
                    if (!inCircularContext) {
                        (circularDiagnostics || (circularDiagnostics = [])).push(ts.createCompilerDiagnostic(ts.Diagnostics.Project_references_may_not_form_a_circular_graph_Cycle_detected_Colon_0, circularityReportStack.join("\r\n")));
                    }
                    return;
                }
                temporaryMarks.set(projPath, true);
                circularityReportStack.push(configFileName);
                var parsed = parseConfigFile(state, configFileName, projPath);
                if (parsed && parsed.projectReferences) {
                    for (var _i = 0, _a = parsed.projectReferences; _i < _a.length; _i++) {
                        var ref = _a[_i];
                        var resolvedRefPath = resolveProjectName(state, ref.path);
                        visit(resolvedRefPath, inCircularContext || ref.circular);
                    }
                }
                circularityReportStack.pop();
                permanentMarks.set(projPath, true);
                (buildOrder || (buildOrder = [])).push(configFileName);
            }
        }
        function getBuildOrder(state) {
            return state.buildOrder || createStateBuildOrder(state);
        }
        function createStateBuildOrder(state) {
            var buildOrder = createBuildOrder(state, state.rootNames.map(function (f) { return resolveProjectName(state, f); }));
            // Clear all to ResolvedConfigFilePaths cache to start fresh
            state.resolvedConfigFilePaths.clear();
            var currentProjects = ts.arrayToSet(getBuildOrderFromAnyBuildOrder(buildOrder), function (resolved) { return toResolvedConfigFilePath(state, resolved); });
            var noopOnDelete = { onDeleteValue: ts.noop };
            // Config file cache
            ts.mutateMapSkippingNewValues(state.configFileCache, currentProjects, noopOnDelete);
            ts.mutateMapSkippingNewValues(state.projectStatus, currentProjects, noopOnDelete);
            ts.mutateMapSkippingNewValues(state.buildInfoChecked, currentProjects, noopOnDelete);
            ts.mutateMapSkippingNewValues(state.builderPrograms, currentProjects, noopOnDelete);
            ts.mutateMapSkippingNewValues(state.diagnostics, currentProjects, noopOnDelete);
            ts.mutateMapSkippingNewValues(state.projectPendingBuild, currentProjects, noopOnDelete);
            ts.mutateMapSkippingNewValues(state.projectErrorsReported, currentProjects, noopOnDelete);
            // Remove watches for the program no longer in the solution
            if (state.watch) {
                ts.mutateMapSkippingNewValues(state.allWatchedConfigFiles, currentProjects, { onDeleteValue: ts.closeFileWatcher });
                ts.mutateMapSkippingNewValues(state.allWatchedWildcardDirectories, currentProjects, { onDeleteValue: function (existingMap) { return existingMap.forEach(ts.closeFileWatcherOf); } });
                ts.mutateMapSkippingNewValues(state.allWatchedInputFiles, currentProjects, { onDeleteValue: function (existingMap) { return existingMap.forEach(ts.closeFileWatcher); } });
            }
            return state.buildOrder = buildOrder;
        }
        function getBuildOrderFor(state, project, onlyReferences) {
            var resolvedProject = project && resolveProjectName(state, project);
            var buildOrderFromState = getBuildOrder(state);
            if (isCircularBuildOrder(buildOrderFromState))
                return buildOrderFromState;
            if (resolvedProject) {
                var projectPath_1 = toResolvedConfigFilePath(state, resolvedProject);
                var projectIndex = ts.findIndex(buildOrderFromState, function (configFileName) { return toResolvedConfigFilePath(state, configFileName) === projectPath_1; });
                if (projectIndex === -1)
                    return undefined;
            }
            var buildOrder = resolvedProject ? createBuildOrder(state, [resolvedProject]) : buildOrderFromState;
            ts.Debug.assert(!isCircularBuildOrder(buildOrder));
            ts.Debug.assert(!onlyReferences || resolvedProject !== undefined);
            ts.Debug.assert(!onlyReferences || buildOrder[buildOrder.length - 1] === resolvedProject);
            return onlyReferences ? buildOrder.slice(0, buildOrder.length - 1) : buildOrder;
        }
        function enableCache(state) {
            if (state.cache) {
                disableCache(state);
            }
            var compilerHost = state.compilerHost, host = state.host;
            var originalReadFileWithCache = state.readFileWithCache;
            var originalGetSourceFile = compilerHost.getSourceFile;
            var _a = ts.changeCompilerHostLikeToUseCache(host, function (fileName) { return toPath(state, fileName); }, function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return originalGetSourceFile.call.apply(originalGetSourceFile, __spreadArrays([compilerHost], args));
            }), originalReadFile = _a.originalReadFile, originalFileExists = _a.originalFileExists, originalDirectoryExists = _a.originalDirectoryExists, originalCreateDirectory = _a.originalCreateDirectory, originalWriteFile = _a.originalWriteFile, getSourceFileWithCache = _a.getSourceFileWithCache, readFileWithCache = _a.readFileWithCache;
            state.readFileWithCache = readFileWithCache;
            compilerHost.getSourceFile = getSourceFileWithCache;
            state.cache = {
                originalReadFile: originalReadFile,
                originalFileExists: originalFileExists,
                originalDirectoryExists: originalDirectoryExists,
                originalCreateDirectory: originalCreateDirectory,
                originalWriteFile: originalWriteFile,
                originalReadFileWithCache: originalReadFileWithCache,
                originalGetSourceFile: originalGetSourceFile,
            };
        }
        function disableCache(state) {
            if (!state.cache)
                return;
            var cache = state.cache, host = state.host, compilerHost = state.compilerHost, extendedConfigCache = state.extendedConfigCache, moduleResolutionCache = state.moduleResolutionCache;
            host.readFile = cache.originalReadFile;
            host.fileExists = cache.originalFileExists;
            host.directoryExists = cache.originalDirectoryExists;
            host.createDirectory = cache.originalCreateDirectory;
            host.writeFile = cache.originalWriteFile;
            compilerHost.getSourceFile = cache.originalGetSourceFile;
            state.readFileWithCache = cache.originalReadFileWithCache;
            extendedConfigCache.clear();
            if (moduleResolutionCache) {
                moduleResolutionCache.directoryToModuleNameMap.clear();
                moduleResolutionCache.moduleNameToDirectoryMap.clear();
            }
            state.cache = undefined;
        }
        function clearProjectStatus(state, resolved) {
            state.projectStatus.delete(resolved);
            state.diagnostics.delete(resolved);
        }
        function addProjToQueue(_a, proj, reloadLevel) {
            var projectPendingBuild = _a.projectPendingBuild;
            var value = projectPendingBuild.get(proj);
            if (value === undefined) {
                projectPendingBuild.set(proj, reloadLevel);
            }
            else if (value < reloadLevel) {
                projectPendingBuild.set(proj, reloadLevel);
            }
        }
        function setupInitialBuild(state, cancellationToken) {
            // Set initial build if not already built
            if (!state.allProjectBuildPending)
                return;
            state.allProjectBuildPending = false;
            if (state.options.watch) {
                reportWatchStatus(state, ts.Diagnostics.Starting_compilation_in_watch_mode);
            }
            enableCache(state);
            var buildOrder = getBuildOrderFromAnyBuildOrder(getBuildOrder(state));
            buildOrder.forEach(function (configFileName) {
                return state.projectPendingBuild.set(toResolvedConfigFilePath(state, configFileName), ts.ConfigFileProgramReloadLevel.None);
            });
            if (cancellationToken) {
                cancellationToken.throwIfCancellationRequested();
            }
        }
        var InvalidatedProjectKind;
        (function (InvalidatedProjectKind) {
            InvalidatedProjectKind[InvalidatedProjectKind["Build"] = 0] = "Build";
            InvalidatedProjectKind[InvalidatedProjectKind["UpdateBundle"] = 1] = "UpdateBundle";
            InvalidatedProjectKind[InvalidatedProjectKind["UpdateOutputFileStamps"] = 2] = "UpdateOutputFileStamps";
        })(InvalidatedProjectKind = ts.InvalidatedProjectKind || (ts.InvalidatedProjectKind = {}));
        function doneInvalidatedProject(state, projectPath) {
            state.projectPendingBuild.delete(projectPath);
            state.currentInvalidatedProject = undefined;
            return state.diagnostics.has(projectPath) ?
                ts.ExitStatus.DiagnosticsPresent_OutputsSkipped :
                ts.ExitStatus.Success;
        }
        function createUpdateOutputFileStampsProject(state, project, projectPath, config, buildOrder) {
            var updateOutputFileStampsPending = true;
            return {
                kind: InvalidatedProjectKind.UpdateOutputFileStamps,
                project: project,
                projectPath: projectPath,
                buildOrder: buildOrder,
                getCompilerOptions: function () { return config.options; },
                getCurrentDirectory: function () { return state.currentDirectory; },
                updateOutputFileStatmps: function () {
                    updateOutputTimestamps(state, config, projectPath);
                    updateOutputFileStampsPending = false;
                },
                done: function () {
                    if (updateOutputFileStampsPending) {
                        updateOutputTimestamps(state, config, projectPath);
                    }
                    return doneInvalidatedProject(state, projectPath);
                }
            };
        }
        function createBuildOrUpdateInvalidedProject(kind, state, project, projectPath, projectIndex, config, buildOrder) {
            var Step;
            (function (Step) {
                Step[Step["CreateProgram"] = 0] = "CreateProgram";
                Step[Step["SyntaxDiagnostics"] = 1] = "SyntaxDiagnostics";
                Step[Step["SemanticDiagnostics"] = 2] = "SemanticDiagnostics";
                Step[Step["Emit"] = 3] = "Emit";
                Step[Step["EmitBundle"] = 4] = "EmitBundle";
                Step[Step["BuildInvalidatedProjectOfBundle"] = 5] = "BuildInvalidatedProjectOfBundle";
                Step[Step["QueueReferencingProjects"] = 6] = "QueueReferencingProjects";
                Step[Step["Done"] = 7] = "Done";
            })(Step || (Step = {}));
            var step = kind === InvalidatedProjectKind.Build ? Step.CreateProgram : Step.EmitBundle;
            var program;
            var buildResult;
            var invalidatedProjectOfBundle;
            return kind === InvalidatedProjectKind.Build ?
                {
                    kind: kind,
                    project: project,
                    projectPath: projectPath,
                    buildOrder: buildOrder,
                    getCompilerOptions: function () { return config.options; },
                    getCurrentDirectory: function () { return state.currentDirectory; },
                    getBuilderProgram: function () { return withProgramOrUndefined(ts.identity); },
                    getProgram: function () {
                        return withProgramOrUndefined(function (program) { return program.getProgramOrUndefined(); });
                    },
                    getSourceFile: function (fileName) {
                        return withProgramOrUndefined(function (program) { return program.getSourceFile(fileName); });
                    },
                    getSourceFiles: function () {
                        return withProgramOrEmptyArray(function (program) { return program.getSourceFiles(); });
                    },
                    getOptionsDiagnostics: function (cancellationToken) {
                        return withProgramOrEmptyArray(function (program) { return program.getOptionsDiagnostics(cancellationToken); });
                    },
                    getGlobalDiagnostics: function (cancellationToken) {
                        return withProgramOrEmptyArray(function (program) { return program.getGlobalDiagnostics(cancellationToken); });
                    },
                    getConfigFileParsingDiagnostics: function () {
                        return withProgramOrEmptyArray(function (program) { return program.getConfigFileParsingDiagnostics(); });
                    },
                    getSyntacticDiagnostics: function (sourceFile, cancellationToken) {
                        return withProgramOrEmptyArray(function (program) { return program.getSyntacticDiagnostics(sourceFile, cancellationToken); });
                    },
                    getAllDependencies: function (sourceFile) {
                        return withProgramOrEmptyArray(function (program) { return program.getAllDependencies(sourceFile); });
                    },
                    getSemanticDiagnostics: function (sourceFile, cancellationToken) {
                        return withProgramOrEmptyArray(function (program) { return program.getSemanticDiagnostics(sourceFile, cancellationToken); });
                    },
                    getSemanticDiagnosticsOfNextAffectedFile: function (cancellationToken, ignoreSourceFile) {
                        return withProgramOrUndefined(function (program) {
                            return (program.getSemanticDiagnosticsOfNextAffectedFile) &&
                                program.getSemanticDiagnosticsOfNextAffectedFile(cancellationToken, ignoreSourceFile);
                        });
                    },
                    emit: function (targetSourceFile, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers) {
                        if (targetSourceFile || emitOnlyDtsFiles) {
                            return withProgramOrUndefined(function (program) { return program.emit(targetSourceFile, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers); });
                        }
                        executeSteps(Step.SemanticDiagnostics, cancellationToken);
                        if (step !== Step.Emit)
                            return undefined;
                        return emit(writeFile, cancellationToken, customTransformers);
                    },
                    done: done
                } :
                {
                    kind: kind,
                    project: project,
                    projectPath: projectPath,
                    buildOrder: buildOrder,
                    getCompilerOptions: function () { return config.options; },
                    getCurrentDirectory: function () { return state.currentDirectory; },
                    emit: function (writeFile, customTransformers) {
                        if (step !== Step.EmitBundle)
                            return invalidatedProjectOfBundle;
                        return emitBundle(writeFile, customTransformers);
                    },
                    done: done,
                };
            function done(cancellationToken, writeFile, customTransformers) {
                executeSteps(Step.Done, cancellationToken, writeFile, customTransformers);
                return doneInvalidatedProject(state, projectPath);
            }
            function withProgramOrUndefined(action) {
                executeSteps(Step.CreateProgram);
                return program && action(program);
            }
            function withProgramOrEmptyArray(action) {
                return withProgramOrUndefined(action) || ts.emptyArray;
            }
            function createProgram() {
                ts.Debug.assert(program === undefined);
                if (state.options.dry) {
                    reportStatus(state, ts.Diagnostics.A_non_dry_build_would_build_project_0, project);
                    buildResult = BuildResultFlags.Success;
                    step = Step.QueueReferencingProjects;
                    return;
                }
                if (state.options.verbose)
                    reportStatus(state, ts.Diagnostics.Building_project_0, project);
                if (config.fileNames.length === 0) {
                    reportAndStoreErrors(state, projectPath, ts.getConfigFileParsingDiagnostics(config));
                    // Nothing to build - must be a solution file, basically
                    buildResult = BuildResultFlags.None;
                    step = Step.QueueReferencingProjects;
                    return;
                }
                var host = state.host, compilerHost = state.compilerHost;
                state.projectCompilerOptions = config.options;
                // Update module resolution cache if needed
                updateModuleResolutionCache(state, project, config);
                // Create program
                program = host.createProgram(config.fileNames, config.options, compilerHost, getOldProgram(state, projectPath, config), ts.getConfigFileParsingDiagnostics(config), config.projectReferences);
                step++;
            }
            function handleDiagnostics(diagnostics, errorFlags, errorType) {
                if (diagnostics.length) {
                    buildResult = buildErrors(state, projectPath, program, config, diagnostics, errorFlags, errorType);
                    step = Step.QueueReferencingProjects;
                }
                else {
                    step++;
                }
            }
            function getSyntaxDiagnostics(cancellationToken) {
                ts.Debug.assertDefined(program);
                handleDiagnostics(__spreadArrays(program.getConfigFileParsingDiagnostics(), program.getOptionsDiagnostics(cancellationToken), program.getGlobalDiagnostics(cancellationToken), program.getSyntacticDiagnostics(/*sourceFile*/ undefined, cancellationToken)), BuildResultFlags.SyntaxErrors, "Syntactic");
            }
            function getSemanticDiagnostics(cancellationToken) {
                handleDiagnostics(ts.Debug.assertDefined(program).getSemanticDiagnostics(/*sourceFile*/ undefined, cancellationToken), BuildResultFlags.TypeErrors, "Semantic");
            }
            function emit(writeFileCallback, cancellationToken, customTransformers) {
                ts.Debug.assertDefined(program);
                ts.Debug.assert(step === Step.Emit);
                // Before emitting lets backup state, so we can revert it back if there are declaration errors to handle emit and declaration errors correctly
                program.backupState();
                var declDiagnostics;
                var reportDeclarationDiagnostics = function (d) { return (declDiagnostics || (declDiagnostics = [])).push(d); };
                var outputFiles = [];
                var emitResult = ts.emitFilesAndReportErrors(program, reportDeclarationDiagnostics, 
                /*writeFileName*/ undefined, 
                /*reportSummary*/ undefined, function (name, text, writeByteOrderMark) { return outputFiles.push({ name: name, text: text, writeByteOrderMark: writeByteOrderMark }); }, cancellationToken, 
                /*emitOnlyDts*/ false, customTransformers).emitResult;
                // Don't emit .d.ts if there are decl file errors
                if (declDiagnostics) {
                    program.restoreState();
                    buildResult = buildErrors(state, projectPath, program, config, declDiagnostics, BuildResultFlags.DeclarationEmitErrors, "Declaration file");
                    step = Step.QueueReferencingProjects;
                    return {
                        emitSkipped: true,
                        diagnostics: emitResult.diagnostics
                    };
                }
                // Actual Emit
                var host = state.host, compilerHost = state.compilerHost;
                var resultFlags = BuildResultFlags.DeclarationOutputUnchanged;
                var newestDeclarationFileContentChangedTime = minimumDate;
                var anyDtsChanged = false;
                var emitterDiagnostics = ts.createDiagnosticCollection();
                var emittedOutputs = ts.createMap();
                outputFiles.forEach(function (_a) {
                    var name = _a.name, text = _a.text, writeByteOrderMark = _a.writeByteOrderMark;
                    var priorChangeTime;
                    if (!anyDtsChanged && isDeclarationFile(name)) {
                        // Check for unchanged .d.ts files
                        if (host.fileExists(name) && state.readFileWithCache(name) === text) {
                            priorChangeTime = host.getModifiedTime(name);
                        }
                        else {
                            resultFlags &= ~BuildResultFlags.DeclarationOutputUnchanged;
                            anyDtsChanged = true;
                        }
                    }
                    emittedOutputs.set(toPath(state, name), name);
                    ts.writeFile(writeFileCallback ? { writeFile: writeFileCallback } : compilerHost, emitterDiagnostics, name, text, writeByteOrderMark);
                    if (priorChangeTime !== undefined) {
                        newestDeclarationFileContentChangedTime = newer(priorChangeTime, newestDeclarationFileContentChangedTime);
                    }
                });
                finishEmit(emitterDiagnostics, emittedOutputs, newestDeclarationFileContentChangedTime, 
                /*newestDeclarationFileContentChangedTimeIsMaximumDate*/ anyDtsChanged, outputFiles.length ? outputFiles[0].name : ts.getFirstProjectOutput(config, !host.useCaseSensitiveFileNames()), resultFlags);
                return emitResult;
            }
            function finishEmit(emitterDiagnostics, emittedOutputs, priorNewestUpdateTime, newestDeclarationFileContentChangedTimeIsMaximumDate, oldestOutputFileName, resultFlags) {
                var emitDiagnostics = emitterDiagnostics.getDiagnostics();
                if (emitDiagnostics.length) {
                    buildResult = buildErrors(state, projectPath, program, config, emitDiagnostics, BuildResultFlags.EmitErrors, "Emit");
                    step = Step.QueueReferencingProjects;
                    return emitDiagnostics;
                }
                if (state.writeFileName) {
                    emittedOutputs.forEach(function (name) { return listEmittedFile(state, config, name); });
                    if (program)
                        ts.listFiles(program, state.writeFileName);
                }
                // Update time stamps for rest of the outputs
                var newestDeclarationFileContentChangedTime = updateOutputTimestampsWorker(state, config, priorNewestUpdateTime, ts.Diagnostics.Updating_unchanged_output_timestamps_of_project_0, emittedOutputs);
                state.diagnostics.delete(projectPath);
                state.projectStatus.set(projectPath, {
                    type: ts.UpToDateStatusType.UpToDate,
                    newestDeclarationFileContentChangedTime: newestDeclarationFileContentChangedTimeIsMaximumDate ?
                        maximumDate :
                        newestDeclarationFileContentChangedTime,
                    oldestOutputFileName: oldestOutputFileName
                });
                afterProgramDone(state, projectPath, program, config);
                state.projectCompilerOptions = state.baseCompilerOptions;
                step = Step.QueueReferencingProjects;
                buildResult = resultFlags;
                return emitDiagnostics;
            }
            function emitBundle(writeFileCallback, customTransformers) {
                ts.Debug.assert(kind === InvalidatedProjectKind.UpdateBundle);
                if (state.options.dry) {
                    reportStatus(state, ts.Diagnostics.A_non_dry_build_would_update_output_of_project_0, project);
                    buildResult = BuildResultFlags.Success;
                    step = Step.QueueReferencingProjects;
                    return undefined;
                }
                if (state.options.verbose)
                    reportStatus(state, ts.Diagnostics.Updating_output_of_project_0, project);
                // Update js, and source map
                var compilerHost = state.compilerHost;
                state.projectCompilerOptions = config.options;
                var outputFiles = ts.emitUsingBuildInfo(config, compilerHost, function (ref) {
                    var refName = resolveProjectName(state, ref.path);
                    return parseConfigFile(state, refName, toResolvedConfigFilePath(state, refName));
                }, customTransformers);
                if (ts.isString(outputFiles)) {
                    reportStatus(state, ts.Diagnostics.Cannot_update_output_of_project_0_because_there_was_error_reading_file_1, project, relName(state, outputFiles));
                    step = Step.BuildInvalidatedProjectOfBundle;
                    return invalidatedProjectOfBundle = createBuildOrUpdateInvalidedProject(InvalidatedProjectKind.Build, state, project, projectPath, projectIndex, config, buildOrder);
                }
                // Actual Emit
                ts.Debug.assert(!!outputFiles.length);
                var emitterDiagnostics = ts.createDiagnosticCollection();
                var emittedOutputs = ts.createMap();
                outputFiles.forEach(function (_a) {
                    var name = _a.name, text = _a.text, writeByteOrderMark = _a.writeByteOrderMark;
                    emittedOutputs.set(toPath(state, name), name);
                    ts.writeFile(writeFileCallback ? { writeFile: writeFileCallback } : compilerHost, emitterDiagnostics, name, text, writeByteOrderMark);
                });
                var emitDiagnostics = finishEmit(emitterDiagnostics, emittedOutputs, minimumDate, 
                /*newestDeclarationFileContentChangedTimeIsMaximumDate*/ false, outputFiles[0].name, BuildResultFlags.DeclarationOutputUnchanged);
                return { emitSkipped: false, diagnostics: emitDiagnostics };
            }
            function executeSteps(till, cancellationToken, writeFile, customTransformers) {
                while (step <= till && step < Step.Done) {
                    var currentStep = step;
                    switch (step) {
                        case Step.CreateProgram:
                            createProgram();
                            break;
                        case Step.SyntaxDiagnostics:
                            getSyntaxDiagnostics(cancellationToken);
                            break;
                        case Step.SemanticDiagnostics:
                            getSemanticDiagnostics(cancellationToken);
                            break;
                        case Step.Emit:
                            emit(writeFile, cancellationToken, customTransformers);
                            break;
                        case Step.EmitBundle:
                            emitBundle(writeFile, customTransformers);
                            break;
                        case Step.BuildInvalidatedProjectOfBundle:
                            ts.Debug.assertDefined(invalidatedProjectOfBundle).done(cancellationToken);
                            step = Step.Done;
                            break;
                        case Step.QueueReferencingProjects:
                            queueReferencingProjects(state, project, projectPath, projectIndex, config, buildOrder, ts.Debug.assertDefined(buildResult));
                            step++;
                            break;
                        // Should never be done
                        case Step.Done:
                        default:
                            ts.assertType(step);
                    }
                    ts.Debug.assert(step > currentStep);
                }
            }
        }
        function needsBuild(_a, status, config) {
            var options = _a.options;
            if (status.type !== ts.UpToDateStatusType.OutOfDateWithPrepend || options.force)
                return true;
            return config.fileNames.length === 0 ||
                !!ts.getConfigFileParsingDiagnostics(config).length ||
                !ts.isIncrementalCompilation(config.options);
        }
        function getNextInvalidatedProject(state, buildOrder, reportQueue) {
            if (!state.projectPendingBuild.size)
                return undefined;
            if (isCircularBuildOrder(buildOrder))
                return undefined;
            if (state.currentInvalidatedProject) {
                // Only if same buildOrder the currentInvalidated project can be sent again
                return ts.arrayIsEqualTo(state.currentInvalidatedProject.buildOrder, buildOrder) ?
                    state.currentInvalidatedProject :
                    undefined;
            }
            var options = state.options, projectPendingBuild = state.projectPendingBuild;
            for (var projectIndex = 0; projectIndex < buildOrder.length; projectIndex++) {
                var project = buildOrder[projectIndex];
                var projectPath = toResolvedConfigFilePath(state, project);
                var reloadLevel = state.projectPendingBuild.get(projectPath);
                if (reloadLevel === undefined)
                    continue;
                if (reportQueue) {
                    reportQueue = false;
                    reportBuildQueue(state, buildOrder);
                }
                var config = parseConfigFile(state, project, projectPath);
                if (!config) {
                    reportParseConfigFileDiagnostic(state, projectPath);
                    projectPendingBuild.delete(projectPath);
                    continue;
                }
                if (reloadLevel === ts.ConfigFileProgramReloadLevel.Full) {
                    watchConfigFile(state, project, projectPath, config);
                    watchWildCardDirectories(state, project, projectPath, config);
                    watchInputFiles(state, project, projectPath, config);
                }
                else if (reloadLevel === ts.ConfigFileProgramReloadLevel.Partial) {
                    // Update file names
                    var result = ts.getFileNamesFromConfigSpecs(config.configFileSpecs, ts.getDirectoryPath(project), config.options, state.parseConfigFileHost);
                    ts.updateErrorForNoInputFiles(result, project, config.configFileSpecs, config.errors, ts.canJsonReportNoInutFiles(config.raw));
                    config.fileNames = result.fileNames;
                    watchInputFiles(state, project, projectPath, config);
                }
                var status = getUpToDateStatus(state, config, projectPath);
                verboseReportProjectStatus(state, project, status);
                if (!options.force) {
                    if (status.type === ts.UpToDateStatusType.UpToDate) {
                        reportAndStoreErrors(state, projectPath, ts.getConfigFileParsingDiagnostics(config));
                        projectPendingBuild.delete(projectPath);
                        // Up to date, skip
                        if (options.dry) {
                            // In a dry build, inform the user of this fact
                            reportStatus(state, ts.Diagnostics.Project_0_is_up_to_date, project);
                        }
                        continue;
                    }
                    if (status.type === ts.UpToDateStatusType.UpToDateWithUpstreamTypes) {
                        reportAndStoreErrors(state, projectPath, ts.getConfigFileParsingDiagnostics(config));
                        return createUpdateOutputFileStampsProject(state, project, projectPath, config, buildOrder);
                    }
                }
                if (status.type === ts.UpToDateStatusType.UpstreamBlocked) {
                    reportAndStoreErrors(state, projectPath, ts.getConfigFileParsingDiagnostics(config));
                    projectPendingBuild.delete(projectPath);
                    if (options.verbose) {
                        reportStatus(state, status.upstreamProjectBlocked ?
                            ts.Diagnostics.Skipping_build_of_project_0_because_its_dependency_1_was_not_built :
                            ts.Diagnostics.Skipping_build_of_project_0_because_its_dependency_1_has_errors, project, status.upstreamProjectName);
                    }
                    continue;
                }
                if (status.type === ts.UpToDateStatusType.ContainerOnly) {
                    reportAndStoreErrors(state, projectPath, ts.getConfigFileParsingDiagnostics(config));
                    projectPendingBuild.delete(projectPath);
                    // Do nothing
                    continue;
                }
                return createBuildOrUpdateInvalidedProject(needsBuild(state, status, config) ?
                    InvalidatedProjectKind.Build :
                    InvalidatedProjectKind.UpdateBundle, state, project, projectPath, projectIndex, config, buildOrder);
            }
            return undefined;
        }
        function listEmittedFile(_a, proj, file) {
            var writeFileName = _a.writeFileName;
            if (writeFileName && proj.options.listEmittedFiles) {
                writeFileName("TSFILE: " + file);
            }
        }
        function getOldProgram(_a, proj, parsed) {
            var options = _a.options, builderPrograms = _a.builderPrograms, compilerHost = _a.compilerHost;
            if (options.force)
                return undefined;
            var value = builderPrograms.get(proj);
            if (value)
                return value;
            return ts.readBuilderProgram(parsed.options, compilerHost);
        }
        function afterProgramDone(_a, proj, program, config) {
            var host = _a.host, watch = _a.watch, builderPrograms = _a.builderPrograms;
            if (program) {
                if (host.afterProgramEmitAndDiagnostics) {
                    host.afterProgramEmitAndDiagnostics(program);
                }
                if (watch) {
                    program.releaseProgram();
                    builderPrograms.set(proj, program);
                }
            }
            else if (host.afterEmitBundle) {
                host.afterEmitBundle(config);
            }
        }
        function buildErrors(state, resolvedPath, program, config, diagnostics, errorFlags, errorType) {
            reportAndStoreErrors(state, resolvedPath, diagnostics);
            // List files if any other build error using program (emit errors already report files)
            if (program && state.writeFileName)
                ts.listFiles(program, state.writeFileName);
            state.projectStatus.set(resolvedPath, { type: ts.UpToDateStatusType.Unbuildable, reason: errorType + " errors" });
            afterProgramDone(state, resolvedPath, program, config);
            state.projectCompilerOptions = state.baseCompilerOptions;
            return errorFlags;
        }
        function updateModuleResolutionCache(state, proj, config) {
            if (!state.moduleResolutionCache)
                return;
            // Update module resolution cache if needed
            var moduleResolutionCache = state.moduleResolutionCache;
            var projPath = toPath(state, proj);
            if (moduleResolutionCache.directoryToModuleNameMap.redirectsMap.size === 0) {
                // The own map will be for projectCompilerOptions
                ts.Debug.assert(moduleResolutionCache.moduleNameToDirectoryMap.redirectsMap.size === 0);
                moduleResolutionCache.directoryToModuleNameMap.redirectsMap.set(projPath, moduleResolutionCache.directoryToModuleNameMap.ownMap);
                moduleResolutionCache.moduleNameToDirectoryMap.redirectsMap.set(projPath, moduleResolutionCache.moduleNameToDirectoryMap.ownMap);
            }
            else {
                // Set correct own map
                ts.Debug.assert(moduleResolutionCache.moduleNameToDirectoryMap.redirectsMap.size > 0);
                var ref = {
                    sourceFile: config.options.configFile,
                    commandLine: config
                };
                moduleResolutionCache.directoryToModuleNameMap.setOwnMap(moduleResolutionCache.directoryToModuleNameMap.getOrCreateMapOfCacheRedirects(ref));
                moduleResolutionCache.moduleNameToDirectoryMap.setOwnMap(moduleResolutionCache.moduleNameToDirectoryMap.getOrCreateMapOfCacheRedirects(ref));
            }
            moduleResolutionCache.directoryToModuleNameMap.setOwnOptions(config.options);
            moduleResolutionCache.moduleNameToDirectoryMap.setOwnOptions(config.options);
        }
        function checkConfigFileUpToDateStatus(state, configFile, oldestOutputFileTime, oldestOutputFileName) {
            // Check tsconfig time
            var tsconfigTime = state.host.getModifiedTime(configFile) || ts.missingFileModifiedTime;
            if (oldestOutputFileTime < tsconfigTime) {
                return {
                    type: ts.UpToDateStatusType.OutOfDateWithSelf,
                    outOfDateOutputFileName: oldestOutputFileName,
                    newerInputFileName: configFile
                };
            }
        }
        function getUpToDateStatusWorker(state, project, resolvedPath) {
            var newestInputFileName = undefined;
            var newestInputFileTime = minimumDate;
            var host = state.host;
            // Get timestamps of input files
            for (var _i = 0, _a = project.fileNames; _i < _a.length; _i++) {
                var inputFile = _a[_i];
                if (!host.fileExists(inputFile)) {
                    return {
                        type: ts.UpToDateStatusType.Unbuildable,
                        reason: inputFile + " does not exist"
                    };
                }
                var inputTime = host.getModifiedTime(inputFile) || ts.missingFileModifiedTime;
                if (inputTime > newestInputFileTime) {
                    newestInputFileName = inputFile;
                    newestInputFileTime = inputTime;
                }
            }
            // Container if no files are specified in the project
            if (!project.fileNames.length && !ts.canJsonReportNoInutFiles(project.raw)) {
                return {
                    type: ts.UpToDateStatusType.ContainerOnly
                };
            }
            // Collect the expected outputs of this project
            var outputs = ts.getAllProjectOutputs(project, !host.useCaseSensitiveFileNames());
            // Now see if all outputs are newer than the newest input
            var oldestOutputFileName = "(none)";
            var oldestOutputFileTime = maximumDate;
            var newestOutputFileName = "(none)";
            var newestOutputFileTime = minimumDate;
            var missingOutputFileName;
            var newestDeclarationFileContentChangedTime = minimumDate;
            var isOutOfDateWithInputs = false;
            for (var _b = 0, outputs_1 = outputs; _b < outputs_1.length; _b++) {
                var output = outputs_1[_b];
                // Output is missing; can stop checking
                // Don't immediately return because we can still be upstream-blocked, which is a higher-priority status
                if (!host.fileExists(output)) {
                    missingOutputFileName = output;
                    break;
                }
                var outputTime = host.getModifiedTime(output) || ts.missingFileModifiedTime;
                if (outputTime < oldestOutputFileTime) {
                    oldestOutputFileTime = outputTime;
                    oldestOutputFileName = output;
                }
                // If an output is older than the newest input, we can stop checking
                // Don't immediately return because we can still be upstream-blocked, which is a higher-priority status
                if (outputTime < newestInputFileTime) {
                    isOutOfDateWithInputs = true;
                    break;
                }
                if (outputTime > newestOutputFileTime) {
                    newestOutputFileTime = outputTime;
                    newestOutputFileName = output;
                }
                // Keep track of when the most recent time a .d.ts file was changed.
                // In addition to file timestamps, we also keep track of when a .d.ts file
                // had its file touched but not had its contents changed - this allows us
                // to skip a downstream typecheck
                if (isDeclarationFile(output)) {
                    var outputModifiedTime = host.getModifiedTime(output) || ts.missingFileModifiedTime;
                    newestDeclarationFileContentChangedTime = newer(newestDeclarationFileContentChangedTime, outputModifiedTime);
                }
            }
            var pseudoUpToDate = false;
            var usesPrepend = false;
            var upstreamChangedProject;
            if (project.projectReferences) {
                state.projectStatus.set(resolvedPath, { type: ts.UpToDateStatusType.ComputingUpstream });
                for (var _c = 0, _d = project.projectReferences; _c < _d.length; _c++) {
                    var ref = _d[_c];
                    usesPrepend = usesPrepend || !!(ref.prepend);
                    var resolvedRef = ts.resolveProjectReferencePath(ref);
                    var resolvedRefPath = toResolvedConfigFilePath(state, resolvedRef);
                    var refStatus = getUpToDateStatus(state, parseConfigFile(state, resolvedRef, resolvedRefPath), resolvedRefPath);
                    // Its a circular reference ignore the status of this project
                    if (refStatus.type === ts.UpToDateStatusType.ComputingUpstream ||
                        refStatus.type === ts.UpToDateStatusType.ContainerOnly) { // Container only ignore this project
                        continue;
                    }
                    // An upstream project is blocked
                    if (refStatus.type === ts.UpToDateStatusType.Unbuildable ||
                        refStatus.type === ts.UpToDateStatusType.UpstreamBlocked) {
                        return {
                            type: ts.UpToDateStatusType.UpstreamBlocked,
                            upstreamProjectName: ref.path,
                            upstreamProjectBlocked: refStatus.type === ts.UpToDateStatusType.UpstreamBlocked
                        };
                    }
                    // If the upstream project is out of date, then so are we (someone shouldn't have asked, though?)
                    if (refStatus.type !== ts.UpToDateStatusType.UpToDate) {
                        return {
                            type: ts.UpToDateStatusType.UpstreamOutOfDate,
                            upstreamProjectName: ref.path
                        };
                    }
                    // Check oldest output file name only if there is no missing output file name
                    if (!missingOutputFileName) {
                        // If the upstream project's newest file is older than our oldest output, we
                        // can't be out of date because of it
                        if (refStatus.newestInputFileTime && refStatus.newestInputFileTime <= oldestOutputFileTime) {
                            continue;
                        }
                        // If the upstream project has only change .d.ts files, and we've built
                        // *after* those files, then we're "psuedo up to date" and eligible for a fast rebuild
                        if (refStatus.newestDeclarationFileContentChangedTime && refStatus.newestDeclarationFileContentChangedTime <= oldestOutputFileTime) {
                            pseudoUpToDate = true;
                            upstreamChangedProject = ref.path;
                            continue;
                        }
                        // We have an output older than an upstream output - we are out of date
                        ts.Debug.assert(oldestOutputFileName !== undefined, "Should have an oldest output filename here");
                        return {
                            type: ts.UpToDateStatusType.OutOfDateWithUpstream,
                            outOfDateOutputFileName: oldestOutputFileName,
                            newerProjectName: ref.path
                        };
                    }
                }
            }
            if (missingOutputFileName !== undefined) {
                return {
                    type: ts.UpToDateStatusType.OutputMissing,
                    missingOutputFileName: missingOutputFileName
                };
            }
            if (isOutOfDateWithInputs) {
                return {
                    type: ts.UpToDateStatusType.OutOfDateWithSelf,
                    outOfDateOutputFileName: oldestOutputFileName,
                    newerInputFileName: newestInputFileName
                };
            }
            else {
                // Check tsconfig time
                var configStatus = checkConfigFileUpToDateStatus(state, project.options.configFilePath, oldestOutputFileTime, oldestOutputFileName);
                if (configStatus)
                    return configStatus;
                // Check extended config time
                var extendedConfigStatus = ts.forEach(project.options.configFile.extendedSourceFiles || ts.emptyArray, function (configFile) { return checkConfigFileUpToDateStatus(state, configFile, oldestOutputFileTime, oldestOutputFileName); });
                if (extendedConfigStatus)
                    return extendedConfigStatus;
            }
            if (!state.buildInfoChecked.has(resolvedPath)) {
                state.buildInfoChecked.set(resolvedPath, true);
                var buildInfoPath = ts.getTsBuildInfoEmitOutputFilePath(project.options);
                if (buildInfoPath) {
                    var value = state.readFileWithCache(buildInfoPath);
                    var buildInfo = value && ts.getBuildInfo(value);
                    if (buildInfo && (buildInfo.bundle || buildInfo.program) && buildInfo.version !== ts.version) {
                        return {
                            type: ts.UpToDateStatusType.TsVersionOutputOfDate,
                            version: buildInfo.version
                        };
                    }
                }
            }
            if (usesPrepend && pseudoUpToDate) {
                return {
                    type: ts.UpToDateStatusType.OutOfDateWithPrepend,
                    outOfDateOutputFileName: oldestOutputFileName,
                    newerProjectName: upstreamChangedProject
                };
            }
            // Up to date
            return {
                type: pseudoUpToDate ? ts.UpToDateStatusType.UpToDateWithUpstreamTypes : ts.UpToDateStatusType.UpToDate,
                newestDeclarationFileContentChangedTime: newestDeclarationFileContentChangedTime,
                newestInputFileTime: newestInputFileTime,
                newestOutputFileTime: newestOutputFileTime,
                newestInputFileName: newestInputFileName,
                newestOutputFileName: newestOutputFileName,
                oldestOutputFileName: oldestOutputFileName
            };
        }
        function getUpToDateStatus(state, project, resolvedPath) {
            if (project === undefined) {
                return { type: ts.UpToDateStatusType.Unbuildable, reason: "File deleted mid-build" };
            }
            var prior = state.projectStatus.get(resolvedPath);
            if (prior !== undefined) {
                return prior;
            }
            var actual = getUpToDateStatusWorker(state, project, resolvedPath);
            state.projectStatus.set(resolvedPath, actual);
            return actual;
        }
        function updateOutputTimestampsWorker(state, proj, priorNewestUpdateTime, verboseMessage, skipOutputs) {
            var host = state.host;
            var outputs = ts.getAllProjectOutputs(proj, !host.useCaseSensitiveFileNames());
            if (!skipOutputs || outputs.length !== skipOutputs.size) {
                var reportVerbose = !!state.options.verbose;
                var now = host.now ? host.now() : new Date();
                for (var _i = 0, outputs_2 = outputs; _i < outputs_2.length; _i++) {
                    var file = outputs_2[_i];
                    if (skipOutputs && skipOutputs.has(toPath(state, file))) {
                        continue;
                    }
                    if (reportVerbose) {
                        reportVerbose = false;
                        reportStatus(state, verboseMessage, proj.options.configFilePath);
                    }
                    if (isDeclarationFile(file)) {
                        priorNewestUpdateTime = newer(priorNewestUpdateTime, host.getModifiedTime(file) || ts.missingFileModifiedTime);
                    }
                    host.setModifiedTime(file, now);
                }
            }
            return priorNewestUpdateTime;
        }
        function updateOutputTimestamps(state, proj, resolvedPath) {
            if (state.options.dry) {
                return reportStatus(state, ts.Diagnostics.A_non_dry_build_would_update_timestamps_for_output_of_project_0, proj.options.configFilePath);
            }
            var priorNewestUpdateTime = updateOutputTimestampsWorker(state, proj, minimumDate, ts.Diagnostics.Updating_output_timestamps_of_project_0);
            state.projectStatus.set(resolvedPath, {
                type: ts.UpToDateStatusType.UpToDate,
                newestDeclarationFileContentChangedTime: priorNewestUpdateTime,
                oldestOutputFileName: ts.getFirstProjectOutput(proj, !state.host.useCaseSensitiveFileNames())
            });
        }
        function queueReferencingProjects(state, project, projectPath, projectIndex, config, buildOrder, buildResult) {
            // Queue only if there are no errors
            if (buildResult & BuildResultFlags.AnyErrors)
                return;
            // Only composite projects can be referenced by other projects
            if (!config.options.composite)
                return;
            // Always use build order to queue projects
            for (var index = projectIndex + 1; index < buildOrder.length; index++) {
                var nextProject = buildOrder[index];
                var nextProjectPath = toResolvedConfigFilePath(state, nextProject);
                if (state.projectPendingBuild.has(nextProjectPath))
                    continue;
                var nextProjectConfig = parseConfigFile(state, nextProject, nextProjectPath);
                if (!nextProjectConfig || !nextProjectConfig.projectReferences)
                    continue;
                for (var _i = 0, _a = nextProjectConfig.projectReferences; _i < _a.length; _i++) {
                    var ref = _a[_i];
                    var resolvedRefPath = resolveProjectName(state, ref.path);
                    if (toResolvedConfigFilePath(state, resolvedRefPath) !== projectPath)
                        continue;
                    // If the project is referenced with prepend, always build downstream projects,
                    // If declaration output is changed, build the project
                    // otherwise mark the project UpToDateWithUpstreamTypes so it updates output time stamps
                    var status = state.projectStatus.get(nextProjectPath);
                    if (status) {
                        switch (status.type) {
                            case ts.UpToDateStatusType.UpToDate:
                                if (buildResult & BuildResultFlags.DeclarationOutputUnchanged) {
                                    if (ref.prepend) {
                                        state.projectStatus.set(nextProjectPath, {
                                            type: ts.UpToDateStatusType.OutOfDateWithPrepend,
                                            outOfDateOutputFileName: status.oldestOutputFileName,
                                            newerProjectName: project
                                        });
                                    }
                                    else {
                                        status.type = ts.UpToDateStatusType.UpToDateWithUpstreamTypes;
                                    }
                                    break;
                                }
                            // falls through
                            case ts.UpToDateStatusType.UpToDateWithUpstreamTypes:
                            case ts.UpToDateStatusType.OutOfDateWithPrepend:
                                if (!(buildResult & BuildResultFlags.DeclarationOutputUnchanged)) {
                                    state.projectStatus.set(nextProjectPath, {
                                        type: ts.UpToDateStatusType.OutOfDateWithUpstream,
                                        outOfDateOutputFileName: status.type === ts.UpToDateStatusType.OutOfDateWithPrepend ? status.outOfDateOutputFileName : status.oldestOutputFileName,
                                        newerProjectName: project
                                    });
                                }
                                break;
                            case ts.UpToDateStatusType.UpstreamBlocked:
                                if (toResolvedConfigFilePath(state, resolveProjectName(state, status.upstreamProjectName)) === projectPath) {
                                    clearProjectStatus(state, nextProjectPath);
                                }
                                break;
                        }
                    }
                    addProjToQueue(state, nextProjectPath, ts.ConfigFileProgramReloadLevel.None);
                    break;
                }
            }
        }
        function build(state, project, cancellationToken, onlyReferences) {
            var buildOrder = getBuildOrderFor(state, project, onlyReferences);
            if (!buildOrder)
                return ts.ExitStatus.InvalidProject_OutputsSkipped;
            setupInitialBuild(state, cancellationToken);
            var reportQueue = true;
            var successfulProjects = 0;
            while (true) {
                var invalidatedProject = getNextInvalidatedProject(state, buildOrder, reportQueue);
                if (!invalidatedProject)
                    break;
                reportQueue = false;
                invalidatedProject.done(cancellationToken);
                if (!state.diagnostics.has(invalidatedProject.projectPath))
                    successfulProjects++;
            }
            disableCache(state);
            reportErrorSummary(state, buildOrder);
            startWatching(state, buildOrder);
            return isCircularBuildOrder(buildOrder)
                ? ts.ExitStatus.ProjectReferenceCycle_OutputsSkipped
                : !buildOrder.some(function (p) { return state.diagnostics.has(toResolvedConfigFilePath(state, p)); })
                    ? ts.ExitStatus.Success
                    : successfulProjects
                        ? ts.ExitStatus.DiagnosticsPresent_OutputsGenerated
                        : ts.ExitStatus.DiagnosticsPresent_OutputsSkipped;
        }
        function clean(state, project, onlyReferences) {
            var buildOrder = getBuildOrderFor(state, project, onlyReferences);
            if (!buildOrder)
                return ts.ExitStatus.InvalidProject_OutputsSkipped;
            if (isCircularBuildOrder(buildOrder)) {
                reportErrors(state, buildOrder.circularDiagnostics);
                return ts.ExitStatus.ProjectReferenceCycle_OutputsSkipped;
            }
            var options = state.options, host = state.host;
            var filesToDelete = options.dry ? [] : undefined;
            for (var _i = 0, buildOrder_1 = buildOrder; _i < buildOrder_1.length; _i++) {
                var proj = buildOrder_1[_i];
                var resolvedPath = toResolvedConfigFilePath(state, proj);
                var parsed = parseConfigFile(state, proj, resolvedPath);
                if (parsed === undefined) {
                    // File has gone missing; fine to ignore here
                    reportParseConfigFileDiagnostic(state, resolvedPath);
                    continue;
                }
                var outputs = ts.getAllProjectOutputs(parsed, !host.useCaseSensitiveFileNames());
                for (var _a = 0, outputs_3 = outputs; _a < outputs_3.length; _a++) {
                    var output = outputs_3[_a];
                    if (host.fileExists(output)) {
                        if (filesToDelete) {
                            filesToDelete.push(output);
                        }
                        else {
                            host.deleteFile(output);
                            invalidateProject(state, resolvedPath, ts.ConfigFileProgramReloadLevel.None);
                        }
                    }
                }
            }
            if (filesToDelete) {
                reportStatus(state, ts.Diagnostics.A_non_dry_build_would_delete_the_following_files_Colon_0, filesToDelete.map(function (f) { return "\r\n * " + f; }).join(""));
            }
            return ts.ExitStatus.Success;
        }
        function invalidateProject(state, resolved, reloadLevel) {
            // If host implements getParsedCommandLine, we cant get list of files from parseConfigFileHost
            if (state.host.getParsedCommandLine && reloadLevel === ts.ConfigFileProgramReloadLevel.Partial) {
                reloadLevel = ts.ConfigFileProgramReloadLevel.Full;
            }
            if (reloadLevel === ts.ConfigFileProgramReloadLevel.Full) {
                state.configFileCache.delete(resolved);
                state.buildOrder = undefined;
            }
            state.needsSummary = true;
            clearProjectStatus(state, resolved);
            addProjToQueue(state, resolved, reloadLevel);
            enableCache(state);
        }
        function invalidateProjectAndScheduleBuilds(state, resolvedPath, reloadLevel) {
            state.reportFileChangeDetected = true;
            invalidateProject(state, resolvedPath, reloadLevel);
            scheduleBuildInvalidatedProject(state);
        }
        function scheduleBuildInvalidatedProject(state) {
            var hostWithWatch = state.hostWithWatch;
            if (!hostWithWatch.setTimeout || !hostWithWatch.clearTimeout) {
                return;
            }
            if (state.timerToBuildInvalidatedProject) {
                hostWithWatch.clearTimeout(state.timerToBuildInvalidatedProject);
            }
            state.timerToBuildInvalidatedProject = hostWithWatch.setTimeout(buildNextInvalidatedProject, 250, state);
        }
        function buildNextInvalidatedProject(state) {
            state.timerToBuildInvalidatedProject = undefined;
            if (state.reportFileChangeDetected) {
                state.reportFileChangeDetected = false;
                state.projectErrorsReported.clear();
                reportWatchStatus(state, ts.Diagnostics.File_change_detected_Starting_incremental_compilation);
            }
            var buildOrder = getBuildOrder(state);
            var invalidatedProject = getNextInvalidatedProject(state, buildOrder, /*reportQueue*/ false);
            if (invalidatedProject) {
                invalidatedProject.done();
                if (state.projectPendingBuild.size) {
                    // Schedule next project for build
                    if (state.watch && !state.timerToBuildInvalidatedProject) {
                        scheduleBuildInvalidatedProject(state);
                    }
                    return;
                }
            }
            disableCache(state);
            reportErrorSummary(state, buildOrder);
        }
        function watchConfigFile(state, resolved, resolvedPath, parsed) {
            if (!state.watch || state.allWatchedConfigFiles.has(resolvedPath))
                return;
            state.allWatchedConfigFiles.set(resolvedPath, state.watchFile(state.hostWithWatch, resolved, function () {
                invalidateProjectAndScheduleBuilds(state, resolvedPath, ts.ConfigFileProgramReloadLevel.Full);
            }, ts.PollingInterval.High, parsed === null || parsed === void 0 ? void 0 : parsed.watchOptions, ts.WatchType.ConfigFile, resolved));
        }
        function isSameFile(state, file1, file2) {
            return ts.comparePaths(file1, file2, state.currentDirectory, !state.host.useCaseSensitiveFileNames()) === 0 /* EqualTo */;
        }
        function isOutputFile(state, fileName, configFile) {
            if (configFile.options.noEmit)
                return false;
            // ts or tsx files are not output
            if (!ts.fileExtensionIs(fileName, ".d.ts" /* Dts */) &&
                (ts.fileExtensionIs(fileName, ".ts" /* Ts */) || ts.fileExtensionIs(fileName, ".tsx" /* Tsx */))) {
                return false;
            }
            // If options have --outFile or --out, check if its that
            var out = configFile.options.outFile || configFile.options.out;
            if (out && (isSameFile(state, fileName, out) || isSameFile(state, fileName, ts.removeFileExtension(out) + ".d.ts" /* Dts */))) {
                return true;
            }
            // If declarationDir is specified, return if its a file in that directory
            if (configFile.options.declarationDir && ts.containsPath(configFile.options.declarationDir, fileName, state.currentDirectory, !state.host.useCaseSensitiveFileNames())) {
                return true;
            }
            // If --outDir, check if file is in that directory
            if (configFile.options.outDir && ts.containsPath(configFile.options.outDir, fileName, state.currentDirectory, !state.host.useCaseSensitiveFileNames())) {
                return true;
            }
            return !ts.forEach(configFile.fileNames, function (inputFile) { return isSameFile(state, fileName, inputFile); });
        }
        function watchWildCardDirectories(state, resolved, resolvedPath, parsed) {
            if (!state.watch)
                return;
            ts.updateWatchingWildcardDirectories(getOrCreateValueMapFromConfigFileMap(state.allWatchedWildcardDirectories, resolvedPath), ts.createMapFromTemplate(parsed.configFileSpecs.wildcardDirectories), function (dir, flags) {
                return state.watchDirectory(state.hostWithWatch, dir, function (fileOrDirectory) {
                    var fileOrDirectoryPath = toPath(state, fileOrDirectory);
                    if (fileOrDirectoryPath !== toPath(state, dir) && ts.hasExtension(fileOrDirectoryPath) && !ts.isSupportedSourceFileName(fileOrDirectory, parsed.options)) {
                        state.writeLog("Project: " + resolved + " Detected file add/remove of non supported extension: " + fileOrDirectory);
                        return;
                    }
                    if (isOutputFile(state, fileOrDirectory, parsed)) {
                        state.writeLog(fileOrDirectory + " is output file");
                        return;
                    }
                    invalidateProjectAndScheduleBuilds(state, resolvedPath, ts.ConfigFileProgramReloadLevel.Partial);
                }, flags, parsed === null || parsed === void 0 ? void 0 : parsed.watchOptions, ts.WatchType.WildcardDirectory, resolved);
            });
        }
        function watchInputFiles(state, resolved, resolvedPath, parsed) {
            if (!state.watch)
                return;
            ts.mutateMap(getOrCreateValueMapFromConfigFileMap(state.allWatchedInputFiles, resolvedPath), ts.arrayToMap(parsed.fileNames, function (fileName) { return toPath(state, fileName); }), {
                createNewValue: function (path, input) { return state.watchFilePath(state.hostWithWatch, input, function () { return invalidateProjectAndScheduleBuilds(state, resolvedPath, ts.ConfigFileProgramReloadLevel.None); }, ts.PollingInterval.Low, parsed === null || parsed === void 0 ? void 0 : parsed.watchOptions, path, ts.WatchType.SourceFile, resolved); },
                onDeleteValue: ts.closeFileWatcher,
            });
        }
        function startWatching(state, buildOrder) {
            if (!state.watchAllProjectsPending)
                return;
            state.watchAllProjectsPending = false;
            for (var _i = 0, _a = getBuildOrderFromAnyBuildOrder(buildOrder); _i < _a.length; _i++) {
                var resolved = _a[_i];
                var resolvedPath = toResolvedConfigFilePath(state, resolved);
                var cfg = parseConfigFile(state, resolved, resolvedPath);
                // Watch this file
                watchConfigFile(state, resolved, resolvedPath, cfg);
                if (cfg) {
                    // Update watchers for wildcard directories
                    watchWildCardDirectories(state, resolved, resolvedPath, cfg);
                    // Watch input files
                    watchInputFiles(state, resolved, resolvedPath, cfg);
                }
            }
        }
        function stopWatching(state) {
            ts.clearMap(state.allWatchedConfigFiles, ts.closeFileWatcher);
            ts.clearMap(state.allWatchedWildcardDirectories, function (watchedWildcardDirectories) { return ts.clearMap(watchedWildcardDirectories, ts.closeFileWatcherOf); });
            ts.clearMap(state.allWatchedInputFiles, function (watchedWildcardDirectories) { return ts.clearMap(watchedWildcardDirectories, ts.closeFileWatcher); });
        }
        function createSolutionBuilderWorker(watch, hostOrHostWithWatch, rootNames, options, baseWatchOptions) {
            var state = createSolutionBuilderState(watch, hostOrHostWithWatch, rootNames, options, baseWatchOptions);
            return {
                build: function (project, cancellationToken) { return build(state, project, cancellationToken); },
                clean: function (project) { return clean(state, project); },
                buildReferences: function (project, cancellationToken) { return build(state, project, cancellationToken, /*onlyReferences*/ true); },
                cleanReferences: function (project) { return clean(state, project, /*onlyReferences*/ true); },
                getNextInvalidatedProject: function (cancellationToken) {
                    setupInitialBuild(state, cancellationToken);
                    return getNextInvalidatedProject(state, getBuildOrder(state), /*reportQueue*/ false);
                },
                getBuildOrder: function () { return getBuildOrder(state); },
                getUpToDateStatusOfProject: function (project) {
                    var configFileName = resolveProjectName(state, project);
                    var configFilePath = toResolvedConfigFilePath(state, configFileName);
                    return getUpToDateStatus(state, parseConfigFile(state, configFileName, configFilePath), configFilePath);
                },
                invalidateProject: function (configFilePath, reloadLevel) { return invalidateProject(state, configFilePath, reloadLevel || ts.ConfigFileProgramReloadLevel.None); },
                buildNextInvalidatedProject: function () { return buildNextInvalidatedProject(state); },
                getAllParsedConfigs: function () { return ts.arrayFrom(ts.mapDefinedIterator(state.configFileCache.values(), function (config) { return isParsedCommandLine(config) ? config : undefined; })); },
                close: function () { return stopWatching(state); },
            };
        }
        function relName(state, path) {
            return ts.convertToRelativePath(path, state.currentDirectory, function (f) { return state.getCanonicalFileName(f); });
        }
        function reportStatus(state, message) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            state.host.reportSolutionBuilderStatus(ts.createCompilerDiagnostic.apply(void 0, __spreadArrays([message], args)));
        }
        function reportWatchStatus(state, message) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            if (state.hostWithWatch.onWatchStatusChange) {
                state.hostWithWatch.onWatchStatusChange(ts.createCompilerDiagnostic.apply(void 0, __spreadArrays([message], args)), state.host.getNewLine(), state.baseCompilerOptions);
            }
        }
        function reportErrors(_a, errors) {
            var host = _a.host;
            errors.forEach(function (err) { return host.reportDiagnostic(err); });
        }
        function reportAndStoreErrors(state, proj, errors) {
            reportErrors(state, errors);
            state.projectErrorsReported.set(proj, true);
            if (errors.length) {
                state.diagnostics.set(proj, errors);
            }
        }
        function reportParseConfigFileDiagnostic(state, proj) {
            reportAndStoreErrors(state, proj, [state.configFileCache.get(proj)]);
        }
        function reportErrorSummary(state, buildOrder) {
            if (!state.needsSummary)
                return;
            state.needsSummary = false;
            var canReportSummary = state.watch || !!state.host.reportErrorSummary;
            var diagnostics = state.diagnostics;
            var totalErrors = 0;
            if (isCircularBuildOrder(buildOrder)) {
                reportBuildQueue(state, buildOrder.buildOrder);
                reportErrors(state, buildOrder.circularDiagnostics);
                if (canReportSummary)
                    totalErrors += ts.getErrorCountForSummary(buildOrder.circularDiagnostics);
            }
            else {
                // Report errors from the other projects
                buildOrder.forEach(function (project) {
                    var projectPath = toResolvedConfigFilePath(state, project);
                    if (!state.projectErrorsReported.has(projectPath)) {
                        reportErrors(state, diagnostics.get(projectPath) || ts.emptyArray);
                    }
                });
                if (canReportSummary)
                    diagnostics.forEach(function (singleProjectErrors) { return totalErrors += ts.getErrorCountForSummary(singleProjectErrors); });
            }
            if (state.watch) {
                reportWatchStatus(state, ts.getWatchErrorSummaryDiagnosticMessage(totalErrors), totalErrors);
            }
            else if (state.host.reportErrorSummary) {
                state.host.reportErrorSummary(totalErrors);
            }
        }
        /**
         * Report the build ordering inferred from the current project graph if we're in verbose mode
         */
        function reportBuildQueue(state, buildQueue) {
            if (state.options.verbose) {
                reportStatus(state, ts.Diagnostics.Projects_in_this_build_Colon_0, buildQueue.map(function (s) { return "\r\n    * " + relName(state, s); }).join(""));
            }
        }
        function reportUpToDateStatus(state, configFileName, status) {
            switch (status.type) {
                case ts.UpToDateStatusType.OutOfDateWithSelf:
                    return reportStatus(state, ts.Diagnostics.Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2, relName(state, configFileName), relName(state, status.outOfDateOutputFileName), relName(state, status.newerInputFileName));
                case ts.UpToDateStatusType.OutOfDateWithUpstream:
                    return reportStatus(state, ts.Diagnostics.Project_0_is_out_of_date_because_oldest_output_1_is_older_than_newest_input_2, relName(state, configFileName), relName(state, status.outOfDateOutputFileName), relName(state, status.newerProjectName));
                case ts.UpToDateStatusType.OutputMissing:
                    return reportStatus(state, ts.Diagnostics.Project_0_is_out_of_date_because_output_file_1_does_not_exist, relName(state, configFileName), relName(state, status.missingOutputFileName));
                case ts.UpToDateStatusType.UpToDate:
                    if (status.newestInputFileTime !== undefined) {
                        return reportStatus(state, ts.Diagnostics.Project_0_is_up_to_date_because_newest_input_1_is_older_than_oldest_output_2, relName(state, configFileName), relName(state, status.newestInputFileName || ""), relName(state, status.oldestOutputFileName || ""));
                    }
                    // Don't report anything for "up to date because it was already built" -- too verbose
                    break;
                case ts.UpToDateStatusType.OutOfDateWithPrepend:
                    return reportStatus(state, ts.Diagnostics.Project_0_is_out_of_date_because_output_of_its_dependency_1_has_changed, relName(state, configFileName), relName(state, status.newerProjectName));
                case ts.UpToDateStatusType.UpToDateWithUpstreamTypes:
                    return reportStatus(state, ts.Diagnostics.Project_0_is_up_to_date_with_d_ts_files_from_its_dependencies, relName(state, configFileName));
                case ts.UpToDateStatusType.UpstreamOutOfDate:
                    return reportStatus(state, ts.Diagnostics.Project_0_is_out_of_date_because_its_dependency_1_is_out_of_date, relName(state, configFileName), relName(state, status.upstreamProjectName));
                case ts.UpToDateStatusType.UpstreamBlocked:
                    return reportStatus(state, status.upstreamProjectBlocked ?
                        ts.Diagnostics.Project_0_can_t_be_built_because_its_dependency_1_was_not_built :
                        ts.Diagnostics.Project_0_can_t_be_built_because_its_dependency_1_has_errors, relName(state, configFileName), relName(state, status.upstreamProjectName));
                case ts.UpToDateStatusType.Unbuildable:
                    return reportStatus(state, ts.Diagnostics.Failed_to_parse_file_0_Colon_1, relName(state, configFileName), status.reason);
                case ts.UpToDateStatusType.TsVersionOutputOfDate:
                    return reportStatus(state, ts.Diagnostics.Project_0_is_out_of_date_because_output_for_it_was_generated_with_version_1_that_differs_with_current_version_2, relName(state, configFileName), status.version, ts.version);
                case ts.UpToDateStatusType.ContainerOnly:
                // Don't report status on "solution" projects
                // falls through
                case ts.UpToDateStatusType.ComputingUpstream:
                    // Should never leak from getUptoDateStatusWorker
                    break;
                default:
                    ts.assertType(status);
            }
        }
        /**
         * Report the up-to-date status of a project if we're in verbose mode
         */
        function verboseReportProjectStatus(state, configFileName, status) {
            if (state.options.verbose) {
                reportUpToDateStatus(state, configFileName, status);
            }
        }
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        var server;
        (function (server) {
            /* @internal */
            server.ActionSet = "action::set";
            /* @internal */
            server.ActionInvalidate = "action::invalidate";
            /* @internal */
            server.ActionPackageInstalled = "action::packageInstalled";
            /* @internal */
            server.EventTypesRegistry = "event::typesRegistry";
            /* @internal */
            server.EventBeginInstallTypes = "event::beginInstallTypes";
            /* @internal */
            server.EventEndInstallTypes = "event::endInstallTypes";
            /* @internal */
            server.EventInitializationFailed = "event::initializationFailed";
            /* @internal */
            var Arguments;
            (function (Arguments) {
                Arguments.GlobalCacheLocation = "--globalTypingsCacheLocation";
                Arguments.LogFile = "--logFile";
                Arguments.EnableTelemetry = "--enableTelemetry";
                Arguments.TypingSafeListLocation = "--typingSafeListLocation";
                Arguments.TypesMapLocation = "--typesMapLocation";
                /**
                 * This argument specifies the location of the NPM executable.
                 * typingsInstaller will run the command with `${npmLocation} install ...`.
                 */
                Arguments.NpmLocation = "--npmLocation";
                /**
                 * Flag indicating that the typings installer should try to validate the default npm location.
                 * If the default npm is not found when this flag is enabled, fallback to `npm install`
                 */
                Arguments.ValidateDefaultNpmLocation = "--validateDefaultNpmLocation";
            })(Arguments = server.Arguments || (server.Arguments = {}));
            /* @internal */
            function hasArgument(argumentName) {
                return ts.sys.args.indexOf(argumentName) >= 0;
            }
            server.hasArgument = hasArgument;
            /* @internal */
            function findArgument(argumentName) {
                var index = ts.sys.args.indexOf(argumentName);
                return index >= 0 && index < ts.sys.args.length - 1
                    ? ts.sys.args[index + 1]
                    : undefined;
            }
            server.findArgument = findArgument;
            /* @internal */
            function nowString() {
                // E.g. "12:34:56.789"
                var d = new Date();
                return d.getHours() + ":" + d.getMinutes() + ":" + d.getSeconds() + "." + d.getMilliseconds();
            }
            server.nowString = nowString;
        })(server = ts.server || (ts.server = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var JsTyping;
        (function (JsTyping) {
            function isTypingUpToDate(cachedTyping, availableTypingVersions) {
                var availableVersion = new ts.Version(ts.getProperty(availableTypingVersions, "ts" + ts.versionMajorMinor) || ts.getProperty(availableTypingVersions, "latest"));
                return availableVersion.compareTo(cachedTyping.version) <= 0;
            }
            JsTyping.isTypingUpToDate = isTypingUpToDate;
            JsTyping.nodeCoreModuleList = [
                "assert",
                "async_hooks",
                "buffer",
                "child_process",
                "cluster",
                "console",
                "constants",
                "crypto",
                "dgram",
                "dns",
                "domain",
                "events",
                "fs",
                "http",
                "https",
                "http2",
                "inspector",
                "net",
                "os",
                "path",
                "perf_hooks",
                "process",
                "punycode",
                "querystring",
                "readline",
                "repl",
                "stream",
                "string_decoder",
                "timers",
                "tls",
                "tty",
                "url",
                "util",
                "v8",
                "vm",
                "zlib"
            ];
            JsTyping.nodeCoreModules = ts.arrayToSet(JsTyping.nodeCoreModuleList);
            function nonRelativeModuleNameForTypingCache(moduleName) {
                return JsTyping.nodeCoreModules.has(moduleName) ? "node" : moduleName;
            }
            JsTyping.nonRelativeModuleNameForTypingCache = nonRelativeModuleNameForTypingCache;
            function loadSafeList(host, safeListPath) {
                var result = ts.readConfigFile(safeListPath, function (path) { return host.readFile(path); });
                return ts.createMapFromTemplate(result.config);
            }
            JsTyping.loadSafeList = loadSafeList;
            function loadTypesMap(host, typesMapPath) {
                var result = ts.readConfigFile(typesMapPath, function (path) { return host.readFile(path); });
                if (result.config) {
                    return ts.createMapFromTemplate(result.config.simpleMap);
                }
                return undefined;
            }
            JsTyping.loadTypesMap = loadTypesMap;
            /**
             * @param host is the object providing I/O related operations.
             * @param fileNames are the file names that belong to the same project
             * @param projectRootPath is the path to the project root directory
             * @param safeListPath is the path used to retrieve the safe list
             * @param packageNameToTypingLocation is the map of package names to their cached typing locations and installed versions
             * @param typeAcquisition is used to customize the typing acquisition process
             * @param compilerOptions are used as a source for typing inference
             */
            function discoverTypings(host, log, fileNames, projectRootPath, safeList, packageNameToTypingLocation, typeAcquisition, unresolvedImports, typesRegistry) {
                if (!typeAcquisition || !typeAcquisition.enable) {
                    return { cachedTypingPaths: [], newTypingNames: [], filesToWatch: [] };
                }
                // A typing name to typing file path mapping
                var inferredTypings = ts.createMap();
                // Only infer typings for .js and .jsx files
                fileNames = ts.mapDefined(fileNames, function (fileName) {
                    var path = ts.normalizePath(fileName);
                    if (ts.hasJSFileExtension(path)) {
                        return path;
                    }
                });
                var filesToWatch = [];
                if (typeAcquisition.include)
                    addInferredTypings(typeAcquisition.include, "Explicitly included types");
                var exclude = typeAcquisition.exclude || [];
                // Directories to search for package.json, bower.json and other typing information
                var possibleSearchDirs = ts.arrayToSet(fileNames, ts.getDirectoryPath);
                possibleSearchDirs.set(projectRootPath, true);
                possibleSearchDirs.forEach(function (_true, searchDir) {
                    var packageJsonPath = ts.combinePaths(searchDir, "package.json");
                    getTypingNamesFromJson(packageJsonPath, filesToWatch);
                    var bowerJsonPath = ts.combinePaths(searchDir, "bower.json");
                    getTypingNamesFromJson(bowerJsonPath, filesToWatch);
                    var bowerComponentsPath = ts.combinePaths(searchDir, "bower_components");
                    getTypingNamesFromPackagesFolder(bowerComponentsPath, filesToWatch);
                    var nodeModulesPath = ts.combinePaths(searchDir, "node_modules");
                    getTypingNamesFromPackagesFolder(nodeModulesPath, filesToWatch);
                });
                getTypingNamesFromSourceFileNames(fileNames);
                // add typings for unresolved imports
                if (unresolvedImports) {
                    var module_1 = ts.deduplicate(unresolvedImports.map(nonRelativeModuleNameForTypingCache), ts.equateStringsCaseSensitive, ts.compareStringsCaseSensitive);
                    addInferredTypings(module_1, "Inferred typings from unresolved imports");
                }
                // Add the cached typing locations for inferred typings that are already installed
                packageNameToTypingLocation.forEach(function (typing, name) {
                    var registryEntry = typesRegistry.get(name);
                    if (inferredTypings.has(name) && inferredTypings.get(name) === undefined && registryEntry !== undefined && isTypingUpToDate(typing, registryEntry)) {
                        inferredTypings.set(name, typing.typingLocation);
                    }
                });
                // Remove typings that the user has added to the exclude list
                for (var _i = 0, exclude_1 = exclude; _i < exclude_1.length; _i++) {
                    var excludeTypingName = exclude_1[_i];
                    var didDelete = inferredTypings.delete(excludeTypingName);
                    if (didDelete && log)
                        log("Typing for " + excludeTypingName + " is in exclude list, will be ignored.");
                }
                var newTypingNames = [];
                var cachedTypingPaths = [];
                inferredTypings.forEach(function (inferred, typing) {
                    if (inferred !== undefined) {
                        cachedTypingPaths.push(inferred);
                    }
                    else {
                        newTypingNames.push(typing);
                    }
                });
                var result = { cachedTypingPaths: cachedTypingPaths, newTypingNames: newTypingNames, filesToWatch: filesToWatch };
                if (log)
                    log("Result: " + JSON.stringify(result));
                return result;
                function addInferredTyping(typingName) {
                    if (!inferredTypings.has(typingName)) {
                        inferredTypings.set(typingName, undefined); // TODO: GH#18217
                    }
                }
                function addInferredTypings(typingNames, message) {
                    if (log)
                        log(message + ": " + JSON.stringify(typingNames));
                    ts.forEach(typingNames, addInferredTyping);
                }
                /**
                 * Get the typing info from common package manager json files like package.json or bower.json
                 */
                function getTypingNamesFromJson(jsonPath, filesToWatch) {
                    if (!host.fileExists(jsonPath)) {
                        return;
                    }
                    filesToWatch.push(jsonPath);
                    var jsonConfig = ts.readConfigFile(jsonPath, function (path) { return host.readFile(path); }).config;
                    var jsonTypingNames = ts.flatMap([jsonConfig.dependencies, jsonConfig.devDependencies, jsonConfig.optionalDependencies, jsonConfig.peerDependencies], ts.getOwnKeys);
                    addInferredTypings(jsonTypingNames, "Typing names in '" + jsonPath + "' dependencies");
                }
                /**
                 * Infer typing names from given file names. For example, the file name "jquery-min.2.3.4.js"
                 * should be inferred to the 'jquery' typing name; and "angular-route.1.2.3.js" should be inferred
                 * to the 'angular-route' typing name.
                 * @param fileNames are the names for source files in the project
                 */
                function getTypingNamesFromSourceFileNames(fileNames) {
                    var fromFileNames = ts.mapDefined(fileNames, function (j) {
                        if (!ts.hasJSFileExtension(j))
                            return undefined;
                        var inferredTypingName = ts.removeFileExtension(ts.getBaseFileName(j.toLowerCase()));
                        var cleanedTypingName = ts.removeMinAndVersionNumbers(inferredTypingName);
                        return safeList.get(cleanedTypingName);
                    });
                    if (fromFileNames.length) {
                        addInferredTypings(fromFileNames, "Inferred typings from file names");
                    }
                    var hasJsxFile = ts.some(fileNames, function (f) { return ts.fileExtensionIs(f, ".jsx" /* Jsx */); });
                    if (hasJsxFile) {
                        if (log)
                            log("Inferred 'react' typings due to presence of '.jsx' extension");
                        addInferredTyping("react");
                    }
                }
                /**
                 * Infer typing names from packages folder (ex: node_module, bower_components)
                 * @param packagesFolderPath is the path to the packages folder
                 */
                function getTypingNamesFromPackagesFolder(packagesFolderPath, filesToWatch) {
                    filesToWatch.push(packagesFolderPath);
                    // Todo: add support for ModuleResolutionHost too
                    if (!host.directoryExists(packagesFolderPath)) {
                        return;
                    }
                    // depth of 2, so we access `node_modules/foo` but not `node_modules/foo/bar`
                    var fileNames = host.readDirectory(packagesFolderPath, [".json" /* Json */], /*excludes*/ undefined, /*includes*/ undefined, /*depth*/ 2);
                    if (log)
                        log("Searching for typing names in " + packagesFolderPath + "; all files: " + JSON.stringify(fileNames));
                    var packageNames = [];
                    for (var _i = 0, fileNames_1 = fileNames; _i < fileNames_1.length; _i++) {
                        var fileName = fileNames_1[_i];
                        var normalizedFileName = ts.normalizePath(fileName);
                        var baseFileName = ts.getBaseFileName(normalizedFileName);
                        if (baseFileName !== "package.json" && baseFileName !== "bower.json") {
                            continue;
                        }
                        var result_1 = ts.readConfigFile(normalizedFileName, function (path) { return host.readFile(path); });
                        var packageJson = result_1.config;
                        // npm 3's package.json contains a "_requiredBy" field
                        // we should include all the top level module names for npm 2, and only module names whose
                        // "_requiredBy" field starts with "#" or equals "/" for npm 3.
                        if (baseFileName === "package.json" && packageJson._requiredBy &&
                            ts.filter(packageJson._requiredBy, function (r) { return r[0] === "#" || r === "/"; }).length === 0) {
                            continue;
                        }
                        // If the package has its own d.ts typings, those will take precedence. Otherwise the package name will be used
                        // to download d.ts files from DefinitelyTyped
                        if (!packageJson.name) {
                            continue;
                        }
                        var ownTypes = packageJson.types || packageJson.typings;
                        if (ownTypes) {
                            var absolutePath = ts.getNormalizedAbsolutePath(ownTypes, ts.getDirectoryPath(normalizedFileName));
                            if (log)
                                log("    Package '" + packageJson.name + "' provides its own types.");
                            inferredTypings.set(packageJson.name, absolutePath);
                        }
                        else {
                            packageNames.push(packageJson.name);
                        }
                    }
                    addInferredTypings(packageNames, "    Found package names");
                }
            }
            JsTyping.discoverTypings = discoverTypings;
            var NameValidationResult;
            (function (NameValidationResult) {
                NameValidationResult[NameValidationResult["Ok"] = 0] = "Ok";
                NameValidationResult[NameValidationResult["EmptyName"] = 1] = "EmptyName";
                NameValidationResult[NameValidationResult["NameTooLong"] = 2] = "NameTooLong";
                NameValidationResult[NameValidationResult["NameStartsWithDot"] = 3] = "NameStartsWithDot";
                NameValidationResult[NameValidationResult["NameStartsWithUnderscore"] = 4] = "NameStartsWithUnderscore";
                NameValidationResult[NameValidationResult["NameContainsNonURISafeCharacters"] = 5] = "NameContainsNonURISafeCharacters";
            })(NameValidationResult = JsTyping.NameValidationResult || (JsTyping.NameValidationResult = {}));
            var maxPackageNameLength = 214;
            /**
             * Validates package name using rules defined at https://docs.npmjs.com/files/package.json
             */
            function validatePackageName(packageName) {
                return validatePackageNameWorker(packageName, /*supportScopedPackage*/ true);
            }
            JsTyping.validatePackageName = validatePackageName;
            function validatePackageNameWorker(packageName, supportScopedPackage) {
                if (!packageName) {
                    return 1 /* EmptyName */;
                }
                if (packageName.length > maxPackageNameLength) {
                    return 2 /* NameTooLong */;
                }
                if (packageName.charCodeAt(0) === 46 /* dot */) {
                    return 3 /* NameStartsWithDot */;
                }
                if (packageName.charCodeAt(0) === 95 /* _ */) {
                    return 4 /* NameStartsWithUnderscore */;
                }
                // check if name is scope package like: starts with @ and has one '/' in the middle
                // scoped packages are not currently supported
                if (supportScopedPackage) {
                    var matches = /^@([^/]+)\/([^/]+)$/.exec(packageName);
                    if (matches) {
                        var scopeResult = validatePackageNameWorker(matches[1], /*supportScopedPackage*/ false);
                        if (scopeResult !== 0 /* Ok */) {
                            return { name: matches[1], isScopeName: true, result: scopeResult };
                        }
                        var packageResult = validatePackageNameWorker(matches[2], /*supportScopedPackage*/ false);
                        if (packageResult !== 0 /* Ok */) {
                            return { name: matches[2], isScopeName: false, result: packageResult };
                        }
                        return 0 /* Ok */;
                    }
                }
                if (encodeURIComponent(packageName) !== packageName) {
                    return 5 /* NameContainsNonURISafeCharacters */;
                }
                return 0 /* Ok */;
            }
            function renderPackageNameValidationFailure(result, typing) {
                return typeof result === "object" ?
                    renderPackageNameValidationFailureWorker(typing, result.result, result.name, result.isScopeName) :
                    renderPackageNameValidationFailureWorker(typing, result, typing, /*isScopeName*/ false);
            }
            JsTyping.renderPackageNameValidationFailure = renderPackageNameValidationFailure;
            function renderPackageNameValidationFailureWorker(typing, result, name, isScopeName) {
                var kind = isScopeName ? "Scope" : "Package";
                switch (result) {
                    case 1 /* EmptyName */:
                        return "'" + typing + "':: " + kind + " name '" + name + "' cannot be empty";
                    case 2 /* NameTooLong */:
                        return "'" + typing + "':: " + kind + " name '" + name + "' should be less than " + maxPackageNameLength + " characters";
                    case 3 /* NameStartsWithDot */:
                        return "'" + typing + "':: " + kind + " name '" + name + "' cannot start with '.'";
                    case 4 /* NameStartsWithUnderscore */:
                        return "'" + typing + "':: " + kind + " name '" + name + "' cannot start with '_'";
                    case 5 /* NameContainsNonURISafeCharacters */:
                        return "'" + typing + "':: " + kind + " name '" + name + "' contains non URI safe characters";
                    case 0 /* Ok */:
                        return ts.Debug.fail(); // Shouldn't have called this.
                    default:
                        throw ts.Debug.assertNever(result);
                }
            }
        })(JsTyping = ts.JsTyping || (ts.JsTyping = {}));
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        var ScriptSnapshot;
        (function (ScriptSnapshot) {
            var StringScriptSnapshot = /** @class */ (function () {
                function StringScriptSnapshot(text) {
                    this.text = text;
                }
                StringScriptSnapshot.prototype.getText = function (start, end) {
                    return start === 0 && end === this.text.length
                        ? this.text
                        : this.text.substring(start, end);
                };
                StringScriptSnapshot.prototype.getLength = function () {
                    return this.text.length;
                };
                StringScriptSnapshot.prototype.getChangeRange = function () {
                    // Text-based snapshots do not support incremental parsing. Return undefined
                    // to signal that to the caller.
                    return undefined;
                };
                return StringScriptSnapshot;
            }());
            function fromString(text) {
                return new StringScriptSnapshot(text);
            }
            ScriptSnapshot.fromString = fromString;
        })(ScriptSnapshot = ts.ScriptSnapshot || (ts.ScriptSnapshot = {}));
        /* @internal */
        var PackageJsonDependencyGroup;
        (function (PackageJsonDependencyGroup) {
            PackageJsonDependencyGroup[PackageJsonDependencyGroup["Dependencies"] = 1] = "Dependencies";
            PackageJsonDependencyGroup[PackageJsonDependencyGroup["DevDependencies"] = 2] = "DevDependencies";
            PackageJsonDependencyGroup[PackageJsonDependencyGroup["PeerDependencies"] = 4] = "PeerDependencies";
            PackageJsonDependencyGroup[PackageJsonDependencyGroup["OptionalDependencies"] = 8] = "OptionalDependencies";
            PackageJsonDependencyGroup[PackageJsonDependencyGroup["All"] = 15] = "All";
        })(PackageJsonDependencyGroup = ts.PackageJsonDependencyGroup || (ts.PackageJsonDependencyGroup = {}));
        /* @internal */
        ts.emptyOptions = {};
        var HighlightSpanKind;
        (function (HighlightSpanKind) {
            HighlightSpanKind["none"] = "none";
            HighlightSpanKind["definition"] = "definition";
            HighlightSpanKind["reference"] = "reference";
            HighlightSpanKind["writtenReference"] = "writtenReference";
        })(HighlightSpanKind = ts.HighlightSpanKind || (ts.HighlightSpanKind = {}));
        var IndentStyle;
        (function (IndentStyle) {
            IndentStyle[IndentStyle["None"] = 0] = "None";
            IndentStyle[IndentStyle["Block"] = 1] = "Block";
            IndentStyle[IndentStyle["Smart"] = 2] = "Smart";
        })(IndentStyle = ts.IndentStyle || (ts.IndentStyle = {}));
        var SemicolonPreference;
        (function (SemicolonPreference) {
            SemicolonPreference["Ignore"] = "ignore";
            SemicolonPreference["Insert"] = "insert";
            SemicolonPreference["Remove"] = "remove";
        })(SemicolonPreference = ts.SemicolonPreference || (ts.SemicolonPreference = {}));
        function getDefaultFormatCodeSettings(newLineCharacter) {
            return {
                indentSize: 4,
                tabSize: 4,
                newLineCharacter: newLineCharacter || "\n",
                convertTabsToSpaces: true,
                indentStyle: IndentStyle.Smart,
                insertSpaceAfterConstructor: false,
                insertSpaceAfterCommaDelimiter: true,
                insertSpaceAfterSemicolonInForStatements: true,
                insertSpaceBeforeAndAfterBinaryOperators: true,
                insertSpaceAfterKeywordsInControlFlowStatements: true,
                insertSpaceAfterFunctionKeywordForAnonymousFunctions: false,
                insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis: false,
                insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets: false,
                insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces: true,
                insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces: false,
                insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces: false,
                insertSpaceBeforeFunctionParenthesis: false,
                placeOpenBraceOnNewLineForFunctions: false,
                placeOpenBraceOnNewLineForControlBlocks: false,
                semicolons: SemicolonPreference.Ignore,
            };
        }
        ts.getDefaultFormatCodeSettings = getDefaultFormatCodeSettings;
        /* @internal */
        ts.testFormatSettings = getDefaultFormatCodeSettings("\n");
        var SymbolDisplayPartKind;
        (function (SymbolDisplayPartKind) {
            SymbolDisplayPartKind[SymbolDisplayPartKind["aliasName"] = 0] = "aliasName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["className"] = 1] = "className";
            SymbolDisplayPartKind[SymbolDisplayPartKind["enumName"] = 2] = "enumName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["fieldName"] = 3] = "fieldName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["interfaceName"] = 4] = "interfaceName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["keyword"] = 5] = "keyword";
            SymbolDisplayPartKind[SymbolDisplayPartKind["lineBreak"] = 6] = "lineBreak";
            SymbolDisplayPartKind[SymbolDisplayPartKind["numericLiteral"] = 7] = "numericLiteral";
            SymbolDisplayPartKind[SymbolDisplayPartKind["stringLiteral"] = 8] = "stringLiteral";
            SymbolDisplayPartKind[SymbolDisplayPartKind["localName"] = 9] = "localName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["methodName"] = 10] = "methodName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["moduleName"] = 11] = "moduleName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["operator"] = 12] = "operator";
            SymbolDisplayPartKind[SymbolDisplayPartKind["parameterName"] = 13] = "parameterName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["propertyName"] = 14] = "propertyName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["punctuation"] = 15] = "punctuation";
            SymbolDisplayPartKind[SymbolDisplayPartKind["space"] = 16] = "space";
            SymbolDisplayPartKind[SymbolDisplayPartKind["text"] = 17] = "text";
            SymbolDisplayPartKind[SymbolDisplayPartKind["typeParameterName"] = 18] = "typeParameterName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["enumMemberName"] = 19] = "enumMemberName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["functionName"] = 20] = "functionName";
            SymbolDisplayPartKind[SymbolDisplayPartKind["regularExpressionLiteral"] = 21] = "regularExpressionLiteral";
        })(SymbolDisplayPartKind = ts.SymbolDisplayPartKind || (ts.SymbolDisplayPartKind = {}));
        var OutliningSpanKind;
        (function (OutliningSpanKind) {
            /** Single or multi-line comments */
            OutliningSpanKind["Comment"] = "comment";
            /** Sections marked by '// #region' and '// #endregion' comments */
            OutliningSpanKind["Region"] = "region";
            /** Declarations and expressions */
            OutliningSpanKind["Code"] = "code";
            /** Contiguous blocks of import declarations */
            OutliningSpanKind["Imports"] = "imports";
        })(OutliningSpanKind = ts.OutliningSpanKind || (ts.OutliningSpanKind = {}));
        var OutputFileType;
        (function (OutputFileType) {
            OutputFileType[OutputFileType["JavaScript"] = 0] = "JavaScript";
            OutputFileType[OutputFileType["SourceMap"] = 1] = "SourceMap";
            OutputFileType[OutputFileType["Declaration"] = 2] = "Declaration";
        })(OutputFileType = ts.OutputFileType || (ts.OutputFileType = {}));
        var EndOfLineState;
        (function (EndOfLineState) {
            EndOfLineState[EndOfLineState["None"] = 0] = "None";
            EndOfLineState[EndOfLineState["InMultiLineCommentTrivia"] = 1] = "InMultiLineCommentTrivia";
            EndOfLineState[EndOfLineState["InSingleQuoteStringLiteral"] = 2] = "InSingleQuoteStringLiteral";
            EndOfLineState[EndOfLineState["InDoubleQuoteStringLiteral"] = 3] = "InDoubleQuoteStringLiteral";
            EndOfLineState[EndOfLineState["InTemplateHeadOrNoSubstitutionTemplate"] = 4] = "InTemplateHeadOrNoSubstitutionTemplate";
            EndOfLineState[EndOfLineState["InTemplateMiddleOrTail"] = 5] = "InTemplateMiddleOrTail";
            EndOfLineState[EndOfLineState["InTemplateSubstitutionPosition"] = 6] = "InTemplateSubstitutionPosition";
        })(EndOfLineState = ts.EndOfLineState || (ts.EndOfLineState = {}));
        var TokenClass;
        (function (TokenClass) {
            TokenClass[TokenClass["Punctuation"] = 0] = "Punctuation";
            TokenClass[TokenClass["Keyword"] = 1] = "Keyword";
            TokenClass[TokenClass["Operator"] = 2] = "Operator";
            TokenClass[TokenClass["Comment"] = 3] = "Comment";
            TokenClass[TokenClass["Whitespace"] = 4] = "Whitespace";
            TokenClass[TokenClass["Identifier"] = 5] = "Identifier";
            TokenClass[TokenClass["NumberLiteral"] = 6] = "NumberLiteral";
            TokenClass[TokenClass["BigIntLiteral"] = 7] = "BigIntLiteral";
            TokenClass[TokenClass["StringLiteral"] = 8] = "StringLiteral";
            TokenClass[TokenClass["RegExpLiteral"] = 9] = "RegExpLiteral";
        })(TokenClass = ts.TokenClass || (ts.TokenClass = {}));
        var ScriptElementKind;
        (function (ScriptElementKind) {
            ScriptElementKind["unknown"] = "";
            ScriptElementKind["warning"] = "warning";
            /** predefined type (void) or keyword (class) */
            ScriptElementKind["keyword"] = "keyword";
            /** top level script node */
            ScriptElementKind["scriptElement"] = "script";
            /** module foo {} */
            ScriptElementKind["moduleElement"] = "module";
            /** class X {} */
            ScriptElementKind["classElement"] = "class";
            /** var x = class X {} */
            ScriptElementKind["localClassElement"] = "local class";
            /** interface Y {} */
            ScriptElementKind["interfaceElement"] = "interface";
            /** type T = ... */
            ScriptElementKind["typeElement"] = "type";
            /** enum E */
            ScriptElementKind["enumElement"] = "enum";
            ScriptElementKind["enumMemberElement"] = "enum member";
            /**
             * Inside module and script only
             * const v = ..
             */
            ScriptElementKind["variableElement"] = "var";
            /** Inside function */
            ScriptElementKind["localVariableElement"] = "local var";
            /**
             * Inside module and script only
             * function f() { }
             */
            ScriptElementKind["functionElement"] = "function";
            /** Inside function */
            ScriptElementKind["localFunctionElement"] = "local function";
            /** class X { [public|private]* foo() {} } */
            ScriptElementKind["memberFunctionElement"] = "method";
            /** class X { [public|private]* [get|set] foo:number; } */
            ScriptElementKind["memberGetAccessorElement"] = "getter";
            ScriptElementKind["memberSetAccessorElement"] = "setter";
            /**
             * class X { [public|private]* foo:number; }
             * interface Y { foo:number; }
             */
            ScriptElementKind["memberVariableElement"] = "property";
            /** class X { constructor() { } } */
            ScriptElementKind["constructorImplementationElement"] = "constructor";
            /** interface Y { ():number; } */
            ScriptElementKind["callSignatureElement"] = "call";
            /** interface Y { []:number; } */
            ScriptElementKind["indexSignatureElement"] = "index";
            /** interface Y { new():Y; } */
            ScriptElementKind["constructSignatureElement"] = "construct";
            /** function foo(*Y*: string) */
            ScriptElementKind["parameterElement"] = "parameter";
            ScriptElementKind["typeParameterElement"] = "type parameter";
            ScriptElementKind["primitiveType"] = "primitive type";
            ScriptElementKind["label"] = "label";
            ScriptElementKind["alias"] = "alias";
            ScriptElementKind["constElement"] = "const";
            ScriptElementKind["letElement"] = "let";
            ScriptElementKind["directory"] = "directory";
            ScriptElementKind["externalModuleName"] = "external module name";
            /**
             * <JsxTagName attribute1 attribute2={0} />
             */
            ScriptElementKind["jsxAttribute"] = "JSX attribute";
            /** String literal */
            ScriptElementKind["string"] = "string";
        })(ScriptElementKind = ts.ScriptElementKind || (ts.ScriptElementKind = {}));
        var ScriptElementKindModifier;
        (function (ScriptElementKindModifier) {
            ScriptElementKindModifier["none"] = "";
            ScriptElementKindModifier["publicMemberModifier"] = "public";
            ScriptElementKindModifier["privateMemberModifier"] = "private";
            ScriptElementKindModifier["protectedMemberModifier"] = "protected";
            ScriptElementKindModifier["exportedModifier"] = "export";
            ScriptElementKindModifier["ambientModifier"] = "declare";
            ScriptElementKindModifier["staticModifier"] = "static";
            ScriptElementKindModifier["abstractModifier"] = "abstract";
            ScriptElementKindModifier["optionalModifier"] = "optional";
            ScriptElementKindModifier["dtsModifier"] = ".d.ts";
            ScriptElementKindModifier["tsModifier"] = ".ts";
            ScriptElementKindModifier["tsxModifier"] = ".tsx";
            ScriptElementKindModifier["jsModifier"] = ".js";
            ScriptElementKindModifier["jsxModifier"] = ".jsx";
            ScriptElementKindModifier["jsonModifier"] = ".json";
        })(ScriptElementKindModifier = ts.ScriptElementKindModifier || (ts.ScriptElementKindModifier = {}));
        var ClassificationTypeNames;
        (function (ClassificationTypeNames) {
            ClassificationTypeNames["comment"] = "comment";
            ClassificationTypeNames["identifier"] = "identifier";
            ClassificationTypeNames["keyword"] = "keyword";
            ClassificationTypeNames["numericLiteral"] = "number";
            ClassificationTypeNames["bigintLiteral"] = "bigint";
            ClassificationTypeNames["operator"] = "operator";
            ClassificationTypeNames["stringLiteral"] = "string";
            ClassificationTypeNames["whiteSpace"] = "whitespace";
            ClassificationTypeNames["text"] = "text";
            ClassificationTypeNames["punctuation"] = "punctuation";
            ClassificationTypeNames["className"] = "class name";
            ClassificationTypeNames["enumName"] = "enum name";
            ClassificationTypeNames["interfaceName"] = "interface name";
            ClassificationTypeNames["moduleName"] = "module name";
            ClassificationTypeNames["typeParameterName"] = "type parameter name";
            ClassificationTypeNames["typeAliasName"] = "type alias name";
            ClassificationTypeNames["parameterName"] = "parameter name";
            ClassificationTypeNames["docCommentTagName"] = "doc comment tag name";
            ClassificationTypeNames["jsxOpenTagName"] = "jsx open tag name";
            ClassificationTypeNames["jsxCloseTagName"] = "jsx close tag name";
            ClassificationTypeNames["jsxSelfClosingTagName"] = "jsx self closing tag name";
            ClassificationTypeNames["jsxAttribute"] = "jsx attribute";
            ClassificationTypeNames["jsxText"] = "jsx text";
            ClassificationTypeNames["jsxAttributeStringLiteralValue"] = "jsx attribute string literal value";
        })(ClassificationTypeNames = ts.ClassificationTypeNames || (ts.ClassificationTypeNames = {}));
        var ClassificationType;
        (function (ClassificationType) {
            ClassificationType[ClassificationType["comment"] = 1] = "comment";
            ClassificationType[ClassificationType["identifier"] = 2] = "identifier";
            ClassificationType[ClassificationType["keyword"] = 3] = "keyword";
            ClassificationType[ClassificationType["numericLiteral"] = 4] = "numericLiteral";
            ClassificationType[ClassificationType["operator"] = 5] = "operator";
            ClassificationType[ClassificationType["stringLiteral"] = 6] = "stringLiteral";
            ClassificationType[ClassificationType["regularExpressionLiteral"] = 7] = "regularExpressionLiteral";
            ClassificationType[ClassificationType["whiteSpace"] = 8] = "whiteSpace";
            ClassificationType[ClassificationType["text"] = 9] = "text";
            ClassificationType[ClassificationType["punctuation"] = 10] = "punctuation";
            ClassificationType[ClassificationType["className"] = 11] = "className";
            ClassificationType[ClassificationType["enumName"] = 12] = "enumName";
            ClassificationType[ClassificationType["interfaceName"] = 13] = "interfaceName";
            ClassificationType[ClassificationType["moduleName"] = 14] = "moduleName";
            ClassificationType[ClassificationType["typeParameterName"] = 15] = "typeParameterName";
            ClassificationType[ClassificationType["typeAliasName"] = 16] = "typeAliasName";
            ClassificationType[ClassificationType["parameterName"] = 17] = "parameterName";
            ClassificationType[ClassificationType["docCommentTagName"] = 18] = "docCommentTagName";
            ClassificationType[ClassificationType["jsxOpenTagName"] = 19] = "jsxOpenTagName";
            ClassificationType[ClassificationType["jsxCloseTagName"] = 20] = "jsxCloseTagName";
            ClassificationType[ClassificationType["jsxSelfClosingTagName"] = 21] = "jsxSelfClosingTagName";
            ClassificationType[ClassificationType["jsxAttribute"] = 22] = "jsxAttribute";
            ClassificationType[ClassificationType["jsxText"] = 23] = "jsxText";
            ClassificationType[ClassificationType["jsxAttributeStringLiteralValue"] = 24] = "jsxAttributeStringLiteralValue";
            ClassificationType[ClassificationType["bigintLiteral"] = 25] = "bigintLiteral";
        })(ClassificationType = ts.ClassificationType || (ts.ClassificationType = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        // These utilities are common to multiple language service features.
        //#region
        ts.scanner = ts.createScanner(99 /* Latest */, /*skipTrivia*/ true);
        var SemanticMeaning;
        (function (SemanticMeaning) {
            SemanticMeaning[SemanticMeaning["None"] = 0] = "None";
            SemanticMeaning[SemanticMeaning["Value"] = 1] = "Value";
            SemanticMeaning[SemanticMeaning["Type"] = 2] = "Type";
            SemanticMeaning[SemanticMeaning["Namespace"] = 4] = "Namespace";
            SemanticMeaning[SemanticMeaning["All"] = 7] = "All";
        })(SemanticMeaning = ts.SemanticMeaning || (ts.SemanticMeaning = {}));
        function getMeaningFromDeclaration(node) {
            switch (node.kind) {
                case 242 /* VariableDeclaration */:
                    return ts.isInJSFile(node) && ts.getJSDocEnumTag(node) ? 7 /* All */ : 1 /* Value */;
                case 156 /* Parameter */:
                case 191 /* BindingElement */:
                case 159 /* PropertyDeclaration */:
                case 158 /* PropertySignature */:
                case 281 /* PropertyAssignment */:
                case 282 /* ShorthandPropertyAssignment */:
                case 161 /* MethodDeclaration */:
                case 160 /* MethodSignature */:
                case 162 /* Constructor */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 244 /* FunctionDeclaration */:
                case 201 /* FunctionExpression */:
                case 202 /* ArrowFunction */:
                case 280 /* CatchClause */:
                case 273 /* JsxAttribute */:
                    return 1 /* Value */;
                case 155 /* TypeParameter */:
                case 246 /* InterfaceDeclaration */:
                case 247 /* TypeAliasDeclaration */:
                case 173 /* TypeLiteral */:
                    return 2 /* Type */;
                case 321 /* JSDocTypedefTag */:
                    // If it has no name node, it shares the name with the value declaration below it.
                    return node.name === undefined ? 1 /* Value */ | 2 /* Type */ : 2 /* Type */;
                case 284 /* EnumMember */:
                case 245 /* ClassDeclaration */:
                    return 1 /* Value */ | 2 /* Type */;
                case 249 /* ModuleDeclaration */:
                    if (ts.isAmbientModule(node)) {
                        return 4 /* Namespace */ | 1 /* Value */;
                    }
                    else if (ts.getModuleInstanceState(node) === 1 /* Instantiated */) {
                        return 4 /* Namespace */ | 1 /* Value */;
                    }
                    else {
                        return 4 /* Namespace */;
                    }
                case 248 /* EnumDeclaration */:
                case 257 /* NamedImports */:
                case 258 /* ImportSpecifier */:
                case 253 /* ImportEqualsDeclaration */:
                case 254 /* ImportDeclaration */:
                case 259 /* ExportAssignment */:
                case 260 /* ExportDeclaration */:
                    return 7 /* All */;
                // An external module can be a Value
                case 290 /* SourceFile */:
                    return 4 /* Namespace */ | 1 /* Value */;
            }
            return 7 /* All */;
        }
        ts.getMeaningFromDeclaration = getMeaningFromDeclaration;
        function getMeaningFromLocation(node) {
            node = getAdjustedReferenceLocation(node);
            if (node.kind === 290 /* SourceFile */) {
                return 1 /* Value */;
            }
            else if (node.parent.kind === 259 /* ExportAssignment */ || node.parent.kind === 265 /* ExternalModuleReference */) {
                return 7 /* All */;
            }
            else if (isInRightSideOfInternalImportEqualsDeclaration(node)) {
                return getMeaningFromRightHandSideOfImportEquals(node);
            }
            else if (ts.isDeclarationName(node)) {
                return getMeaningFromDeclaration(node.parent);
            }
            else if (isTypeReference(node)) {
                return 2 /* Type */;
            }
            else if (isNamespaceReference(node)) {
                return 4 /* Namespace */;
            }
            else if (ts.isTypeParameterDeclaration(node.parent)) {
                ts.Debug.assert(ts.isJSDocTemplateTag(node.parent.parent)); // Else would be handled by isDeclarationName
                return 2 /* Type */;
            }
            else if (ts.isLiteralTypeNode(node.parent)) {
                // This might be T["name"], which is actually referencing a property and not a type. So allow both meanings.
                return 2 /* Type */ | 1 /* Value */;
            }
            else {
                return 1 /* Value */;
            }
        }
        ts.getMeaningFromLocation = getMeaningFromLocation;
        function getMeaningFromRightHandSideOfImportEquals(node) {
            //     import a = |b|; // Namespace
            //     import a = |b.c|; // Value, type, namespace
            //     import a = |b.c|.d; // Namespace
            var name = node.kind === 153 /* QualifiedName */ ? node : ts.isQualifiedName(node.parent) && node.parent.right === node ? node.parent : undefined;
            return name && name.parent.kind === 253 /* ImportEqualsDeclaration */ ? 7 /* All */ : 4 /* Namespace */;
        }
        function isInRightSideOfInternalImportEqualsDeclaration(node) {
            while (node.parent.kind === 153 /* QualifiedName */) {
                node = node.parent;
            }
            return ts.isInternalModuleImportEqualsDeclaration(node.parent) && node.parent.moduleReference === node;
        }
        ts.isInRightSideOfInternalImportEqualsDeclaration = isInRightSideOfInternalImportEqualsDeclaration;
        function isNamespaceReference(node) {
            return isQualifiedNameNamespaceReference(node) || isPropertyAccessNamespaceReference(node);
        }
        function isQualifiedNameNamespaceReference(node) {
            var root = node;
            var isLastClause = true;
            if (root.parent.kind === 153 /* QualifiedName */) {
                while (root.parent && root.parent.kind === 153 /* QualifiedName */) {
                    root = root.parent;
                }
                isLastClause = root.right === node;
            }
            return root.parent.kind === 169 /* TypeReference */ && !isLastClause;
        }
        function isPropertyAccessNamespaceReference(node) {
            var root = node;
            var isLastClause = true;
            if (root.parent.kind === 194 /* PropertyAccessExpression */) {
                while (root.parent && root.parent.kind === 194 /* PropertyAccessExpression */) {
                    root = root.parent;
                }
                isLastClause = root.name === node;
            }
            if (!isLastClause && root.parent.kind === 216 /* ExpressionWithTypeArguments */ && root.parent.parent.kind === 279 /* HeritageClause */) {
                var decl = root.parent.parent.parent;
                return (decl.kind === 245 /* ClassDeclaration */ && root.parent.parent.token === 113 /* ImplementsKeyword */) ||
                    (decl.kind === 246 /* InterfaceDeclaration */ && root.parent.parent.token === 90 /* ExtendsKeyword */);
            }
            return false;
        }
        function isTypeReference(node) {
            if (ts.isRightSideOfQualifiedNameOrPropertyAccess(node)) {
                node = node.parent;
            }
            switch (node.kind) {
                case 104 /* ThisKeyword */:
                    return !ts.isExpressionNode(node);
                case 183 /* ThisType */:
                    return true;
            }
            switch (node.parent.kind) {
                case 169 /* TypeReference */:
                    return true;
                case 188 /* ImportType */:
                    return !node.parent.isTypeOf;
                case 216 /* ExpressionWithTypeArguments */:
                    return !ts.isExpressionWithTypeArgumentsInClassExtendsClause(node.parent);
            }
            return false;
        }
        function isCallExpressionTarget(node, includeElementAccess, skipPastOuterExpressions) {
            if (includeElementAccess === void 0) {
                includeElementAccess = false;
            }
            if (skipPastOuterExpressions === void 0) {
                skipPastOuterExpressions = false;
            }
            return isCalleeWorker(node, ts.isCallExpression, selectExpressionOfCallOrNewExpressionOrDecorator, includeElementAccess, skipPastOuterExpressions);
        }
        ts.isCallExpressionTarget = isCallExpressionTarget;
        function isNewExpressionTarget(node, includeElementAccess, skipPastOuterExpressions) {
            if (includeElementAccess === void 0) {
                includeElementAccess = false;
            }
            if (skipPastOuterExpressions === void 0) {
                skipPastOuterExpressions = false;
            }
            return isCalleeWorker(node, ts.isNewExpression, selectExpressionOfCallOrNewExpressionOrDecorator, includeElementAccess, skipPastOuterExpressions);
        }
        ts.isNewExpressionTarget = isNewExpressionTarget;
        function isCallOrNewExpressionTarget(node, includeElementAccess, skipPastOuterExpressions) {
            if (includeElementAccess === void 0) {
                includeElementAccess = false;
            }
            if (skipPastOuterExpressions === void 0) {
                skipPastOuterExpressions = false;
            }
            return isCalleeWorker(node, ts.isCallOrNewExpression, selectExpressionOfCallOrNewExpressionOrDecorator, includeElementAccess, skipPastOuterExpressions);
        }
        ts.isCallOrNewExpressionTarget = isCallOrNewExpressionTarget;
        function isTaggedTemplateTag(node, includeElementAccess, skipPastOuterExpressions) {
            if (includeElementAccess === void 0) {
                includeElementAccess = false;
            }
            if (skipPastOuterExpressions === void 0) {
                skipPastOuterExpressions = false;
            }
            return isCalleeWorker(node, ts.isTaggedTemplateExpression, selectTagOfTaggedTemplateExpression, includeElementAccess, skipPastOuterExpressions);
        }
        ts.isTaggedTemplateTag = isTaggedTemplateTag;
        function isDecoratorTarget(node, includeElementAccess, skipPastOuterExpressions) {
            if (includeElementAccess === void 0) {
                includeElementAccess = false;
            }
            if (skipPastOuterExpressions === void 0) {
                skipPastOuterExpressions = false;
            }
            return isCalleeWorker(node, ts.isDecorator, selectExpressionOfCallOrNewExpressionOrDecorator, includeElementAccess, skipPastOuterExpressions);
        }
        ts.isDecoratorTarget = isDecoratorTarget;
        function isJsxOpeningLikeElementTagName(node, includeElementAccess, skipPastOuterExpressions) {
            if (includeElementAccess === void 0) {
                includeElementAccess = false;
            }
            if (skipPastOuterExpressions === void 0) {
                skipPastOuterExpressions = false;
            }
            return isCalleeWorker(node, ts.isJsxOpeningLikeElement, selectTagNameOfJsxOpeningLikeElement, includeElementAccess, skipPastOuterExpressions);
        }
        ts.isJsxOpeningLikeElementTagName = isJsxOpeningLikeElementTagName;
        function selectExpressionOfCallOrNewExpressionOrDecorator(node) {
            return node.expression;
        }
        function selectTagOfTaggedTemplateExpression(node) {
            return node.tag;
        }
        function selectTagNameOfJsxOpeningLikeElement(node) {
            return node.tagName;
        }
        function isCalleeWorker(node, pred, calleeSelector, includeElementAccess, skipPastOuterExpressions) {
            var target = includeElementAccess ? climbPastPropertyOrElementAccess(node) : climbPastPropertyAccess(node);
            if (skipPastOuterExpressions) {
                target = ts.skipOuterExpressions(target);
            }
            return !!target && !!target.parent && pred(target.parent) && calleeSelector(target.parent) === target;
        }
        function climbPastPropertyAccess(node) {
            return isRightSideOfPropertyAccess(node) ? node.parent : node;
        }
        ts.climbPastPropertyAccess = climbPastPropertyAccess;
        function climbPastPropertyOrElementAccess(node) {
            return isRightSideOfPropertyAccess(node) || isArgumentExpressionOfElementAccess(node) ? node.parent : node;
        }
        ts.climbPastPropertyOrElementAccess = climbPastPropertyOrElementAccess;
        function getTargetLabel(referenceNode, labelName) {
            while (referenceNode) {
                if (referenceNode.kind === 238 /* LabeledStatement */ && referenceNode.label.escapedText === labelName) {
                    return referenceNode.label;
                }
                referenceNode = referenceNode.parent;
            }
            return undefined;
        }
        ts.getTargetLabel = getTargetLabel;
        function hasPropertyAccessExpressionWithName(node, funcName) {
            if (!ts.isPropertyAccessExpression(node.expression)) {
                return false;
            }
            return node.expression.name.text === funcName;
        }
        ts.hasPropertyAccessExpressionWithName = hasPropertyAccessExpressionWithName;
        function isJumpStatementTarget(node) {
            var _a;
            return ts.isIdentifier(node) && ((_a = ts.tryCast(node.parent, ts.isBreakOrContinueStatement)) === null || _a === void 0 ? void 0 : _a.label) === node;
        }
        ts.isJumpStatementTarget = isJumpStatementTarget;
        function isLabelOfLabeledStatement(node) {
            var _a;
            return ts.isIdentifier(node) && ((_a = ts.tryCast(node.parent, ts.isLabeledStatement)) === null || _a === void 0 ? void 0 : _a.label) === node;
        }
        ts.isLabelOfLabeledStatement = isLabelOfLabeledStatement;
        function isLabelName(node) {
            return isLabelOfLabeledStatement(node) || isJumpStatementTarget(node);
        }
        ts.isLabelName = isLabelName;
        function isTagName(node) {
            var _a;
            return ((_a = ts.tryCast(node.parent, ts.isJSDocTag)) === null || _a === void 0 ? void 0 : _a.tagName) === node;
        }
        ts.isTagName = isTagName;
        function isRightSideOfQualifiedName(node) {
            var _a;
            return ((_a = ts.tryCast(node.parent, ts.isQualifiedName)) === null || _a === void 0 ? void 0 : _a.right) === node;
        }
        ts.isRightSideOfQualifiedName = isRightSideOfQualifiedName;
        function isRightSideOfPropertyAccess(node) {
            var _a;
            return ((_a = ts.tryCast(node.parent, ts.isPropertyAccessExpression)) === null || _a === void 0 ? void 0 : _a.name) === node;
        }
        ts.isRightSideOfPropertyAccess = isRightSideOfPropertyAccess;
        function isArgumentExpressionOfElementAccess(node) {
            var _a;
            return ((_a = ts.tryCast(node.parent, ts.isElementAccessExpression)) === null || _a === void 0 ? void 0 : _a.argumentExpression) === node;
        }
        ts.isArgumentExpressionOfElementAccess = isArgumentExpressionOfElementAccess;
        function isNameOfModuleDeclaration(node) {
            var _a;
            return ((_a = ts.tryCast(node.parent, ts.isModuleDeclaration)) === null || _a === void 0 ? void 0 : _a.name) === node;
        }
        ts.isNameOfModuleDeclaration = isNameOfModuleDeclaration;
        function isNameOfFunctionDeclaration(node) {
            var _a;
            return ts.isIdentifier(node) && ((_a = ts.tryCast(node.parent, ts.isFunctionLike)) === null || _a === void 0 ? void 0 : _a.name) === node;
        }
        ts.isNameOfFunctionDeclaration = isNameOfFunctionDeclaration;
        function isLiteralNameOfPropertyDeclarationOrIndexAccess(node) {
            switch (node.parent.kind) {
                case 159 /* PropertyDeclaration */:
                case 158 /* PropertySignature */:
                case 281 /* PropertyAssignment */:
                case 284 /* EnumMember */:
                case 161 /* MethodDeclaration */:
                case 160 /* MethodSignature */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 249 /* ModuleDeclaration */:
                    return ts.getNameOfDeclaration(node.parent) === node;
                case 195 /* ElementAccessExpression */:
                    return node.parent.argumentExpression === node;
                case 154 /* ComputedPropertyName */:
                    return true;
                case 187 /* LiteralType */:
                    return node.parent.parent.kind === 185 /* IndexedAccessType */;
                default:
                    return false;
            }
        }
        ts.isLiteralNameOfPropertyDeclarationOrIndexAccess = isLiteralNameOfPropertyDeclarationOrIndexAccess;
        function isExpressionOfExternalModuleImportEqualsDeclaration(node) {
            return ts.isExternalModuleImportEqualsDeclaration(node.parent.parent) &&
                ts.getExternalModuleImportEqualsDeclarationExpression(node.parent.parent) === node;
        }
        ts.isExpressionOfExternalModuleImportEqualsDeclaration = isExpressionOfExternalModuleImportEqualsDeclaration;
        function getContainerNode(node) {
            if (ts.isJSDocTypeAlias(node)) {
                // This doesn't just apply to the node immediately under the comment, but to everything in its parent's scope.
                // node.parent = the JSDoc comment, node.parent.parent = the node having the comment.
                // Then we get parent again in the loop.
                node = node.parent.parent;
            }
            while (true) {
                node = node.parent;
                if (!node) {
                    return undefined;
                }
                switch (node.kind) {
                    case 290 /* SourceFile */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                    case 245 /* ClassDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 249 /* ModuleDeclaration */:
                        return node;
                }
            }
        }
        ts.getContainerNode = getContainerNode;
        function getNodeKind(node) {
            switch (node.kind) {
                case 290 /* SourceFile */:
                    return ts.isExternalModule(node) ? "module" /* moduleElement */ : "script" /* scriptElement */;
                case 249 /* ModuleDeclaration */:
                    return "module" /* moduleElement */;
                case 245 /* ClassDeclaration */:
                case 214 /* ClassExpression */:
                    return "class" /* classElement */;
                case 246 /* InterfaceDeclaration */: return "interface" /* interfaceElement */;
                case 247 /* TypeAliasDeclaration */:
                case 314 /* JSDocCallbackTag */:
                case 321 /* JSDocTypedefTag */:
                    return "type" /* typeElement */;
                case 248 /* EnumDeclaration */: return "enum" /* enumElement */;
                case 242 /* VariableDeclaration */:
                    return getKindOfVariableDeclaration(node);
                case 191 /* BindingElement */:
                    return getKindOfVariableDeclaration(ts.getRootDeclaration(node));
                case 202 /* ArrowFunction */:
                case 244 /* FunctionDeclaration */:
                case 201 /* FunctionExpression */:
                    return "function" /* functionElement */;
                case 163 /* GetAccessor */: return "getter" /* memberGetAccessorElement */;
                case 164 /* SetAccessor */: return "setter" /* memberSetAccessorElement */;
                case 161 /* MethodDeclaration */:
                case 160 /* MethodSignature */:
                    return "method" /* memberFunctionElement */;
                case 281 /* PropertyAssignment */:
                    var initializer = node.initializer;
                    return ts.isFunctionLike(initializer) ? "method" /* memberFunctionElement */ : "property" /* memberVariableElement */;
                case 159 /* PropertyDeclaration */:
                case 158 /* PropertySignature */:
                case 282 /* ShorthandPropertyAssignment */:
                case 283 /* SpreadAssignment */:
                    return "property" /* memberVariableElement */;
                case 167 /* IndexSignature */: return "index" /* indexSignatureElement */;
                case 166 /* ConstructSignature */: return "construct" /* constructSignatureElement */;
                case 165 /* CallSignature */: return "call" /* callSignatureElement */;
                case 162 /* Constructor */: return "constructor" /* constructorImplementationElement */;
                case 155 /* TypeParameter */: return "type parameter" /* typeParameterElement */;
                case 284 /* EnumMember */: return "enum member" /* enumMemberElement */;
                case 156 /* Parameter */: return ts.hasModifier(node, 92 /* ParameterPropertyModifier */) ? "property" /* memberVariableElement */ : "parameter" /* parameterElement */;
                case 253 /* ImportEqualsDeclaration */:
                case 258 /* ImportSpecifier */:
                case 263 /* ExportSpecifier */:
                case 256 /* NamespaceImport */:
                case 262 /* NamespaceExport */:
                    return "alias" /* alias */;
                case 209 /* BinaryExpression */:
                    var kind = ts.getAssignmentDeclarationKind(node);
                    var right = node.right;
                    switch (kind) {
                        case 7 /* ObjectDefinePropertyValue */:
                        case 8 /* ObjectDefinePropertyExports */:
                        case 9 /* ObjectDefinePrototypeProperty */:
                        case 0 /* None */:
                            return "" /* unknown */;
                        case 1 /* ExportsProperty */:
                        case 2 /* ModuleExports */:
                            var rightKind = getNodeKind(right);
                            return rightKind === "" /* unknown */ ? "const" /* constElement */ : rightKind;
                        case 3 /* PrototypeProperty */:
                            return ts.isFunctionExpression(right) ? "method" /* memberFunctionElement */ : "property" /* memberVariableElement */;
                        case 4 /* ThisProperty */:
                            return "property" /* memberVariableElement */; // property
                        case 5 /* Property */:
                            // static method / property
                            return ts.isFunctionExpression(right) ? "method" /* memberFunctionElement */ : "property" /* memberVariableElement */;
                        case 6 /* Prototype */:
                            return "local class" /* localClassElement */;
                        default: {
                            ts.assertType(kind);
                            return "" /* unknown */;
                        }
                    }
                case 75 /* Identifier */:
                    return ts.isImportClause(node.parent) ? "alias" /* alias */ : "" /* unknown */;
                case 259 /* ExportAssignment */:
                    var scriptKind = getNodeKind(node.expression);
                    // If the expression didn't come back with something (like it does for an identifiers)
                    return scriptKind === "" /* unknown */ ? "const" /* constElement */ : scriptKind;
                default:
                    return "" /* unknown */;
            }
            function getKindOfVariableDeclaration(v) {
                return ts.isVarConst(v)
                    ? "const" /* constElement */
                    : ts.isLet(v)
                        ? "let" /* letElement */
                        : "var" /* variableElement */;
            }
        }
        ts.getNodeKind = getNodeKind;
        function isThis(node) {
            switch (node.kind) {
                case 104 /* ThisKeyword */:
                    // case SyntaxKind.ThisType: TODO: GH#9267
                    return true;
                case 75 /* Identifier */:
                    // 'this' as a parameter
                    return ts.identifierIsThisKeyword(node) && node.parent.kind === 156 /* Parameter */;
                default:
                    return false;
            }
        }
        ts.isThis = isThis;
        // Matches the beginning of a triple slash directive
        var tripleSlashDirectivePrefixRegex = /^\/\/\/\s*</;
        function getLineStartPositionForPosition(position, sourceFile) {
            var lineStarts = ts.getLineStarts(sourceFile);
            var line = sourceFile.getLineAndCharacterOfPosition(position).line;
            return lineStarts[line];
        }
        ts.getLineStartPositionForPosition = getLineStartPositionForPosition;
        function rangeContainsRange(r1, r2) {
            return startEndContainsRange(r1.pos, r1.end, r2);
        }
        ts.rangeContainsRange = rangeContainsRange;
        function rangeContainsRangeExclusive(r1, r2) {
            return rangeContainsPositionExclusive(r1, r2.pos) && rangeContainsPositionExclusive(r1, r2.end);
        }
        ts.rangeContainsRangeExclusive = rangeContainsRangeExclusive;
        function rangeContainsPosition(r, pos) {
            return r.pos <= pos && pos <= r.end;
        }
        ts.rangeContainsPosition = rangeContainsPosition;
        function rangeContainsPositionExclusive(r, pos) {
            return r.pos < pos && pos < r.end;
        }
        ts.rangeContainsPositionExclusive = rangeContainsPositionExclusive;
        function startEndContainsRange(start, end, range) {
            return start <= range.pos && end >= range.end;
        }
        ts.startEndContainsRange = startEndContainsRange;
        function rangeContainsStartEnd(range, start, end) {
            return range.pos <= start && range.end >= end;
        }
        ts.rangeContainsStartEnd = rangeContainsStartEnd;
        function rangeOverlapsWithStartEnd(r1, start, end) {
            return startEndOverlapsWithStartEnd(r1.pos, r1.end, start, end);
        }
        ts.rangeOverlapsWithStartEnd = rangeOverlapsWithStartEnd;
        function nodeOverlapsWithStartEnd(node, sourceFile, start, end) {
            return startEndOverlapsWithStartEnd(node.getStart(sourceFile), node.end, start, end);
        }
        ts.nodeOverlapsWithStartEnd = nodeOverlapsWithStartEnd;
        function startEndOverlapsWithStartEnd(start1, end1, start2, end2) {
            var start = Math.max(start1, start2);
            var end = Math.min(end1, end2);
            return start < end;
        }
        ts.startEndOverlapsWithStartEnd = startEndOverlapsWithStartEnd;
        /**
         * Assumes `candidate.start <= position` holds.
         */
        function positionBelongsToNode(candidate, position, sourceFile) {
            ts.Debug.assert(candidate.pos <= position);
            return position < candidate.end || !isCompletedNode(candidate, sourceFile);
        }
        ts.positionBelongsToNode = positionBelongsToNode;
        function isCompletedNode(n, sourceFile) {
            if (n === undefined || ts.nodeIsMissing(n)) {
                return false;
            }
            switch (n.kind) {
                case 245 /* ClassDeclaration */:
                case 246 /* InterfaceDeclaration */:
                case 248 /* EnumDeclaration */:
                case 193 /* ObjectLiteralExpression */:
                case 189 /* ObjectBindingPattern */:
                case 173 /* TypeLiteral */:
                case 223 /* Block */:
                case 250 /* ModuleBlock */:
                case 251 /* CaseBlock */:
                case 257 /* NamedImports */:
                case 261 /* NamedExports */:
                    return nodeEndsWith(n, 19 /* CloseBraceToken */, sourceFile);
                case 280 /* CatchClause */:
                    return isCompletedNode(n.block, sourceFile);
                case 197 /* NewExpression */:
                    if (!n.arguments) {
                        return true;
                    }
                // falls through
                case 196 /* CallExpression */:
                case 200 /* ParenthesizedExpression */:
                case 182 /* ParenthesizedType */:
                    return nodeEndsWith(n, 21 /* CloseParenToken */, sourceFile);
                case 170 /* FunctionType */:
                case 171 /* ConstructorType */:
                    return isCompletedNode(n.type, sourceFile);
                case 162 /* Constructor */:
                case 163 /* GetAccessor */:
                case 164 /* SetAccessor */:
                case 244 /* FunctionDeclaration */:
                case 201 /* FunctionExpression */:
                case 161 /* MethodDeclaration */:
                case 160 /* MethodSignature */:
                case 166 /* ConstructSignature */:
                case 165 /* CallSignature */:
                case 202 /* ArrowFunction */:
                    if (n.body) {
                        return isCompletedNode(n.body, sourceFile);
                    }
                    if (n.type) {
                        return isCompletedNode(n.type, sourceFile);
                    }
                    // Even though type parameters can be unclosed, we can get away with
                    // having at least a closing paren.
                    return hasChildOfKind(n, 21 /* CloseParenToken */, sourceFile);
                case 249 /* ModuleDeclaration */:
                    return !!n.body && isCompletedNode(n.body, sourceFile);
                case 227 /* IfStatement */:
                    if (n.elseStatement) {
                        return isCompletedNode(n.elseStatement, sourceFile);
                    }
                    return isCompletedNode(n.thenStatement, sourceFile);
                case 226 /* ExpressionStatement */:
                    return isCompletedNode(n.expression, sourceFile) ||
                        hasChildOfKind(n, 26 /* SemicolonToken */, sourceFile);
                case 192 /* ArrayLiteralExpression */:
                case 190 /* ArrayBindingPattern */:
                case 195 /* ElementAccessExpression */:
                case 154 /* ComputedPropertyName */:
                case 175 /* TupleType */:
                    return nodeEndsWith(n, 23 /* CloseBracketToken */, sourceFile);
                case 167 /* IndexSignature */:
                    if (n.type) {
                        return isCompletedNode(n.type, sourceFile);
                    }
                    return hasChildOfKind(n, 23 /* CloseBracketToken */, sourceFile);
                case 277 /* CaseClause */:
                case 278 /* DefaultClause */:
                    // there is no such thing as terminator token for CaseClause/DefaultClause so for simplicity always consider them non-completed
                    return false;
                case 230 /* ForStatement */:
                case 231 /* ForInStatement */:
                case 232 /* ForOfStatement */:
                case 229 /* WhileStatement */:
                    return isCompletedNode(n.statement, sourceFile);
                case 228 /* DoStatement */:
                    // rough approximation: if DoStatement has While keyword - then if node is completed is checking the presence of ')';
                    return hasChildOfKind(n, 111 /* WhileKeyword */, sourceFile)
                        ? nodeEndsWith(n, 21 /* CloseParenToken */, sourceFile)
                        : isCompletedNode(n.statement, sourceFile);
                case 172 /* TypeQuery */:
                    return isCompletedNode(n.exprName, sourceFile);
                case 204 /* TypeOfExpression */:
                case 203 /* DeleteExpression */:
                case 205 /* VoidExpression */:
                case 212 /* YieldExpression */:
                case 213 /* SpreadElement */:
                    var unaryWordExpression = n;
                    return isCompletedNode(unaryWordExpression.expression, sourceFile);
                case 198 /* TaggedTemplateExpression */:
                    return isCompletedNode(n.template, sourceFile);
                case 211 /* TemplateExpression */:
                    var lastSpan = ts.lastOrUndefined(n.templateSpans);
                    return isCompletedNode(lastSpan, sourceFile);
                case 221 /* TemplateSpan */:
                    return ts.nodeIsPresent(n.literal);
                case 260 /* ExportDeclaration */:
                case 254 /* ImportDeclaration */:
                    return ts.nodeIsPresent(n.moduleSpecifier);
                case 207 /* PrefixUnaryExpression */:
                    return isCompletedNode(n.operand, sourceFile);
                case 209 /* BinaryExpression */:
                    return isCompletedNode(n.right, sourceFile);
                case 210 /* ConditionalExpression */:
                    return isCompletedNode(n.whenFalse, sourceFile);
                default:
                    return true;
            }
        }
        /*
         * Checks if node ends with 'expectedLastToken'.
         * If child at position 'length - 1' is 'SemicolonToken' it is skipped and 'expectedLastToken' is compared with child at position 'length - 2'.
         */
        function nodeEndsWith(n, expectedLastToken, sourceFile) {
            var children = n.getChildren(sourceFile);
            if (children.length) {
                var lastChild = ts.last(children);
                if (lastChild.kind === expectedLastToken) {
                    return true;
                }
                else if (lastChild.kind === 26 /* SemicolonToken */ && children.length !== 1) {
                    return children[children.length - 2].kind === expectedLastToken;
                }
            }
            return false;
        }
        function findListItemInfo(node) {
            var list = findContainingList(node);
            // It is possible at this point for syntaxList to be undefined, either if
            // node.parent had no list child, or if none of its list children contained
            // the span of node. If this happens, return undefined. The caller should
            // handle this case.
            if (!list) {
                return undefined;
            }
            var children = list.getChildren();
            var listItemIndex = ts.indexOfNode(children, node);
            return {
                listItemIndex: listItemIndex,
                list: list
            };
        }
        ts.findListItemInfo = findListItemInfo;
        function hasChildOfKind(n, kind, sourceFile) {
            return !!findChildOfKind(n, kind, sourceFile);
        }
        ts.hasChildOfKind = hasChildOfKind;
        function findChildOfKind(n, kind, sourceFile) {
            return ts.find(n.getChildren(sourceFile), function (c) { return c.kind === kind; });
        }
        ts.findChildOfKind = findChildOfKind;
        function findContainingList(node) {
            // The node might be a list element (nonsynthetic) or a comma (synthetic). Either way, it will
            // be parented by the container of the SyntaxList, not the SyntaxList itself.
            // In order to find the list item index, we first need to locate SyntaxList itself and then search
            // for the position of the relevant node (or comma).
            var syntaxList = ts.find(node.parent.getChildren(), function (c) { return ts.isSyntaxList(c) && rangeContainsRange(c, node); });
            // Either we didn't find an appropriate list, or the list must contain us.
            ts.Debug.assert(!syntaxList || ts.contains(syntaxList.getChildren(), node));
            return syntaxList;
        }
        ts.findContainingList = findContainingList;
        function isDefaultModifier(node) {
            return node.kind === 84 /* DefaultKeyword */;
        }
        function isClassKeyword(node) {
            return node.kind === 80 /* ClassKeyword */;
        }
        function isFunctionKeyword(node) {
            return node.kind === 94 /* FunctionKeyword */;
        }
        function getAdjustedLocationForClass(node) {
            if (ts.isNamedDeclaration(node)) {
                return node.name;
            }
            if (ts.isClassDeclaration(node)) {
                // for class and function declarations, use the `default` modifier
                // when the declaration is unnamed.
                var defaultModifier = ts.find(node.modifiers, isDefaultModifier);
                if (defaultModifier)
                    return defaultModifier;
            }
            if (ts.isClassExpression(node)) {
                // for class expressions, use the `class` keyword when the class is unnamed
                var classKeyword = ts.find(node.getChildren(), isClassKeyword);
                if (classKeyword)
                    return classKeyword;
            }
        }
        function getAdjustedLocationForFunction(node) {
            if (ts.isNamedDeclaration(node)) {
                return node.name;
            }
            if (ts.isFunctionDeclaration(node)) {
                // for class and function declarations, use the `default` modifier
                // when the declaration is unnamed.
                var defaultModifier = ts.find(node.modifiers, isDefaultModifier);
                if (defaultModifier)
                    return defaultModifier;
            }
            if (ts.isFunctionExpression(node)) {
                // for function expressions, use the `function` keyword when the function is unnamed
                var functionKeyword = ts.find(node.getChildren(), isFunctionKeyword);
                if (functionKeyword)
                    return functionKeyword;
            }
        }
        function getAdjustedLocationForDeclaration(node, forRename) {
            if (!forRename) {
                switch (node.kind) {
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                        return getAdjustedLocationForClass(node);
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                        return getAdjustedLocationForFunction(node);
                }
            }
            if (ts.isNamedDeclaration(node)) {
                return node.name;
            }
        }
        function getAdjustedLocationForImportDeclaration(node, forRename) {
            if (node.importClause) {
                if (node.importClause.name && node.importClause.namedBindings) {
                    // do not adjust if we have both a name and named bindings
                    return;
                }
                // /**/import [|name|] from ...;
                // import /**/type [|name|] from ...;
                if (node.importClause.name) {
                    return node.importClause.name;
                }
                // /**/import { [|name|] } from ...;
                // /**/import { propertyName as [|name|] } from ...;
                // /**/import * as [|name|] from ...;
                // import /**/type { [|name|] } from ...;
                // import /**/type { propertyName as [|name|] } from ...;
                // import /**/type * as [|name|] from ...;
                if (node.importClause.namedBindings) {
                    if (ts.isNamedImports(node.importClause.namedBindings)) {
                        // do nothing if there is more than one binding
                        var onlyBinding = ts.singleOrUndefined(node.importClause.namedBindings.elements);
                        if (!onlyBinding) {
                            return;
                        }
                        return onlyBinding.name;
                    }
                    else if (ts.isNamespaceImport(node.importClause.namedBindings)) {
                        return node.importClause.namedBindings.name;
                    }
                }
            }
            if (!forRename) {
                // /**/import "[|module|]";
                // /**/import ... from "[|module|]";
                // import /**/type ... from "[|module|]";
                return node.moduleSpecifier;
            }
        }
        function getAdjustedLocationForExportDeclaration(node, forRename) {
            if (node.exportClause) {
                // /**/export { [|name|] } ...
                // /**/export { propertyName as [|name|] } ...
                // /**/export * as [|name|] ...
                // export /**/type { [|name|] } from ...
                // export /**/type { propertyName as [|name|] } from ...
                // export /**/type * as [|name|] ...
                if (ts.isNamedExports(node.exportClause)) {
                    // do nothing if there is more than one binding
                    var onlyBinding = ts.singleOrUndefined(node.exportClause.elements);
                    if (!onlyBinding) {
                        return;
                    }
                    return node.exportClause.elements[0].name;
                }
                else if (ts.isNamespaceExport(node.exportClause)) {
                    return node.exportClause.name;
                }
            }
            if (!forRename) {
                // /**/export * from "[|module|]";
                // export /**/type * from "[|module|]";
                return node.moduleSpecifier;
            }
        }
        function getAdjustedLocationForHeritageClause(node) {
            // /**/extends [|name|]
            // /**/implements [|name|]
            if (node.types.length === 1) {
                return node.types[0].expression;
            }
            // /**/extends name1, name2 ...
            // /**/implements name1, name2 ...
        }
        function getAdjustedLocation(node, forRename) {
            var parent = node.parent;
            // /**/<modifier> [|name|] ...
            // /**/<modifier> <class|interface|type|enum|module|namespace|function|get|set> [|name|] ...
            // /**/<class|interface|type|enum|module|namespace|function|get|set> [|name|] ...
            // /**/import [|name|] = ...
            //
            // NOTE: If the node is a modifier, we don't adjust its location if it is the `default` modifier as that is handled
            // specially by `getSymbolAtLocation`.
            if (ts.isModifier(node) && (forRename || node.kind !== 84 /* DefaultKeyword */) ? ts.contains(parent.modifiers, node) :
                node.kind === 80 /* ClassKeyword */ ? ts.isClassDeclaration(parent) || ts.isClassExpression(node) :
                    node.kind === 94 /* FunctionKeyword */ ? ts.isFunctionDeclaration(parent) || ts.isFunctionExpression(node) :
                        node.kind === 114 /* InterfaceKeyword */ ? ts.isInterfaceDeclaration(parent) :
                            node.kind === 88 /* EnumKeyword */ ? ts.isEnumDeclaration(parent) :
                                node.kind === 145 /* TypeKeyword */ ? ts.isTypeAliasDeclaration(parent) :
                                    node.kind === 136 /* NamespaceKeyword */ || node.kind === 135 /* ModuleKeyword */ ? ts.isModuleDeclaration(parent) :
                                        node.kind === 96 /* ImportKeyword */ ? ts.isImportEqualsDeclaration(parent) :
                                            node.kind === 131 /* GetKeyword */ ? ts.isGetAccessorDeclaration(parent) :
                                                node.kind === 142 /* SetKeyword */ && ts.isSetAccessorDeclaration(parent)) {
                var location = getAdjustedLocationForDeclaration(parent, forRename);
                if (location) {
                    return location;
                }
            }
            // /**/<var|let|const> [|name|] ...
            if ((node.kind === 109 /* VarKeyword */ || node.kind === 81 /* ConstKeyword */ || node.kind === 115 /* LetKeyword */) &&
                ts.isVariableDeclarationList(parent) && parent.declarations.length === 1) {
                var decl = parent.declarations[0];
                if (ts.isIdentifier(decl.name)) {
                    return decl.name;
                }
            }
            if (node.kind === 145 /* TypeKeyword */) {
                // import /**/type [|name|] from ...;
                // import /**/type { [|name|] } from ...;
                // import /**/type { propertyName as [|name|] } from ...;
                // import /**/type ... from "[|module|]";
                if (ts.isImportClause(parent) && parent.isTypeOnly) {
                    var location = getAdjustedLocationForImportDeclaration(parent.parent, forRename);
                    if (location) {
                        return location;
                    }
                }
                // export /**/type { [|name|] } from ...;
                // export /**/type { propertyName as [|name|] } from ...;
                // export /**/type * from "[|module|]";
                // export /**/type * as ... from "[|module|]";
                if (ts.isExportDeclaration(parent) && parent.isTypeOnly) {
                    var location = getAdjustedLocationForExportDeclaration(parent, forRename);
                    if (location) {
                        return location;
                    }
                }
            }
            // import { propertyName /**/as [|name|] } ...
            // import * /**/as [|name|] ...
            // export { propertyName /**/as [|name|] } ...
            // export * /**/as [|name|] ...
            if (node.kind === 123 /* AsKeyword */) {
                if (ts.isImportSpecifier(parent) && parent.propertyName ||
                    ts.isExportSpecifier(parent) && parent.propertyName ||
                    ts.isNamespaceImport(parent) ||
                    ts.isNamespaceExport(parent)) {
                    return parent.name;
                }
                if (ts.isExportDeclaration(parent) && parent.exportClause && ts.isNamespaceExport(parent.exportClause)) {
                    return parent.exportClause.name;
                }
            }
            // /**/import [|name|] from ...;
            // /**/import { [|name|] } from ...;
            // /**/import { propertyName as [|name|] } from ...;
            // /**/import ... from "[|module|]";
            // /**/import "[|module|]";
            if (node.kind === 96 /* ImportKeyword */ && ts.isImportDeclaration(parent)) {
                var location = getAdjustedLocationForImportDeclaration(parent, forRename);
                if (location) {
                    return location;
                }
            }
            if (node.kind === 89 /* ExportKeyword */) {
                // /**/export { [|name|] } ...;
                // /**/export { propertyName as [|name|] } ...;
                // /**/export * from "[|module|]";
                // /**/export * as ... from "[|module|]";
                if (ts.isExportDeclaration(parent)) {
                    var location = getAdjustedLocationForExportDeclaration(parent, forRename);
                    if (location) {
                        return location;
                    }
                }
                // NOTE: We don't adjust the location of the `default` keyword as that is handled specially by `getSymbolAtLocation`.
                // /**/export default [|name|];
                // /**/export = [|name|];
                if (ts.isExportAssignment(parent)) {
                    return ts.skipOuterExpressions(parent.expression);
                }
            }
            // import name = /**/require("[|module|]");
            if (node.kind === 139 /* RequireKeyword */ && ts.isExternalModuleReference(parent)) {
                return parent.expression;
            }
            // import ... /**/from "[|module|]";
            // export ... /**/from "[|module|]";
            if (node.kind === 149 /* FromKeyword */ && (ts.isImportDeclaration(parent) || ts.isExportDeclaration(parent)) && parent.moduleSpecifier) {
                return parent.moduleSpecifier;
            }
            // class ... /**/extends [|name|] ...
            // class ... /**/implements [|name|] ...
            // class ... /**/implements name1, name2 ...
            // interface ... /**/extends [|name|] ...
            // interface ... /**/extends name1, name2 ...
            if ((node.kind === 90 /* ExtendsKeyword */ || node.kind === 113 /* ImplementsKeyword */) && ts.isHeritageClause(parent) && parent.token === node.kind) {
                var location = getAdjustedLocationForHeritageClause(parent);
                if (location) {
                    return location;
                }
            }
            if (node.kind === 90 /* ExtendsKeyword */) {
                // ... <T /**/extends [|U|]> ...
                if (ts.isTypeParameterDeclaration(parent) && parent.constraint && ts.isTypeReferenceNode(parent.constraint)) {
                    return parent.constraint.typeName;
                }
                // ... T /**/extends [|U|] ? ...
                if (ts.isConditionalTypeNode(parent) && ts.isTypeReferenceNode(parent.extendsType)) {
                    return parent.extendsType.typeName;
                }
            }
            // ... T extends /**/infer [|U|] ? ...
            if (node.kind === 132 /* InferKeyword */ && ts.isInferTypeNode(parent)) {
                return parent.typeParameter.name;
            }
            // { [ [|K|] /**/in keyof T]: ... }
            if (node.kind === 97 /* InKeyword */ && ts.isTypeParameterDeclaration(parent) && ts.isMappedTypeNode(parent.parent)) {
                return parent.name;
            }
            // /**/keyof [|T|]
            if (node.kind === 134 /* KeyOfKeyword */ && ts.isTypeOperatorNode(parent) && parent.operator === 134 /* KeyOfKeyword */ &&
                ts.isTypeReferenceNode(parent.type)) {
                return parent.type.typeName;
            }
            // /**/readonly [|name|][]
            if (node.kind === 138 /* ReadonlyKeyword */ && ts.isTypeOperatorNode(parent) && parent.operator === 138 /* ReadonlyKeyword */ &&
                ts.isArrayTypeNode(parent.type) && ts.isTypeReferenceNode(parent.type.elementType)) {
                return parent.type.elementType.typeName;
            }
            if (!forRename) {
                // /**/new [|name|]
                // /**/void [|name|]
                // /**/void obj.[|name|]
                // /**/typeof [|name|]
                // /**/typeof obj.[|name|]
                // /**/await [|name|]
                // /**/await obj.[|name|]
                // /**/yield [|name|]
                // /**/yield obj.[|name|]
                // /**/delete obj.[|name|]
                if (node.kind === 99 /* NewKeyword */ && ts.isNewExpression(parent) ||
                    node.kind === 110 /* VoidKeyword */ && ts.isVoidExpression(parent) ||
                    node.kind === 108 /* TypeOfKeyword */ && ts.isTypeOfExpression(parent) ||
                    node.kind === 127 /* AwaitKeyword */ && ts.isAwaitExpression(parent) ||
                    node.kind === 121 /* YieldKeyword */ && ts.isYieldExpression(parent) ||
                    node.kind === 85 /* DeleteKeyword */ && ts.isDeleteExpression(parent)) {
                    if (parent.expression) {
                        return ts.skipOuterExpressions(parent.expression);
                    }
                }
                // left /**/in [|name|]
                // left /**/instanceof [|name|]
                if ((node.kind === 97 /* InKeyword */ || node.kind === 98 /* InstanceOfKeyword */) && ts.isBinaryExpression(parent) && parent.operatorToken === node) {
                    return ts.skipOuterExpressions(parent.right);
                }
                // left /**/as [|name|]
                if (node.kind === 123 /* AsKeyword */ && ts.isAsExpression(parent) && ts.isTypeReferenceNode(parent.type)) {
                    return parent.type.typeName;
                }
                // for (... /**/in [|name|])
                // for (... /**/of [|name|])
                if (node.kind === 97 /* InKeyword */ && ts.isForInStatement(parent) ||
                    node.kind === 152 /* OfKeyword */ && ts.isForOfStatement(parent)) {
                    return ts.skipOuterExpressions(parent.expression);
                }
            }
            return node;
        }
        /**
         * Adjusts the location used for "find references" and "go to definition" when the cursor was not
         * on a property name.
         */
        function getAdjustedReferenceLocation(node) {
            return getAdjustedLocation(node, /*forRename*/ false);
        }
        ts.getAdjustedReferenceLocation = getAdjustedReferenceLocation;
        /**
         * Adjusts the location used for "rename" when the cursor was not on a property name.
         */
        function getAdjustedRenameLocation(node) {
            return getAdjustedLocation(node, /*forRename*/ true);
        }
        ts.getAdjustedRenameLocation = getAdjustedRenameLocation;
        /**
         * Gets the token whose text has range [start, end) and
         * position >= start and (position < end or (position === end && token is literal or keyword or identifier))
         */
        function getTouchingPropertyName(sourceFile, position) {
            return getTouchingToken(sourceFile, position, function (n) { return ts.isPropertyNameLiteral(n) || ts.isKeyword(n.kind) || ts.isPrivateIdentifier(n); });
        }
        ts.getTouchingPropertyName = getTouchingPropertyName;
        /**
         * Returns the token if position is in [start, end).
         * If position === end, returns the preceding token if includeItemAtEndPosition(previousToken) === true
         */
        function getTouchingToken(sourceFile, position, includePrecedingTokenAtEndPosition) {
            return getTokenAtPositionWorker(sourceFile, position, /*allowPositionInLeadingTrivia*/ false, includePrecedingTokenAtEndPosition, /*includeEndPosition*/ false);
        }
        ts.getTouchingToken = getTouchingToken;
        /** Returns a token if position is in [start-of-leading-trivia, end) */
        function getTokenAtPosition(sourceFile, position) {
            return getTokenAtPositionWorker(sourceFile, position, /*allowPositionInLeadingTrivia*/ true, /*includePrecedingTokenAtEndPosition*/ undefined, /*includeEndPosition*/ false);
        }
        ts.getTokenAtPosition = getTokenAtPosition;
        /** Get the token whose text contains the position */
        function getTokenAtPositionWorker(sourceFile, position, allowPositionInLeadingTrivia, includePrecedingTokenAtEndPosition, includeEndPosition) {
            var current = sourceFile;
            outer: while (true) {
                // find the child that contains 'position'
                for (var _i = 0, _a = current.getChildren(sourceFile); _i < _a.length; _i++) {
                    var child = _a[_i];
                    var start = allowPositionInLeadingTrivia ? child.getFullStart() : child.getStart(sourceFile, /*includeJsDoc*/ true);
                    if (start > position) {
                        // If this child begins after position, then all subsequent children will as well.
                        break;
                    }
                    var end = child.getEnd();
                    if (position < end || (position === end && (child.kind === 1 /* EndOfFileToken */ || includeEndPosition))) {
                        current = child;
                        continue outer;
                    }
                    else if (includePrecedingTokenAtEndPosition && end === position) {
                        var previousToken = findPrecedingToken(position, sourceFile, child);
                        if (previousToken && includePrecedingTokenAtEndPosition(previousToken)) {
                            return previousToken;
                        }
                    }
                }
                return current;
            }
        }
        /**
         * The token on the left of the position is the token that strictly includes the position
         * or sits to the left of the cursor if it is on a boundary. For example
         *
         *   fo|o               -> will return foo
         *   foo <comment> |bar -> will return foo
         *
         */
        function findTokenOnLeftOfPosition(file, position) {
            // Ideally, getTokenAtPosition should return a token. However, it is currently
            // broken, so we do a check to make sure the result was indeed a token.
            var tokenAtPosition = getTokenAtPosition(file, position);
            if (ts.isToken(tokenAtPosition) && position > tokenAtPosition.getStart(file) && position < tokenAtPosition.getEnd()) {
                return tokenAtPosition;
            }
            return findPrecedingToken(position, file);
        }
        ts.findTokenOnLeftOfPosition = findTokenOnLeftOfPosition;
        function findNextToken(previousToken, parent, sourceFile) {
            return find(parent);
            function find(n) {
                if (ts.isToken(n) && n.pos === previousToken.end) {
                    // this is token that starts at the end of previous token - return it
                    return n;
                }
                return ts.firstDefined(n.getChildren(sourceFile), function (child) {
                    var shouldDiveInChildNode = 
                    // previous token is enclosed somewhere in the child
                    (child.pos <= previousToken.pos && child.end > previousToken.end) ||
                        // previous token ends exactly at the beginning of child
                        (child.pos === previousToken.end);
                    return shouldDiveInChildNode && nodeHasTokens(child, sourceFile) ? find(child) : undefined;
                });
            }
        }
        ts.findNextToken = findNextToken;
        /**
         * Finds the rightmost token satisfying `token.end <= position`,
         * excluding `JsxText` tokens containing only whitespace.
         */
        function findPrecedingToken(position, sourceFile, startNode, excludeJsdoc) {
            var result = find(startNode || sourceFile);
            ts.Debug.assert(!(result && isWhiteSpaceOnlyJsxText(result)));
            return result;
            function find(n) {
                if (isNonWhitespaceToken(n) && n.kind !== 1 /* EndOfFileToken */) {
                    return n;
                }
                var children = n.getChildren(sourceFile);
                for (var i = 0; i < children.length; i++) {
                    var child = children[i];
                    // Note that the span of a node's tokens is [node.getStart(...), node.end).
                    // Given that `position < child.end` and child has constituent tokens, we distinguish these cases:
                    // 1) `position` precedes `child`'s tokens or `child` has no tokens (ie: in a comment or whitespace preceding `child`):
                    // we need to find the last token in a previous child.
                    // 2) `position` is within the same span: we recurse on `child`.
                    if (position < child.end) {
                        var start = child.getStart(sourceFile, /*includeJsDoc*/ !excludeJsdoc);
                        var lookInPreviousChild = (start >= position) || // cursor in the leading trivia
                            !nodeHasTokens(child, sourceFile) ||
                            isWhiteSpaceOnlyJsxText(child);
                        if (lookInPreviousChild) {
                            // actual start of the node is past the position - previous token should be at the end of previous child
                            var candidate_1 = findRightmostChildNodeWithTokens(children, /*exclusiveStartPosition*/ i, sourceFile);
                            return candidate_1 && findRightmostToken(candidate_1, sourceFile);
                        }
                        else {
                            // candidate should be in this node
                            return find(child);
                        }
                    }
                }
                ts.Debug.assert(startNode !== undefined || n.kind === 290 /* SourceFile */ || n.kind === 1 /* EndOfFileToken */ || ts.isJSDocCommentContainingNode(n));
                // Here we know that none of child token nodes embrace the position,
                // the only known case is when position is at the end of the file.
                // Try to find the rightmost token in the file without filtering.
                // Namely we are skipping the check: 'position < node.end'
                var candidate = findRightmostChildNodeWithTokens(children, /*exclusiveStartPosition*/ children.length, sourceFile);
                return candidate && findRightmostToken(candidate, sourceFile);
            }
        }
        ts.findPrecedingToken = findPrecedingToken;
        function isNonWhitespaceToken(n) {
            return ts.isToken(n) && !isWhiteSpaceOnlyJsxText(n);
        }
        function findRightmostToken(n, sourceFile) {
            if (isNonWhitespaceToken(n)) {
                return n;
            }
            var children = n.getChildren(sourceFile);
            var candidate = findRightmostChildNodeWithTokens(children, /*exclusiveStartPosition*/ children.length, sourceFile);
            return candidate && findRightmostToken(candidate, sourceFile);
        }
        /**
         * Finds the rightmost child to the left of `children[exclusiveStartPosition]` which is a non-all-whitespace token or has constituent tokens.
         */
        function findRightmostChildNodeWithTokens(children, exclusiveStartPosition, sourceFile) {
            for (var i = exclusiveStartPosition - 1; i >= 0; i--) {
                var child = children[i];
                if (isWhiteSpaceOnlyJsxText(child)) {
                    ts.Debug.assert(i > 0, "`JsxText` tokens should not be the first child of `JsxElement | JsxSelfClosingElement`");
                }
                else if (nodeHasTokens(children[i], sourceFile)) {
                    return children[i];
                }
            }
        }
        function isInString(sourceFile, position, previousToken) {
            if (previousToken === void 0) {
                previousToken = findPrecedingToken(position, sourceFile);
            }
            if (previousToken && ts.isStringTextContainingNode(previousToken)) {
                var start = previousToken.getStart(sourceFile);
                var end = previousToken.getEnd();
                // To be "in" one of these literals, the position has to be:
                //   1. entirely within the token text.
                //   2. at the end position of an unterminated token.
                //   3. at the end of a regular expression (due to trailing flags like '/foo/g').
                if (start < position && position < end) {
                    return true;
                }
                if (position === end) {
                    return !!previousToken.isUnterminated;
                }
            }
            return false;
        }
        ts.isInString = isInString;
        /**
         * returns true if the position is in between the open and close elements of an JSX expression.
         */
        function isInsideJsxElementOrAttribute(sourceFile, position) {
            var token = getTokenAtPosition(sourceFile, position);
            if (!token) {
                return false;
            }
            if (token.kind === 11 /* JsxText */) {
                return true;
            }
            // <div>Hello |</div>
            if (token.kind === 29 /* LessThanToken */ && token.parent.kind === 11 /* JsxText */) {
                return true;
            }
            // <div> { | </div> or <div a={| </div>
            if (token.kind === 29 /* LessThanToken */ && token.parent.kind === 276 /* JsxExpression */) {
                return true;
            }
            // <div> {
            // |
            // } < /div>
            if (token && token.kind === 19 /* CloseBraceToken */ && token.parent.kind === 276 /* JsxExpression */) {
                return true;
            }
            // <div>|</div>
            if (token.kind === 29 /* LessThanToken */ && token.parent.kind === 269 /* JsxClosingElement */) {
                return true;
            }
            return false;
        }
        ts.isInsideJsxElementOrAttribute = isInsideJsxElementOrAttribute;
        function isWhiteSpaceOnlyJsxText(node) {
            return ts.isJsxText(node) && node.containsOnlyTriviaWhiteSpaces;
        }
        function isInTemplateString(sourceFile, position) {
            var token = getTokenAtPosition(sourceFile, position);
            return ts.isTemplateLiteralKind(token.kind) && position > token.getStart(sourceFile);
        }
        ts.isInTemplateString = isInTemplateString;
        function isInJSXText(sourceFile, position) {
            var token = getTokenAtPosition(sourceFile, position);
            if (ts.isJsxText(token)) {
                return true;
            }
            if (token.kind === 18 /* OpenBraceToken */ && ts.isJsxExpression(token.parent) && ts.isJsxElement(token.parent.parent)) {
                return true;
            }
            if (token.kind === 29 /* LessThanToken */ && ts.isJsxOpeningLikeElement(token.parent) && ts.isJsxElement(token.parent.parent)) {
                return true;
            }
            return false;
        }
        ts.isInJSXText = isInJSXText;
        function findPrecedingMatchingToken(token, matchingTokenKind, sourceFile) {
            var tokenKind = token.kind;
            var remainingMatchingTokens = 0;
            while (true) {
                var preceding = findPrecedingToken(token.getFullStart(), sourceFile);
                if (!preceding) {
                    return undefined;
                }
                token = preceding;
                if (token.kind === matchingTokenKind) {
                    if (remainingMatchingTokens === 0) {
                        return token;
                    }
                    remainingMatchingTokens--;
                }
                else if (token.kind === tokenKind) {
                    remainingMatchingTokens++;
                }
            }
        }
        ts.findPrecedingMatchingToken = findPrecedingMatchingToken;
        function removeOptionality(type, isOptionalExpression, isOptionalChain) {
            return isOptionalExpression ? type.getNonNullableType() :
                isOptionalChain ? type.getNonOptionalType() :
                    type;
        }
        ts.removeOptionality = removeOptionality;
        function isPossiblyTypeArgumentPosition(token, sourceFile, checker) {
            var info = getPossibleTypeArgumentsInfo(token, sourceFile);
            return info !== undefined && (ts.isPartOfTypeNode(info.called) ||
                getPossibleGenericSignatures(info.called, info.nTypeArguments, checker).length !== 0 ||
                isPossiblyTypeArgumentPosition(info.called, sourceFile, checker));
        }
        ts.isPossiblyTypeArgumentPosition = isPossiblyTypeArgumentPosition;
        function getPossibleGenericSignatures(called, typeArgumentCount, checker) {
            var type = checker.getTypeAtLocation(called);
            if (ts.isOptionalChain(called.parent)) {
                type = removeOptionality(type, !!called.parent.questionDotToken, /*isOptionalChain*/ true);
            }
            var signatures = ts.isNewExpression(called.parent) ? type.getConstructSignatures() : type.getCallSignatures();
            return signatures.filter(function (candidate) { return !!candidate.typeParameters && candidate.typeParameters.length >= typeArgumentCount; });
        }
        ts.getPossibleGenericSignatures = getPossibleGenericSignatures;
        // Get info for an expression like `f <` that may be the start of type arguments.
        function getPossibleTypeArgumentsInfo(tokenIn, sourceFile) {
            var token = tokenIn;
            // This function determines if the node could be type argument position
            // Since during editing, when type argument list is not complete,
            // the tree could be of any shape depending on the tokens parsed before current node,
            // scanning of the previous identifier followed by "<" before current node would give us better result
            // Note that we also balance out the already provided type arguments, arrays, object literals while doing so
            var remainingLessThanTokens = 0;
            var nTypeArguments = 0;
            while (token) {
                switch (token.kind) {
                    case 29 /* LessThanToken */:
                        // Found the beginning of the generic argument expression
                        token = findPrecedingToken(token.getFullStart(), sourceFile);
                        if (token && token.kind === 28 /* QuestionDotToken */) {
                            token = findPrecedingToken(token.getFullStart(), sourceFile);
                        }
                        if (!token || !ts.isIdentifier(token))
                            return undefined;
                        if (!remainingLessThanTokens) {
                            return ts.isDeclarationName(token) ? undefined : { called: token, nTypeArguments: nTypeArguments };
                        }
                        remainingLessThanTokens--;
                        break;
                    case 49 /* GreaterThanGreaterThanGreaterThanToken */:
                        remainingLessThanTokens = +3;
                        break;
                    case 48 /* GreaterThanGreaterThanToken */:
                        remainingLessThanTokens = +2;
                        break;
                    case 31 /* GreaterThanToken */:
                        remainingLessThanTokens++;
                        break;
                    case 19 /* CloseBraceToken */:
                        // This can be object type, skip until we find the matching open brace token
                        // Skip until the matching open brace token
                        token = findPrecedingMatchingToken(token, 18 /* OpenBraceToken */, sourceFile);
                        if (!token)
                            return undefined;
                        break;
                    case 21 /* CloseParenToken */:
                        // This can be object type, skip until we find the matching open brace token
                        // Skip until the matching open brace token
                        token = findPrecedingMatchingToken(token, 20 /* OpenParenToken */, sourceFile);
                        if (!token)
                            return undefined;
                        break;
                    case 23 /* CloseBracketToken */:
                        // This can be object type, skip until we find the matching open brace token
                        // Skip until the matching open brace token
                        token = findPrecedingMatchingToken(token, 22 /* OpenBracketToken */, sourceFile);
                        if (!token)
                            return undefined;
                        break;
                    // Valid tokens in a type name. Skip.
                    case 27 /* CommaToken */:
                        nTypeArguments++;
                        break;
                    case 38 /* EqualsGreaterThanToken */:
                    // falls through
                    case 75 /* Identifier */:
                    case 10 /* StringLiteral */:
                    case 8 /* NumericLiteral */:
                    case 9 /* BigIntLiteral */:
                    case 106 /* TrueKeyword */:
                    case 91 /* FalseKeyword */:
                    // falls through
                    case 108 /* TypeOfKeyword */:
                    case 90 /* ExtendsKeyword */:
                    case 134 /* KeyOfKeyword */:
                    case 24 /* DotToken */:
                    case 51 /* BarToken */:
                    case 57 /* QuestionToken */:
                    case 58 /* ColonToken */:
                        break;
                    default:
                        if (ts.isTypeNode(token)) {
                            break;
                        }
                        // Invalid token in type
                        return undefined;
                }
                token = findPrecedingToken(token.getFullStart(), sourceFile);
            }
            return undefined;
        }
        ts.getPossibleTypeArgumentsInfo = getPossibleTypeArgumentsInfo;
        /**
         * Returns true if the cursor at position in sourceFile is within a comment.
         *
         * @param tokenAtPosition Must equal `getTokenAtPosition(sourceFile, position)
         * @param predicate Additional predicate to test on the comment range.
         */
        function isInComment(sourceFile, position, tokenAtPosition) {
            return ts.formatting.getRangeOfEnclosingComment(sourceFile, position, /*precedingToken*/ undefined, tokenAtPosition);
        }
        ts.isInComment = isInComment;
        function hasDocComment(sourceFile, position) {
            var token = getTokenAtPosition(sourceFile, position);
            return !!ts.findAncestor(token, ts.isJSDoc);
        }
        ts.hasDocComment = hasDocComment;
        function nodeHasTokens(n, sourceFile) {
            // If we have a token or node that has a non-zero width, it must have tokens.
            // Note: getWidth() does not take trivia into account.
            return n.kind === 1 /* EndOfFileToken */ ? !!n.jsDoc : n.getWidth(sourceFile) !== 0;
        }
        function getNodeModifiers(node) {
            var flags = ts.isDeclaration(node) ? ts.getCombinedModifierFlags(node) : 0 /* None */;
            var result = [];
            if (flags & 8 /* Private */)
                result.push("private" /* privateMemberModifier */);
            if (flags & 16 /* Protected */)
                result.push("protected" /* protectedMemberModifier */);
            if (flags & 4 /* Public */)
                result.push("public" /* publicMemberModifier */);
            if (flags & 32 /* Static */)
                result.push("static" /* staticModifier */);
            if (flags & 128 /* Abstract */)
                result.push("abstract" /* abstractModifier */);
            if (flags & 1 /* Export */)
                result.push("export" /* exportedModifier */);
            if (node.flags & 8388608 /* Ambient */)
                result.push("declare" /* ambientModifier */);
            if (node.kind === 259 /* ExportAssignment */)
                result.push("export" /* exportedModifier */);
            return result.length > 0 ? result.join(",") : "" /* none */;
        }
        ts.getNodeModifiers = getNodeModifiers;
        function getTypeArgumentOrTypeParameterList(node) {
            if (node.kind === 169 /* TypeReference */ || node.kind === 196 /* CallExpression */) {
                return node.typeArguments;
            }
            if (ts.isFunctionLike(node) || node.kind === 245 /* ClassDeclaration */ || node.kind === 246 /* InterfaceDeclaration */) {
                return node.typeParameters;
            }
            return undefined;
        }
        ts.getTypeArgumentOrTypeParameterList = getTypeArgumentOrTypeParameterList;
        function isComment(kind) {
            return kind === 2 /* SingleLineCommentTrivia */ || kind === 3 /* MultiLineCommentTrivia */;
        }
        ts.isComment = isComment;
        function isStringOrRegularExpressionOrTemplateLiteral(kind) {
            if (kind === 10 /* StringLiteral */
                || kind === 13 /* RegularExpressionLiteral */
                || ts.isTemplateLiteralKind(kind)) {
                return true;
            }
            return false;
        }
        ts.isStringOrRegularExpressionOrTemplateLiteral = isStringOrRegularExpressionOrTemplateLiteral;
        function isPunctuation(kind) {
            return 18 /* FirstPunctuation */ <= kind && kind <= 74 /* LastPunctuation */;
        }
        ts.isPunctuation = isPunctuation;
        function isInsideTemplateLiteral(node, position, sourceFile) {
            return ts.isTemplateLiteralKind(node.kind)
                && (node.getStart(sourceFile) < position && position < node.end) || (!!node.isUnterminated && position === node.end);
        }
        ts.isInsideTemplateLiteral = isInsideTemplateLiteral;
        function isAccessibilityModifier(kind) {
            switch (kind) {
                case 119 /* PublicKeyword */:
                case 117 /* PrivateKeyword */:
                case 118 /* ProtectedKeyword */:
                    return true;
            }
            return false;
        }
        ts.isAccessibilityModifier = isAccessibilityModifier;
        function cloneCompilerOptions(options) {
            var result = ts.clone(options);
            ts.setConfigFileInOptions(result, options && options.configFile);
            return result;
        }
        ts.cloneCompilerOptions = cloneCompilerOptions;
        function isArrayLiteralOrObjectLiteralDestructuringPattern(node) {
            if (node.kind === 192 /* ArrayLiteralExpression */ ||
                node.kind === 193 /* ObjectLiteralExpression */) {
                // [a,b,c] from:
                // [a, b, c] = someExpression;
                if (node.parent.kind === 209 /* BinaryExpression */ &&
                    node.parent.left === node &&
                    node.parent.operatorToken.kind === 62 /* EqualsToken */) {
                    return true;
                }
                // [a, b, c] from:
                // for([a, b, c] of expression)
                if (node.parent.kind === 232 /* ForOfStatement */ &&
                    node.parent.initializer === node) {
                    return true;
                }
                // [a, b, c] of
                // [x, [a, b, c] ] = someExpression
                // or
                // {x, a: {a, b, c} } = someExpression
                if (isArrayLiteralOrObjectLiteralDestructuringPattern(node.parent.kind === 281 /* PropertyAssignment */ ? node.parent.parent : node.parent)) {
                    return true;
                }
            }
            return false;
        }
        ts.isArrayLiteralOrObjectLiteralDestructuringPattern = isArrayLiteralOrObjectLiteralDestructuringPattern;
        function isInReferenceComment(sourceFile, position) {
            return isInReferenceCommentWorker(sourceFile, position, /*shouldBeReference*/ true);
        }
        ts.isInReferenceComment = isInReferenceComment;
        function isInNonReferenceComment(sourceFile, position) {
            return isInReferenceCommentWorker(sourceFile, position, /*shouldBeReference*/ false);
        }
        ts.isInNonReferenceComment = isInNonReferenceComment;
        function isInReferenceCommentWorker(sourceFile, position, shouldBeReference) {
            var range = isInComment(sourceFile, position, /*tokenAtPosition*/ undefined);
            return !!range && shouldBeReference === tripleSlashDirectivePrefixRegex.test(sourceFile.text.substring(range.pos, range.end));
        }
        function createTextSpanFromNode(node, sourceFile, endNode) {
            return ts.createTextSpanFromBounds(node.getStart(sourceFile), (endNode || node).getEnd());
        }
        ts.createTextSpanFromNode = createTextSpanFromNode;
        function createTextRangeFromNode(node, sourceFile) {
            return ts.createRange(node.getStart(sourceFile), node.end);
        }
        ts.createTextRangeFromNode = createTextRangeFromNode;
        function createTextSpanFromRange(range) {
            return ts.createTextSpanFromBounds(range.pos, range.end);
        }
        ts.createTextSpanFromRange = createTextSpanFromRange;
        function createTextRangeFromSpan(span) {
            return ts.createRange(span.start, span.start + span.length);
        }
        ts.createTextRangeFromSpan = createTextRangeFromSpan;
        function createTextChangeFromStartLength(start, length, newText) {
            return createTextChange(ts.createTextSpan(start, length), newText);
        }
        ts.createTextChangeFromStartLength = createTextChangeFromStartLength;
        function createTextChange(span, newText) {
            return { span: span, newText: newText };
        }
        ts.createTextChange = createTextChange;
        ts.typeKeywords = [
            125 /* AnyKeyword */,
            124 /* AssertsKeyword */,
            151 /* BigIntKeyword */,
            128 /* BooleanKeyword */,
            91 /* FalseKeyword */,
            134 /* KeyOfKeyword */,
            137 /* NeverKeyword */,
            100 /* NullKeyword */,
            140 /* NumberKeyword */,
            141 /* ObjectKeyword */,
            138 /* ReadonlyKeyword */,
            143 /* StringKeyword */,
            144 /* SymbolKeyword */,
            106 /* TrueKeyword */,
            110 /* VoidKeyword */,
            146 /* UndefinedKeyword */,
            147 /* UniqueKeyword */,
            148 /* UnknownKeyword */,
        ];
        function isTypeKeyword(kind) {
            return ts.contains(ts.typeKeywords, kind);
        }
        ts.isTypeKeyword = isTypeKeyword;
        function isTypeKeywordToken(node) {
            return node.kind === 145 /* TypeKeyword */;
        }
        ts.isTypeKeywordToken = isTypeKeywordToken;
        /** True if the symbol is for an external module, as opposed to a namespace. */
        function isExternalModuleSymbol(moduleSymbol) {
            return !!(moduleSymbol.flags & 1536 /* Module */) && moduleSymbol.name.charCodeAt(0) === 34 /* doubleQuote */;
        }
        ts.isExternalModuleSymbol = isExternalModuleSymbol;
        function nodeSeenTracker() {
            var seen = [];
            return function (node) {
                var id = ts.getNodeId(node);
                return !seen[id] && (seen[id] = true);
            };
        }
        ts.nodeSeenTracker = nodeSeenTracker;
        function getSnapshotText(snap) {
            return snap.getText(0, snap.getLength());
        }
        ts.getSnapshotText = getSnapshotText;
        function repeatString(str, count) {
            var result = "";
            for (var i = 0; i < count; i++) {
                result += str;
            }
            return result;
        }
        ts.repeatString = repeatString;
        function skipConstraint(type) {
            return type.isTypeParameter() ? type.getConstraint() || type : type;
        }
        ts.skipConstraint = skipConstraint;
        function getNameFromPropertyName(name) {
            return name.kind === 154 /* ComputedPropertyName */
                // treat computed property names where expression is string/numeric literal as just string/numeric literal
                ? ts.isStringOrNumericLiteralLike(name.expression) ? name.expression.text : undefined
                : ts.isPrivateIdentifier(name) ? ts.idText(name) : ts.getTextOfIdentifierOrLiteral(name);
        }
        ts.getNameFromPropertyName = getNameFromPropertyName;
        function programContainsEs6Modules(program) {
            return program.getSourceFiles().some(function (s) { return !s.isDeclarationFile && !program.isSourceFileFromExternalLibrary(s) && !!s.externalModuleIndicator; });
        }
        ts.programContainsEs6Modules = programContainsEs6Modules;
        function compilerOptionsIndicateEs6Modules(compilerOptions) {
            return !!compilerOptions.module || compilerOptions.target >= 2 /* ES2015 */ || !!compilerOptions.noEmit;
        }
        ts.compilerOptionsIndicateEs6Modules = compilerOptionsIndicateEs6Modules;
        function hostUsesCaseSensitiveFileNames(host) {
            return host.useCaseSensitiveFileNames ? host.useCaseSensitiveFileNames() : false;
        }
        ts.hostUsesCaseSensitiveFileNames = hostUsesCaseSensitiveFileNames;
        function hostGetCanonicalFileName(host) {
            return ts.createGetCanonicalFileName(hostUsesCaseSensitiveFileNames(host));
        }
        ts.hostGetCanonicalFileName = hostGetCanonicalFileName;
        function makeImportIfNecessary(defaultImport, namedImports, moduleSpecifier, quotePreference) {
            return defaultImport || namedImports && namedImports.length ? makeImport(defaultImport, namedImports, moduleSpecifier, quotePreference) : undefined;
        }
        ts.makeImportIfNecessary = makeImportIfNecessary;
        function makeImport(defaultImport, namedImports, moduleSpecifier, quotePreference, isTypeOnly) {
            return ts.createImportDeclaration(
            /*decorators*/ undefined, 
            /*modifiers*/ undefined, defaultImport || namedImports
                ? ts.createImportClause(defaultImport, namedImports && namedImports.length ? ts.createNamedImports(namedImports) : undefined, isTypeOnly)
                : undefined, typeof moduleSpecifier === "string" ? makeStringLiteral(moduleSpecifier, quotePreference) : moduleSpecifier);
        }
        ts.makeImport = makeImport;
        function makeStringLiteral(text, quotePreference) {
            return ts.createLiteral(text, quotePreference === 0 /* Single */);
        }
        ts.makeStringLiteral = makeStringLiteral;
        var QuotePreference;
        (function (QuotePreference) {
            QuotePreference[QuotePreference["Single"] = 0] = "Single";
            QuotePreference[QuotePreference["Double"] = 1] = "Double";
        })(QuotePreference = ts.QuotePreference || (ts.QuotePreference = {}));
        function quotePreferenceFromString(str, sourceFile) {
            return ts.isStringDoubleQuoted(str, sourceFile) ? 1 /* Double */ : 0 /* Single */;
        }
        ts.quotePreferenceFromString = quotePreferenceFromString;
        function getQuotePreference(sourceFile, preferences) {
            if (preferences.quotePreference && preferences.quotePreference !== "auto") {
                return preferences.quotePreference === "single" ? 0 /* Single */ : 1 /* Double */;
            }
            else {
                var firstModuleSpecifier = sourceFile.imports && ts.find(sourceFile.imports, ts.isStringLiteral);
                return firstModuleSpecifier ? quotePreferenceFromString(firstModuleSpecifier, sourceFile) : 1 /* Double */;
            }
        }
        ts.getQuotePreference = getQuotePreference;
        function getQuoteFromPreference(qp) {
            switch (qp) {
                case 0 /* Single */: return "'";
                case 1 /* Double */: return '"';
                default: return ts.Debug.assertNever(qp);
            }
        }
        ts.getQuoteFromPreference = getQuoteFromPreference;
        function symbolNameNoDefault(symbol) {
            var escaped = symbolEscapedNameNoDefault(symbol);
            return escaped === undefined ? undefined : ts.unescapeLeadingUnderscores(escaped);
        }
        ts.symbolNameNoDefault = symbolNameNoDefault;
        function symbolEscapedNameNoDefault(symbol) {
            if (symbol.escapedName !== "default" /* Default */) {
                return symbol.escapedName;
            }
            return ts.firstDefined(symbol.declarations, function (decl) {
                var name = ts.getNameOfDeclaration(decl);
                return name && name.kind === 75 /* Identifier */ ? name.escapedText : undefined;
            });
        }
        ts.symbolEscapedNameNoDefault = symbolEscapedNameNoDefault;
        function isObjectBindingElementWithoutPropertyName(bindingElement) {
            return ts.isBindingElement(bindingElement) &&
                ts.isObjectBindingPattern(bindingElement.parent) &&
                ts.isIdentifier(bindingElement.name) &&
                !bindingElement.propertyName;
        }
        ts.isObjectBindingElementWithoutPropertyName = isObjectBindingElementWithoutPropertyName;
        function getPropertySymbolFromBindingElement(checker, bindingElement) {
            var typeOfPattern = checker.getTypeAtLocation(bindingElement.parent);
            return typeOfPattern && checker.getPropertyOfType(typeOfPattern, bindingElement.name.text);
        }
        ts.getPropertySymbolFromBindingElement = getPropertySymbolFromBindingElement;
        /**
         * Find symbol of the given property-name and add the symbol to the given result array
         * @param symbol a symbol to start searching for the given propertyName
         * @param propertyName a name of property to search for
         * @param result an array of symbol of found property symbols
         * @param previousIterationSymbolsCache a cache of symbol from previous iterations of calling this function to prevent infinite revisiting of the same symbol.
         *                                The value of previousIterationSymbol is undefined when the function is first called.
         */
        function getPropertySymbolsFromBaseTypes(symbol, propertyName, checker, cb) {
            var seen = ts.createMap();
            return recur(symbol);
            function recur(symbol) {
                // Use `addToSeen` to ensure we don't infinitely recurse in this situation:
                //      interface C extends C {
                //          /*findRef*/propName: string;
                //      }
                if (!(symbol.flags & (32 /* Class */ | 64 /* Interface */)) || !ts.addToSeen(seen, ts.getSymbolId(symbol)))
                    return;
                return ts.firstDefined(symbol.declarations, function (declaration) {
                    return ts.firstDefined(ts.getAllSuperTypeNodes(declaration), function (typeReference) {
                        var type = checker.getTypeAtLocation(typeReference);
                        var propertySymbol = type && type.symbol && checker.getPropertyOfType(type, propertyName);
                        // Visit the typeReference as well to see if it directly or indirectly uses that property
                        return type && propertySymbol && (ts.firstDefined(checker.getRootSymbols(propertySymbol), cb) || recur(type.symbol));
                    });
                });
            }
        }
        ts.getPropertySymbolsFromBaseTypes = getPropertySymbolsFromBaseTypes;
        function isMemberSymbolInBaseType(memberSymbol, checker) {
            return getPropertySymbolsFromBaseTypes(memberSymbol.parent, memberSymbol.name, checker, function (_) { return true; }) || false;
        }
        ts.isMemberSymbolInBaseType = isMemberSymbolInBaseType;
        function getParentNodeInSpan(node, file, span) {
            if (!node)
                return undefined;
            while (node.parent) {
                if (ts.isSourceFile(node.parent) || !spanContainsNode(span, node.parent, file)) {
                    return node;
                }
                node = node.parent;
            }
        }
        ts.getParentNodeInSpan = getParentNodeInSpan;
        function spanContainsNode(span, node, file) {
            return ts.textSpanContainsPosition(span, node.getStart(file)) &&
                node.getEnd() <= ts.textSpanEnd(span);
        }
        function findModifier(node, kind) {
            return node.modifiers && ts.find(node.modifiers, function (m) { return m.kind === kind; });
        }
        ts.findModifier = findModifier;
        function insertImport(changes, sourceFile, importDecl, blankLineBetween) {
            var lastImportDeclaration = ts.findLast(sourceFile.statements, ts.isAnyImportSyntax);
            if (lastImportDeclaration) {
                changes.insertNodeAfter(sourceFile, lastImportDeclaration, importDecl);
            }
            else {
                changes.insertNodeAtTopOfFile(sourceFile, importDecl, blankLineBetween);
            }
        }
        ts.insertImport = insertImport;
        function getTypeKeywordOfTypeOnlyImport(importClause, sourceFile) {
            ts.Debug.assert(importClause.isTypeOnly);
            return ts.cast(importClause.getChildAt(0, sourceFile), isTypeKeywordToken);
        }
        ts.getTypeKeywordOfTypeOnlyImport = getTypeKeywordOfTypeOnlyImport;
        function textSpansEqual(a, b) {
            return !!a && !!b && a.start === b.start && a.length === b.length;
        }
        ts.textSpansEqual = textSpansEqual;
        function documentSpansEqual(a, b) {
            return a.fileName === b.fileName && textSpansEqual(a.textSpan, b.textSpan);
        }
        ts.documentSpansEqual = documentSpansEqual;
        /**
         * Iterates through 'array' by index and performs the callback on each element of array until the callback
         * returns a truthy value, then returns that value.
         * If no such value is found, the callback is applied to each element of array and undefined is returned.
         */
        function forEachUnique(array, callback) {
            if (array) {
                for (var i = 0; i < array.length; i++) {
                    if (array.indexOf(array[i]) === i) {
                        var result = callback(array[i], i);
                        if (result) {
                            return result;
                        }
                    }
                }
            }
            return undefined;
        }
        ts.forEachUnique = forEachUnique;
        // #endregion
        // Display-part writer helpers
        // #region
        function isFirstDeclarationOfSymbolParameter(symbol) {
            return symbol.declarations && symbol.declarations.length > 0 && symbol.declarations[0].kind === 156 /* Parameter */;
        }
        ts.isFirstDeclarationOfSymbolParameter = isFirstDeclarationOfSymbolParameter;
        var displayPartWriter = getDisplayPartWriter();
        function getDisplayPartWriter() {
            var absoluteMaximumLength = ts.defaultMaximumTruncationLength * 10; // A hard cutoff to avoid overloading the messaging channel in worst-case scenarios
            var displayParts;
            var lineStart;
            var indent;
            var length;
            resetWriter();
            var unknownWrite = function (text) { return writeKind(text, ts.SymbolDisplayPartKind.text); };
            return {
                displayParts: function () {
                    var finalText = displayParts.length && displayParts[displayParts.length - 1].text;
                    if (length > absoluteMaximumLength && finalText && finalText !== "...") {
                        if (!ts.isWhiteSpaceLike(finalText.charCodeAt(finalText.length - 1))) {
                            displayParts.push(displayPart(" ", ts.SymbolDisplayPartKind.space));
                        }
                        displayParts.push(displayPart("...", ts.SymbolDisplayPartKind.punctuation));
                    }
                    return displayParts;
                },
                writeKeyword: function (text) { return writeKind(text, ts.SymbolDisplayPartKind.keyword); },
                writeOperator: function (text) { return writeKind(text, ts.SymbolDisplayPartKind.operator); },
                writePunctuation: function (text) { return writeKind(text, ts.SymbolDisplayPartKind.punctuation); },
                writeTrailingSemicolon: function (text) { return writeKind(text, ts.SymbolDisplayPartKind.punctuation); },
                writeSpace: function (text) { return writeKind(text, ts.SymbolDisplayPartKind.space); },
                writeStringLiteral: function (text) { return writeKind(text, ts.SymbolDisplayPartKind.stringLiteral); },
                writeParameter: function (text) { return writeKind(text, ts.SymbolDisplayPartKind.parameterName); },
                writeProperty: function (text) { return writeKind(text, ts.SymbolDisplayPartKind.propertyName); },
                writeLiteral: function (text) { return writeKind(text, ts.SymbolDisplayPartKind.stringLiteral); },
                writeSymbol: writeSymbol,
                writeLine: writeLine,
                write: unknownWrite,
                writeComment: unknownWrite,
                getText: function () { return ""; },
                getTextPos: function () { return 0; },
                getColumn: function () { return 0; },
                getLine: function () { return 0; },
                isAtStartOfLine: function () { return false; },
                hasTrailingWhitespace: function () { return false; },
                hasTrailingComment: function () { return false; },
                rawWrite: ts.notImplemented,
                getIndent: function () { return indent; },
                increaseIndent: function () { indent++; },
                decreaseIndent: function () { indent--; },
                clear: resetWriter,
                trackSymbol: ts.noop,
                reportInaccessibleThisError: ts.noop,
                reportInaccessibleUniqueSymbolError: ts.noop,
                reportPrivateInBaseOfClassExpression: ts.noop,
            };
            function writeIndent() {
                if (length > absoluteMaximumLength)
                    return;
                if (lineStart) {
                    var indentString = ts.getIndentString(indent);
                    if (indentString) {
                        length += indentString.length;
                        displayParts.push(displayPart(indentString, ts.SymbolDisplayPartKind.space));
                    }
                    lineStart = false;
                }
            }
            function writeKind(text, kind) {
                if (length > absoluteMaximumLength)
                    return;
                writeIndent();
                length += text.length;
                displayParts.push(displayPart(text, kind));
            }
            function writeSymbol(text, symbol) {
                if (length > absoluteMaximumLength)
                    return;
                writeIndent();
                length += text.length;
                displayParts.push(symbolPart(text, symbol));
            }
            function writeLine() {
                if (length > absoluteMaximumLength)
                    return;
                length += 1;
                displayParts.push(lineBreakPart());
                lineStart = true;
            }
            function resetWriter() {
                displayParts = [];
                lineStart = true;
                indent = 0;
                length = 0;
            }
        }
        function symbolPart(text, symbol) {
            return displayPart(text, displayPartKind(symbol));
            function displayPartKind(symbol) {
                var flags = symbol.flags;
                if (flags & 3 /* Variable */) {
                    return isFirstDeclarationOfSymbolParameter(symbol) ? ts.SymbolDisplayPartKind.parameterName : ts.SymbolDisplayPartKind.localName;
                }
                else if (flags & 4 /* Property */) {
                    return ts.SymbolDisplayPartKind.propertyName;
                }
                else if (flags & 32768 /* GetAccessor */) {
                    return ts.SymbolDisplayPartKind.propertyName;
                }
                else if (flags & 65536 /* SetAccessor */) {
                    return ts.SymbolDisplayPartKind.propertyName;
                }
                else if (flags & 8 /* EnumMember */) {
                    return ts.SymbolDisplayPartKind.enumMemberName;
                }
                else if (flags & 16 /* Function */) {
                    return ts.SymbolDisplayPartKind.functionName;
                }
                else if (flags & 32 /* Class */) {
                    return ts.SymbolDisplayPartKind.className;
                }
                else if (flags & 64 /* Interface */) {
                    return ts.SymbolDisplayPartKind.interfaceName;
                }
                else if (flags & 384 /* Enum */) {
                    return ts.SymbolDisplayPartKind.enumName;
                }
                else if (flags & 1536 /* Module */) {
                    return ts.SymbolDisplayPartKind.moduleName;
                }
                else if (flags & 8192 /* Method */) {
                    return ts.SymbolDisplayPartKind.methodName;
                }
                else if (flags & 262144 /* TypeParameter */) {
                    return ts.SymbolDisplayPartKind.typeParameterName;
                }
                else if (flags & 524288 /* TypeAlias */) {
                    return ts.SymbolDisplayPartKind.aliasName;
                }
                else if (flags & 2097152 /* Alias */) {
                    return ts.SymbolDisplayPartKind.aliasName;
                }
                return ts.SymbolDisplayPartKind.text;
            }
        }
        ts.symbolPart = symbolPart;
        function displayPart(text, kind) {
            return { text: text, kind: ts.SymbolDisplayPartKind[kind] };
        }
        ts.displayPart = displayPart;
        function spacePart() {
            return displayPart(" ", ts.SymbolDisplayPartKind.space);
        }
        ts.spacePart = spacePart;
        function keywordPart(kind) {
            return displayPart(ts.tokenToString(kind), ts.SymbolDisplayPartKind.keyword);
        }
        ts.keywordPart = keywordPart;
        function punctuationPart(kind) {
            return displayPart(ts.tokenToString(kind), ts.SymbolDisplayPartKind.punctuation);
        }
        ts.punctuationPart = punctuationPart;
        function operatorPart(kind) {
            return displayPart(ts.tokenToString(kind), ts.SymbolDisplayPartKind.operator);
        }
        ts.operatorPart = operatorPart;
        function textOrKeywordPart(text) {
            var kind = ts.stringToToken(text);
            return kind === undefined
                ? textPart(text)
                : keywordPart(kind);
        }
        ts.textOrKeywordPart = textOrKeywordPart;
        function textPart(text) {
            return displayPart(text, ts.SymbolDisplayPartKind.text);
        }
        ts.textPart = textPart;
        var carriageReturnLineFeed = "\r\n";
        /**
         * The default is CRLF.
         */
        function getNewLineOrDefaultFromHost(host, formatSettings) {
            return (formatSettings && formatSettings.newLineCharacter) ||
                (host.getNewLine && host.getNewLine()) ||
                carriageReturnLineFeed;
        }
        ts.getNewLineOrDefaultFromHost = getNewLineOrDefaultFromHost;
        function lineBreakPart() {
            return displayPart("\n", ts.SymbolDisplayPartKind.lineBreak);
        }
        ts.lineBreakPart = lineBreakPart;
        function mapToDisplayParts(writeDisplayParts) {
            try {
                writeDisplayParts(displayPartWriter);
                return displayPartWriter.displayParts();
            }
            finally {
                displayPartWriter.clear();
            }
        }
        ts.mapToDisplayParts = mapToDisplayParts;
        function typeToDisplayParts(typechecker, type, enclosingDeclaration, flags) {
            if (flags === void 0) {
                flags = 0 /* None */;
            }
            return mapToDisplayParts(function (writer) {
                typechecker.writeType(type, enclosingDeclaration, flags | 1024 /* MultilineObjectLiterals */ | 16384 /* UseAliasDefinedOutsideCurrentScope */, writer);
            });
        }
        ts.typeToDisplayParts = typeToDisplayParts;
        function symbolToDisplayParts(typeChecker, symbol, enclosingDeclaration, meaning, flags) {
            if (flags === void 0) {
                flags = 0 /* None */;
            }
            return mapToDisplayParts(function (writer) {
                typeChecker.writeSymbol(symbol, enclosingDeclaration, meaning, flags | 8 /* UseAliasDefinedOutsideCurrentScope */, writer);
            });
        }
        ts.symbolToDisplayParts = symbolToDisplayParts;
        function signatureToDisplayParts(typechecker, signature, enclosingDeclaration, flags) {
            if (flags === void 0) {
                flags = 0 /* None */;
            }
            flags |= 16384 /* UseAliasDefinedOutsideCurrentScope */ | 1024 /* MultilineObjectLiterals */ | 32 /* WriteTypeArgumentsOfSignature */ | 8192 /* OmitParameterModifiers */;
            return mapToDisplayParts(function (writer) {
                typechecker.writeSignature(signature, enclosingDeclaration, flags, /*signatureKind*/ undefined, writer);
            });
        }
        ts.signatureToDisplayParts = signatureToDisplayParts;
        function isImportOrExportSpecifierName(location) {
            return !!location.parent && ts.isImportOrExportSpecifier(location.parent) && location.parent.propertyName === location;
        }
        ts.isImportOrExportSpecifierName = isImportOrExportSpecifierName;
        function scriptKindIs(fileName, host) {
            var scriptKinds = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                scriptKinds[_i - 2] = arguments[_i];
            }
            var scriptKind = getScriptKind(fileName, host);
            return ts.some(scriptKinds, function (k) { return k === scriptKind; });
        }
        ts.scriptKindIs = scriptKindIs;
        function getScriptKind(fileName, host) {
            // First check to see if the script kind was specified by the host. Chances are the host
            // may override the default script kind for the file extension.
            return ts.ensureScriptKind(fileName, host && host.getScriptKind && host.getScriptKind(fileName));
        }
        ts.getScriptKind = getScriptKind;
        function getSymbolTarget(symbol, checker) {
            var next = symbol;
            while (isAliasSymbol(next) || (isTransientSymbol(next) && next.target)) {
                if (isTransientSymbol(next) && next.target) {
                    next = next.target;
                }
                else {
                    next = ts.skipAlias(next, checker);
                }
            }
            return next;
        }
        ts.getSymbolTarget = getSymbolTarget;
        function isTransientSymbol(symbol) {
            return (symbol.flags & 33554432 /* Transient */) !== 0;
        }
        function isAliasSymbol(symbol) {
            return (symbol.flags & 2097152 /* Alias */) !== 0;
        }
        function getUniqueSymbolId(symbol, checker) {
            return ts.getSymbolId(ts.skipAlias(symbol, checker));
        }
        ts.getUniqueSymbolId = getUniqueSymbolId;
        function getFirstNonSpaceCharacterPosition(text, position) {
            while (ts.isWhiteSpaceLike(text.charCodeAt(position))) {
                position += 1;
            }
            return position;
        }
        ts.getFirstNonSpaceCharacterPosition = getFirstNonSpaceCharacterPosition;
        function getPrecedingNonSpaceCharacterPosition(text, position) {
            while (position > -1 && ts.isWhiteSpaceSingleLine(text.charCodeAt(position))) {
                position -= 1;
            }
            return position + 1;
        }
        ts.getPrecedingNonSpaceCharacterPosition = getPrecedingNonSpaceCharacterPosition;
        /**
         * Creates a deep, memberwise clone of a node with no source map location.
         *
         * WARNING: This is an expensive operation and is only intended to be used in refactorings
         * and code fixes (because those are triggered by explicit user actions).
         */
        function getSynthesizedDeepClone(node, includeTrivia) {
            if (includeTrivia === void 0) {
                includeTrivia = true;
            }
            var clone = node && getSynthesizedDeepCloneWorker(node);
            if (clone && !includeTrivia)
                suppressLeadingAndTrailingTrivia(clone);
            return clone;
        }
        ts.getSynthesizedDeepClone = getSynthesizedDeepClone;
        function getSynthesizedDeepCloneWithRenames(node, includeTrivia, renameMap, checker, callback) {
            if (includeTrivia === void 0) {
                includeTrivia = true;
            }
            var clone;
            if (renameMap && checker && ts.isBindingElement(node) && ts.isIdentifier(node.name) && ts.isObjectBindingPattern(node.parent)) {
                var symbol = checker.getSymbolAtLocation(node.name);
                var renameInfo = symbol && renameMap.get(String(ts.getSymbolId(symbol)));
                if (renameInfo && renameInfo.text !== (node.name || node.propertyName).getText()) {
                    clone = ts.createBindingElement(node.dotDotDotToken, node.propertyName || node.name, renameInfo, node.initializer);
                }
            }
            else if (renameMap && checker && ts.isIdentifier(node)) {
                var symbol = checker.getSymbolAtLocation(node);
                var renameInfo = symbol && renameMap.get(String(ts.getSymbolId(symbol)));
                if (renameInfo) {
                    clone = ts.createIdentifier(renameInfo.text);
                }
            }
            if (!clone) {
                clone = getSynthesizedDeepCloneWorker(node, renameMap, checker, callback);
            }
            if (clone && !includeTrivia)
                suppressLeadingAndTrailingTrivia(clone);
            if (callback && clone)
                callback(node, clone);
            return clone;
        }
        ts.getSynthesizedDeepCloneWithRenames = getSynthesizedDeepCloneWithRenames;
        function getSynthesizedDeepCloneWorker(node, renameMap, checker, callback) {
            var visited = (renameMap || checker || callback) ?
                ts.visitEachChild(node, wrapper, ts.nullTransformationContext) :
                ts.visitEachChild(node, getSynthesizedDeepClone, ts.nullTransformationContext);
            if (visited === node) {
                // This only happens for leaf nodes - internal nodes always see their children change.
                var clone_1 = ts.getSynthesizedClone(node);
                if (ts.isStringLiteral(clone_1)) {
                    clone_1.textSourceNode = node;
                }
                else if (ts.isNumericLiteral(clone_1)) {
                    clone_1.numericLiteralFlags = node.numericLiteralFlags;
                }
                return ts.setTextRange(clone_1, node);
            }
            // PERF: As an optimization, rather than calling getSynthesizedClone, we'll update
            // the new node created by visitEachChild with the extra changes getSynthesizedClone
            // would have made.
            visited.parent = undefined;
            return visited;
            function wrapper(node) {
                return getSynthesizedDeepCloneWithRenames(node, /*includeTrivia*/ true, renameMap, checker, callback);
            }
        }
        function getSynthesizedDeepClones(nodes, includeTrivia) {
            if (includeTrivia === void 0) {
                includeTrivia = true;
            }
            return nodes && ts.createNodeArray(nodes.map(function (n) { return getSynthesizedDeepClone(n, includeTrivia); }), nodes.hasTrailingComma);
        }
        ts.getSynthesizedDeepClones = getSynthesizedDeepClones;
        /**
         * Sets EmitFlags to suppress leading and trailing trivia on the node.
         */
        function suppressLeadingAndTrailingTrivia(node) {
            suppressLeadingTrivia(node);
            suppressTrailingTrivia(node);
        }
        ts.suppressLeadingAndTrailingTrivia = suppressLeadingAndTrailingTrivia;
        /**
         * Sets EmitFlags to suppress leading trivia on the node.
         */
        function suppressLeadingTrivia(node) {
            addEmitFlagsRecursively(node, 512 /* NoLeadingComments */, getFirstChild);
        }
        ts.suppressLeadingTrivia = suppressLeadingTrivia;
        /**
         * Sets EmitFlags to suppress trailing trivia on the node.
         */
        function suppressTrailingTrivia(node) {
            addEmitFlagsRecursively(node, 1024 /* NoTrailingComments */, ts.getLastChild);
        }
        ts.suppressTrailingTrivia = suppressTrailingTrivia;
        function addEmitFlagsRecursively(node, flag, getChild) {
            ts.addEmitFlags(node, flag);
            var child = getChild(node);
            if (child)
                addEmitFlagsRecursively(child, flag, getChild);
        }
        function getFirstChild(node) {
            return node.forEachChild(function (child) { return child; });
        }
        function getUniqueName(baseName, sourceFile) {
            var nameText = baseName;
            for (var i = 1; !ts.isFileLevelUniqueName(sourceFile, nameText); i++) {
                nameText = baseName + "_" + i;
            }
            return nameText;
        }
        ts.getUniqueName = getUniqueName;
        /**
         * @return The index of the (only) reference to the extracted symbol.  We want the cursor
         * to be on the reference, rather than the declaration, because it's closer to where the
         * user was before extracting it.
         */
        function getRenameLocation(edits, renameFilename, name, preferLastLocation) {
            var delta = 0;
            var lastPos = -1;
            for (var _i = 0, edits_1 = edits; _i < edits_1.length; _i++) {
                var _a = edits_1[_i], fileName = _a.fileName, textChanges_2 = _a.textChanges;
                ts.Debug.assert(fileName === renameFilename);
                for (var _b = 0, textChanges_1 = textChanges_2; _b < textChanges_1.length; _b++) {
                    var change = textChanges_1[_b];
                    var span = change.span, newText = change.newText;
                    var index = indexInTextChange(newText, name);
                    if (index !== -1) {
                        lastPos = span.start + delta + index;
                        // If the reference comes first, return immediately.
                        if (!preferLastLocation) {
                            return lastPos;
                        }
                    }
                    delta += newText.length - span.length;
                }
            }
            // If the declaration comes first, return the position of the last occurrence.
            ts.Debug.assert(preferLastLocation);
            ts.Debug.assert(lastPos >= 0);
            return lastPos;
        }
        ts.getRenameLocation = getRenameLocation;
        function copyLeadingComments(sourceNode, targetNode, sourceFile, commentKind, hasTrailingNewLine) {
            ts.forEachLeadingCommentRange(sourceFile.text, sourceNode.pos, getAddCommentsFunction(targetNode, sourceFile, commentKind, hasTrailingNewLine, ts.addSyntheticLeadingComment));
        }
        ts.copyLeadingComments = copyLeadingComments;
        function copyTrailingComments(sourceNode, targetNode, sourceFile, commentKind, hasTrailingNewLine) {
            ts.forEachTrailingCommentRange(sourceFile.text, sourceNode.end, getAddCommentsFunction(targetNode, sourceFile, commentKind, hasTrailingNewLine, ts.addSyntheticTrailingComment));
        }
        ts.copyTrailingComments = copyTrailingComments;
        /**
         * This function copies the trailing comments for the token that comes before `sourceNode`, as leading comments of `targetNode`.
         * This is useful because sometimes a comment that refers to `sourceNode` will be a leading comment for `sourceNode`, according to the
         * notion of trivia ownership, and instead will be a trailing comment for the token before `sourceNode`, e.g.:
         * `function foo(\* not leading comment for a *\ a: string) {}`
         * The comment refers to `a` but belongs to the `(` token, but we might want to copy it.
         */
        function copyTrailingAsLeadingComments(sourceNode, targetNode, sourceFile, commentKind, hasTrailingNewLine) {
            ts.forEachTrailingCommentRange(sourceFile.text, sourceNode.pos, getAddCommentsFunction(targetNode, sourceFile, commentKind, hasTrailingNewLine, ts.addSyntheticLeadingComment));
        }
        ts.copyTrailingAsLeadingComments = copyTrailingAsLeadingComments;
        function getAddCommentsFunction(targetNode, sourceFile, commentKind, hasTrailingNewLine, cb) {
            return function (pos, end, kind, htnl) {
                if (kind === 3 /* MultiLineCommentTrivia */) {
                    // Remove leading /*
                    pos += 2;
                    // Remove trailing */
                    end -= 2;
                }
                else {
                    // Remove leading //
                    pos += 2;
                }
                cb(targetNode, commentKind || kind, sourceFile.text.slice(pos, end), hasTrailingNewLine !== undefined ? hasTrailingNewLine : htnl);
            };
        }
        function indexInTextChange(change, name) {
            if (ts.startsWith(change, name))
                return 0;
            // Add a " " to avoid references inside words
            var idx = change.indexOf(" " + name);
            if (idx === -1)
                idx = change.indexOf("." + name);
            if (idx === -1)
                idx = change.indexOf('"' + name);
            return idx === -1 ? -1 : idx + 1;
        }
        function getContextualTypeFromParent(node, checker) {
            var parent = node.parent;
            switch (parent.kind) {
                case 197 /* NewExpression */:
                    return checker.getContextualType(parent);
                case 209 /* BinaryExpression */: {
                    var _a = parent, left = _a.left, operatorToken = _a.operatorToken, right = _a.right;
                    return isEqualityOperatorKind(operatorToken.kind)
                        ? checker.getTypeAtLocation(node === right ? left : right)
                        : checker.getContextualType(node);
                }
                case 277 /* CaseClause */:
                    return parent.expression === node ? getSwitchedType(parent, checker) : undefined;
                default:
                    return checker.getContextualType(node);
            }
        }
        ts.getContextualTypeFromParent = getContextualTypeFromParent;
        function quote(text, preferences) {
            if (/^\d+$/.test(text)) {
                return text;
            }
            // Editors can pass in undefined or empty string - we want to infer the preference in those cases.
            var quotePreference = preferences.quotePreference || "auto";
            var quoted = JSON.stringify(text);
            switch (quotePreference) {
                // TODO use getQuotePreference to infer the actual quote style.
                case "auto":
                case "double":
                    return quoted;
                case "single":
                    return "'" + ts.stripQuotes(quoted).replace("'", "\\'").replace('\\"', '"') + "'";
                default:
                    return ts.Debug.assertNever(quotePreference);
            }
        }
        ts.quote = quote;
        function isEqualityOperatorKind(kind) {
            switch (kind) {
                case 36 /* EqualsEqualsEqualsToken */:
                case 34 /* EqualsEqualsToken */:
                case 37 /* ExclamationEqualsEqualsToken */:
                case 35 /* ExclamationEqualsToken */:
                    return true;
                default:
                    return false;
            }
        }
        ts.isEqualityOperatorKind = isEqualityOperatorKind;
        function isStringLiteralOrTemplate(node) {
            switch (node.kind) {
                case 10 /* StringLiteral */:
                case 14 /* NoSubstitutionTemplateLiteral */:
                case 211 /* TemplateExpression */:
                case 198 /* TaggedTemplateExpression */:
                    return true;
                default:
                    return false;
            }
        }
        ts.isStringLiteralOrTemplate = isStringLiteralOrTemplate;
        function hasIndexSignature(type) {
            return !!type.getStringIndexType() || !!type.getNumberIndexType();
        }
        ts.hasIndexSignature = hasIndexSignature;
        function getSwitchedType(caseClause, checker) {
            return checker.getTypeAtLocation(caseClause.parent.parent.expression);
        }
        ts.getSwitchedType = getSwitchedType;
        function getTypeNodeIfAccessible(type, enclosingScope, program, host) {
            var checker = program.getTypeChecker();
            var typeIsAccessible = true;
            var notAccessible = function () { typeIsAccessible = false; };
            var res = checker.typeToTypeNode(type, enclosingScope, /*flags*/ undefined, {
                trackSymbol: function (symbol, declaration, meaning) {
                    typeIsAccessible = typeIsAccessible && checker.isSymbolAccessible(symbol, declaration, meaning, /*shouldComputeAliasToMarkVisible*/ false).accessibility === 0 /* Accessible */;
                },
                reportInaccessibleThisError: notAccessible,
                reportPrivateInBaseOfClassExpression: notAccessible,
                reportInaccessibleUniqueSymbolError: notAccessible,
                moduleResolverHost: {
                    readFile: host.readFile,
                    fileExists: host.fileExists,
                    directoryExists: host.directoryExists,
                    getSourceFiles: program.getSourceFiles,
                    getCurrentDirectory: program.getCurrentDirectory,
                    getCommonSourceDirectory: program.getCommonSourceDirectory,
                }
            });
            return typeIsAccessible ? res : undefined;
        }
        ts.getTypeNodeIfAccessible = getTypeNodeIfAccessible;
        function syntaxRequiresTrailingCommaOrSemicolonOrASI(kind) {
            return kind === 165 /* CallSignature */
                || kind === 166 /* ConstructSignature */
                || kind === 167 /* IndexSignature */
                || kind === 158 /* PropertySignature */
                || kind === 160 /* MethodSignature */;
        }
        ts.syntaxRequiresTrailingCommaOrSemicolonOrASI = syntaxRequiresTrailingCommaOrSemicolonOrASI;
        function syntaxRequiresTrailingFunctionBlockOrSemicolonOrASI(kind) {
            return kind === 244 /* FunctionDeclaration */
                || kind === 162 /* Constructor */
                || kind === 161 /* MethodDeclaration */
                || kind === 163 /* GetAccessor */
                || kind === 164 /* SetAccessor */;
        }
        ts.syntaxRequiresTrailingFunctionBlockOrSemicolonOrASI = syntaxRequiresTrailingFunctionBlockOrSemicolonOrASI;
        function syntaxRequiresTrailingModuleBlockOrSemicolonOrASI(kind) {
            return kind === 249 /* ModuleDeclaration */;
        }
        ts.syntaxRequiresTrailingModuleBlockOrSemicolonOrASI = syntaxRequiresTrailingModuleBlockOrSemicolonOrASI;
        function syntaxRequiresTrailingSemicolonOrASI(kind) {
            return kind === 225 /* VariableStatement */
                || kind === 226 /* ExpressionStatement */
                || kind === 228 /* DoStatement */
                || kind === 233 /* ContinueStatement */
                || kind === 234 /* BreakStatement */
                || kind === 235 /* ReturnStatement */
                || kind === 239 /* ThrowStatement */
                || kind === 241 /* DebuggerStatement */
                || kind === 159 /* PropertyDeclaration */
                || kind === 247 /* TypeAliasDeclaration */
                || kind === 254 /* ImportDeclaration */
                || kind === 253 /* ImportEqualsDeclaration */
                || kind === 260 /* ExportDeclaration */
                || kind === 252 /* NamespaceExportDeclaration */
                || kind === 259 /* ExportAssignment */;
        }
        ts.syntaxRequiresTrailingSemicolonOrASI = syntaxRequiresTrailingSemicolonOrASI;
        ts.syntaxMayBeASICandidate = ts.or(syntaxRequiresTrailingCommaOrSemicolonOrASI, syntaxRequiresTrailingFunctionBlockOrSemicolonOrASI, syntaxRequiresTrailingModuleBlockOrSemicolonOrASI, syntaxRequiresTrailingSemicolonOrASI);
        function nodeIsASICandidate(node, sourceFile) {
            var lastToken = node.getLastToken(sourceFile);
            if (lastToken && lastToken.kind === 26 /* SemicolonToken */) {
                return false;
            }
            if (syntaxRequiresTrailingCommaOrSemicolonOrASI(node.kind)) {
                if (lastToken && lastToken.kind === 27 /* CommaToken */) {
                    return false;
                }
            }
            else if (syntaxRequiresTrailingModuleBlockOrSemicolonOrASI(node.kind)) {
                var lastChild = ts.last(node.getChildren(sourceFile));
                if (lastChild && ts.isModuleBlock(lastChild)) {
                    return false;
                }
            }
            else if (syntaxRequiresTrailingFunctionBlockOrSemicolonOrASI(node.kind)) {
                var lastChild = ts.last(node.getChildren(sourceFile));
                if (lastChild && ts.isFunctionBlock(lastChild)) {
                    return false;
                }
            }
            else if (!syntaxRequiresTrailingSemicolonOrASI(node.kind)) {
                return false;
            }
            // See comment in parser’s `parseDoStatement`
            if (node.kind === 228 /* DoStatement */) {
                return true;
            }
            var topNode = ts.findAncestor(node, function (ancestor) { return !ancestor.parent; });
            var nextToken = findNextToken(node, topNode, sourceFile);
            if (!nextToken || nextToken.kind === 19 /* CloseBraceToken */) {
                return true;
            }
            var startLine = sourceFile.getLineAndCharacterOfPosition(node.getEnd()).line;
            var endLine = sourceFile.getLineAndCharacterOfPosition(nextToken.getStart(sourceFile)).line;
            return startLine !== endLine;
        }
        function positionIsASICandidate(pos, context, sourceFile) {
            var contextAncestor = ts.findAncestor(context, function (ancestor) {
                if (ancestor.end !== pos) {
                    return "quit";
                }
                return ts.syntaxMayBeASICandidate(ancestor.kind);
            });
            return !!contextAncestor && nodeIsASICandidate(contextAncestor, sourceFile);
        }
        ts.positionIsASICandidate = positionIsASICandidate;
        function probablyUsesSemicolons(sourceFile) {
            var withSemicolon = 0;
            var withoutSemicolon = 0;
            var nStatementsToObserve = 5;
            ts.forEachChild(sourceFile, function visit(node) {
                if (syntaxRequiresTrailingSemicolonOrASI(node.kind)) {
                    var lastToken = node.getLastToken(sourceFile);
                    if (lastToken && lastToken.kind === 26 /* SemicolonToken */) {
                        withSemicolon++;
                    }
                    else {
                        withoutSemicolon++;
                    }
                }
                if (withSemicolon + withoutSemicolon >= nStatementsToObserve) {
                    return true;
                }
                return ts.forEachChild(node, visit);
            });
            // One statement missing a semicolon isn’t sufficient evidence to say the user
            // doesn’t want semicolons, because they may not even be done writing that statement.
            if (withSemicolon === 0 && withoutSemicolon <= 1) {
                return true;
            }
            // If even 2/5 places have a semicolon, the user probably wants semicolons
            return withSemicolon / withoutSemicolon > 1 / nStatementsToObserve;
        }
        ts.probablyUsesSemicolons = probablyUsesSemicolons;
        function tryGetDirectories(host, directoryName) {
            return tryIOAndConsumeErrors(host, host.getDirectories, directoryName) || [];
        }
        ts.tryGetDirectories = tryGetDirectories;
        function tryReadDirectory(host, path, extensions, exclude, include) {
            return tryIOAndConsumeErrors(host, host.readDirectory, path, extensions, exclude, include) || ts.emptyArray;
        }
        ts.tryReadDirectory = tryReadDirectory;
        function tryFileExists(host, path) {
            return tryIOAndConsumeErrors(host, host.fileExists, path);
        }
        ts.tryFileExists = tryFileExists;
        function tryDirectoryExists(host, path) {
            return tryAndIgnoreErrors(function () { return ts.directoryProbablyExists(path, host); }) || false;
        }
        ts.tryDirectoryExists = tryDirectoryExists;
        function tryAndIgnoreErrors(cb) {
            try {
                return cb();
            }
            catch (_a) {
                return undefined;
            }
        }
        ts.tryAndIgnoreErrors = tryAndIgnoreErrors;
        function tryIOAndConsumeErrors(host, toApply) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            return tryAndIgnoreErrors(function () { return toApply && toApply.apply(host, args); });
        }
        ts.tryIOAndConsumeErrors = tryIOAndConsumeErrors;
        function findPackageJsons(startDirectory, host, stopDirectory) {
            var paths = [];
            ts.forEachAncestorDirectory(startDirectory, function (ancestor) {
                if (ancestor === stopDirectory) {
                    return true;
                }
                var currentConfigPath = ts.combinePaths(ancestor, "package.json");
                if (tryFileExists(host, currentConfigPath)) {
                    paths.push(currentConfigPath);
                }
            });
            return paths;
        }
        ts.findPackageJsons = findPackageJsons;
        function findPackageJson(directory, host) {
            var packageJson;
            ts.forEachAncestorDirectory(directory, function (ancestor) {
                if (ancestor === "node_modules")
                    return true;
                packageJson = ts.findConfigFile(ancestor, function (f) { return tryFileExists(host, f); }, "package.json");
                if (packageJson) {
                    return true; // break out
                }
            });
            return packageJson;
        }
        ts.findPackageJson = findPackageJson;
        function getPackageJsonsVisibleToFile(fileName, host) {
            if (!host.fileExists) {
                return [];
            }
            var packageJsons = [];
            ts.forEachAncestorDirectory(ts.getDirectoryPath(fileName), function (ancestor) {
                var packageJsonFileName = ts.combinePaths(ancestor, "package.json");
                if (host.fileExists(packageJsonFileName)) {
                    var info = createPackageJsonInfo(packageJsonFileName, host);
                    if (info) {
                        packageJsons.push(info);
                    }
                }
            });
            return packageJsons;
        }
        ts.getPackageJsonsVisibleToFile = getPackageJsonsVisibleToFile;
        function createPackageJsonInfo(fileName, host) {
            if (!host.readFile) {
                return undefined;
            }
            var dependencyKeys = ["dependencies", "devDependencies", "optionalDependencies", "peerDependencies"];
            var stringContent = host.readFile(fileName);
            if (!stringContent)
                return undefined;
            var content = tryParseJson(stringContent);
            if (!content)
                return false;
            var info = {};
            for (var _i = 0, dependencyKeys_1 = dependencyKeys; _i < dependencyKeys_1.length; _i++) {
                var key = dependencyKeys_1[_i];
                var dependencies = content[key];
                if (!dependencies) {
                    continue;
                }
                var dependencyMap = ts.createMap();
                for (var packageName in dependencies) {
                    dependencyMap.set(packageName, dependencies[packageName]);
                }
                info[key] = dependencyMap;
            }
            var dependencyGroups = [
                [1 /* Dependencies */, info.dependencies],
                [2 /* DevDependencies */, info.devDependencies],
                [8 /* OptionalDependencies */, info.optionalDependencies],
                [4 /* PeerDependencies */, info.peerDependencies],
            ];
            return __assign(__assign({}, info), { fileName: fileName,
                get: get,
                has: function (dependencyName, inGroups) {
                    return !!get(dependencyName, inGroups);
                } });
            function get(dependencyName, inGroups) {
                if (inGroups === void 0) {
                    inGroups = 15 /* All */;
                }
                for (var _i = 0, dependencyGroups_1 = dependencyGroups; _i < dependencyGroups_1.length; _i++) {
                    var _a = dependencyGroups_1[_i], group_1 = _a[0], deps = _a[1];
                    if (deps && (inGroups & group_1)) {
                        var dep = deps.get(dependencyName);
                        if (dep !== undefined) {
                            return dep;
                        }
                    }
                }
            }
        }
        ts.createPackageJsonInfo = createPackageJsonInfo;
        function tryParseJson(text) {
            try {
                return JSON.parse(text);
            }
            catch (_a) {
                return undefined;
            }
        }
        function consumesNodeCoreModules(sourceFile) {
            return ts.some(sourceFile.imports, function (_a) {
                var text = _a.text;
                return ts.JsTyping.nodeCoreModules.has(text);
            });
        }
        ts.consumesNodeCoreModules = consumesNodeCoreModules;
        function isInsideNodeModules(fileOrDirectory) {
            return ts.contains(ts.getPathComponents(fileOrDirectory), "node_modules");
        }
        ts.isInsideNodeModules = isInsideNodeModules;
        function isDiagnosticWithLocation(diagnostic) {
            return diagnostic.file !== undefined && diagnostic.start !== undefined && diagnostic.length !== undefined;
        }
        ts.isDiagnosticWithLocation = isDiagnosticWithLocation;
        function findDiagnosticForNode(node, sortedFileDiagnostics) {
            var span = createTextSpanFromNode(node);
            var index = ts.binarySearchKey(sortedFileDiagnostics, span, ts.identity, ts.compareTextSpans);
            if (index >= 0) {
                var diagnostic = sortedFileDiagnostics[index];
                ts.Debug.assertEqual(diagnostic.file, node.getSourceFile(), "Diagnostics proided to 'findDiagnosticForNode' must be from a single SourceFile");
                return ts.cast(diagnostic, isDiagnosticWithLocation);
            }
        }
        ts.findDiagnosticForNode = findDiagnosticForNode;
        function getDiagnosticsWithinSpan(span, sortedFileDiagnostics) {
            var _a;
            var index = ts.binarySearchKey(sortedFileDiagnostics, span.start, function (diag) { return diag.start; }, ts.compareValues);
            if (index < 0) {
                index = ~index;
            }
            while (((_a = sortedFileDiagnostics[index - 1]) === null || _a === void 0 ? void 0 : _a.start) === span.start) {
                index--;
            }
            var result = [];
            var end = ts.textSpanEnd(span);
            while (true) {
                var diagnostic = ts.tryCast(sortedFileDiagnostics[index], isDiagnosticWithLocation);
                if (!diagnostic || diagnostic.start > end) {
                    break;
                }
                if (ts.textSpanContainsTextSpan(span, diagnostic)) {
                    result.push(diagnostic);
                }
                index++;
            }
            return result;
        }
        ts.getDiagnosticsWithinSpan = getDiagnosticsWithinSpan;
        /* @internal */
        function getRefactorContextSpan(_a) {
            var startPosition = _a.startPosition, endPosition = _a.endPosition;
            return ts.createTextSpanFromBounds(startPosition, endPosition === undefined ? startPosition : endPosition);
        }
        ts.getRefactorContextSpan = getRefactorContextSpan;
        function mapOneOrMany(valueOrArray, f, resultSelector) {
            if (resultSelector === void 0) {
                resultSelector = ts.identity;
            }
            return valueOrArray ? ts.isArray(valueOrArray) ? resultSelector(ts.map(valueOrArray, f)) : f(valueOrArray, 0) : undefined;
        }
        ts.mapOneOrMany = mapOneOrMany;
        /**
         * If the provided value is an array, the first element of the array is returned; otherwise, the provided value is returned instead.
         */
        function firstOrOnly(valueOrArray) {
            return ts.isArray(valueOrArray) ? ts.first(valueOrArray) : valueOrArray;
        }
        ts.firstOrOnly = firstOrOnly;
        // #endregion
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        /** The classifier is used for syntactic highlighting in editors via the TSServer */
        function createClassifier() {
            var scanner = ts.createScanner(99 /* Latest */, /*skipTrivia*/ false);
            function getClassificationsForLine(text, lexState, syntacticClassifierAbsent) {
                return convertClassificationsToResult(getEncodedLexicalClassifications(text, lexState, syntacticClassifierAbsent), text);
            }
            // If there is a syntactic classifier ('syntacticClassifierAbsent' is false),
            // we will be more conservative in order to avoid conflicting with the syntactic classifier.
            function getEncodedLexicalClassifications(text, lexState, syntacticClassifierAbsent) {
                var token = 0 /* Unknown */;
                var lastNonTriviaToken = 0 /* Unknown */;
                // Just a stack of TemplateHeads and OpenCurlyBraces, used to perform rudimentary (inexact)
                // classification on template strings. Because of the context free nature of templates,
                // the only precise way to classify a template portion would be by propagating the stack across
                // lines, just as we do with the end-of-line state. However, this is a burden for implementers,
                // and the behavior is entirely subsumed by the syntactic classifier anyway, so we instead
                // flatten any nesting when the template stack is non-empty and encode it in the end-of-line state.
                // Situations in which this fails are
                //  1) When template strings are nested across different lines:
                //          `hello ${ `world
                //          ` }`
                //
                //     Where on the second line, you will get the closing of a template,
                //     a closing curly, and a new template.
                //
                //  2) When substitution expressions have curly braces and the curly brace falls on the next line:
                //          `hello ${ () => {
                //          return "world" } } `
                //
                //     Where on the second line, you will get the 'return' keyword,
                //     a string literal, and a template end consisting of '} } `'.
                var templateStack = [];
                var _a = getPrefixFromLexState(lexState), prefix = _a.prefix, pushTemplate = _a.pushTemplate;
                text = prefix + text;
                var offset = prefix.length;
                if (pushTemplate) {
                    templateStack.push(15 /* TemplateHead */);
                }
                scanner.setText(text);
                var endOfLineState = 0 /* None */;
                var spans = [];
                // We can run into an unfortunate interaction between the lexical and syntactic classifier
                // when the user is typing something generic.  Consider the case where the user types:
                //
                //      Foo<number
                //
                // From the lexical classifier's perspective, 'number' is a keyword, and so the word will
                // be classified as such.  However, from the syntactic classifier's tree-based perspective
                // this is simply an expression with the identifier 'number' on the RHS of the less than
                // token.  So the classification will go back to being an identifier.  The moment the user
                // types again, number will become a keyword, then an identifier, etc. etc.
                //
                // To try to avoid this problem, we avoid classifying contextual keywords as keywords
                // when the user is potentially typing something generic.  We just can't do a good enough
                // job at the lexical level, and so well leave it up to the syntactic classifier to make
                // the determination.
                //
                // In order to determine if the user is potentially typing something generic, we use a
                // weak heuristic where we track < and > tokens.  It's a weak heuristic, but should
                // work well enough in practice.
                var angleBracketStack = 0;
                do {
                    token = scanner.scan();
                    if (!ts.isTrivia(token)) {
                        handleToken();
                        lastNonTriviaToken = token;
                    }
                    var end = scanner.getTextPos();
                    pushEncodedClassification(scanner.getTokenPos(), end, offset, classFromKind(token), spans);
                    if (end >= text.length) {
                        var end_1 = getNewEndOfLineState(scanner, token, ts.lastOrUndefined(templateStack));
                        if (end_1 !== undefined) {
                            endOfLineState = end_1;
                        }
                    }
                } while (token !== 1 /* EndOfFileToken */);
                function handleToken() {
                    switch (token) {
                        case 43 /* SlashToken */:
                        case 67 /* SlashEqualsToken */:
                            if (!noRegexTable[lastNonTriviaToken] && scanner.reScanSlashToken() === 13 /* RegularExpressionLiteral */) {
                                token = 13 /* RegularExpressionLiteral */;
                            }
                            break;
                        case 29 /* LessThanToken */:
                            if (lastNonTriviaToken === 75 /* Identifier */) {
                                // Could be the start of something generic.  Keep track of that by bumping
                                // up the current count of generic contexts we may be in.
                                angleBracketStack++;
                            }
                            break;
                        case 31 /* GreaterThanToken */:
                            if (angleBracketStack > 0) {
                                // If we think we're currently in something generic, then mark that that
                                // generic entity is complete.
                                angleBracketStack--;
                            }
                            break;
                        case 125 /* AnyKeyword */:
                        case 143 /* StringKeyword */:
                        case 140 /* NumberKeyword */:
                        case 128 /* BooleanKeyword */:
                        case 144 /* SymbolKeyword */:
                            if (angleBracketStack > 0 && !syntacticClassifierAbsent) {
                                // If it looks like we're could be in something generic, don't classify this
                                // as a keyword.  We may just get overwritten by the syntactic classifier,
                                // causing a noisy experience for the user.
                                token = 75 /* Identifier */;
                            }
                            break;
                        case 15 /* TemplateHead */:
                            templateStack.push(token);
                            break;
                        case 18 /* OpenBraceToken */:
                            // If we don't have anything on the template stack,
                            // then we aren't trying to keep track of a previously scanned template head.
                            if (templateStack.length > 0) {
                                templateStack.push(token);
                            }
                            break;
                        case 19 /* CloseBraceToken */:
                            // If we don't have anything on the template stack,
                            // then we aren't trying to keep track of a previously scanned template head.
                            if (templateStack.length > 0) {
                                var lastTemplateStackToken = ts.lastOrUndefined(templateStack);
                                if (lastTemplateStackToken === 15 /* TemplateHead */) {
                                    token = scanner.reScanTemplateToken();
                                    // Only pop on a TemplateTail; a TemplateMiddle indicates there is more for us.
                                    if (token === 17 /* TemplateTail */) {
                                        templateStack.pop();
                                    }
                                    else {
                                        ts.Debug.assertEqual(token, 16 /* TemplateMiddle */, "Should have been a template middle.");
                                    }
                                }
                                else {
                                    ts.Debug.assertEqual(lastTemplateStackToken, 18 /* OpenBraceToken */, "Should have been an open brace");
                                    templateStack.pop();
                                }
                            }
                            break;
                        default:
                            if (!ts.isKeyword(token)) {
                                break;
                            }
                            if (lastNonTriviaToken === 24 /* DotToken */) {
                                token = 75 /* Identifier */;
                            }
                            else if (ts.isKeyword(lastNonTriviaToken) && ts.isKeyword(token) && !canFollow(lastNonTriviaToken, token)) {
                                // We have two keywords in a row.  Only treat the second as a keyword if
                                // it's a sequence that could legally occur in the language.  Otherwise
                                // treat it as an identifier.  This way, if someone writes "private var"
                                // we recognize that 'var' is actually an identifier here.
                                token = 75 /* Identifier */;
                            }
                    }
                }
                return { endOfLineState: endOfLineState, spans: spans };
            }
            return { getClassificationsForLine: getClassificationsForLine, getEncodedLexicalClassifications: getEncodedLexicalClassifications };
        }
        ts.createClassifier = createClassifier;
        /// We do not have a full parser support to know when we should parse a regex or not
        /// If we consider every slash token to be a regex, we could be missing cases like "1/2/3", where
        /// we have a series of divide operator. this list allows us to be more accurate by ruling out
        /// locations where a regexp cannot exist.
        var noRegexTable = ts.arrayToNumericMap([
            75 /* Identifier */,
            10 /* StringLiteral */,
            8 /* NumericLiteral */,
            9 /* BigIntLiteral */,
            13 /* RegularExpressionLiteral */,
            104 /* ThisKeyword */,
            45 /* PlusPlusToken */,
            46 /* MinusMinusToken */,
            21 /* CloseParenToken */,
            23 /* CloseBracketToken */,
            19 /* CloseBraceToken */,
            106 /* TrueKeyword */,
            91 /* FalseKeyword */,
        ], function (token) { return token; }, function () { return true; });
        function getNewEndOfLineState(scanner, token, lastOnTemplateStack) {
            switch (token) {
                case 10 /* StringLiteral */: {
                    // Check to see if we finished up on a multiline string literal.
                    if (!scanner.isUnterminated())
                        return undefined;
                    var tokenText = scanner.getTokenText();
                    var lastCharIndex = tokenText.length - 1;
                    var numBackslashes = 0;
                    while (tokenText.charCodeAt(lastCharIndex - numBackslashes) === 92 /* backslash */) {
                        numBackslashes++;
                    }
                    // If we have an odd number of backslashes, then the multiline string is unclosed
                    if ((numBackslashes & 1) === 0)
                        return undefined;
                    return tokenText.charCodeAt(0) === 34 /* doubleQuote */ ? 3 /* InDoubleQuoteStringLiteral */ : 2 /* InSingleQuoteStringLiteral */;
                }
                case 3 /* MultiLineCommentTrivia */:
                    // Check to see if the multiline comment was unclosed.
                    return scanner.isUnterminated() ? 1 /* InMultiLineCommentTrivia */ : undefined;
                default:
                    if (ts.isTemplateLiteralKind(token)) {
                        if (!scanner.isUnterminated()) {
                            return undefined;
                        }
                        switch (token) {
                            case 17 /* TemplateTail */:
                                return 5 /* InTemplateMiddleOrTail */;
                            case 14 /* NoSubstitutionTemplateLiteral */:
                                return 4 /* InTemplateHeadOrNoSubstitutionTemplate */;
                            default:
                                return ts.Debug.fail("Only 'NoSubstitutionTemplateLiteral's and 'TemplateTail's can be unterminated; got SyntaxKind #" + token);
                        }
                    }
                    return lastOnTemplateStack === 15 /* TemplateHead */ ? 6 /* InTemplateSubstitutionPosition */ : undefined;
            }
        }
        function pushEncodedClassification(start, end, offset, classification, result) {
            if (classification === 8 /* whiteSpace */) {
                // Don't bother with whitespace classifications.  They're not needed.
                return;
            }
            if (start === 0 && offset > 0) {
                // We're classifying the first token, and this was a case where we prepended text.
                // We should consider the start of this token to be at the start of the original text.
                start += offset;
            }
            var length = end - start;
            if (length > 0) {
                // All our tokens are in relation to the augmented text.  Move them back to be
                // relative to the original text.
                result.push(start - offset, length, classification);
            }
        }
        function convertClassificationsToResult(classifications, text) {
            var entries = [];
            var dense = classifications.spans;
            var lastEnd = 0;
            for (var i = 0; i < dense.length; i += 3) {
                var start = dense[i];
                var length_1 = dense[i + 1];
                var type = dense[i + 2];
                // Make a whitespace entry between the last item and this one.
                if (lastEnd >= 0) {
                    var whitespaceLength_1 = start - lastEnd;
                    if (whitespaceLength_1 > 0) {
                        entries.push({ length: whitespaceLength_1, classification: ts.TokenClass.Whitespace });
                    }
                }
                entries.push({ length: length_1, classification: convertClassification(type) });
                lastEnd = start + length_1;
            }
            var whitespaceLength = text.length - lastEnd;
            if (whitespaceLength > 0) {
                entries.push({ length: whitespaceLength, classification: ts.TokenClass.Whitespace });
            }
            return { entries: entries, finalLexState: classifications.endOfLineState };
        }
        function convertClassification(type) {
            switch (type) {
                case 1 /* comment */: return ts.TokenClass.Comment;
                case 3 /* keyword */: return ts.TokenClass.Keyword;
                case 4 /* numericLiteral */: return ts.TokenClass.NumberLiteral;
                case 25 /* bigintLiteral */: return ts.TokenClass.BigIntLiteral;
                case 5 /* operator */: return ts.TokenClass.Operator;
                case 6 /* stringLiteral */: return ts.TokenClass.StringLiteral;
                case 8 /* whiteSpace */: return ts.TokenClass.Whitespace;
                case 10 /* punctuation */: return ts.TokenClass.Punctuation;
                case 2 /* identifier */:
                case 11 /* className */:
                case 12 /* enumName */:
                case 13 /* interfaceName */:
                case 14 /* moduleName */:
                case 15 /* typeParameterName */:
                case 16 /* typeAliasName */:
                case 9 /* text */:
                case 17 /* parameterName */:
                    return ts.TokenClass.Identifier;
                default:
                    return undefined; // TODO: GH#18217 Debug.assertNever(type);
            }
        }
        /** Returns true if 'keyword2' can legally follow 'keyword1' in any language construct. */
        function canFollow(keyword1, keyword2) {
            if (!ts.isAccessibilityModifier(keyword1)) {
                // Assume any other keyword combination is legal.
                // This can be refined in the future if there are more cases we want the classifier to be better at.
                return true;
            }
            switch (keyword2) {
                case 131 /* GetKeyword */:
                case 142 /* SetKeyword */:
                case 129 /* ConstructorKeyword */:
                case 120 /* StaticKeyword */:
                    return true; // Allow things like "public get", "public constructor" and "public static".
                default:
                    return false; // Any other keyword following "public" is actually an identifier, not a real keyword.
            }
        }
        function getPrefixFromLexState(lexState) {
            // If we're in a string literal, then prepend: "\
            // (and a newline).  That way when we lex we'll think we're still in a string literal.
            //
            // If we're in a multiline comment, then prepend: /*
            // (and a newline).  That way when we lex we'll think we're still in a multiline comment.
            switch (lexState) {
                case 3 /* InDoubleQuoteStringLiteral */:
                    return { prefix: "\"\\\n" };
                case 2 /* InSingleQuoteStringLiteral */:
                    return { prefix: "'\\\n" };
                case 1 /* InMultiLineCommentTrivia */:
                    return { prefix: "/*\n" };
                case 4 /* InTemplateHeadOrNoSubstitutionTemplate */:
                    return { prefix: "`\n" };
                case 5 /* InTemplateMiddleOrTail */:
                    return { prefix: "}\n", pushTemplate: true };
                case 6 /* InTemplateSubstitutionPosition */:
                    return { prefix: "", pushTemplate: true };
                case 0 /* None */:
                    return { prefix: "" };
                default:
                    return ts.Debug.assertNever(lexState);
            }
        }
        function isBinaryExpressionOperatorToken(token) {
            switch (token) {
                case 41 /* AsteriskToken */:
                case 43 /* SlashToken */:
                case 44 /* PercentToken */:
                case 39 /* PlusToken */:
                case 40 /* MinusToken */:
                case 47 /* LessThanLessThanToken */:
                case 48 /* GreaterThanGreaterThanToken */:
                case 49 /* GreaterThanGreaterThanGreaterThanToken */:
                case 29 /* LessThanToken */:
                case 31 /* GreaterThanToken */:
                case 32 /* LessThanEqualsToken */:
                case 33 /* GreaterThanEqualsToken */:
                case 98 /* InstanceOfKeyword */:
                case 97 /* InKeyword */:
                case 123 /* AsKeyword */:
                case 34 /* EqualsEqualsToken */:
                case 35 /* ExclamationEqualsToken */:
                case 36 /* EqualsEqualsEqualsToken */:
                case 37 /* ExclamationEqualsEqualsToken */:
                case 50 /* AmpersandToken */:
                case 52 /* CaretToken */:
                case 51 /* BarToken */:
                case 55 /* AmpersandAmpersandToken */:
                case 56 /* BarBarToken */:
                case 73 /* BarEqualsToken */:
                case 72 /* AmpersandEqualsToken */:
                case 74 /* CaretEqualsToken */:
                case 69 /* LessThanLessThanEqualsToken */:
                case 70 /* GreaterThanGreaterThanEqualsToken */:
                case 71 /* GreaterThanGreaterThanGreaterThanEqualsToken */:
                case 63 /* PlusEqualsToken */:
                case 64 /* MinusEqualsToken */:
                case 65 /* AsteriskEqualsToken */:
                case 67 /* SlashEqualsToken */:
                case 68 /* PercentEqualsToken */:
                case 62 /* EqualsToken */:
                case 27 /* CommaToken */:
                case 60 /* QuestionQuestionToken */:
                    return true;
                default:
                    return false;
            }
        }
        function isPrefixUnaryExpressionOperatorToken(token) {
            switch (token) {
                case 39 /* PlusToken */:
                case 40 /* MinusToken */:
                case 54 /* TildeToken */:
                case 53 /* ExclamationToken */:
                case 45 /* PlusPlusToken */:
                case 46 /* MinusMinusToken */:
                    return true;
                default:
                    return false;
            }
        }
        function classFromKind(token) {
            if (ts.isKeyword(token)) {
                return 3 /* keyword */;
            }
            else if (isBinaryExpressionOperatorToken(token) || isPrefixUnaryExpressionOperatorToken(token)) {
                return 5 /* operator */;
            }
            else if (token >= 18 /* FirstPunctuation */ && token <= 74 /* LastPunctuation */) {
                return 10 /* punctuation */;
            }
            switch (token) {
                case 8 /* NumericLiteral */:
                    return 4 /* numericLiteral */;
                case 9 /* BigIntLiteral */:
                    return 25 /* bigintLiteral */;
                case 10 /* StringLiteral */:
                    return 6 /* stringLiteral */;
                case 13 /* RegularExpressionLiteral */:
                    return 7 /* regularExpressionLiteral */;
                case 7 /* ConflictMarkerTrivia */:
                case 3 /* MultiLineCommentTrivia */:
                case 2 /* SingleLineCommentTrivia */:
                    return 1 /* comment */;
                case 5 /* WhitespaceTrivia */:
                case 4 /* NewLineTrivia */:
                    return 8 /* whiteSpace */;
                case 75 /* Identifier */:
                default:
                    if (ts.isTemplateLiteralKind(token)) {
                        return 6 /* stringLiteral */;
                    }
                    return 2 /* identifier */;
            }
        }
        /* @internal */
        function getSemanticClassifications(typeChecker, cancellationToken, sourceFile, classifiableNames, span) {
            return convertClassificationsToSpans(getEncodedSemanticClassifications(typeChecker, cancellationToken, sourceFile, classifiableNames, span));
        }
        ts.getSemanticClassifications = getSemanticClassifications;
        function checkForClassificationCancellation(cancellationToken, kind) {
            // We don't want to actually call back into our host on every node to find out if we've
            // been canceled.  That would be an enormous amount of chattyness, along with the all
            // the overhead of marshalling the data to/from the host.  So instead we pick a few
            // reasonable node kinds to bother checking on.  These node kinds represent high level
            // constructs that we would expect to see commonly, but just at a far less frequent
            // interval.
            //
            // For example, in checker.ts (around 750k) we only have around 600 of these constructs.
            // That means we're calling back into the host around every 1.2k of the file we process.
            // Lib.d.ts has similar numbers.
            switch (kind) {
                case 249 /* ModuleDeclaration */:
                case 245 /* ClassDeclaration */:
                case 246 /* InterfaceDeclaration */:
                case 244 /* FunctionDeclaration */:
                    cancellationToken.throwIfCancellationRequested();
            }
        }
        /* @internal */
        function getEncodedSemanticClassifications(typeChecker, cancellationToken, sourceFile, classifiableNames, span) {
            var spans = [];
            sourceFile.forEachChild(function cb(node) {
                // Only walk into nodes that intersect the requested span.
                if (!node || !ts.textSpanIntersectsWith(span, node.pos, node.getFullWidth())) {
                    return;
                }
                checkForClassificationCancellation(cancellationToken, node.kind);
                // Only bother calling into the typechecker if this is an identifier that
                // could possibly resolve to a type name.  This makes classification run
                // in a third of the time it would normally take.
                if (ts.isIdentifier(node) && !ts.nodeIsMissing(node) && classifiableNames.has(node.escapedText)) {
                    var symbol = typeChecker.getSymbolAtLocation(node);
                    var type = symbol && classifySymbol(symbol, ts.getMeaningFromLocation(node), typeChecker);
                    if (type) {
                        pushClassification(node.getStart(sourceFile), node.getEnd(), type);
                    }
                }
                node.forEachChild(cb);
            });
            return { spans: spans, endOfLineState: 0 /* None */ };
            function pushClassification(start, end, type) {
                var length = end - start;
                ts.Debug.assert(length > 0, "Classification had non-positive length of " + length);
                spans.push(start);
                spans.push(length);
                spans.push(type);
            }
        }
        ts.getEncodedSemanticClassifications = getEncodedSemanticClassifications;
        function classifySymbol(symbol, meaningAtPosition, checker) {
            var flags = symbol.getFlags();
            if ((flags & 2885600 /* Classifiable */) === 0 /* None */) {
                return undefined;
            }
            else if (flags & 32 /* Class */) {
                return 11 /* className */;
            }
            else if (flags & 384 /* Enum */) {
                return 12 /* enumName */;
            }
            else if (flags & 524288 /* TypeAlias */) {
                return 16 /* typeAliasName */;
            }
            else if (flags & 1536 /* Module */) {
                // Only classify a module as such if
                //  - It appears in a namespace context.
                //  - There exists a module declaration which actually impacts the value side.
                return meaningAtPosition & 4 /* Namespace */ || meaningAtPosition & 1 /* Value */ && hasValueSideModule(symbol) ? 14 /* moduleName */ : undefined;
            }
            else if (flags & 2097152 /* Alias */) {
                return classifySymbol(checker.getAliasedSymbol(symbol), meaningAtPosition, checker);
            }
            else if (meaningAtPosition & 2 /* Type */) {
                return flags & 64 /* Interface */ ? 13 /* interfaceName */ : flags & 262144 /* TypeParameter */ ? 15 /* typeParameterName */ : undefined;
            }
            else {
                return undefined;
            }
        }
        /** Returns true if there exists a module that introduces entities on the value side. */
        function hasValueSideModule(symbol) {
            return ts.some(symbol.declarations, function (declaration) {
                return ts.isModuleDeclaration(declaration) && ts.getModuleInstanceState(declaration) === 1 /* Instantiated */;
            });
        }
        function getClassificationTypeName(type) {
            switch (type) {
                case 1 /* comment */: return "comment" /* comment */;
                case 2 /* identifier */: return "identifier" /* identifier */;
                case 3 /* keyword */: return "keyword" /* keyword */;
                case 4 /* numericLiteral */: return "number" /* numericLiteral */;
                case 25 /* bigintLiteral */: return "bigint" /* bigintLiteral */;
                case 5 /* operator */: return "operator" /* operator */;
                case 6 /* stringLiteral */: return "string" /* stringLiteral */;
                case 8 /* whiteSpace */: return "whitespace" /* whiteSpace */;
                case 9 /* text */: return "text" /* text */;
                case 10 /* punctuation */: return "punctuation" /* punctuation */;
                case 11 /* className */: return "class name" /* className */;
                case 12 /* enumName */: return "enum name" /* enumName */;
                case 13 /* interfaceName */: return "interface name" /* interfaceName */;
                case 14 /* moduleName */: return "module name" /* moduleName */;
                case 15 /* typeParameterName */: return "type parameter name" /* typeParameterName */;
                case 16 /* typeAliasName */: return "type alias name" /* typeAliasName */;
                case 17 /* parameterName */: return "parameter name" /* parameterName */;
                case 18 /* docCommentTagName */: return "doc comment tag name" /* docCommentTagName */;
                case 19 /* jsxOpenTagName */: return "jsx open tag name" /* jsxOpenTagName */;
                case 20 /* jsxCloseTagName */: return "jsx close tag name" /* jsxCloseTagName */;
                case 21 /* jsxSelfClosingTagName */: return "jsx self closing tag name" /* jsxSelfClosingTagName */;
                case 22 /* jsxAttribute */: return "jsx attribute" /* jsxAttribute */;
                case 23 /* jsxText */: return "jsx text" /* jsxText */;
                case 24 /* jsxAttributeStringLiteralValue */: return "jsx attribute string literal value" /* jsxAttributeStringLiteralValue */;
                default: return undefined; // TODO: GH#18217 throw Debug.assertNever(type);
            }
        }
        function convertClassificationsToSpans(classifications) {
            ts.Debug.assert(classifications.spans.length % 3 === 0);
            var dense = classifications.spans;
            var result = [];
            for (var i = 0; i < dense.length; i += 3) {
                result.push({
                    textSpan: ts.createTextSpan(dense[i], dense[i + 1]),
                    classificationType: getClassificationTypeName(dense[i + 2])
                });
            }
            return result;
        }
        /* @internal */
        function getSyntacticClassifications(cancellationToken, sourceFile, span) {
            return convertClassificationsToSpans(getEncodedSyntacticClassifications(cancellationToken, sourceFile, span));
        }
        ts.getSyntacticClassifications = getSyntacticClassifications;
        /* @internal */
        function getEncodedSyntacticClassifications(cancellationToken, sourceFile, span) {
            var spanStart = span.start;
            var spanLength = span.length;
            // Make a scanner we can get trivia from.
            var triviaScanner = ts.createScanner(99 /* Latest */, /*skipTrivia*/ false, sourceFile.languageVariant, sourceFile.text);
            var mergeConflictScanner = ts.createScanner(99 /* Latest */, /*skipTrivia*/ false, sourceFile.languageVariant, sourceFile.text);
            var result = [];
            processElement(sourceFile);
            return { spans: result, endOfLineState: 0 /* None */ };
            function pushClassification(start, length, type) {
                result.push(start);
                result.push(length);
                result.push(type);
            }
            function classifyLeadingTriviaAndGetTokenStart(token) {
                triviaScanner.setTextPos(token.pos);
                while (true) {
                    var start = triviaScanner.getTextPos();
                    // only bother scanning if we have something that could be trivia.
                    if (!ts.couldStartTrivia(sourceFile.text, start)) {
                        return start;
                    }
                    var kind = triviaScanner.scan();
                    var end = triviaScanner.getTextPos();
                    var width = end - start;
                    // The moment we get something that isn't trivia, then stop processing.
                    if (!ts.isTrivia(kind)) {
                        return start;
                    }
                    switch (kind) {
                        case 4 /* NewLineTrivia */:
                        case 5 /* WhitespaceTrivia */:
                            // Don't bother with newlines/whitespace.
                            continue;
                        case 2 /* SingleLineCommentTrivia */:
                        case 3 /* MultiLineCommentTrivia */:
                            // Only bother with the trivia if it at least intersects the span of interest.
                            classifyComment(token, kind, start, width);
                            // Classifying a comment might cause us to reuse the trivia scanner
                            // (because of jsdoc comments).  So after we classify the comment make
                            // sure we set the scanner position back to where it needs to be.
                            triviaScanner.setTextPos(end);
                            continue;
                        case 7 /* ConflictMarkerTrivia */:
                            var text = sourceFile.text;
                            var ch = text.charCodeAt(start);
                            // for the <<<<<<< and >>>>>>> markers, we just add them in as comments
                            // in the classification stream.
                            if (ch === 60 /* lessThan */ || ch === 62 /* greaterThan */) {
                                pushClassification(start, width, 1 /* comment */);
                                continue;
                            }
                            // for the ||||||| and ======== markers, add a comment for the first line,
                            // and then lex all subsequent lines up until the end of the conflict marker.
                            ts.Debug.assert(ch === 124 /* bar */ || ch === 61 /* equals */);
                            classifyDisabledMergeCode(text, start, end);
                            break;
                        case 6 /* ShebangTrivia */:
                            // TODO: Maybe we should classify these.
                            break;
                        default:
                            ts.Debug.assertNever(kind);
                    }
                }
            }
            function classifyComment(token, kind, start, width) {
                if (kind === 3 /* MultiLineCommentTrivia */) {
                    // See if this is a doc comment.  If so, we'll classify certain portions of it
                    // specially.
                    var docCommentAndDiagnostics = ts.parseIsolatedJSDocComment(sourceFile.text, start, width);
                    if (docCommentAndDiagnostics && docCommentAndDiagnostics.jsDoc) {
                        // TODO: This should be predicated on `token["kind"]` being compatible with `HasJSDoc["kind"]`
                        docCommentAndDiagnostics.jsDoc.parent = token;
                        classifyJSDocComment(docCommentAndDiagnostics.jsDoc);
                        return;
                    }
                }
                else if (kind === 2 /* SingleLineCommentTrivia */) {
                    if (tryClassifyTripleSlashComment(start, width)) {
                        return;
                    }
                }
                // Simple comment.  Just add as is.
                pushCommentRange(start, width);
            }
            function pushCommentRange(start, width) {
                pushClassification(start, width, 1 /* comment */);
            }
            function classifyJSDocComment(docComment) {
                var pos = docComment.pos;
                if (docComment.tags) {
                    for (var _i = 0, _a = docComment.tags; _i < _a.length; _i++) {
                        var tag = _a[_i];
                        // As we walk through each tag, classify the portion of text from the end of
                        // the last tag (or the start of the entire doc comment) as 'comment'.
                        if (tag.pos !== pos) {
                            pushCommentRange(pos, tag.pos - pos);
                        }
                        pushClassification(tag.pos, 1, 10 /* punctuation */); // "@"
                        pushClassification(tag.tagName.pos, tag.tagName.end - tag.tagName.pos, 18 /* docCommentTagName */); // e.g. "param"
                        pos = tag.tagName.end;
                        switch (tag.kind) {
                            case 316 /* JSDocParameterTag */:
                                processJSDocParameterTag(tag);
                                break;
                            case 320 /* JSDocTemplateTag */:
                                processJSDocTemplateTag(tag);
                                pos = tag.end;
                                break;
                            case 319 /* JSDocTypeTag */:
                                processElement(tag.typeExpression);
                                pos = tag.end;
                                break;
                            case 317 /* JSDocReturnTag */:
                                processElement(tag.typeExpression);
                                pos = tag.end;
                                break;
                        }
                    }
                }
                if (pos !== docComment.end) {
                    pushCommentRange(pos, docComment.end - pos);
                }
                return;
                function processJSDocParameterTag(tag) {
                    if (tag.isNameFirst) {
                        pushCommentRange(pos, tag.name.pos - pos);
                        pushClassification(tag.name.pos, tag.name.end - tag.name.pos, 17 /* parameterName */);
                        pos = tag.name.end;
                    }
                    if (tag.typeExpression) {
                        pushCommentRange(pos, tag.typeExpression.pos - pos);
                        processElement(tag.typeExpression);
                        pos = tag.typeExpression.end;
                    }
                    if (!tag.isNameFirst) {
                        pushCommentRange(pos, tag.name.pos - pos);
                        pushClassification(tag.name.pos, tag.name.end - tag.name.pos, 17 /* parameterName */);
                        pos = tag.name.end;
                    }
                }
            }
            function tryClassifyTripleSlashComment(start, width) {
                var tripleSlashXMLCommentRegEx = /^(\/\/\/\s*)(<)(?:(\S+)((?:[^/]|\/[^>])*)(\/>)?)?/im;
                var attributeRegex = /(\S+)(\s*)(=)(\s*)('[^']+'|"[^"]+")/img;
                var text = sourceFile.text.substr(start, width);
                var match = tripleSlashXMLCommentRegEx.exec(text);
                if (!match) {
                    return false;
                }
                // Limiting classification to exactly the elements and attributes
                // defined in `ts.commentPragmas` would be excessive, but we can avoid
                // some obvious false positives (e.g. in XML-like doc comments) by
                // checking the element name.
                // eslint-disable-next-line no-in-operator
                if (!match[3] || !(match[3] in ts.commentPragmas)) {
                    return false;
                }
                var pos = start;
                pushCommentRange(pos, match[1].length); // ///
                pos += match[1].length;
                pushClassification(pos, match[2].length, 10 /* punctuation */); // <
                pos += match[2].length;
                pushClassification(pos, match[3].length, 21 /* jsxSelfClosingTagName */); // element name
                pos += match[3].length;
                var attrText = match[4];
                var attrPos = pos;
                while (true) {
                    var attrMatch = attributeRegex.exec(attrText);
                    if (!attrMatch) {
                        break;
                    }
                    var newAttrPos = pos + attrMatch.index;
                    if (newAttrPos > attrPos) {
                        pushCommentRange(attrPos, newAttrPos - attrPos);
                        attrPos = newAttrPos;
                    }
                    pushClassification(attrPos, attrMatch[1].length, 22 /* jsxAttribute */); // attribute name
                    attrPos += attrMatch[1].length;
                    if (attrMatch[2].length) {
                        pushCommentRange(attrPos, attrMatch[2].length); // whitespace
                        attrPos += attrMatch[2].length;
                    }
                    pushClassification(attrPos, attrMatch[3].length, 5 /* operator */); // =
                    attrPos += attrMatch[3].length;
                    if (attrMatch[4].length) {
                        pushCommentRange(attrPos, attrMatch[4].length); // whitespace
                        attrPos += attrMatch[4].length;
                    }
                    pushClassification(attrPos, attrMatch[5].length, 24 /* jsxAttributeStringLiteralValue */); // attribute value
                    attrPos += attrMatch[5].length;
                }
                pos += match[4].length;
                if (pos > attrPos) {
                    pushCommentRange(attrPos, pos - attrPos);
                }
                if (match[5]) {
                    pushClassification(pos, match[5].length, 10 /* punctuation */); // />
                    pos += match[5].length;
                }
                var end = start + width;
                if (pos < end) {
                    pushCommentRange(pos, end - pos);
                }
                return true;
            }
            function processJSDocTemplateTag(tag) {
                for (var _i = 0, _a = tag.getChildren(); _i < _a.length; _i++) {
                    var child = _a[_i];
                    processElement(child);
                }
            }
            function classifyDisabledMergeCode(text, start, end) {
                // Classify the line that the ||||||| or ======= marker is on as a comment.
                // Then just lex all further tokens and add them to the result.
                var i;
                for (i = start; i < end; i++) {
                    if (ts.isLineBreak(text.charCodeAt(i))) {
                        break;
                    }
                }
                pushClassification(start, i - start, 1 /* comment */);
                mergeConflictScanner.setTextPos(i);
                while (mergeConflictScanner.getTextPos() < end) {
                    classifyDisabledCodeToken();
                }
            }
            function classifyDisabledCodeToken() {
                var start = mergeConflictScanner.getTextPos();
                var tokenKind = mergeConflictScanner.scan();
                var end = mergeConflictScanner.getTextPos();
                var type = classifyTokenType(tokenKind);
                if (type) {
                    pushClassification(start, end - start, type);
                }
            }
            /**
             * Returns true if node should be treated as classified and no further processing is required.
             * False will mean that node is not classified and traverse routine should recurse into node contents.
             */
            function tryClassifyNode(node) {
                if (ts.isJSDoc(node)) {
                    return true;
                }
                if (ts.nodeIsMissing(node)) {
                    return true;
                }
                var classifiedElementName = tryClassifyJsxElementName(node);
                if (!ts.isToken(node) && node.kind !== 11 /* JsxText */ && classifiedElementName === undefined) {
                    return false;
                }
                var tokenStart = node.kind === 11 /* JsxText */ ? node.pos : classifyLeadingTriviaAndGetTokenStart(node);
                var tokenWidth = node.end - tokenStart;
                ts.Debug.assert(tokenWidth >= 0);
                if (tokenWidth > 0) {
                    var type = classifiedElementName || classifyTokenType(node.kind, node);
                    if (type) {
                        pushClassification(tokenStart, tokenWidth, type);
                    }
                }
                return true;
            }
            function tryClassifyJsxElementName(token) {
                switch (token.parent && token.parent.kind) {
                    case 268 /* JsxOpeningElement */:
                        if (token.parent.tagName === token) {
                            return 19 /* jsxOpenTagName */;
                        }
                        break;
                    case 269 /* JsxClosingElement */:
                        if (token.parent.tagName === token) {
                            return 20 /* jsxCloseTagName */;
                        }
                        break;
                    case 267 /* JsxSelfClosingElement */:
                        if (token.parent.tagName === token) {
                            return 21 /* jsxSelfClosingTagName */;
                        }
                        break;
                    case 273 /* JsxAttribute */:
                        if (token.parent.name === token) {
                            return 22 /* jsxAttribute */;
                        }
                        break;
                }
                return undefined;
            }
            // for accurate classification, the actual token should be passed in.  however, for
            // cases like 'disabled merge code' classification, we just get the token kind and
            // classify based on that instead.
            function classifyTokenType(tokenKind, token) {
                if (ts.isKeyword(tokenKind)) {
                    return 3 /* keyword */;
                }
                // Special case `<` and `>`: If they appear in a generic context they are punctuation,
                // not operators.
                if (tokenKind === 29 /* LessThanToken */ || tokenKind === 31 /* GreaterThanToken */) {
                    // If the node owning the token has a type argument list or type parameter list, then
                    // we can effectively assume that a '<' and '>' belong to those lists.
                    if (token && ts.getTypeArgumentOrTypeParameterList(token.parent)) {
                        return 10 /* punctuation */;
                    }
                }
                if (ts.isPunctuation(tokenKind)) {
                    if (token) {
                        var parent = token.parent;
                        if (tokenKind === 62 /* EqualsToken */) {
                            // the '=' in a variable declaration is special cased here.
                            if (parent.kind === 242 /* VariableDeclaration */ ||
                                parent.kind === 159 /* PropertyDeclaration */ ||
                                parent.kind === 156 /* Parameter */ ||
                                parent.kind === 273 /* JsxAttribute */) {
                                return 5 /* operator */;
                            }
                        }
                        if (parent.kind === 209 /* BinaryExpression */ ||
                            parent.kind === 207 /* PrefixUnaryExpression */ ||
                            parent.kind === 208 /* PostfixUnaryExpression */ ||
                            parent.kind === 210 /* ConditionalExpression */) {
                            return 5 /* operator */;
                        }
                    }
                    return 10 /* punctuation */;
                }
                else if (tokenKind === 8 /* NumericLiteral */) {
                    return 4 /* numericLiteral */;
                }
                else if (tokenKind === 9 /* BigIntLiteral */) {
                    return 25 /* bigintLiteral */;
                }
                else if (tokenKind === 10 /* StringLiteral */) {
                    return token && token.parent.kind === 273 /* JsxAttribute */ ? 24 /* jsxAttributeStringLiteralValue */ : 6 /* stringLiteral */;
                }
                else if (tokenKind === 13 /* RegularExpressionLiteral */) {
                    // TODO: we should get another classification type for these literals.
                    return 6 /* stringLiteral */;
                }
                else if (ts.isTemplateLiteralKind(tokenKind)) {
                    // TODO (drosen): we should *also* get another classification type for these literals.
                    return 6 /* stringLiteral */;
                }
                else if (tokenKind === 11 /* JsxText */) {
                    return 23 /* jsxText */;
                }
                else if (tokenKind === 75 /* Identifier */) {
                    if (token) {
                        switch (token.parent.kind) {
                            case 245 /* ClassDeclaration */:
                                if (token.parent.name === token) {
                                    return 11 /* className */;
                                }
                                return;
                            case 155 /* TypeParameter */:
                                if (token.parent.name === token) {
                                    return 15 /* typeParameterName */;
                                }
                                return;
                            case 246 /* InterfaceDeclaration */:
                                if (token.parent.name === token) {
                                    return 13 /* interfaceName */;
                                }
                                return;
                            case 248 /* EnumDeclaration */:
                                if (token.parent.name === token) {
                                    return 12 /* enumName */;
                                }
                                return;
                            case 249 /* ModuleDeclaration */:
                                if (token.parent.name === token) {
                                    return 14 /* moduleName */;
                                }
                                return;
                            case 156 /* Parameter */:
                                if (token.parent.name === token) {
                                    return ts.isThisIdentifier(token) ? 3 /* keyword */ : 17 /* parameterName */;
                                }
                                return;
                        }
                    }
                    return 2 /* identifier */;
                }
            }
            function processElement(element) {
                if (!element) {
                    return;
                }
                // Ignore nodes that don't intersect the original span to classify.
                if (ts.decodedTextSpanIntersectsWith(spanStart, spanLength, element.pos, element.getFullWidth())) {
                    checkForClassificationCancellation(cancellationToken, element.kind);
                    for (var _i = 0, _a = element.getChildren(sourceFile); _i < _a.length; _i++) {
                        var child = _a[_i];
                        if (!tryClassifyNode(child)) {
                            // Recurse into our child nodes.
                            processElement(child);
                        }
                    }
                }
            }
        }
        ts.getEncodedSyntacticClassifications = getEncodedSyntacticClassifications;
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var Completions;
        (function (Completions) {
            var StringCompletions;
            (function (StringCompletions) {
                function getStringLiteralCompletions(sourceFile, position, contextToken, checker, options, host, log, preferences) {
                    if (ts.isInReferenceComment(sourceFile, position)) {
                        var entries = getTripleSlashReferenceCompletion(sourceFile, position, options, host);
                        return entries && convertPathCompletions(entries);
                    }
                    if (ts.isInString(sourceFile, position, contextToken)) {
                        if (!contextToken || !ts.isStringLiteralLike(contextToken))
                            return undefined;
                        var entries = getStringLiteralCompletionEntries(sourceFile, contextToken, position, checker, options, host);
                        return convertStringLiteralCompletions(entries, sourceFile, checker, log, preferences);
                    }
                }
                StringCompletions.getStringLiteralCompletions = getStringLiteralCompletions;
                function convertStringLiteralCompletions(completion, sourceFile, checker, log, preferences) {
                    if (completion === undefined) {
                        return undefined;
                    }
                    switch (completion.kind) {
                        case 0 /* Paths */:
                            return convertPathCompletions(completion.paths);
                        case 1 /* Properties */: {
                            var entries = [];
                            Completions.getCompletionEntriesFromSymbols(completion.symbols, entries, sourceFile, sourceFile, checker, 99 /* ESNext */, log, 4 /* String */, preferences); // Target will not be used, so arbitrary
                            return { isGlobalCompletion: false, isMemberCompletion: true, isNewIdentifierLocation: completion.hasIndexSignature, entries: entries };
                        }
                        case 2 /* Types */: {
                            var entries = completion.types.map(function (type) { return ({ name: type.value, kindModifiers: "" /* none */, kind: "string" /* string */, sortText: "0" }); });
                            return { isGlobalCompletion: false, isMemberCompletion: false, isNewIdentifierLocation: completion.isNewIdentifier, entries: entries };
                        }
                        default:
                            return ts.Debug.assertNever(completion);
                    }
                }
                function getStringLiteralCompletionDetails(name, sourceFile, position, contextToken, checker, options, host, cancellationToken) {
                    if (!contextToken || !ts.isStringLiteralLike(contextToken))
                        return undefined;
                    var completions = getStringLiteralCompletionEntries(sourceFile, contextToken, position, checker, options, host);
                    return completions && stringLiteralCompletionDetails(name, contextToken, completions, sourceFile, checker, cancellationToken);
                }
                StringCompletions.getStringLiteralCompletionDetails = getStringLiteralCompletionDetails;
                function stringLiteralCompletionDetails(name, location, completion, sourceFile, checker, cancellationToken) {
                    switch (completion.kind) {
                        case 0 /* Paths */: {
                            var match = ts.find(completion.paths, function (p) { return p.name === name; });
                            return match && Completions.createCompletionDetails(name, kindModifiersFromExtension(match.extension), match.kind, [ts.textPart(name)]);
                        }
                        case 1 /* Properties */: {
                            var match = ts.find(completion.symbols, function (s) { return s.name === name; });
                            return match && Completions.createCompletionDetailsForSymbol(match, checker, sourceFile, location, cancellationToken);
                        }
                        case 2 /* Types */:
                            return ts.find(completion.types, function (t) { return t.value === name; }) ? Completions.createCompletionDetails(name, "" /* none */, "type" /* typeElement */, [ts.textPart(name)]) : undefined;
                        default:
                            return ts.Debug.assertNever(completion);
                    }
                }
                function convertPathCompletions(pathCompletions) {
                    var isGlobalCompletion = false; // We don't want the editor to offer any other completions, such as snippets, inside a comment.
                    var isNewIdentifierLocation = true; // The user may type in a path that doesn't yet exist, creating a "new identifier" with respect to the collection of identifiers the server is aware of.
                    var entries = pathCompletions.map(function (_a) {
                        var name = _a.name, kind = _a.kind, span = _a.span, extension = _a.extension;
                        return ({ name: name, kind: kind, kindModifiers: kindModifiersFromExtension(extension), sortText: Completions.SortText.LocationPriority, replacementSpan: span });
                    });
                    return { isGlobalCompletion: isGlobalCompletion, isMemberCompletion: false, isNewIdentifierLocation: isNewIdentifierLocation, entries: entries };
                }
                function kindModifiersFromExtension(extension) {
                    switch (extension) {
                        case ".d.ts" /* Dts */: return ".d.ts" /* dtsModifier */;
                        case ".js" /* Js */: return ".js" /* jsModifier */;
                        case ".json" /* Json */: return ".json" /* jsonModifier */;
                        case ".jsx" /* Jsx */: return ".jsx" /* jsxModifier */;
                        case ".ts" /* Ts */: return ".ts" /* tsModifier */;
                        case ".tsx" /* Tsx */: return ".tsx" /* tsxModifier */;
                        case ".tsbuildinfo" /* TsBuildInfo */: return ts.Debug.fail("Extension " + ".tsbuildinfo" /* TsBuildInfo */ + " is unsupported.");
                        case undefined: return "" /* none */;
                        default:
                            return ts.Debug.assertNever(extension);
                    }
                }
                var StringLiteralCompletionKind;
                (function (StringLiteralCompletionKind) {
                    StringLiteralCompletionKind[StringLiteralCompletionKind["Paths"] = 0] = "Paths";
                    StringLiteralCompletionKind[StringLiteralCompletionKind["Properties"] = 1] = "Properties";
                    StringLiteralCompletionKind[StringLiteralCompletionKind["Types"] = 2] = "Types";
                })(StringLiteralCompletionKind || (StringLiteralCompletionKind = {}));
                function getStringLiteralCompletionEntries(sourceFile, node, position, typeChecker, compilerOptions, host) {
                    var parent = node.parent;
                    switch (parent.kind) {
                        case 187 /* LiteralType */:
                            switch (parent.parent.kind) {
                                case 169 /* TypeReference */:
                                    return { kind: 2 /* Types */, types: getStringLiteralTypes(typeChecker.getTypeArgumentConstraint(parent)), isNewIdentifier: false };
                                case 185 /* IndexedAccessType */:
                                    // Get all apparent property names
                                    // i.e. interface Foo {
                                    //          foo: string;
                                    //          bar: string;
                                    //      }
                                    //      let x: Foo["/*completion position*/"]
                                    return stringLiteralCompletionsFromProperties(typeChecker.getTypeFromTypeNode(parent.parent.objectType));
                                case 188 /* ImportType */:
                                    return { kind: 0 /* Paths */, paths: getStringLiteralCompletionsFromModuleNames(sourceFile, node, compilerOptions, host, typeChecker) };
                                case 178 /* UnionType */: {
                                    if (!ts.isTypeReferenceNode(parent.parent.parent))
                                        return undefined;
                                    var alreadyUsedTypes_1 = getAlreadyUsedTypesInStringLiteralUnion(parent.parent, parent);
                                    var types = getStringLiteralTypes(typeChecker.getTypeArgumentConstraint(parent.parent)).filter(function (t) { return !ts.contains(alreadyUsedTypes_1, t.value); });
                                    return { kind: 2 /* Types */, types: types, isNewIdentifier: false };
                                }
                                default:
                                    return undefined;
                            }
                        case 281 /* PropertyAssignment */:
                            if (ts.isObjectLiteralExpression(parent.parent) && parent.name === node) {
                                // Get quoted name of properties of the object literal expression
                                // i.e. interface ConfigFiles {
                                //          'jspm:dev': string
                                //      }
                                //      let files: ConfigFiles = {
                                //          '/*completion position*/'
                                //      }
                                //
                                //      function foo(c: ConfigFiles) {}
                                //      foo({
                                //          '/*completion position*/'
                                //      });
                                return stringLiteralCompletionsFromProperties(typeChecker.getContextualType(parent.parent));
                            }
                            return fromContextualType();
                        case 195 /* ElementAccessExpression */: {
                            var _a = parent, expression = _a.expression, argumentExpression = _a.argumentExpression;
                            if (node === argumentExpression) {
                                // Get all names of properties on the expression
                                // i.e. interface A {
                                //      'prop1': string
                                // }
                                // let a: A;
                                // a['/*completion position*/']
                                return stringLiteralCompletionsFromProperties(typeChecker.getTypeAtLocation(expression));
                            }
                            return undefined;
                        }
                        case 196 /* CallExpression */:
                        case 197 /* NewExpression */:
                            if (!ts.isRequireCall(parent, /*checkArgumentIsStringLiteralLike*/ false) && !ts.isImportCall(parent)) {
                                var argumentInfo = ts.SignatureHelp.getArgumentInfoForCompletions(node, position, sourceFile);
                                // Get string literal completions from specialized signatures of the target
                                // i.e. declare function f(a: 'A');
                                // f("/*completion position*/")
                                return argumentInfo ? getStringLiteralCompletionsFromSignature(argumentInfo, typeChecker) : fromContextualType();
                            }
                        // falls through (is `require("")` or `import("")`)
                        case 254 /* ImportDeclaration */:
                        case 260 /* ExportDeclaration */:
                        case 265 /* ExternalModuleReference */:
                            // Get all known external module names or complete a path to a module
                            // i.e. import * as ns from "/*completion position*/";
                            //      var y = import("/*completion position*/");
                            //      import x = require("/*completion position*/");
                            //      var y = require("/*completion position*/");
                            //      export * from "/*completion position*/";
                            return { kind: 0 /* Paths */, paths: getStringLiteralCompletionsFromModuleNames(sourceFile, node, compilerOptions, host, typeChecker) };
                        default:
                            return fromContextualType();
                    }
                    function fromContextualType() {
                        // Get completion for string literal from string literal type
                        // i.e. var x: "hi" | "hello" = "/*completion position*/"
                        return { kind: 2 /* Types */, types: getStringLiteralTypes(ts.getContextualTypeFromParent(node, typeChecker)), isNewIdentifier: false };
                    }
                }
                function getAlreadyUsedTypesInStringLiteralUnion(union, current) {
                    return ts.mapDefined(union.types, function (type) {
                        return type !== current && ts.isLiteralTypeNode(type) && ts.isStringLiteral(type.literal) ? type.literal.text : undefined;
                    });
                }
                function getStringLiteralCompletionsFromSignature(argumentInfo, checker) {
                    var isNewIdentifier = false;
                    var uniques = ts.createMap();
                    var candidates = [];
                    checker.getResolvedSignature(argumentInfo.invocation, candidates, argumentInfo.argumentCount);
                    var types = ts.flatMap(candidates, function (candidate) {
                        if (!ts.signatureHasRestParameter(candidate) && argumentInfo.argumentCount > candidate.parameters.length)
                            return;
                        var type = checker.getParameterType(candidate, argumentInfo.argumentIndex);
                        isNewIdentifier = isNewIdentifier || !!(type.flags & 4 /* String */);
                        return getStringLiteralTypes(type, uniques);
                    });
                    return { kind: 2 /* Types */, types: types, isNewIdentifier: isNewIdentifier };
                }
                function stringLiteralCompletionsFromProperties(type) {
                    return type && {
                        kind: 1 /* Properties */,
                        symbols: type.getApparentProperties().filter(function (prop) { return !ts.isPrivateIdentifierPropertyDeclaration(prop.valueDeclaration); }),
                        hasIndexSignature: ts.hasIndexSignature(type)
                    };
                }
                function getStringLiteralTypes(type, uniques) {
                    if (uniques === void 0) {
                        uniques = ts.createMap();
                    }
                    if (!type)
                        return ts.emptyArray;
                    type = ts.skipConstraint(type);
                    return type.isUnion() ? ts.flatMap(type.types, function (t) { return getStringLiteralTypes(t, uniques); }) :
                        type.isStringLiteral() && !(type.flags & 1024 /* EnumLiteral */) && ts.addToSeen(uniques, type.value) ? [type] : ts.emptyArray;
                }
                function nameAndKind(name, kind, extension) {
                    return { name: name, kind: kind, extension: extension };
                }
                function directoryResult(name) {
                    return nameAndKind(name, "directory" /* directory */, /*extension*/ undefined);
                }
                function addReplacementSpans(text, textStart, names) {
                    var span = getDirectoryFragmentTextSpan(text, textStart);
                    return names.map(function (_a) {
                        var name = _a.name, kind = _a.kind, extension = _a.extension;
                        return ({ name: name, kind: kind, extension: extension, span: span });
                    });
                }
                function getStringLiteralCompletionsFromModuleNames(sourceFile, node, compilerOptions, host, typeChecker) {
                    return addReplacementSpans(node.text, node.getStart(sourceFile) + 1, getStringLiteralCompletionsFromModuleNamesWorker(sourceFile, node, compilerOptions, host, typeChecker));
                }
                function getStringLiteralCompletionsFromModuleNamesWorker(sourceFile, node, compilerOptions, host, typeChecker) {
                    var literalValue = ts.normalizeSlashes(node.text);
                    var scriptPath = sourceFile.path;
                    var scriptDirectory = ts.getDirectoryPath(scriptPath);
                    return isPathRelativeToScript(literalValue) || !compilerOptions.baseUrl && (ts.isRootedDiskPath(literalValue) || ts.isUrl(literalValue))
                        ? getCompletionEntriesForRelativeModules(literalValue, scriptDirectory, compilerOptions, host, scriptPath)
                        : getCompletionEntriesForNonRelativeModules(literalValue, scriptDirectory, compilerOptions, host, typeChecker);
                }
                function getExtensionOptions(compilerOptions, includeExtensions) {
                    if (includeExtensions === void 0) {
                        includeExtensions = false;
                    }
                    return { extensions: getSupportedExtensionsForModuleResolution(compilerOptions), includeExtensions: includeExtensions };
                }
                function getCompletionEntriesForRelativeModules(literalValue, scriptDirectory, compilerOptions, host, scriptPath) {
                    var extensionOptions = getExtensionOptions(compilerOptions);
                    if (compilerOptions.rootDirs) {
                        return getCompletionEntriesForDirectoryFragmentWithRootDirs(compilerOptions.rootDirs, literalValue, scriptDirectory, extensionOptions, compilerOptions, host, scriptPath);
                    }
                    else {
                        return getCompletionEntriesForDirectoryFragment(literalValue, scriptDirectory, extensionOptions, host, scriptPath);
                    }
                }
                function getSupportedExtensionsForModuleResolution(compilerOptions) {
                    var extensions = ts.getSupportedExtensions(compilerOptions);
                    return compilerOptions.resolveJsonModule && ts.getEmitModuleResolutionKind(compilerOptions) === ts.ModuleResolutionKind.NodeJs ?
                        extensions.concat(".json" /* Json */) :
                        extensions;
                }
                /**
                 * Takes a script path and returns paths for all potential folders that could be merged with its
                 * containing folder via the "rootDirs" compiler option
                 */
                function getBaseDirectoriesFromRootDirs(rootDirs, basePath, scriptDirectory, ignoreCase) {
                    // Make all paths absolute/normalized if they are not already
                    rootDirs = rootDirs.map(function (rootDirectory) { return ts.normalizePath(ts.isRootedDiskPath(rootDirectory) ? rootDirectory : ts.combinePaths(basePath, rootDirectory)); });
                    // Determine the path to the directory containing the script relative to the root directory it is contained within
                    var relativeDirectory = ts.firstDefined(rootDirs, function (rootDirectory) {
                        return ts.containsPath(rootDirectory, scriptDirectory, basePath, ignoreCase) ? scriptDirectory.substr(rootDirectory.length) : undefined;
                    }); // TODO: GH#18217
                    // Now find a path for each potential directory that is to be merged with the one containing the script
                    return ts.deduplicate(__spreadArrays(rootDirs.map(function (rootDirectory) { return ts.combinePaths(rootDirectory, relativeDirectory); }), [scriptDirectory]), ts.equateStringsCaseSensitive, ts.compareStringsCaseSensitive);
                }
                function getCompletionEntriesForDirectoryFragmentWithRootDirs(rootDirs, fragment, scriptDirectory, extensionOptions, compilerOptions, host, exclude) {
                    var basePath = compilerOptions.project || host.getCurrentDirectory();
                    var ignoreCase = !(host.useCaseSensitiveFileNames && host.useCaseSensitiveFileNames());
                    var baseDirectories = getBaseDirectoriesFromRootDirs(rootDirs, basePath, scriptDirectory, ignoreCase);
                    return ts.flatMap(baseDirectories, function (baseDirectory) { return getCompletionEntriesForDirectoryFragment(fragment, baseDirectory, extensionOptions, host, exclude); });
                }
                /**
                 * Given a path ending at a directory, gets the completions for the path, and filters for those entries containing the basename.
                 */
                function getCompletionEntriesForDirectoryFragment(fragment, scriptPath, _a, host, exclude, result) {
                    var extensions = _a.extensions, includeExtensions = _a.includeExtensions;
                    if (result === void 0) {
                        result = [];
                    }
                    if (fragment === undefined) {
                        fragment = "";
                    }
                    fragment = ts.normalizeSlashes(fragment);
                    /**
                     * Remove the basename from the path. Note that we don't use the basename to filter completions;
                     * the client is responsible for refining completions.
                     */
                    if (!ts.hasTrailingDirectorySeparator(fragment)) {
                        fragment = ts.getDirectoryPath(fragment);
                    }
                    if (fragment === "") {
                        fragment = "." + ts.directorySeparator;
                    }
                    fragment = ts.ensureTrailingDirectorySeparator(fragment);
                    // const absolutePath = normalizeAndPreserveTrailingSlash(isRootedDiskPath(fragment) ? fragment : combinePaths(scriptPath, fragment)); // TODO(rbuckton): should use resolvePaths
                    var absolutePath = ts.resolvePath(scriptPath, fragment);
                    var baseDirectory = ts.hasTrailingDirectorySeparator(absolutePath) ? absolutePath : ts.getDirectoryPath(absolutePath);
                    var ignoreCase = !(host.useCaseSensitiveFileNames && host.useCaseSensitiveFileNames());
                    if (!ts.tryDirectoryExists(host, baseDirectory))
                        return result;
                    // Enumerate the available files if possible
                    var files = ts.tryReadDirectory(host, baseDirectory, extensions, /*exclude*/ undefined, /*include*/ ["./*"]);
                    if (files) {
                        /**
                         * Multiple file entries might map to the same truncated name once we remove extensions
                         * (happens iff includeExtensions === false)so we use a set-like data structure. Eg:
                         *
                         * both foo.ts and foo.tsx become foo
                         */
                        var foundFiles = ts.createMap(); // maps file to its extension
                        for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
                            var filePath = files_1[_i];
                            filePath = ts.normalizePath(filePath);
                            if (exclude && ts.comparePaths(filePath, exclude, scriptPath, ignoreCase) === 0 /* EqualTo */) {
                                continue;
                            }
                            var foundFileName = includeExtensions || ts.fileExtensionIs(filePath, ".json" /* Json */) ? ts.getBaseFileName(filePath) : ts.removeFileExtension(ts.getBaseFileName(filePath));
                            foundFiles.set(foundFileName, ts.tryGetExtensionFromPath(filePath));
                        }
                        foundFiles.forEach(function (ext, foundFile) {
                            result.push(nameAndKind(foundFile, "script" /* scriptElement */, ext));
                        });
                    }
                    // If possible, get folder completion as well
                    var directories = ts.tryGetDirectories(host, baseDirectory);
                    if (directories) {
                        for (var _b = 0, directories_1 = directories; _b < directories_1.length; _b++) {
                            var directory = directories_1[_b];
                            var directoryName = ts.getBaseFileName(ts.normalizePath(directory));
                            if (directoryName !== "@types") {
                                result.push(directoryResult(directoryName));
                            }
                        }
                    }
                    // check for a version redirect
                    var packageJsonPath = ts.findPackageJson(baseDirectory, host);
                    if (packageJsonPath) {
                        var packageJson = ts.readJson(packageJsonPath, host);
                        var typesVersions = packageJson.typesVersions;
                        if (typeof typesVersions === "object") {
                            var versionResult = ts.getPackageJsonTypesVersionsPaths(typesVersions);
                            var versionPaths = versionResult && versionResult.paths;
                            var rest = absolutePath.slice(ts.ensureTrailingDirectorySeparator(baseDirectory).length);
                            if (versionPaths) {
                                addCompletionEntriesFromPaths(result, rest, baseDirectory, extensions, versionPaths, host);
                            }
                        }
                    }
                    return result;
                }
                function addCompletionEntriesFromPaths(result, fragment, baseDirectory, fileExtensions, paths, host) {
                    for (var path in paths) {
                        if (!ts.hasProperty(paths, path))
                            continue;
                        var patterns = paths[path];
                        if (patterns) {
                            var _loop_1 = function (name, kind, extension) {
                                // Path mappings may provide a duplicate way to get to something we've already added, so don't add again.
                                if (!result.some(function (entry) { return entry.name === name; })) {
                                    result.push(nameAndKind(name, kind, extension));
                                }
                            };
                            for (var _i = 0, _a = getCompletionsForPathMapping(path, patterns, fragment, baseDirectory, fileExtensions, host); _i < _a.length; _i++) {
                                var _b = _a[_i], name = _b.name, kind = _b.kind, extension = _b.extension;
                                _loop_1(name, kind, extension);
                            }
                        }
                    }
                }
                /**
                 * Check all of the declared modules and those in node modules. Possible sources of modules:
                 *      Modules that are found by the type checker
                 *      Modules found relative to "baseUrl" compliler options (including patterns from "paths" compiler option)
                 *      Modules from node_modules (i.e. those listed in package.json)
                 *          This includes all files that are found in node_modules/moduleName/ with acceptable file extensions
                 */
                function getCompletionEntriesForNonRelativeModules(fragment, scriptPath, compilerOptions, host, typeChecker) {
                    var baseUrl = compilerOptions.baseUrl, paths = compilerOptions.paths;
                    var result = [];
                    var extensionOptions = getExtensionOptions(compilerOptions);
                    if (baseUrl) {
                        var projectDir = compilerOptions.project || host.getCurrentDirectory();
                        var absolute = ts.normalizePath(ts.combinePaths(projectDir, baseUrl));
                        getCompletionEntriesForDirectoryFragment(fragment, absolute, extensionOptions, host, /*exclude*/ undefined, result);
                        if (paths) {
                            addCompletionEntriesFromPaths(result, fragment, absolute, extensionOptions.extensions, paths, host);
                        }
                    }
                    var fragmentDirectory = getFragmentDirectory(fragment);
                    for (var _i = 0, _a = getAmbientModuleCompletions(fragment, fragmentDirectory, typeChecker); _i < _a.length; _i++) {
                        var ambientName = _a[_i];
                        result.push(nameAndKind(ambientName, "external module name" /* externalModuleName */, /*extension*/ undefined));
                    }
                    getCompletionEntriesFromTypings(host, compilerOptions, scriptPath, fragmentDirectory, extensionOptions, result);
                    if (ts.getEmitModuleResolutionKind(compilerOptions) === ts.ModuleResolutionKind.NodeJs) {
                        // If looking for a global package name, don't just include everything in `node_modules` because that includes dependencies' own dependencies.
                        // (But do if we didn't find anything, e.g. 'package.json' missing.)
                        var foundGlobal = false;
                        if (fragmentDirectory === undefined) {
                            var _loop_2 = function (moduleName) {
                                if (!result.some(function (entry) { return entry.name === moduleName; })) {
                                    foundGlobal = true;
                                    result.push(nameAndKind(moduleName, "external module name" /* externalModuleName */, /*extension*/ undefined));
                                }
                            };
                            for (var _b = 0, _c = enumerateNodeModulesVisibleToScript(host, scriptPath); _b < _c.length; _b++) {
                                var moduleName = _c[_b];
                                _loop_2(moduleName);
                            }
                        }
                        if (!foundGlobal) {
                            ts.forEachAncestorDirectory(scriptPath, function (ancestor) {
                                var nodeModules = ts.combinePaths(ancestor, "node_modules");
                                if (ts.tryDirectoryExists(host, nodeModules)) {
                                    getCompletionEntriesForDirectoryFragment(fragment, nodeModules, extensionOptions, host, /*exclude*/ undefined, result);
                                }
                            });
                        }
                    }
                    return result;
                }
                function getFragmentDirectory(fragment) {
                    return containsSlash(fragment) ? ts.hasTrailingDirectorySeparator(fragment) ? fragment : ts.getDirectoryPath(fragment) : undefined;
                }
                function getCompletionsForPathMapping(path, patterns, fragment, baseUrl, fileExtensions, host) {
                    if (!ts.endsWith(path, "*")) {
                        // For a path mapping "foo": ["/x/y/z.ts"], add "foo" itself as a completion.
                        return !ts.stringContains(path, "*") ? justPathMappingName(path) : ts.emptyArray;
                    }
                    var pathPrefix = path.slice(0, path.length - 1);
                    var remainingFragment = ts.tryRemovePrefix(fragment, pathPrefix);
                    return remainingFragment === undefined ? justPathMappingName(pathPrefix) : ts.flatMap(patterns, function (pattern) {
                        return getModulesForPathsPattern(remainingFragment, baseUrl, pattern, fileExtensions, host);
                    });
                    function justPathMappingName(name) {
                        return ts.startsWith(name, fragment) ? [directoryResult(name)] : ts.emptyArray;
                    }
                }
                function getModulesForPathsPattern(fragment, baseUrl, pattern, fileExtensions, host) {
                    if (!host.readDirectory) {
                        return undefined;
                    }
                    var parsed = ts.hasZeroOrOneAsteriskCharacter(pattern) ? ts.tryParsePattern(pattern) : undefined;
                    if (!parsed) {
                        return undefined;
                    }
                    // The prefix has two effective parts: the directory path and the base component after the filepath that is not a
                    // full directory component. For example: directory/path/of/prefix/base*
                    var normalizedPrefix = ts.resolvePath(parsed.prefix);
                    var normalizedPrefixDirectory = ts.hasTrailingDirectorySeparator(parsed.prefix) ? normalizedPrefix : ts.getDirectoryPath(normalizedPrefix);
                    var normalizedPrefixBase = ts.hasTrailingDirectorySeparator(parsed.prefix) ? "" : ts.getBaseFileName(normalizedPrefix);
                    var fragmentHasPath = containsSlash(fragment);
                    var fragmentDirectory = fragmentHasPath ? ts.hasTrailingDirectorySeparator(fragment) ? fragment : ts.getDirectoryPath(fragment) : undefined;
                    // Try and expand the prefix to include any path from the fragment so that we can limit the readDirectory call
                    var expandedPrefixDirectory = fragmentHasPath ? ts.combinePaths(normalizedPrefixDirectory, normalizedPrefixBase + fragmentDirectory) : normalizedPrefixDirectory;
                    var normalizedSuffix = ts.normalizePath(parsed.suffix);
                    // Need to normalize after combining: If we combinePaths("a", "../b"), we want "b" and not "a/../b".
                    var baseDirectory = ts.normalizePath(ts.combinePaths(baseUrl, expandedPrefixDirectory));
                    var completePrefix = fragmentHasPath ? baseDirectory : ts.ensureTrailingDirectorySeparator(baseDirectory) + normalizedPrefixBase;
                    // If we have a suffix, then we need to read the directory all the way down. We could create a glob
                    // that encodes the suffix, but we would have to escape the character "?" which readDirectory
                    // doesn't support. For now, this is safer but slower
                    var includeGlob = normalizedSuffix ? "**/*" : "./*";
                    var matches = ts.mapDefined(ts.tryReadDirectory(host, baseDirectory, fileExtensions, /*exclude*/ undefined, [includeGlob]), function (match) {
                        var extension = ts.tryGetExtensionFromPath(match);
                        var name = trimPrefixAndSuffix(match);
                        return name === undefined ? undefined : nameAndKind(ts.removeFileExtension(name), "script" /* scriptElement */, extension);
                    });
                    var directories = ts.mapDefined(ts.tryGetDirectories(host, baseDirectory).map(function (d) { return ts.combinePaths(baseDirectory, d); }), function (dir) {
                        var name = trimPrefixAndSuffix(dir);
                        return name === undefined ? undefined : directoryResult(name);
                    });
                    return __spreadArrays(matches, directories);
                    function trimPrefixAndSuffix(path) {
                        var inner = withoutStartAndEnd(ts.normalizePath(path), completePrefix, normalizedSuffix);
                        return inner === undefined ? undefined : removeLeadingDirectorySeparator(inner);
                    }
                }
                function withoutStartAndEnd(s, start, end) {
                    return ts.startsWith(s, start) && ts.endsWith(s, end) ? s.slice(start.length, s.length - end.length) : undefined;
                }
                function removeLeadingDirectorySeparator(path) {
                    return path[0] === ts.directorySeparator ? path.slice(1) : path;
                }
                function getAmbientModuleCompletions(fragment, fragmentDirectory, checker) {
                    // Get modules that the type checker picked up
                    var ambientModules = checker.getAmbientModules().map(function (sym) { return ts.stripQuotes(sym.name); });
                    var nonRelativeModuleNames = ambientModules.filter(function (moduleName) { return ts.startsWith(moduleName, fragment); });
                    // Nested modules of the form "module-name/sub" need to be adjusted to only return the string
                    // after the last '/' that appears in the fragment because that's where the replacement span
                    // starts
                    if (fragmentDirectory !== undefined) {
                        var moduleNameWithSeparator_1 = ts.ensureTrailingDirectorySeparator(fragmentDirectory);
                        return nonRelativeModuleNames.map(function (nonRelativeModuleName) { return ts.removePrefix(nonRelativeModuleName, moduleNameWithSeparator_1); });
                    }
                    return nonRelativeModuleNames;
                }
                function getTripleSlashReferenceCompletion(sourceFile, position, compilerOptions, host) {
                    var token = ts.getTokenAtPosition(sourceFile, position);
                    var commentRanges = ts.getLeadingCommentRanges(sourceFile.text, token.pos);
                    var range = commentRanges && ts.find(commentRanges, function (commentRange) { return position >= commentRange.pos && position <= commentRange.end; });
                    if (!range) {
                        return undefined;
                    }
                    var text = sourceFile.text.slice(range.pos, position);
                    var match = tripleSlashDirectiveFragmentRegex.exec(text);
                    if (!match) {
                        return undefined;
                    }
                    var prefix = match[1], kind = match[2], toComplete = match[3];
                    var scriptPath = ts.getDirectoryPath(sourceFile.path);
                    var names = kind === "path" ? getCompletionEntriesForDirectoryFragment(toComplete, scriptPath, getExtensionOptions(compilerOptions, /*includeExtensions*/ true), host, sourceFile.path)
                        : kind === "types" ? getCompletionEntriesFromTypings(host, compilerOptions, scriptPath, getFragmentDirectory(toComplete), getExtensionOptions(compilerOptions))
                            : ts.Debug.fail();
                    return addReplacementSpans(toComplete, range.pos + prefix.length, names);
                }
                function getCompletionEntriesFromTypings(host, options, scriptPath, fragmentDirectory, extensionOptions, result) {
                    if (result === void 0) {
                        result = [];
                    }
                    // Check for typings specified in compiler options
                    var seen = ts.createMap();
                    var typeRoots = ts.tryAndIgnoreErrors(function () { return ts.getEffectiveTypeRoots(options, host); }) || ts.emptyArray;
                    for (var _i = 0, typeRoots_1 = typeRoots; _i < typeRoots_1.length; _i++) {
                        var root = typeRoots_1[_i];
                        getCompletionEntriesFromDirectories(root);
                    }
                    // Also get all @types typings installed in visible node_modules directories
                    for (var _a = 0, _b = ts.findPackageJsons(scriptPath, host); _a < _b.length; _a++) {
                        var packageJson = _b[_a];
                        var typesDir = ts.combinePaths(ts.getDirectoryPath(packageJson), "node_modules/@types");
                        getCompletionEntriesFromDirectories(typesDir);
                    }
                    return result;
                    function getCompletionEntriesFromDirectories(directory) {
                        if (!ts.tryDirectoryExists(host, directory))
                            return;
                        for (var _i = 0, _a = ts.tryGetDirectories(host, directory); _i < _a.length; _i++) {
                            var typeDirectoryName = _a[_i];
                            var packageName = ts.unmangleScopedPackageName(typeDirectoryName);
                            if (options.types && !ts.contains(options.types, packageName))
                                continue;
                            if (fragmentDirectory === undefined) {
                                if (!seen.has(packageName)) {
                                    result.push(nameAndKind(packageName, "external module name" /* externalModuleName */, /*extension*/ undefined));
                                    seen.set(packageName, true);
                                }
                            }
                            else {
                                var baseDirectory = ts.combinePaths(directory, typeDirectoryName);
                                var remainingFragment = ts.tryRemoveDirectoryPrefix(fragmentDirectory, packageName, ts.hostGetCanonicalFileName(host));
                                if (remainingFragment !== undefined) {
                                    getCompletionEntriesForDirectoryFragment(remainingFragment, baseDirectory, extensionOptions, host, /*exclude*/ undefined, result);
                                }
                            }
                        }
                    }
                }
                function enumerateNodeModulesVisibleToScript(host, scriptPath) {
                    if (!host.readFile || !host.fileExists)
                        return ts.emptyArray;
                    var result = [];
                    for (var _i = 0, _a = ts.findPackageJsons(scriptPath, host); _i < _a.length; _i++) {
                        var packageJson = _a[_i];
                        var contents = ts.readJson(packageJson, host); // Cast to assert that readFile is defined
                        // Provide completions for all non @types dependencies
                        for (var _b = 0, nodeModulesDependencyKeys_1 = nodeModulesDependencyKeys; _b < nodeModulesDependencyKeys_1.length; _b++) {
                            var key = nodeModulesDependencyKeys_1[_b];
                            var dependencies = contents[key];
                            if (!dependencies)
                                continue;
                            for (var dep in dependencies) {
                                if (dependencies.hasOwnProperty(dep) && !ts.startsWith(dep, "@types/")) {
                                    result.push(dep);
                                }
                            }
                        }
                    }
                    return result;
                }
                // Replace everything after the last directory separator that appears
                function getDirectoryFragmentTextSpan(text, textStart) {
                    var index = Math.max(text.lastIndexOf(ts.directorySeparator), text.lastIndexOf("\\"));
                    var offset = index !== -1 ? index + 1 : 0;
                    // If the range is an identifier, span is unnecessary.
                    var length = text.length - offset;
                    return length === 0 || ts.isIdentifierText(text.substr(offset, length), 99 /* ESNext */) ? undefined : ts.createTextSpan(textStart + offset, length);
                }
                // Returns true if the path is explicitly relative to the script (i.e. relative to . or ..)
                function isPathRelativeToScript(path) {
                    if (path && path.length >= 2 && path.charCodeAt(0) === 46 /* dot */) {
                        var slashIndex = path.length >= 3 && path.charCodeAt(1) === 46 /* dot */ ? 2 : 1;
                        var slashCharCode = path.charCodeAt(slashIndex);
                        return slashCharCode === 47 /* slash */ || slashCharCode === 92 /* backslash */;
                    }
                    return false;
                }
                /**
                 * Matches a triple slash reference directive with an incomplete string literal for its path. Used
                 * to determine if the caret is currently within the string literal and capture the literal fragment
                 * for completions.
                 * For example, this matches
                 *
                 * /// <reference path="fragment
                 *
                 * but not
                 *
                 * /// <reference path="fragment"
                 */
                var tripleSlashDirectiveFragmentRegex = /^(\/\/\/\s*<reference\s+(path|types)\s*=\s*(?:'|"))([^\3"]*)$/;
                var nodeModulesDependencyKeys = ["dependencies", "devDependencies", "peerDependencies", "optionalDependencies"];
                function containsSlash(fragment) {
                    return ts.stringContains(fragment, ts.directorySeparator);
                }
            })(StringCompletions = Completions.StringCompletions || (Completions.StringCompletions = {}));
        })(Completions = ts.Completions || (ts.Completions = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var Completions;
        (function (Completions) {
            var SortText;
            (function (SortText) {
                SortText["LocationPriority"] = "0";
                SortText["OptionalMember"] = "1";
                SortText["MemberDeclaredBySpreadAssignment"] = "2";
                SortText["SuggestedClassMembers"] = "3";
                SortText["GlobalsOrKeywords"] = "4";
                SortText["AutoImportSuggestions"] = "5";
                SortText["JavascriptIdentifiers"] = "6";
            })(SortText = Completions.SortText || (Completions.SortText = {}));
            var SymbolOriginInfoKind;
            (function (SymbolOriginInfoKind) {
                SymbolOriginInfoKind[SymbolOriginInfoKind["ThisType"] = 1] = "ThisType";
                SymbolOriginInfoKind[SymbolOriginInfoKind["SymbolMember"] = 2] = "SymbolMember";
                SymbolOriginInfoKind[SymbolOriginInfoKind["Export"] = 4] = "Export";
                SymbolOriginInfoKind[SymbolOriginInfoKind["Promise"] = 8] = "Promise";
                SymbolOriginInfoKind[SymbolOriginInfoKind["Nullable"] = 16] = "Nullable";
                SymbolOriginInfoKind[SymbolOriginInfoKind["SymbolMemberNoExport"] = 2] = "SymbolMemberNoExport";
                SymbolOriginInfoKind[SymbolOriginInfoKind["SymbolMemberExport"] = 6] = "SymbolMemberExport";
            })(SymbolOriginInfoKind || (SymbolOriginInfoKind = {}));
            function originIsThisType(origin) {
                return !!(origin.kind & 1 /* ThisType */);
            }
            function originIsSymbolMember(origin) {
                return !!(origin.kind & 2 /* SymbolMember */);
            }
            function originIsExport(origin) {
                return !!(origin.kind & 4 /* Export */);
            }
            function originIsPromise(origin) {
                return !!(origin.kind & 8 /* Promise */);
            }
            function originIsNullableMember(origin) {
                return !!(origin.kind & 16 /* Nullable */);
            }
            var KeywordCompletionFilters;
            (function (KeywordCompletionFilters) {
                KeywordCompletionFilters[KeywordCompletionFilters["None"] = 0] = "None";
                KeywordCompletionFilters[KeywordCompletionFilters["All"] = 1] = "All";
                KeywordCompletionFilters[KeywordCompletionFilters["ClassElementKeywords"] = 2] = "ClassElementKeywords";
                KeywordCompletionFilters[KeywordCompletionFilters["InterfaceElementKeywords"] = 3] = "InterfaceElementKeywords";
                KeywordCompletionFilters[KeywordCompletionFilters["ConstructorParameterKeywords"] = 4] = "ConstructorParameterKeywords";
                KeywordCompletionFilters[KeywordCompletionFilters["FunctionLikeBodyKeywords"] = 5] = "FunctionLikeBodyKeywords";
                KeywordCompletionFilters[KeywordCompletionFilters["TypeAssertionKeywords"] = 6] = "TypeAssertionKeywords";
                KeywordCompletionFilters[KeywordCompletionFilters["TypeKeywords"] = 7] = "TypeKeywords";
                KeywordCompletionFilters[KeywordCompletionFilters["Last"] = 7] = "Last";
            })(KeywordCompletionFilters || (KeywordCompletionFilters = {}));
            var GlobalsSearch;
            (function (GlobalsSearch) {
                GlobalsSearch[GlobalsSearch["Continue"] = 0] = "Continue";
                GlobalsSearch[GlobalsSearch["Success"] = 1] = "Success";
                GlobalsSearch[GlobalsSearch["Fail"] = 2] = "Fail";
            })(GlobalsSearch || (GlobalsSearch = {}));
            function createImportSuggestionsForFileCache() {
                var cache;
                var projectVersion;
                var fileName;
                return {
                    isEmpty: function () {
                        return !cache;
                    },
                    clear: function () {
                        cache = undefined;
                        fileName = undefined;
                        projectVersion = undefined;
                    },
                    set: function (file, suggestions, version) {
                        cache = suggestions;
                        fileName = file;
                        if (version) {
                            projectVersion = version;
                        }
                    },
                    get: function (file, checker, version) {
                        if (file !== fileName) {
                            return undefined;
                        }
                        if (version) {
                            return projectVersion === version ? cache : undefined;
                        }
                        ts.forEach(cache, function (suggestion) {
                            var _a, _b, _c;
                            // If the symbol/moduleSymbol was a merged symbol, it will have a new identity
                            // in the checker, even though the symbols to merge are the same (guaranteed by
                            // cache invalidation in synchronizeHostData).
                            if ((_a = suggestion.symbol.declarations) === null || _a === void 0 ? void 0 : _a.length) {
                                suggestion.symbol = checker.getMergedSymbol(suggestion.origin.isDefaultExport
                                    ? (_b = suggestion.symbol.declarations[0].localSymbol) !== null && _b !== void 0 ? _b : suggestion.symbol.declarations[0].symbol : suggestion.symbol.declarations[0].symbol);
                            }
                            if ((_c = suggestion.origin.moduleSymbol.declarations) === null || _c === void 0 ? void 0 : _c.length) {
                                suggestion.origin.moduleSymbol = checker.getMergedSymbol(suggestion.origin.moduleSymbol.declarations[0].symbol);
                            }
                        });
                        return cache;
                    },
                };
            }
            Completions.createImportSuggestionsForFileCache = createImportSuggestionsForFileCache;
            function getCompletionsAtPosition(host, program, log, sourceFile, position, preferences, triggerCharacter) {
                var typeChecker = program.getTypeChecker();
                var compilerOptions = program.getCompilerOptions();
                var contextToken = ts.findPrecedingToken(position, sourceFile);
                if (triggerCharacter && !ts.isInString(sourceFile, position, contextToken) && !isValidTrigger(sourceFile, triggerCharacter, contextToken, position)) {
                    return undefined;
                }
                var stringCompletions = Completions.StringCompletions.getStringLiteralCompletions(sourceFile, position, contextToken, typeChecker, compilerOptions, host, log, preferences);
                if (stringCompletions) {
                    return stringCompletions;
                }
                if (contextToken && ts.isBreakOrContinueStatement(contextToken.parent)
                    && (contextToken.kind === 77 /* BreakKeyword */ || contextToken.kind === 82 /* ContinueKeyword */ || contextToken.kind === 75 /* Identifier */)) {
                    return getLabelCompletionAtPosition(contextToken.parent);
                }
                var completionData = getCompletionData(program, log, sourceFile, isUncheckedFile(sourceFile, compilerOptions), position, preferences, /*detailsEntryId*/ undefined, host);
                if (!completionData) {
                    return undefined;
                }
                switch (completionData.kind) {
                    case 0 /* Data */:
                        return completionInfoFromData(sourceFile, typeChecker, compilerOptions, log, completionData, preferences);
                    case 1 /* JsDocTagName */:
                        // If the current position is a jsDoc tag name, only tag names should be provided for completion
                        return jsdocCompletionInfo(ts.JsDoc.getJSDocTagNameCompletions());
                    case 2 /* JsDocTag */:
                        // If the current position is a jsDoc tag, only tags should be provided for completion
                        return jsdocCompletionInfo(ts.JsDoc.getJSDocTagCompletions());
                    case 3 /* JsDocParameterName */:
                        return jsdocCompletionInfo(ts.JsDoc.getJSDocParameterNameCompletions(completionData.tag));
                    default:
                        return ts.Debug.assertNever(completionData);
                }
            }
            Completions.getCompletionsAtPosition = getCompletionsAtPosition;
            function jsdocCompletionInfo(entries) {
                return { isGlobalCompletion: false, isMemberCompletion: false, isNewIdentifierLocation: false, entries: entries };
            }
            function completionInfoFromData(sourceFile, typeChecker, compilerOptions, log, completionData, preferences) {
                var symbols = completionData.symbols, completionKind = completionData.completionKind, isInSnippetScope = completionData.isInSnippetScope, isNewIdentifierLocation = completionData.isNewIdentifierLocation, location = completionData.location, propertyAccessToConvert = completionData.propertyAccessToConvert, keywordFilters = completionData.keywordFilters, literals = completionData.literals, symbolToOriginInfoMap = completionData.symbolToOriginInfoMap, recommendedCompletion = completionData.recommendedCompletion, isJsxInitializer = completionData.isJsxInitializer, insideJsDocTagTypeExpression = completionData.insideJsDocTagTypeExpression, symbolToSortTextMap = completionData.symbolToSortTextMap;
                if (location && location.parent && ts.isJsxClosingElement(location.parent)) {
                    // In the TypeScript JSX element, if such element is not defined. When users query for completion at closing tag,
                    // instead of simply giving unknown value, the completion will return the tag-name of an associated opening-element.
                    // For example:
                    //     var x = <div> </ /*1*/
                    // The completion list at "1" will contain "div>" with type any
                    // And at `<div> </ /*1*/ >` (with a closing `>`), the completion list will contain "div".
                    var tagName = location.parent.parent.openingElement.tagName;
                    var hasClosingAngleBracket = !!ts.findChildOfKind(location.parent, 31 /* GreaterThanToken */, sourceFile);
                    var entry = {
                        name: tagName.getFullText(sourceFile) + (hasClosingAngleBracket ? "" : ">"),
                        kind: "class" /* classElement */,
                        kindModifiers: undefined,
                        sortText: SortText.LocationPriority,
                    };
                    return { isGlobalCompletion: false, isMemberCompletion: true, isNewIdentifierLocation: false, entries: [entry] };
                }
                var entries = [];
                if (isUncheckedFile(sourceFile, compilerOptions)) {
                    var uniqueNames = getCompletionEntriesFromSymbols(symbols, entries, location, sourceFile, typeChecker, compilerOptions.target, log, completionKind, preferences, propertyAccessToConvert, isJsxInitializer, recommendedCompletion, symbolToOriginInfoMap, symbolToSortTextMap);
                    getJSCompletionEntries(sourceFile, location.pos, uniqueNames, compilerOptions.target, entries); // TODO: GH#18217
                }
                else {
                    if (!isNewIdentifierLocation && (!symbols || symbols.length === 0) && keywordFilters === 0 /* None */) {
                        return undefined;
                    }
                    getCompletionEntriesFromSymbols(symbols, entries, location, sourceFile, typeChecker, compilerOptions.target, log, completionKind, preferences, propertyAccessToConvert, isJsxInitializer, recommendedCompletion, symbolToOriginInfoMap, symbolToSortTextMap);
                }
                if (keywordFilters !== 0 /* None */) {
                    var entryNames = ts.arrayToSet(entries, function (e) { return e.name; });
                    for (var _i = 0, _a = getKeywordCompletions(keywordFilters, !insideJsDocTagTypeExpression && ts.isSourceFileJS(sourceFile)); _i < _a.length; _i++) {
                        var keywordEntry = _a[_i];
                        if (!entryNames.has(keywordEntry.name)) {
                            entries.push(keywordEntry);
                        }
                    }
                }
                for (var _b = 0, literals_1 = literals; _b < literals_1.length; _b++) {
                    var literal = literals_1[_b];
                    entries.push(createCompletionEntryForLiteral(literal));
                }
                return { isGlobalCompletion: isInSnippetScope, isMemberCompletion: isMemberCompletionKind(completionKind), isNewIdentifierLocation: isNewIdentifierLocation, entries: entries };
            }
            function isUncheckedFile(sourceFile, compilerOptions) {
                return ts.isSourceFileJS(sourceFile) && !ts.isCheckJsEnabledForFile(sourceFile, compilerOptions);
            }
            function isMemberCompletionKind(kind) {
                switch (kind) {
                    case 0 /* ObjectPropertyDeclaration */:
                    case 3 /* MemberLike */:
                    case 2 /* PropertyAccess */:
                        return true;
                    default:
                        return false;
                }
            }
            function getJSCompletionEntries(sourceFile, position, uniqueNames, target, entries) {
                ts.getNameTable(sourceFile).forEach(function (pos, name) {
                    // Skip identifiers produced only from the current location
                    if (pos === position) {
                        return;
                    }
                    var realName = ts.unescapeLeadingUnderscores(name);
                    if (ts.addToSeen(uniqueNames, realName) && ts.isIdentifierText(realName, target)) {
                        entries.push({
                            name: realName,
                            kind: "warning" /* warning */,
                            kindModifiers: "",
                            sortText: SortText.JavascriptIdentifiers,
                            isFromUncheckedFile: true
                        });
                    }
                });
            }
            var completionNameForLiteral = function (literal) {
                return typeof literal === "object" ? ts.pseudoBigIntToString(literal) + "n" : JSON.stringify(literal);
            };
            function createCompletionEntryForLiteral(literal) {
                return { name: completionNameForLiteral(literal), kind: "string" /* string */, kindModifiers: "" /* none */, sortText: SortText.LocationPriority };
            }
            function createCompletionEntry(symbol, sortText, location, sourceFile, typeChecker, name, needsConvertPropertyAccess, origin, recommendedCompletion, propertyAccessToConvert, isJsxInitializer, preferences) {
                var insertText;
                var replacementSpan;
                var insertQuestionDot = origin && originIsNullableMember(origin);
                var useBraces = origin && originIsSymbolMember(origin) || needsConvertPropertyAccess;
                if (origin && originIsThisType(origin)) {
                    insertText = needsConvertPropertyAccess
                        ? "this" + (insertQuestionDot ? "?." : "") + "[" + ts.quote(name, preferences) + "]"
                        : "this" + (insertQuestionDot ? "?." : ".") + name;
                }
                // We should only have needsConvertPropertyAccess if there's a property access to convert. But see #21790.
                // Somehow there was a global with a non-identifier name. Hopefully someone will complain about getting a "foo bar" global completion and provide a repro.
                else if ((useBraces || insertQuestionDot) && propertyAccessToConvert) {
                    insertText = useBraces ? needsConvertPropertyAccess ? "[" + ts.quote(name, preferences) + "]" : "[" + name + "]" : name;
                    if (insertQuestionDot || propertyAccessToConvert.questionDotToken) {
                        insertText = "?." + insertText;
                    }
                    var dot = ts.findChildOfKind(propertyAccessToConvert, 24 /* DotToken */, sourceFile) ||
                        ts.findChildOfKind(propertyAccessToConvert, 28 /* QuestionDotToken */, sourceFile);
                    if (!dot) {
                        return undefined;
                    }
                    // If the text after the '.' starts with this name, write over it. Else, add new text.
                    var end = ts.startsWith(name, propertyAccessToConvert.name.text) ? propertyAccessToConvert.name.end : dot.end;
                    replacementSpan = ts.createTextSpanFromBounds(dot.getStart(sourceFile), end);
                }
                if (isJsxInitializer) {
                    if (insertText === undefined)
                        insertText = name;
                    insertText = "{" + insertText + "}";
                    if (typeof isJsxInitializer !== "boolean") {
                        replacementSpan = ts.createTextSpanFromNode(isJsxInitializer, sourceFile);
                    }
                }
                if (origin && originIsPromise(origin) && propertyAccessToConvert) {
                    if (insertText === undefined)
                        insertText = name;
                    var precedingToken = ts.findPrecedingToken(propertyAccessToConvert.pos, sourceFile);
                    var awaitText = "";
                    if (precedingToken && ts.positionIsASICandidate(precedingToken.end, precedingToken.parent, sourceFile)) {
                        awaitText = ";";
                    }
                    awaitText += "(await " + propertyAccessToConvert.expression.getText() + ")";
                    insertText = needsConvertPropertyAccess ? "" + awaitText + insertText : "" + awaitText + (insertQuestionDot ? "?." : ".") + insertText;
                    replacementSpan = ts.createTextSpanFromBounds(propertyAccessToConvert.getStart(sourceFile), propertyAccessToConvert.end);
                }
                if (insertText !== undefined && !preferences.includeCompletionsWithInsertText) {
                    return undefined;
                }
                // TODO(drosen): Right now we just permit *all* semantic meanings when calling
                // 'getSymbolKind' which is permissible given that it is backwards compatible; but
                // really we should consider passing the meaning for the node so that we don't report
                // that a suggestion for a value is an interface.  We COULD also just do what
                // 'getSymbolModifiers' does, which is to use the first declaration.
                // Use a 'sortText' of 0' so that all symbol completion entries come before any other
                // entries (like JavaScript identifier entries).
                return {
                    name: name,
                    kind: ts.SymbolDisplay.getSymbolKind(typeChecker, symbol, location),
                    kindModifiers: ts.SymbolDisplay.getSymbolModifiers(symbol),
                    sortText: sortText,
                    source: getSourceFromOrigin(origin),
                    hasAction: origin && originIsExport(origin) || undefined,
                    isRecommended: isRecommendedCompletionMatch(symbol, recommendedCompletion, typeChecker) || undefined,
                    insertText: insertText,
                    replacementSpan: replacementSpan,
                };
            }
            function isRecommendedCompletionMatch(localSymbol, recommendedCompletion, checker) {
                return localSymbol === recommendedCompletion ||
                    !!(localSymbol.flags & 1048576 /* ExportValue */) && checker.getExportSymbolOfSymbol(localSymbol) === recommendedCompletion;
            }
            function getSourceFromOrigin(origin) {
                return origin && originIsExport(origin) ? ts.stripQuotes(origin.moduleSymbol.name) : undefined;
            }
            function getCompletionEntriesFromSymbols(symbols, entries, location, sourceFile, typeChecker, target, log, kind, preferences, propertyAccessToConvert, isJsxInitializer, recommendedCompletion, symbolToOriginInfoMap, symbolToSortTextMap) {
                var start = ts.timestamp();
                // Tracks unique names.
                // We don't set this for global variables or completions from external module exports, because we can have multiple of those.
                // Based on the order we add things we will always see locals first, then globals, then module exports.
                // So adding a completion for a local will prevent us from adding completions for external module exports sharing the same name.
                var uniques = ts.createMap();
                for (var _i = 0, symbols_1 = symbols; _i < symbols_1.length; _i++) {
                    var symbol = symbols_1[_i];
                    var origin = symbolToOriginInfoMap ? symbolToOriginInfoMap[ts.getSymbolId(symbol)] : undefined;
                    var info = getCompletionEntryDisplayNameForSymbol(symbol, target, origin, kind);
                    if (!info) {
                        continue;
                    }
                    var name = info.name, needsConvertPropertyAccess = info.needsConvertPropertyAccess;
                    if (uniques.has(name)) {
                        continue;
                    }
                    var entry = createCompletionEntry(symbol, symbolToSortTextMap && symbolToSortTextMap[ts.getSymbolId(symbol)] || SortText.LocationPriority, location, sourceFile, typeChecker, name, needsConvertPropertyAccess, origin, recommendedCompletion, propertyAccessToConvert, isJsxInitializer, preferences);
                    if (!entry) {
                        continue;
                    }
                    // Latter case tests whether this is a global variable.
                    if (!origin && !(symbol.parent === undefined && !ts.some(symbol.declarations, function (d) { return d.getSourceFile() === location.getSourceFile(); }))) { // TODO: GH#18217
                        uniques.set(name, true);
                    }
                    entries.push(entry);
                }
                log("getCompletionsAtPosition: getCompletionEntriesFromSymbols: " + (ts.timestamp() - start));
                return uniques;
            }
            Completions.getCompletionEntriesFromSymbols = getCompletionEntriesFromSymbols;
            function getLabelCompletionAtPosition(node) {
                var entries = getLabelStatementCompletions(node);
                if (entries.length) {
                    return { isGlobalCompletion: false, isMemberCompletion: false, isNewIdentifierLocation: false, entries: entries };
                }
            }
            function getLabelStatementCompletions(node) {
                var entries = [];
                var uniques = ts.createMap();
                var current = node;
                while (current) {
                    if (ts.isFunctionLike(current)) {
                        break;
                    }
                    if (ts.isLabeledStatement(current)) {
                        var name = current.label.text;
                        if (!uniques.has(name)) {
                            uniques.set(name, true);
                            entries.push({
                                name: name,
                                kindModifiers: "" /* none */,
                                kind: "label" /* label */,
                                sortText: SortText.LocationPriority
                            });
                        }
                    }
                    current = current.parent;
                }
                return entries;
            }
            function getSymbolCompletionFromEntryId(program, log, sourceFile, position, entryId, host) {
                var compilerOptions = program.getCompilerOptions();
                var completionData = getCompletionData(program, log, sourceFile, isUncheckedFile(sourceFile, compilerOptions), position, { includeCompletionsForModuleExports: true, includeCompletionsWithInsertText: true }, entryId, host);
                if (!completionData) {
                    return { type: "none" };
                }
                if (completionData.kind !== 0 /* Data */) {
                    return { type: "request", request: completionData };
                }
                var symbols = completionData.symbols, literals = completionData.literals, location = completionData.location, completionKind = completionData.completionKind, symbolToOriginInfoMap = completionData.symbolToOriginInfoMap, previousToken = completionData.previousToken, isJsxInitializer = completionData.isJsxInitializer, isTypeOnlyLocation = completionData.isTypeOnlyLocation;
                var literal = ts.find(literals, function (l) { return completionNameForLiteral(l) === entryId.name; });
                if (literal !== undefined)
                    return { type: "literal", literal: literal };
                // Find the symbol with the matching entry name.
                // We don't need to perform character checks here because we're only comparing the
                // name against 'entryName' (which is known to be good), not building a new
                // completion entry.
                return ts.firstDefined(symbols, function (symbol) {
                    var origin = symbolToOriginInfoMap[ts.getSymbolId(symbol)];
                    var info = getCompletionEntryDisplayNameForSymbol(symbol, compilerOptions.target, origin, completionKind);
                    return info && info.name === entryId.name && getSourceFromOrigin(origin) === entryId.source
                        ? { type: "symbol", symbol: symbol, location: location, symbolToOriginInfoMap: symbolToOriginInfoMap, previousToken: previousToken, isJsxInitializer: isJsxInitializer, isTypeOnlyLocation: isTypeOnlyLocation }
                        : undefined;
                }) || { type: "none" };
            }
            function getSymbolName(symbol, origin, target) {
                return origin && originIsExport(origin) && ((origin.isDefaultExport && symbol.escapedName === "default" /* Default */) ||
                    (symbol.escapedName === "export=" /* ExportEquals */))
                    // Name of "export default foo;" is "foo". Name of "export default 0" is the filename converted to camelCase.
                    ? ts.firstDefined(symbol.declarations, function (d) { return ts.isExportAssignment(d) && ts.isIdentifier(d.expression) ? d.expression.text : undefined; })
                        || ts.codefix.moduleSymbolToValidIdentifier(origin.moduleSymbol, target)
                    : symbol.name;
            }
            function getCompletionEntryDetails(program, log, sourceFile, position, entryId, host, formatContext, preferences, cancellationToken) {
                var typeChecker = program.getTypeChecker();
                var compilerOptions = program.getCompilerOptions();
                var name = entryId.name;
                var contextToken = ts.findPrecedingToken(position, sourceFile);
                if (ts.isInString(sourceFile, position, contextToken)) {
                    return Completions.StringCompletions.getStringLiteralCompletionDetails(name, sourceFile, position, contextToken, typeChecker, compilerOptions, host, cancellationToken);
                }
                // Compute all the completion symbols again.
                var symbolCompletion = getSymbolCompletionFromEntryId(program, log, sourceFile, position, entryId, host);
                switch (symbolCompletion.type) {
                    case "request": {
                        var request = symbolCompletion.request;
                        switch (request.kind) {
                            case 1 /* JsDocTagName */:
                                return ts.JsDoc.getJSDocTagNameCompletionDetails(name);
                            case 2 /* JsDocTag */:
                                return ts.JsDoc.getJSDocTagCompletionDetails(name);
                            case 3 /* JsDocParameterName */:
                                return ts.JsDoc.getJSDocParameterNameCompletionDetails(name);
                            default:
                                return ts.Debug.assertNever(request);
                        }
                    }
                    case "symbol": {
                        var symbol = symbolCompletion.symbol, location = symbolCompletion.location, symbolToOriginInfoMap = symbolCompletion.symbolToOriginInfoMap, previousToken = symbolCompletion.previousToken;
                        var _a = getCompletionEntryCodeActionsAndSourceDisplay(symbolToOriginInfoMap, symbol, program, typeChecker, host, compilerOptions, sourceFile, position, previousToken, formatContext, preferences), codeActions = _a.codeActions, sourceDisplay = _a.sourceDisplay;
                        return createCompletionDetailsForSymbol(symbol, typeChecker, sourceFile, location, cancellationToken, codeActions, sourceDisplay); // TODO: GH#18217
                    }
                    case "literal": {
                        var literal = symbolCompletion.literal;
                        return createSimpleDetails(completionNameForLiteral(literal), "string" /* string */, typeof literal === "string" ? ts.SymbolDisplayPartKind.stringLiteral : ts.SymbolDisplayPartKind.numericLiteral);
                    }
                    case "none":
                        // Didn't find a symbol with this name.  See if we can find a keyword instead.
                        return allKeywordsCompletions().some(function (c) { return c.name === name; }) ? createSimpleDetails(name, "keyword" /* keyword */, ts.SymbolDisplayPartKind.keyword) : undefined;
                    default:
                        ts.Debug.assertNever(symbolCompletion);
                }
            }
            Completions.getCompletionEntryDetails = getCompletionEntryDetails;
            function createSimpleDetails(name, kind, kind2) {
                return createCompletionDetails(name, "" /* none */, kind, [ts.displayPart(name, kind2)]);
            }
            function createCompletionDetailsForSymbol(symbol, checker, sourceFile, location, cancellationToken, codeActions, sourceDisplay) {
                var _a = checker.runWithCancellationToken(cancellationToken, function (checker) {
                    return ts.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(checker, symbol, sourceFile, location, location, 7 /* All */);
                }), displayParts = _a.displayParts, documentation = _a.documentation, symbolKind = _a.symbolKind, tags = _a.tags;
                return createCompletionDetails(symbol.name, ts.SymbolDisplay.getSymbolModifiers(symbol), symbolKind, displayParts, documentation, tags, codeActions, sourceDisplay);
            }
            Completions.createCompletionDetailsForSymbol = createCompletionDetailsForSymbol;
            function createCompletionDetails(name, kindModifiers, kind, displayParts, documentation, tags, codeActions, source) {
                return { name: name, kindModifiers: kindModifiers, kind: kind, displayParts: displayParts, documentation: documentation, tags: tags, codeActions: codeActions, source: source };
            }
            Completions.createCompletionDetails = createCompletionDetails;
            function getCompletionEntryCodeActionsAndSourceDisplay(symbolToOriginInfoMap, symbol, program, checker, host, compilerOptions, sourceFile, position, previousToken, formatContext, preferences) {
                var symbolOriginInfo = symbolToOriginInfoMap[ts.getSymbolId(symbol)];
                if (!symbolOriginInfo || !originIsExport(symbolOriginInfo)) {
                    return { codeActions: undefined, sourceDisplay: undefined };
                }
                var moduleSymbol = symbolOriginInfo.moduleSymbol;
                var exportedSymbol = checker.getMergedSymbol(ts.skipAlias(symbol.exportSymbol || symbol, checker));
                var _a = ts.codefix.getImportCompletionAction(exportedSymbol, moduleSymbol, sourceFile, getSymbolName(symbol, symbolOriginInfo, compilerOptions.target), host, program, formatContext, previousToken && ts.isIdentifier(previousToken) ? previousToken.getStart(sourceFile) : position, preferences), moduleSpecifier = _a.moduleSpecifier, codeAction = _a.codeAction;
                return { sourceDisplay: [ts.textPart(moduleSpecifier)], codeActions: [codeAction] };
            }
            function getCompletionEntrySymbol(program, log, sourceFile, position, entryId, host) {
                var completion = getSymbolCompletionFromEntryId(program, log, sourceFile, position, entryId, host);
                return completion.type === "symbol" ? completion.symbol : undefined;
            }
            Completions.getCompletionEntrySymbol = getCompletionEntrySymbol;
            var CompletionDataKind;
            (function (CompletionDataKind) {
                CompletionDataKind[CompletionDataKind["Data"] = 0] = "Data";
                CompletionDataKind[CompletionDataKind["JsDocTagName"] = 1] = "JsDocTagName";
                CompletionDataKind[CompletionDataKind["JsDocTag"] = 2] = "JsDocTag";
                CompletionDataKind[CompletionDataKind["JsDocParameterName"] = 3] = "JsDocParameterName";
            })(CompletionDataKind || (CompletionDataKind = {}));
            var CompletionKind;
            (function (CompletionKind) {
                CompletionKind[CompletionKind["ObjectPropertyDeclaration"] = 0] = "ObjectPropertyDeclaration";
                CompletionKind[CompletionKind["Global"] = 1] = "Global";
                CompletionKind[CompletionKind["PropertyAccess"] = 2] = "PropertyAccess";
                CompletionKind[CompletionKind["MemberLike"] = 3] = "MemberLike";
                CompletionKind[CompletionKind["String"] = 4] = "String";
                CompletionKind[CompletionKind["None"] = 5] = "None";
            })(CompletionKind = Completions.CompletionKind || (Completions.CompletionKind = {}));
            function getRecommendedCompletion(previousToken, contextualType, checker) {
                // For a union, return the first one with a recommended completion.
                return ts.firstDefined(contextualType && (contextualType.isUnion() ? contextualType.types : [contextualType]), function (type) {
                    var symbol = type && type.symbol;
                    // Don't include make a recommended completion for an abstract class
                    return symbol && (symbol.flags & (8 /* EnumMember */ | 384 /* Enum */ | 32 /* Class */) && !ts.isAbstractConstructorSymbol(symbol))
                        ? getFirstSymbolInChain(symbol, previousToken, checker)
                        : undefined;
                });
            }
            function getContextualType(previousToken, position, sourceFile, checker) {
                var parent = previousToken.parent;
                switch (previousToken.kind) {
                    case 75 /* Identifier */:
                        return ts.getContextualTypeFromParent(previousToken, checker);
                    case 62 /* EqualsToken */:
                        switch (parent.kind) {
                            case 242 /* VariableDeclaration */:
                                return checker.getContextualType(parent.initializer); // TODO: GH#18217
                            case 209 /* BinaryExpression */:
                                return checker.getTypeAtLocation(parent.left);
                            case 273 /* JsxAttribute */:
                                return checker.getContextualTypeForJsxAttribute(parent);
                            default:
                                return undefined;
                        }
                    case 99 /* NewKeyword */:
                        return checker.getContextualType(parent);
                    case 78 /* CaseKeyword */:
                        return ts.getSwitchedType(ts.cast(parent, ts.isCaseClause), checker);
                    case 18 /* OpenBraceToken */:
                        return ts.isJsxExpression(parent) && parent.parent.kind !== 266 /* JsxElement */ ? checker.getContextualTypeForJsxAttribute(parent.parent) : undefined;
                    default:
                        var argInfo = ts.SignatureHelp.getArgumentInfoForCompletions(previousToken, position, sourceFile);
                        return argInfo ?
                            // At `,`, treat this as the next argument after the comma.
                            checker.getContextualTypeForArgumentAtIndex(argInfo.invocation, argInfo.argumentIndex + (previousToken.kind === 27 /* CommaToken */ ? 1 : 0)) :
                            ts.isEqualityOperatorKind(previousToken.kind) && ts.isBinaryExpression(parent) && ts.isEqualityOperatorKind(parent.operatorToken.kind) ?
                                // completion at `x ===/**/` should be for the right side
                                checker.getTypeAtLocation(parent.left) :
                                checker.getContextualType(previousToken);
                }
            }
            function getFirstSymbolInChain(symbol, enclosingDeclaration, checker) {
                var chain = checker.getAccessibleSymbolChain(symbol, enclosingDeclaration, /*meaning*/ 67108863 /* All */, /*useOnlyExternalAliasing*/ false);
                if (chain)
                    return ts.first(chain);
                return symbol.parent && (isModuleSymbol(symbol.parent) ? symbol : getFirstSymbolInChain(symbol.parent, enclosingDeclaration, checker));
            }
            function isModuleSymbol(symbol) {
                return symbol.declarations.some(function (d) { return d.kind === 290 /* SourceFile */; });
            }
            function getCompletionData(program, log, sourceFile, isUncheckedFile, position, preferences, detailsEntryId, host) {
                var typeChecker = program.getTypeChecker();
                var start = ts.timestamp();
                var currentToken = ts.getTokenAtPosition(sourceFile, position); // TODO: GH#15853
                // We will check for jsdoc comments with insideComment and getJsDocTagAtPosition. (TODO: that seems rather inefficient to check the same thing so many times.)
                log("getCompletionData: Get current token: " + (ts.timestamp() - start));
                start = ts.timestamp();
                var insideComment = ts.isInComment(sourceFile, position, currentToken);
                log("getCompletionData: Is inside comment: " + (ts.timestamp() - start));
                var insideJsDocTagTypeExpression = false;
                var isInSnippetScope = false;
                if (insideComment) {
                    if (ts.hasDocComment(sourceFile, position)) {
                        if (sourceFile.text.charCodeAt(position - 1) === 64 /* at */) {
                            // The current position is next to the '@' sign, when no tag name being provided yet.
                            // Provide a full list of tag names
                            return { kind: 1 /* JsDocTagName */ };
                        }
                        else {
                            // When completion is requested without "@", we will have check to make sure that
                            // there are no comments prefix the request position. We will only allow "*" and space.
                            // e.g
                            //   /** |c| /*
                            //
                            //   /**
                            //     |c|
                            //    */
                            //
                            //   /**
                            //    * |c|
                            //    */
                            //
                            //   /**
                            //    *         |c|
                            //    */
                            var lineStart = ts.getLineStartPositionForPosition(position, sourceFile);
                            if (!(sourceFile.text.substring(lineStart, position).match(/[^\*|\s|(/\*\*)]/))) {
                                return { kind: 2 /* JsDocTag */ };
                            }
                        }
                    }
                    // Completion should work inside certain JsDoc tags. For example:
                    //     /** @type {number | string} */
                    // Completion should work in the brackets
                    var tag = getJsDocTagAtPosition(currentToken, position);
                    if (tag) {
                        if (tag.tagName.pos <= position && position <= tag.tagName.end) {
                            return { kind: 1 /* JsDocTagName */ };
                        }
                        if (isTagWithTypeExpression(tag) && tag.typeExpression && tag.typeExpression.kind === 294 /* JSDocTypeExpression */) {
                            currentToken = ts.getTokenAtPosition(sourceFile, position);
                            if (!currentToken ||
                                (!ts.isDeclarationName(currentToken) &&
                                    (currentToken.parent.kind !== 322 /* JSDocPropertyTag */ ||
                                        currentToken.parent.name !== currentToken))) {
                                // Use as type location if inside tag's type expression
                                insideJsDocTagTypeExpression = isCurrentlyEditingNode(tag.typeExpression);
                            }
                        }
                        if (!insideJsDocTagTypeExpression && ts.isJSDocParameterTag(tag) && (ts.nodeIsMissing(tag.name) || tag.name.pos <= position && position <= tag.name.end)) {
                            return { kind: 3 /* JsDocParameterName */, tag: tag };
                        }
                    }
                    if (!insideJsDocTagTypeExpression) {
                        // Proceed if the current position is in jsDoc tag expression; otherwise it is a normal
                        // comment or the plain text part of a jsDoc comment, so no completion should be available
                        log("Returning an empty list because completion was inside a regular comment or plain text part of a JsDoc comment.");
                        return undefined;
                    }
                }
                start = ts.timestamp();
                var previousToken = ts.findPrecedingToken(position, sourceFile, /*startNode*/ undefined); // TODO: GH#18217
                log("getCompletionData: Get previous token 1: " + (ts.timestamp() - start));
                // The decision to provide completion depends on the contextToken, which is determined through the previousToken.
                // Note: 'previousToken' (and thus 'contextToken') can be undefined if we are the beginning of the file
                var contextToken = previousToken;
                // Check if the caret is at the end of an identifier; this is a partial identifier that we want to complete: e.g. a.toS|
                // Skip this partial identifier and adjust the contextToken to the token that precedes it.
                if (contextToken && position <= contextToken.end && (ts.isIdentifierOrPrivateIdentifier(contextToken) || ts.isKeyword(contextToken.kind))) {
                    var start_1 = ts.timestamp();
                    contextToken = ts.findPrecedingToken(contextToken.getFullStart(), sourceFile, /*startNode*/ undefined); // TODO: GH#18217
                    log("getCompletionData: Get previous token 2: " + (ts.timestamp() - start_1));
                }
                // Find the node where completion is requested on.
                // Also determine whether we are trying to complete with members of that node
                // or attributes of a JSX tag.
                var node = currentToken;
                var propertyAccessToConvert;
                var isRightOfDot = false;
                var isRightOfQuestionDot = false;
                var isRightOfOpenTag = false;
                var isStartingCloseTag = false;
                var isJsxInitializer = false;
                var location = ts.getTouchingPropertyName(sourceFile, position);
                if (contextToken) {
                    // Bail out if this is a known invalid completion location
                    if (isCompletionListBlocker(contextToken)) {
                        log("Returning an empty list because completion was requested in an invalid position.");
                        return undefined;
                    }
                    var parent = contextToken.parent;
                    if (contextToken.kind === 24 /* DotToken */ || contextToken.kind === 28 /* QuestionDotToken */) {
                        isRightOfDot = contextToken.kind === 24 /* DotToken */;
                        isRightOfQuestionDot = contextToken.kind === 28 /* QuestionDotToken */;
                        switch (parent.kind) {
                            case 194 /* PropertyAccessExpression */:
                                propertyAccessToConvert = parent;
                                node = propertyAccessToConvert.expression;
                                if (node.end === contextToken.pos &&
                                    ts.isCallExpression(node) &&
                                    node.getChildCount(sourceFile) &&
                                    ts.last(node.getChildren(sourceFile)).kind !== 21 /* CloseParenToken */) {
                                    // This is likely dot from incorrectly parsed call expression and user is starting to write spread
                                    // eg: Math.min(./**/)
                                    return undefined;
                                }
                                break;
                            case 153 /* QualifiedName */:
                                node = parent.left;
                                break;
                            case 249 /* ModuleDeclaration */:
                                node = parent.name;
                                break;
                            case 188 /* ImportType */:
                            case 219 /* MetaProperty */:
                                node = parent;
                                break;
                            default:
                                // There is nothing that precedes the dot, so this likely just a stray character
                                // or leading into a '...' token. Just bail out instead.
                                return undefined;
                        }
                    }
                    else if (sourceFile.languageVariant === 1 /* JSX */) {
                        // <UI.Test /* completion position */ />
                        // If the tagname is a property access expression, we will then walk up to the top most of property access expression.
                        // Then, try to get a JSX container and its associated attributes type.
                        if (parent && parent.kind === 194 /* PropertyAccessExpression */) {
                            contextToken = parent;
                            parent = parent.parent;
                        }
                        // Fix location
                        if (currentToken.parent === location) {
                            switch (currentToken.kind) {
                                case 31 /* GreaterThanToken */:
                                    if (currentToken.parent.kind === 266 /* JsxElement */ || currentToken.parent.kind === 268 /* JsxOpeningElement */) {
                                        location = currentToken;
                                    }
                                    break;
                                case 43 /* SlashToken */:
                                    if (currentToken.parent.kind === 267 /* JsxSelfClosingElement */) {
                                        location = currentToken;
                                    }
                                    break;
                            }
                        }
                        switch (parent.kind) {
                            case 269 /* JsxClosingElement */:
                                if (contextToken.kind === 43 /* SlashToken */) {
                                    isStartingCloseTag = true;
                                    location = contextToken;
                                }
                                break;
                            case 209 /* BinaryExpression */:
                                if (!binaryExpressionMayBeOpenTag(parent)) {
                                    break;
                                }
                            // falls through
                            case 267 /* JsxSelfClosingElement */:
                            case 266 /* JsxElement */:
                            case 268 /* JsxOpeningElement */:
                                if (contextToken.kind === 29 /* LessThanToken */) {
                                    isRightOfOpenTag = true;
                                    location = contextToken;
                                }
                                break;
                            case 273 /* JsxAttribute */:
                                switch (previousToken.kind) {
                                    case 62 /* EqualsToken */:
                                        isJsxInitializer = true;
                                        break;
                                    case 75 /* Identifier */:
                                        // For `<div x=[|f/**/|]`, `parent` will be `x` and `previousToken.parent` will be `f` (which is its own JsxAttribute)
                                        // Note for `<div someBool f>` we don't want to treat this as a jsx inializer, instead it's the attribute name.
                                        if (parent !== previousToken.parent &&
                                            !parent.initializer &&
                                            ts.findChildOfKind(parent, 62 /* EqualsToken */, sourceFile)) {
                                            isJsxInitializer = previousToken;
                                        }
                                }
                                break;
                        }
                    }
                }
                var semanticStart = ts.timestamp();
                var completionKind = 5 /* None */;
                var isNewIdentifierLocation = false;
                var keywordFilters = 0 /* None */;
                // This also gets mutated in nested-functions after the return
                var symbols = [];
                var symbolToOriginInfoMap = [];
                var symbolToSortTextMap = [];
                var importSuggestionsCache = host.getImportSuggestionsCache && host.getImportSuggestionsCache();
                var isTypeOnly = isTypeOnlyCompletion();
                if (isRightOfDot || isRightOfQuestionDot) {
                    getTypeScriptMemberSymbols();
                }
                else if (isRightOfOpenTag) {
                    var tagSymbols = ts.Debug.assertEachDefined(typeChecker.getJsxIntrinsicTagNamesAt(location), "getJsxIntrinsicTagNames() should all be defined");
                    tryGetGlobalSymbols();
                    symbols = tagSymbols.concat(symbols);
                    completionKind = 3 /* MemberLike */;
                    keywordFilters = 0 /* None */;
                }
                else if (isStartingCloseTag) {
                    var tagName = contextToken.parent.parent.openingElement.tagName;
                    var tagSymbol = typeChecker.getSymbolAtLocation(tagName);
                    if (tagSymbol) {
                        symbols = [tagSymbol];
                    }
                    completionKind = 3 /* MemberLike */;
                    keywordFilters = 0 /* None */;
                }
                else {
                    // For JavaScript or TypeScript, if we're not after a dot, then just try to get the
                    // global symbols in scope.  These results should be valid for either language as
                    // the set of symbols that can be referenced from this location.
                    if (!tryGetGlobalSymbols()) {
                        return undefined;
                    }
                }
                log("getCompletionData: Semantic work: " + (ts.timestamp() - semanticStart));
                var contextualType = previousToken && getContextualType(previousToken, position, sourceFile, typeChecker);
                var literals = ts.mapDefined(contextualType && (contextualType.isUnion() ? contextualType.types : [contextualType]), function (t) { return t.isLiteral() ? t.value : undefined; });
                var recommendedCompletion = previousToken && contextualType && getRecommendedCompletion(previousToken, contextualType, typeChecker);
                return {
                    kind: 0 /* Data */,
                    symbols: symbols,
                    completionKind: completionKind,
                    isInSnippetScope: isInSnippetScope,
                    propertyAccessToConvert: propertyAccessToConvert,
                    isNewIdentifierLocation: isNewIdentifierLocation,
                    location: location,
                    keywordFilters: keywordFilters,
                    literals: literals,
                    symbolToOriginInfoMap: symbolToOriginInfoMap,
                    recommendedCompletion: recommendedCompletion,
                    previousToken: previousToken,
                    isJsxInitializer: isJsxInitializer,
                    insideJsDocTagTypeExpression: insideJsDocTagTypeExpression,
                    symbolToSortTextMap: symbolToSortTextMap,
                    isTypeOnlyLocation: isTypeOnly
                };
                function isTagWithTypeExpression(tag) {
                    switch (tag.kind) {
                        case 316 /* JSDocParameterTag */:
                        case 322 /* JSDocPropertyTag */:
                        case 317 /* JSDocReturnTag */:
                        case 319 /* JSDocTypeTag */:
                        case 321 /* JSDocTypedefTag */:
                            return true;
                        default:
                            return false;
                    }
                }
                function getTypeScriptMemberSymbols() {
                    // Right of dot member completion list
                    completionKind = 2 /* PropertyAccess */;
                    // Since this is qualified name check it's a type node location
                    var isImportType = ts.isLiteralImportTypeNode(node);
                    var isTypeLocation = insideJsDocTagTypeExpression
                        || (isImportType && !node.isTypeOf)
                        || ts.isPartOfTypeNode(node.parent)
                        || ts.isPossiblyTypeArgumentPosition(contextToken, sourceFile, typeChecker);
                    var isRhsOfImportDeclaration = ts.isInRightSideOfInternalImportEqualsDeclaration(node);
                    if (ts.isEntityName(node) || isImportType) {
                        var isNamespaceName = ts.isModuleDeclaration(node.parent);
                        if (isNamespaceName)
                            isNewIdentifierLocation = true;
                        var symbol = typeChecker.getSymbolAtLocation(node);
                        if (symbol) {
                            symbol = ts.skipAlias(symbol, typeChecker);
                            if (symbol.flags & (1536 /* Module */ | 384 /* Enum */)) {
                                // Extract module or enum members
                                var exportedSymbols = ts.Debug.assertEachDefined(typeChecker.getExportsOfModule(symbol), "getExportsOfModule() should all be defined");
                                var isValidValueAccess_1 = function (symbol) { return typeChecker.isValidPropertyAccess(isImportType ? node : (node.parent), symbol.name); };
                                var isValidTypeAccess_1 = function (symbol) { return symbolCanBeReferencedAtTypeLocation(symbol); };
                                var isValidAccess = isNamespaceName
                                    // At `namespace N.M/**/`, if this is the only declaration of `M`, don't include `M` as a completion.
                                    ? function (symbol) { return !!(symbol.flags & 1920 /* Namespace */) && !symbol.declarations.every(function (d) { return d.parent === node.parent; }); }
                                    : isRhsOfImportDeclaration ?
                                        // Any kind is allowed when dotting off namespace in internal import equals declaration
                                        function (symbol) { return isValidTypeAccess_1(symbol) || isValidValueAccess_1(symbol); } :
                                        isTypeLocation ? isValidTypeAccess_1 : isValidValueAccess_1;
                                for (var _i = 0, exportedSymbols_1 = exportedSymbols; _i < exportedSymbols_1.length; _i++) {
                                    var exportedSymbol = exportedSymbols_1[_i];
                                    if (isValidAccess(exportedSymbol)) {
                                        symbols.push(exportedSymbol);
                                    }
                                }
                                // If the module is merged with a value, we must get the type of the class and add its propertes (for inherited static methods).
                                if (!isTypeLocation &&
                                    symbol.declarations &&
                                    symbol.declarations.some(function (d) { return d.kind !== 290 /* SourceFile */ && d.kind !== 249 /* ModuleDeclaration */ && d.kind !== 248 /* EnumDeclaration */; })) {
                                    var type = typeChecker.getTypeOfSymbolAtLocation(symbol, node).getNonOptionalType();
                                    var insertQuestionDot = false;
                                    if (type.isNullableType()) {
                                        var canCorrectToQuestionDot = isRightOfDot &&
                                            !isRightOfQuestionDot &&
                                            preferences.includeAutomaticOptionalChainCompletions !== false;
                                        if (canCorrectToQuestionDot || isRightOfQuestionDot) {
                                            type = type.getNonNullableType();
                                            if (canCorrectToQuestionDot) {
                                                insertQuestionDot = true;
                                            }
                                        }
                                    }
                                    addTypeProperties(type, !!(node.flags & 32768 /* AwaitContext */), insertQuestionDot);
                                }
                                return;
                            }
                        }
                    }
                    if (ts.isMetaProperty(node) && (node.keywordToken === 99 /* NewKeyword */ || node.keywordToken === 96 /* ImportKeyword */) && contextToken === node.getChildAt(1)) {
                        var completion = (node.keywordToken === 99 /* NewKeyword */) ? "target" : "meta";
                        symbols.push(typeChecker.createSymbol(4 /* Property */, ts.escapeLeadingUnderscores(completion)));
                        return;
                    }
                    if (!isTypeLocation) {
                        var type = typeChecker.getTypeAtLocation(node).getNonOptionalType();
                        var insertQuestionDot = false;
                        if (type.isNullableType()) {
                            var canCorrectToQuestionDot = isRightOfDot &&
                                !isRightOfQuestionDot &&
                                preferences.includeAutomaticOptionalChainCompletions !== false;
                            if (canCorrectToQuestionDot || isRightOfQuestionDot) {
                                type = type.getNonNullableType();
                                if (canCorrectToQuestionDot) {
                                    insertQuestionDot = true;
                                }
                            }
                        }
                        addTypeProperties(type, !!(node.flags & 32768 /* AwaitContext */), insertQuestionDot);
                    }
                }
                function addTypeProperties(type, insertAwait, insertQuestionDot) {
                    isNewIdentifierLocation = !!type.getStringIndexType();
                    if (isRightOfQuestionDot && ts.some(type.getCallSignatures())) {
                        isNewIdentifierLocation = true;
                    }
                    var propertyAccess = node.kind === 188 /* ImportType */ ? node : node.parent;
                    if (isUncheckedFile) {
                        // In javascript files, for union types, we don't just get the members that
                        // the individual types have in common, we also include all the members that
                        // each individual type has. This is because we're going to add all identifiers
                        // anyways. So we might as well elevate the members that were at least part
                        // of the individual types to a higher status since we know what they are.
                        symbols.push.apply(symbols, getPropertiesForCompletion(type, typeChecker));
                    }
                    else {
                        for (var _i = 0, _a = type.getApparentProperties(); _i < _a.length; _i++) {
                            var symbol = _a[_i];
                            if (typeChecker.isValidPropertyAccessForCompletions(propertyAccess, type, symbol)) {
                                addPropertySymbol(symbol, /*insertAwait*/ false, insertQuestionDot);
                            }
                        }
                    }
                    if (insertAwait && preferences.includeCompletionsWithInsertText) {
                        var promiseType = typeChecker.getPromisedTypeOfPromise(type);
                        if (promiseType) {
                            for (var _b = 0, _c = promiseType.getApparentProperties(); _b < _c.length; _b++) {
                                var symbol = _c[_b];
                                if (typeChecker.isValidPropertyAccessForCompletions(propertyAccess, promiseType, symbol)) {
                                    addPropertySymbol(symbol, /* insertAwait */ true, insertQuestionDot);
                                }
                            }
                        }
                    }
                }
                function addPropertySymbol(symbol, insertAwait, insertQuestionDot) {
                    // For a computed property with an accessible name like `Symbol.iterator`,
                    // we'll add a completion for the *name* `Symbol` instead of for the property.
                    // If this is e.g. [Symbol.iterator], add a completion for `Symbol`.
                    var computedPropertyName = ts.firstDefined(symbol.declarations, function (decl) { return ts.tryCast(ts.getNameOfDeclaration(decl), ts.isComputedPropertyName); });
                    if (computedPropertyName) {
                        var leftMostName = getLeftMostName(computedPropertyName.expression); // The completion is for `Symbol`, not `iterator`.
                        var nameSymbol = leftMostName && typeChecker.getSymbolAtLocation(leftMostName);
                        // If this is nested like for `namespace N { export const sym = Symbol(); }`, we'll add the completion for `N`.
                        var firstAccessibleSymbol = nameSymbol && getFirstSymbolInChain(nameSymbol, contextToken, typeChecker);
                        if (firstAccessibleSymbol && !symbolToOriginInfoMap[ts.getSymbolId(firstAccessibleSymbol)]) {
                            symbols.push(firstAccessibleSymbol);
                            var moduleSymbol = firstAccessibleSymbol.parent;
                            symbolToOriginInfoMap[ts.getSymbolId(firstAccessibleSymbol)] =
                                !moduleSymbol || !ts.isExternalModuleSymbol(moduleSymbol)
                                    ? { kind: getNullableSymbolOriginInfoKind(2 /* SymbolMemberNoExport */) }
                                    : { kind: getNullableSymbolOriginInfoKind(6 /* SymbolMemberExport */), moduleSymbol: moduleSymbol, isDefaultExport: false };
                        }
                        else if (preferences.includeCompletionsWithInsertText) {
                            addSymbolOriginInfo(symbol);
                            symbols.push(symbol);
                        }
                    }
                    else {
                        addSymbolOriginInfo(symbol);
                        symbols.push(symbol);
                    }
                    function addSymbolOriginInfo(symbol) {
                        if (preferences.includeCompletionsWithInsertText) {
                            if (insertAwait && !symbolToOriginInfoMap[ts.getSymbolId(symbol)]) {
                                symbolToOriginInfoMap[ts.getSymbolId(symbol)] = { kind: getNullableSymbolOriginInfoKind(8 /* Promise */) };
                            }
                            else if (insertQuestionDot) {
                                symbolToOriginInfoMap[ts.getSymbolId(symbol)] = { kind: 16 /* Nullable */ };
                            }
                        }
                    }
                    function getNullableSymbolOriginInfoKind(kind) {
                        return insertQuestionDot ? kind | 16 /* Nullable */ : kind;
                    }
                }
                /** Given 'a.b.c', returns 'a'. */
                function getLeftMostName(e) {
                    return ts.isIdentifier(e) ? e : ts.isPropertyAccessExpression(e) ? getLeftMostName(e.expression) : undefined;
                }
                function tryGetGlobalSymbols() {
                    var result = tryGetObjectLikeCompletionSymbols()
                        || tryGetImportOrExportClauseCompletionSymbols()
                        || tryGetConstructorCompletion()
                        || tryGetClassLikeCompletionSymbols()
                        || tryGetJsxCompletionSymbols()
                        || (getGlobalCompletions(), 1 /* Success */);
                    return result === 1 /* Success */;
                }
                function tryGetConstructorCompletion() {
                    if (!tryGetConstructorLikeCompletionContainer(contextToken))
                        return 0 /* Continue */;
                    // no members, only keywords
                    completionKind = 5 /* None */;
                    // Declaring new property/method/accessor
                    isNewIdentifierLocation = true;
                    // Has keywords for constructor parameter
                    keywordFilters = 4 /* ConstructorParameterKeywords */;
                    return 1 /* Success */;
                }
                function tryGetJsxCompletionSymbols() {
                    var jsxContainer = tryGetContainingJsxElement(contextToken);
                    // Cursor is inside a JSX self-closing element or opening element
                    var attrsType = jsxContainer && typeChecker.getContextualType(jsxContainer.attributes);
                    if (!attrsType)
                        return 0 /* Continue */;
                    var completionsType = jsxContainer && typeChecker.getContextualType(jsxContainer.attributes, 4 /* Completions */);
                    symbols = filterJsxAttributes(getPropertiesForObjectExpression(attrsType, completionsType, jsxContainer.attributes, typeChecker), jsxContainer.attributes.properties);
                    setSortTextToOptionalMember();
                    completionKind = 3 /* MemberLike */;
                    isNewIdentifierLocation = false;
                    return 1 /* Success */;
                }
                function getGlobalCompletions() {
                    keywordFilters = tryGetFunctionLikeBodyCompletionContainer(contextToken) ? 5 /* FunctionLikeBodyKeywords */ : 1 /* All */;
                    // Get all entities in the current scope.
                    completionKind = 1 /* Global */;
                    isNewIdentifierLocation = isNewIdentifierDefinitionLocation(contextToken);
                    if (previousToken !== contextToken) {
                        ts.Debug.assert(!!previousToken, "Expected 'contextToken' to be defined when different from 'previousToken'.");
                    }
                    // We need to find the node that will give us an appropriate scope to begin
                    // aggregating completion candidates. This is achieved in 'getScopeNode'
                    // by finding the first node that encompasses a position, accounting for whether a node
                    // is "complete" to decide whether a position belongs to the node.
                    //
                    // However, at the end of an identifier, we are interested in the scope of the identifier
                    // itself, but fall outside of the identifier. For instance:
                    //
                    //      xyz => x$
                    //
                    // the cursor is outside of both the 'x' and the arrow function 'xyz => x',
                    // so 'xyz' is not returned in our results.
                    //
                    // We define 'adjustedPosition' so that we may appropriately account for
                    // being at the end of an identifier. The intention is that if requesting completion
                    // at the end of an identifier, it should be effectively equivalent to requesting completion
                    // anywhere inside/at the beginning of the identifier. So in the previous case, the
                    // 'adjustedPosition' will work as if requesting completion in the following:
                    //
                    //      xyz => $x
                    //
                    // If previousToken !== contextToken, then
                    //   - 'contextToken' was adjusted to the token prior to 'previousToken'
                    //      because we were at the end of an identifier.
                    //   - 'previousToken' is defined.
                    var adjustedPosition = previousToken !== contextToken ?
                        previousToken.getStart() :
                        position;
                    var scopeNode = getScopeNode(contextToken, adjustedPosition, sourceFile) || sourceFile;
                    isInSnippetScope = isSnippetScope(scopeNode);
                    var symbolMeanings = (isTypeOnly ? 0 /* None */ : 111551 /* Value */) | 788968 /* Type */ | 1920 /* Namespace */ | 2097152 /* Alias */;
                    symbols = ts.Debug.assertEachDefined(typeChecker.getSymbolsInScope(scopeNode, symbolMeanings), "getSymbolsInScope() should all be defined");
                    for (var _i = 0, symbols_2 = symbols; _i < symbols_2.length; _i++) {
                        var symbol = symbols_2[_i];
                        if (!typeChecker.isArgumentsSymbol(symbol) &&
                            !ts.some(symbol.declarations, function (d) { return d.getSourceFile() === sourceFile; })) {
                            symbolToSortTextMap[ts.getSymbolId(symbol)] = SortText.GlobalsOrKeywords;
                        }
                    }
                    // Need to insert 'this.' before properties of `this` type, so only do that if `includeInsertTextCompletions`
                    if (preferences.includeCompletionsWithInsertText && scopeNode.kind !== 290 /* SourceFile */) {
                        var thisType = typeChecker.tryGetThisTypeAt(scopeNode, /*includeGlobalThis*/ false);
                        if (thisType) {
                            for (var _a = 0, _b = getPropertiesForCompletion(thisType, typeChecker); _a < _b.length; _a++) {
                                var symbol = _b[_a];
                                symbolToOriginInfoMap[ts.getSymbolId(symbol)] = { kind: 1 /* ThisType */ };
                                symbols.push(symbol);
                                symbolToSortTextMap[ts.getSymbolId(symbol)] = SortText.SuggestedClassMembers;
                            }
                        }
                    }
                    if (shouldOfferImportCompletions()) {
                        var lowerCaseTokenText_1 = previousToken && ts.isIdentifier(previousToken) ? previousToken.text.toLowerCase() : "";
                        var autoImportSuggestions = getSymbolsFromOtherSourceFileExports(program.getCompilerOptions().target, host);
                        if (!detailsEntryId && importSuggestionsCache) {
                            importSuggestionsCache.set(sourceFile.fileName, autoImportSuggestions, host.getProjectVersion && host.getProjectVersion());
                        }
                        autoImportSuggestions.forEach(function (_a) {
                            var symbol = _a.symbol, symbolName = _a.symbolName, skipFilter = _a.skipFilter, origin = _a.origin;
                            if (detailsEntryId) {
                                if (detailsEntryId.source && ts.stripQuotes(origin.moduleSymbol.name) !== detailsEntryId.source) {
                                    return;
                                }
                            }
                            else if (!skipFilter && !stringContainsCharactersInOrder(symbolName.toLowerCase(), lowerCaseTokenText_1)) {
                                return;
                            }
                            var symbolId = ts.getSymbolId(symbol);
                            symbols.push(symbol);
                            symbolToOriginInfoMap[symbolId] = origin;
                            symbolToSortTextMap[symbolId] = SortText.AutoImportSuggestions;
                        });
                    }
                    filterGlobalCompletion(symbols);
                }
                function shouldOfferImportCompletions() {
                    // If not already a module, must have modules enabled and not currently be in a commonjs module. (TODO: import completions for commonjs)
                    if (!preferences.includeCompletionsForModuleExports)
                        return false;
                    // If already using ES6 modules, OK to continue using them.
                    if (sourceFile.externalModuleIndicator)
                        return true;
                    // If already using commonjs, don't introduce ES6.
                    if (sourceFile.commonJsModuleIndicator)
                        return false;
                    // If module transpilation is enabled or we're targeting es6 or above, or not emitting, OK.
                    if (ts.compilerOptionsIndicateEs6Modules(program.getCompilerOptions()))
                        return true;
                    // If some file is using ES6 modules, assume that it's OK to add more.
                    return ts.programContainsEs6Modules(program);
                }
                function isSnippetScope(scopeNode) {
                    switch (scopeNode.kind) {
                        case 290 /* SourceFile */:
                        case 211 /* TemplateExpression */:
                        case 276 /* JsxExpression */:
                        case 223 /* Block */:
                            return true;
                        default:
                            return ts.isStatement(scopeNode);
                    }
                }
                function filterGlobalCompletion(symbols) {
                    var isTypeOnly = isTypeOnlyCompletion();
                    if (isTypeOnly) {
                        keywordFilters = isTypeAssertion()
                            ? 6 /* TypeAssertionKeywords */
                            : 7 /* TypeKeywords */;
                    }
                    ts.filterMutate(symbols, function (symbol) {
                        if (!ts.isSourceFile(location)) {
                            // export = /**/ here we want to get all meanings, so any symbol is ok
                            if (ts.isExportAssignment(location.parent)) {
                                return true;
                            }
                            symbol = ts.skipAlias(symbol, typeChecker);
                            // import m = /**/ <-- It can only access namespace (if typing import = x. this would get member symbols and not namespace)
                            if (ts.isInRightSideOfInternalImportEqualsDeclaration(location)) {
                                return !!(symbol.flags & 1920 /* Namespace */);
                            }
                            if (isTypeOnly) {
                                // It's a type, but you can reach it by namespace.type as well
                                return symbolCanBeReferencedAtTypeLocation(symbol);
                            }
                        }
                        // expressions are value space (which includes the value namespaces)
                        return !!(ts.getCombinedLocalAndExportSymbolFlags(symbol) & 111551 /* Value */);
                    });
                }
                function isTypeAssertion() {
                    return ts.isAssertionExpression(contextToken.parent);
                }
                function isTypeOnlyCompletion() {
                    return insideJsDocTagTypeExpression
                        || !isContextTokenValueLocation(contextToken) &&
                            (ts.isPossiblyTypeArgumentPosition(contextToken, sourceFile, typeChecker)
                                || ts.isPartOfTypeNode(location)
                                || isContextTokenTypeLocation(contextToken));
                }
                function isContextTokenValueLocation(contextToken) {
                    return contextToken &&
                        contextToken.kind === 108 /* TypeOfKeyword */ &&
                        (contextToken.parent.kind === 172 /* TypeQuery */ || ts.isTypeOfExpression(contextToken.parent));
                }
                function isContextTokenTypeLocation(contextToken) {
                    if (contextToken) {
                        var parentKind = contextToken.parent.kind;
                        switch (contextToken.kind) {
                            case 58 /* ColonToken */:
                                return parentKind === 159 /* PropertyDeclaration */ ||
                                    parentKind === 158 /* PropertySignature */ ||
                                    parentKind === 156 /* Parameter */ ||
                                    parentKind === 242 /* VariableDeclaration */ ||
                                    ts.isFunctionLikeKind(parentKind);
                            case 62 /* EqualsToken */:
                                return parentKind === 247 /* TypeAliasDeclaration */;
                            case 123 /* AsKeyword */:
                                return parentKind === 217 /* AsExpression */;
                            case 29 /* LessThanToken */:
                                return parentKind === 169 /* TypeReference */ ||
                                    parentKind === 199 /* TypeAssertionExpression */;
                            case 90 /* ExtendsKeyword */:
                                return parentKind === 155 /* TypeParameter */;
                        }
                    }
                    return false;
                }
                /** True if symbol is a type or a module containing at least one type. */
                function symbolCanBeReferencedAtTypeLocation(symbol, seenModules) {
                    if (seenModules === void 0) {
                        seenModules = ts.createMap();
                    }
                    var sym = ts.skipAlias(symbol.exportSymbol || symbol, typeChecker);
                    return !!(sym.flags & 788968 /* Type */) ||
                        !!(sym.flags & 1536 /* Module */) &&
                            ts.addToSeen(seenModules, ts.getSymbolId(sym)) &&
                            typeChecker.getExportsOfModule(sym).some(function (e) { return symbolCanBeReferencedAtTypeLocation(e, seenModules); });
                }
                /**
                 * Gathers symbols that can be imported from other files, de-duplicating along the way. Symbols can be "duplicates"
                 * if re-exported from another module, e.g. `export { foo } from "./a"`. That syntax creates a fresh symbol, but
                 * it’s just an alias to the first, and both have the same name, so we generally want to filter those aliases out,
                 * if and only if the the first can be imported (it may be excluded due to package.json filtering in
                 * `codefix.forEachExternalModuleToImportFrom`).
                 *
                 * Example. Imagine a chain of node_modules re-exporting one original symbol:
                 *
                 * ```js
                 *  node_modules/x/index.js         node_modules/y/index.js           node_modules/z/index.js
                 * +-----------------------+      +--------------------------+      +--------------------------+
                 * |                       |      |                          |      |                          |
                 * | export const foo = 0; | <--- | export { foo } from 'x'; | <--- | export { foo } from 'y'; |
                 * |                       |      |                          |      |                          |
                 * +-----------------------+      +--------------------------+      +--------------------------+
                 * ```
                 *
                 * Also imagine three buckets, which we’ll reference soon:
                 *
                 * ```md
                 * |                  |      |                      |      |                   |
                 * |   **Bucket A**   |      |    **Bucket B**      |      |    **Bucket C**   |
                 * |    Symbols to    |      | Aliases to symbols   |      | Symbols to return |
                 * |    definitely    |      | in Buckets A or C    |      | if nothing better |
                 * |      return      |      | (don’t return these) |      |    comes along    |
                 * |__________________|      |______________________|      |___________________|
                 * ```
                 *
                 * We _probably_ want to show `foo` from 'x', but not from 'y' or 'z'. However, if 'x' is not in a package.json, it
                 * will not appear in a `forEachExternalModuleToImportFrom` iteration. Furthermore, the order of iterations is not
                 * guaranteed, as it is host-dependent. Therefore, when presented with the symbol `foo` from module 'y' alone, we
                 * may not be sure whether or not it should go in the list. So, we’ll take the following steps:
                 *
                 * 1. Resolve alias `foo` from 'y' to the export declaration in 'x', get the symbol there, and see if that symbol is
                 *    already in Bucket A (symbols we already know will be returned). If it is, put `foo` from 'y' in Bucket B
                 *    (symbols that are aliases to symbols in Bucket A). If it’s not, put it in Bucket C.
                 * 2. Next, imagine we see `foo` from module 'z'. Again, we resolve the alias to the nearest export, which is in 'y'.
                 *    At this point, if that nearest export from 'y' is in _any_ of the three buckets, we know the symbol in 'z'
                 *    should never be returned in the final list, so put it in Bucket B.
                 * 3. Next, imagine we see `foo` from module 'x', the original. Syntactically, it doesn’t look like a re-export, so
                 *    we can just check Bucket C to see if we put any aliases to the original in there. If they exist, throw them out.
                 *    Put this symbol in Bucket A.
                 * 4. After we’ve iterated through every symbol of every module, any symbol left in Bucket C means that step 3 didn’t
                 *    occur for that symbol---that is, the original symbol is not in Bucket A, so we should include the alias. Move
                 *    everything from Bucket C to Bucket A.
                 */
                function getSymbolsFromOtherSourceFileExports(target, host) {
                    var cached = importSuggestionsCache && importSuggestionsCache.get(sourceFile.fileName, typeChecker, detailsEntryId && host.getProjectVersion ? host.getProjectVersion() : undefined);
                    if (cached) {
                        log("getSymbolsFromOtherSourceFileExports: Using cached list");
                        return cached;
                    }
                    var startTime = ts.timestamp();
                    log("getSymbolsFromOtherSourceFileExports: Recomputing list" + (detailsEntryId ? " for details entry" : ""));
                    var seenResolvedModules = ts.createMap();
                    /** Bucket B */
                    var aliasesToAlreadyIncludedSymbols = ts.createMap();
                    /** Bucket C */
                    var aliasesToReturnIfOriginalsAreMissing = ts.createMap();
                    /** Bucket A */
                    var results = [];
                    /** Ids present in `results` for faster lookup */
                    var resultSymbolIds = ts.createMap();
                    ts.codefix.forEachExternalModuleToImportFrom(program, host, sourceFile, !detailsEntryId, function (moduleSymbol) {
                        // Perf -- ignore other modules if this is a request for details
                        if (detailsEntryId && detailsEntryId.source && ts.stripQuotes(moduleSymbol.name) !== detailsEntryId.source) {
                            return;
                        }
                        var resolvedModuleSymbol = typeChecker.resolveExternalModuleSymbol(moduleSymbol);
                        // resolvedModuleSymbol may be a namespace. A namespace may be `export =` by multiple module declarations, but only keep the first one.
                        if (!ts.addToSeen(seenResolvedModules, ts.getSymbolId(resolvedModuleSymbol))) {
                            return;
                        }
                        // Don't add another completion for `export =` of a symbol that's already global.
                        // So in `declare namespace foo {} declare module "foo" { export = foo; }`, there will just be the global completion for `foo`.
                        if (resolvedModuleSymbol !== moduleSymbol &&
                            ts.every(resolvedModuleSymbol.declarations, function (d) { return !!d.getSourceFile().externalModuleIndicator && !ts.findAncestor(d, ts.isGlobalScopeAugmentation); })) {
                            pushSymbol(resolvedModuleSymbol, moduleSymbol, /*skipFilter*/ true);
                        }
                        for (var _i = 0, _a = typeChecker.getExportsOfModule(moduleSymbol); _i < _a.length; _i++) {
                            var symbol = _a[_i];
                            // If this is `export { _break as break };` (a keyword) -- skip this and prefer the keyword completion.
                            if (ts.some(symbol.declarations, function (d) { return ts.isExportSpecifier(d) && !!d.propertyName && ts.isIdentifierANonContextualKeyword(d.name); })) {
                                continue;
                            }
                            var symbolId = ts.getSymbolId(symbol).toString();
                            // If `symbol.parent !== moduleSymbol`, this is an `export * from "foo"` re-export. Those don't create new symbols.
                            var isExportStarFromReExport = typeChecker.getMergedSymbol(symbol.parent) !== resolvedModuleSymbol;
                            // If `!!d.parent.parent.moduleSpecifier`, this is `export { foo } from "foo"` re-export, which creates a new symbol (thus isn't caught by the first check).
                            if (isExportStarFromReExport || ts.some(symbol.declarations, function (d) { return ts.isExportSpecifier(d) && !d.propertyName && !!d.parent.parent.moduleSpecifier; })) {
                                // Walk the export chain back one module (step 1 or 2 in diagrammed example).
                                // Or, in the case of `export * from "foo"`, `symbol` already points to the original export, so just use that.
                                var nearestExportSymbol = isExportStarFromReExport ? symbol : getNearestExportSymbol(symbol);
                                if (!nearestExportSymbol)
                                    continue;
                                var nearestExportSymbolId = ts.getSymbolId(nearestExportSymbol).toString();
                                var symbolHasBeenSeen = resultSymbolIds.has(nearestExportSymbolId) || aliasesToAlreadyIncludedSymbols.has(nearestExportSymbolId);
                                if (!symbolHasBeenSeen) {
                                    aliasesToReturnIfOriginalsAreMissing.set(nearestExportSymbolId, { alias: symbol, moduleSymbol: moduleSymbol });
                                    aliasesToAlreadyIncludedSymbols.set(symbolId, true);
                                }
                                else {
                                    // Perf - we know this symbol is an alias to one that’s already covered in `symbols`, so store it here
                                    // in case another symbol re-exports this one; that way we can short-circuit as soon as we see this symbol id.
                                    ts.addToSeen(aliasesToAlreadyIncludedSymbols, symbolId);
                                }
                            }
                            else {
                                // This is not a re-export, so see if we have any aliases pending and remove them (step 3 in diagrammed example)
                                aliasesToReturnIfOriginalsAreMissing.delete(symbolId);
                                pushSymbol(symbol, moduleSymbol);
                            }
                        }
                    });
                    // By this point, any potential duplicates that were actually duplicates have been
                    // removed, so the rest need to be added. (Step 4 in diagrammed example)
                    aliasesToReturnIfOriginalsAreMissing.forEach(function (_a) {
                        var alias = _a.alias, moduleSymbol = _a.moduleSymbol;
                        return pushSymbol(alias, moduleSymbol);
                    });
                    log("getSymbolsFromOtherSourceFileExports: " + (ts.timestamp() - startTime));
                    return results;
                    function pushSymbol(symbol, moduleSymbol, skipFilter) {
                        if (skipFilter === void 0) {
                            skipFilter = false;
                        }
                        var isDefaultExport = symbol.escapedName === "default" /* Default */;
                        if (isDefaultExport) {
                            symbol = ts.getLocalSymbolForExportDefault(symbol) || symbol;
                        }
                        if (typeChecker.isUndefinedSymbol(symbol)) {
                            return;
                        }
                        ts.addToSeen(resultSymbolIds, ts.getSymbolId(symbol));
                        var origin = { kind: 4 /* Export */, moduleSymbol: moduleSymbol, isDefaultExport: isDefaultExport };
                        results.push({
                            symbol: symbol,
                            symbolName: getSymbolName(symbol, origin, target),
                            origin: origin,
                            skipFilter: skipFilter,
                        });
                    }
                }
                function getNearestExportSymbol(fromSymbol) {
                    return findAlias(typeChecker, fromSymbol, function (alias) {
                        return ts.some(alias.declarations, function (d) { return ts.isExportSpecifier(d) || !!d.localSymbol; });
                    });
                }
                /**
                 * True if you could remove some characters in `a` to get `b`.
                 * E.g., true for "abcdef" and "bdf".
                 * But not true for "abcdef" and "dbf".
                 */
                function stringContainsCharactersInOrder(str, characters) {
                    if (characters.length === 0) {
                        return true;
                    }
                    var characterIndex = 0;
                    for (var strIndex = 0; strIndex < str.length; strIndex++) {
                        if (str.charCodeAt(strIndex) === characters.charCodeAt(characterIndex)) {
                            characterIndex++;
                            if (characterIndex === characters.length) {
                                return true;
                            }
                        }
                    }
                    // Did not find all characters
                    return false;
                }
                /**
                 * Finds the first node that "embraces" the position, so that one may
                 * accurately aggregate locals from the closest containing scope.
                 */
                function getScopeNode(initialToken, position, sourceFile) {
                    var scope = initialToken;
                    while (scope && !ts.positionBelongsToNode(scope, position, sourceFile)) {
                        scope = scope.parent;
                    }
                    return scope;
                }
                function isCompletionListBlocker(contextToken) {
                    var start = ts.timestamp();
                    var result = isInStringOrRegularExpressionOrTemplateLiteral(contextToken) ||
                        isSolelyIdentifierDefinitionLocation(contextToken) ||
                        isDotOfNumericLiteral(contextToken) ||
                        isInJsxText(contextToken);
                    log("getCompletionsAtPosition: isCompletionListBlocker: " + (ts.timestamp() - start));
                    return result;
                }
                function isInJsxText(contextToken) {
                    if (contextToken.kind === 11 /* JsxText */) {
                        return true;
                    }
                    if (contextToken.kind === 31 /* GreaterThanToken */ && contextToken.parent) {
                        if (contextToken.parent.kind === 268 /* JsxOpeningElement */) {
                            // Two possibilities:
                            //   1. <div>/**/
                            //      - contextToken: GreaterThanToken (before cursor)
                            //      - location: JSXElement
                            //      - different parents (JSXOpeningElement, JSXElement)
                            //   2. <Component<string> /**/>
                            //      - contextToken: GreaterThanToken (before cursor)
                            //      - location: GreaterThanToken (after cursor)
                            //      - same parent (JSXOpeningElement)
                            return location.parent.kind !== 268 /* JsxOpeningElement */;
                        }
                        if (contextToken.parent.kind === 269 /* JsxClosingElement */ || contextToken.parent.kind === 267 /* JsxSelfClosingElement */) {
                            return !!contextToken.parent.parent && contextToken.parent.parent.kind === 266 /* JsxElement */;
                        }
                    }
                    return false;
                }
                function isNewIdentifierDefinitionLocation(previousToken) {
                    if (previousToken) {
                        var containingNodeKind = previousToken.parent.kind;
                        // Previous token may have been a keyword that was converted to an identifier.
                        switch (keywordForNode(previousToken)) {
                            case 27 /* CommaToken */:
                                return containingNodeKind === 196 /* CallExpression */ // func( a, |
                                    || containingNodeKind === 162 /* Constructor */ // constructor( a, |   /* public, protected, private keywords are allowed here, so show completion */
                                    || containingNodeKind === 197 /* NewExpression */ // new C(a, |
                                    || containingNodeKind === 192 /* ArrayLiteralExpression */ // [a, |
                                    || containingNodeKind === 209 /* BinaryExpression */ // const x = (a, |
                                    || containingNodeKind === 170 /* FunctionType */; // var x: (s: string, list|
                            case 20 /* OpenParenToken */:
                                return containingNodeKind === 196 /* CallExpression */ // func( |
                                    || containingNodeKind === 162 /* Constructor */ // constructor( |
                                    || containingNodeKind === 197 /* NewExpression */ // new C(a|
                                    || containingNodeKind === 200 /* ParenthesizedExpression */ // const x = (a|
                                    || containingNodeKind === 182 /* ParenthesizedType */; // function F(pred: (a| /* this can become an arrow function, where 'a' is the argument */
                            case 22 /* OpenBracketToken */:
                                return containingNodeKind === 192 /* ArrayLiteralExpression */ // [ |
                                    || containingNodeKind === 167 /* IndexSignature */ // [ | : string ]
                                    || containingNodeKind === 154 /* ComputedPropertyName */; // [ |    /* this can become an index signature */
                            case 135 /* ModuleKeyword */: // module |
                            case 136 /* NamespaceKeyword */: // namespace |
                                return true;
                            case 24 /* DotToken */:
                                return containingNodeKind === 249 /* ModuleDeclaration */; // module A.|
                            case 18 /* OpenBraceToken */:
                                return containingNodeKind === 245 /* ClassDeclaration */; // class A{ |
                            case 62 /* EqualsToken */:
                                return containingNodeKind === 242 /* VariableDeclaration */ // const x = a|
                                    || containingNodeKind === 209 /* BinaryExpression */; // x = a|
                            case 15 /* TemplateHead */:
                                return containingNodeKind === 211 /* TemplateExpression */; // `aa ${|
                            case 16 /* TemplateMiddle */:
                                return containingNodeKind === 221 /* TemplateSpan */; // `aa ${10} dd ${|
                            case 119 /* PublicKeyword */:
                            case 117 /* PrivateKeyword */:
                            case 118 /* ProtectedKeyword */:
                                return containingNodeKind === 159 /* PropertyDeclaration */; // class A{ public |
                        }
                    }
                    return false;
                }
                function isInStringOrRegularExpressionOrTemplateLiteral(contextToken) {
                    // To be "in" one of these literals, the position has to be:
                    //   1. entirely within the token text.
                    //   2. at the end position of an unterminated token.
                    //   3. at the end of a regular expression (due to trailing flags like '/foo/g').
                    return (ts.isRegularExpressionLiteral(contextToken) || ts.isStringTextContainingNode(contextToken)) && (ts.rangeContainsPositionExclusive(ts.createTextRangeFromSpan(ts.createTextSpanFromNode(contextToken)), position) ||
                        position === contextToken.end && (!!contextToken.isUnterminated || ts.isRegularExpressionLiteral(contextToken)));
                }
                /**
                 * Aggregates relevant symbols for completion in object literals and object binding patterns.
                 * Relevant symbols are stored in the captured 'symbols' variable.
                 *
                 * @returns true if 'symbols' was successfully populated; false otherwise.
                 */
                function tryGetObjectLikeCompletionSymbols() {
                    var objectLikeContainer = tryGetObjectLikeCompletionContainer(contextToken);
                    if (!objectLikeContainer)
                        return 0 /* Continue */;
                    // We're looking up possible property names from contextual/inferred/declared type.
                    completionKind = 0 /* ObjectPropertyDeclaration */;
                    var typeMembers;
                    var existingMembers;
                    if (objectLikeContainer.kind === 193 /* ObjectLiteralExpression */) {
                        var instantiatedType = typeChecker.getContextualType(objectLikeContainer);
                        var completionsType = instantiatedType && typeChecker.getContextualType(objectLikeContainer, 4 /* Completions */);
                        if (!instantiatedType || !completionsType)
                            return 2 /* Fail */;
                        isNewIdentifierLocation = ts.hasIndexSignature(instantiatedType || completionsType);
                        typeMembers = getPropertiesForObjectExpression(instantiatedType, completionsType, objectLikeContainer, typeChecker);
                        existingMembers = objectLikeContainer.properties;
                    }
                    else {
                        ts.Debug.assert(objectLikeContainer.kind === 189 /* ObjectBindingPattern */);
                        // We are *only* completing on properties from the type being destructured.
                        isNewIdentifierLocation = false;
                        var rootDeclaration = ts.getRootDeclaration(objectLikeContainer.parent);
                        if (!ts.isVariableLike(rootDeclaration))
                            return ts.Debug.fail("Root declaration is not variable-like.");
                        // We don't want to complete using the type acquired by the shape
                        // of the binding pattern; we are only interested in types acquired
                        // through type declaration or inference.
                        // Also proceed if rootDeclaration is a parameter and if its containing function expression/arrow function is contextually typed -
                        // type of parameter will flow in from the contextual type of the function
                        var canGetType = ts.hasInitializer(rootDeclaration) || ts.hasType(rootDeclaration) || rootDeclaration.parent.parent.kind === 232 /* ForOfStatement */;
                        if (!canGetType && rootDeclaration.kind === 156 /* Parameter */) {
                            if (ts.isExpression(rootDeclaration.parent)) {
                                canGetType = !!typeChecker.getContextualType(rootDeclaration.parent);
                            }
                            else if (rootDeclaration.parent.kind === 161 /* MethodDeclaration */ || rootDeclaration.parent.kind === 164 /* SetAccessor */) {
                                canGetType = ts.isExpression(rootDeclaration.parent.parent) && !!typeChecker.getContextualType(rootDeclaration.parent.parent);
                            }
                        }
                        if (canGetType) {
                            var typeForObject_1 = typeChecker.getTypeAtLocation(objectLikeContainer);
                            if (!typeForObject_1)
                                return 2 /* Fail */;
                            // In a binding pattern, get only known properties (unless in the same scope).
                            // Everywhere else we will get all possible properties.
                            var containerClass_1 = ts.getContainingClass(objectLikeContainer);
                            typeMembers = typeChecker.getPropertiesOfType(typeForObject_1).filter(function (symbol) {
                                // either public
                                return !(ts.getDeclarationModifierFlagsFromSymbol(symbol) & 24 /* NonPublicAccessibilityModifier */)
                                    // or we're in it
                                    || containerClass_1 && ts.contains(typeForObject_1.symbol.declarations, containerClass_1);
                            });
                            existingMembers = objectLikeContainer.elements;
                        }
                    }
                    if (typeMembers && typeMembers.length > 0) {
                        // Add filtered items to the completion list
                        symbols = filterObjectMembersList(typeMembers, ts.Debug.assertDefined(existingMembers));
                    }
                    setSortTextToOptionalMember();
                    return 1 /* Success */;
                }
                /**
                 * Aggregates relevant symbols for completion in import clauses and export clauses
                 * whose declarations have a module specifier; for instance, symbols will be aggregated for
                 *
                 *      import { | } from "moduleName";
                 *      export { a as foo, | } from "moduleName";
                 *
                 * but not for
                 *
                 *      export { | };
                 *
                 * Relevant symbols are stored in the captured 'symbols' variable.
                 *
                 * @returns true if 'symbols' was successfully populated; false otherwise.
                 */
                function tryGetImportOrExportClauseCompletionSymbols() {
                    // `import { |` or `import { a as 0, | }`
                    var namedImportsOrExports = contextToken && (contextToken.kind === 18 /* OpenBraceToken */ || contextToken.kind === 27 /* CommaToken */)
                        ? ts.tryCast(contextToken.parent, ts.isNamedImportsOrExports) : undefined;
                    if (!namedImportsOrExports)
                        return 0 /* Continue */;
                    // cursor is in an import clause
                    // try to show exported member for imported module
                    var moduleSpecifier = (namedImportsOrExports.kind === 257 /* NamedImports */ ? namedImportsOrExports.parent.parent : namedImportsOrExports.parent).moduleSpecifier;
                    var moduleSpecifierSymbol = typeChecker.getSymbolAtLocation(moduleSpecifier); // TODO: GH#18217
                    if (!moduleSpecifierSymbol)
                        return 2 /* Fail */;
                    completionKind = 3 /* MemberLike */;
                    isNewIdentifierLocation = false;
                    var exports = typeChecker.getExportsAndPropertiesOfModule(moduleSpecifierSymbol);
                    var existing = ts.arrayToSet(namedImportsOrExports.elements, function (n) { return isCurrentlyEditingNode(n) ? undefined : (n.propertyName || n.name).escapedText; });
                    symbols = exports.filter(function (e) { return e.escapedName !== "default" /* Default */ && !existing.get(e.escapedName); });
                    return 1 /* Success */;
                }
                /**
                 * Aggregates relevant symbols for completion in class declaration
                 * Relevant symbols are stored in the captured 'symbols' variable.
                 */
                function tryGetClassLikeCompletionSymbols() {
                    var decl = tryGetObjectTypeDeclarationCompletionContainer(sourceFile, contextToken, location, position);
                    if (!decl)
                        return 0 /* Continue */;
                    // We're looking up possible property names from parent type.
                    completionKind = 3 /* MemberLike */;
                    // Declaring new property/method/accessor
                    isNewIdentifierLocation = true;
                    keywordFilters = contextToken.kind === 41 /* AsteriskToken */ ? 0 /* None */ :
                        ts.isClassLike(decl) ? 2 /* ClassElementKeywords */ : 3 /* InterfaceElementKeywords */;
                    // If you're in an interface you don't want to repeat things from super-interface. So just stop here.
                    if (!ts.isClassLike(decl))
                        return 1 /* Success */;
                    var classElement = contextToken.kind === 26 /* SemicolonToken */ ? contextToken.parent.parent : contextToken.parent;
                    var classElementModifierFlags = ts.isClassElement(classElement) ? ts.getModifierFlags(classElement) : 0 /* None */;
                    // If this is context token is not something we are editing now, consider if this would lead to be modifier
                    if (contextToken.kind === 75 /* Identifier */ && !isCurrentlyEditingNode(contextToken)) {
                        switch (contextToken.getText()) {
                            case "private":
                                classElementModifierFlags = classElementModifierFlags | 8 /* Private */;
                                break;
                            case "static":
                                classElementModifierFlags = classElementModifierFlags | 32 /* Static */;
                                break;
                        }
                    }
                    // No member list for private methods
                    if (!(classElementModifierFlags & 8 /* Private */)) {
                        // List of property symbols of base type that are not private and already implemented
                        var baseSymbols = ts.flatMap(ts.getAllSuperTypeNodes(decl), function (baseTypeNode) {
                            var type = typeChecker.getTypeAtLocation(baseTypeNode);
                            return type && typeChecker.getPropertiesOfType(classElementModifierFlags & 32 /* Static */ ? typeChecker.getTypeOfSymbolAtLocation(type.symbol, decl) : type);
                        });
                        symbols = filterClassMembersList(baseSymbols, decl.members, classElementModifierFlags);
                    }
                    return 1 /* Success */;
                }
                /**
                 * Returns the immediate owning object literal or binding pattern of a context token,
                 * on the condition that one exists and that the context implies completion should be given.
                 */
                function tryGetObjectLikeCompletionContainer(contextToken) {
                    if (contextToken) {
                        var parent = contextToken.parent;
                        switch (contextToken.kind) {
                            case 18 /* OpenBraceToken */: // const x = { |
                            case 27 /* CommaToken */: // const x = { a: 0, |
                                if (ts.isObjectLiteralExpression(parent) || ts.isObjectBindingPattern(parent)) {
                                    return parent;
                                }
                                break;
                            case 41 /* AsteriskToken */:
                                return ts.isMethodDeclaration(parent) ? ts.tryCast(parent.parent, ts.isObjectLiteralExpression) : undefined;
                            case 75 /* Identifier */:
                                return contextToken.text === "async" && ts.isShorthandPropertyAssignment(contextToken.parent)
                                    ? contextToken.parent.parent : undefined;
                        }
                    }
                    return undefined;
                }
                function isConstructorParameterCompletion(node) {
                    return !!node.parent && ts.isParameter(node.parent) && ts.isConstructorDeclaration(node.parent.parent)
                        && (ts.isParameterPropertyModifier(node.kind) || ts.isDeclarationName(node));
                }
                /**
                 * Returns the immediate owning class declaration of a context token,
                 * on the condition that one exists and that the context implies completion should be given.
                 */
                function tryGetConstructorLikeCompletionContainer(contextToken) {
                    if (contextToken) {
                        var parent = contextToken.parent;
                        switch (contextToken.kind) {
                            case 20 /* OpenParenToken */:
                            case 27 /* CommaToken */:
                                return ts.isConstructorDeclaration(contextToken.parent) ? contextToken.parent : undefined;
                            default:
                                if (isConstructorParameterCompletion(contextToken)) {
                                    return parent.parent;
                                }
                        }
                    }
                    return undefined;
                }
                function tryGetFunctionLikeBodyCompletionContainer(contextToken) {
                    if (contextToken) {
                        var prev_1;
                        var container = ts.findAncestor(contextToken.parent, function (node) {
                            if (ts.isClassLike(node)) {
                                return "quit";
                            }
                            if (ts.isFunctionLikeDeclaration(node) && prev_1 === node.body) {
                                return true;
                            }
                            prev_1 = node;
                            return false;
                        });
                        return container && container;
                    }
                }
                function tryGetContainingJsxElement(contextToken) {
                    if (contextToken) {
                        var parent = contextToken.parent;
                        switch (contextToken.kind) {
                            case 31 /* GreaterThanToken */: // End of a type argument list
                            case 30 /* LessThanSlashToken */:
                            case 43 /* SlashToken */:
                            case 75 /* Identifier */:
                            case 194 /* PropertyAccessExpression */:
                            case 274 /* JsxAttributes */:
                            case 273 /* JsxAttribute */:
                            case 275 /* JsxSpreadAttribute */:
                                if (parent && (parent.kind === 267 /* JsxSelfClosingElement */ || parent.kind === 268 /* JsxOpeningElement */)) {
                                    if (contextToken.kind === 31 /* GreaterThanToken */) {
                                        var precedingToken = ts.findPrecedingToken(contextToken.pos, sourceFile, /*startNode*/ undefined);
                                        if (!parent.typeArguments || (precedingToken && precedingToken.kind === 43 /* SlashToken */))
                                            break;
                                    }
                                    return parent;
                                }
                                else if (parent.kind === 273 /* JsxAttribute */) {
                                    // Currently we parse JsxOpeningLikeElement as:
                                    //      JsxOpeningLikeElement
                                    //          attributes: JsxAttributes
                                    //             properties: NodeArray<JsxAttributeLike>
                                    return parent.parent.parent;
                                }
                                break;
                            // The context token is the closing } or " of an attribute, which means
                            // its parent is a JsxExpression, whose parent is a JsxAttribute,
                            // whose parent is a JsxOpeningLikeElement
                            case 10 /* StringLiteral */:
                                if (parent && ((parent.kind === 273 /* JsxAttribute */) || (parent.kind === 275 /* JsxSpreadAttribute */))) {
                                    // Currently we parse JsxOpeningLikeElement as:
                                    //      JsxOpeningLikeElement
                                    //          attributes: JsxAttributes
                                    //             properties: NodeArray<JsxAttributeLike>
                                    return parent.parent.parent;
                                }
                                break;
                            case 19 /* CloseBraceToken */:
                                if (parent &&
                                    parent.kind === 276 /* JsxExpression */ &&
                                    parent.parent && parent.parent.kind === 273 /* JsxAttribute */) {
                                    // Currently we parse JsxOpeningLikeElement as:
                                    //      JsxOpeningLikeElement
                                    //          attributes: JsxAttributes
                                    //             properties: NodeArray<JsxAttributeLike>
                                    //                  each JsxAttribute can have initializer as JsxExpression
                                    return parent.parent.parent.parent;
                                }
                                if (parent && parent.kind === 275 /* JsxSpreadAttribute */) {
                                    // Currently we parse JsxOpeningLikeElement as:
                                    //      JsxOpeningLikeElement
                                    //          attributes: JsxAttributes
                                    //             properties: NodeArray<JsxAttributeLike>
                                    return parent.parent.parent;
                                }
                                break;
                        }
                    }
                    return undefined;
                }
                /**
                 * @returns true if we are certain that the currently edited location must define a new location; false otherwise.
                 */
                function isSolelyIdentifierDefinitionLocation(contextToken) {
                    var parent = contextToken.parent;
                    var containingNodeKind = parent.kind;
                    switch (contextToken.kind) {
                        case 27 /* CommaToken */:
                            return containingNodeKind === 242 /* VariableDeclaration */ ||
                                isVariableDeclarationListButNotTypeArgument(contextToken) ||
                                containingNodeKind === 225 /* VariableStatement */ ||
                                containingNodeKind === 248 /* EnumDeclaration */ || // enum a { foo, |
                                isFunctionLikeButNotConstructor(containingNodeKind) ||
                                containingNodeKind === 246 /* InterfaceDeclaration */ || // interface A<T, |
                                containingNodeKind === 190 /* ArrayBindingPattern */ || // var [x, y|
                                containingNodeKind === 247 /* TypeAliasDeclaration */ || // type Map, K, |
                                // class A<T, |
                                // var C = class D<T, |
                                (ts.isClassLike(parent) &&
                                    !!parent.typeParameters &&
                                    parent.typeParameters.end >= contextToken.pos);
                        case 24 /* DotToken */:
                            return containingNodeKind === 190 /* ArrayBindingPattern */; // var [.|
                        case 58 /* ColonToken */:
                            return containingNodeKind === 191 /* BindingElement */; // var {x :html|
                        case 22 /* OpenBracketToken */:
                            return containingNodeKind === 190 /* ArrayBindingPattern */; // var [x|
                        case 20 /* OpenParenToken */:
                            return containingNodeKind === 280 /* CatchClause */ ||
                                isFunctionLikeButNotConstructor(containingNodeKind);
                        case 18 /* OpenBraceToken */:
                            return containingNodeKind === 248 /* EnumDeclaration */; // enum a { |
                        case 29 /* LessThanToken */:
                            return containingNodeKind === 245 /* ClassDeclaration */ || // class A< |
                                containingNodeKind === 214 /* ClassExpression */ || // var C = class D< |
                                containingNodeKind === 246 /* InterfaceDeclaration */ || // interface A< |
                                containingNodeKind === 247 /* TypeAliasDeclaration */ || // type List< |
                                ts.isFunctionLikeKind(containingNodeKind);
                        case 120 /* StaticKeyword */:
                            return containingNodeKind === 159 /* PropertyDeclaration */ && !ts.isClassLike(parent.parent);
                        case 25 /* DotDotDotToken */:
                            return containingNodeKind === 156 /* Parameter */ ||
                                (!!parent.parent && parent.parent.kind === 190 /* ArrayBindingPattern */); // var [...z|
                        case 119 /* PublicKeyword */:
                        case 117 /* PrivateKeyword */:
                        case 118 /* ProtectedKeyword */:
                            return containingNodeKind === 156 /* Parameter */ && !ts.isConstructorDeclaration(parent.parent);
                        case 123 /* AsKeyword */:
                            return containingNodeKind === 258 /* ImportSpecifier */ ||
                                containingNodeKind === 263 /* ExportSpecifier */ ||
                                containingNodeKind === 256 /* NamespaceImport */;
                        case 131 /* GetKeyword */:
                        case 142 /* SetKeyword */:
                            return !isFromObjectTypeDeclaration(contextToken);
                        case 80 /* ClassKeyword */:
                        case 88 /* EnumKeyword */:
                        case 114 /* InterfaceKeyword */:
                        case 94 /* FunctionKeyword */:
                        case 109 /* VarKeyword */:
                        case 96 /* ImportKeyword */:
                        case 115 /* LetKeyword */:
                        case 81 /* ConstKeyword */:
                        case 121 /* YieldKeyword */:
                        case 145 /* TypeKeyword */: // type htm|
                            return true;
                        case 41 /* AsteriskToken */:
                            return ts.isFunctionLike(contextToken.parent) && !ts.isMethodDeclaration(contextToken.parent);
                    }
                    // If the previous token is keyword correspoding to class member completion keyword
                    // there will be completion available here
                    if (isClassMemberCompletionKeyword(keywordForNode(contextToken)) && isFromObjectTypeDeclaration(contextToken)) {
                        return false;
                    }
                    if (isConstructorParameterCompletion(contextToken)) {
                        // constructor parameter completion is available only if
                        // - its modifier of the constructor parameter or
                        // - its name of the parameter and not being edited
                        // eg. constructor(a |<- this shouldnt show completion
                        if (!ts.isIdentifier(contextToken) ||
                            ts.isParameterPropertyModifier(keywordForNode(contextToken)) ||
                            isCurrentlyEditingNode(contextToken)) {
                            return false;
                        }
                    }
                    // Previous token may have been a keyword that was converted to an identifier.
                    switch (keywordForNode(contextToken)) {
                        case 122 /* AbstractKeyword */:
                        case 80 /* ClassKeyword */:
                        case 81 /* ConstKeyword */:
                        case 130 /* DeclareKeyword */:
                        case 88 /* EnumKeyword */:
                        case 94 /* FunctionKeyword */:
                        case 114 /* InterfaceKeyword */:
                        case 115 /* LetKeyword */:
                        case 117 /* PrivateKeyword */:
                        case 118 /* ProtectedKeyword */:
                        case 119 /* PublicKeyword */:
                        case 120 /* StaticKeyword */:
                        case 109 /* VarKeyword */:
                        case 121 /* YieldKeyword */:
                            return true;
                        case 126 /* AsyncKeyword */:
                            return ts.isPropertyDeclaration(contextToken.parent);
                    }
                    return ts.isDeclarationName(contextToken)
                        && !ts.isJsxAttribute(contextToken.parent)
                        // Don't block completions if we're in `class C /**/`, because we're *past* the end of the identifier and might want to complete `extends`.
                        // If `contextToken !== previousToken`, this is `class C ex/**/`.
                        && !(ts.isClassLike(contextToken.parent) && (contextToken !== previousToken || position > previousToken.end));
                }
                function isFunctionLikeButNotConstructor(kind) {
                    return ts.isFunctionLikeKind(kind) && kind !== 162 /* Constructor */;
                }
                function isDotOfNumericLiteral(contextToken) {
                    if (contextToken.kind === 8 /* NumericLiteral */) {
                        var text = contextToken.getFullText();
                        return text.charAt(text.length - 1) === ".";
                    }
                    return false;
                }
                function isVariableDeclarationListButNotTypeArgument(node) {
                    return node.parent.kind === 243 /* VariableDeclarationList */
                        && !ts.isPossiblyTypeArgumentPosition(node, sourceFile, typeChecker);
                }
                /**
                 * Filters out completion suggestions for named imports or exports.
                 *
                 * @returns Symbols to be suggested in an object binding pattern or object literal expression, barring those whose declarations
                 *          do not occur at the current position and have not otherwise been typed.
                 */
                function filterObjectMembersList(contextualMemberSymbols, existingMembers) {
                    if (existingMembers.length === 0) {
                        return contextualMemberSymbols;
                    }
                    var membersDeclaredBySpreadAssignment = ts.createMap();
                    var existingMemberNames = ts.createUnderscoreEscapedMap();
                    for (var _i = 0, existingMembers_1 = existingMembers; _i < existingMembers_1.length; _i++) {
                        var m = existingMembers_1[_i];
                        // Ignore omitted expressions for missing members
                        if (m.kind !== 281 /* PropertyAssignment */ &&
                            m.kind !== 282 /* ShorthandPropertyAssignment */ &&
                            m.kind !== 191 /* BindingElement */ &&
                            m.kind !== 161 /* MethodDeclaration */ &&
                            m.kind !== 163 /* GetAccessor */ &&
                            m.kind !== 164 /* SetAccessor */ &&
                            m.kind !== 283 /* SpreadAssignment */) {
                            continue;
                        }
                        // If this is the current item we are editing right now, do not filter it out
                        if (isCurrentlyEditingNode(m)) {
                            continue;
                        }
                        var existingName = void 0;
                        if (ts.isSpreadAssignment(m)) {
                            setMembersDeclaredBySpreadAssignment(m, membersDeclaredBySpreadAssignment);
                        }
                        else if (ts.isBindingElement(m) && m.propertyName) {
                            // include only identifiers in completion list
                            if (m.propertyName.kind === 75 /* Identifier */) {
                                existingName = m.propertyName.escapedText;
                            }
                        }
                        else {
                            // TODO: Account for computed property name
                            // NOTE: if one only performs this step when m.name is an identifier,
                            // things like '__proto__' are not filtered out.
                            var name = ts.getNameOfDeclaration(m);
                            existingName = name && ts.isPropertyNameLiteral(name) ? ts.getEscapedTextOfIdentifierOrLiteral(name) : undefined;
                        }
                        existingMemberNames.set(existingName, true); // TODO: GH#18217
                    }
                    var filteredSymbols = contextualMemberSymbols.filter(function (m) { return !existingMemberNames.get(m.escapedName); });
                    setSortTextToMemberDeclaredBySpreadAssignment(membersDeclaredBySpreadAssignment, filteredSymbols);
                    return filteredSymbols;
                }
                function setMembersDeclaredBySpreadAssignment(declaration, membersDeclaredBySpreadAssignment) {
                    var expression = declaration.expression;
                    var symbol = typeChecker.getSymbolAtLocation(expression);
                    var type = symbol && typeChecker.getTypeOfSymbolAtLocation(symbol, expression);
                    var properties = type && type.properties;
                    if (properties) {
                        properties.forEach(function (property) {
                            membersDeclaredBySpreadAssignment.set(property.name, true);
                        });
                    }
                }
                // Set SortText to OptionalMember if it is an optinoal member
                function setSortTextToOptionalMember() {
                    symbols.forEach(function (m) {
                        if (m.flags & 16777216 /* Optional */) {
                            symbolToSortTextMap[ts.getSymbolId(m)] = symbolToSortTextMap[ts.getSymbolId(m)] || SortText.OptionalMember;
                        }
                    });
                }
                // Set SortText to MemberDeclaredBySpreadAssignment if it is fulfilled by spread assignment
                function setSortTextToMemberDeclaredBySpreadAssignment(membersDeclaredBySpreadAssignment, contextualMemberSymbols) {
                    if (membersDeclaredBySpreadAssignment.size === 0) {
                        return;
                    }
                    for (var _i = 0, contextualMemberSymbols_1 = contextualMemberSymbols; _i < contextualMemberSymbols_1.length; _i++) {
                        var contextualMemberSymbol = contextualMemberSymbols_1[_i];
                        if (membersDeclaredBySpreadAssignment.has(contextualMemberSymbol.name)) {
                            symbolToSortTextMap[ts.getSymbolId(contextualMemberSymbol)] = SortText.MemberDeclaredBySpreadAssignment;
                        }
                    }
                }
                /**
                 * Filters out completion suggestions for class elements.
                 *
                 * @returns Symbols to be suggested in an class element depending on existing memebers and symbol flags
                 */
                function filterClassMembersList(baseSymbols, existingMembers, currentClassElementModifierFlags) {
                    var existingMemberNames = ts.createUnderscoreEscapedMap();
                    for (var _i = 0, existingMembers_2 = existingMembers; _i < existingMembers_2.length; _i++) {
                        var m = existingMembers_2[_i];
                        // Ignore omitted expressions for missing members
                        if (m.kind !== 159 /* PropertyDeclaration */ &&
                            m.kind !== 161 /* MethodDeclaration */ &&
                            m.kind !== 163 /* GetAccessor */ &&
                            m.kind !== 164 /* SetAccessor */) {
                            continue;
                        }
                        // If this is the current item we are editing right now, do not filter it out
                        if (isCurrentlyEditingNode(m)) {
                            continue;
                        }
                        // Dont filter member even if the name matches if it is declared private in the list
                        if (ts.hasModifier(m, 8 /* Private */)) {
                            continue;
                        }
                        // do not filter it out if the static presence doesnt match
                        if (ts.hasModifier(m, 32 /* Static */) !== !!(currentClassElementModifierFlags & 32 /* Static */)) {
                            continue;
                        }
                        var existingName = ts.getPropertyNameForPropertyNameNode(m.name);
                        if (existingName) {
                            existingMemberNames.set(existingName, true);
                        }
                    }
                    return baseSymbols.filter(function (propertySymbol) {
                        return !existingMemberNames.has(propertySymbol.escapedName) &&
                            !!propertySymbol.declarations &&
                            !(ts.getDeclarationModifierFlagsFromSymbol(propertySymbol) & 8 /* Private */);
                    });
                }
                /**
                 * Filters out completion suggestions from 'symbols' according to existing JSX attributes.
                 *
                 * @returns Symbols to be suggested in a JSX element, barring those whose attributes
                 *          do not occur at the current position and have not otherwise been typed.
                 */
                function filterJsxAttributes(symbols, attributes) {
                    var seenNames = ts.createUnderscoreEscapedMap();
                    var membersDeclaredBySpreadAssignment = ts.createMap();
                    for (var _i = 0, attributes_1 = attributes; _i < attributes_1.length; _i++) {
                        var attr = attributes_1[_i];
                        // If this is the current item we are editing right now, do not filter it out
                        if (isCurrentlyEditingNode(attr)) {
                            continue;
                        }
                        if (attr.kind === 273 /* JsxAttribute */) {
                            seenNames.set(attr.name.escapedText, true);
                        }
                        else if (ts.isJsxSpreadAttribute(attr)) {
                            setMembersDeclaredBySpreadAssignment(attr, membersDeclaredBySpreadAssignment);
                        }
                    }
                    var filteredSymbols = symbols.filter(function (a) { return !seenNames.get(a.escapedName); });
                    setSortTextToMemberDeclaredBySpreadAssignment(membersDeclaredBySpreadAssignment, filteredSymbols);
                    return filteredSymbols;
                }
                function isCurrentlyEditingNode(node) {
                    return node.getStart(sourceFile) <= position && position <= node.getEnd();
                }
            }
            function getCompletionEntryDisplayNameForSymbol(symbol, target, origin, kind) {
                var name = getSymbolName(symbol, origin, target);
                if (name === undefined
                    // If the symbol is external module, don't show it in the completion list
                    // (i.e declare module "http" { const x; } | // <= request completion here, "http" should not be there)
                    || symbol.flags & 1536 /* Module */ && ts.isSingleOrDoubleQuote(name.charCodeAt(0))
                    // If the symbol is the internal name of an ES symbol, it is not a valid entry. Internal names for ES symbols start with "__@"
                    || ts.isKnownSymbol(symbol)) {
                    return undefined;
                }
                var validNameResult = { name: name, needsConvertPropertyAccess: false };
                if (ts.isIdentifierText(name, target) || symbol.valueDeclaration && ts.isPrivateIdentifierPropertyDeclaration(symbol.valueDeclaration)) {
                    return validNameResult;
                }
                switch (kind) {
                    case 3 /* MemberLike */:
                        return undefined;
                    case 0 /* ObjectPropertyDeclaration */:
                        // TODO: GH#18169
                        return { name: JSON.stringify(name), needsConvertPropertyAccess: false };
                    case 2 /* PropertyAccess */:
                    case 1 /* Global */: // For a 'this.' completion it will be in a global context, but may have a non-identifier name.
                        // Don't add a completion for a name starting with a space. See https://github.com/Microsoft/TypeScript/pull/20547
                        return name.charCodeAt(0) === 32 /* space */ ? undefined : { name: name, needsConvertPropertyAccess: true };
                    case 5 /* None */:
                    case 4 /* String */:
                        return validNameResult;
                    default:
                        ts.Debug.assertNever(kind);
                }
            }
            // A cache of completion entries for keywords, these do not change between sessions
            var _keywordCompletions = [];
            var allKeywordsCompletions = ts.memoize(function () {
                var res = [];
                for (var i = 77 /* FirstKeyword */; i <= 152 /* LastKeyword */; i++) {
                    res.push({
                        name: ts.tokenToString(i),
                        kind: "keyword" /* keyword */,
                        kindModifiers: "" /* none */,
                        sortText: SortText.GlobalsOrKeywords
                    });
                }
                return res;
            });
            function getKeywordCompletions(keywordFilter, filterOutTsOnlyKeywords) {
                if (!filterOutTsOnlyKeywords)
                    return getTypescriptKeywordCompletions(keywordFilter);
                var index = keywordFilter + 7 /* Last */ + 1;
                return _keywordCompletions[index] ||
                    (_keywordCompletions[index] = getTypescriptKeywordCompletions(keywordFilter)
                        .filter(function (entry) { return !isTypeScriptOnlyKeyword(ts.stringToToken(entry.name)); }));
            }
            function getTypescriptKeywordCompletions(keywordFilter) {
                return _keywordCompletions[keywordFilter] || (_keywordCompletions[keywordFilter] = allKeywordsCompletions().filter(function (entry) {
                    var kind = ts.stringToToken(entry.name);
                    switch (keywordFilter) {
                        case 0 /* None */:
                            return false;
                        case 1 /* All */:
                            return isFunctionLikeBodyKeyword(kind)
                                || kind === 130 /* DeclareKeyword */
                                || kind === 135 /* ModuleKeyword */
                                || kind === 145 /* TypeKeyword */
                                || kind === 136 /* NamespaceKeyword */
                                || kind === 123 /* AsKeyword */
                                || ts.isTypeKeyword(kind) && kind !== 146 /* UndefinedKeyword */;
                        case 5 /* FunctionLikeBodyKeywords */:
                            return isFunctionLikeBodyKeyword(kind);
                        case 2 /* ClassElementKeywords */:
                            return isClassMemberCompletionKeyword(kind);
                        case 3 /* InterfaceElementKeywords */:
                            return isInterfaceOrTypeLiteralCompletionKeyword(kind);
                        case 4 /* ConstructorParameterKeywords */:
                            return ts.isParameterPropertyModifier(kind);
                        case 6 /* TypeAssertionKeywords */:
                            return ts.isTypeKeyword(kind) || kind === 81 /* ConstKeyword */;
                        case 7 /* TypeKeywords */:
                            return ts.isTypeKeyword(kind);
                        default:
                            return ts.Debug.assertNever(keywordFilter);
                    }
                }));
            }
            function isTypeScriptOnlyKeyword(kind) {
                switch (kind) {
                    case 122 /* AbstractKeyword */:
                    case 125 /* AnyKeyword */:
                    case 151 /* BigIntKeyword */:
                    case 128 /* BooleanKeyword */:
                    case 130 /* DeclareKeyword */:
                    case 88 /* EnumKeyword */:
                    case 150 /* GlobalKeyword */:
                    case 113 /* ImplementsKeyword */:
                    case 132 /* InferKeyword */:
                    case 114 /* InterfaceKeyword */:
                    case 133 /* IsKeyword */:
                    case 134 /* KeyOfKeyword */:
                    case 135 /* ModuleKeyword */:
                    case 136 /* NamespaceKeyword */:
                    case 137 /* NeverKeyword */:
                    case 140 /* NumberKeyword */:
                    case 141 /* ObjectKeyword */:
                    case 117 /* PrivateKeyword */:
                    case 118 /* ProtectedKeyword */:
                    case 119 /* PublicKeyword */:
                    case 138 /* ReadonlyKeyword */:
                    case 143 /* StringKeyword */:
                    case 144 /* SymbolKeyword */:
                    case 145 /* TypeKeyword */:
                    case 147 /* UniqueKeyword */:
                    case 148 /* UnknownKeyword */:
                        return true;
                    default:
                        return false;
                }
            }
            function isInterfaceOrTypeLiteralCompletionKeyword(kind) {
                return kind === 138 /* ReadonlyKeyword */;
            }
            function isClassMemberCompletionKeyword(kind) {
                switch (kind) {
                    case 122 /* AbstractKeyword */:
                    case 129 /* ConstructorKeyword */:
                    case 131 /* GetKeyword */:
                    case 142 /* SetKeyword */:
                    case 126 /* AsyncKeyword */:
                    case 130 /* DeclareKeyword */:
                        return true;
                    default:
                        return ts.isClassMemberModifier(kind);
                }
            }
            function isFunctionLikeBodyKeyword(kind) {
                return kind === 126 /* AsyncKeyword */
                    || kind === 127 /* AwaitKeyword */
                    || !ts.isContextualKeyword(kind) && !isClassMemberCompletionKeyword(kind);
            }
            function keywordForNode(node) {
                return ts.isIdentifier(node) ? node.originalKeywordKind || 0 /* Unknown */ : node.kind;
            }
            /** Get the corresponding JSDocTag node if the position is in a jsDoc comment */
            function getJsDocTagAtPosition(node, position) {
                var jsdoc = ts.findAncestor(node, ts.isJSDoc);
                return jsdoc && jsdoc.tags && (ts.rangeContainsPosition(jsdoc, position) ? ts.findLast(jsdoc.tags, function (tag) { return tag.pos < position; }) : undefined);
            }
            function getPropertiesForObjectExpression(contextualType, completionsType, obj, checker) {
                var hasCompletionsType = completionsType && completionsType !== contextualType;
                var type = hasCompletionsType && !(completionsType.flags & 3 /* AnyOrUnknown */)
                    ? checker.getUnionType([contextualType, completionsType])
                    : contextualType;
                var properties = type.isUnion()
                    ? checker.getAllPossiblePropertiesOfTypes(type.types.filter(function (memberType) {
                        // If we're providing completions for an object literal, skip primitive, array-like, or callable types since those shouldn't be implemented by object literals.
                        return !(memberType.flags & 131068 /* Primitive */ ||
                            checker.isArrayLikeType(memberType) ||
                            ts.typeHasCallOrConstructSignatures(memberType, checker) ||
                            checker.isTypeInvalidDueToUnionDiscriminant(memberType, obj));
                    }))
                    : type.getApparentProperties();
                return hasCompletionsType ? properties.filter(hasDeclarationOtherThanSelf) : properties;
                // Filter out members whose only declaration is the object literal itself to avoid
                // self-fulfilling completions like:
                //
                // function f<T>(x: T) {}
                // f({ abc/**/: "" }) // `abc` is a member of `T` but only because it declares itself
                function hasDeclarationOtherThanSelf(member) {
                    return ts.some(member.declarations, function (decl) { return decl.parent !== obj; });
                }
            }
            /**
             * Gets all properties on a type, but if that type is a union of several types,
             * excludes array-like types or callable/constructable types.
             */
            function getPropertiesForCompletion(type, checker) {
                return type.isUnion()
                    ? ts.Debug.assertEachDefined(checker.getAllPossiblePropertiesOfTypes(type.types), "getAllPossiblePropertiesOfTypes() should all be defined")
                    : ts.Debug.assertEachDefined(type.getApparentProperties(), "getApparentProperties() should all be defined");
            }
            /**
             * Returns the immediate owning class declaration of a context token,
             * on the condition that one exists and that the context implies completion should be given.
             */
            function tryGetObjectTypeDeclarationCompletionContainer(sourceFile, contextToken, location, position) {
                // class c { method() { } | method2() { } }
                switch (location.kind) {
                    case 323 /* SyntaxList */:
                        return ts.tryCast(location.parent, ts.isObjectTypeDeclaration);
                    case 1 /* EndOfFileToken */:
                        var cls = ts.tryCast(ts.lastOrUndefined(ts.cast(location.parent, ts.isSourceFile).statements), ts.isObjectTypeDeclaration);
                        if (cls && !ts.findChildOfKind(cls, 19 /* CloseBraceToken */, sourceFile)) {
                            return cls;
                        }
                        break;
                    case 75 /* Identifier */: // class c extends React.Component { a: () => 1\n compon| }
                        if (isFromObjectTypeDeclaration(location)) {
                            return ts.findAncestor(location, ts.isObjectTypeDeclaration);
                        }
                }
                if (!contextToken)
                    return undefined;
                switch (contextToken.kind) {
                    case 62 /* EqualsToken */: // class c { public prop = | /* global completions */ }
                        return undefined;
                    case 26 /* SemicolonToken */: // class c {getValue(): number; | }
                    case 19 /* CloseBraceToken */: // class c { method() { } | }
                        // class c { method() { } b| }
                        return isFromObjectTypeDeclaration(location) && location.parent.name === location
                            ? location.parent.parent
                            : ts.tryCast(location, ts.isObjectTypeDeclaration);
                    case 18 /* OpenBraceToken */: // class c { |
                    case 27 /* CommaToken */: // class c {getValue(): number, | }
                        return ts.tryCast(contextToken.parent, ts.isObjectTypeDeclaration);
                    default:
                        if (!isFromObjectTypeDeclaration(contextToken)) {
                            // class c extends React.Component { a: () => 1\n| }
                            if (ts.getLineAndCharacterOfPosition(sourceFile, contextToken.getEnd()).line !== ts.getLineAndCharacterOfPosition(sourceFile, position).line && ts.isObjectTypeDeclaration(location)) {
                                return location;
                            }
                            return undefined;
                        }
                        var isValidKeyword = ts.isClassLike(contextToken.parent.parent) ? isClassMemberCompletionKeyword : isInterfaceOrTypeLiteralCompletionKeyword;
                        return (isValidKeyword(contextToken.kind) || contextToken.kind === 41 /* AsteriskToken */ || ts.isIdentifier(contextToken) && isValidKeyword(ts.stringToToken(contextToken.text))) // TODO: GH#18217
                            ? contextToken.parent.parent : undefined;
                }
            }
            // TODO: GH#19856 Would like to return `node is Node & { parent: (ClassElement | TypeElement) & { parent: ObjectTypeDeclaration } }` but then compilation takes > 10 minutes
            function isFromObjectTypeDeclaration(node) {
                return node.parent && ts.isClassOrTypeElement(node.parent) && ts.isObjectTypeDeclaration(node.parent.parent);
            }
            function isValidTrigger(sourceFile, triggerCharacter, contextToken, position) {
                switch (triggerCharacter) {
                    case ".":
                    case "@":
                        return true;
                    case '"':
                    case "'":
                    case "`":
                        // Only automatically bring up completions if this is an opening quote.
                        return !!contextToken && ts.isStringLiteralOrTemplate(contextToken) && position === contextToken.getStart(sourceFile) + 1;
                    case "#":
                        return !!contextToken && ts.isPrivateIdentifier(contextToken) && !!ts.getContainingClass(contextToken);
                    case "<":
                        // Opening JSX tag
                        return !!contextToken && contextToken.kind === 29 /* LessThanToken */ && (!ts.isBinaryExpression(contextToken.parent) || binaryExpressionMayBeOpenTag(contextToken.parent));
                    case "/":
                        return !!contextToken && (ts.isStringLiteralLike(contextToken)
                            ? !!ts.tryGetImportFromModuleSpecifier(contextToken)
                            : contextToken.kind === 43 /* SlashToken */ && ts.isJsxClosingElement(contextToken.parent));
                    default:
                        return ts.Debug.assertNever(triggerCharacter);
                }
            }
            function binaryExpressionMayBeOpenTag(_a) {
                var left = _a.left;
                return ts.nodeIsMissing(left);
            }
            function findAlias(typeChecker, symbol, predicate) {
                var currentAlias = symbol;
                while (currentAlias.flags & 2097152 /* Alias */ && (currentAlias = typeChecker.getImmediateAliasedSymbol(currentAlias))) {
                    if (predicate(currentAlias)) {
                        return currentAlias;
                    }
                }
            }
        })(Completions = ts.Completions || (ts.Completions = {}));
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        /* @internal */
        var DocumentHighlights;
        (function (DocumentHighlights) {
            function getDocumentHighlights(program, cancellationToken, sourceFile, position, sourceFilesToSearch) {
                var node = ts.getTouchingPropertyName(sourceFile, position);
                if (node.parent && (ts.isJsxOpeningElement(node.parent) && node.parent.tagName === node || ts.isJsxClosingElement(node.parent))) {
                    // For a JSX element, just highlight the matching tag, not all references.
                    var _a = node.parent.parent, openingElement = _a.openingElement, closingElement = _a.closingElement;
                    var highlightSpans = [openingElement, closingElement].map(function (_a) {
                        var tagName = _a.tagName;
                        return getHighlightSpanForNode(tagName, sourceFile);
                    });
                    return [{ fileName: sourceFile.fileName, highlightSpans: highlightSpans }];
                }
                return getSemanticDocumentHighlights(position, node, program, cancellationToken, sourceFilesToSearch) || getSyntacticDocumentHighlights(node, sourceFile);
            }
            DocumentHighlights.getDocumentHighlights = getDocumentHighlights;
            function getHighlightSpanForNode(node, sourceFile) {
                return {
                    fileName: sourceFile.fileName,
                    textSpan: ts.createTextSpanFromNode(node, sourceFile),
                    kind: "none" /* none */
                };
            }
            function getSemanticDocumentHighlights(position, node, program, cancellationToken, sourceFilesToSearch) {
                var sourceFilesSet = ts.arrayToSet(sourceFilesToSearch, function (f) { return f.fileName; });
                var referenceEntries = ts.FindAllReferences.getReferenceEntriesForNode(position, node, program, sourceFilesToSearch, cancellationToken, /*options*/ undefined, sourceFilesSet);
                if (!referenceEntries)
                    return undefined;
                var map = ts.arrayToMultiMap(referenceEntries.map(ts.FindAllReferences.toHighlightSpan), function (e) { return e.fileName; }, function (e) { return e.span; });
                return ts.arrayFrom(map.entries(), function (_a) {
                    var fileName = _a[0], highlightSpans = _a[1];
                    if (!sourceFilesSet.has(fileName)) {
                        ts.Debug.assert(program.redirectTargetsMap.has(fileName));
                        var redirectTarget_1 = program.getSourceFile(fileName);
                        var redirect = ts.find(sourceFilesToSearch, function (f) { return !!f.redirectInfo && f.redirectInfo.redirectTarget === redirectTarget_1; });
                        fileName = redirect.fileName;
                        ts.Debug.assert(sourceFilesSet.has(fileName));
                    }
                    return { fileName: fileName, highlightSpans: highlightSpans };
                });
            }
            function getSyntacticDocumentHighlights(node, sourceFile) {
                var highlightSpans = getHighlightSpans(node, sourceFile);
                return highlightSpans && [{ fileName: sourceFile.fileName, highlightSpans: highlightSpans }];
            }
            function getHighlightSpans(node, sourceFile) {
                switch (node.kind) {
                    case 95 /* IfKeyword */:
                    case 87 /* ElseKeyword */:
                        return ts.isIfStatement(node.parent) ? getIfElseOccurrences(node.parent, sourceFile) : undefined;
                    case 101 /* ReturnKeyword */:
                        return useParent(node.parent, ts.isReturnStatement, getReturnOccurrences);
                    case 105 /* ThrowKeyword */:
                        return useParent(node.parent, ts.isThrowStatement, getThrowOccurrences);
                    case 107 /* TryKeyword */:
                    case 79 /* CatchKeyword */:
                    case 92 /* FinallyKeyword */:
                        var tryStatement = node.kind === 79 /* CatchKeyword */ ? node.parent.parent : node.parent;
                        return useParent(tryStatement, ts.isTryStatement, getTryCatchFinallyOccurrences);
                    case 103 /* SwitchKeyword */:
                        return useParent(node.parent, ts.isSwitchStatement, getSwitchCaseDefaultOccurrences);
                    case 78 /* CaseKeyword */:
                    case 84 /* DefaultKeyword */: {
                        if (ts.isDefaultClause(node.parent) || ts.isCaseClause(node.parent)) {
                            return useParent(node.parent.parent.parent, ts.isSwitchStatement, getSwitchCaseDefaultOccurrences);
                        }
                        return undefined;
                    }
                    case 77 /* BreakKeyword */:
                    case 82 /* ContinueKeyword */:
                        return useParent(node.parent, ts.isBreakOrContinueStatement, getBreakOrContinueStatementOccurrences);
                    case 93 /* ForKeyword */:
                    case 111 /* WhileKeyword */:
                    case 86 /* DoKeyword */:
                        return useParent(node.parent, function (n) { return ts.isIterationStatement(n, /*lookInLabeledStatements*/ true); }, getLoopBreakContinueOccurrences);
                    case 129 /* ConstructorKeyword */:
                        return getFromAllDeclarations(ts.isConstructorDeclaration, [129 /* ConstructorKeyword */]);
                    case 131 /* GetKeyword */:
                    case 142 /* SetKeyword */:
                        return getFromAllDeclarations(ts.isAccessor, [131 /* GetKeyword */, 142 /* SetKeyword */]);
                    case 127 /* AwaitKeyword */:
                        return useParent(node.parent, ts.isAwaitExpression, getAsyncAndAwaitOccurrences);
                    case 126 /* AsyncKeyword */:
                        return highlightSpans(getAsyncAndAwaitOccurrences(node));
                    case 121 /* YieldKeyword */:
                        return highlightSpans(getYieldOccurrences(node));
                    default:
                        return ts.isModifierKind(node.kind) && (ts.isDeclaration(node.parent) || ts.isVariableStatement(node.parent))
                            ? highlightSpans(getModifierOccurrences(node.kind, node.parent))
                            : undefined;
                }
                function getFromAllDeclarations(nodeTest, keywords) {
                    return useParent(node.parent, nodeTest, function (decl) {
                        return ts.mapDefined(decl.symbol.declarations, function (d) {
                            return nodeTest(d) ? ts.find(d.getChildren(sourceFile), function (c) { return ts.contains(keywords, c.kind); }) : undefined;
                        });
                    });
                }
                function useParent(node, nodeTest, getNodes) {
                    return nodeTest(node) ? highlightSpans(getNodes(node, sourceFile)) : undefined;
                }
                function highlightSpans(nodes) {
                    return nodes && nodes.map(function (node) { return getHighlightSpanForNode(node, sourceFile); });
                }
            }
            /**
             * Aggregates all throw-statements within this node *without* crossing
             * into function boundaries and try-blocks with catch-clauses.
             */
            function aggregateOwnedThrowStatements(node) {
                if (ts.isThrowStatement(node)) {
                    return [node];
                }
                else if (ts.isTryStatement(node)) {
                    // Exceptions thrown within a try block lacking a catch clause are "owned" in the current context.
                    return ts.concatenate(node.catchClause ? aggregateOwnedThrowStatements(node.catchClause) : node.tryBlock && aggregateOwnedThrowStatements(node.tryBlock), node.finallyBlock && aggregateOwnedThrowStatements(node.finallyBlock));
                }
                // Do not cross function boundaries.
                return ts.isFunctionLike(node) ? undefined : flatMapChildren(node, aggregateOwnedThrowStatements);
            }
            /**
             * For lack of a better name, this function takes a throw statement and returns the
             * nearest ancestor that is a try-block (whose try statement has a catch clause),
             * function-block, or source file.
             */
            function getThrowStatementOwner(throwStatement) {
                var child = throwStatement;
                while (child.parent) {
                    var parent = child.parent;
                    if (ts.isFunctionBlock(parent) || parent.kind === 290 /* SourceFile */) {
                        return parent;
                    }
                    // A throw-statement is only owned by a try-statement if the try-statement has
                    // a catch clause, and if the throw-statement occurs within the try block.
                    if (ts.isTryStatement(parent) && parent.tryBlock === child && parent.catchClause) {
                        return child;
                    }
                    child = parent;
                }
                return undefined;
            }
            function aggregateAllBreakAndContinueStatements(node) {
                return ts.isBreakOrContinueStatement(node) ? [node] : ts.isFunctionLike(node) ? undefined : flatMapChildren(node, aggregateAllBreakAndContinueStatements);
            }
            function flatMapChildren(node, cb) {
                var result = [];
                node.forEachChild(function (child) {
                    var value = cb(child);
                    if (value !== undefined) {
                        result.push.apply(result, ts.toArray(value));
                    }
                });
                return result;
            }
            function ownsBreakOrContinueStatement(owner, statement) {
                var actualOwner = getBreakOrContinueOwner(statement);
                return !!actualOwner && actualOwner === owner;
            }
            function getBreakOrContinueOwner(statement) {
                return ts.findAncestor(statement, function (node) {
                    switch (node.kind) {
                        case 237 /* SwitchStatement */:
                            if (statement.kind === 233 /* ContinueStatement */) {
                                return false;
                            }
                        // falls through
                        case 230 /* ForStatement */:
                        case 231 /* ForInStatement */:
                        case 232 /* ForOfStatement */:
                        case 229 /* WhileStatement */:
                        case 228 /* DoStatement */:
                            return !statement.label || isLabeledBy(node, statement.label.escapedText);
                        default:
                            // Don't cross function boundaries.
                            // TODO: GH#20090
                            return ts.isFunctionLike(node) && "quit";
                    }
                });
            }
            function getModifierOccurrences(modifier, declaration) {
                return ts.mapDefined(getNodesToSearchForModifier(declaration, ts.modifierToFlag(modifier)), function (node) { return ts.findModifier(node, modifier); });
            }
            function getNodesToSearchForModifier(declaration, modifierFlag) {
                // Types of node whose children might have modifiers.
                var container = declaration.parent;
                switch (container.kind) {
                    case 250 /* ModuleBlock */:
                    case 290 /* SourceFile */:
                    case 223 /* Block */:
                    case 277 /* CaseClause */:
                    case 278 /* DefaultClause */:
                        // Container is either a class declaration or the declaration is a classDeclaration
                        if (modifierFlag & 128 /* Abstract */ && ts.isClassDeclaration(declaration)) {
                            return __spreadArrays(declaration.members, [declaration]);
                        }
                        else {
                            return container.statements;
                        }
                    case 162 /* Constructor */:
                    case 161 /* MethodDeclaration */:
                    case 244 /* FunctionDeclaration */:
                        return __spreadArrays(container.parameters, (ts.isClassLike(container.parent) ? container.parent.members : []));
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                    case 246 /* InterfaceDeclaration */:
                    case 173 /* TypeLiteral */:
                        var nodes = container.members;
                        // If we're an accessibility modifier, we're in an instance member and should search
                        // the constructor's parameter list for instance members as well.
                        if (modifierFlag & (28 /* AccessibilityModifier */ | 64 /* Readonly */)) {
                            var constructor = ts.find(container.members, ts.isConstructorDeclaration);
                            if (constructor) {
                                return __spreadArrays(nodes, constructor.parameters);
                            }
                        }
                        else if (modifierFlag & 128 /* Abstract */) {
                            return __spreadArrays(nodes, [container]);
                        }
                        return nodes;
                    default:
                        ts.Debug.assertNever(container, "Invalid container kind.");
                }
            }
            function pushKeywordIf(keywordList, token) {
                var expected = [];
                for (var _i = 2; _i < arguments.length; _i++) {
                    expected[_i - 2] = arguments[_i];
                }
                if (token && ts.contains(expected, token.kind)) {
                    keywordList.push(token);
                    return true;
                }
                return false;
            }
            function getLoopBreakContinueOccurrences(loopNode) {
                var keywords = [];
                if (pushKeywordIf(keywords, loopNode.getFirstToken(), 93 /* ForKeyword */, 111 /* WhileKeyword */, 86 /* DoKeyword */)) {
                    // If we succeeded and got a do-while loop, then start looking for a 'while' keyword.
                    if (loopNode.kind === 228 /* DoStatement */) {
                        var loopTokens = loopNode.getChildren();
                        for (var i = loopTokens.length - 1; i >= 0; i--) {
                            if (pushKeywordIf(keywords, loopTokens[i], 111 /* WhileKeyword */)) {
                                break;
                            }
                        }
                    }
                }
                ts.forEach(aggregateAllBreakAndContinueStatements(loopNode.statement), function (statement) {
                    if (ownsBreakOrContinueStatement(loopNode, statement)) {
                        pushKeywordIf(keywords, statement.getFirstToken(), 77 /* BreakKeyword */, 82 /* ContinueKeyword */);
                    }
                });
                return keywords;
            }
            function getBreakOrContinueStatementOccurrences(breakOrContinueStatement) {
                var owner = getBreakOrContinueOwner(breakOrContinueStatement);
                if (owner) {
                    switch (owner.kind) {
                        case 230 /* ForStatement */:
                        case 231 /* ForInStatement */:
                        case 232 /* ForOfStatement */:
                        case 228 /* DoStatement */:
                        case 229 /* WhileStatement */:
                            return getLoopBreakContinueOccurrences(owner);
                        case 237 /* SwitchStatement */:
                            return getSwitchCaseDefaultOccurrences(owner);
                    }
                }
                return undefined;
            }
            function getSwitchCaseDefaultOccurrences(switchStatement) {
                var keywords = [];
                pushKeywordIf(keywords, switchStatement.getFirstToken(), 103 /* SwitchKeyword */);
                // Go through each clause in the switch statement, collecting the 'case'/'default' keywords.
                ts.forEach(switchStatement.caseBlock.clauses, function (clause) {
                    pushKeywordIf(keywords, clause.getFirstToken(), 78 /* CaseKeyword */, 84 /* DefaultKeyword */);
                    ts.forEach(aggregateAllBreakAndContinueStatements(clause), function (statement) {
                        if (ownsBreakOrContinueStatement(switchStatement, statement)) {
                            pushKeywordIf(keywords, statement.getFirstToken(), 77 /* BreakKeyword */);
                        }
                    });
                });
                return keywords;
            }
            function getTryCatchFinallyOccurrences(tryStatement, sourceFile) {
                var keywords = [];
                pushKeywordIf(keywords, tryStatement.getFirstToken(), 107 /* TryKeyword */);
                if (tryStatement.catchClause) {
                    pushKeywordIf(keywords, tryStatement.catchClause.getFirstToken(), 79 /* CatchKeyword */);
                }
                if (tryStatement.finallyBlock) {
                    var finallyKeyword = ts.findChildOfKind(tryStatement, 92 /* FinallyKeyword */, sourceFile);
                    pushKeywordIf(keywords, finallyKeyword, 92 /* FinallyKeyword */);
                }
                return keywords;
            }
            function getThrowOccurrences(throwStatement, sourceFile) {
                var owner = getThrowStatementOwner(throwStatement);
                if (!owner) {
                    return undefined;
                }
                var keywords = [];
                ts.forEach(aggregateOwnedThrowStatements(owner), function (throwStatement) {
                    keywords.push(ts.findChildOfKind(throwStatement, 105 /* ThrowKeyword */, sourceFile));
                });
                // If the "owner" is a function, then we equate 'return' and 'throw' statements in their
                // ability to "jump out" of the function, and include occurrences for both.
                if (ts.isFunctionBlock(owner)) {
                    ts.forEachReturnStatement(owner, function (returnStatement) {
                        keywords.push(ts.findChildOfKind(returnStatement, 101 /* ReturnKeyword */, sourceFile));
                    });
                }
                return keywords;
            }
            function getReturnOccurrences(returnStatement, sourceFile) {
                var func = ts.getContainingFunction(returnStatement);
                if (!func) {
                    return undefined;
                }
                var keywords = [];
                ts.forEachReturnStatement(ts.cast(func.body, ts.isBlock), function (returnStatement) {
                    keywords.push(ts.findChildOfKind(returnStatement, 101 /* ReturnKeyword */, sourceFile));
                });
                // Include 'throw' statements that do not occur within a try block.
                ts.forEach(aggregateOwnedThrowStatements(func.body), function (throwStatement) {
                    keywords.push(ts.findChildOfKind(throwStatement, 105 /* ThrowKeyword */, sourceFile));
                });
                return keywords;
            }
            function getAsyncAndAwaitOccurrences(node) {
                var func = ts.getContainingFunction(node);
                if (!func) {
                    return undefined;
                }
                var keywords = [];
                if (func.modifiers) {
                    func.modifiers.forEach(function (modifier) {
                        pushKeywordIf(keywords, modifier, 126 /* AsyncKeyword */);
                    });
                }
                ts.forEachChild(func, function (child) {
                    traverseWithoutCrossingFunction(child, function (node) {
                        if (ts.isAwaitExpression(node)) {
                            pushKeywordIf(keywords, node.getFirstToken(), 127 /* AwaitKeyword */);
                        }
                    });
                });
                return keywords;
            }
            function getYieldOccurrences(node) {
                var func = ts.getContainingFunction(node);
                if (!func) {
                    return undefined;
                }
                var keywords = [];
                ts.forEachChild(func, function (child) {
                    traverseWithoutCrossingFunction(child, function (node) {
                        if (ts.isYieldExpression(node)) {
                            pushKeywordIf(keywords, node.getFirstToken(), 121 /* YieldKeyword */);
                        }
                    });
                });
                return keywords;
            }
            // Do not cross function/class/interface/module/type boundaries.
            function traverseWithoutCrossingFunction(node, cb) {
                cb(node);
                if (!ts.isFunctionLike(node) && !ts.isClassLike(node) && !ts.isInterfaceDeclaration(node) && !ts.isModuleDeclaration(node) && !ts.isTypeAliasDeclaration(node) && !ts.isTypeNode(node)) {
                    ts.forEachChild(node, function (child) { return traverseWithoutCrossingFunction(child, cb); });
                }
            }
            function getIfElseOccurrences(ifStatement, sourceFile) {
                var keywords = getIfElseKeywords(ifStatement, sourceFile);
                var result = [];
                // We'd like to highlight else/ifs together if they are only separated by whitespace
                // (i.e. the keywords are separated by no comments, no newlines).
                for (var i = 0; i < keywords.length; i++) {
                    if (keywords[i].kind === 87 /* ElseKeyword */ && i < keywords.length - 1) {
                        var elseKeyword = keywords[i];
                        var ifKeyword = keywords[i + 1]; // this *should* always be an 'if' keyword.
                        var shouldCombineElseAndIf = true;
                        // Avoid recalculating getStart() by iterating backwards.
                        for (var j = ifKeyword.getStart(sourceFile) - 1; j >= elseKeyword.end; j--) {
                            if (!ts.isWhiteSpaceSingleLine(sourceFile.text.charCodeAt(j))) {
                                shouldCombineElseAndIf = false;
                                break;
                            }
                        }
                        if (shouldCombineElseAndIf) {
                            result.push({
                                fileName: sourceFile.fileName,
                                textSpan: ts.createTextSpanFromBounds(elseKeyword.getStart(), ifKeyword.end),
                                kind: "reference" /* reference */
                            });
                            i++; // skip the next keyword
                            continue;
                        }
                    }
                    // Ordinary case: just highlight the keyword.
                    result.push(getHighlightSpanForNode(keywords[i], sourceFile));
                }
                return result;
            }
            function getIfElseKeywords(ifStatement, sourceFile) {
                var keywords = [];
                // Traverse upwards through all parent if-statements linked by their else-branches.
                while (ts.isIfStatement(ifStatement.parent) && ifStatement.parent.elseStatement === ifStatement) {
                    ifStatement = ifStatement.parent;
                }
                // Now traverse back down through the else branches, aggregating if/else keywords of if-statements.
                while (true) {
                    var children = ifStatement.getChildren(sourceFile);
                    pushKeywordIf(keywords, children[0], 95 /* IfKeyword */);
                    // Generally the 'else' keyword is second-to-last, so we traverse backwards.
                    for (var i = children.length - 1; i >= 0; i--) {
                        if (pushKeywordIf(keywords, children[i], 87 /* ElseKeyword */)) {
                            break;
                        }
                    }
                    if (!ifStatement.elseStatement || !ts.isIfStatement(ifStatement.elseStatement)) {
                        break;
                    }
                    ifStatement = ifStatement.elseStatement;
                }
                return keywords;
            }
            /**
             * Whether or not a 'node' is preceded by a label of the given string.
             * Note: 'node' cannot be a SourceFile.
             */
            function isLabeledBy(node, labelName) {
                return !!ts.findAncestor(node.parent, function (owner) { return !ts.isLabeledStatement(owner) ? "quit" : owner.label.escapedText === labelName; });
            }
        })(DocumentHighlights = ts.DocumentHighlights || (ts.DocumentHighlights = {}));
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        function createDocumentRegistry(useCaseSensitiveFileNames, currentDirectory) {
            return createDocumentRegistryInternal(useCaseSensitiveFileNames, currentDirectory);
        }
        ts.createDocumentRegistry = createDocumentRegistry;
        /*@internal*/
        function createDocumentRegistryInternal(useCaseSensitiveFileNames, currentDirectory, externalCache) {
            if (currentDirectory === void 0) {
                currentDirectory = "";
            }
            // Maps from compiler setting target (ES3, ES5, etc.) to all the cached documents we have
            // for those settings.
            var buckets = ts.createMap();
            var getCanonicalFileName = ts.createGetCanonicalFileName(!!useCaseSensitiveFileNames);
            function reportStats() {
                var bucketInfoArray = ts.arrayFrom(buckets.keys()).filter(function (name) { return name && name.charAt(0) === "_"; }).map(function (name) {
                    var entries = buckets.get(name);
                    var sourceFiles = [];
                    entries.forEach(function (entry, name) {
                        sourceFiles.push({
                            name: name,
                            refCount: entry.languageServiceRefCount
                        });
                    });
                    sourceFiles.sort(function (x, y) { return y.refCount - x.refCount; });
                    return {
                        bucket: name,
                        sourceFiles: sourceFiles
                    };
                });
                return JSON.stringify(bucketInfoArray, undefined, 2);
            }
            function acquireDocument(fileName, compilationSettings, scriptSnapshot, version, scriptKind) {
                var path = ts.toPath(fileName, currentDirectory, getCanonicalFileName);
                var key = getKeyForCompilationSettings(compilationSettings);
                return acquireDocumentWithKey(fileName, path, compilationSettings, key, scriptSnapshot, version, scriptKind);
            }
            function acquireDocumentWithKey(fileName, path, compilationSettings, key, scriptSnapshot, version, scriptKind) {
                return acquireOrUpdateDocument(fileName, path, compilationSettings, key, scriptSnapshot, version, /*acquiring*/ true, scriptKind);
            }
            function updateDocument(fileName, compilationSettings, scriptSnapshot, version, scriptKind) {
                var path = ts.toPath(fileName, currentDirectory, getCanonicalFileName);
                var key = getKeyForCompilationSettings(compilationSettings);
                return updateDocumentWithKey(fileName, path, compilationSettings, key, scriptSnapshot, version, scriptKind);
            }
            function updateDocumentWithKey(fileName, path, compilationSettings, key, scriptSnapshot, version, scriptKind) {
                return acquireOrUpdateDocument(fileName, path, compilationSettings, key, scriptSnapshot, version, /*acquiring*/ false, scriptKind);
            }
            function acquireOrUpdateDocument(fileName, path, compilationSettings, key, scriptSnapshot, version, acquiring, scriptKind) {
                var bucket = ts.getOrUpdate(buckets, key, ts.createMap);
                var entry = bucket.get(path);
                var scriptTarget = scriptKind === 6 /* JSON */ ? 100 /* JSON */ : compilationSettings.target || 1 /* ES5 */;
                if (!entry && externalCache) {
                    var sourceFile = externalCache.getDocument(key, path);
                    if (sourceFile) {
                        ts.Debug.assert(acquiring);
                        entry = {
                            sourceFile: sourceFile,
                            languageServiceRefCount: 0
                        };
                        bucket.set(path, entry);
                    }
                }
                if (!entry) {
                    // Have never seen this file with these settings.  Create a new source file for it.
                    var sourceFile = ts.createLanguageServiceSourceFile(fileName, scriptSnapshot, scriptTarget, version, /*setNodeParents*/ false, scriptKind);
                    if (externalCache) {
                        externalCache.setDocument(key, path, sourceFile);
                    }
                    entry = {
                        sourceFile: sourceFile,
                        languageServiceRefCount: 1,
                    };
                    bucket.set(path, entry);
                }
                else {
                    // We have an entry for this file.  However, it may be for a different version of
                    // the script snapshot.  If so, update it appropriately.  Otherwise, we can just
                    // return it as is.
                    if (entry.sourceFile.version !== version) {
                        entry.sourceFile = ts.updateLanguageServiceSourceFile(entry.sourceFile, scriptSnapshot, version, scriptSnapshot.getChangeRange(entry.sourceFile.scriptSnapshot)); // TODO: GH#18217
                        if (externalCache) {
                            externalCache.setDocument(key, path, entry.sourceFile);
                        }
                    }
                    // If we're acquiring, then this is the first time this LS is asking for this document.
                    // Increase our ref count so we know there's another LS using the document.  If we're
                    // not acquiring, then that means the LS is 'updating' the file instead, and that means
                    // it has already acquired the document previously.  As such, we do not need to increase
                    // the ref count.
                    if (acquiring) {
                        entry.languageServiceRefCount++;
                    }
                }
                ts.Debug.assert(entry.languageServiceRefCount !== 0);
                return entry.sourceFile;
            }
            function releaseDocument(fileName, compilationSettings) {
                var path = ts.toPath(fileName, currentDirectory, getCanonicalFileName);
                var key = getKeyForCompilationSettings(compilationSettings);
                return releaseDocumentWithKey(path, key);
            }
            function releaseDocumentWithKey(path, key) {
                var bucket = ts.Debug.assertDefined(buckets.get(key));
                var entry = bucket.get(path);
                entry.languageServiceRefCount--;
                ts.Debug.assert(entry.languageServiceRefCount >= 0);
                if (entry.languageServiceRefCount === 0) {
                    bucket.delete(path);
                }
            }
            function getLanguageServiceRefCounts(path) {
                return ts.arrayFrom(buckets.entries(), function (_a) {
                    var key = _a[0], bucket = _a[1];
                    var entry = bucket.get(path);
                    return [key, entry && entry.languageServiceRefCount];
                });
            }
            return {
                acquireDocument: acquireDocument,
                acquireDocumentWithKey: acquireDocumentWithKey,
                updateDocument: updateDocument,
                updateDocumentWithKey: updateDocumentWithKey,
                releaseDocument: releaseDocument,
                releaseDocumentWithKey: releaseDocumentWithKey,
                getLanguageServiceRefCounts: getLanguageServiceRefCounts,
                reportStats: reportStats,
                getKeyForCompilationSettings: getKeyForCompilationSettings
            };
        }
        ts.createDocumentRegistryInternal = createDocumentRegistryInternal;
        function getKeyForCompilationSettings(settings) {
            return ts.sourceFileAffectingCompilerOptions.map(function (option) { return ts.getCompilerOptionValue(settings, option); }).join("|");
        }
    })(ts || (ts = {}));
    /* Code for finding imports of an exported symbol. Used only by FindAllReferences. */
    /* @internal */
    var ts;
    (function (ts) {
        var FindAllReferences;
        (function (FindAllReferences) {
            /** Creates the imports map and returns an ImportTracker that uses it. Call this lazily to avoid calling `getDirectImportsMap` unnecessarily.  */
            function createImportTracker(sourceFiles, sourceFilesSet, checker, cancellationToken) {
                var allDirectImports = getDirectImportsMap(sourceFiles, checker, cancellationToken);
                return function (exportSymbol, exportInfo, isForRename) {
                    var _a = getImportersForExport(sourceFiles, sourceFilesSet, allDirectImports, exportInfo, checker, cancellationToken), directImports = _a.directImports, indirectUsers = _a.indirectUsers;
                    return __assign({ indirectUsers: indirectUsers }, getSearchesFromDirectImports(directImports, exportSymbol, exportInfo.exportKind, checker, isForRename));
                };
            }
            FindAllReferences.createImportTracker = createImportTracker;
            var ExportKind;
            (function (ExportKind) {
                ExportKind[ExportKind["Named"] = 0] = "Named";
                ExportKind[ExportKind["Default"] = 1] = "Default";
                ExportKind[ExportKind["ExportEquals"] = 2] = "ExportEquals";
            })(ExportKind = FindAllReferences.ExportKind || (FindAllReferences.ExportKind = {}));
            var ImportExport;
            (function (ImportExport) {
                ImportExport[ImportExport["Import"] = 0] = "Import";
                ImportExport[ImportExport["Export"] = 1] = "Export";
            })(ImportExport = FindAllReferences.ImportExport || (FindAllReferences.ImportExport = {}));
            /** Returns import statements that directly reference the exporting module, and a list of files that may access the module through a namespace. */
            function getImportersForExport(sourceFiles, sourceFilesSet, allDirectImports, _a, checker, cancellationToken) {
                var exportingModuleSymbol = _a.exportingModuleSymbol, exportKind = _a.exportKind;
                var markSeenDirectImport = ts.nodeSeenTracker();
                var markSeenIndirectUser = ts.nodeSeenTracker();
                var directImports = [];
                var isAvailableThroughGlobal = !!exportingModuleSymbol.globalExports;
                var indirectUserDeclarations = isAvailableThroughGlobal ? undefined : [];
                handleDirectImports(exportingModuleSymbol);
                return { directImports: directImports, indirectUsers: getIndirectUsers() };
                function getIndirectUsers() {
                    if (isAvailableThroughGlobal) {
                        // It has `export as namespace`, so anything could potentially use it.
                        return sourceFiles;
                    }
                    // Module augmentations may use this module's exports without importing it.
                    for (var _i = 0, _a = exportingModuleSymbol.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        if (ts.isExternalModuleAugmentation(decl) && sourceFilesSet.has(decl.getSourceFile().fileName)) {
                            addIndirectUser(decl);
                        }
                    }
                    // This may return duplicates (if there are multiple module declarations in a single source file, all importing the same thing as a namespace), but `State.markSearchedSymbol` will handle that.
                    return indirectUserDeclarations.map(ts.getSourceFileOfNode);
                }
                function handleDirectImports(exportingModuleSymbol) {
                    var theseDirectImports = getDirectImports(exportingModuleSymbol);
                    if (theseDirectImports) {
                        for (var _i = 0, theseDirectImports_1 = theseDirectImports; _i < theseDirectImports_1.length; _i++) {
                            var direct = theseDirectImports_1[_i];
                            if (!markSeenDirectImport(direct)) {
                                continue;
                            }
                            if (cancellationToken)
                                cancellationToken.throwIfCancellationRequested();
                            switch (direct.kind) {
                                case 196 /* CallExpression */:
                                    if (!isAvailableThroughGlobal) {
                                        var parent = direct.parent;
                                        if (exportKind === 2 /* ExportEquals */ && parent.kind === 242 /* VariableDeclaration */) {
                                            var name = parent.name;
                                            if (name.kind === 75 /* Identifier */) {
                                                directImports.push(name);
                                                break;
                                            }
                                        }
                                        // Don't support re-exporting 'require()' calls, so just add a single indirect user.
                                        addIndirectUser(direct.getSourceFile());
                                    }
                                    break;
                                case 75 /* Identifier */: // for 'const x = require("y");
                                    break; // TODO: GH#23879
                                case 253 /* ImportEqualsDeclaration */:
                                    handleNamespaceImport(direct, direct.name, ts.hasModifier(direct, 1 /* Export */), /*alreadyAddedDirect*/ false);
                                    break;
                                case 254 /* ImportDeclaration */:
                                    directImports.push(direct);
                                    var namedBindings = direct.importClause && direct.importClause.namedBindings;
                                    if (namedBindings && namedBindings.kind === 256 /* NamespaceImport */) {
                                        handleNamespaceImport(direct, namedBindings.name, /*isReExport*/ false, /*alreadyAddedDirect*/ true);
                                    }
                                    else if (!isAvailableThroughGlobal && ts.isDefaultImport(direct)) {
                                        addIndirectUser(getSourceFileLikeForImportDeclaration(direct)); // Add a check for indirect uses to handle synthetic default imports
                                    }
                                    break;
                                case 260 /* ExportDeclaration */:
                                    if (!direct.exportClause) {
                                        // This is `export * from "foo"`, so imports of this module may import the export too.
                                        handleDirectImports(getContainingModuleSymbol(direct, checker));
                                    }
                                    else {
                                        // This is `export { foo } from "foo"` and creates an alias symbol, so recursive search will get handle re-exports.
                                        directImports.push(direct);
                                    }
                                    break;
                                case 188 /* ImportType */:
                                    directImports.push(direct);
                                    break;
                                default:
                                    ts.Debug.failBadSyntaxKind(direct, "Unexpected import kind.");
                            }
                        }
                    }
                }
                function handleNamespaceImport(importDeclaration, name, isReExport, alreadyAddedDirect) {
                    if (exportKind === 2 /* ExportEquals */) {
                        // This is a direct import, not import-as-namespace.
                        if (!alreadyAddedDirect)
                            directImports.push(importDeclaration);
                    }
                    else if (!isAvailableThroughGlobal) {
                        var sourceFileLike = getSourceFileLikeForImportDeclaration(importDeclaration);
                        ts.Debug.assert(sourceFileLike.kind === 290 /* SourceFile */ || sourceFileLike.kind === 249 /* ModuleDeclaration */);
                        if (isReExport || findNamespaceReExports(sourceFileLike, name, checker)) {
                            addIndirectUsers(sourceFileLike);
                        }
                        else {
                            addIndirectUser(sourceFileLike);
                        }
                    }
                }
                function addIndirectUser(sourceFileLike) {
                    ts.Debug.assert(!isAvailableThroughGlobal);
                    var isNew = markSeenIndirectUser(sourceFileLike);
                    if (isNew) {
                        indirectUserDeclarations.push(sourceFileLike); // TODO: GH#18217
                    }
                    return isNew;
                }
                /** Adds a module and all of its transitive dependencies as possible indirect users. */
                function addIndirectUsers(sourceFileLike) {
                    if (!addIndirectUser(sourceFileLike)) {
                        return;
                    }
                    var moduleSymbol = checker.getMergedSymbol(sourceFileLike.symbol);
                    ts.Debug.assert(!!(moduleSymbol.flags & 1536 /* Module */));
                    var directImports = getDirectImports(moduleSymbol);
                    if (directImports) {
                        for (var _i = 0, directImports_1 = directImports; _i < directImports_1.length; _i++) {
                            var directImport = directImports_1[_i];
                            if (!ts.isImportTypeNode(directImport)) {
                                addIndirectUsers(getSourceFileLikeForImportDeclaration(directImport));
                            }
                        }
                    }
                }
                function getDirectImports(moduleSymbol) {
                    return allDirectImports.get(ts.getSymbolId(moduleSymbol).toString());
                }
            }
            /**
             * Given the set of direct imports of a module, we need to find which ones import the particular exported symbol.
             * The returned `importSearches` will result in the entire source file being searched.
             * But re-exports will be placed in 'singleReferences' since they cannot be locally referenced.
             */
            function getSearchesFromDirectImports(directImports, exportSymbol, exportKind, checker, isForRename) {
                var importSearches = [];
                var singleReferences = [];
                function addSearch(location, symbol) {
                    importSearches.push([location, symbol]);
                }
                if (directImports) {
                    for (var _i = 0, directImports_2 = directImports; _i < directImports_2.length; _i++) {
                        var decl = directImports_2[_i];
                        handleImport(decl);
                    }
                }
                return { importSearches: importSearches, singleReferences: singleReferences };
                function handleImport(decl) {
                    if (decl.kind === 253 /* ImportEqualsDeclaration */) {
                        if (isExternalModuleImportEquals(decl)) {
                            handleNamespaceImportLike(decl.name);
                        }
                        return;
                    }
                    if (decl.kind === 75 /* Identifier */) {
                        handleNamespaceImportLike(decl);
                        return;
                    }
                    if (decl.kind === 188 /* ImportType */) {
                        if (decl.qualifier) {
                            var firstIdentifier = ts.getFirstIdentifier(decl.qualifier);
                            if (firstIdentifier.escapedText === ts.symbolName(exportSymbol)) {
                                singleReferences.push(firstIdentifier);
                            }
                        }
                        else if (exportKind === 2 /* ExportEquals */) {
                            singleReferences.push(decl.argument.literal);
                        }
                        return;
                    }
                    // Ignore if there's a grammar error
                    if (decl.moduleSpecifier.kind !== 10 /* StringLiteral */) {
                        return;
                    }
                    if (decl.kind === 260 /* ExportDeclaration */) {
                        if (decl.exportClause && ts.isNamedExports(decl.exportClause)) {
                            searchForNamedImport(decl.exportClause);
                        }
                        return;
                    }
                    var _a = decl.importClause || { name: undefined, namedBindings: undefined }, name = _a.name, namedBindings = _a.namedBindings;
                    if (namedBindings) {
                        switch (namedBindings.kind) {
                            case 256 /* NamespaceImport */:
                                handleNamespaceImportLike(namedBindings.name);
                                break;
                            case 257 /* NamedImports */:
                                // 'default' might be accessed as a named import `{ default as foo }`.
                                if (exportKind === 0 /* Named */ || exportKind === 1 /* Default */) {
                                    searchForNamedImport(namedBindings);
                                }
                                break;
                            default:
                                ts.Debug.assertNever(namedBindings);
                        }
                    }
                    // `export =` might be imported by a default import if `--allowSyntheticDefaultImports` is on, so this handles both ExportKind.Default and ExportKind.ExportEquals.
                    // If a default import has the same name as the default export, allow to rename it.
                    // Given `import f` and `export default function f`, we will rename both, but for `import g` we will rename just that.
                    if (name && (exportKind === 1 /* Default */ || exportKind === 2 /* ExportEquals */) && (!isForRename || name.escapedText === ts.symbolEscapedNameNoDefault(exportSymbol))) {
                        var defaultImportAlias = checker.getSymbolAtLocation(name);
                        addSearch(name, defaultImportAlias);
                    }
                }
                /**
                 * `import x = require("./x")` or `import * as x from "./x"`.
                 * An `export =` may be imported by this syntax, so it may be a direct import.
                 * If it's not a direct import, it will be in `indirectUsers`, so we don't have to do anything here.
                 */
                function handleNamespaceImportLike(importName) {
                    // Don't rename an import that already has a different name than the export.
                    if (exportKind === 2 /* ExportEquals */ && (!isForRename || isNameMatch(importName.escapedText))) {
                        addSearch(importName, checker.getSymbolAtLocation(importName));
                    }
                }
                function searchForNamedImport(namedBindings) {
                    if (!namedBindings) {
                        return;
                    }
                    for (var _i = 0, _a = namedBindings.elements; _i < _a.length; _i++) {
                        var element = _a[_i];
                        var name = element.name, propertyName = element.propertyName;
                        if (!isNameMatch((propertyName || name).escapedText)) {
                            continue;
                        }
                        if (propertyName) {
                            // This is `import { foo as bar } from "./a"` or `export { foo as bar } from "./a"`. `foo` isn't a local in the file, so just add it as a single reference.
                            singleReferences.push(propertyName);
                            // If renaming `{ foo as bar }`, don't touch `bar`, just `foo`.
                            // But do rename `foo` in ` { default as foo }` if that's the original export name.
                            if (!isForRename || name.escapedText === exportSymbol.escapedName) {
                                // Search locally for `bar`.
                                addSearch(name, checker.getSymbolAtLocation(name));
                            }
                        }
                        else {
                            var localSymbol = element.kind === 263 /* ExportSpecifier */ && element.propertyName
                                ? checker.getExportSpecifierLocalTargetSymbol(element) // For re-exporting under a different name, we want to get the re-exported symbol.
                                : checker.getSymbolAtLocation(name);
                            addSearch(name, localSymbol);
                        }
                    }
                }
                function isNameMatch(name) {
                    // Use name of "default" even in `export =` case because we may have allowSyntheticDefaultImports
                    return name === exportSymbol.escapedName || exportKind !== 0 /* Named */ && name === "default" /* Default */;
                }
            }
            /** Returns 'true' is the namespace 'name' is re-exported from this module, and 'false' if it is only used locally. */
            function findNamespaceReExports(sourceFileLike, name, checker) {
                var namespaceImportSymbol = checker.getSymbolAtLocation(name);
                return !!forEachPossibleImportOrExportStatement(sourceFileLike, function (statement) {
                    if (!ts.isExportDeclaration(statement))
                        return;
                    var exportClause = statement.exportClause, moduleSpecifier = statement.moduleSpecifier;
                    return !moduleSpecifier && exportClause && ts.isNamedExports(exportClause) &&
                        exportClause.elements.some(function (element) { return checker.getExportSpecifierLocalTargetSymbol(element) === namespaceImportSymbol; });
                });
            }
            function findModuleReferences(program, sourceFiles, searchModuleSymbol) {
                var refs = [];
                var checker = program.getTypeChecker();
                for (var _i = 0, sourceFiles_1 = sourceFiles; _i < sourceFiles_1.length; _i++) {
                    var referencingFile = sourceFiles_1[_i];
                    var searchSourceFile = searchModuleSymbol.valueDeclaration;
                    if (searchSourceFile.kind === 290 /* SourceFile */) {
                        for (var _a = 0, _b = referencingFile.referencedFiles; _a < _b.length; _a++) {
                            var ref = _b[_a];
                            if (program.getSourceFileFromReference(referencingFile, ref) === searchSourceFile) {
                                refs.push({ kind: "reference", referencingFile: referencingFile, ref: ref });
                            }
                        }
                        for (var _c = 0, _d = referencingFile.typeReferenceDirectives; _c < _d.length; _c++) {
                            var ref = _d[_c];
                            var referenced = program.getResolvedTypeReferenceDirectives().get(ref.fileName);
                            if (referenced !== undefined && referenced.resolvedFileName === searchSourceFile.fileName) {
                                refs.push({ kind: "reference", referencingFile: referencingFile, ref: ref });
                            }
                        }
                    }
                    forEachImport(referencingFile, function (_importDecl, moduleSpecifier) {
                        var moduleSymbol = checker.getSymbolAtLocation(moduleSpecifier);
                        if (moduleSymbol === searchModuleSymbol) {
                            refs.push({ kind: "import", literal: moduleSpecifier });
                        }
                    });
                }
                return refs;
            }
            FindAllReferences.findModuleReferences = findModuleReferences;
            /** Returns a map from a module symbol Id to all import statements that directly reference the module. */
            function getDirectImportsMap(sourceFiles, checker, cancellationToken) {
                var map = ts.createMap();
                for (var _i = 0, sourceFiles_2 = sourceFiles; _i < sourceFiles_2.length; _i++) {
                    var sourceFile = sourceFiles_2[_i];
                    if (cancellationToken)
                        cancellationToken.throwIfCancellationRequested();
                    forEachImport(sourceFile, function (importDecl, moduleSpecifier) {
                        var moduleSymbol = checker.getSymbolAtLocation(moduleSpecifier);
                        if (moduleSymbol) {
                            var id = ts.getSymbolId(moduleSymbol).toString();
                            var imports = map.get(id);
                            if (!imports) {
                                map.set(id, imports = []);
                            }
                            imports.push(importDecl);
                        }
                    });
                }
                return map;
            }
            /** Iterates over all statements at the top level or in module declarations. Returns the first truthy result. */
            function forEachPossibleImportOrExportStatement(sourceFileLike, action) {
                return ts.forEach(sourceFileLike.kind === 290 /* SourceFile */ ? sourceFileLike.statements : sourceFileLike.body.statements, function (statement) {
                    return action(statement) || (isAmbientModuleDeclaration(statement) && ts.forEach(statement.body && statement.body.statements, action));
                });
            }
            /** Calls `action` for each import, re-export, or require() in a file. */
            function forEachImport(sourceFile, action) {
                if (sourceFile.externalModuleIndicator || sourceFile.imports !== undefined) {
                    for (var _i = 0, _a = sourceFile.imports; _i < _a.length; _i++) {
                        var i = _a[_i];
                        action(ts.importFromModuleSpecifier(i), i);
                    }
                }
                else {
                    forEachPossibleImportOrExportStatement(sourceFile, function (statement) {
                        switch (statement.kind) {
                            case 260 /* ExportDeclaration */:
                            case 254 /* ImportDeclaration */: {
                                var decl = statement;
                                if (decl.moduleSpecifier && ts.isStringLiteral(decl.moduleSpecifier)) {
                                    action(decl, decl.moduleSpecifier);
                                }
                                break;
                            }
                            case 253 /* ImportEqualsDeclaration */: {
                                var decl = statement;
                                if (isExternalModuleImportEquals(decl)) {
                                    action(decl, decl.moduleReference.expression);
                                }
                                break;
                            }
                        }
                    });
                }
            }
            /**
             * Given a local reference, we might notice that it's an import/export and recursively search for references of that.
             * If at an import, look locally for the symbol it imports.
             * If at an export, look for all imports of it.
             * This doesn't handle export specifiers; that is done in `getReferencesAtExportSpecifier`.
             * @param comingFromExport If we are doing a search for all exports, don't bother looking backwards for the imported symbol, since that's the reason we're here.
             */
            function getImportOrExportSymbol(node, symbol, checker, comingFromExport) {
                return comingFromExport ? getExport() : getExport() || getImport();
                function getExport() {
                    var parent = node.parent;
                    var grandParent = parent.parent;
                    if (symbol.exportSymbol) {
                        if (parent.kind === 194 /* PropertyAccessExpression */) {
                            // When accessing an export of a JS module, there's no alias. The symbol will still be flagged as an export even though we're at the use.
                            // So check that we are at the declaration.
                            return symbol.declarations.some(function (d) { return d === parent; }) && ts.isBinaryExpression(grandParent)
                                ? getSpecialPropertyExport(grandParent, /*useLhsSymbol*/ false)
                                : undefined;
                        }
                        else {
                            return exportInfo(symbol.exportSymbol, getExportKindForDeclaration(parent));
                        }
                    }
                    else {
                        var exportNode = getExportNode(parent, node);
                        if (exportNode && ts.hasModifier(exportNode, 1 /* Export */)) {
                            if (ts.isImportEqualsDeclaration(exportNode) && exportNode.moduleReference === node) {
                                // We're at `Y` in `export import X = Y`. This is not the exported symbol, the left-hand-side is. So treat this as an import statement.
                                if (comingFromExport) {
                                    return undefined;
                                }
                                var lhsSymbol = checker.getSymbolAtLocation(exportNode.name);
                                return { kind: 0 /* Import */, symbol: lhsSymbol };
                            }
                            else {
                                return exportInfo(symbol, getExportKindForDeclaration(exportNode));
                            }
                        }
                        // If we are in `export = a;` or `export default a;`, `parent` is the export assignment.
                        else if (ts.isExportAssignment(parent)) {
                            return getExportAssignmentExport(parent);
                        }
                        // If we are in `export = class A {};` (or `export = class A {};`) at `A`, `parent.parent` is the export assignment.
                        else if (ts.isExportAssignment(grandParent)) {
                            return getExportAssignmentExport(grandParent);
                        }
                        // Similar for `module.exports =` and `exports.A =`.
                        else if (ts.isBinaryExpression(parent)) {
                            return getSpecialPropertyExport(parent, /*useLhsSymbol*/ true);
                        }
                        else if (ts.isBinaryExpression(grandParent)) {
                            return getSpecialPropertyExport(grandParent, /*useLhsSymbol*/ true);
                        }
                        else if (ts.isJSDocTypedefTag(parent)) {
                            return exportInfo(symbol, 0 /* Named */);
                        }
                    }
                    function getExportAssignmentExport(ex) {
                        // Get the symbol for the `export =` node; its parent is the module it's the export of.
                        var exportingModuleSymbol = ts.Debug.assertDefined(ex.symbol.parent, "Expected export symbol to have a parent");
                        var exportKind = ex.isExportEquals ? 2 /* ExportEquals */ : 1 /* Default */;
                        return { kind: 1 /* Export */, symbol: symbol, exportInfo: { exportingModuleSymbol: exportingModuleSymbol, exportKind: exportKind } };
                    }
                    function getSpecialPropertyExport(node, useLhsSymbol) {
                        var kind;
                        switch (ts.getAssignmentDeclarationKind(node)) {
                            case 1 /* ExportsProperty */:
                                kind = 0 /* Named */;
                                break;
                            case 2 /* ModuleExports */:
                                kind = 2 /* ExportEquals */;
                                break;
                            default:
                                return undefined;
                        }
                        var sym = useLhsSymbol ? checker.getSymbolAtLocation(ts.cast(node.left, ts.isPropertyAccessExpression).name) : symbol;
                        // Better detection for GH#20803
                        if (sym && !(checker.getMergedSymbol(sym.parent).flags & 1536 /* Module */)) {
                            ts.Debug.fail("Special property assignment kind does not have a module as its parent. Assignment is " + ts.Debug.formatSymbol(sym) + ", parent is " + ts.Debug.formatSymbol(sym.parent));
                        }
                        return sym && exportInfo(sym, kind);
                    }
                }
                function getImport() {
                    var isImport = isNodeImport(node);
                    if (!isImport)
                        return undefined;
                    // A symbol being imported is always an alias. So get what that aliases to find the local symbol.
                    var importedSymbol = checker.getImmediateAliasedSymbol(symbol);
                    if (!importedSymbol)
                        return undefined;
                    // Search on the local symbol in the exporting module, not the exported symbol.
                    importedSymbol = skipExportSpecifierSymbol(importedSymbol, checker);
                    // Similarly, skip past the symbol for 'export ='
                    if (importedSymbol.escapedName === "export=") {
                        importedSymbol = getExportEqualsLocalSymbol(importedSymbol, checker);
                    }
                    // If the import has a different name than the export, do not continue searching.
                    // If `importedName` is undefined, do continue searching as the export is anonymous.
                    // (All imports returned from this function will be ignored anyway if we are in rename and this is a not a named export.)
                    var importedName = ts.symbolEscapedNameNoDefault(importedSymbol);
                    if (importedName === undefined || importedName === "default" /* Default */ || importedName === symbol.escapedName) {
                        return { kind: 0 /* Import */, symbol: importedSymbol };
                    }
                }
                function exportInfo(symbol, kind) {
                    var exportInfo = getExportInfo(symbol, kind, checker);
                    return exportInfo && { kind: 1 /* Export */, symbol: symbol, exportInfo: exportInfo };
                }
                // Not meant for use with export specifiers or export assignment.
                function getExportKindForDeclaration(node) {
                    return ts.hasModifier(node, 512 /* Default */) ? 1 /* Default */ : 0 /* Named */;
                }
            }
            FindAllReferences.getImportOrExportSymbol = getImportOrExportSymbol;
            function getExportEqualsLocalSymbol(importedSymbol, checker) {
                if (importedSymbol.flags & 2097152 /* Alias */) {
                    return ts.Debug.assertDefined(checker.getImmediateAliasedSymbol(importedSymbol));
                }
                var decl = importedSymbol.valueDeclaration;
                if (ts.isExportAssignment(decl)) { // `export = class {}`
                    return ts.Debug.assertDefined(decl.expression.symbol);
                }
                else if (ts.isBinaryExpression(decl)) { // `module.exports = class {}`
                    return ts.Debug.assertDefined(decl.right.symbol);
                }
                else if (ts.isSourceFile(decl)) { // json module
                    return ts.Debug.assertDefined(decl.symbol);
                }
                return ts.Debug.fail();
            }
            // If a reference is a class expression, the exported node would be its parent.
            // If a reference is a variable declaration, the exported node would be the variable statement.
            function getExportNode(parent, node) {
                var declaration = ts.isVariableDeclaration(parent) ? parent : ts.isBindingElement(parent) ? ts.walkUpBindingElementsAndPatterns(parent) : undefined;
                if (declaration) {
                    return parent.name !== node ? undefined :
                        ts.isCatchClause(declaration.parent) ? undefined : ts.isVariableStatement(declaration.parent.parent) ? declaration.parent.parent : undefined;
                }
                else {
                    return parent;
                }
            }
            function isNodeImport(node) {
                var parent = node.parent;
                switch (parent.kind) {
                    case 253 /* ImportEqualsDeclaration */:
                        return parent.name === node && isExternalModuleImportEquals(parent);
                    case 258 /* ImportSpecifier */:
                        // For a rename import `{ foo as bar }`, don't search for the imported symbol. Just find local uses of `bar`.
                        return !parent.propertyName;
                    case 255 /* ImportClause */:
                    case 256 /* NamespaceImport */:
                        ts.Debug.assert(parent.name === node);
                        return true;
                    default:
                        return false;
                }
            }
            function getExportInfo(exportSymbol, exportKind, checker) {
                var moduleSymbol = exportSymbol.parent;
                if (!moduleSymbol)
                    return undefined; // This can happen if an `export` is not at the top-level (which is a compile error).
                var exportingModuleSymbol = checker.getMergedSymbol(moduleSymbol); // Need to get merged symbol in case there's an augmentation.
                // `export` may appear in a namespace. In that case, just rely on global search.
                return ts.isExternalModuleSymbol(exportingModuleSymbol) ? { exportingModuleSymbol: exportingModuleSymbol, exportKind: exportKind } : undefined;
            }
            FindAllReferences.getExportInfo = getExportInfo;
            /** If at an export specifier, go to the symbol it refers to. */
            function skipExportSpecifierSymbol(symbol, checker) {
                // For `export { foo } from './bar", there's nothing to skip, because it does not create a new alias. But `export { foo } does.
                if (symbol.declarations) {
                    for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                        var declaration = _a[_i];
                        if (ts.isExportSpecifier(declaration) && !declaration.propertyName && !declaration.parent.parent.moduleSpecifier) {
                            return checker.getExportSpecifierLocalTargetSymbol(declaration);
                        }
                    }
                }
                return symbol;
            }
            function getContainingModuleSymbol(importer, checker) {
                return checker.getMergedSymbol(getSourceFileLikeForImportDeclaration(importer).symbol);
            }
            function getSourceFileLikeForImportDeclaration(node) {
                if (node.kind === 196 /* CallExpression */) {
                    return node.getSourceFile();
                }
                var parent = node.parent;
                if (parent.kind === 290 /* SourceFile */) {
                    return parent;
                }
                ts.Debug.assert(parent.kind === 250 /* ModuleBlock */);
                return ts.cast(parent.parent, isAmbientModuleDeclaration);
            }
            function isAmbientModuleDeclaration(node) {
                return node.kind === 249 /* ModuleDeclaration */ && node.name.kind === 10 /* StringLiteral */;
            }
            function isExternalModuleImportEquals(eq) {
                return eq.moduleReference.kind === 265 /* ExternalModuleReference */ && eq.moduleReference.expression.kind === 10 /* StringLiteral */;
            }
        })(FindAllReferences = ts.FindAllReferences || (ts.FindAllReferences = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var FindAllReferences;
        (function (FindAllReferences) {
            var DefinitionKind;
            (function (DefinitionKind) {
                DefinitionKind[DefinitionKind["Symbol"] = 0] = "Symbol";
                DefinitionKind[DefinitionKind["Label"] = 1] = "Label";
                DefinitionKind[DefinitionKind["Keyword"] = 2] = "Keyword";
                DefinitionKind[DefinitionKind["This"] = 3] = "This";
                DefinitionKind[DefinitionKind["String"] = 4] = "String";
            })(DefinitionKind = FindAllReferences.DefinitionKind || (FindAllReferences.DefinitionKind = {}));
            var EntryKind;
            (function (EntryKind) {
                EntryKind[EntryKind["Span"] = 0] = "Span";
                EntryKind[EntryKind["Node"] = 1] = "Node";
                EntryKind[EntryKind["StringLiteral"] = 2] = "StringLiteral";
                EntryKind[EntryKind["SearchedLocalFoundProperty"] = 3] = "SearchedLocalFoundProperty";
                EntryKind[EntryKind["SearchedPropertyFoundLocal"] = 4] = "SearchedPropertyFoundLocal";
            })(EntryKind = FindAllReferences.EntryKind || (FindAllReferences.EntryKind = {}));
            function nodeEntry(node, kind) {
                if (kind === void 0) {
                    kind = 1 /* Node */;
                }
                return {
                    kind: kind,
                    node: node.name || node,
                    context: getContextNodeForNodeEntry(node)
                };
            }
            FindAllReferences.nodeEntry = nodeEntry;
            function isContextWithStartAndEndNode(node) {
                return node && node.kind === undefined;
            }
            FindAllReferences.isContextWithStartAndEndNode = isContextWithStartAndEndNode;
            function getContextNodeForNodeEntry(node) {
                if (ts.isDeclaration(node)) {
                    return getContextNode(node);
                }
                if (!node.parent)
                    return undefined;
                if (!ts.isDeclaration(node.parent) && !ts.isExportAssignment(node.parent)) {
                    // Special property assignment in javascript
                    if (ts.isInJSFile(node)) {
                        var binaryExpression = ts.isBinaryExpression(node.parent) ?
                            node.parent :
                            ts.isAccessExpression(node.parent) &&
                                ts.isBinaryExpression(node.parent.parent) &&
                                node.parent.parent.left === node.parent ?
                                node.parent.parent :
                                undefined;
                        if (binaryExpression && ts.getAssignmentDeclarationKind(binaryExpression) !== 0 /* None */) {
                            return getContextNode(binaryExpression);
                        }
                    }
                    // Jsx Tags
                    if (ts.isJsxOpeningElement(node.parent) || ts.isJsxClosingElement(node.parent)) {
                        return node.parent.parent;
                    }
                    else if (ts.isJsxSelfClosingElement(node.parent) ||
                        ts.isLabeledStatement(node.parent) ||
                        ts.isBreakOrContinueStatement(node.parent)) {
                        return node.parent;
                    }
                    else if (ts.isStringLiteralLike(node)) {
                        var validImport = ts.tryGetImportFromModuleSpecifier(node);
                        if (validImport) {
                            var declOrStatement = ts.findAncestor(validImport, function (node) {
                                return ts.isDeclaration(node) ||
                                    ts.isStatement(node) ||
                                    ts.isJSDocTag(node);
                            });
                            return ts.isDeclaration(declOrStatement) ?
                                getContextNode(declOrStatement) :
                                declOrStatement;
                        }
                    }
                    // Handle computed property name
                    var propertyName = ts.findAncestor(node, ts.isComputedPropertyName);
                    return propertyName ?
                        getContextNode(propertyName.parent) :
                        undefined;
                }
                if (node.parent.name === node || // node is name of declaration, use parent
                    ts.isConstructorDeclaration(node.parent) ||
                    ts.isExportAssignment(node.parent) ||
                    // Property name of the import export specifier or binding pattern, use parent
                    ((ts.isImportOrExportSpecifier(node.parent) || ts.isBindingElement(node.parent))
                        && node.parent.propertyName === node) ||
                    // Is default export
                    (node.kind === 84 /* DefaultKeyword */ && ts.hasModifier(node.parent, 513 /* ExportDefault */))) {
                    return getContextNode(node.parent);
                }
                return undefined;
            }
            function getContextNode(node) {
                if (!node)
                    return undefined;
                switch (node.kind) {
                    case 242 /* VariableDeclaration */:
                        return !ts.isVariableDeclarationList(node.parent) || node.parent.declarations.length !== 1 ?
                            node :
                            ts.isVariableStatement(node.parent.parent) ?
                                node.parent.parent :
                                ts.isForInOrOfStatement(node.parent.parent) ?
                                    getContextNode(node.parent.parent) :
                                    node.parent;
                    case 191 /* BindingElement */:
                        return getContextNode(node.parent.parent);
                    case 258 /* ImportSpecifier */:
                        return node.parent.parent.parent;
                    case 263 /* ExportSpecifier */:
                    case 256 /* NamespaceImport */:
                        return node.parent.parent;
                    case 255 /* ImportClause */:
                        return node.parent;
                    case 209 /* BinaryExpression */:
                        return ts.isExpressionStatement(node.parent) ?
                            node.parent :
                            node;
                    case 232 /* ForOfStatement */:
                    case 231 /* ForInStatement */:
                        return {
                            start: node.initializer,
                            end: node.expression
                        };
                    case 281 /* PropertyAssignment */:
                    case 282 /* ShorthandPropertyAssignment */:
                        return ts.isArrayLiteralOrObjectLiteralDestructuringPattern(node.parent) ?
                            getContextNode(ts.findAncestor(node.parent, function (node) {
                                return ts.isBinaryExpression(node) || ts.isForInOrOfStatement(node);
                            })) :
                            node;
                    default:
                        return node;
                }
            }
            FindAllReferences.getContextNode = getContextNode;
            function toContextSpan(textSpan, sourceFile, context) {
                if (!context)
                    return undefined;
                var contextSpan = isContextWithStartAndEndNode(context) ?
                    getTextSpan(context.start, sourceFile, context.end) :
                    getTextSpan(context, sourceFile);
                return contextSpan.start !== textSpan.start || contextSpan.length !== textSpan.length ?
                    { contextSpan: contextSpan } :
                    undefined;
            }
            FindAllReferences.toContextSpan = toContextSpan;
            var FindReferencesUse;
            (function (FindReferencesUse) {
                /**
                 * When searching for references to a symbol, the location will not be adjusted (this is the default behavior when not specified).
                 */
                FindReferencesUse[FindReferencesUse["Other"] = 0] = "Other";
                /**
                 * When searching for references to a symbol, the location will be adjusted if the cursor was on a keyword.
                 */
                FindReferencesUse[FindReferencesUse["References"] = 1] = "References";
                /**
                 * When searching for references to a symbol, the location will be adjusted if the cursor was on a keyword.
                 * Unlike `References`, the location will only be adjusted keyword belonged to a declaration with a valid name.
                 * If set, we will find fewer references -- if it is referenced by several different names, we still only find references for the original name.
                 */
                FindReferencesUse[FindReferencesUse["Rename"] = 2] = "Rename";
            })(FindReferencesUse = FindAllReferences.FindReferencesUse || (FindAllReferences.FindReferencesUse = {}));
            function findReferencedSymbols(program, cancellationToken, sourceFiles, sourceFile, position) {
                var node = ts.getTouchingPropertyName(sourceFile, position);
                var referencedSymbols = Core.getReferencedSymbolsForNode(position, node, program, sourceFiles, cancellationToken, { use: 1 /* References */ });
                var checker = program.getTypeChecker();
                return !referencedSymbols || !referencedSymbols.length ? undefined : ts.mapDefined(referencedSymbols, function (_a) {
                    var definition = _a.definition, references = _a.references;
                    // Only include referenced symbols that have a valid definition.
                    return definition && {
                        definition: checker.runWithCancellationToken(cancellationToken, function (checker) { return definitionToReferencedSymbolDefinitionInfo(definition, checker, node); }),
                        references: references.map(toReferenceEntry)
                    };
                });
            }
            FindAllReferences.findReferencedSymbols = findReferencedSymbols;
            function getImplementationsAtPosition(program, cancellationToken, sourceFiles, sourceFile, position) {
                var node = ts.getTouchingPropertyName(sourceFile, position);
                var referenceEntries = getImplementationReferenceEntries(program, cancellationToken, sourceFiles, node, position);
                var checker = program.getTypeChecker();
                return ts.map(referenceEntries, function (entry) { return toImplementationLocation(entry, checker); });
            }
            FindAllReferences.getImplementationsAtPosition = getImplementationsAtPosition;
            function getImplementationReferenceEntries(program, cancellationToken, sourceFiles, node, position) {
                if (node.kind === 290 /* SourceFile */) {
                    return undefined;
                }
                var checker = program.getTypeChecker();
                // If invoked directly on a shorthand property assignment, then return
                // the declaration of the symbol being assigned (not the symbol being assigned to).
                if (node.parent.kind === 282 /* ShorthandPropertyAssignment */) {
                    var result_1 = [];
                    Core.getReferenceEntriesForShorthandPropertyAssignment(node, checker, function (node) { return result_1.push(nodeEntry(node)); });
                    return result_1;
                }
                else if (node.kind === 102 /* SuperKeyword */ || ts.isSuperProperty(node.parent)) {
                    // References to and accesses on the super keyword only have one possible implementation, so no
                    // need to "Find all References"
                    var symbol = checker.getSymbolAtLocation(node);
                    return symbol.valueDeclaration && [nodeEntry(symbol.valueDeclaration)];
                }
                else {
                    // Perform "Find all References" and retrieve only those that are implementations
                    return getReferenceEntriesForNode(position, node, program, sourceFiles, cancellationToken, { implementations: true, use: 1 /* References */ });
                }
            }
            function findReferenceOrRenameEntries(program, cancellationToken, sourceFiles, node, position, options, convertEntry) {
                return ts.map(flattenEntries(Core.getReferencedSymbolsForNode(position, node, program, sourceFiles, cancellationToken, options)), function (entry) { return convertEntry(entry, node, program.getTypeChecker()); });
            }
            FindAllReferences.findReferenceOrRenameEntries = findReferenceOrRenameEntries;
            function getReferenceEntriesForNode(position, node, program, sourceFiles, cancellationToken, options, sourceFilesSet) {
                if (options === void 0) {
                    options = {};
                }
                if (sourceFilesSet === void 0) {
                    sourceFilesSet = ts.arrayToSet(sourceFiles, function (f) { return f.fileName; });
                }
                return flattenEntries(Core.getReferencedSymbolsForNode(position, node, program, sourceFiles, cancellationToken, options, sourceFilesSet));
            }
            FindAllReferences.getReferenceEntriesForNode = getReferenceEntriesForNode;
            function flattenEntries(referenceSymbols) {
                return referenceSymbols && ts.flatMap(referenceSymbols, function (r) { return r.references; });
            }
            function definitionToReferencedSymbolDefinitionInfo(def, checker, originalNode) {
                var info = (function () {
                    switch (def.type) {
                        case 0 /* Symbol */: {
                            var symbol = def.symbol;
                            var _a = getDefinitionKindAndDisplayParts(symbol, checker, originalNode), displayParts_1 = _a.displayParts, kind_1 = _a.kind;
                            var name_1 = displayParts_1.map(function (p) { return p.text; }).join("");
                            var declaration = symbol.declarations ? ts.first(symbol.declarations) : undefined;
                            return {
                                node: declaration ?
                                    ts.getNameOfDeclaration(declaration) || declaration :
                                    originalNode,
                                name: name_1,
                                kind: kind_1,
                                displayParts: displayParts_1,
                                context: getContextNode(declaration)
                            };
                        }
                        case 1 /* Label */: {
                            var node_1 = def.node;
                            return { node: node_1, name: node_1.text, kind: "label" /* label */, displayParts: [ts.displayPart(node_1.text, ts.SymbolDisplayPartKind.text)] };
                        }
                        case 2 /* Keyword */: {
                            var node_2 = def.node;
                            var name_2 = ts.tokenToString(node_2.kind);
                            return { node: node_2, name: name_2, kind: "keyword" /* keyword */, displayParts: [{ text: name_2, kind: "keyword" /* keyword */ }] };
                        }
                        case 3 /* This */: {
                            var node_3 = def.node;
                            var symbol = checker.getSymbolAtLocation(node_3);
                            var displayParts_2 = symbol && ts.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(checker, symbol, node_3.getSourceFile(), ts.getContainerNode(node_3), node_3).displayParts || [ts.textPart("this")];
                            return { node: node_3, name: "this", kind: "var" /* variableElement */, displayParts: displayParts_2 };
                        }
                        case 4 /* String */: {
                            var node_4 = def.node;
                            return { node: node_4, name: node_4.text, kind: "var" /* variableElement */, displayParts: [ts.displayPart(ts.getTextOfNode(node_4), ts.SymbolDisplayPartKind.stringLiteral)] };
                        }
                        default:
                            return ts.Debug.assertNever(def);
                    }
                })();
                var node = info.node, name = info.name, kind = info.kind, displayParts = info.displayParts, context = info.context;
                var sourceFile = node.getSourceFile();
                var textSpan = getTextSpan(ts.isComputedPropertyName(node) ? node.expression : node, sourceFile);
                return __assign({ containerKind: "" /* unknown */, containerName: "", fileName: sourceFile.fileName, kind: kind,
                    name: name,
                    textSpan: textSpan,
                    displayParts: displayParts }, toContextSpan(textSpan, sourceFile, context));
            }
            function getDefinitionKindAndDisplayParts(symbol, checker, node) {
                var meaning = Core.getIntersectingMeaningFromDeclarations(node, symbol);
                var enclosingDeclaration = symbol.declarations && ts.firstOrUndefined(symbol.declarations) || node;
                var _a = ts.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(checker, symbol, enclosingDeclaration.getSourceFile(), enclosingDeclaration, enclosingDeclaration, meaning), displayParts = _a.displayParts, symbolKind = _a.symbolKind;
                return { displayParts: displayParts, kind: symbolKind };
            }
            function toRenameLocation(entry, originalNode, checker, providePrefixAndSuffixText) {
                return __assign(__assign({}, entryToDocumentSpan(entry)), (providePrefixAndSuffixText && getPrefixAndSuffixText(entry, originalNode, checker)));
            }
            FindAllReferences.toRenameLocation = toRenameLocation;
            function toReferenceEntry(entry) {
                var documentSpan = entryToDocumentSpan(entry);
                if (entry.kind === 0 /* Span */) {
                    return __assign(__assign({}, documentSpan), { isWriteAccess: false, isDefinition: false });
                }
                var kind = entry.kind, node = entry.node;
                return __assign(__assign({}, documentSpan), { isWriteAccess: isWriteAccessForReference(node), isDefinition: isDefinitionForReference(node), isInString: kind === 2 /* StringLiteral */ ? true : undefined });
            }
            FindAllReferences.toReferenceEntry = toReferenceEntry;
            function entryToDocumentSpan(entry) {
                if (entry.kind === 0 /* Span */) {
                    return { textSpan: entry.textSpan, fileName: entry.fileName };
                }
                else {
                    var sourceFile = entry.node.getSourceFile();
                    var textSpan = getTextSpan(entry.node, sourceFile);
                    return __assign({ textSpan: textSpan, fileName: sourceFile.fileName }, toContextSpan(textSpan, sourceFile, entry.context));
                }
            }
            function getPrefixAndSuffixText(entry, originalNode, checker) {
                if (entry.kind !== 0 /* Span */ && ts.isIdentifier(originalNode)) {
                    var node = entry.node, kind = entry.kind;
                    var name = originalNode.text;
                    var isShorthandAssignment = ts.isShorthandPropertyAssignment(node.parent);
                    if (isShorthandAssignment || ts.isObjectBindingElementWithoutPropertyName(node.parent) && node.parent.name === node) {
                        var prefixColon = { prefixText: name + ": " };
                        var suffixColon = { suffixText: ": " + name };
                        return kind === 3 /* SearchedLocalFoundProperty */ ? prefixColon
                            : kind === 4 /* SearchedPropertyFoundLocal */ ? suffixColon
                                // In `const o = { x }; o.x`, symbolAtLocation at `x` in `{ x }` is the property symbol.
                                // For a binding element `const { x } = o;`, symbolAtLocation at `x` is the property symbol.
                                : isShorthandAssignment ? suffixColon : prefixColon;
                    }
                    else if (ts.isImportSpecifier(entry.node.parent) && !entry.node.parent.propertyName) {
                        // If the original symbol was using this alias, just rename the alias.
                        var originalSymbol = ts.isExportSpecifier(originalNode.parent) ? checker.getExportSpecifierLocalTargetSymbol(originalNode.parent) : checker.getSymbolAtLocation(originalNode);
                        return ts.contains(originalSymbol.declarations, entry.node.parent) ? { prefixText: name + " as " } : ts.emptyOptions;
                    }
                    else if (ts.isExportSpecifier(entry.node.parent) && !entry.node.parent.propertyName) {
                        // If the symbol for the node is same as declared node symbol use prefix text
                        return originalNode === entry.node || checker.getSymbolAtLocation(originalNode) === checker.getSymbolAtLocation(entry.node) ?
                            { prefixText: name + " as " } :
                            { suffixText: " as " + name };
                    }
                }
                return ts.emptyOptions;
            }
            function toImplementationLocation(entry, checker) {
                var documentSpan = entryToDocumentSpan(entry);
                if (entry.kind !== 0 /* Span */) {
                    var node = entry.node;
                    return __assign(__assign({}, documentSpan), implementationKindDisplayParts(node, checker));
                }
                else {
                    return __assign(__assign({}, documentSpan), { kind: "" /* unknown */, displayParts: [] });
                }
            }
            function implementationKindDisplayParts(node, checker) {
                var symbol = checker.getSymbolAtLocation(ts.isDeclaration(node) && node.name ? node.name : node);
                if (symbol) {
                    return getDefinitionKindAndDisplayParts(symbol, checker, node);
                }
                else if (node.kind === 193 /* ObjectLiteralExpression */) {
                    return {
                        kind: "interface" /* interfaceElement */,
                        displayParts: [ts.punctuationPart(20 /* OpenParenToken */), ts.textPart("object literal"), ts.punctuationPart(21 /* CloseParenToken */)]
                    };
                }
                else if (node.kind === 214 /* ClassExpression */) {
                    return {
                        kind: "local class" /* localClassElement */,
                        displayParts: [ts.punctuationPart(20 /* OpenParenToken */), ts.textPart("anonymous local class"), ts.punctuationPart(21 /* CloseParenToken */)]
                    };
                }
                else {
                    return { kind: ts.getNodeKind(node), displayParts: [] };
                }
            }
            function toHighlightSpan(entry) {
                var documentSpan = entryToDocumentSpan(entry);
                if (entry.kind === 0 /* Span */) {
                    return {
                        fileName: documentSpan.fileName,
                        span: {
                            textSpan: documentSpan.textSpan,
                            kind: "reference" /* reference */
                        }
                    };
                }
                var writeAccess = isWriteAccessForReference(entry.node);
                var span = __assign({ textSpan: documentSpan.textSpan, kind: writeAccess ? "writtenReference" /* writtenReference */ : "reference" /* reference */, isInString: entry.kind === 2 /* StringLiteral */ ? true : undefined }, documentSpan.contextSpan && { contextSpan: documentSpan.contextSpan });
                return { fileName: documentSpan.fileName, span: span };
            }
            FindAllReferences.toHighlightSpan = toHighlightSpan;
            function getTextSpan(node, sourceFile, endNode) {
                var start = node.getStart(sourceFile);
                var end = (endNode || node).getEnd();
                if (ts.isStringLiteralLike(node)) {
                    ts.Debug.assert(endNode === undefined);
                    start += 1;
                    end -= 1;
                }
                return ts.createTextSpanFromBounds(start, end);
            }
            function getTextSpanOfEntry(entry) {
                return entry.kind === 0 /* Span */ ? entry.textSpan :
                    getTextSpan(entry.node, entry.node.getSourceFile());
            }
            FindAllReferences.getTextSpanOfEntry = getTextSpanOfEntry;
            /** A node is considered a writeAccess iff it is a name of a declaration or a target of an assignment */
            function isWriteAccessForReference(node) {
                var decl = ts.getDeclarationFromName(node);
                return !!decl && declarationIsWriteAccess(decl) || node.kind === 84 /* DefaultKeyword */ || ts.isWriteAccess(node);
            }
            function isDefinitionForReference(node) {
                return node.kind === 84 /* DefaultKeyword */
                    || !!ts.getDeclarationFromName(node)
                    || ts.isLiteralComputedPropertyDeclarationName(node)
                    || (node.kind === 129 /* ConstructorKeyword */ && ts.isConstructorDeclaration(node.parent));
            }
            /**
             * True if 'decl' provides a value, as in `function f() {}`;
             * false if 'decl' is just a location for a future write, as in 'let x;'
             */
            function declarationIsWriteAccess(decl) {
                // Consider anything in an ambient declaration to be a write access since it may be coming from JS.
                if (!!(decl.flags & 8388608 /* Ambient */))
                    return true;
                switch (decl.kind) {
                    case 209 /* BinaryExpression */:
                    case 191 /* BindingElement */:
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                    case 84 /* DefaultKeyword */:
                    case 248 /* EnumDeclaration */:
                    case 284 /* EnumMember */:
                    case 263 /* ExportSpecifier */:
                    case 255 /* ImportClause */: // default import
                    case 253 /* ImportEqualsDeclaration */:
                    case 258 /* ImportSpecifier */:
                    case 246 /* InterfaceDeclaration */:
                    case 314 /* JSDocCallbackTag */:
                    case 321 /* JSDocTypedefTag */:
                    case 273 /* JsxAttribute */:
                    case 249 /* ModuleDeclaration */:
                    case 252 /* NamespaceExportDeclaration */:
                    case 256 /* NamespaceImport */:
                    case 262 /* NamespaceExport */:
                    case 156 /* Parameter */:
                    case 282 /* ShorthandPropertyAssignment */:
                    case 247 /* TypeAliasDeclaration */:
                    case 155 /* TypeParameter */:
                        return true;
                    case 281 /* PropertyAssignment */:
                        // In `({ x: y } = 0);`, `x` is not a write access. (Won't call this function for `y`.)
                        return !ts.isArrayLiteralOrObjectLiteralDestructuringPattern(decl.parent);
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 162 /* Constructor */:
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        return !!decl.body;
                    case 242 /* VariableDeclaration */:
                    case 159 /* PropertyDeclaration */:
                        return !!decl.initializer || ts.isCatchClause(decl.parent);
                    case 160 /* MethodSignature */:
                    case 158 /* PropertySignature */:
                    case 322 /* JSDocPropertyTag */:
                    case 316 /* JSDocParameterTag */:
                        return false;
                    default:
                        return ts.Debug.failBadSyntaxKind(decl);
                }
            }
            /** Encapsulates the core find-all-references algorithm. */
            var Core;
            (function (Core) {
                /** Core find-all-references algorithm. Handles special cases before delegating to `getReferencedSymbolsForSymbol`. */
                function getReferencedSymbolsForNode(position, node, program, sourceFiles, cancellationToken, options, sourceFilesSet) {
                    if (options === void 0) {
                        options = {};
                    }
                    if (sourceFilesSet === void 0) {
                        sourceFilesSet = ts.arrayToSet(sourceFiles, function (f) { return f.fileName; });
                    }
                    if (options.use === 1 /* References */) {
                        node = ts.getAdjustedReferenceLocation(node);
                    }
                    else if (options.use === 2 /* Rename */) {
                        node = ts.getAdjustedRenameLocation(node);
                    }
                    if (ts.isSourceFile(node)) {
                        var reference = ts.GoToDefinition.getReferenceAtPosition(node, position, program);
                        var moduleSymbol = reference && program.getTypeChecker().getMergedSymbol(reference.file.symbol);
                        return moduleSymbol && getReferencedSymbolsForModule(program, moduleSymbol, /*excludeImportTypeOfExportEquals*/ false, sourceFiles, sourceFilesSet);
                    }
                    if (!options.implementations) {
                        var special = getReferencedSymbolsSpecial(node, sourceFiles, cancellationToken);
                        if (special) {
                            return special;
                        }
                    }
                    var checker = program.getTypeChecker();
                    var symbol = checker.getSymbolAtLocation(node);
                    // Could not find a symbol e.g. unknown identifier
                    if (!symbol) {
                        // String literal might be a property (and thus have a symbol), so do this here rather than in getReferencedSymbolsSpecial.
                        return !options.implementations && ts.isStringLiteral(node) ? getReferencesForStringLiteral(node, sourceFiles, cancellationToken) : undefined;
                    }
                    if (symbol.escapedName === "export=" /* ExportEquals */) {
                        return getReferencedSymbolsForModule(program, symbol.parent, /*excludeImportTypeOfExportEquals*/ false, sourceFiles, sourceFilesSet);
                    }
                    var moduleReferences = getReferencedSymbolsForModuleIfDeclaredBySourceFile(symbol, program, sourceFiles, cancellationToken, options, sourceFilesSet);
                    if (moduleReferences && !(symbol.flags & 33554432 /* Transient */)) {
                        return moduleReferences;
                    }
                    var aliasedSymbol = getMergedAliasedSymbolOfNamespaceExportDeclaration(node, symbol, checker);
                    var moduleReferencesOfExportTarget = aliasedSymbol &&
                        getReferencedSymbolsForModuleIfDeclaredBySourceFile(aliasedSymbol, program, sourceFiles, cancellationToken, options, sourceFilesSet);
                    var references = getReferencedSymbolsForSymbol(symbol, node, sourceFiles, sourceFilesSet, checker, cancellationToken, options);
                    return mergeReferences(program, moduleReferences, references, moduleReferencesOfExportTarget);
                }
                Core.getReferencedSymbolsForNode = getReferencedSymbolsForNode;
                function getMergedAliasedSymbolOfNamespaceExportDeclaration(node, symbol, checker) {
                    if (node.parent && ts.isNamespaceExportDeclaration(node.parent)) {
                        var aliasedSymbol = checker.getAliasedSymbol(symbol);
                        var targetSymbol = checker.getMergedSymbol(aliasedSymbol);
                        if (aliasedSymbol !== targetSymbol) {
                            return targetSymbol;
                        }
                    }
                    return undefined;
                }
                function getReferencedSymbolsForModuleIfDeclaredBySourceFile(symbol, program, sourceFiles, cancellationToken, options, sourceFilesSet) {
                    var moduleSourceFile = (symbol.flags & 1536 /* Module */) && symbol.declarations && ts.find(symbol.declarations, ts.isSourceFile);
                    if (!moduleSourceFile)
                        return undefined;
                    var exportEquals = symbol.exports.get("export=" /* ExportEquals */);
                    // If !!exportEquals, we're about to add references to `import("mod")` anyway, so don't double-count them.
                    var moduleReferences = getReferencedSymbolsForModule(program, symbol, !!exportEquals, sourceFiles, sourceFilesSet);
                    if (!exportEquals || !sourceFilesSet.has(moduleSourceFile.fileName))
                        return moduleReferences;
                    // Continue to get references to 'export ='.
                    var checker = program.getTypeChecker();
                    symbol = ts.skipAlias(exportEquals, checker);
                    return mergeReferences(program, moduleReferences, getReferencedSymbolsForSymbol(symbol, /*node*/ undefined, sourceFiles, sourceFilesSet, checker, cancellationToken, options));
                }
                /**
                 * Merges the references by sorting them (by file index in sourceFiles and their location in it) that point to same definition symbol
                 */
                function mergeReferences(program) {
                    var referencesToMerge = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        referencesToMerge[_i - 1] = arguments[_i];
                    }
                    var result;
                    for (var _a = 0, referencesToMerge_1 = referencesToMerge; _a < referencesToMerge_1.length; _a++) {
                        var references = referencesToMerge_1[_a];
                        if (!references || !references.length)
                            continue;
                        if (!result) {
                            result = references;
                            continue;
                        }
                        var _loop_3 = function (entry) {
                            if (!entry.definition || entry.definition.type !== 0 /* Symbol */) {
                                result.push(entry);
                                return "continue";
                            }
                            var symbol = entry.definition.symbol;
                            var refIndex = ts.findIndex(result, function (ref) {
                                return !!ref.definition &&
                                    ref.definition.type === 0 /* Symbol */ &&
                                    ref.definition.symbol === symbol;
                            });
                            if (refIndex === -1) {
                                result.push(entry);
                                return "continue";
                            }
                            var reference = result[refIndex];
                            result[refIndex] = {
                                definition: reference.definition,
                                references: reference.references.concat(entry.references).sort(function (entry1, entry2) {
                                    var entry1File = getSourceFileIndexOfEntry(program, entry1);
                                    var entry2File = getSourceFileIndexOfEntry(program, entry2);
                                    if (entry1File !== entry2File) {
                                        return ts.compareValues(entry1File, entry2File);
                                    }
                                    var entry1Span = getTextSpanOfEntry(entry1);
                                    var entry2Span = getTextSpanOfEntry(entry2);
                                    return entry1Span.start !== entry2Span.start ?
                                        ts.compareValues(entry1Span.start, entry2Span.start) :
                                        ts.compareValues(entry1Span.length, entry2Span.length);
                                })
                            };
                        };
                        for (var _b = 0, references_1 = references; _b < references_1.length; _b++) {
                            var entry = references_1[_b];
                            _loop_3(entry);
                        }
                    }
                    return result;
                }
                function getSourceFileIndexOfEntry(program, entry) {
                    var sourceFile = entry.kind === 0 /* Span */ ?
                        program.getSourceFile(entry.fileName) :
                        entry.node.getSourceFile();
                    return program.getSourceFiles().indexOf(sourceFile);
                }
                function getReferencedSymbolsForModule(program, symbol, excludeImportTypeOfExportEquals, sourceFiles, sourceFilesSet) {
                    ts.Debug.assert(!!symbol.valueDeclaration);
                    var references = ts.mapDefined(FindAllReferences.findModuleReferences(program, sourceFiles, symbol), function (reference) {
                        if (reference.kind === "import") {
                            var parent = reference.literal.parent;
                            if (ts.isLiteralTypeNode(parent)) {
                                var importType = ts.cast(parent.parent, ts.isImportTypeNode);
                                if (excludeImportTypeOfExportEquals && !importType.qualifier) {
                                    return undefined;
                                }
                            }
                            // import("foo") with no qualifier will reference the `export =` of the module, which may be referenced anyway.
                            return nodeEntry(reference.literal);
                        }
                        else {
                            return {
                                kind: 0 /* Span */,
                                fileName: reference.referencingFile.fileName,
                                textSpan: ts.createTextSpanFromRange(reference.ref),
                            };
                        }
                    });
                    for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        switch (decl.kind) {
                            case 290 /* SourceFile */:
                                // Don't include the source file itself. (This may not be ideal behavior, but awkward to include an entire file as a reference.)
                                break;
                            case 249 /* ModuleDeclaration */:
                                if (sourceFilesSet.has(decl.getSourceFile().fileName)) {
                                    references.push(nodeEntry(decl.name));
                                }
                                break;
                            default:
                                // This may be merged with something.
                                ts.Debug.assert(!!(symbol.flags & 33554432 /* Transient */), "Expected a module symbol to be declared by a SourceFile or ModuleDeclaration.");
                        }
                    }
                    var exported = symbol.exports.get("export=" /* ExportEquals */);
                    if (exported) {
                        for (var _b = 0, _c = exported.declarations; _b < _c.length; _b++) {
                            var decl = _c[_b];
                            var sourceFile = decl.getSourceFile();
                            if (sourceFilesSet.has(sourceFile.fileName)) {
                                // At `module.exports = ...`, reference node is `module`
                                var node = ts.isBinaryExpression(decl) && ts.isPropertyAccessExpression(decl.left) ? decl.left.expression :
                                    ts.isExportAssignment(decl) ? ts.Debug.assertDefined(ts.findChildOfKind(decl, 89 /* ExportKeyword */, sourceFile)) :
                                        ts.getNameOfDeclaration(decl) || decl;
                                references.push(nodeEntry(node));
                            }
                        }
                    }
                    return references.length ? [{ definition: { type: 0 /* Symbol */, symbol: symbol }, references: references }] : ts.emptyArray;
                }
                /** As in a `readonly prop: any` or `constructor(readonly prop: any)`, not a `readonly any[]`. */
                function isReadonlyTypeOperator(node) {
                    return node.kind === 138 /* ReadonlyKeyword */
                        && ts.isTypeOperatorNode(node.parent)
                        && node.parent.operator === 138 /* ReadonlyKeyword */;
                }
                /** getReferencedSymbols for special node kinds. */
                function getReferencedSymbolsSpecial(node, sourceFiles, cancellationToken) {
                    if (ts.isTypeKeyword(node.kind)) {
                        // A void expression (i.e., `void foo()`) is not special, but the `void` type is.
                        if (node.kind === 110 /* VoidKeyword */ && ts.isVoidExpression(node.parent)) {
                            return undefined;
                        }
                        // A modifier readonly (like on a property declaration) is not special;
                        // a readonly type keyword (like `readonly string[]`) is.
                        if (node.kind === 138 /* ReadonlyKeyword */ && !isReadonlyTypeOperator(node)) {
                            return undefined;
                        }
                        // Likewise, when we *are* looking for a special keyword, make sure we
                        // *don’t* include readonly member modifiers.
                        return getAllReferencesForKeyword(sourceFiles, node.kind, cancellationToken, node.kind === 138 /* ReadonlyKeyword */ ? isReadonlyTypeOperator : undefined);
                    }
                    // Labels
                    if (ts.isJumpStatementTarget(node)) {
                        var labelDefinition = ts.getTargetLabel(node.parent, node.text);
                        // if we have a label definition, look within its statement for references, if not, then
                        // the label is undefined and we have no results..
                        return labelDefinition && getLabelReferencesInNode(labelDefinition.parent, labelDefinition);
                    }
                    else if (ts.isLabelOfLabeledStatement(node)) {
                        // it is a label definition and not a target, search within the parent labeledStatement
                        return getLabelReferencesInNode(node.parent, node);
                    }
                    if (ts.isThis(node)) {
                        return getReferencesForThisKeyword(node, sourceFiles, cancellationToken);
                    }
                    if (node.kind === 102 /* SuperKeyword */) {
                        return getReferencesForSuperKeyword(node);
                    }
                    return undefined;
                }
                /** Core find-all-references algorithm for a normal symbol. */
                function getReferencedSymbolsForSymbol(originalSymbol, node, sourceFiles, sourceFilesSet, checker, cancellationToken, options) {
                    var symbol = node && skipPastExportOrImportSpecifierOrUnion(originalSymbol, node, checker, /*useLocalSymbolForExportSpecifier*/ !isForRenameWithPrefixAndSuffixText(options)) || originalSymbol;
                    // Compute the meaning from the location and the symbol it references
                    var searchMeaning = node ? getIntersectingMeaningFromDeclarations(node, symbol) : 7 /* All */;
                    var result = [];
                    var state = new State(sourceFiles, sourceFilesSet, node ? getSpecialSearchKind(node) : 0 /* None */, checker, cancellationToken, searchMeaning, options, result);
                    var exportSpecifier = !isForRenameWithPrefixAndSuffixText(options) ? undefined : ts.find(symbol.declarations, ts.isExportSpecifier);
                    if (exportSpecifier) {
                        // When renaming at an export specifier, rename the export and not the thing being exported.
                        getReferencesAtExportSpecifier(exportSpecifier.name, symbol, exportSpecifier, state.createSearch(node, originalSymbol, /*comingFrom*/ undefined), state, /*addReferencesHere*/ true, /*alwaysGetReferences*/ true);
                    }
                    else if (node && node.kind === 84 /* DefaultKeyword */) {
                        addReference(node, symbol, state);
                        searchForImportsOfExport(node, symbol, { exportingModuleSymbol: ts.Debug.assertDefined(symbol.parent, "Expected export symbol to have a parent"), exportKind: 1 /* Default */ }, state);
                    }
                    else {
                        var search = state.createSearch(node, symbol, /*comingFrom*/ undefined, { allSearchSymbols: node ? populateSearchSymbolSet(symbol, node, checker, options.use === 2 /* Rename */, !!options.providePrefixAndSuffixTextForRename, !!options.implementations) : [symbol] });
                        getReferencesInContainerOrFiles(symbol, state, search);
                    }
                    return result;
                }
                function getReferencesInContainerOrFiles(symbol, state, search) {
                    // Try to get the smallest valid scope that we can limit our search to;
                    // otherwise we'll need to search globally (i.e. include each file).
                    var scope = getSymbolScope(symbol);
                    if (scope) {
                        getReferencesInContainer(scope, scope.getSourceFile(), search, state, /*addReferencesHere*/ !(ts.isSourceFile(scope) && !ts.contains(state.sourceFiles, scope)));
                    }
                    else {
                        // Global search
                        for (var _i = 0, _a = state.sourceFiles; _i < _a.length; _i++) {
                            var sourceFile = _a[_i];
                            state.cancellationToken.throwIfCancellationRequested();
                            searchForName(sourceFile, search, state);
                        }
                    }
                }
                function getSpecialSearchKind(node) {
                    switch (node.kind) {
                        case 129 /* ConstructorKeyword */:
                            return 1 /* Constructor */;
                        case 75 /* Identifier */:
                            if (ts.isClassLike(node.parent)) {
                                ts.Debug.assert(node.parent.name === node);
                                return 2 /* Class */;
                            }
                        // falls through
                        default:
                            return 0 /* None */;
                    }
                }
                /** Handle a few special cases relating to export/import specifiers. */
                function skipPastExportOrImportSpecifierOrUnion(symbol, node, checker, useLocalSymbolForExportSpecifier) {
                    var parent = node.parent;
                    if (ts.isExportSpecifier(parent) && useLocalSymbolForExportSpecifier) {
                        return getLocalSymbolForExportSpecifier(node, symbol, parent, checker);
                    }
                    // If the symbol is declared as part of a declaration like `{ type: "a" } | { type: "b" }`, use the property on the union type to get more references.
                    return ts.firstDefined(symbol.declarations, function (decl) {
                        if (!decl.parent) {
                            // Ignore UMD module and global merge
                            if (symbol.flags & 33554432 /* Transient */)
                                return undefined;
                            // Assertions for GH#21814. We should be handling SourceFile symbols in `getReferencedSymbolsForModule` instead of getting here.
                            ts.Debug.fail("Unexpected symbol at " + ts.Debug.formatSyntaxKind(node.kind) + ": " + ts.Debug.formatSymbol(symbol));
                        }
                        return ts.isTypeLiteralNode(decl.parent) && ts.isUnionTypeNode(decl.parent.parent)
                            ? checker.getPropertyOfType(checker.getTypeFromTypeNode(decl.parent.parent), symbol.name)
                            : undefined;
                    });
                }
                var SpecialSearchKind;
                (function (SpecialSearchKind) {
                    SpecialSearchKind[SpecialSearchKind["None"] = 0] = "None";
                    SpecialSearchKind[SpecialSearchKind["Constructor"] = 1] = "Constructor";
                    SpecialSearchKind[SpecialSearchKind["Class"] = 2] = "Class";
                })(SpecialSearchKind || (SpecialSearchKind = {}));
                function getNonModuleSymbolOfMergedModuleSymbol(symbol) {
                    if (!(symbol.flags & (1536 /* Module */ | 33554432 /* Transient */)))
                        return undefined;
                    var decl = symbol.declarations && ts.find(symbol.declarations, function (d) { return !ts.isSourceFile(d) && !ts.isModuleDeclaration(d); });
                    return decl && decl.symbol;
                }
                /**
                 * Holds all state needed for the finding references.
                 * Unlike `Search`, there is only one `State`.
                 */
                var State = /** @class */ (function () {
                    function State(sourceFiles, sourceFilesSet, specialSearchKind, checker, cancellationToken, searchMeaning, options, result) {
                        this.sourceFiles = sourceFiles;
                        this.sourceFilesSet = sourceFilesSet;
                        this.specialSearchKind = specialSearchKind;
                        this.checker = checker;
                        this.cancellationToken = cancellationToken;
                        this.searchMeaning = searchMeaning;
                        this.options = options;
                        this.result = result;
                        /** Cache for `explicitlyinheritsFrom`. */
                        this.inheritsFromCache = ts.createMap();
                        /**
                         * Type nodes can contain multiple references to the same type. For example:
                         *      let x: Foo & (Foo & Bar) = ...
                         * Because we are returning the implementation locations and not the identifier locations,
                         * duplicate entries would be returned here as each of the type references is part of
                         * the same implementation. For that reason, check before we add a new entry.
                         */
                        this.markSeenContainingTypeReference = ts.nodeSeenTracker();
                        /**
                         * It's possible that we will encounter the right side of `export { foo as bar } from "x";` more than once.
                         * For example:
                         *     // b.ts
                         *     export { foo as bar } from "./a";
                         *     import { bar } from "./b";
                         *
                         * Normally at `foo as bar` we directly add `foo` and do not locally search for it (since it doesn't declare a local).
                         * But another reference to it may appear in the same source file.
                         * See `tests/cases/fourslash/transitiveExportImports3.ts`.
                         */
                        this.markSeenReExportRHS = ts.nodeSeenTracker();
                        this.symbolIdToReferences = [];
                        // Source file ID → symbol ID → Whether the symbol has been searched for in the source file.
                        this.sourceFileToSeenSymbols = [];
                    }
                    State.prototype.includesSourceFile = function (sourceFile) {
                        return this.sourceFilesSet.has(sourceFile.fileName);
                    };
                    /** Gets every place to look for references of an exported symbols. See `ImportsResult` in `importTracker.ts` for more documentation. */
                    State.prototype.getImportSearches = function (exportSymbol, exportInfo) {
                        if (!this.importTracker)
                            this.importTracker = FindAllReferences.createImportTracker(this.sourceFiles, this.sourceFilesSet, this.checker, this.cancellationToken);
                        return this.importTracker(exportSymbol, exportInfo, this.options.use === 2 /* Rename */);
                    };
                    /** @param allSearchSymbols set of additional symbols for use by `includes`. */
                    State.prototype.createSearch = function (location, symbol, comingFrom, searchOptions) {
                        if (searchOptions === void 0) {
                            searchOptions = {};
                        }
                        // Note: if this is an external module symbol, the name doesn't include quotes.
                        // Note: getLocalSymbolForExportDefault handles `export default class C {}`, but not `export default C` or `export { C as default }`.
                        // The other two forms seem to be handled downstream (e.g. in `skipPastExportOrImportSpecifier`), so special-casing the first form
                        // here appears to be intentional).
                        var _a = searchOptions.text, text = _a === void 0 ? ts.stripQuotes(ts.symbolName(ts.getLocalSymbolForExportDefault(symbol) || getNonModuleSymbolOfMergedModuleSymbol(symbol) || symbol)) : _a, _b = searchOptions.allSearchSymbols, allSearchSymbols = _b === void 0 ? [symbol] : _b;
                        var escapedText = ts.escapeLeadingUnderscores(text);
                        var parents = this.options.implementations && location ? getParentSymbolsOfPropertyAccess(location, symbol, this.checker) : undefined;
                        return { symbol: symbol, comingFrom: comingFrom, text: text, escapedText: escapedText, parents: parents, allSearchSymbols: allSearchSymbols, includes: function (sym) { return ts.contains(allSearchSymbols, sym); } };
                    };
                    /**
                     * Callback to add references for a particular searched symbol.
                     * This initializes a reference group, so only call this if you will add at least one reference.
                     */
                    State.prototype.referenceAdder = function (searchSymbol) {
                        var symbolId = ts.getSymbolId(searchSymbol);
                        var references = this.symbolIdToReferences[symbolId];
                        if (!references) {
                            references = this.symbolIdToReferences[symbolId] = [];
                            this.result.push({ definition: { type: 0 /* Symbol */, symbol: searchSymbol }, references: references });
                        }
                        return function (node, kind) { return references.push(nodeEntry(node, kind)); };
                    };
                    /** Add a reference with no associated definition. */
                    State.prototype.addStringOrCommentReference = function (fileName, textSpan) {
                        this.result.push({
                            definition: undefined,
                            references: [{ kind: 0 /* Span */, fileName: fileName, textSpan: textSpan }]
                        });
                    };
                    /** Returns `true` the first time we search for a symbol in a file and `false` afterwards. */
                    State.prototype.markSearchedSymbols = function (sourceFile, symbols) {
                        var sourceId = ts.getNodeId(sourceFile);
                        var seenSymbols = this.sourceFileToSeenSymbols[sourceId] || (this.sourceFileToSeenSymbols[sourceId] = ts.createMap());
                        var anyNewSymbols = false;
                        for (var _i = 0, symbols_3 = symbols; _i < symbols_3.length; _i++) {
                            var sym = symbols_3[_i];
                            anyNewSymbols = ts.addToSeen(seenSymbols, ts.getSymbolId(sym)) || anyNewSymbols;
                        }
                        return anyNewSymbols;
                    };
                    return State;
                }());
                /** Search for all imports of a given exported symbol using `State.getImportSearches`. */
                function searchForImportsOfExport(exportLocation, exportSymbol, exportInfo, state) {
                    var _a = state.getImportSearches(exportSymbol, exportInfo), importSearches = _a.importSearches, singleReferences = _a.singleReferences, indirectUsers = _a.indirectUsers;
                    // For `import { foo as bar }` just add the reference to `foo`, and don't otherwise search in the file.
                    if (singleReferences.length) {
                        var addRef = state.referenceAdder(exportSymbol);
                        for (var _i = 0, singleReferences_1 = singleReferences; _i < singleReferences_1.length; _i++) {
                            var singleRef = singleReferences_1[_i];
                            if (shouldAddSingleReference(singleRef, state))
                                addRef(singleRef);
                        }
                    }
                    // For each import, find all references to that import in its source file.
                    for (var _b = 0, importSearches_1 = importSearches; _b < importSearches_1.length; _b++) {
                        var _c = importSearches_1[_b], importLocation = _c[0], importSymbol = _c[1];
                        getReferencesInSourceFile(importLocation.getSourceFile(), state.createSearch(importLocation, importSymbol, 1 /* Export */), state);
                    }
                    if (indirectUsers.length) {
                        var indirectSearch = void 0;
                        switch (exportInfo.exportKind) {
                            case 0 /* Named */:
                                indirectSearch = state.createSearch(exportLocation, exportSymbol, 1 /* Export */);
                                break;
                            case 1 /* Default */:
                                // Search for a property access to '.default'. This can't be renamed.
                                indirectSearch = state.options.use === 2 /* Rename */ ? undefined : state.createSearch(exportLocation, exportSymbol, 1 /* Export */, { text: "default" });
                                break;
                            case 2 /* ExportEquals */:
                                break;
                        }
                        if (indirectSearch) {
                            for (var _d = 0, indirectUsers_1 = indirectUsers; _d < indirectUsers_1.length; _d++) {
                                var indirectUser = indirectUsers_1[_d];
                                searchForName(indirectUser, indirectSearch, state);
                            }
                        }
                    }
                }
                function eachExportReference(sourceFiles, checker, cancellationToken, exportSymbol, exportingModuleSymbol, exportName, isDefaultExport, cb) {
                    var importTracker = FindAllReferences.createImportTracker(sourceFiles, ts.arrayToSet(sourceFiles, function (f) { return f.fileName; }), checker, cancellationToken);
                    var _a = importTracker(exportSymbol, { exportKind: isDefaultExport ? 1 /* Default */ : 0 /* Named */, exportingModuleSymbol: exportingModuleSymbol }, /*isForRename*/ false), importSearches = _a.importSearches, indirectUsers = _a.indirectUsers;
                    for (var _i = 0, importSearches_2 = importSearches; _i < importSearches_2.length; _i++) {
                        var importLocation = importSearches_2[_i][0];
                        cb(importLocation);
                    }
                    for (var _b = 0, indirectUsers_2 = indirectUsers; _b < indirectUsers_2.length; _b++) {
                        var indirectUser = indirectUsers_2[_b];
                        for (var _c = 0, _d = getPossibleSymbolReferenceNodes(indirectUser, isDefaultExport ? "default" : exportName); _c < _d.length; _c++) {
                            var node = _d[_c];
                            // Import specifiers should be handled by importSearches
                            if (ts.isIdentifier(node) && !ts.isImportOrExportSpecifier(node.parent) && checker.getSymbolAtLocation(node) === exportSymbol) {
                                cb(node);
                            }
                        }
                    }
                }
                Core.eachExportReference = eachExportReference;
                function shouldAddSingleReference(singleRef, state) {
                    if (!hasMatchingMeaning(singleRef, state))
                        return false;
                    if (state.options.use !== 2 /* Rename */)
                        return true;
                    // Don't rename an import type `import("./module-name")` when renaming `name` in `export = name;`
                    if (!ts.isIdentifier(singleRef))
                        return false;
                    // At `default` in `import { default as x }` or `export { default as x }`, do add a reference, but do not rename.
                    return !(ts.isImportOrExportSpecifier(singleRef.parent) && singleRef.escapedText === "default" /* Default */);
                }
                // Go to the symbol we imported from and find references for it.
                function searchForImportedSymbol(symbol, state) {
                    for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                        var declaration = _a[_i];
                        var exportingFile = declaration.getSourceFile();
                        // Need to search in the file even if it's not in the search-file set, because it might export the symbol.
                        getReferencesInSourceFile(exportingFile, state.createSearch(declaration, symbol, 0 /* Import */), state, state.includesSourceFile(exportingFile));
                    }
                }
                /** Search for all occurences of an identifier in a source file (and filter out the ones that match). */
                function searchForName(sourceFile, search, state) {
                    if (ts.getNameTable(sourceFile).get(search.escapedText) !== undefined) {
                        getReferencesInSourceFile(sourceFile, search, state);
                    }
                }
                function getPropertySymbolOfDestructuringAssignment(location, checker) {
                    return ts.isArrayLiteralOrObjectLiteralDestructuringPattern(location.parent.parent)
                        ? checker.getPropertySymbolOfDestructuringAssignment(location)
                        : undefined;
                }
                /**
                 * Determines the smallest scope in which a symbol may have named references.
                 * Note that not every construct has been accounted for. This function can
                 * probably be improved.
                 *
                 * @returns undefined if the scope cannot be determined, implying that
                 * a reference to a symbol can occur anywhere.
                 */
                function getSymbolScope(symbol) {
                    // If this is the symbol of a named function expression or named class expression,
                    // then named references are limited to its own scope.
                    var declarations = symbol.declarations, flags = symbol.flags, parent = symbol.parent, valueDeclaration = symbol.valueDeclaration;
                    if (valueDeclaration && (valueDeclaration.kind === 201 /* FunctionExpression */ || valueDeclaration.kind === 214 /* ClassExpression */)) {
                        return valueDeclaration;
                    }
                    if (!declarations) {
                        return undefined;
                    }
                    // If this is private property or method, the scope is the containing class
                    if (flags & (4 /* Property */ | 8192 /* Method */)) {
                        var privateDeclaration = ts.find(declarations, function (d) { return ts.hasModifier(d, 8 /* Private */) || ts.isPrivateIdentifierPropertyDeclaration(d); });
                        if (privateDeclaration) {
                            return ts.getAncestor(privateDeclaration, 245 /* ClassDeclaration */);
                        }
                        // Else this is a public property and could be accessed from anywhere.
                        return undefined;
                    }
                    // If symbol is of object binding pattern element without property name we would want to
                    // look for property too and that could be anywhere
                    if (declarations.some(ts.isObjectBindingElementWithoutPropertyName)) {
                        return undefined;
                    }
                    /*
                    If the symbol has a parent, it's globally visible unless:
                    - It's a private property (handled above).
                    - It's a type parameter.
                    - The parent is an external module: then we should only search in the module (and recurse on the export later).
                    - But if the parent has `export as namespace`, the symbol is globally visible through that namespace.
                    */
                    var exposedByParent = parent && !(symbol.flags & 262144 /* TypeParameter */);
                    if (exposedByParent && !(ts.isExternalModuleSymbol(parent) && !parent.globalExports)) {
                        return undefined;
                    }
                    var scope;
                    for (var _i = 0, declarations_1 = declarations; _i < declarations_1.length; _i++) {
                        var declaration = declarations_1[_i];
                        var container = ts.getContainerNode(declaration);
                        if (scope && scope !== container) {
                            // Different declarations have different containers, bail out
                            return undefined;
                        }
                        if (!container || container.kind === 290 /* SourceFile */ && !ts.isExternalOrCommonJsModule(container)) {
                            // This is a global variable and not an external module, any declaration defined
                            // within this scope is visible outside the file
                            return undefined;
                        }
                        // The search scope is the container node
                        scope = container;
                    }
                    // If symbol.parent, this means we are in an export of an external module. (Otherwise we would have returned `undefined` above.)
                    // For an export of a module, we may be in a declaration file, and it may be accessed elsewhere. E.g.:
                    //     declare module "a" { export type T = number; }
                    //     declare module "b" { import { T } from "a"; export const x: T; }
                    // So we must search the whole source file. (Because we will mark the source file as seen, we we won't return to it when searching for imports.)
                    return exposedByParent ? scope.getSourceFile() : scope; // TODO: GH#18217
                }
                /** Used as a quick check for whether a symbol is used at all in a file (besides its definition). */
                function isSymbolReferencedInFile(definition, checker, sourceFile) {
                    return eachSymbolReferenceInFile(definition, checker, sourceFile, function () { return true; }) || false;
                }
                Core.isSymbolReferencedInFile = isSymbolReferencedInFile;
                function eachSymbolReferenceInFile(definition, checker, sourceFile, cb) {
                    var symbol = ts.isParameterPropertyDeclaration(definition.parent, definition.parent.parent)
                        ? ts.first(checker.getSymbolsOfParameterPropertyDeclaration(definition.parent, definition.text))
                        : checker.getSymbolAtLocation(definition);
                    if (!symbol)
                        return undefined;
                    for (var _i = 0, _a = getPossibleSymbolReferenceNodes(sourceFile, symbol.name); _i < _a.length; _i++) {
                        var token = _a[_i];
                        if (!ts.isIdentifier(token) || token === definition || token.escapedText !== definition.escapedText)
                            continue;
                        var referenceSymbol = checker.getSymbolAtLocation(token); // See GH#19955 for why the type annotation is necessary
                        if (referenceSymbol === symbol
                            || checker.getShorthandAssignmentValueSymbol(token.parent) === symbol
                            || ts.isExportSpecifier(token.parent) && getLocalSymbolForExportSpecifier(token, referenceSymbol, token.parent, checker) === symbol) {
                            var res = cb(token);
                            if (res)
                                return res;
                        }
                    }
                }
                Core.eachSymbolReferenceInFile = eachSymbolReferenceInFile;
                function eachSignatureCall(signature, sourceFiles, checker, cb) {
                    if (!signature.name || !ts.isIdentifier(signature.name))
                        return;
                    var symbol = ts.Debug.assertDefined(checker.getSymbolAtLocation(signature.name));
                    for (var _i = 0, sourceFiles_3 = sourceFiles; _i < sourceFiles_3.length; _i++) {
                        var sourceFile = sourceFiles_3[_i];
                        for (var _a = 0, _b = getPossibleSymbolReferenceNodes(sourceFile, symbol.name); _a < _b.length; _a++) {
                            var name = _b[_a];
                            if (!ts.isIdentifier(name) || name === signature.name || name.escapedText !== signature.name.escapedText)
                                continue;
                            var called = ts.climbPastPropertyAccess(name);
                            var call = called.parent;
                            if (!ts.isCallExpression(call) || call.expression !== called)
                                continue;
                            var referenceSymbol = checker.getSymbolAtLocation(name);
                            if (referenceSymbol && checker.getRootSymbols(referenceSymbol).some(function (s) { return s === symbol; })) {
                                cb(call);
                            }
                        }
                    }
                }
                Core.eachSignatureCall = eachSignatureCall;
                function getPossibleSymbolReferenceNodes(sourceFile, symbolName, container) {
                    if (container === void 0) {
                        container = sourceFile;
                    }
                    return getPossibleSymbolReferencePositions(sourceFile, symbolName, container).map(function (pos) { return ts.getTouchingPropertyName(sourceFile, pos); });
                }
                function getPossibleSymbolReferencePositions(sourceFile, symbolName, container) {
                    if (container === void 0) {
                        container = sourceFile;
                    }
                    var positions = [];
                    /// TODO: Cache symbol existence for files to save text search
                    // Also, need to make this work for unicode escapes.
                    // Be resilient in the face of a symbol with no name or zero length name
                    if (!symbolName || !symbolName.length) {
                        return positions;
                    }
                    var text = sourceFile.text;
                    var sourceLength = text.length;
                    var symbolNameLength = symbolName.length;
                    var position = text.indexOf(symbolName, container.pos);
                    while (position >= 0) {
                        // If we are past the end, stop looking
                        if (position > container.end)
                            break;
                        // We found a match.  Make sure it's not part of a larger word (i.e. the char
                        // before and after it have to be a non-identifier char).
                        var endPosition = position + symbolNameLength;
                        if ((position === 0 || !ts.isIdentifierPart(text.charCodeAt(position - 1), 99 /* Latest */)) &&
                            (endPosition === sourceLength || !ts.isIdentifierPart(text.charCodeAt(endPosition), 99 /* Latest */))) {
                            // Found a real match.  Keep searching.
                            positions.push(position);
                        }
                        position = text.indexOf(symbolName, position + symbolNameLength + 1);
                    }
                    return positions;
                }
                function getLabelReferencesInNode(container, targetLabel) {
                    var sourceFile = container.getSourceFile();
                    var labelName = targetLabel.text;
                    var references = ts.mapDefined(getPossibleSymbolReferenceNodes(sourceFile, labelName, container), function (node) {
                        // Only pick labels that are either the target label, or have a target that is the target label
                        return node === targetLabel || (ts.isJumpStatementTarget(node) && ts.getTargetLabel(node, labelName) === targetLabel) ? nodeEntry(node) : undefined;
                    });
                    return [{ definition: { type: 1 /* Label */, node: targetLabel }, references: references }];
                }
                function isValidReferencePosition(node, searchSymbolName) {
                    // Compare the length so we filter out strict superstrings of the symbol we are looking for
                    switch (node.kind) {
                        case 76 /* PrivateIdentifier */:
                        case 75 /* Identifier */:
                            return node.text.length === searchSymbolName.length;
                        case 14 /* NoSubstitutionTemplateLiteral */:
                        case 10 /* StringLiteral */: {
                            var str = node;
                            return (ts.isLiteralNameOfPropertyDeclarationOrIndexAccess(str) || ts.isNameOfModuleDeclaration(node) || ts.isExpressionOfExternalModuleImportEqualsDeclaration(node) || (ts.isCallExpression(node.parent) && ts.isBindableObjectDefinePropertyCall(node.parent) && node.parent.arguments[1] === node)) &&
                                str.text.length === searchSymbolName.length;
                        }
                        case 8 /* NumericLiteral */:
                            return ts.isLiteralNameOfPropertyDeclarationOrIndexAccess(node) && node.text.length === searchSymbolName.length;
                        case 84 /* DefaultKeyword */:
                            return "default".length === searchSymbolName.length;
                        default:
                            return false;
                    }
                }
                function getAllReferencesForKeyword(sourceFiles, keywordKind, cancellationToken, filter) {
                    var references = ts.flatMap(sourceFiles, function (sourceFile) {
                        cancellationToken.throwIfCancellationRequested();
                        return ts.mapDefined(getPossibleSymbolReferenceNodes(sourceFile, ts.tokenToString(keywordKind), sourceFile), function (referenceLocation) {
                            if (referenceLocation.kind === keywordKind && (!filter || filter(referenceLocation))) {
                                return nodeEntry(referenceLocation);
                            }
                        });
                    });
                    return references.length ? [{ definition: { type: 2 /* Keyword */, node: references[0].node }, references: references }] : undefined;
                }
                function getReferencesInSourceFile(sourceFile, search, state, addReferencesHere) {
                    if (addReferencesHere === void 0) {
                        addReferencesHere = true;
                    }
                    state.cancellationToken.throwIfCancellationRequested();
                    return getReferencesInContainer(sourceFile, sourceFile, search, state, addReferencesHere);
                }
                /**
                 * Search within node "container" for references for a search value, where the search value is defined as a
                 * tuple of(searchSymbol, searchText, searchLocation, and searchMeaning).
                 * searchLocation: a node where the search value
                 */
                function getReferencesInContainer(container, sourceFile, search, state, addReferencesHere) {
                    if (!state.markSearchedSymbols(sourceFile, search.allSearchSymbols)) {
                        return;
                    }
                    for (var _i = 0, _a = getPossibleSymbolReferencePositions(sourceFile, search.text, container); _i < _a.length; _i++) {
                        var position = _a[_i];
                        getReferencesAtLocation(sourceFile, position, search, state, addReferencesHere);
                    }
                }
                function hasMatchingMeaning(referenceLocation, state) {
                    return !!(ts.getMeaningFromLocation(referenceLocation) & state.searchMeaning);
                }
                function getReferencesAtLocation(sourceFile, position, search, state, addReferencesHere) {
                    var referenceLocation = ts.getTouchingPropertyName(sourceFile, position);
                    if (!isValidReferencePosition(referenceLocation, search.text)) {
                        // This wasn't the start of a token.  Check to see if it might be a
                        // match in a comment or string if that's what the caller is asking
                        // for.
                        if (!state.options.implementations && (state.options.findInStrings && ts.isInString(sourceFile, position) || state.options.findInComments && ts.isInNonReferenceComment(sourceFile, position))) {
                            // In the case where we're looking inside comments/strings, we don't have
                            // an actual definition.  So just use 'undefined' here.  Features like
                            // 'Rename' won't care (as they ignore the definitions), and features like
                            // 'FindReferences' will just filter out these results.
                            state.addStringOrCommentReference(sourceFile.fileName, ts.createTextSpan(position, search.text.length));
                        }
                        return;
                    }
                    if (!hasMatchingMeaning(referenceLocation, state))
                        return;
                    var referenceSymbol = state.checker.getSymbolAtLocation(referenceLocation);
                    if (!referenceSymbol) {
                        return;
                    }
                    var parent = referenceLocation.parent;
                    if (ts.isImportSpecifier(parent) && parent.propertyName === referenceLocation) {
                        // This is added through `singleReferences` in ImportsResult. If we happen to see it again, don't add it again.
                        return;
                    }
                    if (ts.isExportSpecifier(parent)) {
                        ts.Debug.assert(referenceLocation.kind === 75 /* Identifier */);
                        getReferencesAtExportSpecifier(referenceLocation, referenceSymbol, parent, search, state, addReferencesHere);
                        return;
                    }
                    var relatedSymbol = getRelatedSymbol(search, referenceSymbol, referenceLocation, state);
                    if (!relatedSymbol) {
                        getReferenceForShorthandProperty(referenceSymbol, search, state);
                        return;
                    }
                    switch (state.specialSearchKind) {
                        case 0 /* None */:
                            if (addReferencesHere)
                                addReference(referenceLocation, relatedSymbol, state);
                            break;
                        case 1 /* Constructor */:
                            addConstructorReferences(referenceLocation, sourceFile, search, state);
                            break;
                        case 2 /* Class */:
                            addClassStaticThisReferences(referenceLocation, search, state);
                            break;
                        default:
                            ts.Debug.assertNever(state.specialSearchKind);
                    }
                    getImportOrExportReferences(referenceLocation, referenceSymbol, search, state);
                }
                function getReferencesAtExportSpecifier(referenceLocation, referenceSymbol, exportSpecifier, search, state, addReferencesHere, alwaysGetReferences) {
                    ts.Debug.assert(!alwaysGetReferences || !!state.options.providePrefixAndSuffixTextForRename, "If alwaysGetReferences is true, then prefix/suffix text must be enabled");
                    var parent = exportSpecifier.parent, propertyName = exportSpecifier.propertyName, name = exportSpecifier.name;
                    var exportDeclaration = parent.parent;
                    var localSymbol = getLocalSymbolForExportSpecifier(referenceLocation, referenceSymbol, exportSpecifier, state.checker);
                    if (!alwaysGetReferences && !search.includes(localSymbol)) {
                        return;
                    }
                    if (!propertyName) {
                        // Don't rename at `export { default } from "m";`. (but do continue to search for imports of the re-export)
                        if (!(state.options.use === 2 /* Rename */ && (name.escapedText === "default" /* Default */))) {
                            addRef();
                        }
                    }
                    else if (referenceLocation === propertyName) {
                        // For `export { foo as bar } from "baz"`, "`foo`" will be added from the singleReferences for import searches of the original export.
                        // For `export { foo as bar };`, where `foo` is a local, so add it now.
                        if (!exportDeclaration.moduleSpecifier) {
                            addRef();
                        }
                        if (addReferencesHere && state.options.use !== 2 /* Rename */ && state.markSeenReExportRHS(name)) {
                            addReference(name, ts.Debug.assertDefined(exportSpecifier.symbol), state);
                        }
                    }
                    else {
                        if (state.markSeenReExportRHS(referenceLocation)) {
                            addRef();
                        }
                    }
                    // For `export { foo as bar }`, rename `foo`, but not `bar`.
                    if (!isForRenameWithPrefixAndSuffixText(state.options) || alwaysGetReferences) {
                        var isDefaultExport = referenceLocation.originalKeywordKind === 84 /* DefaultKeyword */
                            || exportSpecifier.name.originalKeywordKind === 84 /* DefaultKeyword */;
                        var exportKind = isDefaultExport ? 1 /* Default */ : 0 /* Named */;
                        var exportSymbol = ts.Debug.assertDefined(exportSpecifier.symbol);
                        var exportInfo = FindAllReferences.getExportInfo(exportSymbol, exportKind, state.checker);
                        if (exportInfo) {
                            searchForImportsOfExport(referenceLocation, exportSymbol, exportInfo, state);
                        }
                    }
                    // At `export { x } from "foo"`, also search for the imported symbol `"foo".x`.
                    if (search.comingFrom !== 1 /* Export */ && exportDeclaration.moduleSpecifier && !propertyName && !isForRenameWithPrefixAndSuffixText(state.options)) {
                        var imported = state.checker.getExportSpecifierLocalTargetSymbol(exportSpecifier);
                        if (imported)
                            searchForImportedSymbol(imported, state);
                    }
                    function addRef() {
                        if (addReferencesHere)
                            addReference(referenceLocation, localSymbol, state);
                    }
                }
                function getLocalSymbolForExportSpecifier(referenceLocation, referenceSymbol, exportSpecifier, checker) {
                    return isExportSpecifierAlias(referenceLocation, exportSpecifier) && checker.getExportSpecifierLocalTargetSymbol(exportSpecifier) || referenceSymbol;
                }
                function isExportSpecifierAlias(referenceLocation, exportSpecifier) {
                    var parent = exportSpecifier.parent, propertyName = exportSpecifier.propertyName, name = exportSpecifier.name;
                    ts.Debug.assert(propertyName === referenceLocation || name === referenceLocation);
                    if (propertyName) {
                        // Given `export { foo as bar } [from "someModule"]`: It's an alias at `foo`, but at `bar` it's a new symbol.
                        return propertyName === referenceLocation;
                    }
                    else {
                        // `export { foo } from "foo"` is a re-export.
                        // `export { foo };` is not a re-export, it creates an alias for the local variable `foo`.
                        return !parent.parent.moduleSpecifier;
                    }
                }
                function getImportOrExportReferences(referenceLocation, referenceSymbol, search, state) {
                    var importOrExport = FindAllReferences.getImportOrExportSymbol(referenceLocation, referenceSymbol, state.checker, search.comingFrom === 1 /* Export */);
                    if (!importOrExport)
                        return;
                    var symbol = importOrExport.symbol;
                    if (importOrExport.kind === 0 /* Import */) {
                        if (!(isForRenameWithPrefixAndSuffixText(state.options))) {
                            searchForImportedSymbol(symbol, state);
                        }
                    }
                    else {
                        searchForImportsOfExport(referenceLocation, symbol, importOrExport.exportInfo, state);
                    }
                }
                function getReferenceForShorthandProperty(_a, search, state) {
                    var flags = _a.flags, valueDeclaration = _a.valueDeclaration;
                    var shorthandValueSymbol = state.checker.getShorthandAssignmentValueSymbol(valueDeclaration);
                    var name = valueDeclaration && ts.getNameOfDeclaration(valueDeclaration);
                    /*
                    * Because in short-hand property assignment, an identifier which stored as name of the short-hand property assignment
                    * has two meanings: property name and property value. Therefore when we do findAllReference at the position where
                    * an identifier is declared, the language service should return the position of the variable declaration as well as
                    * the position in short-hand property assignment excluding property accessing. However, if we do findAllReference at the
                    * position of property accessing, the referenceEntry of such position will be handled in the first case.
                    */
                    if (!(flags & 33554432 /* Transient */) && name && search.includes(shorthandValueSymbol)) {
                        addReference(name, shorthandValueSymbol, state);
                    }
                }
                function addReference(referenceLocation, relatedSymbol, state) {
                    var _a = "kind" in relatedSymbol ? relatedSymbol : { kind: undefined, symbol: relatedSymbol }, kind = _a.kind, symbol = _a.symbol; // eslint-disable-line no-in-operator
                    var addRef = state.referenceAdder(symbol);
                    if (state.options.implementations) {
                        addImplementationReferences(referenceLocation, addRef, state);
                    }
                    else {
                        addRef(referenceLocation, kind);
                    }
                }
                /** Adds references when a constructor is used with `new this()` in its own class and `super()` calls in subclasses.  */
                function addConstructorReferences(referenceLocation, sourceFile, search, state) {
                    if (ts.isNewExpressionTarget(referenceLocation)) {
                        addReference(referenceLocation, search.symbol, state);
                    }
                    var pusher = function () { return state.referenceAdder(search.symbol); };
                    if (ts.isClassLike(referenceLocation.parent)) {
                        ts.Debug.assert(referenceLocation.kind === 84 /* DefaultKeyword */ || referenceLocation.parent.name === referenceLocation);
                        // This is the class declaration containing the constructor.
                        findOwnConstructorReferences(search.symbol, sourceFile, pusher());
                    }
                    else {
                        // If this class appears in `extends C`, then the extending class' "super" calls are references.
                        var classExtending = tryGetClassByExtendingIdentifier(referenceLocation);
                        if (classExtending) {
                            findSuperConstructorAccesses(classExtending, pusher());
                            findInheritedConstructorReferences(classExtending, state);
                        }
                    }
                }
                function addClassStaticThisReferences(referenceLocation, search, state) {
                    addReference(referenceLocation, search.symbol, state);
                    var classLike = referenceLocation.parent;
                    if (state.options.use === 2 /* Rename */ || !ts.isClassLike(classLike))
                        return;
                    ts.Debug.assert(classLike.name === referenceLocation);
                    var addRef = state.referenceAdder(search.symbol);
                    for (var _i = 0, _a = classLike.members; _i < _a.length; _i++) {
                        var member = _a[_i];
                        if (!(ts.isMethodOrAccessor(member) && ts.hasModifier(member, 32 /* Static */))) {
                            continue;
                        }
                        if (member.body) {
                            member.body.forEachChild(function cb(node) {
                                if (node.kind === 104 /* ThisKeyword */) {
                                    addRef(node);
                                }
                                else if (!ts.isFunctionLike(node) && !ts.isClassLike(node)) {
                                    node.forEachChild(cb);
                                }
                            });
                        }
                    }
                }
                /**
                 * `classSymbol` is the class where the constructor was defined.
                 * Reference the constructor and all calls to `new this()`.
                 */
                function findOwnConstructorReferences(classSymbol, sourceFile, addNode) {
                    var constructorSymbol = getClassConstructorSymbol(classSymbol);
                    if (constructorSymbol) {
                        for (var _i = 0, _a = constructorSymbol.declarations; _i < _a.length; _i++) {
                            var decl = _a[_i];
                            var ctrKeyword = ts.findChildOfKind(decl, 129 /* ConstructorKeyword */, sourceFile);
                            ts.Debug.assert(decl.kind === 162 /* Constructor */ && !!ctrKeyword);
                            addNode(ctrKeyword);
                        }
                    }
                    if (classSymbol.exports) {
                        classSymbol.exports.forEach(function (member) {
                            var decl = member.valueDeclaration;
                            if (decl && decl.kind === 161 /* MethodDeclaration */) {
                                var body = decl.body;
                                if (body) {
                                    forEachDescendantOfKind(body, 104 /* ThisKeyword */, function (thisKeyword) {
                                        if (ts.isNewExpressionTarget(thisKeyword)) {
                                            addNode(thisKeyword);
                                        }
                                    });
                                }
                            }
                        });
                    }
                }
                function getClassConstructorSymbol(classSymbol) {
                    return classSymbol.members && classSymbol.members.get("__constructor" /* Constructor */);
                }
                /** Find references to `super` in the constructor of an extending class.  */
                function findSuperConstructorAccesses(classDeclaration, addNode) {
                    var constructor = getClassConstructorSymbol(classDeclaration.symbol);
                    if (!constructor) {
                        return;
                    }
                    for (var _i = 0, _a = constructor.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        ts.Debug.assert(decl.kind === 162 /* Constructor */);
                        var body = decl.body;
                        if (body) {
                            forEachDescendantOfKind(body, 102 /* SuperKeyword */, function (node) {
                                if (ts.isCallExpressionTarget(node)) {
                                    addNode(node);
                                }
                            });
                        }
                    }
                }
                function hasOwnConstructor(classDeclaration) {
                    return !!getClassConstructorSymbol(classDeclaration.symbol);
                }
                function findInheritedConstructorReferences(classDeclaration, state) {
                    if (hasOwnConstructor(classDeclaration))
                        return;
                    var classSymbol = classDeclaration.symbol;
                    var search = state.createSearch(/*location*/ undefined, classSymbol, /*comingFrom*/ undefined);
                    getReferencesInContainerOrFiles(classSymbol, state, search);
                }
                function addImplementationReferences(refNode, addReference, state) {
                    // Check if we found a function/propertyAssignment/method with an implementation or initializer
                    if (ts.isDeclarationName(refNode) && isImplementation(refNode.parent)) {
                        addReference(refNode);
                        return;
                    }
                    if (refNode.kind !== 75 /* Identifier */) {
                        return;
                    }
                    if (refNode.parent.kind === 282 /* ShorthandPropertyAssignment */) {
                        // Go ahead and dereference the shorthand assignment by going to its definition
                        getReferenceEntriesForShorthandPropertyAssignment(refNode, state.checker, addReference);
                    }
                    // Check if the node is within an extends or implements clause
                    var containingClass = getContainingClassIfInHeritageClause(refNode);
                    if (containingClass) {
                        addReference(containingClass);
                        return;
                    }
                    // If we got a type reference, try and see if the reference applies to any expressions that can implement an interface
                    // Find the first node whose parent isn't a type node -- i.e., the highest type node.
                    var typeNode = ts.findAncestor(refNode, function (a) { return !ts.isQualifiedName(a.parent) && !ts.isTypeNode(a.parent) && !ts.isTypeElement(a.parent); });
                    var typeHavingNode = typeNode.parent;
                    if (ts.hasType(typeHavingNode) && typeHavingNode.type === typeNode && state.markSeenContainingTypeReference(typeHavingNode)) {
                        if (ts.hasInitializer(typeHavingNode)) {
                            addIfImplementation(typeHavingNode.initializer);
                        }
                        else if (ts.isFunctionLike(typeHavingNode) && typeHavingNode.body) {
                            var body = typeHavingNode.body;
                            if (body.kind === 223 /* Block */) {
                                ts.forEachReturnStatement(body, function (returnStatement) {
                                    if (returnStatement.expression)
                                        addIfImplementation(returnStatement.expression);
                                });
                            }
                            else {
                                addIfImplementation(body);
                            }
                        }
                        else if (ts.isAssertionExpression(typeHavingNode)) {
                            addIfImplementation(typeHavingNode.expression);
                        }
                    }
                    function addIfImplementation(e) {
                        if (isImplementationExpression(e))
                            addReference(e);
                    }
                }
                function getContainingClassIfInHeritageClause(node) {
                    return ts.isIdentifier(node) || ts.isPropertyAccessExpression(node) ? getContainingClassIfInHeritageClause(node.parent)
                        : ts.isExpressionWithTypeArguments(node) ? ts.tryCast(node.parent.parent, ts.isClassLike) : undefined;
                }
                /**
                 * Returns true if this is an expression that can be considered an implementation
                 */
                function isImplementationExpression(node) {
                    switch (node.kind) {
                        case 200 /* ParenthesizedExpression */:
                            return isImplementationExpression(node.expression);
                        case 202 /* ArrowFunction */:
                        case 201 /* FunctionExpression */:
                        case 193 /* ObjectLiteralExpression */:
                        case 214 /* ClassExpression */:
                        case 192 /* ArrayLiteralExpression */:
                            return true;
                        default:
                            return false;
                    }
                }
                /**
                 * Determines if the parent symbol occurs somewhere in the child's ancestry. If the parent symbol
                 * is an interface, determines if some ancestor of the child symbol extends or inherits from it.
                 * Also takes in a cache of previous results which makes this slightly more efficient and is
                 * necessary to avoid potential loops like so:
                 *     class A extends B { }
                 *     class B extends A { }
                 *
                 * We traverse the AST rather than using the type checker because users are typically only interested
                 * in explicit implementations of an interface/class when calling "Go to Implementation". Sibling
                 * implementations of types that share a common ancestor with the type whose implementation we are
                 * searching for need to be filtered out of the results. The type checker doesn't let us make the
                 * distinction between structurally compatible implementations and explicit implementations, so we
                 * must use the AST.
                 *
                 * @param symbol         A class or interface Symbol
                 * @param parent        Another class or interface Symbol
                 * @param cachedResults A map of symbol id pairs (i.e. "child,parent") to booleans indicating previous results
                 */
                function explicitlyInheritsFrom(symbol, parent, cachedResults, checker) {
                    if (symbol === parent) {
                        return true;
                    }
                    var key = ts.getSymbolId(symbol) + "," + ts.getSymbolId(parent);
                    var cached = cachedResults.get(key);
                    if (cached !== undefined) {
                        return cached;
                    }
                    // Set the key so that we don't infinitely recurse
                    cachedResults.set(key, false);
                    var inherits = symbol.declarations.some(function (declaration) {
                        return ts.getAllSuperTypeNodes(declaration).some(function (typeReference) {
                            var type = checker.getTypeAtLocation(typeReference);
                            return !!type && !!type.symbol && explicitlyInheritsFrom(type.symbol, parent, cachedResults, checker);
                        });
                    });
                    cachedResults.set(key, inherits);
                    return inherits;
                }
                function getReferencesForSuperKeyword(superKeyword) {
                    var searchSpaceNode = ts.getSuperContainer(superKeyword, /*stopOnFunctions*/ false);
                    if (!searchSpaceNode) {
                        return undefined;
                    }
                    // Whether 'super' occurs in a static context within a class.
                    var staticFlag = 32 /* Static */;
                    switch (searchSpaceNode.kind) {
                        case 159 /* PropertyDeclaration */:
                        case 158 /* PropertySignature */:
                        case 161 /* MethodDeclaration */:
                        case 160 /* MethodSignature */:
                        case 162 /* Constructor */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            staticFlag &= ts.getModifierFlags(searchSpaceNode);
                            searchSpaceNode = searchSpaceNode.parent; // re-assign to be the owning class
                            break;
                        default:
                            return undefined;
                    }
                    var sourceFile = searchSpaceNode.getSourceFile();
                    var references = ts.mapDefined(getPossibleSymbolReferenceNodes(sourceFile, "super", searchSpaceNode), function (node) {
                        if (node.kind !== 102 /* SuperKeyword */) {
                            return;
                        }
                        var container = ts.getSuperContainer(node, /*stopOnFunctions*/ false);
                        // If we have a 'super' container, we must have an enclosing class.
                        // Now make sure the owning class is the same as the search-space
                        // and has the same static qualifier as the original 'super's owner.
                        return container && (32 /* Static */ & ts.getModifierFlags(container)) === staticFlag && container.parent.symbol === searchSpaceNode.symbol ? nodeEntry(node) : undefined;
                    });
                    return [{ definition: { type: 0 /* Symbol */, symbol: searchSpaceNode.symbol }, references: references }];
                }
                function isParameterName(node) {
                    return node.kind === 75 /* Identifier */ && node.parent.kind === 156 /* Parameter */ && node.parent.name === node;
                }
                function getReferencesForThisKeyword(thisOrSuperKeyword, sourceFiles, cancellationToken) {
                    var searchSpaceNode = ts.getThisContainer(thisOrSuperKeyword, /* includeArrowFunctions */ false);
                    // Whether 'this' occurs in a static context within a class.
                    var staticFlag = 32 /* Static */;
                    switch (searchSpaceNode.kind) {
                        case 161 /* MethodDeclaration */:
                        case 160 /* MethodSignature */:
                            if (ts.isObjectLiteralMethod(searchSpaceNode)) {
                                break;
                            }
                        // falls through
                        case 159 /* PropertyDeclaration */:
                        case 158 /* PropertySignature */:
                        case 162 /* Constructor */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            staticFlag &= ts.getModifierFlags(searchSpaceNode);
                            searchSpaceNode = searchSpaceNode.parent; // re-assign to be the owning class
                            break;
                        case 290 /* SourceFile */:
                            if (ts.isExternalModule(searchSpaceNode) || isParameterName(thisOrSuperKeyword)) {
                                return undefined;
                            }
                        // falls through
                        case 244 /* FunctionDeclaration */:
                        case 201 /* FunctionExpression */:
                            break;
                        // Computed properties in classes are not handled here because references to this are illegal,
                        // so there is no point finding references to them.
                        default:
                            return undefined;
                    }
                    var references = ts.flatMap(searchSpaceNode.kind === 290 /* SourceFile */ ? sourceFiles : [searchSpaceNode.getSourceFile()], function (sourceFile) {
                        cancellationToken.throwIfCancellationRequested();
                        return getPossibleSymbolReferenceNodes(sourceFile, "this", ts.isSourceFile(searchSpaceNode) ? sourceFile : searchSpaceNode).filter(function (node) {
                            if (!ts.isThis(node)) {
                                return false;
                            }
                            var container = ts.getThisContainer(node, /* includeArrowFunctions */ false);
                            switch (searchSpaceNode.kind) {
                                case 201 /* FunctionExpression */:
                                case 244 /* FunctionDeclaration */:
                                    return searchSpaceNode.symbol === container.symbol;
                                case 161 /* MethodDeclaration */:
                                case 160 /* MethodSignature */:
                                    return ts.isObjectLiteralMethod(searchSpaceNode) && searchSpaceNode.symbol === container.symbol;
                                case 214 /* ClassExpression */:
                                case 245 /* ClassDeclaration */:
                                    // Make sure the container belongs to the same class
                                    // and has the appropriate static modifier from the original container.
                                    return container.parent && searchSpaceNode.symbol === container.parent.symbol && (ts.getModifierFlags(container) & 32 /* Static */) === staticFlag;
                                case 290 /* SourceFile */:
                                    return container.kind === 290 /* SourceFile */ && !ts.isExternalModule(container) && !isParameterName(node);
                            }
                        });
                    }).map(function (n) { return nodeEntry(n); });
                    var thisParameter = ts.firstDefined(references, function (r) { return ts.isParameter(r.node.parent) ? r.node : undefined; });
                    return [{
                            definition: { type: 3 /* This */, node: thisParameter || thisOrSuperKeyword },
                            references: references
                        }];
                }
                function getReferencesForStringLiteral(node, sourceFiles, cancellationToken) {
                    var references = ts.flatMap(sourceFiles, function (sourceFile) {
                        cancellationToken.throwIfCancellationRequested();
                        return ts.mapDefined(getPossibleSymbolReferenceNodes(sourceFile, node.text), function (ref) {
                            return ts.isStringLiteral(ref) && ref.text === node.text ? nodeEntry(ref, 2 /* StringLiteral */) : undefined;
                        });
                    });
                    return [{
                            definition: { type: 4 /* String */, node: node },
                            references: references
                        }];
                }
                // For certain symbol kinds, we need to include other symbols in the search set.
                // This is not needed when searching for re-exports.
                function populateSearchSymbolSet(symbol, location, checker, isForRename, providePrefixAndSuffixText, implementations) {
                    var result = [];
                    forEachRelatedSymbol(symbol, location, checker, isForRename, !(isForRename && providePrefixAndSuffixText), function (sym, root, base) { result.push(base || root || sym); }, 
                    /*allowBaseTypes*/ function () { return !implementations; });
                    return result;
                }
                function forEachRelatedSymbol(symbol, location, checker, isForRenamePopulateSearchSymbolSet, onlyIncludeBindingElementAtReferenceLocation, cbSymbol, allowBaseTypes) {
                    var containingObjectLiteralElement = ts.getContainingObjectLiteralElement(location);
                    if (containingObjectLiteralElement) {
                        /* Because in short-hand property assignment, location has two meaning : property name and as value of the property
                        * When we do findAllReference at the position of the short-hand property assignment, we would want to have references to position of
                        * property name and variable declaration of the identifier.
                        * Like in below example, when querying for all references for an identifier 'name', of the property assignment, the language service
                        * should show both 'name' in 'obj' and 'name' in variable declaration
                        *      const name = "Foo";
                        *      const obj = { name };
                        * In order to do that, we will populate the search set with the value symbol of the identifier as a value of the property assignment
                        * so that when matching with potential reference symbol, both symbols from property declaration and variable declaration
                        * will be included correctly.
                        */
                        var shorthandValueSymbol = checker.getShorthandAssignmentValueSymbol(location.parent); // gets the local symbol
                        if (shorthandValueSymbol && isForRenamePopulateSearchSymbolSet) {
                            // When renaming 'x' in `const o = { x }`, just rename the local variable, not the property.
                            return cbSymbol(shorthandValueSymbol, /*rootSymbol*/ undefined, /*baseSymbol*/ undefined, 3 /* SearchedLocalFoundProperty */);
                        }
                        // If the location is in a context sensitive location (i.e. in an object literal) try
                        // to get a contextual type for it, and add the property symbol from the contextual
                        // type to the search set
                        var contextualType = checker.getContextualType(containingObjectLiteralElement.parent);
                        var res_1 = contextualType && ts.firstDefined(ts.getPropertySymbolsFromContextualType(containingObjectLiteralElement, checker, contextualType, /*unionSymbolOk*/ true), function (sym) { return fromRoot(sym, 4 /* SearchedPropertyFoundLocal */); });
                        if (res_1)
                            return res_1;
                        // If the location is name of property symbol from object literal destructuring pattern
                        // Search the property symbol
                        //      for ( { property: p2 } of elems) { }
                        var propertySymbol = getPropertySymbolOfDestructuringAssignment(location, checker);
                        var res1 = propertySymbol && cbSymbol(propertySymbol, /*rootSymbol*/ undefined, /*baseSymbol*/ undefined, 4 /* SearchedPropertyFoundLocal */);
                        if (res1)
                            return res1;
                        var res2 = shorthandValueSymbol && cbSymbol(shorthandValueSymbol, /*rootSymbol*/ undefined, /*baseSymbol*/ undefined, 3 /* SearchedLocalFoundProperty */);
                        if (res2)
                            return res2;
                    }
                    var aliasedSymbol = getMergedAliasedSymbolOfNamespaceExportDeclaration(location, symbol, checker);
                    if (aliasedSymbol) {
                        // In case of UMD module and global merging, search for global as well
                        var res_2 = cbSymbol(aliasedSymbol, /*rootSymbol*/ undefined, /*baseSymbol*/ undefined, 1 /* Node */);
                        if (res_2)
                            return res_2;
                    }
                    var res = fromRoot(symbol);
                    if (res)
                        return res;
                    if (symbol.valueDeclaration && ts.isParameterPropertyDeclaration(symbol.valueDeclaration, symbol.valueDeclaration.parent)) {
                        // For a parameter property, now try on the other symbol (property if this was a parameter, parameter if this was a property).
                        var paramProps = checker.getSymbolsOfParameterPropertyDeclaration(ts.cast(symbol.valueDeclaration, ts.isParameter), symbol.name);
                        ts.Debug.assert(paramProps.length === 2 && !!(paramProps[0].flags & 1 /* FunctionScopedVariable */) && !!(paramProps[1].flags & 4 /* Property */)); // is [parameter, property]
                        return fromRoot(symbol.flags & 1 /* FunctionScopedVariable */ ? paramProps[1] : paramProps[0]);
                    }
                    var exportSpecifier = ts.getDeclarationOfKind(symbol, 263 /* ExportSpecifier */);
                    if (!isForRenamePopulateSearchSymbolSet || exportSpecifier && !exportSpecifier.propertyName) {
                        var localSymbol = exportSpecifier && checker.getExportSpecifierLocalTargetSymbol(exportSpecifier);
                        if (localSymbol) {
                            var res_3 = cbSymbol(localSymbol, /*rootSymbol*/ undefined, /*baseSymbol*/ undefined, 1 /* Node */);
                            if (res_3)
                                return res_3;
                        }
                    }
                    // symbolAtLocation for a binding element is the local symbol. See if the search symbol is the property.
                    // Don't do this when populating search set for a rename when prefix and suffix text will be provided -- just rename the local.
                    if (!isForRenamePopulateSearchSymbolSet) {
                        var bindingElementPropertySymbol = void 0;
                        if (onlyIncludeBindingElementAtReferenceLocation) {
                            bindingElementPropertySymbol = ts.isObjectBindingElementWithoutPropertyName(location.parent) ? ts.getPropertySymbolFromBindingElement(checker, location.parent) : undefined;
                        }
                        else {
                            bindingElementPropertySymbol = getPropertySymbolOfObjectBindingPatternWithoutPropertyName(symbol, checker);
                        }
                        return bindingElementPropertySymbol && fromRoot(bindingElementPropertySymbol, 4 /* SearchedPropertyFoundLocal */);
                    }
                    ts.Debug.assert(isForRenamePopulateSearchSymbolSet);
                    // due to the above assert and the arguments at the uses of this function,
                    // (onlyIncludeBindingElementAtReferenceLocation <=> !providePrefixAndSuffixTextForRename) holds
                    var includeOriginalSymbolOfBindingElement = onlyIncludeBindingElementAtReferenceLocation;
                    if (includeOriginalSymbolOfBindingElement) {
                        var bindingElementPropertySymbol = getPropertySymbolOfObjectBindingPatternWithoutPropertyName(symbol, checker);
                        return bindingElementPropertySymbol && fromRoot(bindingElementPropertySymbol, 4 /* SearchedPropertyFoundLocal */);
                    }
                    function fromRoot(sym, kind) {
                        // If this is a union property:
                        //   - In populateSearchSymbolsSet we will add all the symbols from all its source symbols in all unioned types.
                        //   - In findRelatedSymbol, we will just use the union symbol if any source symbol is included in the search.
                        // If the symbol is an instantiation from a another symbol (e.g. widened symbol):
                        //   - In populateSearchSymbolsSet, add the root the list
                        //   - In findRelatedSymbol, return the source symbol if that is in the search. (Do not return the instantiation symbol.)
                        return ts.firstDefined(checker.getRootSymbols(sym), function (rootSymbol) {
                            return cbSymbol(sym, rootSymbol, /*baseSymbol*/ undefined, kind)
                                // Add symbol of properties/methods of the same name in base classes and implemented interfaces definitions
                                || (rootSymbol.parent && rootSymbol.parent.flags & (32 /* Class */ | 64 /* Interface */) && allowBaseTypes(rootSymbol)
                                    ? ts.getPropertySymbolsFromBaseTypes(rootSymbol.parent, rootSymbol.name, checker, function (base) { return cbSymbol(sym, rootSymbol, base, kind); })
                                    : undefined);
                        });
                    }
                    function getPropertySymbolOfObjectBindingPatternWithoutPropertyName(symbol, checker) {
                        var bindingElement = ts.getDeclarationOfKind(symbol, 191 /* BindingElement */);
                        if (bindingElement && ts.isObjectBindingElementWithoutPropertyName(bindingElement)) {
                            return ts.getPropertySymbolFromBindingElement(checker, bindingElement);
                        }
                    }
                }
                function getRelatedSymbol(search, referenceSymbol, referenceLocation, state) {
                    var checker = state.checker;
                    return forEachRelatedSymbol(referenceSymbol, referenceLocation, checker, /*isForRenamePopulateSearchSymbolSet*/ false, 
                    /*onlyIncludeBindingElementAtReferenceLocation*/ state.options.use !== 2 /* Rename */ || !!state.options.providePrefixAndSuffixTextForRename, function (sym, rootSymbol, baseSymbol, kind) {
                        return search.includes(baseSymbol || rootSymbol || sym)
                            // For a base type, use the symbol for the derived type. For a synthetic (e.g. union) property, use the union symbol.
                            ? { symbol: rootSymbol && !(ts.getCheckFlags(sym) & 6 /* Synthetic */) ? rootSymbol : sym, kind: kind }
                            : undefined;
                    }, 
                    /*allowBaseTypes*/ function (rootSymbol) {
                        return !(search.parents && !search.parents.some(function (parent) { return explicitlyInheritsFrom(rootSymbol.parent, parent, state.inheritsFromCache, checker); }));
                    });
                }
                /**
                 * Given an initial searchMeaning, extracted from a location, widen the search scope based on the declarations
                 * of the corresponding symbol. e.g. if we are searching for "Foo" in value position, but "Foo" references a class
                 * then we need to widen the search to include type positions as well.
                 * On the contrary, if we are searching for "Bar" in type position and we trace bar to an interface, and an uninstantiated
                 * module, we want to keep the search limited to only types, as the two declarations (interface and uninstantiated module)
                 * do not intersect in any of the three spaces.
                 */
                function getIntersectingMeaningFromDeclarations(node, symbol) {
                    var meaning = ts.getMeaningFromLocation(node);
                    var declarations = symbol.declarations;
                    if (declarations) {
                        var lastIterationMeaning = void 0;
                        do {
                            // The result is order-sensitive, for instance if initialMeaning === Namespace, and declarations = [class, instantiated module]
                            // we need to consider both as they initialMeaning intersects with the module in the namespace space, and the module
                            // intersects with the class in the value space.
                            // To achieve that we will keep iterating until the result stabilizes.
                            // Remember the last meaning
                            lastIterationMeaning = meaning;
                            for (var _i = 0, declarations_2 = declarations; _i < declarations_2.length; _i++) {
                                var declaration = declarations_2[_i];
                                var declarationMeaning = ts.getMeaningFromDeclaration(declaration);
                                if (declarationMeaning & meaning) {
                                    meaning |= declarationMeaning;
                                }
                            }
                        } while (meaning !== lastIterationMeaning);
                    }
                    return meaning;
                }
                Core.getIntersectingMeaningFromDeclarations = getIntersectingMeaningFromDeclarations;
                function isImplementation(node) {
                    return !!(node.flags & 8388608 /* Ambient */) ? !(ts.isInterfaceDeclaration(node) || ts.isTypeAliasDeclaration(node)) :
                        (ts.isVariableLike(node) ? ts.hasInitializer(node) :
                            ts.isFunctionLikeDeclaration(node) ? !!node.body :
                                ts.isClassLike(node) || ts.isModuleOrEnumDeclaration(node));
                }
                function getReferenceEntriesForShorthandPropertyAssignment(node, checker, addReference) {
                    var refSymbol = checker.getSymbolAtLocation(node);
                    var shorthandSymbol = checker.getShorthandAssignmentValueSymbol(refSymbol.valueDeclaration);
                    if (shorthandSymbol) {
                        for (var _i = 0, _a = shorthandSymbol.getDeclarations(); _i < _a.length; _i++) {
                            var declaration = _a[_i];
                            if (ts.getMeaningFromDeclaration(declaration) & 1 /* Value */) {
                                addReference(declaration);
                            }
                        }
                    }
                }
                Core.getReferenceEntriesForShorthandPropertyAssignment = getReferenceEntriesForShorthandPropertyAssignment;
                function forEachDescendantOfKind(node, kind, action) {
                    ts.forEachChild(node, function (child) {
                        if (child.kind === kind) {
                            action(child);
                        }
                        forEachDescendantOfKind(child, kind, action);
                    });
                }
                /** Get `C` given `N` if `N` is in the position `class C extends N` or `class C extends foo.N` where `N` is an identifier. */
                function tryGetClassByExtendingIdentifier(node) {
                    return ts.tryGetClassExtendingExpressionWithTypeArguments(ts.climbPastPropertyAccess(node).parent);
                }
                /**
                 * If we are just looking for implementations and this is a property access expression, we need to get the
                 * symbol of the local type of the symbol the property is being accessed on. This is because our search
                 * symbol may have a different parent symbol if the local type's symbol does not declare the property
                 * being accessed (i.e. it is declared in some parent class or interface)
                 */
                function getParentSymbolsOfPropertyAccess(location, symbol, checker) {
                    var propertyAccessExpression = ts.isRightSideOfPropertyAccess(location) ? location.parent : undefined;
                    var lhsType = propertyAccessExpression && checker.getTypeAtLocation(propertyAccessExpression.expression);
                    var res = ts.mapDefined(lhsType && (lhsType.isUnionOrIntersection() ? lhsType.types : lhsType.symbol === symbol.parent ? undefined : [lhsType]), function (t) {
                        return t.symbol && t.symbol.flags & (32 /* Class */ | 64 /* Interface */) ? t.symbol : undefined;
                    });
                    return res.length === 0 ? undefined : res;
                }
                function isForRenameWithPrefixAndSuffixText(options) {
                    return options.use === 2 /* Rename */ && options.providePrefixAndSuffixTextForRename;
                }
            })(Core = FindAllReferences.Core || (FindAllReferences.Core = {}));
        })(FindAllReferences = ts.FindAllReferences || (ts.FindAllReferences = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var CallHierarchy;
        (function (CallHierarchy) {
            /** Indictates whether a node is named function or class expression. */
            function isNamedExpression(node) {
                return (ts.isFunctionExpression(node) || ts.isClassExpression(node)) && ts.isNamedDeclaration(node);
            }
            /** Indicates whether a node is a function, arrow, or class expression assigned to a constant variable. */
            function isConstNamedExpression(node) {
                return (ts.isFunctionExpression(node) || ts.isArrowFunction(node) || ts.isClassExpression(node))
                    && ts.isVariableDeclaration(node.parent)
                    && node === node.parent.initializer
                    && ts.isIdentifier(node.parent.name)
                    && !!(ts.getCombinedNodeFlags(node.parent) & 2 /* Const */);
            }
            /**
             * Indicates whether a node could possibly be a call hierarchy declaration.
             *
             * See `resolveCallHierarchyDeclaration` for the specific rules.
             */
            function isPossibleCallHierarchyDeclaration(node) {
                return ts.isSourceFile(node)
                    || ts.isModuleDeclaration(node)
                    || ts.isFunctionDeclaration(node)
                    || ts.isFunctionExpression(node)
                    || ts.isClassDeclaration(node)
                    || ts.isClassExpression(node)
                    || ts.isMethodDeclaration(node)
                    || ts.isMethodSignature(node)
                    || ts.isGetAccessorDeclaration(node)
                    || ts.isSetAccessorDeclaration(node);
            }
            /**
             * Indicates whether a node is a valid a call hierarchy declaration.
             *
             * See `resolveCallHierarchyDeclaration` for the specific rules.
             */
            function isValidCallHierarchyDeclaration(node) {
                return ts.isSourceFile(node)
                    || ts.isModuleDeclaration(node) && ts.isIdentifier(node.name)
                    || ts.isFunctionDeclaration(node)
                    || ts.isClassDeclaration(node)
                    || ts.isMethodDeclaration(node)
                    || ts.isMethodSignature(node)
                    || ts.isGetAccessorDeclaration(node)
                    || ts.isSetAccessorDeclaration(node)
                    || isNamedExpression(node)
                    || isConstNamedExpression(node);
            }
            /** Gets the node that can be used as a reference to a call hierarchy declaration. */
            function getCallHierarchyDeclarationReferenceNode(node) {
                if (ts.isSourceFile(node))
                    return node;
                if (ts.isNamedDeclaration(node))
                    return node.name;
                if (isConstNamedExpression(node))
                    return node.parent.name;
                return ts.Debug.assertDefined(node.modifiers && ts.find(node.modifiers, isDefaultModifier));
            }
            function isDefaultModifier(node) {
                return node.kind === 84 /* DefaultKeyword */;
            }
            /** Gets the symbol for a call hierarchy declaration. */
            function getSymbolOfCallHierarchyDeclaration(typeChecker, node) {
                var location = getCallHierarchyDeclarationReferenceNode(node);
                return location && typeChecker.getSymbolAtLocation(location);
            }
            /** Gets the text and range for the name of a call hierarchy declaration. */
            function getCallHierarchyItemName(program, node) {
                if (ts.isSourceFile(node)) {
                    return { text: node.fileName, pos: 0, end: 0 };
                }
                if ((ts.isFunctionDeclaration(node) || ts.isClassDeclaration(node)) && !ts.isNamedDeclaration(node)) {
                    var defaultModifier = node.modifiers && ts.find(node.modifiers, isDefaultModifier);
                    if (defaultModifier) {
                        return { text: "default", pos: defaultModifier.getStart(), end: defaultModifier.getEnd() };
                    }
                }
                var declName = isConstNamedExpression(node) ? node.parent.name :
                    ts.Debug.assertDefined(ts.getNameOfDeclaration(node), "Expected call hierarchy item to have a name");
                var text = ts.isIdentifier(declName) ? ts.idText(declName) :
                    ts.isStringOrNumericLiteralLike(declName) ? declName.text :
                        ts.isComputedPropertyName(declName) ?
                            ts.isStringOrNumericLiteralLike(declName.expression) ? declName.expression.text :
                                undefined :
                            undefined;
                if (text === undefined) {
                    var typeChecker = program.getTypeChecker();
                    var symbol = typeChecker.getSymbolAtLocation(declName);
                    if (symbol) {
                        text = typeChecker.symbolToString(symbol, node);
                    }
                }
                if (text === undefined) {
                    // get the text from printing the node on a single line without comments...
                    var printer_1 = ts.createPrinter({ removeComments: true, omitTrailingSemicolon: true });
                    text = ts.usingSingleLineStringWriter(function (writer) { return printer_1.writeNode(4 /* Unspecified */, node, node.getSourceFile(), writer); });
                }
                return { text: text, pos: declName.getStart(), end: declName.getEnd() };
            }
            function findImplementation(typeChecker, node) {
                if (node.body) {
                    return node;
                }
                if (ts.isConstructorDeclaration(node)) {
                    return ts.getFirstConstructorWithBody(node.parent);
                }
                if (ts.isFunctionDeclaration(node) || ts.isMethodDeclaration(node)) {
                    var symbol = getSymbolOfCallHierarchyDeclaration(typeChecker, node);
                    if (symbol && symbol.valueDeclaration && ts.isFunctionLikeDeclaration(symbol.valueDeclaration) && symbol.valueDeclaration.body) {
                        return symbol.valueDeclaration;
                    }
                    return undefined;
                }
                return node;
            }
            function findAllInitialDeclarations(typeChecker, node) {
                var symbol = getSymbolOfCallHierarchyDeclaration(typeChecker, node);
                var declarations;
                if (symbol && symbol.declarations) {
                    var indices = ts.indicesOf(symbol.declarations);
                    var keys_1 = ts.map(symbol.declarations, function (decl) { return ({ file: decl.getSourceFile().fileName, pos: decl.pos }); });
                    indices.sort(function (a, b) { return ts.compareStringsCaseSensitive(keys_1[a].file, keys_1[b].file) || keys_1[a].pos - keys_1[b].pos; });
                    var sortedDeclarations = ts.map(indices, function (i) { return symbol.declarations[i]; });
                    var lastDecl = void 0;
                    for (var _i = 0, sortedDeclarations_1 = sortedDeclarations; _i < sortedDeclarations_1.length; _i++) {
                        var decl = sortedDeclarations_1[_i];
                        if (isValidCallHierarchyDeclaration(decl)) {
                            if (!lastDecl || lastDecl.parent !== decl.parent || lastDecl.end !== decl.pos) {
                                declarations = ts.append(declarations, decl);
                            }
                            lastDecl = decl;
                        }
                    }
                }
                return declarations;
            }
            /** Find the implementation or the first declaration for a call hierarchy declaration. */
            function findImplementationOrAllInitialDeclarations(typeChecker, node) {
                var _a, _b, _c;
                if (ts.isFunctionLikeDeclaration(node)) {
                    return (_b = (_a = findImplementation(typeChecker, node)) !== null && _a !== void 0 ? _a : findAllInitialDeclarations(typeChecker, node)) !== null && _b !== void 0 ? _b : node;
                }
                return (_c = findAllInitialDeclarations(typeChecker, node)) !== null && _c !== void 0 ? _c : node;
            }
            /** Resolves the call hierarchy declaration for a node. */
            function resolveCallHierarchyDeclaration(program, location) {
                // A call hierarchy item must refer to either a SourceFile, Module Declaration, or something intrinsically callable that has a name:
                // - Class Declarations
                // - Class Expressions (with a name)
                // - Function Declarations
                // - Function Expressions (with a name or assigned to a const variable)
                // - Arrow Functions (assigned to a const variable)
                // - Constructors
                // - Methods
                // - Accessors
                //
                // If a call is contained in a non-named callable Node (function expression, arrow function, etc.), then
                // its containing `CallHierarchyItem` is a containing function or SourceFile that matches the above list.
                var typeChecker = program.getTypeChecker();
                var followingSymbol = false;
                while (true) {
                    if (isValidCallHierarchyDeclaration(location)) {
                        return findImplementationOrAllInitialDeclarations(typeChecker, location);
                    }
                    if (isPossibleCallHierarchyDeclaration(location)) {
                        var ancestor = ts.findAncestor(location, isValidCallHierarchyDeclaration);
                        return ancestor && findImplementationOrAllInitialDeclarations(typeChecker, ancestor);
                    }
                    if (ts.isDeclarationName(location)) {
                        if (isValidCallHierarchyDeclaration(location.parent)) {
                            return findImplementationOrAllInitialDeclarations(typeChecker, location.parent);
                        }
                        if (isPossibleCallHierarchyDeclaration(location.parent)) {
                            var ancestor = ts.findAncestor(location.parent, isValidCallHierarchyDeclaration);
                            return ancestor && findImplementationOrAllInitialDeclarations(typeChecker, ancestor);
                        }
                        if (ts.isVariableDeclaration(location.parent) && location.parent.initializer && isConstNamedExpression(location.parent.initializer)) {
                            return location.parent.initializer;
                        }
                        return undefined;
                    }
                    if (ts.isConstructorDeclaration(location)) {
                        if (isValidCallHierarchyDeclaration(location.parent)) {
                            return location.parent;
                        }
                        return undefined;
                    }
                    if (!followingSymbol) {
                        var symbol = typeChecker.getSymbolAtLocation(location);
                        if (symbol) {
                            if (symbol.flags & 2097152 /* Alias */) {
                                symbol = typeChecker.getAliasedSymbol(symbol);
                            }
                            if (symbol.valueDeclaration) {
                                followingSymbol = true;
                                location = symbol.valueDeclaration;
                                continue;
                            }
                        }
                    }
                    return undefined;
                }
            }
            CallHierarchy.resolveCallHierarchyDeclaration = resolveCallHierarchyDeclaration;
            /** Creates a `CallHierarchyItem` for a call hierarchy declaration. */
            function createCallHierarchyItem(program, node) {
                var sourceFile = node.getSourceFile();
                var name = getCallHierarchyItemName(program, node);
                var kind = ts.getNodeKind(node);
                var span = ts.createTextSpanFromBounds(ts.skipTrivia(sourceFile.text, node.getFullStart(), /*stopAfterLineBreak*/ false, /*stopAtComments*/ true), node.getEnd());
                var selectionSpan = ts.createTextSpanFromBounds(name.pos, name.end);
                return { file: sourceFile.fileName, kind: kind, name: name.text, span: span, selectionSpan: selectionSpan };
            }
            CallHierarchy.createCallHierarchyItem = createCallHierarchyItem;
            function isDefined(x) {
                return x !== undefined;
            }
            function convertEntryToCallSite(entry) {
                if (entry.kind === 1 /* Node */) {
                    var node = entry.node;
                    if (ts.isCallOrNewExpressionTarget(node, /*includeElementAccess*/ true, /*skipPastOuterExpressions*/ true)
                        || ts.isTaggedTemplateTag(node, /*includeElementAccess*/ true, /*skipPastOuterExpressions*/ true)
                        || ts.isDecoratorTarget(node, /*includeElementAccess*/ true, /*skipPastOuterExpressions*/ true)
                        || ts.isJsxOpeningLikeElementTagName(node, /*includeElementAccess*/ true, /*skipPastOuterExpressions*/ true)
                        || ts.isRightSideOfPropertyAccess(node)
                        || ts.isArgumentExpressionOfElementAccess(node)) {
                        var sourceFile = node.getSourceFile();
                        var ancestor = ts.findAncestor(node, isValidCallHierarchyDeclaration) || sourceFile;
                        return { declaration: ancestor, range: ts.createTextRangeFromNode(node, sourceFile) };
                    }
                }
            }
            function getCallSiteGroupKey(entry) {
                return "" + ts.getNodeId(entry.declaration);
            }
            function createCallHierarchyIncomingCall(from, fromSpans) {
                return { from: from, fromSpans: fromSpans };
            }
            function convertCallSiteGroupToIncomingCall(program, entries) {
                return createCallHierarchyIncomingCall(createCallHierarchyItem(program, entries[0].declaration), ts.map(entries, function (entry) { return ts.createTextSpanFromRange(entry.range); }));
            }
            /** Gets the call sites that call into the provided call hierarchy declaration. */
            function getIncomingCalls(program, declaration, cancellationToken) {
                // Source files and modules have no incoming calls.
                if (ts.isSourceFile(declaration) || ts.isModuleDeclaration(declaration)) {
                    return [];
                }
                var location = getCallHierarchyDeclarationReferenceNode(declaration);
                var calls = ts.filter(ts.FindAllReferences.findReferenceOrRenameEntries(program, cancellationToken, program.getSourceFiles(), location, /*position*/ 0, { use: 1 /* References */ }, convertEntryToCallSite), isDefined);
                return calls ? ts.group(calls, getCallSiteGroupKey, function (entries) { return convertCallSiteGroupToIncomingCall(program, entries); }) : [];
            }
            CallHierarchy.getIncomingCalls = getIncomingCalls;
            function createCallSiteCollector(program, callSites) {
                function recordCallSite(node) {
                    var target = ts.isTaggedTemplateExpression(node) ? node.tag :
                        ts.isJsxOpeningLikeElement(node) ? node.tagName :
                            ts.isAccessExpression(node) ? node :
                                node.expression;
                    var declaration = resolveCallHierarchyDeclaration(program, target);
                    if (declaration) {
                        var range = ts.createTextRangeFromNode(target, node.getSourceFile());
                        if (ts.isArray(declaration)) {
                            for (var _i = 0, declaration_1 = declaration; _i < declaration_1.length; _i++) {
                                var decl = declaration_1[_i];
                                callSites.push({ declaration: decl, range: range });
                            }
                        }
                        else {
                            callSites.push({ declaration: declaration, range: range });
                        }
                    }
                }
                function collect(node) {
                    if (!node)
                        return;
                    if (node.flags & 8388608 /* Ambient */) {
                        // do not descend into ambient nodes.
                        return;
                    }
                    if (isValidCallHierarchyDeclaration(node)) {
                        // do not descend into other call site declarations, other than class member names
                        if (ts.isClassLike(node)) {
                            for (var _i = 0, _a = node.members; _i < _a.length; _i++) {
                                var member = _a[_i];
                                if (member.name && ts.isComputedPropertyName(member.name)) {
                                    collect(member.name.expression);
                                }
                            }
                        }
                        return;
                    }
                    switch (node.kind) {
                        case 75 /* Identifier */:
                        case 253 /* ImportEqualsDeclaration */:
                        case 254 /* ImportDeclaration */:
                        case 260 /* ExportDeclaration */:
                        case 246 /* InterfaceDeclaration */:
                        case 247 /* TypeAliasDeclaration */:
                            // do not descend into nodes that cannot contain callable nodes
                            return;
                        case 199 /* TypeAssertionExpression */:
                        case 217 /* AsExpression */:
                            // do not descend into the type side of an assertion
                            collect(node.expression);
                            return;
                        case 242 /* VariableDeclaration */:
                        case 156 /* Parameter */:
                            // do not descend into the type of a variable or parameter declaration
                            collect(node.name);
                            collect(node.initializer);
                            return;
                        case 196 /* CallExpression */:
                            // do not descend into the type arguments of a call expression
                            recordCallSite(node);
                            collect(node.expression);
                            ts.forEach(node.arguments, collect);
                            return;
                        case 197 /* NewExpression */:
                            // do not descend into the type arguments of a new expression
                            recordCallSite(node);
                            collect(node.expression);
                            ts.forEach(node.arguments, collect);
                            return;
                        case 198 /* TaggedTemplateExpression */:
                            // do not descend into the type arguments of a tagged template expression
                            recordCallSite(node);
                            collect(node.tag);
                            collect(node.template);
                            return;
                        case 268 /* JsxOpeningElement */:
                        case 267 /* JsxSelfClosingElement */:
                            // do not descend into the type arguments of a JsxOpeningLikeElement
                            recordCallSite(node);
                            collect(node.tagName);
                            collect(node.attributes);
                            return;
                        case 157 /* Decorator */:
                            recordCallSite(node);
                            collect(node.expression);
                            return;
                        case 194 /* PropertyAccessExpression */:
                        case 195 /* ElementAccessExpression */:
                            recordCallSite(node);
                            ts.forEachChild(node, collect);
                            break;
                    }
                    if (ts.isPartOfTypeNode(node)) {
                        // do not descend into types
                        return;
                    }
                    ts.forEachChild(node, collect);
                }
                return collect;
            }
            function collectCallSitesOfSourceFile(node, collect) {
                ts.forEach(node.statements, collect);
            }
            function collectCallSitesOfModuleDeclaration(node, collect) {
                if (!ts.hasModifier(node, 2 /* Ambient */) && node.body && ts.isModuleBlock(node.body)) {
                    ts.forEach(node.body.statements, collect);
                }
            }
            function collectCallSitesOfFunctionLikeDeclaration(typeChecker, node, collect) {
                var implementation = findImplementation(typeChecker, node);
                if (implementation) {
                    ts.forEach(implementation.parameters, collect);
                    collect(implementation.body);
                }
            }
            function collectCallSitesOfClassLikeDeclaration(node, collect) {
                ts.forEach(node.decorators, collect);
                var heritage = ts.getClassExtendsHeritageElement(node);
                if (heritage) {
                    collect(heritage.expression);
                }
                for (var _i = 0, _a = node.members; _i < _a.length; _i++) {
                    var member = _a[_i];
                    ts.forEach(member.decorators, collect);
                    if (ts.isPropertyDeclaration(member)) {
                        collect(member.initializer);
                    }
                    else if (ts.isConstructorDeclaration(member) && member.body) {
                        ts.forEach(member.parameters, collect);
                        collect(member.body);
                    }
                }
            }
            function collectCallSites(program, node) {
                var callSites = [];
                var collect = createCallSiteCollector(program, callSites);
                switch (node.kind) {
                    case 290 /* SourceFile */:
                        collectCallSitesOfSourceFile(node, collect);
                        break;
                    case 249 /* ModuleDeclaration */:
                        collectCallSitesOfModuleDeclaration(node, collect);
                        break;
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        collectCallSitesOfFunctionLikeDeclaration(program.getTypeChecker(), node, collect);
                        break;
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                        collectCallSitesOfClassLikeDeclaration(node, collect);
                        break;
                    default:
                        ts.Debug.assertNever(node);
                }
                return callSites;
            }
            function createCallHierarchyOutgoingCall(to, fromSpans) {
                return { to: to, fromSpans: fromSpans };
            }
            function convertCallSiteGroupToOutgoingCall(program, entries) {
                return createCallHierarchyOutgoingCall(createCallHierarchyItem(program, entries[0].declaration), ts.map(entries, function (entry) { return ts.createTextSpanFromRange(entry.range); }));
            }
            /** Gets the call sites that call out of the provided call hierarchy declaration. */
            function getOutgoingCalls(program, declaration) {
                if (declaration.flags & 8388608 /* Ambient */ || ts.isMethodSignature(declaration)) {
                    return [];
                }
                return ts.group(collectCallSites(program, declaration), getCallSiteGroupKey, function (entries) { return convertCallSiteGroupToOutgoingCall(program, entries); });
            }
            CallHierarchy.getOutgoingCalls = getOutgoingCalls;
        })(CallHierarchy = ts.CallHierarchy || (ts.CallHierarchy = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        function getEditsForFileRename(program, oldFileOrDirPath, newFileOrDirPath, host, formatContext, preferences, sourceMapper) {
            var useCaseSensitiveFileNames = ts.hostUsesCaseSensitiveFileNames(host);
            var getCanonicalFileName = ts.createGetCanonicalFileName(useCaseSensitiveFileNames);
            var oldToNew = getPathUpdater(oldFileOrDirPath, newFileOrDirPath, getCanonicalFileName, sourceMapper);
            var newToOld = getPathUpdater(newFileOrDirPath, oldFileOrDirPath, getCanonicalFileName, sourceMapper);
            return ts.textChanges.ChangeTracker.with({ host: host, formatContext: formatContext, preferences: preferences }, function (changeTracker) {
                updateTsconfigFiles(program, changeTracker, oldToNew, oldFileOrDirPath, newFileOrDirPath, host.getCurrentDirectory(), useCaseSensitiveFileNames);
                updateImports(program, changeTracker, oldToNew, newToOld, host, getCanonicalFileName);
            });
        }
        ts.getEditsForFileRename = getEditsForFileRename;
        // exported for tests
        function getPathUpdater(oldFileOrDirPath, newFileOrDirPath, getCanonicalFileName, sourceMapper) {
            var canonicalOldPath = getCanonicalFileName(oldFileOrDirPath);
            return function (path) {
                var originalPath = sourceMapper && sourceMapper.tryGetSourcePosition({ fileName: path, pos: 0 });
                var updatedPath = getUpdatedPath(originalPath ? originalPath.fileName : path);
                return originalPath
                    ? updatedPath === undefined ? undefined : makeCorrespondingRelativeChange(originalPath.fileName, updatedPath, path, getCanonicalFileName)
                    : updatedPath;
            };
            function getUpdatedPath(pathToUpdate) {
                if (getCanonicalFileName(pathToUpdate) === canonicalOldPath)
                    return newFileOrDirPath;
                var suffix = ts.tryRemoveDirectoryPrefix(pathToUpdate, canonicalOldPath, getCanonicalFileName);
                return suffix === undefined ? undefined : newFileOrDirPath + "/" + suffix;
            }
        }
        ts.getPathUpdater = getPathUpdater;
        // Relative path from a0 to b0 should be same as relative path from a1 to b1. Returns b1.
        function makeCorrespondingRelativeChange(a0, b0, a1, getCanonicalFileName) {
            var rel = ts.getRelativePathFromFile(a0, b0, getCanonicalFileName);
            return combinePathsSafe(ts.getDirectoryPath(a1), rel);
        }
        function updateTsconfigFiles(program, changeTracker, oldToNew, oldFileOrDirPath, newFileOrDirPath, currentDirectory, useCaseSensitiveFileNames) {
            var configFile = program.getCompilerOptions().configFile;
            if (!configFile)
                return;
            var configDir = ts.getDirectoryPath(configFile.fileName);
            var jsonObjectLiteral = ts.getTsConfigObjectLiteralExpression(configFile);
            if (!jsonObjectLiteral)
                return;
            forEachProperty(jsonObjectLiteral, function (property, propertyName) {
                switch (propertyName) {
                    case "files":
                    case "include":
                    case "exclude": {
                        var foundExactMatch = updatePaths(property);
                        if (!foundExactMatch && propertyName === "include" && ts.isArrayLiteralExpression(property.initializer)) {
                            var includes = ts.mapDefined(property.initializer.elements, function (e) { return ts.isStringLiteral(e) ? e.text : undefined; });
                            var matchers = ts.getFileMatcherPatterns(configDir, /*excludes*/ [], includes, useCaseSensitiveFileNames, currentDirectory);
                            // If there isn't some include for this, add a new one.
                            if (ts.getRegexFromPattern(ts.Debug.assertDefined(matchers.includeFilePattern), useCaseSensitiveFileNames).test(oldFileOrDirPath) &&
                                !ts.getRegexFromPattern(ts.Debug.assertDefined(matchers.includeFilePattern), useCaseSensitiveFileNames).test(newFileOrDirPath)) {
                                changeTracker.insertNodeAfter(configFile, ts.last(property.initializer.elements), ts.createStringLiteral(relativePath(newFileOrDirPath)));
                            }
                        }
                        break;
                    }
                    case "compilerOptions":
                        forEachProperty(property.initializer, function (property, propertyName) {
                            var option = ts.getOptionFromName(propertyName);
                            if (option && (option.isFilePath || option.type === "list" && option.element.isFilePath)) {
                                updatePaths(property);
                            }
                            else if (propertyName === "paths") {
                                forEachProperty(property.initializer, function (pathsProperty) {
                                    if (!ts.isArrayLiteralExpression(pathsProperty.initializer))
                                        return;
                                    for (var _i = 0, _a = pathsProperty.initializer.elements; _i < _a.length; _i++) {
                                        var e = _a[_i];
                                        tryUpdateString(e);
                                    }
                                });
                            }
                        });
                        break;
                }
            });
            function updatePaths(property) {
                // Type annotation needed due to #7294
                var elements = ts.isArrayLiteralExpression(property.initializer) ? property.initializer.elements : [property.initializer];
                var foundExactMatch = false;
                for (var _i = 0, elements_1 = elements; _i < elements_1.length; _i++) {
                    var element = elements_1[_i];
                    foundExactMatch = tryUpdateString(element) || foundExactMatch;
                }
                return foundExactMatch;
            }
            function tryUpdateString(element) {
                if (!ts.isStringLiteral(element))
                    return false;
                var elementFileName = combinePathsSafe(configDir, element.text);
                var updated = oldToNew(elementFileName);
                if (updated !== undefined) {
                    changeTracker.replaceRangeWithText(configFile, createStringRange(element, configFile), relativePath(updated));
                    return true;
                }
                return false;
            }
            function relativePath(path) {
                return ts.getRelativePathFromDirectory(configDir, path, /*ignoreCase*/ !useCaseSensitiveFileNames);
            }
        }
        function updateImports(program, changeTracker, oldToNew, newToOld, host, getCanonicalFileName) {
            var allFiles = program.getSourceFiles();
            var _loop_4 = function (sourceFile) {
                var newFromOld = oldToNew(sourceFile.path);
                var newImportFromPath = newFromOld !== undefined ? newFromOld : sourceFile.path;
                var newImportFromDirectory = ts.getDirectoryPath(newImportFromPath);
                var oldFromNew = newToOld(sourceFile.fileName);
                var oldImportFromPath = oldFromNew || sourceFile.fileName;
                var oldImportFromDirectory = ts.getDirectoryPath(oldImportFromPath);
                var importingSourceFileMoved = newFromOld !== undefined || oldFromNew !== undefined;
                updateImportsWorker(sourceFile, changeTracker, function (referenceText) {
                    if (!ts.pathIsRelative(referenceText))
                        return undefined;
                    var oldAbsolute = combinePathsSafe(oldImportFromDirectory, referenceText);
                    var newAbsolute = oldToNew(oldAbsolute);
                    return newAbsolute === undefined ? undefined : ts.ensurePathIsNonModuleName(ts.getRelativePathFromDirectory(newImportFromDirectory, newAbsolute, getCanonicalFileName));
                }, function (importLiteral) {
                    var importedModuleSymbol = program.getTypeChecker().getSymbolAtLocation(importLiteral);
                    // No need to update if it's an ambient module^M
                    if (importedModuleSymbol && importedModuleSymbol.declarations.some(function (d) { return ts.isAmbientModule(d); }))
                        return undefined;
                    var toImport = oldFromNew !== undefined
                        // If we're at the new location (file was already renamed), need to redo module resolution starting from the old location.
                        // TODO:GH#18217
                        ? getSourceFileToImportFromResolved(ts.resolveModuleName(importLiteral.text, oldImportFromPath, program.getCompilerOptions(), host), oldToNew, allFiles)
                        : getSourceFileToImport(importedModuleSymbol, importLiteral, sourceFile, program, host, oldToNew);
                    // Need an update if the imported file moved, or the importing file moved and was using a relative path.
                    return toImport !== undefined && (toImport.updated || (importingSourceFileMoved && ts.pathIsRelative(importLiteral.text)))
                        ? ts.moduleSpecifiers.updateModuleSpecifier(program.getCompilerOptions(), newImportFromPath, toImport.newFileName, host, allFiles, program.redirectTargetsMap, importLiteral.text)
                        : undefined;
                });
            };
            for (var _i = 0, allFiles_1 = allFiles; _i < allFiles_1.length; _i++) {
                var sourceFile = allFiles_1[_i];
                _loop_4(sourceFile);
            }
        }
        function combineNormal(pathA, pathB) {
            return ts.normalizePath(ts.combinePaths(pathA, pathB));
        }
        function combinePathsSafe(pathA, pathB) {
            return ts.ensurePathIsNonModuleName(combineNormal(pathA, pathB));
        }
        function getSourceFileToImport(importedModuleSymbol, importLiteral, importingSourceFile, program, host, oldToNew) {
            if (importedModuleSymbol) {
                // `find` should succeed because we checked for ambient modules before calling this function.
                var oldFileName = ts.find(importedModuleSymbol.declarations, ts.isSourceFile).fileName;
                var newFileName = oldToNew(oldFileName);
                return newFileName === undefined ? { newFileName: oldFileName, updated: false } : { newFileName: newFileName, updated: true };
            }
            else {
                var resolved = host.resolveModuleNames
                    ? host.getResolvedModuleWithFailedLookupLocationsFromCache && host.getResolvedModuleWithFailedLookupLocationsFromCache(importLiteral.text, importingSourceFile.fileName)
                    : program.getResolvedModuleWithFailedLookupLocationsFromCache(importLiteral.text, importingSourceFile.fileName);
                return getSourceFileToImportFromResolved(resolved, oldToNew, program.getSourceFiles());
            }
        }
        function getSourceFileToImportFromResolved(resolved, oldToNew, sourceFiles) {
            // Search through all locations looking for a moved file, and only then test already existing files.
            // This is because if `a.ts` is compiled to `a.js` and `a.ts` is moved, we don't want to resolve anything to `a.js`, but to `a.ts`'s new location.
            if (!resolved)
                return undefined;
            // First try resolved module
            if (resolved.resolvedModule) {
                var result_2 = tryChange(resolved.resolvedModule.resolvedFileName);
                if (result_2)
                    return result_2;
            }
            // Then failed lookups that are in the list of sources
            var result = ts.forEach(resolved.failedLookupLocations, tryChangeWithIgnoringPackageJsonExisting)
                // Then failed lookups except package.json since we dont want to touch them (only included ts/js files)
                || ts.forEach(resolved.failedLookupLocations, tryChangeWithIgnoringPackageJson);
            if (result)
                return result;
            // If nothing changed, then result is resolved module file thats not updated
            return resolved.resolvedModule && { newFileName: resolved.resolvedModule.resolvedFileName, updated: false };
            function tryChangeWithIgnoringPackageJsonExisting(oldFileName) {
                var newFileName = oldToNew(oldFileName);
                return newFileName && ts.find(sourceFiles, function (src) { return src.fileName === newFileName; })
                    ? tryChangeWithIgnoringPackageJson(oldFileName) : undefined;
            }
            function tryChangeWithIgnoringPackageJson(oldFileName) {
                return !ts.endsWith(oldFileName, "/package.json") ? tryChange(oldFileName) : undefined;
            }
            function tryChange(oldFileName) {
                var newFileName = oldToNew(oldFileName);
                return newFileName && { newFileName: newFileName, updated: true };
            }
        }
        function updateImportsWorker(sourceFile, changeTracker, updateRef, updateImport) {
            for (var _i = 0, _a = sourceFile.referencedFiles || ts.emptyArray; _i < _a.length; _i++) { // TODO: GH#26162
                var ref = _a[_i];
                var updated = updateRef(ref.fileName);
                if (updated !== undefined && updated !== sourceFile.text.slice(ref.pos, ref.end))
                    changeTracker.replaceRangeWithText(sourceFile, ref, updated);
            }
            for (var _b = 0, _c = sourceFile.imports; _b < _c.length; _b++) {
                var importStringLiteral = _c[_b];
                var updated = updateImport(importStringLiteral);
                if (updated !== undefined && updated !== importStringLiteral.text)
                    changeTracker.replaceRangeWithText(sourceFile, createStringRange(importStringLiteral, sourceFile), updated);
            }
        }
        function createStringRange(node, sourceFile) {
            return ts.createRange(node.getStart(sourceFile) + 1, node.end - 1);
        }
        function forEachProperty(objectLiteral, cb) {
            if (!ts.isObjectLiteralExpression(objectLiteral))
                return;
            for (var _i = 0, _a = objectLiteral.properties; _i < _a.length; _i++) {
                var property = _a[_i];
                if (ts.isPropertyAssignment(property) && ts.isStringLiteral(property.name)) {
                    cb(property, property.name.text);
                }
            }
        }
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var GoToDefinition;
        (function (GoToDefinition) {
            function getDefinitionAtPosition(program, sourceFile, position) {
                var reference = getReferenceAtPosition(sourceFile, position, program);
                if (reference) {
                    return [getDefinitionInfoForFileReference(reference.fileName, reference.file.fileName)];
                }
                var node = ts.getTouchingPropertyName(sourceFile, position);
                if (node === sourceFile) {
                    return undefined;
                }
                var parent = node.parent;
                var typeChecker = program.getTypeChecker();
                // Labels
                if (ts.isJumpStatementTarget(node)) {
                    var label = ts.getTargetLabel(node.parent, node.text);
                    return label ? [createDefinitionInfoFromName(typeChecker, label, "label" /* label */, node.text, /*containerName*/ undefined)] : undefined; // TODO: GH#18217
                }
                var symbol = getSymbol(node, typeChecker);
                // Could not find a symbol e.g. node is string or number keyword,
                // or the symbol was an internal symbol and does not have a declaration e.g. undefined symbol
                if (!symbol) {
                    return getDefinitionInfoForIndexSignatures(node, typeChecker);
                }
                var calledDeclaration = tryGetSignatureDeclaration(typeChecker, node);
                // Don't go to the component constructor definition for a JSX element, just go to the component definition.
                if (calledDeclaration && !(ts.isJsxOpeningLikeElement(node.parent) && isConstructorLike(calledDeclaration))) {
                    var sigInfo = createDefinitionFromSignatureDeclaration(typeChecker, calledDeclaration);
                    // For a function, if this is the original function definition, return just sigInfo.
                    // If this is the original constructor definition, parent is the class.
                    if (typeChecker.getRootSymbols(symbol).some(function (s) { return symbolMatchesSignature(s, calledDeclaration); }) ||
                        // TODO: GH#25533 Following check shouldn't be necessary if 'require' is an alias
                        symbol.declarations && symbol.declarations.some(function (d) { return ts.isVariableDeclaration(d) && !!d.initializer && ts.isRequireCall(d.initializer, /*checkArgumentIsStringLiteralLike*/ false); })) {
                        return [sigInfo];
                    }
                    else {
                        var defs = getDefinitionFromSymbol(typeChecker, symbol, node, calledDeclaration) || ts.emptyArray;
                        // For a 'super()' call, put the signature first, else put the variable first.
                        return node.kind === 102 /* SuperKeyword */ ? __spreadArrays([sigInfo], defs) : __spreadArrays(defs, [sigInfo]);
                    }
                }
                // Because name in short-hand property assignment has two different meanings: property name and property value,
                // using go-to-definition at such position should go to the variable declaration of the property value rather than
                // go to the declaration of the property name (in this case stay at the same position). However, if go-to-definition
                // is performed at the location of property access, we would like to go to definition of the property in the short-hand
                // assignment. This case and others are handled by the following code.
                if (node.parent.kind === 282 /* ShorthandPropertyAssignment */) {
                    var shorthandSymbol_1 = typeChecker.getShorthandAssignmentValueSymbol(symbol.valueDeclaration);
                    return shorthandSymbol_1 ? shorthandSymbol_1.declarations.map(function (decl) { return createDefinitionInfo(decl, typeChecker, shorthandSymbol_1, node); }) : [];
                }
                // If the node is the name of a BindingElement within an ObjectBindingPattern instead of just returning the
                // declaration the symbol (which is itself), we should try to get to the original type of the ObjectBindingPattern
                // and return the property declaration for the referenced property.
                // For example:
                //      import('./foo').then(({ b/*goto*/ar }) => undefined); => should get use to the declaration in file "./foo"
                //
                //      function bar<T>(onfulfilled: (value: T) => void) { //....}
                //      interface Test {
                //          pr/*destination*/op1: number
                //      }
                //      bar<Test>(({pr/*goto*/op1})=>{});
                if (ts.isPropertyName(node) && ts.isBindingElement(parent) && ts.isObjectBindingPattern(parent.parent) &&
                    (node === (parent.propertyName || parent.name))) {
                    var name_3 = ts.getNameFromPropertyName(node);
                    var type = typeChecker.getTypeAtLocation(parent.parent);
                    return name_3 === undefined ? ts.emptyArray : ts.flatMap(type.isUnion() ? type.types : [type], function (t) {
                        var prop = t.getProperty(name_3);
                        return prop && getDefinitionFromSymbol(typeChecker, prop, node);
                    });
                }
                // If the current location we want to find its definition is in an object literal, try to get the contextual type for the
                // object literal, lookup the property symbol in the contextual type, and use this for goto-definition.
                // For example
                //      interface Props{
                //          /*first*/prop1: number
                //          prop2: boolean
                //      }
                //      function Foo(arg: Props) {}
                //      Foo( { pr/*1*/op1: 10, prop2: true })
                var element = ts.getContainingObjectLiteralElement(node);
                if (element) {
                    var contextualType = element && typeChecker.getContextualType(element.parent);
                    if (contextualType) {
                        return ts.flatMap(ts.getPropertySymbolsFromContextualType(element, typeChecker, contextualType, /*unionSymbolOk*/ false), function (propertySymbol) {
                            return getDefinitionFromSymbol(typeChecker, propertySymbol, node);
                        });
                    }
                }
                return getDefinitionFromSymbol(typeChecker, symbol, node);
            }
            GoToDefinition.getDefinitionAtPosition = getDefinitionAtPosition;
            /**
             * True if we should not add definitions for both the signature symbol and the definition symbol.
             * True for `const |f = |() => 0`, false for `function |f() {} const |g = f;`.
             */
            function symbolMatchesSignature(s, calledDeclaration) {
                return s === calledDeclaration.symbol || s === calledDeclaration.symbol.parent ||
                    !ts.isCallLikeExpression(calledDeclaration.parent) && s === calledDeclaration.parent.symbol;
            }
            function getReferenceAtPosition(sourceFile, position, program) {
                var referencePath = findReferenceInPosition(sourceFile.referencedFiles, position);
                if (referencePath) {
                    var file = program.getSourceFileFromReference(sourceFile, referencePath);
                    return file && { fileName: referencePath.fileName, file: file };
                }
                var typeReferenceDirective = findReferenceInPosition(sourceFile.typeReferenceDirectives, position);
                if (typeReferenceDirective) {
                    var reference = program.getResolvedTypeReferenceDirectives().get(typeReferenceDirective.fileName);
                    var file = reference && program.getSourceFile(reference.resolvedFileName); // TODO:GH#18217
                    return file && { fileName: typeReferenceDirective.fileName, file: file };
                }
                var libReferenceDirective = findReferenceInPosition(sourceFile.libReferenceDirectives, position);
                if (libReferenceDirective) {
                    var file = program.getLibFileFromReference(libReferenceDirective);
                    return file && { fileName: libReferenceDirective.fileName, file: file };
                }
                return undefined;
            }
            GoToDefinition.getReferenceAtPosition = getReferenceAtPosition;
            /// Goto type
            function getTypeDefinitionAtPosition(typeChecker, sourceFile, position) {
                var node = ts.getTouchingPropertyName(sourceFile, position);
                if (node === sourceFile) {
                    return undefined;
                }
                var symbol = typeChecker.getSymbolAtLocation(node);
                if (!symbol)
                    return undefined;
                var typeAtLocation = typeChecker.getTypeOfSymbolAtLocation(symbol, node);
                var returnType = tryGetReturnTypeOfFunction(symbol, typeAtLocation, typeChecker);
                var fromReturnType = returnType && definitionFromType(returnType, typeChecker, node);
                // If a function returns 'void' or some other type with no definition, just return the function definition.
                return fromReturnType && fromReturnType.length !== 0 ? fromReturnType : definitionFromType(typeAtLocation, typeChecker, node);
            }
            GoToDefinition.getTypeDefinitionAtPosition = getTypeDefinitionAtPosition;
            function definitionFromType(type, checker, node) {
                return ts.flatMap(type.isUnion() && !(type.flags & 32 /* Enum */) ? type.types : [type], function (t) {
                    return t.symbol && getDefinitionFromSymbol(checker, t.symbol, node);
                });
            }
            function tryGetReturnTypeOfFunction(symbol, type, checker) {
                // If the type is just a function's inferred type,
                // go-to-type should go to the return type instead, since go-to-definition takes you to the function anyway.
                if (type.symbol === symbol ||
                    // At `const f = () => {}`, the symbol is `f` and the type symbol is at `() => {}`
                    symbol.valueDeclaration && type.symbol && ts.isVariableDeclaration(symbol.valueDeclaration) && symbol.valueDeclaration.initializer === type.symbol.valueDeclaration) {
                    var sigs = type.getCallSignatures();
                    if (sigs.length === 1)
                        return checker.getReturnTypeOfSignature(ts.first(sigs));
                }
                return undefined;
            }
            function getDefinitionAndBoundSpan(program, sourceFile, position) {
                var definitions = getDefinitionAtPosition(program, sourceFile, position);
                if (!definitions || definitions.length === 0) {
                    return undefined;
                }
                // Check if position is on triple slash reference.
                var comment = findReferenceInPosition(sourceFile.referencedFiles, position) ||
                    findReferenceInPosition(sourceFile.typeReferenceDirectives, position) ||
                    findReferenceInPosition(sourceFile.libReferenceDirectives, position);
                if (comment) {
                    return { definitions: definitions, textSpan: ts.createTextSpanFromRange(comment) };
                }
                var node = ts.getTouchingPropertyName(sourceFile, position);
                var textSpan = ts.createTextSpan(node.getStart(), node.getWidth());
                return { definitions: definitions, textSpan: textSpan };
            }
            GoToDefinition.getDefinitionAndBoundSpan = getDefinitionAndBoundSpan;
            // At 'x.foo', see if the type of 'x' has an index signature, and if so find its declarations.
            function getDefinitionInfoForIndexSignatures(node, checker) {
                if (!ts.isPropertyAccessExpression(node.parent) || node.parent.name !== node)
                    return;
                var type = checker.getTypeAtLocation(node.parent.expression);
                return ts.mapDefined(type.isUnionOrIntersection() ? type.types : [type], function (nonUnionType) {
                    var info = checker.getIndexInfoOfType(nonUnionType, 0 /* String */);
                    return info && info.declaration && createDefinitionFromSignatureDeclaration(checker, info.declaration);
                });
            }
            function getSymbol(node, checker) {
                var symbol = checker.getSymbolAtLocation(node);
                // If this is an alias, and the request came at the declaration location
                // get the aliased symbol instead. This allows for goto def on an import e.g.
                //   import {A, B} from "mod";
                // to jump to the implementation directly.
                if (symbol && symbol.flags & 2097152 /* Alias */ && shouldSkipAlias(node, symbol.declarations[0])) {
                    var aliased = checker.getAliasedSymbol(symbol);
                    if (aliased.declarations) {
                        return aliased;
                    }
                }
                if (symbol && ts.isInJSFile(node)) {
                    var requireCall = ts.forEach(symbol.declarations, function (d) { return ts.isVariableDeclaration(d) && !!d.initializer && ts.isRequireCall(d.initializer, /*checkArgumentIsStringLiteralLike*/ true) ? d.initializer : undefined; });
                    if (requireCall) {
                        var moduleSymbol = checker.getSymbolAtLocation(requireCall.arguments[0]);
                        if (moduleSymbol) {
                            return checker.resolveExternalModuleSymbol(moduleSymbol);
                        }
                    }
                }
                return symbol;
            }
            // Go to the original declaration for cases:
            //
            //   (1) when the aliased symbol was declared in the location(parent).
            //   (2) when the aliased symbol is originating from an import.
            //
            function shouldSkipAlias(node, declaration) {
                if (node.kind !== 75 /* Identifier */) {
                    return false;
                }
                if (node.parent === declaration) {
                    return true;
                }
                switch (declaration.kind) {
                    case 255 /* ImportClause */:
                    case 253 /* ImportEqualsDeclaration */:
                        return true;
                    case 258 /* ImportSpecifier */:
                        return declaration.parent.kind === 257 /* NamedImports */;
                    default:
                        return false;
                }
            }
            function getDefinitionFromSymbol(typeChecker, symbol, node, declarationNode) {
                // There are cases when you extend a function by adding properties to it afterwards,
                // we want to strip those extra properties.
                // For deduping purposes, we also want to exclude any declarationNodes if provided.
                var filteredDeclarations = ts.filter(symbol.declarations, function (d) { return d !== declarationNode && (!ts.isAssignmentDeclaration(d) || d === symbol.valueDeclaration); }) || undefined;
                return getConstructSignatureDefinition() || getCallSignatureDefinition() || ts.map(filteredDeclarations, function (declaration) { return createDefinitionInfo(declaration, typeChecker, symbol, node); });
                function getConstructSignatureDefinition() {
                    // Applicable only if we are in a new expression, or we are on a constructor declaration
                    // and in either case the symbol has a construct signature definition, i.e. class
                    if (symbol.flags & 32 /* Class */ && !(symbol.flags & 16 /* Function */) && (ts.isNewExpressionTarget(node) || node.kind === 129 /* ConstructorKeyword */)) {
                        var cls = ts.find(filteredDeclarations, ts.isClassLike) || ts.Debug.fail("Expected declaration to have at least one class-like declaration");
                        return getSignatureDefinition(cls.members, /*selectConstructors*/ true);
                    }
                }
                function getCallSignatureDefinition() {
                    return ts.isCallOrNewExpressionTarget(node) || ts.isNameOfFunctionDeclaration(node)
                        ? getSignatureDefinition(filteredDeclarations, /*selectConstructors*/ false)
                        : undefined;
                }
                function getSignatureDefinition(signatureDeclarations, selectConstructors) {
                    if (!signatureDeclarations) {
                        return undefined;
                    }
                    var declarations = signatureDeclarations.filter(selectConstructors ? ts.isConstructorDeclaration : ts.isFunctionLike);
                    var declarationsWithBody = declarations.filter(function (d) { return !!d.body; });
                    // declarations defined on the global scope can be defined on multiple files. Get all of them.
                    return declarations.length
                        ? declarationsWithBody.length !== 0
                            ? declarationsWithBody.map(function (x) { return createDefinitionInfo(x, typeChecker, symbol, node); })
                            : [createDefinitionInfo(ts.last(declarations), typeChecker, symbol, node)]
                        : undefined;
                }
            }
            /** Creates a DefinitionInfo from a Declaration, using the declaration's name if possible. */
            function createDefinitionInfo(declaration, checker, symbol, node) {
                var symbolName = checker.symbolToString(symbol); // Do not get scoped name, just the name of the symbol
                var symbolKind = ts.SymbolDisplay.getSymbolKind(checker, symbol, node);
                var containerName = symbol.parent ? checker.symbolToString(symbol.parent, node) : "";
                return createDefinitionInfoFromName(checker, declaration, symbolKind, symbolName, containerName);
            }
            /** Creates a DefinitionInfo directly from the name of a declaration. */
            function createDefinitionInfoFromName(checker, declaration, symbolKind, symbolName, containerName) {
                var name = ts.getNameOfDeclaration(declaration) || declaration;
                var sourceFile = name.getSourceFile();
                var textSpan = ts.createTextSpanFromNode(name, sourceFile);
                return __assign(__assign({ fileName: sourceFile.fileName, textSpan: textSpan, kind: symbolKind, name: symbolName, containerKind: undefined,
                    containerName: containerName }, ts.FindAllReferences.toContextSpan(textSpan, sourceFile, ts.FindAllReferences.getContextNode(declaration))), { isLocal: !checker.isDeclarationVisible(declaration) });
            }
            function createDefinitionFromSignatureDeclaration(typeChecker, decl) {
                return createDefinitionInfo(decl, typeChecker, decl.symbol, decl);
            }
            function findReferenceInPosition(refs, pos) {
                return ts.find(refs, function (ref) { return ts.textRangeContainsPositionInclusive(ref, pos); });
            }
            GoToDefinition.findReferenceInPosition = findReferenceInPosition;
            function getDefinitionInfoForFileReference(name, targetFileName) {
                return {
                    fileName: targetFileName,
                    textSpan: ts.createTextSpanFromBounds(0, 0),
                    kind: "script" /* scriptElement */,
                    name: name,
                    containerName: undefined,
                    containerKind: undefined,
                };
            }
            /** Returns a CallLikeExpression where `node` is the target being invoked. */
            function getAncestorCallLikeExpression(node) {
                var target = climbPastManyPropertyAccesses(node);
                var callLike = target.parent;
                return callLike && ts.isCallLikeExpression(callLike) && ts.getInvokedExpression(callLike) === target ? callLike : undefined;
            }
            function climbPastManyPropertyAccesses(node) {
                return ts.isRightSideOfPropertyAccess(node) ? climbPastManyPropertyAccesses(node.parent) : node;
            }
            function tryGetSignatureDeclaration(typeChecker, node) {
                var callLike = getAncestorCallLikeExpression(node);
                var signature = callLike && typeChecker.getResolvedSignature(callLike);
                // Don't go to a function type, go to the value having that type.
                return ts.tryCast(signature && signature.declaration, function (d) { return ts.isFunctionLike(d) && !ts.isFunctionTypeNode(d); });
            }
            function isConstructorLike(node) {
                switch (node.kind) {
                    case 162 /* Constructor */:
                    case 171 /* ConstructorType */:
                    case 166 /* ConstructSignature */:
                        return true;
                    default:
                        return false;
                }
            }
        })(GoToDefinition = ts.GoToDefinition || (ts.GoToDefinition = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var JsDoc;
        (function (JsDoc) {
            var jsDocTagNames = [
                "abstract",
                "access",
                "alias",
                "argument",
                "async",
                "augments",
                "author",
                "borrows",
                "callback",
                "class",
                "classdesc",
                "constant",
                "constructor",
                "constructs",
                "copyright",
                "default",
                "deprecated",
                "description",
                "emits",
                "enum",
                "event",
                "example",
                "exports",
                "extends",
                "external",
                "field",
                "file",
                "fileoverview",
                "fires",
                "function",
                "generator",
                "global",
                "hideconstructor",
                "host",
                "ignore",
                "implements",
                "inheritdoc",
                "inner",
                "instance",
                "interface",
                "kind",
                "lends",
                "license",
                "listens",
                "member",
                "memberof",
                "method",
                "mixes",
                "module",
                "name",
                "namespace",
                "override",
                "package",
                "param",
                "private",
                "property",
                "protected",
                "public",
                "readonly",
                "requires",
                "returns",
                "see",
                "since",
                "static",
                "summary",
                "template",
                "this",
                "throws",
                "todo",
                "tutorial",
                "type",
                "typedef",
                "var",
                "variation",
                "version",
                "virtual",
                "yields"
            ];
            var jsDocTagNameCompletionEntries;
            var jsDocTagCompletionEntries;
            function getJsDocCommentsFromDeclarations(declarations) {
                // Only collect doc comments from duplicate declarations once:
                // In case of a union property there might be same declaration multiple times
                // which only varies in type parameter
                // Eg. const a: Array<string> | Array<number>; a.length
                // The property length will have two declarations of property length coming
                // from Array<T> - Array<string> and Array<number>
                var documentationComment = [];
                ts.forEachUnique(declarations, function (declaration) {
                    for (var _i = 0, _a = getCommentHavingNodes(declaration); _i < _a.length; _i++) {
                        var comment = _a[_i].comment;
                        if (comment === undefined)
                            continue;
                        if (documentationComment.length) {
                            documentationComment.push(ts.lineBreakPart());
                        }
                        documentationComment.push(ts.textPart(comment));
                    }
                });
                return documentationComment;
            }
            JsDoc.getJsDocCommentsFromDeclarations = getJsDocCommentsFromDeclarations;
            function getCommentHavingNodes(declaration) {
                switch (declaration.kind) {
                    case 316 /* JSDocParameterTag */:
                    case 322 /* JSDocPropertyTag */:
                        return [declaration];
                    case 314 /* JSDocCallbackTag */:
                    case 321 /* JSDocTypedefTag */:
                        return [declaration, declaration.parent];
                    default:
                        return ts.getJSDocCommentsAndTags(declaration);
                }
            }
            function getJsDocTagsFromDeclarations(declarations) {
                // Only collect doc comments from duplicate declarations once.
                var tags = [];
                ts.forEachUnique(declarations, function (declaration) {
                    for (var _i = 0, _a = ts.getJSDocTags(declaration); _i < _a.length; _i++) {
                        var tag = _a[_i];
                        tags.push({ name: tag.tagName.text, text: getCommentText(tag) });
                    }
                });
                return tags;
            }
            JsDoc.getJsDocTagsFromDeclarations = getJsDocTagsFromDeclarations;
            function getCommentText(tag) {
                var comment = tag.comment;
                switch (tag.kind) {
                    case 307 /* JSDocAugmentsTag */:
                        return withNode(tag.class);
                    case 320 /* JSDocTemplateTag */:
                        return withList(tag.typeParameters);
                    case 319 /* JSDocTypeTag */:
                        return withNode(tag.typeExpression);
                    case 321 /* JSDocTypedefTag */:
                    case 314 /* JSDocCallbackTag */:
                    case 322 /* JSDocPropertyTag */:
                    case 316 /* JSDocParameterTag */:
                        var name = tag.name;
                        return name ? withNode(name) : comment;
                    default:
                        return comment;
                }
                function withNode(node) {
                    return addComment(node.getText());
                }
                function withList(list) {
                    return addComment(list.map(function (x) { return x.getText(); }).join(", "));
                }
                function addComment(s) {
                    return comment === undefined ? s : s + " " + comment;
                }
            }
            function getJSDocTagNameCompletions() {
                return jsDocTagNameCompletionEntries || (jsDocTagNameCompletionEntries = ts.map(jsDocTagNames, function (tagName) {
                    return {
                        name: tagName,
                        kind: "keyword" /* keyword */,
                        kindModifiers: "",
                        sortText: "0",
                    };
                }));
            }
            JsDoc.getJSDocTagNameCompletions = getJSDocTagNameCompletions;
            JsDoc.getJSDocTagNameCompletionDetails = getJSDocTagCompletionDetails;
            function getJSDocTagCompletions() {
                return jsDocTagCompletionEntries || (jsDocTagCompletionEntries = ts.map(jsDocTagNames, function (tagName) {
                    return {
                        name: "@" + tagName,
                        kind: "keyword" /* keyword */,
                        kindModifiers: "",
                        sortText: "0"
                    };
                }));
            }
            JsDoc.getJSDocTagCompletions = getJSDocTagCompletions;
            function getJSDocTagCompletionDetails(name) {
                return {
                    name: name,
                    kind: "" /* unknown */,
                    kindModifiers: "",
                    displayParts: [ts.textPart(name)],
                    documentation: ts.emptyArray,
                    tags: undefined,
                    codeActions: undefined,
                };
            }
            JsDoc.getJSDocTagCompletionDetails = getJSDocTagCompletionDetails;
            function getJSDocParameterNameCompletions(tag) {
                if (!ts.isIdentifier(tag.name)) {
                    return ts.emptyArray;
                }
                var nameThusFar = tag.name.text;
                var jsdoc = tag.parent;
                var fn = jsdoc.parent;
                if (!ts.isFunctionLike(fn))
                    return [];
                return ts.mapDefined(fn.parameters, function (param) {
                    if (!ts.isIdentifier(param.name))
                        return undefined;
                    var name = param.name.text;
                    if (jsdoc.tags.some(function (t) { return t !== tag && ts.isJSDocParameterTag(t) && ts.isIdentifier(t.name) && t.name.escapedText === name; }) // TODO: GH#18217
                        || nameThusFar !== undefined && !ts.startsWith(name, nameThusFar)) {
                        return undefined;
                    }
                    return { name: name, kind: "parameter" /* parameterElement */, kindModifiers: "", sortText: "0" };
                });
            }
            JsDoc.getJSDocParameterNameCompletions = getJSDocParameterNameCompletions;
            function getJSDocParameterNameCompletionDetails(name) {
                return {
                    name: name,
                    kind: "parameter" /* parameterElement */,
                    kindModifiers: "",
                    displayParts: [ts.textPart(name)],
                    documentation: ts.emptyArray,
                    tags: undefined,
                    codeActions: undefined,
                };
            }
            JsDoc.getJSDocParameterNameCompletionDetails = getJSDocParameterNameCompletionDetails;
            /**
             * Checks if position points to a valid position to add JSDoc comments, and if so,
             * returns the appropriate template. Otherwise returns an empty string.
             * Valid positions are
             *      - outside of comments, statements, and expressions, and
             *      - preceding a:
             *          - function/constructor/method declaration
             *          - class declarations
             *          - variable statements
             *          - namespace declarations
             *          - interface declarations
             *          - method signatures
             *          - type alias declarations
             *
             * Hosts should ideally check that:
             * - The line is all whitespace up to 'position' before performing the insertion.
             * - If the keystroke sequence "/\*\*" induced the call, we also check that the next
             * non-whitespace character is '*', which (approximately) indicates whether we added
             * the second '*' to complete an existing (JSDoc) comment.
             * @param fileName The file in which to perform the check.
             * @param position The (character-indexed) position in the file where the check should
             * be performed.
             */
            function getDocCommentTemplateAtPosition(newLine, sourceFile, position) {
                var tokenAtPos = ts.getTokenAtPosition(sourceFile, position);
                var existingDocComment = ts.findAncestor(tokenAtPos, ts.isJSDoc);
                if (existingDocComment && (existingDocComment.comment !== undefined || ts.length(existingDocComment.tags))) {
                    // Non-empty comment already exists.
                    return undefined;
                }
                var tokenStart = tokenAtPos.getStart(sourceFile);
                // Don't provide a doc comment template based on a *previous* node. (But an existing empty jsdoc comment will likely start before `position`.)
                if (!existingDocComment && tokenStart < position) {
                    return undefined;
                }
                var commentOwnerInfo = getCommentOwnerInfo(tokenAtPos);
                if (!commentOwnerInfo) {
                    return undefined;
                }
                var commentOwner = commentOwnerInfo.commentOwner, parameters = commentOwnerInfo.parameters;
                if (commentOwner.getStart(sourceFile) < position) {
                    return undefined;
                }
                if (!parameters || parameters.length === 0) {
                    // if there are no parameters, just complete to a single line JSDoc comment
                    var singleLineResult = "/** */";
                    return { newText: singleLineResult, caretOffset: 3 };
                }
                var indentationStr = getIndentationStringAtPosition(sourceFile, position);
                // A doc comment consists of the following
                // * The opening comment line
                // * the first line (without a param) for the object's untagged info (this is also where the caret ends up)
                // * the '@param'-tagged lines
                // * TODO: other tags.
                // * the closing comment line
                // * if the caret was directly in front of the object, then we add an extra line and indentation.
                var preamble = "/**" + newLine + indentationStr + " * ";
                var result = preamble + newLine +
                    parameterDocComments(parameters, ts.hasJSFileExtension(sourceFile.fileName), indentationStr, newLine) +
                    indentationStr + " */" +
                    (tokenStart === position ? newLine + indentationStr : "");
                return { newText: result, caretOffset: preamble.length };
            }
            JsDoc.getDocCommentTemplateAtPosition = getDocCommentTemplateAtPosition;
            function getIndentationStringAtPosition(sourceFile, position) {
                var text = sourceFile.text;
                var lineStart = ts.getLineStartPositionForPosition(position, sourceFile);
                var pos = lineStart;
                for (; pos <= position && ts.isWhiteSpaceSingleLine(text.charCodeAt(pos)); pos++)
                    ;
                return text.slice(lineStart, pos);
            }
            function parameterDocComments(parameters, isJavaScriptFile, indentationStr, newLine) {
                return parameters.map(function (_a, i) {
                    var name = _a.name, dotDotDotToken = _a.dotDotDotToken;
                    var paramName = name.kind === 75 /* Identifier */ ? name.text : "param" + i;
                    var type = isJavaScriptFile ? (dotDotDotToken ? "{...any} " : "{any} ") : "";
                    return indentationStr + " * @param " + type + paramName + newLine;
                }).join("");
            }
            function getCommentOwnerInfo(tokenAtPos) {
                return ts.forEachAncestor(tokenAtPos, getCommentOwnerInfoWorker);
            }
            function getCommentOwnerInfoWorker(commentOwner) {
                switch (commentOwner.kind) {
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 161 /* MethodDeclaration */:
                    case 162 /* Constructor */:
                    case 160 /* MethodSignature */:
                        var parameters = commentOwner.parameters;
                        return { commentOwner: commentOwner, parameters: parameters };
                    case 281 /* PropertyAssignment */:
                        return getCommentOwnerInfoWorker(commentOwner.initializer);
                    case 245 /* ClassDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                    case 158 /* PropertySignature */:
                    case 248 /* EnumDeclaration */:
                    case 284 /* EnumMember */:
                    case 247 /* TypeAliasDeclaration */:
                        return { commentOwner: commentOwner };
                    case 225 /* VariableStatement */: {
                        var varStatement = commentOwner;
                        var varDeclarations = varStatement.declarationList.declarations;
                        var parameters_1 = varDeclarations.length === 1 && varDeclarations[0].initializer
                            ? getParametersFromRightHandSideOfAssignment(varDeclarations[0].initializer)
                            : undefined;
                        return { commentOwner: commentOwner, parameters: parameters_1 };
                    }
                    case 290 /* SourceFile */:
                        return "quit";
                    case 249 /* ModuleDeclaration */:
                        // If in walking up the tree, we hit a a nested namespace declaration,
                        // then we must be somewhere within a dotted namespace name; however we don't
                        // want to give back a JSDoc template for the 'b' or 'c' in 'namespace a.b.c { }'.
                        return commentOwner.parent.kind === 249 /* ModuleDeclaration */ ? undefined : { commentOwner: commentOwner };
                    case 209 /* BinaryExpression */: {
                        var be = commentOwner;
                        if (ts.getAssignmentDeclarationKind(be) === 0 /* None */) {
                            return "quit";
                        }
                        var parameters_2 = ts.isFunctionLike(be.right) ? be.right.parameters : ts.emptyArray;
                        return { commentOwner: commentOwner, parameters: parameters_2 };
                    }
                }
            }
            /**
             * Digs into an an initializer or RHS operand of an assignment operation
             * to get the parameters of an apt signature corresponding to a
             * function expression or a class expression.
             *
             * @param rightHandSide the expression which may contain an appropriate set of parameters
             * @returns the parameters of a signature found on the RHS if one exists; otherwise 'emptyArray'.
             */
            function getParametersFromRightHandSideOfAssignment(rightHandSide) {
                while (rightHandSide.kind === 200 /* ParenthesizedExpression */) {
                    rightHandSide = rightHandSide.expression;
                }
                switch (rightHandSide.kind) {
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                        return rightHandSide.parameters;
                    case 214 /* ClassExpression */: {
                        var ctr = ts.find(rightHandSide.members, ts.isConstructorDeclaration);
                        return ctr ? ctr.parameters : ts.emptyArray;
                    }
                }
                return ts.emptyArray;
            }
        })(JsDoc = ts.JsDoc || (ts.JsDoc = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var NavigateTo;
        (function (NavigateTo) {
            function getNavigateToItems(sourceFiles, checker, cancellationToken, searchValue, maxResultCount, excludeDtsFiles) {
                var patternMatcher = ts.createPatternMatcher(searchValue);
                if (!patternMatcher)
                    return ts.emptyArray;
                var rawItems = [];
                var _loop_5 = function (sourceFile) {
                    cancellationToken.throwIfCancellationRequested();
                    if (excludeDtsFiles && sourceFile.isDeclarationFile) {
                        return "continue";
                    }
                    sourceFile.getNamedDeclarations().forEach(function (declarations, name) {
                        getItemsFromNamedDeclaration(patternMatcher, name, declarations, checker, sourceFile.fileName, rawItems);
                    });
                };
                // Search the declarations in all files and output matched NavigateToItem into array of NavigateToItem[]
                for (var _i = 0, sourceFiles_4 = sourceFiles; _i < sourceFiles_4.length; _i++) {
                    var sourceFile = sourceFiles_4[_i];
                    _loop_5(sourceFile);
                }
                rawItems.sort(compareNavigateToItems);
                return (maxResultCount === undefined ? rawItems : rawItems.slice(0, maxResultCount)).map(createNavigateToItem);
            }
            NavigateTo.getNavigateToItems = getNavigateToItems;
            function getItemsFromNamedDeclaration(patternMatcher, name, declarations, checker, fileName, rawItems) {
                // First do a quick check to see if the name of the declaration matches the
                // last portion of the (possibly) dotted name they're searching for.
                var match = patternMatcher.getMatchForLastSegmentOfPattern(name);
                if (!match) {
                    return; // continue to next named declarations
                }
                for (var _i = 0, declarations_3 = declarations; _i < declarations_3.length; _i++) {
                    var declaration = declarations_3[_i];
                    if (!shouldKeepItem(declaration, checker))
                        continue;
                    if (patternMatcher.patternContainsDots) {
                        // If the pattern has dots in it, then also see if the declaration container matches as well.
                        var fullMatch = patternMatcher.getFullMatch(getContainers(declaration), name);
                        if (fullMatch) {
                            rawItems.push({ name: name, fileName: fileName, matchKind: fullMatch.kind, isCaseSensitive: fullMatch.isCaseSensitive, declaration: declaration });
                        }
                    }
                    else {
                        rawItems.push({ name: name, fileName: fileName, matchKind: match.kind, isCaseSensitive: match.isCaseSensitive, declaration: declaration });
                    }
                }
            }
            function shouldKeepItem(declaration, checker) {
                switch (declaration.kind) {
                    case 255 /* ImportClause */:
                    case 258 /* ImportSpecifier */:
                    case 253 /* ImportEqualsDeclaration */:
                        var importer = checker.getSymbolAtLocation(declaration.name); // TODO: GH#18217
                        var imported = checker.getAliasedSymbol(importer);
                        return importer.escapedName !== imported.escapedName;
                    default:
                        return true;
                }
            }
            function tryAddSingleDeclarationName(declaration, containers) {
                var name = ts.getNameOfDeclaration(declaration);
                return !!name && (pushLiteral(name, containers) || name.kind === 154 /* ComputedPropertyName */ && tryAddComputedPropertyName(name.expression, containers));
            }
            // Only added the names of computed properties if they're simple dotted expressions, like:
            //
            //      [X.Y.Z]() { }
            function tryAddComputedPropertyName(expression, containers) {
                return pushLiteral(expression, containers)
                    || ts.isPropertyAccessExpression(expression) && (containers.push(expression.name.text), true) && tryAddComputedPropertyName(expression.expression, containers);
            }
            function pushLiteral(node, containers) {
                return ts.isPropertyNameLiteral(node) && (containers.push(ts.getTextOfIdentifierOrLiteral(node)), true);
            }
            function getContainers(declaration) {
                var containers = [];
                // First, if we started with a computed property name, then add all but the last
                // portion into the container array.
                var name = ts.getNameOfDeclaration(declaration);
                if (name && name.kind === 154 /* ComputedPropertyName */ && !tryAddComputedPropertyName(name.expression, containers)) {
                    return ts.emptyArray;
                }
                // Don't include the last portion.
                containers.shift();
                // Now, walk up our containers, adding all their names to the container array.
                var container = ts.getContainerNode(declaration);
                while (container) {
                    if (!tryAddSingleDeclarationName(container, containers)) {
                        return ts.emptyArray;
                    }
                    container = ts.getContainerNode(container);
                }
                return containers.reverse();
            }
            function compareNavigateToItems(i1, i2) {
                // TODO(cyrusn): get the gamut of comparisons that VS already uses here.
                return ts.compareValues(i1.matchKind, i2.matchKind)
                    || ts.compareStringsCaseSensitiveUI(i1.name, i2.name);
            }
            function createNavigateToItem(rawItem) {
                var declaration = rawItem.declaration;
                var container = ts.getContainerNode(declaration);
                var containerName = container && ts.getNameOfDeclaration(container);
                return {
                    name: rawItem.name,
                    kind: ts.getNodeKind(declaration),
                    kindModifiers: ts.getNodeModifiers(declaration),
                    matchKind: ts.PatternMatchKind[rawItem.matchKind],
                    isCaseSensitive: rawItem.isCaseSensitive,
                    fileName: rawItem.fileName,
                    textSpan: ts.createTextSpanFromNode(declaration),
                    // TODO(jfreeman): What should be the containerName when the container has a computed name?
                    containerName: containerName ? containerName.text : "",
                    containerKind: containerName ? ts.getNodeKind(container) : "" /* unknown */,
                };
            }
        })(NavigateTo = ts.NavigateTo || (ts.NavigateTo = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var NavigationBar;
        (function (NavigationBar) {
            var _a;
            /**
             * Matches all whitespace characters in a string. Eg:
             *
             * "app.
             *
             * onactivated"
             *
             * matches because of the newline, whereas
             *
             * "app.onactivated"
             *
             * does not match.
             */
            var whiteSpaceRegex = /\s+/g;
            /**
             * Maximum amount of characters to return
             * The amount was chosen arbitrarily.
             */
            var maxLength = 150;
            // Keep sourceFile handy so we don't have to search for it every time we need to call `getText`.
            var curCancellationToken;
            var curSourceFile;
            /**
             * For performance, we keep navigation bar parents on a stack rather than passing them through each recursion.
             * `parent` is the current parent and is *not* stored in parentsStack.
             * `startNode` sets a new parent and `endNode` returns to the previous parent.
             */
            var parentsStack = [];
            var parent;
            var trackedEs5ClassesStack = [];
            var trackedEs5Classes;
            // NavigationBarItem requires an array, but will not mutate it, so just give it this for performance.
            var emptyChildItemArray = [];
            function getNavigationBarItems(sourceFile, cancellationToken) {
                curCancellationToken = cancellationToken;
                curSourceFile = sourceFile;
                try {
                    return ts.map(primaryNavBarMenuItems(rootNavigationBarNode(sourceFile)), convertToPrimaryNavBarMenuItem);
                }
                finally {
                    reset();
                }
            }
            NavigationBar.getNavigationBarItems = getNavigationBarItems;
            function getNavigationTree(sourceFile, cancellationToken) {
                curCancellationToken = cancellationToken;
                curSourceFile = sourceFile;
                try {
                    return convertToTree(rootNavigationBarNode(sourceFile));
                }
                finally {
                    reset();
                }
            }
            NavigationBar.getNavigationTree = getNavigationTree;
            function reset() {
                curSourceFile = undefined;
                curCancellationToken = undefined;
                parentsStack = [];
                parent = undefined;
                emptyChildItemArray = [];
            }
            function nodeText(node) {
                return cleanText(node.getText(curSourceFile));
            }
            function navigationBarNodeKind(n) {
                return n.node.kind;
            }
            function pushChild(parent, child) {
                if (parent.children) {
                    parent.children.push(child);
                }
                else {
                    parent.children = [child];
                }
            }
            function rootNavigationBarNode(sourceFile) {
                ts.Debug.assert(!parentsStack.length);
                var root = { node: sourceFile, name: undefined, additionalNodes: undefined, parent: undefined, children: undefined, indent: 0 };
                parent = root;
                for (var _i = 0, _a = sourceFile.statements; _i < _a.length; _i++) {
                    var statement = _a[_i];
                    addChildrenRecursively(statement);
                }
                endNode();
                ts.Debug.assert(!parent && !parentsStack.length);
                return root;
            }
            function addLeafNode(node, name) {
                pushChild(parent, emptyNavigationBarNode(node, name));
            }
            function emptyNavigationBarNode(node, name) {
                return {
                    node: node,
                    name: name || (ts.isDeclaration(node) || ts.isExpression(node) ? ts.getNameOfDeclaration(node) : undefined),
                    additionalNodes: undefined,
                    parent: parent,
                    children: undefined,
                    indent: parent.indent + 1
                };
            }
            function addTrackedEs5Class(name) {
                if (!trackedEs5Classes) {
                    trackedEs5Classes = ts.createMap();
                }
                trackedEs5Classes.set(name, true);
            }
            function endNestedNodes(depth) {
                for (var i = 0; i < depth; i++)
                    endNode();
            }
            function startNestedNodes(targetNode, entityName) {
                var names = [];
                while (!ts.isPropertyNameLiteral(entityName)) {
                    var name = ts.getNameOrArgument(entityName);
                    var nameText = ts.getElementOrPropertyAccessName(entityName);
                    entityName = entityName.expression;
                    if (nameText === "prototype" || ts.isPrivateIdentifier(name))
                        continue;
                    names.push(name);
                }
                names.push(entityName);
                for (var i = names.length - 1; i > 0; i--) {
                    var name = names[i];
                    startNode(targetNode, name);
                }
                return [names.length - 1, names[0]];
            }
            /**
             * Add a new level of NavigationBarNodes.
             * This pushes to the stack, so you must call `endNode` when you are done adding to this node.
             */
            function startNode(node, name) {
                var navNode = emptyNavigationBarNode(node, name);
                pushChild(parent, navNode);
                // Save the old parent
                parentsStack.push(parent);
                trackedEs5ClassesStack.push(trackedEs5Classes);
                parent = navNode;
            }
            /** Call after calling `startNode` and adding children to it. */
            function endNode() {
                if (parent.children) {
                    mergeChildren(parent.children, parent);
                    sortChildren(parent.children);
                }
                parent = parentsStack.pop();
                trackedEs5Classes = trackedEs5ClassesStack.pop();
            }
            function addNodeWithRecursiveChild(node, child, name) {
                startNode(node, name);
                addChildrenRecursively(child);
                endNode();
            }
            /** Look for navigation bar items in node's subtree, adding them to the current `parent`. */
            function addChildrenRecursively(node) {
                var _a;
                curCancellationToken.throwIfCancellationRequested();
                if (!node || ts.isToken(node)) {
                    return;
                }
                switch (node.kind) {
                    case 162 /* Constructor */:
                        // Get parameter properties, and treat them as being on the *same* level as the constructor, not under it.
                        var ctr = node;
                        addNodeWithRecursiveChild(ctr, ctr.body);
                        // Parameter properties are children of the class, not the constructor.
                        for (var _i = 0, _b = ctr.parameters; _i < _b.length; _i++) {
                            var param = _b[_i];
                            if (ts.isParameterPropertyDeclaration(param, ctr)) {
                                addLeafNode(param);
                            }
                        }
                        break;
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                    case 160 /* MethodSignature */:
                        if (!ts.hasDynamicName(node)) {
                            addNodeWithRecursiveChild(node, node.body);
                        }
                        break;
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                        if (!ts.hasDynamicName(node)) {
                            addLeafNode(node);
                        }
                        break;
                    case 255 /* ImportClause */:
                        var importClause = node;
                        // Handle default import case e.g.:
                        //    import d from "mod";
                        if (importClause.name) {
                            addLeafNode(importClause.name);
                        }
                        // Handle named bindings in imports e.g.:
                        //    import * as NS from "mod";
                        //    import {a, b as B} from "mod";
                        var namedBindings = importClause.namedBindings;
                        if (namedBindings) {
                            if (namedBindings.kind === 256 /* NamespaceImport */) {
                                addLeafNode(namedBindings);
                            }
                            else {
                                for (var _c = 0, _d = namedBindings.elements; _c < _d.length; _c++) {
                                    var element = _d[_c];
                                    addLeafNode(element);
                                }
                            }
                        }
                        break;
                    case 282 /* ShorthandPropertyAssignment */:
                        addNodeWithRecursiveChild(node, node.name);
                        break;
                    case 283 /* SpreadAssignment */:
                        var expression = node.expression;
                        // Use the expression as the name of the SpreadAssignment, otherwise show as <unknown>.
                        ts.isIdentifier(expression) ? addLeafNode(node, expression) : addLeafNode(node);
                        break;
                    case 191 /* BindingElement */:
                    case 281 /* PropertyAssignment */:
                    case 242 /* VariableDeclaration */:
                        var _e = node, name = _e.name, initializer = _e.initializer;
                        if (ts.isBindingPattern(name)) {
                            addChildrenRecursively(name);
                        }
                        else if (initializer && isFunctionOrClassExpression(initializer)) {
                            // Add a node for the VariableDeclaration, but not for the initializer.
                            startNode(node);
                            ts.forEachChild(initializer, addChildrenRecursively);
                            endNode();
                        }
                        else {
                            addNodeWithRecursiveChild(node, initializer);
                        }
                        break;
                    case 244 /* FunctionDeclaration */:
                        var nameNode = node.name;
                        // If we see a function declaration track as a possible ES5 class
                        if (nameNode && ts.isIdentifier(nameNode)) {
                            addTrackedEs5Class(nameNode.text);
                        }
                        addNodeWithRecursiveChild(node, node.body);
                        break;
                    case 202 /* ArrowFunction */:
                    case 201 /* FunctionExpression */:
                        addNodeWithRecursiveChild(node, node.body);
                        break;
                    case 248 /* EnumDeclaration */:
                        startNode(node);
                        for (var _f = 0, _g = node.members; _f < _g.length; _f++) {
                            var member = _g[_f];
                            if (!isComputedProperty(member)) {
                                addLeafNode(member);
                            }
                        }
                        endNode();
                        break;
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                    case 246 /* InterfaceDeclaration */:
                        startNode(node);
                        for (var _h = 0, _j = node.members; _h < _j.length; _h++) {
                            var member = _j[_h];
                            addChildrenRecursively(member);
                        }
                        endNode();
                        break;
                    case 249 /* ModuleDeclaration */:
                        addNodeWithRecursiveChild(node, getInteriorModule(node).body);
                        break;
                    case 259 /* ExportAssignment */:
                    case 263 /* ExportSpecifier */:
                    case 253 /* ImportEqualsDeclaration */:
                    case 167 /* IndexSignature */:
                    case 165 /* CallSignature */:
                    case 166 /* ConstructSignature */:
                    case 247 /* TypeAliasDeclaration */:
                        addLeafNode(node);
                        break;
                    case 196 /* CallExpression */:
                    case 209 /* BinaryExpression */: {
                        var special = ts.getAssignmentDeclarationKind(node);
                        switch (special) {
                            case 1 /* ExportsProperty */:
                            case 2 /* ModuleExports */:
                                addNodeWithRecursiveChild(node, node.right);
                                return;
                            case 6 /* Prototype */:
                            case 3 /* PrototypeProperty */: {
                                var binaryExpression = node;
                                var assignmentTarget = binaryExpression.left;
                                var prototypeAccess = special === 3 /* PrototypeProperty */ ?
                                    assignmentTarget.expression :
                                    assignmentTarget;
                                var depth = 0;
                                var className = void 0;
                                // If we see a prototype assignment, start tracking the target as a class
                                // This is only done for simple classes not nested assignments.
                                if (ts.isIdentifier(prototypeAccess.expression)) {
                                    addTrackedEs5Class(prototypeAccess.expression.text);
                                    className = prototypeAccess.expression;
                                }
                                else {
                                    _a = startNestedNodes(binaryExpression, prototypeAccess.expression), depth = _a[0], className = _a[1];
                                }
                                if (special === 6 /* Prototype */) {
                                    if (ts.isObjectLiteralExpression(binaryExpression.right)) {
                                        if (binaryExpression.right.properties.length > 0) {
                                            startNode(binaryExpression, className);
                                            ts.forEachChild(binaryExpression.right, addChildrenRecursively);
                                            endNode();
                                        }
                                    }
                                }
                                else if (ts.isFunctionExpression(binaryExpression.right) || ts.isArrowFunction(binaryExpression.right)) {
                                    addNodeWithRecursiveChild(node, binaryExpression.right, className);
                                }
                                else {
                                    startNode(binaryExpression, className);
                                    addNodeWithRecursiveChild(node, binaryExpression.right, assignmentTarget.name);
                                    endNode();
                                }
                                endNestedNodes(depth);
                                return;
                            }
                            case 7 /* ObjectDefinePropertyValue */:
                            case 9 /* ObjectDefinePrototypeProperty */: {
                                var defineCall = node;
                                var className = special === 7 /* ObjectDefinePropertyValue */ ?
                                    defineCall.arguments[0] :
                                    defineCall.arguments[0].expression;
                                var memberName = defineCall.arguments[1];
                                var _k = startNestedNodes(node, className), depth = _k[0], classNameIdentifier = _k[1];
                                startNode(node, classNameIdentifier);
                                startNode(node, ts.setTextRange(ts.createIdentifier(memberName.text), memberName));
                                addChildrenRecursively(node.arguments[2]);
                                endNode();
                                endNode();
                                endNestedNodes(depth);
                                return;
                            }
                            case 5 /* Property */: {
                                var binaryExpression = node;
                                var assignmentTarget = binaryExpression.left;
                                var targetFunction = assignmentTarget.expression;
                                if (ts.isIdentifier(targetFunction) && ts.getElementOrPropertyAccessName(assignmentTarget) !== "prototype" &&
                                    trackedEs5Classes && trackedEs5Classes.has(targetFunction.text)) {
                                    if (ts.isFunctionExpression(binaryExpression.right) || ts.isArrowFunction(binaryExpression.right)) {
                                        addNodeWithRecursiveChild(node, binaryExpression.right, targetFunction);
                                    }
                                    else if (ts.isBindableStaticAccessExpression(assignmentTarget)) {
                                        startNode(binaryExpression, targetFunction);
                                        addNodeWithRecursiveChild(binaryExpression.left, binaryExpression.right, ts.getNameOrArgument(assignmentTarget));
                                        endNode();
                                    }
                                    return;
                                }
                                break;
                            }
                            case 4 /* ThisProperty */:
                            case 0 /* None */:
                            case 8 /* ObjectDefinePropertyExports */:
                                break;
                            default:
                                ts.Debug.assertNever(special);
                        }
                    }
                    // falls through
                    default:
                        if (ts.hasJSDocNodes(node)) {
                            ts.forEach(node.jsDoc, function (jsDoc) {
                                ts.forEach(jsDoc.tags, function (tag) {
                                    if (ts.isJSDocTypeAlias(tag)) {
                                        addLeafNode(tag);
                                    }
                                });
                            });
                        }
                        ts.forEachChild(node, addChildrenRecursively);
                }
            }
            /** Merge declarations of the same kind. */
            function mergeChildren(children, node) {
                var nameToItems = ts.createMap();
                ts.filterMutate(children, function (child, index) {
                    var declName = child.name || ts.getNameOfDeclaration(child.node);
                    var name = declName && nodeText(declName);
                    if (!name) {
                        // Anonymous items are never merged.
                        return true;
                    }
                    var itemsWithSameName = nameToItems.get(name);
                    if (!itemsWithSameName) {
                        nameToItems.set(name, child);
                        return true;
                    }
                    if (itemsWithSameName instanceof Array) {
                        for (var _i = 0, itemsWithSameName_1 = itemsWithSameName; _i < itemsWithSameName_1.length; _i++) {
                            var itemWithSameName = itemsWithSameName_1[_i];
                            if (tryMerge(itemWithSameName, child, index, node)) {
                                return false;
                            }
                        }
                        itemsWithSameName.push(child);
                        return true;
                    }
                    else {
                        var itemWithSameName = itemsWithSameName;
                        if (tryMerge(itemWithSameName, child, index, node)) {
                            return false;
                        }
                        nameToItems.set(name, [itemWithSameName, child]);
                        return true;
                    }
                });
            }
            var isEs5ClassMember = (_a = {},
                _a[5 /* Property */] = true,
                _a[3 /* PrototypeProperty */] = true,
                _a[7 /* ObjectDefinePropertyValue */] = true,
                _a[9 /* ObjectDefinePrototypeProperty */] = true,
                _a[0 /* None */] = false,
                _a[1 /* ExportsProperty */] = false,
                _a[2 /* ModuleExports */] = false,
                _a[8 /* ObjectDefinePropertyExports */] = false,
                _a[6 /* Prototype */] = true,
                _a[4 /* ThisProperty */] = false,
                _a);
            function tryMergeEs5Class(a, b, bIndex, parent) {
                function isPossibleConstructor(node) {
                    return ts.isFunctionExpression(node) || ts.isFunctionDeclaration(node) || ts.isVariableDeclaration(node);
                }
                var bAssignmentDeclarationKind = ts.isBinaryExpression(b.node) || ts.isCallExpression(b.node) ?
                    ts.getAssignmentDeclarationKind(b.node) :
                    0 /* None */;
                var aAssignmentDeclarationKind = ts.isBinaryExpression(a.node) || ts.isCallExpression(a.node) ?
                    ts.getAssignmentDeclarationKind(a.node) :
                    0 /* None */;
                // We treat this as an es5 class and merge the nodes in in one of several cases
                if ((isEs5ClassMember[bAssignmentDeclarationKind] && isEs5ClassMember[aAssignmentDeclarationKind]) // merge two class elements
                    || (isPossibleConstructor(a.node) && isEs5ClassMember[bAssignmentDeclarationKind]) // ctor function & member
                    || (isPossibleConstructor(b.node) && isEs5ClassMember[aAssignmentDeclarationKind]) // member & ctor function
                    || (ts.isClassDeclaration(a.node) && isEs5ClassMember[bAssignmentDeclarationKind]) // class (generated) & member
                    || (ts.isClassDeclaration(b.node) && isEs5ClassMember[aAssignmentDeclarationKind]) // member & class (generated)
                    || (ts.isClassDeclaration(a.node) && isPossibleConstructor(b.node)) // class (generated) & ctor
                    || (ts.isClassDeclaration(b.node) && isPossibleConstructor(a.node)) // ctor & class (generated)
                ) {
                    var lastANode = a.additionalNodes && ts.lastOrUndefined(a.additionalNodes) || a.node;
                    if ((!ts.isClassDeclaration(a.node) && !ts.isClassDeclaration(b.node)) // If neither outline node is a class
                        || isPossibleConstructor(a.node) || isPossibleConstructor(b.node) // If either function is a constructor function
                    ) {
                        var ctorFunction = isPossibleConstructor(a.node) ? a.node :
                            isPossibleConstructor(b.node) ? b.node :
                                undefined;
                        if (ctorFunction !== undefined) {
                            var ctorNode = ts.setTextRange(ts.createConstructor(/* decorators */ undefined, /* modifiers */ undefined, [], /* body */ undefined), ctorFunction);
                            var ctor = emptyNavigationBarNode(ctorNode);
                            ctor.indent = a.indent + 1;
                            ctor.children = a.node === ctorFunction ? a.children : b.children;
                            a.children = a.node === ctorFunction ? ts.concatenate([ctor], b.children || [b]) : ts.concatenate(a.children || [a], [ctor]);
                        }
                        else {
                            if (a.children || b.children) {
                                a.children = ts.concatenate(a.children || [a], b.children || [b]);
                                if (a.children) {
                                    mergeChildren(a.children, a);
                                    sortChildren(a.children);
                                }
                            }
                        }
                        lastANode = a.node = ts.setTextRange(ts.createClassDeclaration(
                        /* decorators */ undefined, 
                        /* modifiers */ undefined, a.name || ts.createIdentifier("__class__"), 
                        /* typeParameters */ undefined, 
                        /* heritageClauses */ undefined, []), a.node);
                    }
                    else {
                        a.children = ts.concatenate(a.children, b.children);
                        if (a.children) {
                            mergeChildren(a.children, a);
                        }
                    }
                    var bNode = b.node;
                    // We merge if the outline node previous to b (bIndex - 1) is already part of the current class
                    // We do this so that statements between class members that do not generate outline nodes do not split up the class outline:
                    // Ex This should produce one outline node C:
                    //    function C() {}; a = 1; C.prototype.m = function () {}
                    // Ex This will produce 3 outline nodes: C, a, C
                    //    function C() {}; let a = 1; C.prototype.m = function () {}
                    if (parent.children[bIndex - 1].node.end === lastANode.end) {
                        ts.setTextRange(lastANode, { pos: lastANode.pos, end: bNode.end });
                    }
                    else {
                        if (!a.additionalNodes)
                            a.additionalNodes = [];
                        a.additionalNodes.push(ts.setTextRange(ts.createClassDeclaration(
                        /* decorators */ undefined, 
                        /* modifiers */ undefined, a.name || ts.createIdentifier("__class__"), 
                        /* typeParameters */ undefined, 
                        /* heritageClauses */ undefined, []), b.node));
                    }
                    return true;
                }
                return bAssignmentDeclarationKind === 0 /* None */ ? false : true;
            }
            function tryMerge(a, b, bIndex, parent) {
                // const v = false as boolean;
                if (tryMergeEs5Class(a, b, bIndex, parent)) {
                    return true;
                }
                if (shouldReallyMerge(a.node, b.node, parent)) {
                    merge(a, b);
                    return true;
                }
                return false;
            }
            /** a and b have the same name, but they may not be mergeable. */
            function shouldReallyMerge(a, b, parent) {
                if (a.kind !== b.kind || a.parent !== b.parent && !(isOwnChild(a, parent) && isOwnChild(b, parent))) {
                    return false;
                }
                switch (a.kind) {
                    case 159 /* PropertyDeclaration */:
                    case 161 /* MethodDeclaration */:
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                        return ts.hasModifier(a, 32 /* Static */) === ts.hasModifier(b, 32 /* Static */);
                    case 249 /* ModuleDeclaration */:
                        return areSameModule(a, b);
                    default:
                        return true;
                }
            }
            // We want to merge own children like `I` in in `module A { interface I {} } module A { interface I {} }`
            // We don't want to merge unrelated children like `m` in `const o = { a: { m() {} }, b: { m() {} } };`
            function isOwnChild(n, parent) {
                var par = ts.isModuleBlock(n.parent) ? n.parent.parent : n.parent;
                return par === parent.node || ts.contains(parent.additionalNodes, par);
            }
            // We use 1 NavNode to represent 'A.B.C', but there are multiple source nodes.
            // Only merge module nodes that have the same chain. Don't merge 'A.B.C' with 'A'!
            function areSameModule(a, b) {
                // TODO: GH#18217
                return a.body.kind === b.body.kind && (a.body.kind !== 249 /* ModuleDeclaration */ || areSameModule(a.body, b.body));
            }
            /** Merge source into target. Source should be thrown away after this is called. */
            function merge(target, source) {
                var _a;
                target.additionalNodes = target.additionalNodes || [];
                target.additionalNodes.push(source.node);
                if (source.additionalNodes) {
                    (_a = target.additionalNodes).push.apply(_a, source.additionalNodes);
                }
                target.children = ts.concatenate(target.children, source.children);
                if (target.children) {
                    mergeChildren(target.children, target);
                    sortChildren(target.children);
                }
            }
            /** Recursively ensure that each NavNode's children are in sorted order. */
            function sortChildren(children) {
                children.sort(compareChildren);
            }
            function compareChildren(child1, child2) {
                return ts.compareStringsCaseSensitiveUI(tryGetName(child1.node), tryGetName(child2.node)) // TODO: GH#18217
                    || ts.compareValues(navigationBarNodeKind(child1), navigationBarNodeKind(child2));
            }
            /**
             * This differs from getItemName because this is just used for sorting.
             * We only sort nodes by name that have a more-or-less "direct" name, as opposed to `new()` and the like.
             * So `new()` can still come before an `aardvark` method.
             */
            function tryGetName(node) {
                if (node.kind === 249 /* ModuleDeclaration */) {
                    return getModuleName(node);
                }
                var declName = ts.getNameOfDeclaration(node);
                if (declName && ts.isPropertyName(declName)) {
                    var propertyName = ts.getPropertyNameForPropertyNameNode(declName);
                    return propertyName && ts.unescapeLeadingUnderscores(propertyName);
                }
                switch (node.kind) {
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                    case 214 /* ClassExpression */:
                        return getFunctionOrClassName(node);
                    default:
                        return undefined;
                }
            }
            function getItemName(node, name) {
                if (node.kind === 249 /* ModuleDeclaration */) {
                    return cleanText(getModuleName(node));
                }
                if (name) {
                    var text = ts.isIdentifier(name) ? name.text
                        : ts.isElementAccessExpression(name) ? "[" + nodeText(name.argumentExpression) + "]"
                            : nodeText(name);
                    if (text.length > 0) {
                        return cleanText(text);
                    }
                }
                switch (node.kind) {
                    case 290 /* SourceFile */:
                        var sourceFile = node;
                        return ts.isExternalModule(sourceFile)
                            ? "\"" + ts.escapeString(ts.getBaseFileName(ts.removeFileExtension(ts.normalizePath(sourceFile.fileName)))) + "\""
                            : "<global>";
                    case 259 /* ExportAssignment */:
                        return ts.isExportAssignment(node) && node.isExportEquals ? "export=" /* ExportEquals */ : "default" /* Default */;
                    case 202 /* ArrowFunction */:
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                        if (ts.getModifierFlags(node) & 512 /* Default */) {
                            return "default";
                        }
                        // We may get a string with newlines or other whitespace in the case of an object dereference
                        // (eg: "app\n.onactivated"), so we should remove the whitespace for readabiltiy in the
                        // navigation bar.
                        return getFunctionOrClassName(node);
                    case 162 /* Constructor */:
                        return "constructor";
                    case 166 /* ConstructSignature */:
                        return "new()";
                    case 165 /* CallSignature */:
                        return "()";
                    case 167 /* IndexSignature */:
                        return "[]";
                    default:
                        return "<unknown>";
                }
            }
            /** Flattens the NavNode tree to a list of items to appear in the primary navbar menu. */
            function primaryNavBarMenuItems(root) {
                // The primary (middle) navbar menu displays the general code navigation hierarchy, similar to the navtree.
                // The secondary (right) navbar menu displays the child items of whichever primary item is selected.
                // Some less interesting items without their own child navigation items (e.g. a local variable declaration) only show up in the secondary menu.
                var primaryNavBarMenuItems = [];
                function recur(item) {
                    if (shouldAppearInPrimaryNavBarMenu(item)) {
                        primaryNavBarMenuItems.push(item);
                        if (item.children) {
                            for (var _i = 0, _a = item.children; _i < _a.length; _i++) {
                                var child = _a[_i];
                                recur(child);
                            }
                        }
                    }
                }
                recur(root);
                return primaryNavBarMenuItems;
                /** Determines if a node should appear in the primary navbar menu. */
                function shouldAppearInPrimaryNavBarMenu(item) {
                    // Items with children should always appear in the primary navbar menu.
                    if (item.children) {
                        return true;
                    }
                    // Some nodes are otherwise important enough to always include in the primary navigation menu.
                    switch (navigationBarNodeKind(item)) {
                        case 245 /* ClassDeclaration */:
                        case 214 /* ClassExpression */:
                        case 248 /* EnumDeclaration */:
                        case 246 /* InterfaceDeclaration */:
                        case 249 /* ModuleDeclaration */:
                        case 290 /* SourceFile */:
                        case 247 /* TypeAliasDeclaration */:
                        case 321 /* JSDocTypedefTag */:
                        case 314 /* JSDocCallbackTag */:
                            return true;
                        case 202 /* ArrowFunction */:
                        case 244 /* FunctionDeclaration */:
                        case 201 /* FunctionExpression */:
                            return isTopLevelFunctionDeclaration(item);
                        default:
                            return false;
                    }
                    function isTopLevelFunctionDeclaration(item) {
                        if (!item.node.body) {
                            return false;
                        }
                        switch (navigationBarNodeKind(item.parent)) {
                            case 250 /* ModuleBlock */:
                            case 290 /* SourceFile */:
                            case 161 /* MethodDeclaration */:
                            case 162 /* Constructor */:
                                return true;
                            default:
                                return false;
                        }
                    }
                }
            }
            function convertToTree(n) {
                return {
                    text: getItemName(n.node, n.name),
                    kind: ts.getNodeKind(n.node),
                    kindModifiers: getModifiers(n.node),
                    spans: getSpans(n),
                    nameSpan: n.name && getNodeSpan(n.name),
                    childItems: ts.map(n.children, convertToTree)
                };
            }
            function convertToPrimaryNavBarMenuItem(n) {
                return {
                    text: getItemName(n.node, n.name),
                    kind: ts.getNodeKind(n.node),
                    kindModifiers: getModifiers(n.node),
                    spans: getSpans(n),
                    childItems: ts.map(n.children, convertToSecondaryNavBarMenuItem) || emptyChildItemArray,
                    indent: n.indent,
                    bolded: false,
                    grayed: false
                };
                function convertToSecondaryNavBarMenuItem(n) {
                    return {
                        text: getItemName(n.node, n.name),
                        kind: ts.getNodeKind(n.node),
                        kindModifiers: ts.getNodeModifiers(n.node),
                        spans: getSpans(n),
                        childItems: emptyChildItemArray,
                        indent: 0,
                        bolded: false,
                        grayed: false
                    };
                }
            }
            function getSpans(n) {
                var spans = [getNodeSpan(n.node)];
                if (n.additionalNodes) {
                    for (var _i = 0, _a = n.additionalNodes; _i < _a.length; _i++) {
                        var node = _a[_i];
                        spans.push(getNodeSpan(node));
                    }
                }
                return spans;
            }
            function getModuleName(moduleDeclaration) {
                // We want to maintain quotation marks.
                if (ts.isAmbientModule(moduleDeclaration)) {
                    return ts.getTextOfNode(moduleDeclaration.name);
                }
                // Otherwise, we need to aggregate each identifier to build up the qualified name.
                var result = [];
                result.push(ts.getTextOfIdentifierOrLiteral(moduleDeclaration.name));
                while (moduleDeclaration.body && moduleDeclaration.body.kind === 249 /* ModuleDeclaration */) {
                    moduleDeclaration = moduleDeclaration.body;
                    result.push(ts.getTextOfIdentifierOrLiteral(moduleDeclaration.name));
                }
                return result.join(".");
            }
            /**
             * For 'module A.B.C', we want to get the node for 'C'.
             * We store 'A' as associated with a NavNode, and use getModuleName to traverse down again.
             */
            function getInteriorModule(decl) {
                return decl.body && ts.isModuleDeclaration(decl.body) ? getInteriorModule(decl.body) : decl;
            }
            function isComputedProperty(member) {
                return !member.name || member.name.kind === 154 /* ComputedPropertyName */;
            }
            function getNodeSpan(node) {
                return node.kind === 290 /* SourceFile */ ? ts.createTextSpanFromRange(node) : ts.createTextSpanFromNode(node, curSourceFile);
            }
            function getModifiers(node) {
                if (node.parent && node.parent.kind === 242 /* VariableDeclaration */) {
                    node = node.parent;
                }
                return ts.getNodeModifiers(node);
            }
            function getFunctionOrClassName(node) {
                var parent = node.parent;
                if (node.name && ts.getFullWidth(node.name) > 0) {
                    return cleanText(ts.declarationNameToString(node.name));
                }
                // See if it is a var initializer. If so, use the var name.
                else if (ts.isVariableDeclaration(parent)) {
                    return cleanText(ts.declarationNameToString(parent.name));
                }
                // See if it is of the form "<expr> = function(){...}". If so, use the text from the left-hand side.
                else if (ts.isBinaryExpression(parent) && parent.operatorToken.kind === 62 /* EqualsToken */) {
                    return nodeText(parent.left).replace(whiteSpaceRegex, "");
                }
                // See if it is a property assignment, and if so use the property name
                else if (ts.isPropertyAssignment(parent)) {
                    return nodeText(parent.name);
                }
                // Default exports are named "default"
                else if (ts.getModifierFlags(node) & 512 /* Default */) {
                    return "default";
                }
                else if (ts.isClassLike(node)) {
                    return "<class>";
                }
                else if (ts.isCallExpression(parent)) {
                    var name = getCalledExpressionName(parent.expression);
                    if (name !== undefined) {
                        name = cleanText(name);
                        if (name.length > maxLength) {
                            return name + " callback";
                        }
                        var args = cleanText(ts.mapDefined(parent.arguments, function (a) { return ts.isStringLiteralLike(a) ? a.getText(curSourceFile) : undefined; }).join(", "));
                        return name + "(" + args + ") callback";
                    }
                }
                return "<function>";
            }
            // See also 'tryGetPropertyAccessOrIdentifierToString'
            function getCalledExpressionName(expr) {
                if (ts.isIdentifier(expr)) {
                    return expr.text;
                }
                else if (ts.isPropertyAccessExpression(expr)) {
                    var left = getCalledExpressionName(expr.expression);
                    var right = expr.name.text;
                    return left === undefined ? right : left + "." + right;
                }
                else {
                    return undefined;
                }
            }
            function isFunctionOrClassExpression(node) {
                switch (node.kind) {
                    case 202 /* ArrowFunction */:
                    case 201 /* FunctionExpression */:
                    case 214 /* ClassExpression */:
                        return true;
                    default:
                        return false;
                }
            }
            function cleanText(text) {
                // Truncate to maximum amount of characters as we don't want to do a big replace operation.
                text = text.length > maxLength ? text.substring(0, maxLength) + "..." : text;
                // Replaces ECMAScript line terminators and removes the trailing `\` from each line:
                // \n - Line Feed
                // \r - Carriage Return
                // \u2028 - Line separator
                // \u2029 - Paragraph separator
                return text.replace(/\\?(\r?\n|\r|\u2028|\u2029)/g, "");
            }
        })(NavigationBar = ts.NavigationBar || (ts.NavigationBar = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var OrganizeImports;
        (function (OrganizeImports) {
            /**
             * Organize imports by:
             *   1) Removing unused imports
             *   2) Coalescing imports from the same module
             *   3) Sorting imports
             */
            function organizeImports(sourceFile, formatContext, host, program, preferences) {
                var changeTracker = ts.textChanges.ChangeTracker.fromContext({ host: host, formatContext: formatContext, preferences: preferences });
                var coalesceAndOrganizeImports = function (importGroup) { return coalesceImports(removeUnusedImports(importGroup, sourceFile, program)); };
                // All of the old ImportDeclarations in the file, in syntactic order.
                var topLevelImportDecls = sourceFile.statements.filter(ts.isImportDeclaration);
                organizeImportsWorker(topLevelImportDecls, coalesceAndOrganizeImports);
                // All of the old ExportDeclarations in the file, in syntactic order.
                var topLevelExportDecls = sourceFile.statements.filter(ts.isExportDeclaration);
                organizeImportsWorker(topLevelExportDecls, coalesceExports);
                for (var _i = 0, _a = sourceFile.statements.filter(ts.isAmbientModule); _i < _a.length; _i++) {
                    var ambientModule = _a[_i];
                    if (!ambientModule.body) {
                        continue;
                    }
                    var ambientModuleImportDecls = ambientModule.body.statements.filter(ts.isImportDeclaration);
                    organizeImportsWorker(ambientModuleImportDecls, coalesceAndOrganizeImports);
                    var ambientModuleExportDecls = ambientModule.body.statements.filter(ts.isExportDeclaration);
                    organizeImportsWorker(ambientModuleExportDecls, coalesceExports);
                }
                return changeTracker.getChanges();
                function organizeImportsWorker(oldImportDecls, coalesce) {
                    if (ts.length(oldImportDecls) === 0) {
                        return;
                    }
                    // Special case: normally, we'd expect leading and trailing trivia to follow each import
                    // around as it's sorted.  However, we do not want this to happen for leading trivia
                    // on the first import because it is probably the header comment for the file.
                    // Consider: we could do a more careful check that this trivia is actually a header,
                    // but the consequences of being wrong are very minor.
                    ts.suppressLeadingTrivia(oldImportDecls[0]);
                    var oldImportGroups = ts.group(oldImportDecls, function (importDecl) { return getExternalModuleName(importDecl.moduleSpecifier); });
                    var sortedImportGroups = ts.stableSort(oldImportGroups, function (group1, group2) { return compareModuleSpecifiers(group1[0].moduleSpecifier, group2[0].moduleSpecifier); });
                    var newImportDecls = ts.flatMap(sortedImportGroups, function (importGroup) {
                        return getExternalModuleName(importGroup[0].moduleSpecifier)
                            ? coalesce(importGroup)
                            : importGroup;
                    });
                    // Delete or replace the first import.
                    if (newImportDecls.length === 0) {
                        changeTracker.delete(sourceFile, oldImportDecls[0]);
                    }
                    else {
                        // Note: Delete the surrounding trivia because it will have been retained in newImportDecls.
                        changeTracker.replaceNodeWithNodes(sourceFile, oldImportDecls[0], newImportDecls, {
                            leadingTriviaOption: ts.textChanges.LeadingTriviaOption.Exclude,
                            trailingTriviaOption: ts.textChanges.TrailingTriviaOption.Include,
                            suffix: ts.getNewLineOrDefaultFromHost(host, formatContext.options),
                        });
                    }
                    // Delete any subsequent imports.
                    for (var i = 1; i < oldImportDecls.length; i++) {
                        changeTracker.delete(sourceFile, oldImportDecls[i]);
                    }
                }
            }
            OrganizeImports.organizeImports = organizeImports;
            function removeUnusedImports(oldImports, sourceFile, program) {
                var typeChecker = program.getTypeChecker();
                var jsxNamespace = typeChecker.getJsxNamespace(sourceFile);
                var jsxElementsPresent = !!(sourceFile.transformFlags & 2 /* ContainsJsx */);
                var usedImports = [];
                for (var _i = 0, oldImports_1 = oldImports; _i < oldImports_1.length; _i++) {
                    var importDecl = oldImports_1[_i];
                    var importClause = importDecl.importClause, moduleSpecifier = importDecl.moduleSpecifier;
                    if (!importClause) {
                        // Imports without import clauses are assumed to be included for their side effects and are not removed.
                        usedImports.push(importDecl);
                        continue;
                    }
                    var name = importClause.name, namedBindings = importClause.namedBindings;
                    // Default import
                    if (name && !isDeclarationUsed(name)) {
                        name = undefined;
                    }
                    if (namedBindings) {
                        if (ts.isNamespaceImport(namedBindings)) {
                            // Namespace import
                            if (!isDeclarationUsed(namedBindings.name)) {
                                namedBindings = undefined;
                            }
                        }
                        else {
                            // List of named imports
                            var newElements = namedBindings.elements.filter(function (e) { return isDeclarationUsed(e.name); });
                            if (newElements.length < namedBindings.elements.length) {
                                namedBindings = newElements.length
                                    ? ts.updateNamedImports(namedBindings, newElements)
                                    : undefined;
                            }
                        }
                    }
                    if (name || namedBindings) {
                        usedImports.push(updateImportDeclarationAndClause(importDecl, name, namedBindings));
                    }
                    // If a module is imported to be augmented, it’s used
                    else if (hasModuleDeclarationMatchingSpecifier(sourceFile, moduleSpecifier)) {
                        // If we’re in a declaration file, it’s safe to remove the import clause from it
                        if (sourceFile.isDeclarationFile) {
                            usedImports.push(ts.createImportDeclaration(importDecl.decorators, importDecl.modifiers, 
                            /*importClause*/ undefined, moduleSpecifier));
                        }
                        // If we’re not in a declaration file, we can’t remove the import clause even though
                        // the imported symbols are unused, because removing them makes it look like the import
                        // declaration has side effects, which will cause it to be preserved in the JS emit.
                        else {
                            usedImports.push(importDecl);
                        }
                    }
                }
                return usedImports;
                function isDeclarationUsed(identifier) {
                    // The JSX factory symbol is always used if JSX elements are present - even if they are not allowed.
                    return jsxElementsPresent && (identifier.text === jsxNamespace) || ts.FindAllReferences.Core.isSymbolReferencedInFile(identifier, typeChecker, sourceFile);
                }
            }
            function hasModuleDeclarationMatchingSpecifier(sourceFile, moduleSpecifier) {
                var moduleSpecifierText = ts.isStringLiteral(moduleSpecifier) && moduleSpecifier.text;
                return ts.isString(moduleSpecifierText) && ts.some(sourceFile.moduleAugmentations, function (moduleName) {
                    return ts.isStringLiteral(moduleName)
                        && moduleName.text === moduleSpecifierText;
                });
            }
            function getExternalModuleName(specifier) {
                return specifier !== undefined && ts.isStringLiteralLike(specifier)
                    ? specifier.text
                    : undefined;
            }
            // Internal for testing
            /**
             * @param importGroup a list of ImportDeclarations, all with the same module name.
             */
            function coalesceImports(importGroup) {
                var _a;
                if (importGroup.length === 0) {
                    return importGroup;
                }
                var _b = getCategorizedImports(importGroup), importWithoutClause = _b.importWithoutClause, typeOnlyImports = _b.typeOnlyImports, regularImports = _b.regularImports;
                var coalescedImports = [];
                if (importWithoutClause) {
                    coalescedImports.push(importWithoutClause);
                }
                for (var _i = 0, _c = [regularImports, typeOnlyImports]; _i < _c.length; _i++) {
                    var group_2 = _c[_i];
                    var isTypeOnly = group_2 === typeOnlyImports;
                    var defaultImports = group_2.defaultImports, namespaceImports = group_2.namespaceImports, namedImports = group_2.namedImports;
                    // Normally, we don't combine default and namespace imports, but it would be silly to
                    // produce two import declarations in this special case.
                    if (!isTypeOnly && defaultImports.length === 1 && namespaceImports.length === 1 && namedImports.length === 0) {
                        // Add the namespace import to the existing default ImportDeclaration.
                        var defaultImport = defaultImports[0];
                        coalescedImports.push(updateImportDeclarationAndClause(defaultImport, defaultImport.importClause.name, namespaceImports[0].importClause.namedBindings)); // TODO: GH#18217
                        continue;
                    }
                    var sortedNamespaceImports = ts.stableSort(namespaceImports, function (i1, i2) {
                        return compareIdentifiers(i1.importClause.namedBindings.name, i2.importClause.namedBindings.name);
                    }); // TODO: GH#18217
                    for (var _d = 0, sortedNamespaceImports_1 = sortedNamespaceImports; _d < sortedNamespaceImports_1.length; _d++) {
                        var namespaceImport = sortedNamespaceImports_1[_d];
                        // Drop the name, if any
                        coalescedImports.push(updateImportDeclarationAndClause(namespaceImport, /*name*/ undefined, namespaceImport.importClause.namedBindings)); // TODO: GH#18217
                    }
                    if (defaultImports.length === 0 && namedImports.length === 0) {
                        continue;
                    }
                    var newDefaultImport = void 0;
                    var newImportSpecifiers = [];
                    if (defaultImports.length === 1) {
                        newDefaultImport = defaultImports[0].importClause.name;
                    }
                    else {
                        for (var _e = 0, defaultImports_1 = defaultImports; _e < defaultImports_1.length; _e++) {
                            var defaultImport = defaultImports_1[_e];
                            newImportSpecifiers.push(ts.createImportSpecifier(ts.createIdentifier("default"), defaultImport.importClause.name)); // TODO: GH#18217
                        }
                    }
                    newImportSpecifiers.push.apply(newImportSpecifiers, ts.flatMap(namedImports, function (i) { return i.importClause.namedBindings.elements; })); // TODO: GH#18217
                    var sortedImportSpecifiers = sortSpecifiers(newImportSpecifiers);
                    var importDecl = defaultImports.length > 0
                        ? defaultImports[0]
                        : namedImports[0];
                    var newNamedImports = sortedImportSpecifiers.length === 0
                        ? newDefaultImport
                            ? undefined
                            : ts.createNamedImports(ts.emptyArray)
                        : namedImports.length === 0
                            ? ts.createNamedImports(sortedImportSpecifiers)
                            : ts.updateNamedImports(namedImports[0].importClause.namedBindings, sortedImportSpecifiers); // TODO: GH#18217
                    // Type-only imports are not allowed to combine
                    if (isTypeOnly && newDefaultImport && newNamedImports) {
                        coalescedImports.push(updateImportDeclarationAndClause(importDecl, newDefaultImport, /*namedBindings*/ undefined));
                        coalescedImports.push(updateImportDeclarationAndClause((_a = namedImports[0]) !== null && _a !== void 0 ? _a : importDecl, /*name*/ undefined, newNamedImports));
                    }
                    else {
                        coalescedImports.push(updateImportDeclarationAndClause(importDecl, newDefaultImport, newNamedImports));
                    }
                }
                return coalescedImports;
            }
            OrganizeImports.coalesceImports = coalesceImports;
            /*
             * Returns entire import declarations because they may already have been rewritten and
             * may lack parent pointers.  The desired parts can easily be recovered based on the
             * categorization.
             *
             * NB: There may be overlap between `defaultImports` and `namespaceImports`/`namedImports`.
             */
            function getCategorizedImports(importGroup) {
                var importWithoutClause;
                var typeOnlyImports = { defaultImports: [], namespaceImports: [], namedImports: [] };
                var regularImports = { defaultImports: [], namespaceImports: [], namedImports: [] };
                for (var _i = 0, importGroup_1 = importGroup; _i < importGroup_1.length; _i++) {
                    var importDeclaration = importGroup_1[_i];
                    if (importDeclaration.importClause === undefined) {
                        // Only the first such import is interesting - the others are redundant.
                        // Note: Unfortunately, we will lose trivia that was on this node.
                        importWithoutClause = importWithoutClause || importDeclaration;
                        continue;
                    }
                    var group_3 = importDeclaration.importClause.isTypeOnly ? typeOnlyImports : regularImports;
                    var _a = importDeclaration.importClause, name = _a.name, namedBindings = _a.namedBindings;
                    if (name) {
                        group_3.defaultImports.push(importDeclaration);
                    }
                    if (namedBindings) {
                        if (ts.isNamespaceImport(namedBindings)) {
                            group_3.namespaceImports.push(importDeclaration);
                        }
                        else {
                            group_3.namedImports.push(importDeclaration);
                        }
                    }
                }
                return {
                    importWithoutClause: importWithoutClause,
                    typeOnlyImports: typeOnlyImports,
                    regularImports: regularImports,
                };
            }
            // Internal for testing
            /**
             * @param exportGroup a list of ExportDeclarations, all with the same module name.
             */
            function coalesceExports(exportGroup) {
                if (exportGroup.length === 0) {
                    return exportGroup;
                }
                var _a = getCategorizedExports(exportGroup), exportWithoutClause = _a.exportWithoutClause, namedExports = _a.namedExports, typeOnlyExports = _a.typeOnlyExports;
                var coalescedExports = [];
                if (exportWithoutClause) {
                    coalescedExports.push(exportWithoutClause);
                }
                for (var _i = 0, _b = [namedExports, typeOnlyExports]; _i < _b.length; _i++) {
                    var exportGroup_1 = _b[_i];
                    if (exportGroup_1.length === 0) {
                        continue;
                    }
                    var newExportSpecifiers = [];
                    newExportSpecifiers.push.apply(newExportSpecifiers, ts.flatMap(exportGroup_1, function (i) { return i.exportClause && ts.isNamedExports(i.exportClause) ? i.exportClause.elements : ts.emptyArray; }));
                    var sortedExportSpecifiers = sortSpecifiers(newExportSpecifiers);
                    var exportDecl = exportGroup_1[0];
                    coalescedExports.push(ts.updateExportDeclaration(exportDecl, exportDecl.decorators, exportDecl.modifiers, exportDecl.exportClause && (ts.isNamedExports(exportDecl.exportClause) ?
                        ts.updateNamedExports(exportDecl.exportClause, sortedExportSpecifiers) :
                        ts.updateNamespaceExport(exportDecl.exportClause, exportDecl.exportClause.name)), exportDecl.moduleSpecifier, exportDecl.isTypeOnly));
                }
                return coalescedExports;
                /*
                 * Returns entire export declarations because they may already have been rewritten and
                 * may lack parent pointers.  The desired parts can easily be recovered based on the
                 * categorization.
                 */
                function getCategorizedExports(exportGroup) {
                    var exportWithoutClause;
                    var namedExports = [];
                    var typeOnlyExports = [];
                    for (var _i = 0, exportGroup_2 = exportGroup; _i < exportGroup_2.length; _i++) {
                        var exportDeclaration = exportGroup_2[_i];
                        if (exportDeclaration.exportClause === undefined) {
                            // Only the first such export is interesting - the others are redundant.
                            // Note: Unfortunately, we will lose trivia that was on this node.
                            exportWithoutClause = exportWithoutClause || exportDeclaration;
                        }
                        else if (exportDeclaration.isTypeOnly) {
                            typeOnlyExports.push(exportDeclaration);
                        }
                        else {
                            namedExports.push(exportDeclaration);
                        }
                    }
                    return {
                        exportWithoutClause: exportWithoutClause,
                        namedExports: namedExports,
                        typeOnlyExports: typeOnlyExports,
                    };
                }
            }
            OrganizeImports.coalesceExports = coalesceExports;
            function updateImportDeclarationAndClause(importDeclaration, name, namedBindings) {
                return ts.updateImportDeclaration(importDeclaration, importDeclaration.decorators, importDeclaration.modifiers, ts.updateImportClause(importDeclaration.importClause, name, namedBindings, importDeclaration.importClause.isTypeOnly), // TODO: GH#18217
                importDeclaration.moduleSpecifier);
            }
            function sortSpecifiers(specifiers) {
                return ts.stableSort(specifiers, function (s1, s2) {
                    return compareIdentifiers(s1.propertyName || s1.name, s2.propertyName || s2.name) ||
                        compareIdentifiers(s1.name, s2.name);
                });
            }
            /* internal */ // Exported for testing
            function compareModuleSpecifiers(m1, m2) {
                var name1 = getExternalModuleName(m1);
                var name2 = getExternalModuleName(m2);
                return ts.compareBooleans(name1 === undefined, name2 === undefined) ||
                    ts.compareBooleans(ts.isExternalModuleNameRelative(name1), ts.isExternalModuleNameRelative(name2)) ||
                    ts.compareStringsCaseInsensitive(name1, name2);
            }
            OrganizeImports.compareModuleSpecifiers = compareModuleSpecifiers;
            function compareIdentifiers(s1, s2) {
                return ts.compareStringsCaseInsensitive(s1.text, s2.text);
            }
        })(OrganizeImports = ts.OrganizeImports || (ts.OrganizeImports = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var OutliningElementsCollector;
        (function (OutliningElementsCollector) {
            function collectElements(sourceFile, cancellationToken) {
                var res = [];
                addNodeOutliningSpans(sourceFile, cancellationToken, res);
                addRegionOutliningSpans(sourceFile, res);
                return res.sort(function (span1, span2) { return span1.textSpan.start - span2.textSpan.start; });
            }
            OutliningElementsCollector.collectElements = collectElements;
            function addNodeOutliningSpans(sourceFile, cancellationToken, out) {
                var depthRemaining = 40;
                var current = 0;
                // Includes the EOF Token so that comments which aren't attached to statements are included
                var statements = __spreadArrays(sourceFile.statements, [sourceFile.endOfFileToken]);
                var n = statements.length;
                while (current < n) {
                    while (current < n && !ts.isAnyImportSyntax(statements[current])) {
                        visitNonImportNode(statements[current]);
                        current++;
                    }
                    if (current === n)
                        break;
                    var firstImport = current;
                    while (current < n && ts.isAnyImportSyntax(statements[current])) {
                        addOutliningForLeadingCommentsForNode(statements[current], sourceFile, cancellationToken, out);
                        current++;
                    }
                    var lastImport = current - 1;
                    if (lastImport !== firstImport) {
                        out.push(createOutliningSpanFromBounds(ts.findChildOfKind(statements[firstImport], 96 /* ImportKeyword */, sourceFile).getStart(sourceFile), statements[lastImport].getEnd(), "imports" /* Imports */));
                    }
                }
                function visitNonImportNode(n) {
                    var _a;
                    if (depthRemaining === 0)
                        return;
                    cancellationToken.throwIfCancellationRequested();
                    if (ts.isDeclaration(n) || n.kind === 1 /* EndOfFileToken */) {
                        addOutliningForLeadingCommentsForNode(n, sourceFile, cancellationToken, out);
                    }
                    if (isFunctionExpressionAssignedToVariable(n)) {
                        addOutliningForLeadingCommentsForNode(n.parent.parent.parent, sourceFile, cancellationToken, out);
                    }
                    var span = getOutliningSpanForNode(n, sourceFile);
                    if (span)
                        out.push(span);
                    depthRemaining--;
                    if (ts.isCallExpression(n)) {
                        depthRemaining++;
                        visitNonImportNode(n.expression);
                        depthRemaining--;
                        n.arguments.forEach(visitNonImportNode);
                        (_a = n.typeArguments) === null || _a === void 0 ? void 0 : _a.forEach(visitNonImportNode);
                    }
                    else if (ts.isIfStatement(n) && n.elseStatement && ts.isIfStatement(n.elseStatement)) {
                        // Consider an 'else if' to be on the same depth as the 'if'.
                        visitNonImportNode(n.expression);
                        visitNonImportNode(n.thenStatement);
                        depthRemaining++;
                        visitNonImportNode(n.elseStatement);
                        depthRemaining--;
                    }
                    else {
                        n.forEachChild(visitNonImportNode);
                    }
                    depthRemaining++;
                }
                function isFunctionExpressionAssignedToVariable(n) {
                    if (!ts.isFunctionExpression(n) && !ts.isArrowFunction(n)) {
                        return false;
                    }
                    var ancestor = ts.findAncestor(n, ts.isVariableStatement);
                    return !!ancestor && ts.getSingleInitializerOfVariableStatementOrPropertyDeclaration(ancestor) === n;
                }
            }
            function addRegionOutliningSpans(sourceFile, out) {
                var regions = [];
                var lineStarts = sourceFile.getLineStarts();
                for (var _i = 0, lineStarts_1 = lineStarts; _i < lineStarts_1.length; _i++) {
                    var currentLineStart = lineStarts_1[_i];
                    var lineEnd = sourceFile.getLineEndOfPosition(currentLineStart);
                    var lineText = sourceFile.text.substring(currentLineStart, lineEnd);
                    var result = isRegionDelimiter(lineText);
                    if (!result || ts.isInComment(sourceFile, currentLineStart)) {
                        continue;
                    }
                    if (!result[1]) {
                        var span = ts.createTextSpanFromBounds(sourceFile.text.indexOf("//", currentLineStart), lineEnd);
                        regions.push(createOutliningSpan(span, "region" /* Region */, span, /*autoCollapse*/ false, result[2] || "#region"));
                    }
                    else {
                        var region = regions.pop();
                        if (region) {
                            region.textSpan.length = lineEnd - region.textSpan.start;
                            region.hintSpan.length = lineEnd - region.textSpan.start;
                            out.push(region);
                        }
                    }
                }
            }
            var regionDelimiterRegExp = /^\s*\/\/\s*#(end)?region(?:\s+(.*))?(?:\r)?$/;
            function isRegionDelimiter(lineText) {
                return regionDelimiterRegExp.exec(lineText);
            }
            function addOutliningForLeadingCommentsForNode(n, sourceFile, cancellationToken, out) {
                var comments = ts.getLeadingCommentRangesOfNode(n, sourceFile);
                if (!comments)
                    return;
                var firstSingleLineCommentStart = -1;
                var lastSingleLineCommentEnd = -1;
                var singleLineCommentCount = 0;
                var sourceText = sourceFile.getFullText();
                for (var _i = 0, comments_1 = comments; _i < comments_1.length; _i++) {
                    var _a = comments_1[_i], kind = _a.kind, pos = _a.pos, end = _a.end;
                    cancellationToken.throwIfCancellationRequested();
                    switch (kind) {
                        case 2 /* SingleLineCommentTrivia */:
                            // never fold region delimiters into single-line comment regions
                            var commentText = sourceText.slice(pos, end);
                            if (isRegionDelimiter(commentText)) {
                                combineAndAddMultipleSingleLineComments();
                                singleLineCommentCount = 0;
                                break;
                            }
                            // For single line comments, combine consecutive ones (2 or more) into
                            // a single span from the start of the first till the end of the last
                            if (singleLineCommentCount === 0) {
                                firstSingleLineCommentStart = pos;
                            }
                            lastSingleLineCommentEnd = end;
                            singleLineCommentCount++;
                            break;
                        case 3 /* MultiLineCommentTrivia */:
                            combineAndAddMultipleSingleLineComments();
                            out.push(createOutliningSpanFromBounds(pos, end, "comment" /* Comment */));
                            singleLineCommentCount = 0;
                            break;
                        default:
                            ts.Debug.assertNever(kind);
                    }
                }
                combineAndAddMultipleSingleLineComments();
                function combineAndAddMultipleSingleLineComments() {
                    // Only outline spans of two or more consecutive single line comments
                    if (singleLineCommentCount > 1) {
                        out.push(createOutliningSpanFromBounds(firstSingleLineCommentStart, lastSingleLineCommentEnd, "comment" /* Comment */));
                    }
                }
            }
            function createOutliningSpanFromBounds(pos, end, kind) {
                return createOutliningSpan(ts.createTextSpanFromBounds(pos, end), kind);
            }
            function getOutliningSpanForNode(n, sourceFile) {
                switch (n.kind) {
                    case 223 /* Block */:
                        if (ts.isFunctionLike(n.parent)) {
                            return functionSpan(n.parent, n, sourceFile);
                        }
                        // Check if the block is standalone, or 'attached' to some parent statement.
                        // If the latter, we want to collapse the block, but consider its hint span
                        // to be the entire span of the parent.
                        switch (n.parent.kind) {
                            case 228 /* DoStatement */:
                            case 231 /* ForInStatement */:
                            case 232 /* ForOfStatement */:
                            case 230 /* ForStatement */:
                            case 227 /* IfStatement */:
                            case 229 /* WhileStatement */:
                            case 236 /* WithStatement */:
                            case 280 /* CatchClause */:
                                return spanForNode(n.parent);
                            case 240 /* TryStatement */:
                                // Could be the try-block, or the finally-block.
                                var tryStatement = n.parent;
                                if (tryStatement.tryBlock === n) {
                                    return spanForNode(n.parent);
                                }
                                else if (tryStatement.finallyBlock === n) {
                                    var node = ts.findChildOfKind(tryStatement, 92 /* FinallyKeyword */, sourceFile);
                                    if (node)
                                        return spanForNode(node);
                                }
                            // falls through
                            default:
                                // Block was a standalone block.  In this case we want to only collapse
                                // the span of the block, independent of any parent span.
                                return createOutliningSpan(ts.createTextSpanFromNode(n, sourceFile), "code" /* Code */);
                        }
                    case 250 /* ModuleBlock */:
                        return spanForNode(n.parent);
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                    case 246 /* InterfaceDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 251 /* CaseBlock */:
                        return spanForNode(n);
                    case 193 /* ObjectLiteralExpression */:
                        return spanForObjectOrArrayLiteral(n);
                    case 192 /* ArrayLiteralExpression */:
                        return spanForObjectOrArrayLiteral(n, 22 /* OpenBracketToken */);
                    case 266 /* JsxElement */:
                        return spanForJSXElement(n);
                    case 270 /* JsxFragment */:
                        return spanForJSXFragment(n);
                    case 267 /* JsxSelfClosingElement */:
                    case 268 /* JsxOpeningElement */:
                        return spanForJSXAttributes(n.attributes);
                    case 211 /* TemplateExpression */:
                    case 14 /* NoSubstitutionTemplateLiteral */:
                        return spanForTemplateLiteral(n);
                }
                function spanForJSXElement(node) {
                    var textSpan = ts.createTextSpanFromBounds(node.openingElement.getStart(sourceFile), node.closingElement.getEnd());
                    var tagName = node.openingElement.tagName.getText(sourceFile);
                    var bannerText = "<" + tagName + ">...</" + tagName + ">";
                    return createOutliningSpan(textSpan, "code" /* Code */, textSpan, /*autoCollapse*/ false, bannerText);
                }
                function spanForJSXFragment(node) {
                    var textSpan = ts.createTextSpanFromBounds(node.openingFragment.getStart(sourceFile), node.closingFragment.getEnd());
                    var bannerText = "<>...</>";
                    return createOutliningSpan(textSpan, "code" /* Code */, textSpan, /*autoCollapse*/ false, bannerText);
                }
                function spanForJSXAttributes(node) {
                    if (node.properties.length === 0) {
                        return undefined;
                    }
                    return createOutliningSpanFromBounds(node.getStart(sourceFile), node.getEnd(), "code" /* Code */);
                }
                function spanForTemplateLiteral(node) {
                    if (node.kind === 14 /* NoSubstitutionTemplateLiteral */ && node.text.length === 0) {
                        return undefined;
                    }
                    return createOutliningSpanFromBounds(node.getStart(sourceFile), node.getEnd(), "code" /* Code */);
                }
                function spanForObjectOrArrayLiteral(node, open) {
                    if (open === void 0) {
                        open = 18 /* OpenBraceToken */;
                    }
                    // If the block has no leading keywords and is inside an array literal or call expression,
                    // we only want to collapse the span of the block.
                    // Otherwise, the collapsed section will include the end of the previous line.
                    return spanForNode(node, /*autoCollapse*/ false, /*useFullStart*/ !ts.isArrayLiteralExpression(node.parent) && !ts.isCallExpression(node.parent), open);
                }
                function spanForNode(hintSpanNode, autoCollapse, useFullStart, open, close) {
                    if (autoCollapse === void 0) {
                        autoCollapse = false;
                    }
                    if (useFullStart === void 0) {
                        useFullStart = true;
                    }
                    if (open === void 0) {
                        open = 18 /* OpenBraceToken */;
                    }
                    if (close === void 0) {
                        close = open === 18 /* OpenBraceToken */ ? 19 /* CloseBraceToken */ : 23 /* CloseBracketToken */;
                    }
                    var openToken = ts.findChildOfKind(n, open, sourceFile);
                    var closeToken = ts.findChildOfKind(n, close, sourceFile);
                    return openToken && closeToken && spanBetweenTokens(openToken, closeToken, hintSpanNode, sourceFile, autoCollapse, useFullStart);
                }
            }
            function functionSpan(node, body, sourceFile) {
                var openToken = ts.isNodeArrayMultiLine(node.parameters, sourceFile)
                    ? ts.findChildOfKind(node, 20 /* OpenParenToken */, sourceFile)
                    : ts.findChildOfKind(body, 18 /* OpenBraceToken */, sourceFile);
                var closeToken = ts.findChildOfKind(body, 19 /* CloseBraceToken */, sourceFile);
                return openToken && closeToken && spanBetweenTokens(openToken, closeToken, node, sourceFile, /*autoCollapse*/ node.kind !== 202 /* ArrowFunction */);
            }
            function spanBetweenTokens(openToken, closeToken, hintSpanNode, sourceFile, autoCollapse, useFullStart) {
                if (autoCollapse === void 0) {
                    autoCollapse = false;
                }
                if (useFullStart === void 0) {
                    useFullStart = true;
                }
                var textSpan = ts.createTextSpanFromBounds(useFullStart ? openToken.getFullStart() : openToken.getStart(sourceFile), closeToken.getEnd());
                return createOutliningSpan(textSpan, "code" /* Code */, ts.createTextSpanFromNode(hintSpanNode, sourceFile), autoCollapse);
            }
            function createOutliningSpan(textSpan, kind, hintSpan, autoCollapse, bannerText) {
                if (hintSpan === void 0) {
                    hintSpan = textSpan;
                }
                if (autoCollapse === void 0) {
                    autoCollapse = false;
                }
                if (bannerText === void 0) {
                    bannerText = "...";
                }
                return { textSpan: textSpan, kind: kind, hintSpan: hintSpan, bannerText: bannerText, autoCollapse: autoCollapse };
            }
        })(OutliningElementsCollector = ts.OutliningElementsCollector || (ts.OutliningElementsCollector = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        // Note(cyrusn): this enum is ordered from strongest match type to weakest match type.
        var PatternMatchKind;
        (function (PatternMatchKind) {
            PatternMatchKind[PatternMatchKind["exact"] = 0] = "exact";
            PatternMatchKind[PatternMatchKind["prefix"] = 1] = "prefix";
            PatternMatchKind[PatternMatchKind["substring"] = 2] = "substring";
            PatternMatchKind[PatternMatchKind["camelCase"] = 3] = "camelCase";
        })(PatternMatchKind = ts.PatternMatchKind || (ts.PatternMatchKind = {}));
        function createPatternMatch(kind, isCaseSensitive) {
            return {
                kind: kind,
                isCaseSensitive: isCaseSensitive
            };
        }
        function createPatternMatcher(pattern) {
            // We'll often see the same candidate string many times when searching (For example, when
            // we see the name of a module that is used everywhere, or the name of an overload).  As
            // such, we cache the information we compute about the candidate for the life of this
            // pattern matcher so we don't have to compute it multiple times.
            var stringToWordSpans = ts.createMap();
            var dotSeparatedSegments = pattern.trim().split(".").map(function (p) { return createSegment(p.trim()); });
            // A segment is considered invalid if we couldn't find any words in it.
            if (dotSeparatedSegments.some(function (segment) { return !segment.subWordTextChunks.length; }))
                return undefined;
            return {
                getFullMatch: function (containers, candidate) { return getFullMatch(containers, candidate, dotSeparatedSegments, stringToWordSpans); },
                getMatchForLastSegmentOfPattern: function (candidate) { return matchSegment(candidate, ts.last(dotSeparatedSegments), stringToWordSpans); },
                patternContainsDots: dotSeparatedSegments.length > 1
            };
        }
        ts.createPatternMatcher = createPatternMatcher;
        function getFullMatch(candidateContainers, candidate, dotSeparatedSegments, stringToWordSpans) {
            // First, check that the last part of the dot separated pattern matches the name of the
            // candidate.  If not, then there's no point in proceeding and doing the more
            // expensive work.
            var candidateMatch = matchSegment(candidate, ts.last(dotSeparatedSegments), stringToWordSpans);
            if (!candidateMatch) {
                return undefined;
            }
            // -1 because the last part was checked against the name, and only the rest
            // of the parts are checked against the container.
            if (dotSeparatedSegments.length - 1 > candidateContainers.length) {
                // There weren't enough container parts to match against the pattern parts.
                // So this definitely doesn't match.
                return undefined;
            }
            var bestMatch;
            for (var i = dotSeparatedSegments.length - 2, j = candidateContainers.length - 1; i >= 0; i -= 1, j -= 1) {
                bestMatch = betterMatch(bestMatch, matchSegment(candidateContainers[j], dotSeparatedSegments[i], stringToWordSpans));
            }
            return bestMatch;
        }
        function getWordSpans(word, stringToWordSpans) {
            var spans = stringToWordSpans.get(word);
            if (!spans) {
                stringToWordSpans.set(word, spans = breakIntoWordSpans(word));
            }
            return spans;
        }
        function matchTextChunk(candidate, chunk, stringToWordSpans) {
            var index = indexOfIgnoringCase(candidate, chunk.textLowerCase);
            if (index === 0) {
                // a) Check if the word is a prefix of the candidate, in a case insensitive or
                //    sensitive manner. If it does, return that there was an exact match if the word and candidate are the same length, else a prefix match.
                return createPatternMatch(chunk.text.length === candidate.length ? PatternMatchKind.exact : PatternMatchKind.prefix, /*isCaseSensitive:*/ ts.startsWith(candidate, chunk.text));
            }
            if (chunk.isLowerCase) {
                if (index === -1)
                    return undefined;
                // b) If the part is entirely lowercase, then check if it is contained anywhere in the
                //    candidate in a case insensitive manner.  If so, return that there was a substring
                //    match.
                //
                //    Note: We only have a substring match if the lowercase part is prefix match of some
                //    word part. That way we don't match something like 'Class' when the user types 'a'.
                //    But we would match 'FooAttribute' (since 'Attribute' starts with 'a').
                var wordSpans = getWordSpans(candidate, stringToWordSpans);
                for (var _i = 0, wordSpans_1 = wordSpans; _i < wordSpans_1.length; _i++) {
                    var span = wordSpans_1[_i];
                    if (partStartsWith(candidate, span, chunk.text, /*ignoreCase:*/ true)) {
                        return createPatternMatch(PatternMatchKind.substring, /*isCaseSensitive:*/ partStartsWith(candidate, span, chunk.text, /*ignoreCase:*/ false));
                    }
                }
                // c) Is the pattern a substring of the candidate starting on one of the candidate's word boundaries?
                // We could check every character boundary start of the candidate for the pattern. However, that's
                // an m * n operation in the wost case. Instead, find the first instance of the pattern
                // substring, and see if it starts on a capital letter. It seems unlikely that the user will try to
                // filter the list based on a substring that starts on a capital letter and also with a lowercase one.
                // (Pattern: fogbar, Candidate: quuxfogbarFogBar).
                if (chunk.text.length < candidate.length && isUpperCaseLetter(candidate.charCodeAt(index))) {
                    return createPatternMatch(PatternMatchKind.substring, /*isCaseSensitive:*/ false);
                }
            }
            else {
                // d) If the part was not entirely lowercase, then check if it is contained in the
                //    candidate in a case *sensitive* manner. If so, return that there was a substring
                //    match.
                if (candidate.indexOf(chunk.text) > 0) {
                    return createPatternMatch(PatternMatchKind.substring, /*isCaseSensitive:*/ true);
                }
                // e) If the part was not entirely lowercase, then attempt a camel cased match as well.
                if (chunk.characterSpans.length > 0) {
                    var candidateParts = getWordSpans(candidate, stringToWordSpans);
                    var isCaseSensitive = tryCamelCaseMatch(candidate, candidateParts, chunk, /*ignoreCase:*/ false) ? true
                        : tryCamelCaseMatch(candidate, candidateParts, chunk, /*ignoreCase:*/ true) ? false : undefined;
                    if (isCaseSensitive !== undefined) {
                        return createPatternMatch(PatternMatchKind.camelCase, isCaseSensitive);
                    }
                }
            }
        }
        function matchSegment(candidate, segment, stringToWordSpans) {
            // First check if the segment matches as is.  This is also useful if the segment contains
            // characters we would normally strip when splitting into parts that we also may want to
            // match in the candidate.  For example if the segment is "@int" and the candidate is
            // "@int", then that will show up as an exact match here.
            //
            // Note: if the segment contains a space or an asterisk then we must assume that it's a
            // multi-word segment.
            if (every(segment.totalTextChunk.text, function (ch) { return ch !== 32 /* space */ && ch !== 42 /* asterisk */; })) {
                var match = matchTextChunk(candidate, segment.totalTextChunk, stringToWordSpans);
                if (match)
                    return match;
            }
            // The logic for pattern matching is now as follows:
            //
            // 1) Break the segment passed in into words.  Breaking is rather simple and a
            //    good way to think about it that if gives you all the individual alphanumeric words
            //    of the pattern.
            //
            // 2) For each word try to match the word against the candidate value.
            //
            // 3) Matching is as follows:
            //
            //   a) Check if the word is a prefix of the candidate, in a case insensitive or
            //      sensitive manner. If it does, return that there was an exact match if the word and candidate are the same length, else a prefix match.
            //
            //   If the word is entirely lowercase:
            //      b) Then check if it is contained anywhere in the
            //          candidate in a case insensitive manner.  If so, return that there was a substring
            //          match.
            //
            //          Note: We only have a substring match if the lowercase part is prefix match of
            //          some word part. That way we don't match something like 'Class' when the user
            //          types 'a'. But we would match 'FooAttribute' (since 'Attribute' starts with
            //          'a').
            //
            //       c) The word is all lower case. Is it a case insensitive substring of the candidate starting
            //          on a part boundary of the candidate?
            //
            //   Else:
            //       d) If the word was not entirely lowercase, then check if it is contained in the
            //          candidate in a case *sensitive* manner. If so, return that there was a substring
            //          match.
            //
            //       e) If the word was not entirely lowercase, then attempt a camel cased match as
            //          well.
            //
            // Only if all words have some sort of match is the pattern considered matched.
            var subWordTextChunks = segment.subWordTextChunks;
            var bestMatch;
            for (var _i = 0, subWordTextChunks_1 = subWordTextChunks; _i < subWordTextChunks_1.length; _i++) {
                var subWordTextChunk = subWordTextChunks_1[_i];
                bestMatch = betterMatch(bestMatch, matchTextChunk(candidate, subWordTextChunk, stringToWordSpans));
            }
            return bestMatch;
        }
        function betterMatch(a, b) {
            return ts.min(a, b, compareMatches);
        }
        function compareMatches(a, b) {
            return a === undefined ? 1 /* GreaterThan */ : b === undefined ? -1 /* LessThan */
                : ts.compareValues(a.kind, b.kind) || ts.compareBooleans(!a.isCaseSensitive, !b.isCaseSensitive);
        }
        function partStartsWith(candidate, candidateSpan, pattern, ignoreCase, patternSpan) {
            if (patternSpan === void 0) {
                patternSpan = { start: 0, length: pattern.length };
            }
            return patternSpan.length <= candidateSpan.length // If pattern part is longer than the candidate part there can never be a match.
                && everyInRange(0, patternSpan.length, function (i) { return equalChars(pattern.charCodeAt(patternSpan.start + i), candidate.charCodeAt(candidateSpan.start + i), ignoreCase); });
        }
        function equalChars(ch1, ch2, ignoreCase) {
            return ignoreCase ? toLowerCase(ch1) === toLowerCase(ch2) : ch1 === ch2;
        }
        function tryCamelCaseMatch(candidate, candidateParts, chunk, ignoreCase) {
            var chunkCharacterSpans = chunk.characterSpans;
            // Note: we may have more pattern parts than candidate parts.  This is because multiple
            // pattern parts may match a candidate part.  For example "SiUI" against "SimpleUI".
            // We'll have 3 pattern parts Si/U/I against two candidate parts Simple/UI.  However, U
            // and I will both match in UI.
            var currentCandidate = 0;
            var currentChunkSpan = 0;
            var firstMatch;
            var contiguous;
            while (true) {
                // Let's consider our termination cases
                if (currentChunkSpan === chunkCharacterSpans.length) {
                    return true;
                }
                else if (currentCandidate === candidateParts.length) {
                    // No match, since we still have more of the pattern to hit
                    return false;
                }
                var candidatePart = candidateParts[currentCandidate];
                var gotOneMatchThisCandidate = false;
                // Consider the case of matching SiUI against SimpleUIElement. The candidate parts
                // will be Simple/UI/Element, and the pattern parts will be Si/U/I.  We'll match 'Si'
                // against 'Simple' first.  Then we'll match 'U' against 'UI'. However, we want to
                // still keep matching pattern parts against that candidate part.
                for (; currentChunkSpan < chunkCharacterSpans.length; currentChunkSpan++) {
                    var chunkCharacterSpan = chunkCharacterSpans[currentChunkSpan];
                    if (gotOneMatchThisCandidate) {
                        // We've already gotten one pattern part match in this candidate.  We will
                        // only continue trying to consumer pattern parts if the last part and this
                        // part are both upper case.
                        if (!isUpperCaseLetter(chunk.text.charCodeAt(chunkCharacterSpans[currentChunkSpan - 1].start)) ||
                            !isUpperCaseLetter(chunk.text.charCodeAt(chunkCharacterSpans[currentChunkSpan].start))) {
                            break;
                        }
                    }
                    if (!partStartsWith(candidate, candidatePart, chunk.text, ignoreCase, chunkCharacterSpan)) {
                        break;
                    }
                    gotOneMatchThisCandidate = true;
                    firstMatch = firstMatch === undefined ? currentCandidate : firstMatch;
                    // If we were contiguous, then keep that value.  If we weren't, then keep that
                    // value.  If we don't know, then set the value to 'true' as an initial match is
                    // obviously contiguous.
                    contiguous = contiguous === undefined ? true : contiguous;
                    candidatePart = ts.createTextSpan(candidatePart.start + chunkCharacterSpan.length, candidatePart.length - chunkCharacterSpan.length);
                }
                // Check if we matched anything at all.  If we didn't, then we need to unset the
                // contiguous bit if we currently had it set.
                // If we haven't set the bit yet, then that means we haven't matched anything so
                // far, and we don't want to change that.
                if (!gotOneMatchThisCandidate && contiguous !== undefined) {
                    contiguous = false;
                }
                // Move onto the next candidate.
                currentCandidate++;
            }
        }
        function createSegment(text) {
            return {
                totalTextChunk: createTextChunk(text),
                subWordTextChunks: breakPatternIntoTextChunks(text)
            };
        }
        function isUpperCaseLetter(ch) {
            // Fast check for the ascii range.
            if (ch >= 65 /* A */ && ch <= 90 /* Z */) {
                return true;
            }
            if (ch < 127 /* maxAsciiCharacter */ || !ts.isUnicodeIdentifierStart(ch, 99 /* Latest */)) {
                return false;
            }
            // TODO: find a way to determine this for any unicode characters in a
            // non-allocating manner.
            var str = String.fromCharCode(ch);
            return str === str.toUpperCase();
        }
        function isLowerCaseLetter(ch) {
            // Fast check for the ascii range.
            if (ch >= 97 /* a */ && ch <= 122 /* z */) {
                return true;
            }
            if (ch < 127 /* maxAsciiCharacter */ || !ts.isUnicodeIdentifierStart(ch, 99 /* Latest */)) {
                return false;
            }
            // TODO: find a way to determine this for any unicode characters in a
            // non-allocating manner.
            var str = String.fromCharCode(ch);
            return str === str.toLowerCase();
        }
        // Assumes 'value' is already lowercase.
        function indexOfIgnoringCase(str, value) {
            var n = str.length - value.length;
            var _loop_6 = function (start) {
                if (every(value, function (valueChar, i) { return toLowerCase(str.charCodeAt(i + start)) === valueChar; })) {
                    return { value: start };
                }
            };
            for (var start = 0; start <= n; start++) {
                var state_1 = _loop_6(start);
                if (typeof state_1 === "object")
                    return state_1.value;
            }
            return -1;
        }
        function toLowerCase(ch) {
            // Fast convert for the ascii range.
            if (ch >= 65 /* A */ && ch <= 90 /* Z */) {
                return 97 /* a */ + (ch - 65 /* A */);
            }
            if (ch < 127 /* maxAsciiCharacter */) {
                return ch;
            }
            // TODO: find a way to compute this for any unicode characters in a
            // non-allocating manner.
            return String.fromCharCode(ch).toLowerCase().charCodeAt(0);
        }
        function isDigit(ch) {
            // TODO(cyrusn): Find a way to support this for unicode digits.
            return ch >= 48 /* _0 */ && ch <= 57 /* _9 */;
        }
        function isWordChar(ch) {
            return isUpperCaseLetter(ch) || isLowerCaseLetter(ch) || isDigit(ch) || ch === 95 /* _ */ || ch === 36 /* $ */;
        }
        function breakPatternIntoTextChunks(pattern) {
            var result = [];
            var wordStart = 0;
            var wordLength = 0;
            for (var i = 0; i < pattern.length; i++) {
                var ch = pattern.charCodeAt(i);
                if (isWordChar(ch)) {
                    if (wordLength === 0) {
                        wordStart = i;
                    }
                    wordLength++;
                }
                else {
                    if (wordLength > 0) {
                        result.push(createTextChunk(pattern.substr(wordStart, wordLength)));
                        wordLength = 0;
                    }
                }
            }
            if (wordLength > 0) {
                result.push(createTextChunk(pattern.substr(wordStart, wordLength)));
            }
            return result;
        }
        function createTextChunk(text) {
            var textLowerCase = text.toLowerCase();
            return {
                text: text,
                textLowerCase: textLowerCase,
                isLowerCase: text === textLowerCase,
                characterSpans: breakIntoCharacterSpans(text)
            };
        }
        function breakIntoCharacterSpans(identifier) {
            return breakIntoSpans(identifier, /*word:*/ false);
        }
        ts.breakIntoCharacterSpans = breakIntoCharacterSpans;
        function breakIntoWordSpans(identifier) {
            return breakIntoSpans(identifier, /*word:*/ true);
        }
        ts.breakIntoWordSpans = breakIntoWordSpans;
        function breakIntoSpans(identifier, word) {
            var result = [];
            var wordStart = 0;
            for (var i = 1; i < identifier.length; i++) {
                var lastIsDigit = isDigit(identifier.charCodeAt(i - 1));
                var currentIsDigit = isDigit(identifier.charCodeAt(i));
                var hasTransitionFromLowerToUpper = transitionFromLowerToUpper(identifier, word, i);
                var hasTransitionFromUpperToLower = word && transitionFromUpperToLower(identifier, i, wordStart);
                if (charIsPunctuation(identifier.charCodeAt(i - 1)) ||
                    charIsPunctuation(identifier.charCodeAt(i)) ||
                    lastIsDigit !== currentIsDigit ||
                    hasTransitionFromLowerToUpper ||
                    hasTransitionFromUpperToLower) {
                    if (!isAllPunctuation(identifier, wordStart, i)) {
                        result.push(ts.createTextSpan(wordStart, i - wordStart));
                    }
                    wordStart = i;
                }
            }
            if (!isAllPunctuation(identifier, wordStart, identifier.length)) {
                result.push(ts.createTextSpan(wordStart, identifier.length - wordStart));
            }
            return result;
        }
        function charIsPunctuation(ch) {
            switch (ch) {
                case 33 /* exclamation */:
                case 34 /* doubleQuote */:
                case 35 /* hash */:
                case 37 /* percent */:
                case 38 /* ampersand */:
                case 39 /* singleQuote */:
                case 40 /* openParen */:
                case 41 /* closeParen */:
                case 42 /* asterisk */:
                case 44 /* comma */:
                case 45 /* minus */:
                case 46 /* dot */:
                case 47 /* slash */:
                case 58 /* colon */:
                case 59 /* semicolon */:
                case 63 /* question */:
                case 64 /* at */:
                case 91 /* openBracket */:
                case 92 /* backslash */:
                case 93 /* closeBracket */:
                case 95 /* _ */:
                case 123 /* openBrace */:
                case 125 /* closeBrace */:
                    return true;
            }
            return false;
        }
        function isAllPunctuation(identifier, start, end) {
            return every(identifier, function (ch) { return charIsPunctuation(ch) && ch !== 95 /* _ */; }, start, end);
        }
        function transitionFromUpperToLower(identifier, index, wordStart) {
            // Cases this supports:
            // 1) IDisposable -> I, Disposable
            // 2) UIElement -> UI, Element
            // 3) HTMLDocument -> HTML, Document
            //
            // etc.
            // We have a transition from an upper to a lower letter here.  But we only
            // want to break if all the letters that preceded are uppercase.  i.e. if we
            // have "Foo" we don't want to break that into "F, oo".  But if we have
            // "IFoo" or "UIFoo", then we want to break that into "I, Foo" and "UI,
            // Foo".  i.e. the last uppercase letter belongs to the lowercase letters
            // that follows.  Note: this will make the following not split properly:
            // "HELLOthere".  However, these sorts of names do not show up in .Net
            // programs.
            return index !== wordStart
                && index + 1 < identifier.length
                && isUpperCaseLetter(identifier.charCodeAt(index))
                && isLowerCaseLetter(identifier.charCodeAt(index + 1))
                && every(identifier, isUpperCaseLetter, wordStart, index);
        }
        function transitionFromLowerToUpper(identifier, word, index) {
            var lastIsUpper = isUpperCaseLetter(identifier.charCodeAt(index - 1));
            var currentIsUpper = isUpperCaseLetter(identifier.charCodeAt(index));
            // See if the casing indicates we're starting a new word. Note: if we're breaking on
            // words, then just seeing an upper case character isn't enough.  Instead, it has to
            // be uppercase and the previous character can't be uppercase.
            //
            // For example, breaking "AddMetadata" on words would make: Add Metadata
            //
            // on characters would be: A dd M etadata
            //
            // Break "AM" on words would be: AM
            //
            // on characters would be: A M
            //
            // We break the search string on characters.  But we break the symbol name on words.
            return currentIsUpper && (!word || !lastIsUpper);
        }
        function everyInRange(start, end, pred) {
            for (var i = start; i < end; i++) {
                if (!pred(i)) {
                    return false;
                }
            }
            return true;
        }
        function every(s, pred, start, end) {
            if (start === void 0) {
                start = 0;
            }
            if (end === void 0) {
                end = s.length;
            }
            return everyInRange(start, end, function (i) { return pred(s.charCodeAt(i), i); });
        }
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        function preProcessFile(sourceText, readImportFiles, detectJavaScriptImports) {
            if (readImportFiles === void 0) {
                readImportFiles = true;
            }
            if (detectJavaScriptImports === void 0) {
                detectJavaScriptImports = false;
            }
            var pragmaContext = {
                languageVersion: 1 /* ES5 */,
                pragmas: undefined,
                checkJsDirective: undefined,
                referencedFiles: [],
                typeReferenceDirectives: [],
                libReferenceDirectives: [],
                amdDependencies: [],
                hasNoDefaultLib: undefined,
                moduleName: undefined
            };
            var importedFiles = [];
            var ambientExternalModules;
            var lastToken;
            var currentToken;
            var braceNesting = 0;
            // assume that text represent an external module if it contains at least one top level import/export
            // ambient modules that are found inside external modules are interpreted as module augmentations
            var externalModule = false;
            function nextToken() {
                lastToken = currentToken;
                currentToken = ts.scanner.scan();
                if (currentToken === 18 /* OpenBraceToken */) {
                    braceNesting++;
                }
                else if (currentToken === 19 /* CloseBraceToken */) {
                    braceNesting--;
                }
                return currentToken;
            }
            function getFileReference() {
                var fileName = ts.scanner.getTokenValue();
                var pos = ts.scanner.getTokenPos();
                return { fileName: fileName, pos: pos, end: pos + fileName.length };
            }
            function recordAmbientExternalModule() {
                if (!ambientExternalModules) {
                    ambientExternalModules = [];
                }
                ambientExternalModules.push({ ref: getFileReference(), depth: braceNesting });
            }
            function recordModuleName() {
                importedFiles.push(getFileReference());
                markAsExternalModuleIfTopLevel();
            }
            function markAsExternalModuleIfTopLevel() {
                if (braceNesting === 0) {
                    externalModule = true;
                }
            }
            /**
             * Returns true if at least one token was consumed from the stream
             */
            function tryConsumeDeclare() {
                var token = ts.scanner.getToken();
                if (token === 130 /* DeclareKeyword */) {
                    // declare module "mod"
                    token = nextToken();
                    if (token === 135 /* ModuleKeyword */) {
                        token = nextToken();
                        if (token === 10 /* StringLiteral */) {
                            recordAmbientExternalModule();
                        }
                    }
                    return true;
                }
                return false;
            }
            /**
             * Returns true if at least one token was consumed from the stream
             */
            function tryConsumeImport() {
                if (lastToken === 24 /* DotToken */) {
                    return false;
                }
                var token = ts.scanner.getToken();
                if (token === 96 /* ImportKeyword */) {
                    token = nextToken();
                    if (token === 20 /* OpenParenToken */) {
                        token = nextToken();
                        if (token === 10 /* StringLiteral */) {
                            // import("mod");
                            recordModuleName();
                            return true;
                        }
                    }
                    else if (token === 10 /* StringLiteral */) {
                        // import "mod";
                        recordModuleName();
                        return true;
                    }
                    else {
                        if (token === 75 /* Identifier */ || ts.isKeyword(token)) {
                            token = nextToken();
                            if (token === 149 /* FromKeyword */) {
                                token = nextToken();
                                if (token === 10 /* StringLiteral */) {
                                    // import d from "mod";
                                    recordModuleName();
                                    return true;
                                }
                            }
                            else if (token === 62 /* EqualsToken */) {
                                if (tryConsumeRequireCall(/*skipCurrentToken*/ true)) {
                                    return true;
                                }
                            }
                            else if (token === 27 /* CommaToken */) {
                                // consume comma and keep going
                                token = nextToken();
                            }
                            else {
                                // unknown syntax
                                return true;
                            }
                        }
                        if (token === 18 /* OpenBraceToken */) {
                            token = nextToken();
                            // consume "{ a as B, c, d as D}" clauses
                            // make sure that it stops on EOF
                            while (token !== 19 /* CloseBraceToken */ && token !== 1 /* EndOfFileToken */) {
                                token = nextToken();
                            }
                            if (token === 19 /* CloseBraceToken */) {
                                token = nextToken();
                                if (token === 149 /* FromKeyword */) {
                                    token = nextToken();
                                    if (token === 10 /* StringLiteral */) {
                                        // import {a as A} from "mod";
                                        // import d, {a, b as B} from "mod"
                                        recordModuleName();
                                    }
                                }
                            }
                        }
                        else if (token === 41 /* AsteriskToken */) {
                            token = nextToken();
                            if (token === 123 /* AsKeyword */) {
                                token = nextToken();
                                if (token === 75 /* Identifier */ || ts.isKeyword(token)) {
                                    token = nextToken();
                                    if (token === 149 /* FromKeyword */) {
                                        token = nextToken();
                                        if (token === 10 /* StringLiteral */) {
                                            // import * as NS from "mod"
                                            // import d, * as NS from "mod"
                                            recordModuleName();
                                        }
                                    }
                                }
                            }
                        }
                    }
                    return true;
                }
                return false;
            }
            function tryConsumeExport() {
                var token = ts.scanner.getToken();
                if (token === 89 /* ExportKeyword */) {
                    markAsExternalModuleIfTopLevel();
                    token = nextToken();
                    if (token === 18 /* OpenBraceToken */) {
                        token = nextToken();
                        // consume "{ a as B, c, d as D}" clauses
                        // make sure it stops on EOF
                        while (token !== 19 /* CloseBraceToken */ && token !== 1 /* EndOfFileToken */) {
                            token = nextToken();
                        }
                        if (token === 19 /* CloseBraceToken */) {
                            token = nextToken();
                            if (token === 149 /* FromKeyword */) {
                                token = nextToken();
                                if (token === 10 /* StringLiteral */) {
                                    // export {a as A} from "mod";
                                    // export {a, b as B} from "mod"
                                    recordModuleName();
                                }
                            }
                        }
                    }
                    else if (token === 41 /* AsteriskToken */) {
                        token = nextToken();
                        if (token === 149 /* FromKeyword */) {
                            token = nextToken();
                            if (token === 10 /* StringLiteral */) {
                                // export * from "mod"
                                recordModuleName();
                            }
                        }
                    }
                    else if (token === 96 /* ImportKeyword */) {
                        token = nextToken();
                        if (token === 75 /* Identifier */ || ts.isKeyword(token)) {
                            token = nextToken();
                            if (token === 62 /* EqualsToken */) {
                                if (tryConsumeRequireCall(/*skipCurrentToken*/ true)) {
                                    return true;
                                }
                            }
                        }
                    }
                    return true;
                }
                return false;
            }
            function tryConsumeRequireCall(skipCurrentToken) {
                var token = skipCurrentToken ? nextToken() : ts.scanner.getToken();
                if (token === 139 /* RequireKeyword */) {
                    token = nextToken();
                    if (token === 20 /* OpenParenToken */) {
                        token = nextToken();
                        if (token === 10 /* StringLiteral */) {
                            //  require("mod");
                            recordModuleName();
                        }
                    }
                    return true;
                }
                return false;
            }
            function tryConsumeDefine() {
                var token = ts.scanner.getToken();
                if (token === 75 /* Identifier */ && ts.scanner.getTokenValue() === "define") {
                    token = nextToken();
                    if (token !== 20 /* OpenParenToken */) {
                        return true;
                    }
                    token = nextToken();
                    if (token === 10 /* StringLiteral */) {
                        // looks like define ("modname", ... - skip string literal and comma
                        token = nextToken();
                        if (token === 27 /* CommaToken */) {
                            token = nextToken();
                        }
                        else {
                            // unexpected token
                            return true;
                        }
                    }
                    // should be start of dependency list
                    if (token !== 22 /* OpenBracketToken */) {
                        return true;
                    }
                    // skip open bracket
                    token = nextToken();
                    // scan until ']' or EOF
                    while (token !== 23 /* CloseBracketToken */ && token !== 1 /* EndOfFileToken */) {
                        // record string literals as module names
                        if (token === 10 /* StringLiteral */) {
                            recordModuleName();
                        }
                        token = nextToken();
                    }
                    return true;
                }
                return false;
            }
            function processImports() {
                ts.scanner.setText(sourceText);
                nextToken();
                // Look for:
                //    import "mod";
                //    import d from "mod"
                //    import {a as A } from "mod";
                //    import * as NS from "mod"
                //    import d, {a, b as B} from "mod"
                //    import i = require("mod");
                //    import("mod");
                //    export * from "mod"
                //    export {a as b} from "mod"
                //    export import i = require("mod")
                //    (for JavaScript files) require("mod")
                // Do not look for:
                //    AnySymbol.import("mod")
                //    AnySymbol.nested.import("mod")
                while (true) {
                    if (ts.scanner.getToken() === 1 /* EndOfFileToken */) {
                        break;
                    }
                    // check if at least one of alternative have moved scanner forward
                    if (tryConsumeDeclare() ||
                        tryConsumeImport() ||
                        tryConsumeExport() ||
                        (detectJavaScriptImports && (tryConsumeRequireCall(/*skipCurrentToken*/ false) || tryConsumeDefine()))) {
                        continue;
                    }
                    else {
                        nextToken();
                    }
                }
                ts.scanner.setText(undefined);
            }
            if (readImportFiles) {
                processImports();
            }
            ts.processCommentPragmas(pragmaContext, sourceText);
            ts.processPragmasIntoFields(pragmaContext, ts.noop);
            if (externalModule) {
                // for external modules module all nested ambient modules are augmentations
                if (ambientExternalModules) {
                    // move all detected ambient modules to imported files since they need to be resolved
                    for (var _i = 0, ambientExternalModules_1 = ambientExternalModules; _i < ambientExternalModules_1.length; _i++) {
                        var decl = ambientExternalModules_1[_i];
                        importedFiles.push(decl.ref);
                    }
                }
                return { referencedFiles: pragmaContext.referencedFiles, typeReferenceDirectives: pragmaContext.typeReferenceDirectives, libReferenceDirectives: pragmaContext.libReferenceDirectives, importedFiles: importedFiles, isLibFile: !!pragmaContext.hasNoDefaultLib, ambientExternalModules: undefined };
            }
            else {
                // for global scripts ambient modules still can have augmentations - look for ambient modules with depth > 0
                var ambientModuleNames = void 0;
                if (ambientExternalModules) {
                    for (var _a = 0, ambientExternalModules_2 = ambientExternalModules; _a < ambientExternalModules_2.length; _a++) {
                        var decl = ambientExternalModules_2[_a];
                        if (decl.depth === 0) {
                            if (!ambientModuleNames) {
                                ambientModuleNames = [];
                            }
                            ambientModuleNames.push(decl.ref.fileName);
                        }
                        else {
                            importedFiles.push(decl.ref);
                        }
                    }
                }
                return { referencedFiles: pragmaContext.referencedFiles, typeReferenceDirectives: pragmaContext.typeReferenceDirectives, libReferenceDirectives: pragmaContext.libReferenceDirectives, importedFiles: importedFiles, isLibFile: !!pragmaContext.hasNoDefaultLib, ambientExternalModules: ambientModuleNames };
            }
        }
        ts.preProcessFile = preProcessFile;
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var Rename;
        (function (Rename) {
            function getRenameInfo(program, sourceFile, position, options) {
                var node = ts.getAdjustedRenameLocation(ts.getTouchingPropertyName(sourceFile, position));
                if (nodeIsEligibleForRename(node)) {
                    var renameInfo = getRenameInfoForNode(node, program.getTypeChecker(), sourceFile, function (declaration) { return program.isSourceFileDefaultLibrary(declaration.getSourceFile()); }, options);
                    if (renameInfo) {
                        return renameInfo;
                    }
                }
                return getRenameInfoError(ts.Diagnostics.You_cannot_rename_this_element);
            }
            Rename.getRenameInfo = getRenameInfo;
            function getRenameInfoForNode(node, typeChecker, sourceFile, isDefinedInLibraryFile, options) {
                var symbol = typeChecker.getSymbolAtLocation(node);
                if (!symbol)
                    return;
                // Only allow a symbol to be renamed if it actually has at least one declaration.
                var declarations = symbol.declarations;
                if (!declarations || declarations.length === 0)
                    return;
                // Disallow rename for elements that are defined in the standard TypeScript library.
                if (declarations.some(isDefinedInLibraryFile)) {
                    return getRenameInfoError(ts.Diagnostics.You_cannot_rename_elements_that_are_defined_in_the_standard_TypeScript_library);
                }
                // Cannot rename `default` as in `import { default as foo } from "./someModule";
                if (ts.isIdentifier(node) && node.originalKeywordKind === 84 /* DefaultKeyword */ && symbol.parent.flags & 1536 /* Module */) {
                    return undefined;
                }
                if (ts.isStringLiteralLike(node) && ts.tryGetImportFromModuleSpecifier(node)) {
                    return options && options.allowRenameOfImportPath ? getRenameInfoForModule(node, sourceFile, symbol) : undefined;
                }
                var kind = ts.SymbolDisplay.getSymbolKind(typeChecker, symbol, node);
                var specifierName = (ts.isImportOrExportSpecifierName(node) || ts.isStringOrNumericLiteralLike(node) && node.parent.kind === 154 /* ComputedPropertyName */)
                    ? ts.stripQuotes(ts.getTextOfIdentifierOrLiteral(node))
                    : undefined;
                var displayName = specifierName || typeChecker.symbolToString(symbol);
                var fullDisplayName = specifierName || typeChecker.getFullyQualifiedName(symbol);
                return getRenameInfoSuccess(displayName, fullDisplayName, kind, ts.SymbolDisplay.getSymbolModifiers(symbol), node, sourceFile);
            }
            function getRenameInfoForModule(node, sourceFile, moduleSymbol) {
                if (!ts.isExternalModuleNameRelative(node.text)) {
                    return getRenameInfoError(ts.Diagnostics.You_cannot_rename_a_module_via_a_global_import);
                }
                var moduleSourceFile = ts.find(moduleSymbol.declarations, ts.isSourceFile);
                if (!moduleSourceFile)
                    return undefined;
                var withoutIndex = ts.endsWith(node.text, "/index") || ts.endsWith(node.text, "/index.js") ? undefined : ts.tryRemoveSuffix(ts.removeFileExtension(moduleSourceFile.fileName), "/index");
                var name = withoutIndex === undefined ? moduleSourceFile.fileName : withoutIndex;
                var kind = withoutIndex === undefined ? "module" /* moduleElement */ : "directory" /* directory */;
                var indexAfterLastSlash = node.text.lastIndexOf("/") + 1;
                // Span should only be the last component of the path. + 1 to account for the quote character.
                var triggerSpan = ts.createTextSpan(node.getStart(sourceFile) + 1 + indexAfterLastSlash, node.text.length - indexAfterLastSlash);
                return {
                    canRename: true,
                    fileToRename: name,
                    kind: kind,
                    displayName: name,
                    fullDisplayName: name,
                    kindModifiers: "" /* none */,
                    triggerSpan: triggerSpan,
                };
            }
            function getRenameInfoSuccess(displayName, fullDisplayName, kind, kindModifiers, node, sourceFile) {
                return {
                    canRename: true,
                    fileToRename: undefined,
                    kind: kind,
                    displayName: displayName,
                    fullDisplayName: fullDisplayName,
                    kindModifiers: kindModifiers,
                    triggerSpan: createTriggerSpanForNode(node, sourceFile)
                };
            }
            function getRenameInfoError(diagnostic) {
                return { canRename: false, localizedErrorMessage: ts.getLocaleSpecificMessage(diagnostic) };
            }
            function createTriggerSpanForNode(node, sourceFile) {
                var start = node.getStart(sourceFile);
                var width = node.getWidth(sourceFile);
                if (ts.isStringLiteralLike(node)) {
                    // Exclude the quotes
                    start += 1;
                    width -= 2;
                }
                return ts.createTextSpan(start, width);
            }
            function nodeIsEligibleForRename(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                    case 76 /* PrivateIdentifier */:
                    case 10 /* StringLiteral */:
                    case 14 /* NoSubstitutionTemplateLiteral */:
                    case 104 /* ThisKeyword */:
                        return true;
                    case 8 /* NumericLiteral */:
                        return ts.isLiteralNameOfPropertyDeclarationOrIndexAccess(node);
                    default:
                        return false;
                }
            }
        })(Rename = ts.Rename || (ts.Rename = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var SmartSelectionRange;
        (function (SmartSelectionRange) {
            function getSmartSelectionRange(pos, sourceFile) {
                var selectionRange = {
                    textSpan: ts.createTextSpanFromBounds(sourceFile.getFullStart(), sourceFile.getEnd())
                };
                var parentNode = sourceFile;
                outer: while (true) {
                    var children = getSelectionChildren(parentNode);
                    if (!children.length)
                        break;
                    for (var i = 0; i < children.length; i++) {
                        var prevNode = children[i - 1];
                        var node = children[i];
                        var nextNode = children[i + 1];
                        if (node.getStart(sourceFile) > pos) {
                            break outer;
                        }
                        if (positionShouldSnapToNode(sourceFile, pos, node)) {
                            // 1. Blocks are effectively redundant with SyntaxLists.
                            // 2. TemplateSpans, along with the SyntaxLists containing them, are a somewhat unintuitive grouping
                            //    of things that should be considered independently.
                            // 3. A VariableStatement’s children are just a VaraiableDeclarationList and a semicolon.
                            // 4. A lone VariableDeclaration in a VaraibleDeclaration feels redundant with the VariableStatement.
                            //
                            // Dive in without pushing a selection range.
                            if (ts.isBlock(node)
                                || ts.isTemplateSpan(node) || ts.isTemplateHead(node) || ts.isTemplateTail(node)
                                || prevNode && ts.isTemplateHead(prevNode)
                                || ts.isVariableDeclarationList(node) && ts.isVariableStatement(parentNode)
                                || ts.isSyntaxList(node) && ts.isVariableDeclarationList(parentNode)
                                || ts.isVariableDeclaration(node) && ts.isSyntaxList(parentNode) && children.length === 1) {
                                parentNode = node;
                                break;
                            }
                            // Synthesize a stop for '${ ... }' since '${' and '}' actually belong to siblings.
                            if (ts.isTemplateSpan(parentNode) && nextNode && ts.isTemplateMiddleOrTemplateTail(nextNode)) {
                                var start_2 = node.getFullStart() - "${".length;
                                var end_2 = nextNode.getStart() + "}".length;
                                pushSelectionRange(start_2, end_2);
                            }
                            // Blocks with braces, brackets, parens, or JSX tags on separate lines should be
                            // selected from open to close, including whitespace but not including the braces/etc. themselves.
                            var isBetweenMultiLineBookends = ts.isSyntaxList(node)
                                && isListOpener(prevNode)
                                && isListCloser(nextNode)
                                && !ts.positionsAreOnSameLine(prevNode.getStart(), nextNode.getStart(), sourceFile);
                            var jsDocCommentStart = ts.hasJSDocNodes(node) && node.jsDoc[0].getStart();
                            var start = isBetweenMultiLineBookends ? prevNode.getEnd() : node.getStart();
                            var end = isBetweenMultiLineBookends ? nextNode.getStart() : node.getEnd();
                            if (ts.isNumber(jsDocCommentStart)) {
                                pushSelectionRange(jsDocCommentStart, end);
                            }
                            pushSelectionRange(start, end);
                            // String literals should have a stop both inside and outside their quotes.
                            if (ts.isStringLiteral(node) || ts.isTemplateLiteral(node)) {
                                pushSelectionRange(start + 1, end - 1);
                            }
                            parentNode = node;
                            break;
                        }
                        // If we made it to the end of the for loop, we’re done.
                        // In practice, I’ve only seen this happen at the very end
                        // of a SourceFile.
                        if (i === children.length - 1) {
                            break outer;
                        }
                    }
                }
                return selectionRange;
                function pushSelectionRange(start, end) {
                    // Skip empty ranges
                    if (start !== end) {
                        var textSpan = ts.createTextSpanFromBounds(start, end);
                        if (!selectionRange || (
                        // Skip ranges that are identical to the parent
                        !ts.textSpansEqual(textSpan, selectionRange.textSpan) &&
                            // Skip ranges that don’t contain the original position
                            ts.textSpanIntersectsWithPosition(textSpan, pos))) {
                            selectionRange = __assign({ textSpan: textSpan }, selectionRange && { parent: selectionRange });
                        }
                    }
                }
            }
            SmartSelectionRange.getSmartSelectionRange = getSmartSelectionRange;
            /**
             * Like `ts.positionBelongsToNode`, except positions immediately after nodes
             * count too, unless that position belongs to the next node. In effect, makes
             * selections able to snap to preceding tokens when the cursor is on the tail
             * end of them with only whitespace ahead.
             * @param sourceFile The source file containing the nodes.
             * @param pos The position to check.
             * @param node The candidate node to snap to.
             */
            function positionShouldSnapToNode(sourceFile, pos, node) {
                // Can’t use 'ts.positionBelongsToNode()' here because it cleverly accounts
                // for missing nodes, which can’t really be considered when deciding what
                // to select.
                ts.Debug.assert(node.pos <= pos);
                if (pos < node.end) {
                    return true;
                }
                var nodeEnd = node.getEnd();
                if (nodeEnd === pos) {
                    return ts.getTouchingPropertyName(sourceFile, pos).pos < node.end;
                }
                return false;
            }
            var isImport = ts.or(ts.isImportDeclaration, ts.isImportEqualsDeclaration);
            /**
             * Gets the children of a node to be considered for selection ranging,
             * transforming them into an artificial tree according to their intuitive
             * grouping where no grouping actually exists in the parse tree. For example,
             * top-level imports are grouped into their own SyntaxList so they can be
             * selected all together, even though in the AST they’re just siblings of each
             * other as well as of other top-level statements and declarations.
             */
            function getSelectionChildren(node) {
                // Group top-level imports
                if (ts.isSourceFile(node)) {
                    return groupChildren(node.getChildAt(0).getChildren(), isImport);
                }
                // Mapped types _look_ like ObjectTypes with a single member,
                // but in fact don’t contain a SyntaxList or a node containing
                // the “key/value” pair like ObjectTypes do, but it seems intuitive
                // that the selection would snap to those points. The philosophy
                // of choosing a selection range is not so much about what the
                // syntax currently _is_ as what the syntax might easily become
                // if the user is making a selection; e.g., we synthesize a selection
                // around the “key/value” pair not because there’s a node there, but
                // because it allows the mapped type to become an object type with a
                // few keystrokes.
                if (ts.isMappedTypeNode(node)) {
                    var _a = node.getChildren(), openBraceToken = _a[0], children = _a.slice(1);
                    var closeBraceToken = ts.Debug.assertDefined(children.pop());
                    ts.Debug.assertEqual(openBraceToken.kind, 18 /* OpenBraceToken */);
                    ts.Debug.assertEqual(closeBraceToken.kind, 19 /* CloseBraceToken */);
                    // Group `-/+readonly` and `-/+?`
                    var groupedWithPlusMinusTokens = groupChildren(children, function (child) {
                        return child === node.readonlyToken || child.kind === 138 /* ReadonlyKeyword */ ||
                            child === node.questionToken || child.kind === 57 /* QuestionToken */;
                    });
                    // Group type parameter with surrounding brackets
                    var groupedWithBrackets = groupChildren(groupedWithPlusMinusTokens, function (_a) {
                        var kind = _a.kind;
                        return kind === 22 /* OpenBracketToken */ ||
                            kind === 155 /* TypeParameter */ ||
                            kind === 23 /* CloseBracketToken */;
                    });
                    return [
                        openBraceToken,
                        // Pivot on `:`
                        createSyntaxList(splitChildren(groupedWithBrackets, function (_a) {
                            var kind = _a.kind;
                            return kind === 58 /* ColonToken */;
                        })),
                        closeBraceToken,
                    ];
                }
                // Group modifiers and property name, then pivot on `:`.
                if (ts.isPropertySignature(node)) {
                    var children = groupChildren(node.getChildren(), function (child) {
                        return child === node.name || ts.contains(node.modifiers, child);
                    });
                    return splitChildren(children, function (_a) {
                        var kind = _a.kind;
                        return kind === 58 /* ColonToken */;
                    });
                }
                // Group the parameter name with its `...`, then that group with its `?`, then pivot on `=`.
                if (ts.isParameter(node)) {
                    var groupedDotDotDotAndName_1 = groupChildren(node.getChildren(), function (child) {
                        return child === node.dotDotDotToken || child === node.name;
                    });
                    var groupedWithQuestionToken = groupChildren(groupedDotDotDotAndName_1, function (child) {
                        return child === groupedDotDotDotAndName_1[0] || child === node.questionToken;
                    });
                    return splitChildren(groupedWithQuestionToken, function (_a) {
                        var kind = _a.kind;
                        return kind === 62 /* EqualsToken */;
                    });
                }
                // Pivot on '='
                if (ts.isBindingElement(node)) {
                    return splitChildren(node.getChildren(), function (_a) {
                        var kind = _a.kind;
                        return kind === 62 /* EqualsToken */;
                    });
                }
                return node.getChildren();
            }
            /**
             * Groups sibling nodes together into their own SyntaxList if they
             * a) are adjacent, AND b) match a predicate function.
             */
            function groupChildren(children, groupOn) {
                var result = [];
                var group;
                for (var _i = 0, children_1 = children; _i < children_1.length; _i++) {
                    var child = children_1[_i];
                    if (groupOn(child)) {
                        group = group || [];
                        group.push(child);
                    }
                    else {
                        if (group) {
                            result.push(createSyntaxList(group));
                            group = undefined;
                        }
                        result.push(child);
                    }
                }
                if (group) {
                    result.push(createSyntaxList(group));
                }
                return result;
            }
            /**
             * Splits sibling nodes into up to four partitions:
             * 1) everything left of the first node matched by `pivotOn`,
             * 2) the first node matched by `pivotOn`,
             * 3) everything right of the first node matched by `pivotOn`,
             * 4) a trailing semicolon, if `separateTrailingSemicolon` is enabled.
             * The left and right groups, if not empty, will each be grouped into their own containing SyntaxList.
             * @param children The sibling nodes to split.
             * @param pivotOn The predicate function to match the node to be the pivot. The first node that matches
             * the predicate will be used; any others that may match will be included into the right-hand group.
             * @param separateTrailingSemicolon If the last token is a semicolon, it will be returned as a separate
             * child rather than be included in the right-hand group.
             */
            function splitChildren(children, pivotOn, separateTrailingSemicolon) {
                if (separateTrailingSemicolon === void 0) {
                    separateTrailingSemicolon = true;
                }
                if (children.length < 2) {
                    return children;
                }
                var splitTokenIndex = ts.findIndex(children, pivotOn);
                if (splitTokenIndex === -1) {
                    return children;
                }
                var leftChildren = children.slice(0, splitTokenIndex);
                var splitToken = children[splitTokenIndex];
                var lastToken = ts.last(children);
                var separateLastToken = separateTrailingSemicolon && lastToken.kind === 26 /* SemicolonToken */;
                var rightChildren = children.slice(splitTokenIndex + 1, separateLastToken ? children.length - 1 : undefined);
                var result = ts.compact([
                    leftChildren.length ? createSyntaxList(leftChildren) : undefined,
                    splitToken,
                    rightChildren.length ? createSyntaxList(rightChildren) : undefined,
                ]);
                return separateLastToken ? result.concat(lastToken) : result;
            }
            function createSyntaxList(children) {
                ts.Debug.assertGreaterThanOrEqual(children.length, 1);
                var syntaxList = ts.createNode(323 /* SyntaxList */, children[0].pos, ts.last(children).end);
                syntaxList._children = children;
                return syntaxList;
            }
            function isListOpener(token) {
                var kind = token && token.kind;
                return kind === 18 /* OpenBraceToken */
                    || kind === 22 /* OpenBracketToken */
                    || kind === 20 /* OpenParenToken */
                    || kind === 268 /* JsxOpeningElement */;
            }
            function isListCloser(token) {
                var kind = token && token.kind;
                return kind === 19 /* CloseBraceToken */
                    || kind === 23 /* CloseBracketToken */
                    || kind === 21 /* CloseParenToken */
                    || kind === 269 /* JsxClosingElement */;
            }
        })(SmartSelectionRange = ts.SmartSelectionRange || (ts.SmartSelectionRange = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var SignatureHelp;
        (function (SignatureHelp) {
            var InvocationKind;
            (function (InvocationKind) {
                InvocationKind[InvocationKind["Call"] = 0] = "Call";
                InvocationKind[InvocationKind["TypeArgs"] = 1] = "TypeArgs";
                InvocationKind[InvocationKind["Contextual"] = 2] = "Contextual";
            })(InvocationKind || (InvocationKind = {}));
            function getSignatureHelpItems(program, sourceFile, position, triggerReason, cancellationToken) {
                var typeChecker = program.getTypeChecker();
                // Decide whether to show signature help
                var startingToken = ts.findTokenOnLeftOfPosition(sourceFile, position);
                if (!startingToken) {
                    // We are at the beginning of the file
                    return undefined;
                }
                // Only need to be careful if the user typed a character and signature help wasn't showing.
                var onlyUseSyntacticOwners = !!triggerReason && triggerReason.kind === "characterTyped";
                // Bail out quickly in the middle of a string or comment, don't provide signature help unless the user explicitly requested it.
                if (onlyUseSyntacticOwners && (ts.isInString(sourceFile, position, startingToken) || ts.isInComment(sourceFile, position))) {
                    return undefined;
                }
                var isManuallyInvoked = !!triggerReason && triggerReason.kind === "invoked";
                var argumentInfo = getContainingArgumentInfo(startingToken, position, sourceFile, typeChecker, isManuallyInvoked);
                if (!argumentInfo)
                    return undefined;
                cancellationToken.throwIfCancellationRequested();
                // Extra syntactic and semantic filtering of signature help
                var candidateInfo = getCandidateOrTypeInfo(argumentInfo, typeChecker, sourceFile, startingToken, onlyUseSyntacticOwners);
                cancellationToken.throwIfCancellationRequested();
                if (!candidateInfo) {
                    // We didn't have any sig help items produced by the TS compiler.  If this is a JS
                    // file, then see if we can figure out anything better.
                    return ts.isSourceFileJS(sourceFile) ? createJSSignatureHelpItems(argumentInfo, program, cancellationToken) : undefined;
                }
                return typeChecker.runWithCancellationToken(cancellationToken, function (typeChecker) {
                    return candidateInfo.kind === 0 /* Candidate */
                        ? createSignatureHelpItems(candidateInfo.candidates, candidateInfo.resolvedSignature, argumentInfo, sourceFile, typeChecker)
                        : createTypeHelpItems(candidateInfo.symbol, argumentInfo, sourceFile, typeChecker);
                });
            }
            SignatureHelp.getSignatureHelpItems = getSignatureHelpItems;
            var CandidateOrTypeKind;
            (function (CandidateOrTypeKind) {
                CandidateOrTypeKind[CandidateOrTypeKind["Candidate"] = 0] = "Candidate";
                CandidateOrTypeKind[CandidateOrTypeKind["Type"] = 1] = "Type";
            })(CandidateOrTypeKind || (CandidateOrTypeKind = {}));
            function getCandidateOrTypeInfo(_a, checker, sourceFile, startingToken, onlyUseSyntacticOwners) {
                var invocation = _a.invocation, argumentCount = _a.argumentCount;
                switch (invocation.kind) {
                    case 0 /* Call */: {
                        if (onlyUseSyntacticOwners && !isSyntacticOwner(startingToken, invocation.node, sourceFile)) {
                            return undefined;
                        }
                        var candidates = [];
                        var resolvedSignature = checker.getResolvedSignatureForSignatureHelp(invocation.node, candidates, argumentCount); // TODO: GH#18217
                        return candidates.length === 0 ? undefined : { kind: 0 /* Candidate */, candidates: candidates, resolvedSignature: resolvedSignature };
                    }
                    case 1 /* TypeArgs */: {
                        var called = invocation.called;
                        if (onlyUseSyntacticOwners && !containsPrecedingToken(startingToken, sourceFile, ts.isIdentifier(called) ? called.parent : called)) {
                            return undefined;
                        }
                        var candidates = ts.getPossibleGenericSignatures(called, argumentCount, checker);
                        if (candidates.length !== 0)
                            return { kind: 0 /* Candidate */, candidates: candidates, resolvedSignature: ts.first(candidates) };
                        var symbol = checker.getSymbolAtLocation(called);
                        return symbol && { kind: 1 /* Type */, symbol: symbol };
                    }
                    case 2 /* Contextual */:
                        return { kind: 0 /* Candidate */, candidates: [invocation.signature], resolvedSignature: invocation.signature };
                    default:
                        return ts.Debug.assertNever(invocation);
                }
            }
            function isSyntacticOwner(startingToken, node, sourceFile) {
                if (!ts.isCallOrNewExpression(node))
                    return false;
                var invocationChildren = node.getChildren(sourceFile);
                switch (startingToken.kind) {
                    case 20 /* OpenParenToken */:
                        return ts.contains(invocationChildren, startingToken);
                    case 27 /* CommaToken */: {
                        var containingList = ts.findContainingList(startingToken);
                        return !!containingList && ts.contains(invocationChildren, containingList);
                    }
                    case 29 /* LessThanToken */:
                        return containsPrecedingToken(startingToken, sourceFile, node.expression);
                    default:
                        return false;
                }
            }
            function createJSSignatureHelpItems(argumentInfo, program, cancellationToken) {
                if (argumentInfo.invocation.kind === 2 /* Contextual */)
                    return undefined;
                // See if we can find some symbol with the call expression name that has call signatures.
                var expression = getExpressionFromInvocation(argumentInfo.invocation);
                var name = ts.isIdentifier(expression) ? expression.text : ts.isPropertyAccessExpression(expression) ? expression.name.text : undefined;
                var typeChecker = program.getTypeChecker();
                return name === undefined ? undefined : ts.firstDefined(program.getSourceFiles(), function (sourceFile) {
                    return ts.firstDefined(sourceFile.getNamedDeclarations().get(name), function (declaration) {
                        var type = declaration.symbol && typeChecker.getTypeOfSymbolAtLocation(declaration.symbol, declaration);
                        var callSignatures = type && type.getCallSignatures();
                        if (callSignatures && callSignatures.length) {
                            return typeChecker.runWithCancellationToken(cancellationToken, function (typeChecker) { return createSignatureHelpItems(callSignatures, callSignatures[0], argumentInfo, sourceFile, typeChecker); });
                        }
                    });
                });
            }
            function containsPrecedingToken(startingToken, sourceFile, container) {
                var pos = startingToken.getFullStart();
                // There’s a possibility that `startingToken.parent` contains only `startingToken` and
                // missing nodes, none of which are valid to be returned by `findPrecedingToken`. In that
                // case, the preceding token we want is actually higher up the tree—almost definitely the
                // next parent, but theoretically the situation with missing nodes might be happening on
                // multiple nested levels.
                var currentParent = startingToken.parent;
                while (currentParent) {
                    var precedingToken = ts.findPrecedingToken(pos, sourceFile, currentParent, /*excludeJsdoc*/ true);
                    if (precedingToken) {
                        return ts.rangeContainsRange(container, precedingToken);
                    }
                    currentParent = currentParent.parent;
                }
                return ts.Debug.fail("Could not find preceding token");
            }
            function getArgumentInfoForCompletions(node, position, sourceFile) {
                var info = getImmediatelyContainingArgumentInfo(node, position, sourceFile);
                return !info || info.isTypeParameterList || info.invocation.kind !== 0 /* Call */ ? undefined
                    : { invocation: info.invocation.node, argumentCount: info.argumentCount, argumentIndex: info.argumentIndex };
            }
            SignatureHelp.getArgumentInfoForCompletions = getArgumentInfoForCompletions;
            function getArgumentOrParameterListInfo(node, sourceFile) {
                var info = getArgumentOrParameterListAndIndex(node, sourceFile);
                if (!info)
                    return undefined;
                var list = info.list, argumentIndex = info.argumentIndex;
                var argumentCount = getArgumentCount(list);
                if (argumentIndex !== 0) {
                    ts.Debug.assertLessThan(argumentIndex, argumentCount);
                }
                var argumentsSpan = getApplicableSpanForArguments(list, sourceFile);
                return { list: list, argumentIndex: argumentIndex, argumentCount: argumentCount, argumentsSpan: argumentsSpan };
            }
            function getArgumentOrParameterListAndIndex(node, sourceFile) {
                if (node.kind === 29 /* LessThanToken */ || node.kind === 20 /* OpenParenToken */) {
                    // Find the list that starts right *after* the < or ( token.
                    // If the user has just opened a list, consider this item 0.
                    return { list: getChildListThatStartsWithOpenerToken(node.parent, node, sourceFile), argumentIndex: 0 };
                }
                else {
                    // findListItemInfo can return undefined if we are not in parent's argument list
                    // or type argument list. This includes cases where the cursor is:
                    //   - To the right of the closing parenthesis, non-substitution template, or template tail.
                    //   - Between the type arguments and the arguments (greater than token)
                    //   - On the target of the call (parent.func)
                    //   - On the 'new' keyword in a 'new' expression
                    var list = ts.findContainingList(node);
                    return list && { list: list, argumentIndex: getArgumentIndex(list, node) };
                }
            }
            /**
             * Returns relevant information for the argument list and the current argument if we are
             * in the argument of an invocation; returns undefined otherwise.
             */
            function getImmediatelyContainingArgumentInfo(node, position, sourceFile) {
                var parent = node.parent;
                if (ts.isCallOrNewExpression(parent)) {
                    var invocation = parent;
                    // There are 3 cases to handle:
                    //   1. The token introduces a list, and should begin a signature help session
                    //   2. The token is either not associated with a list, or ends a list, so the session should end
                    //   3. The token is buried inside a list, and should give signature help
                    //
                    // The following are examples of each:
                    //
                    //    Case 1:
                    //          foo<#T, U>(#a, b)    -> The token introduces a list, and should begin a signature help session
                    //    Case 2:
                    //          fo#o<T, U>#(a, b)#   -> The token is either not associated with a list, or ends a list, so the session should end
                    //    Case 3:
                    //          foo<T#, U#>(a#, #b#) -> The token is buried inside a list, and should give signature help
                    // Find out if 'node' is an argument, a type argument, or neither
                    var info = getArgumentOrParameterListInfo(node, sourceFile);
                    if (!info)
                        return undefined;
                    var list = info.list, argumentIndex = info.argumentIndex, argumentCount = info.argumentCount, argumentsSpan = info.argumentsSpan;
                    var isTypeParameterList = !!parent.typeArguments && parent.typeArguments.pos === list.pos;
                    return { isTypeParameterList: isTypeParameterList, invocation: { kind: 0 /* Call */, node: invocation }, argumentsSpan: argumentsSpan, argumentIndex: argumentIndex, argumentCount: argumentCount };
                }
                else if (ts.isNoSubstitutionTemplateLiteral(node) && ts.isTaggedTemplateExpression(parent)) {
                    // Check if we're actually inside the template;
                    // otherwise we'll fall out and return undefined.
                    if (ts.isInsideTemplateLiteral(node, position, sourceFile)) {
                        return getArgumentListInfoForTemplate(parent, /*argumentIndex*/ 0, sourceFile);
                    }
                    return undefined;
                }
                else if (ts.isTemplateHead(node) && parent.parent.kind === 198 /* TaggedTemplateExpression */) {
                    var templateExpression = parent;
                    var tagExpression = templateExpression.parent;
                    ts.Debug.assert(templateExpression.kind === 211 /* TemplateExpression */);
                    var argumentIndex = ts.isInsideTemplateLiteral(node, position, sourceFile) ? 0 : 1;
                    return getArgumentListInfoForTemplate(tagExpression, argumentIndex, sourceFile);
                }
                else if (ts.isTemplateSpan(parent) && ts.isTaggedTemplateExpression(parent.parent.parent)) {
                    var templateSpan = parent;
                    var tagExpression = parent.parent.parent;
                    // If we're just after a template tail, don't show signature help.
                    if (ts.isTemplateTail(node) && !ts.isInsideTemplateLiteral(node, position, sourceFile)) {
                        return undefined;
                    }
                    var spanIndex = templateSpan.parent.templateSpans.indexOf(templateSpan);
                    var argumentIndex = getArgumentIndexForTemplatePiece(spanIndex, node, position, sourceFile);
                    return getArgumentListInfoForTemplate(tagExpression, argumentIndex, sourceFile);
                }
                else if (ts.isJsxOpeningLikeElement(parent)) {
                    // Provide a signature help for JSX opening element or JSX self-closing element.
                    // This is not guarantee that JSX tag-name is resolved into stateless function component. (that is done in "getSignatureHelpItems")
                    // i.e
                    //      export function MainButton(props: ButtonProps, context: any): JSX.Element { ... }
                    //      <MainButton /*signatureHelp*/
                    var attributeSpanStart = parent.attributes.pos;
                    var attributeSpanEnd = ts.skipTrivia(sourceFile.text, parent.attributes.end, /*stopAfterLineBreak*/ false);
                    return {
                        isTypeParameterList: false,
                        invocation: { kind: 0 /* Call */, node: parent },
                        argumentsSpan: ts.createTextSpan(attributeSpanStart, attributeSpanEnd - attributeSpanStart),
                        argumentIndex: 0,
                        argumentCount: 1
                    };
                }
                else {
                    var typeArgInfo = ts.getPossibleTypeArgumentsInfo(node, sourceFile);
                    if (typeArgInfo) {
                        var called = typeArgInfo.called, nTypeArguments = typeArgInfo.nTypeArguments;
                        var invocation = { kind: 1 /* TypeArgs */, called: called };
                        var argumentsSpan = ts.createTextSpanFromBounds(called.getStart(sourceFile), node.end);
                        return { isTypeParameterList: true, invocation: invocation, argumentsSpan: argumentsSpan, argumentIndex: nTypeArguments, argumentCount: nTypeArguments + 1 };
                    }
                    return undefined;
                }
            }
            function getImmediatelyContainingArgumentOrContextualParameterInfo(node, position, sourceFile, checker) {
                return tryGetParameterInfo(node, position, sourceFile, checker) || getImmediatelyContainingArgumentInfo(node, position, sourceFile);
            }
            function getHighestBinary(b) {
                return ts.isBinaryExpression(b.parent) ? getHighestBinary(b.parent) : b;
            }
            function countBinaryExpressionParameters(b) {
                return ts.isBinaryExpression(b.left) ? countBinaryExpressionParameters(b.left) + 1 : 2;
            }
            function tryGetParameterInfo(startingToken, _position, sourceFile, checker) {
                var info = getContextualSignatureLocationInfo(startingToken, sourceFile, checker);
                if (!info)
                    return undefined;
                var contextualType = info.contextualType, argumentIndex = info.argumentIndex, argumentCount = info.argumentCount, argumentsSpan = info.argumentsSpan;
                var signatures = contextualType.getCallSignatures();
                if (signatures.length !== 1)
                    return undefined;
                var invocation = { kind: 2 /* Contextual */, signature: ts.first(signatures), node: startingToken, symbol: chooseBetterSymbol(contextualType.symbol) };
                return { isTypeParameterList: false, invocation: invocation, argumentsSpan: argumentsSpan, argumentIndex: argumentIndex, argumentCount: argumentCount };
            }
            function getContextualSignatureLocationInfo(startingToken, sourceFile, checker) {
                if (startingToken.kind !== 20 /* OpenParenToken */ && startingToken.kind !== 27 /* CommaToken */)
                    return undefined;
                var parent = startingToken.parent;
                switch (parent.kind) {
                    case 200 /* ParenthesizedExpression */:
                    case 161 /* MethodDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                        var info = getArgumentOrParameterListInfo(startingToken, sourceFile);
                        if (!info)
                            return undefined;
                        var argumentIndex = info.argumentIndex, argumentCount = info.argumentCount, argumentsSpan = info.argumentsSpan;
                        var contextualType = ts.isMethodDeclaration(parent) ? checker.getContextualTypeForObjectLiteralElement(parent) : checker.getContextualType(parent);
                        return contextualType && { contextualType: contextualType, argumentIndex: argumentIndex, argumentCount: argumentCount, argumentsSpan: argumentsSpan };
                    case 209 /* BinaryExpression */: {
                        var highestBinary = getHighestBinary(parent);
                        var contextualType_1 = checker.getContextualType(highestBinary);
                        var argumentIndex_1 = startingToken.kind === 20 /* OpenParenToken */ ? 0 : countBinaryExpressionParameters(parent) - 1;
                        var argumentCount_1 = countBinaryExpressionParameters(highestBinary);
                        return contextualType_1 && { contextualType: contextualType_1, argumentIndex: argumentIndex_1, argumentCount: argumentCount_1, argumentsSpan: ts.createTextSpanFromNode(parent) };
                    }
                    default:
                        return undefined;
                }
            }
            // The type of a function type node has a symbol at that node, but it's better to use the symbol for a parameter or type alias.
            function chooseBetterSymbol(s) {
                return s.name === "__type" /* Type */
                    ? ts.firstDefined(s.declarations, function (d) { return ts.isFunctionTypeNode(d) ? d.parent.symbol : undefined; }) || s
                    : s;
            }
            function getArgumentIndex(argumentsList, node) {
                // The list we got back can include commas.  In the presence of errors it may
                // also just have nodes without commas.  For example "Foo(a b c)" will have 3
                // args without commas. We want to find what index we're at.  So we count
                // forward until we hit ourselves, only incrementing the index if it isn't a
                // comma.
                //
                // Note: the subtlety around trailing commas (in getArgumentCount) does not apply
                // here.  That's because we're only walking forward until we hit the node we're
                // on.  In that case, even if we're after the trailing comma, we'll still see
                // that trailing comma in the list, and we'll have generated the appropriate
                // arg index.
                var argumentIndex = 0;
                for (var _i = 0, _a = argumentsList.getChildren(); _i < _a.length; _i++) {
                    var child = _a[_i];
                    if (child === node) {
                        break;
                    }
                    if (child.kind !== 27 /* CommaToken */) {
                        argumentIndex++;
                    }
                }
                return argumentIndex;
            }
            function getArgumentCount(argumentsList) {
                // The argument count for a list is normally the number of non-comma children it has.
                // For example, if you have "Foo(a,b)" then there will be three children of the arg
                // list 'a' '<comma>' 'b'.  So, in this case the arg count will be 2.  However, there
                // is a small subtlety.  If you have "Foo(a,)", then the child list will just have
                // 'a' '<comma>'.  So, in the case where the last child is a comma, we increase the
                // arg count by one to compensate.
                //
                // Note: this subtlety only applies to the last comma.  If you had "Foo(a,," then
                // we'll have: 'a' '<comma>' '<missing>'
                // That will give us 2 non-commas.  We then add one for the last comma, giving us an
                // arg count of 3.
                var listChildren = argumentsList.getChildren();
                var argumentCount = ts.countWhere(listChildren, function (arg) { return arg.kind !== 27 /* CommaToken */; });
                if (listChildren.length > 0 && ts.last(listChildren).kind === 27 /* CommaToken */) {
                    argumentCount++;
                }
                return argumentCount;
            }
            // spanIndex is either the index for a given template span.
            // This does not give appropriate results for a NoSubstitutionTemplateLiteral
            function getArgumentIndexForTemplatePiece(spanIndex, node, position, sourceFile) {
                // Because the TemplateStringsArray is the first argument, we have to offset each substitution expression by 1.
                // There are three cases we can encounter:
                //      1. We are precisely in the template literal (argIndex = 0).
                //      2. We are in or to the right of the substitution expression (argIndex = spanIndex + 1).
                //      3. We are directly to the right of the template literal, but because we look for the token on the left,
                //          not enough to put us in the substitution expression; we should consider ourselves part of
                //          the *next* span's expression by offsetting the index (argIndex = (spanIndex + 1) + 1).
                //
                /* eslint-disable no-double-space */
                // Example: f  `# abcd $#{#  1 + 1#  }# efghi ${ #"#hello"#  }  #  `
                //              ^       ^ ^       ^   ^          ^ ^      ^     ^
                // Case:        1       1 3       2   1          3 2      2     1
                /* eslint-enable no-double-space */
                ts.Debug.assert(position >= node.getStart(), "Assumed 'position' could not occur before node.");
                if (ts.isTemplateLiteralToken(node)) {
                    if (ts.isInsideTemplateLiteral(node, position, sourceFile)) {
                        return 0;
                    }
                    return spanIndex + 2;
                }
                return spanIndex + 1;
            }
            function getArgumentListInfoForTemplate(tagExpression, argumentIndex, sourceFile) {
                // argumentCount is either 1 or (numSpans + 1) to account for the template strings array argument.
                var argumentCount = ts.isNoSubstitutionTemplateLiteral(tagExpression.template) ? 1 : tagExpression.template.templateSpans.length + 1;
                if (argumentIndex !== 0) {
                    ts.Debug.assertLessThan(argumentIndex, argumentCount);
                }
                return {
                    isTypeParameterList: false,
                    invocation: { kind: 0 /* Call */, node: tagExpression },
                    argumentsSpan: getApplicableSpanForTaggedTemplate(tagExpression, sourceFile),
                    argumentIndex: argumentIndex,
                    argumentCount: argumentCount
                };
            }
            function getApplicableSpanForArguments(argumentsList, sourceFile) {
                // We use full start and skip trivia on the end because we want to include trivia on
                // both sides. For example,
                //
                //    foo(   /*comment */     a, b, c      /*comment*/     )
                //        |                                               |
                //
                // The applicable span is from the first bar to the second bar (inclusive,
                // but not including parentheses)
                var applicableSpanStart = argumentsList.getFullStart();
                var applicableSpanEnd = ts.skipTrivia(sourceFile.text, argumentsList.getEnd(), /*stopAfterLineBreak*/ false);
                return ts.createTextSpan(applicableSpanStart, applicableSpanEnd - applicableSpanStart);
            }
            function getApplicableSpanForTaggedTemplate(taggedTemplate, sourceFile) {
                var template = taggedTemplate.template;
                var applicableSpanStart = template.getStart();
                var applicableSpanEnd = template.getEnd();
                // We need to adjust the end position for the case where the template does not have a tail.
                // Otherwise, we will not show signature help past the expression.
                // For example,
                //
                //      ` ${ 1 + 1 foo(10)
                //       |       |
                // This is because a Missing node has no width. However, what we actually want is to include trivia
                // leading up to the next token in case the user is about to type in a TemplateMiddle or TemplateTail.
                if (template.kind === 211 /* TemplateExpression */) {
                    var lastSpan = ts.last(template.templateSpans);
                    if (lastSpan.literal.getFullWidth() === 0) {
                        applicableSpanEnd = ts.skipTrivia(sourceFile.text, applicableSpanEnd, /*stopAfterLineBreak*/ false);
                    }
                }
                return ts.createTextSpan(applicableSpanStart, applicableSpanEnd - applicableSpanStart);
            }
            function getContainingArgumentInfo(node, position, sourceFile, checker, isManuallyInvoked) {
                var _loop_7 = function (n) {
                    // If the node is not a subspan of its parent, this is a big problem.
                    // There have been crashes that might be caused by this violation.
                    ts.Debug.assert(ts.rangeContainsRange(n.parent, n), "Not a subspan", function () { return "Child: " + ts.Debug.formatSyntaxKind(n.kind) + ", parent: " + ts.Debug.formatSyntaxKind(n.parent.kind); });
                    var argumentInfo = getImmediatelyContainingArgumentOrContextualParameterInfo(n, position, sourceFile, checker);
                    if (argumentInfo) {
                        return { value: argumentInfo };
                    }
                };
                for (var n = node; !ts.isSourceFile(n) && (isManuallyInvoked || !ts.isBlock(n)); n = n.parent) {
                    var state_2 = _loop_7(n);
                    if (typeof state_2 === "object")
                        return state_2.value;
                }
                return undefined;
            }
            function getChildListThatStartsWithOpenerToken(parent, openerToken, sourceFile) {
                var children = parent.getChildren(sourceFile);
                var indexOfOpenerToken = children.indexOf(openerToken);
                ts.Debug.assert(indexOfOpenerToken >= 0 && children.length > indexOfOpenerToken + 1);
                return children[indexOfOpenerToken + 1];
            }
            function getExpressionFromInvocation(invocation) {
                return invocation.kind === 0 /* Call */ ? ts.getInvokedExpression(invocation.node) : invocation.called;
            }
            function getEnclosingDeclarationFromInvocation(invocation) {
                return invocation.kind === 0 /* Call */ ? invocation.node : invocation.kind === 1 /* TypeArgs */ ? invocation.called : invocation.node;
            }
            var signatureHelpNodeBuilderFlags = 8192 /* OmitParameterModifiers */ | 70221824 /* IgnoreErrors */ | 16384 /* UseAliasDefinedOutsideCurrentScope */;
            function createSignatureHelpItems(candidates, resolvedSignature, _a, sourceFile, typeChecker) {
                var isTypeParameterList = _a.isTypeParameterList, argumentCount = _a.argumentCount, applicableSpan = _a.argumentsSpan, invocation = _a.invocation, argumentIndex = _a.argumentIndex;
                var enclosingDeclaration = getEnclosingDeclarationFromInvocation(invocation);
                var callTargetSymbol = invocation.kind === 2 /* Contextual */ ? invocation.symbol : typeChecker.getSymbolAtLocation(getExpressionFromInvocation(invocation));
                var callTargetDisplayParts = callTargetSymbol ? ts.symbolToDisplayParts(typeChecker, callTargetSymbol, /*enclosingDeclaration*/ undefined, /*meaning*/ undefined) : ts.emptyArray;
                var items = candidates.map(function (candidateSignature) { return getSignatureHelpItem(candidateSignature, callTargetDisplayParts, isTypeParameterList, typeChecker, enclosingDeclaration, sourceFile); });
                if (argumentIndex !== 0) {
                    ts.Debug.assertLessThan(argumentIndex, argumentCount);
                }
                var selectedItemIndex = candidates.indexOf(resolvedSignature);
                ts.Debug.assert(selectedItemIndex !== -1); // If candidates is non-empty it should always include bestSignature. We check for an empty candidates before calling this function.
                return { items: items, applicableSpan: applicableSpan, selectedItemIndex: selectedItemIndex, argumentIndex: argumentIndex, argumentCount: argumentCount };
            }
            function createTypeHelpItems(symbol, _a, sourceFile, checker) {
                var argumentCount = _a.argumentCount, applicableSpan = _a.argumentsSpan, invocation = _a.invocation, argumentIndex = _a.argumentIndex;
                var typeParameters = checker.getLocalTypeParametersOfClassOrInterfaceOrTypeAlias(symbol);
                if (!typeParameters)
                    return undefined;
                var items = [getTypeHelpItem(symbol, typeParameters, checker, getEnclosingDeclarationFromInvocation(invocation), sourceFile)];
                return { items: items, applicableSpan: applicableSpan, selectedItemIndex: 0, argumentIndex: argumentIndex, argumentCount: argumentCount };
            }
            function getTypeHelpItem(symbol, typeParameters, checker, enclosingDeclaration, sourceFile) {
                var typeSymbolDisplay = ts.symbolToDisplayParts(checker, symbol);
                var printer = ts.createPrinter({ removeComments: true });
                var parameters = typeParameters.map(function (t) { return createSignatureHelpParameterForTypeParameter(t, checker, enclosingDeclaration, sourceFile, printer); });
                var documentation = symbol.getDocumentationComment(checker);
                var tags = symbol.getJsDocTags();
                var prefixDisplayParts = __spreadArrays(typeSymbolDisplay, [ts.punctuationPart(29 /* LessThanToken */)]);
                return { isVariadic: false, prefixDisplayParts: prefixDisplayParts, suffixDisplayParts: [ts.punctuationPart(31 /* GreaterThanToken */)], separatorDisplayParts: separatorDisplayParts, parameters: parameters, documentation: documentation, tags: tags };
            }
            var separatorDisplayParts = [ts.punctuationPart(27 /* CommaToken */), ts.spacePart()];
            function getSignatureHelpItem(candidateSignature, callTargetDisplayParts, isTypeParameterList, checker, enclosingDeclaration, sourceFile) {
                var _a = (isTypeParameterList ? itemInfoForTypeParameters : itemInfoForParameters)(candidateSignature, checker, enclosingDeclaration, sourceFile), isVariadic = _a.isVariadic, parameters = _a.parameters, prefix = _a.prefix, suffix = _a.suffix;
                var prefixDisplayParts = __spreadArrays(callTargetDisplayParts, prefix);
                var suffixDisplayParts = __spreadArrays(suffix, returnTypeToDisplayParts(candidateSignature, enclosingDeclaration, checker));
                var documentation = candidateSignature.getDocumentationComment(checker);
                var tags = candidateSignature.getJsDocTags();
                return { isVariadic: isVariadic, prefixDisplayParts: prefixDisplayParts, suffixDisplayParts: suffixDisplayParts, separatorDisplayParts: separatorDisplayParts, parameters: parameters, documentation: documentation, tags: tags };
            }
            function returnTypeToDisplayParts(candidateSignature, enclosingDeclaration, checker) {
                return ts.mapToDisplayParts(function (writer) {
                    writer.writePunctuation(":");
                    writer.writeSpace(" ");
                    var predicate = checker.getTypePredicateOfSignature(candidateSignature);
                    if (predicate) {
                        checker.writeTypePredicate(predicate, enclosingDeclaration, /*flags*/ undefined, writer);
                    }
                    else {
                        checker.writeType(checker.getReturnTypeOfSignature(candidateSignature), enclosingDeclaration, /*flags*/ undefined, writer);
                    }
                });
            }
            function itemInfoForTypeParameters(candidateSignature, checker, enclosingDeclaration, sourceFile) {
                var typeParameters = (candidateSignature.target || candidateSignature).typeParameters;
                var printer = ts.createPrinter({ removeComments: true });
                var parameters = (typeParameters || ts.emptyArray).map(function (t) { return createSignatureHelpParameterForTypeParameter(t, checker, enclosingDeclaration, sourceFile, printer); });
                var parameterParts = ts.mapToDisplayParts(function (writer) {
                    var thisParameter = candidateSignature.thisParameter ? [checker.symbolToParameterDeclaration(candidateSignature.thisParameter, enclosingDeclaration, signatureHelpNodeBuilderFlags)] : [];
                    var params = ts.createNodeArray(__spreadArrays(thisParameter, checker.getExpandedParameters(candidateSignature).map(function (param) { return checker.symbolToParameterDeclaration(param, enclosingDeclaration, signatureHelpNodeBuilderFlags); })));
                    printer.writeList(2576 /* CallExpressionArguments */, params, sourceFile, writer);
                });
                return { isVariadic: false, parameters: parameters, prefix: [ts.punctuationPart(29 /* LessThanToken */)], suffix: __spreadArrays([ts.punctuationPart(31 /* GreaterThanToken */)], parameterParts) };
            }
            function itemInfoForParameters(candidateSignature, checker, enclosingDeclaration, sourceFile) {
                var isVariadic = checker.hasEffectiveRestParameter(candidateSignature);
                var printer = ts.createPrinter({ removeComments: true });
                var typeParameterParts = ts.mapToDisplayParts(function (writer) {
                    if (candidateSignature.typeParameters && candidateSignature.typeParameters.length) {
                        var args = ts.createNodeArray(candidateSignature.typeParameters.map(function (p) { return checker.typeParameterToDeclaration(p, enclosingDeclaration); }));
                        printer.writeList(53776 /* TypeParameters */, args, sourceFile, writer);
                    }
                });
                var parameters = checker.getExpandedParameters(candidateSignature).map(function (p) { return createSignatureHelpParameterForParameter(p, checker, enclosingDeclaration, sourceFile, printer); });
                return { isVariadic: isVariadic, parameters: parameters, prefix: __spreadArrays(typeParameterParts, [ts.punctuationPart(20 /* OpenParenToken */)]), suffix: [ts.punctuationPart(21 /* CloseParenToken */)] };
            }
            function createSignatureHelpParameterForParameter(parameter, checker, enclosingDeclaration, sourceFile, printer) {
                var displayParts = ts.mapToDisplayParts(function (writer) {
                    var param = checker.symbolToParameterDeclaration(parameter, enclosingDeclaration, signatureHelpNodeBuilderFlags);
                    printer.writeNode(4 /* Unspecified */, param, sourceFile, writer);
                });
                var isOptional = checker.isOptionalParameter(parameter.valueDeclaration);
                return { name: parameter.name, documentation: parameter.getDocumentationComment(checker), displayParts: displayParts, isOptional: isOptional };
            }
            function createSignatureHelpParameterForTypeParameter(typeParameter, checker, enclosingDeclaration, sourceFile, printer) {
                var displayParts = ts.mapToDisplayParts(function (writer) {
                    var param = checker.typeParameterToDeclaration(typeParameter, enclosingDeclaration);
                    printer.writeNode(4 /* Unspecified */, param, sourceFile, writer);
                });
                return { name: typeParameter.symbol.name, documentation: typeParameter.symbol.getDocumentationComment(checker), displayParts: displayParts, isOptional: false };
            }
        })(SignatureHelp = ts.SignatureHelp || (ts.SignatureHelp = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var base64UrlRegExp = /^data:(?:application\/json(?:;charset=[uU][tT][fF]-8);base64,([A-Za-z0-9+\/=]+)$)?/;
        function getSourceMapper(host) {
            var getCanonicalFileName = ts.createGetCanonicalFileName(host.useCaseSensitiveFileNames());
            var currentDirectory = host.getCurrentDirectory();
            var sourceFileLike = ts.createMap();
            var documentPositionMappers = ts.createMap();
            return { tryGetSourcePosition: tryGetSourcePosition, tryGetGeneratedPosition: tryGetGeneratedPosition, toLineColumnOffset: toLineColumnOffset, clearCache: clearCache };
            function toPath(fileName) {
                return ts.toPath(fileName, currentDirectory, getCanonicalFileName);
            }
            function getDocumentPositionMapper(generatedFileName, sourceFileName) {
                var path = toPath(generatedFileName);
                var value = documentPositionMappers.get(path);
                if (value)
                    return value;
                var mapper;
                if (host.getDocumentPositionMapper) {
                    mapper = host.getDocumentPositionMapper(generatedFileName, sourceFileName);
                }
                else if (host.readFile) {
                    var file = getSourceFileLike(generatedFileName);
                    mapper = file && ts.getDocumentPositionMapper({ getSourceFileLike: getSourceFileLike, getCanonicalFileName: getCanonicalFileName, log: function (s) { return host.log(s); } }, generatedFileName, ts.getLineInfo(file.text, ts.getLineStarts(file)), function (f) { return !host.fileExists || host.fileExists(f) ? host.readFile(f) : undefined; });
                }
                documentPositionMappers.set(path, mapper || ts.identitySourceMapConsumer);
                return mapper || ts.identitySourceMapConsumer;
            }
            function tryGetSourcePosition(info) {
                if (!ts.isDeclarationFileName(info.fileName))
                    return undefined;
                var file = getSourceFile(info.fileName);
                if (!file)
                    return undefined;
                var newLoc = getDocumentPositionMapper(info.fileName).getSourcePosition(info);
                return !newLoc || newLoc === info ? undefined : tryGetSourcePosition(newLoc) || newLoc;
            }
            function tryGetGeneratedPosition(info) {
                if (ts.isDeclarationFileName(info.fileName))
                    return undefined;
                var sourceFile = getSourceFile(info.fileName);
                if (!sourceFile)
                    return undefined;
                var program = host.getProgram();
                // If this is source file of project reference source (instead of redirect) there is no generated position
                if (program.isSourceOfProjectReferenceRedirect(sourceFile.fileName)) {
                    return undefined;
                }
                var options = program.getCompilerOptions();
                var outPath = options.outFile || options.out;
                var declarationPath = outPath ?
                    ts.removeFileExtension(outPath) + ".d.ts" /* Dts */ :
                    ts.getDeclarationEmitOutputFilePathWorker(info.fileName, program.getCompilerOptions(), currentDirectory, program.getCommonSourceDirectory(), getCanonicalFileName);
                if (declarationPath === undefined)
                    return undefined;
                var newLoc = getDocumentPositionMapper(declarationPath, info.fileName).getGeneratedPosition(info);
                return newLoc === info ? undefined : newLoc;
            }
            function getSourceFile(fileName) {
                var program = host.getProgram();
                if (!program)
                    return undefined;
                var path = toPath(fileName);
                // file returned here could be .d.ts when asked for .ts file if projectReferences and module resolution created this source file
                var file = program.getSourceFileByPath(path);
                return file && file.resolvedPath === path ? file : undefined;
            }
            function getOrCreateSourceFileLike(fileName) {
                var path = toPath(fileName);
                var fileFromCache = sourceFileLike.get(path);
                if (fileFromCache !== undefined)
                    return fileFromCache ? fileFromCache : undefined;
                if (!host.readFile || host.fileExists && !host.fileExists(path)) {
                    sourceFileLike.set(path, false);
                    return undefined;
                }
                // And failing that, check the disk
                var text = host.readFile(path);
                var file = text ? createSourceFileLike(text) : false;
                sourceFileLike.set(path, file);
                return file ? file : undefined;
            }
            // This can be called from source mapper in either source program or program that includes generated file
            function getSourceFileLike(fileName) {
                return !host.getSourceFileLike ?
                    getSourceFile(fileName) || getOrCreateSourceFileLike(fileName) :
                    host.getSourceFileLike(fileName);
            }
            function toLineColumnOffset(fileName, position) {
                var file = getSourceFileLike(fileName); // TODO: GH#18217
                return file.getLineAndCharacterOfPosition(position);
            }
            function clearCache() {
                sourceFileLike.clear();
                documentPositionMappers.clear();
            }
        }
        ts.getSourceMapper = getSourceMapper;
        function getDocumentPositionMapper(host, generatedFileName, generatedFileLineInfo, readMapFile) {
            var mapFileName = ts.tryGetSourceMappingURL(generatedFileLineInfo);
            if (mapFileName) {
                var match = base64UrlRegExp.exec(mapFileName);
                if (match) {
                    if (match[1]) {
                        var base64Object = match[1];
                        return convertDocumentToSourceMapper(host, ts.base64decode(ts.sys, base64Object), generatedFileName);
                    }
                    // Not a data URL we can parse, skip it
                    mapFileName = undefined;
                }
            }
            var possibleMapLocations = [];
            if (mapFileName) {
                possibleMapLocations.push(mapFileName);
            }
            possibleMapLocations.push(generatedFileName + ".map");
            var originalMapFileName = mapFileName && ts.getNormalizedAbsolutePath(mapFileName, ts.getDirectoryPath(generatedFileName));
            for (var _i = 0, possibleMapLocations_1 = possibleMapLocations; _i < possibleMapLocations_1.length; _i++) {
                var location = possibleMapLocations_1[_i];
                var mapFileName_1 = ts.getNormalizedAbsolutePath(location, ts.getDirectoryPath(generatedFileName));
                var mapFileContents = readMapFile(mapFileName_1, originalMapFileName);
                if (ts.isString(mapFileContents)) {
                    return convertDocumentToSourceMapper(host, mapFileContents, mapFileName_1);
                }
                if (mapFileContents !== undefined) {
                    return mapFileContents || undefined;
                }
            }
            return undefined;
        }
        ts.getDocumentPositionMapper = getDocumentPositionMapper;
        function convertDocumentToSourceMapper(host, contents, mapFileName) {
            var map = ts.tryParseRawSourceMap(contents);
            if (!map || !map.sources || !map.file || !map.mappings) {
                // obviously invalid map
                return undefined;
            }
            // Dont support sourcemaps that contain inlined sources
            if (map.sourcesContent && map.sourcesContent.some(ts.isString))
                return undefined;
            return ts.createDocumentPositionMapper(host, map, mapFileName);
        }
        function createSourceFileLike(text, lineMap) {
            return {
                text: text,
                lineMap: lineMap,
                getLineAndCharacterOfPosition: function (pos) {
                    return ts.computeLineAndCharacterOfPosition(ts.getLineStarts(this), pos);
                }
            };
        }
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var visitedNestedConvertibleFunctions = ts.createMap();
        function computeSuggestionDiagnostics(sourceFile, program, cancellationToken) {
            program.getSemanticDiagnostics(sourceFile, cancellationToken);
            var diags = [];
            var checker = program.getTypeChecker();
            if (sourceFile.commonJsModuleIndicator &&
                (ts.programContainsEs6Modules(program) || ts.compilerOptionsIndicateEs6Modules(program.getCompilerOptions())) &&
                containsTopLevelCommonjs(sourceFile)) {
                diags.push(ts.createDiagnosticForNode(getErrorNodeFromCommonJsIndicator(sourceFile.commonJsModuleIndicator), ts.Diagnostics.File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module));
            }
            var isJsFile = ts.isSourceFileJS(sourceFile);
            visitedNestedConvertibleFunctions.clear();
            check(sourceFile);
            if (ts.getAllowSyntheticDefaultImports(program.getCompilerOptions())) {
                for (var _i = 0, _a = sourceFile.imports; _i < _a.length; _i++) {
                    var moduleSpecifier = _a[_i];
                    var importNode = ts.importFromModuleSpecifier(moduleSpecifier);
                    var name = importNameForConvertToDefaultImport(importNode);
                    if (!name)
                        continue;
                    var module_1 = ts.getResolvedModule(sourceFile, moduleSpecifier.text);
                    var resolvedFile = module_1 && program.getSourceFile(module_1.resolvedFileName);
                    if (resolvedFile && resolvedFile.externalModuleIndicator && ts.isExportAssignment(resolvedFile.externalModuleIndicator) && resolvedFile.externalModuleIndicator.isExportEquals) {
                        diags.push(ts.createDiagnosticForNode(name, ts.Diagnostics.Import_may_be_converted_to_a_default_import));
                    }
                }
            }
            ts.addRange(diags, sourceFile.bindSuggestionDiagnostics);
            ts.addRange(diags, program.getSuggestionDiagnostics(sourceFile, cancellationToken));
            return diags.sort(function (d1, d2) { return d1.start - d2.start; });
            function check(node) {
                if (isJsFile) {
                    switch (node.kind) {
                        case 201 /* FunctionExpression */:
                            var decl = ts.getDeclarationOfExpando(node);
                            if (decl) {
                                var symbol_1 = decl.symbol;
                                if (symbol_1 && (symbol_1.exports && symbol_1.exports.size || symbol_1.members && symbol_1.members.size)) {
                                    diags.push(ts.createDiagnosticForNode(ts.isVariableDeclaration(node.parent) ? node.parent.name : node, ts.Diagnostics.This_constructor_function_may_be_converted_to_a_class_declaration));
                                    break;
                                }
                            }
                        // falls through if no diagnostic was created
                        case 244 /* FunctionDeclaration */:
                            var symbol = node.symbol;
                            if (symbol.members && (symbol.members.size > 0)) {
                                diags.push(ts.createDiagnosticForNode(ts.isVariableDeclaration(node.parent) ? node.parent.name : node, ts.Diagnostics.This_constructor_function_may_be_converted_to_a_class_declaration));
                            }
                            break;
                    }
                }
                else {
                    if (ts.isVariableStatement(node) &&
                        node.parent === sourceFile &&
                        node.declarationList.flags & 2 /* Const */ &&
                        node.declarationList.declarations.length === 1) {
                        var init = node.declarationList.declarations[0].initializer;
                        if (init && ts.isRequireCall(init, /*checkArgumentIsStringLiteralLike*/ true)) {
                            diags.push(ts.createDiagnosticForNode(init, ts.Diagnostics.require_call_may_be_converted_to_an_import));
                        }
                    }
                    if (ts.codefix.parameterShouldGetTypeFromJSDoc(node)) {
                        diags.push(ts.createDiagnosticForNode(node.name || node, ts.Diagnostics.JSDoc_types_may_be_moved_to_TypeScript_types));
                    }
                }
                if (ts.isFunctionLikeDeclaration(node)) {
                    addConvertToAsyncFunctionDiagnostics(node, checker, diags);
                }
                node.forEachChild(check);
            }
        }
        ts.computeSuggestionDiagnostics = computeSuggestionDiagnostics;
        // convertToEs6Module only works on top-level, so don't trigger it if commonjs code only appears in nested scopes.
        function containsTopLevelCommonjs(sourceFile) {
            return sourceFile.statements.some(function (statement) {
                switch (statement.kind) {
                    case 225 /* VariableStatement */:
                        return statement.declarationList.declarations.some(function (decl) {
                            return !!decl.initializer && ts.isRequireCall(propertyAccessLeftHandSide(decl.initializer), /*checkArgumentIsStringLiteralLike*/ true);
                        });
                    case 226 /* ExpressionStatement */: {
                        var expression = statement.expression;
                        if (!ts.isBinaryExpression(expression))
                            return ts.isRequireCall(expression, /*checkArgumentIsStringLiteralLike*/ true);
                        var kind = ts.getAssignmentDeclarationKind(expression);
                        return kind === 1 /* ExportsProperty */ || kind === 2 /* ModuleExports */;
                    }
                    default:
                        return false;
                }
            });
        }
        function propertyAccessLeftHandSide(node) {
            return ts.isPropertyAccessExpression(node) ? propertyAccessLeftHandSide(node.expression) : node;
        }
        function importNameForConvertToDefaultImport(node) {
            switch (node.kind) {
                case 254 /* ImportDeclaration */:
                    var importClause = node.importClause, moduleSpecifier = node.moduleSpecifier;
                    return importClause && !importClause.name && importClause.namedBindings && importClause.namedBindings.kind === 256 /* NamespaceImport */ && ts.isStringLiteral(moduleSpecifier)
                        ? importClause.namedBindings.name
                        : undefined;
                case 253 /* ImportEqualsDeclaration */:
                    return node.name;
                default:
                    return undefined;
            }
        }
        function addConvertToAsyncFunctionDiagnostics(node, checker, diags) {
            // need to check function before checking map so that deeper levels of nested callbacks are checked
            if (isConvertibleFunction(node, checker) && !visitedNestedConvertibleFunctions.has(getKeyFromNode(node))) {
                diags.push(ts.createDiagnosticForNode(!node.name && ts.isVariableDeclaration(node.parent) && ts.isIdentifier(node.parent.name) ? node.parent.name : node, ts.Diagnostics.This_may_be_converted_to_an_async_function));
            }
        }
        function isConvertibleFunction(node, checker) {
            return !ts.isAsyncFunction(node) &&
                node.body &&
                ts.isBlock(node.body) &&
                hasReturnStatementWithPromiseHandler(node.body) &&
                returnsPromise(node, checker);
        }
        function returnsPromise(node, checker) {
            var functionType = checker.getTypeAtLocation(node);
            var callSignatures = checker.getSignaturesOfType(functionType, 0 /* Call */);
            var returnType = callSignatures.length ? checker.getReturnTypeOfSignature(callSignatures[0]) : undefined;
            return !!returnType && !!checker.getPromisedTypeOfPromise(returnType);
        }
        function getErrorNodeFromCommonJsIndicator(commonJsModuleIndicator) {
            return ts.isBinaryExpression(commonJsModuleIndicator) ? commonJsModuleIndicator.left : commonJsModuleIndicator;
        }
        function hasReturnStatementWithPromiseHandler(body) {
            return !!ts.forEachReturnStatement(body, isReturnStatementWithFixablePromiseHandler);
        }
        function isReturnStatementWithFixablePromiseHandler(node) {
            return ts.isReturnStatement(node) && !!node.expression && isFixablePromiseHandler(node.expression);
        }
        ts.isReturnStatementWithFixablePromiseHandler = isReturnStatementWithFixablePromiseHandler;
        // Should be kept up to date with transformExpression in convertToAsyncFunction.ts
        function isFixablePromiseHandler(node) {
            // ensure outermost call exists and is a promise handler
            if (!isPromiseHandler(node) || !node.arguments.every(isFixablePromiseArgument)) {
                return false;
            }
            // ensure all chained calls are valid
            var currentNode = node.expression;
            while (isPromiseHandler(currentNode) || ts.isPropertyAccessExpression(currentNode)) {
                if (ts.isCallExpression(currentNode) && !currentNode.arguments.every(isFixablePromiseArgument)) {
                    return false;
                }
                currentNode = currentNode.expression;
            }
            return true;
        }
        ts.isFixablePromiseHandler = isFixablePromiseHandler;
        function isPromiseHandler(node) {
            return ts.isCallExpression(node) && (ts.hasPropertyAccessExpressionWithName(node, "then") || ts.hasPropertyAccessExpressionWithName(node, "catch"));
        }
        // should be kept up to date with getTransformationBody in convertToAsyncFunction.ts
        function isFixablePromiseArgument(arg) {
            switch (arg.kind) {
                case 244 /* FunctionDeclaration */:
                case 201 /* FunctionExpression */:
                case 202 /* ArrowFunction */:
                    visitedNestedConvertibleFunctions.set(getKeyFromNode(arg), true);
                // falls through
                case 100 /* NullKeyword */:
                case 75 /* Identifier */: // identifier includes undefined
                    return true;
                default:
                    return false;
            }
        }
        function getKeyFromNode(exp) {
            return exp.pos.toString() + ":" + exp.end.toString();
        }
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var SymbolDisplay;
        (function (SymbolDisplay) {
            // TODO(drosen): use contextual SemanticMeaning.
            function getSymbolKind(typeChecker, symbol, location) {
                var result = getSymbolKindOfConstructorPropertyMethodAccessorFunctionOrVar(typeChecker, symbol, location);
                if (result !== "" /* unknown */) {
                    return result;
                }
                var flags = ts.getCombinedLocalAndExportSymbolFlags(symbol);
                if (flags & 32 /* Class */) {
                    return ts.getDeclarationOfKind(symbol, 214 /* ClassExpression */) ?
                        "local class" /* localClassElement */ : "class" /* classElement */;
                }
                if (flags & 384 /* Enum */)
                    return "enum" /* enumElement */;
                if (flags & 524288 /* TypeAlias */)
                    return "type" /* typeElement */;
                if (flags & 64 /* Interface */)
                    return "interface" /* interfaceElement */;
                if (flags & 262144 /* TypeParameter */)
                    return "type parameter" /* typeParameterElement */;
                if (flags & 8 /* EnumMember */)
                    return "enum member" /* enumMemberElement */;
                if (flags & 2097152 /* Alias */)
                    return "alias" /* alias */;
                if (flags & 1536 /* Module */)
                    return "module" /* moduleElement */;
                return result;
            }
            SymbolDisplay.getSymbolKind = getSymbolKind;
            function getSymbolKindOfConstructorPropertyMethodAccessorFunctionOrVar(typeChecker, symbol, location) {
                var roots = typeChecker.getRootSymbols(symbol);
                // If this is a method from a mapped type, leave as a method so long as it still has a call signature.
                if (roots.length === 1
                    && ts.first(roots).flags & 8192 /* Method */
                    // Ensure the mapped version is still a method, as opposed to `{ [K in keyof I]: number }`.
                    && typeChecker.getTypeOfSymbolAtLocation(symbol, location).getNonNullableType().getCallSignatures().length !== 0) {
                    return "method" /* memberFunctionElement */;
                }
                if (typeChecker.isUndefinedSymbol(symbol)) {
                    return "var" /* variableElement */;
                }
                if (typeChecker.isArgumentsSymbol(symbol)) {
                    return "local var" /* localVariableElement */;
                }
                if (location.kind === 104 /* ThisKeyword */ && ts.isExpression(location)) {
                    return "parameter" /* parameterElement */;
                }
                var flags = ts.getCombinedLocalAndExportSymbolFlags(symbol);
                if (flags & 3 /* Variable */) {
                    if (ts.isFirstDeclarationOfSymbolParameter(symbol)) {
                        return "parameter" /* parameterElement */;
                    }
                    else if (symbol.valueDeclaration && ts.isVarConst(symbol.valueDeclaration)) {
                        return "const" /* constElement */;
                    }
                    else if (ts.forEach(symbol.declarations, ts.isLet)) {
                        return "let" /* letElement */;
                    }
                    return isLocalVariableOrFunction(symbol) ? "local var" /* localVariableElement */ : "var" /* variableElement */;
                }
                if (flags & 16 /* Function */)
                    return isLocalVariableOrFunction(symbol) ? "local function" /* localFunctionElement */ : "function" /* functionElement */;
                if (flags & 32768 /* GetAccessor */)
                    return "getter" /* memberGetAccessorElement */;
                if (flags & 65536 /* SetAccessor */)
                    return "setter" /* memberSetAccessorElement */;
                if (flags & 8192 /* Method */)
                    return "method" /* memberFunctionElement */;
                if (flags & 16384 /* Constructor */)
                    return "constructor" /* constructorImplementationElement */;
                if (flags & 4 /* Property */) {
                    if (flags & 33554432 /* Transient */ && symbol.checkFlags & 6 /* Synthetic */) {
                        // If union property is result of union of non method (property/accessors/variables), it is labeled as property
                        var unionPropertyKind = ts.forEach(typeChecker.getRootSymbols(symbol), function (rootSymbol) {
                            var rootSymbolFlags = rootSymbol.getFlags();
                            if (rootSymbolFlags & (98308 /* PropertyOrAccessor */ | 3 /* Variable */)) {
                                return "property" /* memberVariableElement */;
                            }
                            // May be a Function if this was from `typeof N` with `namespace N { function f();. }`.
                            ts.Debug.assert(!!(rootSymbolFlags & (8192 /* Method */ | 16 /* Function */)));
                        });
                        if (!unionPropertyKind) {
                            // If this was union of all methods,
                            // make sure it has call signatures before we can label it as method
                            var typeOfUnionProperty = typeChecker.getTypeOfSymbolAtLocation(symbol, location);
                            if (typeOfUnionProperty.getCallSignatures().length) {
                                return "method" /* memberFunctionElement */;
                            }
                            return "property" /* memberVariableElement */;
                        }
                        return unionPropertyKind;
                    }
                    // If we requested completions after `x.` at the top-level, we may be at a source file location.
                    switch (location.parent && location.parent.kind) {
                        // If we've typed a character of the attribute name, will be 'JsxAttribute', else will be 'JsxOpeningElement'.
                        case 268 /* JsxOpeningElement */:
                        case 266 /* JsxElement */:
                        case 267 /* JsxSelfClosingElement */:
                            return location.kind === 75 /* Identifier */ ? "property" /* memberVariableElement */ : "JSX attribute" /* jsxAttribute */;
                        case 273 /* JsxAttribute */:
                            return "JSX attribute" /* jsxAttribute */;
                        default:
                            return "property" /* memberVariableElement */;
                    }
                }
                return "" /* unknown */;
            }
            function getSymbolModifiers(symbol) {
                var nodeModifiers = symbol && symbol.declarations && symbol.declarations.length > 0
                    ? ts.getNodeModifiers(symbol.declarations[0])
                    : "" /* none */;
                var symbolModifiers = symbol && symbol.flags & 16777216 /* Optional */ ?
                    "optional" /* optionalModifier */
                    : "" /* none */;
                return nodeModifiers && symbolModifiers ? nodeModifiers + "," + symbolModifiers : nodeModifiers || symbolModifiers;
            }
            SymbolDisplay.getSymbolModifiers = getSymbolModifiers;
            // TODO(drosen): Currently completion entry details passes the SemanticMeaning.All instead of using semanticMeaning of location
            function getSymbolDisplayPartsDocumentationAndSymbolKind(typeChecker, symbol, sourceFile, enclosingDeclaration, location, semanticMeaning, alias) {
                if (semanticMeaning === void 0) {
                    semanticMeaning = ts.getMeaningFromLocation(location);
                }
                var displayParts = [];
                var documentation = [];
                var tags = [];
                var symbolFlags = ts.getCombinedLocalAndExportSymbolFlags(symbol);
                var symbolKind = semanticMeaning & 1 /* Value */ ? getSymbolKindOfConstructorPropertyMethodAccessorFunctionOrVar(typeChecker, symbol, location) : "" /* unknown */;
                var hasAddedSymbolInfo = false;
                var isThisExpression = location.kind === 104 /* ThisKeyword */ && ts.isInExpressionContext(location);
                var type;
                var printer;
                var documentationFromAlias;
                var tagsFromAlias;
                var hasMultipleSignatures = false;
                if (location.kind === 104 /* ThisKeyword */ && !isThisExpression) {
                    return { displayParts: [ts.keywordPart(104 /* ThisKeyword */)], documentation: [], symbolKind: "primitive type" /* primitiveType */, tags: undefined };
                }
                // Class at constructor site need to be shown as constructor apart from property,method, vars
                if (symbolKind !== "" /* unknown */ || symbolFlags & 32 /* Class */ || symbolFlags & 2097152 /* Alias */) {
                    // If it is accessor they are allowed only if location is at name of the accessor
                    if (symbolKind === "getter" /* memberGetAccessorElement */ || symbolKind === "setter" /* memberSetAccessorElement */) {
                        symbolKind = "property" /* memberVariableElement */;
                    }
                    var signature = void 0;
                    type = isThisExpression ? typeChecker.getTypeAtLocation(location) : typeChecker.getTypeOfSymbolAtLocation(symbol.exportSymbol || symbol, location);
                    if (location.parent && location.parent.kind === 194 /* PropertyAccessExpression */) {
                        var right = location.parent.name;
                        // Either the location is on the right of a property access, or on the left and the right is missing
                        if (right === location || (right && right.getFullWidth() === 0)) {
                            location = location.parent;
                        }
                    }
                    // try get the call/construct signature from the type if it matches
                    var callExpressionLike = void 0;
                    if (ts.isCallOrNewExpression(location)) {
                        callExpressionLike = location;
                    }
                    else if (ts.isCallExpressionTarget(location) || ts.isNewExpressionTarget(location)) {
                        callExpressionLike = location.parent;
                    }
                    else if (location.parent && ts.isJsxOpeningLikeElement(location.parent) && ts.isFunctionLike(symbol.valueDeclaration)) {
                        callExpressionLike = location.parent;
                    }
                    if (callExpressionLike) {
                        signature = typeChecker.getResolvedSignature(callExpressionLike); // TODO: GH#18217
                        var useConstructSignatures = callExpressionLike.kind === 197 /* NewExpression */ || (ts.isCallExpression(callExpressionLike) && callExpressionLike.expression.kind === 102 /* SuperKeyword */);
                        var allSignatures = useConstructSignatures ? type.getConstructSignatures() : type.getCallSignatures();
                        if (!ts.contains(allSignatures, signature.target) && !ts.contains(allSignatures, signature)) {
                            // Get the first signature if there is one -- allSignatures may contain
                            // either the original signature or its target, so check for either
                            signature = allSignatures.length ? allSignatures[0] : undefined;
                        }
                        if (signature) {
                            if (useConstructSignatures && (symbolFlags & 32 /* Class */)) {
                                // Constructor
                                symbolKind = "constructor" /* constructorImplementationElement */;
                                addPrefixForAnyFunctionOrVar(type.symbol, symbolKind);
                            }
                            else if (symbolFlags & 2097152 /* Alias */) {
                                symbolKind = "alias" /* alias */;
                                pushSymbolKind(symbolKind);
                                displayParts.push(ts.spacePart());
                                if (useConstructSignatures) {
                                    displayParts.push(ts.keywordPart(99 /* NewKeyword */));
                                    displayParts.push(ts.spacePart());
                                }
                                addFullSymbolName(symbol);
                            }
                            else {
                                addPrefixForAnyFunctionOrVar(symbol, symbolKind);
                            }
                            switch (symbolKind) {
                                case "JSX attribute" /* jsxAttribute */:
                                case "property" /* memberVariableElement */:
                                case "var" /* variableElement */:
                                case "const" /* constElement */:
                                case "let" /* letElement */:
                                case "parameter" /* parameterElement */:
                                case "local var" /* localVariableElement */:
                                    // If it is call or construct signature of lambda's write type name
                                    displayParts.push(ts.punctuationPart(58 /* ColonToken */));
                                    displayParts.push(ts.spacePart());
                                    if (!(ts.getObjectFlags(type) & 16 /* Anonymous */) && type.symbol) {
                                        ts.addRange(displayParts, ts.symbolToDisplayParts(typeChecker, type.symbol, enclosingDeclaration, /*meaning*/ undefined, 4 /* AllowAnyNodeKind */ | 1 /* WriteTypeParametersOrArguments */));
                                        displayParts.push(ts.lineBreakPart());
                                    }
                                    if (useConstructSignatures) {
                                        displayParts.push(ts.keywordPart(99 /* NewKeyword */));
                                        displayParts.push(ts.spacePart());
                                    }
                                    addSignatureDisplayParts(signature, allSignatures, 262144 /* WriteArrowStyleSignature */);
                                    break;
                                default:
                                    // Just signature
                                    addSignatureDisplayParts(signature, allSignatures);
                            }
                            hasAddedSymbolInfo = true;
                            hasMultipleSignatures = allSignatures.length > 1;
                        }
                    }
                    else if ((ts.isNameOfFunctionDeclaration(location) && !(symbolFlags & 98304 /* Accessor */)) || // name of function declaration
                        (location.kind === 129 /* ConstructorKeyword */ && location.parent.kind === 162 /* Constructor */)) { // At constructor keyword of constructor declaration
                        // get the signature from the declaration and write it
                        var functionDeclaration_1 = location.parent;
                        // Use function declaration to write the signatures only if the symbol corresponding to this declaration
                        var locationIsSymbolDeclaration = symbol.declarations && ts.find(symbol.declarations, function (declaration) {
                            return declaration === (location.kind === 129 /* ConstructorKeyword */ ? functionDeclaration_1.parent : functionDeclaration_1);
                        });
                        if (locationIsSymbolDeclaration) {
                            var allSignatures = functionDeclaration_1.kind === 162 /* Constructor */ ? type.getNonNullableType().getConstructSignatures() : type.getNonNullableType().getCallSignatures();
                            if (!typeChecker.isImplementationOfOverload(functionDeclaration_1)) {
                                signature = typeChecker.getSignatureFromDeclaration(functionDeclaration_1); // TODO: GH#18217
                            }
                            else {
                                signature = allSignatures[0];
                            }
                            if (functionDeclaration_1.kind === 162 /* Constructor */) {
                                // show (constructor) Type(...) signature
                                symbolKind = "constructor" /* constructorImplementationElement */;
                                addPrefixForAnyFunctionOrVar(type.symbol, symbolKind);
                            }
                            else {
                                // (function/method) symbol(..signature)
                                addPrefixForAnyFunctionOrVar(functionDeclaration_1.kind === 165 /* CallSignature */ &&
                                    !(type.symbol.flags & 2048 /* TypeLiteral */ || type.symbol.flags & 4096 /* ObjectLiteral */) ? type.symbol : symbol, symbolKind);
                            }
                            addSignatureDisplayParts(signature, allSignatures);
                            hasAddedSymbolInfo = true;
                            hasMultipleSignatures = allSignatures.length > 1;
                        }
                    }
                }
                if (symbolFlags & 32 /* Class */ && !hasAddedSymbolInfo && !isThisExpression) {
                    addAliasPrefixIfNecessary();
                    if (ts.getDeclarationOfKind(symbol, 214 /* ClassExpression */)) {
                        // Special case for class expressions because we would like to indicate that
                        // the class name is local to the class body (similar to function expression)
                        //      (local class) class <className>
                        pushSymbolKind("local class" /* localClassElement */);
                    }
                    else {
                        // Class declaration has name which is not local.
                        displayParts.push(ts.keywordPart(80 /* ClassKeyword */));
                    }
                    displayParts.push(ts.spacePart());
                    addFullSymbolName(symbol);
                    writeTypeParametersOfSymbol(symbol, sourceFile);
                }
                if ((symbolFlags & 64 /* Interface */) && (semanticMeaning & 2 /* Type */)) {
                    prefixNextMeaning();
                    displayParts.push(ts.keywordPart(114 /* InterfaceKeyword */));
                    displayParts.push(ts.spacePart());
                    addFullSymbolName(symbol);
                    writeTypeParametersOfSymbol(symbol, sourceFile);
                }
                if ((symbolFlags & 524288 /* TypeAlias */) && (semanticMeaning & 2 /* Type */)) {
                    prefixNextMeaning();
                    displayParts.push(ts.keywordPart(145 /* TypeKeyword */));
                    displayParts.push(ts.spacePart());
                    addFullSymbolName(symbol);
                    writeTypeParametersOfSymbol(symbol, sourceFile);
                    displayParts.push(ts.spacePart());
                    displayParts.push(ts.operatorPart(62 /* EqualsToken */));
                    displayParts.push(ts.spacePart());
                    ts.addRange(displayParts, ts.typeToDisplayParts(typeChecker, typeChecker.getDeclaredTypeOfSymbol(symbol), enclosingDeclaration, 8388608 /* InTypeAlias */));
                }
                if (symbolFlags & 384 /* Enum */) {
                    prefixNextMeaning();
                    if (ts.some(symbol.declarations, function (d) { return ts.isEnumDeclaration(d) && ts.isEnumConst(d); })) {
                        displayParts.push(ts.keywordPart(81 /* ConstKeyword */));
                        displayParts.push(ts.spacePart());
                    }
                    displayParts.push(ts.keywordPart(88 /* EnumKeyword */));
                    displayParts.push(ts.spacePart());
                    addFullSymbolName(symbol);
                }
                if (symbolFlags & 1536 /* Module */ && !isThisExpression) {
                    prefixNextMeaning();
                    var declaration = ts.getDeclarationOfKind(symbol, 249 /* ModuleDeclaration */);
                    var isNamespace = declaration && declaration.name && declaration.name.kind === 75 /* Identifier */;
                    displayParts.push(ts.keywordPart(isNamespace ? 136 /* NamespaceKeyword */ : 135 /* ModuleKeyword */));
                    displayParts.push(ts.spacePart());
                    addFullSymbolName(symbol);
                }
                if ((symbolFlags & 262144 /* TypeParameter */) && (semanticMeaning & 2 /* Type */)) {
                    prefixNextMeaning();
                    displayParts.push(ts.punctuationPart(20 /* OpenParenToken */));
                    displayParts.push(ts.textPart("type parameter"));
                    displayParts.push(ts.punctuationPart(21 /* CloseParenToken */));
                    displayParts.push(ts.spacePart());
                    addFullSymbolName(symbol);
                    if (symbol.parent) {
                        // Class/Interface type parameter
                        addInPrefix();
                        addFullSymbolName(symbol.parent, enclosingDeclaration);
                        writeTypeParametersOfSymbol(symbol.parent, enclosingDeclaration);
                    }
                    else {
                        // Method/function type parameter
                        var decl = ts.getDeclarationOfKind(symbol, 155 /* TypeParameter */);
                        if (decl === undefined)
                            return ts.Debug.fail();
                        var declaration = decl.parent;
                        if (declaration) {
                            if (ts.isFunctionLikeKind(declaration.kind)) {
                                addInPrefix();
                                var signature = typeChecker.getSignatureFromDeclaration(declaration); // TODO: GH#18217
                                if (declaration.kind === 166 /* ConstructSignature */) {
                                    displayParts.push(ts.keywordPart(99 /* NewKeyword */));
                                    displayParts.push(ts.spacePart());
                                }
                                else if (declaration.kind !== 165 /* CallSignature */ && declaration.name) {
                                    addFullSymbolName(declaration.symbol);
                                }
                                ts.addRange(displayParts, ts.signatureToDisplayParts(typeChecker, signature, sourceFile, 32 /* WriteTypeArgumentsOfSignature */));
                            }
                            else if (declaration.kind === 247 /* TypeAliasDeclaration */) {
                                // Type alias type parameter
                                // For example
                                //      type list<T> = T[]; // Both T will go through same code path
                                addInPrefix();
                                displayParts.push(ts.keywordPart(145 /* TypeKeyword */));
                                displayParts.push(ts.spacePart());
                                addFullSymbolName(declaration.symbol);
                                writeTypeParametersOfSymbol(declaration.symbol, sourceFile);
                            }
                        }
                    }
                }
                if (symbolFlags & 8 /* EnumMember */) {
                    symbolKind = "enum member" /* enumMemberElement */;
                    addPrefixForAnyFunctionOrVar(symbol, "enum member");
                    var declaration = symbol.declarations[0];
                    if (declaration.kind === 284 /* EnumMember */) {
                        var constantValue = typeChecker.getConstantValue(declaration);
                        if (constantValue !== undefined) {
                            displayParts.push(ts.spacePart());
                            displayParts.push(ts.operatorPart(62 /* EqualsToken */));
                            displayParts.push(ts.spacePart());
                            displayParts.push(ts.displayPart(ts.getTextOfConstantValue(constantValue), typeof constantValue === "number" ? ts.SymbolDisplayPartKind.numericLiteral : ts.SymbolDisplayPartKind.stringLiteral));
                        }
                    }
                }
                if (symbolFlags & 2097152 /* Alias */) {
                    prefixNextMeaning();
                    if (!hasAddedSymbolInfo) {
                        var resolvedSymbol = typeChecker.getAliasedSymbol(symbol);
                        if (resolvedSymbol !== symbol && resolvedSymbol.declarations && resolvedSymbol.declarations.length > 0) {
                            var resolvedNode = resolvedSymbol.declarations[0];
                            var declarationName = ts.getNameOfDeclaration(resolvedNode);
                            if (declarationName) {
                                var isExternalModuleDeclaration = ts.isModuleWithStringLiteralName(resolvedNode) &&
                                    ts.hasModifier(resolvedNode, 2 /* Ambient */);
                                var shouldUseAliasName = symbol.name !== "default" && !isExternalModuleDeclaration;
                                var resolvedInfo = getSymbolDisplayPartsDocumentationAndSymbolKind(typeChecker, resolvedSymbol, ts.getSourceFileOfNode(resolvedNode), resolvedNode, declarationName, semanticMeaning, shouldUseAliasName ? symbol : resolvedSymbol);
                                displayParts.push.apply(displayParts, resolvedInfo.displayParts);
                                displayParts.push(ts.lineBreakPart());
                                documentationFromAlias = resolvedInfo.documentation;
                                tagsFromAlias = resolvedInfo.tags;
                            }
                        }
                    }
                    switch (symbol.declarations[0].kind) {
                        case 252 /* NamespaceExportDeclaration */:
                            displayParts.push(ts.keywordPart(89 /* ExportKeyword */));
                            displayParts.push(ts.spacePart());
                            displayParts.push(ts.keywordPart(136 /* NamespaceKeyword */));
                            break;
                        case 259 /* ExportAssignment */:
                            displayParts.push(ts.keywordPart(89 /* ExportKeyword */));
                            displayParts.push(ts.spacePart());
                            displayParts.push(ts.keywordPart(symbol.declarations[0].isExportEquals ? 62 /* EqualsToken */ : 84 /* DefaultKeyword */));
                            break;
                        case 263 /* ExportSpecifier */:
                            displayParts.push(ts.keywordPart(89 /* ExportKeyword */));
                            break;
                        default:
                            displayParts.push(ts.keywordPart(96 /* ImportKeyword */));
                    }
                    displayParts.push(ts.spacePart());
                    addFullSymbolName(symbol);
                    ts.forEach(symbol.declarations, function (declaration) {
                        if (declaration.kind === 253 /* ImportEqualsDeclaration */) {
                            var importEqualsDeclaration = declaration;
                            if (ts.isExternalModuleImportEqualsDeclaration(importEqualsDeclaration)) {
                                displayParts.push(ts.spacePart());
                                displayParts.push(ts.operatorPart(62 /* EqualsToken */));
                                displayParts.push(ts.spacePart());
                                displayParts.push(ts.keywordPart(139 /* RequireKeyword */));
                                displayParts.push(ts.punctuationPart(20 /* OpenParenToken */));
                                displayParts.push(ts.displayPart(ts.getTextOfNode(ts.getExternalModuleImportEqualsDeclarationExpression(importEqualsDeclaration)), ts.SymbolDisplayPartKind.stringLiteral));
                                displayParts.push(ts.punctuationPart(21 /* CloseParenToken */));
                            }
                            else {
                                var internalAliasSymbol = typeChecker.getSymbolAtLocation(importEqualsDeclaration.moduleReference);
                                if (internalAliasSymbol) {
                                    displayParts.push(ts.spacePart());
                                    displayParts.push(ts.operatorPart(62 /* EqualsToken */));
                                    displayParts.push(ts.spacePart());
                                    addFullSymbolName(internalAliasSymbol, enclosingDeclaration);
                                }
                            }
                            return true;
                        }
                    });
                }
                if (!hasAddedSymbolInfo) {
                    if (symbolKind !== "" /* unknown */) {
                        if (type) {
                            if (isThisExpression) {
                                prefixNextMeaning();
                                displayParts.push(ts.keywordPart(104 /* ThisKeyword */));
                            }
                            else {
                                addPrefixForAnyFunctionOrVar(symbol, symbolKind);
                            }
                            // For properties, variables and local vars: show the type
                            if (symbolKind === "property" /* memberVariableElement */ ||
                                symbolKind === "JSX attribute" /* jsxAttribute */ ||
                                symbolFlags & 3 /* Variable */ ||
                                symbolKind === "local var" /* localVariableElement */ ||
                                isThisExpression) {
                                displayParts.push(ts.punctuationPart(58 /* ColonToken */));
                                displayParts.push(ts.spacePart());
                                // If the type is type parameter, format it specially
                                if (type.symbol && type.symbol.flags & 262144 /* TypeParameter */) {
                                    var typeParameterParts = ts.mapToDisplayParts(function (writer) {
                                        var param = typeChecker.typeParameterToDeclaration(type, enclosingDeclaration);
                                        getPrinter().writeNode(4 /* Unspecified */, param, ts.getSourceFileOfNode(ts.getParseTreeNode(enclosingDeclaration)), writer);
                                    });
                                    ts.addRange(displayParts, typeParameterParts);
                                }
                                else {
                                    ts.addRange(displayParts, ts.typeToDisplayParts(typeChecker, type, enclosingDeclaration));
                                }
                            }
                            else if (symbolFlags & 16 /* Function */ ||
                                symbolFlags & 8192 /* Method */ ||
                                symbolFlags & 16384 /* Constructor */ ||
                                symbolFlags & 131072 /* Signature */ ||
                                symbolFlags & 98304 /* Accessor */ ||
                                symbolKind === "method" /* memberFunctionElement */) {
                                var allSignatures = type.getNonNullableType().getCallSignatures();
                                if (allSignatures.length) {
                                    addSignatureDisplayParts(allSignatures[0], allSignatures);
                                    hasMultipleSignatures = allSignatures.length > 1;
                                }
                            }
                        }
                    }
                    else {
                        symbolKind = getSymbolKind(typeChecker, symbol, location);
                    }
                }
                if (documentation.length === 0 && !hasMultipleSignatures) {
                    documentation = symbol.getDocumentationComment(typeChecker);
                }
                if (documentation.length === 0 && symbolFlags & 4 /* Property */) {
                    // For some special property access expressions like `exports.foo = foo` or `module.exports.foo = foo`
                    // there documentation comments might be attached to the right hand side symbol of their declarations.
                    // The pattern of such special property access is that the parent symbol is the symbol of the file.
                    if (symbol.parent && ts.forEach(symbol.parent.declarations, function (declaration) { return declaration.kind === 290 /* SourceFile */; })) {
                        for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                            var declaration = _a[_i];
                            if (!declaration.parent || declaration.parent.kind !== 209 /* BinaryExpression */) {
                                continue;
                            }
                            var rhsSymbol = typeChecker.getSymbolAtLocation(declaration.parent.right);
                            if (!rhsSymbol) {
                                continue;
                            }
                            documentation = rhsSymbol.getDocumentationComment(typeChecker);
                            tags = rhsSymbol.getJsDocTags();
                            if (documentation.length > 0) {
                                break;
                            }
                        }
                    }
                }
                if (tags.length === 0 && !hasMultipleSignatures) {
                    tags = symbol.getJsDocTags();
                }
                if (documentation.length === 0 && documentationFromAlias) {
                    documentation = documentationFromAlias;
                }
                if (tags.length === 0 && tagsFromAlias) {
                    tags = tagsFromAlias;
                }
                return { displayParts: displayParts, documentation: documentation, symbolKind: symbolKind, tags: tags.length === 0 ? undefined : tags };
                function getPrinter() {
                    if (!printer) {
                        printer = ts.createPrinter({ removeComments: true });
                    }
                    return printer;
                }
                function prefixNextMeaning() {
                    if (displayParts.length) {
                        displayParts.push(ts.lineBreakPart());
                    }
                    addAliasPrefixIfNecessary();
                }
                function addAliasPrefixIfNecessary() {
                    if (alias) {
                        pushSymbolKind("alias" /* alias */);
                        displayParts.push(ts.spacePart());
                    }
                }
                function addInPrefix() {
                    displayParts.push(ts.spacePart());
                    displayParts.push(ts.keywordPart(97 /* InKeyword */));
                    displayParts.push(ts.spacePart());
                }
                function addFullSymbolName(symbolToDisplay, enclosingDeclaration) {
                    if (alias && symbolToDisplay === symbol) {
                        symbolToDisplay = alias;
                    }
                    var fullSymbolDisplayParts = ts.symbolToDisplayParts(typeChecker, symbolToDisplay, enclosingDeclaration || sourceFile, /*meaning*/ undefined, 1 /* WriteTypeParametersOrArguments */ | 2 /* UseOnlyExternalAliasing */ | 4 /* AllowAnyNodeKind */);
                    ts.addRange(displayParts, fullSymbolDisplayParts);
                    if (symbol.flags & 16777216 /* Optional */) {
                        displayParts.push(ts.punctuationPart(57 /* QuestionToken */));
                    }
                }
                function addPrefixForAnyFunctionOrVar(symbol, symbolKind) {
                    prefixNextMeaning();
                    if (symbolKind) {
                        pushSymbolKind(symbolKind);
                        if (symbol && !ts.some(symbol.declarations, function (d) { return ts.isArrowFunction(d) || (ts.isFunctionExpression(d) || ts.isClassExpression(d)) && !d.name; })) {
                            displayParts.push(ts.spacePart());
                            addFullSymbolName(symbol);
                        }
                    }
                }
                function pushSymbolKind(symbolKind) {
                    switch (symbolKind) {
                        case "var" /* variableElement */:
                        case "function" /* functionElement */:
                        case "let" /* letElement */:
                        case "const" /* constElement */:
                        case "constructor" /* constructorImplementationElement */:
                            displayParts.push(ts.textOrKeywordPart(symbolKind));
                            return;
                        default:
                            displayParts.push(ts.punctuationPart(20 /* OpenParenToken */));
                            displayParts.push(ts.textOrKeywordPart(symbolKind));
                            displayParts.push(ts.punctuationPart(21 /* CloseParenToken */));
                            return;
                    }
                }
                function addSignatureDisplayParts(signature, allSignatures, flags) {
                    if (flags === void 0) {
                        flags = 0 /* None */;
                    }
                    ts.addRange(displayParts, ts.signatureToDisplayParts(typeChecker, signature, enclosingDeclaration, flags | 32 /* WriteTypeArgumentsOfSignature */));
                    if (allSignatures.length > 1) {
                        displayParts.push(ts.spacePart());
                        displayParts.push(ts.punctuationPart(20 /* OpenParenToken */));
                        displayParts.push(ts.operatorPart(39 /* PlusToken */));
                        displayParts.push(ts.displayPart((allSignatures.length - 1).toString(), ts.SymbolDisplayPartKind.numericLiteral));
                        displayParts.push(ts.spacePart());
                        displayParts.push(ts.textPart(allSignatures.length === 2 ? "overload" : "overloads"));
                        displayParts.push(ts.punctuationPart(21 /* CloseParenToken */));
                    }
                    documentation = signature.getDocumentationComment(typeChecker);
                    tags = signature.getJsDocTags();
                    if (allSignatures.length > 1 && documentation.length === 0 && tags.length === 0) {
                        documentation = allSignatures[0].getDocumentationComment(typeChecker);
                        tags = allSignatures[0].getJsDocTags();
                    }
                }
                function writeTypeParametersOfSymbol(symbol, enclosingDeclaration) {
                    var typeParameterParts = ts.mapToDisplayParts(function (writer) {
                        var params = typeChecker.symbolToTypeParameterDeclarations(symbol, enclosingDeclaration);
                        getPrinter().writeList(53776 /* TypeParameters */, params, ts.getSourceFileOfNode(ts.getParseTreeNode(enclosingDeclaration)), writer);
                    });
                    ts.addRange(displayParts, typeParameterParts);
                }
            }
            SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind = getSymbolDisplayPartsDocumentationAndSymbolKind;
            function isLocalVariableOrFunction(symbol) {
                if (symbol.parent) {
                    return false; // This is exported symbol
                }
                return ts.forEach(symbol.declarations, function (declaration) {
                    // Function expressions are local
                    if (declaration.kind === 201 /* FunctionExpression */) {
                        return true;
                    }
                    if (declaration.kind !== 242 /* VariableDeclaration */ && declaration.kind !== 244 /* FunctionDeclaration */) {
                        return false;
                    }
                    // If the parent is not sourceFile or module block it is local variable
                    for (var parent = declaration.parent; !ts.isFunctionBlock(parent); parent = parent.parent) {
                        // Reached source file or module block
                        if (parent.kind === 290 /* SourceFile */ || parent.kind === 250 /* ModuleBlock */) {
                            return false;
                        }
                    }
                    // parent is in function block
                    return true;
                });
            }
        })(SymbolDisplay = ts.SymbolDisplay || (ts.SymbolDisplay = {}));
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        /*
         * This function will compile source text from 'input' argument using specified compiler options.
         * If not options are provided - it will use a set of default compiler options.
         * Extra compiler options that will unconditionally be used by this function are:
         * - isolatedModules = true
         * - allowNonTsExtensions = true
         * - noLib = true
         * - noResolve = true
         */
        function transpileModule(input, transpileOptions) {
            var diagnostics = [];
            var options = transpileOptions.compilerOptions ? fixupCompilerOptions(transpileOptions.compilerOptions, diagnostics) : {};
            // mix in default options
            var defaultOptions = ts.getDefaultCompilerOptions();
            for (var key in defaultOptions) {
                if (ts.hasProperty(defaultOptions, key) && options[key] === undefined) {
                    options[key] = defaultOptions[key];
                }
            }
            for (var _i = 0, transpileOptionValueCompilerOptions_1 = ts.transpileOptionValueCompilerOptions; _i < transpileOptionValueCompilerOptions_1.length; _i++) {
                var option = transpileOptionValueCompilerOptions_1[_i];
                options[option.name] = option.transpileOptionValue;
            }
            // transpileModule does not write anything to disk so there is no need to verify that there are no conflicts between input and output paths.
            options.suppressOutputPathCheck = true;
            // Filename can be non-ts file.
            options.allowNonTsExtensions = true;
            // if jsx is specified then treat file as .tsx
            var inputFileName = transpileOptions.fileName || (transpileOptions.compilerOptions && transpileOptions.compilerOptions.jsx ? "module.tsx" : "module.ts");
            var sourceFile = ts.createSourceFile(inputFileName, input, options.target); // TODO: GH#18217
            if (transpileOptions.moduleName) {
                sourceFile.moduleName = transpileOptions.moduleName;
            }
            if (transpileOptions.renamedDependencies) {
                sourceFile.renamedDependencies = ts.createMapFromTemplate(transpileOptions.renamedDependencies);
            }
            var newLine = ts.getNewLineCharacter(options);
            // Output
            var outputText;
            var sourceMapText;
            // Create a compilerHost object to allow the compiler to read and write files
            var compilerHost = {
                getSourceFile: function (fileName) { return fileName === ts.normalizePath(inputFileName) ? sourceFile : undefined; },
                writeFile: function (name, text) {
                    if (ts.fileExtensionIs(name, ".map")) {
                        ts.Debug.assertEqual(sourceMapText, undefined, "Unexpected multiple source map outputs, file:", name);
                        sourceMapText = text;
                    }
                    else {
                        ts.Debug.assertEqual(outputText, undefined, "Unexpected multiple outputs, file:", name);
                        outputText = text;
                    }
                },
                getDefaultLibFileName: function () { return "lib.d.ts"; },
                useCaseSensitiveFileNames: function () { return false; },
                getCanonicalFileName: function (fileName) { return fileName; },
                getCurrentDirectory: function () { return ""; },
                getNewLine: function () { return newLine; },
                fileExists: function (fileName) { return fileName === inputFileName; },
                readFile: function () { return ""; },
                directoryExists: function () { return true; },
                getDirectories: function () { return []; }
            };
            var program = ts.createProgram([inputFileName], options, compilerHost);
            if (transpileOptions.reportDiagnostics) {
                ts.addRange(/*to*/ diagnostics, /*from*/ program.getSyntacticDiagnostics(sourceFile));
                ts.addRange(/*to*/ diagnostics, /*from*/ program.getOptionsDiagnostics());
            }
            // Emit
            program.emit(/*targetSourceFile*/ undefined, /*writeFile*/ undefined, /*cancellationToken*/ undefined, /*emitOnlyDtsFiles*/ undefined, transpileOptions.transformers);
            if (outputText === undefined)
                return ts.Debug.fail("Output generation failed");
            return { outputText: outputText, diagnostics: diagnostics, sourceMapText: sourceMapText };
        }
        ts.transpileModule = transpileModule;
        /*
         * This is a shortcut function for transpileModule - it accepts transpileOptions as parameters and returns only outputText part of the result.
         */
        function transpile(input, compilerOptions, fileName, diagnostics, moduleName) {
            var output = transpileModule(input, { compilerOptions: compilerOptions, fileName: fileName, reportDiagnostics: !!diagnostics, moduleName: moduleName });
            // addRange correctly handles cases when wither 'from' or 'to' argument is missing
            ts.addRange(diagnostics, output.diagnostics);
            return output.outputText;
        }
        ts.transpile = transpile;
        var commandLineOptionsStringToEnum;
        /** JS users may pass in string values for enum compiler options (such as ModuleKind), so convert. */
        /*@internal*/
        function fixupCompilerOptions(options, diagnostics) {
            // Lazily create this value to fix module loading errors.
            commandLineOptionsStringToEnum = commandLineOptionsStringToEnum || ts.filter(ts.optionDeclarations, function (o) {
                return typeof o.type === "object" && !ts.forEachEntry(o.type, function (v) { return typeof v !== "number"; });
            });
            options = ts.cloneCompilerOptions(options);
            var _loop_8 = function (opt) {
                if (!ts.hasProperty(options, opt.name)) {
                    return "continue";
                }
                var value = options[opt.name];
                // Value should be a key of opt.type
                if (ts.isString(value)) {
                    // If value is not a string, this will fail
                    options[opt.name] = ts.parseCustomTypeOption(opt, value, diagnostics);
                }
                else {
                    if (!ts.forEachEntry(opt.type, function (v) { return v === value; })) {
                        // Supplied value isn't a valid enum value.
                        diagnostics.push(ts.createCompilerDiagnosticForInvalidCustomType(opt));
                    }
                }
            };
            for (var _i = 0, commandLineOptionsStringToEnum_1 = commandLineOptionsStringToEnum; _i < commandLineOptionsStringToEnum_1.length; _i++) {
                var opt = commandLineOptionsStringToEnum_1[_i];
                _loop_8(opt);
            }
            return options;
        }
        ts.fixupCompilerOptions = fixupCompilerOptions;
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var formatting;
        (function (formatting) {
            var FormattingRequestKind;
            (function (FormattingRequestKind) {
                FormattingRequestKind[FormattingRequestKind["FormatDocument"] = 0] = "FormatDocument";
                FormattingRequestKind[FormattingRequestKind["FormatSelection"] = 1] = "FormatSelection";
                FormattingRequestKind[FormattingRequestKind["FormatOnEnter"] = 2] = "FormatOnEnter";
                FormattingRequestKind[FormattingRequestKind["FormatOnSemicolon"] = 3] = "FormatOnSemicolon";
                FormattingRequestKind[FormattingRequestKind["FormatOnOpeningCurlyBrace"] = 4] = "FormatOnOpeningCurlyBrace";
                FormattingRequestKind[FormattingRequestKind["FormatOnClosingCurlyBrace"] = 5] = "FormatOnClosingCurlyBrace";
            })(FormattingRequestKind = formatting.FormattingRequestKind || (formatting.FormattingRequestKind = {}));
            var FormattingContext = /** @class */ (function () {
                function FormattingContext(sourceFile, formattingRequestKind, options) {
                    this.sourceFile = sourceFile;
                    this.formattingRequestKind = formattingRequestKind;
                    this.options = options;
                }
                FormattingContext.prototype.updateContext = function (currentRange, currentTokenParent, nextRange, nextTokenParent, commonParent) {
                    this.currentTokenSpan = ts.Debug.assertDefined(currentRange);
                    this.currentTokenParent = ts.Debug.assertDefined(currentTokenParent);
                    this.nextTokenSpan = ts.Debug.assertDefined(nextRange);
                    this.nextTokenParent = ts.Debug.assertDefined(nextTokenParent);
                    this.contextNode = ts.Debug.assertDefined(commonParent);
                    // drop cached results
                    this.contextNodeAllOnSameLine = undefined;
                    this.nextNodeAllOnSameLine = undefined;
                    this.tokensAreOnSameLine = undefined;
                    this.contextNodeBlockIsOnOneLine = undefined;
                    this.nextNodeBlockIsOnOneLine = undefined;
                };
                FormattingContext.prototype.ContextNodeAllOnSameLine = function () {
                    if (this.contextNodeAllOnSameLine === undefined) {
                        this.contextNodeAllOnSameLine = this.NodeIsOnOneLine(this.contextNode);
                    }
                    return this.contextNodeAllOnSameLine;
                };
                FormattingContext.prototype.NextNodeAllOnSameLine = function () {
                    if (this.nextNodeAllOnSameLine === undefined) {
                        this.nextNodeAllOnSameLine = this.NodeIsOnOneLine(this.nextTokenParent);
                    }
                    return this.nextNodeAllOnSameLine;
                };
                FormattingContext.prototype.TokensAreOnSameLine = function () {
                    if (this.tokensAreOnSameLine === undefined) {
                        var startLine = this.sourceFile.getLineAndCharacterOfPosition(this.currentTokenSpan.pos).line;
                        var endLine = this.sourceFile.getLineAndCharacterOfPosition(this.nextTokenSpan.pos).line;
                        this.tokensAreOnSameLine = (startLine === endLine);
                    }
                    return this.tokensAreOnSameLine;
                };
                FormattingContext.prototype.ContextNodeBlockIsOnOneLine = function () {
                    if (this.contextNodeBlockIsOnOneLine === undefined) {
                        this.contextNodeBlockIsOnOneLine = this.BlockIsOnOneLine(this.contextNode);
                    }
                    return this.contextNodeBlockIsOnOneLine;
                };
                FormattingContext.prototype.NextNodeBlockIsOnOneLine = function () {
                    if (this.nextNodeBlockIsOnOneLine === undefined) {
                        this.nextNodeBlockIsOnOneLine = this.BlockIsOnOneLine(this.nextTokenParent);
                    }
                    return this.nextNodeBlockIsOnOneLine;
                };
                FormattingContext.prototype.NodeIsOnOneLine = function (node) {
                    var startLine = this.sourceFile.getLineAndCharacterOfPosition(node.getStart(this.sourceFile)).line;
                    var endLine = this.sourceFile.getLineAndCharacterOfPosition(node.getEnd()).line;
                    return startLine === endLine;
                };
                FormattingContext.prototype.BlockIsOnOneLine = function (node) {
                    var openBrace = ts.findChildOfKind(node, 18 /* OpenBraceToken */, this.sourceFile);
                    var closeBrace = ts.findChildOfKind(node, 19 /* CloseBraceToken */, this.sourceFile);
                    if (openBrace && closeBrace) {
                        var startLine = this.sourceFile.getLineAndCharacterOfPosition(openBrace.getEnd()).line;
                        var endLine = this.sourceFile.getLineAndCharacterOfPosition(closeBrace.getStart(this.sourceFile)).line;
                        return startLine === endLine;
                    }
                    return false;
                };
                return FormattingContext;
            }());
            formatting.FormattingContext = FormattingContext;
        })(formatting = ts.formatting || (ts.formatting = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var formatting;
        (function (formatting) {
            var standardScanner = ts.createScanner(99 /* Latest */, /*skipTrivia*/ false, 0 /* Standard */);
            var jsxScanner = ts.createScanner(99 /* Latest */, /*skipTrivia*/ false, 1 /* JSX */);
            var ScanAction;
            (function (ScanAction) {
                ScanAction[ScanAction["Scan"] = 0] = "Scan";
                ScanAction[ScanAction["RescanGreaterThanToken"] = 1] = "RescanGreaterThanToken";
                ScanAction[ScanAction["RescanSlashToken"] = 2] = "RescanSlashToken";
                ScanAction[ScanAction["RescanTemplateToken"] = 3] = "RescanTemplateToken";
                ScanAction[ScanAction["RescanJsxIdentifier"] = 4] = "RescanJsxIdentifier";
                ScanAction[ScanAction["RescanJsxText"] = 5] = "RescanJsxText";
                ScanAction[ScanAction["RescanJsxAttributeValue"] = 6] = "RescanJsxAttributeValue";
            })(ScanAction || (ScanAction = {}));
            function getFormattingScanner(text, languageVariant, startPos, endPos, cb) {
                var scanner = languageVariant === 1 /* JSX */ ? jsxScanner : standardScanner;
                scanner.setText(text);
                scanner.setTextPos(startPos);
                var wasNewLine = true;
                var leadingTrivia;
                var trailingTrivia;
                var savedPos;
                var lastScanAction;
                var lastTokenInfo;
                var res = cb({
                    advance: advance,
                    readTokenInfo: readTokenInfo,
                    readEOFTokenRange: readEOFTokenRange,
                    isOnToken: isOnToken,
                    isOnEOF: isOnEOF,
                    getCurrentLeadingTrivia: function () { return leadingTrivia; },
                    lastTrailingTriviaWasNewLine: function () { return wasNewLine; },
                    skipToEndOf: skipToEndOf,
                });
                lastTokenInfo = undefined;
                scanner.setText(undefined);
                return res;
                function advance() {
                    lastTokenInfo = undefined;
                    var isStarted = scanner.getStartPos() !== startPos;
                    if (isStarted) {
                        wasNewLine = !!trailingTrivia && ts.last(trailingTrivia).kind === 4 /* NewLineTrivia */;
                    }
                    else {
                        scanner.scan();
                    }
                    leadingTrivia = undefined;
                    trailingTrivia = undefined;
                    var pos = scanner.getStartPos();
                    // Read leading trivia and token
                    while (pos < endPos) {
                        var t = scanner.getToken();
                        if (!ts.isTrivia(t)) {
                            break;
                        }
                        // consume leading trivia
                        scanner.scan();
                        var item = {
                            pos: pos,
                            end: scanner.getStartPos(),
                            kind: t
                        };
                        pos = scanner.getStartPos();
                        leadingTrivia = ts.append(leadingTrivia, item);
                    }
                    savedPos = scanner.getStartPos();
                }
                function shouldRescanGreaterThanToken(node) {
                    switch (node.kind) {
                        case 33 /* GreaterThanEqualsToken */:
                        case 70 /* GreaterThanGreaterThanEqualsToken */:
                        case 71 /* GreaterThanGreaterThanGreaterThanEqualsToken */:
                        case 49 /* GreaterThanGreaterThanGreaterThanToken */:
                        case 48 /* GreaterThanGreaterThanToken */:
                            return true;
                    }
                    return false;
                }
                function shouldRescanJsxIdentifier(node) {
                    if (node.parent) {
                        switch (node.parent.kind) {
                            case 273 /* JsxAttribute */:
                            case 268 /* JsxOpeningElement */:
                            case 269 /* JsxClosingElement */:
                            case 267 /* JsxSelfClosingElement */:
                                // May parse an identifier like `module-layout`; that will be scanned as a keyword at first, but we should parse the whole thing to get an identifier.
                                return ts.isKeyword(node.kind) || node.kind === 75 /* Identifier */;
                        }
                    }
                    return false;
                }
                function shouldRescanJsxText(node) {
                    var isJSXText = ts.isJsxText(node);
                    if (isJSXText) {
                        var containingElement = ts.findAncestor(node.parent, function (p) { return ts.isJsxElement(p); });
                        if (!containingElement)
                            return false; // should never happen
                        return !ts.isParenthesizedExpression(containingElement.parent);
                    }
                    return false;
                }
                function shouldRescanSlashToken(container) {
                    return container.kind === 13 /* RegularExpressionLiteral */;
                }
                function shouldRescanTemplateToken(container) {
                    return container.kind === 16 /* TemplateMiddle */ ||
                        container.kind === 17 /* TemplateTail */;
                }
                function shouldRescanJsxAttributeValue(node) {
                    return node.parent && ts.isJsxAttribute(node.parent) && node.parent.initializer === node;
                }
                function startsWithSlashToken(t) {
                    return t === 43 /* SlashToken */ || t === 67 /* SlashEqualsToken */;
                }
                function readTokenInfo(n) {
                    ts.Debug.assert(isOnToken());
                    // normally scanner returns the smallest available token
                    // check the kind of context node to determine if scanner should have more greedy behavior and consume more text.
                    var expectedScanAction = shouldRescanGreaterThanToken(n) ? 1 /* RescanGreaterThanToken */ :
                        shouldRescanSlashToken(n) ? 2 /* RescanSlashToken */ :
                            shouldRescanTemplateToken(n) ? 3 /* RescanTemplateToken */ :
                                shouldRescanJsxIdentifier(n) ? 4 /* RescanJsxIdentifier */ :
                                    shouldRescanJsxText(n) ? 5 /* RescanJsxText */ :
                                        shouldRescanJsxAttributeValue(n) ? 6 /* RescanJsxAttributeValue */ :
                                            0 /* Scan */;
                    if (lastTokenInfo && expectedScanAction === lastScanAction) {
                        // readTokenInfo was called before with the same expected scan action.
                        // No need to re-scan text, return existing 'lastTokenInfo'
                        // it is ok to call fixTokenKind here since it does not affect
                        // what portion of text is consumed. In contrast rescanning can change it,
                        // i.e. for '>=' when originally scanner eats just one character
                        // and rescanning forces it to consume more.
                        return fixTokenKind(lastTokenInfo, n);
                    }
                    if (scanner.getStartPos() !== savedPos) {
                        ts.Debug.assert(lastTokenInfo !== undefined);
                        // readTokenInfo was called before but scan action differs - rescan text
                        scanner.setTextPos(savedPos);
                        scanner.scan();
                    }
                    var currentToken = getNextToken(n, expectedScanAction);
                    var token = formatting.createTextRangeWithKind(scanner.getStartPos(), scanner.getTextPos(), currentToken);
                    // consume trailing trivia
                    if (trailingTrivia) {
                        trailingTrivia = undefined;
                    }
                    while (scanner.getStartPos() < endPos) {
                        currentToken = scanner.scan();
                        if (!ts.isTrivia(currentToken)) {
                            break;
                        }
                        var trivia = formatting.createTextRangeWithKind(scanner.getStartPos(), scanner.getTextPos(), currentToken);
                        if (!trailingTrivia) {
                            trailingTrivia = [];
                        }
                        trailingTrivia.push(trivia);
                        if (currentToken === 4 /* NewLineTrivia */) {
                            // move past new line
                            scanner.scan();
                            break;
                        }
                    }
                    lastTokenInfo = { leadingTrivia: leadingTrivia, trailingTrivia: trailingTrivia, token: token };
                    return fixTokenKind(lastTokenInfo, n);
                }
                function getNextToken(n, expectedScanAction) {
                    var token = scanner.getToken();
                    lastScanAction = 0 /* Scan */;
                    switch (expectedScanAction) {
                        case 1 /* RescanGreaterThanToken */:
                            if (token === 31 /* GreaterThanToken */) {
                                lastScanAction = 1 /* RescanGreaterThanToken */;
                                var newToken = scanner.reScanGreaterToken();
                                ts.Debug.assert(n.kind === newToken);
                                return newToken;
                            }
                            break;
                        case 2 /* RescanSlashToken */:
                            if (startsWithSlashToken(token)) {
                                lastScanAction = 2 /* RescanSlashToken */;
                                var newToken = scanner.reScanSlashToken();
                                ts.Debug.assert(n.kind === newToken);
                                return newToken;
                            }
                            break;
                        case 3 /* RescanTemplateToken */:
                            if (token === 19 /* CloseBraceToken */) {
                                lastScanAction = 3 /* RescanTemplateToken */;
                                return scanner.reScanTemplateToken();
                            }
                            break;
                        case 4 /* RescanJsxIdentifier */:
                            lastScanAction = 4 /* RescanJsxIdentifier */;
                            return scanner.scanJsxIdentifier();
                        case 5 /* RescanJsxText */:
                            lastScanAction = 5 /* RescanJsxText */;
                            return scanner.reScanJsxToken();
                        case 6 /* RescanJsxAttributeValue */:
                            lastScanAction = 6 /* RescanJsxAttributeValue */;
                            return scanner.reScanJsxAttributeValue();
                        case 0 /* Scan */:
                            break;
                        default:
                            ts.Debug.assertNever(expectedScanAction);
                    }
                    return token;
                }
                function readEOFTokenRange() {
                    ts.Debug.assert(isOnEOF());
                    return formatting.createTextRangeWithKind(scanner.getStartPos(), scanner.getTextPos(), 1 /* EndOfFileToken */);
                }
                function isOnToken() {
                    var current = lastTokenInfo ? lastTokenInfo.token.kind : scanner.getToken();
                    var startPos = lastTokenInfo ? lastTokenInfo.token.pos : scanner.getStartPos();
                    return startPos < endPos && current !== 1 /* EndOfFileToken */ && !ts.isTrivia(current);
                }
                function isOnEOF() {
                    var current = lastTokenInfo ? lastTokenInfo.token.kind : scanner.getToken();
                    return current === 1 /* EndOfFileToken */;
                }
                // when containing node in the tree is token
                // but its kind differs from the kind that was returned by the scanner,
                // then kind needs to be fixed. This might happen in cases
                // when parser interprets token differently, i.e keyword treated as identifier
                function fixTokenKind(tokenInfo, container) {
                    if (ts.isToken(container) && tokenInfo.token.kind !== container.kind) {
                        tokenInfo.token.kind = container.kind;
                    }
                    return tokenInfo;
                }
                function skipToEndOf(node) {
                    scanner.setTextPos(node.end);
                    savedPos = scanner.getStartPos();
                    lastScanAction = undefined;
                    lastTokenInfo = undefined;
                    wasNewLine = false;
                    leadingTrivia = undefined;
                    trailingTrivia = undefined;
                }
            }
            formatting.getFormattingScanner = getFormattingScanner;
        })(formatting = ts.formatting || (ts.formatting = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var formatting;
        (function (formatting) {
            formatting.anyContext = ts.emptyArray;
            var RuleAction;
            (function (RuleAction) {
                RuleAction[RuleAction["StopProcessingSpaceActions"] = 1] = "StopProcessingSpaceActions";
                RuleAction[RuleAction["StopProcessingTokenActions"] = 2] = "StopProcessingTokenActions";
                RuleAction[RuleAction["InsertSpace"] = 4] = "InsertSpace";
                RuleAction[RuleAction["InsertNewLine"] = 8] = "InsertNewLine";
                RuleAction[RuleAction["DeleteSpace"] = 16] = "DeleteSpace";
                RuleAction[RuleAction["DeleteToken"] = 32] = "DeleteToken";
                RuleAction[RuleAction["InsertTrailingSemicolon"] = 64] = "InsertTrailingSemicolon";
                RuleAction[RuleAction["StopAction"] = 3] = "StopAction";
                RuleAction[RuleAction["ModifySpaceAction"] = 28] = "ModifySpaceAction";
                RuleAction[RuleAction["ModifyTokenAction"] = 96] = "ModifyTokenAction";
            })(RuleAction = formatting.RuleAction || (formatting.RuleAction = {}));
            var RuleFlags;
            (function (RuleFlags) {
                RuleFlags[RuleFlags["None"] = 0] = "None";
                RuleFlags[RuleFlags["CanDeleteNewLines"] = 1] = "CanDeleteNewLines";
            })(RuleFlags = formatting.RuleFlags || (formatting.RuleFlags = {}));
        })(formatting = ts.formatting || (ts.formatting = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var formatting;
        (function (formatting) {
            function getAllRules() {
                var allTokens = [];
                for (var token = 0 /* FirstToken */; token <= 152 /* LastToken */; token++) {
                    if (token !== 1 /* EndOfFileToken */) {
                        allTokens.push(token);
                    }
                }
                function anyTokenExcept() {
                    var tokens = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        tokens[_i] = arguments[_i];
                    }
                    return { tokens: allTokens.filter(function (t) { return !tokens.some(function (t2) { return t2 === t; }); }), isSpecific: false };
                }
                var anyToken = { tokens: allTokens, isSpecific: false };
                var anyTokenIncludingMultilineComments = tokenRangeFrom(__spreadArrays(allTokens, [3 /* MultiLineCommentTrivia */]));
                var anyTokenIncludingEOF = tokenRangeFrom(__spreadArrays(allTokens, [1 /* EndOfFileToken */]));
                var keywords = tokenRangeFromRange(77 /* FirstKeyword */, 152 /* LastKeyword */);
                var binaryOperators = tokenRangeFromRange(29 /* FirstBinaryOperator */, 74 /* LastBinaryOperator */);
                var binaryKeywordOperators = [97 /* InKeyword */, 98 /* InstanceOfKeyword */, 152 /* OfKeyword */, 123 /* AsKeyword */, 133 /* IsKeyword */];
                var unaryPrefixOperators = [45 /* PlusPlusToken */, 46 /* MinusMinusToken */, 54 /* TildeToken */, 53 /* ExclamationToken */];
                var unaryPrefixExpressions = [
                    8 /* NumericLiteral */, 9 /* BigIntLiteral */, 75 /* Identifier */, 20 /* OpenParenToken */,
                    22 /* OpenBracketToken */, 18 /* OpenBraceToken */, 104 /* ThisKeyword */, 99 /* NewKeyword */
                ];
                var unaryPreincrementExpressions = [75 /* Identifier */, 20 /* OpenParenToken */, 104 /* ThisKeyword */, 99 /* NewKeyword */];
                var unaryPostincrementExpressions = [75 /* Identifier */, 21 /* CloseParenToken */, 23 /* CloseBracketToken */, 99 /* NewKeyword */];
                var unaryPredecrementExpressions = [75 /* Identifier */, 20 /* OpenParenToken */, 104 /* ThisKeyword */, 99 /* NewKeyword */];
                var unaryPostdecrementExpressions = [75 /* Identifier */, 21 /* CloseParenToken */, 23 /* CloseBracketToken */, 99 /* NewKeyword */];
                var comments = [2 /* SingleLineCommentTrivia */, 3 /* MultiLineCommentTrivia */];
                var typeNames = __spreadArrays([75 /* Identifier */], ts.typeKeywords);
                // Place a space before open brace in a function declaration
                // TypeScript: Function can have return types, which can be made of tons of different token kinds
                var functionOpenBraceLeftTokenRange = anyTokenIncludingMultilineComments;
                // Place a space before open brace in a TypeScript declaration that has braces as children (class, module, enum, etc)
                var typeScriptOpenBraceLeftTokenRange = tokenRangeFrom([75 /* Identifier */, 3 /* MultiLineCommentTrivia */, 80 /* ClassKeyword */, 89 /* ExportKeyword */, 96 /* ImportKeyword */]);
                // Place a space before open brace in a control flow construct
                var controlOpenBraceLeftTokenRange = tokenRangeFrom([21 /* CloseParenToken */, 3 /* MultiLineCommentTrivia */, 86 /* DoKeyword */, 107 /* TryKeyword */, 92 /* FinallyKeyword */, 87 /* ElseKeyword */]);
                // These rules are higher in priority than user-configurable
                var highPriorityCommonRules = [
                    // Leave comments alone
                    rule("IgnoreBeforeComment", anyToken, comments, formatting.anyContext, 1 /* StopProcessingSpaceActions */),
                    rule("IgnoreAfterLineComment", 2 /* SingleLineCommentTrivia */, anyToken, formatting.anyContext, 1 /* StopProcessingSpaceActions */),
                    rule("NotSpaceBeforeColon", anyToken, 58 /* ColonToken */, [isNonJsxSameLineTokenContext, isNotBinaryOpContext, isNotTypeAnnotationContext], 16 /* DeleteSpace */),
                    rule("SpaceAfterColon", 58 /* ColonToken */, anyToken, [isNonJsxSameLineTokenContext, isNotBinaryOpContext], 4 /* InsertSpace */),
                    rule("NoSpaceBeforeQuestionMark", anyToken, 57 /* QuestionToken */, [isNonJsxSameLineTokenContext, isNotBinaryOpContext], 16 /* DeleteSpace */),
                    // insert space after '?' only when it is used in conditional operator
                    rule("SpaceAfterQuestionMarkInConditionalOperator", 57 /* QuestionToken */, anyToken, [isNonJsxSameLineTokenContext, isConditionalOperatorContext], 4 /* InsertSpace */),
                    // in other cases there should be no space between '?' and next token
                    rule("NoSpaceAfterQuestionMark", 57 /* QuestionToken */, anyToken, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBeforeDot", anyToken, [24 /* DotToken */, 28 /* QuestionDotToken */], [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterDot", [24 /* DotToken */, 28 /* QuestionDotToken */], anyToken, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBetweenImportParenInImportType", 96 /* ImportKeyword */, 20 /* OpenParenToken */, [isNonJsxSameLineTokenContext, isImportTypeContext], 16 /* DeleteSpace */),
                    // Special handling of unary operators.
                    // Prefix operators generally shouldn't have a space between
                    // them and their target unary expression.
                    rule("NoSpaceAfterUnaryPrefixOperator", unaryPrefixOperators, unaryPrefixExpressions, [isNonJsxSameLineTokenContext, isNotBinaryOpContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterUnaryPreincrementOperator", 45 /* PlusPlusToken */, unaryPreincrementExpressions, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterUnaryPredecrementOperator", 46 /* MinusMinusToken */, unaryPredecrementExpressions, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBeforeUnaryPostincrementOperator", unaryPostincrementExpressions, 45 /* PlusPlusToken */, [isNonJsxSameLineTokenContext, isNotStatementConditionContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBeforeUnaryPostdecrementOperator", unaryPostdecrementExpressions, 46 /* MinusMinusToken */, [isNonJsxSameLineTokenContext, isNotStatementConditionContext], 16 /* DeleteSpace */),
                    // More unary operator special-casing.
                    // DevDiv 181814: Be careful when removing leading whitespace
                    // around unary operators.  Examples:
                    //      1 - -2  --X--> 1--2
                    //      a + ++b --X--> a+++b
                    rule("SpaceAfterPostincrementWhenFollowedByAdd", 45 /* PlusPlusToken */, 39 /* PlusToken */, [isNonJsxSameLineTokenContext, isBinaryOpContext], 4 /* InsertSpace */),
                    rule("SpaceAfterAddWhenFollowedByUnaryPlus", 39 /* PlusToken */, 39 /* PlusToken */, [isNonJsxSameLineTokenContext, isBinaryOpContext], 4 /* InsertSpace */),
                    rule("SpaceAfterAddWhenFollowedByPreincrement", 39 /* PlusToken */, 45 /* PlusPlusToken */, [isNonJsxSameLineTokenContext, isBinaryOpContext], 4 /* InsertSpace */),
                    rule("SpaceAfterPostdecrementWhenFollowedBySubtract", 46 /* MinusMinusToken */, 40 /* MinusToken */, [isNonJsxSameLineTokenContext, isBinaryOpContext], 4 /* InsertSpace */),
                    rule("SpaceAfterSubtractWhenFollowedByUnaryMinus", 40 /* MinusToken */, 40 /* MinusToken */, [isNonJsxSameLineTokenContext, isBinaryOpContext], 4 /* InsertSpace */),
                    rule("SpaceAfterSubtractWhenFollowedByPredecrement", 40 /* MinusToken */, 46 /* MinusMinusToken */, [isNonJsxSameLineTokenContext, isBinaryOpContext], 4 /* InsertSpace */),
                    rule("NoSpaceAfterCloseBrace", 19 /* CloseBraceToken */, [27 /* CommaToken */, 26 /* SemicolonToken */], [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    // For functions and control block place } on a new line [multi-line rule]
                    rule("NewLineBeforeCloseBraceInBlockContext", anyTokenIncludingMultilineComments, 19 /* CloseBraceToken */, [isMultilineBlockContext], 8 /* InsertNewLine */),
                    // Space/new line after }.
                    rule("SpaceAfterCloseBrace", 19 /* CloseBraceToken */, anyTokenExcept(21 /* CloseParenToken */), [isNonJsxSameLineTokenContext, isAfterCodeBlockContext], 4 /* InsertSpace */),
                    // Special case for (}, else) and (}, while) since else & while tokens are not part of the tree which makes SpaceAfterCloseBrace rule not applied
                    // Also should not apply to })
                    rule("SpaceBetweenCloseBraceAndElse", 19 /* CloseBraceToken */, 87 /* ElseKeyword */, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("SpaceBetweenCloseBraceAndWhile", 19 /* CloseBraceToken */, 111 /* WhileKeyword */, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("NoSpaceBetweenEmptyBraceBrackets", 18 /* OpenBraceToken */, 19 /* CloseBraceToken */, [isNonJsxSameLineTokenContext, isObjectContext], 16 /* DeleteSpace */),
                    // Add a space after control dec context if the next character is an open bracket ex: 'if (false)[a, b] = [1, 2];' -> 'if (false) [a, b] = [1, 2];'
                    rule("SpaceAfterConditionalClosingParen", 21 /* CloseParenToken */, 22 /* OpenBracketToken */, [isControlDeclContext], 4 /* InsertSpace */),
                    rule("NoSpaceBetweenFunctionKeywordAndStar", 94 /* FunctionKeyword */, 41 /* AsteriskToken */, [isFunctionDeclarationOrFunctionExpressionContext], 16 /* DeleteSpace */),
                    rule("SpaceAfterStarInGeneratorDeclaration", 41 /* AsteriskToken */, 75 /* Identifier */, [isFunctionDeclarationOrFunctionExpressionContext], 4 /* InsertSpace */),
                    rule("SpaceAfterFunctionInFuncDecl", 94 /* FunctionKeyword */, anyToken, [isFunctionDeclContext], 4 /* InsertSpace */),
                    // Insert new line after { and before } in multi-line contexts.
                    rule("NewLineAfterOpenBraceInBlockContext", 18 /* OpenBraceToken */, anyToken, [isMultilineBlockContext], 8 /* InsertNewLine */),
                    // For get/set members, we check for (identifier,identifier) since get/set don't have tokens and they are represented as just an identifier token.
                    // Though, we do extra check on the context to make sure we are dealing with get/set node. Example:
                    //      get x() {}
                    //      set x(val) {}
                    rule("SpaceAfterGetSetInMember", [131 /* GetKeyword */, 142 /* SetKeyword */], 75 /* Identifier */, [isFunctionDeclContext], 4 /* InsertSpace */),
                    rule("NoSpaceBetweenYieldKeywordAndStar", 121 /* YieldKeyword */, 41 /* AsteriskToken */, [isNonJsxSameLineTokenContext, isYieldOrYieldStarWithOperand], 16 /* DeleteSpace */),
                    rule("SpaceBetweenYieldOrYieldStarAndOperand", [121 /* YieldKeyword */, 41 /* AsteriskToken */], anyToken, [isNonJsxSameLineTokenContext, isYieldOrYieldStarWithOperand], 4 /* InsertSpace */),
                    rule("NoSpaceBetweenReturnAndSemicolon", 101 /* ReturnKeyword */, 26 /* SemicolonToken */, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("SpaceAfterCertainKeywords", [109 /* VarKeyword */, 105 /* ThrowKeyword */, 99 /* NewKeyword */, 85 /* DeleteKeyword */, 101 /* ReturnKeyword */, 108 /* TypeOfKeyword */, 127 /* AwaitKeyword */], anyToken, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("SpaceAfterLetConstInVariableDeclaration", [115 /* LetKeyword */, 81 /* ConstKeyword */], anyToken, [isNonJsxSameLineTokenContext, isStartOfVariableDeclarationList], 4 /* InsertSpace */),
                    rule("NoSpaceBeforeOpenParenInFuncCall", anyToken, 20 /* OpenParenToken */, [isNonJsxSameLineTokenContext, isFunctionCallOrNewContext, isPreviousTokenNotComma], 16 /* DeleteSpace */),
                    // Special case for binary operators (that are keywords). For these we have to add a space and shouldn't follow any user options.
                    rule("SpaceBeforeBinaryKeywordOperator", anyToken, binaryKeywordOperators, [isNonJsxSameLineTokenContext, isBinaryOpContext], 4 /* InsertSpace */),
                    rule("SpaceAfterBinaryKeywordOperator", binaryKeywordOperators, anyToken, [isNonJsxSameLineTokenContext, isBinaryOpContext], 4 /* InsertSpace */),
                    rule("SpaceAfterVoidOperator", 110 /* VoidKeyword */, anyToken, [isNonJsxSameLineTokenContext, isVoidOpContext], 4 /* InsertSpace */),
                    // Async-await
                    rule("SpaceBetweenAsyncAndOpenParen", 126 /* AsyncKeyword */, 20 /* OpenParenToken */, [isArrowFunctionContext, isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("SpaceBetweenAsyncAndFunctionKeyword", 126 /* AsyncKeyword */, 94 /* FunctionKeyword */, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    // Template string
                    rule("NoSpaceBetweenTagAndTemplateString", [75 /* Identifier */, 21 /* CloseParenToken */], [14 /* NoSubstitutionTemplateLiteral */, 15 /* TemplateHead */], [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    // JSX opening elements
                    rule("SpaceBeforeJsxAttribute", anyToken, 75 /* Identifier */, [isNextTokenParentJsxAttribute, isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("SpaceBeforeSlashInJsxOpeningElement", anyToken, 43 /* SlashToken */, [isJsxSelfClosingElementContext, isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("NoSpaceBeforeGreaterThanTokenInJsxOpeningElement", 43 /* SlashToken */, 31 /* GreaterThanToken */, [isJsxSelfClosingElementContext, isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBeforeEqualInJsxAttribute", anyToken, 62 /* EqualsToken */, [isJsxAttributeContext, isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterEqualInJsxAttribute", 62 /* EqualsToken */, anyToken, [isJsxAttributeContext, isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    // TypeScript-specific rules
                    // Use of module as a function call. e.g.: import m2 = module("m2");
                    rule("NoSpaceAfterModuleImport", [135 /* ModuleKeyword */, 139 /* RequireKeyword */], 20 /* OpenParenToken */, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    // Add a space around certain TypeScript keywords
                    rule("SpaceAfterCertainTypeScriptKeywords", [
                        122 /* AbstractKeyword */,
                        80 /* ClassKeyword */,
                        130 /* DeclareKeyword */,
                        84 /* DefaultKeyword */,
                        88 /* EnumKeyword */,
                        89 /* ExportKeyword */,
                        90 /* ExtendsKeyword */,
                        131 /* GetKeyword */,
                        113 /* ImplementsKeyword */,
                        96 /* ImportKeyword */,
                        114 /* InterfaceKeyword */,
                        135 /* ModuleKeyword */,
                        136 /* NamespaceKeyword */,
                        117 /* PrivateKeyword */,
                        119 /* PublicKeyword */,
                        118 /* ProtectedKeyword */,
                        138 /* ReadonlyKeyword */,
                        142 /* SetKeyword */,
                        120 /* StaticKeyword */,
                        145 /* TypeKeyword */,
                        149 /* FromKeyword */,
                        134 /* KeyOfKeyword */,
                        132 /* InferKeyword */,
                    ], anyToken, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("SpaceBeforeCertainTypeScriptKeywords", anyToken, [90 /* ExtendsKeyword */, 113 /* ImplementsKeyword */, 149 /* FromKeyword */], [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    // Treat string literals in module names as identifiers, and add a space between the literal and the opening Brace braces, e.g.: module "m2" {
                    rule("SpaceAfterModuleName", 10 /* StringLiteral */, 18 /* OpenBraceToken */, [isModuleDeclContext], 4 /* InsertSpace */),
                    // Lambda expressions
                    rule("SpaceBeforeArrow", anyToken, 38 /* EqualsGreaterThanToken */, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("SpaceAfterArrow", 38 /* EqualsGreaterThanToken */, anyToken, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    // Optional parameters and let args
                    rule("NoSpaceAfterEllipsis", 25 /* DotDotDotToken */, 75 /* Identifier */, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterOptionalParameters", 57 /* QuestionToken */, [21 /* CloseParenToken */, 27 /* CommaToken */], [isNonJsxSameLineTokenContext, isNotBinaryOpContext], 16 /* DeleteSpace */),
                    // Remove spaces in empty interface literals. e.g.: x: {}
                    rule("NoSpaceBetweenEmptyInterfaceBraceBrackets", 18 /* OpenBraceToken */, 19 /* CloseBraceToken */, [isNonJsxSameLineTokenContext, isObjectTypeContext], 16 /* DeleteSpace */),
                    // generics and type assertions
                    rule("NoSpaceBeforeOpenAngularBracket", typeNames, 29 /* LessThanToken */, [isNonJsxSameLineTokenContext, isTypeArgumentOrParameterOrAssertionContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBetweenCloseParenAndAngularBracket", 21 /* CloseParenToken */, 29 /* LessThanToken */, [isNonJsxSameLineTokenContext, isTypeArgumentOrParameterOrAssertionContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterOpenAngularBracket", 29 /* LessThanToken */, anyToken, [isNonJsxSameLineTokenContext, isTypeArgumentOrParameterOrAssertionContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBeforeCloseAngularBracket", anyToken, 31 /* GreaterThanToken */, [isNonJsxSameLineTokenContext, isTypeArgumentOrParameterOrAssertionContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterCloseAngularBracket", 31 /* GreaterThanToken */, [20 /* OpenParenToken */, 22 /* OpenBracketToken */, 31 /* GreaterThanToken */, 27 /* CommaToken */], [isNonJsxSameLineTokenContext, isTypeArgumentOrParameterOrAssertionContext, isNotFunctionDeclContext /*To prevent an interference with the SpaceBeforeOpenParenInFuncDecl rule*/], 16 /* DeleteSpace */),
                    // decorators
                    rule("SpaceBeforeAt", [21 /* CloseParenToken */, 75 /* Identifier */], 59 /* AtToken */, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("NoSpaceAfterAt", 59 /* AtToken */, anyToken, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    // Insert space after @ in decorator
                    rule("SpaceAfterDecorator", anyToken, [
                        122 /* AbstractKeyword */,
                        75 /* Identifier */,
                        89 /* ExportKeyword */,
                        84 /* DefaultKeyword */,
                        80 /* ClassKeyword */,
                        120 /* StaticKeyword */,
                        119 /* PublicKeyword */,
                        117 /* PrivateKeyword */,
                        118 /* ProtectedKeyword */,
                        131 /* GetKeyword */,
                        142 /* SetKeyword */,
                        22 /* OpenBracketToken */,
                        41 /* AsteriskToken */,
                    ], [isEndOfDecoratorContextOnSameLine], 4 /* InsertSpace */),
                    rule("NoSpaceBeforeNonNullAssertionOperator", anyToken, 53 /* ExclamationToken */, [isNonJsxSameLineTokenContext, isNonNullAssertionContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterNewKeywordOnConstructorSignature", 99 /* NewKeyword */, 20 /* OpenParenToken */, [isNonJsxSameLineTokenContext, isConstructorSignatureContext], 16 /* DeleteSpace */),
                    rule("SpaceLessThanAndNonJSXTypeAnnotation", 29 /* LessThanToken */, 29 /* LessThanToken */, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                ];
                // These rules are applied after high priority
                var userConfigurableRules = [
                    // Treat constructor as an identifier in a function declaration, and remove spaces between constructor and following left parentheses
                    rule("SpaceAfterConstructor", 129 /* ConstructorKeyword */, 20 /* OpenParenToken */, [isOptionEnabled("insertSpaceAfterConstructor"), isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("NoSpaceAfterConstructor", 129 /* ConstructorKeyword */, 20 /* OpenParenToken */, [isOptionDisabledOrUndefined("insertSpaceAfterConstructor"), isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("SpaceAfterComma", 27 /* CommaToken */, anyToken, [isOptionEnabled("insertSpaceAfterCommaDelimiter"), isNonJsxSameLineTokenContext, isNonJsxElementOrFragmentContext, isNextTokenNotCloseBracket], 4 /* InsertSpace */),
                    rule("NoSpaceAfterComma", 27 /* CommaToken */, anyToken, [isOptionDisabledOrUndefined("insertSpaceAfterCommaDelimiter"), isNonJsxSameLineTokenContext, isNonJsxElementOrFragmentContext], 16 /* DeleteSpace */),
                    // Insert space after function keyword for anonymous functions
                    rule("SpaceAfterAnonymousFunctionKeyword", [94 /* FunctionKeyword */, 41 /* AsteriskToken */], 20 /* OpenParenToken */, [isOptionEnabled("insertSpaceAfterFunctionKeywordForAnonymousFunctions"), isFunctionDeclContext], 4 /* InsertSpace */),
                    rule("NoSpaceAfterAnonymousFunctionKeyword", [94 /* FunctionKeyword */, 41 /* AsteriskToken */], 20 /* OpenParenToken */, [isOptionDisabledOrUndefined("insertSpaceAfterFunctionKeywordForAnonymousFunctions"), isFunctionDeclContext], 16 /* DeleteSpace */),
                    // Insert space after keywords in control flow statements
                    rule("SpaceAfterKeywordInControl", keywords, 20 /* OpenParenToken */, [isOptionEnabled("insertSpaceAfterKeywordsInControlFlowStatements"), isControlDeclContext], 4 /* InsertSpace */),
                    rule("NoSpaceAfterKeywordInControl", keywords, 20 /* OpenParenToken */, [isOptionDisabledOrUndefined("insertSpaceAfterKeywordsInControlFlowStatements"), isControlDeclContext], 16 /* DeleteSpace */),
                    // Insert space after opening and before closing nonempty parenthesis
                    rule("SpaceAfterOpenParen", 20 /* OpenParenToken */, anyToken, [isOptionEnabled("insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis"), isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("SpaceBeforeCloseParen", anyToken, 21 /* CloseParenToken */, [isOptionEnabled("insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis"), isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("SpaceBetweenOpenParens", 20 /* OpenParenToken */, 20 /* OpenParenToken */, [isOptionEnabled("insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis"), isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("NoSpaceBetweenParens", 20 /* OpenParenToken */, 21 /* CloseParenToken */, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterOpenParen", 20 /* OpenParenToken */, anyToken, [isOptionDisabledOrUndefined("insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis"), isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBeforeCloseParen", anyToken, 21 /* CloseParenToken */, [isOptionDisabledOrUndefined("insertSpaceAfterOpeningAndBeforeClosingNonemptyParenthesis"), isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    // Insert space after opening and before closing nonempty brackets
                    rule("SpaceAfterOpenBracket", 22 /* OpenBracketToken */, anyToken, [isOptionEnabled("insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets"), isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("SpaceBeforeCloseBracket", anyToken, 23 /* CloseBracketToken */, [isOptionEnabled("insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets"), isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("NoSpaceBetweenBrackets", 22 /* OpenBracketToken */, 23 /* CloseBracketToken */, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterOpenBracket", 22 /* OpenBracketToken */, anyToken, [isOptionDisabledOrUndefined("insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets"), isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBeforeCloseBracket", anyToken, 23 /* CloseBracketToken */, [isOptionDisabledOrUndefined("insertSpaceAfterOpeningAndBeforeClosingNonemptyBrackets"), isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    // Insert a space after { and before } in single-line contexts, but remove space from empty object literals {}.
                    rule("SpaceAfterOpenBrace", 18 /* OpenBraceToken */, anyToken, [isOptionEnabledOrUndefined("insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces"), isBraceWrappedContext], 4 /* InsertSpace */),
                    rule("SpaceBeforeCloseBrace", anyToken, 19 /* CloseBraceToken */, [isOptionEnabledOrUndefined("insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces"), isBraceWrappedContext], 4 /* InsertSpace */),
                    rule("NoSpaceBetweenEmptyBraceBrackets", 18 /* OpenBraceToken */, 19 /* CloseBraceToken */, [isNonJsxSameLineTokenContext, isObjectContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterOpenBrace", 18 /* OpenBraceToken */, anyToken, [isOptionDisabled("insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces"), isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBeforeCloseBrace", anyToken, 19 /* CloseBraceToken */, [isOptionDisabled("insertSpaceAfterOpeningAndBeforeClosingNonemptyBraces"), isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    // Insert space after opening and before closing template string braces
                    rule("SpaceAfterTemplateHeadAndMiddle", [15 /* TemplateHead */, 16 /* TemplateMiddle */], anyToken, [isOptionEnabled("insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces"), isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("SpaceBeforeTemplateMiddleAndTail", anyToken, [16 /* TemplateMiddle */, 17 /* TemplateTail */], [isOptionEnabled("insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces"), isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    rule("NoSpaceAfterTemplateHeadAndMiddle", [15 /* TemplateHead */, 16 /* TemplateMiddle */], anyToken, [isOptionDisabledOrUndefined("insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces"), isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBeforeTemplateMiddleAndTail", anyToken, [16 /* TemplateMiddle */, 17 /* TemplateTail */], [isOptionDisabledOrUndefined("insertSpaceAfterOpeningAndBeforeClosingTemplateStringBraces"), isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    // No space after { and before } in JSX expression
                    rule("SpaceAfterOpenBraceInJsxExpression", 18 /* OpenBraceToken */, anyToken, [isOptionEnabled("insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces"), isNonJsxSameLineTokenContext, isJsxExpressionContext], 4 /* InsertSpace */),
                    rule("SpaceBeforeCloseBraceInJsxExpression", anyToken, 19 /* CloseBraceToken */, [isOptionEnabled("insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces"), isNonJsxSameLineTokenContext, isJsxExpressionContext], 4 /* InsertSpace */),
                    rule("NoSpaceAfterOpenBraceInJsxExpression", 18 /* OpenBraceToken */, anyToken, [isOptionDisabledOrUndefined("insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces"), isNonJsxSameLineTokenContext, isJsxExpressionContext], 16 /* DeleteSpace */),
                    rule("NoSpaceBeforeCloseBraceInJsxExpression", anyToken, 19 /* CloseBraceToken */, [isOptionDisabledOrUndefined("insertSpaceAfterOpeningAndBeforeClosingJsxExpressionBraces"), isNonJsxSameLineTokenContext, isJsxExpressionContext], 16 /* DeleteSpace */),
                    // Insert space after semicolon in for statement
                    rule("SpaceAfterSemicolonInFor", 26 /* SemicolonToken */, anyToken, [isOptionEnabled("insertSpaceAfterSemicolonInForStatements"), isNonJsxSameLineTokenContext, isForContext], 4 /* InsertSpace */),
                    rule("NoSpaceAfterSemicolonInFor", 26 /* SemicolonToken */, anyToken, [isOptionDisabledOrUndefined("insertSpaceAfterSemicolonInForStatements"), isNonJsxSameLineTokenContext, isForContext], 16 /* DeleteSpace */),
                    // Insert space before and after binary operators
                    rule("SpaceBeforeBinaryOperator", anyToken, binaryOperators, [isOptionEnabled("insertSpaceBeforeAndAfterBinaryOperators"), isNonJsxSameLineTokenContext, isBinaryOpContext], 4 /* InsertSpace */),
                    rule("SpaceAfterBinaryOperator", binaryOperators, anyToken, [isOptionEnabled("insertSpaceBeforeAndAfterBinaryOperators"), isNonJsxSameLineTokenContext, isBinaryOpContext], 4 /* InsertSpace */),
                    rule("NoSpaceBeforeBinaryOperator", anyToken, binaryOperators, [isOptionDisabledOrUndefined("insertSpaceBeforeAndAfterBinaryOperators"), isNonJsxSameLineTokenContext, isBinaryOpContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterBinaryOperator", binaryOperators, anyToken, [isOptionDisabledOrUndefined("insertSpaceBeforeAndAfterBinaryOperators"), isNonJsxSameLineTokenContext, isBinaryOpContext], 16 /* DeleteSpace */),
                    rule("SpaceBeforeOpenParenInFuncDecl", anyToken, 20 /* OpenParenToken */, [isOptionEnabled("insertSpaceBeforeFunctionParenthesis"), isNonJsxSameLineTokenContext, isFunctionDeclContext], 4 /* InsertSpace */),
                    rule("NoSpaceBeforeOpenParenInFuncDecl", anyToken, 20 /* OpenParenToken */, [isOptionDisabledOrUndefined("insertSpaceBeforeFunctionParenthesis"), isNonJsxSameLineTokenContext, isFunctionDeclContext], 16 /* DeleteSpace */),
                    // Open Brace braces after control block
                    rule("NewLineBeforeOpenBraceInControl", controlOpenBraceLeftTokenRange, 18 /* OpenBraceToken */, [isOptionEnabled("placeOpenBraceOnNewLineForControlBlocks"), isControlDeclContext, isBeforeMultilineBlockContext], 8 /* InsertNewLine */, 1 /* CanDeleteNewLines */),
                    // Open Brace braces after function
                    // TypeScript: Function can have return types, which can be made of tons of different token kinds
                    rule("NewLineBeforeOpenBraceInFunction", functionOpenBraceLeftTokenRange, 18 /* OpenBraceToken */, [isOptionEnabled("placeOpenBraceOnNewLineForFunctions"), isFunctionDeclContext, isBeforeMultilineBlockContext], 8 /* InsertNewLine */, 1 /* CanDeleteNewLines */),
                    // Open Brace braces after TypeScript module/class/interface
                    rule("NewLineBeforeOpenBraceInTypeScriptDeclWithBlock", typeScriptOpenBraceLeftTokenRange, 18 /* OpenBraceToken */, [isOptionEnabled("placeOpenBraceOnNewLineForFunctions"), isTypeScriptDeclWithBlockContext, isBeforeMultilineBlockContext], 8 /* InsertNewLine */, 1 /* CanDeleteNewLines */),
                    rule("SpaceAfterTypeAssertion", 31 /* GreaterThanToken */, anyToken, [isOptionEnabled("insertSpaceAfterTypeAssertion"), isNonJsxSameLineTokenContext, isTypeAssertionContext], 4 /* InsertSpace */),
                    rule("NoSpaceAfterTypeAssertion", 31 /* GreaterThanToken */, anyToken, [isOptionDisabledOrUndefined("insertSpaceAfterTypeAssertion"), isNonJsxSameLineTokenContext, isTypeAssertionContext], 16 /* DeleteSpace */),
                    rule("SpaceBeforeTypeAnnotation", anyToken, 58 /* ColonToken */, [isOptionEnabled("insertSpaceBeforeTypeAnnotation"), isNonJsxSameLineTokenContext, isTypeAnnotationContext], 4 /* InsertSpace */),
                    rule("NoSpaceBeforeTypeAnnotation", anyToken, 58 /* ColonToken */, [isOptionDisabledOrUndefined("insertSpaceBeforeTypeAnnotation"), isNonJsxSameLineTokenContext, isTypeAnnotationContext], 16 /* DeleteSpace */),
                    rule("NoOptionalSemicolon", 26 /* SemicolonToken */, anyTokenIncludingEOF, [optionEquals("semicolons", ts.SemicolonPreference.Remove), isSemicolonDeletionContext], 32 /* DeleteToken */),
                    rule("OptionalSemicolon", anyToken, anyTokenIncludingEOF, [optionEquals("semicolons", ts.SemicolonPreference.Insert), isSemicolonInsertionContext], 64 /* InsertTrailingSemicolon */),
                ];
                // These rules are lower in priority than user-configurable. Rules earlier in this list have priority over rules later in the list.
                var lowPriorityCommonRules = [
                    // Space after keyword but not before ; or : or ?
                    rule("NoSpaceBeforeSemicolon", anyToken, 26 /* SemicolonToken */, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("SpaceBeforeOpenBraceInControl", controlOpenBraceLeftTokenRange, 18 /* OpenBraceToken */, [isOptionDisabledOrUndefinedOrTokensOnSameLine("placeOpenBraceOnNewLineForControlBlocks"), isControlDeclContext, isNotFormatOnEnter, isSameLineTokenOrBeforeBlockContext], 4 /* InsertSpace */, 1 /* CanDeleteNewLines */),
                    rule("SpaceBeforeOpenBraceInFunction", functionOpenBraceLeftTokenRange, 18 /* OpenBraceToken */, [isOptionDisabledOrUndefinedOrTokensOnSameLine("placeOpenBraceOnNewLineForFunctions"), isFunctionDeclContext, isBeforeBlockContext, isNotFormatOnEnter, isSameLineTokenOrBeforeBlockContext], 4 /* InsertSpace */, 1 /* CanDeleteNewLines */),
                    rule("SpaceBeforeOpenBraceInTypeScriptDeclWithBlock", typeScriptOpenBraceLeftTokenRange, 18 /* OpenBraceToken */, [isOptionDisabledOrUndefinedOrTokensOnSameLine("placeOpenBraceOnNewLineForFunctions"), isTypeScriptDeclWithBlockContext, isNotFormatOnEnter, isSameLineTokenOrBeforeBlockContext], 4 /* InsertSpace */, 1 /* CanDeleteNewLines */),
                    rule("NoSpaceBeforeComma", anyToken, 27 /* CommaToken */, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    // No space before and after indexer `x[]`
                    rule("NoSpaceBeforeOpenBracket", anyTokenExcept(126 /* AsyncKeyword */, 78 /* CaseKeyword */), 22 /* OpenBracketToken */, [isNonJsxSameLineTokenContext], 16 /* DeleteSpace */),
                    rule("NoSpaceAfterCloseBracket", 23 /* CloseBracketToken */, anyToken, [isNonJsxSameLineTokenContext, isNotBeforeBlockInFunctionDeclarationContext], 16 /* DeleteSpace */),
                    rule("SpaceAfterSemicolon", 26 /* SemicolonToken */, anyToken, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    // Remove extra space between for and await
                    rule("SpaceBetweenForAndAwaitKeyword", 93 /* ForKeyword */, 127 /* AwaitKeyword */, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                    // Add a space between statements. All keywords except (do,else,case) has open/close parens after them.
                    // So, we have a rule to add a space for [),Any], [do,Any], [else,Any], and [case,Any]
                    rule("SpaceBetweenStatements", [21 /* CloseParenToken */, 86 /* DoKeyword */, 87 /* ElseKeyword */, 78 /* CaseKeyword */], anyToken, [isNonJsxSameLineTokenContext, isNonJsxElementOrFragmentContext, isNotForContext], 4 /* InsertSpace */),
                    // This low-pri rule takes care of "try {" and "finally {" in case the rule SpaceBeforeOpenBraceInControl didn't execute on FormatOnEnter.
                    rule("SpaceAfterTryFinally", [107 /* TryKeyword */, 92 /* FinallyKeyword */], 18 /* OpenBraceToken */, [isNonJsxSameLineTokenContext], 4 /* InsertSpace */),
                ];
                return __spreadArrays(highPriorityCommonRules, userConfigurableRules, lowPriorityCommonRules);
            }
            formatting.getAllRules = getAllRules;
            /**
             * A rule takes a two tokens (left/right) and a particular context
             * for which you're meant to look at them. You then declare what should the
             * whitespace annotation be between these tokens via the action param.
             *
             * @param debugName Name to print
             * @param left The left side of the comparison
             * @param right The right side of the comparison
             * @param context A set of filters to narrow down the space in which this formatter rule applies
             * @param action a declaration of the expected whitespace
             * @param flags whether the rule deletes a line or not, defaults to no-op
             */
            function rule(debugName, left, right, context, action, flags) {
                if (flags === void 0) {
                    flags = 0 /* None */;
                }
                return { leftTokenRange: toTokenRange(left), rightTokenRange: toTokenRange(right), rule: { debugName: debugName, context: context, action: action, flags: flags } };
            }
            function tokenRangeFrom(tokens) {
                return { tokens: tokens, isSpecific: true };
            }
            function toTokenRange(arg) {
                return typeof arg === "number" ? tokenRangeFrom([arg]) : ts.isArray(arg) ? tokenRangeFrom(arg) : arg;
            }
            function tokenRangeFromRange(from, to, except) {
                if (except === void 0) {
                    except = [];
                }
                var tokens = [];
                for (var token = from; token <= to; token++) {
                    if (!ts.contains(except, token)) {
                        tokens.push(token);
                    }
                }
                return tokenRangeFrom(tokens);
            }
            ///
            /// Contexts
            ///
            function optionEquals(optionName, optionValue) {
                return function (context) { return context.options && context.options[optionName] === optionValue; };
            }
            function isOptionEnabled(optionName) {
                return function (context) { return context.options && context.options.hasOwnProperty(optionName) && !!context.options[optionName]; };
            }
            function isOptionDisabled(optionName) {
                return function (context) { return context.options && context.options.hasOwnProperty(optionName) && !context.options[optionName]; };
            }
            function isOptionDisabledOrUndefined(optionName) {
                return function (context) { return !context.options || !context.options.hasOwnProperty(optionName) || !context.options[optionName]; };
            }
            function isOptionDisabledOrUndefinedOrTokensOnSameLine(optionName) {
                return function (context) { return !context.options || !context.options.hasOwnProperty(optionName) || !context.options[optionName] || context.TokensAreOnSameLine(); };
            }
            function isOptionEnabledOrUndefined(optionName) {
                return function (context) { return !context.options || !context.options.hasOwnProperty(optionName) || !!context.options[optionName]; };
            }
            function isForContext(context) {
                return context.contextNode.kind === 230 /* ForStatement */;
            }
            function isNotForContext(context) {
                return !isForContext(context);
            }
            function isBinaryOpContext(context) {
                switch (context.contextNode.kind) {
                    case 209 /* BinaryExpression */:
                        return context.contextNode.operatorToken.kind !== 27 /* CommaToken */;
                    case 210 /* ConditionalExpression */:
                    case 180 /* ConditionalType */:
                    case 217 /* AsExpression */:
                    case 263 /* ExportSpecifier */:
                    case 258 /* ImportSpecifier */:
                    case 168 /* TypePredicate */:
                    case 178 /* UnionType */:
                    case 179 /* IntersectionType */:
                        return true;
                    // equals in binding elements: function foo([[x, y] = [1, 2]])
                    case 191 /* BindingElement */:
                    // equals in type X = ...
                    // falls through
                    case 247 /* TypeAliasDeclaration */:
                    // equal in import a = module('a');
                    // falls through
                    case 253 /* ImportEqualsDeclaration */:
                    // equal in let a = 0
                    // falls through
                    case 242 /* VariableDeclaration */:
                    // equal in p = 0
                    // falls through
                    case 156 /* Parameter */:
                    case 284 /* EnumMember */:
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                        return context.currentTokenSpan.kind === 62 /* EqualsToken */ || context.nextTokenSpan.kind === 62 /* EqualsToken */;
                    // "in" keyword in for (let x in []) { }
                    case 231 /* ForInStatement */:
                    // "in" keyword in [P in keyof T]: T[P]
                    // falls through
                    case 155 /* TypeParameter */:
                        return context.currentTokenSpan.kind === 97 /* InKeyword */ || context.nextTokenSpan.kind === 97 /* InKeyword */ || context.currentTokenSpan.kind === 62 /* EqualsToken */ || context.nextTokenSpan.kind === 62 /* EqualsToken */;
                    // Technically, "of" is not a binary operator, but format it the same way as "in"
                    case 232 /* ForOfStatement */:
                        return context.currentTokenSpan.kind === 152 /* OfKeyword */ || context.nextTokenSpan.kind === 152 /* OfKeyword */;
                }
                return false;
            }
            function isNotBinaryOpContext(context) {
                return !isBinaryOpContext(context);
            }
            function isNotTypeAnnotationContext(context) {
                return !isTypeAnnotationContext(context);
            }
            function isTypeAnnotationContext(context) {
                var contextKind = context.contextNode.kind;
                return contextKind === 159 /* PropertyDeclaration */ ||
                    contextKind === 158 /* PropertySignature */ ||
                    contextKind === 156 /* Parameter */ ||
                    contextKind === 242 /* VariableDeclaration */ ||
                    ts.isFunctionLikeKind(contextKind);
            }
            function isConditionalOperatorContext(context) {
                return context.contextNode.kind === 210 /* ConditionalExpression */ ||
                    context.contextNode.kind === 180 /* ConditionalType */;
            }
            function isSameLineTokenOrBeforeBlockContext(context) {
                return context.TokensAreOnSameLine() || isBeforeBlockContext(context);
            }
            function isBraceWrappedContext(context) {
                return context.contextNode.kind === 189 /* ObjectBindingPattern */ ||
                    context.contextNode.kind === 186 /* MappedType */ ||
                    isSingleLineBlockContext(context);
            }
            // This check is done before an open brace in a control construct, a function, or a typescript block declaration
            function isBeforeMultilineBlockContext(context) {
                return isBeforeBlockContext(context) && !(context.NextNodeAllOnSameLine() || context.NextNodeBlockIsOnOneLine());
            }
            function isMultilineBlockContext(context) {
                return isBlockContext(context) && !(context.ContextNodeAllOnSameLine() || context.ContextNodeBlockIsOnOneLine());
            }
            function isSingleLineBlockContext(context) {
                return isBlockContext(context) && (context.ContextNodeAllOnSameLine() || context.ContextNodeBlockIsOnOneLine());
            }
            function isBlockContext(context) {
                return nodeIsBlockContext(context.contextNode);
            }
            function isBeforeBlockContext(context) {
                return nodeIsBlockContext(context.nextTokenParent);
            }
            // IMPORTANT!!! This method must return true ONLY for nodes with open and close braces as immediate children
            function nodeIsBlockContext(node) {
                if (nodeIsTypeScriptDeclWithBlockContext(node)) {
                    // This means we are in a context that looks like a block to the user, but in the grammar is actually not a node (it's a class, module, enum, object type literal, etc).
                    return true;
                }
                switch (node.kind) {
                    case 223 /* Block */:
                    case 251 /* CaseBlock */:
                    case 193 /* ObjectLiteralExpression */:
                    case 250 /* ModuleBlock */:
                        return true;
                }
                return false;
            }
            function isFunctionDeclContext(context) {
                switch (context.contextNode.kind) {
                    case 244 /* FunctionDeclaration */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                    // case SyntaxKind.MemberFunctionDeclaration:
                    // falls through
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */:
                    // case SyntaxKind.MethodSignature:
                    // falls through
                    case 165 /* CallSignature */:
                    case 201 /* FunctionExpression */:
                    case 162 /* Constructor */:
                    case 202 /* ArrowFunction */:
                    // case SyntaxKind.ConstructorDeclaration:
                    // case SyntaxKind.SimpleArrowFunctionExpression:
                    // case SyntaxKind.ParenthesizedArrowFunctionExpression:
                    // falls through
                    case 246 /* InterfaceDeclaration */: // This one is not truly a function, but for formatting purposes, it acts just like one
                        return true;
                }
                return false;
            }
            function isNotFunctionDeclContext(context) {
                return !isFunctionDeclContext(context);
            }
            function isFunctionDeclarationOrFunctionExpressionContext(context) {
                return context.contextNode.kind === 244 /* FunctionDeclaration */ || context.contextNode.kind === 201 /* FunctionExpression */;
            }
            function isTypeScriptDeclWithBlockContext(context) {
                return nodeIsTypeScriptDeclWithBlockContext(context.contextNode);
            }
            function nodeIsTypeScriptDeclWithBlockContext(node) {
                switch (node.kind) {
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                    case 246 /* InterfaceDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 173 /* TypeLiteral */:
                    case 249 /* ModuleDeclaration */:
                    case 260 /* ExportDeclaration */:
                    case 261 /* NamedExports */:
                    case 254 /* ImportDeclaration */:
                    case 257 /* NamedImports */:
                        return true;
                }
                return false;
            }
            function isAfterCodeBlockContext(context) {
                switch (context.currentTokenParent.kind) {
                    case 245 /* ClassDeclaration */:
                    case 249 /* ModuleDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 280 /* CatchClause */:
                    case 250 /* ModuleBlock */:
                    case 237 /* SwitchStatement */:
                        return true;
                    case 223 /* Block */: {
                        var blockParent = context.currentTokenParent.parent;
                        // In a codefix scenario, we can't rely on parents being set. So just always return true.
                        if (!blockParent || blockParent.kind !== 202 /* ArrowFunction */ && blockParent.kind !== 201 /* FunctionExpression */) {
                            return true;
                        }
                    }
                }
                return false;
            }
            function isControlDeclContext(context) {
                switch (context.contextNode.kind) {
                    case 227 /* IfStatement */:
                    case 237 /* SwitchStatement */:
                    case 230 /* ForStatement */:
                    case 231 /* ForInStatement */:
                    case 232 /* ForOfStatement */:
                    case 229 /* WhileStatement */:
                    case 240 /* TryStatement */:
                    case 228 /* DoStatement */:
                    case 236 /* WithStatement */:
                    // TODO
                    // case SyntaxKind.ElseClause:
                    // falls through
                    case 280 /* CatchClause */:
                        return true;
                    default:
                        return false;
                }
            }
            function isObjectContext(context) {
                return context.contextNode.kind === 193 /* ObjectLiteralExpression */;
            }
            function isFunctionCallContext(context) {
                return context.contextNode.kind === 196 /* CallExpression */;
            }
            function isNewContext(context) {
                return context.contextNode.kind === 197 /* NewExpression */;
            }
            function isFunctionCallOrNewContext(context) {
                return isFunctionCallContext(context) || isNewContext(context);
            }
            function isPreviousTokenNotComma(context) {
                return context.currentTokenSpan.kind !== 27 /* CommaToken */;
            }
            function isNextTokenNotCloseBracket(context) {
                return context.nextTokenSpan.kind !== 23 /* CloseBracketToken */;
            }
            function isArrowFunctionContext(context) {
                return context.contextNode.kind === 202 /* ArrowFunction */;
            }
            function isImportTypeContext(context) {
                return context.contextNode.kind === 188 /* ImportType */;
            }
            function isNonJsxSameLineTokenContext(context) {
                return context.TokensAreOnSameLine() && context.contextNode.kind !== 11 /* JsxText */;
            }
            function isNonJsxElementOrFragmentContext(context) {
                return context.contextNode.kind !== 266 /* JsxElement */ && context.contextNode.kind !== 270 /* JsxFragment */;
            }
            function isJsxExpressionContext(context) {
                return context.contextNode.kind === 276 /* JsxExpression */ || context.contextNode.kind === 275 /* JsxSpreadAttribute */;
            }
            function isNextTokenParentJsxAttribute(context) {
                return context.nextTokenParent.kind === 273 /* JsxAttribute */;
            }
            function isJsxAttributeContext(context) {
                return context.contextNode.kind === 273 /* JsxAttribute */;
            }
            function isJsxSelfClosingElementContext(context) {
                return context.contextNode.kind === 267 /* JsxSelfClosingElement */;
            }
            function isNotBeforeBlockInFunctionDeclarationContext(context) {
                return !isFunctionDeclContext(context) && !isBeforeBlockContext(context);
            }
            function isEndOfDecoratorContextOnSameLine(context) {
                return context.TokensAreOnSameLine() &&
                    !!context.contextNode.decorators &&
                    nodeIsInDecoratorContext(context.currentTokenParent) &&
                    !nodeIsInDecoratorContext(context.nextTokenParent);
            }
            function nodeIsInDecoratorContext(node) {
                while (ts.isExpressionNode(node)) {
                    node = node.parent;
                }
                return node.kind === 157 /* Decorator */;
            }
            function isStartOfVariableDeclarationList(context) {
                return context.currentTokenParent.kind === 243 /* VariableDeclarationList */ &&
                    context.currentTokenParent.getStart(context.sourceFile) === context.currentTokenSpan.pos;
            }
            function isNotFormatOnEnter(context) {
                return context.formattingRequestKind !== 2 /* FormatOnEnter */;
            }
            function isModuleDeclContext(context) {
                return context.contextNode.kind === 249 /* ModuleDeclaration */;
            }
            function isObjectTypeContext(context) {
                return context.contextNode.kind === 173 /* TypeLiteral */; // && context.contextNode.parent.kind !== SyntaxKind.InterfaceDeclaration;
            }
            function isConstructorSignatureContext(context) {
                return context.contextNode.kind === 166 /* ConstructSignature */;
            }
            function isTypeArgumentOrParameterOrAssertion(token, parent) {
                if (token.kind !== 29 /* LessThanToken */ && token.kind !== 31 /* GreaterThanToken */) {
                    return false;
                }
                switch (parent.kind) {
                    case 169 /* TypeReference */:
                    case 199 /* TypeAssertionExpression */:
                    case 247 /* TypeAliasDeclaration */:
                    case 245 /* ClassDeclaration */:
                    case 214 /* ClassExpression */:
                    case 246 /* InterfaceDeclaration */:
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                    case 165 /* CallSignature */:
                    case 166 /* ConstructSignature */:
                    case 196 /* CallExpression */:
                    case 197 /* NewExpression */:
                    case 216 /* ExpressionWithTypeArguments */:
                        return true;
                    default:
                        return false;
                }
            }
            function isTypeArgumentOrParameterOrAssertionContext(context) {
                return isTypeArgumentOrParameterOrAssertion(context.currentTokenSpan, context.currentTokenParent) ||
                    isTypeArgumentOrParameterOrAssertion(context.nextTokenSpan, context.nextTokenParent);
            }
            function isTypeAssertionContext(context) {
                return context.contextNode.kind === 199 /* TypeAssertionExpression */;
            }
            function isVoidOpContext(context) {
                return context.currentTokenSpan.kind === 110 /* VoidKeyword */ && context.currentTokenParent.kind === 205 /* VoidExpression */;
            }
            function isYieldOrYieldStarWithOperand(context) {
                return context.contextNode.kind === 212 /* YieldExpression */ && context.contextNode.expression !== undefined;
            }
            function isNonNullAssertionContext(context) {
                return context.contextNode.kind === 218 /* NonNullExpression */;
            }
            function isNotStatementConditionContext(context) {
                return !isStatementConditionContext(context);
            }
            function isStatementConditionContext(context) {
                switch (context.contextNode.kind) {
                    case 227 /* IfStatement */:
                    case 230 /* ForStatement */:
                    case 231 /* ForInStatement */:
                    case 232 /* ForOfStatement */:
                    case 228 /* DoStatement */:
                    case 229 /* WhileStatement */:
                        return true;
                    default:
                        return false;
                }
            }
            function isSemicolonDeletionContext(context) {
                var nextTokenKind = context.nextTokenSpan.kind;
                var nextTokenStart = context.nextTokenSpan.pos;
                if (ts.isTrivia(nextTokenKind)) {
                    var nextRealToken = context.nextTokenParent === context.currentTokenParent
                        ? ts.findNextToken(context.currentTokenParent, ts.findAncestor(context.currentTokenParent, function (a) { return !a.parent; }), context.sourceFile)
                        : context.nextTokenParent.getFirstToken(context.sourceFile);
                    if (!nextRealToken) {
                        return true;
                    }
                    nextTokenKind = nextRealToken.kind;
                    nextTokenStart = nextRealToken.getStart(context.sourceFile);
                }
                var startLine = context.sourceFile.getLineAndCharacterOfPosition(context.currentTokenSpan.pos).line;
                var endLine = context.sourceFile.getLineAndCharacterOfPosition(nextTokenStart).line;
                if (startLine === endLine) {
                    return nextTokenKind === 19 /* CloseBraceToken */
                        || nextTokenKind === 1 /* EndOfFileToken */;
                }
                if (nextTokenKind === 222 /* SemicolonClassElement */ ||
                    nextTokenKind === 26 /* SemicolonToken */) {
                    return false;
                }
                if (context.contextNode.kind === 246 /* InterfaceDeclaration */ ||
                    context.contextNode.kind === 247 /* TypeAliasDeclaration */) {
                    // Can’t remove semicolon after `foo`; it would parse as a method declaration:
                    //
                    // interface I {
                    //   foo;
                    //   (): void
                    // }
                    return !ts.isPropertySignature(context.currentTokenParent)
                        || !!context.currentTokenParent.type
                        || nextTokenKind !== 20 /* OpenParenToken */;
                }
                if (ts.isPropertyDeclaration(context.currentTokenParent)) {
                    return !context.currentTokenParent.initializer;
                }
                return context.currentTokenParent.kind !== 230 /* ForStatement */
                    && context.currentTokenParent.kind !== 224 /* EmptyStatement */
                    && context.currentTokenParent.kind !== 222 /* SemicolonClassElement */
                    && nextTokenKind !== 22 /* OpenBracketToken */
                    && nextTokenKind !== 20 /* OpenParenToken */
                    && nextTokenKind !== 39 /* PlusToken */
                    && nextTokenKind !== 40 /* MinusToken */
                    && nextTokenKind !== 43 /* SlashToken */
                    && nextTokenKind !== 13 /* RegularExpressionLiteral */
                    && nextTokenKind !== 27 /* CommaToken */
                    && nextTokenKind !== 211 /* TemplateExpression */
                    && nextTokenKind !== 15 /* TemplateHead */
                    && nextTokenKind !== 14 /* NoSubstitutionTemplateLiteral */
                    && nextTokenKind !== 24 /* DotToken */;
            }
            function isSemicolonInsertionContext(context) {
                return ts.positionIsASICandidate(context.currentTokenSpan.end, context.currentTokenParent, context.sourceFile);
            }
        })(formatting = ts.formatting || (ts.formatting = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var formatting;
        (function (formatting) {
            function getFormatContext(options) {
                return { options: options, getRules: getRulesMap() };
            }
            formatting.getFormatContext = getFormatContext;
            var rulesMapCache;
            function getRulesMap() {
                if (rulesMapCache === undefined) {
                    rulesMapCache = createRulesMap(formatting.getAllRules());
                }
                return rulesMapCache;
            }
            /**
             * For a given rule action, gets a mask of other rule actions that
             * cannot be applied at the same position.
             */
            function getRuleActionExclusion(ruleAction) {
                var mask = 0;
                if (ruleAction & 1 /* StopProcessingSpaceActions */) {
                    mask |= 28 /* ModifySpaceAction */;
                }
                if (ruleAction & 2 /* StopProcessingTokenActions */) {
                    mask |= 96 /* ModifyTokenAction */;
                }
                if (ruleAction & 28 /* ModifySpaceAction */) {
                    mask |= 28 /* ModifySpaceAction */;
                }
                if (ruleAction & 96 /* ModifyTokenAction */) {
                    mask |= 96 /* ModifyTokenAction */;
                }
                return mask;
            }
            function createRulesMap(rules) {
                var map = buildMap(rules);
                return function (context) {
                    var bucket = map[getRuleBucketIndex(context.currentTokenSpan.kind, context.nextTokenSpan.kind)];
                    if (bucket) {
                        var rules_1 = [];
                        var ruleActionMask = 0;
                        for (var _i = 0, bucket_1 = bucket; _i < bucket_1.length; _i++) {
                            var rule = bucket_1[_i];
                            var acceptRuleActions = ~getRuleActionExclusion(ruleActionMask);
                            if (rule.action & acceptRuleActions && ts.every(rule.context, function (c) { return c(context); })) {
                                rules_1.push(rule);
                                ruleActionMask |= rule.action;
                            }
                        }
                        if (rules_1.length) {
                            return rules_1;
                        }
                    }
                };
            }
            function buildMap(rules) {
                // Map from bucket index to array of rules
                var map = new Array(mapRowLength * mapRowLength);
                // This array is used only during construction of the rulesbucket in the map
                var rulesBucketConstructionStateList = new Array(map.length);
                for (var _i = 0, rules_2 = rules; _i < rules_2.length; _i++) {
                    var rule = rules_2[_i];
                    var specificRule = rule.leftTokenRange.isSpecific && rule.rightTokenRange.isSpecific;
                    for (var _a = 0, _b = rule.leftTokenRange.tokens; _a < _b.length; _a++) {
                        var left = _b[_a];
                        for (var _c = 0, _d = rule.rightTokenRange.tokens; _c < _d.length; _c++) {
                            var right = _d[_c];
                            var index = getRuleBucketIndex(left, right);
                            var rulesBucket = map[index];
                            if (rulesBucket === undefined) {
                                rulesBucket = map[index] = [];
                            }
                            addRule(rulesBucket, rule.rule, specificRule, rulesBucketConstructionStateList, index);
                        }
                    }
                }
                return map;
            }
            function getRuleBucketIndex(row, column) {
                ts.Debug.assert(row <= 152 /* LastKeyword */ && column <= 152 /* LastKeyword */, "Must compute formatting context from tokens");
                return (row * mapRowLength) + column;
            }
            var maskBitSize = 5;
            var mask = 31; // MaskBitSize bits
            var mapRowLength = 152 /* LastToken */ + 1;
            var RulesPosition;
            (function (RulesPosition) {
                RulesPosition[RulesPosition["StopRulesSpecific"] = 0] = "StopRulesSpecific";
                RulesPosition[RulesPosition["StopRulesAny"] = maskBitSize * 1] = "StopRulesAny";
                RulesPosition[RulesPosition["ContextRulesSpecific"] = maskBitSize * 2] = "ContextRulesSpecific";
                RulesPosition[RulesPosition["ContextRulesAny"] = maskBitSize * 3] = "ContextRulesAny";
                RulesPosition[RulesPosition["NoContextRulesSpecific"] = maskBitSize * 4] = "NoContextRulesSpecific";
                RulesPosition[RulesPosition["NoContextRulesAny"] = maskBitSize * 5] = "NoContextRulesAny";
            })(RulesPosition || (RulesPosition = {}));
            // The Rules list contains all the inserted rules into a rulebucket in the following order:
            //    1- Ignore rules with specific token combination
            //    2- Ignore rules with any token combination
            //    3- Context rules with specific token combination
            //    4- Context rules with any token combination
            //    5- Non-context rules with specific token combination
            //    6- Non-context rules with any token combination
            //
            // The member rulesInsertionIndexBitmap is used to describe the number of rules
            // in each sub-bucket (above) hence can be used to know the index of where to insert
            // the next rule. It's a bitmap which contains 6 different sections each is given 5 bits.
            //
            // Example:
            // In order to insert a rule to the end of sub-bucket (3), we get the index by adding
            // the values in the bitmap segments 3rd, 2nd, and 1st.
            function addRule(rules, rule, specificTokens, constructionState, rulesBucketIndex) {
                var position = rule.action & 3 /* StopAction */ ?
                    specificTokens ? RulesPosition.StopRulesSpecific : RulesPosition.StopRulesAny :
                    rule.context !== formatting.anyContext ?
                        specificTokens ? RulesPosition.ContextRulesSpecific : RulesPosition.ContextRulesAny :
                        specificTokens ? RulesPosition.NoContextRulesSpecific : RulesPosition.NoContextRulesAny;
                var state = constructionState[rulesBucketIndex] || 0;
                rules.splice(getInsertionIndex(state, position), 0, rule);
                constructionState[rulesBucketIndex] = increaseInsertionIndex(state, position);
            }
            function getInsertionIndex(indexBitmap, maskPosition) {
                var index = 0;
                for (var pos = 0; pos <= maskPosition; pos += maskBitSize) {
                    index += indexBitmap & mask;
                    indexBitmap >>= maskBitSize;
                }
                return index;
            }
            function increaseInsertionIndex(indexBitmap, maskPosition) {
                var value = ((indexBitmap >> maskPosition) & mask) + 1;
                ts.Debug.assert((value & mask) === value, "Adding more rules into the sub-bucket than allowed. Maximum allowed is 32 rules.");
                return (indexBitmap & ~(mask << maskPosition)) | (value << maskPosition);
            }
        })(formatting = ts.formatting || (ts.formatting = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var formatting;
        (function (formatting) {
            function createTextRangeWithKind(pos, end, kind) {
                var textRangeWithKind = { pos: pos, end: end, kind: kind };
                if (ts.Debug.isDebugging) {
                    Object.defineProperty(textRangeWithKind, "__debugKind", {
                        get: function () { return ts.Debug.formatSyntaxKind(kind); },
                    });
                }
                return textRangeWithKind;
            }
            formatting.createTextRangeWithKind = createTextRangeWithKind;
            var Constants;
            (function (Constants) {
                Constants[Constants["Unknown"] = -1] = "Unknown";
            })(Constants || (Constants = {}));
            function formatOnEnter(position, sourceFile, formatContext) {
                var line = sourceFile.getLineAndCharacterOfPosition(position).line;
                if (line === 0) {
                    return [];
                }
                // After the enter key, the cursor is now at a new line. The new line may or may not contain non-whitespace characters.
                // If the new line has only whitespaces, we won't want to format this line, because that would remove the indentation as
                // trailing whitespaces. So the end of the formatting span should be the later one between:
                //  1. the end of the previous line
                //  2. the last non-whitespace character in the current line
                var endOfFormatSpan = ts.getEndLinePosition(line, sourceFile);
                while (ts.isWhiteSpaceSingleLine(sourceFile.text.charCodeAt(endOfFormatSpan))) {
                    endOfFormatSpan--;
                }
                // if the character at the end of the span is a line break, we shouldn't include it, because it indicates we don't want to
                // touch the current line at all. Also, on some OSes the line break consists of two characters (\r\n), we should test if the
                // previous character before the end of format span is line break character as well.
                if (ts.isLineBreak(sourceFile.text.charCodeAt(endOfFormatSpan))) {
                    endOfFormatSpan--;
                }
                var span = {
                    // get start position for the previous line
                    pos: ts.getStartPositionOfLine(line - 1, sourceFile),
                    // end value is exclusive so add 1 to the result
                    end: endOfFormatSpan + 1
                };
                return formatSpan(span, sourceFile, formatContext, 2 /* FormatOnEnter */);
            }
            formatting.formatOnEnter = formatOnEnter;
            function formatOnSemicolon(position, sourceFile, formatContext) {
                var semicolon = findImmediatelyPrecedingTokenOfKind(position, 26 /* SemicolonToken */, sourceFile);
                return formatNodeLines(findOutermostNodeWithinListLevel(semicolon), sourceFile, formatContext, 3 /* FormatOnSemicolon */);
            }
            formatting.formatOnSemicolon = formatOnSemicolon;
            function formatOnOpeningCurly(position, sourceFile, formatContext) {
                var openingCurly = findImmediatelyPrecedingTokenOfKind(position, 18 /* OpenBraceToken */, sourceFile);
                if (!openingCurly) {
                    return [];
                }
                var curlyBraceRange = openingCurly.parent;
                var outermostNode = findOutermostNodeWithinListLevel(curlyBraceRange);
                /**
                 * We limit the span to end at the opening curly to handle the case where
                 * the brace matched to that just typed will be incorrect after further edits.
                 * For example, we could type the opening curly for the following method
                 * body without brace-matching activated:
                 * ```
                 * class C {
                 *     foo()
                 * }
                 * ```
                 * and we wouldn't want to move the closing brace.
                 */
                var textRange = {
                    pos: ts.getLineStartPositionForPosition(outermostNode.getStart(sourceFile), sourceFile),
                    end: position
                };
                return formatSpan(textRange, sourceFile, formatContext, 4 /* FormatOnOpeningCurlyBrace */);
            }
            formatting.formatOnOpeningCurly = formatOnOpeningCurly;
            function formatOnClosingCurly(position, sourceFile, formatContext) {
                var precedingToken = findImmediatelyPrecedingTokenOfKind(position, 19 /* CloseBraceToken */, sourceFile);
                return formatNodeLines(findOutermostNodeWithinListLevel(precedingToken), sourceFile, formatContext, 5 /* FormatOnClosingCurlyBrace */);
            }
            formatting.formatOnClosingCurly = formatOnClosingCurly;
            function formatDocument(sourceFile, formatContext) {
                var span = {
                    pos: 0,
                    end: sourceFile.text.length
                };
                return formatSpan(span, sourceFile, formatContext, 0 /* FormatDocument */);
            }
            formatting.formatDocument = formatDocument;
            function formatSelection(start, end, sourceFile, formatContext) {
                // format from the beginning of the line
                var span = {
                    pos: ts.getLineStartPositionForPosition(start, sourceFile),
                    end: end,
                };
                return formatSpan(span, sourceFile, formatContext, 1 /* FormatSelection */);
            }
            formatting.formatSelection = formatSelection;
            /**
             * Validating `expectedTokenKind` ensures the token was typed in the context we expect (eg: not a comment).
             * @param expectedTokenKind The kind of the last token constituting the desired parent node.
             */
            function findImmediatelyPrecedingTokenOfKind(end, expectedTokenKind, sourceFile) {
                var precedingToken = ts.findPrecedingToken(end, sourceFile);
                return precedingToken && precedingToken.kind === expectedTokenKind && end === precedingToken.getEnd() ?
                    precedingToken :
                    undefined;
            }
            /**
             * Finds the highest node enclosing `node` at the same list level as `node`
             * and whose end does not exceed `node.end`.
             *
             * Consider typing the following
             * ```
             * let x = 1;
             * while (true) {
             * }
             * ```
             * Upon typing the closing curly, we want to format the entire `while`-statement, but not the preceding
             * variable declaration.
             */
            function findOutermostNodeWithinListLevel(node) {
                var current = node;
                while (current &&
                    current.parent &&
                    current.parent.end === node.end &&
                    !isListElement(current.parent, current)) {
                    current = current.parent;
                }
                return current;
            }
            // Returns true if node is a element in some list in parent
            // i.e. parent is class declaration with the list of members and node is one of members.
            function isListElement(parent, node) {
                switch (parent.kind) {
                    case 245 /* ClassDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                        return ts.rangeContainsRange(parent.members, node);
                    case 249 /* ModuleDeclaration */:
                        var body = parent.body;
                        return !!body && body.kind === 250 /* ModuleBlock */ && ts.rangeContainsRange(body.statements, node);
                    case 290 /* SourceFile */:
                    case 223 /* Block */:
                    case 250 /* ModuleBlock */:
                        return ts.rangeContainsRange(parent.statements, node);
                    case 280 /* CatchClause */:
                        return ts.rangeContainsRange(parent.block.statements, node);
                }
                return false;
            }
            /** find node that fully contains given text range */
            function findEnclosingNode(range, sourceFile) {
                return find(sourceFile);
                function find(n) {
                    var candidate = ts.forEachChild(n, function (c) { return ts.startEndContainsRange(c.getStart(sourceFile), c.end, range) && c; });
                    if (candidate) {
                        var result = find(candidate);
                        if (result) {
                            return result;
                        }
                    }
                    return n;
                }
            }
            /** formatting is not applied to ranges that contain parse errors.
             * This function will return a predicate that for a given text range will tell
             * if there are any parse errors that overlap with the range.
             */
            function prepareRangeContainsErrorFunction(errors, originalRange) {
                if (!errors.length) {
                    return rangeHasNoErrors;
                }
                // pick only errors that fall in range
                var sorted = errors
                    .filter(function (d) { return ts.rangeOverlapsWithStartEnd(originalRange, d.start, d.start + d.length); }) // TODO: GH#18217
                    .sort(function (e1, e2) { return e1.start - e2.start; });
                if (!sorted.length) {
                    return rangeHasNoErrors;
                }
                var index = 0;
                return function (r) {
                    // in current implementation sequence of arguments [r1, r2...] is monotonically increasing.
                    // 'index' tracks the index of the most recent error that was checked.
                    while (true) {
                        if (index >= sorted.length) {
                            // all errors in the range were already checked -> no error in specified range
                            return false;
                        }
                        var error = sorted[index];
                        if (r.end <= error.start) {
                            // specified range ends before the error referred by 'index' - no error in range
                            return false;
                        }
                        if (ts.startEndOverlapsWithStartEnd(r.pos, r.end, error.start, error.start + error.length)) {
                            // specified range overlaps with error range
                            return true;
                        }
                        index++;
                    }
                };
                function rangeHasNoErrors() {
                    return false;
                }
            }
            /**
             * Start of the original range might fall inside the comment - scanner will not yield appropriate results
             * This function will look for token that is located before the start of target range
             * and return its end as start position for the scanner.
             */
            function getScanStartPosition(enclosingNode, originalRange, sourceFile) {
                var start = enclosingNode.getStart(sourceFile);
                if (start === originalRange.pos && enclosingNode.end === originalRange.end) {
                    return start;
                }
                var precedingToken = ts.findPrecedingToken(originalRange.pos, sourceFile);
                if (!precedingToken) {
                    // no preceding token found - start from the beginning of enclosing node
                    return enclosingNode.pos;
                }
                // preceding token ends after the start of original range (i.e when originalRange.pos falls in the middle of literal)
                // start from the beginning of enclosingNode to handle the entire 'originalRange'
                if (precedingToken.end >= originalRange.pos) {
                    return enclosingNode.pos;
                }
                return precedingToken.end;
            }
            /*
             * For cases like
             * if (a ||
             *     b ||$
             *     c) {...}
             * If we hit Enter at $ we want line '    b ||' to be indented.
             * Formatting will be applied to the last two lines.
             * Node that fully encloses these lines is binary expression 'a ||...'.
             * Initial indentation for this node will be 0.
             * Binary expressions don't introduce new indentation scopes, however it is possible
             * that some parent node on the same line does - like if statement in this case.
             * Note that we are considering parents only from the same line with initial node -
             * if parent is on the different line - its delta was already contributed
             * to the initial indentation.
             */
            function getOwnOrInheritedDelta(n, options, sourceFile) {
                var previousLine = -1 /* Unknown */;
                var child;
                while (n) {
                    var line = sourceFile.getLineAndCharacterOfPosition(n.getStart(sourceFile)).line;
                    if (previousLine !== -1 /* Unknown */ && line !== previousLine) {
                        break;
                    }
                    if (formatting.SmartIndenter.shouldIndentChildNode(options, n, child, sourceFile)) {
                        return options.indentSize;
                    }
                    previousLine = line;
                    child = n;
                    n = n.parent;
                }
                return 0;
            }
            function formatNodeGivenIndentation(node, sourceFileLike, languageVariant, initialIndentation, delta, formatContext) {
                var range = { pos: 0, end: sourceFileLike.text.length };
                return formatting.getFormattingScanner(sourceFileLike.text, languageVariant, range.pos, range.end, function (scanner) {
                    return formatSpanWorker(range, node, initialIndentation, delta, scanner, formatContext, 1 /* FormatSelection */, function (_) { return false; }, // assume that node does not have any errors
                    sourceFileLike);
                });
            }
            formatting.formatNodeGivenIndentation = formatNodeGivenIndentation;
            function formatNodeLines(node, sourceFile, formatContext, requestKind) {
                if (!node) {
                    return [];
                }
                var span = {
                    pos: ts.getLineStartPositionForPosition(node.getStart(sourceFile), sourceFile),
                    end: node.end
                };
                return formatSpan(span, sourceFile, formatContext, requestKind);
            }
            function formatSpan(originalRange, sourceFile, formatContext, requestKind) {
                // find the smallest node that fully wraps the range and compute the initial indentation for the node
                var enclosingNode = findEnclosingNode(originalRange, sourceFile);
                return formatting.getFormattingScanner(sourceFile.text, sourceFile.languageVariant, getScanStartPosition(enclosingNode, originalRange, sourceFile), originalRange.end, function (scanner) { return formatSpanWorker(originalRange, enclosingNode, formatting.SmartIndenter.getIndentationForNode(enclosingNode, originalRange, sourceFile, formatContext.options), getOwnOrInheritedDelta(enclosingNode, formatContext.options, sourceFile), scanner, formatContext, requestKind, prepareRangeContainsErrorFunction(sourceFile.parseDiagnostics, originalRange), sourceFile); });
            }
            function formatSpanWorker(originalRange, enclosingNode, initialIndentation, delta, formattingScanner, _a, requestKind, rangeContainsError, sourceFile) {
                var options = _a.options, getRules = _a.getRules;
                // formatting context is used by rules provider
                var formattingContext = new formatting.FormattingContext(sourceFile, requestKind, options);
                var previousRange;
                var previousParent;
                var previousRangeStartLine;
                var lastIndentedLine;
                var indentationOnLastIndentedLine = -1 /* Unknown */;
                var edits = [];
                formattingScanner.advance();
                if (formattingScanner.isOnToken()) {
                    var startLine = sourceFile.getLineAndCharacterOfPosition(enclosingNode.getStart(sourceFile)).line;
                    var undecoratedStartLine = startLine;
                    if (enclosingNode.decorators) {
                        undecoratedStartLine = sourceFile.getLineAndCharacterOfPosition(ts.getNonDecoratorTokenPosOfNode(enclosingNode, sourceFile)).line;
                    }
                    processNode(enclosingNode, enclosingNode, startLine, undecoratedStartLine, initialIndentation, delta);
                }
                if (!formattingScanner.isOnToken()) {
                    var leadingTrivia = formattingScanner.getCurrentLeadingTrivia();
                    if (leadingTrivia) {
                        indentTriviaItems(leadingTrivia, initialIndentation, /*indentNextTokenOrTrivia*/ false, function (item) { return processRange(item, sourceFile.getLineAndCharacterOfPosition(item.pos), enclosingNode, enclosingNode, /*dynamicIndentation*/ undefined); });
                        trimTrailingWhitespacesForRemainingRange();
                    }
                }
                return edits;
                // local functions
                /** Tries to compute the indentation for a list element.
                 * If list element is not in range then
                 * function will pick its actual indentation
                 * so it can be pushed downstream as inherited indentation.
                 * If list element is in the range - its indentation will be equal
                 * to inherited indentation from its predecessors.
                 */
                function tryComputeIndentationForListItem(startPos, endPos, parentStartLine, range, inheritedIndentation) {
                    if (ts.rangeOverlapsWithStartEnd(range, startPos, endPos) ||
                        ts.rangeContainsStartEnd(range, startPos, endPos) /* Not to miss zero-range nodes e.g. JsxText */) {
                        if (inheritedIndentation !== -1 /* Unknown */) {
                            return inheritedIndentation;
                        }
                    }
                    else {
                        var startLine = sourceFile.getLineAndCharacterOfPosition(startPos).line;
                        var startLinePosition = ts.getLineStartPositionForPosition(startPos, sourceFile);
                        var column = formatting.SmartIndenter.findFirstNonWhitespaceColumn(startLinePosition, startPos, sourceFile, options);
                        if (startLine !== parentStartLine || startPos === column) {
                            // Use the base indent size if it is greater than
                            // the indentation of the inherited predecessor.
                            var baseIndentSize = formatting.SmartIndenter.getBaseIndentation(options);
                            return baseIndentSize > column ? baseIndentSize : column;
                        }
                    }
                    return -1 /* Unknown */;
                }
                function computeIndentation(node, startLine, inheritedIndentation, parent, parentDynamicIndentation, effectiveParentStartLine) {
                    var delta = formatting.SmartIndenter.shouldIndentChildNode(options, node) ? options.indentSize : 0;
                    if (effectiveParentStartLine === startLine) {
                        // if node is located on the same line with the parent
                        // - inherit indentation from the parent
                        // - push children if either parent of node itself has non-zero delta
                        return {
                            indentation: startLine === lastIndentedLine ? indentationOnLastIndentedLine : parentDynamicIndentation.getIndentation(),
                            delta: Math.min(options.indentSize, parentDynamicIndentation.getDelta(node) + delta)
                        };
                    }
                    else if (inheritedIndentation === -1 /* Unknown */) {
                        if (node.kind === 20 /* OpenParenToken */ && startLine === lastIndentedLine) {
                            // the is used for chaining methods formatting
                            // - we need to get the indentation on last line and the delta of parent
                            return { indentation: indentationOnLastIndentedLine, delta: parentDynamicIndentation.getDelta(node) };
                        }
                        else if (formatting.SmartIndenter.childStartsOnTheSameLineWithElseInIfStatement(parent, node, startLine, sourceFile)) {
                            return { indentation: parentDynamicIndentation.getIndentation(), delta: delta };
                        }
                        else if (formatting.SmartIndenter.argumentStartsOnSameLineAsPreviousArgument(parent, node, startLine, sourceFile)) {
                            return { indentation: parentDynamicIndentation.getIndentation(), delta: delta };
                        }
                        else {
                            return { indentation: parentDynamicIndentation.getIndentation() + parentDynamicIndentation.getDelta(node), delta: delta };
                        }
                    }
                    else {
                        return { indentation: inheritedIndentation, delta: delta };
                    }
                }
                function getFirstNonDecoratorTokenOfNode(node) {
                    if (node.modifiers && node.modifiers.length) {
                        return node.modifiers[0].kind;
                    }
                    switch (node.kind) {
                        case 245 /* ClassDeclaration */: return 80 /* ClassKeyword */;
                        case 246 /* InterfaceDeclaration */: return 114 /* InterfaceKeyword */;
                        case 244 /* FunctionDeclaration */: return 94 /* FunctionKeyword */;
                        case 248 /* EnumDeclaration */: return 248 /* EnumDeclaration */;
                        case 163 /* GetAccessor */: return 131 /* GetKeyword */;
                        case 164 /* SetAccessor */: return 142 /* SetKeyword */;
                        case 161 /* MethodDeclaration */:
                            if (node.asteriskToken) {
                                return 41 /* AsteriskToken */;
                            }
                        // falls through
                        case 159 /* PropertyDeclaration */:
                        case 156 /* Parameter */:
                            var name = ts.getNameOfDeclaration(node);
                            if (name) {
                                return name.kind;
                            }
                    }
                }
                function getDynamicIndentation(node, nodeStartLine, indentation, delta) {
                    return {
                        getIndentationForComment: function (kind, tokenIndentation, container) {
                            switch (kind) {
                                // preceding comment to the token that closes the indentation scope inherits the indentation from the scope
                                // ..  {
                                //     // comment
                                // }
                                case 19 /* CloseBraceToken */:
                                case 23 /* CloseBracketToken */:
                                case 21 /* CloseParenToken */:
                                    return indentation + getDelta(container);
                            }
                            return tokenIndentation !== -1 /* Unknown */ ? tokenIndentation : indentation;
                        },
                        // if list end token is LessThanToken '>' then its delta should be explicitly suppressed
                        // so that LessThanToken as a binary operator can still be indented.
                        // foo.then
                        //     <
                        //         number,
                        //         string,
                        //     >();
                        // vs
                        // var a = xValue
                        //     > yValue;
                        getIndentationForToken: function (line, kind, container, suppressDelta) {
                            return !suppressDelta && shouldAddDelta(line, kind, container) ? indentation + getDelta(container) : indentation;
                        },
                        getIndentation: function () { return indentation; },
                        getDelta: getDelta,
                        recomputeIndentation: function (lineAdded) {
                            if (node.parent && formatting.SmartIndenter.shouldIndentChildNode(options, node.parent, node, sourceFile)) {
                                indentation += lineAdded ? options.indentSize : -options.indentSize;
                                delta = formatting.SmartIndenter.shouldIndentChildNode(options, node) ? options.indentSize : 0;
                            }
                        }
                    };
                    function shouldAddDelta(line, kind, container) {
                        switch (kind) {
                            // open and close brace, 'else' and 'while' (in do statement) tokens has indentation of the parent
                            case 18 /* OpenBraceToken */:
                            case 19 /* CloseBraceToken */:
                            case 21 /* CloseParenToken */:
                            case 87 /* ElseKeyword */:
                            case 111 /* WhileKeyword */:
                            case 59 /* AtToken */:
                                return false;
                            case 43 /* SlashToken */:
                            case 31 /* GreaterThanToken */:
                                switch (container.kind) {
                                    case 268 /* JsxOpeningElement */:
                                    case 269 /* JsxClosingElement */:
                                    case 267 /* JsxSelfClosingElement */:
                                        return false;
                                }
                                break;
                            case 22 /* OpenBracketToken */:
                            case 23 /* CloseBracketToken */:
                                if (container.kind !== 186 /* MappedType */) {
                                    return false;
                                }
                                break;
                        }
                        // if token line equals to the line of containing node (this is a first token in the node) - use node indentation
                        return nodeStartLine !== line
                            // if this token is the first token following the list of decorators, we do not need to indent
                            && !(node.decorators && kind === getFirstNonDecoratorTokenOfNode(node));
                    }
                    function getDelta(child) {
                        // Delta value should be zero when the node explicitly prevents indentation of the child node
                        return formatting.SmartIndenter.nodeWillIndentChild(options, node, child, sourceFile, /*indentByDefault*/ true) ? delta : 0;
                    }
                }
                function processNode(node, contextNode, nodeStartLine, undecoratedNodeStartLine, indentation, delta) {
                    if (!ts.rangeOverlapsWithStartEnd(originalRange, node.getStart(sourceFile), node.getEnd())) {
                        return;
                    }
                    var nodeDynamicIndentation = getDynamicIndentation(node, nodeStartLine, indentation, delta);
                    // a useful observations when tracking context node
                    //        /
                    //      [a]
                    //   /   |   \
                    //  [b] [c] [d]
                    // node 'a' is a context node for nodes 'b', 'c', 'd'
                    // except for the leftmost leaf token in [b] - in this case context node ('e') is located somewhere above 'a'
                    // this rule can be applied recursively to child nodes of 'a'.
                    //
                    // context node is set to parent node value after processing every child node
                    // context node is set to parent of the token after processing every token
                    var childContextNode = contextNode;
                    // if there are any tokens that logically belong to node and interleave child nodes
                    // such tokens will be consumed in processChildNode for the child that follows them
                    ts.forEachChild(node, function (child) {
                        processChildNode(child, /*inheritedIndentation*/ -1 /* Unknown */, node, nodeDynamicIndentation, nodeStartLine, undecoratedNodeStartLine, /*isListItem*/ false);
                    }, function (nodes) {
                        processChildNodes(nodes, node, nodeStartLine, nodeDynamicIndentation);
                    });
                    // proceed any tokens in the node that are located after child nodes
                    while (formattingScanner.isOnToken()) {
                        var tokenInfo = formattingScanner.readTokenInfo(node);
                        if (tokenInfo.token.end > node.end) {
                            break;
                        }
                        if (node.kind === 11 /* JsxText */) {
                            // Intentation rules for jsx text are handled by `indentMultilineCommentOrJsxText` inside `processChildNode`; just fastforward past it here
                            formattingScanner.advance();
                            continue;
                        }
                        consumeTokenAndAdvanceScanner(tokenInfo, node, nodeDynamicIndentation, node);
                    }
                    if (!node.parent && formattingScanner.isOnEOF()) {
                        var token = formattingScanner.readEOFTokenRange();
                        if (token.end <= node.end && previousRange) {
                            processPair(token, sourceFile.getLineAndCharacterOfPosition(token.pos).line, node, previousRange, previousRangeStartLine, previousParent, contextNode, nodeDynamicIndentation);
                        }
                    }
                    function processChildNode(child, inheritedIndentation, parent, parentDynamicIndentation, parentStartLine, undecoratedParentStartLine, isListItem, isFirstListItem) {
                        var childStartPos = child.getStart(sourceFile);
                        var childStartLine = sourceFile.getLineAndCharacterOfPosition(childStartPos).line;
                        var undecoratedChildStartLine = childStartLine;
                        if (child.decorators) {
                            undecoratedChildStartLine = sourceFile.getLineAndCharacterOfPosition(ts.getNonDecoratorTokenPosOfNode(child, sourceFile)).line;
                        }
                        // if child is a list item - try to get its indentation, only if parent is within the original range.
                        var childIndentationAmount = -1 /* Unknown */;
                        if (isListItem && ts.rangeContainsRange(originalRange, parent)) {
                            childIndentationAmount = tryComputeIndentationForListItem(childStartPos, child.end, parentStartLine, originalRange, inheritedIndentation);
                            if (childIndentationAmount !== -1 /* Unknown */) {
                                inheritedIndentation = childIndentationAmount;
                            }
                        }
                        // child node is outside the target range - do not dive inside
                        if (!ts.rangeOverlapsWithStartEnd(originalRange, child.pos, child.end)) {
                            if (child.end < originalRange.pos) {
                                formattingScanner.skipToEndOf(child);
                            }
                            return inheritedIndentation;
                        }
                        if (child.getFullWidth() === 0) {
                            return inheritedIndentation;
                        }
                        while (formattingScanner.isOnToken()) {
                            // proceed any parent tokens that are located prior to child.getStart()
                            var tokenInfo = formattingScanner.readTokenInfo(node);
                            if (tokenInfo.token.end > childStartPos) {
                                // stop when formatting scanner advances past the beginning of the child
                                break;
                            }
                            consumeTokenAndAdvanceScanner(tokenInfo, node, parentDynamicIndentation, node);
                        }
                        if (!formattingScanner.isOnToken()) {
                            return inheritedIndentation;
                        }
                        // JSX text shouldn't affect indenting
                        if (ts.isToken(child) && child.kind !== 11 /* JsxText */) {
                            // if child node is a token, it does not impact indentation, proceed it using parent indentation scope rules
                            var tokenInfo = formattingScanner.readTokenInfo(child);
                            ts.Debug.assert(tokenInfo.token.end === child.end, "Token end is child end");
                            consumeTokenAndAdvanceScanner(tokenInfo, node, parentDynamicIndentation, child);
                            return inheritedIndentation;
                        }
                        var effectiveParentStartLine = child.kind === 157 /* Decorator */ ? childStartLine : undecoratedParentStartLine;
                        var childIndentation = computeIndentation(child, childStartLine, childIndentationAmount, node, parentDynamicIndentation, effectiveParentStartLine);
                        processNode(child, childContextNode, childStartLine, undecoratedChildStartLine, childIndentation.indentation, childIndentation.delta);
                        if (child.kind === 11 /* JsxText */) {
                            var range = { pos: child.getStart(), end: child.getEnd() };
                            if (range.pos !== range.end) { // don't indent zero-width jsx text
                                var siblings = parent.getChildren(sourceFile);
                                var currentIndex = ts.findIndex(siblings, function (arg) { return arg.pos === child.pos; });
                                var previousNode = siblings[currentIndex - 1];
                                if (previousNode) {
                                    // The jsx text needs no indentation whatsoever if it ends on the same line the previous sibling ends on
                                    if (sourceFile.getLineAndCharacterOfPosition(range.end).line !== sourceFile.getLineAndCharacterOfPosition(previousNode.end).line) {
                                        // The first line is (already) "indented" if the text starts on the same line as the previous sibling element ends on
                                        var firstLineIsIndented = sourceFile.getLineAndCharacterOfPosition(range.pos).line === sourceFile.getLineAndCharacterOfPosition(previousNode.end).line;
                                        indentMultilineCommentOrJsxText(range, childIndentation.indentation, firstLineIsIndented, /*indentFinalLine*/ false, /*jsxStyle*/ true);
                                    }
                                }
                            }
                        }
                        childContextNode = node;
                        if (isFirstListItem && parent.kind === 192 /* ArrayLiteralExpression */ && inheritedIndentation === -1 /* Unknown */) {
                            inheritedIndentation = childIndentation.indentation;
                        }
                        return inheritedIndentation;
                    }
                    function processChildNodes(nodes, parent, parentStartLine, parentDynamicIndentation) {
                        ts.Debug.assert(ts.isNodeArray(nodes));
                        var listStartToken = getOpenTokenForList(parent, nodes);
                        var listDynamicIndentation = parentDynamicIndentation;
                        var startLine = parentStartLine;
                        if (listStartToken !== 0 /* Unknown */) {
                            // introduce a new indentation scope for lists (including list start and end tokens)
                            while (formattingScanner.isOnToken()) {
                                var tokenInfo = formattingScanner.readTokenInfo(parent);
                                if (tokenInfo.token.end > nodes.pos) {
                                    // stop when formatting scanner moves past the beginning of node list
                                    break;
                                }
                                else if (tokenInfo.token.kind === listStartToken) {
                                    // consume list start token
                                    startLine = sourceFile.getLineAndCharacterOfPosition(tokenInfo.token.pos).line;
                                    consumeTokenAndAdvanceScanner(tokenInfo, parent, parentDynamicIndentation, parent);
                                    var indentationOnListStartToken = void 0;
                                    if (indentationOnLastIndentedLine !== -1 /* Unknown */) {
                                        // scanner just processed list start token so consider last indentation as list indentation
                                        // function foo(): { // last indentation was 0, list item will be indented based on this value
                                        //   foo: number;
                                        // }: {};
                                        indentationOnListStartToken = indentationOnLastIndentedLine;
                                    }
                                    else {
                                        var startLinePosition = ts.getLineStartPositionForPosition(tokenInfo.token.pos, sourceFile);
                                        indentationOnListStartToken = formatting.SmartIndenter.findFirstNonWhitespaceColumn(startLinePosition, tokenInfo.token.pos, sourceFile, options);
                                    }
                                    listDynamicIndentation = getDynamicIndentation(parent, parentStartLine, indentationOnListStartToken, options.indentSize); // TODO: GH#18217
                                }
                                else {
                                    // consume any tokens that precede the list as child elements of 'node' using its indentation scope
                                    consumeTokenAndAdvanceScanner(tokenInfo, parent, parentDynamicIndentation, parent);
                                }
                            }
                        }
                        var inheritedIndentation = -1 /* Unknown */;
                        for (var i = 0; i < nodes.length; i++) {
                            var child = nodes[i];
                            inheritedIndentation = processChildNode(child, inheritedIndentation, node, listDynamicIndentation, startLine, startLine, /*isListItem*/ true, /*isFirstListItem*/ i === 0);
                        }
                        var listEndToken = getCloseTokenForOpenToken(listStartToken);
                        if (listEndToken !== 0 /* Unknown */ && formattingScanner.isOnToken()) {
                            var tokenInfo = formattingScanner.readTokenInfo(parent);
                            if (tokenInfo.token.kind === 27 /* CommaToken */ && ts.isCallLikeExpression(parent)) {
                                formattingScanner.advance();
                                tokenInfo = formattingScanner.isOnToken() ? formattingScanner.readTokenInfo(parent) : undefined;
                            }
                            // consume the list end token only if it is still belong to the parent
                            // there might be the case when current token matches end token but does not considered as one
                            // function (x: function) <--
                            // without this check close paren will be interpreted as list end token for function expression which is wrong
                            if (tokenInfo && tokenInfo.token.kind === listEndToken && ts.rangeContainsRange(parent, tokenInfo.token)) {
                                // consume list end token
                                consumeTokenAndAdvanceScanner(tokenInfo, parent, listDynamicIndentation, parent, /*isListEndToken*/ true);
                            }
                        }
                    }
                    function consumeTokenAndAdvanceScanner(currentTokenInfo, parent, dynamicIndentation, container, isListEndToken) {
                        ts.Debug.assert(ts.rangeContainsRange(parent, currentTokenInfo.token));
                        var lastTriviaWasNewLine = formattingScanner.lastTrailingTriviaWasNewLine();
                        var indentToken = false;
                        if (currentTokenInfo.leadingTrivia) {
                            processTrivia(currentTokenInfo.leadingTrivia, parent, childContextNode, dynamicIndentation);
                        }
                        var lineAction = 0 /* None */;
                        var isTokenInRange = ts.rangeContainsRange(originalRange, currentTokenInfo.token);
                        var tokenStart = sourceFile.getLineAndCharacterOfPosition(currentTokenInfo.token.pos);
                        if (isTokenInRange) {
                            var rangeHasError = rangeContainsError(currentTokenInfo.token);
                            // save previousRange since processRange will overwrite this value with current one
                            var savePreviousRange = previousRange;
                            lineAction = processRange(currentTokenInfo.token, tokenStart, parent, childContextNode, dynamicIndentation);
                            // do not indent comments\token if token range overlaps with some error
                            if (!rangeHasError) {
                                if (lineAction === 0 /* None */) {
                                    // indent token only if end line of previous range does not match start line of the token
                                    var prevEndLine = savePreviousRange && sourceFile.getLineAndCharacterOfPosition(savePreviousRange.end).line;
                                    indentToken = lastTriviaWasNewLine && tokenStart.line !== prevEndLine;
                                }
                                else {
                                    indentToken = lineAction === 1 /* LineAdded */;
                                }
                            }
                        }
                        if (currentTokenInfo.trailingTrivia) {
                            processTrivia(currentTokenInfo.trailingTrivia, parent, childContextNode, dynamicIndentation);
                        }
                        if (indentToken) {
                            var tokenIndentation = (isTokenInRange && !rangeContainsError(currentTokenInfo.token)) ?
                                dynamicIndentation.getIndentationForToken(tokenStart.line, currentTokenInfo.token.kind, container, !!isListEndToken) :
                                -1 /* Unknown */;
                            var indentNextTokenOrTrivia = true;
                            if (currentTokenInfo.leadingTrivia) {
                                var commentIndentation_1 = dynamicIndentation.getIndentationForComment(currentTokenInfo.token.kind, tokenIndentation, container);
                                indentNextTokenOrTrivia = indentTriviaItems(currentTokenInfo.leadingTrivia, commentIndentation_1, indentNextTokenOrTrivia, function (item) { return insertIndentation(item.pos, commentIndentation_1, /*lineAdded*/ false); });
                            }
                            // indent token only if is it is in target range and does not overlap with any error ranges
                            if (tokenIndentation !== -1 /* Unknown */ && indentNextTokenOrTrivia) {
                                insertIndentation(currentTokenInfo.token.pos, tokenIndentation, lineAction === 1 /* LineAdded */);
                                lastIndentedLine = tokenStart.line;
                                indentationOnLastIndentedLine = tokenIndentation;
                            }
                        }
                        formattingScanner.advance();
                        childContextNode = parent;
                    }
                }
                function indentTriviaItems(trivia, commentIndentation, indentNextTokenOrTrivia, indentSingleLine) {
                    for (var _i = 0, trivia_1 = trivia; _i < trivia_1.length; _i++) {
                        var triviaItem = trivia_1[_i];
                        var triviaInRange = ts.rangeContainsRange(originalRange, triviaItem);
                        switch (triviaItem.kind) {
                            case 3 /* MultiLineCommentTrivia */:
                                if (triviaInRange) {
                                    indentMultilineCommentOrJsxText(triviaItem, commentIndentation, /*firstLineIsIndented*/ !indentNextTokenOrTrivia);
                                }
                                indentNextTokenOrTrivia = false;
                                break;
                            case 2 /* SingleLineCommentTrivia */:
                                if (indentNextTokenOrTrivia && triviaInRange) {
                                    indentSingleLine(triviaItem);
                                }
                                indentNextTokenOrTrivia = false;
                                break;
                            case 4 /* NewLineTrivia */:
                                indentNextTokenOrTrivia = true;
                                break;
                        }
                    }
                    return indentNextTokenOrTrivia;
                }
                function processTrivia(trivia, parent, contextNode, dynamicIndentation) {
                    for (var _i = 0, trivia_2 = trivia; _i < trivia_2.length; _i++) {
                        var triviaItem = trivia_2[_i];
                        if (ts.isComment(triviaItem.kind) && ts.rangeContainsRange(originalRange, triviaItem)) {
                            var triviaItemStart = sourceFile.getLineAndCharacterOfPosition(triviaItem.pos);
                            processRange(triviaItem, triviaItemStart, parent, contextNode, dynamicIndentation);
                        }
                    }
                }
                function processRange(range, rangeStart, parent, contextNode, dynamicIndentation) {
                    var rangeHasError = rangeContainsError(range);
                    var lineAction = 0 /* None */;
                    if (!rangeHasError) {
                        if (!previousRange) {
                            // trim whitespaces starting from the beginning of the span up to the current line
                            var originalStart = sourceFile.getLineAndCharacterOfPosition(originalRange.pos);
                            trimTrailingWhitespacesForLines(originalStart.line, rangeStart.line);
                        }
                        else {
                            lineAction =
                                processPair(range, rangeStart.line, parent, previousRange, previousRangeStartLine, previousParent, contextNode, dynamicIndentation);
                        }
                    }
                    previousRange = range;
                    previousParent = parent;
                    previousRangeStartLine = rangeStart.line;
                    return lineAction;
                }
                function processPair(currentItem, currentStartLine, currentParent, previousItem, previousStartLine, previousParent, contextNode, dynamicIndentation) {
                    formattingContext.updateContext(previousItem, previousParent, currentItem, currentParent, contextNode);
                    var rules = getRules(formattingContext);
                    var trimTrailingWhitespaces = false;
                    var lineAction = 0 /* None */;
                    if (rules) {
                        // Apply rules in reverse order so that higher priority rules (which are first in the array)
                        // win in a conflict with lower priority rules.
                        ts.forEachRight(rules, function (rule) {
                            lineAction = applyRuleEdits(rule, previousItem, previousStartLine, currentItem, currentStartLine);
                            switch (lineAction) {
                                case 2 /* LineRemoved */:
                                    // Handle the case where the next line is moved to be the end of this line.
                                    // In this case we don't indent the next line in the next pass.
                                    if (currentParent.getStart(sourceFile) === currentItem.pos) {
                                        dynamicIndentation.recomputeIndentation(/*lineAddedByFormatting*/ false);
                                    }
                                    break;
                                case 1 /* LineAdded */:
                                    // Handle the case where token2 is moved to the new line.
                                    // In this case we indent token2 in the next pass but we set
                                    // sameLineIndent flag to notify the indenter that the indentation is within the line.
                                    if (currentParent.getStart(sourceFile) === currentItem.pos) {
                                        dynamicIndentation.recomputeIndentation(/*lineAddedByFormatting*/ true);
                                    }
                                    break;
                                default:
                                    ts.Debug.assert(lineAction === 0 /* None */);
                            }
                            // We need to trim trailing whitespace between the tokens if they were on different lines, and no rule was applied to put them on the same line
                            trimTrailingWhitespaces = !(rule.action & 16 /* DeleteSpace */) && rule.flags !== 1 /* CanDeleteNewLines */;
                        });
                    }
                    else {
                        trimTrailingWhitespaces = currentItem.kind !== 1 /* EndOfFileToken */;
                    }
                    if (currentStartLine !== previousStartLine && trimTrailingWhitespaces) {
                        // We need to trim trailing whitespace between the tokens if they were on different lines, and no rule was applied to put them on the same line
                        trimTrailingWhitespacesForLines(previousStartLine, currentStartLine, previousItem);
                    }
                    return lineAction;
                }
                function insertIndentation(pos, indentation, lineAdded) {
                    var indentationString = getIndentationString(indentation, options);
                    if (lineAdded) {
                        // new line is added before the token by the formatting rules
                        // insert indentation string at the very beginning of the token
                        recordReplace(pos, 0, indentationString);
                    }
                    else {
                        var tokenStart = sourceFile.getLineAndCharacterOfPosition(pos);
                        var startLinePosition = ts.getStartPositionOfLine(tokenStart.line, sourceFile);
                        if (indentation !== characterToColumn(startLinePosition, tokenStart.character) || indentationIsDifferent(indentationString, startLinePosition)) {
                            recordReplace(startLinePosition, tokenStart.character, indentationString);
                        }
                    }
                }
                function characterToColumn(startLinePosition, characterInLine) {
                    var column = 0;
                    for (var i = 0; i < characterInLine; i++) {
                        if (sourceFile.text.charCodeAt(startLinePosition + i) === 9 /* tab */) {
                            column += options.tabSize - column % options.tabSize;
                        }
                        else {
                            column++;
                        }
                    }
                    return column;
                }
                function indentationIsDifferent(indentationString, startLinePosition) {
                    return indentationString !== sourceFile.text.substr(startLinePosition, indentationString.length);
                }
                function indentMultilineCommentOrJsxText(commentRange, indentation, firstLineIsIndented, indentFinalLine, jsxTextStyleIndent) {
                    if (indentFinalLine === void 0) {
                        indentFinalLine = true;
                    }
                    // split comment in lines
                    var startLine = sourceFile.getLineAndCharacterOfPosition(commentRange.pos).line;
                    var endLine = sourceFile.getLineAndCharacterOfPosition(commentRange.end).line;
                    if (startLine === endLine) {
                        if (!firstLineIsIndented) {
                            // treat as single line comment
                            insertIndentation(commentRange.pos, indentation, /*lineAdded*/ false);
                        }
                        return;
                    }
                    var parts = [];
                    var startPos = commentRange.pos;
                    for (var line = startLine; line < endLine; line++) {
                        var endOfLine = ts.getEndLinePosition(line, sourceFile);
                        parts.push({ pos: startPos, end: endOfLine });
                        startPos = ts.getStartPositionOfLine(line + 1, sourceFile);
                    }
                    if (indentFinalLine) {
                        parts.push({ pos: startPos, end: commentRange.end });
                    }
                    if (parts.length === 0)
                        return;
                    var startLinePos = ts.getStartPositionOfLine(startLine, sourceFile);
                    var nonWhitespaceColumnInFirstPart = formatting.SmartIndenter.findFirstNonWhitespaceCharacterAndColumn(startLinePos, parts[0].pos, sourceFile, options);
                    if (indentation === nonWhitespaceColumnInFirstPart.column && !jsxTextStyleIndent) {
                        return;
                    }
                    var startIndex = 0;
                    if (firstLineIsIndented) {
                        startIndex = 1;
                        startLine++;
                    }
                    // shift all parts on the delta size
                    var delta = indentation - nonWhitespaceColumnInFirstPart.column;
                    for (var i = startIndex; i < parts.length; i++, startLine++) {
                        var startLinePos_1 = ts.getStartPositionOfLine(startLine, sourceFile);
                        var nonWhitespaceCharacterAndColumn = i === 0
                            ? nonWhitespaceColumnInFirstPart
                            : formatting.SmartIndenter.findFirstNonWhitespaceCharacterAndColumn(parts[i].pos, parts[i].end, sourceFile, options);
                        if (jsxTextStyleIndent) {
                            // skip adding indentation to blank lines
                            if (ts.isLineBreak(sourceFile.text.charCodeAt(ts.getStartPositionOfLine(startLine, sourceFile))))
                                continue;
                            // reset delta on every line
                            delta = indentation - nonWhitespaceCharacterAndColumn.column;
                        }
                        var newIndentation = nonWhitespaceCharacterAndColumn.column + delta;
                        if (newIndentation > 0) {
                            var indentationString = getIndentationString(newIndentation, options);
                            recordReplace(startLinePos_1, nonWhitespaceCharacterAndColumn.character, indentationString);
                        }
                        else {
                            recordDelete(startLinePos_1, nonWhitespaceCharacterAndColumn.character);
                        }
                    }
                }
                function trimTrailingWhitespacesForLines(line1, line2, range) {
                    for (var line = line1; line < line2; line++) {
                        var lineStartPosition = ts.getStartPositionOfLine(line, sourceFile);
                        var lineEndPosition = ts.getEndLinePosition(line, sourceFile);
                        // do not trim whitespaces in comments or template expression
                        if (range && (ts.isComment(range.kind) || ts.isStringOrRegularExpressionOrTemplateLiteral(range.kind)) && range.pos <= lineEndPosition && range.end > lineEndPosition) {
                            continue;
                        }
                        var whitespaceStart = getTrailingWhitespaceStartPosition(lineStartPosition, lineEndPosition);
                        if (whitespaceStart !== -1) {
                            ts.Debug.assert(whitespaceStart === lineStartPosition || !ts.isWhiteSpaceSingleLine(sourceFile.text.charCodeAt(whitespaceStart - 1)));
                            recordDelete(whitespaceStart, lineEndPosition + 1 - whitespaceStart);
                        }
                    }
                }
                /**
                 * @param start The position of the first character in range
                 * @param end The position of the last character in range
                 */
                function getTrailingWhitespaceStartPosition(start, end) {
                    var pos = end;
                    while (pos >= start && ts.isWhiteSpaceSingleLine(sourceFile.text.charCodeAt(pos))) {
                        pos--;
                    }
                    if (pos !== end) {
                        return pos + 1;
                    }
                    return -1;
                }
                /**
                 * Trimming will be done for lines after the previous range
                 */
                function trimTrailingWhitespacesForRemainingRange() {
                    var startPosition = previousRange ? previousRange.end : originalRange.pos;
                    var startLine = sourceFile.getLineAndCharacterOfPosition(startPosition).line;
                    var endLine = sourceFile.getLineAndCharacterOfPosition(originalRange.end).line;
                    trimTrailingWhitespacesForLines(startLine, endLine + 1, previousRange);
                }
                function recordDelete(start, len) {
                    if (len) {
                        edits.push(ts.createTextChangeFromStartLength(start, len, ""));
                    }
                }
                function recordReplace(start, len, newText) {
                    if (len || newText) {
                        edits.push(ts.createTextChangeFromStartLength(start, len, newText));
                    }
                }
                function recordInsert(start, text) {
                    if (text) {
                        edits.push(ts.createTextChangeFromStartLength(start, 0, text));
                    }
                }
                function applyRuleEdits(rule, previousRange, previousStartLine, currentRange, currentStartLine) {
                    var onLaterLine = currentStartLine !== previousStartLine;
                    switch (rule.action) {
                        case 1 /* StopProcessingSpaceActions */:
                            // no action required
                            return 0 /* None */;
                        case 16 /* DeleteSpace */:
                            if (previousRange.end !== currentRange.pos) {
                                // delete characters starting from t1.end up to t2.pos exclusive
                                recordDelete(previousRange.end, currentRange.pos - previousRange.end);
                                return onLaterLine ? 2 /* LineRemoved */ : 0 /* None */;
                            }
                            break;
                        case 32 /* DeleteToken */:
                            recordDelete(previousRange.pos, previousRange.end - previousRange.pos);
                            break;
                        case 8 /* InsertNewLine */:
                            // exit early if we on different lines and rule cannot change number of newlines
                            // if line1 and line2 are on subsequent lines then no edits are required - ok to exit
                            // if line1 and line2 are separated with more than one newline - ok to exit since we cannot delete extra new lines
                            if (rule.flags !== 1 /* CanDeleteNewLines */ && previousStartLine !== currentStartLine) {
                                return 0 /* None */;
                            }
                            // edit should not be applied if we have one line feed between elements
                            var lineDelta = currentStartLine - previousStartLine;
                            if (lineDelta !== 1) {
                                recordReplace(previousRange.end, currentRange.pos - previousRange.end, options.newLineCharacter);
                                return onLaterLine ? 0 /* None */ : 1 /* LineAdded */;
                            }
                            break;
                        case 4 /* InsertSpace */:
                            // exit early if we on different lines and rule cannot change number of newlines
                            if (rule.flags !== 1 /* CanDeleteNewLines */ && previousStartLine !== currentStartLine) {
                                return 0 /* None */;
                            }
                            var posDelta = currentRange.pos - previousRange.end;
                            if (posDelta !== 1 || sourceFile.text.charCodeAt(previousRange.end) !== 32 /* space */) {
                                recordReplace(previousRange.end, currentRange.pos - previousRange.end, " ");
                                return onLaterLine ? 2 /* LineRemoved */ : 0 /* None */;
                            }
                            break;
                        case 64 /* InsertTrailingSemicolon */:
                            recordInsert(previousRange.end, ";");
                    }
                    return 0 /* None */;
                }
            }
            var LineAction;
            (function (LineAction) {
                LineAction[LineAction["None"] = 0] = "None";
                LineAction[LineAction["LineAdded"] = 1] = "LineAdded";
                LineAction[LineAction["LineRemoved"] = 2] = "LineRemoved";
            })(LineAction || (LineAction = {}));
            /**
             * @param precedingToken pass `null` if preceding token was already computed and result was `undefined`.
             */
            function getRangeOfEnclosingComment(sourceFile, position, precedingToken, tokenAtPosition) {
                if (tokenAtPosition === void 0) {
                    tokenAtPosition = ts.getTokenAtPosition(sourceFile, position);
                }
                var jsdoc = ts.findAncestor(tokenAtPosition, ts.isJSDoc);
                if (jsdoc)
                    tokenAtPosition = jsdoc.parent;
                var tokenStart = tokenAtPosition.getStart(sourceFile);
                if (tokenStart <= position && position < tokenAtPosition.getEnd()) {
                    return undefined;
                }
                // eslint-disable-next-line no-null/no-null
                precedingToken = precedingToken === null ? undefined : precedingToken === undefined ? ts.findPrecedingToken(position, sourceFile) : precedingToken;
                // Between two consecutive tokens, all comments are either trailing on the former
                // or leading on the latter (and none are in both lists).
                var trailingRangesOfPreviousToken = precedingToken && ts.getTrailingCommentRanges(sourceFile.text, precedingToken.end);
                var leadingCommentRangesOfNextToken = ts.getLeadingCommentRangesOfNode(tokenAtPosition, sourceFile);
                var commentRanges = ts.concatenate(trailingRangesOfPreviousToken, leadingCommentRangesOfNextToken);
                return commentRanges && ts.find(commentRanges, function (range) {
                    return ts.rangeContainsPositionExclusive(range, position) ||
                        // The end marker of a single-line comment does not include the newline character.
                        // With caret at `^`, in the following case, we are inside a comment (^ denotes the cursor position):
                        //
                        //    // asdf   ^\n
                        //
                        // But for closed multi-line comments, we don't want to be inside the comment in the following case:
                        //
                        //    /* asdf */^
                        //
                        // However, unterminated multi-line comments *do* contain their end.
                        //
                        // Internally, we represent the end of the comment at the newline and closing '/', respectively.
                        //
                        position === range.end && (range.kind === 2 /* SingleLineCommentTrivia */ || position === sourceFile.getFullWidth());
                });
            }
            formatting.getRangeOfEnclosingComment = getRangeOfEnclosingComment;
            function getOpenTokenForList(node, list) {
                switch (node.kind) {
                    case 162 /* Constructor */:
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                    case 202 /* ArrowFunction */:
                        if (node.typeParameters === list) {
                            return 29 /* LessThanToken */;
                        }
                        else if (node.parameters === list) {
                            return 20 /* OpenParenToken */;
                        }
                        break;
                    case 196 /* CallExpression */:
                    case 197 /* NewExpression */:
                        if (node.typeArguments === list) {
                            return 29 /* LessThanToken */;
                        }
                        else if (node.arguments === list) {
                            return 20 /* OpenParenToken */;
                        }
                        break;
                    case 169 /* TypeReference */:
                        if (node.typeArguments === list) {
                            return 29 /* LessThanToken */;
                        }
                        break;
                    case 173 /* TypeLiteral */:
                        return 18 /* OpenBraceToken */;
                }
                return 0 /* Unknown */;
            }
            function getCloseTokenForOpenToken(kind) {
                switch (kind) {
                    case 20 /* OpenParenToken */:
                        return 21 /* CloseParenToken */;
                    case 29 /* LessThanToken */:
                        return 31 /* GreaterThanToken */;
                    case 18 /* OpenBraceToken */:
                        return 19 /* CloseBraceToken */;
                }
                return 0 /* Unknown */;
            }
            var internedSizes;
            var internedTabsIndentation;
            var internedSpacesIndentation;
            function getIndentationString(indentation, options) {
                // reset interned strings if FormatCodeOptions were changed
                var resetInternedStrings = !internedSizes || (internedSizes.tabSize !== options.tabSize || internedSizes.indentSize !== options.indentSize);
                if (resetInternedStrings) {
                    internedSizes = { tabSize: options.tabSize, indentSize: options.indentSize };
                    internedTabsIndentation = internedSpacesIndentation = undefined;
                }
                if (!options.convertTabsToSpaces) {
                    var tabs = Math.floor(indentation / options.tabSize);
                    var spaces = indentation - tabs * options.tabSize;
                    var tabString = void 0;
                    if (!internedTabsIndentation) {
                        internedTabsIndentation = [];
                    }
                    if (internedTabsIndentation[tabs] === undefined) {
                        internedTabsIndentation[tabs] = tabString = ts.repeatString("\t", tabs);
                    }
                    else {
                        tabString = internedTabsIndentation[tabs];
                    }
                    return spaces ? tabString + ts.repeatString(" ", spaces) : tabString;
                }
                else {
                    var spacesString = void 0;
                    var quotient = Math.floor(indentation / options.indentSize);
                    var remainder = indentation % options.indentSize;
                    if (!internedSpacesIndentation) {
                        internedSpacesIndentation = [];
                    }
                    if (internedSpacesIndentation[quotient] === undefined) {
                        spacesString = ts.repeatString(" ", options.indentSize * quotient);
                        internedSpacesIndentation[quotient] = spacesString;
                    }
                    else {
                        spacesString = internedSpacesIndentation[quotient];
                    }
                    return remainder ? spacesString + ts.repeatString(" ", remainder) : spacesString;
                }
            }
            formatting.getIndentationString = getIndentationString;
        })(formatting = ts.formatting || (ts.formatting = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var formatting;
        (function (formatting) {
            var SmartIndenter;
            (function (SmartIndenter) {
                var Value;
                (function (Value) {
                    Value[Value["Unknown"] = -1] = "Unknown";
                })(Value || (Value = {}));
                /**
                 * @param assumeNewLineBeforeCloseBrace
                 * `false` when called on text from a real source file.
                 * `true` when we need to assume `position` is on a newline.
                 *
                 * This is useful for codefixes. Consider
                 * ```
                 * function f() {
                 * |}
                 * ```
                 * with `position` at `|`.
                 *
                 * When inserting some text after an open brace, we would like to get indentation as if a newline was already there.
                 * By default indentation at `position` will be 0 so 'assumeNewLineBeforeCloseBrace' overrides this behavior.
                 */
                function getIndentation(position, sourceFile, options, assumeNewLineBeforeCloseBrace) {
                    if (assumeNewLineBeforeCloseBrace === void 0) {
                        assumeNewLineBeforeCloseBrace = false;
                    }
                    if (position > sourceFile.text.length) {
                        return getBaseIndentation(options); // past EOF
                    }
                    // no indentation when the indent style is set to none,
                    // so we can return fast
                    if (options.indentStyle === ts.IndentStyle.None) {
                        return 0;
                    }
                    var precedingToken = ts.findPrecedingToken(position, sourceFile, /*startNode*/ undefined, /*excludeJsdoc*/ true);
                    // eslint-disable-next-line no-null/no-null
                    var enclosingCommentRange = formatting.getRangeOfEnclosingComment(sourceFile, position, precedingToken || null);
                    if (enclosingCommentRange && enclosingCommentRange.kind === 3 /* MultiLineCommentTrivia */) {
                        return getCommentIndent(sourceFile, position, options, enclosingCommentRange);
                    }
                    if (!precedingToken) {
                        return getBaseIndentation(options);
                    }
                    // no indentation in string \regex\template literals
                    var precedingTokenIsLiteral = ts.isStringOrRegularExpressionOrTemplateLiteral(precedingToken.kind);
                    if (precedingTokenIsLiteral && precedingToken.getStart(sourceFile) <= position && position < precedingToken.end) {
                        return 0;
                    }
                    var lineAtPosition = sourceFile.getLineAndCharacterOfPosition(position).line;
                    // indentation is first non-whitespace character in a previous line
                    // for block indentation, we should look for a line which contains something that's not
                    // whitespace.
                    if (options.indentStyle === ts.IndentStyle.Block) {
                        return getBlockIndent(sourceFile, position, options);
                    }
                    if (precedingToken.kind === 27 /* CommaToken */ && precedingToken.parent.kind !== 209 /* BinaryExpression */) {
                        // previous token is comma that separates items in list - find the previous item and try to derive indentation from it
                        var actualIndentation = getActualIndentationForListItemBeforeComma(precedingToken, sourceFile, options);
                        if (actualIndentation !== -1 /* Unknown */) {
                            return actualIndentation;
                        }
                    }
                    var containerList = getListByPosition(position, precedingToken.parent, sourceFile);
                    // use list position if the preceding token is before any list items
                    if (containerList && !ts.rangeContainsRange(containerList, precedingToken)) {
                        return getActualIndentationForListStartLine(containerList, sourceFile, options) + options.indentSize; // TODO: GH#18217
                    }
                    return getSmartIndent(sourceFile, position, precedingToken, lineAtPosition, assumeNewLineBeforeCloseBrace, options);
                }
                SmartIndenter.getIndentation = getIndentation;
                function getCommentIndent(sourceFile, position, options, enclosingCommentRange) {
                    var previousLine = ts.getLineAndCharacterOfPosition(sourceFile, position).line - 1;
                    var commentStartLine = ts.getLineAndCharacterOfPosition(sourceFile, enclosingCommentRange.pos).line;
                    ts.Debug.assert(commentStartLine >= 0);
                    if (previousLine <= commentStartLine) {
                        return findFirstNonWhitespaceColumn(ts.getStartPositionOfLine(commentStartLine, sourceFile), position, sourceFile, options);
                    }
                    var startPositionOfLine = ts.getStartPositionOfLine(previousLine, sourceFile);
                    var _a = findFirstNonWhitespaceCharacterAndColumn(startPositionOfLine, position, sourceFile, options), column = _a.column, character = _a.character;
                    if (column === 0) {
                        return column;
                    }
                    var firstNonWhitespaceCharacterCode = sourceFile.text.charCodeAt(startPositionOfLine + character);
                    return firstNonWhitespaceCharacterCode === 42 /* asterisk */ ? column - 1 : column;
                }
                function getBlockIndent(sourceFile, position, options) {
                    // move backwards until we find a line with a non-whitespace character,
                    // then find the first non-whitespace character for that line.
                    var current = position;
                    while (current > 0) {
                        var char = sourceFile.text.charCodeAt(current);
                        if (!ts.isWhiteSpaceLike(char)) {
                            break;
                        }
                        current--;
                    }
                    var lineStart = ts.getLineStartPositionForPosition(current, sourceFile);
                    return findFirstNonWhitespaceColumn(lineStart, current, sourceFile, options);
                }
                function getSmartIndent(sourceFile, position, precedingToken, lineAtPosition, assumeNewLineBeforeCloseBrace, options) {
                    // try to find node that can contribute to indentation and includes 'position' starting from 'precedingToken'
                    // if such node is found - compute initial indentation for 'position' inside this node
                    var previous;
                    var current = precedingToken;
                    while (current) {
                        if (ts.positionBelongsToNode(current, position, sourceFile) && shouldIndentChildNode(options, current, previous, sourceFile, /*isNextChild*/ true)) {
                            var currentStart = getStartLineAndCharacterForNode(current, sourceFile);
                            var nextTokenKind = nextTokenIsCurlyBraceOnSameLineAsCursor(precedingToken, current, lineAtPosition, sourceFile);
                            var indentationDelta = nextTokenKind !== 0 /* Unknown */
                                // handle cases when codefix is about to be inserted before the close brace
                                ? assumeNewLineBeforeCloseBrace && nextTokenKind === 2 /* CloseBrace */ ? options.indentSize : 0
                                : lineAtPosition !== currentStart.line ? options.indentSize : 0;
                            return getIndentationForNodeWorker(current, currentStart, /*ignoreActualIndentationRange*/ undefined, indentationDelta, sourceFile, /*isNextChild*/ true, options); // TODO: GH#18217
                        }
                        // check if current node is a list item - if yes, take indentation from it
                        // do not consider parent-child line sharing yet:
                        // function foo(a
                        //    | preceding node 'a' does share line with its parent but indentation is expected
                        var actualIndentation = getActualIndentationForListItem(current, sourceFile, options, /*listIndentsChild*/ true);
                        if (actualIndentation !== -1 /* Unknown */) {
                            return actualIndentation;
                        }
                        previous = current;
                        current = current.parent;
                    }
                    // no parent was found - return the base indentation of the SourceFile
                    return getBaseIndentation(options);
                }
                function getIndentationForNode(n, ignoreActualIndentationRange, sourceFile, options) {
                    var start = sourceFile.getLineAndCharacterOfPosition(n.getStart(sourceFile));
                    return getIndentationForNodeWorker(n, start, ignoreActualIndentationRange, /*indentationDelta*/ 0, sourceFile, /*isNextChild*/ false, options);
                }
                SmartIndenter.getIndentationForNode = getIndentationForNode;
                function getBaseIndentation(options) {
                    return options.baseIndentSize || 0;
                }
                SmartIndenter.getBaseIndentation = getBaseIndentation;
                function getIndentationForNodeWorker(current, currentStart, ignoreActualIndentationRange, indentationDelta, sourceFile, isNextChild, options) {
                    var parent = current.parent;
                    // Walk up the tree and collect indentation for parent-child node pairs. Indentation is not added if
                    // * parent and child nodes start on the same line, or
                    // * parent is an IfStatement and child starts on the same line as an 'else clause'.
                    while (parent) {
                        var useActualIndentation = true;
                        if (ignoreActualIndentationRange) {
                            var start = current.getStart(sourceFile);
                            useActualIndentation = start < ignoreActualIndentationRange.pos || start > ignoreActualIndentationRange.end;
                        }
                        var containingListOrParentStart = getContainingListOrParentStart(parent, current, sourceFile);
                        var parentAndChildShareLine = containingListOrParentStart.line === currentStart.line ||
                            childStartsOnTheSameLineWithElseInIfStatement(parent, current, currentStart.line, sourceFile);
                        if (useActualIndentation) {
                            // check if current node is a list item - if yes, take indentation from it
                            var actualIndentation = getActualIndentationForListItem(current, sourceFile, options, !parentAndChildShareLine);
                            if (actualIndentation !== -1 /* Unknown */) {
                                return actualIndentation + indentationDelta;
                            }
                            // try to fetch actual indentation for current node from source text
                            actualIndentation = getActualIndentationForNode(current, parent, currentStart, parentAndChildShareLine, sourceFile, options);
                            if (actualIndentation !== -1 /* Unknown */) {
                                return actualIndentation + indentationDelta;
                            }
                        }
                        // increase indentation if parent node wants its content to be indented and parent and child nodes don't start on the same line
                        if (shouldIndentChildNode(options, parent, current, sourceFile, isNextChild) && !parentAndChildShareLine) {
                            indentationDelta += options.indentSize;
                        }
                        // In our AST, a call argument's `parent` is the call-expression, not the argument list.
                        // We would like to increase indentation based on the relationship between an argument and its argument-list,
                        // so we spoof the starting position of the (parent) call-expression to match the (non-parent) argument-list.
                        // But, the spoofed start-value could then cause a problem when comparing the start position of the call-expression
                        // to *its* parent (in the case of an iife, an expression statement), adding an extra level of indentation.
                        //
                        // Instead, when at an argument, we unspoof the starting position of the enclosing call expression
                        // *after* applying indentation for the argument.
                        var useTrueStart = isArgumentAndStartLineOverlapsExpressionBeingCalled(parent, current, currentStart.line, sourceFile);
                        current = parent;
                        parent = current.parent;
                        currentStart = useTrueStart ? sourceFile.getLineAndCharacterOfPosition(current.getStart(sourceFile)) : containingListOrParentStart;
                    }
                    return indentationDelta + getBaseIndentation(options);
                }
                function getContainingListOrParentStart(parent, child, sourceFile) {
                    var containingList = getContainingList(child, sourceFile);
                    var startPos = containingList ? containingList.pos : parent.getStart(sourceFile);
                    return sourceFile.getLineAndCharacterOfPosition(startPos);
                }
                /*
                 * Function returns Value.Unknown if indentation cannot be determined
                 */
                function getActualIndentationForListItemBeforeComma(commaToken, sourceFile, options) {
                    // previous token is comma that separates items in list - find the previous item and try to derive indentation from it
                    var commaItemInfo = ts.findListItemInfo(commaToken);
                    if (commaItemInfo && commaItemInfo.listItemIndex > 0) {
                        return deriveActualIndentationFromList(commaItemInfo.list.getChildren(), commaItemInfo.listItemIndex - 1, sourceFile, options);
                    }
                    else {
                        // handle broken code gracefully
                        return -1 /* Unknown */;
                    }
                }
                /*
                 * Function returns Value.Unknown if actual indentation for node should not be used (i.e because node is nested expression)
                 */
                function getActualIndentationForNode(current, parent, currentLineAndChar, parentAndChildShareLine, sourceFile, options) {
                    // actual indentation is used for statements\declarations if one of cases below is true:
                    // - parent is SourceFile - by default immediate children of SourceFile are not indented except when user indents them manually
                    // - parent and child are not on the same line
                    var useActualIndentation = (ts.isDeclaration(current) || ts.isStatementButNotDeclaration(current)) &&
                        (parent.kind === 290 /* SourceFile */ || !parentAndChildShareLine);
                    if (!useActualIndentation) {
                        return -1 /* Unknown */;
                    }
                    return findColumnForFirstNonWhitespaceCharacterInLine(currentLineAndChar, sourceFile, options);
                }
                var NextTokenKind;
                (function (NextTokenKind) {
                    NextTokenKind[NextTokenKind["Unknown"] = 0] = "Unknown";
                    NextTokenKind[NextTokenKind["OpenBrace"] = 1] = "OpenBrace";
                    NextTokenKind[NextTokenKind["CloseBrace"] = 2] = "CloseBrace";
                })(NextTokenKind || (NextTokenKind = {}));
                function nextTokenIsCurlyBraceOnSameLineAsCursor(precedingToken, current, lineAtPosition, sourceFile) {
                    var nextToken = ts.findNextToken(precedingToken, current, sourceFile);
                    if (!nextToken) {
                        return 0 /* Unknown */;
                    }
                    if (nextToken.kind === 18 /* OpenBraceToken */) {
                        // open braces are always indented at the parent level
                        return 1 /* OpenBrace */;
                    }
                    else if (nextToken.kind === 19 /* CloseBraceToken */) {
                        // close braces are indented at the parent level if they are located on the same line with cursor
                        // this means that if new line will be added at $ position, this case will be indented
                        // class A {
                        //    $
                        // }
                        /// and this one - not
                        // class A {
                        // $}
                        var nextTokenStartLine = getStartLineAndCharacterForNode(nextToken, sourceFile).line;
                        return lineAtPosition === nextTokenStartLine ? 2 /* CloseBrace */ : 0 /* Unknown */;
                    }
                    return 0 /* Unknown */;
                }
                function getStartLineAndCharacterForNode(n, sourceFile) {
                    return sourceFile.getLineAndCharacterOfPosition(n.getStart(sourceFile));
                }
                function isArgumentAndStartLineOverlapsExpressionBeingCalled(parent, child, childStartLine, sourceFile) {
                    if (!(ts.isCallExpression(parent) && ts.contains(parent.arguments, child))) {
                        return false;
                    }
                    var expressionOfCallExpressionEnd = parent.expression.getEnd();
                    var expressionOfCallExpressionEndLine = ts.getLineAndCharacterOfPosition(sourceFile, expressionOfCallExpressionEnd).line;
                    return expressionOfCallExpressionEndLine === childStartLine;
                }
                SmartIndenter.isArgumentAndStartLineOverlapsExpressionBeingCalled = isArgumentAndStartLineOverlapsExpressionBeingCalled;
                function childStartsOnTheSameLineWithElseInIfStatement(parent, child, childStartLine, sourceFile) {
                    if (parent.kind === 227 /* IfStatement */ && parent.elseStatement === child) {
                        var elseKeyword = ts.findChildOfKind(parent, 87 /* ElseKeyword */, sourceFile);
                        ts.Debug.assert(elseKeyword !== undefined);
                        var elseKeywordStartLine = getStartLineAndCharacterForNode(elseKeyword, sourceFile).line;
                        return elseKeywordStartLine === childStartLine;
                    }
                    return false;
                }
                SmartIndenter.childStartsOnTheSameLineWithElseInIfStatement = childStartsOnTheSameLineWithElseInIfStatement;
                function argumentStartsOnSameLineAsPreviousArgument(parent, child, childStartLine, sourceFile) {
                    if (ts.isCallOrNewExpression(parent)) {
                        if (!parent.arguments)
                            return false;
                        var currentNode = ts.find(parent.arguments, function (arg) { return arg.pos === child.pos; });
                        // If it's not one of the arguments, don't look past this
                        if (!currentNode)
                            return false;
                        var currentIndex = parent.arguments.indexOf(currentNode);
                        if (currentIndex === 0)
                            return false; // Can't look at previous node if first
                        var previousNode = parent.arguments[currentIndex - 1];
                        var lineOfPreviousNode = ts.getLineAndCharacterOfPosition(sourceFile, previousNode.getEnd()).line;
                        if (childStartLine === lineOfPreviousNode) {
                            return true;
                        }
                    }
                    return false;
                }
                SmartIndenter.argumentStartsOnSameLineAsPreviousArgument = argumentStartsOnSameLineAsPreviousArgument;
                function getContainingList(node, sourceFile) {
                    return node.parent && getListByRange(node.getStart(sourceFile), node.getEnd(), node.parent, sourceFile);
                }
                SmartIndenter.getContainingList = getContainingList;
                function getListByPosition(pos, node, sourceFile) {
                    return node && getListByRange(pos, pos, node, sourceFile);
                }
                function getListByRange(start, end, node, sourceFile) {
                    switch (node.kind) {
                        case 169 /* TypeReference */:
                            return getList(node.typeArguments);
                        case 193 /* ObjectLiteralExpression */:
                            return getList(node.properties);
                        case 192 /* ArrayLiteralExpression */:
                            return getList(node.elements);
                        case 173 /* TypeLiteral */:
                            return getList(node.members);
                        case 244 /* FunctionDeclaration */:
                        case 201 /* FunctionExpression */:
                        case 202 /* ArrowFunction */:
                        case 161 /* MethodDeclaration */:
                        case 160 /* MethodSignature */:
                        case 165 /* CallSignature */:
                        case 162 /* Constructor */:
                        case 171 /* ConstructorType */:
                        case 166 /* ConstructSignature */:
                            return getList(node.typeParameters) || getList(node.parameters);
                        case 245 /* ClassDeclaration */:
                        case 214 /* ClassExpression */:
                        case 246 /* InterfaceDeclaration */:
                        case 247 /* TypeAliasDeclaration */:
                        case 320 /* JSDocTemplateTag */:
                            return getList(node.typeParameters);
                        case 197 /* NewExpression */:
                        case 196 /* CallExpression */:
                            return getList(node.typeArguments) || getList(node.arguments);
                        case 243 /* VariableDeclarationList */:
                            return getList(node.declarations);
                        case 257 /* NamedImports */:
                        case 261 /* NamedExports */:
                            return getList(node.elements);
                        case 189 /* ObjectBindingPattern */:
                        case 190 /* ArrayBindingPattern */:
                            return getList(node.elements);
                    }
                    function getList(list) {
                        return list && ts.rangeContainsStartEnd(getVisualListRange(node, list, sourceFile), start, end) ? list : undefined;
                    }
                }
                function getVisualListRange(node, list, sourceFile) {
                    var children = node.getChildren(sourceFile);
                    for (var i = 1; i < children.length - 1; i++) {
                        if (children[i].pos === list.pos && children[i].end === list.end) {
                            return { pos: children[i - 1].end, end: children[i + 1].getStart(sourceFile) };
                        }
                    }
                    return list;
                }
                function getActualIndentationForListStartLine(list, sourceFile, options) {
                    if (!list) {
                        return -1 /* Unknown */;
                    }
                    return findColumnForFirstNonWhitespaceCharacterInLine(sourceFile.getLineAndCharacterOfPosition(list.pos), sourceFile, options);
                }
                function getActualIndentationForListItem(node, sourceFile, options, listIndentsChild) {
                    if (node.parent && node.parent.kind === 243 /* VariableDeclarationList */) {
                        // VariableDeclarationList has no wrapping tokens
                        return -1 /* Unknown */;
                    }
                    var containingList = getContainingList(node, sourceFile);
                    if (containingList) {
                        var index = containingList.indexOf(node);
                        if (index !== -1) {
                            var result = deriveActualIndentationFromList(containingList, index, sourceFile, options);
                            if (result !== -1 /* Unknown */) {
                                return result;
                            }
                        }
                        return getActualIndentationForListStartLine(containingList, sourceFile, options) + (listIndentsChild ? options.indentSize : 0); // TODO: GH#18217
                    }
                    return -1 /* Unknown */;
                }
                function deriveActualIndentationFromList(list, index, sourceFile, options) {
                    ts.Debug.assert(index >= 0 && index < list.length);
                    var node = list[index];
                    // walk toward the start of the list starting from current node and check if the line is the same for all items.
                    // if end line for item [i - 1] differs from the start line for item [i] - find column of the first non-whitespace character on the line of item [i]
                    var lineAndCharacter = getStartLineAndCharacterForNode(node, sourceFile);
                    for (var i = index - 1; i >= 0; i--) {
                        if (list[i].kind === 27 /* CommaToken */) {
                            continue;
                        }
                        // skip list items that ends on the same line with the current list element
                        var prevEndLine = sourceFile.getLineAndCharacterOfPosition(list[i].end).line;
                        if (prevEndLine !== lineAndCharacter.line) {
                            return findColumnForFirstNonWhitespaceCharacterInLine(lineAndCharacter, sourceFile, options);
                        }
                        lineAndCharacter = getStartLineAndCharacterForNode(list[i], sourceFile);
                    }
                    return -1 /* Unknown */;
                }
                function findColumnForFirstNonWhitespaceCharacterInLine(lineAndCharacter, sourceFile, options) {
                    var lineStart = sourceFile.getPositionOfLineAndCharacter(lineAndCharacter.line, 0);
                    return findFirstNonWhitespaceColumn(lineStart, lineStart + lineAndCharacter.character, sourceFile, options);
                }
                /**
                 * Character is the actual index of the character since the beginning of the line.
                 * Column - position of the character after expanding tabs to spaces.
                 * "0\t2$"
                 * value of 'character' for '$' is 3
                 * value of 'column' for '$' is 6 (assuming that tab size is 4)
                 */
                function findFirstNonWhitespaceCharacterAndColumn(startPos, endPos, sourceFile, options) {
                    var character = 0;
                    var column = 0;
                    for (var pos = startPos; pos < endPos; pos++) {
                        var ch = sourceFile.text.charCodeAt(pos);
                        if (!ts.isWhiteSpaceSingleLine(ch)) {
                            break;
                        }
                        if (ch === 9 /* tab */) {
                            column += options.tabSize + (column % options.tabSize);
                        }
                        else {
                            column++;
                        }
                        character++;
                    }
                    return { column: column, character: character };
                }
                SmartIndenter.findFirstNonWhitespaceCharacterAndColumn = findFirstNonWhitespaceCharacterAndColumn;
                function findFirstNonWhitespaceColumn(startPos, endPos, sourceFile, options) {
                    return findFirstNonWhitespaceCharacterAndColumn(startPos, endPos, sourceFile, options).column;
                }
                SmartIndenter.findFirstNonWhitespaceColumn = findFirstNonWhitespaceColumn;
                function nodeWillIndentChild(settings, parent, child, sourceFile, indentByDefault) {
                    var childKind = child ? child.kind : 0 /* Unknown */;
                    switch (parent.kind) {
                        case 226 /* ExpressionStatement */:
                        case 245 /* ClassDeclaration */:
                        case 214 /* ClassExpression */:
                        case 246 /* InterfaceDeclaration */:
                        case 248 /* EnumDeclaration */:
                        case 247 /* TypeAliasDeclaration */:
                        case 192 /* ArrayLiteralExpression */:
                        case 223 /* Block */:
                        case 250 /* ModuleBlock */:
                        case 193 /* ObjectLiteralExpression */:
                        case 173 /* TypeLiteral */:
                        case 186 /* MappedType */:
                        case 175 /* TupleType */:
                        case 251 /* CaseBlock */:
                        case 278 /* DefaultClause */:
                        case 277 /* CaseClause */:
                        case 200 /* ParenthesizedExpression */:
                        case 194 /* PropertyAccessExpression */:
                        case 196 /* CallExpression */:
                        case 197 /* NewExpression */:
                        case 225 /* VariableStatement */:
                        case 259 /* ExportAssignment */:
                        case 235 /* ReturnStatement */:
                        case 210 /* ConditionalExpression */:
                        case 190 /* ArrayBindingPattern */:
                        case 189 /* ObjectBindingPattern */:
                        case 268 /* JsxOpeningElement */:
                        case 271 /* JsxOpeningFragment */:
                        case 267 /* JsxSelfClosingElement */:
                        case 276 /* JsxExpression */:
                        case 160 /* MethodSignature */:
                        case 165 /* CallSignature */:
                        case 166 /* ConstructSignature */:
                        case 156 /* Parameter */:
                        case 170 /* FunctionType */:
                        case 171 /* ConstructorType */:
                        case 182 /* ParenthesizedType */:
                        case 198 /* TaggedTemplateExpression */:
                        case 206 /* AwaitExpression */:
                        case 261 /* NamedExports */:
                        case 257 /* NamedImports */:
                        case 263 /* ExportSpecifier */:
                        case 258 /* ImportSpecifier */:
                        case 159 /* PropertyDeclaration */:
                            return true;
                        case 242 /* VariableDeclaration */:
                        case 281 /* PropertyAssignment */:
                        case 209 /* BinaryExpression */:
                            if (!settings.indentMultiLineObjectLiteralBeginningOnBlankLine && sourceFile && childKind === 193 /* ObjectLiteralExpression */) { // TODO: GH#18217
                                return rangeIsOnOneLine(sourceFile, child);
                            }
                            if (parent.kind !== 209 /* BinaryExpression */) {
                                return true;
                            }
                            break;
                        case 228 /* DoStatement */:
                        case 229 /* WhileStatement */:
                        case 231 /* ForInStatement */:
                        case 232 /* ForOfStatement */:
                        case 230 /* ForStatement */:
                        case 227 /* IfStatement */:
                        case 244 /* FunctionDeclaration */:
                        case 201 /* FunctionExpression */:
                        case 161 /* MethodDeclaration */:
                        case 202 /* ArrowFunction */:
                        case 162 /* Constructor */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                            return childKind !== 223 /* Block */;
                        case 260 /* ExportDeclaration */:
                            return childKind !== 261 /* NamedExports */;
                        case 254 /* ImportDeclaration */:
                            return childKind !== 255 /* ImportClause */ ||
                                (!!child.namedBindings && child.namedBindings.kind !== 257 /* NamedImports */);
                        case 266 /* JsxElement */:
                            return childKind !== 269 /* JsxClosingElement */;
                        case 270 /* JsxFragment */:
                            return childKind !== 272 /* JsxClosingFragment */;
                        case 179 /* IntersectionType */:
                        case 178 /* UnionType */:
                            if (childKind === 173 /* TypeLiteral */) {
                                return false;
                            }
                        // falls through
                    }
                    // No explicit rule for given nodes so the result will follow the default value argument
                    return indentByDefault;
                }
                SmartIndenter.nodeWillIndentChild = nodeWillIndentChild;
                function isControlFlowEndingStatement(kind, parent) {
                    switch (kind) {
                        case 235 /* ReturnStatement */:
                        case 239 /* ThrowStatement */:
                        case 233 /* ContinueStatement */:
                        case 234 /* BreakStatement */:
                            return parent.kind !== 223 /* Block */;
                        default:
                            return false;
                    }
                }
                /**
                 * True when the parent node should indent the given child by an explicit rule.
                 * @param isNextChild If true, we are judging indent of a hypothetical child *after* this one, not the current child.
                 */
                function shouldIndentChildNode(settings, parent, child, sourceFile, isNextChild) {
                    if (isNextChild === void 0) {
                        isNextChild = false;
                    }
                    return nodeWillIndentChild(settings, parent, child, sourceFile, /*indentByDefault*/ false)
                        && !(isNextChild && child && isControlFlowEndingStatement(child.kind, parent));
                }
                SmartIndenter.shouldIndentChildNode = shouldIndentChildNode;
                function rangeIsOnOneLine(sourceFile, range) {
                    var rangeStart = ts.skipTrivia(sourceFile.text, range.pos);
                    var startLine = sourceFile.getLineAndCharacterOfPosition(rangeStart).line;
                    var endLine = sourceFile.getLineAndCharacterOfPosition(range.end).line;
                    return startLine === endLine;
                }
            })(SmartIndenter = formatting.SmartIndenter || (formatting.SmartIndenter = {}));
        })(formatting = ts.formatting || (ts.formatting = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var textChanges;
        (function (textChanges_3) {
            /**
             * Currently for simplicity we store recovered positions on the node itself.
             * It can be changed to side-table later if we decide that current design is too invasive.
             */
            function getPos(n) {
                var result = n.__pos;
                ts.Debug.assert(typeof result === "number");
                return result;
            }
            function setPos(n, pos) {
                ts.Debug.assert(typeof pos === "number");
                n.__pos = pos;
            }
            function getEnd(n) {
                var result = n.__end;
                ts.Debug.assert(typeof result === "number");
                return result;
            }
            function setEnd(n, end) {
                ts.Debug.assert(typeof end === "number");
                n.__end = end;
            }
            var LeadingTriviaOption;
            (function (LeadingTriviaOption) {
                /** Exclude all leading trivia (use getStart()) */
                LeadingTriviaOption[LeadingTriviaOption["Exclude"] = 0] = "Exclude";
                /** Include leading trivia and,
                 * if there are no line breaks between the node and the previous token,
                 * include all trivia between the node and the previous token
                 */
                LeadingTriviaOption[LeadingTriviaOption["IncludeAll"] = 1] = "IncludeAll";
            })(LeadingTriviaOption = textChanges_3.LeadingTriviaOption || (textChanges_3.LeadingTriviaOption = {}));
            var TrailingTriviaOption;
            (function (TrailingTriviaOption) {
                /** Exclude all trailing trivia (use getEnd()) */
                TrailingTriviaOption[TrailingTriviaOption["Exclude"] = 0] = "Exclude";
                /** Include trailing trivia */
                TrailingTriviaOption[TrailingTriviaOption["Include"] = 1] = "Include";
            })(TrailingTriviaOption = textChanges_3.TrailingTriviaOption || (textChanges_3.TrailingTriviaOption = {}));
            function skipWhitespacesAndLineBreaks(text, start) {
                return ts.skipTrivia(text, start, /*stopAfterLineBreak*/ false, /*stopAtComments*/ true);
            }
            function hasCommentsBeforeLineBreak(text, start) {
                var i = start;
                while (i < text.length) {
                    var ch = text.charCodeAt(i);
                    if (ts.isWhiteSpaceSingleLine(ch)) {
                        i++;
                        continue;
                    }
                    return ch === 47 /* slash */;
                }
                return false;
            }
            var useNonAdjustedPositions = {
                leadingTriviaOption: LeadingTriviaOption.Exclude,
                trailingTriviaOption: TrailingTriviaOption.Exclude,
            };
            var ChangeKind;
            (function (ChangeKind) {
                ChangeKind[ChangeKind["Remove"] = 0] = "Remove";
                ChangeKind[ChangeKind["ReplaceWithSingleNode"] = 1] = "ReplaceWithSingleNode";
                ChangeKind[ChangeKind["ReplaceWithMultipleNodes"] = 2] = "ReplaceWithMultipleNodes";
                ChangeKind[ChangeKind["Text"] = 3] = "Text";
            })(ChangeKind || (ChangeKind = {}));
            function getAdjustedRange(sourceFile, startNode, endNode, options) {
                return { pos: getAdjustedStartPosition(sourceFile, startNode, options), end: getAdjustedEndPosition(sourceFile, endNode, options) };
            }
            function getAdjustedStartPosition(sourceFile, node, options) {
                var leadingTriviaOption = options.leadingTriviaOption;
                if (leadingTriviaOption === LeadingTriviaOption.Exclude) {
                    return node.getStart(sourceFile);
                }
                var fullStart = node.getFullStart();
                var start = node.getStart(sourceFile);
                if (fullStart === start) {
                    return start;
                }
                var fullStartLine = ts.getLineStartPositionForPosition(fullStart, sourceFile);
                var startLine = ts.getLineStartPositionForPosition(start, sourceFile);
                if (startLine === fullStartLine) {
                    // full start and start of the node are on the same line
                    //   a,     b;
                    //    ^     ^
                    //    |   start
                    // fullstart
                    // when b is replaced - we usually want to keep the leading trvia
                    // when b is deleted - we delete it
                    return leadingTriviaOption === LeadingTriviaOption.IncludeAll ? fullStart : start;
                }
                // get start position of the line following the line that contains fullstart position
                // (but only if the fullstart isn't the very beginning of the file)
                var nextLineStart = fullStart > 0 ? 1 : 0;
                var adjustedStartPosition = ts.getStartPositionOfLine(ts.getLineOfLocalPosition(sourceFile, fullStartLine) + nextLineStart, sourceFile);
                // skip whitespaces/newlines
                adjustedStartPosition = skipWhitespacesAndLineBreaks(sourceFile.text, adjustedStartPosition);
                return ts.getStartPositionOfLine(ts.getLineOfLocalPosition(sourceFile, adjustedStartPosition), sourceFile);
            }
            function getAdjustedEndPosition(sourceFile, node, options) {
                var end = node.end;
                var trailingTriviaOption = options.trailingTriviaOption;
                if (trailingTriviaOption === TrailingTriviaOption.Exclude || (ts.isExpression(node) && trailingTriviaOption !== TrailingTriviaOption.Include)) {
                    return end;
                }
                var newEnd = ts.skipTrivia(sourceFile.text, end, /*stopAfterLineBreak*/ true);
                return newEnd !== end && (trailingTriviaOption === TrailingTriviaOption.Include || ts.isLineBreak(sourceFile.text.charCodeAt(newEnd - 1)))
                    ? newEnd
                    : end;
            }
            /**
             * Checks if 'candidate' argument is a legal separator in the list that contains 'node' as an element
             */
            function isSeparator(node, candidate) {
                return !!candidate && !!node.parent && (candidate.kind === 27 /* CommaToken */ || (candidate.kind === 26 /* SemicolonToken */ && node.parent.kind === 193 /* ObjectLiteralExpression */));
            }
            function spaces(count) {
                var s = "";
                for (var i = 0; i < count; i++) {
                    s += " ";
                }
                return s;
            }
            function isThisTypeAnnotatable(containingFunction) {
                return ts.isFunctionExpression(containingFunction) || ts.isFunctionDeclaration(containingFunction);
            }
            textChanges_3.isThisTypeAnnotatable = isThisTypeAnnotatable;
            var ChangeTracker = /** @class */ (function () {
                /** Public for tests only. Other callers should use `ChangeTracker.with`. */
                function ChangeTracker(newLineCharacter, formatContext) {
                    this.newLineCharacter = newLineCharacter;
                    this.formatContext = formatContext;
                    this.changes = [];
                    this.newFiles = [];
                    this.classesWithNodesInsertedAtStart = ts.createMap(); // Set<ClassDeclaration> implemented as Map<node id, ClassDeclaration>
                    this.deletedNodes = [];
                }
                ChangeTracker.fromContext = function (context) {
                    return new ChangeTracker(ts.getNewLineOrDefaultFromHost(context.host, context.formatContext.options), context.formatContext);
                };
                ChangeTracker.with = function (context, cb) {
                    var tracker = ChangeTracker.fromContext(context);
                    cb(tracker);
                    return tracker.getChanges();
                };
                ChangeTracker.prototype.pushRaw = function (sourceFile, change) {
                    ts.Debug.assertEqual(sourceFile.fileName, change.fileName);
                    for (var _i = 0, _a = change.textChanges; _i < _a.length; _i++) {
                        var c = _a[_i];
                        this.changes.push({
                            kind: ChangeKind.Text,
                            sourceFile: sourceFile,
                            text: c.newText,
                            range: ts.createTextRangeFromSpan(c.span),
                        });
                    }
                };
                ChangeTracker.prototype.deleteRange = function (sourceFile, range) {
                    this.changes.push({ kind: ChangeKind.Remove, sourceFile: sourceFile, range: range });
                };
                ChangeTracker.prototype.delete = function (sourceFile, node) {
                    this.deletedNodes.push({ sourceFile: sourceFile, node: node });
                };
                ChangeTracker.prototype.deleteModifier = function (sourceFile, modifier) {
                    this.deleteRange(sourceFile, { pos: modifier.getStart(sourceFile), end: ts.skipTrivia(sourceFile.text, modifier.end, /*stopAfterLineBreak*/ true) });
                };
                ChangeTracker.prototype.deleteNodeRange = function (sourceFile, startNode, endNode, options) {
                    if (options === void 0) {
                        options = { leadingTriviaOption: LeadingTriviaOption.IncludeAll };
                    }
                    var startPosition = getAdjustedStartPosition(sourceFile, startNode, options);
                    var endPosition = getAdjustedEndPosition(sourceFile, endNode, options);
                    this.deleteRange(sourceFile, { pos: startPosition, end: endPosition });
                };
                ChangeTracker.prototype.deleteNodeRangeExcludingEnd = function (sourceFile, startNode, afterEndNode, options) {
                    if (options === void 0) {
                        options = { leadingTriviaOption: LeadingTriviaOption.IncludeAll };
                    }
                    var startPosition = getAdjustedStartPosition(sourceFile, startNode, options);
                    var endPosition = afterEndNode === undefined ? sourceFile.text.length : getAdjustedStartPosition(sourceFile, afterEndNode, options);
                    this.deleteRange(sourceFile, { pos: startPosition, end: endPosition });
                };
                ChangeTracker.prototype.replaceRange = function (sourceFile, range, newNode, options) {
                    if (options === void 0) {
                        options = {};
                    }
                    this.changes.push({ kind: ChangeKind.ReplaceWithSingleNode, sourceFile: sourceFile, range: range, options: options, node: newNode });
                };
                ChangeTracker.prototype.replaceNode = function (sourceFile, oldNode, newNode, options) {
                    if (options === void 0) {
                        options = useNonAdjustedPositions;
                    }
                    this.replaceRange(sourceFile, getAdjustedRange(sourceFile, oldNode, oldNode, options), newNode, options);
                };
                ChangeTracker.prototype.replaceNodeRange = function (sourceFile, startNode, endNode, newNode, options) {
                    if (options === void 0) {
                        options = useNonAdjustedPositions;
                    }
                    this.replaceRange(sourceFile, getAdjustedRange(sourceFile, startNode, endNode, options), newNode, options);
                };
                ChangeTracker.prototype.replaceRangeWithNodes = function (sourceFile, range, newNodes, options) {
                    if (options === void 0) {
                        options = {};
                    }
                    this.changes.push({ kind: ChangeKind.ReplaceWithMultipleNodes, sourceFile: sourceFile, range: range, options: options, nodes: newNodes });
                };
                ChangeTracker.prototype.replaceNodeWithNodes = function (sourceFile, oldNode, newNodes, options) {
                    if (options === void 0) {
                        options = useNonAdjustedPositions;
                    }
                    this.replaceRangeWithNodes(sourceFile, getAdjustedRange(sourceFile, oldNode, oldNode, options), newNodes, options);
                };
                ChangeTracker.prototype.replaceNodeWithText = function (sourceFile, oldNode, text) {
                    this.replaceRangeWithText(sourceFile, getAdjustedRange(sourceFile, oldNode, oldNode, useNonAdjustedPositions), text);
                };
                ChangeTracker.prototype.replaceNodeRangeWithNodes = function (sourceFile, startNode, endNode, newNodes, options) {
                    if (options === void 0) {
                        options = useNonAdjustedPositions;
                    }
                    this.replaceRangeWithNodes(sourceFile, getAdjustedRange(sourceFile, startNode, endNode, options), newNodes, options);
                };
                ChangeTracker.prototype.nextCommaToken = function (sourceFile, node) {
                    var next = ts.findNextToken(node, node.parent, sourceFile);
                    return next && next.kind === 27 /* CommaToken */ ? next : undefined;
                };
                ChangeTracker.prototype.replacePropertyAssignment = function (sourceFile, oldNode, newNode) {
                    var suffix = this.nextCommaToken(sourceFile, oldNode) ? "" : ("," + this.newLineCharacter);
                    this.replaceNode(sourceFile, oldNode, newNode, { suffix: suffix });
                };
                ChangeTracker.prototype.insertNodeAt = function (sourceFile, pos, newNode, options) {
                    if (options === void 0) {
                        options = {};
                    }
                    this.replaceRange(sourceFile, ts.createRange(pos), newNode, options);
                };
                ChangeTracker.prototype.insertNodesAt = function (sourceFile, pos, newNodes, options) {
                    if (options === void 0) {
                        options = {};
                    }
                    this.replaceRangeWithNodes(sourceFile, ts.createRange(pos), newNodes, options);
                };
                ChangeTracker.prototype.insertNodeAtTopOfFile = function (sourceFile, newNode, blankLineBetween) {
                    var pos = getInsertionPositionAtSourceFileTop(sourceFile);
                    this.insertNodeAt(sourceFile, pos, newNode, {
                        prefix: pos === 0 ? undefined : this.newLineCharacter,
                        suffix: (ts.isLineBreak(sourceFile.text.charCodeAt(pos)) ? "" : this.newLineCharacter) + (blankLineBetween ? this.newLineCharacter : ""),
                    });
                };
                ChangeTracker.prototype.insertNodeBefore = function (sourceFile, before, newNode, blankLineBetween) {
                    if (blankLineBetween === void 0) {
                        blankLineBetween = false;
                    }
                    this.insertNodeAt(sourceFile, getAdjustedStartPosition(sourceFile, before, {}), newNode, this.getOptionsForInsertNodeBefore(before, blankLineBetween));
                };
                ChangeTracker.prototype.insertModifierBefore = function (sourceFile, modifier, before) {
                    var pos = before.getStart(sourceFile);
                    this.insertNodeAt(sourceFile, pos, ts.createToken(modifier), { suffix: " " });
                };
                ChangeTracker.prototype.insertLastModifierBefore = function (sourceFile, modifier, before) {
                    if (!before.modifiers) {
                        this.insertModifierBefore(sourceFile, modifier, before);
                        return;
                    }
                    var pos = before.modifiers.end;
                    this.insertNodeAt(sourceFile, pos, ts.createToken(modifier), { prefix: " " });
                };
                ChangeTracker.prototype.insertCommentBeforeLine = function (sourceFile, lineNumber, position, commentText) {
                    var lineStartPosition = ts.getStartPositionOfLine(lineNumber, sourceFile);
                    var startPosition = ts.getFirstNonSpaceCharacterPosition(sourceFile.text, lineStartPosition);
                    // First try to see if we can put the comment on the previous line.
                    // We need to make sure that we are not in the middle of a string literal or a comment.
                    // If so, we do not want to separate the node from its comment if we can.
                    // Otherwise, add an extra new line immediately before the error span.
                    var insertAtLineStart = isValidLocationToAddComment(sourceFile, startPosition);
                    var token = ts.getTouchingToken(sourceFile, insertAtLineStart ? startPosition : position);
                    var indent = sourceFile.text.slice(lineStartPosition, startPosition);
                    var text = (insertAtLineStart ? "" : this.newLineCharacter) + "//" + commentText + this.newLineCharacter + indent;
                    this.insertText(sourceFile, token.getStart(sourceFile), text);
                };
                ChangeTracker.prototype.insertJsdocCommentBefore = function (sourceFile, node, tag) {
                    var fnStart = node.getStart(sourceFile);
                    if (node.jsDoc) {
                        for (var _i = 0, _a = node.jsDoc; _i < _a.length; _i++) {
                            var jsdoc = _a[_i];
                            this.deleteRange(sourceFile, {
                                pos: ts.getLineStartPositionForPosition(jsdoc.getStart(sourceFile), sourceFile),
                                end: getAdjustedEndPosition(sourceFile, jsdoc, /*options*/ {})
                            });
                        }
                    }
                    var startPosition = ts.getPrecedingNonSpaceCharacterPosition(sourceFile.text, fnStart - 1);
                    var indent = sourceFile.text.slice(startPosition, fnStart);
                    this.insertNodeAt(sourceFile, fnStart, tag, { preserveLeadingWhitespace: false, suffix: this.newLineCharacter + indent });
                };
                ChangeTracker.prototype.replaceRangeWithText = function (sourceFile, range, text) {
                    this.changes.push({ kind: ChangeKind.Text, sourceFile: sourceFile, range: range, text: text });
                };
                ChangeTracker.prototype.insertText = function (sourceFile, pos, text) {
                    this.replaceRangeWithText(sourceFile, ts.createRange(pos), text);
                };
                /** Prefer this over replacing a node with another that has a type annotation, as it avoids reformatting the other parts of the node. */
                ChangeTracker.prototype.tryInsertTypeAnnotation = function (sourceFile, node, type) {
                    var endNode;
                    if (ts.isFunctionLike(node)) {
                        endNode = ts.findChildOfKind(node, 21 /* CloseParenToken */, sourceFile);
                        if (!endNode) {
                            if (!ts.isArrowFunction(node))
                                return false; // Function missing parentheses, give up
                            // If no `)`, is an arrow function `x => x`, so use the end of the first parameter
                            endNode = ts.first(node.parameters);
                        }
                    }
                    else {
                        endNode = node.kind !== 242 /* VariableDeclaration */ && node.questionToken ? node.questionToken : node.name;
                    }
                    this.insertNodeAt(sourceFile, endNode.end, type, { prefix: ": " });
                    return true;
                };
                ChangeTracker.prototype.tryInsertThisTypeAnnotation = function (sourceFile, node, type) {
                    var start = ts.findChildOfKind(node, 20 /* OpenParenToken */, sourceFile).getStart(sourceFile) + 1;
                    var suffix = node.parameters.length ? ", " : "";
                    this.insertNodeAt(sourceFile, start, type, { prefix: "this: ", suffix: suffix });
                };
                ChangeTracker.prototype.insertTypeParameters = function (sourceFile, node, typeParameters) {
                    // If no `(`, is an arrow function `x => x`, so use the pos of the first parameter
                    var start = (ts.findChildOfKind(node, 20 /* OpenParenToken */, sourceFile) || ts.first(node.parameters)).getStart(sourceFile);
                    this.insertNodesAt(sourceFile, start, typeParameters, { prefix: "<", suffix: ">" });
                };
                ChangeTracker.prototype.getOptionsForInsertNodeBefore = function (before, doubleNewlines) {
                    if (ts.isStatement(before) || ts.isClassElement(before)) {
                        return { suffix: doubleNewlines ? this.newLineCharacter + this.newLineCharacter : this.newLineCharacter };
                    }
                    else if (ts.isVariableDeclaration(before)) { // insert `x = 1, ` into `const x = 1, y = 2;
                        return { suffix: ", " };
                    }
                    else if (ts.isParameter(before)) {
                        return {};
                    }
                    else if (ts.isStringLiteral(before) && ts.isImportDeclaration(before.parent) || ts.isNamedImports(before)) {
                        return { suffix: ", " };
                    }
                    return ts.Debug.failBadSyntaxKind(before); // We haven't handled this kind of node yet -- add it
                };
                ChangeTracker.prototype.insertNodeAtConstructorStart = function (sourceFile, ctr, newStatement) {
                    var firstStatement = ts.firstOrUndefined(ctr.body.statements);
                    if (!firstStatement || !ctr.body.multiLine) {
                        this.replaceConstructorBody(sourceFile, ctr, __spreadArrays([newStatement], ctr.body.statements));
                    }
                    else {
                        this.insertNodeBefore(sourceFile, firstStatement, newStatement);
                    }
                };
                ChangeTracker.prototype.insertNodeAtConstructorEnd = function (sourceFile, ctr, newStatement) {
                    var lastStatement = ts.lastOrUndefined(ctr.body.statements);
                    if (!lastStatement || !ctr.body.multiLine) {
                        this.replaceConstructorBody(sourceFile, ctr, __spreadArrays(ctr.body.statements, [newStatement]));
                    }
                    else {
                        this.insertNodeAfter(sourceFile, lastStatement, newStatement);
                    }
                };
                ChangeTracker.prototype.replaceConstructorBody = function (sourceFile, ctr, statements) {
                    this.replaceNode(sourceFile, ctr.body, ts.createBlock(statements, /*multiLine*/ true));
                };
                ChangeTracker.prototype.insertNodeAtEndOfScope = function (sourceFile, scope, newNode) {
                    var pos = getAdjustedStartPosition(sourceFile, scope.getLastToken(), {});
                    this.insertNodeAt(sourceFile, pos, newNode, {
                        prefix: ts.isLineBreak(sourceFile.text.charCodeAt(scope.getLastToken().pos)) ? this.newLineCharacter : this.newLineCharacter + this.newLineCharacter,
                        suffix: this.newLineCharacter
                    });
                };
                ChangeTracker.prototype.insertNodeAtClassStart = function (sourceFile, cls, newElement) {
                    this.insertNodeAtStartWorker(sourceFile, cls, newElement);
                };
                ChangeTracker.prototype.insertNodeAtObjectStart = function (sourceFile, obj, newElement) {
                    this.insertNodeAtStartWorker(sourceFile, obj, newElement);
                };
                ChangeTracker.prototype.insertNodeAtStartWorker = function (sourceFile, cls, newElement) {
                    var _a;
                    var indentation = (_a = this.guessIndentationFromExistingMembers(sourceFile, cls)) !== null && _a !== void 0 ? _a : this.computeIndentationForNewMember(sourceFile, cls);
                    this.insertNodeAt(sourceFile, getMembersOrProperties(cls).pos, newElement, this.getInsertNodeAtStartInsertOptions(sourceFile, cls, indentation));
                };
                /**
                 * Tries to guess the indentation from the existing members of a class/interface/object. All members must be on
                 * new lines and must share the same indentation.
                 */
                ChangeTracker.prototype.guessIndentationFromExistingMembers = function (sourceFile, cls) {
                    var indentation;
                    var lastRange = cls;
                    for (var _i = 0, _a = getMembersOrProperties(cls); _i < _a.length; _i++) {
                        var member = _a[_i];
                        if (ts.rangeStartPositionsAreOnSameLine(lastRange, member, sourceFile)) {
                            // each indented member must be on a new line
                            return undefined;
                        }
                        var memberStart = member.getStart(sourceFile);
                        var memberIndentation = ts.formatting.SmartIndenter.findFirstNonWhitespaceColumn(ts.getLineStartPositionForPosition(memberStart, sourceFile), memberStart, sourceFile, this.formatContext.options);
                        if (indentation === undefined) {
                            indentation = memberIndentation;
                        }
                        else if (memberIndentation !== indentation) {
                            // indentation of multiple members is not consistent
                            return undefined;
                        }
                        lastRange = member;
                    }
                    return indentation;
                };
                ChangeTracker.prototype.computeIndentationForNewMember = function (sourceFile, cls) {
                    var _a;
                    var clsStart = cls.getStart(sourceFile);
                    return ts.formatting.SmartIndenter.findFirstNonWhitespaceColumn(ts.getLineStartPositionForPosition(clsStart, sourceFile), clsStart, sourceFile, this.formatContext.options)
                        + ((_a = this.formatContext.options.indentSize) !== null && _a !== void 0 ? _a : 4);
                };
                ChangeTracker.prototype.getInsertNodeAtStartInsertOptions = function (sourceFile, cls, indentation) {
                    // Rules:
                    // - Always insert leading newline.
                    // - For object literals:
                    //   - Add a trailing comma if there are existing members in the node, or the source file is not a JSON file
                    //     (because trailing commas are generally illegal in a JSON file).
                    //   - Add a leading comma if the source file is not a JSON file, there are existing insertions,
                    //     and the node is empty (because we didn't add a trailing comma per the previous rule).
                    // - Only insert a trailing newline if body is single-line and there are no other insertions for the node.
                    //   NOTE: This is handled in `finishClassesWithNodesInsertedAtStart`.
                    var members = getMembersOrProperties(cls);
                    var isEmpty = members.length === 0;
                    var isFirstInsertion = ts.addToSeen(this.classesWithNodesInsertedAtStart, ts.getNodeId(cls), { node: cls, sourceFile: sourceFile });
                    var insertTrailingComma = ts.isObjectLiteralExpression(cls) && (!ts.isJsonSourceFile(sourceFile) || !isEmpty);
                    var insertLeadingComma = ts.isObjectLiteralExpression(cls) && ts.isJsonSourceFile(sourceFile) && isEmpty && !isFirstInsertion;
                    return {
                        indentation: indentation,
                        prefix: (insertLeadingComma ? "," : "") + this.newLineCharacter,
                        suffix: insertTrailingComma ? "," : ""
                    };
                };
                ChangeTracker.prototype.insertNodeAfterComma = function (sourceFile, after, newNode) {
                    var endPosition = this.insertNodeAfterWorker(sourceFile, this.nextCommaToken(sourceFile, after) || after, newNode);
                    this.insertNodeAt(sourceFile, endPosition, newNode, this.getInsertNodeAfterOptions(sourceFile, after));
                };
                ChangeTracker.prototype.insertNodeAfter = function (sourceFile, after, newNode) {
                    var endPosition = this.insertNodeAfterWorker(sourceFile, after, newNode);
                    this.insertNodeAt(sourceFile, endPosition, newNode, this.getInsertNodeAfterOptions(sourceFile, after));
                };
                ChangeTracker.prototype.insertNodeAtEndOfList = function (sourceFile, list, newNode) {
                    this.insertNodeAt(sourceFile, list.end, newNode, { prefix: ", " });
                };
                ChangeTracker.prototype.insertNodesAfter = function (sourceFile, after, newNodes) {
                    var endPosition = this.insertNodeAfterWorker(sourceFile, after, ts.first(newNodes));
                    this.insertNodesAt(sourceFile, endPosition, newNodes, this.getInsertNodeAfterOptions(sourceFile, after));
                };
                ChangeTracker.prototype.insertNodeAfterWorker = function (sourceFile, after, newNode) {
                    if (needSemicolonBetween(after, newNode)) {
                        // check if previous statement ends with semicolon
                        // if not - insert semicolon to preserve the code from changing the meaning due to ASI
                        if (sourceFile.text.charCodeAt(after.end - 1) !== 59 /* semicolon */) {
                            this.replaceRange(sourceFile, ts.createRange(after.end), ts.createToken(26 /* SemicolonToken */));
                        }
                    }
                    var endPosition = getAdjustedEndPosition(sourceFile, after, {});
                    return endPosition;
                };
                ChangeTracker.prototype.getInsertNodeAfterOptions = function (sourceFile, after) {
                    var options = this.getInsertNodeAfterOptionsWorker(after);
                    return __assign(__assign({}, options), { prefix: after.end === sourceFile.end && ts.isStatement(after) ? (options.prefix ? "\n" + options.prefix : "\n") : options.prefix });
                };
                ChangeTracker.prototype.getInsertNodeAfterOptionsWorker = function (node) {
                    switch (node.kind) {
                        case 245 /* ClassDeclaration */:
                        case 249 /* ModuleDeclaration */:
                            return { prefix: this.newLineCharacter, suffix: this.newLineCharacter };
                        case 242 /* VariableDeclaration */:
                        case 10 /* StringLiteral */:
                        case 75 /* Identifier */:
                            return { prefix: ", " };
                        case 281 /* PropertyAssignment */:
                            return { suffix: "," + this.newLineCharacter };
                        case 89 /* ExportKeyword */:
                            return { prefix: " " };
                        case 156 /* Parameter */:
                            return {};
                        default:
                            ts.Debug.assert(ts.isStatement(node) || ts.isClassOrTypeElement(node)); // Else we haven't handled this kind of node yet -- add it
                            return { suffix: this.newLineCharacter };
                    }
                };
                ChangeTracker.prototype.insertName = function (sourceFile, node, name) {
                    ts.Debug.assert(!node.name);
                    if (node.kind === 202 /* ArrowFunction */) {
                        var arrow = ts.findChildOfKind(node, 38 /* EqualsGreaterThanToken */, sourceFile);
                        var lparen = ts.findChildOfKind(node, 20 /* OpenParenToken */, sourceFile);
                        if (lparen) {
                            // `() => {}` --> `function f() {}`
                            this.insertNodesAt(sourceFile, lparen.getStart(sourceFile), [ts.createToken(94 /* FunctionKeyword */), ts.createIdentifier(name)], { joiner: " " });
                            deleteNode(this, sourceFile, arrow);
                        }
                        else {
                            // `x => {}` -> `function f(x) {}`
                            this.insertText(sourceFile, ts.first(node.parameters).getStart(sourceFile), "function " + name + "(");
                            // Replacing full range of arrow to get rid of the leading space -- replace ` =>` with `)`
                            this.replaceRange(sourceFile, arrow, ts.createToken(21 /* CloseParenToken */));
                        }
                        if (node.body.kind !== 223 /* Block */) {
                            // `() => 0` => `function f() { return 0; }`
                            this.insertNodesAt(sourceFile, node.body.getStart(sourceFile), [ts.createToken(18 /* OpenBraceToken */), ts.createToken(101 /* ReturnKeyword */)], { joiner: " ", suffix: " " });
                            this.insertNodesAt(sourceFile, node.body.end, [ts.createToken(26 /* SemicolonToken */), ts.createToken(19 /* CloseBraceToken */)], { joiner: " " });
                        }
                    }
                    else {
                        var pos = ts.findChildOfKind(node, node.kind === 201 /* FunctionExpression */ ? 94 /* FunctionKeyword */ : 80 /* ClassKeyword */, sourceFile).end;
                        this.insertNodeAt(sourceFile, pos, ts.createIdentifier(name), { prefix: " " });
                    }
                };
                ChangeTracker.prototype.insertExportModifier = function (sourceFile, node) {
                    this.insertText(sourceFile, node.getStart(sourceFile), "export ");
                };
                /**
                 * This function should be used to insert nodes in lists when nodes don't carry separators as the part of the node range,
                 * i.e. arguments in arguments lists, parameters in parameter lists etc.
                 * Note that separators are part of the node in statements and class elements.
                 */
                ChangeTracker.prototype.insertNodeInListAfter = function (sourceFile, after, newNode, containingList) {
                    if (containingList === void 0) {
                        containingList = ts.formatting.SmartIndenter.getContainingList(after, sourceFile);
                    }
                    if (!containingList) {
                        ts.Debug.fail("node is not a list element");
                        return;
                    }
                    var index = ts.indexOfNode(containingList, after);
                    if (index < 0) {
                        return;
                    }
                    var end = after.getEnd();
                    if (index !== containingList.length - 1) {
                        // any element except the last one
                        // use next sibling as an anchor
                        var nextToken = ts.getTokenAtPosition(sourceFile, after.end);
                        if (nextToken && isSeparator(after, nextToken)) {
                            // for list
                            // a, b, c
                            // create change for adding 'e' after 'a' as
                            // - find start of next element after a (it is b)
                            // - use this start as start and end position in final change
                            // - build text of change by formatting the text of node + separator + whitespace trivia of b
                            // in multiline case it will work as
                            //   a,
                            //   b,
                            //   c,
                            // result - '*' denotes leading trivia that will be inserted after new text (displayed as '#')
                            //   a,*
                            // ***insertedtext<separator>#
                            // ###b,
                            //   c,
                            // find line and character of the next element
                            var lineAndCharOfNextElement = ts.getLineAndCharacterOfPosition(sourceFile, skipWhitespacesAndLineBreaks(sourceFile.text, containingList[index + 1].getFullStart()));
                            // find line and character of the token that precedes next element (usually it is separator)
                            var lineAndCharOfNextToken = ts.getLineAndCharacterOfPosition(sourceFile, nextToken.end);
                            var prefix = void 0;
                            var startPos = void 0;
                            if (lineAndCharOfNextToken.line === lineAndCharOfNextElement.line) {
                                // next element is located on the same line with separator:
                                // a,$$$$b
                                //  ^    ^
                                //  |    |-next element
                                //  |-separator
                                // where $$$ is some leading trivia
                                // for a newly inserted node we'll maintain the same relative position comparing to separator and replace leading trivia with spaces
                                // a,    x,$$$$b
                                //  ^    ^     ^
                                //  |    |     |-next element
                                //  |    |-new inserted node padded with spaces
                                //  |-separator
                                startPos = nextToken.end;
                                prefix = spaces(lineAndCharOfNextElement.character - lineAndCharOfNextToken.character);
                            }
                            else {
                                // next element is located on different line that separator
                                // let insert position be the beginning of the line that contains next element
                                startPos = ts.getStartPositionOfLine(lineAndCharOfNextElement.line, sourceFile);
                            }
                            // write separator and leading trivia of the next element as suffix
                            var suffix = "" + ts.tokenToString(nextToken.kind) + sourceFile.text.substring(nextToken.end, containingList[index + 1].getStart(sourceFile));
                            this.replaceRange(sourceFile, ts.createRange(startPos, containingList[index + 1].getStart(sourceFile)), newNode, { prefix: prefix, suffix: suffix });
                        }
                    }
                    else {
                        var afterStart = after.getStart(sourceFile);
                        var afterStartLinePosition = ts.getLineStartPositionForPosition(afterStart, sourceFile);
                        var separator = void 0;
                        var multilineList = false;
                        // insert element after the last element in the list that has more than one item
                        // pick the element preceding the after element to:
                        // - pick the separator
                        // - determine if list is a multiline
                        if (containingList.length === 1) {
                            // if list has only one element then we'll format is as multiline if node has comment in trailing trivia, or as singleline otherwise
                            // i.e. var x = 1 // this is x
                            //     | new element will be inserted at this position
                            separator = 27 /* CommaToken */;
                        }
                        else {
                            // element has more than one element, pick separator from the list
                            var tokenBeforeInsertPosition = ts.findPrecedingToken(after.pos, sourceFile);
                            separator = isSeparator(after, tokenBeforeInsertPosition) ? tokenBeforeInsertPosition.kind : 27 /* CommaToken */;
                            // determine if list is multiline by checking lines of after element and element that precedes it.
                            var afterMinusOneStartLinePosition = ts.getLineStartPositionForPosition(containingList[index - 1].getStart(sourceFile), sourceFile);
                            multilineList = afterMinusOneStartLinePosition !== afterStartLinePosition;
                        }
                        if (hasCommentsBeforeLineBreak(sourceFile.text, after.end)) {
                            // in this case we'll always treat containing list as multiline
                            multilineList = true;
                        }
                        if (multilineList) {
                            // insert separator immediately following the 'after' node to preserve comments in trailing trivia
                            this.replaceRange(sourceFile, ts.createRange(end), ts.createToken(separator));
                            // use the same indentation as 'after' item
                            var indentation = ts.formatting.SmartIndenter.findFirstNonWhitespaceColumn(afterStartLinePosition, afterStart, sourceFile, this.formatContext.options);
                            // insert element before the line break on the line that contains 'after' element
                            var insertPos = ts.skipTrivia(sourceFile.text, end, /*stopAfterLineBreak*/ true, /*stopAtComments*/ false);
                            if (insertPos !== end && ts.isLineBreak(sourceFile.text.charCodeAt(insertPos - 1))) {
                                insertPos--;
                            }
                            this.replaceRange(sourceFile, ts.createRange(insertPos), newNode, { indentation: indentation, prefix: this.newLineCharacter });
                        }
                        else {
                            this.replaceRange(sourceFile, ts.createRange(end), newNode, { prefix: ts.tokenToString(separator) + " " });
                        }
                    }
                };
                ChangeTracker.prototype.parenthesizeExpression = function (sourceFile, expression) {
                    this.replaceRange(sourceFile, ts.rangeOfNode(expression), ts.createParen(expression));
                };
                ChangeTracker.prototype.finishClassesWithNodesInsertedAtStart = function () {
                    var _this = this;
                    this.classesWithNodesInsertedAtStart.forEach(function (_a) {
                        var node = _a.node, sourceFile = _a.sourceFile;
                        var _b = getClassOrObjectBraceEnds(node, sourceFile), openBraceEnd = _b[0], closeBraceEnd = _b[1];
                        if (openBraceEnd !== undefined && closeBraceEnd !== undefined) {
                            var isEmpty = getMembersOrProperties(node).length === 0;
                            var isSingleLine = ts.positionsAreOnSameLine(openBraceEnd, closeBraceEnd, sourceFile);
                            if (isEmpty && isSingleLine && openBraceEnd !== closeBraceEnd - 1) {
                                // For `class C { }` remove the whitespace inside the braces.
                                _this.deleteRange(sourceFile, ts.createRange(openBraceEnd, closeBraceEnd - 1));
                            }
                            if (isSingleLine) {
                                _this.insertText(sourceFile, closeBraceEnd - 1, _this.newLineCharacter);
                            }
                        }
                    });
                };
                ChangeTracker.prototype.finishDeleteDeclarations = function () {
                    var _this = this;
                    var deletedNodesInLists = new ts.NodeSet(); // Stores nodes in lists that we already deleted. Used to avoid deleting `, ` twice in `a, b`.
                    var _loop_9 = function (sourceFile, node) {
                        if (!this_1.deletedNodes.some(function (d) { return d.sourceFile === sourceFile && ts.rangeContainsRangeExclusive(d.node, node); })) {
                            if (ts.isArray(node)) {
                                this_1.deleteRange(sourceFile, ts.rangeOfTypeParameters(node));
                            }
                            else {
                                deleteDeclaration.deleteDeclaration(this_1, deletedNodesInLists, sourceFile, node);
                            }
                        }
                    };
                    var this_1 = this;
                    for (var _i = 0, _a = this.deletedNodes; _i < _a.length; _i++) {
                        var _b = _a[_i], sourceFile = _b.sourceFile, node = _b.node;
                        _loop_9(sourceFile, node);
                    }
                    deletedNodesInLists.forEach(function (node) {
                        var sourceFile = node.getSourceFile();
                        var list = ts.formatting.SmartIndenter.getContainingList(node, sourceFile);
                        if (node !== ts.last(list))
                            return;
                        var lastNonDeletedIndex = ts.findLastIndex(list, function (n) { return !deletedNodesInLists.has(n); }, list.length - 2);
                        if (lastNonDeletedIndex !== -1) {
                            _this.deleteRange(sourceFile, { pos: list[lastNonDeletedIndex].end, end: startPositionToDeleteNodeInList(sourceFile, list[lastNonDeletedIndex + 1]) });
                        }
                    });
                };
                /**
                 * Note: after calling this, the TextChanges object must be discarded!
                 * @param validate only for tests
                 *    The reason we must validate as part of this method is that `getNonFormattedText` changes the node's positions,
                 *    so we can only call this once and can't get the non-formatted text separately.
                 */
                ChangeTracker.prototype.getChanges = function (validate) {
                    this.finishDeleteDeclarations();
                    this.finishClassesWithNodesInsertedAtStart();
                    var changes = changesToText.getTextChangesFromChanges(this.changes, this.newLineCharacter, this.formatContext, validate);
                    for (var _i = 0, _a = this.newFiles; _i < _a.length; _i++) {
                        var _b = _a[_i], oldFile = _b.oldFile, fileName = _b.fileName, statements = _b.statements;
                        changes.push(changesToText.newFileChanges(oldFile, fileName, statements, this.newLineCharacter, this.formatContext));
                    }
                    return changes;
                };
                ChangeTracker.prototype.createNewFile = function (oldFile, fileName, statements) {
                    this.newFiles.push({ oldFile: oldFile, fileName: fileName, statements: statements });
                };
                return ChangeTracker;
            }());
            textChanges_3.ChangeTracker = ChangeTracker;
            // find first non-whitespace position in the leading trivia of the node
            function startPositionToDeleteNodeInList(sourceFile, node) {
                return ts.skipTrivia(sourceFile.text, getAdjustedStartPosition(sourceFile, node, { leadingTriviaOption: LeadingTriviaOption.IncludeAll }), /*stopAfterLineBreak*/ false, /*stopAtComments*/ true);
            }
            function getClassOrObjectBraceEnds(cls, sourceFile) {
                var open = ts.findChildOfKind(cls, 18 /* OpenBraceToken */, sourceFile);
                var close = ts.findChildOfKind(cls, 19 /* CloseBraceToken */, sourceFile);
                return [open === null || open === void 0 ? void 0 : open.end, close === null || close === void 0 ? void 0 : close.end];
            }
            function getMembersOrProperties(cls) {
                return ts.isObjectLiteralExpression(cls) ? cls.properties : cls.members;
            }
            function getNewFileText(statements, scriptKind, newLineCharacter, formatContext) {
                return changesToText.newFileChangesWorker(/*oldFile*/ undefined, scriptKind, statements, newLineCharacter, formatContext);
            }
            textChanges_3.getNewFileText = getNewFileText;
            var changesToText;
            (function (changesToText) {
                function getTextChangesFromChanges(changes, newLineCharacter, formatContext, validate) {
                    return ts.group(changes, function (c) { return c.sourceFile.path; }).map(function (changesInFile) {
                        var sourceFile = changesInFile[0].sourceFile;
                        // order changes by start position
                        // If the start position is the same, put the shorter range first, since an empty range (x, x) may precede (x, y) but not vice-versa.
                        var normalized = ts.stableSort(changesInFile, function (a, b) { return (a.range.pos - b.range.pos) || (a.range.end - b.range.end); });
                        var _loop_10 = function (i) {
                            ts.Debug.assert(normalized[i].range.end <= normalized[i + 1].range.pos, "Changes overlap", function () {
                                return JSON.stringify(normalized[i].range) + " and " + JSON.stringify(normalized[i + 1].range);
                            });
                        };
                        // verify that change intervals do not overlap, except possibly at end points.
                        for (var i = 0; i < normalized.length - 1; i++) {
                            _loop_10(i);
                        }
                        var textChanges = normalized.map(function (c) {
                            return ts.createTextChange(ts.createTextSpanFromRange(c.range), computeNewText(c, sourceFile, newLineCharacter, formatContext, validate));
                        });
                        return { fileName: sourceFile.fileName, textChanges: textChanges };
                    });
                }
                changesToText.getTextChangesFromChanges = getTextChangesFromChanges;
                function newFileChanges(oldFile, fileName, statements, newLineCharacter, formatContext) {
                    var text = newFileChangesWorker(oldFile, ts.getScriptKindFromFileName(fileName), statements, newLineCharacter, formatContext);
                    return { fileName: fileName, textChanges: [ts.createTextChange(ts.createTextSpan(0, 0), text)], isNewFile: true };
                }
                changesToText.newFileChanges = newFileChanges;
                function newFileChangesWorker(oldFile, scriptKind, statements, newLineCharacter, formatContext) {
                    // TODO: this emits the file, parses it back, then formats it that -- may be a less roundabout way to do this
                    var nonFormattedText = statements.map(function (s) { return getNonformattedText(s, oldFile, newLineCharacter).text; }).join(newLineCharacter);
                    var sourceFile = ts.createSourceFile("any file name", nonFormattedText, 99 /* ESNext */, /*setParentNodes*/ true, scriptKind);
                    var changes = ts.formatting.formatDocument(sourceFile, formatContext);
                    return applyChanges(nonFormattedText, changes) + newLineCharacter;
                }
                changesToText.newFileChangesWorker = newFileChangesWorker;
                function computeNewText(change, sourceFile, newLineCharacter, formatContext, validate) {
                    if (change.kind === ChangeKind.Remove) {
                        return "";
                    }
                    if (change.kind === ChangeKind.Text) {
                        return change.text;
                    }
                    var _a = change.options, options = _a === void 0 ? {} : _a, pos = change.range.pos;
                    var format = function (n) { return getFormattedTextOfNode(n, sourceFile, pos, options, newLineCharacter, formatContext, validate); };
                    var text = change.kind === ChangeKind.ReplaceWithMultipleNodes
                        ? change.nodes.map(function (n) { return ts.removeSuffix(format(n), newLineCharacter); }).join(change.options.joiner || newLineCharacter) // TODO: GH#18217
                        : format(change.node);
                    // strip initial indentation (spaces or tabs) if text will be inserted in the middle of the line
                    var noIndent = (options.preserveLeadingWhitespace || options.indentation !== undefined || ts.getLineStartPositionForPosition(pos, sourceFile) === pos) ? text : text.replace(/^\s+/, "");
                    return (options.prefix || "") + noIndent + (options.suffix || "");
                }
                function getFormatCodeSettingsForWriting(_a, sourceFile) {
                    var options = _a.options;
                    var shouldAutoDetectSemicolonPreference = !options.semicolons || options.semicolons === ts.SemicolonPreference.Ignore;
                    var shouldRemoveSemicolons = options.semicolons === ts.SemicolonPreference.Remove || shouldAutoDetectSemicolonPreference && !ts.probablyUsesSemicolons(sourceFile);
                    return __assign(__assign({}, options), { semicolons: shouldRemoveSemicolons ? ts.SemicolonPreference.Remove : ts.SemicolonPreference.Ignore });
                }
                /** Note: this may mutate `nodeIn`. */
                function getFormattedTextOfNode(nodeIn, sourceFile, pos, _a, newLineCharacter, formatContext, validate) {
                    var indentation = _a.indentation, prefix = _a.prefix, delta = _a.delta;
                    var _b = getNonformattedText(nodeIn, sourceFile, newLineCharacter), node = _b.node, text = _b.text;
                    if (validate)
                        validate(node, text);
                    var formatOptions = getFormatCodeSettingsForWriting(formatContext, sourceFile);
                    var initialIndentation = indentation !== undefined
                        ? indentation
                        : ts.formatting.SmartIndenter.getIndentation(pos, sourceFile, formatOptions, prefix === newLineCharacter || ts.getLineStartPositionForPosition(pos, sourceFile) === pos);
                    if (delta === undefined) {
                        delta = ts.formatting.SmartIndenter.shouldIndentChildNode(formatOptions, nodeIn) ? (formatOptions.indentSize || 0) : 0;
                    }
                    var file = { text: text, getLineAndCharacterOfPosition: function (pos) { return ts.getLineAndCharacterOfPosition(this, pos); } };
                    var changes = ts.formatting.formatNodeGivenIndentation(node, file, sourceFile.languageVariant, initialIndentation, delta, __assign(__assign({}, formatContext), { options: formatOptions }));
                    return applyChanges(text, changes);
                }
                /** Note: output node may be mutated input node. */
                function getNonformattedText(node, sourceFile, newLineCharacter) {
                    var writer = createWriter(newLineCharacter);
                    var newLine = newLineCharacter === "\n" ? 1 /* LineFeed */ : 0 /* CarriageReturnLineFeed */;
                    ts.createPrinter({ newLine: newLine, neverAsciiEscape: true }, writer).writeNode(4 /* Unspecified */, node, sourceFile, writer);
                    return { text: writer.getText(), node: assignPositionsToNode(node) };
                }
                changesToText.getNonformattedText = getNonformattedText;
            })(changesToText || (changesToText = {}));
            function applyChanges(text, changes) {
                for (var i = changes.length - 1; i >= 0; i--) {
                    var _a = changes[i], span = _a.span, newText = _a.newText;
                    text = "" + text.substring(0, span.start) + newText + text.substring(ts.textSpanEnd(span));
                }
                return text;
            }
            textChanges_3.applyChanges = applyChanges;
            function isTrivia(s) {
                return ts.skipTrivia(s, 0) === s.length;
            }
            function assignPositionsToNode(node) {
                var visited = ts.visitEachChild(node, assignPositionsToNode, ts.nullTransformationContext, assignPositionsToNodeArray, assignPositionsToNode); // TODO: GH#18217
                // create proxy node for non synthesized nodes
                var newNode = ts.nodeIsSynthesized(visited) ? visited : Object.create(visited);
                newNode.pos = getPos(node);
                newNode.end = getEnd(node);
                return newNode;
            }
            function assignPositionsToNodeArray(nodes, visitor, test, start, count) {
                var visited = ts.visitNodes(nodes, visitor, test, start, count);
                if (!visited) {
                    return visited;
                }
                // clone nodearray if necessary
                var nodeArray = visited === nodes ? ts.createNodeArray(visited.slice(0)) : visited;
                nodeArray.pos = getPos(nodes);
                nodeArray.end = getEnd(nodes);
                return nodeArray;
            }
            function createWriter(newLine) {
                var lastNonTriviaPosition = 0;
                var writer = ts.createTextWriter(newLine);
                var onEmitNode = function (hint, node, printCallback) {
                    if (node) {
                        setPos(node, lastNonTriviaPosition);
                    }
                    printCallback(hint, node);
                    if (node) {
                        setEnd(node, lastNonTriviaPosition);
                    }
                };
                var onBeforeEmitNodeArray = function (nodes) {
                    if (nodes) {
                        setPos(nodes, lastNonTriviaPosition);
                    }
                };
                var onAfterEmitNodeArray = function (nodes) {
                    if (nodes) {
                        setEnd(nodes, lastNonTriviaPosition);
                    }
                };
                var onBeforeEmitToken = function (node) {
                    if (node) {
                        setPos(node, lastNonTriviaPosition);
                    }
                };
                var onAfterEmitToken = function (node) {
                    if (node) {
                        setEnd(node, lastNonTriviaPosition);
                    }
                };
                function setLastNonTriviaPosition(s, force) {
                    if (force || !isTrivia(s)) {
                        lastNonTriviaPosition = writer.getTextPos();
                        var i = 0;
                        while (ts.isWhiteSpaceLike(s.charCodeAt(s.length - i - 1))) {
                            i++;
                        }
                        // trim trailing whitespaces
                        lastNonTriviaPosition -= i;
                    }
                }
                function write(s) {
                    writer.write(s);
                    setLastNonTriviaPosition(s, /*force*/ false);
                }
                function writeComment(s) {
                    writer.writeComment(s);
                }
                function writeKeyword(s) {
                    writer.writeKeyword(s);
                    setLastNonTriviaPosition(s, /*force*/ false);
                }
                function writeOperator(s) {
                    writer.writeOperator(s);
                    setLastNonTriviaPosition(s, /*force*/ false);
                }
                function writePunctuation(s) {
                    writer.writePunctuation(s);
                    setLastNonTriviaPosition(s, /*force*/ false);
                }
                function writeTrailingSemicolon(s) {
                    writer.writeTrailingSemicolon(s);
                    setLastNonTriviaPosition(s, /*force*/ false);
                }
                function writeParameter(s) {
                    writer.writeParameter(s);
                    setLastNonTriviaPosition(s, /*force*/ false);
                }
                function writeProperty(s) {
                    writer.writeProperty(s);
                    setLastNonTriviaPosition(s, /*force*/ false);
                }
                function writeSpace(s) {
                    writer.writeSpace(s);
                    setLastNonTriviaPosition(s, /*force*/ false);
                }
                function writeStringLiteral(s) {
                    writer.writeStringLiteral(s);
                    setLastNonTriviaPosition(s, /*force*/ false);
                }
                function writeSymbol(s, sym) {
                    writer.writeSymbol(s, sym);
                    setLastNonTriviaPosition(s, /*force*/ false);
                }
                function writeLine() {
                    writer.writeLine();
                }
                function increaseIndent() {
                    writer.increaseIndent();
                }
                function decreaseIndent() {
                    writer.decreaseIndent();
                }
                function getText() {
                    return writer.getText();
                }
                function rawWrite(s) {
                    writer.rawWrite(s);
                    setLastNonTriviaPosition(s, /*force*/ false);
                }
                function writeLiteral(s) {
                    writer.writeLiteral(s);
                    setLastNonTriviaPosition(s, /*force*/ true);
                }
                function getTextPos() {
                    return writer.getTextPos();
                }
                function getLine() {
                    return writer.getLine();
                }
                function getColumn() {
                    return writer.getColumn();
                }
                function getIndent() {
                    return writer.getIndent();
                }
                function isAtStartOfLine() {
                    return writer.isAtStartOfLine();
                }
                function clear() {
                    writer.clear();
                    lastNonTriviaPosition = 0;
                }
                return {
                    onEmitNode: onEmitNode,
                    onBeforeEmitNodeArray: onBeforeEmitNodeArray,
                    onAfterEmitNodeArray: onAfterEmitNodeArray,
                    onBeforeEmitToken: onBeforeEmitToken,
                    onAfterEmitToken: onAfterEmitToken,
                    write: write,
                    writeComment: writeComment,
                    writeKeyword: writeKeyword,
                    writeOperator: writeOperator,
                    writePunctuation: writePunctuation,
                    writeTrailingSemicolon: writeTrailingSemicolon,
                    writeParameter: writeParameter,
                    writeProperty: writeProperty,
                    writeSpace: writeSpace,
                    writeStringLiteral: writeStringLiteral,
                    writeSymbol: writeSymbol,
                    writeLine: writeLine,
                    increaseIndent: increaseIndent,
                    decreaseIndent: decreaseIndent,
                    getText: getText,
                    rawWrite: rawWrite,
                    writeLiteral: writeLiteral,
                    getTextPos: getTextPos,
                    getLine: getLine,
                    getColumn: getColumn,
                    getIndent: getIndent,
                    isAtStartOfLine: isAtStartOfLine,
                    hasTrailingComment: function () { return writer.hasTrailingComment(); },
                    hasTrailingWhitespace: function () { return writer.hasTrailingWhitespace(); },
                    clear: clear
                };
            }
            function getInsertionPositionAtSourceFileTop(sourceFile) {
                var lastPrologue;
                for (var _i = 0, _a = sourceFile.statements; _i < _a.length; _i++) {
                    var node = _a[_i];
                    if (ts.isPrologueDirective(node)) {
                        lastPrologue = node;
                    }
                    else {
                        break;
                    }
                }
                var position = 0;
                var text = sourceFile.text;
                if (lastPrologue) {
                    position = lastPrologue.end;
                    advancePastLineBreak();
                    return position;
                }
                var shebang = ts.getShebang(text);
                if (shebang !== undefined) {
                    position = shebang.length;
                    advancePastLineBreak();
                }
                var ranges = ts.getLeadingCommentRanges(text, position);
                if (!ranges)
                    return position;
                // Find the first attached comment to the first node and add before it
                var lastComment;
                var firstNodeLine;
                for (var _b = 0, ranges_1 = ranges; _b < ranges_1.length; _b++) {
                    var range = ranges_1[_b];
                    if (range.kind === 3 /* MultiLineCommentTrivia */) {
                        if (ts.isPinnedComment(text, range.pos)) {
                            lastComment = { range: range, pinnedOrTripleSlash: true };
                            continue;
                        }
                    }
                    else if (ts.isRecognizedTripleSlashComment(text, range.pos, range.end)) {
                        lastComment = { range: range, pinnedOrTripleSlash: true };
                        continue;
                    }
                    if (lastComment) {
                        // Always insert after pinned or triple slash comments
                        if (lastComment.pinnedOrTripleSlash)
                            break;
                        // There was a blank line between the last comment and this comment.
                        // This comment is not part of the copyright comments
                        var commentLine = sourceFile.getLineAndCharacterOfPosition(range.pos).line;
                        var lastCommentEndLine = sourceFile.getLineAndCharacterOfPosition(lastComment.range.end).line;
                        if (commentLine >= lastCommentEndLine + 2)
                            break;
                    }
                    if (sourceFile.statements.length) {
                        if (firstNodeLine === undefined)
                            firstNodeLine = sourceFile.getLineAndCharacterOfPosition(sourceFile.statements[0].getStart()).line;
                        var commentEndLine = sourceFile.getLineAndCharacterOfPosition(range.end).line;
                        if (firstNodeLine < commentEndLine + 2)
                            break;
                    }
                    lastComment = { range: range, pinnedOrTripleSlash: false };
                }
                if (lastComment) {
                    position = lastComment.range.end;
                    advancePastLineBreak();
                }
                return position;
                function advancePastLineBreak() {
                    if (position < text.length) {
                        var charCode = text.charCodeAt(position);
                        if (ts.isLineBreak(charCode)) {
                            position++;
                            if (position < text.length && charCode === 13 /* carriageReturn */ && text.charCodeAt(position) === 10 /* lineFeed */) {
                                position++;
                            }
                        }
                    }
                }
            }
            function isValidLocationToAddComment(sourceFile, position) {
                return !ts.isInComment(sourceFile, position) && !ts.isInString(sourceFile, position) && !ts.isInTemplateString(sourceFile, position) && !ts.isInJSXText(sourceFile, position);
            }
            textChanges_3.isValidLocationToAddComment = isValidLocationToAddComment;
            function needSemicolonBetween(a, b) {
                return (ts.isPropertySignature(a) || ts.isPropertyDeclaration(a)) && ts.isClassOrTypeElement(b) && b.name.kind === 154 /* ComputedPropertyName */
                    || ts.isStatementButNotDeclaration(a) && ts.isStatementButNotDeclaration(b); // TODO: only if b would start with a `(` or `[`
            }
            var deleteDeclaration;
            (function (deleteDeclaration_1) {
                function deleteDeclaration(changes, deletedNodesInLists, sourceFile, node) {
                    switch (node.kind) {
                        case 156 /* Parameter */: {
                            var oldFunction = node.parent;
                            if (ts.isArrowFunction(oldFunction) &&
                                oldFunction.parameters.length === 1 &&
                                !ts.findChildOfKind(oldFunction, 20 /* OpenParenToken */, sourceFile)) {
                                // Lambdas with exactly one parameter are special because, after removal, there
                                // must be an empty parameter list (i.e. `()`) and this won't necessarily be the
                                // case if the parameter is simply removed (e.g. in `x => 1`).
                                changes.replaceNodeWithText(sourceFile, node, "()");
                            }
                            else {
                                deleteNodeInList(changes, deletedNodesInLists, sourceFile, node);
                            }
                            break;
                        }
                        case 254 /* ImportDeclaration */:
                            var isFirstImport = sourceFile.imports.length && node === ts.first(sourceFile.imports).parent || node === ts.find(sourceFile.statements, ts.isImportDeclaration);
                            deleteNode(changes, sourceFile, node, 
                            // For first import, leave header comment in place
                            isFirstImport ? { leadingTriviaOption: LeadingTriviaOption.Exclude } : undefined);
                            break;
                        case 191 /* BindingElement */:
                            var pattern = node.parent;
                            var preserveComma = pattern.kind === 190 /* ArrayBindingPattern */ && node !== ts.last(pattern.elements);
                            if (preserveComma) {
                                deleteNode(changes, sourceFile, node);
                            }
                            else {
                                deleteNodeInList(changes, deletedNodesInLists, sourceFile, node);
                            }
                            break;
                        case 242 /* VariableDeclaration */:
                            deleteVariableDeclaration(changes, deletedNodesInLists, sourceFile, node);
                            break;
                        case 155 /* TypeParameter */:
                            deleteNodeInList(changes, deletedNodesInLists, sourceFile, node);
                            break;
                        case 258 /* ImportSpecifier */:
                            var namedImports = node.parent;
                            if (namedImports.elements.length === 1) {
                                deleteImportBinding(changes, sourceFile, namedImports);
                            }
                            else {
                                deleteNodeInList(changes, deletedNodesInLists, sourceFile, node);
                            }
                            break;
                        case 256 /* NamespaceImport */:
                            deleteImportBinding(changes, sourceFile, node);
                            break;
                        default:
                            if (ts.isImportClause(node.parent) && node.parent.name === node) {
                                deleteDefaultImport(changes, sourceFile, node.parent);
                            }
                            else if (ts.isCallLikeExpression(node.parent)) {
                                deleteNodeInList(changes, deletedNodesInLists, sourceFile, node);
                            }
                            else {
                                deleteNode(changes, sourceFile, node, node.kind === 26 /* SemicolonToken */ ? { trailingTriviaOption: TrailingTriviaOption.Exclude } : undefined);
                            }
                    }
                }
                deleteDeclaration_1.deleteDeclaration = deleteDeclaration;
                function deleteDefaultImport(changes, sourceFile, importClause) {
                    if (!importClause.namedBindings) {
                        // Delete the whole import
                        deleteNode(changes, sourceFile, importClause.parent);
                    }
                    else {
                        // import |d,| * as ns from './file'
                        var start = importClause.name.getStart(sourceFile);
                        var nextToken = ts.getTokenAtPosition(sourceFile, importClause.name.end);
                        if (nextToken && nextToken.kind === 27 /* CommaToken */) {
                            // shift first non-whitespace position after comma to the start position of the node
                            var end = ts.skipTrivia(sourceFile.text, nextToken.end, /*stopAfterLineBreaks*/ false, /*stopAtComments*/ true);
                            changes.deleteRange(sourceFile, { pos: start, end: end });
                        }
                        else {
                            deleteNode(changes, sourceFile, importClause.name);
                        }
                    }
                }
                function deleteImportBinding(changes, sourceFile, node) {
                    if (node.parent.name) {
                        // Delete named imports while preserving the default import
                        // import d|, * as ns| from './file'
                        // import d|, { a }| from './file'
                        var previousToken = ts.Debug.assertDefined(ts.getTokenAtPosition(sourceFile, node.pos - 1));
                        changes.deleteRange(sourceFile, { pos: previousToken.getStart(sourceFile), end: node.end });
                    }
                    else {
                        // Delete the entire import declaration
                        // |import * as ns from './file'|
                        // |import { a } from './file'|
                        var importDecl = ts.getAncestor(node, 254 /* ImportDeclaration */);
                        deleteNode(changes, sourceFile, importDecl);
                    }
                }
                function deleteVariableDeclaration(changes, deletedNodesInLists, sourceFile, node) {
                    var parent = node.parent;
                    if (parent.kind === 280 /* CatchClause */) {
                        // TODO: There's currently no unused diagnostic for this, could be a suggestion
                        changes.deleteNodeRange(sourceFile, ts.findChildOfKind(parent, 20 /* OpenParenToken */, sourceFile), ts.findChildOfKind(parent, 21 /* CloseParenToken */, sourceFile));
                        return;
                    }
                    if (parent.declarations.length !== 1) {
                        deleteNodeInList(changes, deletedNodesInLists, sourceFile, node);
                        return;
                    }
                    var gp = parent.parent;
                    switch (gp.kind) {
                        case 232 /* ForOfStatement */:
                        case 231 /* ForInStatement */:
                            changes.replaceNode(sourceFile, node, ts.createObjectLiteral());
                            break;
                        case 230 /* ForStatement */:
                            deleteNode(changes, sourceFile, parent);
                            break;
                        case 225 /* VariableStatement */:
                            deleteNode(changes, sourceFile, gp);
                            break;
                        default:
                            ts.Debug.assertNever(gp);
                    }
                }
            })(deleteDeclaration || (deleteDeclaration = {}));
            /** Warning: This deletes comments too. See `copyComments` in `convertFunctionToEs6Class`. */
            // Exported for tests only! (TODO: improve tests to not need this)
            function deleteNode(changes, sourceFile, node, options) {
                if (options === void 0) {
                    options = { leadingTriviaOption: LeadingTriviaOption.IncludeAll };
                }
                var startPosition = getAdjustedStartPosition(sourceFile, node, options);
                var endPosition = getAdjustedEndPosition(sourceFile, node, options);
                changes.deleteRange(sourceFile, { pos: startPosition, end: endPosition });
            }
            textChanges_3.deleteNode = deleteNode;
            function deleteNodeInList(changes, deletedNodesInLists, sourceFile, node) {
                var containingList = ts.Debug.assertDefined(ts.formatting.SmartIndenter.getContainingList(node, sourceFile));
                var index = ts.indexOfNode(containingList, node);
                ts.Debug.assert(index !== -1);
                if (containingList.length === 1) {
                    deleteNode(changes, sourceFile, node);
                    return;
                }
                // Note: We will only delete a comma *after* a node. This will leave a trailing comma if we delete the last node.
                // That's handled in the end by `finishTrailingCommaAfterDeletingNodesInList`.
                ts.Debug.assert(!deletedNodesInLists.has(node), "Deleting a node twice");
                deletedNodesInLists.add(node);
                changes.deleteRange(sourceFile, {
                    pos: startPositionToDeleteNodeInList(sourceFile, node),
                    end: index === containingList.length - 1 ? getAdjustedEndPosition(sourceFile, node, {}) : startPositionToDeleteNodeInList(sourceFile, containingList[index + 1]),
                });
            }
        })(textChanges = ts.textChanges || (ts.textChanges = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var errorCodeToFixes = ts.createMultiMap();
            var fixIdToRegistration = ts.createMap();
            function diagnosticToString(diag) {
                return ts.isArray(diag)
                    ? ts.formatStringFromArgs(ts.getLocaleSpecificMessage(diag[0]), diag.slice(1))
                    : ts.getLocaleSpecificMessage(diag);
            }
            function createCodeFixActionWithoutFixAll(fixName, changes, description) {
                return createCodeFixActionWorker(fixName, diagnosticToString(description), changes, /*fixId*/ undefined, /*fixAllDescription*/ undefined);
            }
            codefix.createCodeFixActionWithoutFixAll = createCodeFixActionWithoutFixAll;
            function createCodeFixAction(fixName, changes, description, fixId, fixAllDescription, command) {
                return createCodeFixActionWorker(fixName, diagnosticToString(description), changes, fixId, diagnosticToString(fixAllDescription), command);
            }
            codefix.createCodeFixAction = createCodeFixAction;
            function createCodeFixActionWorker(fixName, description, changes, fixId, fixAllDescription, command) {
                return { fixName: fixName, description: description, changes: changes, fixId: fixId, fixAllDescription: fixAllDescription, commands: command ? [command] : undefined };
            }
            function registerCodeFix(reg) {
                for (var _i = 0, _a = reg.errorCodes; _i < _a.length; _i++) {
                    var error = _a[_i];
                    errorCodeToFixes.add(String(error), reg);
                }
                if (reg.fixIds) {
                    for (var _b = 0, _c = reg.fixIds; _b < _c.length; _b++) {
                        var fixId = _c[_b];
                        ts.Debug.assert(!fixIdToRegistration.has(fixId));
                        fixIdToRegistration.set(fixId, reg);
                    }
                }
            }
            codefix.registerCodeFix = registerCodeFix;
            function getSupportedErrorCodes() {
                return ts.arrayFrom(errorCodeToFixes.keys());
            }
            codefix.getSupportedErrorCodes = getSupportedErrorCodes;
            function removeFixIdIfFixAllUnavailable(registration, diagnostics) {
                var errorCodes = registration.errorCodes;
                var maybeFixableDiagnostics = 0;
                for (var _i = 0, diagnostics_1 = diagnostics; _i < diagnostics_1.length; _i++) {
                    var diag = diagnostics_1[_i];
                    if (ts.contains(errorCodes, diag.code))
                        maybeFixableDiagnostics++;
                    if (maybeFixableDiagnostics > 1)
                        break;
                }
                var fixAllUnavailable = maybeFixableDiagnostics < 2;
                return function (_a) {
                    var fixId = _a.fixId, fixAllDescription = _a.fixAllDescription, action = __rest(_a, ["fixId", "fixAllDescription"]);
                    return fixAllUnavailable ? action : __assign(__assign({}, action), { fixId: fixId, fixAllDescription: fixAllDescription });
                };
            }
            function getFixes(context) {
                var diagnostics = getDiagnostics(context);
                var registrations = errorCodeToFixes.get(String(context.errorCode));
                return ts.flatMap(registrations, function (f) { return ts.map(f.getCodeActions(context), removeFixIdIfFixAllUnavailable(f, diagnostics)); });
            }
            codefix.getFixes = getFixes;
            function getAllFixes(context) {
                // Currently fixId is always a string.
                return fixIdToRegistration.get(ts.cast(context.fixId, ts.isString)).getAllCodeActions(context);
            }
            codefix.getAllFixes = getAllFixes;
            function createCombinedCodeActions(changes, commands) {
                return { changes: changes, commands: commands };
            }
            codefix.createCombinedCodeActions = createCombinedCodeActions;
            function createFileTextChanges(fileName, textChanges) {
                return { fileName: fileName, textChanges: textChanges };
            }
            codefix.createFileTextChanges = createFileTextChanges;
            function codeFixAll(context, errorCodes, use) {
                var commands = [];
                var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return eachDiagnostic(context, errorCodes, function (diag) { return use(t, diag, commands); }); });
                return createCombinedCodeActions(changes, commands.length === 0 ? undefined : commands);
            }
            codefix.codeFixAll = codeFixAll;
            function eachDiagnostic(context, errorCodes, cb) {
                for (var _i = 0, _a = getDiagnostics(context); _i < _a.length; _i++) {
                    var diag = _a[_i];
                    if (ts.contains(errorCodes, diag.code)) {
                        cb(diag);
                    }
                }
            }
            codefix.eachDiagnostic = eachDiagnostic;
            function getDiagnostics(_a) {
                var program = _a.program, sourceFile = _a.sourceFile, cancellationToken = _a.cancellationToken;
                return program.getSemanticDiagnostics(sourceFile, cancellationToken).concat(ts.computeSuggestionDiagnostics(sourceFile, program, cancellationToken));
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var refactor;
        (function (refactor_1) {
            // A map with the refactor code as key, the refactor itself as value
            // e.g.  nonSuggestableRefactors[refactorCode] -> the refactor you want
            var refactors = ts.createMap();
            /** @param name An unique code associated with each refactor. Does not have to be human-readable. */
            function registerRefactor(name, refactor) {
                refactors.set(name, refactor);
            }
            refactor_1.registerRefactor = registerRefactor;
            function getApplicableRefactors(context) {
                return ts.arrayFrom(ts.flatMapIterator(refactors.values(), function (refactor) {
                    return context.cancellationToken && context.cancellationToken.isCancellationRequested() ? undefined : refactor.getAvailableActions(context);
                }));
            }
            refactor_1.getApplicableRefactors = getApplicableRefactors;
            function getEditsForRefactor(context, refactorName, actionName) {
                var refactor = refactors.get(refactorName);
                return refactor && refactor.getEditsForAction(context, actionName);
            }
            refactor_1.getEditsForRefactor = getEditsForRefactor;
        })(refactor = ts.refactor || (ts.refactor = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "addConvertToUnknownForNonOverlappingTypes";
            var errorCodes = [ts.Diagnostics.Conversion_of_type_0_to_type_1_may_be_a_mistake_because_neither_type_sufficiently_overlaps_with_the_other_If_this_was_intentional_convert_the_expression_to_unknown_first.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return makeChange(t, context.sourceFile, context.span.start); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Add_unknown_conversion_for_non_overlapping_types, fixId, ts.Diagnostics.Add_unknown_to_all_conversions_of_non_overlapping_types)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) { return codefix.codeFixAll(context, errorCodes, function (changes, diag) { return makeChange(changes, diag.file, diag.start); }); },
            });
            function makeChange(changeTracker, sourceFile, pos) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                var assertion = ts.Debug.assertDefined(ts.findAncestor(token, function (n) { return ts.isAsExpression(n) || ts.isTypeAssertion(n); }), "Expected to find an assertion expression");
                var replacement = ts.isAsExpression(assertion)
                    ? ts.createAsExpression(assertion.expression, ts.createKeywordTypeNode(148 /* UnknownKeyword */))
                    : ts.createTypeAssertion(ts.createKeywordTypeNode(148 /* UnknownKeyword */), assertion.expression);
                changeTracker.replaceNode(sourceFile, assertion.expression, replacement);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            codefix.registerCodeFix({
                errorCodes: [ts.Diagnostics.await_expressions_are_only_allowed_at_the_top_level_of_a_file_when_that_file_is_a_module_but_this_file_has_no_imports_or_exports_Consider_adding_an_empty_export_to_make_this_file_a_module.code],
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (changes) {
                        var exportDeclaration = ts.createExportDeclaration(
                        /*decorators*/ undefined, 
                        /*modifiers*/ undefined, ts.createNamedExports([]), 
                        /*moduleSpecifier*/ undefined, 
                        /*isTypeOnly*/ false);
                        changes.insertNodeAtEndOfScope(sourceFile, sourceFile, exportDeclaration);
                    });
                    return [codefix.createCodeFixActionWithoutFixAll("addEmptyExportDeclaration", changes, ts.Diagnostics.Add_export_to_make_this_file_into_a_module)];
                },
            });
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "addMissingAwait";
            var propertyAccessCode = ts.Diagnostics.Property_0_does_not_exist_on_type_1.code;
            var callableConstructableErrorCodes = [
                ts.Diagnostics.This_expression_is_not_callable.code,
                ts.Diagnostics.This_expression_is_not_constructable.code,
            ];
            var errorCodes = __spreadArrays([
                ts.Diagnostics.An_arithmetic_operand_must_be_of_type_any_number_bigint_or_an_enum_type.code,
                ts.Diagnostics.The_left_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type.code,
                ts.Diagnostics.The_right_hand_side_of_an_arithmetic_operation_must_be_of_type_any_number_bigint_or_an_enum_type.code,
                ts.Diagnostics.Operator_0_cannot_be_applied_to_type_1.code,
                ts.Diagnostics.Operator_0_cannot_be_applied_to_types_1_and_2.code,
                ts.Diagnostics.This_condition_will_always_return_0_since_the_types_1_and_2_have_no_overlap.code,
                ts.Diagnostics.Type_0_is_not_an_array_type.code,
                ts.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type.code,
                ts.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type_Use_compiler_option_downlevelIteration_to_allow_iterating_of_iterators.code,
                ts.Diagnostics.Type_0_is_not_an_array_type_or_a_string_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator.code,
                ts.Diagnostics.Type_0_is_not_an_array_type_or_does_not_have_a_Symbol_iterator_method_that_returns_an_iterator.code,
                ts.Diagnostics.Type_0_must_have_a_Symbol_iterator_method_that_returns_an_iterator.code,
                ts.Diagnostics.Type_0_must_have_a_Symbol_asyncIterator_method_that_returns_an_async_iterator.code,
                ts.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1.code,
                propertyAccessCode
            ], callableConstructableErrorCodes);
            codefix.registerCodeFix({
                fixIds: [fixId],
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, errorCode = context.errorCode, span = context.span, cancellationToken = context.cancellationToken, program = context.program;
                    var expression = getFixableErrorSpanExpression(sourceFile, errorCode, span, cancellationToken, program);
                    if (!expression) {
                        return;
                    }
                    var checker = context.program.getTypeChecker();
                    var trackChanges = function (cb) { return ts.textChanges.ChangeTracker.with(context, cb); };
                    return ts.compact([
                        getDeclarationSiteFix(context, expression, errorCode, checker, trackChanges),
                        getUseSiteFix(context, expression, errorCode, checker, trackChanges)
                    ]);
                },
                getAllCodeActions: function (context) {
                    var sourceFile = context.sourceFile, program = context.program, cancellationToken = context.cancellationToken;
                    var checker = context.program.getTypeChecker();
                    var fixedDeclarations = ts.createMap();
                    return codefix.codeFixAll(context, errorCodes, function (t, diagnostic) {
                        var expression = getFixableErrorSpanExpression(sourceFile, diagnostic.code, diagnostic, cancellationToken, program);
                        if (!expression) {
                            return;
                        }
                        var trackChanges = function (cb) { return (cb(t), []); };
                        return getDeclarationSiteFix(context, expression, diagnostic.code, checker, trackChanges, fixedDeclarations)
                            || getUseSiteFix(context, expression, diagnostic.code, checker, trackChanges, fixedDeclarations);
                    });
                },
            });
            function getDeclarationSiteFix(context, expression, errorCode, checker, trackChanges, fixedDeclarations) {
                var sourceFile = context.sourceFile, program = context.program, cancellationToken = context.cancellationToken;
                var awaitableInitializers = findAwaitableInitializers(expression, sourceFile, cancellationToken, program, checker);
                if (awaitableInitializers) {
                    var initializerChanges = trackChanges(function (t) {
                        ts.forEach(awaitableInitializers.initializers, function (_a) {
                            var expression = _a.expression;
                            return makeChange(t, errorCode, sourceFile, checker, expression, fixedDeclarations);
                        });
                        if (fixedDeclarations && awaitableInitializers.needsSecondPassForFixAll) {
                            makeChange(t, errorCode, sourceFile, checker, expression, fixedDeclarations);
                        }
                    });
                    // No fix-all because it will already be included once with the use site fix,
                    // and for simplicity the fix-all doesn‘t let the user choose between use-site and declaration-site fixes.
                    return codefix.createCodeFixActionWithoutFixAll("addMissingAwaitToInitializer", initializerChanges, awaitableInitializers.initializers.length === 1
                        ? [ts.Diagnostics.Add_await_to_initializer_for_0, awaitableInitializers.initializers[0].declarationSymbol.name]
                        : ts.Diagnostics.Add_await_to_initializers);
                }
            }
            function getUseSiteFix(context, expression, errorCode, checker, trackChanges, fixedDeclarations) {
                var changes = trackChanges(function (t) { return makeChange(t, errorCode, context.sourceFile, checker, expression, fixedDeclarations); });
                return codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Add_await, fixId, ts.Diagnostics.Fix_all_expressions_possibly_missing_await);
            }
            function isMissingAwaitError(sourceFile, errorCode, span, cancellationToken, program) {
                var checker = program.getDiagnosticsProducingTypeChecker();
                var diagnostics = checker.getDiagnostics(sourceFile, cancellationToken);
                return ts.some(diagnostics, function (_a) {
                    var start = _a.start, length = _a.length, relatedInformation = _a.relatedInformation, code = _a.code;
                    return ts.isNumber(start) && ts.isNumber(length) && ts.textSpansEqual({ start: start, length: length }, span) &&
                        code === errorCode &&
                        !!relatedInformation &&
                        ts.some(relatedInformation, function (related) { return related.code === ts.Diagnostics.Did_you_forget_to_use_await.code; });
                });
            }
            function getFixableErrorSpanExpression(sourceFile, errorCode, span, cancellationToken, program) {
                var token = ts.getTokenAtPosition(sourceFile, span.start);
                // Checker has already done work to determine that await might be possible, and has attached
                // related info to the node, so start by finding the expression that exactly matches up
                // with the diagnostic range.
                var expression = ts.findAncestor(token, function (node) {
                    if (node.getStart(sourceFile) < span.start || node.getEnd() > ts.textSpanEnd(span)) {
                        return "quit";
                    }
                    return ts.isExpression(node) && ts.textSpansEqual(span, ts.createTextSpanFromNode(node, sourceFile));
                });
                return expression
                    && isMissingAwaitError(sourceFile, errorCode, span, cancellationToken, program)
                    && isInsideAwaitableBody(expression) ? expression : undefined;
            }
            function findAwaitableInitializers(expression, sourceFile, cancellationToken, program, checker) {
                var identifiers = getIdentifiersFromErrorSpanExpression(expression, checker);
                if (!identifiers) {
                    return;
                }
                var isCompleteFix = identifiers.isCompleteFix;
                var initializers;
                var _loop_11 = function (identifier) {
                    var symbol = checker.getSymbolAtLocation(identifier);
                    if (!symbol) {
                        return "continue";
                    }
                    var declaration = ts.tryCast(symbol.valueDeclaration, ts.isVariableDeclaration);
                    var variableName = declaration && ts.tryCast(declaration.name, ts.isIdentifier);
                    var variableStatement = ts.getAncestor(declaration, 225 /* VariableStatement */);
                    if (!declaration || !variableStatement ||
                        declaration.type ||
                        !declaration.initializer ||
                        variableStatement.getSourceFile() !== sourceFile ||
                        ts.hasModifier(variableStatement, 1 /* Export */) ||
                        !variableName ||
                        !isInsideAwaitableBody(declaration.initializer)) {
                        isCompleteFix = false;
                        return "continue";
                    }
                    var diagnostics = program.getSemanticDiagnostics(sourceFile, cancellationToken);
                    var isUsedElsewhere = ts.FindAllReferences.Core.eachSymbolReferenceInFile(variableName, checker, sourceFile, function (reference) {
                        return identifier !== reference && !symbolReferenceIsAlsoMissingAwait(reference, diagnostics, sourceFile, checker);
                    });
                    if (isUsedElsewhere) {
                        isCompleteFix = false;
                        return "continue";
                    }
                    (initializers || (initializers = [])).push({
                        expression: declaration.initializer,
                        declarationSymbol: symbol,
                    });
                };
                for (var _i = 0, _a = identifiers.identifiers; _i < _a.length; _i++) {
                    var identifier = _a[_i];
                    _loop_11(identifier);
                }
                return initializers && {
                    initializers: initializers,
                    needsSecondPassForFixAll: !isCompleteFix,
                };
            }
            function getIdentifiersFromErrorSpanExpression(expression, checker) {
                if (ts.isPropertyAccessExpression(expression.parent) && ts.isIdentifier(expression.parent.expression)) {
                    return { identifiers: [expression.parent.expression], isCompleteFix: true };
                }
                if (ts.isIdentifier(expression)) {
                    return { identifiers: [expression], isCompleteFix: true };
                }
                if (ts.isBinaryExpression(expression)) {
                    var sides = void 0;
                    var isCompleteFix = true;
                    for (var _i = 0, _a = [expression.left, expression.right]; _i < _a.length; _i++) {
                        var side = _a[_i];
                        var type = checker.getTypeAtLocation(side);
                        if (checker.getPromisedTypeOfPromise(type)) {
                            if (!ts.isIdentifier(side)) {
                                isCompleteFix = false;
                                continue;
                            }
                            (sides || (sides = [])).push(side);
                        }
                    }
                    return sides && { identifiers: sides, isCompleteFix: isCompleteFix };
                }
            }
            function symbolReferenceIsAlsoMissingAwait(reference, diagnostics, sourceFile, checker) {
                var errorNode = ts.isPropertyAccessExpression(reference.parent) ? reference.parent.name :
                    ts.isBinaryExpression(reference.parent) ? reference.parent :
                        reference;
                var diagnostic = ts.find(diagnostics, function (diagnostic) {
                    return diagnostic.start === errorNode.getStart(sourceFile) &&
                        diagnostic.start + diagnostic.length === errorNode.getEnd();
                });
                return diagnostic && ts.contains(errorCodes, diagnostic.code) ||
                    // A Promise is usually not correct in a binary expression (it’s not valid
                    // in an arithmetic expression and an equality comparison seems unusual),
                    // but if the other side of the binary expression has an error, the side
                    // is typed `any` which will squash the error that would identify this
                    // Promise as an invalid operand. So if the whole binary expression is
                    // typed `any` as a result, there is a strong likelihood that this Promise
                    // is accidentally missing `await`.
                    checker.getTypeAtLocation(errorNode).flags & 1 /* Any */;
            }
            function isInsideAwaitableBody(node) {
                return node.kind & 32768 /* AwaitContext */ || !!ts.findAncestor(node, function (ancestor) {
                    return ancestor.parent && ts.isArrowFunction(ancestor.parent) && ancestor.parent.body === ancestor ||
                        ts.isBlock(ancestor) && (ancestor.parent.kind === 244 /* FunctionDeclaration */ ||
                            ancestor.parent.kind === 201 /* FunctionExpression */ ||
                            ancestor.parent.kind === 202 /* ArrowFunction */ ||
                            ancestor.parent.kind === 161 /* MethodDeclaration */);
                });
            }
            function makeChange(changeTracker, errorCode, sourceFile, checker, insertionSite, fixedDeclarations) {
                if (ts.isBinaryExpression(insertionSite)) {
                    for (var _i = 0, _a = [insertionSite.left, insertionSite.right]; _i < _a.length; _i++) {
                        var side = _a[_i];
                        if (fixedDeclarations && ts.isIdentifier(side)) {
                            var symbol = checker.getSymbolAtLocation(side);
                            if (symbol && fixedDeclarations.has(ts.getSymbolId(symbol).toString())) {
                                continue;
                            }
                        }
                        var type = checker.getTypeAtLocation(side);
                        var newNode = checker.getPromisedTypeOfPromise(type) ? ts.createAwait(side) : side;
                        changeTracker.replaceNode(sourceFile, side, newNode);
                    }
                }
                else if (errorCode === propertyAccessCode && ts.isPropertyAccessExpression(insertionSite.parent)) {
                    if (fixedDeclarations && ts.isIdentifier(insertionSite.parent.expression)) {
                        var symbol = checker.getSymbolAtLocation(insertionSite.parent.expression);
                        if (symbol && fixedDeclarations.has(ts.getSymbolId(symbol).toString())) {
                            return;
                        }
                    }
                    changeTracker.replaceNode(sourceFile, insertionSite.parent.expression, ts.createParen(ts.createAwait(insertionSite.parent.expression)));
                    insertLeadingSemicolonIfNeeded(changeTracker, insertionSite.parent.expression, sourceFile);
                }
                else if (ts.contains(callableConstructableErrorCodes, errorCode) && ts.isCallOrNewExpression(insertionSite.parent)) {
                    if (fixedDeclarations && ts.isIdentifier(insertionSite)) {
                        var symbol = checker.getSymbolAtLocation(insertionSite);
                        if (symbol && fixedDeclarations.has(ts.getSymbolId(symbol).toString())) {
                            return;
                        }
                    }
                    changeTracker.replaceNode(sourceFile, insertionSite, ts.createParen(ts.createAwait(insertionSite)));
                    insertLeadingSemicolonIfNeeded(changeTracker, insertionSite, sourceFile);
                }
                else {
                    if (fixedDeclarations && ts.isVariableDeclaration(insertionSite.parent) && ts.isIdentifier(insertionSite.parent.name)) {
                        var symbol = checker.getSymbolAtLocation(insertionSite.parent.name);
                        if (symbol && !ts.addToSeen(fixedDeclarations, ts.getSymbolId(symbol))) {
                            return;
                        }
                    }
                    changeTracker.replaceNode(sourceFile, insertionSite, ts.createAwait(insertionSite));
                }
            }
            function insertLeadingSemicolonIfNeeded(changeTracker, beforeNode, sourceFile) {
                var precedingToken = ts.findPrecedingToken(beforeNode.pos, sourceFile);
                if (precedingToken && ts.positionIsASICandidate(precedingToken.end, precedingToken.parent, sourceFile)) {
                    changeTracker.insertText(sourceFile, beforeNode.getStart(sourceFile), ";");
                }
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "addMissingConst";
            var errorCodes = [
                ts.Diagnostics.Cannot_find_name_0.code,
                ts.Diagnostics.No_value_exists_in_scope_for_the_shorthand_property_0_Either_declare_one_or_provide_an_initializer.code
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return makeChange(t, context.sourceFile, context.span.start, context.program); });
                    if (changes.length > 0) {
                        return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Add_const_to_unresolved_variable, fixId, ts.Diagnostics.Add_const_to_all_unresolved_variables)];
                    }
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    var fixedNodes = new ts.NodeSet();
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) { return makeChange(changes, diag.file, diag.start, context.program, fixedNodes); });
                },
            });
            function makeChange(changeTracker, sourceFile, pos, program, fixedNodes) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                var forInitializer = ts.findAncestor(token, function (node) {
                    return ts.isForInOrOfStatement(node.parent) ? node.parent.initializer === node :
                        isPossiblyPartOfDestructuring(node) ? false : "quit";
                });
                if (forInitializer)
                    return applyChange(changeTracker, forInitializer, sourceFile, fixedNodes);
                var parent = token.parent;
                if (ts.isBinaryExpression(parent) && parent.operatorToken.kind === 62 /* EqualsToken */ && ts.isExpressionStatement(parent.parent)) {
                    return applyChange(changeTracker, token, sourceFile, fixedNodes);
                }
                if (ts.isArrayLiteralExpression(parent)) {
                    var checker_1 = program.getTypeChecker();
                    if (!ts.every(parent.elements, function (element) { return arrayElementCouldBeVariableDeclaration(element, checker_1); })) {
                        return;
                    }
                    return applyChange(changeTracker, parent, sourceFile, fixedNodes);
                }
                var commaExpression = ts.findAncestor(token, function (node) {
                    return ts.isExpressionStatement(node.parent) ? true :
                        isPossiblyPartOfCommaSeperatedInitializer(node) ? false : "quit";
                });
                if (commaExpression) {
                    var checker = program.getTypeChecker();
                    if (!expressionCouldBeVariableDeclaration(commaExpression, checker)) {
                        return;
                    }
                    return applyChange(changeTracker, commaExpression, sourceFile, fixedNodes);
                }
            }
            function applyChange(changeTracker, initializer, sourceFile, fixedNodes) {
                if (!fixedNodes || fixedNodes.tryAdd(initializer)) {
                    changeTracker.insertModifierBefore(sourceFile, 81 /* ConstKeyword */, initializer);
                }
            }
            function isPossiblyPartOfDestructuring(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                    case 192 /* ArrayLiteralExpression */:
                    case 193 /* ObjectLiteralExpression */:
                    case 281 /* PropertyAssignment */:
                    case 282 /* ShorthandPropertyAssignment */:
                        return true;
                    default:
                        return false;
                }
            }
            function arrayElementCouldBeVariableDeclaration(expression, checker) {
                var identifier = ts.isIdentifier(expression) ? expression :
                    ts.isAssignmentExpression(expression, /*excludeCompoundAssignment*/ true) && ts.isIdentifier(expression.left) ? expression.left :
                        undefined;
                return !!identifier && !checker.getSymbolAtLocation(identifier);
            }
            function isPossiblyPartOfCommaSeperatedInitializer(node) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                    case 209 /* BinaryExpression */:
                    case 27 /* CommaToken */:
                        return true;
                    default:
                        return false;
                }
            }
            function expressionCouldBeVariableDeclaration(expression, checker) {
                if (!ts.isBinaryExpression(expression)) {
                    return false;
                }
                if (expression.operatorToken.kind === 27 /* CommaToken */) {
                    return ts.every([expression.left, expression.right], function (expression) { return expressionCouldBeVariableDeclaration(expression, checker); });
                }
                return expression.operatorToken.kind === 62 /* EqualsToken */
                    && ts.isIdentifier(expression.left)
                    && !checker.getSymbolAtLocation(expression.left);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "addMissingDeclareProperty";
            var errorCodes = [
                ts.Diagnostics.Property_0_will_overwrite_the_base_property_in_1_If_this_is_intentional_add_an_initializer_Otherwise_add_a_declare_modifier_or_remove_the_redundant_declaration.code,
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return makeChange(t, context.sourceFile, context.span.start); });
                    if (changes.length > 0) {
                        return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Prefix_with_declare, fixId, ts.Diagnostics.Prefix_all_incorrect_property_declarations_with_declare)];
                    }
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    var fixedNodes = new ts.NodeSet();
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) { return makeChange(changes, diag.file, diag.start, fixedNodes); });
                },
            });
            function makeChange(changeTracker, sourceFile, pos, fixedNodes) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                if (!ts.isIdentifier(token)) {
                    return;
                }
                var declaration = token.parent;
                if (declaration.kind === 159 /* PropertyDeclaration */ &&
                    (!fixedNodes || fixedNodes.tryAdd(declaration))) {
                    changeTracker.insertModifierBefore(sourceFile, 130 /* DeclareKeyword */, declaration);
                }
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "addMissingInvocationForDecorator";
            var errorCodes = [ts.Diagnostics._0_accepts_too_few_arguments_to_be_used_as_a_decorator_here_Did_you_mean_to_call_it_first_and_write_0.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return makeChange(t, context.sourceFile, context.span.start); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Call_decorator_expression, fixId, ts.Diagnostics.Add_to_all_uncalled_decorators)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) { return codefix.codeFixAll(context, errorCodes, function (changes, diag) { return makeChange(changes, diag.file, diag.start); }); },
            });
            function makeChange(changeTracker, sourceFile, pos) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                var decorator = ts.findAncestor(token, ts.isDecorator);
                ts.Debug.assert(!!decorator, "Expected position to be owned by a decorator.");
                var replacement = ts.createCall(decorator.expression, /*typeArguments*/ undefined, /*argumentsArray*/ undefined);
                changeTracker.replaceNode(sourceFile, decorator.expression, replacement);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "addNameToNamelessParameter";
            var errorCodes = [ts.Diagnostics.Parameter_has_a_name_but_no_type_Did_you_mean_0_Colon_1.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return makeChange(t, context.sourceFile, context.span.start); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Add_parameter_name, fixId, ts.Diagnostics.Add_names_to_all_parameters_without_names)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) { return codefix.codeFixAll(context, errorCodes, function (changes, diag) { return makeChange(changes, diag.file, diag.start); }); },
            });
            function makeChange(changeTracker, sourceFile, pos) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                if (!ts.isIdentifier(token)) {
                    return ts.Debug.fail("add-name-to-nameless-parameter operates on identifiers, but got a " + ts.Debug.formatSyntaxKind(token.kind));
                }
                var param = token.parent;
                if (!ts.isParameter(param)) {
                    return ts.Debug.fail("Tried to add a parameter name to a non-parameter: " + ts.Debug.formatSyntaxKind(token.kind));
                }
                var i = param.parent.parameters.indexOf(param);
                ts.Debug.assert(!param.type, "Tried to add a parameter name to a parameter that already had one.");
                ts.Debug.assert(i > -1, "Parameter not found in parent parameter list.");
                var replacement = ts.createParameter(
                /*decorators*/ undefined, param.modifiers, param.dotDotDotToken, "arg" + i, param.questionToken, ts.createTypeReferenceNode(token, /*typeArguments*/ undefined), param.initializer);
                changeTracker.replaceNode(sourceFile, token, replacement);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "annotateWithTypeFromJSDoc";
            var errorCodes = [ts.Diagnostics.JSDoc_types_may_be_moved_to_TypeScript_types.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var decl = getDeclaration(context.sourceFile, context.span.start);
                    if (!decl)
                        return;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, context.sourceFile, decl); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Annotate_with_type_from_JSDoc, fixId, ts.Diagnostics.Annotate_everything_with_types_from_JSDoc)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var decl = getDeclaration(diag.file, diag.start);
                        if (decl)
                            doChange(changes, diag.file, decl);
                    });
                },
            });
            function getDeclaration(file, pos) {
                var name = ts.getTokenAtPosition(file, pos);
                // For an arrow function with no name, 'name' lands on the first parameter.
                return ts.tryCast(ts.isParameter(name.parent) ? name.parent.parent : name.parent, parameterShouldGetTypeFromJSDoc);
            }
            function parameterShouldGetTypeFromJSDoc(node) {
                return isDeclarationWithType(node) && hasUsableJSDoc(node);
            }
            codefix.parameterShouldGetTypeFromJSDoc = parameterShouldGetTypeFromJSDoc;
            function hasUsableJSDoc(decl) {
                return ts.isFunctionLikeDeclaration(decl)
                    ? decl.parameters.some(hasUsableJSDoc) || (!decl.type && !!ts.getJSDocReturnType(decl))
                    : !decl.type && !!ts.getJSDocType(decl);
            }
            function doChange(changes, sourceFile, decl) {
                if (ts.isFunctionLikeDeclaration(decl) && (ts.getJSDocReturnType(decl) || decl.parameters.some(function (p) { return !!ts.getJSDocType(p); }))) {
                    if (!decl.typeParameters) {
                        var typeParameters = ts.getJSDocTypeParameterDeclarations(decl);
                        if (typeParameters.length)
                            changes.insertTypeParameters(sourceFile, decl, typeParameters);
                    }
                    var needParens = ts.isArrowFunction(decl) && !ts.findChildOfKind(decl, 20 /* OpenParenToken */, sourceFile);
                    if (needParens)
                        changes.insertNodeBefore(sourceFile, ts.first(decl.parameters), ts.createToken(20 /* OpenParenToken */));
                    for (var _i = 0, _a = decl.parameters; _i < _a.length; _i++) {
                        var param = _a[_i];
                        if (!param.type) {
                            var paramType = ts.getJSDocType(param);
                            if (paramType)
                                changes.tryInsertTypeAnnotation(sourceFile, param, transformJSDocType(paramType));
                        }
                    }
                    if (needParens)
                        changes.insertNodeAfter(sourceFile, ts.last(decl.parameters), ts.createToken(21 /* CloseParenToken */));
                    if (!decl.type) {
                        var returnType = ts.getJSDocReturnType(decl);
                        if (returnType)
                            changes.tryInsertTypeAnnotation(sourceFile, decl, transformJSDocType(returnType));
                    }
                }
                else {
                    var jsdocType = ts.Debug.assertDefined(ts.getJSDocType(decl), "A JSDocType for this declaration should exist"); // If not defined, shouldn't have been an error to fix
                    ts.Debug.assert(!decl.type, "The JSDocType decl should have a type"); // If defined, shouldn't have been an error to fix.
                    changes.tryInsertTypeAnnotation(sourceFile, decl, transformJSDocType(jsdocType));
                }
            }
            function isDeclarationWithType(node) {
                return ts.isFunctionLikeDeclaration(node) ||
                    node.kind === 242 /* VariableDeclaration */ ||
                    node.kind === 158 /* PropertySignature */ ||
                    node.kind === 159 /* PropertyDeclaration */;
            }
            function transformJSDocType(node) {
                switch (node.kind) {
                    case 295 /* JSDocAllType */:
                    case 296 /* JSDocUnknownType */:
                        return ts.createTypeReferenceNode("any", ts.emptyArray);
                    case 299 /* JSDocOptionalType */:
                        return transformJSDocOptionalType(node);
                    case 298 /* JSDocNonNullableType */:
                        return transformJSDocType(node.type);
                    case 297 /* JSDocNullableType */:
                        return transformJSDocNullableType(node);
                    case 301 /* JSDocVariadicType */:
                        return transformJSDocVariadicType(node);
                    case 300 /* JSDocFunctionType */:
                        return transformJSDocFunctionType(node);
                    case 169 /* TypeReference */:
                        return transformJSDocTypeReference(node);
                    default:
                        var visited = ts.visitEachChild(node, transformJSDocType, /*context*/ undefined); // TODO: GH#18217
                        ts.setEmitFlags(visited, 1 /* SingleLine */);
                        return visited;
                }
            }
            function transformJSDocOptionalType(node) {
                return ts.createUnionTypeNode([ts.visitNode(node.type, transformJSDocType), ts.createTypeReferenceNode("undefined", ts.emptyArray)]);
            }
            function transformJSDocNullableType(node) {
                return ts.createUnionTypeNode([ts.visitNode(node.type, transformJSDocType), ts.createTypeReferenceNode("null", ts.emptyArray)]);
            }
            function transformJSDocVariadicType(node) {
                return ts.createArrayTypeNode(ts.visitNode(node.type, transformJSDocType));
            }
            function transformJSDocFunctionType(node) {
                return ts.createFunctionTypeNode(ts.emptyArray, node.parameters.map(transformJSDocParameter), node.type);
            }
            function transformJSDocParameter(node) {
                var index = node.parent.parameters.indexOf(node);
                var isRest = node.type.kind === 301 /* JSDocVariadicType */ && index === node.parent.parameters.length - 1; // TODO: GH#18217
                var name = node.name || (isRest ? "rest" : "arg" + index);
                var dotdotdot = isRest ? ts.createToken(25 /* DotDotDotToken */) : node.dotDotDotToken;
                return ts.createParameter(node.decorators, node.modifiers, dotdotdot, name, node.questionToken, ts.visitNode(node.type, transformJSDocType), node.initializer);
            }
            function transformJSDocTypeReference(node) {
                var name = node.typeName;
                var args = node.typeArguments;
                if (ts.isIdentifier(node.typeName)) {
                    if (ts.isJSDocIndexSignature(node)) {
                        return transformJSDocIndexSignature(node);
                    }
                    var text = node.typeName.text;
                    switch (node.typeName.text) {
                        case "String":
                        case "Boolean":
                        case "Object":
                        case "Number":
                            text = text.toLowerCase();
                            break;
                        case "array":
                        case "date":
                        case "promise":
                            text = text[0].toUpperCase() + text.slice(1);
                            break;
                    }
                    name = ts.createIdentifier(text);
                    if ((text === "Array" || text === "Promise") && !node.typeArguments) {
                        args = ts.createNodeArray([ts.createTypeReferenceNode("any", ts.emptyArray)]);
                    }
                    else {
                        args = ts.visitNodes(node.typeArguments, transformJSDocType);
                    }
                }
                return ts.createTypeReferenceNode(name, args);
            }
            function transformJSDocIndexSignature(node) {
                var index = ts.createParameter(
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, 
                /*dotDotDotToken*/ undefined, node.typeArguments[0].kind === 140 /* NumberKeyword */ ? "n" : "s", 
                /*questionToken*/ undefined, ts.createTypeReferenceNode(node.typeArguments[0].kind === 140 /* NumberKeyword */ ? "number" : "string", []), 
                /*initializer*/ undefined);
                var indexSignature = ts.createTypeLiteralNode([ts.createIndexSignature(/*decorators*/ undefined, /*modifiers*/ undefined, [index], node.typeArguments[1])]);
                ts.setEmitFlags(indexSignature, 1 /* SingleLine */);
                return indexSignature;
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "inferFromUsage";
            var errorCodes = [
                // Variable declarations
                ts.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined.code,
                // Variable uses
                ts.Diagnostics.Variable_0_implicitly_has_an_1_type.code,
                // Parameter declarations
                ts.Diagnostics.Parameter_0_implicitly_has_an_1_type.code,
                ts.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type.code,
                // Get Accessor declarations
                ts.Diagnostics.Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation.code,
                ts.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type.code,
                // Set Accessor declarations
                ts.Diagnostics.Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation.code,
                // Property declarations
                ts.Diagnostics.Member_0_implicitly_has_an_1_type.code,
                //// Suggestions
                // Variable declarations
                ts.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_but_a_better_type_may_be_inferred_from_usage.code,
                // Variable uses
                ts.Diagnostics.Variable_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code,
                // Parameter declarations
                ts.Diagnostics.Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code,
                ts.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage.code,
                // Get Accessor declarations
                ts.Diagnostics.Property_0_implicitly_has_type_any_but_a_better_type_for_its_get_accessor_may_be_inferred_from_usage.code,
                ts.Diagnostics._0_implicitly_has_an_1_return_type_but_a_better_type_may_be_inferred_from_usage.code,
                // Set Accessor declarations
                ts.Diagnostics.Property_0_implicitly_has_type_any_but_a_better_type_for_its_set_accessor_may_be_inferred_from_usage.code,
                // Property declarations
                ts.Diagnostics.Member_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code,
                // Function expressions and declarations
                ts.Diagnostics.this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation.code,
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, program = context.program, start = context.span.start, errorCode = context.errorCode, cancellationToken = context.cancellationToken, host = context.host, formatContext = context.formatContext, preferences = context.preferences;
                    var token = ts.getTokenAtPosition(sourceFile, start);
                    var declaration;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (changes) { declaration = doChange(changes, sourceFile, token, errorCode, program, cancellationToken, /*markSeen*/ ts.returnTrue, host, formatContext, preferences); });
                    var name = declaration && ts.getNameOfDeclaration(declaration);
                    return !name || changes.length === 0 ? undefined
                        : [codefix.createCodeFixAction(fixId, changes, [getDiagnostic(errorCode, token), name.getText(sourceFile)], fixId, ts.Diagnostics.Infer_all_types_from_usage)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    var sourceFile = context.sourceFile, program = context.program, cancellationToken = context.cancellationToken, host = context.host, formatContext = context.formatContext, preferences = context.preferences;
                    var markSeen = ts.nodeSeenTracker();
                    return codefix.codeFixAll(context, errorCodes, function (changes, err) {
                        doChange(changes, sourceFile, ts.getTokenAtPosition(err.file, err.start), err.code, program, cancellationToken, markSeen, host, formatContext, preferences);
                    });
                },
            });
            function getDiagnostic(errorCode, token) {
                switch (errorCode) {
                    case ts.Diagnostics.Parameter_0_implicitly_has_an_1_type.code:
                    case ts.Diagnostics.Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code:
                        return ts.isSetAccessorDeclaration(ts.getContainingFunction(token)) ? ts.Diagnostics.Infer_type_of_0_from_usage : ts.Diagnostics.Infer_parameter_types_from_usage; // TODO: GH#18217
                    case ts.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type.code:
                    case ts.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage.code:
                        return ts.Diagnostics.Infer_parameter_types_from_usage;
                    case ts.Diagnostics.this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation.code:
                        return ts.Diagnostics.Infer_this_type_of_0_from_usage;
                    default:
                        return ts.Diagnostics.Infer_type_of_0_from_usage;
                }
            }
            /** Map suggestion code to error code */
            function mapSuggestionDiagnostic(errorCode) {
                switch (errorCode) {
                    case ts.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_but_a_better_type_may_be_inferred_from_usage.code:
                        return ts.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined.code;
                    case ts.Diagnostics.Variable_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code:
                        return ts.Diagnostics.Variable_0_implicitly_has_an_1_type.code;
                    case ts.Diagnostics.Parameter_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code:
                        return ts.Diagnostics.Parameter_0_implicitly_has_an_1_type.code;
                    case ts.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type_but_a_better_type_may_be_inferred_from_usage.code:
                        return ts.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type.code;
                    case ts.Diagnostics.Property_0_implicitly_has_type_any_but_a_better_type_for_its_get_accessor_may_be_inferred_from_usage.code:
                        return ts.Diagnostics.Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation.code;
                    case ts.Diagnostics._0_implicitly_has_an_1_return_type_but_a_better_type_may_be_inferred_from_usage.code:
                        return ts.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type.code;
                    case ts.Diagnostics.Property_0_implicitly_has_type_any_but_a_better_type_for_its_set_accessor_may_be_inferred_from_usage.code:
                        return ts.Diagnostics.Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation.code;
                    case ts.Diagnostics.Member_0_implicitly_has_an_1_type_but_a_better_type_may_be_inferred_from_usage.code:
                        return ts.Diagnostics.Member_0_implicitly_has_an_1_type.code;
                }
                return errorCode;
            }
            function doChange(changes, sourceFile, token, errorCode, program, cancellationToken, markSeen, host, formatContext, preferences) {
                if (!ts.isParameterPropertyModifier(token.kind) && token.kind !== 75 /* Identifier */ && token.kind !== 25 /* DotDotDotToken */ && token.kind !== 104 /* ThisKeyword */) {
                    return undefined;
                }
                var parent = token.parent;
                errorCode = mapSuggestionDiagnostic(errorCode);
                switch (errorCode) {
                    // Variable and Property declarations
                    case ts.Diagnostics.Member_0_implicitly_has_an_1_type.code:
                    case ts.Diagnostics.Variable_0_implicitly_has_type_1_in_some_locations_where_its_type_cannot_be_determined.code:
                        if ((ts.isVariableDeclaration(parent) && markSeen(parent)) || ts.isPropertyDeclaration(parent) || ts.isPropertySignature(parent)) { // handle bad location
                            annotateVariableDeclaration(changes, sourceFile, parent, program, host, cancellationToken, formatContext, preferences);
                            return parent;
                        }
                        if (ts.isPropertyAccessExpression(parent)) {
                            var type = inferTypeForVariableFromUsage(parent.name, program, cancellationToken);
                            var typeNode = ts.getTypeNodeIfAccessible(type, parent, program, host);
                            if (typeNode) {
                                // Note that the codefix will never fire with an existing `@type` tag, so there is no need to merge tags
                                var typeTag = ts.createJSDocTypeTag(ts.createJSDocTypeExpression(typeNode), /*comment*/ "");
                                addJSDocTags(changes, sourceFile, ts.cast(parent.parent.parent, ts.isExpressionStatement), [typeTag]);
                            }
                            return parent;
                        }
                        return undefined;
                    case ts.Diagnostics.Variable_0_implicitly_has_an_1_type.code: {
                        var symbol = program.getTypeChecker().getSymbolAtLocation(token);
                        if (symbol && symbol.valueDeclaration && ts.isVariableDeclaration(symbol.valueDeclaration) && markSeen(symbol.valueDeclaration)) {
                            annotateVariableDeclaration(changes, sourceFile, symbol.valueDeclaration, program, host, cancellationToken, formatContext, preferences);
                            return symbol.valueDeclaration;
                        }
                        return undefined;
                    }
                }
                var containingFunction = ts.getContainingFunction(token);
                if (containingFunction === undefined) {
                    return undefined;
                }
                switch (errorCode) {
                    // Parameter declarations
                    case ts.Diagnostics.Parameter_0_implicitly_has_an_1_type.code:
                        if (ts.isSetAccessorDeclaration(containingFunction)) {
                            annotateSetAccessor(changes, sourceFile, containingFunction, program, host, cancellationToken, formatContext, preferences);
                            return containingFunction;
                        }
                    // falls through
                    case ts.Diagnostics.Rest_parameter_0_implicitly_has_an_any_type.code:
                        if (markSeen(containingFunction)) {
                            var param = ts.cast(parent, ts.isParameter);
                            annotateParameters(changes, sourceFile, param, containingFunction, program, host, cancellationToken, formatContext, preferences);
                            return param;
                        }
                        return undefined;
                    // Get Accessor declarations
                    case ts.Diagnostics.Property_0_implicitly_has_type_any_because_its_get_accessor_lacks_a_return_type_annotation.code:
                    case ts.Diagnostics._0_which_lacks_return_type_annotation_implicitly_has_an_1_return_type.code:
                        if (ts.isGetAccessorDeclaration(containingFunction) && ts.isIdentifier(containingFunction.name)) {
                            annotate(changes, sourceFile, containingFunction, inferTypeForVariableFromUsage(containingFunction.name, program, cancellationToken), program, host, formatContext, preferences);
                            return containingFunction;
                        }
                        return undefined;
                    // Set Accessor declarations
                    case ts.Diagnostics.Property_0_implicitly_has_type_any_because_its_set_accessor_lacks_a_parameter_type_annotation.code:
                        if (ts.isSetAccessorDeclaration(containingFunction)) {
                            annotateSetAccessor(changes, sourceFile, containingFunction, program, host, cancellationToken, formatContext, preferences);
                            return containingFunction;
                        }
                        return undefined;
                    // Function 'this'
                    case ts.Diagnostics.this_implicitly_has_type_any_because_it_does_not_have_a_type_annotation.code:
                        if (ts.textChanges.isThisTypeAnnotatable(containingFunction) && markSeen(containingFunction)) {
                            annotateThis(changes, sourceFile, containingFunction, program, host, cancellationToken);
                            return containingFunction;
                        }
                        return undefined;
                    default:
                        return ts.Debug.fail(String(errorCode));
                }
            }
            function annotateVariableDeclaration(changes, sourceFile, declaration, program, host, cancellationToken, formatContext, preferences) {
                if (ts.isIdentifier(declaration.name)) {
                    annotate(changes, sourceFile, declaration, inferTypeForVariableFromUsage(declaration.name, program, cancellationToken), program, host, formatContext, preferences);
                }
            }
            function annotateParameters(changes, sourceFile, parameterDeclaration, containingFunction, program, host, cancellationToken, formatContext, preferences) {
                if (!ts.isIdentifier(parameterDeclaration.name)) {
                    return;
                }
                var parameterInferences = inferTypeForParametersFromUsage(containingFunction, sourceFile, program, cancellationToken);
                ts.Debug.assert(containingFunction.parameters.length === parameterInferences.length, "Parameter count and inference count should match");
                if (ts.isInJSFile(containingFunction)) {
                    annotateJSDocParameters(changes, sourceFile, parameterInferences, program, host);
                }
                else {
                    var needParens = ts.isArrowFunction(containingFunction) && !ts.findChildOfKind(containingFunction, 20 /* OpenParenToken */, sourceFile);
                    if (needParens)
                        changes.insertNodeBefore(sourceFile, ts.first(containingFunction.parameters), ts.createToken(20 /* OpenParenToken */));
                    for (var _i = 0, parameterInferences_1 = parameterInferences; _i < parameterInferences_1.length; _i++) {
                        var _a = parameterInferences_1[_i], declaration = _a.declaration, type = _a.type;
                        if (declaration && !declaration.type && !declaration.initializer) {
                            annotate(changes, sourceFile, declaration, type, program, host, formatContext, preferences);
                        }
                    }
                    if (needParens)
                        changes.insertNodeAfter(sourceFile, ts.last(containingFunction.parameters), ts.createToken(21 /* CloseParenToken */));
                }
            }
            function annotateThis(changes, sourceFile, containingFunction, program, host, cancellationToken) {
                var references = getFunctionReferences(containingFunction, sourceFile, program, cancellationToken);
                if (!references || !references.length) {
                    return;
                }
                var thisInference = inferTypeFromReferences(program, references, cancellationToken).thisParameter();
                var typeNode = ts.getTypeNodeIfAccessible(thisInference, containingFunction, program, host);
                if (!typeNode) {
                    return;
                }
                if (ts.isInJSFile(containingFunction)) {
                    annotateJSDocThis(changes, sourceFile, containingFunction, typeNode);
                }
                else {
                    changes.tryInsertThisTypeAnnotation(sourceFile, containingFunction, typeNode);
                }
            }
            function annotateJSDocThis(changes, sourceFile, containingFunction, typeNode) {
                addJSDocTags(changes, sourceFile, containingFunction, [
                    ts.createJSDocThisTag(ts.createJSDocTypeExpression(typeNode)),
                ]);
            }
            function annotateSetAccessor(changes, sourceFile, setAccessorDeclaration, program, host, cancellationToken, formatContext, preferences) {
                var param = ts.firstOrUndefined(setAccessorDeclaration.parameters);
                if (param && ts.isIdentifier(setAccessorDeclaration.name) && ts.isIdentifier(param.name)) {
                    var type = inferTypeForVariableFromUsage(setAccessorDeclaration.name, program, cancellationToken);
                    if (type === program.getTypeChecker().getAnyType()) {
                        type = inferTypeForVariableFromUsage(param.name, program, cancellationToken);
                    }
                    if (ts.isInJSFile(setAccessorDeclaration)) {
                        annotateJSDocParameters(changes, sourceFile, [{ declaration: param, type: type }], program, host);
                    }
                    else {
                        annotate(changes, sourceFile, param, type, program, host, formatContext, preferences);
                    }
                }
            }
            function annotate(changes, sourceFile, declaration, type, program, host, formatContext, preferences) {
                var typeNode = ts.getTypeNodeIfAccessible(type, declaration, program, host);
                if (typeNode) {
                    if (ts.isInJSFile(sourceFile) && declaration.kind !== 158 /* PropertySignature */) {
                        var parent = ts.isVariableDeclaration(declaration) ? ts.tryCast(declaration.parent.parent, ts.isVariableStatement) : declaration;
                        if (!parent) {
                            return;
                        }
                        var typeExpression = ts.createJSDocTypeExpression(typeNode);
                        var typeTag = ts.isGetAccessorDeclaration(declaration) ? ts.createJSDocReturnTag(typeExpression, "") : ts.createJSDocTypeTag(typeExpression, "");
                        addJSDocTags(changes, sourceFile, parent, [typeTag]);
                    }
                    else if (!tryReplaceImportTypeNodeWithAutoImport(typeNode, changes, sourceFile, declaration, type, program, host, formatContext, preferences)) {
                        changes.tryInsertTypeAnnotation(sourceFile, declaration, typeNode);
                    }
                }
            }
            function tryReplaceImportTypeNodeWithAutoImport(typeNode, changes, sourceFile, declaration, type, program, host, formatContext, preferences) {
                var _a;
                if (ts.isLiteralImportTypeNode(typeNode) && typeNode.qualifier && type.symbol) {
                    // Replace 'import("./a").SomeType' with 'SomeType' and an actual import if possible
                    var moduleSymbol = (_a = ts.find(type.symbol.declarations, function (d) { return !!d.getSourceFile().externalModuleIndicator; })) === null || _a === void 0 ? void 0 : _a.getSourceFile().symbol;
                    // Symbol for the left-most thing after the dot
                    if (moduleSymbol) {
                        var symbol = ts.getFirstIdentifier(typeNode.qualifier).symbol;
                        var action = codefix.getImportCompletionAction(symbol, moduleSymbol, sourceFile, symbol.name, host, program, formatContext, declaration.pos, preferences);
                        if (action.codeAction.changes.length && changes.tryInsertTypeAnnotation(sourceFile, declaration, ts.createTypeReferenceNode(typeNode.qualifier, typeNode.typeArguments))) {
                            for (var _i = 0, _b = action.codeAction.changes; _i < _b.length; _i++) {
                                var change = _b[_i];
                                var file = sourceFile.fileName === change.fileName ? sourceFile : ts.Debug.assertDefined(program.getSourceFile(change.fileName));
                                changes.pushRaw(file, change);
                            }
                            return true;
                        }
                    }
                }
                return false;
            }
            function annotateJSDocParameters(changes, sourceFile, parameterInferences, program, host) {
                var signature = parameterInferences.length && parameterInferences[0].declaration.parent;
                if (!signature) {
                    return;
                }
                var paramTags = ts.mapDefined(parameterInferences, function (inference) {
                    var param = inference.declaration;
                    // only infer parameters that have (1) no type and (2) an accessible inferred type
                    if (param.initializer || ts.getJSDocType(param) || !ts.isIdentifier(param.name))
                        return;
                    var typeNode = inference.type && ts.getTypeNodeIfAccessible(inference.type, param, program, host);
                    var name = ts.getSynthesizedClone(param.name);
                    ts.setEmitFlags(name, 1536 /* NoComments */ | 2048 /* NoNestedComments */);
                    return typeNode && ts.createJSDocParamTag(name, !!inference.isOptional, ts.createJSDocTypeExpression(typeNode), "");
                });
                addJSDocTags(changes, sourceFile, signature, paramTags);
            }
            function addJSDocTags(changes, sourceFile, parent, newTags) {
                var comments = ts.mapDefined(parent.jsDoc, function (j) { return j.comment; });
                var oldTags = ts.flatMapToMutable(parent.jsDoc, function (j) { return j.tags; });
                var unmergedNewTags = newTags.filter(function (newTag) {
                    return !oldTags || !oldTags.some(function (tag, i) {
                        var merged = tryMergeJsdocTags(tag, newTag);
                        if (merged)
                            oldTags[i] = merged;
                        return !!merged;
                    });
                });
                var tag = ts.createJSDocComment(comments.join("\n"), ts.createNodeArray(__spreadArrays((oldTags || ts.emptyArray), unmergedNewTags)));
                var jsDocNode = parent.kind === 202 /* ArrowFunction */ ? getJsDocNodeForArrowFunction(parent) : parent;
                jsDocNode.jsDoc = parent.jsDoc;
                jsDocNode.jsDocCache = parent.jsDocCache;
                changes.insertJsdocCommentBefore(sourceFile, jsDocNode, tag);
            }
            function getJsDocNodeForArrowFunction(signature) {
                if (signature.parent.kind === 159 /* PropertyDeclaration */) {
                    return signature.parent;
                }
                return signature.parent.parent;
            }
            function tryMergeJsdocTags(oldTag, newTag) {
                if (oldTag.kind !== newTag.kind) {
                    return undefined;
                }
                switch (oldTag.kind) {
                    case 316 /* JSDocParameterTag */: {
                        var oldParam = oldTag;
                        var newParam = newTag;
                        return ts.isIdentifier(oldParam.name) && ts.isIdentifier(newParam.name) && oldParam.name.escapedText === newParam.name.escapedText
                            ? ts.createJSDocParamTag(newParam.name, newParam.isBracketed, newParam.typeExpression, oldParam.comment)
                            : undefined;
                    }
                    case 317 /* JSDocReturnTag */:
                        return ts.createJSDocReturnTag(newTag.typeExpression, oldTag.comment);
                }
            }
            function getReferences(token, program, cancellationToken) {
                // Position shouldn't matter since token is not a SourceFile.
                return ts.mapDefined(ts.FindAllReferences.getReferenceEntriesForNode(-1, token, program, program.getSourceFiles(), cancellationToken), function (entry) {
                    return entry.kind !== 0 /* Span */ ? ts.tryCast(entry.node, ts.isIdentifier) : undefined;
                });
            }
            function inferTypeForVariableFromUsage(token, program, cancellationToken) {
                var references = getReferences(token, program, cancellationToken);
                return inferTypeFromReferences(program, references, cancellationToken).single();
            }
            function inferTypeForParametersFromUsage(func, sourceFile, program, cancellationToken) {
                var references = getFunctionReferences(func, sourceFile, program, cancellationToken);
                return references && inferTypeFromReferences(program, references, cancellationToken).parameters(func) ||
                    func.parameters.map(function (p) {
                        return ({
                            declaration: p,
                            type: ts.isIdentifier(p.name) ? inferTypeForVariableFromUsage(p.name, program, cancellationToken) : program.getTypeChecker().getAnyType()
                        });
                    });
            }
            function getFunctionReferences(containingFunction, sourceFile, program, cancellationToken) {
                var searchToken;
                switch (containingFunction.kind) {
                    case 162 /* Constructor */:
                        searchToken = ts.findChildOfKind(containingFunction, 129 /* ConstructorKeyword */, sourceFile);
                        break;
                    case 202 /* ArrowFunction */:
                    case 201 /* FunctionExpression */:
                        var parent = containingFunction.parent;
                        searchToken = ts.isVariableDeclaration(parent) && ts.isIdentifier(parent.name) ?
                            parent.name :
                            containingFunction.name;
                        break;
                    case 244 /* FunctionDeclaration */:
                    case 161 /* MethodDeclaration */:
                        searchToken = containingFunction.name;
                        break;
                }
                if (!searchToken) {
                    return undefined;
                }
                return getReferences(searchToken, program, cancellationToken);
            }
            function inferTypeFromReferences(program, references, cancellationToken) {
                var checker = program.getTypeChecker();
                var builtinConstructors = {
                    string: function () { return checker.getStringType(); },
                    number: function () { return checker.getNumberType(); },
                    Array: function (t) { return checker.createArrayType(t); },
                    Promise: function (t) { return checker.createPromiseType(t); },
                };
                var builtins = [
                    checker.getStringType(),
                    checker.getNumberType(),
                    checker.createArrayType(checker.getAnyType()),
                    checker.createPromiseType(checker.getAnyType()),
                ];
                return {
                    single: single,
                    parameters: parameters,
                    thisParameter: thisParameter,
                };
                function createEmptyUsage() {
                    return {
                        isNumber: undefined,
                        isString: undefined,
                        isNumberOrString: undefined,
                        candidateTypes: undefined,
                        properties: undefined,
                        calls: undefined,
                        constructs: undefined,
                        numberIndex: undefined,
                        stringIndex: undefined,
                        candidateThisTypes: undefined,
                        inferredTypes: undefined,
                    };
                }
                function combineUsages(usages) {
                    var combinedProperties = ts.createUnderscoreEscapedMap();
                    for (var _i = 0, usages_1 = usages; _i < usages_1.length; _i++) {
                        var u = usages_1[_i];
                        if (u.properties) {
                            u.properties.forEach(function (p, name) {
                                if (!combinedProperties.has(name)) {
                                    combinedProperties.set(name, []);
                                }
                                combinedProperties.get(name).push(p);
                            });
                        }
                    }
                    var properties = ts.createUnderscoreEscapedMap();
                    combinedProperties.forEach(function (ps, name) {
                        properties.set(name, combineUsages(ps));
                    });
                    return {
                        isNumber: usages.some(function (u) { return u.isNumber; }),
                        isString: usages.some(function (u) { return u.isString; }),
                        isNumberOrString: usages.some(function (u) { return u.isNumberOrString; }),
                        candidateTypes: ts.flatMap(usages, function (u) { return u.candidateTypes; }),
                        properties: properties,
                        calls: ts.flatMap(usages, function (u) { return u.calls; }),
                        constructs: ts.flatMap(usages, function (u) { return u.constructs; }),
                        numberIndex: ts.forEach(usages, function (u) { return u.numberIndex; }),
                        stringIndex: ts.forEach(usages, function (u) { return u.stringIndex; }),
                        candidateThisTypes: ts.flatMap(usages, function (u) { return u.candidateThisTypes; }),
                        inferredTypes: undefined,
                    };
                }
                function single() {
                    return combineTypes(inferTypesFromReferencesSingle(references));
                }
                function parameters(declaration) {
                    if (references.length === 0 || !declaration.parameters) {
                        return undefined;
                    }
                    var usage = createEmptyUsage();
                    for (var _i = 0, references_2 = references; _i < references_2.length; _i++) {
                        var reference = references_2[_i];
                        cancellationToken.throwIfCancellationRequested();
                        calculateUsageOfNode(reference, usage);
                    }
                    var calls = __spreadArrays(usage.constructs || [], usage.calls || []);
                    return declaration.parameters.map(function (parameter, parameterIndex) {
                        var types = [];
                        var isRest = ts.isRestParameter(parameter);
                        var isOptional = false;
                        for (var _i = 0, calls_1 = calls; _i < calls_1.length; _i++) {
                            var call = calls_1[_i];
                            if (call.argumentTypes.length <= parameterIndex) {
                                isOptional = ts.isInJSFile(declaration);
                                types.push(checker.getUndefinedType());
                            }
                            else if (isRest) {
                                for (var i = parameterIndex; i < call.argumentTypes.length; i++) {
                                    types.push(checker.getBaseTypeOfLiteralType(call.argumentTypes[i]));
                                }
                            }
                            else {
                                types.push(checker.getBaseTypeOfLiteralType(call.argumentTypes[parameterIndex]));
                            }
                        }
                        if (ts.isIdentifier(parameter.name)) {
                            var inferred = inferTypesFromReferencesSingle(getReferences(parameter.name, program, cancellationToken));
                            types.push.apply(types, (isRest ? ts.mapDefined(inferred, checker.getElementTypeOfArrayType) : inferred));
                        }
                        var type = combineTypes(types);
                        return {
                            type: isRest ? checker.createArrayType(type) : type,
                            isOptional: isOptional && !isRest,
                            declaration: parameter
                        };
                    });
                }
                function thisParameter() {
                    var usage = createEmptyUsage();
                    for (var _i = 0, references_3 = references; _i < references_3.length; _i++) {
                        var reference = references_3[_i];
                        cancellationToken.throwIfCancellationRequested();
                        calculateUsageOfNode(reference, usage);
                    }
                    return combineTypes(usage.candidateThisTypes || ts.emptyArray);
                }
                function inferTypesFromReferencesSingle(references) {
                    var usage = createEmptyUsage();
                    for (var _i = 0, references_4 = references; _i < references_4.length; _i++) {
                        var reference = references_4[_i];
                        cancellationToken.throwIfCancellationRequested();
                        calculateUsageOfNode(reference, usage);
                    }
                    return inferTypes(usage);
                }
                function calculateUsageOfNode(node, usage) {
                    while (ts.isRightSideOfQualifiedNameOrPropertyAccess(node)) {
                        node = node.parent;
                    }
                    switch (node.parent.kind) {
                        case 226 /* ExpressionStatement */:
                            addCandidateType(usage, checker.getVoidType());
                            break;
                        case 208 /* PostfixUnaryExpression */:
                            usage.isNumber = true;
                            break;
                        case 207 /* PrefixUnaryExpression */:
                            inferTypeFromPrefixUnaryExpression(node.parent, usage);
                            break;
                        case 209 /* BinaryExpression */:
                            inferTypeFromBinaryExpression(node, node.parent, usage);
                            break;
                        case 277 /* CaseClause */:
                        case 278 /* DefaultClause */:
                            inferTypeFromSwitchStatementLabel(node.parent, usage);
                            break;
                        case 196 /* CallExpression */:
                        case 197 /* NewExpression */:
                            if (node.parent.expression === node) {
                                inferTypeFromCallExpression(node.parent, usage);
                            }
                            else {
                                inferTypeFromContextualType(node, usage);
                            }
                            break;
                        case 194 /* PropertyAccessExpression */:
                            inferTypeFromPropertyAccessExpression(node.parent, usage);
                            break;
                        case 195 /* ElementAccessExpression */:
                            inferTypeFromPropertyElementExpression(node.parent, node, usage);
                            break;
                        case 281 /* PropertyAssignment */:
                        case 282 /* ShorthandPropertyAssignment */:
                            inferTypeFromPropertyAssignment(node.parent, usage);
                            break;
                        case 159 /* PropertyDeclaration */:
                            inferTypeFromPropertyDeclaration(node.parent, usage);
                            break;
                        case 242 /* VariableDeclaration */: {
                            var _a = node.parent, name = _a.name, initializer = _a.initializer;
                            if (node === name) {
                                if (initializer) { // This can happen for `let x = null;` which still has an implicit-any error.
                                    addCandidateType(usage, checker.getTypeAtLocation(initializer));
                                }
                                break;
                            }
                        }
                        // falls through
                        default:
                            return inferTypeFromContextualType(node, usage);
                    }
                }
                function inferTypeFromContextualType(node, usage) {
                    if (ts.isExpressionNode(node)) {
                        addCandidateType(usage, checker.getContextualType(node));
                    }
                }
                function inferTypeFromPrefixUnaryExpression(node, usage) {
                    switch (node.operator) {
                        case 45 /* PlusPlusToken */:
                        case 46 /* MinusMinusToken */:
                        case 40 /* MinusToken */:
                        case 54 /* TildeToken */:
                            usage.isNumber = true;
                            break;
                        case 39 /* PlusToken */:
                            usage.isNumberOrString = true;
                            break;
                        // case SyntaxKind.ExclamationToken:
                        // no inferences here;
                    }
                }
                function inferTypeFromBinaryExpression(node, parent, usage) {
                    switch (parent.operatorToken.kind) {
                        // ExponentiationOperator
                        case 42 /* AsteriskAsteriskToken */:
                        // MultiplicativeOperator
                        // falls through
                        case 41 /* AsteriskToken */:
                        case 43 /* SlashToken */:
                        case 44 /* PercentToken */:
                        // ShiftOperator
                        // falls through
                        case 47 /* LessThanLessThanToken */:
                        case 48 /* GreaterThanGreaterThanToken */:
                        case 49 /* GreaterThanGreaterThanGreaterThanToken */:
                        // BitwiseOperator
                        // falls through
                        case 50 /* AmpersandToken */:
                        case 51 /* BarToken */:
                        case 52 /* CaretToken */:
                        // CompoundAssignmentOperator
                        // falls through
                        case 64 /* MinusEqualsToken */:
                        case 66 /* AsteriskAsteriskEqualsToken */:
                        case 65 /* AsteriskEqualsToken */:
                        case 67 /* SlashEqualsToken */:
                        case 68 /* PercentEqualsToken */:
                        case 72 /* AmpersandEqualsToken */:
                        case 73 /* BarEqualsToken */:
                        case 74 /* CaretEqualsToken */:
                        case 69 /* LessThanLessThanEqualsToken */:
                        case 71 /* GreaterThanGreaterThanGreaterThanEqualsToken */:
                        case 70 /* GreaterThanGreaterThanEqualsToken */:
                        // AdditiveOperator
                        // falls through
                        case 40 /* MinusToken */:
                        // RelationalOperator
                        // falls through
                        case 29 /* LessThanToken */:
                        case 32 /* LessThanEqualsToken */:
                        case 31 /* GreaterThanToken */:
                        case 33 /* GreaterThanEqualsToken */:
                            var operandType = checker.getTypeAtLocation(parent.left === node ? parent.right : parent.left);
                            if (operandType.flags & 1056 /* EnumLike */) {
                                addCandidateType(usage, operandType);
                            }
                            else {
                                usage.isNumber = true;
                            }
                            break;
                        case 63 /* PlusEqualsToken */:
                        case 39 /* PlusToken */:
                            var otherOperandType = checker.getTypeAtLocation(parent.left === node ? parent.right : parent.left);
                            if (otherOperandType.flags & 1056 /* EnumLike */) {
                                addCandidateType(usage, otherOperandType);
                            }
                            else if (otherOperandType.flags & 296 /* NumberLike */) {
                                usage.isNumber = true;
                            }
                            else if (otherOperandType.flags & 132 /* StringLike */) {
                                usage.isString = true;
                            }
                            else if (otherOperandType.flags & 1 /* Any */) {
                                // do nothing, maybe we'll learn something elsewhere
                            }
                            else {
                                usage.isNumberOrString = true;
                            }
                            break;
                        //  AssignmentOperators
                        case 62 /* EqualsToken */:
                        case 34 /* EqualsEqualsToken */:
                        case 36 /* EqualsEqualsEqualsToken */:
                        case 37 /* ExclamationEqualsEqualsToken */:
                        case 35 /* ExclamationEqualsToken */:
                            addCandidateType(usage, checker.getTypeAtLocation(parent.left === node ? parent.right : parent.left));
                            break;
                        case 97 /* InKeyword */:
                            if (node === parent.left) {
                                usage.isString = true;
                            }
                            break;
                        // LogicalOperator Or NullishCoalescing
                        case 56 /* BarBarToken */:
                        case 60 /* QuestionQuestionToken */:
                            if (node === parent.left &&
                                (node.parent.parent.kind === 242 /* VariableDeclaration */ || ts.isAssignmentExpression(node.parent.parent, /*excludeCompoundAssignment*/ true))) {
                                // var x = x || {};
                                // TODO: use getFalsyflagsOfType
                                addCandidateType(usage, checker.getTypeAtLocation(parent.right));
                            }
                            break;
                        case 55 /* AmpersandAmpersandToken */:
                        case 27 /* CommaToken */:
                        case 98 /* InstanceOfKeyword */:
                            // nothing to infer here
                            break;
                    }
                }
                function inferTypeFromSwitchStatementLabel(parent, usage) {
                    addCandidateType(usage, checker.getTypeAtLocation(parent.parent.parent.expression));
                }
                function inferTypeFromCallExpression(parent, usage) {
                    var call = {
                        argumentTypes: [],
                        return_: createEmptyUsage()
                    };
                    if (parent.arguments) {
                        for (var _i = 0, _a = parent.arguments; _i < _a.length; _i++) {
                            var argument = _a[_i];
                            call.argumentTypes.push(checker.getTypeAtLocation(argument));
                        }
                    }
                    calculateUsageOfNode(parent, call.return_);
                    if (parent.kind === 196 /* CallExpression */) {
                        (usage.calls || (usage.calls = [])).push(call);
                    }
                    else {
                        (usage.constructs || (usage.constructs = [])).push(call);
                    }
                }
                function inferTypeFromPropertyAccessExpression(parent, usage) {
                    var name = ts.escapeLeadingUnderscores(parent.name.text);
                    if (!usage.properties) {
                        usage.properties = ts.createUnderscoreEscapedMap();
                    }
                    var propertyUsage = usage.properties.get(name) || createEmptyUsage();
                    calculateUsageOfNode(parent, propertyUsage);
                    usage.properties.set(name, propertyUsage);
                }
                function inferTypeFromPropertyElementExpression(parent, node, usage) {
                    if (node === parent.argumentExpression) {
                        usage.isNumberOrString = true;
                        return;
                    }
                    else {
                        var indexType = checker.getTypeAtLocation(parent.argumentExpression);
                        var indexUsage = createEmptyUsage();
                        calculateUsageOfNode(parent, indexUsage);
                        if (indexType.flags & 296 /* NumberLike */) {
                            usage.numberIndex = indexUsage;
                        }
                        else {
                            usage.stringIndex = indexUsage;
                        }
                    }
                }
                function inferTypeFromPropertyAssignment(assignment, usage) {
                    var nodeWithRealType = ts.isVariableDeclaration(assignment.parent.parent) ?
                        assignment.parent.parent :
                        assignment.parent;
                    addCandidateThisType(usage, checker.getTypeAtLocation(nodeWithRealType));
                }
                function inferTypeFromPropertyDeclaration(declaration, usage) {
                    addCandidateThisType(usage, checker.getTypeAtLocation(declaration.parent));
                }
                function removeLowPriorityInferences(inferences, priorities) {
                    var toRemove = [];
                    for (var _i = 0, inferences_1 = inferences; _i < inferences_1.length; _i++) {
                        var i = inferences_1[_i];
                        for (var _a = 0, priorities_1 = priorities; _a < priorities_1.length; _a++) {
                            var _b = priorities_1[_a], high = _b.high, low = _b.low;
                            if (high(i)) {
                                ts.Debug.assert(!low(i), "Priority can't have both low and high");
                                toRemove.push(low);
                            }
                        }
                    }
                    return inferences.filter(function (i) { return toRemove.every(function (f) { return !f(i); }); });
                }
                function combineFromUsage(usage) {
                    return combineTypes(inferTypes(usage));
                }
                function combineTypes(inferences) {
                    if (!inferences.length)
                        return checker.getAnyType();
                    // 1. string or number individually override string | number
                    // 2. non-any, non-void overrides any or void
                    // 3. non-nullable, non-any, non-void, non-anonymous overrides anonymous types
                    var stringNumber = checker.getUnionType([checker.getStringType(), checker.getNumberType()]);
                    var priorities = [
                        {
                            high: function (t) { return t === checker.getStringType() || t === checker.getNumberType(); },
                            low: function (t) { return t === stringNumber; }
                        },
                        {
                            high: function (t) { return !(t.flags & (1 /* Any */ | 16384 /* Void */)); },
                            low: function (t) { return !!(t.flags & (1 /* Any */ | 16384 /* Void */)); }
                        },
                        {
                            high: function (t) { return !(t.flags & (98304 /* Nullable */ | 1 /* Any */ | 16384 /* Void */)) && !(ts.getObjectFlags(t) & 16 /* Anonymous */); },
                            low: function (t) { return !!(ts.getObjectFlags(t) & 16 /* Anonymous */); }
                        }
                    ];
                    var good = removeLowPriorityInferences(inferences, priorities);
                    var anons = good.filter(function (i) { return ts.getObjectFlags(i) & 16 /* Anonymous */; });
                    if (anons.length) {
                        good = good.filter(function (i) { return !(ts.getObjectFlags(i) & 16 /* Anonymous */); });
                        good.push(combineAnonymousTypes(anons));
                    }
                    return checker.getWidenedType(checker.getUnionType(good.map(checker.getBaseTypeOfLiteralType), 2 /* Subtype */));
                }
                function combineAnonymousTypes(anons) {
                    if (anons.length === 1) {
                        return anons[0];
                    }
                    var calls = [];
                    var constructs = [];
                    var stringIndices = [];
                    var numberIndices = [];
                    var stringIndexReadonly = false;
                    var numberIndexReadonly = false;
                    var props = ts.createMultiMap();
                    for (var _i = 0, anons_1 = anons; _i < anons_1.length; _i++) {
                        var anon = anons_1[_i];
                        for (var _a = 0, _b = checker.getPropertiesOfType(anon); _a < _b.length; _a++) {
                            var p = _b[_a];
                            props.add(p.name, checker.getTypeOfSymbolAtLocation(p, p.valueDeclaration));
                        }
                        calls.push.apply(calls, checker.getSignaturesOfType(anon, 0 /* Call */));
                        constructs.push.apply(constructs, checker.getSignaturesOfType(anon, 1 /* Construct */));
                        if (anon.stringIndexInfo) {
                            stringIndices.push(anon.stringIndexInfo.type);
                            stringIndexReadonly = stringIndexReadonly || anon.stringIndexInfo.isReadonly;
                        }
                        if (anon.numberIndexInfo) {
                            numberIndices.push(anon.numberIndexInfo.type);
                            numberIndexReadonly = numberIndexReadonly || anon.numberIndexInfo.isReadonly;
                        }
                    }
                    var members = ts.mapEntries(props, function (name, types) {
                        var isOptional = types.length < anons.length ? 16777216 /* Optional */ : 0;
                        var s = checker.createSymbol(4 /* Property */ | isOptional, name);
                        s.type = checker.getUnionType(types);
                        return [name, s];
                    });
                    return checker.createAnonymousType(anons[0].symbol, members, calls, constructs, stringIndices.length ? checker.createIndexInfo(checker.getUnionType(stringIndices), stringIndexReadonly) : undefined, numberIndices.length ? checker.createIndexInfo(checker.getUnionType(numberIndices), numberIndexReadonly) : undefined);
                }
                function inferTypes(usage) {
                    var types = [];
                    if (usage.isNumber) {
                        types.push(checker.getNumberType());
                    }
                    if (usage.isString) {
                        types.push(checker.getStringType());
                    }
                    if (usage.isNumberOrString) {
                        types.push(checker.getUnionType([checker.getStringType(), checker.getNumberType()]));
                    }
                    if (usage.numberIndex) {
                        types.push(checker.createArrayType(combineFromUsage(usage.numberIndex)));
                    }
                    if (usage.properties && usage.properties.size
                        || usage.calls && usage.calls.length
                        || usage.constructs && usage.constructs.length
                        || usage.stringIndex) {
                        types.push(inferStructuralType(usage));
                    }
                    types.push.apply(types, (usage.candidateTypes || []).map(function (t) { return checker.getBaseTypeOfLiteralType(t); }));
                    types.push.apply(types, inferNamedTypesFromProperties(usage));
                    return types;
                }
                function inferStructuralType(usage) {
                    var members = ts.createUnderscoreEscapedMap();
                    if (usage.properties) {
                        usage.properties.forEach(function (u, name) {
                            var symbol = checker.createSymbol(4 /* Property */, name);
                            symbol.type = combineFromUsage(u);
                            members.set(name, symbol);
                        });
                    }
                    var callSignatures = usage.calls ? [getSignatureFromCalls(usage.calls)] : [];
                    var constructSignatures = usage.constructs ? [getSignatureFromCalls(usage.constructs)] : [];
                    var stringIndexInfo = usage.stringIndex && checker.createIndexInfo(combineFromUsage(usage.stringIndex), /*isReadonly*/ false);
                    return checker.createAnonymousType(/*symbol*/ undefined, members, callSignatures, constructSignatures, stringIndexInfo, /*numberIndexInfo*/ undefined); // TODO: GH#18217
                }
                function inferNamedTypesFromProperties(usage) {
                    if (!usage.properties || !usage.properties.size)
                        return [];
                    var types = builtins.filter(function (t) { return allPropertiesAreAssignableToUsage(t, usage); });
                    if (0 < types.length && types.length < 3) {
                        return types.map(function (t) { return inferInstantiationFromUsage(t, usage); });
                    }
                    return [];
                }
                function allPropertiesAreAssignableToUsage(type, usage) {
                    if (!usage.properties)
                        return false;
                    return !ts.forEachEntry(usage.properties, function (propUsage, name) {
                        var source = checker.getTypeOfPropertyOfType(type, name);
                        if (!source) {
                            return true;
                        }
                        if (propUsage.calls) {
                            var sigs = checker.getSignaturesOfType(source, 0 /* Call */);
                            return !sigs.length || !checker.isTypeAssignableTo(source, getFunctionFromCalls(propUsage.calls));
                        }
                        else {
                            return !checker.isTypeAssignableTo(source, combineFromUsage(propUsage));
                        }
                    });
                }
                /**
                 * inference is limited to
                 * 1. generic types with a single parameter
                 * 2. inference to/from calls with a single signature
                 */
                function inferInstantiationFromUsage(type, usage) {
                    if (!(ts.getObjectFlags(type) & 4 /* Reference */) || !usage.properties) {
                        return type;
                    }
                    var generic = type.target;
                    var singleTypeParameter = ts.singleOrUndefined(generic.typeParameters);
                    if (!singleTypeParameter)
                        return type;
                    var types = [];
                    usage.properties.forEach(function (propUsage, name) {
                        var genericPropertyType = checker.getTypeOfPropertyOfType(generic, name);
                        ts.Debug.assert(!!genericPropertyType, "generic should have all the properties of its reference.");
                        types.push.apply(types, inferTypeParameters(genericPropertyType, combineFromUsage(propUsage), singleTypeParameter));
                    });
                    return builtinConstructors[type.symbol.escapedName](combineTypes(types));
                }
                function inferTypeParameters(genericType, usageType, typeParameter) {
                    if (genericType === typeParameter) {
                        return [usageType];
                    }
                    else if (genericType.flags & 3145728 /* UnionOrIntersection */) {
                        return ts.flatMap(genericType.types, function (t) { return inferTypeParameters(t, usageType, typeParameter); });
                    }
                    else if (ts.getObjectFlags(genericType) & 4 /* Reference */ && ts.getObjectFlags(usageType) & 4 /* Reference */) {
                        // this is wrong because we need a reference to the targetType to, so we can check that it's also a reference
                        var genericArgs = checker.getTypeArguments(genericType);
                        var usageArgs = checker.getTypeArguments(usageType);
                        var types = [];
                        if (genericArgs && usageArgs) {
                            for (var i = 0; i < genericArgs.length; i++) {
                                if (usageArgs[i]) {
                                    types.push.apply(types, inferTypeParameters(genericArgs[i], usageArgs[i], typeParameter));
                                }
                            }
                        }
                        return types;
                    }
                    var genericSigs = checker.getSignaturesOfType(genericType, 0 /* Call */);
                    var usageSigs = checker.getSignaturesOfType(usageType, 0 /* Call */);
                    if (genericSigs.length === 1 && usageSigs.length === 1) {
                        return inferFromSignatures(genericSigs[0], usageSigs[0], typeParameter);
                    }
                    return [];
                }
                function inferFromSignatures(genericSig, usageSig, typeParameter) {
                    var types = [];
                    for (var i = 0; i < genericSig.parameters.length; i++) {
                        var genericParam = genericSig.parameters[i];
                        var usageParam = usageSig.parameters[i];
                        var isRest = genericSig.declaration && ts.isRestParameter(genericSig.declaration.parameters[i]);
                        if (!usageParam) {
                            break;
                        }
                        var genericParamType = checker.getTypeOfSymbolAtLocation(genericParam, genericParam.valueDeclaration);
                        var elementType = isRest && checker.getElementTypeOfArrayType(genericParamType);
                        if (elementType) {
                            genericParamType = elementType;
                        }
                        var targetType = usageParam.type || checker.getTypeOfSymbolAtLocation(usageParam, usageParam.valueDeclaration);
                        types.push.apply(types, inferTypeParameters(genericParamType, targetType, typeParameter));
                    }
                    var genericReturn = checker.getReturnTypeOfSignature(genericSig);
                    var usageReturn = checker.getReturnTypeOfSignature(usageSig);
                    types.push.apply(types, inferTypeParameters(genericReturn, usageReturn, typeParameter));
                    return types;
                }
                function getFunctionFromCalls(calls) {
                    return checker.createAnonymousType(undefined, ts.createSymbolTable(), [getSignatureFromCalls(calls)], ts.emptyArray, /*stringIndexInfo*/ undefined, /*numberIndexInfo*/ undefined);
                }
                function getSignatureFromCalls(calls) {
                    var parameters = [];
                    var length = Math.max.apply(Math, calls.map(function (c) { return c.argumentTypes.length; }));
                    var _loop_12 = function (i) {
                        var symbol = checker.createSymbol(1 /* FunctionScopedVariable */, ts.escapeLeadingUnderscores("arg" + i));
                        symbol.type = combineTypes(calls.map(function (call) { return call.argumentTypes[i] || checker.getUndefinedType(); }));
                        if (calls.some(function (call) { return call.argumentTypes[i] === undefined; })) {
                            symbol.flags |= 16777216 /* Optional */;
                        }
                        parameters.push(symbol);
                    };
                    for (var i = 0; i < length; i++) {
                        _loop_12(i);
                    }
                    var returnType = combineFromUsage(combineUsages(calls.map(function (call) { return call.return_; })));
                    // TODO: GH#18217
                    return checker.createSignature(/*declaration*/ undefined, /*typeParameters*/ undefined, /*thisParameter*/ undefined, parameters, returnType, /*typePredicate*/ undefined, length, 0 /* None */);
                }
                function addCandidateType(usage, type) {
                    if (type && !(type.flags & 1 /* Any */) && !(type.flags & 131072 /* Never */)) {
                        (usage.candidateTypes || (usage.candidateTypes = [])).push(type);
                    }
                }
                function addCandidateThisType(usage, type) {
                    if (type && !(type.flags & 1 /* Any */) && !(type.flags & 131072 /* Never */)) {
                        (usage.candidateThisTypes || (usage.candidateThisTypes = [])).push(type);
                    }
                }
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "convertFunctionToEs6Class";
            var errorCodes = [ts.Diagnostics.This_constructor_function_may_be_converted_to_a_class_declaration.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, context.sourceFile, context.span.start, context.program.getTypeChecker()); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Convert_function_to_an_ES2015_class, fixId, ts.Diagnostics.Convert_all_constructor_functions_to_classes)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) { return codefix.codeFixAll(context, errorCodes, function (changes, err) { return doChange(changes, err.file, err.start, context.program.getTypeChecker()); }); },
            });
            function doChange(changes, sourceFile, position, checker) {
                var ctorSymbol = checker.getSymbolAtLocation(ts.getTokenAtPosition(sourceFile, position));
                if (!ctorSymbol || !(ctorSymbol.flags & (16 /* Function */ | 3 /* Variable */))) {
                    // Bad input
                    return undefined;
                }
                var ctorDeclaration = ctorSymbol.valueDeclaration;
                var precedingNode;
                var newClassDeclaration;
                switch (ctorDeclaration.kind) {
                    case 244 /* FunctionDeclaration */:
                        precedingNode = ctorDeclaration;
                        changes.delete(sourceFile, ctorDeclaration);
                        newClassDeclaration = createClassFromFunctionDeclaration(ctorDeclaration);
                        break;
                    case 242 /* VariableDeclaration */:
                        precedingNode = ctorDeclaration.parent.parent;
                        newClassDeclaration = createClassFromVariableDeclaration(ctorDeclaration);
                        if (ctorDeclaration.parent.declarations.length === 1) {
                            ts.copyLeadingComments(precedingNode, newClassDeclaration, sourceFile); // TODO: GH#18217
                            changes.delete(sourceFile, precedingNode);
                        }
                        else {
                            changes.delete(sourceFile, ctorDeclaration);
                        }
                        break;
                }
                if (!newClassDeclaration) {
                    return undefined;
                }
                ts.copyLeadingComments(ctorDeclaration, newClassDeclaration, sourceFile);
                // Because the preceding node could be touched, we need to insert nodes before delete nodes.
                changes.insertNodeAfter(sourceFile, precedingNode, newClassDeclaration);
                function createClassElementsFromSymbol(symbol) {
                    var memberElements = [];
                    // all instance members are stored in the "member" array of symbol
                    if (symbol.members) {
                        symbol.members.forEach(function (member) {
                            var memberElement = createClassElement(member, /*modifiers*/ undefined);
                            if (memberElement) {
                                memberElements.push(memberElement);
                            }
                        });
                    }
                    // all static members are stored in the "exports" array of symbol
                    if (symbol.exports) {
                        symbol.exports.forEach(function (member) {
                            var memberElement = createClassElement(member, [ts.createToken(120 /* StaticKeyword */)]);
                            if (memberElement) {
                                memberElements.push(memberElement);
                            }
                        });
                    }
                    return memberElements;
                    function shouldConvertDeclaration(_target, source) {
                        // Right now the only thing we can convert are function expressions - other values shouldn't get
                        // transformed. We can update this once ES public class properties are available.
                        return ts.isFunctionLike(source);
                    }
                    function createClassElement(symbol, modifiers) {
                        // Right now the only thing we can convert are function expressions, which are marked as methods
                        if (!(symbol.flags & 8192 /* Method */)) {
                            return;
                        }
                        var memberDeclaration = symbol.valueDeclaration;
                        var assignmentBinaryExpression = memberDeclaration.parent;
                        if (!shouldConvertDeclaration(memberDeclaration, assignmentBinaryExpression.right)) {
                            return;
                        }
                        // delete the entire statement if this expression is the sole expression to take care of the semicolon at the end
                        var nodeToDelete = assignmentBinaryExpression.parent && assignmentBinaryExpression.parent.kind === 226 /* ExpressionStatement */
                            ? assignmentBinaryExpression.parent : assignmentBinaryExpression;
                        changes.delete(sourceFile, nodeToDelete);
                        if (!assignmentBinaryExpression.right) {
                            return ts.createProperty([], modifiers, symbol.name, /*questionToken*/ undefined, 
                            /*type*/ undefined, /*initializer*/ undefined);
                        }
                        switch (assignmentBinaryExpression.right.kind) {
                            case 201 /* FunctionExpression */: {
                                var functionExpression = assignmentBinaryExpression.right;
                                var fullModifiers = ts.concatenate(modifiers, getModifierKindFromSource(functionExpression, 126 /* AsyncKeyword */));
                                var method = ts.createMethod(/*decorators*/ undefined, fullModifiers, /*asteriskToken*/ undefined, memberDeclaration.name, /*questionToken*/ undefined, 
                                /*typeParameters*/ undefined, functionExpression.parameters, /*type*/ undefined, functionExpression.body);
                                ts.copyLeadingComments(assignmentBinaryExpression, method, sourceFile);
                                return method;
                            }
                            case 202 /* ArrowFunction */: {
                                var arrowFunction = assignmentBinaryExpression.right;
                                var arrowFunctionBody = arrowFunction.body;
                                var bodyBlock = void 0;
                                // case 1: () => { return [1,2,3] }
                                if (arrowFunctionBody.kind === 223 /* Block */) {
                                    bodyBlock = arrowFunctionBody;
                                }
                                // case 2: () => [1,2,3]
                                else {
                                    bodyBlock = ts.createBlock([ts.createReturn(arrowFunctionBody)]);
                                }
                                var fullModifiers = ts.concatenate(modifiers, getModifierKindFromSource(arrowFunction, 126 /* AsyncKeyword */));
                                var method = ts.createMethod(/*decorators*/ undefined, fullModifiers, /*asteriskToken*/ undefined, memberDeclaration.name, /*questionToken*/ undefined, 
                                /*typeParameters*/ undefined, arrowFunction.parameters, /*type*/ undefined, bodyBlock);
                                ts.copyLeadingComments(assignmentBinaryExpression, method, sourceFile);
                                return method;
                            }
                            default: {
                                // Don't try to declare members in JavaScript files
                                if (ts.isSourceFileJS(sourceFile)) {
                                    return;
                                }
                                var prop = ts.createProperty(/*decorators*/ undefined, modifiers, memberDeclaration.name, /*questionToken*/ undefined, 
                                /*type*/ undefined, assignmentBinaryExpression.right);
                                ts.copyLeadingComments(assignmentBinaryExpression.parent, prop, sourceFile);
                                return prop;
                            }
                        }
                    }
                }
                function createClassFromVariableDeclaration(node) {
                    var initializer = node.initializer;
                    if (!initializer || initializer.kind !== 201 /* FunctionExpression */) {
                        return undefined;
                    }
                    if (node.name.kind !== 75 /* Identifier */) {
                        return undefined;
                    }
                    var memberElements = createClassElementsFromSymbol(node.symbol);
                    if (initializer.body) {
                        memberElements.unshift(ts.createConstructor(/*decorators*/ undefined, /*modifiers*/ undefined, initializer.parameters, initializer.body));
                    }
                    var modifiers = getModifierKindFromSource(precedingNode, 89 /* ExportKeyword */);
                    var cls = ts.createClassDeclaration(/*decorators*/ undefined, modifiers, node.name, 
                    /*typeParameters*/ undefined, /*heritageClauses*/ undefined, memberElements);
                    // Don't call copyComments here because we'll already leave them in place
                    return cls;
                }
                function createClassFromFunctionDeclaration(node) {
                    var memberElements = createClassElementsFromSymbol(ctorSymbol);
                    if (node.body) {
                        memberElements.unshift(ts.createConstructor(/*decorators*/ undefined, /*modifiers*/ undefined, node.parameters, node.body));
                    }
                    var modifiers = getModifierKindFromSource(node, 89 /* ExportKeyword */);
                    var cls = ts.createClassDeclaration(/*decorators*/ undefined, modifiers, node.name, 
                    /*typeParameters*/ undefined, /*heritageClauses*/ undefined, memberElements);
                    // Don't call copyComments here because we'll already leave them in place
                    return cls;
                }
            }
            function getModifierKindFromSource(source, kind) {
                return ts.filter(source.modifiers, function (modifier) { return modifier.kind === kind; });
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "convertToAsyncFunction";
            var errorCodes = [ts.Diagnostics.This_may_be_converted_to_an_async_function.code];
            var codeActionSucceeded = true;
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    codeActionSucceeded = true;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return convertToAsyncFunction(t, context.sourceFile, context.span.start, context.program.getTypeChecker(), context); });
                    return codeActionSucceeded ? [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Convert_to_async_function, fixId, ts.Diagnostics.Convert_all_to_async_functions)] : [];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) { return codefix.codeFixAll(context, errorCodes, function (changes, err) { return convertToAsyncFunction(changes, err.file, err.start, context.program.getTypeChecker(), context); }); },
            });
            var SynthBindingNameKind;
            (function (SynthBindingNameKind) {
                SynthBindingNameKind[SynthBindingNameKind["Identifier"] = 0] = "Identifier";
                SynthBindingNameKind[SynthBindingNameKind["BindingPattern"] = 1] = "BindingPattern";
            })(SynthBindingNameKind || (SynthBindingNameKind = {}));
            function convertToAsyncFunction(changes, sourceFile, position, checker, context) {
                // get the function declaration - returns a promise
                var tokenAtPosition = ts.getTokenAtPosition(sourceFile, position);
                var functionToConvert;
                // if the parent of a FunctionLikeDeclaration is a variable declaration, the convertToAsync diagnostic will be reported on the variable name
                if (ts.isIdentifier(tokenAtPosition) && ts.isVariableDeclaration(tokenAtPosition.parent) &&
                    tokenAtPosition.parent.initializer && ts.isFunctionLikeDeclaration(tokenAtPosition.parent.initializer)) {
                    functionToConvert = tokenAtPosition.parent.initializer;
                }
                else {
                    functionToConvert = ts.tryCast(ts.getContainingFunction(ts.getTokenAtPosition(sourceFile, position)), ts.isFunctionLikeDeclaration);
                }
                if (!functionToConvert) {
                    return;
                }
                var synthNamesMap = ts.createMap();
                var originalTypeMap = ts.createMap();
                var allVarNames = [];
                var isInJavascript = ts.isInJSFile(functionToConvert);
                var setOfExpressionsToReturn = getAllPromiseExpressionsToReturn(functionToConvert, checker);
                var functionToConvertRenamed = renameCollidingVarNames(functionToConvert, checker, synthNamesMap, context, setOfExpressionsToReturn, originalTypeMap, allVarNames);
                var constIdentifiers = getConstIdentifiers(synthNamesMap);
                var returnStatements = functionToConvertRenamed.body && ts.isBlock(functionToConvertRenamed.body) ? getReturnStatementsWithPromiseHandlers(functionToConvertRenamed.body) : ts.emptyArray;
                var transformer = { checker: checker, synthNamesMap: synthNamesMap, allVarNames: allVarNames, setOfExpressionsToReturn: setOfExpressionsToReturn, constIdentifiers: constIdentifiers, originalTypeMap: originalTypeMap, isInJSFile: isInJavascript };
                if (!returnStatements.length) {
                    return;
                }
                // add the async keyword
                changes.insertLastModifierBefore(sourceFile, 126 /* AsyncKeyword */, functionToConvert);
                function startTransformation(node, nodeToReplace) {
                    var newNodes = transformExpression(node, transformer, node);
                    changes.replaceNodeWithNodes(sourceFile, nodeToReplace, newNodes);
                }
                var _loop_13 = function (statement) {
                    ts.forEachChild(statement, function visit(node) {
                        if (ts.isCallExpression(node)) {
                            startTransformation(node, statement);
                        }
                        else if (!ts.isFunctionLike(node)) {
                            ts.forEachChild(node, visit);
                        }
                    });
                };
                for (var _i = 0, returnStatements_1 = returnStatements; _i < returnStatements_1.length; _i++) {
                    var statement = returnStatements_1[_i];
                    _loop_13(statement);
                }
            }
            function getReturnStatementsWithPromiseHandlers(body) {
                var res = [];
                ts.forEachReturnStatement(body, function (ret) {
                    if (ts.isReturnStatementWithFixablePromiseHandler(ret))
                        res.push(ret);
                });
                return res;
            }
            // Returns the identifiers that are never reassigned in the refactor
            function getConstIdentifiers(synthNamesMap) {
                var constIdentifiers = [];
                synthNamesMap.forEach(function (val) {
                    if (val.numberOfAssignmentsOriginal === 0) {
                        constIdentifiers.push(val.identifier);
                    }
                });
                return constIdentifiers;
            }
            /*
                Finds all of the expressions of promise type that should not be saved in a variable during the refactor
            */
            function getAllPromiseExpressionsToReturn(func, checker) {
                if (!func.body) {
                    return ts.createMap();
                }
                var setOfExpressionsToReturn = ts.createMap();
                ts.forEachChild(func.body, function visit(node) {
                    if (isPromiseReturningExpression(node, checker, "then")) {
                        setOfExpressionsToReturn.set(ts.getNodeId(node).toString(), true);
                        ts.forEach(node.arguments, visit);
                    }
                    else if (isPromiseReturningExpression(node, checker, "catch")) {
                        setOfExpressionsToReturn.set(ts.getNodeId(node).toString(), true);
                        // if .catch() is the last call in the chain, move leftward in the chain until we hit something else that should be returned
                        ts.forEachChild(node, visit);
                    }
                    else if (isPromiseReturningExpression(node, checker)) {
                        setOfExpressionsToReturn.set(ts.getNodeId(node).toString(), true);
                        // don't recurse here, since we won't refactor any children or arguments of the expression
                    }
                    else {
                        ts.forEachChild(node, visit);
                    }
                });
                return setOfExpressionsToReturn;
            }
            /*
                Returns true if node is a promise returning expression
                If name is not undefined, node is a promise returning call of name
            */
            function isPromiseReturningExpression(node, checker, name) {
                var isNodeExpression = name ? ts.isCallExpression(node) : ts.isExpression(node);
                var isExpressionOfName = isNodeExpression && (!name || ts.hasPropertyAccessExpressionWithName(node, name));
                var nodeType = isExpressionOfName && checker.getTypeAtLocation(node);
                return !!(nodeType && checker.getPromisedTypeOfPromise(nodeType));
            }
            function declaredInFile(symbol, sourceFile) {
                return symbol.valueDeclaration && symbol.valueDeclaration.getSourceFile() === sourceFile;
            }
            /*
                Renaming of identifiers may be neccesary as the refactor changes scopes -
                This function collects all existing identifier names and names of identifiers that will be created in the refactor.
                It then checks for any collisions and renames them through getSynthesizedDeepClone
            */
            function renameCollidingVarNames(nodeToRename, checker, synthNamesMap, context, setOfAllExpressionsToReturn, originalType, allVarNames) {
                var identsToRenameMap = ts.createMap(); // key is the symbol id
                var collidingSymbolMap = ts.createMap();
                ts.forEachChild(nodeToRename, function visit(node) {
                    if (!ts.isIdentifier(node)) {
                        ts.forEachChild(node, visit);
                        return;
                    }
                    var symbol = checker.getSymbolAtLocation(node);
                    var isDefinedInFile = symbol && declaredInFile(symbol, context.sourceFile);
                    if (symbol && isDefinedInFile) {
                        var type = checker.getTypeAtLocation(node);
                        var lastCallSignature = getLastCallSignature(type, checker);
                        var symbolIdString = ts.getSymbolId(symbol).toString();
                        // if the identifier refers to a function we want to add the new synthesized variable for the declaration (ex. blob in let blob = res(arg))
                        // Note - the choice of the last call signature is arbitrary
                        if (lastCallSignature && !ts.isFunctionLikeDeclaration(node.parent) && !synthNamesMap.has(symbolIdString)) {
                            var firstParameter = ts.firstOrUndefined(lastCallSignature.parameters);
                            var ident = firstParameter && ts.isParameter(firstParameter.valueDeclaration) && ts.tryCast(firstParameter.valueDeclaration.name, ts.isIdentifier) || ts.createOptimisticUniqueName("result");
                            var synthName = getNewNameIfConflict(ident, collidingSymbolMap);
                            synthNamesMap.set(symbolIdString, synthName);
                            allVarNames.push({ identifier: synthName.identifier, symbol: symbol });
                            addNameToFrequencyMap(collidingSymbolMap, ident.text, symbol);
                        }
                        // we only care about identifiers that are parameters, declarations, or binding elements (don't care about other uses)
                        else if (node.parent && (ts.isParameter(node.parent) || ts.isVariableDeclaration(node.parent) || ts.isBindingElement(node.parent))) {
                            var originalName = node.text;
                            var collidingSymbols = collidingSymbolMap.get(originalName);
                            // if the identifier name conflicts with a different identifier that we've already seen
                            if (collidingSymbols && collidingSymbols.some(function (prevSymbol) { return prevSymbol !== symbol; })) {
                                var newName = getNewNameIfConflict(node, collidingSymbolMap);
                                identsToRenameMap.set(symbolIdString, newName.identifier);
                                synthNamesMap.set(symbolIdString, newName);
                                allVarNames.push({ identifier: newName.identifier, symbol: symbol });
                                addNameToFrequencyMap(collidingSymbolMap, originalName, symbol);
                            }
                            else {
                                var identifier = ts.getSynthesizedDeepClone(node);
                                identsToRenameMap.set(symbolIdString, identifier);
                                synthNamesMap.set(symbolIdString, createSynthIdentifier(identifier, [], allVarNames.filter(function (elem) { return elem.identifier.text === node.text; }).length /*, numberOfAssignmentsSynthesized: 0*/));
                                if ((ts.isParameter(node.parent) && isExpressionOrCallOnTypePromise(node.parent.parent)) || ts.isVariableDeclaration(node.parent)) {
                                    allVarNames.push({ identifier: identifier, symbol: symbol });
                                    addNameToFrequencyMap(collidingSymbolMap, originalName, symbol);
                                }
                            }
                        }
                    }
                });
                return ts.getSynthesizedDeepCloneWithRenames(nodeToRename, /*includeTrivia*/ true, identsToRenameMap, checker, deepCloneCallback);
                function isExpressionOrCallOnTypePromise(child) {
                    var node = child.parent;
                    if (ts.isCallExpression(node) || ts.isIdentifier(node) && !setOfAllExpressionsToReturn.get(ts.getNodeId(node).toString())) {
                        var nodeType = checker.getTypeAtLocation(node);
                        var isPromise = nodeType && checker.getPromisedTypeOfPromise(nodeType);
                        return !!isPromise;
                    }
                    return false;
                }
                function deepCloneCallback(node, clone) {
                    if (ts.isIdentifier(node)) {
                        var symbol = checker.getSymbolAtLocation(node);
                        var symboldIdString = symbol && ts.getSymbolId(symbol).toString();
                        var renameInfo = symbol && synthNamesMap.get(symboldIdString);
                        if (renameInfo) {
                            var type = checker.getTypeAtLocation(node);
                            originalType.set(ts.getNodeId(clone).toString(), type);
                        }
                    }
                    var val = setOfAllExpressionsToReturn.get(ts.getNodeId(node).toString());
                    if (val !== undefined) {
                        setOfAllExpressionsToReturn.delete(ts.getNodeId(node).toString());
                        setOfAllExpressionsToReturn.set(ts.getNodeId(clone).toString(), val);
                    }
                }
            }
            function addNameToFrequencyMap(renamedVarNameFrequencyMap, originalName, symbol) {
                if (renamedVarNameFrequencyMap.has(originalName)) {
                    renamedVarNameFrequencyMap.get(originalName).push(symbol);
                }
                else {
                    renamedVarNameFrequencyMap.set(originalName, [symbol]);
                }
            }
            function getNewNameIfConflict(name, originalNames) {
                var numVarsSameName = (originalNames.get(name.text) || ts.emptyArray).length;
                var numberOfAssignmentsOriginal = 0;
                var identifier = numVarsSameName === 0 ? name : ts.createIdentifier(name.text + "_" + numVarsSameName);
                return createSynthIdentifier(identifier, [], numberOfAssignmentsOriginal);
            }
            // dispatch function to recursively build the refactoring
            // should be kept up to date with isFixablePromiseHandler in suggestionDiagnostics.ts
            function transformExpression(node, transformer, outermostParent, prevArgName) {
                if (!node) {
                    return ts.emptyArray;
                }
                var originalType = ts.isIdentifier(node) && transformer.originalTypeMap.get(ts.getNodeId(node).toString());
                var nodeType = originalType || transformer.checker.getTypeAtLocation(node);
                if (ts.isCallExpression(node) && ts.hasPropertyAccessExpressionWithName(node, "then") && nodeType && !!transformer.checker.getPromisedTypeOfPromise(nodeType)) {
                    return transformThen(node, transformer, outermostParent, prevArgName);
                }
                else if (ts.isCallExpression(node) && ts.hasPropertyAccessExpressionWithName(node, "catch") && nodeType && !!transformer.checker.getPromisedTypeOfPromise(nodeType)) {
                    return transformCatch(node, transformer, prevArgName);
                }
                else if (ts.isPropertyAccessExpression(node)) {
                    return transformExpression(node.expression, transformer, outermostParent, prevArgName);
                }
                else if (nodeType && transformer.checker.getPromisedTypeOfPromise(nodeType)) {
                    return transformPromiseCall(node, transformer, prevArgName);
                }
                codeActionSucceeded = false;
                return ts.emptyArray;
            }
            function transformCatch(node, transformer, prevArgName) {
                var func = node.arguments[0];
                var argName = getArgBindingName(func, transformer);
                var shouldReturn = transformer.setOfExpressionsToReturn.get(ts.getNodeId(node).toString());
                var possibleNameForVarDecl;
                /*
                    If there is another call in the chain after the .catch() we are transforming, we will need to save the result of both paths (try block and catch block)
                    To do this, we will need to synthesize a variable that we were not aware of while we were adding identifiers to the synthNamesMap
                    We will use the prevArgName and then update the synthNamesMap with a new variable name for the next transformation step
                */
                if (prevArgName && !shouldReturn) {
                    if (isSynthIdentifier(prevArgName)) {
                        possibleNameForVarDecl = prevArgName;
                        transformer.synthNamesMap.forEach(function (val, key) {
                            if (val.identifier.text === prevArgName.identifier.text) {
                                var newSynthName = createUniqueSynthName(prevArgName);
                                transformer.synthNamesMap.set(key, newSynthName);
                            }
                        });
                    }
                    else {
                        possibleNameForVarDecl = createSynthIdentifier(ts.createOptimisticUniqueName("result"), prevArgName.types);
                    }
                    possibleNameForVarDecl.numberOfAssignmentsOriginal = 2; // Try block and catch block
                    // update the constIdentifiers list
                    if (transformer.constIdentifiers.some(function (elem) { return elem.text === possibleNameForVarDecl.identifier.text; })) {
                        transformer.constIdentifiers.push(createUniqueSynthName(possibleNameForVarDecl).identifier);
                    }
                }
                var tryBlock = ts.createBlock(transformExpression(node.expression, transformer, node, possibleNameForVarDecl));
                var transformationBody = getTransformationBody(func, possibleNameForVarDecl, argName, node, transformer);
                var catchArg = argName ? isSynthIdentifier(argName) ? argName.identifier.text : argName.bindingPattern : "e";
                var catchVariableDeclaration = ts.createVariableDeclaration(catchArg);
                var catchClause = ts.createCatchClause(catchVariableDeclaration, ts.createBlock(transformationBody));
                /*
                    In order to avoid an implicit any, we will synthesize a type for the declaration using the unions of the types of both paths (try block and catch block)
                */
                var varDeclList;
                var varDeclIdentifier;
                if (possibleNameForVarDecl && !shouldReturn) {
                    varDeclIdentifier = ts.getSynthesizedDeepClone(possibleNameForVarDecl.identifier);
                    var typeArray = possibleNameForVarDecl.types;
                    var unionType = transformer.checker.getUnionType(typeArray, 2 /* Subtype */);
                    var unionTypeNode = transformer.isInJSFile ? undefined : transformer.checker.typeToTypeNode(unionType);
                    var varDecl = [ts.createVariableDeclaration(varDeclIdentifier, unionTypeNode)];
                    varDeclList = ts.createVariableStatement(/*modifiers*/ undefined, ts.createVariableDeclarationList(varDecl, 1 /* Let */));
                }
                var tryStatement = ts.createTry(tryBlock, catchClause, /*finallyBlock*/ undefined);
                var destructuredResult = prevArgName && varDeclIdentifier && isSynthBindingPattern(prevArgName)
                    && ts.createVariableStatement(/* modifiers */ undefined, ts.createVariableDeclarationList([ts.createVariableDeclaration(ts.getSynthesizedDeepCloneWithRenames(prevArgName.bindingPattern), /* type */ undefined, varDeclIdentifier)], 2 /* Const */));
                return ts.compact([varDeclList, tryStatement, destructuredResult]);
            }
            function getIdentifierTextsFromBindingName(bindingName) {
                if (ts.isIdentifier(bindingName))
                    return [bindingName.text];
                return ts.flatMap(bindingName.elements, function (element) {
                    if (ts.isOmittedExpression(element))
                        return [];
                    return getIdentifierTextsFromBindingName(element.name);
                });
            }
            function createUniqueSynthName(prevArgName) {
                var renamedPrevArg = ts.createOptimisticUniqueName(prevArgName.identifier.text);
                return createSynthIdentifier(renamedPrevArg);
            }
            function transformThen(node, transformer, outermostParent, prevArgName) {
                var _a = node.arguments, res = _a[0], rej = _a[1];
                if (!res) {
                    return transformExpression(node.expression, transformer, outermostParent);
                }
                var argNameRes = getArgBindingName(res, transformer);
                var transformationBody = getTransformationBody(res, prevArgName, argNameRes, node, transformer);
                if (rej) {
                    var argNameRej = getArgBindingName(rej, transformer);
                    var tryBlock = ts.createBlock(transformExpression(node.expression, transformer, node, argNameRes).concat(transformationBody));
                    var transformationBody2 = getTransformationBody(rej, prevArgName, argNameRej, node, transformer);
                    var catchArg = argNameRej ? isSynthIdentifier(argNameRej) ? argNameRej.identifier.text : argNameRej.bindingPattern : "e";
                    var catchVariableDeclaration = ts.createVariableDeclaration(catchArg);
                    var catchClause = ts.createCatchClause(catchVariableDeclaration, ts.createBlock(transformationBody2));
                    return [ts.createTry(tryBlock, catchClause, /* finallyBlock */ undefined)];
                }
                return transformExpression(node.expression, transformer, node, argNameRes).concat(transformationBody);
            }
            function getFlagOfBindingName(bindingName, constIdentifiers) {
                var identifiers = getIdentifierTextsFromBindingName(getNode(bindingName));
                var inArr = constIdentifiers.some(function (elem) { return ts.contains(identifiers, elem.text); });
                return inArr ? 2 /* Const */ : 1 /* Let */;
            }
            function transformPromiseCall(node, transformer, prevArgName) {
                var shouldReturn = transformer.setOfExpressionsToReturn.get(ts.getNodeId(node).toString());
                // the identifier is empty when the handler (.then()) ignores the argument - In this situation we do not need to save the result of the promise returning call
                var originalNodeParent = node.original ? node.original.parent : node.parent;
                if (prevArgName && !shouldReturn && (!originalNodeParent || ts.isPropertyAccessExpression(originalNodeParent))) {
                    return createTransformedStatement(prevArgName, ts.createAwait(node), transformer);
                }
                else if (!prevArgName && !shouldReturn && (!originalNodeParent || ts.isPropertyAccessExpression(originalNodeParent))) {
                    return [ts.createStatement(ts.createAwait(node))];
                }
                return [ts.createReturn(ts.getSynthesizedDeepClone(node))];
            }
            function createTransformedStatement(prevArgName, rightHandSide, transformer) {
                if (!prevArgName || isEmpty(prevArgName)) {
                    // if there's no argName to assign to, there still might be side effects
                    return [ts.createStatement(rightHandSide)];
                }
                if (isSynthIdentifier(prevArgName) && prevArgName.types.length < prevArgName.numberOfAssignmentsOriginal) {
                    // if the variable has already been declared, we don't need "let" or "const"
                    return [ts.createStatement(ts.createAssignment(ts.getSynthesizedDeepClone(prevArgName.identifier), rightHandSide))];
                }
                return [ts.createVariableStatement(/*modifiers*/ undefined, (ts.createVariableDeclarationList([ts.createVariableDeclaration(ts.getSynthesizedDeepClone(getNode(prevArgName)), /*type*/ undefined, rightHandSide)], getFlagOfBindingName(prevArgName, transformer.constIdentifiers))))];
            }
            // should be kept up to date with isFixablePromiseArgument in suggestionDiagnostics.ts
            function getTransformationBody(func, prevArgName, argName, parent, transformer) {
                var shouldReturn = transformer.setOfExpressionsToReturn.get(ts.getNodeId(parent).toString());
                switch (func.kind) {
                    case 100 /* NullKeyword */:
                        // do not produce a transformed statement for a null argument
                        break;
                    case 75 /* Identifier */: // identifier includes undefined
                        if (!argName) {
                            // undefined was argument passed to promise handler
                            break;
                        }
                        var synthCall = ts.createCall(ts.getSynthesizedDeepClone(func), /*typeArguments*/ undefined, isSynthIdentifier(argName) ? [argName.identifier] : []);
                        if (shouldReturn) {
                            return [ts.createReturn(synthCall)];
                        }
                        var type = transformer.originalTypeMap.get(ts.getNodeId(func).toString()) || transformer.checker.getTypeAtLocation(func);
                        var callSignatures = transformer.checker.getSignaturesOfType(type, 0 /* Call */);
                        if (!callSignatures.length) {
                            // if identifier in handler has no call signatures, it's invalid
                            codeActionSucceeded = false;
                            break;
                        }
                        var returnType = callSignatures[0].getReturnType();
                        var varDeclOrAssignment = createTransformedStatement(prevArgName, ts.createAwait(synthCall), transformer);
                        if (prevArgName) {
                            prevArgName.types.push(returnType);
                        }
                        return varDeclOrAssignment;
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */: {
                        var funcBody = func.body;
                        // Arrow functions with block bodies { } will enter this control flow
                        if (ts.isBlock(funcBody)) {
                            var refactoredStmts = [];
                            var seenReturnStatement = false;
                            for (var _i = 0, _a = funcBody.statements; _i < _a.length; _i++) {
                                var statement = _a[_i];
                                if (ts.isReturnStatement(statement)) {
                                    seenReturnStatement = true;
                                }
                                if (ts.isReturnStatementWithFixablePromiseHandler(statement)) {
                                    refactoredStmts = refactoredStmts.concat(getInnerTransformationBody(transformer, [statement], prevArgName));
                                }
                                else {
                                    refactoredStmts.push(statement);
                                }
                            }
                            return shouldReturn ? refactoredStmts.map(function (s) { return ts.getSynthesizedDeepClone(s); }) :
                                removeReturns(refactoredStmts, prevArgName, transformer, seenReturnStatement);
                        }
                        else {
                            var innerRetStmts = ts.isFixablePromiseHandler(funcBody) ? [ts.createReturn(funcBody)] : ts.emptyArray;
                            var innerCbBody = getInnerTransformationBody(transformer, innerRetStmts, prevArgName);
                            if (innerCbBody.length > 0) {
                                return innerCbBody;
                            }
                            var type_1 = transformer.checker.getTypeAtLocation(func);
                            var returnType_1 = getLastCallSignature(type_1, transformer.checker).getReturnType();
                            var rightHandSide = ts.getSynthesizedDeepClone(funcBody);
                            var possiblyAwaitedRightHandSide = !!transformer.checker.getPromisedTypeOfPromise(returnType_1) ? ts.createAwait(rightHandSide) : rightHandSide;
                            if (!shouldReturn) {
                                var transformedStatement = createTransformedStatement(prevArgName, possiblyAwaitedRightHandSide, transformer);
                                if (prevArgName) {
                                    prevArgName.types.push(returnType_1);
                                }
                                return transformedStatement;
                            }
                            else {
                                return [ts.createReturn(possiblyAwaitedRightHandSide)];
                            }
                        }
                    }
                    default:
                        // If no cases apply, we've found a transformation body we don't know how to handle, so the refactoring should no-op to avoid deleting code.
                        codeActionSucceeded = false;
                        break;
                }
                return ts.emptyArray;
            }
            function getLastCallSignature(type, checker) {
                var callSignatures = checker.getSignaturesOfType(type, 0 /* Call */);
                return ts.lastOrUndefined(callSignatures);
            }
            function removeReturns(stmts, prevArgName, transformer, seenReturnStatement) {
                var ret = [];
                for (var _i = 0, stmts_1 = stmts; _i < stmts_1.length; _i++) {
                    var stmt = stmts_1[_i];
                    if (ts.isReturnStatement(stmt)) {
                        if (stmt.expression) {
                            var possiblyAwaitedExpression = isPromiseReturningExpression(stmt.expression, transformer.checker) ? ts.createAwait(stmt.expression) : stmt.expression;
                            if (prevArgName === undefined) {
                                ret.push(ts.createExpressionStatement(possiblyAwaitedExpression));
                            }
                            else {
                                ret.push(ts.createVariableStatement(/*modifiers*/ undefined, (ts.createVariableDeclarationList([ts.createVariableDeclaration(getNode(prevArgName), /*type*/ undefined, possiblyAwaitedExpression)], getFlagOfBindingName(prevArgName, transformer.constIdentifiers)))));
                            }
                        }
                    }
                    else {
                        ret.push(ts.getSynthesizedDeepClone(stmt));
                    }
                }
                // if block has no return statement, need to define prevArgName as undefined to prevent undeclared variables
                if (!seenReturnStatement && prevArgName !== undefined) {
                    ret.push(ts.createVariableStatement(/*modifiers*/ undefined, (ts.createVariableDeclarationList([ts.createVariableDeclaration(getNode(prevArgName), /*type*/ undefined, ts.createIdentifier("undefined"))], getFlagOfBindingName(prevArgName, transformer.constIdentifiers)))));
                }
                return ret;
            }
            function getInnerTransformationBody(transformer, innerRetStmts, prevArgName) {
                var innerCbBody = [];
                for (var _i = 0, innerRetStmts_1 = innerRetStmts; _i < innerRetStmts_1.length; _i++) {
                    var stmt = innerRetStmts_1[_i];
                    ts.forEachChild(stmt, function visit(node) {
                        if (ts.isCallExpression(node)) {
                            var temp = transformExpression(node, transformer, node, prevArgName);
                            innerCbBody = innerCbBody.concat(temp);
                            if (innerCbBody.length > 0) {
                                return;
                            }
                        }
                        else if (!ts.isFunctionLike(node)) {
                            ts.forEachChild(node, visit);
                        }
                    });
                }
                return innerCbBody;
            }
            function getArgBindingName(funcNode, transformer) {
                var numberOfAssignmentsOriginal = 0;
                var types = [];
                var name;
                if (ts.isFunctionLikeDeclaration(funcNode)) {
                    if (funcNode.parameters.length > 0) {
                        var param = funcNode.parameters[0].name;
                        name = getMappedBindingNameOrDefault(param);
                    }
                }
                else if (ts.isIdentifier(funcNode)) {
                    name = getMapEntryOrDefault(funcNode);
                }
                // return undefined argName when arg is null or undefined
                // eslint-disable-next-line no-in-operator
                if (!name || "identifier" in name && name.identifier.text === "undefined") {
                    return undefined;
                }
                return name;
                function getMappedBindingNameOrDefault(bindingName) {
                    if (ts.isIdentifier(bindingName))
                        return getMapEntryOrDefault(bindingName);
                    var elements = ts.flatMap(bindingName.elements, function (element) {
                        if (ts.isOmittedExpression(element))
                            return [];
                        return [getMappedBindingNameOrDefault(element.name)];
                    });
                    return createSynthBindingPattern(bindingName, elements);
                }
                function getMapEntryOrDefault(identifier) {
                    var originalNode = getOriginalNode(identifier);
                    var symbol = getSymbol(originalNode);
                    if (!symbol) {
                        return createSynthIdentifier(identifier, types, numberOfAssignmentsOriginal);
                    }
                    var mapEntry = transformer.synthNamesMap.get(ts.getSymbolId(symbol).toString());
                    return mapEntry || createSynthIdentifier(identifier, types, numberOfAssignmentsOriginal);
                }
                function getSymbol(node) {
                    return node.symbol ? node.symbol : transformer.checker.getSymbolAtLocation(node);
                }
                function getOriginalNode(node) {
                    return node.original ? node.original : node;
                }
            }
            function isEmpty(bindingName) {
                if (!bindingName) {
                    return true;
                }
                if (isSynthIdentifier(bindingName)) {
                    return !bindingName.identifier.text;
                }
                return ts.every(bindingName.elements, isEmpty);
            }
            function getNode(bindingName) {
                return isSynthIdentifier(bindingName) ? bindingName.identifier : bindingName.bindingPattern;
            }
            function createSynthIdentifier(identifier, types, numberOfAssignmentsOriginal) {
                if (types === void 0) {
                    types = [];
                }
                if (numberOfAssignmentsOriginal === void 0) {
                    numberOfAssignmentsOriginal = 0;
                }
                return { kind: 0 /* Identifier */, identifier: identifier, types: types, numberOfAssignmentsOriginal: numberOfAssignmentsOriginal };
            }
            function createSynthBindingPattern(bindingPattern, elements, types) {
                if (elements === void 0) {
                    elements = ts.emptyArray;
                }
                if (types === void 0) {
                    types = [];
                }
                return { kind: 1 /* BindingPattern */, bindingPattern: bindingPattern, elements: elements, types: types };
            }
            function isSynthIdentifier(bindingName) {
                return bindingName.kind === 0 /* Identifier */;
            }
            function isSynthBindingPattern(bindingName) {
                return bindingName.kind === 1 /* BindingPattern */;
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            codefix.registerCodeFix({
                errorCodes: [ts.Diagnostics.File_is_a_CommonJS_module_it_may_be_converted_to_an_ES6_module.code],
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, program = context.program, preferences = context.preferences;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (changes) {
                        var moduleExportsChangedToDefault = convertFileToEs6Module(sourceFile, program.getTypeChecker(), changes, program.getCompilerOptions().target, ts.getQuotePreference(sourceFile, preferences));
                        if (moduleExportsChangedToDefault) {
                            for (var _i = 0, _a = program.getSourceFiles(); _i < _a.length; _i++) {
                                var importingFile = _a[_i];
                                fixImportOfModuleExports(importingFile, sourceFile, changes, ts.getQuotePreference(importingFile, preferences));
                            }
                        }
                    });
                    // No support for fix-all since this applies to the whole file at once anyway.
                    return [codefix.createCodeFixActionWithoutFixAll("convertToEs6Module", changes, ts.Diagnostics.Convert_to_ES6_module)];
                },
            });
            function fixImportOfModuleExports(importingFile, exportingFile, changes, quotePreference) {
                for (var _i = 0, _a = importingFile.imports; _i < _a.length; _i++) {
                    var moduleSpecifier = _a[_i];
                    var imported = ts.getResolvedModule(importingFile, moduleSpecifier.text);
                    if (!imported || imported.resolvedFileName !== exportingFile.fileName) {
                        continue;
                    }
                    var importNode = ts.importFromModuleSpecifier(moduleSpecifier);
                    switch (importNode.kind) {
                        case 253 /* ImportEqualsDeclaration */:
                            changes.replaceNode(importingFile, importNode, ts.makeImport(importNode.name, /*namedImports*/ undefined, moduleSpecifier, quotePreference));
                            break;
                        case 196 /* CallExpression */:
                            if (ts.isRequireCall(importNode, /*checkArgumentIsStringLiteralLike*/ false)) {
                                changes.replaceNode(importingFile, importNode, ts.createPropertyAccess(ts.getSynthesizedDeepClone(importNode), "default"));
                            }
                            break;
                    }
                }
            }
            /** @returns Whether we converted a `module.exports =` to a default export. */
            function convertFileToEs6Module(sourceFile, checker, changes, target, quotePreference) {
                var identifiers = { original: collectFreeIdentifiers(sourceFile), additional: ts.createMap() };
                var exports = collectExportRenames(sourceFile, checker, identifiers);
                convertExportsAccesses(sourceFile, exports, changes);
                var moduleExportsChangedToDefault = false;
                for (var _i = 0, _a = sourceFile.statements; _i < _a.length; _i++) {
                    var statement = _a[_i];
                    var moduleExportsChanged = convertStatement(sourceFile, statement, checker, changes, identifiers, target, exports, quotePreference);
                    moduleExportsChangedToDefault = moduleExportsChangedToDefault || moduleExportsChanged;
                }
                return moduleExportsChangedToDefault;
            }
            function collectExportRenames(sourceFile, checker, identifiers) {
                var res = ts.createMap();
                forEachExportReference(sourceFile, function (node) {
                    var _a = node.name, text = _a.text, originalKeywordKind = _a.originalKeywordKind;
                    if (!res.has(text) && (originalKeywordKind !== undefined && ts.isNonContextualKeyword(originalKeywordKind)
                        || checker.resolveName(text, node, 111551 /* Value */, /*excludeGlobals*/ true))) {
                        // Unconditionally add an underscore in case `text` is a keyword.
                        res.set(text, makeUniqueName("_" + text, identifiers));
                    }
                });
                return res;
            }
            function convertExportsAccesses(sourceFile, exports, changes) {
                forEachExportReference(sourceFile, function (node, isAssignmentLhs) {
                    if (isAssignmentLhs) {
                        return;
                    }
                    var text = node.name.text;
                    changes.replaceNode(sourceFile, node, ts.createIdentifier(exports.get(text) || text));
                });
            }
            function forEachExportReference(sourceFile, cb) {
                sourceFile.forEachChild(function recur(node) {
                    if (ts.isPropertyAccessExpression(node) && ts.isExportsOrModuleExportsOrAlias(sourceFile, node.expression) && ts.isIdentifier(node.name)) {
                        var parent = node.parent;
                        cb(node, ts.isBinaryExpression(parent) && parent.left === node && parent.operatorToken.kind === 62 /* EqualsToken */);
                    }
                    node.forEachChild(recur);
                });
            }
            function convertStatement(sourceFile, statement, checker, changes, identifiers, target, exports, quotePreference) {
                switch (statement.kind) {
                    case 225 /* VariableStatement */:
                        convertVariableStatement(sourceFile, statement, changes, checker, identifiers, target, quotePreference);
                        return false;
                    case 226 /* ExpressionStatement */: {
                        var expression = statement.expression;
                        switch (expression.kind) {
                            case 196 /* CallExpression */: {
                                if (ts.isRequireCall(expression, /*checkArgumentIsStringLiteralLike*/ true)) {
                                    // For side-effecting require() call, just make a side-effecting import.
                                    changes.replaceNode(sourceFile, statement, ts.makeImport(/*name*/ undefined, /*namedImports*/ undefined, expression.arguments[0], quotePreference));
                                }
                                return false;
                            }
                            case 209 /* BinaryExpression */: {
                                var operatorToken = expression.operatorToken;
                                return operatorToken.kind === 62 /* EqualsToken */ && convertAssignment(sourceFile, checker, expression, changes, exports);
                            }
                        }
                    }
                    // falls through
                    default:
                        return false;
                }
            }
            function convertVariableStatement(sourceFile, statement, changes, checker, identifiers, target, quotePreference) {
                var declarationList = statement.declarationList;
                var foundImport = false;
                var newNodes = ts.flatMap(declarationList.declarations, function (decl) {
                    var name = decl.name, initializer = decl.initializer;
                    if (initializer) {
                        if (ts.isExportsOrModuleExportsOrAlias(sourceFile, initializer)) {
                            // `const alias = module.exports;` can be removed.
                            foundImport = true;
                            return [];
                        }
                        else if (ts.isRequireCall(initializer, /*checkArgumentIsStringLiteralLike*/ true)) {
                            foundImport = true;
                            return convertSingleImport(sourceFile, name, initializer.arguments[0], changes, checker, identifiers, target, quotePreference);
                        }
                        else if (ts.isPropertyAccessExpression(initializer) && ts.isRequireCall(initializer.expression, /*checkArgumentIsStringLiteralLike*/ true)) {
                            foundImport = true;
                            return convertPropertyAccessImport(name, initializer.name.text, initializer.expression.arguments[0], identifiers, quotePreference);
                        }
                    }
                    // Move it out to its own variable statement. (This will not be used if `!foundImport`)
                    return ts.createVariableStatement(/*modifiers*/ undefined, ts.createVariableDeclarationList([decl], declarationList.flags));
                });
                if (foundImport) {
                    // useNonAdjustedEndPosition to ensure we don't eat the newline after the statement.
                    changes.replaceNodeWithNodes(sourceFile, statement, newNodes);
                }
            }
            /** Converts `const name = require("moduleSpecifier").propertyName` */
            function convertPropertyAccessImport(name, propertyName, moduleSpecifier, identifiers, quotePreference) {
                switch (name.kind) {
                    case 189 /* ObjectBindingPattern */:
                    case 190 /* ArrayBindingPattern */: {
                        // `const [a, b] = require("c").d` --> `import { d } from "c"; const [a, b] = d;`
                        var tmp = makeUniqueName(propertyName, identifiers);
                        return [
                            makeSingleImport(tmp, propertyName, moduleSpecifier, quotePreference),
                            makeConst(/*modifiers*/ undefined, name, ts.createIdentifier(tmp)),
                        ];
                    }
                    case 75 /* Identifier */:
                        // `const a = require("b").c` --> `import { c as a } from "./b";
                        return [makeSingleImport(name.text, propertyName, moduleSpecifier, quotePreference)];
                    default:
                        return ts.Debug.assertNever(name, "Convert to ES6 module got invalid syntax form " + name.kind);
                }
            }
            function convertAssignment(sourceFile, checker, assignment, changes, exports) {
                var left = assignment.left, right = assignment.right;
                if (!ts.isPropertyAccessExpression(left)) {
                    return false;
                }
                if (ts.isExportsOrModuleExportsOrAlias(sourceFile, left)) {
                    if (ts.isExportsOrModuleExportsOrAlias(sourceFile, right)) {
                        // `const alias = module.exports;` or `module.exports = alias;` can be removed.
                        changes.delete(sourceFile, assignment.parent);
                    }
                    else {
                        var replacement = ts.isObjectLiteralExpression(right) ? tryChangeModuleExportsObject(right)
                            : ts.isRequireCall(right, /*checkArgumentIsStringLiteralLike*/ true) ? convertReExportAll(right.arguments[0], checker)
                                : undefined;
                        if (replacement) {
                            changes.replaceNodeWithNodes(sourceFile, assignment.parent, replacement[0]);
                            return replacement[1];
                        }
                        else {
                            changes.replaceRangeWithText(sourceFile, ts.createRange(left.getStart(sourceFile), right.pos), "export default");
                            return true;
                        }
                    }
                }
                else if (ts.isExportsOrModuleExportsOrAlias(sourceFile, left.expression)) {
                    convertNamedExport(sourceFile, assignment, changes, exports);
                }
                return false;
            }
            /**
             * Convert `module.exports = { ... }` to individual exports..
             * We can't always do this if the module has interesting members -- then it will be a default export instead.
             */
            function tryChangeModuleExportsObject(object) {
                var statements = ts.mapAllOrFail(object.properties, function (prop) {
                    switch (prop.kind) {
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                        // TODO: Maybe we should handle this? See fourslash test `refactorConvertToEs6Module_export_object_shorthand.ts`.
                        // falls through
                        case 282 /* ShorthandPropertyAssignment */:
                        case 283 /* SpreadAssignment */:
                            return undefined;
                        case 281 /* PropertyAssignment */:
                            return !ts.isIdentifier(prop.name) ? undefined : convertExportsDotXEquals_replaceNode(prop.name.text, prop.initializer);
                        case 161 /* MethodDeclaration */:
                            return !ts.isIdentifier(prop.name) ? undefined : functionExpressionToDeclaration(prop.name.text, [ts.createToken(89 /* ExportKeyword */)], prop);
                        default:
                            ts.Debug.assertNever(prop, "Convert to ES6 got invalid prop kind " + prop.kind);
                    }
                });
                return statements && [statements, false];
            }
            function convertNamedExport(sourceFile, assignment, changes, exports) {
                // If "originalKeywordKind" was set, this is e.g. `exports.
                var text = assignment.left.name.text;
                var rename = exports.get(text);
                if (rename !== undefined) {
                    /*
                    const _class = 0;
                    export { _class as class };
                    */
                    var newNodes = [
                        makeConst(/*modifiers*/ undefined, rename, assignment.right),
                        makeExportDeclaration([ts.createExportSpecifier(rename, text)]),
                    ];
                    changes.replaceNodeWithNodes(sourceFile, assignment.parent, newNodes);
                }
                else {
                    convertExportsPropertyAssignment(assignment, sourceFile, changes);
                }
            }
            function convertReExportAll(reExported, checker) {
                // `module.exports = require("x");` ==> `export * from "x"; export { default } from "x";`
                var moduleSpecifier = reExported.text;
                var moduleSymbol = checker.getSymbolAtLocation(reExported);
                var exports = moduleSymbol ? moduleSymbol.exports : ts.emptyUnderscoreEscapedMap;
                return exports.has("export=") ? [[reExportDefault(moduleSpecifier)], true] :
                    !exports.has("default") ? [[reExportStar(moduleSpecifier)], false] :
                        // If there's some non-default export, must include both `export *` and `export default`.
                        exports.size > 1 ? [[reExportStar(moduleSpecifier), reExportDefault(moduleSpecifier)], true] : [[reExportDefault(moduleSpecifier)], true];
            }
            function reExportStar(moduleSpecifier) {
                return makeExportDeclaration(/*exportClause*/ undefined, moduleSpecifier);
            }
            function reExportDefault(moduleSpecifier) {
                return makeExportDeclaration([ts.createExportSpecifier(/*propertyName*/ undefined, "default")], moduleSpecifier);
            }
            function convertExportsPropertyAssignment(_a, sourceFile, changes) {
                var left = _a.left, right = _a.right, parent = _a.parent;
                var name = left.name.text;
                if ((ts.isFunctionExpression(right) || ts.isArrowFunction(right) || ts.isClassExpression(right)) && (!right.name || right.name.text === name)) {
                    // `exports.f = function() {}` -> `export function f() {}` -- Replace `exports.f = ` with `export `, and insert the name after `function`.
                    changes.replaceRange(sourceFile, { pos: left.getStart(sourceFile), end: right.getStart(sourceFile) }, ts.createToken(89 /* ExportKeyword */), { suffix: " " });
                    if (!right.name)
                        changes.insertName(sourceFile, right, name);
                    var semi = ts.findChildOfKind(parent, 26 /* SemicolonToken */, sourceFile);
                    if (semi)
                        changes.delete(sourceFile, semi);
                }
                else {
                    // `exports.f = function g() {}` -> `export const f = function g() {}` -- just replace `exports.` with `export const `
                    changes.replaceNodeRangeWithNodes(sourceFile, left.expression, ts.findChildOfKind(left, 24 /* DotToken */, sourceFile), [ts.createToken(89 /* ExportKeyword */), ts.createToken(81 /* ConstKeyword */)], { joiner: " ", suffix: " " });
                }
            }
            // TODO: GH#22492 this will cause an error if a change has been made inside the body of the node.
            function convertExportsDotXEquals_replaceNode(name, exported) {
                var modifiers = [ts.createToken(89 /* ExportKeyword */)];
                switch (exported.kind) {
                    case 201 /* FunctionExpression */: {
                        var expressionName = exported.name;
                        if (expressionName && expressionName.text !== name) {
                            // `exports.f = function g() {}` -> `export const f = function g() {}`
                            return exportConst();
                        }
                    }
                    // falls through
                    case 202 /* ArrowFunction */:
                        // `exports.f = function() {}` --> `export function f() {}`
                        return functionExpressionToDeclaration(name, modifiers, exported);
                    case 214 /* ClassExpression */:
                        // `exports.C = class {}` --> `export class C {}`
                        return classExpressionToDeclaration(name, modifiers, exported);
                    default:
                        return exportConst();
                }
                function exportConst() {
                    // `exports.x = 0;` --> `export const x = 0;`
                    return makeConst(modifiers, ts.createIdentifier(name), exported); // TODO: GH#18217
                }
            }
            /**
             * Converts `const <<name>> = require("x");`.
             * Returns nodes that will replace the variable declaration for the commonjs import.
             * May also make use `changes` to remove qualifiers at the use sites of imports, to change `mod.x` to `x`.
             */
            function convertSingleImport(file, name, moduleSpecifier, changes, checker, identifiers, target, quotePreference) {
                switch (name.kind) {
                    case 189 /* ObjectBindingPattern */: {
                        var importSpecifiers = ts.mapAllOrFail(name.elements, function (e) {
                            return e.dotDotDotToken || e.initializer || e.propertyName && !ts.isIdentifier(e.propertyName) || !ts.isIdentifier(e.name)
                                ? undefined
                                // (TODO: GH#18217)
                                // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
                                : makeImportSpecifier(e.propertyName && e.propertyName.text, e.name.text);
                        });
                        if (importSpecifiers) {
                            return [ts.makeImport(/*name*/ undefined, importSpecifiers, moduleSpecifier, quotePreference)];
                        }
                    }
                    // falls through -- object destructuring has an interesting pattern and must be a variable declaration
                    case 190 /* ArrayBindingPattern */: {
                        /*
                        import x from "x";
                        const [a, b, c] = x;
                        */
                        var tmp = makeUniqueName(codefix.moduleSpecifierToValidIdentifier(moduleSpecifier.text, target), identifiers);
                        return [
                            ts.makeImport(ts.createIdentifier(tmp), /*namedImports*/ undefined, moduleSpecifier, quotePreference),
                            makeConst(/*modifiers*/ undefined, ts.getSynthesizedDeepClone(name), ts.createIdentifier(tmp)),
                        ];
                    }
                    case 75 /* Identifier */:
                        return convertSingleIdentifierImport(file, name, moduleSpecifier, changes, checker, identifiers, quotePreference);
                    default:
                        return ts.Debug.assertNever(name, "Convert to ES6 module got invalid name kind " + name.kind);
                }
            }
            /**
             * Convert `import x = require("x").`
             * Also converts uses like `x.y()` to `y()` and uses a named import.
             */
            function convertSingleIdentifierImport(file, name, moduleSpecifier, changes, checker, identifiers, quotePreference) {
                var nameSymbol = checker.getSymbolAtLocation(name);
                // Maps from module property name to name actually used. (The same if there isn't shadowing.)
                var namedBindingsNames = ts.createMap();
                // True if there is some non-property use like `x()` or `f(x)`.
                var needDefaultImport = false;
                for (var _i = 0, _a = identifiers.original.get(name.text); _i < _a.length; _i++) {
                    var use = _a[_i];
                    if (checker.getSymbolAtLocation(use) !== nameSymbol || use === name) {
                        // This was a use of a different symbol with the same name, due to shadowing. Ignore.
                        continue;
                    }
                    var parent = use.parent;
                    if (ts.isPropertyAccessExpression(parent)) {
                        var expression = parent.expression, propertyName = parent.name.text;
                        ts.Debug.assert(expression === use, "Didn't expect expression === use"); // Else shouldn't have been in `collectIdentifiers`
                        var idName = namedBindingsNames.get(propertyName);
                        if (idName === undefined) {
                            idName = makeUniqueName(propertyName, identifiers);
                            namedBindingsNames.set(propertyName, idName);
                        }
                        changes.replaceNode(file, parent, ts.createIdentifier(idName));
                    }
                    else {
                        needDefaultImport = true;
                    }
                }
                var namedBindings = namedBindingsNames.size === 0 ? undefined : ts.arrayFrom(ts.mapIterator(namedBindingsNames.entries(), function (_a) {
                    var propertyName = _a[0], idName = _a[1];
                    return ts.createImportSpecifier(propertyName === idName ? undefined : ts.createIdentifier(propertyName), ts.createIdentifier(idName));
                }));
                if (!namedBindings) {
                    // If it was unused, ensure that we at least import *something*.
                    needDefaultImport = true;
                }
                return [ts.makeImport(needDefaultImport ? ts.getSynthesizedDeepClone(name) : undefined, namedBindings, moduleSpecifier, quotePreference)];
            }
            // Identifiers helpers
            function makeUniqueName(name, identifiers) {
                while (identifiers.original.has(name) || identifiers.additional.has(name)) {
                    name = "_" + name;
                }
                identifiers.additional.set(name, true);
                return name;
            }
            function collectFreeIdentifiers(file) {
                var map = ts.createMultiMap();
                forEachFreeIdentifier(file, function (id) { return map.add(id.text, id); });
                return map;
            }
            /**
             * A free identifier is an identifier that can be accessed through name lookup as a local variable.
             * In the expression `x.y`, `x` is a free identifier, but `y` is not.
             */
            function forEachFreeIdentifier(node, cb) {
                if (ts.isIdentifier(node) && isFreeIdentifier(node))
                    cb(node);
                node.forEachChild(function (child) { return forEachFreeIdentifier(child, cb); });
            }
            function isFreeIdentifier(node) {
                var parent = node.parent;
                switch (parent.kind) {
                    case 194 /* PropertyAccessExpression */:
                        return parent.name !== node;
                    case 191 /* BindingElement */:
                        return parent.propertyName !== node;
                    case 258 /* ImportSpecifier */:
                        return parent.propertyName !== node;
                    default:
                        return true;
                }
            }
            // Node helpers
            function functionExpressionToDeclaration(name, additionalModifiers, fn) {
                return ts.createFunctionDeclaration(ts.getSynthesizedDeepClones(fn.decorators), // TODO: GH#19915 Don't think this is even legal.
                ts.concatenate(additionalModifiers, ts.getSynthesizedDeepClones(fn.modifiers)), ts.getSynthesizedDeepClone(fn.asteriskToken), name, ts.getSynthesizedDeepClones(fn.typeParameters), ts.getSynthesizedDeepClones(fn.parameters), ts.getSynthesizedDeepClone(fn.type), ts.convertToFunctionBody(ts.getSynthesizedDeepClone(fn.body)));
            }
            function classExpressionToDeclaration(name, additionalModifiers, cls) {
                return ts.createClassDeclaration(ts.getSynthesizedDeepClones(cls.decorators), // TODO: GH#19915 Don't think this is even legal.
                ts.concatenate(additionalModifiers, ts.getSynthesizedDeepClones(cls.modifiers)), name, ts.getSynthesizedDeepClones(cls.typeParameters), ts.getSynthesizedDeepClones(cls.heritageClauses), ts.getSynthesizedDeepClones(cls.members));
            }
            function makeSingleImport(localName, propertyName, moduleSpecifier, quotePreference) {
                return propertyName === "default"
                    ? ts.makeImport(ts.createIdentifier(localName), /*namedImports*/ undefined, moduleSpecifier, quotePreference)
                    : ts.makeImport(/*name*/ undefined, [makeImportSpecifier(propertyName, localName)], moduleSpecifier, quotePreference);
            }
            function makeImportSpecifier(propertyName, name) {
                return ts.createImportSpecifier(propertyName !== undefined && propertyName !== name ? ts.createIdentifier(propertyName) : undefined, ts.createIdentifier(name));
            }
            function makeConst(modifiers, name, init) {
                return ts.createVariableStatement(modifiers, ts.createVariableDeclarationList([ts.createVariableDeclaration(name, /*type*/ undefined, init)], 2 /* Const */));
            }
            function makeExportDeclaration(exportSpecifiers, moduleSpecifier) {
                return ts.createExportDeclaration(
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, exportSpecifiers && ts.createNamedExports(exportSpecifiers), moduleSpecifier === undefined ? undefined : ts.createLiteral(moduleSpecifier));
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "correctQualifiedNameToIndexedAccessType";
            var errorCodes = [ts.Diagnostics.Cannot_access_0_1_because_0_is_a_type_but_not_a_namespace_Did_you_mean_to_retrieve_the_type_of_the_property_1_in_0_with_0_1.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var qualifiedName = getQualifiedName(context.sourceFile, context.span.start);
                    if (!qualifiedName)
                        return undefined;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, context.sourceFile, qualifiedName); });
                    var newText = qualifiedName.left.text + "[\"" + qualifiedName.right.text + "\"]";
                    return [codefix.createCodeFixAction(fixId, changes, [ts.Diagnostics.Rewrite_as_the_indexed_access_type_0, newText], fixId, ts.Diagnostics.Rewrite_all_as_indexed_access_types)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var q = getQualifiedName(diag.file, diag.start);
                        if (q) {
                            doChange(changes, diag.file, q);
                        }
                    });
                },
            });
            function getQualifiedName(sourceFile, pos) {
                var qualifiedName = ts.findAncestor(ts.getTokenAtPosition(sourceFile, pos), ts.isQualifiedName);
                ts.Debug.assert(!!qualifiedName, "Expected position to be owned by a qualified name.");
                return ts.isIdentifier(qualifiedName.left) ? qualifiedName : undefined;
            }
            function doChange(changeTracker, sourceFile, qualifiedName) {
                var rightText = qualifiedName.right.text;
                var replacement = ts.createIndexedAccessTypeNode(ts.createTypeReferenceNode(qualifiedName.left, /*typeArguments*/ undefined), ts.createLiteralTypeNode(ts.createLiteral(rightText)));
                changeTracker.replaceNode(sourceFile, qualifiedName, replacement);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var errorCodes = [ts.Diagnostics.Re_exporting_a_type_when_the_isolatedModules_flag_is_provided_requires_using_export_type.code];
            var fixId = "convertToTypeOnlyExport";
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return fixSingleExportDeclaration(t, getExportSpecifierForDiagnosticSpan(context.span, context.sourceFile), context); });
                    if (changes.length) {
                        return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Convert_to_type_only_export, fixId, ts.Diagnostics.Convert_all_re_exported_types_to_type_only_exports)];
                    }
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    var fixedExportDeclarations = ts.createMap();
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var exportSpecifier = getExportSpecifierForDiagnosticSpan(diag, context.sourceFile);
                        if (exportSpecifier && !ts.addToSeen(fixedExportDeclarations, ts.getNodeId(exportSpecifier.parent.parent))) {
                            fixSingleExportDeclaration(changes, exportSpecifier, context);
                        }
                    });
                }
            });
            function getExportSpecifierForDiagnosticSpan(span, sourceFile) {
                return ts.tryCast(ts.getTokenAtPosition(sourceFile, span.start).parent, ts.isExportSpecifier);
            }
            function fixSingleExportDeclaration(changes, exportSpecifier, context) {
                if (!exportSpecifier) {
                    return;
                }
                var exportClause = exportSpecifier.parent;
                var exportDeclaration = exportClause.parent;
                var typeExportSpecifiers = getTypeExportSpecifiers(exportSpecifier, context);
                if (typeExportSpecifiers.length === exportClause.elements.length) {
                    changes.replaceNode(context.sourceFile, exportDeclaration, ts.updateExportDeclaration(exportDeclaration, exportDeclaration.decorators, exportDeclaration.modifiers, exportClause, exportDeclaration.moduleSpecifier, 
                    /*isTypeOnly*/ true));
                }
                else {
                    var valueExportDeclaration = ts.updateExportDeclaration(exportDeclaration, exportDeclaration.decorators, exportDeclaration.modifiers, ts.updateNamedExports(exportClause, ts.filter(exportClause.elements, function (e) { return !ts.contains(typeExportSpecifiers, e); })), exportDeclaration.moduleSpecifier, 
                    /*isTypeOnly*/ false);
                    var typeExportDeclaration = ts.createExportDeclaration(
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, ts.createNamedExports(typeExportSpecifiers), exportDeclaration.moduleSpecifier, 
                    /*isTypeOnly*/ true);
                    changes.replaceNode(context.sourceFile, exportDeclaration, valueExportDeclaration);
                    changes.insertNodeAfter(context.sourceFile, exportDeclaration, typeExportDeclaration);
                }
            }
            function getTypeExportSpecifiers(originExportSpecifier, context) {
                var exportClause = originExportSpecifier.parent;
                if (exportClause.elements.length === 1) {
                    return exportClause.elements;
                }
                var diagnostics = ts.getDiagnosticsWithinSpan(ts.createTextSpanFromNode(exportClause), context.program.getSemanticDiagnostics(context.sourceFile, context.cancellationToken));
                return ts.filter(exportClause.elements, function (element) {
                    var _a;
                    return element === originExportSpecifier || ((_a = ts.findDiagnosticForNode(element, diagnostics)) === null || _a === void 0 ? void 0 : _a.code) === errorCodes[0];
                });
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var errorCodes = [
                ts.Diagnostics.Class_0_incorrectly_implements_interface_1.code,
                ts.Diagnostics.Class_0_incorrectly_implements_class_1_Did_you_mean_to_extend_1_and_inherit_its_members_as_a_subclass.code
            ];
            var fixId = "fixClassIncorrectlyImplementsInterface"; // TODO: share a group with fixClassDoesntImplementInheritedAbstractMember?
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, span = context.span;
                    var classDeclaration = getClass(sourceFile, span.start);
                    return ts.mapDefined(ts.getClassImplementsHeritageClauseElements(classDeclaration), function (implementedTypeNode) {
                        var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return addMissingDeclarations(context, implementedTypeNode, sourceFile, classDeclaration, t, context.preferences); });
                        return changes.length === 0 ? undefined : codefix.createCodeFixAction(fixId, changes, [ts.Diagnostics.Implement_interface_0, implementedTypeNode.getText(sourceFile)], fixId, ts.Diagnostics.Implement_all_unimplemented_interfaces);
                    });
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    var seenClassDeclarations = ts.createMap();
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var classDeclaration = getClass(diag.file, diag.start);
                        if (ts.addToSeen(seenClassDeclarations, ts.getNodeId(classDeclaration))) {
                            for (var _i = 0, _a = ts.getClassImplementsHeritageClauseElements(classDeclaration); _i < _a.length; _i++) {
                                var implementedTypeNode = _a[_i];
                                addMissingDeclarations(context, implementedTypeNode, diag.file, classDeclaration, changes, context.preferences);
                            }
                        }
                    });
                },
            });
            function getClass(sourceFile, pos) {
                return ts.Debug.assertDefined(ts.getContainingClass(ts.getTokenAtPosition(sourceFile, pos)), "There should be a containing class");
            }
            function symbolPointsToNonPrivateMember(symbol) {
                return !symbol.valueDeclaration || !(ts.getModifierFlags(symbol.valueDeclaration) & 8 /* Private */);
            }
            function addMissingDeclarations(context, implementedTypeNode, sourceFile, classDeclaration, changeTracker, preferences) {
                var checker = context.program.getTypeChecker();
                var maybeHeritageClauseSymbol = getHeritageClauseSymbolTable(classDeclaration, checker);
                // Note that this is ultimately derived from a map indexed by symbol names,
                // so duplicates cannot occur.
                var implementedType = checker.getTypeAtLocation(implementedTypeNode);
                var implementedTypeSymbols = checker.getPropertiesOfType(implementedType);
                var nonPrivateAndNotExistedInHeritageClauseMembers = implementedTypeSymbols.filter(ts.and(symbolPointsToNonPrivateMember, function (symbol) { return !maybeHeritageClauseSymbol.has(symbol.escapedName); }));
                var classType = checker.getTypeAtLocation(classDeclaration);
                var constructor = ts.find(classDeclaration.members, function (m) { return ts.isConstructorDeclaration(m); });
                if (!classType.getNumberIndexType()) {
                    createMissingIndexSignatureDeclaration(implementedType, 1 /* Number */);
                }
                if (!classType.getStringIndexType()) {
                    createMissingIndexSignatureDeclaration(implementedType, 0 /* String */);
                }
                codefix.createMissingMemberNodes(classDeclaration, nonPrivateAndNotExistedInHeritageClauseMembers, context, preferences, function (member) { return insertInterfaceMemberNode(sourceFile, classDeclaration, member); });
                function createMissingIndexSignatureDeclaration(type, kind) {
                    var indexInfoOfKind = checker.getIndexInfoOfType(type, kind);
                    if (indexInfoOfKind) {
                        insertInterfaceMemberNode(sourceFile, classDeclaration, checker.indexInfoToIndexSignatureDeclaration(indexInfoOfKind, kind, classDeclaration, /*flags*/ undefined, codefix.getNoopSymbolTrackerWithResolver(context)));
                    }
                }
                // Either adds the node at the top of the class, or if there's a constructor right after that
                function insertInterfaceMemberNode(sourceFile, cls, newElement) {
                    if (constructor) {
                        changeTracker.insertNodeAfter(sourceFile, constructor, newElement);
                    }
                    else {
                        changeTracker.insertNodeAtClassStart(sourceFile, cls, newElement);
                    }
                }
            }
            function getHeritageClauseSymbolTable(classDeclaration, checker) {
                var heritageClauseNode = ts.getEffectiveBaseTypeNode(classDeclaration);
                if (!heritageClauseNode)
                    return ts.createSymbolTable();
                var heritageClauseType = checker.getTypeAtLocation(heritageClauseNode);
                var heritageClauseTypeSymbols = checker.getPropertiesOfType(heritageClauseType);
                return ts.createSymbolTable(heritageClauseTypeSymbols.filter(symbolPointsToNonPrivateMember));
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            codefix.importFixName = "import";
            var importFixId = "fixMissingImport";
            var errorCodes = [
                ts.Diagnostics.Cannot_find_name_0.code,
                ts.Diagnostics.Cannot_find_name_0_Did_you_mean_1.code,
                ts.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0.code,
                ts.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0.code,
                ts.Diagnostics.Cannot_find_namespace_0.code,
                ts.Diagnostics._0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead.code,
                ts.Diagnostics._0_only_refers_to_a_type_but_is_being_used_as_a_value_here.code,
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var errorCode = context.errorCode, preferences = context.preferences, sourceFile = context.sourceFile, span = context.span;
                    var info = getFixesInfo(context, errorCode, span.start);
                    if (!info)
                        return undefined;
                    var fixes = info.fixes, symbolName = info.symbolName;
                    var quotePreference = ts.getQuotePreference(sourceFile, preferences);
                    return fixes.map(function (fix) { return codeActionForFix(context, sourceFile, symbolName, fix, quotePreference); });
                },
                fixIds: [importFixId],
                getAllCodeActions: function (context) {
                    var sourceFile = context.sourceFile, preferences = context.preferences;
                    // Namespace fixes don't conflict, so just build a list.
                    var addToNamespace = [];
                    var importType = [];
                    // Keys are import clause node IDs.
                    var addToExisting = ts.createMap();
                    var newImports = ts.createMap();
                    var lastModuleSpecifier;
                    codefix.eachDiagnostic(context, errorCodes, function (diag) {
                        var info = getFixesInfo(context, diag.code, diag.start);
                        if (!info || !info.fixes.length)
                            return;
                        var fixes = info.fixes, symbolName = info.symbolName;
                        var fix = ts.first(fixes);
                        switch (fix.kind) {
                            case 0 /* UseNamespace */:
                                addToNamespace.push(fix);
                                break;
                            case 1 /* ImportType */:
                                importType.push(fix);
                                break;
                            case 2 /* AddToExisting */: {
                                var importClause = fix.importClause, importKind = fix.importKind, canUseTypeOnlyImport = fix.canUseTypeOnlyImport;
                                var key = String(ts.getNodeId(importClause));
                                var entry = addToExisting.get(key);
                                if (!entry) {
                                    addToExisting.set(key, entry = { importClause: importClause, defaultImport: undefined, namedImports: [], canUseTypeOnlyImport: canUseTypeOnlyImport });
                                }
                                if (importKind === 0 /* Named */) {
                                    ts.pushIfUnique(entry.namedImports, symbolName);
                                }
                                else {
                                    ts.Debug.assert(entry.defaultImport === undefined || entry.defaultImport === symbolName, "(Add to Existing) Default import should be missing or match symbolName");
                                    entry.defaultImport = symbolName;
                                }
                                break;
                            }
                            case 3 /* AddNew */: {
                                var moduleSpecifier = fix.moduleSpecifier, importKind = fix.importKind;
                                var entry = newImports.get(moduleSpecifier);
                                if (!entry) {
                                    newImports.set(moduleSpecifier, entry = { defaultImport: undefined, namedImports: [], namespaceLikeImport: undefined });
                                    lastModuleSpecifier = moduleSpecifier;
                                }
                                switch (importKind) {
                                    case 1 /* Default */:
                                        ts.Debug.assert(entry.defaultImport === undefined || entry.defaultImport === symbolName, "(Add new) Default import should be missing or match symbolName");
                                        entry.defaultImport = symbolName;
                                        break;
                                    case 0 /* Named */:
                                        ts.pushIfUnique(entry.namedImports, symbolName);
                                        break;
                                    case 3 /* Equals */:
                                    case 2 /* Namespace */:
                                        ts.Debug.assert(entry.namespaceLikeImport === undefined || entry.namespaceLikeImport.name === symbolName, "Namespacelike import shoudl be missing or match symbolName");
                                        entry.namespaceLikeImport = { importKind: importKind, name: symbolName };
                                        break;
                                }
                                break;
                            }
                            default:
                                ts.Debug.assertNever(fix, "fix wasn't never - got kind " + fix.kind);
                        }
                    });
                    return codefix.createCombinedCodeActions(ts.textChanges.ChangeTracker.with(context, function (changes) {
                        var quotePreference = ts.getQuotePreference(sourceFile, preferences);
                        for (var _i = 0, addToNamespace_1 = addToNamespace; _i < addToNamespace_1.length; _i++) {
                            var fix = addToNamespace_1[_i];
                            addNamespaceQualifier(changes, sourceFile, fix);
                        }
                        for (var _a = 0, importType_1 = importType; _a < importType_1.length; _a++) {
                            var fix = importType_1[_a];
                            addImportType(changes, sourceFile, fix, quotePreference);
                        }
                        addToExisting.forEach(function (_a) {
                            var importClause = _a.importClause, defaultImport = _a.defaultImport, namedImports = _a.namedImports, canUseTypeOnlyImport = _a.canUseTypeOnlyImport;
                            doAddExistingFix(changes, sourceFile, importClause, defaultImport, namedImports, canUseTypeOnlyImport);
                        });
                        newImports.forEach(function (imports, moduleSpecifier) {
                            addNewImports(changes, sourceFile, moduleSpecifier, quotePreference, imports, /*blankLineBetween*/ lastModuleSpecifier === moduleSpecifier);
                        });
                    }));
                },
            });
            // Sorted with the preferred fix coming first.
            var ImportFixKind;
            (function (ImportFixKind) {
                ImportFixKind[ImportFixKind["UseNamespace"] = 0] = "UseNamespace";
                ImportFixKind[ImportFixKind["ImportType"] = 1] = "ImportType";
                ImportFixKind[ImportFixKind["AddToExisting"] = 2] = "AddToExisting";
                ImportFixKind[ImportFixKind["AddNew"] = 3] = "AddNew";
            })(ImportFixKind || (ImportFixKind = {}));
            var ImportKind;
            (function (ImportKind) {
                ImportKind[ImportKind["Named"] = 0] = "Named";
                ImportKind[ImportKind["Default"] = 1] = "Default";
                ImportKind[ImportKind["Namespace"] = 2] = "Namespace";
                ImportKind[ImportKind["Equals"] = 3] = "Equals";
                ImportKind[ImportKind["ConstEquals"] = 4] = "ConstEquals";
            })(ImportKind || (ImportKind = {}));
            function getImportCompletionAction(exportedSymbol, moduleSymbol, sourceFile, symbolName, host, program, formatContext, position, preferences) {
                var exportInfos = getAllReExportingModules(sourceFile, exportedSymbol, moduleSymbol, symbolName, sourceFile, program.getCompilerOptions(), program.getTypeChecker(), program.getSourceFiles());
                ts.Debug.assert(exportInfos.some(function (info) { return info.moduleSymbol === moduleSymbol; }), "Some exportInfo should match the specified moduleSymbol");
                // We sort the best codefixes first, so taking `first` is best for completions.
                var moduleSpecifier = ts.first(getNewImportInfos(program, sourceFile, position, exportInfos, host, preferences)).moduleSpecifier;
                var fix = ts.first(getFixForImport(exportInfos, symbolName, position, program, sourceFile, host, preferences));
                return { moduleSpecifier: moduleSpecifier, codeAction: codeFixActionToCodeAction(codeActionForFix({ host: host, formatContext: formatContext, preferences: preferences }, sourceFile, symbolName, fix, ts.getQuotePreference(sourceFile, preferences))) };
            }
            codefix.getImportCompletionAction = getImportCompletionAction;
            function codeFixActionToCodeAction(_a) {
                var description = _a.description, changes = _a.changes, commands = _a.commands;
                return { description: description, changes: changes, commands: commands };
            }
            function getAllReExportingModules(importingFile, exportedSymbol, exportingModuleSymbol, symbolName, sourceFile, compilerOptions, checker, allSourceFiles) {
                var result = [];
                forEachExternalModule(checker, allSourceFiles, function (moduleSymbol, moduleFile) {
                    // Don't import from a re-export when looking "up" like to `./index` or `../index`.
                    if (moduleFile && moduleSymbol !== exportingModuleSymbol && ts.startsWith(sourceFile.fileName, ts.getDirectoryPath(moduleFile.fileName))) {
                        return;
                    }
                    var defaultInfo = getDefaultLikeExportInfo(importingFile, moduleSymbol, checker, compilerOptions);
                    if (defaultInfo && defaultInfo.name === symbolName && ts.skipAlias(defaultInfo.symbol, checker) === exportedSymbol) {
                        result.push({ moduleSymbol: moduleSymbol, importKind: defaultInfo.kind, exportedSymbolIsTypeOnly: isTypeOnlySymbol(defaultInfo.symbol, checker) });
                    }
                    for (var _i = 0, _a = checker.getExportsOfModule(moduleSymbol); _i < _a.length; _i++) {
                        var exported = _a[_i];
                        if (exported.name === symbolName && ts.skipAlias(exported, checker) === exportedSymbol) {
                            result.push({ moduleSymbol: moduleSymbol, importKind: 0 /* Named */, exportedSymbolIsTypeOnly: isTypeOnlySymbol(exported, checker) });
                        }
                    }
                });
                return result;
            }
            function isTypeOnlySymbol(s, checker) {
                return !(ts.skipAlias(s, checker).flags & 111551 /* Value */);
            }
            function isTypeOnlyPosition(sourceFile, position) {
                return ts.isValidTypeOnlyAliasUseSite(ts.getTokenAtPosition(sourceFile, position));
            }
            function getFixForImport(exportInfos, symbolName, 
            /** undefined only for missing JSX namespace */
            position, program, sourceFile, host, preferences) {
                var checker = program.getTypeChecker();
                var existingImports = ts.flatMap(exportInfos, function (info) { return getExistingImportDeclarations(info, checker, sourceFile); });
                var useNamespace = position === undefined ? undefined : tryUseExistingNamespaceImport(existingImports, symbolName, position, checker);
                var addToExisting = tryAddToExistingImport(existingImports, position !== undefined && isTypeOnlyPosition(sourceFile, position));
                // Don't bother providing an action to add a new import if we can add to an existing one.
                var addImport = addToExisting ? [addToExisting] : getFixesForAddImport(exportInfos, existingImports, program, sourceFile, position, host, preferences);
                return __spreadArrays((useNamespace ? [useNamespace] : ts.emptyArray), addImport);
            }
            function tryUseExistingNamespaceImport(existingImports, symbolName, position, checker) {
                // It is possible that multiple import statements with the same specifier exist in the file.
                // e.g.
                //
                //     import * as ns from "foo";
                //     import { member1, member2 } from "foo";
                //
                //     member3/**/ <-- cusor here
                //
                // in this case we should provie 2 actions:
                //     1. change "member3" to "ns.member3"
                //     2. add "member3" to the second import statement's import list
                // and it is up to the user to decide which one fits best.
                return ts.firstDefined(existingImports, function (_a) {
                    var declaration = _a.declaration;
                    var namespace = getNamespaceImportName(declaration);
                    if (namespace) {
                        var moduleSymbol = checker.getAliasedSymbol(checker.getSymbolAtLocation(namespace));
                        if (moduleSymbol && moduleSymbol.exports.has(ts.escapeLeadingUnderscores(symbolName))) {
                            return { kind: 0 /* UseNamespace */, namespacePrefix: namespace.text, position: position };
                        }
                    }
                });
            }
            function tryAddToExistingImport(existingImports, canUseTypeOnlyImport) {
                return ts.firstDefined(existingImports, function (_a) {
                    var declaration = _a.declaration, importKind = _a.importKind;
                    if (declaration.kind !== 254 /* ImportDeclaration */)
                        return undefined;
                    var importClause = declaration.importClause;
                    if (!importClause)
                        return undefined;
                    var name = importClause.name, namedBindings = importClause.namedBindings;
                    return importKind === 1 /* Default */ && !name || importKind === 0 /* Named */ && (!namedBindings || namedBindings.kind === 257 /* NamedImports */)
                        ? { kind: 2 /* AddToExisting */, importClause: importClause, importKind: importKind, canUseTypeOnlyImport: canUseTypeOnlyImport }
                        : undefined;
                });
            }
            function getNamespaceImportName(declaration) {
                if (declaration.kind === 254 /* ImportDeclaration */) {
                    var namedBindings = declaration.importClause && ts.isImportClause(declaration.importClause) && declaration.importClause.namedBindings;
                    return namedBindings && namedBindings.kind === 256 /* NamespaceImport */ ? namedBindings.name : undefined;
                }
                else {
                    return declaration.name;
                }
            }
            function getExistingImportDeclarations(_a, checker, sourceFile) {
                var moduleSymbol = _a.moduleSymbol, importKind = _a.importKind, exportedSymbolIsTypeOnly = _a.exportedSymbolIsTypeOnly;
                // Can't use an es6 import for a type in JS.
                return exportedSymbolIsTypeOnly && ts.isSourceFileJS(sourceFile) ? ts.emptyArray : ts.mapDefined(sourceFile.imports, function (moduleSpecifier) {
                    var i = ts.importFromModuleSpecifier(moduleSpecifier);
                    return (i.kind === 254 /* ImportDeclaration */ || i.kind === 253 /* ImportEqualsDeclaration */)
                        && checker.getSymbolAtLocation(moduleSpecifier) === moduleSymbol ? { declaration: i, importKind: importKind, exportedSymbolIsTypeOnly: exportedSymbolIsTypeOnly } : undefined;
                });
            }
            function getNewImportInfos(program, sourceFile, position, moduleSymbols, host, preferences) {
                var isJs = ts.isSourceFileJS(sourceFile);
                var allowsImportingSpecifier = createAutoImportFilter(sourceFile, program, host).allowsImportingSpecifier;
                var choicesForEachExportingModule = ts.flatMap(moduleSymbols, function (_a) {
                    var moduleSymbol = _a.moduleSymbol, importKind = _a.importKind, exportedSymbolIsTypeOnly = _a.exportedSymbolIsTypeOnly;
                    return ts.moduleSpecifiers.getModuleSpecifiers(moduleSymbol, program.getCompilerOptions(), sourceFile, host, program.getSourceFiles(), preferences, program.redirectTargetsMap)
                        .map(function (moduleSpecifier) {
                        // `position` should only be undefined at a missing jsx namespace, in which case we shouldn't be looking for pure types.
                        return exportedSymbolIsTypeOnly && isJs
                            ? { kind: 1 /* ImportType */, moduleSpecifier: moduleSpecifier, position: ts.Debug.assertDefined(position, "position should be defined") }
                            : { kind: 3 /* AddNew */, moduleSpecifier: moduleSpecifier, importKind: importKind };
                    });
                });
                // Sort by presence in package.json, then shortest paths first
                return ts.sort(choicesForEachExportingModule, function (a, b) {
                    var allowsImportingA = allowsImportingSpecifier(a.moduleSpecifier);
                    var allowsImportingB = allowsImportingSpecifier(b.moduleSpecifier);
                    if (allowsImportingA && !allowsImportingB) {
                        return -1;
                    }
                    if (allowsImportingB && !allowsImportingA) {
                        return 1;
                    }
                    return a.moduleSpecifier.length - b.moduleSpecifier.length;
                });
            }
            function getFixesForAddImport(exportInfos, existingImports, program, sourceFile, position, host, preferences) {
                var existingDeclaration = ts.firstDefined(existingImports, newImportInfoFromExistingSpecifier);
                return existingDeclaration ? [existingDeclaration] : getNewImportInfos(program, sourceFile, position, exportInfos, host, preferences);
            }
            function newImportInfoFromExistingSpecifier(_a) {
                var declaration = _a.declaration, importKind = _a.importKind;
                var expression = declaration.kind === 254 /* ImportDeclaration */
                    ? declaration.moduleSpecifier
                    : declaration.moduleReference.kind === 265 /* ExternalModuleReference */
                        ? declaration.moduleReference.expression
                        : undefined;
                return expression && ts.isStringLiteral(expression)
                    ? { kind: 3 /* AddNew */, moduleSpecifier: expression.text, importKind: importKind }
                    : undefined;
            }
            function getFixesInfo(context, errorCode, pos) {
                var symbolToken = ts.getTokenAtPosition(context.sourceFile, pos);
                var info = errorCode === ts.Diagnostics._0_refers_to_a_UMD_global_but_the_current_file_is_a_module_Consider_adding_an_import_instead.code
                    ? getFixesInfoForUMDImport(context, symbolToken)
                    : ts.isIdentifier(symbolToken) ? getFixesInfoForNonUMDImport(context, symbolToken) : undefined;
                return info && __assign(__assign({}, info), { fixes: ts.sort(info.fixes, function (a, b) { return a.kind - b.kind; }) });
            }
            function getFixesInfoForUMDImport(_a, token) {
                var sourceFile = _a.sourceFile, program = _a.program, host = _a.host, preferences = _a.preferences;
                var checker = program.getTypeChecker();
                var umdSymbol = getUmdSymbol(token, checker);
                if (!umdSymbol)
                    return undefined;
                var symbol = checker.getAliasedSymbol(umdSymbol);
                var symbolName = umdSymbol.name;
                var exportInfos = [{ moduleSymbol: symbol, importKind: getUmdImportKind(sourceFile, program.getCompilerOptions()), exportedSymbolIsTypeOnly: false }];
                var fixes = getFixForImport(exportInfos, symbolName, ts.isIdentifier(token) ? token.getStart(sourceFile) : undefined, program, sourceFile, host, preferences);
                return { fixes: fixes, symbolName: symbolName };
            }
            function getUmdSymbol(token, checker) {
                // try the identifier to see if it is the umd symbol
                var umdSymbol = ts.isIdentifier(token) ? checker.getSymbolAtLocation(token) : undefined;
                if (ts.isUMDExportSymbol(umdSymbol))
                    return umdSymbol;
                // The error wasn't for the symbolAtLocation, it was for the JSX tag itself, which needs access to e.g. `React`.
                var parent = token.parent;
                return (ts.isJsxOpeningLikeElement(parent) && parent.tagName === token) || ts.isJsxOpeningFragment(parent)
                    ? ts.tryCast(checker.resolveName(checker.getJsxNamespace(parent), ts.isJsxOpeningLikeElement(parent) ? token : parent, 111551 /* Value */, /*excludeGlobals*/ false), ts.isUMDExportSymbol)
                    : undefined;
            }
            function getUmdImportKind(importingFile, compilerOptions) {
                // Import a synthetic `default` if enabled.
                if (ts.getAllowSyntheticDefaultImports(compilerOptions)) {
                    return 1 /* Default */;
                }
                // When a synthetic `default` is unavailable, use `import..require` if the module kind supports it.
                var moduleKind = ts.getEmitModuleKind(compilerOptions);
                switch (moduleKind) {
                    case ts.ModuleKind.AMD:
                    case ts.ModuleKind.CommonJS:
                    case ts.ModuleKind.UMD:
                        if (ts.isInJSFile(importingFile)) {
                            return ts.isExternalModule(importingFile) ? 2 /* Namespace */ : 4 /* ConstEquals */;
                        }
                        return 3 /* Equals */;
                    case ts.ModuleKind.System:
                    case ts.ModuleKind.ES2015:
                    case ts.ModuleKind.ES2020:
                    case ts.ModuleKind.ESNext:
                    case ts.ModuleKind.None:
                        // Fall back to the `import * as ns` style import.
                        return 2 /* Namespace */;
                    default:
                        return ts.Debug.assertNever(moduleKind, "Unexpected moduleKind " + moduleKind);
                }
            }
            function getFixesInfoForNonUMDImport(_a, symbolToken) {
                var sourceFile = _a.sourceFile, program = _a.program, cancellationToken = _a.cancellationToken, host = _a.host, preferences = _a.preferences;
                var checker = program.getTypeChecker();
                // If we're at `<Foo/>`, we must check if `Foo` is already in scope, and if so, get an import for `React` instead.
                var symbolName = ts.isJsxOpeningLikeElement(symbolToken.parent)
                    && symbolToken.parent.tagName === symbolToken
                    && (ts.isIntrinsicJsxName(symbolToken.text) || checker.resolveName(symbolToken.text, symbolToken, 67108863 /* All */, /*excludeGlobals*/ false))
                    ? checker.getJsxNamespace(sourceFile)
                    : symbolToken.text;
                // "default" is a keyword and not a legal identifier for the import, so we don't expect it here
                ts.Debug.assert(symbolName !== "default" /* Default */, "'default' isn't a legal identifier and couldn't occur here");
                var exportInfos = getExportInfos(symbolName, ts.getMeaningFromLocation(symbolToken), cancellationToken, sourceFile, checker, program, host);
                var fixes = ts.arrayFrom(ts.flatMapIterator(exportInfos.entries(), function (_a) {
                    var _ = _a[0], exportInfos = _a[1];
                    return getFixForImport(exportInfos, symbolName, symbolToken.getStart(sourceFile), program, sourceFile, host, preferences);
                }));
                return { fixes: fixes, symbolName: symbolName };
            }
            // Returns a map from an exported symbol's ID to a list of every way it's (re-)exported.
            function getExportInfos(symbolName, currentTokenMeaning, cancellationToken, sourceFile, checker, program, host) {
                // For each original symbol, keep all re-exports of that symbol together so we can call `getCodeActionsForImport` on the whole group at once.
                // Maps symbol id to info for modules providing that symbol (original export + re-exports).
                var originalSymbolToExportInfos = ts.createMultiMap();
                function addSymbol(moduleSymbol, exportedSymbol, importKind) {
                    originalSymbolToExportInfos.add(ts.getUniqueSymbolId(exportedSymbol, checker).toString(), { moduleSymbol: moduleSymbol, importKind: importKind, exportedSymbolIsTypeOnly: isTypeOnlySymbol(exportedSymbol, checker) });
                }
                forEachExternalModuleToImportFrom(program, host, sourceFile, /*filterByPackageJson*/ true, function (moduleSymbol) {
                    cancellationToken.throwIfCancellationRequested();
                    var defaultInfo = getDefaultLikeExportInfo(sourceFile, moduleSymbol, checker, program.getCompilerOptions());
                    if (defaultInfo && defaultInfo.name === symbolName && symbolHasMeaning(defaultInfo.symbolForMeaning, currentTokenMeaning)) {
                        addSymbol(moduleSymbol, defaultInfo.symbol, defaultInfo.kind);
                    }
                    // check exports with the same name
                    var exportSymbolWithIdenticalName = checker.tryGetMemberInModuleExportsAndProperties(symbolName, moduleSymbol);
                    if (exportSymbolWithIdenticalName && symbolHasMeaning(exportSymbolWithIdenticalName, currentTokenMeaning)) {
                        addSymbol(moduleSymbol, exportSymbolWithIdenticalName, 0 /* Named */);
                    }
                });
                return originalSymbolToExportInfos;
            }
            function getDefaultLikeExportInfo(importingFile, moduleSymbol, checker, compilerOptions) {
                var exported = getDefaultLikeExportWorker(importingFile, moduleSymbol, checker, compilerOptions);
                if (!exported)
                    return undefined;
                var symbol = exported.symbol, kind = exported.kind;
                var info = getDefaultExportInfoWorker(symbol, moduleSymbol, checker, compilerOptions);
                return info && __assign({ symbol: symbol, kind: kind }, info);
            }
            function getDefaultLikeExportWorker(importingFile, moduleSymbol, checker, compilerOptions) {
                var defaultExport = checker.tryGetMemberInModuleExports("default" /* Default */, moduleSymbol);
                if (defaultExport)
                    return { symbol: defaultExport, kind: 1 /* Default */ };
                var exportEquals = checker.resolveExternalModuleSymbol(moduleSymbol);
                return exportEquals === moduleSymbol ? undefined : { symbol: exportEquals, kind: getExportEqualsImportKind(importingFile, compilerOptions) };
            }
            function getExportEqualsImportKind(importingFile, compilerOptions) {
                var allowSyntheticDefaults = ts.getAllowSyntheticDefaultImports(compilerOptions);
                // 1. 'import =' will not work in es2015+, so the decision is between a default
                //    and a namespace import, based on allowSyntheticDefaultImports/esModuleInterop.
                if (ts.getEmitModuleKind(compilerOptions) >= ts.ModuleKind.ES2015) {
                    return allowSyntheticDefaults ? 1 /* Default */ : 2 /* Namespace */;
                }
                // 2. 'import =' will not work in JavaScript, so the decision is between a default
                //    and const/require.
                if (ts.isInJSFile(importingFile)) {
                    return ts.isExternalModule(importingFile) ? 1 /* Default */ : 4 /* ConstEquals */;
                }
                // 3. At this point the most correct choice is probably 'import =', but people
                //    really hate that, so look to see if the importing file has any precedent
                //    on how to handle it.
                for (var _i = 0, _a = importingFile.statements; _i < _a.length; _i++) {
                    var statement = _a[_i];
                    if (ts.isImportEqualsDeclaration(statement)) {
                        return 3 /* Equals */;
                    }
                }
                // 4. We have no precedent to go on, so just use a default import if
                //    allowSyntheticDefaultImports/esModuleInterop is enabled.
                return allowSyntheticDefaults ? 1 /* Default */ : 3 /* Equals */;
            }
            function getDefaultExportInfoWorker(defaultExport, moduleSymbol, checker, compilerOptions) {
                var localSymbol = ts.getLocalSymbolForExportDefault(defaultExport);
                if (localSymbol)
                    return { symbolForMeaning: localSymbol, name: localSymbol.name };
                var name = getNameForExportDefault(defaultExport);
                if (name !== undefined)
                    return { symbolForMeaning: defaultExport, name: name };
                if (defaultExport.flags & 2097152 /* Alias */) {
                    var aliased = checker.getImmediateAliasedSymbol(defaultExport);
                    return aliased && getDefaultExportInfoWorker(aliased, ts.Debug.assertDefined(aliased.parent, "Alias targets of default exports must have a parent"), checker, compilerOptions);
                }
                if (defaultExport.escapedName !== "default" /* Default */ &&
                    defaultExport.escapedName !== "export=" /* ExportEquals */) {
                    return { symbolForMeaning: defaultExport, name: defaultExport.getName() };
                }
                return { symbolForMeaning: defaultExport, name: moduleSymbolToValidIdentifier(moduleSymbol, compilerOptions.target) };
            }
            function getNameForExportDefault(symbol) {
                return symbol.declarations && ts.firstDefined(symbol.declarations, function (declaration) {
                    if (ts.isExportAssignment(declaration)) {
                        if (ts.isIdentifier(declaration.expression)) {
                            return declaration.expression.text;
                        }
                    }
                    else if (ts.isExportSpecifier(declaration)) {
                        ts.Debug.assert(declaration.name.text === "default" /* Default */, "Expected the specifier to be a default export");
                        return declaration.propertyName && declaration.propertyName.text;
                    }
                });
            }
            function codeActionForFix(context, sourceFile, symbolName, fix, quotePreference) {
                var diag;
                var changes = ts.textChanges.ChangeTracker.with(context, function (tracker) {
                    diag = codeActionForFixWorker(tracker, sourceFile, symbolName, fix, quotePreference);
                });
                return codefix.createCodeFixAction(codefix.importFixName, changes, diag, importFixId, ts.Diagnostics.Add_all_missing_imports);
            }
            function codeActionForFixWorker(changes, sourceFile, symbolName, fix, quotePreference) {
                switch (fix.kind) {
                    case 0 /* UseNamespace */:
                        addNamespaceQualifier(changes, sourceFile, fix);
                        return [ts.Diagnostics.Change_0_to_1, symbolName, fix.namespacePrefix + "." + symbolName];
                    case 1 /* ImportType */:
                        addImportType(changes, sourceFile, fix, quotePreference);
                        return [ts.Diagnostics.Change_0_to_1, symbolName, getImportTypePrefix(fix.moduleSpecifier, quotePreference) + symbolName];
                    case 2 /* AddToExisting */: {
                        var importClause = fix.importClause, importKind = fix.importKind, canUseTypeOnlyImport = fix.canUseTypeOnlyImport;
                        doAddExistingFix(changes, sourceFile, importClause, importKind === 1 /* Default */ ? symbolName : undefined, importKind === 0 /* Named */ ? [symbolName] : ts.emptyArray, canUseTypeOnlyImport);
                        var moduleSpecifierWithoutQuotes = ts.stripQuotes(importClause.parent.moduleSpecifier.getText());
                        return [importKind === 1 /* Default */ ? ts.Diagnostics.Add_default_import_0_to_existing_import_declaration_from_1 : ts.Diagnostics.Add_0_to_existing_import_declaration_from_1, symbolName, moduleSpecifierWithoutQuotes]; // you too!
                    }
                    case 3 /* AddNew */: {
                        var importKind = fix.importKind, moduleSpecifier = fix.moduleSpecifier;
                        addNewImports(changes, sourceFile, moduleSpecifier, quotePreference, importKind === 1 /* Default */ ? { defaultImport: symbolName, namedImports: ts.emptyArray, namespaceLikeImport: undefined }
                            : importKind === 0 /* Named */ ? { defaultImport: undefined, namedImports: [symbolName], namespaceLikeImport: undefined }
                                : { defaultImport: undefined, namedImports: ts.emptyArray, namespaceLikeImport: { importKind: importKind, name: symbolName } }, /*blankLineBetween*/ true);
                        return [importKind === 1 /* Default */ ? ts.Diagnostics.Import_default_0_from_module_1 : ts.Diagnostics.Import_0_from_module_1, symbolName, moduleSpecifier];
                    }
                    default:
                        return ts.Debug.assertNever(fix, "Unexpected fix kind " + fix.kind);
                }
            }
            function doAddExistingFix(changes, sourceFile, clause, defaultImport, namedImports, canUseTypeOnlyImport) {
                var convertTypeOnlyToRegular = !canUseTypeOnlyImport && clause.isTypeOnly;
                if (defaultImport) {
                    ts.Debug.assert(!clause.name, "Cannot add a default import to an import clause that already has one");
                    changes.insertNodeAt(sourceFile, clause.getStart(sourceFile), ts.createIdentifier(defaultImport), { suffix: ", " });
                }
                if (namedImports.length) {
                    var specifiers = namedImports.map(function (name) { return ts.createImportSpecifier(/*propertyName*/ undefined, ts.createIdentifier(name)); });
                    if (clause.namedBindings && ts.cast(clause.namedBindings, ts.isNamedImports).elements.length) {
                        for (var _i = 0, specifiers_1 = specifiers; _i < specifiers_1.length; _i++) {
                            var spec = specifiers_1[_i];
                            changes.insertNodeInListAfter(sourceFile, ts.last(ts.cast(clause.namedBindings, ts.isNamedImports).elements), spec);
                        }
                    }
                    else {
                        if (specifiers.length) {
                            var namedImports_1 = ts.createNamedImports(specifiers);
                            if (clause.namedBindings) {
                                changes.replaceNode(sourceFile, clause.namedBindings, namedImports_1);
                            }
                            else {
                                changes.insertNodeAfter(sourceFile, ts.Debug.assertDefined(clause.name, "Import clause must have either named imports or a default import"), namedImports_1);
                            }
                        }
                    }
                }
                if (convertTypeOnlyToRegular) {
                    changes.delete(sourceFile, ts.getTypeKeywordOfTypeOnlyImport(clause, sourceFile));
                }
            }
            function addNamespaceQualifier(changes, sourceFile, _a) {
                var namespacePrefix = _a.namespacePrefix, position = _a.position;
                changes.insertText(sourceFile, position, namespacePrefix + ".");
            }
            function addImportType(changes, sourceFile, _a, quotePreference) {
                var moduleSpecifier = _a.moduleSpecifier, position = _a.position;
                changes.insertText(sourceFile, position, getImportTypePrefix(moduleSpecifier, quotePreference));
            }
            function getImportTypePrefix(moduleSpecifier, quotePreference) {
                var quote = ts.getQuoteFromPreference(quotePreference);
                return "import(" + quote + moduleSpecifier + quote + ").";
            }
            function addNewImports(changes, sourceFile, moduleSpecifier, quotePreference, _a, blankLineBetween) {
                var defaultImport = _a.defaultImport, namedImports = _a.namedImports, namespaceLikeImport = _a.namespaceLikeImport;
                var quotedModuleSpecifier = ts.makeStringLiteral(moduleSpecifier, quotePreference);
                if (defaultImport !== undefined || namedImports.length) {
                    ts.insertImport(changes, sourceFile, ts.makeImport(defaultImport === undefined ? undefined : ts.createIdentifier(defaultImport), namedImports.map(function (n) { return ts.createImportSpecifier(/*propertyName*/ undefined, ts.createIdentifier(n)); }), moduleSpecifier, quotePreference), /*blankLineBetween*/ blankLineBetween);
                }
                if (namespaceLikeImport) {
                    ts.insertImport(changes, sourceFile, namespaceLikeImport.importKind === 3 /* Equals */ ? ts.createImportEqualsDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, ts.createIdentifier(namespaceLikeImport.name), ts.createExternalModuleReference(quotedModuleSpecifier)) :
                        namespaceLikeImport.importKind === 4 /* ConstEquals */ ? createConstEqualsRequireDeclaration(namespaceLikeImport.name, quotedModuleSpecifier) :
                            ts.createImportDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, ts.createImportClause(/*name*/ undefined, ts.createNamespaceImport(ts.createIdentifier(namespaceLikeImport.name))), quotedModuleSpecifier), /*blankLineBetween*/ blankLineBetween);
                }
            }
            function createConstEqualsRequireDeclaration(name, quotedModuleSpecifier) {
                return ts.createVariableStatement(/*modifiers*/ undefined, ts.createVariableDeclarationList([
                    ts.createVariableDeclaration(ts.createIdentifier(name), 
                    /*type*/ undefined, ts.createCall(ts.createIdentifier("require"), /*typeArguments*/ undefined, [quotedModuleSpecifier]))
                ], 2 /* Const */));
            }
            function symbolHasMeaning(_a, meaning) {
                var declarations = _a.declarations;
                return ts.some(declarations, function (decl) { return !!(ts.getMeaningFromDeclaration(decl) & meaning); });
            }
            function forEachExternalModuleToImportFrom(program, host, from, filterByPackageJson, cb) {
                var filteredCount = 0;
                var packageJson = filterByPackageJson && createAutoImportFilter(from, program, host);
                var allSourceFiles = program.getSourceFiles();
                var globalTypingsCache = host.getGlobalTypingsCacheLocation && host.getGlobalTypingsCacheLocation();
                forEachExternalModule(program.getTypeChecker(), allSourceFiles, function (module, sourceFile) {
                    if (sourceFile === undefined) {
                        if (!packageJson || packageJson.allowsImportingAmbientModule(module, allSourceFiles)) {
                            cb(module);
                        }
                        else if (packageJson) {
                            filteredCount++;
                        }
                    }
                    else if (sourceFile &&
                        sourceFile !== from &&
                        isImportablePath(from.fileName, sourceFile.fileName, ts.hostGetCanonicalFileName(host), globalTypingsCache)) {
                        if (!packageJson || packageJson.allowsImportingSourceFile(sourceFile, allSourceFiles)) {
                            cb(module);
                        }
                        else if (packageJson) {
                            filteredCount++;
                        }
                    }
                });
                if (host.log) {
                    host.log("forEachExternalModuleToImportFrom: filtered out " + filteredCount + " modules by package.json contents");
                }
            }
            codefix.forEachExternalModuleToImportFrom = forEachExternalModuleToImportFrom;
            function forEachExternalModule(checker, allSourceFiles, cb) {
                for (var _i = 0, _a = checker.getAmbientModules(); _i < _a.length; _i++) {
                    var ambient = _a[_i];
                    cb(ambient, /*sourceFile*/ undefined);
                }
                for (var _b = 0, allSourceFiles_1 = allSourceFiles; _b < allSourceFiles_1.length; _b++) {
                    var sourceFile = allSourceFiles_1[_b];
                    if (ts.isExternalOrCommonJsModule(sourceFile)) {
                        cb(checker.getMergedSymbol(sourceFile.symbol), sourceFile);
                    }
                }
            }
            /**
             * Don't include something from a `node_modules` that isn't actually reachable by a global import.
             * A relative import to node_modules is usually a bad idea.
             */
            function isImportablePath(fromPath, toPath, getCanonicalFileName, globalCachePath) {
                // If it's in a `node_modules` but is not reachable from here via a global import, don't bother.
                var toNodeModules = ts.forEachAncestorDirectory(toPath, function (ancestor) { return ts.getBaseFileName(ancestor) === "node_modules" ? ancestor : undefined; });
                var toNodeModulesParent = toNodeModules && ts.getDirectoryPath(getCanonicalFileName(toNodeModules));
                return toNodeModulesParent === undefined
                    || ts.startsWith(getCanonicalFileName(fromPath), toNodeModulesParent)
                    || (!!globalCachePath && ts.startsWith(getCanonicalFileName(globalCachePath), toNodeModulesParent));
            }
            function moduleSymbolToValidIdentifier(moduleSymbol, target) {
                return moduleSpecifierToValidIdentifier(ts.removeFileExtension(ts.stripQuotes(moduleSymbol.name)), target);
            }
            codefix.moduleSymbolToValidIdentifier = moduleSymbolToValidIdentifier;
            function moduleSpecifierToValidIdentifier(moduleSpecifier, target) {
                var baseName = ts.getBaseFileName(ts.removeSuffix(moduleSpecifier, "/index"));
                var res = "";
                var lastCharWasValid = true;
                var firstCharCode = baseName.charCodeAt(0);
                if (ts.isIdentifierStart(firstCharCode, target)) {
                    res += String.fromCharCode(firstCharCode);
                }
                else {
                    lastCharWasValid = false;
                }
                for (var i = 1; i < baseName.length; i++) {
                    var ch = baseName.charCodeAt(i);
                    var isValid = ts.isIdentifierPart(ch, target);
                    if (isValid) {
                        var char = String.fromCharCode(ch);
                        if (!lastCharWasValid) {
                            char = char.toUpperCase();
                        }
                        res += char;
                    }
                    lastCharWasValid = isValid;
                }
                // Need `|| "_"` to ensure result isn't empty.
                return !ts.isStringANonContextualKeyword(res) ? res || "_" : "_" + res;
            }
            codefix.moduleSpecifierToValidIdentifier = moduleSpecifierToValidIdentifier;
            function createAutoImportFilter(fromFile, program, host) {
                var packageJsons = host.getPackageJsonsVisibleToFile && host.getPackageJsonsVisibleToFile(fromFile.fileName) || ts.getPackageJsonsVisibleToFile(fromFile.fileName, host);
                var dependencyGroups = 1 /* Dependencies */ | 2 /* DevDependencies */ | 8 /* OptionalDependencies */;
                // Mix in `getProbablySymlinks` from Program when host doesn't have it
                // in order for non-Project hosts to have a symlinks cache.
                var moduleSpecifierResolutionHost = {
                    directoryExists: ts.maybeBind(host, host.directoryExists),
                    fileExists: ts.maybeBind(host, host.fileExists),
                    getCurrentDirectory: ts.maybeBind(host, host.getCurrentDirectory),
                    readFile: ts.maybeBind(host, host.readFile),
                    useCaseSensitiveFileNames: ts.maybeBind(host, host.useCaseSensitiveFileNames),
                    getProbableSymlinks: ts.maybeBind(host, host.getProbableSymlinks) || program.getProbableSymlinks,
                    getGlobalTypingsCacheLocation: ts.maybeBind(host, host.getGlobalTypingsCacheLocation),
                };
                var usesNodeCoreModules;
                return { allowsImportingAmbientModule: allowsImportingAmbientModule, allowsImportingSourceFile: allowsImportingSourceFile, allowsImportingSpecifier: allowsImportingSpecifier };
                function moduleSpecifierIsCoveredByPackageJson(specifier) {
                    var packageName = getNodeModuleRootSpecifier(specifier);
                    for (var _i = 0, packageJsons_1 = packageJsons; _i < packageJsons_1.length; _i++) {
                        var packageJson = packageJsons_1[_i];
                        if (packageJson.has(packageName, dependencyGroups) || packageJson.has(ts.getTypesPackageName(packageName), dependencyGroups)) {
                            return true;
                        }
                    }
                    return false;
                }
                function allowsImportingAmbientModule(moduleSymbol, allSourceFiles) {
                    if (!packageJsons.length) {
                        return true;
                    }
                    var declaringSourceFile = moduleSymbol.valueDeclaration.getSourceFile();
                    var declaringNodeModuleName = getNodeModulesPackageNameFromFileName(declaringSourceFile.fileName, allSourceFiles);
                    if (typeof declaringNodeModuleName === "undefined") {
                        return true;
                    }
                    var declaredModuleSpecifier = ts.stripQuotes(moduleSymbol.getName());
                    if (isAllowedCoreNodeModulesImport(declaredModuleSpecifier)) {
                        return true;
                    }
                    return moduleSpecifierIsCoveredByPackageJson(declaringNodeModuleName)
                        || moduleSpecifierIsCoveredByPackageJson(declaredModuleSpecifier);
                }
                function allowsImportingSourceFile(sourceFile, allSourceFiles) {
                    if (!packageJsons.length) {
                        return true;
                    }
                    var moduleSpecifier = getNodeModulesPackageNameFromFileName(sourceFile.fileName, allSourceFiles);
                    if (!moduleSpecifier) {
                        return true;
                    }
                    return moduleSpecifierIsCoveredByPackageJson(moduleSpecifier);
                }
                /**
                 * Use for a specific module specifier that has already been resolved.
                 * Use `allowsImportingAmbientModule` or `allowsImportingSourceFile` to resolve
                 * the best module specifier for a given module _and_ determine if it’s importable.
                 */
                function allowsImportingSpecifier(moduleSpecifier) {
                    if (!packageJsons.length || isAllowedCoreNodeModulesImport(moduleSpecifier)) {
                        return true;
                    }
                    if (ts.pathIsRelative(moduleSpecifier) || ts.isRootedDiskPath(moduleSpecifier)) {
                        return true;
                    }
                    return moduleSpecifierIsCoveredByPackageJson(moduleSpecifier);
                }
                function isAllowedCoreNodeModulesImport(moduleSpecifier) {
                    // If we’re in JavaScript, it can be difficult to tell whether the user wants to import
                    // from Node core modules or not. We can start by seeing if the user is actually using
                    // any node core modules, as opposed to simply having @types/node accidentally as a
                    // dependency of a dependency.
                    if (ts.isSourceFileJS(fromFile) && ts.JsTyping.nodeCoreModules.has(moduleSpecifier)) {
                        if (usesNodeCoreModules === undefined) {
                            usesNodeCoreModules = ts.consumesNodeCoreModules(fromFile);
                        }
                        if (usesNodeCoreModules) {
                            return true;
                        }
                    }
                    return false;
                }
                function getNodeModulesPackageNameFromFileName(importedFileName, allSourceFiles) {
                    if (!ts.stringContains(importedFileName, "node_modules")) {
                        return undefined;
                    }
                    var specifier = ts.moduleSpecifiers.getNodeModulesPackageName(host.getCompilationSettings(), fromFile.path, importedFileName, moduleSpecifierResolutionHost, allSourceFiles, program.redirectTargetsMap);
                    if (!specifier) {
                        return undefined;
                    }
                    // Paths here are not node_modules, so we don’t care about them;
                    // returning anything will trigger a lookup in package.json.
                    if (!ts.pathIsRelative(specifier) && !ts.isRootedDiskPath(specifier)) {
                        return getNodeModuleRootSpecifier(specifier);
                    }
                }
                function getNodeModuleRootSpecifier(fullSpecifier) {
                    var components = ts.getPathComponents(ts.getPackageNameFromTypesPackageName(fullSpecifier)).slice(1);
                    // Scoped packages
                    if (ts.startsWith(components[0], "@")) {
                        return components[0] + "/" + components[1];
                    }
                    return components[0];
                }
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "fixSpelling";
            var errorCodes = [
                ts.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2.code,
                ts.Diagnostics.Cannot_find_name_0_Did_you_mean_1.code,
                ts.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0.code,
                ts.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0.code,
                ts.Diagnostics.Module_0_has_no_exported_member_1_Did_you_mean_2.code,
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile;
                    var info = getInfo(sourceFile, context.span.start, context);
                    if (!info)
                        return undefined;
                    var node = info.node, suggestedSymbol = info.suggestedSymbol;
                    var target = context.host.getCompilationSettings().target;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, sourceFile, node, suggestedSymbol, target); });
                    return [codefix.createCodeFixAction("spelling", changes, [ts.Diagnostics.Change_spelling_to_0, ts.symbolName(suggestedSymbol)], fixId, ts.Diagnostics.Fix_all_detected_spelling_errors)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var info = getInfo(diag.file, diag.start, context);
                        var target = context.host.getCompilationSettings().target;
                        if (info)
                            doChange(changes, context.sourceFile, info.node, info.suggestedSymbol, target);
                    });
                },
            });
            function getInfo(sourceFile, pos, context) {
                // This is the identifier of the misspelled word. eg:
                // this.speling = 1;
                //      ^^^^^^^
                var node = ts.getTokenAtPosition(sourceFile, pos);
                var checker = context.program.getTypeChecker();
                var suggestedSymbol;
                if (ts.isPropertyAccessExpression(node.parent) && node.parent.name === node) {
                    ts.Debug.assert(ts.isIdentifierOrPrivateIdentifier(node), "Expected an identifier for spelling (property access)");
                    var containingType = checker.getTypeAtLocation(node.parent.expression);
                    if (node.parent.flags & 32 /* OptionalChain */) {
                        containingType = checker.getNonNullableType(containingType);
                    }
                    var name = node;
                    suggestedSymbol = checker.getSuggestedSymbolForNonexistentProperty(name, containingType);
                }
                else if (ts.isImportSpecifier(node.parent) && node.parent.name === node) {
                    ts.Debug.assert(node.kind === 75 /* Identifier */, "Expected an identifier for spelling (import)");
                    var importDeclaration = ts.findAncestor(node, ts.isImportDeclaration);
                    var resolvedSourceFile = getResolvedSourceFileFromImportDeclaration(sourceFile, context, importDeclaration);
                    if (resolvedSourceFile && resolvedSourceFile.symbol) {
                        suggestedSymbol = checker.getSuggestedSymbolForNonexistentModule(node, resolvedSourceFile.symbol);
                    }
                }
                else {
                    var meaning = ts.getMeaningFromLocation(node);
                    var name = ts.getTextOfNode(node);
                    ts.Debug.assert(name !== undefined, "name should be defined");
                    suggestedSymbol = checker.getSuggestedSymbolForNonexistentSymbol(node, name, convertSemanticMeaningToSymbolFlags(meaning));
                }
                return suggestedSymbol === undefined ? undefined : { node: node, suggestedSymbol: suggestedSymbol };
            }
            function doChange(changes, sourceFile, node, suggestedSymbol, target) {
                var suggestion = ts.symbolName(suggestedSymbol);
                if (!ts.isIdentifierText(suggestion, target) && ts.isPropertyAccessExpression(node.parent)) {
                    var valDecl = suggestedSymbol.valueDeclaration;
                    if (ts.isNamedDeclaration(valDecl) && ts.isPrivateIdentifier(valDecl.name)) {
                        changes.replaceNode(sourceFile, node, ts.createIdentifier(suggestion));
                    }
                    else {
                        changes.replaceNode(sourceFile, node.parent, ts.createElementAccess(node.parent.expression, ts.createLiteral(suggestion)));
                    }
                }
                else {
                    changes.replaceNode(sourceFile, node, ts.createIdentifier(suggestion));
                }
            }
            function convertSemanticMeaningToSymbolFlags(meaning) {
                var flags = 0;
                if (meaning & 4 /* Namespace */) {
                    flags |= 1920 /* Namespace */;
                }
                if (meaning & 2 /* Type */) {
                    flags |= 788968 /* Type */;
                }
                if (meaning & 1 /* Value */) {
                    flags |= 111551 /* Value */;
                }
                return flags;
            }
            function getResolvedSourceFileFromImportDeclaration(sourceFile, context, importDeclaration) {
                if (!importDeclaration || !ts.isStringLiteralLike(importDeclaration.moduleSpecifier))
                    return undefined;
                var resolvedModule = ts.getResolvedModule(sourceFile, importDeclaration.moduleSpecifier.text);
                if (!resolvedModule)
                    return undefined;
                return context.program.getSourceFile(resolvedModule.resolvedFileName);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixName = "addMissingMember";
            var errorCodes = [
                ts.Diagnostics.Property_0_does_not_exist_on_type_1.code,
                ts.Diagnostics.Property_0_does_not_exist_on_type_1_Did_you_mean_2.code,
                ts.Diagnostics.Property_0_is_missing_in_type_1_but_required_in_type_2.code,
                ts.Diagnostics.Type_0_is_missing_the_following_properties_from_type_1_Colon_2.code,
                ts.Diagnostics.Type_0_is_missing_the_following_properties_from_type_1_Colon_2_and_3_more.code
            ];
            var fixId = "addMissingMember";
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var info = getInfo(context.sourceFile, context.span.start, context.program.getTypeChecker(), context.program);
                    if (!info)
                        return undefined;
                    if (info.kind === 0 /* Enum */) {
                        var token_1 = info.token, parentDeclaration_1 = info.parentDeclaration;
                        var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return addEnumMemberDeclaration(t, context.program.getTypeChecker(), token_1, parentDeclaration_1); });
                        return [codefix.createCodeFixAction(fixName, changes, [ts.Diagnostics.Add_missing_enum_member_0, token_1.text], fixId, ts.Diagnostics.Add_all_missing_members)];
                    }
                    var parentDeclaration = info.parentDeclaration, declSourceFile = info.declSourceFile, inJs = info.inJs, makeStatic = info.makeStatic, token = info.token, call = info.call;
                    var methodCodeAction = call && getActionForMethodDeclaration(context, declSourceFile, parentDeclaration, token, call, makeStatic, inJs, context.preferences);
                    var addMember = inJs && !ts.isInterfaceDeclaration(parentDeclaration) ?
                        ts.singleElementArray(getActionsForAddMissingMemberInJavascriptFile(context, declSourceFile, parentDeclaration, token, makeStatic)) :
                        getActionsForAddMissingMemberInTypeScriptFile(context, declSourceFile, parentDeclaration, token, makeStatic);
                    return ts.concatenate(ts.singleElementArray(methodCodeAction), addMember);
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    var program = context.program, preferences = context.preferences;
                    var checker = program.getTypeChecker();
                    var seen = ts.createMap();
                    var typeDeclToMembers = new ts.NodeMap();
                    return codefix.createCombinedCodeActions(ts.textChanges.ChangeTracker.with(context, function (changes) {
                        codefix.eachDiagnostic(context, errorCodes, function (diag) {
                            var info = getInfo(diag.file, diag.start, checker, context.program);
                            if (!info || !ts.addToSeen(seen, ts.getNodeId(info.parentDeclaration) + "#" + info.token.text)) {
                                return;
                            }
                            if (info.kind === 0 /* Enum */) {
                                var token = info.token, parentDeclaration = info.parentDeclaration;
                                addEnumMemberDeclaration(changes, checker, token, parentDeclaration);
                            }
                            else {
                                var parentDeclaration = info.parentDeclaration, token_2 = info.token;
                                var infos = typeDeclToMembers.getOrUpdate(parentDeclaration, function () { return []; });
                                if (!infos.some(function (i) { return i.token.text === token_2.text; }))
                                    infos.push(info);
                            }
                        });
                        typeDeclToMembers.forEach(function (infos, classDeclaration) {
                            var supers = getAllSupers(classDeclaration, checker);
                            var _loop_14 = function (info) {
                                // If some superclass added this property, don't add it again.
                                if (supers.some(function (superClassOrInterface) {
                                    var superInfos = typeDeclToMembers.get(superClassOrInterface);
                                    return !!superInfos && superInfos.some(function (_a) {
                                        var token = _a.token;
                                        return token.text === info.token.text;
                                    });
                                }))
                                    return "continue";
                                var parentDeclaration = info.parentDeclaration, declSourceFile = info.declSourceFile, inJs = info.inJs, makeStatic = info.makeStatic, token = info.token, call = info.call;
                                // Always prefer to add a method declaration if possible.
                                if (call && !ts.isPrivateIdentifier(token)) {
                                    addMethodDeclaration(context, changes, declSourceFile, parentDeclaration, token, call, makeStatic, inJs, preferences);
                                }
                                else {
                                    if (inJs && !ts.isInterfaceDeclaration(parentDeclaration)) {
                                        addMissingMemberInJs(changes, declSourceFile, parentDeclaration, token, makeStatic);
                                    }
                                    else {
                                        var typeNode = getTypeNode(program.getTypeChecker(), parentDeclaration, token);
                                        addPropertyDeclaration(changes, declSourceFile, parentDeclaration, token.text, typeNode, makeStatic);
                                    }
                                }
                            };
                            for (var _i = 0, infos_1 = infos; _i < infos_1.length; _i++) {
                                var info = infos_1[_i];
                                _loop_14(info);
                            }
                        });
                    }));
                },
            });
            function getAllSupers(decl, checker) {
                var res = [];
                while (decl) {
                    var superElement = ts.getClassExtendsHeritageElement(decl);
                    var superSymbol = superElement && checker.getSymbolAtLocation(superElement.expression);
                    var superDecl = superSymbol && ts.find(superSymbol.declarations, ts.isClassLike);
                    if (superDecl) {
                        res.push(superDecl);
                    }
                    decl = superDecl;
                }
                return res;
            }
            var InfoKind;
            (function (InfoKind) {
                InfoKind[InfoKind["Enum"] = 0] = "Enum";
                InfoKind[InfoKind["ClassOrInterface"] = 1] = "ClassOrInterface";
            })(InfoKind || (InfoKind = {}));
            function getInfo(tokenSourceFile, tokenPos, checker, program) {
                // The identifier of the missing property. eg:
                // this.missing = 1;
                //      ^^^^^^^
                var token = ts.getTokenAtPosition(tokenSourceFile, tokenPos);
                if (!ts.isIdentifier(token) && !ts.isPrivateIdentifier(token)) {
                    return undefined;
                }
                var parent = token.parent;
                if (!ts.isPropertyAccessExpression(parent))
                    return undefined;
                var leftExpressionType = ts.skipConstraint(checker.getTypeAtLocation(parent.expression));
                var symbol = leftExpressionType.symbol;
                if (!symbol || !symbol.declarations)
                    return undefined;
                var isClass = ts.find(symbol.declarations, ts.isClassLike);
                // Don't suggest adding private identifiers to anything other than a class.
                if (!isClass && ts.isPrivateIdentifier(token)) {
                    return undefined;
                }
                // Prefer to change the class instead of the interface if they are merged
                var classOrInterface = isClass || ts.find(symbol.declarations, ts.isInterfaceDeclaration);
                if (classOrInterface && !program.isSourceFileFromExternalLibrary(classOrInterface.getSourceFile())) {
                    var makeStatic = (leftExpressionType.target || leftExpressionType) !== checker.getDeclaredTypeOfSymbol(symbol);
                    // Static private identifier properties are not supported yet.
                    if (makeStatic && ts.isPrivateIdentifier(token))
                        return undefined;
                    var declSourceFile = classOrInterface.getSourceFile();
                    var inJs = ts.isSourceFileJS(declSourceFile);
                    var call = ts.tryCast(parent.parent, ts.isCallExpression);
                    return { kind: 1 /* ClassOrInterface */, token: token, parentDeclaration: classOrInterface, makeStatic: makeStatic, declSourceFile: declSourceFile, inJs: inJs, call: call };
                }
                var enumDeclaration = ts.find(symbol.declarations, ts.isEnumDeclaration);
                if (enumDeclaration && !ts.isPrivateIdentifier(token) && !program.isSourceFileFromExternalLibrary(enumDeclaration.getSourceFile())) {
                    return { kind: 0 /* Enum */, token: token, parentDeclaration: enumDeclaration };
                }
                return undefined;
            }
            function getActionsForAddMissingMemberInJavascriptFile(context, declSourceFile, classDeclaration, token, makeStatic) {
                var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return addMissingMemberInJs(t, declSourceFile, classDeclaration, token, makeStatic); });
                if (changes.length === 0) {
                    return undefined;
                }
                var diagnostic = makeStatic ? ts.Diagnostics.Initialize_static_property_0 :
                    ts.isPrivateIdentifier(token) ? ts.Diagnostics.Declare_a_private_field_named_0 : ts.Diagnostics.Initialize_property_0_in_the_constructor;
                return codefix.createCodeFixAction(fixName, changes, [diagnostic, token.text], fixId, ts.Diagnostics.Add_all_missing_members);
            }
            function addMissingMemberInJs(changeTracker, declSourceFile, classDeclaration, token, makeStatic) {
                var tokenName = token.text;
                if (makeStatic) {
                    if (classDeclaration.kind === 214 /* ClassExpression */) {
                        return;
                    }
                    var className = classDeclaration.name.getText();
                    var staticInitialization = initializePropertyToUndefined(ts.createIdentifier(className), tokenName);
                    changeTracker.insertNodeAfter(declSourceFile, classDeclaration, staticInitialization);
                }
                else if (ts.isPrivateIdentifier(token)) {
                    var property = ts.createProperty(
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, tokenName, 
                    /*questionToken*/ undefined, 
                    /*type*/ undefined, 
                    /*initializer*/ undefined);
                    var lastProp = getNodeToInsertPropertyAfter(classDeclaration);
                    if (lastProp) {
                        changeTracker.insertNodeAfter(declSourceFile, lastProp, property);
                    }
                    else {
                        changeTracker.insertNodeAtClassStart(declSourceFile, classDeclaration, property);
                    }
                }
                else {
                    var classConstructor = ts.getFirstConstructorWithBody(classDeclaration);
                    if (!classConstructor) {
                        return;
                    }
                    var propertyInitialization = initializePropertyToUndefined(ts.createThis(), tokenName);
                    changeTracker.insertNodeAtConstructorEnd(declSourceFile, classConstructor, propertyInitialization);
                }
            }
            function initializePropertyToUndefined(obj, propertyName) {
                return ts.createStatement(ts.createAssignment(ts.createPropertyAccess(obj, propertyName), ts.createIdentifier("undefined")));
            }
            function getActionsForAddMissingMemberInTypeScriptFile(context, declSourceFile, classDeclaration, token, makeStatic) {
                var typeNode = getTypeNode(context.program.getTypeChecker(), classDeclaration, token);
                var addProp = createAddPropertyDeclarationAction(context, declSourceFile, classDeclaration, makeStatic, token.text, typeNode);
                return makeStatic || ts.isPrivateIdentifier(token) ? [addProp] : [addProp, createAddIndexSignatureAction(context, declSourceFile, classDeclaration, token.text, typeNode)];
            }
            function getTypeNode(checker, classDeclaration, token) {
                var typeNode;
                if (token.parent.parent.kind === 209 /* BinaryExpression */) {
                    var binaryExpression = token.parent.parent;
                    var otherExpression = token.parent === binaryExpression.left ? binaryExpression.right : binaryExpression.left;
                    var widenedType = checker.getWidenedType(checker.getBaseTypeOfLiteralType(checker.getTypeAtLocation(otherExpression)));
                    typeNode = checker.typeToTypeNode(widenedType, classDeclaration);
                }
                else {
                    var contextualType = checker.getContextualType(token.parent);
                    typeNode = contextualType ? checker.typeToTypeNode(contextualType) : undefined;
                }
                return typeNode || ts.createKeywordTypeNode(125 /* AnyKeyword */);
            }
            function createAddPropertyDeclarationAction(context, declSourceFile, classDeclaration, makeStatic, tokenName, typeNode) {
                var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return addPropertyDeclaration(t, declSourceFile, classDeclaration, tokenName, typeNode, makeStatic); });
                return codefix.createCodeFixAction(fixName, changes, [makeStatic ? ts.Diagnostics.Declare_static_property_0 : ts.Diagnostics.Declare_property_0, tokenName], fixId, ts.Diagnostics.Add_all_missing_members);
            }
            function addPropertyDeclaration(changeTracker, declSourceFile, classDeclaration, tokenName, typeNode, makeStatic) {
                var property = ts.createProperty(
                /*decorators*/ undefined, 
                /*modifiers*/ makeStatic ? [ts.createToken(120 /* StaticKeyword */)] : undefined, tokenName, 
                /*questionToken*/ undefined, typeNode, 
                /*initializer*/ undefined);
                var lastProp = getNodeToInsertPropertyAfter(classDeclaration);
                if (lastProp) {
                    changeTracker.insertNodeAfter(declSourceFile, lastProp, property);
                }
                else {
                    changeTracker.insertNodeAtClassStart(declSourceFile, classDeclaration, property);
                }
            }
            // Gets the last of the first run of PropertyDeclarations, or undefined if the class does not start with a PropertyDeclaration.
            function getNodeToInsertPropertyAfter(cls) {
                var res;
                for (var _i = 0, _a = cls.members; _i < _a.length; _i++) {
                    var member = _a[_i];
                    if (!ts.isPropertyDeclaration(member))
                        break;
                    res = member;
                }
                return res;
            }
            function createAddIndexSignatureAction(context, declSourceFile, classDeclaration, tokenName, typeNode) {
                // Index signatures cannot have the static modifier.
                var stringTypeNode = ts.createKeywordTypeNode(143 /* StringKeyword */);
                var indexingParameter = ts.createParameter(
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, 
                /*dotDotDotToken*/ undefined, "x", 
                /*questionToken*/ undefined, stringTypeNode, 
                /*initializer*/ undefined);
                var indexSignature = ts.createIndexSignature(
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, [indexingParameter], typeNode);
                var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return t.insertNodeAtClassStart(declSourceFile, classDeclaration, indexSignature); });
                // No fixId here because code-fix-all currently only works on adding individual named properties.
                return codefix.createCodeFixActionWithoutFixAll(fixName, changes, [ts.Diagnostics.Add_index_signature_for_property_0, tokenName]);
            }
            function getActionForMethodDeclaration(context, declSourceFile, classDeclaration, token, callExpression, makeStatic, inJs, preferences) {
                // Private methods are not implemented yet.
                if (ts.isPrivateIdentifier(token)) {
                    return undefined;
                }
                var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return addMethodDeclaration(context, t, declSourceFile, classDeclaration, token, callExpression, makeStatic, inJs, preferences); });
                return codefix.createCodeFixAction(fixName, changes, [makeStatic ? ts.Diagnostics.Declare_static_method_0 : ts.Diagnostics.Declare_method_0, token.text], fixId, ts.Diagnostics.Add_all_missing_members);
            }
            function addMethodDeclaration(context, changeTracker, declSourceFile, typeDecl, token, callExpression, makeStatic, inJs, preferences) {
                var methodDeclaration = codefix.createMethodFromCallExpression(context, callExpression, token.text, inJs, makeStatic, preferences, typeDecl);
                var containingMethodDeclaration = ts.getAncestor(callExpression, 161 /* MethodDeclaration */);
                if (containingMethodDeclaration && containingMethodDeclaration.parent === typeDecl) {
                    changeTracker.insertNodeAfter(declSourceFile, containingMethodDeclaration, methodDeclaration);
                }
                else {
                    changeTracker.insertNodeAtClassStart(declSourceFile, typeDecl, methodDeclaration);
                }
            }
            function addEnumMemberDeclaration(changes, checker, token, enumDeclaration) {
                /**
                 * create initializer only literal enum that has string initializer.
                 * value of initializer is a string literal that equal to name of enum member.
                 * numeric enum or empty enum will not create initializer.
                 */
                var hasStringInitializer = ts.some(enumDeclaration.members, function (member) {
                    var type = checker.getTypeAtLocation(member);
                    return !!(type && type.flags & 132 /* StringLike */);
                });
                var enumMember = ts.createEnumMember(token, hasStringInitializer ? ts.createStringLiteral(token.text) : undefined);
                changes.replaceNode(enumDeclaration.getSourceFile(), enumDeclaration, ts.updateEnumDeclaration(enumDeclaration, enumDeclaration.decorators, enumDeclaration.modifiers, enumDeclaration.name, ts.concatenate(enumDeclaration.members, ts.singleElementArray(enumMember))));
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "addMissingNewOperator";
            var errorCodes = [ts.Diagnostics.Value_of_type_0_is_not_callable_Did_you_mean_to_include_new.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, span = context.span;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return addMissingNewOperator(t, sourceFile, span); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Add_missing_new_operator_to_call, fixId, ts.Diagnostics.Add_missing_new_operator_to_all_calls)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        return addMissingNewOperator(changes, context.sourceFile, diag);
                    });
                },
            });
            function addMissingNewOperator(changes, sourceFile, span) {
                var call = ts.cast(findAncestorMatchingSpan(sourceFile, span), ts.isCallExpression);
                var newExpression = ts.createNew(call.expression, call.typeArguments, call.arguments);
                changes.replaceNode(sourceFile, call, newExpression);
            }
            function findAncestorMatchingSpan(sourceFile, span) {
                var token = ts.getTokenAtPosition(sourceFile, span.start);
                var end = ts.textSpanEnd(span);
                while (token.end < end) {
                    token = token.parent;
                }
                return token;
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixName = "fixCannotFindModule";
            var fixIdInstallTypesPackage = "installTypesPackage";
            var errorCodeCannotFindModule = ts.Diagnostics.Cannot_find_module_0.code;
            var errorCodes = [
                errorCodeCannotFindModule,
                ts.Diagnostics.Could_not_find_a_declaration_file_for_module_0_1_implicitly_has_an_any_type.code,
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var host = context.host, sourceFile = context.sourceFile, start = context.span.start;
                    var packageName = tryGetImportedPackageName(sourceFile, start);
                    if (packageName === undefined)
                        return undefined;
                    var typesPackageName = getTypesPackageNameToInstall(packageName, host, context.errorCode);
                    return typesPackageName === undefined
                        ? []
                        : [codefix.createCodeFixAction(fixName, /*changes*/ [], [ts.Diagnostics.Install_0, typesPackageName], fixIdInstallTypesPackage, ts.Diagnostics.Install_all_missing_types_packages, getInstallCommand(sourceFile.fileName, typesPackageName))];
                },
                fixIds: [fixIdInstallTypesPackage],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (_changes, diag, commands) {
                        var packageName = tryGetImportedPackageName(diag.file, diag.start);
                        if (packageName === undefined)
                            return undefined;
                        switch (context.fixId) {
                            case fixIdInstallTypesPackage: {
                                var pkg = getTypesPackageNameToInstall(packageName, context.host, diag.code);
                                if (pkg) {
                                    commands.push(getInstallCommand(diag.file.fileName, pkg));
                                }
                                break;
                            }
                            default:
                                ts.Debug.fail("Bad fixId: " + context.fixId);
                        }
                    });
                },
            });
            function getInstallCommand(fileName, packageName) {
                return { type: "install package", file: fileName, packageName: packageName };
            }
            function tryGetImportedPackageName(sourceFile, pos) {
                var moduleName = ts.cast(ts.getTokenAtPosition(sourceFile, pos), ts.isStringLiteral).text;
                var packageName = ts.parsePackageName(moduleName).packageName;
                return ts.isExternalModuleNameRelative(packageName) ? undefined : packageName;
            }
            function getTypesPackageNameToInstall(packageName, host, diagCode) {
                return diagCode === errorCodeCannotFindModule
                    ? (ts.JsTyping.nodeCoreModules.has(packageName) ? "@types/node" : undefined)
                    : (host.isKnownTypesPackageName(packageName) ? ts.getTypesPackageName(packageName) : undefined); // TODO: GH#18217
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var errorCodes = [
                ts.Diagnostics.Non_abstract_class_0_does_not_implement_inherited_abstract_member_1_from_class_2.code,
                ts.Diagnostics.Non_abstract_class_expression_does_not_implement_inherited_abstract_member_0_from_class_1.code,
            ];
            var fixId = "fixClassDoesntImplementInheritedAbstractMember";
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, span = context.span;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) {
                        return addMissingMembers(getClass(sourceFile, span.start), sourceFile, context, t, context.preferences);
                    });
                    return changes.length === 0 ? undefined : [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Implement_inherited_abstract_class, fixId, ts.Diagnostics.Implement_all_inherited_abstract_classes)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    var seenClassDeclarations = ts.createMap();
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var classDeclaration = getClass(diag.file, diag.start);
                        if (ts.addToSeen(seenClassDeclarations, ts.getNodeId(classDeclaration))) {
                            addMissingMembers(classDeclaration, context.sourceFile, context, changes, context.preferences);
                        }
                    });
                },
            });
            function getClass(sourceFile, pos) {
                // Token is the identifier in the case of a class declaration
                // or the class keyword token in the case of a class expression.
                var token = ts.getTokenAtPosition(sourceFile, pos);
                return ts.cast(token.parent, ts.isClassLike);
            }
            function addMissingMembers(classDeclaration, sourceFile, context, changeTracker, preferences) {
                var extendsNode = ts.getEffectiveBaseTypeNode(classDeclaration);
                var checker = context.program.getTypeChecker();
                var instantiatedExtendsType = checker.getTypeAtLocation(extendsNode);
                // Note that this is ultimately derived from a map indexed by symbol names,
                // so duplicates cannot occur.
                var abstractAndNonPrivateExtendsSymbols = checker.getPropertiesOfType(instantiatedExtendsType).filter(symbolPointsToNonPrivateAndAbstractMember);
                codefix.createMissingMemberNodes(classDeclaration, abstractAndNonPrivateExtendsSymbols, context, preferences, function (member) { return changeTracker.insertNodeAtClassStart(sourceFile, classDeclaration, member); });
            }
            function symbolPointsToNonPrivateAndAbstractMember(symbol) {
                // See `codeFixClassExtendAbstractProtectedProperty.ts` in https://github.com/Microsoft/TypeScript/pull/11547/files
                // (now named `codeFixClassExtendAbstractPrivateProperty.ts`)
                var flags = ts.getModifierFlags(ts.first(symbol.getDeclarations()));
                return !(flags & 8 /* Private */) && !!(flags & 128 /* Abstract */);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "classSuperMustPrecedeThisAccess";
            var errorCodes = [ts.Diagnostics.super_must_be_called_before_accessing_this_in_the_constructor_of_a_derived_class.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, span = context.span;
                    var nodes = getNodes(sourceFile, span.start);
                    if (!nodes)
                        return undefined;
                    var constructor = nodes.constructor, superCall = nodes.superCall;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, sourceFile, constructor, superCall); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Make_super_call_the_first_statement_in_the_constructor, fixId, ts.Diagnostics.Make_all_super_calls_the_first_statement_in_their_constructor)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    var sourceFile = context.sourceFile;
                    var seenClasses = ts.createMap(); // Ensure we only do this once per class.
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var nodes = getNodes(diag.file, diag.start);
                        if (!nodes)
                            return;
                        var constructor = nodes.constructor, superCall = nodes.superCall;
                        if (ts.addToSeen(seenClasses, ts.getNodeId(constructor.parent))) {
                            doChange(changes, sourceFile, constructor, superCall);
                        }
                    });
                },
            });
            function doChange(changes, sourceFile, constructor, superCall) {
                changes.insertNodeAtConstructorStart(sourceFile, constructor, superCall);
                changes.delete(sourceFile, superCall);
            }
            function getNodes(sourceFile, pos) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                if (token.kind !== 104 /* ThisKeyword */)
                    return undefined;
                var constructor = ts.getContainingFunction(token);
                var superCall = findSuperCall(constructor.body);
                // figure out if the `this` access is actually inside the supercall
                // i.e. super(this.a), since in that case we won't suggest a fix
                return superCall && !superCall.expression.arguments.some(function (arg) { return ts.isPropertyAccessExpression(arg) && arg.expression === token; }) ? { constructor: constructor, superCall: superCall } : undefined;
            }
            function findSuperCall(n) {
                return ts.isExpressionStatement(n) && ts.isSuperCall(n.expression)
                    ? n
                    : ts.isFunctionLike(n)
                        ? undefined
                        : ts.forEachChild(n, findSuperCall);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "constructorForDerivedNeedSuperCall";
            var errorCodes = [ts.Diagnostics.Constructors_for_derived_classes_must_contain_a_super_call.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, span = context.span;
                    var ctr = getNode(sourceFile, span.start);
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, sourceFile, ctr); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Add_missing_super_call, fixId, ts.Diagnostics.Add_all_missing_super_calls)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        return doChange(changes, context.sourceFile, getNode(diag.file, diag.start));
                    });
                },
            });
            function getNode(sourceFile, pos) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                ts.Debug.assert(token.kind === 129 /* ConstructorKeyword */, "token should be at the constructor keyword");
                return token.parent;
            }
            function doChange(changes, sourceFile, ctr) {
                var superCall = ts.createStatement(ts.createCall(ts.createSuper(), /*typeArguments*/ undefined, /*argumentsArray*/ ts.emptyArray));
                changes.insertNodeAtConstructorStart(sourceFile, ctr, superCall);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "enableExperimentalDecorators";
            var errorCodes = [
                ts.Diagnostics.Experimental_support_for_decorators_is_a_feature_that_is_subject_to_change_in_a_future_release_Set_the_experimentalDecorators_option_in_your_tsconfig_or_jsconfig_to_remove_this_warning.code
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var configFile = context.program.getCompilerOptions().configFile;
                    if (configFile === undefined) {
                        return undefined;
                    }
                    var changes = ts.textChanges.ChangeTracker.with(context, function (changeTracker) { return doChange(changeTracker, configFile); });
                    return [codefix.createCodeFixActionWithoutFixAll(fixId, changes, ts.Diagnostics.Enable_the_experimentalDecorators_option_in_your_configuration_file)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes) {
                        var configFile = context.program.getCompilerOptions().configFile;
                        if (configFile === undefined) {
                            return undefined;
                        }
                        doChange(changes, configFile);
                    });
                },
            });
            function doChange(changeTracker, configFile) {
                codefix.setJsonCompilerOptionValue(changeTracker, configFile, "experimentalDecorators", ts.createTrue());
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixID = "fixEnableJsxFlag";
            var errorCodes = [ts.Diagnostics.Cannot_use_JSX_unless_the_jsx_flag_is_provided.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var configFile = context.program.getCompilerOptions().configFile;
                    if (configFile === undefined) {
                        return undefined;
                    }
                    var changes = ts.textChanges.ChangeTracker.with(context, function (changeTracker) {
                        return doChange(changeTracker, configFile);
                    });
                    return [
                        codefix.createCodeFixActionWithoutFixAll(fixID, changes, ts.Diagnostics.Enable_the_jsx_flag_in_your_configuration_file)
                    ];
                },
                fixIds: [fixID],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes) {
                        var configFile = context.program.getCompilerOptions().configFile;
                        if (configFile === undefined) {
                            return undefined;
                        }
                        doChange(changes, configFile);
                    });
                }
            });
            function doChange(changeTracker, configFile) {
                codefix.setJsonCompilerOptionValue(changeTracker, configFile, "jsx", ts.createStringLiteral("react"));
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            codefix.registerCodeFix({
                errorCodes: [ts.Diagnostics.Top_level_await_expressions_are_only_allowed_when_the_module_option_is_set_to_esnext_or_system_and_the_target_option_is_set_to_es2017_or_higher.code],
                getCodeActions: function (context) {
                    var compilerOptions = context.program.getCompilerOptions();
                    var configFile = compilerOptions.configFile;
                    if (configFile === undefined) {
                        return undefined;
                    }
                    var codeFixes = [];
                    var moduleKind = ts.getEmitModuleKind(compilerOptions);
                    var moduleOutOfRange = moduleKind >= ts.ModuleKind.ES2015 && moduleKind < ts.ModuleKind.ESNext;
                    if (moduleOutOfRange) {
                        var changes = ts.textChanges.ChangeTracker.with(context, function (changes) {
                            codefix.setJsonCompilerOptionValue(changes, configFile, "module", ts.createStringLiteral("esnext"));
                        });
                        codeFixes.push(codefix.createCodeFixActionWithoutFixAll("fixModuleOption", changes, [ts.Diagnostics.Set_the_module_option_in_your_configuration_file_to_0, "esnext"]));
                    }
                    var target = ts.getEmitScriptTarget(compilerOptions);
                    var targetOutOfRange = target < 4 /* ES2017 */ || target > 99 /* ESNext */;
                    if (targetOutOfRange) {
                        var changes = ts.textChanges.ChangeTracker.with(context, function (tracker) {
                            var configObject = ts.getTsConfigObjectLiteralExpression(configFile);
                            if (!configObject)
                                return;
                            var options = [["target", ts.createStringLiteral("es2017")]];
                            if (moduleKind === ts.ModuleKind.CommonJS) {
                                // Ensure we preserve the default module kind (commonjs), as targets >= ES2015 have a default module kind of es2015.
                                options.push(["module", ts.createStringLiteral("commonjs")]);
                            }
                            codefix.setJsonCompilerOptionValues(tracker, configFile, options);
                        });
                        codeFixes.push(codefix.createCodeFixActionWithoutFixAll("fixTargetOption", changes, [ts.Diagnostics.Set_the_target_option_in_your_configuration_file_to_0, "es2017"]));
                    }
                    return codeFixes.length ? codeFixes : undefined;
                }
            });
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "extendsInterfaceBecomesImplements";
            var errorCodes = [ts.Diagnostics.Cannot_extend_an_interface_0_Did_you_mean_implements.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile;
                    var nodes = getNodes(sourceFile, context.span.start);
                    if (!nodes)
                        return undefined;
                    var extendsToken = nodes.extendsToken, heritageClauses = nodes.heritageClauses;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChanges(t, sourceFile, extendsToken, heritageClauses); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Change_extends_to_implements, fixId, ts.Diagnostics.Change_all_extended_interfaces_to_implements)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var nodes = getNodes(diag.file, diag.start);
                        if (nodes)
                            doChanges(changes, diag.file, nodes.extendsToken, nodes.heritageClauses);
                    });
                },
            });
            function getNodes(sourceFile, pos) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                var heritageClauses = ts.getContainingClass(token).heritageClauses;
                var extendsToken = heritageClauses[0].getFirstToken();
                return extendsToken.kind === 90 /* ExtendsKeyword */ ? { extendsToken: extendsToken, heritageClauses: heritageClauses } : undefined;
            }
            function doChanges(changes, sourceFile, extendsToken, heritageClauses) {
                changes.replaceNode(sourceFile, extendsToken, ts.createToken(113 /* ImplementsKeyword */));
                // If there is already an implements clause, replace the implements keyword with a comma.
                if (heritageClauses.length === 2 &&
                    heritageClauses[0].token === 90 /* ExtendsKeyword */ &&
                    heritageClauses[1].token === 113 /* ImplementsKeyword */) {
                    var implementsToken = heritageClauses[1].getFirstToken();
                    var implementsFullStart = implementsToken.getFullStart();
                    changes.replaceRange(sourceFile, { pos: implementsFullStart, end: implementsFullStart }, ts.createToken(27 /* CommaToken */));
                    // Rough heuristic: delete trailing whitespace after keyword so that it's not excessive.
                    // (Trailing because leading might be indentation, which is more sensitive.)
                    var text = sourceFile.text;
                    var end = implementsToken.end;
                    while (end < text.length && ts.isWhiteSpaceSingleLine(text.charCodeAt(end))) {
                        end++;
                    }
                    changes.deleteRange(sourceFile, { pos: implementsToken.getStart(), end: end });
                }
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "forgottenThisPropertyAccess";
            var didYouMeanStaticMemberCode = ts.Diagnostics.Cannot_find_name_0_Did_you_mean_the_static_member_1_0.code;
            var errorCodes = [
                ts.Diagnostics.Cannot_find_name_0_Did_you_mean_the_instance_member_this_0.code,
                ts.Diagnostics.Private_identifiers_are_not_allowed_outside_class_bodies.code,
                didYouMeanStaticMemberCode,
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile;
                    var info = getInfo(sourceFile, context.span.start, context.errorCode);
                    if (!info) {
                        return undefined;
                    }
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, sourceFile, info); });
                    return [codefix.createCodeFixAction(fixId, changes, [ts.Diagnostics.Add_0_to_unresolved_variable, info.className || "this"], fixId, ts.Diagnostics.Add_qualifier_to_all_unresolved_variables_matching_a_member_name)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var info = getInfo(diag.file, diag.start, diag.code);
                        if (info)
                            doChange(changes, context.sourceFile, info);
                    });
                },
            });
            function getInfo(sourceFile, pos, diagCode) {
                var node = ts.getTokenAtPosition(sourceFile, pos);
                if (ts.isIdentifier(node)) {
                    return { node: node, className: diagCode === didYouMeanStaticMemberCode ? ts.getContainingClass(node).name.text : undefined };
                }
            }
            function doChange(changes, sourceFile, _a) {
                var node = _a.node, className = _a.className;
                // TODO (https://github.com/Microsoft/TypeScript/issues/21246): use shared helper
                ts.suppressLeadingAndTrailingTrivia(node);
                changes.replaceNode(sourceFile, node, ts.createPropertyAccess(className ? ts.createIdentifier(className) : ts.createThis(), node));
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixName = "unusedIdentifier";
            var fixIdPrefix = "unusedIdentifier_prefix";
            var fixIdDelete = "unusedIdentifier_delete";
            var fixIdInfer = "unusedIdentifier_infer";
            var errorCodes = [
                ts.Diagnostics._0_is_declared_but_its_value_is_never_read.code,
                ts.Diagnostics._0_is_declared_but_never_used.code,
                ts.Diagnostics.Property_0_is_declared_but_its_value_is_never_read.code,
                ts.Diagnostics.All_imports_in_import_declaration_are_unused.code,
                ts.Diagnostics.All_destructured_elements_are_unused.code,
                ts.Diagnostics.All_variables_are_unused.code,
                ts.Diagnostics.All_type_parameters_are_unused.code,
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var errorCode = context.errorCode, sourceFile = context.sourceFile, program = context.program;
                    var checker = program.getTypeChecker();
                    var sourceFiles = program.getSourceFiles();
                    var token = ts.getTokenAtPosition(sourceFile, context.span.start);
                    if (ts.isJSDocTemplateTag(token)) {
                        return [createDeleteFix(ts.textChanges.ChangeTracker.with(context, function (t) { return t.delete(sourceFile, token); }), ts.Diagnostics.Remove_template_tag)];
                    }
                    if (token.kind === 29 /* LessThanToken */) {
                        var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return deleteTypeParameters(t, sourceFile, token); });
                        return [createDeleteFix(changes, ts.Diagnostics.Remove_type_parameters)];
                    }
                    var importDecl = tryGetFullImport(token);
                    if (importDecl) {
                        var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return t.delete(sourceFile, importDecl); });
                        return [createDeleteFix(changes, [ts.Diagnostics.Remove_import_from_0, ts.showModuleSpecifier(importDecl)])];
                    }
                    var delDestructure = ts.textChanges.ChangeTracker.with(context, function (t) {
                        return tryDeleteFullDestructure(token, t, sourceFile, checker, sourceFiles, /*isFixAll*/ false);
                    });
                    if (delDestructure.length) {
                        return [createDeleteFix(delDestructure, ts.Diagnostics.Remove_destructuring)];
                    }
                    var delVar = ts.textChanges.ChangeTracker.with(context, function (t) { return tryDeleteFullVariableStatement(sourceFile, token, t); });
                    if (delVar.length) {
                        return [createDeleteFix(delVar, ts.Diagnostics.Remove_variable_statement)];
                    }
                    var result = [];
                    if (token.kind === 132 /* InferKeyword */) {
                        var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return changeInferToUnknown(t, sourceFile, token); });
                        var name = ts.cast(token.parent, ts.isInferTypeNode).typeParameter.name.text;
                        result.push(codefix.createCodeFixAction(fixName, changes, [ts.Diagnostics.Replace_infer_0_with_unknown, name], fixIdInfer, ts.Diagnostics.Replace_all_unused_infer_with_unknown));
                    }
                    else {
                        var deletion = ts.textChanges.ChangeTracker.with(context, function (t) {
                            return tryDeleteDeclaration(sourceFile, token, t, checker, sourceFiles, /*isFixAll*/ false);
                        });
                        if (deletion.length) {
                            var name = ts.isComputedPropertyName(token.parent) ? token.parent : token;
                            result.push(createDeleteFix(deletion, [ts.Diagnostics.Remove_unused_declaration_for_Colon_0, name.getText(sourceFile)]));
                        }
                    }
                    var prefix = ts.textChanges.ChangeTracker.with(context, function (t) { return tryPrefixDeclaration(t, errorCode, sourceFile, token); });
                    if (prefix.length) {
                        result.push(codefix.createCodeFixAction(fixName, prefix, [ts.Diagnostics.Prefix_0_with_an_underscore, token.getText(sourceFile)], fixIdPrefix, ts.Diagnostics.Prefix_all_unused_declarations_with_where_possible));
                    }
                    return result;
                },
                fixIds: [fixIdPrefix, fixIdDelete, fixIdInfer],
                getAllCodeActions: function (context) {
                    var sourceFile = context.sourceFile, program = context.program;
                    var checker = program.getTypeChecker();
                    var sourceFiles = program.getSourceFiles();
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var token = ts.getTokenAtPosition(sourceFile, diag.start);
                        switch (context.fixId) {
                            case fixIdPrefix:
                                tryPrefixDeclaration(changes, diag.code, sourceFile, token);
                                break;
                            case fixIdDelete: {
                                if (token.kind === 132 /* InferKeyword */)
                                    break; // Can't delete
                                var importDecl = tryGetFullImport(token);
                                if (importDecl) {
                                    changes.delete(sourceFile, importDecl);
                                }
                                else if (ts.isJSDocTemplateTag(token)) {
                                    changes.delete(sourceFile, token);
                                }
                                else if (token.kind === 29 /* LessThanToken */) {
                                    deleteTypeParameters(changes, sourceFile, token);
                                }
                                else if (!tryDeleteFullDestructure(token, changes, sourceFile, checker, sourceFiles, /*isFixAll*/ true) &&
                                    !tryDeleteFullVariableStatement(sourceFile, token, changes)) {
                                    tryDeleteDeclaration(sourceFile, token, changes, checker, sourceFiles, /*isFixAll*/ true);
                                }
                                break;
                            }
                            case fixIdInfer:
                                if (token.kind === 132 /* InferKeyword */) {
                                    changeInferToUnknown(changes, sourceFile, token);
                                }
                                break;
                            default:
                                ts.Debug.fail(JSON.stringify(context.fixId));
                        }
                    });
                },
            });
            function changeInferToUnknown(changes, sourceFile, token) {
                changes.replaceNode(sourceFile, token.parent, ts.createKeywordTypeNode(148 /* UnknownKeyword */));
            }
            function createDeleteFix(changes, diag) {
                return codefix.createCodeFixAction(fixName, changes, diag, fixIdDelete, ts.Diagnostics.Delete_all_unused_declarations);
            }
            function deleteTypeParameters(changes, sourceFile, token) {
                changes.delete(sourceFile, ts.Debug.assertDefined(ts.cast(token.parent, ts.isDeclarationWithTypeParameterChildren).typeParameters, "The type parameter to delete should exist"));
            }
            // Sometimes the diagnostic span is an entire ImportDeclaration, so we should remove the whole thing.
            function tryGetFullImport(token) {
                return token.kind === 96 /* ImportKeyword */ ? ts.tryCast(token.parent, ts.isImportDeclaration) : undefined;
            }
            function tryDeleteFullDestructure(token, changes, sourceFile, checker, sourceFiles, isFixAll) {
                if (token.kind !== 18 /* OpenBraceToken */ || !ts.isObjectBindingPattern(token.parent))
                    return false;
                var decl = token.parent.parent;
                if (decl.kind === 156 /* Parameter */) {
                    tryDeleteParameter(changes, sourceFile, decl, checker, sourceFiles, isFixAll);
                }
                else {
                    changes.delete(sourceFile, decl);
                }
                return true;
            }
            function tryDeleteFullVariableStatement(sourceFile, token, changes) {
                var declarationList = ts.tryCast(token.parent, ts.isVariableDeclarationList);
                if (declarationList && declarationList.getChildren(sourceFile)[0] === token) {
                    changes.delete(sourceFile, declarationList.parent.kind === 225 /* VariableStatement */ ? declarationList.parent : declarationList);
                    return true;
                }
                return false;
            }
            function tryPrefixDeclaration(changes, errorCode, sourceFile, token) {
                // Don't offer to prefix a property.
                if (errorCode === ts.Diagnostics.Property_0_is_declared_but_its_value_is_never_read.code)
                    return;
                if (token.kind === 132 /* InferKeyword */) {
                    token = ts.cast(token.parent, ts.isInferTypeNode).typeParameter.name;
                }
                if (ts.isIdentifier(token) && canPrefix(token)) {
                    changes.replaceNode(sourceFile, token, ts.createIdentifier("_" + token.text));
                }
            }
            function canPrefix(token) {
                switch (token.parent.kind) {
                    case 156 /* Parameter */:
                    case 155 /* TypeParameter */:
                        return true;
                    case 242 /* VariableDeclaration */: {
                        var varDecl = token.parent;
                        switch (varDecl.parent.parent.kind) {
                            case 232 /* ForOfStatement */:
                            case 231 /* ForInStatement */:
                                return true;
                        }
                    }
                }
                return false;
            }
            function tryDeleteDeclaration(sourceFile, token, changes, checker, sourceFiles, isFixAll) {
                tryDeleteDeclarationWorker(token, changes, sourceFile, checker, sourceFiles, isFixAll);
                if (ts.isIdentifier(token))
                    deleteAssignments(changes, sourceFile, token, checker);
            }
            function deleteAssignments(changes, sourceFile, token, checker) {
                ts.FindAllReferences.Core.eachSymbolReferenceInFile(token, checker, sourceFile, function (ref) {
                    if (ts.isPropertyAccessExpression(ref.parent) && ref.parent.name === ref)
                        ref = ref.parent;
                    if (ts.isBinaryExpression(ref.parent) && ts.isExpressionStatement(ref.parent.parent) && ref.parent.left === ref) {
                        changes.delete(sourceFile, ref.parent.parent);
                    }
                });
            }
            function tryDeleteDeclarationWorker(token, changes, sourceFile, checker, sourceFiles, isFixAll) {
                var parent = token.parent;
                if (ts.isParameter(parent)) {
                    tryDeleteParameter(changes, sourceFile, parent, checker, sourceFiles, isFixAll);
                }
                else {
                    changes.delete(sourceFile, ts.isImportClause(parent) ? token : ts.isComputedPropertyName(parent) ? parent.parent : parent);
                }
            }
            function tryDeleteParameter(changes, sourceFile, p, checker, sourceFiles, isFixAll) {
                if (mayDeleteParameter(p, checker, isFixAll)) {
                    if (p.modifiers && p.modifiers.length > 0
                        && (!ts.isIdentifier(p.name) || ts.FindAllReferences.Core.isSymbolReferencedInFile(p.name, checker, sourceFile))) {
                        p.modifiers.forEach(function (modifier) {
                            changes.deleteModifier(sourceFile, modifier);
                        });
                    }
                    else {
                        changes.delete(sourceFile, p);
                        deleteUnusedArguments(changes, sourceFile, p, sourceFiles, checker);
                    }
                }
            }
            function mayDeleteParameter(p, checker, isFixAll) {
                var parent = p.parent;
                switch (parent.kind) {
                    case 161 /* MethodDeclaration */:
                        // Don't remove a parameter if this overrides something.
                        var symbol = checker.getSymbolAtLocation(parent.name);
                        if (ts.isMemberSymbolInBaseType(symbol, checker))
                            return false;
                    // falls through
                    case 162 /* Constructor */:
                    case 244 /* FunctionDeclaration */:
                        return true;
                    case 201 /* FunctionExpression */:
                    case 202 /* ArrowFunction */: {
                        // Can't remove a non-last parameter in a callback. Can remove a parameter in code-fix-all if future parameters are also unused.
                        var parameters = parent.parameters;
                        var index = parameters.indexOf(p);
                        ts.Debug.assert(index !== -1, "The parameter should already be in the list");
                        return isFixAll
                            ? parameters.slice(index + 1).every(function (p) { return p.name.kind === 75 /* Identifier */ && !p.symbol.isReferenced; })
                            : index === parameters.length - 1;
                    }
                    case 164 /* SetAccessor */:
                        // Setter must have a parameter
                        return false;
                    default:
                        return ts.Debug.failBadSyntaxKind(parent);
                }
            }
            function deleteUnusedArguments(changes, sourceFile, deletedParameter, sourceFiles, checker) {
                ts.FindAllReferences.Core.eachSignatureCall(deletedParameter.parent, sourceFiles, checker, function (call) {
                    var index = deletedParameter.parent.parameters.indexOf(deletedParameter);
                    if (call.arguments.length > index) { // Just in case the call didn't provide enough arguments.
                        changes.delete(sourceFile, call.arguments[index]);
                    }
                });
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "fixUnreachableCode";
            var errorCodes = [ts.Diagnostics.Unreachable_code_detected.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, context.sourceFile, context.span.start, context.span.length, context.errorCode); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Remove_unreachable_code, fixId, ts.Diagnostics.Remove_all_unreachable_code)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) { return codefix.codeFixAll(context, errorCodes, function (changes, diag) { return doChange(changes, diag.file, diag.start, diag.length, diag.code); }); },
            });
            function doChange(changes, sourceFile, start, length, errorCode) {
                var token = ts.getTokenAtPosition(sourceFile, start);
                var statement = ts.findAncestor(token, ts.isStatement);
                if (statement.getStart(sourceFile) !== token.getStart(sourceFile)) {
                    var logData = JSON.stringify({
                        statementKind: ts.Debug.formatSyntaxKind(statement.kind),
                        tokenKind: ts.Debug.formatSyntaxKind(token.kind),
                        errorCode: errorCode,
                        start: start,
                        length: length
                    });
                    ts.Debug.fail("Token and statement should start at the same point. " + logData);
                }
                var container = (ts.isBlock(statement.parent) ? statement.parent : statement).parent;
                if (!ts.isBlock(statement.parent) || statement === ts.first(statement.parent.statements)) {
                    switch (container.kind) {
                        case 227 /* IfStatement */:
                            if (container.elseStatement) {
                                if (ts.isBlock(statement.parent)) {
                                    break;
                                }
                                else {
                                    changes.replaceNode(sourceFile, statement, ts.createBlock(ts.emptyArray));
                                }
                                return;
                            }
                        // falls through
                        case 229 /* WhileStatement */:
                        case 230 /* ForStatement */:
                            changes.delete(sourceFile, container);
                            return;
                    }
                }
                if (ts.isBlock(statement.parent)) {
                    var end_3 = start + length;
                    var lastStatement = ts.Debug.assertDefined(lastWhere(ts.sliceAfter(statement.parent.statements, statement), function (s) { return s.pos < end_3; }), "Some statement should be last");
                    changes.deleteNodeRange(sourceFile, statement, lastStatement);
                }
                else {
                    changes.delete(sourceFile, statement);
                }
            }
            function lastWhere(a, pred) {
                var last;
                for (var _i = 0, a_1 = a; _i < a_1.length; _i++) {
                    var value = a_1[_i];
                    if (!pred(value))
                        break;
                    last = value;
                }
                return last;
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "fixUnusedLabel";
            var errorCodes = [ts.Diagnostics.Unused_label.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, context.sourceFile, context.span.start); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Remove_unused_label, fixId, ts.Diagnostics.Remove_all_unused_labels)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) { return codefix.codeFixAll(context, errorCodes, function (changes, diag) { return doChange(changes, diag.file, diag.start); }); },
            });
            function doChange(changes, sourceFile, start) {
                var token = ts.getTokenAtPosition(sourceFile, start);
                var labeledStatement = ts.cast(token.parent, ts.isLabeledStatement);
                var pos = token.getStart(sourceFile);
                var statementPos = labeledStatement.statement.getStart(sourceFile);
                // If label is on a separate line, just delete the rest of that line, but not the indentation of the labeled statement.
                var end = ts.positionsAreOnSameLine(pos, statementPos, sourceFile) ? statementPos
                    : ts.skipTrivia(sourceFile.text, ts.findChildOfKind(labeledStatement, 58 /* ColonToken */, sourceFile).end, /*stopAfterLineBreak*/ true);
                changes.deleteRange(sourceFile, { pos: pos, end: end });
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixIdPlain = "fixJSDocTypes_plain";
            var fixIdNullable = "fixJSDocTypes_nullable";
            var errorCodes = [ts.Diagnostics.JSDoc_types_can_only_be_used_inside_documentation_comments.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile;
                    var checker = context.program.getTypeChecker();
                    var info = getInfo(sourceFile, context.span.start, checker);
                    if (!info)
                        return undefined;
                    var typeNode = info.typeNode, type = info.type;
                    var original = typeNode.getText(sourceFile);
                    var actions = [fix(type, fixIdPlain, ts.Diagnostics.Change_all_jsdoc_style_types_to_TypeScript)];
                    if (typeNode.kind === 297 /* JSDocNullableType */) {
                        // for nullable types, suggest the flow-compatible `T | null | undefined`
                        // in addition to the jsdoc/closure-compatible `T | null`
                        actions.push(fix(checker.getNullableType(type, 32768 /* Undefined */), fixIdNullable, ts.Diagnostics.Change_all_jsdoc_style_types_to_TypeScript_and_add_undefined_to_nullable_types));
                    }
                    return actions;
                    function fix(type, fixId, fixAllDescription) {
                        var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, sourceFile, typeNode, type, checker); });
                        return codefix.createCodeFixAction("jdocTypes", changes, [ts.Diagnostics.Change_0_to_1, original, checker.typeToString(type)], fixId, fixAllDescription);
                    }
                },
                fixIds: [fixIdPlain, fixIdNullable],
                getAllCodeActions: function (context) {
                    var fixId = context.fixId, program = context.program, sourceFile = context.sourceFile;
                    var checker = program.getTypeChecker();
                    return codefix.codeFixAll(context, errorCodes, function (changes, err) {
                        var info = getInfo(err.file, err.start, checker);
                        if (!info)
                            return;
                        var typeNode = info.typeNode, type = info.type;
                        var fixedType = typeNode.kind === 297 /* JSDocNullableType */ && fixId === fixIdNullable ? checker.getNullableType(type, 32768 /* Undefined */) : type;
                        doChange(changes, sourceFile, typeNode, fixedType, checker);
                    });
                }
            });
            function doChange(changes, sourceFile, oldTypeNode, newType, checker) {
                changes.replaceNode(sourceFile, oldTypeNode, checker.typeToTypeNode(newType, /*enclosingDeclaration*/ oldTypeNode)); // TODO: GH#18217
            }
            function getInfo(sourceFile, pos, checker) {
                var decl = ts.findAncestor(ts.getTokenAtPosition(sourceFile, pos), isTypeContainer);
                var typeNode = decl && decl.type;
                return typeNode && { typeNode: typeNode, type: checker.getTypeFromTypeNode(typeNode) };
            }
            function isTypeContainer(node) {
                // NOTE: Some locations are not handled yet:
                // MappedTypeNode.typeParameters and SignatureDeclaration.typeParameters, as well as CallExpression.typeArguments
                switch (node.kind) {
                    case 217 /* AsExpression */:
                    case 165 /* CallSignature */:
                    case 166 /* ConstructSignature */:
                    case 244 /* FunctionDeclaration */:
                    case 163 /* GetAccessor */:
                    case 167 /* IndexSignature */:
                    case 186 /* MappedType */:
                    case 161 /* MethodDeclaration */:
                    case 160 /* MethodSignature */:
                    case 156 /* Parameter */:
                    case 159 /* PropertyDeclaration */:
                    case 158 /* PropertySignature */:
                    case 164 /* SetAccessor */:
                    case 247 /* TypeAliasDeclaration */:
                    case 199 /* TypeAssertionExpression */:
                    case 242 /* VariableDeclaration */:
                        return true;
                    default:
                        return false;
                }
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "fixAwaitInSyncFunction";
            var errorCodes = [
                ts.Diagnostics.await_expressions_are_only_allowed_within_async_functions_and_at_the_top_levels_of_modules.code,
                ts.Diagnostics.A_for_await_of_statement_is_only_allowed_within_an_async_function_or_async_generator.code,
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, span = context.span;
                    var nodes = getNodes(sourceFile, span.start);
                    if (!nodes)
                        return undefined;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, sourceFile, nodes); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Add_async_modifier_to_containing_function, fixId, ts.Diagnostics.Add_all_missing_async_modifiers)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    var seen = ts.createMap();
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var nodes = getNodes(diag.file, diag.start);
                        if (!nodes || !ts.addToSeen(seen, ts.getNodeId(nodes.insertBefore)))
                            return;
                        doChange(changes, context.sourceFile, nodes);
                    });
                },
            });
            function getReturnType(expr) {
                if (expr.type) {
                    return expr.type;
                }
                if (ts.isVariableDeclaration(expr.parent) &&
                    expr.parent.type &&
                    ts.isFunctionTypeNode(expr.parent.type)) {
                    return expr.parent.type.type;
                }
            }
            function getNodes(sourceFile, start) {
                var token = ts.getTokenAtPosition(sourceFile, start);
                var containingFunction = ts.getContainingFunction(token);
                if (!containingFunction) {
                    return;
                }
                var insertBefore;
                switch (containingFunction.kind) {
                    case 161 /* MethodDeclaration */:
                        insertBefore = containingFunction.name;
                        break;
                    case 244 /* FunctionDeclaration */:
                    case 201 /* FunctionExpression */:
                        insertBefore = ts.findChildOfKind(containingFunction, 94 /* FunctionKeyword */, sourceFile);
                        break;
                    case 202 /* ArrowFunction */:
                        insertBefore = ts.findChildOfKind(containingFunction, 20 /* OpenParenToken */, sourceFile) || ts.first(containingFunction.parameters);
                        break;
                    default:
                        return;
                }
                return insertBefore && {
                    insertBefore: insertBefore,
                    returnType: getReturnType(containingFunction)
                };
            }
            function doChange(changes, sourceFile, _a) {
                var insertBefore = _a.insertBefore, returnType = _a.returnType;
                if (returnType) {
                    var entityName = ts.getEntityNameFromTypeNode(returnType);
                    if (!entityName || entityName.kind !== 75 /* Identifier */ || entityName.text !== "Promise") {
                        changes.replaceNode(sourceFile, returnType, ts.createTypeReferenceNode("Promise", ts.createNodeArray([returnType])));
                    }
                }
                changes.insertModifierBefore(sourceFile, 126 /* AsyncKeyword */, insertBefore);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixName = "disableJsDiagnostics";
            var fixId = "disableJsDiagnostics";
            var errorCodes = ts.mapDefined(Object.keys(ts.Diagnostics), function (key) {
                var diag = ts.Diagnostics[key];
                return diag.category === ts.DiagnosticCategory.Error ? diag.code : undefined;
            });
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, program = context.program, span = context.span, host = context.host, formatContext = context.formatContext;
                    if (!ts.isInJSFile(sourceFile) || !ts.isCheckJsEnabledForFile(sourceFile, program.getCompilerOptions())) {
                        return undefined;
                    }
                    var fixes = [
                        // fixId unnecessary because adding `// @ts-nocheck` even once will ignore every error in the file.
                        codefix.createCodeFixActionWithoutFixAll(fixName, [codefix.createFileTextChanges(sourceFile.fileName, [
                                ts.createTextChange(sourceFile.checkJsDirective
                                    ? ts.createTextSpanFromBounds(sourceFile.checkJsDirective.pos, sourceFile.checkJsDirective.end)
                                    : ts.createTextSpan(0, 0), "// @ts-nocheck" + ts.getNewLineOrDefaultFromHost(host, formatContext.options)),
                            ])], ts.Diagnostics.Disable_checking_for_this_file),
                    ];
                    if (ts.textChanges.isValidLocationToAddComment(sourceFile, span.start)) {
                        fixes.unshift(codefix.createCodeFixAction(fixName, ts.textChanges.ChangeTracker.with(context, function (t) { return makeChange(t, sourceFile, span.start); }), ts.Diagnostics.Ignore_this_error_message, fixId, ts.Diagnostics.Add_ts_ignore_to_all_error_messages));
                    }
                    return fixes;
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    var seenLines = ts.createMap();
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        if (ts.textChanges.isValidLocationToAddComment(diag.file, diag.start)) {
                            makeChange(changes, diag.file, diag.start, seenLines);
                        }
                    });
                },
            });
            function makeChange(changes, sourceFile, position, seenLines) {
                var lineNumber = ts.getLineAndCharacterOfPosition(sourceFile, position).line;
                // Only need to add `// @ts-ignore` for a line once.
                if (!seenLines || ts.addToSeen(seenLines, lineNumber)) {
                    changes.insertCommentBeforeLine(sourceFile, lineNumber, position, " @ts-ignore");
                }
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            /**
             * Finds members of the resolved type that are missing in the class pointed to by class decl
             * and generates source code for the missing members.
             * @param possiblyMissingSymbols The collection of symbols to filter and then get insertions for.
             * @returns Empty string iff there are no member insertions.
             */
            function createMissingMemberNodes(classDeclaration, possiblyMissingSymbols, context, preferences, out) {
                var classMembers = classDeclaration.symbol.members;
                for (var _i = 0, possiblyMissingSymbols_1 = possiblyMissingSymbols; _i < possiblyMissingSymbols_1.length; _i++) {
                    var symbol = possiblyMissingSymbols_1[_i];
                    if (!classMembers.has(symbol.escapedName)) {
                        addNewNodeForMemberSymbol(symbol, classDeclaration, context, preferences, out);
                    }
                }
            }
            codefix.createMissingMemberNodes = createMissingMemberNodes;
            function getModuleSpecifierResolverHost(context) {
                return {
                    directoryExists: context.host.directoryExists ? function (d) { return context.host.directoryExists(d); } : undefined,
                    fileExists: context.host.fileExists ? function (f) { return context.host.fileExists(f); } : undefined,
                    getCurrentDirectory: context.host.getCurrentDirectory ? function () { return context.host.getCurrentDirectory(); } : undefined,
                    readFile: context.host.readFile ? function (f) { return context.host.readFile(f); } : undefined,
                    useCaseSensitiveFileNames: context.host.useCaseSensitiveFileNames ? function () { return context.host.useCaseSensitiveFileNames(); } : undefined,
                    getSourceFiles: function () { return context.program.getSourceFiles(); },
                    getCommonSourceDirectory: function () { return context.program.getCommonSourceDirectory(); },
                };
            }
            function getNoopSymbolTrackerWithResolver(context) {
                return {
                    trackSymbol: ts.noop,
                    moduleResolverHost: getModuleSpecifierResolverHost(context),
                };
            }
            codefix.getNoopSymbolTrackerWithResolver = getNoopSymbolTrackerWithResolver;
            /**
             * @returns Empty string iff there we can't figure out a representation for `symbol` in `enclosingDeclaration`.
             */
            function addNewNodeForMemberSymbol(symbol, enclosingDeclaration, context, preferences, out) {
                var declarations = symbol.getDeclarations();
                if (!(declarations && declarations.length)) {
                    return undefined;
                }
                var checker = context.program.getTypeChecker();
                var declaration = declarations[0];
                var name = ts.getSynthesizedDeepClone(ts.getNameOfDeclaration(declaration), /*includeTrivia*/ false);
                var visibilityModifier = createVisibilityModifier(ts.getModifierFlags(declaration));
                var modifiers = visibilityModifier ? ts.createNodeArray([visibilityModifier]) : undefined;
                var type = checker.getWidenedType(checker.getTypeOfSymbolAtLocation(symbol, enclosingDeclaration));
                var optional = !!(symbol.flags & 16777216 /* Optional */);
                var ambient = !!(enclosingDeclaration.flags & 8388608 /* Ambient */);
                switch (declaration.kind) {
                    case 158 /* PropertySignature */:
                    case 159 /* PropertyDeclaration */:
                        var flags = preferences.quotePreference === "single" ? 268435456 /* UseSingleQuotesForStringLiteralType */ : undefined;
                        var typeNode = checker.typeToTypeNode(type, enclosingDeclaration, flags, getNoopSymbolTrackerWithResolver(context));
                        out(ts.createProperty(
                        /*decorators*/ undefined, modifiers, name, optional ? ts.createToken(57 /* QuestionToken */) : undefined, typeNode, 
                        /*initializer*/ undefined));
                        break;
                    case 163 /* GetAccessor */:
                    case 164 /* SetAccessor */: {
                        var allAccessors = ts.getAllAccessorDeclarations(declarations, declaration);
                        var typeNode_1 = checker.typeToTypeNode(type, enclosingDeclaration, /*flags*/ undefined, getNoopSymbolTrackerWithResolver(context));
                        var orderedAccessors = allAccessors.secondAccessor
                            ? [allAccessors.firstAccessor, allAccessors.secondAccessor]
                            : [allAccessors.firstAccessor];
                        for (var _i = 0, orderedAccessors_1 = orderedAccessors; _i < orderedAccessors_1.length; _i++) {
                            var accessor = orderedAccessors_1[_i];
                            if (ts.isGetAccessorDeclaration(accessor)) {
                                out(ts.createGetAccessor(
                                /*decorators*/ undefined, modifiers, name, ts.emptyArray, typeNode_1, ambient ? undefined : createStubbedMethodBody(preferences)));
                            }
                            else {
                                ts.Debug.assertNode(accessor, ts.isSetAccessorDeclaration, "The counterpart to a getter should be a setter");
                                var parameter = ts.getSetAccessorValueParameter(accessor);
                                var parameterName = parameter && ts.isIdentifier(parameter.name) ? ts.idText(parameter.name) : undefined;
                                out(ts.createSetAccessor(
                                /*decorators*/ undefined, modifiers, name, createDummyParameters(1, [parameterName], [typeNode_1], 1, /*inJs*/ false), ambient ? undefined : createStubbedMethodBody(preferences)));
                            }
                        }
                        break;
                    }
                    case 160 /* MethodSignature */:
                    case 161 /* MethodDeclaration */:
                        // The signature for the implementation appears as an entry in `signatures` iff
                        // there is only one signature.
                        // If there are overloads and an implementation signature, it appears as an
                        // extra declaration that isn't a signature for `type`.
                        // If there is more than one overload but no implementation signature
                        // (eg: an abstract method or interface declaration), there is a 1-1
                        // correspondence of declarations and signatures.
                        var signatures = checker.getSignaturesOfType(type, 0 /* Call */);
                        if (!ts.some(signatures)) {
                            break;
                        }
                        if (declarations.length === 1) {
                            ts.Debug.assert(signatures.length === 1, "One declaration implies one signature");
                            var signature = signatures[0];
                            outputMethod(signature, modifiers, name, ambient ? undefined : createStubbedMethodBody(preferences));
                            break;
                        }
                        for (var _a = 0, signatures_1 = signatures; _a < signatures_1.length; _a++) {
                            var signature = signatures_1[_a];
                            // Need to ensure nodes are fresh each time so they can have different positions.
                            outputMethod(signature, ts.getSynthesizedDeepClones(modifiers, /*includeTrivia*/ false), ts.getSynthesizedDeepClone(name, /*includeTrivia*/ false));
                        }
                        if (!ambient) {
                            if (declarations.length > signatures.length) {
                                var signature = checker.getSignatureFromDeclaration(declarations[declarations.length - 1]);
                                outputMethod(signature, modifiers, name, createStubbedMethodBody(preferences));
                            }
                            else {
                                ts.Debug.assert(declarations.length === signatures.length, "Declarations and signatures should match count");
                                out(createMethodImplementingSignatures(signatures, name, optional, modifiers, preferences));
                            }
                        }
                        break;
                }
                function outputMethod(signature, modifiers, name, body) {
                    var method = signatureToMethodDeclaration(context, signature, enclosingDeclaration, modifiers, name, optional, body);
                    if (method)
                        out(method);
                }
            }
            function signatureToMethodDeclaration(context, signature, enclosingDeclaration, modifiers, name, optional, body) {
                var program = context.program;
                var signatureDeclaration = program.getTypeChecker().signatureToSignatureDeclaration(signature, 161 /* MethodDeclaration */, enclosingDeclaration, 1 /* NoTruncation */ | 256 /* SuppressAnyReturnType */, getNoopSymbolTrackerWithResolver(context));
                if (!signatureDeclaration) {
                    return undefined;
                }
                signatureDeclaration.decorators = undefined;
                signatureDeclaration.modifiers = modifiers;
                signatureDeclaration.name = name;
                signatureDeclaration.questionToken = optional ? ts.createToken(57 /* QuestionToken */) : undefined;
                signatureDeclaration.body = body;
                return signatureDeclaration;
            }
            function createMethodFromCallExpression(context, call, methodName, inJs, makeStatic, preferences, contextNode) {
                var body = !ts.isInterfaceDeclaration(contextNode);
                var typeArguments = call.typeArguments, args = call.arguments, parent = call.parent;
                var checker = context.program.getTypeChecker();
                var tracker = getNoopSymbolTrackerWithResolver(context);
                var types = ts.map(args, function (arg) {
                    // Widen the type so we don't emit nonsense annotations like "function fn(x: 3) {"
                    return checker.typeToTypeNode(checker.getBaseTypeOfLiteralType(checker.getTypeAtLocation(arg)), contextNode, /*flags*/ undefined, tracker);
                });
                var names = ts.map(args, function (arg) {
                    return ts.isIdentifier(arg) ? arg.text : ts.isPropertyAccessExpression(arg) && ts.isIdentifier(arg.name) ? arg.name.text : undefined;
                });
                var contextualType = checker.getContextualType(call);
                var returnType = (inJs || !contextualType) ? undefined : checker.typeToTypeNode(contextualType, contextNode, /*flags*/ undefined, tracker);
                return ts.createMethod(
                /*decorators*/ undefined, 
                /*modifiers*/ makeStatic ? [ts.createToken(120 /* StaticKeyword */)] : undefined, 
                /*asteriskToken*/ ts.isYieldExpression(parent) ? ts.createToken(41 /* AsteriskToken */) : undefined, methodName, 
                /*questionToken*/ undefined, 
                /*typeParameters*/ inJs ? undefined : ts.map(typeArguments, function (_, i) {
                    return ts.createTypeParameterDeclaration(84 /* T */ + typeArguments.length - 1 <= 90 /* Z */ ? String.fromCharCode(84 /* T */ + i) : "T" + i);
                }), 
                /*parameters*/ createDummyParameters(args.length, names, types, /*minArgumentCount*/ undefined, inJs), 
                /*type*/ returnType, body ? createStubbedMethodBody(preferences) : undefined);
            }
            codefix.createMethodFromCallExpression = createMethodFromCallExpression;
            function createDummyParameters(argCount, names, types, minArgumentCount, inJs) {
                var parameters = [];
                for (var i = 0; i < argCount; i++) {
                    var newParameter = ts.createParameter(
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, 
                    /*dotDotDotToken*/ undefined, 
                    /*name*/ names && names[i] || "arg" + i, 
                    /*questionToken*/ minArgumentCount !== undefined && i >= minArgumentCount ? ts.createToken(57 /* QuestionToken */) : undefined, 
                    /*type*/ inJs ? undefined : types && types[i] || ts.createKeywordTypeNode(125 /* AnyKeyword */), 
                    /*initializer*/ undefined);
                    parameters.push(newParameter);
                }
                return parameters;
            }
            function createMethodImplementingSignatures(signatures, name, optional, modifiers, preferences) {
                /** This is *a* signature with the maximal number of arguments,
                 * such that if there is a "maximal" signature without rest arguments,
                 * this is one of them.
                 */
                var maxArgsSignature = signatures[0];
                var minArgumentCount = signatures[0].minArgumentCount;
                var someSigHasRestParameter = false;
                for (var _i = 0, signatures_2 = signatures; _i < signatures_2.length; _i++) {
                    var sig = signatures_2[_i];
                    minArgumentCount = Math.min(sig.minArgumentCount, minArgumentCount);
                    if (ts.signatureHasRestParameter(sig)) {
                        someSigHasRestParameter = true;
                    }
                    if (sig.parameters.length >= maxArgsSignature.parameters.length && (!ts.signatureHasRestParameter(sig) || ts.signatureHasRestParameter(maxArgsSignature))) {
                        maxArgsSignature = sig;
                    }
                }
                var maxNonRestArgs = maxArgsSignature.parameters.length - (ts.signatureHasRestParameter(maxArgsSignature) ? 1 : 0);
                var maxArgsParameterSymbolNames = maxArgsSignature.parameters.map(function (symbol) { return symbol.name; });
                var parameters = createDummyParameters(maxNonRestArgs, maxArgsParameterSymbolNames, /* types */ undefined, minArgumentCount, /*inJs*/ false);
                if (someSigHasRestParameter) {
                    var anyArrayType = ts.createArrayTypeNode(ts.createKeywordTypeNode(125 /* AnyKeyword */));
                    var restParameter = ts.createParameter(
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, ts.createToken(25 /* DotDotDotToken */), maxArgsParameterSymbolNames[maxNonRestArgs] || "rest", 
                    /*questionToken*/ maxNonRestArgs >= minArgumentCount ? ts.createToken(57 /* QuestionToken */) : undefined, anyArrayType, 
                    /*initializer*/ undefined);
                    parameters.push(restParameter);
                }
                return createStubbedMethod(modifiers, name, optional, 
                /*typeParameters*/ undefined, parameters, 
                /*returnType*/ undefined, preferences);
            }
            function createStubbedMethod(modifiers, name, optional, typeParameters, parameters, returnType, preferences) {
                return ts.createMethod(
                /*decorators*/ undefined, modifiers, 
                /*asteriskToken*/ undefined, name, optional ? ts.createToken(57 /* QuestionToken */) : undefined, typeParameters, parameters, returnType, createStubbedMethodBody(preferences));
            }
            function createStubbedMethodBody(preferences) {
                return ts.createBlock([ts.createThrow(ts.createNew(ts.createIdentifier("Error"), 
                    /*typeArguments*/ undefined, 
                    // TODO Handle auto quote preference.
                    [ts.createLiteral("Method not implemented.", /*isSingleQuote*/ preferences.quotePreference === "single")]))], 
                /*multiline*/ true);
            }
            function createVisibilityModifier(flags) {
                if (flags & 4 /* Public */) {
                    return ts.createToken(119 /* PublicKeyword */);
                }
                else if (flags & 16 /* Protected */) {
                    return ts.createToken(118 /* ProtectedKeyword */);
                }
                return undefined;
            }
            function setJsonCompilerOptionValues(changeTracker, configFile, options) {
                var tsconfigObjectLiteral = ts.getTsConfigObjectLiteralExpression(configFile);
                if (!tsconfigObjectLiteral)
                    return undefined;
                var compilerOptionsProperty = findJsonProperty(tsconfigObjectLiteral, "compilerOptions");
                if (compilerOptionsProperty === undefined) {
                    changeTracker.insertNodeAtObjectStart(configFile, tsconfigObjectLiteral, createJsonPropertyAssignment("compilerOptions", ts.createObjectLiteral(options.map(function (_a) {
                        var optionName = _a[0], optionValue = _a[1];
                        return createJsonPropertyAssignment(optionName, optionValue);
                    }), /*multiLine*/ true)));
                    return;
                }
                var compilerOptions = compilerOptionsProperty.initializer;
                if (!ts.isObjectLiteralExpression(compilerOptions)) {
                    return;
                }
                for (var _i = 0, options_1 = options; _i < options_1.length; _i++) {
                    var _a = options_1[_i], optionName = _a[0], optionValue = _a[1];
                    var optionProperty = findJsonProperty(compilerOptions, optionName);
                    if (optionProperty === undefined) {
                        changeTracker.insertNodeAtObjectStart(configFile, compilerOptions, createJsonPropertyAssignment(optionName, optionValue));
                    }
                    else {
                        changeTracker.replaceNode(configFile, optionProperty.initializer, optionValue);
                    }
                }
            }
            codefix.setJsonCompilerOptionValues = setJsonCompilerOptionValues;
            function setJsonCompilerOptionValue(changeTracker, configFile, optionName, optionValue) {
                setJsonCompilerOptionValues(changeTracker, configFile, [[optionName, optionValue]]);
            }
            codefix.setJsonCompilerOptionValue = setJsonCompilerOptionValue;
            function createJsonPropertyAssignment(name, initializer) {
                return ts.createPropertyAssignment(ts.createStringLiteral(name), initializer);
            }
            codefix.createJsonPropertyAssignment = createJsonPropertyAssignment;
            function findJsonProperty(obj, name) {
                return ts.find(obj.properties, function (p) { return ts.isPropertyAssignment(p) && !!p.name && ts.isStringLiteral(p.name) && p.name.text === name; });
            }
            codefix.findJsonProperty = findJsonProperty;
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixName = "invalidImportSyntax";
            function getCodeFixesForImportDeclaration(context, node) {
                var sourceFile = ts.getSourceFileOfNode(node);
                var namespace = ts.getNamespaceDeclarationNode(node);
                var opts = context.program.getCompilerOptions();
                var variations = [];
                // import Bluebird from "bluebird";
                variations.push(createAction(context, sourceFile, node, ts.makeImport(namespace.name, /*namedImports*/ undefined, node.moduleSpecifier, ts.getQuotePreference(sourceFile, context.preferences))));
                if (ts.getEmitModuleKind(opts) === ts.ModuleKind.CommonJS) {
                    // import Bluebird = require("bluebird");
                    variations.push(createAction(context, sourceFile, node, ts.createImportEqualsDeclaration(
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, namespace.name, ts.createExternalModuleReference(node.moduleSpecifier))));
                }
                return variations;
            }
            function createAction(context, sourceFile, node, replacement) {
                var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return t.replaceNode(sourceFile, node, replacement); });
                return codefix.createCodeFixActionWithoutFixAll(fixName, changes, [ts.Diagnostics.Replace_import_with_0, changes[0].textChanges[0].newText]);
            }
            codefix.registerCodeFix({
                errorCodes: [
                    ts.Diagnostics.This_expression_is_not_callable.code,
                    ts.Diagnostics.This_expression_is_not_constructable.code,
                ],
                getCodeActions: getActionsForUsageOfInvalidImport
            });
            function getActionsForUsageOfInvalidImport(context) {
                var sourceFile = context.sourceFile;
                var targetKind = ts.Diagnostics.This_expression_is_not_callable.code === context.errorCode ? 196 /* CallExpression */ : 197 /* NewExpression */;
                var node = ts.findAncestor(ts.getTokenAtPosition(sourceFile, context.span.start), function (a) { return a.kind === targetKind; });
                if (!node) {
                    return [];
                }
                var expr = node.expression;
                return getImportCodeFixesForExpression(context, expr);
            }
            codefix.registerCodeFix({
                errorCodes: [
                    // The following error codes cover pretty much all assignability errors that could involve an expression
                    ts.Diagnostics.Argument_of_type_0_is_not_assignable_to_parameter_of_type_1.code,
                    ts.Diagnostics.Type_0_does_not_satisfy_the_constraint_1.code,
                    ts.Diagnostics.Type_0_is_not_assignable_to_type_1.code,
                    ts.Diagnostics.Type_0_is_not_assignable_to_type_1_Two_different_types_with_this_name_exist_but_they_are_unrelated.code,
                    ts.Diagnostics.Type_predicate_0_is_not_assignable_to_1.code,
                    ts.Diagnostics.Property_0_of_type_1_is_not_assignable_to_string_index_type_2.code,
                    ts.Diagnostics.Property_0_of_type_1_is_not_assignable_to_numeric_index_type_2.code,
                    ts.Diagnostics.Numeric_index_type_0_is_not_assignable_to_string_index_type_1.code,
                    ts.Diagnostics.Property_0_in_type_1_is_not_assignable_to_the_same_property_in_base_type_2.code,
                    ts.Diagnostics.Property_0_in_type_1_is_not_assignable_to_type_2.code,
                    ts.Diagnostics.Property_0_of_JSX_spread_attribute_is_not_assignable_to_target_property.code,
                    ts.Diagnostics.The_this_context_of_type_0_is_not_assignable_to_method_s_this_of_type_1.code,
                ],
                getCodeActions: getActionsForInvalidImportLocation
            });
            function getActionsForInvalidImportLocation(context) {
                var sourceFile = context.sourceFile;
                var node = ts.findAncestor(ts.getTokenAtPosition(sourceFile, context.span.start), function (a) { return a.getStart() === context.span.start && a.getEnd() === (context.span.start + context.span.length); });
                if (!node) {
                    return [];
                }
                return getImportCodeFixesForExpression(context, node);
            }
            function getImportCodeFixesForExpression(context, expr) {
                var type = context.program.getTypeChecker().getTypeAtLocation(expr);
                if (!(type.symbol && type.symbol.originatingImport)) {
                    return [];
                }
                var fixes = [];
                var relatedImport = type.symbol.originatingImport; // TODO: GH#18217
                if (!ts.isImportCall(relatedImport)) {
                    ts.addRange(fixes, getCodeFixesForImportDeclaration(context, relatedImport));
                }
                if (ts.isExpression(expr) && !(ts.isNamedDeclaration(expr.parent) && expr.parent.name === expr)) {
                    var sourceFile_1 = context.sourceFile;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return t.replaceNode(sourceFile_1, expr, ts.createPropertyAccess(expr, "default"), {}); });
                    fixes.push(codefix.createCodeFixActionWithoutFixAll(fixName, changes, ts.Diagnostics.Use_synthetic_default_member));
                }
                return fixes;
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixName = "strictClassInitialization";
            var fixIdAddDefiniteAssignmentAssertions = "addMissingPropertyDefiniteAssignmentAssertions";
            var fixIdAddUndefinedType = "addMissingPropertyUndefinedType";
            var fixIdAddInitializer = "addMissingPropertyInitializer";
            var errorCodes = [ts.Diagnostics.Property_0_has_no_initializer_and_is_not_definitely_assigned_in_the_constructor.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var propertyDeclaration = getPropertyDeclaration(context.sourceFile, context.span.start);
                    if (!propertyDeclaration)
                        return;
                    var result = [
                        getActionForAddMissingUndefinedType(context, propertyDeclaration),
                        getActionForAddMissingDefiniteAssignmentAssertion(context, propertyDeclaration)
                    ];
                    ts.append(result, getActionForAddMissingInitializer(context, propertyDeclaration));
                    return result;
                },
                fixIds: [fixIdAddDefiniteAssignmentAssertions, fixIdAddUndefinedType, fixIdAddInitializer],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var propertyDeclaration = getPropertyDeclaration(diag.file, diag.start);
                        if (!propertyDeclaration)
                            return;
                        switch (context.fixId) {
                            case fixIdAddDefiniteAssignmentAssertions:
                                addDefiniteAssignmentAssertion(changes, diag.file, propertyDeclaration);
                                break;
                            case fixIdAddUndefinedType:
                                addUndefinedType(changes, diag.file, propertyDeclaration);
                                break;
                            case fixIdAddInitializer:
                                var checker = context.program.getTypeChecker();
                                var initializer = getInitializer(checker, propertyDeclaration);
                                if (!initializer)
                                    return;
                                addInitializer(changes, diag.file, propertyDeclaration, initializer);
                                break;
                            default:
                                ts.Debug.fail(JSON.stringify(context.fixId));
                        }
                    });
                },
            });
            function getPropertyDeclaration(sourceFile, pos) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                return ts.isIdentifier(token) ? ts.cast(token.parent, ts.isPropertyDeclaration) : undefined;
            }
            function getActionForAddMissingDefiniteAssignmentAssertion(context, propertyDeclaration) {
                var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return addDefiniteAssignmentAssertion(t, context.sourceFile, propertyDeclaration); });
                return codefix.createCodeFixAction(fixName, changes, [ts.Diagnostics.Add_definite_assignment_assertion_to_property_0, propertyDeclaration.getText()], fixIdAddDefiniteAssignmentAssertions, ts.Diagnostics.Add_definite_assignment_assertions_to_all_uninitialized_properties);
            }
            function addDefiniteAssignmentAssertion(changeTracker, propertyDeclarationSourceFile, propertyDeclaration) {
                var property = ts.updateProperty(propertyDeclaration, propertyDeclaration.decorators, propertyDeclaration.modifiers, propertyDeclaration.name, ts.createToken(53 /* ExclamationToken */), propertyDeclaration.type, propertyDeclaration.initializer);
                changeTracker.replaceNode(propertyDeclarationSourceFile, propertyDeclaration, property);
            }
            function getActionForAddMissingUndefinedType(context, propertyDeclaration) {
                var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return addUndefinedType(t, context.sourceFile, propertyDeclaration); });
                return codefix.createCodeFixAction(fixName, changes, [ts.Diagnostics.Add_undefined_type_to_property_0, propertyDeclaration.name.getText()], fixIdAddUndefinedType, ts.Diagnostics.Add_undefined_type_to_all_uninitialized_properties);
            }
            function addUndefinedType(changeTracker, propertyDeclarationSourceFile, propertyDeclaration) {
                var undefinedTypeNode = ts.createKeywordTypeNode(146 /* UndefinedKeyword */);
                var type = propertyDeclaration.type; // TODO: GH#18217
                var types = ts.isUnionTypeNode(type) ? type.types.concat(undefinedTypeNode) : [type, undefinedTypeNode];
                changeTracker.replaceNode(propertyDeclarationSourceFile, type, ts.createUnionTypeNode(types));
            }
            function getActionForAddMissingInitializer(context, propertyDeclaration) {
                var checker = context.program.getTypeChecker();
                var initializer = getInitializer(checker, propertyDeclaration);
                if (!initializer)
                    return undefined;
                var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return addInitializer(t, context.sourceFile, propertyDeclaration, initializer); });
                return codefix.createCodeFixAction(fixName, changes, [ts.Diagnostics.Add_initializer_to_property_0, propertyDeclaration.name.getText()], fixIdAddInitializer, ts.Diagnostics.Add_initializers_to_all_uninitialized_properties);
            }
            function addInitializer(changeTracker, propertyDeclarationSourceFile, propertyDeclaration, initializer) {
                var property = ts.updateProperty(propertyDeclaration, propertyDeclaration.decorators, propertyDeclaration.modifiers, propertyDeclaration.name, propertyDeclaration.questionToken, propertyDeclaration.type, initializer);
                changeTracker.replaceNode(propertyDeclarationSourceFile, propertyDeclaration, property);
            }
            function getInitializer(checker, propertyDeclaration) {
                return getDefaultValueFromType(checker, checker.getTypeFromTypeNode(propertyDeclaration.type)); // TODO: GH#18217
            }
            function getDefaultValueFromType(checker, type) {
                if (type.flags & 512 /* BooleanLiteral */) {
                    return (type === checker.getFalseType() || type === checker.getFalseType(/*fresh*/ true)) ? ts.createFalse() : ts.createTrue();
                }
                else if (type.isLiteral()) {
                    return ts.createLiteral(type.value);
                }
                else if (type.isUnion()) {
                    return ts.firstDefined(type.types, function (t) { return getDefaultValueFromType(checker, t); });
                }
                else if (type.isClass()) {
                    var classDeclaration = ts.getClassLikeDeclarationOfSymbol(type.symbol);
                    if (!classDeclaration || ts.hasModifier(classDeclaration, 128 /* Abstract */))
                        return undefined;
                    var constructorDeclaration = ts.getFirstConstructorWithBody(classDeclaration);
                    if (constructorDeclaration && constructorDeclaration.parameters.length)
                        return undefined;
                    return ts.createNew(ts.createIdentifier(type.symbol.name), /*typeArguments*/ undefined, /*argumentsArray*/ undefined);
                }
                else if (checker.isArrayLikeType(type)) {
                    return ts.createArrayLiteral();
                }
                return undefined;
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "requireInTs";
            var errorCodes = [ts.Diagnostics.require_call_may_be_converted_to_an_import.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, context.sourceFile, context.span.start, context.program); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Convert_require_to_import, fixId, ts.Diagnostics.Convert_all_require_to_import)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) { return codefix.codeFixAll(context, errorCodes, function (changes, diag) { return doChange(changes, diag.file, diag.start, context.program); }); },
            });
            function doChange(changes, sourceFile, pos, program) {
                var _a = getInfo(sourceFile, pos), statement = _a.statement, name = _a.name, required = _a.required;
                changes.replaceNode(sourceFile, statement, ts.getAllowSyntheticDefaultImports(program.getCompilerOptions())
                    ? ts.createImportDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, ts.createImportClause(name, /*namedBindings*/ undefined), required)
                    : ts.createImportEqualsDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, name, ts.createExternalModuleReference(required)));
            }
            function getInfo(sourceFile, pos) {
                var parent = ts.getTokenAtPosition(sourceFile, pos).parent;
                if (!ts.isRequireCall(parent, /*checkArgumentIsStringLiteralLike*/ true))
                    throw ts.Debug.failBadSyntaxKind(parent);
                var decl = ts.cast(parent.parent, ts.isVariableDeclaration);
                return { statement: ts.cast(decl.parent.parent, ts.isVariableStatement), name: ts.cast(decl.name, ts.isIdentifier), required: parent.arguments[0] };
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "useDefaultImport";
            var errorCodes = [ts.Diagnostics.Import_may_be_converted_to_a_default_import.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, start = context.span.start;
                    var info = getInfo(sourceFile, start);
                    if (!info)
                        return undefined;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, sourceFile, info, context.preferences); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Convert_to_default_import, fixId, ts.Diagnostics.Convert_all_to_default_imports)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var info = getInfo(diag.file, diag.start);
                        if (info)
                            doChange(changes, diag.file, info, context.preferences);
                    });
                },
            });
            function getInfo(sourceFile, pos) {
                var name = ts.getTokenAtPosition(sourceFile, pos);
                if (!ts.isIdentifier(name))
                    return undefined; // bad input
                var parent = name.parent;
                if (ts.isImportEqualsDeclaration(parent) && ts.isExternalModuleReference(parent.moduleReference)) {
                    return { importNode: parent, name: name, moduleSpecifier: parent.moduleReference.expression };
                }
                else if (ts.isNamespaceImport(parent)) {
                    var importNode = parent.parent.parent;
                    return { importNode: importNode, name: name, moduleSpecifier: importNode.moduleSpecifier };
                }
            }
            function doChange(changes, sourceFile, info, preferences) {
                changes.replaceNode(sourceFile, info.importNode, ts.makeImport(info.name, /*namedImports*/ undefined, info.moduleSpecifier, ts.getQuotePreference(sourceFile, preferences)));
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "useBigintLiteral";
            var errorCodes = [
                ts.Diagnostics.Numeric_literals_with_absolute_values_equal_to_2_53_or_greater_are_too_large_to_be_represented_accurately_as_integers.code,
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return makeChange(t, context.sourceFile, context.span); });
                    if (changes.length > 0) {
                        return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Convert_to_a_bigint_numeric_literal, fixId, ts.Diagnostics.Convert_all_to_bigint_numeric_literals)];
                    }
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) { return makeChange(changes, diag.file, diag); });
                },
            });
            function makeChange(changeTracker, sourceFile, span) {
                var numericLiteral = ts.tryCast(ts.getTokenAtPosition(sourceFile, span.start), ts.isNumericLiteral);
                if (!numericLiteral) {
                    return;
                }
                // We use .getText to overcome parser inaccuracies: https://github.com/microsoft/TypeScript/issues/33298
                var newText = numericLiteral.getText(sourceFile) + "n";
                changeTracker.replaceNode(sourceFile, numericLiteral, ts.createBigIntLiteral(newText));
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixIdAddMissingTypeof = "fixAddModuleReferTypeMissingTypeof";
            var fixId = fixIdAddMissingTypeof;
            var errorCodes = [ts.Diagnostics.Module_0_does_not_refer_to_a_type_but_is_used_as_a_type_here_Did_you_mean_typeof_import_0.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, span = context.span;
                    var importType = getImportTypeNode(sourceFile, span.start);
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, sourceFile, importType); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Add_missing_typeof, fixId, ts.Diagnostics.Add_missing_typeof)];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        return doChange(changes, context.sourceFile, getImportTypeNode(diag.file, diag.start));
                    });
                },
            });
            function getImportTypeNode(sourceFile, pos) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                ts.Debug.assert(token.kind === 96 /* ImportKeyword */, "This token should be an ImportKeyword");
                ts.Debug.assert(token.parent.kind === 188 /* ImportType */, "Token parent should be an ImportType");
                return token.parent;
            }
            function doChange(changes, sourceFile, importType) {
                var newTypeNode = ts.updateImportTypeNode(importType, importType.argument, importType.qualifier, importType.typeArguments, /* isTypeOf */ true);
                changes.replaceNode(sourceFile, importType, newTypeNode);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixIdAddMissingTypeof = "fixConvertToMappedObjectType";
            var fixId = fixIdAddMissingTypeof;
            var errorCodes = [ts.Diagnostics.An_index_signature_parameter_type_cannot_be_a_union_type_Consider_using_a_mapped_object_type_instead.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, span = context.span;
                    var info = getInfo(sourceFile, span.start);
                    if (!info)
                        return undefined;
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, sourceFile, info); });
                    var name = ts.idText(info.container.name);
                    return [codefix.createCodeFixAction(fixId, changes, [ts.Diagnostics.Convert_0_to_mapped_object_type, name], fixId, [ts.Diagnostics.Convert_0_to_mapped_object_type, name])];
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) {
                        var info = getInfo(diag.file, diag.start);
                        if (info)
                            doChange(changes, diag.file, info);
                    });
                }
            });
            function getInfo(sourceFile, pos) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                var indexSignature = ts.cast(token.parent.parent, ts.isIndexSignatureDeclaration);
                if (ts.isClassDeclaration(indexSignature.parent))
                    return undefined;
                var container = ts.isInterfaceDeclaration(indexSignature.parent) ? indexSignature.parent : ts.cast(indexSignature.parent.parent, ts.isTypeAliasDeclaration);
                return { indexSignature: indexSignature, container: container };
            }
            function createTypeAliasFromInterface(declaration, type) {
                return ts.createTypeAliasDeclaration(declaration.decorators, declaration.modifiers, declaration.name, declaration.typeParameters, type);
            }
            function doChange(changes, sourceFile, _a) {
                var indexSignature = _a.indexSignature, container = _a.container;
                var members = ts.isInterfaceDeclaration(container) ? container.members : container.type.members;
                var otherMembers = members.filter(function (member) { return !ts.isIndexSignatureDeclaration(member); });
                var parameter = ts.first(indexSignature.parameters);
                var mappedTypeParameter = ts.createTypeParameterDeclaration(ts.cast(parameter.name, ts.isIdentifier), parameter.type);
                var mappedIntersectionType = ts.createMappedTypeNode(ts.hasReadonlyModifier(indexSignature) ? ts.createModifier(138 /* ReadonlyKeyword */) : undefined, mappedTypeParameter, indexSignature.questionToken, indexSignature.type);
                var intersectionType = ts.createIntersectionTypeNode(__spreadArrays(ts.getAllSuperTypeNodes(container), [
                    mappedIntersectionType
                ], (otherMembers.length ? [ts.createTypeLiteralNode(otherMembers)] : ts.emptyArray)));
                changes.replaceNode(sourceFile, container, createTypeAliasFromInterface(container, intersectionType));
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "removeUnnecessaryAwait";
            var errorCodes = [
                ts.Diagnostics.await_has_no_effect_on_the_type_of_this_expression.code,
            ];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return makeChange(t, context.sourceFile, context.span); });
                    if (changes.length > 0) {
                        return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Remove_unnecessary_await, fixId, ts.Diagnostics.Remove_all_unnecessary_uses_of_await)];
                    }
                },
                fixIds: [fixId],
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, diag) { return makeChange(changes, diag.file, diag); });
                },
            });
            function makeChange(changeTracker, sourceFile, span) {
                var awaitKeyword = ts.tryCast(ts.getTokenAtPosition(sourceFile, span.start), function (node) { return node.kind === 127 /* AwaitKeyword */; });
                var awaitExpression = awaitKeyword && ts.tryCast(awaitKeyword.parent, ts.isAwaitExpression);
                if (!awaitExpression) {
                    return;
                }
                var expressionToReplace = awaitExpression;
                var hasSurroundingParens = ts.isParenthesizedExpression(awaitExpression.parent);
                if (hasSurroundingParens) {
                    var leftMostExpression = ts.getLeftmostExpression(awaitExpression.expression, /*stopAtCallExpressions*/ false);
                    if (ts.isIdentifier(leftMostExpression)) {
                        var precedingToken = ts.findPrecedingToken(awaitExpression.parent.pos, sourceFile);
                        if (precedingToken && precedingToken.kind !== 99 /* NewKeyword */) {
                            expressionToReplace = awaitExpression.parent;
                        }
                    }
                }
                changeTracker.replaceNode(sourceFile, expressionToReplace, awaitExpression.expression);
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var errorCodes = [ts.Diagnostics.A_type_only_import_can_specify_a_default_import_or_named_bindings_but_not_both.code];
            var fixId = "splitTypeOnlyImport";
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                fixIds: [fixId],
                getCodeActions: function (context) {
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) {
                        return splitTypeOnlyImport(t, getImportDeclaration(context.sourceFile, context.span), context);
                    });
                    if (changes.length) {
                        return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Split_into_two_separate_import_declarations, fixId, ts.Diagnostics.Split_all_invalid_type_only_imports)];
                    }
                },
                getAllCodeActions: function (context) {
                    return codefix.codeFixAll(context, errorCodes, function (changes, error) {
                        splitTypeOnlyImport(changes, getImportDeclaration(context.sourceFile, error), context);
                    });
                },
            });
            function getImportDeclaration(sourceFile, span) {
                return ts.findAncestor(ts.getTokenAtPosition(sourceFile, span.start), ts.isImportDeclaration);
            }
            function splitTypeOnlyImport(changes, importDeclaration, context) {
                if (!importDeclaration) {
                    return;
                }
                var importClause = ts.Debug.assertDefined(importDeclaration.importClause);
                changes.replaceNode(context.sourceFile, importDeclaration, ts.updateImportDeclaration(importDeclaration, importDeclaration.decorators, importDeclaration.modifiers, ts.updateImportClause(importClause, importClause.name, /*namedBindings*/ undefined, importClause.isTypeOnly), importDeclaration.moduleSpecifier));
                changes.insertNodeAfter(context.sourceFile, importDeclaration, ts.createImportDeclaration(
                /*decorators*/ undefined, 
                /*modifiers*/ undefined, ts.updateImportClause(importClause, /*name*/ undefined, importClause.namedBindings, importClause.isTypeOnly), importDeclaration.moduleSpecifier));
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var codefix;
        (function (codefix) {
            var fixId = "fixConvertConstToLet";
            var errorCodes = [ts.Diagnostics.Cannot_assign_to_0_because_it_is_a_constant.code];
            codefix.registerCodeFix({
                errorCodes: errorCodes,
                getCodeActions: function (context) {
                    var sourceFile = context.sourceFile, span = context.span, program = context.program;
                    var variableStatement = getVariableStatement(sourceFile, span.start, program);
                    var changes = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(t, sourceFile, variableStatement); });
                    return [codefix.createCodeFixAction(fixId, changes, ts.Diagnostics.Convert_const_to_let, fixId, ts.Diagnostics.Convert_const_to_let)];
                },
                fixIds: [fixId]
            });
            function getVariableStatement(sourceFile, pos, program) {
                var token = ts.getTokenAtPosition(sourceFile, pos);
                var checker = program.getTypeChecker();
                var symbol = checker.getSymbolAtLocation(token);
                if (symbol) {
                    return symbol.valueDeclaration.parent.parent;
                }
            }
            function doChange(changes, sourceFile, variableStatement) {
                if (!variableStatement) {
                    return;
                }
                var start = variableStatement.getStart();
                changes.replaceRangeWithText(sourceFile, { pos: start, end: start + 5 }, "let");
            }
        })(codefix = ts.codefix || (ts.codefix = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var refactor;
        (function (refactor) {
            var refactorName = "Convert export";
            var actionNameDefaultToNamed = "Convert default export to named export";
            var actionNameNamedToDefault = "Convert named export to default export";
            refactor.registerRefactor(refactorName, {
                getAvailableActions: function (context) {
                    var info = getInfo(context);
                    if (!info)
                        return ts.emptyArray;
                    var description = info.wasDefault ? ts.Diagnostics.Convert_default_export_to_named_export.message : ts.Diagnostics.Convert_named_export_to_default_export.message;
                    var actionName = info.wasDefault ? actionNameDefaultToNamed : actionNameNamedToDefault;
                    return [{ name: refactorName, description: description, actions: [{ name: actionName, description: description }] }];
                },
                getEditsForAction: function (context, actionName) {
                    ts.Debug.assert(actionName === actionNameDefaultToNamed || actionName === actionNameNamedToDefault, "Unexpected action name");
                    var edits = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(context.file, context.program, ts.Debug.assertDefined(getInfo(context), "context must have info"), t, context.cancellationToken); });
                    return { edits: edits, renameFilename: undefined, renameLocation: undefined };
                },
            });
            function getInfo(context) {
                var file = context.file;
                var span = ts.getRefactorContextSpan(context);
                var token = ts.getTokenAtPosition(file, span.start);
                var exportNode = ts.getParentNodeInSpan(token, file, span);
                if (!exportNode || (!ts.isSourceFile(exportNode.parent) && !(ts.isModuleBlock(exportNode.parent) && ts.isAmbientModule(exportNode.parent.parent)))) {
                    return undefined;
                }
                var exportingModuleSymbol = ts.isSourceFile(exportNode.parent) ? exportNode.parent.symbol : exportNode.parent.parent.symbol;
                var flags = ts.getModifierFlags(exportNode);
                var wasDefault = !!(flags & 512 /* Default */);
                // If source file already has a default export, don't offer refactor.
                if (!(flags & 1 /* Export */) || !wasDefault && exportingModuleSymbol.exports.has("default" /* Default */)) {
                    return undefined;
                }
                switch (exportNode.kind) {
                    case 244 /* FunctionDeclaration */:
                    case 245 /* ClassDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 247 /* TypeAliasDeclaration */:
                    case 249 /* ModuleDeclaration */: {
                        var node = exportNode;
                        return node.name && ts.isIdentifier(node.name) ? { exportNode: node, exportName: node.name, wasDefault: wasDefault, exportingModuleSymbol: exportingModuleSymbol } : undefined;
                    }
                    case 225 /* VariableStatement */: {
                        var vs = exportNode;
                        // Must be `export const x = something;`.
                        if (!(vs.declarationList.flags & 2 /* Const */) || vs.declarationList.declarations.length !== 1) {
                            return undefined;
                        }
                        var decl = ts.first(vs.declarationList.declarations);
                        if (!decl.initializer)
                            return undefined;
                        ts.Debug.assert(!wasDefault, "Can't have a default flag here");
                        return ts.isIdentifier(decl.name) ? { exportNode: vs, exportName: decl.name, wasDefault: wasDefault, exportingModuleSymbol: exportingModuleSymbol } : undefined;
                    }
                    default:
                        return undefined;
                }
            }
            function doChange(exportingSourceFile, program, info, changes, cancellationToken) {
                changeExport(exportingSourceFile, info, changes, program.getTypeChecker());
                changeImports(program, info, changes, cancellationToken);
            }
            function changeExport(exportingSourceFile, _a, changes, checker) {
                var wasDefault = _a.wasDefault, exportNode = _a.exportNode, exportName = _a.exportName;
                if (wasDefault) {
                    changes.delete(exportingSourceFile, ts.Debug.assertDefined(ts.findModifier(exportNode, 84 /* DefaultKeyword */), "Should find a default keyword in modifier list"));
                }
                else {
                    var exportKeyword = ts.Debug.assertDefined(ts.findModifier(exportNode, 89 /* ExportKeyword */), "Should find an export keyword in modifier list");
                    switch (exportNode.kind) {
                        case 244 /* FunctionDeclaration */:
                        case 245 /* ClassDeclaration */:
                        case 246 /* InterfaceDeclaration */:
                            changes.insertNodeAfter(exportingSourceFile, exportKeyword, ts.createToken(84 /* DefaultKeyword */));
                            break;
                        case 225 /* VariableStatement */:
                            // If 'x' isn't used in this file, `export const x = 0;` --> `export default 0;`
                            if (!ts.FindAllReferences.Core.isSymbolReferencedInFile(exportName, checker, exportingSourceFile)) {
                                // We checked in `getInfo` that an initializer exists.
                                changes.replaceNode(exportingSourceFile, exportNode, ts.createExportDefault(ts.Debug.assertDefined(ts.first(exportNode.declarationList.declarations).initializer, "Initializer was previously known to be present")));
                                break;
                            }
                        // falls through
                        case 248 /* EnumDeclaration */:
                        case 247 /* TypeAliasDeclaration */:
                        case 249 /* ModuleDeclaration */:
                            // `export type T = number;` -> `type T = number; export default T;`
                            changes.deleteModifier(exportingSourceFile, exportKeyword);
                            changes.insertNodeAfter(exportingSourceFile, exportNode, ts.createExportDefault(ts.createIdentifier(exportName.text)));
                            break;
                        default:
                            ts.Debug.assertNever(exportNode, "Unexpected exportNode kind " + exportNode.kind);
                    }
                }
            }
            function changeImports(program, _a, changes, cancellationToken) {
                var wasDefault = _a.wasDefault, exportName = _a.exportName, exportingModuleSymbol = _a.exportingModuleSymbol;
                var checker = program.getTypeChecker();
                var exportSymbol = ts.Debug.assertDefined(checker.getSymbolAtLocation(exportName), "Export name should resolve to a symbol");
                ts.FindAllReferences.Core.eachExportReference(program.getSourceFiles(), checker, cancellationToken, exportSymbol, exportingModuleSymbol, exportName.text, wasDefault, function (ref) {
                    var importingSourceFile = ref.getSourceFile();
                    if (wasDefault) {
                        changeDefaultToNamedImport(importingSourceFile, ref, changes, exportName.text);
                    }
                    else {
                        changeNamedToDefaultImport(importingSourceFile, ref, changes);
                    }
                });
            }
            function changeDefaultToNamedImport(importingSourceFile, ref, changes, exportName) {
                var parent = ref.parent;
                switch (parent.kind) {
                    case 194 /* PropertyAccessExpression */:
                        // `a.default` --> `a.foo`
                        changes.replaceNode(importingSourceFile, ref, ts.createIdentifier(exportName));
                        break;
                    case 258 /* ImportSpecifier */:
                    case 263 /* ExportSpecifier */: {
                        var spec = parent;
                        // `default as foo` --> `foo`, `default as bar` --> `foo as bar`
                        changes.replaceNode(importingSourceFile, spec, makeImportSpecifier(exportName, spec.name.text));
                        break;
                    }
                    case 255 /* ImportClause */: {
                        var clause = parent;
                        ts.Debug.assert(clause.name === ref, "Import clause name should match provided ref");
                        var spec = makeImportSpecifier(exportName, ref.text);
                        var namedBindings = clause.namedBindings;
                        if (!namedBindings) {
                            // `import foo from "./a";` --> `import { foo } from "./a";`
                            changes.replaceNode(importingSourceFile, ref, ts.createNamedImports([spec]));
                        }
                        else if (namedBindings.kind === 256 /* NamespaceImport */) {
                            // `import foo, * as a from "./a";` --> `import * as a from ".a/"; import { foo } from "./a";`
                            changes.deleteRange(importingSourceFile, { pos: ref.getStart(importingSourceFile), end: namedBindings.getStart(importingSourceFile) });
                            var quotePreference = ts.isStringLiteral(clause.parent.moduleSpecifier) ? ts.quotePreferenceFromString(clause.parent.moduleSpecifier, importingSourceFile) : 1 /* Double */;
                            var newImport = ts.makeImport(/*default*/ undefined, [makeImportSpecifier(exportName, ref.text)], clause.parent.moduleSpecifier, quotePreference);
                            changes.insertNodeAfter(importingSourceFile, clause.parent, newImport);
                        }
                        else {
                            // `import foo, { bar } from "./a"` --> `import { bar, foo } from "./a";`
                            changes.delete(importingSourceFile, ref);
                            changes.insertNodeAtEndOfList(importingSourceFile, namedBindings.elements, spec);
                        }
                        break;
                    }
                    default:
                        ts.Debug.failBadSyntaxKind(parent);
                }
            }
            function changeNamedToDefaultImport(importingSourceFile, ref, changes) {
                var parent = ref.parent;
                switch (parent.kind) {
                    case 194 /* PropertyAccessExpression */:
                        // `a.foo` --> `a.default`
                        changes.replaceNode(importingSourceFile, ref, ts.createIdentifier("default"));
                        break;
                    case 258 /* ImportSpecifier */: {
                        // `import { foo } from "./a";` --> `import foo from "./a";`
                        // `import { foo as bar } from "./a";` --> `import bar from "./a";`
                        var defaultImport = ts.createIdentifier(parent.name.text);
                        if (parent.parent.elements.length === 1) {
                            changes.replaceNode(importingSourceFile, parent.parent, defaultImport);
                        }
                        else {
                            changes.delete(importingSourceFile, parent);
                            changes.insertNodeBefore(importingSourceFile, parent.parent, defaultImport);
                        }
                        break;
                    }
                    case 263 /* ExportSpecifier */: {
                        // `export { foo } from "./a";` --> `export { default as foo } from "./a";`
                        // `export { foo as bar } from "./a";` --> `export { default as bar } from "./a";`
                        // `export { foo as default } from "./a";` --> `export { default } from "./a";`
                        // (Because `export foo from "./a";` isn't valid syntax.)
                        changes.replaceNode(importingSourceFile, parent, makeExportSpecifier("default", parent.name.text));
                        break;
                    }
                    default:
                        ts.Debug.assertNever(parent, "Unexpected parent kind " + parent.kind);
                }
            }
            function makeImportSpecifier(propertyName, name) {
                return ts.createImportSpecifier(propertyName === name ? undefined : ts.createIdentifier(propertyName), ts.createIdentifier(name));
            }
            function makeExportSpecifier(propertyName, name) {
                return ts.createExportSpecifier(propertyName === name ? undefined : ts.createIdentifier(propertyName), ts.createIdentifier(name));
            }
        })(refactor = ts.refactor || (ts.refactor = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var refactor;
        (function (refactor) {
            var refactorName = "Convert import";
            var actionNameNamespaceToNamed = "Convert namespace import to named imports";
            var actionNameNamedToNamespace = "Convert named imports to namespace import";
            refactor.registerRefactor(refactorName, {
                getAvailableActions: function (context) {
                    var i = getImportToConvert(context);
                    if (!i)
                        return ts.emptyArray;
                    var description = i.kind === 256 /* NamespaceImport */ ? ts.Diagnostics.Convert_namespace_import_to_named_imports.message : ts.Diagnostics.Convert_named_imports_to_namespace_import.message;
                    var actionName = i.kind === 256 /* NamespaceImport */ ? actionNameNamespaceToNamed : actionNameNamedToNamespace;
                    return [{ name: refactorName, description: description, actions: [{ name: actionName, description: description }] }];
                },
                getEditsForAction: function (context, actionName) {
                    ts.Debug.assert(actionName === actionNameNamespaceToNamed || actionName === actionNameNamedToNamespace, "Unexpected action name");
                    var edits = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(context.file, context.program, t, ts.Debug.assertDefined(getImportToConvert(context), "Context must provide an import to convert")); });
                    return { edits: edits, renameFilename: undefined, renameLocation: undefined };
                }
            });
            // Can convert imports of the form `import * as m from "m";` or `import d, { x, y } from "m";`.
            function getImportToConvert(context) {
                var file = context.file;
                var span = ts.getRefactorContextSpan(context);
                var token = ts.getTokenAtPosition(file, span.start);
                var importDecl = ts.getParentNodeInSpan(token, file, span);
                if (!importDecl || !ts.isImportDeclaration(importDecl))
                    return undefined;
                var importClause = importDecl.importClause;
                return importClause && importClause.namedBindings;
            }
            function doChange(sourceFile, program, changes, toConvert) {
                var checker = program.getTypeChecker();
                if (toConvert.kind === 256 /* NamespaceImport */) {
                    doChangeNamespaceToNamed(sourceFile, checker, changes, toConvert, ts.getAllowSyntheticDefaultImports(program.getCompilerOptions()));
                }
                else {
                    doChangeNamedToNamespace(sourceFile, checker, changes, toConvert);
                }
            }
            function doChangeNamespaceToNamed(sourceFile, checker, changes, toConvert, allowSyntheticDefaultImports) {
                var usedAsNamespaceOrDefault = false;
                var nodesToReplace = [];
                var conflictingNames = ts.createMap();
                ts.FindAllReferences.Core.eachSymbolReferenceInFile(toConvert.name, checker, sourceFile, function (id) {
                    if (!ts.isPropertyAccessExpression(id.parent)) {
                        usedAsNamespaceOrDefault = true;
                    }
                    else {
                        var parent = ts.cast(id.parent, ts.isPropertyAccessExpression);
                        var exportName = parent.name.text;
                        if (checker.resolveName(exportName, id, 67108863 /* All */, /*excludeGlobals*/ true)) {
                            conflictingNames.set(exportName, true);
                        }
                        ts.Debug.assert(parent.expression === id, "Parent expression should match id");
                        nodesToReplace.push(parent);
                    }
                });
                // We may need to change `mod.x` to `_x` to avoid a name conflict.
                var exportNameToImportName = ts.createMap();
                for (var _i = 0, nodesToReplace_1 = nodesToReplace; _i < nodesToReplace_1.length; _i++) {
                    var propertyAccess = nodesToReplace_1[_i];
                    var exportName = propertyAccess.name.text;
                    var importName = exportNameToImportName.get(exportName);
                    if (importName === undefined) {
                        exportNameToImportName.set(exportName, importName = conflictingNames.has(exportName) ? ts.getUniqueName(exportName, sourceFile) : exportName);
                    }
                    changes.replaceNode(sourceFile, propertyAccess, ts.createIdentifier(importName));
                }
                var importSpecifiers = [];
                exportNameToImportName.forEach(function (name, propertyName) {
                    importSpecifiers.push(ts.createImportSpecifier(name === propertyName ? undefined : ts.createIdentifier(propertyName), ts.createIdentifier(name)));
                });
                var importDecl = toConvert.parent.parent;
                if (usedAsNamespaceOrDefault && !allowSyntheticDefaultImports) {
                    // Need to leave the namespace import alone
                    changes.insertNodeAfter(sourceFile, importDecl, updateImport(importDecl, /*defaultImportName*/ undefined, importSpecifiers));
                }
                else {
                    changes.replaceNode(sourceFile, importDecl, updateImport(importDecl, usedAsNamespaceOrDefault ? ts.createIdentifier(toConvert.name.text) : undefined, importSpecifiers));
                }
            }
            function doChangeNamedToNamespace(sourceFile, checker, changes, toConvert) {
                var importDecl = toConvert.parent.parent;
                var moduleSpecifier = importDecl.moduleSpecifier;
                var preferredName = moduleSpecifier && ts.isStringLiteral(moduleSpecifier) ? ts.codefix.moduleSpecifierToValidIdentifier(moduleSpecifier.text, 99 /* ESNext */) : "module";
                var namespaceNameConflicts = toConvert.elements.some(function (element) {
                    return ts.FindAllReferences.Core.eachSymbolReferenceInFile(element.name, checker, sourceFile, function (id) {
                        return !!checker.resolveName(preferredName, id, 67108863 /* All */, /*excludeGlobals*/ true);
                    }) || false;
                });
                var namespaceImportName = namespaceNameConflicts ? ts.getUniqueName(preferredName, sourceFile) : preferredName;
                var neededNamedImports = [];
                var _loop_15 = function (element) {
                    var propertyName = (element.propertyName || element.name).text;
                    ts.FindAllReferences.Core.eachSymbolReferenceInFile(element.name, checker, sourceFile, function (id) {
                        var access = ts.createPropertyAccess(ts.createIdentifier(namespaceImportName), propertyName);
                        if (ts.isShorthandPropertyAssignment(id.parent)) {
                            changes.replaceNode(sourceFile, id.parent, ts.createPropertyAssignment(id.text, access));
                        }
                        else if (ts.isExportSpecifier(id.parent) && !id.parent.propertyName) {
                            if (!neededNamedImports.some(function (n) { return n.name === element.name; })) {
                                neededNamedImports.push(ts.createImportSpecifier(element.propertyName && ts.createIdentifier(element.propertyName.text), ts.createIdentifier(element.name.text)));
                            }
                        }
                        else {
                            changes.replaceNode(sourceFile, id, access);
                        }
                    });
                };
                for (var _i = 0, _a = toConvert.elements; _i < _a.length; _i++) {
                    var element = _a[_i];
                    _loop_15(element);
                }
                changes.replaceNode(sourceFile, toConvert, ts.createNamespaceImport(ts.createIdentifier(namespaceImportName)));
                if (neededNamedImports.length) {
                    changes.insertNodeAfter(sourceFile, toConvert.parent.parent, updateImport(importDecl, /*defaultImportName*/ undefined, neededNamedImports));
                }
            }
            function updateImport(old, defaultImportName, elements) {
                return ts.createImportDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, ts.createImportClause(defaultImportName, elements && elements.length ? ts.createNamedImports(elements) : undefined), old.moduleSpecifier);
            }
        })(refactor = ts.refactor || (ts.refactor = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var refactor;
        (function (refactor) {
            var extractSymbol;
            (function (extractSymbol) {
                var refactorName = "Extract Symbol";
                refactor.registerRefactor(refactorName, { getAvailableActions: getAvailableActions, getEditsForAction: getEditsForAction });
                /**
                 * Compute the associated code actions
                 * Exported for tests.
                 */
                function getAvailableActions(context) {
                    var rangeToExtract = getRangeToExtract(context.file, ts.getRefactorContextSpan(context));
                    var targetRange = rangeToExtract.targetRange;
                    if (targetRange === undefined) {
                        return ts.emptyArray;
                    }
                    var extractions = getPossibleExtractions(targetRange, context);
                    if (extractions === undefined) {
                        // No extractions possible
                        return ts.emptyArray;
                    }
                    var functionActions = [];
                    var usedFunctionNames = ts.createMap();
                    var constantActions = [];
                    var usedConstantNames = ts.createMap();
                    var i = 0;
                    for (var _i = 0, extractions_1 = extractions; _i < extractions_1.length; _i++) {
                        var _a = extractions_1[_i], functionExtraction = _a.functionExtraction, constantExtraction = _a.constantExtraction;
                        // Skip these since we don't have a way to report errors yet
                        if (functionExtraction.errors.length === 0) {
                            // Don't issue refactorings with duplicated names.
                            // Scopes come back in "innermost first" order, so extractions will
                            // preferentially go into nearer scopes
                            var description = functionExtraction.description;
                            if (!usedFunctionNames.has(description)) {
                                usedFunctionNames.set(description, true);
                                functionActions.push({
                                    description: description,
                                    name: "function_scope_" + i
                                });
                            }
                        }
                        // Skip these since we don't have a way to report errors yet
                        if (constantExtraction.errors.length === 0) {
                            // Don't issue refactorings with duplicated names.
                            // Scopes come back in "innermost first" order, so extractions will
                            // preferentially go into nearer scopes
                            var description = constantExtraction.description;
                            if (!usedConstantNames.has(description)) {
                                usedConstantNames.set(description, true);
                                constantActions.push({
                                    description: description,
                                    name: "constant_scope_" + i
                                });
                            }
                        }
                        // *do* increment i anyway because we'll look for the i-th scope
                        // later when actually doing the refactoring if the user requests it
                        i++;
                    }
                    var infos = [];
                    if (constantActions.length) {
                        infos.push({
                            name: refactorName,
                            description: ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_constant),
                            actions: constantActions
                        });
                    }
                    if (functionActions.length) {
                        infos.push({
                            name: refactorName,
                            description: ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_function),
                            actions: functionActions
                        });
                    }
                    return infos.length ? infos : ts.emptyArray;
                }
                extractSymbol.getAvailableActions = getAvailableActions;
                /* Exported for tests */
                function getEditsForAction(context, actionName) {
                    var rangeToExtract = getRangeToExtract(context.file, ts.getRefactorContextSpan(context));
                    var targetRange = rangeToExtract.targetRange; // TODO:GH#18217
                    var parsedFunctionIndexMatch = /^function_scope_(\d+)$/.exec(actionName);
                    if (parsedFunctionIndexMatch) {
                        var index = +parsedFunctionIndexMatch[1];
                        ts.Debug.assert(isFinite(index), "Expected to parse a finite number from the function scope index");
                        return getFunctionExtractionAtIndex(targetRange, context, index);
                    }
                    var parsedConstantIndexMatch = /^constant_scope_(\d+)$/.exec(actionName);
                    if (parsedConstantIndexMatch) {
                        var index = +parsedConstantIndexMatch[1];
                        ts.Debug.assert(isFinite(index), "Expected to parse a finite number from the constant scope index");
                        return getConstantExtractionAtIndex(targetRange, context, index);
                    }
                    ts.Debug.fail("Unrecognized action name");
                }
                extractSymbol.getEditsForAction = getEditsForAction;
                // Move these into diagnostic messages if they become user-facing
                var Messages;
                (function (Messages) {
                    function createMessage(message) {
                        return { message: message, code: 0, category: ts.DiagnosticCategory.Message, key: message };
                    }
                    Messages.cannotExtractRange = createMessage("Cannot extract range.");
                    Messages.cannotExtractImport = createMessage("Cannot extract import statement.");
                    Messages.cannotExtractSuper = createMessage("Cannot extract super call.");
                    Messages.cannotExtractJSDoc = createMessage("Cannot extract JSDoc.");
                    Messages.cannotExtractEmpty = createMessage("Cannot extract empty range.");
                    Messages.expressionExpected = createMessage("expression expected.");
                    Messages.uselessConstantType = createMessage("No reason to extract constant of type.");
                    Messages.statementOrExpressionExpected = createMessage("Statement or expression expected.");
                    Messages.cannotExtractRangeContainingConditionalBreakOrContinueStatements = createMessage("Cannot extract range containing conditional break or continue statements.");
                    Messages.cannotExtractRangeContainingConditionalReturnStatement = createMessage("Cannot extract range containing conditional return statement.");
                    Messages.cannotExtractRangeContainingLabeledBreakOrContinueStatementWithTargetOutsideOfTheRange = createMessage("Cannot extract range containing labeled break or continue with target outside of the range.");
                    Messages.cannotExtractRangeThatContainsWritesToReferencesLocatedOutsideOfTheTargetRangeInGenerators = createMessage("Cannot extract range containing writes to references located outside of the target range in generators.");
                    Messages.typeWillNotBeVisibleInTheNewScope = createMessage("Type will not visible in the new scope.");
                    Messages.functionWillNotBeVisibleInTheNewScope = createMessage("Function will not visible in the new scope.");
                    Messages.cannotExtractIdentifier = createMessage("Select more than a single identifier.");
                    Messages.cannotExtractExportedEntity = createMessage("Cannot extract exported declaration");
                    Messages.cannotWriteInExpression = createMessage("Cannot write back side-effects when extracting an expression");
                    Messages.cannotExtractReadonlyPropertyInitializerOutsideConstructor = createMessage("Cannot move initialization of read-only class property outside of the constructor");
                    Messages.cannotExtractAmbientBlock = createMessage("Cannot extract code from ambient contexts");
                    Messages.cannotAccessVariablesFromNestedScopes = createMessage("Cannot access variables from nested scopes");
                    Messages.cannotExtractToOtherFunctionLike = createMessage("Cannot extract method to a function-like scope that is not a function");
                    Messages.cannotExtractToJSClass = createMessage("Cannot extract constant to a class scope in JS");
                    Messages.cannotExtractToExpressionArrowFunction = createMessage("Cannot extract constant to an arrow function without a block");
                })(Messages = extractSymbol.Messages || (extractSymbol.Messages = {}));
                var RangeFacts;
                (function (RangeFacts) {
                    RangeFacts[RangeFacts["None"] = 0] = "None";
                    RangeFacts[RangeFacts["HasReturn"] = 1] = "HasReturn";
                    RangeFacts[RangeFacts["IsGenerator"] = 2] = "IsGenerator";
                    RangeFacts[RangeFacts["IsAsyncFunction"] = 4] = "IsAsyncFunction";
                    RangeFacts[RangeFacts["UsesThis"] = 8] = "UsesThis";
                    /**
                     * The range is in a function which needs the 'static' modifier in a class
                     */
                    RangeFacts[RangeFacts["InStaticRegion"] = 16] = "InStaticRegion";
                })(RangeFacts || (RangeFacts = {}));
                /**
                 * getRangeToExtract takes a span inside a text file and returns either an expression or an array
                 * of statements representing the minimum set of nodes needed to extract the entire span. This
                 * process may fail, in which case a set of errors is returned instead (these are currently
                 * not shown to the user, but can be used by us diagnostically)
                 */
                // exported only for tests
                function getRangeToExtract(sourceFile, span) {
                    var length = span.length;
                    if (length === 0) {
                        return { errors: [ts.createFileDiagnostic(sourceFile, span.start, length, Messages.cannotExtractEmpty)] };
                    }
                    // Walk up starting from the the start position until we find a non-SourceFile node that subsumes the selected span.
                    // This may fail (e.g. you select two statements in the root of a source file)
                    var start = ts.getParentNodeInSpan(ts.getTokenAtPosition(sourceFile, span.start), sourceFile, span);
                    // Do the same for the ending position
                    var end = ts.getParentNodeInSpan(ts.findTokenOnLeftOfPosition(sourceFile, ts.textSpanEnd(span)), sourceFile, span);
                    var declarations = [];
                    // We'll modify these flags as we walk the tree to collect data
                    // about what things need to be done as part of the extraction.
                    var rangeFacts = RangeFacts.None;
                    if (!start || !end) {
                        // cannot find either start or end node
                        return { errors: [ts.createFileDiagnostic(sourceFile, span.start, length, Messages.cannotExtractRange)] };
                    }
                    if (start.parent !== end.parent) {
                        // start and end nodes belong to different subtrees
                        return { errors: [ts.createFileDiagnostic(sourceFile, span.start, length, Messages.cannotExtractRange)] };
                    }
                    if (start !== end) {
                        // start and end should be statements and parent should be either block or a source file
                        if (!isBlockLike(start.parent)) {
                            return { errors: [ts.createFileDiagnostic(sourceFile, span.start, length, Messages.cannotExtractRange)] };
                        }
                        var statements = [];
                        var start2 = start; // TODO: GH#18217 Need to alias `start` to get this to compile. See https://github.com/Microsoft/TypeScript/issues/19955#issuecomment-344118248
                        for (var _i = 0, _a = start2.parent.statements; _i < _a.length; _i++) {
                            var statement = _a[_i];
                            if (statement === start || statements.length) {
                                var errors_1 = checkNode(statement);
                                if (errors_1) {
                                    return { errors: errors_1 };
                                }
                                statements.push(statement);
                            }
                            if (statement === end) {
                                break;
                            }
                        }
                        if (!statements.length) {
                            // https://github.com/Microsoft/TypeScript/issues/20559
                            // Ranges like [|case 1: break;|] will fail to populate `statements` because
                            // they will never find `start` in `start.parent.statements`.
                            // Consider: We could support ranges like [|case 1:|] by refining them to just
                            // the expression.
                            return { errors: [ts.createFileDiagnostic(sourceFile, span.start, length, Messages.cannotExtractRange)] };
                        }
                        return { targetRange: { range: statements, facts: rangeFacts, declarations: declarations } };
                    }
                    if (ts.isJSDoc(start)) {
                        return { errors: [ts.createFileDiagnostic(sourceFile, span.start, length, Messages.cannotExtractJSDoc)] };
                    }
                    if (ts.isReturnStatement(start) && !start.expression) {
                        // Makes no sense to extract an expression-less return statement.
                        return { errors: [ts.createFileDiagnostic(sourceFile, span.start, length, Messages.cannotExtractRange)] };
                    }
                    // We have a single node (start)
                    var node = refineNode(start);
                    var errors = checkRootNode(node) || checkNode(node);
                    if (errors) {
                        return { errors: errors };
                    }
                    return { targetRange: { range: getStatementOrExpressionRange(node), facts: rangeFacts, declarations: declarations } }; // TODO: GH#18217
                    /**
                     * Attempt to refine the extraction node (generally, by shrinking it) to produce better results.
                     * @param node The unrefined extraction node.
                     */
                    function refineNode(node) {
                        if (ts.isReturnStatement(node)) {
                            if (node.expression) {
                                return node.expression;
                            }
                        }
                        else if (ts.isVariableStatement(node)) {
                            var numInitializers = 0;
                            var lastInitializer = void 0;
                            for (var _i = 0, _a = node.declarationList.declarations; _i < _a.length; _i++) {
                                var declaration = _a[_i];
                                if (declaration.initializer) {
                                    numInitializers++;
                                    lastInitializer = declaration.initializer;
                                }
                            }
                            if (numInitializers === 1) {
                                return lastInitializer;
                            }
                            // No special handling if there are multiple initializers.
                        }
                        else if (ts.isVariableDeclaration(node)) {
                            if (node.initializer) {
                                return node.initializer;
                            }
                        }
                        return node;
                    }
                    function checkRootNode(node) {
                        if (ts.isIdentifier(ts.isExpressionStatement(node) ? node.expression : node)) {
                            return [ts.createDiagnosticForNode(node, Messages.cannotExtractIdentifier)];
                        }
                        return undefined;
                    }
                    function checkForStaticContext(nodeToCheck, containingClass) {
                        var current = nodeToCheck;
                        while (current !== containingClass) {
                            if (current.kind === 159 /* PropertyDeclaration */) {
                                if (ts.hasModifier(current, 32 /* Static */)) {
                                    rangeFacts |= RangeFacts.InStaticRegion;
                                }
                                break;
                            }
                            else if (current.kind === 156 /* Parameter */) {
                                var ctorOrMethod = ts.getContainingFunction(current);
                                if (ctorOrMethod.kind === 162 /* Constructor */) {
                                    rangeFacts |= RangeFacts.InStaticRegion;
                                }
                                break;
                            }
                            else if (current.kind === 161 /* MethodDeclaration */) {
                                if (ts.hasModifier(current, 32 /* Static */)) {
                                    rangeFacts |= RangeFacts.InStaticRegion;
                                }
                            }
                            current = current.parent;
                        }
                    }
                    // Verifies whether we can actually extract this node or not.
                    function checkNode(nodeToCheck) {
                        var PermittedJumps;
                        (function (PermittedJumps) {
                            PermittedJumps[PermittedJumps["None"] = 0] = "None";
                            PermittedJumps[PermittedJumps["Break"] = 1] = "Break";
                            PermittedJumps[PermittedJumps["Continue"] = 2] = "Continue";
                            PermittedJumps[PermittedJumps["Return"] = 4] = "Return";
                        })(PermittedJumps || (PermittedJumps = {}));
                        // We believe it's true because the node is from the (unmodified) tree.
                        ts.Debug.assert(nodeToCheck.pos <= nodeToCheck.end, "This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809 (1)");
                        // For understanding how skipTrivia functioned:
                        ts.Debug.assert(!ts.positionIsSynthesized(nodeToCheck.pos), "This failure could trigger https://github.com/Microsoft/TypeScript/issues/20809 (2)");
                        if (!ts.isStatement(nodeToCheck) && !(ts.isExpressionNode(nodeToCheck) && isExtractableExpression(nodeToCheck))) {
                            return [ts.createDiagnosticForNode(nodeToCheck, Messages.statementOrExpressionExpected)];
                        }
                        if (nodeToCheck.flags & 8388608 /* Ambient */) {
                            return [ts.createDiagnosticForNode(nodeToCheck, Messages.cannotExtractAmbientBlock)];
                        }
                        // If we're in a class, see whether we're in a static region (static property initializer, static method, class constructor parameter default)
                        var containingClass = ts.getContainingClass(nodeToCheck);
                        if (containingClass) {
                            checkForStaticContext(nodeToCheck, containingClass);
                        }
                        var errors;
                        var permittedJumps = 4 /* Return */;
                        var seenLabels;
                        visit(nodeToCheck);
                        return errors;
                        function visit(node) {
                            if (errors) {
                                // already found an error - can stop now
                                return true;
                            }
                            if (ts.isDeclaration(node)) {
                                var declaringNode = (node.kind === 242 /* VariableDeclaration */) ? node.parent.parent : node;
                                if (ts.hasModifier(declaringNode, 1 /* Export */)) {
                                    // TODO: GH#18217 Silly to use `errors ||` since it's definitely not defined (see top of `visit`)
                                    // Also, if we're only pushing one error, just use `let error: Diagnostic | undefined`!
                                    // Also TODO: GH#19956
                                    (errors || (errors = [])).push(ts.createDiagnosticForNode(node, Messages.cannotExtractExportedEntity));
                                    return true;
                                }
                                declarations.push(node.symbol);
                            }
                            // Some things can't be extracted in certain situations
                            switch (node.kind) {
                                case 254 /* ImportDeclaration */:
                                    (errors || (errors = [])).push(ts.createDiagnosticForNode(node, Messages.cannotExtractImport));
                                    return true;
                                case 102 /* SuperKeyword */:
                                    // For a super *constructor call*, we have to be extracting the entire class,
                                    // but a super *method call* simply implies a 'this' reference
                                    if (node.parent.kind === 196 /* CallExpression */) {
                                        // Super constructor call
                                        var containingClass_1 = ts.getContainingClass(node); // TODO:GH#18217
                                        if (containingClass_1.pos < span.start || containingClass_1.end >= (span.start + span.length)) {
                                            (errors || (errors = [])).push(ts.createDiagnosticForNode(node, Messages.cannotExtractSuper));
                                            return true;
                                        }
                                    }
                                    else {
                                        rangeFacts |= RangeFacts.UsesThis;
                                    }
                                    break;
                            }
                            if (ts.isFunctionLikeDeclaration(node) || ts.isClassLike(node)) {
                                switch (node.kind) {
                                    case 244 /* FunctionDeclaration */:
                                    case 245 /* ClassDeclaration */:
                                        if (ts.isSourceFile(node.parent) && node.parent.externalModuleIndicator === undefined) {
                                            // You cannot extract global declarations
                                            (errors || (errors = [])).push(ts.createDiagnosticForNode(node, Messages.functionWillNotBeVisibleInTheNewScope));
                                        }
                                        break;
                                }
                                // do not dive into functions or classes
                                return false;
                            }
                            var savedPermittedJumps = permittedJumps;
                            switch (node.kind) {
                                case 227 /* IfStatement */:
                                    permittedJumps = 0 /* None */;
                                    break;
                                case 240 /* TryStatement */:
                                    // forbid all jumps inside try blocks
                                    permittedJumps = 0 /* None */;
                                    break;
                                case 223 /* Block */:
                                    if (node.parent && node.parent.kind === 240 /* TryStatement */ && node.parent.finallyBlock === node) {
                                        // allow unconditional returns from finally blocks
                                        permittedJumps = 4 /* Return */;
                                    }
                                    break;
                                case 278 /* DefaultClause */:
                                case 277 /* CaseClause */:
                                    // allow unlabeled break inside case clauses
                                    permittedJumps |= 1 /* Break */;
                                    break;
                                default:
                                    if (ts.isIterationStatement(node, /*lookInLabeledStatements*/ false)) {
                                        // allow unlabeled break/continue inside loops
                                        permittedJumps |= 1 /* Break */ | 2 /* Continue */;
                                    }
                                    break;
                            }
                            switch (node.kind) {
                                case 183 /* ThisType */:
                                case 104 /* ThisKeyword */:
                                    rangeFacts |= RangeFacts.UsesThis;
                                    break;
                                case 238 /* LabeledStatement */: {
                                    var label = node.label;
                                    (seenLabels || (seenLabels = [])).push(label.escapedText);
                                    ts.forEachChild(node, visit);
                                    seenLabels.pop();
                                    break;
                                }
                                case 234 /* BreakStatement */:
                                case 233 /* ContinueStatement */: {
                                    var label = node.label;
                                    if (label) {
                                        if (!ts.contains(seenLabels, label.escapedText)) {
                                            // attempts to jump to label that is not in range to be extracted
                                            (errors || (errors = [])).push(ts.createDiagnosticForNode(node, Messages.cannotExtractRangeContainingLabeledBreakOrContinueStatementWithTargetOutsideOfTheRange));
                                        }
                                    }
                                    else {
                                        if (!(permittedJumps & (node.kind === 234 /* BreakStatement */ ? 1 /* Break */ : 2 /* Continue */))) {
                                            // attempt to break or continue in a forbidden context
                                            (errors || (errors = [])).push(ts.createDiagnosticForNode(node, Messages.cannotExtractRangeContainingConditionalBreakOrContinueStatements));
                                        }
                                    }
                                    break;
                                }
                                case 206 /* AwaitExpression */:
                                    rangeFacts |= RangeFacts.IsAsyncFunction;
                                    break;
                                case 212 /* YieldExpression */:
                                    rangeFacts |= RangeFacts.IsGenerator;
                                    break;
                                case 235 /* ReturnStatement */:
                                    if (permittedJumps & 4 /* Return */) {
                                        rangeFacts |= RangeFacts.HasReturn;
                                    }
                                    else {
                                        (errors || (errors = [])).push(ts.createDiagnosticForNode(node, Messages.cannotExtractRangeContainingConditionalReturnStatement));
                                    }
                                    break;
                                default:
                                    ts.forEachChild(node, visit);
                                    break;
                            }
                            permittedJumps = savedPermittedJumps;
                        }
                    }
                }
                extractSymbol.getRangeToExtract = getRangeToExtract;
                function getStatementOrExpressionRange(node) {
                    if (ts.isStatement(node)) {
                        return [node];
                    }
                    else if (ts.isExpressionNode(node)) {
                        // If our selection is the expression in an ExpressionStatement, expand
                        // the selection to include the enclosing Statement (this stops us
                        // from trying to care about the return value of the extracted function
                        // and eliminates double semicolon insertion in certain scenarios)
                        return ts.isExpressionStatement(node.parent) ? [node.parent] : node;
                    }
                    return undefined;
                }
                function isScope(node) {
                    return ts.isFunctionLikeDeclaration(node) || ts.isSourceFile(node) || ts.isModuleBlock(node) || ts.isClassLike(node);
                }
                /**
                 * Computes possible places we could extract the function into. For example,
                 * you may be able to extract into a class method *or* local closure *or* namespace function,
                 * depending on what's in the extracted body.
                 */
                function collectEnclosingScopes(range) {
                    var current = isReadonlyArray(range.range) ? ts.first(range.range) : range.range;
                    if (range.facts & RangeFacts.UsesThis) {
                        // if range uses this as keyword or as type inside the class then it can only be extracted to a method of the containing class
                        var containingClass = ts.getContainingClass(current);
                        if (containingClass) {
                            var containingFunction = ts.findAncestor(current, ts.isFunctionLikeDeclaration);
                            return containingFunction
                                ? [containingFunction, containingClass]
                                : [containingClass];
                        }
                    }
                    var scopes = [];
                    while (true) {
                        current = current.parent;
                        // A function parameter's initializer is actually in the outer scope, not the function declaration
                        if (current.kind === 156 /* Parameter */) {
                            // Skip all the way to the outer scope of the function that declared this parameter
                            current = ts.findAncestor(current, function (parent) { return ts.isFunctionLikeDeclaration(parent); }).parent;
                        }
                        // We want to find the nearest parent where we can place an "equivalent" sibling to the node we're extracting out of.
                        // Walk up to the closest parent of a place where we can logically put a sibling:
                        //  * Function declaration
                        //  * Class declaration or expression
                        //  * Module/namespace or source file
                        if (isScope(current)) {
                            scopes.push(current);
                            if (current.kind === 290 /* SourceFile */) {
                                return scopes;
                            }
                        }
                    }
                }
                function getFunctionExtractionAtIndex(targetRange, context, requestedChangesIndex) {
                    var _a = getPossibleExtractionsWorker(targetRange, context), scopes = _a.scopes, _b = _a.readsAndWrites, target = _b.target, usagesPerScope = _b.usagesPerScope, functionErrorsPerScope = _b.functionErrorsPerScope, exposedVariableDeclarations = _b.exposedVariableDeclarations;
                    ts.Debug.assert(!functionErrorsPerScope[requestedChangesIndex].length, "The extraction went missing? How?");
                    context.cancellationToken.throwIfCancellationRequested(); // TODO: GH#18217
                    return extractFunctionInScope(target, scopes[requestedChangesIndex], usagesPerScope[requestedChangesIndex], exposedVariableDeclarations, targetRange, context);
                }
                function getConstantExtractionAtIndex(targetRange, context, requestedChangesIndex) {
                    var _a = getPossibleExtractionsWorker(targetRange, context), scopes = _a.scopes, _b = _a.readsAndWrites, target = _b.target, usagesPerScope = _b.usagesPerScope, constantErrorsPerScope = _b.constantErrorsPerScope, exposedVariableDeclarations = _b.exposedVariableDeclarations;
                    ts.Debug.assert(!constantErrorsPerScope[requestedChangesIndex].length, "The extraction went missing? How?");
                    ts.Debug.assert(exposedVariableDeclarations.length === 0, "Extract constant accepted a range containing a variable declaration?");
                    context.cancellationToken.throwIfCancellationRequested();
                    var expression = ts.isExpression(target)
                        ? target
                        : target.statements[0].expression;
                    return extractConstantInScope(expression, scopes[requestedChangesIndex], usagesPerScope[requestedChangesIndex], targetRange.facts, context);
                }
                /**
                 * Given a piece of text to extract ('targetRange'), computes a list of possible extractions.
                 * Each returned ExtractResultForScope corresponds to a possible target scope and is either a set of changes
                 * or an error explaining why we can't extract into that scope.
                 */
                function getPossibleExtractions(targetRange, context) {
                    var _a = getPossibleExtractionsWorker(targetRange, context), scopes = _a.scopes, _b = _a.readsAndWrites, functionErrorsPerScope = _b.functionErrorsPerScope, constantErrorsPerScope = _b.constantErrorsPerScope;
                    // Need the inner type annotation to avoid https://github.com/Microsoft/TypeScript/issues/7547
                    var extractions = scopes.map(function (scope, i) {
                        var functionDescriptionPart = getDescriptionForFunctionInScope(scope);
                        var constantDescriptionPart = getDescriptionForConstantInScope(scope);
                        var scopeDescription = ts.isFunctionLikeDeclaration(scope)
                            ? getDescriptionForFunctionLikeDeclaration(scope)
                            : ts.isClassLike(scope)
                                ? getDescriptionForClassLikeDeclaration(scope)
                                : getDescriptionForModuleLikeDeclaration(scope);
                        var functionDescription;
                        var constantDescription;
                        if (scopeDescription === 1 /* Global */) {
                            functionDescription = ts.formatStringFromArgs(ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_to_0_in_1_scope), [functionDescriptionPart, "global"]);
                            constantDescription = ts.formatStringFromArgs(ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_to_0_in_1_scope), [constantDescriptionPart, "global"]);
                        }
                        else if (scopeDescription === 0 /* Module */) {
                            functionDescription = ts.formatStringFromArgs(ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_to_0_in_1_scope), [functionDescriptionPart, "module"]);
                            constantDescription = ts.formatStringFromArgs(ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_to_0_in_1_scope), [constantDescriptionPart, "module"]);
                        }
                        else {
                            functionDescription = ts.formatStringFromArgs(ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_to_0_in_1), [functionDescriptionPart, scopeDescription]);
                            constantDescription = ts.formatStringFromArgs(ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_to_0_in_1), [constantDescriptionPart, scopeDescription]);
                        }
                        // Customize the phrasing for the innermost scope to increase clarity.
                        if (i === 0 && !ts.isClassLike(scope)) {
                            constantDescription = ts.formatStringFromArgs(ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_to_0_in_enclosing_scope), [constantDescriptionPart]);
                        }
                        return {
                            functionExtraction: {
                                description: functionDescription,
                                errors: functionErrorsPerScope[i],
                            },
                            constantExtraction: {
                                description: constantDescription,
                                errors: constantErrorsPerScope[i],
                            },
                        };
                    });
                    return extractions;
                }
                function getPossibleExtractionsWorker(targetRange, context) {
                    var sourceFile = context.file;
                    var scopes = collectEnclosingScopes(targetRange);
                    var enclosingTextRange = getEnclosingTextRange(targetRange, sourceFile);
                    var readsAndWrites = collectReadsAndWrites(targetRange, scopes, enclosingTextRange, sourceFile, context.program.getTypeChecker(), context.cancellationToken);
                    return { scopes: scopes, readsAndWrites: readsAndWrites };
                }
                function getDescriptionForFunctionInScope(scope) {
                    return ts.isFunctionLikeDeclaration(scope)
                        ? "inner function"
                        : ts.isClassLike(scope)
                            ? "method"
                            : "function";
                }
                function getDescriptionForConstantInScope(scope) {
                    return ts.isClassLike(scope)
                        ? "readonly field"
                        : "constant";
                }
                function getDescriptionForFunctionLikeDeclaration(scope) {
                    switch (scope.kind) {
                        case 162 /* Constructor */:
                            return "constructor";
                        case 201 /* FunctionExpression */:
                        case 244 /* FunctionDeclaration */:
                            return scope.name
                                ? "function '" + scope.name.text + "'"
                                : "anonymous function";
                        case 202 /* ArrowFunction */:
                            return "arrow function";
                        case 161 /* MethodDeclaration */:
                            return "method '" + scope.name.getText() + "'";
                        case 163 /* GetAccessor */:
                            return "'get " + scope.name.getText() + "'";
                        case 164 /* SetAccessor */:
                            return "'set " + scope.name.getText() + "'";
                        default:
                            throw ts.Debug.assertNever(scope, "Unexpected scope kind " + scope.kind);
                    }
                }
                function getDescriptionForClassLikeDeclaration(scope) {
                    return scope.kind === 245 /* ClassDeclaration */
                        ? scope.name ? "class '" + scope.name.text + "'" : "anonymous class declaration"
                        : scope.name ? "class expression '" + scope.name.text + "'" : "anonymous class expression";
                }
                function getDescriptionForModuleLikeDeclaration(scope) {
                    return scope.kind === 250 /* ModuleBlock */
                        ? "namespace '" + scope.parent.name.getText() + "'"
                        : scope.externalModuleIndicator ? 0 /* Module */ : 1 /* Global */;
                }
                var SpecialScope;
                (function (SpecialScope) {
                    SpecialScope[SpecialScope["Module"] = 0] = "Module";
                    SpecialScope[SpecialScope["Global"] = 1] = "Global";
                })(SpecialScope || (SpecialScope = {}));
                /**
                 * Result of 'extractRange' operation for a specific scope.
                 * Stores either a list of changes that should be applied to extract a range or a list of errors
                 */
                function extractFunctionInScope(node, scope, _a, exposedVariableDeclarations, range, context) {
                    var usagesInScope = _a.usages, typeParameterUsages = _a.typeParameterUsages, substitutions = _a.substitutions;
                    var checker = context.program.getTypeChecker();
                    // Make a unique name for the extracted function
                    var file = scope.getSourceFile();
                    var functionNameText = ts.getUniqueName(ts.isClassLike(scope) ? "newMethod" : "newFunction", file);
                    var isJS = ts.isInJSFile(scope);
                    var functionName = ts.createIdentifier(functionNameText);
                    var returnType;
                    var parameters = [];
                    var callArguments = [];
                    var writes;
                    usagesInScope.forEach(function (usage, name) {
                        var typeNode;
                        if (!isJS) {
                            var type = checker.getTypeOfSymbolAtLocation(usage.symbol, usage.node);
                            // Widen the type so we don't emit nonsense annotations like "function fn(x: 3) {"
                            type = checker.getBaseTypeOfLiteralType(type);
                            typeNode = checker.typeToTypeNode(type, scope, 1 /* NoTruncation */);
                        }
                        var paramDecl = ts.createParameter(
                        /*decorators*/ undefined, 
                        /*modifiers*/ undefined, 
                        /*dotDotDotToken*/ undefined, 
                        /*name*/ name, 
                        /*questionToken*/ undefined, typeNode);
                        parameters.push(paramDecl);
                        if (usage.usage === 2 /* Write */) {
                            (writes || (writes = [])).push(usage);
                        }
                        callArguments.push(ts.createIdentifier(name));
                    });
                    var typeParametersAndDeclarations = ts.arrayFrom(typeParameterUsages.values()).map(function (type) { return ({ type: type, declaration: getFirstDeclaration(type) }); });
                    var sortedTypeParametersAndDeclarations = typeParametersAndDeclarations.sort(compareTypesByDeclarationOrder);
                    var typeParameters = sortedTypeParametersAndDeclarations.length === 0
                        ? undefined
                        : sortedTypeParametersAndDeclarations.map(function (t) { return t.declaration; });
                    // Strictly speaking, we should check whether each name actually binds to the appropriate type
                    // parameter.  In cases of shadowing, they may not.
                    var callTypeArguments = typeParameters !== undefined
                        ? typeParameters.map(function (decl) { return ts.createTypeReferenceNode(decl.name, /*typeArguments*/ undefined); })
                        : undefined;
                    // Provide explicit return types for contextually-typed functions
                    // to avoid problems when there are literal types present
                    if (ts.isExpression(node) && !isJS) {
                        var contextualType = checker.getContextualType(node);
                        returnType = checker.typeToTypeNode(contextualType, scope, 1 /* NoTruncation */); // TODO: GH#18217
                    }
                    var _b = transformFunctionBody(node, exposedVariableDeclarations, writes, substitutions, !!(range.facts & RangeFacts.HasReturn)), body = _b.body, returnValueProperty = _b.returnValueProperty;
                    ts.suppressLeadingAndTrailingTrivia(body);
                    var newFunction;
                    if (ts.isClassLike(scope)) {
                        // always create private method in TypeScript files
                        var modifiers = isJS ? [] : [ts.createToken(117 /* PrivateKeyword */)];
                        if (range.facts & RangeFacts.InStaticRegion) {
                            modifiers.push(ts.createToken(120 /* StaticKeyword */));
                        }
                        if (range.facts & RangeFacts.IsAsyncFunction) {
                            modifiers.push(ts.createToken(126 /* AsyncKeyword */));
                        }
                        newFunction = ts.createMethod(
                        /*decorators*/ undefined, modifiers.length ? modifiers : undefined, range.facts & RangeFacts.IsGenerator ? ts.createToken(41 /* AsteriskToken */) : undefined, functionName, 
                        /*questionToken*/ undefined, typeParameters, parameters, returnType, body);
                    }
                    else {
                        newFunction = ts.createFunctionDeclaration(
                        /*decorators*/ undefined, range.facts & RangeFacts.IsAsyncFunction ? [ts.createToken(126 /* AsyncKeyword */)] : undefined, range.facts & RangeFacts.IsGenerator ? ts.createToken(41 /* AsteriskToken */) : undefined, functionName, typeParameters, parameters, returnType, body);
                    }
                    var changeTracker = ts.textChanges.ChangeTracker.fromContext(context);
                    var minInsertionPos = (isReadonlyArray(range.range) ? ts.last(range.range) : range.range).end;
                    var nodeToInsertBefore = getNodeToInsertFunctionBefore(minInsertionPos, scope);
                    if (nodeToInsertBefore) {
                        changeTracker.insertNodeBefore(context.file, nodeToInsertBefore, newFunction, /*blankLineBetween*/ true);
                    }
                    else {
                        changeTracker.insertNodeAtEndOfScope(context.file, scope, newFunction);
                    }
                    var newNodes = [];
                    // replace range with function call
                    var called = getCalledExpression(scope, range, functionNameText);
                    var call = ts.createCall(called, callTypeArguments, // Note that no attempt is made to take advantage of type argument inference
                    callArguments);
                    if (range.facts & RangeFacts.IsGenerator) {
                        call = ts.createYield(ts.createToken(41 /* AsteriskToken */), call);
                    }
                    if (range.facts & RangeFacts.IsAsyncFunction) {
                        call = ts.createAwait(call);
                    }
                    if (exposedVariableDeclarations.length && !writes) {
                        // No need to mix declarations and writes.
                        // How could any variables be exposed if there's a return statement?
                        ts.Debug.assert(!returnValueProperty, "Expected no returnValueProperty");
                        ts.Debug.assert(!(range.facts & RangeFacts.HasReturn), "Expected RangeFacts.HasReturn flag to be unset");
                        if (exposedVariableDeclarations.length === 1) {
                            // Declaring exactly one variable: let x = newFunction();
                            var variableDeclaration = exposedVariableDeclarations[0];
                            newNodes.push(ts.createVariableStatement(
                            /*modifiers*/ undefined, ts.createVariableDeclarationList([ts.createVariableDeclaration(ts.getSynthesizedDeepClone(variableDeclaration.name), /*type*/ ts.getSynthesizedDeepClone(variableDeclaration.type), /*initializer*/ call)], // TODO (acasey): test binding patterns
                            variableDeclaration.parent.flags)));
                        }
                        else {
                            // Declaring multiple variables / return properties:
                            //   let {x, y} = newFunction();
                            var bindingElements = [];
                            var typeElements = [];
                            var commonNodeFlags = exposedVariableDeclarations[0].parent.flags;
                            var sawExplicitType = false;
                            for (var _i = 0, exposedVariableDeclarations_1 = exposedVariableDeclarations; _i < exposedVariableDeclarations_1.length; _i++) {
                                var variableDeclaration = exposedVariableDeclarations_1[_i];
                                bindingElements.push(ts.createBindingElement(
                                /*dotDotDotToken*/ undefined, 
                                /*propertyName*/ undefined, 
                                /*name*/ ts.getSynthesizedDeepClone(variableDeclaration.name)));
                                // Being returned through an object literal will have widened the type.
                                var variableType = checker.typeToTypeNode(checker.getBaseTypeOfLiteralType(checker.getTypeAtLocation(variableDeclaration)), scope, 1 /* NoTruncation */);
                                typeElements.push(ts.createPropertySignature(
                                /*modifiers*/ undefined, 
                                /*name*/ variableDeclaration.symbol.name, 
                                /*questionToken*/ undefined, 
                                /*type*/ variableType, 
                                /*initializer*/ undefined));
                                sawExplicitType = sawExplicitType || variableDeclaration.type !== undefined;
                                commonNodeFlags = commonNodeFlags & variableDeclaration.parent.flags;
                            }
                            var typeLiteral = sawExplicitType ? ts.createTypeLiteralNode(typeElements) : undefined;
                            if (typeLiteral) {
                                ts.setEmitFlags(typeLiteral, 1 /* SingleLine */);
                            }
                            newNodes.push(ts.createVariableStatement(
                            /*modifiers*/ undefined, ts.createVariableDeclarationList([ts.createVariableDeclaration(ts.createObjectBindingPattern(bindingElements), 
                                /*type*/ typeLiteral, 
                                /*initializer*/ call)], commonNodeFlags)));
                        }
                    }
                    else if (exposedVariableDeclarations.length || writes) {
                        if (exposedVariableDeclarations.length) {
                            // CONSIDER: we're going to create one statement per variable, but we could actually preserve their original grouping.
                            for (var _c = 0, exposedVariableDeclarations_2 = exposedVariableDeclarations; _c < exposedVariableDeclarations_2.length; _c++) {
                                var variableDeclaration = exposedVariableDeclarations_2[_c];
                                var flags = variableDeclaration.parent.flags;
                                if (flags & 2 /* Const */) {
                                    flags = (flags & ~2 /* Const */) | 1 /* Let */;
                                }
                                newNodes.push(ts.createVariableStatement(
                                /*modifiers*/ undefined, ts.createVariableDeclarationList([ts.createVariableDeclaration(variableDeclaration.symbol.name, getTypeDeepCloneUnionUndefined(variableDeclaration.type))], flags)));
                            }
                        }
                        if (returnValueProperty) {
                            // has both writes and return, need to create variable declaration to hold return value;
                            newNodes.push(ts.createVariableStatement(
                            /*modifiers*/ undefined, ts.createVariableDeclarationList([ts.createVariableDeclaration(returnValueProperty, getTypeDeepCloneUnionUndefined(returnType))], 1 /* Let */)));
                        }
                        var assignments = getPropertyAssignmentsForWritesAndVariableDeclarations(exposedVariableDeclarations, writes);
                        if (returnValueProperty) {
                            assignments.unshift(ts.createShorthandPropertyAssignment(returnValueProperty));
                        }
                        // propagate writes back
                        if (assignments.length === 1) {
                            // We would only have introduced a return value property if there had been
                            // other assignments to make.
                            ts.Debug.assert(!returnValueProperty, "Shouldn't have returnValueProperty here");
                            newNodes.push(ts.createStatement(ts.createAssignment(assignments[0].name, call)));
                            if (range.facts & RangeFacts.HasReturn) {
                                newNodes.push(ts.createReturn());
                            }
                        }
                        else {
                            // emit e.g.
                            //   { a, b, __return } = newFunction(a, b);
                            //   return __return;
                            newNodes.push(ts.createStatement(ts.createAssignment(ts.createObjectLiteral(assignments), call)));
                            if (returnValueProperty) {
                                newNodes.push(ts.createReturn(ts.createIdentifier(returnValueProperty)));
                            }
                        }
                    }
                    else {
                        if (range.facts & RangeFacts.HasReturn) {
                            newNodes.push(ts.createReturn(call));
                        }
                        else if (isReadonlyArray(range.range)) {
                            newNodes.push(ts.createStatement(call));
                        }
                        else {
                            newNodes.push(call);
                        }
                    }
                    if (isReadonlyArray(range.range)) {
                        changeTracker.replaceNodeRangeWithNodes(context.file, ts.first(range.range), ts.last(range.range), newNodes);
                    }
                    else {
                        changeTracker.replaceNodeWithNodes(context.file, range.range, newNodes);
                    }
                    var edits = changeTracker.getChanges();
                    var renameRange = isReadonlyArray(range.range) ? ts.first(range.range) : range.range;
                    var renameFilename = renameRange.getSourceFile().fileName;
                    var renameLocation = ts.getRenameLocation(edits, renameFilename, functionNameText, /*isDeclaredBeforeUse*/ false);
                    return { renameFilename: renameFilename, renameLocation: renameLocation, edits: edits };
                    function getTypeDeepCloneUnionUndefined(typeNode) {
                        if (typeNode === undefined) {
                            return undefined;
                        }
                        var clone = ts.getSynthesizedDeepClone(typeNode);
                        var withoutParens = clone;
                        while (ts.isParenthesizedTypeNode(withoutParens)) {
                            withoutParens = withoutParens.type;
                        }
                        return ts.isUnionTypeNode(withoutParens) && ts.find(withoutParens.types, function (t) { return t.kind === 146 /* UndefinedKeyword */; })
                            ? clone
                            : ts.createUnionTypeNode([clone, ts.createKeywordTypeNode(146 /* UndefinedKeyword */)]);
                    }
                }
                /**
                 * Result of 'extractRange' operation for a specific scope.
                 * Stores either a list of changes that should be applied to extract a range or a list of errors
                 */
                function extractConstantInScope(node, scope, _a, rangeFacts, context) {
                    var _b;
                    var substitutions = _a.substitutions;
                    var checker = context.program.getTypeChecker();
                    // Make a unique name for the extracted variable
                    var file = scope.getSourceFile();
                    var localNameText = ts.getUniqueName(ts.isClassLike(scope) ? "newProperty" : "newLocal", file);
                    var isJS = ts.isInJSFile(scope);
                    var variableType = isJS || !checker.isContextSensitive(node)
                        ? undefined
                        : checker.typeToTypeNode(checker.getContextualType(node), scope, 1 /* NoTruncation */); // TODO: GH#18217
                    var initializer = transformConstantInitializer(node, substitutions);
                    (_b = transformFunctionInitializerAndType(variableType, initializer), variableType = _b.variableType, initializer = _b.initializer);
                    ts.suppressLeadingAndTrailingTrivia(initializer);
                    var changeTracker = ts.textChanges.ChangeTracker.fromContext(context);
                    if (ts.isClassLike(scope)) {
                        ts.Debug.assert(!isJS, "Cannot extract to a JS class"); // See CannotExtractToJSClass
                        var modifiers = [];
                        modifiers.push(ts.createToken(117 /* PrivateKeyword */));
                        if (rangeFacts & RangeFacts.InStaticRegion) {
                            modifiers.push(ts.createToken(120 /* StaticKeyword */));
                        }
                        modifiers.push(ts.createToken(138 /* ReadonlyKeyword */));
                        var newVariable = ts.createProperty(
                        /*decorators*/ undefined, modifiers, localNameText, 
                        /*questionToken*/ undefined, variableType, initializer);
                        var localReference = ts.createPropertyAccess(rangeFacts & RangeFacts.InStaticRegion
                            ? ts.createIdentifier(scope.name.getText()) // TODO: GH#18217
                            : ts.createThis(), ts.createIdentifier(localNameText));
                        // Declare
                        var maxInsertionPos = node.pos;
                        var nodeToInsertBefore = getNodeToInsertPropertyBefore(maxInsertionPos, scope);
                        changeTracker.insertNodeBefore(context.file, nodeToInsertBefore, newVariable, /*blankLineBetween*/ true);
                        // Consume
                        changeTracker.replaceNode(context.file, node, localReference);
                    }
                    else {
                        var newVariableDeclaration = ts.createVariableDeclaration(localNameText, variableType, initializer);
                        // If the node is part of an initializer in a list of variable declarations, insert a new
                        // variable declaration into the list (in case it depends on earlier ones).
                        // CONSIDER: If the declaration list isn't const, we might want to split it into multiple
                        // lists so that the newly extracted one can be const.
                        var oldVariableDeclaration = getContainingVariableDeclarationIfInList(node, scope);
                        if (oldVariableDeclaration) {
                            // Declare
                            // CONSIDER: could detect that each is on a separate line (See `extractConstant_VariableList_MultipleLines` in `extractConstants.ts`)
                            changeTracker.insertNodeBefore(context.file, oldVariableDeclaration, newVariableDeclaration);
                            // Consume
                            var localReference = ts.createIdentifier(localNameText);
                            changeTracker.replaceNode(context.file, node, localReference);
                        }
                        else if (node.parent.kind === 226 /* ExpressionStatement */ && scope === ts.findAncestor(node, isScope)) {
                            // If the parent is an expression statement and the target scope is the immediately enclosing one,
                            // replace the statement with the declaration.
                            var newVariableStatement = ts.createVariableStatement(
                            /*modifiers*/ undefined, ts.createVariableDeclarationList([newVariableDeclaration], 2 /* Const */));
                            changeTracker.replaceNode(context.file, node.parent, newVariableStatement);
                        }
                        else {
                            var newVariableStatement = ts.createVariableStatement(
                            /*modifiers*/ undefined, ts.createVariableDeclarationList([newVariableDeclaration], 2 /* Const */));
                            // Declare
                            var nodeToInsertBefore = getNodeToInsertConstantBefore(node, scope);
                            if (nodeToInsertBefore.pos === 0) {
                                changeTracker.insertNodeAtTopOfFile(context.file, newVariableStatement, /*blankLineBetween*/ false);
                            }
                            else {
                                changeTracker.insertNodeBefore(context.file, nodeToInsertBefore, newVariableStatement, /*blankLineBetween*/ false);
                            }
                            // Consume
                            if (node.parent.kind === 226 /* ExpressionStatement */) {
                                // If the parent is an expression statement, delete it.
                                changeTracker.delete(context.file, node.parent);
                            }
                            else {
                                var localReference = ts.createIdentifier(localNameText);
                                changeTracker.replaceNode(context.file, node, localReference);
                            }
                        }
                    }
                    var edits = changeTracker.getChanges();
                    var renameFilename = node.getSourceFile().fileName;
                    var renameLocation = ts.getRenameLocation(edits, renameFilename, localNameText, /*isDeclaredBeforeUse*/ true);
                    return { renameFilename: renameFilename, renameLocation: renameLocation, edits: edits };
                    function transformFunctionInitializerAndType(variableType, initializer) {
                        // If no contextual type exists there is nothing to transfer to the function signature
                        if (variableType === undefined)
                            return { variableType: variableType, initializer: initializer };
                        // Only do this for function expressions and arrow functions that are not generic
                        if (!ts.isFunctionExpression(initializer) && !ts.isArrowFunction(initializer) || !!initializer.typeParameters)
                            return { variableType: variableType, initializer: initializer };
                        var functionType = checker.getTypeAtLocation(node);
                        var functionSignature = ts.singleOrUndefined(checker.getSignaturesOfType(functionType, 0 /* Call */));
                        // If no function signature, maybe there was an error, do nothing
                        if (!functionSignature)
                            return { variableType: variableType, initializer: initializer };
                        // If the function signature has generic type parameters we don't attempt to move the parameters
                        if (!!functionSignature.getTypeParameters())
                            return { variableType: variableType, initializer: initializer };
                        // We add parameter types if needed
                        var parameters = [];
                        var hasAny = false;
                        for (var _i = 0, _a = initializer.parameters; _i < _a.length; _i++) {
                            var p = _a[_i];
                            if (p.type) {
                                parameters.push(p);
                            }
                            else {
                                var paramType = checker.getTypeAtLocation(p);
                                if (paramType === checker.getAnyType())
                                    hasAny = true;
                                parameters.push(ts.updateParameter(p, p.decorators, p.modifiers, p.dotDotDotToken, p.name, p.questionToken, p.type || checker.typeToTypeNode(paramType, scope, 1 /* NoTruncation */), p.initializer));
                            }
                        }
                        // If a parameter was inferred as any we skip adding function parameters at all.
                        // Turning an implicit any (which under common settings is a error) to an explicit
                        // is probably actually a worse refactor outcome.
                        if (hasAny)
                            return { variableType: variableType, initializer: initializer };
                        variableType = undefined;
                        if (ts.isArrowFunction(initializer)) {
                            initializer = ts.updateArrowFunction(initializer, node.modifiers, initializer.typeParameters, parameters, initializer.type || checker.typeToTypeNode(functionSignature.getReturnType(), scope, 1 /* NoTruncation */), initializer.equalsGreaterThanToken, initializer.body);
                        }
                        else {
                            if (functionSignature && !!functionSignature.thisParameter) {
                                var firstParameter = ts.firstOrUndefined(parameters);
                                // If the function signature has a this parameter and if the first defined parameter is not the this parameter, we must add it
                                // Note: If this parameter was already there, it would have been previously updated with the type if not type was present
                                if ((!firstParameter || (ts.isIdentifier(firstParameter.name) && firstParameter.name.escapedText !== "this"))) {
                                    var thisType = checker.getTypeOfSymbolAtLocation(functionSignature.thisParameter, node);
                                    parameters.splice(0, 0, ts.createParameter(
                                    /* decorators */ undefined, 
                                    /* modifiers */ undefined, 
                                    /* dotDotDotToken */ undefined, "this", 
                                    /* questionToken */ undefined, checker.typeToTypeNode(thisType, scope, 1 /* NoTruncation */)));
                                }
                            }
                            initializer = ts.updateFunctionExpression(initializer, node.modifiers, initializer.asteriskToken, initializer.name, initializer.typeParameters, parameters, initializer.type || checker.typeToTypeNode(functionSignature.getReturnType(), scope, 1 /* NoTruncation */), initializer.body);
                        }
                        return { variableType: variableType, initializer: initializer };
                    }
                }
                function getContainingVariableDeclarationIfInList(node, scope) {
                    var prevNode;
                    while (node !== undefined && node !== scope) {
                        if (ts.isVariableDeclaration(node) &&
                            node.initializer === prevNode &&
                            ts.isVariableDeclarationList(node.parent) &&
                            node.parent.declarations.length > 1) {
                            return node;
                        }
                        prevNode = node;
                        node = node.parent;
                    }
                }
                function getFirstDeclaration(type) {
                    var firstDeclaration;
                    var symbol = type.symbol;
                    if (symbol && symbol.declarations) {
                        for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                            var declaration = _a[_i];
                            if (firstDeclaration === undefined || declaration.pos < firstDeclaration.pos) {
                                firstDeclaration = declaration;
                            }
                        }
                    }
                    return firstDeclaration;
                }
                function compareTypesByDeclarationOrder(_a, _b) {
                    var type1 = _a.type, declaration1 = _a.declaration;
                    var type2 = _b.type, declaration2 = _b.declaration;
                    return ts.compareProperties(declaration1, declaration2, "pos", ts.compareValues)
                        || ts.compareStringsCaseSensitive(type1.symbol ? type1.symbol.getName() : "", type2.symbol ? type2.symbol.getName() : "")
                        || ts.compareValues(type1.id, type2.id);
                }
                function getCalledExpression(scope, range, functionNameText) {
                    var functionReference = ts.createIdentifier(functionNameText);
                    if (ts.isClassLike(scope)) {
                        var lhs = range.facts & RangeFacts.InStaticRegion ? ts.createIdentifier(scope.name.text) : ts.createThis(); // TODO: GH#18217
                        return ts.createPropertyAccess(lhs, functionReference);
                    }
                    else {
                        return functionReference;
                    }
                }
                function transformFunctionBody(body, exposedVariableDeclarations, writes, substitutions, hasReturn) {
                    var hasWritesOrVariableDeclarations = writes !== undefined || exposedVariableDeclarations.length > 0;
                    if (ts.isBlock(body) && !hasWritesOrVariableDeclarations && substitutions.size === 0) {
                        // already block, no declarations or writes to propagate back, no substitutions - can use node as is
                        return { body: ts.createBlock(body.statements, /*multLine*/ true), returnValueProperty: undefined };
                    }
                    var returnValueProperty;
                    var ignoreReturns = false;
                    var statements = ts.createNodeArray(ts.isBlock(body) ? body.statements.slice(0) : [ts.isStatement(body) ? body : ts.createReturn(body)]);
                    // rewrite body if either there are writes that should be propagated back via return statements or there are substitutions
                    if (hasWritesOrVariableDeclarations || substitutions.size) {
                        var rewrittenStatements = ts.visitNodes(statements, visitor).slice();
                        if (hasWritesOrVariableDeclarations && !hasReturn && ts.isStatement(body)) {
                            // add return at the end to propagate writes back in case if control flow falls out of the function body
                            // it is ok to know that range has at least one return since it we only allow unconditional returns
                            var assignments = getPropertyAssignmentsForWritesAndVariableDeclarations(exposedVariableDeclarations, writes);
                            if (assignments.length === 1) {
                                rewrittenStatements.push(ts.createReturn(assignments[0].name));
                            }
                            else {
                                rewrittenStatements.push(ts.createReturn(ts.createObjectLiteral(assignments)));
                            }
                        }
                        return { body: ts.createBlock(rewrittenStatements, /*multiLine*/ true), returnValueProperty: returnValueProperty };
                    }
                    else {
                        return { body: ts.createBlock(statements, /*multiLine*/ true), returnValueProperty: undefined };
                    }
                    function visitor(node) {
                        if (!ignoreReturns && node.kind === 235 /* ReturnStatement */ && hasWritesOrVariableDeclarations) {
                            var assignments = getPropertyAssignmentsForWritesAndVariableDeclarations(exposedVariableDeclarations, writes);
                            if (node.expression) {
                                if (!returnValueProperty) {
                                    returnValueProperty = "__return";
                                }
                                assignments.unshift(ts.createPropertyAssignment(returnValueProperty, ts.visitNode(node.expression, visitor)));
                            }
                            if (assignments.length === 1) {
                                return ts.createReturn(assignments[0].name);
                            }
                            else {
                                return ts.createReturn(ts.createObjectLiteral(assignments));
                            }
                        }
                        else {
                            var oldIgnoreReturns = ignoreReturns;
                            ignoreReturns = ignoreReturns || ts.isFunctionLikeDeclaration(node) || ts.isClassLike(node);
                            var substitution = substitutions.get(ts.getNodeId(node).toString());
                            var result = substitution ? ts.getSynthesizedDeepClone(substitution) : ts.visitEachChild(node, visitor, ts.nullTransformationContext);
                            ignoreReturns = oldIgnoreReturns;
                            return result;
                        }
                    }
                }
                function transformConstantInitializer(initializer, substitutions) {
                    return substitutions.size
                        ? visitor(initializer)
                        : initializer;
                    function visitor(node) {
                        var substitution = substitutions.get(ts.getNodeId(node).toString());
                        return substitution ? ts.getSynthesizedDeepClone(substitution) : ts.visitEachChild(node, visitor, ts.nullTransformationContext);
                    }
                }
                function getStatementsOrClassElements(scope) {
                    if (ts.isFunctionLikeDeclaration(scope)) {
                        var body = scope.body; // TODO: GH#18217
                        if (ts.isBlock(body)) {
                            return body.statements;
                        }
                    }
                    else if (ts.isModuleBlock(scope) || ts.isSourceFile(scope)) {
                        return scope.statements;
                    }
                    else if (ts.isClassLike(scope)) {
                        return scope.members;
                    }
                    else {
                        ts.assertType(scope);
                    }
                    return ts.emptyArray;
                }
                /**
                 * If `scope` contains a function after `minPos`, then return the first such function.
                 * Otherwise, return `undefined`.
                 */
                function getNodeToInsertFunctionBefore(minPos, scope) {
                    return ts.find(getStatementsOrClassElements(scope), function (child) {
                        return child.pos >= minPos && ts.isFunctionLikeDeclaration(child) && !ts.isConstructorDeclaration(child);
                    });
                }
                function getNodeToInsertPropertyBefore(maxPos, scope) {
                    var members = scope.members;
                    ts.Debug.assert(members.length > 0, "Found no members"); // There must be at least one child, since we extracted from one.
                    var prevMember;
                    var allProperties = true;
                    for (var _i = 0, members_1 = members; _i < members_1.length; _i++) {
                        var member = members_1[_i];
                        if (member.pos > maxPos) {
                            return prevMember || members[0];
                        }
                        if (allProperties && !ts.isPropertyDeclaration(member)) {
                            // If it is non-vacuously true that all preceding members are properties,
                            // insert before the current member (i.e. at the end of the list of properties).
                            if (prevMember !== undefined) {
                                return member;
                            }
                            allProperties = false;
                        }
                        prevMember = member;
                    }
                    if (prevMember === undefined)
                        return ts.Debug.fail(); // If the loop didn't return, then it did set prevMember.
                    return prevMember;
                }
                function getNodeToInsertConstantBefore(node, scope) {
                    ts.Debug.assert(!ts.isClassLike(scope));
                    var prevScope;
                    for (var curr = node; curr !== scope; curr = curr.parent) {
                        if (isScope(curr)) {
                            prevScope = curr;
                        }
                    }
                    for (var curr = (prevScope || node).parent;; curr = curr.parent) {
                        if (isBlockLike(curr)) {
                            var prevStatement = void 0;
                            for (var _i = 0, _a = curr.statements; _i < _a.length; _i++) {
                                var statement = _a[_i];
                                if (statement.pos > node.pos) {
                                    break;
                                }
                                prevStatement = statement;
                            }
                            if (!prevStatement && ts.isCaseClause(curr)) {
                                // We must have been in the expression of the case clause.
                                ts.Debug.assert(ts.isSwitchStatement(curr.parent.parent), "Grandparent isn't a switch statement");
                                return curr.parent.parent;
                            }
                            // There must be at least one statement since we started in one.
                            return ts.Debug.assertDefined(prevStatement, "prevStatement failed to get set");
                        }
                        ts.Debug.assert(curr !== scope, "Didn't encounter a block-like before encountering scope");
                    }
                }
                function getPropertyAssignmentsForWritesAndVariableDeclarations(exposedVariableDeclarations, writes) {
                    var variableAssignments = ts.map(exposedVariableDeclarations, function (v) { return ts.createShorthandPropertyAssignment(v.symbol.name); });
                    var writeAssignments = ts.map(writes, function (w) { return ts.createShorthandPropertyAssignment(w.symbol.name); });
                    // TODO: GH#18217 `variableAssignments` not possibly undefined!
                    return variableAssignments === undefined
                        ? writeAssignments
                        : writeAssignments === undefined
                            ? variableAssignments
                            : variableAssignments.concat(writeAssignments);
                }
                function isReadonlyArray(v) {
                    return ts.isArray(v);
                }
                /**
                 * Produces a range that spans the entirety of nodes, given a selection
                 * that might start/end in the middle of nodes.
                 *
                 * For example, when the user makes a selection like this
                 *                     v---v
                 *   var someThing = foo + bar;
                 *  this returns     ^-------^
                 */
                function getEnclosingTextRange(targetRange, sourceFile) {
                    return isReadonlyArray(targetRange.range)
                        ? { pos: ts.first(targetRange.range).getStart(sourceFile), end: ts.last(targetRange.range).getEnd() }
                        : targetRange.range;
                }
                var Usage;
                (function (Usage) {
                    // value should be passed to extracted method
                    Usage[Usage["Read"] = 1] = "Read";
                    // value should be passed to extracted method and propagated back
                    Usage[Usage["Write"] = 2] = "Write";
                })(Usage || (Usage = {}));
                function collectReadsAndWrites(targetRange, scopes, enclosingTextRange, sourceFile, checker, cancellationToken) {
                    var allTypeParameterUsages = ts.createMap(); // Key is type ID
                    var usagesPerScope = [];
                    var substitutionsPerScope = [];
                    var functionErrorsPerScope = [];
                    var constantErrorsPerScope = [];
                    var visibleDeclarationsInExtractedRange = [];
                    var exposedVariableSymbolSet = ts.createMap(); // Key is symbol ID
                    var exposedVariableDeclarations = [];
                    var firstExposedNonVariableDeclaration;
                    var expression = !isReadonlyArray(targetRange.range)
                        ? targetRange.range
                        : targetRange.range.length === 1 && ts.isExpressionStatement(targetRange.range[0])
                            ? targetRange.range[0].expression
                            : undefined;
                    var expressionDiagnostic;
                    if (expression === undefined) {
                        var statements = targetRange.range;
                        var start = ts.first(statements).getStart();
                        var end = ts.last(statements).end;
                        expressionDiagnostic = ts.createFileDiagnostic(sourceFile, start, end - start, Messages.expressionExpected);
                    }
                    else if (checker.getTypeAtLocation(expression).flags & (16384 /* Void */ | 131072 /* Never */)) {
                        expressionDiagnostic = ts.createDiagnosticForNode(expression, Messages.uselessConstantType);
                    }
                    // initialize results
                    for (var _i = 0, scopes_1 = scopes; _i < scopes_1.length; _i++) {
                        var scope = scopes_1[_i];
                        usagesPerScope.push({ usages: ts.createMap(), typeParameterUsages: ts.createMap(), substitutions: ts.createMap() });
                        substitutionsPerScope.push(ts.createMap());
                        functionErrorsPerScope.push(ts.isFunctionLikeDeclaration(scope) && scope.kind !== 244 /* FunctionDeclaration */
                            ? [ts.createDiagnosticForNode(scope, Messages.cannotExtractToOtherFunctionLike)]
                            : []);
                        var constantErrors = [];
                        if (expressionDiagnostic) {
                            constantErrors.push(expressionDiagnostic);
                        }
                        if (ts.isClassLike(scope) && ts.isInJSFile(scope)) {
                            constantErrors.push(ts.createDiagnosticForNode(scope, Messages.cannotExtractToJSClass));
                        }
                        if (ts.isArrowFunction(scope) && !ts.isBlock(scope.body)) {
                            // TODO (https://github.com/Microsoft/TypeScript/issues/18924): allow this
                            constantErrors.push(ts.createDiagnosticForNode(scope, Messages.cannotExtractToExpressionArrowFunction));
                        }
                        constantErrorsPerScope.push(constantErrors);
                    }
                    var seenUsages = ts.createMap();
                    var target = isReadonlyArray(targetRange.range) ? ts.createBlock(targetRange.range) : targetRange.range;
                    var unmodifiedNode = isReadonlyArray(targetRange.range) ? ts.first(targetRange.range) : targetRange.range;
                    var inGenericContext = isInGenericContext(unmodifiedNode);
                    collectUsages(target);
                    // Unfortunately, this code takes advantage of the knowledge that the generated method
                    // will use the contextual type of an expression as the return type of the extracted
                    // method (and will therefore "use" all the types involved).
                    if (inGenericContext && !isReadonlyArray(targetRange.range)) {
                        var contextualType = checker.getContextualType(targetRange.range); // TODO: GH#18217
                        recordTypeParameterUsages(contextualType);
                    }
                    if (allTypeParameterUsages.size > 0) {
                        var seenTypeParameterUsages = ts.createMap(); // Key is type ID
                        var i_1 = 0;
                        for (var curr = unmodifiedNode; curr !== undefined && i_1 < scopes.length; curr = curr.parent) {
                            if (curr === scopes[i_1]) {
                                // Copy current contents of seenTypeParameterUsages into scope.
                                seenTypeParameterUsages.forEach(function (typeParameter, id) {
                                    usagesPerScope[i_1].typeParameterUsages.set(id, typeParameter);
                                });
                                i_1++;
                            }
                            // Note that we add the current node's type parameters *after* updating the corresponding scope.
                            if (ts.isDeclarationWithTypeParameters(curr)) {
                                for (var _a = 0, _b = ts.getEffectiveTypeParameterDeclarations(curr); _a < _b.length; _a++) {
                                    var typeParameterDecl = _b[_a];
                                    var typeParameter = checker.getTypeAtLocation(typeParameterDecl);
                                    if (allTypeParameterUsages.has(typeParameter.id.toString())) {
                                        seenTypeParameterUsages.set(typeParameter.id.toString(), typeParameter);
                                    }
                                }
                            }
                        }
                        // If we didn't get through all the scopes, then there were some that weren't in our
                        // parent chain (impossible at time of writing).  A conservative solution would be to
                        // copy allTypeParameterUsages into all remaining scopes.
                        ts.Debug.assert(i_1 === scopes.length, "Should have iterated all scopes");
                    }
                    // If there are any declarations in the extracted block that are used in the same enclosing
                    // lexical scope, we can't move the extraction "up" as those declarations will become unreachable
                    if (visibleDeclarationsInExtractedRange.length) {
                        var containingLexicalScopeOfExtraction = ts.isBlockScope(scopes[0], scopes[0].parent)
                            ? scopes[0]
                            : ts.getEnclosingBlockScopeContainer(scopes[0]);
                        ts.forEachChild(containingLexicalScopeOfExtraction, checkForUsedDeclarations);
                    }
                    var _loop_16 = function (i) {
                        var scopeUsages = usagesPerScope[i];
                        // Special case: in the innermost scope, all usages are available.
                        // (The computed value reflects the value at the top-level of the scope, but the
                        // local will actually be declared at the same level as the extracted expression).
                        if (i > 0 && (scopeUsages.usages.size > 0 || scopeUsages.typeParameterUsages.size > 0)) {
                            var errorNode = isReadonlyArray(targetRange.range) ? targetRange.range[0] : targetRange.range;
                            constantErrorsPerScope[i].push(ts.createDiagnosticForNode(errorNode, Messages.cannotAccessVariablesFromNestedScopes));
                        }
                        var hasWrite = false;
                        var readonlyClassPropertyWrite;
                        usagesPerScope[i].usages.forEach(function (value) {
                            if (value.usage === 2 /* Write */) {
                                hasWrite = true;
                                if (value.symbol.flags & 106500 /* ClassMember */ &&
                                    value.symbol.valueDeclaration &&
                                    ts.hasModifier(value.symbol.valueDeclaration, 64 /* Readonly */)) {
                                    readonlyClassPropertyWrite = value.symbol.valueDeclaration;
                                }
                            }
                        });
                        // If an expression was extracted, then there shouldn't have been any variable declarations.
                        ts.Debug.assert(isReadonlyArray(targetRange.range) || exposedVariableDeclarations.length === 0, "No variable declarations expected if something was extracted");
                        if (hasWrite && !isReadonlyArray(targetRange.range)) {
                            var diag = ts.createDiagnosticForNode(targetRange.range, Messages.cannotWriteInExpression);
                            functionErrorsPerScope[i].push(diag);
                            constantErrorsPerScope[i].push(diag);
                        }
                        else if (readonlyClassPropertyWrite && i > 0) {
                            var diag = ts.createDiagnosticForNode(readonlyClassPropertyWrite, Messages.cannotExtractReadonlyPropertyInitializerOutsideConstructor);
                            functionErrorsPerScope[i].push(diag);
                            constantErrorsPerScope[i].push(diag);
                        }
                        else if (firstExposedNonVariableDeclaration) {
                            var diag = ts.createDiagnosticForNode(firstExposedNonVariableDeclaration, Messages.cannotExtractExportedEntity);
                            functionErrorsPerScope[i].push(diag);
                            constantErrorsPerScope[i].push(diag);
                        }
                    };
                    for (var i = 0; i < scopes.length; i++) {
                        _loop_16(i);
                    }
                    return { target: target, usagesPerScope: usagesPerScope, functionErrorsPerScope: functionErrorsPerScope, constantErrorsPerScope: constantErrorsPerScope, exposedVariableDeclarations: exposedVariableDeclarations };
                    function isInGenericContext(node) {
                        return !!ts.findAncestor(node, function (n) { return ts.isDeclarationWithTypeParameters(n) && ts.getEffectiveTypeParameterDeclarations(n).length !== 0; });
                    }
                    function recordTypeParameterUsages(type) {
                        // PERF: This is potentially very expensive.  `type` could be a library type with
                        // a lot of properties, each of which the walker will visit.  Unfortunately, the
                        // solution isn't as trivial as filtering to user types because of (e.g.) Array.
                        var symbolWalker = checker.getSymbolWalker(function () { return (cancellationToken.throwIfCancellationRequested(), true); });
                        var visitedTypes = symbolWalker.walkType(type).visitedTypes;
                        for (var _i = 0, visitedTypes_1 = visitedTypes; _i < visitedTypes_1.length; _i++) {
                            var visitedType = visitedTypes_1[_i];
                            if (visitedType.isTypeParameter()) {
                                allTypeParameterUsages.set(visitedType.id.toString(), visitedType);
                            }
                        }
                    }
                    function collectUsages(node, valueUsage) {
                        if (valueUsage === void 0) {
                            valueUsage = 1 /* Read */;
                        }
                        if (inGenericContext) {
                            var type = checker.getTypeAtLocation(node);
                            recordTypeParameterUsages(type);
                        }
                        if (ts.isDeclaration(node) && node.symbol) {
                            visibleDeclarationsInExtractedRange.push(node);
                        }
                        if (ts.isAssignmentExpression(node)) {
                            // use 'write' as default usage for values
                            collectUsages(node.left, 2 /* Write */);
                            collectUsages(node.right);
                        }
                        else if (ts.isUnaryExpressionWithWrite(node)) {
                            collectUsages(node.operand, 2 /* Write */);
                        }
                        else if (ts.isPropertyAccessExpression(node) || ts.isElementAccessExpression(node)) {
                            // use 'write' as default usage for values
                            ts.forEachChild(node, collectUsages);
                        }
                        else if (ts.isIdentifier(node)) {
                            if (!node.parent) {
                                return;
                            }
                            if (ts.isQualifiedName(node.parent) && node !== node.parent.left) {
                                return;
                            }
                            if (ts.isPropertyAccessExpression(node.parent) && node !== node.parent.expression) {
                                return;
                            }
                            recordUsage(node, valueUsage, /*isTypeNode*/ ts.isPartOfTypeNode(node));
                        }
                        else {
                            ts.forEachChild(node, collectUsages);
                        }
                    }
                    function recordUsage(n, usage, isTypeNode) {
                        var symbolId = recordUsagebySymbol(n, usage, isTypeNode);
                        if (symbolId) {
                            for (var i = 0; i < scopes.length; i++) {
                                // push substitution from map<symbolId, subst> to map<nodeId, subst> to simplify rewriting
                                var substitution = substitutionsPerScope[i].get(symbolId);
                                if (substitution) {
                                    usagesPerScope[i].substitutions.set(ts.getNodeId(n).toString(), substitution);
                                }
                            }
                        }
                    }
                    function recordUsagebySymbol(identifier, usage, isTypeName) {
                        var symbol = getSymbolReferencedByIdentifier(identifier);
                        if (!symbol) {
                            // cannot find symbol - do nothing
                            return undefined;
                        }
                        var symbolId = ts.getSymbolId(symbol).toString();
                        var lastUsage = seenUsages.get(symbolId);
                        // there are two kinds of value usages
                        // - reads - if range contains a read from the value located outside of the range then value should be passed as a parameter
                        // - writes - if range contains a write to a value located outside the range the value should be passed as a parameter and
                        //   returned as a return value
                        // 'write' case is a superset of 'read' so if we already have processed 'write' of some symbol there is not need to handle 'read'
                        // since all information is already recorded
                        if (lastUsage && lastUsage >= usage) {
                            return symbolId;
                        }
                        seenUsages.set(symbolId, usage);
                        if (lastUsage) {
                            // if we get here this means that we are trying to handle 'write' and 'read' was already processed
                            // walk scopes and update existing records.
                            for (var _i = 0, usagesPerScope_1 = usagesPerScope; _i < usagesPerScope_1.length; _i++) {
                                var perScope = usagesPerScope_1[_i];
                                var prevEntry = perScope.usages.get(identifier.text);
                                if (prevEntry) {
                                    perScope.usages.set(identifier.text, { usage: usage, symbol: symbol, node: identifier });
                                }
                            }
                            return symbolId;
                        }
                        // find first declaration in this file
                        var decls = symbol.getDeclarations();
                        var declInFile = decls && ts.find(decls, function (d) { return d.getSourceFile() === sourceFile; });
                        if (!declInFile) {
                            return undefined;
                        }
                        if (ts.rangeContainsStartEnd(enclosingTextRange, declInFile.getStart(), declInFile.end)) {
                            // declaration is located in range to be extracted - do nothing
                            return undefined;
                        }
                        if (targetRange.facts & RangeFacts.IsGenerator && usage === 2 /* Write */) {
                            // this is write to a reference located outside of the target scope and range is extracted into generator
                            // currently this is unsupported scenario
                            var diag = ts.createDiagnosticForNode(identifier, Messages.cannotExtractRangeThatContainsWritesToReferencesLocatedOutsideOfTheTargetRangeInGenerators);
                            for (var _a = 0, functionErrorsPerScope_1 = functionErrorsPerScope; _a < functionErrorsPerScope_1.length; _a++) {
                                var errors = functionErrorsPerScope_1[_a];
                                errors.push(diag);
                            }
                            for (var _b = 0, constantErrorsPerScope_1 = constantErrorsPerScope; _b < constantErrorsPerScope_1.length; _b++) {
                                var errors = constantErrorsPerScope_1[_b];
                                errors.push(diag);
                            }
                        }
                        for (var i = 0; i < scopes.length; i++) {
                            var scope = scopes[i];
                            var resolvedSymbol = checker.resolveName(symbol.name, scope, symbol.flags, /*excludeGlobals*/ false);
                            if (resolvedSymbol === symbol) {
                                continue;
                            }
                            if (!substitutionsPerScope[i].has(symbolId)) {
                                var substitution = tryReplaceWithQualifiedNameOrPropertyAccess(symbol.exportSymbol || symbol, scope, isTypeName);
                                if (substitution) {
                                    substitutionsPerScope[i].set(symbolId, substitution);
                                }
                                else if (isTypeName) {
                                    // If the symbol is a type parameter that won't be in scope, we'll pass it as a type argument
                                    // so there's no problem.
                                    if (!(symbol.flags & 262144 /* TypeParameter */)) {
                                        var diag = ts.createDiagnosticForNode(identifier, Messages.typeWillNotBeVisibleInTheNewScope);
                                        functionErrorsPerScope[i].push(diag);
                                        constantErrorsPerScope[i].push(diag);
                                    }
                                }
                                else {
                                    usagesPerScope[i].usages.set(identifier.text, { usage: usage, symbol: symbol, node: identifier });
                                }
                            }
                        }
                        return symbolId;
                    }
                    function checkForUsedDeclarations(node) {
                        // If this node is entirely within the original extraction range, we don't need to do anything.
                        if (node === targetRange.range || (isReadonlyArray(targetRange.range) && targetRange.range.indexOf(node) >= 0)) {
                            return;
                        }
                        // Otherwise check and recurse.
                        var sym = ts.isIdentifier(node)
                            ? getSymbolReferencedByIdentifier(node)
                            : checker.getSymbolAtLocation(node);
                        if (sym) {
                            var decl = ts.find(visibleDeclarationsInExtractedRange, function (d) { return d.symbol === sym; });
                            if (decl) {
                                if (ts.isVariableDeclaration(decl)) {
                                    var idString = decl.symbol.id.toString();
                                    if (!exposedVariableSymbolSet.has(idString)) {
                                        exposedVariableDeclarations.push(decl);
                                        exposedVariableSymbolSet.set(idString, true);
                                    }
                                }
                                else {
                                    // CONSIDER: this includes binding elements, which we could
                                    // expose in the same way as variables.
                                    firstExposedNonVariableDeclaration = firstExposedNonVariableDeclaration || decl;
                                }
                            }
                        }
                        ts.forEachChild(node, checkForUsedDeclarations);
                    }
                    /**
                     * Return the symbol referenced by an identifier (even if it declares a different symbol).
                     */
                    function getSymbolReferencedByIdentifier(identifier) {
                        // If the identifier is both a property name and its value, we're only interested in its value
                        // (since the name is a declaration and will be included in the extracted range).
                        return identifier.parent && ts.isShorthandPropertyAssignment(identifier.parent) && identifier.parent.name === identifier
                            ? checker.getShorthandAssignmentValueSymbol(identifier.parent)
                            : checker.getSymbolAtLocation(identifier);
                    }
                    function tryReplaceWithQualifiedNameOrPropertyAccess(symbol, scopeDecl, isTypeNode) {
                        if (!symbol) {
                            return undefined;
                        }
                        var decls = symbol.getDeclarations();
                        if (decls && decls.some(function (d) { return d.parent === scopeDecl; })) {
                            return ts.createIdentifier(symbol.name);
                        }
                        var prefix = tryReplaceWithQualifiedNameOrPropertyAccess(symbol.parent, scopeDecl, isTypeNode);
                        if (prefix === undefined) {
                            return undefined;
                        }
                        return isTypeNode
                            ? ts.createQualifiedName(prefix, ts.createIdentifier(symbol.name))
                            : ts.createPropertyAccess(prefix, symbol.name);
                    }
                }
                /**
                 * Computes whether or not a node represents an expression in a position where it could
                 * be extracted.
                 * The isExpression() in utilities.ts returns some false positives we need to handle,
                 * such as `import x from 'y'` -- the 'y' is a StringLiteral but is *not* an expression
                 * in the sense of something that you could extract on
                 */
                function isExtractableExpression(node) {
                    var parent = node.parent;
                    switch (parent.kind) {
                        case 284 /* EnumMember */:
                            return false;
                    }
                    switch (node.kind) {
                        case 10 /* StringLiteral */:
                            return parent.kind !== 254 /* ImportDeclaration */ &&
                                parent.kind !== 258 /* ImportSpecifier */;
                        case 213 /* SpreadElement */:
                        case 189 /* ObjectBindingPattern */:
                        case 191 /* BindingElement */:
                            return false;
                        case 75 /* Identifier */:
                            return parent.kind !== 191 /* BindingElement */ &&
                                parent.kind !== 258 /* ImportSpecifier */ &&
                                parent.kind !== 263 /* ExportSpecifier */;
                    }
                    return true;
                }
                function isBlockLike(node) {
                    switch (node.kind) {
                        case 223 /* Block */:
                        case 290 /* SourceFile */:
                        case 250 /* ModuleBlock */:
                        case 277 /* CaseClause */:
                            return true;
                        default:
                            return false;
                    }
                }
            })(extractSymbol = refactor.extractSymbol || (refactor.extractSymbol = {}));
        })(refactor = ts.refactor || (ts.refactor = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var refactor;
        (function (refactor) {
            var refactorName = "Extract type";
            var extractToTypeAlias = "Extract to type alias";
            var extractToInterface = "Extract to interface";
            var extractToTypeDef = "Extract to typedef";
            refactor.registerRefactor(refactorName, {
                getAvailableActions: function (context) {
                    var info = getRangeToExtract(context);
                    if (!info)
                        return ts.emptyArray;
                    return [{
                            name: refactorName,
                            description: ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_type),
                            actions: info.isJS ? [{
                                    name: extractToTypeDef, description: ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_to_typedef)
                                }] : ts.append([{
                                    name: extractToTypeAlias, description: ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_to_type_alias)
                                }], info.typeElements && {
                                name: extractToInterface, description: ts.getLocaleSpecificMessage(ts.Diagnostics.Extract_to_interface)
                            })
                        }];
                },
                getEditsForAction: function (context, actionName) {
                    var file = context.file;
                    var info = ts.Debug.assertDefined(getRangeToExtract(context), "Expected to find a range to extract");
                    var name = ts.getUniqueName("NewType", file);
                    var edits = ts.textChanges.ChangeTracker.with(context, function (changes) {
                        switch (actionName) {
                            case extractToTypeAlias:
                                ts.Debug.assert(!info.isJS, "Invalid actionName/JS combo");
                                return doTypeAliasChange(changes, file, name, info);
                            case extractToTypeDef:
                                ts.Debug.assert(info.isJS, "Invalid actionName/JS combo");
                                return doTypedefChange(changes, file, name, info);
                            case extractToInterface:
                                ts.Debug.assert(!info.isJS && !!info.typeElements, "Invalid actionName/JS combo");
                                return doInterfaceChange(changes, file, name, info);
                            default:
                                ts.Debug.fail("Unexpected action name");
                        }
                    });
                    var renameFilename = file.fileName;
                    var renameLocation = ts.getRenameLocation(edits, renameFilename, name, /*preferLastLocation*/ false);
                    return { edits: edits, renameFilename: renameFilename, renameLocation: renameLocation };
                }
            });
            function getRangeToExtract(context) {
                var file = context.file, startPosition = context.startPosition;
                var isJS = ts.isSourceFileJS(file);
                var current = ts.getTokenAtPosition(file, startPosition);
                var range = ts.createTextRangeFromSpan(ts.getRefactorContextSpan(context));
                var selection = ts.findAncestor(current, (function (node) { return node.parent && rangeContainsSkipTrivia(range, node, file) && !rangeContainsSkipTrivia(range, node.parent, file); }));
                if (!selection || !ts.isTypeNode(selection))
                    return undefined;
                var checker = context.program.getTypeChecker();
                var firstStatement = ts.Debug.assertDefined(ts.findAncestor(selection, ts.isStatement), "Should find a statement");
                var typeParameters = collectTypeParameters(checker, selection, firstStatement, file);
                if (!typeParameters)
                    return undefined;
                var typeElements = flattenTypeLiteralNodeReference(checker, selection);
                return { isJS: isJS, selection: selection, firstStatement: firstStatement, typeParameters: typeParameters, typeElements: typeElements };
            }
            function flattenTypeLiteralNodeReference(checker, node) {
                if (!node)
                    return undefined;
                if (ts.isIntersectionTypeNode(node)) {
                    var result = [];
                    var seen_1 = ts.createMap();
                    for (var _i = 0, _a = node.types; _i < _a.length; _i++) {
                        var type = _a[_i];
                        var flattenedTypeMembers = flattenTypeLiteralNodeReference(checker, type);
                        if (!flattenedTypeMembers || !flattenedTypeMembers.every(function (type) { return type.name && ts.addToSeen(seen_1, ts.getNameFromPropertyName(type.name)); })) {
                            return undefined;
                        }
                        ts.addRange(result, flattenedTypeMembers);
                    }
                    return result;
                }
                else if (ts.isParenthesizedTypeNode(node)) {
                    return flattenTypeLiteralNodeReference(checker, node.type);
                }
                else if (ts.isTypeLiteralNode(node)) {
                    return node.members;
                }
                return undefined;
            }
            function rangeContainsSkipTrivia(r1, node, file) {
                return ts.rangeContainsStartEnd(r1, ts.skipTrivia(file.text, node.pos), node.end);
            }
            function collectTypeParameters(checker, selection, statement, file) {
                var result = [];
                return visitor(selection) ? undefined : result;
                function visitor(node) {
                    if (ts.isTypeReferenceNode(node)) {
                        if (ts.isIdentifier(node.typeName)) {
                            var symbol = checker.resolveName(node.typeName.text, node.typeName, 262144 /* TypeParameter */, /* excludeGlobals */ true);
                            if (symbol) {
                                var declaration = ts.cast(ts.first(symbol.declarations), ts.isTypeParameterDeclaration);
                                if (rangeContainsSkipTrivia(statement, declaration, file) && !rangeContainsSkipTrivia(selection, declaration, file)) {
                                    result.push(declaration);
                                }
                            }
                        }
                    }
                    else if (ts.isInferTypeNode(node)) {
                        var conditionalTypeNode = ts.findAncestor(node, function (n) { return ts.isConditionalTypeNode(n) && rangeContainsSkipTrivia(n.extendsType, node, file); });
                        if (!conditionalTypeNode || !rangeContainsSkipTrivia(selection, conditionalTypeNode, file)) {
                            return true;
                        }
                    }
                    else if ((ts.isTypePredicateNode(node) || ts.isThisTypeNode(node))) {
                        var functionLikeNode = ts.findAncestor(node.parent, ts.isFunctionLike);
                        if (functionLikeNode && functionLikeNode.type && rangeContainsSkipTrivia(functionLikeNode.type, node, file) && !rangeContainsSkipTrivia(selection, functionLikeNode, file)) {
                            return true;
                        }
                    }
                    else if (ts.isTypeQueryNode(node)) {
                        if (ts.isIdentifier(node.exprName)) {
                            var symbol = checker.resolveName(node.exprName.text, node.exprName, 111551 /* Value */, /* excludeGlobals */ false);
                            if (symbol && rangeContainsSkipTrivia(statement, symbol.valueDeclaration, file) && !rangeContainsSkipTrivia(selection, symbol.valueDeclaration, file)) {
                                return true;
                            }
                        }
                        else {
                            if (ts.isThisIdentifier(node.exprName.left) && !rangeContainsSkipTrivia(selection, node.parent, file)) {
                                return true;
                            }
                        }
                    }
                    return ts.forEachChild(node, visitor);
                }
            }
            function doTypeAliasChange(changes, file, name, info) {
                var firstStatement = info.firstStatement, selection = info.selection, typeParameters = info.typeParameters;
                var newTypeNode = ts.createTypeAliasDeclaration(
                /* decorators */ undefined, 
                /* modifiers */ undefined, name, typeParameters.map(function (id) { return ts.updateTypeParameterDeclaration(id, id.name, id.constraint, /* defaultType */ undefined); }), selection);
                changes.insertNodeBefore(file, firstStatement, newTypeNode, /* blankLineBetween */ true);
                changes.replaceNode(file, selection, ts.createTypeReferenceNode(name, typeParameters.map(function (id) { return ts.createTypeReferenceNode(id.name, /* typeArguments */ undefined); })));
            }
            function doInterfaceChange(changes, file, name, info) {
                var firstStatement = info.firstStatement, selection = info.selection, typeParameters = info.typeParameters, typeElements = info.typeElements;
                var newTypeNode = ts.createInterfaceDeclaration(
                /* decorators */ undefined, 
                /* modifiers */ undefined, name, typeParameters, 
                /* heritageClauses */ undefined, typeElements);
                changes.insertNodeBefore(file, firstStatement, newTypeNode, /* blankLineBetween */ true);
                changes.replaceNode(file, selection, ts.createTypeReferenceNode(name, typeParameters.map(function (id) { return ts.createTypeReferenceNode(id.name, /* typeArguments */ undefined); })));
            }
            function doTypedefChange(changes, file, name, info) {
                var firstStatement = info.firstStatement, selection = info.selection, typeParameters = info.typeParameters;
                var node = ts.createNode(321 /* JSDocTypedefTag */);
                node.tagName = ts.createIdentifier("typedef"); // TODO: jsdoc factory https://github.com/Microsoft/TypeScript/pull/29539
                node.fullName = ts.createIdentifier(name);
                node.name = node.fullName;
                node.typeExpression = ts.createJSDocTypeExpression(selection);
                var templates = [];
                ts.forEach(typeParameters, function (typeParameter) {
                    var constraint = ts.getEffectiveConstraintOfTypeParameter(typeParameter);
                    var template = ts.createNode(320 /* JSDocTemplateTag */);
                    template.tagName = ts.createIdentifier("template");
                    template.constraint = constraint && ts.cast(constraint, ts.isJSDocTypeExpression);
                    var parameter = ts.createNode(155 /* TypeParameter */);
                    parameter.name = typeParameter.name;
                    template.typeParameters = ts.createNodeArray([parameter]);
                    templates.push(template);
                });
                changes.insertNodeBefore(file, firstStatement, ts.createJSDocComment(/* comment */ undefined, ts.createNodeArray(ts.concatenate(templates, [node]))), /* blankLineBetween */ true);
                changes.replaceNode(file, selection, ts.createTypeReferenceNode(name, typeParameters.map(function (id) { return ts.createTypeReferenceNode(id.name, /* typeArguments */ undefined); })));
            }
        })(refactor = ts.refactor || (ts.refactor = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var refactor;
        (function (refactor) {
            var generateGetAccessorAndSetAccessor;
            (function (generateGetAccessorAndSetAccessor) {
                var actionName = "Generate 'get' and 'set' accessors";
                var actionDescription = ts.Diagnostics.Generate_get_and_set_accessors.message;
                refactor.registerRefactor(actionName, { getEditsForAction: getEditsForAction, getAvailableActions: getAvailableActions });
                function getAvailableActions(context) {
                    if (!getConvertibleFieldAtPosition(context))
                        return ts.emptyArray;
                    return [{
                            name: actionName,
                            description: actionDescription,
                            actions: [
                                {
                                    name: actionName,
                                    description: actionDescription
                                }
                            ]
                        }];
                }
                function getEditsForAction(context, _actionName) {
                    var file = context.file;
                    var fieldInfo = getConvertibleFieldAtPosition(context);
                    if (!fieldInfo)
                        return undefined;
                    var isJS = ts.isSourceFileJS(file);
                    var changeTracker = ts.textChanges.ChangeTracker.fromContext(context);
                    var isStatic = fieldInfo.isStatic, isReadonly = fieldInfo.isReadonly, fieldName = fieldInfo.fieldName, accessorName = fieldInfo.accessorName, originalName = fieldInfo.originalName, type = fieldInfo.type, container = fieldInfo.container, declaration = fieldInfo.declaration, renameAccessor = fieldInfo.renameAccessor;
                    ts.suppressLeadingAndTrailingTrivia(fieldName);
                    ts.suppressLeadingAndTrailingTrivia(declaration);
                    ts.suppressLeadingAndTrailingTrivia(container);
                    var isInClassLike = ts.isClassLike(container);
                    // avoid Readonly modifier because it will convert to get accessor
                    var modifierFlags = ts.getModifierFlags(declaration) & ~64 /* Readonly */;
                    var accessorModifiers = isInClassLike
                        ? !modifierFlags || modifierFlags & 8 /* Private */
                            ? getModifiers(isJS, isStatic, 119 /* PublicKeyword */)
                            : ts.createNodeArray(ts.createModifiersFromModifierFlags(modifierFlags))
                        : undefined;
                    var fieldModifiers = isInClassLike ? getModifiers(isJS, isStatic, 117 /* PrivateKeyword */) : undefined;
                    updateFieldDeclaration(changeTracker, file, declaration, fieldName, fieldModifiers);
                    var getAccessor = generateGetAccessor(fieldName, accessorName, type, accessorModifiers, isStatic, container);
                    ts.suppressLeadingAndTrailingTrivia(getAccessor);
                    insertAccessor(changeTracker, file, getAccessor, declaration, container);
                    if (isReadonly) {
                        // readonly modifier only existed in classLikeDeclaration
                        var constructor = ts.getFirstConstructorWithBody(container);
                        if (constructor) {
                            updateReadonlyPropertyInitializerStatementConstructor(changeTracker, file, constructor, fieldName.text, originalName);
                        }
                    }
                    else {
                        var setAccessor = generateSetAccessor(fieldName, accessorName, type, accessorModifiers, isStatic, container);
                        ts.suppressLeadingAndTrailingTrivia(setAccessor);
                        insertAccessor(changeTracker, file, setAccessor, declaration, container);
                    }
                    var edits = changeTracker.getChanges();
                    var renameFilename = file.fileName;
                    var nameNeedRename = renameAccessor ? accessorName : fieldName;
                    var renameLocationOffset = ts.isIdentifier(nameNeedRename) ? 0 : -1;
                    var renameLocation = renameLocationOffset + ts.getRenameLocation(edits, renameFilename, nameNeedRename.text, /*preferLastLocation*/ ts.isParameter(declaration));
                    return { renameFilename: renameFilename, renameLocation: renameLocation, edits: edits };
                }
                function isConvertibleName(name) {
                    return ts.isIdentifier(name) || ts.isStringLiteral(name);
                }
                function isAcceptedDeclaration(node) {
                    return ts.isParameterPropertyDeclaration(node, node.parent) || ts.isPropertyDeclaration(node) || ts.isPropertyAssignment(node);
                }
                function createPropertyName(name, originalName) {
                    return ts.isIdentifier(originalName) ? ts.createIdentifier(name) : ts.createLiteral(name);
                }
                function createAccessorAccessExpression(fieldName, isStatic, container) {
                    var leftHead = isStatic ? container.name : ts.createThis(); // TODO: GH#18217
                    return ts.isIdentifier(fieldName) ? ts.createPropertyAccess(leftHead, fieldName) : ts.createElementAccess(leftHead, ts.createLiteral(fieldName));
                }
                function getModifiers(isJS, isStatic, accessModifier) {
                    var modifiers = ts.append(!isJS ? [ts.createToken(accessModifier)] : undefined, isStatic ? ts.createToken(120 /* StaticKeyword */) : undefined);
                    return modifiers && ts.createNodeArray(modifiers);
                }
                function startsWithUnderscore(name) {
                    return name.charCodeAt(0) === 95 /* _ */;
                }
                function getConvertibleFieldAtPosition(context) {
                    var file = context.file, startPosition = context.startPosition, endPosition = context.endPosition;
                    var node = ts.getTokenAtPosition(file, startPosition);
                    var declaration = ts.findAncestor(node.parent, isAcceptedDeclaration);
                    // make sure declaration have AccessibilityModifier or Static Modifier or Readonly Modifier
                    var meaning = 28 /* AccessibilityModifier */ | 32 /* Static */ | 64 /* Readonly */;
                    if (!declaration || !ts.nodeOverlapsWithStartEnd(declaration.name, file, startPosition, endPosition) // TODO: GH#18217
                        || !isConvertibleName(declaration.name) || (ts.getModifierFlags(declaration) | meaning) !== meaning)
                        return undefined;
                    var name = declaration.name.text;
                    var startWithUnderscore = startsWithUnderscore(name);
                    var fieldName = createPropertyName(startWithUnderscore ? name : ts.getUniqueName("_" + name, file), declaration.name);
                    var accessorName = createPropertyName(startWithUnderscore ? ts.getUniqueName(name.substring(1), file) : name, declaration.name);
                    return {
                        isStatic: ts.hasStaticModifier(declaration),
                        isReadonly: ts.hasReadonlyModifier(declaration),
                        type: ts.getTypeAnnotationNode(declaration),
                        container: declaration.kind === 156 /* Parameter */ ? declaration.parent.parent : declaration.parent,
                        originalName: declaration.name.text,
                        declaration: declaration,
                        fieldName: fieldName,
                        accessorName: accessorName,
                        renameAccessor: startWithUnderscore
                    };
                }
                function generateGetAccessor(fieldName, accessorName, type, modifiers, isStatic, container) {
                    return ts.createGetAccessor(
                    /*decorators*/ undefined, modifiers, accessorName, 
                    /*parameters*/ undefined, // TODO: GH#18217
                    type, ts.createBlock([
                        ts.createReturn(createAccessorAccessExpression(fieldName, isStatic, container))
                    ], /*multiLine*/ true));
                }
                function generateSetAccessor(fieldName, accessorName, type, modifiers, isStatic, container) {
                    return ts.createSetAccessor(
                    /*decorators*/ undefined, modifiers, accessorName, [ts.createParameter(
                        /*decorators*/ undefined, 
                        /*modifiers*/ undefined, 
                        /*dotDotDotToken*/ undefined, ts.createIdentifier("value"), 
                        /*questionToken*/ undefined, type)], ts.createBlock([
                        ts.createStatement(ts.createAssignment(createAccessorAccessExpression(fieldName, isStatic, container), ts.createIdentifier("value")))
                    ], /*multiLine*/ true));
                }
                function updatePropertyDeclaration(changeTracker, file, declaration, fieldName, modifiers) {
                    var property = ts.updateProperty(declaration, declaration.decorators, modifiers, fieldName, declaration.questionToken || declaration.exclamationToken, declaration.type, declaration.initializer);
                    changeTracker.replaceNode(file, declaration, property);
                }
                function updatePropertyAssignmentDeclaration(changeTracker, file, declaration, fieldName) {
                    var assignment = ts.updatePropertyAssignment(declaration, fieldName, declaration.initializer);
                    changeTracker.replacePropertyAssignment(file, declaration, assignment);
                }
                function updateFieldDeclaration(changeTracker, file, declaration, fieldName, modifiers) {
                    if (ts.isPropertyDeclaration(declaration)) {
                        updatePropertyDeclaration(changeTracker, file, declaration, fieldName, modifiers);
                    }
                    else if (ts.isPropertyAssignment(declaration)) {
                        updatePropertyAssignmentDeclaration(changeTracker, file, declaration, fieldName);
                    }
                    else {
                        changeTracker.replaceNode(file, declaration, ts.updateParameter(declaration, declaration.decorators, modifiers, declaration.dotDotDotToken, ts.cast(fieldName, ts.isIdentifier), declaration.questionToken, declaration.type, declaration.initializer));
                    }
                }
                function insertAccessor(changeTracker, file, accessor, declaration, container) {
                    ts.isParameterPropertyDeclaration(declaration, declaration.parent) ? changeTracker.insertNodeAtClassStart(file, container, accessor) :
                        ts.isPropertyAssignment(declaration) ? changeTracker.insertNodeAfterComma(file, declaration, accessor) :
                            changeTracker.insertNodeAfter(file, declaration, accessor);
                }
                function updateReadonlyPropertyInitializerStatementConstructor(changeTracker, file, constructor, fieldName, originalName) {
                    if (!constructor.body)
                        return;
                    constructor.body.forEachChild(function recur(node) {
                        if (ts.isElementAccessExpression(node) &&
                            node.expression.kind === 104 /* ThisKeyword */ &&
                            ts.isStringLiteral(node.argumentExpression) &&
                            node.argumentExpression.text === originalName &&
                            ts.isWriteAccess(node)) {
                            changeTracker.replaceNode(file, node.argumentExpression, ts.createStringLiteral(fieldName));
                        }
                        if (ts.isPropertyAccessExpression(node) && node.expression.kind === 104 /* ThisKeyword */ && node.name.text === originalName && ts.isWriteAccess(node)) {
                            changeTracker.replaceNode(file, node.name, ts.createIdentifier(fieldName));
                        }
                        if (!ts.isFunctionLike(node) && !ts.isClassLike(node)) {
                            node.forEachChild(recur);
                        }
                    });
                }
            })(generateGetAccessorAndSetAccessor = refactor.generateGetAccessorAndSetAccessor || (refactor.generateGetAccessorAndSetAccessor = {}));
        })(refactor = ts.refactor || (ts.refactor = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var refactor;
        (function (refactor) {
            var refactorName = "Move to a new file";
            refactor.registerRefactor(refactorName, {
                getAvailableActions: function (context) {
                    if (!context.preferences.allowTextChangesInNewFiles || getStatementsToMove(context) === undefined)
                        return ts.emptyArray;
                    var description = ts.getLocaleSpecificMessage(ts.Diagnostics.Move_to_a_new_file);
                    return [{ name: refactorName, description: description, actions: [{ name: refactorName, description: description }] }];
                },
                getEditsForAction: function (context, actionName) {
                    ts.Debug.assert(actionName === refactorName, "Wrong refactor invoked");
                    var statements = ts.Debug.assertDefined(getStatementsToMove(context));
                    var edits = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(context.file, context.program, statements, t, context.host, context.preferences); });
                    return { edits: edits, renameFilename: undefined, renameLocation: undefined };
                }
            });
            function getRangeToMove(context) {
                var file = context.file;
                var range = ts.createTextRangeFromSpan(ts.getRefactorContextSpan(context));
                var statements = file.statements;
                var startNodeIndex = ts.findIndex(statements, function (s) { return s.end > range.pos; });
                if (startNodeIndex === -1)
                    return undefined;
                var startStatement = statements[startNodeIndex];
                if (ts.isNamedDeclaration(startStatement) && startStatement.name && ts.rangeContainsRange(startStatement.name, range)) {
                    return { toMove: [statements[startNodeIndex]], afterLast: statements[startNodeIndex + 1] };
                }
                // Can't only partially include the start node or be partially into the next node
                if (range.pos > startStatement.getStart(file))
                    return undefined;
                var afterEndNodeIndex = ts.findIndex(statements, function (s) { return s.end > range.end; }, startNodeIndex);
                // Can't be partially into the next node
                if (afterEndNodeIndex !== -1 && (afterEndNodeIndex === 0 || statements[afterEndNodeIndex].getStart(file) < range.end))
                    return undefined;
                return {
                    toMove: statements.slice(startNodeIndex, afterEndNodeIndex === -1 ? statements.length : afterEndNodeIndex),
                    afterLast: afterEndNodeIndex === -1 ? undefined : statements[afterEndNodeIndex],
                };
            }
            function doChange(oldFile, program, toMove, changes, host, preferences) {
                var checker = program.getTypeChecker();
                var usage = getUsageInfo(oldFile, toMove.all, checker);
                var currentDirectory = ts.getDirectoryPath(oldFile.fileName);
                var extension = ts.extensionFromPath(oldFile.fileName);
                var newModuleName = makeUniqueModuleName(getNewModuleName(usage.movedSymbols), extension, currentDirectory, host);
                var newFileNameWithExtension = newModuleName + extension;
                // If previous file was global, this is easy.
                changes.createNewFile(oldFile, ts.combinePaths(currentDirectory, newFileNameWithExtension), getNewStatementsAndRemoveFromOldFile(oldFile, usage, changes, toMove, program, newModuleName, preferences));
                addNewFileToTsconfig(program, changes, oldFile.fileName, newFileNameWithExtension, ts.hostGetCanonicalFileName(host));
            }
            // Filters imports out of the range of statements to move. Imports will be copied to the new file anyway, and may still be needed in the old file.
            function getStatementsToMove(context) {
                var rangeToMove = getRangeToMove(context);
                if (rangeToMove === undefined)
                    return undefined;
                var all = [];
                var ranges = [];
                var toMove = rangeToMove.toMove, afterLast = rangeToMove.afterLast;
                ts.getRangesWhere(toMove, function (s) { return !isPureImport(s); }, function (start, afterEndIndex) {
                    for (var i = start; i < afterEndIndex; i++)
                        all.push(toMove[i]);
                    ranges.push({ first: toMove[start], afterLast: afterLast });
                });
                return all.length === 0 ? undefined : { all: all, ranges: ranges };
            }
            function isPureImport(node) {
                switch (node.kind) {
                    case 254 /* ImportDeclaration */:
                        return true;
                    case 253 /* ImportEqualsDeclaration */:
                        return !ts.hasModifier(node, 1 /* Export */);
                    case 225 /* VariableStatement */:
                        return node.declarationList.declarations.every(function (d) { return !!d.initializer && ts.isRequireCall(d.initializer, /*checkArgumentIsStringLiteralLike*/ true); });
                    default:
                        return false;
                }
            }
            function addNewFileToTsconfig(program, changes, oldFileName, newFileNameWithExtension, getCanonicalFileName) {
                var cfg = program.getCompilerOptions().configFile;
                if (!cfg)
                    return;
                var newFileAbsolutePath = ts.normalizePath(ts.combinePaths(oldFileName, "..", newFileNameWithExtension));
                var newFilePath = ts.getRelativePathFromFile(cfg.fileName, newFileAbsolutePath, getCanonicalFileName);
                var cfgObject = cfg.statements[0] && ts.tryCast(cfg.statements[0].expression, ts.isObjectLiteralExpression);
                var filesProp = cfgObject && ts.find(cfgObject.properties, function (prop) {
                    return ts.isPropertyAssignment(prop) && ts.isStringLiteral(prop.name) && prop.name.text === "files";
                });
                if (filesProp && ts.isArrayLiteralExpression(filesProp.initializer)) {
                    changes.insertNodeInListAfter(cfg, ts.last(filesProp.initializer.elements), ts.createLiteral(newFilePath), filesProp.initializer.elements);
                }
            }
            function getNewStatementsAndRemoveFromOldFile(oldFile, usage, changes, toMove, program, newModuleName, preferences) {
                var checker = program.getTypeChecker();
                if (!oldFile.externalModuleIndicator && !oldFile.commonJsModuleIndicator) {
                    deleteMovedStatements(oldFile, toMove.ranges, changes);
                    return toMove.all;
                }
                var useEs6ModuleSyntax = !!oldFile.externalModuleIndicator;
                var quotePreference = ts.getQuotePreference(oldFile, preferences);
                var importsFromNewFile = createOldFileImportsFromNewFile(usage.oldFileImportsFromNewFile, newModuleName, useEs6ModuleSyntax, quotePreference);
                if (importsFromNewFile) {
                    ts.insertImport(changes, oldFile, importsFromNewFile, /*blankLineBetween*/ true);
                }
                deleteUnusedOldImports(oldFile, toMove.all, changes, usage.unusedImportsFromOldFile, checker);
                deleteMovedStatements(oldFile, toMove.ranges, changes);
                updateImportsInOtherFiles(changes, program, oldFile, usage.movedSymbols, newModuleName);
                return __spreadArrays(getNewFileImportsAndAddExportInOldFile(oldFile, usage.oldImportsNeededByNewFile, usage.newFileImportsFromOldFile, changes, checker, useEs6ModuleSyntax, quotePreference), addExports(oldFile, toMove.all, usage.oldFileImportsFromNewFile, useEs6ModuleSyntax));
            }
            function deleteMovedStatements(sourceFile, moved, changes) {
                for (var _i = 0, moved_1 = moved; _i < moved_1.length; _i++) {
                    var _a = moved_1[_i], first_1 = _a.first, afterLast = _a.afterLast;
                    changes.deleteNodeRangeExcludingEnd(sourceFile, first_1, afterLast);
                }
            }
            function deleteUnusedOldImports(oldFile, toMove, changes, toDelete, checker) {
                for (var _i = 0, _a = oldFile.statements; _i < _a.length; _i++) {
                    var statement = _a[_i];
                    if (ts.contains(toMove, statement))
                        continue;
                    forEachImportInStatement(statement, function (i) { return deleteUnusedImports(oldFile, i, changes, function (name) { return toDelete.has(checker.getSymbolAtLocation(name)); }); });
                }
            }
            function updateImportsInOtherFiles(changes, program, oldFile, movedSymbols, newModuleName) {
                var checker = program.getTypeChecker();
                var _loop_17 = function (sourceFile) {
                    if (sourceFile === oldFile)
                        return "continue";
                    var _loop_18 = function (statement) {
                        forEachImportInStatement(statement, function (importNode) {
                            if (checker.getSymbolAtLocation(moduleSpecifierFromImport(importNode)) !== oldFile.symbol)
                                return;
                            var shouldMove = function (name) {
                                var symbol = ts.isBindingElement(name.parent)
                                    ? ts.getPropertySymbolFromBindingElement(checker, name.parent)
                                    : ts.skipAlias(checker.getSymbolAtLocation(name), checker); // TODO: GH#18217
                                return !!symbol && movedSymbols.has(symbol);
                            };
                            deleteUnusedImports(sourceFile, importNode, changes, shouldMove); // These will be changed to imports from the new file
                            var newModuleSpecifier = ts.combinePaths(ts.getDirectoryPath(moduleSpecifierFromImport(importNode).text), newModuleName);
                            var newImportDeclaration = filterImport(importNode, ts.createLiteral(newModuleSpecifier), shouldMove);
                            if (newImportDeclaration)
                                changes.insertNodeAfter(sourceFile, statement, newImportDeclaration);
                            var ns = getNamespaceLikeImport(importNode);
                            if (ns)
                                updateNamespaceLikeImport(changes, sourceFile, checker, movedSymbols, newModuleName, newModuleSpecifier, ns, importNode);
                        });
                    };
                    for (var _i = 0, _a = sourceFile.statements; _i < _a.length; _i++) {
                        var statement = _a[_i];
                        _loop_18(statement);
                    }
                };
                for (var _i = 0, _a = program.getSourceFiles(); _i < _a.length; _i++) {
                    var sourceFile = _a[_i];
                    _loop_17(sourceFile);
                }
            }
            function getNamespaceLikeImport(node) {
                switch (node.kind) {
                    case 254 /* ImportDeclaration */:
                        return node.importClause && node.importClause.namedBindings && node.importClause.namedBindings.kind === 256 /* NamespaceImport */ ?
                            node.importClause.namedBindings.name : undefined;
                    case 253 /* ImportEqualsDeclaration */:
                        return node.name;
                    case 242 /* VariableDeclaration */:
                        return ts.tryCast(node.name, ts.isIdentifier);
                    default:
                        return ts.Debug.assertNever(node, "Unexpected node kind " + node.kind);
                }
            }
            function updateNamespaceLikeImport(changes, sourceFile, checker, movedSymbols, newModuleName, newModuleSpecifier, oldImportId, oldImportNode) {
                var preferredNewNamespaceName = ts.codefix.moduleSpecifierToValidIdentifier(newModuleName, 99 /* ESNext */);
                var needUniqueName = false;
                var toChange = [];
                ts.FindAllReferences.Core.eachSymbolReferenceInFile(oldImportId, checker, sourceFile, function (ref) {
                    if (!ts.isPropertyAccessExpression(ref.parent))
                        return;
                    needUniqueName = needUniqueName || !!checker.resolveName(preferredNewNamespaceName, ref, 67108863 /* All */, /*excludeGlobals*/ true);
                    if (movedSymbols.has(checker.getSymbolAtLocation(ref.parent.name))) {
                        toChange.push(ref);
                    }
                });
                if (toChange.length) {
                    var newNamespaceName = needUniqueName ? ts.getUniqueName(preferredNewNamespaceName, sourceFile) : preferredNewNamespaceName;
                    for (var _i = 0, toChange_1 = toChange; _i < toChange_1.length; _i++) {
                        var ref = toChange_1[_i];
                        changes.replaceNode(sourceFile, ref, ts.createIdentifier(newNamespaceName));
                    }
                    changes.insertNodeAfter(sourceFile, oldImportNode, updateNamespaceLikeImportNode(oldImportNode, newModuleName, newModuleSpecifier));
                }
            }
            function updateNamespaceLikeImportNode(node, newNamespaceName, newModuleSpecifier) {
                var newNamespaceId = ts.createIdentifier(newNamespaceName);
                var newModuleString = ts.createLiteral(newModuleSpecifier);
                switch (node.kind) {
                    case 254 /* ImportDeclaration */:
                        return ts.createImportDeclaration(
                        /*decorators*/ undefined, /*modifiers*/ undefined, ts.createImportClause(/*name*/ undefined, ts.createNamespaceImport(newNamespaceId)), newModuleString);
                    case 253 /* ImportEqualsDeclaration */:
                        return ts.createImportEqualsDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, newNamespaceId, ts.createExternalModuleReference(newModuleString));
                    case 242 /* VariableDeclaration */:
                        return ts.createVariableDeclaration(newNamespaceId, /*type*/ undefined, createRequireCall(newModuleString));
                    default:
                        return ts.Debug.assertNever(node, "Unexpected node kind " + node.kind);
                }
            }
            function moduleSpecifierFromImport(i) {
                return (i.kind === 254 /* ImportDeclaration */ ? i.moduleSpecifier
                    : i.kind === 253 /* ImportEqualsDeclaration */ ? i.moduleReference.expression
                        : i.initializer.arguments[0]);
            }
            function forEachImportInStatement(statement, cb) {
                if (ts.isImportDeclaration(statement)) {
                    if (ts.isStringLiteral(statement.moduleSpecifier))
                        cb(statement);
                }
                else if (ts.isImportEqualsDeclaration(statement)) {
                    if (ts.isExternalModuleReference(statement.moduleReference) && ts.isStringLiteralLike(statement.moduleReference.expression)) {
                        cb(statement);
                    }
                }
                else if (ts.isVariableStatement(statement)) {
                    for (var _i = 0, _a = statement.declarationList.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        if (decl.initializer && ts.isRequireCall(decl.initializer, /*checkArgumentIsStringLiteralLike*/ true)) {
                            cb(decl);
                        }
                    }
                }
            }
            function createOldFileImportsFromNewFile(newFileNeedExport, newFileNameWithExtension, useEs6Imports, quotePreference) {
                var defaultImport;
                var imports = [];
                newFileNeedExport.forEach(function (symbol) {
                    if (symbol.escapedName === "default" /* Default */) {
                        defaultImport = ts.createIdentifier(ts.symbolNameNoDefault(symbol)); // TODO: GH#18217
                    }
                    else {
                        imports.push(symbol.name);
                    }
                });
                return makeImportOrRequire(defaultImport, imports, newFileNameWithExtension, useEs6Imports, quotePreference);
            }
            function makeImportOrRequire(defaultImport, imports, path, useEs6Imports, quotePreference) {
                path = ts.ensurePathIsNonModuleName(path);
                if (useEs6Imports) {
                    var specifiers = imports.map(function (i) { return ts.createImportSpecifier(/*propertyName*/ undefined, ts.createIdentifier(i)); });
                    return ts.makeImportIfNecessary(defaultImport, specifiers, path, quotePreference);
                }
                else {
                    ts.Debug.assert(!defaultImport, "No default import should exist"); // If there's a default export, it should have been an es6 module.
                    var bindingElements = imports.map(function (i) { return ts.createBindingElement(/*dotDotDotToken*/ undefined, /*propertyName*/ undefined, i); });
                    return bindingElements.length
                        ? makeVariableStatement(ts.createObjectBindingPattern(bindingElements), /*type*/ undefined, createRequireCall(ts.createLiteral(path)))
                        : undefined;
                }
            }
            function makeVariableStatement(name, type, initializer, flags) {
                if (flags === void 0) {
                    flags = 2 /* Const */;
                }
                return ts.createVariableStatement(/*modifiers*/ undefined, ts.createVariableDeclarationList([ts.createVariableDeclaration(name, type, initializer)], flags));
            }
            function createRequireCall(moduleSpecifier) {
                return ts.createCall(ts.createIdentifier("require"), /*typeArguments*/ undefined, [moduleSpecifier]);
            }
            function addExports(sourceFile, toMove, needExport, useEs6Exports) {
                return ts.flatMap(toMove, function (statement) {
                    if (isTopLevelDeclarationStatement(statement) &&
                        !isExported(sourceFile, statement, useEs6Exports) &&
                        forEachTopLevelDeclaration(statement, function (d) { return needExport.has(ts.Debug.assertDefined(d.symbol)); })) {
                        var exports = addExport(statement, useEs6Exports);
                        if (exports)
                            return exports;
                    }
                    return statement;
                });
            }
            function deleteUnusedImports(sourceFile, importDecl, changes, isUnused) {
                switch (importDecl.kind) {
                    case 254 /* ImportDeclaration */:
                        deleteUnusedImportsInDeclaration(sourceFile, importDecl, changes, isUnused);
                        break;
                    case 253 /* ImportEqualsDeclaration */:
                        if (isUnused(importDecl.name)) {
                            changes.delete(sourceFile, importDecl);
                        }
                        break;
                    case 242 /* VariableDeclaration */:
                        deleteUnusedImportsInVariableDeclaration(sourceFile, importDecl, changes, isUnused);
                        break;
                    default:
                        ts.Debug.assertNever(importDecl, "Unexpected import decl kind " + importDecl.kind);
                }
            }
            function deleteUnusedImportsInDeclaration(sourceFile, importDecl, changes, isUnused) {
                if (!importDecl.importClause)
                    return;
                var _a = importDecl.importClause, name = _a.name, namedBindings = _a.namedBindings;
                var defaultUnused = !name || isUnused(name);
                var namedBindingsUnused = !namedBindings ||
                    (namedBindings.kind === 256 /* NamespaceImport */ ? isUnused(namedBindings.name) : namedBindings.elements.length !== 0 && namedBindings.elements.every(function (e) { return isUnused(e.name); }));
                if (defaultUnused && namedBindingsUnused) {
                    changes.delete(sourceFile, importDecl);
                }
                else {
                    if (name && defaultUnused) {
                        changes.delete(sourceFile, name);
                    }
                    if (namedBindings) {
                        if (namedBindingsUnused) {
                            changes.replaceNode(sourceFile, importDecl.importClause, ts.updateImportClause(importDecl.importClause, name, /*namedBindings*/ undefined, importDecl.importClause.isTypeOnly));
                        }
                        else if (namedBindings.kind === 257 /* NamedImports */) {
                            for (var _i = 0, _b = namedBindings.elements; _i < _b.length; _i++) {
                                var element = _b[_i];
                                if (isUnused(element.name))
                                    changes.delete(sourceFile, element);
                            }
                        }
                    }
                }
            }
            function deleteUnusedImportsInVariableDeclaration(sourceFile, varDecl, changes, isUnused) {
                var name = varDecl.name;
                switch (name.kind) {
                    case 75 /* Identifier */:
                        if (isUnused(name)) {
                            changes.delete(sourceFile, name);
                        }
                        break;
                    case 190 /* ArrayBindingPattern */:
                        break;
                    case 189 /* ObjectBindingPattern */:
                        if (name.elements.every(function (e) { return ts.isIdentifier(e.name) && isUnused(e.name); })) {
                            changes.delete(sourceFile, ts.isVariableDeclarationList(varDecl.parent) && varDecl.parent.declarations.length === 1 ? varDecl.parent.parent : varDecl);
                        }
                        else {
                            for (var _i = 0, _a = name.elements; _i < _a.length; _i++) {
                                var element = _a[_i];
                                if (ts.isIdentifier(element.name) && isUnused(element.name)) {
                                    changes.delete(sourceFile, element.name);
                                }
                            }
                        }
                        break;
                }
            }
            function getNewFileImportsAndAddExportInOldFile(oldFile, importsToCopy, newFileImportsFromOldFile, changes, checker, useEs6ModuleSyntax, quotePreference) {
                var copiedOldImports = [];
                for (var _i = 0, _a = oldFile.statements; _i < _a.length; _i++) {
                    var oldStatement = _a[_i];
                    forEachImportInStatement(oldStatement, function (i) {
                        ts.append(copiedOldImports, filterImport(i, moduleSpecifierFromImport(i), function (name) { return importsToCopy.has(checker.getSymbolAtLocation(name)); }));
                    });
                }
                // Also, import things used from the old file, and insert 'export' modifiers as necessary in the old file.
                var oldFileDefault;
                var oldFileNamedImports = [];
                var markSeenTop = ts.nodeSeenTracker(); // Needed because multiple declarations may appear in `const x = 0, y = 1;`.
                newFileImportsFromOldFile.forEach(function (symbol) {
                    for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                        var decl = _a[_i];
                        if (!isTopLevelDeclaration(decl))
                            continue;
                        var name = nameOfTopLevelDeclaration(decl);
                        if (!name)
                            continue;
                        var top = getTopLevelDeclarationStatement(decl);
                        if (markSeenTop(top)) {
                            addExportToChanges(oldFile, top, changes, useEs6ModuleSyntax);
                        }
                        if (ts.hasModifier(decl, 512 /* Default */)) {
                            oldFileDefault = name;
                        }
                        else {
                            oldFileNamedImports.push(name.text);
                        }
                    }
                });
                ts.append(copiedOldImports, makeImportOrRequire(oldFileDefault, oldFileNamedImports, ts.removeFileExtension(ts.getBaseFileName(oldFile.fileName)), useEs6ModuleSyntax, quotePreference));
                return copiedOldImports;
            }
            function makeUniqueModuleName(moduleName, extension, inDirectory, host) {
                var newModuleName = moduleName;
                for (var i = 1;; i++) {
                    var name = ts.combinePaths(inDirectory, newModuleName + extension);
                    if (!host.fileExists(name))
                        return newModuleName; // TODO: GH#18217
                    newModuleName = moduleName + "." + i;
                }
            }
            function getNewModuleName(movedSymbols) {
                return movedSymbols.forEachEntry(ts.symbolNameNoDefault) || "newFile";
            }
            function getUsageInfo(oldFile, toMove, checker) {
                var movedSymbols = new SymbolSet();
                var oldImportsNeededByNewFile = new SymbolSet();
                var newFileImportsFromOldFile = new SymbolSet();
                var containsJsx = ts.find(toMove, function (statement) { return !!(statement.transformFlags & 2 /* ContainsJsx */); });
                var jsxNamespaceSymbol = getJsxNamespaceSymbol(containsJsx);
                if (jsxNamespaceSymbol) { // Might not exist (e.g. in non-compiling code)
                    oldImportsNeededByNewFile.add(jsxNamespaceSymbol);
                }
                for (var _i = 0, toMove_1 = toMove; _i < toMove_1.length; _i++) {
                    var statement = toMove_1[_i];
                    forEachTopLevelDeclaration(statement, function (decl) {
                        movedSymbols.add(ts.Debug.assertDefined(ts.isExpressionStatement(decl) ? checker.getSymbolAtLocation(decl.expression.left) : decl.symbol, "Need a symbol here"));
                    });
                }
                for (var _a = 0, toMove_2 = toMove; _a < toMove_2.length; _a++) {
                    var statement = toMove_2[_a];
                    forEachReference(statement, checker, function (symbol) {
                        if (!symbol.declarations)
                            return;
                        for (var _i = 0, _a = symbol.declarations; _i < _a.length; _i++) {
                            var decl = _a[_i];
                            if (isInImport(decl)) {
                                oldImportsNeededByNewFile.add(symbol);
                            }
                            else if (isTopLevelDeclaration(decl) && sourceFileOfTopLevelDeclaration(decl) === oldFile && !movedSymbols.has(symbol)) {
                                newFileImportsFromOldFile.add(symbol);
                            }
                        }
                    });
                }
                var unusedImportsFromOldFile = oldImportsNeededByNewFile.clone();
                var oldFileImportsFromNewFile = new SymbolSet();
                for (var _b = 0, _c = oldFile.statements; _b < _c.length; _b++) {
                    var statement = _c[_b];
                    if (ts.contains(toMove, statement))
                        continue;
                    // jsxNamespaceSymbol will only be set iff it is in oldImportsNeededByNewFile.
                    if (jsxNamespaceSymbol && !!(statement.transformFlags & 2 /* ContainsJsx */)) {
                        unusedImportsFromOldFile.delete(jsxNamespaceSymbol);
                    }
                    forEachReference(statement, checker, function (symbol) {
                        if (movedSymbols.has(symbol))
                            oldFileImportsFromNewFile.add(symbol);
                        unusedImportsFromOldFile.delete(symbol);
                    });
                }
                return { movedSymbols: movedSymbols, newFileImportsFromOldFile: newFileImportsFromOldFile, oldFileImportsFromNewFile: oldFileImportsFromNewFile, oldImportsNeededByNewFile: oldImportsNeededByNewFile, unusedImportsFromOldFile: unusedImportsFromOldFile };
                function getJsxNamespaceSymbol(containsJsx) {
                    if (containsJsx === undefined) {
                        return undefined;
                    }
                    var jsxNamespace = checker.getJsxNamespace(containsJsx);
                    // Strictly speaking, this could resolve to a symbol other than the JSX namespace.
                    // This will produce erroneous output (probably, an incorrectly copied import) but
                    // is expected to be very rare and easily reversible.
                    var jsxNamespaceSymbol = checker.resolveName(jsxNamespace, containsJsx, 1920 /* Namespace */, /*excludeGlobals*/ true);
                    return !!jsxNamespaceSymbol && ts.some(jsxNamespaceSymbol.declarations, isInImport)
                        ? jsxNamespaceSymbol
                        : undefined;
                }
            }
            // Below should all be utilities
            function isInImport(decl) {
                switch (decl.kind) {
                    case 253 /* ImportEqualsDeclaration */:
                    case 258 /* ImportSpecifier */:
                    case 255 /* ImportClause */:
                        return true;
                    case 242 /* VariableDeclaration */:
                        return isVariableDeclarationInImport(decl);
                    case 191 /* BindingElement */:
                        return ts.isVariableDeclaration(decl.parent.parent) && isVariableDeclarationInImport(decl.parent.parent);
                    default:
                        return false;
                }
            }
            function isVariableDeclarationInImport(decl) {
                return ts.isSourceFile(decl.parent.parent.parent) &&
                    !!decl.initializer && ts.isRequireCall(decl.initializer, /*checkArgumentIsStringLiteralLike*/ true);
            }
            function filterImport(i, moduleSpecifier, keep) {
                switch (i.kind) {
                    case 254 /* ImportDeclaration */: {
                        var clause = i.importClause;
                        if (!clause)
                            return undefined;
                        var defaultImport = clause.name && keep(clause.name) ? clause.name : undefined;
                        var namedBindings = clause.namedBindings && filterNamedBindings(clause.namedBindings, keep);
                        return defaultImport || namedBindings
                            ? ts.createImportDeclaration(/*decorators*/ undefined, /*modifiers*/ undefined, ts.createImportClause(defaultImport, namedBindings), moduleSpecifier)
                            : undefined;
                    }
                    case 253 /* ImportEqualsDeclaration */:
                        return keep(i.name) ? i : undefined;
                    case 242 /* VariableDeclaration */: {
                        var name = filterBindingName(i.name, keep);
                        return name ? makeVariableStatement(name, i.type, createRequireCall(moduleSpecifier), i.parent.flags) : undefined;
                    }
                    default:
                        return ts.Debug.assertNever(i, "Unexpected import kind " + i.kind);
                }
            }
            function filterNamedBindings(namedBindings, keep) {
                if (namedBindings.kind === 256 /* NamespaceImport */) {
                    return keep(namedBindings.name) ? namedBindings : undefined;
                }
                else {
                    var newElements = namedBindings.elements.filter(function (e) { return keep(e.name); });
                    return newElements.length ? ts.createNamedImports(newElements) : undefined;
                }
            }
            function filterBindingName(name, keep) {
                switch (name.kind) {
                    case 75 /* Identifier */:
                        return keep(name) ? name : undefined;
                    case 190 /* ArrayBindingPattern */:
                        return name;
                    case 189 /* ObjectBindingPattern */: {
                        // We can't handle nested destructurings or property names well here, so just copy them all.
                        var newElements = name.elements.filter(function (prop) { return prop.propertyName || !ts.isIdentifier(prop.name) || keep(prop.name); });
                        return newElements.length ? ts.createObjectBindingPattern(newElements) : undefined;
                    }
                }
            }
            function forEachReference(node, checker, onReference) {
                node.forEachChild(function cb(node) {
                    if (ts.isIdentifier(node) && !ts.isDeclarationName(node)) {
                        var sym = checker.getSymbolAtLocation(node);
                        if (sym)
                            onReference(sym);
                    }
                    else {
                        node.forEachChild(cb);
                    }
                });
            }
            var SymbolSet = /** @class */ (function () {
                function SymbolSet() {
                    this.map = ts.createMap();
                }
                SymbolSet.prototype.add = function (symbol) {
                    this.map.set(String(ts.getSymbolId(symbol)), symbol);
                };
                SymbolSet.prototype.has = function (symbol) {
                    return this.map.has(String(ts.getSymbolId(symbol)));
                };
                SymbolSet.prototype.delete = function (symbol) {
                    this.map.delete(String(ts.getSymbolId(symbol)));
                };
                SymbolSet.prototype.forEach = function (cb) {
                    this.map.forEach(cb);
                };
                SymbolSet.prototype.forEachEntry = function (cb) {
                    return ts.forEachEntry(this.map, cb);
                };
                SymbolSet.prototype.clone = function () {
                    var clone = new SymbolSet();
                    ts.copyEntries(this.map, clone.map);
                    return clone;
                };
                return SymbolSet;
            }());
            function isTopLevelDeclaration(node) {
                return isNonVariableTopLevelDeclaration(node) && ts.isSourceFile(node.parent) || ts.isVariableDeclaration(node) && ts.isSourceFile(node.parent.parent.parent);
            }
            function sourceFileOfTopLevelDeclaration(node) {
                return ts.isVariableDeclaration(node) ? node.parent.parent.parent : node.parent;
            }
            function isTopLevelDeclarationStatement(node) {
                ts.Debug.assert(ts.isSourceFile(node.parent), "Node parent should be a SourceFile");
                return isNonVariableTopLevelDeclaration(node) || ts.isVariableStatement(node);
            }
            function isNonVariableTopLevelDeclaration(node) {
                switch (node.kind) {
                    case 244 /* FunctionDeclaration */:
                    case 245 /* ClassDeclaration */:
                    case 249 /* ModuleDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 247 /* TypeAliasDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                    case 253 /* ImportEqualsDeclaration */:
                        return true;
                    default:
                        return false;
                }
            }
            function forEachTopLevelDeclaration(statement, cb) {
                switch (statement.kind) {
                    case 244 /* FunctionDeclaration */:
                    case 245 /* ClassDeclaration */:
                    case 249 /* ModuleDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 247 /* TypeAliasDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                    case 253 /* ImportEqualsDeclaration */:
                        return cb(statement);
                    case 225 /* VariableStatement */:
                        return ts.firstDefined(statement.declarationList.declarations, function (decl) { return forEachTopLevelDeclarationInBindingName(decl.name, cb); });
                    case 226 /* ExpressionStatement */: {
                        var expression = statement.expression;
                        return ts.isBinaryExpression(expression) && ts.getAssignmentDeclarationKind(expression) === 1 /* ExportsProperty */
                            ? cb(statement)
                            : undefined;
                    }
                }
            }
            function forEachTopLevelDeclarationInBindingName(name, cb) {
                switch (name.kind) {
                    case 75 /* Identifier */:
                        return cb(ts.cast(name.parent, function (x) { return ts.isVariableDeclaration(x) || ts.isBindingElement(x); }));
                    case 190 /* ArrayBindingPattern */:
                    case 189 /* ObjectBindingPattern */:
                        return ts.firstDefined(name.elements, function (em) { return ts.isOmittedExpression(em) ? undefined : forEachTopLevelDeclarationInBindingName(em.name, cb); });
                    default:
                        return ts.Debug.assertNever(name, "Unexpected name kind " + name.kind);
                }
            }
            function nameOfTopLevelDeclaration(d) {
                return ts.isExpressionStatement(d) ? ts.tryCast(d.expression.left.name, ts.isIdentifier) : ts.tryCast(d.name, ts.isIdentifier);
            }
            function getTopLevelDeclarationStatement(d) {
                switch (d.kind) {
                    case 242 /* VariableDeclaration */:
                        return d.parent.parent;
                    case 191 /* BindingElement */:
                        return getTopLevelDeclarationStatement(ts.cast(d.parent.parent, function (p) { return ts.isVariableDeclaration(p) || ts.isBindingElement(p); }));
                    default:
                        return d;
                }
            }
            function addExportToChanges(sourceFile, decl, changes, useEs6Exports) {
                if (isExported(sourceFile, decl, useEs6Exports))
                    return;
                if (useEs6Exports) {
                    if (!ts.isExpressionStatement(decl))
                        changes.insertExportModifier(sourceFile, decl);
                }
                else {
                    var names = getNamesToExportInCommonJS(decl);
                    if (names.length !== 0)
                        changes.insertNodesAfter(sourceFile, decl, names.map(createExportAssignment));
                }
            }
            function isExported(sourceFile, decl, useEs6Exports) {
                if (useEs6Exports) {
                    return !ts.isExpressionStatement(decl) && ts.hasModifier(decl, 1 /* Export */);
                }
                else {
                    return getNamesToExportInCommonJS(decl).some(function (name) { return sourceFile.symbol.exports.has(ts.escapeLeadingUnderscores(name)); });
                }
            }
            function addExport(decl, useEs6Exports) {
                return useEs6Exports ? [addEs6Export(decl)] : addCommonjsExport(decl);
            }
            function addEs6Export(d) {
                var modifiers = ts.concatenate([ts.createModifier(89 /* ExportKeyword */)], d.modifiers);
                switch (d.kind) {
                    case 244 /* FunctionDeclaration */:
                        return ts.updateFunctionDeclaration(d, d.decorators, modifiers, d.asteriskToken, d.name, d.typeParameters, d.parameters, d.type, d.body);
                    case 245 /* ClassDeclaration */:
                        return ts.updateClassDeclaration(d, d.decorators, modifiers, d.name, d.typeParameters, d.heritageClauses, d.members);
                    case 225 /* VariableStatement */:
                        return ts.updateVariableStatement(d, modifiers, d.declarationList);
                    case 249 /* ModuleDeclaration */:
                        return ts.updateModuleDeclaration(d, d.decorators, modifiers, d.name, d.body);
                    case 248 /* EnumDeclaration */:
                        return ts.updateEnumDeclaration(d, d.decorators, modifiers, d.name, d.members);
                    case 247 /* TypeAliasDeclaration */:
                        return ts.updateTypeAliasDeclaration(d, d.decorators, modifiers, d.name, d.typeParameters, d.type);
                    case 246 /* InterfaceDeclaration */:
                        return ts.updateInterfaceDeclaration(d, d.decorators, modifiers, d.name, d.typeParameters, d.heritageClauses, d.members);
                    case 253 /* ImportEqualsDeclaration */:
                        return ts.updateImportEqualsDeclaration(d, d.decorators, modifiers, d.name, d.moduleReference);
                    case 226 /* ExpressionStatement */:
                        return ts.Debug.fail(); // Shouldn't try to add 'export' keyword to `exports.x = ...`
                    default:
                        return ts.Debug.assertNever(d, "Unexpected declaration kind " + d.kind);
                }
            }
            function addCommonjsExport(decl) {
                return __spreadArrays([decl], getNamesToExportInCommonJS(decl).map(createExportAssignment));
            }
            function getNamesToExportInCommonJS(decl) {
                switch (decl.kind) {
                    case 244 /* FunctionDeclaration */:
                    case 245 /* ClassDeclaration */:
                        return [decl.name.text]; // TODO: GH#18217
                    case 225 /* VariableStatement */:
                        return ts.mapDefined(decl.declarationList.declarations, function (d) { return ts.isIdentifier(d.name) ? d.name.text : undefined; });
                    case 249 /* ModuleDeclaration */:
                    case 248 /* EnumDeclaration */:
                    case 247 /* TypeAliasDeclaration */:
                    case 246 /* InterfaceDeclaration */:
                    case 253 /* ImportEqualsDeclaration */:
                        return ts.emptyArray;
                    case 226 /* ExpressionStatement */:
                        return ts.Debug.fail("Can't export an ExpressionStatement"); // Shouldn't try to add 'export' keyword to `exports.x = ...`
                    default:
                        return ts.Debug.assertNever(decl, "Unexpected decl kind " + decl.kind);
                }
            }
            /** Creates `exports.x = x;` */
            function createExportAssignment(name) {
                return ts.createExpressionStatement(ts.createBinary(ts.createPropertyAccess(ts.createIdentifier("exports"), ts.createIdentifier(name)), 62 /* EqualsToken */, ts.createIdentifier(name)));
            }
        })(refactor = ts.refactor || (ts.refactor = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var refactor;
        (function (refactor) {
            var addOrRemoveBracesToArrowFunction;
            (function (addOrRemoveBracesToArrowFunction) {
                var refactorName = "Add or remove braces in an arrow function";
                var refactorDescription = ts.Diagnostics.Add_or_remove_braces_in_an_arrow_function.message;
                var addBracesActionName = "Add braces to arrow function";
                var removeBracesActionName = "Remove braces from arrow function";
                var addBracesActionDescription = ts.Diagnostics.Add_braces_to_arrow_function.message;
                var removeBracesActionDescription = ts.Diagnostics.Remove_braces_from_arrow_function.message;
                refactor.registerRefactor(refactorName, { getEditsForAction: getEditsForAction, getAvailableActions: getAvailableActions });
                function getAvailableActions(context) {
                    var file = context.file, startPosition = context.startPosition;
                    var info = getConvertibleArrowFunctionAtPosition(file, startPosition);
                    if (!info)
                        return ts.emptyArray;
                    return [{
                            name: refactorName,
                            description: refactorDescription,
                            actions: [
                                info.addBraces ?
                                    {
                                        name: addBracesActionName,
                                        description: addBracesActionDescription
                                    } : {
                                    name: removeBracesActionName,
                                    description: removeBracesActionDescription
                                }
                            ]
                        }];
                }
                function getEditsForAction(context, actionName) {
                    var file = context.file, startPosition = context.startPosition;
                    var info = getConvertibleArrowFunctionAtPosition(file, startPosition);
                    if (!info)
                        return undefined;
                    var expression = info.expression, returnStatement = info.returnStatement, func = info.func;
                    var body;
                    if (actionName === addBracesActionName) {
                        var returnStatement_1 = ts.createReturn(expression);
                        body = ts.createBlock([returnStatement_1], /* multiLine */ true);
                        ts.suppressLeadingAndTrailingTrivia(body);
                        ts.copyLeadingComments(expression, returnStatement_1, file, 3 /* MultiLineCommentTrivia */, /* hasTrailingNewLine */ true);
                    }
                    else if (actionName === removeBracesActionName && returnStatement) {
                        var actualExpression = expression || ts.createVoidZero();
                        body = needsParentheses(actualExpression) ? ts.createParen(actualExpression) : actualExpression;
                        ts.suppressLeadingAndTrailingTrivia(body);
                        ts.copyLeadingComments(returnStatement, body, file, 3 /* MultiLineCommentTrivia */, /* hasTrailingNewLine */ false);
                    }
                    else {
                        ts.Debug.fail("invalid action");
                    }
                    var edits = ts.textChanges.ChangeTracker.with(context, function (t) { return t.replaceNode(file, func.body, body); });
                    return { renameFilename: undefined, renameLocation: undefined, edits: edits };
                }
                function needsParentheses(expression) {
                    return ts.isBinaryExpression(expression) && expression.operatorToken.kind === 27 /* CommaToken */ || ts.isObjectLiteralExpression(expression);
                }
                function getConvertibleArrowFunctionAtPosition(file, startPosition) {
                    var node = ts.getTokenAtPosition(file, startPosition);
                    var func = ts.getContainingFunction(node);
                    if (!func || !ts.isArrowFunction(func) || (!ts.rangeContainsRange(func, node) || ts.rangeContainsRange(func.body, node)))
                        return undefined;
                    if (ts.isExpression(func.body)) {
                        return {
                            func: func,
                            addBraces: true,
                            expression: func.body
                        };
                    }
                    else if (func.body.statements.length === 1) {
                        var firstStatement = ts.first(func.body.statements);
                        if (ts.isReturnStatement(firstStatement)) {
                            return {
                                func: func,
                                addBraces: false,
                                expression: firstStatement.expression,
                                returnStatement: firstStatement
                            };
                        }
                    }
                    return undefined;
                }
            })(addOrRemoveBracesToArrowFunction = refactor.addOrRemoveBracesToArrowFunction || (refactor.addOrRemoveBracesToArrowFunction = {}));
        })(refactor = ts.refactor || (ts.refactor = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var refactor;
        (function (refactor) {
            var convertParamsToDestructuredObject;
            (function (convertParamsToDestructuredObject) {
                var refactorName = "Convert parameters to destructured object";
                var minimumParameterLength = 2;
                refactor.registerRefactor(refactorName, { getEditsForAction: getEditsForAction, getAvailableActions: getAvailableActions });
                function getAvailableActions(context) {
                    var file = context.file, startPosition = context.startPosition;
                    var isJSFile = ts.isSourceFileJS(file);
                    if (isJSFile)
                        return ts.emptyArray; // TODO: GH#30113
                    var functionDeclaration = getFunctionDeclarationAtPosition(file, startPosition, context.program.getTypeChecker());
                    if (!functionDeclaration)
                        return ts.emptyArray;
                    var description = ts.getLocaleSpecificMessage(ts.Diagnostics.Convert_parameters_to_destructured_object);
                    return [{
                            name: refactorName,
                            description: description,
                            actions: [{
                                    name: refactorName,
                                    description: description
                                }]
                        }];
                }
                function getEditsForAction(context, actionName) {
                    ts.Debug.assert(actionName === refactorName, "Unexpected action name");
                    var file = context.file, startPosition = context.startPosition, program = context.program, cancellationToken = context.cancellationToken, host = context.host;
                    var functionDeclaration = getFunctionDeclarationAtPosition(file, startPosition, program.getTypeChecker());
                    if (!functionDeclaration || !cancellationToken)
                        return undefined;
                    var groupedReferences = getGroupedReferences(functionDeclaration, program, cancellationToken);
                    if (groupedReferences.valid) {
                        var edits = ts.textChanges.ChangeTracker.with(context, function (t) { return doChange(file, program, host, t, functionDeclaration, groupedReferences); });
                        return { renameFilename: undefined, renameLocation: undefined, edits: edits };
                    }
                    return { edits: [] }; // TODO: GH#30113
                }
                function doChange(sourceFile, program, host, changes, functionDeclaration, groupedReferences) {
                    var newParamDeclaration = ts.map(createNewParameters(functionDeclaration, program, host), function (param) { return ts.getSynthesizedDeepClone(param); });
                    changes.replaceNodeRangeWithNodes(sourceFile, ts.first(functionDeclaration.parameters), ts.last(functionDeclaration.parameters), newParamDeclaration, { joiner: ", ",
                        // indentation is set to 0 because otherwise the object parameter will be indented if there is a `this` parameter
                        indentation: 0,
                        leadingTriviaOption: ts.textChanges.LeadingTriviaOption.IncludeAll,
                        trailingTriviaOption: ts.textChanges.TrailingTriviaOption.Include
                    });
                    var functionCalls = ts.sortAndDeduplicate(groupedReferences.functionCalls, /*comparer*/ function (a, b) { return ts.compareValues(a.pos, b.pos); });
                    for (var _i = 0, functionCalls_1 = functionCalls; _i < functionCalls_1.length; _i++) {
                        var call = functionCalls_1[_i];
                        if (call.arguments && call.arguments.length) {
                            var newArgument = ts.getSynthesizedDeepClone(createNewArgument(functionDeclaration, call.arguments), /*includeTrivia*/ true);
                            changes.replaceNodeRange(ts.getSourceFileOfNode(call), ts.first(call.arguments), ts.last(call.arguments), newArgument, { leadingTriviaOption: ts.textChanges.LeadingTriviaOption.IncludeAll, trailingTriviaOption: ts.textChanges.TrailingTriviaOption.Include });
                        }
                    }
                }
                function getGroupedReferences(functionDeclaration, program, cancellationToken) {
                    var functionNames = getFunctionNames(functionDeclaration);
                    var classNames = ts.isConstructorDeclaration(functionDeclaration) ? getClassNames(functionDeclaration) : [];
                    var names = ts.deduplicate(__spreadArrays(functionNames, classNames), ts.equateValues);
                    var checker = program.getTypeChecker();
                    var references = ts.flatMap(names, /*mapfn*/ function (/*mapfn*/ name) { return ts.FindAllReferences.getReferenceEntriesForNode(-1, name, program, program.getSourceFiles(), cancellationToken); });
                    var groupedReferences = groupReferences(references);
                    if (!ts.every(groupedReferences.declarations, /*callback*/ function (/*callback*/ decl) { return ts.contains(names, decl); })) {
                        groupedReferences.valid = false;
                    }
                    return groupedReferences;
                    function groupReferences(referenceEntries) {
                        var classReferences = { accessExpressions: [], typeUsages: [] };
                        var groupedReferences = { functionCalls: [], declarations: [], classReferences: classReferences, valid: true };
                        var functionSymbols = ts.map(functionNames, getSymbolTargetAtLocation);
                        var classSymbols = ts.map(classNames, getSymbolTargetAtLocation);
                        var isConstructor = ts.isConstructorDeclaration(functionDeclaration);
                        for (var _i = 0, referenceEntries_1 = referenceEntries; _i < referenceEntries_1.length; _i++) {
                            var entry = referenceEntries_1[_i];
                            if (entry.kind !== 1 /* Node */) {
                                groupedReferences.valid = false;
                                continue;
                            }
                            /* We compare symbols because in some cases find all references wil return a reference that may or may not be to the refactored function.
                            Example from the refactorConvertParamsToDestructuredObject_methodCallUnion.ts test:
                                class A { foo(a: number, b: number) { return a + b; } }
                                class B { foo(c: number, d: number) { return c + d; } }
                                declare const ab: A | B;
                                ab.foo(1, 2);
                            Find all references will return `ab.foo(1, 2)` as a reference to A's `foo` but we could be calling B's `foo`.
                            When looking for constructor calls, however, the symbol on the constructor call reference is going to be the corresponding class symbol.
                            So we need to add a special case for this because when calling a constructor of a class through one of its subclasses,
                            the symbols are going to be different.
                            */
                            if (ts.contains(functionSymbols, getSymbolTargetAtLocation(entry.node)) || ts.isNewExpressionTarget(entry.node)) {
                                var importOrExportReference = entryToImportOrExport(entry);
                                if (importOrExportReference) {
                                    continue;
                                }
                                var decl = entryToDeclaration(entry);
                                if (decl) {
                                    groupedReferences.declarations.push(decl);
                                    continue;
                                }
                                var call = entryToFunctionCall(entry);
                                if (call) {
                                    groupedReferences.functionCalls.push(call);
                                    continue;
                                }
                            }
                            // if the refactored function is a constructor, we must also check if the references to its class are valid
                            if (isConstructor && ts.contains(classSymbols, getSymbolTargetAtLocation(entry.node))) {
                                var importOrExportReference = entryToImportOrExport(entry);
                                if (importOrExportReference) {
                                    continue;
                                }
                                var decl = entryToDeclaration(entry);
                                if (decl) {
                                    groupedReferences.declarations.push(decl);
                                    continue;
                                }
                                var accessExpression = entryToAccessExpression(entry);
                                if (accessExpression) {
                                    classReferences.accessExpressions.push(accessExpression);
                                    continue;
                                }
                                // Only class declarations are allowed to be used as a type (in a heritage clause),
                                // otherwise `findAllReferences` might not be able to track constructor calls.
                                if (ts.isClassDeclaration(functionDeclaration.parent)) {
                                    var type = entryToType(entry);
                                    if (type) {
                                        classReferences.typeUsages.push(type);
                                        continue;
                                    }
                                }
                            }
                            groupedReferences.valid = false;
                        }
                        return groupedReferences;
                    }
                    function getSymbolTargetAtLocation(node) {
                        var symbol = checker.getSymbolAtLocation(node);
                        return symbol && ts.getSymbolTarget(symbol, checker);
                    }
                }
                function entryToImportOrExport(entry) {
                    var node = entry.node;
                    if (ts.isImportSpecifier(node.parent)
                        || ts.isImportClause(node.parent)
                        || ts.isImportEqualsDeclaration(node.parent)
                        || ts.isNamespaceImport(node.parent)) {
                        return node;
                    }
                    if (ts.isExportSpecifier(node.parent) || ts.isExportAssignment(node.parent)) {
                        return node;
                    }
                    return undefined;
                }
                function entryToDeclaration(entry) {
                    if (ts.isDeclaration(entry.node.parent)) {
                        return entry.node;
                    }
                    return undefined;
                }
                function entryToFunctionCall(entry) {
                    if (entry.node.parent) {
                        var functionReference = entry.node;
                        var parent = functionReference.parent;
                        switch (parent.kind) {
                            // foo(...) or super(...) or new Foo(...)
                            case 196 /* CallExpression */:
                            case 197 /* NewExpression */:
                                var callOrNewExpression = ts.tryCast(parent, ts.isCallOrNewExpression);
                                if (callOrNewExpression && callOrNewExpression.expression === functionReference) {
                                    return callOrNewExpression;
                                }
                                break;
                            // x.foo(...)
                            case 194 /* PropertyAccessExpression */:
                                var propertyAccessExpression = ts.tryCast(parent, ts.isPropertyAccessExpression);
                                if (propertyAccessExpression && propertyAccessExpression.parent && propertyAccessExpression.name === functionReference) {
                                    var callOrNewExpression_1 = ts.tryCast(propertyAccessExpression.parent, ts.isCallOrNewExpression);
                                    if (callOrNewExpression_1 && callOrNewExpression_1.expression === propertyAccessExpression) {
                                        return callOrNewExpression_1;
                                    }
                                }
                                break;
                            // x["foo"](...)
                            case 195 /* ElementAccessExpression */:
                                var elementAccessExpression = ts.tryCast(parent, ts.isElementAccessExpression);
                                if (elementAccessExpression && elementAccessExpression.parent && elementAccessExpression.argumentExpression === functionReference) {
                                    var callOrNewExpression_2 = ts.tryCast(elementAccessExpression.parent, ts.isCallOrNewExpression);
                                    if (callOrNewExpression_2 && callOrNewExpression_2.expression === elementAccessExpression) {
                                        return callOrNewExpression_2;
                                    }
                                }
                                break;
                        }
                    }
                    return undefined;
                }
                function entryToAccessExpression(entry) {
                    if (entry.node.parent) {
                        var reference = entry.node;
                        var parent = reference.parent;
                        switch (parent.kind) {
                            // `C.foo`
                            case 194 /* PropertyAccessExpression */:
                                var propertyAccessExpression = ts.tryCast(parent, ts.isPropertyAccessExpression);
                                if (propertyAccessExpression && propertyAccessExpression.expression === reference) {
                                    return propertyAccessExpression;
                                }
                                break;
                            // `C["foo"]`
                            case 195 /* ElementAccessExpression */:
                                var elementAccessExpression = ts.tryCast(parent, ts.isElementAccessExpression);
                                if (elementAccessExpression && elementAccessExpression.expression === reference) {
                                    return elementAccessExpression;
                                }
                                break;
                        }
                    }
                    return undefined;
                }
                function entryToType(entry) {
                    var reference = entry.node;
                    if (ts.getMeaningFromLocation(reference) === 2 /* Type */ || ts.isExpressionWithTypeArgumentsInClassExtendsClause(reference.parent)) {
                        return reference;
                    }
                    return undefined;
                }
                function getFunctionDeclarationAtPosition(file, startPosition, checker) {
                    var node = ts.getTouchingToken(file, startPosition);
                    var functionDeclaration = ts.getContainingFunctionDeclaration(node);
                    // don't offer refactor on top-level JSDoc
                    if (isTopLevelJSDoc(node))
                        return undefined;
                    if (functionDeclaration
                        && isValidFunctionDeclaration(functionDeclaration, checker)
                        && ts.rangeContainsRange(functionDeclaration, node)
                        && !(functionDeclaration.body && ts.rangeContainsRange(functionDeclaration.body, node)))
                        return functionDeclaration;
                    return undefined;
                }
                function isTopLevelJSDoc(node) {
                    var containingJSDoc = ts.findAncestor(node, ts.isJSDocNode);
                    if (containingJSDoc) {
                        var containingNonJSDoc = ts.findAncestor(containingJSDoc, function (n) { return !ts.isJSDocNode(n); });
                        return !!containingNonJSDoc && ts.isFunctionLikeDeclaration(containingNonJSDoc);
                    }
                    return false;
                }
                function isValidFunctionDeclaration(functionDeclaration, checker) {
                    if (!isValidParameterNodeArray(functionDeclaration.parameters, checker))
                        return false;
                    switch (functionDeclaration.kind) {
                        case 244 /* FunctionDeclaration */:
                            return hasNameOrDefault(functionDeclaration) && isSingleImplementation(functionDeclaration, checker);
                        case 161 /* MethodDeclaration */:
                            return isSingleImplementation(functionDeclaration, checker);
                        case 162 /* Constructor */:
                            if (ts.isClassDeclaration(functionDeclaration.parent)) {
                                return hasNameOrDefault(functionDeclaration.parent) && isSingleImplementation(functionDeclaration, checker);
                            }
                            else {
                                return isValidVariableDeclaration(functionDeclaration.parent.parent)
                                    && isSingleImplementation(functionDeclaration, checker);
                            }
                        case 201 /* FunctionExpression */:
                        case 202 /* ArrowFunction */:
                            return isValidVariableDeclaration(functionDeclaration.parent);
                    }
                    return false;
                }
                function isSingleImplementation(functionDeclaration, checker) {
                    return !!functionDeclaration.body && !checker.isImplementationOfOverload(functionDeclaration);
                }
                function hasNameOrDefault(functionOrClassDeclaration) {
                    if (!functionOrClassDeclaration.name) {
                        var defaultKeyword = ts.findModifier(functionOrClassDeclaration, 84 /* DefaultKeyword */);
                        return !!defaultKeyword;
                    }
                    return true;
                }
                function isValidParameterNodeArray(parameters, checker) {
                    return getRefactorableParametersLength(parameters) >= minimumParameterLength
                        && ts.every(parameters, /*callback*/ function (/*callback*/ paramDecl) { return isValidParameterDeclaration(paramDecl, checker); });
                }
                function isValidParameterDeclaration(parameterDeclaration, checker) {
                    if (ts.isRestParameter(parameterDeclaration)) {
                        var type = checker.getTypeAtLocation(parameterDeclaration);
                        if (!checker.isArrayType(type) && !checker.isTupleType(type))
                            return false;
                    }
                    return !parameterDeclaration.modifiers && !parameterDeclaration.decorators && ts.isIdentifier(parameterDeclaration.name);
                }
                function isValidVariableDeclaration(node) {
                    return ts.isVariableDeclaration(node) && ts.isVarConst(node) && ts.isIdentifier(node.name) && !node.type; // TODO: GH#30113
                }
                function hasThisParameter(parameters) {
                    return parameters.length > 0 && ts.isThis(parameters[0].name);
                }
                function getRefactorableParametersLength(parameters) {
                    if (hasThisParameter(parameters)) {
                        return parameters.length - 1;
                    }
                    return parameters.length;
                }
                function getRefactorableParameters(parameters) {
                    if (hasThisParameter(parameters)) {
                        parameters = ts.createNodeArray(parameters.slice(1), parameters.hasTrailingComma);
                    }
                    return parameters;
                }
                function createPropertyOrShorthandAssignment(name, initializer) {
                    if (ts.isIdentifier(initializer) && ts.getTextOfIdentifierOrLiteral(initializer) === name) {
                        return ts.createShorthandPropertyAssignment(name);
                    }
                    return ts.createPropertyAssignment(name, initializer);
                }
                function createNewArgument(functionDeclaration, functionArguments) {
                    var parameters = getRefactorableParameters(functionDeclaration.parameters);
                    var hasRestParameter = ts.isRestParameter(ts.last(parameters));
                    var nonRestArguments = hasRestParameter ? functionArguments.slice(0, parameters.length - 1) : functionArguments;
                    var properties = ts.map(nonRestArguments, function (arg, i) {
                        var parameterName = getParameterName(parameters[i]);
                        var property = createPropertyOrShorthandAssignment(parameterName, arg);
                        ts.suppressLeadingAndTrailingTrivia(property.name);
                        if (ts.isPropertyAssignment(property))
                            ts.suppressLeadingAndTrailingTrivia(property.initializer);
                        copyComments(arg, property);
                        return property;
                    });
                    if (hasRestParameter && functionArguments.length >= parameters.length) {
                        var restArguments = functionArguments.slice(parameters.length - 1);
                        var restProperty = ts.createPropertyAssignment(getParameterName(ts.last(parameters)), ts.createArrayLiteral(restArguments));
                        properties.push(restProperty);
                    }
                    var objectLiteral = ts.createObjectLiteral(properties, /*multiLine*/ false);
                    return objectLiteral;
                }
                function createNewParameters(functionDeclaration, program, host) {
                    var checker = program.getTypeChecker();
                    var refactorableParameters = getRefactorableParameters(functionDeclaration.parameters);
                    var bindingElements = ts.map(refactorableParameters, createBindingElementFromParameterDeclaration);
                    var objectParameterName = ts.createObjectBindingPattern(bindingElements);
                    var objectParameterType = createParameterTypeNode(refactorableParameters);
                    var objectInitializer;
                    // If every parameter in the original function was optional, add an empty object initializer to the new object parameter
                    if (ts.every(refactorableParameters, isOptionalParameter)) {
                        objectInitializer = ts.createObjectLiteral();
                    }
                    var objectParameter = ts.createParameter(
                    /*decorators*/ undefined, 
                    /*modifiers*/ undefined, 
                    /*dotDotDotToken*/ undefined, objectParameterName, 
                    /*questionToken*/ undefined, objectParameterType, objectInitializer);
                    if (hasThisParameter(functionDeclaration.parameters)) {
                        var thisParameter = functionDeclaration.parameters[0];
                        var newThisParameter = ts.createParameter(
                        /*decorators*/ undefined, 
                        /*modifiers*/ undefined, 
                        /*dotDotDotToken*/ undefined, thisParameter.name, 
                        /*questionToken*/ undefined, thisParameter.type);
                        ts.suppressLeadingAndTrailingTrivia(newThisParameter.name);
                        copyComments(thisParameter.name, newThisParameter.name);
                        if (thisParameter.type) {
                            ts.suppressLeadingAndTrailingTrivia(newThisParameter.type);
                            copyComments(thisParameter.type, newThisParameter.type);
                        }
                        return ts.createNodeArray([newThisParameter, objectParameter]);
                    }
                    return ts.createNodeArray([objectParameter]);
                    function createBindingElementFromParameterDeclaration(parameterDeclaration) {
                        var element = ts.createBindingElement(
                        /*dotDotDotToken*/ undefined, 
                        /*propertyName*/ undefined, getParameterName(parameterDeclaration), ts.isRestParameter(parameterDeclaration) && isOptionalParameter(parameterDeclaration) ? ts.createArrayLiteral() : parameterDeclaration.initializer);
                        ts.suppressLeadingAndTrailingTrivia(element);
                        if (parameterDeclaration.initializer && element.initializer) {
                            copyComments(parameterDeclaration.initializer, element.initializer);
                        }
                        return element;
                    }
                    function createParameterTypeNode(parameters) {
                        var members = ts.map(parameters, createPropertySignatureFromParameterDeclaration);
                        var typeNode = ts.addEmitFlags(ts.createTypeLiteralNode(members), 1 /* SingleLine */);
                        return typeNode;
                    }
                    function createPropertySignatureFromParameterDeclaration(parameterDeclaration) {
                        var parameterType = parameterDeclaration.type;
                        if (!parameterType && (parameterDeclaration.initializer || ts.isRestParameter(parameterDeclaration))) {
                            parameterType = getTypeNode(parameterDeclaration);
                        }
                        var propertySignature = ts.createPropertySignature(
                        /*modifiers*/ undefined, getParameterName(parameterDeclaration), isOptionalParameter(parameterDeclaration) ? ts.createToken(57 /* QuestionToken */) : parameterDeclaration.questionToken, parameterType, 
                        /*initializer*/ undefined);
                        ts.suppressLeadingAndTrailingTrivia(propertySignature);
                        copyComments(parameterDeclaration.name, propertySignature.name);
                        if (parameterDeclaration.type && propertySignature.type) {
                            copyComments(parameterDeclaration.type, propertySignature.type);
                        }
                        return propertySignature;
                    }
                    function getTypeNode(node) {
                        var type = checker.getTypeAtLocation(node);
                        return ts.getTypeNodeIfAccessible(type, node, program, host);
                    }
                    function isOptionalParameter(parameterDeclaration) {
                        if (ts.isRestParameter(parameterDeclaration)) {
                            var type = checker.getTypeAtLocation(parameterDeclaration);
                            return !checker.isTupleType(type);
                        }
                        return checker.isOptionalParameter(parameterDeclaration);
                    }
                }
                function copyComments(sourceNode, targetNode) {
                    var sourceFile = sourceNode.getSourceFile();
                    var text = sourceFile.text;
                    if (hasLeadingLineBreak(sourceNode, text)) {
                        ts.copyLeadingComments(sourceNode, targetNode, sourceFile);
                    }
                    else {
                        ts.copyTrailingAsLeadingComments(sourceNode, targetNode, sourceFile);
                    }
                    ts.copyTrailingComments(sourceNode, targetNode, sourceFile);
                }
                function hasLeadingLineBreak(node, text) {
                    var start = node.getFullStart();
                    var end = node.getStart();
                    for (var i = start; i < end; i++) {
                        if (text.charCodeAt(i) === 10 /* lineFeed */)
                            return true;
                    }
                    return false;
                }
                function getParameterName(paramDeclaration) {
                    return ts.getTextOfIdentifierOrLiteral(paramDeclaration.name);
                }
                function getClassNames(constructorDeclaration) {
                    switch (constructorDeclaration.parent.kind) {
                        case 245 /* ClassDeclaration */:
                            var classDeclaration = constructorDeclaration.parent;
                            if (classDeclaration.name)
                                return [classDeclaration.name];
                            // If the class declaration doesn't have a name, it should have a default modifier.
                            // We validated this in `isValidFunctionDeclaration` through `hasNameOrDefault`
                            var defaultModifier = ts.Debug.assertDefined(ts.findModifier(classDeclaration, 84 /* DefaultKeyword */), "Nameless class declaration should be a default export");
                            return [defaultModifier];
                        case 214 /* ClassExpression */:
                            var classExpression = constructorDeclaration.parent;
                            var variableDeclaration = constructorDeclaration.parent.parent;
                            var className = classExpression.name;
                            if (className)
                                return [className, variableDeclaration.name];
                            return [variableDeclaration.name];
                    }
                }
                function getFunctionNames(functionDeclaration) {
                    switch (functionDeclaration.kind) {
                        case 244 /* FunctionDeclaration */:
                            if (functionDeclaration.name)
                                return [functionDeclaration.name];
                            // If the function declaration doesn't have a name, it should have a default modifier.
                            // We validated this in `isValidFunctionDeclaration` through `hasNameOrDefault`
                            var defaultModifier = ts.Debug.assertDefined(ts.findModifier(functionDeclaration, 84 /* DefaultKeyword */), "Nameless function declaration should be a default export");
                            return [defaultModifier];
                        case 161 /* MethodDeclaration */:
                            return [functionDeclaration.name];
                        case 162 /* Constructor */:
                            var ctrKeyword = ts.Debug.assertDefined(ts.findChildOfKind(functionDeclaration, 129 /* ConstructorKeyword */, functionDeclaration.getSourceFile()), "Constructor declaration should have constructor keyword");
                            if (functionDeclaration.parent.kind === 214 /* ClassExpression */) {
                                var variableDeclaration = functionDeclaration.parent.parent;
                                return [variableDeclaration.name, ctrKeyword];
                            }
                            return [ctrKeyword];
                        case 202 /* ArrowFunction */:
                            return [functionDeclaration.parent.name];
                        case 201 /* FunctionExpression */:
                            if (functionDeclaration.name)
                                return [functionDeclaration.name, functionDeclaration.parent.name];
                            return [functionDeclaration.parent.name];
                        default:
                            return ts.Debug.assertNever(functionDeclaration, "Unexpected function declaration kind " + functionDeclaration.kind);
                    }
                }
            })(convertParamsToDestructuredObject = refactor.convertParamsToDestructuredObject || (refactor.convertParamsToDestructuredObject = {}));
        })(refactor = ts.refactor || (ts.refactor = {}));
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var refactor;
        (function (refactor) {
            var convertStringOrTemplateLiteral;
            (function (convertStringOrTemplateLiteral) {
                var refactorName = "Convert to template string";
                var refactorDescription = ts.getLocaleSpecificMessage(ts.Diagnostics.Convert_to_template_string);
                refactor.registerRefactor(refactorName, { getEditsForAction: getEditsForAction, getAvailableActions: getAvailableActions });
                function getAvailableActions(context) {
                    var file = context.file, startPosition = context.startPosition;
                    var node = getNodeOrParentOfParentheses(file, startPosition);
                    var maybeBinary = getParentBinaryExpression(node);
                    var refactorInfo = { name: refactorName, description: refactorDescription, actions: [] };
                    if (ts.isBinaryExpression(maybeBinary) && isStringConcatenationValid(maybeBinary)) {
                        refactorInfo.actions.push({ name: refactorName, description: refactorDescription });
                        return [refactorInfo];
                    }
                    return ts.emptyArray;
                }
                function getNodeOrParentOfParentheses(file, startPosition) {
                    var node = ts.getTokenAtPosition(file, startPosition);
                    var nestedBinary = getParentBinaryExpression(node);
                    var isNonStringBinary = !isStringConcatenationValid(nestedBinary);
                    if (isNonStringBinary &&
                        ts.isParenthesizedExpression(nestedBinary.parent) &&
                        ts.isBinaryExpression(nestedBinary.parent.parent)) {
                        return nestedBinary.parent.parent;
                    }
                    return node;
                }
                function getEditsForAction(context, actionName) {
                    var file = context.file, startPosition = context.startPosition;
                    var node = getNodeOrParentOfParentheses(file, startPosition);
                    switch (actionName) {
                        case refactorDescription:
                            return { edits: getEditsForToTemplateLiteral(context, node) };
                        default:
                            return ts.Debug.fail("invalid action");
                    }
                }
                function getEditsForToTemplateLiteral(context, node) {
                    var maybeBinary = getParentBinaryExpression(node);
                    var file = context.file;
                    var templateLiteral = nodesToTemplate(treeToArray(maybeBinary), file);
                    var trailingCommentRanges = ts.getTrailingCommentRanges(file.text, maybeBinary.end);
                    if (trailingCommentRanges) {
                        var lastComment = trailingCommentRanges[trailingCommentRanges.length - 1];
                        var trailingRange_1 = { pos: trailingCommentRanges[0].pos, end: lastComment.end };
                        // since suppressTrailingTrivia(maybeBinary) does not work, the trailing comment is removed manually
                        // otherwise it would have the trailing comment twice
                        return ts.textChanges.ChangeTracker.with(context, function (t) {
                            t.deleteRange(file, trailingRange_1);
                            t.replaceNode(file, maybeBinary, templateLiteral);
                        });
                    }
                    else {
                        return ts.textChanges.ChangeTracker.with(context, function (t) { return t.replaceNode(file, maybeBinary, templateLiteral); });
                    }
                }
                function isNotEqualsOperator(node) {
                    return node.operatorToken.kind !== 62 /* EqualsToken */;
                }
                function getParentBinaryExpression(expr) {
                    while (ts.isBinaryExpression(expr.parent) && isNotEqualsOperator(expr.parent)) {
                        expr = expr.parent;
                    }
                    return expr;
                }
                function isStringConcatenationValid(node) {
                    var _a = treeToArray(node), containsString = _a.containsString, areOperatorsValid = _a.areOperatorsValid;
                    return containsString && areOperatorsValid;
                }
                function treeToArray(current) {
                    if (ts.isBinaryExpression(current)) {
                        var _a = treeToArray(current.left), nodes = _a.nodes, operators = _a.operators, leftHasString = _a.containsString, leftOperatorValid = _a.areOperatorsValid;
                        if (!leftHasString && !ts.isStringLiteral(current.right)) {
                            return { nodes: [current], operators: [], containsString: false, areOperatorsValid: true };
                        }
                        var currentOperatorValid = current.operatorToken.kind === 39 /* PlusToken */;
                        var areOperatorsValid = leftOperatorValid && currentOperatorValid;
                        nodes.push(current.right);
                        operators.push(current.operatorToken);
                        return { nodes: nodes, operators: operators, containsString: true, areOperatorsValid: areOperatorsValid };
                    }
                    return { nodes: [current], operators: [], containsString: ts.isStringLiteral(current), areOperatorsValid: true };
                }
                // to copy comments following the operator
                // "foo" + /* comment */ "bar"
                var copyTrailingOperatorComments = function (operators, file) {
                    return function (index, targetNode) {
                        if (index < operators.length) {
                            ts.copyTrailingComments(operators[index], targetNode, file, 3 /* MultiLineCommentTrivia */, /* hasTrailingNewLine */ false);
                        }
                    };
                };
                // to copy comments following the string
                // "foo" /* comment */ + "bar" /* comment */ + "bar2"
                var copyCommentFromMultiNode = function (nodes, file, copyOperatorComments) {
                    return function (indexes, targetNode) {
                        while (indexes.length > 0) {
                            var index = indexes.shift();
                            ts.copyTrailingComments(nodes[index], targetNode, file, 3 /* MultiLineCommentTrivia */, /* hasTrailingNewLine */ false);
                            copyOperatorComments(index, targetNode);
                        }
                    };
                };
                function concatConsecutiveString(index, nodes) {
                    var text = "";
                    var indexes = [];
                    while (index < nodes.length && ts.isStringLiteral(nodes[index])) {
                        var stringNode = nodes[index];
                        text = text + stringNode.text;
                        indexes.push(index);
                        index++;
                    }
                    text = ts.escapeString(text);
                    return [index, text, indexes];
                }
                function nodesToTemplate(_a, file) {
                    var nodes = _a.nodes, operators = _a.operators;
                    var copyOperatorComments = copyTrailingOperatorComments(operators, file);
                    var copyCommentFromStringLiterals = copyCommentFromMultiNode(nodes, file, copyOperatorComments);
                    var _b = concatConsecutiveString(0, nodes), begin = _b[0], headText = _b[1], headIndexes = _b[2];
                    if (begin === nodes.length) {
                        var noSubstitutionTemplateLiteral = ts.createNoSubstitutionTemplateLiteral(headText);
                        copyCommentFromStringLiterals(headIndexes, noSubstitutionTemplateLiteral);
                        return noSubstitutionTemplateLiteral;
                    }
                    var templateSpans = [];
                    var templateHead = ts.createTemplateHead(headText);
                    copyCommentFromStringLiterals(headIndexes, templateHead);
                    for (var i = begin; i < nodes.length; i++) {
                        var currentNode = getExpressionFromParenthesesOrExpression(nodes[i]);
                        copyOperatorComments(i, currentNode);
                        var _c = concatConsecutiveString(i + 1, nodes), newIndex = _c[0], subsequentText = _c[1], stringIndexes = _c[2];
                        i = newIndex - 1;
                        var templatePart = i === nodes.length - 1 ? ts.createTemplateTail(subsequentText) : ts.createTemplateMiddle(subsequentText);
                        copyCommentFromStringLiterals(stringIndexes, templatePart);
                        templateSpans.push(ts.createTemplateSpan(currentNode, templatePart));
                    }
                    return ts.createTemplateExpression(templateHead, templateSpans);
                }
                // to copy comments following the opening & closing parentheses
                // "foo" + ( /* comment */ 5 + 5 ) /* comment */ + "bar"
                function copyCommentsWhenParenthesized(node) {
                    var file = node.getSourceFile();
                    ts.copyTrailingComments(node, node.expression, file, 3 /* MultiLineCommentTrivia */, /* hasTrailingNewLine */ false);
                    ts.copyTrailingAsLeadingComments(node.expression, node.expression, file, 3 /* MultiLineCommentTrivia */, /* hasTrailingNewLine */ false);
                }
                function getExpressionFromParenthesesOrExpression(node) {
                    if (ts.isParenthesizedExpression(node)) {
                        copyCommentsWhenParenthesized(node);
                        node = node.expression;
                    }
                    return node;
                }
            })(convertStringOrTemplateLiteral = refactor.convertStringOrTemplateLiteral || (refactor.convertStringOrTemplateLiteral = {}));
        })(refactor = ts.refactor || (ts.refactor = {}));
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        /** The version of the language service API */
        ts.servicesVersion = "0.8";
        function createNode(kind, pos, end, parent) {
            var node = ts.isNodeKind(kind) ? new NodeObject(kind, pos, end) :
                kind === 75 /* Identifier */ ? new IdentifierObject(75 /* Identifier */, pos, end) :
                    kind === 76 /* PrivateIdentifier */ ? new PrivateIdentifierObject(76 /* PrivateIdentifier */, pos, end) :
                        new TokenObject(kind, pos, end);
            node.parent = parent;
            node.flags = parent.flags & 25358336 /* ContextFlags */;
            return node;
        }
        var NodeObject = /** @class */ (function () {
            function NodeObject(kind, pos, end) {
                this.pos = pos;
                this.end = end;
                this.flags = 0 /* None */;
                this.modifierFlagsCache = 0 /* None */;
                this.transformFlags = 0 /* None */;
                this.parent = undefined;
                this.kind = kind;
            }
            NodeObject.prototype.assertHasRealPosition = function (message) {
                // eslint-disable-next-line debug-assert
                ts.Debug.assert(!ts.positionIsSynthesized(this.pos) && !ts.positionIsSynthesized(this.end), message || "Node must have a real position for this operation");
            };
            NodeObject.prototype.getSourceFile = function () {
                return ts.getSourceFileOfNode(this);
            };
            NodeObject.prototype.getStart = function (sourceFile, includeJsDocComment) {
                this.assertHasRealPosition();
                return ts.getTokenPosOfNode(this, sourceFile, includeJsDocComment);
            };
            NodeObject.prototype.getFullStart = function () {
                this.assertHasRealPosition();
                return this.pos;
            };
            NodeObject.prototype.getEnd = function () {
                this.assertHasRealPosition();
                return this.end;
            };
            NodeObject.prototype.getWidth = function (sourceFile) {
                this.assertHasRealPosition();
                return this.getEnd() - this.getStart(sourceFile);
            };
            NodeObject.prototype.getFullWidth = function () {
                this.assertHasRealPosition();
                return this.end - this.pos;
            };
            NodeObject.prototype.getLeadingTriviaWidth = function (sourceFile) {
                this.assertHasRealPosition();
                return this.getStart(sourceFile) - this.pos;
            };
            NodeObject.prototype.getFullText = function (sourceFile) {
                this.assertHasRealPosition();
                return (sourceFile || this.getSourceFile()).text.substring(this.pos, this.end);
            };
            NodeObject.prototype.getText = function (sourceFile) {
                this.assertHasRealPosition();
                if (!sourceFile) {
                    sourceFile = this.getSourceFile();
                }
                return sourceFile.text.substring(this.getStart(sourceFile), this.getEnd());
            };
            NodeObject.prototype.getChildCount = function (sourceFile) {
                return this.getChildren(sourceFile).length;
            };
            NodeObject.prototype.getChildAt = function (index, sourceFile) {
                return this.getChildren(sourceFile)[index];
            };
            NodeObject.prototype.getChildren = function (sourceFile) {
                this.assertHasRealPosition("Node without a real position cannot be scanned and thus has no token nodes - use forEachChild and collect the result if that's fine");
                return this._children || (this._children = createChildren(this, sourceFile));
            };
            NodeObject.prototype.getFirstToken = function (sourceFile) {
                this.assertHasRealPosition();
                var children = this.getChildren(sourceFile);
                if (!children.length) {
                    return undefined;
                }
                var child = ts.find(children, function (kid) { return kid.kind < 294 /* FirstJSDocNode */ || kid.kind > 322 /* LastJSDocNode */; });
                return child.kind < 153 /* FirstNode */ ?
                    child :
                    child.getFirstToken(sourceFile);
            };
            NodeObject.prototype.getLastToken = function (sourceFile) {
                this.assertHasRealPosition();
                var children = this.getChildren(sourceFile);
                var child = ts.lastOrUndefined(children);
                if (!child) {
                    return undefined;
                }
                return child.kind < 153 /* FirstNode */ ? child : child.getLastToken(sourceFile);
            };
            NodeObject.prototype.forEachChild = function (cbNode, cbNodeArray) {
                return ts.forEachChild(this, cbNode, cbNodeArray);
            };
            return NodeObject;
        }());
        function createChildren(node, sourceFile) {
            if (!ts.isNodeKind(node.kind)) {
                return ts.emptyArray;
            }
            var children = [];
            if (ts.isJSDocCommentContainingNode(node)) {
                /** Don't add trivia for "tokens" since this is in a comment. */
                node.forEachChild(function (child) { children.push(child); });
                return children;
            }
            ts.scanner.setText((sourceFile || node.getSourceFile()).text);
            var pos = node.pos;
            var processNode = function (child) {
                addSyntheticNodes(children, pos, child.pos, node);
                children.push(child);
                pos = child.end;
            };
            var processNodes = function (nodes) {
                addSyntheticNodes(children, pos, nodes.pos, node);
                children.push(createSyntaxList(nodes, node));
                pos = nodes.end;
            };
            // jsDocComments need to be the first children
            ts.forEach(node.jsDoc, processNode);
            // For syntactic classifications, all trivia are classified together, including jsdoc comments.
            // For that to work, the jsdoc comments should still be the leading trivia of the first child.
            // Restoring the scanner position ensures that.
            pos = node.pos;
            node.forEachChild(processNode, processNodes);
            addSyntheticNodes(children, pos, node.end, node);
            ts.scanner.setText(undefined);
            return children;
        }
        function addSyntheticNodes(nodes, pos, end, parent) {
            ts.scanner.setTextPos(pos);
            while (pos < end) {
                var token = ts.scanner.scan();
                var textPos = ts.scanner.getTextPos();
                if (textPos <= end) {
                    if (token === 75 /* Identifier */) {
                        ts.Debug.fail("Did not expect " + ts.Debug.formatSyntaxKind(parent.kind) + " to have an Identifier in its trivia");
                    }
                    nodes.push(createNode(token, pos, textPos, parent));
                }
                pos = textPos;
                if (token === 1 /* EndOfFileToken */) {
                    break;
                }
            }
        }
        function createSyntaxList(nodes, parent) {
            var list = createNode(323 /* SyntaxList */, nodes.pos, nodes.end, parent);
            list._children = [];
            var pos = nodes.pos;
            for (var _i = 0, nodes_1 = nodes; _i < nodes_1.length; _i++) {
                var node = nodes_1[_i];
                addSyntheticNodes(list._children, pos, node.pos, parent);
                list._children.push(node);
                pos = node.end;
            }
            addSyntheticNodes(list._children, pos, nodes.end, parent);
            return list;
        }
        var TokenOrIdentifierObject = /** @class */ (function () {
            function TokenOrIdentifierObject(pos, end) {
                // Set properties in same order as NodeObject
                this.pos = pos;
                this.end = end;
                this.flags = 0 /* None */;
                this.modifierFlagsCache = 0 /* None */;
                this.transformFlags = 0 /* None */;
                this.parent = undefined;
            }
            TokenOrIdentifierObject.prototype.getSourceFile = function () {
                return ts.getSourceFileOfNode(this);
            };
            TokenOrIdentifierObject.prototype.getStart = function (sourceFile, includeJsDocComment) {
                return ts.getTokenPosOfNode(this, sourceFile, includeJsDocComment);
            };
            TokenOrIdentifierObject.prototype.getFullStart = function () {
                return this.pos;
            };
            TokenOrIdentifierObject.prototype.getEnd = function () {
                return this.end;
            };
            TokenOrIdentifierObject.prototype.getWidth = function (sourceFile) {
                return this.getEnd() - this.getStart(sourceFile);
            };
            TokenOrIdentifierObject.prototype.getFullWidth = function () {
                return this.end - this.pos;
            };
            TokenOrIdentifierObject.prototype.getLeadingTriviaWidth = function (sourceFile) {
                return this.getStart(sourceFile) - this.pos;
            };
            TokenOrIdentifierObject.prototype.getFullText = function (sourceFile) {
                return (sourceFile || this.getSourceFile()).text.substring(this.pos, this.end);
            };
            TokenOrIdentifierObject.prototype.getText = function (sourceFile) {
                if (!sourceFile) {
                    sourceFile = this.getSourceFile();
                }
                return sourceFile.text.substring(this.getStart(sourceFile), this.getEnd());
            };
            TokenOrIdentifierObject.prototype.getChildCount = function () {
                return 0;
            };
            TokenOrIdentifierObject.prototype.getChildAt = function () {
                return undefined; // TODO: GH#18217
            };
            TokenOrIdentifierObject.prototype.getChildren = function () {
                return this.kind === 1 /* EndOfFileToken */ ? this.jsDoc || ts.emptyArray : ts.emptyArray;
            };
            TokenOrIdentifierObject.prototype.getFirstToken = function () {
                return undefined;
            };
            TokenOrIdentifierObject.prototype.getLastToken = function () {
                return undefined;
            };
            TokenOrIdentifierObject.prototype.forEachChild = function () {
                return undefined;
            };
            return TokenOrIdentifierObject;
        }());
        var SymbolObject = /** @class */ (function () {
            function SymbolObject(flags, name) {
                this.flags = flags;
                this.escapedName = name;
            }
            SymbolObject.prototype.getFlags = function () {
                return this.flags;
            };
            Object.defineProperty(SymbolObject.prototype, "name", {
                get: function () {
                    return ts.symbolName(this);
                },
                enumerable: true,
                configurable: true
            });
            SymbolObject.prototype.getEscapedName = function () {
                return this.escapedName;
            };
            SymbolObject.prototype.getName = function () {
                return this.name;
            };
            SymbolObject.prototype.getDeclarations = function () {
                return this.declarations;
            };
            SymbolObject.prototype.getDocumentationComment = function (checker) {
                if (!this.documentationComment) {
                    this.documentationComment = ts.emptyArray; // Set temporarily to avoid an infinite loop finding inherited docs
                    this.documentationComment = getDocumentationComment(this.declarations, checker);
                }
                return this.documentationComment;
            };
            SymbolObject.prototype.getJsDocTags = function () {
                if (this.tags === undefined) {
                    this.tags = ts.JsDoc.getJsDocTagsFromDeclarations(this.declarations);
                }
                return this.tags;
            };
            return SymbolObject;
        }());
        var TokenObject = /** @class */ (function (_super) {
            __extends(TokenObject, _super);
            function TokenObject(kind, pos, end) {
                var _this = _super.call(this, pos, end) || this;
                _this.kind = kind;
                return _this;
            }
            return TokenObject;
        }(TokenOrIdentifierObject));
        var IdentifierObject = /** @class */ (function (_super) {
            __extends(IdentifierObject, _super);
            function IdentifierObject(_kind, pos, end) {
                var _this = _super.call(this, pos, end) || this;
                _this.kind = 75 /* Identifier */;
                return _this;
            }
            Object.defineProperty(IdentifierObject.prototype, "text", {
                get: function () {
                    return ts.idText(this);
                },
                enumerable: true,
                configurable: true
            });
            return IdentifierObject;
        }(TokenOrIdentifierObject));
        IdentifierObject.prototype.kind = 75 /* Identifier */;
        var PrivateIdentifierObject = /** @class */ (function (_super) {
            __extends(PrivateIdentifierObject, _super);
            function PrivateIdentifierObject(_kind, pos, end) {
                return _super.call(this, pos, end) || this;
            }
            Object.defineProperty(PrivateIdentifierObject.prototype, "text", {
                get: function () {
                    return ts.idText(this);
                },
                enumerable: true,
                configurable: true
            });
            return PrivateIdentifierObject;
        }(TokenOrIdentifierObject));
        PrivateIdentifierObject.prototype.kind = 76 /* PrivateIdentifier */;
        var TypeObject = /** @class */ (function () {
            function TypeObject(checker, flags) {
                this.checker = checker;
                this.flags = flags;
            }
            TypeObject.prototype.getFlags = function () {
                return this.flags;
            };
            TypeObject.prototype.getSymbol = function () {
                return this.symbol;
            };
            TypeObject.prototype.getProperties = function () {
                return this.checker.getPropertiesOfType(this);
            };
            TypeObject.prototype.getProperty = function (propertyName) {
                return this.checker.getPropertyOfType(this, propertyName);
            };
            TypeObject.prototype.getApparentProperties = function () {
                return this.checker.getAugmentedPropertiesOfType(this);
            };
            TypeObject.prototype.getCallSignatures = function () {
                return this.checker.getSignaturesOfType(this, 0 /* Call */);
            };
            TypeObject.prototype.getConstructSignatures = function () {
                return this.checker.getSignaturesOfType(this, 1 /* Construct */);
            };
            TypeObject.prototype.getStringIndexType = function () {
                return this.checker.getIndexTypeOfType(this, 0 /* String */);
            };
            TypeObject.prototype.getNumberIndexType = function () {
                return this.checker.getIndexTypeOfType(this, 1 /* Number */);
            };
            TypeObject.prototype.getBaseTypes = function () {
                return this.isClassOrInterface() ? this.checker.getBaseTypes(this) : undefined;
            };
            TypeObject.prototype.isNullableType = function () {
                return this.checker.isNullableType(this);
            };
            TypeObject.prototype.getNonNullableType = function () {
                return this.checker.getNonNullableType(this);
            };
            TypeObject.prototype.getNonOptionalType = function () {
                return this.checker.getNonOptionalType(this);
            };
            TypeObject.prototype.getConstraint = function () {
                return this.checker.getBaseConstraintOfType(this);
            };
            TypeObject.prototype.getDefault = function () {
                return this.checker.getDefaultFromTypeParameter(this);
            };
            TypeObject.prototype.isUnion = function () {
                return !!(this.flags & 1048576 /* Union */);
            };
            TypeObject.prototype.isIntersection = function () {
                return !!(this.flags & 2097152 /* Intersection */);
            };
            TypeObject.prototype.isUnionOrIntersection = function () {
                return !!(this.flags & 3145728 /* UnionOrIntersection */);
            };
            TypeObject.prototype.isLiteral = function () {
                return !!(this.flags & 384 /* StringOrNumberLiteral */);
            };
            TypeObject.prototype.isStringLiteral = function () {
                return !!(this.flags & 128 /* StringLiteral */);
            };
            TypeObject.prototype.isNumberLiteral = function () {
                return !!(this.flags & 256 /* NumberLiteral */);
            };
            TypeObject.prototype.isTypeParameter = function () {
                return !!(this.flags & 262144 /* TypeParameter */);
            };
            TypeObject.prototype.isClassOrInterface = function () {
                return !!(ts.getObjectFlags(this) & 3 /* ClassOrInterface */);
            };
            TypeObject.prototype.isClass = function () {
                return !!(ts.getObjectFlags(this) & 1 /* Class */);
            };
            Object.defineProperty(TypeObject.prototype, "typeArguments", {
                /**
                 * This polyfills `referenceType.typeArguments` for API consumers
                 */
                get: function () {
                    if (ts.getObjectFlags(this) & 4 /* Reference */) {
                        return this.checker.getTypeArguments(this);
                    }
                    return undefined;
                },
                enumerable: true,
                configurable: true
            });
            return TypeObject;
        }());
        var SignatureObject = /** @class */ (function () {
            function SignatureObject(checker, flags) {
                this.checker = checker;
                this.flags = flags;
            }
            SignatureObject.prototype.getDeclaration = function () {
                return this.declaration;
            };
            SignatureObject.prototype.getTypeParameters = function () {
                return this.typeParameters;
            };
            SignatureObject.prototype.getParameters = function () {
                return this.parameters;
            };
            SignatureObject.prototype.getReturnType = function () {
                return this.checker.getReturnTypeOfSignature(this);
            };
            SignatureObject.prototype.getDocumentationComment = function () {
                return this.documentationComment || (this.documentationComment = getDocumentationComment(ts.singleElementArray(this.declaration), this.checker));
            };
            SignatureObject.prototype.getJsDocTags = function () {
                if (this.jsDocTags === undefined) {
                    this.jsDocTags = this.declaration ? ts.JsDoc.getJsDocTagsFromDeclarations([this.declaration]) : [];
                }
                return this.jsDocTags;
            };
            return SignatureObject;
        }());
        /**
         * Returns whether or not the given node has a JSDoc "inheritDoc" tag on it.
         * @param node the Node in question.
         * @returns `true` if `node` has a JSDoc "inheritDoc" tag on it, otherwise `false`.
         */
        function hasJSDocInheritDocTag(node) {
            return ts.getJSDocTags(node).some(function (tag) { return tag.tagName.text === "inheritDoc"; });
        }
        function getDocumentationComment(declarations, checker) {
            if (!declarations)
                return ts.emptyArray;
            var doc = ts.JsDoc.getJsDocCommentsFromDeclarations(declarations);
            if (doc.length === 0 || declarations.some(hasJSDocInheritDocTag)) {
                ts.forEachUnique(declarations, function (declaration) {
                    var inheritedDocs = findInheritedJSDocComments(declaration, declaration.symbol.name, checker); // TODO: GH#18217
                    // TODO: GH#16312 Return a ReadonlyArray, avoid copying inheritedDocs
                    if (inheritedDocs)
                        doc = doc.length === 0 ? inheritedDocs.slice() : inheritedDocs.concat(ts.lineBreakPart(), doc);
                });
            }
            return doc;
        }
        /**
         * Attempts to find JSDoc comments for possibly-inherited properties.  Checks superclasses then traverses
         * implemented interfaces until a symbol is found with the same name and with documentation.
         * @param declaration The possibly-inherited declaration to find comments for.
         * @param propertyName The name of the possibly-inherited property.
         * @param typeChecker A TypeChecker, used to find inherited properties.
         * @returns A filled array of documentation comments if any were found, otherwise an empty array.
         */
        function findInheritedJSDocComments(declaration, propertyName, typeChecker) {
            return ts.firstDefined(declaration.parent ? ts.getAllSuperTypeNodes(declaration.parent) : ts.emptyArray, function (superTypeNode) {
                var superType = typeChecker.getTypeAtLocation(superTypeNode);
                var baseProperty = superType && typeChecker.getPropertyOfType(superType, propertyName);
                var inheritedDocs = baseProperty && baseProperty.getDocumentationComment(typeChecker);
                return inheritedDocs && inheritedDocs.length ? inheritedDocs : undefined;
            });
        }
        var SourceFileObject = /** @class */ (function (_super) {
            __extends(SourceFileObject, _super);
            function SourceFileObject(kind, pos, end) {
                var _this = _super.call(this, kind, pos, end) || this;
                _this.kind = 290 /* SourceFile */;
                return _this;
            }
            SourceFileObject.prototype.update = function (newText, textChangeRange) {
                return ts.updateSourceFile(this, newText, textChangeRange);
            };
            SourceFileObject.prototype.getLineAndCharacterOfPosition = function (position) {
                return ts.getLineAndCharacterOfPosition(this, position);
            };
            SourceFileObject.prototype.getLineStarts = function () {
                return ts.getLineStarts(this);
            };
            SourceFileObject.prototype.getPositionOfLineAndCharacter = function (line, character, allowEdits) {
                return ts.computePositionOfLineAndCharacter(ts.getLineStarts(this), line, character, this.text, allowEdits);
            };
            SourceFileObject.prototype.getLineEndOfPosition = function (pos) {
                var line = this.getLineAndCharacterOfPosition(pos).line;
                var lineStarts = this.getLineStarts();
                var lastCharPos;
                if (line + 1 >= lineStarts.length) {
                    lastCharPos = this.getEnd();
                }
                if (!lastCharPos) {
                    lastCharPos = lineStarts[line + 1] - 1;
                }
                var fullText = this.getFullText();
                // if the new line is "\r\n", we should return the last non-new-line-character position
                return fullText[lastCharPos] === "\n" && fullText[lastCharPos - 1] === "\r" ? lastCharPos - 1 : lastCharPos;
            };
            SourceFileObject.prototype.getNamedDeclarations = function () {
                if (!this.namedDeclarations) {
                    this.namedDeclarations = this.computeNamedDeclarations();
                }
                return this.namedDeclarations;
            };
            SourceFileObject.prototype.computeNamedDeclarations = function () {
                var result = ts.createMultiMap();
                this.forEachChild(visit);
                return result;
                function addDeclaration(declaration) {
                    var name = getDeclarationName(declaration);
                    if (name) {
                        result.add(name, declaration);
                    }
                }
                function getDeclarations(name) {
                    var declarations = result.get(name);
                    if (!declarations) {
                        result.set(name, declarations = []);
                    }
                    return declarations;
                }
                function getDeclarationName(declaration) {
                    var name = ts.getNonAssignedNameOfDeclaration(declaration);
                    return name && (ts.isComputedPropertyName(name) && ts.isPropertyAccessExpression(name.expression) ? name.expression.name.text
                        : ts.isPropertyName(name) ? ts.getNameFromPropertyName(name) : undefined);
                }
                function visit(node) {
                    switch (node.kind) {
                        case 244 /* FunctionDeclaration */:
                        case 201 /* FunctionExpression */:
                        case 161 /* MethodDeclaration */:
                        case 160 /* MethodSignature */:
                            var functionDeclaration = node;
                            var declarationName = getDeclarationName(functionDeclaration);
                            if (declarationName) {
                                var declarations = getDeclarations(declarationName);
                                var lastDeclaration = ts.lastOrUndefined(declarations);
                                // Check whether this declaration belongs to an "overload group".
                                if (lastDeclaration && functionDeclaration.parent === lastDeclaration.parent && functionDeclaration.symbol === lastDeclaration.symbol) {
                                    // Overwrite the last declaration if it was an overload
                                    // and this one is an implementation.
                                    if (functionDeclaration.body && !lastDeclaration.body) {
                                        declarations[declarations.length - 1] = functionDeclaration;
                                    }
                                }
                                else {
                                    declarations.push(functionDeclaration);
                                }
                            }
                            ts.forEachChild(node, visit);
                            break;
                        case 245 /* ClassDeclaration */:
                        case 214 /* ClassExpression */:
                        case 246 /* InterfaceDeclaration */:
                        case 247 /* TypeAliasDeclaration */:
                        case 248 /* EnumDeclaration */:
                        case 249 /* ModuleDeclaration */:
                        case 253 /* ImportEqualsDeclaration */:
                        case 263 /* ExportSpecifier */:
                        case 258 /* ImportSpecifier */:
                        case 255 /* ImportClause */:
                        case 256 /* NamespaceImport */:
                        case 163 /* GetAccessor */:
                        case 164 /* SetAccessor */:
                        case 173 /* TypeLiteral */:
                            addDeclaration(node);
                            ts.forEachChild(node, visit);
                            break;
                        case 156 /* Parameter */:
                            // Only consider parameter properties
                            if (!ts.hasModifier(node, 92 /* ParameterPropertyModifier */)) {
                                break;
                            }
                        // falls through
                        case 242 /* VariableDeclaration */:
                        case 191 /* BindingElement */: {
                            var decl = node;
                            if (ts.isBindingPattern(decl.name)) {
                                ts.forEachChild(decl.name, visit);
                                break;
                            }
                            if (decl.initializer) {
                                visit(decl.initializer);
                            }
                        }
                        // falls through
                        case 284 /* EnumMember */:
                        case 159 /* PropertyDeclaration */:
                        case 158 /* PropertySignature */:
                            addDeclaration(node);
                            break;
                        case 260 /* ExportDeclaration */:
                            // Handle named exports case e.g.:
                            //    export {a, b as B} from "mod";
                            var exportDeclaration = node;
                            if (exportDeclaration.exportClause) {
                                if (ts.isNamedExports(exportDeclaration.exportClause)) {
                                    ts.forEach(exportDeclaration.exportClause.elements, visit);
                                }
                                else {
                                    visit(exportDeclaration.exportClause.name);
                                }
                            }
                            break;
                        case 254 /* ImportDeclaration */:
                            var importClause = node.importClause;
                            if (importClause) {
                                // Handle default import case e.g.:
                                //    import d from "mod";
                                if (importClause.name) {
                                    addDeclaration(importClause.name);
                                }
                                // Handle named bindings in imports e.g.:
                                //    import * as NS from "mod";
                                //    import {a, b as B} from "mod";
                                if (importClause.namedBindings) {
                                    if (importClause.namedBindings.kind === 256 /* NamespaceImport */) {
                                        addDeclaration(importClause.namedBindings);
                                    }
                                    else {
                                        ts.forEach(importClause.namedBindings.elements, visit);
                                    }
                                }
                            }
                            break;
                        case 209 /* BinaryExpression */:
                            if (ts.getAssignmentDeclarationKind(node) !== 0 /* None */) {
                                addDeclaration(node);
                            }
                        // falls through
                        default:
                            ts.forEachChild(node, visit);
                    }
                }
            };
            return SourceFileObject;
        }(NodeObject));
        var SourceMapSourceObject = /** @class */ (function () {
            function SourceMapSourceObject(fileName, text, skipTrivia) {
                this.fileName = fileName;
                this.text = text;
                this.skipTrivia = skipTrivia;
            }
            SourceMapSourceObject.prototype.getLineAndCharacterOfPosition = function (pos) {
                return ts.getLineAndCharacterOfPosition(this, pos);
            };
            return SourceMapSourceObject;
        }());
        function getServicesObjectAllocator() {
            return {
                getNodeConstructor: function () { return NodeObject; },
                getTokenConstructor: function () { return TokenObject; },
                getIdentifierConstructor: function () { return IdentifierObject; },
                getPrivateIdentifierConstructor: function () { return PrivateIdentifierObject; },
                getSourceFileConstructor: function () { return SourceFileObject; },
                getSymbolConstructor: function () { return SymbolObject; },
                getTypeConstructor: function () { return TypeObject; },
                getSignatureConstructor: function () { return SignatureObject; },
                getSourceMapSourceConstructor: function () { return SourceMapSourceObject; },
            };
        }
        function toEditorSettings(optionsAsMap) {
            var allPropertiesAreCamelCased = true;
            for (var key in optionsAsMap) {
                if (ts.hasProperty(optionsAsMap, key) && !isCamelCase(key)) {
                    allPropertiesAreCamelCased = false;
                    break;
                }
            }
            if (allPropertiesAreCamelCased) {
                return optionsAsMap;
            }
            var settings = {};
            for (var key in optionsAsMap) {
                if (ts.hasProperty(optionsAsMap, key)) {
                    var newKey = isCamelCase(key) ? key : key.charAt(0).toLowerCase() + key.substr(1);
                    settings[newKey] = optionsAsMap[key];
                }
            }
            return settings;
        }
        ts.toEditorSettings = toEditorSettings;
        function isCamelCase(s) {
            return !s.length || s.charAt(0) === s.charAt(0).toLowerCase();
        }
        function displayPartsToString(displayParts) {
            if (displayParts) {
                return ts.map(displayParts, function (displayPart) { return displayPart.text; }).join("");
            }
            return "";
        }
        ts.displayPartsToString = displayPartsToString;
        function getDefaultCompilerOptions() {
            // Always default to "ScriptTarget.ES5" for the language service
            return {
                target: 1 /* ES5 */,
                jsx: 1 /* Preserve */
            };
        }
        ts.getDefaultCompilerOptions = getDefaultCompilerOptions;
        function getSupportedCodeFixes() {
            return ts.codefix.getSupportedErrorCodes();
        }
        ts.getSupportedCodeFixes = getSupportedCodeFixes;
        // Cache host information about script Should be refreshed
        // at each language service public entry point, since we don't know when
        // the set of scripts handled by the host changes.
        var HostCache = /** @class */ (function () {
            function HostCache(host, getCanonicalFileName) {
                this.host = host;
                // script id => script index
                this.currentDirectory = host.getCurrentDirectory();
                this.fileNameToEntry = ts.createMap();
                // Initialize the list with the root file names
                var rootFileNames = host.getScriptFileNames();
                for (var _i = 0, rootFileNames_1 = rootFileNames; _i < rootFileNames_1.length; _i++) {
                    var fileName = rootFileNames_1[_i];
                    this.createEntry(fileName, ts.toPath(fileName, this.currentDirectory, getCanonicalFileName));
                }
                // store the compilation settings
                this._compilationSettings = host.getCompilationSettings() || getDefaultCompilerOptions();
            }
            HostCache.prototype.compilationSettings = function () {
                return this._compilationSettings;
            };
            HostCache.prototype.getProjectReferences = function () {
                return this.host.getProjectReferences && this.host.getProjectReferences();
            };
            HostCache.prototype.createEntry = function (fileName, path) {
                var entry;
                var scriptSnapshot = this.host.getScriptSnapshot(fileName);
                if (scriptSnapshot) {
                    entry = {
                        hostFileName: fileName,
                        version: this.host.getScriptVersion(fileName),
                        scriptSnapshot: scriptSnapshot,
                        scriptKind: ts.getScriptKind(fileName, this.host)
                    };
                }
                else {
                    entry = fileName;
                }
                this.fileNameToEntry.set(path, entry);
                return entry;
            };
            HostCache.prototype.getEntryByPath = function (path) {
                return this.fileNameToEntry.get(path);
            };
            HostCache.prototype.getHostFileInformation = function (path) {
                var entry = this.fileNameToEntry.get(path);
                return !ts.isString(entry) ? entry : undefined;
            };
            HostCache.prototype.getOrCreateEntryByPath = function (fileName, path) {
                var info = this.getEntryByPath(path) || this.createEntry(fileName, path);
                return ts.isString(info) ? undefined : info; // TODO: GH#18217
            };
            HostCache.prototype.getRootFileNames = function () {
                var names = [];
                this.fileNameToEntry.forEach(function (entry) {
                    if (ts.isString(entry)) {
                        names.push(entry);
                    }
                    else {
                        names.push(entry.hostFileName);
                    }
                });
                return names;
            };
            HostCache.prototype.getVersion = function (path) {
                var file = this.getHostFileInformation(path);
                return (file && file.version); // TODO: GH#18217
            };
            HostCache.prototype.getScriptSnapshot = function (path) {
                var file = this.getHostFileInformation(path);
                return (file && file.scriptSnapshot); // TODO: GH#18217
            };
            return HostCache;
        }());
        var SyntaxTreeCache = /** @class */ (function () {
            function SyntaxTreeCache(host) {
                this.host = host;
            }
            SyntaxTreeCache.prototype.getCurrentSourceFile = function (fileName) {
                var scriptSnapshot = this.host.getScriptSnapshot(fileName);
                if (!scriptSnapshot) {
                    // The host does not know about this file.
                    throw new Error("Could not find file: '" + fileName + "'.");
                }
                var scriptKind = ts.getScriptKind(fileName, this.host);
                var version = this.host.getScriptVersion(fileName);
                var sourceFile;
                if (this.currentFileName !== fileName) {
                    // This is a new file, just parse it
                    sourceFile = createLanguageServiceSourceFile(fileName, scriptSnapshot, 99 /* Latest */, version, /*setNodeParents*/ true, scriptKind);
                }
                else if (this.currentFileVersion !== version) {
                    // This is the same file, just a newer version. Incrementally parse the file.
                    var editRange = scriptSnapshot.getChangeRange(this.currentFileScriptSnapshot);
                    sourceFile = updateLanguageServiceSourceFile(this.currentSourceFile, scriptSnapshot, version, editRange);
                }
                if (sourceFile) {
                    // All done, ensure state is up to date
                    this.currentFileVersion = version;
                    this.currentFileName = fileName;
                    this.currentFileScriptSnapshot = scriptSnapshot;
                    this.currentSourceFile = sourceFile;
                }
                return this.currentSourceFile;
            };
            return SyntaxTreeCache;
        }());
        function setSourceFileFields(sourceFile, scriptSnapshot, version) {
            sourceFile.version = version;
            sourceFile.scriptSnapshot = scriptSnapshot;
        }
        function createLanguageServiceSourceFile(fileName, scriptSnapshot, scriptTarget, version, setNodeParents, scriptKind) {
            var sourceFile = ts.createSourceFile(fileName, ts.getSnapshotText(scriptSnapshot), scriptTarget, setNodeParents, scriptKind);
            setSourceFileFields(sourceFile, scriptSnapshot, version);
            return sourceFile;
        }
        ts.createLanguageServiceSourceFile = createLanguageServiceSourceFile;
        function updateLanguageServiceSourceFile(sourceFile, scriptSnapshot, version, textChangeRange, aggressiveChecks) {
            // If we were given a text change range, and our version or open-ness changed, then
            // incrementally parse this file.
            if (textChangeRange) {
                if (version !== sourceFile.version) {
                    var newText = void 0;
                    // grab the fragment from the beginning of the original text to the beginning of the span
                    var prefix = textChangeRange.span.start !== 0
                        ? sourceFile.text.substr(0, textChangeRange.span.start)
                        : "";
                    // grab the fragment from the end of the span till the end of the original text
                    var suffix = ts.textSpanEnd(textChangeRange.span) !== sourceFile.text.length
                        ? sourceFile.text.substr(ts.textSpanEnd(textChangeRange.span))
                        : "";
                    if (textChangeRange.newLength === 0) {
                        // edit was a deletion - just combine prefix and suffix
                        newText = prefix && suffix ? prefix + suffix : prefix || suffix;
                    }
                    else {
                        // it was actual edit, fetch the fragment of new text that correspond to new span
                        var changedText = scriptSnapshot.getText(textChangeRange.span.start, textChangeRange.span.start + textChangeRange.newLength);
                        // combine prefix, changed text and suffix
                        newText = prefix && suffix
                            ? prefix + changedText + suffix
                            : prefix
                                ? (prefix + changedText)
                                : (changedText + suffix);
                    }
                    var newSourceFile = ts.updateSourceFile(sourceFile, newText, textChangeRange, aggressiveChecks);
                    setSourceFileFields(newSourceFile, scriptSnapshot, version);
                    // after incremental parsing nameTable might not be up-to-date
                    // drop it so it can be lazily recreated later
                    newSourceFile.nameTable = undefined;
                    // dispose all resources held by old script snapshot
                    if (sourceFile !== newSourceFile && sourceFile.scriptSnapshot) {
                        if (sourceFile.scriptSnapshot.dispose) {
                            sourceFile.scriptSnapshot.dispose();
                        }
                        sourceFile.scriptSnapshot = undefined;
                    }
                    return newSourceFile;
                }
            }
            // Otherwise, just create a new source file.
            return createLanguageServiceSourceFile(sourceFile.fileName, scriptSnapshot, sourceFile.languageVersion, version, /*setNodeParents*/ true, sourceFile.scriptKind);
        }
        ts.updateLanguageServiceSourceFile = updateLanguageServiceSourceFile;
        var CancellationTokenObject = /** @class */ (function () {
            function CancellationTokenObject(cancellationToken) {
                this.cancellationToken = cancellationToken;
            }
            CancellationTokenObject.prototype.isCancellationRequested = function () {
                return !!this.cancellationToken && this.cancellationToken.isCancellationRequested();
            };
            CancellationTokenObject.prototype.throwIfCancellationRequested = function () {
                if (this.isCancellationRequested()) {
                    throw new ts.OperationCanceledException();
                }
            };
            return CancellationTokenObject;
        }());
        /* @internal */
        /** A cancellation that throttles calls to the host */
        var ThrottledCancellationToken = /** @class */ (function () {
            function ThrottledCancellationToken(hostCancellationToken, throttleWaitMilliseconds) {
                if (throttleWaitMilliseconds === void 0) {
                    throttleWaitMilliseconds = 20;
                }
                this.hostCancellationToken = hostCancellationToken;
                this.throttleWaitMilliseconds = throttleWaitMilliseconds;
                // Store when we last tried to cancel.  Checking cancellation can be expensive (as we have
                // to marshall over to the host layer).  So we only bother actually checking once enough
                // time has passed.
                this.lastCancellationCheckTime = 0;
            }
            ThrottledCancellationToken.prototype.isCancellationRequested = function () {
                var time = ts.timestamp();
                var duration = Math.abs(time - this.lastCancellationCheckTime);
                if (duration >= this.throttleWaitMilliseconds) {
                    // Check no more than once every throttle wait milliseconds
                    this.lastCancellationCheckTime = time;
                    return this.hostCancellationToken.isCancellationRequested();
                }
                return false;
            };
            ThrottledCancellationToken.prototype.throwIfCancellationRequested = function () {
                if (this.isCancellationRequested()) {
                    throw new ts.OperationCanceledException();
                }
            };
            return ThrottledCancellationToken;
        }());
        ts.ThrottledCancellationToken = ThrottledCancellationToken;
        function createLanguageService(host, documentRegistry, syntaxOnly) {
            var _a;
            if (documentRegistry === void 0) {
                documentRegistry = ts.createDocumentRegistry(host.useCaseSensitiveFileNames && host.useCaseSensitiveFileNames(), host.getCurrentDirectory());
            }
            if (syntaxOnly === void 0) {
                syntaxOnly = false;
            }
            var syntaxTreeCache = new SyntaxTreeCache(host);
            var program;
            var lastProjectVersion;
            var lastTypesRootVersion = 0;
            var cancellationToken = new CancellationTokenObject(host.getCancellationToken && host.getCancellationToken());
            var currentDirectory = host.getCurrentDirectory();
            // Check if the localized messages json is set, otherwise query the host for it
            if (!ts.localizedDiagnosticMessages && host.getLocalizedDiagnosticMessages) {
                ts.setLocalizedDiagnosticMessages(host.getLocalizedDiagnosticMessages());
            }
            function log(message) {
                if (host.log) {
                    host.log(message);
                }
            }
            var useCaseSensitiveFileNames = ts.hostUsesCaseSensitiveFileNames(host);
            var getCanonicalFileName = ts.createGetCanonicalFileName(useCaseSensitiveFileNames);
            var sourceMapper = ts.getSourceMapper({
                useCaseSensitiveFileNames: function () { return useCaseSensitiveFileNames; },
                getCurrentDirectory: function () { return currentDirectory; },
                getProgram: getProgram,
                fileExists: ts.maybeBind(host, host.fileExists),
                readFile: ts.maybeBind(host, host.readFile),
                getDocumentPositionMapper: ts.maybeBind(host, host.getDocumentPositionMapper),
                getSourceFileLike: ts.maybeBind(host, host.getSourceFileLike),
                log: log
            });
            function getValidSourceFile(fileName) {
                var sourceFile = program.getSourceFile(fileName);
                if (!sourceFile) {
                    var error = new Error("Could not find source file: '" + fileName + "'.");
                    // We've been having trouble debugging this, so attach sidecar data for the tsserver log.
                    // See https://github.com/microsoft/TypeScript/issues/30180.
                    error.ProgramFiles = program.getSourceFiles().map(function (f) { return f.fileName; });
                    throw error;
                }
                return sourceFile;
            }
            function synchronizeHostData() {
                ts.Debug.assert(!syntaxOnly);
                // perform fast check if host supports it
                if (host.getProjectVersion) {
                    var hostProjectVersion = host.getProjectVersion();
                    if (hostProjectVersion) {
                        if (lastProjectVersion === hostProjectVersion && !host.hasChangedAutomaticTypeDirectiveNames) {
                            return;
                        }
                        lastProjectVersion = hostProjectVersion;
                    }
                }
                var typeRootsVersion = host.getTypeRootsVersion ? host.getTypeRootsVersion() : 0;
                if (lastTypesRootVersion !== typeRootsVersion) {
                    log("TypeRoots version has changed; provide new program");
                    program = undefined; // TODO: GH#18217
                    lastTypesRootVersion = typeRootsVersion;
                }
                // Get a fresh cache of the host information
                var hostCache = new HostCache(host, getCanonicalFileName);
                var rootFileNames = hostCache.getRootFileNames();
                var hasInvalidatedResolution = host.hasInvalidatedResolution || ts.returnFalse;
                var projectReferences = hostCache.getProjectReferences();
                // If the program is already up-to-date, we can reuse it
                if (ts.isProgramUptoDate(program, rootFileNames, hostCache.compilationSettings(), function (path) { return hostCache.getVersion(path); }, fileExists, hasInvalidatedResolution, !!host.hasChangedAutomaticTypeDirectiveNames, projectReferences)) {
                    return;
                }
                // IMPORTANT - It is critical from this moment onward that we do not check
                // cancellation tokens.  We are about to mutate source files from a previous program
                // instance.  If we cancel midway through, we may end up in an inconsistent state where
                // the program points to old source files that have been invalidated because of
                // incremental parsing.
                var newSettings = hostCache.compilationSettings();
                // Now create a new compiler
                var compilerHost = {
                    getSourceFile: getOrCreateSourceFile,
                    getSourceFileByPath: getOrCreateSourceFileByPath,
                    getCancellationToken: function () { return cancellationToken; },
                    getCanonicalFileName: getCanonicalFileName,
                    useCaseSensitiveFileNames: function () { return useCaseSensitiveFileNames; },
                    getNewLine: function () { return ts.getNewLineCharacter(newSettings, function () { return ts.getNewLineOrDefaultFromHost(host); }); },
                    getDefaultLibFileName: function (options) { return host.getDefaultLibFileName(options); },
                    writeFile: ts.noop,
                    getCurrentDirectory: function () { return currentDirectory; },
                    fileExists: fileExists,
                    readFile: readFile,
                    realpath: host.realpath && (function (path) { return host.realpath(path); }),
                    directoryExists: function (directoryName) {
                        return ts.directoryProbablyExists(directoryName, host);
                    },
                    getDirectories: function (path) {
                        return host.getDirectories ? host.getDirectories(path) : [];
                    },
                    readDirectory: function (path, extensions, exclude, include, depth) {
                        ts.Debug.assertDefined(host.readDirectory, "'LanguageServiceHost.readDirectory' must be implemented to correctly process 'projectReferences'");
                        return host.readDirectory(path, extensions, exclude, include, depth);
                    },
                    onReleaseOldSourceFile: onReleaseOldSourceFile,
                    hasInvalidatedResolution: hasInvalidatedResolution,
                    hasChangedAutomaticTypeDirectiveNames: host.hasChangedAutomaticTypeDirectiveNames
                };
                if (host.trace) {
                    compilerHost.trace = function (message) { return host.trace(message); };
                }
                if (host.resolveModuleNames) {
                    compilerHost.resolveModuleNames = function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i] = arguments[_i];
                        }
                        return host.resolveModuleNames.apply(host, args);
                    };
                }
                if (host.resolveTypeReferenceDirectives) {
                    compilerHost.resolveTypeReferenceDirectives = function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i] = arguments[_i];
                        }
                        return host.resolveTypeReferenceDirectives.apply(host, args);
                    };
                }
                if (host.setResolvedProjectReferenceCallbacks) {
                    compilerHost.setResolvedProjectReferenceCallbacks = function (callbacks) { return host.setResolvedProjectReferenceCallbacks(callbacks); };
                }
                if (host.useSourceOfProjectReferenceRedirect) {
                    compilerHost.useSourceOfProjectReferenceRedirect = function () { return host.useSourceOfProjectReferenceRedirect(); };
                }
                var documentRegistryBucketKey = documentRegistry.getKeyForCompilationSettings(newSettings);
                var options = {
                    rootNames: rootFileNames,
                    options: newSettings,
                    host: compilerHost,
                    oldProgram: program,
                    projectReferences: projectReferences
                };
                program = ts.createProgram(options);
                // hostCache is captured in the closure for 'getOrCreateSourceFile' but it should not be used past this point.
                // It needs to be cleared to allow all collected snapshots to be released
                hostCache = undefined;
                // We reset this cache on structure invalidation so we don't hold on to outdated files for long; however we can't use the `compilerHost` above,
                // Because it only functions until `hostCache` is cleared, while we'll potentially need the functionality to lazily read sourcemap files during
                // the course of whatever called `synchronizeHostData`
                sourceMapper.clearCache();
                // Make sure all the nodes in the program are both bound, and have their parent
                // pointers set property.
                program.getTypeChecker();
                return;
                function fileExists(fileName) {
                    var path = ts.toPath(fileName, currentDirectory, getCanonicalFileName);
                    var entry = hostCache && hostCache.getEntryByPath(path);
                    return entry ?
                        !ts.isString(entry) :
                        (!!host.fileExists && host.fileExists(fileName));
                }
                function readFile(fileName) {
                    // stub missing host functionality
                    var path = ts.toPath(fileName, currentDirectory, getCanonicalFileName);
                    var entry = hostCache && hostCache.getEntryByPath(path);
                    if (entry) {
                        return ts.isString(entry) ? undefined : ts.getSnapshotText(entry.scriptSnapshot);
                    }
                    return host.readFile && host.readFile(fileName);
                }
                // Release any files we have acquired in the old program but are
                // not part of the new program.
                function onReleaseOldSourceFile(oldSourceFile, oldOptions) {
                    var oldSettingsKey = documentRegistry.getKeyForCompilationSettings(oldOptions);
                    documentRegistry.releaseDocumentWithKey(oldSourceFile.resolvedPath, oldSettingsKey);
                }
                function getOrCreateSourceFile(fileName, languageVersion, onError, shouldCreateNewSourceFile) {
                    return getOrCreateSourceFileByPath(fileName, ts.toPath(fileName, currentDirectory, getCanonicalFileName), languageVersion, onError, shouldCreateNewSourceFile);
                }
                function getOrCreateSourceFileByPath(fileName, path, _languageVersion, _onError, shouldCreateNewSourceFile) {
                    ts.Debug.assert(hostCache !== undefined, "getOrCreateSourceFileByPath called after typical CompilerHost lifetime, check the callstack something with a reference to an old host.");
                    // The program is asking for this file, check first if the host can locate it.
                    // If the host can not locate the file, then it does not exist. return undefined
                    // to the program to allow reporting of errors for missing files.
                    var hostFileInformation = hostCache && hostCache.getOrCreateEntryByPath(fileName, path);
                    if (!hostFileInformation) {
                        return undefined;
                    }
                    // Check if the language version has changed since we last created a program; if they are the same,
                    // it is safe to reuse the sourceFiles; if not, then the shape of the AST can change, and the oldSourceFile
                    // can not be reused. we have to dump all syntax trees and create new ones.
                    if (!shouldCreateNewSourceFile) {
                        // Check if the old program had this file already
                        var oldSourceFile = program && program.getSourceFileByPath(path);
                        if (oldSourceFile) {
                            // We already had a source file for this file name.  Go to the registry to
                            // ensure that we get the right up to date version of it.  We need this to
                            // address the following race-condition.  Specifically, say we have the following:
                            //
                            //      LS1
                            //          \
                            //           DocumentRegistry
                            //          /
                            //      LS2
                            //
                            // Each LS has a reference to file 'foo.ts' at version 1.  LS2 then updates
                            // it's version of 'foo.ts' to version 2.  This will cause LS2 and the
                            // DocumentRegistry to have version 2 of the document.  HOwever, LS1 will
                            // have version 1.  And *importantly* this source file will be *corrupt*.
                            // The act of creating version 2 of the file irrevocably damages the version
                            // 1 file.
                            //
                            // So, later when we call into LS1, we need to make sure that it doesn't use
                            // it's source file any more, and instead defers to DocumentRegistry to get
                            // either version 1, version 2 (or some other version) depending on what the
                            // host says should be used.
                            // We do not support the scenario where a host can modify a registered
                            // file's script kind, i.e. in one project some file is treated as ".ts"
                            // and in another as ".js"
                            ts.Debug.assertEqual(hostFileInformation.scriptKind, oldSourceFile.scriptKind, "Registered script kind should match new script kind.", path);
                            return documentRegistry.updateDocumentWithKey(fileName, path, newSettings, documentRegistryBucketKey, hostFileInformation.scriptSnapshot, hostFileInformation.version, hostFileInformation.scriptKind);
                        }
                        // We didn't already have the file.  Fall through and acquire it from the registry.
                    }
                    // Could not find this file in the old program, create a new SourceFile for it.
                    return documentRegistry.acquireDocumentWithKey(fileName, path, newSettings, documentRegistryBucketKey, hostFileInformation.scriptSnapshot, hostFileInformation.version, hostFileInformation.scriptKind);
                }
            }
            // TODO: GH#18217 frequently asserted as defined
            function getProgram() {
                if (syntaxOnly) {
                    ts.Debug.assert(program === undefined);
                    return undefined;
                }
                synchronizeHostData();
                return program;
            }
            function cleanupSemanticCache() {
                program = undefined; // TODO: GH#18217
            }
            function dispose() {
                if (program) {
                    ts.forEach(program.getSourceFiles(), function (f) {
                        return documentRegistry.releaseDocument(f.fileName, program.getCompilerOptions());
                    });
                    program = undefined; // TODO: GH#18217
                }
                host = undefined;
            }
            /// Diagnostics
            function getSyntacticDiagnostics(fileName) {
                synchronizeHostData();
                return program.getSyntacticDiagnostics(getValidSourceFile(fileName), cancellationToken).slice();
            }
            /**
             * getSemanticDiagnostics return array of Diagnostics. If '-d' is not enabled, only report semantic errors
             * If '-d' enabled, report both semantic and emitter errors
             */
            function getSemanticDiagnostics(fileName) {
                synchronizeHostData();
                var targetSourceFile = getValidSourceFile(fileName);
                // Only perform the action per file regardless of '-out' flag as LanguageServiceHost is expected to call this function per file.
                // Therefore only get diagnostics for given file.
                var semanticDiagnostics = program.getSemanticDiagnostics(targetSourceFile, cancellationToken);
                if (!ts.getEmitDeclarations(program.getCompilerOptions())) {
                    return semanticDiagnostics.slice();
                }
                // If '-d' is enabled, check for emitter error. One example of emitter error is export class implements non-export interface
                var declarationDiagnostics = program.getDeclarationDiagnostics(targetSourceFile, cancellationToken);
                return __spreadArrays(semanticDiagnostics, declarationDiagnostics);
            }
            function getSuggestionDiagnostics(fileName) {
                synchronizeHostData();
                return ts.computeSuggestionDiagnostics(getValidSourceFile(fileName), program, cancellationToken);
            }
            function getCompilerOptionsDiagnostics() {
                synchronizeHostData();
                return __spreadArrays(program.getOptionsDiagnostics(cancellationToken), program.getGlobalDiagnostics(cancellationToken));
            }
            function getCompletionsAtPosition(fileName, position, options) {
                if (options === void 0) {
                    options = ts.emptyOptions;
                }
                // Convert from deprecated options names to new names
                var fullPreferences = __assign(__assign({}, ts.identity(options)), { includeCompletionsForModuleExports: options.includeCompletionsForModuleExports || options.includeExternalModuleExports, includeCompletionsWithInsertText: options.includeCompletionsWithInsertText || options.includeInsertTextCompletions });
                synchronizeHostData();
                return ts.Completions.getCompletionsAtPosition(host, program, log, getValidSourceFile(fileName), position, fullPreferences, options.triggerCharacter);
            }
            function getCompletionEntryDetails(fileName, position, name, formattingOptions, source, preferences) {
                if (preferences === void 0) {
                    preferences = ts.emptyOptions;
                }
                synchronizeHostData();
                return ts.Completions.getCompletionEntryDetails(program, log, getValidSourceFile(fileName), position, { name: name, source: source }, host, (formattingOptions && ts.formatting.getFormatContext(formattingOptions)), // TODO: GH#18217
                preferences, cancellationToken);
            }
            function getCompletionEntrySymbol(fileName, position, name, source) {
                synchronizeHostData();
                return ts.Completions.getCompletionEntrySymbol(program, log, getValidSourceFile(fileName), position, { name: name, source: source }, host);
            }
            function getQuickInfoAtPosition(fileName, position) {
                synchronizeHostData();
                var sourceFile = getValidSourceFile(fileName);
                var node = ts.getTouchingPropertyName(sourceFile, position);
                if (node === sourceFile) {
                    // Avoid giving quickInfo for the sourceFile as a whole.
                    return undefined;
                }
                var typeChecker = program.getTypeChecker();
                var nodeForQuickInfo = getNodeForQuickInfo(node);
                var symbol = getSymbolAtLocationForQuickInfo(nodeForQuickInfo, typeChecker);
                if (!symbol || typeChecker.isUnknownSymbol(symbol)) {
                    var type_2 = shouldGetType(sourceFile, nodeForQuickInfo, position) ? typeChecker.getTypeAtLocation(nodeForQuickInfo) : undefined;
                    return type_2 && {
                        kind: "" /* unknown */,
                        kindModifiers: "" /* none */,
                        textSpan: ts.createTextSpanFromNode(nodeForQuickInfo, sourceFile),
                        displayParts: typeChecker.runWithCancellationToken(cancellationToken, function (typeChecker) { return ts.typeToDisplayParts(typeChecker, type_2, ts.getContainerNode(nodeForQuickInfo)); }),
                        documentation: type_2.symbol ? type_2.symbol.getDocumentationComment(typeChecker) : undefined,
                        tags: type_2.symbol ? type_2.symbol.getJsDocTags() : undefined
                    };
                }
                var _a = typeChecker.runWithCancellationToken(cancellationToken, function (typeChecker) {
                    return ts.SymbolDisplay.getSymbolDisplayPartsDocumentationAndSymbolKind(typeChecker, symbol, sourceFile, ts.getContainerNode(nodeForQuickInfo), nodeForQuickInfo);
                }), symbolKind = _a.symbolKind, displayParts = _a.displayParts, documentation = _a.documentation, tags = _a.tags;
                return {
                    kind: symbolKind,
                    kindModifiers: ts.SymbolDisplay.getSymbolModifiers(symbol),
                    textSpan: ts.createTextSpanFromNode(nodeForQuickInfo, sourceFile),
                    displayParts: displayParts,
                    documentation: documentation,
                    tags: tags,
                };
            }
            function getNodeForQuickInfo(node) {
                if (ts.isNewExpression(node.parent) && node.pos === node.parent.pos) {
                    return node.parent.expression;
                }
                return node;
            }
            function shouldGetType(sourceFile, node, position) {
                switch (node.kind) {
                    case 75 /* Identifier */:
                        return !ts.isLabelName(node) && !ts.isTagName(node);
                    case 194 /* PropertyAccessExpression */:
                    case 153 /* QualifiedName */:
                        // Don't return quickInfo if inside the comment in `a/**/.b`
                        return !ts.isInComment(sourceFile, position);
                    case 104 /* ThisKeyword */:
                    case 183 /* ThisType */:
                    case 102 /* SuperKeyword */:
                        return true;
                    default:
                        return false;
                }
            }
            /// Goto definition
            function getDefinitionAtPosition(fileName, position) {
                synchronizeHostData();
                return ts.GoToDefinition.getDefinitionAtPosition(program, getValidSourceFile(fileName), position);
            }
            function getDefinitionAndBoundSpan(fileName, position) {
                synchronizeHostData();
                return ts.GoToDefinition.getDefinitionAndBoundSpan(program, getValidSourceFile(fileName), position);
            }
            function getTypeDefinitionAtPosition(fileName, position) {
                synchronizeHostData();
                return ts.GoToDefinition.getTypeDefinitionAtPosition(program.getTypeChecker(), getValidSourceFile(fileName), position);
            }
            /// Goto implementation
            function getImplementationAtPosition(fileName, position) {
                synchronizeHostData();
                return ts.FindAllReferences.getImplementationsAtPosition(program, cancellationToken, program.getSourceFiles(), getValidSourceFile(fileName), position);
            }
            /// References and Occurrences
            function getOccurrencesAtPosition(fileName, position) {
                return ts.flatMap(getDocumentHighlights(fileName, position, [fileName]), function (entry) { return entry.highlightSpans.map(function (highlightSpan) { return (__assign(__assign({ fileName: entry.fileName, textSpan: highlightSpan.textSpan, isWriteAccess: highlightSpan.kind === "writtenReference" /* writtenReference */, isDefinition: false }, highlightSpan.isInString && { isInString: true }), highlightSpan.contextSpan && { contextSpan: highlightSpan.contextSpan })); }); });
            }
            function getDocumentHighlights(fileName, position, filesToSearch) {
                var normalizedFileName = ts.normalizePath(fileName);
                ts.Debug.assert(filesToSearch.some(function (f) { return ts.normalizePath(f) === normalizedFileName; }));
                synchronizeHostData();
                var sourceFilesToSearch = ts.mapDefined(filesToSearch, function (fileName) { return program.getSourceFile(fileName); });
                var sourceFile = getValidSourceFile(fileName);
                return ts.DocumentHighlights.getDocumentHighlights(program, cancellationToken, sourceFile, position, sourceFilesToSearch);
            }
            function findRenameLocations(fileName, position, findInStrings, findInComments, providePrefixAndSuffixTextForRename) {
                synchronizeHostData();
                var sourceFile = getValidSourceFile(fileName);
                var node = ts.getAdjustedRenameLocation(ts.getTouchingPropertyName(sourceFile, position));
                if (ts.isIdentifier(node) && (ts.isJsxOpeningElement(node.parent) || ts.isJsxClosingElement(node.parent)) && ts.isIntrinsicJsxName(node.escapedText)) {
                    var _a = node.parent.parent, openingElement = _a.openingElement, closingElement = _a.closingElement;
                    return [openingElement, closingElement].map(function (node) {
                        var textSpan = ts.createTextSpanFromNode(node.tagName, sourceFile);
                        return __assign({ fileName: sourceFile.fileName, textSpan: textSpan }, ts.FindAllReferences.toContextSpan(textSpan, sourceFile, node.parent));
                    });
                }
                else {
                    return getReferencesWorker(node, position, { findInStrings: findInStrings, findInComments: findInComments, providePrefixAndSuffixTextForRename: providePrefixAndSuffixTextForRename, use: 2 /* Rename */ }, function (entry, originalNode, checker) { return ts.FindAllReferences.toRenameLocation(entry, originalNode, checker, providePrefixAndSuffixTextForRename || false); });
                }
            }
            function getReferencesAtPosition(fileName, position) {
                synchronizeHostData();
                return getReferencesWorker(ts.getTouchingPropertyName(getValidSourceFile(fileName), position), position, { use: 1 /* References */ }, ts.FindAllReferences.toReferenceEntry);
            }
            function getReferencesWorker(node, position, options, cb) {
                synchronizeHostData();
                // Exclude default library when renaming as commonly user don't want to change that file.
                var sourceFiles = options && options.use === 2 /* Rename */
                    ? program.getSourceFiles().filter(function (sourceFile) { return !program.isSourceFileDefaultLibrary(sourceFile); })
                    : program.getSourceFiles();
                return ts.FindAllReferences.findReferenceOrRenameEntries(program, cancellationToken, sourceFiles, node, position, options, cb);
            }
            function findReferences(fileName, position) {
                synchronizeHostData();
                return ts.FindAllReferences.findReferencedSymbols(program, cancellationToken, program.getSourceFiles(), getValidSourceFile(fileName), position);
            }
            function getNavigateToItems(searchValue, maxResultCount, fileName, excludeDtsFiles) {
                if (excludeDtsFiles === void 0) {
                    excludeDtsFiles = false;
                }
                synchronizeHostData();
                var sourceFiles = fileName ? [getValidSourceFile(fileName)] : program.getSourceFiles();
                return ts.NavigateTo.getNavigateToItems(sourceFiles, program.getTypeChecker(), cancellationToken, searchValue, maxResultCount, excludeDtsFiles);
            }
            function getEmitOutput(fileName, emitOnlyDtsFiles, forceDtsEmit) {
                synchronizeHostData();
                var sourceFile = getValidSourceFile(fileName);
                var customTransformers = host.getCustomTransformers && host.getCustomTransformers();
                return ts.getFileEmitOutput(program, sourceFile, !!emitOnlyDtsFiles, cancellationToken, customTransformers, forceDtsEmit);
            }
            // Signature help
            /**
             * This is a semantic operation.
             */
            function getSignatureHelpItems(fileName, position, _a) {
                var triggerReason = (_a === void 0 ? ts.emptyOptions : _a).triggerReason;
                synchronizeHostData();
                var sourceFile = getValidSourceFile(fileName);
                return ts.SignatureHelp.getSignatureHelpItems(program, sourceFile, position, triggerReason, cancellationToken);
            }
            /// Syntactic features
            function getNonBoundSourceFile(fileName) {
                return syntaxTreeCache.getCurrentSourceFile(fileName);
            }
            function getNameOrDottedNameSpan(fileName, startPos, _endPos) {
                var sourceFile = syntaxTreeCache.getCurrentSourceFile(fileName);
                // Get node at the location
                var node = ts.getTouchingPropertyName(sourceFile, startPos);
                if (node === sourceFile) {
                    return undefined;
                }
                switch (node.kind) {
                    case 194 /* PropertyAccessExpression */:
                    case 153 /* QualifiedName */:
                    case 10 /* StringLiteral */:
                    case 91 /* FalseKeyword */:
                    case 106 /* TrueKeyword */:
                    case 100 /* NullKeyword */:
                    case 102 /* SuperKeyword */:
                    case 104 /* ThisKeyword */:
                    case 183 /* ThisType */:
                    case 75 /* Identifier */:
                        break;
                    // Cant create the text span
                    default:
                        return undefined;
                }
                var nodeForStartPos = node;
                while (true) {
                    if (ts.isRightSideOfPropertyAccess(nodeForStartPos) || ts.isRightSideOfQualifiedName(nodeForStartPos)) {
                        // If on the span is in right side of the the property or qualified name, return the span from the qualified name pos to end of this node
                        nodeForStartPos = nodeForStartPos.parent;
                    }
                    else if (ts.isNameOfModuleDeclaration(nodeForStartPos)) {
                        // If this is name of a module declarations, check if this is right side of dotted module name
                        // If parent of the module declaration which is parent of this node is module declaration and its body is the module declaration that this node is name of
                        // Then this name is name from dotted module
                        if (nodeForStartPos.parent.parent.kind === 249 /* ModuleDeclaration */ &&
                            nodeForStartPos.parent.parent.body === nodeForStartPos.parent) {
                            // Use parent module declarations name for start pos
                            nodeForStartPos = nodeForStartPos.parent.parent.name;
                        }
                        else {
                            // We have to use this name for start pos
                            break;
                        }
                    }
                    else {
                        // Is not a member expression so we have found the node for start pos
                        break;
                    }
                }
                return ts.createTextSpanFromBounds(nodeForStartPos.getStart(), node.getEnd());
            }
            function getBreakpointStatementAtPosition(fileName, position) {
                // doesn't use compiler - no need to synchronize with host
                var sourceFile = syntaxTreeCache.getCurrentSourceFile(fileName);
                return ts.BreakpointResolver.spanInSourceFileAtLocation(sourceFile, position);
            }
            function getNavigationBarItems(fileName) {
                return ts.NavigationBar.getNavigationBarItems(syntaxTreeCache.getCurrentSourceFile(fileName), cancellationToken);
            }
            function getNavigationTree(fileName) {
                return ts.NavigationBar.getNavigationTree(syntaxTreeCache.getCurrentSourceFile(fileName), cancellationToken);
            }
            function isTsOrTsxFile(fileName) {
                var kind = ts.getScriptKind(fileName, host);
                return kind === 3 /* TS */ || kind === 4 /* TSX */;
            }
            function getSemanticClassifications(fileName, span) {
                if (!isTsOrTsxFile(fileName)) {
                    // do not run semantic classification on non-ts-or-tsx files
                    return [];
                }
                synchronizeHostData();
                return ts.getSemanticClassifications(program.getTypeChecker(), cancellationToken, getValidSourceFile(fileName), program.getClassifiableNames(), span);
            }
            function getEncodedSemanticClassifications(fileName, span) {
                if (!isTsOrTsxFile(fileName)) {
                    // do not run semantic classification on non-ts-or-tsx files
                    return { spans: [], endOfLineState: 0 /* None */ };
                }
                synchronizeHostData();
                return ts.getEncodedSemanticClassifications(program.getTypeChecker(), cancellationToken, getValidSourceFile(fileName), program.getClassifiableNames(), span);
            }
            function getSyntacticClassifications(fileName, span) {
                // doesn't use compiler - no need to synchronize with host
                return ts.getSyntacticClassifications(cancellationToken, syntaxTreeCache.getCurrentSourceFile(fileName), span);
            }
            function getEncodedSyntacticClassifications(fileName, span) {
                // doesn't use compiler - no need to synchronize with host
                return ts.getEncodedSyntacticClassifications(cancellationToken, syntaxTreeCache.getCurrentSourceFile(fileName), span);
            }
            function getOutliningSpans(fileName) {
                // doesn't use compiler - no need to synchronize with host
                var sourceFile = syntaxTreeCache.getCurrentSourceFile(fileName);
                return ts.OutliningElementsCollector.collectElements(sourceFile, cancellationToken);
            }
            var braceMatching = ts.createMapFromTemplate((_a = {},
                _a[18 /* OpenBraceToken */] = 19 /* CloseBraceToken */,
                _a[20 /* OpenParenToken */] = 21 /* CloseParenToken */,
                _a[22 /* OpenBracketToken */] = 23 /* CloseBracketToken */,
                _a[31 /* GreaterThanToken */] = 29 /* LessThanToken */,
                _a));
            braceMatching.forEach(function (value, key) { return braceMatching.set(value.toString(), Number(key)); });
            function getBraceMatchingAtPosition(fileName, position) {
                var sourceFile = syntaxTreeCache.getCurrentSourceFile(fileName);
                var token = ts.getTouchingToken(sourceFile, position);
                var matchKind = token.getStart(sourceFile) === position ? braceMatching.get(token.kind.toString()) : undefined;
                var match = matchKind && ts.findChildOfKind(token.parent, matchKind, sourceFile);
                // We want to order the braces when we return the result.
                return match ? [ts.createTextSpanFromNode(token, sourceFile), ts.createTextSpanFromNode(match, sourceFile)].sort(function (a, b) { return a.start - b.start; }) : ts.emptyArray;
            }
            function getIndentationAtPosition(fileName, position, editorOptions) {
                var start = ts.timestamp();
                var settings = toEditorSettings(editorOptions);
                var sourceFile = syntaxTreeCache.getCurrentSourceFile(fileName);
                log("getIndentationAtPosition: getCurrentSourceFile: " + (ts.timestamp() - start));
                start = ts.timestamp();
                var result = ts.formatting.SmartIndenter.getIndentation(position, sourceFile, settings);
                log("getIndentationAtPosition: computeIndentation  : " + (ts.timestamp() - start));
                return result;
            }
            function getFormattingEditsForRange(fileName, start, end, options) {
                var sourceFile = syntaxTreeCache.getCurrentSourceFile(fileName);
                return ts.formatting.formatSelection(start, end, sourceFile, ts.formatting.getFormatContext(toEditorSettings(options)));
            }
            function getFormattingEditsForDocument(fileName, options) {
                return ts.formatting.formatDocument(syntaxTreeCache.getCurrentSourceFile(fileName), ts.formatting.getFormatContext(toEditorSettings(options)));
            }
            function getFormattingEditsAfterKeystroke(fileName, position, key, options) {
                var sourceFile = syntaxTreeCache.getCurrentSourceFile(fileName);
                var formatContext = ts.formatting.getFormatContext(toEditorSettings(options));
                if (!ts.isInComment(sourceFile, position)) {
                    switch (key) {
                        case "{":
                            return ts.formatting.formatOnOpeningCurly(position, sourceFile, formatContext);
                        case "}":
                            return ts.formatting.formatOnClosingCurly(position, sourceFile, formatContext);
                        case ";":
                            return ts.formatting.formatOnSemicolon(position, sourceFile, formatContext);
                        case "\n":
                            return ts.formatting.formatOnEnter(position, sourceFile, formatContext);
                    }
                }
                return [];
            }
            function getCodeFixesAtPosition(fileName, start, end, errorCodes, formatOptions, preferences) {
                if (preferences === void 0) {
                    preferences = ts.emptyOptions;
                }
                synchronizeHostData();
                var sourceFile = getValidSourceFile(fileName);
                var span = ts.createTextSpanFromBounds(start, end);
                var formatContext = ts.formatting.getFormatContext(formatOptions);
                return ts.flatMap(ts.deduplicate(errorCodes, ts.equateValues, ts.compareValues), function (errorCode) {
                    cancellationToken.throwIfCancellationRequested();
                    return ts.codefix.getFixes({ errorCode: errorCode, sourceFile: sourceFile, span: span, program: program, host: host, cancellationToken: cancellationToken, formatContext: formatContext, preferences: preferences });
                });
            }
            function getCombinedCodeFix(scope, fixId, formatOptions, preferences) {
                if (preferences === void 0) {
                    preferences = ts.emptyOptions;
                }
                synchronizeHostData();
                ts.Debug.assert(scope.type === "file");
                var sourceFile = getValidSourceFile(scope.fileName);
                var formatContext = ts.formatting.getFormatContext(formatOptions);
                return ts.codefix.getAllFixes({ fixId: fixId, sourceFile: sourceFile, program: program, host: host, cancellationToken: cancellationToken, formatContext: formatContext, preferences: preferences });
            }
            function organizeImports(scope, formatOptions, preferences) {
                if (preferences === void 0) {
                    preferences = ts.emptyOptions;
                }
                synchronizeHostData();
                ts.Debug.assert(scope.type === "file");
                var sourceFile = getValidSourceFile(scope.fileName);
                var formatContext = ts.formatting.getFormatContext(formatOptions);
                return ts.OrganizeImports.organizeImports(sourceFile, formatContext, host, program, preferences);
            }
            function getEditsForFileRename(oldFilePath, newFilePath, formatOptions, preferences) {
                if (preferences === void 0) {
                    preferences = ts.emptyOptions;
                }
                return ts.getEditsForFileRename(getProgram(), oldFilePath, newFilePath, host, ts.formatting.getFormatContext(formatOptions), preferences, sourceMapper);
            }
            function applyCodeActionCommand(fileName, actionOrFormatSettingsOrUndefined) {
                var action = typeof fileName === "string" ? actionOrFormatSettingsOrUndefined : fileName;
                return ts.isArray(action) ? Promise.all(action.map(function (a) { return applySingleCodeActionCommand(a); })) : applySingleCodeActionCommand(action);
            }
            function applySingleCodeActionCommand(action) {
                var getPath = function (path) { return ts.toPath(path, currentDirectory, getCanonicalFileName); };
                ts.Debug.assertEqual(action.type, "install package");
                return host.installPackage
                    ? host.installPackage({ fileName: getPath(action.file), packageName: action.packageName })
                    : Promise.reject("Host does not implement `installPackage`");
            }
            function getDocCommentTemplateAtPosition(fileName, position) {
                return ts.JsDoc.getDocCommentTemplateAtPosition(ts.getNewLineOrDefaultFromHost(host), syntaxTreeCache.getCurrentSourceFile(fileName), position);
            }
            function isValidBraceCompletionAtPosition(fileName, position, openingBrace) {
                // '<' is currently not supported, figuring out if we're in a Generic Type vs. a comparison is too
                // expensive to do during typing scenarios
                // i.e. whether we're dealing with:
                //      var x = new foo<| ( with class foo<T>{} )
                // or
                //      var y = 3 <|
                if (openingBrace === 60 /* lessThan */) {
                    return false;
                }
                var sourceFile = syntaxTreeCache.getCurrentSourceFile(fileName);
                // Check if in a context where we don't want to perform any insertion
                if (ts.isInString(sourceFile, position)) {
                    return false;
                }
                if (ts.isInsideJsxElementOrAttribute(sourceFile, position)) {
                    return openingBrace === 123 /* openBrace */;
                }
                if (ts.isInTemplateString(sourceFile, position)) {
                    return false;
                }
                switch (openingBrace) {
                    case 39 /* singleQuote */:
                    case 34 /* doubleQuote */:
                    case 96 /* backtick */:
                        return !ts.isInComment(sourceFile, position);
                }
                return true;
            }
            function getJsxClosingTagAtPosition(fileName, position) {
                var sourceFile = syntaxTreeCache.getCurrentSourceFile(fileName);
                var token = ts.findPrecedingToken(position, sourceFile);
                if (!token)
                    return undefined;
                var element = token.kind === 31 /* GreaterThanToken */ && ts.isJsxOpeningElement(token.parent) ? token.parent.parent
                    : ts.isJsxText(token) ? token.parent : undefined;
                if (element && isUnclosedTag(element)) {
                    return { newText: "</" + element.openingElement.tagName.getText(sourceFile) + ">" };
                }
            }
            function isUnclosedTag(_a) {
                var openingElement = _a.openingElement, closingElement = _a.closingElement, parent = _a.parent;
                return !ts.tagNamesAreEquivalent(openingElement.tagName, closingElement.tagName) ||
                    ts.isJsxElement(parent) && ts.tagNamesAreEquivalent(openingElement.tagName, parent.openingElement.tagName) && isUnclosedTag(parent);
            }
            function getSpanOfEnclosingComment(fileName, position, onlyMultiLine) {
                var sourceFile = syntaxTreeCache.getCurrentSourceFile(fileName);
                var range = ts.formatting.getRangeOfEnclosingComment(sourceFile, position);
                return range && (!onlyMultiLine || range.kind === 3 /* MultiLineCommentTrivia */) ? ts.createTextSpanFromRange(range) : undefined;
            }
            function getTodoComments(fileName, descriptors) {
                // Note: while getting todo comments seems like a syntactic operation, we actually
                // treat it as a semantic operation here.  This is because we expect our host to call
                // this on every single file.  If we treat this syntactically, then that will cause
                // us to populate and throw away the tree in our syntax tree cache for each file.  By
                // treating this as a semantic operation, we can access any tree without throwing
                // anything away.
                synchronizeHostData();
                var sourceFile = getValidSourceFile(fileName);
                cancellationToken.throwIfCancellationRequested();
                var fileContents = sourceFile.text;
                var result = [];
                // Exclude node_modules files as we don't want to show the todos of external libraries.
                if (descriptors.length > 0 && !isNodeModulesFile(sourceFile.fileName)) {
                    var regExp = getTodoCommentsRegExp();
                    var matchArray = void 0;
                    while (matchArray = regExp.exec(fileContents)) {
                        cancellationToken.throwIfCancellationRequested();
                        // If we got a match, here is what the match array will look like.  Say the source text is:
                        //
                        //      "    // hack   1"
                        //
                        // The result array with the regexp:    will be:
                        //
                        //      ["// hack   1", "// ", "hack   1", undefined, "hack"]
                        //
                        // Here are the relevant capture groups:
                        //  0) The full match for the entire regexp.
                        //  1) The preamble to the message portion.
                        //  2) The message portion.
                        //  3...N) The descriptor that was matched - by index.  'undefined' for each
                        //         descriptor that didn't match.  an actual value if it did match.
                        //
                        //  i.e. 'undefined' in position 3 above means TODO(jason) didn't match.
                        //       "hack"      in position 4 means HACK did match.
                        var firstDescriptorCaptureIndex = 3;
                        ts.Debug.assert(matchArray.length === descriptors.length + firstDescriptorCaptureIndex);
                        var preamble = matchArray[1];
                        var matchPosition = matchArray.index + preamble.length;
                        // OK, we have found a match in the file.  This is only an acceptable match if
                        // it is contained within a comment.
                        if (!ts.isInComment(sourceFile, matchPosition)) {
                            continue;
                        }
                        var descriptor = void 0;
                        for (var i = 0; i < descriptors.length; i++) {
                            if (matchArray[i + firstDescriptorCaptureIndex]) {
                                descriptor = descriptors[i];
                            }
                        }
                        if (descriptor === undefined)
                            return ts.Debug.fail();
                        // We don't want to match something like 'TODOBY', so we make sure a non
                        // letter/digit follows the match.
                        if (isLetterOrDigit(fileContents.charCodeAt(matchPosition + descriptor.text.length))) {
                            continue;
                        }
                        var message = matchArray[2];
                        result.push({ descriptor: descriptor, message: message, position: matchPosition });
                    }
                }
                return result;
                function escapeRegExp(str) {
                    return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
                }
                function getTodoCommentsRegExp() {
                    // NOTE: `?:` means 'non-capture group'.  It allows us to have groups without having to
                    // filter them out later in the final result array.
                    // TODO comments can appear in one of the following forms:
                    //
                    //  1)      // TODO     or  /////////// TODO
                    //
                    //  2)      /* TODO     or  /********** TODO
                    //
                    //  3)      /*
                    //           *   TODO
                    //           */
                    //
                    // The following three regexps are used to match the start of the text up to the TODO
                    // comment portion.
                    var singleLineCommentStart = /(?:\/\/+\s*)/.source;
                    var multiLineCommentStart = /(?:\/\*+\s*)/.source;
                    var anyNumberOfSpacesAndAsterisksAtStartOfLine = /(?:^(?:\s|\*)*)/.source;
                    // Match any of the above three TODO comment start regexps.
                    // Note that the outermost group *is* a capture group.  We want to capture the preamble
                    // so that we can determine the starting position of the TODO comment match.
                    var preamble = "(" + anyNumberOfSpacesAndAsterisksAtStartOfLine + "|" + singleLineCommentStart + "|" + multiLineCommentStart + ")";
                    // Takes the descriptors and forms a regexp that matches them as if they were literals.
                    // For example, if the descriptors are "TODO(jason)" and "HACK", then this will be:
                    //
                    //      (?:(TODO\(jason\))|(HACK))
                    //
                    // Note that the outermost group is *not* a capture group, but the innermost groups
                    // *are* capture groups.  By capturing the inner literals we can determine after
                    // matching which descriptor we are dealing with.
                    var literals = "(?:" + ts.map(descriptors, function (d) { return "(" + escapeRegExp(d.text) + ")"; }).join("|") + ")";
                    // After matching a descriptor literal, the following regexp matches the rest of the
                    // text up to the end of the line (or */).
                    var endOfLineOrEndOfComment = /(?:$|\*\/)/.source;
                    var messageRemainder = /(?:.*?)/.source;
                    // This is the portion of the match we'll return as part of the TODO comment result. We
                    // match the literal portion up to the end of the line or end of comment.
                    var messagePortion = "(" + literals + messageRemainder + ")";
                    var regExpString = preamble + messagePortion + endOfLineOrEndOfComment;
                    // The final regexp will look like this:
                    // /((?:\/\/+\s*)|(?:\/\*+\s*)|(?:^(?:\s|\*)*))((?:(TODO\(jason\))|(HACK))(?:.*?))(?:$|\*\/)/gim
                    // The flags of the regexp are important here.
                    //  'g' is so that we are doing a global search and can find matches several times
                    //  in the input.
                    //
                    //  'i' is for case insensitivity (We do this to match C# TODO comment code).
                    //
                    //  'm' is so we can find matches in a multi-line input.
                    return new RegExp(regExpString, "gim");
                }
                function isLetterOrDigit(char) {
                    return (char >= 97 /* a */ && char <= 122 /* z */) ||
                        (char >= 65 /* A */ && char <= 90 /* Z */) ||
                        (char >= 48 /* _0 */ && char <= 57 /* _9 */);
                }
                function isNodeModulesFile(path) {
                    return ts.stringContains(path, "/node_modules/");
                }
            }
            function getRenameInfo(fileName, position, options) {
                synchronizeHostData();
                return ts.Rename.getRenameInfo(program, getValidSourceFile(fileName), position, options);
            }
            function getRefactorContext(file, positionOrRange, preferences, formatOptions) {
                var _a = typeof positionOrRange === "number" ? [positionOrRange, undefined] : [positionOrRange.pos, positionOrRange.end], startPosition = _a[0], endPosition = _a[1];
                return {
                    file: file,
                    startPosition: startPosition,
                    endPosition: endPosition,
                    program: getProgram(),
                    host: host,
                    formatContext: ts.formatting.getFormatContext(formatOptions),
                    cancellationToken: cancellationToken,
                    preferences: preferences,
                };
            }
            function getSmartSelectionRange(fileName, position) {
                return ts.SmartSelectionRange.getSmartSelectionRange(position, syntaxTreeCache.getCurrentSourceFile(fileName));
            }
            function getApplicableRefactors(fileName, positionOrRange, preferences) {
                if (preferences === void 0) {
                    preferences = ts.emptyOptions;
                }
                synchronizeHostData();
                var file = getValidSourceFile(fileName);
                return ts.refactor.getApplicableRefactors(getRefactorContext(file, positionOrRange, preferences));
            }
            function getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName, preferences) {
                if (preferences === void 0) {
                    preferences = ts.emptyOptions;
                }
                synchronizeHostData();
                var file = getValidSourceFile(fileName);
                return ts.refactor.getEditsForRefactor(getRefactorContext(file, positionOrRange, preferences, formatOptions), refactorName, actionName);
            }
            function prepareCallHierarchy(fileName, position) {
                synchronizeHostData();
                var declarations = ts.CallHierarchy.resolveCallHierarchyDeclaration(program, ts.getTouchingPropertyName(getValidSourceFile(fileName), position));
                return declarations && ts.mapOneOrMany(declarations, function (declaration) { return ts.CallHierarchy.createCallHierarchyItem(program, declaration); });
            }
            function provideCallHierarchyIncomingCalls(fileName, position) {
                synchronizeHostData();
                var sourceFile = getValidSourceFile(fileName);
                var declaration = ts.firstOrOnly(ts.CallHierarchy.resolveCallHierarchyDeclaration(program, position === 0 ? sourceFile : ts.getTouchingPropertyName(sourceFile, position)));
                return declaration ? ts.CallHierarchy.getIncomingCalls(program, declaration, cancellationToken) : [];
            }
            function provideCallHierarchyOutgoingCalls(fileName, position) {
                synchronizeHostData();
                var sourceFile = getValidSourceFile(fileName);
                var declaration = ts.firstOrOnly(ts.CallHierarchy.resolveCallHierarchyDeclaration(program, position === 0 ? sourceFile : ts.getTouchingPropertyName(sourceFile, position)));
                return declaration ? ts.CallHierarchy.getOutgoingCalls(program, declaration) : [];
            }
            return {
                dispose: dispose,
                cleanupSemanticCache: cleanupSemanticCache,
                getSyntacticDiagnostics: getSyntacticDiagnostics,
                getSemanticDiagnostics: getSemanticDiagnostics,
                getSuggestionDiagnostics: getSuggestionDiagnostics,
                getCompilerOptionsDiagnostics: getCompilerOptionsDiagnostics,
                getSyntacticClassifications: getSyntacticClassifications,
                getSemanticClassifications: getSemanticClassifications,
                getEncodedSyntacticClassifications: getEncodedSyntacticClassifications,
                getEncodedSemanticClassifications: getEncodedSemanticClassifications,
                getCompletionsAtPosition: getCompletionsAtPosition,
                getCompletionEntryDetails: getCompletionEntryDetails,
                getCompletionEntrySymbol: getCompletionEntrySymbol,
                getSignatureHelpItems: getSignatureHelpItems,
                getQuickInfoAtPosition: getQuickInfoAtPosition,
                getDefinitionAtPosition: getDefinitionAtPosition,
                getDefinitionAndBoundSpan: getDefinitionAndBoundSpan,
                getImplementationAtPosition: getImplementationAtPosition,
                getTypeDefinitionAtPosition: getTypeDefinitionAtPosition,
                getReferencesAtPosition: getReferencesAtPosition,
                findReferences: findReferences,
                getOccurrencesAtPosition: getOccurrencesAtPosition,
                getDocumentHighlights: getDocumentHighlights,
                getNameOrDottedNameSpan: getNameOrDottedNameSpan,
                getBreakpointStatementAtPosition: getBreakpointStatementAtPosition,
                getNavigateToItems: getNavigateToItems,
                getRenameInfo: getRenameInfo,
                getSmartSelectionRange: getSmartSelectionRange,
                findRenameLocations: findRenameLocations,
                getNavigationBarItems: getNavigationBarItems,
                getNavigationTree: getNavigationTree,
                getOutliningSpans: getOutliningSpans,
                getTodoComments: getTodoComments,
                getBraceMatchingAtPosition: getBraceMatchingAtPosition,
                getIndentationAtPosition: getIndentationAtPosition,
                getFormattingEditsForRange: getFormattingEditsForRange,
                getFormattingEditsForDocument: getFormattingEditsForDocument,
                getFormattingEditsAfterKeystroke: getFormattingEditsAfterKeystroke,
                getDocCommentTemplateAtPosition: getDocCommentTemplateAtPosition,
                isValidBraceCompletionAtPosition: isValidBraceCompletionAtPosition,
                getJsxClosingTagAtPosition: getJsxClosingTagAtPosition,
                getSpanOfEnclosingComment: getSpanOfEnclosingComment,
                getCodeFixesAtPosition: getCodeFixesAtPosition,
                getCombinedCodeFix: getCombinedCodeFix,
                applyCodeActionCommand: applyCodeActionCommand,
                organizeImports: organizeImports,
                getEditsForFileRename: getEditsForFileRename,
                getEmitOutput: getEmitOutput,
                getNonBoundSourceFile: getNonBoundSourceFile,
                getProgram: getProgram,
                getApplicableRefactors: getApplicableRefactors,
                getEditsForRefactor: getEditsForRefactor,
                toLineColumnOffset: sourceMapper.toLineColumnOffset,
                getSourceMapper: function () { return sourceMapper; },
                prepareCallHierarchy: prepareCallHierarchy,
                provideCallHierarchyIncomingCalls: provideCallHierarchyIncomingCalls,
                provideCallHierarchyOutgoingCalls: provideCallHierarchyOutgoingCalls
            };
        }
        ts.createLanguageService = createLanguageService;
        /* @internal */
        /** Names in the name table are escaped, so an identifier `__foo` will have a name table entry `___foo`. */
        function getNameTable(sourceFile) {
            if (!sourceFile.nameTable) {
                initializeNameTable(sourceFile);
            }
            return sourceFile.nameTable; // TODO: GH#18217
        }
        ts.getNameTable = getNameTable;
        function initializeNameTable(sourceFile) {
            var nameTable = sourceFile.nameTable = ts.createUnderscoreEscapedMap();
            sourceFile.forEachChild(function walk(node) {
                if (ts.isIdentifier(node) && !ts.isTagName(node) && node.escapedText || ts.isStringOrNumericLiteralLike(node) && literalIsName(node)) {
                    var text = ts.getEscapedTextOfIdentifierOrLiteral(node);
                    nameTable.set(text, nameTable.get(text) === undefined ? node.pos : -1);
                }
                else if (ts.isPrivateIdentifier(node)) {
                    var text = node.escapedText;
                    nameTable.set(text, nameTable.get(text) === undefined ? node.pos : -1);
                }
                ts.forEachChild(node, walk);
                if (ts.hasJSDocNodes(node)) {
                    for (var _i = 0, _a = node.jsDoc; _i < _a.length; _i++) {
                        var jsDoc = _a[_i];
                        ts.forEachChild(jsDoc, walk);
                    }
                }
            });
        }
        /**
         * We want to store any numbers/strings if they were a name that could be
         * related to a declaration.  So, if we have 'import x = require("something")'
         * then we want 'something' to be in the name table.  Similarly, if we have
         * "a['propname']" then we want to store "propname" in the name table.
         */
        function literalIsName(node) {
            return ts.isDeclarationName(node) ||
                node.parent.kind === 265 /* ExternalModuleReference */ ||
                isArgumentOfElementAccessExpression(node) ||
                ts.isLiteralComputedPropertyDeclarationName(node);
        }
        /**
         * Returns the containing object literal property declaration given a possible name node, e.g. "a" in x = { "a": 1 }
         */
        /* @internal */
        function getContainingObjectLiteralElement(node) {
            var element = getContainingObjectLiteralElementWorker(node);
            return element && (ts.isObjectLiteralExpression(element.parent) || ts.isJsxAttributes(element.parent)) ? element : undefined;
        }
        ts.getContainingObjectLiteralElement = getContainingObjectLiteralElement;
        function getContainingObjectLiteralElementWorker(node) {
            switch (node.kind) {
                case 10 /* StringLiteral */:
                case 14 /* NoSubstitutionTemplateLiteral */:
                case 8 /* NumericLiteral */:
                    if (node.parent.kind === 154 /* ComputedPropertyName */) {
                        return ts.isObjectLiteralElement(node.parent.parent) ? node.parent.parent : undefined;
                    }
                // falls through
                case 75 /* Identifier */:
                    return ts.isObjectLiteralElement(node.parent) &&
                        (node.parent.parent.kind === 193 /* ObjectLiteralExpression */ || node.parent.parent.kind === 274 /* JsxAttributes */) &&
                        node.parent.name === node ? node.parent : undefined;
            }
            return undefined;
        }
        function getSymbolAtLocationForQuickInfo(node, checker) {
            var object = getContainingObjectLiteralElement(node);
            if (object) {
                var contextualType = checker.getContextualType(object.parent);
                var properties = contextualType && getPropertySymbolsFromContextualType(object, checker, contextualType, /*unionSymbolOk*/ false);
                if (properties && properties.length === 1) {
                    return ts.first(properties);
                }
            }
            return checker.getSymbolAtLocation(node);
        }
        /** Gets all symbols for one property. Does not get symbols for every property. */
        /* @internal */
        function getPropertySymbolsFromContextualType(node, checker, contextualType, unionSymbolOk) {
            var name = ts.getNameFromPropertyName(node.name);
            if (!name)
                return ts.emptyArray;
            if (!contextualType.isUnion()) {
                var symbol = contextualType.getProperty(name);
                return symbol ? [symbol] : ts.emptyArray;
            }
            var discriminatedPropertySymbols = ts.mapDefined(contextualType.types, function (t) { return ts.isObjectLiteralExpression(node.parent) && checker.isTypeInvalidDueToUnionDiscriminant(t, node.parent) ? undefined : t.getProperty(name); });
            if (unionSymbolOk && (discriminatedPropertySymbols.length === 0 || discriminatedPropertySymbols.length === contextualType.types.length)) {
                var symbol = contextualType.getProperty(name);
                if (symbol)
                    return [symbol];
            }
            if (discriminatedPropertySymbols.length === 0) {
                // Bad discriminant -- do again without discriminating
                return ts.mapDefined(contextualType.types, function (t) { return t.getProperty(name); });
            }
            return discriminatedPropertySymbols;
        }
        ts.getPropertySymbolsFromContextualType = getPropertySymbolsFromContextualType;
        function isArgumentOfElementAccessExpression(node) {
            return node &&
                node.parent &&
                node.parent.kind === 195 /* ElementAccessExpression */ &&
                node.parent.argumentExpression === node;
        }
        /**
         * Get the path of the default library files (lib.d.ts) as distributed with the typescript
         * node package.
         * The functionality is not supported if the ts module is consumed outside of a node module.
         */
        function getDefaultLibFilePath(options) {
            // Check __dirname is defined and that we are on a node.js system.
            if (typeof __dirname !== "undefined") {
                return __dirname + ts.directorySeparator + ts.getDefaultLibFileName(options);
            }
            throw new Error("getDefaultLibFilePath is only supported when consumed as a node module. ");
        }
        ts.getDefaultLibFilePath = getDefaultLibFilePath;
        ts.setObjectAllocator(getServicesObjectAllocator());
    })(ts || (ts = {}));
    /* @internal */
    var ts;
    (function (ts) {
        var BreakpointResolver;
        (function (BreakpointResolver) {
            /**
             * Get the breakpoint span in given sourceFile
             */
            function spanInSourceFileAtLocation(sourceFile, position) {
                // Cannot set breakpoint in dts file
                if (sourceFile.isDeclarationFile) {
                    return undefined;
                }
                var tokenAtLocation = ts.getTokenAtPosition(sourceFile, position);
                var lineOfPosition = sourceFile.getLineAndCharacterOfPosition(position).line;
                if (sourceFile.getLineAndCharacterOfPosition(tokenAtLocation.getStart(sourceFile)).line > lineOfPosition) {
                    // Get previous token if the token is returned starts on new line
                    // eg: let x =10; |--- cursor is here
                    //     let y = 10;
                    // token at position will return let keyword on second line as the token but we would like to use
                    // token on same line if trailing trivia (comments or white spaces on same line) part of the last token on that line
                    var preceding = ts.findPrecedingToken(tokenAtLocation.pos, sourceFile);
                    // It's a blank line
                    if (!preceding || sourceFile.getLineAndCharacterOfPosition(preceding.getEnd()).line !== lineOfPosition) {
                        return undefined;
                    }
                    tokenAtLocation = preceding;
                }
                // Cannot set breakpoint in ambient declarations
                if (tokenAtLocation.flags & 8388608 /* Ambient */) {
                    return undefined;
                }
                // Get the span in the node based on its syntax
                return spanInNode(tokenAtLocation);
                function textSpan(startNode, endNode) {
                    var start = startNode.decorators ?
                        ts.skipTrivia(sourceFile.text, startNode.decorators.end) :
                        startNode.getStart(sourceFile);
                    return ts.createTextSpanFromBounds(start, (endNode || startNode).getEnd());
                }
                function textSpanEndingAtNextToken(startNode, previousTokenToFindNextEndToken) {
                    return textSpan(startNode, ts.findNextToken(previousTokenToFindNextEndToken, previousTokenToFindNextEndToken.parent, sourceFile));
                }
                function spanInNodeIfStartsOnSameLine(node, otherwiseOnNode) {
                    if (node && lineOfPosition === sourceFile.getLineAndCharacterOfPosition(node.getStart(sourceFile)).line) {
                        return spanInNode(node);
                    }
                    return spanInNode(otherwiseOnNode);
                }
                function spanInNodeArray(nodeArray) {
                    return ts.createTextSpanFromBounds(ts.skipTrivia(sourceFile.text, nodeArray.pos), nodeArray.end);
                }
                function spanInPreviousNode(node) {
                    return spanInNode(ts.findPrecedingToken(node.pos, sourceFile));
                }
                function spanInNextNode(node) {
                    return spanInNode(ts.findNextToken(node, node.parent, sourceFile));
                }
                function spanInNode(node) {
                    if (node) {
                        var parent = node.parent;
                        switch (node.kind) {
                            case 225 /* VariableStatement */:
                                // Span on first variable declaration
                                return spanInVariableDeclaration(node.declarationList.declarations[0]);
                            case 242 /* VariableDeclaration */:
                            case 159 /* PropertyDeclaration */:
                            case 158 /* PropertySignature */:
                                return spanInVariableDeclaration(node);
                            case 156 /* Parameter */:
                                return spanInParameterDeclaration(node);
                            case 244 /* FunctionDeclaration */:
                            case 161 /* MethodDeclaration */:
                            case 160 /* MethodSignature */:
                            case 163 /* GetAccessor */:
                            case 164 /* SetAccessor */:
                            case 162 /* Constructor */:
                            case 201 /* FunctionExpression */:
                            case 202 /* ArrowFunction */:
                                return spanInFunctionDeclaration(node);
                            case 223 /* Block */:
                                if (ts.isFunctionBlock(node)) {
                                    return spanInFunctionBlock(node);
                                }
                            // falls through
                            case 250 /* ModuleBlock */:
                                return spanInBlock(node);
                            case 280 /* CatchClause */:
                                return spanInBlock(node.block);
                            case 226 /* ExpressionStatement */:
                                // span on the expression
                                return textSpan(node.expression);
                            case 235 /* ReturnStatement */:
                                // span on return keyword and expression if present
                                return textSpan(node.getChildAt(0), node.expression);
                            case 229 /* WhileStatement */:
                                // Span on while(...)
                                return textSpanEndingAtNextToken(node, node.expression);
                            case 228 /* DoStatement */:
                                // span in statement of the do statement
                                return spanInNode(node.statement);
                            case 241 /* DebuggerStatement */:
                                // span on debugger keyword
                                return textSpan(node.getChildAt(0));
                            case 227 /* IfStatement */:
                                // set on if(..) span
                                return textSpanEndingAtNextToken(node, node.expression);
                            case 238 /* LabeledStatement */:
                                // span in statement
                                return spanInNode(node.statement);
                            case 234 /* BreakStatement */:
                            case 233 /* ContinueStatement */:
                                // On break or continue keyword and label if present
                                return textSpan(node.getChildAt(0), node.label);
                            case 230 /* ForStatement */:
                                return spanInForStatement(node);
                            case 231 /* ForInStatement */:
                                // span of for (a in ...)
                                return textSpanEndingAtNextToken(node, node.expression);
                            case 232 /* ForOfStatement */:
                                // span in initializer
                                return spanInInitializerOfForLike(node);
                            case 237 /* SwitchStatement */:
                                // span on switch(...)
                                return textSpanEndingAtNextToken(node, node.expression);
                            case 277 /* CaseClause */:
                            case 278 /* DefaultClause */:
                                // span in first statement of the clause
                                return spanInNode(node.statements[0]);
                            case 240 /* TryStatement */:
                                // span in try block
                                return spanInBlock(node.tryBlock);
                            case 239 /* ThrowStatement */:
                                // span in throw ...
                                return textSpan(node, node.expression);
                            case 259 /* ExportAssignment */:
                                // span on export = id
                                return textSpan(node, node.expression);
                            case 253 /* ImportEqualsDeclaration */:
                                // import statement without including semicolon
                                return textSpan(node, node.moduleReference);
                            case 254 /* ImportDeclaration */:
                                // import statement without including semicolon
                                return textSpan(node, node.moduleSpecifier);
                            case 260 /* ExportDeclaration */:
                                // import statement without including semicolon
                                return textSpan(node, node.moduleSpecifier);
                            case 249 /* ModuleDeclaration */:
                                // span on complete module if it is instantiated
                                if (ts.getModuleInstanceState(node) !== 1 /* Instantiated */) {
                                    return undefined;
                                }
                            // falls through
                            case 245 /* ClassDeclaration */:
                            case 248 /* EnumDeclaration */:
                            case 284 /* EnumMember */:
                            case 191 /* BindingElement */:
                                // span on complete node
                                return textSpan(node);
                            case 236 /* WithStatement */:
                                // span in statement
                                return spanInNode(node.statement);
                            case 157 /* Decorator */:
                                return spanInNodeArray(parent.decorators);
                            case 189 /* ObjectBindingPattern */:
                            case 190 /* ArrayBindingPattern */:
                                return spanInBindingPattern(node);
                            // No breakpoint in interface, type alias
                            case 246 /* InterfaceDeclaration */:
                            case 247 /* TypeAliasDeclaration */:
                                return undefined;
                            // Tokens:
                            case 26 /* SemicolonToken */:
                            case 1 /* EndOfFileToken */:
                                return spanInNodeIfStartsOnSameLine(ts.findPrecedingToken(node.pos, sourceFile));
                            case 27 /* CommaToken */:
                                return spanInPreviousNode(node);
                            case 18 /* OpenBraceToken */:
                                return spanInOpenBraceToken(node);
                            case 19 /* CloseBraceToken */:
                                return spanInCloseBraceToken(node);
                            case 23 /* CloseBracketToken */:
                                return spanInCloseBracketToken(node);
                            case 20 /* OpenParenToken */:
                                return spanInOpenParenToken(node);
                            case 21 /* CloseParenToken */:
                                return spanInCloseParenToken(node);
                            case 58 /* ColonToken */:
                                return spanInColonToken(node);
                            case 31 /* GreaterThanToken */:
                            case 29 /* LessThanToken */:
                                return spanInGreaterThanOrLessThanToken(node);
                            // Keywords:
                            case 111 /* WhileKeyword */:
                                return spanInWhileKeyword(node);
                            case 87 /* ElseKeyword */:
                            case 79 /* CatchKeyword */:
                            case 92 /* FinallyKeyword */:
                                return spanInNextNode(node);
                            case 152 /* OfKeyword */:
                                return spanInOfKeyword(node);
                            default:
                                // Destructuring pattern in destructuring assignment
                                // [a, b, c] of
                                // [a, b, c] = expression
                                if (ts.isArrayLiteralOrObjectLiteralDestructuringPattern(node)) {
                                    return spanInArrayLiteralOrObjectLiteralDestructuringPattern(node);
                                }
                                // Set breakpoint on identifier element of destructuring pattern
                                // `a` or `...c` or `d: x` from
                                // `[a, b, ...c]` or `{ a, b }` or `{ d: x }` from destructuring pattern
                                if ((node.kind === 75 /* Identifier */ ||
                                    node.kind === 213 /* SpreadElement */ ||
                                    node.kind === 281 /* PropertyAssignment */ ||
                                    node.kind === 282 /* ShorthandPropertyAssignment */) &&
                                    ts.isArrayLiteralOrObjectLiteralDestructuringPattern(parent)) {
                                    return textSpan(node);
                                }
                                if (node.kind === 209 /* BinaryExpression */) {
                                    var _a = node, left = _a.left, operatorToken = _a.operatorToken;
                                    // Set breakpoint in destructuring pattern if its destructuring assignment
                                    // [a, b, c] or {a, b, c} of
                                    // [a, b, c] = expression or
                                    // {a, b, c} = expression
                                    if (ts.isArrayLiteralOrObjectLiteralDestructuringPattern(left)) {
                                        return spanInArrayLiteralOrObjectLiteralDestructuringPattern(left);
                                    }
                                    if (operatorToken.kind === 62 /* EqualsToken */ && ts.isArrayLiteralOrObjectLiteralDestructuringPattern(node.parent)) {
                                        // Set breakpoint on assignment expression element of destructuring pattern
                                        // a = expression of
                                        // [a = expression, b, c] = someExpression or
                                        // { a = expression, b, c } = someExpression
                                        return textSpan(node);
                                    }
                                    if (operatorToken.kind === 27 /* CommaToken */) {
                                        return spanInNode(left);
                                    }
                                }
                                if (ts.isExpressionNode(node)) {
                                    switch (parent.kind) {
                                        case 228 /* DoStatement */:
                                            // Set span as if on while keyword
                                            return spanInPreviousNode(node);
                                        case 157 /* Decorator */:
                                            // Set breakpoint on the decorator emit
                                            return spanInNode(node.parent);
                                        case 230 /* ForStatement */:
                                        case 232 /* ForOfStatement */:
                                            return textSpan(node);
                                        case 209 /* BinaryExpression */:
                                            if (node.parent.operatorToken.kind === 27 /* CommaToken */) {
                                                // If this is a comma expression, the breakpoint is possible in this expression
                                                return textSpan(node);
                                            }
                                            break;
                                        case 202 /* ArrowFunction */:
                                            if (node.parent.body === node) {
                                                // If this is body of arrow function, it is allowed to have the breakpoint
                                                return textSpan(node);
                                            }
                                            break;
                                    }
                                }
                                switch (node.parent.kind) {
                                    case 281 /* PropertyAssignment */:
                                        // If this is name of property assignment, set breakpoint in the initializer
                                        if (node.parent.name === node &&
                                            !ts.isArrayLiteralOrObjectLiteralDestructuringPattern(node.parent.parent)) {
                                            return spanInNode(node.parent.initializer);
                                        }
                                        break;
                                    case 199 /* TypeAssertionExpression */:
                                        // Breakpoint in type assertion goes to its operand
                                        if (node.parent.type === node) {
                                            return spanInNextNode(node.parent.type);
                                        }
                                        break;
                                    case 242 /* VariableDeclaration */:
                                    case 156 /* Parameter */: {
                                        // initializer of variable/parameter declaration go to previous node
                                        var _b = node.parent, initializer = _b.initializer, type = _b.type;
                                        if (initializer === node || type === node || ts.isAssignmentOperator(node.kind)) {
                                            return spanInPreviousNode(node);
                                        }
                                        break;
                                    }
                                    case 209 /* BinaryExpression */: {
                                        var left = node.parent.left;
                                        if (ts.isArrayLiteralOrObjectLiteralDestructuringPattern(left) && node !== left) {
                                            // If initializer of destructuring assignment move to previous token
                                            return spanInPreviousNode(node);
                                        }
                                        break;
                                    }
                                    default:
                                        // return type of function go to previous token
                                        if (ts.isFunctionLike(node.parent) && node.parent.type === node) {
                                            return spanInPreviousNode(node);
                                        }
                                }
                                // Default go to parent to set the breakpoint
                                return spanInNode(node.parent);
                        }
                    }
                    function textSpanFromVariableDeclaration(variableDeclaration) {
                        if (ts.isVariableDeclarationList(variableDeclaration.parent) && variableDeclaration.parent.declarations[0] === variableDeclaration) {
                            // First declaration - include let keyword
                            return textSpan(ts.findPrecedingToken(variableDeclaration.pos, sourceFile, variableDeclaration.parent), variableDeclaration);
                        }
                        else {
                            // Span only on this declaration
                            return textSpan(variableDeclaration);
                        }
                    }
                    function spanInVariableDeclaration(variableDeclaration) {
                        // If declaration of for in statement, just set the span in parent
                        if (variableDeclaration.parent.parent.kind === 231 /* ForInStatement */) {
                            return spanInNode(variableDeclaration.parent.parent);
                        }
                        var parent = variableDeclaration.parent;
                        // If this is a destructuring pattern, set breakpoint in binding pattern
                        if (ts.isBindingPattern(variableDeclaration.name)) {
                            return spanInBindingPattern(variableDeclaration.name);
                        }
                        // Breakpoint is possible in variableDeclaration only if there is initialization
                        // or its declaration from 'for of'
                        if (variableDeclaration.initializer ||
                            ts.hasModifier(variableDeclaration, 1 /* Export */) ||
                            parent.parent.kind === 232 /* ForOfStatement */) {
                            return textSpanFromVariableDeclaration(variableDeclaration);
                        }
                        if (ts.isVariableDeclarationList(variableDeclaration.parent) &&
                            variableDeclaration.parent.declarations[0] !== variableDeclaration) {
                            // If we cannot set breakpoint on this declaration, set it on previous one
                            // Because the variable declaration may be binding pattern and
                            // we would like to set breakpoint in last binding element if that's the case,
                            // use preceding token instead
                            return spanInNode(ts.findPrecedingToken(variableDeclaration.pos, sourceFile, variableDeclaration.parent));
                        }
                    }
                    function canHaveSpanInParameterDeclaration(parameter) {
                        // Breakpoint is possible on parameter only if it has initializer, is a rest parameter, or has public or private modifier
                        return !!parameter.initializer || parameter.dotDotDotToken !== undefined ||
                            ts.hasModifier(parameter, 4 /* Public */ | 8 /* Private */);
                    }
                    function spanInParameterDeclaration(parameter) {
                        if (ts.isBindingPattern(parameter.name)) {
                            // Set breakpoint in binding pattern
                            return spanInBindingPattern(parameter.name);
                        }
                        else if (canHaveSpanInParameterDeclaration(parameter)) {
                            return textSpan(parameter);
                        }
                        else {
                            var functionDeclaration = parameter.parent;
                            var indexOfParameter = functionDeclaration.parameters.indexOf(parameter);
                            ts.Debug.assert(indexOfParameter !== -1);
                            if (indexOfParameter !== 0) {
                                // Not a first parameter, go to previous parameter
                                return spanInParameterDeclaration(functionDeclaration.parameters[indexOfParameter - 1]);
                            }
                            else {
                                // Set breakpoint in the function declaration body
                                return spanInNode(functionDeclaration.body);
                            }
                        }
                    }
                    function canFunctionHaveSpanInWholeDeclaration(functionDeclaration) {
                        return ts.hasModifier(functionDeclaration, 1 /* Export */) ||
                            (functionDeclaration.parent.kind === 245 /* ClassDeclaration */ && functionDeclaration.kind !== 162 /* Constructor */);
                    }
                    function spanInFunctionDeclaration(functionDeclaration) {
                        // No breakpoints in the function signature
                        if (!functionDeclaration.body) {
                            return undefined;
                        }
                        if (canFunctionHaveSpanInWholeDeclaration(functionDeclaration)) {
                            // Set the span on whole function declaration
                            return textSpan(functionDeclaration);
                        }
                        // Set span in function body
                        return spanInNode(functionDeclaration.body);
                    }
                    function spanInFunctionBlock(block) {
                        var nodeForSpanInBlock = block.statements.length ? block.statements[0] : block.getLastToken();
                        if (canFunctionHaveSpanInWholeDeclaration(block.parent)) {
                            return spanInNodeIfStartsOnSameLine(block.parent, nodeForSpanInBlock);
                        }
                        return spanInNode(nodeForSpanInBlock);
                    }
                    function spanInBlock(block) {
                        switch (block.parent.kind) {
                            case 249 /* ModuleDeclaration */:
                                if (ts.getModuleInstanceState(block.parent) !== 1 /* Instantiated */) {
                                    return undefined;
                                }
                            // Set on parent if on same line otherwise on first statement
                            // falls through
                            case 229 /* WhileStatement */:
                            case 227 /* IfStatement */:
                            case 231 /* ForInStatement */:
                                return spanInNodeIfStartsOnSameLine(block.parent, block.statements[0]);
                            // Set span on previous token if it starts on same line otherwise on the first statement of the block
                            case 230 /* ForStatement */:
                            case 232 /* ForOfStatement */:
                                return spanInNodeIfStartsOnSameLine(ts.findPrecedingToken(block.pos, sourceFile, block.parent), block.statements[0]);
                        }
                        // Default action is to set on first statement
                        return spanInNode(block.statements[0]);
                    }
                    function spanInInitializerOfForLike(forLikeStatement) {
                        if (forLikeStatement.initializer.kind === 243 /* VariableDeclarationList */) {
                            // Declaration list - set breakpoint in first declaration
                            var variableDeclarationList = forLikeStatement.initializer;
                            if (variableDeclarationList.declarations.length > 0) {
                                return spanInNode(variableDeclarationList.declarations[0]);
                            }
                        }
                        else {
                            // Expression - set breakpoint in it
                            return spanInNode(forLikeStatement.initializer);
                        }
                    }
                    function spanInForStatement(forStatement) {
                        if (forStatement.initializer) {
                            return spanInInitializerOfForLike(forStatement);
                        }
                        if (forStatement.condition) {
                            return textSpan(forStatement.condition);
                        }
                        if (forStatement.incrementor) {
                            return textSpan(forStatement.incrementor);
                        }
                    }
                    function spanInBindingPattern(bindingPattern) {
                        // Set breakpoint in first binding element
                        var firstBindingElement = ts.forEach(bindingPattern.elements, function (element) { return element.kind !== 215 /* OmittedExpression */ ? element : undefined; });
                        if (firstBindingElement) {
                            return spanInNode(firstBindingElement);
                        }
                        // Empty binding pattern of binding element, set breakpoint on binding element
                        if (bindingPattern.parent.kind === 191 /* BindingElement */) {
                            return textSpan(bindingPattern.parent);
                        }
                        // Variable declaration is used as the span
                        return textSpanFromVariableDeclaration(bindingPattern.parent);
                    }
                    function spanInArrayLiteralOrObjectLiteralDestructuringPattern(node) {
                        ts.Debug.assert(node.kind !== 190 /* ArrayBindingPattern */ && node.kind !== 189 /* ObjectBindingPattern */);
                        var elements = node.kind === 192 /* ArrayLiteralExpression */ ? node.elements : node.properties;
                        var firstBindingElement = ts.forEach(elements, function (element) { return element.kind !== 215 /* OmittedExpression */ ? element : undefined; });
                        if (firstBindingElement) {
                            return spanInNode(firstBindingElement);
                        }
                        // Could be ArrayLiteral from destructuring assignment or
                        // just nested element in another destructuring assignment
                        // set breakpoint on assignment when parent is destructuring assignment
                        // Otherwise set breakpoint for this element
                        return textSpan(node.parent.kind === 209 /* BinaryExpression */ ? node.parent : node);
                    }
                    // Tokens:
                    function spanInOpenBraceToken(node) {
                        switch (node.parent.kind) {
                            case 248 /* EnumDeclaration */:
                                var enumDeclaration = node.parent;
                                return spanInNodeIfStartsOnSameLine(ts.findPrecedingToken(node.pos, sourceFile, node.parent), enumDeclaration.members.length ? enumDeclaration.members[0] : enumDeclaration.getLastToken(sourceFile));
                            case 245 /* ClassDeclaration */:
                                var classDeclaration = node.parent;
                                return spanInNodeIfStartsOnSameLine(ts.findPrecedingToken(node.pos, sourceFile, node.parent), classDeclaration.members.length ? classDeclaration.members[0] : classDeclaration.getLastToken(sourceFile));
                            case 251 /* CaseBlock */:
                                return spanInNodeIfStartsOnSameLine(node.parent.parent, node.parent.clauses[0]);
                        }
                        // Default to parent node
                        return spanInNode(node.parent);
                    }
                    function spanInCloseBraceToken(node) {
                        switch (node.parent.kind) {
                            case 250 /* ModuleBlock */:
                                // If this is not an instantiated module block, no bp span
                                if (ts.getModuleInstanceState(node.parent.parent) !== 1 /* Instantiated */) {
                                    return undefined;
                                }
                            // falls through
                            case 248 /* EnumDeclaration */:
                            case 245 /* ClassDeclaration */:
                                // Span on close brace token
                                return textSpan(node);
                            case 223 /* Block */:
                                if (ts.isFunctionBlock(node.parent)) {
                                    // Span on close brace token
                                    return textSpan(node);
                                }
                            // falls through
                            case 280 /* CatchClause */:
                                return spanInNode(ts.lastOrUndefined(node.parent.statements));
                            case 251 /* CaseBlock */:
                                // breakpoint in last statement of the last clause
                                var caseBlock = node.parent;
                                var lastClause = ts.lastOrUndefined(caseBlock.clauses);
                                if (lastClause) {
                                    return spanInNode(ts.lastOrUndefined(lastClause.statements));
                                }
                                return undefined;
                            case 189 /* ObjectBindingPattern */:
                                // Breakpoint in last binding element or binding pattern if it contains no elements
                                var bindingPattern = node.parent;
                                return spanInNode(ts.lastOrUndefined(bindingPattern.elements) || bindingPattern);
                            // Default to parent node
                            default:
                                if (ts.isArrayLiteralOrObjectLiteralDestructuringPattern(node.parent)) {
                                    // Breakpoint in last binding element or binding pattern if it contains no elements
                                    var objectLiteral = node.parent;
                                    return textSpan(ts.lastOrUndefined(objectLiteral.properties) || objectLiteral);
                                }
                                return spanInNode(node.parent);
                        }
                    }
                    function spanInCloseBracketToken(node) {
                        switch (node.parent.kind) {
                            case 190 /* ArrayBindingPattern */:
                                // Breakpoint in last binding element or binding pattern if it contains no elements
                                var bindingPattern = node.parent;
                                return textSpan(ts.lastOrUndefined(bindingPattern.elements) || bindingPattern);
                            default:
                                if (ts.isArrayLiteralOrObjectLiteralDestructuringPattern(node.parent)) {
                                    // Breakpoint in last binding element or binding pattern if it contains no elements
                                    var arrayLiteral = node.parent;
                                    return textSpan(ts.lastOrUndefined(arrayLiteral.elements) || arrayLiteral);
                                }
                                // Default to parent node
                                return spanInNode(node.parent);
                        }
                    }
                    function spanInOpenParenToken(node) {
                        if (node.parent.kind === 228 /* DoStatement */ || // Go to while keyword and do action instead
                            node.parent.kind === 196 /* CallExpression */ ||
                            node.parent.kind === 197 /* NewExpression */) {
                            return spanInPreviousNode(node);
                        }
                        if (node.parent.kind === 200 /* ParenthesizedExpression */) {
                            return spanInNextNode(node);
                        }
                        // Default to parent node
                        return spanInNode(node.parent);
                    }
                    function spanInCloseParenToken(node) {
                        // Is this close paren token of parameter list, set span in previous token
                        switch (node.parent.kind) {
                            case 201 /* FunctionExpression */:
                            case 244 /* FunctionDeclaration */:
                            case 202 /* ArrowFunction */:
                            case 161 /* MethodDeclaration */:
                            case 160 /* MethodSignature */:
                            case 163 /* GetAccessor */:
                            case 164 /* SetAccessor */:
                            case 162 /* Constructor */:
                            case 229 /* WhileStatement */:
                            case 228 /* DoStatement */:
                            case 230 /* ForStatement */:
                            case 232 /* ForOfStatement */:
                            case 196 /* CallExpression */:
                            case 197 /* NewExpression */:
                            case 200 /* ParenthesizedExpression */:
                                return spanInPreviousNode(node);
                            // Default to parent node
                            default:
                                return spanInNode(node.parent);
                        }
                    }
                    function spanInColonToken(node) {
                        // Is this : specifying return annotation of the function declaration
                        if (ts.isFunctionLike(node.parent) ||
                            node.parent.kind === 281 /* PropertyAssignment */ ||
                            node.parent.kind === 156 /* Parameter */) {
                            return spanInPreviousNode(node);
                        }
                        return spanInNode(node.parent);
                    }
                    function spanInGreaterThanOrLessThanToken(node) {
                        if (node.parent.kind === 199 /* TypeAssertionExpression */) {
                            return spanInNextNode(node);
                        }
                        return spanInNode(node.parent);
                    }
                    function spanInWhileKeyword(node) {
                        if (node.parent.kind === 228 /* DoStatement */) {
                            // Set span on while expression
                            return textSpanEndingAtNextToken(node, node.parent.expression);
                        }
                        // Default to parent node
                        return spanInNode(node.parent);
                    }
                    function spanInOfKeyword(node) {
                        if (node.parent.kind === 232 /* ForOfStatement */) {
                            // Set using next token
                            return spanInNextNode(node);
                        }
                        // Default to parent node
                        return spanInNode(node.parent);
                    }
                }
            }
            BreakpointResolver.spanInSourceFileAtLocation = spanInSourceFileAtLocation;
        })(BreakpointResolver = ts.BreakpointResolver || (ts.BreakpointResolver = {}));
    })(ts || (ts = {}));
    var ts;
    (function (ts) {
        /**
         * Transform one or more nodes using the supplied transformers.
         * @param source A single `Node` or an array of `Node` objects.
         * @param transformers An array of `TransformerFactory` callbacks used to process the transformation.
         * @param compilerOptions Optional compiler options.
         */
        function transform(source, transformers, compilerOptions) {
            var diagnostics = [];
            compilerOptions = ts.fixupCompilerOptions(compilerOptions, diagnostics); // TODO: GH#18217
            var nodes = ts.isArray(source) ? source : [source];
            var result = ts.transformNodes(/*resolver*/ undefined, /*emitHost*/ undefined, compilerOptions, nodes, transformers, /*allowDtsFiles*/ true);
            result.diagnostics = ts.concatenate(result.diagnostics, diagnostics);
            return result;
        }
        ts.transform = transform;
    })(ts || (ts = {}));
    //
    // Copyright (c) Microsoft Corporation.  All rights reserved.
    //
    // Licensed under the Apache License, Version 2.0 (the "License");
    // you may not use this file except in compliance with the License.
    // You may obtain a copy of the License at
    //   http://www.apache.org/licenses/LICENSE-2.0
    //
    // Unless required by applicable law or agreed to in writing, software
    // distributed under the License is distributed on an "AS IS" BASIS,
    // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    // See the License for the specific language governing permissions and
    // limitations under the License.
    //
    /* @internal */
    var debugObjectHost = (function () { return this; })(); // eslint-disable-line prefer-const
    // We need to use 'null' to interface with the managed side.
    /* eslint-disable no-in-operator */
    /* @internal */
    var ts;
    (function (ts) {
        function logInternalError(logger, err) {
            if (logger) {
                logger.log("*INTERNAL ERROR* - Exception in typescript services: " + err.message);
            }
        }
        var ScriptSnapshotShimAdapter = /** @class */ (function () {
            function ScriptSnapshotShimAdapter(scriptSnapshotShim) {
                this.scriptSnapshotShim = scriptSnapshotShim;
            }
            ScriptSnapshotShimAdapter.prototype.getText = function (start, end) {
                return this.scriptSnapshotShim.getText(start, end);
            };
            ScriptSnapshotShimAdapter.prototype.getLength = function () {
                return this.scriptSnapshotShim.getLength();
            };
            ScriptSnapshotShimAdapter.prototype.getChangeRange = function (oldSnapshot) {
                var oldSnapshotShim = oldSnapshot;
                var encoded = this.scriptSnapshotShim.getChangeRange(oldSnapshotShim.scriptSnapshotShim);
                /* eslint-disable no-null/no-null */
                if (encoded === null) {
                    return null; // TODO: GH#18217
                }
                /* eslint-enable no-null/no-null */
                var decoded = JSON.parse(encoded); // TODO: GH#18217
                return ts.createTextChangeRange(ts.createTextSpan(decoded.span.start, decoded.span.length), decoded.newLength);
            };
            ScriptSnapshotShimAdapter.prototype.dispose = function () {
                // if scriptSnapshotShim is a COM object then property check becomes method call with no arguments
                // 'in' does not have this effect
                if ("dispose" in this.scriptSnapshotShim) {
                    this.scriptSnapshotShim.dispose(); // TODO: GH#18217 Can we just use `if (this.scriptSnapshotShim.dispose)`?
                }
            };
            return ScriptSnapshotShimAdapter;
        }());
        var LanguageServiceShimHostAdapter = /** @class */ (function () {
            function LanguageServiceShimHostAdapter(shimHost) {
                var _this = this;
                this.shimHost = shimHost;
                this.loggingEnabled = false;
                this.tracingEnabled = false;
                // if shimHost is a COM object then property check will become method call with no arguments.
                // 'in' does not have this effect.
                if ("getModuleResolutionsForFile" in this.shimHost) {
                    this.resolveModuleNames = function (moduleNames, containingFile) {
                        var resolutionsInFile = JSON.parse(_this.shimHost.getModuleResolutionsForFile(containingFile)); // TODO: GH#18217
                        return ts.map(moduleNames, function (name) {
                            var result = ts.getProperty(resolutionsInFile, name);
                            return result ? { resolvedFileName: result, extension: ts.extensionFromPath(result), isExternalLibraryImport: false } : undefined;
                        });
                    };
                }
                if ("directoryExists" in this.shimHost) {
                    this.directoryExists = function (directoryName) { return _this.shimHost.directoryExists(directoryName); };
                }
                if ("getTypeReferenceDirectiveResolutionsForFile" in this.shimHost) {
                    this.resolveTypeReferenceDirectives = function (typeDirectiveNames, containingFile) {
                        var typeDirectivesForFile = JSON.parse(_this.shimHost.getTypeReferenceDirectiveResolutionsForFile(containingFile)); // TODO: GH#18217
                        return ts.map(typeDirectiveNames, function (name) { return ts.getProperty(typeDirectivesForFile, name); });
                    };
                }
            }
            LanguageServiceShimHostAdapter.prototype.log = function (s) {
                if (this.loggingEnabled) {
                    this.shimHost.log(s);
                }
            };
            LanguageServiceShimHostAdapter.prototype.trace = function (s) {
                if (this.tracingEnabled) {
                    this.shimHost.trace(s);
                }
            };
            LanguageServiceShimHostAdapter.prototype.error = function (s) {
                this.shimHost.error(s);
            };
            LanguageServiceShimHostAdapter.prototype.getProjectVersion = function () {
                if (!this.shimHost.getProjectVersion) {
                    // shimmed host does not support getProjectVersion
                    return undefined; // TODO: GH#18217
                }
                return this.shimHost.getProjectVersion();
            };
            LanguageServiceShimHostAdapter.prototype.getTypeRootsVersion = function () {
                if (!this.shimHost.getTypeRootsVersion) {
                    return 0;
                }
                return this.shimHost.getTypeRootsVersion();
            };
            LanguageServiceShimHostAdapter.prototype.useCaseSensitiveFileNames = function () {
                return this.shimHost.useCaseSensitiveFileNames ? this.shimHost.useCaseSensitiveFileNames() : false;
            };
            LanguageServiceShimHostAdapter.prototype.getCompilationSettings = function () {
                var settingsJson = this.shimHost.getCompilationSettings();
                // eslint-disable-next-line no-null/no-null
                if (settingsJson === null || settingsJson === "") {
                    throw Error("LanguageServiceShimHostAdapter.getCompilationSettings: empty compilationSettings");
                }
                var compilerOptions = JSON.parse(settingsJson);
                // permit language service to handle all files (filtering should be performed on the host side)
                compilerOptions.allowNonTsExtensions = true;
                return compilerOptions;
            };
            LanguageServiceShimHostAdapter.prototype.getScriptFileNames = function () {
                var encoded = this.shimHost.getScriptFileNames();
                return JSON.parse(encoded);
            };
            LanguageServiceShimHostAdapter.prototype.getScriptSnapshot = function (fileName) {
                var scriptSnapshot = this.shimHost.getScriptSnapshot(fileName);
                return scriptSnapshot && new ScriptSnapshotShimAdapter(scriptSnapshot);
            };
            LanguageServiceShimHostAdapter.prototype.getScriptKind = function (fileName) {
                if ("getScriptKind" in this.shimHost) {
                    return this.shimHost.getScriptKind(fileName); // TODO: GH#18217
                }
                else {
                    return 0 /* Unknown */;
                }
            };
            LanguageServiceShimHostAdapter.prototype.getScriptVersion = function (fileName) {
                return this.shimHost.getScriptVersion(fileName);
            };
            LanguageServiceShimHostAdapter.prototype.getLocalizedDiagnosticMessages = function () {
                /* eslint-disable no-null/no-null */
                var diagnosticMessagesJson = this.shimHost.getLocalizedDiagnosticMessages();
                if (diagnosticMessagesJson === null || diagnosticMessagesJson === "") {
                    return null;
                }
                try {
                    return JSON.parse(diagnosticMessagesJson);
                }
                catch (e) {
                    this.log(e.description || "diagnosticMessages.generated.json has invalid JSON format");
                    return null;
                }
                /* eslint-enable no-null/no-null */
            };
            LanguageServiceShimHostAdapter.prototype.getCancellationToken = function () {
                var hostCancellationToken = this.shimHost.getCancellationToken();
                return new ts.ThrottledCancellationToken(hostCancellationToken);
            };
            LanguageServiceShimHostAdapter.prototype.getCurrentDirectory = function () {
                return this.shimHost.getCurrentDirectory();
            };
            LanguageServiceShimHostAdapter.prototype.getDirectories = function (path) {
                return JSON.parse(this.shimHost.getDirectories(path));
            };
            LanguageServiceShimHostAdapter.prototype.getDefaultLibFileName = function (options) {
                return this.shimHost.getDefaultLibFileName(JSON.stringify(options));
            };
            LanguageServiceShimHostAdapter.prototype.readDirectory = function (path, extensions, exclude, include, depth) {
                var pattern = ts.getFileMatcherPatterns(path, exclude, include, this.shimHost.useCaseSensitiveFileNames(), this.shimHost.getCurrentDirectory()); // TODO: GH#18217
                return JSON.parse(this.shimHost.readDirectory(path, JSON.stringify(extensions), JSON.stringify(pattern.basePaths), pattern.excludePattern, pattern.includeFilePattern, pattern.includeDirectoryPattern, depth));
            };
            LanguageServiceShimHostAdapter.prototype.readFile = function (path, encoding) {
                return this.shimHost.readFile(path, encoding);
            };
            LanguageServiceShimHostAdapter.prototype.fileExists = function (path) {
                return this.shimHost.fileExists(path);
            };
            return LanguageServiceShimHostAdapter;
        }());
        ts.LanguageServiceShimHostAdapter = LanguageServiceShimHostAdapter;
        var CoreServicesShimHostAdapter = /** @class */ (function () {
            function CoreServicesShimHostAdapter(shimHost) {
                var _this = this;
                this.shimHost = shimHost;
                this.useCaseSensitiveFileNames = this.shimHost.useCaseSensitiveFileNames ? this.shimHost.useCaseSensitiveFileNames() : false;
                if ("directoryExists" in this.shimHost) {
                    this.directoryExists = function (directoryName) { return _this.shimHost.directoryExists(directoryName); };
                }
                else {
                    this.directoryExists = undefined; // TODO: GH#18217
                }
                if ("realpath" in this.shimHost) {
                    this.realpath = function (path) { return _this.shimHost.realpath(path); }; // TODO: GH#18217
                }
                else {
                    this.realpath = undefined; // TODO: GH#18217
                }
            }
            CoreServicesShimHostAdapter.prototype.readDirectory = function (rootDir, extensions, exclude, include, depth) {
                var pattern = ts.getFileMatcherPatterns(rootDir, exclude, include, this.shimHost.useCaseSensitiveFileNames(), this.shimHost.getCurrentDirectory()); // TODO: GH#18217
                return JSON.parse(this.shimHost.readDirectory(rootDir, JSON.stringify(extensions), JSON.stringify(pattern.basePaths), pattern.excludePattern, pattern.includeFilePattern, pattern.includeDirectoryPattern, depth));
            };
            CoreServicesShimHostAdapter.prototype.fileExists = function (fileName) {
                return this.shimHost.fileExists(fileName);
            };
            CoreServicesShimHostAdapter.prototype.readFile = function (fileName) {
                return this.shimHost.readFile(fileName);
            };
            CoreServicesShimHostAdapter.prototype.getDirectories = function (path) {
                return JSON.parse(this.shimHost.getDirectories(path));
            };
            return CoreServicesShimHostAdapter;
        }());
        ts.CoreServicesShimHostAdapter = CoreServicesShimHostAdapter;
        function simpleForwardCall(logger, actionDescription, action, logPerformance) {
            var start;
            if (logPerformance) {
                logger.log(actionDescription);
                start = ts.timestamp();
            }
            var result = action();
            if (logPerformance) {
                var end = ts.timestamp();
                logger.log(actionDescription + " completed in " + (end - start) + " msec");
                if (ts.isString(result)) {
                    var str = result;
                    if (str.length > 128) {
                        str = str.substring(0, 128) + "...";
                    }
                    logger.log("  result.length=" + str.length + ", result='" + JSON.stringify(str) + "'");
                }
            }
            return result;
        }
        function forwardJSONCall(logger, actionDescription, action, logPerformance) {
            return forwardCall(logger, actionDescription, /*returnJson*/ true, action, logPerformance);
        }
        function forwardCall(logger, actionDescription, returnJson, action, logPerformance) {
            try {
                var result = simpleForwardCall(logger, actionDescription, action, logPerformance);
                return returnJson ? JSON.stringify({ result: result }) : result;
            }
            catch (err) {
                if (err instanceof ts.OperationCanceledException) {
                    return JSON.stringify({ canceled: true });
                }
                logInternalError(logger, err);
                err.description = actionDescription;
                return JSON.stringify({ error: err });
            }
        }
        var ShimBase = /** @class */ (function () {
            function ShimBase(factory) {
                this.factory = factory;
                factory.registerShim(this);
            }
            ShimBase.prototype.dispose = function (_dummy) {
                this.factory.unregisterShim(this);
            };
            return ShimBase;
        }());
        function realizeDiagnostics(diagnostics, newLine) {
            return diagnostics.map(function (d) { return realizeDiagnostic(d, newLine); });
        }
        ts.realizeDiagnostics = realizeDiagnostics;
        function realizeDiagnostic(diagnostic, newLine) {
            return {
                message: ts.flattenDiagnosticMessageText(diagnostic.messageText, newLine),
                start: diagnostic.start,
                length: diagnostic.length,
                category: ts.diagnosticCategoryName(diagnostic),
                code: diagnostic.code,
                reportsUnnecessary: diagnostic.reportsUnnecessary,
            };
        }
        var LanguageServiceShimObject = /** @class */ (function (_super) {
            __extends(LanguageServiceShimObject, _super);
            function LanguageServiceShimObject(factory, host, languageService) {
                var _this = _super.call(this, factory) || this;
                _this.host = host;
                _this.languageService = languageService;
                _this.logPerformance = false;
                _this.logger = _this.host;
                return _this;
            }
            LanguageServiceShimObject.prototype.forwardJSONCall = function (actionDescription, action) {
                return forwardJSONCall(this.logger, actionDescription, action, this.logPerformance);
            };
            /// DISPOSE
            /**
             * Ensure (almost) deterministic release of internal Javascript resources when
             * some external native objects holds onto us (e.g. Com/Interop).
             */
            LanguageServiceShimObject.prototype.dispose = function (dummy) {
                this.logger.log("dispose()");
                this.languageService.dispose();
                this.languageService = null; // eslint-disable-line no-null/no-null
                // force a GC
                if (debugObjectHost && debugObjectHost.CollectGarbage) {
                    debugObjectHost.CollectGarbage();
                    this.logger.log("CollectGarbage()");
                }
                this.logger = null; // eslint-disable-line no-null/no-null
                _super.prototype.dispose.call(this, dummy);
            };
            /// REFRESH
            /**
             * Update the list of scripts known to the compiler
             */
            LanguageServiceShimObject.prototype.refresh = function (throwOnError) {
                this.forwardJSONCall("refresh(" + throwOnError + ")", function () { return null; } // eslint-disable-line no-null/no-null
                );
            };
            LanguageServiceShimObject.prototype.cleanupSemanticCache = function () {
                var _this = this;
                this.forwardJSONCall("cleanupSemanticCache()", function () {
                    _this.languageService.cleanupSemanticCache();
                    return null; // eslint-disable-line no-null/no-null
                });
            };
            LanguageServiceShimObject.prototype.realizeDiagnostics = function (diagnostics) {
                var newLine = ts.getNewLineOrDefaultFromHost(this.host);
                return realizeDiagnostics(diagnostics, newLine);
            };
            LanguageServiceShimObject.prototype.getSyntacticClassifications = function (fileName, start, length) {
                var _this = this;
                return this.forwardJSONCall("getSyntacticClassifications('" + fileName + "', " + start + ", " + length + ")", function () { return _this.languageService.getSyntacticClassifications(fileName, ts.createTextSpan(start, length)); });
            };
            LanguageServiceShimObject.prototype.getSemanticClassifications = function (fileName, start, length) {
                var _this = this;
                return this.forwardJSONCall("getSemanticClassifications('" + fileName + "', " + start + ", " + length + ")", function () { return _this.languageService.getSemanticClassifications(fileName, ts.createTextSpan(start, length)); });
            };
            LanguageServiceShimObject.prototype.getEncodedSyntacticClassifications = function (fileName, start, length) {
                var _this = this;
                return this.forwardJSONCall("getEncodedSyntacticClassifications('" + fileName + "', " + start + ", " + length + ")", 
                // directly serialize the spans out to a string.  This is much faster to decode
                // on the managed side versus a full JSON array.
                function () { return convertClassifications(_this.languageService.getEncodedSyntacticClassifications(fileName, ts.createTextSpan(start, length))); });
            };
            LanguageServiceShimObject.prototype.getEncodedSemanticClassifications = function (fileName, start, length) {
                var _this = this;
                return this.forwardJSONCall("getEncodedSemanticClassifications('" + fileName + "', " + start + ", " + length + ")", 
                // directly serialize the spans out to a string.  This is much faster to decode
                // on the managed side versus a full JSON array.
                function () { return convertClassifications(_this.languageService.getEncodedSemanticClassifications(fileName, ts.createTextSpan(start, length))); });
            };
            LanguageServiceShimObject.prototype.getSyntacticDiagnostics = function (fileName) {
                var _this = this;
                return this.forwardJSONCall("getSyntacticDiagnostics('" + fileName + "')", function () {
                    var diagnostics = _this.languageService.getSyntacticDiagnostics(fileName);
                    return _this.realizeDiagnostics(diagnostics);
                });
            };
            LanguageServiceShimObject.prototype.getSemanticDiagnostics = function (fileName) {
                var _this = this;
                return this.forwardJSONCall("getSemanticDiagnostics('" + fileName + "')", function () {
                    var diagnostics = _this.languageService.getSemanticDiagnostics(fileName);
                    return _this.realizeDiagnostics(diagnostics);
                });
            };
            LanguageServiceShimObject.prototype.getSuggestionDiagnostics = function (fileName) {
                var _this = this;
                return this.forwardJSONCall("getSuggestionDiagnostics('" + fileName + "')", function () { return _this.realizeDiagnostics(_this.languageService.getSuggestionDiagnostics(fileName)); });
            };
            LanguageServiceShimObject.prototype.getCompilerOptionsDiagnostics = function () {
                var _this = this;
                return this.forwardJSONCall("getCompilerOptionsDiagnostics()", function () {
                    var diagnostics = _this.languageService.getCompilerOptionsDiagnostics();
                    return _this.realizeDiagnostics(diagnostics);
                });
            };
            /// QUICKINFO
            /**
             * Computes a string representation of the type at the requested position
             * in the active file.
             */
            LanguageServiceShimObject.prototype.getQuickInfoAtPosition = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("getQuickInfoAtPosition('" + fileName + "', " + position + ")", function () { return _this.languageService.getQuickInfoAtPosition(fileName, position); });
            };
            /// NAMEORDOTTEDNAMESPAN
            /**
             * Computes span information of the name or dotted name at the requested position
             * in the active file.
             */
            LanguageServiceShimObject.prototype.getNameOrDottedNameSpan = function (fileName, startPos, endPos) {
                var _this = this;
                return this.forwardJSONCall("getNameOrDottedNameSpan('" + fileName + "', " + startPos + ", " + endPos + ")", function () { return _this.languageService.getNameOrDottedNameSpan(fileName, startPos, endPos); });
            };
            /**
             * STATEMENTSPAN
             * Computes span information of statement at the requested position in the active file.
             */
            LanguageServiceShimObject.prototype.getBreakpointStatementAtPosition = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("getBreakpointStatementAtPosition('" + fileName + "', " + position + ")", function () { return _this.languageService.getBreakpointStatementAtPosition(fileName, position); });
            };
            /// SIGNATUREHELP
            LanguageServiceShimObject.prototype.getSignatureHelpItems = function (fileName, position, options) {
                var _this = this;
                return this.forwardJSONCall("getSignatureHelpItems('" + fileName + "', " + position + ")", function () { return _this.languageService.getSignatureHelpItems(fileName, position, options); });
            };
            /// GOTO DEFINITION
            /**
             * Computes the definition location and file for the symbol
             * at the requested position.
             */
            LanguageServiceShimObject.prototype.getDefinitionAtPosition = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("getDefinitionAtPosition('" + fileName + "', " + position + ")", function () { return _this.languageService.getDefinitionAtPosition(fileName, position); });
            };
            /**
             * Computes the definition location and file for the symbol
             * at the requested position.
             */
            LanguageServiceShimObject.prototype.getDefinitionAndBoundSpan = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("getDefinitionAndBoundSpan('" + fileName + "', " + position + ")", function () { return _this.languageService.getDefinitionAndBoundSpan(fileName, position); });
            };
            /// GOTO Type
            /**
             * Computes the definition location of the type of the symbol
             * at the requested position.
             */
            LanguageServiceShimObject.prototype.getTypeDefinitionAtPosition = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("getTypeDefinitionAtPosition('" + fileName + "', " + position + ")", function () { return _this.languageService.getTypeDefinitionAtPosition(fileName, position); });
            };
            /// GOTO Implementation
            /**
             * Computes the implementation location of the symbol
             * at the requested position.
             */
            LanguageServiceShimObject.prototype.getImplementationAtPosition = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("getImplementationAtPosition('" + fileName + "', " + position + ")", function () { return _this.languageService.getImplementationAtPosition(fileName, position); });
            };
            LanguageServiceShimObject.prototype.getRenameInfo = function (fileName, position, options) {
                var _this = this;
                return this.forwardJSONCall("getRenameInfo('" + fileName + "', " + position + ")", function () { return _this.languageService.getRenameInfo(fileName, position, options); });
            };
            LanguageServiceShimObject.prototype.getSmartSelectionRange = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("getSmartSelectionRange('" + fileName + "', " + position + ")", function () { return _this.languageService.getSmartSelectionRange(fileName, position); });
            };
            LanguageServiceShimObject.prototype.findRenameLocations = function (fileName, position, findInStrings, findInComments, providePrefixAndSuffixTextForRename) {
                var _this = this;
                return this.forwardJSONCall("findRenameLocations('" + fileName + "', " + position + ", " + findInStrings + ", " + findInComments + ", " + providePrefixAndSuffixTextForRename + ")", function () { return _this.languageService.findRenameLocations(fileName, position, findInStrings, findInComments, providePrefixAndSuffixTextForRename); });
            };
            /// GET BRACE MATCHING
            LanguageServiceShimObject.prototype.getBraceMatchingAtPosition = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("getBraceMatchingAtPosition('" + fileName + "', " + position + ")", function () { return _this.languageService.getBraceMatchingAtPosition(fileName, position); });
            };
            LanguageServiceShimObject.prototype.isValidBraceCompletionAtPosition = function (fileName, position, openingBrace) {
                var _this = this;
                return this.forwardJSONCall("isValidBraceCompletionAtPosition('" + fileName + "', " + position + ", " + openingBrace + ")", function () { return _this.languageService.isValidBraceCompletionAtPosition(fileName, position, openingBrace); });
            };
            LanguageServiceShimObject.prototype.getSpanOfEnclosingComment = function (fileName, position, onlyMultiLine) {
                var _this = this;
                return this.forwardJSONCall("getSpanOfEnclosingComment('" + fileName + "', " + position + ")", function () { return _this.languageService.getSpanOfEnclosingComment(fileName, position, onlyMultiLine); });
            };
            /// GET SMART INDENT
            LanguageServiceShimObject.prototype.getIndentationAtPosition = function (fileName, position, options /*Services.EditorOptions*/) {
                var _this = this;
                return this.forwardJSONCall("getIndentationAtPosition('" + fileName + "', " + position + ")", function () {
                    var localOptions = JSON.parse(options);
                    return _this.languageService.getIndentationAtPosition(fileName, position, localOptions);
                });
            };
            /// GET REFERENCES
            LanguageServiceShimObject.prototype.getReferencesAtPosition = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("getReferencesAtPosition('" + fileName + "', " + position + ")", function () { return _this.languageService.getReferencesAtPosition(fileName, position); });
            };
            LanguageServiceShimObject.prototype.findReferences = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("findReferences('" + fileName + "', " + position + ")", function () { return _this.languageService.findReferences(fileName, position); });
            };
            LanguageServiceShimObject.prototype.getOccurrencesAtPosition = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("getOccurrencesAtPosition('" + fileName + "', " + position + ")", function () { return _this.languageService.getOccurrencesAtPosition(fileName, position); });
            };
            LanguageServiceShimObject.prototype.getDocumentHighlights = function (fileName, position, filesToSearch) {
                var _this = this;
                return this.forwardJSONCall("getDocumentHighlights('" + fileName + "', " + position + ")", function () {
                    var results = _this.languageService.getDocumentHighlights(fileName, position, JSON.parse(filesToSearch));
                    // workaround for VS document highlighting issue - keep only items from the initial file
                    var normalizedName = ts.toFileNameLowerCase(ts.normalizeSlashes(fileName));
                    return ts.filter(results, function (r) { return ts.toFileNameLowerCase(ts.normalizeSlashes(r.fileName)) === normalizedName; });
                });
            };
            /// COMPLETION LISTS
            /**
             * Get a string based representation of the completions
             * to provide at the given source position and providing a member completion
             * list if requested.
             */
            LanguageServiceShimObject.prototype.getCompletionsAtPosition = function (fileName, position, preferences) {
                var _this = this;
                return this.forwardJSONCall("getCompletionsAtPosition('" + fileName + "', " + position + ", " + preferences + ")", function () { return _this.languageService.getCompletionsAtPosition(fileName, position, preferences); });
            };
            /** Get a string based representation of a completion list entry details */
            LanguageServiceShimObject.prototype.getCompletionEntryDetails = function (fileName, position, entryName, formatOptions, source, preferences) {
                var _this = this;
                return this.forwardJSONCall("getCompletionEntryDetails('" + fileName + "', " + position + ", '" + entryName + "')", function () {
                    var localOptions = formatOptions === undefined ? undefined : JSON.parse(formatOptions);
                    return _this.languageService.getCompletionEntryDetails(fileName, position, entryName, localOptions, source, preferences);
                });
            };
            LanguageServiceShimObject.prototype.getFormattingEditsForRange = function (fileName, start, end, options /*Services.FormatCodeOptions*/) {
                var _this = this;
                return this.forwardJSONCall("getFormattingEditsForRange('" + fileName + "', " + start + ", " + end + ")", function () {
                    var localOptions = JSON.parse(options);
                    return _this.languageService.getFormattingEditsForRange(fileName, start, end, localOptions);
                });
            };
            LanguageServiceShimObject.prototype.getFormattingEditsForDocument = function (fileName, options /*Services.FormatCodeOptions*/) {
                var _this = this;
                return this.forwardJSONCall("getFormattingEditsForDocument('" + fileName + "')", function () {
                    var localOptions = JSON.parse(options);
                    return _this.languageService.getFormattingEditsForDocument(fileName, localOptions);
                });
            };
            LanguageServiceShimObject.prototype.getFormattingEditsAfterKeystroke = function (fileName, position, key, options /*Services.FormatCodeOptions*/) {
                var _this = this;
                return this.forwardJSONCall("getFormattingEditsAfterKeystroke('" + fileName + "', " + position + ", '" + key + "')", function () {
                    var localOptions = JSON.parse(options);
                    return _this.languageService.getFormattingEditsAfterKeystroke(fileName, position, key, localOptions);
                });
            };
            LanguageServiceShimObject.prototype.getDocCommentTemplateAtPosition = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("getDocCommentTemplateAtPosition('" + fileName + "', " + position + ")", function () { return _this.languageService.getDocCommentTemplateAtPosition(fileName, position); });
            };
            /// NAVIGATE TO
            /** Return a list of symbols that are interesting to navigate to */
            LanguageServiceShimObject.prototype.getNavigateToItems = function (searchValue, maxResultCount, fileName) {
                var _this = this;
                return this.forwardJSONCall("getNavigateToItems('" + searchValue + "', " + maxResultCount + ", " + fileName + ")", function () { return _this.languageService.getNavigateToItems(searchValue, maxResultCount, fileName); });
            };
            LanguageServiceShimObject.prototype.getNavigationBarItems = function (fileName) {
                var _this = this;
                return this.forwardJSONCall("getNavigationBarItems('" + fileName + "')", function () { return _this.languageService.getNavigationBarItems(fileName); });
            };
            LanguageServiceShimObject.prototype.getNavigationTree = function (fileName) {
                var _this = this;
                return this.forwardJSONCall("getNavigationTree('" + fileName + "')", function () { return _this.languageService.getNavigationTree(fileName); });
            };
            LanguageServiceShimObject.prototype.getOutliningSpans = function (fileName) {
                var _this = this;
                return this.forwardJSONCall("getOutliningSpans('" + fileName + "')", function () { return _this.languageService.getOutliningSpans(fileName); });
            };
            LanguageServiceShimObject.prototype.getTodoComments = function (fileName, descriptors) {
                var _this = this;
                return this.forwardJSONCall("getTodoComments('" + fileName + "')", function () { return _this.languageService.getTodoComments(fileName, JSON.parse(descriptors)); });
            };
            /// CALL HIERARCHY
            LanguageServiceShimObject.prototype.prepareCallHierarchy = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("prepareCallHierarchy('" + fileName + "', " + position + ")", function () { return _this.languageService.prepareCallHierarchy(fileName, position); });
            };
            LanguageServiceShimObject.prototype.provideCallHierarchyIncomingCalls = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("provideCallHierarchyIncomingCalls('" + fileName + "', " + position + ")", function () { return _this.languageService.provideCallHierarchyIncomingCalls(fileName, position); });
            };
            LanguageServiceShimObject.prototype.provideCallHierarchyOutgoingCalls = function (fileName, position) {
                var _this = this;
                return this.forwardJSONCall("provideCallHierarchyOutgoingCalls('" + fileName + "', " + position + ")", function () { return _this.languageService.provideCallHierarchyOutgoingCalls(fileName, position); });
            };
            /// Emit
            LanguageServiceShimObject.prototype.getEmitOutput = function (fileName) {
                var _this = this;
                return this.forwardJSONCall("getEmitOutput('" + fileName + "')", function () { return _this.languageService.getEmitOutput(fileName); });
            };
            LanguageServiceShimObject.prototype.getEmitOutputObject = function (fileName) {
                var _this = this;
                return forwardCall(this.logger, "getEmitOutput('" + fileName + "')", 
                /*returnJson*/ false, function () { return _this.languageService.getEmitOutput(fileName); }, this.logPerformance);
            };
            return LanguageServiceShimObject;
        }(ShimBase));
        function convertClassifications(classifications) {
            return { spans: classifications.spans.join(","), endOfLineState: classifications.endOfLineState };
        }
        var ClassifierShimObject = /** @class */ (function (_super) {
            __extends(ClassifierShimObject, _super);
            function ClassifierShimObject(factory, logger) {
                var _this = _super.call(this, factory) || this;
                _this.logger = logger;
                _this.logPerformance = false;
                _this.classifier = ts.createClassifier();
                return _this;
            }
            ClassifierShimObject.prototype.getEncodedLexicalClassifications = function (text, lexState, syntacticClassifierAbsent) {
                var _this = this;
                if (syntacticClassifierAbsent === void 0) {
                    syntacticClassifierAbsent = false;
                }
                return forwardJSONCall(this.logger, "getEncodedLexicalClassifications", function () { return convertClassifications(_this.classifier.getEncodedLexicalClassifications(text, lexState, syntacticClassifierAbsent)); }, this.logPerformance);
            };
            /// COLORIZATION
            ClassifierShimObject.prototype.getClassificationsForLine = function (text, lexState, classifyKeywordsInGenerics) {
                if (classifyKeywordsInGenerics === void 0) {
                    classifyKeywordsInGenerics = false;
                }
                var classification = this.classifier.getClassificationsForLine(text, lexState, classifyKeywordsInGenerics);
                var result = "";
                for (var _i = 0, _a = classification.entries; _i < _a.length; _i++) {
                    var item = _a[_i];
                    result += item.length + "\n";
                    result += item.classification + "\n";
                }
                result += classification.finalLexState;
                return result;
            };
            return ClassifierShimObject;
        }(ShimBase));
        var CoreServicesShimObject = /** @class */ (function (_super) {
            __extends(CoreServicesShimObject, _super);
            function CoreServicesShimObject(factory, logger, host) {
                var _this = _super.call(this, factory) || this;
                _this.logger = logger;
                _this.host = host;
                _this.logPerformance = false;
                return _this;
            }
            CoreServicesShimObject.prototype.forwardJSONCall = function (actionDescription, action) {
                return forwardJSONCall(this.logger, actionDescription, action, this.logPerformance);
            };
            CoreServicesShimObject.prototype.resolveModuleName = function (fileName, moduleName, compilerOptionsJson) {
                var _this = this;
                return this.forwardJSONCall("resolveModuleName('" + fileName + "')", function () {
                    var compilerOptions = JSON.parse(compilerOptionsJson);
                    var result = ts.resolveModuleName(moduleName, ts.normalizeSlashes(fileName), compilerOptions, _this.host);
                    var resolvedFileName = result.resolvedModule ? result.resolvedModule.resolvedFileName : undefined;
                    if (result.resolvedModule && result.resolvedModule.extension !== ".ts" /* Ts */ && result.resolvedModule.extension !== ".tsx" /* Tsx */ && result.resolvedModule.extension !== ".d.ts" /* Dts */) {
                        resolvedFileName = undefined;
                    }
                    return {
                        resolvedFileName: resolvedFileName,
                        failedLookupLocations: result.failedLookupLocations
                    };
                });
            };
            CoreServicesShimObject.prototype.resolveTypeReferenceDirective = function (fileName, typeReferenceDirective, compilerOptionsJson) {
                var _this = this;
                return this.forwardJSONCall("resolveTypeReferenceDirective(" + fileName + ")", function () {
                    var compilerOptions = JSON.parse(compilerOptionsJson);
                    var result = ts.resolveTypeReferenceDirective(typeReferenceDirective, ts.normalizeSlashes(fileName), compilerOptions, _this.host);
                    return {
                        resolvedFileName: result.resolvedTypeReferenceDirective ? result.resolvedTypeReferenceDirective.resolvedFileName : undefined,
                        primary: result.resolvedTypeReferenceDirective ? result.resolvedTypeReferenceDirective.primary : true,
                        failedLookupLocations: result.failedLookupLocations
                    };
                });
            };
            CoreServicesShimObject.prototype.getPreProcessedFileInfo = function (fileName, sourceTextSnapshot) {
                var _this = this;
                return this.forwardJSONCall("getPreProcessedFileInfo('" + fileName + "')", function () {
                    // for now treat files as JavaScript
                    var result = ts.preProcessFile(ts.getSnapshotText(sourceTextSnapshot), /* readImportFiles */ true, /* detectJavaScriptImports */ true);
                    return {
                        referencedFiles: _this.convertFileReferences(result.referencedFiles),
                        importedFiles: _this.convertFileReferences(result.importedFiles),
                        ambientExternalModules: result.ambientExternalModules,
                        isLibFile: result.isLibFile,
                        typeReferenceDirectives: _this.convertFileReferences(result.typeReferenceDirectives),
                        libReferenceDirectives: _this.convertFileReferences(result.libReferenceDirectives)
                    };
                });
            };
            CoreServicesShimObject.prototype.getAutomaticTypeDirectiveNames = function (compilerOptionsJson) {
                var _this = this;
                return this.forwardJSONCall("getAutomaticTypeDirectiveNames('" + compilerOptionsJson + "')", function () {
                    var compilerOptions = JSON.parse(compilerOptionsJson);
                    return ts.getAutomaticTypeDirectiveNames(compilerOptions, _this.host);
                });
            };
            CoreServicesShimObject.prototype.convertFileReferences = function (refs) {
                if (!refs) {
                    return undefined;
                }
                var result = [];
                for (var _i = 0, refs_1 = refs; _i < refs_1.length; _i++) {
                    var ref = refs_1[_i];
                    result.push({
                        path: ts.normalizeSlashes(ref.fileName),
                        position: ref.pos,
                        length: ref.end - ref.pos
                    });
                }
                return result;
            };
            CoreServicesShimObject.prototype.getTSConfigFileInfo = function (fileName, sourceTextSnapshot) {
                var _this = this;
                return this.forwardJSONCall("getTSConfigFileInfo('" + fileName + "')", function () {
                    var result = ts.parseJsonText(fileName, ts.getSnapshotText(sourceTextSnapshot));
                    var normalizedFileName = ts.normalizeSlashes(fileName);
                    var configFile = ts.parseJsonSourceFileConfigFileContent(result, _this.host, ts.getDirectoryPath(normalizedFileName), /*existingOptions*/ {}, normalizedFileName);
                    return {
                        options: configFile.options,
                        typeAcquisition: configFile.typeAcquisition,
                        files: configFile.fileNames,
                        raw: configFile.raw,
                        errors: realizeDiagnostics(__spreadArrays(result.parseDiagnostics, configFile.errors), "\r\n")
                    };
                });
            };
            CoreServicesShimObject.prototype.getDefaultCompilationSettings = function () {
                return this.forwardJSONCall("getDefaultCompilationSettings()", function () { return ts.getDefaultCompilerOptions(); });
            };
            CoreServicesShimObject.prototype.discoverTypings = function (discoverTypingsJson) {
                var _this = this;
                var getCanonicalFileName = ts.createGetCanonicalFileName(/*useCaseSensitivefileNames:*/ false);
                return this.forwardJSONCall("discoverTypings()", function () {
                    var info = JSON.parse(discoverTypingsJson);
                    if (_this.safeList === undefined) {
                        _this.safeList = ts.JsTyping.loadSafeList(_this.host, ts.toPath(info.safeListPath, info.safeListPath, getCanonicalFileName));
                    }
                    return ts.JsTyping.discoverTypings(_this.host, function (msg) { return _this.logger.log(msg); }, info.fileNames, ts.toPath(info.projectRootPath, info.projectRootPath, getCanonicalFileName), _this.safeList, info.packageNameToTypingLocation, info.typeAcquisition, info.unresolvedImports, info.typesRegistry);
                });
            };
            return CoreServicesShimObject;
        }(ShimBase));
        var TypeScriptServicesFactory = /** @class */ (function () {
            function TypeScriptServicesFactory() {
                this._shims = [];
            }
            /*
             * Returns script API version.
             */
            TypeScriptServicesFactory.prototype.getServicesVersion = function () {
                return ts.servicesVersion;
            };
            TypeScriptServicesFactory.prototype.createLanguageServiceShim = function (host) {
                try {
                    if (this.documentRegistry === undefined) {
                        this.documentRegistry = ts.createDocumentRegistry(host.useCaseSensitiveFileNames && host.useCaseSensitiveFileNames(), host.getCurrentDirectory());
                    }
                    var hostAdapter = new LanguageServiceShimHostAdapter(host);
                    var languageService = ts.createLanguageService(hostAdapter, this.documentRegistry, /*syntaxOnly*/ false);
                    return new LanguageServiceShimObject(this, host, languageService);
                }
                catch (err) {
                    logInternalError(host, err);
                    throw err;
                }
            };
            TypeScriptServicesFactory.prototype.createClassifierShim = function (logger) {
                try {
                    return new ClassifierShimObject(this, logger);
                }
                catch (err) {
                    logInternalError(logger, err);
                    throw err;
                }
            };
            TypeScriptServicesFactory.prototype.createCoreServicesShim = function (host) {
                try {
                    var adapter = new CoreServicesShimHostAdapter(host);
                    return new CoreServicesShimObject(this, host, adapter);
                }
                catch (err) {
                    logInternalError(host, err);
                    throw err;
                }
            };
            TypeScriptServicesFactory.prototype.close = function () {
                // Forget all the registered shims
                ts.clear(this._shims);
                this.documentRegistry = undefined;
            };
            TypeScriptServicesFactory.prototype.registerShim = function (shim) {
                this._shims.push(shim);
            };
            TypeScriptServicesFactory.prototype.unregisterShim = function (shim) {
                for (var i = 0; i < this._shims.length; i++) {
                    if (this._shims[i] === shim) {
                        delete this._shims[i];
                        return;
                    }
                }
                throw new Error("Invalid operation");
            };
            return TypeScriptServicesFactory;
        }());
        ts.TypeScriptServicesFactory = TypeScriptServicesFactory;
    })(ts || (ts = {}));
    /* eslint-enable no-in-operator */
    // We polyfill `globalThis` here so re can reliably patch the global scope
    // in the contexts we want to in the same way across script and module formats
    /* eslint-enable no-var */
    ((function () {
        if (typeof globalThis === "object")
            return;
        try {
            Object.defineProperty(Object.prototype, "__magic__", {
                get: function () {
                    return this;
                },
                configurable: true
            });
            //@ts-ignore
            __magic__.globalThis = __magic__;
            // The previous line should have made `globalThis` globally
            // available, but it fails in Internet Explorer 10 and older.
            // Detect this failure and fall back.
            if (typeof globalThis === "undefined") {
                // Assume `window` exists.
                //@ts-ignore
                window.globalThis = window;
            }
            //@ts-ignore
            delete Object.prototype.__magic__;
        }
        catch (error) {
            // In IE8, Object.defineProperty only works on DOM objects.
            // If we hit this code path, assume `window` exists.
            //@ts-ignore
            window.globalThis = window;
        }
    })());
    // #endregion The polyfill ends here.
    // if `process` is undefined, we're probably not running in node - patch legacy members onto the global scope
    // @ts-ignore
    if (typeof process === "undefined" || process.browser) {
        /// TODO: this is used by VS, clean this up on both sides of the interface
        //@ts-ignore
        globalThis.TypeScript = globalThis.TypeScript || {};
        //@ts-ignore
        globalThis.TypeScript.Services = globalThis.TypeScript.Services || {};
        //@ts-ignore
        globalThis.TypeScript.Services.TypeScriptServicesFactory = ts.TypeScriptServicesFactory;
        // 'toolsVersion' gets consumed by the managed side, so it's not unused.
        // TODO: it should be moved into a namespace though.
        //@ts-ignore
        globalThis.toolsVersion = ts.versionMajorMinor;
    }
    if (typeof module !== "undefined" && module.exports) {
        module.exports = ts;
    }
},
/* source-map-support/source-map-support.js */ function _(require, module, exports) {
    var SourceMapConsumer = require(46) /* source-map */.SourceMapConsumer;
    var path = require('path');
    var fs;
    try {
        fs = require('fs');
        if (!fs.existsSync || !fs.readFileSync) {
            // fs doesn't have all methods we need
            fs = null;
        }
    }
    catch (err) {
        /* nop */
    }
    var bufferFrom = require(57) /* buffer-from */;
    // Only install once if called multiple times
    var errorFormatterInstalled = false;
    var uncaughtShimInstalled = false;
    // If true, the caches are reset before a stack trace formatting operation
    var emptyCacheBetweenOperations = false;
    // Supports {browser, node, auto}
    var environment = "auto";
    // Maps a file path to a string containing the file contents
    var fileContentsCache = {};
    // Maps a file path to a source map for that file
    var sourceMapCache = {};
    // Regex for detecting source maps
    var reSourceMap = /^data:application\/json[^,]+base64,/;
    // Priority list of retrieve handlers
    var retrieveFileHandlers = [];
    var retrieveMapHandlers = [];
    function isInBrowser() {
        if (environment === "browser")
            return true;
        if (environment === "node")
            return false;
        return ((typeof window !== 'undefined') && (typeof XMLHttpRequest === 'function') && !(window.require && window.module && window.process && window.process.type === "renderer"));
    }
    function hasGlobalProcessEventEmitter() {
        return ((typeof process === 'object') && (process !== null) && (typeof process.on === 'function'));
    }
    function handlerExec(list) {
        return function (arg) {
            for (var i = 0; i < list.length; i++) {
                var ret = list[i](arg);
                if (ret) {
                    return ret;
                }
            }
            return null;
        };
    }
    var retrieveFile = handlerExec(retrieveFileHandlers);
    retrieveFileHandlers.push(function (path) {
        // Trim the path to make sure there is no extra whitespace.
        path = path.trim();
        if (/^file:/.test(path)) {
            // existsSync/readFileSync can't handle file protocol, but once stripped, it works
            path = path.replace(/file:\/\/\/(\w:)?/, function (protocol, drive) {
                return drive ?
                    '' : // file:///C:/dir/file -> C:/dir/file
                    '/'; // file:///root-dir/file -> /root-dir/file
            });
        }
        if (path in fileContentsCache) {
            return fileContentsCache[path];
        }
        var contents = '';
        try {
            if (!fs) {
                // Use SJAX if we are in the browser
                var xhr = new XMLHttpRequest();
                xhr.open('GET', path, /** async */ false);
                xhr.send(null);
                if (xhr.readyState === 4 && xhr.status === 200) {
                    contents = xhr.responseText;
                }
            }
            else if (fs.existsSync(path)) {
                // Otherwise, use the filesystem
                contents = fs.readFileSync(path, 'utf8');
            }
        }
        catch (er) {
            /* ignore any errors */
        }
        return fileContentsCache[path] = contents;
    });
    // Support URLs relative to a directory, but be careful about a protocol prefix
    // in case we are in the browser (i.e. directories may start with "http://" or "file:///")
    function supportRelativeURL(file, url) {
        if (!file)
            return url;
        var dir = path.dirname(file);
        var match = /^\w+:\/\/[^\/]*/.exec(dir);
        var protocol = match ? match[0] : '';
        var startPath = dir.slice(protocol.length);
        if (protocol && /^\/\w\:/.test(startPath)) {
            // handle file:///C:/ paths
            protocol += '/';
            return protocol + path.resolve(dir.slice(protocol.length), url).replace(/\\/g, '/');
        }
        return protocol + path.resolve(dir.slice(protocol.length), url);
    }
    function retrieveSourceMapURL(source) {
        var fileData;
        if (isInBrowser()) {
            try {
                var xhr = new XMLHttpRequest();
                xhr.open('GET', source, false);
                xhr.send(null);
                fileData = xhr.readyState === 4 ? xhr.responseText : null;
                // Support providing a sourceMappingURL via the SourceMap header
                var sourceMapHeader = xhr.getResponseHeader("SourceMap") ||
                    xhr.getResponseHeader("X-SourceMap");
                if (sourceMapHeader) {
                    return sourceMapHeader;
                }
            }
            catch (e) {
            }
        }
        // Get the URL of the source map
        fileData = retrieveFile(source);
        var re = /(?:\/\/[@#][\s]*sourceMappingURL=([^\s'"]+)[\s]*$)|(?:\/\*[@#][\s]*sourceMappingURL=([^\s*'"]+)[\s]*(?:\*\/)[\s]*$)/mg;
        // Keep executing the search to find the *last* sourceMappingURL to avoid
        // picking up sourceMappingURLs from comments, strings, etc.
        var lastMatch, match;
        while (match = re.exec(fileData))
            lastMatch = match;
        if (!lastMatch)
            return null;
        return lastMatch[1];
    }
    ;
    // Can be overridden by the retrieveSourceMap option to install. Takes a
    // generated source filename; returns a {map, optional url} object, or null if
    // there is no source map.  The map field may be either a string or the parsed
    // JSON object (ie, it must be a valid argument to the SourceMapConsumer
    // constructor).
    var retrieveSourceMap = handlerExec(retrieveMapHandlers);
    retrieveMapHandlers.push(function (source) {
        var sourceMappingURL = retrieveSourceMapURL(source);
        if (!sourceMappingURL)
            return null;
        // Read the contents of the source map
        var sourceMapData;
        if (reSourceMap.test(sourceMappingURL)) {
            // Support source map URL as a data url
            var rawData = sourceMappingURL.slice(sourceMappingURL.indexOf(',') + 1);
            sourceMapData = bufferFrom(rawData, "base64").toString();
            sourceMappingURL = source;
        }
        else {
            // Support source map URLs relative to the source URL
            sourceMappingURL = supportRelativeURL(source, sourceMappingURL);
            sourceMapData = retrieveFile(sourceMappingURL);
        }
        if (!sourceMapData) {
            return null;
        }
        return {
            url: sourceMappingURL,
            map: sourceMapData
        };
    });
    function mapSourcePosition(position) {
        var sourceMap = sourceMapCache[position.source];
        if (!sourceMap) {
            // Call the (overrideable) retrieveSourceMap function to get the source map.
            var urlAndMap = retrieveSourceMap(position.source);
            if (urlAndMap) {
                sourceMap = sourceMapCache[position.source] = {
                    url: urlAndMap.url,
                    map: new SourceMapConsumer(urlAndMap.map)
                };
                // Load all sources stored inline with the source map into the file cache
                // to pretend like they are already loaded. They may not exist on disk.
                if (sourceMap.map.sourcesContent) {
                    sourceMap.map.sources.forEach(function (source, i) {
                        var contents = sourceMap.map.sourcesContent[i];
                        if (contents) {
                            var url = supportRelativeURL(sourceMap.url, source);
                            fileContentsCache[url] = contents;
                        }
                    });
                }
            }
            else {
                sourceMap = sourceMapCache[position.source] = {
                    url: null,
                    map: null
                };
            }
        }
        // Resolve the source URL relative to the URL of the source map
        if (sourceMap && sourceMap.map && typeof sourceMap.map.originalPositionFor === 'function') {
            var originalPosition = sourceMap.map.originalPositionFor(position);
            // Only return the original position if a matching line was found. If no
            // matching line is found then we return position instead, which will cause
            // the stack trace to print the path and line for the compiled file. It is
            // better to give a precise location in the compiled file than a vague
            // location in the original file.
            if (originalPosition.source !== null) {
                originalPosition.source = supportRelativeURL(sourceMap.url, originalPosition.source);
                return originalPosition;
            }
        }
        return position;
    }
    // Parses code generated by FormatEvalOrigin(), a function inside V8:
    // https://code.google.com/p/v8/source/browse/trunk/src/messages.js
    function mapEvalOrigin(origin) {
        // Most eval() calls are in this format
        var match = /^eval at ([^(]+) \((.+):(\d+):(\d+)\)$/.exec(origin);
        if (match) {
            var position = mapSourcePosition({
                source: match[2],
                line: +match[3],
                column: match[4] - 1
            });
            return 'eval at ' + match[1] + ' (' + position.source + ':' +
                position.line + ':' + (position.column + 1) + ')';
        }
        // Parse nested eval() calls using recursion
        match = /^eval at ([^(]+) \((.+)\)$/.exec(origin);
        if (match) {
            return 'eval at ' + match[1] + ' (' + mapEvalOrigin(match[2]) + ')';
        }
        // Make sure we still return useful information if we didn't find anything
        return origin;
    }
    // This is copied almost verbatim from the V8 source code at
    // https://code.google.com/p/v8/source/browse/trunk/src/messages.js. The
    // implementation of wrapCallSite() used to just forward to the actual source
    // code of CallSite.prototype.toString but unfortunately a new release of V8
    // did something to the prototype chain and broke the shim. The only fix I
    // could find was copy/paste.
    function CallSiteToString() {
        var fileName;
        var fileLocation = "";
        if (this.isNative()) {
            fileLocation = "native";
        }
        else {
            fileName = this.getScriptNameOrSourceURL();
            if (!fileName && this.isEval()) {
                fileLocation = this.getEvalOrigin();
                fileLocation += ", "; // Expecting source position to follow.
            }
            if (fileName) {
                fileLocation += fileName;
            }
            else {
                // Source code does not originate from a file and is not native, but we
                // can still get the source position inside the source string, e.g. in
                // an eval string.
                fileLocation += "<anonymous>";
            }
            var lineNumber = this.getLineNumber();
            if (lineNumber != null) {
                fileLocation += ":" + lineNumber;
                var columnNumber = this.getColumnNumber();
                if (columnNumber) {
                    fileLocation += ":" + columnNumber;
                }
            }
        }
        var line = "";
        var functionName = this.getFunctionName();
        var addSuffix = true;
        var isConstructor = this.isConstructor();
        var isMethodCall = !(this.isToplevel() || isConstructor);
        if (isMethodCall) {
            var typeName = this.getTypeName();
            // Fixes shim to be backward compatable with Node v0 to v4
            if (typeName === "[object Object]") {
                typeName = "null";
            }
            var methodName = this.getMethodName();
            if (functionName) {
                if (typeName && functionName.indexOf(typeName) != 0) {
                    line += typeName + ".";
                }
                line += functionName;
                if (methodName && functionName.indexOf("." + methodName) != functionName.length - methodName.length - 1) {
                    line += " [as " + methodName + "]";
                }
            }
            else {
                line += typeName + "." + (methodName || "<anonymous>");
            }
        }
        else if (isConstructor) {
            line += "new " + (functionName || "<anonymous>");
        }
        else if (functionName) {
            line += functionName;
        }
        else {
            line += fileLocation;
            addSuffix = false;
        }
        if (addSuffix) {
            line += " (" + fileLocation + ")";
        }
        return line;
    }
    function cloneCallSite(frame) {
        var object = {};
        Object.getOwnPropertyNames(Object.getPrototypeOf(frame)).forEach(function (name) {
            object[name] = /^(?:is|get)/.test(name) ? function () { return frame[name].call(frame); } : frame[name];
        });
        object.toString = CallSiteToString;
        return object;
    }
    function wrapCallSite(frame, state) {
        // provides interface backward compatibility
        if (state === undefined) {
            state = { nextPosition: null, curPosition: null };
        }
        if (frame.isNative()) {
            state.curPosition = null;
            return frame;
        }
        // Most call sites will return the source file from getFileName(), but code
        // passed to eval() ending in "//# sourceURL=..." will return the source file
        // from getScriptNameOrSourceURL() instead
        var source = frame.getFileName() || frame.getScriptNameOrSourceURL();
        if (source) {
            var line = frame.getLineNumber();
            var column = frame.getColumnNumber() - 1;
            // Fix position in Node where some (internal) code is prepended.
            // See https://github.com/evanw/node-source-map-support/issues/36
            // Header removed in node at ^10.16 || >=11.11.0
            // v11 is not an LTS candidate, we can just test the one version with it.
            // Test node versions for: 10.16-19, 10.20+, 12-19, 20-99, 100+, or 11.11
            var noHeader = /^v(10\.1[6-9]|10\.[2-9][0-9]|10\.[0-9]{3,}|1[2-9]\d*|[2-9]\d|\d{3,}|11\.11)/;
            var headerLength = noHeader.test(process.version) ? 0 : 62;
            if (line === 1 && column > headerLength && !isInBrowser() && !frame.isEval()) {
                column -= headerLength;
            }
            var position = mapSourcePosition({
                source: source,
                line: line,
                column: column
            });
            state.curPosition = position;
            frame = cloneCallSite(frame);
            var originalFunctionName = frame.getFunctionName;
            frame.getFunctionName = function () {
                if (state.nextPosition == null) {
                    return originalFunctionName();
                }
                return state.nextPosition.name || originalFunctionName();
            };
            frame.getFileName = function () { return position.source; };
            frame.getLineNumber = function () { return position.line; };
            frame.getColumnNumber = function () { return position.column + 1; };
            frame.getScriptNameOrSourceURL = function () { return position.source; };
            return frame;
        }
        // Code called using eval() needs special handling
        var origin = frame.isEval() && frame.getEvalOrigin();
        if (origin) {
            origin = mapEvalOrigin(origin);
            frame = cloneCallSite(frame);
            frame.getEvalOrigin = function () { return origin; };
            return frame;
        }
        // If we get here then we were unable to change the source position
        return frame;
    }
    // This function is part of the V8 stack trace API, for more info see:
    // https://v8.dev/docs/stack-trace-api
    function prepareStackTrace(error, stack) {
        if (emptyCacheBetweenOperations) {
            fileContentsCache = {};
            sourceMapCache = {};
        }
        var name = error.name || 'Error';
        var message = error.message || '';
        var errorString = name + ": " + message;
        var state = { nextPosition: null, curPosition: null };
        var processedStack = [];
        for (var i = stack.length - 1; i >= 0; i--) {
            processedStack.push('\n    at ' + wrapCallSite(stack[i], state));
            state.nextPosition = state.curPosition;
        }
        state.curPosition = state.nextPosition = null;
        return errorString + processedStack.reverse().join('');
    }
    // Generate position and snippet of original source with pointer
    function getErrorSource(error) {
        var match = /\n    at [^(]+ \((.*):(\d+):(\d+)\)/.exec(error.stack);
        if (match) {
            var source = match[1];
            var line = +match[2];
            var column = +match[3];
            // Support the inline sourceContents inside the source map
            var contents = fileContentsCache[source];
            // Support files on disk
            if (!contents && fs && fs.existsSync(source)) {
                try {
                    contents = fs.readFileSync(source, 'utf8');
                }
                catch (er) {
                    contents = '';
                }
            }
            // Format the line from the original source code like node does
            if (contents) {
                var code = contents.split(/(?:\r\n|\r|\n)/)[line - 1];
                if (code) {
                    return source + ':' + line + '\n' + code + '\n' +
                        new Array(column).join(' ') + '^';
                }
            }
        }
        return null;
    }
    function printErrorAndExit(error) {
        var source = getErrorSource(error);
        // Ensure error is printed synchronously and not truncated
        if (process.stderr._handle && process.stderr._handle.setBlocking) {
            process.stderr._handle.setBlocking(true);
        }
        if (source) {
            console.error();
            console.error(source);
        }
        console.error(error.stack);
        process.exit(1);
    }
    function shimEmitUncaughtException() {
        var origEmit = process.emit;
        process.emit = function (type) {
            if (type === 'uncaughtException') {
                var hasStack = (arguments[1] && arguments[1].stack);
                var hasListeners = (this.listeners(type).length > 0);
                if (hasStack && !hasListeners) {
                    return printErrorAndExit(arguments[1]);
                }
            }
            return origEmit.apply(this, arguments);
        };
    }
    var originalRetrieveFileHandlers = retrieveFileHandlers.slice(0);
    var originalRetrieveMapHandlers = retrieveMapHandlers.slice(0);
    exports.wrapCallSite = wrapCallSite;
    exports.getErrorSource = getErrorSource;
    exports.mapSourcePosition = mapSourcePosition;
    exports.retrieveSourceMap = retrieveSourceMap;
    exports.install = function (options) {
        options = options || {};
        if (options.environment) {
            environment = options.environment;
            if (["node", "browser", "auto"].indexOf(environment) === -1) {
                throw new Error("environment " + environment + " was unknown. Available options are {auto, browser, node}");
            }
        }
        // Allow sources to be found by methods other than reading the files
        // directly from disk.
        if (options.retrieveFile) {
            if (options.overrideRetrieveFile) {
                retrieveFileHandlers.length = 0;
            }
            retrieveFileHandlers.unshift(options.retrieveFile);
        }
        // Allow source maps to be found by methods other than reading the files
        // directly from disk.
        if (options.retrieveSourceMap) {
            if (options.overrideRetrieveSourceMap) {
                retrieveMapHandlers.length = 0;
            }
            retrieveMapHandlers.unshift(options.retrieveSourceMap);
        }
        // Support runtime transpilers that include inline source maps
        if (options.hookRequire && !isInBrowser()) {
            var Module;
            try {
                Module = require('module');
            }
            catch (err) {
                // NOP: Loading in catch block to convert webpack error to warning.
            }
            var $compile = Module.prototype._compile;
            if (!$compile.__sourceMapSupport) {
                Module.prototype._compile = function (content, filename) {
                    fileContentsCache[filename] = content;
                    sourceMapCache[filename] = undefined;
                    return $compile.call(this, content, filename);
                };
                Module.prototype._compile.__sourceMapSupport = true;
            }
        }
        // Configure options
        if (!emptyCacheBetweenOperations) {
            emptyCacheBetweenOperations = 'emptyCacheBetweenOperations' in options ?
                options.emptyCacheBetweenOperations : false;
        }
        // Install the error reformatter
        if (!errorFormatterInstalled) {
            errorFormatterInstalled = true;
            Error.prepareStackTrace = prepareStackTrace;
        }
        if (!uncaughtShimInstalled) {
            var installHandler = 'handleUncaughtExceptions' in options ?
                options.handleUncaughtExceptions : true;
            // Provide the option to not install the uncaught exception handler. This is
            // to support other uncaught exception handlers (in test frameworks, for
            // example). If this handler is not installed and there are no other uncaught
            // exception handlers, uncaught exceptions will be caught by node's built-in
            // exception handler and the process will still be terminated. However, the
            // generated JavaScript code will be shown above the stack trace instead of
            // the original source code.
            if (installHandler && hasGlobalProcessEventEmitter()) {
                uncaughtShimInstalled = true;
                shimEmitUncaughtException();
            }
        }
    };
    exports.resetRetrieveHandlers = function () {
        retrieveFileHandlers.length = 0;
        retrieveMapHandlers.length = 0;
        retrieveFileHandlers = originalRetrieveFileHandlers.slice(0);
        retrieveMapHandlers = originalRetrieveMapHandlers.slice(0);
        retrieveSourceMap = handlerExec(retrieveMapHandlers);
        retrieveFile = handlerExec(retrieveFileHandlers);
    };
},
/* source-map/source-map.js */ function _(require, module, exports) {
    /*
     * Copyright 2009-2011 Mozilla Foundation and contributors
     * Licensed under the New BSD license. See LICENSE.txt or:
     * http://opensource.org/licenses/BSD-3-Clause
     */
    exports.SourceMapGenerator = require(47) /* ./lib/source-map-generator */.SourceMapGenerator;
    exports.SourceMapConsumer = require(53) /* ./lib/source-map-consumer */.SourceMapConsumer;
    exports.SourceNode = require(56) /* ./lib/source-node */.SourceNode;
},
/* source-map/lib/source-map-generator.js */ function _(require, module, exports) {
    /* -*- Mode: js; js-indent-level: 2; -*- */
    /*
     * Copyright 2011 Mozilla Foundation and contributors
     * Licensed under the New BSD license. See LICENSE or:
     * http://opensource.org/licenses/BSD-3-Clause
     */
    var base64VLQ = require(48) /* ./base64-vlq */;
    var util = require(50) /* ./util */;
    var ArraySet = require(51) /* ./array-set */.ArraySet;
    var MappingList = require(52) /* ./mapping-list */.MappingList;
    /**
     * An instance of the SourceMapGenerator represents a source map which is
     * being built incrementally. You may pass an object with the following
     * properties:
     *
     *   - file: The filename of the generated source.
     *   - sourceRoot: A root for all relative URLs in this source map.
     */
    function SourceMapGenerator(aArgs) {
        if (!aArgs) {
            aArgs = {};
        }
        this._file = util.getArg(aArgs, 'file', null);
        this._sourceRoot = util.getArg(aArgs, 'sourceRoot', null);
        this._skipValidation = util.getArg(aArgs, 'skipValidation', false);
        this._sources = new ArraySet();
        this._names = new ArraySet();
        this._mappings = new MappingList();
        this._sourcesContents = null;
    }
    SourceMapGenerator.prototype._version = 3;
    /**
     * Creates a new SourceMapGenerator based on a SourceMapConsumer
     *
     * @param aSourceMapConsumer The SourceMap.
     */
    SourceMapGenerator.fromSourceMap =
        function SourceMapGenerator_fromSourceMap(aSourceMapConsumer) {
            var sourceRoot = aSourceMapConsumer.sourceRoot;
            var generator = new SourceMapGenerator({
                file: aSourceMapConsumer.file,
                sourceRoot: sourceRoot
            });
            aSourceMapConsumer.eachMapping(function (mapping) {
                var newMapping = {
                    generated: {
                        line: mapping.generatedLine,
                        column: mapping.generatedColumn
                    }
                };
                if (mapping.source != null) {
                    newMapping.source = mapping.source;
                    if (sourceRoot != null) {
                        newMapping.source = util.relative(sourceRoot, newMapping.source);
                    }
                    newMapping.original = {
                        line: mapping.originalLine,
                        column: mapping.originalColumn
                    };
                    if (mapping.name != null) {
                        newMapping.name = mapping.name;
                    }
                }
                generator.addMapping(newMapping);
            });
            aSourceMapConsumer.sources.forEach(function (sourceFile) {
                var content = aSourceMapConsumer.sourceContentFor(sourceFile);
                if (content != null) {
                    generator.setSourceContent(sourceFile, content);
                }
            });
            return generator;
        };
    /**
     * Add a single mapping from original source line and column to the generated
     * source's line and column for this source map being created. The mapping
     * object should have the following properties:
     *
     *   - generated: An object with the generated line and column positions.
     *   - original: An object with the original line and column positions.
     *   - source: The original source file (relative to the sourceRoot).
     *   - name: An optional original token name for this mapping.
     */
    SourceMapGenerator.prototype.addMapping =
        function SourceMapGenerator_addMapping(aArgs) {
            var generated = util.getArg(aArgs, 'generated');
            var original = util.getArg(aArgs, 'original', null);
            var source = util.getArg(aArgs, 'source', null);
            var name = util.getArg(aArgs, 'name', null);
            if (!this._skipValidation) {
                this._validateMapping(generated, original, source, name);
            }
            if (source != null) {
                source = String(source);
                if (!this._sources.has(source)) {
                    this._sources.add(source);
                }
            }
            if (name != null) {
                name = String(name);
                if (!this._names.has(name)) {
                    this._names.add(name);
                }
            }
            this._mappings.add({
                generatedLine: generated.line,
                generatedColumn: generated.column,
                originalLine: original != null && original.line,
                originalColumn: original != null && original.column,
                source: source,
                name: name
            });
        };
    /**
     * Set the source content for a source file.
     */
    SourceMapGenerator.prototype.setSourceContent =
        function SourceMapGenerator_setSourceContent(aSourceFile, aSourceContent) {
            var source = aSourceFile;
            if (this._sourceRoot != null) {
                source = util.relative(this._sourceRoot, source);
            }
            if (aSourceContent != null) {
                // Add the source content to the _sourcesContents map.
                // Create a new _sourcesContents map if the property is null.
                if (!this._sourcesContents) {
                    this._sourcesContents = Object.create(null);
                }
                this._sourcesContents[util.toSetString(source)] = aSourceContent;
            }
            else if (this._sourcesContents) {
                // Remove the source file from the _sourcesContents map.
                // If the _sourcesContents map is empty, set the property to null.
                delete this._sourcesContents[util.toSetString(source)];
                if (Object.keys(this._sourcesContents).length === 0) {
                    this._sourcesContents = null;
                }
            }
        };
    /**
     * Applies the mappings of a sub-source-map for a specific source file to the
     * source map being generated. Each mapping to the supplied source file is
     * rewritten using the supplied source map. Note: The resolution for the
     * resulting mappings is the minimium of this map and the supplied map.
     *
     * @param aSourceMapConsumer The source map to be applied.
     * @param aSourceFile Optional. The filename of the source file.
     *        If omitted, SourceMapConsumer's file property will be used.
     * @param aSourceMapPath Optional. The dirname of the path to the source map
     *        to be applied. If relative, it is relative to the SourceMapConsumer.
     *        This parameter is needed when the two source maps aren't in the same
     *        directory, and the source map to be applied contains relative source
     *        paths. If so, those relative source paths need to be rewritten
     *        relative to the SourceMapGenerator.
     */
    SourceMapGenerator.prototype.applySourceMap =
        function SourceMapGenerator_applySourceMap(aSourceMapConsumer, aSourceFile, aSourceMapPath) {
            var sourceFile = aSourceFile;
            // If aSourceFile is omitted, we will use the file property of the SourceMap
            if (aSourceFile == null) {
                if (aSourceMapConsumer.file == null) {
                    throw new Error('SourceMapGenerator.prototype.applySourceMap requires either an explicit source file, ' +
                        'or the source map\'s "file" property. Both were omitted.');
                }
                sourceFile = aSourceMapConsumer.file;
            }
            var sourceRoot = this._sourceRoot;
            // Make "sourceFile" relative if an absolute Url is passed.
            if (sourceRoot != null) {
                sourceFile = util.relative(sourceRoot, sourceFile);
            }
            // Applying the SourceMap can add and remove items from the sources and
            // the names array.
            var newSources = new ArraySet();
            var newNames = new ArraySet();
            // Find mappings for the "sourceFile"
            this._mappings.unsortedForEach(function (mapping) {
                if (mapping.source === sourceFile && mapping.originalLine != null) {
                    // Check if it can be mapped by the source map, then update the mapping.
                    var original = aSourceMapConsumer.originalPositionFor({
                        line: mapping.originalLine,
                        column: mapping.originalColumn
                    });
                    if (original.source != null) {
                        // Copy mapping
                        mapping.source = original.source;
                        if (aSourceMapPath != null) {
                            mapping.source = util.join(aSourceMapPath, mapping.source);
                        }
                        if (sourceRoot != null) {
                            mapping.source = util.relative(sourceRoot, mapping.source);
                        }
                        mapping.originalLine = original.line;
                        mapping.originalColumn = original.column;
                        if (original.name != null) {
                            mapping.name = original.name;
                        }
                    }
                }
                var source = mapping.source;
                if (source != null && !newSources.has(source)) {
                    newSources.add(source);
                }
                var name = mapping.name;
                if (name != null && !newNames.has(name)) {
                    newNames.add(name);
                }
            }, this);
            this._sources = newSources;
            this._names = newNames;
            // Copy sourcesContents of applied map.
            aSourceMapConsumer.sources.forEach(function (sourceFile) {
                var content = aSourceMapConsumer.sourceContentFor(sourceFile);
                if (content != null) {
                    if (aSourceMapPath != null) {
                        sourceFile = util.join(aSourceMapPath, sourceFile);
                    }
                    if (sourceRoot != null) {
                        sourceFile = util.relative(sourceRoot, sourceFile);
                    }
                    this.setSourceContent(sourceFile, content);
                }
            }, this);
        };
    /**
     * A mapping can have one of the three levels of data:
     *
     *   1. Just the generated position.
     *   2. The Generated position, original position, and original source.
     *   3. Generated and original position, original source, as well as a name
     *      token.
     *
     * To maintain consistency, we validate that any new mapping being added falls
     * in to one of these categories.
     */
    SourceMapGenerator.prototype._validateMapping =
        function SourceMapGenerator_validateMapping(aGenerated, aOriginal, aSource, aName) {
            // When aOriginal is truthy but has empty values for .line and .column,
            // it is most likely a programmer error. In this case we throw a very
            // specific error message to try to guide them the right way.
            // For example: https://github.com/Polymer/polymer-bundler/pull/519
            if (aOriginal && typeof aOriginal.line !== 'number' && typeof aOriginal.column !== 'number') {
                throw new Error('original.line and original.column are not numbers -- you probably meant to omit ' +
                    'the original mapping entirely and only map the generated position. If so, pass ' +
                    'null for the original mapping instead of an object with empty or null values.');
            }
            if (aGenerated && 'line' in aGenerated && 'column' in aGenerated
                && aGenerated.line > 0 && aGenerated.column >= 0
                && !aOriginal && !aSource && !aName) {
                // Case 1.
                return;
            }
            else if (aGenerated && 'line' in aGenerated && 'column' in aGenerated
                && aOriginal && 'line' in aOriginal && 'column' in aOriginal
                && aGenerated.line > 0 && aGenerated.column >= 0
                && aOriginal.line > 0 && aOriginal.column >= 0
                && aSource) {
                // Cases 2 and 3.
                return;
            }
            else {
                throw new Error('Invalid mapping: ' + JSON.stringify({
                    generated: aGenerated,
                    source: aSource,
                    original: aOriginal,
                    name: aName
                }));
            }
        };
    /**
     * Serialize the accumulated mappings in to the stream of base 64 VLQs
     * specified by the source map format.
     */
    SourceMapGenerator.prototype._serializeMappings =
        function SourceMapGenerator_serializeMappings() {
            var previousGeneratedColumn = 0;
            var previousGeneratedLine = 1;
            var previousOriginalColumn = 0;
            var previousOriginalLine = 0;
            var previousName = 0;
            var previousSource = 0;
            var result = '';
            var next;
            var mapping;
            var nameIdx;
            var sourceIdx;
            var mappings = this._mappings.toArray();
            for (var i = 0, len = mappings.length; i < len; i++) {
                mapping = mappings[i];
                next = '';
                if (mapping.generatedLine !== previousGeneratedLine) {
                    previousGeneratedColumn = 0;
                    while (mapping.generatedLine !== previousGeneratedLine) {
                        next += ';';
                        previousGeneratedLine++;
                    }
                }
                else {
                    if (i > 0) {
                        if (!util.compareByGeneratedPositionsInflated(mapping, mappings[i - 1])) {
                            continue;
                        }
                        next += ',';
                    }
                }
                next += base64VLQ.encode(mapping.generatedColumn
                    - previousGeneratedColumn);
                previousGeneratedColumn = mapping.generatedColumn;
                if (mapping.source != null) {
                    sourceIdx = this._sources.indexOf(mapping.source);
                    next += base64VLQ.encode(sourceIdx - previousSource);
                    previousSource = sourceIdx;
                    // lines are stored 0-based in SourceMap spec version 3
                    next += base64VLQ.encode(mapping.originalLine - 1
                        - previousOriginalLine);
                    previousOriginalLine = mapping.originalLine - 1;
                    next += base64VLQ.encode(mapping.originalColumn
                        - previousOriginalColumn);
                    previousOriginalColumn = mapping.originalColumn;
                    if (mapping.name != null) {
                        nameIdx = this._names.indexOf(mapping.name);
                        next += base64VLQ.encode(nameIdx - previousName);
                        previousName = nameIdx;
                    }
                }
                result += next;
            }
            return result;
        };
    SourceMapGenerator.prototype._generateSourcesContent =
        function SourceMapGenerator_generateSourcesContent(aSources, aSourceRoot) {
            return aSources.map(function (source) {
                if (!this._sourcesContents) {
                    return null;
                }
                if (aSourceRoot != null) {
                    source = util.relative(aSourceRoot, source);
                }
                var key = util.toSetString(source);
                return Object.prototype.hasOwnProperty.call(this._sourcesContents, key)
                    ? this._sourcesContents[key]
                    : null;
            }, this);
        };
    /**
     * Externalize the source map.
     */
    SourceMapGenerator.prototype.toJSON =
        function SourceMapGenerator_toJSON() {
            var map = {
                version: this._version,
                sources: this._sources.toArray(),
                names: this._names.toArray(),
                mappings: this._serializeMappings()
            };
            if (this._file != null) {
                map.file = this._file;
            }
            if (this._sourceRoot != null) {
                map.sourceRoot = this._sourceRoot;
            }
            if (this._sourcesContents) {
                map.sourcesContent = this._generateSourcesContent(map.sources, map.sourceRoot);
            }
            return map;
        };
    /**
     * Render the source map being generated to a string.
     */
    SourceMapGenerator.prototype.toString =
        function SourceMapGenerator_toString() {
            return JSON.stringify(this.toJSON());
        };
    exports.SourceMapGenerator = SourceMapGenerator;
},
/* source-map/lib/base64-vlq.js */ function _(require, module, exports) {
    /* -*- Mode: js; js-indent-level: 2; -*- */
    /*
     * Copyright 2011 Mozilla Foundation and contributors
     * Licensed under the New BSD license. See LICENSE or:
     * http://opensource.org/licenses/BSD-3-Clause
     *
     * Based on the Base 64 VLQ implementation in Closure Compiler:
     * https://code.google.com/p/closure-compiler/source/browse/trunk/src/com/google/debugging/sourcemap/Base64VLQ.java
     *
     * Copyright 2011 The Closure Compiler Authors. All rights reserved.
     * Redistribution and use in source and binary forms, with or without
     * modification, are permitted provided that the following conditions are
     * met:
     *
     *  * Redistributions of source code must retain the above copyright
     *    notice, this list of conditions and the following disclaimer.
     *  * Redistributions in binary form must reproduce the above
     *    copyright notice, this list of conditions and the following
     *    disclaimer in the documentation and/or other materials provided
     *    with the distribution.
     *  * Neither the name of Google Inc. nor the names of its
     *    contributors may be used to endorse or promote products derived
     *    from this software without specific prior written permission.
     *
     * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
     * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
     * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
     * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
     * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
     * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
     * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
     * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
     * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
     * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
     * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
     */
    var base64 = require(49) /* ./base64 */;
    // A single base 64 digit can contain 6 bits of data. For the base 64 variable
    // length quantities we use in the source map spec, the first bit is the sign,
    // the next four bits are the actual value, and the 6th bit is the
    // continuation bit. The continuation bit tells us whether there are more
    // digits in this value following this digit.
    //
    //   Continuation
    //   |    Sign
    //   |    |
    //   V    V
    //   101011
    var VLQ_BASE_SHIFT = 5;
    // binary: 100000
    var VLQ_BASE = 1 << VLQ_BASE_SHIFT;
    // binary: 011111
    var VLQ_BASE_MASK = VLQ_BASE - 1;
    // binary: 100000
    var VLQ_CONTINUATION_BIT = VLQ_BASE;
    /**
     * Converts from a two-complement value to a value where the sign bit is
     * placed in the least significant bit.  For example, as decimals:
     *   1 becomes 2 (10 binary), -1 becomes 3 (11 binary)
     *   2 becomes 4 (100 binary), -2 becomes 5 (101 binary)
     */
    function toVLQSigned(aValue) {
        return aValue < 0
            ? ((-aValue) << 1) + 1
            : (aValue << 1) + 0;
    }
    /**
     * Converts to a two-complement value from a value where the sign bit is
     * placed in the least significant bit.  For example, as decimals:
     *   2 (10 binary) becomes 1, 3 (11 binary) becomes -1
     *   4 (100 binary) becomes 2, 5 (101 binary) becomes -2
     */
    function fromVLQSigned(aValue) {
        var isNegative = (aValue & 1) === 1;
        var shifted = aValue >> 1;
        return isNegative
            ? -shifted
            : shifted;
    }
    /**
     * Returns the base 64 VLQ encoded value.
     */
    exports.encode = function base64VLQ_encode(aValue) {
        var encoded = "";
        var digit;
        var vlq = toVLQSigned(aValue);
        do {
            digit = vlq & VLQ_BASE_MASK;
            vlq >>>= VLQ_BASE_SHIFT;
            if (vlq > 0) {
                // There are still more digits in this value, so we must make sure the
                // continuation bit is marked.
                digit |= VLQ_CONTINUATION_BIT;
            }
            encoded += base64.encode(digit);
        } while (vlq > 0);
        return encoded;
    };
    /**
     * Decodes the next base 64 VLQ value from the given string and returns the
     * value and the rest of the string via the out parameter.
     */
    exports.decode = function base64VLQ_decode(aStr, aIndex, aOutParam) {
        var strLen = aStr.length;
        var result = 0;
        var shift = 0;
        var continuation, digit;
        do {
            if (aIndex >= strLen) {
                throw new Error("Expected more digits in base 64 VLQ value.");
            }
            digit = base64.decode(aStr.charCodeAt(aIndex++));
            if (digit === -1) {
                throw new Error("Invalid base64 digit: " + aStr.charAt(aIndex - 1));
            }
            continuation = !!(digit & VLQ_CONTINUATION_BIT);
            digit &= VLQ_BASE_MASK;
            result = result + (digit << shift);
            shift += VLQ_BASE_SHIFT;
        } while (continuation);
        aOutParam.value = fromVLQSigned(result);
        aOutParam.rest = aIndex;
    };
},
/* source-map/lib/base64.js */ function _(require, module, exports) {
    /* -*- Mode: js; js-indent-level: 2; -*- */
    /*
     * Copyright 2011 Mozilla Foundation and contributors
     * Licensed under the New BSD license. See LICENSE or:
     * http://opensource.org/licenses/BSD-3-Clause
     */
    var intToCharMap = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'.split('');
    /**
     * Encode an integer in the range of 0 to 63 to a single base 64 digit.
     */
    exports.encode = function (number) {
        if (0 <= number && number < intToCharMap.length) {
            return intToCharMap[number];
        }
        throw new TypeError("Must be between 0 and 63: " + number);
    };
    /**
     * Decode a single base 64 character code digit to an integer. Returns -1 on
     * failure.
     */
    exports.decode = function (charCode) {
        var bigA = 65; // 'A'
        var bigZ = 90; // 'Z'
        var littleA = 97; // 'a'
        var littleZ = 122; // 'z'
        var zero = 48; // '0'
        var nine = 57; // '9'
        var plus = 43; // '+'
        var slash = 47; // '/'
        var littleOffset = 26;
        var numberOffset = 52;
        // 0 - 25: ABCDEFGHIJKLMNOPQRSTUVWXYZ
        if (bigA <= charCode && charCode <= bigZ) {
            return (charCode - bigA);
        }
        // 26 - 51: abcdefghijklmnopqrstuvwxyz
        if (littleA <= charCode && charCode <= littleZ) {
            return (charCode - littleA + littleOffset);
        }
        // 52 - 61: 0123456789
        if (zero <= charCode && charCode <= nine) {
            return (charCode - zero + numberOffset);
        }
        // 62: +
        if (charCode == plus) {
            return 62;
        }
        // 63: /
        if (charCode == slash) {
            return 63;
        }
        // Invalid base64 digit.
        return -1;
    };
},
/* source-map/lib/util.js */ function _(require, module, exports) {
    /* -*- Mode: js; js-indent-level: 2; -*- */
    /*
     * Copyright 2011 Mozilla Foundation and contributors
     * Licensed under the New BSD license. See LICENSE or:
     * http://opensource.org/licenses/BSD-3-Clause
     */
    /**
     * This is a helper function for getting values from parameter/options
     * objects.
     *
     * @param args The object we are extracting values from
     * @param name The name of the property we are getting.
     * @param defaultValue An optional value to return if the property is missing
     * from the object. If this is not specified and the property is missing, an
     * error will be thrown.
     */
    function getArg(aArgs, aName, aDefaultValue) {
        if (aName in aArgs) {
            return aArgs[aName];
        }
        else if (arguments.length === 3) {
            return aDefaultValue;
        }
        else {
            throw new Error('"' + aName + '" is a required argument.');
        }
    }
    exports.getArg = getArg;
    var urlRegexp = /^(?:([\w+\-.]+):)?\/\/(?:(\w+:\w+)@)?([\w.]*)(?::(\d+))?(\S*)$/;
    var dataUrlRegexp = /^data:.+\,.+$/;
    function urlParse(aUrl) {
        var match = aUrl.match(urlRegexp);
        if (!match) {
            return null;
        }
        return {
            scheme: match[1],
            auth: match[2],
            host: match[3],
            port: match[4],
            path: match[5]
        };
    }
    exports.urlParse = urlParse;
    function urlGenerate(aParsedUrl) {
        var url = '';
        if (aParsedUrl.scheme) {
            url += aParsedUrl.scheme + ':';
        }
        url += '//';
        if (aParsedUrl.auth) {
            url += aParsedUrl.auth + '@';
        }
        if (aParsedUrl.host) {
            url += aParsedUrl.host;
        }
        if (aParsedUrl.port) {
            url += ":" + aParsedUrl.port;
        }
        if (aParsedUrl.path) {
            url += aParsedUrl.path;
        }
        return url;
    }
    exports.urlGenerate = urlGenerate;
    /**
     * Normalizes a path, or the path portion of a URL:
     *
     * - Replaces consecutive slashes with one slash.
     * - Removes unnecessary '.' parts.
     * - Removes unnecessary '<dir>/..' parts.
     *
     * Based on code in the Node.js 'path' core module.
     *
     * @param aPath The path or url to normalize.
     */
    function normalize(aPath) {
        var path = aPath;
        var url = urlParse(aPath);
        if (url) {
            if (!url.path) {
                return aPath;
            }
            path = url.path;
        }
        var isAbsolute = exports.isAbsolute(path);
        var parts = path.split(/\/+/);
        for (var part, up = 0, i = parts.length - 1; i >= 0; i--) {
            part = parts[i];
            if (part === '.') {
                parts.splice(i, 1);
            }
            else if (part === '..') {
                up++;
            }
            else if (up > 0) {
                if (part === '') {
                    // The first part is blank if the path is absolute. Trying to go
                    // above the root is a no-op. Therefore we can remove all '..' parts
                    // directly after the root.
                    parts.splice(i + 1, up);
                    up = 0;
                }
                else {
                    parts.splice(i, 2);
                    up--;
                }
            }
        }
        path = parts.join('/');
        if (path === '') {
            path = isAbsolute ? '/' : '.';
        }
        if (url) {
            url.path = path;
            return urlGenerate(url);
        }
        return path;
    }
    exports.normalize = normalize;
    /**
     * Joins two paths/URLs.
     *
     * @param aRoot The root path or URL.
     * @param aPath The path or URL to be joined with the root.
     *
     * - If aPath is a URL or a data URI, aPath is returned, unless aPath is a
     *   scheme-relative URL: Then the scheme of aRoot, if any, is prepended
     *   first.
     * - Otherwise aPath is a path. If aRoot is a URL, then its path portion
     *   is updated with the result and aRoot is returned. Otherwise the result
     *   is returned.
     *   - If aPath is absolute, the result is aPath.
     *   - Otherwise the two paths are joined with a slash.
     * - Joining for example 'http://' and 'www.example.com' is also supported.
     */
    function join(aRoot, aPath) {
        if (aRoot === "") {
            aRoot = ".";
        }
        if (aPath === "") {
            aPath = ".";
        }
        var aPathUrl = urlParse(aPath);
        var aRootUrl = urlParse(aRoot);
        if (aRootUrl) {
            aRoot = aRootUrl.path || '/';
        }
        // `join(foo, '//www.example.org')`
        if (aPathUrl && !aPathUrl.scheme) {
            if (aRootUrl) {
                aPathUrl.scheme = aRootUrl.scheme;
            }
            return urlGenerate(aPathUrl);
        }
        if (aPathUrl || aPath.match(dataUrlRegexp)) {
            return aPath;
        }
        // `join('http://', 'www.example.com')`
        if (aRootUrl && !aRootUrl.host && !aRootUrl.path) {
            aRootUrl.host = aPath;
            return urlGenerate(aRootUrl);
        }
        var joined = aPath.charAt(0) === '/'
            ? aPath
            : normalize(aRoot.replace(/\/+$/, '') + '/' + aPath);
        if (aRootUrl) {
            aRootUrl.path = joined;
            return urlGenerate(aRootUrl);
        }
        return joined;
    }
    exports.join = join;
    exports.isAbsolute = function (aPath) {
        return aPath.charAt(0) === '/' || !!aPath.match(urlRegexp);
    };
    /**
     * Make a path relative to a URL or another path.
     *
     * @param aRoot The root path or URL.
     * @param aPath The path or URL to be made relative to aRoot.
     */
    function relative(aRoot, aPath) {
        if (aRoot === "") {
            aRoot = ".";
        }
        aRoot = aRoot.replace(/\/$/, '');
        // It is possible for the path to be above the root. In this case, simply
        // checking whether the root is a prefix of the path won't work. Instead, we
        // need to remove components from the root one by one, until either we find
        // a prefix that fits, or we run out of components to remove.
        var level = 0;
        while (aPath.indexOf(aRoot + '/') !== 0) {
            var index = aRoot.lastIndexOf("/");
            if (index < 0) {
                return aPath;
            }
            // If the only part of the root that is left is the scheme (i.e. http://,
            // file:///, etc.), one or more slashes (/), or simply nothing at all, we
            // have exhausted all components, so the path is not relative to the root.
            aRoot = aRoot.slice(0, index);
            if (aRoot.match(/^([^\/]+:\/)?\/*$/)) {
                return aPath;
            }
            ++level;
        }
        // Make sure we add a "../" for each component we removed from the root.
        return Array(level + 1).join("../") + aPath.substr(aRoot.length + 1);
    }
    exports.relative = relative;
    var supportsNullProto = (function () {
        var obj = Object.create(null);
        return !('__proto__' in obj);
    }());
    function identity(s) {
        return s;
    }
    /**
     * Because behavior goes wacky when you set `__proto__` on objects, we
     * have to prefix all the strings in our set with an arbitrary character.
     *
     * See https://github.com/mozilla/source-map/pull/31 and
     * https://github.com/mozilla/source-map/issues/30
     *
     * @param String aStr
     */
    function toSetString(aStr) {
        if (isProtoString(aStr)) {
            return '$' + aStr;
        }
        return aStr;
    }
    exports.toSetString = supportsNullProto ? identity : toSetString;
    function fromSetString(aStr) {
        if (isProtoString(aStr)) {
            return aStr.slice(1);
        }
        return aStr;
    }
    exports.fromSetString = supportsNullProto ? identity : fromSetString;
    function isProtoString(s) {
        if (!s) {
            return false;
        }
        var length = s.length;
        if (length < 9 /* "__proto__".length */) {
            return false;
        }
        if (s.charCodeAt(length - 1) !== 95 /* '_' */ ||
            s.charCodeAt(length - 2) !== 95 /* '_' */ ||
            s.charCodeAt(length - 3) !== 111 /* 'o' */ ||
            s.charCodeAt(length - 4) !== 116 /* 't' */ ||
            s.charCodeAt(length - 5) !== 111 /* 'o' */ ||
            s.charCodeAt(length - 6) !== 114 /* 'r' */ ||
            s.charCodeAt(length - 7) !== 112 /* 'p' */ ||
            s.charCodeAt(length - 8) !== 95 /* '_' */ ||
            s.charCodeAt(length - 9) !== 95 /* '_' */) {
            return false;
        }
        for (var i = length - 10; i >= 0; i--) {
            if (s.charCodeAt(i) !== 36 /* '$' */) {
                return false;
            }
        }
        return true;
    }
    /**
     * Comparator between two mappings where the original positions are compared.
     *
     * Optionally pass in `true` as `onlyCompareGenerated` to consider two
     * mappings with the same original source/line/column, but different generated
     * line and column the same. Useful when searching for a mapping with a
     * stubbed out mapping.
     */
    function compareByOriginalPositions(mappingA, mappingB, onlyCompareOriginal) {
        var cmp = mappingA.source - mappingB.source;
        if (cmp !== 0) {
            return cmp;
        }
        cmp = mappingA.originalLine - mappingB.originalLine;
        if (cmp !== 0) {
            return cmp;
        }
        cmp = mappingA.originalColumn - mappingB.originalColumn;
        if (cmp !== 0 || onlyCompareOriginal) {
            return cmp;
        }
        cmp = mappingA.generatedColumn - mappingB.generatedColumn;
        if (cmp !== 0) {
            return cmp;
        }
        cmp = mappingA.generatedLine - mappingB.generatedLine;
        if (cmp !== 0) {
            return cmp;
        }
        return mappingA.name - mappingB.name;
    }
    exports.compareByOriginalPositions = compareByOriginalPositions;
    /**
     * Comparator between two mappings with deflated source and name indices where
     * the generated positions are compared.
     *
     * Optionally pass in `true` as `onlyCompareGenerated` to consider two
     * mappings with the same generated line and column, but different
     * source/name/original line and column the same. Useful when searching for a
     * mapping with a stubbed out mapping.
     */
    function compareByGeneratedPositionsDeflated(mappingA, mappingB, onlyCompareGenerated) {
        var cmp = mappingA.generatedLine - mappingB.generatedLine;
        if (cmp !== 0) {
            return cmp;
        }
        cmp = mappingA.generatedColumn - mappingB.generatedColumn;
        if (cmp !== 0 || onlyCompareGenerated) {
            return cmp;
        }
        cmp = mappingA.source - mappingB.source;
        if (cmp !== 0) {
            return cmp;
        }
        cmp = mappingA.originalLine - mappingB.originalLine;
        if (cmp !== 0) {
            return cmp;
        }
        cmp = mappingA.originalColumn - mappingB.originalColumn;
        if (cmp !== 0) {
            return cmp;
        }
        return mappingA.name - mappingB.name;
    }
    exports.compareByGeneratedPositionsDeflated = compareByGeneratedPositionsDeflated;
    function strcmp(aStr1, aStr2) {
        if (aStr1 === aStr2) {
            return 0;
        }
        if (aStr1 > aStr2) {
            return 1;
        }
        return -1;
    }
    /**
     * Comparator between two mappings with inflated source and name strings where
     * the generated positions are compared.
     */
    function compareByGeneratedPositionsInflated(mappingA, mappingB) {
        var cmp = mappingA.generatedLine - mappingB.generatedLine;
        if (cmp !== 0) {
            return cmp;
        }
        cmp = mappingA.generatedColumn - mappingB.generatedColumn;
        if (cmp !== 0) {
            return cmp;
        }
        cmp = strcmp(mappingA.source, mappingB.source);
        if (cmp !== 0) {
            return cmp;
        }
        cmp = mappingA.originalLine - mappingB.originalLine;
        if (cmp !== 0) {
            return cmp;
        }
        cmp = mappingA.originalColumn - mappingB.originalColumn;
        if (cmp !== 0) {
            return cmp;
        }
        return strcmp(mappingA.name, mappingB.name);
    }
    exports.compareByGeneratedPositionsInflated = compareByGeneratedPositionsInflated;
},
/* source-map/lib/array-set.js */ function _(require, module, exports) {
    /* -*- Mode: js; js-indent-level: 2; -*- */
    /*
     * Copyright 2011 Mozilla Foundation and contributors
     * Licensed under the New BSD license. See LICENSE or:
     * http://opensource.org/licenses/BSD-3-Clause
     */
    var util = require(50) /* ./util */;
    var has = Object.prototype.hasOwnProperty;
    var hasNativeMap = typeof Map !== "undefined";
    /**
     * A data structure which is a combination of an array and a set. Adding a new
     * member is O(1), testing for membership is O(1), and finding the index of an
     * element is O(1). Removing elements from the set is not supported. Only
     * strings are supported for membership.
     */
    function ArraySet() {
        this._array = [];
        this._set = hasNativeMap ? new Map() : Object.create(null);
    }
    /**
     * Static method for creating ArraySet instances from an existing array.
     */
    ArraySet.fromArray = function ArraySet_fromArray(aArray, aAllowDuplicates) {
        var set = new ArraySet();
        for (var i = 0, len = aArray.length; i < len; i++) {
            set.add(aArray[i], aAllowDuplicates);
        }
        return set;
    };
    /**
     * Return how many unique items are in this ArraySet. If duplicates have been
     * added, than those do not count towards the size.
     *
     * @returns Number
     */
    ArraySet.prototype.size = function ArraySet_size() {
        return hasNativeMap ? this._set.size : Object.getOwnPropertyNames(this._set).length;
    };
    /**
     * Add the given string to this set.
     *
     * @param String aStr
     */
    ArraySet.prototype.add = function ArraySet_add(aStr, aAllowDuplicates) {
        var sStr = hasNativeMap ? aStr : util.toSetString(aStr);
        var isDuplicate = hasNativeMap ? this.has(aStr) : has.call(this._set, sStr);
        var idx = this._array.length;
        if (!isDuplicate || aAllowDuplicates) {
            this._array.push(aStr);
        }
        if (!isDuplicate) {
            if (hasNativeMap) {
                this._set.set(aStr, idx);
            }
            else {
                this._set[sStr] = idx;
            }
        }
    };
    /**
     * Is the given string a member of this set?
     *
     * @param String aStr
     */
    ArraySet.prototype.has = function ArraySet_has(aStr) {
        if (hasNativeMap) {
            return this._set.has(aStr);
        }
        else {
            var sStr = util.toSetString(aStr);
            return has.call(this._set, sStr);
        }
    };
    /**
     * What is the index of the given string in the array?
     *
     * @param String aStr
     */
    ArraySet.prototype.indexOf = function ArraySet_indexOf(aStr) {
        if (hasNativeMap) {
            var idx = this._set.get(aStr);
            if (idx >= 0) {
                return idx;
            }
        }
        else {
            var sStr = util.toSetString(aStr);
            if (has.call(this._set, sStr)) {
                return this._set[sStr];
            }
        }
        throw new Error('"' + aStr + '" is not in the set.');
    };
    /**
     * What is the element at the given index?
     *
     * @param Number aIdx
     */
    ArraySet.prototype.at = function ArraySet_at(aIdx) {
        if (aIdx >= 0 && aIdx < this._array.length) {
            return this._array[aIdx];
        }
        throw new Error('No element indexed by ' + aIdx);
    };
    /**
     * Returns the array representation of this set (which has the proper indices
     * indicated by indexOf). Note that this is a copy of the internal array used
     * for storing the members so that no one can mess with internal state.
     */
    ArraySet.prototype.toArray = function ArraySet_toArray() {
        return this._array.slice();
    };
    exports.ArraySet = ArraySet;
},
/* source-map/lib/mapping-list.js */ function _(require, module, exports) {
    /* -*- Mode: js; js-indent-level: 2; -*- */
    /*
     * Copyright 2014 Mozilla Foundation and contributors
     * Licensed under the New BSD license. See LICENSE or:
     * http://opensource.org/licenses/BSD-3-Clause
     */
    var util = require(50) /* ./util */;
    /**
     * Determine whether mappingB is after mappingA with respect to generated
     * position.
     */
    function generatedPositionAfter(mappingA, mappingB) {
        // Optimized for most common case
        var lineA = mappingA.generatedLine;
        var lineB = mappingB.generatedLine;
        var columnA = mappingA.generatedColumn;
        var columnB = mappingB.generatedColumn;
        return lineB > lineA || lineB == lineA && columnB >= columnA ||
            util.compareByGeneratedPositionsInflated(mappingA, mappingB) <= 0;
    }
    /**
     * A data structure to provide a sorted view of accumulated mappings in a
     * performance conscious manner. It trades a neglibable overhead in general
     * case for a large speedup in case of mappings being added in order.
     */
    function MappingList() {
        this._array = [];
        this._sorted = true;
        // Serves as infimum
        this._last = { generatedLine: -1, generatedColumn: 0 };
    }
    /**
     * Iterate through internal items. This method takes the same arguments that
     * `Array.prototype.forEach` takes.
     *
     * NOTE: The order of the mappings is NOT guaranteed.
     */
    MappingList.prototype.unsortedForEach =
        function MappingList_forEach(aCallback, aThisArg) {
            this._array.forEach(aCallback, aThisArg);
        };
    /**
     * Add the given source mapping.
     *
     * @param Object aMapping
     */
    MappingList.prototype.add = function MappingList_add(aMapping) {
        if (generatedPositionAfter(this._last, aMapping)) {
            this._last = aMapping;
            this._array.push(aMapping);
        }
        else {
            this._sorted = false;
            this._array.push(aMapping);
        }
    };
    /**
     * Returns the flat, sorted array of mappings. The mappings are sorted by
     * generated position.
     *
     * WARNING: This method returns internal data without copying, for
     * performance. The return value must NOT be mutated, and should be treated as
     * an immutable borrow. If you want to take ownership, you must make your own
     * copy.
     */
    MappingList.prototype.toArray = function MappingList_toArray() {
        if (!this._sorted) {
            this._array.sort(util.compareByGeneratedPositionsInflated);
            this._sorted = true;
        }
        return this._array;
    };
    exports.MappingList = MappingList;
},
/* source-map/lib/source-map-consumer.js */ function _(require, module, exports) {
    /* -*- Mode: js; js-indent-level: 2; -*- */
    /*
     * Copyright 2011 Mozilla Foundation and contributors
     * Licensed under the New BSD license. See LICENSE or:
     * http://opensource.org/licenses/BSD-3-Clause
     */
    var util = require(50) /* ./util */;
    var binarySearch = require(54) /* ./binary-search */;
    var ArraySet = require(51) /* ./array-set */.ArraySet;
    var base64VLQ = require(48) /* ./base64-vlq */;
    var quickSort = require(55) /* ./quick-sort */.quickSort;
    function SourceMapConsumer(aSourceMap) {
        var sourceMap = aSourceMap;
        if (typeof aSourceMap === 'string') {
            sourceMap = JSON.parse(aSourceMap.replace(/^\)\]\}'/, ''));
        }
        return sourceMap.sections != null
            ? new IndexedSourceMapConsumer(sourceMap)
            : new BasicSourceMapConsumer(sourceMap);
    }
    SourceMapConsumer.fromSourceMap = function (aSourceMap) {
        return BasicSourceMapConsumer.fromSourceMap(aSourceMap);
    };
    /**
     * The version of the source mapping spec that we are consuming.
     */
    SourceMapConsumer.prototype._version = 3;
    // `__generatedMappings` and `__originalMappings` are arrays that hold the
    // parsed mapping coordinates from the source map's "mappings" attribute. They
    // are lazily instantiated, accessed via the `_generatedMappings` and
    // `_originalMappings` getters respectively, and we only parse the mappings
    // and create these arrays once queried for a source location. We jump through
    // these hoops because there can be many thousands of mappings, and parsing
    // them is expensive, so we only want to do it if we must.
    //
    // Each object in the arrays is of the form:
    //
    //     {
    //       generatedLine: The line number in the generated code,
    //       generatedColumn: The column number in the generated code,
    //       source: The path to the original source file that generated this
    //               chunk of code,
    //       originalLine: The line number in the original source that
    //                     corresponds to this chunk of generated code,
    //       originalColumn: The column number in the original source that
    //                       corresponds to this chunk of generated code,
    //       name: The name of the original symbol which generated this chunk of
    //             code.
    //     }
    //
    // All properties except for `generatedLine` and `generatedColumn` can be
    // `null`.
    //
    // `_generatedMappings` is ordered by the generated positions.
    //
    // `_originalMappings` is ordered by the original positions.
    SourceMapConsumer.prototype.__generatedMappings = null;
    Object.defineProperty(SourceMapConsumer.prototype, '_generatedMappings', {
        get: function () {
            if (!this.__generatedMappings) {
                this._parseMappings(this._mappings, this.sourceRoot);
            }
            return this.__generatedMappings;
        }
    });
    SourceMapConsumer.prototype.__originalMappings = null;
    Object.defineProperty(SourceMapConsumer.prototype, '_originalMappings', {
        get: function () {
            if (!this.__originalMappings) {
                this._parseMappings(this._mappings, this.sourceRoot);
            }
            return this.__originalMappings;
        }
    });
    SourceMapConsumer.prototype._charIsMappingSeparator =
        function SourceMapConsumer_charIsMappingSeparator(aStr, index) {
            var c = aStr.charAt(index);
            return c === ";" || c === ",";
        };
    /**
     * Parse the mappings in a string in to a data structure which we can easily
     * query (the ordered arrays in the `this.__generatedMappings` and
     * `this.__originalMappings` properties).
     */
    SourceMapConsumer.prototype._parseMappings =
        function SourceMapConsumer_parseMappings(aStr, aSourceRoot) {
            throw new Error("Subclasses must implement _parseMappings");
        };
    SourceMapConsumer.GENERATED_ORDER = 1;
    SourceMapConsumer.ORIGINAL_ORDER = 2;
    SourceMapConsumer.GREATEST_LOWER_BOUND = 1;
    SourceMapConsumer.LEAST_UPPER_BOUND = 2;
    /**
     * Iterate over each mapping between an original source/line/column and a
     * generated line/column in this source map.
     *
     * @param Function aCallback
     *        The function that is called with each mapping.
     * @param Object aContext
     *        Optional. If specified, this object will be the value of `this` every
     *        time that `aCallback` is called.
     * @param aOrder
     *        Either `SourceMapConsumer.GENERATED_ORDER` or
     *        `SourceMapConsumer.ORIGINAL_ORDER`. Specifies whether you want to
     *        iterate over the mappings sorted by the generated file's line/column
     *        order or the original's source/line/column order, respectively. Defaults to
     *        `SourceMapConsumer.GENERATED_ORDER`.
     */
    SourceMapConsumer.prototype.eachMapping =
        function SourceMapConsumer_eachMapping(aCallback, aContext, aOrder) {
            var context = aContext || null;
            var order = aOrder || SourceMapConsumer.GENERATED_ORDER;
            var mappings;
            switch (order) {
                case SourceMapConsumer.GENERATED_ORDER:
                    mappings = this._generatedMappings;
                    break;
                case SourceMapConsumer.ORIGINAL_ORDER:
                    mappings = this._originalMappings;
                    break;
                default:
                    throw new Error("Unknown order of iteration.");
            }
            var sourceRoot = this.sourceRoot;
            mappings.map(function (mapping) {
                var source = mapping.source === null ? null : this._sources.at(mapping.source);
                if (source != null && sourceRoot != null) {
                    source = util.join(sourceRoot, source);
                }
                return {
                    source: source,
                    generatedLine: mapping.generatedLine,
                    generatedColumn: mapping.generatedColumn,
                    originalLine: mapping.originalLine,
                    originalColumn: mapping.originalColumn,
                    name: mapping.name === null ? null : this._names.at(mapping.name)
                };
            }, this).forEach(aCallback, context);
        };
    /**
     * Returns all generated line and column information for the original source,
     * line, and column provided. If no column is provided, returns all mappings
     * corresponding to a either the line we are searching for or the next
     * closest line that has any mappings. Otherwise, returns all mappings
     * corresponding to the given line and either the column we are searching for
     * or the next closest column that has any offsets.
     *
     * The only argument is an object with the following properties:
     *
     *   - source: The filename of the original source.
     *   - line: The line number in the original source.
     *   - column: Optional. the column number in the original source.
     *
     * and an array of objects is returned, each with the following properties:
     *
     *   - line: The line number in the generated source, or null.
     *   - column: The column number in the generated source, or null.
     */
    SourceMapConsumer.prototype.allGeneratedPositionsFor =
        function SourceMapConsumer_allGeneratedPositionsFor(aArgs) {
            var line = util.getArg(aArgs, 'line');
            // When there is no exact match, BasicSourceMapConsumer.prototype._findMapping
            // returns the index of the closest mapping less than the needle. By
            // setting needle.originalColumn to 0, we thus find the last mapping for
            // the given line, provided such a mapping exists.
            var needle = {
                source: util.getArg(aArgs, 'source'),
                originalLine: line,
                originalColumn: util.getArg(aArgs, 'column', 0)
            };
            if (this.sourceRoot != null) {
                needle.source = util.relative(this.sourceRoot, needle.source);
            }
            if (!this._sources.has(needle.source)) {
                return [];
            }
            needle.source = this._sources.indexOf(needle.source);
            var mappings = [];
            var index = this._findMapping(needle, this._originalMappings, "originalLine", "originalColumn", util.compareByOriginalPositions, binarySearch.LEAST_UPPER_BOUND);
            if (index >= 0) {
                var mapping = this._originalMappings[index];
                if (aArgs.column === undefined) {
                    var originalLine = mapping.originalLine;
                    // Iterate until either we run out of mappings, or we run into
                    // a mapping for a different line than the one we found. Since
                    // mappings are sorted, this is guaranteed to find all mappings for
                    // the line we found.
                    while (mapping && mapping.originalLine === originalLine) {
                        mappings.push({
                            line: util.getArg(mapping, 'generatedLine', null),
                            column: util.getArg(mapping, 'generatedColumn', null),
                            lastColumn: util.getArg(mapping, 'lastGeneratedColumn', null)
                        });
                        mapping = this._originalMappings[++index];
                    }
                }
                else {
                    var originalColumn = mapping.originalColumn;
                    // Iterate until either we run out of mappings, or we run into
                    // a mapping for a different line than the one we were searching for.
                    // Since mappings are sorted, this is guaranteed to find all mappings for
                    // the line we are searching for.
                    while (mapping &&
                        mapping.originalLine === line &&
                        mapping.originalColumn == originalColumn) {
                        mappings.push({
                            line: util.getArg(mapping, 'generatedLine', null),
                            column: util.getArg(mapping, 'generatedColumn', null),
                            lastColumn: util.getArg(mapping, 'lastGeneratedColumn', null)
                        });
                        mapping = this._originalMappings[++index];
                    }
                }
            }
            return mappings;
        };
    exports.SourceMapConsumer = SourceMapConsumer;
    /**
     * A BasicSourceMapConsumer instance represents a parsed source map which we can
     * query for information about the original file positions by giving it a file
     * position in the generated source.
     *
     * The only parameter is the raw source map (either as a JSON string, or
     * already parsed to an object). According to the spec, source maps have the
     * following attributes:
     *
     *   - version: Which version of the source map spec this map is following.
     *   - sources: An array of URLs to the original source files.
     *   - names: An array of identifiers which can be referrenced by individual mappings.
     *   - sourceRoot: Optional. The URL root from which all sources are relative.
     *   - sourcesContent: Optional. An array of contents of the original source files.
     *   - mappings: A string of base64 VLQs which contain the actual mappings.
     *   - file: Optional. The generated file this source map is associated with.
     *
     * Here is an example source map, taken from the source map spec[0]:
     *
     *     {
     *       version : 3,
     *       file: "out.js",
     *       sourceRoot : "",
     *       sources: ["foo.js", "bar.js"],
     *       names: ["src", "maps", "are", "fun"],
     *       mappings: "AA,AB;;ABCDE;"
     *     }
     *
     * [0]: https://docs.google.com/document/d/1U1RGAehQwRypUTovF1KRlpiOFze0b-_2gc6fAH0KY0k/edit?pli=1#
     */
    function BasicSourceMapConsumer(aSourceMap) {
        var sourceMap = aSourceMap;
        if (typeof aSourceMap === 'string') {
            sourceMap = JSON.parse(aSourceMap.replace(/^\)\]\}'/, ''));
        }
        var version = util.getArg(sourceMap, 'version');
        var sources = util.getArg(sourceMap, 'sources');
        // Sass 3.3 leaves out the 'names' array, so we deviate from the spec (which
        // requires the array) to play nice here.
        var names = util.getArg(sourceMap, 'names', []);
        var sourceRoot = util.getArg(sourceMap, 'sourceRoot', null);
        var sourcesContent = util.getArg(sourceMap, 'sourcesContent', null);
        var mappings = util.getArg(sourceMap, 'mappings');
        var file = util.getArg(sourceMap, 'file', null);
        // Once again, Sass deviates from the spec and supplies the version as a
        // string rather than a number, so we use loose equality checking here.
        if (version != this._version) {
            throw new Error('Unsupported version: ' + version);
        }
        sources = sources
            .map(String)
            // Some source maps produce relative source paths like "./foo.js" instead of
            // "foo.js".  Normalize these first so that future comparisons will succeed.
            // See bugzil.la/1090768.
            .map(util.normalize)
            // Always ensure that absolute sources are internally stored relative to
            // the source root, if the source root is absolute. Not doing this would
            // be particularly problematic when the source root is a prefix of the
            // source (valid, but why??). See github issue #199 and bugzil.la/1188982.
            .map(function (source) {
            return sourceRoot && util.isAbsolute(sourceRoot) && util.isAbsolute(source)
                ? util.relative(sourceRoot, source)
                : source;
        });
        // Pass `true` below to allow duplicate names and sources. While source maps
        // are intended to be compressed and deduplicated, the TypeScript compiler
        // sometimes generates source maps with duplicates in them. See Github issue
        // #72 and bugzil.la/889492.
        this._names = ArraySet.fromArray(names.map(String), true);
        this._sources = ArraySet.fromArray(sources, true);
        this.sourceRoot = sourceRoot;
        this.sourcesContent = sourcesContent;
        this._mappings = mappings;
        this.file = file;
    }
    BasicSourceMapConsumer.prototype = Object.create(SourceMapConsumer.prototype);
    BasicSourceMapConsumer.prototype.consumer = SourceMapConsumer;
    /**
     * Create a BasicSourceMapConsumer from a SourceMapGenerator.
     *
     * @param SourceMapGenerator aSourceMap
     *        The source map that will be consumed.
     * @returns BasicSourceMapConsumer
     */
    BasicSourceMapConsumer.fromSourceMap =
        function SourceMapConsumer_fromSourceMap(aSourceMap) {
            var smc = Object.create(BasicSourceMapConsumer.prototype);
            var names = smc._names = ArraySet.fromArray(aSourceMap._names.toArray(), true);
            var sources = smc._sources = ArraySet.fromArray(aSourceMap._sources.toArray(), true);
            smc.sourceRoot = aSourceMap._sourceRoot;
            smc.sourcesContent = aSourceMap._generateSourcesContent(smc._sources.toArray(), smc.sourceRoot);
            smc.file = aSourceMap._file;
            // Because we are modifying the entries (by converting string sources and
            // names to indices into the sources and names ArraySets), we have to make
            // a copy of the entry or else bad things happen. Shared mutable state
            // strikes again! See github issue #191.
            var generatedMappings = aSourceMap._mappings.toArray().slice();
            var destGeneratedMappings = smc.__generatedMappings = [];
            var destOriginalMappings = smc.__originalMappings = [];
            for (var i = 0, length = generatedMappings.length; i < length; i++) {
                var srcMapping = generatedMappings[i];
                var destMapping = new Mapping;
                destMapping.generatedLine = srcMapping.generatedLine;
                destMapping.generatedColumn = srcMapping.generatedColumn;
                if (srcMapping.source) {
                    destMapping.source = sources.indexOf(srcMapping.source);
                    destMapping.originalLine = srcMapping.originalLine;
                    destMapping.originalColumn = srcMapping.originalColumn;
                    if (srcMapping.name) {
                        destMapping.name = names.indexOf(srcMapping.name);
                    }
                    destOriginalMappings.push(destMapping);
                }
                destGeneratedMappings.push(destMapping);
            }
            quickSort(smc.__originalMappings, util.compareByOriginalPositions);
            return smc;
        };
    /**
     * The version of the source mapping spec that we are consuming.
     */
    BasicSourceMapConsumer.prototype._version = 3;
    /**
     * The list of original sources.
     */
    Object.defineProperty(BasicSourceMapConsumer.prototype, 'sources', {
        get: function () {
            return this._sources.toArray().map(function (s) {
                return this.sourceRoot != null ? util.join(this.sourceRoot, s) : s;
            }, this);
        }
    });
    /**
     * Provide the JIT with a nice shape / hidden class.
     */
    function Mapping() {
        this.generatedLine = 0;
        this.generatedColumn = 0;
        this.source = null;
        this.originalLine = null;
        this.originalColumn = null;
        this.name = null;
    }
    /**
     * Parse the mappings in a string in to a data structure which we can easily
     * query (the ordered arrays in the `this.__generatedMappings` and
     * `this.__originalMappings` properties).
     */
    BasicSourceMapConsumer.prototype._parseMappings =
        function SourceMapConsumer_parseMappings(aStr, aSourceRoot) {
            var generatedLine = 1;
            var previousGeneratedColumn = 0;
            var previousOriginalLine = 0;
            var previousOriginalColumn = 0;
            var previousSource = 0;
            var previousName = 0;
            var length = aStr.length;
            var index = 0;
            var cachedSegments = {};
            var temp = {};
            var originalMappings = [];
            var generatedMappings = [];
            var mapping, str, segment, end, value;
            while (index < length) {
                if (aStr.charAt(index) === ';') {
                    generatedLine++;
                    index++;
                    previousGeneratedColumn = 0;
                }
                else if (aStr.charAt(index) === ',') {
                    index++;
                }
                else {
                    mapping = new Mapping();
                    mapping.generatedLine = generatedLine;
                    // Because each offset is encoded relative to the previous one,
                    // many segments often have the same encoding. We can exploit this
                    // fact by caching the parsed variable length fields of each segment,
                    // allowing us to avoid a second parse if we encounter the same
                    // segment again.
                    for (end = index; end < length; end++) {
                        if (this._charIsMappingSeparator(aStr, end)) {
                            break;
                        }
                    }
                    str = aStr.slice(index, end);
                    segment = cachedSegments[str];
                    if (segment) {
                        index += str.length;
                    }
                    else {
                        segment = [];
                        while (index < end) {
                            base64VLQ.decode(aStr, index, temp);
                            value = temp.value;
                            index = temp.rest;
                            segment.push(value);
                        }
                        if (segment.length === 2) {
                            throw new Error('Found a source, but no line and column');
                        }
                        if (segment.length === 3) {
                            throw new Error('Found a source and line, but no column');
                        }
                        cachedSegments[str] = segment;
                    }
                    // Generated column.
                    mapping.generatedColumn = previousGeneratedColumn + segment[0];
                    previousGeneratedColumn = mapping.generatedColumn;
                    if (segment.length > 1) {
                        // Original source.
                        mapping.source = previousSource + segment[1];
                        previousSource += segment[1];
                        // Original line.
                        mapping.originalLine = previousOriginalLine + segment[2];
                        previousOriginalLine = mapping.originalLine;
                        // Lines are stored 0-based
                        mapping.originalLine += 1;
                        // Original column.
                        mapping.originalColumn = previousOriginalColumn + segment[3];
                        previousOriginalColumn = mapping.originalColumn;
                        if (segment.length > 4) {
                            // Original name.
                            mapping.name = previousName + segment[4];
                            previousName += segment[4];
                        }
                    }
                    generatedMappings.push(mapping);
                    if (typeof mapping.originalLine === 'number') {
                        originalMappings.push(mapping);
                    }
                }
            }
            quickSort(generatedMappings, util.compareByGeneratedPositionsDeflated);
            this.__generatedMappings = generatedMappings;
            quickSort(originalMappings, util.compareByOriginalPositions);
            this.__originalMappings = originalMappings;
        };
    /**
     * Find the mapping that best matches the hypothetical "needle" mapping that
     * we are searching for in the given "haystack" of mappings.
     */
    BasicSourceMapConsumer.prototype._findMapping =
        function SourceMapConsumer_findMapping(aNeedle, aMappings, aLineName, aColumnName, aComparator, aBias) {
            // To return the position we are searching for, we must first find the
            // mapping for the given position and then return the opposite position it
            // points to. Because the mappings are sorted, we can use binary search to
            // find the best mapping.
            if (aNeedle[aLineName] <= 0) {
                throw new TypeError('Line must be greater than or equal to 1, got '
                    + aNeedle[aLineName]);
            }
            if (aNeedle[aColumnName] < 0) {
                throw new TypeError('Column must be greater than or equal to 0, got '
                    + aNeedle[aColumnName]);
            }
            return binarySearch.search(aNeedle, aMappings, aComparator, aBias);
        };
    /**
     * Compute the last column for each generated mapping. The last column is
     * inclusive.
     */
    BasicSourceMapConsumer.prototype.computeColumnSpans =
        function SourceMapConsumer_computeColumnSpans() {
            for (var index = 0; index < this._generatedMappings.length; ++index) {
                var mapping = this._generatedMappings[index];
                // Mappings do not contain a field for the last generated columnt. We
                // can come up with an optimistic estimate, however, by assuming that
                // mappings are contiguous (i.e. given two consecutive mappings, the
                // first mapping ends where the second one starts).
                if (index + 1 < this._generatedMappings.length) {
                    var nextMapping = this._generatedMappings[index + 1];
                    if (mapping.generatedLine === nextMapping.generatedLine) {
                        mapping.lastGeneratedColumn = nextMapping.generatedColumn - 1;
                        continue;
                    }
                }
                // The last mapping for each line spans the entire line.
                mapping.lastGeneratedColumn = Infinity;
            }
        };
    /**
     * Returns the original source, line, and column information for the generated
     * source's line and column positions provided. The only argument is an object
     * with the following properties:
     *
     *   - line: The line number in the generated source.
     *   - column: The column number in the generated source.
     *   - bias: Either 'SourceMapConsumer.GREATEST_LOWER_BOUND' or
     *     'SourceMapConsumer.LEAST_UPPER_BOUND'. Specifies whether to return the
     *     closest element that is smaller than or greater than the one we are
     *     searching for, respectively, if the exact element cannot be found.
     *     Defaults to 'SourceMapConsumer.GREATEST_LOWER_BOUND'.
     *
     * and an object is returned with the following properties:
     *
     *   - source: The original source file, or null.
     *   - line: The line number in the original source, or null.
     *   - column: The column number in the original source, or null.
     *   - name: The original identifier, or null.
     */
    BasicSourceMapConsumer.prototype.originalPositionFor =
        function SourceMapConsumer_originalPositionFor(aArgs) {
            var needle = {
                generatedLine: util.getArg(aArgs, 'line'),
                generatedColumn: util.getArg(aArgs, 'column')
            };
            var index = this._findMapping(needle, this._generatedMappings, "generatedLine", "generatedColumn", util.compareByGeneratedPositionsDeflated, util.getArg(aArgs, 'bias', SourceMapConsumer.GREATEST_LOWER_BOUND));
            if (index >= 0) {
                var mapping = this._generatedMappings[index];
                if (mapping.generatedLine === needle.generatedLine) {
                    var source = util.getArg(mapping, 'source', null);
                    if (source !== null) {
                        source = this._sources.at(source);
                        if (this.sourceRoot != null) {
                            source = util.join(this.sourceRoot, source);
                        }
                    }
                    var name = util.getArg(mapping, 'name', null);
                    if (name !== null) {
                        name = this._names.at(name);
                    }
                    return {
                        source: source,
                        line: util.getArg(mapping, 'originalLine', null),
                        column: util.getArg(mapping, 'originalColumn', null),
                        name: name
                    };
                }
            }
            return {
                source: null,
                line: null,
                column: null,
                name: null
            };
        };
    /**
     * Return true if we have the source content for every source in the source
     * map, false otherwise.
     */
    BasicSourceMapConsumer.prototype.hasContentsOfAllSources =
        function BasicSourceMapConsumer_hasContentsOfAllSources() {
            if (!this.sourcesContent) {
                return false;
            }
            return this.sourcesContent.length >= this._sources.size() &&
                !this.sourcesContent.some(function (sc) { return sc == null; });
        };
    /**
     * Returns the original source content. The only argument is the url of the
     * original source file. Returns null if no original source content is
     * available.
     */
    BasicSourceMapConsumer.prototype.sourceContentFor =
        function SourceMapConsumer_sourceContentFor(aSource, nullOnMissing) {
            if (!this.sourcesContent) {
                return null;
            }
            if (this.sourceRoot != null) {
                aSource = util.relative(this.sourceRoot, aSource);
            }
            if (this._sources.has(aSource)) {
                return this.sourcesContent[this._sources.indexOf(aSource)];
            }
            var url;
            if (this.sourceRoot != null
                && (url = util.urlParse(this.sourceRoot))) {
                // XXX: file:// URIs and absolute paths lead to unexpected behavior for
                // many users. We can help them out when they expect file:// URIs to
                // behave like it would if they were running a local HTTP server. See
                // https://bugzilla.mozilla.org/show_bug.cgi?id=885597.
                var fileUriAbsPath = aSource.replace(/^file:\/\//, "");
                if (url.scheme == "file"
                    && this._sources.has(fileUriAbsPath)) {
                    return this.sourcesContent[this._sources.indexOf(fileUriAbsPath)];
                }
                if ((!url.path || url.path == "/")
                    && this._sources.has("/" + aSource)) {
                    return this.sourcesContent[this._sources.indexOf("/" + aSource)];
                }
            }
            // This function is used recursively from
            // IndexedSourceMapConsumer.prototype.sourceContentFor. In that case, we
            // don't want to throw if we can't find the source - we just want to
            // return null, so we provide a flag to exit gracefully.
            if (nullOnMissing) {
                return null;
            }
            else {
                throw new Error('"' + aSource + '" is not in the SourceMap.');
            }
        };
    /**
     * Returns the generated line and column information for the original source,
     * line, and column positions provided. The only argument is an object with
     * the following properties:
     *
     *   - source: The filename of the original source.
     *   - line: The line number in the original source.
     *   - column: The column number in the original source.
     *   - bias: Either 'SourceMapConsumer.GREATEST_LOWER_BOUND' or
     *     'SourceMapConsumer.LEAST_UPPER_BOUND'. Specifies whether to return the
     *     closest element that is smaller than or greater than the one we are
     *     searching for, respectively, if the exact element cannot be found.
     *     Defaults to 'SourceMapConsumer.GREATEST_LOWER_BOUND'.
     *
     * and an object is returned with the following properties:
     *
     *   - line: The line number in the generated source, or null.
     *   - column: The column number in the generated source, or null.
     */
    BasicSourceMapConsumer.prototype.generatedPositionFor =
        function SourceMapConsumer_generatedPositionFor(aArgs) {
            var source = util.getArg(aArgs, 'source');
            if (this.sourceRoot != null) {
                source = util.relative(this.sourceRoot, source);
            }
            if (!this._sources.has(source)) {
                return {
                    line: null,
                    column: null,
                    lastColumn: null
                };
            }
            source = this._sources.indexOf(source);
            var needle = {
                source: source,
                originalLine: util.getArg(aArgs, 'line'),
                originalColumn: util.getArg(aArgs, 'column')
            };
            var index = this._findMapping(needle, this._originalMappings, "originalLine", "originalColumn", util.compareByOriginalPositions, util.getArg(aArgs, 'bias', SourceMapConsumer.GREATEST_LOWER_BOUND));
            if (index >= 0) {
                var mapping = this._originalMappings[index];
                if (mapping.source === needle.source) {
                    return {
                        line: util.getArg(mapping, 'generatedLine', null),
                        column: util.getArg(mapping, 'generatedColumn', null),
                        lastColumn: util.getArg(mapping, 'lastGeneratedColumn', null)
                    };
                }
            }
            return {
                line: null,
                column: null,
                lastColumn: null
            };
        };
    exports.BasicSourceMapConsumer = BasicSourceMapConsumer;
    /**
     * An IndexedSourceMapConsumer instance represents a parsed source map which
     * we can query for information. It differs from BasicSourceMapConsumer in
     * that it takes "indexed" source maps (i.e. ones with a "sections" field) as
     * input.
     *
     * The only parameter is a raw source map (either as a JSON string, or already
     * parsed to an object). According to the spec for indexed source maps, they
     * have the following attributes:
     *
     *   - version: Which version of the source map spec this map is following.
     *   - file: Optional. The generated file this source map is associated with.
     *   - sections: A list of section definitions.
     *
     * Each value under the "sections" field has two fields:
     *   - offset: The offset into the original specified at which this section
     *       begins to apply, defined as an object with a "line" and "column"
     *       field.
     *   - map: A source map definition. This source map could also be indexed,
     *       but doesn't have to be.
     *
     * Instead of the "map" field, it's also possible to have a "url" field
     * specifying a URL to retrieve a source map from, but that's currently
     * unsupported.
     *
     * Here's an example source map, taken from the source map spec[0], but
     * modified to omit a section which uses the "url" field.
     *
     *  {
     *    version : 3,
     *    file: "app.js",
     *    sections: [{
     *      offset: {line:100, column:10},
     *      map: {
     *        version : 3,
     *        file: "section.js",
     *        sources: ["foo.js", "bar.js"],
     *        names: ["src", "maps", "are", "fun"],
     *        mappings: "AAAA,E;;ABCDE;"
     *      }
     *    }],
     *  }
     *
     * [0]: https://docs.google.com/document/d/1U1RGAehQwRypUTovF1KRlpiOFze0b-_2gc6fAH0KY0k/edit#heading=h.535es3xeprgt
     */
    function IndexedSourceMapConsumer(aSourceMap) {
        var sourceMap = aSourceMap;
        if (typeof aSourceMap === 'string') {
            sourceMap = JSON.parse(aSourceMap.replace(/^\)\]\}'/, ''));
        }
        var version = util.getArg(sourceMap, 'version');
        var sections = util.getArg(sourceMap, 'sections');
        if (version != this._version) {
            throw new Error('Unsupported version: ' + version);
        }
        this._sources = new ArraySet();
        this._names = new ArraySet();
        var lastOffset = {
            line: -1,
            column: 0
        };
        this._sections = sections.map(function (s) {
            if (s.url) {
                // The url field will require support for asynchronicity.
                // See https://github.com/mozilla/source-map/issues/16
                throw new Error('Support for url field in sections not implemented.');
            }
            var offset = util.getArg(s, 'offset');
            var offsetLine = util.getArg(offset, 'line');
            var offsetColumn = util.getArg(offset, 'column');
            if (offsetLine < lastOffset.line ||
                (offsetLine === lastOffset.line && offsetColumn < lastOffset.column)) {
                throw new Error('Section offsets must be ordered and non-overlapping.');
            }
            lastOffset = offset;
            return {
                generatedOffset: {
                    // The offset fields are 0-based, but we use 1-based indices when
                    // encoding/decoding from VLQ.
                    generatedLine: offsetLine + 1,
                    generatedColumn: offsetColumn + 1
                },
                consumer: new SourceMapConsumer(util.getArg(s, 'map'))
            };
        });
    }
    IndexedSourceMapConsumer.prototype = Object.create(SourceMapConsumer.prototype);
    IndexedSourceMapConsumer.prototype.constructor = SourceMapConsumer;
    /**
     * The version of the source mapping spec that we are consuming.
     */
    IndexedSourceMapConsumer.prototype._version = 3;
    /**
     * The list of original sources.
     */
    Object.defineProperty(IndexedSourceMapConsumer.prototype, 'sources', {
        get: function () {
            var sources = [];
            for (var i = 0; i < this._sections.length; i++) {
                for (var j = 0; j < this._sections[i].consumer.sources.length; j++) {
                    sources.push(this._sections[i].consumer.sources[j]);
                }
            }
            return sources;
        }
    });
    /**
     * Returns the original source, line, and column information for the generated
     * source's line and column positions provided. The only argument is an object
     * with the following properties:
     *
     *   - line: The line number in the generated source.
     *   - column: The column number in the generated source.
     *
     * and an object is returned with the following properties:
     *
     *   - source: The original source file, or null.
     *   - line: The line number in the original source, or null.
     *   - column: The column number in the original source, or null.
     *   - name: The original identifier, or null.
     */
    IndexedSourceMapConsumer.prototype.originalPositionFor =
        function IndexedSourceMapConsumer_originalPositionFor(aArgs) {
            var needle = {
                generatedLine: util.getArg(aArgs, 'line'),
                generatedColumn: util.getArg(aArgs, 'column')
            };
            // Find the section containing the generated position we're trying to map
            // to an original position.
            var sectionIndex = binarySearch.search(needle, this._sections, function (needle, section) {
                var cmp = needle.generatedLine - section.generatedOffset.generatedLine;
                if (cmp) {
                    return cmp;
                }
                return (needle.generatedColumn -
                    section.generatedOffset.generatedColumn);
            });
            var section = this._sections[sectionIndex];
            if (!section) {
                return {
                    source: null,
                    line: null,
                    column: null,
                    name: null
                };
            }
            return section.consumer.originalPositionFor({
                line: needle.generatedLine -
                    (section.generatedOffset.generatedLine - 1),
                column: needle.generatedColumn -
                    (section.generatedOffset.generatedLine === needle.generatedLine
                        ? section.generatedOffset.generatedColumn - 1
                        : 0),
                bias: aArgs.bias
            });
        };
    /**
     * Return true if we have the source content for every source in the source
     * map, false otherwise.
     */
    IndexedSourceMapConsumer.prototype.hasContentsOfAllSources =
        function IndexedSourceMapConsumer_hasContentsOfAllSources() {
            return this._sections.every(function (s) {
                return s.consumer.hasContentsOfAllSources();
            });
        };
    /**
     * Returns the original source content. The only argument is the url of the
     * original source file. Returns null if no original source content is
     * available.
     */
    IndexedSourceMapConsumer.prototype.sourceContentFor =
        function IndexedSourceMapConsumer_sourceContentFor(aSource, nullOnMissing) {
            for (var i = 0; i < this._sections.length; i++) {
                var section = this._sections[i];
                var content = section.consumer.sourceContentFor(aSource, true);
                if (content) {
                    return content;
                }
            }
            if (nullOnMissing) {
                return null;
            }
            else {
                throw new Error('"' + aSource + '" is not in the SourceMap.');
            }
        };
    /**
     * Returns the generated line and column information for the original source,
     * line, and column positions provided. The only argument is an object with
     * the following properties:
     *
     *   - source: The filename of the original source.
     *   - line: The line number in the original source.
     *   - column: The column number in the original source.
     *
     * and an object is returned with the following properties:
     *
     *   - line: The line number in the generated source, or null.
     *   - column: The column number in the generated source, or null.
     */
    IndexedSourceMapConsumer.prototype.generatedPositionFor =
        function IndexedSourceMapConsumer_generatedPositionFor(aArgs) {
            for (var i = 0; i < this._sections.length; i++) {
                var section = this._sections[i];
                // Only consider this section if the requested source is in the list of
                // sources of the consumer.
                if (section.consumer.sources.indexOf(util.getArg(aArgs, 'source')) === -1) {
                    continue;
                }
                var generatedPosition = section.consumer.generatedPositionFor(aArgs);
                if (generatedPosition) {
                    var ret = {
                        line: generatedPosition.line +
                            (section.generatedOffset.generatedLine - 1),
                        column: generatedPosition.column +
                            (section.generatedOffset.generatedLine === generatedPosition.line
                                ? section.generatedOffset.generatedColumn - 1
                                : 0)
                    };
                    return ret;
                }
            }
            return {
                line: null,
                column: null
            };
        };
    /**
     * Parse the mappings in a string in to a data structure which we can easily
     * query (the ordered arrays in the `this.__generatedMappings` and
     * `this.__originalMappings` properties).
     */
    IndexedSourceMapConsumer.prototype._parseMappings =
        function IndexedSourceMapConsumer_parseMappings(aStr, aSourceRoot) {
            this.__generatedMappings = [];
            this.__originalMappings = [];
            for (var i = 0; i < this._sections.length; i++) {
                var section = this._sections[i];
                var sectionMappings = section.consumer._generatedMappings;
                for (var j = 0; j < sectionMappings.length; j++) {
                    var mapping = sectionMappings[j];
                    var source = section.consumer._sources.at(mapping.source);
                    if (section.consumer.sourceRoot !== null) {
                        source = util.join(section.consumer.sourceRoot, source);
                    }
                    this._sources.add(source);
                    source = this._sources.indexOf(source);
                    var name = section.consumer._names.at(mapping.name);
                    this._names.add(name);
                    name = this._names.indexOf(name);
                    // The mappings coming from the consumer for the section have
                    // generated positions relative to the start of the section, so we
                    // need to offset them to be relative to the start of the concatenated
                    // generated file.
                    var adjustedMapping = {
                        source: source,
                        generatedLine: mapping.generatedLine +
                            (section.generatedOffset.generatedLine - 1),
                        generatedColumn: mapping.generatedColumn +
                            (section.generatedOffset.generatedLine === mapping.generatedLine
                                ? section.generatedOffset.generatedColumn - 1
                                : 0),
                        originalLine: mapping.originalLine,
                        originalColumn: mapping.originalColumn,
                        name: name
                    };
                    this.__generatedMappings.push(adjustedMapping);
                    if (typeof adjustedMapping.originalLine === 'number') {
                        this.__originalMappings.push(adjustedMapping);
                    }
                }
            }
            quickSort(this.__generatedMappings, util.compareByGeneratedPositionsDeflated);
            quickSort(this.__originalMappings, util.compareByOriginalPositions);
        };
    exports.IndexedSourceMapConsumer = IndexedSourceMapConsumer;
},
/* source-map/lib/binary-search.js */ function _(require, module, exports) {
    /* -*- Mode: js; js-indent-level: 2; -*- */
    /*
     * Copyright 2011 Mozilla Foundation and contributors
     * Licensed under the New BSD license. See LICENSE or:
     * http://opensource.org/licenses/BSD-3-Clause
     */
    exports.GREATEST_LOWER_BOUND = 1;
    exports.LEAST_UPPER_BOUND = 2;
    /**
     * Recursive implementation of binary search.
     *
     * @param aLow Indices here and lower do not contain the needle.
     * @param aHigh Indices here and higher do not contain the needle.
     * @param aNeedle The element being searched for.
     * @param aHaystack The non-empty array being searched.
     * @param aCompare Function which takes two elements and returns -1, 0, or 1.
     * @param aBias Either 'binarySearch.GREATEST_LOWER_BOUND' or
     *     'binarySearch.LEAST_UPPER_BOUND'. Specifies whether to return the
     *     closest element that is smaller than or greater than the one we are
     *     searching for, respectively, if the exact element cannot be found.
     */
    function recursiveSearch(aLow, aHigh, aNeedle, aHaystack, aCompare, aBias) {
        // This function terminates when one of the following is true:
        //
        //   1. We find the exact element we are looking for.
        //
        //   2. We did not find the exact element, but we can return the index of
        //      the next-closest element.
        //
        //   3. We did not find the exact element, and there is no next-closest
        //      element than the one we are searching for, so we return -1.
        var mid = Math.floor((aHigh - aLow) / 2) + aLow;
        var cmp = aCompare(aNeedle, aHaystack[mid], true);
        if (cmp === 0) {
            // Found the element we are looking for.
            return mid;
        }
        else if (cmp > 0) {
            // Our needle is greater than aHaystack[mid].
            if (aHigh - mid > 1) {
                // The element is in the upper half.
                return recursiveSearch(mid, aHigh, aNeedle, aHaystack, aCompare, aBias);
            }
            // The exact needle element was not found in this haystack. Determine if
            // we are in termination case (3) or (2) and return the appropriate thing.
            if (aBias == exports.LEAST_UPPER_BOUND) {
                return aHigh < aHaystack.length ? aHigh : -1;
            }
            else {
                return mid;
            }
        }
        else {
            // Our needle is less than aHaystack[mid].
            if (mid - aLow > 1) {
                // The element is in the lower half.
                return recursiveSearch(aLow, mid, aNeedle, aHaystack, aCompare, aBias);
            }
            // we are in termination case (3) or (2) and return the appropriate thing.
            if (aBias == exports.LEAST_UPPER_BOUND) {
                return mid;
            }
            else {
                return aLow < 0 ? -1 : aLow;
            }
        }
    }
    /**
     * This is an implementation of binary search which will always try and return
     * the index of the closest element if there is no exact hit. This is because
     * mappings between original and generated line/col pairs are single points,
     * and there is an implicit region between each of them, so a miss just means
     * that you aren't on the very start of a region.
     *
     * @param aNeedle The element you are looking for.
     * @param aHaystack The array that is being searched.
     * @param aCompare A function which takes the needle and an element in the
     *     array and returns -1, 0, or 1 depending on whether the needle is less
     *     than, equal to, or greater than the element, respectively.
     * @param aBias Either 'binarySearch.GREATEST_LOWER_BOUND' or
     *     'binarySearch.LEAST_UPPER_BOUND'. Specifies whether to return the
     *     closest element that is smaller than or greater than the one we are
     *     searching for, respectively, if the exact element cannot be found.
     *     Defaults to 'binarySearch.GREATEST_LOWER_BOUND'.
     */
    exports.search = function search(aNeedle, aHaystack, aCompare, aBias) {
        if (aHaystack.length === 0) {
            return -1;
        }
        var index = recursiveSearch(-1, aHaystack.length, aNeedle, aHaystack, aCompare, aBias || exports.GREATEST_LOWER_BOUND);
        if (index < 0) {
            return -1;
        }
        // We have found either the exact element, or the next-closest element than
        // the one we are searching for. However, there may be more than one such
        // element. Make sure we always return the smallest of these.
        while (index - 1 >= 0) {
            if (aCompare(aHaystack[index], aHaystack[index - 1], true) !== 0) {
                break;
            }
            --index;
        }
        return index;
    };
},
/* source-map/lib/quick-sort.js */ function _(require, module, exports) {
    /* -*- Mode: js; js-indent-level: 2; -*- */
    /*
     * Copyright 2011 Mozilla Foundation and contributors
     * Licensed under the New BSD license. See LICENSE or:
     * http://opensource.org/licenses/BSD-3-Clause
     */
    // It turns out that some (most?) JavaScript engines don't self-host
    // `Array.prototype.sort`. This makes sense because C++ will likely remain
    // faster than JS when doing raw CPU-intensive sorting. However, when using a
    // custom comparator function, calling back and forth between the VM's C++ and
    // JIT'd JS is rather slow *and* loses JIT type information, resulting in
    // worse generated code for the comparator function than would be optimal. In
    // fact, when sorting with a comparator, these costs outweigh the benefits of
    // sorting in C++. By using our own JS-implemented Quick Sort (below), we get
    // a ~3500ms mean speed-up in `bench/bench.html`.
    /**
     * Swap the elements indexed by `x` and `y` in the array `ary`.
     *
     * @param {Array} ary
     *        The array.
     * @param {Number} x
     *        The index of the first item.
     * @param {Number} y
     *        The index of the second item.
     */
    function swap(ary, x, y) {
        var temp = ary[x];
        ary[x] = ary[y];
        ary[y] = temp;
    }
    /**
     * Returns a random integer within the range `low .. high` inclusive.
     *
     * @param {Number} low
     *        The lower bound on the range.
     * @param {Number} high
     *        The upper bound on the range.
     */
    function randomIntInRange(low, high) {
        return Math.round(low + (Math.random() * (high - low)));
    }
    /**
     * The Quick Sort algorithm.
     *
     * @param {Array} ary
     *        An array to sort.
     * @param {function} comparator
     *        Function to use to compare two items.
     * @param {Number} p
     *        Start index of the array
     * @param {Number} r
     *        End index of the array
     */
    function doQuickSort(ary, comparator, p, r) {
        // If our lower bound is less than our upper bound, we (1) partition the
        // array into two pieces and (2) recurse on each half. If it is not, this is
        // the empty array and our base case.
        if (p < r) {
            // (1) Partitioning.
            //
            // The partitioning chooses a pivot between `p` and `r` and moves all
            // elements that are less than or equal to the pivot to the before it, and
            // all the elements that are greater than it after it. The effect is that
            // once partition is done, the pivot is in the exact place it will be when
            // the array is put in sorted order, and it will not need to be moved
            // again. This runs in O(n) time.
            // Always choose a random pivot so that an input array which is reverse
            // sorted does not cause O(n^2) running time.
            var pivotIndex = randomIntInRange(p, r);
            var i = p - 1;
            swap(ary, pivotIndex, r);
            var pivot = ary[r];
            // Immediately after `j` is incremented in this loop, the following hold
            // true:
            //
            //   * Every element in `ary[p .. i]` is less than or equal to the pivot.
            //
            //   * Every element in `ary[i+1 .. j-1]` is greater than the pivot.
            for (var j = p; j < r; j++) {
                if (comparator(ary[j], pivot) <= 0) {
                    i += 1;
                    swap(ary, i, j);
                }
            }
            swap(ary, i + 1, j);
            var q = i + 1;
            // (2) Recurse on each half.
            doQuickSort(ary, comparator, p, q - 1);
            doQuickSort(ary, comparator, q + 1, r);
        }
    }
    /**
     * Sort the given array in-place with the given comparator function.
     *
     * @param {Array} ary
     *        An array to sort.
     * @param {function} comparator
     *        Function to use to compare two items.
     */
    exports.quickSort = function (ary, comparator) {
        doQuickSort(ary, comparator, 0, ary.length - 1);
    };
},
/* source-map/lib/source-node.js */ function _(require, module, exports) {
    /* -*- Mode: js; js-indent-level: 2; -*- */
    /*
     * Copyright 2011 Mozilla Foundation and contributors
     * Licensed under the New BSD license. See LICENSE or:
     * http://opensource.org/licenses/BSD-3-Clause
     */
    var SourceMapGenerator = require(47) /* ./source-map-generator */.SourceMapGenerator;
    var util = require(50) /* ./util */;
    // Matches a Windows-style `\r\n` newline or a `\n` newline used by all other
    // operating systems these days (capturing the result).
    var REGEX_NEWLINE = /(\r?\n)/;
    // Newline character code for charCodeAt() comparisons
    var NEWLINE_CODE = 10;
    // Private symbol for identifying `SourceNode`s when multiple versions of
    // the source-map library are loaded. This MUST NOT CHANGE across
    // versions!
    var isSourceNode = "$$$isSourceNode$$$";
    /**
     * SourceNodes provide a way to abstract over interpolating/concatenating
     * snippets of generated JavaScript source code while maintaining the line and
     * column information associated with the original source code.
     *
     * @param aLine The original line number.
     * @param aColumn The original column number.
     * @param aSource The original source's filename.
     * @param aChunks Optional. An array of strings which are snippets of
     *        generated JS, or other SourceNodes.
     * @param aName The original identifier.
     */
    function SourceNode(aLine, aColumn, aSource, aChunks, aName) {
        this.children = [];
        this.sourceContents = {};
        this.line = aLine == null ? null : aLine;
        this.column = aColumn == null ? null : aColumn;
        this.source = aSource == null ? null : aSource;
        this.name = aName == null ? null : aName;
        this[isSourceNode] = true;
        if (aChunks != null)
            this.add(aChunks);
    }
    /**
     * Creates a SourceNode from generated code and a SourceMapConsumer.
     *
     * @param aGeneratedCode The generated code
     * @param aSourceMapConsumer The SourceMap for the generated code
     * @param aRelativePath Optional. The path that relative sources in the
     *        SourceMapConsumer should be relative to.
     */
    SourceNode.fromStringWithSourceMap =
        function SourceNode_fromStringWithSourceMap(aGeneratedCode, aSourceMapConsumer, aRelativePath) {
            // The SourceNode we want to fill with the generated code
            // and the SourceMap
            var node = new SourceNode();
            // All even indices of this array are one line of the generated code,
            // while all odd indices are the newlines between two adjacent lines
            // (since `REGEX_NEWLINE` captures its match).
            // Processed fragments are accessed by calling `shiftNextLine`.
            var remainingLines = aGeneratedCode.split(REGEX_NEWLINE);
            var remainingLinesIndex = 0;
            var shiftNextLine = function () {
                var lineContents = getNextLine();
                // The last line of a file might not have a newline.
                var newLine = getNextLine() || "";
                return lineContents + newLine;
                function getNextLine() {
                    return remainingLinesIndex < remainingLines.length ?
                        remainingLines[remainingLinesIndex++] : undefined;
                }
            };
            // We need to remember the position of "remainingLines"
            var lastGeneratedLine = 1, lastGeneratedColumn = 0;
            // The generate SourceNodes we need a code range.
            // To extract it current and last mapping is used.
            // Here we store the last mapping.
            var lastMapping = null;
            aSourceMapConsumer.eachMapping(function (mapping) {
                if (lastMapping !== null) {
                    // We add the code from "lastMapping" to "mapping":
                    // First check if there is a new line in between.
                    if (lastGeneratedLine < mapping.generatedLine) {
                        // Associate first line with "lastMapping"
                        addMappingWithCode(lastMapping, shiftNextLine());
                        lastGeneratedLine++;
                        lastGeneratedColumn = 0;
                        // The remaining code is added without mapping
                    }
                    else {
                        // There is no new line in between.
                        // Associate the code between "lastGeneratedColumn" and
                        // "mapping.generatedColumn" with "lastMapping"
                        var nextLine = remainingLines[remainingLinesIndex];
                        var code = nextLine.substr(0, mapping.generatedColumn -
                            lastGeneratedColumn);
                        remainingLines[remainingLinesIndex] = nextLine.substr(mapping.generatedColumn -
                            lastGeneratedColumn);
                        lastGeneratedColumn = mapping.generatedColumn;
                        addMappingWithCode(lastMapping, code);
                        // No more remaining code, continue
                        lastMapping = mapping;
                        return;
                    }
                }
                // We add the generated code until the first mapping
                // to the SourceNode without any mapping.
                // Each line is added as separate string.
                while (lastGeneratedLine < mapping.generatedLine) {
                    node.add(shiftNextLine());
                    lastGeneratedLine++;
                }
                if (lastGeneratedColumn < mapping.generatedColumn) {
                    var nextLine = remainingLines[remainingLinesIndex];
                    node.add(nextLine.substr(0, mapping.generatedColumn));
                    remainingLines[remainingLinesIndex] = nextLine.substr(mapping.generatedColumn);
                    lastGeneratedColumn = mapping.generatedColumn;
                }
                lastMapping = mapping;
            }, this);
            // We have processed all mappings.
            if (remainingLinesIndex < remainingLines.length) {
                if (lastMapping) {
                    // Associate the remaining code in the current line with "lastMapping"
                    addMappingWithCode(lastMapping, shiftNextLine());
                }
                // and add the remaining lines without any mapping
                node.add(remainingLines.splice(remainingLinesIndex).join(""));
            }
            // Copy sourcesContent into SourceNode
            aSourceMapConsumer.sources.forEach(function (sourceFile) {
                var content = aSourceMapConsumer.sourceContentFor(sourceFile);
                if (content != null) {
                    if (aRelativePath != null) {
                        sourceFile = util.join(aRelativePath, sourceFile);
                    }
                    node.setSourceContent(sourceFile, content);
                }
            });
            return node;
            function addMappingWithCode(mapping, code) {
                if (mapping === null || mapping.source === undefined) {
                    node.add(code);
                }
                else {
                    var source = aRelativePath
                        ? util.join(aRelativePath, mapping.source)
                        : mapping.source;
                    node.add(new SourceNode(mapping.originalLine, mapping.originalColumn, source, code, mapping.name));
                }
            }
        };
    /**
     * Add a chunk of generated JS to this source node.
     *
     * @param aChunk A string snippet of generated JS code, another instance of
     *        SourceNode, or an array where each member is one of those things.
     */
    SourceNode.prototype.add = function SourceNode_add(aChunk) {
        if (Array.isArray(aChunk)) {
            aChunk.forEach(function (chunk) {
                this.add(chunk);
            }, this);
        }
        else if (aChunk[isSourceNode] || typeof aChunk === "string") {
            if (aChunk) {
                this.children.push(aChunk);
            }
        }
        else {
            throw new TypeError("Expected a SourceNode, string, or an array of SourceNodes and strings. Got " + aChunk);
        }
        return this;
    };
    /**
     * Add a chunk of generated JS to the beginning of this source node.
     *
     * @param aChunk A string snippet of generated JS code, another instance of
     *        SourceNode, or an array where each member is one of those things.
     */
    SourceNode.prototype.prepend = function SourceNode_prepend(aChunk) {
        if (Array.isArray(aChunk)) {
            for (var i = aChunk.length - 1; i >= 0; i--) {
                this.prepend(aChunk[i]);
            }
        }
        else if (aChunk[isSourceNode] || typeof aChunk === "string") {
            this.children.unshift(aChunk);
        }
        else {
            throw new TypeError("Expected a SourceNode, string, or an array of SourceNodes and strings. Got " + aChunk);
        }
        return this;
    };
    /**
     * Walk over the tree of JS snippets in this node and its children. The
     * walking function is called once for each snippet of JS and is passed that
     * snippet and the its original associated source's line/column location.
     *
     * @param aFn The traversal function.
     */
    SourceNode.prototype.walk = function SourceNode_walk(aFn) {
        var chunk;
        for (var i = 0, len = this.children.length; i < len; i++) {
            chunk = this.children[i];
            if (chunk[isSourceNode]) {
                chunk.walk(aFn);
            }
            else {
                if (chunk !== '') {
                    aFn(chunk, { source: this.source,
                        line: this.line,
                        column: this.column,
                        name: this.name });
                }
            }
        }
    };
    /**
     * Like `String.prototype.join` except for SourceNodes. Inserts `aStr` between
     * each of `this.children`.
     *
     * @param aSep The separator.
     */
    SourceNode.prototype.join = function SourceNode_join(aSep) {
        var newChildren;
        var i;
        var len = this.children.length;
        if (len > 0) {
            newChildren = [];
            for (i = 0; i < len - 1; i++) {
                newChildren.push(this.children[i]);
                newChildren.push(aSep);
            }
            newChildren.push(this.children[i]);
            this.children = newChildren;
        }
        return this;
    };
    /**
     * Call String.prototype.replace on the very right-most source snippet. Useful
     * for trimming whitespace from the end of a source node, etc.
     *
     * @param aPattern The pattern to replace.
     * @param aReplacement The thing to replace the pattern with.
     */
    SourceNode.prototype.replaceRight = function SourceNode_replaceRight(aPattern, aReplacement) {
        var lastChild = this.children[this.children.length - 1];
        if (lastChild[isSourceNode]) {
            lastChild.replaceRight(aPattern, aReplacement);
        }
        else if (typeof lastChild === 'string') {
            this.children[this.children.length - 1] = lastChild.replace(aPattern, aReplacement);
        }
        else {
            this.children.push(''.replace(aPattern, aReplacement));
        }
        return this;
    };
    /**
     * Set the source content for a source file. This will be added to the SourceMapGenerator
     * in the sourcesContent field.
     *
     * @param aSourceFile The filename of the source file
     * @param aSourceContent The content of the source file
     */
    SourceNode.prototype.setSourceContent =
        function SourceNode_setSourceContent(aSourceFile, aSourceContent) {
            this.sourceContents[util.toSetString(aSourceFile)] = aSourceContent;
        };
    /**
     * Walk over the tree of SourceNodes. The walking function is called for each
     * source file content and is passed the filename and source content.
     *
     * @param aFn The traversal function.
     */
    SourceNode.prototype.walkSourceContents =
        function SourceNode_walkSourceContents(aFn) {
            for (var i = 0, len = this.children.length; i < len; i++) {
                if (this.children[i][isSourceNode]) {
                    this.children[i].walkSourceContents(aFn);
                }
            }
            var sources = Object.keys(this.sourceContents);
            for (var i = 0, len = sources.length; i < len; i++) {
                aFn(util.fromSetString(sources[i]), this.sourceContents[sources[i]]);
            }
        };
    /**
     * Return the string representation of this source node. Walks over the tree
     * and concatenates all the various snippets together to one string.
     */
    SourceNode.prototype.toString = function SourceNode_toString() {
        var str = "";
        this.walk(function (chunk) {
            str += chunk;
        });
        return str;
    };
    /**
     * Returns the string representation of this source node along with a source
     * map.
     */
    SourceNode.prototype.toStringWithSourceMap = function SourceNode_toStringWithSourceMap(aArgs) {
        var generated = {
            code: "",
            line: 1,
            column: 0
        };
        var map = new SourceMapGenerator(aArgs);
        var sourceMappingActive = false;
        var lastOriginalSource = null;
        var lastOriginalLine = null;
        var lastOriginalColumn = null;
        var lastOriginalName = null;
        this.walk(function (chunk, original) {
            generated.code += chunk;
            if (original.source !== null
                && original.line !== null
                && original.column !== null) {
                if (lastOriginalSource !== original.source
                    || lastOriginalLine !== original.line
                    || lastOriginalColumn !== original.column
                    || lastOriginalName !== original.name) {
                    map.addMapping({
                        source: original.source,
                        original: {
                            line: original.line,
                            column: original.column
                        },
                        generated: {
                            line: generated.line,
                            column: generated.column
                        },
                        name: original.name
                    });
                }
                lastOriginalSource = original.source;
                lastOriginalLine = original.line;
                lastOriginalColumn = original.column;
                lastOriginalName = original.name;
                sourceMappingActive = true;
            }
            else if (sourceMappingActive) {
                map.addMapping({
                    generated: {
                        line: generated.line,
                        column: generated.column
                    }
                });
                lastOriginalSource = null;
                sourceMappingActive = false;
            }
            for (var idx = 0, length = chunk.length; idx < length; idx++) {
                if (chunk.charCodeAt(idx) === NEWLINE_CODE) {
                    generated.line++;
                    generated.column = 0;
                    // Mappings end at eol
                    if (idx + 1 === length) {
                        lastOriginalSource = null;
                        sourceMappingActive = false;
                    }
                    else if (sourceMappingActive) {
                        map.addMapping({
                            source: original.source,
                            original: {
                                line: original.line,
                                column: original.column
                            },
                            generated: {
                                line: generated.line,
                                column: generated.column
                            },
                            name: original.name
                        });
                    }
                }
                else {
                    generated.column++;
                }
            }
        });
        this.walkSourceContents(function (sourceFile, sourceContent) {
            map.setSourceContent(sourceFile, sourceContent);
        });
        return { code: generated.code, map: map };
    };
    exports.SourceNode = SourceNode;
},
/* buffer-from/index.js */ function _(require, module, exports) {
    var toString = Object.prototype.toString;
    var isModern = (typeof Buffer.alloc === 'function' &&
        typeof Buffer.allocUnsafe === 'function' &&
        typeof Buffer.from === 'function');
    function isArrayBuffer(input) {
        return toString.call(input).slice(8, -1) === 'ArrayBuffer';
    }
    function fromArrayBuffer(obj, byteOffset, length) {
        byteOffset >>>= 0;
        var maxLength = obj.byteLength - byteOffset;
        if (maxLength < 0) {
            throw new RangeError("'offset' is out of bounds");
        }
        if (length === undefined) {
            length = maxLength;
        }
        else {
            length >>>= 0;
            if (length > maxLength) {
                throw new RangeError("'length' is out of bounds");
            }
        }
        return isModern
            ? Buffer.from(obj.slice(byteOffset, byteOffset + length))
            : new Buffer(new Uint8Array(obj.slice(byteOffset, byteOffset + length)));
    }
    function fromString(string, encoding) {
        if (typeof encoding !== 'string' || encoding === '') {
            encoding = 'utf8';
        }
        if (!Buffer.isEncoding(encoding)) {
            throw new TypeError('"encoding" must be a valid string encoding');
        }
        return isModern
            ? Buffer.from(string, encoding)
            : new Buffer(string, encoding);
    }
    function bufferFrom(value, encodingOrOffset, length) {
        if (typeof value === 'number') {
            throw new TypeError('"value" argument must not be a number');
        }
        if (isArrayBuffer(value)) {
            return fromArrayBuffer(value, encodingOrOffset, length);
        }
        if (typeof value === 'string') {
            return fromString(value, encodingOrOffset);
        }
        return isModern
            ? Buffer.from(value)
            : new Buffer(value);
    }
    module.exports = bufferFrom;
},
/* build.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const cp = __importStar(require("child_process"));
    const path_1 = require("path");
    const sys_1 = require(43) /* ./sys */;
    const compiler_1 = require(59) /* ./compiler */;
    const linker_1 = require(66) /* ./linker */;
    const tsconfig_json = __importStar(require(76) /* ./tsconfig.ext.json */);
    const chalk_1 = __importDefault(require(60) /* chalk */);
    const { cyan, magenta, red } = chalk_1.default;
    const eslint_1 = require(77) /* eslint */;
    require(595) /* @typescript-eslint/eslint-plugin */;
    require(747) /* @typescript-eslint/parser */;
    const readline = __importStar(require("readline"));
    function print(str) {
        console.log(str);
    }
    function npm_install(base_dir) {
        const npm = process.platform != "win32" ? "npm" : "npm.cmd";
        const { status } = cp.spawnSync(npm, ["install"], { stdio: "inherit", cwd: base_dir });
        if (status != null && status != 0) {
            print(`${cyan("npm install")} failed with exit code ${red(`${status}`)}.`);
            process.exit(status);
        }
    }
    function is_up_to_date(base_dir, file, metadata) {
        const contents = sys_1.read(path_1.join(base_dir, file));
        if (contents == null)
            return false;
        const old_hash = metadata.signatures[file];
        if (old_hash == null)
            return false;
        const new_hash = sys_1.hash(contents);
        return old_hash == new_hash;
    }
    function needs_install(base_dir, metadata) {
        if (!sys_1.directory_exists(path_1.join(base_dir, "node_modules")))
            return `New development environment.`;
        else if (!is_up_to_date(base_dir, "package.json", metadata))
            return `package.json has changed.`;
        else if (!is_up_to_date(base_dir, "package-lock.json", metadata))
            return `package-lock.json has changed.`;
        else
            return null;
    }
    function lint(config_file, paths) {
        const engine = new eslint_1.CLIEngine({
            configFile: config_file,
            extensions: [".ts", ".js"],
        });
        const report = engine.executeOnFiles(paths);
        eslint_1.CLIEngine.outputFixes(report);
        const ok = report.errorCount == 0;
        if (!ok) {
            const formatter = engine.getFormatter();
            const output = formatter(report.results);
            for (const line of output.trim().split("\n"))
                print(line);
        }
        return ok;
    }
    async function init(base_dir, _bokehjs_dir, base_setup) {
        print(`Working directory: ${cyan(base_dir)}`);
        const setup = {
            interactive: !!base_setup.interactive,
            bokehjs_version: base_setup.bokehjs_version != null ? base_setup.bokehjs_version : base_setup.bokeh_version.split("-")[0],
            bokeh_version: base_setup.bokeh_version,
        };
        const paths = {
            bokeh_ext: path_1.join(base_dir, "bokeh.ext.json"),
            package: path_1.join(base_dir, "package.json"),
            package_lock: path_1.join(base_dir, "package-lock.json"),
            tsconfig: path_1.join(base_dir, "tsconfig.json"),
            index: path_1.join(base_dir, "index.ts"),
        };
        const is_extension = sys_1.file_exists(paths.bokeh_ext);
        if (is_extension) {
            print("Already a bokeh extension. Quitting.");
            return false;
        }
        function write_json(path, json) {
            sys_1.write(path, JSON.stringify(json, undefined, 2));
            print(`Wrote ${cyan(path)}`);
        }
        const bokeh_ext_json = {};
        write_json(paths.bokeh_ext, bokeh_ext_json);
        const package_json = {
            name: path_1.basename(base_dir),
            version: "0.0.1",
            description: "",
            license: "BSD-3-Clause",
            keywords: [],
            repository: {},
            dependencies: {
                bokehjs: `^${setup.bokehjs_version}`,
            },
            devDependencies: {},
        };
        if (setup.interactive) {
            const rl = readline.createInterface({ input: process.stdin, output: process.stdout });
            async function ask(question, default_value) {
                return new Promise((resolve, _reject) => {
                    rl.question(`${question} `, (answer) => {
                        resolve(answer.length != 0 ? answer : default_value);
                    });
                });
            }
            async function ask_yn(question) {
                const ret = await ask(`${question} [y/n]`, "y");
                switch (ret) {
                    case "y":
                        return true;
                    case "n":
                        return false;
                    default: {
                        print(`${red("Invalid input")}. Assuming no.`);
                        return false;
                    }
                }
            }
            if (await ask_yn(`Create ${cyan("package.json")}? This will allow you to specify external dependencies.`)) {
                const { name } = package_json;
                package_json.name = await ask(`  What's the extension's name? [${name}]`, name);
                const { version } = package_json;
                package_json.version = await ask(`  What's the extension's version? [${version}]`, version);
                const { description } = package_json;
                package_json.description = await ask(`  What's the extension's description? [${description}]`, description);
                write_json(paths.package, package_json);
            }
            if (await ask_yn(`Create ${cyan("tsconfig.json")}? This will allow for customized configuration and improved IDE experience.`)) {
                write_json(paths.tsconfig, tsconfig_json);
            }
            rl.close();
        }
        else {
            write_json(paths.package, package_json);
            write_json(paths.tsconfig, tsconfig_json);
        }
        sys_1.write(paths.index, "");
        print(`Created empty ${cyan("index.ts")}. This is the entry point of your extension.`);
        const rel = path_1.relative(process.cwd(), base_dir);
        print(`You can build your extension with ${magenta(`bokeh build ${rel}`)}`);
        print("All done.");
        return true;
    }
    exports.init = init;
    async function build(base_dir, bokehjs_dir, base_setup) {
        print(`Working directory: ${cyan(base_dir)}`);
        const setup = {
            rebuild: !!base_setup.rebuild,
            bokeh_version: base_setup.bokeh_version,
        };
        const bokeh_ext_json_path = path_1.join(base_dir, "bokeh.ext.json");
        const is_extension = sys_1.file_exists(bokeh_ext_json_path);
        if (!is_extension) {
            print("Not a bokeh extension. Quitting.");
            return false;
        }
        const metadata_path = path_1.join(base_dir, ".bokeh");
        const metadata = (() => {
            let obj = sys_1.read_json(metadata_path);
            if (obj == null)
                obj = {};
            if (obj.signatures == null)
                obj.signatures = {};
            return obj;
        })();
        if (metadata.bokeh_version != setup.bokeh_version) {
            print("Using different version of bokeh, rebuilding from scratch.");
            setup.rebuild = true;
        }
        const package_json_path = path_1.join(base_dir, "package.json");
        const package_lock_json_path = path_1.join(base_dir, "package-lock.json");
        const is_package = sys_1.file_exists(package_json_path);
        if (!is_package) {
            print(`${cyan(package_json_path)} doesn't exist. Not a npm package.`);
        }
        else {
            if (setup.rebuild) {
                print(`Running ${cyan("npm install")}.`);
                npm_install(base_dir);
            }
            else {
                const result = needs_install(base_dir, metadata);
                if (result != null) {
                    print(`${result} Running ${cyan("npm install")}.`);
                    npm_install(base_dir);
                }
            }
        }
        const tsconfig_path = path_1.join(base_dir, "tsconfig.json");
        const tsconfig = (() => {
            const preconfigure = {
                baseUrl: base_dir,
                paths: {
                    "@bokehjs/*": [
                        path_1.join(bokehjs_dir, "js/lib/*"),
                        path_1.join(bokehjs_dir, "js/types/*"),
                    ],
                },
            };
            if (sys_1.file_exists(tsconfig_path)) {
                print(`Using ${cyan(tsconfig_path)}`);
                return compiler_1.read_tsconfig(tsconfig_path, is_package ? undefined : preconfigure);
            }
            else
                return compiler_1.parse_tsconfig(tsconfig_json, base_dir, preconfigure);
        })();
        if (compiler_1.is_failed(tsconfig)) {
            print(compiler_1.report_diagnostics(tsconfig.diagnostics).text);
            return false;
        }
        const { files, options } = tsconfig;
        const transformers = compiler_1.default_transformers(options);
        const host = compiler_1.compiler_host(new Map(), options, bokehjs_dir);
        print(`Compiling TypeScript (${magenta(files.length + " files")})`);
        const tsoutput = compiler_1.compile_files(files, options, transformers, host);
        if (compiler_1.is_failed(tsoutput)) {
            print(compiler_1.report_diagnostics(tsoutput.diagnostics).text);
            if (options.noEmitOnError)
                return false;
        }
        const lint_config = path_1.join(base_dir, "eslint.json");
        if (sys_1.file_exists(lint_config)) {
            print(`Linting sources`);
            lint(lint_config, files);
        }
        const dist_dir = path_1.join(base_dir, "dist");
        const lib_dir = options.outDir || dist_dir;
        const artifact = path_1.basename(base_dir);
        const bases = [lib_dir];
        if (is_package)
            bases.push(path_1.join(base_dir, "node_modules"));
        const linker = new linker_1.Linker({
            entries: [path_1.join(lib_dir, "index.js")],
            bases,
            cache: path_1.join(dist_dir, `${artifact}.json`),
            excluded: (dep) => dep == "tslib" || dep.startsWith("@bokehjs/"),
            plugin: true,
            target: "ES2017",
        });
        print("Linking modules");
        if (!setup.rebuild)
            linker.load_cache();
        const bundles = linker.link();
        linker.store_cache();
        const outputs = [path_1.join(dist_dir, `${artifact}.js`)];
        const min_js = (js) => sys_1.rename(js, { ext: '.min.js' });
        function bundle(minified, outputs) {
            bundles
                .map((bundle) => bundle.assemble(minified))
                .map((artifact, i) => artifact.write(outputs[i]));
        }
        bundle(false, outputs);
        bundle(true, outputs.map(min_js));
        sys_1.write(metadata_path, JSON.stringify({
            bokeh_version: setup.bokeh_version,
            signatures: {
                "package.json": sys_1.hash_file(package_json_path),
                "package-lock.json": sys_1.hash_file(package_lock_json_path),
                "tsconfig.json": sys_1.hash_file(tsconfig_path),
            },
        }));
        print(`Output written to ${cyan(dist_dir)}`);
        print("All done.");
        return !compiler_1.is_failed(tsoutput);
    }
    exports.build = build;
},
/* compiler.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const chalk_1 = __importDefault(require(60) /* chalk */);
    const ts = __importStar(require(44) /* typescript */);
    const path_1 = require("path");
    const transforms = __importStar(require(65) /* ./transforms */);
    const sys_1 = require(43) /* ./sys */;
    function is_failed(obj) {
        return "diagnostics" in obj && obj.diagnostics != null;
    }
    exports.is_failed = is_failed;
    function normalize(path) {
        return path.replace(/\\/g, "/");
    }
    const diagnostics_host = {
        getCanonicalFileName: (path) => path,
        getCurrentDirectory: ts.sys.getCurrentDirectory,
        getNewLine: () => ts.sys.newLine,
    };
    function report_diagnostics(diagnostics) {
        const errors = ts.sortAndDeduplicateDiagnostics(diagnostics);
        const text = ts.formatDiagnosticsWithColorAndContext(errors, diagnostics_host);
        return { count: errors.length, text };
    }
    exports.report_diagnostics = report_diagnostics;
    function compiler_host(inputs, options, bokehjs_dir) {
        const default_host = ts.createCompilerHost(options);
        const host = Object.assign(Object.assign({}, default_host), { fileExists(name) {
                return inputs.get(name) != null || default_host.fileExists(name);
            },
            readFile(name) {
                return inputs.get(name) != null ? inputs.get(name) : default_host.readFile(name);
            },
            getSourceFile(name, target, _onError) {
                const source = inputs.get(name);
                if (source != null)
                    return ts.createSourceFile(name, source, target);
                else
                    return default_host.getSourceFile(name, target, _onError);
            } });
        if (bokehjs_dir != null) {
            host.getDefaultLibLocation = () => {
                // bokeh/server/static or bokehjs/build
                if (path_1.basename(bokehjs_dir) == "static")
                    return path_1.join(bokehjs_dir, "lib");
                else
                    return path_1.join(path_1.dirname(bokehjs_dir), "node_modules/typescript/lib");
            };
        }
        return host;
    }
    exports.compiler_host = compiler_host;
    function default_transformers(options, css_dir) {
        const transformers = {
            before: [],
            after: [],
            afterDeclarations: [],
        };
        const import_txt = transforms.import_txt((txt_path) => sys_1.read(txt_path));
        transformers.before.push(import_txt);
        const import_css = transforms.import_css((css_path) => {
            const resolved_path = css_path.startsWith(".") || css_dir == null ? css_path : path_1.join(css_dir, css_path);
            return sys_1.read(resolved_path);
        });
        transformers.before.push(import_css);
        const insert_class_name = transforms.insert_class_name();
        transformers.before.push(insert_class_name);
        const add_init_class = transforms.add_init_class();
        transformers.before.push(add_init_class);
        const base = options.baseUrl;
        if (base != null) {
            const relativize_modules = transforms.relativize_modules((file, module_path) => {
                if (!module_path.startsWith(".") && !module_path.startsWith("/")) {
                    const module_file = path_1.join(base, module_path);
                    if (ts.sys.fileExists(module_file) ||
                        ts.sys.fileExists(module_file + ".ts") ||
                        ts.sys.fileExists(path_1.join(module_file, "index.ts"))) {
                        const rel_path = normalize(path_1.relative(path_1.dirname(file), module_file));
                        return rel_path.startsWith(".") ? rel_path : `./${rel_path}`;
                    }
                }
                return null;
            });
            transformers.after.push(relativize_modules);
            transformers.afterDeclarations.push(relativize_modules);
        }
        return transformers;
    }
    exports.default_transformers = default_transformers;
    function compile_files(inputs, options, transformers, host) {
        const program = ts.createProgram(inputs, options, host);
        const emitted = program.emit(undefined, undefined, undefined, false, transformers);
        const diagnostics = ts.getPreEmitDiagnostics(program).concat(emitted.diagnostics);
        return diagnostics.length != 0 ? { diagnostics } : {};
    }
    exports.compile_files = compile_files;
    function parse_tsconfig(tsconfig_json, base_dir, preconfigure) {
        const host = {
            useCaseSensitiveFileNames: ts.sys.useCaseSensitiveFileNames,
            readDirectory: ts.sys.readDirectory,
            fileExists: ts.sys.fileExists,
            readFile: ts.sys.readFile,
        };
        const tsconfig = ts.parseJsonConfigFileContent(tsconfig_json, host, base_dir, preconfigure);
        if (tsconfig.errors.length != 0) {
            return { diagnostics: tsconfig.errors };
        }
        return { files: tsconfig.fileNames, options: tsconfig.options };
    }
    exports.parse_tsconfig = parse_tsconfig;
    function read_tsconfig(tsconfig_path, preconfigure) {
        const tsconfig_file = ts.readConfigFile(tsconfig_path, ts.sys.readFile);
        if (tsconfig_file.error != null) {
            return { diagnostics: [tsconfig_file.error] };
        }
        return parse_tsconfig(tsconfig_file.config, path_1.dirname(tsconfig_path), preconfigure);
    }
    exports.read_tsconfig = read_tsconfig;
    function compile_project(tsconfig_path, config) {
        const preconfigure = (() => {
            const { out_dir } = config;
            if (out_dir != null) {
                if (typeof out_dir == "string")
                    return { outDir: out_dir };
                else
                    return { outDir: out_dir.js, declarationDir: out_dir.dts, declaration: true };
            }
            else
                return {};
        })();
        const tsconfig = read_tsconfig(tsconfig_path, preconfigure);
        if (is_failed(tsconfig))
            return { diagnostics: tsconfig.diagnostics };
        const { files, options } = tsconfig;
        const transformers = default_transformers(tsconfig.options, config.css_dir);
        const host = compiler_host(new Map(), options, config.bokehjs_dir);
        return compile_files(files, options, transformers, host);
    }
    exports.compile_project = compile_project;
    function compile_typescript(tsconfig_path, config) {
        const result = compile_project(tsconfig_path, config);
        if (is_failed(result)) {
            const failure = report_diagnostics(result.diagnostics);
            if (config.log != null)
                config.log(`There were ${chalk_1.default.red("" + failure.count)} TypeScript errors:\n${failure.text}`);
            return false;
        }
        return true;
    }
    exports.compile_typescript = compile_typescript;
},
/* chalk/source/index.js */ function _(require, module, exports) {
    const ansiStyles = require(25) /* ansi-styles */;
    const { stdout: stdoutColor, stderr: stderrColor } = require(61) /* supports-color */;
    const { stringReplaceAll, stringEncaseCRLFWithFirstIndex } = require(63) /* ./util */;
    // `supportsColor.level` → `ansiStyles.color[name]` mapping
    const levelMapping = [
        'ansi',
        'ansi',
        'ansi256',
        'ansi16m'
    ];
    const styles = Object.create(null);
    const applyOptions = (object, options = {}) => {
        if (options.level > 3 || options.level < 0) {
            throw new Error('The `level` option should be an integer from 0 to 3');
        }
        // Detect level if not set manually
        const colorLevel = stdoutColor ? stdoutColor.level : 0;
        object.level = options.level === undefined ? colorLevel : options.level;
    };
    class ChalkClass {
        constructor(options) {
            return chalkFactory(options);
        }
    }
    const chalkFactory = options => {
        const chalk = {};
        applyOptions(chalk, options);
        chalk.template = (...arguments_) => chalkTag(chalk.template, ...arguments_);
        Object.setPrototypeOf(chalk, Chalk.prototype);
        Object.setPrototypeOf(chalk.template, chalk);
        chalk.template.constructor = () => {
            throw new Error('`chalk.constructor()` is deprecated. Use `new chalk.Instance()` instead.');
        };
        chalk.template.Instance = ChalkClass;
        return chalk.template;
    };
    function Chalk(options) {
        return chalkFactory(options);
    }
    for (const [styleName, style] of Object.entries(ansiStyles)) {
        styles[styleName] = {
            get() {
                const builder = createBuilder(this, createStyler(style.open, style.close, this._styler), this._isEmpty);
                Object.defineProperty(this, styleName, { value: builder });
                return builder;
            }
        };
    }
    styles.visible = {
        get() {
            const builder = createBuilder(this, this._styler, true);
            Object.defineProperty(this, 'visible', { value: builder });
            return builder;
        }
    };
    const usedModels = ['rgb', 'hex', 'keyword', 'hsl', 'hsv', 'hwb', 'ansi', 'ansi256'];
    for (const model of usedModels) {
        styles[model] = {
            get() {
                const { level } = this;
                return function (...arguments_) {
                    const styler = createStyler(ansiStyles.color[levelMapping[level]][model](...arguments_), ansiStyles.color.close, this._styler);
                    return createBuilder(this, styler, this._isEmpty);
                };
            }
        };
    }
    for (const model of usedModels) {
        const bgModel = 'bg' + model[0].toUpperCase() + model.slice(1);
        styles[bgModel] = {
            get() {
                const { level } = this;
                return function (...arguments_) {
                    const styler = createStyler(ansiStyles.bgColor[levelMapping[level]][model](...arguments_), ansiStyles.bgColor.close, this._styler);
                    return createBuilder(this, styler, this._isEmpty);
                };
            }
        };
    }
    const proto = Object.defineProperties(() => { }, {
        ...styles,
        level: {
            enumerable: true,
            get() {
                return this._generator.level;
            },
            set(level) {
                this._generator.level = level;
            }
        }
    });
    const createStyler = (open, close, parent) => {
        let openAll;
        let closeAll;
        if (parent === undefined) {
            openAll = open;
            closeAll = close;
        }
        else {
            openAll = parent.openAll + open;
            closeAll = close + parent.closeAll;
        }
        return {
            open,
            close,
            openAll,
            closeAll,
            parent
        };
    };
    const createBuilder = (self, _styler, _isEmpty) => {
        const builder = (...arguments_) => {
            // Single argument is hot path, implicit coercion is faster than anything
            // eslint-disable-next-line no-implicit-coercion
            return applyStyle(builder, (arguments_.length === 1) ? ('' + arguments_[0]) : arguments_.join(' '));
        };
        // `__proto__` is used because we must return a function, but there is
        // no way to create a function with a different prototype
        builder.__proto__ = proto; // eslint-disable-line no-proto
        builder._generator = self;
        builder._styler = _styler;
        builder._isEmpty = _isEmpty;
        return builder;
    };
    const applyStyle = (self, string) => {
        if (self.level <= 0 || !string) {
            return self._isEmpty ? '' : string;
        }
        let styler = self._styler;
        if (styler === undefined) {
            return string;
        }
        const { openAll, closeAll } = styler;
        if (string.indexOf('\u001B') !== -1) {
            while (styler !== undefined) {
                // Replace any instances already present with a re-opening code
                // otherwise only the part of the string until said closing code
                // will be colored, and the rest will simply be 'plain'.
                string = stringReplaceAll(string, styler.close, styler.open);
                styler = styler.parent;
            }
        }
        // We can move both next actions out of loop, because remaining actions in loop won't have
        // any/visible effect on parts we add here. Close the styling before a linebreak and reopen
        // after next line to fix a bleed issue on macOS: https://github.com/chalk/chalk/pull/92
        const lfIndex = string.indexOf('\n');
        if (lfIndex !== -1) {
            string = stringEncaseCRLFWithFirstIndex(string, closeAll, openAll, lfIndex);
        }
        return openAll + string + closeAll;
    };
    let template;
    const chalkTag = (chalk, ...strings) => {
        const [firstString] = strings;
        if (!Array.isArray(firstString)) {
            // If chalk() was called by itself or with a string,
            // return the string itself as a string.
            return strings.join(' ');
        }
        const arguments_ = strings.slice(1);
        const parts = [firstString.raw[0]];
        for (let i = 1; i < firstString.length; i++) {
            parts.push(String(arguments_[i - 1]).replace(/[{}\\]/g, '\\$&'), String(firstString.raw[i]));
        }
        if (template === undefined) {
            template = require(64) /* ./templates */;
        }
        return template(chalk, parts.join(''));
    };
    Object.defineProperties(Chalk.prototype, styles);
    const chalk = Chalk(); // eslint-disable-line new-cap
    chalk.supportsColor = stdoutColor;
    chalk.stderr = Chalk({ level: stderrColor ? stderrColor.level : 0 }); // eslint-disable-line new-cap
    chalk.stderr.supportsColor = stderrColor;
    // For TypeScript
    chalk.Level = {
        None: 0,
        Basic: 1,
        Ansi256: 2,
        TrueColor: 3,
        0: 'None',
        1: 'Basic',
        2: 'Ansi256',
        3: 'TrueColor'
    };
    module.exports = chalk;
},
/* supports-color/index.js */ function _(require, module, exports) {
    const os = require('os');
    const tty = require('tty');
    const hasFlag = require(62) /* has-flag */;
    const { env } = process;
    let forceColor;
    if (hasFlag('no-color') ||
        hasFlag('no-colors') ||
        hasFlag('color=false') ||
        hasFlag('color=never')) {
        forceColor = 0;
    }
    else if (hasFlag('color') ||
        hasFlag('colors') ||
        hasFlag('color=true') ||
        hasFlag('color=always')) {
        forceColor = 1;
    }
    if ('FORCE_COLOR' in env) {
        if (env.FORCE_COLOR === 'true') {
            forceColor = 1;
        }
        else if (env.FORCE_COLOR === 'false') {
            forceColor = 0;
        }
        else {
            forceColor = env.FORCE_COLOR.length === 0 ? 1 : Math.min(parseInt(env.FORCE_COLOR, 10), 3);
        }
    }
    function translateLevel(level) {
        if (level === 0) {
            return false;
        }
        return {
            level,
            hasBasic: true,
            has256: level >= 2,
            has16m: level >= 3
        };
    }
    function supportsColor(haveStream, streamIsTTY) {
        if (forceColor === 0) {
            return 0;
        }
        if (hasFlag('color=16m') ||
            hasFlag('color=full') ||
            hasFlag('color=truecolor')) {
            return 3;
        }
        if (hasFlag('color=256')) {
            return 2;
        }
        if (haveStream && !streamIsTTY && forceColor === undefined) {
            return 0;
        }
        const min = forceColor || 0;
        if (env.TERM === 'dumb') {
            return min;
        }
        if (process.platform === 'win32') {
            // Windows 10 build 10586 is the first Windows release that supports 256 colors.
            // Windows 10 build 14931 is the first release that supports 16m/TrueColor.
            const osRelease = os.release().split('.');
            if (Number(osRelease[0]) >= 10 &&
                Number(osRelease[2]) >= 10586) {
                return Number(osRelease[2]) >= 14931 ? 3 : 2;
            }
            return 1;
        }
        if ('CI' in env) {
            if (['TRAVIS', 'CIRCLECI', 'APPVEYOR', 'GITLAB_CI'].some(sign => sign in env) || env.CI_NAME === 'codeship') {
                return 1;
            }
            return min;
        }
        if ('TEAMCITY_VERSION' in env) {
            return /^(9\.(0*[1-9]\d*)\.|\d{2,}\.)/.test(env.TEAMCITY_VERSION) ? 1 : 0;
        }
        if ('GITHUB_ACTIONS' in env) {
            return 1;
        }
        if (env.COLORTERM === 'truecolor') {
            return 3;
        }
        if ('TERM_PROGRAM' in env) {
            const version = parseInt((env.TERM_PROGRAM_VERSION || '').split('.')[0], 10);
            switch (env.TERM_PROGRAM) {
                case 'iTerm.app':
                    return version >= 3 ? 3 : 2;
                case 'Apple_Terminal':
                    return 2;
                // No default
            }
        }
        if (/-256(color)?$/i.test(env.TERM)) {
            return 2;
        }
        if (/^screen|^xterm|^vt100|^vt220|^rxvt|color|ansi|cygwin|linux/i.test(env.TERM)) {
            return 1;
        }
        if ('COLORTERM' in env) {
            return 1;
        }
        return min;
    }
    function getSupportLevel(stream) {
        const level = supportsColor(stream, stream && stream.isTTY);
        return translateLevel(level);
    }
    module.exports = {
        supportsColor: getSupportLevel,
        stdout: translateLevel(supportsColor(true, tty.isatty(1))),
        stderr: translateLevel(supportsColor(true, tty.isatty(2)))
    };
},
/* has-flag/index.js */ function _(require, module, exports) {
    module.exports = (flag, argv = process.argv) => {
        const prefix = flag.startsWith('-') ? '' : (flag.length === 1 ? '-' : '--');
        const position = argv.indexOf(prefix + flag);
        const terminatorPosition = argv.indexOf('--');
        return position !== -1 && (terminatorPosition === -1 || position < terminatorPosition);
    };
},
/* chalk/source/util.js */ function _(require, module, exports) {
    const stringReplaceAll = (string, substring, replacer) => {
        let index = string.indexOf(substring);
        if (index === -1) {
            return string;
        }
        const substringLength = substring.length;
        let endIndex = 0;
        let returnValue = '';
        do {
            returnValue += string.substr(endIndex, index - endIndex) + substring + replacer;
            endIndex = index + substringLength;
            index = string.indexOf(substring, endIndex);
        } while (index !== -1);
        returnValue += string.substr(endIndex);
        return returnValue;
    };
    const stringEncaseCRLFWithFirstIndex = (string, prefix, postfix, index) => {
        let endIndex = 0;
        let returnValue = '';
        do {
            const gotCR = string[index - 1] === '\r';
            returnValue += string.substr(endIndex, (gotCR ? index - 1 : index) - endIndex) + prefix + (gotCR ? '\r\n' : '\n') + postfix;
            endIndex = index + 1;
            index = string.indexOf('\n', endIndex);
        } while (index !== -1);
        returnValue += string.substr(endIndex);
        return returnValue;
    };
    module.exports = {
        stringReplaceAll,
        stringEncaseCRLFWithFirstIndex
    };
},
/* chalk/source/templates.js */ function _(require, module, exports) {
    const TEMPLATE_REGEX = /(?:\\(u(?:[a-f\d]{4}|\{[a-f\d]{1,6}\})|x[a-f\d]{2}|.))|(?:\{(~)?(\w+(?:\([^)]*\))?(?:\.\w+(?:\([^)]*\))?)*)(?:[ \t]|(?=\r?\n)))|(\})|((?:.|[\r\n\f])+?)/gi;
    const STYLE_REGEX = /(?:^|\.)(\w+)(?:\(([^)]*)\))?/g;
    const STRING_REGEX = /^(['"])((?:\\.|(?!\1)[^\\])*)\1$/;
    const ESCAPE_REGEX = /\\(u(?:[a-f\d]{4}|\{[a-f\d]{1,6}\})|x[a-f\d]{2}|.)|([^\\])/gi;
    const ESCAPES = new Map([
        ['n', '\n'],
        ['r', '\r'],
        ['t', '\t'],
        ['b', '\b'],
        ['f', '\f'],
        ['v', '\v'],
        ['0', '\0'],
        ['\\', '\\'],
        ['e', '\u001B'],
        ['a', '\u0007']
    ]);
    function unescape(c) {
        const u = c[0] === 'u';
        const bracket = c[1] === '{';
        if ((u && !bracket && c.length === 5) || (c[0] === 'x' && c.length === 3)) {
            return String.fromCharCode(parseInt(c.slice(1), 16));
        }
        if (u && bracket) {
            return String.fromCodePoint(parseInt(c.slice(2, -1), 16));
        }
        return ESCAPES.get(c) || c;
    }
    function parseArguments(name, arguments_) {
        const results = [];
        const chunks = arguments_.trim().split(/\s*,\s*/g);
        let matches;
        for (const chunk of chunks) {
            const number = Number(chunk);
            if (!Number.isNaN(number)) {
                results.push(number);
            }
            else if ((matches = chunk.match(STRING_REGEX))) {
                results.push(matches[2].replace(ESCAPE_REGEX, (m, escape, character) => escape ? unescape(escape) : character));
            }
            else {
                throw new Error(`Invalid Chalk template style argument: ${chunk} (in style '${name}')`);
            }
        }
        return results;
    }
    function parseStyle(style) {
        STYLE_REGEX.lastIndex = 0;
        const results = [];
        let matches;
        while ((matches = STYLE_REGEX.exec(style)) !== null) {
            const name = matches[1];
            if (matches[2]) {
                const args = parseArguments(name, matches[2]);
                results.push([name].concat(args));
            }
            else {
                results.push([name]);
            }
        }
        return results;
    }
    function buildStyle(chalk, styles) {
        const enabled = {};
        for (const layer of styles) {
            for (const style of layer.styles) {
                enabled[style[0]] = layer.inverse ? null : style.slice(1);
            }
        }
        let current = chalk;
        for (const [styleName, styles] of Object.entries(enabled)) {
            if (!Array.isArray(styles)) {
                continue;
            }
            if (!(styleName in current)) {
                throw new Error(`Unknown Chalk style: ${styleName}`);
            }
            current = styles.length > 0 ? current[styleName](...styles) : current[styleName];
        }
        return current;
    }
    module.exports = (chalk, temporary) => {
        const styles = [];
        const chunks = [];
        let chunk = [];
        // eslint-disable-next-line max-params
        temporary.replace(TEMPLATE_REGEX, (m, escapeCharacter, inverse, style, close, character) => {
            if (escapeCharacter) {
                chunk.push(unescape(escapeCharacter));
            }
            else if (style) {
                const string = chunk.join('');
                chunk = [];
                chunks.push(styles.length === 0 ? string : buildStyle(chalk, styles)(string));
                styles.push({ inverse, styles: parseStyle(style) });
            }
            else if (close) {
                if (styles.length === 0) {
                    throw new Error('Found extraneous } in Chalk template literal');
                }
                chunks.push(buildStyle(chalk, styles)(chunk.join('')));
                chunk = [];
                styles.pop();
            }
            else {
                chunk.push(character);
            }
        });
        chunks.push(chunk.join(''));
        if (styles.length > 0) {
            const errMsg = `Chalk template literal is missing ${styles.length} closing bracket${styles.length === 1 ? '' : 's'} (\`}\`)`;
            throw new Error(errMsg);
        }
        return chunks.join('');
    };
},
/* transforms.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = __importStar(require(44) /* typescript */);
    function apply(node, ...transforms) {
        const result = ts.transform(node, transforms);
        return result.transformed[0];
    }
    exports.apply = apply;
    function is_require(node) {
        return ts.isCallExpression(node) &&
            ts.isIdentifier(node.expression) &&
            node.expression.text === "require" &&
            node.arguments.length === 1;
    }
    function relativize_modules(relativize) {
        function relativize_specifier(source, expr) {
            if (expr != null && ts.isStringLiteralLike(expr) && expr.text.length > 0) {
                const relative = relativize(source.fileName, expr.text);
                if (relative != null)
                    return ts.createLiteral(relative);
            }
            return null;
        }
        return (context) => (root) => {
            function visit(node) {
                if (ts.isImportDeclaration(node)) {
                    const moduleSpecifier = relativize_specifier(root, node.moduleSpecifier);
                    if (moduleSpecifier != null) {
                        const { decorators, modifiers, importClause } = node;
                        return ts.updateImportDeclaration(node, decorators, modifiers, importClause, moduleSpecifier);
                    }
                }
                if (ts.isExportDeclaration(node)) {
                    const moduleSpecifier = relativize_specifier(root, node.moduleSpecifier);
                    if (moduleSpecifier != null) {
                        const { decorators, modifiers, exportClause } = node;
                        return ts.updateExportDeclaration(node, decorators, modifiers, exportClause, moduleSpecifier, false);
                    }
                }
                if (is_require(node)) {
                    const moduleSpecifier = relativize_specifier(root, node.arguments[0]);
                    if (moduleSpecifier != null) {
                        const { expression, typeArguments } = node;
                        return ts.updateCall(node, expression, typeArguments, [moduleSpecifier]);
                    }
                }
                return ts.visitEachChild(node, visit, context);
            }
            return ts.visitNode(root, visit);
        };
    }
    exports.relativize_modules = relativize_modules;
    function import_txt(load) {
        return (context) => (root) => {
            function visit(node) {
                if (ts.isImportDeclaration(node)) {
                    const { importClause, moduleSpecifier } = node;
                    if (ts.isStringLiteralLike(moduleSpecifier)) {
                        const txt_path = moduleSpecifier.text;
                        if (txt_path.endsWith(".txt") && importClause != null && importClause.name != null) {
                            const txt_text = load(txt_path);
                            if (txt_text != null) {
                                return ts.createVariableDeclaration(importClause.name, ts.createKeywordTypeNode(ts.SyntaxKind.StringKeyword), ts.createStringLiteral(txt_text));
                            }
                        }
                    }
                }
                return ts.visitEachChild(node, visit, context);
            }
            return ts.visitNode(root, visit);
        };
    }
    exports.import_txt = import_txt;
    function css_loader(css_text) {
        const dom = ts.createTempVariable(undefined);
        return [
            ts.createImportDeclaration(undefined, undefined, ts.createImportClause(undefined, ts.createNamespaceImport(dom)), ts.createStringLiteral("core/dom")),
            ts.createExpressionStatement(ts.createCall(ts.createPropertyAccess(ts.createPropertyAccess(dom, "styles"), "append"), undefined, [ts.createStringLiteral(css_text)])),
        ];
    }
    function import_css(load) {
        return (context) => (root) => {
            function visit(node) {
                if (ts.isImportDeclaration(node)) {
                    const { importClause, moduleSpecifier } = node;
                    if (ts.isStringLiteralLike(moduleSpecifier)) {
                        const css_path = moduleSpecifier.text;
                        if (importClause == null && css_path.endsWith(".css")) {
                            const css_text = load(css_path);
                            if (css_text != null) {
                                return css_loader(css_text);
                            }
                        }
                    }
                }
                return ts.visitEachChild(node, visit, context);
            }
            return ts.visitNode(root, visit);
        };
    }
    exports.import_css = import_css;
    function is_static(node) {
        return node.modifiers != null && node.modifiers.find((modifier) => modifier.kind == ts.SyntaxKind.StaticKeyword) != null;
    }
    function add_init_class() {
        return (context) => (root) => {
            function visit(node) {
                node = ts.visitEachChild(node, visit, context);
                if (ts.isClassDeclaration(node) && node.name != null) {
                    const name = `init_${node.name.getText()}`;
                    if (node.members.find((member) => ts.isMethodDeclaration(member) && member.name.getText() == name && is_static(member)) != null) {
                        const init = ts.createExpressionStatement(ts.createCall(ts.createPropertyAccess(node.name, name), undefined, undefined));
                        return [node, init];
                    }
                }
                return node;
            }
            return ts.visitNode(root, visit);
        };
    }
    exports.add_init_class = add_init_class;
    function insert_class_name() {
        function has__name__(node) {
            return node.members.find((member) => ts.isPropertyDeclaration(member) && member.name.getText() == "__name__" && is_static(member)) != null;
        }
        return (context) => (root) => {
            function visit(node) {
                node = ts.visitEachChild(node, visit, context);
                if (ts.isClassDeclaration(node) && node.name != null && !has__name__(node)) {
                    const property = ts.createProperty(undefined, ts.createModifiersFromModifierFlags(ts.ModifierFlags.Static), "__name__", undefined, undefined, ts.createStringLiteral(node.name.text));
                    node = ts.updateClassDeclaration(node, node.decorators, node.modifiers, node.name, node.typeParameters, node.heritageClauses, [property, ...node.members]);
                }
                return node;
            }
            return ts.visitNode(root, visit);
        };
    }
    exports.insert_class_name = insert_class_name;
    function remove_use_strict() {
        return (_context) => (root) => {
            const statements = root.statements.filter((node) => {
                if (ts.isExpressionStatement(node)) {
                    const expr = node.expression;
                    if (ts.isStringLiteral(expr) && expr.text == "use strict")
                        return false;
                }
                return true;
            });
            return ts.updateSourceFileNode(root, statements);
        };
    }
    exports.remove_use_strict = remove_use_strict;
    function remove_esmodule() {
        return (_context) => (root) => {
            const statements = root.statements.filter((node) => {
                if (ts.isExpressionStatement(node)) {
                    const expr = node.expression;
                    if (ts.isCallExpression(expr) && expr.arguments.length == 3) {
                        const [, arg] = expr.arguments;
                        if (ts.isStringLiteral(arg) && arg.text == "__esModule")
                            return false;
                    }
                }
                return true;
            });
            return ts.updateSourceFileNode(root, statements);
        };
    }
    exports.remove_esmodule = remove_esmodule;
    function isImportCall(node) {
        return ts.isCallExpression(node) && node.expression.kind == ts.SyntaxKind.ImportKeyword;
    }
    function collect_imports(imports) {
        return (context) => (root) => {
            function visit(node) {
                if (ts.isImportDeclaration(node) || ts.isExportDeclaration(node)) {
                    const name = node.moduleSpecifier;
                    if (name != null && ts.isStringLiteral(name) && name.text.length != 0)
                        imports.add(name.text);
                }
                else if (isImportCall(node)) {
                    const [name] = node.arguments;
                    if (ts.isStringLiteral(name) && name.text.length != 0)
                        imports.add(name.text);
                }
                return ts.visitEachChild(node, visit, context);
            }
            return ts.visitNode(root, visit);
        };
    }
    exports.collect_imports = collect_imports;
    function collect_deps(source) {
        function traverse(node) {
            if (is_require(node)) {
                const [arg] = node.arguments;
                if (ts.isStringLiteral(arg) && arg.text.length > 0)
                    deps.add(arg.text);
            }
            ts.forEachChild(node, traverse);
        }
        const deps = new Set();
        traverse(source);
        return [...deps];
    }
    exports.collect_deps = collect_deps;
    function rewrite_deps(resolve) {
        return (context) => (root) => {
            function visit(node) {
                if (is_require(node)) {
                    const [arg] = node.arguments;
                    if (ts.isStringLiteral(arg) && arg.text.length > 0) {
                        const dep = arg.text;
                        const val = resolve(dep);
                        if (val != null) {
                            node = ts.updateCall(node, node.expression, node.typeArguments, [ts.createLiteral(val)]);
                            ts.addSyntheticTrailingComment(node, ts.SyntaxKind.MultiLineCommentTrivia, ` ${dep} `, false);
                        }
                        return node;
                    }
                }
                return ts.visitEachChild(node, visit, context);
            }
            return ts.visitNode(root, visit);
        };
    }
    exports.rewrite_deps = rewrite_deps;
    // XXX: this is pretty naive, but affects very litte code
    function rename_exports() {
        return (context) => (root) => {
            function is_exports(node) {
                return ts.isIdentifier(node) && node.text == "exports";
            }
            const has_exports = root.statements.some((stmt) => {
                return ts.isVariableStatement(stmt) && stmt.declarationList.declarations.some((decl) => is_exports(decl.name));
            });
            if (has_exports) {
                function visit(node) {
                    if (is_exports(node)) {
                        const updated = ts.createIdentifier("exports$1");
                        const original = node;
                        ts.setOriginalNode(updated, original);
                        ts.setTextRange(updated, original);
                        return updated;
                    }
                    return ts.visitEachChild(node, visit, context);
                }
                return ts.visitNode(root, visit);
            }
            else
                return root;
        };
    }
    exports.rename_exports = rename_exports;
    function add_json_export() {
        return (_context) => (root) => {
            if (root.statements.length == 1) {
                const [statement] = root.statements;
                if (ts.isExpressionStatement(statement)) {
                    const left = ts.createPropertyAccess(ts.createIdentifier("module"), "exports");
                    const right = statement.expression;
                    const assign = ts.createStatement(ts.createAssignment(left, right));
                    return ts.updateSourceFileNode(root, [assign]);
                }
            }
            return root;
        };
    }
    exports.add_json_export = add_json_export;
    function wrap_in_function(module_name) {
        return (_context) => (root) => {
            const p = (name) => ts.createParameter(undefined, undefined, undefined, name);
            const params = [p("require"), p("module"), p("exports")];
            const block = ts.createBlock(root.statements, true);
            const func = ts.createFunctionDeclaration(undefined, undefined, undefined, "_", undefined, params, undefined, block);
            ts.addSyntheticLeadingComment(func, ts.SyntaxKind.MultiLineCommentTrivia, ` ${module_name} `, false);
            return ts.updateSourceFileNode(root, [func]);
        };
    }
    exports.wrap_in_function = wrap_in_function;
    function parse_es(file, code, target = ts.ScriptTarget.ES2017) {
        return ts.createSourceFile(file, code != null ? code : ts.sys.readFile(file), target, true, ts.ScriptKind.JS);
    }
    exports.parse_es = parse_es;
    function print_es(source) {
        const printer = ts.createPrinter();
        return printer.printNode(ts.EmitHint.SourceFile, source, source);
    }
    exports.print_es = print_es;
},
/* linker.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const path_1 = require("path");
    const module_1 = __importDefault(require("module"));
    const crypto_1 = __importDefault(require("crypto"));
    const ts = __importStar(require(44) /* typescript */);
    const terser = __importStar(require(67) /* terser */);
    const combine = __importStar(require(68) /* combine-source-map */);
    const convert = __importStar(require(69) /* convert-source-map */);
    const sys_1 = require(43) /* ./sys */;
    const compiler_1 = require(59) /* ./compiler */;
    const preludes = __importStar(require(75) /* ./prelude */);
    const transforms = __importStar(require(65) /* ./transforms */);
    const root_path = process.cwd();
    const cache_version = 3;
    const to_obj = (map) => {
        const obj = Object.create(null);
        for (const [key, val] of map) {
            obj[key] = val;
        }
        return obj;
    };
    const dense_assembly = {
        prefix: "[",
        suffix: "]",
        wrap: (_, source) => source,
    };
    const sparse_assembly = {
        prefix: "{",
        suffix: "}",
        wrap: (id, source) => `${id}: ${source}`,
    };
    class Bundle {
        constructor(entry, artifacts, builtins, prelude, assembly) {
            this.entry = entry;
            this.artifacts = artifacts;
            this.builtins = builtins;
            this.prelude = prelude;
            this.assembly = assembly;
        }
        assemble(minified = false) {
            let line = 0;
            let sources = "";
            const sourcemap = combine.create();
            const aliases = new Map();
            const externals = new Map();
            const newlines = (source) => {
                const result = source.match(/\n/g);
                return result != null ? result.length : 0;
            };
            const safe_id = (module) => {
                const { id } = module;
                return typeof id == "number" ? id.toString() : JSON.stringify(id);
            };
            const { entry, artifacts, prelude, assembly: { prefix, suffix, wrap } } = this;
            sources += `${prelude}(${prefix}\n`;
            line += newlines(sources);
            for (const artifact of artifacts) {
                const { module } = artifact;
                if (module.canonical != null)
                    aliases.set(module.canonical, module.id);
                for (const external of module.externals)
                    externals.set(external, true);
                for (const external of module.shims)
                    externals.set(external, false);
                const start = wrap(safe_id(module), "");
                sources += start;
                line += newlines(start);
                const source_with_sourcemap = minified ? artifact.code.min_source : artifact.code.source;
                const source = combine.removeComments(source_with_sourcemap).trimRight();
                sources += source;
                const map_path = path_1.join("@@", path_1.relative(root_path, module.file));
                sourcemap.addFile({ source: source_with_sourcemap, sourceFile: map_path }, { line });
                line += newlines(source);
                const end = ",\n";
                sources += end;
                line += newlines(end);
            }
            const aliases_json = JSON.stringify(to_obj(aliases));
            const externals_json = JSON.stringify(to_obj(externals));
            sources += `${suffix}, ${safe_id(entry)}, ${aliases_json}, ${externals_json});\n})\n`;
            const source_map = convert.fromBase64(sourcemap.base64()).toObject();
            return new Artifact(sources, minified ? null : source_map, aliases);
        }
    }
    exports.Bundle = Bundle;
    Bundle.__name__ = "Bundle";
    class Artifact {
        constructor(source, sourcemap, exported) {
            this.source = source;
            this.sourcemap = sourcemap;
            this.exported = exported;
        }
        full_source(name) {
            if (this.sourcemap != null)
                return `${this.source}\n${convert.generateMapFileComment(name)}\n`;
            else
                return `${this.source}\n`;
        }
        get module_names() {
            return Array.from(this.exported.keys());
        }
        write(path) {
            const dir = path_1.dirname(path);
            const name = path_1.basename(path, ".js");
            sys_1.write(path, this.full_source(name + ".js.map"));
            if (this.sourcemap != null) {
                sys_1.write(path_1.join(dir, name + ".js.map"), JSON.stringify(this.sourcemap));
            }
        }
    }
    exports.Artifact = Artifact;
    Artifact.__name__ = "Artifact";
    class Linker {
        constructor(opts) {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
            this.ext = ".js";
            this.entries = opts.entries.map((path) => path_1.resolve(path));
            this.bases = ((_a = opts.bases) !== null && _a !== void 0 ? _a : []).map((path) => path_1.resolve(path));
            this.excludes = new Set(((_b = opts.excludes) !== null && _b !== void 0 ? _b : []).map((path) => path_1.resolve(path)));
            this.external_modules = new Set(((_c = opts.externals) !== null && _c !== void 0 ? _c : []).filter((s) => typeof s === "string"));
            this.external_regex = ((_d = opts.externals) !== null && _d !== void 0 ? _d : []).filter((s) => s instanceof RegExp);
            this.excluded = (_e = opts.excluded) !== null && _e !== void 0 ? _e : (() => false);
            this.builtins = (_f = opts.builtins) !== null && _f !== void 0 ? _f : false;
            this.exports = new Set((_g = opts.exports) !== null && _g !== void 0 ? _g : []);
            this.prelude = ((_h = opts.prelude) !== null && _h !== void 0 ? _h : preludes.prelude)();
            this.plugin_prelude = ((_j = opts.plugin_prelude) !== null && _j !== void 0 ? _j : preludes.plugin_prelude)();
            if (this.builtins) {
                this.external_modules.add("module");
                this.external_modules.add("constants");
                for (const lib of module_1.default.builtinModules)
                    this.external_modules.add(lib);
            }
            for (const entry of this.entries) {
                if (!sys_1.file_exists(entry))
                    throw new Error(`entry path ${entry} doesn't exist or isn't a file`);
            }
            for (const base of this.bases) {
                if (!sys_1.directory_exists(base))
                    throw new Error(`base path ${base} doesn't exist or isn't a directory`);
            }
            this.cache_path = opts.cache;
            this.cache = new Map();
            this.target = opts.target != null ? opts.target : null;
            this.minify = opts.minify != null ? opts.minify : true;
            this.plugin = opts.plugin != null ? opts.plugin : false;
            this.shims = new Set((_k = opts.shims) !== null && _k !== void 0 ? _k : []);
        }
        is_external(dep) {
            return this.external_modules.has(dep) || this.external_regex.some((re) => re.test(dep));
        }
        is_shimmed(dep) {
            return this.shims.has(dep);
        }
        link() {
            const [entries] = this.resolve(this.entries);
            const [main, ...plugins] = entries;
            const dirnames = plugins.map((plugin) => path_1.dirname(plugin.file));
            const is_excluded = (module) => {
                return dirnames.find((e) => module.file.startsWith(e)) != null;
            };
            const main_modules = this.reachable(main, is_excluded);
            const parents = new Set(main_modules);
            const plugin_modules = [];
            for (const plugin of plugins) {
                const files = this.reachable(plugin, (module) => parents.has(module));
                plugin_modules.push(files);
            }
            const all_modules = main_modules.concat(...plugin_modules);
            if (!this.plugin)
                all_modules.forEach((module, i) => module.id = i);
            else
                all_modules.forEach((module) => module.id = module.hash.slice(0, 10));
            const transformers = (module) => {
                const transformers = [];
                switch (module.type) {
                    case "js": {
                        const remove_use_strict = transforms.remove_use_strict();
                        transformers.push(remove_use_strict);
                        // TODO: don't remove __esModule, just make it more space efficient
                        // const remove_esmodule = transforms.remove_esmodule()
                        // transformers.push(remove_esmodule)
                        const rewrite_deps = transforms.rewrite_deps((dep) => {
                            const module_dep = module.dependencies.get(dep);
                            return module_dep != null ? module_dep.id : undefined;
                        });
                        transformers.push(rewrite_deps);
                        break;
                    }
                    case "json": {
                        transformers.push(transforms.add_json_export());
                        break;
                    }
                    case "css": {
                        // ???
                        break;
                    }
                }
                transformers.push(transforms.wrap_in_function(module.base_path));
                return transformers;
            };
            const print = (module) => {
                let ast = module.ast || this.parse_module(module);
                ast = transforms.apply(ast, ...transformers(module));
                const source = transforms.print_es(ast);
                return convert.removeMapFileComments(source);
            };
            const deps_changed = (module, cached) => {
                if (module.dependencies.size != cached.dependencies.size)
                    return false;
                for (const [dep, module_dep] of module.dependencies) {
                    const cached_dep = cached.dependencies.get(dep);
                    if (cached_dep == null || cached_dep.id != module_dep.id)
                        return true;
                }
                return false;
            };
            const artifacts = (modules) => {
                return modules.map((module) => {
                    const cached = this.cache.get(module.file);
                    let code;
                    if (module.changed || (cached != null && deps_changed(module, cached.module))) {
                        const source = print(module);
                        const ecma = this.target == "ES2020" ? 2020 : (this.target == "ES2017" ? 2017 : 5);
                        const minified = this.minify ? minify(module, source, ecma) : { min_source: source };
                        code = Object.assign({ source }, minified);
                    }
                    else
                        code = cached.code;
                    return { module, code };
                });
            };
            const main_prelude = !this.plugin ? this.prelude : this.plugin_prelude;
            const main_assembly = !this.plugin ? dense_assembly : sparse_assembly;
            const main_bundle = new Bundle(main, artifacts(main_modules), this.builtins, main_prelude, main_assembly);
            const plugin_bundles = [];
            for (let j = 0; j < plugins.length; j++) {
                const plugin_bundle = new Bundle(plugins[j], artifacts(plugin_modules[j]), this.builtins, this.plugin_prelude, sparse_assembly);
                plugin_bundles.push(plugin_bundle);
            }
            const bundles = [main_bundle, ...plugin_bundles];
            this.cache.clear();
            for (const bundle of bundles) {
                for (const artifact of bundle.artifacts) {
                    this.cache.set(artifact.module.file, artifact);
                }
            }
            return bundles;
        }
        load_cache() {
            const { cache_path } = this;
            if (cache_path == null || !sys_1.file_exists(cache_path))
                return;
            this.cache.clear();
            const json = JSON.parse(sys_1.read(cache_path));
            if (json.version !== cache_version) {
                console.warn("ignoring cache due to format version mismatch");
                return;
            }
            for (const { module, code } of json.artifacts) {
                const artifact = {
                    module: Object.assign(Object.assign({}, module), { ast: undefined, changed: false, dependencies: new Map(), dependency_map: new Map(module.dependency_map), dependency_paths: new Map(module.dependency_paths), externals: new Set(module.externals), shims: new Set(module.shims) }),
                    code,
                };
                this.cache.set(artifact.module.file, artifact);
            }
            for (const { module } of this.cache.values()) {
                for (const [dep, file] of module.dependency_paths) {
                    module.dependencies.set(dep, this.cache.get(file).module);
                }
            }
        }
        store_cache() {
            if (this.cache_path == null)
                return;
            const artifacts = [];
            for (const artifact of this.cache.values()) {
                const module = Object.assign({}, artifact.module);
                delete module.changed;
                delete module.ast;
                delete module.dependencies;
                artifacts.push({
                    module: Object.assign(Object.assign({}, module), { dependency_map: [...module.dependency_map.entries()], dependency_paths: [...module.dependency_paths.entries()], externals: [...module.externals.values()], shims: [...module.shims.values()] }),
                    code: artifact.code,
                });
            }
            const json = JSON.stringify({ version: cache_version, artifacts });
            sys_1.write(this.cache_path, json);
        }
        resolve_package(dir) {
            const index = (() => {
                const pkg_path = path_1.join(dir, "package.json");
                if (sys_1.file_exists(pkg_path)) {
                    const pkg = JSON.parse(sys_1.read(pkg_path));
                    if (this.target != null && pkg.module != null)
                        return pkg.module;
                    if (pkg.main != null)
                        return pkg.main;
                }
                return "index.js";
            })();
            const path = path_1.join(dir, index);
            if (sys_1.file_exists(path))
                return path;
            const js_file = path + ".js";
            if (sys_1.file_exists(js_file))
                return js_file;
            const json_file = path + ".json";
            if (sys_1.file_exists(json_file))
                return json_file;
            if (sys_1.directory_exists(path)) {
                const index = path_1.join(path, "index.js");
                if (sys_1.file_exists(index))
                    return index;
            }
            return null;
        }
        resolve_relative(dep, parent) {
            const path = path_1.resolve(path_1.dirname(parent.file), dep);
            if (sys_1.file_exists(path))
                return path;
            const js_file = path + ".js";
            const json_file = path + ".json";
            const has_js_file = sys_1.file_exists(js_file);
            const has_json_file = sys_1.file_exists(json_file);
            const has_file = has_js_file || has_json_file;
            if (sys_1.directory_exists(path)) {
                const pkg_file = this.resolve_package(path);
                if (pkg_file != null) {
                    if (!has_file)
                        return pkg_file;
                    else
                        return new Error(`both ${has_js_file ? js_file : json_file} and ${pkg_file} exist`);
                }
            }
            if (has_js_file)
                return js_file;
            else if (has_json_file)
                return json_file;
            else
                return new Error(`can't resolve '${dep}' from '${parent.file}'`);
        }
        resolve_absolute(dep, parent) {
            for (const base of this.bases) {
                let path = path_1.join(base, dep);
                if (sys_1.file_exists(path))
                    return path;
                const js_file = path + ".js";
                if (sys_1.file_exists(js_file))
                    return js_file;
                const json_file = path + ".json";
                if (sys_1.file_exists(json_file))
                    return json_file;
                if (sys_1.directory_exists(path)) {
                    const file = this.resolve_package(path);
                    if (file != null)
                        return file;
                }
                if (parent.file.startsWith(base)) {
                    let base_path = parent.file;
                    while (true) {
                        base_path = path_1.dirname(base_path);
                        if (base_path == base)
                            break;
                        path = path_1.join(base_path, "node_modules", dep);
                        if (sys_1.directory_exists(path)) {
                            const file = this.resolve_package(path);
                            if (file != null)
                                return file;
                        }
                    }
                }
            }
            return new Error(`can't resolve '${dep}' from '${parent.file}'`);
        }
        resolve_file(dep, parent) {
            if (dep.startsWith("."))
                return this.resolve_relative(dep, parent);
            else
                return this.resolve_absolute(dep, parent);
        }
        parse_module({ file, source, type }) {
            const { ES2017, JSON } = ts.ScriptTarget;
            return transforms.parse_es(file, source, type == "json" ? JSON : ES2017);
        }
        new_module(file) {
            let source = sys_1.read(file);
            if (source == null) {
                throw new Error(`'${file} doesn't exist`);
            }
            const hash = crypto_1.default.createHash("sha256").update(source).digest("hex");
            const type = (() => {
                switch (path_1.extname(file)) {
                    case ".json": return "json";
                    case ".css": return "css";
                    case ".mjs": return "js";
                    case ".js": return "js";
                    default:
                        throw new Error(`unsupported extension of ${file}`);
                }
            })();
            const [base, base_path, canonical, resolution] = (() => {
                var _a;
                const [primary, ...secondary] = this.bases;
                function canonicalize(path) {
                    return path.replace(/\.js$/, "").replace(/\\/g, "/");
                }
                function get_package(base, path) {
                    const root = path_1.join(base, path);
                    base = path_1.normalize(base);
                    path = path_1.normalize(root);
                    while (path != base) {
                        if (sys_1.directory_exists(path)) {
                            const pkg_path = path_1.join(path, "package.json");
                            if (sys_1.file_exists(pkg_path))
                                return { dir: path, pkg: JSON.parse(sys_1.read(pkg_path)) };
                        }
                        path = path_1.dirname(path);
                    }
                    throw new Error(`can't resolve package.json for ${root}`);
                }
                const path = path_1.relative(primary, file);
                if (!path.startsWith("..")) {
                    return [primary, path, canonicalize(path), "ESM"];
                }
                for (const base of secondary) {
                    const path = path_1.relative(base, file);
                    if (!path.startsWith("..")) {
                        const { dir, pkg } = get_package(base, path);
                        const reso = pkg.module != null ? "ESM" : "CJS";
                        const entry = (_a = pkg.module) !== null && _a !== void 0 ? _a : pkg.name;
                        const primary = path_1.join(dir, entry) == path_1.join(base, path);
                        const name = canonicalize(primary ? path_1.basename(dir) : path);
                        const exported = this.exports.has(name);
                        return [base, path, exported ? name : undefined, reso];
                    }
                }
                throw new Error(`${file} is not under any of base paths`);
            })();
            const cached = this.cache.get(file);
            let ast;
            let dependency_paths;
            let externals;
            let shims;
            const changed = cached == null || cached.module.hash != hash;
            if (changed) {
                let collected = null;
                if (type == "js") {
                    if (this.target != null && resolution == "ESM") {
                        const { ES2020, ES2017, ES5 } = ts.ScriptTarget;
                        const target = this.target == "ES2020" ? ES2020 : (this.target == "ES2017" ? ES2017 : ES5);
                        const imports = new Set(["tslib"]);
                        const transform = { before: [transforms.collect_imports(imports), transforms.rename_exports()], after: [] };
                        const { output, error } = transpile(file, source, target, transform);
                        if (error)
                            throw new Error(error);
                        else {
                            source = output;
                            collected = [...imports];
                        }
                    }
                }
                ast = this.parse_module({ file, source, type });
                if (collected == null)
                    collected = transforms.collect_deps(ast);
                const filtered = collected.filter((dep) => !this.is_external(dep) && !this.excluded(dep) && !this.is_shimmed(dep));
                dependency_paths = new Map();
                for (const dep of filtered) {
                    const resolved = this.resolve_file(dep, { file });
                    if (resolved instanceof Error)
                        console.log(resolved);
                    else
                        dependency_paths.set(dep, resolved);
                }
                externals = new Set(collected.filter((dep) => this.is_external(dep)));
                shims = new Set(collected.filter((dep) => this.is_shimmed(dep)));
            }
            else {
                dependency_paths = cached.module.dependency_paths;
                externals = cached.module.externals;
                shims = cached.module.shims;
                source = cached.module.source;
            }
            return {
                file,
                base,
                base_path,
                canonical,
                resolution,
                id: NaN,
                hash,
                changed,
                source,
                ast,
                type,
                dependency_paths,
                dependency_map: new Map(),
                dependencies: new Map(),
                externals,
                shims,
            };
        }
        resolve(files) {
            const visited = new Map();
            const pending = [...files];
            for (;;) {
                const file = pending.shift();
                if (file == null)
                    break;
                if (visited.has(file) || this.excludes.has(file))
                    continue;
                const module = this.new_module(file);
                visited.set(module.file, module);
                pending.unshift(...module.dependency_paths.values());
            }
            for (const module of visited.values()) {
                for (const [dep, file] of module.dependency_paths) {
                    module.dependencies.set(dep, visited.get(file));
                }
            }
            const entries = files.map((file) => visited.get(file));
            return [entries, [...visited.values()]];
        }
        reachable(entry, is_excluded) {
            const reached = new Set();
            const pending = [entry];
            for (;;) {
                const module = pending.shift();
                if (module == null)
                    break;
                if (reached.has(module) || this.excludes.has(module.file) || is_excluded(module))
                    continue;
                reached.add(module);
                pending.unshift(...module.dependencies.values());
            }
            return [...reached.values()];
        }
    }
    exports.Linker = Linker;
    Linker.__name__ = "Linker";
    function transpile(file, source, target, transformers) {
        const { outputText: output, diagnostics } = ts.transpileModule(source, {
            fileName: file,
            reportDiagnostics: true,
            compilerOptions: {
                target,
                module: ts.ModuleKind.CommonJS,
                esModuleInterop: true,
                importHelpers: true,
            },
            transformers,
        });
        if (diagnostics == null || diagnostics.length == 0)
            return { output };
        else {
            const { text } = compiler_1.report_diagnostics(diagnostics);
            return { output, error: text };
        }
    }
    exports.transpile = transpile;
    function minify(module, source, ecma) {
        const name = path_1.basename(module.file);
        const min_js = sys_1.rename(name, { ext: '.min.js' });
        const min_js_map = sys_1.rename(name, { ext: '.min.js.map' });
        const minify_opts = {
            ecma,
            output: {
                comments: /^!|copyright|license|\(c\)/i,
            },
            sourceMap: {
                filename: path_1.basename(min_js),
                url: path_1.basename(min_js_map),
            },
        };
        const { code, map, error } = terser.minify(source, minify_opts);
        if (error != null) {
            const { message, line, col } = error;
            throw new Error(`${module.file}:${line - 1}:${col}: ${message}`);
        }
        return { min_source: code || "", min_map: typeof map === "string" ? map : undefined };
    }
    exports.minify = minify;
},
/* terser/dist/bundle.min.js */ function _(require, module, exports) {
    !function (e, t) { "object" == typeof exports && "undefined" != typeof module ? t(exports, require(46) /* source-map */) : "function" == typeof define && define.amd ? define(["exports", "source-map"], t) : t((e = e || self).Terser = {}, e.sourceMap); }(this, (function (e, t) {
        "use strict";
        function n(e) { return e.split(""); }
        function i(e, t) { return t.includes(e); }
        t = t && t.hasOwnProperty("default") ? t.default : t;
        class o extends Error {
            constructor(e, t) { super(), this.name = "DefaultsError", this.message = e, this.defs = t; }
        }
        function r(e, t, n) { !0 === e && (e = {}); const i = e || {}; if (n)
            for (const e in i)
                if (D(i, e) && !D(t, e))
                    throw new o("`" + e + "` is not a supported option", t); for (const n in t)
            if (D(t, n))
                if (e && D(e, n))
                    if ("ecma" === n) {
                        let t = 0 | e[n];
                        t > 5 && t < 2015 && (t += 2009), i[n] = t;
                    }
                    else
                        i[n] = e && D(e, n) ? e[n] : t[n];
                else
                    i[n] = t[n]; return i; }
        function a() { }
        function s() { return !1; }
        function u() { return !0; }
        function c() { return this; }
        function l() { return null; }
        var f = function () { function e(e, r, a) { var s, u = [], c = []; function l() { var l = r(e[s], s), f = l instanceof o; return f && (l = l.v), l instanceof n ? (l = l.v) instanceof i ? c.push.apply(c, a ? l.v.slice().reverse() : l.v) : c.push(l) : l !== t && (l instanceof i ? u.push.apply(u, a ? l.v.slice().reverse() : l.v) : u.push(l)), f; } if (Array.isArray(e))
            if (a) {
                for (s = e.length; --s >= 0 && !l();)
                    ;
                u.reverse(), c.reverse();
            }
            else
                for (s = 0; s < e.length && !l(); ++s)
                    ;
        else
            for (s in e)
                if (D(e, s) && l())
                    break; return c.concat(u); } e.at_top = function (e) { return new n(e); }, e.splice = function (e) { return new i(e); }, e.last = function (e) { return new o(e); }; var t = e.skip = {}; function n(e) { this.v = e; } function i(e) { this.v = e; } function o(e) { this.v = e; } return e; }();
        function p(e, t, n) { return n || (n = {}), t && (n.start || (n.start = t.start), n.end || (n.end = t.end)), new e(n); }
        function _(e, t) { e.includes(t) || e.push(t); }
        function d(e, t) { return e.replace(/{(.+?)}/g, (function (e, n) { return t && t[n]; })); }
        function m(e, t) { for (var n = e.length; --n >= 0;)
            e[n] === t && e.splice(n, 1); }
        function h(e, t) { if (e.length < 2)
            return e.slice(); return function e(n) { if (n.length <= 1)
            return n; var i = Math.floor(n.length / 2), o = n.slice(0, i), r = n.slice(i); return function (e, n) { for (var i = [], o = 0, r = 0, a = 0; o < e.length && r < n.length;)
            t(e[o], n[r]) <= 0 ? i[a++] = e[o++] : i[a++] = n[r++]; return o < e.length && i.push.apply(i, e.slice(o)), r < n.length && i.push.apply(i, n.slice(r)), i; }(o = e(o), r = e(r)); }(e); }
        function E(e) { return Array.isArray(e) || (e = e.split(" ")), new Set(e); }
        function g(e, t, n) { e.has(t) ? e.get(t).push(n) : e.set(t, [n]); }
        function D(e, t) { return Object.prototype.hasOwnProperty.call(e, t); }
        function S(e, t) { return !0 === e || e instanceof RegExp && e.test(t); }
        var v = { "\n": "n", "\r": "r", "\u2028": "u2028", "\u2029": "u2029" };
        function A(e) { return e.replace(/[\n\r\u2028\u2029]/g, (function (t, n) { return ("\\" == e[n - 1] && ("\\" != e[n - 2] || /(?:^|[^\\])(?:\\{2})*$/.test(e.slice(0, n - 1))) ? "" : "\\") + v[t]; })); }
        function T(e, t) { return e._annotations & t; }
        function y(e, t) { e._annotations |= t; }
        var b = "break case catch class const continue debugger default delete do else export extends finally for function if in instanceof let new return switch throw try typeof var void while with", C = "false null true", O = "enum implements import interface package private protected public static super this " + C + " " + b, F = "return new delete throw else case yield await";
        b = E(b), O = E(O), F = E(F), C = E(C);
        var M = E(n("+-*&%=<>!?|~^")), R = /[0-9a-f]/i, w = /^0x[0-9a-f]+$/i, x = /^0[0-7]+$/, N = /^0o[0-7]+$/i, k = /^0b[01]+$/i, I = /^\d*\.?\d*(?:e[+-]?\d*(?:\d\.?|\.?\d)\d*)?$/i, L = /^(0[xob])?[0-9a-f]+n$/i, V = E(["in", "instanceof", "typeof", "new", "void", "delete", "++", "--", "+", "-", "!", "~", "&", "|", "^", "*", "**", "/", "%", ">>", "<<", ">>>", "<", ">", "<=", ">=", "==", "===", "!=", "!==", "?", "=", "+=", "-=", "/=", "*=", "**=", "%=", ">>=", "<<=", ">>>=", "|=", "^=", "&=", "&&", "||"]), P = E(n("  \n\r\t\f\v​           \u2028\u2029  　\ufeff")), B = E(n("\n\r\u2028\u2029")), K = E(n(";]),:")), U = E(n("[{(,;:")), G = E(n("[]{}(),;:")), H = { ID_Start: /[A-Za-z\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA\u05F0-\u05F2\u0620-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0840-\u0858\u08A0-\u08B4\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971-\u0980\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0AF9\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D\u0C58-\u0C5A\u0C60\u0C61\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0CF1\u0CF2\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D\u0D4E\u0D5F-\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6\u0EDC-\u0EDF\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8C\u1000-\u102A\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081\u108E\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7\u17DC\u1820-\u1877\u1880-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191E\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u1A00-\u1A16\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF\u1BBA-\u1BE5\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1CE9-\u1CEC\u1CEE-\u1CF1\u1CF5\u1CF6\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F\u2090-\u209C\u2102\u2107\u210A-\u2113\u2115\u2118-\u211D\u2124\u2126\u2128\u212A-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2CF2\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303C\u3041-\u3096\u309B-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FD5\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B\uA640-\uA66E\uA67F-\uA69D\uA6A0-\uA6EF\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AD\uA7B0-\uA7B7\uA7F7-\uA801\uA803-\uA805\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB\uA8FD\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uA9E0-\uA9E4\uA9E6-\uA9EF\uA9FA-\uA9FE\uAA00-\uAA28\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA7E-\uAAAF\uAAB1\uAAB5\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uAAE0-\uAAEA\uAAF2-\uAAF4\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDE80-\uDE9C\uDEA0-\uDED0\uDF00-\uDF1F\uDF30-\uDF4A\uDF50-\uDF75\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00\uDE10-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE4\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC03-\uDC37\uDC83-\uDCAF\uDCD0-\uDCE8\uDD03-\uDD26\uDD50-\uDD72\uDD76\uDD83-\uDDB2\uDDC1-\uDDC4\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE2B\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEDE\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3D\uDF50\uDF5D-\uDF61]|\uD805[\uDC80-\uDCAF\uDCC4\uDCC5\uDCC7\uDD80-\uDDAE\uDDD8-\uDDDB\uDE00-\uDE2F\uDE44\uDE80-\uDEAA\uDF00-\uDF19]|\uD806[\uDCA0-\uDCDF\uDCFF\uDEC0-\uDEF8]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDED0-\uDEED\uDF00-\uDF2F\uDF40-\uDF43\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50\uDF93-\uDF9F]|\uD82C[\uDC00\uDC01]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB]|\uD83A[\uDC00-\uDCC4]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1]|\uD87E[\uDC00-\uDE1D]/, ID_Continue: /[0-9A-Z_a-z\xAA\xB5\xB7\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0300-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u0483-\u0487\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u0591-\u05BD\u05BF\u05C1\u05C2\u05C4\u05C5\u05C7\u05D0-\u05EA\u05F0-\u05F2\u0610-\u061A\u0620-\u0669\u066E-\u06D3\u06D5-\u06DC\u06DF-\u06E8\u06EA-\u06FC\u06FF\u0710-\u074A\u074D-\u07B1\u07C0-\u07F5\u07FA\u0800-\u082D\u0840-\u085B\u08A0-\u08B4\u08E3-\u0963\u0966-\u096F\u0971-\u0983\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BC-\u09C4\u09C7\u09C8\u09CB-\u09CE\u09D7\u09DC\u09DD\u09DF-\u09E3\u09E6-\u09F1\u0A01-\u0A03\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A3C\u0A3E-\u0A42\u0A47\u0A48\u0A4B-\u0A4D\u0A51\u0A59-\u0A5C\u0A5E\u0A66-\u0A75\u0A81-\u0A83\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABC-\u0AC5\u0AC7-\u0AC9\u0ACB-\u0ACD\u0AD0\u0AE0-\u0AE3\u0AE6-\u0AEF\u0AF9\u0B01-\u0B03\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3C-\u0B44\u0B47\u0B48\u0B4B-\u0B4D\u0B56\u0B57\u0B5C\u0B5D\u0B5F-\u0B63\u0B66-\u0B6F\u0B71\u0B82\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BBE-\u0BC2\u0BC6-\u0BC8\u0BCA-\u0BCD\u0BD0\u0BD7\u0BE6-\u0BEF\u0C00-\u0C03\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D-\u0C44\u0C46-\u0C48\u0C4A-\u0C4D\u0C55\u0C56\u0C58-\u0C5A\u0C60-\u0C63\u0C66-\u0C6F\u0C81-\u0C83\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBC-\u0CC4\u0CC6-\u0CC8\u0CCA-\u0CCD\u0CD5\u0CD6\u0CDE\u0CE0-\u0CE3\u0CE6-\u0CEF\u0CF1\u0CF2\u0D01-\u0D03\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D-\u0D44\u0D46-\u0D48\u0D4A-\u0D4E\u0D57\u0D5F-\u0D63\u0D66-\u0D6F\u0D7A-\u0D7F\u0D82\u0D83\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0DCA\u0DCF-\u0DD4\u0DD6\u0DD8-\u0DDF\u0DE6-\u0DEF\u0DF2\u0DF3\u0E01-\u0E3A\u0E40-\u0E4E\u0E50-\u0E59\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB9\u0EBB-\u0EBD\u0EC0-\u0EC4\u0EC6\u0EC8-\u0ECD\u0ED0-\u0ED9\u0EDC-\u0EDF\u0F00\u0F18\u0F19\u0F20-\u0F29\u0F35\u0F37\u0F39\u0F3E-\u0F47\u0F49-\u0F6C\u0F71-\u0F84\u0F86-\u0F97\u0F99-\u0FBC\u0FC6\u1000-\u1049\u1050-\u109D\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u135D-\u135F\u1369-\u1371\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1714\u1720-\u1734\u1740-\u1753\u1760-\u176C\u176E-\u1770\u1772\u1773\u1780-\u17D3\u17D7\u17DC\u17DD\u17E0-\u17E9\u180B-\u180D\u1810-\u1819\u1820-\u1877\u1880-\u18AA\u18B0-\u18F5\u1900-\u191E\u1920-\u192B\u1930-\u193B\u1946-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u19D0-\u19DA\u1A00-\u1A1B\u1A20-\u1A5E\u1A60-\u1A7C\u1A7F-\u1A89\u1A90-\u1A99\u1AA7\u1AB0-\u1ABD\u1B00-\u1B4B\u1B50-\u1B59\u1B6B-\u1B73\u1B80-\u1BF3\u1C00-\u1C37\u1C40-\u1C49\u1C4D-\u1C7D\u1CD0-\u1CD2\u1CD4-\u1CF6\u1CF8\u1CF9\u1D00-\u1DF5\u1DFC-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u203F\u2040\u2054\u2071\u207F\u2090-\u209C\u20D0-\u20DC\u20E1\u20E5-\u20F0\u2102\u2107\u210A-\u2113\u2115\u2118-\u211D\u2124\u2126\u2128\u212A-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D7F-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2DE0-\u2DFF\u3005-\u3007\u3021-\u302F\u3031-\u3035\u3038-\u303C\u3041-\u3096\u3099-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FD5\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA62B\uA640-\uA66F\uA674-\uA67D\uA67F-\uA6F1\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AD\uA7B0-\uA7B7\uA7F7-\uA827\uA840-\uA873\uA880-\uA8C4\uA8D0-\uA8D9\uA8E0-\uA8F7\uA8FB\uA8FD\uA900-\uA92D\uA930-\uA953\uA960-\uA97C\uA980-\uA9C0\uA9CF-\uA9D9\uA9E0-\uA9FE\uAA00-\uAA36\uAA40-\uAA4D\uAA50-\uAA59\uAA60-\uAA76\uAA7A-\uAAC2\uAADB-\uAADD\uAAE0-\uAAEF\uAAF2-\uAAF6\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABEA\uABEC\uABED\uABF0-\uABF9\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE00-\uFE0F\uFE20-\uFE2F\uFE33\uFE34\uFE4D-\uFE4F\uFE70-\uFE74\uFE76-\uFEFC\uFF10-\uFF19\uFF21-\uFF3A\uFF3F\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDDFD\uDE80-\uDE9C\uDEA0-\uDED0\uDEE0\uDF00-\uDF1F\uDF30-\uDF4A\uDF50-\uDF7A\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDCA0-\uDCA9\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00-\uDE03\uDE05\uDE06\uDE0C-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE38-\uDE3A\uDE3F\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE6\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC00-\uDC46\uDC66-\uDC6F\uDC7F-\uDCBA\uDCD0-\uDCE8\uDCF0-\uDCF9\uDD00-\uDD34\uDD36-\uDD3F\uDD50-\uDD73\uDD76\uDD80-\uDDC4\uDDCA-\uDDCC\uDDD0-\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE37\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEEA\uDEF0-\uDEF9\uDF00-\uDF03\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3C-\uDF44\uDF47\uDF48\uDF4B-\uDF4D\uDF50\uDF57\uDF5D-\uDF63\uDF66-\uDF6C\uDF70-\uDF74]|\uD805[\uDC80-\uDCC5\uDCC7\uDCD0-\uDCD9\uDD80-\uDDB5\uDDB8-\uDDC0\uDDD8-\uDDDD\uDE00-\uDE40\uDE44\uDE50-\uDE59\uDE80-\uDEB7\uDEC0-\uDEC9\uDF00-\uDF19\uDF1D-\uDF2B\uDF30-\uDF39]|\uD806[\uDCA0-\uDCE9\uDCFF\uDEC0-\uDEF8]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDE60-\uDE69\uDED0-\uDEED\uDEF0-\uDEF4\uDF00-\uDF36\uDF40-\uDF43\uDF50-\uDF59\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50-\uDF7E\uDF8F-\uDF9F]|\uD82C[\uDC00\uDC01]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99\uDC9D\uDC9E]|\uD834[\uDD65-\uDD69\uDD6D-\uDD72\uDD7B-\uDD82\uDD85-\uDD8B\uDDAA-\uDDAD\uDE42-\uDE44]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB\uDFCE-\uDFFF]|\uD836[\uDE00-\uDE36\uDE3B-\uDE6C\uDE75\uDE84\uDE9B-\uDE9F\uDEA1-\uDEAF]|\uD83A[\uDC00-\uDCC4\uDCD0-\uDCD6]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1]|\uD87E[\uDC00-\uDE1D]|\uDB40[\uDD00-\uDDEF]/ };
        function X(e, t) { if (z(e.charCodeAt(t))) {
            if (W(e.charCodeAt(t + 1)))
                return e.charAt(t) + e.charAt(t + 1);
        }
        else if (W(e.charCodeAt(t)) && z(e.charCodeAt(t - 1)))
            return e.charAt(t - 1) + e.charAt(t); return e.charAt(t); }
        function z(e) { return e >= 55296 && e <= 56319; }
        function W(e) { return e >= 56320 && e <= 57343; }
        function Y(e) { return e >= 48 && e <= 57; }
        function q(e) { var t = e.charCodeAt(0); return H.ID_Start.test(e) || 36 == t || 95 == t; }
        function $(e) { var t = e.charCodeAt(0); return H.ID_Continue.test(e) || 36 == t || 95 == t || 8204 == t || 8205 == t; }
        function j(e) { return /^[a-z_$][a-z0-9_$]*$/i.test(e); }
        function Z(e, t = !0) { if (!t && e.includes("e"))
            return NaN; if (w.test(e))
            return parseInt(e.substr(2), 16); if (x.test(e))
            return parseInt(e.substr(1), 8); if (N.test(e))
            return parseInt(e.substr(2), 8); if (k.test(e))
            return parseInt(e.substr(2), 2); if (I.test(e))
            return parseFloat(e); var n = parseFloat(e); return n == e ? n : void 0; }
        class J extends Error {
            constructor(e, t, n, i, o) { super(), this.name = "SyntaxError", this.message = e, this.filename = t, this.line = n, this.col = i, this.pos = o; }
        }
        function Q(e, t, n, i, o) { throw new J(e, t, n, i, o); }
        function ee(e, t, n) { return e.type == t && (null == n || e.value == n); }
        var te = {};
        function ne(e, t, n, i) { var o = { text: e, filename: t, pos: 0, tokpos: 0, line: 1, tokline: 0, col: 0, tokcol: 0, newline_before: !1, regex_allowed: !1, brace_counter: 0, template_braces: [], comments_before: [], directives: {}, directive_stack: [] }; function r() { return X(o.text, o.pos); } function a(e, t) { var n = X(o.text, o.pos++); if (e && !n)
            throw te; return B.has(n) ? (o.newline_before = o.newline_before || !t, ++o.line, o.col = 0, "\r" == n && "\n" == r() && (++o.pos, n = "\n")) : (n.length > 1 && (++o.pos, ++o.col), ++o.col), n; } function s(e) { for (; e--;)
            a(); } function u(e) { return o.text.substr(o.pos, e.length) == e; } function c(e, t) { var n = o.text.indexOf(e, o.pos); if (t && -1 == n)
            throw te; return n; } function l() { o.tokline = o.line, o.tokcol = o.col, o.tokpos = o.pos; } var f = !1, p = null; function _(n, i, r) { o.regex_allowed = "operator" == n && !oe.has(i) || "keyword" == n && F.has(i) || "punc" == n && U.has(i) || "arrow" == n, "punc" == n && "." == i ? f = !0 : r || (f = !1); var a = { type: n, value: i, line: o.tokline, col: o.tokcol, pos: o.tokpos, endline: o.line, endcol: o.col, endpos: o.pos, nlb: o.newline_before, file: t }; return /^(?:num|string|regexp)$/i.test(n) && (a.raw = e.substring(a.pos, a.endpos)), r || (a.comments_before = o.comments_before, a.comments_after = o.comments_before = []), o.newline_before = !1, a = new le(a), r || (p = a), a; } function d() { for (; P.has(r());)
            a(); } function m(e) { Q(e, t, o.tokline, o.tokcol, o.tokpos); } function h(e) { var t = !1, n = !1, i = !1, o = "." == e, s = !1, u = function (e) { for (var t, n = "", i = 0; (t = r()) && e(t, i++);)
            n += a(); return n; }((function (r, a) { if (s)
            return !1; switch (r.charCodeAt(0)) {
            case 98:
            case 66: return i = !0;
            case 111:
            case 79:
            case 120:
            case 88: return !i && (i = !0);
            case 101:
            case 69: return !!i || !t && (t = n = !0);
            case 45: return n || 0 == a && !e;
            case 43: return n;
            case n = !1, 46: return !(o || i || t) && (o = !0);
        } return "n" === r ? (s = !0, !0) : R.test(r); })); if (e && (u = e + u), x.test(u) && K.has_directive("use strict") && m("Legacy octal literals are not allowed in strict mode"), u.endsWith("n")) {
            const e = u.slice(0, -1), t = Z(e, !1);
            if (!o && L.test(u) && !isNaN(t))
                return _("big_int", e);
            m("Invalid or unexpected token");
        } var c = Z(u); if (!isNaN(c))
            return _("num", c); m("Invalid syntax: " + u); } function E(e) { return e >= "0" && e <= "7"; } function g(e, t, n) { var i, s = a(!0, e); switch (s.charCodeAt(0)) {
            case 110: return "\n";
            case 114: return "\r";
            case 116: return "\t";
            case 98: return "\b";
            case 118: return "\v";
            case 102: return "\f";
            case 120: return String.fromCharCode(D(2, t));
            case 117:
                if ("{" == r()) {
                    for (a(!0), "}" === r() && m("Expecting hex-character between {}"); "0" == r();)
                        a(!0);
                    var u, l = c("}", !0) - o.pos;
                    return (l > 6 || (u = D(l, t)) > 1114111) && m("Unicode reference out of bounds"), a(!0), (i = u) > 65535 ? (i -= 65536, String.fromCharCode(55296 + (i >> 10)) + String.fromCharCode(i % 1024 + 56320)) : String.fromCharCode(i);
                }
                return String.fromCharCode(D(4, t));
            case 10: return "";
            case 13: if ("\n" == r())
                return a(!0, e), "";
        } if (E(s)) {
            if (n && t) {
                "0" === s && !E(r()) || m("Octal escape sequences are not allowed in template strings");
            }
            return function (e, t) { var n = r(); n >= "0" && n <= "7" && (e += a(!0))[0] <= "3" && (n = r()) >= "0" && n <= "7" && (e += a(!0)); if ("0" === e)
                return "\0"; e.length > 0 && K.has_directive("use strict") && t && m("Legacy octal escape sequences are not allowed in strict mode"); return String.fromCharCode(parseInt(e, 8)); }(s, t);
        } return s; } function D(e, t) { for (var n = 0; e > 0; --e) {
            if (!t && isNaN(parseInt(r(), 16)))
                return parseInt(n, 16) || "";
            var i = a(!0);
            isNaN(parseInt(i, 16)) && m("Invalid hex-character pattern in string"), n += i;
        } return parseInt(n, 16); } var S = I("Unterminated string constant", (function () { for (var e = a(), t = "";;) {
            var n = a(!0, !0);
            if ("\\" == n)
                n = g(!0, !0);
            else if ("\r" == n || "\n" == n)
                m("Unterminated string constant");
            else if (n == e)
                break;
            t += n;
        } var i = _("string", t); return i.quote = e, i; })), v = I("Unterminated template", (function (e) { e && o.template_braces.push(o.brace_counter); var t, n, i = "", s = ""; for (a(!0, !0); "`" != (t = a(!0, !0));) {
            if ("\r" == t)
                "\n" == r() && ++o.pos, t = "\n";
            else if ("$" == t && "{" == r())
                return a(!0, !0), o.brace_counter++, (n = _(e ? "template_head" : "template_substitution", i)).raw = s, n;
            if (s += t, "\\" == t) {
                var u = o.pos;
                t = g(!0, !(p && ("name" === p.type || "punc" === p.type && (")" === p.value || "]" === p.value))), !0), s += o.text.substr(u, o.pos - u);
            }
            i += t;
        } return o.template_braces.pop(), (n = _(e ? "template_head" : "template_substitution", i)).raw = s, n.end = !0, n; })); function A(e) { var t, n = o.regex_allowed, i = function () { for (var e = o.text, t = o.pos, n = o.text.length; t < n; ++t) {
            var i = e[t];
            if (B.has(i))
                return t;
        } return -1; }(); return -1 == i ? (t = o.text.substr(o.pos), o.pos = o.text.length) : (t = o.text.substring(o.pos, i), o.pos = i), o.col = o.tokcol + (o.pos - o.tokpos), o.comments_before.push(_(e, t, !0)), o.regex_allowed = n, K; } var T = I("Unterminated multiline comment", (function () { var e = o.regex_allowed, t = c("*/", !0), n = o.text.substring(o.pos, t).replace(/\r\n|\r|\u2028|\u2029/g, "\n"); return s(function (e) { for (var t = 0, n = 0; n < e.length; n++)
            z(e.charCodeAt(n)) && W(e.charCodeAt(n + 1)) && (t++, n++); return e.length - t; }(n) + 2), o.comments_before.push(_("comment2", n, !0)), o.newline_before = o.newline_before || n.includes("\n"), o.regex_allowed = e, K; })), y = I("Unterminated identifier name", (function () { var e, t, n = !1, i = function () { return n = !0, a(), "u" !== r() && m("Expecting UnicodeEscapeSequence -- uXXXX or u{XXXX}"), g(!1, !0); }; if ("\\" === (e = r()))
            q(e = i()) || m("First identifier char is an invalid identifier char");
        else {
            if (!q(e))
                return "";
            a();
        } for (; null != (t = r());) {
            if ("\\" === (t = r()))
                $(t = i()) || m("Invalid escaped identifier char");
            else {
                if (!$(t))
                    break;
                a();
            }
            e += t;
        } return O.has(e) && n && m("Escaped characters are not allowed in keywords"), e; })), w = I("Unterminated regular expression", (function (e) { for (var t, n = !1, i = !1; t = a(!0);)
            if (B.has(t))
                m("Unexpected line terminator");
            else if (n)
                e += "\\" + t, n = !1;
            else if ("[" == t)
                i = !0, e += t;
            else if ("]" == t && i)
                i = !1, e += t;
            else {
                if ("/" == t && !i)
                    break;
                "\\" == t ? n = !0 : e += t;
            } return _("regexp", { source: e, flags: y() }); })); function N(e) { return _("operator", function e(t) { if (!r())
            return t; var n = t + r(); return V.has(n) ? (a(), e(n)) : t; }(e || a())); } function k() { switch (a(), r()) {
            case "/": return a(), A("comment1");
            case "*": return a(), T();
        } return o.regex_allowed ? w("") : N("/"); } function I(e, t) { return function (n) { try {
            return t(n);
        }
        catch (t) {
            if (t !== te)
                throw t;
            m(e);
        } }; } function K(e) { if (null != e)
            return w(e); for (i && 0 == o.pos && u("#!") && (l(), s(2), A("comment5"));;) {
            if (d(), l(), n) {
                if (u("\x3c!--")) {
                    s(4), A("comment3");
                    continue;
                }
                if (u("--\x3e") && o.newline_before) {
                    s(3), A("comment4");
                    continue;
                }
            }
            var t = r();
            if (!t)
                return _("eof");
            var c = t.charCodeAt(0);
            switch (c) {
                case 34:
                case 39: return S();
                case 46: return a(), Y(r().charCodeAt(0)) ? h(".") : "." === r() ? (a(), a(), _("expand", "...")) : _("punc", ".");
                case 47:
                    var p = k();
                    if (p === K)
                        continue;
                    return p;
                case 61: return a(), ">" === r() ? (a(), _("arrow", "=>")) : N("=");
                case 96: return v(!0);
                case 123:
                    o.brace_counter++;
                    break;
                case 125: if (o.brace_counter--, o.template_braces.length > 0 && o.template_braces[o.template_braces.length - 1] === o.brace_counter)
                    return v(!1);
            }
            if (Y(c))
                return h();
            if (G.has(t))
                return _("punc", a());
            if (M.has(t))
                return N();
            if (92 == c || q(t))
                return E = void 0, E = y(), f ? _("name", E) : C.has(E) ? _("atom", E) : b.has(E) ? V.has(E) ? _("operator", E) : _("keyword", E) : _("name", E);
            break;
        } var E; m("Unexpected character '" + t + "'"); } return K.next = a, K.peek = r, K.context = function (e) { return e && (o = e), o; }, K.add_directive = function (e) { o.directive_stack[o.directive_stack.length - 1].push(e), void 0 === o.directives[e] ? o.directives[e] = 1 : o.directives[e]++; }, K.push_directives_stack = function () { o.directive_stack.push([]); }, K.pop_directives_stack = function () { for (var e = o.directive_stack[o.directive_stack.length - 1], t = 0; t < e.length; t++)
            o.directives[e[t]]--; o.directive_stack.pop(); }, K.has_directive = function (e) { return o.directives[e] > 0; }, K; }
        var ie = E(["typeof", "void", "delete", "--", "++", "!", "~", "-", "+"]), oe = E(["--", "++"]), re = E(["=", "+=", "-=", "/=", "*=", "**=", "%=", ">>=", "<<=", ">>>=", "|=", "^=", "&="]), ae = function (e, t) { for (var n = 0; n < e.length; ++n)
            for (var i = e[n], o = 0; o < i.length; ++o)
                t[i[o]] = n + 1; return t; }([["||"], ["&&"], ["|"], ["^"], ["&"], ["==", "===", "!=", "!=="], ["<", ">", "<=", ">=", "in", "instanceof"], [">>", "<<", ">>>"], ["+", "-"], ["*", "/", "%"], ["**"]], {}), se = E(["atom", "num", "big_int", "string", "regexp", "name"]);
        function ue(e, t) { const n = new Map; t = r(t, { bare_returns: !1, ecma: 2017, expression: !1, filename: null, html5_comments: !0, module: !1, shebang: !0, strict: !1, toplevel: null }, !0); var i = { input: "string" == typeof e ? ne(e, t.filename, t.html5_comments, t.shebang) : e, token: null, prev: null, peeked: null, in_function: 0, in_async: -1, in_generator: -1, in_directives: !0, in_loop: 0, labels: [] }; function o(e, t) { return ee(i.token, e, t); } function a() { return i.peeked || (i.peeked = i.input()); } function s() { return i.prev = i.token, i.peeked || a(), i.token = i.peeked, i.peeked = null, i.in_directives = i.in_directives && ("string" == i.token.type || o("punc", ";")), i.token; } function u() { return i.prev; } function c(e, t, n, o) { var r = i.input.context(); Q(e, r.filename, null != t ? t : r.tokline, null != n ? n : r.tokcol, null != o ? o : r.tokpos); } function l(e, t) { c(t, e.line, e.col); } function f(e) { null == e && (e = i.token), l(e, "Unexpected token: " + e.type + " (" + e.value + ")"); } function p(e, t) { if (o(e, t))
            return s(); l(i.token, "Unexpected token " + i.token.type + " «" + i.token.value + "», expected " + e + " «" + t + "»"); } function _(e) { return p("punc", e); } function d(e) { return e.nlb || !e.comments_before.every(e => !e.nlb); } function m() { return !t.strict && (o("eof") || o("punc", "}") || d(i.token)); } function h() { return i.in_generator === i.in_function; } function E() { return i.in_async === i.in_function; } function g(e) { o("punc", ";") ? s() : e || m() || f(); } function D() { _("("); var e = et(!0); return _(")"), e; } function S(e) { return function (...t) { const n = i.token, o = e(...t); return o.start = n, o.end = u(), o; }; } function v() { (o("operator", "/") || o("operator", "/=")) && (i.peeked = null, i.token = i.input(i.token.value.substr(1))); } i.token = s(); var A = S((function (e, n, r) { switch (v(), i.token.type) {
            case "string":
                if (i.in_directives) {
                    var h = a();
                    !i.token.raw.includes("\\") && (ee(h, "punc", ";") || ee(h, "punc", "}") || d(h) || ee(h, "eof")) ? i.input.add_directive(i.token.value) : i.in_directives = !1;
                }
                var S = i.in_directives, y = T();
                return S && y.body instanceof ln ? new de(y.body) : y;
            case "template_head":
            case "num":
            case "big_int":
            case "regexp":
            case "operator":
            case "atom": return T();
            case "name":
                if ("async" == i.token.value && ee(a(), "keyword", "function"))
                    return s(), s(), n && c("functions are not allowed as the body of a loop"), F(Pe, !1, !0, e);
                if ("import" == i.token.value && !ee(a(), "punc", "(")) {
                    s();
                    var C = function () { var e, t, n = u(); o("name") && (e = le(Qt)); o("punc", ",") && s(); ((t = J(!0)) || e) && p("name", "from"); var r = i.token; "string" !== r.type && f(); return s(), new pt({ start: n, imported_name: e, imported_names: t, module_name: new ln({ start: r, value: r.value, quote: r.quote, end: r }), end: i.token }); }();
                    return g(), C;
                }
                return ee(a(), "punc", ":") ? function () { var e = le(tn); "await" === e.name && E() && l(i.prev, "await cannot be used as label inside async function"); i.labels.some(t => t.name === e.name) && c("Label " + e.name + " defined twice"); _(":"), i.labels.push(e); var t = A(); i.labels.pop(), t instanceof Te || e.references.forEach((function (t) { t instanceof $e && (t = t.label.start, c("Continue label `" + e.name + "` refers to non-IterationStatement.", t.line, t.col, t.pos)); })); return new Ae({ body: t, label: e }); }() : T();
            case "punc": switch (i.token.value) {
                case "{": return new De({ start: i.token, body: N(), end: u() });
                case "[":
                case "(": return T();
                case ";": return i.in_directives = !1, s(), new Se;
                default: f();
            }
            case "keyword": switch (i.token.value) {
                case "break": return s(), b(qe);
                case "continue": return s(), b($e);
                case "debugger": return s(), g(), new _e;
                case "do":
                    s();
                    var O = St(A);
                    p("keyword", "while");
                    var M = D();
                    return g(!0), new be({ body: O, condition: M });
                case "while": return s(), new Ce({ condition: D(), body: St((function () { return A(!1, !0); })) });
                case "for": return s(), function () { var e = "`for await` invalid in this context", t = i.token; "name" == t.type && "await" == t.value ? (E() || l(t, e), s()) : t = !1; _("("); var n = null; if (o("punc", ";"))
                    t && l(t, e);
                else {
                    n = o("keyword", "var") ? (s(), L(!0)) : o("keyword", "let") ? (s(), V(!0)) : o("keyword", "const") ? (s(), P(!0)) : et(!0, !0);
                    var r = o("operator", "in"), a = o("name", "of");
                    if (t && !a && l(t, e), r || a)
                        return n instanceof at ? n.definitions.length > 1 && l(n.start, "Only one variable declaration allowed in for..in loop") : He(n) || (n = Xe(n)) instanceof Be || l(n.start, "Invalid left-hand side in for..in loop"), s(), r ? function (e) { var t = et(!0); return _(")"), new Fe({ init: e, object: t, body: St((function () { return A(!1, !0); })) }); }(n) : function (e, t) { var n = e instanceof at ? e.definitions[0].name : null, i = et(!0); return _(")"), new Me({ await: t, init: e, name: n, object: i, body: St((function () { return A(!1, !0); })) }); }(n, !!t);
                } return function (e) { _(";"); var t = o("punc", ";") ? null : et(!0); _(";"); var n = o("punc", ")") ? null : et(!0); return _(")"), new Oe({ init: e, condition: t, step: n, body: St((function () { return A(!1, !0); })) }); }(n); }();
                case "class": return s(), n && c("classes are not allowed as the body of a loop"), r && c("classes are not allowed as the body of an if"), q(Lt);
                case "function": return s(), n && c("functions are not allowed as the body of a loop"), F(Pe, !1, !1, e);
                case "if": return s(), function () { var e = D(), t = A(!1, !1, !0), n = null; o("keyword", "else") && (s(), n = A(!1, !1, !0)); return new Je({ condition: e, body: t, alternative: n }); }();
                case "return":
                    0 != i.in_function || t.bare_returns || c("'return' outside of function"), s();
                    var w = null;
                    return o("punc", ";") ? s() : m() || (w = et(!0), g()), new ze({ value: w });
                case "switch": return s(), new Qe({ expression: D(), body: St(k) });
                case "throw":
                    s(), d(i.token) && c("Illegal newline after 'throw'");
                    w = et(!0);
                    return g(), new We({ value: w });
                case "try": return s(), function () { var e = N(), t = null, n = null; if (o("keyword", "catch")) {
                    var r = i.token;
                    if (s(), o("punc", "{"))
                        var a = null;
                    else {
                        _("(");
                        a = R(void 0, Jt);
                        _(")");
                    }
                    t = new ot({ start: r, argname: a, body: N(), end: u() });
                } if (o("keyword", "finally")) {
                    r = i.token;
                    s(), n = new rt({ start: r, body: N(), end: u() });
                } t || n || c("Missing catch/finally blocks"); return new it({ body: e, bcatch: t, bfinally: n }); }();
                case "var":
                    s();
                    C = L();
                    return g(), C;
                case "let":
                    s();
                    C = V();
                    return g(), C;
                case "const":
                    s();
                    C = P();
                    return g(), C;
                case "with": return i.input.has_directive("use strict") && c("Strict mode may not include a with statement"), s(), new Re({ expression: D(), body: A() });
                case "export": if (!ee(a(), "punc", "(")) {
                    s();
                    C = function () { var e, t, n, r, c, l = i.token; if (o("keyword", "default"))
                        e = !0, s();
                    else if (t = J(!1)) {
                        if (o("name", "from")) {
                            s();
                            var p = i.token;
                            return "string" !== p.type && f(), s(), new _t({ start: l, is_default: e, exported_names: t, module_name: new ln({ start: p, value: p.value, quote: p.quote, end: p }), end: u() });
                        }
                        return new _t({ start: l, is_default: e, exported_names: t, end: u() });
                    } o("punc", "{") || e && (o("keyword", "class") || o("keyword", "function")) && ee(a(), "punc") ? (r = et(!1), g()) : (n = A(e)) instanceof at && e ? f(n.start) : n instanceof at || n instanceof ke || n instanceof Lt ? c = n : n instanceof me ? r = n.body : f(n.start); return new _t({ start: l, is_default: e, exported_value: r, exported_definition: c, end: u() }); }();
                    return o("punc", ";") && g(), C;
                }
            }
        } f(); })); function T(e) { return new me({ body: (e = et(!0), g(), e) }); } function b(e) { var t, n = null; m() || (n = le(an, !0)), null != n ? ((t = i.labels.find(e => e.name === n.name)) || c("Undefined label " + n.name), n.thedef = t) : 0 == i.in_loop && c(e.TYPE + " not inside a loop or switch"), g(); var o = new e({ label: n }); return t && t.references.push(o), o; } var C = function (e, t, n) { d(i.token) && c("Unexpected newline before arrow (=>)"), p("arrow", "=>"); var r = x(o("punc", "{"), !1, n), a = r instanceof Array && r.length ? r[r.length - 1].end : r instanceof Array ? e : r.end; return new Ve({ start: e, end: a, async: n, argnames: t, body: r }); }, F = function (e, t, n, i) { var r = e === Pe, a = o("operator", "*"); a && s(); var c = o("name") ? le(r ? Wt : $t) : null; r && !c && (i ? e = Le : f()), !c || e === Ie || c instanceof Kt || f(u()); var l = [], p = x(!0, a || t, n, c, l); return new e({ start: l.start, end: p.end, is_generator: a, async: n, name: c, argnames: l, body: p }); }; function M(e, t) { var n = new Set, i = !1, o = !1, r = !1, a = !!t, s = { add_parameter: function (t) { if (n.has(t.value))
                !1 === i && (i = t), s.check_strict();
            else if (n.add(t.value), e)
                switch (t.value) {
                    case "arguments":
                    case "eval":
                    case "yield":
                        a && l(t, "Unexpected " + t.value + " identifier as parameter inside strict mode");
                        break;
                    default: O.has(t.value) && f();
                } }, mark_default_assignment: function (e) { !1 === o && (o = e); }, mark_spread: function (e) { !1 === r && (r = e); }, mark_strict_mode: function () { a = !0; }, is_strict: function () { return !1 !== o || !1 !== r || a; }, check_strict: function () { s.is_strict() && !1 !== i && l(i, "Parameter " + i.value + " was used already"); } }; return s; } function R(e, t) { var n, r = !1; return void 0 === e && (e = M(!0, i.input.has_directive("use strict"))), o("expand", "...") && (r = i.token, e.mark_spread(i.token), s()), n = w(e, t), o("operator", "=") && !1 === r && (e.mark_default_assignment(i.token), s(), n = new Ct({ start: n.start, left: n, operator: "=", right: et(!1), end: i.token })), !1 !== r && (o("punc", ")") || f(), n = new Ne({ start: r, expression: n, end: r })), e.check_strict(), n; } function w(e, t) { var n, r = [], l = !0, p = !1, d = i.token; if (void 0 === e && (e = M(!1, i.input.has_directive("use strict"))), t = void 0 === t ? zt : t, o("punc", "[")) {
            for (s(); !o("punc", "]");) {
                if (l ? l = !1 : _(","), o("expand", "...") && (p = !0, n = i.token, e.mark_spread(i.token), s()), o("punc"))
                    switch (i.token.value) {
                        case ",":
                            r.push(new gn({ start: i.token, end: i.token }));
                            continue;
                        case "]": break;
                        case "[":
                        case "{":
                            r.push(w(e, t));
                            break;
                        default: f();
                    }
                else
                    o("name") ? (e.add_parameter(i.token), r.push(le(t))) : c("Invalid function parameter");
                o("operator", "=") && !1 === p && (e.mark_default_assignment(i.token), s(), r[r.length - 1] = new Ct({ start: r[r.length - 1].start, left: r[r.length - 1], operator: "=", right: et(!1), end: i.token })), p && (o("punc", "]") || c("Rest element must be last element"), r[r.length - 1] = new Ne({ start: n, expression: r[r.length - 1], end: n }));
            }
            return _("]"), e.check_strict(), new Be({ start: d, names: r, is_array: !0, end: u() });
        } if (o("punc", "{")) {
            for (s(); !o("punc", "}");) {
                if (l ? l = !1 : _(","), o("expand", "...") && (p = !0, n = i.token, e.mark_spread(i.token), s()), o("name") && (ee(a(), "punc") || ee(a(), "operator")) && [",", "}", "="].includes(a().value)) {
                    e.add_parameter(i.token);
                    var m = u(), h = le(t);
                    p ? r.push(new Ne({ start: n, expression: h, end: h.end })) : r.push(new Rt({ start: m, key: h.name, value: h, end: h.end }));
                }
                else {
                    if (o("punc", "}"))
                        continue;
                    var E = i.token, g = te();
                    null === g ? f(u()) : "name" !== u().type || o("punc", ":") ? (_(":"), r.push(new Rt({ start: E, quote: E.quote, key: g, value: w(e, t), end: u() }))) : r.push(new Rt({ start: u(), key: g, value: new t({ start: u(), name: g, end: u() }), end: u() }));
                }
                p ? o("punc", "}") || c("Rest element must be last element") : o("operator", "=") && (e.mark_default_assignment(i.token), s(), r[r.length - 1].value = new Ct({ start: r[r.length - 1].value.start, left: r[r.length - 1].value, operator: "=", right: et(!1), end: i.token }));
            }
            return _("}"), e.check_strict(), new Be({ start: d, names: r, is_array: !1, end: u() });
        } if (o("name"))
            return e.add_parameter(i.token), le(t); c("Invalid function parameter"); } function x(e, n, r, a, u) { var c = i.in_loop, l = i.labels, p = i.in_generator, d = i.in_async; if (++i.in_function, n && (i.in_generator = i.in_function), r && (i.in_async = i.in_function), u && function (e) { var n = M(!0, i.input.has_directive("use strict")); for (_("("); !o("punc", ")");) {
            var r = R(n);
            if (e.push(r), o("punc", ")") || (_(","), o("punc", ")") && t.ecma < 2017 && f()), r instanceof Ne)
                break;
        } s(); }(u), e && (i.in_directives = !0), i.in_loop = 0, i.labels = [], e) {
            i.input.push_directives_stack();
            var m = N();
            a && ce(a), u && u.forEach(ce), i.input.pop_directives_stack();
        }
        else
            m = [new ze({ start: i.token, value: et(!1), end: i.token })]; return --i.in_function, i.in_loop = c, i.labels = l, i.in_generator = p, i.in_async = d, m; } function N() { _("{"); for (var e = []; !o("punc", "}");)
            o("eof") && f(), e.push(A()); return s(), e; } function k() { _("{"); for (var e, t = [], n = null, r = null; !o("punc", "}");)
            o("eof") && f(), o("keyword", "case") ? (r && (r.end = u()), n = [], r = new nt({ start: (e = i.token, s(), e), expression: et(!0), body: n }), t.push(r), _(":")) : o("keyword", "default") ? (r && (r.end = u()), n = [], r = new tt({ start: (e = i.token, s(), _(":"), e), body: n }), t.push(r)) : (n || f(), n.push(A())); return r && (r.end = u()), s(), t; } function I(e, t) { for (var n, r = [];;) {
            var a = "var" === t ? Ut : "const" === t ? Ht : "let" === t ? Xt : null;
            if (o("punc", "{") || o("punc", "[") ? n = new lt({ start: i.token, name: w(void 0, a), value: o("operator", "=") ? (p("operator", "="), et(!1, e)) : null, end: u() }) : "import" == (n = new lt({ start: i.token, name: le(a), value: o("operator", "=") ? (s(), et(!1, e)) : e || "const" !== t ? null : c("Missing initializer in const declaration"), end: u() })).name.name && c("Unexpected token: import"), r.push(n), !o("punc", ","))
                break;
            s();
        } return r; } var L = function (e) { return new st({ start: u(), definitions: I(e, "var"), end: u() }); }, V = function (e) { return new ut({ start: u(), definitions: I(e, "let"), end: u() }); }, P = function (e) { return new ct({ start: u(), definitions: I(e, "const"), end: u() }); }; function B() { var e, t = i.token; switch (t.type) {
            case "name":
                e = ue(nn);
                break;
            case "num":
                e = new fn({ start: t, end: t, value: t.value });
                break;
            case "big_int":
                e = new pn({ start: t, end: t, value: t.value });
                break;
            case "string":
                e = new ln({ start: t, end: t, value: t.value, quote: t.quote });
                break;
            case "regexp":
                e = new _n({ start: t, end: t, value: t.value });
                break;
            case "atom": switch (t.value) {
                case "false":
                    e = new vn({ start: t, end: t });
                    break;
                case "true":
                    e = new An({ start: t, end: t });
                    break;
                case "null": e = new mn({ start: t, end: t });
            }
        } return s(), e; } function U(e, t, n, i) { var o = function (e, t) { return t ? new Ct({ start: e.start, left: e, operator: "=", right: t, end: t.end }) : e; }; return e instanceof Ft ? o(new Be({ start: e.start, end: e.end, is_array: !1, names: e.properties.map(U) }), i) : e instanceof Rt ? (e.value = U(e.value, 0, [e.key]), o(e, i)) : e instanceof gn ? e : e instanceof Be ? (e.names = e.names.map(U), o(e, i)) : e instanceof nn ? o(new zt({ name: e.name, start: e.start, end: e.end }), i) : e instanceof Ne ? (e.expression = U(e.expression), o(e, i)) : e instanceof Ot ? o(new Be({ start: e.start, end: e.end, is_array: !0, names: e.elements.map(U) }), i) : e instanceof bt ? o(U(e.left, void 0, void 0, e.right), i) : e instanceof Ct ? (e.left = U(e.left, 0, [e.left]), e) : void c("Invalid function parameter", e.start.line, e.start.col); } var G = function (e, r) { if (o("operator", "new"))
            return function (e) { var n = i.token; if (p("operator", "new"), o("punc", "."))
                return s(), p("name", "target"), he(new Bt({ start: n, end: u() }), e); var r, a = G(!1); o("punc", "(") ? (s(), r = X(")", t.ecma >= 2017)) : r = []; var c = new mt({ start: n, expression: a, args: r, end: u() }); return pe(c), he(c, e); }(e); var c, l = i.token, d = o("name", "async") && "[" != (c = a()).value && "arrow" != c.type && B(); if (o("punc")) {
            switch (i.token.value) {
                case "(":
                    if (d && !e)
                        break;
                    var m = function (e, n) { var r, a, c, l = []; for (_("("); !o("punc", ")");)
                        r && f(r), o("expand", "...") ? (r = i.token, n && (a = i.token), s(), l.push(new Ne({ start: u(), expression: et(), end: i.token }))) : l.push(et()), o("punc", ")") || (_(","), o("punc", ")") && (t.ecma < 2017 && f(), c = u(), n && (a = c))); return _(")"), e && o("arrow", "=>") ? r && c && f(c) : a && f(a), l; }(r, !d);
                    if (r && o("arrow", "=>"))
                        return C(l, m.map(U), !!d);
                    var h = d ? new dt({ expression: d, args: m }) : 1 == m.length ? m[0] : new ht({ expressions: m });
                    if (h.start) {
                        const e = l.comments_before.length;
                        if (n.set(l, e), h.start.comments_before.unshift(...l.comments_before), l.comments_before = h.start.comments_before, 0 == e && l.comments_before.length > 0) {
                            var E = l.comments_before[0];
                            E.nlb || (E.nlb = l.nlb, l.nlb = !1);
                        }
                        l.comments_after = h.start.comments_after;
                    }
                    h.start = l;
                    var g = u();
                    return h.end && (g.comments_before = h.end.comments_before, h.end.comments_after.push(...g.comments_after), g.comments_after = h.end.comments_after), h.end = g, h instanceof dt && pe(h), he(h, e);
                case "[": return he(z(), e);
                case "{": return he(Y(), e);
            }
            d || f();
        } if (r && o("name") && ee(a(), "arrow")) {
            var D = new zt({ name: i.token.value, start: l, end: l });
            return s(), C(l, [D], !!d);
        } if (o("keyword", "function")) {
            s();
            var S = F(Le, !1, !!d);
            return S.start = l, S.end = u(), he(S, e);
        } if (d)
            return he(d, e); if (o("keyword", "class")) {
            s();
            var v = q(Vt);
            return v.start = l, v.end = u(), he(v, e);
        } return o("template_head") ? he(H(), e) : se.has(i.token.type) ? he(B(), e) : void f(); }; function H() { var e = [], t = i.token; for (e.push(new Ge({ start: i.token, raw: i.token.raw, value: i.token.value, end: i.token })); !i.token.end;)
            s(), v(), e.push(et(!0)), ee("template_substitution") || f(), e.push(new Ge({ start: i.token, raw: i.token.raw, value: i.token.value, end: i.token })); return s(), new Ue({ start: t, segments: e, end: i.token }); } function X(e, t, n) { for (var r = !0, a = []; !o("punc", e) && (r ? r = !1 : _(","), !t || !o("punc", e));)
            o("punc", ",") && n ? a.push(new gn({ start: i.token, end: i.token })) : o("expand", "...") ? (s(), a.push(new Ne({ start: u(), expression: et(), end: i.token }))) : a.push(et(!1)); return s(), a; } var z = S((function () { return _("["), new Ot({ elements: X("]", !t.strict, !0) }); })), W = S((e, t) => F(Ie, e, t)), Y = S((function () { var e = i.token, n = !0, r = []; for (_("{"); !o("punc", "}") && (n ? n = !1 : _(","), t.strict || !o("punc", "}"));)
            if ("expand" != (e = i.token).type) {
                var a, c = te();
                if (o("punc", ":"))
                    null === c ? f(u()) : (s(), a = et(!1));
                else {
                    var l = $(c, e);
                    if (l) {
                        r.push(l);
                        continue;
                    }
                    a = new nn({ start: u(), name: c, end: u() });
                }
                o("operator", "=") && (s(), a = new bt({ start: e, left: a, operator: "=", right: et(!1), end: u() })), r.push(new Rt({ start: e, quote: e.quote, key: c instanceof fe ? c : "" + c, value: a, end: u() }));
            }
            else
                s(), r.push(new Ne({ start: e, expression: et(!1), end: u() })); return s(), new Ft({ properties: r }); })); function q(e) { var t, n, r, a, c = []; for (i.input.push_directives_stack(), i.input.add_directive("use strict"), "name" == i.token.type && "extends" != i.token.value && (r = le(e === Lt ? jt : Zt)), e !== Lt || r || f(), "extends" == i.token.value && (s(), a = et(!0)), _("{"); o("punc", ";");)
            s(); for (; !o("punc", "}");)
            for (t = i.token, (n = $(te(), t, !0)) || f(), c.push(n); o("punc", ";");)
                s(); return i.input.pop_directives_stack(), s(), new e({ start: t, name: r, extends: a, properties: c, end: u() }); } function $(e, t, n) { var r = function (e, t) { return "string" == typeof e || "number" == typeof e ? new Yt({ start: t, name: "" + e, end: u() }) : (null === e && f(), e); }; var a = !1, c = !1, l = !1, p = t; if (n && "static" === e && !o("punc", "(") && (c = !0, p = i.token, e = te()), "async" !== e || o("punc", "(") || o("punc", ",") || o("punc", "}") || o("operator", "=") || (a = !0, p = i.token, e = te()), null === e && (l = !0, p = i.token, null === (e = te()) && f()), o("punc", "("))
            return e = r(e, t), new Nt({ start: t, static: c, is_generator: l, async: a, key: e, quote: e instanceof Yt ? p.quote : void 0, value: W(l, a), end: u() }); const _ = i.token; if ("get" == e) {
            if (!o("punc") || o("punc", "["))
                return e = r(te(), t), new xt({ start: t, static: c, key: e, quote: e instanceof Yt ? _.quote : void 0, value: W(), end: u() });
        }
        else if ("set" == e && (!o("punc") || o("punc", "[")))
            return e = r(te(), t), new wt({ start: t, static: c, key: e, quote: e instanceof Yt ? _.quote : void 0, value: W(), end: u() }); if (n) {
            const n = (e => "string" == typeof e || "number" == typeof e ? new qt({ start: p, end: p, name: "" + e }) : (null === e && f(), e))(e), i = n instanceof qt ? p.quote : void 0;
            if (o("operator", "="))
                return s(), new It({ start: t, static: c, quote: i, key: n, value: et(!1), end: u() });
            if (o("name") || o("punc", ";") || o("punc", "}"))
                return new It({ start: t, static: c, quote: i, key: n, end: u() });
        } } function j(e) { function t(e) { return new e({ name: te(), start: u(), end: u() }); } var n, r, a = e ? en : rn, c = e ? Qt : on, l = i.token; return e ? n = t(a) : r = t(c), o("name", "as") ? (s(), e ? r = t(c) : n = t(a)) : e ? r = new c(n) : n = new a(r), new ft({ start: l, foreign_name: n, name: r, end: u() }); } function Z(e, t) { var n, o = e ? en : rn, r = e ? Qt : on, a = i.token, s = u(); return t = t || new r({ name: "*", start: a, end: s }), n = new o({ name: "*", start: a, end: s }), new ft({ start: a, foreign_name: n, name: t, end: s }); } function J(e) { var t; if (o("punc", "{")) {
            for (s(), t = []; !o("punc", "}");)
                t.push(j(e)), o("punc", ",") && s();
            s();
        }
        else if (o("operator", "*")) {
            var n;
            s(), e && o("name", "as") && (s(), n = le(e ? Qt : rn)), t = [Z(e, n)];
        } return t; } function te() { var e = i.token; switch (e.type) {
            case "punc":
                if ("[" === e.value) {
                    s();
                    var t = et(!1);
                    return _("]"), t;
                }
                f(e);
            case "operator":
                if ("*" === e.value)
                    return s(), null;
                ["delete", "in", "instanceof", "new", "typeof", "void"].includes(e.value) || f(e);
            case "name": "yield" == e.value && (h() ? l(e, "Yield cannot be used as identifier inside generators") : ee(a(), "punc", ":") || ee(a(), "punc", "(") || !i.input.has_directive("use strict") || l(e, "Unexpected yield identifier inside strict mode"));
            case "string":
            case "num":
            case "big_int":
            case "keyword":
            case "atom": return s(), e.value;
            default: f(e);
        } } function ue(e) { var t = i.token.value; return new ("this" == t ? sn : "super" == t ? un : e)({ name: String(t), start: i.token, end: i.token }); } function ce(e) { var t = e.name; h() && "yield" == t && l(e.start, "Yield cannot be used as identifier inside generators"), i.input.has_directive("use strict") && ("yield" == t && l(e.start, "Unexpected yield identifier inside strict mode"), e instanceof Kt && ("arguments" == t || "eval" == t) && l(e.start, "Unexpected " + t + " in strict mode")); } function le(e, t) { if (!o("name"))
            return t || c("Name expected"), null; var n = ue(e); return ce(n), s(), n; } function pe(e) { var t = e.start, i = t.comments_before; const o = n.get(t); for (var r = null != o ? o : i.length; --r >= 0;) {
            var a = i[r];
            if (/[@#]__/.test(a.value)) {
                if (/[@#]__PURE__/.test(a.value)) {
                    y(e, Fn);
                    break;
                }
                if (/[@#]__INLINE__/.test(a.value)) {
                    y(e, Mn);
                    break;
                }
                if (/[@#]__NOINLINE__/.test(a.value)) {
                    y(e, Rn);
                    break;
                }
            }
        } } var he = function (e, t) { var n, r = e.start; if (o("punc", "."))
            return s(), he(new gt({ start: r, expression: e, property: (n = i.token, "name" != n.type && f(), s(), n.value), end: u() }), t); if (o("punc", "[")) {
            s();
            var a = et(!0);
            return _("]"), he(new Dt({ start: r, expression: e, property: a, end: u() }), t);
        } if (t && o("punc", "(")) {
            s();
            var c = new dt({ start: r, expression: e, args: Ee(), end: u() });
            return pe(c), he(c, !0);
        } return o("template_head") ? he(new Ke({ start: r, prefix: e, template_string: H(), end: u() }), t) : e; }; function Ee() { for (var e = []; !o("punc", ")");)
            o("expand", "...") ? (s(), e.push(new Ne({ start: u(), expression: et(!1), end: u() }))) : e.push(et(!1)), o("punc", ")") || (_(","), o("punc", ")") && t.ecma < 2017 && f()); return s(), e; } var ge = function (e, t) { var n = i.token; if ("name" == n.type && "await" == n.value) {
            if (E())
                return s(), E() || c("Unexpected await expression outside async function", i.prev.line, i.prev.col, i.prev.pos), new je({ start: u(), end: i.token, expression: ge(!0) });
            i.input.has_directive("use strict") && l(i.token, "Unexpected await identifier inside strict mode");
        } if (o("operator") && ie.has(n.value)) {
            s(), v();
            var r = ve(vt, n, ge(e));
            return r.start = n, r.end = u(), r;
        } for (var a = G(e, t); o("operator") && oe.has(i.token.value) && !d(i.token);)
            a instanceof Ve && f(), (a = ve(At, i.token, a)).start = n, a.end = i.token, s(); return a; }; function ve(e, t, n) { var o = t.value; switch (o) {
            case "++":
            case "--":
                He(n) || c("Invalid use of " + o + " operator", t.line, t.col, t.pos);
                break;
            case "delete": n instanceof nn && i.input.has_directive("use strict") && c("Calling delete on expression not allowed in strict mode", n.start.line, n.start.col, n.start.pos);
        } return new e({ operator: o, expression: n }); } var ye = function (e, t, n) { var r = o("operator") ? i.token.value : null; "in" == r && n && (r = null), "**" == r && e instanceof vt && !ee(e.start, "punc", "(") && "--" !== e.operator && "++" !== e.operator && f(e.start); var a = null != r ? ae[r] : null; if (null != a && (a > t || "**" === r && t === a)) {
            s();
            var u = ye(ge(!0), a, n);
            return ye(new Tt({ start: e.start, left: e, operator: r, right: u, end: u.end }), t, n);
        } return e; }; var we = function (e) { var t = i.token, n = function (e) { return ye(ge(!0, !0), 0, e); }(e); if (o("operator", "?")) {
            s();
            var r = et(!1);
            return _(":"), new yt({ start: t, condition: n, consequent: r, alternative: et(!1, e), end: u() });
        } return n; }; function He(e) { return e instanceof Et || e instanceof nn; } function Xe(e) { if (e instanceof Ft)
            e = new Be({ start: e.start, names: e.properties.map(Xe), is_array: !1, end: e.end });
        else if (e instanceof Ot) {
            for (var t = [], n = 0; n < e.elements.length; n++)
                e.elements[n] instanceof Ne && (n + 1 !== e.elements.length && l(e.elements[n].start, "Spread must the be last element in destructuring array"), e.elements[n].expression = Xe(e.elements[n].expression)), t.push(Xe(e.elements[n]));
            e = new Be({ start: e.start, names: t, is_array: !0, end: e.end });
        }
        else
            e instanceof Mt ? e.value = Xe(e.value) : e instanceof bt && (e = new Ct({ start: e.start, left: e.left, operator: "=", right: e.right, end: e.end })); return e; } var Ye = function (e) { v(); var t = i.token; if ("name" == t.type && "yield" == t.value) {
            if (h())
                return s(), function () { h() || c("Unexpected yield expression outside generator function", i.prev.line, i.prev.col, i.prev.pos); var e = i.token, t = !1, n = !0; return m() || o("punc") && K.has(i.token.value) ? n = !1 : o("operator", "*") && (t = !0, s()), new Ze({ start: e, is_star: t, expression: n ? et() : null, end: u() }); }();
            i.input.has_directive("use strict") && l(i.token, "Unexpected yield identifier inside strict mode");
        } var n = we(e), r = i.token.value; if (o("operator") && re.has(r)) {
            if (He(n) || (n = Xe(n)) instanceof Be)
                return s(), new bt({ start: t, left: n, operator: r, right: Ye(e), end: u() });
            c("Invalid assignment");
        } return n; }, et = function (e, t) { for (var n = i.token, r = []; r.push(Ye(t)), e && o("punc", ",");)
            s(), e = !0; return 1 == r.length ? r[0] : new ht({ start: n, expressions: r, end: a() }); }; function St(e) { ++i.in_loop; var t = e(); return --i.in_loop, t; } return t.expression ? et(!0) : function () { var e = i.token, n = []; for (i.input.push_directives_stack(), t.module && i.input.add_directive("use strict"); !o("eof");)
            n.push(A()); i.input.pop_directives_stack(); var r = u(), a = t.toplevel; return a ? (a.body = a.body.concat(n), a.end = r) : a = new xe({ start: e, body: n, end: r }), a; }(); }
        function ce(e, t, n, i = fe) { var o = t = t ? t.split(/\s+/) : []; i && i.PROPS && (t = t.concat(i.PROPS)); for (var r = "return function AST_" + e + "(props){ if (props) { ", a = t.length; --a >= 0;)
            r += "this." + t[a] + " = props." + t[a] + ";"; const s = i && Object.create(i.prototype); (s && s.initialize || n && n.initialize) && (r += "this.initialize();"), r += "}", r += "this.flags = 0;", r += "}"; var u = new Function(r)(); if (s && (u.prototype = s, u.BASE = i), i && i.SUBCLASSES.push(u), u.prototype.CTOR = u, u.prototype.constructor = u, u.PROPS = t || null, u.SELF_PROPS = o, u.SUBCLASSES = [], e && (u.prototype.TYPE = u.TYPE = e), n)
            for (a in n)
                D(n, a) && ("$" === a[0] ? u[a.substr(1)] = n[a] : u.prototype[a] = n[a]); return u.DEFMETHOD = function (e, t) { this.prototype[e] = t; }, u; }
        var le = ce("Token", "type value line col pos endline endcol endpos nlb comments_before comments_after file raw quote end", {}, null), fe = ce("Node", "start end", { _clone: function (e) { if (e) {
                var t = this.clone();
                return t.transform(new On((function (e) { if (e !== t)
                    return e.clone(!0); })));
            } return new this.CTOR(this); }, clone: function (e) { return this._clone(e); }, $documentation: "Base class of all AST nodes", $propdoc: { start: "[AST_Token] The first token of this node", end: "[AST_Token] The last token of this node" }, _walk: function (e) { return e._visit(this); }, walk: function (e) { return this._walk(e); }, _children_backwards: () => { } }, null);
        fe.warn_function = null, fe.warn = function (e, t) { fe.warn_function && fe.warn_function(d(e, t)); };
        var pe = ce("Statement", null, { $documentation: "Base class of all statements" }), _e = ce("Debugger", null, { $documentation: "Represents a debugger statement" }, pe), de = ce("Directive", "value quote", { $documentation: 'Represents a directive, like "use strict";', $propdoc: { value: "[string] The value of this directive as a plain string (it's not an AST_String!)", quote: "[string] the original quote character" } }, pe), me = ce("SimpleStatement", "body", { $documentation: "A statement consisting of an expression, i.e. a = 1 + 2", $propdoc: { body: "[AST_Node] an expression node (should not be instanceof AST_Statement)" }, _walk: function (e) { return e._visit(this, (function () { this.body._walk(e); })); }, _children_backwards(e) { e(this.body); } }, pe);
        function he(e, t) { const n = e.body; for (var i = 0, o = n.length; i < o; i++)
            n[i]._walk(t); }
        function Ee(e) { var t = this._clone(e); return this.block_scope && (t.block_scope = this.block_scope.clone()), t; }
        var ge = ce("Block", "body block_scope", { $documentation: "A body of statements (usually braced)", $propdoc: { body: "[AST_Statement*] an array of statements", block_scope: "[AST_Scope] the block scope" }, _walk: function (e) { return e._visit(this, (function () { he(this, e); })); }, _children_backwards(e) { let t = this.body.length; for (; t--;)
                e(this.body[t]); }, clone: Ee }, pe), De = ce("BlockStatement", null, { $documentation: "A block statement" }, ge), Se = ce("EmptyStatement", null, { $documentation: "The empty statement (empty block or simply a semicolon)" }, pe), ve = ce("StatementWithBody", "body", { $documentation: "Base class for all statements that contain one nested body: `For`, `ForIn`, `Do`, `While`, `With`", $propdoc: { body: "[AST_Statement] the body; this should always be present, even if it's an AST_EmptyStatement" } }, pe), Ae = ce("LabeledStatement", "label", { $documentation: "Statement with a label", $propdoc: { label: "[AST_Label] a label definition" }, _walk: function (e) { return e._visit(this, (function () { this.label._walk(e), this.body._walk(e); })); }, _children_backwards(e) { e(this.body), e(this.label); }, clone: function (e) { var t = this._clone(e); if (e) {
                var n = t.label, i = this.label;
                t.walk(new Cn((function (e) { e instanceof Ye && e.label && e.label.thedef === i && (e.label.thedef = n, n.references.push(e)); })));
            } return t; } }, ve), Te = ce("IterationStatement", "block_scope", { $documentation: "Internal class.  All loops inherit from it.", $propdoc: { block_scope: "[AST_Scope] the block scope for this iteration statement." }, clone: Ee }, ve), ye = ce("DWLoop", "condition", { $documentation: "Base class for do/while statements", $propdoc: { condition: "[AST_Node] the loop condition.  Should not be instanceof AST_Statement" } }, Te), be = ce("Do", null, { $documentation: "A `do` statement", _walk: function (e) { return e._visit(this, (function () { this.body._walk(e), this.condition._walk(e); })); }, _children_backwards(e) { e(this.condition), e(this.body); } }, ye), Ce = ce("While", null, { $documentation: "A `while` statement", _walk: function (e) { return e._visit(this, (function () { this.condition._walk(e), this.body._walk(e); })); }, _children_backwards(e) { e(this.body), e(this.condition); } }, ye), Oe = ce("For", "init condition step", { $documentation: "A `for` statement", $propdoc: { init: "[AST_Node?] the `for` initialization code, or null if empty", condition: "[AST_Node?] the `for` termination clause, or null if empty", step: "[AST_Node?] the `for` update clause, or null if empty" }, _walk: function (e) { return e._visit(this, (function () { this.init && this.init._walk(e), this.condition && this.condition._walk(e), this.step && this.step._walk(e), this.body._walk(e); })); }, _children_backwards(e) { e(this.body), this.step && e(this.step), this.condition && e(this.condition), this.init && e(this.init); } }, Te), Fe = ce("ForIn", "init object", { $documentation: "A `for ... in` statement", $propdoc: { init: "[AST_Node] the `for/in` initialization code", object: "[AST_Node] the object that we're looping through" }, _walk: function (e) { return e._visit(this, (function () { this.init._walk(e), this.object._walk(e), this.body._walk(e); })); }, _children_backwards(e) { e(this.body), this.object && e(this.object), this.init && e(this.init); } }, Te), Me = ce("ForOf", "await", { $documentation: "A `for ... of` statement" }, Fe), Re = ce("With", "expression", { $documentation: "A `with` statement", $propdoc: { expression: "[AST_Node] the `with` expression" }, _walk: function (e) { return e._visit(this, (function () { this.expression._walk(e), this.body._walk(e); })); }, _children_backwards(e) { e(this.body), e(this.expression); } }, ve), we = ce("Scope", "variables functions uses_with uses_eval parent_scope enclosed cname _var_name_cache", { $documentation: "Base class for all statements introducing a lexical scope", $propdoc: { variables: "[Map/S] a map of name -> SymbolDef for all variables/functions defined in this scope", functions: "[Map/S] like `variables`, but only lists function declarations", uses_with: "[boolean/S] tells whether this scope uses the `with` statement", uses_eval: "[boolean/S] tells whether this scope contains a direct call to the global `eval`", parent_scope: "[AST_Scope?/S] link to the parent scope", enclosed: "[SymbolDef*/S] a list of all symbol definitions that are accessed from this scope or any subscopes", cname: "[integer/S] current index for mangling variables (used internally by the mangler)" }, get_defun_scope: function () { for (var e = this; e.is_block_scope();)
                e = e.parent_scope; return e; }, clone: function (e) { var t = this._clone(e); return this.variables && (t.variables = new Map(this.variables)), this.functions && (t.functions = new Map(this.functions)), this.enclosed && (t.enclosed = this.enclosed.slice()), this._block_scope && (t._block_scope = this._block_scope), t; }, pinned: function () { return this.uses_eval || this.uses_with; } }, ge), xe = ce("Toplevel", "globals", { $documentation: "The toplevel scope", $propdoc: { globals: "[Map/S] a map of name -> SymbolDef for all undeclared names" }, wrap_commonjs: function (e) { var t = this.body, n = "(function(exports){'$ORIG';})(typeof " + e + "=='undefined'?(" + e + "={}):" + e + ");"; return n = (n = ue(n)).transform(new On((function (e) { if (e instanceof de && "$ORIG" == e.value)
                return f.splice(t); }))); }, wrap_enclose: function (e) { "string" != typeof e && (e = ""); var t = e.indexOf(":"); t < 0 && (t = e.length); var n = this.body; return ue(["(function(", e.slice(0, t), '){"$ORIG"})(', e.slice(t + 1), ")"].join("")).transform(new On((function (e) { if (e instanceof de && "$ORIG" == e.value)
                return f.splice(n); }))); } }, we), Ne = ce("Expansion", "expression", { $documentation: "An expandible argument, such as ...rest, a splat, such as [1,2,...all], or an expansion in a variable declaration, such as var [first, ...rest] = list", $propdoc: { expression: "[AST_Node] the thing to be expanded" }, _walk: function (e) { return e._visit(this, (function () { this.expression.walk(e); })); }, _children_backwards(e) { e(this.expression); } }), ke = ce("Lambda", "name argnames uses_arguments is_generator async", { $documentation: "Base class for functions", $propdoc: { name: "[AST_SymbolDeclaration?] the name of this function", argnames: "[AST_SymbolFunarg|AST_Destructuring|AST_Expansion|AST_DefaultAssign*] array of function arguments, destructurings, or expanding arguments", uses_arguments: "[boolean/S] tells whether this function accesses the arguments array", is_generator: "[boolean] is this a generator method", async: "[boolean] is this method async" }, args_as_names: function () { for (var e = [], t = 0; t < this.argnames.length; t++)
                this.argnames[t] instanceof Be ? e.push(...this.argnames[t].all_symbols()) : e.push(this.argnames[t]); return e; }, _walk: function (e) { return e._visit(this, (function () { this.name && this.name._walk(e); for (var t = this.argnames, n = 0, i = t.length; n < i; n++)
                t[n]._walk(e); he(this, e); })); }, _children_backwards(e) { let t = this.body.length; for (; t--;)
                e(this.body[t]); for (t = this.argnames.length; t--;)
                e(this.argnames[t]); this.name && e(this.name); } }, we), Ie = ce("Accessor", null, { $documentation: "A setter/getter function.  The `name` property is always null." }, ke), Le = ce("Function", null, { $documentation: "A function expression" }, ke), Ve = ce("Arrow", null, { $documentation: "An ES6 Arrow function ((a) => b)" }, ke), Pe = ce("Defun", null, { $documentation: "A function definition" }, ke), Be = ce("Destructuring", "names is_array", { $documentation: "A destructuring of several names. Used in destructuring assignment and with destructuring function argument names", $propdoc: { names: "[AST_Node*] Array of properties or elements", is_array: "[Boolean] Whether the destructuring represents an object or array" }, _walk: function (e) { return e._visit(this, (function () { this.names.forEach((function (t) { t._walk(e); })); })); }, _children_backwards(e) { let t = this.names.length; for (; t--;)
                e(this.names[t]); }, all_symbols: function () { var e = []; return this.walk(new Cn((function (t) { t instanceof Pt && e.push(t); }))), e; } }), Ke = ce("PrefixedTemplateString", "template_string prefix", { $documentation: "A templatestring with a prefix, such as String.raw`foobarbaz`", $propdoc: { template_string: "[AST_TemplateString] The template string", prefix: "[AST_SymbolRef|AST_PropAccess] The prefix, which can be a symbol such as `foo` or a dotted expression such as `String.raw`." }, _walk: function (e) { return e._visit(this, (function () { this.prefix._walk(e), this.template_string._walk(e); })); }, _children_backwards(e) { e(this.template_string), e(this.prefix); } }), Ue = ce("TemplateString", "segments", { $documentation: "A template string literal", $propdoc: { segments: "[AST_Node*] One or more segments, starting with AST_TemplateSegment. AST_Node may follow AST_TemplateSegment, but each AST_Node must be followed by AST_TemplateSegment." }, _walk: function (e) { return e._visit(this, (function () { this.segments.forEach((function (t) { t._walk(e); })); })); }, _children_backwards(e) { let t = this.segments.length; for (; t--;)
                e(this.segments[t]); } }), Ge = ce("TemplateSegment", "value raw", { $documentation: "A segment of a template string literal", $propdoc: { value: "Content of the segment", raw: "Raw content of the segment" } }), He = ce("Jump", null, { $documentation: "Base class for “jumps” (for now that's `return`, `throw`, `break` and `continue`)" }, pe), Xe = ce("Exit", "value", { $documentation: "Base class for “exits” (`return` and `throw`)", $propdoc: { value: "[AST_Node?] the value returned or thrown by this statement; could be null for AST_Return" }, _walk: function (e) { return e._visit(this, this.value && function () { this.value._walk(e); }); }, _children_backwards(e) { this.value && e(this.value); } }, He), ze = ce("Return", null, { $documentation: "A `return` statement" }, Xe), We = ce("Throw", null, { $documentation: "A `throw` statement" }, Xe), Ye = ce("LoopControl", "label", { $documentation: "Base class for loop control statements (`break` and `continue`)", $propdoc: { label: "[AST_LabelRef?] the label, or null if none" }, _walk: function (e) { return e._visit(this, this.label && function () { this.label._walk(e); }); }, _children_backwards(e) { this.label && e(this.label); } }, He), qe = ce("Break", null, { $documentation: "A `break` statement" }, Ye), $e = ce("Continue", null, { $documentation: "A `continue` statement" }, Ye), je = ce("Await", "expression", { $documentation: "An `await` statement", $propdoc: { expression: "[AST_Node] the mandatory expression being awaited" }, _walk: function (e) { return e._visit(this, (function () { this.expression._walk(e); })); }, _children_backwards(e) { e(this.expression); } }), Ze = ce("Yield", "expression is_star", { $documentation: "A `yield` statement", $propdoc: { expression: "[AST_Node?] the value returned or thrown by this statement; could be null (representing undefined) but only when is_star is set to false", is_star: "[Boolean] Whether this is a yield or yield* statement" }, _walk: function (e) { return e._visit(this, this.expression && function () { this.expression._walk(e); }); }, _children_backwards(e) { this.expression && e(this.expression); } }), Je = ce("If", "condition alternative", { $documentation: "A `if` statement", $propdoc: { condition: "[AST_Node] the `if` condition", alternative: "[AST_Statement?] the `else` part, or null if not present" }, _walk: function (e) { return e._visit(this, (function () { this.condition._walk(e), this.body._walk(e), this.alternative && this.alternative._walk(e); })); }, _children_backwards(e) { this.alternative && e(this.alternative), e(this.body), e(this.condition); } }, ve), Qe = ce("Switch", "expression", { $documentation: "A `switch` statement", $propdoc: { expression: "[AST_Node] the `switch` “discriminant”" }, _walk: function (e) { return e._visit(this, (function () { this.expression._walk(e), he(this, e); })); }, _children_backwards(e) { let t = this.body.length; for (; t--;)
                e(this.body[t]); e(this.expression); } }, ge), et = ce("SwitchBranch", null, { $documentation: "Base class for `switch` branches" }, ge), tt = ce("Default", null, { $documentation: "A `default` switch branch" }, et), nt = ce("Case", "expression", { $documentation: "A `case` switch branch", $propdoc: { expression: "[AST_Node] the `case` expression" }, _walk: function (e) { return e._visit(this, (function () { this.expression._walk(e), he(this, e); })); }, _children_backwards(e) { let t = this.body.length; for (; t--;)
                e(this.body[t]); e(this.expression); } }, et), it = ce("Try", "bcatch bfinally", { $documentation: "A `try` statement", $propdoc: { bcatch: "[AST_Catch?] the catch block, or null if not present", bfinally: "[AST_Finally?] the finally block, or null if not present" }, _walk: function (e) { return e._visit(this, (function () { he(this, e), this.bcatch && this.bcatch._walk(e), this.bfinally && this.bfinally._walk(e); })); }, _children_backwards(e) { this.bfinally && e(this.bfinally), this.bcatch && e(this.bcatch); let t = this.body.length; for (; t--;)
                e(this.body[t]); } }, ge), ot = ce("Catch", "argname", { $documentation: "A `catch` node; only makes sense as part of a `try` statement", $propdoc: { argname: "[AST_SymbolCatch|AST_Destructuring|AST_Expansion|AST_DefaultAssign] symbol for the exception" }, _walk: function (e) { return e._visit(this, (function () { this.argname && this.argname._walk(e), he(this, e); })); }, _children_backwards(e) { let t = this.body.length; for (; t--;)
                e(this.body[t]); this.argname && e(this.argname); } }, ge), rt = ce("Finally", null, { $documentation: "A `finally` node; only makes sense as part of a `try` statement" }, ge), at = ce("Definitions", "definitions", { $documentation: "Base class for `var` or `const` nodes (variable declarations/initializations)", $propdoc: { definitions: "[AST_VarDef*] array of variable definitions" }, _walk: function (e) { return e._visit(this, (function () { for (var t = this.definitions, n = 0, i = t.length; n < i; n++)
                t[n]._walk(e); })); }, _children_backwards(e) { let t = this.definitions.length; for (; t--;)
                e(this.definitions[t]); } }, pe), st = ce("Var", null, { $documentation: "A `var` statement" }, at), ut = ce("Let", null, { $documentation: "A `let` statement" }, at), ct = ce("Const", null, { $documentation: "A `const` statement" }, at), lt = ce("VarDef", "name value", { $documentation: "A variable declaration; only appears in a AST_Definitions node", $propdoc: { name: "[AST_Destructuring|AST_SymbolConst|AST_SymbolLet|AST_SymbolVar] name of the variable", value: "[AST_Node?] initializer, or null of there's no initializer" }, _walk: function (e) { return e._visit(this, (function () { this.name._walk(e), this.value && this.value._walk(e); })); }, _children_backwards(e) { this.value && e(this.value), e(this.name); } }), ft = ce("NameMapping", "foreign_name name", { $documentation: "The part of the export/import statement that declare names from a module.", $propdoc: { foreign_name: "[AST_SymbolExportForeign|AST_SymbolImportForeign] The name being exported/imported (as specified in the module)", name: "[AST_SymbolExport|AST_SymbolImport] The name as it is visible to this module." }, _walk: function (e) { return e._visit(this, (function () { this.foreign_name._walk(e), this.name._walk(e); })); }, _children_backwards(e) { e(this.name), e(this.foreign_name); } }), pt = ce("Import", "imported_name imported_names module_name", { $documentation: "An `import` statement", $propdoc: { imported_name: "[AST_SymbolImport] The name of the variable holding the module's default export.", imported_names: "[AST_NameMapping*] The names of non-default imported variables", module_name: "[AST_String] String literal describing where this module came from" }, _walk: function (e) { return e._visit(this, (function () { this.imported_name && this.imported_name._walk(e), this.imported_names && this.imported_names.forEach((function (t) { t._walk(e); })), this.module_name._walk(e); })); }, _children_backwards(e) { if (e(this.module_name), this.imported_names) {
                let t = this.imported_names.length;
                for (; t--;)
                    e(this.imported_names[t]);
            } this.imported_name && e(this.imported_name); } }), _t = ce("Export", "exported_definition exported_value is_default exported_names module_name", { $documentation: "An `export` statement", $propdoc: { exported_definition: "[AST_Defun|AST_Definitions|AST_DefClass?] An exported definition", exported_value: "[AST_Node?] An exported value", exported_names: "[AST_NameMapping*?] List of exported names", module_name: "[AST_String?] Name of the file to load exports from", is_default: "[Boolean] Whether this is the default exported value of this module" }, _walk: function (e) { return e._visit(this, (function () { this.exported_definition && this.exported_definition._walk(e), this.exported_value && this.exported_value._walk(e), this.exported_names && this.exported_names.forEach((function (t) { t._walk(e); })), this.module_name && this.module_name._walk(e); })); }, _children_backwards(e) { if (this.module_name && e(this.module_name), this.exported_names) {
                let t = this.exported_names.length;
                for (; t--;)
                    e(this.exported_names[t]);
            } this.exported_value && e(this.exported_value), this.exported_definition && e(this.exported_definition); } }, pe), dt = ce("Call", "expression args _annotations", { $documentation: "A function call expression", $propdoc: { expression: "[AST_Node] expression to invoke as function", args: "[AST_Node*] array of arguments", _annotations: "[number] bitfield containing information about the call" }, initialize() { null == this._annotations && (this._annotations = 0); }, _walk(e) { return e._visit(this, (function () { for (var t = this.args, n = 0, i = t.length; n < i; n++)
                t[n]._walk(e); this.expression._walk(e); })); }, _children_backwards(e) { let t = this.args.length; for (; t--;)
                e(this.args[t]); e(this.expression); } }), mt = ce("New", null, { $documentation: "An object instantiation.  Derives from a function call since it has exactly the same properties" }, dt), ht = ce("Sequence", "expressions", { $documentation: "A sequence expression (comma-separated expressions)", $propdoc: { expressions: "[AST_Node*] array of expressions (at least two)" }, _walk: function (e) { return e._visit(this, (function () { this.expressions.forEach((function (t) { t._walk(e); })); })); }, _children_backwards(e) { let t = this.expressions.length; for (; t--;)
                e(this.expressions[t]); } }), Et = ce("PropAccess", "expression property", { $documentation: 'Base class for property access expressions, i.e. `a.foo` or `a["foo"]`', $propdoc: { expression: "[AST_Node] the “container” expression", property: "[AST_Node|string] the property to access.  For AST_Dot this is always a plain string, while for AST_Sub it's an arbitrary AST_Node" } }), gt = ce("Dot", "quote", { $documentation: "A dotted property access expression", $propdoc: { quote: "[string] the original quote character when transformed from AST_Sub" }, _walk: function (e) { return e._visit(this, (function () { this.expression._walk(e); })); }, _children_backwards(e) { e(this.expression); } }, Et), Dt = ce("Sub", null, { $documentation: 'Index-style property access, i.e. `a["foo"]`', _walk: function (e) { return e._visit(this, (function () { this.expression._walk(e), this.property._walk(e); })); }, _children_backwards(e) { e(this.property), e(this.expression); } }, Et), St = ce("Unary", "operator expression", { $documentation: "Base class for unary expressions", $propdoc: { operator: "[string] the operator", expression: "[AST_Node] expression that this unary operator applies to" }, _walk: function (e) { return e._visit(this, (function () { this.expression._walk(e); })); }, _children_backwards(e) { e(this.expression); } }), vt = ce("UnaryPrefix", null, { $documentation: "Unary prefix expression, i.e. `typeof i` or `++i`" }, St), At = ce("UnaryPostfix", null, { $documentation: "Unary postfix expression, i.e. `i++`" }, St), Tt = ce("Binary", "operator left right", { $documentation: "Binary expression, i.e. `a + b`", $propdoc: { left: "[AST_Node] left-hand side expression", operator: "[string] the operator", right: "[AST_Node] right-hand side expression" }, _walk: function (e) { return e._visit(this, (function () { this.left._walk(e), this.right._walk(e); })); }, _children_backwards(e) { e(this.right), e(this.left); } }), yt = ce("Conditional", "condition consequent alternative", { $documentation: "Conditional expression using the ternary operator, i.e. `a ? b : c`", $propdoc: { condition: "[AST_Node]", consequent: "[AST_Node]", alternative: "[AST_Node]" }, _walk: function (e) { return e._visit(this, (function () { this.condition._walk(e), this.consequent._walk(e), this.alternative._walk(e); })); }, _children_backwards(e) { e(this.alternative), e(this.consequent), e(this.condition); } }), bt = ce("Assign", null, { $documentation: "An assignment expression — `a = b + 5`" }, Tt), Ct = ce("DefaultAssign", null, { $documentation: "A default assignment expression like in `(a = 3) => a`" }, Tt), Ot = ce("Array", "elements", { $documentation: "An array literal", $propdoc: { elements: "[AST_Node*] array of elements" }, _walk: function (e) { return e._visit(this, (function () { for (var t = this.elements, n = 0, i = t.length; n < i; n++)
                t[n]._walk(e); })); }, _children_backwards(e) { let t = this.elements.length; for (; t--;)
                e(this.elements[t]); } }), Ft = ce("Object", "properties", { $documentation: "An object literal", $propdoc: { properties: "[AST_ObjectProperty*] array of properties" }, _walk: function (e) { return e._visit(this, (function () { for (var t = this.properties, n = 0, i = t.length; n < i; n++)
                t[n]._walk(e); })); }, _children_backwards(e) { let t = this.properties.length; for (; t--;)
                e(this.properties[t]); } }), Mt = ce("ObjectProperty", "key value", { $documentation: "Base class for literal object properties", $propdoc: { key: "[string|AST_Node] property name. For ObjectKeyVal this is a string. For getters, setters and computed property this is an AST_Node.", value: "[AST_Node] property value.  For getters and setters this is an AST_Accessor." }, _walk: function (e) { return e._visit(this, (function () { this.key instanceof fe && this.key._walk(e), this.value._walk(e); })); }, _children_backwards(e) { e(this.value), this.key instanceof fe && e(this.key); } }), Rt = ce("ObjectKeyVal", "quote", { $documentation: "A key: value object property", $propdoc: { quote: "[string] the original quote character" }, computed_key() { return this.key instanceof fe; } }, Mt), wt = ce("ObjectSetter", "quote static", { $propdoc: { quote: "[string|undefined] the original quote character, if any", static: "[boolean] whether this is a static setter (classes only)" }, $documentation: "An object setter property", computed_key() { return !(this.key instanceof Yt); } }, Mt), xt = ce("ObjectGetter", "quote static", { $propdoc: { quote: "[string|undefined] the original quote character, if any", static: "[boolean] whether this is a static getter (classes only)" }, $documentation: "An object getter property", computed_key() { return !(this.key instanceof Yt); } }, Mt), Nt = ce("ConciseMethod", "quote static is_generator async", { $propdoc: { quote: "[string|undefined] the original quote character, if any", static: "[boolean] is this method static (classes only)", is_generator: "[boolean] is this a generator method", async: "[boolean] is this method async" }, $documentation: "An ES6 concise method inside an object or class", computed_key() { return !(this.key instanceof Yt); } }, Mt), kt = ce("Class", "name extends properties", { $propdoc: { name: "[AST_SymbolClass|AST_SymbolDefClass?] optional class name.", extends: "[AST_Node]? optional parent class", properties: "[AST_ObjectProperty*] array of properties" }, $documentation: "An ES6 class", _walk: function (e) { return e._visit(this, (function () { this.name && this.name._walk(e), this.extends && this.extends._walk(e), this.properties.forEach(t => t._walk(e)); })); }, _children_backwards(e) { let t = this.properties.length; for (; t--;)
                e(this.properties[t]); this.extends && e(this.extends), this.name && e(this.name); } }, we), It = ce("ClassProperty", "static quote", { $documentation: "A class property", $propdoc: { static: "[boolean] whether this is a static key", quote: "[string] which quote is being used" }, _walk: function (e) { return e._visit(this, (function () { this.key instanceof fe && this.key._walk(e), this.value instanceof fe && this.value._walk(e); })); }, _children_backwards(e) { this.value instanceof fe && e(this.value), this.key instanceof fe && e(this.key); }, computed_key() { return !(this.key instanceof qt); } }, Mt), Lt = ce("DefClass", null, { $documentation: "A class definition" }, kt), Vt = ce("ClassExpression", null, { $documentation: "A class expression." }, kt), Pt = ce("Symbol", "scope name thedef", { $propdoc: { name: "[string] name of this symbol", scope: "[AST_Scope/S] the current scope (not necessarily the definition scope)", thedef: "[SymbolDef/S] the definition of this symbol" }, $documentation: "Base class for all symbols" }), Bt = ce("NewTarget", null, { $documentation: "A reference to new.target" }), Kt = ce("SymbolDeclaration", "init", { $documentation: "A declaration symbol (symbol in var/const, function name or argument, symbol in catch)" }, Pt), Ut = ce("SymbolVar", null, { $documentation: "Symbol defining a variable" }, Kt), Gt = ce("SymbolBlockDeclaration", null, { $documentation: "Base class for block-scoped declaration symbols" }, Kt), Ht = ce("SymbolConst", null, { $documentation: "A constant declaration" }, Gt), Xt = ce("SymbolLet", null, { $documentation: "A block-scoped `let` declaration" }, Gt), zt = ce("SymbolFunarg", null, { $documentation: "Symbol naming a function argument" }, Ut), Wt = ce("SymbolDefun", null, { $documentation: "Symbol defining a function" }, Kt), Yt = ce("SymbolMethod", null, { $documentation: "Symbol in an object defining a method" }, Pt), qt = ce("SymbolClassProperty", null, { $documentation: "Symbol for a class property" }, Pt), $t = ce("SymbolLambda", null, { $documentation: "Symbol naming a function expression" }, Kt), jt = ce("SymbolDefClass", null, { $documentation: "Symbol naming a class's name in a class declaration. Lexically scoped to its containing scope, and accessible within the class." }, Gt), Zt = ce("SymbolClass", null, { $documentation: "Symbol naming a class's name. Lexically scoped to the class." }, Kt), Jt = ce("SymbolCatch", null, { $documentation: "Symbol naming the exception in catch" }, Gt), Qt = ce("SymbolImport", null, { $documentation: "Symbol referring to an imported name" }, Gt), en = ce("SymbolImportForeign", null, { $documentation: "A symbol imported from a module, but it is defined in the other module, and its real name is irrelevant for this module's purposes" }, Pt), tn = ce("Label", "references", { $documentation: "Symbol naming a label (declaration)", $propdoc: { references: "[AST_LoopControl*] a list of nodes referring to this label" }, initialize: function () { this.references = [], this.thedef = this; } }, Pt), nn = ce("SymbolRef", null, { $documentation: "Reference to some symbol (not definition/declaration)" }, Pt), on = ce("SymbolExport", null, { $documentation: "Symbol referring to a name to export" }, nn), rn = ce("SymbolExportForeign", null, { $documentation: "A symbol exported from this module, but it is used in the other module, and its real name is irrelevant for this module's purposes" }, Pt), an = ce("LabelRef", null, { $documentation: "Reference to a label symbol" }, Pt), sn = ce("This", null, { $documentation: "The `this` symbol" }, Pt), un = ce("Super", null, { $documentation: "The `super` symbol" }, sn), cn = ce("Constant", null, { $documentation: "Base class for all constants", getValue: function () { return this.value; } }), ln = ce("String", "value quote", { $documentation: "A string literal", $propdoc: { value: "[string] the contents of this string", quote: "[string] the original quote character" } }, cn), fn = ce("Number", "value literal", { $documentation: "A number literal", $propdoc: { value: "[number] the numeric value", literal: "[string] numeric value as string (optional)" } }, cn), pn = ce("BigInt", "value", { $documentation: "A big int literal", $propdoc: { value: "[string] big int value" } }, cn), _n = ce("RegExp", "value", { $documentation: "A regexp literal", $propdoc: { value: "[RegExp] the actual regexp" } }, cn), dn = ce("Atom", null, { $documentation: "Base class for atoms" }, cn), mn = ce("Null", null, { $documentation: "The `null` atom", value: null }, dn), hn = ce("NaN", null, { $documentation: "The impossible value", value: NaN }, dn), En = ce("Undefined", null, { $documentation: "The `undefined` value", value: void 0 }, dn), gn = ce("Hole", null, { $documentation: "A hole in an array", value: void 0 }, dn), Dn = ce("Infinity", null, { $documentation: "The `Infinity` value", value: 1 / 0 }, dn), Sn = ce("Boolean", null, { $documentation: "Base class for booleans" }, dn), vn = ce("False", null, { $documentation: "The `false` atom", value: !1 }, Sn), An = ce("True", null, { $documentation: "The `true` atom", value: !0 }, Sn);
        function Tn(e, t, n = [e]) { const i = n.push.bind(n); for (; n.length;) {
            const e = n.pop(), o = t(e, n);
            if (o) {
                if (o === bn)
                    return !0;
            }
            else
                e._children_backwards(i);
        } return !1; }
        function yn(e, t, n) { const i = [e], o = i.push.bind(i), r = n ? n.slice() : [], a = []; let s; const u = { parent: (e = 0) => -1 === e ? s : n && e >= r.length ? (e -= r.length, n[n.length - (e + 1)]) : r[r.length - (1 + e)] }; for (; i.length;) {
            for (s = i.pop(); a.length && i.length == a[a.length - 1];)
                r.pop(), a.pop();
            const e = t(s, u);
            if (e) {
                if (e === bn)
                    return !0;
                continue;
            }
            const n = i.length;
            s._children_backwards(o), i.length > n && (r.push(s), a.push(n - 1));
        } return !1; }
        const bn = Symbol("abort walk");
        class Cn {
            constructor(e) { this.visit = e, this.stack = [], this.directives = Object.create(null); }
            _visit(e, t) { this.push(e); var n = this.visit(e, t ? function () { t.call(e); } : a); return !n && t && t.call(e), this.pop(), n; }
            parent(e) { return this.stack[this.stack.length - 2 - (e || 0)]; }
            push(e) { e instanceof ke ? this.directives = Object.create(this.directives) : e instanceof de && !this.directives[e.value] ? this.directives[e.value] = e : e instanceof kt && (this.directives = Object.create(this.directives), this.directives["use strict"] || (this.directives["use strict"] = e)), this.stack.push(e); }
            pop() { var e = this.stack.pop(); (e instanceof ke || e instanceof kt) && (this.directives = Object.getPrototypeOf(this.directives)); }
            self() { return this.stack[this.stack.length - 1]; }
            find_parent(e) { for (var t = this.stack, n = t.length; --n >= 0;) {
                var i = t[n];
                if (i instanceof e)
                    return i;
            } }
            has_directive(e) { var t = this.directives[e]; if (t)
                return t; var n = this.stack[this.stack.length - 1]; if (n instanceof we && n.body)
                for (var i = 0; i < n.body.length; ++i) {
                    var o = n.body[i];
                    if (!(o instanceof de))
                        break;
                    if (o.value == e)
                        return o;
                } }
            loopcontrol_target(e) { var t = this.stack; if (e.label)
                for (var n = t.length; --n >= 0;) {
                    if ((i = t[n]) instanceof Ae && i.label.name == e.label.name)
                        return i.body;
                }
            else
                for (n = t.length; --n >= 0;) {
                    var i;
                    if ((i = t[n]) instanceof Te || e instanceof qe && i instanceof Qe)
                        return i;
                } }
        }
        class On extends Cn {
            constructor(e, t) { super(), this.before = e, this.after = t; }
        }
        const Fn = 1, Mn = 2, Rn = 4;
        var wn = Object.freeze({ __proto__: null, AST_Accessor: Ie, AST_Array: Ot, AST_Arrow: Ve, AST_Assign: bt, AST_Atom: dn, AST_Await: je, AST_BigInt: pn, AST_Binary: Tt, AST_Block: ge, AST_BlockStatement: De, AST_Boolean: Sn, AST_Break: qe, AST_Call: dt, AST_Case: nt, AST_Catch: ot, AST_Class: kt, AST_ClassExpression: Vt, AST_ClassProperty: It, AST_ConciseMethod: Nt, AST_Conditional: yt, AST_Const: ct, AST_Constant: cn, AST_Continue: $e, AST_Debugger: _e, AST_Default: tt, AST_DefaultAssign: Ct, AST_DefClass: Lt, AST_Definitions: at, AST_Defun: Pe, AST_Destructuring: Be, AST_Directive: de, AST_Do: be, AST_Dot: gt, AST_DWLoop: ye, AST_EmptyStatement: Se, AST_Exit: Xe, AST_Expansion: Ne, AST_Export: _t, AST_False: vn, AST_Finally: rt, AST_For: Oe, AST_ForIn: Fe, AST_ForOf: Me, AST_Function: Le, AST_Hole: gn, AST_If: Je, AST_Import: pt, AST_Infinity: Dn, AST_IterationStatement: Te, AST_Jump: He, AST_Label: tn, AST_LabeledStatement: Ae, AST_LabelRef: an, AST_Lambda: ke, AST_Let: ut, AST_LoopControl: Ye, AST_NameMapping: ft, AST_NaN: hn, AST_New: mt, AST_NewTarget: Bt, AST_Node: fe, AST_Null: mn, AST_Number: fn, AST_Object: Ft, AST_ObjectGetter: xt, AST_ObjectKeyVal: Rt, AST_ObjectProperty: Mt, AST_ObjectSetter: wt, AST_PrefixedTemplateString: Ke, AST_PropAccess: Et, AST_RegExp: _n, AST_Return: ze, AST_Scope: we, AST_Sequence: ht, AST_SimpleStatement: me, AST_Statement: pe, AST_StatementWithBody: ve, AST_String: ln, AST_Sub: Dt, AST_Super: un, AST_Switch: Qe, AST_SwitchBranch: et, AST_Symbol: Pt, AST_SymbolBlockDeclaration: Gt, AST_SymbolCatch: Jt, AST_SymbolClass: Zt, AST_SymbolClassProperty: qt, AST_SymbolConst: Ht, AST_SymbolDeclaration: Kt, AST_SymbolDefClass: jt, AST_SymbolDefun: Wt, AST_SymbolExport: on, AST_SymbolExportForeign: rn, AST_SymbolFunarg: zt, AST_SymbolImport: Qt, AST_SymbolImportForeign: en, AST_SymbolLambda: $t, AST_SymbolLet: Xt, AST_SymbolMethod: Yt, AST_SymbolRef: nn, AST_SymbolVar: Ut, AST_TemplateSegment: Ge, AST_TemplateString: Ue, AST_This: sn, AST_Throw: We, AST_Token: le, AST_Toplevel: xe, AST_True: An, AST_Try: it, AST_Unary: St, AST_UnaryPostfix: At, AST_UnaryPrefix: vt, AST_Undefined: En, AST_Var: st, AST_VarDef: lt, AST_While: Ce, AST_With: Re, AST_Yield: Ze, TreeTransformer: On, TreeWalker: Cn, walk: Tn, walk_abort: bn, walk_body: he, walk_parent: yn, _INLINE: Mn, _NOINLINE: Rn, _PURE: Fn });
        function xn(e, t) { e.DEFMETHOD("transform", (function (e, n) { let i = void 0; if (e.push(this), e.before && (i = e.before(this, t, n)), void 0 === i && (i = this, t(i, e), e.after)) {
            const t = e.after(i, n);
            void 0 !== t && (i = t);
        } return e.pop(), i; })); }
        function Nn(e, t) { return f(e, (function (e) { return e.transform(t, !0); })); }
        function kn(e) { let t = e.parent(-1); for (let n, i = 0; n = e.parent(i); i++) {
            if (n instanceof pe && n.body === t)
                return !0;
            if (!(n instanceof ht && n.expressions[0] === t || "Call" === n.TYPE && n.expression === t || n instanceof Ke && n.prefix === t || n instanceof gt && n.expression === t || n instanceof Dt && n.expression === t || n instanceof yt && n.condition === t || n instanceof Tt && n.left === t || n instanceof At && n.expression === t))
                return !1;
            t = n;
        } }
        xn(fe, a), xn(Ae, (function (e, t) { e.label = e.label.transform(t), e.body = e.body.transform(t); })), xn(me, (function (e, t) { e.body = e.body.transform(t); })), xn(ge, (function (e, t) { e.body = Nn(e.body, t); })), xn(be, (function (e, t) { e.body = e.body.transform(t), e.condition = e.condition.transform(t); })), xn(Ce, (function (e, t) { e.condition = e.condition.transform(t), e.body = e.body.transform(t); })), xn(Oe, (function (e, t) { e.init && (e.init = e.init.transform(t)), e.condition && (e.condition = e.condition.transform(t)), e.step && (e.step = e.step.transform(t)), e.body = e.body.transform(t); })), xn(Fe, (function (e, t) { e.init = e.init.transform(t), e.object = e.object.transform(t), e.body = e.body.transform(t); })), xn(Re, (function (e, t) { e.expression = e.expression.transform(t), e.body = e.body.transform(t); })), xn(Xe, (function (e, t) { e.value && (e.value = e.value.transform(t)); })), xn(Ye, (function (e, t) { e.label && (e.label = e.label.transform(t)); })), xn(Je, (function (e, t) { e.condition = e.condition.transform(t), e.body = e.body.transform(t), e.alternative && (e.alternative = e.alternative.transform(t)); })), xn(Qe, (function (e, t) { e.expression = e.expression.transform(t), e.body = Nn(e.body, t); })), xn(nt, (function (e, t) { e.expression = e.expression.transform(t), e.body = Nn(e.body, t); })), xn(it, (function (e, t) { e.body = Nn(e.body, t), e.bcatch && (e.bcatch = e.bcatch.transform(t)), e.bfinally && (e.bfinally = e.bfinally.transform(t)); })), xn(ot, (function (e, t) { e.argname && (e.argname = e.argname.transform(t)), e.body = Nn(e.body, t); })), xn(at, (function (e, t) { e.definitions = Nn(e.definitions, t); })), xn(lt, (function (e, t) { e.name = e.name.transform(t), e.value && (e.value = e.value.transform(t)); })), xn(Be, (function (e, t) { e.names = Nn(e.names, t); })), xn(ke, (function (e, t) { e.name && (e.name = e.name.transform(t)), e.argnames = Nn(e.argnames, t), e.body instanceof fe ? e.body = e.body.transform(t) : e.body = Nn(e.body, t); })), xn(dt, (function (e, t) { e.expression = e.expression.transform(t), e.args = Nn(e.args, t); })), xn(ht, (function (e, t) { const n = Nn(e.expressions, t); e.expressions = n.length ? n : [new fn({ value: 0 })]; })), xn(gt, (function (e, t) { e.expression = e.expression.transform(t); })), xn(Dt, (function (e, t) { e.expression = e.expression.transform(t), e.property = e.property.transform(t); })), xn(Ze, (function (e, t) { e.expression && (e.expression = e.expression.transform(t)); })), xn(je, (function (e, t) { e.expression = e.expression.transform(t); })), xn(St, (function (e, t) { e.expression = e.expression.transform(t); })), xn(Tt, (function (e, t) { e.left = e.left.transform(t), e.right = e.right.transform(t); })), xn(yt, (function (e, t) { e.condition = e.condition.transform(t), e.consequent = e.consequent.transform(t), e.alternative = e.alternative.transform(t); })), xn(Ot, (function (e, t) { e.elements = Nn(e.elements, t); })), xn(Ft, (function (e, t) { e.properties = Nn(e.properties, t); })), xn(Mt, (function (e, t) { e.key instanceof fe && (e.key = e.key.transform(t)), e.value && (e.value = e.value.transform(t)); })), xn(kt, (function (e, t) { e.name && (e.name = e.name.transform(t)), e.extends && (e.extends = e.extends.transform(t)), e.properties = Nn(e.properties, t); })), xn(Ne, (function (e, t) { e.expression = e.expression.transform(t); })), xn(ft, (function (e, t) { e.foreign_name = e.foreign_name.transform(t), e.name = e.name.transform(t); })), xn(pt, (function (e, t) { e.imported_name && (e.imported_name = e.imported_name.transform(t)), e.imported_names && Nn(e.imported_names, t), e.module_name = e.module_name.transform(t); })), xn(_t, (function (e, t) { e.exported_definition && (e.exported_definition = e.exported_definition.transform(t)), e.exported_value && (e.exported_value = e.exported_value.transform(t)), e.exported_names && Nn(e.exported_names, t), e.module_name && (e.module_name = e.module_name.transform(t)); })), xn(Ue, (function (e, t) { e.segments = Nn(e.segments, t); })), xn(Ke, (function (e, t) { e.prefix = e.prefix.transform(t), e.template_string = e.template_string.transform(t); }));
        const In = /^$|[;{][\s\n]*$/, Ln = /[@#]__(PURE|INLINE|NOINLINE)__/g;
        function Vn(e) { return ("comment2" === e.type || "comment1" === e.type) && /@preserve|@lic|@cc_on|^\**!/i.test(e.value); }
        function Pn(e) { var t = !e; void 0 === (e = r(e, { ascii_only: !1, beautify: !1, braces: !1, comments: "some", ecma: 5, ie8: !1, indent_level: 4, indent_start: 0, inline_script: !0, keep_numbers: !1, keep_quoted_props: !1, max_line_len: !1, preamble: null, preserve_annotations: !1, quote_keys: !1, quote_style: 0, safari10: !1, semicolons: !0, shebang: !0, shorthand: void 0, source_map: null, webkit: !1, width: 80, wrap_iife: !1, wrap_func_args: !0 }, !0)).shorthand && (e.shorthand = e.ecma > 5); var n = s; if (e.comments) {
            let t = e.comments;
            if ("string" == typeof e.comments && /^\/.*\/[a-zA-Z]*$/.test(e.comments)) {
                var i = e.comments.lastIndexOf("/");
                t = new RegExp(e.comments.substr(1, i - 1), e.comments.substr(i + 1));
            }
            n = t instanceof RegExp ? function (e) { return "comment5" != e.type && t.test(e.value); } : "function" == typeof t ? function (e) { return "comment5" != e.type && t(this, e); } : "some" === t ? Vn : u;
        } var o = 0, c = 0, l = 1, f = 0, p = ""; let _ = new Set; var d = e.ascii_only ? function (t, n) { return e.ecma >= 2015 && (t = t.replace(/[\ud800-\udbff][\udc00-\udfff]/g, (function (e) { return "\\u{" + function (e, t) { return z(e.charCodeAt(t)) ? 65536 + (e.charCodeAt(t) - 55296 << 10) + e.charCodeAt(t + 1) - 56320 : e.charCodeAt(t); }(e, 0).toString(16) + "}"; }))), t.replace(/[\u0000-\u001f\u007f-\uffff]/g, (function (e) { var t = e.charCodeAt(0).toString(16); if (t.length <= 2 && !n) {
            for (; t.length < 2;)
                t = "0" + t;
            return "\\x" + t;
        } for (; t.length < 4;)
            t = "0" + t; return "\\u" + t; })); } : function (e) { for (var t = "", n = 0, i = e.length; n < i; n++)
            z(e.charCodeAt(n)) && !W(e.charCodeAt(n + 1)) || W(e.charCodeAt(n)) && !z(e.charCodeAt(n - 1)) ? t += "\\u" + e.charCodeAt(n).toString(16) : t += e[n]; return t; }; function m(t, n) { var i = function (t, n) { var i = 0, o = 0; function r() { return "'" + t.replace(/\x27/g, "\\'") + "'"; } function a() { return '"' + t.replace(/\x22/g, '\\"') + '"'; } if (t = t.replace(/[\\\b\f\n\r\v\t\x22\x27\u2028\u2029\0\ufeff]/g, (function (n, r) { switch (n) {
            case '"': return ++i, '"';
            case "'": return ++o, "'";
            case "\\": return "\\\\";
            case "\n": return "\\n";
            case "\r": return "\\r";
            case "\t": return "\\t";
            case "\b": return "\\b";
            case "\f": return "\\f";
            case "\v": return e.ie8 ? "\\x0B" : "\\v";
            case "\u2028": return "\\u2028";
            case "\u2029": return "\\u2029";
            case "\ufeff": return "\\ufeff";
            case "\0": return /[0-9]/.test(X(t, r + 1)) ? "\\x00" : "\\0";
        } return n; })), t = d(t), "`" === n)
            return "`" + t.replace(/`/g, "\\`") + "`"; switch (e.quote_style) {
            case 1: return r();
            case 2: return a();
            case 3: return "'" == n ? r() : a();
            default: return i > o ? r() : a();
        } }(t, n); return e.inline_script && (i = (i = (i = i.replace(/<\x2f(script)([>\/\t\n\f\r ])/gi, "<\\/$1$2")).replace(/\x3c!--/g, "\\x3c!--")).replace(/--\x3e/g, "--\\x3e")), i; } var h, g, D = !1, S = !1, v = !1, A = 0, T = !1, y = !1, b = -1, C = "", O = e.source_map && [], F = O ? function () { O.forEach((function (t) { try {
            e.source_map.add(t.token.file, t.line, t.col, t.token.line, t.token.col, t.name || "name" != t.token.type ? t.name : t.token.value);
        }
        catch (e) {
            null != t.token.file && fe.warn("Couldn't figure out mapping for {file}:{line},{col} → {cline},{ccol} [{name}]", { file: t.token.file, line: t.token.line, col: t.token.col, cline: t.line, ccol: t.col, name: t.name || "" });
        } })), O = []; } : a, M = e.max_line_len ? function () { if (c > e.max_line_len) {
            if (A) {
                var t = p.slice(0, A), n = p.slice(A);
                if (O) {
                    var i = n.length - c;
                    O.forEach((function (e) { e.line++, e.col += i; }));
                }
                p = t + "\n" + n, l++, f++, c = n.length;
            }
            c > e.max_line_len && fe.warn("Output exceeds {max_line_len} characters", e);
        } A && (A = 0, F()); } : a, R = E("( [ + * / - , . `"); function w(t) { var n = X(t = String(t), 0); T && n && (T = !1, "\n" !== n && (w("\n"), N())), y && n && (y = !1, /[\s;})]/.test(n) || x()), b = -1; var i = C.charAt(C.length - 1); v && (v = !1, (":" !== i || "}" !== n) && (n && ";}".includes(n) || ";" === i) || (e.semicolons || R.has(n) ? (p += ";", c++, f++) : (M(), c > 0 && (p += "\n", f++, l++, c = 0), /^\s+$/.test(t) && (v = !0)), e.beautify || (S = !1))), S && (($(i) && ($(n) || "\\" == n) || "/" == n && n == i || ("+" == n || "-" == n) && n == C) && (p += " ", c++, f++), S = !1), h && (O.push({ token: h, name: g, line: l, col: c }), h = !1, A || F()), p += t, D = "(" == t[t.length - 1], f += t.length; var o = t.split(/\r?\n/), r = o.length - 1; l += r, c += o[0].length, r > 0 && (M(), c = o[r].length), C = t; } var x = e.beautify ? function () { w(" "); } : function () { S = !0; }, N = e.beautify ? function (t) { var n; e.beautify && w((n = t ? .5 : 0, " ".repeat(e.indent_start + o - n * e.indent_level))); } : a, k = e.beautify ? function (e, t) { !0 === e && (e = P()); var n = o; o = e; var i = t(); return o = n, i; } : function (e, t) { return t(); }, I = e.beautify ? function () { if (b < 0)
            return w("\n"); "\n" != p[b] && (p = p.slice(0, b) + "\n" + p.slice(b), f++, l++), b++; } : e.max_line_len ? function () { M(), A = p.length; } : a, L = e.beautify ? function () { w(";"); } : function () { v = !0; }; function V() { v = !1, w(";"); } function P() { return o + e.indent_level; } function B() { return A && M(), p; } function K() { let e = p.length - 1; for (; e >= 0;) {
            const t = p.charCodeAt(e);
            if (10 === t)
                return !0;
            if (32 !== t)
                return !1;
            e--;
        } return !0; } function U(t) { return e.preserve_annotations || (t = t.replace(Ln, " ")), /^\s*$/.test(t) ? "" : t.replace(/(<\s*\/\s*)(script)/i, "<\\/$2"); } var G = []; return { get: B, toString: B, indent: N, in_directive: !1, use_asm: null, active_scope: null, indentation: function () { return o; }, current_width: function () { return c - o; }, should_break: function () { return e.width && this.current_width() >= e.width; }, has_parens: function () { return D; }, newline: I, print: w, star: function () { w("*"); }, space: x, comma: function () { w(","), x(); }, colon: function () { w(":"), x(); }, last: function () { return C; }, semicolon: L, force_semicolon: V, to_utf8: d, print_name: function (e) { w(function (e) { return e = e.toString(), e = d(e, !0); }(e)); }, print_string: function (e, t, n) { var i = m(e, t); !0 !== n || i.includes("\\") || (In.test(p) || V(), V()), w(i); }, print_template_string_chars: function (e) { var t = m(e, "`").replace(/\${/g, "\\${"); return w(t.substr(1, t.length - 2)); }, encode_string: m, next_indent: P, with_indent: k, with_block: function (e) { var t; return w("{"), I(), k(P(), (function () { t = e(); })), N(), w("}"), t; }, with_parens: function (e) { w("("); var t = e(); return w(")"), t; }, with_square: function (e) { w("["); var t = e(); return w("]"), t; }, add_mapping: O ? function (e, t) { h = e, g = t; } : a, option: function (t) { return e[t]; }, printed_comments: _, prepend_comments: t ? a : function (t) { var i = t.start; if (!i)
                return; var o = this.printed_comments; const r = t instanceof Xe && t.value; if (i.comments_before && o.has(i.comments_before)) {
                if (!r)
                    return;
                i.comments_before = [];
            } var a = i.comments_before; if (a || (a = i.comments_before = []), o.add(a), r) {
                var s = new Cn((function (e) { var t = s.parent(); if (!(t instanceof Xe || t instanceof Tt && t.left === e || "Call" == t.TYPE && t.expression === e || t instanceof yt && t.condition === e || t instanceof gt && t.expression === e || t instanceof ht && t.expressions[0] === e || t instanceof Dt && t.expression === e || t instanceof At))
                    return !0; if (e.start) {
                    var n = e.start.comments_before;
                    n && !o.has(n) && (o.add(n), a = a.concat(n));
                } }));
                s.push(t), t.value.walk(s);
            } if (0 == f) {
                a.length > 0 && e.shebang && "comment5" === a[0].type && !o.has(a[0]) && (w("#!" + a.shift().value + "\n"), N());
                var u = e.preamble;
                u && w(u.replace(/\r\n?|[\n\u2028\u2029]|\s*$/g, "\n"));
            } if (0 != (a = a.filter(n, t).filter(e => !o.has(e))).length) {
                var c = K();
                a.forEach((function (e, t) { if (o.add(e), c || (e.nlb ? (w("\n"), N(), c = !0) : t > 0 && x()), /comment[134]/.test(e.type))
                    (n = U(e.value)) && (w("//" + n + "\n"), N()), c = !0;
                else if ("comment2" == e.type) {
                    var n;
                    (n = U(e.value)) && w("/*" + n + "*/"), c = !1;
                } })), c || (i.nlb ? (w("\n"), N()) : x());
            } }, append_comments: t || n === s ? a : function (e, t) { var i = e.end; if (i) {
                var o = this.printed_comments, r = i[t ? "comments_before" : "comments_after"];
                if (r && !o.has(r) && (e instanceof pe || r.every(e => !/comment[134]/.test(e.type)))) {
                    o.add(r);
                    var a = p.length;
                    r.filter(n, e).forEach((function (e, n) { if (!o.has(e))
                        if (o.add(e), y = !1, T ? (w("\n"), N(), T = !1) : e.nlb && (n > 0 || !K()) ? (w("\n"), N()) : (n > 0 || !t) && x(), /comment[134]/.test(e.type)) {
                            const t = U(e.value);
                            t && w("//" + t), T = !0;
                        }
                        else if ("comment2" == e.type) {
                            const t = U(e.value);
                            t && w("/*" + t + "*/"), y = !0;
                        } })), p.length > a && (b = a);
                }
            } }, line: function () { return l; }, col: function () { return c; }, pos: function () { return f; }, push_node: function (e) { G.push(e); }, pop_node: function () { return G.pop(); }, parent: function (e) { return G[G.length - 2 - (e || 0)]; } }; }
        !function () { function e(e, t) { e.DEFMETHOD("_codegen", t); } function t(e, n) { Array.isArray(e) ? e.forEach((function (e) { t(e, n); })) : e.DEFMETHOD("needs_parens", n); } function n(e, t, n, i) { var o = e.length - 1; n.in_directive = i, e.forEach((function (e, i) { !0 !== n.in_directive || e instanceof de || e instanceof Se || e instanceof me && e.body instanceof ln || (n.in_directive = !1), e instanceof Se || (n.indent(), e.print(n), i == o && t || (n.newline(), t && n.newline())), !0 === n.in_directive && e instanceof me && e.body instanceof ln && (n.in_directive = !1); })), n.in_directive = !1; } function i(e, t) { t.print("{"), t.with_indent(t.next_indent(), (function () { t.append_comments(e, !0); })), t.print("}"); } function o(e, t, o) { e.body.length > 0 ? t.with_block((function () { n(e.body, !1, t, o); })) : i(e, t); } function r(e, t, n) { var i = !1; n && (i = Tn(e, e => e instanceof we || (e instanceof Tt && "in" == e.operator ? bn : void 0))), e.print(t, i); } function u(e, t, n) { return n.option("quote_keys") ? n.print_string(e) : "" + +e == e && e >= 0 ? n.option("keep_numbers") ? n.print(e) : n.print(p(e)) : (O.has(e) ? !n.option("ie8") : j(e)) ? t && n.option("keep_quoted_props") ? n.print_string(e, t) : n.print_name(e) : n.print_string(e, t); } fe.DEFMETHOD("print", (function (e, t) { var n = this, i = n._codegen; function o() { e.prepend_comments(n), n.add_source_map(e), i(n, e), e.append_comments(n); } n instanceof we ? e.active_scope = n : !e.use_asm && n instanceof de && "use asm" == n.value && (e.use_asm = e.active_scope), e.push_node(n), t || n.needs_parens(e) ? e.with_parens(o) : o(), e.pop_node(), n === e.use_asm && (e.use_asm = null); })), fe.DEFMETHOD("_print", fe.prototype.print), fe.DEFMETHOD("print_to_string", (function (e) { var t = Pn(e); return this.print(t), t.get(); })), t(fe, s), t(Le, (function (e) { if (!e.has_parens() && kn(e))
            return !0; var t; if (e.option("webkit") && ((t = e.parent()) instanceof Et && t.expression === this))
            return !0; if (e.option("wrap_iife") && ((t = e.parent()) instanceof dt && t.expression === this))
            return !0; if (e.option("wrap_func_args") && ((t = e.parent()) instanceof dt && t.args.includes(this)))
            return !0; return !1; })), t(Ve, (function (e) { var t = e.parent(); return t instanceof Et && t.expression === this; })), t(Ft, (function (e) { return !e.has_parens() && kn(e); })), t(Vt, kn), t(St, (function (e) { var t = e.parent(); return t instanceof Et && t.expression === this || t instanceof dt && t.expression === this || t instanceof Tt && "**" === t.operator && this instanceof vt && t.left === this && "++" !== this.operator && "--" !== this.operator; })), t(je, (function (e) { var t = e.parent(); return t instanceof Et && t.expression === this || t instanceof dt && t.expression === this || e.option("safari10") && t instanceof vt; })), t(ht, (function (e) { var t = e.parent(); return t instanceof dt || t instanceof St || t instanceof Tt || t instanceof lt || t instanceof Et || t instanceof Ot || t instanceof Mt || t instanceof yt || t instanceof Ve || t instanceof Ct || t instanceof Ne || t instanceof Me && this === t.object || t instanceof Ze || t instanceof _t; })), t(Tt, (function (e) { var t = e.parent(); if (t instanceof dt && t.expression === this)
            return !0; if (t instanceof St)
            return !0; if (t instanceof Et && t.expression === this)
            return !0; if (t instanceof Tt) {
            var n = t.operator, i = ae[n], o = this.operator, r = ae[o];
            if (i > r || i == r && (this === t.right || "**" == n))
                return !0;
        } })), t(Ze, (function (e) { var t = e.parent(); return t instanceof Tt && "=" !== t.operator || (t instanceof dt && t.expression === this || (t instanceof yt && t.condition === this || (t instanceof St || (t instanceof Et && t.expression === this || void 0)))); })), t(Et, (function (e) { var t = e.parent(); if (t instanceof mt && t.expression === this)
            return Tn(this, e => e instanceof we || (e instanceof dt ? bn : void 0)); })), t(dt, (function (e) { var t, n = e.parent(); return !!(n instanceof mt && n.expression === this || n instanceof _t && n.is_default && this.expression instanceof Le) || this.expression instanceof Le && n instanceof Et && n.expression === this && (t = e.parent(1)) instanceof bt && t.left === n; })), t(mt, (function (e) { var t = e.parent(); if (0 === this.args.length && (t instanceof Et || t instanceof dt && t.expression === this))
            return !0; })), t(fn, (function (e) { var t = e.parent(); if (t instanceof Et && t.expression === this) {
            var n = this.getValue();
            if (n < 0 || /^0/.test(p(n)))
                return !0;
        } })), t(pn, (function (e) { var t = e.parent(); if (t instanceof Et && t.expression === this && this.getValue().startsWith("-"))
            return !0; })), t([bt, yt], (function (e) { var t = e.parent(); return t instanceof St || (t instanceof Tt && !(t instanceof bt) || (t instanceof dt && t.expression === this || (t instanceof yt && t.condition === this || (t instanceof Et && t.expression === this || (this instanceof bt && this.left instanceof Be && !1 === this.left.is_array || void 0))))); })), e(de, (function (e, t) { t.print_string(e.value, e.quote), t.semicolon(); })), e(Ne, (function (e, t) { t.print("..."), e.expression.print(t); })), e(Be, (function (e, t) { t.print(e.is_array ? "[" : "{"); var n = e.names.length; e.names.forEach((function (e, i) { i > 0 && t.comma(), e.print(t), i == n - 1 && e instanceof gn && t.comma(); })), t.print(e.is_array ? "]" : "}"); })), e(_e, (function (e, t) { t.print("debugger"), t.semicolon(); })), ve.DEFMETHOD("_do_print_body", (function (e) { f(this.body, e); })), e(pe, (function (e, t) { e.body.print(t), t.semicolon(); })), e(xe, (function (e, t) { n(e.body, !0, t, !0), t.print(""); })), e(Ae, (function (e, t) { e.label.print(t), t.colon(), e.body.print(t); })), e(me, (function (e, t) { e.body.print(t), t.semicolon(); })), e(De, (function (e, t) { o(e, t); })), e(Se, (function (e, t) { t.semicolon(); })), e(be, (function (e, t) { t.print("do"), t.space(), _(e.body, t), t.space(), t.print("while"), t.space(), t.with_parens((function () { e.condition.print(t); })), t.semicolon(); })), e(Ce, (function (e, t) { t.print("while"), t.space(), t.with_parens((function () { e.condition.print(t); })), t.space(), e._do_print_body(t); })), e(Oe, (function (e, t) { t.print("for"), t.space(), t.with_parens((function () { e.init ? (e.init instanceof at ? e.init.print(t) : r(e.init, t, !0), t.print(";"), t.space()) : t.print(";"), e.condition ? (e.condition.print(t), t.print(";"), t.space()) : t.print(";"), e.step && e.step.print(t); })), t.space(), e._do_print_body(t); })), e(Fe, (function (e, t) { t.print("for"), e.await && (t.space(), t.print("await")), t.space(), t.with_parens((function () { e.init.print(t), t.space(), t.print(e instanceof Me ? "of" : "in"), t.space(), e.object.print(t); })), t.space(), e._do_print_body(t); })), e(Re, (function (e, t) { t.print("with"), t.space(), t.with_parens((function () { e.expression.print(t); })), t.space(), e._do_print_body(t); })), ke.DEFMETHOD("_do_print", (function (e, t) { var n = this; t || (n.async && (e.print("async"), e.space()), e.print("function"), n.is_generator && e.star(), n.name && e.space()), n.name instanceof Pt ? n.name.print(e) : t && n.name instanceof fe && e.with_square((function () { n.name.print(e); })), e.with_parens((function () { n.argnames.forEach((function (t, n) { n && e.comma(), t.print(e); })); })), e.space(), o(n, e, !0); })), e(ke, (function (e, t) { e._do_print(t); })), e(Ke, (function (e, t) { var n = e.prefix, i = n instanceof ke || n instanceof Tt || n instanceof yt || n instanceof ht || n instanceof St || n instanceof gt && n.expression instanceof Ft; i && t.print("("), e.prefix.print(t), i && t.print(")"), e.template_string.print(t); })), e(Ue, (function (e, t) { var n = t.parent() instanceof Ke; t.print("`"); for (var i = 0; i < e.segments.length; i++)
            e.segments[i] instanceof Ge ? n ? t.print(e.segments[i].raw) : t.print_template_string_chars(e.segments[i].value) : (t.print("${"), e.segments[i].print(t), t.print("}")); t.print("`"); })), Ve.DEFMETHOD("_do_print", (function (e) { var t = this, n = e.parent(), i = n instanceof Tt && !(n instanceof bt) || n instanceof St || n instanceof dt && t === n.expression; i && e.print("("), t.async && (e.print("async"), e.space()), 1 === t.argnames.length && t.argnames[0] instanceof Pt ? t.argnames[0].print(e) : e.with_parens((function () { t.argnames.forEach((function (t, n) { n && e.comma(), t.print(e); })); })), e.space(), e.print("=>"), e.space(); const r = t.body[0]; if (1 === t.body.length && r instanceof ze) {
            const t = r.value;
            t ? !function e(t) { return t instanceof Ft || (t instanceof ht ? e(t.expressions[0]) : "Call" === t.TYPE ? e(t.expression) : t instanceof Ke ? e(t.prefix) : t instanceof gt || t instanceof Dt ? e(t.expression) : t instanceof yt ? e(t.condition) : t instanceof Tt ? e(t.left) : t instanceof At && e(t.expression)); }(t) ? t.print(e) : (e.print("("), t.print(e), e.print(")")) : e.print("{}");
        }
        else
            o(t, e); i && e.print(")"); })), Xe.DEFMETHOD("_do_print", (function (e, t) { if (e.print(t), this.value) {
            e.space();
            const t = this.value.start.comments_before;
            t && t.length && !e.printed_comments.has(t) ? (e.print("("), this.value.print(e), e.print(")")) : this.value.print(e);
        } e.semicolon(); })), e(ze, (function (e, t) { e._do_print(t, "return"); })), e(We, (function (e, t) { e._do_print(t, "throw"); })), e(Ze, (function (e, t) { var n = e.is_star ? "*" : ""; t.print("yield" + n), e.expression && (t.space(), e.expression.print(t)); })), e(je, (function (e, t) { t.print("await"), t.space(); var n = e.expression, i = !(n instanceof dt || n instanceof nn || n instanceof Et || n instanceof St || n instanceof cn); i && t.print("("), e.expression.print(t), i && t.print(")"); })), Ye.DEFMETHOD("_do_print", (function (e, t) { e.print(t), this.label && (e.space(), this.label.print(e)), e.semicolon(); })), e(qe, (function (e, t) { e._do_print(t, "break"); })), e($e, (function (e, t) { e._do_print(t, "continue"); })), e(Je, (function (e, t) { t.print("if"), t.space(), t.with_parens((function () { e.condition.print(t); })), t.space(), e.alternative ? (!function (e, t) { var n = e.body; if (t.option("braces") || t.option("ie8") && n instanceof be)
            return _(n, t); if (!n)
            return t.force_semicolon(); for (;;)
            if (n instanceof Je) {
                if (!n.alternative)
                    return void _(e.body, t);
                n = n.alternative;
            }
            else {
                if (!(n instanceof ve))
                    break;
                n = n.body;
            } f(e.body, t); }(e, t), t.space(), t.print("else"), t.space(), e.alternative instanceof Je ? e.alternative.print(t) : f(e.alternative, t)) : e._do_print_body(t); })), e(Qe, (function (e, t) { t.print("switch"), t.space(), t.with_parens((function () { e.expression.print(t); })), t.space(); var n = e.body.length - 1; n < 0 ? i(e, t) : t.with_block((function () { e.body.forEach((function (e, i) { t.indent(!0), e.print(t), i < n && e.body.length > 0 && t.newline(); })); })); })), et.DEFMETHOD("_do_print_body", (function (e) { e.newline(), this.body.forEach((function (t) { e.indent(), t.print(e), e.newline(); })); })), e(tt, (function (e, t) { t.print("default:"), e._do_print_body(t); })), e(nt, (function (e, t) { t.print("case"), t.space(), e.expression.print(t), t.print(":"), e._do_print_body(t); })), e(it, (function (e, t) { t.print("try"), t.space(), o(e, t), e.bcatch && (t.space(), e.bcatch.print(t)), e.bfinally && (t.space(), e.bfinally.print(t)); })), e(ot, (function (e, t) { t.print("catch"), e.argname && (t.space(), t.with_parens((function () { e.argname.print(t); }))), t.space(), o(e, t); })), e(rt, (function (e, t) { t.print("finally"), t.space(), o(e, t); })), at.DEFMETHOD("_do_print", (function (e, t) { e.print(t), e.space(), this.definitions.forEach((function (t, n) { n && e.comma(), t.print(e); })); var n = e.parent(); (!(n instanceof Oe || n instanceof Fe) || n && n.init !== this) && e.semicolon(); })), e(ut, (function (e, t) { e._do_print(t, "let"); })), e(st, (function (e, t) { e._do_print(t, "var"); })), e(ct, (function (e, t) { e._do_print(t, "const"); })), e(pt, (function (e, t) { t.print("import"), t.space(), e.imported_name && e.imported_name.print(t), e.imported_name && e.imported_names && (t.print(","), t.space()), e.imported_names && (1 === e.imported_names.length && "*" === e.imported_names[0].foreign_name.name ? e.imported_names[0].print(t) : (t.print("{"), e.imported_names.forEach((function (n, i) { t.space(), n.print(t), i < e.imported_names.length - 1 && t.print(","); })), t.space(), t.print("}"))), (e.imported_name || e.imported_names) && (t.space(), t.print("from"), t.space()), e.module_name.print(t), t.semicolon(); })), e(ft, (function (e, t) { var n = t.parent() instanceof pt, i = e.name.definition(); (i && i.mangled_name || e.name.name) !== e.foreign_name.name ? (n ? t.print(e.foreign_name.name) : e.name.print(t), t.space(), t.print("as"), t.space(), n ? e.name.print(t) : t.print(e.foreign_name.name)) : e.name.print(t); })), e(_t, (function (e, t) { if (t.print("export"), t.space(), e.is_default && (t.print("default"), t.space()), e.exported_names)
            1 === e.exported_names.length && "*" === e.exported_names[0].name.name ? e.exported_names[0].print(t) : (t.print("{"), e.exported_names.forEach((function (n, i) { t.space(), n.print(t), i < e.exported_names.length - 1 && t.print(","); })), t.space(), t.print("}"));
        else if (e.exported_value)
            e.exported_value.print(t);
        else if (e.exported_definition && (e.exported_definition.print(t), e.exported_definition instanceof at))
            return; e.module_name && (t.space(), t.print("from"), t.space(), e.module_name.print(t)), (e.exported_value && !(e.exported_value instanceof Pe || e.exported_value instanceof Le || e.exported_value instanceof kt) || e.module_name || e.exported_names) && t.semicolon(); })), e(lt, (function (e, t) { if (e.name.print(t), e.value) {
            t.space(), t.print("="), t.space();
            var n = t.parent(1), i = n instanceof Oe || n instanceof Fe;
            r(e.value, t, i);
        } })), e(dt, (function (e, t) { e.expression.print(t), e instanceof mt && 0 === e.args.length || ((e.expression instanceof dt || e.expression instanceof ke) && t.add_mapping(e.start), t.with_parens((function () { e.args.forEach((function (e, n) { n && t.comma(), e.print(t); })); }))); })), e(mt, (function (e, t) { t.print("new"), t.space(), dt.prototype._codegen(e, t); })), ht.DEFMETHOD("_do_print", (function (e) { this.expressions.forEach((function (t, n) { n > 0 && (e.comma(), e.should_break() && (e.newline(), e.indent())), t.print(e); })); })), e(ht, (function (e, t) { e._do_print(t); })), e(gt, (function (e, t) { var n = e.expression; n.print(t); var i = e.property; t.option("ie8") && O.has(i) ? (t.print("["), t.add_mapping(e.end), t.print_string(i), t.print("]")) : (n instanceof fn && n.getValue() >= 0 && (/[xa-f.)]/i.test(t.last()) || t.print(".")), t.print("."), t.add_mapping(e.end), t.print_name(i)); })), e(Dt, (function (e, t) { e.expression.print(t), t.print("["), e.property.print(t), t.print("]"); })), e(vt, (function (e, t) { var n = e.operator; t.print(n), (/^[a-z]/i.test(n) || /[+-]$/.test(n) && e.expression instanceof vt && /^[+-]/.test(e.expression.operator)) && t.space(), e.expression.print(t); })), e(At, (function (e, t) { e.expression.print(t), t.print(e.operator); })), e(Tt, (function (e, t) { var n = e.operator; e.left.print(t), ">" == n[0] && e.left instanceof At && "--" == e.left.operator ? t.print(" ") : t.space(), t.print(n), ("<" == n || "<<" == n) && e.right instanceof vt && "!" == e.right.operator && e.right.expression instanceof vt && "--" == e.right.expression.operator ? t.print(" ") : t.space(), e.right.print(t); })), e(yt, (function (e, t) { e.condition.print(t), t.space(), t.print("?"), t.space(), e.consequent.print(t), t.space(), t.colon(), e.alternative.print(t); })), e(Ot, (function (e, t) { t.with_square((function () { var n = e.elements, i = n.length; i > 0 && t.space(), n.forEach((function (e, n) { n && t.comma(), e.print(t), n === i - 1 && e instanceof gn && t.comma(); })), i > 0 && t.space(); })); })), e(Ft, (function (e, t) { e.properties.length > 0 ? t.with_block((function () { e.properties.forEach((function (e, n) { n && (t.print(","), t.newline()), t.indent(), e.print(t); })), t.newline(); })) : i(e, t); })), e(kt, (function (e, t) { if (t.print("class"), t.space(), e.name && (e.name.print(t), t.space()), e.extends) {
            var n = !(e.extends instanceof nn || e.extends instanceof Et || e.extends instanceof Vt || e.extends instanceof Le);
            t.print("extends"), n ? t.print("(") : t.space(), e.extends.print(t), n ? t.print(")") : t.space();
        } e.properties.length > 0 ? t.with_block((function () { e.properties.forEach((function (e, n) { n && t.newline(), t.indent(), e.print(t); })), t.newline(); })) : t.print("{}"); })), e(Bt, (function (e, t) { t.print("new.target"); })), e(Rt, (function (e, t) { function n(e) { var t = e.definition(); return t ? t.mangled_name || t.name : e.name; } var i = t.option("shorthand"); i && e.value instanceof Pt && j(e.key) && n(e.value) === e.key && !O.has(e.key) ? u(e.key, e.quote, t) : i && e.value instanceof Ct && e.value.left instanceof Pt && j(e.key) && n(e.value.left) === e.key ? (u(e.key, e.quote, t), t.space(), t.print("="), t.space(), e.value.right.print(t)) : (e.key instanceof fe ? t.with_square((function () { e.key.print(t); })) : u(e.key, e.quote, t), t.colon(), e.value.print(t)); })), e(It, (e, t) => { e.static && (t.print("static"), t.space()), e.key instanceof qt ? u(e.key.name, e.quote, t) : (t.print("["), e.key.print(t), t.print("]")), e.value && (t.print("="), e.value.print(t)), t.semicolon(); }), Mt.DEFMETHOD("_print_getter_setter", (function (e, t) { var n = this; n.static && (t.print("static"), t.space()), e && (t.print(e), t.space()), n.key instanceof Yt ? u(n.key.name, n.quote, t) : t.with_square((function () { n.key.print(t); })), n.value._do_print(t, !0); })), e(wt, (function (e, t) { e._print_getter_setter("set", t); })), e(xt, (function (e, t) { e._print_getter_setter("get", t); })), e(Nt, (function (e, t) { var n; e.is_generator && e.async ? n = "async*" : e.is_generator ? n = "*" : e.async && (n = "async"), e._print_getter_setter(n, t); })), Pt.DEFMETHOD("_do_print", (function (e) { var t = this.definition(); e.print_name(t ? t.mangled_name || t.name : this.name); })), e(Pt, (function (e, t) { e._do_print(t); })), e(gn, a), e(sn, (function (e, t) { t.print("this"); })), e(un, (function (e, t) { t.print("super"); })), e(cn, (function (e, t) { t.print(e.getValue()); })), e(ln, (function (e, t) { t.print_string(e.getValue(), e.quote, t.in_directive); })), e(fn, (function (e, t) { (t.option("keep_numbers") || t.use_asm) && e.start && null != e.start.raw ? t.print(e.start.raw) : t.print(p(e.getValue())); })), e(pn, (function (e, t) { t.print(e.getValue() + "n"); })); const c = /(<\s*\/\s*script)/i, l = (e, t) => t.replace("/", "\\/"); function f(e, t) { t.option("braces") ? _(e, t) : !e || e instanceof Se ? t.force_semicolon() : e.print(t); } function p(e) { var t, n, i, o = e.toString(10).replace(/^0\./, ".").replace("e+", "e"), r = [o]; return Math.floor(e) === e && (e < 0 ? r.push("-0x" + (-e).toString(16).toLowerCase()) : r.push("0x" + e.toString(16).toLowerCase())), (t = /^\.0+/.exec(o)) ? (n = t[0].length, i = o.slice(n), r.push(i + "e-" + (i.length + n - 1))) : (t = /0+$/.exec(o)) ? (n = t[0].length, r.push(o.slice(0, -n) + "e" + n)) : (t = /^(\d)\.(\d+)e(-?\d+)$/.exec(o)) && r.push(t[1] + t[2] + "e" + (t[3] - t[2].length)), function (e) { for (var t = e[0], n = t.length, i = 1; i < e.length; ++i)
            e[i].length < n && (n = (t = e[i]).length); return t; }(r); } function _(e, t) { !e || e instanceof Se ? t.print("{}") : e instanceof De ? e.print(t) : t.with_block((function () { t.indent(), e.print(t), t.newline(); })); } function d(e, t) { e.forEach((function (e) { e.DEFMETHOD("add_source_map", t); })); } e(_n, (function (e, t) { let { source: n, flags: i } = e.getValue(); n = A(n), i = i ? function (e) { const t = new Set(e.split("")); let n = ""; for (const e of "gimuy")
            t.has(e) && (n += e, t.delete(e)); return t.size && t.forEach(e => { n += e; }), n; }(i) : "", n = n.replace(c, l), t.print(t.to_utf8(`/${n}/${i}`)); const o = t.parent(); o instanceof Tt && /^\w/.test(o.operator) && o.left === e && t.print(" "); })), d([fe, Ae, xe], a), d([Ot, De, ot, kt, cn, _e, at, de, rt, He, ke, mt, Ft, ve, Pt, Qe, et, Ue, Ge, it], (function (e) { e.add_mapping(this.start); })), d([xt, wt], (function (e) { e.add_mapping(this.start, this.key.name); })), d([Mt], (function (e) { e.add_mapping(this.start, this.key); })); }();
        const Bn = (e, t) => e.TYPE === t.TYPE && e.shallow_cmp(t), Kn = e => { const t = Object.keys(e).map(t => { if ("eq" === e[t])
            return `this.${t} === other.${t}`; if ("exist" === e[t])
            return `(this.${t} == null ? other.${t} == null : this.${t} === other.${t})`; throw new Error(`mkshallow: Unexpected instruction: ${e[t]}`); }).join(" && "); return new Function("other", "return " + t); }, Un = () => !0;
        fe.prototype.shallow_cmp = function () { throw new Error("did not find a shallow_cmp function for " + this.constructor.name); }, _e.prototype.shallow_cmp = Un, de.prototype.shallow_cmp = Kn({ value: "eq" }), me.prototype.shallow_cmp = Un, ge.prototype.shallow_cmp = Un, Se.prototype.shallow_cmp = Un, Ae.prototype.shallow_cmp = Kn({ "label.name": "eq" }), be.prototype.shallow_cmp = Un, Ce.prototype.shallow_cmp = Un, Oe.prototype.shallow_cmp = Kn({ init: "exist", condition: "exist", step: "exist" }), Fe.prototype.shallow_cmp = Un, Me.prototype.shallow_cmp = Un, Re.prototype.shallow_cmp = Un, xe.prototype.shallow_cmp = Un, Ne.prototype.shallow_cmp = Un, ke.prototype.shallow_cmp = Kn({ is_generator: "eq", async: "eq" }), Be.prototype.shallow_cmp = Kn({ is_array: "eq" }), Ke.prototype.shallow_cmp = Un, Ue.prototype.shallow_cmp = Un, Ge.prototype.shallow_cmp = Kn({ value: "eq" }), He.prototype.shallow_cmp = Un, Ye.prototype.shallow_cmp = Un, je.prototype.shallow_cmp = Un, Ze.prototype.shallow_cmp = Kn({ is_star: "eq" }), Je.prototype.shallow_cmp = Kn({ alternative: "exist" }), Qe.prototype.shallow_cmp = Un, et.prototype.shallow_cmp = Un, it.prototype.shallow_cmp = Kn({ bcatch: "exist", bfinally: "exist" }), ot.prototype.shallow_cmp = Kn({ argname: "exist" }), rt.prototype.shallow_cmp = Un, at.prototype.shallow_cmp = Un, lt.prototype.shallow_cmp = Kn({ value: "exist" }), ft.prototype.shallow_cmp = Un, pt.prototype.shallow_cmp = Kn({ imported_name: "exist", imported_names: "exist" }), _t.prototype.shallow_cmp = Kn({ exported_definition: "exist", exported_value: "exist", exported_names: "exist", module_name: "eq", is_default: "eq" }), dt.prototype.shallow_cmp = Un, ht.prototype.shallow_cmp = Un, Et.prototype.shallow_cmp = Un, gt.prototype.shallow_cmp = Kn({ property: "eq" }), St.prototype.shallow_cmp = Kn({ operator: "eq" }), Tt.prototype.shallow_cmp = Kn({ operator: "eq" }), yt.prototype.shallow_cmp = Un, Ot.prototype.shallow_cmp = Un, Ft.prototype.shallow_cmp = Un, Mt.prototype.shallow_cmp = Un, Rt.prototype.shallow_cmp = Kn({ key: "eq" }), wt.prototype.shallow_cmp = Kn({ static: "eq" }), xt.prototype.shallow_cmp = Kn({ static: "eq" }), Nt.prototype.shallow_cmp = Kn({ static: "eq", is_generator: "eq", async: "eq" }), kt.prototype.shallow_cmp = Kn({ name: "exist", extends: "exist" }), It.prototype.shallow_cmp = Kn({ static: "eq" }), Pt.prototype.shallow_cmp = Kn({ name: "eq" }), Bt.prototype.shallow_cmp = Un, sn.prototype.shallow_cmp = Un, un.prototype.shallow_cmp = Un, ln.prototype.shallow_cmp = Kn({ value: "eq" }), fn.prototype.shallow_cmp = Kn({ value: "eq" }), pn.prototype.shallow_cmp = Kn({ value: "eq" }), _n.prototype.shallow_cmp = function (e) { return this.value.flags === e.value.flags && this.value.source === e.value.source; }, dn.prototype.shallow_cmp = Un;
        let Gn = null, Hn = null;
        class Xn {
            constructor(e, t, n) { this.name = t.name, this.orig = [t], this.init = n, this.eliminated = 0, this.assignments = 0, this.scope = e, this.replaced = 0, this.global = !1, this.export = 0, this.mangled_name = null, this.undeclared = !1, this.id = Xn.next_id++, this.chained = !1, this.direct_access = !1, this.escaped = 0, this.recursive_refs = 0, this.references = [], this.should_replace = void 0, this.single_use = !1, this.fixed = !1, Object.seal(this); }
            fixed_value() { return !this.fixed || this.fixed instanceof fe ? this.fixed : this.fixed(); }
            unmangleable(e) { return e || (e = {}), !!(Gn && Gn.has(this.id) && S(e.keep_fnames, this.orig[0].name)) || (this.global && !e.toplevel || 1 & this.export || this.undeclared || !e.eval && this.scope.pinned() || (this.orig[0] instanceof $t || this.orig[0] instanceof Wt) && S(e.keep_fnames, this.orig[0].name) || this.orig[0] instanceof Yt || (this.orig[0] instanceof Zt || this.orig[0] instanceof jt) && S(e.keep_classnames, this.orig[0].name)); }
            mangle(e) { const t = e.cache && e.cache.props; if (this.global && t && t.has(this.name))
                this.mangled_name = t.get(this.name);
            else if (!this.mangled_name && !this.unmangleable(e)) {
                var n = this.scope, i = this.orig[0];
                e.ie8 && i instanceof $t && (n = n.parent_scope);
                const o = zn(this);
                this.mangled_name = o ? o.mangled_name || o.name : n.next_mangled(e, this), this.global && t && t.set(this.name, this.mangled_name);
            } }
        }
        function zn(e) { if (e.orig[0] instanceof Jt && e.scope.is_block_scope())
            return e.scope.get_defun_scope().variables.get(e.name); }
        function Wn(e, t) { var n = e.enclosed; e: for (;;) {
            var i = Yn(++e.cname);
            if (!O.has(i) && !(t.reserved.has(i) || Hn && Hn.has(i))) {
                for (let e = n.length; --e >= 0;) {
                    const o = n[e];
                    if (i == (o.mangled_name || o.unmangleable(t) && o.name))
                        continue e;
                }
                return i;
            }
        } }
        Xn.next_id = 1, xe.DEFMETHOD("figure_out_scope", (function (e) { e = r(e, { cache: null, ie8: !1, safari10: !1 }); var t = this.parent_scope = null, n = new Map, i = null, o = null, a = [], s = new Cn((r, c) => { if (r.is_block_scope()) {
            const n = t;
            r.block_scope = t = new we(r), t._block_scope = !0;
            const i = r instanceof ot ? n.parent_scope : n;
            if (t.init_scope_vars(i), t.uses_with = n.uses_with, t.uses_eval = n.uses_eval, e.safari10 && (r instanceof Oe || r instanceof Fe) && a.push(t), r instanceof Qe) {
                const e = t;
                t = n, r.expression.walk(s), t = e;
                for (let e = 0; e < r.body.length; e++)
                    r.body[e].walk(s);
            }
            else
                c();
            return t = n, !0;
        } if (r instanceof Be) {
            const e = o;
            return o = r, c(), o = e, !0;
        } if (r instanceof we) {
            r.init_scope_vars(t);
            var l = t, f = i, p = n;
            return i = t = r, n = new Map, c(), t = l, i = f, n = p, !0;
        } if (r instanceof Ae) {
            var _ = r.label;
            if (n.has(_.name))
                throw new Error(d("Label {name} defined twice", _));
            return n.set(_.name, _), c(), n.delete(_.name), !0;
        } if (r instanceof Re)
            for (var m = t; m; m = m.parent_scope)
                m.uses_with = !0;
        else {
            if (r instanceof Pt && (r.scope = t), r instanceof tn && (r.thedef = r, r.references = []), r instanceof $t)
                i.def_function(r, "arguments" == r.name ? void 0 : i);
            else if (r instanceof Wt)
                u((r.scope = i.parent_scope.get_defun_scope()).def_function(r, i), 1);
            else if (r instanceof Zt)
                u(i.def_variable(r, i), 1);
            else if (r instanceof Qt)
                t.def_variable(r);
            else if (r instanceof jt)
                u((r.scope = i.parent_scope).def_function(r, i), 1);
            else if (r instanceof Ut || r instanceof Xt || r instanceof Ht || r instanceof Jt) {
                if ((h = r instanceof Gt ? t.def_variable(r, null) : i.def_variable(r, "SymbolVar" == r.TYPE ? null : void 0)).orig.every(e => e === r || (r instanceof Gt ? e instanceof $t : !(e instanceof Xt || e instanceof Ht))) || Q(`"${r.name}" is redeclared`, r.start.file, r.start.line, r.start.col, r.start.pos), r instanceof zt || u(h, 2), i !== t) {
                    r.mark_enclosed();
                    var h = t.find_variable(r);
                    r.thedef !== h && (r.thedef = h, r.reference());
                }
            }
            else if (r instanceof an) {
                var E = n.get(r.name);
                if (!E)
                    throw new Error(d("Undefined label {name} [{line},{col}]", { name: r.name, line: r.start.line, col: r.start.col }));
                r.thedef = E;
            }
            t instanceof xe || !(r instanceof _t || r instanceof pt) || Q(`"${r.TYPE}" statement may only appear at the top level`, r.start.file, r.start.line, r.start.col, r.start.pos);
        } }); function u(e, t) { if (o) {
            var n = 0;
            do {
                t++;
            } while (s.parent(n++) !== o);
        } var i = s.parent(t); if (e.export = i instanceof _t ? 1 : 0) {
            var r = i.exported_definition;
            (r instanceof Pe || r instanceof Lt) && i.is_default && (e.export = 2);
        } } this.walk(s), this.globals = new Map; s = new Cn(e => { if (e instanceof Ye && e.label)
            return e.label.thedef.references.push(e), !0; if (e instanceof nn) {
            var t, n = e.name;
            if ("eval" == n && s.parent() instanceof dt)
                for (var i = e.scope; i && !i.uses_eval; i = i.parent_scope)
                    i.uses_eval = !0;
            return s.parent() instanceof ft && s.parent(1).module_name || !(t = e.scope.find_variable(n)) ? (t = this.def_global(e), e instanceof on && (t.export = 1)) : t.scope instanceof ke && "arguments" == n && (t.scope.uses_arguments = !0), e.thedef = t, e.reference(), !e.scope.is_block_scope() || t.orig[0] instanceof Gt || (e.scope = e.scope.get_defun_scope()), !0;
        } var o; if (e instanceof Jt && (o = zn(e.definition())))
            for (i = e.scope; i && (_(i.enclosed, o), i !== o.scope);)
                i = i.parent_scope; }); if (this.walk(s), (e.ie8 || e.safari10) && Tn(this, e => { if (e instanceof Jt) {
            var t = e.name, n = e.thedef.references, i = e.scope.get_defun_scope(), o = i.find_variable(t) || this.globals.get(t) || i.def_variable(e);
            return n.forEach((function (e) { e.thedef = o, e.reference(); })), e.thedef = o, e.reference(), !0;
        } }), e.safari10)
            for (const e of a)
                e.parent_scope.variables.forEach((function (t) { _(e.enclosed, t); })); })), xe.DEFMETHOD("def_global", (function (e) { var t = this.globals, n = e.name; if (t.has(n))
            return t.get(n); var i = new Xn(this, e); return i.undeclared = !0, i.global = !0, t.set(n, i), i; })), we.DEFMETHOD("init_scope_vars", (function (e) { this.variables = new Map, this.functions = new Map, this.uses_with = !1, this.uses_eval = !1, this.parent_scope = e, this.enclosed = [], this.cname = -1, this._var_name_cache = null; })), we.DEFMETHOD("var_names", (function e() { var t = this._var_name_cache; return t || (this._var_name_cache = t = new Set(this.parent_scope ? e.call(this.parent_scope) : null), this._added_var_names && this._added_var_names.forEach(e => { t.add(e); }), this.enclosed.forEach((function (e) { t.add(e.name); })), this.variables.forEach((function (e, n) { t.add(n); }))), t; })), we.DEFMETHOD("add_var_name", (function (e) { this._added_var_names || (this._added_var_names = new Set), this._added_var_names.add(e), this._var_name_cache || this.var_names(), this._var_name_cache.add(e); })), we.DEFMETHOD("add_child_scope", (function (e) { if (e.parent_scope === this)
            return; e.parent_scope = this, e._var_name_cache = null, e._added_var_names && e._added_var_names.forEach(t => e.add_var_name(t)); const t = new Set(e.enclosed), n = (() => { const e = []; let t = this; do {
            e.push(t);
        } while (t = t.parent_scope); return e.reverse(), e; })(), i = []; for (const e of n) {
            i.forEach(t => _(e.enclosed, t));
            for (const n of e.variables.values())
                t.has(n) && (_(i, n), _(e.enclosed, n));
        } })), fe.DEFMETHOD("is_block_scope", s), kt.DEFMETHOD("is_block_scope", s), ke.DEFMETHOD("is_block_scope", s), xe.DEFMETHOD("is_block_scope", s), et.DEFMETHOD("is_block_scope", s), ge.DEFMETHOD("is_block_scope", u), we.DEFMETHOD("is_block_scope", (function () { return this._block_scope || !1; })), Te.DEFMETHOD("is_block_scope", u), ke.DEFMETHOD("init_scope_vars", (function () { we.prototype.init_scope_vars.apply(this, arguments), this.uses_arguments = !1, this.def_variable(new zt({ name: "arguments", start: this.start, end: this.end })); })), Ve.DEFMETHOD("init_scope_vars", (function () { we.prototype.init_scope_vars.apply(this, arguments), this.uses_arguments = !1; })), Pt.DEFMETHOD("mark_enclosed", (function () { for (var e = this.definition(), t = this.scope; t && (_(t.enclosed, e), t !== e.scope);)
            t = t.parent_scope; })), Pt.DEFMETHOD("reference", (function () { this.definition().references.push(this), this.mark_enclosed(); })), we.DEFMETHOD("find_variable", (function (e) { return e instanceof Pt && (e = e.name), this.variables.get(e) || this.parent_scope && this.parent_scope.find_variable(e); })), we.DEFMETHOD("def_function", (function (e, t) { var n = this.def_variable(e, t); return (!n.init || n.init instanceof Pe) && (n.init = t), this.functions.set(e.name, n), n; })), we.DEFMETHOD("def_variable", (function (e, t) { var n = this.variables.get(e.name); return n ? (n.orig.push(e), n.init && (n.scope !== e.scope || n.init instanceof Le) && (n.init = t)) : (n = new Xn(this, e, t), this.variables.set(e.name, n), n.global = !this.parent_scope), e.thedef = n; })), we.DEFMETHOD("next_mangled", (function (e) { return Wn(this, e); })), xe.DEFMETHOD("next_mangled", (function (e) { let t; const n = this.mangled_names; do {
            t = Wn(this, e);
        } while (n.has(t)); return t; })), Le.DEFMETHOD("next_mangled", (function (e, t) { for (var n = t.orig[0] instanceof zt && this.name && this.name.definition(), i = n ? n.mangled_name || n.name : null;;) {
            var o = Wn(this, e);
            if (!i || i != o)
                return o;
        } })), Pt.DEFMETHOD("unmangleable", (function (e) { var t = this.definition(); return !t || t.unmangleable(e); })), tn.DEFMETHOD("unmangleable", s), Pt.DEFMETHOD("unreferenced", (function () { return !this.definition().references.length && !this.scope.pinned(); })), Pt.DEFMETHOD("definition", (function () { return this.thedef; })), Pt.DEFMETHOD("global", (function () { return this.thedef.global; })), xe.DEFMETHOD("_default_mangler_options", (function (e) { return (e = r(e, { eval: !1, ie8: !1, keep_classnames: !1, keep_fnames: !1, module: !1, reserved: [], toplevel: !1 })).module && (e.toplevel = !0), Array.isArray(e.reserved) || e.reserved instanceof Set || (e.reserved = []), e.reserved = new Set(e.reserved), e.reserved.add("arguments"), e; })), xe.DEFMETHOD("mangle_names", (function (e) { e = this._default_mangler_options(e); var t = -1, n = []; e.keep_fnames && (Gn = new Set); const i = this.mangled_names = new Set; e.cache && (this.globals.forEach(r), e.cache.props && e.cache.props.forEach((function (e) { i.add(e); }))); var o = new Cn((function (i, o) { if (i instanceof Ae) {
            var a = t;
            return o(), t = a, !0;
        } if (i instanceof we)
            i.variables.forEach(r);
        else if (i.is_block_scope())
            i.block_scope.variables.forEach(r);
        else if (Gn && i instanceof lt && i.value instanceof ke && !i.value.name && S(e.keep_fnames, i.name.name))
            Gn.add(i.name.definition().id);
        else {
            if (i instanceof tn) {
                let e;
                do {
                    e = Yn(++t);
                } while (O.has(e));
                return i.mangled_name = e, !0;
            }
            !e.ie8 && !e.safari10 && i instanceof Jt && n.push(i.definition());
        } })); function r(t) { !(e.reserved.has(t.name) || 1 & t.export) && n.push(t); } this.walk(o), (e.keep_fnames || e.keep_classnames) && (Hn = new Set, n.forEach(t => { t.name.length < 6 && t.unmangleable(e) && Hn.add(t.name); })), n.forEach(t => { t.mangle(e); }), Gn = null, Hn = null; })), xe.DEFMETHOD("find_colliding_names", (function (e) { const t = e.cache && e.cache.props, n = new Set; return e.reserved.forEach(i), this.globals.forEach(o), this.walk(new Cn((function (e) { e instanceof we && e.variables.forEach(o), e instanceof Jt && o(e.definition()); }))), n; function i(e) { n.add(e); } function o(n) { var o = n.name; if (n.global && t && t.has(o))
            o = t.get(o);
        else if (!n.unmangleable(e))
            return; i(o); } })), xe.DEFMETHOD("expand_names", (function (e) { Yn.reset(), Yn.sort(), e = this._default_mangler_options(e); var t = this.find_colliding_names(e), n = 0; function i(i) { if (i.global && e.cache)
            return; if (i.unmangleable(e))
            return; if (e.reserved.has(i.name))
            return; const o = zn(i), r = i.name = o ? o.name : function () { var e; do {
            e = Yn(n++);
        } while (t.has(e) || O.has(e)); return e; }(); i.orig.forEach((function (e) { e.name = r; })), i.references.forEach((function (e) { e.name = r; })); } this.globals.forEach(i), this.walk(new Cn((function (e) { e instanceof we && e.variables.forEach(i), e instanceof Jt && i(e.definition()); }))); })), fe.DEFMETHOD("tail_node", c), ht.DEFMETHOD("tail_node", (function () { return this.expressions[this.expressions.length - 1]; })), xe.DEFMETHOD("compute_char_frequency", (function (e) { e = this._default_mangler_options(e); try {
            fe.prototype.print = function (t, n) { this._print(t, n), this instanceof Pt && !this.unmangleable(e) ? Yn.consider(this.name, -1) : e.properties && (this instanceof gt ? Yn.consider(this.property, -1) : this instanceof Dt && function e(t) { t instanceof ln ? Yn.consider(t.value, -1) : t instanceof yt ? (e(t.consequent), e(t.alternative)) : t instanceof ht && e(t.tail_node()); }(this.property)); }, Yn.consider(this.print_to_string(), 1);
        }
        finally {
            fe.prototype.print = fe.prototype._print;
        } Yn.sort(); }));
        const Yn = (() => { const e = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ$_".split(""), t = "0123456789".split(""); let n, i; function o() { i = new Map, e.forEach((function (e) { i.set(e, 0); })), t.forEach((function (e) { i.set(e, 0); })); } function r(e, t) { return i.get(t) - i.get(e); } function a(e) { var t = "", i = 54; e++; do {
            e--, t += n[e % i], e = Math.floor(e / i), i = 64;
        } while (e > 0); return t; } return a.consider = function (e, t) { for (var n = e.length; --n >= 0;)
            i.set(e[n], i.get(e[n]) + t); }, a.sort = function () { n = h(e, r).concat(h(t, r)); }, a.reset = o, o(), a; })();
        function qn() { const e = {}; return Object.keys($n({ 0: 0 })).forEach(t => { const n = $n({ [t]: { 0: 0 } }); n && (e[t] = n); }), e; }
        function $n(e) { var t = Eo("", e); return t.error && t.error.defs; }
        let jn = void 0;
        fe.prototype.size = function (e, t) { jn = qn.mangle; let n = 0; return yn(this, (e, t) => { n += e._size(t); }, t || e && e.stack), jn = void 0, n; }, fe.prototype._size = () => 0, _e.prototype._size = () => 8, de.prototype._size = function () { return 2 + this.value.length; };
        const Zn = e => e.length && e.length - 1;
        ge.prototype._size = function () { return 2 + Zn(this.body); }, xe.prototype._size = function () { return Zn(this.body); }, Se.prototype._size = () => 1, Ae.prototype._size = () => 2, be.prototype._size = () => 9, Ce.prototype._size = () => 7, Oe.prototype._size = () => 8, Fe.prototype._size = () => 8, Re.prototype._size = () => 6, Ne.prototype._size = () => 3;
        const Jn = e => (e.is_generator ? 1 : 0) + (e.async ? 6 : 0);
        Ie.prototype._size = function () { return Jn(this) + 4 + Zn(this.argnames) + Zn(this.body); }, Le.prototype._size = function (e) { return 2 * !!kn(e) + Jn(this) + 12 + Zn(this.argnames) + Zn(this.body); }, Pe.prototype._size = function () { return Jn(this) + 13 + Zn(this.argnames) + Zn(this.body); }, Ve.prototype._size = function () { let e = 2 + Zn(this.argnames); return 1 === this.argnames.length && this.argnames[0] instanceof Pt || (e += 2), Jn(this) + e + Array.isArray(this.body) ? Zn(this.body) : this.body._size(); }, Be.prototype._size = () => 2, Ue.prototype._size = function () { return 2 + 3 * Math.floor(this.segments.length / 2); }, Ge.prototype._size = function () { return this.value.length; }, ze.prototype._size = function () { return this.value ? 7 : 6; }, We.prototype._size = () => 6, qe.prototype._size = function () { return this.label ? 6 : 5; }, $e.prototype._size = function () { return this.label ? 9 : 8; }, Je.prototype._size = () => 4, Qe.prototype._size = function () { return 8 + Zn(this.body); }, nt.prototype._size = function () { return 5 + Zn(this.body); }, tt.prototype._size = function () { return 8 + Zn(this.body); }, it.prototype._size = function () { return 3 + Zn(this.body); }, ot.prototype._size = function () { let e = 7 + Zn(this.body); return this.argname && (e += 2), e; }, rt.prototype._size = function () { return 7 + Zn(this.body); };
        const Qn = (e, t) => e + Zn(t.definitions);
        st.prototype._size = function () { return Qn(4, this); }, ut.prototype._size = function () { return Qn(4, this); }, ct.prototype._size = function () { return Qn(6, this); }, lt.prototype._size = function () { return this.value ? 1 : 0; }, ft.prototype._size = function () { return this.name ? 4 : 0; }, pt.prototype._size = function () { let e = 6; return this.imported_name && (e += 1), (this.imported_name || this.imported_names) && (e += 5), this.imported_names && (e += 2 + Zn(this.imported_names)), e; }, _t.prototype._size = function () { let e = 7 + (this.is_default ? 8 : 0); return this.exported_value && (e += this.exported_value._size()), this.exported_names && (e += 2 + Zn(this.exported_names)), this.module_name && (e += 5), e; }, dt.prototype._size = function () { return 2 + Zn(this.args); }, mt.prototype._size = function () { return 6 + Zn(this.args); }, ht.prototype._size = function () { return Zn(this.expressions); }, gt.prototype._size = function () { return this.property.length + 1; }, Dt.prototype._size = () => 2, St.prototype._size = function () { return "typeof" === this.operator ? 7 : "void" === this.operator ? 5 : this.operator.length; }, Tt.prototype._size = function (e) { if ("in" === this.operator)
            return 4; let t = this.operator.length; return ("+" === this.operator || "-" === this.operator) && this.right instanceof St && this.right.operator === this.operator && (t += 1), this.needs_parens(e) && (t += 2), t; }, yt.prototype._size = () => 3, Ot.prototype._size = function () { return 2 + Zn(this.elements); }, Ft.prototype._size = function (e) { let t = 2; return kn(e) && (t += 2), t + Zn(this.properties); };
        const ei = e => "string" == typeof e ? e.length : 0;
        Rt.prototype._size = function () { return ei(this.key) + 1; };
        const ti = e => e ? 7 : 0;
        xt.prototype._size = function () { return 5 + ti(this.static) + ei(this.key); }, wt.prototype._size = function () { return 5 + ti(this.static) + ei(this.key); }, Nt.prototype._size = function () { return ti(this.static) + ei(this.key) + Jn(this); }, kt.prototype._size = function () { return (this.name ? 8 : 7) + (this.extends ? 8 : 0); }, It.prototype._size = function () { return ti(this.static) + ("string" == typeof this.key ? this.key.length + 2 : 0) + (this.value ? 1 : 0); }, Pt.prototype._size = function () { return !jn || this.definition().unmangleable(jn) ? this.name.length : 2; }, qt.prototype._size = function () { return this.name.length; }, nn.prototype._size = function () { const { name: e, thedef: t } = this; return t && t.global ? e.length : "arguments" === e ? 9 : 2; }, Bt.prototype._size = () => 10, en.prototype._size = function () { return this.name.length; }, rn.prototype._size = function () { return this.name.length; }, sn.prototype._size = () => 4, un.prototype._size = () => 5, ln.prototype._size = function () { return this.value.length + 2; }, fn.prototype._size = function () { const { value: e } = this; return 0 === e ? 1 : e > 0 && Math.floor(e) === e ? Math.floor(Math.log10(e) + 1) : e.toString().length; }, pn.prototype._size = function () { return this.value.length; }, _n.prototype._size = function () { return this.value.toString().length; }, mn.prototype._size = () => 4, hn.prototype._size = () => 3, En.prototype._size = () => 6, gn.prototype._size = () => 0, Dn.prototype._size = () => 8, An.prototype._size = () => 4, vn.prototype._size = () => 5, je.prototype._size = () => 6, Ze.prototype._size = () => 6;
        const ni = (e, t) => e.flags & t, ii = (e, t) => { e.flags |= t; }, oi = (e, t) => { e.flags &= ~t; };
        class ri extends Cn {
            constructor(e, t) { super(), void 0 === e.defaults || e.defaults || (t = !0), this.options = r(e, { arguments: !1, arrows: !t, booleans: !t, booleans_as_integers: !1, collapse_vars: !t, comparisons: !t, computed_props: !t, conditionals: !t, dead_code: !t, defaults: !0, directives: !t, drop_console: !1, drop_debugger: !t, ecma: 5, evaluate: !t, expression: !1, global_defs: !1, hoist_funs: !1, hoist_props: !t, hoist_vars: !1, ie8: !1, if_return: !t, inline: !t, join_vars: !t, keep_classnames: !1, keep_fargs: !0, keep_fnames: !1, keep_infinity: !1, loops: !t, module: !1, negate_iife: !t, passes: 1, properties: !t, pure_getters: !t && "strict", pure_funcs: null, reduce_funcs: null, reduce_vars: !t, sequences: !t, side_effects: !t, switches: !t, top_retain: null, toplevel: !(!e || !e.top_retain), typeofs: !t, unsafe: !1, unsafe_arrows: !1, unsafe_comps: !1, unsafe_Function: !1, unsafe_math: !1, unsafe_symbols: !1, unsafe_methods: !1, unsafe_proto: !1, unsafe_regexp: !1, unsafe_undefined: !1, unused: !t, warnings: !1 }, !0); var n = this.options.global_defs; if ("object" == typeof n)
                for (var i in n)
                    "@" === i[0] && D(n, i) && (n[i.slice(1)] = ue(n[i], { expression: !0 })); !0 === this.options.inline && (this.options.inline = 3); var o = this.options.pure_funcs; this.pure_funcs = "function" == typeof o ? o : o ? function (e) { return !o.includes(e.expression.print_to_string()); } : u; var a = this.options.top_retain; a instanceof RegExp ? this.top_retain = function (e) { return a.test(e.name); } : "function" == typeof a ? this.top_retain = a : a && ("string" == typeof a && (a = a.split(/,/)), this.top_retain = function (e) { return a.includes(e.name); }), this.options.module && (this.directives["use strict"] = !0, this.options.toplevel = !0); var s = this.options.toplevel; this.toplevel = "string" == typeof s ? { funcs: /funcs/.test(s), vars: /vars/.test(s) } : { funcs: s, vars: s }; var c = this.options.sequences; this.sequences_limit = 1 == c ? 800 : 0 | c, this.warnings_produced = {}, this.evaluated_regexps = new Map; }
            option(e) { return this.options[e]; }
            exposed(e) { if (e.export)
                return !0; if (e.global)
                for (var t = 0, n = e.orig.length; t < n; t++)
                    if (!this.toplevel[e.orig[t] instanceof Wt ? "funcs" : "vars"])
                        return !0; return !1; }
            in_boolean_context() { if (!this.option("booleans"))
                return !1; for (var e, t = this.self(), n = 0; e = this.parent(n); n++) {
                if (e instanceof me || e instanceof yt && e.condition === t || e instanceof ye && e.condition === t || e instanceof Oe && e.condition === t || e instanceof Je && e.condition === t || e instanceof vt && "!" == e.operator && e.expression === t)
                    return !0;
                if (!(e instanceof Tt && ("&&" == e.operator || "||" == e.operator) || e instanceof yt || e.tail_node() === t))
                    return !1;
                t = e;
            } }
            compress(e) { e = e.resolve_defines(this), this.option("expression") && e.process_expression(!0); for (var t = +this.options.passes || 1, n = 1 / 0, i = !1, o = { ie8: this.option("ie8") }, r = 0; r < t; r++)
                if (e.figure_out_scope(o), 0 === r && this.option("drop_console") && (e = e.drop_console()), (r > 0 || this.option("reduce_vars")) && e.reset_opt_flags(this), e = e.transform(this), t > 1) {
                    let t = 0;
                    if (Tn(e, () => { t++; }), this.info("pass " + r + ": last_count: " + n + ", count: " + t), t < n)
                        n = t, i = !1;
                    else {
                        if (i)
                            break;
                        i = !0;
                    }
                } return this.option("expression") && e.process_expression(!1), e; }
            info(...e) { "verbose" == this.options.warnings && fe.warn(...e); }
            warn(e, t) { if (this.options.warnings) {
                var n = d(e, t);
                n in this.warnings_produced || (this.warnings_produced[n] = !0, fe.warn.apply(fe, arguments));
            } }
            clear_warnings() { this.warnings_produced = {}; }
            before(e, t) { if (ni(e, 256))
                return e; var n = !1; e instanceof we && (e = (e = e.hoist_properties(this)).hoist_declarations(this), n = !0), t(e, this), t(e, this); var i = e.optimize(this); return n && i instanceof we && (i.drop_unused(this), t(i, this)), i === e && ii(i, 256), i; }
        }
        function ai(e, t) { e.DEFMETHOD("optimize", (function (e) { if (ni(this, 512))
            return this; if (e.has_directive("use asm"))
            return this; var n = t(this, e); return ii(n, 512), n; })); }
        function si(e, t) { if (!((t = Ri(t)) instanceof fe)) {
            var n;
            if (e instanceof Ot) {
                var i = e.elements;
                if ("length" == t)
                    return mi(i.length, e);
                "number" == typeof t && t in i && (n = i[t]);
            }
            else if (e instanceof Ft) {
                t = "" + t;
                for (var o = e.properties, r = o.length; --r >= 0;) {
                    if (!(o[r] instanceof Rt))
                        return;
                    n || o[r].key !== t || (n = o[r].value);
                }
            }
            return n instanceof nn && n.fixed_value() || n;
        } }
        function ui(e, t, n, i, o, r) { var a = t.parent(o), s = ki(n, a); if (s)
            return s; if (!r && a instanceof dt && a.expression === n && !(i instanceof Ve) && !(i instanceof kt) && !a.is_expr_pure(e) && (!(i instanceof Le) || !(a instanceof mt) && i.contains_this()))
            return !0; if (a instanceof Ot)
            return ui(e, t, a, a, o + 1); if (a instanceof Rt && n === a.value) {
            var u = t.parent(o + 1);
            return ui(e, t, u, u, o + 2);
        } if (a instanceof Et && a.expression === n) {
            var c = si(i, a.property);
            return !r && ui(e, t, a, c, o + 1);
        } }
        function ci(e) { return e instanceof Ve || e instanceof Le; }
        function li(e) { if (e instanceof sn)
            return !0; if (e instanceof nn)
            return e.definition().orig[0] instanceof $t; if (e instanceof Et) {
            if ((e = e.expression) instanceof nn) {
                if (e.is_immutable())
                    return !1;
                e = e.fixed_value();
            }
            return !e || !(e instanceof _n) && (e instanceof cn || li(e));
        } return !1; }
        function fi(e, t) { if (!(e instanceof nn))
            return !1; for (var n = e.definition().orig, i = n.length; --i >= 0;)
            if (n[i] instanceof t)
                return !0; }
        function pi(e, t) { for (let n = 0;; n++) {
            const i = e.parent(n);
            if (i instanceof xe)
                return t ? i : void 0;
            if (i instanceof ke)
                return i;
            if (i.block_scope)
                return i.block_scope;
        } }
        function _i(e, t) { for (var n, i = 0; (n = e.parent(i++)) && !(n instanceof we);)
            if (n instanceof ot && n.argname) {
                n = n.argname.definition().scope;
                break;
            } return n.find_variable(t); }
        function di(e, t) { if (1 == t.length)
            return t[0]; if (0 == t.length)
            throw new Error("trying to create a sequence with length zero!"); return p(ht, e, { expressions: t.reduce(Ei, []) }); }
        function mi(e, t) { switch (typeof e) {
            case "string": return p(ln, t, { value: e });
            case "number": return isNaN(e) ? p(hn, t) : isFinite(e) ? 1 / e < 0 ? p(vt, t, { operator: "-", expression: p(fn, t, { value: -e }) }) : p(fn, t, { value: e }) : e < 0 ? p(vt, t, { operator: "-", expression: p(Dn, t) }) : p(Dn, t);
            case "boolean": return p(e ? An : vn, t);
            case "undefined": return p(En, t);
            default:
                if (null === e)
                    return p(mn, t, { value: null });
                if (e instanceof RegExp)
                    return p(_n, t, { value: { source: A(e.source), flags: e.flags } });
                throw new Error(d("Can't handle constant of type: {type}", { type: typeof e }));
        } }
        function hi(e, t, n) { return e instanceof vt && "delete" == e.operator || e instanceof dt && e.expression === t && (n instanceof Et || n instanceof nn && "eval" == n.name) ? di(t, [p(fn, t, { value: 0 }), n]) : n; }
        function Ei(e, t) { return t instanceof ht ? e.push(...t.expressions) : e.push(t), e; }
        function gi(e) { if (null === e)
            return []; if (e instanceof De)
            return e.body; if (e instanceof Se)
            return []; if (e instanceof pe)
            return [e]; throw new Error("Can't convert thing to statement array"); }
        function Di(e) { return null === e || (e instanceof Se || e instanceof De && 0 == e.body.length); }
        function Si(e) { return !(e instanceof Lt || e instanceof Pe || e instanceof ut || e instanceof ct || e instanceof _t || e instanceof pt); }
        function vi(e) { return e instanceof Te && e.body instanceof De ? e.body : e; }
        function Ai(e) { return "Call" == e.TYPE && (e.expression instanceof Le || Ai(e.expression)); }
        function Ti(e) { return e instanceof nn && e.definition().undeclared; }
        ai(fe, (function (e) { return e; })), xe.DEFMETHOD("drop_console", (function () { return this.transform(new On((function (e) { if ("Call" == e.TYPE) {
            var t = e.expression;
            if (t instanceof Et) {
                for (var n = t.expression; n.expression;)
                    n = n.expression;
                if (Ti(n) && "console" == n.name)
                    return p(En, e);
            }
        } }))); })), fe.DEFMETHOD("equivalent_to", (function (e) { return ((e, t) => { if (!Bn(e, t))
            return !1; const n = [e], i = [t], o = n.push.bind(n), r = i.push.bind(i); for (; n.length && i.length;) {
            const e = n.pop(), t = i.pop();
            if (!Bn(e, t))
                return !1;
            e._children_backwards(o), t._children_backwards(r);
        } return 0 == n.length && 0 == i.length; })(this, e); })), we.DEFMETHOD("process_expression", (function (e, t) { var n = this, i = new On((function (o) { if (e && o instanceof me)
            return p(ze, o, { value: o.body }); if (!e && o instanceof ze) {
            if (t) {
                var r = o.value && o.value.drop_side_effect_free(t, !0);
                return r ? p(me, o, { body: r }) : p(Se, o);
            }
            return p(me, o, { body: o.value || p(vt, o, { operator: "void", expression: p(fn, o, { value: 0 }) }) });
        } if (o instanceof kt || o instanceof ke && o !== n)
            return o; if (o instanceof ge) {
            var a = o.body.length - 1;
            a >= 0 && (o.body[a] = o.body[a].transform(i));
        }
        else
            o instanceof Je ? (o.body = o.body.transform(i), o.alternative && (o.alternative = o.alternative.transform(i))) : o instanceof Re && (o.body = o.body.transform(i)); return o; })); n.transform(i); })), function (e) { function t(e, t) { t.assignments = 0, t.chained = !1, t.direct_access = !1, t.escaped = 0, t.recursive_refs = 0, t.references = [], t.should_replace = void 0, t.single_use = void 0, t.scope.pinned() ? t.fixed = !1 : t.orig[0] instanceof Ht || !e.exposed(t) ? t.fixed = t.init : t.fixed = !1; } function n(e, n, i) { i.variables.forEach((function (i) { t(n, i), null === i.fixed ? (e.defs_to_safe_ids.set(i.id, e.safe_ids), s(e, i, !0)) : i.fixed && (e.loop_ids.set(i.id, e.in_loop), s(e, i, !0)); })); } function i(e, n) { n.block_scope && n.block_scope.variables.forEach(n => { t(e, n); }); } function o(e) { e.safe_ids = Object.create(e.safe_ids); } function r(e) { e.safe_ids = Object.getPrototypeOf(e.safe_ids); } function s(e, t, n) { e.safe_ids[t.id] = n; } function u(e, t) { if ("m" == t.single_use)
            return !1; if (e.safe_ids[t.id]) {
            if (null == t.fixed) {
                var n = t.orig[0];
                if (n instanceof zt || "arguments" == n.name)
                    return !1;
                t.fixed = p(En, n);
            }
            return !0;
        } return t.fixed instanceof Pe; } function c(e, t, n, i) { if (void 0 === t.fixed)
            return !0; let o; return null === t.fixed && (o = e.defs_to_safe_ids.get(t.id)) ? (o[t.id] = !1, e.defs_to_safe_ids.delete(t.id), !0) : !!D(e.safe_ids, t.id) && (!!u(e, t) && (!1 !== t.fixed && (!(null != t.fixed && (!i || t.references.length > t.assignments)) && (t.fixed instanceof Pe ? i instanceof fe && t.fixed.parent_scope === n : t.orig.every(e => !(e instanceof Ht || e instanceof Wt || e instanceof $t)))))); } function l(e, t, n, i, o, r, a) { var s = e.parent(r); if (o) {
            if (o.is_constant())
                return;
            if (o instanceof Vt)
                return;
        } if (s instanceof bt && "=" == s.operator && i === s.right || s instanceof dt && (i !== s.expression || s instanceof mt) || s instanceof Xe && i === s.value && i.scope !== t.scope || s instanceof lt && i === s.value || s instanceof Ze && i === s.value && i.scope !== t.scope)
            return !(a > 1) || o && o.is_constant_expression(n) || (a = 1), void ((!t.escaped || t.escaped > a) && (t.escaped = a)); if (s instanceof Ot || s instanceof je || s instanceof Tt && xi.has(s.operator) || s instanceof yt && i !== s.condition || s instanceof Ne || s instanceof ht && i === s.tail_node())
            l(e, t, n, s, s, r + 1, a);
        else if (s instanceof Rt && i === s.value) {
            var u = e.parent(r + 1);
            l(e, t, n, u, u, r + 2, a);
        }
        else if (s instanceof Et && i === s.expression && (l(e, t, n, s, o = si(o, s.property), r + 1, a + 1), o))
            return; r > 0 || s instanceof ht && i !== s.tail_node() || s instanceof me || (t.direct_access = !0); } e(fe, a); const f = e => Tn(e, e => { if (e instanceof Pt) {
            var t = e.definition();
            t && (e instanceof nn && t.references.push(e), t.fixed = !1);
        } }); e(Ie, (function (e, t, i) { return o(e), n(e, i, this), t(), r(e), !0; })), e(bt, (function (e, t, n) { var i = this; if (i.left instanceof Be)
            f(i.left);
        else {
            var o = i.left;
            if (o instanceof nn) {
                var r = o.definition(), a = c(e, r, o.scope, i.right);
                if (r.assignments++, a) {
                    var u = r.fixed;
                    if (u || "=" == i.operator) {
                        var _ = "=" == i.operator, d = _ ? i.right : i;
                        if (!ui(n, e, i, d, 0))
                            return r.references.push(o), _ || (r.chained = !0), r.fixed = _ ? function () { return i.right; } : function () { return p(Tt, i, { operator: i.operator.slice(0, -1), left: u instanceof fe ? u : u(), right: i.right }); }, s(e, r, !1), i.right.walk(e), s(e, r, !0), l(e, r, o.scope, i, d, 0, 1), !0;
                    }
                }
            }
        } })), e(Tt, (function (e) { if (xi.has(this.operator))
            return this.left.walk(e), o(e), this.right.walk(e), r(e), !0; })), e(ge, (function (e, t, n) { i(n, this); })), e(nt, (function (e) { return o(e), this.expression.walk(e), r(e), o(e), he(this, e), r(e), !0; })), e(kt, (function (e, t) { return oi(this, 16), o(e), t(), r(e), !0; })), e(yt, (function (e) { return this.condition.walk(e), o(e), this.consequent.walk(e), r(e), o(e), this.alternative.walk(e), r(e), !0; })), e(tt, (function (e, t) { return o(e), t(), r(e), !0; })), e(ke, (function (e, t, i) { return oi(this, 16), o(e), n(e, i, this), this.uses_arguments ? (t(), void r(e)) : (!this.name && (a = e.parent()) instanceof dt && a.expression === this && !a.args.some(e => e instanceof Ne) && this.argnames.every(e => e instanceof Pt) && this.argnames.forEach((t, n) => { if (t.definition) {
            var i = t.definition();
            i.orig.length > 1 || (void 0 !== i.fixed || this.uses_arguments && !e.has_directive("use strict") ? i.fixed = !1 : (i.fixed = function () { return a.args[n] || p(En, a); }, e.loop_ids.set(i.id, e.in_loop), s(e, i, !0)));
        } }), t(), r(e), !0); var a; })), e(be, (function (e, t, n) { i(n, this); const a = e.in_loop; return e.in_loop = this, o(e), this.body.walk(e), qi(this) && (r(e), o(e)), this.condition.walk(e), r(e), e.in_loop = a, !0; })), e(Oe, (function (e, t, n) { i(n, this), this.init && this.init.walk(e); const a = e.in_loop; return e.in_loop = this, o(e), this.condition && this.condition.walk(e), this.body.walk(e), this.step && (qi(this) && (r(e), o(e)), this.step.walk(e)), r(e), e.in_loop = a, !0; })), e(Fe, (function (e, t, n) { i(n, this), f(this.init), this.object.walk(e); const a = e.in_loop; return e.in_loop = this, o(e), this.body.walk(e), r(e), e.in_loop = a, !0; })), e(Je, (function (e) { return this.condition.walk(e), o(e), this.body.walk(e), r(e), this.alternative && (o(e), this.alternative.walk(e), r(e)), !0; })), e(Ae, (function (e) { return o(e), this.body.walk(e), r(e), !0; })), e(Jt, (function () { this.definition().fixed = !1; })), e(nn, (function (e, t, n) { var i, o, r = this.definition(); r.references.push(this), 1 == r.references.length && !r.fixed && r.orig[0] instanceof Wt && e.loop_ids.set(r.id, e.in_loop), void 0 !== r.fixed && u(e, r) ? r.fixed && ((i = this.fixed_value()) instanceof ke && Zi(e, r) ? r.recursive_refs++ : i && !n.exposed(r) && function (e, t, n) { return t.option("unused") && !n.scope.pinned() && n.references.length - n.recursive_refs == 1 && e.loop_ids.get(n.id) === e.in_loop; }(e, n, r) ? r.single_use = i instanceof ke && !i.pinned() || i instanceof kt || r.scope === this.scope && i.is_constant_expression() : r.single_use = !1, ui(n, e, this, i, 0, !!(o = i) && (o.is_constant() || o instanceof ke || o instanceof sn)) && (r.single_use ? r.single_use = "m" : r.fixed = !1)) : r.fixed = !1, l(e, r, this.scope, this, i, 0, 1); })), e(xe, (function (e, i, o) { this.globals.forEach((function (e) { t(o, e); })), n(e, o, this); })), e(it, (function (e, t, n) { return i(n, this), o(e), he(this, e), r(e), this.bcatch && (o(e), this.bcatch.walk(e), r(e)), this.bfinally && this.bfinally.walk(e), !0; })), e(St, (function (e) { var t = this; if ("++" === t.operator || "--" === t.operator) {
            var n = t.expression;
            if (n instanceof nn) {
                var i = n.definition(), o = c(e, i, n.scope, !0);
                if (i.assignments++, o) {
                    var r = i.fixed;
                    if (r)
                        return i.references.push(n), i.chained = !0, i.fixed = function () { return p(Tt, t, { operator: t.operator.slice(0, -1), left: p(vt, t, { operator: "+", expression: r instanceof fe ? r : r() }), right: p(fn, t, { value: 1 }) }); }, s(e, i, !0), !0;
                }
            }
        } })), e(lt, (function (e, t) { var n = this; if (n.name instanceof Be)
            f(n.name);
        else {
            var i = n.name.definition();
            if (n.value) {
                if (c(e, i, n.name.scope, n.value))
                    return i.fixed = function () { return n.value; }, e.loop_ids.set(i.id, e.in_loop), s(e, i, !1), t(), s(e, i, !0), !0;
                i.fixed = !1;
            }
        } })), e(Ce, (function (e, t, n) { i(n, this); const a = e.in_loop; return e.in_loop = this, o(e), t(), r(e), e.in_loop = a, !0; })); }((function (e, t) { e.DEFMETHOD("reduce_vars", t); })), xe.DEFMETHOD("reset_opt_flags", (function (e) { const t = this, n = e.option("reduce_vars"), i = new Cn((function (o, r) { if (oi(o, 1792), n)
            return e.top_retain && o instanceof Pe && i.parent() === t && ii(o, 1024), o.reduce_vars(i, r, e); })); i.safe_ids = Object.create(null), i.in_loop = null, i.loop_ids = new Map, i.defs_to_safe_ids = new Map, t.walk(i); })), Pt.DEFMETHOD("fixed_value", (function () { var e = this.thedef.fixed; return !e || e instanceof fe ? e : e(); })), nn.DEFMETHOD("is_immutable", (function () { var e = this.definition().orig; return 1 == e.length && e[0] instanceof $t; }));
        var yi = E("Array Boolean clearInterval clearTimeout console Date decodeURI decodeURIComponent encodeURI encodeURIComponent Error escape eval EvalError Function isFinite isNaN JSON Math Number parseFloat parseInt RangeError ReferenceError RegExp Object setInterval setTimeout String SyntaxError TypeError unescape URIError");
        nn.DEFMETHOD("is_declared", (function (e) { return !this.definition().undeclared || e.option("unsafe") && yi.has(this.name); }));
        var bi, Ci = E("Infinity NaN undefined");
        function Oi(e) { return e instanceof Dn || e instanceof hn || e instanceof En; }
        function Fi(e, t) { var n, o, r = t.find_parent(we).get_defun_scope(); !function () { var e = t.self(), i = 0; do {
            if (e instanceof ot || e instanceof rt)
                i++;
            else if (e instanceof Te)
                n = !0;
            else {
                if (e instanceof we) {
                    r = e;
                    break;
                }
                e instanceof it && (o = !0);
            }
        } while (e = t.parent(i++)); }(); var a, s = 10; do {
            a = !1, c(e), t.option("dead_code") && _(e, t), t.option("if_return") && l(e, t), t.sequences_limit > 0 && (h(e, t), g(e, t)), t.option("join_vars") && S(e), t.option("collapse_vars") && u(e, t);
        } while (a && s-- > 0); function u(e, t) { if (r.pinned())
            return e; for (var s, u = [], c = e.length, l = new On((function (e) { if (M)
            return e; if (!F)
            return e !== d[m] ? e : ++m < d.length ? I(e) : (F = !0, (g = function e(t, n, i) { var o = l.parent(n); if (o instanceof bt)
                return i && !(o.left instanceof Et || v.has(o.left.name)) ? e(o, n + 1, i) : t; if (o instanceof Tt)
                return !i || xi.has(o.operator) && o.left !== t ? t : e(o, n + 1, i); if (o instanceof dt)
                return t; if (o instanceof nt)
                return t; if (o instanceof yt)
                return i && o.condition === t ? e(o, n + 1, i) : t; if (o instanceof at)
                return e(o, n + 1, !0); if (o instanceof Xe)
                return i ? e(o, n + 1, i) : t; if (o instanceof Je)
                return i && o.condition === t ? e(o, n + 1, i) : t; if (o instanceof Te)
                return t; if (o instanceof ht)
                return e(o, n + 1, o.tail_node() !== t); if (o instanceof me)
                return e(o, n + 1, !0); return o instanceof Qe || o instanceof lt ? t : null; }(e, 0)) === e && (M = !0), e); var n, i = l.parent(); if (e instanceof bt && "=" != e.operator && S.equivalent_to(e.left) || e instanceof je || e instanceof dt && S instanceof Et && S.equivalent_to(e.expression) || e instanceof _e || e instanceof Be || e instanceof Ne && e.expression instanceof Pt && e.expression.definition().references.length > 1 || e instanceof Te && !(e instanceof Oe) || e instanceof Ye || e instanceof it || e instanceof Re || e instanceof Ze || e instanceof _t || e instanceof kt || i instanceof Oe && e !== i.init || !b && e instanceof nn && !e.is_declared(t) && !Hi.has(e) || e instanceof nn && i instanceof dt && T(i, Rn))
            return M = !0, e; if (D || A && b || !(i instanceof Tt && xi.has(i.operator) && i.left !== e || i instanceof yt && i.condition !== e || i instanceof Je && i.condition !== e) || (D = i), w && !(e instanceof Kt) && S.equivalent_to(e)) {
            if (D)
                return M = !0, e;
            if (ki(e, i))
                return E && R++, e;
            if (R++, E && h instanceof lt)
                return e;
            if (a = M = !0, t.info("Collapsing {name} [{file}:{line},{col}]", { name: e.print_to_string(), file: e.start.file, line: e.start.line, col: e.start.col }), h instanceof At)
                return p(vt, h, h);
            if (h instanceof lt) {
                var s = h.name.definition(), u = h.value;
                return s.references.length - s.replaced != 1 || t.exposed(s) ? p(bt, h, { operator: "=", left: p(nn, h.name, h.name), right: u }) : (s.replaced++, O && Oi(u) ? u.transform(t) : hi(i, e, u));
            }
            return oi(h, 32), h;
        } return (e instanceof dt || e instanceof Xe && (y || S instanceof Et || W(S)) || e instanceof Et && (y || e.expression.may_throw_on_access(t)) || e instanceof nn && (v.get(e.name) || y && W(e)) || e instanceof lt && e.value && (v.has(e.name.name) || y && W(e.name)) || (n = ki(e.left, e)) && (n instanceof Et || v.has(n.name)) || C && (o ? e.has_side_effects(t) : function e(t, n) { if (t instanceof bt)
            return e(t.left, !0); if (t instanceof St)
            return e(t.expression, !0); if (t instanceof lt)
            return t.value && e(t.value); if (n) {
            if (t instanceof gt)
                return e(t.expression, !0);
            if (t instanceof Dt)
                return e(t.expression, !0);
            if (t instanceof nn)
                return t.definition().scope !== r;
        } return !1; }(e))) && (g = e, e instanceof we && (M = !0)), I(e); }), (function (e) { M || (g === e && (M = !0), D === e && (D = null)); })), _ = new On((function (e) { if (M)
            return e; if (!F) {
            if (e !== d[m])
                return e;
            if (++m < d.length)
                return;
            return F = !0, e;
        } return e instanceof nn && e.name == k.name ? (--R || (M = !0), ki(e, _.parent()) ? e : (k.replaced++, E.replaced--, h.value)) : e instanceof tt || e instanceof we ? e : void 0; })); --c >= 0;) {
            0 == c && t.option("unused") && V();
            var d = [];
            for (P(e[c]); u.length > 0;) {
                d = u.pop();
                var m = 0, h = d[d.length - 1], E = null, g = null, D = null, S = B(h);
                if (S && !li(S) && !S.has_side_effects(t)) {
                    var v = U(h), A = H(S);
                    S instanceof nn && v.set(S.name, !1);
                    var y = X(h), b = z(), C = h.may_throw(t), O = h.name instanceof zt, F = O, M = !1, R = 0, w = !s || !F;
                    if (!w) {
                        for (var x = t.self().argnames.lastIndexOf(h.name) + 1; !M && x < s.length; x++)
                            s[x].transform(l);
                        w = !0;
                    }
                    for (var N = c; !M && N < e.length; N++)
                        e[N].transform(l);
                    if (E) {
                        var k = h.name.definition();
                        if (M && k.references.length - k.replaced > R)
                            R = !1;
                        else {
                            M = !1, m = 0, F = O;
                            for (N = c; !M && N < e.length; N++)
                                e[N].transform(_);
                            E.single_use = !1;
                        }
                    }
                    R && !G(h) && e.splice(c, 1);
                }
            }
        } function I(e) { if (e instanceof we)
            return e; if (e instanceof Qe) {
            e.expression = e.expression.transform(l);
            for (var t = 0, n = e.body.length; !M && t < n; t++) {
                var i = e.body[t];
                if (i instanceof nt) {
                    if (!F) {
                        if (i !== d[m])
                            continue;
                        m++;
                    }
                    if (i.expression = i.expression.transform(l), !b)
                        break;
                }
            }
            return M = !0, e;
        } } function L(e, t, n) { var i = !1, o = !(e instanceof Ve); return t.walk(new Cn((function (t, a) { if (i)
            return !0; if (t instanceof nn && (e.variables.has(t.name) || function (e, t) { if (e.global)
            return !1; let n = e.scope; for (; n && n !== t;) {
            if (n.variables.has(e.name))
                return !0;
            n = n.parent_scope;
        } return !1; }(t.definition(), e))) {
            var s = t.definition().scope;
            if (s !== r)
                for (; s = s.parent_scope;)
                    if (s === r)
                        return !0;
            return i = !0;
        } if ((n || o) && t instanceof sn)
            return i = !0; if (t instanceof we && !(t instanceof Ve)) {
            var u = o;
            return o = !1, a(), o = u, !0;
        } }))), i; } function V() { var e, n = t.self(); if (ci(n) && !n.name && !n.uses_arguments && !n.pinned() && (e = t.parent()) instanceof dt && e.expression === n && e.args.every(e => !(e instanceof Ne))) {
            var o = t.has_directive("use strict");
            o && !i(o, n.body) && (o = !1);
            var r = n.argnames.length;
            s = e.args.slice(r);
            for (var a = new Set, c = r; --c >= 0;) {
                var l = n.argnames[c], f = e.args[c];
                const i = l.definition && l.definition();
                if (!(i && i.orig.length > 1) && (s.unshift(p(lt, l, { name: l, value: f })), !a.has(l.name)))
                    if (a.add(l.name), l instanceof Ne) {
                        var _ = e.args.slice(c);
                        _.every(e => !L(n, e, o)) && u.unshift([p(lt, l, { name: l.expression, value: p(Ot, e, { elements: _ }) })]);
                    }
                    else
                        f ? (f instanceof ke && f.pinned() || L(n, f, o)) && (f = null) : f = p(En, l).transform(t), f && u.unshift([p(lt, l, { name: l, value: f })]);
            }
        } } function P(e) { if (d.push(e), e instanceof bt)
            e.left.has_side_effects(t) || u.push(d.slice()), P(e.right);
        else if (e instanceof Tt)
            P(e.left), P(e.right);
        else if (e instanceof dt && !T(e, Rn))
            P(e.expression), e.args.forEach(P);
        else if (e instanceof nt)
            P(e.expression);
        else if (e instanceof yt)
            P(e.condition), P(e.consequent), P(e.alternative);
        else if (!(e instanceof at) || !t.option("unused") && e instanceof ct)
            e instanceof ye ? (P(e.condition), e.body instanceof ge || P(e.body)) : e instanceof Xe ? e.value && P(e.value) : e instanceof Oe ? (e.init && P(e.init), e.condition && P(e.condition), e.step && P(e.step), e.body instanceof ge || P(e.body)) : e instanceof Fe ? (P(e.object), e.body instanceof ge || P(e.body)) : e instanceof Je ? (P(e.condition), e.body instanceof ge || P(e.body), !e.alternative || e.alternative instanceof ge || P(e.alternative)) : e instanceof ht ? e.expressions.forEach(P) : e instanceof me ? P(e.body) : e instanceof Qe ? (P(e.expression), e.body.forEach(P)) : e instanceof St ? "++" != e.operator && "--" != e.operator || u.push(d.slice()) : e instanceof lt && e.value && (u.push(d.slice()), P(e.value));
        else {
            var n = e.definitions.length, i = n - 200;
            for (i < 0 && (i = 0); i < n; i++)
                P(e.definitions[i]);
        } d.pop(); } function B(e) { if (!(e instanceof lt && e.name instanceof Kt)) {
            const t = e[e instanceof bt ? "left" : "expression"];
            return !fi(t, Ht) && !fi(t, Xt) && t;
        } var n = e.name.definition(); if (i(e.name, n.orig)) {
            var o = n.references.length - n.replaced;
            if (o)
                return n.orig.length - n.eliminated > 1 && !(e.name instanceof zt) || (o > 1 ? function (e) { var t = e.value; if (t instanceof nn && "arguments" != t.name) {
                    var n = t.definition();
                    if (!n.undeclared)
                        return E = n;
                } }(e) : !t.exposed(n)) ? p(nn, e.name, e.name) : void 0;
        } } function K(e) { return e[e instanceof bt ? "right" : "value"]; } function U(e) { var n = new Map; if (e instanceof St)
            return n; var i = new Cn((function (e) { for (var o = e; o instanceof Et;)
            o = o.expression; (o instanceof nn || o instanceof sn) && n.set(o.name, n.get(o.name) || ui(t, i, e, e, 0)); })); return K(e).walk(i), n; } function G(n) { if (n.name instanceof zt) {
            var i = t.parent(), o = t.self().argnames, r = o.indexOf(n.name);
            if (r < 0)
                i.args.length = Math.min(i.args.length, o.length - 1);
            else {
                var a = i.args;
                a[r] && (a[r] = p(fn, a[r], { value: 0 }));
            }
            return !0;
        } var s = !1; return e[c].transform(new On((function (e, t, i) { return s ? e : e === n || e.body === n ? (s = !0, e instanceof lt ? (e.value = null, e) : i ? f.skip : null) : void 0; }), (function (e) { if (e instanceof ht)
            switch (e.expressions.length) {
                case 0: return null;
                case 1: return e.expressions[0];
            } }))); } function H(e) { for (; e instanceof Et;)
            e = e.expression; return e instanceof nn && e.definition().scope === r && !(n && (v.has(e.name) || h instanceof St || h instanceof bt && "=" != h.operator)); } function X(e) { return e instanceof St ? Ni.has(e.operator) : K(e).has_side_effects(t); } function z() { if (y)
            return !1; if (E)
            return !0; if (S instanceof nn) {
            var e = S.definition();
            if (e.references.length - e.replaced == (h instanceof lt ? 1 : 2))
                return !0;
        } return !1; } function W(e) { if (!e.definition)
            return !0; var t = e.definition(); return !(1 == t.orig.length && t.orig[0] instanceof Wt) && (t.scope.get_defun_scope() !== r || !t.references.every(e => { var t = e.scope.get_defun_scope(); return "Scope" == t.TYPE && (t = t.parent_scope), t === r; })); } } function c(e) { for (var t = [], n = 0; n < e.length;) {
            var i = e[n];
            i instanceof De && i.body.every(Si) ? (a = !0, c(i.body), e.splice(n, 1, ...i.body), n += i.body.length) : i instanceof Se ? (a = !0, e.splice(n, 1)) : i instanceof de ? t.indexOf(i.value) < 0 ? (n++, t.push(i.value)) : (a = !0, e.splice(n, 1)) : n++;
        } } function l(e, t) { for (var n = t.self(), i = function (e) { for (var t = 0, n = e.length; --n >= 0;) {
            var i = e[n];
            if (i instanceof Je && i.body instanceof ze && ++t > 1)
                return !0;
        } return !1; }(e), o = n instanceof ke, r = e.length; --r >= 0;) {
            var s = e[r], u = S(r), c = e[u];
            if (o && !c && s instanceof ze) {
                if (!s.value) {
                    a = !0, e.splice(r, 1);
                    continue;
                }
                if (s.value instanceof vt && "void" == s.value.operator) {
                    a = !0, e[r] = p(me, s, { body: s.value.expression });
                    continue;
                }
            }
            if (s instanceof Je) {
                var l;
                if (E(l = Xi(s.body))) {
                    l.label && m(l.label.thedef.references, l), a = !0, (s = s.clone()).condition = s.condition.negate(t);
                    var f = D(s.body, l);
                    s.body = p(De, s, { body: gi(s.alternative).concat(g()) }), s.alternative = p(De, s, { body: f }), e[r] = s.transform(t);
                    continue;
                }
                if (E(l = Xi(s.alternative))) {
                    l.label && m(l.label.thedef.references, l), a = !0, (s = s.clone()).body = p(De, s.body, { body: gi(s.body).concat(g()) });
                    f = D(s.alternative, l);
                    s.alternative = p(De, s.alternative, { body: f }), e[r] = s.transform(t);
                    continue;
                }
            }
            if (s instanceof Je && s.body instanceof ze) {
                var _ = s.body.value;
                if (!_ && !s.alternative && (o && !c || c instanceof ze && !c.value)) {
                    a = !0, e[r] = p(me, s.condition, { body: s.condition });
                    continue;
                }
                if (_ && !s.alternative && c instanceof ze && c.value) {
                    a = !0, (s = s.clone()).alternative = c, e[r] = s.transform(t), e.splice(u, 1);
                    continue;
                }
                if (_ && !s.alternative && (!c && o && i || c instanceof ze)) {
                    a = !0, (s = s.clone()).alternative = c || p(ze, s, { value: null }), e[r] = s.transform(t), c && e.splice(u, 1);
                    continue;
                }
                var h = e[v(r)];
                if (t.option("sequences") && o && !s.alternative && h instanceof Je && h.body instanceof ze && S(u) == e.length && c instanceof me) {
                    a = !0, (s = s.clone()).alternative = p(De, c, { body: [c, p(ze, c, { value: null })] }), e[r] = s.transform(t), e.splice(u, 1);
                    continue;
                }
            }
        } function E(i) { if (!i)
            return !1; for (var a = r + 1, s = e.length; a < s; a++) {
            var u = e[a];
            if (u instanceof ct || u instanceof ut)
                return !1;
        } var c = i instanceof Ye ? t.loopcontrol_target(i) : null; return i instanceof ze && o && function (e) { return !e || e instanceof vt && "void" == e.operator; }(i.value) || i instanceof $e && n === vi(c) || i instanceof qe && c instanceof De && n === c; } function g() { var t = e.slice(r + 1); return e.length = r + 1, t.filter((function (t) { return !(t instanceof Pe) || (e.push(t), !1); })); } function D(e, t) { var n = gi(e).slice(0, -1); return t.value && n.push(p(me, t.value, { body: t.value.expression })), n; } function S(t) { for (var n = t + 1, i = e.length; n < i; n++) {
            var o = e[n];
            if (!(o instanceof st && d(o)))
                break;
        } return n; } function v(t) { for (var n = t; --n >= 0;) {
            var i = e[n];
            if (!(i instanceof st && d(i)))
                break;
        } return n; } } function _(e, t) { for (var n, i = t.self(), o = 0, r = 0, s = e.length; o < s; o++) {
            var u = e[o];
            if (u instanceof Ye) {
                var c = t.loopcontrol_target(u);
                u instanceof qe && !(c instanceof Te) && vi(c) === i || u instanceof $e && vi(c) === i ? u.label && m(u.label.thedef.references, u) : e[r++] = u;
            }
            else
                e[r++] = u;
            if (Xi(u)) {
                n = e.slice(o + 1);
                break;
            }
        } e.length = r, a = r != s, n && n.forEach((function (n) { Mi(t, n, e); })); } function d(e) { return e.definitions.every(e => !e.value); } function h(e, t) { if (!(e.length < 2)) {
            for (var n = [], i = 0, o = 0, r = e.length; o < r; o++) {
                var s = e[o];
                if (s instanceof me) {
                    n.length >= t.sequences_limit && c();
                    var u = s.body;
                    n.length > 0 && (u = u.drop_side_effect_free(t)), u && Ei(n, u);
                }
                else
                    s instanceof at && d(s) || s instanceof Pe || c(), e[i++] = s;
            }
            c(), e.length = i, i != r && (a = !0);
        } function c() { if (n.length) {
            var t = di(n[0], n);
            e[i++] = p(me, t, { body: t }), n = [];
        } } } function E(e, t) { if (!(e instanceof De))
            return e; for (var n = null, i = 0, o = e.body.length; i < o; i++) {
            var r = e.body[i];
            if (r instanceof st && d(r))
                t.push(r);
            else {
                if (n)
                    return !1;
                n = r;
            }
        } return n; } function g(e, t) { function n(e) { o--, a = !0; var n = i.body; return di(n, [n, e]).transform(t); } for (var i, o = 0, r = 0; r < e.length; r++) {
            var s = e[r];
            if (i)
                if (s instanceof Xe)
                    s.value = n(s.value || p(En, s).transform(t));
                else if (s instanceof Oe) {
                    if (!(s.init instanceof at)) {
                        Tn(i.body, e => e instanceof we || (e instanceof Tt && "in" === e.operator ? bn : void 0)) || (s.init ? s.init = n(s.init) : (s.init = i.body, o--, a = !0));
                    }
                }
                else
                    s instanceof Fe ? s.init instanceof ct || s.init instanceof ut || (s.object = n(s.object)) : s instanceof Je ? s.condition = n(s.condition) : (s instanceof Qe || s instanceof Re) && (s.expression = n(s.expression));
            if (t.option("conditionals") && s instanceof Je) {
                var u = [], c = E(s.body, u), l = E(s.alternative, u);
                if (!1 !== c && !1 !== l && u.length > 0) {
                    var f = u.length;
                    u.push(p(Je, s, { condition: s.condition, body: c || p(Se, s.body), alternative: l })), u.unshift(o, 1), [].splice.apply(e, u), r += f, o += f + 1, i = null, a = !0;
                    continue;
                }
            }
            e[o++] = s, i = s instanceof me ? s : null;
        } e.length = o; } function D(e, n) { if (e instanceof at) {
            var i, o = e.definitions[e.definitions.length - 1];
            if (o.value instanceof Ft)
                if (n instanceof bt ? i = [n] : n instanceof ht && (i = n.expressions.slice()), i) {
                    var a = !1;
                    do {
                        var s = i[0];
                        if (!(s instanceof bt))
                            break;
                        if ("=" != s.operator)
                            break;
                        if (!(s.left instanceof Et))
                            break;
                        var u = s.left.expression;
                        if (!(u instanceof nn))
                            break;
                        if (o.name.name != u.name)
                            break;
                        if (!s.right.is_constant_expression(r))
                            break;
                        var c = s.left.property;
                        if (c instanceof fe && (c = c.evaluate(t)), c instanceof fe)
                            break;
                        c = "" + c;
                        var l = t.option("ecma") < 2015 && t.has_directive("use strict") ? function (e) { return e.key != c && e.key && e.key.name != c; } : function (e) { return e.key && e.key.name != c; };
                        if (!o.value.properties.every(l))
                            break;
                        var f = o.value.properties.filter((function (e) { return e.key === c; }))[0];
                        f ? f.value = new ht({ start: f.start, expressions: [f.value.clone(), s.right.clone()], end: f.end }) : o.value.properties.push(p(Rt, s, { key: c, value: s.right })), i.shift(), a = !0;
                    } while (i.length);
                    return a && i;
                }
        } } function S(e) { for (var t, n = 0, i = -1, o = e.length; n < o; n++) {
            var r = e[n], s = e[i];
            if (r instanceof at)
                s && s.TYPE == r.TYPE ? (s.definitions = s.definitions.concat(r.definitions), a = !0) : t && t.TYPE == r.TYPE && d(r) ? (t.definitions = t.definitions.concat(r.definitions), a = !0) : (e[++i] = r, t = r);
            else if (r instanceof Xe)
                r.value = c(r.value);
            else if (r instanceof Oe) {
                (u = D(s, r.init)) ? (a = !0, r.init = u.length ? di(r.init, u) : null, e[++i] = r) : s instanceof st && (!r.init || r.init.TYPE == s.TYPE) ? (r.init && (s.definitions = s.definitions.concat(r.init.definitions)), r.init = s, e[i] = r, a = !0) : t && r.init && t.TYPE == r.init.TYPE && d(r.init) ? (t.definitions = t.definitions.concat(r.init.definitions), r.init = null, e[++i] = r, a = !0) : e[++i] = r;
            }
            else if (r instanceof Fe)
                r.object = c(r.object);
            else if (r instanceof Je)
                r.condition = c(r.condition);
            else if (r instanceof me) {
                var u;
                if (u = D(s, r.body)) {
                    if (a = !0, !u.length)
                        continue;
                    r.body = di(r.body, u);
                }
                e[++i] = r;
            }
            else
                r instanceof Qe || r instanceof Re ? r.expression = c(r.expression) : e[++i] = r;
        } function c(t) { e[++i] = r; var n = D(s, t); return n ? (a = !0, n.length ? di(t, n) : t instanceof ht ? t.tail_node().left : t.left) : t; } e.length = i + 1; } }
        function Mi(e, t, n) { t instanceof Pe || e.warn("Dropping unreachable code [{file}:{line},{col}]", t.start), Tn(t, i => i instanceof st ? (e.warn("Declarations in unreachable code! [{file}:{line},{col}]", i.start), i.remove_initializers(), n.push(i), !0) : i instanceof Pe && (i === t || !e.has_directive("use strict")) ? (n.push(i === t ? i : p(st, i, { definitions: [p(lt, i, { name: p(Ut, i.name, i.name), value: null })] })), !0) : i instanceof we || void 0); }
        function Ri(e) { return e instanceof cn ? e.getValue() : e instanceof vt && "void" == e.operator && e.expression instanceof cn ? void 0 : e; }
        function wi(e, t) { return ni(e, 8) || e instanceof En || e instanceof vt && "void" == e.operator && !e.expression.has_side_effects(t); }
        !function (e) { function t(e) { return /strict/.test(e.option("pure_getters")); } fe.DEFMETHOD("may_throw_on_access", (function (e) { return !e.option("pure_getters") || this._dot_throw(e); })), e(fe, t), e(mn, u), e(En, u), e(cn, s), e(Ot, s), e(Ft, (function (e) { if (!t(e))
            return !1; for (var n = this.properties.length; --n >= 0;)
            if (this.properties[n]._dot_throw(e))
                return !0; return !1; })), e(Mt, s), e(xt, u), e(Ne, (function (e) { return this.expression._dot_throw(e); })), e(Le, s), e(Ve, s), e(At, s), e(vt, (function () { return "void" == this.operator; })), e(Tt, (function (e) { return ("&&" == this.operator || "||" == this.operator) && (this.left._dot_throw(e) || this.right._dot_throw(e)); })), e(bt, (function (e) { return "=" == this.operator && this.right._dot_throw(e); })), e(yt, (function (e) { return this.consequent._dot_throw(e) || this.alternative._dot_throw(e); })), e(gt, (function (e) { return !!t(e) && !(this.expression instanceof Le && "prototype" == this.property); })), e(ht, (function (e) { return this.tail_node()._dot_throw(e); })), e(nn, (function (e) { if (ni(this, 8))
            return !0; if (!t(e))
            return !1; if (Ti(this) && this.is_declared(e))
            return !1; if (this.is_immutable())
            return !1; var n = this.fixed_value(); return !n || n._dot_throw(e); })); }((function (e, t) { e.DEFMETHOD("_dot_throw", t); })), function (e) { const t = E("! delete"), n = E("in instanceof == != === !== < <= >= >"); e(fe, s), e(vt, (function () { return t.has(this.operator); })), e(Tt, (function () { return n.has(this.operator) || xi.has(this.operator) && this.left.is_boolean() && this.right.is_boolean(); })), e(yt, (function () { return this.consequent.is_boolean() && this.alternative.is_boolean(); })), e(bt, (function () { return "=" == this.operator && this.right.is_boolean(); })), e(ht, (function () { return this.tail_node().is_boolean(); })), e(An, u), e(vn, u); }((function (e, t) { e.DEFMETHOD("is_boolean", t); })), function (e) { e(fe, s), e(fn, u); var t = E("+ - ~ ++ --"); e(St, (function () { return t.has(this.operator); })); var n = E("- * / % & | ^ << >> >>>"); e(Tt, (function (e) { return n.has(this.operator) || "+" == this.operator && this.left.is_number(e) && this.right.is_number(e); })), e(bt, (function (e) { return n.has(this.operator.slice(0, -1)) || "=" == this.operator && this.right.is_number(e); })), e(ht, (function (e) { return this.tail_node().is_number(e); })), e(yt, (function (e) { return this.consequent.is_number(e) && this.alternative.is_number(e); })); }((function (e, t) { e.DEFMETHOD("is_number", t); })), (bi = function (e, t) { e.DEFMETHOD("is_string", t); })(fe, s), bi(ln, u), bi(Ue, (function () { return 1 === this.segments.length; })), bi(vt, (function () { return "typeof" == this.operator; })), bi(Tt, (function (e) { return "+" == this.operator && (this.left.is_string(e) || this.right.is_string(e)); })), bi(bt, (function (e) { return ("=" == this.operator || "+=" == this.operator) && this.right.is_string(e); })), bi(ht, (function (e) { return this.tail_node().is_string(e); })), bi(yt, (function (e) { return this.consequent.is_string(e) && this.alternative.is_string(e); }));
        var xi = E("&& ||"), Ni = E("delete ++ --");
        function ki(e, t) { return t instanceof St && Ni.has(t.operator) ? t.expression : t instanceof bt && t.left === e ? e : void 0; }
        function Ii(e, t) { return e.size() > t.size() ? t : e; }
        function Li(e, t) { return Ii(p(me, e, { body: e }), p(me, t, { body: t })).body; }
        function Vi(e, t, n) { return (kn(e) ? Li : Ii)(t, n); }
        function Pi(e) { const t = new Map; for (var n of Object.keys(e))
            t.set(n, E(e[n])); return t; }
        !function (e) { function t(e, t) { e.warn("global_defs " + t.print_to_string() + " redefined [{file}:{line},{col}]", t.start); } xe.DEFMETHOD("resolve_defines", (function (e) { return e.option("global_defs") ? (this.figure_out_scope({ ie8: e.option("ie8") }), this.transform(new On((function (n) { var i = n._find_defs(e, ""); if (i) {
            for (var o, r = 0, a = n; (o = this.parent(r++)) && o instanceof Et && o.expression === a;)
                a = o;
            if (!ki(a, o))
                return i;
            t(e, n);
        } })))) : this; })), e(fe, a), e(gt, (function (e, t) { return this.expression._find_defs(e, "." + this.property + t); })), e(Kt, (function (e) { this.global() && D(e.option("global_defs"), this.name) && t(e, this); })), e(nn, (function (e, t) { if (this.global()) {
            var n = e.option("global_defs"), i = this.name + t;
            return D(n, i) ? function e(t, n) { if (t instanceof fe)
                return p(t.CTOR, n, t); if (Array.isArray(t))
                return p(Ot, n, { elements: t.map((function (t) { return e(t, n); })) }); if (t && "object" == typeof t) {
                var i = [];
                for (var o in t)
                    D(t, o) && i.push(p(Rt, n, { key: o, value: e(t[o], n) }));
                return p(Ft, n, { properties: i });
            } return mi(t, n); }(n[i], this) : void 0;
        } })); }((function (e, t) { e.DEFMETHOD("_find_defs", t); }));
        var Bi = ["constructor", "toString", "valueOf"], Ki = Pi({ Array: ["indexOf", "join", "lastIndexOf", "slice"].concat(Bi), Boolean: Bi, Function: Bi, Number: ["toExponential", "toFixed", "toPrecision"].concat(Bi), Object: Bi, RegExp: ["test"].concat(Bi), String: ["charAt", "charCodeAt", "concat", "indexOf", "italics", "lastIndexOf", "match", "replace", "search", "slice", "split", "substr", "substring", "toLowerCase", "toUpperCase", "trim"].concat(Bi) }), Ui = Pi({ Array: ["isArray"], Math: ["abs", "acos", "asin", "atan", "ceil", "cos", "exp", "floor", "log", "round", "sin", "sqrt", "tan", "atan2", "pow", "max", "min"], Number: ["isFinite", "isNaN"], Object: ["create", "getOwnPropertyDescriptor", "getOwnPropertyNames", "getPrototypeOf", "isExtensible", "isFrozen", "isSealed", "keys"], String: ["fromCharCode"] });
        !function (e) { fe.DEFMETHOD("evaluate", (function (e) { if (!e.option("evaluate"))
            return this; var t = this._eval(e, 1); return !t || t instanceof RegExp ? t : "function" == typeof t || "object" == typeof t ? this : t; })); var t = E("! ~ - + void"); fe.DEFMETHOD("is_constant", (function () { return this instanceof cn ? !(this instanceof _n) : this instanceof vt && this.expression instanceof cn && t.has(this.operator); })), e(pe, (function () { throw new Error(d("Cannot evaluate a statement [{file}:{line},{col}]", this.start)); })), e(ke, c), e(kt, c), e(fe, c), e(cn, (function () { return this.getValue(); })), e(pn, c), e(_n, (function (e) { let t = e.evaluated_regexps.get(this); if (void 0 === t) {
            try {
                t = (0, eval)(this.print_to_string());
            }
            catch (e) {
                t = null;
            }
            e.evaluated_regexps.set(this, t);
        } return t || this; })), e(Ue, (function () { return 1 !== this.segments.length ? this : this.segments[0].value; })), e(Le, (function (e) { if (e.option("unsafe")) {
            var t = function () { };
            return t.node = this, t.toString = function () { return this.node.print_to_string(); }, t;
        } return this; })), e(Ot, (function (e, t) { if (e.option("unsafe")) {
            for (var n = [], i = 0, o = this.elements.length; i < o; i++) {
                var r = this.elements[i], a = r._eval(e, t);
                if (r === a)
                    return this;
                n.push(a);
            }
            return n;
        } return this; })), e(Ft, (function (e, t) { if (e.option("unsafe")) {
            for (var n = {}, i = 0, o = this.properties.length; i < o; i++) {
                var r = this.properties[i];
                if (r instanceof Ne)
                    return this;
                var a = r.key;
                if (a instanceof Pt)
                    a = a.name;
                else if (a instanceof fe && (a = a._eval(e, t)) === r.key)
                    return this;
                if ("function" == typeof Object.prototype[a])
                    return this;
                if (!(r.value instanceof Le) && (n[a] = r.value._eval(e, t), n[a] === r.value))
                    return this;
            }
            return n;
        } return this; })); var n = E("! typeof void"); e(vt, (function (e, t) { var i = this.expression; if (e.option("typeofs") && "typeof" == this.operator && (i instanceof ke || i instanceof nn && i.fixed_value() instanceof ke))
            return "function"; if (n.has(this.operator) || t++, (i = i._eval(e, t)) === this.expression)
            return this; switch (this.operator) {
            case "!": return !i;
            case "typeof": return i instanceof RegExp ? this : typeof i;
            case "void": return;
            case "~": return ~i;
            case "-": return -i;
            case "+": return +i;
        } return this; })); var i = E("&& || === !=="); e(Tt, (function (e, t) { i.has(this.operator) || t++; var n = this.left._eval(e, t); if (n === this.left)
            return this; var o, r = this.right._eval(e, t); if (r === this.right)
            return this; switch (this.operator) {
            case "&&":
                o = n && r;
                break;
            case "||":
                o = n || r;
                break;
            case "|":
                o = n | r;
                break;
            case "&":
                o = n & r;
                break;
            case "^":
                o = n ^ r;
                break;
            case "+":
                o = n + r;
                break;
            case "*":
                o = n * r;
                break;
            case "**":
                o = Math.pow(n, r);
                break;
            case "/":
                o = n / r;
                break;
            case "%":
                o = n % r;
                break;
            case "-":
                o = n - r;
                break;
            case "<<":
                o = n << r;
                break;
            case ">>":
                o = n >> r;
                break;
            case ">>>":
                o = n >>> r;
                break;
            case "==":
                o = n == r;
                break;
            case "===":
                o = n === r;
                break;
            case "!=":
                o = n != r;
                break;
            case "!==":
                o = n !== r;
                break;
            case "<":
                o = n < r;
                break;
            case "<=":
                o = n <= r;
                break;
            case ">":
                o = n > r;
                break;
            case ">=":
                o = n >= r;
                break;
            default: return this;
        } return isNaN(o) && e.find_parent(Re) ? this : o; })), e(yt, (function (e, t) { var n = this.condition._eval(e, t); if (n === this.condition)
            return this; var i = n ? this.consequent : this.alternative, o = i._eval(e, t); return o === i ? this : o; })), e(nn, (function (e, t) { var n, i = this.fixed_value(); if (!i)
            return this; if (D(i, "_eval"))
            n = i._eval();
        else {
            if (this._eval = c, n = i._eval(e, t), delete this._eval, n === i)
                return this;
            i._eval = function () { return n; };
        } if (n && "object" == typeof n) {
            var o = this.definition().escaped;
            if (o && t > o)
                return this;
        } return n; })); var o = { Array: Array, Math: Math, Number: Number, Object: Object, String: String }, r = Pi({ Math: ["E", "LN10", "LN2", "LOG2E", "LOG10E", "PI", "SQRT1_2", "SQRT2"], Number: ["MAX_VALUE", "MIN_VALUE", "NaN", "NEGATIVE_INFINITY", "POSITIVE_INFINITY"] }); e(Et, (function (e, t) { if (e.option("unsafe")) {
            var n = this.property;
            if (n instanceof fe && (n = n._eval(e, t)) === this.property)
                return this;
            var i, a = this.expression;
            if (Ti(a)) {
                var s, u = "hasOwnProperty" === a.name && "call" === n && (s = e.parent() && e.parent().args) && s && s[0] && s[0].evaluate(e);
                if (null == (u = u instanceof gt ? u.expression : u) || u.thedef && u.thedef.undeclared)
                    return this.clone();
                var c = r.get(a.name);
                if (!c || !c.has(n))
                    return this;
                i = o[a.name];
            }
            else {
                if (!(i = a._eval(e, t + 1)) || i === a || !D(i, n))
                    return this;
                if ("function" == typeof i)
                    switch (n) {
                        case "name": return i.node.name ? i.node.name.name : "";
                        case "length": return i.node.argnames.length;
                        default: return this;
                    }
            }
            return i[n];
        } return this; })), e(dt, (function (e, t) { var n = this.expression; if (e.option("unsafe") && n instanceof Et) {
            var i, r = n.property;
            if (r instanceof fe && (r = r._eval(e, t)) === n.property)
                return this;
            var a = n.expression;
            if (Ti(a)) {
                var s = "hasOwnProperty" === a.name && "call" === r && this.args[0] && this.args[0].evaluate(e);
                if (null == (s = s instanceof gt ? s.expression : s) || s.thedef && s.thedef.undeclared)
                    return this.clone();
                var u = Ui.get(a.name);
                if (!u || !u.has(r))
                    return this;
                i = o[a.name];
            }
            else {
                if ((i = a._eval(e, t + 1)) === a || !i)
                    return this;
                var c = Ki.get(i.constructor.name);
                if (!c || !c.has(r))
                    return this;
            }
            for (var l = [], f = 0, p = this.args.length; f < p; f++) {
                var _ = this.args[f], d = _._eval(e, t);
                if (_ === d)
                    return this;
                l.push(d);
            }
            try {
                return i[r].apply(i, l);
            }
            catch (t) {
                e.warn("Error evaluating {code} [{file}:{line},{col}]", { code: this.print_to_string(), file: this.start.file, line: this.start.line, col: this.start.col });
            }
        } return this; })), e(mt, c); }((function (e, t) { e.DEFMETHOD("_eval", t); })), function (e) { function t(e) { return p(vt, e, { operator: "!", expression: e }); } function n(e, n, i) { var o = t(e); if (i) {
            var r = p(me, n, { body: n });
            return Ii(o, r) === r ? n : o;
        } return Ii(o, n); } e(fe, (function () { return t(this); })), e(pe, (function () { throw new Error("Cannot negate a statement"); })), e(Le, (function () { return t(this); })), e(Ve, (function () { return t(this); })), e(vt, (function () { return "!" == this.operator ? this.expression : t(this); })), e(ht, (function (e) { var t = this.expressions.slice(); return t.push(t.pop().negate(e)), di(this, t); })), e(yt, (function (e, t) { var i = this.clone(); return i.consequent = i.consequent.negate(e), i.alternative = i.alternative.negate(e), n(this, i, t); })), e(Tt, (function (e, i) { var o = this.clone(), r = this.operator; if (e.option("unsafe_comps"))
            switch (r) {
                case "<=": return o.operator = ">", o;
                case "<": return o.operator = ">=", o;
                case ">=": return o.operator = "<", o;
                case ">": return o.operator = "<=", o;
            } switch (r) {
            case "==": return o.operator = "!=", o;
            case "!=": return o.operator = "==", o;
            case "===": return o.operator = "!==", o;
            case "!==": return o.operator = "===", o;
            case "&&": return o.operator = "||", o.left = o.left.negate(e, i), o.right = o.right.negate(e), n(this, o, i);
            case "||": return o.operator = "&&", o.left = o.left.negate(e, i), o.right = o.right.negate(e), n(this, o, i);
        } return t(this); })); }((function (e, t) { e.DEFMETHOD("negate", (function (e, n) { return t.call(this, e, n); })); }));
        var Gi = E("Boolean decodeURI decodeURIComponent Date encodeURI encodeURIComponent Error escape EvalError isFinite isNaN Number Object parseFloat parseInt RangeError ReferenceError String SyntaxError TypeError unescape URIError");
        dt.DEFMETHOD("is_expr_pure", (function (e) { if (e.option("unsafe")) {
            var t = this.expression, n = this.args && this.args[0] && this.args[0].evaluate(e);
            if (t.expression && "hasOwnProperty" === t.expression.name && (null == n || n.thedef && n.thedef.undeclared))
                return !1;
            if (Ti(t) && Gi.has(t.name))
                return !0;
            let i;
            if (t instanceof gt && Ti(t.expression) && (i = Ui.get(t.expression.name)) && i.has(t.property))
                return !0;
        } return !!T(this, Fn) || !e.pure_funcs(this); })), fe.DEFMETHOD("is_call_pure", s), gt.DEFMETHOD("is_call_pure", (function (e) { if (!e.option("unsafe"))
            return; const t = this.expression; let n; return t instanceof Ot ? n = Ki.get("Array") : t.is_boolean() ? n = Ki.get("Boolean") : t.is_number(e) ? n = Ki.get("Number") : t instanceof _n ? n = Ki.get("RegExp") : t.is_string(e) ? n = Ki.get("String") : this.may_throw_on_access(e) || (n = Ki.get("Object")), n && n.has(this.property); }));
        const Hi = new Set(["Number", "String", "Array", "Object", "Function", "Promise"]);
        function Xi(e) { return e && e.aborts(); }
        !function (e) { function t(e, t) { for (var n = e.length; --n >= 0;)
            if (e[n].has_side_effects(t))
                return !0; return !1; } e(fe, u), e(Se, s), e(cn, s), e(sn, s), e(ge, (function (e) { return t(this.body, e); })), e(dt, (function (e) { return !(this.is_expr_pure(e) || this.expression.is_call_pure(e) && !this.expression.has_side_effects(e)) || t(this.args, e); })), e(Qe, (function (e) { return this.expression.has_side_effects(e) || t(this.body, e); })), e(nt, (function (e) { return this.expression.has_side_effects(e) || t(this.body, e); })), e(it, (function (e) { return t(this.body, e) || this.bcatch && this.bcatch.has_side_effects(e) || this.bfinally && this.bfinally.has_side_effects(e); })), e(Je, (function (e) { return this.condition.has_side_effects(e) || this.body && this.body.has_side_effects(e) || this.alternative && this.alternative.has_side_effects(e); })), e(Ae, (function (e) { return this.body.has_side_effects(e); })), e(me, (function (e) { return this.body.has_side_effects(e); })), e(ke, s), e(kt, (function (e) { return !(!this.extends || !this.extends.has_side_effects(e)) || t(this.properties, e); })), e(Tt, (function (e) { return this.left.has_side_effects(e) || this.right.has_side_effects(e); })), e(bt, u), e(yt, (function (e) { return this.condition.has_side_effects(e) || this.consequent.has_side_effects(e) || this.alternative.has_side_effects(e); })), e(St, (function (e) { return Ni.has(this.operator) || this.expression.has_side_effects(e); })), e(nn, (function (e) { return !this.is_declared(e) && !Hi.has(this.name); })), e(qt, s), e(Kt, s), e(Ft, (function (e) { return t(this.properties, e); })), e(Mt, (function (e) { return this.computed_key() && this.key.has_side_effects(e) || this.value.has_side_effects(e); })), e(It, (function (e) { return this.computed_key() && this.key.has_side_effects(e) || this.static && this.value && this.value.has_side_effects(e); })), e(Nt, (function (e) { return this.computed_key() && this.key.has_side_effects(e); })), e(xt, (function (e) { return this.computed_key() && this.key.has_side_effects(e); })), e(wt, (function (e) { return this.computed_key() && this.key.has_side_effects(e); })), e(Ot, (function (e) { return t(this.elements, e); })), e(gt, (function (e) { return this.expression.may_throw_on_access(e) || this.expression.has_side_effects(e); })), e(Dt, (function (e) { return this.expression.may_throw_on_access(e) || this.expression.has_side_effects(e) || this.property.has_side_effects(e); })), e(ht, (function (e) { return t(this.expressions, e); })), e(at, (function (e) { return t(this.definitions, e); })), e(lt, (function () { return this.value; })), e(Ge, s), e(Ue, (function (e) { return t(this.segments, e); })); }((function (e, t) { e.DEFMETHOD("has_side_effects", t); })), function (e) { function t(e, t) { for (var n = e.length; --n >= 0;)
            if (e[n].may_throw(t))
                return !0; return !1; } e(fe, u), e(cn, s), e(Se, s), e(ke, s), e(Kt, s), e(sn, s), e(kt, (function (e) { return !(!this.extends || !this.extends.may_throw(e)) || t(this.properties, e); })), e(Ot, (function (e) { return t(this.elements, e); })), e(bt, (function (e) { return !!this.right.may_throw(e) || !(!e.has_directive("use strict") && "=" == this.operator && this.left instanceof nn) && this.left.may_throw(e); })), e(Tt, (function (e) { return this.left.may_throw(e) || this.right.may_throw(e); })), e(ge, (function (e) { return t(this.body, e); })), e(dt, (function (e) { return !!t(this.args, e) || !this.is_expr_pure(e) && (!!this.expression.may_throw(e) || (!(this.expression instanceof ke) || t(this.expression.body, e))); })), e(nt, (function (e) { return this.expression.may_throw(e) || t(this.body, e); })), e(yt, (function (e) { return this.condition.may_throw(e) || this.consequent.may_throw(e) || this.alternative.may_throw(e); })), e(at, (function (e) { return t(this.definitions, e); })), e(gt, (function (e) { return this.expression.may_throw_on_access(e) || this.expression.may_throw(e); })), e(Je, (function (e) { return this.condition.may_throw(e) || this.body && this.body.may_throw(e) || this.alternative && this.alternative.may_throw(e); })), e(Ae, (function (e) { return this.body.may_throw(e); })), e(Ft, (function (e) { return t(this.properties, e); })), e(Mt, (function (e) { return this.value.may_throw(e); })), e(It, (function (e) { return this.computed_key() && this.key.may_throw(e) || this.static && this.value && this.value.may_throw(e); })), e(Nt, (function (e) { return this.computed_key() && this.key.may_throw(e); })), e(xt, (function (e) { return this.computed_key() && this.key.may_throw(e); })), e(wt, (function (e) { return this.computed_key() && this.key.may_throw(e); })), e(ze, (function (e) { return this.value && this.value.may_throw(e); })), e(ht, (function (e) { return t(this.expressions, e); })), e(me, (function (e) { return this.body.may_throw(e); })), e(Dt, (function (e) { return this.expression.may_throw_on_access(e) || this.expression.may_throw(e) || this.property.may_throw(e); })), e(Qe, (function (e) { return this.expression.may_throw(e) || t(this.body, e); })), e(nn, (function (e) { return !this.is_declared(e) && !Hi.has(this.name); })), e(qt, s), e(it, (function (e) { return this.bcatch ? this.bcatch.may_throw(e) : t(this.body, e) || this.bfinally && this.bfinally.may_throw(e); })), e(St, (function (e) { return !("typeof" == this.operator && this.expression instanceof nn) && this.expression.may_throw(e); })), e(lt, (function (e) { return !!this.value && this.value.may_throw(e); })); }((function (e, t) { e.DEFMETHOD("may_throw", t); })), function (e) { function t(e) { let t = !0; return Tn(this, n => { if (n instanceof nn) {
            if (ni(this, 16))
                return t = !1, bn;
            var o = n.definition();
            if (i(o, this.enclosed) && !this.variables.has(o.name)) {
                if (e) {
                    var r = e.find_variable(n);
                    if (o.undeclared ? !r : r === o)
                        return t = "f", !0;
                }
                return t = !1, bn;
            }
            return !0;
        } if (n instanceof sn && this instanceof Ve)
            return t = !1, bn; }), t; } e(fe, s), e(cn, u), e(kt, (function (e) { return !(this.extends && !this.extends.is_constant_expression(e)) && t.call(this, e); })), e(ke, t), e(St, (function () { return this.expression.is_constant_expression(); })), e(Tt, (function () { return this.left.is_constant_expression() && this.right.is_constant_expression(); })), e(Ot, (function () { return this.elements.every(e => e.is_constant_expression()); })), e(Ft, (function () { return this.properties.every(e => e.is_constant_expression()); })), e(Mt, (function () { return !(this.key instanceof fe) && this.value.is_constant_expression(); })); }((function (e, t) { e.DEFMETHOD("is_constant_expression", t); })), function (e) { function t() { for (var e = 0; e < this.body.length; e++)
            if (Xi(this.body[e]))
                return this.body[e]; return null; } e(pe, l), e(He, c), e(pt, (function () { return null; })), e(De, t), e(et, t), e(Je, (function () { return this.alternative && Xi(this.body) && Xi(this.alternative) && this; })); }((function (e, t) { e.DEFMETHOD("aborts", t); }));
        var zi = new Set(["use asm", "use strict"]);
        function Wi(e, t) { return Fi(e.body, t), t.option("side_effects") && 1 == e.body.length && e.body[0] === t.has_directive("use strict") && (e.body.length = 0), e; }
        ai(de, (function (e, t) { return !t.option("directives") || zi.has(e.value) && t.has_directive(e.value) === e ? e : p(Se, e); })), ai(_e, (function (e, t) { return t.option("drop_debugger") ? p(Se, e) : e; })), ai(Ae, (function (e, t) { return e.body instanceof qe && t.loopcontrol_target(e.body) === e.body ? p(Se, e) : 0 == e.label.references.length ? e.body : e; })), ai(ge, (function (e, t) { return Fi(e.body, t), e; })), ai(De, (function (e, t) { switch (Fi(e.body, t), e.body.length) {
            case 1:
                if (!t.has_directive("use strict") && t.parent() instanceof Je && !((n = e.body[0]) instanceof ct || n instanceof ut || n instanceof kt) || Si(e.body[0]))
                    return e.body[0];
                break;
            case 0: return p(Se, e);
        } var n; return e; })), ai(ke, Wi);
        const Yi = /keep_assign/;
        function qi(e, t) { var n = !1, i = new Cn((function (t) { return !!(n || t instanceof we) || (t instanceof Ye && i.loopcontrol_target(t) === e ? n = !0 : void 0); })); return t instanceof Ae && i.push(t), i.push(e), e.body.walk(i), n; }
        function $i(e, t) { return t.top_retain && e instanceof Pe && ni(e, 1024) && e.name && t.top_retain(e.name); }
        we.DEFMETHOD("drop_unused", (function (e) { if (!e.option("unused"))
            return; if (e.has_directive("use asm"))
            return; var t = this; if (t.pinned())
            return; var n = !(t instanceof xe) || e.toplevel.funcs, i = !(t instanceof xe) || e.toplevel.vars; const o = Yi.test(e.option("unused")) ? s : function (e) { return e instanceof bt && (ni(e, 32) || "=" == e.operator) ? e.left : e instanceof St && ni(e, 32) ? e.expression : void 0; }; var r = new Map, a = new Map; t instanceof xe && e.top_retain && t.variables.forEach((function (t) { e.top_retain(t) && !r.has(t.id) && r.set(t.id, t); })); var u = new Map, c = new Map, l = this, _ = new Cn((function (o, s) { if (o instanceof ke && o.uses_arguments && !_.has_directive("use strict") && o.argnames.forEach((function (e) { if (e instanceof Kt) {
            var t = e.definition();
            r.has(t.id) || r.set(t.id, t);
        } })), o !== t) {
            if (o instanceof Pe || o instanceof Lt) {
                var f = o.name.definition();
                return (_.parent() instanceof _t || !n && l === t) && f.global && !r.has(f.id) && r.set(f.id, f), o instanceof Lt && o.extends && (o.extends.has_side_effects(e) || o.extends.may_throw(e)) && o.extends.walk(_), g(c, f.id, o), !0;
            }
            if (o instanceof zt && l === t && g(u, o.definition().id, o), o instanceof at && l === t) {
                const t = _.parent() instanceof _t;
                return o.definitions.forEach((function (n) { if (n.name instanceof Ut && g(u, n.name.definition().id, n), !t && i || Tn(n.name, e => { if (e instanceof Kt) {
                    const n = e.definition();
                    !t && !n.global || r.has(n.id) || r.set(n.id, n);
                } }), n.value) {
                    if (n.name instanceof Be)
                        n.walk(_);
                    else {
                        var o = n.name.definition();
                        g(c, o.id, n.value), o.chained || n.name.fixed_value() !== n.value || a.set(o.id, n);
                    }
                    n.value.has_side_effects(e) && n.value.walk(_);
                } })), !0;
            }
            return h(o, s);
        } })); t.walk(_), _ = new Cn(h), r.forEach((function (e) { var t = c.get(e.id); t && t.forEach((function (e) { e.walk(_); })); })); var d = new On((function (s, c, _) { var h = d.parent(); if (i) {
            const e = o(s);
            if (e instanceof nn) {
                var E = e.definition(), g = r.has(E.id);
                if (s instanceof bt) {
                    if (!g || a.has(E.id) && a.get(E.id) !== s)
                        return hi(h, s, s.right.transform(d));
                }
                else if (!g)
                    return _ ? f.skip : p(fn, s, { value: 0 });
            }
        } if (l === t) {
            if (s.name && (s instanceof Vt && !S(e.option("keep_classnames"), (E = s.name.definition()).name) || s instanceof Le && !S(e.option("keep_fnames"), (E = s.name.definition()).name)) && (!r.has(E.id) || E.orig.length > 1) && (s.name = null), s instanceof ke && !(s instanceof Ie))
                for (var D = !e.option("keep_fargs"), v = s.argnames, A = v.length; --A >= 0;) {
                    var T = v[A];
                    T instanceof Ne && (T = T.expression), T instanceof Ct && (T = T.left), T instanceof Be || r.has(T.definition().id) ? D = !1 : (ii(T, 1), D && (v.pop(), e[T.unreferenced() ? "warn" : "info"]("Dropping unused function argument {name} [{file}:{line},{col}]", R(T))));
                }
            if ((s instanceof Pe || s instanceof Lt) && s !== t) {
                const t = s.name.definition();
                if (!(t.global && !n || r.has(t.id))) {
                    if (e[s.name.unreferenced() ? "warn" : "info"]("Dropping unused function {name} [{file}:{line},{col}]", R(s.name)), t.eliminated++, s instanceof Lt) {
                        const t = s.drop_side_effect_free(e);
                        if (t)
                            return p(me, s, { body: t });
                    }
                    return _ ? f.skip : p(Se, s);
                }
            }
            if (s instanceof at && !(h instanceof Fe && h.init === s)) {
                var y = !(h instanceof xe || s instanceof st), b = [], C = [], O = [], F = [];
                switch (s.definitions.forEach((function (t) { t.value && (t.value = t.value.transform(d)); var n = t.name instanceof Be, o = n ? new Xn(null, { name: "<destructure>" }) : t.name.definition(); if (y && o.global)
                    return O.push(t); if (!i && !y || n && (t.name.names.length || t.name.is_array || 1 != e.option("pure_getters")) || r.has(o.id)) {
                    if (t.value && a.has(o.id) && a.get(o.id) !== t && (t.value = t.value.drop_side_effect_free(e)), t.name instanceof Ut) {
                        var c = u.get(o.id);
                        if (c.length > 1 && (!t.value || o.orig.indexOf(t.name) > o.eliminated)) {
                            if (e.warn("Dropping duplicated definition of variable {name} [{file}:{line},{col}]", R(t.name)), t.value) {
                                var l = p(nn, t.name, t.name);
                                o.references.push(l);
                                var f = p(bt, t, { operator: "=", left: l, right: t.value });
                                a.get(o.id) === t && a.set(o.id, f), F.push(f.transform(d));
                            }
                            return m(c, t), void o.eliminated++;
                        }
                    }
                    t.value ? (F.length > 0 && (O.length > 0 ? (F.push(t.value), t.value = di(t.value, F)) : b.push(p(me, s, { body: di(s, F) })), F = []), O.push(t)) : C.push(t);
                }
                else if (o.orig[0] instanceof Jt) {
                    (_ = t.value && t.value.drop_side_effect_free(e)) && F.push(_), t.value = null, C.push(t);
                }
                else {
                    var _;
                    (_ = t.value && t.value.drop_side_effect_free(e)) ? (n || e.warn("Side effects in initialization of unused variable {name} [{file}:{line},{col}]", R(t.name)), F.push(_)) : n || e[t.name.unreferenced() ? "warn" : "info"]("Dropping unused variable {name} [{file}:{line},{col}]", R(t.name)), o.eliminated++;
                } })), (C.length > 0 || O.length > 0) && (s.definitions = C.concat(O), b.push(s)), F.length > 0 && b.push(p(me, s, { body: di(s, F) })), b.length) {
                    case 0: return _ ? f.skip : p(Se, s);
                    case 1: return b[0];
                    default: return _ ? f.splice(b) : p(De, s, { body: b });
                }
            }
            if (s instanceof Oe)
                return c(s, this), s.init instanceof De && (M = s.init, s.init = M.body.pop(), M.body.push(s)), s.init instanceof me ? s.init = s.init.body : Di(s.init) && (s.init = null), M ? _ ? f.splice(M.body) : M : s;
            if (s instanceof Ae && s.body instanceof Oe) {
                if (c(s, this), s.body instanceof De) {
                    var M = s.body;
                    return s.body = M.body.pop(), M.body.push(s), _ ? f.splice(M.body) : M;
                }
                return s;
            }
            if (s instanceof De)
                return c(s, this), _ && s.body.every(Si) ? f.splice(s.body) : s;
            if (s instanceof we) {
                const e = l;
                return l = s, c(s, this), l = e, s;
            }
        } function R(e) { return { name: e.name, file: e.start.file, line: e.start.line, col: e.start.col }; } })); function h(e, n) { var i; const s = o(e); if (s instanceof nn && !fi(e.left, Gt) && t.variables.get(s.name) === (i = s.definition()))
            return e instanceof bt && (e.right.walk(_), i.chained || e.left.fixed_value() !== e.right || a.set(i.id, e)), !0; if (e instanceof nn) {
            if (i = e.definition(), !r.has(i.id) && (r.set(i.id, i), i.orig[0] instanceof Jt)) {
                const e = i.scope.is_block_scope() && i.scope.get_defun_scope().variables.get(i.name);
                e && r.set(e.id, e);
            }
            return !0;
        } if (e instanceof we) {
            var u = l;
            return l = e, n(), l = u, !0;
        } } t.transform(d); })), we.DEFMETHOD("hoist_declarations", (function (e) { var t = this; if (e.has_directive("use asm"))
            return t; if (!Array.isArray(t.body))
            return t; var n = e.option("hoist_funs"), i = e.option("hoist_vars"); if (n || i) {
            var o = [], r = [], a = new Map, s = 0, u = 0;
            Tn(t, e => e instanceof we && e !== t || (e instanceof st ? (++u, !0) : void 0)), i = i && u > 1;
            var c = new On((function (u) { if (u !== t) {
                if (u instanceof de)
                    return o.push(u), p(Se, u);
                if (n && u instanceof Pe && !(c.parent() instanceof _t) && c.parent() === t)
                    return r.push(u), p(Se, u);
                if (i && u instanceof st) {
                    u.definitions.forEach((function (e) { e.name instanceof Be || (a.set(e.name.name, e), ++s); }));
                    var l = u.to_assignments(e), f = c.parent();
                    if (f instanceof Fe && f.init === u) {
                        if (null == l) {
                            var _ = u.definitions[0].name;
                            return p(nn, _, _);
                        }
                        return l;
                    }
                    return f instanceof Oe && f.init === u ? l : l ? p(me, u, { body: l }) : p(Se, u);
                }
                if (u instanceof we)
                    return u;
            } }));
            if (t = t.transform(c), s > 0) {
                var l = [];
                const e = t instanceof ke, n = e ? t.args_as_names() : null;
                if (a.forEach((t, i) => { e && n.some(e => e.name === t.name.name) ? a.delete(i) : ((t = t.clone()).value = null, l.push(t), a.set(i, t)); }), l.length > 0) {
                    for (var f = 0; f < t.body.length;) {
                        if (t.body[f] instanceof me) {
                            var _, d, h = t.body[f].body;
                            if (h instanceof bt && "=" == h.operator && (_ = h.left) instanceof Pt && a.has(_.name)) {
                                if ((E = a.get(_.name)).value)
                                    break;
                                E.value = h.right, m(l, E), l.push(E), t.body.splice(f, 1);
                                continue;
                            }
                            if (h instanceof ht && (d = h.expressions[0]) instanceof bt && "=" == d.operator && (_ = d.left) instanceof Pt && a.has(_.name)) {
                                var E;
                                if ((E = a.get(_.name)).value)
                                    break;
                                E.value = d.right, m(l, E), l.push(E), t.body[f].body = di(h, h.expressions.slice(1));
                                continue;
                            }
                        }
                        if (t.body[f] instanceof Se)
                            t.body.splice(f, 1);
                        else {
                            if (!(t.body[f] instanceof De))
                                break;
                            var g = [f, 1].concat(t.body[f].body);
                            t.body.splice.apply(t.body, g);
                        }
                    }
                    l = p(st, t, { definitions: l }), r.push(l);
                }
            }
            t.body = o.concat(r, t.body);
        } return t; })), we.DEFMETHOD("make_var_name", (function (e) { for (var t = this.var_names(), n = e = e.replace(/(?:^[^a-z_$]|[^a-z0-9_$])/gi, "_"), i = 0; t.has(n); i++)
            n = e + "$" + i; return this.add_var_name(n), n; })), we.DEFMETHOD("hoist_properties", (function (e) { var t = this; if (!e.option("hoist_props") || e.has_directive("use asm"))
            return t; var n = t instanceof xe && e.top_retain || s, i = new Map, o = new On((function (r, a) { if (r instanceof at && o.parent() instanceof _t)
            return r; if (r instanceof lt) {
            const o = r.name;
            let u, c;
            if (o.scope === t && 1 != (u = o.definition()).escaped && !u.assignments && !u.direct_access && !u.single_use && !e.exposed(u) && !n(u) && (c = o.fixed_value()) === r.value && c instanceof Ft && c.properties.every(e => "string" == typeof e.key)) {
                a(r, this);
                const e = new Map, t = [];
                return c.properties.forEach((function (n) { t.push(p(lt, r, { name: s(o, n.key, e), value: n.value })); })), i.set(u.id, e), f.splice(t);
            }
        }
        else if (r instanceof Et && r.expression instanceof nn) {
            const e = i.get(r.expression.definition().id);
            if (e) {
                const t = e.get(String(Ri(r.property))), n = p(nn, r, { name: t.name, scope: r.expression.scope, thedef: t });
                return n.reference({}), n;
            }
        } function s(e, n, i) { const o = p(e.CTOR, e, { name: t.make_var_name(e.name + "_" + n), scope: t }), r = t.def_variable(o); return i.set(String(n), r), t.enclosed.push(r), o; } })); return t.transform(o); })), function (e) { function t(e, t, n) { var i = e.length; if (!i)
            return null; for (var o = [], r = !1, a = 0; a < i; a++) {
            var s = e[a].drop_side_effect_free(t, n);
            r |= s !== e[a], s && (o.push(s), n = !1);
        } return r ? o.length ? o : null : e; } e(fe, c), e(cn, l), e(sn, l), e(dt, (function (e, n) { if (!this.is_expr_pure(e)) {
            if (this.expression.is_call_pure(e)) {
                var i = this.args.slice();
                return i.unshift(this.expression.expression), (i = t(i, e, n)) && di(this, i);
            }
            if (ci(this.expression) && (!this.expression.name || !this.expression.name.definition().references.length)) {
                var o = this.clone();
                return o.expression.process_expression(!1, e), o;
            }
            return this;
        } T(this, Fn) && e.warn("Dropping __PURE__ call [{file}:{line},{col}]", this.start); var r = t(this.args, e, n); return r && di(this, r); })), e(Ie, l), e(Le, l), e(Ve, l), e(kt, (function (e) { const t = [], n = this.extends && this.extends.drop_side_effect_free(e); n && t.push(n); for (const n of this.properties) {
            const i = n.drop_side_effect_free(e);
            i && t.push(i);
        } return t.length ? di(this, t) : null; })), e(Tt, (function (e, t) { var n = this.right.drop_side_effect_free(e); if (!n)
            return this.left.drop_side_effect_free(e, t); if (xi.has(this.operator)) {
            if (n === this.right)
                return this;
            var i = this.clone();
            return i.right = n, i;
        } var o = this.left.drop_side_effect_free(e, t); return o ? di(this, [o, n]) : this.right.drop_side_effect_free(e, t); })), e(bt, (function (e) { var t = this.left; if (t.has_side_effects(e) || e.has_directive("use strict") && t instanceof Et && t.expression.is_constant())
            return this; for (ii(this, 32); t instanceof Et;)
            t = t.expression; return t.is_constant_expression(e.find_parent(we)) ? this.right.drop_side_effect_free(e) : this; })), e(yt, (function (e) { var t = this.consequent.drop_side_effect_free(e), n = this.alternative.drop_side_effect_free(e); if (t === this.consequent && n === this.alternative)
            return this; if (!t)
            return n ? p(Tt, this, { operator: "||", left: this.condition, right: n }) : this.condition.drop_side_effect_free(e); if (!n)
            return p(Tt, this, { operator: "&&", left: this.condition, right: t }); var i = this.clone(); return i.consequent = t, i.alternative = n, i; })), e(St, (function (e, t) { if (Ni.has(this.operator))
            return this.expression.has_side_effects(e) ? oi(this, 32) : ii(this, 32), this; if ("typeof" == this.operator && this.expression instanceof nn)
            return null; var n = this.expression.drop_side_effect_free(e, t); return t && n && Ai(n) ? n === this.expression && "!" == this.operator ? this : n.negate(e, t) : n; })), e(nn, (function (e) { return this.is_declared(e) || Hi.has(this.name) ? null : this; })), e(Ft, (function (e, n) { var i = t(this.properties, e, n); return i && di(this, i); })), e(Mt, (function (e, t) { const n = this instanceof Rt && this.key instanceof fe && this.key.drop_side_effect_free(e, t), i = this.value.drop_side_effect_free(e, t); return n && i ? di(this, [n, i]) : n || i; })), e(It, (function (e) { const t = this.computed_key() && this.key.drop_side_effect_free(e), n = this.static && this.value && this.value.drop_side_effect_free(e); return t && n ? di(this, [t, n]) : t || n || null; })), e(Nt, (function () { return this.computed_key() ? this.key : null; })), e(xt, (function () { return this.computed_key() ? this.key : null; })), e(wt, (function () { return this.computed_key() ? this.key : null; })), e(Ot, (function (e, n) { var i = t(this.elements, e, n); return i && di(this, i); })), e(gt, (function (e, t) { return this.expression.may_throw_on_access(e) ? this : this.expression.drop_side_effect_free(e, t); })), e(Dt, (function (e, t) { if (this.expression.may_throw_on_access(e))
            return this; var n = this.expression.drop_side_effect_free(e, t); if (!n)
            return this.property.drop_side_effect_free(e, t); var i = this.property.drop_side_effect_free(e); return i ? di(this, [n, i]) : n; })), e(ht, (function (e) { var t = this.tail_node(), n = t.drop_side_effect_free(e); if (n === t)
            return this; var i = this.expressions.slice(0, -1); return n && i.push(n), i.length ? di(this, i) : p(fn, this, { value: 0 }); })), e(Ne, (function (e, t) { return this.expression.drop_side_effect_free(e, t); })), e(Ge, l), e(Ue, (function (e) { var n = t(this.segments, e, kn); return n && di(this, n); })); }((function (e, t) { e.DEFMETHOD("drop_side_effect_free", t); })), ai(me, (function (e, t) { if (t.option("side_effects")) {
            var n = e.body, i = n.drop_side_effect_free(t, !0);
            if (!i)
                return t.warn("Dropping side-effect-free statement [{file}:{line},{col}]", e.start), p(Se, e);
            if (i !== n)
                return p(me, e, { body: i });
        } return e; })), ai(Ce, (function (e, t) { return t.option("loops") ? p(Oe, e, e).optimize(t) : e; })), ai(be, (function (e, t) { if (!t.option("loops"))
            return e; var n = e.condition.tail_node().evaluate(t); if (!(n instanceof fe)) {
            if (n)
                return p(Oe, e, { body: p(De, e.body, { body: [e.body, p(me, e.condition, { body: e.condition })] }) }).optimize(t);
            if (!qi(e, t.parent()))
                return p(De, e.body, { body: [e.body, p(me, e.condition, { body: e.condition })] }).optimize(t);
        } return e; })), ai(Oe, (function (e, t) { if (!t.option("loops"))
            return e; if (t.option("side_effects") && e.init && (e.init = e.init.drop_side_effect_free(t)), e.condition) {
            var n = e.condition.evaluate(t);
            if (!(n instanceof fe))
                if (n)
                    e.condition = null;
                else if (!t.option("dead_code")) {
                    var i = e.condition;
                    e.condition = mi(n, e.condition), e.condition = Ii(e.condition.transform(t), i);
                }
            if (t.option("dead_code") && (n instanceof fe && (n = e.condition.tail_node().evaluate(t)), !n)) {
                var o = [];
                return Mi(t, e.body, o), e.init instanceof pe ? o.push(e.init) : e.init && o.push(p(me, e.init, { body: e.init })), o.push(p(me, e.condition, { body: e.condition })), p(De, e, { body: o }).optimize(t);
            }
        } return function e(t, n) { var i = t.body instanceof De ? t.body.body[0] : t.body; if (n.option("dead_code") && r(i)) {
            var o = [];
            return t.init instanceof pe ? o.push(t.init) : t.init && o.push(p(me, t.init, { body: t.init })), t.condition && o.push(p(me, t.condition, { body: t.condition })), Mi(n, t.body, o), p(De, t, { body: o });
        } return i instanceof Je && (r(i.body) ? (t.condition ? t.condition = p(Tt, t.condition, { left: t.condition, operator: "&&", right: i.condition.negate(n) }) : t.condition = i.condition.negate(n), a(i.alternative)) : r(i.alternative) && (t.condition ? t.condition = p(Tt, t.condition, { left: t.condition, operator: "&&", right: i.condition }) : t.condition = i.condition, a(i.body))), t; function r(e) { return e instanceof qe && n.loopcontrol_target(e) === n.self(); } function a(i) { i = gi(i), t.body instanceof De ? (t.body = t.body.clone(), t.body.body = i.concat(t.body.body.slice(1)), t.body = t.body.transform(n)) : t.body = p(De, t.body, { body: i }).transform(n), t = e(t, n); } }(e, t); })), ai(Je, (function (e, t) { if (Di(e.alternative) && (e.alternative = null), !t.option("conditionals"))
            return e; var n = e.condition.evaluate(t); if (!(t.option("dead_code") || n instanceof fe)) {
            var i = e.condition;
            e.condition = mi(n, i), e.condition = Ii(e.condition.transform(t), i);
        } if (t.option("dead_code")) {
            if (n instanceof fe && (n = e.condition.tail_node().evaluate(t)), !n) {
                t.warn("Condition always false [{file}:{line},{col}]", e.condition.start);
                var o = [];
                return Mi(t, e.body, o), o.push(p(me, e.condition, { body: e.condition })), e.alternative && o.push(e.alternative), p(De, e, { body: o }).optimize(t);
            }
            if (!(n instanceof fe))
                return t.warn("Condition always true [{file}:{line},{col}]", e.condition.start), (o = []).push(p(me, e.condition, { body: e.condition })), o.push(e.body), e.alternative && Mi(t, e.alternative, o), p(De, e, { body: o }).optimize(t);
        } var r = e.condition.negate(t), a = e.condition.size(), s = r.size(), u = s < a; if (e.alternative && u) {
            u = !1, e.condition = r;
            var c = e.body;
            e.body = e.alternative || p(Se, e), e.alternative = c;
        } if (Di(e.body) && Di(e.alternative))
            return p(me, e.condition, { body: e.condition.clone() }).optimize(t); if (e.body instanceof me && e.alternative instanceof me)
            return p(me, e, { body: p(yt, e, { condition: e.condition, consequent: e.body.body, alternative: e.alternative.body }) }).optimize(t); if (Di(e.alternative) && e.body instanceof me)
            return a === s && !u && e.condition instanceof Tt && "||" == e.condition.operator && (u = !0), u ? p(me, e, { body: p(Tt, e, { operator: "||", left: r, right: e.body.body }) }).optimize(t) : p(me, e, { body: p(Tt, e, { operator: "&&", left: e.condition, right: e.body.body }) }).optimize(t); if (e.body instanceof Se && e.alternative instanceof me)
            return p(me, e, { body: p(Tt, e, { operator: "||", left: e.condition, right: e.alternative.body }) }).optimize(t); if (e.body instanceof Xe && e.alternative instanceof Xe && e.body.TYPE == e.alternative.TYPE)
            return p(e.body.CTOR, e, { value: p(yt, e, { condition: e.condition, consequent: e.body.value || p(En, e.body), alternative: e.alternative.value || p(En, e.alternative) }).transform(t) }).optimize(t); if (e.body instanceof Je && !e.body.alternative && !e.alternative && (e = p(Je, e, { condition: p(Tt, e.condition, { operator: "&&", left: e.condition, right: e.body.condition }), body: e.body.body, alternative: null })), Xi(e.body) && e.alternative) {
            var l = e.alternative;
            return e.alternative = null, p(De, e, { body: [e, l] }).optimize(t);
        } if (Xi(e.alternative)) {
            o = e.body;
            return e.body = e.alternative, e.condition = u ? r : e.condition.negate(t), e.alternative = null, p(De, e, { body: [e, o] }).optimize(t);
        } return e; })), ai(Qe, (function (e, t) { if (!t.option("switches"))
            return e; var n, i = e.expression.evaluate(t); if (!(i instanceof fe)) {
            var o = e.expression;
            e.expression = mi(i, o), e.expression = Ii(e.expression.transform(t), o);
        } if (!t.option("dead_code"))
            return e; i instanceof fe && (i = e.expression.tail_node().evaluate(t)); for (var r, a, s = [], u = [], c = 0, l = e.body.length; c < l && !a; c++) {
            if ((n = e.body[c]) instanceof tt)
                r ? D(n, u[u.length - 1]) : r = n;
            else if (!(i instanceof fe)) {
                if (!((E = n.expression.evaluate(t)) instanceof fe) && E !== i) {
                    D(n, u[u.length - 1]);
                    continue;
                }
                if (E instanceof fe && (E = n.expression.tail_node().evaluate(t)), E === i && (a = n, r)) {
                    var f = u.indexOf(r);
                    u.splice(f, 1), D(r, u[f - 1]), r = null;
                }
            }
            if (Xi(n)) {
                var _ = u[u.length - 1];
                Xi(_) && _.body.length == n.body.length && p(De, _, _).equivalent_to(p(De, n, n)) && (_.body = []);
            }
            u.push(n);
        } for (; c < l;)
            D(e.body[c++], u[u.length - 1]); for (u.length > 0 && (u[0].body = s.concat(u[0].body)), e.body = u; n = u[u.length - 1];) {
            var d = n.body[n.body.length - 1];
            if (d instanceof qe && t.loopcontrol_target(d) === e && n.body.pop(), n.body.length || n instanceof nt && (r || n.expression.has_side_effects(t)))
                break;
            u.pop() === r && (r = null);
        } if (0 == u.length)
            return p(De, e, { body: s.concat(p(me, e.expression, { body: e.expression })) }).optimize(t); if (1 == u.length && (u[0] === a || u[0] === r)) {
            var m = !1, h = new Cn((function (t) { if (m || t instanceof ke || t instanceof me)
                return !0; t instanceof qe && h.loopcontrol_target(t) === e && (m = !0); }));
            if (e.walk(h), !m) {
                var E, g = u[0].body.slice();
                return (E = u[0].expression) && g.unshift(p(me, E, { body: E })), g.unshift(p(me, e.expression, { body: e.expression })), p(De, e, { body: g }).optimize(t);
            }
        } return e; function D(e, n) { n && !Xi(n) ? n.body = n.body.concat(e.body) : Mi(t, e, s); } })), ai(it, (function (e, t) { if (Fi(e.body, t), e.bcatch && e.bfinally && e.bfinally.body.every(Di) && (e.bfinally = null), t.option("dead_code") && e.body.every(Di)) {
            var n = [];
            return e.bcatch && Mi(t, e.bcatch, n), e.bfinally && n.push(...e.bfinally.body), p(De, e, { body: n }).optimize(t);
        } return e; })), at.DEFMETHOD("remove_initializers", (function () { var e = []; this.definitions.forEach((function (t) { t.name instanceof Kt ? (t.value = null, e.push(t)) : Tn(t.name, n => { n instanceof Kt && e.push(p(lt, t, { name: n, value: null })); }); })), this.definitions = e; })), at.DEFMETHOD("to_assignments", (function (e) { var t = e.option("reduce_vars"), n = this.definitions.reduce((function (e, n) { if (!n.value || n.name instanceof Be) {
            if (n.value) {
                var i = p(lt, n, { name: n.name, value: n.value }), o = p(st, n, { definitions: [i] });
                e.push(o);
            }
        }
        else {
            var r = p(nn, n.name, n.name);
            e.push(p(bt, n, { operator: "=", left: r, right: n.value })), t && (r.definition().fixed = !1);
        } return (n = n.name.definition()).eliminated++, n.replaced--, e; }), []); return 0 == n.length ? null : di(this, n); })), ai(at, (function (e) { return 0 == e.definitions.length ? p(Se, e) : e; })), ai(pt, (function (e) { return e; })), ai(dt, (function (e, t) { var n = e.expression, i = n; ro(e, t, e.args); var o = e.args.every(e => !(e instanceof Ne)); if (t.option("reduce_vars") && i instanceof nn && !T(e, Rn)) {
            const e = i.fixed_value();
            $i(e, t) || (i = e);
        } var r = i instanceof ke; if (t.option("unused") && o && r && !i.uses_arguments && !i.pinned()) {
            for (var a = 0, s = 0, u = 0, c = e.args.length; u < c; u++) {
                if (i.argnames[u] instanceof Ne) {
                    if (ni(i.argnames[u].expression, 1))
                        for (; u < c;) {
                            (D = e.args[u++].drop_side_effect_free(t)) && (e.args[a++] = D);
                        }
                    else
                        for (; u < c;)
                            e.args[a++] = e.args[u++];
                    s = a;
                    break;
                }
                var l = u >= i.argnames.length;
                if (l || ni(i.argnames[u], 1)) {
                    if (D = e.args[u].drop_side_effect_free(t))
                        e.args[a++] = D;
                    else if (!l) {
                        e.args[a++] = p(fn, e.args[u], { value: 0 });
                        continue;
                    }
                }
                else
                    e.args[a++] = e.args[u];
                s = a;
            }
            e.args.length = s;
        } if (t.option("unsafe"))
            if (Ti(n))
                switch (n.name) {
                    case "Array":
                        if (1 != e.args.length)
                            return p(Ot, e, { elements: e.args }).optimize(t);
                        if (e.args[0] instanceof fn && e.args[0].value <= 11) {
                            const t = [];
                            for (let n = 0; n < e.args[0].value; n++)
                                t.push(new gn);
                            return new Ot({ elements: t });
                        }
                        break;
                    case "Object":
                        if (0 == e.args.length)
                            return p(Ft, e, { properties: [] });
                        break;
                    case "String":
                        if (0 == e.args.length)
                            return p(ln, e, { value: "" });
                        if (e.args.length <= 1)
                            return p(Tt, e, { left: e.args[0], operator: "+", right: p(ln, e, { value: "" }) }).optimize(t);
                        break;
                    case "Number":
                        if (0 == e.args.length)
                            return p(fn, e, { value: 0 });
                        if (1 == e.args.length && t.option("unsafe_math"))
                            return p(vt, e, { expression: e.args[0], operator: "+" }).optimize(t);
                        break;
                    case "Symbol":
                        1 == e.args.length && e.args[0] instanceof ln && t.option("unsafe_symbols") && (e.args.length = 0);
                        break;
                    case "Boolean":
                        if (0 == e.args.length)
                            return p(vn, e);
                        if (1 == e.args.length)
                            return p(vt, e, { expression: p(vt, e, { expression: e.args[0], operator: "!" }), operator: "!" }).optimize(t);
                        break;
                    case "RegExp":
                        var f = [];
                        if (e.args.length >= 1 && e.args.length <= 2 && e.args.every(e => { var n = e.evaluate(t); return f.push(n), e !== n; })) {
                            let [n, i] = f;
                            n = A(new RegExp(n).source);
                            const o = p(_n, e, { value: { source: n, flags: i } });
                            if (o._eval(t) !== o)
                                return o;
                            t.warn("Error converting {expr} [{file}:{line},{col}]", { expr: e.print_to_string(), file: e.start.file, line: e.start.line, col: e.start.col });
                        }
                }
            else if (n instanceof gt)
                switch (n.property) {
                    case "toString":
                        if (0 == e.args.length && !n.expression.may_throw_on_access(t))
                            return p(Tt, e, { left: p(ln, e, { value: "" }), operator: "+", right: n.expression }).optimize(t);
                        break;
                    case "join":
                        if (n.expression instanceof Ot)
                            e: {
                                var _;
                                if (!(e.args.length > 0 && (_ = e.args[0].evaluate(t)) === e.args[0])) {
                                    var d, m = [], h = [];
                                    for (u = 0, c = n.expression.elements.length; u < c; u++) {
                                        var E = n.expression.elements[u];
                                        if (E instanceof Ne)
                                            break e;
                                        var g = E.evaluate(t);
                                        g !== E ? h.push(g) : (h.length > 0 && (m.push(p(ln, e, { value: h.join(_) })), h.length = 0), m.push(E));
                                    }
                                    return h.length > 0 && m.push(p(ln, e, { value: h.join(_) })), 0 == m.length ? p(ln, e, { value: "" }) : 1 == m.length ? m[0].is_string(t) ? m[0] : p(Tt, m[0], { operator: "+", left: p(ln, e, { value: "" }), right: m[0] }) : "" == _ ? (d = m[0].is_string(t) || m[1].is_string(t) ? m.shift() : p(ln, e, { value: "" }), m.reduce((function (e, t) { return p(Tt, t, { operator: "+", left: e, right: t }); }), d).optimize(t)) : ((D = e.clone()).expression = D.expression.clone(), D.expression.expression = D.expression.expression.clone(), D.expression.expression.elements = m, Vi(t, e, D));
                                    var D;
                                }
                            }
                        break;
                    case "charAt":
                        if (n.expression.is_string(t)) {
                            var S = e.args[0], v = S ? S.evaluate(t) : 0;
                            if (v !== S)
                                return p(Dt, n, { expression: n.expression, property: mi(0 | v, S || n) }).optimize(t);
                        }
                        break;
                    case "apply":
                        if (2 == e.args.length && e.args[1] instanceof Ot)
                            return (L = e.args[1].elements.slice()).unshift(e.args[0]), p(dt, e, { expression: p(gt, n, { expression: n.expression, property: "call" }), args: L }).optimize(t);
                        break;
                    case "call":
                        var y = n.expression;
                        if (y instanceof nn && (y = y.fixed_value()), y instanceof ke && !y.contains_this())
                            return (e.args.length ? di(this, [e.args[0], p(dt, e, { expression: n.expression, args: e.args.slice(1) })]) : p(dt, e, { expression: n.expression, args: [] })).optimize(t);
                } if (t.option("unsafe_Function") && Ti(n) && "Function" == n.name) {
            if (0 == e.args.length)
                return p(Le, e, { argnames: [], body: [] }).optimize(t);
            if (e.args.every(e => e instanceof ln))
                try {
                    var b = ue(M = "n(function(" + e.args.slice(0, -1).map((function (e) { return e.value; })).join(",") + "){" + e.args[e.args.length - 1].value + "})"), C = { ie8: t.option("ie8") };
                    b.figure_out_scope(C);
                    var O, F = new ri(t.options);
                    (b = b.transform(F)).figure_out_scope(C), Yn.reset(), b.compute_char_frequency(C), b.mangle_names(C), Tn(b, e => { if (ci(e))
                        return O = e, bn; });
                    var M = Pn();
                    return De.prototype._codegen.call(O, O, M), e.args = [p(ln, e, { value: O.argnames.map((function (e) { return e.print_to_string(); })).join(",") }), p(ln, e.args[e.args.length - 1], { value: M.get().replace(/^{|}$/g, "") })], e;
                }
                catch (n) {
                    if (!(n instanceof J))
                        throw n;
                    t.warn("Error parsing code passed to new Function [{file}:{line},{col}]", e.args[e.args.length - 1].start), t.warn(n.toString());
                }
        } var R = r && i.body[0], w = r && !i.is_generator && !i.async, x = w && t.option("inline") && !e.is_expr_pure(t); if (x && R instanceof ze) {
            let n = R.value;
            if (!n || n.is_constant_expression()) {
                n = n ? n.clone(!0) : p(En, e);
                const i = e.args.concat(n);
                return di(e, i).optimize(t);
            }
            if (1 === i.argnames.length && i.argnames[0] instanceof zt && e.args.length < 2 && n instanceof nn && n.name === i.argnames[0].name)
                return (e.args[0] || p(En)).optimize(t);
        } if (x) {
            var N, k, I = -1;
            let r, a;
            if (o && !i.uses_arguments && !i.pinned() && !(t.parent() instanceof kt) && !(i.name && i instanceof Le) && (a = function (e) { var n = i.body, o = n.length; if (t.option("inline") < 3)
                return 1 == o && P(e); e = null; for (var r = 0; r < o; r++) {
                var a = n[r];
                if (a instanceof st) {
                    if (e && !a.definitions.every(e => !e.value))
                        return !1;
                }
                else {
                    if (e)
                        return !1;
                    a instanceof Se || (e = a);
                }
            } return P(e); }(R)) && (n === i || T(e, Mn) || t.option("unused") && 1 == (r = n.definition()).references.length && !Zi(t, r) && i.is_constant_expression(n.scope)) && !T(e, Fn | Rn) && !i.contains_this() && function () { var n = new Set; do {
                if ((N = t.parent(++I)).is_block_scope() && N.block_scope && N.block_scope.variables.forEach((function (e) { n.add(e.name); })), N instanceof ot)
                    N.argname && n.add(N.argname.name);
                else if (N instanceof Te)
                    k = [];
                else if (N instanceof nn && N.fixed_value() instanceof we)
                    return !1;
            } while (!(N instanceof we)); var o = !(N instanceof xe) || t.toplevel.vars, r = t.option("inline"); return !!function (e, t) { for (var n = i.body.length, o = 0; o < n; o++) {
                var r = i.body[o];
                if (r instanceof st) {
                    if (!t)
                        return !1;
                    for (var a = r.definitions.length; --a >= 0;) {
                        var s = r.definitions[a].name;
                        if (s instanceof Be || e.has(s.name) || Ci.has(s.name) || N.var_names().has(s.name))
                            return !1;
                        k && k.push(s.definition());
                    }
                }
            } return !0; }(n, r >= 3 && o) && (!!function (e, t) { for (var n = 0, o = i.argnames.length; n < o; n++) {
                var r = i.argnames[n];
                if (r instanceof Ct) {
                    if (ni(r.left, 1))
                        continue;
                    return !1;
                }
                if (r instanceof Be)
                    return !1;
                if (r instanceof Ne) {
                    if (ni(r.expression, 1))
                        continue;
                    return !1;
                }
                if (!ni(r, 1)) {
                    if (!t || e.has(r.name) || Ci.has(r.name) || N.var_names().has(r.name))
                        return !1;
                    k && k.push(r.definition());
                }
            } return !0; }(n, r >= 2 && o) && (!!function () { var t = new Set; const n = e => { if (e instanceof we) {
                var n = new Set;
                return e.enclosed.forEach((function (e) { n.add(e.name); })), e.variables.forEach((function (e) { n.delete(e); })), n.forEach((function (e) { t.add(e); })), !0;
            } }; for (let t = 0; t < e.args.length; t++)
                Tn(e.args[t], n); if (0 == t.size)
                return !0; for (let e = 0, n = i.argnames.length; e < n; e++) {
                var o = i.argnames[e];
                if (!(o instanceof Ct && ni(o.left, 1)) && (!(o instanceof Ne && ni(o.expression, 1)) && !ni(o, 1) && t.has(o.name)))
                    return !1;
            } for (let e = 0, n = i.body.length; e < n; e++) {
                var r = i.body[e];
                if (r instanceof st)
                    for (var a = r.definitions.length; --a >= 0;) {
                        var s = r.definitions[a].name;
                        if (s instanceof Be || t.has(s.name))
                            return !1;
                    }
            } return !0; }() && (!k || 0 == k.length || !eo(i, k)))); }() && !Ji(N, i) && !(N instanceof kt))
                return ii(i, 256), pi(t, !0).add_child_scope(i), di(e, function (n) { var o = [], r = []; if (function (t, n) { for (var o = i.argnames.length, r = e.args.length; --r >= o;)
                    n.push(e.args[r]); for (r = o; --r >= 0;) {
                    var a = i.argnames[r], s = e.args[r];
                    if (ni(a, 1) || !a.name || N.var_names().has(a.name))
                        s && n.push(s);
                    else {
                        var u = p(Ut, a, a);
                        a.definition().orig.push(u), !s && k && (s = p(En, e)), B(t, n, u, s);
                    }
                } t.reverse(), n.reverse(); }(o, r), function (e, t) { for (var n = t.length, o = 0, r = i.body.length; o < r; o++) {
                    var a = i.body[o];
                    if (a instanceof st)
                        for (var s = 0, u = a.definitions.length; s < u; s++) {
                            var c = a.definitions[s], l = c.name;
                            if (B(e, t, l, c.value), k && i.argnames.every(e => e.name != l.name)) {
                                var f = i.variables.get(l.name), _ = p(nn, l, l);
                                f.references.push(_), t.splice(n++, 0, p(bt, c, { operator: "=", left: _, right: p(En, l) }));
                            }
                        }
                } }(o, r), r.push(n), o.length) {
                    const e = N.body.indexOf(t.parent(I - 1)) + 1;
                    N.body.splice(e, 0, p(st, i, { definitions: o }));
                } return r.map(e => e.clone(!0)); }(a)).optimize(t);
        } if (w && t.option("side_effects") && i.body.every(Di)) {
            var L = e.args.concat(p(En, e));
            return di(e, L).optimize(t);
        } if (t.option("negate_iife") && t.parent() instanceof me && Ai(e))
            return e.negate(t, !0); var V = e.evaluate(t); return V !== e ? (V = mi(V, e).optimize(t), Vi(t, V, e)) : e; function P(t) { return t ? t instanceof ze ? t.value ? t.value.clone(!0) : p(En, e) : t instanceof me ? p(vt, t, { operator: "void", expression: t.body.clone(!0) }) : void 0 : p(En, e); } function B(t, n, i, o) { var r = i.definition(); N.variables.set(i.name, r), N.enclosed.push(r), N.var_names().has(i.name) || (N.add_var_name(i.name), t.push(p(lt, i, { name: i, value: null }))); var a = p(nn, i, i); r.references.push(a), o && n.push(p(bt, e, { operator: "=", left: a, right: o.clone() })); } })), ai(mt, (function (e, t) { return t.option("unsafe") && Ti(e.expression) && ["Object", "RegExp", "Function", "Error", "Array"].includes(e.expression.name) ? p(dt, e, e).transform(t) : e; })), ai(ht, (function (e, t) { if (!t.option("side_effects"))
            return e; var n, i, o = []; n = kn(t), i = e.expressions.length - 1, e.expressions.forEach((function (e, r) { r < i && (e = e.drop_side_effect_free(t, n)), e && (Ei(o, e), n = !1); })); var r = o.length - 1; return function () { for (; r > 0 && wi(o[r], t);)
            r--; r < o.length - 1 && (o[r] = p(vt, e, { operator: "void", expression: o[r] }), o.length = r + 1); }(), 0 == r ? ((e = hi(t.parent(), t.self(), o[0])) instanceof ht || (e = e.optimize(t)), e) : (e.expressions = o, e); })), St.DEFMETHOD("lift_sequences", (function (e) { if (e.option("sequences") && this.expression instanceof ht) {
            var t = this.expression.expressions.slice(), n = this.clone();
            return n.expression = t.pop(), t.push(n), di(this, t).optimize(e);
        } return this; })), ai(At, (function (e, t) { return e.lift_sequences(t); })), ai(vt, (function (e, t) { var n = e.expression; if ("delete" == e.operator && !(n instanceof nn || n instanceof Et || Oi(n))) {
            if (n instanceof ht) {
                const i = n.expressions.slice();
                return i.push(p(An, e)), di(e, i).optimize(t);
            }
            return di(e, [n, p(An, e)]).optimize(t);
        } var i = e.lift_sequences(t); if (i !== e)
            return i; if (t.option("side_effects") && "void" == e.operator)
            return (n = n.drop_side_effect_free(t)) ? (e.expression = n, e) : p(En, e).optimize(t); if (t.in_boolean_context())
            switch (e.operator) {
                case "!":
                    if (n instanceof vt && "!" == n.operator)
                        return n.expression;
                    n instanceof Tt && (e = Vi(t, e, n.negate(t, kn(t))));
                    break;
                case "typeof": return t.warn("Boolean expression always true [{file}:{line},{col}]", e.start), (n instanceof nn ? p(An, e) : di(e, [n, p(An, e)])).optimize(t);
            } if ("-" == e.operator && n instanceof Dn && (n = n.transform(t)), n instanceof Tt && ("+" == e.operator || "-" == e.operator) && ("*" == n.operator || "/" == n.operator || "%" == n.operator))
            return p(Tt, e, { operator: n.operator, left: p(vt, n.left, { operator: e.operator, expression: n.left }), right: n.right }); if ("-" != e.operator || !(n instanceof fn || n instanceof Dn || n instanceof pn)) {
            var o = e.evaluate(t);
            if (o !== e)
                return Vi(t, o = mi(o, e).optimize(t), e);
        } return e; })), Tt.DEFMETHOD("lift_sequences", (function (e) { if (e.option("sequences")) {
            if (this.left instanceof ht) {
                var t = this.left.expressions.slice();
                return (n = this.clone()).left = t.pop(), t.push(n), di(this, t).optimize(e);
            }
            if (this.right instanceof ht && !this.left.has_side_effects(e)) {
                for (var n, i = "=" == this.operator && this.left instanceof nn, o = (t = this.right.expressions).length - 1, r = 0; r < o && (i || !t[r].has_side_effects(e)); r++)
                    ;
                if (r == o)
                    return t = t.slice(), (n = this.clone()).right = t.pop(), t.push(n), di(this, t).optimize(e);
                if (r > 0)
                    return (n = this.clone()).right = di(this.right, t.slice(r)), (t = t.slice(0, r)).push(n), di(this, t).optimize(e);
            }
        } return this; }));
        var ji = E("== === != !== * & | ^");
        function Zi(e, t) { for (var n, i = 0; n = e.parent(i); i++)
            if (n instanceof ke || n instanceof kt) {
                var o = n.name;
                if (o && o.definition() === t)
                    break;
            } return n; }
        function Ji(e, t) { for (const n of t.enclosed) {
            if (t.variables.has(n.name))
                continue;
            const i = e.find_variable(n.name);
            if (i) {
                if (i === n)
                    continue;
                return !0;
            }
        } return !1; }
        function Qi(e, t) { return e instanceof nn || e.TYPE === t.TYPE; }
        function eo(e, t) { const n = e => { if (e instanceof nn && i(e.definition(), t))
            return bn; }; return yn(e, (t, i) => { if (t instanceof we && t !== e) {
            var o = i.parent();
            if (o instanceof dt && o.expression === t)
                return;
            return !Tn(t, n) || bn;
        } }); }
        ai(Tt, (function (e, t) { function n() { return e.left.is_constant() || e.right.is_constant() || !e.left.has_side_effects(t) && !e.right.has_side_effects(t); } function i(t) { if (n()) {
            t && (e.operator = t);
            var i = e.left;
            e.left = e.right, e.right = i;
        } } if (ji.has(e.operator) && e.right.is_constant() && !e.left.is_constant() && (e.left instanceof Tt && ae[e.left.operator] >= ae[e.operator] || i()), e = e.lift_sequences(t), t.option("comparisons"))
            switch (e.operator) {
                case "===":
                case "!==":
                    var o = !0;
                    (e.left.is_string(t) && e.right.is_string(t) || e.left.is_number(t) && e.right.is_number(t) || e.left.is_boolean() && e.right.is_boolean() || e.left.equivalent_to(e.right)) && (e.operator = e.operator.substr(0, 2));
                case "==":
                case "!=":
                    if (!o && wi(e.left, t))
                        e.left = p(mn, e.left);
                    else if (t.option("typeofs") && e.left instanceof ln && "undefined" == e.left.value && e.right instanceof vt && "typeof" == e.right.operator) {
                        var r = e.right.expression;
                        (r instanceof nn ? !r.is_declared(t) : r instanceof Et && t.option("ie8")) || (e.right = r, e.left = p(En, e.left).optimize(t), 2 == e.operator.length && (e.operator += "="));
                    }
                    else if (e.left instanceof nn && e.right instanceof nn && e.left.definition() === e.right.definition() && ((u = e.left.fixed_value()) instanceof Ot || u instanceof ke || u instanceof Ft || u instanceof kt))
                        return p("=" == e.operator[0] ? An : vn, e);
                    break;
                case "&&":
                case "||":
                    var a = e.left;
                    if (a.operator == e.operator && (a = a.right), a instanceof Tt && a.operator == ("&&" == e.operator ? "!==" : "===") && e.right instanceof Tt && a.operator == e.right.operator && (wi(a.left, t) && e.right.left instanceof mn || a.left instanceof mn && wi(e.right.left, t)) && !a.right.has_side_effects(t) && a.right.equivalent_to(e.right.right)) {
                        var s = p(Tt, e, { operator: a.operator.slice(0, -1), left: p(mn, e), right: a.right });
                        return a !== e.left && (s = p(Tt, e, { operator: e.operator, left: e.left.left, right: s })), s;
                    }
            } var u; if ("+" == e.operator && t.in_boolean_context()) {
            var c = e.left.evaluate(t), l = e.right.evaluate(t);
            if (c && "string" == typeof c)
                return t.warn("+ in boolean context always true [{file}:{line},{col}]", e.start), di(e, [e.right, p(An, e)]).optimize(t);
            if (l && "string" == typeof l)
                return t.warn("+ in boolean context always true [{file}:{line},{col}]", e.start), di(e, [e.left, p(An, e)]).optimize(t);
        } if (t.option("comparisons") && e.is_boolean()) {
            if (!(t.parent() instanceof Tt) || t.parent() instanceof bt) {
                var f = p(vt, e, { operator: "!", expression: e.negate(t, kn(t)) });
                e = Vi(t, e, f);
            }
            if (t.option("unsafe_comps"))
                switch (e.operator) {
                    case "<":
                        i(">");
                        break;
                    case "<=": i(">=");
                }
        } if ("+" == e.operator) {
            if (e.right instanceof ln && "" == e.right.getValue() && e.left.is_string(t))
                return e.left;
            if (e.left instanceof ln && "" == e.left.getValue() && e.right.is_string(t))
                return e.right;
            if (e.left instanceof Tt && "+" == e.left.operator && e.left.left instanceof ln && "" == e.left.left.getValue() && e.right.is_string(t))
                return e.left = e.left.right, e.transform(t);
        } if (t.option("evaluate")) {
            switch (e.operator) {
                case "&&":
                    if (!(c = !!ni(e.left, 2) || !ni(e.left, 4) && e.left.evaluate(t)))
                        return t.warn("Condition left of && always false [{file}:{line},{col}]", e.start), hi(t.parent(), t.self(), e.left).optimize(t);
                    if (!(c instanceof fe))
                        return t.warn("Condition left of && always true [{file}:{line},{col}]", e.start), di(e, [e.left, e.right]).optimize(t);
                    if (l = e.right.evaluate(t)) {
                        if (!(l instanceof fe)) {
                            if ("&&" == (_ = t.parent()).operator && _.left === t.self() || t.in_boolean_context())
                                return t.warn("Dropping side-effect-free && [{file}:{line},{col}]", e.start), e.left.optimize(t);
                        }
                    }
                    else {
                        if (t.in_boolean_context())
                            return t.warn("Boolean && always false [{file}:{line},{col}]", e.start), di(e, [e.left, p(vn, e)]).optimize(t);
                        ii(e, 4);
                    }
                    if ("||" == e.left.operator)
                        if (!(d = e.left.right.evaluate(t)))
                            return p(yt, e, { condition: e.left.left, consequent: e.right, alternative: e.left.right }).optimize(t);
                    break;
                case "||":
                    var _, d;
                    if (!(c = !!ni(e.left, 2) || !ni(e.left, 4) && e.left.evaluate(t)))
                        return t.warn("Condition left of || always false [{file}:{line},{col}]", e.start), di(e, [e.left, e.right]).optimize(t);
                    if (!(c instanceof fe))
                        return t.warn("Condition left of || always true [{file}:{line},{col}]", e.start), hi(t.parent(), t.self(), e.left).optimize(t);
                    if (l = e.right.evaluate(t)) {
                        if (!(l instanceof fe)) {
                            if (t.in_boolean_context())
                                return t.warn("Boolean || always true [{file}:{line},{col}]", e.start), di(e, [e.left, p(An, e)]).optimize(t);
                            ii(e, 2);
                        }
                    }
                    else if ("||" == (_ = t.parent()).operator && _.left === t.self() || t.in_boolean_context())
                        return t.warn("Dropping side-effect-free || [{file}:{line},{col}]", e.start), e.left.optimize(t);
                    if ("&&" == e.left.operator)
                        if ((d = e.left.right.evaluate(t)) && !(d instanceof fe))
                            return p(yt, e, { condition: e.left.left, consequent: e.left.right, alternative: e.right }).optimize(t);
            }
            var m = !0;
            switch (e.operator) {
                case "+":
                    if (e.left instanceof cn && e.right instanceof Tt && "+" == e.right.operator && e.right.left instanceof cn && e.right.is_string(t) && (e = p(Tt, e, { operator: "+", left: p(ln, e.left, { value: "" + e.left.getValue() + e.right.left.getValue(), start: e.left.start, end: e.right.left.end }), right: e.right.right })), e.right instanceof cn && e.left instanceof Tt && "+" == e.left.operator && e.left.right instanceof cn && e.left.is_string(t) && (e = p(Tt, e, { operator: "+", left: e.left.left, right: p(ln, e.right, { value: "" + e.left.right.getValue() + e.right.getValue(), start: e.left.right.start, end: e.right.end }) })), e.left instanceof Tt && "+" == e.left.operator && e.left.is_string(t) && e.left.right instanceof cn && e.right instanceof Tt && "+" == e.right.operator && e.right.left instanceof cn && e.right.is_string(t) && (e = p(Tt, e, { operator: "+", left: p(Tt, e.left, { operator: "+", left: e.left.left, right: p(ln, e.left.right, { value: "" + e.left.right.getValue() + e.right.left.getValue(), start: e.left.right.start, end: e.right.left.end }) }), right: e.right.right })), e.right instanceof vt && "-" == e.right.operator && e.left.is_number(t)) {
                        e = p(Tt, e, { operator: "-", left: e.left, right: e.right.expression });
                        break;
                    }
                    if (e.left instanceof vt && "-" == e.left.operator && n() && e.right.is_number(t)) {
                        e = p(Tt, e, { operator: "-", left: e.right, right: e.left.expression });
                        break;
                    }
                case "*": m = t.option("unsafe_math");
                case "&":
                case "|":
                case "^":
                    if (e.left.is_number(t) && e.right.is_number(t) && n() && !(e.left instanceof Tt && e.left.operator != e.operator && ae[e.left.operator] >= ae[e.operator])) {
                        var h = p(Tt, e, { operator: e.operator, left: e.right, right: e.left });
                        e = e.right instanceof cn && !(e.left instanceof cn) ? Vi(t, h, e) : Vi(t, e, h);
                    }
                    m && e.is_number(t) && (e.right instanceof Tt && e.right.operator == e.operator && (e = p(Tt, e, { operator: e.operator, left: p(Tt, e.left, { operator: e.operator, left: e.left, right: e.right.left, start: e.left.start, end: e.right.left.end }), right: e.right.right })), e.right instanceof cn && e.left instanceof Tt && e.left.operator == e.operator && (e.left.left instanceof cn ? e = p(Tt, e, { operator: e.operator, left: p(Tt, e.left, { operator: e.operator, left: e.left.left, right: e.right, start: e.left.left.start, end: e.right.end }), right: e.left.right }) : e.left.right instanceof cn && (e = p(Tt, e, { operator: e.operator, left: p(Tt, e.left, { operator: e.operator, left: e.left.right, right: e.right, start: e.left.right.start, end: e.right.end }), right: e.left.left }))), e.left instanceof Tt && e.left.operator == e.operator && e.left.right instanceof cn && e.right instanceof Tt && e.right.operator == e.operator && e.right.left instanceof cn && (e = p(Tt, e, { operator: e.operator, left: p(Tt, e.left, { operator: e.operator, left: p(Tt, e.left.left, { operator: e.operator, left: e.left.right, right: e.right.left, start: e.left.right.start, end: e.right.left.end }), right: e.left.left }), right: e.right.right })));
            }
        } if (e.right instanceof Tt && e.right.operator == e.operator && (xi.has(e.operator) || "+" == e.operator && (e.right.left.is_string(t) || e.left.is_string(t) && e.right.right.is_string(t))))
            return e.left = p(Tt, e.left, { operator: e.operator, left: e.left, right: e.right.left }), e.right = e.right.right, e.transform(t); var E = e.evaluate(t); return E !== e ? (E = mi(E, e).optimize(t), Vi(t, E, e)) : e; })), ai(on, (function (e) { return e; })), ai(nn, (function (e, t) { if (!t.option("ie8") && Ti(e) && (!e.scope.uses_with || !t.find_parent(Re)))
            switch (e.name) {
                case "undefined": return p(En, e).optimize(t);
                case "NaN": return p(hn, e).optimize(t);
                case "Infinity": return p(Dn, e).optimize(t);
            } var n = t.parent(); if (t.option("reduce_vars") && ki(e, n) !== e) {
            const f = e.definition();
            if (t.top_retain && f.global && t.top_retain(f))
                return f.fixed = !1, f.should_replace = !1, f.single_use = !1, e;
            var i = e.fixed_value(), o = f.single_use && !(n instanceof dt && n.is_expr_pure(t) || T(n, Rn));
            if (o && (i instanceof ke || i instanceof kt))
                if ($i(i, t))
                    o = !1;
                else if (f.scope !== e.scope && (1 == f.escaped || ni(i, 16) || function (e) { for (var t, n = 0; t = e.parent(n++);) {
                    if (t instanceof pe)
                        return !1;
                    if (t instanceof Ot || t instanceof Rt || t instanceof Ft)
                        return !0;
                } return !1; }(t)))
                    o = !1;
                else if (Zi(t, f))
                    o = !1;
                else if ((f.scope !== e.scope || f.orig[0] instanceof zt) && "f" == (o = i.is_constant_expression(e.scope))) {
                    var r = e.scope;
                    do {
                        (r instanceof Pe || ci(r)) && ii(r, 16);
                    } while (r = r.parent_scope);
                }
            if (o && i instanceof ke) {
                const r = pi(t, !0);
                o = f.scope === e.scope && !Ji(r, i) || n instanceof dt && n.expression === e && !Ji(r, i);
            }
            if (o && i instanceof kt) {
                o = (!i.extends || !i.extends.may_throw(t) && !i.extends.has_side_effects(t)) && !i.properties.some(e => e.may_throw(t) || e.has_side_effects(t));
            }
            if (o && i) {
                if (i instanceof Lt && (ii(i, 256), i = p(Vt, i, i)), i instanceof Pe && (ii(i, 256), i = p(Le, i, i)), f.recursive_refs > 0 && i.name instanceof Wt) {
                    const e = i.name.definition();
                    let t = i.variables.get(i.name.name), n = t && t.orig[0];
                    n instanceof $t || (n = p($t, i.name, i.name), n.scope = i, i.name = n, t = i.def_function(n)), Tn(i, n => { n instanceof nn && n.definition() === e && (n.thedef = t, t.references.push(n)); });
                }
                return (i instanceof ke || i instanceof kt) && pi(t, !0).add_child_scope(i), i.optimize(t);
            }
            if (i && void 0 === f.should_replace) {
                let e;
                if (i instanceof sn)
                    f.orig[0] instanceof zt || !f.references.every(e => f.scope === e.scope) || (e = i);
                else {
                    var a = i.evaluate(t);
                    a === i || !t.option("unsafe_regexp") && a instanceof RegExp || (e = mi(a, i));
                }
                if (e) {
                    var s, u = e.optimize(t).size();
                    Tn(i, e => { if (e instanceof nn)
                        return bn; }) ? s = function () { var n = e.optimize(t); return n === e ? n.clone(!0) : n; } : (u = Math.min(u, i.size()), s = function () { var n = Ii(e.optimize(t), i); return n === e || n === i ? n.clone(!0) : n; });
                    var c = f.name.length, l = 0;
                    t.option("unused") && !t.exposed(f) && (l = (c + 2 + u) / (f.references.length - f.assignments)), f.should_replace = u <= c + l && s;
                }
                else
                    f.should_replace = !1;
            }
            if (f.should_replace)
                return f.should_replace();
        } return e; })), ai(En, (function (e, t) { if (t.option("unsafe_undefined")) {
            var n = _i(t, "undefined");
            if (n) {
                var i = p(nn, e, { name: "undefined", scope: n.scope, thedef: n });
                return ii(i, 8), i;
            }
        } var o = ki(t.self(), t.parent()); return o && Qi(o, e) ? e : p(vt, e, { operator: "void", expression: p(fn, e, { value: 0 }) }); })), ai(Dn, (function (e, t) { var n = ki(t.self(), t.parent()); return n && Qi(n, e) ? e : !t.option("keep_infinity") || n && !Qi(n, e) || _i(t, "Infinity") ? p(Tt, e, { operator: "/", left: p(fn, e, { value: 1 }), right: p(fn, e, { value: 0 }) }) : e; })), ai(hn, (function (e, t) { var n = ki(t.self(), t.parent()); return n && !Qi(n, e) || _i(t, "NaN") ? p(Tt, e, { operator: "/", left: p(fn, e, { value: 0 }), right: p(fn, e, { value: 0 }) }) : e; }));
        const to = E("+ - / * % >> << >>> | ^ &"), no = E("* | ^ &");
        function io(e, t) { return e instanceof nn && (e = e.fixed_value()), !!e && (!(e instanceof ke || e instanceof kt) || (!(e instanceof ke && e.contains_this()) || t.parent() instanceof mt)); }
        function oo(e, t) { return t.in_boolean_context() ? Vi(t, e, di(e, [e, p(An, e)]).optimize(t)) : e; }
        function ro(e, t, n) { for (var i = 0; i < n.length; i++) {
            var o = n[i];
            if (o instanceof Ne) {
                var r = o.expression;
                r instanceof Ot && (n.splice(i, 1, ...r.elements), i--);
            }
        } return e; }
        function ao(e, t) { if (!t.option("computed_props"))
            return e; if (!(e.key instanceof cn))
            return e; if (e.key instanceof ln || e.key instanceof fn) {
            if ("__proto__" === e.key.value)
                return e;
            if ("constructor" == e.key.value && t.parent() instanceof kt)
                return e;
            e.key = e instanceof Rt ? e.key.value : p(e instanceof It ? qt : Yt, e.key, { name: e.key.value });
        } return e; }
        ai(bt, (function (e, t) { var n; if (t.option("dead_code") && e.left instanceof nn && (n = e.left.definition()).scope === t.find_parent(ke)) {
            var i, o = 0, r = e;
            do {
                if (i = r, (r = t.parent(o++)) instanceof Xe) {
                    if (a(o, r))
                        break;
                    if (eo(n.scope, [n]))
                        break;
                    return "=" == e.operator ? e.right : (n.fixed = !1, p(Tt, e, { operator: e.operator.slice(0, -1), left: e.left, right: e.right }).optimize(t));
                }
            } while (r instanceof Tt && r.right === i || r instanceof ht && r.tail_node() === i);
        } return "=" == (e = e.lift_sequences(t)).operator && e.left instanceof nn && e.right instanceof Tt && (e.right.left instanceof nn && e.right.left.name == e.left.name && to.has(e.right.operator) ? (e.operator = e.right.operator + "=", e.right = e.right.right) : e.right.right instanceof nn && e.right.right.name == e.left.name && no.has(e.right.operator) && !e.right.left.has_side_effects(t) && (e.operator = e.right.operator + "=", e.right = e.right.left)), e; function a(n, i) { var o = e.right; e.right = p(mn, o); var r = i.may_throw(t); e.right = o; for (var a, s = e.left.definition().scope; (a = t.parent(n++)) !== s;)
            if (a instanceof it) {
                if (a.bfinally)
                    return !0;
                if (r && a.bcatch)
                    return !0;
            } } })), ai(Ct, (function (e, t) { if (!t.option("evaluate"))
            return e; var n = e.right.evaluate(t); return void 0 === n ? e = e.left : n !== e.right && (n = mi(n, e.right), e.right = Ii(n, e.right)), e; })), ai(yt, (function (e, t) { if (!t.option("conditionals"))
            return e; if (e.condition instanceof ht) {
            var n = e.condition.expressions.slice();
            return e.condition = n.pop(), n.push(e), di(e, n);
        } var i = e.condition.evaluate(t); if (i !== e.condition)
            return i ? (t.warn("Condition always true [{file}:{line},{col}]", e.start), hi(t.parent(), t.self(), e.consequent)) : (t.warn("Condition always false [{file}:{line},{col}]", e.start), hi(t.parent(), t.self(), e.alternative)); var o = i.negate(t, kn(t)); Vi(t, i, o) === o && (e = p(yt, e, { condition: o, consequent: e.alternative, alternative: e.consequent })); var r, a = e.condition, s = e.consequent, u = e.alternative; if (a instanceof nn && s instanceof nn && a.definition() === s.definition())
            return p(Tt, e, { operator: "||", left: a, right: u }); if (s instanceof bt && u instanceof bt && s.operator == u.operator && s.left.equivalent_to(u.left) && (!e.condition.has_side_effects(t) || "=" == s.operator && !s.left.has_side_effects(t)))
            return p(bt, e, { operator: s.operator, left: s.left, right: p(yt, e, { condition: e.condition, consequent: s.right, alternative: u.right }) }); if (s instanceof dt && u.TYPE === s.TYPE && s.args.length > 0 && s.args.length == u.args.length && s.expression.equivalent_to(u.expression) && !e.condition.has_side_effects(t) && !s.expression.has_side_effects(t) && "number" == typeof (r = function () { for (var e = s.args, t = u.args, n = 0, i = e.length; n < i; n++) {
            if (e[n] instanceof Ne)
                return;
            if (!e[n].equivalent_to(t[n])) {
                if (t[n] instanceof Ne)
                    return;
                for (var o = n + 1; o < i; o++) {
                    if (e[o] instanceof Ne)
                        return;
                    if (!e[o].equivalent_to(t[o]))
                        return;
                }
                return n;
            }
        } }())) {
            var c = s.clone();
            return c.args[r] = p(yt, e, { condition: e.condition, consequent: s.args[r], alternative: u.args[r] }), c;
        } if (u instanceof yt && s.equivalent_to(u.consequent))
            return p(yt, e, { condition: p(Tt, e, { operator: "||", left: a, right: u.condition }), consequent: s, alternative: u.alternative }).optimize(t); if (u instanceof ht && s.equivalent_to(u.expressions[u.expressions.length - 1]))
            return di(e, [p(Tt, e, { operator: "||", left: a, right: di(e, u.expressions.slice(0, -1)) }), s]).optimize(t); if (u instanceof Tt && "&&" == u.operator && s.equivalent_to(u.right))
            return p(Tt, e, { operator: "&&", left: p(Tt, e, { operator: "||", left: a, right: u.left }), right: s }).optimize(t); if (s instanceof yt && s.alternative.equivalent_to(u))
            return p(yt, e, { condition: p(Tt, e, { left: e.condition, operator: "&&", right: s.condition }), consequent: s.consequent, alternative: u }); if (s.equivalent_to(u))
            return di(e, [e.condition, s]).optimize(t); if (s instanceof Tt && "||" == s.operator && s.right.equivalent_to(u))
            return p(Tt, e, { operator: "||", left: p(Tt, e, { operator: "&&", left: e.condition, right: s.left }), right: u }).optimize(t); var l = t.in_boolean_context(); return _(e.consequent) ? d(e.alternative) ? f(e.condition) : p(Tt, e, { operator: "||", left: f(e.condition), right: e.alternative }) : d(e.consequent) ? _(e.alternative) ? f(e.condition.negate(t)) : p(Tt, e, { operator: "&&", left: f(e.condition.negate(t)), right: e.alternative }) : _(e.alternative) ? p(Tt, e, { operator: "||", left: f(e.condition.negate(t)), right: e.consequent }) : d(e.alternative) ? p(Tt, e, { operator: "&&", left: f(e.condition), right: e.consequent }) : e; function f(e) { return e.is_boolean() ? e : p(vt, e, { operator: "!", expression: e.negate(t) }); } function _(e) { return e instanceof An || l && e instanceof cn && e.getValue() || e instanceof vt && "!" == e.operator && e.expression instanceof cn && !e.expression.getValue(); } function d(e) { return e instanceof vn || l && e instanceof cn && !e.getValue() || e instanceof vt && "!" == e.operator && e.expression instanceof cn && e.expression.getValue(); } })), ai(Sn, (function (e, t) { if (t.in_boolean_context())
            return p(fn, e, { value: +e.value }); var n = t.parent(); return t.option("booleans_as_integers") ? (n instanceof Tt && ("===" == n.operator || "!==" == n.operator) && (n.operator = n.operator.replace(/=$/, "")), p(fn, e, { value: +e.value })) : t.option("booleans") ? n instanceof Tt && ("==" == n.operator || "!=" == n.operator) ? (t.warn("Non-strict equality against boolean: {operator} {value} [{file}:{line},{col}]", { operator: n.operator, value: e.value, file: n.start.file, line: n.start.line, col: n.start.col }), p(fn, e, { value: +e.value })) : p(vt, e, { operator: "!", expression: p(fn, e, { value: 1 - e.value }) }) : e; })), ai(Dt, (function (e, t) { var n, i = e.expression, o = e.property; if (t.option("properties")) {
            var r = o.evaluate(t);
            if (r !== o) {
                if ("string" == typeof r)
                    if ("undefined" == r)
                        r = void 0;
                    else
                        (A = parseFloat(r)).toString() == r && (r = A);
                o = e.property = Ii(o, mi(r, o).transform(t));
                var a = "" + r;
                if (j(a) && a.length <= o.size() + 1)
                    return p(gt, e, { expression: i, property: a, quote: o.quote }).optimize(t);
            }
        } e: if (t.option("arguments") && i instanceof nn && "arguments" == i.name && 1 == i.definition().orig.length && (n = i.scope) instanceof ke && n.uses_arguments && !(n instanceof Ve) && o instanceof fn) {
            for (var s = o.getValue(), u = new Set, c = n.argnames, l = 0; l < c.length; l++) {
                if (!(c[l] instanceof zt))
                    break e;
                var f = c[l].name;
                if (u.has(f))
                    break e;
                u.add(f);
            }
            var _ = n.argnames[s];
            if (_ && t.has_directive("use strict")) {
                var d = _.definition();
                (!t.option("reduce_vars") || d.assignments || d.orig.length > 1) && (_ = null);
            }
            else if (!_ && !t.option("keep_fargs") && s < n.argnames.length + 5)
                for (; s >= n.argnames.length;)
                    _ = p(zt, n, { name: n.make_var_name("argument_" + n.argnames.length), scope: n }), n.argnames.push(_), n.enclosed.push(n.def_variable(_));
            if (_) {
                var m = p(nn, e, _);
                return m.reference({}), oi(_, 1), m;
            }
        } if (ki(e, t.parent()))
            return e; if (r !== o) {
            var h = e.flatten_object(a, t);
            h && (i = e.expression = h.expression, o = e.property = h.property);
        } if (t.option("properties") && t.option("side_effects") && o instanceof fn && i instanceof Ot) {
            s = o.getValue();
            var E = i.elements, g = E[s];
            e: if (io(g, t)) {
                for (var D = !0, S = [], v = E.length; --v > s;) {
                    (A = E[v].drop_side_effect_free(t)) && (S.unshift(A), D && A.has_side_effects(t) && (D = !1));
                }
                if (g instanceof Ne)
                    break e;
                for (g = g instanceof gn ? p(En, g) : g, D || S.unshift(g); --v >= 0;) {
                    var A;
                    if ((A = E[v]) instanceof Ne)
                        break e;
                    (A = A.drop_side_effect_free(t)) ? S.unshift(A) : s--;
                }
                return D ? (S.push(g), di(e, S).optimize(t)) : p(Dt, e, { expression: p(Ot, i, { elements: S }), property: p(fn, o, { value: s }) });
            }
        } var T = e.evaluate(t); return T !== e ? Vi(t, T = mi(T, e).optimize(t), e) : e; })), ke.DEFMETHOD("contains_this", (function () { return Tn(this, e => e instanceof sn ? bn : e !== this && e instanceof we && !(e instanceof Ve) || void 0); })), Et.DEFMETHOD("flatten_object", (function (e, t) { if (t.option("properties")) {
            var n = t.option("unsafe_arrows") && t.option("ecma") >= 2015, i = this.expression;
            if (i instanceof Ft)
                for (var o = i.properties, r = o.length; --r >= 0;) {
                    var a = o[r];
                    if ("" + (a instanceof Nt ? a.key.name : a.key) == e) {
                        if (!o.every(e => e instanceof Rt || n && e instanceof Nt && !e.is_generator))
                            break;
                        if (!io(a.value, t))
                            break;
                        return p(Dt, this, { expression: p(Ot, i, { elements: o.map((function (e) { var t = e.value; t instanceof Ie && (t = p(Le, t, t)); var n = e.key; return n instanceof fe && !(n instanceof Yt) ? di(e, [n, t]) : t; })) }), property: p(fn, this, { value: r }) });
                    }
                }
        } })), ai(gt, (function (e, t) { "arguments" != e.property && "caller" != e.property || t.warn("Function.prototype.{prop} not supported [{file}:{line},{col}]", { prop: e.property, file: e.start.file, line: e.start.line, col: e.start.col }); const n = t.parent(); if (ki(e, n))
            return e; if (t.option("unsafe_proto") && e.expression instanceof gt && "prototype" == e.expression.property) {
            var i = e.expression.expression;
            if (Ti(i))
                switch (i.name) {
                    case "Array":
                        e.expression = p(Ot, e.expression, { elements: [] });
                        break;
                    case "Function":
                        e.expression = p(Le, e.expression, { argnames: [], body: [] });
                        break;
                    case "Number":
                        e.expression = p(fn, e.expression, { value: 0 });
                        break;
                    case "Object":
                        e.expression = p(Ft, e.expression, { properties: [] });
                        break;
                    case "RegExp":
                        e.expression = p(_n, e.expression, { value: { source: "t", flags: "" } });
                        break;
                    case "String": e.expression = p(ln, e.expression, { value: "" });
                }
        } if (!(n instanceof dt && T(n, Rn))) {
            const n = e.flatten_object(e.property, t);
            if (n)
                return n.optimize(t);
        } let o = e.evaluate(t); return o !== e ? (o = mi(o, e).optimize(t), Vi(t, o, e)) : e; })), ai(Ot, (function (e, t) { var n = oo(e, t); return n !== e ? n : ro(e, 0, e.elements); })), ai(Ft, (function (e, t) { var n = oo(e, t); if (n !== e)
            return n; for (var i = e.properties, o = 0; o < i.length; o++) {
            var r = i[o];
            if (r instanceof Ne) {
                var a = r.expression;
                a instanceof Ft ? (i.splice.apply(i, [o, 1].concat(r.expression.properties)), o--) : a instanceof cn && !(a instanceof ln) && i.splice(o, 1);
            }
        } return e; })), ai(_n, oo), ai(ze, (function (e, t) { return e.value && wi(e.value, t) && (e.value = null), e; })), ai(Ve, Wi), ai(Le, (function (e, t) { if (e = Wi(e, t), t.option("unsafe_arrows") && t.option("ecma") >= 2015 && !e.name && !e.is_generator && !e.uses_arguments && !e.pinned()) {
            if (!Tn(e, e => { if (e instanceof sn)
                return bn; }))
                return p(Ve, e, e).optimize(t);
        } return e; })), ai(kt, (function (e) { return e; })), ai(Ze, (function (e, t) { return e.expression && !e.is_star && wi(e.expression, t) && (e.expression = null), e; })), ai(Ue, (function (e, t) { if (!t.option("evaluate") || t.parent() instanceof Ke)
            return e; for (var n = [], i = 0; i < e.segments.length; i++) {
            var o = e.segments[i];
            if (o instanceof fe) {
                var r = o.evaluate(t);
                if (r !== o && (r + "").length <= o.size() + "${}".length) {
                    n[n.length - 1].value = n[n.length - 1].value + r + e.segments[++i].value;
                    continue;
                }
            }
            n.push(o);
        } return e.segments = n, 1 == n.length ? p(ln, e, n[0]) : e; })), ai(Ke, (function (e) { return e; })), ai(Mt, ao), ai(Nt, (function (e, t) { if (ao(e, t), t.option("arrows") && t.parent() instanceof Ft && !e.is_generator && !e.value.uses_arguments && !e.value.pinned() && 1 == e.value.body.length && e.value.body[0] instanceof ze && e.value.body[0].value && !e.value.contains_this()) {
            var n = p(Ve, e.value, e.value);
            return n.async = e.async, n.is_generator = e.is_generator, p(Rt, e, { key: e.key instanceof Yt ? e.key.name : e.key, value: n, quote: e.quote });
        } return e; })), ai(Rt, (function (e, t) { ao(e, t); var n = t.option("unsafe_methods"); if (n && t.option("ecma") >= 2015 && (!(n instanceof RegExp) || n.test(e.key + ""))) {
            var i = e.key, o = e.value;
            if ((o instanceof Ve && Array.isArray(o.body) && !o.contains_this() || o instanceof Le) && !o.name)
                return p(Nt, e, { async: o.async, is_generator: o.is_generator, key: i instanceof fe ? i : p(Yt, e, { name: i }), value: p(Ie, o, o), quote: e.quote });
        } return e; })), ai(Be, (function (e, t) { if (1 == t.option("pure_getters") && t.option("unused") && !e.is_array && Array.isArray(e.names) && !function (e) { for (var t = [/^VarDef$/, /^(Const|Let|Var)$/, /^Export$/], n = 0, i = 0, o = t.length; n < o; i++) {
            var r = e.parent(i);
            if (!r)
                return !1;
            if (0 !== n || "Destructuring" != r.TYPE) {
                if (!t[n].test(r.TYPE))
                    return !1;
                n++;
            }
        } return !0; }(t)) {
            for (var n = [], i = 0; i < e.names.length; i++) {
                var o = e.names[i];
                o instanceof Rt && "string" == typeof o.key && o.value instanceof Kt && !r(t, o.value.definition()) || n.push(o);
            }
            n.length != e.names.length && (e.names = n);
        } return e; function r(e, t) { return !!t.references.length || !!t.global && (!e.toplevel.vars || !!e.top_retain && e.top_retain(t)); } }));
        var so = ["$&", "$'", "$*", "$+", "$1", "$2", "$3", "$4", "$5", "$6", "$7", "$8", "$9", "$_", "$`", "$input", "@@iterator", "ABORT_ERR", "ACTIVE", "ACTIVE_ATTRIBUTES", "ACTIVE_TEXTURE", "ACTIVE_UNIFORMS", "ADDITION", "ALIASED_LINE_WIDTH_RANGE", "ALIASED_POINT_SIZE_RANGE", "ALLOW_KEYBOARD_INPUT", "ALLPASS", "ALPHA", "ALPHA_BITS", "ALT_MASK", "ALWAYS", "ANY_TYPE", "ANY_UNORDERED_NODE_TYPE", "ARRAY_BUFFER", "ARRAY_BUFFER_BINDING", "ATTACHED_SHADERS", "ATTRIBUTE_NODE", "AT_TARGET", "AddSearchProvider", "AnalyserNode", "AnimationEvent", "AnonXMLHttpRequest", "ApplicationCache", "ApplicationCacheErrorEvent", "Array", "ArrayBuffer", "Attr", "Audio", "AudioBuffer", "AudioBufferSourceNode", "AudioContext", "AudioDestinationNode", "AudioListener", "AudioNode", "AudioParam", "AudioProcessingEvent", "AudioStreamTrack", "AutocompleteErrorEvent", "BACK", "BAD_BOUNDARYPOINTS_ERR", "BANDPASS", "BLEND", "BLEND_COLOR", "BLEND_DST_ALPHA", "BLEND_DST_RGB", "BLEND_EQUATION", "BLEND_EQUATION_ALPHA", "BLEND_EQUATION_RGB", "BLEND_SRC_ALPHA", "BLEND_SRC_RGB", "BLUE_BITS", "BLUR", "BOOL", "BOOLEAN_TYPE", "BOOL_VEC2", "BOOL_VEC3", "BOOL_VEC4", "BOTH", "BROWSER_DEFAULT_WEBGL", "BUBBLING_PHASE", "BUFFER_SIZE", "BUFFER_USAGE", "BYTE", "BYTES_PER_ELEMENT", "BarProp", "BaseHref", "BatteryManager", "BeforeLoadEvent", "BeforeUnloadEvent", "BiquadFilterNode", "Blob", "BlobEvent", "Boolean", "CAPTURING_PHASE", "CCW", "CDATASection", "CDATA_SECTION_NODE", "CHANGE", "CHARSET_RULE", "CHECKING", "CLAMP_TO_EDGE", "CLICK", "CLOSED", "CLOSING", "COLOR_ATTACHMENT0", "COLOR_BUFFER_BIT", "COLOR_CLEAR_VALUE", "COLOR_WRITEMASK", "COMMENT_NODE", "COMPILE_STATUS", "COMPRESSED_RGBA_S3TC_DXT1_EXT", "COMPRESSED_RGBA_S3TC_DXT3_EXT", "COMPRESSED_RGBA_S3TC_DXT5_EXT", "COMPRESSED_RGB_S3TC_DXT1_EXT", "COMPRESSED_TEXTURE_FORMATS", "CONNECTING", "CONSTANT_ALPHA", "CONSTANT_COLOR", "CONSTRAINT_ERR", "CONTEXT_LOST_WEBGL", "CONTROL_MASK", "COUNTER_STYLE_RULE", "CSS", "CSS2Properties", "CSSCharsetRule", "CSSConditionRule", "CSSCounterStyleRule", "CSSFontFaceRule", "CSSFontFeatureValuesRule", "CSSGroupingRule", "CSSImportRule", "CSSKeyframeRule", "CSSKeyframesRule", "CSSMediaRule", "CSSMozDocumentRule", "CSSNameSpaceRule", "CSSPageRule", "CSSPrimitiveValue", "CSSRule", "CSSRuleList", "CSSStyleDeclaration", "CSSStyleRule", "CSSStyleSheet", "CSSSupportsRule", "CSSUnknownRule", "CSSValue", "CSSValueList", "CSSVariablesDeclaration", "CSSVariablesRule", "CSSViewportRule", "CSS_ATTR", "CSS_CM", "CSS_COUNTER", "CSS_CUSTOM", "CSS_DEG", "CSS_DIMENSION", "CSS_EMS", "CSS_EXS", "CSS_FILTER_BLUR", "CSS_FILTER_BRIGHTNESS", "CSS_FILTER_CONTRAST", "CSS_FILTER_CUSTOM", "CSS_FILTER_DROP_SHADOW", "CSS_FILTER_GRAYSCALE", "CSS_FILTER_HUE_ROTATE", "CSS_FILTER_INVERT", "CSS_FILTER_OPACITY", "CSS_FILTER_REFERENCE", "CSS_FILTER_SATURATE", "CSS_FILTER_SEPIA", "CSS_GRAD", "CSS_HZ", "CSS_IDENT", "CSS_IN", "CSS_INHERIT", "CSS_KHZ", "CSS_MATRIX", "CSS_MATRIX3D", "CSS_MM", "CSS_MS", "CSS_NUMBER", "CSS_PC", "CSS_PERCENTAGE", "CSS_PERSPECTIVE", "CSS_PRIMITIVE_VALUE", "CSS_PT", "CSS_PX", "CSS_RAD", "CSS_RECT", "CSS_RGBCOLOR", "CSS_ROTATE", "CSS_ROTATE3D", "CSS_ROTATEX", "CSS_ROTATEY", "CSS_ROTATEZ", "CSS_S", "CSS_SCALE", "CSS_SCALE3D", "CSS_SCALEX", "CSS_SCALEY", "CSS_SCALEZ", "CSS_SKEW", "CSS_SKEWX", "CSS_SKEWY", "CSS_STRING", "CSS_TRANSLATE", "CSS_TRANSLATE3D", "CSS_TRANSLATEX", "CSS_TRANSLATEY", "CSS_TRANSLATEZ", "CSS_UNKNOWN", "CSS_URI", "CSS_VALUE_LIST", "CSS_VH", "CSS_VMAX", "CSS_VMIN", "CSS_VW", "CULL_FACE", "CULL_FACE_MODE", "CURRENT_PROGRAM", "CURRENT_VERTEX_ATTRIB", "CUSTOM", "CW", "CanvasGradient", "CanvasPattern", "CanvasRenderingContext2D", "CaretPosition", "ChannelMergerNode", "ChannelSplitterNode", "CharacterData", "ClientRect", "ClientRectList", "Clipboard", "ClipboardEvent", "CloseEvent", "Collator", "CommandEvent", "Comment", "CompositionEvent", "Console", "Controllers", "ConvolverNode", "Counter", "Crypto", "CryptoKey", "CustomEvent", "DATABASE_ERR", "DATA_CLONE_ERR", "DATA_ERR", "DBLCLICK", "DECR", "DECR_WRAP", "DELETE_STATUS", "DEPTH_ATTACHMENT", "DEPTH_BITS", "DEPTH_BUFFER_BIT", "DEPTH_CLEAR_VALUE", "DEPTH_COMPONENT", "DEPTH_COMPONENT16", "DEPTH_FUNC", "DEPTH_RANGE", "DEPTH_STENCIL", "DEPTH_STENCIL_ATTACHMENT", "DEPTH_TEST", "DEPTH_WRITEMASK", "DIRECTION_DOWN", "DIRECTION_LEFT", "DIRECTION_RIGHT", "DIRECTION_UP", "DISABLED", "DISPATCH_REQUEST_ERR", "DITHER", "DOCUMENT_FRAGMENT_NODE", "DOCUMENT_NODE", "DOCUMENT_POSITION_CONTAINED_BY", "DOCUMENT_POSITION_CONTAINS", "DOCUMENT_POSITION_DISCONNECTED", "DOCUMENT_POSITION_FOLLOWING", "DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC", "DOCUMENT_POSITION_PRECEDING", "DOCUMENT_TYPE_NODE", "DOMCursor", "DOMError", "DOMException", "DOMImplementation", "DOMImplementationLS", "DOMMatrix", "DOMMatrixReadOnly", "DOMParser", "DOMPoint", "DOMPointReadOnly", "DOMQuad", "DOMRect", "DOMRectList", "DOMRectReadOnly", "DOMRequest", "DOMSTRING_SIZE_ERR", "DOMSettableTokenList", "DOMStringList", "DOMStringMap", "DOMTokenList", "DOMTransactionEvent", "DOM_DELTA_LINE", "DOM_DELTA_PAGE", "DOM_DELTA_PIXEL", "DOM_INPUT_METHOD_DROP", "DOM_INPUT_METHOD_HANDWRITING", "DOM_INPUT_METHOD_IME", "DOM_INPUT_METHOD_KEYBOARD", "DOM_INPUT_METHOD_MULTIMODAL", "DOM_INPUT_METHOD_OPTION", "DOM_INPUT_METHOD_PASTE", "DOM_INPUT_METHOD_SCRIPT", "DOM_INPUT_METHOD_UNKNOWN", "DOM_INPUT_METHOD_VOICE", "DOM_KEY_LOCATION_JOYSTICK", "DOM_KEY_LOCATION_LEFT", "DOM_KEY_LOCATION_MOBILE", "DOM_KEY_LOCATION_NUMPAD", "DOM_KEY_LOCATION_RIGHT", "DOM_KEY_LOCATION_STANDARD", "DOM_VK_0", "DOM_VK_1", "DOM_VK_2", "DOM_VK_3", "DOM_VK_4", "DOM_VK_5", "DOM_VK_6", "DOM_VK_7", "DOM_VK_8", "DOM_VK_9", "DOM_VK_A", "DOM_VK_ACCEPT", "DOM_VK_ADD", "DOM_VK_ALT", "DOM_VK_ALTGR", "DOM_VK_AMPERSAND", "DOM_VK_ASTERISK", "DOM_VK_AT", "DOM_VK_ATTN", "DOM_VK_B", "DOM_VK_BACKSPACE", "DOM_VK_BACK_QUOTE", "DOM_VK_BACK_SLASH", "DOM_VK_BACK_SPACE", "DOM_VK_C", "DOM_VK_CANCEL", "DOM_VK_CAPS_LOCK", "DOM_VK_CIRCUMFLEX", "DOM_VK_CLEAR", "DOM_VK_CLOSE_BRACKET", "DOM_VK_CLOSE_CURLY_BRACKET", "DOM_VK_CLOSE_PAREN", "DOM_VK_COLON", "DOM_VK_COMMA", "DOM_VK_CONTEXT_MENU", "DOM_VK_CONTROL", "DOM_VK_CONVERT", "DOM_VK_CRSEL", "DOM_VK_CTRL", "DOM_VK_D", "DOM_VK_DECIMAL", "DOM_VK_DELETE", "DOM_VK_DIVIDE", "DOM_VK_DOLLAR", "DOM_VK_DOUBLE_QUOTE", "DOM_VK_DOWN", "DOM_VK_E", "DOM_VK_EISU", "DOM_VK_END", "DOM_VK_ENTER", "DOM_VK_EQUALS", "DOM_VK_EREOF", "DOM_VK_ESCAPE", "DOM_VK_EXCLAMATION", "DOM_VK_EXECUTE", "DOM_VK_EXSEL", "DOM_VK_F", "DOM_VK_F1", "DOM_VK_F10", "DOM_VK_F11", "DOM_VK_F12", "DOM_VK_F13", "DOM_VK_F14", "DOM_VK_F15", "DOM_VK_F16", "DOM_VK_F17", "DOM_VK_F18", "DOM_VK_F19", "DOM_VK_F2", "DOM_VK_F20", "DOM_VK_F21", "DOM_VK_F22", "DOM_VK_F23", "DOM_VK_F24", "DOM_VK_F25", "DOM_VK_F26", "DOM_VK_F27", "DOM_VK_F28", "DOM_VK_F29", "DOM_VK_F3", "DOM_VK_F30", "DOM_VK_F31", "DOM_VK_F32", "DOM_VK_F33", "DOM_VK_F34", "DOM_VK_F35", "DOM_VK_F36", "DOM_VK_F4", "DOM_VK_F5", "DOM_VK_F6", "DOM_VK_F7", "DOM_VK_F8", "DOM_VK_F9", "DOM_VK_FINAL", "DOM_VK_FRONT", "DOM_VK_G", "DOM_VK_GREATER_THAN", "DOM_VK_H", "DOM_VK_HANGUL", "DOM_VK_HANJA", "DOM_VK_HASH", "DOM_VK_HELP", "DOM_VK_HK_TOGGLE", "DOM_VK_HOME", "DOM_VK_HYPHEN_MINUS", "DOM_VK_I", "DOM_VK_INSERT", "DOM_VK_J", "DOM_VK_JUNJA", "DOM_VK_K", "DOM_VK_KANA", "DOM_VK_KANJI", "DOM_VK_L", "DOM_VK_LEFT", "DOM_VK_LEFT_TAB", "DOM_VK_LESS_THAN", "DOM_VK_M", "DOM_VK_META", "DOM_VK_MODECHANGE", "DOM_VK_MULTIPLY", "DOM_VK_N", "DOM_VK_NONCONVERT", "DOM_VK_NUMPAD0", "DOM_VK_NUMPAD1", "DOM_VK_NUMPAD2", "DOM_VK_NUMPAD3", "DOM_VK_NUMPAD4", "DOM_VK_NUMPAD5", "DOM_VK_NUMPAD6", "DOM_VK_NUMPAD7", "DOM_VK_NUMPAD8", "DOM_VK_NUMPAD9", "DOM_VK_NUM_LOCK", "DOM_VK_O", "DOM_VK_OEM_1", "DOM_VK_OEM_102", "DOM_VK_OEM_2", "DOM_VK_OEM_3", "DOM_VK_OEM_4", "DOM_VK_OEM_5", "DOM_VK_OEM_6", "DOM_VK_OEM_7", "DOM_VK_OEM_8", "DOM_VK_OEM_COMMA", "DOM_VK_OEM_MINUS", "DOM_VK_OEM_PERIOD", "DOM_VK_OEM_PLUS", "DOM_VK_OPEN_BRACKET", "DOM_VK_OPEN_CURLY_BRACKET", "DOM_VK_OPEN_PAREN", "DOM_VK_P", "DOM_VK_PA1", "DOM_VK_PAGEDOWN", "DOM_VK_PAGEUP", "DOM_VK_PAGE_DOWN", "DOM_VK_PAGE_UP", "DOM_VK_PAUSE", "DOM_VK_PERCENT", "DOM_VK_PERIOD", "DOM_VK_PIPE", "DOM_VK_PLAY", "DOM_VK_PLUS", "DOM_VK_PRINT", "DOM_VK_PRINTSCREEN", "DOM_VK_PROCESSKEY", "DOM_VK_PROPERITES", "DOM_VK_Q", "DOM_VK_QUESTION_MARK", "DOM_VK_QUOTE", "DOM_VK_R", "DOM_VK_REDO", "DOM_VK_RETURN", "DOM_VK_RIGHT", "DOM_VK_S", "DOM_VK_SCROLL_LOCK", "DOM_VK_SELECT", "DOM_VK_SEMICOLON", "DOM_VK_SEPARATOR", "DOM_VK_SHIFT", "DOM_VK_SLASH", "DOM_VK_SLEEP", "DOM_VK_SPACE", "DOM_VK_SUBTRACT", "DOM_VK_T", "DOM_VK_TAB", "DOM_VK_TILDE", "DOM_VK_U", "DOM_VK_UNDERSCORE", "DOM_VK_UNDO", "DOM_VK_UNICODE", "DOM_VK_UP", "DOM_VK_V", "DOM_VK_VOLUME_DOWN", "DOM_VK_VOLUME_MUTE", "DOM_VK_VOLUME_UP", "DOM_VK_W", "DOM_VK_WIN", "DOM_VK_WINDOW", "DOM_VK_WIN_ICO_00", "DOM_VK_WIN_ICO_CLEAR", "DOM_VK_WIN_ICO_HELP", "DOM_VK_WIN_OEM_ATTN", "DOM_VK_WIN_OEM_AUTO", "DOM_VK_WIN_OEM_BACKTAB", "DOM_VK_WIN_OEM_CLEAR", "DOM_VK_WIN_OEM_COPY", "DOM_VK_WIN_OEM_CUSEL", "DOM_VK_WIN_OEM_ENLW", "DOM_VK_WIN_OEM_FINISH", "DOM_VK_WIN_OEM_FJ_JISHO", "DOM_VK_WIN_OEM_FJ_LOYA", "DOM_VK_WIN_OEM_FJ_MASSHOU", "DOM_VK_WIN_OEM_FJ_ROYA", "DOM_VK_WIN_OEM_FJ_TOUROKU", "DOM_VK_WIN_OEM_JUMP", "DOM_VK_WIN_OEM_PA1", "DOM_VK_WIN_OEM_PA2", "DOM_VK_WIN_OEM_PA3", "DOM_VK_WIN_OEM_RESET", "DOM_VK_WIN_OEM_WSCTRL", "DOM_VK_X", "DOM_VK_XF86XK_ADD_FAVORITE", "DOM_VK_XF86XK_APPLICATION_LEFT", "DOM_VK_XF86XK_APPLICATION_RIGHT", "DOM_VK_XF86XK_AUDIO_CYCLE_TRACK", "DOM_VK_XF86XK_AUDIO_FORWARD", "DOM_VK_XF86XK_AUDIO_LOWER_VOLUME", "DOM_VK_XF86XK_AUDIO_MEDIA", "DOM_VK_XF86XK_AUDIO_MUTE", "DOM_VK_XF86XK_AUDIO_NEXT", "DOM_VK_XF86XK_AUDIO_PAUSE", "DOM_VK_XF86XK_AUDIO_PLAY", "DOM_VK_XF86XK_AUDIO_PREV", "DOM_VK_XF86XK_AUDIO_RAISE_VOLUME", "DOM_VK_XF86XK_AUDIO_RANDOM_PLAY", "DOM_VK_XF86XK_AUDIO_RECORD", "DOM_VK_XF86XK_AUDIO_REPEAT", "DOM_VK_XF86XK_AUDIO_REWIND", "DOM_VK_XF86XK_AUDIO_STOP", "DOM_VK_XF86XK_AWAY", "DOM_VK_XF86XK_BACK", "DOM_VK_XF86XK_BACK_FORWARD", "DOM_VK_XF86XK_BATTERY", "DOM_VK_XF86XK_BLUE", "DOM_VK_XF86XK_BLUETOOTH", "DOM_VK_XF86XK_BOOK", "DOM_VK_XF86XK_BRIGHTNESS_ADJUST", "DOM_VK_XF86XK_CALCULATOR", "DOM_VK_XF86XK_CALENDAR", "DOM_VK_XF86XK_CD", "DOM_VK_XF86XK_CLOSE", "DOM_VK_XF86XK_COMMUNITY", "DOM_VK_XF86XK_CONTRAST_ADJUST", "DOM_VK_XF86XK_COPY", "DOM_VK_XF86XK_CUT", "DOM_VK_XF86XK_CYCLE_ANGLE", "DOM_VK_XF86XK_DISPLAY", "DOM_VK_XF86XK_DOCUMENTS", "DOM_VK_XF86XK_DOS", "DOM_VK_XF86XK_EJECT", "DOM_VK_XF86XK_EXCEL", "DOM_VK_XF86XK_EXPLORER", "DOM_VK_XF86XK_FAVORITES", "DOM_VK_XF86XK_FINANCE", "DOM_VK_XF86XK_FORWARD", "DOM_VK_XF86XK_FRAME_BACK", "DOM_VK_XF86XK_FRAME_FORWARD", "DOM_VK_XF86XK_GAME", "DOM_VK_XF86XK_GO", "DOM_VK_XF86XK_GREEN", "DOM_VK_XF86XK_HIBERNATE", "DOM_VK_XF86XK_HISTORY", "DOM_VK_XF86XK_HOME_PAGE", "DOM_VK_XF86XK_HOT_LINKS", "DOM_VK_XF86XK_I_TOUCH", "DOM_VK_XF86XK_KBD_BRIGHTNESS_DOWN", "DOM_VK_XF86XK_KBD_BRIGHTNESS_UP", "DOM_VK_XF86XK_KBD_LIGHT_ON_OFF", "DOM_VK_XF86XK_LAUNCH0", "DOM_VK_XF86XK_LAUNCH1", "DOM_VK_XF86XK_LAUNCH2", "DOM_VK_XF86XK_LAUNCH3", "DOM_VK_XF86XK_LAUNCH4", "DOM_VK_XF86XK_LAUNCH5", "DOM_VK_XF86XK_LAUNCH6", "DOM_VK_XF86XK_LAUNCH7", "DOM_VK_XF86XK_LAUNCH8", "DOM_VK_XF86XK_LAUNCH9", "DOM_VK_XF86XK_LAUNCH_A", "DOM_VK_XF86XK_LAUNCH_B", "DOM_VK_XF86XK_LAUNCH_C", "DOM_VK_XF86XK_LAUNCH_D", "DOM_VK_XF86XK_LAUNCH_E", "DOM_VK_XF86XK_LAUNCH_F", "DOM_VK_XF86XK_LIGHT_BULB", "DOM_VK_XF86XK_LOG_OFF", "DOM_VK_XF86XK_MAIL", "DOM_VK_XF86XK_MAIL_FORWARD", "DOM_VK_XF86XK_MARKET", "DOM_VK_XF86XK_MEETING", "DOM_VK_XF86XK_MEMO", "DOM_VK_XF86XK_MENU_KB", "DOM_VK_XF86XK_MENU_PB", "DOM_VK_XF86XK_MESSENGER", "DOM_VK_XF86XK_MON_BRIGHTNESS_DOWN", "DOM_VK_XF86XK_MON_BRIGHTNESS_UP", "DOM_VK_XF86XK_MUSIC", "DOM_VK_XF86XK_MY_COMPUTER", "DOM_VK_XF86XK_MY_SITES", "DOM_VK_XF86XK_NEW", "DOM_VK_XF86XK_NEWS", "DOM_VK_XF86XK_OFFICE_HOME", "DOM_VK_XF86XK_OPEN", "DOM_VK_XF86XK_OPEN_URL", "DOM_VK_XF86XK_OPTION", "DOM_VK_XF86XK_PASTE", "DOM_VK_XF86XK_PHONE", "DOM_VK_XF86XK_PICTURES", "DOM_VK_XF86XK_POWER_DOWN", "DOM_VK_XF86XK_POWER_OFF", "DOM_VK_XF86XK_RED", "DOM_VK_XF86XK_REFRESH", "DOM_VK_XF86XK_RELOAD", "DOM_VK_XF86XK_REPLY", "DOM_VK_XF86XK_ROCKER_DOWN", "DOM_VK_XF86XK_ROCKER_ENTER", "DOM_VK_XF86XK_ROCKER_UP", "DOM_VK_XF86XK_ROTATE_WINDOWS", "DOM_VK_XF86XK_ROTATION_KB", "DOM_VK_XF86XK_ROTATION_PB", "DOM_VK_XF86XK_SAVE", "DOM_VK_XF86XK_SCREEN_SAVER", "DOM_VK_XF86XK_SCROLL_CLICK", "DOM_VK_XF86XK_SCROLL_DOWN", "DOM_VK_XF86XK_SCROLL_UP", "DOM_VK_XF86XK_SEARCH", "DOM_VK_XF86XK_SEND", "DOM_VK_XF86XK_SHOP", "DOM_VK_XF86XK_SPELL", "DOM_VK_XF86XK_SPLIT_SCREEN", "DOM_VK_XF86XK_STANDBY", "DOM_VK_XF86XK_START", "DOM_VK_XF86XK_STOP", "DOM_VK_XF86XK_SUBTITLE", "DOM_VK_XF86XK_SUPPORT", "DOM_VK_XF86XK_SUSPEND", "DOM_VK_XF86XK_TASK_PANE", "DOM_VK_XF86XK_TERMINAL", "DOM_VK_XF86XK_TIME", "DOM_VK_XF86XK_TOOLS", "DOM_VK_XF86XK_TOP_MENU", "DOM_VK_XF86XK_TO_DO_LIST", "DOM_VK_XF86XK_TRAVEL", "DOM_VK_XF86XK_USER1KB", "DOM_VK_XF86XK_USER2KB", "DOM_VK_XF86XK_USER_PB", "DOM_VK_XF86XK_UWB", "DOM_VK_XF86XK_VENDOR_HOME", "DOM_VK_XF86XK_VIDEO", "DOM_VK_XF86XK_VIEW", "DOM_VK_XF86XK_WAKE_UP", "DOM_VK_XF86XK_WEB_CAM", "DOM_VK_XF86XK_WHEEL_BUTTON", "DOM_VK_XF86XK_WLAN", "DOM_VK_XF86XK_WORD", "DOM_VK_XF86XK_WWW", "DOM_VK_XF86XK_XFER", "DOM_VK_XF86XK_YELLOW", "DOM_VK_XF86XK_ZOOM_IN", "DOM_VK_XF86XK_ZOOM_OUT", "DOM_VK_Y", "DOM_VK_Z", "DOM_VK_ZOOM", "DONE", "DONT_CARE", "DOWNLOADING", "DRAGDROP", "DST_ALPHA", "DST_COLOR", "DYNAMIC_DRAW", "DataChannel", "DataTransfer", "DataTransferItem", "DataTransferItemList", "DataView", "Date", "DateTimeFormat", "DelayNode", "DesktopNotification", "DesktopNotificationCenter", "DeviceLightEvent", "DeviceMotionEvent", "DeviceOrientationEvent", "DeviceProximityEvent", "DeviceStorage", "DeviceStorageChangeEvent", "Document", "DocumentFragment", "DocumentType", "DragEvent", "DynamicsCompressorNode", "E", "ELEMENT_ARRAY_BUFFER", "ELEMENT_ARRAY_BUFFER_BINDING", "ELEMENT_NODE", "EMPTY", "ENCODING_ERR", "ENDED", "END_TO_END", "END_TO_START", "ENTITY_NODE", "ENTITY_REFERENCE_NODE", "EPSILON", "EQUAL", "EQUALPOWER", "ERROR", "EXPONENTIAL_DISTANCE", "Element", "ElementQuery", "Entity", "EntityReference", "Error", "ErrorEvent", "EvalError", "Event", "EventException", "EventSource", "EventTarget", "External", "FASTEST", "FIDOSDK", "FILTER_ACCEPT", "FILTER_INTERRUPT", "FILTER_REJECT", "FILTER_SKIP", "FINISHED_STATE", "FIRST_ORDERED_NODE_TYPE", "FLOAT", "FLOAT_MAT2", "FLOAT_MAT3", "FLOAT_MAT4", "FLOAT_VEC2", "FLOAT_VEC3", "FLOAT_VEC4", "FOCUS", "FONT_FACE_RULE", "FONT_FEATURE_VALUES_RULE", "FRAGMENT_SHADER", "FRAGMENT_SHADER_DERIVATIVE_HINT_OES", "FRAMEBUFFER", "FRAMEBUFFER_ATTACHMENT_OBJECT_NAME", "FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE", "FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE", "FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL", "FRAMEBUFFER_BINDING", "FRAMEBUFFER_COMPLETE", "FRAMEBUFFER_INCOMPLETE_ATTACHMENT", "FRAMEBUFFER_INCOMPLETE_DIMENSIONS", "FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT", "FRAMEBUFFER_UNSUPPORTED", "FRONT", "FRONT_AND_BACK", "FRONT_FACE", "FUNC_ADD", "FUNC_REVERSE_SUBTRACT", "FUNC_SUBTRACT", "Feed", "FeedEntry", "File", "FileError", "FileList", "FileReader", "FindInPage", "Float32Array", "Float64Array", "FocusEvent", "FontFace", "FormData", "Function", "GENERATE_MIPMAP_HINT", "GEQUAL", "GREATER", "GREEN_BITS", "GainNode", "Gamepad", "GamepadButton", "GamepadEvent", "GestureEvent", "HAVE_CURRENT_DATA", "HAVE_ENOUGH_DATA", "HAVE_FUTURE_DATA", "HAVE_METADATA", "HAVE_NOTHING", "HEADERS_RECEIVED", "HIDDEN", "HIERARCHY_REQUEST_ERR", "HIGHPASS", "HIGHSHELF", "HIGH_FLOAT", "HIGH_INT", "HORIZONTAL", "HORIZONTAL_AXIS", "HRTF", "HTMLAllCollection", "HTMLAnchorElement", "HTMLAppletElement", "HTMLAreaElement", "HTMLAudioElement", "HTMLBRElement", "HTMLBaseElement", "HTMLBaseFontElement", "HTMLBlockquoteElement", "HTMLBodyElement", "HTMLButtonElement", "HTMLCanvasElement", "HTMLCollection", "HTMLCommandElement", "HTMLContentElement", "HTMLDListElement", "HTMLDataElement", "HTMLDataListElement", "HTMLDetailsElement", "HTMLDialogElement", "HTMLDirectoryElement", "HTMLDivElement", "HTMLDocument", "HTMLElement", "HTMLEmbedElement", "HTMLFieldSetElement", "HTMLFontElement", "HTMLFormControlsCollection", "HTMLFormElement", "HTMLFrameElement", "HTMLFrameSetElement", "HTMLHRElement", "HTMLHeadElement", "HTMLHeadingElement", "HTMLHtmlElement", "HTMLIFrameElement", "HTMLImageElement", "HTMLInputElement", "HTMLIsIndexElement", "HTMLKeygenElement", "HTMLLIElement", "HTMLLabelElement", "HTMLLegendElement", "HTMLLinkElement", "HTMLMapElement", "HTMLMarqueeElement", "HTMLMediaElement", "HTMLMenuElement", "HTMLMenuItemElement", "HTMLMetaElement", "HTMLMeterElement", "HTMLModElement", "HTMLOListElement", "HTMLObjectElement", "HTMLOptGroupElement", "HTMLOptionElement", "HTMLOptionsCollection", "HTMLOutputElement", "HTMLParagraphElement", "HTMLParamElement", "HTMLPictureElement", "HTMLPreElement", "HTMLProgressElement", "HTMLPropertiesCollection", "HTMLQuoteElement", "HTMLScriptElement", "HTMLSelectElement", "HTMLShadowElement", "HTMLSourceElement", "HTMLSpanElement", "HTMLStyleElement", "HTMLTableCaptionElement", "HTMLTableCellElement", "HTMLTableColElement", "HTMLTableElement", "HTMLTableRowElement", "HTMLTableSectionElement", "HTMLTemplateElement", "HTMLTextAreaElement", "HTMLTimeElement", "HTMLTitleElement", "HTMLTrackElement", "HTMLUListElement", "HTMLUnknownElement", "HTMLVideoElement", "HashChangeEvent", "Headers", "History", "ICE_CHECKING", "ICE_CLOSED", "ICE_COMPLETED", "ICE_CONNECTED", "ICE_FAILED", "ICE_GATHERING", "ICE_WAITING", "IDBCursor", "IDBCursorWithValue", "IDBDatabase", "IDBDatabaseException", "IDBFactory", "IDBFileHandle", "IDBFileRequest", "IDBIndex", "IDBKeyRange", "IDBMutableFile", "IDBObjectStore", "IDBOpenDBRequest", "IDBRequest", "IDBTransaction", "IDBVersionChangeEvent", "IDLE", "IMPLEMENTATION_COLOR_READ_FORMAT", "IMPLEMENTATION_COLOR_READ_TYPE", "IMPORT_RULE", "INCR", "INCR_WRAP", "INDEX_SIZE_ERR", "INT", "INT_VEC2", "INT_VEC3", "INT_VEC4", "INUSE_ATTRIBUTE_ERR", "INVALID_ACCESS_ERR", "INVALID_CHARACTER_ERR", "INVALID_ENUM", "INVALID_EXPRESSION_ERR", "INVALID_FRAMEBUFFER_OPERATION", "INVALID_MODIFICATION_ERR", "INVALID_NODE_TYPE_ERR", "INVALID_OPERATION", "INVALID_STATE_ERR", "INVALID_VALUE", "INVERSE_DISTANCE", "INVERT", "IceCandidate", "Image", "ImageBitmap", "ImageData", "Infinity", "InputEvent", "InputMethodContext", "InstallTrigger", "Int16Array", "Int32Array", "Int8Array", "Intent", "InternalError", "Intl", "IsSearchProviderInstalled", "Iterator", "JSON", "KEEP", "KEYDOWN", "KEYFRAMES_RULE", "KEYFRAME_RULE", "KEYPRESS", "KEYUP", "KeyEvent", "KeyboardEvent", "LENGTHADJUST_SPACING", "LENGTHADJUST_SPACINGANDGLYPHS", "LENGTHADJUST_UNKNOWN", "LEQUAL", "LESS", "LINEAR", "LINEAR_DISTANCE", "LINEAR_MIPMAP_LINEAR", "LINEAR_MIPMAP_NEAREST", "LINES", "LINE_LOOP", "LINE_STRIP", "LINE_WIDTH", "LINK_STATUS", "LIVE", "LN10", "LN2", "LOADED", "LOADING", "LOG10E", "LOG2E", "LOWPASS", "LOWSHELF", "LOW_FLOAT", "LOW_INT", "LSException", "LSParserFilter", "LUMINANCE", "LUMINANCE_ALPHA", "LocalMediaStream", "Location", "MAX_COMBINED_TEXTURE_IMAGE_UNITS", "MAX_CUBE_MAP_TEXTURE_SIZE", "MAX_FRAGMENT_UNIFORM_VECTORS", "MAX_RENDERBUFFER_SIZE", "MAX_SAFE_INTEGER", "MAX_TEXTURE_IMAGE_UNITS", "MAX_TEXTURE_MAX_ANISOTROPY_EXT", "MAX_TEXTURE_SIZE", "MAX_VALUE", "MAX_VARYING_VECTORS", "MAX_VERTEX_ATTRIBS", "MAX_VERTEX_TEXTURE_IMAGE_UNITS", "MAX_VERTEX_UNIFORM_VECTORS", "MAX_VIEWPORT_DIMS", "MEDIA_ERR_ABORTED", "MEDIA_ERR_DECODE", "MEDIA_ERR_ENCRYPTED", "MEDIA_ERR_NETWORK", "MEDIA_ERR_SRC_NOT_SUPPORTED", "MEDIA_KEYERR_CLIENT", "MEDIA_KEYERR_DOMAIN", "MEDIA_KEYERR_HARDWARECHANGE", "MEDIA_KEYERR_OUTPUT", "MEDIA_KEYERR_SERVICE", "MEDIA_KEYERR_UNKNOWN", "MEDIA_RULE", "MEDIUM_FLOAT", "MEDIUM_INT", "META_MASK", "MIN_SAFE_INTEGER", "MIN_VALUE", "MIRRORED_REPEAT", "MODE_ASYNCHRONOUS", "MODE_SYNCHRONOUS", "MODIFICATION", "MOUSEDOWN", "MOUSEDRAG", "MOUSEMOVE", "MOUSEOUT", "MOUSEOVER", "MOUSEUP", "MOZ_KEYFRAMES_RULE", "MOZ_KEYFRAME_RULE", "MOZ_SOURCE_CURSOR", "MOZ_SOURCE_ERASER", "MOZ_SOURCE_KEYBOARD", "MOZ_SOURCE_MOUSE", "MOZ_SOURCE_PEN", "MOZ_SOURCE_TOUCH", "MOZ_SOURCE_UNKNOWN", "MSGESTURE_FLAG_BEGIN", "MSGESTURE_FLAG_CANCEL", "MSGESTURE_FLAG_END", "MSGESTURE_FLAG_INERTIA", "MSGESTURE_FLAG_NONE", "MSPOINTER_TYPE_MOUSE", "MSPOINTER_TYPE_PEN", "MSPOINTER_TYPE_TOUCH", "MS_ASYNC_CALLBACK_STATUS_ASSIGN_DELEGATE", "MS_ASYNC_CALLBACK_STATUS_CANCEL", "MS_ASYNC_CALLBACK_STATUS_CHOOSEANY", "MS_ASYNC_CALLBACK_STATUS_ERROR", "MS_ASYNC_CALLBACK_STATUS_JOIN", "MS_ASYNC_OP_STATUS_CANCELED", "MS_ASYNC_OP_STATUS_ERROR", "MS_ASYNC_OP_STATUS_SUCCESS", "MS_MANIPULATION_STATE_ACTIVE", "MS_MANIPULATION_STATE_CANCELLED", "MS_MANIPULATION_STATE_COMMITTED", "MS_MANIPULATION_STATE_DRAGGING", "MS_MANIPULATION_STATE_INERTIA", "MS_MANIPULATION_STATE_PRESELECT", "MS_MANIPULATION_STATE_SELECTING", "MS_MANIPULATION_STATE_STOPPED", "MS_MEDIA_ERR_ENCRYPTED", "MS_MEDIA_KEYERR_CLIENT", "MS_MEDIA_KEYERR_DOMAIN", "MS_MEDIA_KEYERR_HARDWARECHANGE", "MS_MEDIA_KEYERR_OUTPUT", "MS_MEDIA_KEYERR_SERVICE", "MS_MEDIA_KEYERR_UNKNOWN", "Map", "Math", "MediaController", "MediaDevices", "MediaElementAudioSourceNode", "MediaEncryptedEvent", "MediaError", "MediaKeyError", "MediaKeyEvent", "MediaKeyMessageEvent", "MediaKeyNeededEvent", "MediaKeySession", "MediaKeyStatusMap", "MediaKeySystemAccess", "MediaKeys", "MediaList", "MediaQueryList", "MediaQueryListEvent", "MediaRecorder", "MediaSource", "MediaStream", "MediaStreamAudioDestinationNode", "MediaStreamAudioSourceNode", "MediaStreamEvent", "MediaStreamTrack", "MediaStreamTrackEvent", "MessageChannel", "MessageEvent", "MessagePort", "Methods", "MimeType", "MimeTypeArray", "MouseEvent", "MouseScrollEvent", "MozAnimation", "MozAnimationDelay", "MozAnimationDirection", "MozAnimationDuration", "MozAnimationFillMode", "MozAnimationIterationCount", "MozAnimationName", "MozAnimationPlayState", "MozAnimationTimingFunction", "MozAppearance", "MozBackfaceVisibility", "MozBinding", "MozBorderBottomColors", "MozBorderEnd", "MozBorderEndColor", "MozBorderEndStyle", "MozBorderEndWidth", "MozBorderImage", "MozBorderLeftColors", "MozBorderRightColors", "MozBorderStart", "MozBorderStartColor", "MozBorderStartStyle", "MozBorderStartWidth", "MozBorderTopColors", "MozBoxAlign", "MozBoxDirection", "MozBoxFlex", "MozBoxOrdinalGroup", "MozBoxOrient", "MozBoxPack", "MozBoxSizing", "MozCSSKeyframeRule", "MozCSSKeyframesRule", "MozColumnCount", "MozColumnFill", "MozColumnGap", "MozColumnRule", "MozColumnRuleColor", "MozColumnRuleStyle", "MozColumnRuleWidth", "MozColumnWidth", "MozColumns", "MozContactChangeEvent", "MozFloatEdge", "MozFontFeatureSettings", "MozFontLanguageOverride", "MozForceBrokenImageIcon", "MozHyphens", "MozImageRegion", "MozMarginEnd", "MozMarginStart", "MozMmsEvent", "MozMmsMessage", "MozMobileMessageThread", "MozOSXFontSmoothing", "MozOrient", "MozOutlineRadius", "MozOutlineRadiusBottomleft", "MozOutlineRadiusBottomright", "MozOutlineRadiusTopleft", "MozOutlineRadiusTopright", "MozPaddingEnd", "MozPaddingStart", "MozPerspective", "MozPerspectiveOrigin", "MozPowerManager", "MozSettingsEvent", "MozSmsEvent", "MozSmsMessage", "MozStackSizing", "MozTabSize", "MozTextAlignLast", "MozTextDecorationColor", "MozTextDecorationLine", "MozTextDecorationStyle", "MozTextSizeAdjust", "MozTransform", "MozTransformOrigin", "MozTransformStyle", "MozTransition", "MozTransitionDelay", "MozTransitionDuration", "MozTransitionProperty", "MozTransitionTimingFunction", "MozUserFocus", "MozUserInput", "MozUserModify", "MozUserSelect", "MozWindowDragging", "MozWindowShadow", "MutationEvent", "MutationObserver", "MutationRecord", "NAMESPACE_ERR", "NAMESPACE_RULE", "NEAREST", "NEAREST_MIPMAP_LINEAR", "NEAREST_MIPMAP_NEAREST", "NEGATIVE_INFINITY", "NETWORK_EMPTY", "NETWORK_ERR", "NETWORK_IDLE", "NETWORK_LOADED", "NETWORK_LOADING", "NETWORK_NO_SOURCE", "NEVER", "NEW", "NEXT", "NEXT_NO_DUPLICATE", "NICEST", "NODE_AFTER", "NODE_BEFORE", "NODE_BEFORE_AND_AFTER", "NODE_INSIDE", "NONE", "NON_TRANSIENT_ERR", "NOTATION_NODE", "NOTCH", "NOTEQUAL", "NOT_ALLOWED_ERR", "NOT_FOUND_ERR", "NOT_READABLE_ERR", "NOT_SUPPORTED_ERR", "NO_DATA_ALLOWED_ERR", "NO_ERR", "NO_ERROR", "NO_MODIFICATION_ALLOWED_ERR", "NUMBER_TYPE", "NUM_COMPRESSED_TEXTURE_FORMATS", "NaN", "NamedNodeMap", "Navigator", "NearbyLinks", "NetworkInformation", "Node", "NodeFilter", "NodeIterator", "NodeList", "Notation", "Notification", "NotifyPaintEvent", "Number", "NumberFormat", "OBSOLETE", "ONE", "ONE_MINUS_CONSTANT_ALPHA", "ONE_MINUS_CONSTANT_COLOR", "ONE_MINUS_DST_ALPHA", "ONE_MINUS_DST_COLOR", "ONE_MINUS_SRC_ALPHA", "ONE_MINUS_SRC_COLOR", "OPEN", "OPENED", "OPENING", "ORDERED_NODE_ITERATOR_TYPE", "ORDERED_NODE_SNAPSHOT_TYPE", "OUT_OF_MEMORY", "Object", "OfflineAudioCompletionEvent", "OfflineAudioContext", "OfflineResourceList", "Option", "OscillatorNode", "OverflowEvent", "PACK_ALIGNMENT", "PAGE_RULE", "PARSE_ERR", "PATHSEG_ARC_ABS", "PATHSEG_ARC_REL", "PATHSEG_CLOSEPATH", "PATHSEG_CURVETO_CUBIC_ABS", "PATHSEG_CURVETO_CUBIC_REL", "PATHSEG_CURVETO_CUBIC_SMOOTH_ABS", "PATHSEG_CURVETO_CUBIC_SMOOTH_REL", "PATHSEG_CURVETO_QUADRATIC_ABS", "PATHSEG_CURVETO_QUADRATIC_REL", "PATHSEG_CURVETO_QUADRATIC_SMOOTH_ABS", "PATHSEG_CURVETO_QUADRATIC_SMOOTH_REL", "PATHSEG_LINETO_ABS", "PATHSEG_LINETO_HORIZONTAL_ABS", "PATHSEG_LINETO_HORIZONTAL_REL", "PATHSEG_LINETO_REL", "PATHSEG_LINETO_VERTICAL_ABS", "PATHSEG_LINETO_VERTICAL_REL", "PATHSEG_MOVETO_ABS", "PATHSEG_MOVETO_REL", "PATHSEG_UNKNOWN", "PATH_EXISTS_ERR", "PEAKING", "PERMISSION_DENIED", "PERSISTENT", "PI", "PLAYING_STATE", "POINTS", "POLYGON_OFFSET_FACTOR", "POLYGON_OFFSET_FILL", "POLYGON_OFFSET_UNITS", "POSITION_UNAVAILABLE", "POSITIVE_INFINITY", "PREV", "PREV_NO_DUPLICATE", "PROCESSING_INSTRUCTION_NODE", "PageChangeEvent", "PageTransitionEvent", "PaintRequest", "PaintRequestList", "PannerNode", "Path2D", "Performance", "PerformanceEntry", "PerformanceMark", "PerformanceMeasure", "PerformanceNavigation", "PerformanceResourceTiming", "PerformanceTiming", "PeriodicWave", "Plugin", "PluginArray", "PopStateEvent", "PopupBlockedEvent", "ProcessingInstruction", "ProgressEvent", "Promise", "PropertyNodeList", "Proxy", "PushManager", "PushSubscription", "Q", "QUOTA_ERR", "QUOTA_EXCEEDED_ERR", "QueryInterface", "READ_ONLY", "READ_ONLY_ERR", "READ_WRITE", "RED_BITS", "REMOVAL", "RENDERBUFFER", "RENDERBUFFER_ALPHA_SIZE", "RENDERBUFFER_BINDING", "RENDERBUFFER_BLUE_SIZE", "RENDERBUFFER_DEPTH_SIZE", "RENDERBUFFER_GREEN_SIZE", "RENDERBUFFER_HEIGHT", "RENDERBUFFER_INTERNAL_FORMAT", "RENDERBUFFER_RED_SIZE", "RENDERBUFFER_STENCIL_SIZE", "RENDERBUFFER_WIDTH", "RENDERER", "RENDERING_INTENT_ABSOLUTE_COLORIMETRIC", "RENDERING_INTENT_AUTO", "RENDERING_INTENT_PERCEPTUAL", "RENDERING_INTENT_RELATIVE_COLORIMETRIC", "RENDERING_INTENT_SATURATION", "RENDERING_INTENT_UNKNOWN", "REPEAT", "REPLACE", "RGB", "RGB565", "RGB5_A1", "RGBA", "RGBA4", "RGBColor", "ROTATION_CLOCKWISE", "ROTATION_COUNTERCLOCKWISE", "RTCDataChannelEvent", "RTCIceCandidate", "RTCPeerConnectionIceEvent", "RTCRtpReceiver", "RTCRtpSender", "RTCSessionDescription", "RTCStatsReport", "RadioNodeList", "Range", "RangeError", "RangeException", "RecordErrorEvent", "Rect", "ReferenceError", "RegExp", "Request", "Response", "SAMPLER_2D", "SAMPLER_CUBE", "SAMPLES", "SAMPLE_ALPHA_TO_COVERAGE", "SAMPLE_BUFFERS", "SAMPLE_COVERAGE", "SAMPLE_COVERAGE_INVERT", "SAMPLE_COVERAGE_VALUE", "SAWTOOTH", "SCHEDULED_STATE", "SCISSOR_BOX", "SCISSOR_TEST", "SCROLL_PAGE_DOWN", "SCROLL_PAGE_UP", "SDP_ANSWER", "SDP_OFFER", "SDP_PRANSWER", "SECURITY_ERR", "SELECT", "SERIALIZE_ERR", "SEVERITY_ERROR", "SEVERITY_FATAL_ERROR", "SEVERITY_WARNING", "SHADER_COMPILER", "SHADER_TYPE", "SHADING_LANGUAGE_VERSION", "SHIFT_MASK", "SHORT", "SHOWING", "SHOW_ALL", "SHOW_ATTRIBUTE", "SHOW_CDATA_SECTION", "SHOW_COMMENT", "SHOW_DOCUMENT", "SHOW_DOCUMENT_FRAGMENT", "SHOW_DOCUMENT_TYPE", "SHOW_ELEMENT", "SHOW_ENTITY", "SHOW_ENTITY_REFERENCE", "SHOW_NOTATION", "SHOW_PROCESSING_INSTRUCTION", "SHOW_TEXT", "SINE", "SOUNDFIELD", "SQLException", "SQRT1_2", "SQRT2", "SQUARE", "SRC_ALPHA", "SRC_ALPHA_SATURATE", "SRC_COLOR", "START_TO_END", "START_TO_START", "STATIC_DRAW", "STENCIL_ATTACHMENT", "STENCIL_BACK_FAIL", "STENCIL_BACK_FUNC", "STENCIL_BACK_PASS_DEPTH_FAIL", "STENCIL_BACK_PASS_DEPTH_PASS", "STENCIL_BACK_REF", "STENCIL_BACK_VALUE_MASK", "STENCIL_BACK_WRITEMASK", "STENCIL_BITS", "STENCIL_BUFFER_BIT", "STENCIL_CLEAR_VALUE", "STENCIL_FAIL", "STENCIL_FUNC", "STENCIL_INDEX", "STENCIL_INDEX8", "STENCIL_PASS_DEPTH_FAIL", "STENCIL_PASS_DEPTH_PASS", "STENCIL_REF", "STENCIL_TEST", "STENCIL_VALUE_MASK", "STENCIL_WRITEMASK", "STREAM_DRAW", "STRING_TYPE", "STYLE_RULE", "SUBPIXEL_BITS", "SUPPORTS_RULE", "SVGAElement", "SVGAltGlyphDefElement", "SVGAltGlyphElement", "SVGAltGlyphItemElement", "SVGAngle", "SVGAnimateColorElement", "SVGAnimateElement", "SVGAnimateMotionElement", "SVGAnimateTransformElement", "SVGAnimatedAngle", "SVGAnimatedBoolean", "SVGAnimatedEnumeration", "SVGAnimatedInteger", "SVGAnimatedLength", "SVGAnimatedLengthList", "SVGAnimatedNumber", "SVGAnimatedNumberList", "SVGAnimatedPreserveAspectRatio", "SVGAnimatedRect", "SVGAnimatedString", "SVGAnimatedTransformList", "SVGAnimationElement", "SVGCircleElement", "SVGClipPathElement", "SVGColor", "SVGComponentTransferFunctionElement", "SVGCursorElement", "SVGDefsElement", "SVGDescElement", "SVGDiscardElement", "SVGDocument", "SVGElement", "SVGElementInstance", "SVGElementInstanceList", "SVGEllipseElement", "SVGException", "SVGFEBlendElement", "SVGFEColorMatrixElement", "SVGFEComponentTransferElement", "SVGFECompositeElement", "SVGFEConvolveMatrixElement", "SVGFEDiffuseLightingElement", "SVGFEDisplacementMapElement", "SVGFEDistantLightElement", "SVGFEDropShadowElement", "SVGFEFloodElement", "SVGFEFuncAElement", "SVGFEFuncBElement", "SVGFEFuncGElement", "SVGFEFuncRElement", "SVGFEGaussianBlurElement", "SVGFEImageElement", "SVGFEMergeElement", "SVGFEMergeNodeElement", "SVGFEMorphologyElement", "SVGFEOffsetElement", "SVGFEPointLightElement", "SVGFESpecularLightingElement", "SVGFESpotLightElement", "SVGFETileElement", "SVGFETurbulenceElement", "SVGFilterElement", "SVGFontElement", "SVGFontFaceElement", "SVGFontFaceFormatElement", "SVGFontFaceNameElement", "SVGFontFaceSrcElement", "SVGFontFaceUriElement", "SVGForeignObjectElement", "SVGGElement", "SVGGeometryElement", "SVGGlyphElement", "SVGGlyphRefElement", "SVGGradientElement", "SVGGraphicsElement", "SVGHKernElement", "SVGImageElement", "SVGLength", "SVGLengthList", "SVGLineElement", "SVGLinearGradientElement", "SVGMPathElement", "SVGMarkerElement", "SVGMaskElement", "SVGMatrix", "SVGMetadataElement", "SVGMissingGlyphElement", "SVGNumber", "SVGNumberList", "SVGPaint", "SVGPathElement", "SVGPathSeg", "SVGPathSegArcAbs", "SVGPathSegArcRel", "SVGPathSegClosePath", "SVGPathSegCurvetoCubicAbs", "SVGPathSegCurvetoCubicRel", "SVGPathSegCurvetoCubicSmoothAbs", "SVGPathSegCurvetoCubicSmoothRel", "SVGPathSegCurvetoQuadraticAbs", "SVGPathSegCurvetoQuadraticRel", "SVGPathSegCurvetoQuadraticSmoothAbs", "SVGPathSegCurvetoQuadraticSmoothRel", "SVGPathSegLinetoAbs", "SVGPathSegLinetoHorizontalAbs", "SVGPathSegLinetoHorizontalRel", "SVGPathSegLinetoRel", "SVGPathSegLinetoVerticalAbs", "SVGPathSegLinetoVerticalRel", "SVGPathSegList", "SVGPathSegMovetoAbs", "SVGPathSegMovetoRel", "SVGPatternElement", "SVGPoint", "SVGPointList", "SVGPolygonElement", "SVGPolylineElement", "SVGPreserveAspectRatio", "SVGRadialGradientElement", "SVGRect", "SVGRectElement", "SVGRenderingIntent", "SVGSVGElement", "SVGScriptElement", "SVGSetElement", "SVGStopElement", "SVGStringList", "SVGStyleElement", "SVGSwitchElement", "SVGSymbolElement", "SVGTRefElement", "SVGTSpanElement", "SVGTextContentElement", "SVGTextElement", "SVGTextPathElement", "SVGTextPositioningElement", "SVGTitleElement", "SVGTransform", "SVGTransformList", "SVGUnitTypes", "SVGUseElement", "SVGVKernElement", "SVGViewElement", "SVGViewSpec", "SVGZoomAndPan", "SVGZoomEvent", "SVG_ANGLETYPE_DEG", "SVG_ANGLETYPE_GRAD", "SVG_ANGLETYPE_RAD", "SVG_ANGLETYPE_UNKNOWN", "SVG_ANGLETYPE_UNSPECIFIED", "SVG_CHANNEL_A", "SVG_CHANNEL_B", "SVG_CHANNEL_G", "SVG_CHANNEL_R", "SVG_CHANNEL_UNKNOWN", "SVG_COLORTYPE_CURRENTCOLOR", "SVG_COLORTYPE_RGBCOLOR", "SVG_COLORTYPE_RGBCOLOR_ICCCOLOR", "SVG_COLORTYPE_UNKNOWN", "SVG_EDGEMODE_DUPLICATE", "SVG_EDGEMODE_NONE", "SVG_EDGEMODE_UNKNOWN", "SVG_EDGEMODE_WRAP", "SVG_FEBLEND_MODE_COLOR", "SVG_FEBLEND_MODE_COLOR_BURN", "SVG_FEBLEND_MODE_COLOR_DODGE", "SVG_FEBLEND_MODE_DARKEN", "SVG_FEBLEND_MODE_DIFFERENCE", "SVG_FEBLEND_MODE_EXCLUSION", "SVG_FEBLEND_MODE_HARD_LIGHT", "SVG_FEBLEND_MODE_HUE", "SVG_FEBLEND_MODE_LIGHTEN", "SVG_FEBLEND_MODE_LUMINOSITY", "SVG_FEBLEND_MODE_MULTIPLY", "SVG_FEBLEND_MODE_NORMAL", "SVG_FEBLEND_MODE_OVERLAY", "SVG_FEBLEND_MODE_SATURATION", "SVG_FEBLEND_MODE_SCREEN", "SVG_FEBLEND_MODE_SOFT_LIGHT", "SVG_FEBLEND_MODE_UNKNOWN", "SVG_FECOLORMATRIX_TYPE_HUEROTATE", "SVG_FECOLORMATRIX_TYPE_LUMINANCETOALPHA", "SVG_FECOLORMATRIX_TYPE_MATRIX", "SVG_FECOLORMATRIX_TYPE_SATURATE", "SVG_FECOLORMATRIX_TYPE_UNKNOWN", "SVG_FECOMPONENTTRANSFER_TYPE_DISCRETE", "SVG_FECOMPONENTTRANSFER_TYPE_GAMMA", "SVG_FECOMPONENTTRANSFER_TYPE_IDENTITY", "SVG_FECOMPONENTTRANSFER_TYPE_LINEAR", "SVG_FECOMPONENTTRANSFER_TYPE_TABLE", "SVG_FECOMPONENTTRANSFER_TYPE_UNKNOWN", "SVG_FECOMPOSITE_OPERATOR_ARITHMETIC", "SVG_FECOMPOSITE_OPERATOR_ATOP", "SVG_FECOMPOSITE_OPERATOR_IN", "SVG_FECOMPOSITE_OPERATOR_OUT", "SVG_FECOMPOSITE_OPERATOR_OVER", "SVG_FECOMPOSITE_OPERATOR_UNKNOWN", "SVG_FECOMPOSITE_OPERATOR_XOR", "SVG_INVALID_VALUE_ERR", "SVG_LENGTHTYPE_CM", "SVG_LENGTHTYPE_EMS", "SVG_LENGTHTYPE_EXS", "SVG_LENGTHTYPE_IN", "SVG_LENGTHTYPE_MM", "SVG_LENGTHTYPE_NUMBER", "SVG_LENGTHTYPE_PC", "SVG_LENGTHTYPE_PERCENTAGE", "SVG_LENGTHTYPE_PT", "SVG_LENGTHTYPE_PX", "SVG_LENGTHTYPE_UNKNOWN", "SVG_MARKERUNITS_STROKEWIDTH", "SVG_MARKERUNITS_UNKNOWN", "SVG_MARKERUNITS_USERSPACEONUSE", "SVG_MARKER_ORIENT_ANGLE", "SVG_MARKER_ORIENT_AUTO", "SVG_MARKER_ORIENT_UNKNOWN", "SVG_MASKTYPE_ALPHA", "SVG_MASKTYPE_LUMINANCE", "SVG_MATRIX_NOT_INVERTABLE", "SVG_MEETORSLICE_MEET", "SVG_MEETORSLICE_SLICE", "SVG_MEETORSLICE_UNKNOWN", "SVG_MORPHOLOGY_OPERATOR_DILATE", "SVG_MORPHOLOGY_OPERATOR_ERODE", "SVG_MORPHOLOGY_OPERATOR_UNKNOWN", "SVG_PAINTTYPE_CURRENTCOLOR", "SVG_PAINTTYPE_NONE", "SVG_PAINTTYPE_RGBCOLOR", "SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR", "SVG_PAINTTYPE_UNKNOWN", "SVG_PAINTTYPE_URI", "SVG_PAINTTYPE_URI_CURRENTCOLOR", "SVG_PAINTTYPE_URI_NONE", "SVG_PAINTTYPE_URI_RGBCOLOR", "SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR", "SVG_PRESERVEASPECTRATIO_NONE", "SVG_PRESERVEASPECTRATIO_UNKNOWN", "SVG_PRESERVEASPECTRATIO_XMAXYMAX", "SVG_PRESERVEASPECTRATIO_XMAXYMID", "SVG_PRESERVEASPECTRATIO_XMAXYMIN", "SVG_PRESERVEASPECTRATIO_XMIDYMAX", "SVG_PRESERVEASPECTRATIO_XMIDYMID", "SVG_PRESERVEASPECTRATIO_XMIDYMIN", "SVG_PRESERVEASPECTRATIO_XMINYMAX", "SVG_PRESERVEASPECTRATIO_XMINYMID", "SVG_PRESERVEASPECTRATIO_XMINYMIN", "SVG_SPREADMETHOD_PAD", "SVG_SPREADMETHOD_REFLECT", "SVG_SPREADMETHOD_REPEAT", "SVG_SPREADMETHOD_UNKNOWN", "SVG_STITCHTYPE_NOSTITCH", "SVG_STITCHTYPE_STITCH", "SVG_STITCHTYPE_UNKNOWN", "SVG_TRANSFORM_MATRIX", "SVG_TRANSFORM_ROTATE", "SVG_TRANSFORM_SCALE", "SVG_TRANSFORM_SKEWX", "SVG_TRANSFORM_SKEWY", "SVG_TRANSFORM_TRANSLATE", "SVG_TRANSFORM_UNKNOWN", "SVG_TURBULENCE_TYPE_FRACTALNOISE", "SVG_TURBULENCE_TYPE_TURBULENCE", "SVG_TURBULENCE_TYPE_UNKNOWN", "SVG_UNIT_TYPE_OBJECTBOUNDINGBOX", "SVG_UNIT_TYPE_UNKNOWN", "SVG_UNIT_TYPE_USERSPACEONUSE", "SVG_WRONG_TYPE_ERR", "SVG_ZOOMANDPAN_DISABLE", "SVG_ZOOMANDPAN_MAGNIFY", "SVG_ZOOMANDPAN_UNKNOWN", "SYNTAX_ERR", "SavedPages", "Screen", "ScreenOrientation", "Script", "ScriptProcessorNode", "ScrollAreaEvent", "SecurityPolicyViolationEvent", "Selection", "ServiceWorker", "ServiceWorkerContainer", "ServiceWorkerRegistration", "SessionDescription", "Set", "ShadowRoot", "SharedWorker", "SimpleGestureEvent", "SpeechSynthesisEvent", "SpeechSynthesisUtterance", "StopIteration", "Storage", "StorageEvent", "String", "StyleSheet", "StyleSheetList", "SubtleCrypto", "Symbol", "SyntaxError", "TEMPORARY", "TEXTPATH_METHODTYPE_ALIGN", "TEXTPATH_METHODTYPE_STRETCH", "TEXTPATH_METHODTYPE_UNKNOWN", "TEXTPATH_SPACINGTYPE_AUTO", "TEXTPATH_SPACINGTYPE_EXACT", "TEXTPATH_SPACINGTYPE_UNKNOWN", "TEXTURE", "TEXTURE0", "TEXTURE1", "TEXTURE10", "TEXTURE11", "TEXTURE12", "TEXTURE13", "TEXTURE14", "TEXTURE15", "TEXTURE16", "TEXTURE17", "TEXTURE18", "TEXTURE19", "TEXTURE2", "TEXTURE20", "TEXTURE21", "TEXTURE22", "TEXTURE23", "TEXTURE24", "TEXTURE25", "TEXTURE26", "TEXTURE27", "TEXTURE28", "TEXTURE29", "TEXTURE3", "TEXTURE30", "TEXTURE31", "TEXTURE4", "TEXTURE5", "TEXTURE6", "TEXTURE7", "TEXTURE8", "TEXTURE9", "TEXTURE_2D", "TEXTURE_BINDING_2D", "TEXTURE_BINDING_CUBE_MAP", "TEXTURE_CUBE_MAP", "TEXTURE_CUBE_MAP_NEGATIVE_X", "TEXTURE_CUBE_MAP_NEGATIVE_Y", "TEXTURE_CUBE_MAP_NEGATIVE_Z", "TEXTURE_CUBE_MAP_POSITIVE_X", "TEXTURE_CUBE_MAP_POSITIVE_Y", "TEXTURE_CUBE_MAP_POSITIVE_Z", "TEXTURE_MAG_FILTER", "TEXTURE_MAX_ANISOTROPY_EXT", "TEXTURE_MIN_FILTER", "TEXTURE_WRAP_S", "TEXTURE_WRAP_T", "TEXT_NODE", "TIMEOUT", "TIMEOUT_ERR", "TOO_LARGE_ERR", "TRANSACTION_INACTIVE_ERR", "TRIANGLE", "TRIANGLES", "TRIANGLE_FAN", "TRIANGLE_STRIP", "TYPE_BACK_FORWARD", "TYPE_ERR", "TYPE_MISMATCH_ERR", "TYPE_NAVIGATE", "TYPE_RELOAD", "TYPE_RESERVED", "Text", "TextDecoder", "TextEncoder", "TextEvent", "TextMetrics", "TextTrack", "TextTrackCue", "TextTrackCueList", "TextTrackList", "TimeEvent", "TimeRanges", "Touch", "TouchEvent", "TouchList", "TrackEvent", "TransitionEvent", "TreeWalker", "TypeError", "UIEvent", "UNCACHED", "UNKNOWN_ERR", "UNKNOWN_RULE", "UNMASKED_RENDERER_WEBGL", "UNMASKED_VENDOR_WEBGL", "UNORDERED_NODE_ITERATOR_TYPE", "UNORDERED_NODE_SNAPSHOT_TYPE", "UNPACK_ALIGNMENT", "UNPACK_COLORSPACE_CONVERSION_WEBGL", "UNPACK_FLIP_Y_WEBGL", "UNPACK_PREMULTIPLY_ALPHA_WEBGL", "UNSCHEDULED_STATE", "UNSENT", "UNSIGNED_BYTE", "UNSIGNED_INT", "UNSIGNED_SHORT", "UNSIGNED_SHORT_4_4_4_4", "UNSIGNED_SHORT_5_5_5_1", "UNSIGNED_SHORT_5_6_5", "UNSPECIFIED_EVENT_TYPE_ERR", "UPDATEREADY", "URIError", "URL", "URLSearchParams", "URLUnencoded", "URL_MISMATCH_ERR", "UTC", "Uint16Array", "Uint32Array", "Uint8Array", "Uint8ClampedArray", "UserMessageHandler", "UserMessageHandlersNamespace", "UserProximityEvent", "VALIDATE_STATUS", "VALIDATION_ERR", "VARIABLES_RULE", "VENDOR", "VERSION", "VERSION_CHANGE", "VERSION_ERR", "VERTEX_ATTRIB_ARRAY_BUFFER_BINDING", "VERTEX_ATTRIB_ARRAY_DIVISOR_ANGLE", "VERTEX_ATTRIB_ARRAY_ENABLED", "VERTEX_ATTRIB_ARRAY_NORMALIZED", "VERTEX_ATTRIB_ARRAY_POINTER", "VERTEX_ATTRIB_ARRAY_SIZE", "VERTEX_ATTRIB_ARRAY_STRIDE", "VERTEX_ATTRIB_ARRAY_TYPE", "VERTEX_SHADER", "VERTICAL", "VERTICAL_AXIS", "VER_ERR", "VIEWPORT", "VIEWPORT_RULE", "VTTCue", "VTTRegion", "ValidityState", "VideoStreamTrack", "WEBKIT_FILTER_RULE", "WEBKIT_KEYFRAMES_RULE", "WEBKIT_KEYFRAME_RULE", "WEBKIT_REGION_RULE", "WRONG_DOCUMENT_ERR", "WaveShaperNode", "WeakMap", "WeakSet", "WebGLActiveInfo", "WebGLBuffer", "WebGLContextEvent", "WebGLFramebuffer", "WebGLProgram", "WebGLRenderbuffer", "WebGLRenderingContext", "WebGLShader", "WebGLShaderPrecisionFormat", "WebGLTexture", "WebGLUniformLocation", "WebGLVertexArray", "WebKitAnimationEvent", "WebKitBlobBuilder", "WebKitCSSFilterRule", "WebKitCSSFilterValue", "WebKitCSSKeyframeRule", "WebKitCSSKeyframesRule", "WebKitCSSMatrix", "WebKitCSSRegionRule", "WebKitCSSTransformValue", "WebKitDataCue", "WebKitGamepad", "WebKitMediaKeyError", "WebKitMediaKeyMessageEvent", "WebKitMediaKeySession", "WebKitMediaKeys", "WebKitMediaSource", "WebKitMutationObserver", "WebKitNamespace", "WebKitPlaybackTargetAvailabilityEvent", "WebKitPoint", "WebKitShadowRoot", "WebKitSourceBuffer", "WebKitSourceBufferList", "WebKitTransitionEvent", "WebSocket", "WheelEvent", "Window", "Worker", "XMLDocument", "XMLHttpRequest", "XMLHttpRequestEventTarget", "XMLHttpRequestException", "XMLHttpRequestProgressEvent", "XMLHttpRequestUpload", "XMLSerializer", "XMLStylesheetProcessingInstruction", "XPathEvaluator", "XPathException", "XPathExpression", "XPathNSResolver", "XPathResult", "XSLTProcessor", "ZERO", "_XD0M_", "_YD0M_", "__defineGetter__", "__defineSetter__", "__lookupGetter__", "__lookupSetter__", "__opera", "__proto__", "_browserjsran", "a", "aLink", "abbr", "abort", "abs", "absolute", "acceleration", "accelerationIncludingGravity", "accelerator", "accept", "acceptCharset", "acceptNode", "accessKey", "accessKeyLabel", "accuracy", "acos", "acosh", "action", "actionURL", "active", "activeCues", "activeElement", "activeSourceBuffers", "activeSourceCount", "activeTexture", "add", "addBehavior", "addCandidate", "addColorStop", "addCue", "addElement", "addEventListener", "addFilter", "addFromString", "addFromUri", "addIceCandidate", "addImport", "addListener", "addNamed", "addPageRule", "addPath", "addPointer", "addRange", "addRegion", "addRule", "addSearchEngine", "addSourceBuffer", "addStream", "addTextTrack", "addTrack", "addWakeLockListener", "addedNodes", "additionalName", "additiveSymbols", "addons", "adoptNode", "adr", "advance", "alert", "algorithm", "align", "align-content", "align-items", "align-self", "alignContent", "alignItems", "alignSelf", "alignmentBaseline", "alinkColor", "all", "allSettled", "allowFullscreen", "allowedDirections", "alpha", "alt", "altGraphKey", "altHtml", "altKey", "altLeft", "altitude", "altitudeAccuracy", "amplitude", "ancestorOrigins", "anchor", "anchorNode", "anchorOffset", "anchors", "angle", "animVal", "animate", "animatedInstanceRoot", "animatedNormalizedPathSegList", "animatedPathSegList", "animatedPoints", "animation", "animation-delay", "animation-direction", "animation-duration", "animation-fill-mode", "animation-iteration-count", "animation-name", "animation-play-state", "animation-timing-function", "animationDelay", "animationDirection", "animationDuration", "animationFillMode", "animationIterationCount", "animationName", "animationPlayState", "animationStartTime", "animationTimingFunction", "animationsPaused", "anniversary", "any", "app", "appCodeName", "appMinorVersion", "appName", "appNotifications", "appVersion", "append", "appendBuffer", "appendChild", "appendData", "appendItem", "appendMedium", "appendNamed", "appendRule", "appendStream", "appendWindowEnd", "appendWindowStart", "applets", "applicationCache", "apply", "applyElement", "arc", "arcTo", "archive", "areas", "arguments", "arrayBuffer", "asin", "asinh", "assert", "assign", "async", "atEnd", "atan", "atan2", "atanh", "atob", "attachEvent", "attachShader", "attachShadow", "attachments", "attack", "attrChange", "attrName", "attributeFilter", "attributeName", "attributeNamespace", "attributeOldValue", "attributes", "audioTracks", "autoIncrement", "autobuffer", "autocapitalize", "autocomplete", "autocorrect", "autofocus", "autoplay", "availHeight", "availLeft", "availTop", "availWidth", "availability", "available", "aversion", "axes", "axis", "azimuth", "b", "back", "backface-visibility", "backfaceVisibility", "background", "background-attachment", "background-blend-mode", "background-clip", "background-color", "background-image", "background-origin", "background-position", "background-repeat", "background-size", "backgroundAttachment", "backgroundBlendMode", "backgroundClip", "backgroundColor", "backgroundImage", "backgroundOrigin", "backgroundPosition", "backgroundPositionX", "backgroundPositionY", "backgroundRepeat", "backgroundSize", "badInput", "balance", "baseFrequencyX", "baseFrequencyY", "baseNode", "baseOffset", "baseURI", "baseVal", "baselineShift", "battery", "bday", "beginElement", "beginElementAt", "beginPath", "behavior", "behaviorCookie", "behaviorPart", "behaviorUrns", "beta", "bezierCurveTo", "bgColor", "bgProperties", "bias", "big", "binaryType", "bind", "bindAttribLocation", "bindBuffer", "bindFramebuffer", "bindRenderbuffer", "bindTexture", "blendColor", "blendEquation", "blendEquationSeparate", "blendFunc", "blendFuncSeparate", "blink", "blob", "blockDirection", "blue", "blur", "body", "bodyUsed", "bold", "bookmarks", "booleanValue", "border", "border-bottom", "border-bottom-color", "border-bottom-left-radius", "border-bottom-right-radius", "border-bottom-style", "border-bottom-width", "border-collapse", "border-color", "border-image", "border-image-outset", "border-image-repeat", "border-image-slice", "border-image-source", "border-image-width", "border-left", "border-left-color", "border-left-style", "border-left-width", "border-radius", "border-right", "border-right-color", "border-right-style", "border-right-width", "border-spacing", "border-style", "border-top", "border-top-color", "border-top-left-radius", "border-top-right-radius", "border-top-style", "border-top-width", "border-width", "borderBottom", "borderBottomColor", "borderBottomLeftRadius", "borderBottomRightRadius", "borderBottomStyle", "borderBottomWidth", "borderCollapse", "borderColor", "borderColorDark", "borderColorLight", "borderImage", "borderImageOutset", "borderImageRepeat", "borderImageSlice", "borderImageSource", "borderImageWidth", "borderLeft", "borderLeftColor", "borderLeftStyle", "borderLeftWidth", "borderRadius", "borderRight", "borderRightColor", "borderRightStyle", "borderRightWidth", "borderSpacing", "borderStyle", "borderTop", "borderTopColor", "borderTopLeftRadius", "borderTopRightRadius", "borderTopStyle", "borderTopWidth", "borderWidth", "bottom", "bottomMargin", "bound", "boundElements", "boundingClientRect", "boundingHeight", "boundingLeft", "boundingTop", "boundingWidth", "bounds", "box-decoration-break", "box-shadow", "box-sizing", "boxDecorationBreak", "boxShadow", "boxSizing", "breakAfter", "breakBefore", "breakInside", "browserLanguage", "btoa", "bubbles", "buffer", "bufferData", "bufferDepth", "bufferSize", "bufferSubData", "buffered", "bufferedAmount", "buildID", "buildNumber", "button", "buttonID", "buttons", "byteLength", "byteOffset", "c", "call", "caller", "canBeFormatted", "canBeMounted", "canBeShared", "canHaveChildren", "canHaveHTML", "canPlayType", "cancel", "cancelAnimationFrame", "cancelBubble", "cancelScheduledValues", "cancelable", "candidate", "canvas", "caption", "caption-side", "captionSide", "capture", "captureEvents", "captureStackTrace", "caretPositionFromPoint", "caretRangeFromPoint", "cast", "catch", "category", "cbrt", "cd", "ceil", "cellIndex", "cellPadding", "cellSpacing", "cells", "ch", "chOff", "chain", "challenge", "changedTouches", "channel", "channelCount", "channelCountMode", "channelInterpretation", "char", "charAt", "charCode", "charCodeAt", "charIndex", "characterData", "characterDataOldValue", "characterSet", "charging", "chargingTime", "charset", "checkEnclosure", "checkFramebufferStatus", "checkIntersection", "checkValidity", "checked", "childElementCount", "childList", "childNodes", "children", "chrome", "ciphertext", "cite", "classList", "className", "classid", "clear", "clearAttributes", "clearColor", "clearData", "clearDepth", "clearImmediate", "clearInterval", "clearMarks", "clearMeasures", "clearParameters", "clearRect", "clearResourceTimings", "clearShadow", "clearStencil", "clearTimeout", "clearWatch", "click", "clickCount", "clientHeight", "clientInformation", "clientLeft", "clientRect", "clientRects", "clientTop", "clientWidth", "clientX", "clientY", "clip", "clip-path", "clip-rule", "clipBottom", "clipLeft", "clipPath", "clipPathUnits", "clipRight", "clipRule", "clipTop", "clipboardData", "clone", "cloneContents", "cloneNode", "cloneRange", "close", "closePath", "closed", "closest", "clz", "clz32", "cmp", "code", "codeBase", "codePointAt", "codeType", "colSpan", "collapse", "collapseToEnd", "collapseToStart", "collapsed", "collect", "colno", "color", "color-interpolation", "color-interpolation-filters", "colorDepth", "colorInterpolation", "colorInterpolationFilters", "colorMask", "colorType", "cols", "columnCount", "columnFill", "columnGap", "columnNumber", "columnRule", "columnRuleColor", "columnRuleStyle", "columnRuleWidth", "columnSpan", "columnWidth", "columns", "command", "commitPreferences", "commonAncestorContainer", "compact", "compareBoundaryPoints", "compareDocumentPosition", "compareEndPoints", "compareNode", "comparePoint", "compatMode", "compatible", "compile", "compileShader", "complete", "componentFromPoint", "compositionEndOffset", "compositionStartOffset", "compressedTexImage2D", "compressedTexSubImage2D", "concat", "conditionText", "coneInnerAngle", "coneOuterAngle", "coneOuterGain", "confirm", "confirmComposition", "confirmSiteSpecificTrackingException", "confirmWebWideTrackingException", "connect", "connectEnd", "connectStart", "connected", "connection", "connectionSpeed", "console", "consolidate", "constrictionActive", "constructor", "contactID", "contains", "containsNode", "content", "contentDocument", "contentEditable", "contentOverflow", "contentScriptType", "contentStyleType", "contentType", "contentWindow", "context", "contextMenu", "contextmenu", "continue", "continuous", "control", "controller", "controls", "convertToSpecifiedUnits", "cookie", "cookieEnabled", "coords", "copyFromChannel", "copyTexImage2D", "copyTexSubImage2D", "copyToChannel", "copyWithin", "correspondingElement", "correspondingUseElement", "cos", "cosh", "count", "counter-increment", "counter-reset", "counterIncrement", "counterReset", "cpuClass", "cpuSleepAllowed", "create", "createAnalyser", "createAnswer", "createAttribute", "createAttributeNS", "createBiquadFilter", "createBuffer", "createBufferSource", "createCDATASection", "createCSSStyleSheet", "createCaption", "createChannelMerger", "createChannelSplitter", "createComment", "createContextualFragment", "createControlRange", "createConvolver", "createDTMFSender", "createDataChannel", "createDelay", "createDelayNode", "createDocument", "createDocumentFragment", "createDocumentType", "createDynamicsCompressor", "createElement", "createElementNS", "createEntityReference", "createEvent", "createEventObject", "createExpression", "createFramebuffer", "createFunction", "createGain", "createGainNode", "createHTMLDocument", "createImageBitmap", "createImageData", "createIndex", "createJavaScriptNode", "createLinearGradient", "createMediaElementSource", "createMediaKeys", "createMediaStreamDestination", "createMediaStreamSource", "createMutableFile", "createNSResolver", "createNodeIterator", "createNotification", "createObjectStore", "createObjectURL", "createOffer", "createOscillator", "createPanner", "createPattern", "createPeriodicWave", "createPopup", "createProcessingInstruction", "createProgram", "createRadialGradient", "createRange", "createRangeCollection", "createRenderbuffer", "createSVGAngle", "createSVGLength", "createSVGMatrix", "createSVGNumber", "createSVGPathSegArcAbs", "createSVGPathSegArcRel", "createSVGPathSegClosePath", "createSVGPathSegCurvetoCubicAbs", "createSVGPathSegCurvetoCubicRel", "createSVGPathSegCurvetoCubicSmoothAbs", "createSVGPathSegCurvetoCubicSmoothRel", "createSVGPathSegCurvetoQuadraticAbs", "createSVGPathSegCurvetoQuadraticRel", "createSVGPathSegCurvetoQuadraticSmoothAbs", "createSVGPathSegCurvetoQuadraticSmoothRel", "createSVGPathSegLinetoAbs", "createSVGPathSegLinetoHorizontalAbs", "createSVGPathSegLinetoHorizontalRel", "createSVGPathSegLinetoRel", "createSVGPathSegLinetoVerticalAbs", "createSVGPathSegLinetoVerticalRel", "createSVGPathSegMovetoAbs", "createSVGPathSegMovetoRel", "createSVGPoint", "createSVGRect", "createSVGTransform", "createSVGTransformFromMatrix", "createScriptProcessor", "createSession", "createShader", "createShadowRoot", "createStereoPanner", "createStyleSheet", "createTBody", "createTFoot", "createTHead", "createTextNode", "createTextRange", "createTexture", "createTouch", "createTouchList", "createTreeWalker", "createWaveShaper", "creationTime", "crossOrigin", "crypto", "csi", "cssFloat", "cssRules", "cssText", "cssValueType", "ctrlKey", "ctrlLeft", "cues", "cullFace", "currentNode", "currentPage", "currentScale", "currentScript", "currentSrc", "currentState", "currentStyle", "currentTarget", "currentTime", "currentTranslate", "currentView", "cursor", "curve", "customError", "cx", "cy", "d", "data", "dataFld", "dataFormatAs", "dataPageSize", "dataSrc", "dataTransfer", "database", "dataset", "dateTime", "db", "debug", "debuggerEnabled", "declare", "decode", "decodeAudioData", "decodeURI", "decodeURIComponent", "decodingInfo", "decrypt", "default", "defaultCharset", "defaultChecked", "defaultMuted", "defaultPlaybackRate", "defaultPrevented", "defaultSelected", "defaultStatus", "defaultURL", "defaultValue", "defaultView", "defaultstatus", "defer", "defineMagicFunction", "defineMagicVariable", "defineProperties", "defineProperty", "delayTime", "delete", "deleteBuffer", "deleteCaption", "deleteCell", "deleteContents", "deleteData", "deleteDatabase", "deleteFramebuffer", "deleteFromDocument", "deleteIndex", "deleteMedium", "deleteObjectStore", "deleteProgram", "deleteRenderbuffer", "deleteRow", "deleteRule", "deleteShader", "deleteTFoot", "deleteTHead", "deleteTexture", "deliverChangeRecords", "delivery", "deliveryInfo", "deliveryStatus", "deliveryTimestamp", "delta", "deltaMode", "deltaX", "deltaY", "deltaZ", "depthFunc", "depthMask", "depthRange", "deriveBits", "deriveKey", "description", "deselectAll", "designMode", "destination", "destinationURL", "detach", "detachEvent", "detachShader", "detail", "detune", "devicePixelRatio", "deviceXDPI", "deviceYDPI", "diffuseConstant", "digest", "dimensions", "dir", "dirName", "direction", "dirxml", "disable", "disableVertexAttribArray", "disabled", "dischargingTime", "disconnect", "dispatchEvent", "display", "distanceModel", "divisor", "djsapi", "djsproxy", "doImport", "doNotTrack", "doScroll", "doctype", "document", "documentElement", "documentMode", "documentURI", "dolphin", "dolphinGameCenter", "dolphininfo", "dolphinmeta", "domComplete", "domContentLoadedEventEnd", "domContentLoadedEventStart", "domInteractive", "domLoading", "domain", "domainLookupEnd", "domainLookupStart", "dominant-baseline", "dominantBaseline", "done", "dopplerFactor", "download", "dragDrop", "draggable", "drawArrays", "drawArraysInstancedANGLE", "drawCustomFocusRing", "drawElements", "drawElementsInstancedANGLE", "drawFocusIfNeeded", "drawImage", "drawImageFromRect", "drawSystemFocusRing", "drawingBufferHeight", "drawingBufferWidth", "dropEffect", "droppedVideoFrames", "dropzone", "dump", "duplicate", "duration", "dvname", "dvnum", "dx", "dy", "dynsrc", "e", "edgeMode", "effectAllowed", "elapsedTime", "elementFromPoint", "elements", "elevation", "ellipse", "email", "embeds", "empty", "empty-cells", "emptyCells", "enable", "enableBackground", "enableStyleSheetsForSet", "enableVertexAttribArray", "enabled", "enabledPlugin", "encode", "encodeURI", "encodeURIComponent", "encoding", "encrypt", "enctype", "end", "endContainer", "endElement", "endElementAt", "endOfStream", "endOffset", "endTime", "ended", "endsWith", "entities", "entries", "entryType", "enumerate", "enumerateEditable", "error", "errorCode", "escape", "eval", "evaluate", "event", "eventPhase", "every", "exception", "exec", "execCommand", "execCommandShowHelp", "execScript", "exitFullscreen", "exitPointerLock", "exp", "expand", "expandEntityReferences", "expando", "expansion", "expiryDate", "explicitOriginalTarget", "expm1", "exponent", "exponentialRampToValueAtTime", "exportKey", "extend", "extensions", "extentNode", "extentOffset", "external", "externalResourcesRequired", "extractContents", "extractable", "f", "face", "factoryReset", "fallback", "familyName", "farthestViewportElement", "fastSeek", "fatal", "fetch", "fetchStart", "fftSize", "fgColor", "fileCreatedDate", "fileHandle", "fileModifiedDate", "fileName", "fileSize", "fileUpdatedDate", "filename", "files", "fill", "fill-opacity", "fill-rule", "fillOpacity", "fillRect", "fillRule", "fillStyle", "fillText", "filter", "filterResX", "filterResY", "filterUnits", "filters", "finally", "find", "findIndex", "findRule", "findText", "finish", "fireEvent", "firstChild", "firstElementChild", "firstPage", "fixed", "flex", "flex-basis", "flex-direction", "flex-flow", "flex-grow", "flex-shrink", "flex-wrap", "flexBasis", "flexDirection", "flexFlow", "flexGrow", "flexShrink", "flexWrap", "flipX", "flipY", "float", "flood-color", "flood-opacity", "floodColor", "floodOpacity", "floor", "flush", "focus", "focusNode", "focusOffset", "font", "font-family", "font-feature-settings", "font-kerning", "font-language-override", "font-size", "font-size-adjust", "font-stretch", "font-style", "font-synthesis", "font-variant", "font-variant-alternates", "font-variant-caps", "font-variant-east-asian", "font-variant-ligatures", "font-variant-numeric", "font-variant-position", "font-weight", "fontFamily", "fontFeatureSettings", "fontKerning", "fontLanguageOverride", "fontSize", "fontSizeAdjust", "fontSmoothingEnabled", "fontStretch", "fontStyle", "fontSynthesis", "fontVariant", "fontVariantAlternates", "fontVariantCaps", "fontVariantEastAsian", "fontVariantLigatures", "fontVariantNumeric", "fontVariantPosition", "fontWeight", "fontcolor", "fonts", "fontsize", "for", "forEach", "forceRedraw", "form", "formAction", "formEnctype", "formMethod", "formNoValidate", "formTarget", "format", "formatToParts", "forms", "forward", "fr", "frame", "frameBorder", "frameElement", "frameSpacing", "framebufferRenderbuffer", "framebufferTexture2D", "frames", "freeSpace", "freeze", "frequency", "frequencyBinCount", "from", "fromCharCode", "fromCodePoint", "fromElement", "frontFace", "fround", "fullScreen", "fullscreenElement", "fullscreenEnabled", "fx", "fy", "gain", "gamepad", "gamma", "genderIdentity", "generateKey", "generateMipmap", "generateRequest", "geolocation", "gestureObject", "get", "getActiveAttrib", "getActiveUniform", "getAdjacentText", "getAll", "getAllResponseHeaders", "getAsFile", "getAsString", "getAttachedShaders", "getAttribLocation", "getAttribute", "getAttributeNS", "getAttributeNode", "getAttributeNodeNS", "getAudioTracks", "getBBox", "getBattery", "getBlob", "getBookmark", "getBoundingClientRect", "getBufferParameter", "getByteFrequencyData", "getByteTimeDomainData", "getCSSCanvasContext", "getCTM", "getCandidateWindowClientRect", "getChannelData", "getCharNumAtPosition", "getClientRect", "getClientRects", "getCompositionAlternatives", "getComputedStyle", "getComputedTextLength", "getConfiguration", "getContext", "getContextAttributes", "getCounterValue", "getCueAsHTML", "getCueById", "getCurrentPosition", "getCurrentTime", "getData", "getDatabaseNames", "getDate", "getDay", "getDefaultComputedStyle", "getDestinationInsertionPoints", "getDistributedNodes", "getEditable", "getElementById", "getElementsByClassName", "getElementsByName", "getElementsByTagName", "getElementsByTagNameNS", "getEnclosureList", "getEndPositionOfChar", "getEntries", "getEntriesByName", "getEntriesByType", "getError", "getExtension", "getExtentOfChar", "getFeature", "getFile", "getFloat32", "getFloat64", "getFloatFrequencyData", "getFloatTimeDomainData", "getFloatValue", "getFramebufferAttachmentParameter", "getFrequencyResponse", "getFullYear", "getGamepads", "getHours", "getImageData", "getInt16", "getInt32", "getInt8", "getIntersectionList", "getItem", "getItems", "getKey", "getLineDash", "getLocalStreams", "getMarks", "getMatchedCSSRules", "getMeasures", "getMetadata", "getMilliseconds", "getMinutes", "getModifierState", "getMonth", "getNamedItem", "getNamedItemNS", "getNotifier", "getNumberOfChars", "getOverrideHistoryNavigationMode", "getOverrideStyle", "getOwnPropertyDescriptor", "getOwnPropertyNames", "getOwnPropertySymbols", "getParameter", "getPathSegAtLength", "getPointAtLength", "getPreference", "getPreferenceDefault", "getPresentationAttribute", "getPreventDefault", "getProgramInfoLog", "getProgramParameter", "getPropertyCSSValue", "getPropertyPriority", "getPropertyShorthand", "getPropertyValue", "getPrototypeOf", "getRGBColorValue", "getRandomValues", "getRangeAt", "getReceivers", "getRectValue", "getRegistration", "getRemoteStreams", "getRenderbufferParameter", "getResponseHeader", "getRoot", "getRotationOfChar", "getSVGDocument", "getScreenCTM", "getSeconds", "getSelection", "getSenders", "getShaderInfoLog", "getShaderParameter", "getShaderPrecisionFormat", "getShaderSource", "getSimpleDuration", "getSiteIcons", "getSources", "getSpeculativeParserUrls", "getStartPositionOfChar", "getStartTime", "getStats", "getStorageUpdates", "getStreamById", "getStringValue", "getSubStringLength", "getSubscription", "getSupportedExtensions", "getTexParameter", "getTime", "getTimezoneOffset", "getTotalLength", "getTrackById", "getTracks", "getTransformToElement", "getUTCDate", "getUTCDay", "getUTCFullYear", "getUTCHours", "getUTCMilliseconds", "getUTCMinutes", "getUTCMonth", "getUTCSeconds", "getUint16", "getUint32", "getUint8", "getUniform", "getUniformLocation", "getUserMedia", "getValues", "getVarDate", "getVariableValue", "getVertexAttrib", "getVertexAttribOffset", "getVideoPlaybackQuality", "getVideoTracks", "getWakeLockState", "getYear", "givenName", "global", "globalAlpha", "globalCompositeOperation", "glyphOrientationHorizontal", "glyphOrientationVertical", "glyphRef", "go", "gradientTransform", "gradientUnits", "grammars", "green", "group", "groupCollapsed", "groupEnd", "hardwareConcurrency", "has", "hasAttribute", "hasAttributeNS", "hasAttributes", "hasChildNodes", "hasComposition", "hasExtension", "hasFeature", "hasFocus", "hasLayout", "hasOwnProperty", "hash", "head", "headers", "heading", "height", "hidden", "hide", "hideFocus", "high", "hint", "history", "honorificPrefix", "honorificSuffix", "horizontalOverflow", "host", "hostname", "href", "hreflang", "hspace", "html5TagCheckInerface", "htmlFor", "htmlText", "httpEquiv", "hwTimestamp", "hypot", "iccId", "iceConnectionState", "iceGatheringState", "icon", "id", "identifier", "identity", "ignoreBOM", "ignoreCase", "image-orientation", "image-rendering", "imageOrientation", "imageRendering", "images", "ime-mode", "imeMode", "implementation", "importKey", "importNode", "importStylesheet", "imports", "impp", "imul", "in1", "in2", "inBandMetadataTrackDispatchType", "inRange", "includes", "incremental", "indeterminate", "index", "indexNames", "indexOf", "indexedDB", "inertiaDestinationX", "inertiaDestinationY", "info", "init", "initAnimationEvent", "initBeforeLoadEvent", "initClipboardEvent", "initCloseEvent", "initCommandEvent", "initCompositionEvent", "initCustomEvent", "initData", "initDeviceMotionEvent", "initDeviceOrientationEvent", "initDragEvent", "initErrorEvent", "initEvent", "initFocusEvent", "initGestureEvent", "initHashChangeEvent", "initKeyEvent", "initKeyboardEvent", "initMSManipulationEvent", "initMessageEvent", "initMouseEvent", "initMouseScrollEvent", "initMouseWheelEvent", "initMutationEvent", "initNSMouseEvent", "initOverflowEvent", "initPageEvent", "initPageTransitionEvent", "initPointerEvent", "initPopStateEvent", "initProgressEvent", "initScrollAreaEvent", "initSimpleGestureEvent", "initStorageEvent", "initTextEvent", "initTimeEvent", "initTouchEvent", "initTransitionEvent", "initUIEvent", "initWebKitAnimationEvent", "initWebKitTransitionEvent", "initWebKitWheelEvent", "initWheelEvent", "initialTime", "initialize", "initiatorType", "inner", "innerHTML", "innerHeight", "innerText", "innerWidth", "input", "inputBuffer", "inputEncoding", "inputMethod", "insertAdjacentElement", "insertAdjacentHTML", "insertAdjacentText", "insertBefore", "insertCell", "insertData", "insertItemBefore", "insertNode", "insertRow", "insertRule", "instanceRoot", "intercept", "interimResults", "internalSubset", "intersectsNode", "interval", "invalidIteratorState", "inverse", "invertSelf", "is", "is2D", "isAlternate", "isArray", "isBingCurrentSearchDefault", "isBuffer", "isCandidateWindowVisible", "isChar", "isCollapsed", "isComposing", "isContentEditable", "isContentHandlerRegistered", "isContextLost", "isDefaultNamespace", "isDisabled", "isEnabled", "isEqual", "isEqualNode", "isExtensible", "isFinite", "isFramebuffer", "isFrozen", "isGenerator", "isId", "isInjected", "isInteger", "isMap", "isMultiLine", "isNaN", "isOpen", "isPointInFill", "isPointInPath", "isPointInRange", "isPointInStroke", "isPrefAlternate", "isPrimary", "isProgram", "isPropertyImplicit", "isProtocolHandlerRegistered", "isPrototypeOf", "isRenderbuffer", "isSafeInteger", "isSameNode", "isSealed", "isShader", "isSupported", "isTextEdit", "isTexture", "isTrusted", "isTypeSupported", "isView", "isolation", "italics", "item", "itemId", "itemProp", "itemRef", "itemScope", "itemType", "itemValue", "iterateNext", "iterator", "javaEnabled", "jobTitle", "join", "json", "justify-content", "justifyContent", "k1", "k2", "k3", "k4", "kernelMatrix", "kernelUnitLengthX", "kernelUnitLengthY", "kerning", "key", "keyCode", "keyFor", "keyIdentifier", "keyLightEnabled", "keyLocation", "keyPath", "keySystem", "keyText", "keyUsage", "keys", "keytype", "kind", "knee", "label", "labels", "lang", "language", "languages", "largeArcFlag", "lastChild", "lastElementChild", "lastEventId", "lastIndex", "lastIndexOf", "lastMatch", "lastMessageSubject", "lastMessageType", "lastModified", "lastModifiedDate", "lastPage", "lastParen", "lastState", "lastStyleSheetSet", "latitude", "layerX", "layerY", "layoutFlow", "layoutGrid", "layoutGridChar", "layoutGridLine", "layoutGridMode", "layoutGridType", "lbound", "left", "leftContext", "leftMargin", "length", "lengthAdjust", "lengthComputable", "letter-spacing", "letterSpacing", "level", "lighting-color", "lightingColor", "limitingConeAngle", "line", "line-height", "lineAlign", "lineBreak", "lineCap", "lineDashOffset", "lineHeight", "lineJoin", "lineNumber", "lineTo", "lineWidth", "linearRampToValueAtTime", "lineno", "link", "linkColor", "linkProgram", "links", "list", "list-style", "list-style-image", "list-style-position", "list-style-type", "listStyle", "listStyleImage", "listStylePosition", "listStyleType", "listener", "load", "loadEventEnd", "loadEventStart", "loadTimes", "loaded", "localDescription", "localName", "localStorage", "locale", "localeCompare", "location", "locationbar", "lock", "lockedFile", "log", "log10", "log1p", "log2", "logicalXDPI", "logicalYDPI", "longDesc", "longitude", "lookupNamespaceURI", "lookupPrefix", "loop", "loopEnd", "loopStart", "looping", "low", "lower", "lowerBound", "lowerOpen", "lowsrc", "m11", "m12", "m13", "m14", "m21", "m22", "m23", "m24", "m31", "m32", "m33", "m34", "m41", "m42", "m43", "m44", "manifest", "map", "mapping", "margin", "margin-bottom", "margin-left", "margin-right", "margin-top", "marginBottom", "marginHeight", "marginLeft", "marginRight", "marginTop", "marginWidth", "mark", "marker", "marker-end", "marker-mid", "marker-offset", "marker-start", "markerEnd", "markerHeight", "markerMid", "markerOffset", "markerStart", "markerUnits", "markerWidth", "marks", "mask", "mask-type", "maskContentUnits", "maskType", "maskUnits", "match", "matchMedia", "matchMedium", "matches", "matrix", "matrixTransform", "max", "max-height", "max-width", "maxAlternatives", "maxChannelCount", "maxConnectionsPerServer", "maxDecibels", "maxDistance", "maxHeight", "maxLength", "maxTouchPoints", "maxValue", "maxWidth", "measure", "measureText", "media", "mediaCapabilities", "mediaDevices", "mediaElement", "mediaGroup", "mediaKeys", "mediaText", "meetOrSlice", "memory", "menubar", "mergeAttributes", "message", "messageClass", "messageHandlers", "metaKey", "method", "mimeType", "mimeTypes", "min", "min-height", "min-width", "minDecibels", "minHeight", "minValue", "minWidth", "miterLimit", "mix-blend-mode", "mixBlendMode", "mode", "modify", "mount", "move", "moveBy", "moveEnd", "moveFirst", "moveFocusDown", "moveFocusLeft", "moveFocusRight", "moveFocusUp", "moveNext", "moveRow", "moveStart", "moveTo", "moveToBookmark", "moveToElementText", "moveToPoint", "mozAdd", "mozAnimationStartTime", "mozAnon", "mozApps", "mozAudioCaptured", "mozAudioChannelType", "mozAutoplayEnabled", "mozCancelAnimationFrame", "mozCancelFullScreen", "mozCancelRequestAnimationFrame", "mozCaptureStream", "mozCaptureStreamUntilEnded", "mozClearDataAt", "mozContact", "mozContacts", "mozCreateFileHandle", "mozCurrentTransform", "mozCurrentTransformInverse", "mozCursor", "mozDash", "mozDashOffset", "mozDecodedFrames", "mozExitPointerLock", "mozFillRule", "mozFragmentEnd", "mozFrameDelay", "mozFullScreen", "mozFullScreenElement", "mozFullScreenEnabled", "mozGetAll", "mozGetAllKeys", "mozGetAsFile", "mozGetDataAt", "mozGetMetadata", "mozGetUserMedia", "mozHasAudio", "mozHasItem", "mozHidden", "mozImageSmoothingEnabled", "mozIndexedDB", "mozInnerScreenX", "mozInnerScreenY", "mozInputSource", "mozIsTextField", "mozItem", "mozItemCount", "mozItems", "mozLength", "mozLockOrientation", "mozMatchesSelector", "mozMovementX", "mozMovementY", "mozOpaque", "mozOrientation", "mozPaintCount", "mozPaintedFrames", "mozParsedFrames", "mozPay", "mozPointerLockElement", "mozPresentedFrames", "mozPreservesPitch", "mozPressure", "mozPrintCallback", "mozRTCIceCandidate", "mozRTCPeerConnection", "mozRTCSessionDescription", "mozRemove", "mozRequestAnimationFrame", "mozRequestFullScreen", "mozRequestPointerLock", "mozSetDataAt", "mozSetImageElement", "mozSourceNode", "mozSrcObject", "mozSystem", "mozTCPSocket", "mozTextStyle", "mozTypesAt", "mozUnlockOrientation", "mozUserCancelled", "mozVisibilityState", "msAnimation", "msAnimationDelay", "msAnimationDirection", "msAnimationDuration", "msAnimationFillMode", "msAnimationIterationCount", "msAnimationName", "msAnimationPlayState", "msAnimationStartTime", "msAnimationTimingFunction", "msBackfaceVisibility", "msBlockProgression", "msCSSOMElementFloatMetrics", "msCaching", "msCachingEnabled", "msCancelRequestAnimationFrame", "msCapsLockWarningOff", "msClearImmediate", "msClose", "msContentZoomChaining", "msContentZoomFactor", "msContentZoomLimit", "msContentZoomLimitMax", "msContentZoomLimitMin", "msContentZoomSnap", "msContentZoomSnapPoints", "msContentZoomSnapType", "msContentZooming", "msConvertURL", "msCrypto", "msDoNotTrack", "msElementsFromPoint", "msElementsFromRect", "msExitFullscreen", "msExtendedCode", "msFillRule", "msFirstPaint", "msFlex", "msFlexAlign", "msFlexDirection", "msFlexFlow", "msFlexItemAlign", "msFlexLinePack", "msFlexNegative", "msFlexOrder", "msFlexPack", "msFlexPositive", "msFlexPreferredSize", "msFlexWrap", "msFlowFrom", "msFlowInto", "msFontFeatureSettings", "msFullscreenElement", "msFullscreenEnabled", "msGetInputContext", "msGetRegionContent", "msGetUntransformedBounds", "msGraphicsTrustStatus", "msGridColumn", "msGridColumnAlign", "msGridColumnSpan", "msGridColumns", "msGridRow", "msGridRowAlign", "msGridRowSpan", "msGridRows", "msHidden", "msHighContrastAdjust", "msHyphenateLimitChars", "msHyphenateLimitLines", "msHyphenateLimitZone", "msHyphens", "msImageSmoothingEnabled", "msImeAlign", "msIndexedDB", "msInterpolationMode", "msIsStaticHTML", "msKeySystem", "msKeys", "msLaunchUri", "msLockOrientation", "msManipulationViewsEnabled", "msMatchMedia", "msMatchesSelector", "msMaxTouchPoints", "msOrientation", "msOverflowStyle", "msPerspective", "msPerspectiveOrigin", "msPlayToDisabled", "msPlayToPreferredSourceUri", "msPlayToPrimary", "msPointerEnabled", "msRegionOverflow", "msReleasePointerCapture", "msRequestAnimationFrame", "msRequestFullscreen", "msSaveBlob", "msSaveOrOpenBlob", "msScrollChaining", "msScrollLimit", "msScrollLimitXMax", "msScrollLimitXMin", "msScrollLimitYMax", "msScrollLimitYMin", "msScrollRails", "msScrollSnapPointsX", "msScrollSnapPointsY", "msScrollSnapType", "msScrollSnapX", "msScrollSnapY", "msScrollTranslation", "msSetImmediate", "msSetMediaKeys", "msSetPointerCapture", "msTextCombineHorizontal", "msTextSizeAdjust", "msToBlob", "msTouchAction", "msTouchSelect", "msTraceAsyncCallbackCompleted", "msTraceAsyncCallbackStarting", "msTraceAsyncOperationCompleted", "msTraceAsyncOperationStarting", "msTransform", "msTransformOrigin", "msTransformStyle", "msTransition", "msTransitionDelay", "msTransitionDuration", "msTransitionProperty", "msTransitionTimingFunction", "msUnlockOrientation", "msUpdateAsyncCallbackRelation", "msUserSelect", "msVisibilityState", "msWrapFlow", "msWrapMargin", "msWrapThrough", "msWriteProfilerMark", "msZoom", "msZoomTo", "mt", "multiEntry", "multiSelectionObj", "multiline", "multiple", "multiply", "multiplySelf", "mutableFile", "muted", "n", "name", "nameProp", "namedItem", "namedRecordset", "names", "namespaceURI", "namespaces", "naturalHeight", "naturalWidth", "navigate", "navigation", "navigationMode", "navigationStart", "navigator", "near", "nearestViewportElement", "negative", "netscape", "networkState", "newScale", "newTranslate", "newURL", "newValue", "newValueSpecifiedUnits", "newVersion", "newhome", "next", "nextElementSibling", "nextNode", "nextPage", "nextSibling", "nickname", "noHref", "noResize", "noShade", "noValidate", "noWrap", "nodeName", "nodeType", "nodeValue", "normalize", "normalizedPathSegList", "notationName", "notations", "note", "noteGrainOn", "noteOff", "noteOn", "now", "numOctaves", "number", "numberOfChannels", "numberOfInputs", "numberOfItems", "numberOfOutputs", "numberValue", "oMatchesSelector", "object", "object-fit", "object-position", "objectFit", "objectPosition", "objectStore", "objectStoreNames", "observe", "of", "offscreenBuffering", "offset", "offsetHeight", "offsetLeft", "offsetNode", "offsetParent", "offsetTop", "offsetWidth", "offsetX", "offsetY", "ok", "oldURL", "oldValue", "oldVersion", "olderShadowRoot", "onLine", "onabort", "onactivate", "onactive", "onaddstream", "onaddtrack", "onafterprint", "onafterscriptexecute", "onafterupdate", "onaudioend", "onaudioprocess", "onaudiostart", "onautocomplete", "onautocompleteerror", "onbeforeactivate", "onbeforecopy", "onbeforecut", "onbeforedeactivate", "onbeforeeditfocus", "onbeforepaste", "onbeforeprint", "onbeforescriptexecute", "onbeforeunload", "onbeforeupdate", "onblocked", "onblur", "onbounce", "onboundary", "oncached", "oncancel", "oncandidatewindowhide", "oncandidatewindowshow", "oncandidatewindowupdate", "oncanplay", "oncanplaythrough", "once", "oncellchange", "onchange", "onchargingchange", "onchargingtimechange", "onchecking", "onclick", "onclose", "oncompassneedscalibration", "oncomplete", "oncontextmenu", "oncontrolselect", "oncopy", "oncuechange", "oncut", "ondataavailable", "ondatachannel", "ondatasetchanged", "ondatasetcomplete", "ondblclick", "ondeactivate", "ondevicelight", "ondevicemotion", "ondeviceorientation", "ondeviceproximity", "ondischargingtimechange", "ondisplay", "ondownloading", "ondrag", "ondragend", "ondragenter", "ondragleave", "ondragover", "ondragstart", "ondrop", "ondurationchange", "onemptied", "onencrypted", "onend", "onended", "onenter", "onerror", "onerrorupdate", "onexit", "onfilterchange", "onfinish", "onfocus", "onfocusin", "onfocusout", "onfullscreenchange", "onfullscreenerror", "ongesturechange", "ongestureend", "ongesturestart", "ongotpointercapture", "onhashchange", "onhelp", "onicecandidate", "oniceconnectionstatechange", "oninactive", "oninput", "oninvalid", "onkeydown", "onkeypress", "onkeyup", "onlanguagechange", "onlayoutcomplete", "onlevelchange", "onload", "onloadeddata", "onloadedmetadata", "onloadend", "onloadstart", "onlosecapture", "onlostpointercapture", "only", "onmark", "onmessage", "onmousedown", "onmouseenter", "onmouseleave", "onmousemove", "onmouseout", "onmouseover", "onmouseup", "onmousewheel", "onmove", "onmoveend", "onmovestart", "onmozfullscreenchange", "onmozfullscreenerror", "onmozorientationchange", "onmozpointerlockchange", "onmozpointerlockerror", "onmscontentzoom", "onmsfullscreenchange", "onmsfullscreenerror", "onmsgesturechange", "onmsgesturedoubletap", "onmsgestureend", "onmsgesturehold", "onmsgesturestart", "onmsgesturetap", "onmsgotpointercapture", "onmsinertiastart", "onmslostpointercapture", "onmsmanipulationstatechanged", "onmsneedkey", "onmsorientationchange", "onmspointercancel", "onmspointerdown", "onmspointerenter", "onmspointerhover", "onmspointerleave", "onmspointermove", "onmspointerout", "onmspointerover", "onmspointerup", "onmssitemodejumplistitemremoved", "onmsthumbnailclick", "onnegotiationneeded", "onnomatch", "onnoupdate", "onobsolete", "onoffline", "ononline", "onopen", "onorientationchange", "onpagechange", "onpagehide", "onpageshow", "onpaste", "onpause", "onplay", "onplaying", "onpluginstreamstart", "onpointercancel", "onpointerdown", "onpointerenter", "onpointerleave", "onpointerlockchange", "onpointerlockerror", "onpointermove", "onpointerout", "onpointerover", "onpointerup", "onpopstate", "onprogress", "onpropertychange", "onratechange", "onreadystatechange", "onremovestream", "onremovetrack", "onreset", "onresize", "onresizeend", "onresizestart", "onresourcetimingbufferfull", "onresult", "onresume", "onrowenter", "onrowexit", "onrowsdelete", "onrowsinserted", "onscroll", "onsearch", "onseeked", "onseeking", "onselect", "onselectionchange", "onselectstart", "onshow", "onsignalingstatechange", "onsoundend", "onsoundstart", "onspeechend", "onspeechstart", "onstalled", "onstart", "onstatechange", "onstop", "onstorage", "onstoragecommit", "onsubmit", "onsuccess", "onsuspend", "ontextinput", "ontimeout", "ontimeupdate", "ontoggle", "ontouchcancel", "ontouchend", "ontouchmove", "ontouchstart", "ontransitionend", "onunload", "onupdateready", "onupgradeneeded", "onuserproximity", "onversionchange", "onvoiceschanged", "onvolumechange", "onwaiting", "onwarning", "onwebkitanimationend", "onwebkitanimationiteration", "onwebkitanimationstart", "onwebkitcurrentplaybacktargetiswirelesschanged", "onwebkitfullscreenchange", "onwebkitfullscreenerror", "onwebkitkeyadded", "onwebkitkeyerror", "onwebkitkeymessage", "onwebkitneedkey", "onwebkitorientationchange", "onwebkitplaybacktargetavailabilitychanged", "onwebkitpointerlockchange", "onwebkitpointerlockerror", "onwebkitresourcetimingbufferfull", "onwebkittransitionend", "onwheel", "onzoom", "opacity", "open", "openCursor", "openDatabase", "openKeyCursor", "opener", "opera", "operationType", "operator", "opr", "optimum", "options", "order", "orderX", "orderY", "ordered", "org", "orient", "orientAngle", "orientType", "orientation", "origin", "originalTarget", "orphans", "oscpu", "outerHTML", "outerHeight", "outerText", "outerWidth", "outline", "outline-color", "outline-offset", "outline-style", "outline-width", "outlineColor", "outlineOffset", "outlineStyle", "outlineWidth", "outputBuffer", "overflow", "overflow-x", "overflow-y", "overflowX", "overflowY", "overrideMimeType", "oversample", "ownerDocument", "ownerElement", "ownerNode", "ownerRule", "ownerSVGElement", "owningElement", "p1", "p2", "p3", "p4", "pad", "padding", "padding-bottom", "padding-left", "padding-right", "padding-top", "paddingBottom", "paddingLeft", "paddingRight", "paddingTop", "page", "page-break-after", "page-break-before", "page-break-inside", "pageBreakAfter", "pageBreakBefore", "pageBreakInside", "pageCount", "pageX", "pageXOffset", "pageY", "pageYOffset", "pages", "paint-order", "paintOrder", "paintRequests", "paintType", "palette", "panningModel", "parent", "parentElement", "parentNode", "parentRule", "parentStyleSheet", "parentTextEdit", "parentWindow", "parse", "parseFloat", "parseFromString", "parseInt", "participants", "passive", "password", "pasteHTML", "path", "pathLength", "pathSegList", "pathSegType", "pathSegTypeAsLetter", "pathname", "pattern", "patternContentUnits", "patternMismatch", "patternTransform", "patternUnits", "pause", "pauseAnimations", "pauseOnExit", "paused", "pending", "performance", "permission", "persisted", "personalbar", "perspective", "perspective-origin", "perspectiveOrigin", "phoneticFamilyName", "phoneticGivenName", "photo", "ping", "pitch", "pixelBottom", "pixelDepth", "pixelHeight", "pixelLeft", "pixelRight", "pixelStorei", "pixelTop", "pixelUnitToMillimeterX", "pixelUnitToMillimeterY", "pixelWidth", "placeholder", "platform", "play", "playbackRate", "playbackState", "playbackTime", "played", "plugins", "pluginspage", "pname", "pointer-events", "pointerBeforeReferenceNode", "pointerEnabled", "pointerEvents", "pointerId", "pointerLockElement", "pointerType", "points", "pointsAtX", "pointsAtY", "pointsAtZ", "polygonOffset", "pop", "popupWindowFeatures", "popupWindowName", "popupWindowURI", "port", "port1", "port2", "ports", "posBottom", "posHeight", "posLeft", "posRight", "posTop", "posWidth", "position", "positionAlign", "postError", "postMessage", "poster", "pow", "powerOff", "preMultiplySelf", "precision", "preferredStyleSheetSet", "preferredStylesheetSet", "prefix", "preload", "prepend", "preserveAlpha", "preserveAspectRatio", "preserveAspectRatioString", "pressed", "pressure", "prevValue", "preventDefault", "preventExtensions", "previousElementSibling", "previousNode", "previousPage", "previousScale", "previousSibling", "previousTranslate", "primaryKey", "primitiveType", "primitiveUnits", "principals", "print", "privateKey", "probablySupportsContext", "process", "processIceMessage", "product", "productSub", "profile", "profileEnd", "profiles", "prompt", "properties", "propertyIsEnumerable", "propertyName", "protocol", "protocolLong", "prototype", "pseudoClass", "pseudoElement", "publicId", "publicKey", "published", "push", "pushNotification", "pushState", "put", "putImageData", "quadraticCurveTo", "qualifier", "queryCommandEnabled", "queryCommandIndeterm", "queryCommandState", "queryCommandSupported", "queryCommandText", "queryCommandValue", "querySelector", "querySelectorAll", "quote", "quotes", "r", "r1", "r2", "race", "radiogroup", "radiusX", "radiusY", "random", "range", "rangeCount", "rangeMax", "rangeMin", "rangeOffset", "rangeOverflow", "rangeParent", "rangeUnderflow", "rate", "ratio", "raw", "read", "readAsArrayBuffer", "readAsBinaryString", "readAsBlob", "readAsDataURL", "readAsText", "readOnly", "readPixels", "readReportRequested", "readyState", "reason", "reboot", "receiver", "receivers", "recordNumber", "recordset", "rect", "red", "redirectCount", "redirectEnd", "redirectStart", "reduce", "reduceRight", "reduction", "refDistance", "refX", "refY", "referenceNode", "referrer", "refresh", "region", "regionAnchorX", "regionAnchorY", "regionId", "regions", "register", "registerContentHandler", "registerElement", "registerProtocolHandler", "reject", "rel", "relList", "relatedNode", "relatedTarget", "release", "releaseCapture", "releaseEvents", "releasePointerCapture", "releaseShaderCompiler", "reliable", "reload", "remainingSpace", "remoteDescription", "remove", "removeAllRanges", "removeAttribute", "removeAttributeNS", "removeAttributeNode", "removeBehavior", "removeChild", "removeCue", "removeEventListener", "removeFilter", "removeImport", "removeItem", "removeListener", "removeNamedItem", "removeNamedItemNS", "removeNode", "removeParameter", "removeProperty", "removeRange", "removeRegion", "removeRule", "removeSiteSpecificTrackingException", "removeSourceBuffer", "removeStream", "removeTrack", "removeVariable", "removeWakeLockListener", "removeWebWideTrackingException", "removedNodes", "renderbufferStorage", "renderedBuffer", "renderingMode", "repeat", "replace", "replaceAdjacentText", "replaceChild", "replaceData", "replaceId", "replaceItem", "replaceNode", "replaceState", "replaceTrack", "replaceWholeText", "reportValidity", "requestAnimationFrame", "requestAutocomplete", "requestData", "requestFullscreen", "requestMediaKeySystemAccess", "requestPermission", "requestPointerLock", "requestStart", "requestingWindow", "required", "requiredExtensions", "requiredFeatures", "reset", "resetTransform", "resize", "resizeBy", "resizeTo", "resolve", "response", "responseBody", "responseEnd", "responseStart", "responseText", "responseType", "responseURL", "responseXML", "restore", "result", "resultType", "resume", "returnValue", "rev", "reverse", "reversed", "revocable", "revokeObjectURL", "rgbColor", "right", "rightContext", "rightMargin", "rolloffFactor", "root", "rootElement", "rotate", "rotateAxisAngle", "rotateAxisAngleSelf", "rotateFromVector", "rotateFromVectorSelf", "rotateSelf", "rotation", "rotationRate", "round", "rowIndex", "rowSpan", "rows", "rubyAlign", "rubyOverhang", "rubyPosition", "rules", "runtime", "runtimeStyle", "rx", "ry", "safari", "sampleCoverage", "sampleRate", "sandbox", "save", "scale", "scale3d", "scale3dSelf", "scaleNonUniform", "scaleNonUniformSelf", "scaleSelf", "scheme", "scissor", "scope", "scopeName", "scoped", "screen", "screenBrightness", "screenEnabled", "screenLeft", "screenPixelToMillimeterX", "screenPixelToMillimeterY", "screenTop", "screenX", "screenY", "scripts", "scroll", "scroll-behavior", "scrollAmount", "scrollBehavior", "scrollBy", "scrollByLines", "scrollByPages", "scrollDelay", "scrollHeight", "scrollIntoView", "scrollIntoViewIfNeeded", "scrollLeft", "scrollLeftMax", "scrollMaxX", "scrollMaxY", "scrollTo", "scrollTop", "scrollTopMax", "scrollWidth", "scrollX", "scrollY", "scrollbar3dLightColor", "scrollbarArrowColor", "scrollbarBaseColor", "scrollbarDarkShadowColor", "scrollbarFaceColor", "scrollbarHighlightColor", "scrollbarShadowColor", "scrollbarTrackColor", "scrollbars", "scrolling", "sdp", "sdpMLineIndex", "sdpMid", "seal", "search", "searchBox", "searchBoxJavaBridge_", "searchParams", "sectionRowIndex", "secureConnectionStart", "security", "seed", "seekable", "seeking", "select", "selectAllChildren", "selectNode", "selectNodeContents", "selectNodes", "selectSingleNode", "selectSubString", "selected", "selectedIndex", "selectedOptions", "selectedStyleSheetSet", "selectedStylesheetSet", "selection", "selectionDirection", "selectionEnd", "selectionStart", "selector", "selectorText", "self", "send", "sendAsBinary", "sendBeacon", "sender", "sentTimestamp", "separator", "serializeToString", "serviceWorker", "sessionId", "sessionStorage", "set", "setActive", "setAlpha", "setAttribute", "setAttributeNS", "setAttributeNode", "setAttributeNodeNS", "setBaseAndExtent", "setBingCurrentSearchDefault", "setCapture", "setColor", "setCompositeOperation", "setCurrentTime", "setCustomValidity", "setData", "setDate", "setDragImage", "setEnd", "setEndAfter", "setEndBefore", "setEndPoint", "setFillColor", "setFilterRes", "setFloat32", "setFloat64", "setFloatValue", "setFullYear", "setHours", "setImmediate", "setInt16", "setInt32", "setInt8", "setInterval", "setItem", "setLineCap", "setLineDash", "setLineJoin", "setLineWidth", "setLocalDescription", "setMatrix", "setMatrixValue", "setMediaKeys", "setMilliseconds", "setMinutes", "setMiterLimit", "setMonth", "setNamedItem", "setNamedItemNS", "setNonUserCodeExceptions", "setOrientToAngle", "setOrientToAuto", "setOrientation", "setOverrideHistoryNavigationMode", "setPaint", "setParameter", "setPeriodicWave", "setPointerCapture", "setPosition", "setPreference", "setProperty", "setPrototypeOf", "setRGBColor", "setRGBColorICCColor", "setRadius", "setRangeText", "setRemoteDescription", "setRequestHeader", "setResizable", "setResourceTimingBufferSize", "setRotate", "setScale", "setSeconds", "setSelectionRange", "setServerCertificate", "setShadow", "setSkewX", "setSkewY", "setStart", "setStartAfter", "setStartBefore", "setStdDeviation", "setStringValue", "setStrokeColor", "setSuggestResult", "setTargetAtTime", "setTargetValueAtTime", "setTime", "setTimeout", "setTransform", "setTranslate", "setUTCDate", "setUTCFullYear", "setUTCHours", "setUTCMilliseconds", "setUTCMinutes", "setUTCMonth", "setUTCSeconds", "setUint16", "setUint32", "setUint8", "setUri", "setValueAtTime", "setValueCurveAtTime", "setVariable", "setVelocity", "setVersion", "setYear", "settingName", "settingValue", "sex", "shaderSource", "shadowBlur", "shadowColor", "shadowOffsetX", "shadowOffsetY", "shadowRoot", "shape", "shape-rendering", "shapeRendering", "sheet", "shift", "shiftKey", "shiftLeft", "show", "showHelp", "showModal", "showModalDialog", "showModelessDialog", "showNotification", "sidebar", "sign", "signalingState", "sin", "singleNodeValue", "sinh", "size", "sizeToContent", "sizes", "skewX", "skewXSelf", "skewY", "skewYSelf", "slice", "slope", "small", "smil", "smooth", "smoothingTimeConstant", "snapToLines", "snapshotItem", "snapshotLength", "some", "sort", "source", "sourceBuffer", "sourceBuffers", "sourceIndex", "spacing", "span", "speakAs", "speaking", "specified", "specularConstant", "specularExponent", "speechSynthesis", "speed", "speedOfSound", "spellcheck", "splice", "split", "splitText", "spreadMethod", "sqrt", "src", "srcElement", "srcFilter", "srcUrn", "srcdoc", "srclang", "srcset", "stack", "stackTraceLimit", "stacktrace", "standalone", "standby", "start", "startContainer", "startIce", "startOffset", "startRendering", "startTime", "startsWith", "state", "status", "statusMessage", "statusText", "statusbar", "stdDeviationX", "stdDeviationY", "stencilFunc", "stencilFuncSeparate", "stencilMask", "stencilMaskSeparate", "stencilOp", "stencilOpSeparate", "step", "stepDown", "stepMismatch", "stepUp", "sticky", "stitchTiles", "stop", "stop-color", "stop-opacity", "stopColor", "stopImmediatePropagation", "stopOpacity", "stopPropagation", "storageArea", "storageName", "storageStatus", "storeSiteSpecificTrackingException", "storeWebWideTrackingException", "stpVersion", "stream", "strike", "stringValue", "stringify", "stroke", "stroke-dasharray", "stroke-dashoffset", "stroke-linecap", "stroke-linejoin", "stroke-miterlimit", "stroke-opacity", "stroke-width", "strokeDasharray", "strokeDashoffset", "strokeLinecap", "strokeLinejoin", "strokeMiterlimit", "strokeOpacity", "strokeRect", "strokeStyle", "strokeText", "strokeWidth", "style", "styleFloat", "styleMedia", "styleSheet", "styleSheetSets", "styleSheets", "sub", "subarray", "subject", "submit", "subscribe", "substr", "substring", "substringData", "subtle", "subtree", "suffix", "suffixes", "summary", "sup", "supports", "surfaceScale", "surroundContents", "suspend", "suspendRedraw", "swapCache", "swapNode", "sweepFlag", "symbols", "system", "systemCode", "systemId", "systemLanguage", "systemXDPI", "systemYDPI", "tBodies", "tFoot", "tHead", "tabIndex", "table", "table-layout", "tableLayout", "tableValues", "tag", "tagName", "tagUrn", "tags", "taintEnabled", "takeRecords", "tan", "tanh", "target", "targetElement", "targetTouches", "targetX", "targetY", "tel", "terminate", "test", "texImage2D", "texParameterf", "texParameteri", "texSubImage2D", "text", "text-align", "text-anchor", "text-decoration", "text-decoration-color", "text-decoration-line", "text-decoration-style", "text-indent", "text-overflow", "text-rendering", "text-shadow", "text-transform", "textAlign", "textAlignLast", "textAnchor", "textAutospace", "textBaseline", "textContent", "textDecoration", "textDecorationBlink", "textDecorationColor", "textDecorationLine", "textDecorationLineThrough", "textDecorationNone", "textDecorationOverline", "textDecorationStyle", "textDecorationUnderline", "textIndent", "textJustify", "textJustifyTrim", "textKashida", "textKashidaSpace", "textLength", "textOverflow", "textRendering", "textShadow", "textTracks", "textTransform", "textUnderlinePosition", "then", "threadId", "threshold", "tiltX", "tiltY", "time", "timeEnd", "timeStamp", "timeout", "timestamp", "timestampOffset", "timing", "title", "toArray", "toBlob", "toDataURL", "toDateString", "toElement", "toExponential", "toFixed", "toFloat32Array", "toFloat64Array", "toGMTString", "toISOString", "toJSON", "toLocaleDateString", "toLocaleFormat", "toLocaleLowerCase", "toLocaleString", "toLocaleTimeString", "toLocaleUpperCase", "toLowerCase", "toMethod", "toPrecision", "toSdp", "toSource", "toStaticHTML", "toString", "toStringTag", "toTimeString", "toUTCString", "toUpperCase", "toggle", "toggleLongPressEnabled", "tooLong", "toolbar", "top", "topMargin", "total", "totalFrameDelay", "totalVideoFrames", "touchAction", "touches", "trace", "track", "transaction", "transactions", "transform", "transform-origin", "transform-style", "transformOrigin", "transformPoint", "transformString", "transformStyle", "transformToDocument", "transformToFragment", "transition", "transition-delay", "transition-duration", "transition-property", "transition-timing-function", "transitionDelay", "transitionDuration", "transitionProperty", "transitionTimingFunction", "translate", "translateSelf", "translationX", "translationY", "trim", "trimLeft", "trimRight", "trueSpeed", "trunc", "truncate", "type", "typeDetail", "typeMismatch", "typeMustMatch", "types", "ubound", "undefined", "unescape", "uneval", "unicode-bidi", "unicodeBidi", "uniform1f", "uniform1fv", "uniform1i", "uniform1iv", "uniform2f", "uniform2fv", "uniform2i", "uniform2iv", "uniform3f", "uniform3fv", "uniform3i", "uniform3iv", "uniform4f", "uniform4fv", "uniform4i", "uniform4iv", "uniformMatrix2fv", "uniformMatrix3fv", "uniformMatrix4fv", "unique", "uniqueID", "uniqueNumber", "unitType", "units", "unloadEventEnd", "unloadEventStart", "unlock", "unmount", "unobserve", "unpause", "unpauseAnimations", "unreadCount", "unregister", "unregisterContentHandler", "unregisterProtocolHandler", "unscopables", "unselectable", "unshift", "unsubscribe", "unsuspendRedraw", "unsuspendRedrawAll", "unwatch", "unwrapKey", "update", "updateCommands", "updateIce", "updateInterval", "updateSettings", "updated", "updating", "upload", "upper", "upperBound", "upperOpen", "uri", "url", "urn", "urns", "usages", "useCurrentView", "useMap", "useProgram", "usedSpace", "userAgent", "userLanguage", "username", "v8BreakIterator", "vAlign", "vLink", "valid", "validateProgram", "validationMessage", "validity", "value", "valueAsDate", "valueAsNumber", "valueAsString", "valueInSpecifiedUnits", "valueMissing", "valueOf", "valueText", "valueType", "values", "vector-effect", "vectorEffect", "velocityAngular", "velocityExpansion", "velocityX", "velocityY", "vendor", "vendorSub", "verify", "version", "vertexAttrib1f", "vertexAttrib1fv", "vertexAttrib2f", "vertexAttrib2fv", "vertexAttrib3f", "vertexAttrib3fv", "vertexAttrib4f", "vertexAttrib4fv", "vertexAttribDivisorANGLE", "vertexAttribPointer", "vertical", "vertical-align", "verticalAlign", "verticalOverflow", "vibrate", "videoHeight", "videoTracks", "videoWidth", "view", "viewBox", "viewBoxString", "viewTarget", "viewTargetString", "viewport", "viewportAnchorX", "viewportAnchorY", "viewportElement", "visibility", "visibilityState", "visible", "vlinkColor", "voice", "volume", "vrml", "vspace", "w", "wand", "warn", "wasClean", "watch", "watchPosition", "webdriver", "webkitAddKey", "webkitAnimation", "webkitAnimationDelay", "webkitAnimationDirection", "webkitAnimationDuration", "webkitAnimationFillMode", "webkitAnimationIterationCount", "webkitAnimationName", "webkitAnimationPlayState", "webkitAnimationTimingFunction", "webkitAppearance", "webkitAudioContext", "webkitAudioDecodedByteCount", "webkitAudioPannerNode", "webkitBackfaceVisibility", "webkitBackground", "webkitBackgroundAttachment", "webkitBackgroundClip", "webkitBackgroundColor", "webkitBackgroundImage", "webkitBackgroundOrigin", "webkitBackgroundPosition", "webkitBackgroundPositionX", "webkitBackgroundPositionY", "webkitBackgroundRepeat", "webkitBackgroundSize", "webkitBackingStorePixelRatio", "webkitBorderImage", "webkitBorderImageOutset", "webkitBorderImageRepeat", "webkitBorderImageSlice", "webkitBorderImageSource", "webkitBorderImageWidth", "webkitBoxAlign", "webkitBoxDirection", "webkitBoxFlex", "webkitBoxOrdinalGroup", "webkitBoxOrient", "webkitBoxPack", "webkitBoxSizing", "webkitCancelAnimationFrame", "webkitCancelFullScreen", "webkitCancelKeyRequest", "webkitCancelRequestAnimationFrame", "webkitClearResourceTimings", "webkitClosedCaptionsVisible", "webkitConvertPointFromNodeToPage", "webkitConvertPointFromPageToNode", "webkitCreateShadowRoot", "webkitCurrentFullScreenElement", "webkitCurrentPlaybackTargetIsWireless", "webkitDirectionInvertedFromDevice", "webkitDisplayingFullscreen", "webkitEnterFullScreen", "webkitEnterFullscreen", "webkitExitFullScreen", "webkitExitFullscreen", "webkitExitPointerLock", "webkitFullScreenKeyboardInputAllowed", "webkitFullscreenElement", "webkitFullscreenEnabled", "webkitGenerateKeyRequest", "webkitGetAsEntry", "webkitGetDatabaseNames", "webkitGetEntries", "webkitGetEntriesByName", "webkitGetEntriesByType", "webkitGetFlowByName", "webkitGetGamepads", "webkitGetImageDataHD", "webkitGetNamedFlows", "webkitGetRegionFlowRanges", "webkitGetUserMedia", "webkitHasClosedCaptions", "webkitHidden", "webkitIDBCursor", "webkitIDBDatabase", "webkitIDBDatabaseError", "webkitIDBDatabaseException", "webkitIDBFactory", "webkitIDBIndex", "webkitIDBKeyRange", "webkitIDBObjectStore", "webkitIDBRequest", "webkitIDBTransaction", "webkitImageSmoothingEnabled", "webkitIndexedDB", "webkitInitMessageEvent", "webkitIsFullScreen", "webkitKeys", "webkitLineDashOffset", "webkitLockOrientation", "webkitMatchesSelector", "webkitMediaStream", "webkitNotifications", "webkitOfflineAudioContext", "webkitOrientation", "webkitPeerConnection00", "webkitPersistentStorage", "webkitPointerLockElement", "webkitPostMessage", "webkitPreservesPitch", "webkitPutImageDataHD", "webkitRTCPeerConnection", "webkitRegionOverset", "webkitRequestAnimationFrame", "webkitRequestFileSystem", "webkitRequestFullScreen", "webkitRequestFullscreen", "webkitRequestPointerLock", "webkitResolveLocalFileSystemURL", "webkitSetMediaKeys", "webkitSetResourceTimingBufferSize", "webkitShadowRoot", "webkitShowPlaybackTargetPicker", "webkitSlice", "webkitSpeechGrammar", "webkitSpeechGrammarList", "webkitSpeechRecognition", "webkitSpeechRecognitionError", "webkitSpeechRecognitionEvent", "webkitStorageInfo", "webkitSupportsFullscreen", "webkitTemporaryStorage", "webkitTextSizeAdjust", "webkitTransform", "webkitTransformOrigin", "webkitTransition", "webkitTransitionDelay", "webkitTransitionDuration", "webkitTransitionProperty", "webkitTransitionTimingFunction", "webkitURL", "webkitUnlockOrientation", "webkitUserSelect", "webkitVideoDecodedByteCount", "webkitVisibilityState", "webkitWirelessVideoPlaybackDisabled", "webkitdropzone", "webstore", "weight", "whatToShow", "wheelDelta", "wheelDeltaX", "wheelDeltaY", "which", "white-space", "whiteSpace", "wholeText", "widows", "width", "will-change", "willChange", "willValidate", "window", "withCredentials", "word-break", "word-spacing", "word-wrap", "wordBreak", "wordSpacing", "wordWrap", "wrap", "wrapKey", "write", "writeln", "writingMode", "x", "x1", "x2", "xChannelSelector", "xmlEncoding", "xmlStandalone", "xmlVersion", "xmlbase", "xmllang", "xmlspace", "y", "y1", "y2", "yChannelSelector", "yandex", "z", "z-index", "zIndex", "zoom", "zoomAndPan", "zoomRectScreen"];
        function uo(e, t) { function n(e) { _(t, e); } e.walk(new Cn((function (e) { e instanceof Rt && e.quote ? n(e.key) : e instanceof Mt && e.quote ? n(e.key.name) : e instanceof Dt && co(e.property, n); }))); }
        function co(e, t) { e.walk(new Cn((function (e) { return e instanceof ht ? co(e.tail_node(), t) : e instanceof ln ? t(e.value) : e instanceof yt && (co(e.consequent, t), co(e.alternative, t)), !0; }))); }
        function lo(e, t) { var n = (t = r(t, { builtins: !1, cache: null, debug: !1, keep_quoted: !1, only_cache: !1, regex: null, reserved: null, undeclared: !1 }, !0)).reserved; Array.isArray(n) || (n = [n]); var i = new Set(n); t.builtins || function (e) { so.forEach(i); var t = {}, n = "object" == typeof global ? global : self; function i(t) { e.add(t); } ["Symbol", "Map", "Promise", "Proxy", "Reflect", "Set", "WeakMap", "WeakSet"].forEach((function (e) { t[e] = n[e] || new Function; })), ["null", "true", "false", "Infinity", "-Infinity", "undefined"].forEach(i), [Object, Array, Function, Number, String, Boolean, Error, Math, Date, RegExp, t.Symbol, ArrayBuffer, DataView, decodeURI, decodeURIComponent, encodeURI, encodeURIComponent, eval, EvalError, Float32Array, Float64Array, Int8Array, Int16Array, Int32Array, isFinite, isNaN, JSON, t.Map, parseFloat, parseInt, t.Promise, t.Proxy, RangeError, ReferenceError, t.Reflect, t.Set, SyntaxError, TypeError, Uint8Array, Uint8ClampedArray, Uint16Array, Uint32Array, URIError, t.WeakMap, t.WeakSet].forEach((function (e) { Object.getOwnPropertyNames(e).map(i), e.prototype && Object.getOwnPropertyNames(e.prototype).map(i); })); }(i); var o, a = -1; t.cache ? (o = t.cache.props).forEach((function (e) { i.add(e); })) : o = new Map; var s, u = t.regex && new RegExp(t.regex), c = !1 !== t.debug; c && (s = !0 === t.debug ? "" : t.debug); var l = new Set, f = new Set, p = "strict" === t.keep_quoted; return e.walk(new Cn((function (e) { if (e instanceof Rt)
            "string" != typeof e.key || p && e.quote || m(e.key);
        else if (e instanceof Mt)
            p && e.key.end.quote || m(e.key.name);
        else if (e instanceof gt) {
            var n = !!t.undeclared;
            if (!n) {
                for (var i = e; i.expression;)
                    i = i.expression;
                n = !(i.thedef && i.thedef.undeclared);
            }
            !n || p && e.quote || m(e.property);
        }
        else
            e instanceof Dt ? p || co(e.property, m) : e instanceof dt && "Object.defineProperty" == e.expression.print_to_string() && co(e.args[1], m); }))), e.transform(new On((function (e) { e instanceof Rt ? "string" != typeof e.key || p && e.quote || (e.key = h(e.key)) : e instanceof Mt ? p && e.key.end.quote || (e.key.name = h(e.key.name)) : e instanceof gt ? p && e.quote || (e.property = h(e.property)) : !t.keep_quoted && e instanceof Dt ? e.property = E(e.property) : e instanceof dt && "Object.defineProperty" == e.expression.print_to_string() && (e.args[1] = E(e.args[1])); }))); function _(e) { return !f.has(e) && (!i.has(e) && (t.only_cache ? o.has(e) : !/^-?[0-9]+(\.[0-9]+)?(e[+-][0-9]+)?$/.test(e))); } function d(e) { return !(u && !u.test(e)) && (!i.has(e) && (o.has(e) || l.has(e))); } function m(e) { _(e) && l.add(e), d(e) || f.add(e); } function h(e) { if (!d(e))
            return e; var t = o.get(e); if (!t) {
            if (c) {
                var n = "_$" + e + "$" + s + "_";
                _(n) && (t = n);
            }
            if (!t)
                do {
                    t = Yn(++a);
                } while (!_(t));
            o.set(e, t);
        } return t; } function E(e) { return e.transform(new On((function (e) { if (e instanceof ht) {
            var t = e.expressions.length - 1;
            e.expressions[t] = E(e.expressions[t]);
        }
        else
            e instanceof ln ? e.value = h(e.value) : e instanceof yt && (e.consequent = E(e.consequent), e.alternative = E(e.alternative)); return e; }))); } }
        var fo = "undefined" == typeof atob ? function (e) { return Buffer.from(e, "base64").toString(); } : atob, po = "undefined" == typeof btoa ? function (e) { return Buffer.from(e).toString("base64"); } : btoa;
        function _o(e, t, n) { t[e] && n.forEach((function (n) { t[n] && ("object" != typeof t[n] && (t[n] = {}), e in t[n] || (t[n][e] = t[e])); })); }
        function mo(e) { e && ("props" in e ? e.props instanceof Map || (e.props = function (e) { var t = new Map; for (var n in e)
            D(e, n) && "$" === n.charAt(0) && t.set(n.substr(1), e[n]); return t; }(e.props)) : e.props = new Map); }
        function ho(e) { return { props: (t = e.props, n = Object.create(null), t.forEach((function (e, t) { n["$" + t] = e; })), n) }; var t, n; }
        function Eo(e, n) { var i, o, a = fe.warn_function; try {
            var s, u = (n = r(n, { compress: {}, ecma: void 0, enclose: !1, ie8: !1, keep_classnames: void 0, keep_fnames: !1, mangle: {}, module: !1, nameCache: null, output: {}, parse: {}, rename: void 0, safari10: !1, sourceMap: !1, timings: !1, toplevel: !1, warnings: !1, wrap: !1 }, !0)).timings && { start: Date.now() };
            void 0 === n.keep_classnames && (n.keep_classnames = n.keep_fnames), void 0 === n.rename && (n.rename = n.compress && n.mangle), _o("ecma", n, ["parse", "compress", "output"]), _o("ie8", n, ["compress", "mangle", "output"]), _o("keep_classnames", n, ["compress", "mangle"]), _o("keep_fnames", n, ["compress", "mangle"]), _o("module", n, ["parse", "compress", "mangle"]), _o("safari10", n, ["mangle", "output"]), _o("toplevel", n, ["compress", "mangle"]), _o("warnings", n, ["compress"]), n.mangle && (n.mangle = r(n.mangle, { cache: n.nameCache && (n.nameCache.vars || {}), eval: !1, ie8: !1, keep_classnames: !1, keep_fnames: !1, module: !1, properties: !1, reserved: [], safari10: !1, toplevel: !1 }, !0), n.mangle.properties && ("object" != typeof n.mangle.properties && (n.mangle.properties = {}), n.mangle.properties.keep_quoted && (s = n.mangle.properties.reserved, Array.isArray(s) || (s = []), n.mangle.properties.reserved = s), !n.nameCache || "cache" in n.mangle.properties || (n.mangle.properties.cache = n.nameCache.props || {})), mo(n.mangle.cache), mo(n.mangle.properties.cache)), n.sourceMap && (n.sourceMap = r(n.sourceMap, { asObject: !1, content: null, filename: null, includeSources: !1, root: null, url: null }, !0));
            var c, l = [];
            if (n.warnings && !fe.warn_function && (fe.warn_function = function (e) { l.push(e); }), u && (u.parse = Date.now()), e instanceof xe)
                c = e;
            else {
                for (var f in "string" == typeof e && (e = [e]), n.parse = n.parse || {}, n.parse.toplevel = null, e)
                    if (D(e, f) && (n.parse.filename = f, n.parse.toplevel = ue(e[f], n.parse), n.sourceMap && "inline" == n.sourceMap.content)) {
                        if (Object.keys(e).length > 1)
                            throw new Error("inline source map only works with singular input");
                        n.sourceMap.content = (i = e[f], o = void 0, (o = /(?:^|[^.])\/\/# sourceMappingURL=data:application\/json(;[\w=-]*)?;base64,([+/0-9A-Za-z]*=*)\s*$/.exec(i)) ? fo(o[2]) : (fe.warn("inline source map not found"), null));
                    }
                c = n.parse.toplevel;
            }
            s && "strict" !== n.mangle.properties.keep_quoted && uo(c, s), n.wrap && (c = c.wrap_commonjs(n.wrap)), n.enclose && (c = c.wrap_enclose(n.enclose)), u && (u.rename = Date.now()), u && (u.compress = Date.now()), n.compress && (c = new ri(n.compress).compress(c)), u && (u.scope = Date.now()), n.mangle && c.figure_out_scope(n.mangle), u && (u.mangle = Date.now()), n.mangle && (Yn.reset(), c.compute_char_frequency(n.mangle), c.mangle_names(n.mangle)), u && (u.properties = Date.now()), n.mangle && n.mangle.properties && (c = lo(c, n.mangle.properties)), u && (u.output = Date.now());
            var p = {};
            if (n.output.ast && (p.ast = c), !D(n.output, "code") || n.output.code) {
                if (n.sourceMap && ("string" == typeof n.sourceMap.content && (n.sourceMap.content = JSON.parse(n.sourceMap.content)), n.output.source_map = function (e) { e = r(e, { file: null, root: null, orig: null, orig_line_diff: 0, dest_line_diff: 0 }); var n = new t.SourceMapGenerator({ file: e.file, sourceRoot: e.root }), i = e.orig && new t.SourceMapConsumer(e.orig); return i && i.sources.forEach((function (e) { var t = i.sourceContentFor(e, !0); t && n.setSourceContent(e, t); })), { add: function (t, o, r, a, s, u) { if (i) {
                        var c = i.originalPositionFor({ line: a, column: s });
                        if (null === c.source)
                            return;
                        t = c.source, a = c.line, s = c.column, u = c.name || u;
                    } n.addMapping({ generated: { line: o + e.dest_line_diff, column: r }, original: { line: a + e.orig_line_diff, column: s }, source: t, name: u }); }, get: function () { return n; }, toString: function () { return JSON.stringify(n.toJSON()); } }; }({ file: n.sourceMap.filename, orig: n.sourceMap.content, root: n.sourceMap.root }), n.sourceMap.includeSources)) {
                    if (e instanceof xe)
                        throw new Error("original source content unavailable");
                    for (var f in e)
                        D(e, f) && n.output.source_map.get().setSourceContent(f, e[f]);
                }
                delete n.output.ast, delete n.output.code;
                var _ = Pn(n.output);
                if (c.print(_), p.code = _.get(), n.sourceMap)
                    if (n.sourceMap.asObject ? p.map = n.output.source_map.get().toJSON() : p.map = n.output.source_map.toString(), "inline" == n.sourceMap.url) {
                        var d = "object" == typeof p.map ? JSON.stringify(p.map) : p.map;
                        p.code += "\n//# sourceMappingURL=data:application/json;charset=utf-8;base64," + po(d);
                    }
                    else
                        n.sourceMap.url && (p.code += "\n//# sourceMappingURL=" + n.sourceMap.url);
            }
            return n.nameCache && n.mangle && (n.mangle.cache && (n.nameCache.vars = ho(n.mangle.cache)), n.mangle.properties && n.mangle.properties.cache && (n.nameCache.props = ho(n.mangle.properties.cache))), u && (u.end = Date.now(), p.timings = { parse: .001 * (u.rename - u.parse), rename: .001 * (u.compress - u.rename), compress: .001 * (u.scope - u.compress), scope: .001 * (u.mangle - u.scope), mangle: .001 * (u.properties - u.mangle), properties: .001 * (u.output - u.properties), output: .001 * (u.end - u.output), total: .001 * (u.end - u.start) }), l.length && (p.warnings = l), p;
        }
        catch (e) {
            return { error: e };
        }
        finally {
            fe.warn_function = a;
        } }
        !function () { var e = function (e) { for (var t = !0, n = 0; n < e.length; n++)
            t && e[n] instanceof pe && e[n].body instanceof ln ? e[n] = new de({ start: e[n].start, end: e[n].end, value: e[n].body.value }) : !t || e[n] instanceof pe && e[n].body instanceof ln || (t = !1); return e; }, t = { Program: function (t) { return new xe({ start: i(t), end: o(t), body: e(t.body.map(s)) }); }, ArrayPattern: function (e) { return new Be({ start: i(e), end: o(e), names: e.elements.map((function (e) { return null === e ? new gn : s(e); })), is_array: !0 }); }, ObjectPattern: function (e) { return new Be({ start: i(e), end: o(e), names: e.properties.map(s), is_array: !1 }); }, AssignmentPattern: function (e) { var t = Tt; if (a.length > 2) {
                var n = a[a.length - 2];
                "FunctionDeclaration" !== n.type && "FunctionExpression" !== n.type && "ArrowFunctionExpression" !== n.type || (t = Ct);
            } return new t({ start: i(e), end: o(e), left: s(e.left), operator: "=", right: s(e.right) }); }, SpreadElement: function (e) { return new Ne({ start: i(e), end: o(e), expression: s(e.argument) }); }, RestElement: function (e) { return new Ne({ start: i(e), end: o(e), expression: s(e.argument) }); }, TemplateElement: function (e) { return new Ge({ start: i(e), end: o(e), value: e.value.cooked, raw: e.value.raw }); }, TemplateLiteral: function (e) { for (var t = [], n = 0; n < e.quasis.length; n++)
                t.push(s(e.quasis[n])), e.expressions[n] && t.push(s(e.expressions[n])); return new Ue({ start: i(e), end: o(e), segments: t }); }, TaggedTemplateExpression: function (e) { return new Ke({ start: i(e), end: o(e), template_string: s(e.quasi), prefix: s(e.tag) }); }, FunctionDeclaration: function (t) { return new Pe({ start: i(t), end: o(t), name: s(t.id), argnames: t.params.map(s), is_generator: t.generator, async: t.async, body: e(s(t.body).body) }); }, FunctionExpression: function (t) { return new Le({ start: i(t), end: o(t), name: s(t.id), argnames: t.params.map(s), is_generator: t.generator, async: t.async, body: e(s(t.body).body) }); }, ArrowFunctionExpression: function (e) { const t = "BlockStatement" === e.body.type ? s(e.body).body : [p(ze, {}, { value: s(e.body) })]; return new Ve({ start: i(e), end: o(e), argnames: e.params.map(s), body: t, async: e.async }); }, ExpressionStatement: function (e) { return new me({ start: i(e), end: o(e), body: s(e.expression) }); }, TryStatement: function (e) { var t = e.handlers || [e.handler]; if (t.length > 1 || e.guardedHandlers && e.guardedHandlers.length)
                throw new Error("Multiple catch clauses are not supported."); return new it({ start: i(e), end: o(e), body: s(e.block).body, bcatch: s(t[0]), bfinally: e.finalizer ? new rt(s(e.finalizer)) : null }); }, Property: function (e) { var t = e.key, n = { start: i(t || e.value), end: o(e.value), key: "Identifier" == t.type ? t.name : t.value, value: s(e.value) }; return e.computed && (n.key = s(e.key)), e.method ? (n.is_generator = e.value.generator, n.async = e.value.async, e.computed ? n.key = s(e.key) : n.key = new Yt({ name: n.key }), new Nt(n)) : "init" == e.kind ? ("Identifier" != t.type && "Literal" != t.type && (n.key = s(t)), new Rt(n)) : ("string" != typeof n.key && "number" != typeof n.key || (n.key = new Yt({ name: n.key })), n.value = new Ie(n.value), "get" == e.kind ? new xt(n) : "set" == e.kind ? new wt(n) : "method" == e.kind ? (n.async = e.value.async, n.is_generator = e.value.generator, n.quote = e.computed ? '"' : null, new Nt(n)) : void 0); }, MethodDefinition: function (e) { var t = { start: i(e), end: o(e), key: e.computed ? s(e.key) : new Yt({ name: e.key.name || e.key.value }), value: s(e.value), static: e.static }; return "get" == e.kind ? new xt(t) : "set" == e.kind ? new wt(t) : (t.is_generator = e.value.generator, t.async = e.value.async, new Nt(t)); }, FieldDefinition: function (e) { let t; if (e.computed)
                t = s(e.key);
            else {
                if ("Identifier" !== e.key.type)
                    throw new Error("Non-Identifier key in FieldDefinition");
                t = s(e.key);
            } return new It({ start: i(e), end: o(e), key: t, value: s(e.value), static: e.static }); }, ArrayExpression: function (e) { return new Ot({ start: i(e), end: o(e), elements: e.elements.map((function (e) { return null === e ? new gn : s(e); })) }); }, ObjectExpression: function (e) { return new Ft({ start: i(e), end: o(e), properties: e.properties.map((function (e) { return "SpreadElement" === e.type || (e.type = "Property"), s(e); })) }); }, SequenceExpression: function (e) { return new ht({ start: i(e), end: o(e), expressions: e.expressions.map(s) }); }, MemberExpression: function (e) { return new (e.computed ? Dt : gt)({ start: i(e), end: o(e), property: e.computed ? s(e.property) : e.property.name, expression: s(e.object) }); }, SwitchCase: function (e) { return new (e.test ? nt : tt)({ start: i(e), end: o(e), expression: s(e.test), body: e.consequent.map(s) }); }, VariableDeclaration: function (e) { return new ("const" === e.kind ? ct : "let" === e.kind ? ut : st)({ start: i(e), end: o(e), definitions: e.declarations.map(s) }); }, ImportDeclaration: function (e) { var t = null, n = null; return e.specifiers.forEach((function (e) { "ImportSpecifier" === e.type ? (n || (n = []), n.push(new ft({ start: i(e), end: o(e), foreign_name: s(e.imported), name: s(e.local) }))) : "ImportDefaultSpecifier" === e.type ? t = s(e.local) : "ImportNamespaceSpecifier" === e.type && (n || (n = []), n.push(new ft({ start: i(e), end: o(e), foreign_name: new en({ name: "*" }), name: s(e.local) }))); })), new pt({ start: i(e), end: o(e), imported_name: t, imported_names: n, module_name: s(e.source) }); }, ExportAllDeclaration: function (e) { return new _t({ start: i(e), end: o(e), exported_names: [new ft({ name: new rn({ name: "*" }), foreign_name: new rn({ name: "*" }) })], module_name: s(e.source) }); }, ExportNamedDeclaration: function (e) { return new _t({ start: i(e), end: o(e), exported_definition: s(e.declaration), exported_names: e.specifiers && e.specifiers.length ? e.specifiers.map((function (e) { return new ft({ foreign_name: s(e.exported), name: s(e.local) }); })) : null, module_name: s(e.source) }); }, ExportDefaultDeclaration: function (e) { return new _t({ start: i(e), end: o(e), exported_value: s(e.declaration), is_default: !0 }); }, Literal: function (e) { var t = e.value, n = { start: i(e), end: o(e) }, r = e.regex; if (r && r.pattern)
                return n.value = { source: r.pattern, flags: r.flags }, new _n(n); if (r) {
                const i = e.raw || t, o = i.match(/^\/(.*)\/(\w*)$/);
                if (!o)
                    throw new Error("Invalid regex source " + i);
                const [r, a, s] = o;
                return n.value = { source: a, flags: s }, new _n(n);
            } if (null === t)
                return new mn(n); switch (typeof t) {
                case "string": return n.value = t, new ln(n);
                case "number": return n.value = t, new fn(n);
                case "boolean": return new (t ? An : vn)(n);
            } }, MetaProperty: function (e) { if ("new" === e.meta.name && "target" === e.property.name)
                return new Bt({ start: i(e), end: o(e) }); }, Identifier: function (e) { var t = a[a.length - 2]; return new ("LabeledStatement" == t.type ? tn : "VariableDeclarator" == t.type && t.id === e ? "const" == t.kind ? Ht : "let" == t.kind ? Xt : Ut : /Import.*Specifier/.test(t.type) ? t.local === e ? Qt : en : "ExportSpecifier" == t.type ? t.local === e ? on : rn : "FunctionExpression" == t.type ? t.id === e ? $t : zt : "FunctionDeclaration" == t.type ? t.id === e ? Wt : zt : "ArrowFunctionExpression" == t.type ? t.params.includes(e) ? zt : nn : "ClassExpression" == t.type ? t.id === e ? Zt : nn : "Property" == t.type ? t.key === e && t.computed || t.value === e ? nn : Yt : "FieldDefinition" == t.type ? t.key === e && t.computed || t.value === e ? nn : qt : "ClassDeclaration" == t.type ? t.id === e ? jt : nn : "MethodDefinition" == t.type ? t.computed ? nn : Yt : "CatchClause" == t.type ? Jt : "BreakStatement" == t.type || "ContinueStatement" == t.type ? an : nn)({ start: i(e), end: o(e), name: e.name }); }, BigIntLiteral: e => new pn({ start: i(e), end: o(e), value: e.value }) }; function n(e) { if ("Literal" == e.type)
            return null != e.raw ? e.raw : e.value + ""; } function i(e) { var t = e.loc, i = t && t.start, o = e.range; return new le({ file: t && t.source, line: i && i.line, col: i && i.column, pos: o ? o[0] : e.start, endline: i && i.line, endcol: i && i.column, endpos: o ? o[0] : e.start, raw: n(e) }); } function o(e) { var t = e.loc, i = t && t.end, o = e.range; return new le({ file: t && t.source, line: i && i.line, col: i && i.column, pos: o ? o[1] : e.end, endline: i && i.line, endcol: i && i.column, endpos: o ? o[1] : e.end, raw: n(e) }); } function r(e, n, r) { var a = "function From_Moz_" + e + "(M){\n"; a += "return new U2." + n.name + "({\nstart: my_start_token(M),\nend: my_end_token(M)"; var c = "function To_Moz_" + e + "(M){\n"; c += "return {\ntype: " + JSON.stringify(e), r && r.split(/\s*,\s*/).forEach((function (e) { var t = /([a-z0-9$_]+)([=@>%])([a-z0-9$_]+)/i.exec(e); if (!t)
            throw new Error("Can't understand property map: " + e); var n = t[1], i = t[2], o = t[3]; switch (a += ",\n" + o + ": ", c += ",\n" + n + ": ", i) {
            case "@":
                a += "M." + n + ".map(from_moz)", c += "M." + o + ".map(to_moz)";
                break;
            case ">":
                a += "from_moz(M." + n + ")", c += "to_moz(M." + o + ")";
                break;
            case "=":
                a += "M." + n, c += "M." + o;
                break;
            case "%":
                a += "from_moz(M." + n + ").body", c += "to_moz_block(M)";
                break;
            default: throw new Error("Can't understand operator in propmap: " + e);
        } })), a += "\n})\n}", c += "\n}\n}", a = new Function("U2", "my_start_token", "my_end_token", "from_moz", "return(" + a + ")")(wn, i, o, s), c = new Function("to_moz", "to_moz_block", "to_moz_scope", "return(" + c + ")")(l, _, d), t[e] = a, u(n, c); } t.UpdateExpression = t.UnaryExpression = function (e) { return new (("prefix" in e ? e.prefix : "UnaryExpression" == e.type) ? vt : At)({ start: i(e), end: o(e), operator: e.operator, expression: s(e.argument) }); }, t.ClassDeclaration = t.ClassExpression = function (e) { return new ("ClassDeclaration" === e.type ? Lt : Vt)({ start: i(e), end: o(e), name: s(e.id), extends: s(e.superClass), properties: e.body.body.map(s) }); }, r("EmptyStatement", Se), r("BlockStatement", De, "body@body"), r("IfStatement", Je, "test>condition, consequent>body, alternate>alternative"), r("LabeledStatement", Ae, "label>label, body>body"), r("BreakStatement", qe, "label>label"), r("ContinueStatement", $e, "label>label"), r("WithStatement", Re, "object>expression, body>body"), r("SwitchStatement", Qe, "discriminant>expression, cases@body"), r("ReturnStatement", ze, "argument>value"), r("ThrowStatement", We, "argument>value"), r("WhileStatement", Ce, "test>condition, body>body"), r("DoWhileStatement", be, "test>condition, body>body"), r("ForStatement", Oe, "init>init, test>condition, update>step, body>body"), r("ForInStatement", Fe, "left>init, right>object, body>body"), r("ForOfStatement", Me, "left>init, right>object, body>body, await=await"), r("AwaitExpression", je, "argument>expression"), r("YieldExpression", Ze, "argument>expression, delegate=is_star"), r("DebuggerStatement", _e), r("VariableDeclarator", lt, "id>name, init>value"), r("CatchClause", ot, "param>argname, body%body"), r("ThisExpression", sn), r("Super", un), r("BinaryExpression", Tt, "operator=operator, left>left, right>right"), r("LogicalExpression", Tt, "operator=operator, left>left, right>right"), r("AssignmentExpression", bt, "operator=operator, left>left, right>right"), r("ConditionalExpression", yt, "test>condition, consequent>consequent, alternate>alternative"), r("NewExpression", mt, "callee>expression, arguments@args"), r("CallExpression", dt, "callee>expression, arguments@args"), u(xe, (function (e) { return d("Program", e); })), u(Ne, (function (e) { return { type: f() ? "RestElement" : "SpreadElement", argument: l(e.expression) }; })), u(Ke, (function (e) { return { type: "TaggedTemplateExpression", tag: l(e.prefix), quasi: l(e.template_string) }; })), u(Ue, (function (e) { for (var t = [], n = [], i = 0; i < e.segments.length; i++)
            i % 2 != 0 ? n.push(l(e.segments[i])) : t.push({ type: "TemplateElement", value: { raw: e.segments[i].raw, cooked: e.segments[i].value }, tail: i === e.segments.length - 1 }); return { type: "TemplateLiteral", quasis: t, expressions: n }; })), u(Pe, (function (e) { return { type: "FunctionDeclaration", id: l(e.name), params: e.argnames.map(l), generator: e.is_generator, async: e.async, body: d("BlockStatement", e) }; })), u(Le, (function (e, t) { var n = void 0 !== t.is_generator ? t.is_generator : e.is_generator; return { type: "FunctionExpression", id: l(e.name), params: e.argnames.map(l), generator: n, async: e.async, body: d("BlockStatement", e) }; })), u(Ve, (function (e) { var t = { type: "BlockStatement", body: e.body.map(l) }; return { type: "ArrowFunctionExpression", params: e.argnames.map(l), async: e.async, body: t }; })), u(Be, (function (e) { return e.is_array ? { type: "ArrayPattern", elements: e.names.map(l) } : { type: "ObjectPattern", properties: e.names.map(l) }; })), u(de, (function (e) { return { type: "ExpressionStatement", expression: { type: "Literal", value: e.value } }; })), u(me, (function (e) { return { type: "ExpressionStatement", expression: l(e.body) }; })), u(et, (function (e) { return { type: "SwitchCase", test: l(e.expression), consequent: e.body.map(l) }; })), u(it, (function (e) { return { type: "TryStatement", block: _(e), handler: l(e.bcatch), guardedHandlers: [], finalizer: l(e.bfinally) }; })), u(ot, (function (e) { return { type: "CatchClause", param: l(e.argname), guard: null, body: _(e) }; })), u(at, (function (e) { return { type: "VariableDeclaration", kind: e instanceof ct ? "const" : e instanceof ut ? "let" : "var", declarations: e.definitions.map(l) }; })), u(_t, (function (e) { return e.exported_names ? "*" === e.exported_names[0].name.name ? { type: "ExportAllDeclaration", source: l(e.module_name) } : { type: "ExportNamedDeclaration", specifiers: e.exported_names.map((function (e) { return { type: "ExportSpecifier", exported: l(e.foreign_name), local: l(e.name) }; })), declaration: l(e.exported_definition), source: l(e.module_name) } : { type: e.is_default ? "ExportDefaultDeclaration" : "ExportNamedDeclaration", declaration: l(e.exported_value || e.exported_definition) }; })), u(pt, (function (e) { var t = []; return e.imported_name && t.push({ type: "ImportDefaultSpecifier", local: l(e.imported_name) }), e.imported_names && "*" === e.imported_names[0].foreign_name.name ? t.push({ type: "ImportNamespaceSpecifier", local: l(e.imported_names[0].name) }) : e.imported_names && e.imported_names.forEach((function (e) { t.push({ type: "ImportSpecifier", local: l(e.name), imported: l(e.foreign_name) }); })), { type: "ImportDeclaration", specifiers: t, source: l(e.module_name) }; })), u(ht, (function (e) { return { type: "SequenceExpression", expressions: e.expressions.map(l) }; })), u(Et, (function (e) { var t = e instanceof Dt; return { type: "MemberExpression", object: l(e.expression), computed: t, property: t ? l(e.property) : { type: "Identifier", name: e.property } }; })), u(St, (function (e) { return { type: "++" == e.operator || "--" == e.operator ? "UpdateExpression" : "UnaryExpression", operator: e.operator, prefix: e instanceof vt, argument: l(e.expression) }; })), u(Tt, (function (e) { return "=" == e.operator && f() ? { type: "AssignmentPattern", left: l(e.left), right: l(e.right) } : { type: "&&" == e.operator || "||" == e.operator ? "LogicalExpression" : "BinaryExpression", left: l(e.left), operator: e.operator, right: l(e.right) }; })), u(Ot, (function (e) { return { type: "ArrayExpression", elements: e.elements.map(l) }; })), u(Ft, (function (e) { return { type: "ObjectExpression", properties: e.properties.map(l) }; })), u(Mt, (function (e, t) { var n, i = e.key instanceof fe ? l(e.key) : { type: "Identifier", value: e.key }; "number" == typeof e.key && (i = { type: "Literal", value: Number(e.key) }), "string" == typeof e.key && (i = { type: "Identifier", name: e.key }); var o = "string" == typeof e.key || "number" == typeof e.key, r = !o && (!(e.key instanceof Pt) || e.key instanceof nn); return e instanceof Rt ? (n = "init", r = !o) : e instanceof xt ? n = "get" : e instanceof wt && (n = "set"), e instanceof It ? { type: "FieldDefinition", computed: r, key: i, value: l(e.value), static: e.static } : t instanceof kt ? { type: "MethodDefinition", computed: r, kind: n, static: e.static, key: l(e.key), value: l(e.value) } : { type: "Property", computed: r, kind: n, key: i, value: l(e.value) }; })), u(Nt, (function (e, t) { return t instanceof Ft ? { type: "Property", computed: !(e.key instanceof Pt) || e.key instanceof nn, kind: "init", method: !0, shorthand: !1, key: l(e.key), value: l(e.value) } : { type: "MethodDefinition", computed: !(e.key instanceof Pt) || e.key instanceof nn, kind: "constructor" === e.key ? "constructor" : "method", static: e.static, key: l(e.key), value: l(e.value) }; })), u(kt, (function (e) { return { type: e instanceof Vt ? "ClassExpression" : "ClassDeclaration", superClass: l(e.extends), id: e.name ? l(e.name) : null, body: { type: "ClassBody", body: e.properties.map(l) } }; })), u(Bt, (function () { return { type: "MetaProperty", meta: { type: "Identifier", name: "new" }, property: { type: "Identifier", name: "target" } }; })), u(Pt, (function (e, t) { if (e instanceof Yt && t.quote)
            return { type: "Literal", value: e.name }; var n = e.definition(); return { type: "Identifier", name: n ? n.mangled_name || n.name : e.name }; })), u(_n, (function (e) { const t = e.value.source, n = e.value.flags; return { type: "Literal", value: null, raw: e.print_to_string(), regex: { pattern: t, flags: n } }; })), u(cn, (function (e) { var t = e.value; return "number" == typeof t && (t < 0 || 0 === t && 1 / t < 0) ? { type: "UnaryExpression", operator: "-", prefix: !0, argument: { type: "Literal", value: -t, raw: e.start.raw } } : { type: "Literal", value: t, raw: e.start.raw }; })), u(dn, (function (e) { return { type: "Identifier", name: String(e.value) }; })), u(pn, e => ({ type: "BigIntLiteral", value: e.value })), Sn.DEFMETHOD("to_mozilla_ast", cn.prototype.to_mozilla_ast), mn.DEFMETHOD("to_mozilla_ast", cn.prototype.to_mozilla_ast), gn.DEFMETHOD("to_mozilla_ast", (function () { return null; })), ge.DEFMETHOD("to_mozilla_ast", De.prototype.to_mozilla_ast), ke.DEFMETHOD("to_mozilla_ast", Le.prototype.to_mozilla_ast); var a = null; function s(e) { a.push(e); var n = null != e ? t[e.type](e) : null; return a.pop(), n; } function u(e, t) { e.DEFMETHOD("to_mozilla_ast", (function (e) { return n = this, i = t(this, e), o = n.start, r = n.end, o && r ? (null != o.pos && null != r.endpos && (i.range = [o.pos, r.endpos]), o.line && (i.loc = { start: { line: o.line, column: o.col }, end: r.endline ? { line: r.endline, column: r.endcol } : null }, o.file && (i.loc.source = o.file)), i) : i; var n, i, o, r; })); } fe.from_mozilla_ast = function (e) { var t = a; a = []; var n = s(e); return a = t, n; }; var c = null; function l(e) { null === c && (c = []), c.push(e); var t = null != e ? e.to_mozilla_ast(c[c.length - 2]) : null; return c.pop(), 0 === c.length && (c = null), t; } function f() { for (var e = c.length; e--;)
            if (c[e] instanceof Be)
                return !0; return !1; } function _(e) { return { type: "BlockStatement", body: e.body.map(l) }; } function d(e, t) { var n = t.body.map(l); return t.body[0] instanceof me && t.body[0].body instanceof ln && n.unshift(l(new Se(t.body[0]))), { type: e, body: n }; } }(), e.AST_Accessor = Ie, e.AST_Array = Ot, e.AST_Arrow = Ve, e.AST_Assign = bt, e.AST_Atom = dn, e.AST_Await = je, e.AST_Binary = Tt, e.AST_Block = ge, e.AST_BlockStatement = De, e.AST_Boolean = Sn, e.AST_Break = qe, e.AST_Call = dt, e.AST_Case = nt, e.AST_Catch = ot, e.AST_Class = kt, e.AST_ClassExpression = Vt, e.AST_ConciseMethod = Nt, e.AST_Conditional = yt, e.AST_Const = ct, e.AST_Constant = cn, e.AST_Continue = $e, e.AST_DWLoop = ye, e.AST_Debugger = _e, e.AST_DefClass = Lt, e.AST_Default = tt, e.AST_DefaultAssign = Ct, e.AST_Definitions = at, e.AST_Defun = Pe, e.AST_Destructuring = Be, e.AST_Directive = de, e.AST_Do = be, e.AST_Dot = gt, e.AST_EmptyStatement = Se, e.AST_Exit = Xe, e.AST_Expansion = Ne, e.AST_Export = _t, e.AST_False = vn, e.AST_Finally = rt, e.AST_For = Oe, e.AST_ForIn = Fe, e.AST_ForOf = Me, e.AST_Function = Le, e.AST_Hole = gn, e.AST_If = Je, e.AST_Import = pt, e.AST_Infinity = Dn, e.AST_IterationStatement = Te, e.AST_Jump = He, e.AST_Label = tn, e.AST_LabelRef = an, e.AST_LabeledStatement = Ae, e.AST_Lambda = ke, e.AST_Let = ut, e.AST_LoopControl = Ye, e.AST_NaN = hn, e.AST_NameMapping = ft, e.AST_New = mt, e.AST_NewTarget = Bt, e.AST_Node = fe, e.AST_Null = mn, e.AST_Number = fn, e.AST_Object = Ft, e.AST_ObjectGetter = xt, e.AST_ObjectKeyVal = Rt, e.AST_ObjectProperty = Mt, e.AST_ObjectSetter = wt, e.AST_PrefixedTemplateString = Ke, e.AST_PropAccess = Et, e.AST_RegExp = _n, e.AST_Return = ze, e.AST_Scope = we, e.AST_Sequence = ht, e.AST_SimpleStatement = me, e.AST_Statement = pe, e.AST_StatementWithBody = ve, e.AST_String = ln, e.AST_Sub = Dt, e.AST_Super = un, e.AST_Switch = Qe, e.AST_SwitchBranch = et, e.AST_Symbol = Pt, e.AST_SymbolBlockDeclaration = Gt, e.AST_SymbolCatch = Jt, e.AST_SymbolClass = Zt, e.AST_SymbolConst = Ht, e.AST_SymbolDeclaration = Kt, e.AST_SymbolDefClass = jt, e.AST_SymbolDefun = Wt, e.AST_SymbolExport = on, e.AST_SymbolExportForeign = rn, e.AST_SymbolFunarg = zt, e.AST_SymbolImport = Qt, e.AST_SymbolImportForeign = en, e.AST_SymbolLambda = $t, e.AST_SymbolLet = Xt, e.AST_SymbolMethod = Yt, e.AST_SymbolRef = nn, e.AST_SymbolVar = Ut, e.AST_TemplateSegment = Ge, e.AST_TemplateString = Ue, e.AST_This = sn, e.AST_Throw = We, e.AST_Token = le, e.AST_Toplevel = xe, e.AST_True = An, e.AST_Try = it, e.AST_Unary = St, e.AST_UnaryPostfix = At, e.AST_UnaryPrefix = vt, e.AST_Undefined = En, e.AST_Var = st, e.AST_VarDef = lt, e.AST_While = Ce, e.AST_With = Re, e.AST_Yield = Ze, e.Compressor = ri, e.OutputStream = Pn, e.TreeTransformer = On, e.TreeWalker = Cn, e._INLINE = Mn, e._JS_Parse_Error = J, e._NOINLINE = Rn, e._PURE = Fn, e._has_annotation = T, e._tokenizer = ne, e.base54 = Yn, e.default_options = qn, e.defaults = r, e.mangle_properties = lo, e.minify = Eo, e.parse = ue, e.push_uniq = _, e.reserve_quoted_keys = uo, e.string_template = d, e.to_ascii = fo;
    }));
},
/* combine-source-map/index.js */ function _(require, module, exports) {
    var path = require('path');
    var convert = require(69) /* convert-source-map */;
    var memoize = require(71) /* lodash.memoize */;
    var createGenerator = require(72) /* inline-source-map */;
    var pathIsAbsolute = require(73) /* ./lib/path-is-absolute */;
    var mappingsFromMap = require(74) /* ./lib/mappings-from-map */;
    var protocolRx = /^[a-z]+:\/\//;
    /**
     * Rebases a relative path in 'sourceFile' to be relative
     * to the path where 'sourceFile' is located.
     *
     * This is necessary before adding relative paths to the
     * new combined map to ensure all paths are relative to their
     * original source.
     *
     * The 'sourceRoot' from the original source map is joined
     * as well to ensure the complete path.
     *
     * Resulting paths that are absolute are passed along directly.
     *
     * @param sourceFile {String} path to the original source file that references a map
     * @param relativeRoot {String} sourceRoot in sourceFile's map to combine with relativePath
     * @param relativePath {String} source path from sourceFile's map
     */
    var rebaseRelativePath = memoize(function (sourceFile, relativeRoot, relativePath) {
        if (!relativePath) {
            return relativePath;
        }
        // join relative path to root (e.g. 'src/' + 'file.js')
        var relativeRootedPath = relativeRoot ? path.join(relativeRoot, relativePath) : relativePath;
        relativeRootedPath = relativeRootedPath.replace(/\\/g, '/');
        sourceFile = sourceFile.replace(/\\/g, '/');
        if (sourceFile === relativeRootedPath || // same path,
            pathIsAbsolute(relativeRootedPath) || // absolute path, nor
            protocolRx.test(relativeRootedPath)) { // absolute protocol need rebasing
            return relativeRootedPath;
        }
        // make relative to source file
        return path.join(path.dirname(sourceFile), relativeRootedPath).replace(/\\/g, '/');
    }, function (a, b, c) {
        return a + '::' + b + '::' + c;
    });
    function resolveMap(source) {
        var gen = convert.fromSource(source);
        return gen ? gen.toObject() : null;
    }
    function hasInlinedSource(existingMap) {
        return existingMap.sourcesContent && !!existingMap.sourcesContent[0];
    }
    function Combiner(file, sourceRoot) {
        // since we include the original code in the map sourceRoot actually not needed
        this.generator = createGenerator({ file: file || 'generated.js', sourceRoot: sourceRoot });
    }
    Combiner.prototype._addGeneratedMap = function (sourceFile, source, offset) {
        this.generator.addGeneratedMappings(sourceFile, source, offset);
        this.generator.addSourceContent(sourceFile, source);
        return this;
    };
    Combiner.prototype._addExistingMap = function (sourceFile, source, existingMap, offset) {
        var mappings = mappingsFromMap(existingMap);
        // add all of the sources from the map
        for (var i = 0, len = existingMap.sources.length; i < len; i++) {
            if (!existingMap.sourcesContent)
                continue;
            this.generator.addSourceContent(rebaseRelativePath(sourceFile, existingMap.sourceRoot, existingMap.sources[i]), existingMap.sourcesContent[i]);
        }
        // add the mappings, preserving the original mapping 'source'
        mappings.forEach(function (mapping) {
            // Add the mappings one at a time because 'inline-source-map' doesn't handle
            // mapping source filenames. The mapping.source already takes sourceRoot into account
            // per the SMConsumer.eachMapping function, so pass null for the root here.
            this.generator.addMappings(rebaseRelativePath(sourceFile, null, mapping.source), [mapping], offset);
        }, this);
        return this;
    };
    /**
     * Adds map to underlying source map.
     * If source contains a source map comment that has the source of the original file inlined it will offset these
     * mappings and include them.
     * If no source map comment is found or it has no source inlined, mappings for the file will be generated and included
     *
     * @name addMap
     * @function
     * @param opts {Object} { sourceFile: {String}, source: {String} }
     * @param offset {Object} { line: {Number}, column: {Number} }
     */
    Combiner.prototype.addFile = function (opts, offset) {
        offset = offset || {};
        if (!offset.hasOwnProperty('line'))
            offset.line = 0;
        if (!offset.hasOwnProperty('column'))
            offset.column = 0;
        var existingMap = resolveMap(opts.source);
        return existingMap && hasInlinedSource(existingMap)
            ? this._addExistingMap(opts.sourceFile, opts.source, existingMap, offset)
            : this._addGeneratedMap(opts.sourceFile, opts.source, offset);
    };
    /**
    * @name base64
    * @function
    * @return {String} base64 encoded combined source map
    */
    Combiner.prototype.base64 = function () {
        return this.generator.base64Encode();
    };
    /**
     * @name comment
     * @function
     * @return {String} base64 encoded sourceMappingUrl comment of the combined source map
     */
    Combiner.prototype.comment = function () {
        return this.generator.inlineMappingUrl();
    };
    /**
     * @name create
     * @function
     * @param file {String} optional name of the generated file
     * @param sourceRoot {String} optional sourceRoot of the map to be generated
     * @return {Object} Combiner instance to which source maps can be added and later combined
     */
    exports.create = function (file, sourceRoot) { return new Combiner(file, sourceRoot); };
    /**
     * @name removeComments
     * @function
     * @param src
     * @return {String} src with all sourceMappingUrl comments removed
     */
    exports.removeComments = function (src) {
        if (!src.replace)
            return src;
        return src.replace(convert.commentRegex, '').replace(convert.mapFileCommentRegex, '');
    };
},
/* convert-source-map/index.js */ function _(require, module, exports) {
    var fs = require('fs');
    var path = require('path');
    var SafeBuffer = require(70) /* safe-buffer */;
    Object.defineProperty(exports, 'commentRegex', {
        get: function getCommentRegex() {
            return /^\s*\/(?:\/|\*)[@#]\s+sourceMappingURL=data:(?:application|text)\/json;(?:charset[:=]\S+?;)?base64,(?:.*)$/mg;
        }
    });
    Object.defineProperty(exports, 'mapFileCommentRegex', {
        get: function getMapFileCommentRegex() {
            // Matches sourceMappingURL in either // or /* comment styles.
            return /(?:\/\/[@#][ \t]+sourceMappingURL=([^\s'"`]+?)[ \t]*$)|(?:\/\*[@#][ \t]+sourceMappingURL=([^\*]+?)[ \t]*(?:\*\/){1}[ \t]*$)/mg;
        }
    });
    function decodeBase64(base64) {
        return SafeBuffer.Buffer.from(base64, 'base64').toString();
    }
    function stripComment(sm) {
        return sm.split(',').pop();
    }
    function readFromFileMap(sm, dir) {
        // NOTE: this will only work on the server since it attempts to read the map file
        var r = exports.mapFileCommentRegex.exec(sm);
        // for some odd reason //# .. captures in 1 and /* .. */ in 2
        var filename = r[1] || r[2];
        var filepath = path.resolve(dir, filename);
        try {
            return fs.readFileSync(filepath, 'utf8');
        }
        catch (e) {
            throw new Error('An error occurred while trying to read the map file at ' + filepath + '\n' + e);
        }
    }
    function Converter(sm, opts) {
        opts = opts || {};
        if (opts.isFileComment)
            sm = readFromFileMap(sm, opts.commentFileDir);
        if (opts.hasComment)
            sm = stripComment(sm);
        if (opts.isEncoded)
            sm = decodeBase64(sm);
        if (opts.isJSON || opts.isEncoded)
            sm = JSON.parse(sm);
        this.sourcemap = sm;
    }
    Converter.prototype.toJSON = function (space) {
        return JSON.stringify(this.sourcemap, null, space);
    };
    Converter.prototype.toBase64 = function () {
        var json = this.toJSON();
        return SafeBuffer.Buffer.from(json, 'utf8').toString('base64');
    };
    Converter.prototype.toComment = function (options) {
        var base64 = this.toBase64();
        var data = 'sourceMappingURL=data:application/json;charset=utf-8;base64,' + base64;
        return options && options.multiline ? '/*# ' + data + ' */' : '//# ' + data;
    };
    // returns copy instead of original
    Converter.prototype.toObject = function () {
        return JSON.parse(this.toJSON());
    };
    Converter.prototype.addProperty = function (key, value) {
        if (this.sourcemap.hasOwnProperty(key))
            throw new Error('property "' + key + '" already exists on the sourcemap, use set property instead');
        return this.setProperty(key, value);
    };
    Converter.prototype.setProperty = function (key, value) {
        this.sourcemap[key] = value;
        return this;
    };
    Converter.prototype.getProperty = function (key) {
        return this.sourcemap[key];
    };
    exports.fromObject = function (obj) {
        return new Converter(obj);
    };
    exports.fromJSON = function (json) {
        return new Converter(json, { isJSON: true });
    };
    exports.fromBase64 = function (base64) {
        return new Converter(base64, { isEncoded: true });
    };
    exports.fromComment = function (comment) {
        comment = comment
            .replace(/^\/\*/g, '//')
            .replace(/\*\/$/g, '');
        return new Converter(comment, { isEncoded: true, hasComment: true });
    };
    exports.fromMapFileComment = function (comment, dir) {
        return new Converter(comment, { commentFileDir: dir, isFileComment: true, isJSON: true });
    };
    // Finds last sourcemap comment in file or returns null if none was found
    exports.fromSource = function (content) {
        var m = content.match(exports.commentRegex);
        return m ? exports.fromComment(m.pop()) : null;
    };
    // Finds last sourcemap comment in file or returns null if none was found
    exports.fromMapFileSource = function (content, dir) {
        var m = content.match(exports.mapFileCommentRegex);
        return m ? exports.fromMapFileComment(m.pop(), dir) : null;
    };
    exports.removeComments = function (src) {
        return src.replace(exports.commentRegex, '');
    };
    exports.removeMapFileComments = function (src) {
        return src.replace(exports.mapFileCommentRegex, '');
    };
    exports.generateMapFileComment = function (file, options) {
        var data = 'sourceMappingURL=' + file;
        return options && options.multiline ? '/*# ' + data + ' */' : '//# ' + data;
    };
},
/* safe-buffer/index.js */ function _(require, module, exports) {
    /* eslint-disable node/no-deprecated-api */
    var buffer = require('buffer');
    var Buffer = buffer.Buffer;
    // alternative to using Object.keys for old browsers
    function copyProps(src, dst) {
        for (var key in src) {
            dst[key] = src[key];
        }
    }
    if (Buffer.from && Buffer.alloc && Buffer.allocUnsafe && Buffer.allocUnsafeSlow) {
        module.exports = buffer;
    }
    else {
        // Copy properties from require('buffer')
        copyProps(buffer, exports);
        exports.Buffer = SafeBuffer;
    }
    function SafeBuffer(arg, encodingOrOffset, length) {
        return Buffer(arg, encodingOrOffset, length);
    }
    // Copy static methods from Buffer
    copyProps(Buffer, SafeBuffer);
    SafeBuffer.from = function (arg, encodingOrOffset, length) {
        if (typeof arg === 'number') {
            throw new TypeError('Argument must not be a number');
        }
        return Buffer(arg, encodingOrOffset, length);
    };
    SafeBuffer.alloc = function (size, fill, encoding) {
        if (typeof size !== 'number') {
            throw new TypeError('Argument must be a number');
        }
        var buf = Buffer(size);
        if (fill !== undefined) {
            if (typeof encoding === 'string') {
                buf.fill(fill, encoding);
            }
            else {
                buf.fill(fill);
            }
        }
        else {
            buf.fill(0);
        }
        return buf;
    };
    SafeBuffer.allocUnsafe = function (size) {
        if (typeof size !== 'number') {
            throw new TypeError('Argument must be a number');
        }
        return Buffer(size);
    };
    SafeBuffer.allocUnsafeSlow = function (size) {
        if (typeof size !== 'number') {
            throw new TypeError('Argument must be a number');
        }
        return buffer.SlowBuffer(size);
    };
},
/* lodash.memoize/index.js */ function _(require, module, exports) {
    /**
     * lodash 3.0.4 (Custom Build) <https://lodash.com/>
     * Build: `lodash modern modularize exports="npm" -o ./`
     * Copyright 2012-2015 The Dojo Foundation <http://dojofoundation.org/>
     * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
     * Copyright 2009-2015 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
     * Available under MIT license <https://lodash.com/license>
     */
    /** Used as the `TypeError` message for "Functions" methods. */
    var FUNC_ERROR_TEXT = 'Expected a function';
    /** Used for native method references. */
    var objectProto = Object.prototype;
    /** Used to check objects for own properties. */
    var hasOwnProperty = objectProto.hasOwnProperty;
    /**
     * Creates a cache object to store key/value pairs.
     *
     * @private
     * @static
     * @name Cache
     * @memberOf _.memoize
     */
    function MapCache() {
        this.__data__ = {};
    }
    /**
     * Removes `key` and its value from the cache.
     *
     * @private
     * @name delete
     * @memberOf _.memoize.Cache
     * @param {string} key The key of the value to remove.
     * @returns {boolean} Returns `true` if the entry was removed successfully, else `false`.
     */
    function mapDelete(key) {
        return this.has(key) && delete this.__data__[key];
    }
    /**
     * Gets the cached value for `key`.
     *
     * @private
     * @name get
     * @memberOf _.memoize.Cache
     * @param {string} key The key of the value to get.
     * @returns {*} Returns the cached value.
     */
    function mapGet(key) {
        return key == '__proto__' ? undefined : this.__data__[key];
    }
    /**
     * Checks if a cached value for `key` exists.
     *
     * @private
     * @name has
     * @memberOf _.memoize.Cache
     * @param {string} key The key of the entry to check.
     * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
     */
    function mapHas(key) {
        return key != '__proto__' && hasOwnProperty.call(this.__data__, key);
    }
    /**
     * Sets `value` to `key` of the cache.
     *
     * @private
     * @name set
     * @memberOf _.memoize.Cache
     * @param {string} key The key of the value to cache.
     * @param {*} value The value to cache.
     * @returns {Object} Returns the cache object.
     */
    function mapSet(key, value) {
        if (key != '__proto__') {
            this.__data__[key] = value;
        }
        return this;
    }
    /**
     * Creates a function that memoizes the result of `func`. If `resolver` is
     * provided it determines the cache key for storing the result based on the
     * arguments provided to the memoized function. By default, the first argument
     * provided to the memoized function is coerced to a string and used as the
     * cache key. The `func` is invoked with the `this` binding of the memoized
     * function.
     *
     * **Note:** The cache is exposed as the `cache` property on the memoized
     * function. Its creation may be customized by replacing the `_.memoize.Cache`
     * constructor with one whose instances implement the [`Map`](https://people.mozilla.org/~jorendorff/es6-draft.html#sec-properties-of-the-map-prototype-object)
     * method interface of `get`, `has`, and `set`.
     *
     * @static
     * @memberOf _
     * @category Function
     * @param {Function} func The function to have its output memoized.
     * @param {Function} [resolver] The function to resolve the cache key.
     * @returns {Function} Returns the new memoizing function.
     * @example
     *
     * var upperCase = _.memoize(function(string) {
     *   return string.toUpperCase();
     * });
     *
     * upperCase('fred');
     * // => 'FRED'
     *
     * // modifying the result cache
     * upperCase.cache.set('fred', 'BARNEY');
     * upperCase('fred');
     * // => 'BARNEY'
     *
     * // replacing `_.memoize.Cache`
     * var object = { 'user': 'fred' };
     * var other = { 'user': 'barney' };
     * var identity = _.memoize(_.identity);
     *
     * identity(object);
     * // => { 'user': 'fred' }
     * identity(other);
     * // => { 'user': 'fred' }
     *
     * _.memoize.Cache = WeakMap;
     * var identity = _.memoize(_.identity);
     *
     * identity(object);
     * // => { 'user': 'fred' }
     * identity(other);
     * // => { 'user': 'barney' }
     */
    function memoize(func, resolver) {
        if (typeof func != 'function' || (resolver && typeof resolver != 'function')) {
            throw new TypeError(FUNC_ERROR_TEXT);
        }
        var memoized = function () {
            var args = arguments, key = resolver ? resolver.apply(this, args) : args[0], cache = memoized.cache;
            if (cache.has(key)) {
                return cache.get(key);
            }
            var result = func.apply(this, args);
            memoized.cache = cache.set(key, result);
            return result;
        };
        memoized.cache = new memoize.Cache;
        return memoized;
    }
    // Add functions to the `Map` cache.
    MapCache.prototype['delete'] = mapDelete;
    MapCache.prototype.get = mapGet;
    MapCache.prototype.has = mapHas;
    MapCache.prototype.set = mapSet;
    // Assign cache to `_.memoize`.
    memoize.Cache = MapCache;
    module.exports = memoize;
},
/* inline-source-map/index.js */ function _(require, module, exports) {
    var SourceMapGenerator = require(46) /* source-map */.SourceMapGenerator;
    function offsetMapping(mapping, offset) {
        return { line: offset.line + mapping.line, column: offset.column + mapping.column };
    }
    function newlinesIn(src) {
        if (!src)
            return 0;
        var newlines = src.match(/\n/g);
        return newlines ? newlines.length : 0;
    }
    function Generator(opts) {
        opts = opts || {};
        this.generator = new SourceMapGenerator({ file: opts.file || '', sourceRoot: opts.sourceRoot || '' });
        this.sourcesContent = undefined;
        this.opts = opts;
    }
    /**
     * Adds the given mappings to the generator and offsets them if offset is given
     *
     * @name addMappings
     * @function
     * @param sourceFile {String} name of the source file
     * @param mappings {Array{{Object}} each object has the form { original: { line: _, column: _ }, generated: { line: _, column: _ } }
     * @param offset {Object} offset to apply to each mapping. Has the form { line: _, column: _ }
     * @return {Object} the generator to allow chaining
     */
    Generator.prototype.addMappings = function (sourceFile, mappings, offset) {
        var generator = this.generator;
        offset = offset || {};
        offset.line = offset.hasOwnProperty('line') ? offset.line : 0;
        offset.column = offset.hasOwnProperty('column') ? offset.column : 0;
        mappings.forEach(function (m) {
            // only set source if we have original position to handle edgecase (see inline-source-map tests)
            generator.addMapping({
                source: m.original ? sourceFile : undefined,
                original: m.original,
                generated: offsetMapping(m.generated, offset)
            });
        });
        return this;
    };
    /**
     * Generates mappings for the given source, assuming that no translation from original to generated is necessary.
     *
     * @name addGeneratedMappings
     * @function
     * @param sourceFile {String} name of the source file
     * @param source {String} source of the file
     * @param offset {Object} offset to apply to each mapping. Has the form { line: _, column: _ }
     * @return {Object} the generator to allow chaining
     */
    Generator.prototype.addGeneratedMappings = function (sourceFile, source, offset) {
        var mappings = [], linesToGenerate = newlinesIn(source) + 1;
        for (var line = 1; line <= linesToGenerate; line++) {
            var location = { line: line, column: 0 };
            mappings.push({ original: location, generated: location });
        }
        return this.addMappings(sourceFile, mappings, offset);
    };
    /**
     * Adds source content for the given source file.
     *
     * @name addSourceContent
     * @function
     * @param sourceFile {String} The source file for which a mapping is included
     * @param sourcesContent {String} The content of the source file
     * @return {Object} The generator to allow chaining
     */
    Generator.prototype.addSourceContent = function (sourceFile, sourcesContent) {
        this.sourcesContent = this.sourcesContent || {};
        this.sourcesContent[sourceFile] = sourcesContent;
        return this;
    };
    /**
     * @name base64Encode
     * @function
     * @return {String} bas64 encoded representation of the added mappings
     */
    Generator.prototype.base64Encode = function () {
        var map = this.toString();
        return new Buffer(map).toString('base64');
    };
    /**
     * @name inlineMappingUrl
     * @function
     * @return {String} comment with base64 encoded representation of the added mappings. Can be inlined at the end of the generated file.
     */
    Generator.prototype.inlineMappingUrl = function () {
        var charset = this.opts.charset || 'utf-8';
        return '//# sourceMappingURL=data:application/json;charset=' + charset + ';base64,' + this.base64Encode();
    };
    Generator.prototype.toJSON = function () {
        var map = this.generator.toJSON();
        if (!this.sourcesContent)
            return map;
        var toSourcesContent = (function (s) {
            if (typeof this.sourcesContent[s] === 'string') {
                return this.sourcesContent[s];
            }
            else {
                return null;
            }
        }).bind(this);
        map.sourcesContent = map.sources.map(toSourcesContent);
        return map;
    };
    Generator.prototype.toString = function () {
        return JSON.stringify(this);
    };
    Generator.prototype._mappings = function () {
        return this.generator._mappings._array;
    };
    Generator.prototype.gen = function () {
        return this.generator;
    };
    module.exports = function (opts) { return new Generator(opts); };
    module.exports.Generator = Generator;
},
/* combine-source-map/lib/path-is-absolute.js */ function _(require, module, exports) {
    function posix(path) {
        return path.charAt(0) === '/';
    }
    ;
    function win32(path) {
        // https://github.com/joyent/node/blob/b3fcc245fb25539909ef1d5eaa01dbf92e168633/lib/path.js#L56
        var splitDeviceRe = /^([a-zA-Z]:|[\\\/]{2}[^\\\/]+[\\\/]+[^\\\/]+)?([\\\/])?([\s\S]*?)$/;
        var result = splitDeviceRe.exec(path);
        var device = result[1] || '';
        var isUnc = !!device && device.charAt(1) !== ':';
        // UNC paths are always absolute
        return !!result[2] || isUnc;
    }
    ;
    module.exports = process.platform === 'win32' ? win32 : posix;
    module.exports.posix = posix;
    module.exports.win32 = win32;
},
/* combine-source-map/lib/mappings-from-map.js */ function _(require, module, exports) {
    var SMConsumer = require(46) /* source-map */.SourceMapConsumer;
    /**
     * @name mappingsFromMap
     * @function
     * @param map {Object} the JSON.parse()'ed map
     * @return {Array} array of mappings
     */
    module.exports = function (map) {
        var consumer = new SMConsumer(map);
        var mappings = [];
        consumer.eachMapping(function (mapping) {
            // only set source if we have original position to handle edgecase (see inline-source-map tests)
            mappings.push({
                original: mapping.originalColumn != null ? {
                    column: mapping.originalColumn,
                    line: mapping.originalLine
                } : undefined,
                generated: {
                    column: mapping.generatedColumn,
                    line: mapping.generatedLine
                },
                source: mapping.originalColumn != null ? mapping.source : undefined,
                name: mapping.name
            });
        });
        return mappings;
    };
},
/* prelude.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    // TODO import license from "../../../LICENSE.txt"
    const license = `\
Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

Neither the name of Anaconda nor the names of any contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.
`;
    function comment(text) {
        return `/*!${"\n"}${text.trim().split("\n").map((line) => ` * ${line}`).join("\n")}${"\n"}*/`;
    }
    function str(obj) {
        return JSON.stringify(obj);
    }
    function prelude() {
        return `\
${comment(license)}
(function(root, factory) {
  const bokeh = factory();
  if (root.Bokeh === undefined) {
    root.Bokeh = bokeh;
  }
  const Bokeh = root.Bokeh;
  Bokeh[bokeh.version] = bokeh;
})(this, function() {
  var define;
  var parent_require = typeof require === "function" && require
  return (function(modules, entry, aliases, externals) {
    if (aliases === undefined) aliases = {};
    if (externals === undefined) externals = {};

    var cache = {};

    var normalize = function(name) {
      if (typeof name === "number")
        return name;

      if (name === "bokehjs")
        return entry;

      var prefix = "@bokehjs/"
      if (name.slice(0, prefix.length) === prefix)
        name = name.slice(prefix.length)

      var alias = aliases[name]
      if (alias != null)
        return alias;

      var trailing = name.length > 0 && name[name.lenght-1] === "/";
      var index = aliases[name + (trailing ? "" : "/") + "index"];
      if (index != null)
        return index;

      return name;
    }

    var require = function(name) {
      var mod = cache[name];
      if (!mod) {
        var id = normalize(name);

        mod = cache[id];
        if (!mod) {
          if (!modules[id]) {
            if (externals[id] === false || (externals[id] == true && parent_require)) {
              try {
                mod = {exports: externals[id] ? parent_require(id) : {}};
                cache[id] = cache[name] = mod;
                return mod.exports;
              } catch (e) {}
            }

            var err = new Error("Cannot find module '" + name + "'");
            err.code = 'MODULE_NOT_FOUND';
            throw err;
          }

          mod = {exports: {}};
          cache[id] = cache[name] = mod;
          modules[id].call(mod.exports, require, mod, mod.exports);
        } else
          cache[name] = mod;
      }

      return mod.exports;
    }

    var main = require(entry);
    main.require = require;

    if (typeof Proxy !== "undefined") {
      // allow Bokeh.loader["@bokehjs/module/name"] syntax
      main.loader = new Proxy({}, {
        get: function(_obj, module) {
          return require(module);
        }
      });
    }

    main.register_plugin = function(plugin_modules, plugin_entry, plugin_aliases, plugin_externals) {
      if (plugin_aliases === undefined) plugin_aliases = {};
      if (plugin_externals === undefined) plugin_externals = {};

      for (var name in plugin_modules) {
        modules[name] = plugin_modules[name];
      }

      for (var name in plugin_aliases) {
        aliases[name] = plugin_aliases[name];
      }

      for (var name in plugin_externals) {
        externals[name] = plugin_externals[name];
      }

      var plugin = require(plugin_entry);

      for (var name in plugin) {
        main[name] = plugin[name];
      }

      return plugin;
    }

    return main;
  })
`;
    }
    exports.prelude = prelude;
    function plugin_prelude(options) {
        return `\
${comment(license)}
(function(root, factory) {
  factory(root["Bokeh"], ${str(options === null || options === void 0 ? void 0 : options.version)});
})(this, function(Bokeh, version) {
  var define;
  return (function(modules, entry, aliases, externals) {
    const bokeh = typeof Bokeh !== "undefined" && (version != null ? Bokeh[version] : Bokeh);
    if (bokeh != null) {
      return bokeh.register_plugin(modules, entry, aliases);
    } else {
      throw new Error("Cannot find Bokeh " + version + ". You have to load it prior to loading plugins.");
    }
  })
`;
    }
    exports.plugin_prelude = plugin_prelude;
    function default_prelude(options) {
        return `\
(function(root, factory) {
  ${(options === null || options === void 0 ? void 0 : options.global) != null ? `root[${str(options.global)}] = factory()` : "Object.assign(root, factory())"};
})(this, function() {
  var parent_require = typeof require === "function" && require
  return (function(modules, entry, aliases, externals) {
    if (aliases === undefined) aliases = {};
    if (externals === undefined) externals = {};

    var cache = {};

    var normalize = function(name) {
      var alias = aliases[name];
      return alias != null ? alias : name;
    }

    var require = function(name) {
      var mod = cache[name];
      if (!mod) {
        var id = normalize(name);

        mod = cache[id];
        if (!mod) {
          if (!modules[id]) {
            if (externals[id] === false || (externals[id] == true && parent_require)) {
              try {
                mod = {exports: externals[id] ? parent_require(id) : {}};
                cache[id] = cache[name] = mod;
                return mod.exports;
              } catch (e) {}
            }

            var err = new Error("Cannot find module '" + name + "'");
            err.code = 'MODULE_NOT_FOUND';
            throw err;
          }

          mod = {exports: {}};
          cache[id] = cache[name] = mod;
          modules[id].call(mod.exports, require, mod, mod.exports);
        } else
          cache[name] = mod;
      }

      return mod.exports;
    }

    var main = require(entry);
    return main;
  })
`;
    }
    exports.default_prelude = default_prelude;
},
/* tsconfig.ext.json */ function _(require, module, exports) {
    module.exports = {
        "compilerOptions": {
            "noImplicitAny": true,
            "noImplicitThis": true,
            "noImplicitReturns": true,
            "noUnusedLocals": true,
            "noUnusedParameters": true,
            "strictNullChecks": true,
            "strictBindCallApply": false,
            "strictFunctionTypes": false,
            "strictPropertyInitialization": false,
            "alwaysStrict": true,
            "noErrorTruncation": true,
            "noEmitOnError": false,
            "allowJs": false,
            "declaration": true,
            "sourceMap": true,
            "importHelpers": false,
            "experimentalDecorators": true,
            "module": "ES2020",
            "moduleResolution": "node",
            "resolveJsonModule": true,
            "esModuleInterop": true,
            "target": "ES2017",
            "lib": ["es2017", "dom", "dom.iterable"],
            "baseUrl": ".",
            "outDir": "./dist/lib"
        },
        "include": ["./**/*.ts"]
    };
},
/* eslint/lib/api.js */ function _(require, module, exports) {
    const { CLIEngine } = require(78) /* ./cli-engine */;
    const { Linter } = require(85) /* ./linter */;
    const { RuleTester } = require(593) /* ./rule-tester */;
    const { SourceCode } = require(471) /* ./source-code */;
    module.exports = {
        Linter,
        CLIEngine,
        RuleTester,
        SourceCode
    };
    // DOTO: remove deprecated API.
    let deprecatedLinterInstance = null;
    Object.defineProperty(module.exports, "linter", {
        enumerable: false,
        get() {
            if (!deprecatedLinterInstance) {
                deprecatedLinterInstance = new Linter();
            }
            return deprecatedLinterInstance;
        }
    });
},
/* eslint/lib/cli-engine/index.js */ function _(require, module, exports) {
    const { CLIEngine } = require(79) /* ./cli-engine */;
    module.exports = {
        CLIEngine
    };
},
/* eslint/lib/cli-engine/cli-engine.js */ function _(require, module, exports) {
    /*
     * The CLI object should *not* call process.exit() directly. It should only return
     * exit codes. This allows other programs to use the CLI object and still control
     * when the program exits.
     */
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const fs = require("fs");
    const path = require("path");
    const defaultOptions = require(80) /* ../../conf/default-cli-options */;
    const pkg = require(81) /* ../../package.json */;
    const ConfigOps = require(82) /* ../shared/config-ops */;
    const naming = require(83) /* ../shared/naming */;
    const ModuleResolver = require(84) /* ../shared/relative-module-resolver */;
    const { Linter } = require(85) /* ../linter */;
    const builtInRules = require(118) /* ../rules */;
    const { CascadingConfigArrayFactory } = require(518) /* ./cascading-config-array-factory */;
    const { IgnorePattern, getUsedExtractedConfigs } = require(525) /* ./config-array */;
    const { FileEnumerator } = require(566) /* ./file-enumerator */;
    const hash = require(570) /* ./hash */;
    const LintResultCache = require(572) /* ./lint-result-cache */;
    const debug = require(120) /* debug */("eslint:cli-engine");
    const validFixTypes = new Set(["problem", "suggestion", "layout"]);
    //------------------------------------------------------------------------------
    // Typedefs
    //------------------------------------------------------------------------------
    // For VSCode IntelliSense
    /** @typedef {import("../shared/types").ConfigData} ConfigData */
    /** @typedef {import("../shared/types").LintMessage} LintMessage */
    /** @typedef {import("../shared/types").ParserOptions} ParserOptions */
    /** @typedef {import("../shared/types").Plugin} Plugin */
    /** @typedef {import("../shared/types").RuleConf} RuleConf */
    /** @typedef {import("../shared/types").Rule} Rule */
    /** @typedef {ReturnType<CascadingConfigArrayFactory["getConfigArrayForFile"]>} ConfigArray */
    /** @typedef {ReturnType<ConfigArray["extractConfig"]>} ExtractedConfig */
    /**
     * The options to configure a CLI engine with.
     * @typedef {Object} CLIEngineOptions
     * @property {boolean} allowInlineConfig Enable or disable inline configuration comments.
     * @property {ConfigData} baseConfig Base config object, extended by all configs used with this CLIEngine instance
     * @property {boolean} cache Enable result caching.
     * @property {string} cacheLocation The cache file to use instead of .eslintcache.
     * @property {string} configFile The configuration file to use.
     * @property {string} cwd The value to use for the current working directory.
     * @property {string[]} envs An array of environments to load.
     * @property {string[]} extensions An array of file extensions to check.
     * @property {boolean|Function} fix Execute in autofix mode. If a function, should return a boolean.
     * @property {string[]} fixTypes Array of rule types to apply fixes for.
     * @property {string[]} globals An array of global variables to declare.
     * @property {boolean} ignore False disables use of .eslintignore.
     * @property {string} ignorePath The ignore file to use instead of .eslintignore.
     * @property {string|string[]} ignorePattern One or more glob patterns to ignore.
     * @property {boolean} useEslintrc False disables looking for .eslintrc
     * @property {string} parser The name of the parser to use.
     * @property {ParserOptions} parserOptions An object of parserOption settings to use.
     * @property {string[]} plugins An array of plugins to load.
     * @property {Record<string,RuleConf>} rules An object of rules to use.
     * @property {string[]} rulePaths An array of directories to load custom rules from.
     * @property {boolean} reportUnusedDisableDirectives `true` adds reports for unused eslint-disable directives
     * @property {boolean} globInputPaths Set to false to skip glob resolution of input file paths to lint (default: true). If false, each input file paths is assumed to be a non-glob path to an existing file.
     * @property {string} resolvePluginsRelativeTo The folder where plugins should be resolved from, defaulting to the CWD
     */
    /**
     * A linting result.
     * @typedef {Object} LintResult
     * @property {string} filePath The path to the file that was linted.
     * @property {LintMessage[]} messages All of the messages for the result.
     * @property {number} errorCount Number of errors for the result.
     * @property {number} warningCount Number of warnings for the result.
     * @property {number} fixableErrorCount Number of fixable errors for the result.
     * @property {number} fixableWarningCount Number of fixable warnings for the result.
     * @property {string} [source] The source code of the file that was linted.
     * @property {string} [output] The source code of the file that was linted, with as many fixes applied as possible.
     */
    /**
     * Information of deprecated rules.
     * @typedef {Object} DeprecatedRuleInfo
     * @property {string} ruleId The rule ID.
     * @property {string[]} replacedBy The rule IDs that replace this deprecated rule.
     */
    /**
     * Linting results.
     * @typedef {Object} LintReport
     * @property {LintResult[]} results All of the result.
     * @property {number} errorCount Number of errors for the result.
     * @property {number} warningCount Number of warnings for the result.
     * @property {number} fixableErrorCount Number of fixable errors for the result.
     * @property {number} fixableWarningCount Number of fixable warnings for the result.
     * @property {DeprecatedRuleInfo[]} usedDeprecatedRules The list of used deprecated rules.
     */
    /**
     * Private data for CLIEngine.
     * @typedef {Object} CLIEngineInternalSlots
     * @property {Map<string, Plugin>} additionalPluginPool The map for additional plugins.
     * @property {string} cacheFilePath The path to the cache of lint results.
     * @property {CascadingConfigArrayFactory} configArrayFactory The factory of configs.
     * @property {(filePath: string) => boolean} defaultIgnores The default predicate function to check if a file ignored or not.
     * @property {FileEnumerator} fileEnumerator The file enumerator.
     * @property {ConfigArray[]} lastConfigArrays The list of config arrays that the last `executeOnFiles` or `executeOnText` used.
     * @property {LintResultCache|null} lintResultCache The cache of lint results.
     * @property {Linter} linter The linter instance which has loaded rules.
     * @property {CLIEngineOptions} options The normalized options of this instance.
     */
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /** @type {WeakMap<CLIEngine, CLIEngineInternalSlots>} */
    const internalSlotsMap = new WeakMap();
    /**
     * Determines if each fix type in an array is supported by ESLint and throws
     * an error if not.
     * @param {string[]} fixTypes An array of fix types to check.
     * @returns {void}
     * @throws {Error} If an invalid fix type is found.
     */
    function validateFixTypes(fixTypes) {
        for (const fixType of fixTypes) {
            if (!validFixTypes.has(fixType)) {
                throw new Error(`Invalid fix type "${fixType}" found.`);
            }
        }
    }
    /**
     * It will calculate the error and warning count for collection of messages per file
     * @param {LintMessage[]} messages Collection of messages
     * @returns {Object} Contains the stats
     * @private
     */
    function calculateStatsPerFile(messages) {
        return messages.reduce((stat, message) => {
            if (message.fatal || message.severity === 2) {
                stat.errorCount++;
                if (message.fix) {
                    stat.fixableErrorCount++;
                }
            }
            else {
                stat.warningCount++;
                if (message.fix) {
                    stat.fixableWarningCount++;
                }
            }
            return stat;
        }, {
            errorCount: 0,
            warningCount: 0,
            fixableErrorCount: 0,
            fixableWarningCount: 0
        });
    }
    /**
     * It will calculate the error and warning count for collection of results from all files
     * @param {LintResult[]} results Collection of messages from all the files
     * @returns {Object} Contains the stats
     * @private
     */
    function calculateStatsPerRun(results) {
        return results.reduce((stat, result) => {
            stat.errorCount += result.errorCount;
            stat.warningCount += result.warningCount;
            stat.fixableErrorCount += result.fixableErrorCount;
            stat.fixableWarningCount += result.fixableWarningCount;
            return stat;
        }, {
            errorCount: 0,
            warningCount: 0,
            fixableErrorCount: 0,
            fixableWarningCount: 0
        });
    }
    /**
     * Processes an source code using ESLint.
     * @param {Object} config The config object.
     * @param {string} config.text The source code to verify.
     * @param {string} config.cwd The path to the current working directory.
     * @param {string|undefined} config.filePath The path to the file of `text`. If this is undefined, it uses `<text>`.
     * @param {ConfigArray} config.config The config.
     * @param {boolean} config.fix If `true` then it does fix.
     * @param {boolean} config.allowInlineConfig If `true` then it uses directive comments.
     * @param {boolean} config.reportUnusedDisableDirectives If `true` then it reports unused `eslint-disable` comments.
     * @param {RegExp} config.extensionRegExp The `RegExp` object that tests if a file path has the allowed file extensions.
     * @param {Linter} config.linter The linter instance to verify.
     * @returns {LintResult} The result of linting.
     * @private
     */
    function verifyText({ text, cwd, filePath: providedFilePath, config, fix, allowInlineConfig, reportUnusedDisableDirectives, extensionRegExp, linter }) {
        const filePath = providedFilePath || "<text>";
        debug(`Lint ${filePath}`);
        /*
         * Verify.
         * `config.extractConfig(filePath)` requires an absolute path, but `linter`
         * doesn't know CWD, so it gives `linter` an absolute path always.
         */
        const filePathToVerify = filePath === "<text>" ? path.join(cwd, filePath) : filePath;
        const { fixed, messages, output } = linter.verifyAndFix(text, config, {
            allowInlineConfig,
            filename: filePathToVerify,
            fix,
            reportUnusedDisableDirectives,
            /**
             * Check if the linter should adopt a given code block or not.
             * Currently, the linter adopts code blocks if the name matches `--ext` option.
             * In the future, `overrides` in the configuration would affect the adoption (https://github.com/eslint/rfcs/pull/20).
             * @param {string} blockFilename The virtual filename of a code block.
             * @returns {boolean} `true` if the linter should adopt the code block.
             */
            filterCodeBlock(blockFilename) {
                return extensionRegExp.test(blockFilename);
            }
        });
        // Tweak and return.
        const result = {
            filePath,
            messages,
            ...calculateStatsPerFile(messages)
        };
        if (fixed) {
            result.output = output;
        }
        if (result.errorCount + result.warningCount > 0 &&
            typeof result.output === "undefined") {
            result.source = text;
        }
        return result;
    }
    /**
     * Returns result with warning by ignore settings
     * @param {string} filePath File path of checked code
     * @param {string} baseDir  Absolute path of base directory
     * @returns {LintResult} Result with single warning
     * @private
     */
    function createIgnoreResult(filePath, baseDir) {
        let message;
        const isHidden = /^\./u.test(path.basename(filePath));
        const isInNodeModules = baseDir && path.relative(baseDir, filePath).startsWith("node_modules");
        const isInBowerComponents = baseDir && path.relative(baseDir, filePath).startsWith("bower_components");
        if (isHidden) {
            message = "File ignored by default.  Use a negated ignore pattern (like \"--ignore-pattern '!<relative/path/to/filename>'\") to override.";
        }
        else if (isInNodeModules) {
            message = "File ignored by default. Use \"--ignore-pattern '!node_modules/*'\" to override.";
        }
        else if (isInBowerComponents) {
            message = "File ignored by default. Use \"--ignore-pattern '!bower_components/*'\" to override.";
        }
        else {
            message = "File ignored because of a matching ignore pattern. Use \"--no-ignore\" to override.";
        }
        return {
            filePath: path.resolve(filePath),
            messages: [
                {
                    fatal: false,
                    severity: 1,
                    message
                }
            ],
            errorCount: 0,
            warningCount: 1,
            fixableErrorCount: 0,
            fixableWarningCount: 0
        };
    }
    /**
     * Get a rule.
     * @param {string} ruleId The rule ID to get.
     * @param {ConfigArray[]} configArrays The config arrays that have plugin rules.
     * @returns {Rule|null} The rule or null.
     */
    function getRule(ruleId, configArrays) {
        for (const configArray of configArrays) {
            const rule = configArray.pluginRules.get(ruleId);
            if (rule) {
                return rule;
            }
        }
        return builtInRules.get(ruleId) || null;
    }
    /**
     * Collect used deprecated rules.
     * @param {ConfigArray[]} usedConfigArrays The config arrays which were used.
     * @returns {IterableIterator<DeprecatedRuleInfo>} Used deprecated rules.
     */
    function* iterateRuleDeprecationWarnings(usedConfigArrays) {
        const processedRuleIds = new Set();
        // Flatten used configs.
        /** @type {ExtractedConfig[]} */
        const configs = [].concat(...usedConfigArrays.map(getUsedExtractedConfigs));
        // Traverse rule configs.
        for (const config of configs) {
            for (const [ruleId, ruleConfig] of Object.entries(config.rules)) {
                // Skip if it was processed.
                if (processedRuleIds.has(ruleId)) {
                    continue;
                }
                processedRuleIds.add(ruleId);
                // Skip if it's not used.
                if (!ConfigOps.getRuleSeverity(ruleConfig)) {
                    continue;
                }
                const rule = getRule(ruleId, usedConfigArrays);
                // Skip if it's not deprecated.
                if (!(rule && rule.meta && rule.meta.deprecated)) {
                    continue;
                }
                // This rule was used and deprecated.
                yield {
                    ruleId,
                    replacedBy: rule.meta.replacedBy || []
                };
            }
        }
    }
    /**
     * Checks if the given message is an error message.
     * @param {LintMessage} message The message to check.
     * @returns {boolean} Whether or not the message is an error message.
     * @private
     */
    function isErrorMessage(message) {
        return message.severity === 2;
    }
    /**
     * return the cacheFile to be used by eslint, based on whether the provided parameter is
     * a directory or looks like a directory (ends in `path.sep`), in which case the file
     * name will be the `cacheFile/.cache_hashOfCWD`
     *
     * if cacheFile points to a file or looks like a file then in will just use that file
     * @param {string} cacheFile The name of file to be used to store the cache
     * @param {string} cwd Current working directory
     * @returns {string} the resolved path to the cache file
     */
    function getCacheFile(cacheFile, cwd) {
        /*
         * make sure the path separators are normalized for the environment/os
         * keeping the trailing path separator if present
         */
        const normalizedCacheFile = path.normalize(cacheFile);
        const resolvedCacheFile = path.resolve(cwd, normalizedCacheFile);
        const looksLikeADirectory = normalizedCacheFile.slice(-1) === path.sep;
        /**
         * return the name for the cache file in case the provided parameter is a directory
         * @returns {string} the resolved path to the cacheFile
         */
        function getCacheFileForDirectory() {
            return path.join(resolvedCacheFile, `.cache_${hash(cwd)}`);
        }
        let fileStats;
        try {
            fileStats = fs.lstatSync(resolvedCacheFile);
        }
        catch (ex) {
            fileStats = null;
        }
        /*
         * in case the file exists we need to verify if the provided path
         * is a directory or a file. If it is a directory we want to create a file
         * inside that directory
         */
        if (fileStats) {
            /*
             * is a directory or is a file, but the original file the user provided
             * looks like a directory but `path.resolve` removed the `last path.sep`
             * so we need to still treat this like a directory
             */
            if (fileStats.isDirectory() || looksLikeADirectory) {
                return getCacheFileForDirectory();
            }
            // is file so just use that file
            return resolvedCacheFile;
        }
        /*
         * here we known the file or directory doesn't exist,
         * so we will try to infer if its a directory if it looks like a directory
         * for the current operating system.
         */
        // if the last character passed is a path separator we assume is a directory
        if (looksLikeADirectory) {
            return getCacheFileForDirectory();
        }
        return resolvedCacheFile;
    }
    /**
     * Convert a string array to a boolean map.
     * @param {string[]|null} keys The keys to assign true.
     * @param {boolean} defaultValue The default value for each property.
     * @param {string} displayName The property name which is used in error message.
     * @returns {Record<string,boolean>} The boolean map.
     */
    function toBooleanMap(keys, defaultValue, displayName) {
        if (keys && !Array.isArray(keys)) {
            throw new Error(`${displayName} must be an array.`);
        }
        if (keys && keys.length > 0) {
            return keys.reduce((map, def) => {
                const [key, value] = def.split(":");
                if (key !== "__proto__") {
                    map[key] = value === void 0
                        ? defaultValue
                        : value === "true";
                }
                return map;
            }, {});
        }
        return void 0;
    }
    /**
     * Create a config data from CLI options.
     * @param {CLIEngineOptions} options The options
     * @returns {ConfigData|null} The created config data.
     */
    function createConfigDataFromOptions(options) {
        const { ignorePattern, parser, parserOptions, plugins, rules } = options;
        const env = toBooleanMap(options.envs, true, "envs");
        const globals = toBooleanMap(options.globals, false, "globals");
        if (env === void 0 &&
            globals === void 0 &&
            (ignorePattern === void 0 || ignorePattern.length === 0) &&
            parser === void 0 &&
            parserOptions === void 0 &&
            plugins === void 0 &&
            rules === void 0) {
            return null;
        }
        return {
            env,
            globals,
            ignorePatterns: ignorePattern,
            parser,
            parserOptions,
            plugins,
            rules
        };
    }
    /**
     * Checks whether a directory exists at the given location
     * @param {string} resolvedPath A path from the CWD
     * @returns {boolean} `true` if a directory exists
     */
    function directoryExists(resolvedPath) {
        try {
            return fs.statSync(resolvedPath).isDirectory();
        }
        catch (error) {
            if (error && error.code === "ENOENT") {
                return false;
            }
            throw error;
        }
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    class CLIEngine {
        /**
         * Creates a new instance of the core CLI engine.
         * @param {CLIEngineOptions} providedOptions The options for this instance.
         */
        constructor(providedOptions) {
            const options = Object.assign(Object.create(null), defaultOptions, { cwd: process.cwd() }, providedOptions);
            if (options.fix === void 0) {
                options.fix = false;
            }
            const additionalPluginPool = new Map();
            const cacheFilePath = getCacheFile(options.cacheLocation || options.cacheFile, options.cwd);
            const configArrayFactory = new CascadingConfigArrayFactory({
                additionalPluginPool,
                baseConfig: options.baseConfig || null,
                cliConfig: createConfigDataFromOptions(options),
                cwd: options.cwd,
                ignorePath: options.ignorePath,
                resolvePluginsRelativeTo: options.resolvePluginsRelativeTo,
                rulePaths: options.rulePaths,
                specificConfigPath: options.configFile,
                useEslintrc: options.useEslintrc
            });
            const fileEnumerator = new FileEnumerator({
                configArrayFactory,
                cwd: options.cwd,
                extensions: options.extensions,
                globInputPaths: options.globInputPaths,
                errorOnUnmatchedPattern: options.errorOnUnmatchedPattern,
                ignore: options.ignore
            });
            const lintResultCache = options.cache ? new LintResultCache(cacheFilePath) : null;
            const linter = new Linter({ cwd: options.cwd });
            /** @type {ConfigArray[]} */
            const lastConfigArrays = [configArrayFactory.getConfigArrayForFile()];
            // Store private data.
            internalSlotsMap.set(this, {
                additionalPluginPool,
                cacheFilePath,
                configArrayFactory,
                defaultIgnores: IgnorePattern.createDefaultIgnore(options.cwd),
                fileEnumerator,
                lastConfigArrays,
                lintResultCache,
                linter,
                options
            });
            // setup special filter for fixes
            if (options.fix && options.fixTypes && options.fixTypes.length > 0) {
                debug(`Using fix types ${options.fixTypes}`);
                // throw an error if any invalid fix types are found
                validateFixTypes(options.fixTypes);
                // convert to Set for faster lookup
                const fixTypes = new Set(options.fixTypes);
                // save original value of options.fix in case it's a function
                const originalFix = (typeof options.fix === "function")
                    ? options.fix : () => true;
                options.fix = message => {
                    const rule = message.ruleId && getRule(message.ruleId, lastConfigArrays);
                    const matches = rule && rule.meta && fixTypes.has(rule.meta.type);
                    return matches && originalFix(message);
                };
            }
        }
        getRules() {
            const { lastConfigArrays } = internalSlotsMap.get(this);
            return new Map(function* () {
                yield* builtInRules;
                for (const configArray of lastConfigArrays) {
                    yield* configArray.pluginRules;
                }
            }());
        }
        /**
         * Returns results that only contains errors.
         * @param {LintResult[]} results The results to filter.
         * @returns {LintResult[]} The filtered results.
         */
        static getErrorResults(results) {
            const filtered = [];
            results.forEach(result => {
                const filteredMessages = result.messages.filter(isErrorMessage);
                if (filteredMessages.length > 0) {
                    filtered.push({
                        ...result,
                        messages: filteredMessages,
                        errorCount: filteredMessages.length,
                        warningCount: 0,
                        fixableErrorCount: result.fixableErrorCount,
                        fixableWarningCount: 0
                    });
                }
            });
            return filtered;
        }
        /**
         * Outputs fixes from the given results to files.
         * @param {LintReport} report The report object created by CLIEngine.
         * @returns {void}
         */
        static outputFixes(report) {
            report.results.filter(result => Object.prototype.hasOwnProperty.call(result, "output")).forEach(result => {
                fs.writeFileSync(result.filePath, result.output);
            });
        }
        /**
         * Add a plugin by passing its configuration
         * @param {string} name Name of the plugin.
         * @param {Plugin} pluginObject Plugin configuration object.
         * @returns {void}
         */
        addPlugin(name, pluginObject) {
            const { additionalPluginPool, configArrayFactory, lastConfigArrays } = internalSlotsMap.get(this);
            additionalPluginPool.set(name, pluginObject);
            configArrayFactory.clearCache();
            lastConfigArrays.length = 1;
            lastConfigArrays[0] = configArrayFactory.getConfigArrayForFile();
        }
        /**
         * Resolves the patterns passed into executeOnFiles() into glob-based patterns
         * for easier handling.
         * @param {string[]} patterns The file patterns passed on the command line.
         * @returns {string[]} The equivalent glob patterns.
         */
        resolveFileGlobPatterns(patterns) {
            const { options } = internalSlotsMap.get(this);
            if (options.globInputPaths === false) {
                return patterns.filter(Boolean);
            }
            const extensions = options.extensions.map(ext => ext.replace(/^\./u, ""));
            const dirSuffix = `/**/*.{${extensions.join(",")}}`;
            return patterns.filter(Boolean).map(pathname => {
                const resolvedPath = path.resolve(options.cwd, pathname);
                const newPath = directoryExists(resolvedPath)
                    ? pathname.replace(/[/\\]$/u, "") + dirSuffix
                    : pathname;
                return path.normalize(newPath).replace(/\\/gu, "/");
            });
        }
        /**
         * Executes the current configuration on an array of file and directory names.
         * @param {string[]} patterns An array of file and directory names.
         * @returns {LintReport} The results for all files that were linted.
         */
        executeOnFiles(patterns) {
            const { cacheFilePath, fileEnumerator, lastConfigArrays, lintResultCache, linter, options: { allowInlineConfig, cache, cwd, fix, reportUnusedDisableDirectives } } = internalSlotsMap.get(this);
            const results = [];
            const startTime = Date.now();
            // Clear the last used config arrays.
            lastConfigArrays.length = 0;
            // Delete cache file; should this do here?
            if (!cache) {
                try {
                    fs.unlinkSync(cacheFilePath);
                }
                catch (error) {
                    const errorCode = error && error.code;
                    // Ignore errors when no such file exists or file system is read only (and cache file does not exist)
                    if (errorCode !== "ENOENT" && !(errorCode === "EROFS" && !fs.existsSync(cacheFilePath))) {
                        throw error;
                    }
                }
            }
            // Iterate source code files.
            for (const { config, filePath, ignored } of fileEnumerator.iterateFiles(patterns)) {
                if (ignored) {
                    results.push(createIgnoreResult(filePath, cwd));
                    continue;
                }
                /*
                 * Store used configs for:
                 * - this method uses to collect used deprecated rules.
                 * - `getRules()` method uses to collect all loaded rules.
                 * - `--fix-type` option uses to get the loaded rule's meta data.
                 */
                if (!lastConfigArrays.includes(config)) {
                    lastConfigArrays.push(config);
                }
                // Skip if there is cached result.
                if (lintResultCache) {
                    const cachedResult = lintResultCache.getCachedLintResults(filePath, config);
                    if (cachedResult) {
                        const hadMessages = cachedResult.messages &&
                            cachedResult.messages.length > 0;
                        if (hadMessages && fix) {
                            debug(`Reprocessing cached file to allow autofix: ${filePath}`);
                        }
                        else {
                            debug(`Skipping file since it hasn't changed: ${filePath}`);
                            results.push(cachedResult);
                            continue;
                        }
                    }
                }
                // Do lint.
                const result = verifyText({
                    text: fs.readFileSync(filePath, "utf8"),
                    filePath,
                    config,
                    cwd,
                    fix,
                    allowInlineConfig,
                    reportUnusedDisableDirectives,
                    extensionRegExp: fileEnumerator.extensionRegExp,
                    linter
                });
                results.push(result);
                /*
                 * Store the lint result in the LintResultCache.
                 * NOTE: The LintResultCache will remove the file source and any
                 * other properties that are difficult to serialize, and will
                 * hydrate those properties back in on future lint runs.
                 */
                if (lintResultCache) {
                    lintResultCache.setCachedLintResults(filePath, config, result);
                }
            }
            // Persist the cache to disk.
            if (lintResultCache) {
                lintResultCache.reconcile();
            }
            // Collect used deprecated rules.
            const usedDeprecatedRules = Array.from(iterateRuleDeprecationWarnings(lastConfigArrays));
            debug(`Linting complete in: ${Date.now() - startTime}ms`);
            return {
                results,
                ...calculateStatsPerRun(results),
                usedDeprecatedRules
            };
        }
        /**
         * Executes the current configuration on text.
         * @param {string} text A string of JavaScript code to lint.
         * @param {string} [filename] An optional string representing the texts filename.
         * @param {boolean} [warnIgnored] Always warn when a file is ignored
         * @returns {LintReport} The results for the linting.
         */
        executeOnText(text, filename, warnIgnored) {
            const { configArrayFactory, fileEnumerator, lastConfigArrays, linter, options: { allowInlineConfig, cwd, fix, reportUnusedDisableDirectives } } = internalSlotsMap.get(this);
            const results = [];
            const startTime = Date.now();
            const resolvedFilename = filename && path.resolve(cwd, filename);
            // Clear the last used config arrays.
            lastConfigArrays.length = 0;
            if (resolvedFilename && this.isPathIgnored(resolvedFilename)) {
                if (warnIgnored) {
                    results.push(createIgnoreResult(resolvedFilename, cwd));
                }
            }
            else {
                const config = configArrayFactory.getConfigArrayForFile(resolvedFilename || "__placeholder__.js");
                /*
                 * Store used configs for:
                 * - this method uses to collect used deprecated rules.
                 * - `getRules()` method uses to collect all loaded rules.
                 * - `--fix-type` option uses to get the loaded rule's meta data.
                 */
                lastConfigArrays.push(config);
                // Do lint.
                results.push(verifyText({
                    text,
                    filePath: resolvedFilename,
                    config,
                    cwd,
                    fix,
                    allowInlineConfig,
                    reportUnusedDisableDirectives,
                    extensionRegExp: fileEnumerator.extensionRegExp,
                    linter
                }));
            }
            // Collect used deprecated rules.
            const usedDeprecatedRules = Array.from(iterateRuleDeprecationWarnings(lastConfigArrays));
            debug(`Linting complete in: ${Date.now() - startTime}ms`);
            return {
                results,
                ...calculateStatsPerRun(results),
                usedDeprecatedRules
            };
        }
        /**
         * Returns a configuration object for the given file based on the CLI options.
         * This is the same logic used by the ESLint CLI executable to determine
         * configuration for each file it processes.
         * @param {string} filePath The path of the file to retrieve a config object for.
         * @returns {ConfigData} A configuration object for the file.
         */
        getConfigForFile(filePath) {
            const { configArrayFactory, options } = internalSlotsMap.get(this);
            const absolutePath = path.resolve(options.cwd, filePath);
            if (directoryExists(absolutePath)) {
                throw Object.assign(new Error("'filePath' should not be a directory path."), { messageTemplate: "print-config-with-directory-path" });
            }
            return configArrayFactory
                .getConfigArrayForFile(absolutePath)
                .extractConfig(absolutePath)
                .toCompatibleObjectAsConfigFileContent();
        }
        /**
         * Checks if a given path is ignored by ESLint.
         * @param {string} filePath The path of the file to check.
         * @returns {boolean} Whether or not the given path is ignored.
         */
        isPathIgnored(filePath) {
            const { configArrayFactory, defaultIgnores, options: { cwd, ignore } } = internalSlotsMap.get(this);
            const absolutePath = path.resolve(cwd, filePath);
            if (ignore) {
                const config = configArrayFactory
                    .getConfigArrayForFile(absolutePath)
                    .extractConfig(absolutePath);
                const ignores = config.ignores || defaultIgnores;
                return ignores(absolutePath);
            }
            return defaultIgnores(absolutePath);
        }
        /**
         * Returns the formatter representing the given format or null if no formatter
         * with the given name can be found.
         * @param {string} [format] The name of the format to load or the path to a
         *      custom formatter.
         * @returns {Function} The formatter function or null if not found.
         */
        getFormatter(format) {
            // default is stylish
            const resolvedFormatName = format || "stylish";
            // only strings are valid formatters
            if (typeof resolvedFormatName === "string") {
                // replace \ with / for Windows compatibility
                const normalizedFormatName = resolvedFormatName.replace(/\\/gu, "/");
                const slots = internalSlotsMap.get(this);
                const cwd = slots ? slots.options.cwd : process.cwd();
                const namespace = naming.getNamespaceFromTerm(normalizedFormatName);
                let formatterPath;
                // if there's a slash, then it's a file (TODO: this check seems dubious for scoped npm packages)
                if (!namespace && normalizedFormatName.indexOf("/") > -1) {
                    formatterPath = path.resolve(cwd, normalizedFormatName);
                }
                else {
                    try {
                        const npmFormat = naming.normalizePackageName(normalizedFormatName, "eslint-formatter");
                        formatterPath = ModuleResolver.resolve(npmFormat, path.join(cwd, "__placeholder__.js"));
                    }
                    catch (e) {
                        formatterPath = path.resolve(__dirname, "formatters", normalizedFormatName);
                    }
                }
                try {
                    return require(formatterPath);
                }
                catch (ex) {
                    ex.message = `There was a problem loading formatter: ${formatterPath}\nError: ${ex.message}`;
                    throw ex;
                }
            }
            else {
                return null;
            }
        }
    }
    CLIEngine.version = pkg.version;
    CLIEngine.getFormatter = CLIEngine.prototype.getFormatter;
    module.exports = {
        CLIEngine,
        /**
         * Get the internal slots of a given CLIEngine instance for tests.
         * @param {CLIEngine} instance The CLIEngine instance to get.
         * @returns {CLIEngineInternalSlots} The internal slots.
         */
        getCLIEngineInternalSlots(instance) {
            return internalSlotsMap.get(instance);
        }
    };
},
/* eslint/conf/default-cli-options.js */ function _(require, module, exports) {
    module.exports = {
        configFile: null,
        baseConfig: false,
        rulePaths: [],
        useEslintrc: true,
        envs: [],
        globals: [],
        extensions: [".js"],
        ignore: true,
        ignorePath: void 0,
        cache: false,
        /*
         * in order to honor the cacheFile option if specified
         * this option should not have a default value otherwise
         * it will always be used
         */
        cacheLocation: "",
        cacheFile: ".eslintcache",
        fix: false,
        allowInlineConfig: true,
        reportUnusedDisableDirectives: void 0,
        globInputPaths: true
    };
},
/* eslint/package.json */ function _(require, module, exports) {
    module.exports = {
        "_args": [
            [
                "eslint@6.8.0",
                "/Users/bryan/work/bokeh/bokehjs"
            ]
        ],
        "_development": true,
        "_from": "eslint@6.8.0",
        "_id": "eslint@6.8.0",
        "_inBundle": false,
        "_integrity": "sha512-K+Iayyo2LtyYhDSYwz5D5QdWw0hCacNzyq1Y821Xna2xSJj7cijoLLYmLxTQgcgZ9mC61nryMy9S7GRbYpI5Ig==",
        "_location": "/eslint",
        "_phantomChildren": {
            "escape-string-regexp": "1.0.5"
        },
        "_requested": {
            "type": "version",
            "registry": true,
            "raw": "eslint@6.8.0",
            "name": "eslint",
            "escapedName": "eslint",
            "rawSpec": "6.8.0",
            "saveSpec": null,
            "fetchSpec": "6.8.0"
        },
        "_requiredBy": [
            "#DEV:/"
        ],
        "_resolved": "https://registry.npmjs.org/eslint/-/eslint-6.8.0.tgz",
        "_spec": "6.8.0",
        "_where": "/Users/bryan/work/bokeh/bokehjs",
        "author": {
            "name": "Nicholas C. Zakas",
            "email": "nicholas+npm@nczconsulting.com"
        },
        "bin": {
            "eslint": "bin/eslint.js"
        },
        "bugs": {
            "url": "https://github.com/eslint/eslint/issues/"
        },
        "dependencies": {
            "@babel/code-frame": "^7.0.0",
            "ajv": "^6.10.0",
            "chalk": "^2.1.0",
            "cross-spawn": "^6.0.5",
            "debug": "^4.0.1",
            "doctrine": "^3.0.0",
            "eslint-scope": "^5.0.0",
            "eslint-utils": "^1.4.3",
            "eslint-visitor-keys": "^1.1.0",
            "espree": "^6.1.2",
            "esquery": "^1.0.1",
            "esutils": "^2.0.2",
            "file-entry-cache": "^5.0.1",
            "functional-red-black-tree": "^1.0.1",
            "glob-parent": "^5.0.0",
            "globals": "^12.1.0",
            "ignore": "^4.0.6",
            "import-fresh": "^3.0.0",
            "imurmurhash": "^0.1.4",
            "inquirer": "^7.0.0",
            "is-glob": "^4.0.0",
            "js-yaml": "^3.13.1",
            "json-stable-stringify-without-jsonify": "^1.0.1",
            "levn": "^0.3.0",
            "lodash": "^4.17.14",
            "minimatch": "^3.0.4",
            "mkdirp": "^0.5.1",
            "natural-compare": "^1.4.0",
            "optionator": "^0.8.3",
            "progress": "^2.0.0",
            "regexpp": "^2.0.1",
            "semver": "^6.1.2",
            "strip-ansi": "^5.2.0",
            "strip-json-comments": "^3.0.1",
            "table": "^5.2.3",
            "text-table": "^0.2.0",
            "v8-compile-cache": "^2.0.3"
        },
        "description": "An AST-based pattern checker for JavaScript.",
        "devDependencies": {
            "@babel/core": "^7.4.3",
            "@babel/preset-env": "^7.4.3",
            "acorn": "^7.1.0",
            "babel-loader": "^8.0.5",
            "chai": "^4.0.1",
            "cheerio": "^0.22.0",
            "common-tags": "^1.8.0",
            "core-js": "^3.1.3",
            "dateformat": "^3.0.3",
            "ejs": "^2.6.1",
            "eslint": "file:.",
            "eslint-config-eslint": "file:packages/eslint-config-eslint",
            "eslint-plugin-eslint-plugin": "^2.0.1",
            "eslint-plugin-internal-rules": "file:tools/internal-rules",
            "eslint-plugin-jsdoc": "^15.9.5",
            "eslint-plugin-node": "^9.0.0",
            "eslint-release": "^1.2.0",
            "eslump": "^2.0.0",
            "esprima": "^4.0.1",
            "glob": "^7.1.3",
            "jsdoc": "^3.5.5",
            "karma": "^4.0.1",
            "karma-chrome-launcher": "^2.2.0",
            "karma-mocha": "^1.3.0",
            "karma-mocha-reporter": "^2.2.3",
            "karma-webpack": "^4.0.0-rc.6",
            "leche": "^2.2.3",
            "lint-staged": "^8.1.5",
            "load-perf": "^0.2.0",
            "markdownlint": "^0.15.0",
            "markdownlint-cli": "^0.17.0",
            "metro-memory-fs": "^0.54.1",
            "mocha": "^6.1.2",
            "mocha-junit-reporter": "^1.23.0",
            "npm-license": "^0.3.3",
            "nyc": "^14.1.1",
            "proxyquire": "^2.0.1",
            "puppeteer": "^1.18.0",
            "recast": "^0.18.1",
            "regenerator-runtime": "^0.13.2",
            "shelljs": "^0.8.2",
            "sinon": "^7.3.2",
            "temp": "^0.9.0",
            "webpack": "^4.35.0",
            "webpack-cli": "^3.3.5",
            "yorkie": "^2.0.0"
        },
        "engines": {
            "node": "^8.10.0 || ^10.13.0 || >=11.10.1"
        },
        "files": [
            "LICENSE",
            "README.md",
            "bin",
            "conf",
            "lib",
            "messages"
        ],
        "funding": "https://opencollective.com/eslint",
        "gitHooks": {
            "pre-commit": "lint-staged"
        },
        "homepage": "https://eslint.org",
        "keywords": [
            "ast",
            "lint",
            "javascript",
            "ecmascript",
            "espree"
        ],
        "license": "MIT",
        "lint-staged": {
            "*.js": [
                "eslint --fix",
                "git add"
            ],
            "*.md": "markdownlint"
        },
        "main": "./lib/api.js",
        "name": "eslint",
        "repository": {
            "type": "git",
            "url": "git+https://github.com/eslint/eslint.git"
        },
        "scripts": {
            "docs": "node Makefile.js docs",
            "fix": "node Makefile.js lint -- fix",
            "fuzz": "node Makefile.js fuzz",
            "generate-alpharelease": "node Makefile.js generatePrerelease -- alpha",
            "generate-betarelease": "node Makefile.js generatePrerelease -- beta",
            "generate-rcrelease": "node Makefile.js generatePrerelease -- rc",
            "generate-release": "node Makefile.js generateRelease",
            "gensite": "node Makefile.js gensite",
            "lint": "node Makefile.js lint",
            "perf": "node Makefile.js perf",
            "publish-release": "node Makefile.js publishRelease",
            "test": "node Makefile.js test",
            "test:cli": "mocha",
            "webpack": "node Makefile.js webpack"
        },
        "version": "6.8.0"
    };
},
/* eslint/lib/shared/config-ops.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Private
    //------------------------------------------------------------------------------
    const RULE_SEVERITY_STRINGS = ["off", "warn", "error"], RULE_SEVERITY = RULE_SEVERITY_STRINGS.reduce((map, value, index) => {
        map[value] = index;
        return map;
    }, {}), VALID_SEVERITIES = [0, 1, 2, "off", "warn", "error"];
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    module.exports = {
        /**
         * Normalizes the severity value of a rule's configuration to a number
         * @param {(number|string|[number, ...*]|[string, ...*])} ruleConfig A rule's configuration value, generally
         * received from the user. A valid config value is either 0, 1, 2, the string "off" (treated the same as 0),
         * the string "warn" (treated the same as 1), the string "error" (treated the same as 2), or an array
         * whose first element is one of the above values. Strings are matched case-insensitively.
         * @returns {(0|1|2)} The numeric severity value if the config value was valid, otherwise 0.
         */
        getRuleSeverity(ruleConfig) {
            const severityValue = Array.isArray(ruleConfig) ? ruleConfig[0] : ruleConfig;
            if (severityValue === 0 || severityValue === 1 || severityValue === 2) {
                return severityValue;
            }
            if (typeof severityValue === "string") {
                return RULE_SEVERITY[severityValue.toLowerCase()] || 0;
            }
            return 0;
        },
        /**
         * Converts old-style severity settings (0, 1, 2) into new-style
         * severity settings (off, warn, error) for all rules. Assumption is that severity
         * values have already been validated as correct.
         * @param {Object} config The config object to normalize.
         * @returns {void}
         */
        normalizeToStrings(config) {
            if (config.rules) {
                Object.keys(config.rules).forEach(ruleId => {
                    const ruleConfig = config.rules[ruleId];
                    if (typeof ruleConfig === "number") {
                        config.rules[ruleId] = RULE_SEVERITY_STRINGS[ruleConfig] || RULE_SEVERITY_STRINGS[0];
                    }
                    else if (Array.isArray(ruleConfig) && typeof ruleConfig[0] === "number") {
                        ruleConfig[0] = RULE_SEVERITY_STRINGS[ruleConfig[0]] || RULE_SEVERITY_STRINGS[0];
                    }
                });
            }
        },
        /**
         * Determines if the severity for the given rule configuration represents an error.
         * @param {int|string|Array} ruleConfig The configuration for an individual rule.
         * @returns {boolean} True if the rule represents an error, false if not.
         */
        isErrorSeverity(ruleConfig) {
            return module.exports.getRuleSeverity(ruleConfig) === 2;
        },
        /**
         * Checks whether a given config has valid severity or not.
         * @param {number|string|Array} ruleConfig The configuration for an individual rule.
         * @returns {boolean} `true` if the configuration has valid severity.
         */
        isValidSeverity(ruleConfig) {
            let severity = Array.isArray(ruleConfig) ? ruleConfig[0] : ruleConfig;
            if (typeof severity === "string") {
                severity = severity.toLowerCase();
            }
            return VALID_SEVERITIES.indexOf(severity) !== -1;
        },
        /**
         * Checks whether every rule of a given config has valid severity or not.
         * @param {Object} config The configuration for rules.
         * @returns {boolean} `true` if the configuration has valid severity.
         */
        isEverySeverityValid(config) {
            return Object.keys(config).every(ruleId => this.isValidSeverity(config[ruleId]));
        },
        /**
         * Normalizes a value for a global in a config
         * @param {(boolean|string|null)} configuredValue The value given for a global in configuration or in
         * a global directive comment
         * @returns {("readable"|"writeable"|"off")} The value normalized as a string
         * @throws Error if global value is invalid
         */
        normalizeConfigGlobal(configuredValue) {
            switch (configuredValue) {
                case "off":
                    return "off";
                case true:
                case "true":
                case "writeable":
                case "writable":
                    return "writable";
                case null:
                case false:
                case "false":
                case "readable":
                case "readonly":
                    return "readonly";
                default:
                    throw new Error(`'${configuredValue}' is not a valid configuration for a global (use 'readonly', 'writable', or 'off')`);
            }
        }
    };
},
/* eslint/lib/shared/naming.js */ function _(require, module, exports) {
    const NAMESPACE_REGEX = /^@.*\//iu;
    /**
     * Brings package name to correct format based on prefix
     * @param {string} name The name of the package.
     * @param {string} prefix Can be either "eslint-plugin", "eslint-config" or "eslint-formatter"
     * @returns {string} Normalized name of the package
     * @private
     */
    function normalizePackageName(name, prefix) {
        let normalizedName = name;
        /**
         * On Windows, name can come in with Windows slashes instead of Unix slashes.
         * Normalize to Unix first to avoid errors later on.
         * https://github.com/eslint/eslint/issues/5644
         */
        if (normalizedName.includes("\\")) {
            normalizedName = normalizedName.replace(/\\/gu, "/");
        }
        if (normalizedName.charAt(0) === "@") {
            /**
             * it's a scoped package
             * package name is the prefix, or just a username
             */
            const scopedPackageShortcutRegex = new RegExp(`^(@[^/]+)(?:/(?:${prefix})?)?$`, "u"), scopedPackageNameRegex = new RegExp(`^${prefix}(-|$)`, "u");
            if (scopedPackageShortcutRegex.test(normalizedName)) {
                normalizedName = normalizedName.replace(scopedPackageShortcutRegex, `$1/${prefix}`);
            }
            else if (!scopedPackageNameRegex.test(normalizedName.split("/")[1])) {
                /**
                 * for scoped packages, insert the prefix after the first / unless
                 * the path is already @scope/eslint or @scope/eslint-xxx-yyy
                 */
                normalizedName = normalizedName.replace(/^@([^/]+)\/(.*)$/u, `@$1/${prefix}-$2`);
            }
        }
        else if (!normalizedName.startsWith(`${prefix}-`)) {
            normalizedName = `${prefix}-${normalizedName}`;
        }
        return normalizedName;
    }
    /**
     * Removes the prefix from a fullname.
     * @param {string} fullname The term which may have the prefix.
     * @param {string} prefix The prefix to remove.
     * @returns {string} The term without prefix.
     */
    function getShorthandName(fullname, prefix) {
        if (fullname[0] === "@") {
            let matchResult = new RegExp(`^(@[^/]+)/${prefix}$`, "u").exec(fullname);
            if (matchResult) {
                return matchResult[1];
            }
            matchResult = new RegExp(`^(@[^/]+)/${prefix}-(.+)$`, "u").exec(fullname);
            if (matchResult) {
                return `${matchResult[1]}/${matchResult[2]}`;
            }
        }
        else if (fullname.startsWith(`${prefix}-`)) {
            return fullname.slice(prefix.length + 1);
        }
        return fullname;
    }
    /**
     * Gets the scope (namespace) of a term.
     * @param {string} term The term which may have the namespace.
     * @returns {string} The namepace of the term if it has one.
     */
    function getNamespaceFromTerm(term) {
        const match = term.match(NAMESPACE_REGEX);
        return match ? match[0] : "";
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    module.exports = {
        normalizePackageName,
        getShorthandName,
        getNamespaceFromTerm
    };
},
/* eslint/lib/shared/relative-module-resolver.js */ function _(require, module, exports) {
    const Module = require("module");
    const path = require("path");
    // Polyfill Node's `Module.createRequire` if not present. We only support the case where the argument is a filepath, not a URL.
    const createRequire = (
    // Added in v12.2.0
    Module.createRequire ||
        // Added in v10.12.0, but deprecated in v12.2.0.
        Module.createRequireFromPath ||
        // Polyfill - This is not executed on the tests on node@>=10.
        /* istanbul ignore next */
        (filename => {
            const mod = new Module(filename, null);
            mod.filename = filename;
            mod.paths = Module._nodeModulePaths(path.dirname(filename)); // eslint-disable-line no-underscore-dangle
            mod._compile("module.exports = require;", filename); // eslint-disable-line no-underscore-dangle
            return mod.exports;
        }));
    module.exports = {
        /**
         * Resolves a Node module relative to another module
         * @param {string} moduleName The name of a Node module, or a path to a Node module.
         *
         * @param {string} relativeToPath An absolute path indicating the module that `moduleName` should be resolved relative to. This must be
         * a file rather than a directory, but the file need not actually exist.
         * @returns {string} The absolute path that would result from calling `require.resolve(moduleName)` in a file located at `relativeToPath`
         */
        resolve(moduleName, relativeToPath) {
            try {
                return createRequire(relativeToPath).resolve(moduleName);
            }
            catch (error) {
                if (typeof error === "object" &&
                    error !== null &&
                    error.code === "MODULE_NOT_FOUND" &&
                    !error.requireStack &&
                    error.message.includes(moduleName)) {
                    error.message += `\nRequire stack:\n- ${relativeToPath}`;
                }
                throw error;
            }
        }
    };
},
/* eslint/lib/linter/index.js */ function _(require, module, exports) {
    const { Linter } = require(86) /* ./linter */;
    const interpolate = require(512) /* ./interpolate */;
    const SourceCodeFixer = require(515) /* ./source-code-fixer */;
    module.exports = {
        Linter,
        // For testers.
        SourceCodeFixer,
        interpolate
    };
},
/* eslint/lib/linter/linter.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const path = require("path"), eslintScope = require(87) /* eslint-scope */, evk = require(100) /* eslint-visitor-keys */, espree = require(102) /* espree */, lodash = require(111) /* lodash */, BuiltInEnvironments = require(112) /* ../../conf/environments */, pkg = require(81) /* ../../package.json */, astUtils = require(115) /* ../shared/ast-utils */, ConfigOps = require(82) /* ../shared/config-ops */, validator = require(116) /* ../shared/config-validator */, Traverser = require(337) /* ../shared/traverser */, { SourceCode } = require(471) /* ../source-code */, CodePathAnalyzer = require(486) /* ./code-path-analysis/code-path-analyzer */, applyDisableDirectives = require(493) /* ./apply-disable-directives */, ConfigCommentParser = require(494) /* ./config-comment-parser */, NodeEventGenerator = require(507) /* ./node-event-generator */, createReportTranslator = require(510) /* ./report-translator */, Rules = require(513) /* ./rules */, createEmitter = require(514) /* ./safe-emitter */, SourceCodeFixer = require(515) /* ./source-code-fixer */, timing = require(516) /* ./timing */, ruleReplacements = require(517) /* ../../conf/replacements.json */;
    const debug = require(120) /* debug */("eslint:linter");
    const MAX_AUTOFIX_PASSES = 10;
    const DEFAULT_PARSER_NAME = "espree";
    const commentParser = new ConfigCommentParser();
    const DEFAULT_ERROR_LOC = { start: { line: 1, column: 0 }, end: { line: 1, column: 1 } };
    //------------------------------------------------------------------------------
    // Typedefs
    //------------------------------------------------------------------------------
    /** @typedef {InstanceType<import("../cli-engine/config-array")["ConfigArray"]>} ConfigArray */
    /** @typedef {InstanceType<import("../cli-engine/config-array")["ExtractedConfig"]>} ExtractedConfig */
    /** @typedef {import("../shared/types").ConfigData} ConfigData */
    /** @typedef {import("../shared/types").Environment} Environment */
    /** @typedef {import("../shared/types").GlobalConf} GlobalConf */
    /** @typedef {import("../shared/types").LintMessage} LintMessage */
    /** @typedef {import("../shared/types").ParserOptions} ParserOptions */
    /** @typedef {import("../shared/types").Processor} Processor */
    /** @typedef {import("../shared/types").Rule} Rule */
    /**
     * @template T
     * @typedef {{ [P in keyof T]-?: T[P] }} Required
     */
    /**
     * @typedef {Object} DisableDirective
     * @property {("disable"|"enable"|"disable-line"|"disable-next-line")} type
     * @property {number} line
     * @property {number} column
     * @property {(string|null)} ruleId
     */
    /**
     * The private data for `Linter` instance.
     * @typedef {Object} LinterInternalSlots
     * @property {ConfigArray|null} lastConfigArray The `ConfigArray` instance that the last `verify()` call used.
     * @property {SourceCode|null} lastSourceCode The `SourceCode` instance that the last `verify()` call used.
     * @property {Map<string, Parser>} parserMap The loaded parsers.
     * @property {Rules} ruleMap The loaded rules.
     */
    /**
     * @typedef {Object} VerifyOptions
     * @property {boolean} [allowInlineConfig] Allow/disallow inline comments' ability
     *      to change config once it is set. Defaults to true if not supplied.
     *      Useful if you want to validate JS without comments overriding rules.
     * @property {boolean} [disableFixes] if `true` then the linter doesn't make `fix`
     *      properties into the lint result.
     * @property {string} [filename] the filename of the source code.
     * @property {boolean | "off" | "warn" | "error"} [reportUnusedDisableDirectives] Adds reported errors for
     *      unused `eslint-disable` directives.
     */
    /**
     * @typedef {Object} ProcessorOptions
     * @property {(filename:string, text:string) => boolean} [filterCodeBlock] the
     *      predicate function that selects adopt code blocks.
     * @property {Processor["postprocess"]} [postprocess] postprocessor for report
     *      messages. If provided, this should accept an array of the message lists
     *      for each code block returned from the preprocessor, apply a mapping to
     *      the messages as appropriate, and return a one-dimensional array of
     *      messages.
     * @property {Processor["preprocess"]} [preprocess] preprocessor for source text.
     *      If provided, this should accept a string of source text, and return an
     *      array of code blocks to lint.
     */
    /**
     * @typedef {Object} FixOptions
     * @property {boolean | ((message: LintMessage) => boolean)} [fix] Determines
     *      whether fixes should be applied.
     */
    /**
     * @typedef {Object} InternalOptions
     * @property {string | null} warnInlineConfig The config name what `noInlineConfig` setting came from. If `noInlineConfig` setting didn't exist, this is null. If this is a config name, then the linter warns directive comments.
     * @property {"off" | "warn" | "error"} reportUnusedDisableDirectives (boolean values were normalized)
     */
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Ensures that variables representing built-in properties of the Global Object,
     * and any globals declared by special block comments, are present in the global
     * scope.
     * @param {Scope} globalScope The global scope.
     * @param {Object} configGlobals The globals declared in configuration
     * @param {{exportedVariables: Object, enabledGlobals: Object}} commentDirectives Directives from comment configuration
     * @returns {void}
     */
    function addDeclaredGlobals(globalScope, configGlobals, { exportedVariables, enabledGlobals }) {
        // Define configured global variables.
        for (const id of new Set([...Object.keys(configGlobals), ...Object.keys(enabledGlobals)])) {
            /*
             * `ConfigOps.normalizeConfigGlobal` will throw an error if a configured global value is invalid. However, these errors would
             * typically be caught when validating a config anyway (validity for inline global comments is checked separately).
             */
            const configValue = configGlobals[id] === void 0 ? void 0 : ConfigOps.normalizeConfigGlobal(configGlobals[id]);
            const commentValue = enabledGlobals[id] && enabledGlobals[id].value;
            const value = commentValue || configValue;
            const sourceComments = enabledGlobals[id] && enabledGlobals[id].comments;
            if (value === "off") {
                continue;
            }
            let variable = globalScope.set.get(id);
            if (!variable) {
                variable = new eslintScope.Variable(id, globalScope);
                globalScope.variables.push(variable);
                globalScope.set.set(id, variable);
            }
            variable.eslintImplicitGlobalSetting = configValue;
            variable.eslintExplicitGlobal = sourceComments !== void 0;
            variable.eslintExplicitGlobalComments = sourceComments;
            variable.writeable = (value === "writable");
        }
        // mark all exported variables as such
        Object.keys(exportedVariables).forEach(name => {
            const variable = globalScope.set.get(name);
            if (variable) {
                variable.eslintUsed = true;
            }
        });
        /*
         * "through" contains all references which definitions cannot be found.
         * Since we augment the global scope using configuration, we need to update
         * references and remove the ones that were added by configuration.
         */
        globalScope.through = globalScope.through.filter(reference => {
            const name = reference.identifier.name;
            const variable = globalScope.set.get(name);
            if (variable) {
                /*
                 * Links the variable and the reference.
                 * And this reference is removed from `Scope#through`.
                 */
                reference.resolved = variable;
                variable.references.push(reference);
                return false;
            }
            return true;
        });
    }
    /**
     * creates a missing-rule message.
     * @param {string} ruleId the ruleId to create
     * @returns {string} created error message
     * @private
     */
    function createMissingRuleMessage(ruleId) {
        return Object.prototype.hasOwnProperty.call(ruleReplacements.rules, ruleId)
            ? `Rule '${ruleId}' was removed and replaced by: ${ruleReplacements.rules[ruleId].join(", ")}`
            : `Definition for rule '${ruleId}' was not found.`;
    }
    /**
     * creates a linting problem
     * @param {Object} options to create linting error
     * @param {string} [options.ruleId] the ruleId to report
     * @param {Object} [options.loc] the loc to report
     * @param {string} [options.message] the error message to report
     * @param {string} [options.severity] the error message to report
     * @returns {LintMessage} created problem, returns a missing-rule problem if only provided ruleId.
     * @private
     */
    function createLintingProblem(options) {
        const { ruleId = null, loc = DEFAULT_ERROR_LOC, message = createMissingRuleMessage(options.ruleId), severity = 2 } = options;
        return {
            ruleId,
            message,
            line: loc.start.line,
            column: loc.start.column + 1,
            endLine: loc.end.line,
            endColumn: loc.end.column + 1,
            severity,
            nodeType: null
        };
    }
    /**
     * Creates a collection of disable directives from a comment
     * @param {Object} options to create disable directives
     * @param {("disable"|"enable"|"disable-line"|"disable-next-line")} options.type The type of directive comment
     * @param {{line: number, column: number}} options.loc The 0-based location of the comment token
     * @param {string} options.value The value after the directive in the comment
     * comment specified no specific rules, so it applies to all rules (e.g. `eslint-disable`)
     * @param {function(string): {create: Function}} options.ruleMapper A map from rule IDs to defined rules
     * @returns {Object} Directives and problems from the comment
     */
    function createDisableDirectives(options) {
        const { type, loc, value, ruleMapper } = options;
        const ruleIds = Object.keys(commentParser.parseListConfig(value));
        const directiveRules = ruleIds.length ? ruleIds : [null];
        const result = {
            directives: [],
            directiveProblems: [] // problems in directives
        };
        for (const ruleId of directiveRules) {
            // push to directives, if the rule is defined(including null, e.g. /*eslint enable*/)
            if (ruleId === null || ruleMapper(ruleId) !== null) {
                result.directives.push({ type, line: loc.start.line, column: loc.start.column + 1, ruleId });
            }
            else {
                result.directiveProblems.push(createLintingProblem({ ruleId, loc }));
            }
        }
        return result;
    }
    /**
     * Parses comments in file to extract file-specific config of rules, globals
     * and environments and merges them with global config; also code blocks
     * where reporting is disabled or enabled and merges them with reporting config.
     * @param {string} filename The file being checked.
     * @param {ASTNode} ast The top node of the AST.
     * @param {function(string): {create: Function}} ruleMapper A map from rule IDs to defined rules
     * @param {string|null} warnInlineConfig If a string then it should warn directive comments as disabled. The string value is the config name what the setting came from.
     * @returns {{configuredRules: Object, enabledGlobals: {value:string,comment:Token}[], exportedVariables: Object, problems: Problem[], disableDirectives: DisableDirective[]}}
     * A collection of the directive comments that were found, along with any problems that occurred when parsing
     */
    function getDirectiveComments(filename, ast, ruleMapper, warnInlineConfig) {
        const configuredRules = {};
        const enabledGlobals = Object.create(null);
        const exportedVariables = {};
        const problems = [];
        const disableDirectives = [];
        ast.comments.filter(token => token.type !== "Shebang").forEach(comment => {
            const trimmedCommentText = comment.value.trim();
            const match = /^(eslint(?:-env|-enable|-disable(?:(?:-next)?-line)?)?|exported|globals?)(?:\s|$)/u.exec(trimmedCommentText);
            if (!match) {
                return;
            }
            const directiveText = match[1];
            const lineCommentSupported = /^eslint-disable-(next-)?line$/u.test(directiveText);
            if (comment.type === "Line" && !lineCommentSupported) {
                return;
            }
            if (warnInlineConfig) {
                const kind = comment.type === "Block" ? `/*${directiveText}*/` : `//${directiveText}`;
                problems.push(createLintingProblem({
                    ruleId: null,
                    message: `'${kind}' has no effect because you have 'noInlineConfig' setting in ${warnInlineConfig}.`,
                    loc: comment.loc,
                    severity: 1
                }));
                return;
            }
            if (lineCommentSupported && comment.loc.start.line !== comment.loc.end.line) {
                const message = `${directiveText} comment should not span multiple lines.`;
                problems.push(createLintingProblem({
                    ruleId: null,
                    message,
                    loc: comment.loc
                }));
                return;
            }
            const directiveValue = trimmedCommentText.slice(match.index + directiveText.length);
            switch (directiveText) {
                case "eslint-disable":
                case "eslint-enable":
                case "eslint-disable-next-line":
                case "eslint-disable-line": {
                    const directiveType = directiveText.slice("eslint-".length);
                    const options = { type: directiveType, loc: comment.loc, value: directiveValue, ruleMapper };
                    const { directives, directiveProblems } = createDisableDirectives(options);
                    disableDirectives.push(...directives);
                    problems.push(...directiveProblems);
                    break;
                }
                case "exported":
                    Object.assign(exportedVariables, commentParser.parseStringConfig(directiveValue, comment));
                    break;
                case "globals":
                case "global":
                    for (const [id, { value }] of Object.entries(commentParser.parseStringConfig(directiveValue, comment))) {
                        let normalizedValue;
                        try {
                            normalizedValue = ConfigOps.normalizeConfigGlobal(value);
                        }
                        catch (err) {
                            problems.push(createLintingProblem({
                                ruleId: null,
                                loc: comment.loc,
                                message: err.message
                            }));
                            continue;
                        }
                        if (enabledGlobals[id]) {
                            enabledGlobals[id].comments.push(comment);
                            enabledGlobals[id].value = normalizedValue;
                        }
                        else {
                            enabledGlobals[id] = {
                                comments: [comment],
                                value: normalizedValue
                            };
                        }
                    }
                    break;
                case "eslint": {
                    const parseResult = commentParser.parseJsonConfig(directiveValue, comment.loc);
                    if (parseResult.success) {
                        Object.keys(parseResult.config).forEach(name => {
                            const rule = ruleMapper(name);
                            const ruleValue = parseResult.config[name];
                            if (rule === null) {
                                problems.push(createLintingProblem({ ruleId: name, loc: comment.loc }));
                                return;
                            }
                            try {
                                validator.validateRuleOptions(rule, name, ruleValue);
                            }
                            catch (err) {
                                problems.push(createLintingProblem({
                                    ruleId: name,
                                    message: err.message,
                                    loc: comment.loc
                                }));
                                // do not apply the config, if found invalid options.
                                return;
                            }
                            configuredRules[name] = ruleValue;
                        });
                    }
                    else {
                        problems.push(parseResult.error);
                    }
                    break;
                }
                // no default
            }
        });
        return {
            configuredRules,
            enabledGlobals,
            exportedVariables,
            problems,
            disableDirectives
        };
    }
    /**
     * Normalize ECMAScript version from the initial config
     * @param  {number} ecmaVersion ECMAScript version from the initial config
     * @returns {number} normalized ECMAScript version
     */
    function normalizeEcmaVersion(ecmaVersion) {
        /*
         * Calculate ECMAScript edition number from official year version starting with
         * ES2015, which corresponds with ES6 (or a difference of 2009).
         */
        return ecmaVersion >= 2015 ? ecmaVersion - 2009 : ecmaVersion;
    }
    const eslintEnvPattern = /\/\*\s*eslint-env\s(.+?)\*\//gu;
    /**
     * Checks whether or not there is a comment which has "eslint-env *" in a given text.
     * @param {string} text A source code text to check.
     * @returns {Object|null} A result of parseListConfig() with "eslint-env *" comment.
     */
    function findEslintEnv(text) {
        let match, retv;
        eslintEnvPattern.lastIndex = 0;
        while ((match = eslintEnvPattern.exec(text))) {
            retv = Object.assign(retv || {}, commentParser.parseListConfig(match[1]));
        }
        return retv;
    }
    /**
     * Convert "/path/to/<text>" to "<text>".
     * `CLIEngine#executeOnText()` method gives "/path/to/<text>" if the filename
     * was omitted because `configArray.extractConfig()` requires an absolute path.
     * But the linter should pass `<text>` to `RuleContext#getFilename()` in that
     * case.
     * Also, code blocks can have their virtual filename. If the parent filename was
     * `<text>`, the virtual filename is `<text>/0_foo.js` or something like (i.e.,
     * it's not an absolute path).
     * @param {string} filename The filename to normalize.
     * @returns {string} The normalized filename.
     */
    function normalizeFilename(filename) {
        const parts = filename.split(path.sep);
        const index = parts.lastIndexOf("<text>");
        return index === -1 ? filename : parts.slice(index).join(path.sep);
    }
    /**
     * Normalizes the possible options for `linter.verify` and `linter.verifyAndFix` to a
     * consistent shape.
     * @param {VerifyOptions} providedOptions Options
     * @param {ConfigData} config Config.
     * @returns {Required<VerifyOptions> & InternalOptions} Normalized options
     */
    function normalizeVerifyOptions(providedOptions, config) {
        const disableInlineConfig = config.noInlineConfig === true;
        const ignoreInlineConfig = providedOptions.allowInlineConfig === false;
        const configNameOfNoInlineConfig = config.configNameOfNoInlineConfig
            ? ` (${config.configNameOfNoInlineConfig})`
            : "";
        let reportUnusedDisableDirectives = providedOptions.reportUnusedDisableDirectives;
        if (typeof reportUnusedDisableDirectives === "boolean") {
            reportUnusedDisableDirectives = reportUnusedDisableDirectives ? "error" : "off";
        }
        if (typeof reportUnusedDisableDirectives !== "string") {
            reportUnusedDisableDirectives = config.reportUnusedDisableDirectives ? "warn" : "off";
        }
        return {
            filename: normalizeFilename(providedOptions.filename || "<input>"),
            allowInlineConfig: !ignoreInlineConfig,
            warnInlineConfig: disableInlineConfig && !ignoreInlineConfig
                ? `your config${configNameOfNoInlineConfig}`
                : null,
            reportUnusedDisableDirectives,
            disableFixes: Boolean(providedOptions.disableFixes)
        };
    }
    /**
     * Combines the provided parserOptions with the options from environments
     * @param {string} parserName The parser name which uses this options.
     * @param {ParserOptions} providedOptions The provided 'parserOptions' key in a config
     * @param {Environment[]} enabledEnvironments The environments enabled in configuration and with inline comments
     * @returns {ParserOptions} Resulting parser options after merge
     */
    function resolveParserOptions(parserName, providedOptions, enabledEnvironments) {
        const parserOptionsFromEnv = enabledEnvironments
            .filter(env => env.parserOptions)
            .reduce((parserOptions, env) => lodash.merge(parserOptions, env.parserOptions), {});
        const mergedParserOptions = lodash.merge(parserOptionsFromEnv, providedOptions || {});
        const isModule = mergedParserOptions.sourceType === "module";
        if (isModule) {
            /*
             * can't have global return inside of modules
             * TODO: espree validate parserOptions.globalReturn when sourceType is setting to module.(@aladdin-add)
             */
            mergedParserOptions.ecmaFeatures = Object.assign({}, mergedParserOptions.ecmaFeatures, { globalReturn: false });
        }
        /*
         * TODO: @aladdin-add
         * 1. for a 3rd-party parser, do not normalize parserOptions
         * 2. for espree, no need to do this (espree will do it)
         */
        mergedParserOptions.ecmaVersion = normalizeEcmaVersion(mergedParserOptions.ecmaVersion);
        return mergedParserOptions;
    }
    /**
     * Combines the provided globals object with the globals from environments
     * @param {Record<string, GlobalConf>} providedGlobals The 'globals' key in a config
     * @param {Environment[]} enabledEnvironments The environments enabled in configuration and with inline comments
     * @returns {Record<string, GlobalConf>} The resolved globals object
     */
    function resolveGlobals(providedGlobals, enabledEnvironments) {
        return Object.assign({}, ...enabledEnvironments.filter(env => env.globals).map(env => env.globals), providedGlobals);
    }
    /**
     * Strips Unicode BOM from a given text.
     * @param {string} text A text to strip.
     * @returns {string} The stripped text.
     */
    function stripUnicodeBOM(text) {
        /*
         * Check Unicode BOM.
         * In JavaScript, string data is stored as UTF-16, so BOM is 0xFEFF.
         * http://www.ecma-international.org/ecma-262/6.0/#sec-unicode-format-control-characters
         */
        if (text.charCodeAt(0) === 0xFEFF) {
            return text.slice(1);
        }
        return text;
    }
    /**
     * Get the options for a rule (not including severity), if any
     * @param {Array|number} ruleConfig rule configuration
     * @returns {Array} of rule options, empty Array if none
     */
    function getRuleOptions(ruleConfig) {
        if (Array.isArray(ruleConfig)) {
            return ruleConfig.slice(1);
        }
        return [];
    }
    /**
     * Analyze scope of the given AST.
     * @param {ASTNode} ast The `Program` node to analyze.
     * @param {ParserOptions} parserOptions The parser options.
     * @param {Record<string, string[]>} visitorKeys The visitor keys.
     * @returns {ScopeManager} The analysis result.
     */
    function analyzeScope(ast, parserOptions, visitorKeys) {
        const ecmaFeatures = parserOptions.ecmaFeatures || {};
        const ecmaVersion = parserOptions.ecmaVersion || 5;
        return eslintScope.analyze(ast, {
            ignoreEval: true,
            nodejsScope: ecmaFeatures.globalReturn,
            impliedStrict: ecmaFeatures.impliedStrict,
            ecmaVersion,
            sourceType: parserOptions.sourceType || "script",
            childVisitorKeys: visitorKeys || evk.KEYS,
            fallback: Traverser.getKeys
        });
    }
    /**
     * Parses text into an AST. Moved out here because the try-catch prevents
     * optimization of functions, so it's best to keep the try-catch as isolated
     * as possible
     * @param {string} text The text to parse.
     * @param {Parser} parser The parser to parse.
     * @param {ParserOptions} providedParserOptions Options to pass to the parser
     * @param {string} filePath The path to the file being parsed.
     * @returns {{success: false, error: Problem}|{success: true, sourceCode: SourceCode}}
     * An object containing the AST and parser services if parsing was successful, or the error if parsing failed
     * @private
     */
    function parse(text, parser, providedParserOptions, filePath) {
        const textToParse = stripUnicodeBOM(text).replace(astUtils.shebangPattern, (match, captured) => `//${captured}`);
        const parserOptions = Object.assign({}, providedParserOptions, {
            loc: true,
            range: true,
            raw: true,
            tokens: true,
            comment: true,
            eslintVisitorKeys: true,
            eslintScopeManager: true,
            filePath
        });
        /*
         * Check for parsing errors first. If there's a parsing error, nothing
         * else can happen. However, a parsing error does not throw an error
         * from this method - it's just considered a fatal error message, a
         * problem that ESLint identified just like any other.
         */
        try {
            const parseResult = (typeof parser.parseForESLint === "function")
                ? parser.parseForESLint(textToParse, parserOptions)
                : { ast: parser.parse(textToParse, parserOptions) };
            const ast = parseResult.ast;
            const parserServices = parseResult.services || {};
            const visitorKeys = parseResult.visitorKeys || evk.KEYS;
            const scopeManager = parseResult.scopeManager || analyzeScope(ast, parserOptions, visitorKeys);
            return {
                success: true,
                /*
                 * Save all values that `parseForESLint()` returned.
                 * If a `SourceCode` object is given as the first parameter instead of source code text,
                 * linter skips the parsing process and reuses the source code object.
                 * In that case, linter needs all the values that `parseForESLint()` returned.
                 */
                sourceCode: new SourceCode({
                    text,
                    ast,
                    parserServices,
                    scopeManager,
                    visitorKeys
                })
            };
        }
        catch (ex) {
            // If the message includes a leading line number, strip it:
            const message = `Parsing error: ${ex.message.replace(/^line \d+:/iu, "").trim()}`;
            debug("%s\n%s", message, ex.stack);
            return {
                success: false,
                error: {
                    ruleId: null,
                    fatal: true,
                    severity: 2,
                    message,
                    line: ex.lineNumber,
                    column: ex.column
                }
            };
        }
    }
    /**
     * Gets the scope for the current node
     * @param {ScopeManager} scopeManager The scope manager for this AST
     * @param {ASTNode} currentNode The node to get the scope of
     * @returns {eslint-scope.Scope} The scope information for this node
     */
    function getScope(scopeManager, currentNode) {
        // On Program node, get the outermost scope to avoid return Node.js special function scope or ES modules scope.
        const inner = currentNode.type !== "Program";
        for (let node = currentNode; node; node = node.parent) {
            const scope = scopeManager.acquire(node, inner);
            if (scope) {
                if (scope.type === "function-expression-name") {
                    return scope.childScopes[0];
                }
                return scope;
            }
        }
        return scopeManager.scopes[0];
    }
    /**
     * Marks a variable as used in the current scope
     * @param {ScopeManager} scopeManager The scope manager for this AST. The scope may be mutated by this function.
     * @param {ASTNode} currentNode The node currently being traversed
     * @param {Object} parserOptions The options used to parse this text
     * @param {string} name The name of the variable that should be marked as used.
     * @returns {boolean} True if the variable was found and marked as used, false if not.
     */
    function markVariableAsUsed(scopeManager, currentNode, parserOptions, name) {
        const hasGlobalReturn = parserOptions.ecmaFeatures && parserOptions.ecmaFeatures.globalReturn;
        const specialScope = hasGlobalReturn || parserOptions.sourceType === "module";
        const currentScope = getScope(scopeManager, currentNode);
        // Special Node.js scope means we need to start one level deeper
        const initialScope = currentScope.type === "global" && specialScope ? currentScope.childScopes[0] : currentScope;
        for (let scope = initialScope; scope; scope = scope.upper) {
            const variable = scope.variables.find(scopeVar => scopeVar.name === name);
            if (variable) {
                variable.eslintUsed = true;
                return true;
            }
        }
        return false;
    }
    /**
     * Runs a rule, and gets its listeners
     * @param {Rule} rule A normalized rule with a `create` method
     * @param {Context} ruleContext The context that should be passed to the rule
     * @returns {Object} A map of selector listeners provided by the rule
     */
    function createRuleListeners(rule, ruleContext) {
        try {
            return rule.create(ruleContext);
        }
        catch (ex) {
            ex.message = `Error while loading rule '${ruleContext.id}': ${ex.message}`;
            throw ex;
        }
    }
    /**
     * Gets all the ancestors of a given node
     * @param {ASTNode} node The node
     * @returns {ASTNode[]} All the ancestor nodes in the AST, not including the provided node, starting
     * from the root node and going inwards to the parent node.
     */
    function getAncestors(node) {
        const ancestorsStartingAtParent = [];
        for (let ancestor = node.parent; ancestor; ancestor = ancestor.parent) {
            ancestorsStartingAtParent.push(ancestor);
        }
        return ancestorsStartingAtParent.reverse();
    }
    // methods that exist on SourceCode object
    const DEPRECATED_SOURCECODE_PASSTHROUGHS = {
        getSource: "getText",
        getSourceLines: "getLines",
        getAllComments: "getAllComments",
        getNodeByRangeIndex: "getNodeByRangeIndex",
        getComments: "getComments",
        getCommentsBefore: "getCommentsBefore",
        getCommentsAfter: "getCommentsAfter",
        getCommentsInside: "getCommentsInside",
        getJSDocComment: "getJSDocComment",
        getFirstToken: "getFirstToken",
        getFirstTokens: "getFirstTokens",
        getLastToken: "getLastToken",
        getLastTokens: "getLastTokens",
        getTokenAfter: "getTokenAfter",
        getTokenBefore: "getTokenBefore",
        getTokenByRangeStart: "getTokenByRangeStart",
        getTokens: "getTokens",
        getTokensAfter: "getTokensAfter",
        getTokensBefore: "getTokensBefore",
        getTokensBetween: "getTokensBetween"
    };
    const BASE_TRAVERSAL_CONTEXT = Object.freeze(Object.keys(DEPRECATED_SOURCECODE_PASSTHROUGHS).reduce((contextInfo, methodName) => Object.assign(contextInfo, {
        [methodName](...args) {
            return this.getSourceCode()[DEPRECATED_SOURCECODE_PASSTHROUGHS[methodName]](...args);
        }
    }), {}));
    /**
     * Runs the given rules on the given SourceCode object
     * @param {SourceCode} sourceCode A SourceCode object for the given text
     * @param {Object} configuredRules The rules configuration
     * @param {function(string): Rule} ruleMapper A mapper function from rule names to rules
     * @param {Object} parserOptions The options that were passed to the parser
     * @param {string} parserName The name of the parser in the config
     * @param {Object} settings The settings that were enabled in the config
     * @param {string} filename The reported filename of the code
     * @param {boolean} disableFixes If true, it doesn't make `fix` properties.
     * @param {string | undefined} cwd cwd of the cli
     * @returns {Problem[]} An array of reported problems
     */
    function runRules(sourceCode, configuredRules, ruleMapper, parserOptions, parserName, settings, filename, disableFixes, cwd) {
        const emitter = createEmitter();
        const nodeQueue = [];
        let currentNode = sourceCode.ast;
        Traverser.traverse(sourceCode.ast, {
            enter(node, parent) {
                node.parent = parent;
                nodeQueue.push({ isEntering: true, node });
            },
            leave(node) {
                nodeQueue.push({ isEntering: false, node });
            },
            visitorKeys: sourceCode.visitorKeys
        });
        /*
         * Create a frozen object with the ruleContext properties and methods that are shared by all rules.
         * All rule contexts will inherit from this object. This avoids the performance penalty of copying all the
         * properties once for each rule.
         */
        const sharedTraversalContext = Object.freeze(Object.assign(Object.create(BASE_TRAVERSAL_CONTEXT), {
            getAncestors: () => getAncestors(currentNode),
            getDeclaredVariables: sourceCode.scopeManager.getDeclaredVariables.bind(sourceCode.scopeManager),
            getCwd: () => cwd,
            getFilename: () => filename,
            getScope: () => getScope(sourceCode.scopeManager, currentNode),
            getSourceCode: () => sourceCode,
            markVariableAsUsed: name => markVariableAsUsed(sourceCode.scopeManager, currentNode, parserOptions, name),
            parserOptions,
            parserPath: parserName,
            parserServices: sourceCode.parserServices,
            settings
        }));
        const lintingProblems = [];
        Object.keys(configuredRules).forEach(ruleId => {
            const severity = ConfigOps.getRuleSeverity(configuredRules[ruleId]);
            // not load disabled rules
            if (severity === 0) {
                return;
            }
            const rule = ruleMapper(ruleId);
            if (rule === null) {
                lintingProblems.push(createLintingProblem({ ruleId }));
                return;
            }
            const messageIds = rule.meta && rule.meta.messages;
            let reportTranslator = null;
            const ruleContext = Object.freeze(Object.assign(Object.create(sharedTraversalContext), {
                id: ruleId,
                options: getRuleOptions(configuredRules[ruleId]),
                report(...args) {
                    /*
                     * Create a report translator lazily.
                     * In a vast majority of cases, any given rule reports zero errors on a given
                     * piece of code. Creating a translator lazily avoids the performance cost of
                     * creating a new translator function for each rule that usually doesn't get
                     * called.
                     *
                     * Using lazy report translators improves end-to-end performance by about 3%
                     * with Node 8.4.0.
                     */
                    if (reportTranslator === null) {
                        reportTranslator = createReportTranslator({
                            ruleId,
                            severity,
                            sourceCode,
                            messageIds,
                            disableFixes
                        });
                    }
                    const problem = reportTranslator(...args);
                    if (problem.fix && rule.meta && !rule.meta.fixable) {
                        throw new Error("Fixable rules should export a `meta.fixable` property.");
                    }
                    lintingProblems.push(problem);
                }
            }));
            const ruleListeners = createRuleListeners(rule, ruleContext);
            // add all the selectors from the rule as listeners
            Object.keys(ruleListeners).forEach(selector => {
                emitter.on(selector, timing.enabled
                    ? timing.time(ruleId, ruleListeners[selector])
                    : ruleListeners[selector]);
            });
        });
        const eventGenerator = new CodePathAnalyzer(new NodeEventGenerator(emitter));
        nodeQueue.forEach(traversalInfo => {
            currentNode = traversalInfo.node;
            try {
                if (traversalInfo.isEntering) {
                    eventGenerator.enterNode(currentNode);
                }
                else {
                    eventGenerator.leaveNode(currentNode);
                }
            }
            catch (err) {
                err.currentNode = currentNode;
                throw err;
            }
        });
        return lintingProblems;
    }
    /**
     * Ensure the source code to be a string.
     * @param {string|SourceCode} textOrSourceCode The text or source code object.
     * @returns {string} The source code text.
     */
    function ensureText(textOrSourceCode) {
        if (typeof textOrSourceCode === "object") {
            const { hasBOM, text } = textOrSourceCode;
            const bom = hasBOM ? "\uFEFF" : "";
            return bom + text;
        }
        return String(textOrSourceCode);
    }
    /**
     * Get an environment.
     * @param {LinterInternalSlots} slots The internal slots of Linter.
     * @param {string} envId The environment ID to get.
     * @returns {Environment|null} The environment.
     */
    function getEnv(slots, envId) {
        return ((slots.lastConfigArray && slots.lastConfigArray.pluginEnvironments.get(envId)) ||
            BuiltInEnvironments.get(envId) ||
            null);
    }
    /**
     * Get a rule.
     * @param {LinterInternalSlots} slots The internal slots of Linter.
     * @param {string} ruleId The rule ID to get.
     * @returns {Rule|null} The rule.
     */
    function getRule(slots, ruleId) {
        return ((slots.lastConfigArray && slots.lastConfigArray.pluginRules.get(ruleId)) ||
            slots.ruleMap.get(ruleId));
    }
    /**
     * Normalize the value of the cwd
     * @param {string | undefined} cwd raw value of the cwd, path to a directory that should be considered as the current working directory, can be undefined.
     * @returns {string | undefined} normalized cwd
     */
    function normalizeCwd(cwd) {
        if (cwd) {
            return cwd;
        }
        if (typeof process === "object") {
            return process.cwd();
        }
        // It's more explicit to assign the undefined
        // eslint-disable-next-line no-undefined
        return undefined;
    }
    /**
     * The map to store private data.
     * @type {WeakMap<Linter, LinterInternalSlots>}
     */
    const internalSlotsMap = new WeakMap();
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * Object that is responsible for verifying JavaScript text
     * @name eslint
     */
    class Linter {
        /**
         * Initialize the Linter.
         * @param {Object} [config] the config object
         * @param {string} [config.cwd]  path to a directory that should be considered as the current working directory, can be undefined.
         */
        constructor({ cwd } = {}) {
            internalSlotsMap.set(this, {
                cwd: normalizeCwd(cwd),
                lastConfigArray: null,
                lastSourceCode: null,
                parserMap: new Map([["espree", espree]]),
                ruleMap: new Rules()
            });
            this.version = pkg.version;
        }
        /**
         * Getter for package version.
         * @static
         * @returns {string} The version from package.json.
         */
        static get version() {
            return pkg.version;
        }
        /**
         * Same as linter.verify, except without support for processors.
         * @param {string|SourceCode} textOrSourceCode The text to parse or a SourceCode object.
         * @param {ConfigData} providedConfig An ESLintConfig instance to configure everything.
         * @param {VerifyOptions} [providedOptions] The optional filename of the file being checked.
         * @returns {LintMessage[]} The results as an array of messages or an empty array if no messages.
         */
        _verifyWithoutProcessors(textOrSourceCode, providedConfig, providedOptions) {
            const slots = internalSlotsMap.get(this);
            const config = providedConfig || {};
            const options = normalizeVerifyOptions(providedOptions, config);
            let text;
            // evaluate arguments
            if (typeof textOrSourceCode === "string") {
                slots.lastSourceCode = null;
                text = textOrSourceCode;
            }
            else {
                slots.lastSourceCode = textOrSourceCode;
                text = textOrSourceCode.text;
            }
            // Resolve parser.
            let parserName = DEFAULT_PARSER_NAME;
            let parser = espree;
            if (typeof config.parser === "object" && config.parser !== null) {
                parserName = config.parser.filePath;
                parser = config.parser.definition;
            }
            else if (typeof config.parser === "string") {
                if (!slots.parserMap.has(config.parser)) {
                    return [{
                            ruleId: null,
                            fatal: true,
                            severity: 2,
                            message: `Configured parser '${config.parser}' was not found.`,
                            line: 0,
                            column: 0
                        }];
                }
                parserName = config.parser;
                parser = slots.parserMap.get(config.parser);
            }
            // search and apply "eslint-env *".
            const envInFile = options.allowInlineConfig && !options.warnInlineConfig
                ? findEslintEnv(text)
                : {};
            const resolvedEnvConfig = Object.assign({ builtin: true }, config.env, envInFile);
            const enabledEnvs = Object.keys(resolvedEnvConfig)
                .filter(envName => resolvedEnvConfig[envName])
                .map(envName => getEnv(slots, envName))
                .filter(env => env);
            const parserOptions = resolveParserOptions(parserName, config.parserOptions || {}, enabledEnvs);
            const configuredGlobals = resolveGlobals(config.globals || {}, enabledEnvs);
            const settings = config.settings || {};
            if (!slots.lastSourceCode) {
                const parseResult = parse(text, parser, parserOptions, options.filename);
                if (!parseResult.success) {
                    return [parseResult.error];
                }
                slots.lastSourceCode = parseResult.sourceCode;
            }
            else {
                /*
                 * If the given source code object as the first argument does not have scopeManager, analyze the scope.
                 * This is for backward compatibility (SourceCode is frozen so it cannot rebind).
                 */
                if (!slots.lastSourceCode.scopeManager) {
                    slots.lastSourceCode = new SourceCode({
                        text: slots.lastSourceCode.text,
                        ast: slots.lastSourceCode.ast,
                        parserServices: slots.lastSourceCode.parserServices,
                        visitorKeys: slots.lastSourceCode.visitorKeys,
                        scopeManager: analyzeScope(slots.lastSourceCode.ast, parserOptions)
                    });
                }
            }
            const sourceCode = slots.lastSourceCode;
            const commentDirectives = options.allowInlineConfig
                ? getDirectiveComments(options.filename, sourceCode.ast, ruleId => getRule(slots, ruleId), options.warnInlineConfig)
                : { configuredRules: {}, enabledGlobals: {}, exportedVariables: {}, problems: [], disableDirectives: [] };
            // augment global scope with declared global variables
            addDeclaredGlobals(sourceCode.scopeManager.scopes[0], configuredGlobals, { exportedVariables: commentDirectives.exportedVariables, enabledGlobals: commentDirectives.enabledGlobals });
            const configuredRules = Object.assign({}, config.rules, commentDirectives.configuredRules);
            let lintingProblems;
            try {
                lintingProblems = runRules(sourceCode, configuredRules, ruleId => getRule(slots, ruleId), parserOptions, parserName, settings, options.filename, options.disableFixes, slots.cwd);
            }
            catch (err) {
                err.message += `\nOccurred while linting ${options.filename}`;
                debug("An error occurred while traversing");
                debug("Filename:", options.filename);
                if (err.currentNode) {
                    const { line } = err.currentNode.loc.start;
                    debug("Line:", line);
                    err.message += `:${line}`;
                }
                debug("Parser Options:", parserOptions);
                debug("Parser Path:", parserName);
                debug("Settings:", settings);
                throw err;
            }
            return applyDisableDirectives({
                directives: commentDirectives.disableDirectives,
                problems: lintingProblems
                    .concat(commentDirectives.problems)
                    .sort((problemA, problemB) => problemA.line - problemB.line || problemA.column - problemB.column),
                reportUnusedDisableDirectives: options.reportUnusedDisableDirectives
            });
        }
        /**
         * Verifies the text against the rules specified by the second argument.
         * @param {string|SourceCode} textOrSourceCode The text to parse or a SourceCode object.
         * @param {ConfigData|ConfigArray} config An ESLintConfig instance to configure everything.
         * @param {(string|(VerifyOptions&ProcessorOptions))} [filenameOrOptions] The optional filename of the file being checked.
         *      If this is not set, the filename will default to '<input>' in the rule context. If
         *      an object, then it has "filename", "allowInlineConfig", and some properties.
         * @returns {LintMessage[]} The results as an array of messages or an empty array if no messages.
         */
        verify(textOrSourceCode, config, filenameOrOptions) {
            debug("Verify");
            const options = typeof filenameOrOptions === "string"
                ? { filename: filenameOrOptions }
                : filenameOrOptions || {};
            // CLIEngine passes a `ConfigArray` object.
            if (config && typeof config.extractConfig === "function") {
                return this._verifyWithConfigArray(textOrSourceCode, config, options);
            }
            /*
             * `Linter` doesn't support `overrides` property in configuration.
             * So we cannot apply multiple processors.
             */
            if (options.preprocess || options.postprocess) {
                return this._verifyWithProcessor(textOrSourceCode, config, options);
            }
            return this._verifyWithoutProcessors(textOrSourceCode, config, options);
        }
        /**
         * Verify a given code with `ConfigArray`.
         * @param {string|SourceCode} textOrSourceCode The source code.
         * @param {ConfigArray} configArray The config array.
         * @param {VerifyOptions&ProcessorOptions} options The options.
         * @returns {LintMessage[]} The found problems.
         */
        _verifyWithConfigArray(textOrSourceCode, configArray, options) {
            debug("With ConfigArray: %s", options.filename);
            // Store the config array in order to get plugin envs and rules later.
            internalSlotsMap.get(this).lastConfigArray = configArray;
            // Extract the final config for this file.
            const config = configArray.extractConfig(options.filename);
            const processor = config.processor &&
                configArray.pluginProcessors.get(config.processor);
            // Verify.
            if (processor) {
                debug("Apply the processor: %o", config.processor);
                const { preprocess, postprocess, supportsAutofix } = processor;
                const disableFixes = options.disableFixes || !supportsAutofix;
                return this._verifyWithProcessor(textOrSourceCode, config, { ...options, disableFixes, postprocess, preprocess }, configArray);
            }
            return this._verifyWithoutProcessors(textOrSourceCode, config, options);
        }
        /**
         * Verify with a processor.
         * @param {string|SourceCode} textOrSourceCode The source code.
         * @param {ConfigData|ExtractedConfig} config The config array.
         * @param {VerifyOptions&ProcessorOptions} options The options.
         * @param {ConfigArray} [configForRecursive] The `CofnigArray` object to apply multiple processors recursively.
         * @returns {LintMessage[]} The found problems.
         */
        _verifyWithProcessor(textOrSourceCode, config, options, configForRecursive) {
            const filename = options.filename || "<input>";
            const filenameToExpose = normalizeFilename(filename);
            const text = ensureText(textOrSourceCode);
            const preprocess = options.preprocess || (rawText => [rawText]);
            const postprocess = options.postprocess || lodash.flatten;
            const filterCodeBlock = options.filterCodeBlock ||
                (blockFilename => blockFilename.endsWith(".js"));
            const originalExtname = path.extname(filename);
            const messageLists = preprocess(text, filenameToExpose).map((block, i) => {
                debug("A code block was found: %o", block.filename || "(unnamed)");
                // Keep the legacy behavior.
                if (typeof block === "string") {
                    return this._verifyWithoutProcessors(block, config, options);
                }
                const blockText = block.text;
                const blockName = path.join(filename, `${i}_${block.filename}`);
                // Skip this block if filtered.
                if (!filterCodeBlock(blockName, blockText)) {
                    debug("This code block was skipped.");
                    return [];
                }
                // Resolve configuration again if the file extension was changed.
                if (configForRecursive && path.extname(blockName) !== originalExtname) {
                    debug("Resolving configuration again because the file extension was changed.");
                    return this._verifyWithConfigArray(blockText, configForRecursive, { ...options, filename: blockName });
                }
                // Does lint.
                return this._verifyWithoutProcessors(blockText, config, { ...options, filename: blockName });
            });
            return postprocess(messageLists, filenameToExpose);
        }
        /**
         * Gets the SourceCode object representing the parsed source.
         * @returns {SourceCode} The SourceCode object.
         */
        getSourceCode() {
            return internalSlotsMap.get(this).lastSourceCode;
        }
        /**
         * Defines a new linting rule.
         * @param {string} ruleId A unique rule identifier
         * @param {Function | Rule} ruleModule Function from context to object mapping AST node types to event handlers
         * @returns {void}
         */
        defineRule(ruleId, ruleModule) {
            internalSlotsMap.get(this).ruleMap.define(ruleId, ruleModule);
        }
        /**
         * Defines many new linting rules.
         * @param {Record<string, Function | Rule>} rulesToDefine map from unique rule identifier to rule
         * @returns {void}
         */
        defineRules(rulesToDefine) {
            Object.getOwnPropertyNames(rulesToDefine).forEach(ruleId => {
                this.defineRule(ruleId, rulesToDefine[ruleId]);
            });
        }
        /**
         * Gets an object with all loaded rules.
         * @returns {Map<string, Rule>} All loaded rules
         */
        getRules() {
            const { lastConfigArray, ruleMap } = internalSlotsMap.get(this);
            return new Map(function* () {
                yield* ruleMap;
                if (lastConfigArray) {
                    yield* lastConfigArray.pluginRules;
                }
            }());
        }
        /**
         * Define a new parser module
         * @param {string} parserId Name of the parser
         * @param {Parser} parserModule The parser object
         * @returns {void}
         */
        defineParser(parserId, parserModule) {
            internalSlotsMap.get(this).parserMap.set(parserId, parserModule);
        }
        /**
         * Performs multiple autofix passes over the text until as many fixes as possible
         * have been applied.
         * @param {string} text The source text to apply fixes to.
         * @param {ConfigData|ConfigArray} config The ESLint config object to use.
         * @param {VerifyOptions&ProcessorOptions&FixOptions} options The ESLint options object to use.
         * @returns {{fixed:boolean,messages:LintMessage[],output:string}} The result of the fix operation as returned from the
         *      SourceCodeFixer.
         */
        verifyAndFix(text, config, options) {
            let messages = [], fixedResult, fixed = false, passNumber = 0, currentText = text;
            const debugTextDescription = options && options.filename || `${text.slice(0, 10)}...`;
            const shouldFix = options && typeof options.fix !== "undefined" ? options.fix : true;
            /**
             * This loop continues until one of the following is true:
             *
             * 1. No more fixes have been applied.
             * 2. Ten passes have been made.
             *
             * That means anytime a fix is successfully applied, there will be another pass.
             * Essentially, guaranteeing a minimum of two passes.
             */
            do {
                passNumber++;
                debug(`Linting code for ${debugTextDescription} (pass ${passNumber})`);
                messages = this.verify(currentText, config, options);
                debug(`Generating fixed text for ${debugTextDescription} (pass ${passNumber})`);
                fixedResult = SourceCodeFixer.applyFixes(currentText, messages, shouldFix);
                /*
                 * stop if there are any syntax errors.
                 * 'fixedResult.output' is a empty string.
                 */
                if (messages.length === 1 && messages[0].fatal) {
                    break;
                }
                // keep track if any fixes were ever applied - important for return value
                fixed = fixed || fixedResult.fixed;
                // update to use the fixed output instead of the original text
                currentText = fixedResult.output;
            } while (fixedResult.fixed &&
                passNumber < MAX_AUTOFIX_PASSES);
            /*
             * If the last result had fixes, we need to lint again to be sure we have
             * the most up-to-date information.
             */
            if (fixedResult.fixed) {
                fixedResult.messages = this.verify(currentText, config, options);
            }
            // ensure the last result properly reflects if fixes were done
            fixedResult.fixed = fixed;
            fixedResult.output = currentText;
            return fixedResult;
        }
    }
    module.exports = {
        Linter,
        /**
         * Get the internal slots of a given Linter instance for tests.
         * @param {Linter} instance The Linter instance to get.
         * @returns {LinterInternalSlots} The internal slots.
         */
        getLinterInternalSlots(instance) {
            return internalSlotsMap.get(instance);
        }
    };
},
/* eslint-scope/lib/index.js */ function _(require, module, exports) {
    /* eslint no-underscore-dangle: ["error", { "allow": ["__currentScope"] }] */
    const assert = require("assert");
    const ScopeManager = require(88) /* ./scope-manager */;
    const Referencer = require(95) /* ./referencer */;
    const Reference = require(92) /* ./reference */;
    const Variable = require(93) /* ./variable */;
    const Scope = require(89) /* ./scope */.Scope;
    const version = require(99) /* ../package.json */.version;
    /**
     * Set the default options
     * @returns {Object} options
     */
    function defaultOptions() {
        return {
            optimistic: false,
            directive: false,
            nodejsScope: false,
            impliedStrict: false,
            sourceType: "script",
            ecmaVersion: 5,
            childVisitorKeys: null,
            fallback: "iteration"
        };
    }
    /**
     * Preform deep update on option object
     * @param {Object} target - Options
     * @param {Object} override - Updates
     * @returns {Object} Updated options
     */
    function updateDeeply(target, override) {
        /**
         * Is hash object
         * @param {Object} value - Test value
         * @returns {boolean} Result
         */
        function isHashObject(value) {
            return typeof value === "object" && value instanceof Object && !(value instanceof Array) && !(value instanceof RegExp);
        }
        for (const key in override) {
            if (Object.prototype.hasOwnProperty.call(override, key)) {
                const val = override[key];
                if (isHashObject(val)) {
                    if (isHashObject(target[key])) {
                        updateDeeply(target[key], val);
                    }
                    else {
                        target[key] = updateDeeply({}, val);
                    }
                }
                else {
                    target[key] = val;
                }
            }
        }
        return target;
    }
    /**
     * Main interface function. Takes an Espree syntax tree and returns the
     * analyzed scopes.
     * @function analyze
     * @param {espree.Tree} tree - Abstract Syntax Tree
     * @param {Object} providedOptions - Options that tailor the scope analysis
     * @param {boolean} [providedOptions.optimistic=false] - the optimistic flag
     * @param {boolean} [providedOptions.directive=false]- the directive flag
     * @param {boolean} [providedOptions.ignoreEval=false]- whether to check 'eval()' calls
     * @param {boolean} [providedOptions.nodejsScope=false]- whether the whole
     * script is executed under node.js environment. When enabled, escope adds
     * a function scope immediately following the global scope.
     * @param {boolean} [providedOptions.impliedStrict=false]- implied strict mode
     * (if ecmaVersion >= 5).
     * @param {string} [providedOptions.sourceType='script']- the source type of the script. one of 'script' and 'module'
     * @param {number} [providedOptions.ecmaVersion=5]- which ECMAScript version is considered
     * @param {Object} [providedOptions.childVisitorKeys=null] - Additional known visitor keys. See [esrecurse](https://github.com/estools/esrecurse)'s the `childVisitorKeys` option.
     * @param {string} [providedOptions.fallback='iteration'] - A kind of the fallback in order to encounter with unknown node. See [esrecurse](https://github.com/estools/esrecurse)'s the `fallback` option.
     * @returns {ScopeManager} ScopeManager
     */
    function analyze(tree, providedOptions) {
        const options = updateDeeply(defaultOptions(), providedOptions);
        const scopeManager = new ScopeManager(options);
        const referencer = new Referencer(options, scopeManager);
        referencer.visit(tree);
        assert(scopeManager.__currentScope === null, "currentScope should be null.");
        return scopeManager;
    }
    module.exports = {
        /** @name module:escope.version */
        version,
        /** @name module:escope.Reference */
        Reference,
        /** @name module:escope.Variable */
        Variable,
        /** @name module:escope.Scope */
        Scope,
        /** @name module:escope.ScopeManager */
        ScopeManager,
        analyze
    };
},
/* eslint-scope/lib/scope-manager.js */ function _(require, module, exports) {
    /* eslint-disable no-underscore-dangle */
    const Scope = require(89) /* ./scope */;
    const assert = require("assert");
    const GlobalScope = Scope.GlobalScope;
    const CatchScope = Scope.CatchScope;
    const WithScope = Scope.WithScope;
    const ModuleScope = Scope.ModuleScope;
    const ClassScope = Scope.ClassScope;
    const SwitchScope = Scope.SwitchScope;
    const FunctionScope = Scope.FunctionScope;
    const ForScope = Scope.ForScope;
    const FunctionExpressionNameScope = Scope.FunctionExpressionNameScope;
    const BlockScope = Scope.BlockScope;
    /**
     * @class ScopeManager
     */
    class ScopeManager {
        constructor(options) {
            this.scopes = [];
            this.globalScope = null;
            this.__nodeToScope = new WeakMap();
            this.__currentScope = null;
            this.__options = options;
            this.__declaredVariables = new WeakMap();
        }
        __useDirective() {
            return this.__options.directive;
        }
        __isOptimistic() {
            return this.__options.optimistic;
        }
        __ignoreEval() {
            return this.__options.ignoreEval;
        }
        __isNodejsScope() {
            return this.__options.nodejsScope;
        }
        isModule() {
            return this.__options.sourceType === "module";
        }
        isImpliedStrict() {
            return this.__options.impliedStrict;
        }
        isStrictModeSupported() {
            return this.__options.ecmaVersion >= 5;
        }
        // Returns appropriate scope for this node.
        __get(node) {
            return this.__nodeToScope.get(node);
        }
        /**
         * Get variables that are declared by the node.
         *
         * "are declared by the node" means the node is same as `Variable.defs[].node` or `Variable.defs[].parent`.
         * If the node declares nothing, this method returns an empty array.
         * CAUTION: This API is experimental. See https://github.com/estools/escope/pull/69 for more details.
         *
         * @param {Espree.Node} node - a node to get.
         * @returns {Variable[]} variables that declared by the node.
         */
        getDeclaredVariables(node) {
            return this.__declaredVariables.get(node) || [];
        }
        /**
         * acquire scope from node.
         * @method ScopeManager#acquire
         * @param {Espree.Node} node - node for the acquired scope.
         * @param {boolean=} inner - look up the most inner scope, default value is false.
         * @returns {Scope?} Scope from node
         */
        acquire(node, inner) {
            /**
             * predicate
             * @param {Scope} testScope - scope to test
             * @returns {boolean} predicate
             */
            function predicate(testScope) {
                if (testScope.type === "function" && testScope.functionExpressionScope) {
                    return false;
                }
                return true;
            }
            const scopes = this.__get(node);
            if (!scopes || scopes.length === 0) {
                return null;
            }
            // Heuristic selection from all scopes.
            // If you would like to get all scopes, please use ScopeManager#acquireAll.
            if (scopes.length === 1) {
                return scopes[0];
            }
            if (inner) {
                for (let i = scopes.length - 1; i >= 0; --i) {
                    const scope = scopes[i];
                    if (predicate(scope)) {
                        return scope;
                    }
                }
            }
            else {
                for (let i = 0, iz = scopes.length; i < iz; ++i) {
                    const scope = scopes[i];
                    if (predicate(scope)) {
                        return scope;
                    }
                }
            }
            return null;
        }
        /**
         * acquire all scopes from node.
         * @method ScopeManager#acquireAll
         * @param {Espree.Node} node - node for the acquired scope.
         * @returns {Scopes?} Scope array
         */
        acquireAll(node) {
            return this.__get(node);
        }
        /**
         * release the node.
         * @method ScopeManager#release
         * @param {Espree.Node} node - releasing node.
         * @param {boolean=} inner - look up the most inner scope, default value is false.
         * @returns {Scope?} upper scope for the node.
         */
        release(node, inner) {
            const scopes = this.__get(node);
            if (scopes && scopes.length) {
                const scope = scopes[0].upper;
                if (!scope) {
                    return null;
                }
                return this.acquire(scope.block, inner);
            }
            return null;
        }
        attach() { } // eslint-disable-line class-methods-use-this
        detach() { } // eslint-disable-line class-methods-use-this
        __nestScope(scope) {
            if (scope instanceof GlobalScope) {
                assert(this.__currentScope === null);
                this.globalScope = scope;
            }
            this.__currentScope = scope;
            return scope;
        }
        __nestGlobalScope(node) {
            return this.__nestScope(new GlobalScope(this, node));
        }
        __nestBlockScope(node) {
            return this.__nestScope(new BlockScope(this, this.__currentScope, node));
        }
        __nestFunctionScope(node, isMethodDefinition) {
            return this.__nestScope(new FunctionScope(this, this.__currentScope, node, isMethodDefinition));
        }
        __nestForScope(node) {
            return this.__nestScope(new ForScope(this, this.__currentScope, node));
        }
        __nestCatchScope(node) {
            return this.__nestScope(new CatchScope(this, this.__currentScope, node));
        }
        __nestWithScope(node) {
            return this.__nestScope(new WithScope(this, this.__currentScope, node));
        }
        __nestClassScope(node) {
            return this.__nestScope(new ClassScope(this, this.__currentScope, node));
        }
        __nestSwitchScope(node) {
            return this.__nestScope(new SwitchScope(this, this.__currentScope, node));
        }
        __nestModuleScope(node) {
            return this.__nestScope(new ModuleScope(this, this.__currentScope, node));
        }
        __nestFunctionExpressionNameScope(node) {
            return this.__nestScope(new FunctionExpressionNameScope(this, this.__currentScope, node));
        }
        __isES6() {
            return this.__options.ecmaVersion >= 6;
        }
    }
    module.exports = ScopeManager;
},
/* eslint-scope/lib/scope.js */ function _(require, module, exports) {
    /* eslint-disable no-underscore-dangle */
    /* eslint-disable no-undefined */
    const Syntax = require(90) /* estraverse */.Syntax;
    const Reference = require(92) /* ./reference */;
    const Variable = require(93) /* ./variable */;
    const Definition = require(94) /* ./definition */.Definition;
    const assert = require("assert");
    /**
     * Test if scope is struct
     * @param {Scope} scope - scope
     * @param {Block} block - block
     * @param {boolean} isMethodDefinition - is method definiton
     * @param {boolean} useDirective - use directive
     * @returns {boolean} is strict scope
     */
    function isStrictScope(scope, block, isMethodDefinition, useDirective) {
        let body;
        // When upper scope is exists and strict, inner scope is also strict.
        if (scope.upper && scope.upper.isStrict) {
            return true;
        }
        if (isMethodDefinition) {
            return true;
        }
        if (scope.type === "class" || scope.type === "module") {
            return true;
        }
        if (scope.type === "block" || scope.type === "switch") {
            return false;
        }
        if (scope.type === "function") {
            if (block.type === Syntax.ArrowFunctionExpression && block.body.type !== Syntax.BlockStatement) {
                return false;
            }
            if (block.type === Syntax.Program) {
                body = block;
            }
            else {
                body = block.body;
            }
            if (!body) {
                return false;
            }
        }
        else if (scope.type === "global") {
            body = block;
        }
        else {
            return false;
        }
        // Search 'use strict' directive.
        if (useDirective) {
            for (let i = 0, iz = body.body.length; i < iz; ++i) {
                const stmt = body.body[i];
                if (stmt.type !== Syntax.DirectiveStatement) {
                    break;
                }
                if (stmt.raw === "\"use strict\"" || stmt.raw === "'use strict'") {
                    return true;
                }
            }
        }
        else {
            for (let i = 0, iz = body.body.length; i < iz; ++i) {
                const stmt = body.body[i];
                if (stmt.type !== Syntax.ExpressionStatement) {
                    break;
                }
                const expr = stmt.expression;
                if (expr.type !== Syntax.Literal || typeof expr.value !== "string") {
                    break;
                }
                if (expr.raw !== null && expr.raw !== undefined) {
                    if (expr.raw === "\"use strict\"" || expr.raw === "'use strict'") {
                        return true;
                    }
                }
                else {
                    if (expr.value === "use strict") {
                        return true;
                    }
                }
            }
        }
        return false;
    }
    /**
     * Register scope
     * @param {ScopeManager} scopeManager - scope manager
     * @param {Scope} scope - scope
     * @returns {void}
     */
    function registerScope(scopeManager, scope) {
        scopeManager.scopes.push(scope);
        const scopes = scopeManager.__nodeToScope.get(scope.block);
        if (scopes) {
            scopes.push(scope);
        }
        else {
            scopeManager.__nodeToScope.set(scope.block, [scope]);
        }
    }
    /**
     * Should be statically
     * @param {Object} def - def
     * @returns {boolean} should be statically
     */
    function shouldBeStatically(def) {
        return ((def.type === Variable.ClassName) ||
            (def.type === Variable.Variable && def.parent.kind !== "var"));
    }
    /**
     * @class Scope
     */
    class Scope {
        constructor(scopeManager, type, upperScope, block, isMethodDefinition) {
            /**
             * One of 'module', 'block', 'switch', 'function', 'catch', 'with', 'function', 'class', 'global'.
             * @member {String} Scope#type
             */
            this.type = type;
            /**
             * The scoped {@link Variable}s of this scope, as <code>{ Variable.name
             * : Variable }</code>.
             * @member {Map} Scope#set
             */
            this.set = new Map();
            /**
             * The tainted variables of this scope, as <code>{ Variable.name :
             * boolean }</code>.
             * @member {Map} Scope#taints */
            this.taints = new Map();
            /**
             * Generally, through the lexical scoping of JS you can always know
             * which variable an identifier in the source code refers to. There are
             * a few exceptions to this rule. With 'global' and 'with' scopes you
             * can only decide at runtime which variable a reference refers to.
             * Moreover, if 'eval()' is used in a scope, it might introduce new
             * bindings in this or its parent scopes.
             * All those scopes are considered 'dynamic'.
             * @member {boolean} Scope#dynamic
             */
            this.dynamic = this.type === "global" || this.type === "with";
            /**
             * A reference to the scope-defining syntax node.
             * @member {espree.Node} Scope#block
             */
            this.block = block;
            /**
             * The {@link Reference|references} that are not resolved with this scope.
             * @member {Reference[]} Scope#through
             */
            this.through = [];
            /**
             * The scoped {@link Variable}s of this scope. In the case of a
             * 'function' scope this includes the automatic argument <em>arguments</em> as
             * its first element, as well as all further formal arguments.
             * @member {Variable[]} Scope#variables
             */
            this.variables = [];
            /**
             * Any variable {@link Reference|reference} found in this scope. This
             * includes occurrences of local variables as well as variables from
             * parent scopes (including the global scope). For local variables
             * this also includes defining occurrences (like in a 'var' statement).
             * In a 'function' scope this does not include the occurrences of the
             * formal parameter in the parameter list.
             * @member {Reference[]} Scope#references
             */
            this.references = [];
            /**
             * For 'global' and 'function' scopes, this is a self-reference. For
             * other scope types this is the <em>variableScope</em> value of the
             * parent scope.
             * @member {Scope} Scope#variableScope
             */
            this.variableScope =
                (this.type === "global" || this.type === "function" || this.type === "module") ? this : upperScope.variableScope;
            /**
             * Whether this scope is created by a FunctionExpression.
             * @member {boolean} Scope#functionExpressionScope
             */
            this.functionExpressionScope = false;
            /**
             * Whether this is a scope that contains an 'eval()' invocation.
             * @member {boolean} Scope#directCallToEvalScope
             */
            this.directCallToEvalScope = false;
            /**
             * @member {boolean} Scope#thisFound
             */
            this.thisFound = false;
            this.__left = [];
            /**
             * Reference to the parent {@link Scope|scope}.
             * @member {Scope} Scope#upper
             */
            this.upper = upperScope;
            /**
             * Whether 'use strict' is in effect in this scope.
             * @member {boolean} Scope#isStrict
             */
            this.isStrict = isStrictScope(this, block, isMethodDefinition, scopeManager.__useDirective());
            /**
             * List of nested {@link Scope}s.
             * @member {Scope[]} Scope#childScopes
             */
            this.childScopes = [];
            if (this.upper) {
                this.upper.childScopes.push(this);
            }
            this.__declaredVariables = scopeManager.__declaredVariables;
            registerScope(scopeManager, this);
        }
        __shouldStaticallyClose(scopeManager) {
            return (!this.dynamic || scopeManager.__isOptimistic());
        }
        __shouldStaticallyCloseForGlobal(ref) {
            // On global scope, let/const/class declarations should be resolved statically.
            const name = ref.identifier.name;
            if (!this.set.has(name)) {
                return false;
            }
            const variable = this.set.get(name);
            const defs = variable.defs;
            return defs.length > 0 && defs.every(shouldBeStatically);
        }
        __staticCloseRef(ref) {
            if (!this.__resolve(ref)) {
                this.__delegateToUpperScope(ref);
            }
        }
        __dynamicCloseRef(ref) {
            // notify all names are through to global
            let current = this;
            do {
                current.through.push(ref);
                current = current.upper;
            } while (current);
        }
        __globalCloseRef(ref) {
            // let/const/class declarations should be resolved statically.
            // others should be resolved dynamically.
            if (this.__shouldStaticallyCloseForGlobal(ref)) {
                this.__staticCloseRef(ref);
            }
            else {
                this.__dynamicCloseRef(ref);
            }
        }
        __close(scopeManager) {
            let closeRef;
            if (this.__shouldStaticallyClose(scopeManager)) {
                closeRef = this.__staticCloseRef;
            }
            else if (this.type !== "global") {
                closeRef = this.__dynamicCloseRef;
            }
            else {
                closeRef = this.__globalCloseRef;
            }
            // Try Resolving all references in this scope.
            for (let i = 0, iz = this.__left.length; i < iz; ++i) {
                const ref = this.__left[i];
                closeRef.call(this, ref);
            }
            this.__left = null;
            return this.upper;
        }
        // To override by function scopes.
        // References in default parameters isn't resolved to variables which are in their function body.
        __isValidResolution(ref, variable) {
            return true;
        }
        __resolve(ref) {
            const name = ref.identifier.name;
            if (!this.set.has(name)) {
                return false;
            }
            const variable = this.set.get(name);
            if (!this.__isValidResolution(ref, variable)) {
                return false;
            }
            variable.references.push(ref);
            variable.stack = variable.stack && ref.from.variableScope === this.variableScope;
            if (ref.tainted) {
                variable.tainted = true;
                this.taints.set(variable.name, true);
            }
            ref.resolved = variable;
            return true;
        }
        __delegateToUpperScope(ref) {
            if (this.upper) {
                this.upper.__left.push(ref);
            }
            this.through.push(ref);
        }
        __addDeclaredVariablesOfNode(variable, node) {
            if (node === null || node === undefined) {
                return;
            }
            let variables = this.__declaredVariables.get(node);
            if (variables === null || variables === undefined) {
                variables = [];
                this.__declaredVariables.set(node, variables);
            }
            if (variables.indexOf(variable) === -1) {
                variables.push(variable);
            }
        }
        __defineGeneric(name, set, variables, node, def) {
            let variable;
            variable = set.get(name);
            if (!variable) {
                variable = new Variable(name, this);
                set.set(name, variable);
                variables.push(variable);
            }
            if (def) {
                variable.defs.push(def);
                this.__addDeclaredVariablesOfNode(variable, def.node);
                this.__addDeclaredVariablesOfNode(variable, def.parent);
            }
            if (node) {
                variable.identifiers.push(node);
            }
        }
        __define(node, def) {
            if (node && node.type === Syntax.Identifier) {
                this.__defineGeneric(node.name, this.set, this.variables, node, def);
            }
        }
        __referencing(node, assign, writeExpr, maybeImplicitGlobal, partial, init) {
            // because Array element may be null
            if (!node || node.type !== Syntax.Identifier) {
                return;
            }
            // Specially handle like `this`.
            if (node.name === "super") {
                return;
            }
            const ref = new Reference(node, this, assign || Reference.READ, writeExpr, maybeImplicitGlobal, !!partial, !!init);
            this.references.push(ref);
            this.__left.push(ref);
        }
        __detectEval() {
            let current = this;
            this.directCallToEvalScope = true;
            do {
                current.dynamic = true;
                current = current.upper;
            } while (current);
        }
        __detectThis() {
            this.thisFound = true;
        }
        __isClosed() {
            return this.__left === null;
        }
        /**
         * returns resolved {Reference}
         * @method Scope#resolve
         * @param {Espree.Identifier} ident - identifier to be resolved.
         * @returns {Reference} reference
         */
        resolve(ident) {
            let ref, i, iz;
            assert(this.__isClosed(), "Scope should be closed.");
            assert(ident.type === Syntax.Identifier, "Target should be identifier.");
            for (i = 0, iz = this.references.length; i < iz; ++i) {
                ref = this.references[i];
                if (ref.identifier === ident) {
                    return ref;
                }
            }
            return null;
        }
        /**
         * returns this scope is static
         * @method Scope#isStatic
         * @returns {boolean} static
         */
        isStatic() {
            return !this.dynamic;
        }
        /**
         * returns this scope has materialized arguments
         * @method Scope#isArgumentsMaterialized
         * @returns {boolean} arguemnts materialized
         */
        isArgumentsMaterialized() {
            return true;
        }
        /**
         * returns this scope has materialized `this` reference
         * @method Scope#isThisMaterialized
         * @returns {boolean} this materialized
         */
        isThisMaterialized() {
            return true;
        }
        isUsedName(name) {
            if (this.set.has(name)) {
                return true;
            }
            for (let i = 0, iz = this.through.length; i < iz; ++i) {
                if (this.through[i].identifier.name === name) {
                    return true;
                }
            }
            return false;
        }
    }
    class GlobalScope extends Scope {
        constructor(scopeManager, block) {
            super(scopeManager, "global", null, block, false);
            this.implicit = {
                set: new Map(),
                variables: [],
                /**
                * List of {@link Reference}s that are left to be resolved (i.e. which
                * need to be linked to the variable they refer to).
                * @member {Reference[]} Scope#implicit#left
                */
                left: []
            };
        }
        __close(scopeManager) {
            const implicit = [];
            for (let i = 0, iz = this.__left.length; i < iz; ++i) {
                const ref = this.__left[i];
                if (ref.__maybeImplicitGlobal && !this.set.has(ref.identifier.name)) {
                    implicit.push(ref.__maybeImplicitGlobal);
                }
            }
            // create an implicit global variable from assignment expression
            for (let i = 0, iz = implicit.length; i < iz; ++i) {
                const info = implicit[i];
                this.__defineImplicit(info.pattern, new Definition(Variable.ImplicitGlobalVariable, info.pattern, info.node, null, null, null));
            }
            this.implicit.left = this.__left;
            return super.__close(scopeManager);
        }
        __defineImplicit(node, def) {
            if (node && node.type === Syntax.Identifier) {
                this.__defineGeneric(node.name, this.implicit.set, this.implicit.variables, node, def);
            }
        }
    }
    class ModuleScope extends Scope {
        constructor(scopeManager, upperScope, block) {
            super(scopeManager, "module", upperScope, block, false);
        }
    }
    class FunctionExpressionNameScope extends Scope {
        constructor(scopeManager, upperScope, block) {
            super(scopeManager, "function-expression-name", upperScope, block, false);
            this.__define(block.id, new Definition(Variable.FunctionName, block.id, block, null, null, null));
            this.functionExpressionScope = true;
        }
    }
    class CatchScope extends Scope {
        constructor(scopeManager, upperScope, block) {
            super(scopeManager, "catch", upperScope, block, false);
        }
    }
    class WithScope extends Scope {
        constructor(scopeManager, upperScope, block) {
            super(scopeManager, "with", upperScope, block, false);
        }
        __close(scopeManager) {
            if (this.__shouldStaticallyClose(scopeManager)) {
                return super.__close(scopeManager);
            }
            for (let i = 0, iz = this.__left.length; i < iz; ++i) {
                const ref = this.__left[i];
                ref.tainted = true;
                this.__delegateToUpperScope(ref);
            }
            this.__left = null;
            return this.upper;
        }
    }
    class BlockScope extends Scope {
        constructor(scopeManager, upperScope, block) {
            super(scopeManager, "block", upperScope, block, false);
        }
    }
    class SwitchScope extends Scope {
        constructor(scopeManager, upperScope, block) {
            super(scopeManager, "switch", upperScope, block, false);
        }
    }
    class FunctionScope extends Scope {
        constructor(scopeManager, upperScope, block, isMethodDefinition) {
            super(scopeManager, "function", upperScope, block, isMethodDefinition);
            // section 9.2.13, FunctionDeclarationInstantiation.
            // NOTE Arrow functions never have an arguments objects.
            if (this.block.type !== Syntax.ArrowFunctionExpression) {
                this.__defineArguments();
            }
        }
        isArgumentsMaterialized() {
            // TODO(Constellation)
            // We can more aggressive on this condition like this.
            //
            // function t() {
            //     // arguments of t is always hidden.
            //     function arguments() {
            //     }
            // }
            if (this.block.type === Syntax.ArrowFunctionExpression) {
                return false;
            }
            if (!this.isStatic()) {
                return true;
            }
            const variable = this.set.get("arguments");
            assert(variable, "Always have arguments variable.");
            return variable.tainted || variable.references.length !== 0;
        }
        isThisMaterialized() {
            if (!this.isStatic()) {
                return true;
            }
            return this.thisFound;
        }
        __defineArguments() {
            this.__defineGeneric("arguments", this.set, this.variables, null, null);
            this.taints.set("arguments", true);
        }
        // References in default parameters isn't resolved to variables which are in their function body.
        //     const x = 1
        //     function f(a = x) { // This `x` is resolved to the `x` in the outer scope.
        //         const x = 2
        //         console.log(a)
        //     }
        __isValidResolution(ref, variable) {
            // If `options.nodejsScope` is true, `this.block` becomes a Program node.
            if (this.block.type === "Program") {
                return true;
            }
            const bodyStart = this.block.body.range[0];
            // It's invalid resolution in the following case:
            return !(variable.scope === this &&
                ref.identifier.range[0] < bodyStart && // the reference is in the parameter part.
                variable.defs.every(d => d.name.range[0] >= bodyStart) // the variable is in the body.
            );
        }
    }
    class ForScope extends Scope {
        constructor(scopeManager, upperScope, block) {
            super(scopeManager, "for", upperScope, block, false);
        }
    }
    class ClassScope extends Scope {
        constructor(scopeManager, upperScope, block) {
            super(scopeManager, "class", upperScope, block, false);
        }
    }
    module.exports = {
        Scope,
        GlobalScope,
        ModuleScope,
        FunctionExpressionNameScope,
        CatchScope,
        WithScope,
        BlockScope,
        SwitchScope,
        FunctionScope,
        ForScope,
        ClassScope
    };
},
/* estraverse/estraverse.js */ function _(require, module, exports) {
    /*
      Copyright (C) 2012-2013 Yusuke Suzuki <utatane.tea@gmail.com>
      Copyright (C) 2012 Ariya Hidayat <ariya.hidayat@gmail.com>
    
      Redistribution and use in source and binary forms, with or without
      modification, are permitted provided that the following conditions are met:
    
        * Redistributions of source code must retain the above copyright
          notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
          notice, this list of conditions and the following disclaimer in the
          documentation and/or other materials provided with the distribution.
    
      THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
      AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
      IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
      ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
      DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
      (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
      LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
      ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
      (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
      THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    */
    /*jslint vars:false, bitwise:true*/
    /*jshint indent:4*/
    /*global exports:true*/
    (function clone(exports) {
        'use strict';
        var Syntax, VisitorOption, VisitorKeys, BREAK, SKIP, REMOVE;
        function deepCopy(obj) {
            var ret = {}, key, val;
            for (key in obj) {
                if (obj.hasOwnProperty(key)) {
                    val = obj[key];
                    if (typeof val === 'object' && val !== null) {
                        ret[key] = deepCopy(val);
                    }
                    else {
                        ret[key] = val;
                    }
                }
            }
            return ret;
        }
        // based on LLVM libc++ upper_bound / lower_bound
        // MIT License
        function upperBound(array, func) {
            var diff, len, i, current;
            len = array.length;
            i = 0;
            while (len) {
                diff = len >>> 1;
                current = i + diff;
                if (func(array[current])) {
                    len = diff;
                }
                else {
                    i = current + 1;
                    len -= diff + 1;
                }
            }
            return i;
        }
        Syntax = {
            AssignmentExpression: 'AssignmentExpression',
            AssignmentPattern: 'AssignmentPattern',
            ArrayExpression: 'ArrayExpression',
            ArrayPattern: 'ArrayPattern',
            ArrowFunctionExpression: 'ArrowFunctionExpression',
            AwaitExpression: 'AwaitExpression',
            BlockStatement: 'BlockStatement',
            BinaryExpression: 'BinaryExpression',
            BreakStatement: 'BreakStatement',
            CallExpression: 'CallExpression',
            CatchClause: 'CatchClause',
            ClassBody: 'ClassBody',
            ClassDeclaration: 'ClassDeclaration',
            ClassExpression: 'ClassExpression',
            ComprehensionBlock: 'ComprehensionBlock',
            ComprehensionExpression: 'ComprehensionExpression',
            ConditionalExpression: 'ConditionalExpression',
            ContinueStatement: 'ContinueStatement',
            DebuggerStatement: 'DebuggerStatement',
            DirectiveStatement: 'DirectiveStatement',
            DoWhileStatement: 'DoWhileStatement',
            EmptyStatement: 'EmptyStatement',
            ExportAllDeclaration: 'ExportAllDeclaration',
            ExportDefaultDeclaration: 'ExportDefaultDeclaration',
            ExportNamedDeclaration: 'ExportNamedDeclaration',
            ExportSpecifier: 'ExportSpecifier',
            ExpressionStatement: 'ExpressionStatement',
            ForStatement: 'ForStatement',
            ForInStatement: 'ForInStatement',
            ForOfStatement: 'ForOfStatement',
            FunctionDeclaration: 'FunctionDeclaration',
            FunctionExpression: 'FunctionExpression',
            GeneratorExpression: 'GeneratorExpression',
            Identifier: 'Identifier',
            IfStatement: 'IfStatement',
            ImportExpression: 'ImportExpression',
            ImportDeclaration: 'ImportDeclaration',
            ImportDefaultSpecifier: 'ImportDefaultSpecifier',
            ImportNamespaceSpecifier: 'ImportNamespaceSpecifier',
            ImportSpecifier: 'ImportSpecifier',
            Literal: 'Literal',
            LabeledStatement: 'LabeledStatement',
            LogicalExpression: 'LogicalExpression',
            MemberExpression: 'MemberExpression',
            MetaProperty: 'MetaProperty',
            MethodDefinition: 'MethodDefinition',
            ModuleSpecifier: 'ModuleSpecifier',
            NewExpression: 'NewExpression',
            ObjectExpression: 'ObjectExpression',
            ObjectPattern: 'ObjectPattern',
            Program: 'Program',
            Property: 'Property',
            RestElement: 'RestElement',
            ReturnStatement: 'ReturnStatement',
            SequenceExpression: 'SequenceExpression',
            SpreadElement: 'SpreadElement',
            Super: 'Super',
            SwitchStatement: 'SwitchStatement',
            SwitchCase: 'SwitchCase',
            TaggedTemplateExpression: 'TaggedTemplateExpression',
            TemplateElement: 'TemplateElement',
            TemplateLiteral: 'TemplateLiteral',
            ThisExpression: 'ThisExpression',
            ThrowStatement: 'ThrowStatement',
            TryStatement: 'TryStatement',
            UnaryExpression: 'UnaryExpression',
            UpdateExpression: 'UpdateExpression',
            VariableDeclaration: 'VariableDeclaration',
            VariableDeclarator: 'VariableDeclarator',
            WhileStatement: 'WhileStatement',
            WithStatement: 'WithStatement',
            YieldExpression: 'YieldExpression'
        };
        VisitorKeys = {
            AssignmentExpression: ['left', 'right'],
            AssignmentPattern: ['left', 'right'],
            ArrayExpression: ['elements'],
            ArrayPattern: ['elements'],
            ArrowFunctionExpression: ['params', 'body'],
            AwaitExpression: ['argument'],
            BlockStatement: ['body'],
            BinaryExpression: ['left', 'right'],
            BreakStatement: ['label'],
            CallExpression: ['callee', 'arguments'],
            CatchClause: ['param', 'body'],
            ClassBody: ['body'],
            ClassDeclaration: ['id', 'superClass', 'body'],
            ClassExpression: ['id', 'superClass', 'body'],
            ComprehensionBlock: ['left', 'right'],
            ComprehensionExpression: ['blocks', 'filter', 'body'],
            ConditionalExpression: ['test', 'consequent', 'alternate'],
            ContinueStatement: ['label'],
            DebuggerStatement: [],
            DirectiveStatement: [],
            DoWhileStatement: ['body', 'test'],
            EmptyStatement: [],
            ExportAllDeclaration: ['source'],
            ExportDefaultDeclaration: ['declaration'],
            ExportNamedDeclaration: ['declaration', 'specifiers', 'source'],
            ExportSpecifier: ['exported', 'local'],
            ExpressionStatement: ['expression'],
            ForStatement: ['init', 'test', 'update', 'body'],
            ForInStatement: ['left', 'right', 'body'],
            ForOfStatement: ['left', 'right', 'body'],
            FunctionDeclaration: ['id', 'params', 'body'],
            FunctionExpression: ['id', 'params', 'body'],
            GeneratorExpression: ['blocks', 'filter', 'body'],
            Identifier: [],
            IfStatement: ['test', 'consequent', 'alternate'],
            ImportExpression: ['source'],
            ImportDeclaration: ['specifiers', 'source'],
            ImportDefaultSpecifier: ['local'],
            ImportNamespaceSpecifier: ['local'],
            ImportSpecifier: ['imported', 'local'],
            Literal: [],
            LabeledStatement: ['label', 'body'],
            LogicalExpression: ['left', 'right'],
            MemberExpression: ['object', 'property'],
            MetaProperty: ['meta', 'property'],
            MethodDefinition: ['key', 'value'],
            ModuleSpecifier: [],
            NewExpression: ['callee', 'arguments'],
            ObjectExpression: ['properties'],
            ObjectPattern: ['properties'],
            Program: ['body'],
            Property: ['key', 'value'],
            RestElement: ['argument'],
            ReturnStatement: ['argument'],
            SequenceExpression: ['expressions'],
            SpreadElement: ['argument'],
            Super: [],
            SwitchStatement: ['discriminant', 'cases'],
            SwitchCase: ['test', 'consequent'],
            TaggedTemplateExpression: ['tag', 'quasi'],
            TemplateElement: [],
            TemplateLiteral: ['quasis', 'expressions'],
            ThisExpression: [],
            ThrowStatement: ['argument'],
            TryStatement: ['block', 'handler', 'finalizer'],
            UnaryExpression: ['argument'],
            UpdateExpression: ['argument'],
            VariableDeclaration: ['declarations'],
            VariableDeclarator: ['id', 'init'],
            WhileStatement: ['test', 'body'],
            WithStatement: ['object', 'body'],
            YieldExpression: ['argument']
        };
        // unique id
        BREAK = {};
        SKIP = {};
        REMOVE = {};
        VisitorOption = {
            Break: BREAK,
            Skip: SKIP,
            Remove: REMOVE
        };
        function Reference(parent, key) {
            this.parent = parent;
            this.key = key;
        }
        Reference.prototype.replace = function replace(node) {
            this.parent[this.key] = node;
        };
        Reference.prototype.remove = function remove() {
            if (Array.isArray(this.parent)) {
                this.parent.splice(this.key, 1);
                return true;
            }
            else {
                this.replace(null);
                return false;
            }
        };
        function Element(node, path, wrap, ref) {
            this.node = node;
            this.path = path;
            this.wrap = wrap;
            this.ref = ref;
        }
        function Controller() { }
        // API:
        // return property path array from root to current node
        Controller.prototype.path = function path() {
            var i, iz, j, jz, result, element;
            function addToPath(result, path) {
                if (Array.isArray(path)) {
                    for (j = 0, jz = path.length; j < jz; ++j) {
                        result.push(path[j]);
                    }
                }
                else {
                    result.push(path);
                }
            }
            // root node
            if (!this.__current.path) {
                return null;
            }
            // first node is sentinel, second node is root element
            result = [];
            for (i = 2, iz = this.__leavelist.length; i < iz; ++i) {
                element = this.__leavelist[i];
                addToPath(result, element.path);
            }
            addToPath(result, this.__current.path);
            return result;
        };
        // API:
        // return type of current node
        Controller.prototype.type = function () {
            var node = this.current();
            return node.type || this.__current.wrap;
        };
        // API:
        // return array of parent elements
        Controller.prototype.parents = function parents() {
            var i, iz, result;
            // first node is sentinel
            result = [];
            for (i = 1, iz = this.__leavelist.length; i < iz; ++i) {
                result.push(this.__leavelist[i].node);
            }
            return result;
        };
        // API:
        // return current node
        Controller.prototype.current = function current() {
            return this.__current.node;
        };
        Controller.prototype.__execute = function __execute(callback, element) {
            var previous, result;
            result = undefined;
            previous = this.__current;
            this.__current = element;
            this.__state = null;
            if (callback) {
                result = callback.call(this, element.node, this.__leavelist[this.__leavelist.length - 1].node);
            }
            this.__current = previous;
            return result;
        };
        // API:
        // notify control skip / break
        Controller.prototype.notify = function notify(flag) {
            this.__state = flag;
        };
        // API:
        // skip child nodes of current node
        Controller.prototype.skip = function () {
            this.notify(SKIP);
        };
        // API:
        // break traversals
        Controller.prototype['break'] = function () {
            this.notify(BREAK);
        };
        // API:
        // remove node
        Controller.prototype.remove = function () {
            this.notify(REMOVE);
        };
        Controller.prototype.__initialize = function (root, visitor) {
            this.visitor = visitor;
            this.root = root;
            this.__worklist = [];
            this.__leavelist = [];
            this.__current = null;
            this.__state = null;
            this.__fallback = null;
            if (visitor.fallback === 'iteration') {
                this.__fallback = Object.keys;
            }
            else if (typeof visitor.fallback === 'function') {
                this.__fallback = visitor.fallback;
            }
            this.__keys = VisitorKeys;
            if (visitor.keys) {
                this.__keys = Object.assign(Object.create(this.__keys), visitor.keys);
            }
        };
        function isNode(node) {
            if (node == null) {
                return false;
            }
            return typeof node === 'object' && typeof node.type === 'string';
        }
        function isProperty(nodeType, key) {
            return (nodeType === Syntax.ObjectExpression || nodeType === Syntax.ObjectPattern) && 'properties' === key;
        }
        Controller.prototype.traverse = function traverse(root, visitor) {
            var worklist, leavelist, element, node, nodeType, ret, key, current, current2, candidates, candidate, sentinel;
            this.__initialize(root, visitor);
            sentinel = {};
            // reference
            worklist = this.__worklist;
            leavelist = this.__leavelist;
            // initialize
            worklist.push(new Element(root, null, null, null));
            leavelist.push(new Element(null, null, null, null));
            while (worklist.length) {
                element = worklist.pop();
                if (element === sentinel) {
                    element = leavelist.pop();
                    ret = this.__execute(visitor.leave, element);
                    if (this.__state === BREAK || ret === BREAK) {
                        return;
                    }
                    continue;
                }
                if (element.node) {
                    ret = this.__execute(visitor.enter, element);
                    if (this.__state === BREAK || ret === BREAK) {
                        return;
                    }
                    worklist.push(sentinel);
                    leavelist.push(element);
                    if (this.__state === SKIP || ret === SKIP) {
                        continue;
                    }
                    node = element.node;
                    nodeType = node.type || element.wrap;
                    candidates = this.__keys[nodeType];
                    if (!candidates) {
                        if (this.__fallback) {
                            candidates = this.__fallback(node);
                        }
                        else {
                            throw new Error('Unknown node type ' + nodeType + '.');
                        }
                    }
                    current = candidates.length;
                    while ((current -= 1) >= 0) {
                        key = candidates[current];
                        candidate = node[key];
                        if (!candidate) {
                            continue;
                        }
                        if (Array.isArray(candidate)) {
                            current2 = candidate.length;
                            while ((current2 -= 1) >= 0) {
                                if (!candidate[current2]) {
                                    continue;
                                }
                                if (isProperty(nodeType, candidates[current])) {
                                    element = new Element(candidate[current2], [key, current2], 'Property', null);
                                }
                                else if (isNode(candidate[current2])) {
                                    element = new Element(candidate[current2], [key, current2], null, null);
                                }
                                else {
                                    continue;
                                }
                                worklist.push(element);
                            }
                        }
                        else if (isNode(candidate)) {
                            worklist.push(new Element(candidate, key, null, null));
                        }
                    }
                }
            }
        };
        Controller.prototype.replace = function replace(root, visitor) {
            var worklist, leavelist, node, nodeType, target, element, current, current2, candidates, candidate, sentinel, outer, key;
            function removeElem(element) {
                var i, key, nextElem, parent;
                if (element.ref.remove()) {
                    // When the reference is an element of an array.
                    key = element.ref.key;
                    parent = element.ref.parent;
                    // If removed from array, then decrease following items' keys.
                    i = worklist.length;
                    while (i--) {
                        nextElem = worklist[i];
                        if (nextElem.ref && nextElem.ref.parent === parent) {
                            if (nextElem.ref.key < key) {
                                break;
                            }
                            --nextElem.ref.key;
                        }
                    }
                }
            }
            this.__initialize(root, visitor);
            sentinel = {};
            // reference
            worklist = this.__worklist;
            leavelist = this.__leavelist;
            // initialize
            outer = {
                root: root
            };
            element = new Element(root, null, null, new Reference(outer, 'root'));
            worklist.push(element);
            leavelist.push(element);
            while (worklist.length) {
                element = worklist.pop();
                if (element === sentinel) {
                    element = leavelist.pop();
                    target = this.__execute(visitor.leave, element);
                    // node may be replaced with null,
                    // so distinguish between undefined and null in this place
                    if (target !== undefined && target !== BREAK && target !== SKIP && target !== REMOVE) {
                        // replace
                        element.ref.replace(target);
                    }
                    if (this.__state === REMOVE || target === REMOVE) {
                        removeElem(element);
                    }
                    if (this.__state === BREAK || target === BREAK) {
                        return outer.root;
                    }
                    continue;
                }
                target = this.__execute(visitor.enter, element);
                // node may be replaced with null,
                // so distinguish between undefined and null in this place
                if (target !== undefined && target !== BREAK && target !== SKIP && target !== REMOVE) {
                    // replace
                    element.ref.replace(target);
                    element.node = target;
                }
                if (this.__state === REMOVE || target === REMOVE) {
                    removeElem(element);
                    element.node = null;
                }
                if (this.__state === BREAK || target === BREAK) {
                    return outer.root;
                }
                // node may be null
                node = element.node;
                if (!node) {
                    continue;
                }
                worklist.push(sentinel);
                leavelist.push(element);
                if (this.__state === SKIP || target === SKIP) {
                    continue;
                }
                nodeType = node.type || element.wrap;
                candidates = this.__keys[nodeType];
                if (!candidates) {
                    if (this.__fallback) {
                        candidates = this.__fallback(node);
                    }
                    else {
                        throw new Error('Unknown node type ' + nodeType + '.');
                    }
                }
                current = candidates.length;
                while ((current -= 1) >= 0) {
                    key = candidates[current];
                    candidate = node[key];
                    if (!candidate) {
                        continue;
                    }
                    if (Array.isArray(candidate)) {
                        current2 = candidate.length;
                        while ((current2 -= 1) >= 0) {
                            if (!candidate[current2]) {
                                continue;
                            }
                            if (isProperty(nodeType, candidates[current])) {
                                element = new Element(candidate[current2], [key, current2], 'Property', new Reference(candidate, current2));
                            }
                            else if (isNode(candidate[current2])) {
                                element = new Element(candidate[current2], [key, current2], null, new Reference(candidate, current2));
                            }
                            else {
                                continue;
                            }
                            worklist.push(element);
                        }
                    }
                    else if (isNode(candidate)) {
                        worklist.push(new Element(candidate, key, null, new Reference(node, key)));
                    }
                }
            }
            return outer.root;
        };
        function traverse(root, visitor) {
            var controller = new Controller();
            return controller.traverse(root, visitor);
        }
        function replace(root, visitor) {
            var controller = new Controller();
            return controller.replace(root, visitor);
        }
        function extendCommentRange(comment, tokens) {
            var target;
            target = upperBound(tokens, function search(token) {
                return token.range[0] > comment.range[0];
            });
            comment.extendedRange = [comment.range[0], comment.range[1]];
            if (target !== tokens.length) {
                comment.extendedRange[1] = tokens[target].range[0];
            }
            target -= 1;
            if (target >= 0) {
                comment.extendedRange[0] = tokens[target].range[1];
            }
            return comment;
        }
        function attachComments(tree, providedComments, tokens) {
            // At first, we should calculate extended comment ranges.
            var comments = [], comment, len, i, cursor;
            if (!tree.range) {
                throw new Error('attachComments needs range information');
            }
            // tokens array is empty, we attach comments to tree as 'leadingComments'
            if (!tokens.length) {
                if (providedComments.length) {
                    for (i = 0, len = providedComments.length; i < len; i += 1) {
                        comment = deepCopy(providedComments[i]);
                        comment.extendedRange = [0, tree.range[0]];
                        comments.push(comment);
                    }
                    tree.leadingComments = comments;
                }
                return tree;
            }
            for (i = 0, len = providedComments.length; i < len; i += 1) {
                comments.push(extendCommentRange(deepCopy(providedComments[i]), tokens));
            }
            // This is based on John Freeman's implementation.
            cursor = 0;
            traverse(tree, {
                enter: function (node) {
                    var comment;
                    while (cursor < comments.length) {
                        comment = comments[cursor];
                        if (comment.extendedRange[1] > node.range[0]) {
                            break;
                        }
                        if (comment.extendedRange[1] === node.range[0]) {
                            if (!node.leadingComments) {
                                node.leadingComments = [];
                            }
                            node.leadingComments.push(comment);
                            comments.splice(cursor, 1);
                        }
                        else {
                            cursor += 1;
                        }
                    }
                    // already out of owned node
                    if (cursor === comments.length) {
                        return VisitorOption.Break;
                    }
                    if (comments[cursor].extendedRange[0] > node.range[1]) {
                        return VisitorOption.Skip;
                    }
                }
            });
            cursor = 0;
            traverse(tree, {
                leave: function (node) {
                    var comment;
                    while (cursor < comments.length) {
                        comment = comments[cursor];
                        if (node.range[1] < comment.extendedRange[0]) {
                            break;
                        }
                        if (node.range[1] === comment.extendedRange[0]) {
                            if (!node.trailingComments) {
                                node.trailingComments = [];
                            }
                            node.trailingComments.push(comment);
                            comments.splice(cursor, 1);
                        }
                        else {
                            cursor += 1;
                        }
                    }
                    // already out of owned node
                    if (cursor === comments.length) {
                        return VisitorOption.Break;
                    }
                    if (comments[cursor].extendedRange[0] > node.range[1]) {
                        return VisitorOption.Skip;
                    }
                }
            });
            return tree;
        }
        exports.version = require(91) /* ./package.json */.version;
        exports.Syntax = Syntax;
        exports.traverse = traverse;
        exports.replace = replace;
        exports.attachComments = attachComments;
        exports.VisitorKeys = VisitorKeys;
        exports.VisitorOption = VisitorOption;
        exports.Controller = Controller;
        exports.cloneEnvironment = function () { return clone({}); };
        return exports;
    }(exports));
},
/* estraverse/package.json */ function _(require, module, exports) {
    module.exports = {
        "_args": [
            [
                "estraverse@4.3.0",
                "/Users/bryan/work/bokeh/bokehjs"
            ]
        ],
        "_development": true,
        "_from": "estraverse@4.3.0",
        "_id": "estraverse@4.3.0",
        "_inBundle": false,
        "_integrity": "sha512-39nnKffWz8xN1BU/2c79n9nB9HDzo0niYUqx6xyqUnyoAnQyyWpOTdZEeiCch8BBu515t4wp9ZmgVfVhn9EBpw==",
        "_location": "/estraverse",
        "_phantomChildren": {},
        "_requested": {
            "type": "version",
            "registry": true,
            "raw": "estraverse@4.3.0",
            "name": "estraverse",
            "escapedName": "estraverse",
            "rawSpec": "4.3.0",
            "saveSpec": null,
            "fetchSpec": "4.3.0"
        },
        "_requiredBy": [
            "/eslint-scope",
            "/esquery",
            "/esrecurse"
        ],
        "_resolved": "https://registry.npmjs.org/estraverse/-/estraverse-4.3.0.tgz",
        "_spec": "4.3.0",
        "_where": "/Users/bryan/work/bokeh/bokehjs",
        "bugs": {
            "url": "https://github.com/estools/estraverse/issues"
        },
        "description": "ECMAScript JS AST traversal functions",
        "devDependencies": {
            "babel-preset-env": "^1.6.1",
            "babel-register": "^6.3.13",
            "chai": "^2.1.1",
            "espree": "^1.11.0",
            "gulp": "^3.8.10",
            "gulp-bump": "^0.2.2",
            "gulp-filter": "^2.0.0",
            "gulp-git": "^1.0.1",
            "gulp-tag-version": "^1.3.0",
            "jshint": "^2.5.6",
            "mocha": "^2.1.0"
        },
        "engines": {
            "node": ">=4.0"
        },
        "homepage": "https://github.com/estools/estraverse",
        "license": "BSD-2-Clause",
        "main": "estraverse.js",
        "maintainers": [
            {
                "name": "Yusuke Suzuki",
                "email": "utatane.tea@gmail.com",
                "url": "http://github.com/Constellation"
            }
        ],
        "name": "estraverse",
        "repository": {
            "type": "git",
            "url": "git+ssh://git@github.com/estools/estraverse.git"
        },
        "scripts": {
            "lint": "jshint estraverse.js",
            "test": "npm run-script lint && npm run-script unit-test",
            "unit-test": "mocha --compilers js:babel-register"
        },
        "version": "4.3.0"
    };
},
/* eslint-scope/lib/reference.js */ function _(require, module, exports) {
    const READ = 0x1;
    const WRITE = 0x2;
    const RW = READ | WRITE;
    /**
     * A Reference represents a single occurrence of an identifier in code.
     * @class Reference
     */
    class Reference {
        constructor(ident, scope, flag, writeExpr, maybeImplicitGlobal, partial, init) {
            /**
             * Identifier syntax node.
             * @member {espreeIdentifier} Reference#identifier
             */
            this.identifier = ident;
            /**
             * Reference to the enclosing Scope.
             * @member {Scope} Reference#from
             */
            this.from = scope;
            /**
             * Whether the reference comes from a dynamic scope (such as 'eval',
             * 'with', etc.), and may be trapped by dynamic scopes.
             * @member {boolean} Reference#tainted
             */
            this.tainted = false;
            /**
             * The variable this reference is resolved with.
             * @member {Variable} Reference#resolved
             */
            this.resolved = null;
            /**
             * The read-write mode of the reference. (Value is one of {@link
             * Reference.READ}, {@link Reference.RW}, {@link Reference.WRITE}).
             * @member {number} Reference#flag
             * @private
             */
            this.flag = flag;
            if (this.isWrite()) {
                /**
                 * If reference is writeable, this is the tree being written to it.
                 * @member {espreeNode} Reference#writeExpr
                 */
                this.writeExpr = writeExpr;
                /**
                 * Whether the Reference might refer to a partial value of writeExpr.
                 * @member {boolean} Reference#partial
                 */
                this.partial = partial;
                /**
                 * Whether the Reference is to write of initialization.
                 * @member {boolean} Reference#init
                 */
                this.init = init;
            }
            this.__maybeImplicitGlobal = maybeImplicitGlobal;
        }
        /**
         * Whether the reference is static.
         * @method Reference#isStatic
         * @returns {boolean} static
         */
        isStatic() {
            return !this.tainted && this.resolved && this.resolved.scope.isStatic();
        }
        /**
         * Whether the reference is writeable.
         * @method Reference#isWrite
         * @returns {boolean} write
         */
        isWrite() {
            return !!(this.flag & Reference.WRITE);
        }
        /**
         * Whether the reference is readable.
         * @method Reference#isRead
         * @returns {boolean} read
         */
        isRead() {
            return !!(this.flag & Reference.READ);
        }
        /**
         * Whether the reference is read-only.
         * @method Reference#isReadOnly
         * @returns {boolean} read only
         */
        isReadOnly() {
            return this.flag === Reference.READ;
        }
        /**
         * Whether the reference is write-only.
         * @method Reference#isWriteOnly
         * @returns {boolean} write only
         */
        isWriteOnly() {
            return this.flag === Reference.WRITE;
        }
        /**
         * Whether the reference is read-write.
         * @method Reference#isReadWrite
         * @returns {boolean} read write
         */
        isReadWrite() {
            return this.flag === Reference.RW;
        }
    }
    /**
     * @constant Reference.READ
     * @private
     */
    Reference.READ = READ;
    /**
     * @constant Reference.WRITE
     * @private
     */
    Reference.WRITE = WRITE;
    /**
     * @constant Reference.RW
     * @private
     */
    Reference.RW = RW;
    module.exports = Reference;
},
/* eslint-scope/lib/variable.js */ function _(require, module, exports) {
    /**
     * A Variable represents a locally scoped identifier. These include arguments to
     * functions.
     * @class Variable
     */
    class Variable {
        constructor(name, scope) {
            /**
             * The variable name, as given in the source code.
             * @member {String} Variable#name
             */
            this.name = name;
            /**
             * List of defining occurrences of this variable (like in 'var ...'
             * statements or as parameter), as AST nodes.
             * @member {espree.Identifier[]} Variable#identifiers
             */
            this.identifiers = [];
            /**
             * List of {@link Reference|references} of this variable (excluding parameter entries)
             * in its defining scope and all nested scopes. For defining
             * occurrences only see {@link Variable#defs}.
             * @member {Reference[]} Variable#references
             */
            this.references = [];
            /**
             * List of defining occurrences of this variable (like in 'var ...'
             * statements or as parameter), as custom objects.
             * @member {Definition[]} Variable#defs
             */
            this.defs = [];
            this.tainted = false;
            /**
             * Whether this is a stack variable.
             * @member {boolean} Variable#stack
             */
            this.stack = true;
            /**
             * Reference to the enclosing Scope.
             * @member {Scope} Variable#scope
             */
            this.scope = scope;
        }
    }
    Variable.CatchClause = "CatchClause";
    Variable.Parameter = "Parameter";
    Variable.FunctionName = "FunctionName";
    Variable.ClassName = "ClassName";
    Variable.Variable = "Variable";
    Variable.ImportBinding = "ImportBinding";
    Variable.ImplicitGlobalVariable = "ImplicitGlobalVariable";
    module.exports = Variable;
},
/* eslint-scope/lib/definition.js */ function _(require, module, exports) {
    const Variable = require(93) /* ./variable */;
    /**
     * @class Definition
     */
    class Definition {
        constructor(type, name, node, parent, index, kind) {
            /**
             * @member {String} Definition#type - type of the occurrence (e.g. "Parameter", "Variable", ...).
             */
            this.type = type;
            /**
             * @member {espree.Identifier} Definition#name - the identifier AST node of the occurrence.
             */
            this.name = name;
            /**
             * @member {espree.Node} Definition#node - the enclosing node of the identifier.
             */
            this.node = node;
            /**
             * @member {espree.Node?} Definition#parent - the enclosing statement node of the identifier.
             */
            this.parent = parent;
            /**
             * @member {Number?} Definition#index - the index in the declaration statement.
             */
            this.index = index;
            /**
             * @member {String?} Definition#kind - the kind of the declaration statement.
             */
            this.kind = kind;
        }
    }
    /**
     * @class ParameterDefinition
     */
    class ParameterDefinition extends Definition {
        constructor(name, node, index, rest) {
            super(Variable.Parameter, name, node, null, index, null);
            /**
             * Whether the parameter definition is a part of a rest parameter.
             * @member {boolean} ParameterDefinition#rest
             */
            this.rest = rest;
        }
    }
    module.exports = {
        ParameterDefinition,
        Definition
    };
},
/* eslint-scope/lib/referencer.js */ function _(require, module, exports) {
    /* eslint-disable no-underscore-dangle */
    /* eslint-disable no-undefined */
    const Syntax = require(90) /* estraverse */.Syntax;
    const esrecurse = require(96) /* esrecurse */;
    const Reference = require(92) /* ./reference */;
    const Variable = require(93) /* ./variable */;
    const PatternVisitor = require(98) /* ./pattern-visitor */;
    const definition = require(94) /* ./definition */;
    const assert = require("assert");
    const ParameterDefinition = definition.ParameterDefinition;
    const Definition = definition.Definition;
    /**
     * Traverse identifier in pattern
     * @param {Object} options - options
     * @param {pattern} rootPattern - root pattern
     * @param {Refencer} referencer - referencer
     * @param {callback} callback - callback
     * @returns {void}
     */
    function traverseIdentifierInPattern(options, rootPattern, referencer, callback) {
        // Call the callback at left hand identifier nodes, and Collect right hand nodes.
        const visitor = new PatternVisitor(options, rootPattern, callback);
        visitor.visit(rootPattern);
        // Process the right hand nodes recursively.
        if (referencer !== null && referencer !== undefined) {
            visitor.rightHandNodes.forEach(referencer.visit, referencer);
        }
    }
    // Importing ImportDeclaration.
    // http://people.mozilla.org/~jorendorff/es6-draft.html#sec-moduledeclarationinstantiation
    // https://github.com/estree/estree/blob/master/es6.md#importdeclaration
    // FIXME: Now, we don't create module environment, because the context is
    // implementation dependent.
    class Importer extends esrecurse.Visitor {
        constructor(declaration, referencer) {
            super(null, referencer.options);
            this.declaration = declaration;
            this.referencer = referencer;
        }
        visitImport(id, specifier) {
            this.referencer.visitPattern(id, pattern => {
                this.referencer.currentScope().__define(pattern, new Definition(Variable.ImportBinding, pattern, specifier, this.declaration, null, null));
            });
        }
        ImportNamespaceSpecifier(node) {
            const local = (node.local || node.id);
            if (local) {
                this.visitImport(local, node);
            }
        }
        ImportDefaultSpecifier(node) {
            const local = (node.local || node.id);
            this.visitImport(local, node);
        }
        ImportSpecifier(node) {
            const local = (node.local || node.id);
            if (node.name) {
                this.visitImport(node.name, node);
            }
            else {
                this.visitImport(local, node);
            }
        }
    }
    // Referencing variables and creating bindings.
    class Referencer extends esrecurse.Visitor {
        constructor(options, scopeManager) {
            super(null, options);
            this.options = options;
            this.scopeManager = scopeManager;
            this.parent = null;
            this.isInnerMethodDefinition = false;
        }
        currentScope() {
            return this.scopeManager.__currentScope;
        }
        close(node) {
            while (this.currentScope() && node === this.currentScope().block) {
                this.scopeManager.__currentScope = this.currentScope().__close(this.scopeManager);
            }
        }
        pushInnerMethodDefinition(isInnerMethodDefinition) {
            const previous = this.isInnerMethodDefinition;
            this.isInnerMethodDefinition = isInnerMethodDefinition;
            return previous;
        }
        popInnerMethodDefinition(isInnerMethodDefinition) {
            this.isInnerMethodDefinition = isInnerMethodDefinition;
        }
        referencingDefaultValue(pattern, assignments, maybeImplicitGlobal, init) {
            const scope = this.currentScope();
            assignments.forEach(assignment => {
                scope.__referencing(pattern, Reference.WRITE, assignment.right, maybeImplicitGlobal, pattern !== assignment.left, init);
            });
        }
        visitPattern(node, options, callback) {
            let visitPatternOptions = options;
            let visitPatternCallback = callback;
            if (typeof options === "function") {
                visitPatternCallback = options;
                visitPatternOptions = { processRightHandNodes: false };
            }
            traverseIdentifierInPattern(this.options, node, visitPatternOptions.processRightHandNodes ? this : null, visitPatternCallback);
        }
        visitFunction(node) {
            let i, iz;
            // FunctionDeclaration name is defined in upper scope
            // NOTE: Not referring variableScope. It is intended.
            // Since
            //  in ES5, FunctionDeclaration should be in FunctionBody.
            //  in ES6, FunctionDeclaration should be block scoped.
            if (node.type === Syntax.FunctionDeclaration) {
                // id is defined in upper scope
                this.currentScope().__define(node.id, new Definition(Variable.FunctionName, node.id, node, null, null, null));
            }
            // FunctionExpression with name creates its special scope;
            // FunctionExpressionNameScope.
            if (node.type === Syntax.FunctionExpression && node.id) {
                this.scopeManager.__nestFunctionExpressionNameScope(node);
            }
            // Consider this function is in the MethodDefinition.
            this.scopeManager.__nestFunctionScope(node, this.isInnerMethodDefinition);
            const that = this;
            /**
             * Visit pattern callback
             * @param {pattern} pattern - pattern
             * @param {Object} info - info
             * @returns {void}
             */
            function visitPatternCallback(pattern, info) {
                that.currentScope().__define(pattern, new ParameterDefinition(pattern, node, i, info.rest));
                that.referencingDefaultValue(pattern, info.assignments, null, true);
            }
            // Process parameter declarations.
            for (i = 0, iz = node.params.length; i < iz; ++i) {
                this.visitPattern(node.params[i], { processRightHandNodes: true }, visitPatternCallback);
            }
            // if there's a rest argument, add that
            if (node.rest) {
                this.visitPattern({
                    type: "RestElement",
                    argument: node.rest
                }, pattern => {
                    this.currentScope().__define(pattern, new ParameterDefinition(pattern, node, node.params.length, true));
                });
            }
            // In TypeScript there are a number of function-like constructs which have no body,
            // so check it exists before traversing
            if (node.body) {
                // Skip BlockStatement to prevent creating BlockStatement scope.
                if (node.body.type === Syntax.BlockStatement) {
                    this.visitChildren(node.body);
                }
                else {
                    this.visit(node.body);
                }
            }
            this.close(node);
        }
        visitClass(node) {
            if (node.type === Syntax.ClassDeclaration) {
                this.currentScope().__define(node.id, new Definition(Variable.ClassName, node.id, node, null, null, null));
            }
            this.visit(node.superClass);
            this.scopeManager.__nestClassScope(node);
            if (node.id) {
                this.currentScope().__define(node.id, new Definition(Variable.ClassName, node.id, node));
            }
            this.visit(node.body);
            this.close(node);
        }
        visitProperty(node) {
            let previous;
            if (node.computed) {
                this.visit(node.key);
            }
            const isMethodDefinition = node.type === Syntax.MethodDefinition;
            if (isMethodDefinition) {
                previous = this.pushInnerMethodDefinition(true);
            }
            this.visit(node.value);
            if (isMethodDefinition) {
                this.popInnerMethodDefinition(previous);
            }
        }
        visitForIn(node) {
            if (node.left.type === Syntax.VariableDeclaration && node.left.kind !== "var") {
                this.scopeManager.__nestForScope(node);
            }
            if (node.left.type === Syntax.VariableDeclaration) {
                this.visit(node.left);
                this.visitPattern(node.left.declarations[0].id, pattern => {
                    this.currentScope().__referencing(pattern, Reference.WRITE, node.right, null, true, true);
                });
            }
            else {
                this.visitPattern(node.left, { processRightHandNodes: true }, (pattern, info) => {
                    let maybeImplicitGlobal = null;
                    if (!this.currentScope().isStrict) {
                        maybeImplicitGlobal = {
                            pattern,
                            node
                        };
                    }
                    this.referencingDefaultValue(pattern, info.assignments, maybeImplicitGlobal, false);
                    this.currentScope().__referencing(pattern, Reference.WRITE, node.right, maybeImplicitGlobal, true, false);
                });
            }
            this.visit(node.right);
            this.visit(node.body);
            this.close(node);
        }
        visitVariableDeclaration(variableTargetScope, type, node, index) {
            const decl = node.declarations[index];
            const init = decl.init;
            this.visitPattern(decl.id, { processRightHandNodes: true }, (pattern, info) => {
                variableTargetScope.__define(pattern, new Definition(type, pattern, decl, node, index, node.kind));
                this.referencingDefaultValue(pattern, info.assignments, null, true);
                if (init) {
                    this.currentScope().__referencing(pattern, Reference.WRITE, init, null, !info.topLevel, true);
                }
            });
        }
        AssignmentExpression(node) {
            if (PatternVisitor.isPattern(node.left)) {
                if (node.operator === "=") {
                    this.visitPattern(node.left, { processRightHandNodes: true }, (pattern, info) => {
                        let maybeImplicitGlobal = null;
                        if (!this.currentScope().isStrict) {
                            maybeImplicitGlobal = {
                                pattern,
                                node
                            };
                        }
                        this.referencingDefaultValue(pattern, info.assignments, maybeImplicitGlobal, false);
                        this.currentScope().__referencing(pattern, Reference.WRITE, node.right, maybeImplicitGlobal, !info.topLevel, false);
                    });
                }
                else {
                    this.currentScope().__referencing(node.left, Reference.RW, node.right);
                }
            }
            else {
                this.visit(node.left);
            }
            this.visit(node.right);
        }
        CatchClause(node) {
            this.scopeManager.__nestCatchScope(node);
            this.visitPattern(node.param, { processRightHandNodes: true }, (pattern, info) => {
                this.currentScope().__define(pattern, new Definition(Variable.CatchClause, node.param, node, null, null, null));
                this.referencingDefaultValue(pattern, info.assignments, null, true);
            });
            this.visit(node.body);
            this.close(node);
        }
        Program(node) {
            this.scopeManager.__nestGlobalScope(node);
            if (this.scopeManager.__isNodejsScope()) {
                // Force strictness of GlobalScope to false when using node.js scope.
                this.currentScope().isStrict = false;
                this.scopeManager.__nestFunctionScope(node, false);
            }
            if (this.scopeManager.__isES6() && this.scopeManager.isModule()) {
                this.scopeManager.__nestModuleScope(node);
            }
            if (this.scopeManager.isStrictModeSupported() && this.scopeManager.isImpliedStrict()) {
                this.currentScope().isStrict = true;
            }
            this.visitChildren(node);
            this.close(node);
        }
        Identifier(node) {
            this.currentScope().__referencing(node);
        }
        UpdateExpression(node) {
            if (PatternVisitor.isPattern(node.argument)) {
                this.currentScope().__referencing(node.argument, Reference.RW, null);
            }
            else {
                this.visitChildren(node);
            }
        }
        MemberExpression(node) {
            this.visit(node.object);
            if (node.computed) {
                this.visit(node.property);
            }
        }
        Property(node) {
            this.visitProperty(node);
        }
        MethodDefinition(node) {
            this.visitProperty(node);
        }
        BreakStatement() { } // eslint-disable-line class-methods-use-this
        ContinueStatement() { } // eslint-disable-line class-methods-use-this
        LabeledStatement(node) {
            this.visit(node.body);
        }
        ForStatement(node) {
            // Create ForStatement declaration.
            // NOTE: In ES6, ForStatement dynamically generates
            // per iteration environment. However, escope is
            // a static analyzer, we only generate one scope for ForStatement.
            if (node.init && node.init.type === Syntax.VariableDeclaration && node.init.kind !== "var") {
                this.scopeManager.__nestForScope(node);
            }
            this.visitChildren(node);
            this.close(node);
        }
        ClassExpression(node) {
            this.visitClass(node);
        }
        ClassDeclaration(node) {
            this.visitClass(node);
        }
        CallExpression(node) {
            // Check this is direct call to eval
            if (!this.scopeManager.__ignoreEval() && node.callee.type === Syntax.Identifier && node.callee.name === "eval") {
                // NOTE: This should be `variableScope`. Since direct eval call always creates Lexical environment and
                // let / const should be enclosed into it. Only VariableDeclaration affects on the caller's environment.
                this.currentScope().variableScope.__detectEval();
            }
            this.visitChildren(node);
        }
        BlockStatement(node) {
            if (this.scopeManager.__isES6()) {
                this.scopeManager.__nestBlockScope(node);
            }
            this.visitChildren(node);
            this.close(node);
        }
        ThisExpression() {
            this.currentScope().variableScope.__detectThis();
        }
        WithStatement(node) {
            this.visit(node.object);
            // Then nest scope for WithStatement.
            this.scopeManager.__nestWithScope(node);
            this.visit(node.body);
            this.close(node);
        }
        VariableDeclaration(node) {
            const variableTargetScope = (node.kind === "var") ? this.currentScope().variableScope : this.currentScope();
            for (let i = 0, iz = node.declarations.length; i < iz; ++i) {
                const decl = node.declarations[i];
                this.visitVariableDeclaration(variableTargetScope, Variable.Variable, node, i);
                if (decl.init) {
                    this.visit(decl.init);
                }
            }
        }
        // sec 13.11.8
        SwitchStatement(node) {
            this.visit(node.discriminant);
            if (this.scopeManager.__isES6()) {
                this.scopeManager.__nestSwitchScope(node);
            }
            for (let i = 0, iz = node.cases.length; i < iz; ++i) {
                this.visit(node.cases[i]);
            }
            this.close(node);
        }
        FunctionDeclaration(node) {
            this.visitFunction(node);
        }
        FunctionExpression(node) {
            this.visitFunction(node);
        }
        ForOfStatement(node) {
            this.visitForIn(node);
        }
        ForInStatement(node) {
            this.visitForIn(node);
        }
        ArrowFunctionExpression(node) {
            this.visitFunction(node);
        }
        ImportDeclaration(node) {
            assert(this.scopeManager.__isES6() && this.scopeManager.isModule(), "ImportDeclaration should appear when the mode is ES6 and in the module context.");
            const importer = new Importer(node, this);
            importer.visit(node);
        }
        visitExportDeclaration(node) {
            if (node.source) {
                return;
            }
            if (node.declaration) {
                this.visit(node.declaration);
                return;
            }
            this.visitChildren(node);
        }
        ExportDeclaration(node) {
            this.visitExportDeclaration(node);
        }
        ExportNamedDeclaration(node) {
            this.visitExportDeclaration(node);
        }
        ExportSpecifier(node) {
            const local = (node.id || node.local);
            this.visit(local);
        }
        MetaProperty() {
            // do nothing.
        }
    }
    module.exports = Referencer;
},
/* esrecurse/esrecurse.js */ function _(require, module, exports) {
    /*
      Copyright (C) 2014 Yusuke Suzuki <utatane.tea@gmail.com>
    
      Redistribution and use in source and binary forms, with or without
      modification, are permitted provided that the following conditions are met:
    
        * Redistributions of source code must retain the above copyright
          notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
          notice, this list of conditions and the following disclaimer in the
          documentation and/or other materials provided with the distribution.
    
      THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
      AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
      IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
      ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
      DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
      (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
      LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
      ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
      (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
      THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    */
    (function () {
        'use strict';
        var estraverse = require(90) /* estraverse */;
        function isNode(node) {
            if (node == null) {
                return false;
            }
            return typeof node === 'object' && typeof node.type === 'string';
        }
        function isProperty(nodeType, key) {
            return (nodeType === estraverse.Syntax.ObjectExpression || nodeType === estraverse.Syntax.ObjectPattern) && key === 'properties';
        }
        function Visitor(visitor, options) {
            options = options || {};
            this.__visitor = visitor || this;
            this.__childVisitorKeys = options.childVisitorKeys
                ? Object.assign({}, estraverse.VisitorKeys, options.childVisitorKeys)
                : estraverse.VisitorKeys;
            if (options.fallback === 'iteration') {
                this.__fallback = Object.keys;
            }
            else if (typeof options.fallback === 'function') {
                this.__fallback = options.fallback;
            }
        }
        /* Default method for visiting children.
         * When you need to call default visiting operation inside custom visiting
         * operation, you can use it with `this.visitChildren(node)`.
         */
        Visitor.prototype.visitChildren = function (node) {
            var type, children, i, iz, j, jz, child;
            if (node == null) {
                return;
            }
            type = node.type || estraverse.Syntax.Property;
            children = this.__childVisitorKeys[type];
            if (!children) {
                if (this.__fallback) {
                    children = this.__fallback(node);
                }
                else {
                    throw new Error('Unknown node type ' + type + '.');
                }
            }
            for (i = 0, iz = children.length; i < iz; ++i) {
                child = node[children[i]];
                if (child) {
                    if (Array.isArray(child)) {
                        for (j = 0, jz = child.length; j < jz; ++j) {
                            if (child[j]) {
                                if (isNode(child[j]) || isProperty(type, children[i])) {
                                    this.visit(child[j]);
                                }
                            }
                        }
                    }
                    else if (isNode(child)) {
                        this.visit(child);
                    }
                }
            }
        };
        /* Dispatching node. */
        Visitor.prototype.visit = function (node) {
            var type;
            if (node == null) {
                return;
            }
            type = node.type || estraverse.Syntax.Property;
            if (this.__visitor[type]) {
                this.__visitor[type].call(this, node);
                return;
            }
            this.visitChildren(node);
        };
        exports.version = require(97) /* ./package.json */.version;
        exports.Visitor = Visitor;
        exports.visit = function (node, visitor, options) {
            var v = new Visitor(visitor, options);
            v.visit(node);
        };
    }());
},
/* esrecurse/package.json */ function _(require, module, exports) {
    module.exports = {
        "_args": [
            [
                "esrecurse@4.2.1",
                "/Users/bryan/work/bokeh/bokehjs"
            ]
        ],
        "_development": true,
        "_from": "esrecurse@4.2.1",
        "_id": "esrecurse@4.2.1",
        "_inBundle": false,
        "_integrity": "sha512-64RBB++fIOAXPw3P9cy89qfMlvZEXZkqqJkjqqXIvzP5ezRZjW+lPWjw35UX/3EhUPFYbg5ER4JYgDw4007/DQ==",
        "_location": "/esrecurse",
        "_phantomChildren": {},
        "_requested": {
            "type": "version",
            "registry": true,
            "raw": "esrecurse@4.2.1",
            "name": "esrecurse",
            "escapedName": "esrecurse",
            "rawSpec": "4.2.1",
            "saveSpec": null,
            "fetchSpec": "4.2.1"
        },
        "_requiredBy": [
            "/eslint-scope"
        ],
        "_resolved": "https://registry.npmjs.org/esrecurse/-/esrecurse-4.2.1.tgz",
        "_spec": "4.2.1",
        "_where": "/Users/bryan/work/bokeh/bokehjs",
        "babel": {
            "presets": [
                "es2015"
            ]
        },
        "bugs": {
            "url": "https://github.com/estools/esrecurse/issues"
        },
        "dependencies": {
            "estraverse": "^4.1.0"
        },
        "description": "ECMAScript AST recursive visitor",
        "devDependencies": {
            "babel-cli": "^6.24.1",
            "babel-eslint": "^7.2.3",
            "babel-preset-es2015": "^6.24.1",
            "babel-register": "^6.24.1",
            "chai": "^4.0.2",
            "esprima": "^4.0.0",
            "gulp": "^3.9.0",
            "gulp-bump": "^2.7.0",
            "gulp-eslint": "^4.0.0",
            "gulp-filter": "^5.0.0",
            "gulp-git": "^2.4.1",
            "gulp-mocha": "^4.3.1",
            "gulp-tag-version": "^1.2.1",
            "jsdoc": "^3.3.0-alpha10",
            "minimist": "^1.1.0"
        },
        "engines": {
            "node": ">=4.0"
        },
        "homepage": "https://github.com/estools/esrecurse",
        "license": "BSD-2-Clause",
        "main": "esrecurse.js",
        "maintainers": [
            {
                "name": "Yusuke Suzuki",
                "email": "utatane.tea@gmail.com",
                "url": "https://github.com/Constellation"
            }
        ],
        "name": "esrecurse",
        "repository": {
            "type": "git",
            "url": "git+https://github.com/estools/esrecurse.git"
        },
        "scripts": {
            "lint": "gulp lint",
            "test": "gulp travis",
            "unit-test": "gulp test"
        },
        "version": "4.2.1"
    };
},
/* eslint-scope/lib/pattern-visitor.js */ function _(require, module, exports) {
    /* eslint-disable no-undefined */
    const Syntax = require(90) /* estraverse */.Syntax;
    const esrecurse = require(96) /* esrecurse */;
    /**
     * Get last array element
     * @param {array} xs - array
     * @returns {any} Last elment
     */
    function getLast(xs) {
        return xs[xs.length - 1] || null;
    }
    class PatternVisitor extends esrecurse.Visitor {
        static isPattern(node) {
            const nodeType = node.type;
            return (nodeType === Syntax.Identifier ||
                nodeType === Syntax.ObjectPattern ||
                nodeType === Syntax.ArrayPattern ||
                nodeType === Syntax.SpreadElement ||
                nodeType === Syntax.RestElement ||
                nodeType === Syntax.AssignmentPattern);
        }
        constructor(options, rootPattern, callback) {
            super(null, options);
            this.rootPattern = rootPattern;
            this.callback = callback;
            this.assignments = [];
            this.rightHandNodes = [];
            this.restElements = [];
        }
        Identifier(pattern) {
            const lastRestElement = getLast(this.restElements);
            this.callback(pattern, {
                topLevel: pattern === this.rootPattern,
                rest: lastRestElement !== null && lastRestElement !== undefined && lastRestElement.argument === pattern,
                assignments: this.assignments
            });
        }
        Property(property) {
            // Computed property's key is a right hand node.
            if (property.computed) {
                this.rightHandNodes.push(property.key);
            }
            // If it's shorthand, its key is same as its value.
            // If it's shorthand and has its default value, its key is same as its value.left (the value is AssignmentPattern).
            // If it's not shorthand, the name of new variable is its value's.
            this.visit(property.value);
        }
        ArrayPattern(pattern) {
            for (let i = 0, iz = pattern.elements.length; i < iz; ++i) {
                const element = pattern.elements[i];
                this.visit(element);
            }
        }
        AssignmentPattern(pattern) {
            this.assignments.push(pattern);
            this.visit(pattern.left);
            this.rightHandNodes.push(pattern.right);
            this.assignments.pop();
        }
        RestElement(pattern) {
            this.restElements.push(pattern);
            this.visit(pattern.argument);
            this.restElements.pop();
        }
        MemberExpression(node) {
            // Computed property's key is a right hand node.
            if (node.computed) {
                this.rightHandNodes.push(node.property);
            }
            // the object is only read, write to its property.
            this.rightHandNodes.push(node.object);
        }
        //
        // ForInStatement.left and AssignmentExpression.left are LeftHandSideExpression.
        // By spec, LeftHandSideExpression is Pattern or MemberExpression.
        //   (see also: https://github.com/estree/estree/pull/20#issuecomment-74584758)
        // But espree 2.0 parses to ArrayExpression, ObjectExpression, etc...
        //
        SpreadElement(node) {
            this.visit(node.argument);
        }
        ArrayExpression(node) {
            node.elements.forEach(this.visit, this);
        }
        AssignmentExpression(node) {
            this.assignments.push(node);
            this.visit(node.left);
            this.rightHandNodes.push(node.right);
            this.assignments.pop();
        }
        CallExpression(node) {
            // arguments are right hand nodes.
            node.arguments.forEach(a => {
                this.rightHandNodes.push(a);
            });
            this.visit(node.callee);
        }
    }
    module.exports = PatternVisitor;
},
/* eslint-scope/package.json */ function _(require, module, exports) {
    module.exports = {
        "_args": [
            [
                "eslint-scope@5.0.0",
                "/Users/bryan/work/bokeh/bokehjs"
            ]
        ],
        "_development": true,
        "_from": "eslint-scope@5.0.0",
        "_id": "eslint-scope@5.0.0",
        "_inBundle": false,
        "_integrity": "sha512-oYrhJW7S0bxAFDvWqzvMPRm6pcgcnWc4QnofCAqRTRfQC0JcwenzGglTtsLyIuuWFfkqDG9vz67cnttSd53djw==",
        "_location": "/eslint-scope",
        "_phantomChildren": {},
        "_requested": {
            "type": "version",
            "registry": true,
            "raw": "eslint-scope@5.0.0",
            "name": "eslint-scope",
            "escapedName": "eslint-scope",
            "rawSpec": "5.0.0",
            "saveSpec": null,
            "fetchSpec": "5.0.0"
        },
        "_requiredBy": [
            "/@typescript-eslint/experimental-utils",
            "/eslint"
        ],
        "_resolved": "https://registry.npmjs.org/eslint-scope/-/eslint-scope-5.0.0.tgz",
        "_spec": "5.0.0",
        "_where": "/Users/bryan/work/bokeh/bokehjs",
        "bugs": {
            "url": "https://github.com/eslint/eslint-scope/issues"
        },
        "dependencies": {
            "esrecurse": "^4.1.0",
            "estraverse": "^4.1.1"
        },
        "description": "ECMAScript scope analyzer for ESLint",
        "devDependencies": {
            "@typescript-eslint/parser": "^1.11.0",
            "chai": "^4.2.0",
            "eslint": "^6.0.1",
            "eslint-config-eslint": "^5.0.1",
            "eslint-plugin-node": "^9.1.0",
            "eslint-release": "^1.0.0",
            "espree": "^6.0.0",
            "istanbul": "^0.4.5",
            "mocha": "^6.1.4",
            "npm-license": "^0.3.3",
            "shelljs": "^0.8.3",
            "typescript": "^3.5.2"
        },
        "engines": {
            "node": ">=8.0.0"
        },
        "files": [
            "LICENSE",
            "README.md",
            "lib"
        ],
        "homepage": "http://github.com/eslint/eslint-scope",
        "license": "BSD-2-Clause",
        "main": "lib/index.js",
        "name": "eslint-scope",
        "repository": {
            "type": "git",
            "url": "git+https://github.com/eslint/eslint-scope.git"
        },
        "scripts": {
            "generate-alpharelease": "eslint-generate-prerelease alpha",
            "generate-betarelease": "eslint-generate-prerelease beta",
            "generate-rcrelease": "eslint-generate-prerelease rc",
            "generate-release": "eslint-generate-release",
            "lint": "node Makefile.js lint",
            "publish-release": "eslint-publish-release",
            "test": "node Makefile.js test"
        },
        "version": "5.0.0"
    };
},
/* eslint-visitor-keys/lib/index.js */ function _(require, module, exports) {
    const KEYS = require(101) /* ./visitor-keys.json */;
    // Types.
    const NODE_TYPES = Object.freeze(Object.keys(KEYS));
    // Freeze the keys.
    for (const type of NODE_TYPES) {
        Object.freeze(KEYS[type]);
    }
    Object.freeze(KEYS);
    // List to ignore keys.
    const KEY_BLACKLIST = new Set([
        "parent",
        "leadingComments",
        "trailingComments"
    ]);
    /**
     * Check whether a given key should be used or not.
     * @param {string} key The key to check.
     * @returns {boolean} `true` if the key should be used.
     */
    function filterKey(key) {
        return !KEY_BLACKLIST.has(key) && key[0] !== "_";
    }
    //------------------------------------------------------------------------------
    // Public interfaces
    //------------------------------------------------------------------------------
    module.exports = Object.freeze({
        /**
         * Visitor keys.
         * @type {{ [type: string]: string[] | undefined }}
         */
        KEYS,
        /**
         * Get visitor keys of a given node.
         * @param {Object} node The AST node to get keys.
         * @returns {string[]} Visitor keys of the node.
         */
        getKeys(node) {
            return Object.keys(node).filter(filterKey);
        },
        // Disable valid-jsdoc rule because it reports syntax error on the type of @returns.
        // eslint-disable-next-line valid-jsdoc
        /**
         * Make the union set with `KEYS` and given keys.
         * @param {Object} additionalKeys The additional keys.
         * @returns {{ [type: string]: string[] | undefined }} The union set.
         */
        unionWith(additionalKeys) {
            const retv = Object.assign({}, KEYS);
            for (const type of Object.keys(additionalKeys)) {
                if (retv.hasOwnProperty(type)) {
                    const keys = new Set(additionalKeys[type]);
                    for (const key of retv[type]) {
                        keys.add(key);
                    }
                    retv[type] = Object.freeze(Array.from(keys));
                }
                else {
                    retv[type] = Object.freeze(Array.from(additionalKeys[type]));
                }
            }
            return Object.freeze(retv);
        }
    });
},
/* eslint-visitor-keys/lib/visitor-keys.json */ function _(require, module, exports) {
    module.exports = {
        "AssignmentExpression": [
            "left",
            "right"
        ],
        "AssignmentPattern": [
            "left",
            "right"
        ],
        "ArrayExpression": [
            "elements"
        ],
        "ArrayPattern": [
            "elements"
        ],
        "ArrowFunctionExpression": [
            "params",
            "body"
        ],
        "AwaitExpression": [
            "argument"
        ],
        "BlockStatement": [
            "body"
        ],
        "BinaryExpression": [
            "left",
            "right"
        ],
        "BreakStatement": [
            "label"
        ],
        "CallExpression": [
            "callee",
            "arguments"
        ],
        "CatchClause": [
            "param",
            "body"
        ],
        "ClassBody": [
            "body"
        ],
        "ClassDeclaration": [
            "id",
            "superClass",
            "body"
        ],
        "ClassExpression": [
            "id",
            "superClass",
            "body"
        ],
        "ConditionalExpression": [
            "test",
            "consequent",
            "alternate"
        ],
        "ContinueStatement": [
            "label"
        ],
        "DebuggerStatement": [],
        "DoWhileStatement": [
            "body",
            "test"
        ],
        "EmptyStatement": [],
        "ExportAllDeclaration": [
            "source"
        ],
        "ExportDefaultDeclaration": [
            "declaration"
        ],
        "ExportNamedDeclaration": [
            "declaration",
            "specifiers",
            "source"
        ],
        "ExportSpecifier": [
            "exported",
            "local"
        ],
        "ExpressionStatement": [
            "expression"
        ],
        "ExperimentalRestProperty": [
            "argument"
        ],
        "ExperimentalSpreadProperty": [
            "argument"
        ],
        "ForStatement": [
            "init",
            "test",
            "update",
            "body"
        ],
        "ForInStatement": [
            "left",
            "right",
            "body"
        ],
        "ForOfStatement": [
            "left",
            "right",
            "body"
        ],
        "FunctionDeclaration": [
            "id",
            "params",
            "body"
        ],
        "FunctionExpression": [
            "id",
            "params",
            "body"
        ],
        "Identifier": [],
        "IfStatement": [
            "test",
            "consequent",
            "alternate"
        ],
        "ImportDeclaration": [
            "specifiers",
            "source"
        ],
        "ImportDefaultSpecifier": [
            "local"
        ],
        "ImportExpression": [
            "source"
        ],
        "ImportNamespaceSpecifier": [
            "local"
        ],
        "ImportSpecifier": [
            "imported",
            "local"
        ],
        "JSXAttribute": [
            "name",
            "value"
        ],
        "JSXClosingElement": [
            "name"
        ],
        "JSXElement": [
            "openingElement",
            "children",
            "closingElement"
        ],
        "JSXEmptyExpression": [],
        "JSXExpressionContainer": [
            "expression"
        ],
        "JSXIdentifier": [],
        "JSXMemberExpression": [
            "object",
            "property"
        ],
        "JSXNamespacedName": [
            "namespace",
            "name"
        ],
        "JSXOpeningElement": [
            "name",
            "attributes"
        ],
        "JSXSpreadAttribute": [
            "argument"
        ],
        "JSXText": [],
        "JSXFragment": [
            "openingFragment",
            "children",
            "closingFragment"
        ],
        "Literal": [],
        "LabeledStatement": [
            "label",
            "body"
        ],
        "LogicalExpression": [
            "left",
            "right"
        ],
        "MemberExpression": [
            "object",
            "property"
        ],
        "MetaProperty": [
            "meta",
            "property"
        ],
        "MethodDefinition": [
            "key",
            "value"
        ],
        "NewExpression": [
            "callee",
            "arguments"
        ],
        "ObjectExpression": [
            "properties"
        ],
        "ObjectPattern": [
            "properties"
        ],
        "Program": [
            "body"
        ],
        "Property": [
            "key",
            "value"
        ],
        "RestElement": [
            "argument"
        ],
        "ReturnStatement": [
            "argument"
        ],
        "SequenceExpression": [
            "expressions"
        ],
        "SpreadElement": [
            "argument"
        ],
        "Super": [],
        "SwitchStatement": [
            "discriminant",
            "cases"
        ],
        "SwitchCase": [
            "test",
            "consequent"
        ],
        "TaggedTemplateExpression": [
            "tag",
            "quasi"
        ],
        "TemplateElement": [],
        "TemplateLiteral": [
            "quasis",
            "expressions"
        ],
        "ThisExpression": [],
        "ThrowStatement": [
            "argument"
        ],
        "TryStatement": [
            "block",
            "handler",
            "finalizer"
        ],
        "UnaryExpression": [
            "argument"
        ],
        "UpdateExpression": [
            "argument"
        ],
        "VariableDeclaration": [
            "declarations"
        ],
        "VariableDeclarator": [
            "id",
            "init"
        ],
        "WhileStatement": [
            "test",
            "body"
        ],
        "WithStatement": [
            "object",
            "body"
        ],
        "YieldExpression": [
            "argument"
        ]
    };
},
/* espree/espree.js */ function _(require, module, exports) {
    const acorn = require(103) /* acorn */;
    const jsx = require(104) /* acorn-jsx */;
    const astNodeTypes = require(106) /* ./lib/ast-node-types */;
    const espree = require(107) /* ./lib/espree */;
    const { getLatestEcmaVersion, getSupportedEcmaVersions } = require(109) /* ./lib/options */;
    // To initialize lazily.
    const parsers = {
        _regular: null,
        _jsx: null,
        get regular() {
            if (this._regular === null) {
                this._regular = acorn.Parser.extend(espree());
            }
            return this._regular;
        },
        get jsx() {
            if (this._jsx === null) {
                this._jsx = acorn.Parser.extend(jsx(), espree());
            }
            return this._jsx;
        },
        get(options) {
            const useJsx = Boolean(options &&
                options.ecmaFeatures &&
                options.ecmaFeatures.jsx);
            return useJsx ? this.jsx : this.regular;
        }
    };
    //------------------------------------------------------------------------------
    // Tokenizer
    //------------------------------------------------------------------------------
    /**
     * Tokenizes the given code.
     * @param {string} code The code to tokenize.
     * @param {Object} options Options defining how to tokenize.
     * @returns {Token[]} An array of tokens.
     * @throws {SyntaxError} If the input code is invalid.
     * @private
     */
    function tokenize(code, options) {
        const Parser = parsers.get(options);
        // Ensure to collect tokens.
        if (!options || options.tokens !== true) {
            options = Object.assign({}, options, { tokens: true }); // eslint-disable-line no-param-reassign
        }
        return new Parser(options, code).tokenize();
    }
    //------------------------------------------------------------------------------
    // Parser
    //------------------------------------------------------------------------------
    /**
     * Parses the given code.
     * @param {string} code The code to tokenize.
     * @param {Object} options Options defining how to tokenize.
     * @returns {ASTNode} The "Program" AST node.
     * @throws {SyntaxError} If the input code is invalid.
     */
    function parse(code, options) {
        const Parser = parsers.get(options);
        return new Parser(options, code).parse();
    }
    //------------------------------------------------------------------------------
    // Public
    //------------------------------------------------------------------------------
    exports.version = require(110) /* ./package.json */.version;
    exports.tokenize = tokenize;
    exports.parse = parse;
    // Deep copy.
    /* istanbul ignore next */
    exports.Syntax = (function () {
        let name, types = {};
        if (typeof Object.create === "function") {
            types = Object.create(null);
        }
        for (name in astNodeTypes) {
            if (Object.hasOwnProperty.call(astNodeTypes, name)) {
                types[name] = astNodeTypes[name];
            }
        }
        if (typeof Object.freeze === "function") {
            Object.freeze(types);
        }
        return types;
    }());
    /* istanbul ignore next */
    exports.VisitorKeys = (function () {
        return require(100) /* eslint-visitor-keys */.KEYS;
    }());
    exports.latestEcmaVersion = getLatestEcmaVersion();
    exports.supportedEcmaVersions = getSupportedEcmaVersions();
},
/* acorn/dist/acorn.js */ function _(require, module, exports) {
    (function (global, factory) {
        typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
            typeof define === 'function' && define.amd ? define(['exports'], factory) :
                (global = global || self, factory(global.acorn = {}));
    }(this, function (exports) {
        'use strict';
        // Reserved word lists for various dialects of the language
        var reservedWords = {
            3: "abstract boolean byte char class double enum export extends final float goto implements import int interface long native package private protected public short static super synchronized throws transient volatile",
            5: "class enum extends super const export import",
            6: "enum",
            strict: "implements interface let package private protected public static yield",
            strictBind: "eval arguments"
        };
        // And the keywords
        var ecma5AndLessKeywords = "break case catch continue debugger default do else finally for function if return switch throw try var while with null true false instanceof typeof void delete new in this";
        var keywords = {
            5: ecma5AndLessKeywords,
            "5module": ecma5AndLessKeywords + " export import",
            6: ecma5AndLessKeywords + " const class extends export import super"
        };
        var keywordRelationalOperator = /^in(stanceof)?$/;
        // ## Character categories
        // Big ugly regular expressions that match characters in the
        // whitespace, identifier, and identifier-start categories. These
        // are only applied when a character is found to actually have a
        // code point above 128.
        // Generated by `bin/generate-identifier-regex.js`.
        var nonASCIIidentifierStartChars = "\xaa\xb5\xba\xc0-\xd6\xd8-\xf6\xf8-\u02c1\u02c6-\u02d1\u02e0-\u02e4\u02ec\u02ee\u0370-\u0374\u0376\u0377\u037a-\u037d\u037f\u0386\u0388-\u038a\u038c\u038e-\u03a1\u03a3-\u03f5\u03f7-\u0481\u048a-\u052f\u0531-\u0556\u0559\u0560-\u0588\u05d0-\u05ea\u05ef-\u05f2\u0620-\u064a\u066e\u066f\u0671-\u06d3\u06d5\u06e5\u06e6\u06ee\u06ef\u06fa-\u06fc\u06ff\u0710\u0712-\u072f\u074d-\u07a5\u07b1\u07ca-\u07ea\u07f4\u07f5\u07fa\u0800-\u0815\u081a\u0824\u0828\u0840-\u0858\u0860-\u086a\u08a0-\u08b4\u08b6-\u08bd\u0904-\u0939\u093d\u0950\u0958-\u0961\u0971-\u0980\u0985-\u098c\u098f\u0990\u0993-\u09a8\u09aa-\u09b0\u09b2\u09b6-\u09b9\u09bd\u09ce\u09dc\u09dd\u09df-\u09e1\u09f0\u09f1\u09fc\u0a05-\u0a0a\u0a0f\u0a10\u0a13-\u0a28\u0a2a-\u0a30\u0a32\u0a33\u0a35\u0a36\u0a38\u0a39\u0a59-\u0a5c\u0a5e\u0a72-\u0a74\u0a85-\u0a8d\u0a8f-\u0a91\u0a93-\u0aa8\u0aaa-\u0ab0\u0ab2\u0ab3\u0ab5-\u0ab9\u0abd\u0ad0\u0ae0\u0ae1\u0af9\u0b05-\u0b0c\u0b0f\u0b10\u0b13-\u0b28\u0b2a-\u0b30\u0b32\u0b33\u0b35-\u0b39\u0b3d\u0b5c\u0b5d\u0b5f-\u0b61\u0b71\u0b83\u0b85-\u0b8a\u0b8e-\u0b90\u0b92-\u0b95\u0b99\u0b9a\u0b9c\u0b9e\u0b9f\u0ba3\u0ba4\u0ba8-\u0baa\u0bae-\u0bb9\u0bd0\u0c05-\u0c0c\u0c0e-\u0c10\u0c12-\u0c28\u0c2a-\u0c39\u0c3d\u0c58-\u0c5a\u0c60\u0c61\u0c80\u0c85-\u0c8c\u0c8e-\u0c90\u0c92-\u0ca8\u0caa-\u0cb3\u0cb5-\u0cb9\u0cbd\u0cde\u0ce0\u0ce1\u0cf1\u0cf2\u0d05-\u0d0c\u0d0e-\u0d10\u0d12-\u0d3a\u0d3d\u0d4e\u0d54-\u0d56\u0d5f-\u0d61\u0d7a-\u0d7f\u0d85-\u0d96\u0d9a-\u0db1\u0db3-\u0dbb\u0dbd\u0dc0-\u0dc6\u0e01-\u0e30\u0e32\u0e33\u0e40-\u0e46\u0e81\u0e82\u0e84\u0e86-\u0e8a\u0e8c-\u0ea3\u0ea5\u0ea7-\u0eb0\u0eb2\u0eb3\u0ebd\u0ec0-\u0ec4\u0ec6\u0edc-\u0edf\u0f00\u0f40-\u0f47\u0f49-\u0f6c\u0f88-\u0f8c\u1000-\u102a\u103f\u1050-\u1055\u105a-\u105d\u1061\u1065\u1066\u106e-\u1070\u1075-\u1081\u108e\u10a0-\u10c5\u10c7\u10cd\u10d0-\u10fa\u10fc-\u1248\u124a-\u124d\u1250-\u1256\u1258\u125a-\u125d\u1260-\u1288\u128a-\u128d\u1290-\u12b0\u12b2-\u12b5\u12b8-\u12be\u12c0\u12c2-\u12c5\u12c8-\u12d6\u12d8-\u1310\u1312-\u1315\u1318-\u135a\u1380-\u138f\u13a0-\u13f5\u13f8-\u13fd\u1401-\u166c\u166f-\u167f\u1681-\u169a\u16a0-\u16ea\u16ee-\u16f8\u1700-\u170c\u170e-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176c\u176e-\u1770\u1780-\u17b3\u17d7\u17dc\u1820-\u1878\u1880-\u18a8\u18aa\u18b0-\u18f5\u1900-\u191e\u1950-\u196d\u1970-\u1974\u1980-\u19ab\u19b0-\u19c9\u1a00-\u1a16\u1a20-\u1a54\u1aa7\u1b05-\u1b33\u1b45-\u1b4b\u1b83-\u1ba0\u1bae\u1baf\u1bba-\u1be5\u1c00-\u1c23\u1c4d-\u1c4f\u1c5a-\u1c7d\u1c80-\u1c88\u1c90-\u1cba\u1cbd-\u1cbf\u1ce9-\u1cec\u1cee-\u1cf3\u1cf5\u1cf6\u1cfa\u1d00-\u1dbf\u1e00-\u1f15\u1f18-\u1f1d\u1f20-\u1f45\u1f48-\u1f4d\u1f50-\u1f57\u1f59\u1f5b\u1f5d\u1f5f-\u1f7d\u1f80-\u1fb4\u1fb6-\u1fbc\u1fbe\u1fc2-\u1fc4\u1fc6-\u1fcc\u1fd0-\u1fd3\u1fd6-\u1fdb\u1fe0-\u1fec\u1ff2-\u1ff4\u1ff6-\u1ffc\u2071\u207f\u2090-\u209c\u2102\u2107\u210a-\u2113\u2115\u2118-\u211d\u2124\u2126\u2128\u212a-\u2139\u213c-\u213f\u2145-\u2149\u214e\u2160-\u2188\u2c00-\u2c2e\u2c30-\u2c5e\u2c60-\u2ce4\u2ceb-\u2cee\u2cf2\u2cf3\u2d00-\u2d25\u2d27\u2d2d\u2d30-\u2d67\u2d6f\u2d80-\u2d96\u2da0-\u2da6\u2da8-\u2dae\u2db0-\u2db6\u2db8-\u2dbe\u2dc0-\u2dc6\u2dc8-\u2dce\u2dd0-\u2dd6\u2dd8-\u2dde\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303c\u3041-\u3096\u309b-\u309f\u30a1-\u30fa\u30fc-\u30ff\u3105-\u312f\u3131-\u318e\u31a0-\u31ba\u31f0-\u31ff\u3400-\u4db5\u4e00-\u9fef\ua000-\ua48c\ua4d0-\ua4fd\ua500-\ua60c\ua610-\ua61f\ua62a\ua62b\ua640-\ua66e\ua67f-\ua69d\ua6a0-\ua6ef\ua717-\ua71f\ua722-\ua788\ua78b-\ua7bf\ua7c2-\ua7c6\ua7f7-\ua801\ua803-\ua805\ua807-\ua80a\ua80c-\ua822\ua840-\ua873\ua882-\ua8b3\ua8f2-\ua8f7\ua8fb\ua8fd\ua8fe\ua90a-\ua925\ua930-\ua946\ua960-\ua97c\ua984-\ua9b2\ua9cf\ua9e0-\ua9e4\ua9e6-\ua9ef\ua9fa-\ua9fe\uaa00-\uaa28\uaa40-\uaa42\uaa44-\uaa4b\uaa60-\uaa76\uaa7a\uaa7e-\uaaaf\uaab1\uaab5\uaab6\uaab9-\uaabd\uaac0\uaac2\uaadb-\uaadd\uaae0-\uaaea\uaaf2-\uaaf4\uab01-\uab06\uab09-\uab0e\uab11-\uab16\uab20-\uab26\uab28-\uab2e\uab30-\uab5a\uab5c-\uab67\uab70-\uabe2\uac00-\ud7a3\ud7b0-\ud7c6\ud7cb-\ud7fb\uf900-\ufa6d\ufa70-\ufad9\ufb00-\ufb06\ufb13-\ufb17\ufb1d\ufb1f-\ufb28\ufb2a-\ufb36\ufb38-\ufb3c\ufb3e\ufb40\ufb41\ufb43\ufb44\ufb46-\ufbb1\ufbd3-\ufd3d\ufd50-\ufd8f\ufd92-\ufdc7\ufdf0-\ufdfb\ufe70-\ufe74\ufe76-\ufefc\uff21-\uff3a\uff41-\uff5a\uff66-\uffbe\uffc2-\uffc7\uffca-\uffcf\uffd2-\uffd7\uffda-\uffdc";
        var nonASCIIidentifierChars = "\u200c\u200d\xb7\u0300-\u036f\u0387\u0483-\u0487\u0591-\u05bd\u05bf\u05c1\u05c2\u05c4\u05c5\u05c7\u0610-\u061a\u064b-\u0669\u0670\u06d6-\u06dc\u06df-\u06e4\u06e7\u06e8\u06ea-\u06ed\u06f0-\u06f9\u0711\u0730-\u074a\u07a6-\u07b0\u07c0-\u07c9\u07eb-\u07f3\u07fd\u0816-\u0819\u081b-\u0823\u0825-\u0827\u0829-\u082d\u0859-\u085b\u08d3-\u08e1\u08e3-\u0903\u093a-\u093c\u093e-\u094f\u0951-\u0957\u0962\u0963\u0966-\u096f\u0981-\u0983\u09bc\u09be-\u09c4\u09c7\u09c8\u09cb-\u09cd\u09d7\u09e2\u09e3\u09e6-\u09ef\u09fe\u0a01-\u0a03\u0a3c\u0a3e-\u0a42\u0a47\u0a48\u0a4b-\u0a4d\u0a51\u0a66-\u0a71\u0a75\u0a81-\u0a83\u0abc\u0abe-\u0ac5\u0ac7-\u0ac9\u0acb-\u0acd\u0ae2\u0ae3\u0ae6-\u0aef\u0afa-\u0aff\u0b01-\u0b03\u0b3c\u0b3e-\u0b44\u0b47\u0b48\u0b4b-\u0b4d\u0b56\u0b57\u0b62\u0b63\u0b66-\u0b6f\u0b82\u0bbe-\u0bc2\u0bc6-\u0bc8\u0bca-\u0bcd\u0bd7\u0be6-\u0bef\u0c00-\u0c04\u0c3e-\u0c44\u0c46-\u0c48\u0c4a-\u0c4d\u0c55\u0c56\u0c62\u0c63\u0c66-\u0c6f\u0c81-\u0c83\u0cbc\u0cbe-\u0cc4\u0cc6-\u0cc8\u0cca-\u0ccd\u0cd5\u0cd6\u0ce2\u0ce3\u0ce6-\u0cef\u0d00-\u0d03\u0d3b\u0d3c\u0d3e-\u0d44\u0d46-\u0d48\u0d4a-\u0d4d\u0d57\u0d62\u0d63\u0d66-\u0d6f\u0d82\u0d83\u0dca\u0dcf-\u0dd4\u0dd6\u0dd8-\u0ddf\u0de6-\u0def\u0df2\u0df3\u0e31\u0e34-\u0e3a\u0e47-\u0e4e\u0e50-\u0e59\u0eb1\u0eb4-\u0ebc\u0ec8-\u0ecd\u0ed0-\u0ed9\u0f18\u0f19\u0f20-\u0f29\u0f35\u0f37\u0f39\u0f3e\u0f3f\u0f71-\u0f84\u0f86\u0f87\u0f8d-\u0f97\u0f99-\u0fbc\u0fc6\u102b-\u103e\u1040-\u1049\u1056-\u1059\u105e-\u1060\u1062-\u1064\u1067-\u106d\u1071-\u1074\u1082-\u108d\u108f-\u109d\u135d-\u135f\u1369-\u1371\u1712-\u1714\u1732-\u1734\u1752\u1753\u1772\u1773\u17b4-\u17d3\u17dd\u17e0-\u17e9\u180b-\u180d\u1810-\u1819\u18a9\u1920-\u192b\u1930-\u193b\u1946-\u194f\u19d0-\u19da\u1a17-\u1a1b\u1a55-\u1a5e\u1a60-\u1a7c\u1a7f-\u1a89\u1a90-\u1a99\u1ab0-\u1abd\u1b00-\u1b04\u1b34-\u1b44\u1b50-\u1b59\u1b6b-\u1b73\u1b80-\u1b82\u1ba1-\u1bad\u1bb0-\u1bb9\u1be6-\u1bf3\u1c24-\u1c37\u1c40-\u1c49\u1c50-\u1c59\u1cd0-\u1cd2\u1cd4-\u1ce8\u1ced\u1cf4\u1cf7-\u1cf9\u1dc0-\u1df9\u1dfb-\u1dff\u203f\u2040\u2054\u20d0-\u20dc\u20e1\u20e5-\u20f0\u2cef-\u2cf1\u2d7f\u2de0-\u2dff\u302a-\u302f\u3099\u309a\ua620-\ua629\ua66f\ua674-\ua67d\ua69e\ua69f\ua6f0\ua6f1\ua802\ua806\ua80b\ua823-\ua827\ua880\ua881\ua8b4-\ua8c5\ua8d0-\ua8d9\ua8e0-\ua8f1\ua8ff-\ua909\ua926-\ua92d\ua947-\ua953\ua980-\ua983\ua9b3-\ua9c0\ua9d0-\ua9d9\ua9e5\ua9f0-\ua9f9\uaa29-\uaa36\uaa43\uaa4c\uaa4d\uaa50-\uaa59\uaa7b-\uaa7d\uaab0\uaab2-\uaab4\uaab7\uaab8\uaabe\uaabf\uaac1\uaaeb-\uaaef\uaaf5\uaaf6\uabe3-\uabea\uabec\uabed\uabf0-\uabf9\ufb1e\ufe00-\ufe0f\ufe20-\ufe2f\ufe33\ufe34\ufe4d-\ufe4f\uff10-\uff19\uff3f";
        var nonASCIIidentifierStart = new RegExp("[" + nonASCIIidentifierStartChars + "]");
        var nonASCIIidentifier = new RegExp("[" + nonASCIIidentifierStartChars + nonASCIIidentifierChars + "]");
        nonASCIIidentifierStartChars = nonASCIIidentifierChars = null;
        // These are a run-length and offset encoded representation of the
        // >0xffff code points that are a valid part of identifiers. The
        // offset starts at 0x10000, and each pair of numbers represents an
        // offset to the next range, and then a size of the range. They were
        // generated by bin/generate-identifier-regex.js
        // eslint-disable-next-line comma-spacing
        var astralIdentifierStartCodes = [0, 11, 2, 25, 2, 18, 2, 1, 2, 14, 3, 13, 35, 122, 70, 52, 268, 28, 4, 48, 48, 31, 14, 29, 6, 37, 11, 29, 3, 35, 5, 7, 2, 4, 43, 157, 19, 35, 5, 35, 5, 39, 9, 51, 157, 310, 10, 21, 11, 7, 153, 5, 3, 0, 2, 43, 2, 1, 4, 0, 3, 22, 11, 22, 10, 30, 66, 18, 2, 1, 11, 21, 11, 25, 71, 55, 7, 1, 65, 0, 16, 3, 2, 2, 2, 28, 43, 28, 4, 28, 36, 7, 2, 27, 28, 53, 11, 21, 11, 18, 14, 17, 111, 72, 56, 50, 14, 50, 14, 35, 477, 28, 11, 0, 9, 21, 155, 22, 13, 52, 76, 44, 33, 24, 27, 35, 30, 0, 12, 34, 4, 0, 13, 47, 15, 3, 22, 0, 2, 0, 36, 17, 2, 24, 85, 6, 2, 0, 2, 3, 2, 14, 2, 9, 8, 46, 39, 7, 3, 1, 3, 21, 2, 6, 2, 1, 2, 4, 4, 0, 19, 0, 13, 4, 159, 52, 19, 3, 21, 0, 33, 47, 21, 1, 2, 0, 185, 46, 42, 3, 37, 47, 21, 0, 60, 42, 14, 0, 72, 26, 230, 43, 117, 63, 32, 0, 161, 7, 3, 38, 17, 0, 2, 0, 29, 0, 11, 39, 8, 0, 22, 0, 12, 45, 20, 0, 35, 56, 264, 8, 2, 36, 18, 0, 50, 29, 113, 6, 2, 1, 2, 37, 22, 0, 26, 5, 2, 1, 2, 31, 15, 0, 328, 18, 270, 921, 103, 110, 18, 195, 2749, 1070, 4050, 582, 8634, 568, 8, 30, 114, 29, 19, 47, 17, 3, 32, 20, 6, 18, 689, 63, 129, 74, 6, 0, 67, 12, 65, 1, 2, 0, 29, 6135, 9, 754, 9486, 286, 50, 2, 18, 3, 9, 395, 2309, 106, 6, 12, 4, 8, 8, 9, 5991, 84, 2, 70, 2, 1, 3, 0, 3, 1, 3, 3, 2, 11, 2, 0, 2, 6, 2, 64, 2, 3, 3, 7, 2, 6, 2, 27, 2, 3, 2, 4, 2, 0, 4, 6, 2, 339, 3, 24, 2, 24, 2, 30, 2, 24, 2, 30, 2, 24, 2, 30, 2, 24, 2, 30, 2, 24, 2, 7, 2357, 44, 11, 6, 17, 0, 370, 43, 1301, 196, 60, 67, 8, 0, 1205, 3, 2, 26, 2, 1, 2, 0, 3, 0, 2, 9, 2, 3, 2, 0, 2, 0, 7, 0, 5, 0, 2, 0, 2, 0, 2, 2, 2, 1, 2, 0, 3, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 1, 2, 0, 3, 3, 2, 6, 2, 3, 2, 3, 2, 0, 2, 9, 2, 16, 6, 2, 2, 4, 2, 16, 4421, 42710, 42, 4148, 12, 221, 3, 5761, 15, 7472, 3104, 541];
        // eslint-disable-next-line comma-spacing
        var astralIdentifierCodes = [509, 0, 227, 0, 150, 4, 294, 9, 1368, 2, 2, 1, 6, 3, 41, 2, 5, 0, 166, 1, 574, 3, 9, 9, 525, 10, 176, 2, 54, 14, 32, 9, 16, 3, 46, 10, 54, 9, 7, 2, 37, 13, 2, 9, 6, 1, 45, 0, 13, 2, 49, 13, 9, 3, 4, 9, 83, 11, 7, 0, 161, 11, 6, 9, 7, 3, 56, 1, 2, 6, 3, 1, 3, 2, 10, 0, 11, 1, 3, 6, 4, 4, 193, 17, 10, 9, 5, 0, 82, 19, 13, 9, 214, 6, 3, 8, 28, 1, 83, 16, 16, 9, 82, 12, 9, 9, 84, 14, 5, 9, 243, 14, 166, 9, 232, 6, 3, 6, 4, 0, 29, 9, 41, 6, 2, 3, 9, 0, 10, 10, 47, 15, 406, 7, 2, 7, 17, 9, 57, 21, 2, 13, 123, 5, 4, 0, 2, 1, 2, 6, 2, 0, 9, 9, 49, 4, 2, 1, 2, 4, 9, 9, 330, 3, 19306, 9, 135, 4, 60, 6, 26, 9, 1014, 0, 2, 54, 8, 3, 19723, 1, 5319, 4, 4, 5, 9, 7, 3, 6, 31, 3, 149, 2, 1418, 49, 513, 54, 5, 49, 9, 0, 15, 0, 23, 4, 2, 14, 1361, 6, 2, 16, 3, 6, 2, 1, 2, 4, 262, 6, 10, 9, 419, 13, 1495, 6, 110, 6, 6, 9, 792487, 239];
        // This has a complexity linear to the value of the code. The
        // assumption is that looking up astral identifier characters is
        // rare.
        function isInAstralSet(code, set) {
            var pos = 0x10000;
            for (var i = 0; i < set.length; i += 2) {
                pos += set[i];
                if (pos > code) {
                    return false;
                }
                pos += set[i + 1];
                if (pos >= code) {
                    return true;
                }
            }
        }
        // Test whether a given character code starts an identifier.
        function isIdentifierStart(code, astral) {
            if (code < 65) {
                return code === 36;
            }
            if (code < 91) {
                return true;
            }
            if (code < 97) {
                return code === 95;
            }
            if (code < 123) {
                return true;
            }
            if (code <= 0xffff) {
                return code >= 0xaa && nonASCIIidentifierStart.test(String.fromCharCode(code));
            }
            if (astral === false) {
                return false;
            }
            return isInAstralSet(code, astralIdentifierStartCodes);
        }
        // Test whether a given character is part of an identifier.
        function isIdentifierChar(code, astral) {
            if (code < 48) {
                return code === 36;
            }
            if (code < 58) {
                return true;
            }
            if (code < 65) {
                return false;
            }
            if (code < 91) {
                return true;
            }
            if (code < 97) {
                return code === 95;
            }
            if (code < 123) {
                return true;
            }
            if (code <= 0xffff) {
                return code >= 0xaa && nonASCIIidentifier.test(String.fromCharCode(code));
            }
            if (astral === false) {
                return false;
            }
            return isInAstralSet(code, astralIdentifierStartCodes) || isInAstralSet(code, astralIdentifierCodes);
        }
        // ## Token types
        // The assignment of fine-grained, information-carrying type objects
        // allows the tokenizer to store the information it has about a
        // token in a way that is very cheap for the parser to look up.
        // All token type variables start with an underscore, to make them
        // easy to recognize.
        // The `beforeExpr` property is used to disambiguate between regular
        // expressions and divisions. It is set on all token types that can
        // be followed by an expression (thus, a slash after them would be a
        // regular expression).
        //
        // The `startsExpr` property is used to check if the token ends a
        // `yield` expression. It is set on all token types that either can
        // directly start an expression (like a quotation mark) or can
        // continue an expression (like the body of a string).
        //
        // `isLoop` marks a keyword as starting a loop, which is important
        // to know when parsing a label, in order to allow or disallow
        // continue jumps to that label.
        var TokenType = function TokenType(label, conf) {
            if (conf === void 0)
                conf = {};
            this.label = label;
            this.keyword = conf.keyword;
            this.beforeExpr = !!conf.beforeExpr;
            this.startsExpr = !!conf.startsExpr;
            this.isLoop = !!conf.isLoop;
            this.isAssign = !!conf.isAssign;
            this.prefix = !!conf.prefix;
            this.postfix = !!conf.postfix;
            this.binop = conf.binop || null;
            this.updateContext = null;
        };
        function binop(name, prec) {
            return new TokenType(name, { beforeExpr: true, binop: prec });
        }
        var beforeExpr = { beforeExpr: true }, startsExpr = { startsExpr: true };
        // Map keyword names to token types.
        var keywords$1 = {};
        // Succinct definitions of keyword token types
        function kw(name, options) {
            if (options === void 0)
                options = {};
            options.keyword = name;
            return keywords$1[name] = new TokenType(name, options);
        }
        var types = {
            num: new TokenType("num", startsExpr),
            regexp: new TokenType("regexp", startsExpr),
            string: new TokenType("string", startsExpr),
            name: new TokenType("name", startsExpr),
            eof: new TokenType("eof"),
            // Punctuation token types.
            bracketL: new TokenType("[", { beforeExpr: true, startsExpr: true }),
            bracketR: new TokenType("]"),
            braceL: new TokenType("{", { beforeExpr: true, startsExpr: true }),
            braceR: new TokenType("}"),
            parenL: new TokenType("(", { beforeExpr: true, startsExpr: true }),
            parenR: new TokenType(")"),
            comma: new TokenType(",", beforeExpr),
            semi: new TokenType(";", beforeExpr),
            colon: new TokenType(":", beforeExpr),
            dot: new TokenType("."),
            question: new TokenType("?", beforeExpr),
            arrow: new TokenType("=>", beforeExpr),
            template: new TokenType("template"),
            invalidTemplate: new TokenType("invalidTemplate"),
            ellipsis: new TokenType("...", beforeExpr),
            backQuote: new TokenType("`", startsExpr),
            dollarBraceL: new TokenType("${", { beforeExpr: true, startsExpr: true }),
            // Operators. These carry several kinds of properties to help the
            // parser use them properly (the presence of these properties is
            // what categorizes them as operators).
            //
            // `binop`, when present, specifies that this operator is a binary
            // operator, and will refer to its precedence.
            //
            // `prefix` and `postfix` mark the operator as a prefix or postfix
            // unary operator.
            //
            // `isAssign` marks all of `=`, `+=`, `-=` etcetera, which act as
            // binary operators with a very low precedence, that should result
            // in AssignmentExpression nodes.
            eq: new TokenType("=", { beforeExpr: true, isAssign: true }),
            assign: new TokenType("_=", { beforeExpr: true, isAssign: true }),
            incDec: new TokenType("++/--", { prefix: true, postfix: true, startsExpr: true }),
            prefix: new TokenType("!/~", { beforeExpr: true, prefix: true, startsExpr: true }),
            logicalOR: binop("||", 1),
            logicalAND: binop("&&", 2),
            bitwiseOR: binop("|", 3),
            bitwiseXOR: binop("^", 4),
            bitwiseAND: binop("&", 5),
            equality: binop("==/!=/===/!==", 6),
            relational: binop("</>/<=/>=", 7),
            bitShift: binop("<</>>/>>>", 8),
            plusMin: new TokenType("+/-", { beforeExpr: true, binop: 9, prefix: true, startsExpr: true }),
            modulo: binop("%", 10),
            star: binop("*", 10),
            slash: binop("/", 10),
            starstar: new TokenType("**", { beforeExpr: true }),
            // Keyword token types.
            _break: kw("break"),
            _case: kw("case", beforeExpr),
            _catch: kw("catch"),
            _continue: kw("continue"),
            _debugger: kw("debugger"),
            _default: kw("default", beforeExpr),
            _do: kw("do", { isLoop: true, beforeExpr: true }),
            _else: kw("else", beforeExpr),
            _finally: kw("finally"),
            _for: kw("for", { isLoop: true }),
            _function: kw("function", startsExpr),
            _if: kw("if"),
            _return: kw("return", beforeExpr),
            _switch: kw("switch"),
            _throw: kw("throw", beforeExpr),
            _try: kw("try"),
            _var: kw("var"),
            _const: kw("const"),
            _while: kw("while", { isLoop: true }),
            _with: kw("with"),
            _new: kw("new", { beforeExpr: true, startsExpr: true }),
            _this: kw("this", startsExpr),
            _super: kw("super", startsExpr),
            _class: kw("class", startsExpr),
            _extends: kw("extends", beforeExpr),
            _export: kw("export"),
            _import: kw("import", startsExpr),
            _null: kw("null", startsExpr),
            _true: kw("true", startsExpr),
            _false: kw("false", startsExpr),
            _in: kw("in", { beforeExpr: true, binop: 7 }),
            _instanceof: kw("instanceof", { beforeExpr: true, binop: 7 }),
            _typeof: kw("typeof", { beforeExpr: true, prefix: true, startsExpr: true }),
            _void: kw("void", { beforeExpr: true, prefix: true, startsExpr: true }),
            _delete: kw("delete", { beforeExpr: true, prefix: true, startsExpr: true })
        };
        // Matches a whole line break (where CRLF is considered a single
        // line break). Used to count lines.
        var lineBreak = /\r\n?|\n|\u2028|\u2029/;
        var lineBreakG = new RegExp(lineBreak.source, "g");
        function isNewLine(code, ecma2019String) {
            return code === 10 || code === 13 || (!ecma2019String && (code === 0x2028 || code === 0x2029));
        }
        var nonASCIIwhitespace = /[\u1680\u2000-\u200a\u202f\u205f\u3000\ufeff]/;
        var skipWhiteSpace = /(?:\s|\/\/.*|\/\*[^]*?\*\/)*/g;
        var ref = Object.prototype;
        var hasOwnProperty = ref.hasOwnProperty;
        var toString = ref.toString;
        // Checks if an object has a property.
        function has(obj, propName) {
            return hasOwnProperty.call(obj, propName);
        }
        var isArray = Array.isArray || (function (obj) {
            return (toString.call(obj) === "[object Array]");
        });
        function wordsRegexp(words) {
            return new RegExp("^(?:" + words.replace(/ /g, "|") + ")$");
        }
        // These are used when `options.locations` is on, for the
        // `startLoc` and `endLoc` properties.
        var Position = function Position(line, col) {
            this.line = line;
            this.column = col;
        };
        Position.prototype.offset = function offset(n) {
            return new Position(this.line, this.column + n);
        };
        var SourceLocation = function SourceLocation(p, start, end) {
            this.start = start;
            this.end = end;
            if (p.sourceFile !== null) {
                this.source = p.sourceFile;
            }
        };
        // The `getLineInfo` function is mostly useful when the
        // `locations` option is off (for performance reasons) and you
        // want to find the line/column position for a given character
        // offset. `input` should be the code string that the offset refers
        // into.
        function getLineInfo(input, offset) {
            for (var line = 1, cur = 0;;) {
                lineBreakG.lastIndex = cur;
                var match = lineBreakG.exec(input);
                if (match && match.index < offset) {
                    ++line;
                    cur = match.index + match[0].length;
                }
                else {
                    return new Position(line, offset - cur);
                }
            }
        }
        // A second optional argument can be given to further configure
        // the parser process. These options are recognized:
        var defaultOptions = {
            // `ecmaVersion` indicates the ECMAScript version to parse. Must be
            // either 3, 5, 6 (2015), 7 (2016), 8 (2017), 9 (2018), or 10
            // (2019). This influences support for strict mode, the set of
            // reserved words, and support for new syntax features. The default
            // is 10.
            ecmaVersion: 10,
            // `sourceType` indicates the mode the code should be parsed in.
            // Can be either `"script"` or `"module"`. This influences global
            // strict mode and parsing of `import` and `export` declarations.
            sourceType: "script",
            // `onInsertedSemicolon` can be a callback that will be called
            // when a semicolon is automatically inserted. It will be passed
            // the position of the comma as an offset, and if `locations` is
            // enabled, it is given the location as a `{line, column}` object
            // as second argument.
            onInsertedSemicolon: null,
            // `onTrailingComma` is similar to `onInsertedSemicolon`, but for
            // trailing commas.
            onTrailingComma: null,
            // By default, reserved words are only enforced if ecmaVersion >= 5.
            // Set `allowReserved` to a boolean value to explicitly turn this on
            // an off. When this option has the value "never", reserved words
            // and keywords can also not be used as property names.
            allowReserved: null,
            // When enabled, a return at the top level is not considered an
            // error.
            allowReturnOutsideFunction: false,
            // When enabled, import/export statements are not constrained to
            // appearing at the top of the program.
            allowImportExportEverywhere: false,
            // When enabled, await identifiers are allowed to appear at the top-level scope,
            // but they are still not allowed in non-async functions.
            allowAwaitOutsideFunction: false,
            // When enabled, hashbang directive in the beginning of file
            // is allowed and treated as a line comment.
            allowHashBang: false,
            // When `locations` is on, `loc` properties holding objects with
            // `start` and `end` properties in `{line, column}` form (with
            // line being 1-based and column 0-based) will be attached to the
            // nodes.
            locations: false,
            // A function can be passed as `onToken` option, which will
            // cause Acorn to call that function with object in the same
            // format as tokens returned from `tokenizer().getToken()`. Note
            // that you are not allowed to call the parser from the
            // callback—that will corrupt its internal state.
            onToken: null,
            // A function can be passed as `onComment` option, which will
            // cause Acorn to call that function with `(block, text, start,
            // end)` parameters whenever a comment is skipped. `block` is a
            // boolean indicating whether this is a block (`/* */`) comment,
            // `text` is the content of the comment, and `start` and `end` are
            // character offsets that denote the start and end of the comment.
            // When the `locations` option is on, two more parameters are
            // passed, the full `{line, column}` locations of the start and
            // end of the comments. Note that you are not allowed to call the
            // parser from the callback—that will corrupt its internal state.
            onComment: null,
            // Nodes have their start and end characters offsets recorded in
            // `start` and `end` properties (directly on the node, rather than
            // the `loc` object, which holds line/column data. To also add a
            // [semi-standardized][range] `range` property holding a `[start,
            // end]` array with the same numbers, set the `ranges` option to
            // `true`.
            //
            // [range]: https://bugzilla.mozilla.org/show_bug.cgi?id=745678
            ranges: false,
            // It is possible to parse multiple files into a single AST by
            // passing the tree produced by parsing the first file as
            // `program` option in subsequent parses. This will add the
            // toplevel forms of the parsed file to the `Program` (top) node
            // of an existing parse tree.
            program: null,
            // When `locations` is on, you can pass this to record the source
            // file in every node's `loc` object.
            sourceFile: null,
            // This value, if given, is stored in every node, whether
            // `locations` is on or off.
            directSourceFile: null,
            // When enabled, parenthesized expressions are represented by
            // (non-standard) ParenthesizedExpression nodes
            preserveParens: false
        };
        // Interpret and default an options object
        function getOptions(opts) {
            var options = {};
            for (var opt in defaultOptions) {
                options[opt] = opts && has(opts, opt) ? opts[opt] : defaultOptions[opt];
            }
            if (options.ecmaVersion >= 2015) {
                options.ecmaVersion -= 2009;
            }
            if (options.allowReserved == null) {
                options.allowReserved = options.ecmaVersion < 5;
            }
            if (isArray(options.onToken)) {
                var tokens = options.onToken;
                options.onToken = function (token) { return tokens.push(token); };
            }
            if (isArray(options.onComment)) {
                options.onComment = pushComment(options, options.onComment);
            }
            return options;
        }
        function pushComment(options, array) {
            return function (block, text, start, end, startLoc, endLoc) {
                var comment = {
                    type: block ? "Block" : "Line",
                    value: text,
                    start: start,
                    end: end
                };
                if (options.locations) {
                    comment.loc = new SourceLocation(this, startLoc, endLoc);
                }
                if (options.ranges) {
                    comment.range = [start, end];
                }
                array.push(comment);
            };
        }
        // Each scope gets a bitset that may contain these flags
        var SCOPE_TOP = 1, SCOPE_FUNCTION = 2, SCOPE_VAR = SCOPE_TOP | SCOPE_FUNCTION, SCOPE_ASYNC = 4, SCOPE_GENERATOR = 8, SCOPE_ARROW = 16, SCOPE_SIMPLE_CATCH = 32, SCOPE_SUPER = 64, SCOPE_DIRECT_SUPER = 128;
        function functionFlags(async, generator) {
            return SCOPE_FUNCTION | (async ? SCOPE_ASYNC : 0) | (generator ? SCOPE_GENERATOR : 0);
        }
        // Used in checkLVal and declareName to determine the type of a binding
        var BIND_NONE = 0, // Not a binding
        BIND_VAR = 1, // Var-style binding
        BIND_LEXICAL = 2, // Let- or const-style binding
        BIND_FUNCTION = 3, // Function declaration
        BIND_SIMPLE_CATCH = 4, // Simple (identifier pattern) catch binding
        BIND_OUTSIDE = 5; // Special case for function names as bound inside the function
        var Parser = function Parser(options, input, startPos) {
            this.options = options = getOptions(options);
            this.sourceFile = options.sourceFile;
            this.keywords = wordsRegexp(keywords[options.ecmaVersion >= 6 ? 6 : options.sourceType === "module" ? "5module" : 5]);
            var reserved = "";
            if (options.allowReserved !== true) {
                for (var v = options.ecmaVersion;; v--) {
                    if (reserved = reservedWords[v]) {
                        break;
                    }
                }
                if (options.sourceType === "module") {
                    reserved += " await";
                }
            }
            this.reservedWords = wordsRegexp(reserved);
            var reservedStrict = (reserved ? reserved + " " : "") + reservedWords.strict;
            this.reservedWordsStrict = wordsRegexp(reservedStrict);
            this.reservedWordsStrictBind = wordsRegexp(reservedStrict + " " + reservedWords.strictBind);
            this.input = String(input);
            // Used to signal to callers of `readWord1` whether the word
            // contained any escape sequences. This is needed because words with
            // escape sequences must not be interpreted as keywords.
            this.containsEsc = false;
            // Set up token state
            // The current position of the tokenizer in the input.
            if (startPos) {
                this.pos = startPos;
                this.lineStart = this.input.lastIndexOf("\n", startPos - 1) + 1;
                this.curLine = this.input.slice(0, this.lineStart).split(lineBreak).length;
            }
            else {
                this.pos = this.lineStart = 0;
                this.curLine = 1;
            }
            // Properties of the current token:
            // Its type
            this.type = types.eof;
            // For tokens that include more information than their type, the value
            this.value = null;
            // Its start and end offset
            this.start = this.end = this.pos;
            // And, if locations are used, the {line, column} object
            // corresponding to those offsets
            this.startLoc = this.endLoc = this.curPosition();
            // Position information for the previous token
            this.lastTokEndLoc = this.lastTokStartLoc = null;
            this.lastTokStart = this.lastTokEnd = this.pos;
            // The context stack is used to superficially track syntactic
            // context to predict whether a regular expression is allowed in a
            // given position.
            this.context = this.initialContext();
            this.exprAllowed = true;
            // Figure out if it's a module code.
            this.inModule = options.sourceType === "module";
            this.strict = this.inModule || this.strictDirective(this.pos);
            // Used to signify the start of a potential arrow function
            this.potentialArrowAt = -1;
            // Positions to delayed-check that yield/await does not exist in default parameters.
            this.yieldPos = this.awaitPos = this.awaitIdentPos = 0;
            // Labels in scope.
            this.labels = [];
            // Thus-far undefined exports.
            this.undefinedExports = {};
            // If enabled, skip leading hashbang line.
            if (this.pos === 0 && options.allowHashBang && this.input.slice(0, 2) === "#!") {
                this.skipLineComment(2);
            }
            // Scope tracking for duplicate variable names (see scope.js)
            this.scopeStack = [];
            this.enterScope(SCOPE_TOP);
            // For RegExp validation
            this.regexpState = null;
        };
        var prototypeAccessors = { inFunction: { configurable: true }, inGenerator: { configurable: true }, inAsync: { configurable: true }, allowSuper: { configurable: true }, allowDirectSuper: { configurable: true }, treatFunctionsAsVar: { configurable: true } };
        Parser.prototype.parse = function parse() {
            var node = this.options.program || this.startNode();
            this.nextToken();
            return this.parseTopLevel(node);
        };
        prototypeAccessors.inFunction.get = function () { return (this.currentVarScope().flags & SCOPE_FUNCTION) > 0; };
        prototypeAccessors.inGenerator.get = function () { return (this.currentVarScope().flags & SCOPE_GENERATOR) > 0; };
        prototypeAccessors.inAsync.get = function () { return (this.currentVarScope().flags & SCOPE_ASYNC) > 0; };
        prototypeAccessors.allowSuper.get = function () { return (this.currentThisScope().flags & SCOPE_SUPER) > 0; };
        prototypeAccessors.allowDirectSuper.get = function () { return (this.currentThisScope().flags & SCOPE_DIRECT_SUPER) > 0; };
        prototypeAccessors.treatFunctionsAsVar.get = function () { return this.treatFunctionsAsVarInScope(this.currentScope()); };
        // Switch to a getter for 7.0.0.
        Parser.prototype.inNonArrowFunction = function inNonArrowFunction() { return (this.currentThisScope().flags & SCOPE_FUNCTION) > 0; };
        Parser.extend = function extend() {
            var plugins = [], len = arguments.length;
            while (len--)
                plugins[len] = arguments[len];
            var cls = this;
            for (var i = 0; i < plugins.length; i++) {
                cls = plugins[i](cls);
            }
            return cls;
        };
        Parser.parse = function parse(input, options) {
            return new this(options, input).parse();
        };
        Parser.parseExpressionAt = function parseExpressionAt(input, pos, options) {
            var parser = new this(options, input, pos);
            parser.nextToken();
            return parser.parseExpression();
        };
        Parser.tokenizer = function tokenizer(input, options) {
            return new this(options, input);
        };
        Object.defineProperties(Parser.prototype, prototypeAccessors);
        var pp = Parser.prototype;
        // ## Parser utilities
        var literal = /^(?:'((?:\\.|[^'])*?)'|"((?:\\.|[^"])*?)")/;
        pp.strictDirective = function (start) {
            for (;;) {
                // Try to find string literal.
                skipWhiteSpace.lastIndex = start;
                start += skipWhiteSpace.exec(this.input)[0].length;
                var match = literal.exec(this.input.slice(start));
                if (!match) {
                    return false;
                }
                if ((match[1] || match[2]) === "use strict") {
                    return true;
                }
                start += match[0].length;
                // Skip semicolon, if any.
                skipWhiteSpace.lastIndex = start;
                start += skipWhiteSpace.exec(this.input)[0].length;
                if (this.input[start] === ";") {
                    start++;
                }
            }
        };
        // Predicate that tests whether the next token is of the given
        // type, and if yes, consumes it as a side effect.
        pp.eat = function (type) {
            if (this.type === type) {
                this.next();
                return true;
            }
            else {
                return false;
            }
        };
        // Tests whether parsed token is a contextual keyword.
        pp.isContextual = function (name) {
            return this.type === types.name && this.value === name && !this.containsEsc;
        };
        // Consumes contextual keyword if possible.
        pp.eatContextual = function (name) {
            if (!this.isContextual(name)) {
                return false;
            }
            this.next();
            return true;
        };
        // Asserts that following token is given contextual keyword.
        pp.expectContextual = function (name) {
            if (!this.eatContextual(name)) {
                this.unexpected();
            }
        };
        // Test whether a semicolon can be inserted at the current position.
        pp.canInsertSemicolon = function () {
            return this.type === types.eof ||
                this.type === types.braceR ||
                lineBreak.test(this.input.slice(this.lastTokEnd, this.start));
        };
        pp.insertSemicolon = function () {
            if (this.canInsertSemicolon()) {
                if (this.options.onInsertedSemicolon) {
                    this.options.onInsertedSemicolon(this.lastTokEnd, this.lastTokEndLoc);
                }
                return true;
            }
        };
        // Consume a semicolon, or, failing that, see if we are allowed to
        // pretend that there is a semicolon at this position.
        pp.semicolon = function () {
            if (!this.eat(types.semi) && !this.insertSemicolon()) {
                this.unexpected();
            }
        };
        pp.afterTrailingComma = function (tokType, notNext) {
            if (this.type === tokType) {
                if (this.options.onTrailingComma) {
                    this.options.onTrailingComma(this.lastTokStart, this.lastTokStartLoc);
                }
                if (!notNext) {
                    this.next();
                }
                return true;
            }
        };
        // Expect a token of a given type. If found, consume it, otherwise,
        // raise an unexpected token error.
        pp.expect = function (type) {
            this.eat(type) || this.unexpected();
        };
        // Raise an unexpected token error.
        pp.unexpected = function (pos) {
            this.raise(pos != null ? pos : this.start, "Unexpected token");
        };
        function DestructuringErrors() {
            this.shorthandAssign =
                this.trailingComma =
                    this.parenthesizedAssign =
                        this.parenthesizedBind =
                            this.doubleProto =
                                -1;
        }
        pp.checkPatternErrors = function (refDestructuringErrors, isAssign) {
            if (!refDestructuringErrors) {
                return;
            }
            if (refDestructuringErrors.trailingComma > -1) {
                this.raiseRecoverable(refDestructuringErrors.trailingComma, "Comma is not permitted after the rest element");
            }
            var parens = isAssign ? refDestructuringErrors.parenthesizedAssign : refDestructuringErrors.parenthesizedBind;
            if (parens > -1) {
                this.raiseRecoverable(parens, "Parenthesized pattern");
            }
        };
        pp.checkExpressionErrors = function (refDestructuringErrors, andThrow) {
            if (!refDestructuringErrors) {
                return false;
            }
            var shorthandAssign = refDestructuringErrors.shorthandAssign;
            var doubleProto = refDestructuringErrors.doubleProto;
            if (!andThrow) {
                return shorthandAssign >= 0 || doubleProto >= 0;
            }
            if (shorthandAssign >= 0) {
                this.raise(shorthandAssign, "Shorthand property assignments are valid only in destructuring patterns");
            }
            if (doubleProto >= 0) {
                this.raiseRecoverable(doubleProto, "Redefinition of __proto__ property");
            }
        };
        pp.checkYieldAwaitInDefaultParams = function () {
            if (this.yieldPos && (!this.awaitPos || this.yieldPos < this.awaitPos)) {
                this.raise(this.yieldPos, "Yield expression cannot be a default value");
            }
            if (this.awaitPos) {
                this.raise(this.awaitPos, "Await expression cannot be a default value");
            }
        };
        pp.isSimpleAssignTarget = function (expr) {
            if (expr.type === "ParenthesizedExpression") {
                return this.isSimpleAssignTarget(expr.expression);
            }
            return expr.type === "Identifier" || expr.type === "MemberExpression";
        };
        var pp$1 = Parser.prototype;
        // ### Statement parsing
        // Parse a program. Initializes the parser, reads any number of
        // statements, and wraps them in a Program node.  Optionally takes a
        // `program` argument.  If present, the statements will be appended
        // to its body instead of creating a new node.
        pp$1.parseTopLevel = function (node) {
            var exports = {};
            if (!node.body) {
                node.body = [];
            }
            while (this.type !== types.eof) {
                var stmt = this.parseStatement(null, true, exports);
                node.body.push(stmt);
            }
            if (this.inModule) {
                for (var i = 0, list = Object.keys(this.undefinedExports); i < list.length; i += 1) {
                    var name = list[i];
                    this.raiseRecoverable(this.undefinedExports[name].start, ("Export '" + name + "' is not defined"));
                }
            }
            this.adaptDirectivePrologue(node.body);
            this.next();
            node.sourceType = this.options.sourceType;
            return this.finishNode(node, "Program");
        };
        var loopLabel = { kind: "loop" }, switchLabel = { kind: "switch" };
        pp$1.isLet = function (context) {
            if (this.options.ecmaVersion < 6 || !this.isContextual("let")) {
                return false;
            }
            skipWhiteSpace.lastIndex = this.pos;
            var skip = skipWhiteSpace.exec(this.input);
            var next = this.pos + skip[0].length, nextCh = this.input.charCodeAt(next);
            // For ambiguous cases, determine if a LexicalDeclaration (or only a
            // Statement) is allowed here. If context is not empty then only a Statement
            // is allowed. However, `let [` is an explicit negative lookahead for
            // ExpressionStatement, so special-case it first.
            if (nextCh === 91) {
                return true;
            } // '['
            if (context) {
                return false;
            }
            if (nextCh === 123) {
                return true;
            } // '{'
            if (isIdentifierStart(nextCh, true)) {
                var pos = next + 1;
                while (isIdentifierChar(this.input.charCodeAt(pos), true)) {
                    ++pos;
                }
                var ident = this.input.slice(next, pos);
                if (!keywordRelationalOperator.test(ident)) {
                    return true;
                }
            }
            return false;
        };
        // check 'async [no LineTerminator here] function'
        // - 'async /*foo*/ function' is OK.
        // - 'async /*\n*/ function' is invalid.
        pp$1.isAsyncFunction = function () {
            if (this.options.ecmaVersion < 8 || !this.isContextual("async")) {
                return false;
            }
            skipWhiteSpace.lastIndex = this.pos;
            var skip = skipWhiteSpace.exec(this.input);
            var next = this.pos + skip[0].length;
            return !lineBreak.test(this.input.slice(this.pos, next)) &&
                this.input.slice(next, next + 8) === "function" &&
                (next + 8 === this.input.length || !isIdentifierChar(this.input.charAt(next + 8)));
        };
        // Parse a single statement.
        //
        // If expecting a statement and finding a slash operator, parse a
        // regular expression literal. This is to handle cases like
        // `if (foo) /blah/.exec(foo)`, where looking at the previous token
        // does not help.
        pp$1.parseStatement = function (context, topLevel, exports) {
            var starttype = this.type, node = this.startNode(), kind;
            if (this.isLet(context)) {
                starttype = types._var;
                kind = "let";
            }
            // Most types of statements are recognized by the keyword they
            // start with. Many are trivial to parse, some require a bit of
            // complexity.
            switch (starttype) {
                case types._break:
                case types._continue: return this.parseBreakContinueStatement(node, starttype.keyword);
                case types._debugger: return this.parseDebuggerStatement(node);
                case types._do: return this.parseDoStatement(node);
                case types._for: return this.parseForStatement(node);
                case types._function:
                    // Function as sole body of either an if statement or a labeled statement
                    // works, but not when it is part of a labeled statement that is the sole
                    // body of an if statement.
                    if ((context && (this.strict || context !== "if" && context !== "label")) && this.options.ecmaVersion >= 6) {
                        this.unexpected();
                    }
                    return this.parseFunctionStatement(node, false, !context);
                case types._class:
                    if (context) {
                        this.unexpected();
                    }
                    return this.parseClass(node, true);
                case types._if: return this.parseIfStatement(node);
                case types._return: return this.parseReturnStatement(node);
                case types._switch: return this.parseSwitchStatement(node);
                case types._throw: return this.parseThrowStatement(node);
                case types._try: return this.parseTryStatement(node);
                case types._const:
                case types._var:
                    kind = kind || this.value;
                    if (context && kind !== "var") {
                        this.unexpected();
                    }
                    return this.parseVarStatement(node, kind);
                case types._while: return this.parseWhileStatement(node);
                case types._with: return this.parseWithStatement(node);
                case types.braceL: return this.parseBlock(true, node);
                case types.semi: return this.parseEmptyStatement(node);
                case types._export:
                case types._import:
                    if (this.options.ecmaVersion > 10 && starttype === types._import) {
                        skipWhiteSpace.lastIndex = this.pos;
                        var skip = skipWhiteSpace.exec(this.input);
                        var next = this.pos + skip[0].length, nextCh = this.input.charCodeAt(next);
                        if (nextCh === 40) // '('
                         {
                            return this.parseExpressionStatement(node, this.parseExpression());
                        }
                    }
                    if (!this.options.allowImportExportEverywhere) {
                        if (!topLevel) {
                            this.raise(this.start, "'import' and 'export' may only appear at the top level");
                        }
                        if (!this.inModule) {
                            this.raise(this.start, "'import' and 'export' may appear only with 'sourceType: module'");
                        }
                    }
                    return starttype === types._import ? this.parseImport(node) : this.parseExport(node, exports);
                // If the statement does not start with a statement keyword or a
                // brace, it's an ExpressionStatement or LabeledStatement. We
                // simply start parsing an expression, and afterwards, if the
                // next token is a colon and the expression was a simple
                // Identifier node, we switch to interpreting it as a label.
                default:
                    if (this.isAsyncFunction()) {
                        if (context) {
                            this.unexpected();
                        }
                        this.next();
                        return this.parseFunctionStatement(node, true, !context);
                    }
                    var maybeName = this.value, expr = this.parseExpression();
                    if (starttype === types.name && expr.type === "Identifier" && this.eat(types.colon)) {
                        return this.parseLabeledStatement(node, maybeName, expr, context);
                    }
                    else {
                        return this.parseExpressionStatement(node, expr);
                    }
            }
        };
        pp$1.parseBreakContinueStatement = function (node, keyword) {
            var isBreak = keyword === "break";
            this.next();
            if (this.eat(types.semi) || this.insertSemicolon()) {
                node.label = null;
            }
            else if (this.type !== types.name) {
                this.unexpected();
            }
            else {
                node.label = this.parseIdent();
                this.semicolon();
            }
            // Verify that there is an actual destination to break or
            // continue to.
            var i = 0;
            for (; i < this.labels.length; ++i) {
                var lab = this.labels[i];
                if (node.label == null || lab.name === node.label.name) {
                    if (lab.kind != null && (isBreak || lab.kind === "loop")) {
                        break;
                    }
                    if (node.label && isBreak) {
                        break;
                    }
                }
            }
            if (i === this.labels.length) {
                this.raise(node.start, "Unsyntactic " + keyword);
            }
            return this.finishNode(node, isBreak ? "BreakStatement" : "ContinueStatement");
        };
        pp$1.parseDebuggerStatement = function (node) {
            this.next();
            this.semicolon();
            return this.finishNode(node, "DebuggerStatement");
        };
        pp$1.parseDoStatement = function (node) {
            this.next();
            this.labels.push(loopLabel);
            node.body = this.parseStatement("do");
            this.labels.pop();
            this.expect(types._while);
            node.test = this.parseParenExpression();
            if (this.options.ecmaVersion >= 6) {
                this.eat(types.semi);
            }
            else {
                this.semicolon();
            }
            return this.finishNode(node, "DoWhileStatement");
        };
        // Disambiguating between a `for` and a `for`/`in` or `for`/`of`
        // loop is non-trivial. Basically, we have to parse the init `var`
        // statement or expression, disallowing the `in` operator (see
        // the second parameter to `parseExpression`), and then check
        // whether the next token is `in` or `of`. When there is no init
        // part (semicolon immediately after the opening parenthesis), it
        // is a regular `for` loop.
        pp$1.parseForStatement = function (node) {
            this.next();
            var awaitAt = (this.options.ecmaVersion >= 9 && (this.inAsync || (!this.inFunction && this.options.allowAwaitOutsideFunction)) && this.eatContextual("await")) ? this.lastTokStart : -1;
            this.labels.push(loopLabel);
            this.enterScope(0);
            this.expect(types.parenL);
            if (this.type === types.semi) {
                if (awaitAt > -1) {
                    this.unexpected(awaitAt);
                }
                return this.parseFor(node, null);
            }
            var isLet = this.isLet();
            if (this.type === types._var || this.type === types._const || isLet) {
                var init$1 = this.startNode(), kind = isLet ? "let" : this.value;
                this.next();
                this.parseVar(init$1, true, kind);
                this.finishNode(init$1, "VariableDeclaration");
                if ((this.type === types._in || (this.options.ecmaVersion >= 6 && this.isContextual("of"))) && init$1.declarations.length === 1) {
                    if (this.options.ecmaVersion >= 9) {
                        if (this.type === types._in) {
                            if (awaitAt > -1) {
                                this.unexpected(awaitAt);
                            }
                        }
                        else {
                            node.await = awaitAt > -1;
                        }
                    }
                    return this.parseForIn(node, init$1);
                }
                if (awaitAt > -1) {
                    this.unexpected(awaitAt);
                }
                return this.parseFor(node, init$1);
            }
            var refDestructuringErrors = new DestructuringErrors;
            var init = this.parseExpression(true, refDestructuringErrors);
            if (this.type === types._in || (this.options.ecmaVersion >= 6 && this.isContextual("of"))) {
                if (this.options.ecmaVersion >= 9) {
                    if (this.type === types._in) {
                        if (awaitAt > -1) {
                            this.unexpected(awaitAt);
                        }
                    }
                    else {
                        node.await = awaitAt > -1;
                    }
                }
                this.toAssignable(init, false, refDestructuringErrors);
                this.checkLVal(init);
                return this.parseForIn(node, init);
            }
            else {
                this.checkExpressionErrors(refDestructuringErrors, true);
            }
            if (awaitAt > -1) {
                this.unexpected(awaitAt);
            }
            return this.parseFor(node, init);
        };
        pp$1.parseFunctionStatement = function (node, isAsync, declarationPosition) {
            this.next();
            return this.parseFunction(node, FUNC_STATEMENT | (declarationPosition ? 0 : FUNC_HANGING_STATEMENT), false, isAsync);
        };
        pp$1.parseIfStatement = function (node) {
            this.next();
            node.test = this.parseParenExpression();
            // allow function declarations in branches, but only in non-strict mode
            node.consequent = this.parseStatement("if");
            node.alternate = this.eat(types._else) ? this.parseStatement("if") : null;
            return this.finishNode(node, "IfStatement");
        };
        pp$1.parseReturnStatement = function (node) {
            if (!this.inFunction && !this.options.allowReturnOutsideFunction) {
                this.raise(this.start, "'return' outside of function");
            }
            this.next();
            // In `return` (and `break`/`continue`), the keywords with
            // optional arguments, we eagerly look for a semicolon or the
            // possibility to insert one.
            if (this.eat(types.semi) || this.insertSemicolon()) {
                node.argument = null;
            }
            else {
                node.argument = this.parseExpression();
                this.semicolon();
            }
            return this.finishNode(node, "ReturnStatement");
        };
        pp$1.parseSwitchStatement = function (node) {
            this.next();
            node.discriminant = this.parseParenExpression();
            node.cases = [];
            this.expect(types.braceL);
            this.labels.push(switchLabel);
            this.enterScope(0);
            // Statements under must be grouped (by label) in SwitchCase
            // nodes. `cur` is used to keep the node that we are currently
            // adding statements to.
            var cur;
            for (var sawDefault = false; this.type !== types.braceR;) {
                if (this.type === types._case || this.type === types._default) {
                    var isCase = this.type === types._case;
                    if (cur) {
                        this.finishNode(cur, "SwitchCase");
                    }
                    node.cases.push(cur = this.startNode());
                    cur.consequent = [];
                    this.next();
                    if (isCase) {
                        cur.test = this.parseExpression();
                    }
                    else {
                        if (sawDefault) {
                            this.raiseRecoverable(this.lastTokStart, "Multiple default clauses");
                        }
                        sawDefault = true;
                        cur.test = null;
                    }
                    this.expect(types.colon);
                }
                else {
                    if (!cur) {
                        this.unexpected();
                    }
                    cur.consequent.push(this.parseStatement(null));
                }
            }
            this.exitScope();
            if (cur) {
                this.finishNode(cur, "SwitchCase");
            }
            this.next(); // Closing brace
            this.labels.pop();
            return this.finishNode(node, "SwitchStatement");
        };
        pp$1.parseThrowStatement = function (node) {
            this.next();
            if (lineBreak.test(this.input.slice(this.lastTokEnd, this.start))) {
                this.raise(this.lastTokEnd, "Illegal newline after throw");
            }
            node.argument = this.parseExpression();
            this.semicolon();
            return this.finishNode(node, "ThrowStatement");
        };
        // Reused empty array added for node fields that are always empty.
        var empty = [];
        pp$1.parseTryStatement = function (node) {
            this.next();
            node.block = this.parseBlock();
            node.handler = null;
            if (this.type === types._catch) {
                var clause = this.startNode();
                this.next();
                if (this.eat(types.parenL)) {
                    clause.param = this.parseBindingAtom();
                    var simple = clause.param.type === "Identifier";
                    this.enterScope(simple ? SCOPE_SIMPLE_CATCH : 0);
                    this.checkLVal(clause.param, simple ? BIND_SIMPLE_CATCH : BIND_LEXICAL);
                    this.expect(types.parenR);
                }
                else {
                    if (this.options.ecmaVersion < 10) {
                        this.unexpected();
                    }
                    clause.param = null;
                    this.enterScope(0);
                }
                clause.body = this.parseBlock(false);
                this.exitScope();
                node.handler = this.finishNode(clause, "CatchClause");
            }
            node.finalizer = this.eat(types._finally) ? this.parseBlock() : null;
            if (!node.handler && !node.finalizer) {
                this.raise(node.start, "Missing catch or finally clause");
            }
            return this.finishNode(node, "TryStatement");
        };
        pp$1.parseVarStatement = function (node, kind) {
            this.next();
            this.parseVar(node, false, kind);
            this.semicolon();
            return this.finishNode(node, "VariableDeclaration");
        };
        pp$1.parseWhileStatement = function (node) {
            this.next();
            node.test = this.parseParenExpression();
            this.labels.push(loopLabel);
            node.body = this.parseStatement("while");
            this.labels.pop();
            return this.finishNode(node, "WhileStatement");
        };
        pp$1.parseWithStatement = function (node) {
            if (this.strict) {
                this.raise(this.start, "'with' in strict mode");
            }
            this.next();
            node.object = this.parseParenExpression();
            node.body = this.parseStatement("with");
            return this.finishNode(node, "WithStatement");
        };
        pp$1.parseEmptyStatement = function (node) {
            this.next();
            return this.finishNode(node, "EmptyStatement");
        };
        pp$1.parseLabeledStatement = function (node, maybeName, expr, context) {
            for (var i$1 = 0, list = this.labels; i$1 < list.length; i$1 += 1) {
                var label = list[i$1];
                if (label.name === maybeName) {
                    this.raise(expr.start, "Label '" + maybeName + "' is already declared");
                }
            }
            var kind = this.type.isLoop ? "loop" : this.type === types._switch ? "switch" : null;
            for (var i = this.labels.length - 1; i >= 0; i--) {
                var label$1 = this.labels[i];
                if (label$1.statementStart === node.start) {
                    // Update information about previous labels on this node
                    label$1.statementStart = this.start;
                    label$1.kind = kind;
                }
                else {
                    break;
                }
            }
            this.labels.push({ name: maybeName, kind: kind, statementStart: this.start });
            node.body = this.parseStatement(context ? context.indexOf("label") === -1 ? context + "label" : context : "label");
            this.labels.pop();
            node.label = expr;
            return this.finishNode(node, "LabeledStatement");
        };
        pp$1.parseExpressionStatement = function (node, expr) {
            node.expression = expr;
            this.semicolon();
            return this.finishNode(node, "ExpressionStatement");
        };
        // Parse a semicolon-enclosed block of statements, handling `"use
        // strict"` declarations when `allowStrict` is true (used for
        // function bodies).
        pp$1.parseBlock = function (createNewLexicalScope, node) {
            if (createNewLexicalScope === void 0)
                createNewLexicalScope = true;
            if (node === void 0)
                node = this.startNode();
            node.body = [];
            this.expect(types.braceL);
            if (createNewLexicalScope) {
                this.enterScope(0);
            }
            while (!this.eat(types.braceR)) {
                var stmt = this.parseStatement(null);
                node.body.push(stmt);
            }
            if (createNewLexicalScope) {
                this.exitScope();
            }
            return this.finishNode(node, "BlockStatement");
        };
        // Parse a regular `for` loop. The disambiguation code in
        // `parseStatement` will already have parsed the init statement or
        // expression.
        pp$1.parseFor = function (node, init) {
            node.init = init;
            this.expect(types.semi);
            node.test = this.type === types.semi ? null : this.parseExpression();
            this.expect(types.semi);
            node.update = this.type === types.parenR ? null : this.parseExpression();
            this.expect(types.parenR);
            node.body = this.parseStatement("for");
            this.exitScope();
            this.labels.pop();
            return this.finishNode(node, "ForStatement");
        };
        // Parse a `for`/`in` and `for`/`of` loop, which are almost
        // same from parser's perspective.
        pp$1.parseForIn = function (node, init) {
            var isForIn = this.type === types._in;
            this.next();
            if (init.type === "VariableDeclaration" &&
                init.declarations[0].init != null &&
                (!isForIn ||
                    this.options.ecmaVersion < 8 ||
                    this.strict ||
                    init.kind !== "var" ||
                    init.declarations[0].id.type !== "Identifier")) {
                this.raise(init.start, ((isForIn ? "for-in" : "for-of") + " loop variable declaration may not have an initializer"));
            }
            else if (init.type === "AssignmentPattern") {
                this.raise(init.start, "Invalid left-hand side in for-loop");
            }
            node.left = init;
            node.right = isForIn ? this.parseExpression() : this.parseMaybeAssign();
            this.expect(types.parenR);
            node.body = this.parseStatement("for");
            this.exitScope();
            this.labels.pop();
            return this.finishNode(node, isForIn ? "ForInStatement" : "ForOfStatement");
        };
        // Parse a list of variable declarations.
        pp$1.parseVar = function (node, isFor, kind) {
            node.declarations = [];
            node.kind = kind;
            for (;;) {
                var decl = this.startNode();
                this.parseVarId(decl, kind);
                if (this.eat(types.eq)) {
                    decl.init = this.parseMaybeAssign(isFor);
                }
                else if (kind === "const" && !(this.type === types._in || (this.options.ecmaVersion >= 6 && this.isContextual("of")))) {
                    this.unexpected();
                }
                else if (decl.id.type !== "Identifier" && !(isFor && (this.type === types._in || this.isContextual("of")))) {
                    this.raise(this.lastTokEnd, "Complex binding patterns require an initialization value");
                }
                else {
                    decl.init = null;
                }
                node.declarations.push(this.finishNode(decl, "VariableDeclarator"));
                if (!this.eat(types.comma)) {
                    break;
                }
            }
            return node;
        };
        pp$1.parseVarId = function (decl, kind) {
            decl.id = this.parseBindingAtom();
            this.checkLVal(decl.id, kind === "var" ? BIND_VAR : BIND_LEXICAL, false);
        };
        var FUNC_STATEMENT = 1, FUNC_HANGING_STATEMENT = 2, FUNC_NULLABLE_ID = 4;
        // Parse a function declaration or literal (depending on the
        // `statement & FUNC_STATEMENT`).
        // Remove `allowExpressionBody` for 7.0.0, as it is only called with false
        pp$1.parseFunction = function (node, statement, allowExpressionBody, isAsync) {
            this.initFunction(node);
            if (this.options.ecmaVersion >= 9 || this.options.ecmaVersion >= 6 && !isAsync) {
                if (this.type === types.star && (statement & FUNC_HANGING_STATEMENT)) {
                    this.unexpected();
                }
                node.generator = this.eat(types.star);
            }
            if (this.options.ecmaVersion >= 8) {
                node.async = !!isAsync;
            }
            if (statement & FUNC_STATEMENT) {
                node.id = (statement & FUNC_NULLABLE_ID) && this.type !== types.name ? null : this.parseIdent();
                if (node.id && !(statement & FUNC_HANGING_STATEMENT)) 
                // If it is a regular function declaration in sloppy mode, then it is
                // subject to Annex B semantics (BIND_FUNCTION). Otherwise, the binding
                // mode depends on properties of the current scope (see
                // treatFunctionsAsVar).
                {
                    this.checkLVal(node.id, (this.strict || node.generator || node.async) ? this.treatFunctionsAsVar ? BIND_VAR : BIND_LEXICAL : BIND_FUNCTION);
                }
            }
            var oldYieldPos = this.yieldPos, oldAwaitPos = this.awaitPos, oldAwaitIdentPos = this.awaitIdentPos;
            this.yieldPos = 0;
            this.awaitPos = 0;
            this.awaitIdentPos = 0;
            this.enterScope(functionFlags(node.async, node.generator));
            if (!(statement & FUNC_STATEMENT)) {
                node.id = this.type === types.name ? this.parseIdent() : null;
            }
            this.parseFunctionParams(node);
            this.parseFunctionBody(node, allowExpressionBody, false);
            this.yieldPos = oldYieldPos;
            this.awaitPos = oldAwaitPos;
            this.awaitIdentPos = oldAwaitIdentPos;
            return this.finishNode(node, (statement & FUNC_STATEMENT) ? "FunctionDeclaration" : "FunctionExpression");
        };
        pp$1.parseFunctionParams = function (node) {
            this.expect(types.parenL);
            node.params = this.parseBindingList(types.parenR, false, this.options.ecmaVersion >= 8);
            this.checkYieldAwaitInDefaultParams();
        };
        // Parse a class declaration or literal (depending on the
        // `isStatement` parameter).
        pp$1.parseClass = function (node, isStatement) {
            this.next();
            // ecma-262 14.6 Class Definitions
            // A class definition is always strict mode code.
            var oldStrict = this.strict;
            this.strict = true;
            this.parseClassId(node, isStatement);
            this.parseClassSuper(node);
            var classBody = this.startNode();
            var hadConstructor = false;
            classBody.body = [];
            this.expect(types.braceL);
            while (!this.eat(types.braceR)) {
                var element = this.parseClassElement(node.superClass !== null);
                if (element) {
                    classBody.body.push(element);
                    if (element.type === "MethodDefinition" && element.kind === "constructor") {
                        if (hadConstructor) {
                            this.raise(element.start, "Duplicate constructor in the same class");
                        }
                        hadConstructor = true;
                    }
                }
            }
            node.body = this.finishNode(classBody, "ClassBody");
            this.strict = oldStrict;
            return this.finishNode(node, isStatement ? "ClassDeclaration" : "ClassExpression");
        };
        pp$1.parseClassElement = function (constructorAllowsSuper) {
            var this$1 = this;
            if (this.eat(types.semi)) {
                return null;
            }
            var method = this.startNode();
            var tryContextual = function (k, noLineBreak) {
                if (noLineBreak === void 0)
                    noLineBreak = false;
                var start = this$1.start, startLoc = this$1.startLoc;
                if (!this$1.eatContextual(k)) {
                    return false;
                }
                if (this$1.type !== types.parenL && (!noLineBreak || !this$1.canInsertSemicolon())) {
                    return true;
                }
                if (method.key) {
                    this$1.unexpected();
                }
                method.computed = false;
                method.key = this$1.startNodeAt(start, startLoc);
                method.key.name = k;
                this$1.finishNode(method.key, "Identifier");
                return false;
            };
            method.kind = "method";
            method.static = tryContextual("static");
            var isGenerator = this.eat(types.star);
            var isAsync = false;
            if (!isGenerator) {
                if (this.options.ecmaVersion >= 8 && tryContextual("async", true)) {
                    isAsync = true;
                    isGenerator = this.options.ecmaVersion >= 9 && this.eat(types.star);
                }
                else if (tryContextual("get")) {
                    method.kind = "get";
                }
                else if (tryContextual("set")) {
                    method.kind = "set";
                }
            }
            if (!method.key) {
                this.parsePropertyName(method);
            }
            var key = method.key;
            var allowsDirectSuper = false;
            if (!method.computed && !method.static && (key.type === "Identifier" && key.name === "constructor" ||
                key.type === "Literal" && key.value === "constructor")) {
                if (method.kind !== "method") {
                    this.raise(key.start, "Constructor can't have get/set modifier");
                }
                if (isGenerator) {
                    this.raise(key.start, "Constructor can't be a generator");
                }
                if (isAsync) {
                    this.raise(key.start, "Constructor can't be an async method");
                }
                method.kind = "constructor";
                allowsDirectSuper = constructorAllowsSuper;
            }
            else if (method.static && key.type === "Identifier" && key.name === "prototype") {
                this.raise(key.start, "Classes may not have a static property named prototype");
            }
            this.parseClassMethod(method, isGenerator, isAsync, allowsDirectSuper);
            if (method.kind === "get" && method.value.params.length !== 0) {
                this.raiseRecoverable(method.value.start, "getter should have no params");
            }
            if (method.kind === "set" && method.value.params.length !== 1) {
                this.raiseRecoverable(method.value.start, "setter should have exactly one param");
            }
            if (method.kind === "set" && method.value.params[0].type === "RestElement") {
                this.raiseRecoverable(method.value.params[0].start, "Setter cannot use rest params");
            }
            return method;
        };
        pp$1.parseClassMethod = function (method, isGenerator, isAsync, allowsDirectSuper) {
            method.value = this.parseMethod(isGenerator, isAsync, allowsDirectSuper);
            return this.finishNode(method, "MethodDefinition");
        };
        pp$1.parseClassId = function (node, isStatement) {
            if (this.type === types.name) {
                node.id = this.parseIdent();
                if (isStatement) {
                    this.checkLVal(node.id, BIND_LEXICAL, false);
                }
            }
            else {
                if (isStatement === true) {
                    this.unexpected();
                }
                node.id = null;
            }
        };
        pp$1.parseClassSuper = function (node) {
            node.superClass = this.eat(types._extends) ? this.parseExprSubscripts() : null;
        };
        // Parses module export declaration.
        pp$1.parseExport = function (node, exports) {
            this.next();
            // export * from '...'
            if (this.eat(types.star)) {
                this.expectContextual("from");
                if (this.type !== types.string) {
                    this.unexpected();
                }
                node.source = this.parseExprAtom();
                this.semicolon();
                return this.finishNode(node, "ExportAllDeclaration");
            }
            if (this.eat(types._default)) { // export default ...
                this.checkExport(exports, "default", this.lastTokStart);
                var isAsync;
                if (this.type === types._function || (isAsync = this.isAsyncFunction())) {
                    var fNode = this.startNode();
                    this.next();
                    if (isAsync) {
                        this.next();
                    }
                    node.declaration = this.parseFunction(fNode, FUNC_STATEMENT | FUNC_NULLABLE_ID, false, isAsync);
                }
                else if (this.type === types._class) {
                    var cNode = this.startNode();
                    node.declaration = this.parseClass(cNode, "nullableID");
                }
                else {
                    node.declaration = this.parseMaybeAssign();
                    this.semicolon();
                }
                return this.finishNode(node, "ExportDefaultDeclaration");
            }
            // export var|const|let|function|class ...
            if (this.shouldParseExportStatement()) {
                node.declaration = this.parseStatement(null);
                if (node.declaration.type === "VariableDeclaration") {
                    this.checkVariableExport(exports, node.declaration.declarations);
                }
                else {
                    this.checkExport(exports, node.declaration.id.name, node.declaration.id.start);
                }
                node.specifiers = [];
                node.source = null;
            }
            else { // export { x, y as z } [from '...']
                node.declaration = null;
                node.specifiers = this.parseExportSpecifiers(exports);
                if (this.eatContextual("from")) {
                    if (this.type !== types.string) {
                        this.unexpected();
                    }
                    node.source = this.parseExprAtom();
                }
                else {
                    for (var i = 0, list = node.specifiers; i < list.length; i += 1) {
                        // check for keywords used as local names
                        var spec = list[i];
                        this.checkUnreserved(spec.local);
                        // check if export is defined
                        this.checkLocalExport(spec.local);
                    }
                    node.source = null;
                }
                this.semicolon();
            }
            return this.finishNode(node, "ExportNamedDeclaration");
        };
        pp$1.checkExport = function (exports, name, pos) {
            if (!exports) {
                return;
            }
            if (has(exports, name)) {
                this.raiseRecoverable(pos, "Duplicate export '" + name + "'");
            }
            exports[name] = true;
        };
        pp$1.checkPatternExport = function (exports, pat) {
            var type = pat.type;
            if (type === "Identifier") {
                this.checkExport(exports, pat.name, pat.start);
            }
            else if (type === "ObjectPattern") {
                for (var i = 0, list = pat.properties; i < list.length; i += 1) {
                    var prop = list[i];
                    this.checkPatternExport(exports, prop);
                }
            }
            else if (type === "ArrayPattern") {
                for (var i$1 = 0, list$1 = pat.elements; i$1 < list$1.length; i$1 += 1) {
                    var elt = list$1[i$1];
                    if (elt) {
                        this.checkPatternExport(exports, elt);
                    }
                }
            }
            else if (type === "Property") {
                this.checkPatternExport(exports, pat.value);
            }
            else if (type === "AssignmentPattern") {
                this.checkPatternExport(exports, pat.left);
            }
            else if (type === "RestElement") {
                this.checkPatternExport(exports, pat.argument);
            }
            else if (type === "ParenthesizedExpression") {
                this.checkPatternExport(exports, pat.expression);
            }
        };
        pp$1.checkVariableExport = function (exports, decls) {
            if (!exports) {
                return;
            }
            for (var i = 0, list = decls; i < list.length; i += 1) {
                var decl = list[i];
                this.checkPatternExport(exports, decl.id);
            }
        };
        pp$1.shouldParseExportStatement = function () {
            return this.type.keyword === "var" ||
                this.type.keyword === "const" ||
                this.type.keyword === "class" ||
                this.type.keyword === "function" ||
                this.isLet() ||
                this.isAsyncFunction();
        };
        // Parses a comma-separated list of module exports.
        pp$1.parseExportSpecifiers = function (exports) {
            var nodes = [], first = true;
            // export { x, y as z } [from '...']
            this.expect(types.braceL);
            while (!this.eat(types.braceR)) {
                if (!first) {
                    this.expect(types.comma);
                    if (this.afterTrailingComma(types.braceR)) {
                        break;
                    }
                }
                else {
                    first = false;
                }
                var node = this.startNode();
                node.local = this.parseIdent(true);
                node.exported = this.eatContextual("as") ? this.parseIdent(true) : node.local;
                this.checkExport(exports, node.exported.name, node.exported.start);
                nodes.push(this.finishNode(node, "ExportSpecifier"));
            }
            return nodes;
        };
        // Parses import declaration.
        pp$1.parseImport = function (node) {
            this.next();
            // import '...'
            if (this.type === types.string) {
                node.specifiers = empty;
                node.source = this.parseExprAtom();
            }
            else {
                node.specifiers = this.parseImportSpecifiers();
                this.expectContextual("from");
                node.source = this.type === types.string ? this.parseExprAtom() : this.unexpected();
            }
            this.semicolon();
            return this.finishNode(node, "ImportDeclaration");
        };
        // Parses a comma-separated list of module imports.
        pp$1.parseImportSpecifiers = function () {
            var nodes = [], first = true;
            if (this.type === types.name) {
                // import defaultObj, { x, y as z } from '...'
                var node = this.startNode();
                node.local = this.parseIdent();
                this.checkLVal(node.local, BIND_LEXICAL);
                nodes.push(this.finishNode(node, "ImportDefaultSpecifier"));
                if (!this.eat(types.comma)) {
                    return nodes;
                }
            }
            if (this.type === types.star) {
                var node$1 = this.startNode();
                this.next();
                this.expectContextual("as");
                node$1.local = this.parseIdent();
                this.checkLVal(node$1.local, BIND_LEXICAL);
                nodes.push(this.finishNode(node$1, "ImportNamespaceSpecifier"));
                return nodes;
            }
            this.expect(types.braceL);
            while (!this.eat(types.braceR)) {
                if (!first) {
                    this.expect(types.comma);
                    if (this.afterTrailingComma(types.braceR)) {
                        break;
                    }
                }
                else {
                    first = false;
                }
                var node$2 = this.startNode();
                node$2.imported = this.parseIdent(true);
                if (this.eatContextual("as")) {
                    node$2.local = this.parseIdent();
                }
                else {
                    this.checkUnreserved(node$2.imported);
                    node$2.local = node$2.imported;
                }
                this.checkLVal(node$2.local, BIND_LEXICAL);
                nodes.push(this.finishNode(node$2, "ImportSpecifier"));
            }
            return nodes;
        };
        // Set `ExpressionStatement#directive` property for directive prologues.
        pp$1.adaptDirectivePrologue = function (statements) {
            for (var i = 0; i < statements.length && this.isDirectiveCandidate(statements[i]); ++i) {
                statements[i].directive = statements[i].expression.raw.slice(1, -1);
            }
        };
        pp$1.isDirectiveCandidate = function (statement) {
            return (statement.type === "ExpressionStatement" &&
                statement.expression.type === "Literal" &&
                typeof statement.expression.value === "string" &&
                // Reject parenthesized strings.
                (this.input[statement.start] === "\"" || this.input[statement.start] === "'"));
        };
        var pp$2 = Parser.prototype;
        // Convert existing expression atom to assignable pattern
        // if possible.
        pp$2.toAssignable = function (node, isBinding, refDestructuringErrors) {
            if (this.options.ecmaVersion >= 6 && node) {
                switch (node.type) {
                    case "Identifier":
                        if (this.inAsync && node.name === "await") {
                            this.raise(node.start, "Cannot use 'await' as identifier inside an async function");
                        }
                        break;
                    case "ObjectPattern":
                    case "ArrayPattern":
                    case "RestElement":
                        break;
                    case "ObjectExpression":
                        node.type = "ObjectPattern";
                        if (refDestructuringErrors) {
                            this.checkPatternErrors(refDestructuringErrors, true);
                        }
                        for (var i = 0, list = node.properties; i < list.length; i += 1) {
                            var prop = list[i];
                            this.toAssignable(prop, isBinding);
                            // Early error:
                            //   AssignmentRestProperty[Yield, Await] :
                            //     `...` DestructuringAssignmentTarget[Yield, Await]
                            //
                            //   It is a Syntax Error if |DestructuringAssignmentTarget| is an |ArrayLiteral| or an |ObjectLiteral|.
                            if (prop.type === "RestElement" &&
                                (prop.argument.type === "ArrayPattern" || prop.argument.type === "ObjectPattern")) {
                                this.raise(prop.argument.start, "Unexpected token");
                            }
                        }
                        break;
                    case "Property":
                        // AssignmentProperty has type === "Property"
                        if (node.kind !== "init") {
                            this.raise(node.key.start, "Object pattern can't contain getter or setter");
                        }
                        this.toAssignable(node.value, isBinding);
                        break;
                    case "ArrayExpression":
                        node.type = "ArrayPattern";
                        if (refDestructuringErrors) {
                            this.checkPatternErrors(refDestructuringErrors, true);
                        }
                        this.toAssignableList(node.elements, isBinding);
                        break;
                    case "SpreadElement":
                        node.type = "RestElement";
                        this.toAssignable(node.argument, isBinding);
                        if (node.argument.type === "AssignmentPattern") {
                            this.raise(node.argument.start, "Rest elements cannot have a default value");
                        }
                        break;
                    case "AssignmentExpression":
                        if (node.operator !== "=") {
                            this.raise(node.left.end, "Only '=' operator can be used for specifying default value.");
                        }
                        node.type = "AssignmentPattern";
                        delete node.operator;
                        this.toAssignable(node.left, isBinding);
                    // falls through to AssignmentPattern
                    case "AssignmentPattern":
                        break;
                    case "ParenthesizedExpression":
                        this.toAssignable(node.expression, isBinding, refDestructuringErrors);
                        break;
                    case "MemberExpression":
                        if (!isBinding) {
                            break;
                        }
                    default:
                        this.raise(node.start, "Assigning to rvalue");
                }
            }
            else if (refDestructuringErrors) {
                this.checkPatternErrors(refDestructuringErrors, true);
            }
            return node;
        };
        // Convert list of expression atoms to binding list.
        pp$2.toAssignableList = function (exprList, isBinding) {
            var end = exprList.length;
            for (var i = 0; i < end; i++) {
                var elt = exprList[i];
                if (elt) {
                    this.toAssignable(elt, isBinding);
                }
            }
            if (end) {
                var last = exprList[end - 1];
                if (this.options.ecmaVersion === 6 && isBinding && last && last.type === "RestElement" && last.argument.type !== "Identifier") {
                    this.unexpected(last.argument.start);
                }
            }
            return exprList;
        };
        // Parses spread element.
        pp$2.parseSpread = function (refDestructuringErrors) {
            var node = this.startNode();
            this.next();
            node.argument = this.parseMaybeAssign(false, refDestructuringErrors);
            return this.finishNode(node, "SpreadElement");
        };
        pp$2.parseRestBinding = function () {
            var node = this.startNode();
            this.next();
            // RestElement inside of a function parameter must be an identifier
            if (this.options.ecmaVersion === 6 && this.type !== types.name) {
                this.unexpected();
            }
            node.argument = this.parseBindingAtom();
            return this.finishNode(node, "RestElement");
        };
        // Parses lvalue (assignable) atom.
        pp$2.parseBindingAtom = function () {
            if (this.options.ecmaVersion >= 6) {
                switch (this.type) {
                    case types.bracketL:
                        var node = this.startNode();
                        this.next();
                        node.elements = this.parseBindingList(types.bracketR, true, true);
                        return this.finishNode(node, "ArrayPattern");
                    case types.braceL:
                        return this.parseObj(true);
                }
            }
            return this.parseIdent();
        };
        pp$2.parseBindingList = function (close, allowEmpty, allowTrailingComma) {
            var elts = [], first = true;
            while (!this.eat(close)) {
                if (first) {
                    first = false;
                }
                else {
                    this.expect(types.comma);
                }
                if (allowEmpty && this.type === types.comma) {
                    elts.push(null);
                }
                else if (allowTrailingComma && this.afterTrailingComma(close)) {
                    break;
                }
                else if (this.type === types.ellipsis) {
                    var rest = this.parseRestBinding();
                    this.parseBindingListItem(rest);
                    elts.push(rest);
                    if (this.type === types.comma) {
                        this.raise(this.start, "Comma is not permitted after the rest element");
                    }
                    this.expect(close);
                    break;
                }
                else {
                    var elem = this.parseMaybeDefault(this.start, this.startLoc);
                    this.parseBindingListItem(elem);
                    elts.push(elem);
                }
            }
            return elts;
        };
        pp$2.parseBindingListItem = function (param) {
            return param;
        };
        // Parses assignment pattern around given atom if possible.
        pp$2.parseMaybeDefault = function (startPos, startLoc, left) {
            left = left || this.parseBindingAtom();
            if (this.options.ecmaVersion < 6 || !this.eat(types.eq)) {
                return left;
            }
            var node = this.startNodeAt(startPos, startLoc);
            node.left = left;
            node.right = this.parseMaybeAssign();
            return this.finishNode(node, "AssignmentPattern");
        };
        // Verify that a node is an lval — something that can be assigned
        // to.
        // bindingType can be either:
        // 'var' indicating that the lval creates a 'var' binding
        // 'let' indicating that the lval creates a lexical ('let' or 'const') binding
        // 'none' indicating that the binding should be checked for illegal identifiers, but not for duplicate references
        pp$2.checkLVal = function (expr, bindingType, checkClashes) {
            if (bindingType === void 0)
                bindingType = BIND_NONE;
            switch (expr.type) {
                case "Identifier":
                    if (bindingType === BIND_LEXICAL && expr.name === "let") {
                        this.raiseRecoverable(expr.start, "let is disallowed as a lexically bound name");
                    }
                    if (this.strict && this.reservedWordsStrictBind.test(expr.name)) {
                        this.raiseRecoverable(expr.start, (bindingType ? "Binding " : "Assigning to ") + expr.name + " in strict mode");
                    }
                    if (checkClashes) {
                        if (has(checkClashes, expr.name)) {
                            this.raiseRecoverable(expr.start, "Argument name clash");
                        }
                        checkClashes[expr.name] = true;
                    }
                    if (bindingType !== BIND_NONE && bindingType !== BIND_OUTSIDE) {
                        this.declareName(expr.name, bindingType, expr.start);
                    }
                    break;
                case "MemberExpression":
                    if (bindingType) {
                        this.raiseRecoverable(expr.start, "Binding member expression");
                    }
                    break;
                case "ObjectPattern":
                    for (var i = 0, list = expr.properties; i < list.length; i += 1) {
                        var prop = list[i];
                        this.checkLVal(prop, bindingType, checkClashes);
                    }
                    break;
                case "Property":
                    // AssignmentProperty has type === "Property"
                    this.checkLVal(expr.value, bindingType, checkClashes);
                    break;
                case "ArrayPattern":
                    for (var i$1 = 0, list$1 = expr.elements; i$1 < list$1.length; i$1 += 1) {
                        var elem = list$1[i$1];
                        if (elem) {
                            this.checkLVal(elem, bindingType, checkClashes);
                        }
                    }
                    break;
                case "AssignmentPattern":
                    this.checkLVal(expr.left, bindingType, checkClashes);
                    break;
                case "RestElement":
                    this.checkLVal(expr.argument, bindingType, checkClashes);
                    break;
                case "ParenthesizedExpression":
                    this.checkLVal(expr.expression, bindingType, checkClashes);
                    break;
                default:
                    this.raise(expr.start, (bindingType ? "Binding" : "Assigning to") + " rvalue");
            }
        };
        // A recursive descent parser operates by defining functions for all
        var pp$3 = Parser.prototype;
        // Check if property name clashes with already added.
        // Object/class getters and setters are not allowed to clash —
        // either with each other or with an init property — and in
        // strict mode, init properties are also not allowed to be repeated.
        pp$3.checkPropClash = function (prop, propHash, refDestructuringErrors) {
            if (this.options.ecmaVersion >= 9 && prop.type === "SpreadElement") {
                return;
            }
            if (this.options.ecmaVersion >= 6 && (prop.computed || prop.method || prop.shorthand)) {
                return;
            }
            var key = prop.key;
            var name;
            switch (key.type) {
                case "Identifier":
                    name = key.name;
                    break;
                case "Literal":
                    name = String(key.value);
                    break;
                default: return;
            }
            var kind = prop.kind;
            if (this.options.ecmaVersion >= 6) {
                if (name === "__proto__" && kind === "init") {
                    if (propHash.proto) {
                        if (refDestructuringErrors) {
                            if (refDestructuringErrors.doubleProto < 0) {
                                refDestructuringErrors.doubleProto = key.start;
                            }
                            // Backwards-compat kludge. Can be removed in version 6.0
                        }
                        else {
                            this.raiseRecoverable(key.start, "Redefinition of __proto__ property");
                        }
                    }
                    propHash.proto = true;
                }
                return;
            }
            name = "$" + name;
            var other = propHash[name];
            if (other) {
                var redefinition;
                if (kind === "init") {
                    redefinition = this.strict && other.init || other.get || other.set;
                }
                else {
                    redefinition = other.init || other[kind];
                }
                if (redefinition) {
                    this.raiseRecoverable(key.start, "Redefinition of property");
                }
            }
            else {
                other = propHash[name] = {
                    init: false,
                    get: false,
                    set: false
                };
            }
            other[kind] = true;
        };
        // ### Expression parsing
        // These nest, from the most general expression type at the top to
        // 'atomic', nondivisible expression types at the bottom. Most of
        // the functions will simply let the function(s) below them parse,
        // and, *if* the syntactic construct they handle is present, wrap
        // the AST node that the inner parser gave them in another node.
        // Parse a full expression. The optional arguments are used to
        // forbid the `in` operator (in for loops initalization expressions)
        // and provide reference for storing '=' operator inside shorthand
        // property assignment in contexts where both object expression
        // and object pattern might appear (so it's possible to raise
        // delayed syntax error at correct position).
        pp$3.parseExpression = function (noIn, refDestructuringErrors) {
            var startPos = this.start, startLoc = this.startLoc;
            var expr = this.parseMaybeAssign(noIn, refDestructuringErrors);
            if (this.type === types.comma) {
                var node = this.startNodeAt(startPos, startLoc);
                node.expressions = [expr];
                while (this.eat(types.comma)) {
                    node.expressions.push(this.parseMaybeAssign(noIn, refDestructuringErrors));
                }
                return this.finishNode(node, "SequenceExpression");
            }
            return expr;
        };
        // Parse an assignment expression. This includes applications of
        // operators like `+=`.
        pp$3.parseMaybeAssign = function (noIn, refDestructuringErrors, afterLeftParse) {
            if (this.isContextual("yield")) {
                if (this.inGenerator) {
                    return this.parseYield(noIn);
                }
                // The tokenizer will assume an expression is allowed after
                // `yield`, but this isn't that kind of yield
                else {
                    this.exprAllowed = false;
                }
            }
            var ownDestructuringErrors = false, oldParenAssign = -1, oldTrailingComma = -1;
            if (refDestructuringErrors) {
                oldParenAssign = refDestructuringErrors.parenthesizedAssign;
                oldTrailingComma = refDestructuringErrors.trailingComma;
                refDestructuringErrors.parenthesizedAssign = refDestructuringErrors.trailingComma = -1;
            }
            else {
                refDestructuringErrors = new DestructuringErrors;
                ownDestructuringErrors = true;
            }
            var startPos = this.start, startLoc = this.startLoc;
            if (this.type === types.parenL || this.type === types.name) {
                this.potentialArrowAt = this.start;
            }
            var left = this.parseMaybeConditional(noIn, refDestructuringErrors);
            if (afterLeftParse) {
                left = afterLeftParse.call(this, left, startPos, startLoc);
            }
            if (this.type.isAssign) {
                var node = this.startNodeAt(startPos, startLoc);
                node.operator = this.value;
                node.left = this.type === types.eq ? this.toAssignable(left, false, refDestructuringErrors) : left;
                if (!ownDestructuringErrors) {
                    refDestructuringErrors.parenthesizedAssign = refDestructuringErrors.trailingComma = refDestructuringErrors.doubleProto = -1;
                }
                if (refDestructuringErrors.shorthandAssign >= node.left.start) {
                    refDestructuringErrors.shorthandAssign = -1;
                } // reset because shorthand default was used correctly
                this.checkLVal(left);
                this.next();
                node.right = this.parseMaybeAssign(noIn);
                return this.finishNode(node, "AssignmentExpression");
            }
            else {
                if (ownDestructuringErrors) {
                    this.checkExpressionErrors(refDestructuringErrors, true);
                }
            }
            if (oldParenAssign > -1) {
                refDestructuringErrors.parenthesizedAssign = oldParenAssign;
            }
            if (oldTrailingComma > -1) {
                refDestructuringErrors.trailingComma = oldTrailingComma;
            }
            return left;
        };
        // Parse a ternary conditional (`?:`) operator.
        pp$3.parseMaybeConditional = function (noIn, refDestructuringErrors) {
            var startPos = this.start, startLoc = this.startLoc;
            var expr = this.parseExprOps(noIn, refDestructuringErrors);
            if (this.checkExpressionErrors(refDestructuringErrors)) {
                return expr;
            }
            if (this.eat(types.question)) {
                var node = this.startNodeAt(startPos, startLoc);
                node.test = expr;
                node.consequent = this.parseMaybeAssign();
                this.expect(types.colon);
                node.alternate = this.parseMaybeAssign(noIn);
                return this.finishNode(node, "ConditionalExpression");
            }
            return expr;
        };
        // Start the precedence parser.
        pp$3.parseExprOps = function (noIn, refDestructuringErrors) {
            var startPos = this.start, startLoc = this.startLoc;
            var expr = this.parseMaybeUnary(refDestructuringErrors, false);
            if (this.checkExpressionErrors(refDestructuringErrors)) {
                return expr;
            }
            return expr.start === startPos && expr.type === "ArrowFunctionExpression" ? expr : this.parseExprOp(expr, startPos, startLoc, -1, noIn);
        };
        // Parse binary operators with the operator precedence parsing
        // algorithm. `left` is the left-hand side of the operator.
        // `minPrec` provides context that allows the function to stop and
        // defer further parser to one of its callers when it encounters an
        // operator that has a lower precedence than the set it is parsing.
        pp$3.parseExprOp = function (left, leftStartPos, leftStartLoc, minPrec, noIn) {
            var prec = this.type.binop;
            if (prec != null && (!noIn || this.type !== types._in)) {
                if (prec > minPrec) {
                    var logical = this.type === types.logicalOR || this.type === types.logicalAND;
                    var op = this.value;
                    this.next();
                    var startPos = this.start, startLoc = this.startLoc;
                    var right = this.parseExprOp(this.parseMaybeUnary(null, false), startPos, startLoc, prec, noIn);
                    var node = this.buildBinary(leftStartPos, leftStartLoc, left, right, op, logical);
                    return this.parseExprOp(node, leftStartPos, leftStartLoc, minPrec, noIn);
                }
            }
            return left;
        };
        pp$3.buildBinary = function (startPos, startLoc, left, right, op, logical) {
            var node = this.startNodeAt(startPos, startLoc);
            node.left = left;
            node.operator = op;
            node.right = right;
            return this.finishNode(node, logical ? "LogicalExpression" : "BinaryExpression");
        };
        // Parse unary operators, both prefix and postfix.
        pp$3.parseMaybeUnary = function (refDestructuringErrors, sawUnary) {
            var startPos = this.start, startLoc = this.startLoc, expr;
            if (this.isContextual("await") && (this.inAsync || (!this.inFunction && this.options.allowAwaitOutsideFunction))) {
                expr = this.parseAwait();
                sawUnary = true;
            }
            else if (this.type.prefix) {
                var node = this.startNode(), update = this.type === types.incDec;
                node.operator = this.value;
                node.prefix = true;
                this.next();
                node.argument = this.parseMaybeUnary(null, true);
                this.checkExpressionErrors(refDestructuringErrors, true);
                if (update) {
                    this.checkLVal(node.argument);
                }
                else if (this.strict && node.operator === "delete" &&
                    node.argument.type === "Identifier") {
                    this.raiseRecoverable(node.start, "Deleting local variable in strict mode");
                }
                else {
                    sawUnary = true;
                }
                expr = this.finishNode(node, update ? "UpdateExpression" : "UnaryExpression");
            }
            else {
                expr = this.parseExprSubscripts(refDestructuringErrors);
                if (this.checkExpressionErrors(refDestructuringErrors)) {
                    return expr;
                }
                while (this.type.postfix && !this.canInsertSemicolon()) {
                    var node$1 = this.startNodeAt(startPos, startLoc);
                    node$1.operator = this.value;
                    node$1.prefix = false;
                    node$1.argument = expr;
                    this.checkLVal(expr);
                    this.next();
                    expr = this.finishNode(node$1, "UpdateExpression");
                }
            }
            if (!sawUnary && this.eat(types.starstar)) {
                return this.buildBinary(startPos, startLoc, expr, this.parseMaybeUnary(null, false), "**", false);
            }
            else {
                return expr;
            }
        };
        // Parse call, dot, and `[]`-subscript expressions.
        pp$3.parseExprSubscripts = function (refDestructuringErrors) {
            var startPos = this.start, startLoc = this.startLoc;
            var expr = this.parseExprAtom(refDestructuringErrors);
            if (expr.type === "ArrowFunctionExpression" && this.input.slice(this.lastTokStart, this.lastTokEnd) !== ")") {
                return expr;
            }
            var result = this.parseSubscripts(expr, startPos, startLoc);
            if (refDestructuringErrors && result.type === "MemberExpression") {
                if (refDestructuringErrors.parenthesizedAssign >= result.start) {
                    refDestructuringErrors.parenthesizedAssign = -1;
                }
                if (refDestructuringErrors.parenthesizedBind >= result.start) {
                    refDestructuringErrors.parenthesizedBind = -1;
                }
            }
            return result;
        };
        pp$3.parseSubscripts = function (base, startPos, startLoc, noCalls) {
            var maybeAsyncArrow = this.options.ecmaVersion >= 8 && base.type === "Identifier" && base.name === "async" &&
                this.lastTokEnd === base.end && !this.canInsertSemicolon() && this.input.slice(base.start, base.end) === "async";
            while (true) {
                var element = this.parseSubscript(base, startPos, startLoc, noCalls, maybeAsyncArrow);
                if (element === base || element.type === "ArrowFunctionExpression") {
                    return element;
                }
                base = element;
            }
        };
        pp$3.parseSubscript = function (base, startPos, startLoc, noCalls, maybeAsyncArrow) {
            var computed = this.eat(types.bracketL);
            if (computed || this.eat(types.dot)) {
                var node = this.startNodeAt(startPos, startLoc);
                node.object = base;
                node.property = computed ? this.parseExpression() : this.parseIdent(this.options.allowReserved !== "never");
                node.computed = !!computed;
                if (computed) {
                    this.expect(types.bracketR);
                }
                base = this.finishNode(node, "MemberExpression");
            }
            else if (!noCalls && this.eat(types.parenL)) {
                var refDestructuringErrors = new DestructuringErrors, oldYieldPos = this.yieldPos, oldAwaitPos = this.awaitPos, oldAwaitIdentPos = this.awaitIdentPos;
                this.yieldPos = 0;
                this.awaitPos = 0;
                this.awaitIdentPos = 0;
                var exprList = this.parseExprList(types.parenR, this.options.ecmaVersion >= 8, false, refDestructuringErrors);
                if (maybeAsyncArrow && !this.canInsertSemicolon() && this.eat(types.arrow)) {
                    this.checkPatternErrors(refDestructuringErrors, false);
                    this.checkYieldAwaitInDefaultParams();
                    if (this.awaitIdentPos > 0) {
                        this.raise(this.awaitIdentPos, "Cannot use 'await' as identifier inside an async function");
                    }
                    this.yieldPos = oldYieldPos;
                    this.awaitPos = oldAwaitPos;
                    this.awaitIdentPos = oldAwaitIdentPos;
                    return this.parseArrowExpression(this.startNodeAt(startPos, startLoc), exprList, true);
                }
                this.checkExpressionErrors(refDestructuringErrors, true);
                this.yieldPos = oldYieldPos || this.yieldPos;
                this.awaitPos = oldAwaitPos || this.awaitPos;
                this.awaitIdentPos = oldAwaitIdentPos || this.awaitIdentPos;
                var node$1 = this.startNodeAt(startPos, startLoc);
                node$1.callee = base;
                node$1.arguments = exprList;
                base = this.finishNode(node$1, "CallExpression");
            }
            else if (this.type === types.backQuote) {
                var node$2 = this.startNodeAt(startPos, startLoc);
                node$2.tag = base;
                node$2.quasi = this.parseTemplate({ isTagged: true });
                base = this.finishNode(node$2, "TaggedTemplateExpression");
            }
            return base;
        };
        // Parse an atomic expression — either a single token that is an
        // expression, an expression started by a keyword like `function` or
        // `new`, or an expression wrapped in punctuation like `()`, `[]`,
        // or `{}`.
        pp$3.parseExprAtom = function (refDestructuringErrors) {
            // If a division operator appears in an expression position, the
            // tokenizer got confused, and we force it to read a regexp instead.
            if (this.type === types.slash) {
                this.readRegexp();
            }
            var node, canBeArrow = this.potentialArrowAt === this.start;
            switch (this.type) {
                case types._super:
                    if (!this.allowSuper) {
                        this.raise(this.start, "'super' keyword outside a method");
                    }
                    node = this.startNode();
                    this.next();
                    if (this.type === types.parenL && !this.allowDirectSuper) {
                        this.raise(node.start, "super() call outside constructor of a subclass");
                    }
                    // The `super` keyword can appear at below:
                    // SuperProperty:
                    //     super [ Expression ]
                    //     super . IdentifierName
                    // SuperCall:
                    //     super ( Arguments )
                    if (this.type !== types.dot && this.type !== types.bracketL && this.type !== types.parenL) {
                        this.unexpected();
                    }
                    return this.finishNode(node, "Super");
                case types._this:
                    node = this.startNode();
                    this.next();
                    return this.finishNode(node, "ThisExpression");
                case types.name:
                    var startPos = this.start, startLoc = this.startLoc, containsEsc = this.containsEsc;
                    var id = this.parseIdent(false);
                    if (this.options.ecmaVersion >= 8 && !containsEsc && id.name === "async" && !this.canInsertSemicolon() && this.eat(types._function)) {
                        return this.parseFunction(this.startNodeAt(startPos, startLoc), 0, false, true);
                    }
                    if (canBeArrow && !this.canInsertSemicolon()) {
                        if (this.eat(types.arrow)) {
                            return this.parseArrowExpression(this.startNodeAt(startPos, startLoc), [id], false);
                        }
                        if (this.options.ecmaVersion >= 8 && id.name === "async" && this.type === types.name && !containsEsc) {
                            id = this.parseIdent(false);
                            if (this.canInsertSemicolon() || !this.eat(types.arrow)) {
                                this.unexpected();
                            }
                            return this.parseArrowExpression(this.startNodeAt(startPos, startLoc), [id], true);
                        }
                    }
                    return id;
                case types.regexp:
                    var value = this.value;
                    node = this.parseLiteral(value.value);
                    node.regex = { pattern: value.pattern, flags: value.flags };
                    return node;
                case types.num:
                case types.string:
                    return this.parseLiteral(this.value);
                case types._null:
                case types._true:
                case types._false:
                    node = this.startNode();
                    node.value = this.type === types._null ? null : this.type === types._true;
                    node.raw = this.type.keyword;
                    this.next();
                    return this.finishNode(node, "Literal");
                case types.parenL:
                    var start = this.start, expr = this.parseParenAndDistinguishExpression(canBeArrow);
                    if (refDestructuringErrors) {
                        if (refDestructuringErrors.parenthesizedAssign < 0 && !this.isSimpleAssignTarget(expr)) {
                            refDestructuringErrors.parenthesizedAssign = start;
                        }
                        if (refDestructuringErrors.parenthesizedBind < 0) {
                            refDestructuringErrors.parenthesizedBind = start;
                        }
                    }
                    return expr;
                case types.bracketL:
                    node = this.startNode();
                    this.next();
                    node.elements = this.parseExprList(types.bracketR, true, true, refDestructuringErrors);
                    return this.finishNode(node, "ArrayExpression");
                case types.braceL:
                    return this.parseObj(false, refDestructuringErrors);
                case types._function:
                    node = this.startNode();
                    this.next();
                    return this.parseFunction(node, 0);
                case types._class:
                    return this.parseClass(this.startNode(), false);
                case types._new:
                    return this.parseNew();
                case types.backQuote:
                    return this.parseTemplate();
                case types._import:
                    if (this.options.ecmaVersion >= 11) {
                        return this.parseExprImport();
                    }
                    else {
                        return this.unexpected();
                    }
                default:
                    this.unexpected();
            }
        };
        pp$3.parseExprImport = function () {
            var node = this.startNode();
            this.next(); // skip `import`
            switch (this.type) {
                case types.parenL:
                    return this.parseDynamicImport(node);
                default:
                    this.unexpected();
            }
        };
        pp$3.parseDynamicImport = function (node) {
            this.next(); // skip `(`
            // Parse node.source.
            node.source = this.parseMaybeAssign();
            // Verify ending.
            if (!this.eat(types.parenR)) {
                var errorPos = this.start;
                if (this.eat(types.comma) && this.eat(types.parenR)) {
                    this.raiseRecoverable(errorPos, "Trailing comma is not allowed in import()");
                }
                else {
                    this.unexpected(errorPos);
                }
            }
            return this.finishNode(node, "ImportExpression");
        };
        pp$3.parseLiteral = function (value) {
            var node = this.startNode();
            node.value = value;
            node.raw = this.input.slice(this.start, this.end);
            if (node.raw.charCodeAt(node.raw.length - 1) === 110) {
                node.bigint = node.raw.slice(0, -1);
            }
            this.next();
            return this.finishNode(node, "Literal");
        };
        pp$3.parseParenExpression = function () {
            this.expect(types.parenL);
            var val = this.parseExpression();
            this.expect(types.parenR);
            return val;
        };
        pp$3.parseParenAndDistinguishExpression = function (canBeArrow) {
            var startPos = this.start, startLoc = this.startLoc, val, allowTrailingComma = this.options.ecmaVersion >= 8;
            if (this.options.ecmaVersion >= 6) {
                this.next();
                var innerStartPos = this.start, innerStartLoc = this.startLoc;
                var exprList = [], first = true, lastIsComma = false;
                var refDestructuringErrors = new DestructuringErrors, oldYieldPos = this.yieldPos, oldAwaitPos = this.awaitPos, spreadStart;
                this.yieldPos = 0;
                this.awaitPos = 0;
                // Do not save awaitIdentPos to allow checking awaits nested in parameters
                while (this.type !== types.parenR) {
                    first ? first = false : this.expect(types.comma);
                    if (allowTrailingComma && this.afterTrailingComma(types.parenR, true)) {
                        lastIsComma = true;
                        break;
                    }
                    else if (this.type === types.ellipsis) {
                        spreadStart = this.start;
                        exprList.push(this.parseParenItem(this.parseRestBinding()));
                        if (this.type === types.comma) {
                            this.raise(this.start, "Comma is not permitted after the rest element");
                        }
                        break;
                    }
                    else {
                        exprList.push(this.parseMaybeAssign(false, refDestructuringErrors, this.parseParenItem));
                    }
                }
                var innerEndPos = this.start, innerEndLoc = this.startLoc;
                this.expect(types.parenR);
                if (canBeArrow && !this.canInsertSemicolon() && this.eat(types.arrow)) {
                    this.checkPatternErrors(refDestructuringErrors, false);
                    this.checkYieldAwaitInDefaultParams();
                    this.yieldPos = oldYieldPos;
                    this.awaitPos = oldAwaitPos;
                    return this.parseParenArrowList(startPos, startLoc, exprList);
                }
                if (!exprList.length || lastIsComma) {
                    this.unexpected(this.lastTokStart);
                }
                if (spreadStart) {
                    this.unexpected(spreadStart);
                }
                this.checkExpressionErrors(refDestructuringErrors, true);
                this.yieldPos = oldYieldPos || this.yieldPos;
                this.awaitPos = oldAwaitPos || this.awaitPos;
                if (exprList.length > 1) {
                    val = this.startNodeAt(innerStartPos, innerStartLoc);
                    val.expressions = exprList;
                    this.finishNodeAt(val, "SequenceExpression", innerEndPos, innerEndLoc);
                }
                else {
                    val = exprList[0];
                }
            }
            else {
                val = this.parseParenExpression();
            }
            if (this.options.preserveParens) {
                var par = this.startNodeAt(startPos, startLoc);
                par.expression = val;
                return this.finishNode(par, "ParenthesizedExpression");
            }
            else {
                return val;
            }
        };
        pp$3.parseParenItem = function (item) {
            return item;
        };
        pp$3.parseParenArrowList = function (startPos, startLoc, exprList) {
            return this.parseArrowExpression(this.startNodeAt(startPos, startLoc), exprList);
        };
        // New's precedence is slightly tricky. It must allow its argument to
        // be a `[]` or dot subscript expression, but not a call — at least,
        // not without wrapping it in parentheses. Thus, it uses the noCalls
        // argument to parseSubscripts to prevent it from consuming the
        // argument list.
        var empty$1 = [];
        pp$3.parseNew = function () {
            if (this.containsEsc) {
                this.raiseRecoverable(this.start, "Escape sequence in keyword new");
            }
            var node = this.startNode();
            var meta = this.parseIdent(true);
            if (this.options.ecmaVersion >= 6 && this.eat(types.dot)) {
                node.meta = meta;
                var containsEsc = this.containsEsc;
                node.property = this.parseIdent(true);
                if (node.property.name !== "target" || containsEsc) {
                    this.raiseRecoverable(node.property.start, "The only valid meta property for new is new.target");
                }
                if (!this.inNonArrowFunction()) {
                    this.raiseRecoverable(node.start, "new.target can only be used in functions");
                }
                return this.finishNode(node, "MetaProperty");
            }
            var startPos = this.start, startLoc = this.startLoc, isImport = this.type === types._import;
            node.callee = this.parseSubscripts(this.parseExprAtom(), startPos, startLoc, true);
            if (isImport && node.callee.type === "ImportExpression") {
                this.raise(startPos, "Cannot use new with import()");
            }
            if (this.eat(types.parenL)) {
                node.arguments = this.parseExprList(types.parenR, this.options.ecmaVersion >= 8, false);
            }
            else {
                node.arguments = empty$1;
            }
            return this.finishNode(node, "NewExpression");
        };
        // Parse template expression.
        pp$3.parseTemplateElement = function (ref) {
            var isTagged = ref.isTagged;
            var elem = this.startNode();
            if (this.type === types.invalidTemplate) {
                if (!isTagged) {
                    this.raiseRecoverable(this.start, "Bad escape sequence in untagged template literal");
                }
                elem.value = {
                    raw: this.value,
                    cooked: null
                };
            }
            else {
                elem.value = {
                    raw: this.input.slice(this.start, this.end).replace(/\r\n?/g, "\n"),
                    cooked: this.value
                };
            }
            this.next();
            elem.tail = this.type === types.backQuote;
            return this.finishNode(elem, "TemplateElement");
        };
        pp$3.parseTemplate = function (ref) {
            if (ref === void 0)
                ref = {};
            var isTagged = ref.isTagged;
            if (isTagged === void 0)
                isTagged = false;
            var node = this.startNode();
            this.next();
            node.expressions = [];
            var curElt = this.parseTemplateElement({ isTagged: isTagged });
            node.quasis = [curElt];
            while (!curElt.tail) {
                if (this.type === types.eof) {
                    this.raise(this.pos, "Unterminated template literal");
                }
                this.expect(types.dollarBraceL);
                node.expressions.push(this.parseExpression());
                this.expect(types.braceR);
                node.quasis.push(curElt = this.parseTemplateElement({ isTagged: isTagged }));
            }
            this.next();
            return this.finishNode(node, "TemplateLiteral");
        };
        pp$3.isAsyncProp = function (prop) {
            return !prop.computed && prop.key.type === "Identifier" && prop.key.name === "async" &&
                (this.type === types.name || this.type === types.num || this.type === types.string || this.type === types.bracketL || this.type.keyword || (this.options.ecmaVersion >= 9 && this.type === types.star)) &&
                !lineBreak.test(this.input.slice(this.lastTokEnd, this.start));
        };
        // Parse an object literal or binding pattern.
        pp$3.parseObj = function (isPattern, refDestructuringErrors) {
            var node = this.startNode(), first = true, propHash = {};
            node.properties = [];
            this.next();
            while (!this.eat(types.braceR)) {
                if (!first) {
                    this.expect(types.comma);
                    if (this.options.ecmaVersion >= 5 && this.afterTrailingComma(types.braceR)) {
                        break;
                    }
                }
                else {
                    first = false;
                }
                var prop = this.parseProperty(isPattern, refDestructuringErrors);
                if (!isPattern) {
                    this.checkPropClash(prop, propHash, refDestructuringErrors);
                }
                node.properties.push(prop);
            }
            return this.finishNode(node, isPattern ? "ObjectPattern" : "ObjectExpression");
        };
        pp$3.parseProperty = function (isPattern, refDestructuringErrors) {
            var prop = this.startNode(), isGenerator, isAsync, startPos, startLoc;
            if (this.options.ecmaVersion >= 9 && this.eat(types.ellipsis)) {
                if (isPattern) {
                    prop.argument = this.parseIdent(false);
                    if (this.type === types.comma) {
                        this.raise(this.start, "Comma is not permitted after the rest element");
                    }
                    return this.finishNode(prop, "RestElement");
                }
                // To disallow parenthesized identifier via `this.toAssignable()`.
                if (this.type === types.parenL && refDestructuringErrors) {
                    if (refDestructuringErrors.parenthesizedAssign < 0) {
                        refDestructuringErrors.parenthesizedAssign = this.start;
                    }
                    if (refDestructuringErrors.parenthesizedBind < 0) {
                        refDestructuringErrors.parenthesizedBind = this.start;
                    }
                }
                // Parse argument.
                prop.argument = this.parseMaybeAssign(false, refDestructuringErrors);
                // To disallow trailing comma via `this.toAssignable()`.
                if (this.type === types.comma && refDestructuringErrors && refDestructuringErrors.trailingComma < 0) {
                    refDestructuringErrors.trailingComma = this.start;
                }
                // Finish
                return this.finishNode(prop, "SpreadElement");
            }
            if (this.options.ecmaVersion >= 6) {
                prop.method = false;
                prop.shorthand = false;
                if (isPattern || refDestructuringErrors) {
                    startPos = this.start;
                    startLoc = this.startLoc;
                }
                if (!isPattern) {
                    isGenerator = this.eat(types.star);
                }
            }
            var containsEsc = this.containsEsc;
            this.parsePropertyName(prop);
            if (!isPattern && !containsEsc && this.options.ecmaVersion >= 8 && !isGenerator && this.isAsyncProp(prop)) {
                isAsync = true;
                isGenerator = this.options.ecmaVersion >= 9 && this.eat(types.star);
                this.parsePropertyName(prop, refDestructuringErrors);
            }
            else {
                isAsync = false;
            }
            this.parsePropertyValue(prop, isPattern, isGenerator, isAsync, startPos, startLoc, refDestructuringErrors, containsEsc);
            return this.finishNode(prop, "Property");
        };
        pp$3.parsePropertyValue = function (prop, isPattern, isGenerator, isAsync, startPos, startLoc, refDestructuringErrors, containsEsc) {
            if ((isGenerator || isAsync) && this.type === types.colon) {
                this.unexpected();
            }
            if (this.eat(types.colon)) {
                prop.value = isPattern ? this.parseMaybeDefault(this.start, this.startLoc) : this.parseMaybeAssign(false, refDestructuringErrors);
                prop.kind = "init";
            }
            else if (this.options.ecmaVersion >= 6 && this.type === types.parenL) {
                if (isPattern) {
                    this.unexpected();
                }
                prop.kind = "init";
                prop.method = true;
                prop.value = this.parseMethod(isGenerator, isAsync);
            }
            else if (!isPattern && !containsEsc &&
                this.options.ecmaVersion >= 5 && !prop.computed && prop.key.type === "Identifier" &&
                (prop.key.name === "get" || prop.key.name === "set") &&
                (this.type !== types.comma && this.type !== types.braceR)) {
                if (isGenerator || isAsync) {
                    this.unexpected();
                }
                prop.kind = prop.key.name;
                this.parsePropertyName(prop);
                prop.value = this.parseMethod(false);
                var paramCount = prop.kind === "get" ? 0 : 1;
                if (prop.value.params.length !== paramCount) {
                    var start = prop.value.start;
                    if (prop.kind === "get") {
                        this.raiseRecoverable(start, "getter should have no params");
                    }
                    else {
                        this.raiseRecoverable(start, "setter should have exactly one param");
                    }
                }
                else {
                    if (prop.kind === "set" && prop.value.params[0].type === "RestElement") {
                        this.raiseRecoverable(prop.value.params[0].start, "Setter cannot use rest params");
                    }
                }
            }
            else if (this.options.ecmaVersion >= 6 && !prop.computed && prop.key.type === "Identifier") {
                if (isGenerator || isAsync) {
                    this.unexpected();
                }
                this.checkUnreserved(prop.key);
                if (prop.key.name === "await" && !this.awaitIdentPos) {
                    this.awaitIdentPos = startPos;
                }
                prop.kind = "init";
                if (isPattern) {
                    prop.value = this.parseMaybeDefault(startPos, startLoc, prop.key);
                }
                else if (this.type === types.eq && refDestructuringErrors) {
                    if (refDestructuringErrors.shorthandAssign < 0) {
                        refDestructuringErrors.shorthandAssign = this.start;
                    }
                    prop.value = this.parseMaybeDefault(startPos, startLoc, prop.key);
                }
                else {
                    prop.value = prop.key;
                }
                prop.shorthand = true;
            }
            else {
                this.unexpected();
            }
        };
        pp$3.parsePropertyName = function (prop) {
            if (this.options.ecmaVersion >= 6) {
                if (this.eat(types.bracketL)) {
                    prop.computed = true;
                    prop.key = this.parseMaybeAssign();
                    this.expect(types.bracketR);
                    return prop.key;
                }
                else {
                    prop.computed = false;
                }
            }
            return prop.key = this.type === types.num || this.type === types.string ? this.parseExprAtom() : this.parseIdent(this.options.allowReserved !== "never");
        };
        // Initialize empty function node.
        pp$3.initFunction = function (node) {
            node.id = null;
            if (this.options.ecmaVersion >= 6) {
                node.generator = node.expression = false;
            }
            if (this.options.ecmaVersion >= 8) {
                node.async = false;
            }
        };
        // Parse object or class method.
        pp$3.parseMethod = function (isGenerator, isAsync, allowDirectSuper) {
            var node = this.startNode(), oldYieldPos = this.yieldPos, oldAwaitPos = this.awaitPos, oldAwaitIdentPos = this.awaitIdentPos;
            this.initFunction(node);
            if (this.options.ecmaVersion >= 6) {
                node.generator = isGenerator;
            }
            if (this.options.ecmaVersion >= 8) {
                node.async = !!isAsync;
            }
            this.yieldPos = 0;
            this.awaitPos = 0;
            this.awaitIdentPos = 0;
            this.enterScope(functionFlags(isAsync, node.generator) | SCOPE_SUPER | (allowDirectSuper ? SCOPE_DIRECT_SUPER : 0));
            this.expect(types.parenL);
            node.params = this.parseBindingList(types.parenR, false, this.options.ecmaVersion >= 8);
            this.checkYieldAwaitInDefaultParams();
            this.parseFunctionBody(node, false, true);
            this.yieldPos = oldYieldPos;
            this.awaitPos = oldAwaitPos;
            this.awaitIdentPos = oldAwaitIdentPos;
            return this.finishNode(node, "FunctionExpression");
        };
        // Parse arrow function expression with given parameters.
        pp$3.parseArrowExpression = function (node, params, isAsync) {
            var oldYieldPos = this.yieldPos, oldAwaitPos = this.awaitPos, oldAwaitIdentPos = this.awaitIdentPos;
            this.enterScope(functionFlags(isAsync, false) | SCOPE_ARROW);
            this.initFunction(node);
            if (this.options.ecmaVersion >= 8) {
                node.async = !!isAsync;
            }
            this.yieldPos = 0;
            this.awaitPos = 0;
            this.awaitIdentPos = 0;
            node.params = this.toAssignableList(params, true);
            this.parseFunctionBody(node, true, false);
            this.yieldPos = oldYieldPos;
            this.awaitPos = oldAwaitPos;
            this.awaitIdentPos = oldAwaitIdentPos;
            return this.finishNode(node, "ArrowFunctionExpression");
        };
        // Parse function body and check parameters.
        pp$3.parseFunctionBody = function (node, isArrowFunction, isMethod) {
            var isExpression = isArrowFunction && this.type !== types.braceL;
            var oldStrict = this.strict, useStrict = false;
            if (isExpression) {
                node.body = this.parseMaybeAssign();
                node.expression = true;
                this.checkParams(node, false);
            }
            else {
                var nonSimple = this.options.ecmaVersion >= 7 && !this.isSimpleParamList(node.params);
                if (!oldStrict || nonSimple) {
                    useStrict = this.strictDirective(this.end);
                    // If this is a strict mode function, verify that argument names
                    // are not repeated, and it does not try to bind the words `eval`
                    // or `arguments`.
                    if (useStrict && nonSimple) {
                        this.raiseRecoverable(node.start, "Illegal 'use strict' directive in function with non-simple parameter list");
                    }
                }
                // Start a new scope with regard to labels and the `inFunction`
                // flag (restore them to their old value afterwards).
                var oldLabels = this.labels;
                this.labels = [];
                if (useStrict) {
                    this.strict = true;
                }
                // Add the params to varDeclaredNames to ensure that an error is thrown
                // if a let/const declaration in the function clashes with one of the params.
                this.checkParams(node, !oldStrict && !useStrict && !isArrowFunction && !isMethod && this.isSimpleParamList(node.params));
                node.body = this.parseBlock(false);
                node.expression = false;
                this.adaptDirectivePrologue(node.body.body);
                this.labels = oldLabels;
            }
            this.exitScope();
            // Ensure the function name isn't a forbidden identifier in strict mode, e.g. 'eval'
            if (this.strict && node.id) {
                this.checkLVal(node.id, BIND_OUTSIDE);
            }
            this.strict = oldStrict;
        };
        pp$3.isSimpleParamList = function (params) {
            for (var i = 0, list = params; i < list.length; i += 1) {
                var param = list[i];
                if (param.type !== "Identifier") {
                    return false;
                }
            }
            return true;
        };
        // Checks function params for various disallowed patterns such as using "eval"
        // or "arguments" and duplicate parameters.
        pp$3.checkParams = function (node, allowDuplicates) {
            var nameHash = {};
            for (var i = 0, list = node.params; i < list.length; i += 1) {
                var param = list[i];
                this.checkLVal(param, BIND_VAR, allowDuplicates ? null : nameHash);
            }
        };
        // Parses a comma-separated list of expressions, and returns them as
        // an array. `close` is the token type that ends the list, and
        // `allowEmpty` can be turned on to allow subsequent commas with
        // nothing in between them to be parsed as `null` (which is needed
        // for array literals).
        pp$3.parseExprList = function (close, allowTrailingComma, allowEmpty, refDestructuringErrors) {
            var elts = [], first = true;
            while (!this.eat(close)) {
                if (!first) {
                    this.expect(types.comma);
                    if (allowTrailingComma && this.afterTrailingComma(close)) {
                        break;
                    }
                }
                else {
                    first = false;
                }
                var elt = (void 0);
                if (allowEmpty && this.type === types.comma) {
                    elt = null;
                }
                else if (this.type === types.ellipsis) {
                    elt = this.parseSpread(refDestructuringErrors);
                    if (refDestructuringErrors && this.type === types.comma && refDestructuringErrors.trailingComma < 0) {
                        refDestructuringErrors.trailingComma = this.start;
                    }
                }
                else {
                    elt = this.parseMaybeAssign(false, refDestructuringErrors);
                }
                elts.push(elt);
            }
            return elts;
        };
        pp$3.checkUnreserved = function (ref) {
            var start = ref.start;
            var end = ref.end;
            var name = ref.name;
            if (this.inGenerator && name === "yield") {
                this.raiseRecoverable(start, "Cannot use 'yield' as identifier inside a generator");
            }
            if (this.inAsync && name === "await") {
                this.raiseRecoverable(start, "Cannot use 'await' as identifier inside an async function");
            }
            if (this.keywords.test(name)) {
                this.raise(start, ("Unexpected keyword '" + name + "'"));
            }
            if (this.options.ecmaVersion < 6 &&
                this.input.slice(start, end).indexOf("\\") !== -1) {
                return;
            }
            var re = this.strict ? this.reservedWordsStrict : this.reservedWords;
            if (re.test(name)) {
                if (!this.inAsync && name === "await") {
                    this.raiseRecoverable(start, "Cannot use keyword 'await' outside an async function");
                }
                this.raiseRecoverable(start, ("The keyword '" + name + "' is reserved"));
            }
        };
        // Parse the next token as an identifier. If `liberal` is true (used
        // when parsing properties), it will also convert keywords into
        // identifiers.
        pp$3.parseIdent = function (liberal, isBinding) {
            var node = this.startNode();
            if (this.type === types.name) {
                node.name = this.value;
            }
            else if (this.type.keyword) {
                node.name = this.type.keyword;
                // To fix https://github.com/acornjs/acorn/issues/575
                // `class` and `function` keywords push new context into this.context.
                // But there is no chance to pop the context if the keyword is consumed as an identifier such as a property name.
                // If the previous token is a dot, this does not apply because the context-managing code already ignored the keyword
                if ((node.name === "class" || node.name === "function") &&
                    (this.lastTokEnd !== this.lastTokStart + 1 || this.input.charCodeAt(this.lastTokStart) !== 46)) {
                    this.context.pop();
                }
            }
            else {
                this.unexpected();
            }
            this.next(!!liberal);
            this.finishNode(node, "Identifier");
            if (!liberal) {
                this.checkUnreserved(node);
                if (node.name === "await" && !this.awaitIdentPos) {
                    this.awaitIdentPos = node.start;
                }
            }
            return node;
        };
        // Parses yield expression inside generator.
        pp$3.parseYield = function (noIn) {
            if (!this.yieldPos) {
                this.yieldPos = this.start;
            }
            var node = this.startNode();
            this.next();
            if (this.type === types.semi || this.canInsertSemicolon() || (this.type !== types.star && !this.type.startsExpr)) {
                node.delegate = false;
                node.argument = null;
            }
            else {
                node.delegate = this.eat(types.star);
                node.argument = this.parseMaybeAssign(noIn);
            }
            return this.finishNode(node, "YieldExpression");
        };
        pp$3.parseAwait = function () {
            if (!this.awaitPos) {
                this.awaitPos = this.start;
            }
            var node = this.startNode();
            this.next();
            node.argument = this.parseMaybeUnary(null, false);
            return this.finishNode(node, "AwaitExpression");
        };
        var pp$4 = Parser.prototype;
        // This function is used to raise exceptions on parse errors. It
        // takes an offset integer (into the current `input`) to indicate
        // the location of the error, attaches the position to the end
        // of the error message, and then raises a `SyntaxError` with that
        // message.
        pp$4.raise = function (pos, message) {
            var loc = getLineInfo(this.input, pos);
            message += " (" + loc.line + ":" + loc.column + ")";
            var err = new SyntaxError(message);
            err.pos = pos;
            err.loc = loc;
            err.raisedAt = this.pos;
            throw err;
        };
        pp$4.raiseRecoverable = pp$4.raise;
        pp$4.curPosition = function () {
            if (this.options.locations) {
                return new Position(this.curLine, this.pos - this.lineStart);
            }
        };
        var pp$5 = Parser.prototype;
        var Scope = function Scope(flags) {
            this.flags = flags;
            // A list of var-declared names in the current lexical scope
            this.var = [];
            // A list of lexically-declared names in the current lexical scope
            this.lexical = [];
            // A list of lexically-declared FunctionDeclaration names in the current lexical scope
            this.functions = [];
        };
        // The functions in this module keep track of declared variables in the current scope in order to detect duplicate variable names.
        pp$5.enterScope = function (flags) {
            this.scopeStack.push(new Scope(flags));
        };
        pp$5.exitScope = function () {
            this.scopeStack.pop();
        };
        // The spec says:
        // > At the top level of a function, or script, function declarations are
        // > treated like var declarations rather than like lexical declarations.
        pp$5.treatFunctionsAsVarInScope = function (scope) {
            return (scope.flags & SCOPE_FUNCTION) || !this.inModule && (scope.flags & SCOPE_TOP);
        };
        pp$5.declareName = function (name, bindingType, pos) {
            var redeclared = false;
            if (bindingType === BIND_LEXICAL) {
                var scope = this.currentScope();
                redeclared = scope.lexical.indexOf(name) > -1 || scope.functions.indexOf(name) > -1 || scope.var.indexOf(name) > -1;
                scope.lexical.push(name);
                if (this.inModule && (scope.flags & SCOPE_TOP)) {
                    delete this.undefinedExports[name];
                }
            }
            else if (bindingType === BIND_SIMPLE_CATCH) {
                var scope$1 = this.currentScope();
                scope$1.lexical.push(name);
            }
            else if (bindingType === BIND_FUNCTION) {
                var scope$2 = this.currentScope();
                if (this.treatFunctionsAsVar) {
                    redeclared = scope$2.lexical.indexOf(name) > -1;
                }
                else {
                    redeclared = scope$2.lexical.indexOf(name) > -1 || scope$2.var.indexOf(name) > -1;
                }
                scope$2.functions.push(name);
            }
            else {
                for (var i = this.scopeStack.length - 1; i >= 0; --i) {
                    var scope$3 = this.scopeStack[i];
                    if (scope$3.lexical.indexOf(name) > -1 && !((scope$3.flags & SCOPE_SIMPLE_CATCH) && scope$3.lexical[0] === name) ||
                        !this.treatFunctionsAsVarInScope(scope$3) && scope$3.functions.indexOf(name) > -1) {
                        redeclared = true;
                        break;
                    }
                    scope$3.var.push(name);
                    if (this.inModule && (scope$3.flags & SCOPE_TOP)) {
                        delete this.undefinedExports[name];
                    }
                    if (scope$3.flags & SCOPE_VAR) {
                        break;
                    }
                }
            }
            if (redeclared) {
                this.raiseRecoverable(pos, ("Identifier '" + name + "' has already been declared"));
            }
        };
        pp$5.checkLocalExport = function (id) {
            // scope.functions must be empty as Module code is always strict.
            if (this.scopeStack[0].lexical.indexOf(id.name) === -1 &&
                this.scopeStack[0].var.indexOf(id.name) === -1) {
                this.undefinedExports[id.name] = id;
            }
        };
        pp$5.currentScope = function () {
            return this.scopeStack[this.scopeStack.length - 1];
        };
        pp$5.currentVarScope = function () {
            for (var i = this.scopeStack.length - 1;; i--) {
                var scope = this.scopeStack[i];
                if (scope.flags & SCOPE_VAR) {
                    return scope;
                }
            }
        };
        // Could be useful for `this`, `new.target`, `super()`, `super.property`, and `super[property]`.
        pp$5.currentThisScope = function () {
            for (var i = this.scopeStack.length - 1;; i--) {
                var scope = this.scopeStack[i];
                if (scope.flags & SCOPE_VAR && !(scope.flags & SCOPE_ARROW)) {
                    return scope;
                }
            }
        };
        var Node = function Node(parser, pos, loc) {
            this.type = "";
            this.start = pos;
            this.end = 0;
            if (parser.options.locations) {
                this.loc = new SourceLocation(parser, loc);
            }
            if (parser.options.directSourceFile) {
                this.sourceFile = parser.options.directSourceFile;
            }
            if (parser.options.ranges) {
                this.range = [pos, 0];
            }
        };
        // Start an AST node, attaching a start offset.
        var pp$6 = Parser.prototype;
        pp$6.startNode = function () {
            return new Node(this, this.start, this.startLoc);
        };
        pp$6.startNodeAt = function (pos, loc) {
            return new Node(this, pos, loc);
        };
        // Finish an AST node, adding `type` and `end` properties.
        function finishNodeAt(node, type, pos, loc) {
            node.type = type;
            node.end = pos;
            if (this.options.locations) {
                node.loc.end = loc;
            }
            if (this.options.ranges) {
                node.range[1] = pos;
            }
            return node;
        }
        pp$6.finishNode = function (node, type) {
            return finishNodeAt.call(this, node, type, this.lastTokEnd, this.lastTokEndLoc);
        };
        // Finish node at given position
        pp$6.finishNodeAt = function (node, type, pos, loc) {
            return finishNodeAt.call(this, node, type, pos, loc);
        };
        // The algorithm used to determine whether a regexp can appear at a
        var TokContext = function TokContext(token, isExpr, preserveSpace, override, generator) {
            this.token = token;
            this.isExpr = !!isExpr;
            this.preserveSpace = !!preserveSpace;
            this.override = override;
            this.generator = !!generator;
        };
        var types$1 = {
            b_stat: new TokContext("{", false),
            b_expr: new TokContext("{", true),
            b_tmpl: new TokContext("${", false),
            p_stat: new TokContext("(", false),
            p_expr: new TokContext("(", true),
            q_tmpl: new TokContext("`", true, true, function (p) { return p.tryReadTemplateToken(); }),
            f_stat: new TokContext("function", false),
            f_expr: new TokContext("function", true),
            f_expr_gen: new TokContext("function", true, false, null, true),
            f_gen: new TokContext("function", false, false, null, true)
        };
        var pp$7 = Parser.prototype;
        pp$7.initialContext = function () {
            return [types$1.b_stat];
        };
        pp$7.braceIsBlock = function (prevType) {
            var parent = this.curContext();
            if (parent === types$1.f_expr || parent === types$1.f_stat) {
                return true;
            }
            if (prevType === types.colon && (parent === types$1.b_stat || parent === types$1.b_expr)) {
                return !parent.isExpr;
            }
            // The check for `tt.name && exprAllowed` detects whether we are
            // after a `yield` or `of` construct. See the `updateContext` for
            // `tt.name`.
            if (prevType === types._return || prevType === types.name && this.exprAllowed) {
                return lineBreak.test(this.input.slice(this.lastTokEnd, this.start));
            }
            if (prevType === types._else || prevType === types.semi || prevType === types.eof || prevType === types.parenR || prevType === types.arrow) {
                return true;
            }
            if (prevType === types.braceL) {
                return parent === types$1.b_stat;
            }
            if (prevType === types._var || prevType === types._const || prevType === types.name) {
                return false;
            }
            return !this.exprAllowed;
        };
        pp$7.inGeneratorContext = function () {
            for (var i = this.context.length - 1; i >= 1; i--) {
                var context = this.context[i];
                if (context.token === "function") {
                    return context.generator;
                }
            }
            return false;
        };
        pp$7.updateContext = function (prevType) {
            var update, type = this.type;
            if (type.keyword && prevType === types.dot) {
                this.exprAllowed = false;
            }
            else if (update = type.updateContext) {
                update.call(this, prevType);
            }
            else {
                this.exprAllowed = type.beforeExpr;
            }
        };
        // Token-specific context update code
        types.parenR.updateContext = types.braceR.updateContext = function () {
            if (this.context.length === 1) {
                this.exprAllowed = true;
                return;
            }
            var out = this.context.pop();
            if (out === types$1.b_stat && this.curContext().token === "function") {
                out = this.context.pop();
            }
            this.exprAllowed = !out.isExpr;
        };
        types.braceL.updateContext = function (prevType) {
            this.context.push(this.braceIsBlock(prevType) ? types$1.b_stat : types$1.b_expr);
            this.exprAllowed = true;
        };
        types.dollarBraceL.updateContext = function () {
            this.context.push(types$1.b_tmpl);
            this.exprAllowed = true;
        };
        types.parenL.updateContext = function (prevType) {
            var statementParens = prevType === types._if || prevType === types._for || prevType === types._with || prevType === types._while;
            this.context.push(statementParens ? types$1.p_stat : types$1.p_expr);
            this.exprAllowed = true;
        };
        types.incDec.updateContext = function () {
            // tokExprAllowed stays unchanged
        };
        types._function.updateContext = types._class.updateContext = function (prevType) {
            if (prevType.beforeExpr && prevType !== types.semi && prevType !== types._else &&
                !(prevType === types._return && lineBreak.test(this.input.slice(this.lastTokEnd, this.start))) &&
                !((prevType === types.colon || prevType === types.braceL) && this.curContext() === types$1.b_stat)) {
                this.context.push(types$1.f_expr);
            }
            else {
                this.context.push(types$1.f_stat);
            }
            this.exprAllowed = false;
        };
        types.backQuote.updateContext = function () {
            if (this.curContext() === types$1.q_tmpl) {
                this.context.pop();
            }
            else {
                this.context.push(types$1.q_tmpl);
            }
            this.exprAllowed = false;
        };
        types.star.updateContext = function (prevType) {
            if (prevType === types._function) {
                var index = this.context.length - 1;
                if (this.context[index] === types$1.f_expr) {
                    this.context[index] = types$1.f_expr_gen;
                }
                else {
                    this.context[index] = types$1.f_gen;
                }
            }
            this.exprAllowed = true;
        };
        types.name.updateContext = function (prevType) {
            var allowed = false;
            if (this.options.ecmaVersion >= 6 && prevType !== types.dot) {
                if (this.value === "of" && !this.exprAllowed ||
                    this.value === "yield" && this.inGeneratorContext()) {
                    allowed = true;
                }
            }
            this.exprAllowed = allowed;
        };
        // This file contains Unicode properties extracted from the ECMAScript
        // specification. The lists are extracted like so:
        // $$('#table-binary-unicode-properties > figure > table > tbody > tr > td:nth-child(1) code').map(el => el.innerText)
        // #table-binary-unicode-properties
        var ecma9BinaryProperties = "ASCII ASCII_Hex_Digit AHex Alphabetic Alpha Any Assigned Bidi_Control Bidi_C Bidi_Mirrored Bidi_M Case_Ignorable CI Cased Changes_When_Casefolded CWCF Changes_When_Casemapped CWCM Changes_When_Lowercased CWL Changes_When_NFKC_Casefolded CWKCF Changes_When_Titlecased CWT Changes_When_Uppercased CWU Dash Default_Ignorable_Code_Point DI Deprecated Dep Diacritic Dia Emoji Emoji_Component Emoji_Modifier Emoji_Modifier_Base Emoji_Presentation Extender Ext Grapheme_Base Gr_Base Grapheme_Extend Gr_Ext Hex_Digit Hex IDS_Binary_Operator IDSB IDS_Trinary_Operator IDST ID_Continue IDC ID_Start IDS Ideographic Ideo Join_Control Join_C Logical_Order_Exception LOE Lowercase Lower Math Noncharacter_Code_Point NChar Pattern_Syntax Pat_Syn Pattern_White_Space Pat_WS Quotation_Mark QMark Radical Regional_Indicator RI Sentence_Terminal STerm Soft_Dotted SD Terminal_Punctuation Term Unified_Ideograph UIdeo Uppercase Upper Variation_Selector VS White_Space space XID_Continue XIDC XID_Start XIDS";
        var ecma10BinaryProperties = ecma9BinaryProperties + " Extended_Pictographic";
        var ecma11BinaryProperties = ecma10BinaryProperties;
        var unicodeBinaryProperties = {
            9: ecma9BinaryProperties,
            10: ecma10BinaryProperties,
            11: ecma11BinaryProperties
        };
        // #table-unicode-general-category-values
        var unicodeGeneralCategoryValues = "Cased_Letter LC Close_Punctuation Pe Connector_Punctuation Pc Control Cc cntrl Currency_Symbol Sc Dash_Punctuation Pd Decimal_Number Nd digit Enclosing_Mark Me Final_Punctuation Pf Format Cf Initial_Punctuation Pi Letter L Letter_Number Nl Line_Separator Zl Lowercase_Letter Ll Mark M Combining_Mark Math_Symbol Sm Modifier_Letter Lm Modifier_Symbol Sk Nonspacing_Mark Mn Number N Open_Punctuation Ps Other C Other_Letter Lo Other_Number No Other_Punctuation Po Other_Symbol So Paragraph_Separator Zp Private_Use Co Punctuation P punct Separator Z Space_Separator Zs Spacing_Mark Mc Surrogate Cs Symbol S Titlecase_Letter Lt Unassigned Cn Uppercase_Letter Lu";
        // #table-unicode-script-values
        var ecma9ScriptValues = "Adlam Adlm Ahom Ahom Anatolian_Hieroglyphs Hluw Arabic Arab Armenian Armn Avestan Avst Balinese Bali Bamum Bamu Bassa_Vah Bass Batak Batk Bengali Beng Bhaiksuki Bhks Bopomofo Bopo Brahmi Brah Braille Brai Buginese Bugi Buhid Buhd Canadian_Aboriginal Cans Carian Cari Caucasian_Albanian Aghb Chakma Cakm Cham Cham Cherokee Cher Common Zyyy Coptic Copt Qaac Cuneiform Xsux Cypriot Cprt Cyrillic Cyrl Deseret Dsrt Devanagari Deva Duployan Dupl Egyptian_Hieroglyphs Egyp Elbasan Elba Ethiopic Ethi Georgian Geor Glagolitic Glag Gothic Goth Grantha Gran Greek Grek Gujarati Gujr Gurmukhi Guru Han Hani Hangul Hang Hanunoo Hano Hatran Hatr Hebrew Hebr Hiragana Hira Imperial_Aramaic Armi Inherited Zinh Qaai Inscriptional_Pahlavi Phli Inscriptional_Parthian Prti Javanese Java Kaithi Kthi Kannada Knda Katakana Kana Kayah_Li Kali Kharoshthi Khar Khmer Khmr Khojki Khoj Khudawadi Sind Lao Laoo Latin Latn Lepcha Lepc Limbu Limb Linear_A Lina Linear_B Linb Lisu Lisu Lycian Lyci Lydian Lydi Mahajani Mahj Malayalam Mlym Mandaic Mand Manichaean Mani Marchen Marc Masaram_Gondi Gonm Meetei_Mayek Mtei Mende_Kikakui Mend Meroitic_Cursive Merc Meroitic_Hieroglyphs Mero Miao Plrd Modi Modi Mongolian Mong Mro Mroo Multani Mult Myanmar Mymr Nabataean Nbat New_Tai_Lue Talu Newa Newa Nko Nkoo Nushu Nshu Ogham Ogam Ol_Chiki Olck Old_Hungarian Hung Old_Italic Ital Old_North_Arabian Narb Old_Permic Perm Old_Persian Xpeo Old_South_Arabian Sarb Old_Turkic Orkh Oriya Orya Osage Osge Osmanya Osma Pahawh_Hmong Hmng Palmyrene Palm Pau_Cin_Hau Pauc Phags_Pa Phag Phoenician Phnx Psalter_Pahlavi Phlp Rejang Rjng Runic Runr Samaritan Samr Saurashtra Saur Sharada Shrd Shavian Shaw Siddham Sidd SignWriting Sgnw Sinhala Sinh Sora_Sompeng Sora Soyombo Soyo Sundanese Sund Syloti_Nagri Sylo Syriac Syrc Tagalog Tglg Tagbanwa Tagb Tai_Le Tale Tai_Tham Lana Tai_Viet Tavt Takri Takr Tamil Taml Tangut Tang Telugu Telu Thaana Thaa Thai Thai Tibetan Tibt Tifinagh Tfng Tirhuta Tirh Ugaritic Ugar Vai Vaii Warang_Citi Wara Yi Yiii Zanabazar_Square Zanb";
        var ecma10ScriptValues = ecma9ScriptValues + " Dogra Dogr Gunjala_Gondi Gong Hanifi_Rohingya Rohg Makasar Maka Medefaidrin Medf Old_Sogdian Sogo Sogdian Sogd";
        var ecma11ScriptValues = ecma10ScriptValues + " Elymaic Elym Nandinagari Nand Nyiakeng_Puachue_Hmong Hmnp Wancho Wcho";
        var unicodeScriptValues = {
            9: ecma9ScriptValues,
            10: ecma10ScriptValues,
            11: ecma11ScriptValues
        };
        var data = {};
        function buildUnicodeData(ecmaVersion) {
            var d = data[ecmaVersion] = {
                binary: wordsRegexp(unicodeBinaryProperties[ecmaVersion] + " " + unicodeGeneralCategoryValues),
                nonBinary: {
                    General_Category: wordsRegexp(unicodeGeneralCategoryValues),
                    Script: wordsRegexp(unicodeScriptValues[ecmaVersion])
                }
            };
            d.nonBinary.Script_Extensions = d.nonBinary.Script;
            d.nonBinary.gc = d.nonBinary.General_Category;
            d.nonBinary.sc = d.nonBinary.Script;
            d.nonBinary.scx = d.nonBinary.Script_Extensions;
        }
        buildUnicodeData(9);
        buildUnicodeData(10);
        buildUnicodeData(11);
        var pp$8 = Parser.prototype;
        var RegExpValidationState = function RegExpValidationState(parser) {
            this.parser = parser;
            this.validFlags = "gim" + (parser.options.ecmaVersion >= 6 ? "uy" : "") + (parser.options.ecmaVersion >= 9 ? "s" : "");
            this.unicodeProperties = data[parser.options.ecmaVersion >= 11 ? 11 : parser.options.ecmaVersion];
            this.source = "";
            this.flags = "";
            this.start = 0;
            this.switchU = false;
            this.switchN = false;
            this.pos = 0;
            this.lastIntValue = 0;
            this.lastStringValue = "";
            this.lastAssertionIsQuantifiable = false;
            this.numCapturingParens = 0;
            this.maxBackReference = 0;
            this.groupNames = [];
            this.backReferenceNames = [];
        };
        RegExpValidationState.prototype.reset = function reset(start, pattern, flags) {
            var unicode = flags.indexOf("u") !== -1;
            this.start = start | 0;
            this.source = pattern + "";
            this.flags = flags;
            this.switchU = unicode && this.parser.options.ecmaVersion >= 6;
            this.switchN = unicode && this.parser.options.ecmaVersion >= 9;
        };
        RegExpValidationState.prototype.raise = function raise(message) {
            this.parser.raiseRecoverable(this.start, ("Invalid regular expression: /" + (this.source) + "/: " + message));
        };
        // If u flag is given, this returns the code point at the index (it combines a surrogate pair).
        // Otherwise, this returns the code unit of the index (can be a part of a surrogate pair).
        RegExpValidationState.prototype.at = function at(i) {
            var s = this.source;
            var l = s.length;
            if (i >= l) {
                return -1;
            }
            var c = s.charCodeAt(i);
            if (!this.switchU || c <= 0xD7FF || c >= 0xE000 || i + 1 >= l) {
                return c;
            }
            var next = s.charCodeAt(i + 1);
            return next >= 0xDC00 && next <= 0xDFFF ? (c << 10) + next - 0x35FDC00 : c;
        };
        RegExpValidationState.prototype.nextIndex = function nextIndex(i) {
            var s = this.source;
            var l = s.length;
            if (i >= l) {
                return l;
            }
            var c = s.charCodeAt(i), next;
            if (!this.switchU || c <= 0xD7FF || c >= 0xE000 || i + 1 >= l ||
                (next = s.charCodeAt(i + 1)) < 0xDC00 || next > 0xDFFF) {
                return i + 1;
            }
            return i + 2;
        };
        RegExpValidationState.prototype.current = function current() {
            return this.at(this.pos);
        };
        RegExpValidationState.prototype.lookahead = function lookahead() {
            return this.at(this.nextIndex(this.pos));
        };
        RegExpValidationState.prototype.advance = function advance() {
            this.pos = this.nextIndex(this.pos);
        };
        RegExpValidationState.prototype.eat = function eat(ch) {
            if (this.current() === ch) {
                this.advance();
                return true;
            }
            return false;
        };
        function codePointToString(ch) {
            if (ch <= 0xFFFF) {
                return String.fromCharCode(ch);
            }
            ch -= 0x10000;
            return String.fromCharCode((ch >> 10) + 0xD800, (ch & 0x03FF) + 0xDC00);
        }
        /**
         * Validate the flags part of a given RegExpLiteral.
         *
         * @param {RegExpValidationState} state The state to validate RegExp.
         * @returns {void}
         */
        pp$8.validateRegExpFlags = function (state) {
            var validFlags = state.validFlags;
            var flags = state.flags;
            for (var i = 0; i < flags.length; i++) {
                var flag = flags.charAt(i);
                if (validFlags.indexOf(flag) === -1) {
                    this.raise(state.start, "Invalid regular expression flag");
                }
                if (flags.indexOf(flag, i + 1) > -1) {
                    this.raise(state.start, "Duplicate regular expression flag");
                }
            }
        };
        /**
         * Validate the pattern part of a given RegExpLiteral.
         *
         * @param {RegExpValidationState} state The state to validate RegExp.
         * @returns {void}
         */
        pp$8.validateRegExpPattern = function (state) {
            this.regexp_pattern(state);
            // The goal symbol for the parse is |Pattern[~U, ~N]|. If the result of
            // parsing contains a |GroupName|, reparse with the goal symbol
            // |Pattern[~U, +N]| and use this result instead. Throw a *SyntaxError*
            // exception if _P_ did not conform to the grammar, if any elements of _P_
            // were not matched by the parse, or if any Early Error conditions exist.
            if (!state.switchN && this.options.ecmaVersion >= 9 && state.groupNames.length > 0) {
                state.switchN = true;
                this.regexp_pattern(state);
            }
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-Pattern
        pp$8.regexp_pattern = function (state) {
            state.pos = 0;
            state.lastIntValue = 0;
            state.lastStringValue = "";
            state.lastAssertionIsQuantifiable = false;
            state.numCapturingParens = 0;
            state.maxBackReference = 0;
            state.groupNames.length = 0;
            state.backReferenceNames.length = 0;
            this.regexp_disjunction(state);
            if (state.pos !== state.source.length) {
                // Make the same messages as V8.
                if (state.eat(0x29 /* ) */)) {
                    state.raise("Unmatched ')'");
                }
                if (state.eat(0x5D /* ] */) || state.eat(0x7D /* } */)) {
                    state.raise("Lone quantifier brackets");
                }
            }
            if (state.maxBackReference > state.numCapturingParens) {
                state.raise("Invalid escape");
            }
            for (var i = 0, list = state.backReferenceNames; i < list.length; i += 1) {
                var name = list[i];
                if (state.groupNames.indexOf(name) === -1) {
                    state.raise("Invalid named capture referenced");
                }
            }
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-Disjunction
        pp$8.regexp_disjunction = function (state) {
            this.regexp_alternative(state);
            while (state.eat(0x7C /* | */)) {
                this.regexp_alternative(state);
            }
            // Make the same message as V8.
            if (this.regexp_eatQuantifier(state, true)) {
                state.raise("Nothing to repeat");
            }
            if (state.eat(0x7B /* { */)) {
                state.raise("Lone quantifier brackets");
            }
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-Alternative
        pp$8.regexp_alternative = function (state) {
            while (state.pos < state.source.length && this.regexp_eatTerm(state)) { }
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-Term
        pp$8.regexp_eatTerm = function (state) {
            if (this.regexp_eatAssertion(state)) {
                // Handle `QuantifiableAssertion Quantifier` alternative.
                // `state.lastAssertionIsQuantifiable` is true if the last eaten Assertion
                // is a QuantifiableAssertion.
                if (state.lastAssertionIsQuantifiable && this.regexp_eatQuantifier(state)) {
                    // Make the same message as V8.
                    if (state.switchU) {
                        state.raise("Invalid quantifier");
                    }
                }
                return true;
            }
            if (state.switchU ? this.regexp_eatAtom(state) : this.regexp_eatExtendedAtom(state)) {
                this.regexp_eatQuantifier(state);
                return true;
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-Assertion
        pp$8.regexp_eatAssertion = function (state) {
            var start = state.pos;
            state.lastAssertionIsQuantifiable = false;
            // ^, $
            if (state.eat(0x5E /* ^ */) || state.eat(0x24 /* $ */)) {
                return true;
            }
            // \b \B
            if (state.eat(0x5C /* \ */)) {
                if (state.eat(0x42 /* B */) || state.eat(0x62 /* b */)) {
                    return true;
                }
                state.pos = start;
            }
            // Lookahead / Lookbehind
            if (state.eat(0x28 /* ( */) && state.eat(0x3F /* ? */)) {
                var lookbehind = false;
                if (this.options.ecmaVersion >= 9) {
                    lookbehind = state.eat(0x3C /* < */);
                }
                if (state.eat(0x3D /* = */) || state.eat(0x21 /* ! */)) {
                    this.regexp_disjunction(state);
                    if (!state.eat(0x29 /* ) */)) {
                        state.raise("Unterminated group");
                    }
                    state.lastAssertionIsQuantifiable = !lookbehind;
                    return true;
                }
            }
            state.pos = start;
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-Quantifier
        pp$8.regexp_eatQuantifier = function (state, noError) {
            if (noError === void 0)
                noError = false;
            if (this.regexp_eatQuantifierPrefix(state, noError)) {
                state.eat(0x3F /* ? */);
                return true;
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-QuantifierPrefix
        pp$8.regexp_eatQuantifierPrefix = function (state, noError) {
            return (state.eat(0x2A /* * */) ||
                state.eat(0x2B /* + */) ||
                state.eat(0x3F /* ? */) ||
                this.regexp_eatBracedQuantifier(state, noError));
        };
        pp$8.regexp_eatBracedQuantifier = function (state, noError) {
            var start = state.pos;
            if (state.eat(0x7B /* { */)) {
                var min = 0, max = -1;
                if (this.regexp_eatDecimalDigits(state)) {
                    min = state.lastIntValue;
                    if (state.eat(0x2C /* , */) && this.regexp_eatDecimalDigits(state)) {
                        max = state.lastIntValue;
                    }
                    if (state.eat(0x7D /* } */)) {
                        // SyntaxError in https://www.ecma-international.org/ecma-262/8.0/#sec-term
                        if (max !== -1 && max < min && !noError) {
                            state.raise("numbers out of order in {} quantifier");
                        }
                        return true;
                    }
                }
                if (state.switchU && !noError) {
                    state.raise("Incomplete quantifier");
                }
                state.pos = start;
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-Atom
        pp$8.regexp_eatAtom = function (state) {
            return (this.regexp_eatPatternCharacters(state) ||
                state.eat(0x2E /* . */) ||
                this.regexp_eatReverseSolidusAtomEscape(state) ||
                this.regexp_eatCharacterClass(state) ||
                this.regexp_eatUncapturingGroup(state) ||
                this.regexp_eatCapturingGroup(state));
        };
        pp$8.regexp_eatReverseSolidusAtomEscape = function (state) {
            var start = state.pos;
            if (state.eat(0x5C /* \ */)) {
                if (this.regexp_eatAtomEscape(state)) {
                    return true;
                }
                state.pos = start;
            }
            return false;
        };
        pp$8.regexp_eatUncapturingGroup = function (state) {
            var start = state.pos;
            if (state.eat(0x28 /* ( */)) {
                if (state.eat(0x3F /* ? */) && state.eat(0x3A /* : */)) {
                    this.regexp_disjunction(state);
                    if (state.eat(0x29 /* ) */)) {
                        return true;
                    }
                    state.raise("Unterminated group");
                }
                state.pos = start;
            }
            return false;
        };
        pp$8.regexp_eatCapturingGroup = function (state) {
            if (state.eat(0x28 /* ( */)) {
                if (this.options.ecmaVersion >= 9) {
                    this.regexp_groupSpecifier(state);
                }
                else if (state.current() === 0x3F /* ? */) {
                    state.raise("Invalid group");
                }
                this.regexp_disjunction(state);
                if (state.eat(0x29 /* ) */)) {
                    state.numCapturingParens += 1;
                    return true;
                }
                state.raise("Unterminated group");
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-ExtendedAtom
        pp$8.regexp_eatExtendedAtom = function (state) {
            return (state.eat(0x2E /* . */) ||
                this.regexp_eatReverseSolidusAtomEscape(state) ||
                this.regexp_eatCharacterClass(state) ||
                this.regexp_eatUncapturingGroup(state) ||
                this.regexp_eatCapturingGroup(state) ||
                this.regexp_eatInvalidBracedQuantifier(state) ||
                this.regexp_eatExtendedPatternCharacter(state));
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-InvalidBracedQuantifier
        pp$8.regexp_eatInvalidBracedQuantifier = function (state) {
            if (this.regexp_eatBracedQuantifier(state, true)) {
                state.raise("Nothing to repeat");
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-SyntaxCharacter
        pp$8.regexp_eatSyntaxCharacter = function (state) {
            var ch = state.current();
            if (isSyntaxCharacter(ch)) {
                state.lastIntValue = ch;
                state.advance();
                return true;
            }
            return false;
        };
        function isSyntaxCharacter(ch) {
            return (ch === 0x24 /* $ */ ||
                ch >= 0x28 /* ( */ && ch <= 0x2B /* + */ ||
                ch === 0x2E /* . */ ||
                ch === 0x3F /* ? */ ||
                ch >= 0x5B /* [ */ && ch <= 0x5E /* ^ */ ||
                ch >= 0x7B /* { */ && ch <= 0x7D /* } */);
        }
        // https://www.ecma-international.org/ecma-262/8.0/#prod-PatternCharacter
        // But eat eager.
        pp$8.regexp_eatPatternCharacters = function (state) {
            var start = state.pos;
            var ch = 0;
            while ((ch = state.current()) !== -1 && !isSyntaxCharacter(ch)) {
                state.advance();
            }
            return state.pos !== start;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-ExtendedPatternCharacter
        pp$8.regexp_eatExtendedPatternCharacter = function (state) {
            var ch = state.current();
            if (ch !== -1 &&
                ch !== 0x24 /* $ */ &&
                !(ch >= 0x28 /* ( */ && ch <= 0x2B /* + */) &&
                ch !== 0x2E /* . */ &&
                ch !== 0x3F /* ? */ &&
                ch !== 0x5B /* [ */ &&
                ch !== 0x5E /* ^ */ &&
                ch !== 0x7C /* | */) {
                state.advance();
                return true;
            }
            return false;
        };
        // GroupSpecifier[U] ::
        //   [empty]
        //   `?` GroupName[?U]
        pp$8.regexp_groupSpecifier = function (state) {
            if (state.eat(0x3F /* ? */)) {
                if (this.regexp_eatGroupName(state)) {
                    if (state.groupNames.indexOf(state.lastStringValue) !== -1) {
                        state.raise("Duplicate capture group name");
                    }
                    state.groupNames.push(state.lastStringValue);
                    return;
                }
                state.raise("Invalid group");
            }
        };
        // GroupName[U] ::
        //   `<` RegExpIdentifierName[?U] `>`
        // Note: this updates `state.lastStringValue` property with the eaten name.
        pp$8.regexp_eatGroupName = function (state) {
            state.lastStringValue = "";
            if (state.eat(0x3C /* < */)) {
                if (this.regexp_eatRegExpIdentifierName(state) && state.eat(0x3E /* > */)) {
                    return true;
                }
                state.raise("Invalid capture group name");
            }
            return false;
        };
        // RegExpIdentifierName[U] ::
        //   RegExpIdentifierStart[?U]
        //   RegExpIdentifierName[?U] RegExpIdentifierPart[?U]
        // Note: this updates `state.lastStringValue` property with the eaten name.
        pp$8.regexp_eatRegExpIdentifierName = function (state) {
            state.lastStringValue = "";
            if (this.regexp_eatRegExpIdentifierStart(state)) {
                state.lastStringValue += codePointToString(state.lastIntValue);
                while (this.regexp_eatRegExpIdentifierPart(state)) {
                    state.lastStringValue += codePointToString(state.lastIntValue);
                }
                return true;
            }
            return false;
        };
        // RegExpIdentifierStart[U] ::
        //   UnicodeIDStart
        //   `$`
        //   `_`
        //   `\` RegExpUnicodeEscapeSequence[?U]
        pp$8.regexp_eatRegExpIdentifierStart = function (state) {
            var start = state.pos;
            var ch = state.current();
            state.advance();
            if (ch === 0x5C /* \ */ && this.regexp_eatRegExpUnicodeEscapeSequence(state)) {
                ch = state.lastIntValue;
            }
            if (isRegExpIdentifierStart(ch)) {
                state.lastIntValue = ch;
                return true;
            }
            state.pos = start;
            return false;
        };
        function isRegExpIdentifierStart(ch) {
            return isIdentifierStart(ch, true) || ch === 0x24 /* $ */ || ch === 0x5F; /* _ */
        }
        // RegExpIdentifierPart[U] ::
        //   UnicodeIDContinue
        //   `$`
        //   `_`
        //   `\` RegExpUnicodeEscapeSequence[?U]
        //   <ZWNJ>
        //   <ZWJ>
        pp$8.regexp_eatRegExpIdentifierPart = function (state) {
            var start = state.pos;
            var ch = state.current();
            state.advance();
            if (ch === 0x5C /* \ */ && this.regexp_eatRegExpUnicodeEscapeSequence(state)) {
                ch = state.lastIntValue;
            }
            if (isRegExpIdentifierPart(ch)) {
                state.lastIntValue = ch;
                return true;
            }
            state.pos = start;
            return false;
        };
        function isRegExpIdentifierPart(ch) {
            return isIdentifierChar(ch, true) || ch === 0x24 /* $ */ || ch === 0x5F /* _ */ || ch === 0x200C /* <ZWNJ> */ || ch === 0x200D; /* <ZWJ> */
        }
        // https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-AtomEscape
        pp$8.regexp_eatAtomEscape = function (state) {
            if (this.regexp_eatBackReference(state) ||
                this.regexp_eatCharacterClassEscape(state) ||
                this.regexp_eatCharacterEscape(state) ||
                (state.switchN && this.regexp_eatKGroupName(state))) {
                return true;
            }
            if (state.switchU) {
                // Make the same message as V8.
                if (state.current() === 0x63 /* c */) {
                    state.raise("Invalid unicode escape");
                }
                state.raise("Invalid escape");
            }
            return false;
        };
        pp$8.regexp_eatBackReference = function (state) {
            var start = state.pos;
            if (this.regexp_eatDecimalEscape(state)) {
                var n = state.lastIntValue;
                if (state.switchU) {
                    // For SyntaxError in https://www.ecma-international.org/ecma-262/8.0/#sec-atomescape
                    if (n > state.maxBackReference) {
                        state.maxBackReference = n;
                    }
                    return true;
                }
                if (n <= state.numCapturingParens) {
                    return true;
                }
                state.pos = start;
            }
            return false;
        };
        pp$8.regexp_eatKGroupName = function (state) {
            if (state.eat(0x6B /* k */)) {
                if (this.regexp_eatGroupName(state)) {
                    state.backReferenceNames.push(state.lastStringValue);
                    return true;
                }
                state.raise("Invalid named reference");
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-CharacterEscape
        pp$8.regexp_eatCharacterEscape = function (state) {
            return (this.regexp_eatControlEscape(state) ||
                this.regexp_eatCControlLetter(state) ||
                this.regexp_eatZero(state) ||
                this.regexp_eatHexEscapeSequence(state) ||
                this.regexp_eatRegExpUnicodeEscapeSequence(state) ||
                (!state.switchU && this.regexp_eatLegacyOctalEscapeSequence(state)) ||
                this.regexp_eatIdentityEscape(state));
        };
        pp$8.regexp_eatCControlLetter = function (state) {
            var start = state.pos;
            if (state.eat(0x63 /* c */)) {
                if (this.regexp_eatControlLetter(state)) {
                    return true;
                }
                state.pos = start;
            }
            return false;
        };
        pp$8.regexp_eatZero = function (state) {
            if (state.current() === 0x30 /* 0 */ && !isDecimalDigit(state.lookahead())) {
                state.lastIntValue = 0;
                state.advance();
                return true;
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-ControlEscape
        pp$8.regexp_eatControlEscape = function (state) {
            var ch = state.current();
            if (ch === 0x74 /* t */) {
                state.lastIntValue = 0x09; /* \t */
                state.advance();
                return true;
            }
            if (ch === 0x6E /* n */) {
                state.lastIntValue = 0x0A; /* \n */
                state.advance();
                return true;
            }
            if (ch === 0x76 /* v */) {
                state.lastIntValue = 0x0B; /* \v */
                state.advance();
                return true;
            }
            if (ch === 0x66 /* f */) {
                state.lastIntValue = 0x0C; /* \f */
                state.advance();
                return true;
            }
            if (ch === 0x72 /* r */) {
                state.lastIntValue = 0x0D; /* \r */
                state.advance();
                return true;
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-ControlLetter
        pp$8.regexp_eatControlLetter = function (state) {
            var ch = state.current();
            if (isControlLetter(ch)) {
                state.lastIntValue = ch % 0x20;
                state.advance();
                return true;
            }
            return false;
        };
        function isControlLetter(ch) {
            return ((ch >= 0x41 /* A */ && ch <= 0x5A /* Z */) ||
                (ch >= 0x61 /* a */ && ch <= 0x7A /* z */));
        }
        // https://www.ecma-international.org/ecma-262/8.0/#prod-RegExpUnicodeEscapeSequence
        pp$8.regexp_eatRegExpUnicodeEscapeSequence = function (state) {
            var start = state.pos;
            if (state.eat(0x75 /* u */)) {
                if (this.regexp_eatFixedHexDigits(state, 4)) {
                    var lead = state.lastIntValue;
                    if (state.switchU && lead >= 0xD800 && lead <= 0xDBFF) {
                        var leadSurrogateEnd = state.pos;
                        if (state.eat(0x5C /* \ */) && state.eat(0x75 /* u */) && this.regexp_eatFixedHexDigits(state, 4)) {
                            var trail = state.lastIntValue;
                            if (trail >= 0xDC00 && trail <= 0xDFFF) {
                                state.lastIntValue = (lead - 0xD800) * 0x400 + (trail - 0xDC00) + 0x10000;
                                return true;
                            }
                        }
                        state.pos = leadSurrogateEnd;
                        state.lastIntValue = lead;
                    }
                    return true;
                }
                if (state.switchU &&
                    state.eat(0x7B /* { */) &&
                    this.regexp_eatHexDigits(state) &&
                    state.eat(0x7D /* } */) &&
                    isValidUnicode(state.lastIntValue)) {
                    return true;
                }
                if (state.switchU) {
                    state.raise("Invalid unicode escape");
                }
                state.pos = start;
            }
            return false;
        };
        function isValidUnicode(ch) {
            return ch >= 0 && ch <= 0x10FFFF;
        }
        // https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-IdentityEscape
        pp$8.regexp_eatIdentityEscape = function (state) {
            if (state.switchU) {
                if (this.regexp_eatSyntaxCharacter(state)) {
                    return true;
                }
                if (state.eat(0x2F /* / */)) {
                    state.lastIntValue = 0x2F; /* / */
                    return true;
                }
                return false;
            }
            var ch = state.current();
            if (ch !== 0x63 /* c */ && (!state.switchN || ch !== 0x6B /* k */)) {
                state.lastIntValue = ch;
                state.advance();
                return true;
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-DecimalEscape
        pp$8.regexp_eatDecimalEscape = function (state) {
            state.lastIntValue = 0;
            var ch = state.current();
            if (ch >= 0x31 /* 1 */ && ch <= 0x39 /* 9 */) {
                do {
                    state.lastIntValue = 10 * state.lastIntValue + (ch - 0x30 /* 0 */);
                    state.advance();
                } while ((ch = state.current()) >= 0x30 /* 0 */ && ch <= 0x39 /* 9 */);
                return true;
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-CharacterClassEscape
        pp$8.regexp_eatCharacterClassEscape = function (state) {
            var ch = state.current();
            if (isCharacterClassEscape(ch)) {
                state.lastIntValue = -1;
                state.advance();
                return true;
            }
            if (state.switchU &&
                this.options.ecmaVersion >= 9 &&
                (ch === 0x50 /* P */ || ch === 0x70 /* p */)) {
                state.lastIntValue = -1;
                state.advance();
                if (state.eat(0x7B /* { */) &&
                    this.regexp_eatUnicodePropertyValueExpression(state) &&
                    state.eat(0x7D /* } */)) {
                    return true;
                }
                state.raise("Invalid property name");
            }
            return false;
        };
        function isCharacterClassEscape(ch) {
            return (ch === 0x64 /* d */ ||
                ch === 0x44 /* D */ ||
                ch === 0x73 /* s */ ||
                ch === 0x53 /* S */ ||
                ch === 0x77 /* w */ ||
                ch === 0x57 /* W */);
        }
        // UnicodePropertyValueExpression ::
        //   UnicodePropertyName `=` UnicodePropertyValue
        //   LoneUnicodePropertyNameOrValue
        pp$8.regexp_eatUnicodePropertyValueExpression = function (state) {
            var start = state.pos;
            // UnicodePropertyName `=` UnicodePropertyValue
            if (this.regexp_eatUnicodePropertyName(state) && state.eat(0x3D /* = */)) {
                var name = state.lastStringValue;
                if (this.regexp_eatUnicodePropertyValue(state)) {
                    var value = state.lastStringValue;
                    this.regexp_validateUnicodePropertyNameAndValue(state, name, value);
                    return true;
                }
            }
            state.pos = start;
            // LoneUnicodePropertyNameOrValue
            if (this.regexp_eatLoneUnicodePropertyNameOrValue(state)) {
                var nameOrValue = state.lastStringValue;
                this.regexp_validateUnicodePropertyNameOrValue(state, nameOrValue);
                return true;
            }
            return false;
        };
        pp$8.regexp_validateUnicodePropertyNameAndValue = function (state, name, value) {
            if (!has(state.unicodeProperties.nonBinary, name)) {
                state.raise("Invalid property name");
            }
            if (!state.unicodeProperties.nonBinary[name].test(value)) {
                state.raise("Invalid property value");
            }
        };
        pp$8.regexp_validateUnicodePropertyNameOrValue = function (state, nameOrValue) {
            if (!state.unicodeProperties.binary.test(nameOrValue)) {
                state.raise("Invalid property name");
            }
        };
        // UnicodePropertyName ::
        //   UnicodePropertyNameCharacters
        pp$8.regexp_eatUnicodePropertyName = function (state) {
            var ch = 0;
            state.lastStringValue = "";
            while (isUnicodePropertyNameCharacter(ch = state.current())) {
                state.lastStringValue += codePointToString(ch);
                state.advance();
            }
            return state.lastStringValue !== "";
        };
        function isUnicodePropertyNameCharacter(ch) {
            return isControlLetter(ch) || ch === 0x5F; /* _ */
        }
        // UnicodePropertyValue ::
        //   UnicodePropertyValueCharacters
        pp$8.regexp_eatUnicodePropertyValue = function (state) {
            var ch = 0;
            state.lastStringValue = "";
            while (isUnicodePropertyValueCharacter(ch = state.current())) {
                state.lastStringValue += codePointToString(ch);
                state.advance();
            }
            return state.lastStringValue !== "";
        };
        function isUnicodePropertyValueCharacter(ch) {
            return isUnicodePropertyNameCharacter(ch) || isDecimalDigit(ch);
        }
        // LoneUnicodePropertyNameOrValue ::
        //   UnicodePropertyValueCharacters
        pp$8.regexp_eatLoneUnicodePropertyNameOrValue = function (state) {
            return this.regexp_eatUnicodePropertyValue(state);
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-CharacterClass
        pp$8.regexp_eatCharacterClass = function (state) {
            if (state.eat(0x5B /* [ */)) {
                state.eat(0x5E /* ^ */);
                this.regexp_classRanges(state);
                if (state.eat(0x5D /* ] */)) {
                    return true;
                }
                // Unreachable since it threw "unterminated regular expression" error before.
                state.raise("Unterminated character class");
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-ClassRanges
        // https://www.ecma-international.org/ecma-262/8.0/#prod-NonemptyClassRanges
        // https://www.ecma-international.org/ecma-262/8.0/#prod-NonemptyClassRangesNoDash
        pp$8.regexp_classRanges = function (state) {
            while (this.regexp_eatClassAtom(state)) {
                var left = state.lastIntValue;
                if (state.eat(0x2D /* - */) && this.regexp_eatClassAtom(state)) {
                    var right = state.lastIntValue;
                    if (state.switchU && (left === -1 || right === -1)) {
                        state.raise("Invalid character class");
                    }
                    if (left !== -1 && right !== -1 && left > right) {
                        state.raise("Range out of order in character class");
                    }
                }
            }
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-ClassAtom
        // https://www.ecma-international.org/ecma-262/8.0/#prod-ClassAtomNoDash
        pp$8.regexp_eatClassAtom = function (state) {
            var start = state.pos;
            if (state.eat(0x5C /* \ */)) {
                if (this.regexp_eatClassEscape(state)) {
                    return true;
                }
                if (state.switchU) {
                    // Make the same message as V8.
                    var ch$1 = state.current();
                    if (ch$1 === 0x63 /* c */ || isOctalDigit(ch$1)) {
                        state.raise("Invalid class escape");
                    }
                    state.raise("Invalid escape");
                }
                state.pos = start;
            }
            var ch = state.current();
            if (ch !== 0x5D /* ] */) {
                state.lastIntValue = ch;
                state.advance();
                return true;
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-ClassEscape
        pp$8.regexp_eatClassEscape = function (state) {
            var start = state.pos;
            if (state.eat(0x62 /* b */)) {
                state.lastIntValue = 0x08; /* <BS> */
                return true;
            }
            if (state.switchU && state.eat(0x2D /* - */)) {
                state.lastIntValue = 0x2D; /* - */
                return true;
            }
            if (!state.switchU && state.eat(0x63 /* c */)) {
                if (this.regexp_eatClassControlLetter(state)) {
                    return true;
                }
                state.pos = start;
            }
            return (this.regexp_eatCharacterClassEscape(state) ||
                this.regexp_eatCharacterEscape(state));
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-ClassControlLetter
        pp$8.regexp_eatClassControlLetter = function (state) {
            var ch = state.current();
            if (isDecimalDigit(ch) || ch === 0x5F /* _ */) {
                state.lastIntValue = ch % 0x20;
                state.advance();
                return true;
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-HexEscapeSequence
        pp$8.regexp_eatHexEscapeSequence = function (state) {
            var start = state.pos;
            if (state.eat(0x78 /* x */)) {
                if (this.regexp_eatFixedHexDigits(state, 2)) {
                    return true;
                }
                if (state.switchU) {
                    state.raise("Invalid escape");
                }
                state.pos = start;
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-DecimalDigits
        pp$8.regexp_eatDecimalDigits = function (state) {
            var start = state.pos;
            var ch = 0;
            state.lastIntValue = 0;
            while (isDecimalDigit(ch = state.current())) {
                state.lastIntValue = 10 * state.lastIntValue + (ch - 0x30 /* 0 */);
                state.advance();
            }
            return state.pos !== start;
        };
        function isDecimalDigit(ch) {
            return ch >= 0x30 /* 0 */ && ch <= 0x39; /* 9 */
        }
        // https://www.ecma-international.org/ecma-262/8.0/#prod-HexDigits
        pp$8.regexp_eatHexDigits = function (state) {
            var start = state.pos;
            var ch = 0;
            state.lastIntValue = 0;
            while (isHexDigit(ch = state.current())) {
                state.lastIntValue = 16 * state.lastIntValue + hexToInt(ch);
                state.advance();
            }
            return state.pos !== start;
        };
        function isHexDigit(ch) {
            return ((ch >= 0x30 /* 0 */ && ch <= 0x39 /* 9 */) ||
                (ch >= 0x41 /* A */ && ch <= 0x46 /* F */) ||
                (ch >= 0x61 /* a */ && ch <= 0x66 /* f */));
        }
        function hexToInt(ch) {
            if (ch >= 0x41 /* A */ && ch <= 0x46 /* F */) {
                return 10 + (ch - 0x41 /* A */);
            }
            if (ch >= 0x61 /* a */ && ch <= 0x66 /* f */) {
                return 10 + (ch - 0x61 /* a */);
            }
            return ch - 0x30; /* 0 */
        }
        // https://www.ecma-international.org/ecma-262/8.0/#prod-annexB-LegacyOctalEscapeSequence
        // Allows only 0-377(octal) i.e. 0-255(decimal).
        pp$8.regexp_eatLegacyOctalEscapeSequence = function (state) {
            if (this.regexp_eatOctalDigit(state)) {
                var n1 = state.lastIntValue;
                if (this.regexp_eatOctalDigit(state)) {
                    var n2 = state.lastIntValue;
                    if (n1 <= 3 && this.regexp_eatOctalDigit(state)) {
                        state.lastIntValue = n1 * 64 + n2 * 8 + state.lastIntValue;
                    }
                    else {
                        state.lastIntValue = n1 * 8 + n2;
                    }
                }
                else {
                    state.lastIntValue = n1;
                }
                return true;
            }
            return false;
        };
        // https://www.ecma-international.org/ecma-262/8.0/#prod-OctalDigit
        pp$8.regexp_eatOctalDigit = function (state) {
            var ch = state.current();
            if (isOctalDigit(ch)) {
                state.lastIntValue = ch - 0x30; /* 0 */
                state.advance();
                return true;
            }
            state.lastIntValue = 0;
            return false;
        };
        function isOctalDigit(ch) {
            return ch >= 0x30 /* 0 */ && ch <= 0x37; /* 7 */
        }
        // https://www.ecma-international.org/ecma-262/8.0/#prod-Hex4Digits
        // https://www.ecma-international.org/ecma-262/8.0/#prod-HexDigit
        // And HexDigit HexDigit in https://www.ecma-international.org/ecma-262/8.0/#prod-HexEscapeSequence
        pp$8.regexp_eatFixedHexDigits = function (state, length) {
            var start = state.pos;
            state.lastIntValue = 0;
            for (var i = 0; i < length; ++i) {
                var ch = state.current();
                if (!isHexDigit(ch)) {
                    state.pos = start;
                    return false;
                }
                state.lastIntValue = 16 * state.lastIntValue + hexToInt(ch);
                state.advance();
            }
            return true;
        };
        // Object type used to represent tokens. Note that normally, tokens
        // simply exist as properties on the parser object. This is only
        // used for the onToken callback and the external tokenizer.
        var Token = function Token(p) {
            this.type = p.type;
            this.value = p.value;
            this.start = p.start;
            this.end = p.end;
            if (p.options.locations) {
                this.loc = new SourceLocation(p, p.startLoc, p.endLoc);
            }
            if (p.options.ranges) {
                this.range = [p.start, p.end];
            }
        };
        // ## Tokenizer
        var pp$9 = Parser.prototype;
        // Move to the next token
        pp$9.next = function (ignoreEscapeSequenceInKeyword) {
            if (!ignoreEscapeSequenceInKeyword && this.type.keyword && this.containsEsc) {
                this.raiseRecoverable(this.start, "Escape sequence in keyword " + this.type.keyword);
            }
            if (this.options.onToken) {
                this.options.onToken(new Token(this));
            }
            this.lastTokEnd = this.end;
            this.lastTokStart = this.start;
            this.lastTokEndLoc = this.endLoc;
            this.lastTokStartLoc = this.startLoc;
            this.nextToken();
        };
        pp$9.getToken = function () {
            this.next();
            return new Token(this);
        };
        // If we're in an ES6 environment, make parsers iterable
        if (typeof Symbol !== "undefined") {
            pp$9[Symbol.iterator] = function () {
                var this$1 = this;
                return {
                    next: function () {
                        var token = this$1.getToken();
                        return {
                            done: token.type === types.eof,
                            value: token
                        };
                    }
                };
            };
        }
        // Toggle strict mode. Re-reads the next number or string to please
        // pedantic tests (`"use strict"; 010;` should fail).
        pp$9.curContext = function () {
            return this.context[this.context.length - 1];
        };
        // Read a single token, updating the parser object's token-related
        // properties.
        pp$9.nextToken = function () {
            var curContext = this.curContext();
            if (!curContext || !curContext.preserveSpace) {
                this.skipSpace();
            }
            this.start = this.pos;
            if (this.options.locations) {
                this.startLoc = this.curPosition();
            }
            if (this.pos >= this.input.length) {
                return this.finishToken(types.eof);
            }
            if (curContext.override) {
                return curContext.override(this);
            }
            else {
                this.readToken(this.fullCharCodeAtPos());
            }
        };
        pp$9.readToken = function (code) {
            // Identifier or keyword. '\uXXXX' sequences are allowed in
            // identifiers, so '\' also dispatches to that.
            if (isIdentifierStart(code, this.options.ecmaVersion >= 6) || code === 92 /* '\' */) {
                return this.readWord();
            }
            return this.getTokenFromCode(code);
        };
        pp$9.fullCharCodeAtPos = function () {
            var code = this.input.charCodeAt(this.pos);
            if (code <= 0xd7ff || code >= 0xe000) {
                return code;
            }
            var next = this.input.charCodeAt(this.pos + 1);
            return (code << 10) + next - 0x35fdc00;
        };
        pp$9.skipBlockComment = function () {
            var startLoc = this.options.onComment && this.curPosition();
            var start = this.pos, end = this.input.indexOf("*/", this.pos += 2);
            if (end === -1) {
                this.raise(this.pos - 2, "Unterminated comment");
            }
            this.pos = end + 2;
            if (this.options.locations) {
                lineBreakG.lastIndex = start;
                var match;
                while ((match = lineBreakG.exec(this.input)) && match.index < this.pos) {
                    ++this.curLine;
                    this.lineStart = match.index + match[0].length;
                }
            }
            if (this.options.onComment) {
                this.options.onComment(true, this.input.slice(start + 2, end), start, this.pos, startLoc, this.curPosition());
            }
        };
        pp$9.skipLineComment = function (startSkip) {
            var start = this.pos;
            var startLoc = this.options.onComment && this.curPosition();
            var ch = this.input.charCodeAt(this.pos += startSkip);
            while (this.pos < this.input.length && !isNewLine(ch)) {
                ch = this.input.charCodeAt(++this.pos);
            }
            if (this.options.onComment) {
                this.options.onComment(false, this.input.slice(start + startSkip, this.pos), start, this.pos, startLoc, this.curPosition());
            }
        };
        // Called at the start of the parse and after every token. Skips
        // whitespace and comments, and.
        pp$9.skipSpace = function () {
            loop: while (this.pos < this.input.length) {
                var ch = this.input.charCodeAt(this.pos);
                switch (ch) {
                    case 32:
                    case 160: // ' '
                        ++this.pos;
                        break;
                    case 13:
                        if (this.input.charCodeAt(this.pos + 1) === 10) {
                            ++this.pos;
                        }
                    case 10:
                    case 8232:
                    case 8233:
                        ++this.pos;
                        if (this.options.locations) {
                            ++this.curLine;
                            this.lineStart = this.pos;
                        }
                        break;
                    case 47: // '/'
                        switch (this.input.charCodeAt(this.pos + 1)) {
                            case 42: // '*'
                                this.skipBlockComment();
                                break;
                            case 47:
                                this.skipLineComment(2);
                                break;
                            default:
                                break loop;
                        }
                        break;
                    default:
                        if (ch > 8 && ch < 14 || ch >= 5760 && nonASCIIwhitespace.test(String.fromCharCode(ch))) {
                            ++this.pos;
                        }
                        else {
                            break loop;
                        }
                }
            }
        };
        // Called at the end of every token. Sets `end`, `val`, and
        // maintains `context` and `exprAllowed`, and skips the space after
        // the token, so that the next one's `start` will point at the
        // right position.
        pp$9.finishToken = function (type, val) {
            this.end = this.pos;
            if (this.options.locations) {
                this.endLoc = this.curPosition();
            }
            var prevType = this.type;
            this.type = type;
            this.value = val;
            this.updateContext(prevType);
        };
        // ### Token reading
        // This is the function that is called to fetch the next token. It
        // is somewhat obscure, because it works in character codes rather
        // than characters, and because operator parsing has been inlined
        // into it.
        //
        // All in the name of speed.
        //
        pp$9.readToken_dot = function () {
            var next = this.input.charCodeAt(this.pos + 1);
            if (next >= 48 && next <= 57) {
                return this.readNumber(true);
            }
            var next2 = this.input.charCodeAt(this.pos + 2);
            if (this.options.ecmaVersion >= 6 && next === 46 && next2 === 46) { // 46 = dot '.'
                this.pos += 3;
                return this.finishToken(types.ellipsis);
            }
            else {
                ++this.pos;
                return this.finishToken(types.dot);
            }
        };
        pp$9.readToken_slash = function () {
            var next = this.input.charCodeAt(this.pos + 1);
            if (this.exprAllowed) {
                ++this.pos;
                return this.readRegexp();
            }
            if (next === 61) {
                return this.finishOp(types.assign, 2);
            }
            return this.finishOp(types.slash, 1);
        };
        pp$9.readToken_mult_modulo_exp = function (code) {
            var next = this.input.charCodeAt(this.pos + 1);
            var size = 1;
            var tokentype = code === 42 ? types.star : types.modulo;
            // exponentiation operator ** and **=
            if (this.options.ecmaVersion >= 7 && code === 42 && next === 42) {
                ++size;
                tokentype = types.starstar;
                next = this.input.charCodeAt(this.pos + 2);
            }
            if (next === 61) {
                return this.finishOp(types.assign, size + 1);
            }
            return this.finishOp(tokentype, size);
        };
        pp$9.readToken_pipe_amp = function (code) {
            var next = this.input.charCodeAt(this.pos + 1);
            if (next === code) {
                return this.finishOp(code === 124 ? types.logicalOR : types.logicalAND, 2);
            }
            if (next === 61) {
                return this.finishOp(types.assign, 2);
            }
            return this.finishOp(code === 124 ? types.bitwiseOR : types.bitwiseAND, 1);
        };
        pp$9.readToken_caret = function () {
            var next = this.input.charCodeAt(this.pos + 1);
            if (next === 61) {
                return this.finishOp(types.assign, 2);
            }
            return this.finishOp(types.bitwiseXOR, 1);
        };
        pp$9.readToken_plus_min = function (code) {
            var next = this.input.charCodeAt(this.pos + 1);
            if (next === code) {
                if (next === 45 && !this.inModule && this.input.charCodeAt(this.pos + 2) === 62 &&
                    (this.lastTokEnd === 0 || lineBreak.test(this.input.slice(this.lastTokEnd, this.pos)))) {
                    // A `-->` line comment
                    this.skipLineComment(3);
                    this.skipSpace();
                    return this.nextToken();
                }
                return this.finishOp(types.incDec, 2);
            }
            if (next === 61) {
                return this.finishOp(types.assign, 2);
            }
            return this.finishOp(types.plusMin, 1);
        };
        pp$9.readToken_lt_gt = function (code) {
            var next = this.input.charCodeAt(this.pos + 1);
            var size = 1;
            if (next === code) {
                size = code === 62 && this.input.charCodeAt(this.pos + 2) === 62 ? 3 : 2;
                if (this.input.charCodeAt(this.pos + size) === 61) {
                    return this.finishOp(types.assign, size + 1);
                }
                return this.finishOp(types.bitShift, size);
            }
            if (next === 33 && code === 60 && !this.inModule && this.input.charCodeAt(this.pos + 2) === 45 &&
                this.input.charCodeAt(this.pos + 3) === 45) {
                // `<!--`, an XML-style comment that should be interpreted as a line comment
                this.skipLineComment(4);
                this.skipSpace();
                return this.nextToken();
            }
            if (next === 61) {
                size = 2;
            }
            return this.finishOp(types.relational, size);
        };
        pp$9.readToken_eq_excl = function (code) {
            var next = this.input.charCodeAt(this.pos + 1);
            if (next === 61) {
                return this.finishOp(types.equality, this.input.charCodeAt(this.pos + 2) === 61 ? 3 : 2);
            }
            if (code === 61 && next === 62 && this.options.ecmaVersion >= 6) { // '=>'
                this.pos += 2;
                return this.finishToken(types.arrow);
            }
            return this.finishOp(code === 61 ? types.eq : types.prefix, 1);
        };
        pp$9.getTokenFromCode = function (code) {
            switch (code) {
                // The interpretation of a dot depends on whether it is followed
                // by a digit or another two dots.
                case 46: // '.'
                    return this.readToken_dot();
                // Punctuation tokens.
                case 40:
                    ++this.pos;
                    return this.finishToken(types.parenL);
                case 41:
                    ++this.pos;
                    return this.finishToken(types.parenR);
                case 59:
                    ++this.pos;
                    return this.finishToken(types.semi);
                case 44:
                    ++this.pos;
                    return this.finishToken(types.comma);
                case 91:
                    ++this.pos;
                    return this.finishToken(types.bracketL);
                case 93:
                    ++this.pos;
                    return this.finishToken(types.bracketR);
                case 123:
                    ++this.pos;
                    return this.finishToken(types.braceL);
                case 125:
                    ++this.pos;
                    return this.finishToken(types.braceR);
                case 58:
                    ++this.pos;
                    return this.finishToken(types.colon);
                case 63:
                    ++this.pos;
                    return this.finishToken(types.question);
                case 96: // '`'
                    if (this.options.ecmaVersion < 6) {
                        break;
                    }
                    ++this.pos;
                    return this.finishToken(types.backQuote);
                case 48: // '0'
                    var next = this.input.charCodeAt(this.pos + 1);
                    if (next === 120 || next === 88) {
                        return this.readRadixNumber(16);
                    } // '0x', '0X' - hex number
                    if (this.options.ecmaVersion >= 6) {
                        if (next === 111 || next === 79) {
                            return this.readRadixNumber(8);
                        } // '0o', '0O' - octal number
                        if (next === 98 || next === 66) {
                            return this.readRadixNumber(2);
                        } // '0b', '0B' - binary number
                    }
                // Anything else beginning with a digit is an integer, octal
                // number, or float.
                case 49:
                case 50:
                case 51:
                case 52:
                case 53:
                case 54:
                case 55:
                case 56:
                case 57: // 1-9
                    return this.readNumber(false);
                // Quotes produce strings.
                case 34:
                case 39: // '"', "'"
                    return this.readString(code);
                // Operators are parsed inline in tiny state machines. '=' (61) is
                // often referred to. `finishOp` simply skips the amount of
                // characters it is given as second argument, and returns a token
                // of the type given by its first argument.
                case 47: // '/'
                    return this.readToken_slash();
                case 37:
                case 42: // '%*'
                    return this.readToken_mult_modulo_exp(code);
                case 124:
                case 38: // '|&'
                    return this.readToken_pipe_amp(code);
                case 94: // '^'
                    return this.readToken_caret();
                case 43:
                case 45: // '+-'
                    return this.readToken_plus_min(code);
                case 60:
                case 62: // '<>'
                    return this.readToken_lt_gt(code);
                case 61:
                case 33: // '=!'
                    return this.readToken_eq_excl(code);
                case 126: // '~'
                    return this.finishOp(types.prefix, 1);
            }
            this.raise(this.pos, "Unexpected character '" + codePointToString$1(code) + "'");
        };
        pp$9.finishOp = function (type, size) {
            var str = this.input.slice(this.pos, this.pos + size);
            this.pos += size;
            return this.finishToken(type, str);
        };
        pp$9.readRegexp = function () {
            var escaped, inClass, start = this.pos;
            for (;;) {
                if (this.pos >= this.input.length) {
                    this.raise(start, "Unterminated regular expression");
                }
                var ch = this.input.charAt(this.pos);
                if (lineBreak.test(ch)) {
                    this.raise(start, "Unterminated regular expression");
                }
                if (!escaped) {
                    if (ch === "[") {
                        inClass = true;
                    }
                    else if (ch === "]" && inClass) {
                        inClass = false;
                    }
                    else if (ch === "/" && !inClass) {
                        break;
                    }
                    escaped = ch === "\\";
                }
                else {
                    escaped = false;
                }
                ++this.pos;
            }
            var pattern = this.input.slice(start, this.pos);
            ++this.pos;
            var flagsStart = this.pos;
            var flags = this.readWord1();
            if (this.containsEsc) {
                this.unexpected(flagsStart);
            }
            // Validate pattern
            var state = this.regexpState || (this.regexpState = new RegExpValidationState(this));
            state.reset(start, pattern, flags);
            this.validateRegExpFlags(state);
            this.validateRegExpPattern(state);
            // Create Literal#value property value.
            var value = null;
            try {
                value = new RegExp(pattern, flags);
            }
            catch (e) {
                // ESTree requires null if it failed to instantiate RegExp object.
                // https://github.com/estree/estree/blob/a27003adf4fd7bfad44de9cef372a2eacd527b1c/es5.md#regexpliteral
            }
            return this.finishToken(types.regexp, { pattern: pattern, flags: flags, value: value });
        };
        // Read an integer in the given radix. Return null if zero digits
        // were read, the integer value otherwise. When `len` is given, this
        // will return `null` unless the integer has exactly `len` digits.
        pp$9.readInt = function (radix, len) {
            var start = this.pos, total = 0;
            for (var i = 0, e = len == null ? Infinity : len; i < e; ++i) {
                var code = this.input.charCodeAt(this.pos), val = (void 0);
                if (code >= 97) {
                    val = code - 97 + 10;
                } // a
                else if (code >= 65) {
                    val = code - 65 + 10;
                } // A
                else if (code >= 48 && code <= 57) {
                    val = code - 48;
                } // 0-9
                else {
                    val = Infinity;
                }
                if (val >= radix) {
                    break;
                }
                ++this.pos;
                total = total * radix + val;
            }
            if (this.pos === start || len != null && this.pos - start !== len) {
                return null;
            }
            return total;
        };
        pp$9.readRadixNumber = function (radix) {
            var start = this.pos;
            this.pos += 2; // 0x
            var val = this.readInt(radix);
            if (val == null) {
                this.raise(this.start + 2, "Expected number in radix " + radix);
            }
            if (this.options.ecmaVersion >= 11 && this.input.charCodeAt(this.pos) === 110) {
                val = typeof BigInt !== "undefined" ? BigInt(this.input.slice(start, this.pos)) : null;
                ++this.pos;
            }
            else if (isIdentifierStart(this.fullCharCodeAtPos())) {
                this.raise(this.pos, "Identifier directly after number");
            }
            return this.finishToken(types.num, val);
        };
        // Read an integer, octal integer, or floating-point number.
        pp$9.readNumber = function (startsWithDot) {
            var start = this.pos;
            if (!startsWithDot && this.readInt(10) === null) {
                this.raise(start, "Invalid number");
            }
            var octal = this.pos - start >= 2 && this.input.charCodeAt(start) === 48;
            if (octal && this.strict) {
                this.raise(start, "Invalid number");
            }
            var next = this.input.charCodeAt(this.pos);
            if (!octal && !startsWithDot && this.options.ecmaVersion >= 11 && next === 110) {
                var str$1 = this.input.slice(start, this.pos);
                var val$1 = typeof BigInt !== "undefined" ? BigInt(str$1) : null;
                ++this.pos;
                if (isIdentifierStart(this.fullCharCodeAtPos())) {
                    this.raise(this.pos, "Identifier directly after number");
                }
                return this.finishToken(types.num, val$1);
            }
            if (octal && /[89]/.test(this.input.slice(start, this.pos))) {
                octal = false;
            }
            if (next === 46 && !octal) { // '.'
                ++this.pos;
                this.readInt(10);
                next = this.input.charCodeAt(this.pos);
            }
            if ((next === 69 || next === 101) && !octal) { // 'eE'
                next = this.input.charCodeAt(++this.pos);
                if (next === 43 || next === 45) {
                    ++this.pos;
                } // '+-'
                if (this.readInt(10) === null) {
                    this.raise(start, "Invalid number");
                }
            }
            if (isIdentifierStart(this.fullCharCodeAtPos())) {
                this.raise(this.pos, "Identifier directly after number");
            }
            var str = this.input.slice(start, this.pos);
            var val = octal ? parseInt(str, 8) : parseFloat(str);
            return this.finishToken(types.num, val);
        };
        // Read a string value, interpreting backslash-escapes.
        pp$9.readCodePoint = function () {
            var ch = this.input.charCodeAt(this.pos), code;
            if (ch === 123) { // '{'
                if (this.options.ecmaVersion < 6) {
                    this.unexpected();
                }
                var codePos = ++this.pos;
                code = this.readHexChar(this.input.indexOf("}", this.pos) - this.pos);
                ++this.pos;
                if (code > 0x10FFFF) {
                    this.invalidStringToken(codePos, "Code point out of bounds");
                }
            }
            else {
                code = this.readHexChar(4);
            }
            return code;
        };
        function codePointToString$1(code) {
            // UTF-16 Decoding
            if (code <= 0xFFFF) {
                return String.fromCharCode(code);
            }
            code -= 0x10000;
            return String.fromCharCode((code >> 10) + 0xD800, (code & 1023) + 0xDC00);
        }
        pp$9.readString = function (quote) {
            var out = "", chunkStart = ++this.pos;
            for (;;) {
                if (this.pos >= this.input.length) {
                    this.raise(this.start, "Unterminated string constant");
                }
                var ch = this.input.charCodeAt(this.pos);
                if (ch === quote) {
                    break;
                }
                if (ch === 92) { // '\'
                    out += this.input.slice(chunkStart, this.pos);
                    out += this.readEscapedChar(false);
                    chunkStart = this.pos;
                }
                else {
                    if (isNewLine(ch, this.options.ecmaVersion >= 10)) {
                        this.raise(this.start, "Unterminated string constant");
                    }
                    ++this.pos;
                }
            }
            out += this.input.slice(chunkStart, this.pos++);
            return this.finishToken(types.string, out);
        };
        // Reads template string tokens.
        var INVALID_TEMPLATE_ESCAPE_ERROR = {};
        pp$9.tryReadTemplateToken = function () {
            this.inTemplateElement = true;
            try {
                this.readTmplToken();
            }
            catch (err) {
                if (err === INVALID_TEMPLATE_ESCAPE_ERROR) {
                    this.readInvalidTemplateToken();
                }
                else {
                    throw err;
                }
            }
            this.inTemplateElement = false;
        };
        pp$9.invalidStringToken = function (position, message) {
            if (this.inTemplateElement && this.options.ecmaVersion >= 9) {
                throw INVALID_TEMPLATE_ESCAPE_ERROR;
            }
            else {
                this.raise(position, message);
            }
        };
        pp$9.readTmplToken = function () {
            var out = "", chunkStart = this.pos;
            for (;;) {
                if (this.pos >= this.input.length) {
                    this.raise(this.start, "Unterminated template");
                }
                var ch = this.input.charCodeAt(this.pos);
                if (ch === 96 || ch === 36 && this.input.charCodeAt(this.pos + 1) === 123) { // '`', '${'
                    if (this.pos === this.start && (this.type === types.template || this.type === types.invalidTemplate)) {
                        if (ch === 36) {
                            this.pos += 2;
                            return this.finishToken(types.dollarBraceL);
                        }
                        else {
                            ++this.pos;
                            return this.finishToken(types.backQuote);
                        }
                    }
                    out += this.input.slice(chunkStart, this.pos);
                    return this.finishToken(types.template, out);
                }
                if (ch === 92) { // '\'
                    out += this.input.slice(chunkStart, this.pos);
                    out += this.readEscapedChar(true);
                    chunkStart = this.pos;
                }
                else if (isNewLine(ch)) {
                    out += this.input.slice(chunkStart, this.pos);
                    ++this.pos;
                    switch (ch) {
                        case 13:
                            if (this.input.charCodeAt(this.pos) === 10) {
                                ++this.pos;
                            }
                        case 10:
                            out += "\n";
                            break;
                        default:
                            out += String.fromCharCode(ch);
                            break;
                    }
                    if (this.options.locations) {
                        ++this.curLine;
                        this.lineStart = this.pos;
                    }
                    chunkStart = this.pos;
                }
                else {
                    ++this.pos;
                }
            }
        };
        // Reads a template token to search for the end, without validating any escape sequences
        pp$9.readInvalidTemplateToken = function () {
            for (; this.pos < this.input.length; this.pos++) {
                switch (this.input[this.pos]) {
                    case "\\":
                        ++this.pos;
                        break;
                    case "$":
                        if (this.input[this.pos + 1] !== "{") {
                            break;
                        }
                    // falls through
                    case "`":
                        return this.finishToken(types.invalidTemplate, this.input.slice(this.start, this.pos));
                    // no default
                }
            }
            this.raise(this.start, "Unterminated template");
        };
        // Used to read escaped characters
        pp$9.readEscapedChar = function (inTemplate) {
            var ch = this.input.charCodeAt(++this.pos);
            ++this.pos;
            switch (ch) {
                case 110: return "\n"; // 'n' -> '\n'
                case 114: return "\r"; // 'r' -> '\r'
                case 120: return String.fromCharCode(this.readHexChar(2)); // 'x'
                case 117: return codePointToString$1(this.readCodePoint()); // 'u'
                case 116: return "\t"; // 't' -> '\t'
                case 98: return "\b"; // 'b' -> '\b'
                case 118: return "\u000b"; // 'v' -> '\u000b'
                case 102: return "\f"; // 'f' -> '\f'
                case 13: if (this.input.charCodeAt(this.pos) === 10) {
                    ++this.pos;
                } // '\r\n'
                case 10: // ' \n'
                    if (this.options.locations) {
                        this.lineStart = this.pos;
                        ++this.curLine;
                    }
                    return "";
                case 56:
                case 57:
                    if (inTemplate) {
                        var codePos = this.pos - 1;
                        this.invalidStringToken(codePos, "Invalid escape sequence in template string");
                        return null;
                    }
                default:
                    if (ch >= 48 && ch <= 55) {
                        var octalStr = this.input.substr(this.pos - 1, 3).match(/^[0-7]+/)[0];
                        var octal = parseInt(octalStr, 8);
                        if (octal > 255) {
                            octalStr = octalStr.slice(0, -1);
                            octal = parseInt(octalStr, 8);
                        }
                        this.pos += octalStr.length - 1;
                        ch = this.input.charCodeAt(this.pos);
                        if ((octalStr !== "0" || ch === 56 || ch === 57) && (this.strict || inTemplate)) {
                            this.invalidStringToken(this.pos - 1 - octalStr.length, inTemplate
                                ? "Octal literal in template string"
                                : "Octal literal in strict mode");
                        }
                        return String.fromCharCode(octal);
                    }
                    if (isNewLine(ch)) {
                        // Unicode new line characters after \ get removed from output in both
                        // template literals and strings
                        return "";
                    }
                    return String.fromCharCode(ch);
            }
        };
        // Used to read character escape sequences ('\x', '\u', '\U').
        pp$9.readHexChar = function (len) {
            var codePos = this.pos;
            var n = this.readInt(16, len);
            if (n === null) {
                this.invalidStringToken(codePos, "Bad character escape sequence");
            }
            return n;
        };
        // Read an identifier, and return it as a string. Sets `this.containsEsc`
        // to whether the word contained a '\u' escape.
        //
        // Incrementally adds only escaped chars, adding other chunks as-is
        // as a micro-optimization.
        pp$9.readWord1 = function () {
            this.containsEsc = false;
            var word = "", first = true, chunkStart = this.pos;
            var astral = this.options.ecmaVersion >= 6;
            while (this.pos < this.input.length) {
                var ch = this.fullCharCodeAtPos();
                if (isIdentifierChar(ch, astral)) {
                    this.pos += ch <= 0xffff ? 1 : 2;
                }
                else if (ch === 92) { // "\"
                    this.containsEsc = true;
                    word += this.input.slice(chunkStart, this.pos);
                    var escStart = this.pos;
                    if (this.input.charCodeAt(++this.pos) !== 117) // "u"
                     {
                        this.invalidStringToken(this.pos, "Expecting Unicode escape sequence \\uXXXX");
                    }
                    ++this.pos;
                    var esc = this.readCodePoint();
                    if (!(first ? isIdentifierStart : isIdentifierChar)(esc, astral)) {
                        this.invalidStringToken(escStart, "Invalid Unicode escape");
                    }
                    word += codePointToString$1(esc);
                    chunkStart = this.pos;
                }
                else {
                    break;
                }
                first = false;
            }
            return word + this.input.slice(chunkStart, this.pos);
        };
        // Read an identifier or keyword token. Will check for reserved
        // words when necessary.
        pp$9.readWord = function () {
            var word = this.readWord1();
            var type = types.name;
            if (this.keywords.test(word)) {
                type = keywords$1[word];
            }
            return this.finishToken(type, word);
        };
        // Acorn is a tiny, fast JavaScript parser written in JavaScript.
        var version = "7.1.0";
        Parser.acorn = {
            Parser: Parser,
            version: version,
            defaultOptions: defaultOptions,
            Position: Position,
            SourceLocation: SourceLocation,
            getLineInfo: getLineInfo,
            Node: Node,
            TokenType: TokenType,
            tokTypes: types,
            keywordTypes: keywords$1,
            TokContext: TokContext,
            tokContexts: types$1,
            isIdentifierChar: isIdentifierChar,
            isIdentifierStart: isIdentifierStart,
            Token: Token,
            isNewLine: isNewLine,
            lineBreak: lineBreak,
            lineBreakG: lineBreakG,
            nonASCIIwhitespace: nonASCIIwhitespace
        };
        // The main exported interface (under `self.acorn` when in the
        // browser) is a `parse` function that takes a code string and
        // returns an abstract syntax tree as specified by [Mozilla parser
        // API][api].
        //
        // [api]: https://developer.mozilla.org/en-US/docs/SpiderMonkey/Parser_API
        function parse(input, options) {
            return Parser.parse(input, options);
        }
        // This function tries to parse a single expression at a given
        // offset in a string. Useful for parsing mixed-language formats
        // that embed JavaScript expressions.
        function parseExpressionAt(input, pos, options) {
            return Parser.parseExpressionAt(input, pos, options);
        }
        // Acorn is organized as a tokenizer and a recursive-descent parser.
        // The `tokenizer` export provides an interface to the tokenizer.
        function tokenizer(input, options) {
            return Parser.tokenizer(input, options);
        }
        exports.Node = Node;
        exports.Parser = Parser;
        exports.Position = Position;
        exports.SourceLocation = SourceLocation;
        exports.TokContext = TokContext;
        exports.Token = Token;
        exports.TokenType = TokenType;
        exports.defaultOptions = defaultOptions;
        exports.getLineInfo = getLineInfo;
        exports.isIdentifierChar = isIdentifierChar;
        exports.isIdentifierStart = isIdentifierStart;
        exports.isNewLine = isNewLine;
        exports.keywordTypes = keywords$1;
        exports.lineBreak = lineBreak;
        exports.lineBreakG = lineBreakG;
        exports.nonASCIIwhitespace = nonASCIIwhitespace;
        exports.parse = parse;
        exports.parseExpressionAt = parseExpressionAt;
        exports.tokContexts = types$1;
        exports.tokTypes = types;
        exports.tokenizer = tokenizer;
        exports.version = version;
        Object.defineProperty(exports, '__esModule', { value: true });
    }));
},
/* acorn-jsx/index.js */ function _(require, module, exports) {
    const XHTMLEntities = require(105) /* ./xhtml */;
    const hexNumber = /^[\da-fA-F]+$/;
    const decimalNumber = /^\d+$/;
    // The map to `acorn-jsx` tokens from `acorn` namespace objects.
    const acornJsxMap = new WeakMap();
    // Get the original tokens for the given `acorn` namespace object.
    function getJsxTokens(acorn) {
        acorn = acorn.Parser.acorn || acorn;
        let acornJsx = acornJsxMap.get(acorn);
        if (!acornJsx) {
            const tt = acorn.tokTypes;
            const TokContext = acorn.TokContext;
            const TokenType = acorn.TokenType;
            const tc_oTag = new TokContext('<tag', false);
            const tc_cTag = new TokContext('</tag', false);
            const tc_expr = new TokContext('<tag>...</tag>', true, true);
            const tokContexts = {
                tc_oTag: tc_oTag,
                tc_cTag: tc_cTag,
                tc_expr: tc_expr
            };
            const tokTypes = {
                jsxName: new TokenType('jsxName'),
                jsxText: new TokenType('jsxText', { beforeExpr: true }),
                jsxTagStart: new TokenType('jsxTagStart'),
                jsxTagEnd: new TokenType('jsxTagEnd')
            };
            tokTypes.jsxTagStart.updateContext = function () {
                this.context.push(tc_expr); // treat as beginning of JSX expression
                this.context.push(tc_oTag); // start opening tag context
                this.exprAllowed = false;
            };
            tokTypes.jsxTagEnd.updateContext = function (prevType) {
                let out = this.context.pop();
                if (out === tc_oTag && prevType === tt.slash || out === tc_cTag) {
                    this.context.pop();
                    this.exprAllowed = this.curContext() === tc_expr;
                }
                else {
                    this.exprAllowed = true;
                }
            };
            acornJsx = { tokContexts: tokContexts, tokTypes: tokTypes };
            acornJsxMap.set(acorn, acornJsx);
        }
        return acornJsx;
    }
    // Transforms JSX element name to string.
    function getQualifiedJSXName(object) {
        if (!object)
            return object;
        if (object.type === 'JSXIdentifier')
            return object.name;
        if (object.type === 'JSXNamespacedName')
            return object.namespace.name + ':' + object.name.name;
        if (object.type === 'JSXMemberExpression')
            return getQualifiedJSXName(object.object) + '.' +
                getQualifiedJSXName(object.property);
    }
    module.exports = function (options) {
        options = options || {};
        return function (Parser) {
            return plugin({
                allowNamespaces: options.allowNamespaces !== false,
                allowNamespacedObjects: !!options.allowNamespacedObjects
            }, Parser);
        };
    };
    // This is `tokTypes` of the peer dep.
    // This can be different instances from the actual `tokTypes` this plugin uses.
    Object.defineProperty(module.exports, "tokTypes", {
        get: function get_tokTypes() {
            return getJsxTokens(require(103) /* acorn */).tokTypes;
        },
        configurable: true,
        enumerable: true
    });
    function plugin(options, Parser) {
        const acorn = Parser.acorn || require(103) /* acorn */;
        const acornJsx = getJsxTokens(acorn);
        const tt = acorn.tokTypes;
        const tok = acornJsx.tokTypes;
        const tokContexts = acorn.tokContexts;
        const tc_oTag = acornJsx.tokContexts.tc_oTag;
        const tc_cTag = acornJsx.tokContexts.tc_cTag;
        const tc_expr = acornJsx.tokContexts.tc_expr;
        const isNewLine = acorn.isNewLine;
        const isIdentifierStart = acorn.isIdentifierStart;
        const isIdentifierChar = acorn.isIdentifierChar;
        return class extends Parser {
            // Expose actual `tokTypes` and `tokContexts` to other plugins.
            static get acornJsx() {
                return acornJsx;
            }
            // Reads inline JSX contents token.
            jsx_readToken() {
                let out = '', chunkStart = this.pos;
                for (;;) {
                    if (this.pos >= this.input.length)
                        this.raise(this.start, 'Unterminated JSX contents');
                    let ch = this.input.charCodeAt(this.pos);
                    switch (ch) {
                        case 60: // '<'
                        case 123: // '{'
                            if (this.pos === this.start) {
                                if (ch === 60 && this.exprAllowed) {
                                    ++this.pos;
                                    return this.finishToken(tok.jsxTagStart);
                                }
                                return this.getTokenFromCode(ch);
                            }
                            out += this.input.slice(chunkStart, this.pos);
                            return this.finishToken(tok.jsxText, out);
                        case 38: // '&'
                            out += this.input.slice(chunkStart, this.pos);
                            out += this.jsx_readEntity();
                            chunkStart = this.pos;
                            break;
                        case 62: // '>'
                        case 125: // '}'
                            this.raise(this.pos, "Unexpected token `" + this.input[this.pos] + "`. Did you mean `" +
                                (ch === 62 ? "&gt;" : "&rbrace;") + "` or " + "`{\"" + this.input[this.pos] + "\"}" + "`?");
                        default:
                            if (isNewLine(ch)) {
                                out += this.input.slice(chunkStart, this.pos);
                                out += this.jsx_readNewLine(true);
                                chunkStart = this.pos;
                            }
                            else {
                                ++this.pos;
                            }
                    }
                }
            }
            jsx_readNewLine(normalizeCRLF) {
                let ch = this.input.charCodeAt(this.pos);
                let out;
                ++this.pos;
                if (ch === 13 && this.input.charCodeAt(this.pos) === 10) {
                    ++this.pos;
                    out = normalizeCRLF ? '\n' : '\r\n';
                }
                else {
                    out = String.fromCharCode(ch);
                }
                if (this.options.locations) {
                    ++this.curLine;
                    this.lineStart = this.pos;
                }
                return out;
            }
            jsx_readString(quote) {
                let out = '', chunkStart = ++this.pos;
                for (;;) {
                    if (this.pos >= this.input.length)
                        this.raise(this.start, 'Unterminated string constant');
                    let ch = this.input.charCodeAt(this.pos);
                    if (ch === quote)
                        break;
                    if (ch === 38) { // '&'
                        out += this.input.slice(chunkStart, this.pos);
                        out += this.jsx_readEntity();
                        chunkStart = this.pos;
                    }
                    else if (isNewLine(ch)) {
                        out += this.input.slice(chunkStart, this.pos);
                        out += this.jsx_readNewLine(false);
                        chunkStart = this.pos;
                    }
                    else {
                        ++this.pos;
                    }
                }
                out += this.input.slice(chunkStart, this.pos++);
                return this.finishToken(tt.string, out);
            }
            jsx_readEntity() {
                let str = '', count = 0, entity;
                let ch = this.input[this.pos];
                if (ch !== '&')
                    this.raise(this.pos, 'Entity must start with an ampersand');
                let startPos = ++this.pos;
                while (this.pos < this.input.length && count++ < 10) {
                    ch = this.input[this.pos++];
                    if (ch === ';') {
                        if (str[0] === '#') {
                            if (str[1] === 'x') {
                                str = str.substr(2);
                                if (hexNumber.test(str))
                                    entity = String.fromCharCode(parseInt(str, 16));
                            }
                            else {
                                str = str.substr(1);
                                if (decimalNumber.test(str))
                                    entity = String.fromCharCode(parseInt(str, 10));
                            }
                        }
                        else {
                            entity = XHTMLEntities[str];
                        }
                        break;
                    }
                    str += ch;
                }
                if (!entity) {
                    this.pos = startPos;
                    return '&';
                }
                return entity;
            }
            // Read a JSX identifier (valid tag or attribute name).
            //
            // Optimized version since JSX identifiers can't contain
            // escape characters and so can be read as single slice.
            // Also assumes that first character was already checked
            // by isIdentifierStart in readToken.
            jsx_readWord() {
                let ch, start = this.pos;
                do {
                    ch = this.input.charCodeAt(++this.pos);
                } while (isIdentifierChar(ch) || ch === 45); // '-'
                return this.finishToken(tok.jsxName, this.input.slice(start, this.pos));
            }
            // Parse next token as JSX identifier
            jsx_parseIdentifier() {
                let node = this.startNode();
                if (this.type === tok.jsxName)
                    node.name = this.value;
                else if (this.type.keyword)
                    node.name = this.type.keyword;
                else
                    this.unexpected();
                this.next();
                return this.finishNode(node, 'JSXIdentifier');
            }
            // Parse namespaced identifier.
            jsx_parseNamespacedName() {
                let startPos = this.start, startLoc = this.startLoc;
                let name = this.jsx_parseIdentifier();
                if (!options.allowNamespaces || !this.eat(tt.colon))
                    return name;
                var node = this.startNodeAt(startPos, startLoc);
                node.namespace = name;
                node.name = this.jsx_parseIdentifier();
                return this.finishNode(node, 'JSXNamespacedName');
            }
            // Parses element name in any form - namespaced, member
            // or single identifier.
            jsx_parseElementName() {
                if (this.type === tok.jsxTagEnd)
                    return '';
                let startPos = this.start, startLoc = this.startLoc;
                let node = this.jsx_parseNamespacedName();
                if (this.type === tt.dot && node.type === 'JSXNamespacedName' && !options.allowNamespacedObjects) {
                    this.unexpected();
                }
                while (this.eat(tt.dot)) {
                    let newNode = this.startNodeAt(startPos, startLoc);
                    newNode.object = node;
                    newNode.property = this.jsx_parseIdentifier();
                    node = this.finishNode(newNode, 'JSXMemberExpression');
                }
                return node;
            }
            // Parses any type of JSX attribute value.
            jsx_parseAttributeValue() {
                switch (this.type) {
                    case tt.braceL:
                        let node = this.jsx_parseExpressionContainer();
                        if (node.expression.type === 'JSXEmptyExpression')
                            this.raise(node.start, 'JSX attributes must only be assigned a non-empty expression');
                        return node;
                    case tok.jsxTagStart:
                    case tt.string:
                        return this.parseExprAtom();
                    default:
                        this.raise(this.start, 'JSX value should be either an expression or a quoted JSX text');
                }
            }
            // JSXEmptyExpression is unique type since it doesn't actually parse anything,
            // and so it should start at the end of last read token (left brace) and finish
            // at the beginning of the next one (right brace).
            jsx_parseEmptyExpression() {
                let node = this.startNodeAt(this.lastTokEnd, this.lastTokEndLoc);
                return this.finishNodeAt(node, 'JSXEmptyExpression', this.start, this.startLoc);
            }
            // Parses JSX expression enclosed into curly brackets.
            jsx_parseExpressionContainer() {
                let node = this.startNode();
                this.next();
                node.expression = this.type === tt.braceR
                    ? this.jsx_parseEmptyExpression()
                    : this.parseExpression();
                this.expect(tt.braceR);
                return this.finishNode(node, 'JSXExpressionContainer');
            }
            // Parses following JSX attribute name-value pair.
            jsx_parseAttribute() {
                let node = this.startNode();
                if (this.eat(tt.braceL)) {
                    this.expect(tt.ellipsis);
                    node.argument = this.parseMaybeAssign();
                    this.expect(tt.braceR);
                    return this.finishNode(node, 'JSXSpreadAttribute');
                }
                node.name = this.jsx_parseNamespacedName();
                node.value = this.eat(tt.eq) ? this.jsx_parseAttributeValue() : null;
                return this.finishNode(node, 'JSXAttribute');
            }
            // Parses JSX opening tag starting after '<'.
            jsx_parseOpeningElementAt(startPos, startLoc) {
                let node = this.startNodeAt(startPos, startLoc);
                node.attributes = [];
                let nodeName = this.jsx_parseElementName();
                if (nodeName)
                    node.name = nodeName;
                while (this.type !== tt.slash && this.type !== tok.jsxTagEnd)
                    node.attributes.push(this.jsx_parseAttribute());
                node.selfClosing = this.eat(tt.slash);
                this.expect(tok.jsxTagEnd);
                return this.finishNode(node, nodeName ? 'JSXOpeningElement' : 'JSXOpeningFragment');
            }
            // Parses JSX closing tag starting after '</'.
            jsx_parseClosingElementAt(startPos, startLoc) {
                let node = this.startNodeAt(startPos, startLoc);
                let nodeName = this.jsx_parseElementName();
                if (nodeName)
                    node.name = nodeName;
                this.expect(tok.jsxTagEnd);
                return this.finishNode(node, nodeName ? 'JSXClosingElement' : 'JSXClosingFragment');
            }
            // Parses entire JSX element, including it's opening tag
            // (starting after '<'), attributes, contents and closing tag.
            jsx_parseElementAt(startPos, startLoc) {
                let node = this.startNodeAt(startPos, startLoc);
                let children = [];
                let openingElement = this.jsx_parseOpeningElementAt(startPos, startLoc);
                let closingElement = null;
                if (!openingElement.selfClosing) {
                    contents: for (;;) {
                        switch (this.type) {
                            case tok.jsxTagStart:
                                startPos = this.start;
                                startLoc = this.startLoc;
                                this.next();
                                if (this.eat(tt.slash)) {
                                    closingElement = this.jsx_parseClosingElementAt(startPos, startLoc);
                                    break contents;
                                }
                                children.push(this.jsx_parseElementAt(startPos, startLoc));
                                break;
                            case tok.jsxText:
                                children.push(this.parseExprAtom());
                                break;
                            case tt.braceL:
                                children.push(this.jsx_parseExpressionContainer());
                                break;
                            default:
                                this.unexpected();
                        }
                    }
                    if (getQualifiedJSXName(closingElement.name) !== getQualifiedJSXName(openingElement.name)) {
                        this.raise(closingElement.start, 'Expected corresponding JSX closing tag for <' + getQualifiedJSXName(openingElement.name) + '>');
                    }
                }
                let fragmentOrElement = openingElement.name ? 'Element' : 'Fragment';
                node['opening' + fragmentOrElement] = openingElement;
                node['closing' + fragmentOrElement] = closingElement;
                node.children = children;
                if (this.type === tt.relational && this.value === "<") {
                    this.raise(this.start, "Adjacent JSX elements must be wrapped in an enclosing tag");
                }
                return this.finishNode(node, 'JSX' + fragmentOrElement);
            }
            // Parse JSX text
            jsx_parseText() {
                let node = this.parseLiteral(this.value);
                node.type = "JSXText";
                return node;
            }
            // Parses entire JSX element from current position.
            jsx_parseElement() {
                let startPos = this.start, startLoc = this.startLoc;
                this.next();
                return this.jsx_parseElementAt(startPos, startLoc);
            }
            parseExprAtom(refShortHandDefaultPos) {
                if (this.type === tok.jsxText)
                    return this.jsx_parseText();
                else if (this.type === tok.jsxTagStart)
                    return this.jsx_parseElement();
                else
                    return super.parseExprAtom(refShortHandDefaultPos);
            }
            readToken(code) {
                let context = this.curContext();
                if (context === tc_expr)
                    return this.jsx_readToken();
                if (context === tc_oTag || context === tc_cTag) {
                    if (isIdentifierStart(code))
                        return this.jsx_readWord();
                    if (code == 62) {
                        ++this.pos;
                        return this.finishToken(tok.jsxTagEnd);
                    }
                    if ((code === 34 || code === 39) && context == tc_oTag)
                        return this.jsx_readString(code);
                }
                if (code === 60 && this.exprAllowed && this.input.charCodeAt(this.pos + 1) !== 33) {
                    ++this.pos;
                    return this.finishToken(tok.jsxTagStart);
                }
                return super.readToken(code);
            }
            updateContext(prevType) {
                if (this.type == tt.braceL) {
                    var curContext = this.curContext();
                    if (curContext == tc_oTag)
                        this.context.push(tokContexts.b_expr);
                    else if (curContext == tc_expr)
                        this.context.push(tokContexts.b_tmpl);
                    else
                        super.updateContext(prevType);
                    this.exprAllowed = true;
                }
                else if (this.type === tt.slash && prevType === tok.jsxTagStart) {
                    this.context.length -= 2; // do not consider JSX expr -> JSX open tag -> ... anymore
                    this.context.push(tc_cTag); // reconsider as closing tag context
                    this.exprAllowed = false;
                }
                else {
                    return super.updateContext(prevType);
                }
            }
        };
    }
},
/* acorn-jsx/xhtml.js */ function _(require, module, exports) {
    module.exports = {
        quot: '\u0022',
        amp: '&',
        apos: '\u0027',
        lt: '<',
        gt: '>',
        nbsp: '\u00A0',
        iexcl: '\u00A1',
        cent: '\u00A2',
        pound: '\u00A3',
        curren: '\u00A4',
        yen: '\u00A5',
        brvbar: '\u00A6',
        sect: '\u00A7',
        uml: '\u00A8',
        copy: '\u00A9',
        ordf: '\u00AA',
        laquo: '\u00AB',
        not: '\u00AC',
        shy: '\u00AD',
        reg: '\u00AE',
        macr: '\u00AF',
        deg: '\u00B0',
        plusmn: '\u00B1',
        sup2: '\u00B2',
        sup3: '\u00B3',
        acute: '\u00B4',
        micro: '\u00B5',
        para: '\u00B6',
        middot: '\u00B7',
        cedil: '\u00B8',
        sup1: '\u00B9',
        ordm: '\u00BA',
        raquo: '\u00BB',
        frac14: '\u00BC',
        frac12: '\u00BD',
        frac34: '\u00BE',
        iquest: '\u00BF',
        Agrave: '\u00C0',
        Aacute: '\u00C1',
        Acirc: '\u00C2',
        Atilde: '\u00C3',
        Auml: '\u00C4',
        Aring: '\u00C5',
        AElig: '\u00C6',
        Ccedil: '\u00C7',
        Egrave: '\u00C8',
        Eacute: '\u00C9',
        Ecirc: '\u00CA',
        Euml: '\u00CB',
        Igrave: '\u00CC',
        Iacute: '\u00CD',
        Icirc: '\u00CE',
        Iuml: '\u00CF',
        ETH: '\u00D0',
        Ntilde: '\u00D1',
        Ograve: '\u00D2',
        Oacute: '\u00D3',
        Ocirc: '\u00D4',
        Otilde: '\u00D5',
        Ouml: '\u00D6',
        times: '\u00D7',
        Oslash: '\u00D8',
        Ugrave: '\u00D9',
        Uacute: '\u00DA',
        Ucirc: '\u00DB',
        Uuml: '\u00DC',
        Yacute: '\u00DD',
        THORN: '\u00DE',
        szlig: '\u00DF',
        agrave: '\u00E0',
        aacute: '\u00E1',
        acirc: '\u00E2',
        atilde: '\u00E3',
        auml: '\u00E4',
        aring: '\u00E5',
        aelig: '\u00E6',
        ccedil: '\u00E7',
        egrave: '\u00E8',
        eacute: '\u00E9',
        ecirc: '\u00EA',
        euml: '\u00EB',
        igrave: '\u00EC',
        iacute: '\u00ED',
        icirc: '\u00EE',
        iuml: '\u00EF',
        eth: '\u00F0',
        ntilde: '\u00F1',
        ograve: '\u00F2',
        oacute: '\u00F3',
        ocirc: '\u00F4',
        otilde: '\u00F5',
        ouml: '\u00F6',
        divide: '\u00F7',
        oslash: '\u00F8',
        ugrave: '\u00F9',
        uacute: '\u00FA',
        ucirc: '\u00FB',
        uuml: '\u00FC',
        yacute: '\u00FD',
        thorn: '\u00FE',
        yuml: '\u00FF',
        OElig: '\u0152',
        oelig: '\u0153',
        Scaron: '\u0160',
        scaron: '\u0161',
        Yuml: '\u0178',
        fnof: '\u0192',
        circ: '\u02C6',
        tilde: '\u02DC',
        Alpha: '\u0391',
        Beta: '\u0392',
        Gamma: '\u0393',
        Delta: '\u0394',
        Epsilon: '\u0395',
        Zeta: '\u0396',
        Eta: '\u0397',
        Theta: '\u0398',
        Iota: '\u0399',
        Kappa: '\u039A',
        Lambda: '\u039B',
        Mu: '\u039C',
        Nu: '\u039D',
        Xi: '\u039E',
        Omicron: '\u039F',
        Pi: '\u03A0',
        Rho: '\u03A1',
        Sigma: '\u03A3',
        Tau: '\u03A4',
        Upsilon: '\u03A5',
        Phi: '\u03A6',
        Chi: '\u03A7',
        Psi: '\u03A8',
        Omega: '\u03A9',
        alpha: '\u03B1',
        beta: '\u03B2',
        gamma: '\u03B3',
        delta: '\u03B4',
        epsilon: '\u03B5',
        zeta: '\u03B6',
        eta: '\u03B7',
        theta: '\u03B8',
        iota: '\u03B9',
        kappa: '\u03BA',
        lambda: '\u03BB',
        mu: '\u03BC',
        nu: '\u03BD',
        xi: '\u03BE',
        omicron: '\u03BF',
        pi: '\u03C0',
        rho: '\u03C1',
        sigmaf: '\u03C2',
        sigma: '\u03C3',
        tau: '\u03C4',
        upsilon: '\u03C5',
        phi: '\u03C6',
        chi: '\u03C7',
        psi: '\u03C8',
        omega: '\u03C9',
        thetasym: '\u03D1',
        upsih: '\u03D2',
        piv: '\u03D6',
        ensp: '\u2002',
        emsp: '\u2003',
        thinsp: '\u2009',
        zwnj: '\u200C',
        zwj: '\u200D',
        lrm: '\u200E',
        rlm: '\u200F',
        ndash: '\u2013',
        mdash: '\u2014',
        lsquo: '\u2018',
        rsquo: '\u2019',
        sbquo: '\u201A',
        ldquo: '\u201C',
        rdquo: '\u201D',
        bdquo: '\u201E',
        dagger: '\u2020',
        Dagger: '\u2021',
        bull: '\u2022',
        hellip: '\u2026',
        permil: '\u2030',
        prime: '\u2032',
        Prime: '\u2033',
        lsaquo: '\u2039',
        rsaquo: '\u203A',
        oline: '\u203E',
        frasl: '\u2044',
        euro: '\u20AC',
        image: '\u2111',
        weierp: '\u2118',
        real: '\u211C',
        trade: '\u2122',
        alefsym: '\u2135',
        larr: '\u2190',
        uarr: '\u2191',
        rarr: '\u2192',
        darr: '\u2193',
        harr: '\u2194',
        crarr: '\u21B5',
        lArr: '\u21D0',
        uArr: '\u21D1',
        rArr: '\u21D2',
        dArr: '\u21D3',
        hArr: '\u21D4',
        forall: '\u2200',
        part: '\u2202',
        exist: '\u2203',
        empty: '\u2205',
        nabla: '\u2207',
        isin: '\u2208',
        notin: '\u2209',
        ni: '\u220B',
        prod: '\u220F',
        sum: '\u2211',
        minus: '\u2212',
        lowast: '\u2217',
        radic: '\u221A',
        prop: '\u221D',
        infin: '\u221E',
        ang: '\u2220',
        and: '\u2227',
        or: '\u2228',
        cap: '\u2229',
        cup: '\u222A',
        'int': '\u222B',
        there4: '\u2234',
        sim: '\u223C',
        cong: '\u2245',
        asymp: '\u2248',
        ne: '\u2260',
        equiv: '\u2261',
        le: '\u2264',
        ge: '\u2265',
        sub: '\u2282',
        sup: '\u2283',
        nsub: '\u2284',
        sube: '\u2286',
        supe: '\u2287',
        oplus: '\u2295',
        otimes: '\u2297',
        perp: '\u22A5',
        sdot: '\u22C5',
        lceil: '\u2308',
        rceil: '\u2309',
        lfloor: '\u230A',
        rfloor: '\u230B',
        lang: '\u2329',
        rang: '\u232A',
        loz: '\u25CA',
        spades: '\u2660',
        clubs: '\u2663',
        hearts: '\u2665',
        diams: '\u2666'
    };
},
/* espree/lib/ast-node-types.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    // None!
    //------------------------------------------------------------------------------
    // Public
    //------------------------------------------------------------------------------
    module.exports = {
        AssignmentExpression: "AssignmentExpression",
        AssignmentPattern: "AssignmentPattern",
        ArrayExpression: "ArrayExpression",
        ArrayPattern: "ArrayPattern",
        ArrowFunctionExpression: "ArrowFunctionExpression",
        AwaitExpression: "AwaitExpression",
        BlockStatement: "BlockStatement",
        BinaryExpression: "BinaryExpression",
        BreakStatement: "BreakStatement",
        CallExpression: "CallExpression",
        CatchClause: "CatchClause",
        ClassBody: "ClassBody",
        ClassDeclaration: "ClassDeclaration",
        ClassExpression: "ClassExpression",
        ConditionalExpression: "ConditionalExpression",
        ContinueStatement: "ContinueStatement",
        DoWhileStatement: "DoWhileStatement",
        DebuggerStatement: "DebuggerStatement",
        EmptyStatement: "EmptyStatement",
        ExpressionStatement: "ExpressionStatement",
        ForStatement: "ForStatement",
        ForInStatement: "ForInStatement",
        ForOfStatement: "ForOfStatement",
        FunctionDeclaration: "FunctionDeclaration",
        FunctionExpression: "FunctionExpression",
        Identifier: "Identifier",
        IfStatement: "IfStatement",
        Literal: "Literal",
        LabeledStatement: "LabeledStatement",
        LogicalExpression: "LogicalExpression",
        MemberExpression: "MemberExpression",
        MetaProperty: "MetaProperty",
        MethodDefinition: "MethodDefinition",
        NewExpression: "NewExpression",
        ObjectExpression: "ObjectExpression",
        ObjectPattern: "ObjectPattern",
        Program: "Program",
        Property: "Property",
        RestElement: "RestElement",
        ReturnStatement: "ReturnStatement",
        SequenceExpression: "SequenceExpression",
        SpreadElement: "SpreadElement",
        Super: "Super",
        SwitchCase: "SwitchCase",
        SwitchStatement: "SwitchStatement",
        TaggedTemplateExpression: "TaggedTemplateExpression",
        TemplateElement: "TemplateElement",
        TemplateLiteral: "TemplateLiteral",
        ThisExpression: "ThisExpression",
        ThrowStatement: "ThrowStatement",
        TryStatement: "TryStatement",
        UnaryExpression: "UnaryExpression",
        UpdateExpression: "UpdateExpression",
        VariableDeclaration: "VariableDeclaration",
        VariableDeclarator: "VariableDeclarator",
        WhileStatement: "WhileStatement",
        WithStatement: "WithStatement",
        YieldExpression: "YieldExpression",
        JSXIdentifier: "JSXIdentifier",
        JSXNamespacedName: "JSXNamespacedName",
        JSXMemberExpression: "JSXMemberExpression",
        JSXEmptyExpression: "JSXEmptyExpression",
        JSXExpressionContainer: "JSXExpressionContainer",
        JSXElement: "JSXElement",
        JSXClosingElement: "JSXClosingElement",
        JSXOpeningElement: "JSXOpeningElement",
        JSXAttribute: "JSXAttribute",
        JSXSpreadAttribute: "JSXSpreadAttribute",
        JSXText: "JSXText",
        ExportDefaultDeclaration: "ExportDefaultDeclaration",
        ExportNamedDeclaration: "ExportNamedDeclaration",
        ExportAllDeclaration: "ExportAllDeclaration",
        ExportSpecifier: "ExportSpecifier",
        ImportDeclaration: "ImportDeclaration",
        ImportSpecifier: "ImportSpecifier",
        ImportDefaultSpecifier: "ImportDefaultSpecifier",
        ImportNamespaceSpecifier: "ImportNamespaceSpecifier"
    };
},
/* espree/lib/espree.js */ function _(require, module, exports) {
    /* eslint-disable no-param-reassign*/
    const TokenTranslator = require(108) /* ./token-translator */;
    const { normalizeOptions } = require(109) /* ./options */;
    const STATE = Symbol("espree's internal state");
    const ESPRIMA_FINISH_NODE = Symbol("espree's esprimaFinishNode");
    /**
     * Converts an Acorn comment to a Esprima comment.
     * @param {boolean} block True if it's a block comment, false if not.
     * @param {string} text The text of the comment.
     * @param {int} start The index at which the comment starts.
     * @param {int} end The index at which the comment ends.
     * @param {Location} startLoc The location at which the comment starts.
     * @param {Location} endLoc The location at which the comment ends.
     * @returns {Object} The comment object.
     * @private
     */
    function convertAcornCommentToEsprimaComment(block, text, start, end, startLoc, endLoc) {
        const comment = {
            type: block ? "Block" : "Line",
            value: text
        };
        if (typeof start === "number") {
            comment.start = start;
            comment.end = end;
            comment.range = [start, end];
        }
        if (typeof startLoc === "object") {
            comment.loc = {
                start: startLoc,
                end: endLoc
            };
        }
        return comment;
    }
    module.exports = () => Parser => {
        const tokTypes = Object.assign({}, Parser.acorn.tokTypes);
        if (Parser.acornJsx) {
            Object.assign(tokTypes, Parser.acornJsx.tokTypes);
        }
        return class Espree extends Parser {
            constructor(opts, code) {
                if (typeof opts !== "object" || opts === null) {
                    opts = {};
                }
                if (typeof code !== "string" && !(code instanceof String)) {
                    code = String(code);
                }
                const options = normalizeOptions(opts);
                const ecmaFeatures = options.ecmaFeatures || {};
                const tokenTranslator = options.tokens === true
                    ? new TokenTranslator(tokTypes, code)
                    : null;
                // Initialize acorn parser.
                super({
                    // TODO: use {...options} when spread is supported(Node.js >= 8.3.0).
                    ecmaVersion: options.ecmaVersion,
                    sourceType: options.sourceType,
                    ranges: options.ranges,
                    locations: options.locations,
                    // Truthy value is true for backward compatibility.
                    allowReturnOutsideFunction: Boolean(ecmaFeatures.globalReturn),
                    // Collect tokens
                    onToken: token => {
                        if (tokenTranslator) {
                            // Use `tokens`, `ecmaVersion`, and `jsxAttrValueToken` in the state.
                            tokenTranslator.onToken(token, this[STATE]);
                        }
                        if (token.type !== tokTypes.eof) {
                            this[STATE].lastToken = token;
                        }
                    },
                    // Collect comments
                    onComment: (block, text, start, end, startLoc, endLoc) => {
                        if (this[STATE].comments) {
                            const comment = convertAcornCommentToEsprimaComment(block, text, start, end, startLoc, endLoc);
                            this[STATE].comments.push(comment);
                        }
                    }
                }, code);
                // Initialize internal state.
                this[STATE] = {
                    tokens: tokenTranslator ? [] : null,
                    comments: options.comment === true ? [] : null,
                    impliedStrict: ecmaFeatures.impliedStrict === true && this.options.ecmaVersion >= 5,
                    ecmaVersion: this.options.ecmaVersion,
                    jsxAttrValueToken: false,
                    lastToken: null
                };
            }
            tokenize() {
                do {
                    this.next();
                } while (this.type !== tokTypes.eof);
                // Consume the final eof token
                this.next();
                const extra = this[STATE];
                const tokens = extra.tokens;
                if (extra.comments) {
                    tokens.comments = extra.comments;
                }
                return tokens;
            }
            finishNode(...args) {
                const result = super.finishNode(...args);
                return this[ESPRIMA_FINISH_NODE](result);
            }
            finishNodeAt(...args) {
                const result = super.finishNodeAt(...args);
                return this[ESPRIMA_FINISH_NODE](result);
            }
            parse() {
                const extra = this[STATE];
                const program = super.parse();
                program.sourceType = this.options.sourceType;
                if (extra.comments) {
                    program.comments = extra.comments;
                }
                if (extra.tokens) {
                    program.tokens = extra.tokens;
                }
                /*
                 * Adjust opening and closing position of program to match Esprima.
                 * Acorn always starts programs at range 0 whereas Esprima starts at the
                 * first AST node's start (the only real difference is when there's leading
                 * whitespace or leading comments). Acorn also counts trailing whitespace
                 * as part of the program whereas Esprima only counts up to the last token.
                 */
                if (program.range) {
                    program.range[0] = program.body.length ? program.body[0].range[0] : program.range[0];
                    program.range[1] = extra.lastToken ? extra.lastToken.range[1] : program.range[1];
                }
                if (program.loc) {
                    program.loc.start = program.body.length ? program.body[0].loc.start : program.loc.start;
                    program.loc.end = extra.lastToken ? extra.lastToken.loc.end : program.loc.end;
                }
                return program;
            }
            parseTopLevel(node) {
                if (this[STATE].impliedStrict) {
                    this.strict = true;
                }
                return super.parseTopLevel(node);
            }
            /**
             * Overwrites the default raise method to throw Esprima-style errors.
             * @param {int} pos The position of the error.
             * @param {string} message The error message.
             * @throws {SyntaxError} A syntax error.
             * @returns {void}
             */
            raise(pos, message) {
                const loc = Parser.acorn.getLineInfo(this.input, pos);
                const err = new SyntaxError(message);
                err.index = pos;
                err.lineNumber = loc.line;
                err.column = loc.column + 1; // acorn uses 0-based columns
                throw err;
            }
            /**
             * Overwrites the default raise method to throw Esprima-style errors.
             * @param {int} pos The position of the error.
             * @param {string} message The error message.
             * @throws {SyntaxError} A syntax error.
             * @returns {void}
             */
            raiseRecoverable(pos, message) {
                this.raise(pos, message);
            }
            /**
             * Overwrites the default unexpected method to throw Esprima-style errors.
             * @param {int} pos The position of the error.
             * @throws {SyntaxError} A syntax error.
             * @returns {void}
             */
            unexpected(pos) {
                let message = "Unexpected token";
                if (pos !== null && pos !== void 0) {
                    this.pos = pos;
                    if (this.options.locations) {
                        while (this.pos < this.lineStart) {
                            this.lineStart = this.input.lastIndexOf("\n", this.lineStart - 2) + 1;
                            --this.curLine;
                        }
                    }
                    this.nextToken();
                }
                if (this.end > this.start) {
                    message += ` ${this.input.slice(this.start, this.end)}`;
                }
                this.raise(this.start, message);
            }
            /*
            * Esprima-FB represents JSX strings as tokens called "JSXText", but Acorn-JSX
            * uses regular tt.string without any distinction between this and regular JS
            * strings. As such, we intercept an attempt to read a JSX string and set a flag
            * on extra so that when tokens are converted, the next token will be switched
            * to JSXText via onToken.
            */
            jsx_readString(quote) {
                const result = super.jsx_readString(quote);
                if (this.type === tokTypes.string) {
                    this[STATE].jsxAttrValueToken = true;
                }
                return result;
            }
            /**
             * Performs last-minute Esprima-specific compatibility checks and fixes.
             * @param {ASTNode} result The node to check.
             * @returns {ASTNode} The finished node.
             */
            [ESPRIMA_FINISH_NODE](result) {
                // Acorn doesn't count the opening and closing backticks as part of templates
                // so we have to adjust ranges/locations appropriately.
                if (result.type === "TemplateElement") {
                    // additional adjustment needed if ${ is the last token
                    const terminalDollarBraceL = this.input.slice(result.end, result.end + 2) === "${";
                    if (result.range) {
                        result.range[0]--;
                        result.range[1] += (terminalDollarBraceL ? 2 : 1);
                    }
                    if (result.loc) {
                        result.loc.start.column--;
                        result.loc.end.column += (terminalDollarBraceL ? 2 : 1);
                    }
                }
                if (result.type.indexOf("Function") > -1 && !result.generator) {
                    result.generator = false;
                }
                return result;
            }
        };
    };
},
/* espree/lib/token-translator.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    // none!
    //------------------------------------------------------------------------------
    // Private
    //------------------------------------------------------------------------------
    // Esprima Token Types
    const Token = {
        Boolean: "Boolean",
        EOF: "<end>",
        Identifier: "Identifier",
        Keyword: "Keyword",
        Null: "Null",
        Numeric: "Numeric",
        Punctuator: "Punctuator",
        String: "String",
        RegularExpression: "RegularExpression",
        Template: "Template",
        JSXIdentifier: "JSXIdentifier",
        JSXText: "JSXText"
    };
    /**
     * Converts part of a template into an Esprima token.
     * @param {AcornToken[]} tokens The Acorn tokens representing the template.
     * @param {string} code The source code.
     * @returns {EsprimaToken} The Esprima equivalent of the template token.
     * @private
     */
    function convertTemplatePart(tokens, code) {
        const firstToken = tokens[0], lastTemplateToken = tokens[tokens.length - 1];
        const token = {
            type: Token.Template,
            value: code.slice(firstToken.start, lastTemplateToken.end)
        };
        if (firstToken.loc) {
            token.loc = {
                start: firstToken.loc.start,
                end: lastTemplateToken.loc.end
            };
        }
        if (firstToken.range) {
            token.start = firstToken.range[0];
            token.end = lastTemplateToken.range[1];
            token.range = [token.start, token.end];
        }
        return token;
    }
    /**
     * Contains logic to translate Acorn tokens into Esprima tokens.
     * @param {Object} acornTokTypes The Acorn token types.
     * @param {string} code The source code Acorn is parsing. This is necessary
     *      to correct the "value" property of some tokens.
     * @constructor
     */
    function TokenTranslator(acornTokTypes, code) {
        // token types
        this._acornTokTypes = acornTokTypes;
        // token buffer for templates
        this._tokens = [];
        // track the last curly brace
        this._curlyBrace = null;
        // the source code
        this._code = code;
    }
    TokenTranslator.prototype = {
        constructor: TokenTranslator,
        /**
         * Translates a single Esprima token to a single Acorn token. This may be
         * inaccurate due to how templates are handled differently in Esprima and
         * Acorn, but should be accurate for all other tokens.
         * @param {AcornToken} token The Acorn token to translate.
         * @param {Object} extra Espree extra object.
         * @returns {EsprimaToken} The Esprima version of the token.
         */
        translate(token, extra) {
            const type = token.type, tt = this._acornTokTypes;
            if (type === tt.name) {
                token.type = Token.Identifier;
                // TODO: See if this is an Acorn bug
                if (token.value === "static") {
                    token.type = Token.Keyword;
                }
                if (extra.ecmaVersion > 5 && (token.value === "yield" || token.value === "let")) {
                    token.type = Token.Keyword;
                }
            }
            else if (type === tt.semi || type === tt.comma ||
                type === tt.parenL || type === tt.parenR ||
                type === tt.braceL || type === tt.braceR ||
                type === tt.dot || type === tt.bracketL ||
                type === tt.colon || type === tt.question ||
                type === tt.bracketR || type === tt.ellipsis ||
                type === tt.arrow || type === tt.jsxTagStart ||
                type === tt.incDec || type === tt.starstar ||
                type === tt.jsxTagEnd || type === tt.prefix ||
                (type.binop && !type.keyword) ||
                type.isAssign) {
                token.type = Token.Punctuator;
                token.value = this._code.slice(token.start, token.end);
            }
            else if (type === tt.jsxName) {
                token.type = Token.JSXIdentifier;
            }
            else if (type.label === "jsxText" || type === tt.jsxAttrValueToken) {
                token.type = Token.JSXText;
            }
            else if (type.keyword) {
                if (type.keyword === "true" || type.keyword === "false") {
                    token.type = Token.Boolean;
                }
                else if (type.keyword === "null") {
                    token.type = Token.Null;
                }
                else {
                    token.type = Token.Keyword;
                }
            }
            else if (type === tt.num) {
                token.type = Token.Numeric;
                token.value = this._code.slice(token.start, token.end);
            }
            else if (type === tt.string) {
                if (extra.jsxAttrValueToken) {
                    extra.jsxAttrValueToken = false;
                    token.type = Token.JSXText;
                }
                else {
                    token.type = Token.String;
                }
                token.value = this._code.slice(token.start, token.end);
            }
            else if (type === tt.regexp) {
                token.type = Token.RegularExpression;
                const value = token.value;
                token.regex = {
                    flags: value.flags,
                    pattern: value.pattern
                };
                token.value = `/${value.pattern}/${value.flags}`;
            }
            return token;
        },
        /**
         * Function to call during Acorn's onToken handler.
         * @param {AcornToken} token The Acorn token.
         * @param {Object} extra The Espree extra object.
         * @returns {void}
         */
        onToken(token, extra) {
            const that = this, tt = this._acornTokTypes, tokens = extra.tokens, templateTokens = this._tokens;
            /**
             * Flushes the buffered template tokens and resets the template
             * tracking.
             * @returns {void}
             * @private
             */
            function translateTemplateTokens() {
                tokens.push(convertTemplatePart(that._tokens, that._code));
                that._tokens = [];
            }
            if (token.type === tt.eof) {
                // might be one last curlyBrace
                if (this._curlyBrace) {
                    tokens.push(this.translate(this._curlyBrace, extra));
                }
                return;
            }
            if (token.type === tt.backQuote) {
                // if there's already a curly, it's not part of the template
                if (this._curlyBrace) {
                    tokens.push(this.translate(this._curlyBrace, extra));
                    this._curlyBrace = null;
                }
                templateTokens.push(token);
                // it's the end
                if (templateTokens.length > 1) {
                    translateTemplateTokens();
                }
                return;
            }
            if (token.type === tt.dollarBraceL) {
                templateTokens.push(token);
                translateTemplateTokens();
                return;
            }
            if (token.type === tt.braceR) {
                // if there's already a curly, it's not part of the template
                if (this._curlyBrace) {
                    tokens.push(this.translate(this._curlyBrace, extra));
                }
                // store new curly for later
                this._curlyBrace = token;
                return;
            }
            if (token.type === tt.template || token.type === tt.invalidTemplate) {
                if (this._curlyBrace) {
                    templateTokens.push(this._curlyBrace);
                    this._curlyBrace = null;
                }
                templateTokens.push(token);
                return;
            }
            if (this._curlyBrace) {
                tokens.push(this.translate(this._curlyBrace, extra));
                this._curlyBrace = null;
            }
            tokens.push(this.translate(token, extra));
        }
    };
    //------------------------------------------------------------------------------
    // Public
    //------------------------------------------------------------------------------
    module.exports = TokenTranslator;
},
/* espree/lib/options.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const DEFAULT_ECMA_VERSION = 5;
    const SUPPORTED_VERSIONS = [
        3,
        5,
        6,
        7,
        8,
        9,
        10,
        11
    ];
    /**
     * Normalize ECMAScript version from the initial config
     * @param {number} ecmaVersion ECMAScript version from the initial config
     * @throws {Error} throws an error if the ecmaVersion is invalid.
     * @returns {number} normalized ECMAScript version
     */
    function normalizeEcmaVersion(ecmaVersion = DEFAULT_ECMA_VERSION) {
        if (typeof ecmaVersion !== "number") {
            throw new Error(`ecmaVersion must be a number. Received value of type ${typeof ecmaVersion} instead.`);
        }
        let version = ecmaVersion;
        // Calculate ECMAScript edition number from official year version starting with
        // ES2015, which corresponds with ES6 (or a difference of 2009).
        if (version >= 2015) {
            version -= 2009;
        }
        if (!SUPPORTED_VERSIONS.includes(version)) {
            throw new Error("Invalid ecmaVersion.");
        }
        return version;
    }
    /**
     * Normalize sourceType from the initial config
     * @param {string} sourceType to normalize
     * @throws {Error} throw an error if sourceType is invalid
     * @returns {string} normalized sourceType
     */
    function normalizeSourceType(sourceType = "script") {
        if (sourceType === "script" || sourceType === "module") {
            return sourceType;
        }
        throw new Error("Invalid sourceType.");
    }
    /**
     * Normalize parserOptions
     * @param {Object} options the parser options to normalize
     * @throws {Error} throw an error if found invalid option.
     * @returns {Object} normalized options
     */
    function normalizeOptions(options) {
        const ecmaVersion = normalizeEcmaVersion(options.ecmaVersion);
        const sourceType = normalizeSourceType(options.sourceType);
        const ranges = options.range === true;
        const locations = options.loc === true;
        if (sourceType === "module" && ecmaVersion < 6) {
            throw new Error("sourceType 'module' is not supported when ecmaVersion < 2015. Consider adding `{ ecmaVersion: 2015 }` to the parser options.");
        }
        return Object.assign({}, options, { ecmaVersion, sourceType, ranges, locations });
    }
    /**
     * Get the latest ECMAScript version supported by Espree.
     * @returns {number} The latest ECMAScript version.
     */
    function getLatestEcmaVersion() {
        return SUPPORTED_VERSIONS[SUPPORTED_VERSIONS.length - 1];
    }
    /**
     * Get the list of ECMAScript versions supported by Espree.
     * @returns {number[]} An array containing the supported ECMAScript versions.
     */
    function getSupportedEcmaVersions() {
        return [...SUPPORTED_VERSIONS];
    }
    //------------------------------------------------------------------------------
    // Public
    //------------------------------------------------------------------------------
    module.exports = {
        normalizeOptions,
        getLatestEcmaVersion,
        getSupportedEcmaVersions
    };
},
/* espree/package.json */ function _(require, module, exports) {
    module.exports = {
        "_args": [
            [
                "espree@6.2.0",
                "/Users/bryan/work/bokeh/bokehjs"
            ]
        ],
        "_development": true,
        "_from": "espree@6.2.0",
        "_id": "espree@6.2.0",
        "_inBundle": false,
        "_integrity": "sha512-Xs8airJ7RQolnDIbLtRutmfvSsAe0xqMMAantCN/GMoqf81TFbeI1T7Jpd56qYu1uuh32dOG5W/X9uO+ghPXzA==",
        "_location": "/espree",
        "_phantomChildren": {},
        "_requested": {
            "type": "version",
            "registry": true,
            "raw": "espree@6.2.0",
            "name": "espree",
            "escapedName": "espree",
            "rawSpec": "6.2.0",
            "saveSpec": null,
            "fetchSpec": "6.2.0"
        },
        "_requiredBy": [
            "/eslint"
        ],
        "_resolved": "https://registry.npmjs.org/espree/-/espree-6.2.0.tgz",
        "_spec": "6.2.0",
        "_where": "/Users/bryan/work/bokeh/bokehjs",
        "author": {
            "name": "Nicholas C. Zakas",
            "email": "nicholas+npm@nczconsulting.com"
        },
        "bugs": {
            "url": "http://github.com/eslint/espree.git"
        },
        "dependencies": {
            "acorn": "^7.1.0",
            "acorn-jsx": "^5.2.0",
            "eslint-visitor-keys": "^1.1.0"
        },
        "description": "An Esprima-compatible JavaScript parser built on Acorn",
        "devDependencies": {
            "browserify": "^16.5.0",
            "chai": "^4.2.0",
            "eslint": "^6.0.1",
            "eslint-config-eslint": "^5.0.1",
            "eslint-plugin-node": "^9.1.0",
            "eslint-release": "^1.0.0",
            "esprima": "latest",
            "esprima-fb": "^8001.2001.0-dev-harmony-fb",
            "json-diff": "^0.5.4",
            "leche": "^2.3.0",
            "mocha": "^6.2.0",
            "nyc": "^14.1.1",
            "regenerate": "^1.4.0",
            "shelljs": "^0.3.0",
            "shelljs-nodecli": "^0.1.1",
            "unicode-6.3.0": "^0.7.5"
        },
        "engines": {
            "node": ">=6.0.0"
        },
        "files": [
            "lib",
            "espree.js"
        ],
        "homepage": "https://github.com/eslint/espree",
        "keywords": [
            "ast",
            "ecmascript",
            "javascript",
            "parser",
            "syntax",
            "acorn"
        ],
        "license": "BSD-2-Clause",
        "main": "espree.js",
        "name": "espree",
        "repository": {
            "type": "git",
            "url": "git+https://github.com/eslint/espree.git"
        },
        "scripts": {
            "browserify": "node Makefile.js browserify",
            "generate-alpharelease": "eslint-generate-prerelease alpha",
            "generate-betarelease": "eslint-generate-prerelease beta",
            "generate-rcrelease": "eslint-generate-prerelease rc",
            "generate-regex": "node tools/generate-identifier-regex.js",
            "generate-release": "eslint-generate-release",
            "lint": "node Makefile.js lint",
            "publish-release": "eslint-publish-release",
            "test": "npm run-script lint && node Makefile.js test"
        },
        "version": "6.2.0"
    };
},
/* lodash/lodash.js */ function _(require, module, exports) {
    /**
     * @license
     * Lodash <https://lodash.com/>
     * Copyright OpenJS Foundation and other contributors <https://openjsf.org/>
     * Released under MIT license <https://lodash.com/license>
     * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
     * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
     */
    ;
    (function () {
        /** Used as a safe reference for `undefined` in pre-ES5 environments. */
        var undefined;
        /** Used as the semantic version number. */
        var VERSION = '4.17.15';
        /** Used as the size to enable large array optimizations. */
        var LARGE_ARRAY_SIZE = 200;
        /** Error message constants. */
        var CORE_ERROR_TEXT = 'Unsupported core-js use. Try https://npms.io/search?q=ponyfill.', FUNC_ERROR_TEXT = 'Expected a function';
        /** Used to stand-in for `undefined` hash values. */
        var HASH_UNDEFINED = '__lodash_hash_undefined__';
        /** Used as the maximum memoize cache size. */
        var MAX_MEMOIZE_SIZE = 500;
        /** Used as the internal argument placeholder. */
        var PLACEHOLDER = '__lodash_placeholder__';
        /** Used to compose bitmasks for cloning. */
        var CLONE_DEEP_FLAG = 1, CLONE_FLAT_FLAG = 2, CLONE_SYMBOLS_FLAG = 4;
        /** Used to compose bitmasks for value comparisons. */
        var COMPARE_PARTIAL_FLAG = 1, COMPARE_UNORDERED_FLAG = 2;
        /** Used to compose bitmasks for function metadata. */
        var WRAP_BIND_FLAG = 1, WRAP_BIND_KEY_FLAG = 2, WRAP_CURRY_BOUND_FLAG = 4, WRAP_CURRY_FLAG = 8, WRAP_CURRY_RIGHT_FLAG = 16, WRAP_PARTIAL_FLAG = 32, WRAP_PARTIAL_RIGHT_FLAG = 64, WRAP_ARY_FLAG = 128, WRAP_REARG_FLAG = 256, WRAP_FLIP_FLAG = 512;
        /** Used as default options for `_.truncate`. */
        var DEFAULT_TRUNC_LENGTH = 30, DEFAULT_TRUNC_OMISSION = '...';
        /** Used to detect hot functions by number of calls within a span of milliseconds. */
        var HOT_COUNT = 800, HOT_SPAN = 16;
        /** Used to indicate the type of lazy iteratees. */
        var LAZY_FILTER_FLAG = 1, LAZY_MAP_FLAG = 2, LAZY_WHILE_FLAG = 3;
        /** Used as references for various `Number` constants. */
        var INFINITY = 1 / 0, MAX_SAFE_INTEGER = 9007199254740991, MAX_INTEGER = 1.7976931348623157e+308, NAN = 0 / 0;
        /** Used as references for the maximum length and index of an array. */
        var MAX_ARRAY_LENGTH = 4294967295, MAX_ARRAY_INDEX = MAX_ARRAY_LENGTH - 1, HALF_MAX_ARRAY_LENGTH = MAX_ARRAY_LENGTH >>> 1;
        /** Used to associate wrap methods with their bit flags. */
        var wrapFlags = [
            ['ary', WRAP_ARY_FLAG],
            ['bind', WRAP_BIND_FLAG],
            ['bindKey', WRAP_BIND_KEY_FLAG],
            ['curry', WRAP_CURRY_FLAG],
            ['curryRight', WRAP_CURRY_RIGHT_FLAG],
            ['flip', WRAP_FLIP_FLAG],
            ['partial', WRAP_PARTIAL_FLAG],
            ['partialRight', WRAP_PARTIAL_RIGHT_FLAG],
            ['rearg', WRAP_REARG_FLAG]
        ];
        /** `Object#toString` result references. */
        var argsTag = '[object Arguments]', arrayTag = '[object Array]', asyncTag = '[object AsyncFunction]', boolTag = '[object Boolean]', dateTag = '[object Date]', domExcTag = '[object DOMException]', errorTag = '[object Error]', funcTag = '[object Function]', genTag = '[object GeneratorFunction]', mapTag = '[object Map]', numberTag = '[object Number]', nullTag = '[object Null]', objectTag = '[object Object]', promiseTag = '[object Promise]', proxyTag = '[object Proxy]', regexpTag = '[object RegExp]', setTag = '[object Set]', stringTag = '[object String]', symbolTag = '[object Symbol]', undefinedTag = '[object Undefined]', weakMapTag = '[object WeakMap]', weakSetTag = '[object WeakSet]';
        var arrayBufferTag = '[object ArrayBuffer]', dataViewTag = '[object DataView]', float32Tag = '[object Float32Array]', float64Tag = '[object Float64Array]', int8Tag = '[object Int8Array]', int16Tag = '[object Int16Array]', int32Tag = '[object Int32Array]', uint8Tag = '[object Uint8Array]', uint8ClampedTag = '[object Uint8ClampedArray]', uint16Tag = '[object Uint16Array]', uint32Tag = '[object Uint32Array]';
        /** Used to match empty string literals in compiled template source. */
        var reEmptyStringLeading = /\b__p \+= '';/g, reEmptyStringMiddle = /\b(__p \+=) '' \+/g, reEmptyStringTrailing = /(__e\(.*?\)|\b__t\)) \+\n'';/g;
        /** Used to match HTML entities and HTML characters. */
        var reEscapedHtml = /&(?:amp|lt|gt|quot|#39);/g, reUnescapedHtml = /[&<>"']/g, reHasEscapedHtml = RegExp(reEscapedHtml.source), reHasUnescapedHtml = RegExp(reUnescapedHtml.source);
        /** Used to match template delimiters. */
        var reEscape = /<%-([\s\S]+?)%>/g, reEvaluate = /<%([\s\S]+?)%>/g, reInterpolate = /<%=([\s\S]+?)%>/g;
        /** Used to match property names within property paths. */
        var reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/, reIsPlainProp = /^\w*$/, rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g;
        /**
         * Used to match `RegExp`
         * [syntax characters](http://ecma-international.org/ecma-262/7.0/#sec-patterns).
         */
        var reRegExpChar = /[\\^$.*+?()[\]{}|]/g, reHasRegExpChar = RegExp(reRegExpChar.source);
        /** Used to match leading and trailing whitespace. */
        var reTrim = /^\s+|\s+$/g, reTrimStart = /^\s+/, reTrimEnd = /\s+$/;
        /** Used to match wrap detail comments. */
        var reWrapComment = /\{(?:\n\/\* \[wrapped with .+\] \*\/)?\n?/, reWrapDetails = /\{\n\/\* \[wrapped with (.+)\] \*/, reSplitDetails = /,? & /;
        /** Used to match words composed of alphanumeric characters. */
        var reAsciiWord = /[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g;
        /** Used to match backslashes in property paths. */
        var reEscapeChar = /\\(\\)?/g;
        /**
         * Used to match
         * [ES template delimiters](http://ecma-international.org/ecma-262/7.0/#sec-template-literal-lexical-components).
         */
        var reEsTemplate = /\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g;
        /** Used to match `RegExp` flags from their coerced string values. */
        var reFlags = /\w*$/;
        /** Used to detect bad signed hexadecimal string values. */
        var reIsBadHex = /^[-+]0x[0-9a-f]+$/i;
        /** Used to detect binary string values. */
        var reIsBinary = /^0b[01]+$/i;
        /** Used to detect host constructors (Safari). */
        var reIsHostCtor = /^\[object .+?Constructor\]$/;
        /** Used to detect octal string values. */
        var reIsOctal = /^0o[0-7]+$/i;
        /** Used to detect unsigned integer values. */
        var reIsUint = /^(?:0|[1-9]\d*)$/;
        /** Used to match Latin Unicode letters (excluding mathematical operators). */
        var reLatin = /[\xc0-\xd6\xd8-\xf6\xf8-\xff\u0100-\u017f]/g;
        /** Used to ensure capturing order of template delimiters. */
        var reNoMatch = /($^)/;
        /** Used to match unescaped characters in compiled string literals. */
        var reUnescapedString = /['\n\r\u2028\u2029\\]/g;
        /** Used to compose unicode character classes. */
        var rsAstralRange = '\\ud800-\\udfff', rsComboMarksRange = '\\u0300-\\u036f', reComboHalfMarksRange = '\\ufe20-\\ufe2f', rsComboSymbolsRange = '\\u20d0-\\u20ff', rsComboRange = rsComboMarksRange + reComboHalfMarksRange + rsComboSymbolsRange, rsDingbatRange = '\\u2700-\\u27bf', rsLowerRange = 'a-z\\xdf-\\xf6\\xf8-\\xff', rsMathOpRange = '\\xac\\xb1\\xd7\\xf7', rsNonCharRange = '\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\xbf', rsPunctuationRange = '\\u2000-\\u206f', rsSpaceRange = ' \\t\\x0b\\f\\xa0\\ufeff\\n\\r\\u2028\\u2029\\u1680\\u180e\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200a\\u202f\\u205f\\u3000', rsUpperRange = 'A-Z\\xc0-\\xd6\\xd8-\\xde', rsVarRange = '\\ufe0e\\ufe0f', rsBreakRange = rsMathOpRange + rsNonCharRange + rsPunctuationRange + rsSpaceRange;
        /** Used to compose unicode capture groups. */
        var rsApos = "['\u2019]", rsAstral = '[' + rsAstralRange + ']', rsBreak = '[' + rsBreakRange + ']', rsCombo = '[' + rsComboRange + ']', rsDigits = '\\d+', rsDingbat = '[' + rsDingbatRange + ']', rsLower = '[' + rsLowerRange + ']', rsMisc = '[^' + rsAstralRange + rsBreakRange + rsDigits + rsDingbatRange + rsLowerRange + rsUpperRange + ']', rsFitz = '\\ud83c[\\udffb-\\udfff]', rsModifier = '(?:' + rsCombo + '|' + rsFitz + ')', rsNonAstral = '[^' + rsAstralRange + ']', rsRegional = '(?:\\ud83c[\\udde6-\\uddff]){2}', rsSurrPair = '[\\ud800-\\udbff][\\udc00-\\udfff]', rsUpper = '[' + rsUpperRange + ']', rsZWJ = '\\u200d';
        /** Used to compose unicode regexes. */
        var rsMiscLower = '(?:' + rsLower + '|' + rsMisc + ')', rsMiscUpper = '(?:' + rsUpper + '|' + rsMisc + ')', rsOptContrLower = '(?:' + rsApos + '(?:d|ll|m|re|s|t|ve))?', rsOptContrUpper = '(?:' + rsApos + '(?:D|LL|M|RE|S|T|VE))?', reOptMod = rsModifier + '?', rsOptVar = '[' + rsVarRange + ']?', rsOptJoin = '(?:' + rsZWJ + '(?:' + [rsNonAstral, rsRegional, rsSurrPair].join('|') + ')' + rsOptVar + reOptMod + ')*', rsOrdLower = '\\d*(?:1st|2nd|3rd|(?![123])\\dth)(?=\\b|[A-Z_])', rsOrdUpper = '\\d*(?:1ST|2ND|3RD|(?![123])\\dTH)(?=\\b|[a-z_])', rsSeq = rsOptVar + reOptMod + rsOptJoin, rsEmoji = '(?:' + [rsDingbat, rsRegional, rsSurrPair].join('|') + ')' + rsSeq, rsSymbol = '(?:' + [rsNonAstral + rsCombo + '?', rsCombo, rsRegional, rsSurrPair, rsAstral].join('|') + ')';
        /** Used to match apostrophes. */
        var reApos = RegExp(rsApos, 'g');
        /**
         * Used to match [combining diacritical marks](https://en.wikipedia.org/wiki/Combining_Diacritical_Marks) and
         * [combining diacritical marks for symbols](https://en.wikipedia.org/wiki/Combining_Diacritical_Marks_for_Symbols).
         */
        var reComboMark = RegExp(rsCombo, 'g');
        /** Used to match [string symbols](https://mathiasbynens.be/notes/javascript-unicode). */
        var reUnicode = RegExp(rsFitz + '(?=' + rsFitz + ')|' + rsSymbol + rsSeq, 'g');
        /** Used to match complex or compound words. */
        var reUnicodeWord = RegExp([
            rsUpper + '?' + rsLower + '+' + rsOptContrLower + '(?=' + [rsBreak, rsUpper, '$'].join('|') + ')',
            rsMiscUpper + '+' + rsOptContrUpper + '(?=' + [rsBreak, rsUpper + rsMiscLower, '$'].join('|') + ')',
            rsUpper + '?' + rsMiscLower + '+' + rsOptContrLower,
            rsUpper + '+' + rsOptContrUpper,
            rsOrdUpper,
            rsOrdLower,
            rsDigits,
            rsEmoji
        ].join('|'), 'g');
        /** Used to detect strings with [zero-width joiners or code points from the astral planes](http://eev.ee/blog/2015/09/12/dark-corners-of-unicode/). */
        var reHasUnicode = RegExp('[' + rsZWJ + rsAstralRange + rsComboRange + rsVarRange + ']');
        /** Used to detect strings that need a more robust regexp to match words. */
        var reHasUnicodeWord = /[a-z][A-Z]|[A-Z]{2}[a-z]|[0-9][a-zA-Z]|[a-zA-Z][0-9]|[^a-zA-Z0-9 ]/;
        /** Used to assign default `context` object properties. */
        var contextProps = [
            'Array', 'Buffer', 'DataView', 'Date', 'Error', 'Float32Array', 'Float64Array',
            'Function', 'Int8Array', 'Int16Array', 'Int32Array', 'Map', 'Math', 'Object',
            'Promise', 'RegExp', 'Set', 'String', 'Symbol', 'TypeError', 'Uint8Array',
            'Uint8ClampedArray', 'Uint16Array', 'Uint32Array', 'WeakMap',
            '_', 'clearTimeout', 'isFinite', 'parseInt', 'setTimeout'
        ];
        /** Used to make template sourceURLs easier to identify. */
        var templateCounter = -1;
        /** Used to identify `toStringTag` values of typed arrays. */
        var typedArrayTags = {};
        typedArrayTags[float32Tag] = typedArrayTags[float64Tag] =
            typedArrayTags[int8Tag] = typedArrayTags[int16Tag] =
                typedArrayTags[int32Tag] = typedArrayTags[uint8Tag] =
                    typedArrayTags[uint8ClampedTag] = typedArrayTags[uint16Tag] =
                        typedArrayTags[uint32Tag] = true;
        typedArrayTags[argsTag] = typedArrayTags[arrayTag] =
            typedArrayTags[arrayBufferTag] = typedArrayTags[boolTag] =
                typedArrayTags[dataViewTag] = typedArrayTags[dateTag] =
                    typedArrayTags[errorTag] = typedArrayTags[funcTag] =
                        typedArrayTags[mapTag] = typedArrayTags[numberTag] =
                            typedArrayTags[objectTag] = typedArrayTags[regexpTag] =
                                typedArrayTags[setTag] = typedArrayTags[stringTag] =
                                    typedArrayTags[weakMapTag] = false;
        /** Used to identify `toStringTag` values supported by `_.clone`. */
        var cloneableTags = {};
        cloneableTags[argsTag] = cloneableTags[arrayTag] =
            cloneableTags[arrayBufferTag] = cloneableTags[dataViewTag] =
                cloneableTags[boolTag] = cloneableTags[dateTag] =
                    cloneableTags[float32Tag] = cloneableTags[float64Tag] =
                        cloneableTags[int8Tag] = cloneableTags[int16Tag] =
                            cloneableTags[int32Tag] = cloneableTags[mapTag] =
                                cloneableTags[numberTag] = cloneableTags[objectTag] =
                                    cloneableTags[regexpTag] = cloneableTags[setTag] =
                                        cloneableTags[stringTag] = cloneableTags[symbolTag] =
                                            cloneableTags[uint8Tag] = cloneableTags[uint8ClampedTag] =
                                                cloneableTags[uint16Tag] = cloneableTags[uint32Tag] = true;
        cloneableTags[errorTag] = cloneableTags[funcTag] =
            cloneableTags[weakMapTag] = false;
        /** Used to map Latin Unicode letters to basic Latin letters. */
        var deburredLetters = {
            // Latin-1 Supplement block.
            '\xc0': 'A', '\xc1': 'A', '\xc2': 'A', '\xc3': 'A', '\xc4': 'A', '\xc5': 'A',
            '\xe0': 'a', '\xe1': 'a', '\xe2': 'a', '\xe3': 'a', '\xe4': 'a', '\xe5': 'a',
            '\xc7': 'C', '\xe7': 'c',
            '\xd0': 'D', '\xf0': 'd',
            '\xc8': 'E', '\xc9': 'E', '\xca': 'E', '\xcb': 'E',
            '\xe8': 'e', '\xe9': 'e', '\xea': 'e', '\xeb': 'e',
            '\xcc': 'I', '\xcd': 'I', '\xce': 'I', '\xcf': 'I',
            '\xec': 'i', '\xed': 'i', '\xee': 'i', '\xef': 'i',
            '\xd1': 'N', '\xf1': 'n',
            '\xd2': 'O', '\xd3': 'O', '\xd4': 'O', '\xd5': 'O', '\xd6': 'O', '\xd8': 'O',
            '\xf2': 'o', '\xf3': 'o', '\xf4': 'o', '\xf5': 'o', '\xf6': 'o', '\xf8': 'o',
            '\xd9': 'U', '\xda': 'U', '\xdb': 'U', '\xdc': 'U',
            '\xf9': 'u', '\xfa': 'u', '\xfb': 'u', '\xfc': 'u',
            '\xdd': 'Y', '\xfd': 'y', '\xff': 'y',
            '\xc6': 'Ae', '\xe6': 'ae',
            '\xde': 'Th', '\xfe': 'th',
            '\xdf': 'ss',
            // Latin Extended-A block.
            '\u0100': 'A', '\u0102': 'A', '\u0104': 'A',
            '\u0101': 'a', '\u0103': 'a', '\u0105': 'a',
            '\u0106': 'C', '\u0108': 'C', '\u010a': 'C', '\u010c': 'C',
            '\u0107': 'c', '\u0109': 'c', '\u010b': 'c', '\u010d': 'c',
            '\u010e': 'D', '\u0110': 'D', '\u010f': 'd', '\u0111': 'd',
            '\u0112': 'E', '\u0114': 'E', '\u0116': 'E', '\u0118': 'E', '\u011a': 'E',
            '\u0113': 'e', '\u0115': 'e', '\u0117': 'e', '\u0119': 'e', '\u011b': 'e',
            '\u011c': 'G', '\u011e': 'G', '\u0120': 'G', '\u0122': 'G',
            '\u011d': 'g', '\u011f': 'g', '\u0121': 'g', '\u0123': 'g',
            '\u0124': 'H', '\u0126': 'H', '\u0125': 'h', '\u0127': 'h',
            '\u0128': 'I', '\u012a': 'I', '\u012c': 'I', '\u012e': 'I', '\u0130': 'I',
            '\u0129': 'i', '\u012b': 'i', '\u012d': 'i', '\u012f': 'i', '\u0131': 'i',
            '\u0134': 'J', '\u0135': 'j',
            '\u0136': 'K', '\u0137': 'k', '\u0138': 'k',
            '\u0139': 'L', '\u013b': 'L', '\u013d': 'L', '\u013f': 'L', '\u0141': 'L',
            '\u013a': 'l', '\u013c': 'l', '\u013e': 'l', '\u0140': 'l', '\u0142': 'l',
            '\u0143': 'N', '\u0145': 'N', '\u0147': 'N', '\u014a': 'N',
            '\u0144': 'n', '\u0146': 'n', '\u0148': 'n', '\u014b': 'n',
            '\u014c': 'O', '\u014e': 'O', '\u0150': 'O',
            '\u014d': 'o', '\u014f': 'o', '\u0151': 'o',
            '\u0154': 'R', '\u0156': 'R', '\u0158': 'R',
            '\u0155': 'r', '\u0157': 'r', '\u0159': 'r',
            '\u015a': 'S', '\u015c': 'S', '\u015e': 'S', '\u0160': 'S',
            '\u015b': 's', '\u015d': 's', '\u015f': 's', '\u0161': 's',
            '\u0162': 'T', '\u0164': 'T', '\u0166': 'T',
            '\u0163': 't', '\u0165': 't', '\u0167': 't',
            '\u0168': 'U', '\u016a': 'U', '\u016c': 'U', '\u016e': 'U', '\u0170': 'U', '\u0172': 'U',
            '\u0169': 'u', '\u016b': 'u', '\u016d': 'u', '\u016f': 'u', '\u0171': 'u', '\u0173': 'u',
            '\u0174': 'W', '\u0175': 'w',
            '\u0176': 'Y', '\u0177': 'y', '\u0178': 'Y',
            '\u0179': 'Z', '\u017b': 'Z', '\u017d': 'Z',
            '\u017a': 'z', '\u017c': 'z', '\u017e': 'z',
            '\u0132': 'IJ', '\u0133': 'ij',
            '\u0152': 'Oe', '\u0153': 'oe',
            '\u0149': "'n", '\u017f': 's'
        };
        /** Used to map characters to HTML entities. */
        var htmlEscapes = {
            '&': '&amp;',
            '<': '&lt;',
            '>': '&gt;',
            '"': '&quot;',
            "'": '&#39;'
        };
        /** Used to map HTML entities to characters. */
        var htmlUnescapes = {
            '&amp;': '&',
            '&lt;': '<',
            '&gt;': '>',
            '&quot;': '"',
            '&#39;': "'"
        };
        /** Used to escape characters for inclusion in compiled string literals. */
        var stringEscapes = {
            '\\': '\\',
            "'": "'",
            '\n': 'n',
            '\r': 'r',
            '\u2028': 'u2028',
            '\u2029': 'u2029'
        };
        /** Built-in method references without a dependency on `root`. */
        var freeParseFloat = parseFloat, freeParseInt = parseInt;
        /** Detect free variable `global` from Node.js. */
        var freeGlobal = typeof global == 'object' && global && global.Object === Object && global;
        /** Detect free variable `self`. */
        var freeSelf = typeof self == 'object' && self && self.Object === Object && self;
        /** Used as a reference to the global object. */
        var root = freeGlobal || freeSelf || Function('return this')();
        /** Detect free variable `exports`. */
        var freeExports = typeof exports == 'object' && exports && !exports.nodeType && exports;
        /** Detect free variable `module`. */
        var freeModule = freeExports && typeof module == 'object' && module && !module.nodeType && module;
        /** Detect the popular CommonJS extension `module.exports`. */
        var moduleExports = freeModule && freeModule.exports === freeExports;
        /** Detect free variable `process` from Node.js. */
        var freeProcess = moduleExports && freeGlobal.process;
        /** Used to access faster Node.js helpers. */
        var nodeUtil = (function () {
            try {
                // Use `util.types` for Node.js 10+.
                var types = freeModule && freeModule.require && freeModule.require('util').types;
                if (types) {
                    return types;
                }
                // Legacy `process.binding('util')` for Node.js < 10.
                return freeProcess && freeProcess.binding && freeProcess.binding('util');
            }
            catch (e) { }
        }());
        /* Node.js helper references. */
        var nodeIsArrayBuffer = nodeUtil && nodeUtil.isArrayBuffer, nodeIsDate = nodeUtil && nodeUtil.isDate, nodeIsMap = nodeUtil && nodeUtil.isMap, nodeIsRegExp = nodeUtil && nodeUtil.isRegExp, nodeIsSet = nodeUtil && nodeUtil.isSet, nodeIsTypedArray = nodeUtil && nodeUtil.isTypedArray;
        /*--------------------------------------------------------------------------*/
        /**
         * A faster alternative to `Function#apply`, this function invokes `func`
         * with the `this` binding of `thisArg` and the arguments of `args`.
         *
         * @private
         * @param {Function} func The function to invoke.
         * @param {*} thisArg The `this` binding of `func`.
         * @param {Array} args The arguments to invoke `func` with.
         * @returns {*} Returns the result of `func`.
         */
        function apply(func, thisArg, args) {
            switch (args.length) {
                case 0: return func.call(thisArg);
                case 1: return func.call(thisArg, args[0]);
                case 2: return func.call(thisArg, args[0], args[1]);
                case 3: return func.call(thisArg, args[0], args[1], args[2]);
            }
            return func.apply(thisArg, args);
        }
        /**
         * A specialized version of `baseAggregator` for arrays.
         *
         * @private
         * @param {Array} [array] The array to iterate over.
         * @param {Function} setter The function to set `accumulator` values.
         * @param {Function} iteratee The iteratee to transform keys.
         * @param {Object} accumulator The initial aggregated object.
         * @returns {Function} Returns `accumulator`.
         */
        function arrayAggregator(array, setter, iteratee, accumulator) {
            var index = -1, length = array == null ? 0 : array.length;
            while (++index < length) {
                var value = array[index];
                setter(accumulator, value, iteratee(value), array);
            }
            return accumulator;
        }
        /**
         * A specialized version of `_.forEach` for arrays without support for
         * iteratee shorthands.
         *
         * @private
         * @param {Array} [array] The array to iterate over.
         * @param {Function} iteratee The function invoked per iteration.
         * @returns {Array} Returns `array`.
         */
        function arrayEach(array, iteratee) {
            var index = -1, length = array == null ? 0 : array.length;
            while (++index < length) {
                if (iteratee(array[index], index, array) === false) {
                    break;
                }
            }
            return array;
        }
        /**
         * A specialized version of `_.forEachRight` for arrays without support for
         * iteratee shorthands.
         *
         * @private
         * @param {Array} [array] The array to iterate over.
         * @param {Function} iteratee The function invoked per iteration.
         * @returns {Array} Returns `array`.
         */
        function arrayEachRight(array, iteratee) {
            var length = array == null ? 0 : array.length;
            while (length--) {
                if (iteratee(array[length], length, array) === false) {
                    break;
                }
            }
            return array;
        }
        /**
         * A specialized version of `_.every` for arrays without support for
         * iteratee shorthands.
         *
         * @private
         * @param {Array} [array] The array to iterate over.
         * @param {Function} predicate The function invoked per iteration.
         * @returns {boolean} Returns `true` if all elements pass the predicate check,
         *  else `false`.
         */
        function arrayEvery(array, predicate) {
            var index = -1, length = array == null ? 0 : array.length;
            while (++index < length) {
                if (!predicate(array[index], index, array)) {
                    return false;
                }
            }
            return true;
        }
        /**
         * A specialized version of `_.filter` for arrays without support for
         * iteratee shorthands.
         *
         * @private
         * @param {Array} [array] The array to iterate over.
         * @param {Function} predicate The function invoked per iteration.
         * @returns {Array} Returns the new filtered array.
         */
        function arrayFilter(array, predicate) {
            var index = -1, length = array == null ? 0 : array.length, resIndex = 0, result = [];
            while (++index < length) {
                var value = array[index];
                if (predicate(value, index, array)) {
                    result[resIndex++] = value;
                }
            }
            return result;
        }
        /**
         * A specialized version of `_.includes` for arrays without support for
         * specifying an index to search from.
         *
         * @private
         * @param {Array} [array] The array to inspect.
         * @param {*} target The value to search for.
         * @returns {boolean} Returns `true` if `target` is found, else `false`.
         */
        function arrayIncludes(array, value) {
            var length = array == null ? 0 : array.length;
            return !!length && baseIndexOf(array, value, 0) > -1;
        }
        /**
         * This function is like `arrayIncludes` except that it accepts a comparator.
         *
         * @private
         * @param {Array} [array] The array to inspect.
         * @param {*} target The value to search for.
         * @param {Function} comparator The comparator invoked per element.
         * @returns {boolean} Returns `true` if `target` is found, else `false`.
         */
        function arrayIncludesWith(array, value, comparator) {
            var index = -1, length = array == null ? 0 : array.length;
            while (++index < length) {
                if (comparator(value, array[index])) {
                    return true;
                }
            }
            return false;
        }
        /**
         * A specialized version of `_.map` for arrays without support for iteratee
         * shorthands.
         *
         * @private
         * @param {Array} [array] The array to iterate over.
         * @param {Function} iteratee The function invoked per iteration.
         * @returns {Array} Returns the new mapped array.
         */
        function arrayMap(array, iteratee) {
            var index = -1, length = array == null ? 0 : array.length, result = Array(length);
            while (++index < length) {
                result[index] = iteratee(array[index], index, array);
            }
            return result;
        }
        /**
         * Appends the elements of `values` to `array`.
         *
         * @private
         * @param {Array} array The array to modify.
         * @param {Array} values The values to append.
         * @returns {Array} Returns `array`.
         */
        function arrayPush(array, values) {
            var index = -1, length = values.length, offset = array.length;
            while (++index < length) {
                array[offset + index] = values[index];
            }
            return array;
        }
        /**
         * A specialized version of `_.reduce` for arrays without support for
         * iteratee shorthands.
         *
         * @private
         * @param {Array} [array] The array to iterate over.
         * @param {Function} iteratee The function invoked per iteration.
         * @param {*} [accumulator] The initial value.
         * @param {boolean} [initAccum] Specify using the first element of `array` as
         *  the initial value.
         * @returns {*} Returns the accumulated value.
         */
        function arrayReduce(array, iteratee, accumulator, initAccum) {
            var index = -1, length = array == null ? 0 : array.length;
            if (initAccum && length) {
                accumulator = array[++index];
            }
            while (++index < length) {
                accumulator = iteratee(accumulator, array[index], index, array);
            }
            return accumulator;
        }
        /**
         * A specialized version of `_.reduceRight` for arrays without support for
         * iteratee shorthands.
         *
         * @private
         * @param {Array} [array] The array to iterate over.
         * @param {Function} iteratee The function invoked per iteration.
         * @param {*} [accumulator] The initial value.
         * @param {boolean} [initAccum] Specify using the last element of `array` as
         *  the initial value.
         * @returns {*} Returns the accumulated value.
         */
        function arrayReduceRight(array, iteratee, accumulator, initAccum) {
            var length = array == null ? 0 : array.length;
            if (initAccum && length) {
                accumulator = array[--length];
            }
            while (length--) {
                accumulator = iteratee(accumulator, array[length], length, array);
            }
            return accumulator;
        }
        /**
         * A specialized version of `_.some` for arrays without support for iteratee
         * shorthands.
         *
         * @private
         * @param {Array} [array] The array to iterate over.
         * @param {Function} predicate The function invoked per iteration.
         * @returns {boolean} Returns `true` if any element passes the predicate check,
         *  else `false`.
         */
        function arraySome(array, predicate) {
            var index = -1, length = array == null ? 0 : array.length;
            while (++index < length) {
                if (predicate(array[index], index, array)) {
                    return true;
                }
            }
            return false;
        }
        /**
         * Gets the size of an ASCII `string`.
         *
         * @private
         * @param {string} string The string inspect.
         * @returns {number} Returns the string size.
         */
        var asciiSize = baseProperty('length');
        /**
         * Converts an ASCII `string` to an array.
         *
         * @private
         * @param {string} string The string to convert.
         * @returns {Array} Returns the converted array.
         */
        function asciiToArray(string) {
            return string.split('');
        }
        /**
         * Splits an ASCII `string` into an array of its words.
         *
         * @private
         * @param {string} The string to inspect.
         * @returns {Array} Returns the words of `string`.
         */
        function asciiWords(string) {
            return string.match(reAsciiWord) || [];
        }
        /**
         * The base implementation of methods like `_.findKey` and `_.findLastKey`,
         * without support for iteratee shorthands, which iterates over `collection`
         * using `eachFunc`.
         *
         * @private
         * @param {Array|Object} collection The collection to inspect.
         * @param {Function} predicate The function invoked per iteration.
         * @param {Function} eachFunc The function to iterate over `collection`.
         * @returns {*} Returns the found element or its key, else `undefined`.
         */
        function baseFindKey(collection, predicate, eachFunc) {
            var result;
            eachFunc(collection, function (value, key, collection) {
                if (predicate(value, key, collection)) {
                    result = key;
                    return false;
                }
            });
            return result;
        }
        /**
         * The base implementation of `_.findIndex` and `_.findLastIndex` without
         * support for iteratee shorthands.
         *
         * @private
         * @param {Array} array The array to inspect.
         * @param {Function} predicate The function invoked per iteration.
         * @param {number} fromIndex The index to search from.
         * @param {boolean} [fromRight] Specify iterating from right to left.
         * @returns {number} Returns the index of the matched value, else `-1`.
         */
        function baseFindIndex(array, predicate, fromIndex, fromRight) {
            var length = array.length, index = fromIndex + (fromRight ? 1 : -1);
            while ((fromRight ? index-- : ++index < length)) {
                if (predicate(array[index], index, array)) {
                    return index;
                }
            }
            return -1;
        }
        /**
         * The base implementation of `_.indexOf` without `fromIndex` bounds checks.
         *
         * @private
         * @param {Array} array The array to inspect.
         * @param {*} value The value to search for.
         * @param {number} fromIndex The index to search from.
         * @returns {number} Returns the index of the matched value, else `-1`.
         */
        function baseIndexOf(array, value, fromIndex) {
            return value === value
                ? strictIndexOf(array, value, fromIndex)
                : baseFindIndex(array, baseIsNaN, fromIndex);
        }
        /**
         * This function is like `baseIndexOf` except that it accepts a comparator.
         *
         * @private
         * @param {Array} array The array to inspect.
         * @param {*} value The value to search for.
         * @param {number} fromIndex The index to search from.
         * @param {Function} comparator The comparator invoked per element.
         * @returns {number} Returns the index of the matched value, else `-1`.
         */
        function baseIndexOfWith(array, value, fromIndex, comparator) {
            var index = fromIndex - 1, length = array.length;
            while (++index < length) {
                if (comparator(array[index], value)) {
                    return index;
                }
            }
            return -1;
        }
        /**
         * The base implementation of `_.isNaN` without support for number objects.
         *
         * @private
         * @param {*} value The value to check.
         * @returns {boolean} Returns `true` if `value` is `NaN`, else `false`.
         */
        function baseIsNaN(value) {
            return value !== value;
        }
        /**
         * The base implementation of `_.mean` and `_.meanBy` without support for
         * iteratee shorthands.
         *
         * @private
         * @param {Array} array The array to iterate over.
         * @param {Function} iteratee The function invoked per iteration.
         * @returns {number} Returns the mean.
         */
        function baseMean(array, iteratee) {
            var length = array == null ? 0 : array.length;
            return length ? (baseSum(array, iteratee) / length) : NAN;
        }
        /**
         * The base implementation of `_.property` without support for deep paths.
         *
         * @private
         * @param {string} key The key of the property to get.
         * @returns {Function} Returns the new accessor function.
         */
        function baseProperty(key) {
            return function (object) {
                return object == null ? undefined : object[key];
            };
        }
        /**
         * The base implementation of `_.propertyOf` without support for deep paths.
         *
         * @private
         * @param {Object} object The object to query.
         * @returns {Function} Returns the new accessor function.
         */
        function basePropertyOf(object) {
            return function (key) {
                return object == null ? undefined : object[key];
            };
        }
        /**
         * The base implementation of `_.reduce` and `_.reduceRight`, without support
         * for iteratee shorthands, which iterates over `collection` using `eachFunc`.
         *
         * @private
         * @param {Array|Object} collection The collection to iterate over.
         * @param {Function} iteratee The function invoked per iteration.
         * @param {*} accumulator The initial value.
         * @param {boolean} initAccum Specify using the first or last element of
         *  `collection` as the initial value.
         * @param {Function} eachFunc The function to iterate over `collection`.
         * @returns {*} Returns the accumulated value.
         */
        function baseReduce(collection, iteratee, accumulator, initAccum, eachFunc) {
            eachFunc(collection, function (value, index, collection) {
                accumulator = initAccum
                    ? (initAccum = false, value)
                    : iteratee(accumulator, value, index, collection);
            });
            return accumulator;
        }
        /**
         * The base implementation of `_.sortBy` which uses `comparer` to define the
         * sort order of `array` and replaces criteria objects with their corresponding
         * values.
         *
         * @private
         * @param {Array} array The array to sort.
         * @param {Function} comparer The function to define sort order.
         * @returns {Array} Returns `array`.
         */
        function baseSortBy(array, comparer) {
            var length = array.length;
            array.sort(comparer);
            while (length--) {
                array[length] = array[length].value;
            }
            return array;
        }
        /**
         * The base implementation of `_.sum` and `_.sumBy` without support for
         * iteratee shorthands.
         *
         * @private
         * @param {Array} array The array to iterate over.
         * @param {Function} iteratee The function invoked per iteration.
         * @returns {number} Returns the sum.
         */
        function baseSum(array, iteratee) {
            var result, index = -1, length = array.length;
            while (++index < length) {
                var current = iteratee(array[index]);
                if (current !== undefined) {
                    result = result === undefined ? current : (result + current);
                }
            }
            return result;
        }
        /**
         * The base implementation of `_.times` without support for iteratee shorthands
         * or max array length checks.
         *
         * @private
         * @param {number} n The number of times to invoke `iteratee`.
         * @param {Function} iteratee The function invoked per iteration.
         * @returns {Array} Returns the array of results.
         */
        function baseTimes(n, iteratee) {
            var index = -1, result = Array(n);
            while (++index < n) {
                result[index] = iteratee(index);
            }
            return result;
        }
        /**
         * The base implementation of `_.toPairs` and `_.toPairsIn` which creates an array
         * of key-value pairs for `object` corresponding to the property names of `props`.
         *
         * @private
         * @param {Object} object The object to query.
         * @param {Array} props The property names to get values for.
         * @returns {Object} Returns the key-value pairs.
         */
        function baseToPairs(object, props) {
            return arrayMap(props, function (key) {
                return [key, object[key]];
            });
        }
        /**
         * The base implementation of `_.unary` without support for storing metadata.
         *
         * @private
         * @param {Function} func The function to cap arguments for.
         * @returns {Function} Returns the new capped function.
         */
        function baseUnary(func) {
            return function (value) {
                return func(value);
            };
        }
        /**
         * The base implementation of `_.values` and `_.valuesIn` which creates an
         * array of `object` property values corresponding to the property names
         * of `props`.
         *
         * @private
         * @param {Object} object The object to query.
         * @param {Array} props The property names to get values for.
         * @returns {Object} Returns the array of property values.
         */
        function baseValues(object, props) {
            return arrayMap(props, function (key) {
                return object[key];
            });
        }
        /**
         * Checks if a `cache` value for `key` exists.
         *
         * @private
         * @param {Object} cache The cache to query.
         * @param {string} key The key of the entry to check.
         * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
         */
        function cacheHas(cache, key) {
            return cache.has(key);
        }
        /**
         * Used by `_.trim` and `_.trimStart` to get the index of the first string symbol
         * that is not found in the character symbols.
         *
         * @private
         * @param {Array} strSymbols The string symbols to inspect.
         * @param {Array} chrSymbols The character symbols to find.
         * @returns {number} Returns the index of the first unmatched string symbol.
         */
        function charsStartIndex(strSymbols, chrSymbols) {
            var index = -1, length = strSymbols.length;
            while (++index < length && baseIndexOf(chrSymbols, strSymbols[index], 0) > -1) { }
            return index;
        }
        /**
         * Used by `_.trim` and `_.trimEnd` to get the index of the last string symbol
         * that is not found in the character symbols.
         *
         * @private
         * @param {Array} strSymbols The string symbols to inspect.
         * @param {Array} chrSymbols The character symbols to find.
         * @returns {number} Returns the index of the last unmatched string symbol.
         */
        function charsEndIndex(strSymbols, chrSymbols) {
            var index = strSymbols.length;
            while (index-- && baseIndexOf(chrSymbols, strSymbols[index], 0) > -1) { }
            return index;
        }
        /**
         * Gets the number of `placeholder` occurrences in `array`.
         *
         * @private
         * @param {Array} array The array to inspect.
         * @param {*} placeholder The placeholder to search for.
         * @returns {number} Returns the placeholder count.
         */
        function countHolders(array, placeholder) {
            var length = array.length, result = 0;
            while (length--) {
                if (array[length] === placeholder) {
                    ++result;
                }
            }
            return result;
        }
        /**
         * Used by `_.deburr` to convert Latin-1 Supplement and Latin Extended-A
         * letters to basic Latin letters.
         *
         * @private
         * @param {string} letter The matched letter to deburr.
         * @returns {string} Returns the deburred letter.
         */
        var deburrLetter = basePropertyOf(deburredLetters);
        /**
         * Used by `_.escape` to convert characters to HTML entities.
         *
         * @private
         * @param {string} chr The matched character to escape.
         * @returns {string} Returns the escaped character.
         */
        var escapeHtmlChar = basePropertyOf(htmlEscapes);
        /**
         * Used by `_.template` to escape characters for inclusion in compiled string literals.
         *
         * @private
         * @param {string} chr The matched character to escape.
         * @returns {string} Returns the escaped character.
         */
        function escapeStringChar(chr) {
            return '\\' + stringEscapes[chr];
        }
        /**
         * Gets the value at `key` of `object`.
         *
         * @private
         * @param {Object} [object] The object to query.
         * @param {string} key The key of the property to get.
         * @returns {*} Returns the property value.
         */
        function getValue(object, key) {
            return object == null ? undefined : object[key];
        }
        /**
         * Checks if `string` contains Unicode symbols.
         *
         * @private
         * @param {string} string The string to inspect.
         * @returns {boolean} Returns `true` if a symbol is found, else `false`.
         */
        function hasUnicode(string) {
            return reHasUnicode.test(string);
        }
        /**
         * Checks if `string` contains a word composed of Unicode symbols.
         *
         * @private
         * @param {string} string The string to inspect.
         * @returns {boolean} Returns `true` if a word is found, else `false`.
         */
        function hasUnicodeWord(string) {
            return reHasUnicodeWord.test(string);
        }
        /**
         * Converts `iterator` to an array.
         *
         * @private
         * @param {Object} iterator The iterator to convert.
         * @returns {Array} Returns the converted array.
         */
        function iteratorToArray(iterator) {
            var data, result = [];
            while (!(data = iterator.next()).done) {
                result.push(data.value);
            }
            return result;
        }
        /**
         * Converts `map` to its key-value pairs.
         *
         * @private
         * @param {Object} map The map to convert.
         * @returns {Array} Returns the key-value pairs.
         */
        function mapToArray(map) {
            var index = -1, result = Array(map.size);
            map.forEach(function (value, key) {
                result[++index] = [key, value];
            });
            return result;
        }
        /**
         * Creates a unary function that invokes `func` with its argument transformed.
         *
         * @private
         * @param {Function} func The function to wrap.
         * @param {Function} transform The argument transform.
         * @returns {Function} Returns the new function.
         */
        function overArg(func, transform) {
            return function (arg) {
                return func(transform(arg));
            };
        }
        /**
         * Replaces all `placeholder` elements in `array` with an internal placeholder
         * and returns an array of their indexes.
         *
         * @private
         * @param {Array} array The array to modify.
         * @param {*} placeholder The placeholder to replace.
         * @returns {Array} Returns the new array of placeholder indexes.
         */
        function replaceHolders(array, placeholder) {
            var index = -1, length = array.length, resIndex = 0, result = [];
            while (++index < length) {
                var value = array[index];
                if (value === placeholder || value === PLACEHOLDER) {
                    array[index] = PLACEHOLDER;
                    result[resIndex++] = index;
                }
            }
            return result;
        }
        /**
         * Converts `set` to an array of its values.
         *
         * @private
         * @param {Object} set The set to convert.
         * @returns {Array} Returns the values.
         */
        function setToArray(set) {
            var index = -1, result = Array(set.size);
            set.forEach(function (value) {
                result[++index] = value;
            });
            return result;
        }
        /**
         * Converts `set` to its value-value pairs.
         *
         * @private
         * @param {Object} set The set to convert.
         * @returns {Array} Returns the value-value pairs.
         */
        function setToPairs(set) {
            var index = -1, result = Array(set.size);
            set.forEach(function (value) {
                result[++index] = [value, value];
            });
            return result;
        }
        /**
         * A specialized version of `_.indexOf` which performs strict equality
         * comparisons of values, i.e. `===`.
         *
         * @private
         * @param {Array} array The array to inspect.
         * @param {*} value The value to search for.
         * @param {number} fromIndex The index to search from.
         * @returns {number} Returns the index of the matched value, else `-1`.
         */
        function strictIndexOf(array, value, fromIndex) {
            var index = fromIndex - 1, length = array.length;
            while (++index < length) {
                if (array[index] === value) {
                    return index;
                }
            }
            return -1;
        }
        /**
         * A specialized version of `_.lastIndexOf` which performs strict equality
         * comparisons of values, i.e. `===`.
         *
         * @private
         * @param {Array} array The array to inspect.
         * @param {*} value The value to search for.
         * @param {number} fromIndex The index to search from.
         * @returns {number} Returns the index of the matched value, else `-1`.
         */
        function strictLastIndexOf(array, value, fromIndex) {
            var index = fromIndex + 1;
            while (index--) {
                if (array[index] === value) {
                    return index;
                }
            }
            return index;
        }
        /**
         * Gets the number of symbols in `string`.
         *
         * @private
         * @param {string} string The string to inspect.
         * @returns {number} Returns the string size.
         */
        function stringSize(string) {
            return hasUnicode(string)
                ? unicodeSize(string)
                : asciiSize(string);
        }
        /**
         * Converts `string` to an array.
         *
         * @private
         * @param {string} string The string to convert.
         * @returns {Array} Returns the converted array.
         */
        function stringToArray(string) {
            return hasUnicode(string)
                ? unicodeToArray(string)
                : asciiToArray(string);
        }
        /**
         * Used by `_.unescape` to convert HTML entities to characters.
         *
         * @private
         * @param {string} chr The matched character to unescape.
         * @returns {string} Returns the unescaped character.
         */
        var unescapeHtmlChar = basePropertyOf(htmlUnescapes);
        /**
         * Gets the size of a Unicode `string`.
         *
         * @private
         * @param {string} string The string inspect.
         * @returns {number} Returns the string size.
         */
        function unicodeSize(string) {
            var result = reUnicode.lastIndex = 0;
            while (reUnicode.test(string)) {
                ++result;
            }
            return result;
        }
        /**
         * Converts a Unicode `string` to an array.
         *
         * @private
         * @param {string} string The string to convert.
         * @returns {Array} Returns the converted array.
         */
        function unicodeToArray(string) {
            return string.match(reUnicode) || [];
        }
        /**
         * Splits a Unicode `string` into an array of its words.
         *
         * @private
         * @param {string} The string to inspect.
         * @returns {Array} Returns the words of `string`.
         */
        function unicodeWords(string) {
            return string.match(reUnicodeWord) || [];
        }
        /*--------------------------------------------------------------------------*/
        /**
         * Create a new pristine `lodash` function using the `context` object.
         *
         * @static
         * @memberOf _
         * @since 1.1.0
         * @category Util
         * @param {Object} [context=root] The context object.
         * @returns {Function} Returns a new `lodash` function.
         * @example
         *
         * _.mixin({ 'foo': _.constant('foo') });
         *
         * var lodash = _.runInContext();
         * lodash.mixin({ 'bar': lodash.constant('bar') });
         *
         * _.isFunction(_.foo);
         * // => true
         * _.isFunction(_.bar);
         * // => false
         *
         * lodash.isFunction(lodash.foo);
         * // => false
         * lodash.isFunction(lodash.bar);
         * // => true
         *
         * // Create a suped-up `defer` in Node.js.
         * var defer = _.runInContext({ 'setTimeout': setImmediate }).defer;
         */
        var runInContext = (function runInContext(context) {
            context = context == null ? root : _.defaults(root.Object(), context, _.pick(root, contextProps));
            /** Built-in constructor references. */
            var Array = context.Array, Date = context.Date, Error = context.Error, Function = context.Function, Math = context.Math, Object = context.Object, RegExp = context.RegExp, String = context.String, TypeError = context.TypeError;
            /** Used for built-in method references. */
            var arrayProto = Array.prototype, funcProto = Function.prototype, objectProto = Object.prototype;
            /** Used to detect overreaching core-js shims. */
            var coreJsData = context['__core-js_shared__'];
            /** Used to resolve the decompiled source of functions. */
            var funcToString = funcProto.toString;
            /** Used to check objects for own properties. */
            var hasOwnProperty = objectProto.hasOwnProperty;
            /** Used to generate unique IDs. */
            var idCounter = 0;
            /** Used to detect methods masquerading as native. */
            var maskSrcKey = (function () {
                var uid = /[^.]+$/.exec(coreJsData && coreJsData.keys && coreJsData.keys.IE_PROTO || '');
                return uid ? ('Symbol(src)_1.' + uid) : '';
            }());
            /**
             * Used to resolve the
             * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
             * of values.
             */
            var nativeObjectToString = objectProto.toString;
            /** Used to infer the `Object` constructor. */
            var objectCtorString = funcToString.call(Object);
            /** Used to restore the original `_` reference in `_.noConflict`. */
            var oldDash = root._;
            /** Used to detect if a method is native. */
            var reIsNative = RegExp('^' +
                funcToString.call(hasOwnProperty).replace(reRegExpChar, '\\$&')
                    .replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, '$1.*?') + '$');
            /** Built-in value references. */
            var Buffer = moduleExports ? context.Buffer : undefined, Symbol = context.Symbol, Uint8Array = context.Uint8Array, allocUnsafe = Buffer ? Buffer.allocUnsafe : undefined, getPrototype = overArg(Object.getPrototypeOf, Object), objectCreate = Object.create, propertyIsEnumerable = objectProto.propertyIsEnumerable, splice = arrayProto.splice, spreadableSymbol = Symbol ? Symbol.isConcatSpreadable : undefined, symIterator = Symbol ? Symbol.iterator : undefined, symToStringTag = Symbol ? Symbol.toStringTag : undefined;
            var defineProperty = (function () {
                try {
                    var func = getNative(Object, 'defineProperty');
                    func({}, '', {});
                    return func;
                }
                catch (e) { }
            }());
            /** Mocked built-ins. */
            var ctxClearTimeout = context.clearTimeout !== root.clearTimeout && context.clearTimeout, ctxNow = Date && Date.now !== root.Date.now && Date.now, ctxSetTimeout = context.setTimeout !== root.setTimeout && context.setTimeout;
            /* Built-in method references for those with the same name as other `lodash` methods. */
            var nativeCeil = Math.ceil, nativeFloor = Math.floor, nativeGetSymbols = Object.getOwnPropertySymbols, nativeIsBuffer = Buffer ? Buffer.isBuffer : undefined, nativeIsFinite = context.isFinite, nativeJoin = arrayProto.join, nativeKeys = overArg(Object.keys, Object), nativeMax = Math.max, nativeMin = Math.min, nativeNow = Date.now, nativeParseInt = context.parseInt, nativeRandom = Math.random, nativeReverse = arrayProto.reverse;
            /* Built-in method references that are verified to be native. */
            var DataView = getNative(context, 'DataView'), Map = getNative(context, 'Map'), Promise = getNative(context, 'Promise'), Set = getNative(context, 'Set'), WeakMap = getNative(context, 'WeakMap'), nativeCreate = getNative(Object, 'create');
            /** Used to store function metadata. */
            var metaMap = WeakMap && new WeakMap;
            /** Used to lookup unminified function names. */
            var realNames = {};
            /** Used to detect maps, sets, and weakmaps. */
            var dataViewCtorString = toSource(DataView), mapCtorString = toSource(Map), promiseCtorString = toSource(Promise), setCtorString = toSource(Set), weakMapCtorString = toSource(WeakMap);
            /** Used to convert symbols to primitives and strings. */
            var symbolProto = Symbol ? Symbol.prototype : undefined, symbolValueOf = symbolProto ? symbolProto.valueOf : undefined, symbolToString = symbolProto ? symbolProto.toString : undefined;
            /*------------------------------------------------------------------------*/
            /**
             * Creates a `lodash` object which wraps `value` to enable implicit method
             * chain sequences. Methods that operate on and return arrays, collections,
             * and functions can be chained together. Methods that retrieve a single value
             * or may return a primitive value will automatically end the chain sequence
             * and return the unwrapped value. Otherwise, the value must be unwrapped
             * with `_#value`.
             *
             * Explicit chain sequences, which must be unwrapped with `_#value`, may be
             * enabled using `_.chain`.
             *
             * The execution of chained methods is lazy, that is, it's deferred until
             * `_#value` is implicitly or explicitly called.
             *
             * Lazy evaluation allows several methods to support shortcut fusion.
             * Shortcut fusion is an optimization to merge iteratee calls; this avoids
             * the creation of intermediate arrays and can greatly reduce the number of
             * iteratee executions. Sections of a chain sequence qualify for shortcut
             * fusion if the section is applied to an array and iteratees accept only
             * one argument. The heuristic for whether a section qualifies for shortcut
             * fusion is subject to change.
             *
             * Chaining is supported in custom builds as long as the `_#value` method is
             * directly or indirectly included in the build.
             *
             * In addition to lodash methods, wrappers have `Array` and `String` methods.
             *
             * The wrapper `Array` methods are:
             * `concat`, `join`, `pop`, `push`, `shift`, `sort`, `splice`, and `unshift`
             *
             * The wrapper `String` methods are:
             * `replace` and `split`
             *
             * The wrapper methods that support shortcut fusion are:
             * `at`, `compact`, `drop`, `dropRight`, `dropWhile`, `filter`, `find`,
             * `findLast`, `head`, `initial`, `last`, `map`, `reject`, `reverse`, `slice`,
             * `tail`, `take`, `takeRight`, `takeRightWhile`, `takeWhile`, and `toArray`
             *
             * The chainable wrapper methods are:
             * `after`, `ary`, `assign`, `assignIn`, `assignInWith`, `assignWith`, `at`,
             * `before`, `bind`, `bindAll`, `bindKey`, `castArray`, `chain`, `chunk`,
             * `commit`, `compact`, `concat`, `conforms`, `constant`, `countBy`, `create`,
             * `curry`, `debounce`, `defaults`, `defaultsDeep`, `defer`, `delay`,
             * `difference`, `differenceBy`, `differenceWith`, `drop`, `dropRight`,
             * `dropRightWhile`, `dropWhile`, `extend`, `extendWith`, `fill`, `filter`,
             * `flatMap`, `flatMapDeep`, `flatMapDepth`, `flatten`, `flattenDeep`,
             * `flattenDepth`, `flip`, `flow`, `flowRight`, `fromPairs`, `functions`,
             * `functionsIn`, `groupBy`, `initial`, `intersection`, `intersectionBy`,
             * `intersectionWith`, `invert`, `invertBy`, `invokeMap`, `iteratee`, `keyBy`,
             * `keys`, `keysIn`, `map`, `mapKeys`, `mapValues`, `matches`, `matchesProperty`,
             * `memoize`, `merge`, `mergeWith`, `method`, `methodOf`, `mixin`, `negate`,
             * `nthArg`, `omit`, `omitBy`, `once`, `orderBy`, `over`, `overArgs`,
             * `overEvery`, `overSome`, `partial`, `partialRight`, `partition`, `pick`,
             * `pickBy`, `plant`, `property`, `propertyOf`, `pull`, `pullAll`, `pullAllBy`,
             * `pullAllWith`, `pullAt`, `push`, `range`, `rangeRight`, `rearg`, `reject`,
             * `remove`, `rest`, `reverse`, `sampleSize`, `set`, `setWith`, `shuffle`,
             * `slice`, `sort`, `sortBy`, `splice`, `spread`, `tail`, `take`, `takeRight`,
             * `takeRightWhile`, `takeWhile`, `tap`, `throttle`, `thru`, `toArray`,
             * `toPairs`, `toPairsIn`, `toPath`, `toPlainObject`, `transform`, `unary`,
             * `union`, `unionBy`, `unionWith`, `uniq`, `uniqBy`, `uniqWith`, `unset`,
             * `unshift`, `unzip`, `unzipWith`, `update`, `updateWith`, `values`,
             * `valuesIn`, `without`, `wrap`, `xor`, `xorBy`, `xorWith`, `zip`,
             * `zipObject`, `zipObjectDeep`, and `zipWith`
             *
             * The wrapper methods that are **not** chainable by default are:
             * `add`, `attempt`, `camelCase`, `capitalize`, `ceil`, `clamp`, `clone`,
             * `cloneDeep`, `cloneDeepWith`, `cloneWith`, `conformsTo`, `deburr`,
             * `defaultTo`, `divide`, `each`, `eachRight`, `endsWith`, `eq`, `escape`,
             * `escapeRegExp`, `every`, `find`, `findIndex`, `findKey`, `findLast`,
             * `findLastIndex`, `findLastKey`, `first`, `floor`, `forEach`, `forEachRight`,
             * `forIn`, `forInRight`, `forOwn`, `forOwnRight`, `get`, `gt`, `gte`, `has`,
             * `hasIn`, `head`, `identity`, `includes`, `indexOf`, `inRange`, `invoke`,
             * `isArguments`, `isArray`, `isArrayBuffer`, `isArrayLike`, `isArrayLikeObject`,
             * `isBoolean`, `isBuffer`, `isDate`, `isElement`, `isEmpty`, `isEqual`,
             * `isEqualWith`, `isError`, `isFinite`, `isFunction`, `isInteger`, `isLength`,
             * `isMap`, `isMatch`, `isMatchWith`, `isNaN`, `isNative`, `isNil`, `isNull`,
             * `isNumber`, `isObject`, `isObjectLike`, `isPlainObject`, `isRegExp`,
             * `isSafeInteger`, `isSet`, `isString`, `isUndefined`, `isTypedArray`,
             * `isWeakMap`, `isWeakSet`, `join`, `kebabCase`, `last`, `lastIndexOf`,
             * `lowerCase`, `lowerFirst`, `lt`, `lte`, `max`, `maxBy`, `mean`, `meanBy`,
             * `min`, `minBy`, `multiply`, `noConflict`, `noop`, `now`, `nth`, `pad`,
             * `padEnd`, `padStart`, `parseInt`, `pop`, `random`, `reduce`, `reduceRight`,
             * `repeat`, `result`, `round`, `runInContext`, `sample`, `shift`, `size`,
             * `snakeCase`, `some`, `sortedIndex`, `sortedIndexBy`, `sortedLastIndex`,
             * `sortedLastIndexBy`, `startCase`, `startsWith`, `stubArray`, `stubFalse`,
             * `stubObject`, `stubString`, `stubTrue`, `subtract`, `sum`, `sumBy`,
             * `template`, `times`, `toFinite`, `toInteger`, `toJSON`, `toLength`,
             * `toLower`, `toNumber`, `toSafeInteger`, `toString`, `toUpper`, `trim`,
             * `trimEnd`, `trimStart`, `truncate`, `unescape`, `uniqueId`, `upperCase`,
             * `upperFirst`, `value`, and `words`
             *
             * @name _
             * @constructor
             * @category Seq
             * @param {*} value The value to wrap in a `lodash` instance.
             * @returns {Object} Returns the new `lodash` wrapper instance.
             * @example
             *
             * function square(n) {
             *   return n * n;
             * }
             *
             * var wrapped = _([1, 2, 3]);
             *
             * // Returns an unwrapped value.
             * wrapped.reduce(_.add);
             * // => 6
             *
             * // Returns a wrapped value.
             * var squares = wrapped.map(square);
             *
             * _.isArray(squares);
             * // => false
             *
             * _.isArray(squares.value());
             * // => true
             */
            function lodash(value) {
                if (isObjectLike(value) && !isArray(value) && !(value instanceof LazyWrapper)) {
                    if (value instanceof LodashWrapper) {
                        return value;
                    }
                    if (hasOwnProperty.call(value, '__wrapped__')) {
                        return wrapperClone(value);
                    }
                }
                return new LodashWrapper(value);
            }
            /**
             * The base implementation of `_.create` without support for assigning
             * properties to the created object.
             *
             * @private
             * @param {Object} proto The object to inherit from.
             * @returns {Object} Returns the new object.
             */
            var baseCreate = (function () {
                function object() { }
                return function (proto) {
                    if (!isObject(proto)) {
                        return {};
                    }
                    if (objectCreate) {
                        return objectCreate(proto);
                    }
                    object.prototype = proto;
                    var result = new object;
                    object.prototype = undefined;
                    return result;
                };
            }());
            /**
             * The function whose prototype chain sequence wrappers inherit from.
             *
             * @private
             */
            function baseLodash() {
                // No operation performed.
            }
            /**
             * The base constructor for creating `lodash` wrapper objects.
             *
             * @private
             * @param {*} value The value to wrap.
             * @param {boolean} [chainAll] Enable explicit method chain sequences.
             */
            function LodashWrapper(value, chainAll) {
                this.__wrapped__ = value;
                this.__actions__ = [];
                this.__chain__ = !!chainAll;
                this.__index__ = 0;
                this.__values__ = undefined;
            }
            /**
             * By default, the template delimiters used by lodash are like those in
             * embedded Ruby (ERB) as well as ES2015 template strings. Change the
             * following template settings to use alternative delimiters.
             *
             * @static
             * @memberOf _
             * @type {Object}
             */
            lodash.templateSettings = {
                /**
                 * Used to detect `data` property values to be HTML-escaped.
                 *
                 * @memberOf _.templateSettings
                 * @type {RegExp}
                 */
                'escape': reEscape,
                /**
                 * Used to detect code to be evaluated.
                 *
                 * @memberOf _.templateSettings
                 * @type {RegExp}
                 */
                'evaluate': reEvaluate,
                /**
                 * Used to detect `data` property values to inject.
                 *
                 * @memberOf _.templateSettings
                 * @type {RegExp}
                 */
                'interpolate': reInterpolate,
                /**
                 * Used to reference the data object in the template text.
                 *
                 * @memberOf _.templateSettings
                 * @type {string}
                 */
                'variable': '',
                /**
                 * Used to import variables into the compiled template.
                 *
                 * @memberOf _.templateSettings
                 * @type {Object}
                 */
                'imports': {
                    /**
                     * A reference to the `lodash` function.
                     *
                     * @memberOf _.templateSettings.imports
                     * @type {Function}
                     */
                    '_': lodash
                }
            };
            // Ensure wrappers are instances of `baseLodash`.
            lodash.prototype = baseLodash.prototype;
            lodash.prototype.constructor = lodash;
            LodashWrapper.prototype = baseCreate(baseLodash.prototype);
            LodashWrapper.prototype.constructor = LodashWrapper;
            /*------------------------------------------------------------------------*/
            /**
             * Creates a lazy wrapper object which wraps `value` to enable lazy evaluation.
             *
             * @private
             * @constructor
             * @param {*} value The value to wrap.
             */
            function LazyWrapper(value) {
                this.__wrapped__ = value;
                this.__actions__ = [];
                this.__dir__ = 1;
                this.__filtered__ = false;
                this.__iteratees__ = [];
                this.__takeCount__ = MAX_ARRAY_LENGTH;
                this.__views__ = [];
            }
            /**
             * Creates a clone of the lazy wrapper object.
             *
             * @private
             * @name clone
             * @memberOf LazyWrapper
             * @returns {Object} Returns the cloned `LazyWrapper` object.
             */
            function lazyClone() {
                var result = new LazyWrapper(this.__wrapped__);
                result.__actions__ = copyArray(this.__actions__);
                result.__dir__ = this.__dir__;
                result.__filtered__ = this.__filtered__;
                result.__iteratees__ = copyArray(this.__iteratees__);
                result.__takeCount__ = this.__takeCount__;
                result.__views__ = copyArray(this.__views__);
                return result;
            }
            /**
             * Reverses the direction of lazy iteration.
             *
             * @private
             * @name reverse
             * @memberOf LazyWrapper
             * @returns {Object} Returns the new reversed `LazyWrapper` object.
             */
            function lazyReverse() {
                if (this.__filtered__) {
                    var result = new LazyWrapper(this);
                    result.__dir__ = -1;
                    result.__filtered__ = true;
                }
                else {
                    result = this.clone();
                    result.__dir__ *= -1;
                }
                return result;
            }
            /**
             * Extracts the unwrapped value from its lazy wrapper.
             *
             * @private
             * @name value
             * @memberOf LazyWrapper
             * @returns {*} Returns the unwrapped value.
             */
            function lazyValue() {
                var array = this.__wrapped__.value(), dir = this.__dir__, isArr = isArray(array), isRight = dir < 0, arrLength = isArr ? array.length : 0, view = getView(0, arrLength, this.__views__), start = view.start, end = view.end, length = end - start, index = isRight ? end : (start - 1), iteratees = this.__iteratees__, iterLength = iteratees.length, resIndex = 0, takeCount = nativeMin(length, this.__takeCount__);
                if (!isArr || (!isRight && arrLength == length && takeCount == length)) {
                    return baseWrapperValue(array, this.__actions__);
                }
                var result = [];
                outer: while (length-- && resIndex < takeCount) {
                    index += dir;
                    var iterIndex = -1, value = array[index];
                    while (++iterIndex < iterLength) {
                        var data = iteratees[iterIndex], iteratee = data.iteratee, type = data.type, computed = iteratee(value);
                        if (type == LAZY_MAP_FLAG) {
                            value = computed;
                        }
                        else if (!computed) {
                            if (type == LAZY_FILTER_FLAG) {
                                continue outer;
                            }
                            else {
                                break outer;
                            }
                        }
                    }
                    result[resIndex++] = value;
                }
                return result;
            }
            // Ensure `LazyWrapper` is an instance of `baseLodash`.
            LazyWrapper.prototype = baseCreate(baseLodash.prototype);
            LazyWrapper.prototype.constructor = LazyWrapper;
            /*------------------------------------------------------------------------*/
            /**
             * Creates a hash object.
             *
             * @private
             * @constructor
             * @param {Array} [entries] The key-value pairs to cache.
             */
            function Hash(entries) {
                var index = -1, length = entries == null ? 0 : entries.length;
                this.clear();
                while (++index < length) {
                    var entry = entries[index];
                    this.set(entry[0], entry[1]);
                }
            }
            /**
             * Removes all key-value entries from the hash.
             *
             * @private
             * @name clear
             * @memberOf Hash
             */
            function hashClear() {
                this.__data__ = nativeCreate ? nativeCreate(null) : {};
                this.size = 0;
            }
            /**
             * Removes `key` and its value from the hash.
             *
             * @private
             * @name delete
             * @memberOf Hash
             * @param {Object} hash The hash to modify.
             * @param {string} key The key of the value to remove.
             * @returns {boolean} Returns `true` if the entry was removed, else `false`.
             */
            function hashDelete(key) {
                var result = this.has(key) && delete this.__data__[key];
                this.size -= result ? 1 : 0;
                return result;
            }
            /**
             * Gets the hash value for `key`.
             *
             * @private
             * @name get
             * @memberOf Hash
             * @param {string} key The key of the value to get.
             * @returns {*} Returns the entry value.
             */
            function hashGet(key) {
                var data = this.__data__;
                if (nativeCreate) {
                    var result = data[key];
                    return result === HASH_UNDEFINED ? undefined : result;
                }
                return hasOwnProperty.call(data, key) ? data[key] : undefined;
            }
            /**
             * Checks if a hash value for `key` exists.
             *
             * @private
             * @name has
             * @memberOf Hash
             * @param {string} key The key of the entry to check.
             * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
             */
            function hashHas(key) {
                var data = this.__data__;
                return nativeCreate ? (data[key] !== undefined) : hasOwnProperty.call(data, key);
            }
            /**
             * Sets the hash `key` to `value`.
             *
             * @private
             * @name set
             * @memberOf Hash
             * @param {string} key The key of the value to set.
             * @param {*} value The value to set.
             * @returns {Object} Returns the hash instance.
             */
            function hashSet(key, value) {
                var data = this.__data__;
                this.size += this.has(key) ? 0 : 1;
                data[key] = (nativeCreate && value === undefined) ? HASH_UNDEFINED : value;
                return this;
            }
            // Add methods to `Hash`.
            Hash.prototype.clear = hashClear;
            Hash.prototype['delete'] = hashDelete;
            Hash.prototype.get = hashGet;
            Hash.prototype.has = hashHas;
            Hash.prototype.set = hashSet;
            /*------------------------------------------------------------------------*/
            /**
             * Creates an list cache object.
             *
             * @private
             * @constructor
             * @param {Array} [entries] The key-value pairs to cache.
             */
            function ListCache(entries) {
                var index = -1, length = entries == null ? 0 : entries.length;
                this.clear();
                while (++index < length) {
                    var entry = entries[index];
                    this.set(entry[0], entry[1]);
                }
            }
            /**
             * Removes all key-value entries from the list cache.
             *
             * @private
             * @name clear
             * @memberOf ListCache
             */
            function listCacheClear() {
                this.__data__ = [];
                this.size = 0;
            }
            /**
             * Removes `key` and its value from the list cache.
             *
             * @private
             * @name delete
             * @memberOf ListCache
             * @param {string} key The key of the value to remove.
             * @returns {boolean} Returns `true` if the entry was removed, else `false`.
             */
            function listCacheDelete(key) {
                var data = this.__data__, index = assocIndexOf(data, key);
                if (index < 0) {
                    return false;
                }
                var lastIndex = data.length - 1;
                if (index == lastIndex) {
                    data.pop();
                }
                else {
                    splice.call(data, index, 1);
                }
                --this.size;
                return true;
            }
            /**
             * Gets the list cache value for `key`.
             *
             * @private
             * @name get
             * @memberOf ListCache
             * @param {string} key The key of the value to get.
             * @returns {*} Returns the entry value.
             */
            function listCacheGet(key) {
                var data = this.__data__, index = assocIndexOf(data, key);
                return index < 0 ? undefined : data[index][1];
            }
            /**
             * Checks if a list cache value for `key` exists.
             *
             * @private
             * @name has
             * @memberOf ListCache
             * @param {string} key The key of the entry to check.
             * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
             */
            function listCacheHas(key) {
                return assocIndexOf(this.__data__, key) > -1;
            }
            /**
             * Sets the list cache `key` to `value`.
             *
             * @private
             * @name set
             * @memberOf ListCache
             * @param {string} key The key of the value to set.
             * @param {*} value The value to set.
             * @returns {Object} Returns the list cache instance.
             */
            function listCacheSet(key, value) {
                var data = this.__data__, index = assocIndexOf(data, key);
                if (index < 0) {
                    ++this.size;
                    data.push([key, value]);
                }
                else {
                    data[index][1] = value;
                }
                return this;
            }
            // Add methods to `ListCache`.
            ListCache.prototype.clear = listCacheClear;
            ListCache.prototype['delete'] = listCacheDelete;
            ListCache.prototype.get = listCacheGet;
            ListCache.prototype.has = listCacheHas;
            ListCache.prototype.set = listCacheSet;
            /*------------------------------------------------------------------------*/
            /**
             * Creates a map cache object to store key-value pairs.
             *
             * @private
             * @constructor
             * @param {Array} [entries] The key-value pairs to cache.
             */
            function MapCache(entries) {
                var index = -1, length = entries == null ? 0 : entries.length;
                this.clear();
                while (++index < length) {
                    var entry = entries[index];
                    this.set(entry[0], entry[1]);
                }
            }
            /**
             * Removes all key-value entries from the map.
             *
             * @private
             * @name clear
             * @memberOf MapCache
             */
            function mapCacheClear() {
                this.size = 0;
                this.__data__ = {
                    'hash': new Hash,
                    'map': new (Map || ListCache),
                    'string': new Hash
                };
            }
            /**
             * Removes `key` and its value from the map.
             *
             * @private
             * @name delete
             * @memberOf MapCache
             * @param {string} key The key of the value to remove.
             * @returns {boolean} Returns `true` if the entry was removed, else `false`.
             */
            function mapCacheDelete(key) {
                var result = getMapData(this, key)['delete'](key);
                this.size -= result ? 1 : 0;
                return result;
            }
            /**
             * Gets the map value for `key`.
             *
             * @private
             * @name get
             * @memberOf MapCache
             * @param {string} key The key of the value to get.
             * @returns {*} Returns the entry value.
             */
            function mapCacheGet(key) {
                return getMapData(this, key).get(key);
            }
            /**
             * Checks if a map value for `key` exists.
             *
             * @private
             * @name has
             * @memberOf MapCache
             * @param {string} key The key of the entry to check.
             * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
             */
            function mapCacheHas(key) {
                return getMapData(this, key).has(key);
            }
            /**
             * Sets the map `key` to `value`.
             *
             * @private
             * @name set
             * @memberOf MapCache
             * @param {string} key The key of the value to set.
             * @param {*} value The value to set.
             * @returns {Object} Returns the map cache instance.
             */
            function mapCacheSet(key, value) {
                var data = getMapData(this, key), size = data.size;
                data.set(key, value);
                this.size += data.size == size ? 0 : 1;
                return this;
            }
            // Add methods to `MapCache`.
            MapCache.prototype.clear = mapCacheClear;
            MapCache.prototype['delete'] = mapCacheDelete;
            MapCache.prototype.get = mapCacheGet;
            MapCache.prototype.has = mapCacheHas;
            MapCache.prototype.set = mapCacheSet;
            /*------------------------------------------------------------------------*/
            /**
             *
             * Creates an array cache object to store unique values.
             *
             * @private
             * @constructor
             * @param {Array} [values] The values to cache.
             */
            function SetCache(values) {
                var index = -1, length = values == null ? 0 : values.length;
                this.__data__ = new MapCache;
                while (++index < length) {
                    this.add(values[index]);
                }
            }
            /**
             * Adds `value` to the array cache.
             *
             * @private
             * @name add
             * @memberOf SetCache
             * @alias push
             * @param {*} value The value to cache.
             * @returns {Object} Returns the cache instance.
             */
            function setCacheAdd(value) {
                this.__data__.set(value, HASH_UNDEFINED);
                return this;
            }
            /**
             * Checks if `value` is in the array cache.
             *
             * @private
             * @name has
             * @memberOf SetCache
             * @param {*} value The value to search for.
             * @returns {number} Returns `true` if `value` is found, else `false`.
             */
            function setCacheHas(value) {
                return this.__data__.has(value);
            }
            // Add methods to `SetCache`.
            SetCache.prototype.add = SetCache.prototype.push = setCacheAdd;
            SetCache.prototype.has = setCacheHas;
            /*------------------------------------------------------------------------*/
            /**
             * Creates a stack cache object to store key-value pairs.
             *
             * @private
             * @constructor
             * @param {Array} [entries] The key-value pairs to cache.
             */
            function Stack(entries) {
                var data = this.__data__ = new ListCache(entries);
                this.size = data.size;
            }
            /**
             * Removes all key-value entries from the stack.
             *
             * @private
             * @name clear
             * @memberOf Stack
             */
            function stackClear() {
                this.__data__ = new ListCache;
                this.size = 0;
            }
            /**
             * Removes `key` and its value from the stack.
             *
             * @private
             * @name delete
             * @memberOf Stack
             * @param {string} key The key of the value to remove.
             * @returns {boolean} Returns `true` if the entry was removed, else `false`.
             */
            function stackDelete(key) {
                var data = this.__data__, result = data['delete'](key);
                this.size = data.size;
                return result;
            }
            /**
             * Gets the stack value for `key`.
             *
             * @private
             * @name get
             * @memberOf Stack
             * @param {string} key The key of the value to get.
             * @returns {*} Returns the entry value.
             */
            function stackGet(key) {
                return this.__data__.get(key);
            }
            /**
             * Checks if a stack value for `key` exists.
             *
             * @private
             * @name has
             * @memberOf Stack
             * @param {string} key The key of the entry to check.
             * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
             */
            function stackHas(key) {
                return this.__data__.has(key);
            }
            /**
             * Sets the stack `key` to `value`.
             *
             * @private
             * @name set
             * @memberOf Stack
             * @param {string} key The key of the value to set.
             * @param {*} value The value to set.
             * @returns {Object} Returns the stack cache instance.
             */
            function stackSet(key, value) {
                var data = this.__data__;
                if (data instanceof ListCache) {
                    var pairs = data.__data__;
                    if (!Map || (pairs.length < LARGE_ARRAY_SIZE - 1)) {
                        pairs.push([key, value]);
                        this.size = ++data.size;
                        return this;
                    }
                    data = this.__data__ = new MapCache(pairs);
                }
                data.set(key, value);
                this.size = data.size;
                return this;
            }
            // Add methods to `Stack`.
            Stack.prototype.clear = stackClear;
            Stack.prototype['delete'] = stackDelete;
            Stack.prototype.get = stackGet;
            Stack.prototype.has = stackHas;
            Stack.prototype.set = stackSet;
            /*------------------------------------------------------------------------*/
            /**
             * Creates an array of the enumerable property names of the array-like `value`.
             *
             * @private
             * @param {*} value The value to query.
             * @param {boolean} inherited Specify returning inherited property names.
             * @returns {Array} Returns the array of property names.
             */
            function arrayLikeKeys(value, inherited) {
                var isArr = isArray(value), isArg = !isArr && isArguments(value), isBuff = !isArr && !isArg && isBuffer(value), isType = !isArr && !isArg && !isBuff && isTypedArray(value), skipIndexes = isArr || isArg || isBuff || isType, result = skipIndexes ? baseTimes(value.length, String) : [], length = result.length;
                for (var key in value) {
                    if ((inherited || hasOwnProperty.call(value, key)) &&
                        !(skipIndexes && (
                        // Safari 9 has enumerable `arguments.length` in strict mode.
                        key == 'length' ||
                            // Node.js 0.10 has enumerable non-index properties on buffers.
                            (isBuff && (key == 'offset' || key == 'parent')) ||
                            // PhantomJS 2 has enumerable non-index properties on typed arrays.
                            (isType && (key == 'buffer' || key == 'byteLength' || key == 'byteOffset')) ||
                            // Skip index properties.
                            isIndex(key, length)))) {
                        result.push(key);
                    }
                }
                return result;
            }
            /**
             * A specialized version of `_.sample` for arrays.
             *
             * @private
             * @param {Array} array The array to sample.
             * @returns {*} Returns the random element.
             */
            function arraySample(array) {
                var length = array.length;
                return length ? array[baseRandom(0, length - 1)] : undefined;
            }
            /**
             * A specialized version of `_.sampleSize` for arrays.
             *
             * @private
             * @param {Array} array The array to sample.
             * @param {number} n The number of elements to sample.
             * @returns {Array} Returns the random elements.
             */
            function arraySampleSize(array, n) {
                return shuffleSelf(copyArray(array), baseClamp(n, 0, array.length));
            }
            /**
             * A specialized version of `_.shuffle` for arrays.
             *
             * @private
             * @param {Array} array The array to shuffle.
             * @returns {Array} Returns the new shuffled array.
             */
            function arrayShuffle(array) {
                return shuffleSelf(copyArray(array));
            }
            /**
             * This function is like `assignValue` except that it doesn't assign
             * `undefined` values.
             *
             * @private
             * @param {Object} object The object to modify.
             * @param {string} key The key of the property to assign.
             * @param {*} value The value to assign.
             */
            function assignMergeValue(object, key, value) {
                if ((value !== undefined && !eq(object[key], value)) ||
                    (value === undefined && !(key in object))) {
                    baseAssignValue(object, key, value);
                }
            }
            /**
             * Assigns `value` to `key` of `object` if the existing value is not equivalent
             * using [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
             * for equality comparisons.
             *
             * @private
             * @param {Object} object The object to modify.
             * @param {string} key The key of the property to assign.
             * @param {*} value The value to assign.
             */
            function assignValue(object, key, value) {
                var objValue = object[key];
                if (!(hasOwnProperty.call(object, key) && eq(objValue, value)) ||
                    (value === undefined && !(key in object))) {
                    baseAssignValue(object, key, value);
                }
            }
            /**
             * Gets the index at which the `key` is found in `array` of key-value pairs.
             *
             * @private
             * @param {Array} array The array to inspect.
             * @param {*} key The key to search for.
             * @returns {number} Returns the index of the matched value, else `-1`.
             */
            function assocIndexOf(array, key) {
                var length = array.length;
                while (length--) {
                    if (eq(array[length][0], key)) {
                        return length;
                    }
                }
                return -1;
            }
            /**
             * Aggregates elements of `collection` on `accumulator` with keys transformed
             * by `iteratee` and values set by `setter`.
             *
             * @private
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} setter The function to set `accumulator` values.
             * @param {Function} iteratee The iteratee to transform keys.
             * @param {Object} accumulator The initial aggregated object.
             * @returns {Function} Returns `accumulator`.
             */
            function baseAggregator(collection, setter, iteratee, accumulator) {
                baseEach(collection, function (value, key, collection) {
                    setter(accumulator, value, iteratee(value), collection);
                });
                return accumulator;
            }
            /**
             * The base implementation of `_.assign` without support for multiple sources
             * or `customizer` functions.
             *
             * @private
             * @param {Object} object The destination object.
             * @param {Object} source The source object.
             * @returns {Object} Returns `object`.
             */
            function baseAssign(object, source) {
                return object && copyObject(source, keys(source), object);
            }
            /**
             * The base implementation of `_.assignIn` without support for multiple sources
             * or `customizer` functions.
             *
             * @private
             * @param {Object} object The destination object.
             * @param {Object} source The source object.
             * @returns {Object} Returns `object`.
             */
            function baseAssignIn(object, source) {
                return object && copyObject(source, keysIn(source), object);
            }
            /**
             * The base implementation of `assignValue` and `assignMergeValue` without
             * value checks.
             *
             * @private
             * @param {Object} object The object to modify.
             * @param {string} key The key of the property to assign.
             * @param {*} value The value to assign.
             */
            function baseAssignValue(object, key, value) {
                if (key == '__proto__' && defineProperty) {
                    defineProperty(object, key, {
                        'configurable': true,
                        'enumerable': true,
                        'value': value,
                        'writable': true
                    });
                }
                else {
                    object[key] = value;
                }
            }
            /**
             * The base implementation of `_.at` without support for individual paths.
             *
             * @private
             * @param {Object} object The object to iterate over.
             * @param {string[]} paths The property paths to pick.
             * @returns {Array} Returns the picked elements.
             */
            function baseAt(object, paths) {
                var index = -1, length = paths.length, result = Array(length), skip = object == null;
                while (++index < length) {
                    result[index] = skip ? undefined : get(object, paths[index]);
                }
                return result;
            }
            /**
             * The base implementation of `_.clamp` which doesn't coerce arguments.
             *
             * @private
             * @param {number} number The number to clamp.
             * @param {number} [lower] The lower bound.
             * @param {number} upper The upper bound.
             * @returns {number} Returns the clamped number.
             */
            function baseClamp(number, lower, upper) {
                if (number === number) {
                    if (upper !== undefined) {
                        number = number <= upper ? number : upper;
                    }
                    if (lower !== undefined) {
                        number = number >= lower ? number : lower;
                    }
                }
                return number;
            }
            /**
             * The base implementation of `_.clone` and `_.cloneDeep` which tracks
             * traversed objects.
             *
             * @private
             * @param {*} value The value to clone.
             * @param {boolean} bitmask The bitmask flags.
             *  1 - Deep clone
             *  2 - Flatten inherited properties
             *  4 - Clone symbols
             * @param {Function} [customizer] The function to customize cloning.
             * @param {string} [key] The key of `value`.
             * @param {Object} [object] The parent object of `value`.
             * @param {Object} [stack] Tracks traversed objects and their clone counterparts.
             * @returns {*} Returns the cloned value.
             */
            function baseClone(value, bitmask, customizer, key, object, stack) {
                var result, isDeep = bitmask & CLONE_DEEP_FLAG, isFlat = bitmask & CLONE_FLAT_FLAG, isFull = bitmask & CLONE_SYMBOLS_FLAG;
                if (customizer) {
                    result = object ? customizer(value, key, object, stack) : customizer(value);
                }
                if (result !== undefined) {
                    return result;
                }
                if (!isObject(value)) {
                    return value;
                }
                var isArr = isArray(value);
                if (isArr) {
                    result = initCloneArray(value);
                    if (!isDeep) {
                        return copyArray(value, result);
                    }
                }
                else {
                    var tag = getTag(value), isFunc = tag == funcTag || tag == genTag;
                    if (isBuffer(value)) {
                        return cloneBuffer(value, isDeep);
                    }
                    if (tag == objectTag || tag == argsTag || (isFunc && !object)) {
                        result = (isFlat || isFunc) ? {} : initCloneObject(value);
                        if (!isDeep) {
                            return isFlat
                                ? copySymbolsIn(value, baseAssignIn(result, value))
                                : copySymbols(value, baseAssign(result, value));
                        }
                    }
                    else {
                        if (!cloneableTags[tag]) {
                            return object ? value : {};
                        }
                        result = initCloneByTag(value, tag, isDeep);
                    }
                }
                // Check for circular references and return its corresponding clone.
                stack || (stack = new Stack);
                var stacked = stack.get(value);
                if (stacked) {
                    return stacked;
                }
                stack.set(value, result);
                if (isSet(value)) {
                    value.forEach(function (subValue) {
                        result.add(baseClone(subValue, bitmask, customizer, subValue, value, stack));
                    });
                }
                else if (isMap(value)) {
                    value.forEach(function (subValue, key) {
                        result.set(key, baseClone(subValue, bitmask, customizer, key, value, stack));
                    });
                }
                var keysFunc = isFull
                    ? (isFlat ? getAllKeysIn : getAllKeys)
                    : (isFlat ? keysIn : keys);
                var props = isArr ? undefined : keysFunc(value);
                arrayEach(props || value, function (subValue, key) {
                    if (props) {
                        key = subValue;
                        subValue = value[key];
                    }
                    // Recursively populate clone (susceptible to call stack limits).
                    assignValue(result, key, baseClone(subValue, bitmask, customizer, key, value, stack));
                });
                return result;
            }
            /**
             * The base implementation of `_.conforms` which doesn't clone `source`.
             *
             * @private
             * @param {Object} source The object of property predicates to conform to.
             * @returns {Function} Returns the new spec function.
             */
            function baseConforms(source) {
                var props = keys(source);
                return function (object) {
                    return baseConformsTo(object, source, props);
                };
            }
            /**
             * The base implementation of `_.conformsTo` which accepts `props` to check.
             *
             * @private
             * @param {Object} object The object to inspect.
             * @param {Object} source The object of property predicates to conform to.
             * @returns {boolean} Returns `true` if `object` conforms, else `false`.
             */
            function baseConformsTo(object, source, props) {
                var length = props.length;
                if (object == null) {
                    return !length;
                }
                object = Object(object);
                while (length--) {
                    var key = props[length], predicate = source[key], value = object[key];
                    if ((value === undefined && !(key in object)) || !predicate(value)) {
                        return false;
                    }
                }
                return true;
            }
            /**
             * The base implementation of `_.delay` and `_.defer` which accepts `args`
             * to provide to `func`.
             *
             * @private
             * @param {Function} func The function to delay.
             * @param {number} wait The number of milliseconds to delay invocation.
             * @param {Array} args The arguments to provide to `func`.
             * @returns {number|Object} Returns the timer id or timeout object.
             */
            function baseDelay(func, wait, args) {
                if (typeof func != 'function') {
                    throw new TypeError(FUNC_ERROR_TEXT);
                }
                return setTimeout(function () { func.apply(undefined, args); }, wait);
            }
            /**
             * The base implementation of methods like `_.difference` without support
             * for excluding multiple arrays or iteratee shorthands.
             *
             * @private
             * @param {Array} array The array to inspect.
             * @param {Array} values The values to exclude.
             * @param {Function} [iteratee] The iteratee invoked per element.
             * @param {Function} [comparator] The comparator invoked per element.
             * @returns {Array} Returns the new array of filtered values.
             */
            function baseDifference(array, values, iteratee, comparator) {
                var index = -1, includes = arrayIncludes, isCommon = true, length = array.length, result = [], valuesLength = values.length;
                if (!length) {
                    return result;
                }
                if (iteratee) {
                    values = arrayMap(values, baseUnary(iteratee));
                }
                if (comparator) {
                    includes = arrayIncludesWith;
                    isCommon = false;
                }
                else if (values.length >= LARGE_ARRAY_SIZE) {
                    includes = cacheHas;
                    isCommon = false;
                    values = new SetCache(values);
                }
                outer: while (++index < length) {
                    var value = array[index], computed = iteratee == null ? value : iteratee(value);
                    value = (comparator || value !== 0) ? value : 0;
                    if (isCommon && computed === computed) {
                        var valuesIndex = valuesLength;
                        while (valuesIndex--) {
                            if (values[valuesIndex] === computed) {
                                continue outer;
                            }
                        }
                        result.push(value);
                    }
                    else if (!includes(values, computed, comparator)) {
                        result.push(value);
                    }
                }
                return result;
            }
            /**
             * The base implementation of `_.forEach` without support for iteratee shorthands.
             *
             * @private
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} iteratee The function invoked per iteration.
             * @returns {Array|Object} Returns `collection`.
             */
            var baseEach = createBaseEach(baseForOwn);
            /**
             * The base implementation of `_.forEachRight` without support for iteratee shorthands.
             *
             * @private
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} iteratee The function invoked per iteration.
             * @returns {Array|Object} Returns `collection`.
             */
            var baseEachRight = createBaseEach(baseForOwnRight, true);
            /**
             * The base implementation of `_.every` without support for iteratee shorthands.
             *
             * @private
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} predicate The function invoked per iteration.
             * @returns {boolean} Returns `true` if all elements pass the predicate check,
             *  else `false`
             */
            function baseEvery(collection, predicate) {
                var result = true;
                baseEach(collection, function (value, index, collection) {
                    result = !!predicate(value, index, collection);
                    return result;
                });
                return result;
            }
            /**
             * The base implementation of methods like `_.max` and `_.min` which accepts a
             * `comparator` to determine the extremum value.
             *
             * @private
             * @param {Array} array The array to iterate over.
             * @param {Function} iteratee The iteratee invoked per iteration.
             * @param {Function} comparator The comparator used to compare values.
             * @returns {*} Returns the extremum value.
             */
            function baseExtremum(array, iteratee, comparator) {
                var index = -1, length = array.length;
                while (++index < length) {
                    var value = array[index], current = iteratee(value);
                    if (current != null && (computed === undefined
                        ? (current === current && !isSymbol(current))
                        : comparator(current, computed))) {
                        var computed = current, result = value;
                    }
                }
                return result;
            }
            /**
             * The base implementation of `_.fill` without an iteratee call guard.
             *
             * @private
             * @param {Array} array The array to fill.
             * @param {*} value The value to fill `array` with.
             * @param {number} [start=0] The start position.
             * @param {number} [end=array.length] The end position.
             * @returns {Array} Returns `array`.
             */
            function baseFill(array, value, start, end) {
                var length = array.length;
                start = toInteger(start);
                if (start < 0) {
                    start = -start > length ? 0 : (length + start);
                }
                end = (end === undefined || end > length) ? length : toInteger(end);
                if (end < 0) {
                    end += length;
                }
                end = start > end ? 0 : toLength(end);
                while (start < end) {
                    array[start++] = value;
                }
                return array;
            }
            /**
             * The base implementation of `_.filter` without support for iteratee shorthands.
             *
             * @private
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} predicate The function invoked per iteration.
             * @returns {Array} Returns the new filtered array.
             */
            function baseFilter(collection, predicate) {
                var result = [];
                baseEach(collection, function (value, index, collection) {
                    if (predicate(value, index, collection)) {
                        result.push(value);
                    }
                });
                return result;
            }
            /**
             * The base implementation of `_.flatten` with support for restricting flattening.
             *
             * @private
             * @param {Array} array The array to flatten.
             * @param {number} depth The maximum recursion depth.
             * @param {boolean} [predicate=isFlattenable] The function invoked per iteration.
             * @param {boolean} [isStrict] Restrict to values that pass `predicate` checks.
             * @param {Array} [result=[]] The initial result value.
             * @returns {Array} Returns the new flattened array.
             */
            function baseFlatten(array, depth, predicate, isStrict, result) {
                var index = -1, length = array.length;
                predicate || (predicate = isFlattenable);
                result || (result = []);
                while (++index < length) {
                    var value = array[index];
                    if (depth > 0 && predicate(value)) {
                        if (depth > 1) {
                            // Recursively flatten arrays (susceptible to call stack limits).
                            baseFlatten(value, depth - 1, predicate, isStrict, result);
                        }
                        else {
                            arrayPush(result, value);
                        }
                    }
                    else if (!isStrict) {
                        result[result.length] = value;
                    }
                }
                return result;
            }
            /**
             * The base implementation of `baseForOwn` which iterates over `object`
             * properties returned by `keysFunc` and invokes `iteratee` for each property.
             * Iteratee functions may exit iteration early by explicitly returning `false`.
             *
             * @private
             * @param {Object} object The object to iterate over.
             * @param {Function} iteratee The function invoked per iteration.
             * @param {Function} keysFunc The function to get the keys of `object`.
             * @returns {Object} Returns `object`.
             */
            var baseFor = createBaseFor();
            /**
             * This function is like `baseFor` except that it iterates over properties
             * in the opposite order.
             *
             * @private
             * @param {Object} object The object to iterate over.
             * @param {Function} iteratee The function invoked per iteration.
             * @param {Function} keysFunc The function to get the keys of `object`.
             * @returns {Object} Returns `object`.
             */
            var baseForRight = createBaseFor(true);
            /**
             * The base implementation of `_.forOwn` without support for iteratee shorthands.
             *
             * @private
             * @param {Object} object The object to iterate over.
             * @param {Function} iteratee The function invoked per iteration.
             * @returns {Object} Returns `object`.
             */
            function baseForOwn(object, iteratee) {
                return object && baseFor(object, iteratee, keys);
            }
            /**
             * The base implementation of `_.forOwnRight` without support for iteratee shorthands.
             *
             * @private
             * @param {Object} object The object to iterate over.
             * @param {Function} iteratee The function invoked per iteration.
             * @returns {Object} Returns `object`.
             */
            function baseForOwnRight(object, iteratee) {
                return object && baseForRight(object, iteratee, keys);
            }
            /**
             * The base implementation of `_.functions` which creates an array of
             * `object` function property names filtered from `props`.
             *
             * @private
             * @param {Object} object The object to inspect.
             * @param {Array} props The property names to filter.
             * @returns {Array} Returns the function names.
             */
            function baseFunctions(object, props) {
                return arrayFilter(props, function (key) {
                    return isFunction(object[key]);
                });
            }
            /**
             * The base implementation of `_.get` without support for default values.
             *
             * @private
             * @param {Object} object The object to query.
             * @param {Array|string} path The path of the property to get.
             * @returns {*} Returns the resolved value.
             */
            function baseGet(object, path) {
                path = castPath(path, object);
                var index = 0, length = path.length;
                while (object != null && index < length) {
                    object = object[toKey(path[index++])];
                }
                return (index && index == length) ? object : undefined;
            }
            /**
             * The base implementation of `getAllKeys` and `getAllKeysIn` which uses
             * `keysFunc` and `symbolsFunc` to get the enumerable property names and
             * symbols of `object`.
             *
             * @private
             * @param {Object} object The object to query.
             * @param {Function} keysFunc The function to get the keys of `object`.
             * @param {Function} symbolsFunc The function to get the symbols of `object`.
             * @returns {Array} Returns the array of property names and symbols.
             */
            function baseGetAllKeys(object, keysFunc, symbolsFunc) {
                var result = keysFunc(object);
                return isArray(object) ? result : arrayPush(result, symbolsFunc(object));
            }
            /**
             * The base implementation of `getTag` without fallbacks for buggy environments.
             *
             * @private
             * @param {*} value The value to query.
             * @returns {string} Returns the `toStringTag`.
             */
            function baseGetTag(value) {
                if (value == null) {
                    return value === undefined ? undefinedTag : nullTag;
                }
                return (symToStringTag && symToStringTag in Object(value))
                    ? getRawTag(value)
                    : objectToString(value);
            }
            /**
             * The base implementation of `_.gt` which doesn't coerce arguments.
             *
             * @private
             * @param {*} value The value to compare.
             * @param {*} other The other value to compare.
             * @returns {boolean} Returns `true` if `value` is greater than `other`,
             *  else `false`.
             */
            function baseGt(value, other) {
                return value > other;
            }
            /**
             * The base implementation of `_.has` without support for deep paths.
             *
             * @private
             * @param {Object} [object] The object to query.
             * @param {Array|string} key The key to check.
             * @returns {boolean} Returns `true` if `key` exists, else `false`.
             */
            function baseHas(object, key) {
                return object != null && hasOwnProperty.call(object, key);
            }
            /**
             * The base implementation of `_.hasIn` without support for deep paths.
             *
             * @private
             * @param {Object} [object] The object to query.
             * @param {Array|string} key The key to check.
             * @returns {boolean} Returns `true` if `key` exists, else `false`.
             */
            function baseHasIn(object, key) {
                return object != null && key in Object(object);
            }
            /**
             * The base implementation of `_.inRange` which doesn't coerce arguments.
             *
             * @private
             * @param {number} number The number to check.
             * @param {number} start The start of the range.
             * @param {number} end The end of the range.
             * @returns {boolean} Returns `true` if `number` is in the range, else `false`.
             */
            function baseInRange(number, start, end) {
                return number >= nativeMin(start, end) && number < nativeMax(start, end);
            }
            /**
             * The base implementation of methods like `_.intersection`, without support
             * for iteratee shorthands, that accepts an array of arrays to inspect.
             *
             * @private
             * @param {Array} arrays The arrays to inspect.
             * @param {Function} [iteratee] The iteratee invoked per element.
             * @param {Function} [comparator] The comparator invoked per element.
             * @returns {Array} Returns the new array of shared values.
             */
            function baseIntersection(arrays, iteratee, comparator) {
                var includes = comparator ? arrayIncludesWith : arrayIncludes, length = arrays[0].length, othLength = arrays.length, othIndex = othLength, caches = Array(othLength), maxLength = Infinity, result = [];
                while (othIndex--) {
                    var array = arrays[othIndex];
                    if (othIndex && iteratee) {
                        array = arrayMap(array, baseUnary(iteratee));
                    }
                    maxLength = nativeMin(array.length, maxLength);
                    caches[othIndex] = !comparator && (iteratee || (length >= 120 && array.length >= 120))
                        ? new SetCache(othIndex && array)
                        : undefined;
                }
                array = arrays[0];
                var index = -1, seen = caches[0];
                outer: while (++index < length && result.length < maxLength) {
                    var value = array[index], computed = iteratee ? iteratee(value) : value;
                    value = (comparator || value !== 0) ? value : 0;
                    if (!(seen
                        ? cacheHas(seen, computed)
                        : includes(result, computed, comparator))) {
                        othIndex = othLength;
                        while (--othIndex) {
                            var cache = caches[othIndex];
                            if (!(cache
                                ? cacheHas(cache, computed)
                                : includes(arrays[othIndex], computed, comparator))) {
                                continue outer;
                            }
                        }
                        if (seen) {
                            seen.push(computed);
                        }
                        result.push(value);
                    }
                }
                return result;
            }
            /**
             * The base implementation of `_.invert` and `_.invertBy` which inverts
             * `object` with values transformed by `iteratee` and set by `setter`.
             *
             * @private
             * @param {Object} object The object to iterate over.
             * @param {Function} setter The function to set `accumulator` values.
             * @param {Function} iteratee The iteratee to transform values.
             * @param {Object} accumulator The initial inverted object.
             * @returns {Function} Returns `accumulator`.
             */
            function baseInverter(object, setter, iteratee, accumulator) {
                baseForOwn(object, function (value, key, object) {
                    setter(accumulator, iteratee(value), key, object);
                });
                return accumulator;
            }
            /**
             * The base implementation of `_.invoke` without support for individual
             * method arguments.
             *
             * @private
             * @param {Object} object The object to query.
             * @param {Array|string} path The path of the method to invoke.
             * @param {Array} args The arguments to invoke the method with.
             * @returns {*} Returns the result of the invoked method.
             */
            function baseInvoke(object, path, args) {
                path = castPath(path, object);
                object = parent(object, path);
                var func = object == null ? object : object[toKey(last(path))];
                return func == null ? undefined : apply(func, object, args);
            }
            /**
             * The base implementation of `_.isArguments`.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is an `arguments` object,
             */
            function baseIsArguments(value) {
                return isObjectLike(value) && baseGetTag(value) == argsTag;
            }
            /**
             * The base implementation of `_.isArrayBuffer` without Node.js optimizations.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is an array buffer, else `false`.
             */
            function baseIsArrayBuffer(value) {
                return isObjectLike(value) && baseGetTag(value) == arrayBufferTag;
            }
            /**
             * The base implementation of `_.isDate` without Node.js optimizations.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a date object, else `false`.
             */
            function baseIsDate(value) {
                return isObjectLike(value) && baseGetTag(value) == dateTag;
            }
            /**
             * The base implementation of `_.isEqual` which supports partial comparisons
             * and tracks traversed objects.
             *
             * @private
             * @param {*} value The value to compare.
             * @param {*} other The other value to compare.
             * @param {boolean} bitmask The bitmask flags.
             *  1 - Unordered comparison
             *  2 - Partial comparison
             * @param {Function} [customizer] The function to customize comparisons.
             * @param {Object} [stack] Tracks traversed `value` and `other` objects.
             * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
             */
            function baseIsEqual(value, other, bitmask, customizer, stack) {
                if (value === other) {
                    return true;
                }
                if (value == null || other == null || (!isObjectLike(value) && !isObjectLike(other))) {
                    return value !== value && other !== other;
                }
                return baseIsEqualDeep(value, other, bitmask, customizer, baseIsEqual, stack);
            }
            /**
             * A specialized version of `baseIsEqual` for arrays and objects which performs
             * deep comparisons and tracks traversed objects enabling objects with circular
             * references to be compared.
             *
             * @private
             * @param {Object} object The object to compare.
             * @param {Object} other The other object to compare.
             * @param {number} bitmask The bitmask flags. See `baseIsEqual` for more details.
             * @param {Function} customizer The function to customize comparisons.
             * @param {Function} equalFunc The function to determine equivalents of values.
             * @param {Object} [stack] Tracks traversed `object` and `other` objects.
             * @returns {boolean} Returns `true` if the objects are equivalent, else `false`.
             */
            function baseIsEqualDeep(object, other, bitmask, customizer, equalFunc, stack) {
                var objIsArr = isArray(object), othIsArr = isArray(other), objTag = objIsArr ? arrayTag : getTag(object), othTag = othIsArr ? arrayTag : getTag(other);
                objTag = objTag == argsTag ? objectTag : objTag;
                othTag = othTag == argsTag ? objectTag : othTag;
                var objIsObj = objTag == objectTag, othIsObj = othTag == objectTag, isSameTag = objTag == othTag;
                if (isSameTag && isBuffer(object)) {
                    if (!isBuffer(other)) {
                        return false;
                    }
                    objIsArr = true;
                    objIsObj = false;
                }
                if (isSameTag && !objIsObj) {
                    stack || (stack = new Stack);
                    return (objIsArr || isTypedArray(object))
                        ? equalArrays(object, other, bitmask, customizer, equalFunc, stack)
                        : equalByTag(object, other, objTag, bitmask, customizer, equalFunc, stack);
                }
                if (!(bitmask & COMPARE_PARTIAL_FLAG)) {
                    var objIsWrapped = objIsObj && hasOwnProperty.call(object, '__wrapped__'), othIsWrapped = othIsObj && hasOwnProperty.call(other, '__wrapped__');
                    if (objIsWrapped || othIsWrapped) {
                        var objUnwrapped = objIsWrapped ? object.value() : object, othUnwrapped = othIsWrapped ? other.value() : other;
                        stack || (stack = new Stack);
                        return equalFunc(objUnwrapped, othUnwrapped, bitmask, customizer, stack);
                    }
                }
                if (!isSameTag) {
                    return false;
                }
                stack || (stack = new Stack);
                return equalObjects(object, other, bitmask, customizer, equalFunc, stack);
            }
            /**
             * The base implementation of `_.isMap` without Node.js optimizations.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a map, else `false`.
             */
            function baseIsMap(value) {
                return isObjectLike(value) && getTag(value) == mapTag;
            }
            /**
             * The base implementation of `_.isMatch` without support for iteratee shorthands.
             *
             * @private
             * @param {Object} object The object to inspect.
             * @param {Object} source The object of property values to match.
             * @param {Array} matchData The property names, values, and compare flags to match.
             * @param {Function} [customizer] The function to customize comparisons.
             * @returns {boolean} Returns `true` if `object` is a match, else `false`.
             */
            function baseIsMatch(object, source, matchData, customizer) {
                var index = matchData.length, length = index, noCustomizer = !customizer;
                if (object == null) {
                    return !length;
                }
                object = Object(object);
                while (index--) {
                    var data = matchData[index];
                    if ((noCustomizer && data[2])
                        ? data[1] !== object[data[0]]
                        : !(data[0] in object)) {
                        return false;
                    }
                }
                while (++index < length) {
                    data = matchData[index];
                    var key = data[0], objValue = object[key], srcValue = data[1];
                    if (noCustomizer && data[2]) {
                        if (objValue === undefined && !(key in object)) {
                            return false;
                        }
                    }
                    else {
                        var stack = new Stack;
                        if (customizer) {
                            var result = customizer(objValue, srcValue, key, object, source, stack);
                        }
                        if (!(result === undefined
                            ? baseIsEqual(srcValue, objValue, COMPARE_PARTIAL_FLAG | COMPARE_UNORDERED_FLAG, customizer, stack)
                            : result)) {
                            return false;
                        }
                    }
                }
                return true;
            }
            /**
             * The base implementation of `_.isNative` without bad shim checks.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a native function,
             *  else `false`.
             */
            function baseIsNative(value) {
                if (!isObject(value) || isMasked(value)) {
                    return false;
                }
                var pattern = isFunction(value) ? reIsNative : reIsHostCtor;
                return pattern.test(toSource(value));
            }
            /**
             * The base implementation of `_.isRegExp` without Node.js optimizations.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a regexp, else `false`.
             */
            function baseIsRegExp(value) {
                return isObjectLike(value) && baseGetTag(value) == regexpTag;
            }
            /**
             * The base implementation of `_.isSet` without Node.js optimizations.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a set, else `false`.
             */
            function baseIsSet(value) {
                return isObjectLike(value) && getTag(value) == setTag;
            }
            /**
             * The base implementation of `_.isTypedArray` without Node.js optimizations.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a typed array, else `false`.
             */
            function baseIsTypedArray(value) {
                return isObjectLike(value) &&
                    isLength(value.length) && !!typedArrayTags[baseGetTag(value)];
            }
            /**
             * The base implementation of `_.iteratee`.
             *
             * @private
             * @param {*} [value=_.identity] The value to convert to an iteratee.
             * @returns {Function} Returns the iteratee.
             */
            function baseIteratee(value) {
                // Don't store the `typeof` result in a variable to avoid a JIT bug in Safari 9.
                // See https://bugs.webkit.org/show_bug.cgi?id=156034 for more details.
                if (typeof value == 'function') {
                    return value;
                }
                if (value == null) {
                    return identity;
                }
                if (typeof value == 'object') {
                    return isArray(value)
                        ? baseMatchesProperty(value[0], value[1])
                        : baseMatches(value);
                }
                return property(value);
            }
            /**
             * The base implementation of `_.keys` which doesn't treat sparse arrays as dense.
             *
             * @private
             * @param {Object} object The object to query.
             * @returns {Array} Returns the array of property names.
             */
            function baseKeys(object) {
                if (!isPrototype(object)) {
                    return nativeKeys(object);
                }
                var result = [];
                for (var key in Object(object)) {
                    if (hasOwnProperty.call(object, key) && key != 'constructor') {
                        result.push(key);
                    }
                }
                return result;
            }
            /**
             * The base implementation of `_.keysIn` which doesn't treat sparse arrays as dense.
             *
             * @private
             * @param {Object} object The object to query.
             * @returns {Array} Returns the array of property names.
             */
            function baseKeysIn(object) {
                if (!isObject(object)) {
                    return nativeKeysIn(object);
                }
                var isProto = isPrototype(object), result = [];
                for (var key in object) {
                    if (!(key == 'constructor' && (isProto || !hasOwnProperty.call(object, key)))) {
                        result.push(key);
                    }
                }
                return result;
            }
            /**
             * The base implementation of `_.lt` which doesn't coerce arguments.
             *
             * @private
             * @param {*} value The value to compare.
             * @param {*} other The other value to compare.
             * @returns {boolean} Returns `true` if `value` is less than `other`,
             *  else `false`.
             */
            function baseLt(value, other) {
                return value < other;
            }
            /**
             * The base implementation of `_.map` without support for iteratee shorthands.
             *
             * @private
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} iteratee The function invoked per iteration.
             * @returns {Array} Returns the new mapped array.
             */
            function baseMap(collection, iteratee) {
                var index = -1, result = isArrayLike(collection) ? Array(collection.length) : [];
                baseEach(collection, function (value, key, collection) {
                    result[++index] = iteratee(value, key, collection);
                });
                return result;
            }
            /**
             * The base implementation of `_.matches` which doesn't clone `source`.
             *
             * @private
             * @param {Object} source The object of property values to match.
             * @returns {Function} Returns the new spec function.
             */
            function baseMatches(source) {
                var matchData = getMatchData(source);
                if (matchData.length == 1 && matchData[0][2]) {
                    return matchesStrictComparable(matchData[0][0], matchData[0][1]);
                }
                return function (object) {
                    return object === source || baseIsMatch(object, source, matchData);
                };
            }
            /**
             * The base implementation of `_.matchesProperty` which doesn't clone `srcValue`.
             *
             * @private
             * @param {string} path The path of the property to get.
             * @param {*} srcValue The value to match.
             * @returns {Function} Returns the new spec function.
             */
            function baseMatchesProperty(path, srcValue) {
                if (isKey(path) && isStrictComparable(srcValue)) {
                    return matchesStrictComparable(toKey(path), srcValue);
                }
                return function (object) {
                    var objValue = get(object, path);
                    return (objValue === undefined && objValue === srcValue)
                        ? hasIn(object, path)
                        : baseIsEqual(srcValue, objValue, COMPARE_PARTIAL_FLAG | COMPARE_UNORDERED_FLAG);
                };
            }
            /**
             * The base implementation of `_.merge` without support for multiple sources.
             *
             * @private
             * @param {Object} object The destination object.
             * @param {Object} source The source object.
             * @param {number} srcIndex The index of `source`.
             * @param {Function} [customizer] The function to customize merged values.
             * @param {Object} [stack] Tracks traversed source values and their merged
             *  counterparts.
             */
            function baseMerge(object, source, srcIndex, customizer, stack) {
                if (object === source) {
                    return;
                }
                baseFor(source, function (srcValue, key) {
                    stack || (stack = new Stack);
                    if (isObject(srcValue)) {
                        baseMergeDeep(object, source, key, srcIndex, baseMerge, customizer, stack);
                    }
                    else {
                        var newValue = customizer
                            ? customizer(safeGet(object, key), srcValue, (key + ''), object, source, stack)
                            : undefined;
                        if (newValue === undefined) {
                            newValue = srcValue;
                        }
                        assignMergeValue(object, key, newValue);
                    }
                }, keysIn);
            }
            /**
             * A specialized version of `baseMerge` for arrays and objects which performs
             * deep merges and tracks traversed objects enabling objects with circular
             * references to be merged.
             *
             * @private
             * @param {Object} object The destination object.
             * @param {Object} source The source object.
             * @param {string} key The key of the value to merge.
             * @param {number} srcIndex The index of `source`.
             * @param {Function} mergeFunc The function to merge values.
             * @param {Function} [customizer] The function to customize assigned values.
             * @param {Object} [stack] Tracks traversed source values and their merged
             *  counterparts.
             */
            function baseMergeDeep(object, source, key, srcIndex, mergeFunc, customizer, stack) {
                var objValue = safeGet(object, key), srcValue = safeGet(source, key), stacked = stack.get(srcValue);
                if (stacked) {
                    assignMergeValue(object, key, stacked);
                    return;
                }
                var newValue = customizer
                    ? customizer(objValue, srcValue, (key + ''), object, source, stack)
                    : undefined;
                var isCommon = newValue === undefined;
                if (isCommon) {
                    var isArr = isArray(srcValue), isBuff = !isArr && isBuffer(srcValue), isTyped = !isArr && !isBuff && isTypedArray(srcValue);
                    newValue = srcValue;
                    if (isArr || isBuff || isTyped) {
                        if (isArray(objValue)) {
                            newValue = objValue;
                        }
                        else if (isArrayLikeObject(objValue)) {
                            newValue = copyArray(objValue);
                        }
                        else if (isBuff) {
                            isCommon = false;
                            newValue = cloneBuffer(srcValue, true);
                        }
                        else if (isTyped) {
                            isCommon = false;
                            newValue = cloneTypedArray(srcValue, true);
                        }
                        else {
                            newValue = [];
                        }
                    }
                    else if (isPlainObject(srcValue) || isArguments(srcValue)) {
                        newValue = objValue;
                        if (isArguments(objValue)) {
                            newValue = toPlainObject(objValue);
                        }
                        else if (!isObject(objValue) || isFunction(objValue)) {
                            newValue = initCloneObject(srcValue);
                        }
                    }
                    else {
                        isCommon = false;
                    }
                }
                if (isCommon) {
                    // Recursively merge objects and arrays (susceptible to call stack limits).
                    stack.set(srcValue, newValue);
                    mergeFunc(newValue, srcValue, srcIndex, customizer, stack);
                    stack['delete'](srcValue);
                }
                assignMergeValue(object, key, newValue);
            }
            /**
             * The base implementation of `_.nth` which doesn't coerce arguments.
             *
             * @private
             * @param {Array} array The array to query.
             * @param {number} n The index of the element to return.
             * @returns {*} Returns the nth element of `array`.
             */
            function baseNth(array, n) {
                var length = array.length;
                if (!length) {
                    return;
                }
                n += n < 0 ? length : 0;
                return isIndex(n, length) ? array[n] : undefined;
            }
            /**
             * The base implementation of `_.orderBy` without param guards.
             *
             * @private
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function[]|Object[]|string[]} iteratees The iteratees to sort by.
             * @param {string[]} orders The sort orders of `iteratees`.
             * @returns {Array} Returns the new sorted array.
             */
            function baseOrderBy(collection, iteratees, orders) {
                var index = -1;
                iteratees = arrayMap(iteratees.length ? iteratees : [identity], baseUnary(getIteratee()));
                var result = baseMap(collection, function (value, key, collection) {
                    var criteria = arrayMap(iteratees, function (iteratee) {
                        return iteratee(value);
                    });
                    return { 'criteria': criteria, 'index': ++index, 'value': value };
                });
                return baseSortBy(result, function (object, other) {
                    return compareMultiple(object, other, orders);
                });
            }
            /**
             * The base implementation of `_.pick` without support for individual
             * property identifiers.
             *
             * @private
             * @param {Object} object The source object.
             * @param {string[]} paths The property paths to pick.
             * @returns {Object} Returns the new object.
             */
            function basePick(object, paths) {
                return basePickBy(object, paths, function (value, path) {
                    return hasIn(object, path);
                });
            }
            /**
             * The base implementation of  `_.pickBy` without support for iteratee shorthands.
             *
             * @private
             * @param {Object} object The source object.
             * @param {string[]} paths The property paths to pick.
             * @param {Function} predicate The function invoked per property.
             * @returns {Object} Returns the new object.
             */
            function basePickBy(object, paths, predicate) {
                var index = -1, length = paths.length, result = {};
                while (++index < length) {
                    var path = paths[index], value = baseGet(object, path);
                    if (predicate(value, path)) {
                        baseSet(result, castPath(path, object), value);
                    }
                }
                return result;
            }
            /**
             * A specialized version of `baseProperty` which supports deep paths.
             *
             * @private
             * @param {Array|string} path The path of the property to get.
             * @returns {Function} Returns the new accessor function.
             */
            function basePropertyDeep(path) {
                return function (object) {
                    return baseGet(object, path);
                };
            }
            /**
             * The base implementation of `_.pullAllBy` without support for iteratee
             * shorthands.
             *
             * @private
             * @param {Array} array The array to modify.
             * @param {Array} values The values to remove.
             * @param {Function} [iteratee] The iteratee invoked per element.
             * @param {Function} [comparator] The comparator invoked per element.
             * @returns {Array} Returns `array`.
             */
            function basePullAll(array, values, iteratee, comparator) {
                var indexOf = comparator ? baseIndexOfWith : baseIndexOf, index = -1, length = values.length, seen = array;
                if (array === values) {
                    values = copyArray(values);
                }
                if (iteratee) {
                    seen = arrayMap(array, baseUnary(iteratee));
                }
                while (++index < length) {
                    var fromIndex = 0, value = values[index], computed = iteratee ? iteratee(value) : value;
                    while ((fromIndex = indexOf(seen, computed, fromIndex, comparator)) > -1) {
                        if (seen !== array) {
                            splice.call(seen, fromIndex, 1);
                        }
                        splice.call(array, fromIndex, 1);
                    }
                }
                return array;
            }
            /**
             * The base implementation of `_.pullAt` without support for individual
             * indexes or capturing the removed elements.
             *
             * @private
             * @param {Array} array The array to modify.
             * @param {number[]} indexes The indexes of elements to remove.
             * @returns {Array} Returns `array`.
             */
            function basePullAt(array, indexes) {
                var length = array ? indexes.length : 0, lastIndex = length - 1;
                while (length--) {
                    var index = indexes[length];
                    if (length == lastIndex || index !== previous) {
                        var previous = index;
                        if (isIndex(index)) {
                            splice.call(array, index, 1);
                        }
                        else {
                            baseUnset(array, index);
                        }
                    }
                }
                return array;
            }
            /**
             * The base implementation of `_.random` without support for returning
             * floating-point numbers.
             *
             * @private
             * @param {number} lower The lower bound.
             * @param {number} upper The upper bound.
             * @returns {number} Returns the random number.
             */
            function baseRandom(lower, upper) {
                return lower + nativeFloor(nativeRandom() * (upper - lower + 1));
            }
            /**
             * The base implementation of `_.range` and `_.rangeRight` which doesn't
             * coerce arguments.
             *
             * @private
             * @param {number} start The start of the range.
             * @param {number} end The end of the range.
             * @param {number} step The value to increment or decrement by.
             * @param {boolean} [fromRight] Specify iterating from right to left.
             * @returns {Array} Returns the range of numbers.
             */
            function baseRange(start, end, step, fromRight) {
                var index = -1, length = nativeMax(nativeCeil((end - start) / (step || 1)), 0), result = Array(length);
                while (length--) {
                    result[fromRight ? length : ++index] = start;
                    start += step;
                }
                return result;
            }
            /**
             * The base implementation of `_.repeat` which doesn't coerce arguments.
             *
             * @private
             * @param {string} string The string to repeat.
             * @param {number} n The number of times to repeat the string.
             * @returns {string} Returns the repeated string.
             */
            function baseRepeat(string, n) {
                var result = '';
                if (!string || n < 1 || n > MAX_SAFE_INTEGER) {
                    return result;
                }
                // Leverage the exponentiation by squaring algorithm for a faster repeat.
                // See https://en.wikipedia.org/wiki/Exponentiation_by_squaring for more details.
                do {
                    if (n % 2) {
                        result += string;
                    }
                    n = nativeFloor(n / 2);
                    if (n) {
                        string += string;
                    }
                } while (n);
                return result;
            }
            /**
             * The base implementation of `_.rest` which doesn't validate or coerce arguments.
             *
             * @private
             * @param {Function} func The function to apply a rest parameter to.
             * @param {number} [start=func.length-1] The start position of the rest parameter.
             * @returns {Function} Returns the new function.
             */
            function baseRest(func, start) {
                return setToString(overRest(func, start, identity), func + '');
            }
            /**
             * The base implementation of `_.sample`.
             *
             * @private
             * @param {Array|Object} collection The collection to sample.
             * @returns {*} Returns the random element.
             */
            function baseSample(collection) {
                return arraySample(values(collection));
            }
            /**
             * The base implementation of `_.sampleSize` without param guards.
             *
             * @private
             * @param {Array|Object} collection The collection to sample.
             * @param {number} n The number of elements to sample.
             * @returns {Array} Returns the random elements.
             */
            function baseSampleSize(collection, n) {
                var array = values(collection);
                return shuffleSelf(array, baseClamp(n, 0, array.length));
            }
            /**
             * The base implementation of `_.set`.
             *
             * @private
             * @param {Object} object The object to modify.
             * @param {Array|string} path The path of the property to set.
             * @param {*} value The value to set.
             * @param {Function} [customizer] The function to customize path creation.
             * @returns {Object} Returns `object`.
             */
            function baseSet(object, path, value, customizer) {
                if (!isObject(object)) {
                    return object;
                }
                path = castPath(path, object);
                var index = -1, length = path.length, lastIndex = length - 1, nested = object;
                while (nested != null && ++index < length) {
                    var key = toKey(path[index]), newValue = value;
                    if (index != lastIndex) {
                        var objValue = nested[key];
                        newValue = customizer ? customizer(objValue, key, nested) : undefined;
                        if (newValue === undefined) {
                            newValue = isObject(objValue)
                                ? objValue
                                : (isIndex(path[index + 1]) ? [] : {});
                        }
                    }
                    assignValue(nested, key, newValue);
                    nested = nested[key];
                }
                return object;
            }
            /**
             * The base implementation of `setData` without support for hot loop shorting.
             *
             * @private
             * @param {Function} func The function to associate metadata with.
             * @param {*} data The metadata.
             * @returns {Function} Returns `func`.
             */
            var baseSetData = !metaMap ? identity : function (func, data) {
                metaMap.set(func, data);
                return func;
            };
            /**
             * The base implementation of `setToString` without support for hot loop shorting.
             *
             * @private
             * @param {Function} func The function to modify.
             * @param {Function} string The `toString` result.
             * @returns {Function} Returns `func`.
             */
            var baseSetToString = !defineProperty ? identity : function (func, string) {
                return defineProperty(func, 'toString', {
                    'configurable': true,
                    'enumerable': false,
                    'value': constant(string),
                    'writable': true
                });
            };
            /**
             * The base implementation of `_.shuffle`.
             *
             * @private
             * @param {Array|Object} collection The collection to shuffle.
             * @returns {Array} Returns the new shuffled array.
             */
            function baseShuffle(collection) {
                return shuffleSelf(values(collection));
            }
            /**
             * The base implementation of `_.slice` without an iteratee call guard.
             *
             * @private
             * @param {Array} array The array to slice.
             * @param {number} [start=0] The start position.
             * @param {number} [end=array.length] The end position.
             * @returns {Array} Returns the slice of `array`.
             */
            function baseSlice(array, start, end) {
                var index = -1, length = array.length;
                if (start < 0) {
                    start = -start > length ? 0 : (length + start);
                }
                end = end > length ? length : end;
                if (end < 0) {
                    end += length;
                }
                length = start > end ? 0 : ((end - start) >>> 0);
                start >>>= 0;
                var result = Array(length);
                while (++index < length) {
                    result[index] = array[index + start];
                }
                return result;
            }
            /**
             * The base implementation of `_.some` without support for iteratee shorthands.
             *
             * @private
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} predicate The function invoked per iteration.
             * @returns {boolean} Returns `true` if any element passes the predicate check,
             *  else `false`.
             */
            function baseSome(collection, predicate) {
                var result;
                baseEach(collection, function (value, index, collection) {
                    result = predicate(value, index, collection);
                    return !result;
                });
                return !!result;
            }
            /**
             * The base implementation of `_.sortedIndex` and `_.sortedLastIndex` which
             * performs a binary search of `array` to determine the index at which `value`
             * should be inserted into `array` in order to maintain its sort order.
             *
             * @private
             * @param {Array} array The sorted array to inspect.
             * @param {*} value The value to evaluate.
             * @param {boolean} [retHighest] Specify returning the highest qualified index.
             * @returns {number} Returns the index at which `value` should be inserted
             *  into `array`.
             */
            function baseSortedIndex(array, value, retHighest) {
                var low = 0, high = array == null ? low : array.length;
                if (typeof value == 'number' && value === value && high <= HALF_MAX_ARRAY_LENGTH) {
                    while (low < high) {
                        var mid = (low + high) >>> 1, computed = array[mid];
                        if (computed !== null && !isSymbol(computed) &&
                            (retHighest ? (computed <= value) : (computed < value))) {
                            low = mid + 1;
                        }
                        else {
                            high = mid;
                        }
                    }
                    return high;
                }
                return baseSortedIndexBy(array, value, identity, retHighest);
            }
            /**
             * The base implementation of `_.sortedIndexBy` and `_.sortedLastIndexBy`
             * which invokes `iteratee` for `value` and each element of `array` to compute
             * their sort ranking. The iteratee is invoked with one argument; (value).
             *
             * @private
             * @param {Array} array The sorted array to inspect.
             * @param {*} value The value to evaluate.
             * @param {Function} iteratee The iteratee invoked per element.
             * @param {boolean} [retHighest] Specify returning the highest qualified index.
             * @returns {number} Returns the index at which `value` should be inserted
             *  into `array`.
             */
            function baseSortedIndexBy(array, value, iteratee, retHighest) {
                value = iteratee(value);
                var low = 0, high = array == null ? 0 : array.length, valIsNaN = value !== value, valIsNull = value === null, valIsSymbol = isSymbol(value), valIsUndefined = value === undefined;
                while (low < high) {
                    var mid = nativeFloor((low + high) / 2), computed = iteratee(array[mid]), othIsDefined = computed !== undefined, othIsNull = computed === null, othIsReflexive = computed === computed, othIsSymbol = isSymbol(computed);
                    if (valIsNaN) {
                        var setLow = retHighest || othIsReflexive;
                    }
                    else if (valIsUndefined) {
                        setLow = othIsReflexive && (retHighest || othIsDefined);
                    }
                    else if (valIsNull) {
                        setLow = othIsReflexive && othIsDefined && (retHighest || !othIsNull);
                    }
                    else if (valIsSymbol) {
                        setLow = othIsReflexive && othIsDefined && !othIsNull && (retHighest || !othIsSymbol);
                    }
                    else if (othIsNull || othIsSymbol) {
                        setLow = false;
                    }
                    else {
                        setLow = retHighest ? (computed <= value) : (computed < value);
                    }
                    if (setLow) {
                        low = mid + 1;
                    }
                    else {
                        high = mid;
                    }
                }
                return nativeMin(high, MAX_ARRAY_INDEX);
            }
            /**
             * The base implementation of `_.sortedUniq` and `_.sortedUniqBy` without
             * support for iteratee shorthands.
             *
             * @private
             * @param {Array} array The array to inspect.
             * @param {Function} [iteratee] The iteratee invoked per element.
             * @returns {Array} Returns the new duplicate free array.
             */
            function baseSortedUniq(array, iteratee) {
                var index = -1, length = array.length, resIndex = 0, result = [];
                while (++index < length) {
                    var value = array[index], computed = iteratee ? iteratee(value) : value;
                    if (!index || !eq(computed, seen)) {
                        var seen = computed;
                        result[resIndex++] = value === 0 ? 0 : value;
                    }
                }
                return result;
            }
            /**
             * The base implementation of `_.toNumber` which doesn't ensure correct
             * conversions of binary, hexadecimal, or octal string values.
             *
             * @private
             * @param {*} value The value to process.
             * @returns {number} Returns the number.
             */
            function baseToNumber(value) {
                if (typeof value == 'number') {
                    return value;
                }
                if (isSymbol(value)) {
                    return NAN;
                }
                return +value;
            }
            /**
             * The base implementation of `_.toString` which doesn't convert nullish
             * values to empty strings.
             *
             * @private
             * @param {*} value The value to process.
             * @returns {string} Returns the string.
             */
            function baseToString(value) {
                // Exit early for strings to avoid a performance hit in some environments.
                if (typeof value == 'string') {
                    return value;
                }
                if (isArray(value)) {
                    // Recursively convert values (susceptible to call stack limits).
                    return arrayMap(value, baseToString) + '';
                }
                if (isSymbol(value)) {
                    return symbolToString ? symbolToString.call(value) : '';
                }
                var result = (value + '');
                return (result == '0' && (1 / value) == -INFINITY) ? '-0' : result;
            }
            /**
             * The base implementation of `_.uniqBy` without support for iteratee shorthands.
             *
             * @private
             * @param {Array} array The array to inspect.
             * @param {Function} [iteratee] The iteratee invoked per element.
             * @param {Function} [comparator] The comparator invoked per element.
             * @returns {Array} Returns the new duplicate free array.
             */
            function baseUniq(array, iteratee, comparator) {
                var index = -1, includes = arrayIncludes, length = array.length, isCommon = true, result = [], seen = result;
                if (comparator) {
                    isCommon = false;
                    includes = arrayIncludesWith;
                }
                else if (length >= LARGE_ARRAY_SIZE) {
                    var set = iteratee ? null : createSet(array);
                    if (set) {
                        return setToArray(set);
                    }
                    isCommon = false;
                    includes = cacheHas;
                    seen = new SetCache;
                }
                else {
                    seen = iteratee ? [] : result;
                }
                outer: while (++index < length) {
                    var value = array[index], computed = iteratee ? iteratee(value) : value;
                    value = (comparator || value !== 0) ? value : 0;
                    if (isCommon && computed === computed) {
                        var seenIndex = seen.length;
                        while (seenIndex--) {
                            if (seen[seenIndex] === computed) {
                                continue outer;
                            }
                        }
                        if (iteratee) {
                            seen.push(computed);
                        }
                        result.push(value);
                    }
                    else if (!includes(seen, computed, comparator)) {
                        if (seen !== result) {
                            seen.push(computed);
                        }
                        result.push(value);
                    }
                }
                return result;
            }
            /**
             * The base implementation of `_.unset`.
             *
             * @private
             * @param {Object} object The object to modify.
             * @param {Array|string} path The property path to unset.
             * @returns {boolean} Returns `true` if the property is deleted, else `false`.
             */
            function baseUnset(object, path) {
                path = castPath(path, object);
                object = parent(object, path);
                return object == null || delete object[toKey(last(path))];
            }
            /**
             * The base implementation of `_.update`.
             *
             * @private
             * @param {Object} object The object to modify.
             * @param {Array|string} path The path of the property to update.
             * @param {Function} updater The function to produce the updated value.
             * @param {Function} [customizer] The function to customize path creation.
             * @returns {Object} Returns `object`.
             */
            function baseUpdate(object, path, updater, customizer) {
                return baseSet(object, path, updater(baseGet(object, path)), customizer);
            }
            /**
             * The base implementation of methods like `_.dropWhile` and `_.takeWhile`
             * without support for iteratee shorthands.
             *
             * @private
             * @param {Array} array The array to query.
             * @param {Function} predicate The function invoked per iteration.
             * @param {boolean} [isDrop] Specify dropping elements instead of taking them.
             * @param {boolean} [fromRight] Specify iterating from right to left.
             * @returns {Array} Returns the slice of `array`.
             */
            function baseWhile(array, predicate, isDrop, fromRight) {
                var length = array.length, index = fromRight ? length : -1;
                while ((fromRight ? index-- : ++index < length) &&
                    predicate(array[index], index, array)) { }
                return isDrop
                    ? baseSlice(array, (fromRight ? 0 : index), (fromRight ? index + 1 : length))
                    : baseSlice(array, (fromRight ? index + 1 : 0), (fromRight ? length : index));
            }
            /**
             * The base implementation of `wrapperValue` which returns the result of
             * performing a sequence of actions on the unwrapped `value`, where each
             * successive action is supplied the return value of the previous.
             *
             * @private
             * @param {*} value The unwrapped value.
             * @param {Array} actions Actions to perform to resolve the unwrapped value.
             * @returns {*} Returns the resolved value.
             */
            function baseWrapperValue(value, actions) {
                var result = value;
                if (result instanceof LazyWrapper) {
                    result = result.value();
                }
                return arrayReduce(actions, function (result, action) {
                    return action.func.apply(action.thisArg, arrayPush([result], action.args));
                }, result);
            }
            /**
             * The base implementation of methods like `_.xor`, without support for
             * iteratee shorthands, that accepts an array of arrays to inspect.
             *
             * @private
             * @param {Array} arrays The arrays to inspect.
             * @param {Function} [iteratee] The iteratee invoked per element.
             * @param {Function} [comparator] The comparator invoked per element.
             * @returns {Array} Returns the new array of values.
             */
            function baseXor(arrays, iteratee, comparator) {
                var length = arrays.length;
                if (length < 2) {
                    return length ? baseUniq(arrays[0]) : [];
                }
                var index = -1, result = Array(length);
                while (++index < length) {
                    var array = arrays[index], othIndex = -1;
                    while (++othIndex < length) {
                        if (othIndex != index) {
                            result[index] = baseDifference(result[index] || array, arrays[othIndex], iteratee, comparator);
                        }
                    }
                }
                return baseUniq(baseFlatten(result, 1), iteratee, comparator);
            }
            /**
             * This base implementation of `_.zipObject` which assigns values using `assignFunc`.
             *
             * @private
             * @param {Array} props The property identifiers.
             * @param {Array} values The property values.
             * @param {Function} assignFunc The function to assign values.
             * @returns {Object} Returns the new object.
             */
            function baseZipObject(props, values, assignFunc) {
                var index = -1, length = props.length, valsLength = values.length, result = {};
                while (++index < length) {
                    var value = index < valsLength ? values[index] : undefined;
                    assignFunc(result, props[index], value);
                }
                return result;
            }
            /**
             * Casts `value` to an empty array if it's not an array like object.
             *
             * @private
             * @param {*} value The value to inspect.
             * @returns {Array|Object} Returns the cast array-like object.
             */
            function castArrayLikeObject(value) {
                return isArrayLikeObject(value) ? value : [];
            }
            /**
             * Casts `value` to `identity` if it's not a function.
             *
             * @private
             * @param {*} value The value to inspect.
             * @returns {Function} Returns cast function.
             */
            function castFunction(value) {
                return typeof value == 'function' ? value : identity;
            }
            /**
             * Casts `value` to a path array if it's not one.
             *
             * @private
             * @param {*} value The value to inspect.
             * @param {Object} [object] The object to query keys on.
             * @returns {Array} Returns the cast property path array.
             */
            function castPath(value, object) {
                if (isArray(value)) {
                    return value;
                }
                return isKey(value, object) ? [value] : stringToPath(toString(value));
            }
            /**
             * A `baseRest` alias which can be replaced with `identity` by module
             * replacement plugins.
             *
             * @private
             * @type {Function}
             * @param {Function} func The function to apply a rest parameter to.
             * @returns {Function} Returns the new function.
             */
            var castRest = baseRest;
            /**
             * Casts `array` to a slice if it's needed.
             *
             * @private
             * @param {Array} array The array to inspect.
             * @param {number} start The start position.
             * @param {number} [end=array.length] The end position.
             * @returns {Array} Returns the cast slice.
             */
            function castSlice(array, start, end) {
                var length = array.length;
                end = end === undefined ? length : end;
                return (!start && end >= length) ? array : baseSlice(array, start, end);
            }
            /**
             * A simple wrapper around the global [`clearTimeout`](https://mdn.io/clearTimeout).
             *
             * @private
             * @param {number|Object} id The timer id or timeout object of the timer to clear.
             */
            var clearTimeout = ctxClearTimeout || function (id) {
                return root.clearTimeout(id);
            };
            /**
             * Creates a clone of  `buffer`.
             *
             * @private
             * @param {Buffer} buffer The buffer to clone.
             * @param {boolean} [isDeep] Specify a deep clone.
             * @returns {Buffer} Returns the cloned buffer.
             */
            function cloneBuffer(buffer, isDeep) {
                if (isDeep) {
                    return buffer.slice();
                }
                var length = buffer.length, result = allocUnsafe ? allocUnsafe(length) : new buffer.constructor(length);
                buffer.copy(result);
                return result;
            }
            /**
             * Creates a clone of `arrayBuffer`.
             *
             * @private
             * @param {ArrayBuffer} arrayBuffer The array buffer to clone.
             * @returns {ArrayBuffer} Returns the cloned array buffer.
             */
            function cloneArrayBuffer(arrayBuffer) {
                var result = new arrayBuffer.constructor(arrayBuffer.byteLength);
                new Uint8Array(result).set(new Uint8Array(arrayBuffer));
                return result;
            }
            /**
             * Creates a clone of `dataView`.
             *
             * @private
             * @param {Object} dataView The data view to clone.
             * @param {boolean} [isDeep] Specify a deep clone.
             * @returns {Object} Returns the cloned data view.
             */
            function cloneDataView(dataView, isDeep) {
                var buffer = isDeep ? cloneArrayBuffer(dataView.buffer) : dataView.buffer;
                return new dataView.constructor(buffer, dataView.byteOffset, dataView.byteLength);
            }
            /**
             * Creates a clone of `regexp`.
             *
             * @private
             * @param {Object} regexp The regexp to clone.
             * @returns {Object} Returns the cloned regexp.
             */
            function cloneRegExp(regexp) {
                var result = new regexp.constructor(regexp.source, reFlags.exec(regexp));
                result.lastIndex = regexp.lastIndex;
                return result;
            }
            /**
             * Creates a clone of the `symbol` object.
             *
             * @private
             * @param {Object} symbol The symbol object to clone.
             * @returns {Object} Returns the cloned symbol object.
             */
            function cloneSymbol(symbol) {
                return symbolValueOf ? Object(symbolValueOf.call(symbol)) : {};
            }
            /**
             * Creates a clone of `typedArray`.
             *
             * @private
             * @param {Object} typedArray The typed array to clone.
             * @param {boolean} [isDeep] Specify a deep clone.
             * @returns {Object} Returns the cloned typed array.
             */
            function cloneTypedArray(typedArray, isDeep) {
                var buffer = isDeep ? cloneArrayBuffer(typedArray.buffer) : typedArray.buffer;
                return new typedArray.constructor(buffer, typedArray.byteOffset, typedArray.length);
            }
            /**
             * Compares values to sort them in ascending order.
             *
             * @private
             * @param {*} value The value to compare.
             * @param {*} other The other value to compare.
             * @returns {number} Returns the sort order indicator for `value`.
             */
            function compareAscending(value, other) {
                if (value !== other) {
                    var valIsDefined = value !== undefined, valIsNull = value === null, valIsReflexive = value === value, valIsSymbol = isSymbol(value);
                    var othIsDefined = other !== undefined, othIsNull = other === null, othIsReflexive = other === other, othIsSymbol = isSymbol(other);
                    if ((!othIsNull && !othIsSymbol && !valIsSymbol && value > other) ||
                        (valIsSymbol && othIsDefined && othIsReflexive && !othIsNull && !othIsSymbol) ||
                        (valIsNull && othIsDefined && othIsReflexive) ||
                        (!valIsDefined && othIsReflexive) ||
                        !valIsReflexive) {
                        return 1;
                    }
                    if ((!valIsNull && !valIsSymbol && !othIsSymbol && value < other) ||
                        (othIsSymbol && valIsDefined && valIsReflexive && !valIsNull && !valIsSymbol) ||
                        (othIsNull && valIsDefined && valIsReflexive) ||
                        (!othIsDefined && valIsReflexive) ||
                        !othIsReflexive) {
                        return -1;
                    }
                }
                return 0;
            }
            /**
             * Used by `_.orderBy` to compare multiple properties of a value to another
             * and stable sort them.
             *
             * If `orders` is unspecified, all values are sorted in ascending order. Otherwise,
             * specify an order of "desc" for descending or "asc" for ascending sort order
             * of corresponding values.
             *
             * @private
             * @param {Object} object The object to compare.
             * @param {Object} other The other object to compare.
             * @param {boolean[]|string[]} orders The order to sort by for each property.
             * @returns {number} Returns the sort order indicator for `object`.
             */
            function compareMultiple(object, other, orders) {
                var index = -1, objCriteria = object.criteria, othCriteria = other.criteria, length = objCriteria.length, ordersLength = orders.length;
                while (++index < length) {
                    var result = compareAscending(objCriteria[index], othCriteria[index]);
                    if (result) {
                        if (index >= ordersLength) {
                            return result;
                        }
                        var order = orders[index];
                        return result * (order == 'desc' ? -1 : 1);
                    }
                }
                // Fixes an `Array#sort` bug in the JS engine embedded in Adobe applications
                // that causes it, under certain circumstances, to provide the same value for
                // `object` and `other`. See https://github.com/jashkenas/underscore/pull/1247
                // for more details.
                //
                // This also ensures a stable sort in V8 and other engines.
                // See https://bugs.chromium.org/p/v8/issues/detail?id=90 for more details.
                return object.index - other.index;
            }
            /**
             * Creates an array that is the composition of partially applied arguments,
             * placeholders, and provided arguments into a single array of arguments.
             *
             * @private
             * @param {Array} args The provided arguments.
             * @param {Array} partials The arguments to prepend to those provided.
             * @param {Array} holders The `partials` placeholder indexes.
             * @params {boolean} [isCurried] Specify composing for a curried function.
             * @returns {Array} Returns the new array of composed arguments.
             */
            function composeArgs(args, partials, holders, isCurried) {
                var argsIndex = -1, argsLength = args.length, holdersLength = holders.length, leftIndex = -1, leftLength = partials.length, rangeLength = nativeMax(argsLength - holdersLength, 0), result = Array(leftLength + rangeLength), isUncurried = !isCurried;
                while (++leftIndex < leftLength) {
                    result[leftIndex] = partials[leftIndex];
                }
                while (++argsIndex < holdersLength) {
                    if (isUncurried || argsIndex < argsLength) {
                        result[holders[argsIndex]] = args[argsIndex];
                    }
                }
                while (rangeLength--) {
                    result[leftIndex++] = args[argsIndex++];
                }
                return result;
            }
            /**
             * This function is like `composeArgs` except that the arguments composition
             * is tailored for `_.partialRight`.
             *
             * @private
             * @param {Array} args The provided arguments.
             * @param {Array} partials The arguments to append to those provided.
             * @param {Array} holders The `partials` placeholder indexes.
             * @params {boolean} [isCurried] Specify composing for a curried function.
             * @returns {Array} Returns the new array of composed arguments.
             */
            function composeArgsRight(args, partials, holders, isCurried) {
                var argsIndex = -1, argsLength = args.length, holdersIndex = -1, holdersLength = holders.length, rightIndex = -1, rightLength = partials.length, rangeLength = nativeMax(argsLength - holdersLength, 0), result = Array(rangeLength + rightLength), isUncurried = !isCurried;
                while (++argsIndex < rangeLength) {
                    result[argsIndex] = args[argsIndex];
                }
                var offset = argsIndex;
                while (++rightIndex < rightLength) {
                    result[offset + rightIndex] = partials[rightIndex];
                }
                while (++holdersIndex < holdersLength) {
                    if (isUncurried || argsIndex < argsLength) {
                        result[offset + holders[holdersIndex]] = args[argsIndex++];
                    }
                }
                return result;
            }
            /**
             * Copies the values of `source` to `array`.
             *
             * @private
             * @param {Array} source The array to copy values from.
             * @param {Array} [array=[]] The array to copy values to.
             * @returns {Array} Returns `array`.
             */
            function copyArray(source, array) {
                var index = -1, length = source.length;
                array || (array = Array(length));
                while (++index < length) {
                    array[index] = source[index];
                }
                return array;
            }
            /**
             * Copies properties of `source` to `object`.
             *
             * @private
             * @param {Object} source The object to copy properties from.
             * @param {Array} props The property identifiers to copy.
             * @param {Object} [object={}] The object to copy properties to.
             * @param {Function} [customizer] The function to customize copied values.
             * @returns {Object} Returns `object`.
             */
            function copyObject(source, props, object, customizer) {
                var isNew = !object;
                object || (object = {});
                var index = -1, length = props.length;
                while (++index < length) {
                    var key = props[index];
                    var newValue = customizer
                        ? customizer(object[key], source[key], key, object, source)
                        : undefined;
                    if (newValue === undefined) {
                        newValue = source[key];
                    }
                    if (isNew) {
                        baseAssignValue(object, key, newValue);
                    }
                    else {
                        assignValue(object, key, newValue);
                    }
                }
                return object;
            }
            /**
             * Copies own symbols of `source` to `object`.
             *
             * @private
             * @param {Object} source The object to copy symbols from.
             * @param {Object} [object={}] The object to copy symbols to.
             * @returns {Object} Returns `object`.
             */
            function copySymbols(source, object) {
                return copyObject(source, getSymbols(source), object);
            }
            /**
             * Copies own and inherited symbols of `source` to `object`.
             *
             * @private
             * @param {Object} source The object to copy symbols from.
             * @param {Object} [object={}] The object to copy symbols to.
             * @returns {Object} Returns `object`.
             */
            function copySymbolsIn(source, object) {
                return copyObject(source, getSymbolsIn(source), object);
            }
            /**
             * Creates a function like `_.groupBy`.
             *
             * @private
             * @param {Function} setter The function to set accumulator values.
             * @param {Function} [initializer] The accumulator object initializer.
             * @returns {Function} Returns the new aggregator function.
             */
            function createAggregator(setter, initializer) {
                return function (collection, iteratee) {
                    var func = isArray(collection) ? arrayAggregator : baseAggregator, accumulator = initializer ? initializer() : {};
                    return func(collection, setter, getIteratee(iteratee, 2), accumulator);
                };
            }
            /**
             * Creates a function like `_.assign`.
             *
             * @private
             * @param {Function} assigner The function to assign values.
             * @returns {Function} Returns the new assigner function.
             */
            function createAssigner(assigner) {
                return baseRest(function (object, sources) {
                    var index = -1, length = sources.length, customizer = length > 1 ? sources[length - 1] : undefined, guard = length > 2 ? sources[2] : undefined;
                    customizer = (assigner.length > 3 && typeof customizer == 'function')
                        ? (length--, customizer)
                        : undefined;
                    if (guard && isIterateeCall(sources[0], sources[1], guard)) {
                        customizer = length < 3 ? undefined : customizer;
                        length = 1;
                    }
                    object = Object(object);
                    while (++index < length) {
                        var source = sources[index];
                        if (source) {
                            assigner(object, source, index, customizer);
                        }
                    }
                    return object;
                });
            }
            /**
             * Creates a `baseEach` or `baseEachRight` function.
             *
             * @private
             * @param {Function} eachFunc The function to iterate over a collection.
             * @param {boolean} [fromRight] Specify iterating from right to left.
             * @returns {Function} Returns the new base function.
             */
            function createBaseEach(eachFunc, fromRight) {
                return function (collection, iteratee) {
                    if (collection == null) {
                        return collection;
                    }
                    if (!isArrayLike(collection)) {
                        return eachFunc(collection, iteratee);
                    }
                    var length = collection.length, index = fromRight ? length : -1, iterable = Object(collection);
                    while ((fromRight ? index-- : ++index < length)) {
                        if (iteratee(iterable[index], index, iterable) === false) {
                            break;
                        }
                    }
                    return collection;
                };
            }
            /**
             * Creates a base function for methods like `_.forIn` and `_.forOwn`.
             *
             * @private
             * @param {boolean} [fromRight] Specify iterating from right to left.
             * @returns {Function} Returns the new base function.
             */
            function createBaseFor(fromRight) {
                return function (object, iteratee, keysFunc) {
                    var index = -1, iterable = Object(object), props = keysFunc(object), length = props.length;
                    while (length--) {
                        var key = props[fromRight ? length : ++index];
                        if (iteratee(iterable[key], key, iterable) === false) {
                            break;
                        }
                    }
                    return object;
                };
            }
            /**
             * Creates a function that wraps `func` to invoke it with the optional `this`
             * binding of `thisArg`.
             *
             * @private
             * @param {Function} func The function to wrap.
             * @param {number} bitmask The bitmask flags. See `createWrap` for more details.
             * @param {*} [thisArg] The `this` binding of `func`.
             * @returns {Function} Returns the new wrapped function.
             */
            function createBind(func, bitmask, thisArg) {
                var isBind = bitmask & WRAP_BIND_FLAG, Ctor = createCtor(func);
                function wrapper() {
                    var fn = (this && this !== root && this instanceof wrapper) ? Ctor : func;
                    return fn.apply(isBind ? thisArg : this, arguments);
                }
                return wrapper;
            }
            /**
             * Creates a function like `_.lowerFirst`.
             *
             * @private
             * @param {string} methodName The name of the `String` case method to use.
             * @returns {Function} Returns the new case function.
             */
            function createCaseFirst(methodName) {
                return function (string) {
                    string = toString(string);
                    var strSymbols = hasUnicode(string)
                        ? stringToArray(string)
                        : undefined;
                    var chr = strSymbols
                        ? strSymbols[0]
                        : string.charAt(0);
                    var trailing = strSymbols
                        ? castSlice(strSymbols, 1).join('')
                        : string.slice(1);
                    return chr[methodName]() + trailing;
                };
            }
            /**
             * Creates a function like `_.camelCase`.
             *
             * @private
             * @param {Function} callback The function to combine each word.
             * @returns {Function} Returns the new compounder function.
             */
            function createCompounder(callback) {
                return function (string) {
                    return arrayReduce(words(deburr(string).replace(reApos, '')), callback, '');
                };
            }
            /**
             * Creates a function that produces an instance of `Ctor` regardless of
             * whether it was invoked as part of a `new` expression or by `call` or `apply`.
             *
             * @private
             * @param {Function} Ctor The constructor to wrap.
             * @returns {Function} Returns the new wrapped function.
             */
            function createCtor(Ctor) {
                return function () {
                    // Use a `switch` statement to work with class constructors. See
                    // http://ecma-international.org/ecma-262/7.0/#sec-ecmascript-function-objects-call-thisargument-argumentslist
                    // for more details.
                    var args = arguments;
                    switch (args.length) {
                        case 0: return new Ctor;
                        case 1: return new Ctor(args[0]);
                        case 2: return new Ctor(args[0], args[1]);
                        case 3: return new Ctor(args[0], args[1], args[2]);
                        case 4: return new Ctor(args[0], args[1], args[2], args[3]);
                        case 5: return new Ctor(args[0], args[1], args[2], args[3], args[4]);
                        case 6: return new Ctor(args[0], args[1], args[2], args[3], args[4], args[5]);
                        case 7: return new Ctor(args[0], args[1], args[2], args[3], args[4], args[5], args[6]);
                    }
                    var thisBinding = baseCreate(Ctor.prototype), result = Ctor.apply(thisBinding, args);
                    // Mimic the constructor's `return` behavior.
                    // See https://es5.github.io/#x13.2.2 for more details.
                    return isObject(result) ? result : thisBinding;
                };
            }
            /**
             * Creates a function that wraps `func` to enable currying.
             *
             * @private
             * @param {Function} func The function to wrap.
             * @param {number} bitmask The bitmask flags. See `createWrap` for more details.
             * @param {number} arity The arity of `func`.
             * @returns {Function} Returns the new wrapped function.
             */
            function createCurry(func, bitmask, arity) {
                var Ctor = createCtor(func);
                function wrapper() {
                    var length = arguments.length, args = Array(length), index = length, placeholder = getHolder(wrapper);
                    while (index--) {
                        args[index] = arguments[index];
                    }
                    var holders = (length < 3 && args[0] !== placeholder && args[length - 1] !== placeholder)
                        ? []
                        : replaceHolders(args, placeholder);
                    length -= holders.length;
                    if (length < arity) {
                        return createRecurry(func, bitmask, createHybrid, wrapper.placeholder, undefined, args, holders, undefined, undefined, arity - length);
                    }
                    var fn = (this && this !== root && this instanceof wrapper) ? Ctor : func;
                    return apply(fn, this, args);
                }
                return wrapper;
            }
            /**
             * Creates a `_.find` or `_.findLast` function.
             *
             * @private
             * @param {Function} findIndexFunc The function to find the collection index.
             * @returns {Function} Returns the new find function.
             */
            function createFind(findIndexFunc) {
                return function (collection, predicate, fromIndex) {
                    var iterable = Object(collection);
                    if (!isArrayLike(collection)) {
                        var iteratee = getIteratee(predicate, 3);
                        collection = keys(collection);
                        predicate = function (key) { return iteratee(iterable[key], key, iterable); };
                    }
                    var index = findIndexFunc(collection, predicate, fromIndex);
                    return index > -1 ? iterable[iteratee ? collection[index] : index] : undefined;
                };
            }
            /**
             * Creates a `_.flow` or `_.flowRight` function.
             *
             * @private
             * @param {boolean} [fromRight] Specify iterating from right to left.
             * @returns {Function} Returns the new flow function.
             */
            function createFlow(fromRight) {
                return flatRest(function (funcs) {
                    var length = funcs.length, index = length, prereq = LodashWrapper.prototype.thru;
                    if (fromRight) {
                        funcs.reverse();
                    }
                    while (index--) {
                        var func = funcs[index];
                        if (typeof func != 'function') {
                            throw new TypeError(FUNC_ERROR_TEXT);
                        }
                        if (prereq && !wrapper && getFuncName(func) == 'wrapper') {
                            var wrapper = new LodashWrapper([], true);
                        }
                    }
                    index = wrapper ? index : length;
                    while (++index < length) {
                        func = funcs[index];
                        var funcName = getFuncName(func), data = funcName == 'wrapper' ? getData(func) : undefined;
                        if (data && isLaziable(data[0]) &&
                            data[1] == (WRAP_ARY_FLAG | WRAP_CURRY_FLAG | WRAP_PARTIAL_FLAG | WRAP_REARG_FLAG) &&
                            !data[4].length && data[9] == 1) {
                            wrapper = wrapper[getFuncName(data[0])].apply(wrapper, data[3]);
                        }
                        else {
                            wrapper = (func.length == 1 && isLaziable(func))
                                ? wrapper[funcName]()
                                : wrapper.thru(func);
                        }
                    }
                    return function () {
                        var args = arguments, value = args[0];
                        if (wrapper && args.length == 1 && isArray(value)) {
                            return wrapper.plant(value).value();
                        }
                        var index = 0, result = length ? funcs[index].apply(this, args) : value;
                        while (++index < length) {
                            result = funcs[index].call(this, result);
                        }
                        return result;
                    };
                });
            }
            /**
             * Creates a function that wraps `func` to invoke it with optional `this`
             * binding of `thisArg`, partial application, and currying.
             *
             * @private
             * @param {Function|string} func The function or method name to wrap.
             * @param {number} bitmask The bitmask flags. See `createWrap` for more details.
             * @param {*} [thisArg] The `this` binding of `func`.
             * @param {Array} [partials] The arguments to prepend to those provided to
             *  the new function.
             * @param {Array} [holders] The `partials` placeholder indexes.
             * @param {Array} [partialsRight] The arguments to append to those provided
             *  to the new function.
             * @param {Array} [holdersRight] The `partialsRight` placeholder indexes.
             * @param {Array} [argPos] The argument positions of the new function.
             * @param {number} [ary] The arity cap of `func`.
             * @param {number} [arity] The arity of `func`.
             * @returns {Function} Returns the new wrapped function.
             */
            function createHybrid(func, bitmask, thisArg, partials, holders, partialsRight, holdersRight, argPos, ary, arity) {
                var isAry = bitmask & WRAP_ARY_FLAG, isBind = bitmask & WRAP_BIND_FLAG, isBindKey = bitmask & WRAP_BIND_KEY_FLAG, isCurried = bitmask & (WRAP_CURRY_FLAG | WRAP_CURRY_RIGHT_FLAG), isFlip = bitmask & WRAP_FLIP_FLAG, Ctor = isBindKey ? undefined : createCtor(func);
                function wrapper() {
                    var length = arguments.length, args = Array(length), index = length;
                    while (index--) {
                        args[index] = arguments[index];
                    }
                    if (isCurried) {
                        var placeholder = getHolder(wrapper), holdersCount = countHolders(args, placeholder);
                    }
                    if (partials) {
                        args = composeArgs(args, partials, holders, isCurried);
                    }
                    if (partialsRight) {
                        args = composeArgsRight(args, partialsRight, holdersRight, isCurried);
                    }
                    length -= holdersCount;
                    if (isCurried && length < arity) {
                        var newHolders = replaceHolders(args, placeholder);
                        return createRecurry(func, bitmask, createHybrid, wrapper.placeholder, thisArg, args, newHolders, argPos, ary, arity - length);
                    }
                    var thisBinding = isBind ? thisArg : this, fn = isBindKey ? thisBinding[func] : func;
                    length = args.length;
                    if (argPos) {
                        args = reorder(args, argPos);
                    }
                    else if (isFlip && length > 1) {
                        args.reverse();
                    }
                    if (isAry && ary < length) {
                        args.length = ary;
                    }
                    if (this && this !== root && this instanceof wrapper) {
                        fn = Ctor || createCtor(fn);
                    }
                    return fn.apply(thisBinding, args);
                }
                return wrapper;
            }
            /**
             * Creates a function like `_.invertBy`.
             *
             * @private
             * @param {Function} setter The function to set accumulator values.
             * @param {Function} toIteratee The function to resolve iteratees.
             * @returns {Function} Returns the new inverter function.
             */
            function createInverter(setter, toIteratee) {
                return function (object, iteratee) {
                    return baseInverter(object, setter, toIteratee(iteratee), {});
                };
            }
            /**
             * Creates a function that performs a mathematical operation on two values.
             *
             * @private
             * @param {Function} operator The function to perform the operation.
             * @param {number} [defaultValue] The value used for `undefined` arguments.
             * @returns {Function} Returns the new mathematical operation function.
             */
            function createMathOperation(operator, defaultValue) {
                return function (value, other) {
                    var result;
                    if (value === undefined && other === undefined) {
                        return defaultValue;
                    }
                    if (value !== undefined) {
                        result = value;
                    }
                    if (other !== undefined) {
                        if (result === undefined) {
                            return other;
                        }
                        if (typeof value == 'string' || typeof other == 'string') {
                            value = baseToString(value);
                            other = baseToString(other);
                        }
                        else {
                            value = baseToNumber(value);
                            other = baseToNumber(other);
                        }
                        result = operator(value, other);
                    }
                    return result;
                };
            }
            /**
             * Creates a function like `_.over`.
             *
             * @private
             * @param {Function} arrayFunc The function to iterate over iteratees.
             * @returns {Function} Returns the new over function.
             */
            function createOver(arrayFunc) {
                return flatRest(function (iteratees) {
                    iteratees = arrayMap(iteratees, baseUnary(getIteratee()));
                    return baseRest(function (args) {
                        var thisArg = this;
                        return arrayFunc(iteratees, function (iteratee) {
                            return apply(iteratee, thisArg, args);
                        });
                    });
                });
            }
            /**
             * Creates the padding for `string` based on `length`. The `chars` string
             * is truncated if the number of characters exceeds `length`.
             *
             * @private
             * @param {number} length The padding length.
             * @param {string} [chars=' '] The string used as padding.
             * @returns {string} Returns the padding for `string`.
             */
            function createPadding(length, chars) {
                chars = chars === undefined ? ' ' : baseToString(chars);
                var charsLength = chars.length;
                if (charsLength < 2) {
                    return charsLength ? baseRepeat(chars, length) : chars;
                }
                var result = baseRepeat(chars, nativeCeil(length / stringSize(chars)));
                return hasUnicode(chars)
                    ? castSlice(stringToArray(result), 0, length).join('')
                    : result.slice(0, length);
            }
            /**
             * Creates a function that wraps `func` to invoke it with the `this` binding
             * of `thisArg` and `partials` prepended to the arguments it receives.
             *
             * @private
             * @param {Function} func The function to wrap.
             * @param {number} bitmask The bitmask flags. See `createWrap` for more details.
             * @param {*} thisArg The `this` binding of `func`.
             * @param {Array} partials The arguments to prepend to those provided to
             *  the new function.
             * @returns {Function} Returns the new wrapped function.
             */
            function createPartial(func, bitmask, thisArg, partials) {
                var isBind = bitmask & WRAP_BIND_FLAG, Ctor = createCtor(func);
                function wrapper() {
                    var argsIndex = -1, argsLength = arguments.length, leftIndex = -1, leftLength = partials.length, args = Array(leftLength + argsLength), fn = (this && this !== root && this instanceof wrapper) ? Ctor : func;
                    while (++leftIndex < leftLength) {
                        args[leftIndex] = partials[leftIndex];
                    }
                    while (argsLength--) {
                        args[leftIndex++] = arguments[++argsIndex];
                    }
                    return apply(fn, isBind ? thisArg : this, args);
                }
                return wrapper;
            }
            /**
             * Creates a `_.range` or `_.rangeRight` function.
             *
             * @private
             * @param {boolean} [fromRight] Specify iterating from right to left.
             * @returns {Function} Returns the new range function.
             */
            function createRange(fromRight) {
                return function (start, end, step) {
                    if (step && typeof step != 'number' && isIterateeCall(start, end, step)) {
                        end = step = undefined;
                    }
                    // Ensure the sign of `-0` is preserved.
                    start = toFinite(start);
                    if (end === undefined) {
                        end = start;
                        start = 0;
                    }
                    else {
                        end = toFinite(end);
                    }
                    step = step === undefined ? (start < end ? 1 : -1) : toFinite(step);
                    return baseRange(start, end, step, fromRight);
                };
            }
            /**
             * Creates a function that performs a relational operation on two values.
             *
             * @private
             * @param {Function} operator The function to perform the operation.
             * @returns {Function} Returns the new relational operation function.
             */
            function createRelationalOperation(operator) {
                return function (value, other) {
                    if (!(typeof value == 'string' && typeof other == 'string')) {
                        value = toNumber(value);
                        other = toNumber(other);
                    }
                    return operator(value, other);
                };
            }
            /**
             * Creates a function that wraps `func` to continue currying.
             *
             * @private
             * @param {Function} func The function to wrap.
             * @param {number} bitmask The bitmask flags. See `createWrap` for more details.
             * @param {Function} wrapFunc The function to create the `func` wrapper.
             * @param {*} placeholder The placeholder value.
             * @param {*} [thisArg] The `this` binding of `func`.
             * @param {Array} [partials] The arguments to prepend to those provided to
             *  the new function.
             * @param {Array} [holders] The `partials` placeholder indexes.
             * @param {Array} [argPos] The argument positions of the new function.
             * @param {number} [ary] The arity cap of `func`.
             * @param {number} [arity] The arity of `func`.
             * @returns {Function} Returns the new wrapped function.
             */
            function createRecurry(func, bitmask, wrapFunc, placeholder, thisArg, partials, holders, argPos, ary, arity) {
                var isCurry = bitmask & WRAP_CURRY_FLAG, newHolders = isCurry ? holders : undefined, newHoldersRight = isCurry ? undefined : holders, newPartials = isCurry ? partials : undefined, newPartialsRight = isCurry ? undefined : partials;
                bitmask |= (isCurry ? WRAP_PARTIAL_FLAG : WRAP_PARTIAL_RIGHT_FLAG);
                bitmask &= ~(isCurry ? WRAP_PARTIAL_RIGHT_FLAG : WRAP_PARTIAL_FLAG);
                if (!(bitmask & WRAP_CURRY_BOUND_FLAG)) {
                    bitmask &= ~(WRAP_BIND_FLAG | WRAP_BIND_KEY_FLAG);
                }
                var newData = [
                    func, bitmask, thisArg, newPartials, newHolders, newPartialsRight,
                    newHoldersRight, argPos, ary, arity
                ];
                var result = wrapFunc.apply(undefined, newData);
                if (isLaziable(func)) {
                    setData(result, newData);
                }
                result.placeholder = placeholder;
                return setWrapToString(result, func, bitmask);
            }
            /**
             * Creates a function like `_.round`.
             *
             * @private
             * @param {string} methodName The name of the `Math` method to use when rounding.
             * @returns {Function} Returns the new round function.
             */
            function createRound(methodName) {
                var func = Math[methodName];
                return function (number, precision) {
                    number = toNumber(number);
                    precision = precision == null ? 0 : nativeMin(toInteger(precision), 292);
                    if (precision && nativeIsFinite(number)) {
                        // Shift with exponential notation to avoid floating-point issues.
                        // See [MDN](https://mdn.io/round#Examples) for more details.
                        var pair = (toString(number) + 'e').split('e'), value = func(pair[0] + 'e' + (+pair[1] + precision));
                        pair = (toString(value) + 'e').split('e');
                        return +(pair[0] + 'e' + (+pair[1] - precision));
                    }
                    return func(number);
                };
            }
            /**
             * Creates a set object of `values`.
             *
             * @private
             * @param {Array} values The values to add to the set.
             * @returns {Object} Returns the new set.
             */
            var createSet = !(Set && (1 / setToArray(new Set([, -0]))[1]) == INFINITY) ? noop : function (values) {
                return new Set(values);
            };
            /**
             * Creates a `_.toPairs` or `_.toPairsIn` function.
             *
             * @private
             * @param {Function} keysFunc The function to get the keys of a given object.
             * @returns {Function} Returns the new pairs function.
             */
            function createToPairs(keysFunc) {
                return function (object) {
                    var tag = getTag(object);
                    if (tag == mapTag) {
                        return mapToArray(object);
                    }
                    if (tag == setTag) {
                        return setToPairs(object);
                    }
                    return baseToPairs(object, keysFunc(object));
                };
            }
            /**
             * Creates a function that either curries or invokes `func` with optional
             * `this` binding and partially applied arguments.
             *
             * @private
             * @param {Function|string} func The function or method name to wrap.
             * @param {number} bitmask The bitmask flags.
             *    1 - `_.bind`
             *    2 - `_.bindKey`
             *    4 - `_.curry` or `_.curryRight` of a bound function
             *    8 - `_.curry`
             *   16 - `_.curryRight`
             *   32 - `_.partial`
             *   64 - `_.partialRight`
             *  128 - `_.rearg`
             *  256 - `_.ary`
             *  512 - `_.flip`
             * @param {*} [thisArg] The `this` binding of `func`.
             * @param {Array} [partials] The arguments to be partially applied.
             * @param {Array} [holders] The `partials` placeholder indexes.
             * @param {Array} [argPos] The argument positions of the new function.
             * @param {number} [ary] The arity cap of `func`.
             * @param {number} [arity] The arity of `func`.
             * @returns {Function} Returns the new wrapped function.
             */
            function createWrap(func, bitmask, thisArg, partials, holders, argPos, ary, arity) {
                var isBindKey = bitmask & WRAP_BIND_KEY_FLAG;
                if (!isBindKey && typeof func != 'function') {
                    throw new TypeError(FUNC_ERROR_TEXT);
                }
                var length = partials ? partials.length : 0;
                if (!length) {
                    bitmask &= ~(WRAP_PARTIAL_FLAG | WRAP_PARTIAL_RIGHT_FLAG);
                    partials = holders = undefined;
                }
                ary = ary === undefined ? ary : nativeMax(toInteger(ary), 0);
                arity = arity === undefined ? arity : toInteger(arity);
                length -= holders ? holders.length : 0;
                if (bitmask & WRAP_PARTIAL_RIGHT_FLAG) {
                    var partialsRight = partials, holdersRight = holders;
                    partials = holders = undefined;
                }
                var data = isBindKey ? undefined : getData(func);
                var newData = [
                    func, bitmask, thisArg, partials, holders, partialsRight, holdersRight,
                    argPos, ary, arity
                ];
                if (data) {
                    mergeData(newData, data);
                }
                func = newData[0];
                bitmask = newData[1];
                thisArg = newData[2];
                partials = newData[3];
                holders = newData[4];
                arity = newData[9] = newData[9] === undefined
                    ? (isBindKey ? 0 : func.length)
                    : nativeMax(newData[9] - length, 0);
                if (!arity && bitmask & (WRAP_CURRY_FLAG | WRAP_CURRY_RIGHT_FLAG)) {
                    bitmask &= ~(WRAP_CURRY_FLAG | WRAP_CURRY_RIGHT_FLAG);
                }
                if (!bitmask || bitmask == WRAP_BIND_FLAG) {
                    var result = createBind(func, bitmask, thisArg);
                }
                else if (bitmask == WRAP_CURRY_FLAG || bitmask == WRAP_CURRY_RIGHT_FLAG) {
                    result = createCurry(func, bitmask, arity);
                }
                else if ((bitmask == WRAP_PARTIAL_FLAG || bitmask == (WRAP_BIND_FLAG | WRAP_PARTIAL_FLAG)) && !holders.length) {
                    result = createPartial(func, bitmask, thisArg, partials);
                }
                else {
                    result = createHybrid.apply(undefined, newData);
                }
                var setter = data ? baseSetData : setData;
                return setWrapToString(setter(result, newData), func, bitmask);
            }
            /**
             * Used by `_.defaults` to customize its `_.assignIn` use to assign properties
             * of source objects to the destination object for all destination properties
             * that resolve to `undefined`.
             *
             * @private
             * @param {*} objValue The destination value.
             * @param {*} srcValue The source value.
             * @param {string} key The key of the property to assign.
             * @param {Object} object The parent object of `objValue`.
             * @returns {*} Returns the value to assign.
             */
            function customDefaultsAssignIn(objValue, srcValue, key, object) {
                if (objValue === undefined ||
                    (eq(objValue, objectProto[key]) && !hasOwnProperty.call(object, key))) {
                    return srcValue;
                }
                return objValue;
            }
            /**
             * Used by `_.defaultsDeep` to customize its `_.merge` use to merge source
             * objects into destination objects that are passed thru.
             *
             * @private
             * @param {*} objValue The destination value.
             * @param {*} srcValue The source value.
             * @param {string} key The key of the property to merge.
             * @param {Object} object The parent object of `objValue`.
             * @param {Object} source The parent object of `srcValue`.
             * @param {Object} [stack] Tracks traversed source values and their merged
             *  counterparts.
             * @returns {*} Returns the value to assign.
             */
            function customDefaultsMerge(objValue, srcValue, key, object, source, stack) {
                if (isObject(objValue) && isObject(srcValue)) {
                    // Recursively merge objects and arrays (susceptible to call stack limits).
                    stack.set(srcValue, objValue);
                    baseMerge(objValue, srcValue, undefined, customDefaultsMerge, stack);
                    stack['delete'](srcValue);
                }
                return objValue;
            }
            /**
             * Used by `_.omit` to customize its `_.cloneDeep` use to only clone plain
             * objects.
             *
             * @private
             * @param {*} value The value to inspect.
             * @param {string} key The key of the property to inspect.
             * @returns {*} Returns the uncloned value or `undefined` to defer cloning to `_.cloneDeep`.
             */
            function customOmitClone(value) {
                return isPlainObject(value) ? undefined : value;
            }
            /**
             * A specialized version of `baseIsEqualDeep` for arrays with support for
             * partial deep comparisons.
             *
             * @private
             * @param {Array} array The array to compare.
             * @param {Array} other The other array to compare.
             * @param {number} bitmask The bitmask flags. See `baseIsEqual` for more details.
             * @param {Function} customizer The function to customize comparisons.
             * @param {Function} equalFunc The function to determine equivalents of values.
             * @param {Object} stack Tracks traversed `array` and `other` objects.
             * @returns {boolean} Returns `true` if the arrays are equivalent, else `false`.
             */
            function equalArrays(array, other, bitmask, customizer, equalFunc, stack) {
                var isPartial = bitmask & COMPARE_PARTIAL_FLAG, arrLength = array.length, othLength = other.length;
                if (arrLength != othLength && !(isPartial && othLength > arrLength)) {
                    return false;
                }
                // Assume cyclic values are equal.
                var stacked = stack.get(array);
                if (stacked && stack.get(other)) {
                    return stacked == other;
                }
                var index = -1, result = true, seen = (bitmask & COMPARE_UNORDERED_FLAG) ? new SetCache : undefined;
                stack.set(array, other);
                stack.set(other, array);
                // Ignore non-index properties.
                while (++index < arrLength) {
                    var arrValue = array[index], othValue = other[index];
                    if (customizer) {
                        var compared = isPartial
                            ? customizer(othValue, arrValue, index, other, array, stack)
                            : customizer(arrValue, othValue, index, array, other, stack);
                    }
                    if (compared !== undefined) {
                        if (compared) {
                            continue;
                        }
                        result = false;
                        break;
                    }
                    // Recursively compare arrays (susceptible to call stack limits).
                    if (seen) {
                        if (!arraySome(other, function (othValue, othIndex) {
                            if (!cacheHas(seen, othIndex) &&
                                (arrValue === othValue || equalFunc(arrValue, othValue, bitmask, customizer, stack))) {
                                return seen.push(othIndex);
                            }
                        })) {
                            result = false;
                            break;
                        }
                    }
                    else if (!(arrValue === othValue ||
                        equalFunc(arrValue, othValue, bitmask, customizer, stack))) {
                        result = false;
                        break;
                    }
                }
                stack['delete'](array);
                stack['delete'](other);
                return result;
            }
            /**
             * A specialized version of `baseIsEqualDeep` for comparing objects of
             * the same `toStringTag`.
             *
             * **Note:** This function only supports comparing values with tags of
             * `Boolean`, `Date`, `Error`, `Number`, `RegExp`, or `String`.
             *
             * @private
             * @param {Object} object The object to compare.
             * @param {Object} other The other object to compare.
             * @param {string} tag The `toStringTag` of the objects to compare.
             * @param {number} bitmask The bitmask flags. See `baseIsEqual` for more details.
             * @param {Function} customizer The function to customize comparisons.
             * @param {Function} equalFunc The function to determine equivalents of values.
             * @param {Object} stack Tracks traversed `object` and `other` objects.
             * @returns {boolean} Returns `true` if the objects are equivalent, else `false`.
             */
            function equalByTag(object, other, tag, bitmask, customizer, equalFunc, stack) {
                switch (tag) {
                    case dataViewTag:
                        if ((object.byteLength != other.byteLength) ||
                            (object.byteOffset != other.byteOffset)) {
                            return false;
                        }
                        object = object.buffer;
                        other = other.buffer;
                    case arrayBufferTag:
                        if ((object.byteLength != other.byteLength) ||
                            !equalFunc(new Uint8Array(object), new Uint8Array(other))) {
                            return false;
                        }
                        return true;
                    case boolTag:
                    case dateTag:
                    case numberTag:
                        // Coerce booleans to `1` or `0` and dates to milliseconds.
                        // Invalid dates are coerced to `NaN`.
                        return eq(+object, +other);
                    case errorTag:
                        return object.name == other.name && object.message == other.message;
                    case regexpTag:
                    case stringTag:
                        // Coerce regexes to strings and treat strings, primitives and objects,
                        // as equal. See http://www.ecma-international.org/ecma-262/7.0/#sec-regexp.prototype.tostring
                        // for more details.
                        return object == (other + '');
                    case mapTag:
                        var convert = mapToArray;
                    case setTag:
                        var isPartial = bitmask & COMPARE_PARTIAL_FLAG;
                        convert || (convert = setToArray);
                        if (object.size != other.size && !isPartial) {
                            return false;
                        }
                        // Assume cyclic values are equal.
                        var stacked = stack.get(object);
                        if (stacked) {
                            return stacked == other;
                        }
                        bitmask |= COMPARE_UNORDERED_FLAG;
                        // Recursively compare objects (susceptible to call stack limits).
                        stack.set(object, other);
                        var result = equalArrays(convert(object), convert(other), bitmask, customizer, equalFunc, stack);
                        stack['delete'](object);
                        return result;
                    case symbolTag:
                        if (symbolValueOf) {
                            return symbolValueOf.call(object) == symbolValueOf.call(other);
                        }
                }
                return false;
            }
            /**
             * A specialized version of `baseIsEqualDeep` for objects with support for
             * partial deep comparisons.
             *
             * @private
             * @param {Object} object The object to compare.
             * @param {Object} other The other object to compare.
             * @param {number} bitmask The bitmask flags. See `baseIsEqual` for more details.
             * @param {Function} customizer The function to customize comparisons.
             * @param {Function} equalFunc The function to determine equivalents of values.
             * @param {Object} stack Tracks traversed `object` and `other` objects.
             * @returns {boolean} Returns `true` if the objects are equivalent, else `false`.
             */
            function equalObjects(object, other, bitmask, customizer, equalFunc, stack) {
                var isPartial = bitmask & COMPARE_PARTIAL_FLAG, objProps = getAllKeys(object), objLength = objProps.length, othProps = getAllKeys(other), othLength = othProps.length;
                if (objLength != othLength && !isPartial) {
                    return false;
                }
                var index = objLength;
                while (index--) {
                    var key = objProps[index];
                    if (!(isPartial ? key in other : hasOwnProperty.call(other, key))) {
                        return false;
                    }
                }
                // Assume cyclic values are equal.
                var stacked = stack.get(object);
                if (stacked && stack.get(other)) {
                    return stacked == other;
                }
                var result = true;
                stack.set(object, other);
                stack.set(other, object);
                var skipCtor = isPartial;
                while (++index < objLength) {
                    key = objProps[index];
                    var objValue = object[key], othValue = other[key];
                    if (customizer) {
                        var compared = isPartial
                            ? customizer(othValue, objValue, key, other, object, stack)
                            : customizer(objValue, othValue, key, object, other, stack);
                    }
                    // Recursively compare objects (susceptible to call stack limits).
                    if (!(compared === undefined
                        ? (objValue === othValue || equalFunc(objValue, othValue, bitmask, customizer, stack))
                        : compared)) {
                        result = false;
                        break;
                    }
                    skipCtor || (skipCtor = key == 'constructor');
                }
                if (result && !skipCtor) {
                    var objCtor = object.constructor, othCtor = other.constructor;
                    // Non `Object` object instances with different constructors are not equal.
                    if (objCtor != othCtor &&
                        ('constructor' in object && 'constructor' in other) &&
                        !(typeof objCtor == 'function' && objCtor instanceof objCtor &&
                            typeof othCtor == 'function' && othCtor instanceof othCtor)) {
                        result = false;
                    }
                }
                stack['delete'](object);
                stack['delete'](other);
                return result;
            }
            /**
             * A specialized version of `baseRest` which flattens the rest array.
             *
             * @private
             * @param {Function} func The function to apply a rest parameter to.
             * @returns {Function} Returns the new function.
             */
            function flatRest(func) {
                return setToString(overRest(func, undefined, flatten), func + '');
            }
            /**
             * Creates an array of own enumerable property names and symbols of `object`.
             *
             * @private
             * @param {Object} object The object to query.
             * @returns {Array} Returns the array of property names and symbols.
             */
            function getAllKeys(object) {
                return baseGetAllKeys(object, keys, getSymbols);
            }
            /**
             * Creates an array of own and inherited enumerable property names and
             * symbols of `object`.
             *
             * @private
             * @param {Object} object The object to query.
             * @returns {Array} Returns the array of property names and symbols.
             */
            function getAllKeysIn(object) {
                return baseGetAllKeys(object, keysIn, getSymbolsIn);
            }
            /**
             * Gets metadata for `func`.
             *
             * @private
             * @param {Function} func The function to query.
             * @returns {*} Returns the metadata for `func`.
             */
            var getData = !metaMap ? noop : function (func) {
                return metaMap.get(func);
            };
            /**
             * Gets the name of `func`.
             *
             * @private
             * @param {Function} func The function to query.
             * @returns {string} Returns the function name.
             */
            function getFuncName(func) {
                var result = (func.name + ''), array = realNames[result], length = hasOwnProperty.call(realNames, result) ? array.length : 0;
                while (length--) {
                    var data = array[length], otherFunc = data.func;
                    if (otherFunc == null || otherFunc == func) {
                        return data.name;
                    }
                }
                return result;
            }
            /**
             * Gets the argument placeholder value for `func`.
             *
             * @private
             * @param {Function} func The function to inspect.
             * @returns {*} Returns the placeholder value.
             */
            function getHolder(func) {
                var object = hasOwnProperty.call(lodash, 'placeholder') ? lodash : func;
                return object.placeholder;
            }
            /**
             * Gets the appropriate "iteratee" function. If `_.iteratee` is customized,
             * this function returns the custom method, otherwise it returns `baseIteratee`.
             * If arguments are provided, the chosen function is invoked with them and
             * its result is returned.
             *
             * @private
             * @param {*} [value] The value to convert to an iteratee.
             * @param {number} [arity] The arity of the created iteratee.
             * @returns {Function} Returns the chosen function or its result.
             */
            function getIteratee() {
                var result = lodash.iteratee || iteratee;
                result = result === iteratee ? baseIteratee : result;
                return arguments.length ? result(arguments[0], arguments[1]) : result;
            }
            /**
             * Gets the data for `map`.
             *
             * @private
             * @param {Object} map The map to query.
             * @param {string} key The reference key.
             * @returns {*} Returns the map data.
             */
            function getMapData(map, key) {
                var data = map.__data__;
                return isKeyable(key)
                    ? data[typeof key == 'string' ? 'string' : 'hash']
                    : data.map;
            }
            /**
             * Gets the property names, values, and compare flags of `object`.
             *
             * @private
             * @param {Object} object The object to query.
             * @returns {Array} Returns the match data of `object`.
             */
            function getMatchData(object) {
                var result = keys(object), length = result.length;
                while (length--) {
                    var key = result[length], value = object[key];
                    result[length] = [key, value, isStrictComparable(value)];
                }
                return result;
            }
            /**
             * Gets the native function at `key` of `object`.
             *
             * @private
             * @param {Object} object The object to query.
             * @param {string} key The key of the method to get.
             * @returns {*} Returns the function if it's native, else `undefined`.
             */
            function getNative(object, key) {
                var value = getValue(object, key);
                return baseIsNative(value) ? value : undefined;
            }
            /**
             * A specialized version of `baseGetTag` which ignores `Symbol.toStringTag` values.
             *
             * @private
             * @param {*} value The value to query.
             * @returns {string} Returns the raw `toStringTag`.
             */
            function getRawTag(value) {
                var isOwn = hasOwnProperty.call(value, symToStringTag), tag = value[symToStringTag];
                try {
                    value[symToStringTag] = undefined;
                    var unmasked = true;
                }
                catch (e) { }
                var result = nativeObjectToString.call(value);
                if (unmasked) {
                    if (isOwn) {
                        value[symToStringTag] = tag;
                    }
                    else {
                        delete value[symToStringTag];
                    }
                }
                return result;
            }
            /**
             * Creates an array of the own enumerable symbols of `object`.
             *
             * @private
             * @param {Object} object The object to query.
             * @returns {Array} Returns the array of symbols.
             */
            var getSymbols = !nativeGetSymbols ? stubArray : function (object) {
                if (object == null) {
                    return [];
                }
                object = Object(object);
                return arrayFilter(nativeGetSymbols(object), function (symbol) {
                    return propertyIsEnumerable.call(object, symbol);
                });
            };
            /**
             * Creates an array of the own and inherited enumerable symbols of `object`.
             *
             * @private
             * @param {Object} object The object to query.
             * @returns {Array} Returns the array of symbols.
             */
            var getSymbolsIn = !nativeGetSymbols ? stubArray : function (object) {
                var result = [];
                while (object) {
                    arrayPush(result, getSymbols(object));
                    object = getPrototype(object);
                }
                return result;
            };
            /**
             * Gets the `toStringTag` of `value`.
             *
             * @private
             * @param {*} value The value to query.
             * @returns {string} Returns the `toStringTag`.
             */
            var getTag = baseGetTag;
            // Fallback for data views, maps, sets, and weak maps in IE 11 and promises in Node.js < 6.
            if ((DataView && getTag(new DataView(new ArrayBuffer(1))) != dataViewTag) ||
                (Map && getTag(new Map) != mapTag) ||
                (Promise && getTag(Promise.resolve()) != promiseTag) ||
                (Set && getTag(new Set) != setTag) ||
                (WeakMap && getTag(new WeakMap) != weakMapTag)) {
                getTag = function (value) {
                    var result = baseGetTag(value), Ctor = result == objectTag ? value.constructor : undefined, ctorString = Ctor ? toSource(Ctor) : '';
                    if (ctorString) {
                        switch (ctorString) {
                            case dataViewCtorString: return dataViewTag;
                            case mapCtorString: return mapTag;
                            case promiseCtorString: return promiseTag;
                            case setCtorString: return setTag;
                            case weakMapCtorString: return weakMapTag;
                        }
                    }
                    return result;
                };
            }
            /**
             * Gets the view, applying any `transforms` to the `start` and `end` positions.
             *
             * @private
             * @param {number} start The start of the view.
             * @param {number} end The end of the view.
             * @param {Array} transforms The transformations to apply to the view.
             * @returns {Object} Returns an object containing the `start` and `end`
             *  positions of the view.
             */
            function getView(start, end, transforms) {
                var index = -1, length = transforms.length;
                while (++index < length) {
                    var data = transforms[index], size = data.size;
                    switch (data.type) {
                        case 'drop':
                            start += size;
                            break;
                        case 'dropRight':
                            end -= size;
                            break;
                        case 'take':
                            end = nativeMin(end, start + size);
                            break;
                        case 'takeRight':
                            start = nativeMax(start, end - size);
                            break;
                    }
                }
                return { 'start': start, 'end': end };
            }
            /**
             * Extracts wrapper details from the `source` body comment.
             *
             * @private
             * @param {string} source The source to inspect.
             * @returns {Array} Returns the wrapper details.
             */
            function getWrapDetails(source) {
                var match = source.match(reWrapDetails);
                return match ? match[1].split(reSplitDetails) : [];
            }
            /**
             * Checks if `path` exists on `object`.
             *
             * @private
             * @param {Object} object The object to query.
             * @param {Array|string} path The path to check.
             * @param {Function} hasFunc The function to check properties.
             * @returns {boolean} Returns `true` if `path` exists, else `false`.
             */
            function hasPath(object, path, hasFunc) {
                path = castPath(path, object);
                var index = -1, length = path.length, result = false;
                while (++index < length) {
                    var key = toKey(path[index]);
                    if (!(result = object != null && hasFunc(object, key))) {
                        break;
                    }
                    object = object[key];
                }
                if (result || ++index != length) {
                    return result;
                }
                length = object == null ? 0 : object.length;
                return !!length && isLength(length) && isIndex(key, length) &&
                    (isArray(object) || isArguments(object));
            }
            /**
             * Initializes an array clone.
             *
             * @private
             * @param {Array} array The array to clone.
             * @returns {Array} Returns the initialized clone.
             */
            function initCloneArray(array) {
                var length = array.length, result = new array.constructor(length);
                // Add properties assigned by `RegExp#exec`.
                if (length && typeof array[0] == 'string' && hasOwnProperty.call(array, 'index')) {
                    result.index = array.index;
                    result.input = array.input;
                }
                return result;
            }
            /**
             * Initializes an object clone.
             *
             * @private
             * @param {Object} object The object to clone.
             * @returns {Object} Returns the initialized clone.
             */
            function initCloneObject(object) {
                return (typeof object.constructor == 'function' && !isPrototype(object))
                    ? baseCreate(getPrototype(object))
                    : {};
            }
            /**
             * Initializes an object clone based on its `toStringTag`.
             *
             * **Note:** This function only supports cloning values with tags of
             * `Boolean`, `Date`, `Error`, `Map`, `Number`, `RegExp`, `Set`, or `String`.
             *
             * @private
             * @param {Object} object The object to clone.
             * @param {string} tag The `toStringTag` of the object to clone.
             * @param {boolean} [isDeep] Specify a deep clone.
             * @returns {Object} Returns the initialized clone.
             */
            function initCloneByTag(object, tag, isDeep) {
                var Ctor = object.constructor;
                switch (tag) {
                    case arrayBufferTag:
                        return cloneArrayBuffer(object);
                    case boolTag:
                    case dateTag:
                        return new Ctor(+object);
                    case dataViewTag:
                        return cloneDataView(object, isDeep);
                    case float32Tag:
                    case float64Tag:
                    case int8Tag:
                    case int16Tag:
                    case int32Tag:
                    case uint8Tag:
                    case uint8ClampedTag:
                    case uint16Tag:
                    case uint32Tag:
                        return cloneTypedArray(object, isDeep);
                    case mapTag:
                        return new Ctor;
                    case numberTag:
                    case stringTag:
                        return new Ctor(object);
                    case regexpTag:
                        return cloneRegExp(object);
                    case setTag:
                        return new Ctor;
                    case symbolTag:
                        return cloneSymbol(object);
                }
            }
            /**
             * Inserts wrapper `details` in a comment at the top of the `source` body.
             *
             * @private
             * @param {string} source The source to modify.
             * @returns {Array} details The details to insert.
             * @returns {string} Returns the modified source.
             */
            function insertWrapDetails(source, details) {
                var length = details.length;
                if (!length) {
                    return source;
                }
                var lastIndex = length - 1;
                details[lastIndex] = (length > 1 ? '& ' : '') + details[lastIndex];
                details = details.join(length > 2 ? ', ' : ' ');
                return source.replace(reWrapComment, '{\n/* [wrapped with ' + details + '] */\n');
            }
            /**
             * Checks if `value` is a flattenable `arguments` object or array.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is flattenable, else `false`.
             */
            function isFlattenable(value) {
                return isArray(value) || isArguments(value) ||
                    !!(spreadableSymbol && value && value[spreadableSymbol]);
            }
            /**
             * Checks if `value` is a valid array-like index.
             *
             * @private
             * @param {*} value The value to check.
             * @param {number} [length=MAX_SAFE_INTEGER] The upper bounds of a valid index.
             * @returns {boolean} Returns `true` if `value` is a valid index, else `false`.
             */
            function isIndex(value, length) {
                var type = typeof value;
                length = length == null ? MAX_SAFE_INTEGER : length;
                return !!length &&
                    (type == 'number' ||
                        (type != 'symbol' && reIsUint.test(value))) &&
                    (value > -1 && value % 1 == 0 && value < length);
            }
            /**
             * Checks if the given arguments are from an iteratee call.
             *
             * @private
             * @param {*} value The potential iteratee value argument.
             * @param {*} index The potential iteratee index or key argument.
             * @param {*} object The potential iteratee object argument.
             * @returns {boolean} Returns `true` if the arguments are from an iteratee call,
             *  else `false`.
             */
            function isIterateeCall(value, index, object) {
                if (!isObject(object)) {
                    return false;
                }
                var type = typeof index;
                if (type == 'number'
                    ? (isArrayLike(object) && isIndex(index, object.length))
                    : (type == 'string' && index in object)) {
                    return eq(object[index], value);
                }
                return false;
            }
            /**
             * Checks if `value` is a property name and not a property path.
             *
             * @private
             * @param {*} value The value to check.
             * @param {Object} [object] The object to query keys on.
             * @returns {boolean} Returns `true` if `value` is a property name, else `false`.
             */
            function isKey(value, object) {
                if (isArray(value)) {
                    return false;
                }
                var type = typeof value;
                if (type == 'number' || type == 'symbol' || type == 'boolean' ||
                    value == null || isSymbol(value)) {
                    return true;
                }
                return reIsPlainProp.test(value) || !reIsDeepProp.test(value) ||
                    (object != null && value in Object(object));
            }
            /**
             * Checks if `value` is suitable for use as unique object key.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is suitable, else `false`.
             */
            function isKeyable(value) {
                var type = typeof value;
                return (type == 'string' || type == 'number' || type == 'symbol' || type == 'boolean')
                    ? (value !== '__proto__')
                    : (value === null);
            }
            /**
             * Checks if `func` has a lazy counterpart.
             *
             * @private
             * @param {Function} func The function to check.
             * @returns {boolean} Returns `true` if `func` has a lazy counterpart,
             *  else `false`.
             */
            function isLaziable(func) {
                var funcName = getFuncName(func), other = lodash[funcName];
                if (typeof other != 'function' || !(funcName in LazyWrapper.prototype)) {
                    return false;
                }
                if (func === other) {
                    return true;
                }
                var data = getData(other);
                return !!data && func === data[0];
            }
            /**
             * Checks if `func` has its source masked.
             *
             * @private
             * @param {Function} func The function to check.
             * @returns {boolean} Returns `true` if `func` is masked, else `false`.
             */
            function isMasked(func) {
                return !!maskSrcKey && (maskSrcKey in func);
            }
            /**
             * Checks if `func` is capable of being masked.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `func` is maskable, else `false`.
             */
            var isMaskable = coreJsData ? isFunction : stubFalse;
            /**
             * Checks if `value` is likely a prototype object.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a prototype, else `false`.
             */
            function isPrototype(value) {
                var Ctor = value && value.constructor, proto = (typeof Ctor == 'function' && Ctor.prototype) || objectProto;
                return value === proto;
            }
            /**
             * Checks if `value` is suitable for strict equality comparisons, i.e. `===`.
             *
             * @private
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` if suitable for strict
             *  equality comparisons, else `false`.
             */
            function isStrictComparable(value) {
                return value === value && !isObject(value);
            }
            /**
             * A specialized version of `matchesProperty` for source values suitable
             * for strict equality comparisons, i.e. `===`.
             *
             * @private
             * @param {string} key The key of the property to get.
             * @param {*} srcValue The value to match.
             * @returns {Function} Returns the new spec function.
             */
            function matchesStrictComparable(key, srcValue) {
                return function (object) {
                    if (object == null) {
                        return false;
                    }
                    return object[key] === srcValue &&
                        (srcValue !== undefined || (key in Object(object)));
                };
            }
            /**
             * A specialized version of `_.memoize` which clears the memoized function's
             * cache when it exceeds `MAX_MEMOIZE_SIZE`.
             *
             * @private
             * @param {Function} func The function to have its output memoized.
             * @returns {Function} Returns the new memoized function.
             */
            function memoizeCapped(func) {
                var result = memoize(func, function (key) {
                    if (cache.size === MAX_MEMOIZE_SIZE) {
                        cache.clear();
                    }
                    return key;
                });
                var cache = result.cache;
                return result;
            }
            /**
             * Merges the function metadata of `source` into `data`.
             *
             * Merging metadata reduces the number of wrappers used to invoke a function.
             * This is possible because methods like `_.bind`, `_.curry`, and `_.partial`
             * may be applied regardless of execution order. Methods like `_.ary` and
             * `_.rearg` modify function arguments, making the order in which they are
             * executed important, preventing the merging of metadata. However, we make
             * an exception for a safe combined case where curried functions have `_.ary`
             * and or `_.rearg` applied.
             *
             * @private
             * @param {Array} data The destination metadata.
             * @param {Array} source The source metadata.
             * @returns {Array} Returns `data`.
             */
            function mergeData(data, source) {
                var bitmask = data[1], srcBitmask = source[1], newBitmask = bitmask | srcBitmask, isCommon = newBitmask < (WRAP_BIND_FLAG | WRAP_BIND_KEY_FLAG | WRAP_ARY_FLAG);
                var isCombo = ((srcBitmask == WRAP_ARY_FLAG) && (bitmask == WRAP_CURRY_FLAG)) ||
                    ((srcBitmask == WRAP_ARY_FLAG) && (bitmask == WRAP_REARG_FLAG) && (data[7].length <= source[8])) ||
                    ((srcBitmask == (WRAP_ARY_FLAG | WRAP_REARG_FLAG)) && (source[7].length <= source[8]) && (bitmask == WRAP_CURRY_FLAG));
                // Exit early if metadata can't be merged.
                if (!(isCommon || isCombo)) {
                    return data;
                }
                // Use source `thisArg` if available.
                if (srcBitmask & WRAP_BIND_FLAG) {
                    data[2] = source[2];
                    // Set when currying a bound function.
                    newBitmask |= bitmask & WRAP_BIND_FLAG ? 0 : WRAP_CURRY_BOUND_FLAG;
                }
                // Compose partial arguments.
                var value = source[3];
                if (value) {
                    var partials = data[3];
                    data[3] = partials ? composeArgs(partials, value, source[4]) : value;
                    data[4] = partials ? replaceHolders(data[3], PLACEHOLDER) : source[4];
                }
                // Compose partial right arguments.
                value = source[5];
                if (value) {
                    partials = data[5];
                    data[5] = partials ? composeArgsRight(partials, value, source[6]) : value;
                    data[6] = partials ? replaceHolders(data[5], PLACEHOLDER) : source[6];
                }
                // Use source `argPos` if available.
                value = source[7];
                if (value) {
                    data[7] = value;
                }
                // Use source `ary` if it's smaller.
                if (srcBitmask & WRAP_ARY_FLAG) {
                    data[8] = data[8] == null ? source[8] : nativeMin(data[8], source[8]);
                }
                // Use source `arity` if one is not provided.
                if (data[9] == null) {
                    data[9] = source[9];
                }
                // Use source `func` and merge bitmasks.
                data[0] = source[0];
                data[1] = newBitmask;
                return data;
            }
            /**
             * This function is like
             * [`Object.keys`](http://ecma-international.org/ecma-262/7.0/#sec-object.keys)
             * except that it includes inherited enumerable properties.
             *
             * @private
             * @param {Object} object The object to query.
             * @returns {Array} Returns the array of property names.
             */
            function nativeKeysIn(object) {
                var result = [];
                if (object != null) {
                    for (var key in Object(object)) {
                        result.push(key);
                    }
                }
                return result;
            }
            /**
             * Converts `value` to a string using `Object.prototype.toString`.
             *
             * @private
             * @param {*} value The value to convert.
             * @returns {string} Returns the converted string.
             */
            function objectToString(value) {
                return nativeObjectToString.call(value);
            }
            /**
             * A specialized version of `baseRest` which transforms the rest array.
             *
             * @private
             * @param {Function} func The function to apply a rest parameter to.
             * @param {number} [start=func.length-1] The start position of the rest parameter.
             * @param {Function} transform The rest array transform.
             * @returns {Function} Returns the new function.
             */
            function overRest(func, start, transform) {
                start = nativeMax(start === undefined ? (func.length - 1) : start, 0);
                return function () {
                    var args = arguments, index = -1, length = nativeMax(args.length - start, 0), array = Array(length);
                    while (++index < length) {
                        array[index] = args[start + index];
                    }
                    index = -1;
                    var otherArgs = Array(start + 1);
                    while (++index < start) {
                        otherArgs[index] = args[index];
                    }
                    otherArgs[start] = transform(array);
                    return apply(func, this, otherArgs);
                };
            }
            /**
             * Gets the parent value at `path` of `object`.
             *
             * @private
             * @param {Object} object The object to query.
             * @param {Array} path The path to get the parent value of.
             * @returns {*} Returns the parent value.
             */
            function parent(object, path) {
                return path.length < 2 ? object : baseGet(object, baseSlice(path, 0, -1));
            }
            /**
             * Reorder `array` according to the specified indexes where the element at
             * the first index is assigned as the first element, the element at
             * the second index is assigned as the second element, and so on.
             *
             * @private
             * @param {Array} array The array to reorder.
             * @param {Array} indexes The arranged array indexes.
             * @returns {Array} Returns `array`.
             */
            function reorder(array, indexes) {
                var arrLength = array.length, length = nativeMin(indexes.length, arrLength), oldArray = copyArray(array);
                while (length--) {
                    var index = indexes[length];
                    array[length] = isIndex(index, arrLength) ? oldArray[index] : undefined;
                }
                return array;
            }
            /**
             * Gets the value at `key`, unless `key` is "__proto__" or "constructor".
             *
             * @private
             * @param {Object} object The object to query.
             * @param {string} key The key of the property to get.
             * @returns {*} Returns the property value.
             */
            function safeGet(object, key) {
                if (key === 'constructor' && typeof object[key] === 'function') {
                    return;
                }
                if (key == '__proto__') {
                    return;
                }
                return object[key];
            }
            /**
             * Sets metadata for `func`.
             *
             * **Note:** If this function becomes hot, i.e. is invoked a lot in a short
             * period of time, it will trip its breaker and transition to an identity
             * function to avoid garbage collection pauses in V8. See
             * [V8 issue 2070](https://bugs.chromium.org/p/v8/issues/detail?id=2070)
             * for more details.
             *
             * @private
             * @param {Function} func The function to associate metadata with.
             * @param {*} data The metadata.
             * @returns {Function} Returns `func`.
             */
            var setData = shortOut(baseSetData);
            /**
             * A simple wrapper around the global [`setTimeout`](https://mdn.io/setTimeout).
             *
             * @private
             * @param {Function} func The function to delay.
             * @param {number} wait The number of milliseconds to delay invocation.
             * @returns {number|Object} Returns the timer id or timeout object.
             */
            var setTimeout = ctxSetTimeout || function (func, wait) {
                return root.setTimeout(func, wait);
            };
            /**
             * Sets the `toString` method of `func` to return `string`.
             *
             * @private
             * @param {Function} func The function to modify.
             * @param {Function} string The `toString` result.
             * @returns {Function} Returns `func`.
             */
            var setToString = shortOut(baseSetToString);
            /**
             * Sets the `toString` method of `wrapper` to mimic the source of `reference`
             * with wrapper details in a comment at the top of the source body.
             *
             * @private
             * @param {Function} wrapper The function to modify.
             * @param {Function} reference The reference function.
             * @param {number} bitmask The bitmask flags. See `createWrap` for more details.
             * @returns {Function} Returns `wrapper`.
             */
            function setWrapToString(wrapper, reference, bitmask) {
                var source = (reference + '');
                return setToString(wrapper, insertWrapDetails(source, updateWrapDetails(getWrapDetails(source), bitmask)));
            }
            /**
             * Creates a function that'll short out and invoke `identity` instead
             * of `func` when it's called `HOT_COUNT` or more times in `HOT_SPAN`
             * milliseconds.
             *
             * @private
             * @param {Function} func The function to restrict.
             * @returns {Function} Returns the new shortable function.
             */
            function shortOut(func) {
                var count = 0, lastCalled = 0;
                return function () {
                    var stamp = nativeNow(), remaining = HOT_SPAN - (stamp - lastCalled);
                    lastCalled = stamp;
                    if (remaining > 0) {
                        if (++count >= HOT_COUNT) {
                            return arguments[0];
                        }
                    }
                    else {
                        count = 0;
                    }
                    return func.apply(undefined, arguments);
                };
            }
            /**
             * A specialized version of `_.shuffle` which mutates and sets the size of `array`.
             *
             * @private
             * @param {Array} array The array to shuffle.
             * @param {number} [size=array.length] The size of `array`.
             * @returns {Array} Returns `array`.
             */
            function shuffleSelf(array, size) {
                var index = -1, length = array.length, lastIndex = length - 1;
                size = size === undefined ? length : size;
                while (++index < size) {
                    var rand = baseRandom(index, lastIndex), value = array[rand];
                    array[rand] = array[index];
                    array[index] = value;
                }
                array.length = size;
                return array;
            }
            /**
             * Converts `string` to a property path array.
             *
             * @private
             * @param {string} string The string to convert.
             * @returns {Array} Returns the property path array.
             */
            var stringToPath = memoizeCapped(function (string) {
                var result = [];
                if (string.charCodeAt(0) === 46 /* . */) {
                    result.push('');
                }
                string.replace(rePropName, function (match, number, quote, subString) {
                    result.push(quote ? subString.replace(reEscapeChar, '$1') : (number || match));
                });
                return result;
            });
            /**
             * Converts `value` to a string key if it's not a string or symbol.
             *
             * @private
             * @param {*} value The value to inspect.
             * @returns {string|symbol} Returns the key.
             */
            function toKey(value) {
                if (typeof value == 'string' || isSymbol(value)) {
                    return value;
                }
                var result = (value + '');
                return (result == '0' && (1 / value) == -INFINITY) ? '-0' : result;
            }
            /**
             * Converts `func` to its source code.
             *
             * @private
             * @param {Function} func The function to convert.
             * @returns {string} Returns the source code.
             */
            function toSource(func) {
                if (func != null) {
                    try {
                        return funcToString.call(func);
                    }
                    catch (e) { }
                    try {
                        return (func + '');
                    }
                    catch (e) { }
                }
                return '';
            }
            /**
             * Updates wrapper `details` based on `bitmask` flags.
             *
             * @private
             * @returns {Array} details The details to modify.
             * @param {number} bitmask The bitmask flags. See `createWrap` for more details.
             * @returns {Array} Returns `details`.
             */
            function updateWrapDetails(details, bitmask) {
                arrayEach(wrapFlags, function (pair) {
                    var value = '_.' + pair[0];
                    if ((bitmask & pair[1]) && !arrayIncludes(details, value)) {
                        details.push(value);
                    }
                });
                return details.sort();
            }
            /**
             * Creates a clone of `wrapper`.
             *
             * @private
             * @param {Object} wrapper The wrapper to clone.
             * @returns {Object} Returns the cloned wrapper.
             */
            function wrapperClone(wrapper) {
                if (wrapper instanceof LazyWrapper) {
                    return wrapper.clone();
                }
                var result = new LodashWrapper(wrapper.__wrapped__, wrapper.__chain__);
                result.__actions__ = copyArray(wrapper.__actions__);
                result.__index__ = wrapper.__index__;
                result.__values__ = wrapper.__values__;
                return result;
            }
            /*------------------------------------------------------------------------*/
            /**
             * Creates an array of elements split into groups the length of `size`.
             * If `array` can't be split evenly, the final chunk will be the remaining
             * elements.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Array
             * @param {Array} array The array to process.
             * @param {number} [size=1] The length of each chunk
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {Array} Returns the new array of chunks.
             * @example
             *
             * _.chunk(['a', 'b', 'c', 'd'], 2);
             * // => [['a', 'b'], ['c', 'd']]
             *
             * _.chunk(['a', 'b', 'c', 'd'], 3);
             * // => [['a', 'b', 'c'], ['d']]
             */
            function chunk(array, size, guard) {
                if ((guard ? isIterateeCall(array, size, guard) : size === undefined)) {
                    size = 1;
                }
                else {
                    size = nativeMax(toInteger(size), 0);
                }
                var length = array == null ? 0 : array.length;
                if (!length || size < 1) {
                    return [];
                }
                var index = 0, resIndex = 0, result = Array(nativeCeil(length / size));
                while (index < length) {
                    result[resIndex++] = baseSlice(array, index, (index += size));
                }
                return result;
            }
            /**
             * Creates an array with all falsey values removed. The values `false`, `null`,
             * `0`, `""`, `undefined`, and `NaN` are falsey.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {Array} array The array to compact.
             * @returns {Array} Returns the new array of filtered values.
             * @example
             *
             * _.compact([0, 1, false, 2, '', 3]);
             * // => [1, 2, 3]
             */
            function compact(array) {
                var index = -1, length = array == null ? 0 : array.length, resIndex = 0, result = [];
                while (++index < length) {
                    var value = array[index];
                    if (value) {
                        result[resIndex++] = value;
                    }
                }
                return result;
            }
            /**
             * Creates a new array concatenating `array` with any additional arrays
             * and/or values.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to concatenate.
             * @param {...*} [values] The values to concatenate.
             * @returns {Array} Returns the new concatenated array.
             * @example
             *
             * var array = [1];
             * var other = _.concat(array, 2, [3], [[4]]);
             *
             * console.log(other);
             * // => [1, 2, 3, [4]]
             *
             * console.log(array);
             * // => [1]
             */
            function concat() {
                var length = arguments.length;
                if (!length) {
                    return [];
                }
                var args = Array(length - 1), array = arguments[0], index = length;
                while (index--) {
                    args[index - 1] = arguments[index];
                }
                return arrayPush(isArray(array) ? copyArray(array) : [array], baseFlatten(args, 1));
            }
            /**
             * Creates an array of `array` values not included in the other given arrays
             * using [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
             * for equality comparisons. The order and references of result values are
             * determined by the first array.
             *
             * **Note:** Unlike `_.pullAll`, this method returns a new array.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {...Array} [values] The values to exclude.
             * @returns {Array} Returns the new array of filtered values.
             * @see _.without, _.xor
             * @example
             *
             * _.difference([2, 1], [2, 3]);
             * // => [1]
             */
            var difference = baseRest(function (array, values) {
                return isArrayLikeObject(array)
                    ? baseDifference(array, baseFlatten(values, 1, isArrayLikeObject, true))
                    : [];
            });
            /**
             * This method is like `_.difference` except that it accepts `iteratee` which
             * is invoked for each element of `array` and `values` to generate the criterion
             * by which they're compared. The order and references of result values are
             * determined by the first array. The iteratee is invoked with one argument:
             * (value).
             *
             * **Note:** Unlike `_.pullAllBy`, this method returns a new array.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {...Array} [values] The values to exclude.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {Array} Returns the new array of filtered values.
             * @example
             *
             * _.differenceBy([2.1, 1.2], [2.3, 3.4], Math.floor);
             * // => [1.2]
             *
             * // The `_.property` iteratee shorthand.
             * _.differenceBy([{ 'x': 2 }, { 'x': 1 }], [{ 'x': 1 }], 'x');
             * // => [{ 'x': 2 }]
             */
            var differenceBy = baseRest(function (array, values) {
                var iteratee = last(values);
                if (isArrayLikeObject(iteratee)) {
                    iteratee = undefined;
                }
                return isArrayLikeObject(array)
                    ? baseDifference(array, baseFlatten(values, 1, isArrayLikeObject, true), getIteratee(iteratee, 2))
                    : [];
            });
            /**
             * This method is like `_.difference` except that it accepts `comparator`
             * which is invoked to compare elements of `array` to `values`. The order and
             * references of result values are determined by the first array. The comparator
             * is invoked with two arguments: (arrVal, othVal).
             *
             * **Note:** Unlike `_.pullAllWith`, this method returns a new array.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {...Array} [values] The values to exclude.
             * @param {Function} [comparator] The comparator invoked per element.
             * @returns {Array} Returns the new array of filtered values.
             * @example
             *
             * var objects = [{ 'x': 1, 'y': 2 }, { 'x': 2, 'y': 1 }];
             *
             * _.differenceWith(objects, [{ 'x': 1, 'y': 2 }], _.isEqual);
             * // => [{ 'x': 2, 'y': 1 }]
             */
            var differenceWith = baseRest(function (array, values) {
                var comparator = last(values);
                if (isArrayLikeObject(comparator)) {
                    comparator = undefined;
                }
                return isArrayLikeObject(array)
                    ? baseDifference(array, baseFlatten(values, 1, isArrayLikeObject, true), undefined, comparator)
                    : [];
            });
            /**
             * Creates a slice of `array` with `n` elements dropped from the beginning.
             *
             * @static
             * @memberOf _
             * @since 0.5.0
             * @category Array
             * @param {Array} array The array to query.
             * @param {number} [n=1] The number of elements to drop.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {Array} Returns the slice of `array`.
             * @example
             *
             * _.drop([1, 2, 3]);
             * // => [2, 3]
             *
             * _.drop([1, 2, 3], 2);
             * // => [3]
             *
             * _.drop([1, 2, 3], 5);
             * // => []
             *
             * _.drop([1, 2, 3], 0);
             * // => [1, 2, 3]
             */
            function drop(array, n, guard) {
                var length = array == null ? 0 : array.length;
                if (!length) {
                    return [];
                }
                n = (guard || n === undefined) ? 1 : toInteger(n);
                return baseSlice(array, n < 0 ? 0 : n, length);
            }
            /**
             * Creates a slice of `array` with `n` elements dropped from the end.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Array
             * @param {Array} array The array to query.
             * @param {number} [n=1] The number of elements to drop.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {Array} Returns the slice of `array`.
             * @example
             *
             * _.dropRight([1, 2, 3]);
             * // => [1, 2]
             *
             * _.dropRight([1, 2, 3], 2);
             * // => [1]
             *
             * _.dropRight([1, 2, 3], 5);
             * // => []
             *
             * _.dropRight([1, 2, 3], 0);
             * // => [1, 2, 3]
             */
            function dropRight(array, n, guard) {
                var length = array == null ? 0 : array.length;
                if (!length) {
                    return [];
                }
                n = (guard || n === undefined) ? 1 : toInteger(n);
                n = length - n;
                return baseSlice(array, 0, n < 0 ? 0 : n);
            }
            /**
             * Creates a slice of `array` excluding elements dropped from the end.
             * Elements are dropped until `predicate` returns falsey. The predicate is
             * invoked with three arguments: (value, index, array).
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Array
             * @param {Array} array The array to query.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the slice of `array`.
             * @example
             *
             * var users = [
             *   { 'user': 'barney',  'active': true },
             *   { 'user': 'fred',    'active': false },
             *   { 'user': 'pebbles', 'active': false }
             * ];
             *
             * _.dropRightWhile(users, function(o) { return !o.active; });
             * // => objects for ['barney']
             *
             * // The `_.matches` iteratee shorthand.
             * _.dropRightWhile(users, { 'user': 'pebbles', 'active': false });
             * // => objects for ['barney', 'fred']
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.dropRightWhile(users, ['active', false]);
             * // => objects for ['barney']
             *
             * // The `_.property` iteratee shorthand.
             * _.dropRightWhile(users, 'active');
             * // => objects for ['barney', 'fred', 'pebbles']
             */
            function dropRightWhile(array, predicate) {
                return (array && array.length)
                    ? baseWhile(array, getIteratee(predicate, 3), true, true)
                    : [];
            }
            /**
             * Creates a slice of `array` excluding elements dropped from the beginning.
             * Elements are dropped until `predicate` returns falsey. The predicate is
             * invoked with three arguments: (value, index, array).
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Array
             * @param {Array} array The array to query.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the slice of `array`.
             * @example
             *
             * var users = [
             *   { 'user': 'barney',  'active': false },
             *   { 'user': 'fred',    'active': false },
             *   { 'user': 'pebbles', 'active': true }
             * ];
             *
             * _.dropWhile(users, function(o) { return !o.active; });
             * // => objects for ['pebbles']
             *
             * // The `_.matches` iteratee shorthand.
             * _.dropWhile(users, { 'user': 'barney', 'active': false });
             * // => objects for ['fred', 'pebbles']
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.dropWhile(users, ['active', false]);
             * // => objects for ['pebbles']
             *
             * // The `_.property` iteratee shorthand.
             * _.dropWhile(users, 'active');
             * // => objects for ['barney', 'fred', 'pebbles']
             */
            function dropWhile(array, predicate) {
                return (array && array.length)
                    ? baseWhile(array, getIteratee(predicate, 3), true)
                    : [];
            }
            /**
             * Fills elements of `array` with `value` from `start` up to, but not
             * including, `end`.
             *
             * **Note:** This method mutates `array`.
             *
             * @static
             * @memberOf _
             * @since 3.2.0
             * @category Array
             * @param {Array} array The array to fill.
             * @param {*} value The value to fill `array` with.
             * @param {number} [start=0] The start position.
             * @param {number} [end=array.length] The end position.
             * @returns {Array} Returns `array`.
             * @example
             *
             * var array = [1, 2, 3];
             *
             * _.fill(array, 'a');
             * console.log(array);
             * // => ['a', 'a', 'a']
             *
             * _.fill(Array(3), 2);
             * // => [2, 2, 2]
             *
             * _.fill([4, 6, 8, 10], '*', 1, 3);
             * // => [4, '*', '*', 10]
             */
            function fill(array, value, start, end) {
                var length = array == null ? 0 : array.length;
                if (!length) {
                    return [];
                }
                if (start && typeof start != 'number' && isIterateeCall(array, value, start)) {
                    start = 0;
                    end = length;
                }
                return baseFill(array, value, start, end);
            }
            /**
             * This method is like `_.find` except that it returns the index of the first
             * element `predicate` returns truthy for instead of the element itself.
             *
             * @static
             * @memberOf _
             * @since 1.1.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @param {number} [fromIndex=0] The index to search from.
             * @returns {number} Returns the index of the found element, else `-1`.
             * @example
             *
             * var users = [
             *   { 'user': 'barney',  'active': false },
             *   { 'user': 'fred',    'active': false },
             *   { 'user': 'pebbles', 'active': true }
             * ];
             *
             * _.findIndex(users, function(o) { return o.user == 'barney'; });
             * // => 0
             *
             * // The `_.matches` iteratee shorthand.
             * _.findIndex(users, { 'user': 'fred', 'active': false });
             * // => 1
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.findIndex(users, ['active', false]);
             * // => 0
             *
             * // The `_.property` iteratee shorthand.
             * _.findIndex(users, 'active');
             * // => 2
             */
            function findIndex(array, predicate, fromIndex) {
                var length = array == null ? 0 : array.length;
                if (!length) {
                    return -1;
                }
                var index = fromIndex == null ? 0 : toInteger(fromIndex);
                if (index < 0) {
                    index = nativeMax(length + index, 0);
                }
                return baseFindIndex(array, getIteratee(predicate, 3), index);
            }
            /**
             * This method is like `_.findIndex` except that it iterates over elements
             * of `collection` from right to left.
             *
             * @static
             * @memberOf _
             * @since 2.0.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @param {number} [fromIndex=array.length-1] The index to search from.
             * @returns {number} Returns the index of the found element, else `-1`.
             * @example
             *
             * var users = [
             *   { 'user': 'barney',  'active': true },
             *   { 'user': 'fred',    'active': false },
             *   { 'user': 'pebbles', 'active': false }
             * ];
             *
             * _.findLastIndex(users, function(o) { return o.user == 'pebbles'; });
             * // => 2
             *
             * // The `_.matches` iteratee shorthand.
             * _.findLastIndex(users, { 'user': 'barney', 'active': true });
             * // => 0
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.findLastIndex(users, ['active', false]);
             * // => 2
             *
             * // The `_.property` iteratee shorthand.
             * _.findLastIndex(users, 'active');
             * // => 0
             */
            function findLastIndex(array, predicate, fromIndex) {
                var length = array == null ? 0 : array.length;
                if (!length) {
                    return -1;
                }
                var index = length - 1;
                if (fromIndex !== undefined) {
                    index = toInteger(fromIndex);
                    index = fromIndex < 0
                        ? nativeMax(length + index, 0)
                        : nativeMin(index, length - 1);
                }
                return baseFindIndex(array, getIteratee(predicate, 3), index, true);
            }
            /**
             * Flattens `array` a single level deep.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {Array} array The array to flatten.
             * @returns {Array} Returns the new flattened array.
             * @example
             *
             * _.flatten([1, [2, [3, [4]], 5]]);
             * // => [1, 2, [3, [4]], 5]
             */
            function flatten(array) {
                var length = array == null ? 0 : array.length;
                return length ? baseFlatten(array, 1) : [];
            }
            /**
             * Recursively flattens `array`.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Array
             * @param {Array} array The array to flatten.
             * @returns {Array} Returns the new flattened array.
             * @example
             *
             * _.flattenDeep([1, [2, [3, [4]], 5]]);
             * // => [1, 2, 3, 4, 5]
             */
            function flattenDeep(array) {
                var length = array == null ? 0 : array.length;
                return length ? baseFlatten(array, INFINITY) : [];
            }
            /**
             * Recursively flatten `array` up to `depth` times.
             *
             * @static
             * @memberOf _
             * @since 4.4.0
             * @category Array
             * @param {Array} array The array to flatten.
             * @param {number} [depth=1] The maximum recursion depth.
             * @returns {Array} Returns the new flattened array.
             * @example
             *
             * var array = [1, [2, [3, [4]], 5]];
             *
             * _.flattenDepth(array, 1);
             * // => [1, 2, [3, [4]], 5]
             *
             * _.flattenDepth(array, 2);
             * // => [1, 2, 3, [4], 5]
             */
            function flattenDepth(array, depth) {
                var length = array == null ? 0 : array.length;
                if (!length) {
                    return [];
                }
                depth = depth === undefined ? 1 : toInteger(depth);
                return baseFlatten(array, depth);
            }
            /**
             * The inverse of `_.toPairs`; this method returns an object composed
             * from key-value `pairs`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} pairs The key-value pairs.
             * @returns {Object} Returns the new object.
             * @example
             *
             * _.fromPairs([['a', 1], ['b', 2]]);
             * // => { 'a': 1, 'b': 2 }
             */
            function fromPairs(pairs) {
                var index = -1, length = pairs == null ? 0 : pairs.length, result = {};
                while (++index < length) {
                    var pair = pairs[index];
                    result[pair[0]] = pair[1];
                }
                return result;
            }
            /**
             * Gets the first element of `array`.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @alias first
             * @category Array
             * @param {Array} array The array to query.
             * @returns {*} Returns the first element of `array`.
             * @example
             *
             * _.head([1, 2, 3]);
             * // => 1
             *
             * _.head([]);
             * // => undefined
             */
            function head(array) {
                return (array && array.length) ? array[0] : undefined;
            }
            /**
             * Gets the index at which the first occurrence of `value` is found in `array`
             * using [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
             * for equality comparisons. If `fromIndex` is negative, it's used as the
             * offset from the end of `array`.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {*} value The value to search for.
             * @param {number} [fromIndex=0] The index to search from.
             * @returns {number} Returns the index of the matched value, else `-1`.
             * @example
             *
             * _.indexOf([1, 2, 1, 2], 2);
             * // => 1
             *
             * // Search from the `fromIndex`.
             * _.indexOf([1, 2, 1, 2], 2, 2);
             * // => 3
             */
            function indexOf(array, value, fromIndex) {
                var length = array == null ? 0 : array.length;
                if (!length) {
                    return -1;
                }
                var index = fromIndex == null ? 0 : toInteger(fromIndex);
                if (index < 0) {
                    index = nativeMax(length + index, 0);
                }
                return baseIndexOf(array, value, index);
            }
            /**
             * Gets all but the last element of `array`.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {Array} array The array to query.
             * @returns {Array} Returns the slice of `array`.
             * @example
             *
             * _.initial([1, 2, 3]);
             * // => [1, 2]
             */
            function initial(array) {
                var length = array == null ? 0 : array.length;
                return length ? baseSlice(array, 0, -1) : [];
            }
            /**
             * Creates an array of unique values that are included in all given arrays
             * using [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
             * for equality comparisons. The order and references of result values are
             * determined by the first array.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {...Array} [arrays] The arrays to inspect.
             * @returns {Array} Returns the new array of intersecting values.
             * @example
             *
             * _.intersection([2, 1], [2, 3]);
             * // => [2]
             */
            var intersection = baseRest(function (arrays) {
                var mapped = arrayMap(arrays, castArrayLikeObject);
                return (mapped.length && mapped[0] === arrays[0])
                    ? baseIntersection(mapped)
                    : [];
            });
            /**
             * This method is like `_.intersection` except that it accepts `iteratee`
             * which is invoked for each element of each `arrays` to generate the criterion
             * by which they're compared. The order and references of result values are
             * determined by the first array. The iteratee is invoked with one argument:
             * (value).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {...Array} [arrays] The arrays to inspect.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {Array} Returns the new array of intersecting values.
             * @example
             *
             * _.intersectionBy([2.1, 1.2], [2.3, 3.4], Math.floor);
             * // => [2.1]
             *
             * // The `_.property` iteratee shorthand.
             * _.intersectionBy([{ 'x': 1 }], [{ 'x': 2 }, { 'x': 1 }], 'x');
             * // => [{ 'x': 1 }]
             */
            var intersectionBy = baseRest(function (arrays) {
                var iteratee = last(arrays), mapped = arrayMap(arrays, castArrayLikeObject);
                if (iteratee === last(mapped)) {
                    iteratee = undefined;
                }
                else {
                    mapped.pop();
                }
                return (mapped.length && mapped[0] === arrays[0])
                    ? baseIntersection(mapped, getIteratee(iteratee, 2))
                    : [];
            });
            /**
             * This method is like `_.intersection` except that it accepts `comparator`
             * which is invoked to compare elements of `arrays`. The order and references
             * of result values are determined by the first array. The comparator is
             * invoked with two arguments: (arrVal, othVal).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {...Array} [arrays] The arrays to inspect.
             * @param {Function} [comparator] The comparator invoked per element.
             * @returns {Array} Returns the new array of intersecting values.
             * @example
             *
             * var objects = [{ 'x': 1, 'y': 2 }, { 'x': 2, 'y': 1 }];
             * var others = [{ 'x': 1, 'y': 1 }, { 'x': 1, 'y': 2 }];
             *
             * _.intersectionWith(objects, others, _.isEqual);
             * // => [{ 'x': 1, 'y': 2 }]
             */
            var intersectionWith = baseRest(function (arrays) {
                var comparator = last(arrays), mapped = arrayMap(arrays, castArrayLikeObject);
                comparator = typeof comparator == 'function' ? comparator : undefined;
                if (comparator) {
                    mapped.pop();
                }
                return (mapped.length && mapped[0] === arrays[0])
                    ? baseIntersection(mapped, undefined, comparator)
                    : [];
            });
            /**
             * Converts all elements in `array` into a string separated by `separator`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to convert.
             * @param {string} [separator=','] The element separator.
             * @returns {string} Returns the joined string.
             * @example
             *
             * _.join(['a', 'b', 'c'], '~');
             * // => 'a~b~c'
             */
            function join(array, separator) {
                return array == null ? '' : nativeJoin.call(array, separator);
            }
            /**
             * Gets the last element of `array`.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {Array} array The array to query.
             * @returns {*} Returns the last element of `array`.
             * @example
             *
             * _.last([1, 2, 3]);
             * // => 3
             */
            function last(array) {
                var length = array == null ? 0 : array.length;
                return length ? array[length - 1] : undefined;
            }
            /**
             * This method is like `_.indexOf` except that it iterates over elements of
             * `array` from right to left.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {*} value The value to search for.
             * @param {number} [fromIndex=array.length-1] The index to search from.
             * @returns {number} Returns the index of the matched value, else `-1`.
             * @example
             *
             * _.lastIndexOf([1, 2, 1, 2], 2);
             * // => 3
             *
             * // Search from the `fromIndex`.
             * _.lastIndexOf([1, 2, 1, 2], 2, 2);
             * // => 1
             */
            function lastIndexOf(array, value, fromIndex) {
                var length = array == null ? 0 : array.length;
                if (!length) {
                    return -1;
                }
                var index = length;
                if (fromIndex !== undefined) {
                    index = toInteger(fromIndex);
                    index = index < 0 ? nativeMax(length + index, 0) : nativeMin(index, length - 1);
                }
                return value === value
                    ? strictLastIndexOf(array, value, index)
                    : baseFindIndex(array, baseIsNaN, index, true);
            }
            /**
             * Gets the element at index `n` of `array`. If `n` is negative, the nth
             * element from the end is returned.
             *
             * @static
             * @memberOf _
             * @since 4.11.0
             * @category Array
             * @param {Array} array The array to query.
             * @param {number} [n=0] The index of the element to return.
             * @returns {*} Returns the nth element of `array`.
             * @example
             *
             * var array = ['a', 'b', 'c', 'd'];
             *
             * _.nth(array, 1);
             * // => 'b'
             *
             * _.nth(array, -2);
             * // => 'c';
             */
            function nth(array, n) {
                return (array && array.length) ? baseNth(array, toInteger(n)) : undefined;
            }
            /**
             * Removes all given values from `array` using
             * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
             * for equality comparisons.
             *
             * **Note:** Unlike `_.without`, this method mutates `array`. Use `_.remove`
             * to remove elements from an array by predicate.
             *
             * @static
             * @memberOf _
             * @since 2.0.0
             * @category Array
             * @param {Array} array The array to modify.
             * @param {...*} [values] The values to remove.
             * @returns {Array} Returns `array`.
             * @example
             *
             * var array = ['a', 'b', 'c', 'a', 'b', 'c'];
             *
             * _.pull(array, 'a', 'c');
             * console.log(array);
             * // => ['b', 'b']
             */
            var pull = baseRest(pullAll);
            /**
             * This method is like `_.pull` except that it accepts an array of values to remove.
             *
             * **Note:** Unlike `_.difference`, this method mutates `array`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to modify.
             * @param {Array} values The values to remove.
             * @returns {Array} Returns `array`.
             * @example
             *
             * var array = ['a', 'b', 'c', 'a', 'b', 'c'];
             *
             * _.pullAll(array, ['a', 'c']);
             * console.log(array);
             * // => ['b', 'b']
             */
            function pullAll(array, values) {
                return (array && array.length && values && values.length)
                    ? basePullAll(array, values)
                    : array;
            }
            /**
             * This method is like `_.pullAll` except that it accepts `iteratee` which is
             * invoked for each element of `array` and `values` to generate the criterion
             * by which they're compared. The iteratee is invoked with one argument: (value).
             *
             * **Note:** Unlike `_.differenceBy`, this method mutates `array`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to modify.
             * @param {Array} values The values to remove.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {Array} Returns `array`.
             * @example
             *
             * var array = [{ 'x': 1 }, { 'x': 2 }, { 'x': 3 }, { 'x': 1 }];
             *
             * _.pullAllBy(array, [{ 'x': 1 }, { 'x': 3 }], 'x');
             * console.log(array);
             * // => [{ 'x': 2 }]
             */
            function pullAllBy(array, values, iteratee) {
                return (array && array.length && values && values.length)
                    ? basePullAll(array, values, getIteratee(iteratee, 2))
                    : array;
            }
            /**
             * This method is like `_.pullAll` except that it accepts `comparator` which
             * is invoked to compare elements of `array` to `values`. The comparator is
             * invoked with two arguments: (arrVal, othVal).
             *
             * **Note:** Unlike `_.differenceWith`, this method mutates `array`.
             *
             * @static
             * @memberOf _
             * @since 4.6.0
             * @category Array
             * @param {Array} array The array to modify.
             * @param {Array} values The values to remove.
             * @param {Function} [comparator] The comparator invoked per element.
             * @returns {Array} Returns `array`.
             * @example
             *
             * var array = [{ 'x': 1, 'y': 2 }, { 'x': 3, 'y': 4 }, { 'x': 5, 'y': 6 }];
             *
             * _.pullAllWith(array, [{ 'x': 3, 'y': 4 }], _.isEqual);
             * console.log(array);
             * // => [{ 'x': 1, 'y': 2 }, { 'x': 5, 'y': 6 }]
             */
            function pullAllWith(array, values, comparator) {
                return (array && array.length && values && values.length)
                    ? basePullAll(array, values, undefined, comparator)
                    : array;
            }
            /**
             * Removes elements from `array` corresponding to `indexes` and returns an
             * array of removed elements.
             *
             * **Note:** Unlike `_.at`, this method mutates `array`.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Array
             * @param {Array} array The array to modify.
             * @param {...(number|number[])} [indexes] The indexes of elements to remove.
             * @returns {Array} Returns the new array of removed elements.
             * @example
             *
             * var array = ['a', 'b', 'c', 'd'];
             * var pulled = _.pullAt(array, [1, 3]);
             *
             * console.log(array);
             * // => ['a', 'c']
             *
             * console.log(pulled);
             * // => ['b', 'd']
             */
            var pullAt = flatRest(function (array, indexes) {
                var length = array == null ? 0 : array.length, result = baseAt(array, indexes);
                basePullAt(array, arrayMap(indexes, function (index) {
                    return isIndex(index, length) ? +index : index;
                }).sort(compareAscending));
                return result;
            });
            /**
             * Removes all elements from `array` that `predicate` returns truthy for
             * and returns an array of the removed elements. The predicate is invoked
             * with three arguments: (value, index, array).
             *
             * **Note:** Unlike `_.filter`, this method mutates `array`. Use `_.pull`
             * to pull elements from an array by value.
             *
             * @static
             * @memberOf _
             * @since 2.0.0
             * @category Array
             * @param {Array} array The array to modify.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the new array of removed elements.
             * @example
             *
             * var array = [1, 2, 3, 4];
             * var evens = _.remove(array, function(n) {
             *   return n % 2 == 0;
             * });
             *
             * console.log(array);
             * // => [1, 3]
             *
             * console.log(evens);
             * // => [2, 4]
             */
            function remove(array, predicate) {
                var result = [];
                if (!(array && array.length)) {
                    return result;
                }
                var index = -1, indexes = [], length = array.length;
                predicate = getIteratee(predicate, 3);
                while (++index < length) {
                    var value = array[index];
                    if (predicate(value, index, array)) {
                        result.push(value);
                        indexes.push(index);
                    }
                }
                basePullAt(array, indexes);
                return result;
            }
            /**
             * Reverses `array` so that the first element becomes the last, the second
             * element becomes the second to last, and so on.
             *
             * **Note:** This method mutates `array` and is based on
             * [`Array#reverse`](https://mdn.io/Array/reverse).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to modify.
             * @returns {Array} Returns `array`.
             * @example
             *
             * var array = [1, 2, 3];
             *
             * _.reverse(array);
             * // => [3, 2, 1]
             *
             * console.log(array);
             * // => [3, 2, 1]
             */
            function reverse(array) {
                return array == null ? array : nativeReverse.call(array);
            }
            /**
             * Creates a slice of `array` from `start` up to, but not including, `end`.
             *
             * **Note:** This method is used instead of
             * [`Array#slice`](https://mdn.io/Array/slice) to ensure dense arrays are
             * returned.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Array
             * @param {Array} array The array to slice.
             * @param {number} [start=0] The start position.
             * @param {number} [end=array.length] The end position.
             * @returns {Array} Returns the slice of `array`.
             */
            function slice(array, start, end) {
                var length = array == null ? 0 : array.length;
                if (!length) {
                    return [];
                }
                if (end && typeof end != 'number' && isIterateeCall(array, start, end)) {
                    start = 0;
                    end = length;
                }
                else {
                    start = start == null ? 0 : toInteger(start);
                    end = end === undefined ? length : toInteger(end);
                }
                return baseSlice(array, start, end);
            }
            /**
             * Uses a binary search to determine the lowest index at which `value`
             * should be inserted into `array` in order to maintain its sort order.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {Array} array The sorted array to inspect.
             * @param {*} value The value to evaluate.
             * @returns {number} Returns the index at which `value` should be inserted
             *  into `array`.
             * @example
             *
             * _.sortedIndex([30, 50], 40);
             * // => 1
             */
            function sortedIndex(array, value) {
                return baseSortedIndex(array, value);
            }
            /**
             * This method is like `_.sortedIndex` except that it accepts `iteratee`
             * which is invoked for `value` and each element of `array` to compute their
             * sort ranking. The iteratee is invoked with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The sorted array to inspect.
             * @param {*} value The value to evaluate.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {number} Returns the index at which `value` should be inserted
             *  into `array`.
             * @example
             *
             * var objects = [{ 'x': 4 }, { 'x': 5 }];
             *
             * _.sortedIndexBy(objects, { 'x': 4 }, function(o) { return o.x; });
             * // => 0
             *
             * // The `_.property` iteratee shorthand.
             * _.sortedIndexBy(objects, { 'x': 4 }, 'x');
             * // => 0
             */
            function sortedIndexBy(array, value, iteratee) {
                return baseSortedIndexBy(array, value, getIteratee(iteratee, 2));
            }
            /**
             * This method is like `_.indexOf` except that it performs a binary
             * search on a sorted `array`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {*} value The value to search for.
             * @returns {number} Returns the index of the matched value, else `-1`.
             * @example
             *
             * _.sortedIndexOf([4, 5, 5, 5, 6], 5);
             * // => 1
             */
            function sortedIndexOf(array, value) {
                var length = array == null ? 0 : array.length;
                if (length) {
                    var index = baseSortedIndex(array, value);
                    if (index < length && eq(array[index], value)) {
                        return index;
                    }
                }
                return -1;
            }
            /**
             * This method is like `_.sortedIndex` except that it returns the highest
             * index at which `value` should be inserted into `array` in order to
             * maintain its sort order.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Array
             * @param {Array} array The sorted array to inspect.
             * @param {*} value The value to evaluate.
             * @returns {number} Returns the index at which `value` should be inserted
             *  into `array`.
             * @example
             *
             * _.sortedLastIndex([4, 5, 5, 5, 6], 5);
             * // => 4
             */
            function sortedLastIndex(array, value) {
                return baseSortedIndex(array, value, true);
            }
            /**
             * This method is like `_.sortedLastIndex` except that it accepts `iteratee`
             * which is invoked for `value` and each element of `array` to compute their
             * sort ranking. The iteratee is invoked with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The sorted array to inspect.
             * @param {*} value The value to evaluate.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {number} Returns the index at which `value` should be inserted
             *  into `array`.
             * @example
             *
             * var objects = [{ 'x': 4 }, { 'x': 5 }];
             *
             * _.sortedLastIndexBy(objects, { 'x': 4 }, function(o) { return o.x; });
             * // => 1
             *
             * // The `_.property` iteratee shorthand.
             * _.sortedLastIndexBy(objects, { 'x': 4 }, 'x');
             * // => 1
             */
            function sortedLastIndexBy(array, value, iteratee) {
                return baseSortedIndexBy(array, value, getIteratee(iteratee, 2), true);
            }
            /**
             * This method is like `_.lastIndexOf` except that it performs a binary
             * search on a sorted `array`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {*} value The value to search for.
             * @returns {number} Returns the index of the matched value, else `-1`.
             * @example
             *
             * _.sortedLastIndexOf([4, 5, 5, 5, 6], 5);
             * // => 3
             */
            function sortedLastIndexOf(array, value) {
                var length = array == null ? 0 : array.length;
                if (length) {
                    var index = baseSortedIndex(array, value, true) - 1;
                    if (eq(array[index], value)) {
                        return index;
                    }
                }
                return -1;
            }
            /**
             * This method is like `_.uniq` except that it's designed and optimized
             * for sorted arrays.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @returns {Array} Returns the new duplicate free array.
             * @example
             *
             * _.sortedUniq([1, 1, 2]);
             * // => [1, 2]
             */
            function sortedUniq(array) {
                return (array && array.length)
                    ? baseSortedUniq(array)
                    : [];
            }
            /**
             * This method is like `_.uniqBy` except that it's designed and optimized
             * for sorted arrays.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {Function} [iteratee] The iteratee invoked per element.
             * @returns {Array} Returns the new duplicate free array.
             * @example
             *
             * _.sortedUniqBy([1.1, 1.2, 2.3, 2.4], Math.floor);
             * // => [1.1, 2.3]
             */
            function sortedUniqBy(array, iteratee) {
                return (array && array.length)
                    ? baseSortedUniq(array, getIteratee(iteratee, 2))
                    : [];
            }
            /**
             * Gets all but the first element of `array`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to query.
             * @returns {Array} Returns the slice of `array`.
             * @example
             *
             * _.tail([1, 2, 3]);
             * // => [2, 3]
             */
            function tail(array) {
                var length = array == null ? 0 : array.length;
                return length ? baseSlice(array, 1, length) : [];
            }
            /**
             * Creates a slice of `array` with `n` elements taken from the beginning.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {Array} array The array to query.
             * @param {number} [n=1] The number of elements to take.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {Array} Returns the slice of `array`.
             * @example
             *
             * _.take([1, 2, 3]);
             * // => [1]
             *
             * _.take([1, 2, 3], 2);
             * // => [1, 2]
             *
             * _.take([1, 2, 3], 5);
             * // => [1, 2, 3]
             *
             * _.take([1, 2, 3], 0);
             * // => []
             */
            function take(array, n, guard) {
                if (!(array && array.length)) {
                    return [];
                }
                n = (guard || n === undefined) ? 1 : toInteger(n);
                return baseSlice(array, 0, n < 0 ? 0 : n);
            }
            /**
             * Creates a slice of `array` with `n` elements taken from the end.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Array
             * @param {Array} array The array to query.
             * @param {number} [n=1] The number of elements to take.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {Array} Returns the slice of `array`.
             * @example
             *
             * _.takeRight([1, 2, 3]);
             * // => [3]
             *
             * _.takeRight([1, 2, 3], 2);
             * // => [2, 3]
             *
             * _.takeRight([1, 2, 3], 5);
             * // => [1, 2, 3]
             *
             * _.takeRight([1, 2, 3], 0);
             * // => []
             */
            function takeRight(array, n, guard) {
                var length = array == null ? 0 : array.length;
                if (!length) {
                    return [];
                }
                n = (guard || n === undefined) ? 1 : toInteger(n);
                n = length - n;
                return baseSlice(array, n < 0 ? 0 : n, length);
            }
            /**
             * Creates a slice of `array` with elements taken from the end. Elements are
             * taken until `predicate` returns falsey. The predicate is invoked with
             * three arguments: (value, index, array).
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Array
             * @param {Array} array The array to query.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the slice of `array`.
             * @example
             *
             * var users = [
             *   { 'user': 'barney',  'active': true },
             *   { 'user': 'fred',    'active': false },
             *   { 'user': 'pebbles', 'active': false }
             * ];
             *
             * _.takeRightWhile(users, function(o) { return !o.active; });
             * // => objects for ['fred', 'pebbles']
             *
             * // The `_.matches` iteratee shorthand.
             * _.takeRightWhile(users, { 'user': 'pebbles', 'active': false });
             * // => objects for ['pebbles']
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.takeRightWhile(users, ['active', false]);
             * // => objects for ['fred', 'pebbles']
             *
             * // The `_.property` iteratee shorthand.
             * _.takeRightWhile(users, 'active');
             * // => []
             */
            function takeRightWhile(array, predicate) {
                return (array && array.length)
                    ? baseWhile(array, getIteratee(predicate, 3), false, true)
                    : [];
            }
            /**
             * Creates a slice of `array` with elements taken from the beginning. Elements
             * are taken until `predicate` returns falsey. The predicate is invoked with
             * three arguments: (value, index, array).
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Array
             * @param {Array} array The array to query.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the slice of `array`.
             * @example
             *
             * var users = [
             *   { 'user': 'barney',  'active': false },
             *   { 'user': 'fred',    'active': false },
             *   { 'user': 'pebbles', 'active': true }
             * ];
             *
             * _.takeWhile(users, function(o) { return !o.active; });
             * // => objects for ['barney', 'fred']
             *
             * // The `_.matches` iteratee shorthand.
             * _.takeWhile(users, { 'user': 'barney', 'active': false });
             * // => objects for ['barney']
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.takeWhile(users, ['active', false]);
             * // => objects for ['barney', 'fred']
             *
             * // The `_.property` iteratee shorthand.
             * _.takeWhile(users, 'active');
             * // => []
             */
            function takeWhile(array, predicate) {
                return (array && array.length)
                    ? baseWhile(array, getIteratee(predicate, 3))
                    : [];
            }
            /**
             * Creates an array of unique values, in order, from all given arrays using
             * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
             * for equality comparisons.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {...Array} [arrays] The arrays to inspect.
             * @returns {Array} Returns the new array of combined values.
             * @example
             *
             * _.union([2], [1, 2]);
             * // => [2, 1]
             */
            var union = baseRest(function (arrays) {
                return baseUniq(baseFlatten(arrays, 1, isArrayLikeObject, true));
            });
            /**
             * This method is like `_.union` except that it accepts `iteratee` which is
             * invoked for each element of each `arrays` to generate the criterion by
             * which uniqueness is computed. Result values are chosen from the first
             * array in which the value occurs. The iteratee is invoked with one argument:
             * (value).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {...Array} [arrays] The arrays to inspect.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {Array} Returns the new array of combined values.
             * @example
             *
             * _.unionBy([2.1], [1.2, 2.3], Math.floor);
             * // => [2.1, 1.2]
             *
             * // The `_.property` iteratee shorthand.
             * _.unionBy([{ 'x': 1 }], [{ 'x': 2 }, { 'x': 1 }], 'x');
             * // => [{ 'x': 1 }, { 'x': 2 }]
             */
            var unionBy = baseRest(function (arrays) {
                var iteratee = last(arrays);
                if (isArrayLikeObject(iteratee)) {
                    iteratee = undefined;
                }
                return baseUniq(baseFlatten(arrays, 1, isArrayLikeObject, true), getIteratee(iteratee, 2));
            });
            /**
             * This method is like `_.union` except that it accepts `comparator` which
             * is invoked to compare elements of `arrays`. Result values are chosen from
             * the first array in which the value occurs. The comparator is invoked
             * with two arguments: (arrVal, othVal).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {...Array} [arrays] The arrays to inspect.
             * @param {Function} [comparator] The comparator invoked per element.
             * @returns {Array} Returns the new array of combined values.
             * @example
             *
             * var objects = [{ 'x': 1, 'y': 2 }, { 'x': 2, 'y': 1 }];
             * var others = [{ 'x': 1, 'y': 1 }, { 'x': 1, 'y': 2 }];
             *
             * _.unionWith(objects, others, _.isEqual);
             * // => [{ 'x': 1, 'y': 2 }, { 'x': 2, 'y': 1 }, { 'x': 1, 'y': 1 }]
             */
            var unionWith = baseRest(function (arrays) {
                var comparator = last(arrays);
                comparator = typeof comparator == 'function' ? comparator : undefined;
                return baseUniq(baseFlatten(arrays, 1, isArrayLikeObject, true), undefined, comparator);
            });
            /**
             * Creates a duplicate-free version of an array, using
             * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
             * for equality comparisons, in which only the first occurrence of each element
             * is kept. The order of result values is determined by the order they occur
             * in the array.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @returns {Array} Returns the new duplicate free array.
             * @example
             *
             * _.uniq([2, 1, 2]);
             * // => [2, 1]
             */
            function uniq(array) {
                return (array && array.length) ? baseUniq(array) : [];
            }
            /**
             * This method is like `_.uniq` except that it accepts `iteratee` which is
             * invoked for each element in `array` to generate the criterion by which
             * uniqueness is computed. The order of result values is determined by the
             * order they occur in the array. The iteratee is invoked with one argument:
             * (value).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {Array} Returns the new duplicate free array.
             * @example
             *
             * _.uniqBy([2.1, 1.2, 2.3], Math.floor);
             * // => [2.1, 1.2]
             *
             * // The `_.property` iteratee shorthand.
             * _.uniqBy([{ 'x': 1 }, { 'x': 2 }, { 'x': 1 }], 'x');
             * // => [{ 'x': 1 }, { 'x': 2 }]
             */
            function uniqBy(array, iteratee) {
                return (array && array.length) ? baseUniq(array, getIteratee(iteratee, 2)) : [];
            }
            /**
             * This method is like `_.uniq` except that it accepts `comparator` which
             * is invoked to compare elements of `array`. The order of result values is
             * determined by the order they occur in the array.The comparator is invoked
             * with two arguments: (arrVal, othVal).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {Function} [comparator] The comparator invoked per element.
             * @returns {Array} Returns the new duplicate free array.
             * @example
             *
             * var objects = [{ 'x': 1, 'y': 2 }, { 'x': 2, 'y': 1 }, { 'x': 1, 'y': 2 }];
             *
             * _.uniqWith(objects, _.isEqual);
             * // => [{ 'x': 1, 'y': 2 }, { 'x': 2, 'y': 1 }]
             */
            function uniqWith(array, comparator) {
                comparator = typeof comparator == 'function' ? comparator : undefined;
                return (array && array.length) ? baseUniq(array, undefined, comparator) : [];
            }
            /**
             * This method is like `_.zip` except that it accepts an array of grouped
             * elements and creates an array regrouping the elements to their pre-zip
             * configuration.
             *
             * @static
             * @memberOf _
             * @since 1.2.0
             * @category Array
             * @param {Array} array The array of grouped elements to process.
             * @returns {Array} Returns the new array of regrouped elements.
             * @example
             *
             * var zipped = _.zip(['a', 'b'], [1, 2], [true, false]);
             * // => [['a', 1, true], ['b', 2, false]]
             *
             * _.unzip(zipped);
             * // => [['a', 'b'], [1, 2], [true, false]]
             */
            function unzip(array) {
                if (!(array && array.length)) {
                    return [];
                }
                var length = 0;
                array = arrayFilter(array, function (group) {
                    if (isArrayLikeObject(group)) {
                        length = nativeMax(group.length, length);
                        return true;
                    }
                });
                return baseTimes(length, function (index) {
                    return arrayMap(array, baseProperty(index));
                });
            }
            /**
             * This method is like `_.unzip` except that it accepts `iteratee` to specify
             * how regrouped values should be combined. The iteratee is invoked with the
             * elements of each group: (...group).
             *
             * @static
             * @memberOf _
             * @since 3.8.0
             * @category Array
             * @param {Array} array The array of grouped elements to process.
             * @param {Function} [iteratee=_.identity] The function to combine
             *  regrouped values.
             * @returns {Array} Returns the new array of regrouped elements.
             * @example
             *
             * var zipped = _.zip([1, 2], [10, 20], [100, 200]);
             * // => [[1, 10, 100], [2, 20, 200]]
             *
             * _.unzipWith(zipped, _.add);
             * // => [3, 30, 300]
             */
            function unzipWith(array, iteratee) {
                if (!(array && array.length)) {
                    return [];
                }
                var result = unzip(array);
                if (iteratee == null) {
                    return result;
                }
                return arrayMap(result, function (group) {
                    return apply(iteratee, undefined, group);
                });
            }
            /**
             * Creates an array excluding all given values using
             * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
             * for equality comparisons.
             *
             * **Note:** Unlike `_.pull`, this method returns a new array.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {Array} array The array to inspect.
             * @param {...*} [values] The values to exclude.
             * @returns {Array} Returns the new array of filtered values.
             * @see _.difference, _.xor
             * @example
             *
             * _.without([2, 1, 2, 3], 1, 2);
             * // => [3]
             */
            var without = baseRest(function (array, values) {
                return isArrayLikeObject(array)
                    ? baseDifference(array, values)
                    : [];
            });
            /**
             * Creates an array of unique values that is the
             * [symmetric difference](https://en.wikipedia.org/wiki/Symmetric_difference)
             * of the given arrays. The order of result values is determined by the order
             * they occur in the arrays.
             *
             * @static
             * @memberOf _
             * @since 2.4.0
             * @category Array
             * @param {...Array} [arrays] The arrays to inspect.
             * @returns {Array} Returns the new array of filtered values.
             * @see _.difference, _.without
             * @example
             *
             * _.xor([2, 1], [2, 3]);
             * // => [1, 3]
             */
            var xor = baseRest(function (arrays) {
                return baseXor(arrayFilter(arrays, isArrayLikeObject));
            });
            /**
             * This method is like `_.xor` except that it accepts `iteratee` which is
             * invoked for each element of each `arrays` to generate the criterion by
             * which by which they're compared. The order of result values is determined
             * by the order they occur in the arrays. The iteratee is invoked with one
             * argument: (value).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {...Array} [arrays] The arrays to inspect.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {Array} Returns the new array of filtered values.
             * @example
             *
             * _.xorBy([2.1, 1.2], [2.3, 3.4], Math.floor);
             * // => [1.2, 3.4]
             *
             * // The `_.property` iteratee shorthand.
             * _.xorBy([{ 'x': 1 }], [{ 'x': 2 }, { 'x': 1 }], 'x');
             * // => [{ 'x': 2 }]
             */
            var xorBy = baseRest(function (arrays) {
                var iteratee = last(arrays);
                if (isArrayLikeObject(iteratee)) {
                    iteratee = undefined;
                }
                return baseXor(arrayFilter(arrays, isArrayLikeObject), getIteratee(iteratee, 2));
            });
            /**
             * This method is like `_.xor` except that it accepts `comparator` which is
             * invoked to compare elements of `arrays`. The order of result values is
             * determined by the order they occur in the arrays. The comparator is invoked
             * with two arguments: (arrVal, othVal).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Array
             * @param {...Array} [arrays] The arrays to inspect.
             * @param {Function} [comparator] The comparator invoked per element.
             * @returns {Array} Returns the new array of filtered values.
             * @example
             *
             * var objects = [{ 'x': 1, 'y': 2 }, { 'x': 2, 'y': 1 }];
             * var others = [{ 'x': 1, 'y': 1 }, { 'x': 1, 'y': 2 }];
             *
             * _.xorWith(objects, others, _.isEqual);
             * // => [{ 'x': 2, 'y': 1 }, { 'x': 1, 'y': 1 }]
             */
            var xorWith = baseRest(function (arrays) {
                var comparator = last(arrays);
                comparator = typeof comparator == 'function' ? comparator : undefined;
                return baseXor(arrayFilter(arrays, isArrayLikeObject), undefined, comparator);
            });
            /**
             * Creates an array of grouped elements, the first of which contains the
             * first elements of the given arrays, the second of which contains the
             * second elements of the given arrays, and so on.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Array
             * @param {...Array} [arrays] The arrays to process.
             * @returns {Array} Returns the new array of grouped elements.
             * @example
             *
             * _.zip(['a', 'b'], [1, 2], [true, false]);
             * // => [['a', 1, true], ['b', 2, false]]
             */
            var zip = baseRest(unzip);
            /**
             * This method is like `_.fromPairs` except that it accepts two arrays,
             * one of property identifiers and one of corresponding values.
             *
             * @static
             * @memberOf _
             * @since 0.4.0
             * @category Array
             * @param {Array} [props=[]] The property identifiers.
             * @param {Array} [values=[]] The property values.
             * @returns {Object} Returns the new object.
             * @example
             *
             * _.zipObject(['a', 'b'], [1, 2]);
             * // => { 'a': 1, 'b': 2 }
             */
            function zipObject(props, values) {
                return baseZipObject(props || [], values || [], assignValue);
            }
            /**
             * This method is like `_.zipObject` except that it supports property paths.
             *
             * @static
             * @memberOf _
             * @since 4.1.0
             * @category Array
             * @param {Array} [props=[]] The property identifiers.
             * @param {Array} [values=[]] The property values.
             * @returns {Object} Returns the new object.
             * @example
             *
             * _.zipObjectDeep(['a.b[0].c', 'a.b[1].d'], [1, 2]);
             * // => { 'a': { 'b': [{ 'c': 1 }, { 'd': 2 }] } }
             */
            function zipObjectDeep(props, values) {
                return baseZipObject(props || [], values || [], baseSet);
            }
            /**
             * This method is like `_.zip` except that it accepts `iteratee` to specify
             * how grouped values should be combined. The iteratee is invoked with the
             * elements of each group: (...group).
             *
             * @static
             * @memberOf _
             * @since 3.8.0
             * @category Array
             * @param {...Array} [arrays] The arrays to process.
             * @param {Function} [iteratee=_.identity] The function to combine
             *  grouped values.
             * @returns {Array} Returns the new array of grouped elements.
             * @example
             *
             * _.zipWith([1, 2], [10, 20], [100, 200], function(a, b, c) {
             *   return a + b + c;
             * });
             * // => [111, 222]
             */
            var zipWith = baseRest(function (arrays) {
                var length = arrays.length, iteratee = length > 1 ? arrays[length - 1] : undefined;
                iteratee = typeof iteratee == 'function' ? (arrays.pop(), iteratee) : undefined;
                return unzipWith(arrays, iteratee);
            });
            /*------------------------------------------------------------------------*/
            /**
             * Creates a `lodash` wrapper instance that wraps `value` with explicit method
             * chain sequences enabled. The result of such sequences must be unwrapped
             * with `_#value`.
             *
             * @static
             * @memberOf _
             * @since 1.3.0
             * @category Seq
             * @param {*} value The value to wrap.
             * @returns {Object} Returns the new `lodash` wrapper instance.
             * @example
             *
             * var users = [
             *   { 'user': 'barney',  'age': 36 },
             *   { 'user': 'fred',    'age': 40 },
             *   { 'user': 'pebbles', 'age': 1 }
             * ];
             *
             * var youngest = _
             *   .chain(users)
             *   .sortBy('age')
             *   .map(function(o) {
             *     return o.user + ' is ' + o.age;
             *   })
             *   .head()
             *   .value();
             * // => 'pebbles is 1'
             */
            function chain(value) {
                var result = lodash(value);
                result.__chain__ = true;
                return result;
            }
            /**
             * This method invokes `interceptor` and returns `value`. The interceptor
             * is invoked with one argument; (value). The purpose of this method is to
             * "tap into" a method chain sequence in order to modify intermediate results.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Seq
             * @param {*} value The value to provide to `interceptor`.
             * @param {Function} interceptor The function to invoke.
             * @returns {*} Returns `value`.
             * @example
             *
             * _([1, 2, 3])
             *  .tap(function(array) {
             *    // Mutate input array.
             *    array.pop();
             *  })
             *  .reverse()
             *  .value();
             * // => [2, 1]
             */
            function tap(value, interceptor) {
                interceptor(value);
                return value;
            }
            /**
             * This method is like `_.tap` except that it returns the result of `interceptor`.
             * The purpose of this method is to "pass thru" values replacing intermediate
             * results in a method chain sequence.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Seq
             * @param {*} value The value to provide to `interceptor`.
             * @param {Function} interceptor The function to invoke.
             * @returns {*} Returns the result of `interceptor`.
             * @example
             *
             * _('  abc  ')
             *  .chain()
             *  .trim()
             *  .thru(function(value) {
             *    return [value];
             *  })
             *  .value();
             * // => ['abc']
             */
            function thru(value, interceptor) {
                return interceptor(value);
            }
            /**
             * This method is the wrapper version of `_.at`.
             *
             * @name at
             * @memberOf _
             * @since 1.0.0
             * @category Seq
             * @param {...(string|string[])} [paths] The property paths to pick.
             * @returns {Object} Returns the new `lodash` wrapper instance.
             * @example
             *
             * var object = { 'a': [{ 'b': { 'c': 3 } }, 4] };
             *
             * _(object).at(['a[0].b.c', 'a[1]']).value();
             * // => [3, 4]
             */
            var wrapperAt = flatRest(function (paths) {
                var length = paths.length, start = length ? paths[0] : 0, value = this.__wrapped__, interceptor = function (object) { return baseAt(object, paths); };
                if (length > 1 || this.__actions__.length ||
                    !(value instanceof LazyWrapper) || !isIndex(start)) {
                    return this.thru(interceptor);
                }
                value = value.slice(start, +start + (length ? 1 : 0));
                value.__actions__.push({
                    'func': thru,
                    'args': [interceptor],
                    'thisArg': undefined
                });
                return new LodashWrapper(value, this.__chain__).thru(function (array) {
                    if (length && !array.length) {
                        array.push(undefined);
                    }
                    return array;
                });
            });
            /**
             * Creates a `lodash` wrapper instance with explicit method chain sequences enabled.
             *
             * @name chain
             * @memberOf _
             * @since 0.1.0
             * @category Seq
             * @returns {Object} Returns the new `lodash` wrapper instance.
             * @example
             *
             * var users = [
             *   { 'user': 'barney', 'age': 36 },
             *   { 'user': 'fred',   'age': 40 }
             * ];
             *
             * // A sequence without explicit chaining.
             * _(users).head();
             * // => { 'user': 'barney', 'age': 36 }
             *
             * // A sequence with explicit chaining.
             * _(users)
             *   .chain()
             *   .head()
             *   .pick('user')
             *   .value();
             * // => { 'user': 'barney' }
             */
            function wrapperChain() {
                return chain(this);
            }
            /**
             * Executes the chain sequence and returns the wrapped result.
             *
             * @name commit
             * @memberOf _
             * @since 3.2.0
             * @category Seq
             * @returns {Object} Returns the new `lodash` wrapper instance.
             * @example
             *
             * var array = [1, 2];
             * var wrapped = _(array).push(3);
             *
             * console.log(array);
             * // => [1, 2]
             *
             * wrapped = wrapped.commit();
             * console.log(array);
             * // => [1, 2, 3]
             *
             * wrapped.last();
             * // => 3
             *
             * console.log(array);
             * // => [1, 2, 3]
             */
            function wrapperCommit() {
                return new LodashWrapper(this.value(), this.__chain__);
            }
            /**
             * Gets the next value on a wrapped object following the
             * [iterator protocol](https://mdn.io/iteration_protocols#iterator).
             *
             * @name next
             * @memberOf _
             * @since 4.0.0
             * @category Seq
             * @returns {Object} Returns the next iterator value.
             * @example
             *
             * var wrapped = _([1, 2]);
             *
             * wrapped.next();
             * // => { 'done': false, 'value': 1 }
             *
             * wrapped.next();
             * // => { 'done': false, 'value': 2 }
             *
             * wrapped.next();
             * // => { 'done': true, 'value': undefined }
             */
            function wrapperNext() {
                if (this.__values__ === undefined) {
                    this.__values__ = toArray(this.value());
                }
                var done = this.__index__ >= this.__values__.length, value = done ? undefined : this.__values__[this.__index__++];
                return { 'done': done, 'value': value };
            }
            /**
             * Enables the wrapper to be iterable.
             *
             * @name Symbol.iterator
             * @memberOf _
             * @since 4.0.0
             * @category Seq
             * @returns {Object} Returns the wrapper object.
             * @example
             *
             * var wrapped = _([1, 2]);
             *
             * wrapped[Symbol.iterator]() === wrapped;
             * // => true
             *
             * Array.from(wrapped);
             * // => [1, 2]
             */
            function wrapperToIterator() {
                return this;
            }
            /**
             * Creates a clone of the chain sequence planting `value` as the wrapped value.
             *
             * @name plant
             * @memberOf _
             * @since 3.2.0
             * @category Seq
             * @param {*} value The value to plant.
             * @returns {Object} Returns the new `lodash` wrapper instance.
             * @example
             *
             * function square(n) {
             *   return n * n;
             * }
             *
             * var wrapped = _([1, 2]).map(square);
             * var other = wrapped.plant([3, 4]);
             *
             * other.value();
             * // => [9, 16]
             *
             * wrapped.value();
             * // => [1, 4]
             */
            function wrapperPlant(value) {
                var result, parent = this;
                while (parent instanceof baseLodash) {
                    var clone = wrapperClone(parent);
                    clone.__index__ = 0;
                    clone.__values__ = undefined;
                    if (result) {
                        previous.__wrapped__ = clone;
                    }
                    else {
                        result = clone;
                    }
                    var previous = clone;
                    parent = parent.__wrapped__;
                }
                previous.__wrapped__ = value;
                return result;
            }
            /**
             * This method is the wrapper version of `_.reverse`.
             *
             * **Note:** This method mutates the wrapped array.
             *
             * @name reverse
             * @memberOf _
             * @since 0.1.0
             * @category Seq
             * @returns {Object} Returns the new `lodash` wrapper instance.
             * @example
             *
             * var array = [1, 2, 3];
             *
             * _(array).reverse().value()
             * // => [3, 2, 1]
             *
             * console.log(array);
             * // => [3, 2, 1]
             */
            function wrapperReverse() {
                var value = this.__wrapped__;
                if (value instanceof LazyWrapper) {
                    var wrapped = value;
                    if (this.__actions__.length) {
                        wrapped = new LazyWrapper(this);
                    }
                    wrapped = wrapped.reverse();
                    wrapped.__actions__.push({
                        'func': thru,
                        'args': [reverse],
                        'thisArg': undefined
                    });
                    return new LodashWrapper(wrapped, this.__chain__);
                }
                return this.thru(reverse);
            }
            /**
             * Executes the chain sequence to resolve the unwrapped value.
             *
             * @name value
             * @memberOf _
             * @since 0.1.0
             * @alias toJSON, valueOf
             * @category Seq
             * @returns {*} Returns the resolved unwrapped value.
             * @example
             *
             * _([1, 2, 3]).value();
             * // => [1, 2, 3]
             */
            function wrapperValue() {
                return baseWrapperValue(this.__wrapped__, this.__actions__);
            }
            /*------------------------------------------------------------------------*/
            /**
             * Creates an object composed of keys generated from the results of running
             * each element of `collection` thru `iteratee`. The corresponding value of
             * each key is the number of times the key was returned by `iteratee`. The
             * iteratee is invoked with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 0.5.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [iteratee=_.identity] The iteratee to transform keys.
             * @returns {Object} Returns the composed aggregate object.
             * @example
             *
             * _.countBy([6.1, 4.2, 6.3], Math.floor);
             * // => { '4': 1, '6': 2 }
             *
             * // The `_.property` iteratee shorthand.
             * _.countBy(['one', 'two', 'three'], 'length');
             * // => { '3': 2, '5': 1 }
             */
            var countBy = createAggregator(function (result, value, key) {
                if (hasOwnProperty.call(result, key)) {
                    ++result[key];
                }
                else {
                    baseAssignValue(result, key, 1);
                }
            });
            /**
             * Checks if `predicate` returns truthy for **all** elements of `collection`.
             * Iteration is stopped once `predicate` returns falsey. The predicate is
             * invoked with three arguments: (value, index|key, collection).
             *
             * **Note:** This method returns `true` for
             * [empty collections](https://en.wikipedia.org/wiki/Empty_set) because
             * [everything is true](https://en.wikipedia.org/wiki/Vacuous_truth) of
             * elements of empty collections.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {boolean} Returns `true` if all elements pass the predicate check,
             *  else `false`.
             * @example
             *
             * _.every([true, 1, null, 'yes'], Boolean);
             * // => false
             *
             * var users = [
             *   { 'user': 'barney', 'age': 36, 'active': false },
             *   { 'user': 'fred',   'age': 40, 'active': false }
             * ];
             *
             * // The `_.matches` iteratee shorthand.
             * _.every(users, { 'user': 'barney', 'active': false });
             * // => false
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.every(users, ['active', false]);
             * // => true
             *
             * // The `_.property` iteratee shorthand.
             * _.every(users, 'active');
             * // => false
             */
            function every(collection, predicate, guard) {
                var func = isArray(collection) ? arrayEvery : baseEvery;
                if (guard && isIterateeCall(collection, predicate, guard)) {
                    predicate = undefined;
                }
                return func(collection, getIteratee(predicate, 3));
            }
            /**
             * Iterates over elements of `collection`, returning an array of all elements
             * `predicate` returns truthy for. The predicate is invoked with three
             * arguments: (value, index|key, collection).
             *
             * **Note:** Unlike `_.remove`, this method returns a new array.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the new filtered array.
             * @see _.reject
             * @example
             *
             * var users = [
             *   { 'user': 'barney', 'age': 36, 'active': true },
             *   { 'user': 'fred',   'age': 40, 'active': false }
             * ];
             *
             * _.filter(users, function(o) { return !o.active; });
             * // => objects for ['fred']
             *
             * // The `_.matches` iteratee shorthand.
             * _.filter(users, { 'age': 36, 'active': true });
             * // => objects for ['barney']
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.filter(users, ['active', false]);
             * // => objects for ['fred']
             *
             * // The `_.property` iteratee shorthand.
             * _.filter(users, 'active');
             * // => objects for ['barney']
             */
            function filter(collection, predicate) {
                var func = isArray(collection) ? arrayFilter : baseFilter;
                return func(collection, getIteratee(predicate, 3));
            }
            /**
             * Iterates over elements of `collection`, returning the first element
             * `predicate` returns truthy for. The predicate is invoked with three
             * arguments: (value, index|key, collection).
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object} collection The collection to inspect.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @param {number} [fromIndex=0] The index to search from.
             * @returns {*} Returns the matched element, else `undefined`.
             * @example
             *
             * var users = [
             *   { 'user': 'barney',  'age': 36, 'active': true },
             *   { 'user': 'fred',    'age': 40, 'active': false },
             *   { 'user': 'pebbles', 'age': 1,  'active': true }
             * ];
             *
             * _.find(users, function(o) { return o.age < 40; });
             * // => object for 'barney'
             *
             * // The `_.matches` iteratee shorthand.
             * _.find(users, { 'age': 1, 'active': true });
             * // => object for 'pebbles'
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.find(users, ['active', false]);
             * // => object for 'fred'
             *
             * // The `_.property` iteratee shorthand.
             * _.find(users, 'active');
             * // => object for 'barney'
             */
            var find = createFind(findIndex);
            /**
             * This method is like `_.find` except that it iterates over elements of
             * `collection` from right to left.
             *
             * @static
             * @memberOf _
             * @since 2.0.0
             * @category Collection
             * @param {Array|Object} collection The collection to inspect.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @param {number} [fromIndex=collection.length-1] The index to search from.
             * @returns {*} Returns the matched element, else `undefined`.
             * @example
             *
             * _.findLast([1, 2, 3, 4], function(n) {
             *   return n % 2 == 1;
             * });
             * // => 3
             */
            var findLast = createFind(findLastIndex);
            /**
             * Creates a flattened array of values by running each element in `collection`
             * thru `iteratee` and flattening the mapped results. The iteratee is invoked
             * with three arguments: (value, index|key, collection).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the new flattened array.
             * @example
             *
             * function duplicate(n) {
             *   return [n, n];
             * }
             *
             * _.flatMap([1, 2], duplicate);
             * // => [1, 1, 2, 2]
             */
            function flatMap(collection, iteratee) {
                return baseFlatten(map(collection, iteratee), 1);
            }
            /**
             * This method is like `_.flatMap` except that it recursively flattens the
             * mapped results.
             *
             * @static
             * @memberOf _
             * @since 4.7.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the new flattened array.
             * @example
             *
             * function duplicate(n) {
             *   return [[[n, n]]];
             * }
             *
             * _.flatMapDeep([1, 2], duplicate);
             * // => [1, 1, 2, 2]
             */
            function flatMapDeep(collection, iteratee) {
                return baseFlatten(map(collection, iteratee), INFINITY);
            }
            /**
             * This method is like `_.flatMap` except that it recursively flattens the
             * mapped results up to `depth` times.
             *
             * @static
             * @memberOf _
             * @since 4.7.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @param {number} [depth=1] The maximum recursion depth.
             * @returns {Array} Returns the new flattened array.
             * @example
             *
             * function duplicate(n) {
             *   return [[[n, n]]];
             * }
             *
             * _.flatMapDepth([1, 2], duplicate, 2);
             * // => [[1, 1], [2, 2]]
             */
            function flatMapDepth(collection, iteratee, depth) {
                depth = depth === undefined ? 1 : toInteger(depth);
                return baseFlatten(map(collection, iteratee), depth);
            }
            /**
             * Iterates over elements of `collection` and invokes `iteratee` for each element.
             * The iteratee is invoked with three arguments: (value, index|key, collection).
             * Iteratee functions may exit iteration early by explicitly returning `false`.
             *
             * **Note:** As with other "Collections" methods, objects with a "length"
             * property are iterated like arrays. To avoid this behavior use `_.forIn`
             * or `_.forOwn` for object iteration.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @alias each
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Array|Object} Returns `collection`.
             * @see _.forEachRight
             * @example
             *
             * _.forEach([1, 2], function(value) {
             *   console.log(value);
             * });
             * // => Logs `1` then `2`.
             *
             * _.forEach({ 'a': 1, 'b': 2 }, function(value, key) {
             *   console.log(key);
             * });
             * // => Logs 'a' then 'b' (iteration order is not guaranteed).
             */
            function forEach(collection, iteratee) {
                var func = isArray(collection) ? arrayEach : baseEach;
                return func(collection, getIteratee(iteratee, 3));
            }
            /**
             * This method is like `_.forEach` except that it iterates over elements of
             * `collection` from right to left.
             *
             * @static
             * @memberOf _
             * @since 2.0.0
             * @alias eachRight
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Array|Object} Returns `collection`.
             * @see _.forEach
             * @example
             *
             * _.forEachRight([1, 2], function(value) {
             *   console.log(value);
             * });
             * // => Logs `2` then `1`.
             */
            function forEachRight(collection, iteratee) {
                var func = isArray(collection) ? arrayEachRight : baseEachRight;
                return func(collection, getIteratee(iteratee, 3));
            }
            /**
             * Creates an object composed of keys generated from the results of running
             * each element of `collection` thru `iteratee`. The order of grouped values
             * is determined by the order they occur in `collection`. The corresponding
             * value of each key is an array of elements responsible for generating the
             * key. The iteratee is invoked with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [iteratee=_.identity] The iteratee to transform keys.
             * @returns {Object} Returns the composed aggregate object.
             * @example
             *
             * _.groupBy([6.1, 4.2, 6.3], Math.floor);
             * // => { '4': [4.2], '6': [6.1, 6.3] }
             *
             * // The `_.property` iteratee shorthand.
             * _.groupBy(['one', 'two', 'three'], 'length');
             * // => { '3': ['one', 'two'], '5': ['three'] }
             */
            var groupBy = createAggregator(function (result, value, key) {
                if (hasOwnProperty.call(result, key)) {
                    result[key].push(value);
                }
                else {
                    baseAssignValue(result, key, [value]);
                }
            });
            /**
             * Checks if `value` is in `collection`. If `collection` is a string, it's
             * checked for a substring of `value`, otherwise
             * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
             * is used for equality comparisons. If `fromIndex` is negative, it's used as
             * the offset from the end of `collection`.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object|string} collection The collection to inspect.
             * @param {*} value The value to search for.
             * @param {number} [fromIndex=0] The index to search from.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.reduce`.
             * @returns {boolean} Returns `true` if `value` is found, else `false`.
             * @example
             *
             * _.includes([1, 2, 3], 1);
             * // => true
             *
             * _.includes([1, 2, 3], 1, 2);
             * // => false
             *
             * _.includes({ 'a': 1, 'b': 2 }, 1);
             * // => true
             *
             * _.includes('abcd', 'bc');
             * // => true
             */
            function includes(collection, value, fromIndex, guard) {
                collection = isArrayLike(collection) ? collection : values(collection);
                fromIndex = (fromIndex && !guard) ? toInteger(fromIndex) : 0;
                var length = collection.length;
                if (fromIndex < 0) {
                    fromIndex = nativeMax(length + fromIndex, 0);
                }
                return isString(collection)
                    ? (fromIndex <= length && collection.indexOf(value, fromIndex) > -1)
                    : (!!length && baseIndexOf(collection, value, fromIndex) > -1);
            }
            /**
             * Invokes the method at `path` of each element in `collection`, returning
             * an array of the results of each invoked method. Any additional arguments
             * are provided to each invoked method. If `path` is a function, it's invoked
             * for, and `this` bound to, each element in `collection`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Array|Function|string} path The path of the method to invoke or
             *  the function invoked per iteration.
             * @param {...*} [args] The arguments to invoke each method with.
             * @returns {Array} Returns the array of results.
             * @example
             *
             * _.invokeMap([[5, 1, 7], [3, 2, 1]], 'sort');
             * // => [[1, 5, 7], [1, 2, 3]]
             *
             * _.invokeMap([123, 456], String.prototype.split, '');
             * // => [['1', '2', '3'], ['4', '5', '6']]
             */
            var invokeMap = baseRest(function (collection, path, args) {
                var index = -1, isFunc = typeof path == 'function', result = isArrayLike(collection) ? Array(collection.length) : [];
                baseEach(collection, function (value) {
                    result[++index] = isFunc ? apply(path, value, args) : baseInvoke(value, path, args);
                });
                return result;
            });
            /**
             * Creates an object composed of keys generated from the results of running
             * each element of `collection` thru `iteratee`. The corresponding value of
             * each key is the last element responsible for generating the key. The
             * iteratee is invoked with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [iteratee=_.identity] The iteratee to transform keys.
             * @returns {Object} Returns the composed aggregate object.
             * @example
             *
             * var array = [
             *   { 'dir': 'left', 'code': 97 },
             *   { 'dir': 'right', 'code': 100 }
             * ];
             *
             * _.keyBy(array, function(o) {
             *   return String.fromCharCode(o.code);
             * });
             * // => { 'a': { 'dir': 'left', 'code': 97 }, 'd': { 'dir': 'right', 'code': 100 } }
             *
             * _.keyBy(array, 'dir');
             * // => { 'left': { 'dir': 'left', 'code': 97 }, 'right': { 'dir': 'right', 'code': 100 } }
             */
            var keyBy = createAggregator(function (result, value, key) {
                baseAssignValue(result, key, value);
            });
            /**
             * Creates an array of values by running each element in `collection` thru
             * `iteratee`. The iteratee is invoked with three arguments:
             * (value, index|key, collection).
             *
             * Many lodash methods are guarded to work as iteratees for methods like
             * `_.every`, `_.filter`, `_.map`, `_.mapValues`, `_.reject`, and `_.some`.
             *
             * The guarded methods are:
             * `ary`, `chunk`, `curry`, `curryRight`, `drop`, `dropRight`, `every`,
             * `fill`, `invert`, `parseInt`, `random`, `range`, `rangeRight`, `repeat`,
             * `sampleSize`, `slice`, `some`, `sortBy`, `split`, `take`, `takeRight`,
             * `template`, `trim`, `trimEnd`, `trimStart`, and `words`
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the new mapped array.
             * @example
             *
             * function square(n) {
             *   return n * n;
             * }
             *
             * _.map([4, 8], square);
             * // => [16, 64]
             *
             * _.map({ 'a': 4, 'b': 8 }, square);
             * // => [16, 64] (iteration order is not guaranteed)
             *
             * var users = [
             *   { 'user': 'barney' },
             *   { 'user': 'fred' }
             * ];
             *
             * // The `_.property` iteratee shorthand.
             * _.map(users, 'user');
             * // => ['barney', 'fred']
             */
            function map(collection, iteratee) {
                var func = isArray(collection) ? arrayMap : baseMap;
                return func(collection, getIteratee(iteratee, 3));
            }
            /**
             * This method is like `_.sortBy` except that it allows specifying the sort
             * orders of the iteratees to sort by. If `orders` is unspecified, all values
             * are sorted in ascending order. Otherwise, specify an order of "desc" for
             * descending or "asc" for ascending sort order of corresponding values.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Array[]|Function[]|Object[]|string[]} [iteratees=[_.identity]]
             *  The iteratees to sort by.
             * @param {string[]} [orders] The sort orders of `iteratees`.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.reduce`.
             * @returns {Array} Returns the new sorted array.
             * @example
             *
             * var users = [
             *   { 'user': 'fred',   'age': 48 },
             *   { 'user': 'barney', 'age': 34 },
             *   { 'user': 'fred',   'age': 40 },
             *   { 'user': 'barney', 'age': 36 }
             * ];
             *
             * // Sort by `user` in ascending order and by `age` in descending order.
             * _.orderBy(users, ['user', 'age'], ['asc', 'desc']);
             * // => objects for [['barney', 36], ['barney', 34], ['fred', 48], ['fred', 40]]
             */
            function orderBy(collection, iteratees, orders, guard) {
                if (collection == null) {
                    return [];
                }
                if (!isArray(iteratees)) {
                    iteratees = iteratees == null ? [] : [iteratees];
                }
                orders = guard ? undefined : orders;
                if (!isArray(orders)) {
                    orders = orders == null ? [] : [orders];
                }
                return baseOrderBy(collection, iteratees, orders);
            }
            /**
             * Creates an array of elements split into two groups, the first of which
             * contains elements `predicate` returns truthy for, the second of which
             * contains elements `predicate` returns falsey for. The predicate is
             * invoked with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the array of grouped elements.
             * @example
             *
             * var users = [
             *   { 'user': 'barney',  'age': 36, 'active': false },
             *   { 'user': 'fred',    'age': 40, 'active': true },
             *   { 'user': 'pebbles', 'age': 1,  'active': false }
             * ];
             *
             * _.partition(users, function(o) { return o.active; });
             * // => objects for [['fred'], ['barney', 'pebbles']]
             *
             * // The `_.matches` iteratee shorthand.
             * _.partition(users, { 'age': 1, 'active': false });
             * // => objects for [['pebbles'], ['barney', 'fred']]
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.partition(users, ['active', false]);
             * // => objects for [['barney', 'pebbles'], ['fred']]
             *
             * // The `_.property` iteratee shorthand.
             * _.partition(users, 'active');
             * // => objects for [['fred'], ['barney', 'pebbles']]
             */
            var partition = createAggregator(function (result, value, key) {
                result[key ? 0 : 1].push(value);
            }, function () { return [[], []]; });
            /**
             * Reduces `collection` to a value which is the accumulated result of running
             * each element in `collection` thru `iteratee`, where each successive
             * invocation is supplied the return value of the previous. If `accumulator`
             * is not given, the first element of `collection` is used as the initial
             * value. The iteratee is invoked with four arguments:
             * (accumulator, value, index|key, collection).
             *
             * Many lodash methods are guarded to work as iteratees for methods like
             * `_.reduce`, `_.reduceRight`, and `_.transform`.
             *
             * The guarded methods are:
             * `assign`, `defaults`, `defaultsDeep`, `includes`, `merge`, `orderBy`,
             * and `sortBy`
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @param {*} [accumulator] The initial value.
             * @returns {*} Returns the accumulated value.
             * @see _.reduceRight
             * @example
             *
             * _.reduce([1, 2], function(sum, n) {
             *   return sum + n;
             * }, 0);
             * // => 3
             *
             * _.reduce({ 'a': 1, 'b': 2, 'c': 1 }, function(result, value, key) {
             *   (result[value] || (result[value] = [])).push(key);
             *   return result;
             * }, {});
             * // => { '1': ['a', 'c'], '2': ['b'] } (iteration order is not guaranteed)
             */
            function reduce(collection, iteratee, accumulator) {
                var func = isArray(collection) ? arrayReduce : baseReduce, initAccum = arguments.length < 3;
                return func(collection, getIteratee(iteratee, 4), accumulator, initAccum, baseEach);
            }
            /**
             * This method is like `_.reduce` except that it iterates over elements of
             * `collection` from right to left.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @param {*} [accumulator] The initial value.
             * @returns {*} Returns the accumulated value.
             * @see _.reduce
             * @example
             *
             * var array = [[0, 1], [2, 3], [4, 5]];
             *
             * _.reduceRight(array, function(flattened, other) {
             *   return flattened.concat(other);
             * }, []);
             * // => [4, 5, 2, 3, 0, 1]
             */
            function reduceRight(collection, iteratee, accumulator) {
                var func = isArray(collection) ? arrayReduceRight : baseReduce, initAccum = arguments.length < 3;
                return func(collection, getIteratee(iteratee, 4), accumulator, initAccum, baseEachRight);
            }
            /**
             * The opposite of `_.filter`; this method returns the elements of `collection`
             * that `predicate` does **not** return truthy for.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the new filtered array.
             * @see _.filter
             * @example
             *
             * var users = [
             *   { 'user': 'barney', 'age': 36, 'active': false },
             *   { 'user': 'fred',   'age': 40, 'active': true }
             * ];
             *
             * _.reject(users, function(o) { return !o.active; });
             * // => objects for ['fred']
             *
             * // The `_.matches` iteratee shorthand.
             * _.reject(users, { 'age': 40, 'active': true });
             * // => objects for ['barney']
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.reject(users, ['active', false]);
             * // => objects for ['fred']
             *
             * // The `_.property` iteratee shorthand.
             * _.reject(users, 'active');
             * // => objects for ['barney']
             */
            function reject(collection, predicate) {
                var func = isArray(collection) ? arrayFilter : baseFilter;
                return func(collection, negate(getIteratee(predicate, 3)));
            }
            /**
             * Gets a random element from `collection`.
             *
             * @static
             * @memberOf _
             * @since 2.0.0
             * @category Collection
             * @param {Array|Object} collection The collection to sample.
             * @returns {*} Returns the random element.
             * @example
             *
             * _.sample([1, 2, 3, 4]);
             * // => 2
             */
            function sample(collection) {
                var func = isArray(collection) ? arraySample : baseSample;
                return func(collection);
            }
            /**
             * Gets `n` random elements at unique keys from `collection` up to the
             * size of `collection`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Collection
             * @param {Array|Object} collection The collection to sample.
             * @param {number} [n=1] The number of elements to sample.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {Array} Returns the random elements.
             * @example
             *
             * _.sampleSize([1, 2, 3], 2);
             * // => [3, 1]
             *
             * _.sampleSize([1, 2, 3], 4);
             * // => [2, 3, 1]
             */
            function sampleSize(collection, n, guard) {
                if ((guard ? isIterateeCall(collection, n, guard) : n === undefined)) {
                    n = 1;
                }
                else {
                    n = toInteger(n);
                }
                var func = isArray(collection) ? arraySampleSize : baseSampleSize;
                return func(collection, n);
            }
            /**
             * Creates an array of shuffled values, using a version of the
             * [Fisher-Yates shuffle](https://en.wikipedia.org/wiki/Fisher-Yates_shuffle).
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object} collection The collection to shuffle.
             * @returns {Array} Returns the new shuffled array.
             * @example
             *
             * _.shuffle([1, 2, 3, 4]);
             * // => [4, 1, 3, 2]
             */
            function shuffle(collection) {
                var func = isArray(collection) ? arrayShuffle : baseShuffle;
                return func(collection);
            }
            /**
             * Gets the size of `collection` by returning its length for array-like
             * values or the number of own enumerable string keyed properties for objects.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object|string} collection The collection to inspect.
             * @returns {number} Returns the collection size.
             * @example
             *
             * _.size([1, 2, 3]);
             * // => 3
             *
             * _.size({ 'a': 1, 'b': 2 });
             * // => 2
             *
             * _.size('pebbles');
             * // => 7
             */
            function size(collection) {
                if (collection == null) {
                    return 0;
                }
                if (isArrayLike(collection)) {
                    return isString(collection) ? stringSize(collection) : collection.length;
                }
                var tag = getTag(collection);
                if (tag == mapTag || tag == setTag) {
                    return collection.size;
                }
                return baseKeys(collection).length;
            }
            /**
             * Checks if `predicate` returns truthy for **any** element of `collection`.
             * Iteration is stopped once `predicate` returns truthy. The predicate is
             * invoked with three arguments: (value, index|key, collection).
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {boolean} Returns `true` if any element passes the predicate check,
             *  else `false`.
             * @example
             *
             * _.some([null, 0, 'yes', false], Boolean);
             * // => true
             *
             * var users = [
             *   { 'user': 'barney', 'active': true },
             *   { 'user': 'fred',   'active': false }
             * ];
             *
             * // The `_.matches` iteratee shorthand.
             * _.some(users, { 'user': 'barney', 'active': false });
             * // => false
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.some(users, ['active', false]);
             * // => true
             *
             * // The `_.property` iteratee shorthand.
             * _.some(users, 'active');
             * // => true
             */
            function some(collection, predicate, guard) {
                var func = isArray(collection) ? arraySome : baseSome;
                if (guard && isIterateeCall(collection, predicate, guard)) {
                    predicate = undefined;
                }
                return func(collection, getIteratee(predicate, 3));
            }
            /**
             * Creates an array of elements, sorted in ascending order by the results of
             * running each element in a collection thru each iteratee. This method
             * performs a stable sort, that is, it preserves the original sort order of
             * equal elements. The iteratees are invoked with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Collection
             * @param {Array|Object} collection The collection to iterate over.
             * @param {...(Function|Function[])} [iteratees=[_.identity]]
             *  The iteratees to sort by.
             * @returns {Array} Returns the new sorted array.
             * @example
             *
             * var users = [
             *   { 'user': 'fred',   'age': 48 },
             *   { 'user': 'barney', 'age': 36 },
             *   { 'user': 'fred',   'age': 40 },
             *   { 'user': 'barney', 'age': 34 }
             * ];
             *
             * _.sortBy(users, [function(o) { return o.user; }]);
             * // => objects for [['barney', 36], ['barney', 34], ['fred', 48], ['fred', 40]]
             *
             * _.sortBy(users, ['user', 'age']);
             * // => objects for [['barney', 34], ['barney', 36], ['fred', 40], ['fred', 48]]
             */
            var sortBy = baseRest(function (collection, iteratees) {
                if (collection == null) {
                    return [];
                }
                var length = iteratees.length;
                if (length > 1 && isIterateeCall(collection, iteratees[0], iteratees[1])) {
                    iteratees = [];
                }
                else if (length > 2 && isIterateeCall(iteratees[0], iteratees[1], iteratees[2])) {
                    iteratees = [iteratees[0]];
                }
                return baseOrderBy(collection, baseFlatten(iteratees, 1), []);
            });
            /*------------------------------------------------------------------------*/
            /**
             * Gets the timestamp of the number of milliseconds that have elapsed since
             * the Unix epoch (1 January 1970 00:00:00 UTC).
             *
             * @static
             * @memberOf _
             * @since 2.4.0
             * @category Date
             * @returns {number} Returns the timestamp.
             * @example
             *
             * _.defer(function(stamp) {
             *   console.log(_.now() - stamp);
             * }, _.now());
             * // => Logs the number of milliseconds it took for the deferred invocation.
             */
            var now = ctxNow || function () {
                return root.Date.now();
            };
            /*------------------------------------------------------------------------*/
            /**
             * The opposite of `_.before`; this method creates a function that invokes
             * `func` once it's called `n` or more times.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Function
             * @param {number} n The number of calls before `func` is invoked.
             * @param {Function} func The function to restrict.
             * @returns {Function} Returns the new restricted function.
             * @example
             *
             * var saves = ['profile', 'settings'];
             *
             * var done = _.after(saves.length, function() {
             *   console.log('done saving!');
             * });
             *
             * _.forEach(saves, function(type) {
             *   asyncSave({ 'type': type, 'complete': done });
             * });
             * // => Logs 'done saving!' after the two async saves have completed.
             */
            function after(n, func) {
                if (typeof func != 'function') {
                    throw new TypeError(FUNC_ERROR_TEXT);
                }
                n = toInteger(n);
                return function () {
                    if (--n < 1) {
                        return func.apply(this, arguments);
                    }
                };
            }
            /**
             * Creates a function that invokes `func`, with up to `n` arguments,
             * ignoring any additional arguments.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Function
             * @param {Function} func The function to cap arguments for.
             * @param {number} [n=func.length] The arity cap.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {Function} Returns the new capped function.
             * @example
             *
             * _.map(['6', '8', '10'], _.ary(parseInt, 1));
             * // => [6, 8, 10]
             */
            function ary(func, n, guard) {
                n = guard ? undefined : n;
                n = (func && n == null) ? func.length : n;
                return createWrap(func, WRAP_ARY_FLAG, undefined, undefined, undefined, undefined, n);
            }
            /**
             * Creates a function that invokes `func`, with the `this` binding and arguments
             * of the created function, while it's called less than `n` times. Subsequent
             * calls to the created function return the result of the last `func` invocation.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Function
             * @param {number} n The number of calls at which `func` is no longer invoked.
             * @param {Function} func The function to restrict.
             * @returns {Function} Returns the new restricted function.
             * @example
             *
             * jQuery(element).on('click', _.before(5, addContactToList));
             * // => Allows adding up to 4 contacts to the list.
             */
            function before(n, func) {
                var result;
                if (typeof func != 'function') {
                    throw new TypeError(FUNC_ERROR_TEXT);
                }
                n = toInteger(n);
                return function () {
                    if (--n > 0) {
                        result = func.apply(this, arguments);
                    }
                    if (n <= 1) {
                        func = undefined;
                    }
                    return result;
                };
            }
            /**
             * Creates a function that invokes `func` with the `this` binding of `thisArg`
             * and `partials` prepended to the arguments it receives.
             *
             * The `_.bind.placeholder` value, which defaults to `_` in monolithic builds,
             * may be used as a placeholder for partially applied arguments.
             *
             * **Note:** Unlike native `Function#bind`, this method doesn't set the "length"
             * property of bound functions.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Function
             * @param {Function} func The function to bind.
             * @param {*} thisArg The `this` binding of `func`.
             * @param {...*} [partials] The arguments to be partially applied.
             * @returns {Function} Returns the new bound function.
             * @example
             *
             * function greet(greeting, punctuation) {
             *   return greeting + ' ' + this.user + punctuation;
             * }
             *
             * var object = { 'user': 'fred' };
             *
             * var bound = _.bind(greet, object, 'hi');
             * bound('!');
             * // => 'hi fred!'
             *
             * // Bound with placeholders.
             * var bound = _.bind(greet, object, _, '!');
             * bound('hi');
             * // => 'hi fred!'
             */
            var bind = baseRest(function (func, thisArg, partials) {
                var bitmask = WRAP_BIND_FLAG;
                if (partials.length) {
                    var holders = replaceHolders(partials, getHolder(bind));
                    bitmask |= WRAP_PARTIAL_FLAG;
                }
                return createWrap(func, bitmask, thisArg, partials, holders);
            });
            /**
             * Creates a function that invokes the method at `object[key]` with `partials`
             * prepended to the arguments it receives.
             *
             * This method differs from `_.bind` by allowing bound functions to reference
             * methods that may be redefined or don't yet exist. See
             * [Peter Michaux's article](http://peter.michaux.ca/articles/lazy-function-definition-pattern)
             * for more details.
             *
             * The `_.bindKey.placeholder` value, which defaults to `_` in monolithic
             * builds, may be used as a placeholder for partially applied arguments.
             *
             * @static
             * @memberOf _
             * @since 0.10.0
             * @category Function
             * @param {Object} object The object to invoke the method on.
             * @param {string} key The key of the method.
             * @param {...*} [partials] The arguments to be partially applied.
             * @returns {Function} Returns the new bound function.
             * @example
             *
             * var object = {
             *   'user': 'fred',
             *   'greet': function(greeting, punctuation) {
             *     return greeting + ' ' + this.user + punctuation;
             *   }
             * };
             *
             * var bound = _.bindKey(object, 'greet', 'hi');
             * bound('!');
             * // => 'hi fred!'
             *
             * object.greet = function(greeting, punctuation) {
             *   return greeting + 'ya ' + this.user + punctuation;
             * };
             *
             * bound('!');
             * // => 'hiya fred!'
             *
             * // Bound with placeholders.
             * var bound = _.bindKey(object, 'greet', _, '!');
             * bound('hi');
             * // => 'hiya fred!'
             */
            var bindKey = baseRest(function (object, key, partials) {
                var bitmask = WRAP_BIND_FLAG | WRAP_BIND_KEY_FLAG;
                if (partials.length) {
                    var holders = replaceHolders(partials, getHolder(bindKey));
                    bitmask |= WRAP_PARTIAL_FLAG;
                }
                return createWrap(key, bitmask, object, partials, holders);
            });
            /**
             * Creates a function that accepts arguments of `func` and either invokes
             * `func` returning its result, if at least `arity` number of arguments have
             * been provided, or returns a function that accepts the remaining `func`
             * arguments, and so on. The arity of `func` may be specified if `func.length`
             * is not sufficient.
             *
             * The `_.curry.placeholder` value, which defaults to `_` in monolithic builds,
             * may be used as a placeholder for provided arguments.
             *
             * **Note:** This method doesn't set the "length" property of curried functions.
             *
             * @static
             * @memberOf _
             * @since 2.0.0
             * @category Function
             * @param {Function} func The function to curry.
             * @param {number} [arity=func.length] The arity of `func`.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {Function} Returns the new curried function.
             * @example
             *
             * var abc = function(a, b, c) {
             *   return [a, b, c];
             * };
             *
             * var curried = _.curry(abc);
             *
             * curried(1)(2)(3);
             * // => [1, 2, 3]
             *
             * curried(1, 2)(3);
             * // => [1, 2, 3]
             *
             * curried(1, 2, 3);
             * // => [1, 2, 3]
             *
             * // Curried with placeholders.
             * curried(1)(_, 3)(2);
             * // => [1, 2, 3]
             */
            function curry(func, arity, guard) {
                arity = guard ? undefined : arity;
                var result = createWrap(func, WRAP_CURRY_FLAG, undefined, undefined, undefined, undefined, undefined, arity);
                result.placeholder = curry.placeholder;
                return result;
            }
            /**
             * This method is like `_.curry` except that arguments are applied to `func`
             * in the manner of `_.partialRight` instead of `_.partial`.
             *
             * The `_.curryRight.placeholder` value, which defaults to `_` in monolithic
             * builds, may be used as a placeholder for provided arguments.
             *
             * **Note:** This method doesn't set the "length" property of curried functions.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Function
             * @param {Function} func The function to curry.
             * @param {number} [arity=func.length] The arity of `func`.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {Function} Returns the new curried function.
             * @example
             *
             * var abc = function(a, b, c) {
             *   return [a, b, c];
             * };
             *
             * var curried = _.curryRight(abc);
             *
             * curried(3)(2)(1);
             * // => [1, 2, 3]
             *
             * curried(2, 3)(1);
             * // => [1, 2, 3]
             *
             * curried(1, 2, 3);
             * // => [1, 2, 3]
             *
             * // Curried with placeholders.
             * curried(3)(1, _)(2);
             * // => [1, 2, 3]
             */
            function curryRight(func, arity, guard) {
                arity = guard ? undefined : arity;
                var result = createWrap(func, WRAP_CURRY_RIGHT_FLAG, undefined, undefined, undefined, undefined, undefined, arity);
                result.placeholder = curryRight.placeholder;
                return result;
            }
            /**
             * Creates a debounced function that delays invoking `func` until after `wait`
             * milliseconds have elapsed since the last time the debounced function was
             * invoked. The debounced function comes with a `cancel` method to cancel
             * delayed `func` invocations and a `flush` method to immediately invoke them.
             * Provide `options` to indicate whether `func` should be invoked on the
             * leading and/or trailing edge of the `wait` timeout. The `func` is invoked
             * with the last arguments provided to the debounced function. Subsequent
             * calls to the debounced function return the result of the last `func`
             * invocation.
             *
             * **Note:** If `leading` and `trailing` options are `true`, `func` is
             * invoked on the trailing edge of the timeout only if the debounced function
             * is invoked more than once during the `wait` timeout.
             *
             * If `wait` is `0` and `leading` is `false`, `func` invocation is deferred
             * until to the next tick, similar to `setTimeout` with a timeout of `0`.
             *
             * See [David Corbacho's article](https://css-tricks.com/debouncing-throttling-explained-examples/)
             * for details over the differences between `_.debounce` and `_.throttle`.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Function
             * @param {Function} func The function to debounce.
             * @param {number} [wait=0] The number of milliseconds to delay.
             * @param {Object} [options={}] The options object.
             * @param {boolean} [options.leading=false]
             *  Specify invoking on the leading edge of the timeout.
             * @param {number} [options.maxWait]
             *  The maximum time `func` is allowed to be delayed before it's invoked.
             * @param {boolean} [options.trailing=true]
             *  Specify invoking on the trailing edge of the timeout.
             * @returns {Function} Returns the new debounced function.
             * @example
             *
             * // Avoid costly calculations while the window size is in flux.
             * jQuery(window).on('resize', _.debounce(calculateLayout, 150));
             *
             * // Invoke `sendMail` when clicked, debouncing subsequent calls.
             * jQuery(element).on('click', _.debounce(sendMail, 300, {
             *   'leading': true,
             *   'trailing': false
             * }));
             *
             * // Ensure `batchLog` is invoked once after 1 second of debounced calls.
             * var debounced = _.debounce(batchLog, 250, { 'maxWait': 1000 });
             * var source = new EventSource('/stream');
             * jQuery(source).on('message', debounced);
             *
             * // Cancel the trailing debounced invocation.
             * jQuery(window).on('popstate', debounced.cancel);
             */
            function debounce(func, wait, options) {
                var lastArgs, lastThis, maxWait, result, timerId, lastCallTime, lastInvokeTime = 0, leading = false, maxing = false, trailing = true;
                if (typeof func != 'function') {
                    throw new TypeError(FUNC_ERROR_TEXT);
                }
                wait = toNumber(wait) || 0;
                if (isObject(options)) {
                    leading = !!options.leading;
                    maxing = 'maxWait' in options;
                    maxWait = maxing ? nativeMax(toNumber(options.maxWait) || 0, wait) : maxWait;
                    trailing = 'trailing' in options ? !!options.trailing : trailing;
                }
                function invokeFunc(time) {
                    var args = lastArgs, thisArg = lastThis;
                    lastArgs = lastThis = undefined;
                    lastInvokeTime = time;
                    result = func.apply(thisArg, args);
                    return result;
                }
                function leadingEdge(time) {
                    // Reset any `maxWait` timer.
                    lastInvokeTime = time;
                    // Start the timer for the trailing edge.
                    timerId = setTimeout(timerExpired, wait);
                    // Invoke the leading edge.
                    return leading ? invokeFunc(time) : result;
                }
                function remainingWait(time) {
                    var timeSinceLastCall = time - lastCallTime, timeSinceLastInvoke = time - lastInvokeTime, timeWaiting = wait - timeSinceLastCall;
                    return maxing
                        ? nativeMin(timeWaiting, maxWait - timeSinceLastInvoke)
                        : timeWaiting;
                }
                function shouldInvoke(time) {
                    var timeSinceLastCall = time - lastCallTime, timeSinceLastInvoke = time - lastInvokeTime;
                    // Either this is the first call, activity has stopped and we're at the
                    // trailing edge, the system time has gone backwards and we're treating
                    // it as the trailing edge, or we've hit the `maxWait` limit.
                    return (lastCallTime === undefined || (timeSinceLastCall >= wait) ||
                        (timeSinceLastCall < 0) || (maxing && timeSinceLastInvoke >= maxWait));
                }
                function timerExpired() {
                    var time = now();
                    if (shouldInvoke(time)) {
                        return trailingEdge(time);
                    }
                    // Restart the timer.
                    timerId = setTimeout(timerExpired, remainingWait(time));
                }
                function trailingEdge(time) {
                    timerId = undefined;
                    // Only invoke if we have `lastArgs` which means `func` has been
                    // debounced at least once.
                    if (trailing && lastArgs) {
                        return invokeFunc(time);
                    }
                    lastArgs = lastThis = undefined;
                    return result;
                }
                function cancel() {
                    if (timerId !== undefined) {
                        clearTimeout(timerId);
                    }
                    lastInvokeTime = 0;
                    lastArgs = lastCallTime = lastThis = timerId = undefined;
                }
                function flush() {
                    return timerId === undefined ? result : trailingEdge(now());
                }
                function debounced() {
                    var time = now(), isInvoking = shouldInvoke(time);
                    lastArgs = arguments;
                    lastThis = this;
                    lastCallTime = time;
                    if (isInvoking) {
                        if (timerId === undefined) {
                            return leadingEdge(lastCallTime);
                        }
                        if (maxing) {
                            // Handle invocations in a tight loop.
                            clearTimeout(timerId);
                            timerId = setTimeout(timerExpired, wait);
                            return invokeFunc(lastCallTime);
                        }
                    }
                    if (timerId === undefined) {
                        timerId = setTimeout(timerExpired, wait);
                    }
                    return result;
                }
                debounced.cancel = cancel;
                debounced.flush = flush;
                return debounced;
            }
            /**
             * Defers invoking the `func` until the current call stack has cleared. Any
             * additional arguments are provided to `func` when it's invoked.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Function
             * @param {Function} func The function to defer.
             * @param {...*} [args] The arguments to invoke `func` with.
             * @returns {number} Returns the timer id.
             * @example
             *
             * _.defer(function(text) {
             *   console.log(text);
             * }, 'deferred');
             * // => Logs 'deferred' after one millisecond.
             */
            var defer = baseRest(function (func, args) {
                return baseDelay(func, 1, args);
            });
            /**
             * Invokes `func` after `wait` milliseconds. Any additional arguments are
             * provided to `func` when it's invoked.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Function
             * @param {Function} func The function to delay.
             * @param {number} wait The number of milliseconds to delay invocation.
             * @param {...*} [args] The arguments to invoke `func` with.
             * @returns {number} Returns the timer id.
             * @example
             *
             * _.delay(function(text) {
             *   console.log(text);
             * }, 1000, 'later');
             * // => Logs 'later' after one second.
             */
            var delay = baseRest(function (func, wait, args) {
                return baseDelay(func, toNumber(wait) || 0, args);
            });
            /**
             * Creates a function that invokes `func` with arguments reversed.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Function
             * @param {Function} func The function to flip arguments for.
             * @returns {Function} Returns the new flipped function.
             * @example
             *
             * var flipped = _.flip(function() {
             *   return _.toArray(arguments);
             * });
             *
             * flipped('a', 'b', 'c', 'd');
             * // => ['d', 'c', 'b', 'a']
             */
            function flip(func) {
                return createWrap(func, WRAP_FLIP_FLAG);
            }
            /**
             * Creates a function that memoizes the result of `func`. If `resolver` is
             * provided, it determines the cache key for storing the result based on the
             * arguments provided to the memoized function. By default, the first argument
             * provided to the memoized function is used as the map cache key. The `func`
             * is invoked with the `this` binding of the memoized function.
             *
             * **Note:** The cache is exposed as the `cache` property on the memoized
             * function. Its creation may be customized by replacing the `_.memoize.Cache`
             * constructor with one whose instances implement the
             * [`Map`](http://ecma-international.org/ecma-262/7.0/#sec-properties-of-the-map-prototype-object)
             * method interface of `clear`, `delete`, `get`, `has`, and `set`.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Function
             * @param {Function} func The function to have its output memoized.
             * @param {Function} [resolver] The function to resolve the cache key.
             * @returns {Function} Returns the new memoized function.
             * @example
             *
             * var object = { 'a': 1, 'b': 2 };
             * var other = { 'c': 3, 'd': 4 };
             *
             * var values = _.memoize(_.values);
             * values(object);
             * // => [1, 2]
             *
             * values(other);
             * // => [3, 4]
             *
             * object.a = 2;
             * values(object);
             * // => [1, 2]
             *
             * // Modify the result cache.
             * values.cache.set(object, ['a', 'b']);
             * values(object);
             * // => ['a', 'b']
             *
             * // Replace `_.memoize.Cache`.
             * _.memoize.Cache = WeakMap;
             */
            function memoize(func, resolver) {
                if (typeof func != 'function' || (resolver != null && typeof resolver != 'function')) {
                    throw new TypeError(FUNC_ERROR_TEXT);
                }
                var memoized = function () {
                    var args = arguments, key = resolver ? resolver.apply(this, args) : args[0], cache = memoized.cache;
                    if (cache.has(key)) {
                        return cache.get(key);
                    }
                    var result = func.apply(this, args);
                    memoized.cache = cache.set(key, result) || cache;
                    return result;
                };
                memoized.cache = new (memoize.Cache || MapCache);
                return memoized;
            }
            // Expose `MapCache`.
            memoize.Cache = MapCache;
            /**
             * Creates a function that negates the result of the predicate `func`. The
             * `func` predicate is invoked with the `this` binding and arguments of the
             * created function.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Function
             * @param {Function} predicate The predicate to negate.
             * @returns {Function} Returns the new negated function.
             * @example
             *
             * function isEven(n) {
             *   return n % 2 == 0;
             * }
             *
             * _.filter([1, 2, 3, 4, 5, 6], _.negate(isEven));
             * // => [1, 3, 5]
             */
            function negate(predicate) {
                if (typeof predicate != 'function') {
                    throw new TypeError(FUNC_ERROR_TEXT);
                }
                return function () {
                    var args = arguments;
                    switch (args.length) {
                        case 0: return !predicate.call(this);
                        case 1: return !predicate.call(this, args[0]);
                        case 2: return !predicate.call(this, args[0], args[1]);
                        case 3: return !predicate.call(this, args[0], args[1], args[2]);
                    }
                    return !predicate.apply(this, args);
                };
            }
            /**
             * Creates a function that is restricted to invoking `func` once. Repeat calls
             * to the function return the value of the first invocation. The `func` is
             * invoked with the `this` binding and arguments of the created function.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Function
             * @param {Function} func The function to restrict.
             * @returns {Function} Returns the new restricted function.
             * @example
             *
             * var initialize = _.once(createApplication);
             * initialize();
             * initialize();
             * // => `createApplication` is invoked once
             */
            function once(func) {
                return before(2, func);
            }
            /**
             * Creates a function that invokes `func` with its arguments transformed.
             *
             * @static
             * @since 4.0.0
             * @memberOf _
             * @category Function
             * @param {Function} func The function to wrap.
             * @param {...(Function|Function[])} [transforms=[_.identity]]
             *  The argument transforms.
             * @returns {Function} Returns the new function.
             * @example
             *
             * function doubled(n) {
             *   return n * 2;
             * }
             *
             * function square(n) {
             *   return n * n;
             * }
             *
             * var func = _.overArgs(function(x, y) {
             *   return [x, y];
             * }, [square, doubled]);
             *
             * func(9, 3);
             * // => [81, 6]
             *
             * func(10, 5);
             * // => [100, 10]
             */
            var overArgs = castRest(function (func, transforms) {
                transforms = (transforms.length == 1 && isArray(transforms[0]))
                    ? arrayMap(transforms[0], baseUnary(getIteratee()))
                    : arrayMap(baseFlatten(transforms, 1), baseUnary(getIteratee()));
                var funcsLength = transforms.length;
                return baseRest(function (args) {
                    var index = -1, length = nativeMin(args.length, funcsLength);
                    while (++index < length) {
                        args[index] = transforms[index].call(this, args[index]);
                    }
                    return apply(func, this, args);
                });
            });
            /**
             * Creates a function that invokes `func` with `partials` prepended to the
             * arguments it receives. This method is like `_.bind` except it does **not**
             * alter the `this` binding.
             *
             * The `_.partial.placeholder` value, which defaults to `_` in monolithic
             * builds, may be used as a placeholder for partially applied arguments.
             *
             * **Note:** This method doesn't set the "length" property of partially
             * applied functions.
             *
             * @static
             * @memberOf _
             * @since 0.2.0
             * @category Function
             * @param {Function} func The function to partially apply arguments to.
             * @param {...*} [partials] The arguments to be partially applied.
             * @returns {Function} Returns the new partially applied function.
             * @example
             *
             * function greet(greeting, name) {
             *   return greeting + ' ' + name;
             * }
             *
             * var sayHelloTo = _.partial(greet, 'hello');
             * sayHelloTo('fred');
             * // => 'hello fred'
             *
             * // Partially applied with placeholders.
             * var greetFred = _.partial(greet, _, 'fred');
             * greetFred('hi');
             * // => 'hi fred'
             */
            var partial = baseRest(function (func, partials) {
                var holders = replaceHolders(partials, getHolder(partial));
                return createWrap(func, WRAP_PARTIAL_FLAG, undefined, partials, holders);
            });
            /**
             * This method is like `_.partial` except that partially applied arguments
             * are appended to the arguments it receives.
             *
             * The `_.partialRight.placeholder` value, which defaults to `_` in monolithic
             * builds, may be used as a placeholder for partially applied arguments.
             *
             * **Note:** This method doesn't set the "length" property of partially
             * applied functions.
             *
             * @static
             * @memberOf _
             * @since 1.0.0
             * @category Function
             * @param {Function} func The function to partially apply arguments to.
             * @param {...*} [partials] The arguments to be partially applied.
             * @returns {Function} Returns the new partially applied function.
             * @example
             *
             * function greet(greeting, name) {
             *   return greeting + ' ' + name;
             * }
             *
             * var greetFred = _.partialRight(greet, 'fred');
             * greetFred('hi');
             * // => 'hi fred'
             *
             * // Partially applied with placeholders.
             * var sayHelloTo = _.partialRight(greet, 'hello', _);
             * sayHelloTo('fred');
             * // => 'hello fred'
             */
            var partialRight = baseRest(function (func, partials) {
                var holders = replaceHolders(partials, getHolder(partialRight));
                return createWrap(func, WRAP_PARTIAL_RIGHT_FLAG, undefined, partials, holders);
            });
            /**
             * Creates a function that invokes `func` with arguments arranged according
             * to the specified `indexes` where the argument value at the first index is
             * provided as the first argument, the argument value at the second index is
             * provided as the second argument, and so on.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Function
             * @param {Function} func The function to rearrange arguments for.
             * @param {...(number|number[])} indexes The arranged argument indexes.
             * @returns {Function} Returns the new function.
             * @example
             *
             * var rearged = _.rearg(function(a, b, c) {
             *   return [a, b, c];
             * }, [2, 0, 1]);
             *
             * rearged('b', 'c', 'a')
             * // => ['a', 'b', 'c']
             */
            var rearg = flatRest(function (func, indexes) {
                return createWrap(func, WRAP_REARG_FLAG, undefined, undefined, undefined, indexes);
            });
            /**
             * Creates a function that invokes `func` with the `this` binding of the
             * created function and arguments from `start` and beyond provided as
             * an array.
             *
             * **Note:** This method is based on the
             * [rest parameter](https://mdn.io/rest_parameters).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Function
             * @param {Function} func The function to apply a rest parameter to.
             * @param {number} [start=func.length-1] The start position of the rest parameter.
             * @returns {Function} Returns the new function.
             * @example
             *
             * var say = _.rest(function(what, names) {
             *   return what + ' ' + _.initial(names).join(', ') +
             *     (_.size(names) > 1 ? ', & ' : '') + _.last(names);
             * });
             *
             * say('hello', 'fred', 'barney', 'pebbles');
             * // => 'hello fred, barney, & pebbles'
             */
            function rest(func, start) {
                if (typeof func != 'function') {
                    throw new TypeError(FUNC_ERROR_TEXT);
                }
                start = start === undefined ? start : toInteger(start);
                return baseRest(func, start);
            }
            /**
             * Creates a function that invokes `func` with the `this` binding of the
             * create function and an array of arguments much like
             * [`Function#apply`](http://www.ecma-international.org/ecma-262/7.0/#sec-function.prototype.apply).
             *
             * **Note:** This method is based on the
             * [spread operator](https://mdn.io/spread_operator).
             *
             * @static
             * @memberOf _
             * @since 3.2.0
             * @category Function
             * @param {Function} func The function to spread arguments over.
             * @param {number} [start=0] The start position of the spread.
             * @returns {Function} Returns the new function.
             * @example
             *
             * var say = _.spread(function(who, what) {
             *   return who + ' says ' + what;
             * });
             *
             * say(['fred', 'hello']);
             * // => 'fred says hello'
             *
             * var numbers = Promise.all([
             *   Promise.resolve(40),
             *   Promise.resolve(36)
             * ]);
             *
             * numbers.then(_.spread(function(x, y) {
             *   return x + y;
             * }));
             * // => a Promise of 76
             */
            function spread(func, start) {
                if (typeof func != 'function') {
                    throw new TypeError(FUNC_ERROR_TEXT);
                }
                start = start == null ? 0 : nativeMax(toInteger(start), 0);
                return baseRest(function (args) {
                    var array = args[start], otherArgs = castSlice(args, 0, start);
                    if (array) {
                        arrayPush(otherArgs, array);
                    }
                    return apply(func, this, otherArgs);
                });
            }
            /**
             * Creates a throttled function that only invokes `func` at most once per
             * every `wait` milliseconds. The throttled function comes with a `cancel`
             * method to cancel delayed `func` invocations and a `flush` method to
             * immediately invoke them. Provide `options` to indicate whether `func`
             * should be invoked on the leading and/or trailing edge of the `wait`
             * timeout. The `func` is invoked with the last arguments provided to the
             * throttled function. Subsequent calls to the throttled function return the
             * result of the last `func` invocation.
             *
             * **Note:** If `leading` and `trailing` options are `true`, `func` is
             * invoked on the trailing edge of the timeout only if the throttled function
             * is invoked more than once during the `wait` timeout.
             *
             * If `wait` is `0` and `leading` is `false`, `func` invocation is deferred
             * until to the next tick, similar to `setTimeout` with a timeout of `0`.
             *
             * See [David Corbacho's article](https://css-tricks.com/debouncing-throttling-explained-examples/)
             * for details over the differences between `_.throttle` and `_.debounce`.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Function
             * @param {Function} func The function to throttle.
             * @param {number} [wait=0] The number of milliseconds to throttle invocations to.
             * @param {Object} [options={}] The options object.
             * @param {boolean} [options.leading=true]
             *  Specify invoking on the leading edge of the timeout.
             * @param {boolean} [options.trailing=true]
             *  Specify invoking on the trailing edge of the timeout.
             * @returns {Function} Returns the new throttled function.
             * @example
             *
             * // Avoid excessively updating the position while scrolling.
             * jQuery(window).on('scroll', _.throttle(updatePosition, 100));
             *
             * // Invoke `renewToken` when the click event is fired, but not more than once every 5 minutes.
             * var throttled = _.throttle(renewToken, 300000, { 'trailing': false });
             * jQuery(element).on('click', throttled);
             *
             * // Cancel the trailing throttled invocation.
             * jQuery(window).on('popstate', throttled.cancel);
             */
            function throttle(func, wait, options) {
                var leading = true, trailing = true;
                if (typeof func != 'function') {
                    throw new TypeError(FUNC_ERROR_TEXT);
                }
                if (isObject(options)) {
                    leading = 'leading' in options ? !!options.leading : leading;
                    trailing = 'trailing' in options ? !!options.trailing : trailing;
                }
                return debounce(func, wait, {
                    'leading': leading,
                    'maxWait': wait,
                    'trailing': trailing
                });
            }
            /**
             * Creates a function that accepts up to one argument, ignoring any
             * additional arguments.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Function
             * @param {Function} func The function to cap arguments for.
             * @returns {Function} Returns the new capped function.
             * @example
             *
             * _.map(['6', '8', '10'], _.unary(parseInt));
             * // => [6, 8, 10]
             */
            function unary(func) {
                return ary(func, 1);
            }
            /**
             * Creates a function that provides `value` to `wrapper` as its first
             * argument. Any additional arguments provided to the function are appended
             * to those provided to the `wrapper`. The wrapper is invoked with the `this`
             * binding of the created function.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Function
             * @param {*} value The value to wrap.
             * @param {Function} [wrapper=identity] The wrapper function.
             * @returns {Function} Returns the new function.
             * @example
             *
             * var p = _.wrap(_.escape, function(func, text) {
             *   return '<p>' + func(text) + '</p>';
             * });
             *
             * p('fred, barney, & pebbles');
             * // => '<p>fred, barney, &amp; pebbles</p>'
             */
            function wrap(value, wrapper) {
                return partial(castFunction(wrapper), value);
            }
            /*------------------------------------------------------------------------*/
            /**
             * Casts `value` as an array if it's not one.
             *
             * @static
             * @memberOf _
             * @since 4.4.0
             * @category Lang
             * @param {*} value The value to inspect.
             * @returns {Array} Returns the cast array.
             * @example
             *
             * _.castArray(1);
             * // => [1]
             *
             * _.castArray({ 'a': 1 });
             * // => [{ 'a': 1 }]
             *
             * _.castArray('abc');
             * // => ['abc']
             *
             * _.castArray(null);
             * // => [null]
             *
             * _.castArray(undefined);
             * // => [undefined]
             *
             * _.castArray();
             * // => []
             *
             * var array = [1, 2, 3];
             * console.log(_.castArray(array) === array);
             * // => true
             */
            function castArray() {
                if (!arguments.length) {
                    return [];
                }
                var value = arguments[0];
                return isArray(value) ? value : [value];
            }
            /**
             * Creates a shallow clone of `value`.
             *
             * **Note:** This method is loosely based on the
             * [structured clone algorithm](https://mdn.io/Structured_clone_algorithm)
             * and supports cloning arrays, array buffers, booleans, date objects, maps,
             * numbers, `Object` objects, regexes, sets, strings, symbols, and typed
             * arrays. The own enumerable properties of `arguments` objects are cloned
             * as plain objects. An empty object is returned for uncloneable values such
             * as error objects, functions, DOM nodes, and WeakMaps.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to clone.
             * @returns {*} Returns the cloned value.
             * @see _.cloneDeep
             * @example
             *
             * var objects = [{ 'a': 1 }, { 'b': 2 }];
             *
             * var shallow = _.clone(objects);
             * console.log(shallow[0] === objects[0]);
             * // => true
             */
            function clone(value) {
                return baseClone(value, CLONE_SYMBOLS_FLAG);
            }
            /**
             * This method is like `_.clone` except that it accepts `customizer` which
             * is invoked to produce the cloned value. If `customizer` returns `undefined`,
             * cloning is handled by the method instead. The `customizer` is invoked with
             * up to four arguments; (value [, index|key, object, stack]).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to clone.
             * @param {Function} [customizer] The function to customize cloning.
             * @returns {*} Returns the cloned value.
             * @see _.cloneDeepWith
             * @example
             *
             * function customizer(value) {
             *   if (_.isElement(value)) {
             *     return value.cloneNode(false);
             *   }
             * }
             *
             * var el = _.cloneWith(document.body, customizer);
             *
             * console.log(el === document.body);
             * // => false
             * console.log(el.nodeName);
             * // => 'BODY'
             * console.log(el.childNodes.length);
             * // => 0
             */
            function cloneWith(value, customizer) {
                customizer = typeof customizer == 'function' ? customizer : undefined;
                return baseClone(value, CLONE_SYMBOLS_FLAG, customizer);
            }
            /**
             * This method is like `_.clone` except that it recursively clones `value`.
             *
             * @static
             * @memberOf _
             * @since 1.0.0
             * @category Lang
             * @param {*} value The value to recursively clone.
             * @returns {*} Returns the deep cloned value.
             * @see _.clone
             * @example
             *
             * var objects = [{ 'a': 1 }, { 'b': 2 }];
             *
             * var deep = _.cloneDeep(objects);
             * console.log(deep[0] === objects[0]);
             * // => false
             */
            function cloneDeep(value) {
                return baseClone(value, CLONE_DEEP_FLAG | CLONE_SYMBOLS_FLAG);
            }
            /**
             * This method is like `_.cloneWith` except that it recursively clones `value`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to recursively clone.
             * @param {Function} [customizer] The function to customize cloning.
             * @returns {*} Returns the deep cloned value.
             * @see _.cloneWith
             * @example
             *
             * function customizer(value) {
             *   if (_.isElement(value)) {
             *     return value.cloneNode(true);
             *   }
             * }
             *
             * var el = _.cloneDeepWith(document.body, customizer);
             *
             * console.log(el === document.body);
             * // => false
             * console.log(el.nodeName);
             * // => 'BODY'
             * console.log(el.childNodes.length);
             * // => 20
             */
            function cloneDeepWith(value, customizer) {
                customizer = typeof customizer == 'function' ? customizer : undefined;
                return baseClone(value, CLONE_DEEP_FLAG | CLONE_SYMBOLS_FLAG, customizer);
            }
            /**
             * Checks if `object` conforms to `source` by invoking the predicate
             * properties of `source` with the corresponding property values of `object`.
             *
             * **Note:** This method is equivalent to `_.conforms` when `source` is
             * partially applied.
             *
             * @static
             * @memberOf _
             * @since 4.14.0
             * @category Lang
             * @param {Object} object The object to inspect.
             * @param {Object} source The object of property predicates to conform to.
             * @returns {boolean} Returns `true` if `object` conforms, else `false`.
             * @example
             *
             * var object = { 'a': 1, 'b': 2 };
             *
             * _.conformsTo(object, { 'b': function(n) { return n > 1; } });
             * // => true
             *
             * _.conformsTo(object, { 'b': function(n) { return n > 2; } });
             * // => false
             */
            function conformsTo(object, source) {
                return source == null || baseConformsTo(object, source, keys(source));
            }
            /**
             * Performs a
             * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
             * comparison between two values to determine if they are equivalent.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to compare.
             * @param {*} other The other value to compare.
             * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
             * @example
             *
             * var object = { 'a': 1 };
             * var other = { 'a': 1 };
             *
             * _.eq(object, object);
             * // => true
             *
             * _.eq(object, other);
             * // => false
             *
             * _.eq('a', 'a');
             * // => true
             *
             * _.eq('a', Object('a'));
             * // => false
             *
             * _.eq(NaN, NaN);
             * // => true
             */
            function eq(value, other) {
                return value === other || (value !== value && other !== other);
            }
            /**
             * Checks if `value` is greater than `other`.
             *
             * @static
             * @memberOf _
             * @since 3.9.0
             * @category Lang
             * @param {*} value The value to compare.
             * @param {*} other The other value to compare.
             * @returns {boolean} Returns `true` if `value` is greater than `other`,
             *  else `false`.
             * @see _.lt
             * @example
             *
             * _.gt(3, 1);
             * // => true
             *
             * _.gt(3, 3);
             * // => false
             *
             * _.gt(1, 3);
             * // => false
             */
            var gt = createRelationalOperation(baseGt);
            /**
             * Checks if `value` is greater than or equal to `other`.
             *
             * @static
             * @memberOf _
             * @since 3.9.0
             * @category Lang
             * @param {*} value The value to compare.
             * @param {*} other The other value to compare.
             * @returns {boolean} Returns `true` if `value` is greater than or equal to
             *  `other`, else `false`.
             * @see _.lte
             * @example
             *
             * _.gte(3, 1);
             * // => true
             *
             * _.gte(3, 3);
             * // => true
             *
             * _.gte(1, 3);
             * // => false
             */
            var gte = createRelationalOperation(function (value, other) {
                return value >= other;
            });
            /**
             * Checks if `value` is likely an `arguments` object.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is an `arguments` object,
             *  else `false`.
             * @example
             *
             * _.isArguments(function() { return arguments; }());
             * // => true
             *
             * _.isArguments([1, 2, 3]);
             * // => false
             */
            var isArguments = baseIsArguments(function () { return arguments; }()) ? baseIsArguments : function (value) {
                return isObjectLike(value) && hasOwnProperty.call(value, 'callee') &&
                    !propertyIsEnumerable.call(value, 'callee');
            };
            /**
             * Checks if `value` is classified as an `Array` object.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is an array, else `false`.
             * @example
             *
             * _.isArray([1, 2, 3]);
             * // => true
             *
             * _.isArray(document.body.children);
             * // => false
             *
             * _.isArray('abc');
             * // => false
             *
             * _.isArray(_.noop);
             * // => false
             */
            var isArray = Array.isArray;
            /**
             * Checks if `value` is classified as an `ArrayBuffer` object.
             *
             * @static
             * @memberOf _
             * @since 4.3.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is an array buffer, else `false`.
             * @example
             *
             * _.isArrayBuffer(new ArrayBuffer(2));
             * // => true
             *
             * _.isArrayBuffer(new Array(2));
             * // => false
             */
            var isArrayBuffer = nodeIsArrayBuffer ? baseUnary(nodeIsArrayBuffer) : baseIsArrayBuffer;
            /**
             * Checks if `value` is array-like. A value is considered array-like if it's
             * not a function and has a `value.length` that's an integer greater than or
             * equal to `0` and less than or equal to `Number.MAX_SAFE_INTEGER`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is array-like, else `false`.
             * @example
             *
             * _.isArrayLike([1, 2, 3]);
             * // => true
             *
             * _.isArrayLike(document.body.children);
             * // => true
             *
             * _.isArrayLike('abc');
             * // => true
             *
             * _.isArrayLike(_.noop);
             * // => false
             */
            function isArrayLike(value) {
                return value != null && isLength(value.length) && !isFunction(value);
            }
            /**
             * This method is like `_.isArrayLike` except that it also checks if `value`
             * is an object.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is an array-like object,
             *  else `false`.
             * @example
             *
             * _.isArrayLikeObject([1, 2, 3]);
             * // => true
             *
             * _.isArrayLikeObject(document.body.children);
             * // => true
             *
             * _.isArrayLikeObject('abc');
             * // => false
             *
             * _.isArrayLikeObject(_.noop);
             * // => false
             */
            function isArrayLikeObject(value) {
                return isObjectLike(value) && isArrayLike(value);
            }
            /**
             * Checks if `value` is classified as a boolean primitive or object.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a boolean, else `false`.
             * @example
             *
             * _.isBoolean(false);
             * // => true
             *
             * _.isBoolean(null);
             * // => false
             */
            function isBoolean(value) {
                return value === true || value === false ||
                    (isObjectLike(value) && baseGetTag(value) == boolTag);
            }
            /**
             * Checks if `value` is a buffer.
             *
             * @static
             * @memberOf _
             * @since 4.3.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a buffer, else `false`.
             * @example
             *
             * _.isBuffer(new Buffer(2));
             * // => true
             *
             * _.isBuffer(new Uint8Array(2));
             * // => false
             */
            var isBuffer = nativeIsBuffer || stubFalse;
            /**
             * Checks if `value` is classified as a `Date` object.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a date object, else `false`.
             * @example
             *
             * _.isDate(new Date);
             * // => true
             *
             * _.isDate('Mon April 23 2012');
             * // => false
             */
            var isDate = nodeIsDate ? baseUnary(nodeIsDate) : baseIsDate;
            /**
             * Checks if `value` is likely a DOM element.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a DOM element, else `false`.
             * @example
             *
             * _.isElement(document.body);
             * // => true
             *
             * _.isElement('<body>');
             * // => false
             */
            function isElement(value) {
                return isObjectLike(value) && value.nodeType === 1 && !isPlainObject(value);
            }
            /**
             * Checks if `value` is an empty object, collection, map, or set.
             *
             * Objects are considered empty if they have no own enumerable string keyed
             * properties.
             *
             * Array-like values such as `arguments` objects, arrays, buffers, strings, or
             * jQuery-like collections are considered empty if they have a `length` of `0`.
             * Similarly, maps and sets are considered empty if they have a `size` of `0`.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is empty, else `false`.
             * @example
             *
             * _.isEmpty(null);
             * // => true
             *
             * _.isEmpty(true);
             * // => true
             *
             * _.isEmpty(1);
             * // => true
             *
             * _.isEmpty([1, 2, 3]);
             * // => false
             *
             * _.isEmpty({ 'a': 1 });
             * // => false
             */
            function isEmpty(value) {
                if (value == null) {
                    return true;
                }
                if (isArrayLike(value) &&
                    (isArray(value) || typeof value == 'string' || typeof value.splice == 'function' ||
                        isBuffer(value) || isTypedArray(value) || isArguments(value))) {
                    return !value.length;
                }
                var tag = getTag(value);
                if (tag == mapTag || tag == setTag) {
                    return !value.size;
                }
                if (isPrototype(value)) {
                    return !baseKeys(value).length;
                }
                for (var key in value) {
                    if (hasOwnProperty.call(value, key)) {
                        return false;
                    }
                }
                return true;
            }
            /**
             * Performs a deep comparison between two values to determine if they are
             * equivalent.
             *
             * **Note:** This method supports comparing arrays, array buffers, booleans,
             * date objects, error objects, maps, numbers, `Object` objects, regexes,
             * sets, strings, symbols, and typed arrays. `Object` objects are compared
             * by their own, not inherited, enumerable properties. Functions and DOM
             * nodes are compared by strict equality, i.e. `===`.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to compare.
             * @param {*} other The other value to compare.
             * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
             * @example
             *
             * var object = { 'a': 1 };
             * var other = { 'a': 1 };
             *
             * _.isEqual(object, other);
             * // => true
             *
             * object === other;
             * // => false
             */
            function isEqual(value, other) {
                return baseIsEqual(value, other);
            }
            /**
             * This method is like `_.isEqual` except that it accepts `customizer` which
             * is invoked to compare values. If `customizer` returns `undefined`, comparisons
             * are handled by the method instead. The `customizer` is invoked with up to
             * six arguments: (objValue, othValue [, index|key, object, other, stack]).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to compare.
             * @param {*} other The other value to compare.
             * @param {Function} [customizer] The function to customize comparisons.
             * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
             * @example
             *
             * function isGreeting(value) {
             *   return /^h(?:i|ello)$/.test(value);
             * }
             *
             * function customizer(objValue, othValue) {
             *   if (isGreeting(objValue) && isGreeting(othValue)) {
             *     return true;
             *   }
             * }
             *
             * var array = ['hello', 'goodbye'];
             * var other = ['hi', 'goodbye'];
             *
             * _.isEqualWith(array, other, customizer);
             * // => true
             */
            function isEqualWith(value, other, customizer) {
                customizer = typeof customizer == 'function' ? customizer : undefined;
                var result = customizer ? customizer(value, other) : undefined;
                return result === undefined ? baseIsEqual(value, other, undefined, customizer) : !!result;
            }
            /**
             * Checks if `value` is an `Error`, `EvalError`, `RangeError`, `ReferenceError`,
             * `SyntaxError`, `TypeError`, or `URIError` object.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is an error object, else `false`.
             * @example
             *
             * _.isError(new Error);
             * // => true
             *
             * _.isError(Error);
             * // => false
             */
            function isError(value) {
                if (!isObjectLike(value)) {
                    return false;
                }
                var tag = baseGetTag(value);
                return tag == errorTag || tag == domExcTag ||
                    (typeof value.message == 'string' && typeof value.name == 'string' && !isPlainObject(value));
            }
            /**
             * Checks if `value` is a finite primitive number.
             *
             * **Note:** This method is based on
             * [`Number.isFinite`](https://mdn.io/Number/isFinite).
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a finite number, else `false`.
             * @example
             *
             * _.isFinite(3);
             * // => true
             *
             * _.isFinite(Number.MIN_VALUE);
             * // => true
             *
             * _.isFinite(Infinity);
             * // => false
             *
             * _.isFinite('3');
             * // => false
             */
            function isFinite(value) {
                return typeof value == 'number' && nativeIsFinite(value);
            }
            /**
             * Checks if `value` is classified as a `Function` object.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a function, else `false`.
             * @example
             *
             * _.isFunction(_);
             * // => true
             *
             * _.isFunction(/abc/);
             * // => false
             */
            function isFunction(value) {
                if (!isObject(value)) {
                    return false;
                }
                // The use of `Object#toString` avoids issues with the `typeof` operator
                // in Safari 9 which returns 'object' for typed arrays and other constructors.
                var tag = baseGetTag(value);
                return tag == funcTag || tag == genTag || tag == asyncTag || tag == proxyTag;
            }
            /**
             * Checks if `value` is an integer.
             *
             * **Note:** This method is based on
             * [`Number.isInteger`](https://mdn.io/Number/isInteger).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is an integer, else `false`.
             * @example
             *
             * _.isInteger(3);
             * // => true
             *
             * _.isInteger(Number.MIN_VALUE);
             * // => false
             *
             * _.isInteger(Infinity);
             * // => false
             *
             * _.isInteger('3');
             * // => false
             */
            function isInteger(value) {
                return typeof value == 'number' && value == toInteger(value);
            }
            /**
             * Checks if `value` is a valid array-like length.
             *
             * **Note:** This method is loosely based on
             * [`ToLength`](http://ecma-international.org/ecma-262/7.0/#sec-tolength).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a valid length, else `false`.
             * @example
             *
             * _.isLength(3);
             * // => true
             *
             * _.isLength(Number.MIN_VALUE);
             * // => false
             *
             * _.isLength(Infinity);
             * // => false
             *
             * _.isLength('3');
             * // => false
             */
            function isLength(value) {
                return typeof value == 'number' &&
                    value > -1 && value % 1 == 0 && value <= MAX_SAFE_INTEGER;
            }
            /**
             * Checks if `value` is the
             * [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types)
             * of `Object`. (e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is an object, else `false`.
             * @example
             *
             * _.isObject({});
             * // => true
             *
             * _.isObject([1, 2, 3]);
             * // => true
             *
             * _.isObject(_.noop);
             * // => true
             *
             * _.isObject(null);
             * // => false
             */
            function isObject(value) {
                var type = typeof value;
                return value != null && (type == 'object' || type == 'function');
            }
            /**
             * Checks if `value` is object-like. A value is object-like if it's not `null`
             * and has a `typeof` result of "object".
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
             * @example
             *
             * _.isObjectLike({});
             * // => true
             *
             * _.isObjectLike([1, 2, 3]);
             * // => true
             *
             * _.isObjectLike(_.noop);
             * // => false
             *
             * _.isObjectLike(null);
             * // => false
             */
            function isObjectLike(value) {
                return value != null && typeof value == 'object';
            }
            /**
             * Checks if `value` is classified as a `Map` object.
             *
             * @static
             * @memberOf _
             * @since 4.3.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a map, else `false`.
             * @example
             *
             * _.isMap(new Map);
             * // => true
             *
             * _.isMap(new WeakMap);
             * // => false
             */
            var isMap = nodeIsMap ? baseUnary(nodeIsMap) : baseIsMap;
            /**
             * Performs a partial deep comparison between `object` and `source` to
             * determine if `object` contains equivalent property values.
             *
             * **Note:** This method is equivalent to `_.matches` when `source` is
             * partially applied.
             *
             * Partial comparisons will match empty array and empty object `source`
             * values against any array or object value, respectively. See `_.isEqual`
             * for a list of supported value comparisons.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Lang
             * @param {Object} object The object to inspect.
             * @param {Object} source The object of property values to match.
             * @returns {boolean} Returns `true` if `object` is a match, else `false`.
             * @example
             *
             * var object = { 'a': 1, 'b': 2 };
             *
             * _.isMatch(object, { 'b': 2 });
             * // => true
             *
             * _.isMatch(object, { 'b': 1 });
             * // => false
             */
            function isMatch(object, source) {
                return object === source || baseIsMatch(object, source, getMatchData(source));
            }
            /**
             * This method is like `_.isMatch` except that it accepts `customizer` which
             * is invoked to compare values. If `customizer` returns `undefined`, comparisons
             * are handled by the method instead. The `customizer` is invoked with five
             * arguments: (objValue, srcValue, index|key, object, source).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {Object} object The object to inspect.
             * @param {Object} source The object of property values to match.
             * @param {Function} [customizer] The function to customize comparisons.
             * @returns {boolean} Returns `true` if `object` is a match, else `false`.
             * @example
             *
             * function isGreeting(value) {
             *   return /^h(?:i|ello)$/.test(value);
             * }
             *
             * function customizer(objValue, srcValue) {
             *   if (isGreeting(objValue) && isGreeting(srcValue)) {
             *     return true;
             *   }
             * }
             *
             * var object = { 'greeting': 'hello' };
             * var source = { 'greeting': 'hi' };
             *
             * _.isMatchWith(object, source, customizer);
             * // => true
             */
            function isMatchWith(object, source, customizer) {
                customizer = typeof customizer == 'function' ? customizer : undefined;
                return baseIsMatch(object, source, getMatchData(source), customizer);
            }
            /**
             * Checks if `value` is `NaN`.
             *
             * **Note:** This method is based on
             * [`Number.isNaN`](https://mdn.io/Number/isNaN) and is not the same as
             * global [`isNaN`](https://mdn.io/isNaN) which returns `true` for
             * `undefined` and other non-number values.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is `NaN`, else `false`.
             * @example
             *
             * _.isNaN(NaN);
             * // => true
             *
             * _.isNaN(new Number(NaN));
             * // => true
             *
             * isNaN(undefined);
             * // => true
             *
             * _.isNaN(undefined);
             * // => false
             */
            function isNaN(value) {
                // An `NaN` primitive is the only value that is not equal to itself.
                // Perform the `toStringTag` check first to avoid errors with some
                // ActiveX objects in IE.
                return isNumber(value) && value != +value;
            }
            /**
             * Checks if `value` is a pristine native function.
             *
             * **Note:** This method can't reliably detect native functions in the presence
             * of the core-js package because core-js circumvents this kind of detection.
             * Despite multiple requests, the core-js maintainer has made it clear: any
             * attempt to fix the detection will be obstructed. As a result, we're left
             * with little choice but to throw an error. Unfortunately, this also affects
             * packages, like [babel-polyfill](https://www.npmjs.com/package/babel-polyfill),
             * which rely on core-js.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a native function,
             *  else `false`.
             * @example
             *
             * _.isNative(Array.prototype.push);
             * // => true
             *
             * _.isNative(_);
             * // => false
             */
            function isNative(value) {
                if (isMaskable(value)) {
                    throw new Error(CORE_ERROR_TEXT);
                }
                return baseIsNative(value);
            }
            /**
             * Checks if `value` is `null`.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is `null`, else `false`.
             * @example
             *
             * _.isNull(null);
             * // => true
             *
             * _.isNull(void 0);
             * // => false
             */
            function isNull(value) {
                return value === null;
            }
            /**
             * Checks if `value` is `null` or `undefined`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is nullish, else `false`.
             * @example
             *
             * _.isNil(null);
             * // => true
             *
             * _.isNil(void 0);
             * // => true
             *
             * _.isNil(NaN);
             * // => false
             */
            function isNil(value) {
                return value == null;
            }
            /**
             * Checks if `value` is classified as a `Number` primitive or object.
             *
             * **Note:** To exclude `Infinity`, `-Infinity`, and `NaN`, which are
             * classified as numbers, use the `_.isFinite` method.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a number, else `false`.
             * @example
             *
             * _.isNumber(3);
             * // => true
             *
             * _.isNumber(Number.MIN_VALUE);
             * // => true
             *
             * _.isNumber(Infinity);
             * // => true
             *
             * _.isNumber('3');
             * // => false
             */
            function isNumber(value) {
                return typeof value == 'number' ||
                    (isObjectLike(value) && baseGetTag(value) == numberTag);
            }
            /**
             * Checks if `value` is a plain object, that is, an object created by the
             * `Object` constructor or one with a `[[Prototype]]` of `null`.
             *
             * @static
             * @memberOf _
             * @since 0.8.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a plain object, else `false`.
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             * }
             *
             * _.isPlainObject(new Foo);
             * // => false
             *
             * _.isPlainObject([1, 2, 3]);
             * // => false
             *
             * _.isPlainObject({ 'x': 0, 'y': 0 });
             * // => true
             *
             * _.isPlainObject(Object.create(null));
             * // => true
             */
            function isPlainObject(value) {
                if (!isObjectLike(value) || baseGetTag(value) != objectTag) {
                    return false;
                }
                var proto = getPrototype(value);
                if (proto === null) {
                    return true;
                }
                var Ctor = hasOwnProperty.call(proto, 'constructor') && proto.constructor;
                return typeof Ctor == 'function' && Ctor instanceof Ctor &&
                    funcToString.call(Ctor) == objectCtorString;
            }
            /**
             * Checks if `value` is classified as a `RegExp` object.
             *
             * @static
             * @memberOf _
             * @since 0.1.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a regexp, else `false`.
             * @example
             *
             * _.isRegExp(/abc/);
             * // => true
             *
             * _.isRegExp('/abc/');
             * // => false
             */
            var isRegExp = nodeIsRegExp ? baseUnary(nodeIsRegExp) : baseIsRegExp;
            /**
             * Checks if `value` is a safe integer. An integer is safe if it's an IEEE-754
             * double precision number which isn't the result of a rounded unsafe integer.
             *
             * **Note:** This method is based on
             * [`Number.isSafeInteger`](https://mdn.io/Number/isSafeInteger).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a safe integer, else `false`.
             * @example
             *
             * _.isSafeInteger(3);
             * // => true
             *
             * _.isSafeInteger(Number.MIN_VALUE);
             * // => false
             *
             * _.isSafeInteger(Infinity);
             * // => false
             *
             * _.isSafeInteger('3');
             * // => false
             */
            function isSafeInteger(value) {
                return isInteger(value) && value >= -MAX_SAFE_INTEGER && value <= MAX_SAFE_INTEGER;
            }
            /**
             * Checks if `value` is classified as a `Set` object.
             *
             * @static
             * @memberOf _
             * @since 4.3.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a set, else `false`.
             * @example
             *
             * _.isSet(new Set);
             * // => true
             *
             * _.isSet(new WeakSet);
             * // => false
             */
            var isSet = nodeIsSet ? baseUnary(nodeIsSet) : baseIsSet;
            /**
             * Checks if `value` is classified as a `String` primitive or object.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a string, else `false`.
             * @example
             *
             * _.isString('abc');
             * // => true
             *
             * _.isString(1);
             * // => false
             */
            function isString(value) {
                return typeof value == 'string' ||
                    (!isArray(value) && isObjectLike(value) && baseGetTag(value) == stringTag);
            }
            /**
             * Checks if `value` is classified as a `Symbol` primitive or object.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
             * @example
             *
             * _.isSymbol(Symbol.iterator);
             * // => true
             *
             * _.isSymbol('abc');
             * // => false
             */
            function isSymbol(value) {
                return typeof value == 'symbol' ||
                    (isObjectLike(value) && baseGetTag(value) == symbolTag);
            }
            /**
             * Checks if `value` is classified as a typed array.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a typed array, else `false`.
             * @example
             *
             * _.isTypedArray(new Uint8Array);
             * // => true
             *
             * _.isTypedArray([]);
             * // => false
             */
            var isTypedArray = nodeIsTypedArray ? baseUnary(nodeIsTypedArray) : baseIsTypedArray;
            /**
             * Checks if `value` is `undefined`.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is `undefined`, else `false`.
             * @example
             *
             * _.isUndefined(void 0);
             * // => true
             *
             * _.isUndefined(null);
             * // => false
             */
            function isUndefined(value) {
                return value === undefined;
            }
            /**
             * Checks if `value` is classified as a `WeakMap` object.
             *
             * @static
             * @memberOf _
             * @since 4.3.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a weak map, else `false`.
             * @example
             *
             * _.isWeakMap(new WeakMap);
             * // => true
             *
             * _.isWeakMap(new Map);
             * // => false
             */
            function isWeakMap(value) {
                return isObjectLike(value) && getTag(value) == weakMapTag;
            }
            /**
             * Checks if `value` is classified as a `WeakSet` object.
             *
             * @static
             * @memberOf _
             * @since 4.3.0
             * @category Lang
             * @param {*} value The value to check.
             * @returns {boolean} Returns `true` if `value` is a weak set, else `false`.
             * @example
             *
             * _.isWeakSet(new WeakSet);
             * // => true
             *
             * _.isWeakSet(new Set);
             * // => false
             */
            function isWeakSet(value) {
                return isObjectLike(value) && baseGetTag(value) == weakSetTag;
            }
            /**
             * Checks if `value` is less than `other`.
             *
             * @static
             * @memberOf _
             * @since 3.9.0
             * @category Lang
             * @param {*} value The value to compare.
             * @param {*} other The other value to compare.
             * @returns {boolean} Returns `true` if `value` is less than `other`,
             *  else `false`.
             * @see _.gt
             * @example
             *
             * _.lt(1, 3);
             * // => true
             *
             * _.lt(3, 3);
             * // => false
             *
             * _.lt(3, 1);
             * // => false
             */
            var lt = createRelationalOperation(baseLt);
            /**
             * Checks if `value` is less than or equal to `other`.
             *
             * @static
             * @memberOf _
             * @since 3.9.0
             * @category Lang
             * @param {*} value The value to compare.
             * @param {*} other The other value to compare.
             * @returns {boolean} Returns `true` if `value` is less than or equal to
             *  `other`, else `false`.
             * @see _.gte
             * @example
             *
             * _.lte(1, 3);
             * // => true
             *
             * _.lte(3, 3);
             * // => true
             *
             * _.lte(3, 1);
             * // => false
             */
            var lte = createRelationalOperation(function (value, other) {
                return value <= other;
            });
            /**
             * Converts `value` to an array.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Lang
             * @param {*} value The value to convert.
             * @returns {Array} Returns the converted array.
             * @example
             *
             * _.toArray({ 'a': 1, 'b': 2 });
             * // => [1, 2]
             *
             * _.toArray('abc');
             * // => ['a', 'b', 'c']
             *
             * _.toArray(1);
             * // => []
             *
             * _.toArray(null);
             * // => []
             */
            function toArray(value) {
                if (!value) {
                    return [];
                }
                if (isArrayLike(value)) {
                    return isString(value) ? stringToArray(value) : copyArray(value);
                }
                if (symIterator && value[symIterator]) {
                    return iteratorToArray(value[symIterator]());
                }
                var tag = getTag(value), func = tag == mapTag ? mapToArray : (tag == setTag ? setToArray : values);
                return func(value);
            }
            /**
             * Converts `value` to a finite number.
             *
             * @static
             * @memberOf _
             * @since 4.12.0
             * @category Lang
             * @param {*} value The value to convert.
             * @returns {number} Returns the converted number.
             * @example
             *
             * _.toFinite(3.2);
             * // => 3.2
             *
             * _.toFinite(Number.MIN_VALUE);
             * // => 5e-324
             *
             * _.toFinite(Infinity);
             * // => 1.7976931348623157e+308
             *
             * _.toFinite('3.2');
             * // => 3.2
             */
            function toFinite(value) {
                if (!value) {
                    return value === 0 ? value : 0;
                }
                value = toNumber(value);
                if (value === INFINITY || value === -INFINITY) {
                    var sign = (value < 0 ? -1 : 1);
                    return sign * MAX_INTEGER;
                }
                return value === value ? value : 0;
            }
            /**
             * Converts `value` to an integer.
             *
             * **Note:** This method is loosely based on
             * [`ToInteger`](http://www.ecma-international.org/ecma-262/7.0/#sec-tointeger).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to convert.
             * @returns {number} Returns the converted integer.
             * @example
             *
             * _.toInteger(3.2);
             * // => 3
             *
             * _.toInteger(Number.MIN_VALUE);
             * // => 0
             *
             * _.toInteger(Infinity);
             * // => 1.7976931348623157e+308
             *
             * _.toInteger('3.2');
             * // => 3
             */
            function toInteger(value) {
                var result = toFinite(value), remainder = result % 1;
                return result === result ? (remainder ? result - remainder : result) : 0;
            }
            /**
             * Converts `value` to an integer suitable for use as the length of an
             * array-like object.
             *
             * **Note:** This method is based on
             * [`ToLength`](http://ecma-international.org/ecma-262/7.0/#sec-tolength).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to convert.
             * @returns {number} Returns the converted integer.
             * @example
             *
             * _.toLength(3.2);
             * // => 3
             *
             * _.toLength(Number.MIN_VALUE);
             * // => 0
             *
             * _.toLength(Infinity);
             * // => 4294967295
             *
             * _.toLength('3.2');
             * // => 3
             */
            function toLength(value) {
                return value ? baseClamp(toInteger(value), 0, MAX_ARRAY_LENGTH) : 0;
            }
            /**
             * Converts `value` to a number.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to process.
             * @returns {number} Returns the number.
             * @example
             *
             * _.toNumber(3.2);
             * // => 3.2
             *
             * _.toNumber(Number.MIN_VALUE);
             * // => 5e-324
             *
             * _.toNumber(Infinity);
             * // => Infinity
             *
             * _.toNumber('3.2');
             * // => 3.2
             */
            function toNumber(value) {
                if (typeof value == 'number') {
                    return value;
                }
                if (isSymbol(value)) {
                    return NAN;
                }
                if (isObject(value)) {
                    var other = typeof value.valueOf == 'function' ? value.valueOf() : value;
                    value = isObject(other) ? (other + '') : other;
                }
                if (typeof value != 'string') {
                    return value === 0 ? value : +value;
                }
                value = value.replace(reTrim, '');
                var isBinary = reIsBinary.test(value);
                return (isBinary || reIsOctal.test(value))
                    ? freeParseInt(value.slice(2), isBinary ? 2 : 8)
                    : (reIsBadHex.test(value) ? NAN : +value);
            }
            /**
             * Converts `value` to a plain object flattening inherited enumerable string
             * keyed properties of `value` to own properties of the plain object.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Lang
             * @param {*} value The value to convert.
             * @returns {Object} Returns the converted plain object.
             * @example
             *
             * function Foo() {
             *   this.b = 2;
             * }
             *
             * Foo.prototype.c = 3;
             *
             * _.assign({ 'a': 1 }, new Foo);
             * // => { 'a': 1, 'b': 2 }
             *
             * _.assign({ 'a': 1 }, _.toPlainObject(new Foo));
             * // => { 'a': 1, 'b': 2, 'c': 3 }
             */
            function toPlainObject(value) {
                return copyObject(value, keysIn(value));
            }
            /**
             * Converts `value` to a safe integer. A safe integer can be compared and
             * represented correctly.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to convert.
             * @returns {number} Returns the converted integer.
             * @example
             *
             * _.toSafeInteger(3.2);
             * // => 3
             *
             * _.toSafeInteger(Number.MIN_VALUE);
             * // => 0
             *
             * _.toSafeInteger(Infinity);
             * // => 9007199254740991
             *
             * _.toSafeInteger('3.2');
             * // => 3
             */
            function toSafeInteger(value) {
                return value
                    ? baseClamp(toInteger(value), -MAX_SAFE_INTEGER, MAX_SAFE_INTEGER)
                    : (value === 0 ? value : 0);
            }
            /**
             * Converts `value` to a string. An empty string is returned for `null`
             * and `undefined` values. The sign of `-0` is preserved.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Lang
             * @param {*} value The value to convert.
             * @returns {string} Returns the converted string.
             * @example
             *
             * _.toString(null);
             * // => ''
             *
             * _.toString(-0);
             * // => '-0'
             *
             * _.toString([1, 2, 3]);
             * // => '1,2,3'
             */
            function toString(value) {
                return value == null ? '' : baseToString(value);
            }
            /*------------------------------------------------------------------------*/
            /**
             * Assigns own enumerable string keyed properties of source objects to the
             * destination object. Source objects are applied from left to right.
             * Subsequent sources overwrite property assignments of previous sources.
             *
             * **Note:** This method mutates `object` and is loosely based on
             * [`Object.assign`](https://mdn.io/Object/assign).
             *
             * @static
             * @memberOf _
             * @since 0.10.0
             * @category Object
             * @param {Object} object The destination object.
             * @param {...Object} [sources] The source objects.
             * @returns {Object} Returns `object`.
             * @see _.assignIn
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             * }
             *
             * function Bar() {
             *   this.c = 3;
             * }
             *
             * Foo.prototype.b = 2;
             * Bar.prototype.d = 4;
             *
             * _.assign({ 'a': 0 }, new Foo, new Bar);
             * // => { 'a': 1, 'c': 3 }
             */
            var assign = createAssigner(function (object, source) {
                if (isPrototype(source) || isArrayLike(source)) {
                    copyObject(source, keys(source), object);
                    return;
                }
                for (var key in source) {
                    if (hasOwnProperty.call(source, key)) {
                        assignValue(object, key, source[key]);
                    }
                }
            });
            /**
             * This method is like `_.assign` except that it iterates over own and
             * inherited source properties.
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @alias extend
             * @category Object
             * @param {Object} object The destination object.
             * @param {...Object} [sources] The source objects.
             * @returns {Object} Returns `object`.
             * @see _.assign
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             * }
             *
             * function Bar() {
             *   this.c = 3;
             * }
             *
             * Foo.prototype.b = 2;
             * Bar.prototype.d = 4;
             *
             * _.assignIn({ 'a': 0 }, new Foo, new Bar);
             * // => { 'a': 1, 'b': 2, 'c': 3, 'd': 4 }
             */
            var assignIn = createAssigner(function (object, source) {
                copyObject(source, keysIn(source), object);
            });
            /**
             * This method is like `_.assignIn` except that it accepts `customizer`
             * which is invoked to produce the assigned values. If `customizer` returns
             * `undefined`, assignment is handled by the method instead. The `customizer`
             * is invoked with five arguments: (objValue, srcValue, key, object, source).
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @alias extendWith
             * @category Object
             * @param {Object} object The destination object.
             * @param {...Object} sources The source objects.
             * @param {Function} [customizer] The function to customize assigned values.
             * @returns {Object} Returns `object`.
             * @see _.assignWith
             * @example
             *
             * function customizer(objValue, srcValue) {
             *   return _.isUndefined(objValue) ? srcValue : objValue;
             * }
             *
             * var defaults = _.partialRight(_.assignInWith, customizer);
             *
             * defaults({ 'a': 1 }, { 'b': 2 }, { 'a': 3 });
             * // => { 'a': 1, 'b': 2 }
             */
            var assignInWith = createAssigner(function (object, source, srcIndex, customizer) {
                copyObject(source, keysIn(source), object, customizer);
            });
            /**
             * This method is like `_.assign` except that it accepts `customizer`
             * which is invoked to produce the assigned values. If `customizer` returns
             * `undefined`, assignment is handled by the method instead. The `customizer`
             * is invoked with five arguments: (objValue, srcValue, key, object, source).
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Object
             * @param {Object} object The destination object.
             * @param {...Object} sources The source objects.
             * @param {Function} [customizer] The function to customize assigned values.
             * @returns {Object} Returns `object`.
             * @see _.assignInWith
             * @example
             *
             * function customizer(objValue, srcValue) {
             *   return _.isUndefined(objValue) ? srcValue : objValue;
             * }
             *
             * var defaults = _.partialRight(_.assignWith, customizer);
             *
             * defaults({ 'a': 1 }, { 'b': 2 }, { 'a': 3 });
             * // => { 'a': 1, 'b': 2 }
             */
            var assignWith = createAssigner(function (object, source, srcIndex, customizer) {
                copyObject(source, keys(source), object, customizer);
            });
            /**
             * Creates an array of values corresponding to `paths` of `object`.
             *
             * @static
             * @memberOf _
             * @since 1.0.0
             * @category Object
             * @param {Object} object The object to iterate over.
             * @param {...(string|string[])} [paths] The property paths to pick.
             * @returns {Array} Returns the picked values.
             * @example
             *
             * var object = { 'a': [{ 'b': { 'c': 3 } }, 4] };
             *
             * _.at(object, ['a[0].b.c', 'a[1]']);
             * // => [3, 4]
             */
            var at = flatRest(baseAt);
            /**
             * Creates an object that inherits from the `prototype` object. If a
             * `properties` object is given, its own enumerable string keyed properties
             * are assigned to the created object.
             *
             * @static
             * @memberOf _
             * @since 2.3.0
             * @category Object
             * @param {Object} prototype The object to inherit from.
             * @param {Object} [properties] The properties to assign to the object.
             * @returns {Object} Returns the new object.
             * @example
             *
             * function Shape() {
             *   this.x = 0;
             *   this.y = 0;
             * }
             *
             * function Circle() {
             *   Shape.call(this);
             * }
             *
             * Circle.prototype = _.create(Shape.prototype, {
             *   'constructor': Circle
             * });
             *
             * var circle = new Circle;
             * circle instanceof Circle;
             * // => true
             *
             * circle instanceof Shape;
             * // => true
             */
            function create(prototype, properties) {
                var result = baseCreate(prototype);
                return properties == null ? result : baseAssign(result, properties);
            }
            /**
             * Assigns own and inherited enumerable string keyed properties of source
             * objects to the destination object for all destination properties that
             * resolve to `undefined`. Source objects are applied from left to right.
             * Once a property is set, additional values of the same property are ignored.
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Object
             * @param {Object} object The destination object.
             * @param {...Object} [sources] The source objects.
             * @returns {Object} Returns `object`.
             * @see _.defaultsDeep
             * @example
             *
             * _.defaults({ 'a': 1 }, { 'b': 2 }, { 'a': 3 });
             * // => { 'a': 1, 'b': 2 }
             */
            var defaults = baseRest(function (object, sources) {
                object = Object(object);
                var index = -1;
                var length = sources.length;
                var guard = length > 2 ? sources[2] : undefined;
                if (guard && isIterateeCall(sources[0], sources[1], guard)) {
                    length = 1;
                }
                while (++index < length) {
                    var source = sources[index];
                    var props = keysIn(source);
                    var propsIndex = -1;
                    var propsLength = props.length;
                    while (++propsIndex < propsLength) {
                        var key = props[propsIndex];
                        var value = object[key];
                        if (value === undefined ||
                            (eq(value, objectProto[key]) && !hasOwnProperty.call(object, key))) {
                            object[key] = source[key];
                        }
                    }
                }
                return object;
            });
            /**
             * This method is like `_.defaults` except that it recursively assigns
             * default properties.
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @memberOf _
             * @since 3.10.0
             * @category Object
             * @param {Object} object The destination object.
             * @param {...Object} [sources] The source objects.
             * @returns {Object} Returns `object`.
             * @see _.defaults
             * @example
             *
             * _.defaultsDeep({ 'a': { 'b': 2 } }, { 'a': { 'b': 1, 'c': 3 } });
             * // => { 'a': { 'b': 2, 'c': 3 } }
             */
            var defaultsDeep = baseRest(function (args) {
                args.push(undefined, customDefaultsMerge);
                return apply(mergeWith, undefined, args);
            });
            /**
             * This method is like `_.find` except that it returns the key of the first
             * element `predicate` returns truthy for instead of the element itself.
             *
             * @static
             * @memberOf _
             * @since 1.1.0
             * @category Object
             * @param {Object} object The object to inspect.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @returns {string|undefined} Returns the key of the matched element,
             *  else `undefined`.
             * @example
             *
             * var users = {
             *   'barney':  { 'age': 36, 'active': true },
             *   'fred':    { 'age': 40, 'active': false },
             *   'pebbles': { 'age': 1,  'active': true }
             * };
             *
             * _.findKey(users, function(o) { return o.age < 40; });
             * // => 'barney' (iteration order is not guaranteed)
             *
             * // The `_.matches` iteratee shorthand.
             * _.findKey(users, { 'age': 1, 'active': true });
             * // => 'pebbles'
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.findKey(users, ['active', false]);
             * // => 'fred'
             *
             * // The `_.property` iteratee shorthand.
             * _.findKey(users, 'active');
             * // => 'barney'
             */
            function findKey(object, predicate) {
                return baseFindKey(object, getIteratee(predicate, 3), baseForOwn);
            }
            /**
             * This method is like `_.findKey` except that it iterates over elements of
             * a collection in the opposite order.
             *
             * @static
             * @memberOf _
             * @since 2.0.0
             * @category Object
             * @param {Object} object The object to inspect.
             * @param {Function} [predicate=_.identity] The function invoked per iteration.
             * @returns {string|undefined} Returns the key of the matched element,
             *  else `undefined`.
             * @example
             *
             * var users = {
             *   'barney':  { 'age': 36, 'active': true },
             *   'fred':    { 'age': 40, 'active': false },
             *   'pebbles': { 'age': 1,  'active': true }
             * };
             *
             * _.findLastKey(users, function(o) { return o.age < 40; });
             * // => returns 'pebbles' assuming `_.findKey` returns 'barney'
             *
             * // The `_.matches` iteratee shorthand.
             * _.findLastKey(users, { 'age': 36, 'active': true });
             * // => 'barney'
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.findLastKey(users, ['active', false]);
             * // => 'fred'
             *
             * // The `_.property` iteratee shorthand.
             * _.findLastKey(users, 'active');
             * // => 'pebbles'
             */
            function findLastKey(object, predicate) {
                return baseFindKey(object, getIteratee(predicate, 3), baseForOwnRight);
            }
            /**
             * Iterates over own and inherited enumerable string keyed properties of an
             * object and invokes `iteratee` for each property. The iteratee is invoked
             * with three arguments: (value, key, object). Iteratee functions may exit
             * iteration early by explicitly returning `false`.
             *
             * @static
             * @memberOf _
             * @since 0.3.0
             * @category Object
             * @param {Object} object The object to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Object} Returns `object`.
             * @see _.forInRight
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             *   this.b = 2;
             * }
             *
             * Foo.prototype.c = 3;
             *
             * _.forIn(new Foo, function(value, key) {
             *   console.log(key);
             * });
             * // => Logs 'a', 'b', then 'c' (iteration order is not guaranteed).
             */
            function forIn(object, iteratee) {
                return object == null
                    ? object
                    : baseFor(object, getIteratee(iteratee, 3), keysIn);
            }
            /**
             * This method is like `_.forIn` except that it iterates over properties of
             * `object` in the opposite order.
             *
             * @static
             * @memberOf _
             * @since 2.0.0
             * @category Object
             * @param {Object} object The object to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Object} Returns `object`.
             * @see _.forIn
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             *   this.b = 2;
             * }
             *
             * Foo.prototype.c = 3;
             *
             * _.forInRight(new Foo, function(value, key) {
             *   console.log(key);
             * });
             * // => Logs 'c', 'b', then 'a' assuming `_.forIn` logs 'a', 'b', then 'c'.
             */
            function forInRight(object, iteratee) {
                return object == null
                    ? object
                    : baseForRight(object, getIteratee(iteratee, 3), keysIn);
            }
            /**
             * Iterates over own enumerable string keyed properties of an object and
             * invokes `iteratee` for each property. The iteratee is invoked with three
             * arguments: (value, key, object). Iteratee functions may exit iteration
             * early by explicitly returning `false`.
             *
             * @static
             * @memberOf _
             * @since 0.3.0
             * @category Object
             * @param {Object} object The object to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Object} Returns `object`.
             * @see _.forOwnRight
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             *   this.b = 2;
             * }
             *
             * Foo.prototype.c = 3;
             *
             * _.forOwn(new Foo, function(value, key) {
             *   console.log(key);
             * });
             * // => Logs 'a' then 'b' (iteration order is not guaranteed).
             */
            function forOwn(object, iteratee) {
                return object && baseForOwn(object, getIteratee(iteratee, 3));
            }
            /**
             * This method is like `_.forOwn` except that it iterates over properties of
             * `object` in the opposite order.
             *
             * @static
             * @memberOf _
             * @since 2.0.0
             * @category Object
             * @param {Object} object The object to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Object} Returns `object`.
             * @see _.forOwn
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             *   this.b = 2;
             * }
             *
             * Foo.prototype.c = 3;
             *
             * _.forOwnRight(new Foo, function(value, key) {
             *   console.log(key);
             * });
             * // => Logs 'b' then 'a' assuming `_.forOwn` logs 'a' then 'b'.
             */
            function forOwnRight(object, iteratee) {
                return object && baseForOwnRight(object, getIteratee(iteratee, 3));
            }
            /**
             * Creates an array of function property names from own enumerable properties
             * of `object`.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Object
             * @param {Object} object The object to inspect.
             * @returns {Array} Returns the function names.
             * @see _.functionsIn
             * @example
             *
             * function Foo() {
             *   this.a = _.constant('a');
             *   this.b = _.constant('b');
             * }
             *
             * Foo.prototype.c = _.constant('c');
             *
             * _.functions(new Foo);
             * // => ['a', 'b']
             */
            function functions(object) {
                return object == null ? [] : baseFunctions(object, keys(object));
            }
            /**
             * Creates an array of function property names from own and inherited
             * enumerable properties of `object`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Object
             * @param {Object} object The object to inspect.
             * @returns {Array} Returns the function names.
             * @see _.functions
             * @example
             *
             * function Foo() {
             *   this.a = _.constant('a');
             *   this.b = _.constant('b');
             * }
             *
             * Foo.prototype.c = _.constant('c');
             *
             * _.functionsIn(new Foo);
             * // => ['a', 'b', 'c']
             */
            function functionsIn(object) {
                return object == null ? [] : baseFunctions(object, keysIn(object));
            }
            /**
             * Gets the value at `path` of `object`. If the resolved value is
             * `undefined`, the `defaultValue` is returned in its place.
             *
             * @static
             * @memberOf _
             * @since 3.7.0
             * @category Object
             * @param {Object} object The object to query.
             * @param {Array|string} path The path of the property to get.
             * @param {*} [defaultValue] The value returned for `undefined` resolved values.
             * @returns {*} Returns the resolved value.
             * @example
             *
             * var object = { 'a': [{ 'b': { 'c': 3 } }] };
             *
             * _.get(object, 'a[0].b.c');
             * // => 3
             *
             * _.get(object, ['a', '0', 'b', 'c']);
             * // => 3
             *
             * _.get(object, 'a.b.c', 'default');
             * // => 'default'
             */
            function get(object, path, defaultValue) {
                var result = object == null ? undefined : baseGet(object, path);
                return result === undefined ? defaultValue : result;
            }
            /**
             * Checks if `path` is a direct property of `object`.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Object
             * @param {Object} object The object to query.
             * @param {Array|string} path The path to check.
             * @returns {boolean} Returns `true` if `path` exists, else `false`.
             * @example
             *
             * var object = { 'a': { 'b': 2 } };
             * var other = _.create({ 'a': _.create({ 'b': 2 }) });
             *
             * _.has(object, 'a');
             * // => true
             *
             * _.has(object, 'a.b');
             * // => true
             *
             * _.has(object, ['a', 'b']);
             * // => true
             *
             * _.has(other, 'a');
             * // => false
             */
            function has(object, path) {
                return object != null && hasPath(object, path, baseHas);
            }
            /**
             * Checks if `path` is a direct or inherited property of `object`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Object
             * @param {Object} object The object to query.
             * @param {Array|string} path The path to check.
             * @returns {boolean} Returns `true` if `path` exists, else `false`.
             * @example
             *
             * var object = _.create({ 'a': _.create({ 'b': 2 }) });
             *
             * _.hasIn(object, 'a');
             * // => true
             *
             * _.hasIn(object, 'a.b');
             * // => true
             *
             * _.hasIn(object, ['a', 'b']);
             * // => true
             *
             * _.hasIn(object, 'b');
             * // => false
             */
            function hasIn(object, path) {
                return object != null && hasPath(object, path, baseHasIn);
            }
            /**
             * Creates an object composed of the inverted keys and values of `object`.
             * If `object` contains duplicate values, subsequent values overwrite
             * property assignments of previous values.
             *
             * @static
             * @memberOf _
             * @since 0.7.0
             * @category Object
             * @param {Object} object The object to invert.
             * @returns {Object} Returns the new inverted object.
             * @example
             *
             * var object = { 'a': 1, 'b': 2, 'c': 1 };
             *
             * _.invert(object);
             * // => { '1': 'c', '2': 'b' }
             */
            var invert = createInverter(function (result, value, key) {
                if (value != null &&
                    typeof value.toString != 'function') {
                    value = nativeObjectToString.call(value);
                }
                result[value] = key;
            }, constant(identity));
            /**
             * This method is like `_.invert` except that the inverted object is generated
             * from the results of running each element of `object` thru `iteratee`. The
             * corresponding inverted value of each inverted key is an array of keys
             * responsible for generating the inverted value. The iteratee is invoked
             * with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 4.1.0
             * @category Object
             * @param {Object} object The object to invert.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {Object} Returns the new inverted object.
             * @example
             *
             * var object = { 'a': 1, 'b': 2, 'c': 1 };
             *
             * _.invertBy(object);
             * // => { '1': ['a', 'c'], '2': ['b'] }
             *
             * _.invertBy(object, function(value) {
             *   return 'group' + value;
             * });
             * // => { 'group1': ['a', 'c'], 'group2': ['b'] }
             */
            var invertBy = createInverter(function (result, value, key) {
                if (value != null &&
                    typeof value.toString != 'function') {
                    value = nativeObjectToString.call(value);
                }
                if (hasOwnProperty.call(result, value)) {
                    result[value].push(key);
                }
                else {
                    result[value] = [key];
                }
            }, getIteratee);
            /**
             * Invokes the method at `path` of `object`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Object
             * @param {Object} object The object to query.
             * @param {Array|string} path The path of the method to invoke.
             * @param {...*} [args] The arguments to invoke the method with.
             * @returns {*} Returns the result of the invoked method.
             * @example
             *
             * var object = { 'a': [{ 'b': { 'c': [1, 2, 3, 4] } }] };
             *
             * _.invoke(object, 'a[0].b.c.slice', 1, 3);
             * // => [2, 3]
             */
            var invoke = baseRest(baseInvoke);
            /**
             * Creates an array of the own enumerable property names of `object`.
             *
             * **Note:** Non-object values are coerced to objects. See the
             * [ES spec](http://ecma-international.org/ecma-262/7.0/#sec-object.keys)
             * for more details.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Object
             * @param {Object} object The object to query.
             * @returns {Array} Returns the array of property names.
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             *   this.b = 2;
             * }
             *
             * Foo.prototype.c = 3;
             *
             * _.keys(new Foo);
             * // => ['a', 'b'] (iteration order is not guaranteed)
             *
             * _.keys('hi');
             * // => ['0', '1']
             */
            function keys(object) {
                return isArrayLike(object) ? arrayLikeKeys(object) : baseKeys(object);
            }
            /**
             * Creates an array of the own and inherited enumerable property names of `object`.
             *
             * **Note:** Non-object values are coerced to objects.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Object
             * @param {Object} object The object to query.
             * @returns {Array} Returns the array of property names.
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             *   this.b = 2;
             * }
             *
             * Foo.prototype.c = 3;
             *
             * _.keysIn(new Foo);
             * // => ['a', 'b', 'c'] (iteration order is not guaranteed)
             */
            function keysIn(object) {
                return isArrayLike(object) ? arrayLikeKeys(object, true) : baseKeysIn(object);
            }
            /**
             * The opposite of `_.mapValues`; this method creates an object with the
             * same values as `object` and keys generated by running each own enumerable
             * string keyed property of `object` thru `iteratee`. The iteratee is invoked
             * with three arguments: (value, key, object).
             *
             * @static
             * @memberOf _
             * @since 3.8.0
             * @category Object
             * @param {Object} object The object to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Object} Returns the new mapped object.
             * @see _.mapValues
             * @example
             *
             * _.mapKeys({ 'a': 1, 'b': 2 }, function(value, key) {
             *   return key + value;
             * });
             * // => { 'a1': 1, 'b2': 2 }
             */
            function mapKeys(object, iteratee) {
                var result = {};
                iteratee = getIteratee(iteratee, 3);
                baseForOwn(object, function (value, key, object) {
                    baseAssignValue(result, iteratee(value, key, object), value);
                });
                return result;
            }
            /**
             * Creates an object with the same keys as `object` and values generated
             * by running each own enumerable string keyed property of `object` thru
             * `iteratee`. The iteratee is invoked with three arguments:
             * (value, key, object).
             *
             * @static
             * @memberOf _
             * @since 2.4.0
             * @category Object
             * @param {Object} object The object to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Object} Returns the new mapped object.
             * @see _.mapKeys
             * @example
             *
             * var users = {
             *   'fred':    { 'user': 'fred',    'age': 40 },
             *   'pebbles': { 'user': 'pebbles', 'age': 1 }
             * };
             *
             * _.mapValues(users, function(o) { return o.age; });
             * // => { 'fred': 40, 'pebbles': 1 } (iteration order is not guaranteed)
             *
             * // The `_.property` iteratee shorthand.
             * _.mapValues(users, 'age');
             * // => { 'fred': 40, 'pebbles': 1 } (iteration order is not guaranteed)
             */
            function mapValues(object, iteratee) {
                var result = {};
                iteratee = getIteratee(iteratee, 3);
                baseForOwn(object, function (value, key, object) {
                    baseAssignValue(result, key, iteratee(value, key, object));
                });
                return result;
            }
            /**
             * This method is like `_.assign` except that it recursively merges own and
             * inherited enumerable string keyed properties of source objects into the
             * destination object. Source properties that resolve to `undefined` are
             * skipped if a destination value exists. Array and plain object properties
             * are merged recursively. Other objects and value types are overridden by
             * assignment. Source objects are applied from left to right. Subsequent
             * sources overwrite property assignments of previous sources.
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @memberOf _
             * @since 0.5.0
             * @category Object
             * @param {Object} object The destination object.
             * @param {...Object} [sources] The source objects.
             * @returns {Object} Returns `object`.
             * @example
             *
             * var object = {
             *   'a': [{ 'b': 2 }, { 'd': 4 }]
             * };
             *
             * var other = {
             *   'a': [{ 'c': 3 }, { 'e': 5 }]
             * };
             *
             * _.merge(object, other);
             * // => { 'a': [{ 'b': 2, 'c': 3 }, { 'd': 4, 'e': 5 }] }
             */
            var merge = createAssigner(function (object, source, srcIndex) {
                baseMerge(object, source, srcIndex);
            });
            /**
             * This method is like `_.merge` except that it accepts `customizer` which
             * is invoked to produce the merged values of the destination and source
             * properties. If `customizer` returns `undefined`, merging is handled by the
             * method instead. The `customizer` is invoked with six arguments:
             * (objValue, srcValue, key, object, source, stack).
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Object
             * @param {Object} object The destination object.
             * @param {...Object} sources The source objects.
             * @param {Function} customizer The function to customize assigned values.
             * @returns {Object} Returns `object`.
             * @example
             *
             * function customizer(objValue, srcValue) {
             *   if (_.isArray(objValue)) {
             *     return objValue.concat(srcValue);
             *   }
             * }
             *
             * var object = { 'a': [1], 'b': [2] };
             * var other = { 'a': [3], 'b': [4] };
             *
             * _.mergeWith(object, other, customizer);
             * // => { 'a': [1, 3], 'b': [2, 4] }
             */
            var mergeWith = createAssigner(function (object, source, srcIndex, customizer) {
                baseMerge(object, source, srcIndex, customizer);
            });
            /**
             * The opposite of `_.pick`; this method creates an object composed of the
             * own and inherited enumerable property paths of `object` that are not omitted.
             *
             * **Note:** This method is considerably slower than `_.pick`.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Object
             * @param {Object} object The source object.
             * @param {...(string|string[])} [paths] The property paths to omit.
             * @returns {Object} Returns the new object.
             * @example
             *
             * var object = { 'a': 1, 'b': '2', 'c': 3 };
             *
             * _.omit(object, ['a', 'c']);
             * // => { 'b': '2' }
             */
            var omit = flatRest(function (object, paths) {
                var result = {};
                if (object == null) {
                    return result;
                }
                var isDeep = false;
                paths = arrayMap(paths, function (path) {
                    path = castPath(path, object);
                    isDeep || (isDeep = path.length > 1);
                    return path;
                });
                copyObject(object, getAllKeysIn(object), result);
                if (isDeep) {
                    result = baseClone(result, CLONE_DEEP_FLAG | CLONE_FLAT_FLAG | CLONE_SYMBOLS_FLAG, customOmitClone);
                }
                var length = paths.length;
                while (length--) {
                    baseUnset(result, paths[length]);
                }
                return result;
            });
            /**
             * The opposite of `_.pickBy`; this method creates an object composed of
             * the own and inherited enumerable string keyed properties of `object` that
             * `predicate` doesn't return truthy for. The predicate is invoked with two
             * arguments: (value, key).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Object
             * @param {Object} object The source object.
             * @param {Function} [predicate=_.identity] The function invoked per property.
             * @returns {Object} Returns the new object.
             * @example
             *
             * var object = { 'a': 1, 'b': '2', 'c': 3 };
             *
             * _.omitBy(object, _.isNumber);
             * // => { 'b': '2' }
             */
            function omitBy(object, predicate) {
                return pickBy(object, negate(getIteratee(predicate)));
            }
            /**
             * Creates an object composed of the picked `object` properties.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Object
             * @param {Object} object The source object.
             * @param {...(string|string[])} [paths] The property paths to pick.
             * @returns {Object} Returns the new object.
             * @example
             *
             * var object = { 'a': 1, 'b': '2', 'c': 3 };
             *
             * _.pick(object, ['a', 'c']);
             * // => { 'a': 1, 'c': 3 }
             */
            var pick = flatRest(function (object, paths) {
                return object == null ? {} : basePick(object, paths);
            });
            /**
             * Creates an object composed of the `object` properties `predicate` returns
             * truthy for. The predicate is invoked with two arguments: (value, key).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Object
             * @param {Object} object The source object.
             * @param {Function} [predicate=_.identity] The function invoked per property.
             * @returns {Object} Returns the new object.
             * @example
             *
             * var object = { 'a': 1, 'b': '2', 'c': 3 };
             *
             * _.pickBy(object, _.isNumber);
             * // => { 'a': 1, 'c': 3 }
             */
            function pickBy(object, predicate) {
                if (object == null) {
                    return {};
                }
                var props = arrayMap(getAllKeysIn(object), function (prop) {
                    return [prop];
                });
                predicate = getIteratee(predicate);
                return basePickBy(object, props, function (value, path) {
                    return predicate(value, path[0]);
                });
            }
            /**
             * This method is like `_.get` except that if the resolved value is a
             * function it's invoked with the `this` binding of its parent object and
             * its result is returned.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Object
             * @param {Object} object The object to query.
             * @param {Array|string} path The path of the property to resolve.
             * @param {*} [defaultValue] The value returned for `undefined` resolved values.
             * @returns {*} Returns the resolved value.
             * @example
             *
             * var object = { 'a': [{ 'b': { 'c1': 3, 'c2': _.constant(4) } }] };
             *
             * _.result(object, 'a[0].b.c1');
             * // => 3
             *
             * _.result(object, 'a[0].b.c2');
             * // => 4
             *
             * _.result(object, 'a[0].b.c3', 'default');
             * // => 'default'
             *
             * _.result(object, 'a[0].b.c3', _.constant('default'));
             * // => 'default'
             */
            function result(object, path, defaultValue) {
                path = castPath(path, object);
                var index = -1, length = path.length;
                // Ensure the loop is entered when path is empty.
                if (!length) {
                    length = 1;
                    object = undefined;
                }
                while (++index < length) {
                    var value = object == null ? undefined : object[toKey(path[index])];
                    if (value === undefined) {
                        index = length;
                        value = defaultValue;
                    }
                    object = isFunction(value) ? value.call(object) : value;
                }
                return object;
            }
            /**
             * Sets the value at `path` of `object`. If a portion of `path` doesn't exist,
             * it's created. Arrays are created for missing index properties while objects
             * are created for all other missing properties. Use `_.setWith` to customize
             * `path` creation.
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @memberOf _
             * @since 3.7.0
             * @category Object
             * @param {Object} object The object to modify.
             * @param {Array|string} path The path of the property to set.
             * @param {*} value The value to set.
             * @returns {Object} Returns `object`.
             * @example
             *
             * var object = { 'a': [{ 'b': { 'c': 3 } }] };
             *
             * _.set(object, 'a[0].b.c', 4);
             * console.log(object.a[0].b.c);
             * // => 4
             *
             * _.set(object, ['x', '0', 'y', 'z'], 5);
             * console.log(object.x[0].y.z);
             * // => 5
             */
            function set(object, path, value) {
                return object == null ? object : baseSet(object, path, value);
            }
            /**
             * This method is like `_.set` except that it accepts `customizer` which is
             * invoked to produce the objects of `path`.  If `customizer` returns `undefined`
             * path creation is handled by the method instead. The `customizer` is invoked
             * with three arguments: (nsValue, key, nsObject).
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Object
             * @param {Object} object The object to modify.
             * @param {Array|string} path The path of the property to set.
             * @param {*} value The value to set.
             * @param {Function} [customizer] The function to customize assigned values.
             * @returns {Object} Returns `object`.
             * @example
             *
             * var object = {};
             *
             * _.setWith(object, '[0][1]', 'a', Object);
             * // => { '0': { '1': 'a' } }
             */
            function setWith(object, path, value, customizer) {
                customizer = typeof customizer == 'function' ? customizer : undefined;
                return object == null ? object : baseSet(object, path, value, customizer);
            }
            /**
             * Creates an array of own enumerable string keyed-value pairs for `object`
             * which can be consumed by `_.fromPairs`. If `object` is a map or set, its
             * entries are returned.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @alias entries
             * @category Object
             * @param {Object} object The object to query.
             * @returns {Array} Returns the key-value pairs.
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             *   this.b = 2;
             * }
             *
             * Foo.prototype.c = 3;
             *
             * _.toPairs(new Foo);
             * // => [['a', 1], ['b', 2]] (iteration order is not guaranteed)
             */
            var toPairs = createToPairs(keys);
            /**
             * Creates an array of own and inherited enumerable string keyed-value pairs
             * for `object` which can be consumed by `_.fromPairs`. If `object` is a map
             * or set, its entries are returned.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @alias entriesIn
             * @category Object
             * @param {Object} object The object to query.
             * @returns {Array} Returns the key-value pairs.
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             *   this.b = 2;
             * }
             *
             * Foo.prototype.c = 3;
             *
             * _.toPairsIn(new Foo);
             * // => [['a', 1], ['b', 2], ['c', 3]] (iteration order is not guaranteed)
             */
            var toPairsIn = createToPairs(keysIn);
            /**
             * An alternative to `_.reduce`; this method transforms `object` to a new
             * `accumulator` object which is the result of running each of its own
             * enumerable string keyed properties thru `iteratee`, with each invocation
             * potentially mutating the `accumulator` object. If `accumulator` is not
             * provided, a new object with the same `[[Prototype]]` will be used. The
             * iteratee is invoked with four arguments: (accumulator, value, key, object).
             * Iteratee functions may exit iteration early by explicitly returning `false`.
             *
             * @static
             * @memberOf _
             * @since 1.3.0
             * @category Object
             * @param {Object} object The object to iterate over.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @param {*} [accumulator] The custom accumulator value.
             * @returns {*} Returns the accumulated value.
             * @example
             *
             * _.transform([2, 3, 4], function(result, n) {
             *   result.push(n *= n);
             *   return n % 2 == 0;
             * }, []);
             * // => [4, 9]
             *
             * _.transform({ 'a': 1, 'b': 2, 'c': 1 }, function(result, value, key) {
             *   (result[value] || (result[value] = [])).push(key);
             * }, {});
             * // => { '1': ['a', 'c'], '2': ['b'] }
             */
            function transform(object, iteratee, accumulator) {
                var isArr = isArray(object), isArrLike = isArr || isBuffer(object) || isTypedArray(object);
                iteratee = getIteratee(iteratee, 4);
                if (accumulator == null) {
                    var Ctor = object && object.constructor;
                    if (isArrLike) {
                        accumulator = isArr ? new Ctor : [];
                    }
                    else if (isObject(object)) {
                        accumulator = isFunction(Ctor) ? baseCreate(getPrototype(object)) : {};
                    }
                    else {
                        accumulator = {};
                    }
                }
                (isArrLike ? arrayEach : baseForOwn)(object, function (value, index, object) {
                    return iteratee(accumulator, value, index, object);
                });
                return accumulator;
            }
            /**
             * Removes the property at `path` of `object`.
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Object
             * @param {Object} object The object to modify.
             * @param {Array|string} path The path of the property to unset.
             * @returns {boolean} Returns `true` if the property is deleted, else `false`.
             * @example
             *
             * var object = { 'a': [{ 'b': { 'c': 7 } }] };
             * _.unset(object, 'a[0].b.c');
             * // => true
             *
             * console.log(object);
             * // => { 'a': [{ 'b': {} }] };
             *
             * _.unset(object, ['a', '0', 'b', 'c']);
             * // => true
             *
             * console.log(object);
             * // => { 'a': [{ 'b': {} }] };
             */
            function unset(object, path) {
                return object == null ? true : baseUnset(object, path);
            }
            /**
             * This method is like `_.set` except that accepts `updater` to produce the
             * value to set. Use `_.updateWith` to customize `path` creation. The `updater`
             * is invoked with one argument: (value).
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @memberOf _
             * @since 4.6.0
             * @category Object
             * @param {Object} object The object to modify.
             * @param {Array|string} path The path of the property to set.
             * @param {Function} updater The function to produce the updated value.
             * @returns {Object} Returns `object`.
             * @example
             *
             * var object = { 'a': [{ 'b': { 'c': 3 } }] };
             *
             * _.update(object, 'a[0].b.c', function(n) { return n * n; });
             * console.log(object.a[0].b.c);
             * // => 9
             *
             * _.update(object, 'x[0].y.z', function(n) { return n ? n + 1 : 0; });
             * console.log(object.x[0].y.z);
             * // => 0
             */
            function update(object, path, updater) {
                return object == null ? object : baseUpdate(object, path, castFunction(updater));
            }
            /**
             * This method is like `_.update` except that it accepts `customizer` which is
             * invoked to produce the objects of `path`.  If `customizer` returns `undefined`
             * path creation is handled by the method instead. The `customizer` is invoked
             * with three arguments: (nsValue, key, nsObject).
             *
             * **Note:** This method mutates `object`.
             *
             * @static
             * @memberOf _
             * @since 4.6.0
             * @category Object
             * @param {Object} object The object to modify.
             * @param {Array|string} path The path of the property to set.
             * @param {Function} updater The function to produce the updated value.
             * @param {Function} [customizer] The function to customize assigned values.
             * @returns {Object} Returns `object`.
             * @example
             *
             * var object = {};
             *
             * _.updateWith(object, '[0][1]', _.constant('a'), Object);
             * // => { '0': { '1': 'a' } }
             */
            function updateWith(object, path, updater, customizer) {
                customizer = typeof customizer == 'function' ? customizer : undefined;
                return object == null ? object : baseUpdate(object, path, castFunction(updater), customizer);
            }
            /**
             * Creates an array of the own enumerable string keyed property values of `object`.
             *
             * **Note:** Non-object values are coerced to objects.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Object
             * @param {Object} object The object to query.
             * @returns {Array} Returns the array of property values.
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             *   this.b = 2;
             * }
             *
             * Foo.prototype.c = 3;
             *
             * _.values(new Foo);
             * // => [1, 2] (iteration order is not guaranteed)
             *
             * _.values('hi');
             * // => ['h', 'i']
             */
            function values(object) {
                return object == null ? [] : baseValues(object, keys(object));
            }
            /**
             * Creates an array of the own and inherited enumerable string keyed property
             * values of `object`.
             *
             * **Note:** Non-object values are coerced to objects.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Object
             * @param {Object} object The object to query.
             * @returns {Array} Returns the array of property values.
             * @example
             *
             * function Foo() {
             *   this.a = 1;
             *   this.b = 2;
             * }
             *
             * Foo.prototype.c = 3;
             *
             * _.valuesIn(new Foo);
             * // => [1, 2, 3] (iteration order is not guaranteed)
             */
            function valuesIn(object) {
                return object == null ? [] : baseValues(object, keysIn(object));
            }
            /*------------------------------------------------------------------------*/
            /**
             * Clamps `number` within the inclusive `lower` and `upper` bounds.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Number
             * @param {number} number The number to clamp.
             * @param {number} [lower] The lower bound.
             * @param {number} upper The upper bound.
             * @returns {number} Returns the clamped number.
             * @example
             *
             * _.clamp(-10, -5, 5);
             * // => -5
             *
             * _.clamp(10, -5, 5);
             * // => 5
             */
            function clamp(number, lower, upper) {
                if (upper === undefined) {
                    upper = lower;
                    lower = undefined;
                }
                if (upper !== undefined) {
                    upper = toNumber(upper);
                    upper = upper === upper ? upper : 0;
                }
                if (lower !== undefined) {
                    lower = toNumber(lower);
                    lower = lower === lower ? lower : 0;
                }
                return baseClamp(toNumber(number), lower, upper);
            }
            /**
             * Checks if `n` is between `start` and up to, but not including, `end`. If
             * `end` is not specified, it's set to `start` with `start` then set to `0`.
             * If `start` is greater than `end` the params are swapped to support
             * negative ranges.
             *
             * @static
             * @memberOf _
             * @since 3.3.0
             * @category Number
             * @param {number} number The number to check.
             * @param {number} [start=0] The start of the range.
             * @param {number} end The end of the range.
             * @returns {boolean} Returns `true` if `number` is in the range, else `false`.
             * @see _.range, _.rangeRight
             * @example
             *
             * _.inRange(3, 2, 4);
             * // => true
             *
             * _.inRange(4, 8);
             * // => true
             *
             * _.inRange(4, 2);
             * // => false
             *
             * _.inRange(2, 2);
             * // => false
             *
             * _.inRange(1.2, 2);
             * // => true
             *
             * _.inRange(5.2, 4);
             * // => false
             *
             * _.inRange(-3, -2, -6);
             * // => true
             */
            function inRange(number, start, end) {
                start = toFinite(start);
                if (end === undefined) {
                    end = start;
                    start = 0;
                }
                else {
                    end = toFinite(end);
                }
                number = toNumber(number);
                return baseInRange(number, start, end);
            }
            /**
             * Produces a random number between the inclusive `lower` and `upper` bounds.
             * If only one argument is provided a number between `0` and the given number
             * is returned. If `floating` is `true`, or either `lower` or `upper` are
             * floats, a floating-point number is returned instead of an integer.
             *
             * **Note:** JavaScript follows the IEEE-754 standard for resolving
             * floating-point values which can produce unexpected results.
             *
             * @static
             * @memberOf _
             * @since 0.7.0
             * @category Number
             * @param {number} [lower=0] The lower bound.
             * @param {number} [upper=1] The upper bound.
             * @param {boolean} [floating] Specify returning a floating-point number.
             * @returns {number} Returns the random number.
             * @example
             *
             * _.random(0, 5);
             * // => an integer between 0 and 5
             *
             * _.random(5);
             * // => also an integer between 0 and 5
             *
             * _.random(5, true);
             * // => a floating-point number between 0 and 5
             *
             * _.random(1.2, 5.2);
             * // => a floating-point number between 1.2 and 5.2
             */
            function random(lower, upper, floating) {
                if (floating && typeof floating != 'boolean' && isIterateeCall(lower, upper, floating)) {
                    upper = floating = undefined;
                }
                if (floating === undefined) {
                    if (typeof upper == 'boolean') {
                        floating = upper;
                        upper = undefined;
                    }
                    else if (typeof lower == 'boolean') {
                        floating = lower;
                        lower = undefined;
                    }
                }
                if (lower === undefined && upper === undefined) {
                    lower = 0;
                    upper = 1;
                }
                else {
                    lower = toFinite(lower);
                    if (upper === undefined) {
                        upper = lower;
                        lower = 0;
                    }
                    else {
                        upper = toFinite(upper);
                    }
                }
                if (lower > upper) {
                    var temp = lower;
                    lower = upper;
                    upper = temp;
                }
                if (floating || lower % 1 || upper % 1) {
                    var rand = nativeRandom();
                    return nativeMin(lower + (rand * (upper - lower + freeParseFloat('1e-' + ((rand + '').length - 1)))), upper);
                }
                return baseRandom(lower, upper);
            }
            /*------------------------------------------------------------------------*/
            /**
             * Converts `string` to [camel case](https://en.wikipedia.org/wiki/CamelCase).
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to convert.
             * @returns {string} Returns the camel cased string.
             * @example
             *
             * _.camelCase('Foo Bar');
             * // => 'fooBar'
             *
             * _.camelCase('--foo-bar--');
             * // => 'fooBar'
             *
             * _.camelCase('__FOO_BAR__');
             * // => 'fooBar'
             */
            var camelCase = createCompounder(function (result, word, index) {
                word = word.toLowerCase();
                return result + (index ? capitalize(word) : word);
            });
            /**
             * Converts the first character of `string` to upper case and the remaining
             * to lower case.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to capitalize.
             * @returns {string} Returns the capitalized string.
             * @example
             *
             * _.capitalize('FRED');
             * // => 'Fred'
             */
            function capitalize(string) {
                return upperFirst(toString(string).toLowerCase());
            }
            /**
             * Deburrs `string` by converting
             * [Latin-1 Supplement](https://en.wikipedia.org/wiki/Latin-1_Supplement_(Unicode_block)#Character_table)
             * and [Latin Extended-A](https://en.wikipedia.org/wiki/Latin_Extended-A)
             * letters to basic Latin letters and removing
             * [combining diacritical marks](https://en.wikipedia.org/wiki/Combining_Diacritical_Marks).
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to deburr.
             * @returns {string} Returns the deburred string.
             * @example
             *
             * _.deburr('déjà vu');
             * // => 'deja vu'
             */
            function deburr(string) {
                string = toString(string);
                return string && string.replace(reLatin, deburrLetter).replace(reComboMark, '');
            }
            /**
             * Checks if `string` ends with the given target string.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to inspect.
             * @param {string} [target] The string to search for.
             * @param {number} [position=string.length] The position to search up to.
             * @returns {boolean} Returns `true` if `string` ends with `target`,
             *  else `false`.
             * @example
             *
             * _.endsWith('abc', 'c');
             * // => true
             *
             * _.endsWith('abc', 'b');
             * // => false
             *
             * _.endsWith('abc', 'b', 2);
             * // => true
             */
            function endsWith(string, target, position) {
                string = toString(string);
                target = baseToString(target);
                var length = string.length;
                position = position === undefined
                    ? length
                    : baseClamp(toInteger(position), 0, length);
                var end = position;
                position -= target.length;
                return position >= 0 && string.slice(position, end) == target;
            }
            /**
             * Converts the characters "&", "<", ">", '"', and "'" in `string` to their
             * corresponding HTML entities.
             *
             * **Note:** No other characters are escaped. To escape additional
             * characters use a third-party library like [_he_](https://mths.be/he).
             *
             * Though the ">" character is escaped for symmetry, characters like
             * ">" and "/" don't need escaping in HTML and have no special meaning
             * unless they're part of a tag or unquoted attribute value. See
             * [Mathias Bynens's article](https://mathiasbynens.be/notes/ambiguous-ampersands)
             * (under "semi-related fun fact") for more details.
             *
             * When working with HTML you should always
             * [quote attribute values](http://wonko.com/post/html-escaping) to reduce
             * XSS vectors.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category String
             * @param {string} [string=''] The string to escape.
             * @returns {string} Returns the escaped string.
             * @example
             *
             * _.escape('fred, barney, & pebbles');
             * // => 'fred, barney, &amp; pebbles'
             */
            function escape(string) {
                string = toString(string);
                return (string && reHasUnescapedHtml.test(string))
                    ? string.replace(reUnescapedHtml, escapeHtmlChar)
                    : string;
            }
            /**
             * Escapes the `RegExp` special characters "^", "$", "\", ".", "*", "+",
             * "?", "(", ")", "[", "]", "{", "}", and "|" in `string`.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to escape.
             * @returns {string} Returns the escaped string.
             * @example
             *
             * _.escapeRegExp('[lodash](https://lodash.com/)');
             * // => '\[lodash\]\(https://lodash\.com/\)'
             */
            function escapeRegExp(string) {
                string = toString(string);
                return (string && reHasRegExpChar.test(string))
                    ? string.replace(reRegExpChar, '\\$&')
                    : string;
            }
            /**
             * Converts `string` to
             * [kebab case](https://en.wikipedia.org/wiki/Letter_case#Special_case_styles).
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to convert.
             * @returns {string} Returns the kebab cased string.
             * @example
             *
             * _.kebabCase('Foo Bar');
             * // => 'foo-bar'
             *
             * _.kebabCase('fooBar');
             * // => 'foo-bar'
             *
             * _.kebabCase('__FOO_BAR__');
             * // => 'foo-bar'
             */
            var kebabCase = createCompounder(function (result, word, index) {
                return result + (index ? '-' : '') + word.toLowerCase();
            });
            /**
             * Converts `string`, as space separated words, to lower case.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to convert.
             * @returns {string} Returns the lower cased string.
             * @example
             *
             * _.lowerCase('--Foo-Bar--');
             * // => 'foo bar'
             *
             * _.lowerCase('fooBar');
             * // => 'foo bar'
             *
             * _.lowerCase('__FOO_BAR__');
             * // => 'foo bar'
             */
            var lowerCase = createCompounder(function (result, word, index) {
                return result + (index ? ' ' : '') + word.toLowerCase();
            });
            /**
             * Converts the first character of `string` to lower case.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to convert.
             * @returns {string} Returns the converted string.
             * @example
             *
             * _.lowerFirst('Fred');
             * // => 'fred'
             *
             * _.lowerFirst('FRED');
             * // => 'fRED'
             */
            var lowerFirst = createCaseFirst('toLowerCase');
            /**
             * Pads `string` on the left and right sides if it's shorter than `length`.
             * Padding characters are truncated if they can't be evenly divided by `length`.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to pad.
             * @param {number} [length=0] The padding length.
             * @param {string} [chars=' '] The string used as padding.
             * @returns {string} Returns the padded string.
             * @example
             *
             * _.pad('abc', 8);
             * // => '  abc   '
             *
             * _.pad('abc', 8, '_-');
             * // => '_-abc_-_'
             *
             * _.pad('abc', 3);
             * // => 'abc'
             */
            function pad(string, length, chars) {
                string = toString(string);
                length = toInteger(length);
                var strLength = length ? stringSize(string) : 0;
                if (!length || strLength >= length) {
                    return string;
                }
                var mid = (length - strLength) / 2;
                return (createPadding(nativeFloor(mid), chars) +
                    string +
                    createPadding(nativeCeil(mid), chars));
            }
            /**
             * Pads `string` on the right side if it's shorter than `length`. Padding
             * characters are truncated if they exceed `length`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to pad.
             * @param {number} [length=0] The padding length.
             * @param {string} [chars=' '] The string used as padding.
             * @returns {string} Returns the padded string.
             * @example
             *
             * _.padEnd('abc', 6);
             * // => 'abc   '
             *
             * _.padEnd('abc', 6, '_-');
             * // => 'abc_-_'
             *
             * _.padEnd('abc', 3);
             * // => 'abc'
             */
            function padEnd(string, length, chars) {
                string = toString(string);
                length = toInteger(length);
                var strLength = length ? stringSize(string) : 0;
                return (length && strLength < length)
                    ? (string + createPadding(length - strLength, chars))
                    : string;
            }
            /**
             * Pads `string` on the left side if it's shorter than `length`. Padding
             * characters are truncated if they exceed `length`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to pad.
             * @param {number} [length=0] The padding length.
             * @param {string} [chars=' '] The string used as padding.
             * @returns {string} Returns the padded string.
             * @example
             *
             * _.padStart('abc', 6);
             * // => '   abc'
             *
             * _.padStart('abc', 6, '_-');
             * // => '_-_abc'
             *
             * _.padStart('abc', 3);
             * // => 'abc'
             */
            function padStart(string, length, chars) {
                string = toString(string);
                length = toInteger(length);
                var strLength = length ? stringSize(string) : 0;
                return (length && strLength < length)
                    ? (createPadding(length - strLength, chars) + string)
                    : string;
            }
            /**
             * Converts `string` to an integer of the specified radix. If `radix` is
             * `undefined` or `0`, a `radix` of `10` is used unless `value` is a
             * hexadecimal, in which case a `radix` of `16` is used.
             *
             * **Note:** This method aligns with the
             * [ES5 implementation](https://es5.github.io/#x15.1.2.2) of `parseInt`.
             *
             * @static
             * @memberOf _
             * @since 1.1.0
             * @category String
             * @param {string} string The string to convert.
             * @param {number} [radix=10] The radix to interpret `value` by.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {number} Returns the converted integer.
             * @example
             *
             * _.parseInt('08');
             * // => 8
             *
             * _.map(['6', '08', '10'], _.parseInt);
             * // => [6, 8, 10]
             */
            function parseInt(string, radix, guard) {
                if (guard || radix == null) {
                    radix = 0;
                }
                else if (radix) {
                    radix = +radix;
                }
                return nativeParseInt(toString(string).replace(reTrimStart, ''), radix || 0);
            }
            /**
             * Repeats the given string `n` times.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to repeat.
             * @param {number} [n=1] The number of times to repeat the string.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {string} Returns the repeated string.
             * @example
             *
             * _.repeat('*', 3);
             * // => '***'
             *
             * _.repeat('abc', 2);
             * // => 'abcabc'
             *
             * _.repeat('abc', 0);
             * // => ''
             */
            function repeat(string, n, guard) {
                if ((guard ? isIterateeCall(string, n, guard) : n === undefined)) {
                    n = 1;
                }
                else {
                    n = toInteger(n);
                }
                return baseRepeat(toString(string), n);
            }
            /**
             * Replaces matches for `pattern` in `string` with `replacement`.
             *
             * **Note:** This method is based on
             * [`String#replace`](https://mdn.io/String/replace).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to modify.
             * @param {RegExp|string} pattern The pattern to replace.
             * @param {Function|string} replacement The match replacement.
             * @returns {string} Returns the modified string.
             * @example
             *
             * _.replace('Hi Fred', 'Fred', 'Barney');
             * // => 'Hi Barney'
             */
            function replace() {
                var args = arguments, string = toString(args[0]);
                return args.length < 3 ? string : string.replace(args[1], args[2]);
            }
            /**
             * Converts `string` to
             * [snake case](https://en.wikipedia.org/wiki/Snake_case).
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to convert.
             * @returns {string} Returns the snake cased string.
             * @example
             *
             * _.snakeCase('Foo Bar');
             * // => 'foo_bar'
             *
             * _.snakeCase('fooBar');
             * // => 'foo_bar'
             *
             * _.snakeCase('--FOO-BAR--');
             * // => 'foo_bar'
             */
            var snakeCase = createCompounder(function (result, word, index) {
                return result + (index ? '_' : '') + word.toLowerCase();
            });
            /**
             * Splits `string` by `separator`.
             *
             * **Note:** This method is based on
             * [`String#split`](https://mdn.io/String/split).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to split.
             * @param {RegExp|string} separator The separator pattern to split by.
             * @param {number} [limit] The length to truncate results to.
             * @returns {Array} Returns the string segments.
             * @example
             *
             * _.split('a-b-c', '-', 2);
             * // => ['a', 'b']
             */
            function split(string, separator, limit) {
                if (limit && typeof limit != 'number' && isIterateeCall(string, separator, limit)) {
                    separator = limit = undefined;
                }
                limit = limit === undefined ? MAX_ARRAY_LENGTH : limit >>> 0;
                if (!limit) {
                    return [];
                }
                string = toString(string);
                if (string && (typeof separator == 'string' ||
                    (separator != null && !isRegExp(separator)))) {
                    separator = baseToString(separator);
                    if (!separator && hasUnicode(string)) {
                        return castSlice(stringToArray(string), 0, limit);
                    }
                }
                return string.split(separator, limit);
            }
            /**
             * Converts `string` to
             * [start case](https://en.wikipedia.org/wiki/Letter_case#Stylistic_or_specialised_usage).
             *
             * @static
             * @memberOf _
             * @since 3.1.0
             * @category String
             * @param {string} [string=''] The string to convert.
             * @returns {string} Returns the start cased string.
             * @example
             *
             * _.startCase('--foo-bar--');
             * // => 'Foo Bar'
             *
             * _.startCase('fooBar');
             * // => 'Foo Bar'
             *
             * _.startCase('__FOO_BAR__');
             * // => 'FOO BAR'
             */
            var startCase = createCompounder(function (result, word, index) {
                return result + (index ? ' ' : '') + upperFirst(word);
            });
            /**
             * Checks if `string` starts with the given target string.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to inspect.
             * @param {string} [target] The string to search for.
             * @param {number} [position=0] The position to search from.
             * @returns {boolean} Returns `true` if `string` starts with `target`,
             *  else `false`.
             * @example
             *
             * _.startsWith('abc', 'a');
             * // => true
             *
             * _.startsWith('abc', 'b');
             * // => false
             *
             * _.startsWith('abc', 'b', 1);
             * // => true
             */
            function startsWith(string, target, position) {
                string = toString(string);
                position = position == null
                    ? 0
                    : baseClamp(toInteger(position), 0, string.length);
                target = baseToString(target);
                return string.slice(position, position + target.length) == target;
            }
            /**
             * Creates a compiled template function that can interpolate data properties
             * in "interpolate" delimiters, HTML-escape interpolated data properties in
             * "escape" delimiters, and execute JavaScript in "evaluate" delimiters. Data
             * properties may be accessed as free variables in the template. If a setting
             * object is given, it takes precedence over `_.templateSettings` values.
             *
             * **Note:** In the development build `_.template` utilizes
             * [sourceURLs](http://www.html5rocks.com/en/tutorials/developertools/sourcemaps/#toc-sourceurl)
             * for easier debugging.
             *
             * For more information on precompiling templates see
             * [lodash's custom builds documentation](https://lodash.com/custom-builds).
             *
             * For more information on Chrome extension sandboxes see
             * [Chrome's extensions documentation](https://developer.chrome.com/extensions/sandboxingEval).
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category String
             * @param {string} [string=''] The template string.
             * @param {Object} [options={}] The options object.
             * @param {RegExp} [options.escape=_.templateSettings.escape]
             *  The HTML "escape" delimiter.
             * @param {RegExp} [options.evaluate=_.templateSettings.evaluate]
             *  The "evaluate" delimiter.
             * @param {Object} [options.imports=_.templateSettings.imports]
             *  An object to import into the template as free variables.
             * @param {RegExp} [options.interpolate=_.templateSettings.interpolate]
             *  The "interpolate" delimiter.
             * @param {string} [options.sourceURL='lodash.templateSources[n]']
             *  The sourceURL of the compiled template.
             * @param {string} [options.variable='obj']
             *  The data object variable name.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {Function} Returns the compiled template function.
             * @example
             *
             * // Use the "interpolate" delimiter to create a compiled template.
             * var compiled = _.template('hello <%= user %>!');
             * compiled({ 'user': 'fred' });
             * // => 'hello fred!'
             *
             * // Use the HTML "escape" delimiter to escape data property values.
             * var compiled = _.template('<b><%- value %></b>');
             * compiled({ 'value': '<script>' });
             * // => '<b>&lt;script&gt;</b>'
             *
             * // Use the "evaluate" delimiter to execute JavaScript and generate HTML.
             * var compiled = _.template('<% _.forEach(users, function(user) { %><li><%- user %></li><% }); %>');
             * compiled({ 'users': ['fred', 'barney'] });
             * // => '<li>fred</li><li>barney</li>'
             *
             * // Use the internal `print` function in "evaluate" delimiters.
             * var compiled = _.template('<% print("hello " + user); %>!');
             * compiled({ 'user': 'barney' });
             * // => 'hello barney!'
             *
             * // Use the ES template literal delimiter as an "interpolate" delimiter.
             * // Disable support by replacing the "interpolate" delimiter.
             * var compiled = _.template('hello ${ user }!');
             * compiled({ 'user': 'pebbles' });
             * // => 'hello pebbles!'
             *
             * // Use backslashes to treat delimiters as plain text.
             * var compiled = _.template('<%= "\\<%- value %\\>" %>');
             * compiled({ 'value': 'ignored' });
             * // => '<%- value %>'
             *
             * // Use the `imports` option to import `jQuery` as `jq`.
             * var text = '<% jq.each(users, function(user) { %><li><%- user %></li><% }); %>';
             * var compiled = _.template(text, { 'imports': { 'jq': jQuery } });
             * compiled({ 'users': ['fred', 'barney'] });
             * // => '<li>fred</li><li>barney</li>'
             *
             * // Use the `sourceURL` option to specify a custom sourceURL for the template.
             * var compiled = _.template('hello <%= user %>!', { 'sourceURL': '/basic/greeting.jst' });
             * compiled(data);
             * // => Find the source of "greeting.jst" under the Sources tab or Resources panel of the web inspector.
             *
             * // Use the `variable` option to ensure a with-statement isn't used in the compiled template.
             * var compiled = _.template('hi <%= data.user %>!', { 'variable': 'data' });
             * compiled.source;
             * // => function(data) {
             * //   var __t, __p = '';
             * //   __p += 'hi ' + ((__t = ( data.user )) == null ? '' : __t) + '!';
             * //   return __p;
             * // }
             *
             * // Use custom template delimiters.
             * _.templateSettings.interpolate = /{{([\s\S]+?)}}/g;
             * var compiled = _.template('hello {{ user }}!');
             * compiled({ 'user': 'mustache' });
             * // => 'hello mustache!'
             *
             * // Use the `source` property to inline compiled templates for meaningful
             * // line numbers in error messages and stack traces.
             * fs.writeFileSync(path.join(process.cwd(), 'jst.js'), '\
             *   var JST = {\
             *     "main": ' + _.template(mainText).source + '\
             *   };\
             * ');
             */
            function template(string, options, guard) {
                // Based on John Resig's `tmpl` implementation
                // (http://ejohn.org/blog/javascript-micro-templating/)
                // and Laura Doktorova's doT.js (https://github.com/olado/doT).
                var settings = lodash.templateSettings;
                if (guard && isIterateeCall(string, options, guard)) {
                    options = undefined;
                }
                string = toString(string);
                options = assignInWith({}, options, settings, customDefaultsAssignIn);
                var imports = assignInWith({}, options.imports, settings.imports, customDefaultsAssignIn), importsKeys = keys(imports), importsValues = baseValues(imports, importsKeys);
                var isEscaping, isEvaluating, index = 0, interpolate = options.interpolate || reNoMatch, source = "__p += '";
                // Compile the regexp to match each delimiter.
                var reDelimiters = RegExp((options.escape || reNoMatch).source + '|' +
                    interpolate.source + '|' +
                    (interpolate === reInterpolate ? reEsTemplate : reNoMatch).source + '|' +
                    (options.evaluate || reNoMatch).source + '|$', 'g');
                // Use a sourceURL for easier debugging.
                // The sourceURL gets injected into the source that's eval-ed, so be careful
                // with lookup (in case of e.g. prototype pollution), and strip newlines if any.
                // A newline wouldn't be a valid sourceURL anyway, and it'd enable code injection.
                var sourceURL = '//# sourceURL=' +
                    (hasOwnProperty.call(options, 'sourceURL')
                        ? (options.sourceURL + '').replace(/[\r\n]/g, ' ')
                        : ('lodash.templateSources[' + (++templateCounter) + ']')) + '\n';
                string.replace(reDelimiters, function (match, escapeValue, interpolateValue, esTemplateValue, evaluateValue, offset) {
                    interpolateValue || (interpolateValue = esTemplateValue);
                    // Escape characters that can't be included in string literals.
                    source += string.slice(index, offset).replace(reUnescapedString, escapeStringChar);
                    // Replace delimiters with snippets.
                    if (escapeValue) {
                        isEscaping = true;
                        source += "' +\n__e(" + escapeValue + ") +\n'";
                    }
                    if (evaluateValue) {
                        isEvaluating = true;
                        source += "';\n" + evaluateValue + ";\n__p += '";
                    }
                    if (interpolateValue) {
                        source += "' +\n((__t = (" + interpolateValue + ")) == null ? '' : __t) +\n'";
                    }
                    index = offset + match.length;
                    // The JS engine embedded in Adobe products needs `match` returned in
                    // order to produce the correct `offset` value.
                    return match;
                });
                source += "';\n";
                // If `variable` is not specified wrap a with-statement around the generated
                // code to add the data object to the top of the scope chain.
                // Like with sourceURL, we take care to not check the option's prototype,
                // as this configuration is a code injection vector.
                var variable = hasOwnProperty.call(options, 'variable') && options.variable;
                if (!variable) {
                    source = 'with (obj) {\n' + source + '\n}\n';
                }
                // Cleanup code by stripping empty strings.
                source = (isEvaluating ? source.replace(reEmptyStringLeading, '') : source)
                    .replace(reEmptyStringMiddle, '$1')
                    .replace(reEmptyStringTrailing, '$1;');
                // Frame code as the function body.
                source = 'function(' + (variable || 'obj') + ') {\n' +
                    (variable
                        ? ''
                        : 'obj || (obj = {});\n') +
                    "var __t, __p = ''" +
                    (isEscaping
                        ? ', __e = _.escape'
                        : '') +
                    (isEvaluating
                        ? ', __j = Array.prototype.join;\n' +
                            "function print() { __p += __j.call(arguments, '') }\n"
                        : ';\n') +
                    source +
                    'return __p\n}';
                var result = attempt(function () {
                    return Function(importsKeys, sourceURL + 'return ' + source)
                        .apply(undefined, importsValues);
                });
                // Provide the compiled function's source by its `toString` method or
                // the `source` property as a convenience for inlining compiled templates.
                result.source = source;
                if (isError(result)) {
                    throw result;
                }
                return result;
            }
            /**
             * Converts `string`, as a whole, to lower case just like
             * [String#toLowerCase](https://mdn.io/toLowerCase).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to convert.
             * @returns {string} Returns the lower cased string.
             * @example
             *
             * _.toLower('--Foo-Bar--');
             * // => '--foo-bar--'
             *
             * _.toLower('fooBar');
             * // => 'foobar'
             *
             * _.toLower('__FOO_BAR__');
             * // => '__foo_bar__'
             */
            function toLower(value) {
                return toString(value).toLowerCase();
            }
            /**
             * Converts `string`, as a whole, to upper case just like
             * [String#toUpperCase](https://mdn.io/toUpperCase).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to convert.
             * @returns {string} Returns the upper cased string.
             * @example
             *
             * _.toUpper('--foo-bar--');
             * // => '--FOO-BAR--'
             *
             * _.toUpper('fooBar');
             * // => 'FOOBAR'
             *
             * _.toUpper('__foo_bar__');
             * // => '__FOO_BAR__'
             */
            function toUpper(value) {
                return toString(value).toUpperCase();
            }
            /**
             * Removes leading and trailing whitespace or specified characters from `string`.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to trim.
             * @param {string} [chars=whitespace] The characters to trim.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {string} Returns the trimmed string.
             * @example
             *
             * _.trim('  abc  ');
             * // => 'abc'
             *
             * _.trim('-_-abc-_-', '_-');
             * // => 'abc'
             *
             * _.map(['  foo  ', '  bar  '], _.trim);
             * // => ['foo', 'bar']
             */
            function trim(string, chars, guard) {
                string = toString(string);
                if (string && (guard || chars === undefined)) {
                    return string.replace(reTrim, '');
                }
                if (!string || !(chars = baseToString(chars))) {
                    return string;
                }
                var strSymbols = stringToArray(string), chrSymbols = stringToArray(chars), start = charsStartIndex(strSymbols, chrSymbols), end = charsEndIndex(strSymbols, chrSymbols) + 1;
                return castSlice(strSymbols, start, end).join('');
            }
            /**
             * Removes trailing whitespace or specified characters from `string`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to trim.
             * @param {string} [chars=whitespace] The characters to trim.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {string} Returns the trimmed string.
             * @example
             *
             * _.trimEnd('  abc  ');
             * // => '  abc'
             *
             * _.trimEnd('-_-abc-_-', '_-');
             * // => '-_-abc'
             */
            function trimEnd(string, chars, guard) {
                string = toString(string);
                if (string && (guard || chars === undefined)) {
                    return string.replace(reTrimEnd, '');
                }
                if (!string || !(chars = baseToString(chars))) {
                    return string;
                }
                var strSymbols = stringToArray(string), end = charsEndIndex(strSymbols, stringToArray(chars)) + 1;
                return castSlice(strSymbols, 0, end).join('');
            }
            /**
             * Removes leading whitespace or specified characters from `string`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to trim.
             * @param {string} [chars=whitespace] The characters to trim.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {string} Returns the trimmed string.
             * @example
             *
             * _.trimStart('  abc  ');
             * // => 'abc  '
             *
             * _.trimStart('-_-abc-_-', '_-');
             * // => 'abc-_-'
             */
            function trimStart(string, chars, guard) {
                string = toString(string);
                if (string && (guard || chars === undefined)) {
                    return string.replace(reTrimStart, '');
                }
                if (!string || !(chars = baseToString(chars))) {
                    return string;
                }
                var strSymbols = stringToArray(string), start = charsStartIndex(strSymbols, stringToArray(chars));
                return castSlice(strSymbols, start).join('');
            }
            /**
             * Truncates `string` if it's longer than the given maximum string length.
             * The last characters of the truncated string are replaced with the omission
             * string which defaults to "...".
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to truncate.
             * @param {Object} [options={}] The options object.
             * @param {number} [options.length=30] The maximum string length.
             * @param {string} [options.omission='...'] The string to indicate text is omitted.
             * @param {RegExp|string} [options.separator] The separator pattern to truncate to.
             * @returns {string} Returns the truncated string.
             * @example
             *
             * _.truncate('hi-diddly-ho there, neighborino');
             * // => 'hi-diddly-ho there, neighbo...'
             *
             * _.truncate('hi-diddly-ho there, neighborino', {
             *   'length': 24,
             *   'separator': ' '
             * });
             * // => 'hi-diddly-ho there,...'
             *
             * _.truncate('hi-diddly-ho there, neighborino', {
             *   'length': 24,
             *   'separator': /,? +/
             * });
             * // => 'hi-diddly-ho there...'
             *
             * _.truncate('hi-diddly-ho there, neighborino', {
             *   'omission': ' [...]'
             * });
             * // => 'hi-diddly-ho there, neig [...]'
             */
            function truncate(string, options) {
                var length = DEFAULT_TRUNC_LENGTH, omission = DEFAULT_TRUNC_OMISSION;
                if (isObject(options)) {
                    var separator = 'separator' in options ? options.separator : separator;
                    length = 'length' in options ? toInteger(options.length) : length;
                    omission = 'omission' in options ? baseToString(options.omission) : omission;
                }
                string = toString(string);
                var strLength = string.length;
                if (hasUnicode(string)) {
                    var strSymbols = stringToArray(string);
                    strLength = strSymbols.length;
                }
                if (length >= strLength) {
                    return string;
                }
                var end = length - stringSize(omission);
                if (end < 1) {
                    return omission;
                }
                var result = strSymbols
                    ? castSlice(strSymbols, 0, end).join('')
                    : string.slice(0, end);
                if (separator === undefined) {
                    return result + omission;
                }
                if (strSymbols) {
                    end += (result.length - end);
                }
                if (isRegExp(separator)) {
                    if (string.slice(end).search(separator)) {
                        var match, substring = result;
                        if (!separator.global) {
                            separator = RegExp(separator.source, toString(reFlags.exec(separator)) + 'g');
                        }
                        separator.lastIndex = 0;
                        while ((match = separator.exec(substring))) {
                            var newEnd = match.index;
                        }
                        result = result.slice(0, newEnd === undefined ? end : newEnd);
                    }
                }
                else if (string.indexOf(baseToString(separator), end) != end) {
                    var index = result.lastIndexOf(separator);
                    if (index > -1) {
                        result = result.slice(0, index);
                    }
                }
                return result + omission;
            }
            /**
             * The inverse of `_.escape`; this method converts the HTML entities
             * `&amp;`, `&lt;`, `&gt;`, `&quot;`, and `&#39;` in `string` to
             * their corresponding characters.
             *
             * **Note:** No other HTML entities are unescaped. To unescape additional
             * HTML entities use a third-party library like [_he_](https://mths.be/he).
             *
             * @static
             * @memberOf _
             * @since 0.6.0
             * @category String
             * @param {string} [string=''] The string to unescape.
             * @returns {string} Returns the unescaped string.
             * @example
             *
             * _.unescape('fred, barney, &amp; pebbles');
             * // => 'fred, barney, & pebbles'
             */
            function unescape(string) {
                string = toString(string);
                return (string && reHasEscapedHtml.test(string))
                    ? string.replace(reEscapedHtml, unescapeHtmlChar)
                    : string;
            }
            /**
             * Converts `string`, as space separated words, to upper case.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to convert.
             * @returns {string} Returns the upper cased string.
             * @example
             *
             * _.upperCase('--foo-bar');
             * // => 'FOO BAR'
             *
             * _.upperCase('fooBar');
             * // => 'FOO BAR'
             *
             * _.upperCase('__foo_bar__');
             * // => 'FOO BAR'
             */
            var upperCase = createCompounder(function (result, word, index) {
                return result + (index ? ' ' : '') + word.toUpperCase();
            });
            /**
             * Converts the first character of `string` to upper case.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category String
             * @param {string} [string=''] The string to convert.
             * @returns {string} Returns the converted string.
             * @example
             *
             * _.upperFirst('fred');
             * // => 'Fred'
             *
             * _.upperFirst('FRED');
             * // => 'FRED'
             */
            var upperFirst = createCaseFirst('toUpperCase');
            /**
             * Splits `string` into an array of its words.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category String
             * @param {string} [string=''] The string to inspect.
             * @param {RegExp|string} [pattern] The pattern to match words.
             * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
             * @returns {Array} Returns the words of `string`.
             * @example
             *
             * _.words('fred, barney, & pebbles');
             * // => ['fred', 'barney', 'pebbles']
             *
             * _.words('fred, barney, & pebbles', /[^, ]+/g);
             * // => ['fred', 'barney', '&', 'pebbles']
             */
            function words(string, pattern, guard) {
                string = toString(string);
                pattern = guard ? undefined : pattern;
                if (pattern === undefined) {
                    return hasUnicodeWord(string) ? unicodeWords(string) : asciiWords(string);
                }
                return string.match(pattern) || [];
            }
            /*------------------------------------------------------------------------*/
            /**
             * Attempts to invoke `func`, returning either the result or the caught error
             * object. Any additional arguments are provided to `func` when it's invoked.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Util
             * @param {Function} func The function to attempt.
             * @param {...*} [args] The arguments to invoke `func` with.
             * @returns {*} Returns the `func` result or error object.
             * @example
             *
             * // Avoid throwing errors for invalid selectors.
             * var elements = _.attempt(function(selector) {
             *   return document.querySelectorAll(selector);
             * }, '>_>');
             *
             * if (_.isError(elements)) {
             *   elements = [];
             * }
             */
            var attempt = baseRest(function (func, args) {
                try {
                    return apply(func, undefined, args);
                }
                catch (e) {
                    return isError(e) ? e : new Error(e);
                }
            });
            /**
             * Binds methods of an object to the object itself, overwriting the existing
             * method.
             *
             * **Note:** This method doesn't set the "length" property of bound functions.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Util
             * @param {Object} object The object to bind and assign the bound methods to.
             * @param {...(string|string[])} methodNames The object method names to bind.
             * @returns {Object} Returns `object`.
             * @example
             *
             * var view = {
             *   'label': 'docs',
             *   'click': function() {
             *     console.log('clicked ' + this.label);
             *   }
             * };
             *
             * _.bindAll(view, ['click']);
             * jQuery(element).on('click', view.click);
             * // => Logs 'clicked docs' when clicked.
             */
            var bindAll = flatRest(function (object, methodNames) {
                arrayEach(methodNames, function (key) {
                    key = toKey(key);
                    baseAssignValue(object, key, bind(object[key], object));
                });
                return object;
            });
            /**
             * Creates a function that iterates over `pairs` and invokes the corresponding
             * function of the first predicate to return truthy. The predicate-function
             * pairs are invoked with the `this` binding and arguments of the created
             * function.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Util
             * @param {Array} pairs The predicate-function pairs.
             * @returns {Function} Returns the new composite function.
             * @example
             *
             * var func = _.cond([
             *   [_.matches({ 'a': 1 }),           _.constant('matches A')],
             *   [_.conforms({ 'b': _.isNumber }), _.constant('matches B')],
             *   [_.stubTrue,                      _.constant('no match')]
             * ]);
             *
             * func({ 'a': 1, 'b': 2 });
             * // => 'matches A'
             *
             * func({ 'a': 0, 'b': 1 });
             * // => 'matches B'
             *
             * func({ 'a': '1', 'b': '2' });
             * // => 'no match'
             */
            function cond(pairs) {
                var length = pairs == null ? 0 : pairs.length, toIteratee = getIteratee();
                pairs = !length ? [] : arrayMap(pairs, function (pair) {
                    if (typeof pair[1] != 'function') {
                        throw new TypeError(FUNC_ERROR_TEXT);
                    }
                    return [toIteratee(pair[0]), pair[1]];
                });
                return baseRest(function (args) {
                    var index = -1;
                    while (++index < length) {
                        var pair = pairs[index];
                        if (apply(pair[0], this, args)) {
                            return apply(pair[1], this, args);
                        }
                    }
                });
            }
            /**
             * Creates a function that invokes the predicate properties of `source` with
             * the corresponding property values of a given object, returning `true` if
             * all predicates return truthy, else `false`.
             *
             * **Note:** The created function is equivalent to `_.conformsTo` with
             * `source` partially applied.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Util
             * @param {Object} source The object of property predicates to conform to.
             * @returns {Function} Returns the new spec function.
             * @example
             *
             * var objects = [
             *   { 'a': 2, 'b': 1 },
             *   { 'a': 1, 'b': 2 }
             * ];
             *
             * _.filter(objects, _.conforms({ 'b': function(n) { return n > 1; } }));
             * // => [{ 'a': 1, 'b': 2 }]
             */
            function conforms(source) {
                return baseConforms(baseClone(source, CLONE_DEEP_FLAG));
            }
            /**
             * Creates a function that returns `value`.
             *
             * @static
             * @memberOf _
             * @since 2.4.0
             * @category Util
             * @param {*} value The value to return from the new function.
             * @returns {Function} Returns the new constant function.
             * @example
             *
             * var objects = _.times(2, _.constant({ 'a': 1 }));
             *
             * console.log(objects);
             * // => [{ 'a': 1 }, { 'a': 1 }]
             *
             * console.log(objects[0] === objects[1]);
             * // => true
             */
            function constant(value) {
                return function () {
                    return value;
                };
            }
            /**
             * Checks `value` to determine whether a default value should be returned in
             * its place. The `defaultValue` is returned if `value` is `NaN`, `null`,
             * or `undefined`.
             *
             * @static
             * @memberOf _
             * @since 4.14.0
             * @category Util
             * @param {*} value The value to check.
             * @param {*} defaultValue The default value.
             * @returns {*} Returns the resolved value.
             * @example
             *
             * _.defaultTo(1, 10);
             * // => 1
             *
             * _.defaultTo(undefined, 10);
             * // => 10
             */
            function defaultTo(value, defaultValue) {
                return (value == null || value !== value) ? defaultValue : value;
            }
            /**
             * Creates a function that returns the result of invoking the given functions
             * with the `this` binding of the created function, where each successive
             * invocation is supplied the return value of the previous.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Util
             * @param {...(Function|Function[])} [funcs] The functions to invoke.
             * @returns {Function} Returns the new composite function.
             * @see _.flowRight
             * @example
             *
             * function square(n) {
             *   return n * n;
             * }
             *
             * var addSquare = _.flow([_.add, square]);
             * addSquare(1, 2);
             * // => 9
             */
            var flow = createFlow();
            /**
             * This method is like `_.flow` except that it creates a function that
             * invokes the given functions from right to left.
             *
             * @static
             * @since 3.0.0
             * @memberOf _
             * @category Util
             * @param {...(Function|Function[])} [funcs] The functions to invoke.
             * @returns {Function} Returns the new composite function.
             * @see _.flow
             * @example
             *
             * function square(n) {
             *   return n * n;
             * }
             *
             * var addSquare = _.flowRight([square, _.add]);
             * addSquare(1, 2);
             * // => 9
             */
            var flowRight = createFlow(true);
            /**
             * This method returns the first argument it receives.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Util
             * @param {*} value Any value.
             * @returns {*} Returns `value`.
             * @example
             *
             * var object = { 'a': 1 };
             *
             * console.log(_.identity(object) === object);
             * // => true
             */
            function identity(value) {
                return value;
            }
            /**
             * Creates a function that invokes `func` with the arguments of the created
             * function. If `func` is a property name, the created function returns the
             * property value for a given element. If `func` is an array or object, the
             * created function returns `true` for elements that contain the equivalent
             * source properties, otherwise it returns `false`.
             *
             * @static
             * @since 4.0.0
             * @memberOf _
             * @category Util
             * @param {*} [func=_.identity] The value to convert to a callback.
             * @returns {Function} Returns the callback.
             * @example
             *
             * var users = [
             *   { 'user': 'barney', 'age': 36, 'active': true },
             *   { 'user': 'fred',   'age': 40, 'active': false }
             * ];
             *
             * // The `_.matches` iteratee shorthand.
             * _.filter(users, _.iteratee({ 'user': 'barney', 'active': true }));
             * // => [{ 'user': 'barney', 'age': 36, 'active': true }]
             *
             * // The `_.matchesProperty` iteratee shorthand.
             * _.filter(users, _.iteratee(['user', 'fred']));
             * // => [{ 'user': 'fred', 'age': 40 }]
             *
             * // The `_.property` iteratee shorthand.
             * _.map(users, _.iteratee('user'));
             * // => ['barney', 'fred']
             *
             * // Create custom iteratee shorthands.
             * _.iteratee = _.wrap(_.iteratee, function(iteratee, func) {
             *   return !_.isRegExp(func) ? iteratee(func) : function(string) {
             *     return func.test(string);
             *   };
             * });
             *
             * _.filter(['abc', 'def'], /ef/);
             * // => ['def']
             */
            function iteratee(func) {
                return baseIteratee(typeof func == 'function' ? func : baseClone(func, CLONE_DEEP_FLAG));
            }
            /**
             * Creates a function that performs a partial deep comparison between a given
             * object and `source`, returning `true` if the given object has equivalent
             * property values, else `false`.
             *
             * **Note:** The created function is equivalent to `_.isMatch` with `source`
             * partially applied.
             *
             * Partial comparisons will match empty array and empty object `source`
             * values against any array or object value, respectively. See `_.isEqual`
             * for a list of supported value comparisons.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Util
             * @param {Object} source The object of property values to match.
             * @returns {Function} Returns the new spec function.
             * @example
             *
             * var objects = [
             *   { 'a': 1, 'b': 2, 'c': 3 },
             *   { 'a': 4, 'b': 5, 'c': 6 }
             * ];
             *
             * _.filter(objects, _.matches({ 'a': 4, 'c': 6 }));
             * // => [{ 'a': 4, 'b': 5, 'c': 6 }]
             */
            function matches(source) {
                return baseMatches(baseClone(source, CLONE_DEEP_FLAG));
            }
            /**
             * Creates a function that performs a partial deep comparison between the
             * value at `path` of a given object to `srcValue`, returning `true` if the
             * object value is equivalent, else `false`.
             *
             * **Note:** Partial comparisons will match empty array and empty object
             * `srcValue` values against any array or object value, respectively. See
             * `_.isEqual` for a list of supported value comparisons.
             *
             * @static
             * @memberOf _
             * @since 3.2.0
             * @category Util
             * @param {Array|string} path The path of the property to get.
             * @param {*} srcValue The value to match.
             * @returns {Function} Returns the new spec function.
             * @example
             *
             * var objects = [
             *   { 'a': 1, 'b': 2, 'c': 3 },
             *   { 'a': 4, 'b': 5, 'c': 6 }
             * ];
             *
             * _.find(objects, _.matchesProperty('a', 4));
             * // => { 'a': 4, 'b': 5, 'c': 6 }
             */
            function matchesProperty(path, srcValue) {
                return baseMatchesProperty(path, baseClone(srcValue, CLONE_DEEP_FLAG));
            }
            /**
             * Creates a function that invokes the method at `path` of a given object.
             * Any additional arguments are provided to the invoked method.
             *
             * @static
             * @memberOf _
             * @since 3.7.0
             * @category Util
             * @param {Array|string} path The path of the method to invoke.
             * @param {...*} [args] The arguments to invoke the method with.
             * @returns {Function} Returns the new invoker function.
             * @example
             *
             * var objects = [
             *   { 'a': { 'b': _.constant(2) } },
             *   { 'a': { 'b': _.constant(1) } }
             * ];
             *
             * _.map(objects, _.method('a.b'));
             * // => [2, 1]
             *
             * _.map(objects, _.method(['a', 'b']));
             * // => [2, 1]
             */
            var method = baseRest(function (path, args) {
                return function (object) {
                    return baseInvoke(object, path, args);
                };
            });
            /**
             * The opposite of `_.method`; this method creates a function that invokes
             * the method at a given path of `object`. Any additional arguments are
             * provided to the invoked method.
             *
             * @static
             * @memberOf _
             * @since 3.7.0
             * @category Util
             * @param {Object} object The object to query.
             * @param {...*} [args] The arguments to invoke the method with.
             * @returns {Function} Returns the new invoker function.
             * @example
             *
             * var array = _.times(3, _.constant),
             *     object = { 'a': array, 'b': array, 'c': array };
             *
             * _.map(['a[2]', 'c[0]'], _.methodOf(object));
             * // => [2, 0]
             *
             * _.map([['a', '2'], ['c', '0']], _.methodOf(object));
             * // => [2, 0]
             */
            var methodOf = baseRest(function (object, args) {
                return function (path) {
                    return baseInvoke(object, path, args);
                };
            });
            /**
             * Adds all own enumerable string keyed function properties of a source
             * object to the destination object. If `object` is a function, then methods
             * are added to its prototype as well.
             *
             * **Note:** Use `_.runInContext` to create a pristine `lodash` function to
             * avoid conflicts caused by modifying the original.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Util
             * @param {Function|Object} [object=lodash] The destination object.
             * @param {Object} source The object of functions to add.
             * @param {Object} [options={}] The options object.
             * @param {boolean} [options.chain=true] Specify whether mixins are chainable.
             * @returns {Function|Object} Returns `object`.
             * @example
             *
             * function vowels(string) {
             *   return _.filter(string, function(v) {
             *     return /[aeiou]/i.test(v);
             *   });
             * }
             *
             * _.mixin({ 'vowels': vowels });
             * _.vowels('fred');
             * // => ['e']
             *
             * _('fred').vowels().value();
             * // => ['e']
             *
             * _.mixin({ 'vowels': vowels }, { 'chain': false });
             * _('fred').vowels();
             * // => ['e']
             */
            function mixin(object, source, options) {
                var props = keys(source), methodNames = baseFunctions(source, props);
                if (options == null &&
                    !(isObject(source) && (methodNames.length || !props.length))) {
                    options = source;
                    source = object;
                    object = this;
                    methodNames = baseFunctions(source, keys(source));
                }
                var chain = !(isObject(options) && 'chain' in options) || !!options.chain, isFunc = isFunction(object);
                arrayEach(methodNames, function (methodName) {
                    var func = source[methodName];
                    object[methodName] = func;
                    if (isFunc) {
                        object.prototype[methodName] = function () {
                            var chainAll = this.__chain__;
                            if (chain || chainAll) {
                                var result = object(this.__wrapped__), actions = result.__actions__ = copyArray(this.__actions__);
                                actions.push({ 'func': func, 'args': arguments, 'thisArg': object });
                                result.__chain__ = chainAll;
                                return result;
                            }
                            return func.apply(object, arrayPush([this.value()], arguments));
                        };
                    }
                });
                return object;
            }
            /**
             * Reverts the `_` variable to its previous value and returns a reference to
             * the `lodash` function.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Util
             * @returns {Function} Returns the `lodash` function.
             * @example
             *
             * var lodash = _.noConflict();
             */
            function noConflict() {
                if (root._ === this) {
                    root._ = oldDash;
                }
                return this;
            }
            /**
             * This method returns `undefined`.
             *
             * @static
             * @memberOf _
             * @since 2.3.0
             * @category Util
             * @example
             *
             * _.times(2, _.noop);
             * // => [undefined, undefined]
             */
            function noop() {
                // No operation performed.
            }
            /**
             * Creates a function that gets the argument at index `n`. If `n` is negative,
             * the nth argument from the end is returned.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Util
             * @param {number} [n=0] The index of the argument to return.
             * @returns {Function} Returns the new pass-thru function.
             * @example
             *
             * var func = _.nthArg(1);
             * func('a', 'b', 'c', 'd');
             * // => 'b'
             *
             * var func = _.nthArg(-2);
             * func('a', 'b', 'c', 'd');
             * // => 'c'
             */
            function nthArg(n) {
                n = toInteger(n);
                return baseRest(function (args) {
                    return baseNth(args, n);
                });
            }
            /**
             * Creates a function that invokes `iteratees` with the arguments it receives
             * and returns their results.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Util
             * @param {...(Function|Function[])} [iteratees=[_.identity]]
             *  The iteratees to invoke.
             * @returns {Function} Returns the new function.
             * @example
             *
             * var func = _.over([Math.max, Math.min]);
             *
             * func(1, 2, 3, 4);
             * // => [4, 1]
             */
            var over = createOver(arrayMap);
            /**
             * Creates a function that checks if **all** of the `predicates` return
             * truthy when invoked with the arguments it receives.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Util
             * @param {...(Function|Function[])} [predicates=[_.identity]]
             *  The predicates to check.
             * @returns {Function} Returns the new function.
             * @example
             *
             * var func = _.overEvery([Boolean, isFinite]);
             *
             * func('1');
             * // => true
             *
             * func(null);
             * // => false
             *
             * func(NaN);
             * // => false
             */
            var overEvery = createOver(arrayEvery);
            /**
             * Creates a function that checks if **any** of the `predicates` return
             * truthy when invoked with the arguments it receives.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Util
             * @param {...(Function|Function[])} [predicates=[_.identity]]
             *  The predicates to check.
             * @returns {Function} Returns the new function.
             * @example
             *
             * var func = _.overSome([Boolean, isFinite]);
             *
             * func('1');
             * // => true
             *
             * func(null);
             * // => true
             *
             * func(NaN);
             * // => false
             */
            var overSome = createOver(arraySome);
            /**
             * Creates a function that returns the value at `path` of a given object.
             *
             * @static
             * @memberOf _
             * @since 2.4.0
             * @category Util
             * @param {Array|string} path The path of the property to get.
             * @returns {Function} Returns the new accessor function.
             * @example
             *
             * var objects = [
             *   { 'a': { 'b': 2 } },
             *   { 'a': { 'b': 1 } }
             * ];
             *
             * _.map(objects, _.property('a.b'));
             * // => [2, 1]
             *
             * _.map(_.sortBy(objects, _.property(['a', 'b'])), 'a.b');
             * // => [1, 2]
             */
            function property(path) {
                return isKey(path) ? baseProperty(toKey(path)) : basePropertyDeep(path);
            }
            /**
             * The opposite of `_.property`; this method creates a function that returns
             * the value at a given path of `object`.
             *
             * @static
             * @memberOf _
             * @since 3.0.0
             * @category Util
             * @param {Object} object The object to query.
             * @returns {Function} Returns the new accessor function.
             * @example
             *
             * var array = [0, 1, 2],
             *     object = { 'a': array, 'b': array, 'c': array };
             *
             * _.map(['a[2]', 'c[0]'], _.propertyOf(object));
             * // => [2, 0]
             *
             * _.map([['a', '2'], ['c', '0']], _.propertyOf(object));
             * // => [2, 0]
             */
            function propertyOf(object) {
                return function (path) {
                    return object == null ? undefined : baseGet(object, path);
                };
            }
            /**
             * Creates an array of numbers (positive and/or negative) progressing from
             * `start` up to, but not including, `end`. A step of `-1` is used if a negative
             * `start` is specified without an `end` or `step`. If `end` is not specified,
             * it's set to `start` with `start` then set to `0`.
             *
             * **Note:** JavaScript follows the IEEE-754 standard for resolving
             * floating-point values which can produce unexpected results.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Util
             * @param {number} [start=0] The start of the range.
             * @param {number} end The end of the range.
             * @param {number} [step=1] The value to increment or decrement by.
             * @returns {Array} Returns the range of numbers.
             * @see _.inRange, _.rangeRight
             * @example
             *
             * _.range(4);
             * // => [0, 1, 2, 3]
             *
             * _.range(-4);
             * // => [0, -1, -2, -3]
             *
             * _.range(1, 5);
             * // => [1, 2, 3, 4]
             *
             * _.range(0, 20, 5);
             * // => [0, 5, 10, 15]
             *
             * _.range(0, -4, -1);
             * // => [0, -1, -2, -3]
             *
             * _.range(1, 4, 0);
             * // => [1, 1, 1]
             *
             * _.range(0);
             * // => []
             */
            var range = createRange();
            /**
             * This method is like `_.range` except that it populates values in
             * descending order.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Util
             * @param {number} [start=0] The start of the range.
             * @param {number} end The end of the range.
             * @param {number} [step=1] The value to increment or decrement by.
             * @returns {Array} Returns the range of numbers.
             * @see _.inRange, _.range
             * @example
             *
             * _.rangeRight(4);
             * // => [3, 2, 1, 0]
             *
             * _.rangeRight(-4);
             * // => [-3, -2, -1, 0]
             *
             * _.rangeRight(1, 5);
             * // => [4, 3, 2, 1]
             *
             * _.rangeRight(0, 20, 5);
             * // => [15, 10, 5, 0]
             *
             * _.rangeRight(0, -4, -1);
             * // => [-3, -2, -1, 0]
             *
             * _.rangeRight(1, 4, 0);
             * // => [1, 1, 1]
             *
             * _.rangeRight(0);
             * // => []
             */
            var rangeRight = createRange(true);
            /**
             * This method returns a new empty array.
             *
             * @static
             * @memberOf _
             * @since 4.13.0
             * @category Util
             * @returns {Array} Returns the new empty array.
             * @example
             *
             * var arrays = _.times(2, _.stubArray);
             *
             * console.log(arrays);
             * // => [[], []]
             *
             * console.log(arrays[0] === arrays[1]);
             * // => false
             */
            function stubArray() {
                return [];
            }
            /**
             * This method returns `false`.
             *
             * @static
             * @memberOf _
             * @since 4.13.0
             * @category Util
             * @returns {boolean} Returns `false`.
             * @example
             *
             * _.times(2, _.stubFalse);
             * // => [false, false]
             */
            function stubFalse() {
                return false;
            }
            /**
             * This method returns a new empty object.
             *
             * @static
             * @memberOf _
             * @since 4.13.0
             * @category Util
             * @returns {Object} Returns the new empty object.
             * @example
             *
             * var objects = _.times(2, _.stubObject);
             *
             * console.log(objects);
             * // => [{}, {}]
             *
             * console.log(objects[0] === objects[1]);
             * // => false
             */
            function stubObject() {
                return {};
            }
            /**
             * This method returns an empty string.
             *
             * @static
             * @memberOf _
             * @since 4.13.0
             * @category Util
             * @returns {string} Returns the empty string.
             * @example
             *
             * _.times(2, _.stubString);
             * // => ['', '']
             */
            function stubString() {
                return '';
            }
            /**
             * This method returns `true`.
             *
             * @static
             * @memberOf _
             * @since 4.13.0
             * @category Util
             * @returns {boolean} Returns `true`.
             * @example
             *
             * _.times(2, _.stubTrue);
             * // => [true, true]
             */
            function stubTrue() {
                return true;
            }
            /**
             * Invokes the iteratee `n` times, returning an array of the results of
             * each invocation. The iteratee is invoked with one argument; (index).
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Util
             * @param {number} n The number of times to invoke `iteratee`.
             * @param {Function} [iteratee=_.identity] The function invoked per iteration.
             * @returns {Array} Returns the array of results.
             * @example
             *
             * _.times(3, String);
             * // => ['0', '1', '2']
             *
             *  _.times(4, _.constant(0));
             * // => [0, 0, 0, 0]
             */
            function times(n, iteratee) {
                n = toInteger(n);
                if (n < 1 || n > MAX_SAFE_INTEGER) {
                    return [];
                }
                var index = MAX_ARRAY_LENGTH, length = nativeMin(n, MAX_ARRAY_LENGTH);
                iteratee = getIteratee(iteratee);
                n -= MAX_ARRAY_LENGTH;
                var result = baseTimes(length, iteratee);
                while (++index < n) {
                    iteratee(index);
                }
                return result;
            }
            /**
             * Converts `value` to a property path array.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Util
             * @param {*} value The value to convert.
             * @returns {Array} Returns the new property path array.
             * @example
             *
             * _.toPath('a.b.c');
             * // => ['a', 'b', 'c']
             *
             * _.toPath('a[0].b.c');
             * // => ['a', '0', 'b', 'c']
             */
            function toPath(value) {
                if (isArray(value)) {
                    return arrayMap(value, toKey);
                }
                return isSymbol(value) ? [value] : copyArray(stringToPath(toString(value)));
            }
            /**
             * Generates a unique ID. If `prefix` is given, the ID is appended to it.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Util
             * @param {string} [prefix=''] The value to prefix the ID with.
             * @returns {string} Returns the unique ID.
             * @example
             *
             * _.uniqueId('contact_');
             * // => 'contact_104'
             *
             * _.uniqueId();
             * // => '105'
             */
            function uniqueId(prefix) {
                var id = ++idCounter;
                return toString(prefix) + id;
            }
            /*------------------------------------------------------------------------*/
            /**
             * Adds two numbers.
             *
             * @static
             * @memberOf _
             * @since 3.4.0
             * @category Math
             * @param {number} augend The first number in an addition.
             * @param {number} addend The second number in an addition.
             * @returns {number} Returns the total.
             * @example
             *
             * _.add(6, 4);
             * // => 10
             */
            var add = createMathOperation(function (augend, addend) {
                return augend + addend;
            }, 0);
            /**
             * Computes `number` rounded up to `precision`.
             *
             * @static
             * @memberOf _
             * @since 3.10.0
             * @category Math
             * @param {number} number The number to round up.
             * @param {number} [precision=0] The precision to round up to.
             * @returns {number} Returns the rounded up number.
             * @example
             *
             * _.ceil(4.006);
             * // => 5
             *
             * _.ceil(6.004, 2);
             * // => 6.01
             *
             * _.ceil(6040, -2);
             * // => 6100
             */
            var ceil = createRound('ceil');
            /**
             * Divide two numbers.
             *
             * @static
             * @memberOf _
             * @since 4.7.0
             * @category Math
             * @param {number} dividend The first number in a division.
             * @param {number} divisor The second number in a division.
             * @returns {number} Returns the quotient.
             * @example
             *
             * _.divide(6, 4);
             * // => 1.5
             */
            var divide = createMathOperation(function (dividend, divisor) {
                return dividend / divisor;
            }, 1);
            /**
             * Computes `number` rounded down to `precision`.
             *
             * @static
             * @memberOf _
             * @since 3.10.0
             * @category Math
             * @param {number} number The number to round down.
             * @param {number} [precision=0] The precision to round down to.
             * @returns {number} Returns the rounded down number.
             * @example
             *
             * _.floor(4.006);
             * // => 4
             *
             * _.floor(0.046, 2);
             * // => 0.04
             *
             * _.floor(4060, -2);
             * // => 4000
             */
            var floor = createRound('floor');
            /**
             * Computes the maximum value of `array`. If `array` is empty or falsey,
             * `undefined` is returned.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Math
             * @param {Array} array The array to iterate over.
             * @returns {*} Returns the maximum value.
             * @example
             *
             * _.max([4, 2, 8, 6]);
             * // => 8
             *
             * _.max([]);
             * // => undefined
             */
            function max(array) {
                return (array && array.length)
                    ? baseExtremum(array, identity, baseGt)
                    : undefined;
            }
            /**
             * This method is like `_.max` except that it accepts `iteratee` which is
             * invoked for each element in `array` to generate the criterion by which
             * the value is ranked. The iteratee is invoked with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Math
             * @param {Array} array The array to iterate over.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {*} Returns the maximum value.
             * @example
             *
             * var objects = [{ 'n': 1 }, { 'n': 2 }];
             *
             * _.maxBy(objects, function(o) { return o.n; });
             * // => { 'n': 2 }
             *
             * // The `_.property` iteratee shorthand.
             * _.maxBy(objects, 'n');
             * // => { 'n': 2 }
             */
            function maxBy(array, iteratee) {
                return (array && array.length)
                    ? baseExtremum(array, getIteratee(iteratee, 2), baseGt)
                    : undefined;
            }
            /**
             * Computes the mean of the values in `array`.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Math
             * @param {Array} array The array to iterate over.
             * @returns {number} Returns the mean.
             * @example
             *
             * _.mean([4, 2, 8, 6]);
             * // => 5
             */
            function mean(array) {
                return baseMean(array, identity);
            }
            /**
             * This method is like `_.mean` except that it accepts `iteratee` which is
             * invoked for each element in `array` to generate the value to be averaged.
             * The iteratee is invoked with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 4.7.0
             * @category Math
             * @param {Array} array The array to iterate over.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {number} Returns the mean.
             * @example
             *
             * var objects = [{ 'n': 4 }, { 'n': 2 }, { 'n': 8 }, { 'n': 6 }];
             *
             * _.meanBy(objects, function(o) { return o.n; });
             * // => 5
             *
             * // The `_.property` iteratee shorthand.
             * _.meanBy(objects, 'n');
             * // => 5
             */
            function meanBy(array, iteratee) {
                return baseMean(array, getIteratee(iteratee, 2));
            }
            /**
             * Computes the minimum value of `array`. If `array` is empty or falsey,
             * `undefined` is returned.
             *
             * @static
             * @since 0.1.0
             * @memberOf _
             * @category Math
             * @param {Array} array The array to iterate over.
             * @returns {*} Returns the minimum value.
             * @example
             *
             * _.min([4, 2, 8, 6]);
             * // => 2
             *
             * _.min([]);
             * // => undefined
             */
            function min(array) {
                return (array && array.length)
                    ? baseExtremum(array, identity, baseLt)
                    : undefined;
            }
            /**
             * This method is like `_.min` except that it accepts `iteratee` which is
             * invoked for each element in `array` to generate the criterion by which
             * the value is ranked. The iteratee is invoked with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Math
             * @param {Array} array The array to iterate over.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {*} Returns the minimum value.
             * @example
             *
             * var objects = [{ 'n': 1 }, { 'n': 2 }];
             *
             * _.minBy(objects, function(o) { return o.n; });
             * // => { 'n': 1 }
             *
             * // The `_.property` iteratee shorthand.
             * _.minBy(objects, 'n');
             * // => { 'n': 1 }
             */
            function minBy(array, iteratee) {
                return (array && array.length)
                    ? baseExtremum(array, getIteratee(iteratee, 2), baseLt)
                    : undefined;
            }
            /**
             * Multiply two numbers.
             *
             * @static
             * @memberOf _
             * @since 4.7.0
             * @category Math
             * @param {number} multiplier The first number in a multiplication.
             * @param {number} multiplicand The second number in a multiplication.
             * @returns {number} Returns the product.
             * @example
             *
             * _.multiply(6, 4);
             * // => 24
             */
            var multiply = createMathOperation(function (multiplier, multiplicand) {
                return multiplier * multiplicand;
            }, 1);
            /**
             * Computes `number` rounded to `precision`.
             *
             * @static
             * @memberOf _
             * @since 3.10.0
             * @category Math
             * @param {number} number The number to round.
             * @param {number} [precision=0] The precision to round to.
             * @returns {number} Returns the rounded number.
             * @example
             *
             * _.round(4.006);
             * // => 4
             *
             * _.round(4.006, 2);
             * // => 4.01
             *
             * _.round(4060, -2);
             * // => 4100
             */
            var round = createRound('round');
            /**
             * Subtract two numbers.
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Math
             * @param {number} minuend The first number in a subtraction.
             * @param {number} subtrahend The second number in a subtraction.
             * @returns {number} Returns the difference.
             * @example
             *
             * _.subtract(6, 4);
             * // => 2
             */
            var subtract = createMathOperation(function (minuend, subtrahend) {
                return minuend - subtrahend;
            }, 0);
            /**
             * Computes the sum of the values in `array`.
             *
             * @static
             * @memberOf _
             * @since 3.4.0
             * @category Math
             * @param {Array} array The array to iterate over.
             * @returns {number} Returns the sum.
             * @example
             *
             * _.sum([4, 2, 8, 6]);
             * // => 20
             */
            function sum(array) {
                return (array && array.length)
                    ? baseSum(array, identity)
                    : 0;
            }
            /**
             * This method is like `_.sum` except that it accepts `iteratee` which is
             * invoked for each element in `array` to generate the value to be summed.
             * The iteratee is invoked with one argument: (value).
             *
             * @static
             * @memberOf _
             * @since 4.0.0
             * @category Math
             * @param {Array} array The array to iterate over.
             * @param {Function} [iteratee=_.identity] The iteratee invoked per element.
             * @returns {number} Returns the sum.
             * @example
             *
             * var objects = [{ 'n': 4 }, { 'n': 2 }, { 'n': 8 }, { 'n': 6 }];
             *
             * _.sumBy(objects, function(o) { return o.n; });
             * // => 20
             *
             * // The `_.property` iteratee shorthand.
             * _.sumBy(objects, 'n');
             * // => 20
             */
            function sumBy(array, iteratee) {
                return (array && array.length)
                    ? baseSum(array, getIteratee(iteratee, 2))
                    : 0;
            }
            /*------------------------------------------------------------------------*/
            // Add methods that return wrapped values in chain sequences.
            lodash.after = after;
            lodash.ary = ary;
            lodash.assign = assign;
            lodash.assignIn = assignIn;
            lodash.assignInWith = assignInWith;
            lodash.assignWith = assignWith;
            lodash.at = at;
            lodash.before = before;
            lodash.bind = bind;
            lodash.bindAll = bindAll;
            lodash.bindKey = bindKey;
            lodash.castArray = castArray;
            lodash.chain = chain;
            lodash.chunk = chunk;
            lodash.compact = compact;
            lodash.concat = concat;
            lodash.cond = cond;
            lodash.conforms = conforms;
            lodash.constant = constant;
            lodash.countBy = countBy;
            lodash.create = create;
            lodash.curry = curry;
            lodash.curryRight = curryRight;
            lodash.debounce = debounce;
            lodash.defaults = defaults;
            lodash.defaultsDeep = defaultsDeep;
            lodash.defer = defer;
            lodash.delay = delay;
            lodash.difference = difference;
            lodash.differenceBy = differenceBy;
            lodash.differenceWith = differenceWith;
            lodash.drop = drop;
            lodash.dropRight = dropRight;
            lodash.dropRightWhile = dropRightWhile;
            lodash.dropWhile = dropWhile;
            lodash.fill = fill;
            lodash.filter = filter;
            lodash.flatMap = flatMap;
            lodash.flatMapDeep = flatMapDeep;
            lodash.flatMapDepth = flatMapDepth;
            lodash.flatten = flatten;
            lodash.flattenDeep = flattenDeep;
            lodash.flattenDepth = flattenDepth;
            lodash.flip = flip;
            lodash.flow = flow;
            lodash.flowRight = flowRight;
            lodash.fromPairs = fromPairs;
            lodash.functions = functions;
            lodash.functionsIn = functionsIn;
            lodash.groupBy = groupBy;
            lodash.initial = initial;
            lodash.intersection = intersection;
            lodash.intersectionBy = intersectionBy;
            lodash.intersectionWith = intersectionWith;
            lodash.invert = invert;
            lodash.invertBy = invertBy;
            lodash.invokeMap = invokeMap;
            lodash.iteratee = iteratee;
            lodash.keyBy = keyBy;
            lodash.keys = keys;
            lodash.keysIn = keysIn;
            lodash.map = map;
            lodash.mapKeys = mapKeys;
            lodash.mapValues = mapValues;
            lodash.matches = matches;
            lodash.matchesProperty = matchesProperty;
            lodash.memoize = memoize;
            lodash.merge = merge;
            lodash.mergeWith = mergeWith;
            lodash.method = method;
            lodash.methodOf = methodOf;
            lodash.mixin = mixin;
            lodash.negate = negate;
            lodash.nthArg = nthArg;
            lodash.omit = omit;
            lodash.omitBy = omitBy;
            lodash.once = once;
            lodash.orderBy = orderBy;
            lodash.over = over;
            lodash.overArgs = overArgs;
            lodash.overEvery = overEvery;
            lodash.overSome = overSome;
            lodash.partial = partial;
            lodash.partialRight = partialRight;
            lodash.partition = partition;
            lodash.pick = pick;
            lodash.pickBy = pickBy;
            lodash.property = property;
            lodash.propertyOf = propertyOf;
            lodash.pull = pull;
            lodash.pullAll = pullAll;
            lodash.pullAllBy = pullAllBy;
            lodash.pullAllWith = pullAllWith;
            lodash.pullAt = pullAt;
            lodash.range = range;
            lodash.rangeRight = rangeRight;
            lodash.rearg = rearg;
            lodash.reject = reject;
            lodash.remove = remove;
            lodash.rest = rest;
            lodash.reverse = reverse;
            lodash.sampleSize = sampleSize;
            lodash.set = set;
            lodash.setWith = setWith;
            lodash.shuffle = shuffle;
            lodash.slice = slice;
            lodash.sortBy = sortBy;
            lodash.sortedUniq = sortedUniq;
            lodash.sortedUniqBy = sortedUniqBy;
            lodash.split = split;
            lodash.spread = spread;
            lodash.tail = tail;
            lodash.take = take;
            lodash.takeRight = takeRight;
            lodash.takeRightWhile = takeRightWhile;
            lodash.takeWhile = takeWhile;
            lodash.tap = tap;
            lodash.throttle = throttle;
            lodash.thru = thru;
            lodash.toArray = toArray;
            lodash.toPairs = toPairs;
            lodash.toPairsIn = toPairsIn;
            lodash.toPath = toPath;
            lodash.toPlainObject = toPlainObject;
            lodash.transform = transform;
            lodash.unary = unary;
            lodash.union = union;
            lodash.unionBy = unionBy;
            lodash.unionWith = unionWith;
            lodash.uniq = uniq;
            lodash.uniqBy = uniqBy;
            lodash.uniqWith = uniqWith;
            lodash.unset = unset;
            lodash.unzip = unzip;
            lodash.unzipWith = unzipWith;
            lodash.update = update;
            lodash.updateWith = updateWith;
            lodash.values = values;
            lodash.valuesIn = valuesIn;
            lodash.without = without;
            lodash.words = words;
            lodash.wrap = wrap;
            lodash.xor = xor;
            lodash.xorBy = xorBy;
            lodash.xorWith = xorWith;
            lodash.zip = zip;
            lodash.zipObject = zipObject;
            lodash.zipObjectDeep = zipObjectDeep;
            lodash.zipWith = zipWith;
            // Add aliases.
            lodash.entries = toPairs;
            lodash.entriesIn = toPairsIn;
            lodash.extend = assignIn;
            lodash.extendWith = assignInWith;
            // Add methods to `lodash.prototype`.
            mixin(lodash, lodash);
            /*------------------------------------------------------------------------*/
            // Add methods that return unwrapped values in chain sequences.
            lodash.add = add;
            lodash.attempt = attempt;
            lodash.camelCase = camelCase;
            lodash.capitalize = capitalize;
            lodash.ceil = ceil;
            lodash.clamp = clamp;
            lodash.clone = clone;
            lodash.cloneDeep = cloneDeep;
            lodash.cloneDeepWith = cloneDeepWith;
            lodash.cloneWith = cloneWith;
            lodash.conformsTo = conformsTo;
            lodash.deburr = deburr;
            lodash.defaultTo = defaultTo;
            lodash.divide = divide;
            lodash.endsWith = endsWith;
            lodash.eq = eq;
            lodash.escape = escape;
            lodash.escapeRegExp = escapeRegExp;
            lodash.every = every;
            lodash.find = find;
            lodash.findIndex = findIndex;
            lodash.findKey = findKey;
            lodash.findLast = findLast;
            lodash.findLastIndex = findLastIndex;
            lodash.findLastKey = findLastKey;
            lodash.floor = floor;
            lodash.forEach = forEach;
            lodash.forEachRight = forEachRight;
            lodash.forIn = forIn;
            lodash.forInRight = forInRight;
            lodash.forOwn = forOwn;
            lodash.forOwnRight = forOwnRight;
            lodash.get = get;
            lodash.gt = gt;
            lodash.gte = gte;
            lodash.has = has;
            lodash.hasIn = hasIn;
            lodash.head = head;
            lodash.identity = identity;
            lodash.includes = includes;
            lodash.indexOf = indexOf;
            lodash.inRange = inRange;
            lodash.invoke = invoke;
            lodash.isArguments = isArguments;
            lodash.isArray = isArray;
            lodash.isArrayBuffer = isArrayBuffer;
            lodash.isArrayLike = isArrayLike;
            lodash.isArrayLikeObject = isArrayLikeObject;
            lodash.isBoolean = isBoolean;
            lodash.isBuffer = isBuffer;
            lodash.isDate = isDate;
            lodash.isElement = isElement;
            lodash.isEmpty = isEmpty;
            lodash.isEqual = isEqual;
            lodash.isEqualWith = isEqualWith;
            lodash.isError = isError;
            lodash.isFinite = isFinite;
            lodash.isFunction = isFunction;
            lodash.isInteger = isInteger;
            lodash.isLength = isLength;
            lodash.isMap = isMap;
            lodash.isMatch = isMatch;
            lodash.isMatchWith = isMatchWith;
            lodash.isNaN = isNaN;
            lodash.isNative = isNative;
            lodash.isNil = isNil;
            lodash.isNull = isNull;
            lodash.isNumber = isNumber;
            lodash.isObject = isObject;
            lodash.isObjectLike = isObjectLike;
            lodash.isPlainObject = isPlainObject;
            lodash.isRegExp = isRegExp;
            lodash.isSafeInteger = isSafeInteger;
            lodash.isSet = isSet;
            lodash.isString = isString;
            lodash.isSymbol = isSymbol;
            lodash.isTypedArray = isTypedArray;
            lodash.isUndefined = isUndefined;
            lodash.isWeakMap = isWeakMap;
            lodash.isWeakSet = isWeakSet;
            lodash.join = join;
            lodash.kebabCase = kebabCase;
            lodash.last = last;
            lodash.lastIndexOf = lastIndexOf;
            lodash.lowerCase = lowerCase;
            lodash.lowerFirst = lowerFirst;
            lodash.lt = lt;
            lodash.lte = lte;
            lodash.max = max;
            lodash.maxBy = maxBy;
            lodash.mean = mean;
            lodash.meanBy = meanBy;
            lodash.min = min;
            lodash.minBy = minBy;
            lodash.stubArray = stubArray;
            lodash.stubFalse = stubFalse;
            lodash.stubObject = stubObject;
            lodash.stubString = stubString;
            lodash.stubTrue = stubTrue;
            lodash.multiply = multiply;
            lodash.nth = nth;
            lodash.noConflict = noConflict;
            lodash.noop = noop;
            lodash.now = now;
            lodash.pad = pad;
            lodash.padEnd = padEnd;
            lodash.padStart = padStart;
            lodash.parseInt = parseInt;
            lodash.random = random;
            lodash.reduce = reduce;
            lodash.reduceRight = reduceRight;
            lodash.repeat = repeat;
            lodash.replace = replace;
            lodash.result = result;
            lodash.round = round;
            lodash.runInContext = runInContext;
            lodash.sample = sample;
            lodash.size = size;
            lodash.snakeCase = snakeCase;
            lodash.some = some;
            lodash.sortedIndex = sortedIndex;
            lodash.sortedIndexBy = sortedIndexBy;
            lodash.sortedIndexOf = sortedIndexOf;
            lodash.sortedLastIndex = sortedLastIndex;
            lodash.sortedLastIndexBy = sortedLastIndexBy;
            lodash.sortedLastIndexOf = sortedLastIndexOf;
            lodash.startCase = startCase;
            lodash.startsWith = startsWith;
            lodash.subtract = subtract;
            lodash.sum = sum;
            lodash.sumBy = sumBy;
            lodash.template = template;
            lodash.times = times;
            lodash.toFinite = toFinite;
            lodash.toInteger = toInteger;
            lodash.toLength = toLength;
            lodash.toLower = toLower;
            lodash.toNumber = toNumber;
            lodash.toSafeInteger = toSafeInteger;
            lodash.toString = toString;
            lodash.toUpper = toUpper;
            lodash.trim = trim;
            lodash.trimEnd = trimEnd;
            lodash.trimStart = trimStart;
            lodash.truncate = truncate;
            lodash.unescape = unescape;
            lodash.uniqueId = uniqueId;
            lodash.upperCase = upperCase;
            lodash.upperFirst = upperFirst;
            // Add aliases.
            lodash.each = forEach;
            lodash.eachRight = forEachRight;
            lodash.first = head;
            mixin(lodash, (function () {
                var source = {};
                baseForOwn(lodash, function (func, methodName) {
                    if (!hasOwnProperty.call(lodash.prototype, methodName)) {
                        source[methodName] = func;
                    }
                });
                return source;
            }()), { 'chain': false });
            /*------------------------------------------------------------------------*/
            /**
             * The semantic version number.
             *
             * @static
             * @memberOf _
             * @type {string}
             */
            lodash.VERSION = VERSION;
            // Assign default placeholders.
            arrayEach(['bind', 'bindKey', 'curry', 'curryRight', 'partial', 'partialRight'], function (methodName) {
                lodash[methodName].placeholder = lodash;
            });
            // Add `LazyWrapper` methods for `_.drop` and `_.take` variants.
            arrayEach(['drop', 'take'], function (methodName, index) {
                LazyWrapper.prototype[methodName] = function (n) {
                    n = n === undefined ? 1 : nativeMax(toInteger(n), 0);
                    var result = (this.__filtered__ && !index)
                        ? new LazyWrapper(this)
                        : this.clone();
                    if (result.__filtered__) {
                        result.__takeCount__ = nativeMin(n, result.__takeCount__);
                    }
                    else {
                        result.__views__.push({
                            'size': nativeMin(n, MAX_ARRAY_LENGTH),
                            'type': methodName + (result.__dir__ < 0 ? 'Right' : '')
                        });
                    }
                    return result;
                };
                LazyWrapper.prototype[methodName + 'Right'] = function (n) {
                    return this.reverse()[methodName](n).reverse();
                };
            });
            // Add `LazyWrapper` methods that accept an `iteratee` value.
            arrayEach(['filter', 'map', 'takeWhile'], function (methodName, index) {
                var type = index + 1, isFilter = type == LAZY_FILTER_FLAG || type == LAZY_WHILE_FLAG;
                LazyWrapper.prototype[methodName] = function (iteratee) {
                    var result = this.clone();
                    result.__iteratees__.push({
                        'iteratee': getIteratee(iteratee, 3),
                        'type': type
                    });
                    result.__filtered__ = result.__filtered__ || isFilter;
                    return result;
                };
            });
            // Add `LazyWrapper` methods for `_.head` and `_.last`.
            arrayEach(['head', 'last'], function (methodName, index) {
                var takeName = 'take' + (index ? 'Right' : '');
                LazyWrapper.prototype[methodName] = function () {
                    return this[takeName](1).value()[0];
                };
            });
            // Add `LazyWrapper` methods for `_.initial` and `_.tail`.
            arrayEach(['initial', 'tail'], function (methodName, index) {
                var dropName = 'drop' + (index ? '' : 'Right');
                LazyWrapper.prototype[methodName] = function () {
                    return this.__filtered__ ? new LazyWrapper(this) : this[dropName](1);
                };
            });
            LazyWrapper.prototype.compact = function () {
                return this.filter(identity);
            };
            LazyWrapper.prototype.find = function (predicate) {
                return this.filter(predicate).head();
            };
            LazyWrapper.prototype.findLast = function (predicate) {
                return this.reverse().find(predicate);
            };
            LazyWrapper.prototype.invokeMap = baseRest(function (path, args) {
                if (typeof path == 'function') {
                    return new LazyWrapper(this);
                }
                return this.map(function (value) {
                    return baseInvoke(value, path, args);
                });
            });
            LazyWrapper.prototype.reject = function (predicate) {
                return this.filter(negate(getIteratee(predicate)));
            };
            LazyWrapper.prototype.slice = function (start, end) {
                start = toInteger(start);
                var result = this;
                if (result.__filtered__ && (start > 0 || end < 0)) {
                    return new LazyWrapper(result);
                }
                if (start < 0) {
                    result = result.takeRight(-start);
                }
                else if (start) {
                    result = result.drop(start);
                }
                if (end !== undefined) {
                    end = toInteger(end);
                    result = end < 0 ? result.dropRight(-end) : result.take(end - start);
                }
                return result;
            };
            LazyWrapper.prototype.takeRightWhile = function (predicate) {
                return this.reverse().takeWhile(predicate).reverse();
            };
            LazyWrapper.prototype.toArray = function () {
                return this.take(MAX_ARRAY_LENGTH);
            };
            // Add `LazyWrapper` methods to `lodash.prototype`.
            baseForOwn(LazyWrapper.prototype, function (func, methodName) {
                var checkIteratee = /^(?:filter|find|map|reject)|While$/.test(methodName), isTaker = /^(?:head|last)$/.test(methodName), lodashFunc = lodash[isTaker ? ('take' + (methodName == 'last' ? 'Right' : '')) : methodName], retUnwrapped = isTaker || /^find/.test(methodName);
                if (!lodashFunc) {
                    return;
                }
                lodash.prototype[methodName] = function () {
                    var value = this.__wrapped__, args = isTaker ? [1] : arguments, isLazy = value instanceof LazyWrapper, iteratee = args[0], useLazy = isLazy || isArray(value);
                    var interceptor = function (value) {
                        var result = lodashFunc.apply(lodash, arrayPush([value], args));
                        return (isTaker && chainAll) ? result[0] : result;
                    };
                    if (useLazy && checkIteratee && typeof iteratee == 'function' && iteratee.length != 1) {
                        // Avoid lazy use if the iteratee has a "length" value other than `1`.
                        isLazy = useLazy = false;
                    }
                    var chainAll = this.__chain__, isHybrid = !!this.__actions__.length, isUnwrapped = retUnwrapped && !chainAll, onlyLazy = isLazy && !isHybrid;
                    if (!retUnwrapped && useLazy) {
                        value = onlyLazy ? value : new LazyWrapper(this);
                        var result = func.apply(value, args);
                        result.__actions__.push({ 'func': thru, 'args': [interceptor], 'thisArg': undefined });
                        return new LodashWrapper(result, chainAll);
                    }
                    if (isUnwrapped && onlyLazy) {
                        return func.apply(this, args);
                    }
                    result = this.thru(interceptor);
                    return isUnwrapped ? (isTaker ? result.value()[0] : result.value()) : result;
                };
            });
            // Add `Array` methods to `lodash.prototype`.
            arrayEach(['pop', 'push', 'shift', 'sort', 'splice', 'unshift'], function (methodName) {
                var func = arrayProto[methodName], chainName = /^(?:push|sort|unshift)$/.test(methodName) ? 'tap' : 'thru', retUnwrapped = /^(?:pop|shift)$/.test(methodName);
                lodash.prototype[methodName] = function () {
                    var args = arguments;
                    if (retUnwrapped && !this.__chain__) {
                        var value = this.value();
                        return func.apply(isArray(value) ? value : [], args);
                    }
                    return this[chainName](function (value) {
                        return func.apply(isArray(value) ? value : [], args);
                    });
                };
            });
            // Map minified method names to their real names.
            baseForOwn(LazyWrapper.prototype, function (func, methodName) {
                var lodashFunc = lodash[methodName];
                if (lodashFunc) {
                    var key = lodashFunc.name + '';
                    if (!hasOwnProperty.call(realNames, key)) {
                        realNames[key] = [];
                    }
                    realNames[key].push({ 'name': methodName, 'func': lodashFunc });
                }
            });
            realNames[createHybrid(undefined, WRAP_BIND_KEY_FLAG).name] = [{
                    'name': 'wrapper',
                    'func': undefined
                }];
            // Add methods to `LazyWrapper`.
            LazyWrapper.prototype.clone = lazyClone;
            LazyWrapper.prototype.reverse = lazyReverse;
            LazyWrapper.prototype.value = lazyValue;
            // Add chain sequence methods to the `lodash` wrapper.
            lodash.prototype.at = wrapperAt;
            lodash.prototype.chain = wrapperChain;
            lodash.prototype.commit = wrapperCommit;
            lodash.prototype.next = wrapperNext;
            lodash.prototype.plant = wrapperPlant;
            lodash.prototype.reverse = wrapperReverse;
            lodash.prototype.toJSON = lodash.prototype.valueOf = lodash.prototype.value = wrapperValue;
            // Add lazy aliases.
            lodash.prototype.first = lodash.prototype.head;
            if (symIterator) {
                lodash.prototype[symIterator] = wrapperToIterator;
            }
            return lodash;
        });
        /*--------------------------------------------------------------------------*/
        // Export lodash.
        var _ = runInContext();
        // Some AMD build optimizers, like r.js, check for condition patterns like:
        if (typeof define == 'function' && typeof define.amd == 'object' && define.amd) {
            // Expose Lodash on the global object to prevent errors when Lodash is
            // loaded by a script tag in the presence of an AMD loader.
            // See http://requirejs.org/docs/errors.html#mismatch for more details.
            // Use `_.noConflict` to remove Lodash from the global object.
            root._ = _;
            // Define as an anonymous module so, through path mapping, it can be
            // referenced as the "underscore" module.
            define(function () {
                return _;
            });
        }
        // Check for `exports` after `define` in case a build optimizer adds it.
        else if (freeModule) {
            // Export for Node.js.
            (freeModule.exports = _)._ = _;
            // Export for CommonJS support.
            freeExports._ = _;
        }
        else {
            // Export to the global object.
            root._ = _;
        }
    }.call(this));
},
/* eslint/conf/environments.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const globals = require(113) /* globals */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Get the object that has differentce.
     * @param {Record<string,boolean>} current The newer object.
     * @param {Record<string,boolean>} prev The older object.
     * @returns {Record<string,boolean>} The difference object.
     */
    function getDiff(current, prev) {
        const retv = {};
        for (const [key, value] of Object.entries(current)) {
            if (!Object.hasOwnProperty.call(prev, key)) {
                retv[key] = value;
            }
        }
        return retv;
    }
    const newGlobals2015 = getDiff(globals.es2015, globals.es5); // 19 variables such as Promise, Map, ...
    const newGlobals2017 = {
        Atomics: false,
        SharedArrayBuffer: false
    };
    const newGlobals2020 = {
        BigInt: false,
        BigInt64Array: false,
        BigUint64Array: false
    };
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /** @type {Map<string, import("../lib/shared/types").Environment>} */
    module.exports = new Map(Object.entries({
        // Language
        builtin: {
            globals: globals.es5
        },
        es6: {
            globals: newGlobals2015,
            parserOptions: {
                ecmaVersion: 6
            }
        },
        es2015: {
            globals: newGlobals2015,
            parserOptions: {
                ecmaVersion: 6
            }
        },
        es2017: {
            globals: { ...newGlobals2015, ...newGlobals2017 },
            parserOptions: {
                ecmaVersion: 8
            }
        },
        es2020: {
            globals: { ...newGlobals2015, ...newGlobals2017, ...newGlobals2020 },
            parserOptions: {
                ecmaVersion: 11
            }
        },
        // Platforms
        browser: {
            globals: globals.browser
        },
        node: {
            globals: globals.node,
            parserOptions: {
                ecmaFeatures: {
                    globalReturn: true
                }
            }
        },
        "shared-node-browser": {
            globals: globals["shared-node-browser"]
        },
        worker: {
            globals: globals.worker
        },
        serviceworker: {
            globals: globals.serviceworker
        },
        // Frameworks
        commonjs: {
            globals: globals.commonjs,
            parserOptions: {
                ecmaFeatures: {
                    globalReturn: true
                }
            }
        },
        amd: {
            globals: globals.amd
        },
        mocha: {
            globals: globals.mocha
        },
        jasmine: {
            globals: globals.jasmine
        },
        jest: {
            globals: globals.jest
        },
        phantomjs: {
            globals: globals.phantomjs
        },
        jquery: {
            globals: globals.jquery
        },
        qunit: {
            globals: globals.qunit
        },
        prototypejs: {
            globals: globals.prototypejs
        },
        shelljs: {
            globals: globals.shelljs
        },
        meteor: {
            globals: globals.meteor
        },
        mongo: {
            globals: globals.mongo
        },
        protractor: {
            globals: globals.protractor
        },
        applescript: {
            globals: globals.applescript
        },
        nashorn: {
            globals: globals.nashorn
        },
        atomtest: {
            globals: globals.atomtest
        },
        embertest: {
            globals: globals.embertest
        },
        webextensions: {
            globals: globals.webextensions
        },
        greasemonkey: {
            globals: globals.greasemonkey
        }
    }));
},
/* globals/index.js */ function _(require, module, exports) {
    module.exports = require(114) /* ./globals.json */;
},
/* globals/globals.json */ function _(require, module, exports) {
    module.exports = {
        "builtin": {
            "Array": false,
            "ArrayBuffer": false,
            "Atomics": false,
            "BigInt": false,
            "BigInt64Array": false,
            "BigUint64Array": false,
            "Boolean": false,
            "constructor": false,
            "DataView": false,
            "Date": false,
            "decodeURI": false,
            "decodeURIComponent": false,
            "encodeURI": false,
            "encodeURIComponent": false,
            "Error": false,
            "escape": false,
            "eval": false,
            "EvalError": false,
            "Float32Array": false,
            "Float64Array": false,
            "Function": false,
            "globalThis": false,
            "hasOwnProperty": false,
            "Infinity": false,
            "Int16Array": false,
            "Int32Array": false,
            "Int8Array": false,
            "isFinite": false,
            "isNaN": false,
            "isPrototypeOf": false,
            "JSON": false,
            "Map": false,
            "Math": false,
            "NaN": false,
            "Number": false,
            "Object": false,
            "parseFloat": false,
            "parseInt": false,
            "Promise": false,
            "propertyIsEnumerable": false,
            "Proxy": false,
            "RangeError": false,
            "ReferenceError": false,
            "Reflect": false,
            "RegExp": false,
            "Set": false,
            "SharedArrayBuffer": false,
            "String": false,
            "Symbol": false,
            "SyntaxError": false,
            "toLocaleString": false,
            "toString": false,
            "TypeError": false,
            "Uint16Array": false,
            "Uint32Array": false,
            "Uint8Array": false,
            "Uint8ClampedArray": false,
            "undefined": false,
            "unescape": false,
            "URIError": false,
            "valueOf": false,
            "WeakMap": false,
            "WeakSet": false
        },
        "es5": {
            "Array": false,
            "Boolean": false,
            "constructor": false,
            "Date": false,
            "decodeURI": false,
            "decodeURIComponent": false,
            "encodeURI": false,
            "encodeURIComponent": false,
            "Error": false,
            "escape": false,
            "eval": false,
            "EvalError": false,
            "Function": false,
            "hasOwnProperty": false,
            "Infinity": false,
            "isFinite": false,
            "isNaN": false,
            "isPrototypeOf": false,
            "JSON": false,
            "Math": false,
            "NaN": false,
            "Number": false,
            "Object": false,
            "parseFloat": false,
            "parseInt": false,
            "propertyIsEnumerable": false,
            "RangeError": false,
            "ReferenceError": false,
            "RegExp": false,
            "String": false,
            "SyntaxError": false,
            "toLocaleString": false,
            "toString": false,
            "TypeError": false,
            "undefined": false,
            "unescape": false,
            "URIError": false,
            "valueOf": false
        },
        "es2015": {
            "Array": false,
            "ArrayBuffer": false,
            "Boolean": false,
            "constructor": false,
            "DataView": false,
            "Date": false,
            "decodeURI": false,
            "decodeURIComponent": false,
            "encodeURI": false,
            "encodeURIComponent": false,
            "Error": false,
            "escape": false,
            "eval": false,
            "EvalError": false,
            "Float32Array": false,
            "Float64Array": false,
            "Function": false,
            "hasOwnProperty": false,
            "Infinity": false,
            "Int16Array": false,
            "Int32Array": false,
            "Int8Array": false,
            "isFinite": false,
            "isNaN": false,
            "isPrototypeOf": false,
            "JSON": false,
            "Map": false,
            "Math": false,
            "NaN": false,
            "Number": false,
            "Object": false,
            "parseFloat": false,
            "parseInt": false,
            "Promise": false,
            "propertyIsEnumerable": false,
            "Proxy": false,
            "RangeError": false,
            "ReferenceError": false,
            "Reflect": false,
            "RegExp": false,
            "Set": false,
            "String": false,
            "Symbol": false,
            "SyntaxError": false,
            "toLocaleString": false,
            "toString": false,
            "TypeError": false,
            "Uint16Array": false,
            "Uint32Array": false,
            "Uint8Array": false,
            "Uint8ClampedArray": false,
            "undefined": false,
            "unescape": false,
            "URIError": false,
            "valueOf": false,
            "WeakMap": false,
            "WeakSet": false
        },
        "es2017": {
            "Array": false,
            "ArrayBuffer": false,
            "Atomics": false,
            "Boolean": false,
            "constructor": false,
            "DataView": false,
            "Date": false,
            "decodeURI": false,
            "decodeURIComponent": false,
            "encodeURI": false,
            "encodeURIComponent": false,
            "Error": false,
            "escape": false,
            "eval": false,
            "EvalError": false,
            "Float32Array": false,
            "Float64Array": false,
            "Function": false,
            "hasOwnProperty": false,
            "Infinity": false,
            "Int16Array": false,
            "Int32Array": false,
            "Int8Array": false,
            "isFinite": false,
            "isNaN": false,
            "isPrototypeOf": false,
            "JSON": false,
            "Map": false,
            "Math": false,
            "NaN": false,
            "Number": false,
            "Object": false,
            "parseFloat": false,
            "parseInt": false,
            "Promise": false,
            "propertyIsEnumerable": false,
            "Proxy": false,
            "RangeError": false,
            "ReferenceError": false,
            "Reflect": false,
            "RegExp": false,
            "Set": false,
            "SharedArrayBuffer": false,
            "String": false,
            "Symbol": false,
            "SyntaxError": false,
            "toLocaleString": false,
            "toString": false,
            "TypeError": false,
            "Uint16Array": false,
            "Uint32Array": false,
            "Uint8Array": false,
            "Uint8ClampedArray": false,
            "undefined": false,
            "unescape": false,
            "URIError": false,
            "valueOf": false,
            "WeakMap": false,
            "WeakSet": false
        },
        "browser": {
            "AbortController": false,
            "AbortSignal": false,
            "addEventListener": false,
            "alert": false,
            "AnalyserNode": false,
            "Animation": false,
            "AnimationEffectReadOnly": false,
            "AnimationEffectTiming": false,
            "AnimationEffectTimingReadOnly": false,
            "AnimationEvent": false,
            "AnimationPlaybackEvent": false,
            "AnimationTimeline": false,
            "applicationCache": false,
            "ApplicationCache": false,
            "ApplicationCacheErrorEvent": false,
            "atob": false,
            "Attr": false,
            "Audio": false,
            "AudioBuffer": false,
            "AudioBufferSourceNode": false,
            "AudioContext": false,
            "AudioDestinationNode": false,
            "AudioListener": false,
            "AudioNode": false,
            "AudioParam": false,
            "AudioProcessingEvent": false,
            "AudioScheduledSourceNode": false,
            "AudioWorkletGlobalScope ": false,
            "AudioWorkletNode": false,
            "AudioWorkletProcessor": false,
            "BarProp": false,
            "BaseAudioContext": false,
            "BatteryManager": false,
            "BeforeUnloadEvent": false,
            "BiquadFilterNode": false,
            "Blob": false,
            "BlobEvent": false,
            "blur": false,
            "BroadcastChannel": false,
            "btoa": false,
            "BudgetService": false,
            "ByteLengthQueuingStrategy": false,
            "Cache": false,
            "caches": false,
            "CacheStorage": false,
            "cancelAnimationFrame": false,
            "cancelIdleCallback": false,
            "CanvasCaptureMediaStreamTrack": false,
            "CanvasGradient": false,
            "CanvasPattern": false,
            "CanvasRenderingContext2D": false,
            "ChannelMergerNode": false,
            "ChannelSplitterNode": false,
            "CharacterData": false,
            "clearInterval": false,
            "clearTimeout": false,
            "clientInformation": false,
            "ClipboardEvent": false,
            "close": false,
            "closed": false,
            "CloseEvent": false,
            "Comment": false,
            "CompositionEvent": false,
            "confirm": false,
            "console": false,
            "ConstantSourceNode": false,
            "ConvolverNode": false,
            "CountQueuingStrategy": false,
            "createImageBitmap": false,
            "Credential": false,
            "CredentialsContainer": false,
            "crypto": false,
            "Crypto": false,
            "CryptoKey": false,
            "CSS": false,
            "CSSConditionRule": false,
            "CSSFontFaceRule": false,
            "CSSGroupingRule": false,
            "CSSImportRule": false,
            "CSSKeyframeRule": false,
            "CSSKeyframesRule": false,
            "CSSMediaRule": false,
            "CSSNamespaceRule": false,
            "CSSPageRule": false,
            "CSSRule": false,
            "CSSRuleList": false,
            "CSSStyleDeclaration": false,
            "CSSStyleRule": false,
            "CSSStyleSheet": false,
            "CSSSupportsRule": false,
            "CustomElementRegistry": false,
            "customElements": false,
            "CustomEvent": false,
            "DataTransfer": false,
            "DataTransferItem": false,
            "DataTransferItemList": false,
            "defaultstatus": false,
            "defaultStatus": false,
            "DelayNode": false,
            "DeviceMotionEvent": false,
            "DeviceOrientationEvent": false,
            "devicePixelRatio": false,
            "dispatchEvent": false,
            "document": false,
            "Document": false,
            "DocumentFragment": false,
            "DocumentType": false,
            "DOMError": false,
            "DOMException": false,
            "DOMImplementation": false,
            "DOMMatrix": false,
            "DOMMatrixReadOnly": false,
            "DOMParser": false,
            "DOMPoint": false,
            "DOMPointReadOnly": false,
            "DOMQuad": false,
            "DOMRect": false,
            "DOMRectReadOnly": false,
            "DOMStringList": false,
            "DOMStringMap": false,
            "DOMTokenList": false,
            "DragEvent": false,
            "DynamicsCompressorNode": false,
            "Element": false,
            "ErrorEvent": false,
            "event": false,
            "Event": false,
            "EventSource": false,
            "EventTarget": false,
            "external": false,
            "fetch": false,
            "File": false,
            "FileList": false,
            "FileReader": false,
            "find": false,
            "focus": false,
            "FocusEvent": false,
            "FontFace": false,
            "FontFaceSetLoadEvent": false,
            "FormData": false,
            "frameElement": false,
            "frames": false,
            "GainNode": false,
            "Gamepad": false,
            "GamepadButton": false,
            "GamepadEvent": false,
            "getComputedStyle": false,
            "getSelection": false,
            "HashChangeEvent": false,
            "Headers": false,
            "history": false,
            "History": false,
            "HTMLAllCollection": false,
            "HTMLAnchorElement": false,
            "HTMLAreaElement": false,
            "HTMLAudioElement": false,
            "HTMLBaseElement": false,
            "HTMLBodyElement": false,
            "HTMLBRElement": false,
            "HTMLButtonElement": false,
            "HTMLCanvasElement": false,
            "HTMLCollection": false,
            "HTMLContentElement": false,
            "HTMLDataElement": false,
            "HTMLDataListElement": false,
            "HTMLDetailsElement": false,
            "HTMLDialogElement": false,
            "HTMLDirectoryElement": false,
            "HTMLDivElement": false,
            "HTMLDListElement": false,
            "HTMLDocument": false,
            "HTMLElement": false,
            "HTMLEmbedElement": false,
            "HTMLFieldSetElement": false,
            "HTMLFontElement": false,
            "HTMLFormControlsCollection": false,
            "HTMLFormElement": false,
            "HTMLFrameElement": false,
            "HTMLFrameSetElement": false,
            "HTMLHeadElement": false,
            "HTMLHeadingElement": false,
            "HTMLHRElement": false,
            "HTMLHtmlElement": false,
            "HTMLIFrameElement": false,
            "HTMLImageElement": false,
            "HTMLInputElement": false,
            "HTMLLabelElement": false,
            "HTMLLegendElement": false,
            "HTMLLIElement": false,
            "HTMLLinkElement": false,
            "HTMLMapElement": false,
            "HTMLMarqueeElement": false,
            "HTMLMediaElement": false,
            "HTMLMenuElement": false,
            "HTMLMetaElement": false,
            "HTMLMeterElement": false,
            "HTMLModElement": false,
            "HTMLObjectElement": false,
            "HTMLOListElement": false,
            "HTMLOptGroupElement": false,
            "HTMLOptionElement": false,
            "HTMLOptionsCollection": false,
            "HTMLOutputElement": false,
            "HTMLParagraphElement": false,
            "HTMLParamElement": false,
            "HTMLPictureElement": false,
            "HTMLPreElement": false,
            "HTMLProgressElement": false,
            "HTMLQuoteElement": false,
            "HTMLScriptElement": false,
            "HTMLSelectElement": false,
            "HTMLShadowElement": false,
            "HTMLSlotElement": false,
            "HTMLSourceElement": false,
            "HTMLSpanElement": false,
            "HTMLStyleElement": false,
            "HTMLTableCaptionElement": false,
            "HTMLTableCellElement": false,
            "HTMLTableColElement": false,
            "HTMLTableElement": false,
            "HTMLTableRowElement": false,
            "HTMLTableSectionElement": false,
            "HTMLTemplateElement": false,
            "HTMLTextAreaElement": false,
            "HTMLTimeElement": false,
            "HTMLTitleElement": false,
            "HTMLTrackElement": false,
            "HTMLUListElement": false,
            "HTMLUnknownElement": false,
            "HTMLVideoElement": false,
            "IDBCursor": false,
            "IDBCursorWithValue": false,
            "IDBDatabase": false,
            "IDBFactory": false,
            "IDBIndex": false,
            "IDBKeyRange": false,
            "IDBObjectStore": false,
            "IDBOpenDBRequest": false,
            "IDBRequest": false,
            "IDBTransaction": false,
            "IDBVersionChangeEvent": false,
            "IdleDeadline": false,
            "IIRFilterNode": false,
            "Image": false,
            "ImageBitmap": false,
            "ImageBitmapRenderingContext": false,
            "ImageCapture": false,
            "ImageData": false,
            "indexedDB": false,
            "innerHeight": false,
            "innerWidth": false,
            "InputEvent": false,
            "IntersectionObserver": false,
            "IntersectionObserverEntry": false,
            "Intl": false,
            "isSecureContext": false,
            "KeyboardEvent": false,
            "KeyframeEffect": false,
            "KeyframeEffectReadOnly": false,
            "length": false,
            "localStorage": false,
            "location": true,
            "Location": false,
            "locationbar": false,
            "matchMedia": false,
            "MediaDeviceInfo": false,
            "MediaDevices": false,
            "MediaElementAudioSourceNode": false,
            "MediaEncryptedEvent": false,
            "MediaError": false,
            "MediaKeyMessageEvent": false,
            "MediaKeySession": false,
            "MediaKeyStatusMap": false,
            "MediaKeySystemAccess": false,
            "MediaList": false,
            "MediaQueryList": false,
            "MediaQueryListEvent": false,
            "MediaRecorder": false,
            "MediaSettingsRange": false,
            "MediaSource": false,
            "MediaStream": false,
            "MediaStreamAudioDestinationNode": false,
            "MediaStreamAudioSourceNode": false,
            "MediaStreamEvent": false,
            "MediaStreamTrack": false,
            "MediaStreamTrackEvent": false,
            "menubar": false,
            "MessageChannel": false,
            "MessageEvent": false,
            "MessagePort": false,
            "MIDIAccess": false,
            "MIDIConnectionEvent": false,
            "MIDIInput": false,
            "MIDIInputMap": false,
            "MIDIMessageEvent": false,
            "MIDIOutput": false,
            "MIDIOutputMap": false,
            "MIDIPort": false,
            "MimeType": false,
            "MimeTypeArray": false,
            "MouseEvent": false,
            "moveBy": false,
            "moveTo": false,
            "MutationEvent": false,
            "MutationObserver": false,
            "MutationRecord": false,
            "name": false,
            "NamedNodeMap": false,
            "NavigationPreloadManager": false,
            "navigator": false,
            "Navigator": false,
            "NetworkInformation": false,
            "Node": false,
            "NodeFilter": false,
            "NodeIterator": false,
            "NodeList": false,
            "Notification": false,
            "OfflineAudioCompletionEvent": false,
            "OfflineAudioContext": false,
            "offscreenBuffering": false,
            "OffscreenCanvas": true,
            "onabort": true,
            "onafterprint": true,
            "onanimationend": true,
            "onanimationiteration": true,
            "onanimationstart": true,
            "onappinstalled": true,
            "onauxclick": true,
            "onbeforeinstallprompt": true,
            "onbeforeprint": true,
            "onbeforeunload": true,
            "onblur": true,
            "oncancel": true,
            "oncanplay": true,
            "oncanplaythrough": true,
            "onchange": true,
            "onclick": true,
            "onclose": true,
            "oncontextmenu": true,
            "oncuechange": true,
            "ondblclick": true,
            "ondevicemotion": true,
            "ondeviceorientation": true,
            "ondeviceorientationabsolute": true,
            "ondrag": true,
            "ondragend": true,
            "ondragenter": true,
            "ondragleave": true,
            "ondragover": true,
            "ondragstart": true,
            "ondrop": true,
            "ondurationchange": true,
            "onemptied": true,
            "onended": true,
            "onerror": true,
            "onfocus": true,
            "ongotpointercapture": true,
            "onhashchange": true,
            "oninput": true,
            "oninvalid": true,
            "onkeydown": true,
            "onkeypress": true,
            "onkeyup": true,
            "onlanguagechange": true,
            "onload": true,
            "onloadeddata": true,
            "onloadedmetadata": true,
            "onloadstart": true,
            "onlostpointercapture": true,
            "onmessage": true,
            "onmessageerror": true,
            "onmousedown": true,
            "onmouseenter": true,
            "onmouseleave": true,
            "onmousemove": true,
            "onmouseout": true,
            "onmouseover": true,
            "onmouseup": true,
            "onmousewheel": true,
            "onoffline": true,
            "ononline": true,
            "onpagehide": true,
            "onpageshow": true,
            "onpause": true,
            "onplay": true,
            "onplaying": true,
            "onpointercancel": true,
            "onpointerdown": true,
            "onpointerenter": true,
            "onpointerleave": true,
            "onpointermove": true,
            "onpointerout": true,
            "onpointerover": true,
            "onpointerup": true,
            "onpopstate": true,
            "onprogress": true,
            "onratechange": true,
            "onrejectionhandled": true,
            "onreset": true,
            "onresize": true,
            "onscroll": true,
            "onsearch": true,
            "onseeked": true,
            "onseeking": true,
            "onselect": true,
            "onstalled": true,
            "onstorage": true,
            "onsubmit": true,
            "onsuspend": true,
            "ontimeupdate": true,
            "ontoggle": true,
            "ontransitionend": true,
            "onunhandledrejection": true,
            "onunload": true,
            "onvolumechange": true,
            "onwaiting": true,
            "onwheel": true,
            "open": false,
            "openDatabase": false,
            "opener": false,
            "Option": false,
            "origin": false,
            "OscillatorNode": false,
            "outerHeight": false,
            "outerWidth": false,
            "PageTransitionEvent": false,
            "pageXOffset": false,
            "pageYOffset": false,
            "PannerNode": false,
            "parent": false,
            "Path2D": false,
            "PaymentAddress": false,
            "PaymentRequest": false,
            "PaymentRequestUpdateEvent": false,
            "PaymentResponse": false,
            "performance": false,
            "Performance": false,
            "PerformanceEntry": false,
            "PerformanceLongTaskTiming": false,
            "PerformanceMark": false,
            "PerformanceMeasure": false,
            "PerformanceNavigation": false,
            "PerformanceNavigationTiming": false,
            "PerformanceObserver": false,
            "PerformanceObserverEntryList": false,
            "PerformancePaintTiming": false,
            "PerformanceResourceTiming": false,
            "PerformanceTiming": false,
            "PeriodicWave": false,
            "Permissions": false,
            "PermissionStatus": false,
            "personalbar": false,
            "PhotoCapabilities": false,
            "Plugin": false,
            "PluginArray": false,
            "PointerEvent": false,
            "PopStateEvent": false,
            "postMessage": false,
            "Presentation": false,
            "PresentationAvailability": false,
            "PresentationConnection": false,
            "PresentationConnectionAvailableEvent": false,
            "PresentationConnectionCloseEvent": false,
            "PresentationConnectionList": false,
            "PresentationReceiver": false,
            "PresentationRequest": false,
            "print": false,
            "ProcessingInstruction": false,
            "ProgressEvent": false,
            "PromiseRejectionEvent": false,
            "prompt": false,
            "PushManager": false,
            "PushSubscription": false,
            "PushSubscriptionOptions": false,
            "queueMicrotask": false,
            "RadioNodeList": false,
            "Range": false,
            "ReadableStream": false,
            "registerProcessor": false,
            "RemotePlayback": false,
            "removeEventListener": false,
            "Request": false,
            "requestAnimationFrame": false,
            "requestIdleCallback": false,
            "resizeBy": false,
            "ResizeObserver": false,
            "ResizeObserverEntry": false,
            "resizeTo": false,
            "Response": false,
            "RTCCertificate": false,
            "RTCDataChannel": false,
            "RTCDataChannelEvent": false,
            "RTCDtlsTransport": false,
            "RTCIceCandidate": false,
            "RTCIceGatherer": false,
            "RTCIceTransport": false,
            "RTCPeerConnection": false,
            "RTCPeerConnectionIceEvent": false,
            "RTCRtpContributingSource": false,
            "RTCRtpReceiver": false,
            "RTCRtpSender": false,
            "RTCSctpTransport": false,
            "RTCSessionDescription": false,
            "RTCStatsReport": false,
            "RTCTrackEvent": false,
            "screen": false,
            "Screen": false,
            "screenLeft": false,
            "ScreenOrientation": false,
            "screenTop": false,
            "screenX": false,
            "screenY": false,
            "ScriptProcessorNode": false,
            "scroll": false,
            "scrollbars": false,
            "scrollBy": false,
            "scrollTo": false,
            "scrollX": false,
            "scrollY": false,
            "SecurityPolicyViolationEvent": false,
            "Selection": false,
            "self": false,
            "ServiceWorker": false,
            "ServiceWorkerContainer": false,
            "ServiceWorkerRegistration": false,
            "sessionStorage": false,
            "setInterval": false,
            "setTimeout": false,
            "ShadowRoot": false,
            "SharedWorker": false,
            "SourceBuffer": false,
            "SourceBufferList": false,
            "speechSynthesis": false,
            "SpeechSynthesisEvent": false,
            "SpeechSynthesisUtterance": false,
            "StaticRange": false,
            "status": false,
            "statusbar": false,
            "StereoPannerNode": false,
            "stop": false,
            "Storage": false,
            "StorageEvent": false,
            "StorageManager": false,
            "styleMedia": false,
            "StyleSheet": false,
            "StyleSheetList": false,
            "SubtleCrypto": false,
            "SVGAElement": false,
            "SVGAngle": false,
            "SVGAnimatedAngle": false,
            "SVGAnimatedBoolean": false,
            "SVGAnimatedEnumeration": false,
            "SVGAnimatedInteger": false,
            "SVGAnimatedLength": false,
            "SVGAnimatedLengthList": false,
            "SVGAnimatedNumber": false,
            "SVGAnimatedNumberList": false,
            "SVGAnimatedPreserveAspectRatio": false,
            "SVGAnimatedRect": false,
            "SVGAnimatedString": false,
            "SVGAnimatedTransformList": false,
            "SVGAnimateElement": false,
            "SVGAnimateMotionElement": false,
            "SVGAnimateTransformElement": false,
            "SVGAnimationElement": false,
            "SVGCircleElement": false,
            "SVGClipPathElement": false,
            "SVGComponentTransferFunctionElement": false,
            "SVGDefsElement": false,
            "SVGDescElement": false,
            "SVGDiscardElement": false,
            "SVGElement": false,
            "SVGEllipseElement": false,
            "SVGFEBlendElement": false,
            "SVGFEColorMatrixElement": false,
            "SVGFEComponentTransferElement": false,
            "SVGFECompositeElement": false,
            "SVGFEConvolveMatrixElement": false,
            "SVGFEDiffuseLightingElement": false,
            "SVGFEDisplacementMapElement": false,
            "SVGFEDistantLightElement": false,
            "SVGFEDropShadowElement": false,
            "SVGFEFloodElement": false,
            "SVGFEFuncAElement": false,
            "SVGFEFuncBElement": false,
            "SVGFEFuncGElement": false,
            "SVGFEFuncRElement": false,
            "SVGFEGaussianBlurElement": false,
            "SVGFEImageElement": false,
            "SVGFEMergeElement": false,
            "SVGFEMergeNodeElement": false,
            "SVGFEMorphologyElement": false,
            "SVGFEOffsetElement": false,
            "SVGFEPointLightElement": false,
            "SVGFESpecularLightingElement": false,
            "SVGFESpotLightElement": false,
            "SVGFETileElement": false,
            "SVGFETurbulenceElement": false,
            "SVGFilterElement": false,
            "SVGForeignObjectElement": false,
            "SVGGElement": false,
            "SVGGeometryElement": false,
            "SVGGradientElement": false,
            "SVGGraphicsElement": false,
            "SVGImageElement": false,
            "SVGLength": false,
            "SVGLengthList": false,
            "SVGLinearGradientElement": false,
            "SVGLineElement": false,
            "SVGMarkerElement": false,
            "SVGMaskElement": false,
            "SVGMatrix": false,
            "SVGMetadataElement": false,
            "SVGMPathElement": false,
            "SVGNumber": false,
            "SVGNumberList": false,
            "SVGPathElement": false,
            "SVGPatternElement": false,
            "SVGPoint": false,
            "SVGPointList": false,
            "SVGPolygonElement": false,
            "SVGPolylineElement": false,
            "SVGPreserveAspectRatio": false,
            "SVGRadialGradientElement": false,
            "SVGRect": false,
            "SVGRectElement": false,
            "SVGScriptElement": false,
            "SVGSetElement": false,
            "SVGStopElement": false,
            "SVGStringList": false,
            "SVGStyleElement": false,
            "SVGSVGElement": false,
            "SVGSwitchElement": false,
            "SVGSymbolElement": false,
            "SVGTextContentElement": false,
            "SVGTextElement": false,
            "SVGTextPathElement": false,
            "SVGTextPositioningElement": false,
            "SVGTitleElement": false,
            "SVGTransform": false,
            "SVGTransformList": false,
            "SVGTSpanElement": false,
            "SVGUnitTypes": false,
            "SVGUseElement": false,
            "SVGViewElement": false,
            "TaskAttributionTiming": false,
            "Text": false,
            "TextDecoder": false,
            "TextEncoder": false,
            "TextEvent": false,
            "TextMetrics": false,
            "TextTrack": false,
            "TextTrackCue": false,
            "TextTrackCueList": false,
            "TextTrackList": false,
            "TimeRanges": false,
            "toolbar": false,
            "top": false,
            "Touch": false,
            "TouchEvent": false,
            "TouchList": false,
            "TrackEvent": false,
            "TransitionEvent": false,
            "TreeWalker": false,
            "UIEvent": false,
            "URL": false,
            "URLSearchParams": false,
            "ValidityState": false,
            "visualViewport": false,
            "VisualViewport": false,
            "VTTCue": false,
            "WaveShaperNode": false,
            "WebAssembly": false,
            "WebGL2RenderingContext": false,
            "WebGLActiveInfo": false,
            "WebGLBuffer": false,
            "WebGLContextEvent": false,
            "WebGLFramebuffer": false,
            "WebGLProgram": false,
            "WebGLQuery": false,
            "WebGLRenderbuffer": false,
            "WebGLRenderingContext": false,
            "WebGLSampler": false,
            "WebGLShader": false,
            "WebGLShaderPrecisionFormat": false,
            "WebGLSync": false,
            "WebGLTexture": false,
            "WebGLTransformFeedback": false,
            "WebGLUniformLocation": false,
            "WebGLVertexArrayObject": false,
            "WebSocket": false,
            "WheelEvent": false,
            "window": false,
            "Window": false,
            "Worker": false,
            "WritableStream": false,
            "XMLDocument": false,
            "XMLHttpRequest": false,
            "XMLHttpRequestEventTarget": false,
            "XMLHttpRequestUpload": false,
            "XMLSerializer": false,
            "XPathEvaluator": false,
            "XPathExpression": false,
            "XPathResult": false,
            "XSLTProcessor": false
        },
        "worker": {
            "addEventListener": false,
            "applicationCache": false,
            "atob": false,
            "Blob": false,
            "BroadcastChannel": false,
            "btoa": false,
            "Cache": false,
            "caches": false,
            "clearInterval": false,
            "clearTimeout": false,
            "close": true,
            "console": false,
            "fetch": false,
            "FileReaderSync": false,
            "FormData": false,
            "Headers": false,
            "IDBCursor": false,
            "IDBCursorWithValue": false,
            "IDBDatabase": false,
            "IDBFactory": false,
            "IDBIndex": false,
            "IDBKeyRange": false,
            "IDBObjectStore": false,
            "IDBOpenDBRequest": false,
            "IDBRequest": false,
            "IDBTransaction": false,
            "IDBVersionChangeEvent": false,
            "ImageData": false,
            "importScripts": true,
            "indexedDB": false,
            "location": false,
            "MessageChannel": false,
            "MessagePort": false,
            "name": false,
            "navigator": false,
            "Notification": false,
            "onclose": true,
            "onconnect": true,
            "onerror": true,
            "onlanguagechange": true,
            "onmessage": true,
            "onoffline": true,
            "ononline": true,
            "onrejectionhandled": true,
            "onunhandledrejection": true,
            "performance": false,
            "Performance": false,
            "PerformanceEntry": false,
            "PerformanceMark": false,
            "PerformanceMeasure": false,
            "PerformanceNavigation": false,
            "PerformanceResourceTiming": false,
            "PerformanceTiming": false,
            "postMessage": true,
            "Promise": false,
            "queueMicrotask": false,
            "removeEventListener": false,
            "Request": false,
            "Response": false,
            "self": true,
            "ServiceWorkerRegistration": false,
            "setInterval": false,
            "setTimeout": false,
            "TextDecoder": false,
            "TextEncoder": false,
            "URL": false,
            "URLSearchParams": false,
            "WebSocket": false,
            "Worker": false,
            "WorkerGlobalScope": false,
            "XMLHttpRequest": false
        },
        "node": {
            "__dirname": false,
            "__filename": false,
            "Buffer": false,
            "clearImmediate": false,
            "clearInterval": false,
            "clearTimeout": false,
            "console": false,
            "exports": true,
            "global": false,
            "Intl": false,
            "module": false,
            "process": false,
            "queueMicrotask": false,
            "require": false,
            "setImmediate": false,
            "setInterval": false,
            "setTimeout": false,
            "TextDecoder": false,
            "TextEncoder": false,
            "URL": false,
            "URLSearchParams": false
        },
        "commonjs": {
            "exports": true,
            "global": false,
            "module": false,
            "require": false
        },
        "amd": {
            "define": false,
            "require": false
        },
        "mocha": {
            "after": false,
            "afterEach": false,
            "before": false,
            "beforeEach": false,
            "context": false,
            "describe": false,
            "it": false,
            "mocha": false,
            "run": false,
            "setup": false,
            "specify": false,
            "suite": false,
            "suiteSetup": false,
            "suiteTeardown": false,
            "teardown": false,
            "test": false,
            "xcontext": false,
            "xdescribe": false,
            "xit": false,
            "xspecify": false
        },
        "jasmine": {
            "afterAll": false,
            "afterEach": false,
            "beforeAll": false,
            "beforeEach": false,
            "describe": false,
            "expect": false,
            "expectAsync": false,
            "fail": false,
            "fdescribe": false,
            "fit": false,
            "it": false,
            "jasmine": false,
            "pending": false,
            "runs": false,
            "spyOn": false,
            "spyOnAllFunctions": false,
            "spyOnProperty": false,
            "waits": false,
            "waitsFor": false,
            "xdescribe": false,
            "xit": false
        },
        "jest": {
            "afterAll": false,
            "afterEach": false,
            "beforeAll": false,
            "beforeEach": false,
            "describe": false,
            "expect": false,
            "fdescribe": false,
            "fit": false,
            "it": false,
            "jest": false,
            "pit": false,
            "require": false,
            "test": false,
            "xdescribe": false,
            "xit": false,
            "xtest": false
        },
        "qunit": {
            "asyncTest": false,
            "deepEqual": false,
            "equal": false,
            "expect": false,
            "module": false,
            "notDeepEqual": false,
            "notEqual": false,
            "notOk": false,
            "notPropEqual": false,
            "notStrictEqual": false,
            "ok": false,
            "propEqual": false,
            "QUnit": false,
            "raises": false,
            "start": false,
            "stop": false,
            "strictEqual": false,
            "test": false,
            "throws": false
        },
        "phantomjs": {
            "console": true,
            "exports": true,
            "phantom": true,
            "require": true,
            "WebPage": true
        },
        "couch": {
            "emit": false,
            "exports": false,
            "getRow": false,
            "log": false,
            "module": false,
            "provides": false,
            "require": false,
            "respond": false,
            "send": false,
            "start": false,
            "sum": false
        },
        "rhino": {
            "defineClass": false,
            "deserialize": false,
            "gc": false,
            "help": false,
            "importClass": false,
            "importPackage": false,
            "java": false,
            "load": false,
            "loadClass": false,
            "Packages": false,
            "print": false,
            "quit": false,
            "readFile": false,
            "readUrl": false,
            "runCommand": false,
            "seal": false,
            "serialize": false,
            "spawn": false,
            "sync": false,
            "toint32": false,
            "version": false
        },
        "nashorn": {
            "__DIR__": false,
            "__FILE__": false,
            "__LINE__": false,
            "com": false,
            "edu": false,
            "exit": false,
            "java": false,
            "Java": false,
            "javafx": false,
            "JavaImporter": false,
            "javax": false,
            "JSAdapter": false,
            "load": false,
            "loadWithNewGlobal": false,
            "org": false,
            "Packages": false,
            "print": false,
            "quit": false
        },
        "wsh": {
            "ActiveXObject": true,
            "CollectGarbage": true,
            "Debug": true,
            "Enumerator": true,
            "GetObject": true,
            "RuntimeObject": true,
            "ScriptEngine": true,
            "ScriptEngineBuildVersion": true,
            "ScriptEngineMajorVersion": true,
            "ScriptEngineMinorVersion": true,
            "VBArray": true,
            "WScript": true,
            "WSH": true,
            "XDomainRequest": true
        },
        "jquery": {
            "$": false,
            "jQuery": false
        },
        "yui": {
            "YAHOO": false,
            "YAHOO_config": false,
            "YUI": false,
            "YUI_config": false
        },
        "shelljs": {
            "cat": false,
            "cd": false,
            "chmod": false,
            "config": false,
            "cp": false,
            "dirs": false,
            "echo": false,
            "env": false,
            "error": false,
            "exec": false,
            "exit": false,
            "find": false,
            "grep": false,
            "ln": false,
            "ls": false,
            "mkdir": false,
            "mv": false,
            "popd": false,
            "pushd": false,
            "pwd": false,
            "rm": false,
            "sed": false,
            "set": false,
            "target": false,
            "tempdir": false,
            "test": false,
            "touch": false,
            "which": false
        },
        "prototypejs": {
            "$": false,
            "$$": false,
            "$A": false,
            "$break": false,
            "$continue": false,
            "$F": false,
            "$H": false,
            "$R": false,
            "$w": false,
            "Abstract": false,
            "Ajax": false,
            "Autocompleter": false,
            "Builder": false,
            "Class": false,
            "Control": false,
            "Draggable": false,
            "Draggables": false,
            "Droppables": false,
            "Effect": false,
            "Element": false,
            "Enumerable": false,
            "Event": false,
            "Field": false,
            "Form": false,
            "Hash": false,
            "Insertion": false,
            "ObjectRange": false,
            "PeriodicalExecuter": false,
            "Position": false,
            "Prototype": false,
            "Scriptaculous": false,
            "Selector": false,
            "Sortable": false,
            "SortableObserver": false,
            "Sound": false,
            "Template": false,
            "Toggle": false,
            "Try": false
        },
        "meteor": {
            "_": false,
            "$": false,
            "Accounts": false,
            "AccountsClient": false,
            "AccountsCommon": false,
            "AccountsServer": false,
            "App": false,
            "Assets": false,
            "Blaze": false,
            "check": false,
            "Cordova": false,
            "DDP": false,
            "DDPRateLimiter": false,
            "DDPServer": false,
            "Deps": false,
            "EJSON": false,
            "Email": false,
            "HTTP": false,
            "Log": false,
            "Match": false,
            "Meteor": false,
            "Mongo": false,
            "MongoInternals": false,
            "Npm": false,
            "Package": false,
            "Plugin": false,
            "process": false,
            "Random": false,
            "ReactiveDict": false,
            "ReactiveVar": false,
            "Router": false,
            "ServiceConfiguration": false,
            "Session": false,
            "share": false,
            "Spacebars": false,
            "Template": false,
            "Tinytest": false,
            "Tracker": false,
            "UI": false,
            "Utils": false,
            "WebApp": false,
            "WebAppInternals": false
        },
        "mongo": {
            "_isWindows": false,
            "_rand": false,
            "BulkWriteResult": false,
            "cat": false,
            "cd": false,
            "connect": false,
            "db": false,
            "getHostName": false,
            "getMemInfo": false,
            "hostname": false,
            "ISODate": false,
            "listFiles": false,
            "load": false,
            "ls": false,
            "md5sumFile": false,
            "mkdir": false,
            "Mongo": false,
            "NumberInt": false,
            "NumberLong": false,
            "ObjectId": false,
            "PlanCache": false,
            "print": false,
            "printjson": false,
            "pwd": false,
            "quit": false,
            "removeFile": false,
            "rs": false,
            "sh": false,
            "UUID": false,
            "version": false,
            "WriteResult": false
        },
        "applescript": {
            "$": false,
            "Application": false,
            "Automation": false,
            "console": false,
            "delay": false,
            "Library": false,
            "ObjC": false,
            "ObjectSpecifier": false,
            "Path": false,
            "Progress": false,
            "Ref": false
        },
        "serviceworker": {
            "addEventListener": false,
            "applicationCache": false,
            "atob": false,
            "Blob": false,
            "BroadcastChannel": false,
            "btoa": false,
            "Cache": false,
            "caches": false,
            "CacheStorage": false,
            "clearInterval": false,
            "clearTimeout": false,
            "Client": false,
            "clients": false,
            "Clients": false,
            "close": true,
            "console": false,
            "ExtendableEvent": false,
            "ExtendableMessageEvent": false,
            "fetch": false,
            "FetchEvent": false,
            "FileReaderSync": false,
            "FormData": false,
            "Headers": false,
            "IDBCursor": false,
            "IDBCursorWithValue": false,
            "IDBDatabase": false,
            "IDBFactory": false,
            "IDBIndex": false,
            "IDBKeyRange": false,
            "IDBObjectStore": false,
            "IDBOpenDBRequest": false,
            "IDBRequest": false,
            "IDBTransaction": false,
            "IDBVersionChangeEvent": false,
            "ImageData": false,
            "importScripts": false,
            "indexedDB": false,
            "location": false,
            "MessageChannel": false,
            "MessagePort": false,
            "name": false,
            "navigator": false,
            "Notification": false,
            "onclose": true,
            "onconnect": true,
            "onerror": true,
            "onfetch": true,
            "oninstall": true,
            "onlanguagechange": true,
            "onmessage": true,
            "onmessageerror": true,
            "onnotificationclick": true,
            "onnotificationclose": true,
            "onoffline": true,
            "ononline": true,
            "onpush": true,
            "onpushsubscriptionchange": true,
            "onrejectionhandled": true,
            "onsync": true,
            "onunhandledrejection": true,
            "performance": false,
            "Performance": false,
            "PerformanceEntry": false,
            "PerformanceMark": false,
            "PerformanceMeasure": false,
            "PerformanceNavigation": false,
            "PerformanceResourceTiming": false,
            "PerformanceTiming": false,
            "postMessage": true,
            "Promise": false,
            "queueMicrotask": false,
            "registration": false,
            "removeEventListener": false,
            "Request": false,
            "Response": false,
            "self": false,
            "ServiceWorker": false,
            "ServiceWorkerContainer": false,
            "ServiceWorkerGlobalScope": false,
            "ServiceWorkerMessageEvent": false,
            "ServiceWorkerRegistration": false,
            "setInterval": false,
            "setTimeout": false,
            "skipWaiting": false,
            "TextDecoder": false,
            "TextEncoder": false,
            "URL": false,
            "URLSearchParams": false,
            "WebSocket": false,
            "WindowClient": false,
            "Worker": false,
            "WorkerGlobalScope": false,
            "XMLHttpRequest": false
        },
        "atomtest": {
            "advanceClock": false,
            "fakeClearInterval": false,
            "fakeClearTimeout": false,
            "fakeSetInterval": false,
            "fakeSetTimeout": false,
            "resetTimeouts": false,
            "waitsForPromise": false
        },
        "embertest": {
            "andThen": false,
            "click": false,
            "currentPath": false,
            "currentRouteName": false,
            "currentURL": false,
            "fillIn": false,
            "find": false,
            "findAll": false,
            "findWithAssert": false,
            "keyEvent": false,
            "pauseTest": false,
            "resumeTest": false,
            "triggerEvent": false,
            "visit": false,
            "wait": false
        },
        "protractor": {
            "$": false,
            "$$": false,
            "browser": false,
            "by": false,
            "By": false,
            "DartObject": false,
            "element": false,
            "protractor": false
        },
        "shared-node-browser": {
            "clearInterval": false,
            "clearTimeout": false,
            "console": false,
            "setInterval": false,
            "setTimeout": false,
            "URL": false,
            "URLSearchParams": false
        },
        "webextensions": {
            "browser": false,
            "chrome": false,
            "opr": false
        },
        "greasemonkey": {
            "cloneInto": false,
            "createObjectIn": false,
            "exportFunction": false,
            "GM": false,
            "GM_addStyle": false,
            "GM_deleteValue": false,
            "GM_getResourceText": false,
            "GM_getResourceURL": false,
            "GM_getValue": false,
            "GM_info": false,
            "GM_listValues": false,
            "GM_log": false,
            "GM_openInTab": false,
            "GM_registerMenuCommand": false,
            "GM_setClipboard": false,
            "GM_setValue": false,
            "GM_xmlhttpRequest": false,
            "unsafeWindow": false
        },
        "devtools": {
            "$": false,
            "$_": false,
            "$$": false,
            "$0": false,
            "$1": false,
            "$2": false,
            "$3": false,
            "$4": false,
            "$x": false,
            "chrome": false,
            "clear": false,
            "copy": false,
            "debug": false,
            "dir": false,
            "dirxml": false,
            "getEventListeners": false,
            "inspect": false,
            "keys": false,
            "monitor": false,
            "monitorEvents": false,
            "profile": false,
            "profileEnd": false,
            "queryObjects": false,
            "table": false,
            "undebug": false,
            "unmonitor": false,
            "unmonitorEvents": false,
            "values": false
        }
    };
},
/* eslint/lib/shared/ast-utils.js */ function _(require, module, exports) {
    const breakableTypePattern = /^(?:(?:Do)?While|For(?:In|Of)?|Switch)Statement$/u;
    const lineBreakPattern = /\r\n|[\r\n\u2028\u2029]/u;
    const shebangPattern = /^#!([^\r\n]+)/u;
    /**
     * Creates a version of the `lineBreakPattern` regex with the global flag.
     * Global regexes are mutable, so this needs to be a function instead of a constant.
     * @returns {RegExp} A global regular expression that matches line terminators
     */
    function createGlobalLinebreakMatcher() {
        return new RegExp(lineBreakPattern.source, "gu");
    }
    module.exports = {
        breakableTypePattern,
        lineBreakPattern,
        createGlobalLinebreakMatcher,
        shebangPattern
    };
},
/* eslint/lib/shared/config-validator.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const path = require("path"), util = require("util"), lodash = require(111) /* lodash */, configSchema = require(117) /* ../../conf/config-schema */, BuiltInEnvironments = require(112) /* ../../conf/environments */, BuiltInRules = require(118) /* ../rules */, ConfigOps = require(82) /* ./config-ops */;
    const ajv = require(423) /* ./ajv */();
    const ruleValidators = new WeakMap();
    const noop = Function.prototype;
    //------------------------------------------------------------------------------
    // Private
    //------------------------------------------------------------------------------
    let validateSchema;
    // Defitions for deprecation warnings.
    const deprecationWarningMessages = {
        ESLINT_LEGACY_ECMAFEATURES: "The 'ecmaFeatures' config file property is deprecated, and has no effect."
    };
    const severityMap = {
        error: 2,
        warn: 1,
        off: 0
    };
    /**
     * Gets a complete options schema for a rule.
     * @param {{create: Function, schema: (Array|null)}} rule A new-style rule object
     * @returns {Object} JSON Schema for the rule's options.
     */
    function getRuleOptionsSchema(rule) {
        if (!rule) {
            return null;
        }
        const schema = rule.schema || rule.meta && rule.meta.schema;
        // Given a tuple of schemas, insert warning level at the beginning
        if (Array.isArray(schema)) {
            if (schema.length) {
                return {
                    type: "array",
                    items: schema,
                    minItems: 0,
                    maxItems: schema.length
                };
            }
            return {
                type: "array",
                minItems: 0,
                maxItems: 0
            };
        }
        // Given a full schema, leave it alone
        return schema || null;
    }
    /**
     * Validates a rule's severity and returns the severity value. Throws an error if the severity is invalid.
     * @param {options} options The given options for the rule.
     * @returns {number|string} The rule's severity value
     */
    function validateRuleSeverity(options) {
        const severity = Array.isArray(options) ? options[0] : options;
        const normSeverity = typeof severity === "string" ? severityMap[severity.toLowerCase()] : severity;
        if (normSeverity === 0 || normSeverity === 1 || normSeverity === 2) {
            return normSeverity;
        }
        throw new Error(`\tSeverity should be one of the following: 0 = off, 1 = warn, 2 = error (you passed '${util.inspect(severity).replace(/'/gu, "\"").replace(/\n/gu, "")}').\n`);
    }
    /**
     * Validates the non-severity options passed to a rule, based on its schema.
     * @param {{create: Function}} rule The rule to validate
     * @param {Array} localOptions The options for the rule, excluding severity
     * @returns {void}
     */
    function validateRuleSchema(rule, localOptions) {
        if (!ruleValidators.has(rule)) {
            const schema = getRuleOptionsSchema(rule);
            if (schema) {
                ruleValidators.set(rule, ajv.compile(schema));
            }
        }
        const validateRule = ruleValidators.get(rule);
        if (validateRule) {
            validateRule(localOptions);
            if (validateRule.errors) {
                throw new Error(validateRule.errors.map(error => `\tValue ${JSON.stringify(error.data)} ${error.message}.\n`).join(""));
            }
        }
    }
    /**
     * Validates a rule's options against its schema.
     * @param {{create: Function}|null} rule The rule that the config is being validated for
     * @param {string} ruleId The rule's unique name.
     * @param {Array|number} options The given options for the rule.
     * @param {string|null} source The name of the configuration source to report in any errors. If null or undefined,
     * no source is prepended to the message.
     * @returns {void}
     */
    function validateRuleOptions(rule, ruleId, options, source = null) {
        try {
            const severity = validateRuleSeverity(options);
            if (severity !== 0) {
                validateRuleSchema(rule, Array.isArray(options) ? options.slice(1) : []);
            }
        }
        catch (err) {
            const enhancedMessage = `Configuration for rule "${ruleId}" is invalid:\n${err.message}`;
            if (typeof source === "string") {
                throw new Error(`${source}:\n\t${enhancedMessage}`);
            }
            else {
                throw new Error(enhancedMessage);
            }
        }
    }
    /**
     * Validates an environment object
     * @param {Object} environment The environment config object to validate.
     * @param {string} source The name of the configuration source to report in any errors.
     * @param {function(envId:string): Object} [getAdditionalEnv] A map from strings to loaded environments.
     * @returns {void}
     */
    function validateEnvironment(environment, source, getAdditionalEnv = noop) {
        // not having an environment is ok
        if (!environment) {
            return;
        }
        Object.keys(environment).forEach(id => {
            const env = getAdditionalEnv(id) || BuiltInEnvironments.get(id) || null;
            if (!env) {
                const message = `${source}:\n\tEnvironment key "${id}" is unknown\n`;
                throw new Error(message);
            }
        });
    }
    /**
     * Validates a rules config object
     * @param {Object} rulesConfig The rules config object to validate.
     * @param {string} source The name of the configuration source to report in any errors.
     * @param {function(ruleId:string): Object} getAdditionalRule A map from strings to loaded rules
     * @returns {void}
     */
    function validateRules(rulesConfig, source, getAdditionalRule = noop) {
        if (!rulesConfig) {
            return;
        }
        Object.keys(rulesConfig).forEach(id => {
            const rule = getAdditionalRule(id) || BuiltInRules.get(id) || null;
            validateRuleOptions(rule, id, rulesConfig[id], source);
        });
    }
    /**
     * Validates a `globals` section of a config file
     * @param {Object} globalsConfig The `glboals` section
     * @param {string|null} source The name of the configuration source to report in the event of an error.
     * @returns {void}
     */
    function validateGlobals(globalsConfig, source = null) {
        if (!globalsConfig) {
            return;
        }
        Object.entries(globalsConfig)
            .forEach(([configuredGlobal, configuredValue]) => {
            try {
                ConfigOps.normalizeConfigGlobal(configuredValue);
            }
            catch (err) {
                throw new Error(`ESLint configuration of global '${configuredGlobal}' in ${source} is invalid:\n${err.message}`);
            }
        });
    }
    /**
     * Validate `processor` configuration.
     * @param {string|undefined} processorName The processor name.
     * @param {string} source The name of config file.
     * @param {function(id:string): Processor} getProcessor The getter of defined processors.
     * @returns {void}
     */
    function validateProcessor(processorName, source, getProcessor) {
        if (processorName && !getProcessor(processorName)) {
            throw new Error(`ESLint configuration of processor in '${source}' is invalid: '${processorName}' was not found.`);
        }
    }
    /**
     * Formats an array of schema validation errors.
     * @param {Array} errors An array of error messages to format.
     * @returns {string} Formatted error message
     */
    function formatErrors(errors) {
        return errors.map(error => {
            if (error.keyword === "additionalProperties") {
                const formattedPropertyPath = error.dataPath.length ? `${error.dataPath.slice(1)}.${error.params.additionalProperty}` : error.params.additionalProperty;
                return `Unexpected top-level property "${formattedPropertyPath}"`;
            }
            if (error.keyword === "type") {
                const formattedField = error.dataPath.slice(1);
                const formattedExpectedType = Array.isArray(error.schema) ? error.schema.join("/") : error.schema;
                const formattedValue = JSON.stringify(error.data);
                return `Property "${formattedField}" is the wrong type (expected ${formattedExpectedType} but got \`${formattedValue}\`)`;
            }
            const field = error.dataPath[0] === "." ? error.dataPath.slice(1) : error.dataPath;
            return `"${field}" ${error.message}. Value: ${JSON.stringify(error.data)}`;
        }).map(message => `\t- ${message}.\n`).join("");
    }
    /**
     * Emits a deprecation warning containing a given filepath. A new deprecation warning is emitted
     * for each unique file path, but repeated invocations with the same file path have no effect.
     * No warnings are emitted if the `--no-deprecation` or `--no-warnings` Node runtime flags are active.
     * @param {string} source The name of the configuration source to report the warning for.
     * @param {string} errorCode The warning message to show.
     * @returns {void}
     */
    const emitDeprecationWarning = lodash.memoize((source, errorCode) => {
        const rel = path.relative(process.cwd(), source);
        const message = deprecationWarningMessages[errorCode];
        process.emitWarning(`${message} (found in "${rel}")`, "DeprecationWarning", errorCode);
    });
    /**
     * Validates the top level properties of the config object.
     * @param {Object} config The config object to validate.
     * @param {string} source The name of the configuration source to report in any errors.
     * @returns {void}
     */
    function validateConfigSchema(config, source = null) {
        validateSchema = validateSchema || ajv.compile(configSchema);
        if (!validateSchema(config)) {
            throw new Error(`ESLint configuration in ${source} is invalid:\n${formatErrors(validateSchema.errors)}`);
        }
        if (Object.hasOwnProperty.call(config, "ecmaFeatures")) {
            emitDeprecationWarning(source, "ESLINT_LEGACY_ECMAFEATURES");
        }
    }
    /**
     * Validates an entire config object.
     * @param {Object} config The config object to validate.
     * @param {string} source The name of the configuration source to report in any errors.
     * @param {function(ruleId:string): Object} [getAdditionalRule] A map from strings to loaded rules.
     * @param {function(envId:string): Object} [getAdditionalEnv] A map from strings to loaded envs.
     * @returns {void}
     */
    function validate(config, source, getAdditionalRule, getAdditionalEnv) {
        validateConfigSchema(config, source);
        validateRules(config.rules, source, getAdditionalRule);
        validateEnvironment(config.env, source, getAdditionalEnv);
        validateGlobals(config.globals, source);
        for (const override of config.overrides || []) {
            validateRules(override.rules, source, getAdditionalRule);
            validateEnvironment(override.env, source, getAdditionalEnv);
            validateGlobals(config.globals, source);
        }
    }
    const validated = new WeakSet();
    /**
     * Validate config array object.
     * @param {ConfigArray} configArray The config array to validate.
     * @returns {void}
     */
    function validateConfigArray(configArray) {
        const getPluginEnv = Map.prototype.get.bind(configArray.pluginEnvironments);
        const getPluginProcessor = Map.prototype.get.bind(configArray.pluginProcessors);
        const getPluginRule = Map.prototype.get.bind(configArray.pluginRules);
        // Validate.
        for (const element of configArray) {
            if (validated.has(element)) {
                continue;
            }
            validated.add(element);
            validateEnvironment(element.env, element.name, getPluginEnv);
            validateGlobals(element.globals, element.name);
            validateProcessor(element.processor, element.name, getPluginProcessor);
            validateRules(element.rules, element.name, getPluginRule);
        }
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    module.exports = {
        getRuleOptionsSchema,
        validate,
        validateConfigArray,
        validateConfigSchema,
        validateRuleOptions
    };
},
/* eslint/conf/config-schema.js */ function _(require, module, exports) {
    const baseConfigProperties = {
        $schema: { type: "string" },
        env: { type: "object" },
        extends: { $ref: "#/definitions/stringOrStrings" },
        globals: { type: "object" },
        overrides: {
            type: "array",
            items: { $ref: "#/definitions/overrideConfig" },
            additionalItems: false
        },
        parser: { type: ["string", "null"] },
        parserOptions: { type: "object" },
        plugins: { type: "array" },
        processor: { type: "string" },
        rules: { type: "object" },
        settings: { type: "object" },
        noInlineConfig: { type: "boolean" },
        reportUnusedDisableDirectives: { type: "boolean" },
        ecmaFeatures: { type: "object" } // deprecated; logs a warning when used
    };
    const configSchema = {
        definitions: {
            stringOrStrings: {
                oneOf: [
                    { type: "string" },
                    {
                        type: "array",
                        items: { type: "string" },
                        additionalItems: false
                    }
                ]
            },
            stringOrStringsRequired: {
                oneOf: [
                    { type: "string" },
                    {
                        type: "array",
                        items: { type: "string" },
                        additionalItems: false,
                        minItems: 1
                    }
                ]
            },
            // Config at top-level.
            objectConfig: {
                type: "object",
                properties: {
                    root: { type: "boolean" },
                    ignorePatterns: { $ref: "#/definitions/stringOrStrings" },
                    ...baseConfigProperties
                },
                additionalProperties: false
            },
            // Config in `overrides`.
            overrideConfig: {
                type: "object",
                properties: {
                    excludedFiles: { $ref: "#/definitions/stringOrStrings" },
                    files: { $ref: "#/definitions/stringOrStringsRequired" },
                    ...baseConfigProperties
                },
                required: ["files"],
                additionalProperties: false
            }
        },
        $ref: "#/definitions/objectConfig"
    };
    module.exports = configSchema;
},
/* eslint/lib/rules/index.js */ function _(require, module, exports) {
    /* eslint sort-keys: ["error", "asc"] */
    const { LazyLoadingRuleMap } = require(119) /* ./utils/lazy-loading-rule-map */;
    /** @type {Map<string, import("../shared/types").Rule>} */
    module.exports = new LazyLoadingRuleMap(Object.entries({
        "accessor-pairs": () => require(125) /* ./accessor-pairs */,
        "array-bracket-newline": () => require(131) /* ./array-bracket-newline */,
        "array-bracket-spacing": () => require(132) /* ./array-bracket-spacing */,
        "array-callback-return": () => require(133) /* ./array-callback-return */,
        "array-element-newline": () => require(134) /* ./array-element-newline */,
        "arrow-body-style": () => require(135) /* ./arrow-body-style */,
        "arrow-parens": () => require(136) /* ./arrow-parens */,
        "arrow-spacing": () => require(137) /* ./arrow-spacing */,
        "block-scoped-var": () => require(138) /* ./block-scoped-var */,
        "block-spacing": () => require(139) /* ./block-spacing */,
        "brace-style": () => require(140) /* ./brace-style */,
        "callback-return": () => require(141) /* ./callback-return */,
        camelcase: () => require(142) /* ./camelcase */,
        "capitalized-comments": () => require(143) /* ./capitalized-comments */,
        "class-methods-use-this": () => require(145) /* ./class-methods-use-this */,
        "comma-dangle": () => require(146) /* ./comma-dangle */,
        "comma-spacing": () => require(147) /* ./comma-spacing */,
        "comma-style": () => require(148) /* ./comma-style */,
        complexity: () => require(149) /* ./complexity */,
        "computed-property-spacing": () => require(150) /* ./computed-property-spacing */,
        "consistent-return": () => require(151) /* ./consistent-return */,
        "consistent-this": () => require(152) /* ./consistent-this */,
        "constructor-super": () => require(153) /* ./constructor-super */,
        curly: () => require(154) /* ./curly */,
        "default-case": () => require(155) /* ./default-case */,
        "default-param-last": () => require(156) /* ./default-param-last */,
        "dot-location": () => require(157) /* ./dot-location */,
        "dot-notation": () => require(158) /* ./dot-notation */,
        "eol-last": () => require(160) /* ./eol-last */,
        eqeqeq: () => require(161) /* ./eqeqeq */,
        "for-direction": () => require(162) /* ./for-direction */,
        "func-call-spacing": () => require(163) /* ./func-call-spacing */,
        "func-name-matching": () => require(164) /* ./func-name-matching */,
        "func-names": () => require(165) /* ./func-names */,
        "func-style": () => require(166) /* ./func-style */,
        "function-call-argument-newline": () => require(167) /* ./function-call-argument-newline */,
        "function-paren-newline": () => require(168) /* ./function-paren-newline */,
        "generator-star-spacing": () => require(169) /* ./generator-star-spacing */,
        "getter-return": () => require(170) /* ./getter-return */,
        "global-require": () => require(171) /* ./global-require */,
        "grouped-accessor-pairs": () => require(172) /* ./grouped-accessor-pairs */,
        "guard-for-in": () => require(173) /* ./guard-for-in */,
        "handle-callback-err": () => require(174) /* ./handle-callback-err */,
        "id-blacklist": () => require(175) /* ./id-blacklist */,
        "id-length": () => require(176) /* ./id-length */,
        "id-match": () => require(177) /* ./id-match */,
        "implicit-arrow-linebreak": () => require(178) /* ./implicit-arrow-linebreak */,
        indent: () => require(179) /* ./indent */,
        "indent-legacy": () => require(181) /* ./indent-legacy */,
        "init-declarations": () => require(182) /* ./init-declarations */,
        "jsx-quotes": () => require(183) /* ./jsx-quotes */,
        "key-spacing": () => require(184) /* ./key-spacing */,
        "keyword-spacing": () => require(185) /* ./keyword-spacing */,
        "line-comment-position": () => require(186) /* ./line-comment-position */,
        "linebreak-style": () => require(187) /* ./linebreak-style */,
        "lines-around-comment": () => require(188) /* ./lines-around-comment */,
        "lines-around-directive": () => require(189) /* ./lines-around-directive */,
        "lines-between-class-members": () => require(190) /* ./lines-between-class-members */,
        "max-classes-per-file": () => require(191) /* ./max-classes-per-file */,
        "max-depth": () => require(192) /* ./max-depth */,
        "max-len": () => require(193) /* ./max-len */,
        "max-lines": () => require(194) /* ./max-lines */,
        "max-lines-per-function": () => require(195) /* ./max-lines-per-function */,
        "max-nested-callbacks": () => require(196) /* ./max-nested-callbacks */,
        "max-params": () => require(197) /* ./max-params */,
        "max-statements": () => require(198) /* ./max-statements */,
        "max-statements-per-line": () => require(199) /* ./max-statements-per-line */,
        "multiline-comment-style": () => require(200) /* ./multiline-comment-style */,
        "multiline-ternary": () => require(201) /* ./multiline-ternary */,
        "new-cap": () => require(202) /* ./new-cap */,
        "new-parens": () => require(203) /* ./new-parens */,
        "newline-after-var": () => require(204) /* ./newline-after-var */,
        "newline-before-return": () => require(205) /* ./newline-before-return */,
        "newline-per-chained-call": () => require(206) /* ./newline-per-chained-call */,
        "no-alert": () => require(207) /* ./no-alert */,
        "no-array-constructor": () => require(208) /* ./no-array-constructor */,
        "no-async-promise-executor": () => require(209) /* ./no-async-promise-executor */,
        "no-await-in-loop": () => require(210) /* ./no-await-in-loop */,
        "no-bitwise": () => require(211) /* ./no-bitwise */,
        "no-buffer-constructor": () => require(212) /* ./no-buffer-constructor */,
        "no-caller": () => require(213) /* ./no-caller */,
        "no-case-declarations": () => require(214) /* ./no-case-declarations */,
        "no-catch-shadow": () => require(215) /* ./no-catch-shadow */,
        "no-class-assign": () => require(216) /* ./no-class-assign */,
        "no-compare-neg-zero": () => require(217) /* ./no-compare-neg-zero */,
        "no-cond-assign": () => require(218) /* ./no-cond-assign */,
        "no-confusing-arrow": () => require(219) /* ./no-confusing-arrow */,
        "no-console": () => require(220) /* ./no-console */,
        "no-const-assign": () => require(221) /* ./no-const-assign */,
        "no-constant-condition": () => require(222) /* ./no-constant-condition */,
        "no-constructor-return": () => require(223) /* ./no-constructor-return */,
        "no-continue": () => require(224) /* ./no-continue */,
        "no-control-regex": () => require(225) /* ./no-control-regex */,
        "no-debugger": () => require(227) /* ./no-debugger */,
        "no-delete-var": () => require(228) /* ./no-delete-var */,
        "no-div-regex": () => require(229) /* ./no-div-regex */,
        "no-dupe-args": () => require(230) /* ./no-dupe-args */,
        "no-dupe-class-members": () => require(231) /* ./no-dupe-class-members */,
        "no-dupe-else-if": () => require(232) /* ./no-dupe-else-if */,
        "no-dupe-keys": () => require(233) /* ./no-dupe-keys */,
        "no-duplicate-case": () => require(234) /* ./no-duplicate-case */,
        "no-duplicate-imports": () => require(235) /* ./no-duplicate-imports */,
        "no-else-return": () => require(236) /* ./no-else-return */,
        "no-empty": () => require(238) /* ./no-empty */,
        "no-empty-character-class": () => require(239) /* ./no-empty-character-class */,
        "no-empty-function": () => require(240) /* ./no-empty-function */,
        "no-empty-pattern": () => require(241) /* ./no-empty-pattern */,
        "no-eq-null": () => require(242) /* ./no-eq-null */,
        "no-eval": () => require(243) /* ./no-eval */,
        "no-ex-assign": () => require(244) /* ./no-ex-assign */,
        "no-extend-native": () => require(245) /* ./no-extend-native */,
        "no-extra-bind": () => require(246) /* ./no-extra-bind */,
        "no-extra-boolean-cast": () => require(247) /* ./no-extra-boolean-cast */,
        "no-extra-label": () => require(248) /* ./no-extra-label */,
        "no-extra-parens": () => require(249) /* ./no-extra-parens */,
        "no-extra-semi": () => require(251) /* ./no-extra-semi */,
        "no-fallthrough": () => require(252) /* ./no-fallthrough */,
        "no-floating-decimal": () => require(253) /* ./no-floating-decimal */,
        "no-func-assign": () => require(254) /* ./no-func-assign */,
        "no-global-assign": () => require(255) /* ./no-global-assign */,
        "no-implicit-coercion": () => require(256) /* ./no-implicit-coercion */,
        "no-implicit-globals": () => require(257) /* ./no-implicit-globals */,
        "no-implied-eval": () => require(258) /* ./no-implied-eval */,
        "no-import-assign": () => require(259) /* ./no-import-assign */,
        "no-inline-comments": () => require(260) /* ./no-inline-comments */,
        "no-inner-declarations": () => require(261) /* ./no-inner-declarations */,
        "no-invalid-regexp": () => require(262) /* ./no-invalid-regexp */,
        "no-invalid-this": () => require(263) /* ./no-invalid-this */,
        "no-irregular-whitespace": () => require(264) /* ./no-irregular-whitespace */,
        "no-iterator": () => require(265) /* ./no-iterator */,
        "no-label-var": () => require(266) /* ./no-label-var */,
        "no-labels": () => require(267) /* ./no-labels */,
        "no-lone-blocks": () => require(268) /* ./no-lone-blocks */,
        "no-lonely-if": () => require(269) /* ./no-lonely-if */,
        "no-loop-func": () => require(270) /* ./no-loop-func */,
        "no-magic-numbers": () => require(271) /* ./no-magic-numbers */,
        "no-misleading-character-class": () => require(272) /* ./no-misleading-character-class */,
        "no-mixed-operators": () => require(278) /* ./no-mixed-operators */,
        "no-mixed-requires": () => require(279) /* ./no-mixed-requires */,
        "no-mixed-spaces-and-tabs": () => require(280) /* ./no-mixed-spaces-and-tabs */,
        "no-multi-assign": () => require(281) /* ./no-multi-assign */,
        "no-multi-spaces": () => require(282) /* ./no-multi-spaces */,
        "no-multi-str": () => require(283) /* ./no-multi-str */,
        "no-multiple-empty-lines": () => require(284) /* ./no-multiple-empty-lines */,
        "no-native-reassign": () => require(285) /* ./no-native-reassign */,
        "no-negated-condition": () => require(286) /* ./no-negated-condition */,
        "no-negated-in-lhs": () => require(287) /* ./no-negated-in-lhs */,
        "no-nested-ternary": () => require(288) /* ./no-nested-ternary */,
        "no-new": () => require(289) /* ./no-new */,
        "no-new-func": () => require(290) /* ./no-new-func */,
        "no-new-object": () => require(291) /* ./no-new-object */,
        "no-new-require": () => require(292) /* ./no-new-require */,
        "no-new-symbol": () => require(293) /* ./no-new-symbol */,
        "no-new-wrappers": () => require(294) /* ./no-new-wrappers */,
        "no-obj-calls": () => require(295) /* ./no-obj-calls */,
        "no-octal": () => require(296) /* ./no-octal */,
        "no-octal-escape": () => require(297) /* ./no-octal-escape */,
        "no-param-reassign": () => require(298) /* ./no-param-reassign */,
        "no-path-concat": () => require(299) /* ./no-path-concat */,
        "no-plusplus": () => require(300) /* ./no-plusplus */,
        "no-process-env": () => require(301) /* ./no-process-env */,
        "no-process-exit": () => require(302) /* ./no-process-exit */,
        "no-proto": () => require(303) /* ./no-proto */,
        "no-prototype-builtins": () => require(304) /* ./no-prototype-builtins */,
        "no-redeclare": () => require(305) /* ./no-redeclare */,
        "no-regex-spaces": () => require(306) /* ./no-regex-spaces */,
        "no-restricted-globals": () => require(307) /* ./no-restricted-globals */,
        "no-restricted-imports": () => require(308) /* ./no-restricted-imports */,
        "no-restricted-modules": () => require(310) /* ./no-restricted-modules */,
        "no-restricted-properties": () => require(311) /* ./no-restricted-properties */,
        "no-restricted-syntax": () => require(312) /* ./no-restricted-syntax */,
        "no-return-assign": () => require(313) /* ./no-return-assign */,
        "no-return-await": () => require(314) /* ./no-return-await */,
        "no-script-url": () => require(315) /* ./no-script-url */,
        "no-self-assign": () => require(316) /* ./no-self-assign */,
        "no-self-compare": () => require(317) /* ./no-self-compare */,
        "no-sequences": () => require(318) /* ./no-sequences */,
        "no-setter-return": () => require(319) /* ./no-setter-return */,
        "no-shadow": () => require(320) /* ./no-shadow */,
        "no-shadow-restricted-names": () => require(321) /* ./no-shadow-restricted-names */,
        "no-spaced-func": () => require(322) /* ./no-spaced-func */,
        "no-sparse-arrays": () => require(323) /* ./no-sparse-arrays */,
        "no-sync": () => require(324) /* ./no-sync */,
        "no-tabs": () => require(325) /* ./no-tabs */,
        "no-template-curly-in-string": () => require(326) /* ./no-template-curly-in-string */,
        "no-ternary": () => require(327) /* ./no-ternary */,
        "no-this-before-super": () => require(328) /* ./no-this-before-super */,
        "no-throw-literal": () => require(329) /* ./no-throw-literal */,
        "no-trailing-spaces": () => require(330) /* ./no-trailing-spaces */,
        "no-undef": () => require(331) /* ./no-undef */,
        "no-undef-init": () => require(332) /* ./no-undef-init */,
        "no-undefined": () => require(333) /* ./no-undefined */,
        "no-underscore-dangle": () => require(334) /* ./no-underscore-dangle */,
        "no-unexpected-multiline": () => require(335) /* ./no-unexpected-multiline */,
        "no-unmodified-loop-condition": () => require(336) /* ./no-unmodified-loop-condition */,
        "no-unneeded-ternary": () => require(338) /* ./no-unneeded-ternary */,
        "no-unreachable": () => require(339) /* ./no-unreachable */,
        "no-unsafe-finally": () => require(340) /* ./no-unsafe-finally */,
        "no-unsafe-negation": () => require(341) /* ./no-unsafe-negation */,
        "no-unused-expressions": () => require(342) /* ./no-unused-expressions */,
        "no-unused-labels": () => require(343) /* ./no-unused-labels */,
        "no-unused-vars": () => require(344) /* ./no-unused-vars */,
        "no-use-before-define": () => require(345) /* ./no-use-before-define */,
        "no-useless-call": () => require(346) /* ./no-useless-call */,
        "no-useless-catch": () => require(347) /* ./no-useless-catch */,
        "no-useless-computed-key": () => require(348) /* ./no-useless-computed-key */,
        "no-useless-concat": () => require(349) /* ./no-useless-concat */,
        "no-useless-constructor": () => require(350) /* ./no-useless-constructor */,
        "no-useless-escape": () => require(351) /* ./no-useless-escape */,
        "no-useless-rename": () => require(352) /* ./no-useless-rename */,
        "no-useless-return": () => require(353) /* ./no-useless-return */,
        "no-var": () => require(354) /* ./no-var */,
        "no-void": () => require(355) /* ./no-void */,
        "no-warning-comments": () => require(356) /* ./no-warning-comments */,
        "no-whitespace-before-property": () => require(357) /* ./no-whitespace-before-property */,
        "no-with": () => require(358) /* ./no-with */,
        "nonblock-statement-body-position": () => require(359) /* ./nonblock-statement-body-position */,
        "object-curly-newline": () => require(360) /* ./object-curly-newline */,
        "object-curly-spacing": () => require(361) /* ./object-curly-spacing */,
        "object-property-newline": () => require(362) /* ./object-property-newline */,
        "object-shorthand": () => require(363) /* ./object-shorthand */,
        "one-var": () => require(364) /* ./one-var */,
        "one-var-declaration-per-line": () => require(365) /* ./one-var-declaration-per-line */,
        "operator-assignment": () => require(366) /* ./operator-assignment */,
        "operator-linebreak": () => require(367) /* ./operator-linebreak */,
        "padded-blocks": () => require(368) /* ./padded-blocks */,
        "padding-line-between-statements": () => require(369) /* ./padding-line-between-statements */,
        "prefer-arrow-callback": () => require(370) /* ./prefer-arrow-callback */,
        "prefer-const": () => require(371) /* ./prefer-const */,
        "prefer-destructuring": () => require(372) /* ./prefer-destructuring */,
        "prefer-exponentiation-operator": () => require(373) /* ./prefer-exponentiation-operator */,
        "prefer-named-capture-group": () => require(374) /* ./prefer-named-capture-group */,
        "prefer-numeric-literals": () => require(375) /* ./prefer-numeric-literals */,
        "prefer-object-spread": () => require(376) /* ./prefer-object-spread */,
        "prefer-promise-reject-errors": () => require(377) /* ./prefer-promise-reject-errors */,
        "prefer-reflect": () => require(378) /* ./prefer-reflect */,
        "prefer-regex-literals": () => require(379) /* ./prefer-regex-literals */,
        "prefer-rest-params": () => require(380) /* ./prefer-rest-params */,
        "prefer-spread": () => require(381) /* ./prefer-spread */,
        "prefer-template": () => require(382) /* ./prefer-template */,
        "quote-props": () => require(383) /* ./quote-props */,
        quotes: () => require(384) /* ./quotes */,
        radix: () => require(385) /* ./radix */,
        "require-atomic-updates": () => require(386) /* ./require-atomic-updates */,
        "require-await": () => require(387) /* ./require-await */,
        "require-jsdoc": () => require(388) /* ./require-jsdoc */,
        "require-unicode-regexp": () => require(389) /* ./require-unicode-regexp */,
        "require-yield": () => require(390) /* ./require-yield */,
        "rest-spread-spacing": () => require(391) /* ./rest-spread-spacing */,
        semi: () => require(392) /* ./semi */,
        "semi-spacing": () => require(393) /* ./semi-spacing */,
        "semi-style": () => require(394) /* ./semi-style */,
        "sort-imports": () => require(395) /* ./sort-imports */,
        "sort-keys": () => require(396) /* ./sort-keys */,
        "sort-vars": () => require(398) /* ./sort-vars */,
        "space-before-blocks": () => require(399) /* ./space-before-blocks */,
        "space-before-function-paren": () => require(400) /* ./space-before-function-paren */,
        "space-in-parens": () => require(401) /* ./space-in-parens */,
        "space-infix-ops": () => require(402) /* ./space-infix-ops */,
        "space-unary-ops": () => require(403) /* ./space-unary-ops */,
        "spaced-comment": () => require(404) /* ./spaced-comment */,
        strict: () => require(405) /* ./strict */,
        "switch-colon-spacing": () => require(406) /* ./switch-colon-spacing */,
        "symbol-description": () => require(407) /* ./symbol-description */,
        "template-curly-spacing": () => require(408) /* ./template-curly-spacing */,
        "template-tag-spacing": () => require(409) /* ./template-tag-spacing */,
        "unicode-bom": () => require(410) /* ./unicode-bom */,
        "use-isnan": () => require(411) /* ./use-isnan */,
        "valid-jsdoc": () => require(412) /* ./valid-jsdoc */,
        "valid-typeof": () => require(417) /* ./valid-typeof */,
        "vars-on-top": () => require(418) /* ./vars-on-top */,
        "wrap-iife": () => require(419) /* ./wrap-iife */,
        "wrap-regex": () => require(420) /* ./wrap-regex */,
        "yield-star-spacing": () => require(421) /* ./yield-star-spacing */,
        yoda: () => require(422) /* ./yoda */
    }));
},
/* eslint/lib/rules/utils/lazy-loading-rule-map.js */ function _(require, module, exports) {
    const debug = require(120) /* debug */("eslint:rules");
    /** @typedef {import("./types").Rule} Rule */
    /**
     * The `Map` object that loads each rule when it's accessed.
     * @example
     * const rules = new LazyLoadingRuleMap([
     *     ["eqeqeq", () => require("eqeqeq")],
     *     ["semi", () => require("semi")],
     *     ["no-unused-vars", () => require("no-unused-vars")],
     * ])
     *
     * rules.get("semi") // call `() => require("semi")` here.
     *
     * @extends {Map<string, () => Rule>}
     */
    class LazyLoadingRuleMap extends Map {
        /**
         * Initialize this map.
         * @param {Array<[string, function(): Rule]>} loaders The rule loaders.
         */
        constructor(loaders) {
            let remaining = loaders.length;
            super(debug.enabled
                ? loaders.map(([ruleId, load]) => {
                    let cache = null;
                    return [
                        ruleId,
                        () => {
                            if (!cache) {
                                debug("Loading rule %o (remaining=%d)", ruleId, --remaining);
                                cache = load();
                            }
                            return cache;
                        }
                    ];
                })
                : loaders);
            // `super(...iterable)` uses `this.set()`, so disable it here.
            Object.defineProperty(LazyLoadingRuleMap.prototype, "set", {
                configurable: true,
                value: void 0
            });
        }
        /**
         * Get a rule.
         * Each rule will be loaded on the first access.
         * @param {string} ruleId The rule ID to get.
         * @returns {Rule|undefined} The rule.
         */
        get(ruleId) {
            const load = super.get(ruleId);
            return load && load();
        }
        /**
         * Iterate rules.
         * @returns {IterableIterator<Rule>} Rules.
         */
        *values() {
            for (const load of super.values()) {
                yield load();
            }
        }
        /**
         * Iterate rules.
         * @returns {IterableIterator<[string, Rule]>} Rules.
         */
        *entries() {
            for (const [ruleId, load] of super.entries()) {
                yield [ruleId, load()];
            }
        }
        /**
         * Call a function with each rule.
         * @param {Function} callbackFn The callback function.
         * @param {any} [thisArg] The object to pass to `this` of the callback function.
         * @returns {void}
         */
        forEach(callbackFn, thisArg) {
            for (const [ruleId, load] of super.entries()) {
                callbackFn.call(thisArg, load(), ruleId, this);
            }
        }
    }
    // Forbid mutation.
    Object.defineProperties(LazyLoadingRuleMap.prototype, {
        clear: { configurable: true, value: void 0 },
        delete: { configurable: true, value: void 0 },
        [Symbol.iterator]: {
            configurable: true,
            writable: true,
            value: LazyLoadingRuleMap.prototype.entries
        }
    });
    module.exports = { LazyLoadingRuleMap };
},
/* debug/src/index.js */ function _(require, module, exports) {
    /**
     * Detect Electron renderer / nwjs process, which is node, but we should
     * treat as a browser.
     */
    if (typeof process === 'undefined' || process.type === 'renderer' || process.browser === true || process.__nwjs) {
        module.exports = require(121) /* ./browser.js */;
    }
    else {
        module.exports = require(124) /* ./node.js */;
    }
},
/* debug/src/browser.js */ function _(require, module, exports) {
    /* eslint-env browser */
    /**
     * This is the web browser implementation of `debug()`.
     */
    exports.log = log;
    exports.formatArgs = formatArgs;
    exports.save = save;
    exports.load = load;
    exports.useColors = useColors;
    exports.storage = localstorage();
    /**
     * Colors.
     */
    exports.colors = [
        '#0000CC',
        '#0000FF',
        '#0033CC',
        '#0033FF',
        '#0066CC',
        '#0066FF',
        '#0099CC',
        '#0099FF',
        '#00CC00',
        '#00CC33',
        '#00CC66',
        '#00CC99',
        '#00CCCC',
        '#00CCFF',
        '#3300CC',
        '#3300FF',
        '#3333CC',
        '#3333FF',
        '#3366CC',
        '#3366FF',
        '#3399CC',
        '#3399FF',
        '#33CC00',
        '#33CC33',
        '#33CC66',
        '#33CC99',
        '#33CCCC',
        '#33CCFF',
        '#6600CC',
        '#6600FF',
        '#6633CC',
        '#6633FF',
        '#66CC00',
        '#66CC33',
        '#9900CC',
        '#9900FF',
        '#9933CC',
        '#9933FF',
        '#99CC00',
        '#99CC33',
        '#CC0000',
        '#CC0033',
        '#CC0066',
        '#CC0099',
        '#CC00CC',
        '#CC00FF',
        '#CC3300',
        '#CC3333',
        '#CC3366',
        '#CC3399',
        '#CC33CC',
        '#CC33FF',
        '#CC6600',
        '#CC6633',
        '#CC9900',
        '#CC9933',
        '#CCCC00',
        '#CCCC33',
        '#FF0000',
        '#FF0033',
        '#FF0066',
        '#FF0099',
        '#FF00CC',
        '#FF00FF',
        '#FF3300',
        '#FF3333',
        '#FF3366',
        '#FF3399',
        '#FF33CC',
        '#FF33FF',
        '#FF6600',
        '#FF6633',
        '#FF9900',
        '#FF9933',
        '#FFCC00',
        '#FFCC33'
    ];
    /**
     * Currently only WebKit-based Web Inspectors, Firefox >= v31,
     * and the Firebug extension (any Firefox version) are known
     * to support "%c" CSS customizations.
     *
     * TODO: add a `localStorage` variable to explicitly enable/disable colors
     */
    // eslint-disable-next-line complexity
    function useColors() {
        // NB: In an Electron preload script, document will be defined but not fully
        // initialized. Since we know we're in Chrome, we'll just detect this case
        // explicitly
        if (typeof window !== 'undefined' && window.process && (window.process.type === 'renderer' || window.process.__nwjs)) {
            return true;
        }
        // Internet Explorer and Edge do not support colors.
        if (typeof navigator !== 'undefined' && navigator.userAgent && navigator.userAgent.toLowerCase().match(/(edge|trident)\/(\d+)/)) {
            return false;
        }
        // Is webkit? http://stackoverflow.com/a/16459606/376773
        // document is undefined in react-native: https://github.com/facebook/react-native/pull/1632
        return (typeof document !== 'undefined' && document.documentElement && document.documentElement.style && document.documentElement.style.WebkitAppearance) ||
            // Is firebug? http://stackoverflow.com/a/398120/376773
            (typeof window !== 'undefined' && window.console && (window.console.firebug || (window.console.exception && window.console.table))) ||
            // Is firefox >= v31?
            // https://developer.mozilla.org/en-US/docs/Tools/Web_Console#Styling_messages
            (typeof navigator !== 'undefined' && navigator.userAgent && navigator.userAgent.toLowerCase().match(/firefox\/(\d+)/) && parseInt(RegExp.$1, 10) >= 31) ||
            // Double check webkit in userAgent just in case we are in a worker
            (typeof navigator !== 'undefined' && navigator.userAgent && navigator.userAgent.toLowerCase().match(/applewebkit\/(\d+)/));
    }
    /**
     * Colorize log arguments if enabled.
     *
     * @api public
     */
    function formatArgs(args) {
        args[0] = (this.useColors ? '%c' : '') +
            this.namespace +
            (this.useColors ? ' %c' : ' ') +
            args[0] +
            (this.useColors ? '%c ' : ' ') +
            '+' + module.exports.humanize(this.diff);
        if (!this.useColors) {
            return;
        }
        const c = 'color: ' + this.color;
        args.splice(1, 0, c, 'color: inherit');
        // The final "%c" is somewhat tricky, because there could be other
        // arguments passed either before or after the %c, so we need to
        // figure out the correct index to insert the CSS into
        let index = 0;
        let lastC = 0;
        args[0].replace(/%[a-zA-Z%]/g, match => {
            if (match === '%%') {
                return;
            }
            index++;
            if (match === '%c') {
                // We only are interested in the *last* %c
                // (the user may have provided their own)
                lastC = index;
            }
        });
        args.splice(lastC, 0, c);
    }
    /**
     * Invokes `console.log()` when available.
     * No-op when `console.log` is not a "function".
     *
     * @api public
     */
    function log(...args) {
        // This hackery is required for IE8/9, where
        // the `console.log` function doesn't have 'apply'
        return typeof console === 'object' &&
            console.log &&
            console.log(...args);
    }
    /**
     * Save `namespaces`.
     *
     * @param {String} namespaces
     * @api private
     */
    function save(namespaces) {
        try {
            if (namespaces) {
                exports.storage.setItem('debug', namespaces);
            }
            else {
                exports.storage.removeItem('debug');
            }
        }
        catch (error) {
            // Swallow
            // XXX (@Qix-) should we be logging these?
        }
    }
    /**
     * Load `namespaces`.
     *
     * @return {String} returns the previously persisted debug modes
     * @api private
     */
    function load() {
        let r;
        try {
            r = exports.storage.getItem('debug');
        }
        catch (error) {
            // Swallow
            // XXX (@Qix-) should we be logging these?
        }
        // If debug isn't set in LS, and we're in Electron, try to load $DEBUG
        if (!r && typeof process !== 'undefined' && 'env' in process) {
            r = process.env.DEBUG;
        }
        return r;
    }
    /**
     * Localstorage attempts to return the localstorage.
     *
     * This is necessary because safari throws
     * when a user disables cookies/localstorage
     * and you attempt to access it.
     *
     * @return {LocalStorage}
     * @api private
     */
    function localstorage() {
        try {
            // TVMLKit (Apple TV JS Runtime) does not have a window object, just localStorage in the global context
            // The Browser also has localStorage in the global context.
            return localStorage;
        }
        catch (error) {
            // Swallow
            // XXX (@Qix-) should we be logging these?
        }
    }
    module.exports = require(122) /* ./common */(exports);
    const { formatters } = module.exports;
    /**
     * Map %j to `JSON.stringify()`, since no Web Inspectors do that by default.
     */
    formatters.j = function (v) {
        try {
            return JSON.stringify(v);
        }
        catch (error) {
            return '[UnexpectedJSONParseError]: ' + error.message;
        }
    };
},
/* debug/src/common.js */ function _(require, module, exports) {
    /**
     * This is the common logic for both the Node.js and web browser
     * implementations of `debug()`.
     */
    function setup(env) {
        createDebug.debug = createDebug;
        createDebug.default = createDebug;
        createDebug.coerce = coerce;
        createDebug.disable = disable;
        createDebug.enable = enable;
        createDebug.enabled = enabled;
        createDebug.humanize = require(123) /* ms */;
        Object.keys(env).forEach(key => {
            createDebug[key] = env[key];
        });
        /**
        * Active `debug` instances.
        */
        createDebug.instances = [];
        /**
        * The currently active debug mode names, and names to skip.
        */
        createDebug.names = [];
        createDebug.skips = [];
        /**
        * Map of special "%n" handling functions, for the debug "format" argument.
        *
        * Valid key names are a single, lower or upper-case letter, i.e. "n" and "N".
        */
        createDebug.formatters = {};
        /**
        * Selects a color for a debug namespace
        * @param {String} namespace The namespace string for the for the debug instance to be colored
        * @return {Number|String} An ANSI color code for the given namespace
        * @api private
        */
        function selectColor(namespace) {
            let hash = 0;
            for (let i = 0; i < namespace.length; i++) {
                hash = ((hash << 5) - hash) + namespace.charCodeAt(i);
                hash |= 0; // Convert to 32bit integer
            }
            return createDebug.colors[Math.abs(hash) % createDebug.colors.length];
        }
        createDebug.selectColor = selectColor;
        /**
        * Create a debugger with the given `namespace`.
        *
        * @param {String} namespace
        * @return {Function}
        * @api public
        */
        function createDebug(namespace) {
            let prevTime;
            function debug(...args) {
                // Disabled?
                if (!debug.enabled) {
                    return;
                }
                const self = debug;
                // Set `diff` timestamp
                const curr = Number(new Date());
                const ms = curr - (prevTime || curr);
                self.diff = ms;
                self.prev = prevTime;
                self.curr = curr;
                prevTime = curr;
                args[0] = createDebug.coerce(args[0]);
                if (typeof args[0] !== 'string') {
                    // Anything else let's inspect with %O
                    args.unshift('%O');
                }
                // Apply any `formatters` transformations
                let index = 0;
                args[0] = args[0].replace(/%([a-zA-Z%])/g, (match, format) => {
                    // If we encounter an escaped % then don't increase the array index
                    if (match === '%%') {
                        return match;
                    }
                    index++;
                    const formatter = createDebug.formatters[format];
                    if (typeof formatter === 'function') {
                        const val = args[index];
                        match = formatter.call(self, val);
                        // Now we need to remove `args[index]` since it's inlined in the `format`
                        args.splice(index, 1);
                        index--;
                    }
                    return match;
                });
                // Apply env-specific formatting (colors, etc.)
                createDebug.formatArgs.call(self, args);
                const logFn = self.log || createDebug.log;
                logFn.apply(self, args);
            }
            debug.namespace = namespace;
            debug.enabled = createDebug.enabled(namespace);
            debug.useColors = createDebug.useColors();
            debug.color = selectColor(namespace);
            debug.destroy = destroy;
            debug.extend = extend;
            // Debug.formatArgs = formatArgs;
            // debug.rawLog = rawLog;
            // env-specific initialization logic for debug instances
            if (typeof createDebug.init === 'function') {
                createDebug.init(debug);
            }
            createDebug.instances.push(debug);
            return debug;
        }
        function destroy() {
            const index = createDebug.instances.indexOf(this);
            if (index !== -1) {
                createDebug.instances.splice(index, 1);
                return true;
            }
            return false;
        }
        function extend(namespace, delimiter) {
            const newDebug = createDebug(this.namespace + (typeof delimiter === 'undefined' ? ':' : delimiter) + namespace);
            newDebug.log = this.log;
            return newDebug;
        }
        /**
        * Enables a debug mode by namespaces. This can include modes
        * separated by a colon and wildcards.
        *
        * @param {String} namespaces
        * @api public
        */
        function enable(namespaces) {
            createDebug.save(namespaces);
            createDebug.names = [];
            createDebug.skips = [];
            let i;
            const split = (typeof namespaces === 'string' ? namespaces : '').split(/[\s,]+/);
            const len = split.length;
            for (i = 0; i < len; i++) {
                if (!split[i]) {
                    // ignore empty strings
                    continue;
                }
                namespaces = split[i].replace(/\*/g, '.*?');
                if (namespaces[0] === '-') {
                    createDebug.skips.push(new RegExp('^' + namespaces.substr(1) + '$'));
                }
                else {
                    createDebug.names.push(new RegExp('^' + namespaces + '$'));
                }
            }
            for (i = 0; i < createDebug.instances.length; i++) {
                const instance = createDebug.instances[i];
                instance.enabled = createDebug.enabled(instance.namespace);
            }
        }
        /**
        * Disable debug output.
        *
        * @return {String} namespaces
        * @api public
        */
        function disable() {
            const namespaces = [
                ...createDebug.names.map(toNamespace),
                ...createDebug.skips.map(toNamespace).map(namespace => '-' + namespace)
            ].join(',');
            createDebug.enable('');
            return namespaces;
        }
        /**
        * Returns true if the given mode name is enabled, false otherwise.
        *
        * @param {String} name
        * @return {Boolean}
        * @api public
        */
        function enabled(name) {
            if (name[name.length - 1] === '*') {
                return true;
            }
            let i;
            let len;
            for (i = 0, len = createDebug.skips.length; i < len; i++) {
                if (createDebug.skips[i].test(name)) {
                    return false;
                }
            }
            for (i = 0, len = createDebug.names.length; i < len; i++) {
                if (createDebug.names[i].test(name)) {
                    return true;
                }
            }
            return false;
        }
        /**
        * Convert regexp to namespace
        *
        * @param {RegExp} regxep
        * @return {String} namespace
        * @api private
        */
        function toNamespace(regexp) {
            return regexp.toString()
                .substring(2, regexp.toString().length - 2)
                .replace(/\.\*\?$/, '*');
        }
        /**
        * Coerce `val`.
        *
        * @param {Mixed} val
        * @return {Mixed}
        * @api private
        */
        function coerce(val) {
            if (val instanceof Error) {
                return val.stack || val.message;
            }
            return val;
        }
        createDebug.enable(createDebug.load());
        return createDebug;
    }
    module.exports = setup;
},
/* ms/index.js */ function _(require, module, exports) {
    /**
     * Helpers.
     */
    var s = 1000;
    var m = s * 60;
    var h = m * 60;
    var d = h * 24;
    var w = d * 7;
    var y = d * 365.25;
    /**
     * Parse or format the given `val`.
     *
     * Options:
     *
     *  - `long` verbose formatting [false]
     *
     * @param {String|Number} val
     * @param {Object} [options]
     * @throws {Error} throw an error if val is not a non-empty string or a number
     * @return {String|Number}
     * @api public
     */
    module.exports = function (val, options) {
        options = options || {};
        var type = typeof val;
        if (type === 'string' && val.length > 0) {
            return parse(val);
        }
        else if (type === 'number' && isFinite(val)) {
            return options.long ? fmtLong(val) : fmtShort(val);
        }
        throw new Error('val is not a non-empty string or a valid number. val=' +
            JSON.stringify(val));
    };
    /**
     * Parse the given `str` and return milliseconds.
     *
     * @param {String} str
     * @return {Number}
     * @api private
     */
    function parse(str) {
        str = String(str);
        if (str.length > 100) {
            return;
        }
        var match = /^(-?(?:\d+)?\.?\d+) *(milliseconds?|msecs?|ms|seconds?|secs?|s|minutes?|mins?|m|hours?|hrs?|h|days?|d|weeks?|w|years?|yrs?|y)?$/i.exec(str);
        if (!match) {
            return;
        }
        var n = parseFloat(match[1]);
        var type = (match[2] || 'ms').toLowerCase();
        switch (type) {
            case 'years':
            case 'year':
            case 'yrs':
            case 'yr':
            case 'y':
                return n * y;
            case 'weeks':
            case 'week':
            case 'w':
                return n * w;
            case 'days':
            case 'day':
            case 'd':
                return n * d;
            case 'hours':
            case 'hour':
            case 'hrs':
            case 'hr':
            case 'h':
                return n * h;
            case 'minutes':
            case 'minute':
            case 'mins':
            case 'min':
            case 'm':
                return n * m;
            case 'seconds':
            case 'second':
            case 'secs':
            case 'sec':
            case 's':
                return n * s;
            case 'milliseconds':
            case 'millisecond':
            case 'msecs':
            case 'msec':
            case 'ms':
                return n;
            default:
                return undefined;
        }
    }
    /**
     * Short format for `ms`.
     *
     * @param {Number} ms
     * @return {String}
     * @api private
     */
    function fmtShort(ms) {
        var msAbs = Math.abs(ms);
        if (msAbs >= d) {
            return Math.round(ms / d) + 'd';
        }
        if (msAbs >= h) {
            return Math.round(ms / h) + 'h';
        }
        if (msAbs >= m) {
            return Math.round(ms / m) + 'm';
        }
        if (msAbs >= s) {
            return Math.round(ms / s) + 's';
        }
        return ms + 'ms';
    }
    /**
     * Long format for `ms`.
     *
     * @param {Number} ms
     * @return {String}
     * @api private
     */
    function fmtLong(ms) {
        var msAbs = Math.abs(ms);
        if (msAbs >= d) {
            return plural(ms, msAbs, d, 'day');
        }
        if (msAbs >= h) {
            return plural(ms, msAbs, h, 'hour');
        }
        if (msAbs >= m) {
            return plural(ms, msAbs, m, 'minute');
        }
        if (msAbs >= s) {
            return plural(ms, msAbs, s, 'second');
        }
        return ms + ' ms';
    }
    /**
     * Pluralization helper.
     */
    function plural(ms, msAbs, n, name) {
        var isPlural = msAbs >= n * 1.5;
        return Math.round(ms / n) + ' ' + name + (isPlural ? 's' : '');
    }
},
/* debug/src/node.js */ function _(require, module, exports) {
    /**
     * Module dependencies.
     */
    const tty = require('tty');
    const util = require('util');
    /**
     * This is the Node.js implementation of `debug()`.
     */
    exports.init = init;
    exports.log = log;
    exports.formatArgs = formatArgs;
    exports.save = save;
    exports.load = load;
    exports.useColors = useColors;
    /**
     * Colors.
     */
    exports.colors = [6, 2, 3, 4, 5, 1];
    try {
        // Optional dependency (as in, doesn't need to be installed, NOT like optionalDependencies in package.json)
        // eslint-disable-next-line import/no-extraneous-dependencies
        const supportsColor = require(61) /* supports-color */;
        if (supportsColor && (supportsColor.stderr || supportsColor).level >= 2) {
            exports.colors = [
                20,
                21,
                26,
                27,
                32,
                33,
                38,
                39,
                40,
                41,
                42,
                43,
                44,
                45,
                56,
                57,
                62,
                63,
                68,
                69,
                74,
                75,
                76,
                77,
                78,
                79,
                80,
                81,
                92,
                93,
                98,
                99,
                112,
                113,
                128,
                129,
                134,
                135,
                148,
                149,
                160,
                161,
                162,
                163,
                164,
                165,
                166,
                167,
                168,
                169,
                170,
                171,
                172,
                173,
                178,
                179,
                184,
                185,
                196,
                197,
                198,
                199,
                200,
                201,
                202,
                203,
                204,
                205,
                206,
                207,
                208,
                209,
                214,
                215,
                220,
                221
            ];
        }
    }
    catch (error) {
        // Swallow - we only care if `supports-color` is available; it doesn't have to be.
    }
    /**
     * Build up the default `inspectOpts` object from the environment variables.
     *
     *   $ DEBUG_COLORS=no DEBUG_DEPTH=10 DEBUG_SHOW_HIDDEN=enabled node script.js
     */
    exports.inspectOpts = Object.keys(process.env).filter(key => {
        return /^debug_/i.test(key);
    }).reduce((obj, key) => {
        // Camel-case
        const prop = key
            .substring(6)
            .toLowerCase()
            .replace(/_([a-z])/g, (_, k) => {
            return k.toUpperCase();
        });
        // Coerce string value into JS value
        let val = process.env[key];
        if (/^(yes|on|true|enabled)$/i.test(val)) {
            val = true;
        }
        else if (/^(no|off|false|disabled)$/i.test(val)) {
            val = false;
        }
        else if (val === 'null') {
            val = null;
        }
        else {
            val = Number(val);
        }
        obj[prop] = val;
        return obj;
    }, {});
    /**
     * Is stdout a TTY? Colored output is enabled when `true`.
     */
    function useColors() {
        return 'colors' in exports.inspectOpts ?
            Boolean(exports.inspectOpts.colors) :
            tty.isatty(process.stderr.fd);
    }
    /**
     * Adds ANSI color escape codes if enabled.
     *
     * @api public
     */
    function formatArgs(args) {
        const { namespace: name, useColors } = this;
        if (useColors) {
            const c = this.color;
            const colorCode = '\u001B[3' + (c < 8 ? c : '8;5;' + c);
            const prefix = `  ${colorCode};1m${name} \u001B[0m`;
            args[0] = prefix + args[0].split('\n').join('\n' + prefix);
            args.push(colorCode + 'm+' + module.exports.humanize(this.diff) + '\u001B[0m');
        }
        else {
            args[0] = getDate() + name + ' ' + args[0];
        }
    }
    function getDate() {
        if (exports.inspectOpts.hideDate) {
            return '';
        }
        return new Date().toISOString() + ' ';
    }
    /**
     * Invokes `util.format()` with the specified arguments and writes to stderr.
     */
    function log(...args) {
        return process.stderr.write(util.format(...args) + '\n');
    }
    /**
     * Save `namespaces`.
     *
     * @param {String} namespaces
     * @api private
     */
    function save(namespaces) {
        if (namespaces) {
            process.env.DEBUG = namespaces;
        }
        else {
            // If you set a process.env field to null or undefined, it gets cast to the
            // string 'null' or 'undefined'. Just delete instead.
            delete process.env.DEBUG;
        }
    }
    /**
     * Load `namespaces`.
     *
     * @return {String} returns the previously persisted debug modes
     * @api private
     */
    function load() {
        return process.env.DEBUG;
    }
    /**
     * Init logic for `debug` instances.
     *
     * Create a new `inspectOpts` object in case `useColors` is set
     * differently for a particular `debug` instance.
     */
    function init(debug) {
        debug.inspectOpts = {};
        const keys = Object.keys(exports.inspectOpts);
        for (let i = 0; i < keys.length; i++) {
            debug.inspectOpts[keys[i]] = exports.inspectOpts[keys[i]];
        }
    }
    module.exports = require(122) /* ./common */(exports);
    const { formatters } = module.exports;
    /**
     * Map %o to `util.inspect()`, all on a single line.
     */
    formatters.o = function (v) {
        this.inspectOpts.colors = this.useColors;
        return util.inspect(v, this.inspectOpts)
            .replace(/\s*\n\s*/g, ' ');
    };
    /**
     * Map %O to `util.inspect()`, allowing multiple lines if needed.
     */
    formatters.O = function (v) {
        this.inspectOpts.colors = this.useColors;
        return util.inspect(v, this.inspectOpts);
    };
},
/* eslint/lib/rules/accessor-pairs.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Typedefs
    //------------------------------------------------------------------------------
    /**
     * Property name if it can be computed statically, otherwise the list of the tokens of the key node.
     * @typedef {string|Token[]} Key
     */
    /**
     * Accessor nodes with the same key.
     * @typedef {Object} AccessorData
     * @property {Key} key Accessor's key
     * @property {ASTNode[]} getters List of getter nodes.
     * @property {ASTNode[]} setters List of setter nodes.
     */
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not the given lists represent the equal tokens in the same order.
     * Tokens are compared by their properties, not by instance.
     * @param {Token[]} left First list of tokens.
     * @param {Token[]} right Second list of tokens.
     * @returns {boolean} `true` if the lists have same tokens.
     */
    function areEqualTokenLists(left, right) {
        if (left.length !== right.length) {
            return false;
        }
        for (let i = 0; i < left.length; i++) {
            const leftToken = left[i], rightToken = right[i];
            if (leftToken.type !== rightToken.type || leftToken.value !== rightToken.value) {
                return false;
            }
        }
        return true;
    }
    /**
     * Checks whether or not the given keys are equal.
     * @param {Key} left First key.
     * @param {Key} right Second key.
     * @returns {boolean} `true` if the keys are equal.
     */
    function areEqualKeys(left, right) {
        if (typeof left === "string" && typeof right === "string") {
            // Statically computed names.
            return left === right;
        }
        if (Array.isArray(left) && Array.isArray(right)) {
            // Token lists.
            return areEqualTokenLists(left, right);
        }
        return false;
    }
    /**
     * Checks whether or not a given node is of an accessor kind ('get' or 'set').
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node is of an accessor kind.
     */
    function isAccessorKind(node) {
        return node.kind === "get" || node.kind === "set";
    }
    /**
     * Checks whether or not a given node is an `Identifier` node which was named a given name.
     * @param {ASTNode} node A node to check.
     * @param {string} name An expected name of the node.
     * @returns {boolean} `true` if the node is an `Identifier` node which was named as expected.
     */
    function isIdentifier(node, name) {
        return node.type === "Identifier" && node.name === name;
    }
    /**
     * Checks whether or not a given node is an argument of a specified method call.
     * @param {ASTNode} node A node to check.
     * @param {number} index An expected index of the node in arguments.
     * @param {string} object An expected name of the object of the method.
     * @param {string} property An expected name of the method.
     * @returns {boolean} `true` if the node is an argument of the specified method call.
     */
    function isArgumentOfMethodCall(node, index, object, property) {
        const parent = node.parent;
        return (parent.type === "CallExpression" &&
            parent.callee.type === "MemberExpression" &&
            parent.callee.computed === false &&
            isIdentifier(parent.callee.object, object) &&
            isIdentifier(parent.callee.property, property) &&
            parent.arguments[index] === node);
    }
    /**
     * Checks whether or not a given node is a property descriptor.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node is a property descriptor.
     */
    function isPropertyDescriptor(node) {
        // Object.defineProperty(obj, "foo", {set: ...})
        if (isArgumentOfMethodCall(node, 2, "Object", "defineProperty") ||
            isArgumentOfMethodCall(node, 2, "Reflect", "defineProperty")) {
            return true;
        }
        /*
         * Object.defineProperties(obj, {foo: {set: ...}})
         * Object.create(proto, {foo: {set: ...}})
         */
        const grandparent = node.parent.parent;
        return grandparent.type === "ObjectExpression" && (isArgumentOfMethodCall(grandparent, 1, "Object", "create") ||
            isArgumentOfMethodCall(grandparent, 1, "Object", "defineProperties"));
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce getter and setter pairs in objects and classes",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/accessor-pairs"
            },
            schema: [{
                    type: "object",
                    properties: {
                        getWithoutSet: {
                            type: "boolean",
                            default: false
                        },
                        setWithoutGet: {
                            type: "boolean",
                            default: true
                        },
                        enforceForClassMembers: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }],
            messages: {
                missingGetterInPropertyDescriptor: "Getter is not present in property descriptor.",
                missingSetterInPropertyDescriptor: "Setter is not present in property descriptor.",
                missingGetterInObjectLiteral: "Getter is not present for {{ name }}.",
                missingSetterInObjectLiteral: "Setter is not present for {{ name }}.",
                missingGetterInClass: "Getter is not present for class {{ name }}.",
                missingSetterInClass: "Setter is not present for class {{ name }}."
            }
        },
        create(context) {
            const config = context.options[0] || {};
            const checkGetWithoutSet = config.getWithoutSet === true;
            const checkSetWithoutGet = config.setWithoutGet !== false;
            const enforceForClassMembers = config.enforceForClassMembers === true;
            const sourceCode = context.getSourceCode();
            /**
             * Reports the given node.
             * @param {ASTNode} node The node to report.
             * @param {string} messageKind "missingGetter" or "missingSetter".
             * @returns {void}
             * @private
             */
            function report(node, messageKind) {
                if (node.type === "Property") {
                    context.report({
                        node,
                        messageId: `${messageKind}InObjectLiteral`,
                        loc: astUtils.getFunctionHeadLoc(node.value, sourceCode),
                        data: { name: astUtils.getFunctionNameWithKind(node.value) }
                    });
                }
                else if (node.type === "MethodDefinition") {
                    context.report({
                        node,
                        messageId: `${messageKind}InClass`,
                        loc: astUtils.getFunctionHeadLoc(node.value, sourceCode),
                        data: { name: astUtils.getFunctionNameWithKind(node.value) }
                    });
                }
                else {
                    context.report({
                        node,
                        messageId: `${messageKind}InPropertyDescriptor`
                    });
                }
            }
            /**
             * Reports each of the nodes in the given list using the same messageId.
             * @param {ASTNode[]} nodes Nodes to report.
             * @param {string} messageKind "missingGetter" or "missingSetter".
             * @returns {void}
             * @private
             */
            function reportList(nodes, messageKind) {
                for (const node of nodes) {
                    report(node, messageKind);
                }
            }
            /**
             * Creates a new `AccessorData` object for the given getter or setter node.
             * @param {ASTNode} node A getter or setter node.
             * @returns {AccessorData} New `AccessorData` object that contains the given node.
             * @private
             */
            function createAccessorData(node) {
                const name = astUtils.getStaticPropertyName(node);
                const key = (name !== null) ? name : sourceCode.getTokens(node.key);
                return {
                    key,
                    getters: node.kind === "get" ? [node] : [],
                    setters: node.kind === "set" ? [node] : []
                };
            }
            /**
             * Merges the given `AccessorData` object into the given accessors list.
             * @param {AccessorData[]} accessors The list to merge into.
             * @param {AccessorData} accessorData The object to merge.
             * @returns {AccessorData[]} The same instance with the merged object.
             * @private
             */
            function mergeAccessorData(accessors, accessorData) {
                const equalKeyElement = accessors.find(a => areEqualKeys(a.key, accessorData.key));
                if (equalKeyElement) {
                    equalKeyElement.getters.push(...accessorData.getters);
                    equalKeyElement.setters.push(...accessorData.setters);
                }
                else {
                    accessors.push(accessorData);
                }
                return accessors;
            }
            /**
             * Checks accessor pairs in the given list of nodes.
             * @param {ASTNode[]} nodes The list to check.
             * @returns {void}
             * @private
             */
            function checkList(nodes) {
                const accessors = nodes
                    .filter(isAccessorKind)
                    .map(createAccessorData)
                    .reduce(mergeAccessorData, []);
                for (const { getters, setters } of accessors) {
                    if (checkSetWithoutGet && setters.length && !getters.length) {
                        reportList(setters, "missingGetter");
                    }
                    if (checkGetWithoutSet && getters.length && !setters.length) {
                        reportList(getters, "missingSetter");
                    }
                }
            }
            /**
             * Checks accessor pairs in an object literal.
             * @param {ASTNode} node `ObjectExpression` node to check.
             * @returns {void}
             * @private
             */
            function checkObjectLiteral(node) {
                checkList(node.properties.filter(p => p.type === "Property"));
            }
            /**
             * Checks accessor pairs in a property descriptor.
             * @param {ASTNode} node Property descriptor `ObjectExpression` node to check.
             * @returns {void}
             * @private
             */
            function checkPropertyDescriptor(node) {
                const namesToCheck = node.properties
                    .filter(p => p.type === "Property" && p.kind === "init" && !p.computed)
                    .map(({ key }) => key.name);
                const hasGetter = namesToCheck.includes("get");
                const hasSetter = namesToCheck.includes("set");
                if (checkSetWithoutGet && hasSetter && !hasGetter) {
                    report(node, "missingGetter");
                }
                if (checkGetWithoutSet && hasGetter && !hasSetter) {
                    report(node, "missingSetter");
                }
            }
            /**
             * Checks the given object expression as an object literal and as a possible property descriptor.
             * @param {ASTNode} node `ObjectExpression` node to check.
             * @returns {void}
             * @private
             */
            function checkObjectExpression(node) {
                checkObjectLiteral(node);
                if (isPropertyDescriptor(node)) {
                    checkPropertyDescriptor(node);
                }
            }
            /**
             * Checks the given class body.
             * @param {ASTNode} node `ClassBody` node to check.
             * @returns {void}
             * @private
             */
            function checkClassBody(node) {
                const methodDefinitions = node.body.filter(m => m.type === "MethodDefinition");
                checkList(methodDefinitions.filter(m => m.static));
                checkList(methodDefinitions.filter(m => !m.static));
            }
            const listeners = {};
            if (checkSetWithoutGet || checkGetWithoutSet) {
                listeners.ObjectExpression = checkObjectExpression;
                if (enforceForClassMembers) {
                    listeners.ClassBody = checkClassBody;
                }
            }
            return listeners;
        }
    };
},
/* eslint/lib/rules/utils/ast-utils.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const esutils = require(127) /* esutils */;
    const espree = require(102) /* espree */;
    const lodash = require(111) /* lodash */;
    const { breakableTypePattern, createGlobalLinebreakMatcher, lineBreakPattern, shebangPattern } = require(115) /* ../../shared/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const anyFunctionPattern = /^(?:Function(?:Declaration|Expression)|ArrowFunctionExpression)$/u;
    const anyLoopPattern = /^(?:DoWhile|For|ForIn|ForOf|While)Statement$/u;
    const arrayOrTypedArrayPattern = /Array$/u;
    const arrayMethodPattern = /^(?:every|filter|find|findIndex|forEach|map|some)$/u;
    const bindOrCallOrApplyPattern = /^(?:bind|call|apply)$/u;
    const thisTagPattern = /^[\s*]*@this/mu;
    const COMMENTS_IGNORE_PATTERN = /^\s*(?:eslint|jshint\s+|jslint\s+|istanbul\s+|globals?\s+|exported\s+|jscs)/u;
    const LINEBREAKS = new Set(["\r\n", "\r", "\n", "\u2028", "\u2029"]);
    // A set of node types that can contain a list of statements
    const STATEMENT_LIST_PARENTS = new Set(["Program", "BlockStatement", "SwitchCase"]);
    const DECIMAL_INTEGER_PATTERN = /^(0|[1-9]\d*)$/u;
    const OCTAL_ESCAPE_PATTERN = /^(?:[^\\]|\\[^0-7]|\\0(?![0-9]))*\\(?:[1-7]|0[0-9])/u;
    /**
     * Checks reference if is non initializer and writable.
     * @param {Reference} reference A reference to check.
     * @param {int} index The index of the reference in the references.
     * @param {Reference[]} references The array that the reference belongs to.
     * @returns {boolean} Success/Failure
     * @private
     */
    function isModifyingReference(reference, index, references) {
        const identifier = reference.identifier;
        /*
         * Destructuring assignments can have multiple default value, so
         * possibly there are multiple writeable references for the same
         * identifier.
         */
        const modifyingDifferentIdentifier = index === 0 ||
            references[index - 1].identifier !== identifier;
        return (identifier &&
            reference.init === false &&
            reference.isWrite() &&
            modifyingDifferentIdentifier);
    }
    /**
     * Checks whether the given string starts with uppercase or not.
     * @param {string} s The string to check.
     * @returns {boolean} `true` if the string starts with uppercase.
     */
    function startsWithUpperCase(s) {
        return s[0] !== s[0].toLocaleLowerCase();
    }
    /**
     * Checks whether or not a node is a constructor.
     * @param {ASTNode} node A function node to check.
     * @returns {boolean} Wehether or not a node is a constructor.
     */
    function isES5Constructor(node) {
        return (node.id && startsWithUpperCase(node.id.name));
    }
    /**
     * Finds a function node from ancestors of a node.
     * @param {ASTNode} node A start node to find.
     * @returns {Node|null} A found function node.
     */
    function getUpperFunction(node) {
        for (let currentNode = node; currentNode; currentNode = currentNode.parent) {
            if (anyFunctionPattern.test(currentNode.type)) {
                return currentNode;
            }
        }
        return null;
    }
    /**
     * Checks whether a given node is a function node or not.
     * The following types are function nodes:
     *
     * - ArrowFunctionExpression
     * - FunctionDeclaration
     * - FunctionExpression
     * @param {ASTNode|null} node A node to check.
     * @returns {boolean} `true` if the node is a function node.
     */
    function isFunction(node) {
        return Boolean(node && anyFunctionPattern.test(node.type));
    }
    /**
     * Checks whether a given node is a loop node or not.
     * The following types are loop nodes:
     *
     * - DoWhileStatement
     * - ForInStatement
     * - ForOfStatement
     * - ForStatement
     * - WhileStatement
     * @param {ASTNode|null} node A node to check.
     * @returns {boolean} `true` if the node is a loop node.
     */
    function isLoop(node) {
        return Boolean(node && anyLoopPattern.test(node.type));
    }
    /**
     * Checks whether the given node is in a loop or not.
     * @param {ASTNode} node The node to check.
     * @returns {boolean} `true` if the node is in a loop.
     */
    function isInLoop(node) {
        for (let currentNode = node; currentNode && !isFunction(currentNode); currentNode = currentNode.parent) {
            if (isLoop(currentNode)) {
                return true;
            }
        }
        return false;
    }
    /**
     * Checks whether or not a node is `null` or `undefined`.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} Whether or not the node is a `null` or `undefined`.
     * @public
     */
    function isNullOrUndefined(node) {
        return (module.exports.isNullLiteral(node) ||
            (node.type === "Identifier" && node.name === "undefined") ||
            (node.type === "UnaryExpression" && node.operator === "void"));
    }
    /**
     * Checks whether or not a node is callee.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} Whether or not the node is callee.
     */
    function isCallee(node) {
        return node.parent.type === "CallExpression" && node.parent.callee === node;
    }
    /**
     * Checks whether or not a node is `Reflect.apply`.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} Whether or not the node is a `Reflect.apply`.
     */
    function isReflectApply(node) {
        return (node.type === "MemberExpression" &&
            node.object.type === "Identifier" &&
            node.object.name === "Reflect" &&
            node.property.type === "Identifier" &&
            node.property.name === "apply" &&
            node.computed === false);
    }
    /**
     * Checks whether or not a node is `Array.from`.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} Whether or not the node is a `Array.from`.
     */
    function isArrayFromMethod(node) {
        return (node.type === "MemberExpression" &&
            node.object.type === "Identifier" &&
            arrayOrTypedArrayPattern.test(node.object.name) &&
            node.property.type === "Identifier" &&
            node.property.name === "from" &&
            node.computed === false);
    }
    /**
     * Checks whether or not a node is a method which has `thisArg`.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} Whether or not the node is a method which has `thisArg`.
     */
    function isMethodWhichHasThisArg(node) {
        for (let currentNode = node; currentNode.type === "MemberExpression" && !currentNode.computed; currentNode = currentNode.property) {
            if (currentNode.property.type === "Identifier") {
                return arrayMethodPattern.test(currentNode.property.name);
            }
        }
        return false;
    }
    /**
     * Creates the negate function of the given function.
     * @param {Function} f The function to negate.
     * @returns {Function} Negated function.
     */
    function negate(f) {
        return token => !f(token);
    }
    /**
     * Checks whether or not a node has a `@this` tag in its comments.
     * @param {ASTNode} node A node to check.
     * @param {SourceCode} sourceCode A SourceCode instance to get comments.
     * @returns {boolean} Whether or not the node has a `@this` tag in its comments.
     */
    function hasJSDocThisTag(node, sourceCode) {
        const jsdocComment = sourceCode.getJSDocComment(node);
        if (jsdocComment && thisTagPattern.test(jsdocComment.value)) {
            return true;
        }
        // Checks `@this` in its leading comments for callbacks,
        // because callbacks don't have its JSDoc comment.
        // e.g.
        //     sinon.test(/* @this sinon.Sandbox */function() { this.spy(); });
        return sourceCode.getCommentsBefore(node).some(comment => thisTagPattern.test(comment.value));
    }
    /**
     * Determines if a node is surrounded by parentheses.
     * @param {SourceCode} sourceCode The ESLint source code object
     * @param {ASTNode} node The node to be checked.
     * @returns {boolean} True if the node is parenthesised.
     * @private
     */
    function isParenthesised(sourceCode, node) {
        const previousToken = sourceCode.getTokenBefore(node), nextToken = sourceCode.getTokenAfter(node);
        return Boolean(previousToken && nextToken) &&
            previousToken.value === "(" && previousToken.range[1] <= node.range[0] &&
            nextToken.value === ")" && nextToken.range[0] >= node.range[1];
    }
    /**
     * Checks if the given token is an arrow token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is an arrow token.
     */
    function isArrowToken(token) {
        return token.value === "=>" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a comma token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is a comma token.
     */
    function isCommaToken(token) {
        return token.value === "," && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a dot token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is a dot token.
     */
    function isDotToken(token) {
        return token.value === "." && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a semicolon token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is a semicolon token.
     */
    function isSemicolonToken(token) {
        return token.value === ";" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a colon token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is a colon token.
     */
    function isColonToken(token) {
        return token.value === ":" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is an opening parenthesis token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is an opening parenthesis token.
     */
    function isOpeningParenToken(token) {
        return token.value === "(" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a closing parenthesis token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is a closing parenthesis token.
     */
    function isClosingParenToken(token) {
        return token.value === ")" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is an opening square bracket token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is an opening square bracket token.
     */
    function isOpeningBracketToken(token) {
        return token.value === "[" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a closing square bracket token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is a closing square bracket token.
     */
    function isClosingBracketToken(token) {
        return token.value === "]" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is an opening brace token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is an opening brace token.
     */
    function isOpeningBraceToken(token) {
        return token.value === "{" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a closing brace token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is a closing brace token.
     */
    function isClosingBraceToken(token) {
        return token.value === "}" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a comment token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is a comment token.
     */
    function isCommentToken(token) {
        return token.type === "Line" || token.type === "Block" || token.type === "Shebang";
    }
    /**
     * Checks if the given token is a keyword token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is a keyword token.
     */
    function isKeywordToken(token) {
        return token.type === "Keyword";
    }
    /**
     * Gets the `(` token of the given function node.
     * @param {ASTNode} node The function node to get.
     * @param {SourceCode} sourceCode The source code object to get tokens.
     * @returns {Token} `(` token.
     */
    function getOpeningParenOfParams(node, sourceCode) {
        return node.id
            ? sourceCode.getTokenAfter(node.id, isOpeningParenToken)
            : sourceCode.getFirstToken(node, isOpeningParenToken);
    }
    /**
     * Checks whether or not the tokens of two given nodes are same.
     * @param {ASTNode} left A node 1 to compare.
     * @param {ASTNode} right A node 2 to compare.
     * @param {SourceCode} sourceCode The ESLint source code object.
     * @returns {boolean} the source code for the given node.
     */
    function equalTokens(left, right, sourceCode) {
        const tokensL = sourceCode.getTokens(left);
        const tokensR = sourceCode.getTokens(right);
        if (tokensL.length !== tokensR.length) {
            return false;
        }
        for (let i = 0; i < tokensL.length; ++i) {
            if (tokensL[i].type !== tokensR[i].type ||
                tokensL[i].value !== tokensR[i].value) {
                return false;
            }
        }
        return true;
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    module.exports = {
        COMMENTS_IGNORE_PATTERN,
        LINEBREAKS,
        LINEBREAK_MATCHER: lineBreakPattern,
        SHEBANG_MATCHER: shebangPattern,
        STATEMENT_LIST_PARENTS,
        /**
         * Determines whether two adjacent tokens are on the same line.
         * @param {Object} left The left token object.
         * @param {Object} right The right token object.
         * @returns {boolean} Whether or not the tokens are on the same line.
         * @public
         */
        isTokenOnSameLine(left, right) {
            return left.loc.end.line === right.loc.start.line;
        },
        isNullOrUndefined,
        isCallee,
        isES5Constructor,
        getUpperFunction,
        isFunction,
        isLoop,
        isInLoop,
        isArrayFromMethod,
        isParenthesised,
        createGlobalLinebreakMatcher,
        equalTokens,
        isArrowToken,
        isClosingBraceToken,
        isClosingBracketToken,
        isClosingParenToken,
        isColonToken,
        isCommaToken,
        isCommentToken,
        isDotToken,
        isKeywordToken,
        isNotClosingBraceToken: negate(isClosingBraceToken),
        isNotClosingBracketToken: negate(isClosingBracketToken),
        isNotClosingParenToken: negate(isClosingParenToken),
        isNotColonToken: negate(isColonToken),
        isNotCommaToken: negate(isCommaToken),
        isNotDotToken: negate(isDotToken),
        isNotOpeningBraceToken: negate(isOpeningBraceToken),
        isNotOpeningBracketToken: negate(isOpeningBracketToken),
        isNotOpeningParenToken: negate(isOpeningParenToken),
        isNotSemicolonToken: negate(isSemicolonToken),
        isOpeningBraceToken,
        isOpeningBracketToken,
        isOpeningParenToken,
        isSemicolonToken,
        /**
         * Checks whether or not a given node is a string literal.
         * @param {ASTNode} node A node to check.
         * @returns {boolean} `true` if the node is a string literal.
         */
        isStringLiteral(node) {
            return ((node.type === "Literal" && typeof node.value === "string") ||
                node.type === "TemplateLiteral");
        },
        /**
         * Checks whether a given node is a breakable statement or not.
         * The node is breakable if the node is one of the following type:
         *
         * - DoWhileStatement
         * - ForInStatement
         * - ForOfStatement
         * - ForStatement
         * - SwitchStatement
         * - WhileStatement
         * @param {ASTNode} node A node to check.
         * @returns {boolean} `true` if the node is breakable.
         */
        isBreakableStatement(node) {
            return breakableTypePattern.test(node.type);
        },
        /**
         * Gets references which are non initializer and writable.
         * @param {Reference[]} references An array of references.
         * @returns {Reference[]} An array of only references which are non initializer and writable.
         * @public
         */
        getModifyingReferences(references) {
            return references.filter(isModifyingReference);
        },
        /**
         * Validate that a string passed in is surrounded by the specified character
         * @param  {string} val The text to check.
         * @param  {string} character The character to see if it's surrounded by.
         * @returns {boolean} True if the text is surrounded by the character, false if not.
         * @private
         */
        isSurroundedBy(val, character) {
            return val[0] === character && val[val.length - 1] === character;
        },
        /**
         * Returns whether the provided node is an ESLint directive comment or not
         * @param {Line|Block} node The comment token to be checked
         * @returns {boolean} `true` if the node is an ESLint directive comment
         */
        isDirectiveComment(node) {
            const comment = node.value.trim();
            return (node.type === "Line" && comment.indexOf("eslint-") === 0 ||
                node.type === "Block" && (comment.indexOf("global ") === 0 ||
                    comment.indexOf("eslint ") === 0 ||
                    comment.indexOf("eslint-") === 0));
        },
        /**
         * Gets the trailing statement of a given node.
         *
         *     if (code)
         *         consequent;
         *
         * When taking this `IfStatement`, returns `consequent;` statement.
         * @param {ASTNode} A node to get.
         * @returns {ASTNode|null} The trailing statement's node.
         */
        getTrailingStatement: esutils.ast.trailingStatement,
        /**
         * Finds the variable by a given name in a given scope and its upper scopes.
         * @param {eslint-scope.Scope} initScope A scope to start find.
         * @param {string} name A variable name to find.
         * @returns {eslint-scope.Variable|null} A found variable or `null`.
         */
        getVariableByName(initScope, name) {
            let scope = initScope;
            while (scope) {
                const variable = scope.set.get(name);
                if (variable) {
                    return variable;
                }
                scope = scope.upper;
            }
            return null;
        },
        /**
         * Checks whether or not a given function node is the default `this` binding.
         *
         * First, this checks the node:
         *
         * - The function name does not start with uppercase. It's a convention to capitalize the names
         *   of constructor functions. This check is not performed if `capIsConstructor` is set to `false`.
         * - The function does not have a JSDoc comment that has a @this tag.
         *
         * Next, this checks the location of the node.
         * If the location is below, this judges `this` is valid.
         *
         * - The location is not on an object literal.
         * - The location is not assigned to a variable which starts with an uppercase letter. Applies to anonymous
         *   functions only, as the name of the variable is considered to be the name of the function in this case.
         *   This check is not performed if `capIsConstructor` is set to `false`.
         * - The location is not on an ES2015 class.
         * - Its `bind`/`call`/`apply` method is not called directly.
         * - The function is not a callback of array methods (such as `.forEach()`) if `thisArg` is given.
         * @param {ASTNode} node A function node to check.
         * @param {SourceCode} sourceCode A SourceCode instance to get comments.
         * @param {boolean} [capIsConstructor = true] `false` disables the assumption that functions which name starts
         * with an uppercase or are assigned to a variable which name starts with an uppercase are constructors.
         * @returns {boolean} The function node is the default `this` binding.
         */
        isDefaultThisBinding(node, sourceCode, { capIsConstructor = true } = {}) {
            if ((capIsConstructor && isES5Constructor(node)) ||
                hasJSDocThisTag(node, sourceCode)) {
                return false;
            }
            const isAnonymous = node.id === null;
            let currentNode = node;
            while (currentNode) {
                const parent = currentNode.parent;
                switch (parent.type) {
                    /*
                     * Looks up the destination.
                     * e.g., obj.foo = nativeFoo || function foo() { ... };
                     */
                    case "LogicalExpression":
                    case "ConditionalExpression":
                        currentNode = parent;
                        break;
                    /*
                     * If the upper function is IIFE, checks the destination of the return value.
                     * e.g.
                     *   obj.foo = (function() {
                     *     // setup...
                     *     return function foo() { ... };
                     *   })();
                     *   obj.foo = (() =>
                     *     function foo() { ... }
                     *   )();
                     */
                    case "ReturnStatement": {
                        const func = getUpperFunction(parent);
                        if (func === null || !isCallee(func)) {
                            return true;
                        }
                        currentNode = func.parent;
                        break;
                    }
                    case "ArrowFunctionExpression":
                        if (currentNode !== parent.body || !isCallee(parent)) {
                            return true;
                        }
                        currentNode = parent.parent;
                        break;
                    /*
                     * e.g.
                     *   var obj = { foo() { ... } };
                     *   var obj = { foo: function() { ... } };
                     *   class A { constructor() { ... } }
                     *   class A { foo() { ... } }
                     *   class A { get foo() { ... } }
                     *   class A { set foo() { ... } }
                     *   class A { static foo() { ... } }
                     */
                    case "Property":
                    case "MethodDefinition":
                        return parent.value !== currentNode;
                    /*
                     * e.g.
                     *   obj.foo = function foo() { ... };
                     *   Foo = function() { ... };
                     *   [obj.foo = function foo() { ... }] = a;
                     *   [Foo = function() { ... }] = a;
                     */
                    case "AssignmentExpression":
                    case "AssignmentPattern":
                        if (parent.left.type === "MemberExpression") {
                            return false;
                        }
                        if (capIsConstructor &&
                            isAnonymous &&
                            parent.left.type === "Identifier" &&
                            startsWithUpperCase(parent.left.name)) {
                            return false;
                        }
                        return true;
                    /*
                     * e.g.
                     *   var Foo = function() { ... };
                     */
                    case "VariableDeclarator":
                        return !(capIsConstructor &&
                            isAnonymous &&
                            parent.init === currentNode &&
                            parent.id.type === "Identifier" &&
                            startsWithUpperCase(parent.id.name));
                    /*
                     * e.g.
                     *   var foo = function foo() { ... }.bind(obj);
                     *   (function foo() { ... }).call(obj);
                     *   (function foo() { ... }).apply(obj, []);
                     */
                    case "MemberExpression":
                        return (parent.object !== currentNode ||
                            parent.property.type !== "Identifier" ||
                            !bindOrCallOrApplyPattern.test(parent.property.name) ||
                            !isCallee(parent) ||
                            parent.parent.arguments.length === 0 ||
                            isNullOrUndefined(parent.parent.arguments[0]));
                    /*
                     * e.g.
                     *   Reflect.apply(function() {}, obj, []);
                     *   Array.from([], function() {}, obj);
                     *   list.forEach(function() {}, obj);
                     */
                    case "CallExpression":
                        if (isReflectApply(parent.callee)) {
                            return (parent.arguments.length !== 3 ||
                                parent.arguments[0] !== currentNode ||
                                isNullOrUndefined(parent.arguments[1]));
                        }
                        if (isArrayFromMethod(parent.callee)) {
                            return (parent.arguments.length !== 3 ||
                                parent.arguments[1] !== currentNode ||
                                isNullOrUndefined(parent.arguments[2]));
                        }
                        if (isMethodWhichHasThisArg(parent.callee)) {
                            return (parent.arguments.length !== 2 ||
                                parent.arguments[0] !== currentNode ||
                                isNullOrUndefined(parent.arguments[1]));
                        }
                        return true;
                    // Otherwise `this` is default.
                    default:
                        return true;
                }
            }
            /* istanbul ignore next */
            return true;
        },
        /**
         * Get the precedence level based on the node type
         * @param {ASTNode} node node to evaluate
         * @returns {int} precedence level
         * @private
         */
        getPrecedence(node) {
            switch (node.type) {
                case "SequenceExpression":
                    return 0;
                case "AssignmentExpression":
                case "ArrowFunctionExpression":
                case "YieldExpression":
                    return 1;
                case "ConditionalExpression":
                    return 3;
                case "LogicalExpression":
                    switch (node.operator) {
                        case "||":
                            return 4;
                        case "&&":
                            return 5;
                        // no default
                    }
                /* falls through */
                case "BinaryExpression":
                    switch (node.operator) {
                        case "|":
                            return 6;
                        case "^":
                            return 7;
                        case "&":
                            return 8;
                        case "==":
                        case "!=":
                        case "===":
                        case "!==":
                            return 9;
                        case "<":
                        case "<=":
                        case ">":
                        case ">=":
                        case "in":
                        case "instanceof":
                            return 10;
                        case "<<":
                        case ">>":
                        case ">>>":
                            return 11;
                        case "+":
                        case "-":
                            return 12;
                        case "*":
                        case "/":
                        case "%":
                            return 13;
                        case "**":
                            return 15;
                        // no default
                    }
                /* falls through */
                case "UnaryExpression":
                case "AwaitExpression":
                    return 16;
                case "UpdateExpression":
                    return 17;
                case "CallExpression":
                case "ImportExpression":
                    return 18;
                case "NewExpression":
                    return 19;
                default:
                    return 20;
            }
        },
        /**
         * Checks whether the given node is an empty block node or not.
         * @param {ASTNode|null} node The node to check.
         * @returns {boolean} `true` if the node is an empty block.
         */
        isEmptyBlock(node) {
            return Boolean(node && node.type === "BlockStatement" && node.body.length === 0);
        },
        /**
         * Checks whether the given node is an empty function node or not.
         * @param {ASTNode|null} node The node to check.
         * @returns {boolean} `true` if the node is an empty function.
         */
        isEmptyFunction(node) {
            return isFunction(node) && module.exports.isEmptyBlock(node.body);
        },
        /**
         * Gets the property name of a given node.
         * The node can be a MemberExpression, a Property, or a MethodDefinition.
         *
         * If the name is dynamic, this returns `null`.
         *
         * For examples:
         *
         *     a.b           // => "b"
         *     a["b"]        // => "b"
         *     a['b']        // => "b"
         *     a[`b`]        // => "b"
         *     a[100]        // => "100"
         *     a[b]          // => null
         *     a["a" + "b"]  // => null
         *     a[tag`b`]     // => null
         *     a[`${b}`]     // => null
         *
         *     let a = {b: 1}            // => "b"
         *     let a = {["b"]: 1}        // => "b"
         *     let a = {['b']: 1}        // => "b"
         *     let a = {[`b`]: 1}        // => "b"
         *     let a = {[100]: 1}        // => "100"
         *     let a = {[b]: 1}          // => null
         *     let a = {["a" + "b"]: 1}  // => null
         *     let a = {[tag`b`]: 1}     // => null
         *     let a = {[`${b}`]: 1}     // => null
         * @param {ASTNode} node The node to get.
         * @returns {string|null} The property name if static. Otherwise, null.
         */
        getStaticPropertyName(node) {
            let prop;
            switch (node && node.type) {
                case "Property":
                case "MethodDefinition":
                    prop = node.key;
                    break;
                case "MemberExpression":
                    prop = node.property;
                    break;
                // no default
            }
            switch (prop && prop.type) {
                case "Literal":
                    return String(prop.value);
                case "TemplateLiteral":
                    if (prop.expressions.length === 0 && prop.quasis.length === 1) {
                        return prop.quasis[0].value.cooked;
                    }
                    break;
                case "Identifier":
                    if (!node.computed) {
                        return prop.name;
                    }
                    break;
                // no default
            }
            return null;
        },
        /**
         * Get directives from directive prologue of a Program or Function node.
         * @param {ASTNode} node The node to check.
         * @returns {ASTNode[]} The directives found in the directive prologue.
         */
        getDirectivePrologue(node) {
            const directives = [];
            // Directive prologues only occur at the top of files or functions.
            if (node.type === "Program" ||
                node.type === "FunctionDeclaration" ||
                node.type === "FunctionExpression" ||
                /*
                 * Do not check arrow functions with implicit return.
                 * `() => "use strict";` returns the string `"use strict"`.
                 */
                (node.type === "ArrowFunctionExpression" && node.body.type === "BlockStatement")) {
                const statements = node.type === "Program" ? node.body : node.body.body;
                for (const statement of statements) {
                    if (statement.type === "ExpressionStatement" &&
                        statement.expression.type === "Literal") {
                        directives.push(statement);
                    }
                    else {
                        break;
                    }
                }
            }
            return directives;
        },
        /**
         * Determines whether this node is a decimal integer literal. If a node is a decimal integer literal, a dot added
         * after the node will be parsed as a decimal point, rather than a property-access dot.
         * @param {ASTNode} node The node to check.
         * @returns {boolean} `true` if this node is a decimal integer.
         * @example
         *
         * 5       // true
         * 5.      // false
         * 5.0     // false
         * 05      // false
         * 0x5     // false
         * 0b101   // false
         * 0o5     // false
         * 5e0     // false
         * '5'     // false
         */
        isDecimalInteger(node) {
            return node.type === "Literal" && typeof node.value === "number" &&
                DECIMAL_INTEGER_PATTERN.test(node.raw);
        },
        /**
         * Determines whether this token is a decimal integer numeric token.
         * This is similar to isDecimalInteger(), but for tokens.
         * @param {Token} token The token to check.
         * @returns {boolean} `true` if this token is a decimal integer.
         */
        isDecimalIntegerNumericToken(token) {
            return token.type === "Numeric" && DECIMAL_INTEGER_PATTERN.test(token.value);
        },
        /**
         * Gets the name and kind of the given function node.
         *
         * - `function foo() {}`  .................... `function 'foo'`
         * - `(function foo() {})`  .................. `function 'foo'`
         * - `(function() {})`  ...................... `function`
         * - `function* foo() {}`  ................... `generator function 'foo'`
         * - `(function* foo() {})`  ................. `generator function 'foo'`
         * - `(function*() {})`  ..................... `generator function`
         * - `() => {}`  ............................. `arrow function`
         * - `async () => {}`  ....................... `async arrow function`
         * - `({ foo: function foo() {} })`  ......... `method 'foo'`
         * - `({ foo: function() {} })`  ............. `method 'foo'`
         * - `({ ['foo']: function() {} })`  ......... `method 'foo'`
         * - `({ [foo]: function() {} })`  ........... `method`
         * - `({ foo() {} })`  ....................... `method 'foo'`
         * - `({ foo: function* foo() {} })`  ........ `generator method 'foo'`
         * - `({ foo: function*() {} })`  ............ `generator method 'foo'`
         * - `({ ['foo']: function*() {} })`  ........ `generator method 'foo'`
         * - `({ [foo]: function*() {} })`  .......... `generator method`
         * - `({ *foo() {} })`  ...................... `generator method 'foo'`
         * - `({ foo: async function foo() {} })`  ... `async method 'foo'`
         * - `({ foo: async function() {} })`  ....... `async method 'foo'`
         * - `({ ['foo']: async function() {} })`  ... `async method 'foo'`
         * - `({ [foo]: async function() {} })`  ..... `async method`
         * - `({ async foo() {} })`  ................. `async method 'foo'`
         * - `({ get foo() {} })`  ................... `getter 'foo'`
         * - `({ set foo(a) {} })`  .................. `setter 'foo'`
         * - `class A { constructor() {} }`  ......... `constructor`
         * - `class A { foo() {} }`  ................. `method 'foo'`
         * - `class A { *foo() {} }`  ................ `generator method 'foo'`
         * - `class A { async foo() {} }`  ........... `async method 'foo'`
         * - `class A { ['foo']() {} }`  ............. `method 'foo'`
         * - `class A { *['foo']() {} }`  ............ `generator method 'foo'`
         * - `class A { async ['foo']() {} }`  ....... `async method 'foo'`
         * - `class A { [foo]() {} }`  ............... `method`
         * - `class A { *[foo]() {} }`  .............. `generator method`
         * - `class A { async [foo]() {} }`  ......... `async method`
         * - `class A { get foo() {} }`  ............. `getter 'foo'`
         * - `class A { set foo(a) {} }`  ............ `setter 'foo'`
         * - `class A { static foo() {} }`  .......... `static method 'foo'`
         * - `class A { static *foo() {} }`  ......... `static generator method 'foo'`
         * - `class A { static async foo() {} }`  .... `static async method 'foo'`
         * - `class A { static get foo() {} }`  ...... `static getter 'foo'`
         * - `class A { static set foo(a) {} }`  ..... `static setter 'foo'`
         * @param {ASTNode} node The function node to get.
         * @returns {string} The name and kind of the function node.
         */
        getFunctionNameWithKind(node) {
            const parent = node.parent;
            const tokens = [];
            if (parent.type === "MethodDefinition" && parent.static) {
                tokens.push("static");
            }
            if (node.async) {
                tokens.push("async");
            }
            if (node.generator) {
                tokens.push("generator");
            }
            if (node.type === "ArrowFunctionExpression") {
                tokens.push("arrow", "function");
            }
            else if (parent.type === "Property" || parent.type === "MethodDefinition") {
                if (parent.kind === "constructor") {
                    return "constructor";
                }
                if (parent.kind === "get") {
                    tokens.push("getter");
                }
                else if (parent.kind === "set") {
                    tokens.push("setter");
                }
                else {
                    tokens.push("method");
                }
            }
            else {
                tokens.push("function");
            }
            if (node.id) {
                tokens.push(`'${node.id.name}'`);
            }
            else {
                const name = module.exports.getStaticPropertyName(parent);
                if (name !== null) {
                    tokens.push(`'${name}'`);
                }
            }
            return tokens.join(" ");
        },
        /**
         * Gets the location of the given function node for reporting.
         *
         * - `function foo() {}`
         *    ^^^^^^^^^^^^
         * - `(function foo() {})`
         *     ^^^^^^^^^^^^
         * - `(function() {})`
         *     ^^^^^^^^
         * - `function* foo() {}`
         *    ^^^^^^^^^^^^^
         * - `(function* foo() {})`
         *     ^^^^^^^^^^^^^
         * - `(function*() {})`
         *     ^^^^^^^^^
         * - `() => {}`
         *       ^^
         * - `async () => {}`
         *             ^^
         * - `({ foo: function foo() {} })`
         *       ^^^^^^^^^^^^^^^^^
         * - `({ foo: function() {} })`
         *       ^^^^^^^^^^^^^
         * - `({ ['foo']: function() {} })`
         *       ^^^^^^^^^^^^^^^^^
         * - `({ [foo]: function() {} })`
         *       ^^^^^^^^^^^^^^^
         * - `({ foo() {} })`
         *       ^^^
         * - `({ foo: function* foo() {} })`
         *       ^^^^^^^^^^^^^^^^^^
         * - `({ foo: function*() {} })`
         *       ^^^^^^^^^^^^^^
         * - `({ ['foo']: function*() {} })`
         *       ^^^^^^^^^^^^^^^^^^
         * - `({ [foo]: function*() {} })`
         *       ^^^^^^^^^^^^^^^^
         * - `({ *foo() {} })`
         *       ^^^^
         * - `({ foo: async function foo() {} })`
         *       ^^^^^^^^^^^^^^^^^^^^^^^
         * - `({ foo: async function() {} })`
         *       ^^^^^^^^^^^^^^^^^^^
         * - `({ ['foo']: async function() {} })`
         *       ^^^^^^^^^^^^^^^^^^^^^^^
         * - `({ [foo]: async function() {} })`
         *       ^^^^^^^^^^^^^^^^^^^^^
         * - `({ async foo() {} })`
         *       ^^^^^^^^^
         * - `({ get foo() {} })`
         *       ^^^^^^^
         * - `({ set foo(a) {} })`
         *       ^^^^^^^
         * - `class A { constructor() {} }`
         *              ^^^^^^^^^^^
         * - `class A { foo() {} }`
         *              ^^^
         * - `class A { *foo() {} }`
         *              ^^^^
         * - `class A { async foo() {} }`
         *              ^^^^^^^^^
         * - `class A { ['foo']() {} }`
         *              ^^^^^^^
         * - `class A { *['foo']() {} }`
         *              ^^^^^^^^
         * - `class A { async ['foo']() {} }`
         *              ^^^^^^^^^^^^^
         * - `class A { [foo]() {} }`
         *              ^^^^^
         * - `class A { *[foo]() {} }`
         *              ^^^^^^
         * - `class A { async [foo]() {} }`
         *              ^^^^^^^^^^^
         * - `class A { get foo() {} }`
         *              ^^^^^^^
         * - `class A { set foo(a) {} }`
         *              ^^^^^^^
         * - `class A { static foo() {} }`
         *              ^^^^^^^^^^
         * - `class A { static *foo() {} }`
         *              ^^^^^^^^^^^
         * - `class A { static async foo() {} }`
         *              ^^^^^^^^^^^^^^^^
         * - `class A { static get foo() {} }`
         *              ^^^^^^^^^^^^^^
         * - `class A { static set foo(a) {} }`
         *              ^^^^^^^^^^^^^^
         * @param {ASTNode} node The function node to get.
         * @param {SourceCode} sourceCode The source code object to get tokens.
         * @returns {string} The location of the function node for reporting.
         */
        getFunctionHeadLoc(node, sourceCode) {
            const parent = node.parent;
            let start = null;
            let end = null;
            if (node.type === "ArrowFunctionExpression") {
                const arrowToken = sourceCode.getTokenBefore(node.body, isArrowToken);
                start = arrowToken.loc.start;
                end = arrowToken.loc.end;
            }
            else if (parent.type === "Property" || parent.type === "MethodDefinition") {
                start = parent.loc.start;
                end = getOpeningParenOfParams(node, sourceCode).loc.start;
            }
            else {
                start = node.loc.start;
                end = getOpeningParenOfParams(node, sourceCode).loc.start;
            }
            return {
                start: Object.assign({}, start),
                end: Object.assign({}, end)
            };
        },
        /**
         * Gets next location when the result is not out of bound, otherwise returns null.
         * @param {SourceCode} sourceCode The sourceCode
         * @param {{line: number, column: number}} location The location
         * @returns {{line: number, column: number} | null} Next location
         */
        getNextLocation(sourceCode, location) {
            const index = sourceCode.getIndexFromLoc(location);
            // Avoid out of bound location
            if (index + 1 > sourceCode.text.length) {
                return null;
            }
            return sourceCode.getLocFromIndex(index + 1);
        },
        /**
         * Gets the parenthesized text of a node. This is similar to sourceCode.getText(node), but it also includes any parentheses
         * surrounding the node.
         * @param {SourceCode} sourceCode The source code object
         * @param {ASTNode} node An expression node
         * @returns {string} The text representing the node, with all surrounding parentheses included
         */
        getParenthesisedText(sourceCode, node) {
            let leftToken = sourceCode.getFirstToken(node);
            let rightToken = sourceCode.getLastToken(node);
            while (sourceCode.getTokenBefore(leftToken) &&
                sourceCode.getTokenBefore(leftToken).type === "Punctuator" &&
                sourceCode.getTokenBefore(leftToken).value === "(" &&
                sourceCode.getTokenAfter(rightToken) &&
                sourceCode.getTokenAfter(rightToken).type === "Punctuator" &&
                sourceCode.getTokenAfter(rightToken).value === ")") {
                leftToken = sourceCode.getTokenBefore(leftToken);
                rightToken = sourceCode.getTokenAfter(rightToken);
            }
            return sourceCode.getText().slice(leftToken.range[0], rightToken.range[1]);
        },
        /*
         * Determine if a node has a possiblity to be an Error object
         * @param  {ASTNode} node  ASTNode to check
         * @returns {boolean} True if there is a chance it contains an Error obj
         */
        couldBeError(node) {
            switch (node.type) {
                case "Identifier":
                case "CallExpression":
                case "NewExpression":
                case "MemberExpression":
                case "TaggedTemplateExpression":
                case "YieldExpression":
                case "AwaitExpression":
                    return true; // possibly an error object.
                case "AssignmentExpression":
                    return module.exports.couldBeError(node.right);
                case "SequenceExpression": {
                    const exprs = node.expressions;
                    return exprs.length !== 0 && module.exports.couldBeError(exprs[exprs.length - 1]);
                }
                case "LogicalExpression":
                    return module.exports.couldBeError(node.left) || module.exports.couldBeError(node.right);
                case "ConditionalExpression":
                    return module.exports.couldBeError(node.consequent) || module.exports.couldBeError(node.alternate);
                default:
                    return false;
            }
        },
        /**
         * Determines whether the given node is a `null` literal.
         * @param {ASTNode} node The node to check
         * @returns {boolean} `true` if the node is a `null` literal
         */
        isNullLiteral(node) {
            /*
             * Checking `node.value === null` does not guarantee that a literal is a null literal.
             * When parsing values that cannot be represented in the current environment (e.g. unicode
             * regexes in Node 4), `node.value` is set to `null` because it wouldn't be possible to
             * set `node.value` to a unicode regex. To make sure a literal is actually `null`, check
             * `node.regex` instead. Also see: https://github.com/eslint/eslint/issues/8020
             */
            return node.type === "Literal" && node.value === null && !node.regex && !node.bigint;
        },
        /**
         * Determines whether two tokens can safely be placed next to each other without merging into a single token
         * @param {Token|string} leftValue The left token. If this is a string, it will be tokenized and the last token will be used.
         * @param {Token|string} rightValue The right token. If this is a string, it will be tokenized and the first token will be used.
         * @returns {boolean} If the tokens cannot be safely placed next to each other, returns `false`. If the tokens can be placed
         * next to each other, behavior is undefined (although it should return `true` in most cases).
         */
        canTokensBeAdjacent(leftValue, rightValue) {
            let leftToken;
            if (typeof leftValue === "string") {
                const leftTokens = espree.tokenize(leftValue, { ecmaVersion: 2015 });
                leftToken = leftTokens[leftTokens.length - 1];
            }
            else {
                leftToken = leftValue;
            }
            const rightToken = typeof rightValue === "string" ? espree.tokenize(rightValue, { ecmaVersion: 2015 })[0] : rightValue;
            if (leftToken.type === "Punctuator" || rightToken.type === "Punctuator") {
                if (leftToken.type === "Punctuator" && rightToken.type === "Punctuator") {
                    const PLUS_TOKENS = new Set(["+", "++"]);
                    const MINUS_TOKENS = new Set(["-", "--"]);
                    return !(PLUS_TOKENS.has(leftToken.value) && PLUS_TOKENS.has(rightToken.value) ||
                        MINUS_TOKENS.has(leftToken.value) && MINUS_TOKENS.has(rightToken.value));
                }
                return true;
            }
            if (leftToken.type === "String" || rightToken.type === "String" ||
                leftToken.type === "Template" || rightToken.type === "Template") {
                return true;
            }
            if (leftToken.type !== "Numeric" && rightToken.type === "Numeric" && rightToken.value.startsWith(".")) {
                return true;
            }
            return false;
        },
        /**
         * Get the `loc` object of a given name in a `/*globals` directive comment.
         * @param {SourceCode} sourceCode The source code to convert index to loc.
         * @param {Comment} comment The `/*globals` directive comment which include the name.
         * @param {string} name The name to find.
         * @returns {SourceLocation} The `loc` object.
         */
        getNameLocationInGlobalDirectiveComment(sourceCode, comment, name) {
            const namePattern = new RegExp(`[\\s,]${lodash.escapeRegExp(name)}(?:$|[\\s,:])`, "gu");
            // To ignore the first text "global".
            namePattern.lastIndex = comment.value.indexOf("global") + 6;
            // Search a given variable name.
            const match = namePattern.exec(comment.value);
            // Convert the index to loc.
            return sourceCode.getLocFromIndex(comment.range[0] +
                "/*".length +
                (match ? match.index + 1 : 0));
        },
        /**
         * Determines whether the given raw string contains an octal escape sequence.
         *
         * "\1", "\2" ... "\7"
         * "\00", "\01" ... "\09"
         *
         * "\0", when not followed by a digit, is not an octal escape sequence.
         * @param {string} rawString A string in its raw representation.
         * @returns {boolean} `true` if the string contains at least one octal escape sequence.
         */
        hasOctalEscapeSequence(rawString) {
            return OCTAL_ESCAPE_PATTERN.test(rawString);
        }
    };
},
/* esutils/lib/utils.js */ function _(require, module, exports) {
    /*
      Copyright (C) 2013 Yusuke Suzuki <utatane.tea@gmail.com>
    
      Redistribution and use in source and binary forms, with or without
      modification, are permitted provided that the following conditions are met:
    
        * Redistributions of source code must retain the above copyright
          notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
          notice, this list of conditions and the following disclaimer in the
          documentation and/or other materials provided with the distribution.
    
      THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
      AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
      IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
      ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
      DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
      (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
      LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
      ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
      (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
      THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    */
    (function () {
        'use strict';
        exports.ast = require(128) /* ./ast */;
        exports.code = require(129) /* ./code */;
        exports.keyword = require(130) /* ./keyword */;
    }());
},
/* esutils/lib/ast.js */ function _(require, module, exports) {
    /*
      Copyright (C) 2013 Yusuke Suzuki <utatane.tea@gmail.com>
    
      Redistribution and use in source and binary forms, with or without
      modification, are permitted provided that the following conditions are met:
    
        * Redistributions of source code must retain the above copyright
          notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
          notice, this list of conditions and the following disclaimer in the
          documentation and/or other materials provided with the distribution.
    
      THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS'
      AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
      IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
      ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
      DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
      (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
      LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
      ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
      (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
      THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    */
    (function () {
        'use strict';
        function isExpression(node) {
            if (node == null) {
                return false;
            }
            switch (node.type) {
                case 'ArrayExpression':
                case 'AssignmentExpression':
                case 'BinaryExpression':
                case 'CallExpression':
                case 'ConditionalExpression':
                case 'FunctionExpression':
                case 'Identifier':
                case 'Literal':
                case 'LogicalExpression':
                case 'MemberExpression':
                case 'NewExpression':
                case 'ObjectExpression':
                case 'SequenceExpression':
                case 'ThisExpression':
                case 'UnaryExpression':
                case 'UpdateExpression':
                    return true;
            }
            return false;
        }
        function isIterationStatement(node) {
            if (node == null) {
                return false;
            }
            switch (node.type) {
                case 'DoWhileStatement':
                case 'ForInStatement':
                case 'ForStatement':
                case 'WhileStatement':
                    return true;
            }
            return false;
        }
        function isStatement(node) {
            if (node == null) {
                return false;
            }
            switch (node.type) {
                case 'BlockStatement':
                case 'BreakStatement':
                case 'ContinueStatement':
                case 'DebuggerStatement':
                case 'DoWhileStatement':
                case 'EmptyStatement':
                case 'ExpressionStatement':
                case 'ForInStatement':
                case 'ForStatement':
                case 'IfStatement':
                case 'LabeledStatement':
                case 'ReturnStatement':
                case 'SwitchStatement':
                case 'ThrowStatement':
                case 'TryStatement':
                case 'VariableDeclaration':
                case 'WhileStatement':
                case 'WithStatement':
                    return true;
            }
            return false;
        }
        function isSourceElement(node) {
            return isStatement(node) || node != null && node.type === 'FunctionDeclaration';
        }
        function trailingStatement(node) {
            switch (node.type) {
                case 'IfStatement':
                    if (node.alternate != null) {
                        return node.alternate;
                    }
                    return node.consequent;
                case 'LabeledStatement':
                case 'ForStatement':
                case 'ForInStatement':
                case 'WhileStatement':
                case 'WithStatement':
                    return node.body;
            }
            return null;
        }
        function isProblematicIfStatement(node) {
            var current;
            if (node.type !== 'IfStatement') {
                return false;
            }
            if (node.alternate == null) {
                return false;
            }
            current = node.consequent;
            do {
                if (current.type === 'IfStatement') {
                    if (current.alternate == null) {
                        return true;
                    }
                }
                current = trailingStatement(current);
            } while (current);
            return false;
        }
        module.exports = {
            isExpression: isExpression,
            isStatement: isStatement,
            isIterationStatement: isIterationStatement,
            isSourceElement: isSourceElement,
            isProblematicIfStatement: isProblematicIfStatement,
            trailingStatement: trailingStatement
        };
    }());
},
/* esutils/lib/code.js */ function _(require, module, exports) {
    /*
      Copyright (C) 2013-2014 Yusuke Suzuki <utatane.tea@gmail.com>
      Copyright (C) 2014 Ivan Nikulin <ifaaan@gmail.com>
    
      Redistribution and use in source and binary forms, with or without
      modification, are permitted provided that the following conditions are met:
    
        * Redistributions of source code must retain the above copyright
          notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
          notice, this list of conditions and the following disclaimer in the
          documentation and/or other materials provided with the distribution.
    
      THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
      AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
      IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
      ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
      DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
      (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
      LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
      ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
      (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
      THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    */
    (function () {
        'use strict';
        var ES6Regex, ES5Regex, NON_ASCII_WHITESPACES, IDENTIFIER_START, IDENTIFIER_PART, ch;
        // See `tools/generate-identifier-regex.js`.
        ES5Regex = {
            // ECMAScript 5.1/Unicode v9.0.0 NonAsciiIdentifierStart:
            NonAsciiIdentifierStart: /[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA\u05F0-\u05F2\u0620-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0840-\u0858\u08A0-\u08B4\u08B6-\u08BD\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971-\u0980\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0AF9\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D\u0C58-\u0C5A\u0C60\u0C61\u0C80\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0CF1\u0CF2\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D\u0D4E\u0D54-\u0D56\u0D5F-\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6\u0EDC-\u0EDF\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8C\u1000-\u102A\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081\u108E\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7\u17DC\u1820-\u1877\u1880-\u1884\u1887-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191E\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u1A00-\u1A16\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF\u1BBA-\u1BE5\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1C80-\u1C88\u1CE9-\u1CEC\u1CEE-\u1CF1\u1CF5\u1CF6\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F\u2090-\u209C\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2CF2\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2E2F\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303C\u3041-\u3096\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FD5\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B\uA640-\uA66E\uA67F-\uA69D\uA6A0-\uA6EF\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AE\uA7B0-\uA7B7\uA7F7-\uA801\uA803-\uA805\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB\uA8FD\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uA9E0-\uA9E4\uA9E6-\uA9EF\uA9FA-\uA9FE\uAA00-\uAA28\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA7E-\uAAAF\uAAB1\uAAB5\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uAAE0-\uAAEA\uAAF2-\uAAF4\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]/,
            // ECMAScript 5.1/Unicode v9.0.0 NonAsciiIdentifierPart:
            NonAsciiIdentifierPart: /[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0300-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u0483-\u0487\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u0591-\u05BD\u05BF\u05C1\u05C2\u05C4\u05C5\u05C7\u05D0-\u05EA\u05F0-\u05F2\u0610-\u061A\u0620-\u0669\u066E-\u06D3\u06D5-\u06DC\u06DF-\u06E8\u06EA-\u06FC\u06FF\u0710-\u074A\u074D-\u07B1\u07C0-\u07F5\u07FA\u0800-\u082D\u0840-\u085B\u08A0-\u08B4\u08B6-\u08BD\u08D4-\u08E1\u08E3-\u0963\u0966-\u096F\u0971-\u0983\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BC-\u09C4\u09C7\u09C8\u09CB-\u09CE\u09D7\u09DC\u09DD\u09DF-\u09E3\u09E6-\u09F1\u0A01-\u0A03\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A3C\u0A3E-\u0A42\u0A47\u0A48\u0A4B-\u0A4D\u0A51\u0A59-\u0A5C\u0A5E\u0A66-\u0A75\u0A81-\u0A83\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABC-\u0AC5\u0AC7-\u0AC9\u0ACB-\u0ACD\u0AD0\u0AE0-\u0AE3\u0AE6-\u0AEF\u0AF9\u0B01-\u0B03\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3C-\u0B44\u0B47\u0B48\u0B4B-\u0B4D\u0B56\u0B57\u0B5C\u0B5D\u0B5F-\u0B63\u0B66-\u0B6F\u0B71\u0B82\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BBE-\u0BC2\u0BC6-\u0BC8\u0BCA-\u0BCD\u0BD0\u0BD7\u0BE6-\u0BEF\u0C00-\u0C03\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D-\u0C44\u0C46-\u0C48\u0C4A-\u0C4D\u0C55\u0C56\u0C58-\u0C5A\u0C60-\u0C63\u0C66-\u0C6F\u0C80-\u0C83\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBC-\u0CC4\u0CC6-\u0CC8\u0CCA-\u0CCD\u0CD5\u0CD6\u0CDE\u0CE0-\u0CE3\u0CE6-\u0CEF\u0CF1\u0CF2\u0D01-\u0D03\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D-\u0D44\u0D46-\u0D48\u0D4A-\u0D4E\u0D54-\u0D57\u0D5F-\u0D63\u0D66-\u0D6F\u0D7A-\u0D7F\u0D82\u0D83\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0DCA\u0DCF-\u0DD4\u0DD6\u0DD8-\u0DDF\u0DE6-\u0DEF\u0DF2\u0DF3\u0E01-\u0E3A\u0E40-\u0E4E\u0E50-\u0E59\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB9\u0EBB-\u0EBD\u0EC0-\u0EC4\u0EC6\u0EC8-\u0ECD\u0ED0-\u0ED9\u0EDC-\u0EDF\u0F00\u0F18\u0F19\u0F20-\u0F29\u0F35\u0F37\u0F39\u0F3E-\u0F47\u0F49-\u0F6C\u0F71-\u0F84\u0F86-\u0F97\u0F99-\u0FBC\u0FC6\u1000-\u1049\u1050-\u109D\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u135D-\u135F\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1714\u1720-\u1734\u1740-\u1753\u1760-\u176C\u176E-\u1770\u1772\u1773\u1780-\u17D3\u17D7\u17DC\u17DD\u17E0-\u17E9\u180B-\u180D\u1810-\u1819\u1820-\u1877\u1880-\u18AA\u18B0-\u18F5\u1900-\u191E\u1920-\u192B\u1930-\u193B\u1946-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u19D0-\u19D9\u1A00-\u1A1B\u1A20-\u1A5E\u1A60-\u1A7C\u1A7F-\u1A89\u1A90-\u1A99\u1AA7\u1AB0-\u1ABD\u1B00-\u1B4B\u1B50-\u1B59\u1B6B-\u1B73\u1B80-\u1BF3\u1C00-\u1C37\u1C40-\u1C49\u1C4D-\u1C7D\u1C80-\u1C88\u1CD0-\u1CD2\u1CD4-\u1CF6\u1CF8\u1CF9\u1D00-\u1DF5\u1DFB-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u200C\u200D\u203F\u2040\u2054\u2071\u207F\u2090-\u209C\u20D0-\u20DC\u20E1\u20E5-\u20F0\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D7F-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2DE0-\u2DFF\u2E2F\u3005-\u3007\u3021-\u302F\u3031-\u3035\u3038-\u303C\u3041-\u3096\u3099\u309A\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FD5\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA62B\uA640-\uA66F\uA674-\uA67D\uA67F-\uA6F1\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AE\uA7B0-\uA7B7\uA7F7-\uA827\uA840-\uA873\uA880-\uA8C5\uA8D0-\uA8D9\uA8E0-\uA8F7\uA8FB\uA8FD\uA900-\uA92D\uA930-\uA953\uA960-\uA97C\uA980-\uA9C0\uA9CF-\uA9D9\uA9E0-\uA9FE\uAA00-\uAA36\uAA40-\uAA4D\uAA50-\uAA59\uAA60-\uAA76\uAA7A-\uAAC2\uAADB-\uAADD\uAAE0-\uAAEF\uAAF2-\uAAF6\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABEA\uABEC\uABED\uABF0-\uABF9\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE00-\uFE0F\uFE20-\uFE2F\uFE33\uFE34\uFE4D-\uFE4F\uFE70-\uFE74\uFE76-\uFEFC\uFF10-\uFF19\uFF21-\uFF3A\uFF3F\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]/
        };
        ES6Regex = {
            // ECMAScript 6/Unicode v9.0.0 NonAsciiIdentifierStart:
            NonAsciiIdentifierStart: /[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA\u05F0-\u05F2\u0620-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0840-\u0858\u08A0-\u08B4\u08B6-\u08BD\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971-\u0980\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0AF9\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D\u0C58-\u0C5A\u0C60\u0C61\u0C80\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0CF1\u0CF2\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D\u0D4E\u0D54-\u0D56\u0D5F-\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6\u0EDC-\u0EDF\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8C\u1000-\u102A\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081\u108E\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7\u17DC\u1820-\u1877\u1880-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191E\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u1A00-\u1A16\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF\u1BBA-\u1BE5\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1C80-\u1C88\u1CE9-\u1CEC\u1CEE-\u1CF1\u1CF5\u1CF6\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F\u2090-\u209C\u2102\u2107\u210A-\u2113\u2115\u2118-\u211D\u2124\u2126\u2128\u212A-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2CF2\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303C\u3041-\u3096\u309B-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FD5\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B\uA640-\uA66E\uA67F-\uA69D\uA6A0-\uA6EF\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AE\uA7B0-\uA7B7\uA7F7-\uA801\uA803-\uA805\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB\uA8FD\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uA9E0-\uA9E4\uA9E6-\uA9EF\uA9FA-\uA9FE\uAA00-\uAA28\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA7E-\uAAAF\uAAB1\uAAB5\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uAAE0-\uAAEA\uAAF2-\uAAF4\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDE80-\uDE9C\uDEA0-\uDED0\uDF00-\uDF1F\uDF30-\uDF4A\uDF50-\uDF75\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDCB0-\uDCD3\uDCD8-\uDCFB\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00\uDE10-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE4\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC03-\uDC37\uDC83-\uDCAF\uDCD0-\uDCE8\uDD03-\uDD26\uDD50-\uDD72\uDD76\uDD83-\uDDB2\uDDC1-\uDDC4\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE2B\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEDE\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3D\uDF50\uDF5D-\uDF61]|\uD805[\uDC00-\uDC34\uDC47-\uDC4A\uDC80-\uDCAF\uDCC4\uDCC5\uDCC7\uDD80-\uDDAE\uDDD8-\uDDDB\uDE00-\uDE2F\uDE44\uDE80-\uDEAA\uDF00-\uDF19]|\uD806[\uDCA0-\uDCDF\uDCFF\uDEC0-\uDEF8]|\uD807[\uDC00-\uDC08\uDC0A-\uDC2E\uDC40\uDC72-\uDC8F]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD81C-\uD820\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDED0-\uDEED\uDF00-\uDF2F\uDF40-\uDF43\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50\uDF93-\uDF9F\uDFE0]|\uD821[\uDC00-\uDFEC]|\uD822[\uDC00-\uDEF2]|\uD82C[\uDC00\uDC01]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB]|\uD83A[\uDC00-\uDCC4\uDD00-\uDD43]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1]|\uD87E[\uDC00-\uDE1D]/,
            // ECMAScript 6/Unicode v9.0.0 NonAsciiIdentifierPart:
            NonAsciiIdentifierPart: /[\xAA\xB5\xB7\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0300-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u0483-\u0487\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u0591-\u05BD\u05BF\u05C1\u05C2\u05C4\u05C5\u05C7\u05D0-\u05EA\u05F0-\u05F2\u0610-\u061A\u0620-\u0669\u066E-\u06D3\u06D5-\u06DC\u06DF-\u06E8\u06EA-\u06FC\u06FF\u0710-\u074A\u074D-\u07B1\u07C0-\u07F5\u07FA\u0800-\u082D\u0840-\u085B\u08A0-\u08B4\u08B6-\u08BD\u08D4-\u08E1\u08E3-\u0963\u0966-\u096F\u0971-\u0983\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BC-\u09C4\u09C7\u09C8\u09CB-\u09CE\u09D7\u09DC\u09DD\u09DF-\u09E3\u09E6-\u09F1\u0A01-\u0A03\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A3C\u0A3E-\u0A42\u0A47\u0A48\u0A4B-\u0A4D\u0A51\u0A59-\u0A5C\u0A5E\u0A66-\u0A75\u0A81-\u0A83\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABC-\u0AC5\u0AC7-\u0AC9\u0ACB-\u0ACD\u0AD0\u0AE0-\u0AE3\u0AE6-\u0AEF\u0AF9\u0B01-\u0B03\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3C-\u0B44\u0B47\u0B48\u0B4B-\u0B4D\u0B56\u0B57\u0B5C\u0B5D\u0B5F-\u0B63\u0B66-\u0B6F\u0B71\u0B82\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BBE-\u0BC2\u0BC6-\u0BC8\u0BCA-\u0BCD\u0BD0\u0BD7\u0BE6-\u0BEF\u0C00-\u0C03\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D-\u0C44\u0C46-\u0C48\u0C4A-\u0C4D\u0C55\u0C56\u0C58-\u0C5A\u0C60-\u0C63\u0C66-\u0C6F\u0C80-\u0C83\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBC-\u0CC4\u0CC6-\u0CC8\u0CCA-\u0CCD\u0CD5\u0CD6\u0CDE\u0CE0-\u0CE3\u0CE6-\u0CEF\u0CF1\u0CF2\u0D01-\u0D03\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D-\u0D44\u0D46-\u0D48\u0D4A-\u0D4E\u0D54-\u0D57\u0D5F-\u0D63\u0D66-\u0D6F\u0D7A-\u0D7F\u0D82\u0D83\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0DCA\u0DCF-\u0DD4\u0DD6\u0DD8-\u0DDF\u0DE6-\u0DEF\u0DF2\u0DF3\u0E01-\u0E3A\u0E40-\u0E4E\u0E50-\u0E59\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB9\u0EBB-\u0EBD\u0EC0-\u0EC4\u0EC6\u0EC8-\u0ECD\u0ED0-\u0ED9\u0EDC-\u0EDF\u0F00\u0F18\u0F19\u0F20-\u0F29\u0F35\u0F37\u0F39\u0F3E-\u0F47\u0F49-\u0F6C\u0F71-\u0F84\u0F86-\u0F97\u0F99-\u0FBC\u0FC6\u1000-\u1049\u1050-\u109D\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u135D-\u135F\u1369-\u1371\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1714\u1720-\u1734\u1740-\u1753\u1760-\u176C\u176E-\u1770\u1772\u1773\u1780-\u17D3\u17D7\u17DC\u17DD\u17E0-\u17E9\u180B-\u180D\u1810-\u1819\u1820-\u1877\u1880-\u18AA\u18B0-\u18F5\u1900-\u191E\u1920-\u192B\u1930-\u193B\u1946-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u19D0-\u19DA\u1A00-\u1A1B\u1A20-\u1A5E\u1A60-\u1A7C\u1A7F-\u1A89\u1A90-\u1A99\u1AA7\u1AB0-\u1ABD\u1B00-\u1B4B\u1B50-\u1B59\u1B6B-\u1B73\u1B80-\u1BF3\u1C00-\u1C37\u1C40-\u1C49\u1C4D-\u1C7D\u1C80-\u1C88\u1CD0-\u1CD2\u1CD4-\u1CF6\u1CF8\u1CF9\u1D00-\u1DF5\u1DFB-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u200C\u200D\u203F\u2040\u2054\u2071\u207F\u2090-\u209C\u20D0-\u20DC\u20E1\u20E5-\u20F0\u2102\u2107\u210A-\u2113\u2115\u2118-\u211D\u2124\u2126\u2128\u212A-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D7F-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2DE0-\u2DFF\u3005-\u3007\u3021-\u302F\u3031-\u3035\u3038-\u303C\u3041-\u3096\u3099-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FD5\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA62B\uA640-\uA66F\uA674-\uA67D\uA67F-\uA6F1\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AE\uA7B0-\uA7B7\uA7F7-\uA827\uA840-\uA873\uA880-\uA8C5\uA8D0-\uA8D9\uA8E0-\uA8F7\uA8FB\uA8FD\uA900-\uA92D\uA930-\uA953\uA960-\uA97C\uA980-\uA9C0\uA9CF-\uA9D9\uA9E0-\uA9FE\uAA00-\uAA36\uAA40-\uAA4D\uAA50-\uAA59\uAA60-\uAA76\uAA7A-\uAAC2\uAADB-\uAADD\uAAE0-\uAAEF\uAAF2-\uAAF6\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABEA\uABEC\uABED\uABF0-\uABF9\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE00-\uFE0F\uFE20-\uFE2F\uFE33\uFE34\uFE4D-\uFE4F\uFE70-\uFE74\uFE76-\uFEFC\uFF10-\uFF19\uFF21-\uFF3A\uFF3F\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDDFD\uDE80-\uDE9C\uDEA0-\uDED0\uDEE0\uDF00-\uDF1F\uDF30-\uDF4A\uDF50-\uDF7A\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDCA0-\uDCA9\uDCB0-\uDCD3\uDCD8-\uDCFB\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00-\uDE03\uDE05\uDE06\uDE0C-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE38-\uDE3A\uDE3F\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE6\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC00-\uDC46\uDC66-\uDC6F\uDC7F-\uDCBA\uDCD0-\uDCE8\uDCF0-\uDCF9\uDD00-\uDD34\uDD36-\uDD3F\uDD50-\uDD73\uDD76\uDD80-\uDDC4\uDDCA-\uDDCC\uDDD0-\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE37\uDE3E\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEEA\uDEF0-\uDEF9\uDF00-\uDF03\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3C-\uDF44\uDF47\uDF48\uDF4B-\uDF4D\uDF50\uDF57\uDF5D-\uDF63\uDF66-\uDF6C\uDF70-\uDF74]|\uD805[\uDC00-\uDC4A\uDC50-\uDC59\uDC80-\uDCC5\uDCC7\uDCD0-\uDCD9\uDD80-\uDDB5\uDDB8-\uDDC0\uDDD8-\uDDDD\uDE00-\uDE40\uDE44\uDE50-\uDE59\uDE80-\uDEB7\uDEC0-\uDEC9\uDF00-\uDF19\uDF1D-\uDF2B\uDF30-\uDF39]|\uD806[\uDCA0-\uDCE9\uDCFF\uDEC0-\uDEF8]|\uD807[\uDC00-\uDC08\uDC0A-\uDC36\uDC38-\uDC40\uDC50-\uDC59\uDC72-\uDC8F\uDC92-\uDCA7\uDCA9-\uDCB6]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD81C-\uD820\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDE60-\uDE69\uDED0-\uDEED\uDEF0-\uDEF4\uDF00-\uDF36\uDF40-\uDF43\uDF50-\uDF59\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50-\uDF7E\uDF8F-\uDF9F\uDFE0]|\uD821[\uDC00-\uDFEC]|\uD822[\uDC00-\uDEF2]|\uD82C[\uDC00\uDC01]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99\uDC9D\uDC9E]|\uD834[\uDD65-\uDD69\uDD6D-\uDD72\uDD7B-\uDD82\uDD85-\uDD8B\uDDAA-\uDDAD\uDE42-\uDE44]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB\uDFCE-\uDFFF]|\uD836[\uDE00-\uDE36\uDE3B-\uDE6C\uDE75\uDE84\uDE9B-\uDE9F\uDEA1-\uDEAF]|\uD838[\uDC00-\uDC06\uDC08-\uDC18\uDC1B-\uDC21\uDC23\uDC24\uDC26-\uDC2A]|\uD83A[\uDC00-\uDCC4\uDCD0-\uDCD6\uDD00-\uDD4A\uDD50-\uDD59]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1]|\uD87E[\uDC00-\uDE1D]|\uDB40[\uDD00-\uDDEF]/
        };
        function isDecimalDigit(ch) {
            return 0x30 <= ch && ch <= 0x39; // 0..9
        }
        function isHexDigit(ch) {
            return 0x30 <= ch && ch <= 0x39 || // 0..9
                0x61 <= ch && ch <= 0x66 || // a..f
                0x41 <= ch && ch <= 0x46; // A..F
        }
        function isOctalDigit(ch) {
            return ch >= 0x30 && ch <= 0x37; // 0..7
        }
        // 7.2 White Space
        NON_ASCII_WHITESPACES = [
            0x1680,
            0x2000, 0x2001, 0x2002, 0x2003, 0x2004, 0x2005, 0x2006, 0x2007, 0x2008, 0x2009, 0x200A,
            0x202F, 0x205F,
            0x3000,
            0xFEFF
        ];
        function isWhiteSpace(ch) {
            return ch === 0x20 || ch === 0x09 || ch === 0x0B || ch === 0x0C || ch === 0xA0 ||
                ch >= 0x1680 && NON_ASCII_WHITESPACES.indexOf(ch) >= 0;
        }
        // 7.3 Line Terminators
        function isLineTerminator(ch) {
            return ch === 0x0A || ch === 0x0D || ch === 0x2028 || ch === 0x2029;
        }
        // 7.6 Identifier Names and Identifiers
        function fromCodePoint(cp) {
            if (cp <= 0xFFFF) {
                return String.fromCharCode(cp);
            }
            var cu1 = String.fromCharCode(Math.floor((cp - 0x10000) / 0x400) + 0xD800);
            var cu2 = String.fromCharCode(((cp - 0x10000) % 0x400) + 0xDC00);
            return cu1 + cu2;
        }
        IDENTIFIER_START = new Array(0x80);
        for (ch = 0; ch < 0x80; ++ch) {
            IDENTIFIER_START[ch] =
                ch >= 0x61 && ch <= 0x7A || // a..z
                    ch >= 0x41 && ch <= 0x5A || // A..Z
                    ch === 0x24 || ch === 0x5F; // $ (dollar) and _ (underscore)
        }
        IDENTIFIER_PART = new Array(0x80);
        for (ch = 0; ch < 0x80; ++ch) {
            IDENTIFIER_PART[ch] =
                ch >= 0x61 && ch <= 0x7A || // a..z
                    ch >= 0x41 && ch <= 0x5A || // A..Z
                    ch >= 0x30 && ch <= 0x39 || // 0..9
                    ch === 0x24 || ch === 0x5F; // $ (dollar) and _ (underscore)
        }
        function isIdentifierStartES5(ch) {
            return ch < 0x80 ? IDENTIFIER_START[ch] : ES5Regex.NonAsciiIdentifierStart.test(fromCodePoint(ch));
        }
        function isIdentifierPartES5(ch) {
            return ch < 0x80 ? IDENTIFIER_PART[ch] : ES5Regex.NonAsciiIdentifierPart.test(fromCodePoint(ch));
        }
        function isIdentifierStartES6(ch) {
            return ch < 0x80 ? IDENTIFIER_START[ch] : ES6Regex.NonAsciiIdentifierStart.test(fromCodePoint(ch));
        }
        function isIdentifierPartES6(ch) {
            return ch < 0x80 ? IDENTIFIER_PART[ch] : ES6Regex.NonAsciiIdentifierPart.test(fromCodePoint(ch));
        }
        module.exports = {
            isDecimalDigit: isDecimalDigit,
            isHexDigit: isHexDigit,
            isOctalDigit: isOctalDigit,
            isWhiteSpace: isWhiteSpace,
            isLineTerminator: isLineTerminator,
            isIdentifierStartES5: isIdentifierStartES5,
            isIdentifierPartES5: isIdentifierPartES5,
            isIdentifierStartES6: isIdentifierStartES6,
            isIdentifierPartES6: isIdentifierPartES6
        };
    }());
},
/* esutils/lib/keyword.js */ function _(require, module, exports) {
    /*
      Copyright (C) 2013 Yusuke Suzuki <utatane.tea@gmail.com>
    
      Redistribution and use in source and binary forms, with or without
      modification, are permitted provided that the following conditions are met:
    
        * Redistributions of source code must retain the above copyright
          notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
          notice, this list of conditions and the following disclaimer in the
          documentation and/or other materials provided with the distribution.
    
      THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
      AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
      IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
      ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
      DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
      (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
      LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
      ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
      (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
      THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    */
    (function () {
        'use strict';
        var code = require(129) /* ./code */;
        function isStrictModeReservedWordES6(id) {
            switch (id) {
                case 'implements':
                case 'interface':
                case 'package':
                case 'private':
                case 'protected':
                case 'public':
                case 'static':
                case 'let':
                    return true;
                default:
                    return false;
            }
        }
        function isKeywordES5(id, strict) {
            // yield should not be treated as keyword under non-strict mode.
            if (!strict && id === 'yield') {
                return false;
            }
            return isKeywordES6(id, strict);
        }
        function isKeywordES6(id, strict) {
            if (strict && isStrictModeReservedWordES6(id)) {
                return true;
            }
            switch (id.length) {
                case 2:
                    return (id === 'if') || (id === 'in') || (id === 'do');
                case 3:
                    return (id === 'var') || (id === 'for') || (id === 'new') || (id === 'try');
                case 4:
                    return (id === 'this') || (id === 'else') || (id === 'case') ||
                        (id === 'void') || (id === 'with') || (id === 'enum');
                case 5:
                    return (id === 'while') || (id === 'break') || (id === 'catch') ||
                        (id === 'throw') || (id === 'const') || (id === 'yield') ||
                        (id === 'class') || (id === 'super');
                case 6:
                    return (id === 'return') || (id === 'typeof') || (id === 'delete') ||
                        (id === 'switch') || (id === 'export') || (id === 'import');
                case 7:
                    return (id === 'default') || (id === 'finally') || (id === 'extends');
                case 8:
                    return (id === 'function') || (id === 'continue') || (id === 'debugger');
                case 10:
                    return (id === 'instanceof');
                default:
                    return false;
            }
        }
        function isReservedWordES5(id, strict) {
            return id === 'null' || id === 'true' || id === 'false' || isKeywordES5(id, strict);
        }
        function isReservedWordES6(id, strict) {
            return id === 'null' || id === 'true' || id === 'false' || isKeywordES6(id, strict);
        }
        function isRestrictedWord(id) {
            return id === 'eval' || id === 'arguments';
        }
        function isIdentifierNameES5(id) {
            var i, iz, ch;
            if (id.length === 0) {
                return false;
            }
            ch = id.charCodeAt(0);
            if (!code.isIdentifierStartES5(ch)) {
                return false;
            }
            for (i = 1, iz = id.length; i < iz; ++i) {
                ch = id.charCodeAt(i);
                if (!code.isIdentifierPartES5(ch)) {
                    return false;
                }
            }
            return true;
        }
        function decodeUtf16(lead, trail) {
            return (lead - 0xD800) * 0x400 + (trail - 0xDC00) + 0x10000;
        }
        function isIdentifierNameES6(id) {
            var i, iz, ch, lowCh, check;
            if (id.length === 0) {
                return false;
            }
            check = code.isIdentifierStartES6;
            for (i = 0, iz = id.length; i < iz; ++i) {
                ch = id.charCodeAt(i);
                if (0xD800 <= ch && ch <= 0xDBFF) {
                    ++i;
                    if (i >= iz) {
                        return false;
                    }
                    lowCh = id.charCodeAt(i);
                    if (!(0xDC00 <= lowCh && lowCh <= 0xDFFF)) {
                        return false;
                    }
                    ch = decodeUtf16(ch, lowCh);
                }
                if (!check(ch)) {
                    return false;
                }
                check = code.isIdentifierPartES6;
            }
            return true;
        }
        function isIdentifierES5(id, strict) {
            return isIdentifierNameES5(id) && !isReservedWordES5(id, strict);
        }
        function isIdentifierES6(id, strict) {
            return isIdentifierNameES6(id) && !isReservedWordES6(id, strict);
        }
        module.exports = {
            isKeywordES5: isKeywordES5,
            isKeywordES6: isKeywordES6,
            isReservedWordES5: isReservedWordES5,
            isReservedWordES6: isReservedWordES6,
            isRestrictedWord: isRestrictedWord,
            isIdentifierNameES5: isIdentifierNameES5,
            isIdentifierNameES6: isIdentifierNameES6,
            isIdentifierES5: isIdentifierES5,
            isIdentifierES6: isIdentifierES6
        };
    }());
},
/* eslint/lib/rules/array-bracket-newline.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce linebreaks after opening and before closing array brackets",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/array-bracket-newline"
            },
            fixable: "whitespace",
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["always", "never", "consistent"]
                        },
                        {
                            type: "object",
                            properties: {
                                multiline: {
                                    type: "boolean"
                                },
                                minItems: {
                                    type: ["integer", "null"],
                                    minimum: 0
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                unexpectedOpeningLinebreak: "There should be no linebreak after '['.",
                unexpectedClosingLinebreak: "There should be no linebreak before ']'.",
                missingOpeningLinebreak: "A linebreak is required after '['.",
                missingClosingLinebreak: "A linebreak is required before ']'."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            //----------------------------------------------------------------------
            // Helpers
            //----------------------------------------------------------------------
            /**
             * Normalizes a given option value.
             * @param {string|Object|undefined} option An option value to parse.
             * @returns {{multiline: boolean, minItems: number}} Normalized option object.
             */
            function normalizeOptionValue(option) {
                let consistent = false;
                let multiline = false;
                let minItems = 0;
                if (option) {
                    if (option === "consistent") {
                        consistent = true;
                        minItems = Number.POSITIVE_INFINITY;
                    }
                    else if (option === "always" || option.minItems === 0) {
                        minItems = 0;
                    }
                    else if (option === "never") {
                        minItems = Number.POSITIVE_INFINITY;
                    }
                    else {
                        multiline = Boolean(option.multiline);
                        minItems = option.minItems || Number.POSITIVE_INFINITY;
                    }
                }
                else {
                    consistent = false;
                    multiline = true;
                    minItems = Number.POSITIVE_INFINITY;
                }
                return { consistent, multiline, minItems };
            }
            /**
             * Normalizes a given option value.
             * @param {string|Object|undefined} options An option value to parse.
             * @returns {{ArrayExpression: {multiline: boolean, minItems: number}, ArrayPattern: {multiline: boolean, minItems: number}}} Normalized option object.
             */
            function normalizeOptions(options) {
                const value = normalizeOptionValue(options);
                return { ArrayExpression: value, ArrayPattern: value };
            }
            /**
             * Reports that there shouldn't be a linebreak after the first token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportNoBeginningLinebreak(node, token) {
                context.report({
                    node,
                    loc: token.loc,
                    messageId: "unexpectedOpeningLinebreak",
                    fix(fixer) {
                        const nextToken = sourceCode.getTokenAfter(token, { includeComments: true });
                        if (astUtils.isCommentToken(nextToken)) {
                            return null;
                        }
                        return fixer.removeRange([token.range[1], nextToken.range[0]]);
                    }
                });
            }
            /**
             * Reports that there shouldn't be a linebreak before the last token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportNoEndingLinebreak(node, token) {
                context.report({
                    node,
                    loc: token.loc,
                    messageId: "unexpectedClosingLinebreak",
                    fix(fixer) {
                        const previousToken = sourceCode.getTokenBefore(token, { includeComments: true });
                        if (astUtils.isCommentToken(previousToken)) {
                            return null;
                        }
                        return fixer.removeRange([previousToken.range[1], token.range[0]]);
                    }
                });
            }
            /**
             * Reports that there should be a linebreak after the first token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportRequiredBeginningLinebreak(node, token) {
                context.report({
                    node,
                    loc: token.loc,
                    messageId: "missingOpeningLinebreak",
                    fix(fixer) {
                        return fixer.insertTextAfter(token, "\n");
                    }
                });
            }
            /**
             * Reports that there should be a linebreak before the last token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportRequiredEndingLinebreak(node, token) {
                context.report({
                    node,
                    loc: token.loc,
                    messageId: "missingClosingLinebreak",
                    fix(fixer) {
                        return fixer.insertTextBefore(token, "\n");
                    }
                });
            }
            /**
             * Reports a given node if it violated this rule.
             * @param {ASTNode} node A node to check. This is an ArrayExpression node or an ArrayPattern node.
             * @returns {void}
             */
            function check(node) {
                const elements = node.elements;
                const normalizedOptions = normalizeOptions(context.options[0]);
                const options = normalizedOptions[node.type];
                const openBracket = sourceCode.getFirstToken(node);
                const closeBracket = sourceCode.getLastToken(node);
                const firstIncComment = sourceCode.getTokenAfter(openBracket, { includeComments: true });
                const lastIncComment = sourceCode.getTokenBefore(closeBracket, { includeComments: true });
                const first = sourceCode.getTokenAfter(openBracket);
                const last = sourceCode.getTokenBefore(closeBracket);
                const needsLinebreaks = (elements.length >= options.minItems ||
                    (options.multiline &&
                        elements.length > 0 &&
                        firstIncComment.loc.start.line !== lastIncComment.loc.end.line) ||
                    (elements.length === 0 &&
                        firstIncComment.type === "Block" &&
                        firstIncComment.loc.start.line !== lastIncComment.loc.end.line &&
                        firstIncComment === lastIncComment) ||
                    (options.consistent &&
                        openBracket.loc.end.line !== first.loc.start.line));
                /*
                 * Use tokens or comments to check multiline or not.
                 * But use only tokens to check whether linebreaks are needed.
                 * This allows:
                 *     var arr = [ // eslint-disable-line foo
                 *         'a'
                 *     ]
                 */
                if (needsLinebreaks) {
                    if (astUtils.isTokenOnSameLine(openBracket, first)) {
                        reportRequiredBeginningLinebreak(node, openBracket);
                    }
                    if (astUtils.isTokenOnSameLine(last, closeBracket)) {
                        reportRequiredEndingLinebreak(node, closeBracket);
                    }
                }
                else {
                    if (!astUtils.isTokenOnSameLine(openBracket, first)) {
                        reportNoBeginningLinebreak(node, openBracket);
                    }
                    if (!astUtils.isTokenOnSameLine(last, closeBracket)) {
                        reportNoEndingLinebreak(node, closeBracket);
                    }
                }
            }
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                ArrayPattern: check,
                ArrayExpression: check
            };
        }
    };
},
/* eslint/lib/rules/array-bracket-spacing.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing inside array brackets",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/array-bracket-spacing"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["always", "never"]
                },
                {
                    type: "object",
                    properties: {
                        singleValue: {
                            type: "boolean"
                        },
                        objectsInArrays: {
                            type: "boolean"
                        },
                        arraysInArrays: {
                            type: "boolean"
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpectedSpaceAfter: "There should be no space after '{{tokenValue}}'.",
                unexpectedSpaceBefore: "There should be no space before '{{tokenValue}}'.",
                missingSpaceAfter: "A space is required after '{{tokenValue}}'.",
                missingSpaceBefore: "A space is required before '{{tokenValue}}'."
            }
        },
        create(context) {
            const spaced = context.options[0] === "always", sourceCode = context.getSourceCode();
            /**
             * Determines whether an option is set, relative to the spacing option.
             * If spaced is "always", then check whether option is set to false.
             * If spaced is "never", then check whether option is set to true.
             * @param {Object} option The option to exclude.
             * @returns {boolean} Whether or not the property is excluded.
             */
            function isOptionSet(option) {
                return context.options[1] ? context.options[1][option] === !spaced : false;
            }
            const options = {
                spaced,
                singleElementException: isOptionSet("singleValue"),
                objectsInArraysException: isOptionSet("objectsInArrays"),
                arraysInArraysException: isOptionSet("arraysInArrays")
            };
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Reports that there shouldn't be a space after the first token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportNoBeginningSpace(node, token) {
                const nextToken = sourceCode.getTokenAfter(token);
                context.report({
                    node,
                    loc: { start: token.loc.end, end: nextToken.loc.start },
                    messageId: "unexpectedSpaceAfter",
                    data: {
                        tokenValue: token.value
                    },
                    fix(fixer) {
                        return fixer.removeRange([token.range[1], nextToken.range[0]]);
                    }
                });
            }
            /**
             * Reports that there shouldn't be a space before the last token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportNoEndingSpace(node, token) {
                const previousToken = sourceCode.getTokenBefore(token);
                context.report({
                    node,
                    loc: { start: previousToken.loc.end, end: token.loc.start },
                    messageId: "unexpectedSpaceBefore",
                    data: {
                        tokenValue: token.value
                    },
                    fix(fixer) {
                        return fixer.removeRange([previousToken.range[1], token.range[0]]);
                    }
                });
            }
            /**
             * Reports that there should be a space after the first token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportRequiredBeginningSpace(node, token) {
                context.report({
                    node,
                    loc: token.loc,
                    messageId: "missingSpaceAfter",
                    data: {
                        tokenValue: token.value
                    },
                    fix(fixer) {
                        return fixer.insertTextAfter(token, " ");
                    }
                });
            }
            /**
             * Reports that there should be a space before the last token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportRequiredEndingSpace(node, token) {
                context.report({
                    node,
                    loc: token.loc,
                    messageId: "missingSpaceBefore",
                    data: {
                        tokenValue: token.value
                    },
                    fix(fixer) {
                        return fixer.insertTextBefore(token, " ");
                    }
                });
            }
            /**
             * Determines if a node is an object type
             * @param {ASTNode} node The node to check.
             * @returns {boolean} Whether or not the node is an object type.
             */
            function isObjectType(node) {
                return node && (node.type === "ObjectExpression" || node.type === "ObjectPattern");
            }
            /**
             * Determines if a node is an array type
             * @param {ASTNode} node The node to check.
             * @returns {boolean} Whether or not the node is an array type.
             */
            function isArrayType(node) {
                return node && (node.type === "ArrayExpression" || node.type === "ArrayPattern");
            }
            /**
             * Validates the spacing around array brackets
             * @param {ASTNode} node The node we're checking for spacing
             * @returns {void}
             */
            function validateArraySpacing(node) {
                if (options.spaced && node.elements.length === 0) {
                    return;
                }
                const first = sourceCode.getFirstToken(node), second = sourceCode.getFirstToken(node, 1), last = node.typeAnnotation
                    ? sourceCode.getTokenBefore(node.typeAnnotation)
                    : sourceCode.getLastToken(node), penultimate = sourceCode.getTokenBefore(last), firstElement = node.elements[0], lastElement = node.elements[node.elements.length - 1];
                const openingBracketMustBeSpaced = options.objectsInArraysException && isObjectType(firstElement) ||
                    options.arraysInArraysException && isArrayType(firstElement) ||
                    options.singleElementException && node.elements.length === 1
                    ? !options.spaced : options.spaced;
                const closingBracketMustBeSpaced = options.objectsInArraysException && isObjectType(lastElement) ||
                    options.arraysInArraysException && isArrayType(lastElement) ||
                    options.singleElementException && node.elements.length === 1
                    ? !options.spaced : options.spaced;
                if (astUtils.isTokenOnSameLine(first, second)) {
                    if (openingBracketMustBeSpaced && !sourceCode.isSpaceBetweenTokens(first, second)) {
                        reportRequiredBeginningSpace(node, first);
                    }
                    if (!openingBracketMustBeSpaced && sourceCode.isSpaceBetweenTokens(first, second)) {
                        reportNoBeginningSpace(node, first);
                    }
                }
                if (first !== penultimate && astUtils.isTokenOnSameLine(penultimate, last)) {
                    if (closingBracketMustBeSpaced && !sourceCode.isSpaceBetweenTokens(penultimate, last)) {
                        reportRequiredEndingSpace(node, last);
                    }
                    if (!closingBracketMustBeSpaced && sourceCode.isSpaceBetweenTokens(penultimate, last)) {
                        reportNoEndingSpace(node, last);
                    }
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                ArrayPattern: validateArraySpacing,
                ArrayExpression: validateArraySpacing
            };
        }
    };
},
/* eslint/lib/rules/array-callback-return.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const TARGET_NODE_TYPE = /^(?:Arrow)?FunctionExpression$/u;
    const TARGET_METHODS = /^(?:every|filter|find(?:Index)?|map|reduce(?:Right)?|some|sort)$/u;
    /**
     * Checks a given code path segment is reachable.
     * @param {CodePathSegment} segment A segment to check.
     * @returns {boolean} `true` if the segment is reachable.
     */
    function isReachable(segment) {
        return segment.reachable;
    }
    /**
     * Gets a readable location.
     *
     * - FunctionExpression -> the function name or `function` keyword.
     * - ArrowFunctionExpression -> `=>` token.
     * @param {ASTNode} node A function node to get.
     * @param {SourceCode} sourceCode A source code to get tokens.
     * @returns {ASTNode|Token} The node or the token of a location.
     */
    function getLocation(node, sourceCode) {
        if (node.type === "ArrowFunctionExpression") {
            return sourceCode.getTokenBefore(node.body);
        }
        return node.id || node;
    }
    /**
     * Checks a given node is a MemberExpression node which has the specified name's
     * property.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node is a MemberExpression node which has
     *      the specified name's property
     */
    function isTargetMethod(node) {
        return (node.type === "MemberExpression" &&
            TARGET_METHODS.test(astUtils.getStaticPropertyName(node) || ""));
    }
    /**
     * Checks whether or not a given node is a function expression which is the
     * callback of an array method.
     * @param {ASTNode} node A node to check. This is one of
     *      FunctionExpression or ArrowFunctionExpression.
     * @returns {boolean} `true` if the node is the callback of an array method.
     */
    function isCallbackOfArrayMethod(node) {
        let currentNode = node;
        while (currentNode) {
            const parent = currentNode.parent;
            switch (parent.type) {
                /*
                 * Looks up the destination. e.g.,
                 * foo.every(nativeFoo || function foo() { ... });
                 */
                case "LogicalExpression":
                case "ConditionalExpression":
                    currentNode = parent;
                    break;
                /*
                 * If the upper function is IIFE, checks the destination of the return value.
                 * e.g.
                 *   foo.every((function() {
                 *     // setup...
                 *     return function callback() { ... };
                 *   })());
                 */
                case "ReturnStatement": {
                    const func = astUtils.getUpperFunction(parent);
                    if (func === null || !astUtils.isCallee(func)) {
                        return false;
                    }
                    currentNode = func.parent;
                    break;
                }
                /*
                 * e.g.
                 *   Array.from([], function() {});
                 *   list.every(function() {});
                 */
                case "CallExpression":
                    if (astUtils.isArrayFromMethod(parent.callee)) {
                        return (parent.arguments.length >= 2 &&
                            parent.arguments[1] === currentNode);
                    }
                    if (isTargetMethod(parent.callee)) {
                        return (parent.arguments.length >= 1 &&
                            parent.arguments[0] === currentNode);
                    }
                    return false;
                // Otherwise this node is not target.
                default:
                    return false;
            }
        }
        /* istanbul ignore next: unreachable */
        return false;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "enforce `return` statements in callbacks of array methods",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/array-callback-return"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allowImplicit: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                expectedAtEnd: "Expected to return a value at the end of {{name}}.",
                expectedInside: "Expected to return a value in {{name}}.",
                expectedReturnValue: "{{name}} expected a return value."
            }
        },
        create(context) {
            const options = context.options[0] || { allowImplicit: false };
            let funcInfo = {
                upper: null,
                codePath: null,
                hasReturn: false,
                shouldCheck: false,
                node: null
            };
            /**
             * Checks whether or not the last code path segment is reachable.
             * Then reports this function if the segment is reachable.
             *
             * If the last code path segment is reachable, there are paths which are not
             * returned or thrown.
             * @param {ASTNode} node A node to check.
             * @returns {void}
             */
            function checkLastSegment(node) {
                if (funcInfo.shouldCheck &&
                    funcInfo.codePath.currentSegments.some(isReachable)) {
                    context.report({
                        node,
                        loc: getLocation(node, context.getSourceCode()).loc.start,
                        messageId: funcInfo.hasReturn
                            ? "expectedAtEnd"
                            : "expectedInside",
                        data: {
                            name: astUtils.getFunctionNameWithKind(funcInfo.node)
                        }
                    });
                }
            }
            return {
                // Stacks this function's information.
                onCodePathStart(codePath, node) {
                    funcInfo = {
                        upper: funcInfo,
                        codePath,
                        hasReturn: false,
                        shouldCheck: TARGET_NODE_TYPE.test(node.type) &&
                            node.body.type === "BlockStatement" &&
                            isCallbackOfArrayMethod(node) &&
                            !node.async &&
                            !node.generator,
                        node
                    };
                },
                // Pops this function's information.
                onCodePathEnd() {
                    funcInfo = funcInfo.upper;
                },
                // Checks the return statement is valid.
                ReturnStatement(node) {
                    if (funcInfo.shouldCheck) {
                        funcInfo.hasReturn = true;
                        // if allowImplicit: false, should also check node.argument
                        if (!options.allowImplicit && !node.argument) {
                            context.report({
                                node,
                                messageId: "expectedReturnValue",
                                data: {
                                    name: lodash.upperFirst(astUtils.getFunctionNameWithKind(funcInfo.node))
                                }
                            });
                        }
                    }
                },
                // Reports a given function if the last path is reachable.
                "FunctionExpression:exit": checkLastSegment,
                "ArrowFunctionExpression:exit": checkLastSegment
            };
        }
    };
},
/* eslint/lib/rules/array-element-newline.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce line breaks after each array element",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/array-element-newline"
            },
            fixable: "whitespace",
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["always", "never", "consistent"]
                        },
                        {
                            type: "object",
                            properties: {
                                multiline: {
                                    type: "boolean"
                                },
                                minItems: {
                                    type: ["integer", "null"],
                                    minimum: 0
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                unexpectedLineBreak: "There should be no linebreak here.",
                missingLineBreak: "There should be a linebreak after this element."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            //----------------------------------------------------------------------
            // Helpers
            //----------------------------------------------------------------------
            /**
             * Normalizes a given option value.
             * @param {string|Object|undefined} providedOption An option value to parse.
             * @returns {{multiline: boolean, minItems: number}} Normalized option object.
             */
            function normalizeOptionValue(providedOption) {
                let consistent = false;
                let multiline = false;
                let minItems;
                const option = providedOption || "always";
                if (!option || option === "always" || option.minItems === 0) {
                    minItems = 0;
                }
                else if (option === "never") {
                    minItems = Number.POSITIVE_INFINITY;
                }
                else if (option === "consistent") {
                    consistent = true;
                    minItems = Number.POSITIVE_INFINITY;
                }
                else {
                    multiline = Boolean(option.multiline);
                    minItems = option.minItems || Number.POSITIVE_INFINITY;
                }
                return { consistent, multiline, minItems };
            }
            /**
             * Normalizes a given option value.
             * @param {string|Object|undefined} options An option value to parse.
             * @returns {{ArrayExpression: {multiline: boolean, minItems: number}, ArrayPattern: {multiline: boolean, minItems: number}}} Normalized option object.
             */
            function normalizeOptions(options) {
                const value = normalizeOptionValue(options);
                return { ArrayExpression: value, ArrayPattern: value };
            }
            /**
             * Reports that there shouldn't be a line break after the first token
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportNoLineBreak(token) {
                const tokenBefore = sourceCode.getTokenBefore(token, { includeComments: true });
                context.report({
                    loc: {
                        start: tokenBefore.loc.end,
                        end: token.loc.start
                    },
                    messageId: "unexpectedLineBreak",
                    fix(fixer) {
                        if (astUtils.isCommentToken(tokenBefore)) {
                            return null;
                        }
                        if (!astUtils.isTokenOnSameLine(tokenBefore, token)) {
                            return fixer.replaceTextRange([tokenBefore.range[1], token.range[0]], " ");
                        }
                        /*
                         * This will check if the comma is on the same line as the next element
                         * Following array:
                         * [
                         *     1
                         *     , 2
                         *     , 3
                         * ]
                         *
                         * will be fixed to:
                         * [
                         *     1, 2, 3
                         * ]
                         */
                        const twoTokensBefore = sourceCode.getTokenBefore(tokenBefore, { includeComments: true });
                        if (astUtils.isCommentToken(twoTokensBefore)) {
                            return null;
                        }
                        return fixer.replaceTextRange([twoTokensBefore.range[1], tokenBefore.range[0]], "");
                    }
                });
            }
            /**
             * Reports that there should be a line break after the first token
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportRequiredLineBreak(token) {
                const tokenBefore = sourceCode.getTokenBefore(token, { includeComments: true });
                context.report({
                    loc: {
                        start: tokenBefore.loc.end,
                        end: token.loc.start
                    },
                    messageId: "missingLineBreak",
                    fix(fixer) {
                        return fixer.replaceTextRange([tokenBefore.range[1], token.range[0]], "\n");
                    }
                });
            }
            /**
             * Reports a given node if it violated this rule.
             * @param {ASTNode} node A node to check. This is an ObjectExpression node or an ObjectPattern node.
             * @returns {void}
             */
            function check(node) {
                const elements = node.elements;
                const normalizedOptions = normalizeOptions(context.options[0]);
                const options = normalizedOptions[node.type];
                let elementBreak = false;
                /*
                 * MULTILINE: true
                 * loop through every element and check
                 * if at least one element has linebreaks inside
                 * this ensures that following is not valid (due to elements are on the same line):
                 *
                 * [
                 *      1,
                 *      2,
                 *      3
                 * ]
                 */
                if (options.multiline) {
                    elementBreak = elements
                        .filter(element => element !== null)
                        .some(element => element.loc.start.line !== element.loc.end.line);
                }
                const linebreaksCount = node.elements.map((element, i) => {
                    const previousElement = elements[i - 1];
                    if (i === 0 || element === null || previousElement === null) {
                        return false;
                    }
                    const commaToken = sourceCode.getFirstTokenBetween(previousElement, element, astUtils.isCommaToken);
                    const lastTokenOfPreviousElement = sourceCode.getTokenBefore(commaToken);
                    const firstTokenOfCurrentElement = sourceCode.getTokenAfter(commaToken);
                    return !astUtils.isTokenOnSameLine(lastTokenOfPreviousElement, firstTokenOfCurrentElement);
                }).filter(isBreak => isBreak === true).length;
                const needsLinebreaks = (elements.length >= options.minItems ||
                    (options.multiline &&
                        elementBreak) ||
                    (options.consistent &&
                        linebreaksCount > 0 &&
                        linebreaksCount < node.elements.length));
                elements.forEach((element, i) => {
                    const previousElement = elements[i - 1];
                    if (i === 0 || element === null || previousElement === null) {
                        return;
                    }
                    const commaToken = sourceCode.getFirstTokenBetween(previousElement, element, astUtils.isCommaToken);
                    const lastTokenOfPreviousElement = sourceCode.getTokenBefore(commaToken);
                    const firstTokenOfCurrentElement = sourceCode.getTokenAfter(commaToken);
                    if (needsLinebreaks) {
                        if (astUtils.isTokenOnSameLine(lastTokenOfPreviousElement, firstTokenOfCurrentElement)) {
                            reportRequiredLineBreak(firstTokenOfCurrentElement);
                        }
                    }
                    else {
                        if (!astUtils.isTokenOnSameLine(lastTokenOfPreviousElement, firstTokenOfCurrentElement)) {
                            reportNoLineBreak(firstTokenOfCurrentElement);
                        }
                    }
                });
            }
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                ArrayPattern: check,
                ArrayExpression: check
            };
        }
    };
},
/* eslint/lib/rules/arrow-body-style.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require braces around arrow function bodies",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/arrow-body-style"
            },
            schema: {
                anyOf: [
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["always", "never"]
                            }
                        ],
                        minItems: 0,
                        maxItems: 1
                    },
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["as-needed"]
                            },
                            {
                                type: "object",
                                properties: {
                                    requireReturnForObjectLiteral: { type: "boolean" }
                                },
                                additionalProperties: false
                            }
                        ],
                        minItems: 0,
                        maxItems: 2
                    }
                ]
            },
            fixable: "code",
            messages: {
                unexpectedOtherBlock: "Unexpected block statement surrounding arrow body.",
                unexpectedEmptyBlock: "Unexpected block statement surrounding arrow body; put a value of `undefined` immediately after the `=>`.",
                unexpectedObjectBlock: "Unexpected block statement surrounding arrow body; parenthesize the returned value and move it immediately after the `=>`.",
                unexpectedSingleBlock: "Unexpected block statement surrounding arrow body; move the returned value immediately after the `=>`.",
                expectedBlock: "Expected block statement surrounding arrow body."
            }
        },
        create(context) {
            const options = context.options;
            const always = options[0] === "always";
            const asNeeded = !options[0] || options[0] === "as-needed";
            const never = options[0] === "never";
            const requireReturnForObjectLiteral = options[1] && options[1].requireReturnForObjectLiteral;
            const sourceCode = context.getSourceCode();
            /**
             * Checks whether the given node has ASI problem or not.
             * @param {Token} token The token to check.
             * @returns {boolean} `true` if it changes semantics if `;` or `}` followed by the token are removed.
             */
            function hasASIProblem(token) {
                return token && token.type === "Punctuator" && /^[([/`+-]/u.test(token.value);
            }
            /**
             * Gets the closing parenthesis which is the pair of the given opening parenthesis.
             * @param {Token} token The opening parenthesis token to get.
             * @returns {Token} The found closing parenthesis token.
             */
            function findClosingParen(token) {
                let node = sourceCode.getNodeByRangeIndex(token.range[1]);
                while (!astUtils.isParenthesised(sourceCode, node)) {
                    node = node.parent;
                }
                return sourceCode.getTokenAfter(node);
            }
            /**
             * Determines whether a arrow function body needs braces
             * @param {ASTNode} node The arrow function node.
             * @returns {void}
             */
            function validate(node) {
                const arrowBody = node.body;
                if (arrowBody.type === "BlockStatement") {
                    const blockBody = arrowBody.body;
                    if (blockBody.length !== 1 && !never) {
                        return;
                    }
                    if (asNeeded && requireReturnForObjectLiteral && blockBody[0].type === "ReturnStatement" &&
                        blockBody[0].argument && blockBody[0].argument.type === "ObjectExpression") {
                        return;
                    }
                    if (never || asNeeded && blockBody[0].type === "ReturnStatement") {
                        let messageId;
                        if (blockBody.length === 0) {
                            messageId = "unexpectedEmptyBlock";
                        }
                        else if (blockBody.length > 1) {
                            messageId = "unexpectedOtherBlock";
                        }
                        else if (blockBody[0].argument === null) {
                            messageId = "unexpectedSingleBlock";
                        }
                        else if (astUtils.isOpeningBraceToken(sourceCode.getFirstToken(blockBody[0], { skip: 1 }))) {
                            messageId = "unexpectedObjectBlock";
                        }
                        else {
                            messageId = "unexpectedSingleBlock";
                        }
                        context.report({
                            node,
                            loc: arrowBody.loc.start,
                            messageId,
                            fix(fixer) {
                                const fixes = [];
                                if (blockBody.length !== 1 ||
                                    blockBody[0].type !== "ReturnStatement" ||
                                    !blockBody[0].argument ||
                                    hasASIProblem(sourceCode.getTokenAfter(arrowBody))) {
                                    return fixes;
                                }
                                const openingBrace = sourceCode.getFirstToken(arrowBody);
                                const closingBrace = sourceCode.getLastToken(arrowBody);
                                const firstValueToken = sourceCode.getFirstToken(blockBody[0], 1);
                                const lastValueToken = sourceCode.getLastToken(blockBody[0]);
                                const commentsExist = sourceCode.commentsExistBetween(openingBrace, firstValueToken) ||
                                    sourceCode.commentsExistBetween(lastValueToken, closingBrace);
                                /*
                                 * Remove tokens around the return value.
                                 * If comments don't exist, remove extra spaces as well.
                                 */
                                if (commentsExist) {
                                    fixes.push(fixer.remove(openingBrace), fixer.remove(closingBrace), fixer.remove(sourceCode.getTokenAfter(openingBrace)) // return keyword
                                    );
                                }
                                else {
                                    fixes.push(fixer.removeRange([openingBrace.range[0], firstValueToken.range[0]]), fixer.removeRange([lastValueToken.range[1], closingBrace.range[1]]));
                                }
                                /*
                                 * If the first token of the reutrn value is `{` or the return value is a sequence expression,
                                 * enclose the return value by parentheses to avoid syntax error.
                                 */
                                if (astUtils.isOpeningBraceToken(firstValueToken) || blockBody[0].argument.type === "SequenceExpression") {
                                    fixes.push(fixer.insertTextBefore(firstValueToken, "("), fixer.insertTextAfter(lastValueToken, ")"));
                                }
                                /*
                                 * If the last token of the return statement is semicolon, remove it.
                                 * Non-block arrow body is an expression, not a statement.
                                 */
                                if (astUtils.isSemicolonToken(lastValueToken)) {
                                    fixes.push(fixer.remove(lastValueToken));
                                }
                                return fixes;
                            }
                        });
                    }
                }
                else {
                    if (always || (asNeeded && requireReturnForObjectLiteral && arrowBody.type === "ObjectExpression")) {
                        context.report({
                            node,
                            loc: arrowBody.loc.start,
                            messageId: "expectedBlock",
                            fix(fixer) {
                                const fixes = [];
                                const arrowToken = sourceCode.getTokenBefore(arrowBody, astUtils.isArrowToken);
                                const firstBodyToken = sourceCode.getTokenAfter(arrowToken);
                                const lastBodyToken = sourceCode.getLastToken(node);
                                const isParenthesisedObjectLiteral = astUtils.isOpeningParenToken(firstBodyToken) &&
                                    astUtils.isOpeningBraceToken(sourceCode.getTokenAfter(firstBodyToken));
                                // Wrap the value by a block and a return statement.
                                fixes.push(fixer.insertTextBefore(firstBodyToken, "{return "), fixer.insertTextAfter(lastBodyToken, "}"));
                                // If the value is object literal, remove parentheses which were forced by syntax.
                                if (isParenthesisedObjectLiteral) {
                                    fixes.push(fixer.remove(firstBodyToken), fixer.remove(findClosingParen(firstBodyToken)));
                                }
                                return fixes;
                            }
                        });
                    }
                }
            }
            return {
                "ArrowFunctionExpression:exit": validate
            };
        }
    };
},
/* eslint/lib/rules/arrow-parens.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Get location should be reported by AST node.
     * @param {ASTNode} node AST Node.
     * @returns {Location} Location information.
     */
    function getLocation(node) {
        return {
            start: node.params[0].loc.start,
            end: node.params[node.params.length - 1].loc.end
        };
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require parentheses around arrow function arguments",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/arrow-parens"
            },
            fixable: "code",
            schema: [
                {
                    enum: ["always", "as-needed"]
                },
                {
                    type: "object",
                    properties: {
                        requireForBlockBody: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpectedParens: "Unexpected parentheses around single function argument.",
                expectedParens: "Expected parentheses around arrow function argument.",
                unexpectedParensInline: "Unexpected parentheses around single function argument having a body with no curly braces.",
                expectedParensBlock: "Expected parentheses around arrow function argument having a body with curly braces."
            }
        },
        create(context) {
            const asNeeded = context.options[0] === "as-needed";
            const requireForBlockBody = asNeeded && context.options[1] && context.options[1].requireForBlockBody === true;
            const sourceCode = context.getSourceCode();
            /**
             * Determines whether a arrow function argument end with `)`
             * @param {ASTNode} node The arrow function node.
             * @returns {void}
             */
            function parens(node) {
                const isAsync = node.async;
                const firstTokenOfParam = sourceCode.getFirstToken(node, isAsync ? 1 : 0);
                /**
                 * Remove the parenthesis around a parameter
                 * @param {Fixer} fixer Fixer
                 * @returns {string} fixed parameter
                 */
                function fixParamsWithParenthesis(fixer) {
                    const paramToken = sourceCode.getTokenAfter(firstTokenOfParam);
                    /*
                     * ES8 allows Trailing commas in function parameter lists and calls
                     * https://github.com/eslint/eslint/issues/8834
                     */
                    const closingParenToken = sourceCode.getTokenAfter(paramToken, astUtils.isClosingParenToken);
                    const asyncToken = isAsync ? sourceCode.getTokenBefore(firstTokenOfParam) : null;
                    const shouldAddSpaceForAsync = asyncToken && (asyncToken.range[1] === firstTokenOfParam.range[0]);
                    return fixer.replaceTextRange([
                        firstTokenOfParam.range[0],
                        closingParenToken.range[1]
                    ], `${shouldAddSpaceForAsync ? " " : ""}${paramToken.value}`);
                }
                // "as-needed", { "requireForBlockBody": true }: x => x
                if (requireForBlockBody &&
                    node.params.length === 1 &&
                    node.params[0].type === "Identifier" &&
                    !node.params[0].typeAnnotation &&
                    node.body.type !== "BlockStatement" &&
                    !node.returnType) {
                    if (astUtils.isOpeningParenToken(firstTokenOfParam)) {
                        context.report({
                            node,
                            messageId: "unexpectedParensInline",
                            loc: getLocation(node),
                            fix: fixParamsWithParenthesis
                        });
                    }
                    return;
                }
                if (requireForBlockBody &&
                    node.body.type === "BlockStatement") {
                    if (!astUtils.isOpeningParenToken(firstTokenOfParam)) {
                        context.report({
                            node,
                            messageId: "expectedParensBlock",
                            loc: getLocation(node),
                            fix(fixer) {
                                return fixer.replaceText(firstTokenOfParam, `(${firstTokenOfParam.value})`);
                            }
                        });
                    }
                    return;
                }
                // "as-needed": x => x
                if (asNeeded &&
                    node.params.length === 1 &&
                    node.params[0].type === "Identifier" &&
                    !node.params[0].typeAnnotation &&
                    !node.returnType) {
                    if (astUtils.isOpeningParenToken(firstTokenOfParam)) {
                        context.report({
                            node,
                            messageId: "unexpectedParens",
                            loc: getLocation(node),
                            fix: fixParamsWithParenthesis
                        });
                    }
                    return;
                }
                if (firstTokenOfParam.type === "Identifier") {
                    const after = sourceCode.getTokenAfter(firstTokenOfParam);
                    // (x) => x
                    if (after.value !== ")") {
                        context.report({
                            node,
                            messageId: "expectedParens",
                            loc: getLocation(node),
                            fix(fixer) {
                                return fixer.replaceText(firstTokenOfParam, `(${firstTokenOfParam.value})`);
                            }
                        });
                    }
                }
            }
            return {
                ArrowFunctionExpression: parens
            };
        }
    };
},
/* eslint/lib/rules/arrow-spacing.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing before and after the arrow in arrow functions",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/arrow-spacing"
            },
            fixable: "whitespace",
            schema: [
                {
                    type: "object",
                    properties: {
                        before: {
                            type: "boolean",
                            default: true
                        },
                        after: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                expectedBefore: "Missing space before =>.",
                unexpectedBefore: "Unexpected space before =>.",
                expectedAfter: "Missing space after =>.",
                unexpectedAfter: "Unexpected space after =>."
            }
        },
        create(context) {
            // merge rules with default
            const rule = Object.assign({}, context.options[0]);
            rule.before = rule.before !== false;
            rule.after = rule.after !== false;
            const sourceCode = context.getSourceCode();
            /**
             * Get tokens of arrow(`=>`) and before/after arrow.
             * @param {ASTNode} node The arrow function node.
             * @returns {Object} Tokens of arrow and before/after arrow.
             */
            function getTokens(node) {
                const arrow = sourceCode.getTokenBefore(node.body, astUtils.isArrowToken);
                return {
                    before: sourceCode.getTokenBefore(arrow),
                    arrow,
                    after: sourceCode.getTokenAfter(arrow)
                };
            }
            /**
             * Count spaces before/after arrow(`=>`) token.
             * @param {Object} tokens Tokens before/after arrow.
             * @returns {Object} count of space before/after arrow.
             */
            function countSpaces(tokens) {
                const before = tokens.arrow.range[0] - tokens.before.range[1];
                const after = tokens.after.range[0] - tokens.arrow.range[1];
                return { before, after };
            }
            /**
             * Determines whether space(s) before after arrow(`=>`) is satisfy rule.
             * if before/after value is `true`, there should be space(s).
             * if before/after value is `false`, there should be no space.
             * @param {ASTNode} node The arrow function node.
             * @returns {void}
             */
            function spaces(node) {
                const tokens = getTokens(node);
                const countSpace = countSpaces(tokens);
                if (rule.before) {
                    // should be space(s) before arrow
                    if (countSpace.before === 0) {
                        context.report({
                            node: tokens.before,
                            messageId: "expectedBefore",
                            fix(fixer) {
                                return fixer.insertTextBefore(tokens.arrow, " ");
                            }
                        });
                    }
                }
                else {
                    // should be no space before arrow
                    if (countSpace.before > 0) {
                        context.report({
                            node: tokens.before,
                            messageId: "unexpectedBefore",
                            fix(fixer) {
                                return fixer.removeRange([tokens.before.range[1], tokens.arrow.range[0]]);
                            }
                        });
                    }
                }
                if (rule.after) {
                    // should be space(s) after arrow
                    if (countSpace.after === 0) {
                        context.report({
                            node: tokens.after,
                            messageId: "expectedAfter",
                            fix(fixer) {
                                return fixer.insertTextAfter(tokens.arrow, " ");
                            }
                        });
                    }
                }
                else {
                    // should be no space after arrow
                    if (countSpace.after > 0) {
                        context.report({
                            node: tokens.after,
                            messageId: "unexpectedAfter",
                            fix(fixer) {
                                return fixer.removeRange([tokens.arrow.range[1], tokens.after.range[0]]);
                            }
                        });
                    }
                }
            }
            return {
                ArrowFunctionExpression: spaces
            };
        }
    };
},
/* eslint/lib/rules/block-scoped-var.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce the use of variables within the scope they are defined",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/block-scoped-var"
            },
            schema: [],
            messages: {
                outOfScope: "'{{name}}' used outside of binding context."
            }
        },
        create(context) {
            let stack = [];
            /**
             * Makes a block scope.
             * @param {ASTNode} node A node of a scope.
             * @returns {void}
             */
            function enterScope(node) {
                stack.push(node.range);
            }
            /**
             * Pops the last block scope.
             * @returns {void}
             */
            function exitScope() {
                stack.pop();
            }
            /**
             * Reports a given reference.
             * @param {eslint-scope.Reference} reference A reference to report.
             * @returns {void}
             */
            function report(reference) {
                const identifier = reference.identifier;
                context.report({ node: identifier, messageId: "outOfScope", data: { name: identifier.name } });
            }
            /**
             * Finds and reports references which are outside of valid scopes.
             * @param {ASTNode} node A node to get variables.
             * @returns {void}
             */
            function checkForVariables(node) {
                if (node.kind !== "var") {
                    return;
                }
                // Defines a predicate to check whether or not a given reference is outside of valid scope.
                const scopeRange = stack[stack.length - 1];
                /**
                 * Check if a reference is out of scope
                 * @param {ASTNode} reference node to examine
                 * @returns {boolean} True is its outside the scope
                 * @private
                 */
                function isOutsideOfScope(reference) {
                    const idRange = reference.identifier.range;
                    return idRange[0] < scopeRange[0] || idRange[1] > scopeRange[1];
                }
                // Gets declared variables, and checks its references.
                const variables = context.getDeclaredVariables(node);
                for (let i = 0; i < variables.length; ++i) {
                    // Reports.
                    variables[i]
                        .references
                        .filter(isOutsideOfScope)
                        .forEach(report);
                }
            }
            return {
                Program(node) {
                    stack = [node.range];
                },
                // Manages scopes.
                BlockStatement: enterScope,
                "BlockStatement:exit": exitScope,
                ForStatement: enterScope,
                "ForStatement:exit": exitScope,
                ForInStatement: enterScope,
                "ForInStatement:exit": exitScope,
                ForOfStatement: enterScope,
                "ForOfStatement:exit": exitScope,
                SwitchStatement: enterScope,
                "SwitchStatement:exit": exitScope,
                CatchClause: enterScope,
                "CatchClause:exit": exitScope,
                // Finds and reports references which are outside of valid scope.
                VariableDeclaration: checkForVariables
            };
        }
    };
},
/* eslint/lib/rules/block-spacing.js */ function _(require, module, exports) {
    const util = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "disallow or enforce spaces inside of blocks after opening block and before closing block",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/block-spacing"
            },
            fixable: "whitespace",
            schema: [
                { enum: ["always", "never"] }
            ],
            messages: {
                missing: "Requires a space {{location}} '{{token}}'.",
                extra: "Unexpected space(s) {{location}} '{{token}}'."
            }
        },
        create(context) {
            const always = (context.options[0] !== "never"), messageId = always ? "missing" : "extra", sourceCode = context.getSourceCode();
            /**
             * Gets the open brace token from a given node.
             * @param {ASTNode} node A BlockStatement/SwitchStatement node to get.
             * @returns {Token} The token of the open brace.
             */
            function getOpenBrace(node) {
                if (node.type === "SwitchStatement") {
                    if (node.cases.length > 0) {
                        return sourceCode.getTokenBefore(node.cases[0]);
                    }
                    return sourceCode.getLastToken(node, 1);
                }
                return sourceCode.getFirstToken(node);
            }
            /**
             * Checks whether or not:
             *   - given tokens are on same line.
             *   - there is/isn't a space between given tokens.
             * @param {Token} left A token to check.
             * @param {Token} right The token which is next to `left`.
             * @returns {boolean}
             *    When the option is `"always"`, `true` if there are one or more spaces between given tokens.
             *    When the option is `"never"`, `true` if there are not any spaces between given tokens.
             *    If given tokens are not on same line, it's always `true`.
             */
            function isValid(left, right) {
                return (!util.isTokenOnSameLine(left, right) ||
                    sourceCode.isSpaceBetweenTokens(left, right) === always);
            }
            /**
             * Reports invalid spacing style inside braces.
             * @param {ASTNode} node A BlockStatement/SwitchStatement node to get.
             * @returns {void}
             */
            function checkSpacingInsideBraces(node) {
                // Gets braces and the first/last token of content.
                const openBrace = getOpenBrace(node);
                const closeBrace = sourceCode.getLastToken(node);
                const firstToken = sourceCode.getTokenAfter(openBrace, { includeComments: true });
                const lastToken = sourceCode.getTokenBefore(closeBrace, { includeComments: true });
                // Skip if the node is invalid or empty.
                if (openBrace.type !== "Punctuator" ||
                    openBrace.value !== "{" ||
                    closeBrace.type !== "Punctuator" ||
                    closeBrace.value !== "}" ||
                    firstToken === closeBrace) {
                    return;
                }
                // Skip line comments for option never
                if (!always && firstToken.type === "Line") {
                    return;
                }
                // Check.
                if (!isValid(openBrace, firstToken)) {
                    context.report({
                        node,
                        loc: openBrace.loc.start,
                        messageId,
                        data: {
                            location: "after",
                            token: openBrace.value
                        },
                        fix(fixer) {
                            if (always) {
                                return fixer.insertTextBefore(firstToken, " ");
                            }
                            return fixer.removeRange([openBrace.range[1], firstToken.range[0]]);
                        }
                    });
                }
                if (!isValid(lastToken, closeBrace)) {
                    context.report({
                        node,
                        loc: closeBrace.loc.start,
                        messageId,
                        data: {
                            location: "before",
                            token: closeBrace.value
                        },
                        fix(fixer) {
                            if (always) {
                                return fixer.insertTextAfter(lastToken, " ");
                            }
                            return fixer.removeRange([lastToken.range[1], closeBrace.range[0]]);
                        }
                    });
                }
            }
            return {
                BlockStatement: checkSpacingInsideBraces,
                SwitchStatement: checkSpacingInsideBraces
            };
        }
    };
},
/* eslint/lib/rules/brace-style.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent brace style for blocks",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/brace-style"
            },
            schema: [
                {
                    enum: ["1tbs", "stroustrup", "allman"]
                },
                {
                    type: "object",
                    properties: {
                        allowSingleLine: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "whitespace",
            messages: {
                nextLineOpen: "Opening curly brace does not appear on the same line as controlling statement.",
                sameLineOpen: "Opening curly brace appears on the same line as controlling statement.",
                blockSameLine: "Statement inside of curly braces should be on next line.",
                nextLineClose: "Closing curly brace does not appear on the same line as the subsequent block.",
                singleLineClose: "Closing curly brace should be on the same line as opening curly brace or on the line after the previous block.",
                sameLineClose: "Closing curly brace appears on the same line as the subsequent block."
            }
        },
        create(context) {
            const style = context.options[0] || "1tbs", params = context.options[1] || {}, sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Fixes a place where a newline unexpectedly appears
             * @param {Token} firstToken The token before the unexpected newline
             * @param {Token} secondToken The token after the unexpected newline
             * @returns {Function} A fixer function to remove the newlines between the tokens
             */
            function removeNewlineBetween(firstToken, secondToken) {
                const textRange = [firstToken.range[1], secondToken.range[0]];
                const textBetween = sourceCode.text.slice(textRange[0], textRange[1]);
                // Don't do a fix if there is a comment between the tokens
                if (textBetween.trim()) {
                    return null;
                }
                return fixer => fixer.replaceTextRange(textRange, " ");
            }
            /**
             * Validates a pair of curly brackets based on the user's config
             * @param {Token} openingCurly The opening curly bracket
             * @param {Token} closingCurly The closing curly bracket
             * @returns {void}
             */
            function validateCurlyPair(openingCurly, closingCurly) {
                const tokenBeforeOpeningCurly = sourceCode.getTokenBefore(openingCurly);
                const tokenAfterOpeningCurly = sourceCode.getTokenAfter(openingCurly);
                const tokenBeforeClosingCurly = sourceCode.getTokenBefore(closingCurly);
                const singleLineException = params.allowSingleLine && astUtils.isTokenOnSameLine(openingCurly, closingCurly);
                if (style !== "allman" && !astUtils.isTokenOnSameLine(tokenBeforeOpeningCurly, openingCurly)) {
                    context.report({
                        node: openingCurly,
                        messageId: "nextLineOpen",
                        fix: removeNewlineBetween(tokenBeforeOpeningCurly, openingCurly)
                    });
                }
                if (style === "allman" && astUtils.isTokenOnSameLine(tokenBeforeOpeningCurly, openingCurly) && !singleLineException) {
                    context.report({
                        node: openingCurly,
                        messageId: "sameLineOpen",
                        fix: fixer => fixer.insertTextBefore(openingCurly, "\n")
                    });
                }
                if (astUtils.isTokenOnSameLine(openingCurly, tokenAfterOpeningCurly) && tokenAfterOpeningCurly !== closingCurly && !singleLineException) {
                    context.report({
                        node: openingCurly,
                        messageId: "blockSameLine",
                        fix: fixer => fixer.insertTextAfter(openingCurly, "\n")
                    });
                }
                if (tokenBeforeClosingCurly !== openingCurly && !singleLineException && astUtils.isTokenOnSameLine(tokenBeforeClosingCurly, closingCurly)) {
                    context.report({
                        node: closingCurly,
                        messageId: "singleLineClose",
                        fix: fixer => fixer.insertTextBefore(closingCurly, "\n")
                    });
                }
            }
            /**
             * Validates the location of a token that appears before a keyword (e.g. a newline before `else`)
             * @param {Token} curlyToken The closing curly token. This is assumed to precede a keyword token (such as `else` or `finally`).
             * @returns {void}
             */
            function validateCurlyBeforeKeyword(curlyToken) {
                const keywordToken = sourceCode.getTokenAfter(curlyToken);
                if (style === "1tbs" && !astUtils.isTokenOnSameLine(curlyToken, keywordToken)) {
                    context.report({
                        node: curlyToken,
                        messageId: "nextLineClose",
                        fix: removeNewlineBetween(curlyToken, keywordToken)
                    });
                }
                if (style !== "1tbs" && astUtils.isTokenOnSameLine(curlyToken, keywordToken)) {
                    context.report({
                        node: curlyToken,
                        messageId: "sameLineClose",
                        fix: fixer => fixer.insertTextAfter(curlyToken, "\n")
                    });
                }
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                BlockStatement(node) {
                    if (!astUtils.STATEMENT_LIST_PARENTS.has(node.parent.type)) {
                        validateCurlyPair(sourceCode.getFirstToken(node), sourceCode.getLastToken(node));
                    }
                },
                ClassBody(node) {
                    validateCurlyPair(sourceCode.getFirstToken(node), sourceCode.getLastToken(node));
                },
                SwitchStatement(node) {
                    const closingCurly = sourceCode.getLastToken(node);
                    const openingCurly = sourceCode.getTokenBefore(node.cases.length ? node.cases[0] : closingCurly);
                    validateCurlyPair(openingCurly, closingCurly);
                },
                IfStatement(node) {
                    if (node.consequent.type === "BlockStatement" && node.alternate) {
                        // Handle the keyword after the `if` block (before `else`)
                        validateCurlyBeforeKeyword(sourceCode.getLastToken(node.consequent));
                    }
                },
                TryStatement(node) {
                    // Handle the keyword after the `try` block (before `catch` or `finally`)
                    validateCurlyBeforeKeyword(sourceCode.getLastToken(node.block));
                    if (node.handler && node.finalizer) {
                        // Handle the keyword after the `catch` block (before `finally`)
                        validateCurlyBeforeKeyword(sourceCode.getLastToken(node.handler.body));
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/callback-return.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require `return` statements after callbacks",
                category: "Node.js and CommonJS",
                recommended: false,
                url: "https://eslint.org/docs/rules/callback-return"
            },
            schema: [{
                    type: "array",
                    items: { type: "string" }
                }],
            messages: {
                missingReturn: "Expected return with your callback function."
            }
        },
        create(context) {
            const callbacks = context.options[0] || ["callback", "cb", "next"], sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Find the closest parent matching a list of types.
             * @param {ASTNode} node The node whose parents we are searching
             * @param {Array} types The node types to match
             * @returns {ASTNode} The matched node or undefined.
             */
            function findClosestParentOfType(node, types) {
                if (!node.parent) {
                    return null;
                }
                if (types.indexOf(node.parent.type) === -1) {
                    return findClosestParentOfType(node.parent, types);
                }
                return node.parent;
            }
            /**
             * Check to see if a node contains only identifers
             * @param {ASTNode} node The node to check
             * @returns {boolean} Whether or not the node contains only identifers
             */
            function containsOnlyIdentifiers(node) {
                if (node.type === "Identifier") {
                    return true;
                }
                if (node.type === "MemberExpression") {
                    if (node.object.type === "Identifier") {
                        return true;
                    }
                    if (node.object.type === "MemberExpression") {
                        return containsOnlyIdentifiers(node.object);
                    }
                }
                return false;
            }
            /**
             * Check to see if a CallExpression is in our callback list.
             * @param {ASTNode} node The node to check against our callback names list.
             * @returns {boolean} Whether or not this function matches our callback name.
             */
            function isCallback(node) {
                return containsOnlyIdentifiers(node.callee) && callbacks.indexOf(sourceCode.getText(node.callee)) > -1;
            }
            /**
             * Determines whether or not the callback is part of a callback expression.
             * @param {ASTNode} node The callback node
             * @param {ASTNode} parentNode The expression node
             * @returns {boolean} Whether or not this is part of a callback expression
             */
            function isCallbackExpression(node, parentNode) {
                // ensure the parent node exists and is an expression
                if (!parentNode || parentNode.type !== "ExpressionStatement") {
                    return false;
                }
                // cb()
                if (parentNode.expression === node) {
                    return true;
                }
                // special case for cb && cb() and similar
                if (parentNode.expression.type === "BinaryExpression" || parentNode.expression.type === "LogicalExpression") {
                    if (parentNode.expression.right === node) {
                        return true;
                    }
                }
                return false;
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                CallExpression(node) {
                    // if we're not a callback we can return
                    if (!isCallback(node)) {
                        return;
                    }
                    // find the closest block, return or loop
                    const closestBlock = findClosestParentOfType(node, ["BlockStatement", "ReturnStatement", "ArrowFunctionExpression"]) || {};
                    // if our parent is a return we know we're ok
                    if (closestBlock.type === "ReturnStatement") {
                        return;
                    }
                    // arrow functions don't always have blocks and implicitly return
                    if (closestBlock.type === "ArrowFunctionExpression") {
                        return;
                    }
                    // block statements are part of functions and most if statements
                    if (closestBlock.type === "BlockStatement") {
                        // find the last item in the block
                        const lastItem = closestBlock.body[closestBlock.body.length - 1];
                        // if the callback is the last thing in a block that might be ok
                        if (isCallbackExpression(node, lastItem)) {
                            const parentType = closestBlock.parent.type;
                            // but only if the block is part of a function
                            if (parentType === "FunctionExpression" ||
                                parentType === "FunctionDeclaration" ||
                                parentType === "ArrowFunctionExpression") {
                                return;
                            }
                        }
                        // ending a block with a return is also ok
                        if (lastItem.type === "ReturnStatement") {
                            // but only if the callback is immediately before
                            if (isCallbackExpression(node, closestBlock.body[closestBlock.body.length - 2])) {
                                return;
                            }
                        }
                    }
                    // as long as you're the child of a function at this point you should be asked to return
                    if (findClosestParentOfType(node, ["FunctionDeclaration", "FunctionExpression", "ArrowFunctionExpression"])) {
                        context.report({ node, messageId: "missingReturn" });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/camelcase.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce camelcase naming convention",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/camelcase"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        ignoreDestructuring: {
                            type: "boolean",
                            default: false
                        },
                        ignoreImports: {
                            type: "boolean",
                            default: false
                        },
                        properties: {
                            enum: ["always", "never"]
                        },
                        allow: {
                            type: "array",
                            items: [
                                {
                                    type: "string"
                                }
                            ],
                            minItems: 0,
                            uniqueItems: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                notCamelCase: "Identifier '{{name}}' is not in camel case."
            }
        },
        create(context) {
            const options = context.options[0] || {};
            let properties = options.properties || "";
            const ignoreDestructuring = options.ignoreDestructuring;
            const ignoreImports = options.ignoreImports;
            const allow = options.allow || [];
            if (properties !== "always" && properties !== "never") {
                properties = "always";
            }
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            // contains reported nodes to avoid reporting twice on destructuring with shorthand notation
            const reported = [];
            const ALLOWED_PARENT_TYPES = new Set(["CallExpression", "NewExpression"]);
            /**
             * Checks if a string contains an underscore and isn't all upper-case
             * @param {string} name The string to check.
             * @returns {boolean} if the string is underscored
             * @private
             */
            function isUnderscored(name) {
                // if there's an underscore, it might be A_CONSTANT, which is okay
                return name.includes("_") && name !== name.toUpperCase();
            }
            /**
             * Checks if a string match the ignore list
             * @param {string} name The string to check.
             * @returns {boolean} if the string is ignored
             * @private
             */
            function isAllowed(name) {
                return allow.some(entry => name === entry || name.match(new RegExp(entry, "u")));
            }
            /**
             * Checks if a parent of a node is an ObjectPattern.
             * @param {ASTNode} node The node to check.
             * @returns {boolean} if the node is inside an ObjectPattern
             * @private
             */
            function isInsideObjectPattern(node) {
                let current = node;
                while (current) {
                    const parent = current.parent;
                    if (parent && parent.type === "Property" && parent.computed && parent.key === current) {
                        return false;
                    }
                    if (current.type === "ObjectPattern") {
                        return true;
                    }
                    current = parent;
                }
                return false;
            }
            /**
             * Reports an AST node as a rule violation.
             * @param {ASTNode} node The node to report.
             * @returns {void}
             * @private
             */
            function report(node) {
                if (!reported.includes(node)) {
                    reported.push(node);
                    context.report({ node, messageId: "notCamelCase", data: { name: node.name } });
                }
            }
            return {
                Identifier(node) {
                    /*
                     * Leading and trailing underscores are commonly used to flag
                     * private/protected identifiers, strip them before checking if underscored
                     */
                    const name = node.name, nameIsUnderscored = isUnderscored(name.replace(/^_+|_+$/gu, "")), effectiveParent = (node.parent.type === "MemberExpression") ? node.parent.parent : node.parent;
                    // First, we ignore the node if it match the ignore list
                    if (isAllowed(name)) {
                        return;
                    }
                    // MemberExpressions get special rules
                    if (node.parent.type === "MemberExpression") {
                        // "never" check properties
                        if (properties === "never") {
                            return;
                        }
                        // Always report underscored object names
                        if (node.parent.object.type === "Identifier" && node.parent.object.name === node.name && nameIsUnderscored) {
                            report(node);
                            // Report AssignmentExpressions only if they are the left side of the assignment
                        }
                        else if (effectiveParent.type === "AssignmentExpression" && nameIsUnderscored && (effectiveParent.right.type !== "MemberExpression" || effectiveParent.left.type === "MemberExpression" && effectiveParent.left.property.name === node.name)) {
                            report(node);
                        }
                        /*
                         * Properties have their own rules, and
                         * AssignmentPattern nodes can be treated like Properties:
                         * e.g.: const { no_camelcased = false } = bar;
                         */
                    }
                    else if (node.parent.type === "Property" || node.parent.type === "AssignmentPattern") {
                        if (node.parent.parent && node.parent.parent.type === "ObjectPattern") {
                            if (node.parent.shorthand && node.parent.value.left && nameIsUnderscored) {
                                report(node);
                            }
                            const assignmentKeyEqualsValue = node.parent.key.name === node.parent.value.name;
                            if (isUnderscored(name) && node.parent.computed) {
                                report(node);
                            }
                            // prevent checking righthand side of destructured object
                            if (node.parent.key === node && node.parent.value !== node) {
                                return;
                            }
                            const valueIsUnderscored = node.parent.value.name && nameIsUnderscored;
                            // ignore destructuring if the option is set, unless a new identifier is created
                            if (valueIsUnderscored && !(assignmentKeyEqualsValue && ignoreDestructuring)) {
                                report(node);
                            }
                        }
                        // "never" check properties or always ignore destructuring
                        if (properties === "never" || (ignoreDestructuring && isInsideObjectPattern(node))) {
                            return;
                        }
                        // don't check right hand side of AssignmentExpression to prevent duplicate warnings
                        if (nameIsUnderscored && !ALLOWED_PARENT_TYPES.has(effectiveParent.type) && !(node.parent.right === node)) {
                            report(node);
                        }
                        // Check if it's an import specifier
                    }
                    else if (["ImportSpecifier", "ImportNamespaceSpecifier", "ImportDefaultSpecifier"].includes(node.parent.type)) {
                        if (node.parent.type === "ImportSpecifier" && ignoreImports) {
                            return;
                        }
                        // Report only if the local imported identifier is underscored
                        if (node.parent.local &&
                            node.parent.local.name === node.name &&
                            nameIsUnderscored) {
                            report(node);
                        }
                        // Report anything that is underscored that isn't a CallExpression
                    }
                    else if (nameIsUnderscored && !ALLOWED_PARENT_TYPES.has(effectiveParent.type)) {
                        report(node);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/capitalized-comments.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const LETTER_PATTERN = require(144) /* ./utils/patterns/letters */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const DEFAULT_IGNORE_PATTERN = astUtils.COMMENTS_IGNORE_PATTERN, WHITESPACE = /\s/gu, MAYBE_URL = /^\s*[^:/?#\s]+:\/\/[^?#]/u; // TODO: Combine w/ max-len pattern?
    /*
     * Base schema body for defining the basic capitalization rule, ignorePattern,
     * and ignoreInlineComments values.
     * This can be used in a few different ways in the actual schema.
     */
    const SCHEMA_BODY = {
        type: "object",
        properties: {
            ignorePattern: {
                type: "string"
            },
            ignoreInlineComments: {
                type: "boolean"
            },
            ignoreConsecutiveComments: {
                type: "boolean"
            }
        },
        additionalProperties: false
    };
    const DEFAULTS = {
        ignorePattern: "",
        ignoreInlineComments: false,
        ignoreConsecutiveComments: false
    };
    /**
     * Get normalized options for either block or line comments from the given
     * user-provided options.
     * - If the user-provided options is just a string, returns a normalized
     *   set of options using default values for all other options.
     * - If the user-provided options is an object, then a normalized option
     *   set is returned. Options specified in overrides will take priority
     *   over options specified in the main options object, which will in
     *   turn take priority over the rule's defaults.
     * @param {Object|string} rawOptions The user-provided options.
     * @param {string} which Either "line" or "block".
     * @returns {Object} The normalized options.
     */
    function getNormalizedOptions(rawOptions, which) {
        return Object.assign({}, DEFAULTS, rawOptions[which] || rawOptions);
    }
    /**
     * Get normalized options for block and line comments.
     * @param {Object|string} rawOptions The user-provided options.
     * @returns {Object} An object with "Line" and "Block" keys and corresponding
     * normalized options objects.
     */
    function getAllNormalizedOptions(rawOptions = {}) {
        return {
            Line: getNormalizedOptions(rawOptions, "line"),
            Block: getNormalizedOptions(rawOptions, "block")
        };
    }
    /**
     * Creates a regular expression for each ignorePattern defined in the rule
     * options.
     *
     * This is done in order to avoid invoking the RegExp constructor repeatedly.
     * @param {Object} normalizedOptions The normalized rule options.
     * @returns {void}
     */
    function createRegExpForIgnorePatterns(normalizedOptions) {
        Object.keys(normalizedOptions).forEach(key => {
            const ignorePatternStr = normalizedOptions[key].ignorePattern;
            if (ignorePatternStr) {
                const regExp = RegExp(`^\\s*(?:${ignorePatternStr})`, "u");
                normalizedOptions[key].ignorePatternRegExp = regExp;
            }
        });
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce or disallow capitalization of the first letter of a comment",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/capitalized-comments"
            },
            fixable: "code",
            schema: [
                { enum: ["always", "never"] },
                {
                    oneOf: [
                        SCHEMA_BODY,
                        {
                            type: "object",
                            properties: {
                                line: SCHEMA_BODY,
                                block: SCHEMA_BODY
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                unexpectedLowercaseComment: "Comments should not begin with a lowercase character.",
                unexpectedUppercaseComment: "Comments should not begin with an uppercase character."
            }
        },
        create(context) {
            const capitalize = context.options[0] || "always", normalizedOptions = getAllNormalizedOptions(context.options[1]), sourceCode = context.getSourceCode();
            createRegExpForIgnorePatterns(normalizedOptions);
            //----------------------------------------------------------------------
            // Helpers
            //----------------------------------------------------------------------
            /**
             * Checks whether a comment is an inline comment.
             *
             * For the purpose of this rule, a comment is inline if:
             * 1. The comment is preceded by a token on the same line; and
             * 2. The command is followed by a token on the same line.
             *
             * Note that the comment itself need not be single-line!
             *
             * Also, it follows from this definition that only block comments can
             * be considered as possibly inline. This is because line comments
             * would consume any following tokens on the same line as the comment.
             * @param {ASTNode} comment The comment node to check.
             * @returns {boolean} True if the comment is an inline comment, false
             * otherwise.
             */
            function isInlineComment(comment) {
                const previousToken = sourceCode.getTokenBefore(comment, { includeComments: true }), nextToken = sourceCode.getTokenAfter(comment, { includeComments: true });
                return Boolean(previousToken &&
                    nextToken &&
                    comment.loc.start.line === previousToken.loc.end.line &&
                    comment.loc.end.line === nextToken.loc.start.line);
            }
            /**
             * Determine if a comment follows another comment.
             * @param {ASTNode} comment The comment to check.
             * @returns {boolean} True if the comment follows a valid comment.
             */
            function isConsecutiveComment(comment) {
                const previousTokenOrComment = sourceCode.getTokenBefore(comment, { includeComments: true });
                return Boolean(previousTokenOrComment &&
                    ["Block", "Line"].indexOf(previousTokenOrComment.type) !== -1);
            }
            /**
             * Check a comment to determine if it is valid for this rule.
             * @param {ASTNode} comment The comment node to process.
             * @param {Object} options The options for checking this comment.
             * @returns {boolean} True if the comment is valid, false otherwise.
             */
            function isCommentValid(comment, options) {
                // 1. Check for default ignore pattern.
                if (DEFAULT_IGNORE_PATTERN.test(comment.value)) {
                    return true;
                }
                // 2. Check for custom ignore pattern.
                const commentWithoutAsterisks = comment.value
                    .replace(/\*/gu, "");
                if (options.ignorePatternRegExp && options.ignorePatternRegExp.test(commentWithoutAsterisks)) {
                    return true;
                }
                // 3. Check for inline comments.
                if (options.ignoreInlineComments && isInlineComment(comment)) {
                    return true;
                }
                // 4. Is this a consecutive comment (and are we tolerating those)?
                if (options.ignoreConsecutiveComments && isConsecutiveComment(comment)) {
                    return true;
                }
                // 5. Does the comment start with a possible URL?
                if (MAYBE_URL.test(commentWithoutAsterisks)) {
                    return true;
                }
                // 6. Is the initial word character a letter?
                const commentWordCharsOnly = commentWithoutAsterisks
                    .replace(WHITESPACE, "");
                if (commentWordCharsOnly.length === 0) {
                    return true;
                }
                const firstWordChar = commentWordCharsOnly[0];
                if (!LETTER_PATTERN.test(firstWordChar)) {
                    return true;
                }
                // 7. Check the case of the initial word character.
                const isUppercase = firstWordChar !== firstWordChar.toLocaleLowerCase(), isLowercase = firstWordChar !== firstWordChar.toLocaleUpperCase();
                if (capitalize === "always" && isLowercase) {
                    return false;
                }
                if (capitalize === "never" && isUppercase) {
                    return false;
                }
                return true;
            }
            /**
             * Process a comment to determine if it needs to be reported.
             * @param {ASTNode} comment The comment node to process.
             * @returns {void}
             */
            function processComment(comment) {
                const options = normalizedOptions[comment.type], commentValid = isCommentValid(comment, options);
                if (!commentValid) {
                    const messageId = capitalize === "always"
                        ? "unexpectedLowercaseComment"
                        : "unexpectedUppercaseComment";
                    context.report({
                        node: null,
                        loc: comment.loc,
                        messageId,
                        fix(fixer) {
                            const match = comment.value.match(LETTER_PATTERN);
                            return fixer.replaceTextRange(
                            // Offset match.index by 2 to account for the first 2 characters that start the comment (// or /*)
                            [comment.range[0] + match.index + 2, comment.range[0] + match.index + 3], capitalize === "always" ? match[0].toLocaleUpperCase() : match[0].toLocaleLowerCase());
                        }
                    });
                }
            }
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                Program() {
                    const comments = sourceCode.getAllComments();
                    comments.filter(token => token.type !== "Shebang").forEach(processComment);
                }
            };
        }
    };
},
/* eslint/lib/rules/utils/patterns/letters.js */ function _(require, module, exports) {
    module.exports = /[A-Za-z\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA\u05F0-\u05F2\u0620-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0840-\u0858\u08A0-\u08B2\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971-\u0980\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D\u0C58\u0C59\u0C60\u0C61\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0CF1\u0CF2\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D\u0D4E\u0D60\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6\u0EDC-\u0EDF\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8C\u1000-\u102A\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081\u108E\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F4\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16F1-\u16F8\u1700-\u170C\u170E-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7\u17DC\u1820-\u1877\u1880-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191E\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19C1-\u19C7\u1A00-\u1A16\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF\u1BBA-\u1BE5\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1CE9-\u1CEC\u1CEE-\u1CF1\u1CF5\u1CF6\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F\u2090-\u209C\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2183\u2184\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2CF2\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2E2F\u3005\u3006\u3031-\u3035\u303B\u303C\u3041-\u3096\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312D\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FCC\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B\uA640-\uA66E\uA67F-\uA69D\uA6A0-\uA6E5\uA717-\uA71F\uA722-\uA788\uA78B-\uA78E\uA790-\uA7AD\uA7B0\uA7B1\uA7F7-\uA801\uA803-\uA805\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uA9E0-\uA9E4\uA9E6-\uA9EF\uA9FA-\uA9FE\uAA00-\uAA28\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA7E-\uAAAF\uAAB1\uAAB5\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uAAE0-\uAAEA\uAAF2-\uAAF4\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB5F\uAB64\uAB65\uABC0-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDE80-\uDE9C\uDEA0-\uDED0\uDF00-\uDF1F\uDF30-\uDF40\uDF42-\uDF49\uDF50-\uDF75\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF]|\uD801[\uDC00-\uDC9D\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00\uDE10-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE4\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48]|\uD804[\uDC03-\uDC37\uDC83-\uDCAF\uDCD0-\uDCE8\uDD03-\uDD26\uDD50-\uDD72\uDD76\uDD83-\uDDB2\uDDC1-\uDDC4\uDDDA\uDE00-\uDE11\uDE13-\uDE2B\uDEB0-\uDEDE\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3D\uDF5D-\uDF61]|\uD805[\uDC80-\uDCAF\uDCC4\uDCC5\uDCC7\uDD80-\uDDAE\uDE00-\uDE2F\uDE44\uDE80-\uDEAA]|\uD806[\uDCA0-\uDCDF\uDCFF\uDEC0-\uDEF8]|\uD808[\uDC00-\uDF98]|[\uD80C\uD840-\uD868\uD86A-\uD86C][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDED0-\uDEED\uDF00-\uDF2F\uDF40-\uDF43\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50\uDF93-\uDF9F]|\uD82C[\uDC00\uDC01]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB]|\uD83A[\uDC00-\uDCC4]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D]|\uD87E[\uDC00-\uDE1D]/u;
},
/* eslint/lib/rules/class-methods-use-this.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce that class methods utilize `this`",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/class-methods-use-this"
            },
            schema: [{
                    type: "object",
                    properties: {
                        exceptMethods: {
                            type: "array",
                            items: {
                                type: "string"
                            }
                        }
                    },
                    additionalProperties: false
                }],
            messages: {
                missingThis: "Expected 'this' to be used by class {{name}}."
            }
        },
        create(context) {
            const config = Object.assign({}, context.options[0]);
            const exceptMethods = new Set(config.exceptMethods || []);
            const stack = [];
            /**
             * Initializes the current context to false and pushes it onto the stack.
             * These booleans represent whether 'this' has been used in the context.
             * @returns {void}
             * @private
             */
            function enterFunction() {
                stack.push(false);
            }
            /**
             * Check if the node is an instance method
             * @param {ASTNode} node node to check
             * @returns {boolean} True if its an instance method
             * @private
             */
            function isInstanceMethod(node) {
                return !node.static && node.kind !== "constructor" && node.type === "MethodDefinition";
            }
            /**
             * Check if the node is an instance method not excluded by config
             * @param {ASTNode} node node to check
             * @returns {boolean} True if it is an instance method, and not excluded by config
             * @private
             */
            function isIncludedInstanceMethod(node) {
                return isInstanceMethod(node) &&
                    (node.computed || !exceptMethods.has(node.key.name));
            }
            /**
             * Checks if we are leaving a function that is a method, and reports if 'this' has not been used.
             * Static methods and the constructor are exempt.
             * Then pops the context off the stack.
             * @param {ASTNode} node A function node that was entered.
             * @returns {void}
             * @private
             */
            function exitFunction(node) {
                const methodUsesThis = stack.pop();
                if (isIncludedInstanceMethod(node.parent) && !methodUsesThis) {
                    context.report({
                        node,
                        messageId: "missingThis",
                        data: {
                            name: astUtils.getFunctionNameWithKind(node)
                        }
                    });
                }
            }
            /**
             * Mark the current context as having used 'this'.
             * @returns {void}
             * @private
             */
            function markThisUsed() {
                if (stack.length) {
                    stack[stack.length - 1] = true;
                }
            }
            return {
                FunctionDeclaration: enterFunction,
                "FunctionDeclaration:exit": exitFunction,
                FunctionExpression: enterFunction,
                "FunctionExpression:exit": exitFunction,
                ThisExpression: markThisUsed,
                Super: markThisUsed
            };
        }
    };
},
/* eslint/lib/rules/comma-dangle.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const DEFAULT_OPTIONS = Object.freeze({
        arrays: "never",
        objects: "never",
        imports: "never",
        exports: "never",
        functions: "never"
    });
    /**
     * Checks whether or not a trailing comma is allowed in a given node.
     * If the `lastItem` is `RestElement` or `RestProperty`, it disallows trailing commas.
     * @param {ASTNode} lastItem The node of the last element in the given node.
     * @returns {boolean} `true` if a trailing comma is allowed.
     */
    function isTrailingCommaAllowed(lastItem) {
        return !(lastItem.type === "RestElement" ||
            lastItem.type === "RestProperty" ||
            lastItem.type === "ExperimentalRestProperty");
    }
    /**
     * Normalize option value.
     * @param {string|Object|undefined} optionValue The 1st option value to normalize.
     * @param {number} ecmaVersion The normalized ECMAScript version.
     * @returns {Object} The normalized option value.
     */
    function normalizeOptions(optionValue, ecmaVersion) {
        if (typeof optionValue === "string") {
            return {
                arrays: optionValue,
                objects: optionValue,
                imports: optionValue,
                exports: optionValue,
                functions: (!ecmaVersion || ecmaVersion < 8) ? "ignore" : optionValue
            };
        }
        if (typeof optionValue === "object" && optionValue !== null) {
            return {
                arrays: optionValue.arrays || DEFAULT_OPTIONS.arrays,
                objects: optionValue.objects || DEFAULT_OPTIONS.objects,
                imports: optionValue.imports || DEFAULT_OPTIONS.imports,
                exports: optionValue.exports || DEFAULT_OPTIONS.exports,
                functions: optionValue.functions || DEFAULT_OPTIONS.functions
            };
        }
        return DEFAULT_OPTIONS;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow trailing commas",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/comma-dangle"
            },
            fixable: "code",
            schema: {
                definitions: {
                    value: {
                        enum: [
                            "always-multiline",
                            "always",
                            "never",
                            "only-multiline"
                        ]
                    },
                    valueWithIgnore: {
                        enum: [
                            "always-multiline",
                            "always",
                            "ignore",
                            "never",
                            "only-multiline"
                        ]
                    }
                },
                type: "array",
                items: [
                    {
                        oneOf: [
                            {
                                $ref: "#/definitions/value"
                            },
                            {
                                type: "object",
                                properties: {
                                    arrays: { $ref: "#/definitions/valueWithIgnore" },
                                    objects: { $ref: "#/definitions/valueWithIgnore" },
                                    imports: { $ref: "#/definitions/valueWithIgnore" },
                                    exports: { $ref: "#/definitions/valueWithIgnore" },
                                    functions: { $ref: "#/definitions/valueWithIgnore" }
                                },
                                additionalProperties: false
                            }
                        ]
                    }
                ]
            },
            messages: {
                unexpected: "Unexpected trailing comma.",
                missing: "Missing trailing comma."
            }
        },
        create(context) {
            const options = normalizeOptions(context.options[0], context.parserOptions.ecmaVersion);
            const sourceCode = context.getSourceCode();
            /**
             * Gets the last item of the given node.
             * @param {ASTNode} node The node to get.
             * @returns {ASTNode|null} The last node or null.
             */
            function getLastItem(node) {
                switch (node.type) {
                    case "ObjectExpression":
                    case "ObjectPattern":
                        return lodash.last(node.properties);
                    case "ArrayExpression":
                    case "ArrayPattern":
                        return lodash.last(node.elements);
                    case "ImportDeclaration":
                    case "ExportNamedDeclaration":
                        return lodash.last(node.specifiers);
                    case "FunctionDeclaration":
                    case "FunctionExpression":
                    case "ArrowFunctionExpression":
                        return lodash.last(node.params);
                    case "CallExpression":
                    case "NewExpression":
                        return lodash.last(node.arguments);
                    default:
                        return null;
                }
            }
            /**
             * Gets the trailing comma token of the given node.
             * If the trailing comma does not exist, this returns the token which is
             * the insertion point of the trailing comma token.
             * @param {ASTNode} node The node to get.
             * @param {ASTNode} lastItem The last item of the node.
             * @returns {Token} The trailing comma token or the insertion point.
             */
            function getTrailingToken(node, lastItem) {
                switch (node.type) {
                    case "ObjectExpression":
                    case "ArrayExpression":
                    case "CallExpression":
                    case "NewExpression":
                        return sourceCode.getLastToken(node, 1);
                    default: {
                        const nextToken = sourceCode.getTokenAfter(lastItem);
                        if (astUtils.isCommaToken(nextToken)) {
                            return nextToken;
                        }
                        return sourceCode.getLastToken(lastItem);
                    }
                }
            }
            /**
             * Checks whether or not a given node is multiline.
             * This rule handles a given node as multiline when the closing parenthesis
             * and the last element are not on the same line.
             * @param {ASTNode} node A node to check.
             * @returns {boolean} `true` if the node is multiline.
             */
            function isMultiline(node) {
                const lastItem = getLastItem(node);
                if (!lastItem) {
                    return false;
                }
                const penultimateToken = getTrailingToken(node, lastItem);
                const lastToken = sourceCode.getTokenAfter(penultimateToken);
                return lastToken.loc.end.line !== penultimateToken.loc.end.line;
            }
            /**
             * Reports a trailing comma if it exists.
             * @param {ASTNode} node A node to check. Its type is one of
             *   ObjectExpression, ObjectPattern, ArrayExpression, ArrayPattern,
             *   ImportDeclaration, and ExportNamedDeclaration.
             * @returns {void}
             */
            function forbidTrailingComma(node) {
                const lastItem = getLastItem(node);
                if (!lastItem || (node.type === "ImportDeclaration" && lastItem.type !== "ImportSpecifier")) {
                    return;
                }
                const trailingToken = getTrailingToken(node, lastItem);
                if (astUtils.isCommaToken(trailingToken)) {
                    context.report({
                        node: lastItem,
                        loc: trailingToken.loc,
                        messageId: "unexpected",
                        fix(fixer) {
                            return fixer.remove(trailingToken);
                        }
                    });
                }
            }
            /**
             * Reports the last element of a given node if it does not have a trailing
             * comma.
             *
             * If a given node is `ArrayPattern` which has `RestElement`, the trailing
             * comma is disallowed, so report if it exists.
             * @param {ASTNode} node A node to check. Its type is one of
             *   ObjectExpression, ObjectPattern, ArrayExpression, ArrayPattern,
             *   ImportDeclaration, and ExportNamedDeclaration.
             * @returns {void}
             */
            function forceTrailingComma(node) {
                const lastItem = getLastItem(node);
                if (!lastItem || (node.type === "ImportDeclaration" && lastItem.type !== "ImportSpecifier")) {
                    return;
                }
                if (!isTrailingCommaAllowed(lastItem)) {
                    forbidTrailingComma(node);
                    return;
                }
                const trailingToken = getTrailingToken(node, lastItem);
                if (trailingToken.value !== ",") {
                    context.report({
                        node: lastItem,
                        loc: {
                            start: trailingToken.loc.end,
                            end: astUtils.getNextLocation(sourceCode, trailingToken.loc.end)
                        },
                        messageId: "missing",
                        fix(fixer) {
                            return fixer.insertTextAfter(trailingToken, ",");
                        }
                    });
                }
            }
            /**
             * If a given node is multiline, reports the last element of a given node
             * when it does not have a trailing comma.
             * Otherwise, reports a trailing comma if it exists.
             * @param {ASTNode} node A node to check. Its type is one of
             *   ObjectExpression, ObjectPattern, ArrayExpression, ArrayPattern,
             *   ImportDeclaration, and ExportNamedDeclaration.
             * @returns {void}
             */
            function forceTrailingCommaIfMultiline(node) {
                if (isMultiline(node)) {
                    forceTrailingComma(node);
                }
                else {
                    forbidTrailingComma(node);
                }
            }
            /**
             * Only if a given node is not multiline, reports the last element of a given node
             * when it does not have a trailing comma.
             * Otherwise, reports a trailing comma if it exists.
             * @param {ASTNode} node A node to check. Its type is one of
             *   ObjectExpression, ObjectPattern, ArrayExpression, ArrayPattern,
             *   ImportDeclaration, and ExportNamedDeclaration.
             * @returns {void}
             */
            function allowTrailingCommaIfMultiline(node) {
                if (!isMultiline(node)) {
                    forbidTrailingComma(node);
                }
            }
            const predicate = {
                always: forceTrailingComma,
                "always-multiline": forceTrailingCommaIfMultiline,
                "only-multiline": allowTrailingCommaIfMultiline,
                never: forbidTrailingComma,
                ignore: lodash.noop
            };
            return {
                ObjectExpression: predicate[options.objects],
                ObjectPattern: predicate[options.objects],
                ArrayExpression: predicate[options.arrays],
                ArrayPattern: predicate[options.arrays],
                ImportDeclaration: predicate[options.imports],
                ExportNamedDeclaration: predicate[options.exports],
                FunctionDeclaration: predicate[options.functions],
                FunctionExpression: predicate[options.functions],
                ArrowFunctionExpression: predicate[options.functions],
                CallExpression: predicate[options.functions],
                NewExpression: predicate[options.functions]
            };
        }
    };
},
/* eslint/lib/rules/comma-spacing.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing before and after commas",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/comma-spacing"
            },
            fixable: "whitespace",
            schema: [
                {
                    type: "object",
                    properties: {
                        before: {
                            type: "boolean",
                            default: false
                        },
                        after: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                missing: "A space is required {{loc}} ','.",
                unexpected: "There should be no space {{loc}} ','."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const tokensAndComments = sourceCode.tokensAndComments;
            const options = {
                before: context.options[0] ? context.options[0].before : false,
                after: context.options[0] ? context.options[0].after : true
            };
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            // list of comma tokens to ignore for the check of leading whitespace
            const commaTokensToIgnore = [];
            /**
             * Reports a spacing error with an appropriate message.
             * @param {ASTNode} node The binary expression node to report.
             * @param {string} loc Is the error "before" or "after" the comma?
             * @param {ASTNode} otherNode The node at the left or right of `node`
             * @returns {void}
             * @private
             */
            function report(node, loc, otherNode) {
                context.report({
                    node,
                    fix(fixer) {
                        if (options[loc]) {
                            if (loc === "before") {
                                return fixer.insertTextBefore(node, " ");
                            }
                            return fixer.insertTextAfter(node, " ");
                        }
                        let start, end;
                        const newText = "";
                        if (loc === "before") {
                            start = otherNode.range[1];
                            end = node.range[0];
                        }
                        else {
                            start = node.range[1];
                            end = otherNode.range[0];
                        }
                        return fixer.replaceTextRange([start, end], newText);
                    },
                    messageId: options[loc] ? "missing" : "unexpected",
                    data: {
                        loc
                    }
                });
            }
            /**
             * Validates the spacing around a comma token.
             * @param {Object} tokens The tokens to be validated.
             * @param {Token} tokens.comma The token representing the comma.
             * @param {Token} [tokens.left] The last token before the comma.
             * @param {Token} [tokens.right] The first token after the comma.
             * @param {Token|ASTNode} reportItem The item to use when reporting an error.
             * @returns {void}
             * @private
             */
            function validateCommaItemSpacing(tokens, reportItem) {
                if (tokens.left && astUtils.isTokenOnSameLine(tokens.left, tokens.comma) &&
                    (options.before !== sourceCode.isSpaceBetweenTokens(tokens.left, tokens.comma))) {
                    report(reportItem, "before", tokens.left);
                }
                if (tokens.right && astUtils.isClosingParenToken(tokens.right)) {
                    return;
                }
                if (tokens.right && !options.after && tokens.right.type === "Line") {
                    return;
                }
                if (tokens.right && astUtils.isTokenOnSameLine(tokens.comma, tokens.right) &&
                    (options.after !== sourceCode.isSpaceBetweenTokens(tokens.comma, tokens.right))) {
                    report(reportItem, "after", tokens.right);
                }
            }
            /**
             * Adds null elements of the given ArrayExpression or ArrayPattern node to the ignore list.
             * @param {ASTNode} node An ArrayExpression or ArrayPattern node.
             * @returns {void}
             */
            function addNullElementsToIgnoreList(node) {
                let previousToken = sourceCode.getFirstToken(node);
                node.elements.forEach(element => {
                    let token;
                    if (element === null) {
                        token = sourceCode.getTokenAfter(previousToken);
                        if (astUtils.isCommaToken(token)) {
                            commaTokensToIgnore.push(token);
                        }
                    }
                    else {
                        token = sourceCode.getTokenAfter(element);
                    }
                    previousToken = token;
                });
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                "Program:exit"() {
                    tokensAndComments.forEach((token, i) => {
                        if (!astUtils.isCommaToken(token)) {
                            return;
                        }
                        if (token && token.type === "JSXText") {
                            return;
                        }
                        const previousToken = tokensAndComments[i - 1];
                        const nextToken = tokensAndComments[i + 1];
                        validateCommaItemSpacing({
                            comma: token,
                            left: astUtils.isCommaToken(previousToken) || commaTokensToIgnore.indexOf(token) > -1 ? null : previousToken,
                            right: astUtils.isCommaToken(nextToken) ? null : nextToken
                        }, token);
                    });
                },
                ArrayExpression: addNullElementsToIgnoreList,
                ArrayPattern: addNullElementsToIgnoreList
            };
        }
    };
},
/* eslint/lib/rules/comma-style.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent comma style",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/comma-style"
            },
            fixable: "code",
            schema: [
                {
                    enum: ["first", "last"]
                },
                {
                    type: "object",
                    properties: {
                        exceptions: {
                            type: "object",
                            additionalProperties: {
                                type: "boolean"
                            }
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpectedLineBeforeAndAfterComma: "Bad line breaking before and after ','.",
                expectedCommaFirst: "',' should be placed first.",
                expectedCommaLast: "',' should be placed last."
            }
        },
        create(context) {
            const style = context.options[0] || "last", sourceCode = context.getSourceCode();
            const exceptions = {
                ArrayPattern: true,
                ArrowFunctionExpression: true,
                CallExpression: true,
                FunctionDeclaration: true,
                FunctionExpression: true,
                ImportDeclaration: true,
                ObjectPattern: true,
                NewExpression: true
            };
            if (context.options.length === 2 && Object.prototype.hasOwnProperty.call(context.options[1], "exceptions")) {
                const keys = Object.keys(context.options[1].exceptions);
                for (let i = 0; i < keys.length; i++) {
                    exceptions[keys[i]] = context.options[1].exceptions[keys[i]];
                }
            }
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Modified text based on the style
             * @param {string} styleType Style type
             * @param {string} text Source code text
             * @returns {string} modified text
             * @private
             */
            function getReplacedText(styleType, text) {
                switch (styleType) {
                    case "between":
                        return `,${text.replace(astUtils.LINEBREAK_MATCHER, "")}`;
                    case "first":
                        return `${text},`;
                    case "last":
                        return `,${text}`;
                    default:
                        return "";
                }
            }
            /**
             * Determines the fixer function for a given style.
             * @param {string} styleType comma style
             * @param {ASTNode} previousItemToken The token to check.
             * @param {ASTNode} commaToken The token to check.
             * @param {ASTNode} currentItemToken The token to check.
             * @returns {Function} Fixer function
             * @private
             */
            function getFixerFunction(styleType, previousItemToken, commaToken, currentItemToken) {
                const text = sourceCode.text.slice(previousItemToken.range[1], commaToken.range[0]) +
                    sourceCode.text.slice(commaToken.range[1], currentItemToken.range[0]);
                const range = [previousItemToken.range[1], currentItemToken.range[0]];
                return function (fixer) {
                    return fixer.replaceTextRange(range, getReplacedText(styleType, text));
                };
            }
            /**
             * Validates the spacing around single items in lists.
             * @param {Token} previousItemToken The last token from the previous item.
             * @param {Token} commaToken The token representing the comma.
             * @param {Token} currentItemToken The first token of the current item.
             * @param {Token} reportItem The item to use when reporting an error.
             * @returns {void}
             * @private
             */
            function validateCommaItemSpacing(previousItemToken, commaToken, currentItemToken, reportItem) {
                // if single line
                if (astUtils.isTokenOnSameLine(commaToken, currentItemToken) &&
                    astUtils.isTokenOnSameLine(previousItemToken, commaToken)) {
                    // do nothing.
                }
                else if (!astUtils.isTokenOnSameLine(commaToken, currentItemToken) &&
                    !astUtils.isTokenOnSameLine(previousItemToken, commaToken)) {
                    const comment = sourceCode.getCommentsAfter(commaToken)[0];
                    const styleType = comment && comment.type === "Block" && astUtils.isTokenOnSameLine(commaToken, comment)
                        ? style
                        : "between";
                    // lone comma
                    context.report({
                        node: reportItem,
                        loc: {
                            line: commaToken.loc.end.line,
                            column: commaToken.loc.start.column
                        },
                        messageId: "unexpectedLineBeforeAndAfterComma",
                        fix: getFixerFunction(styleType, previousItemToken, commaToken, currentItemToken)
                    });
                }
                else if (style === "first" && !astUtils.isTokenOnSameLine(commaToken, currentItemToken)) {
                    context.report({
                        node: reportItem,
                        messageId: "expectedCommaFirst",
                        fix: getFixerFunction(style, previousItemToken, commaToken, currentItemToken)
                    });
                }
                else if (style === "last" && astUtils.isTokenOnSameLine(commaToken, currentItemToken)) {
                    context.report({
                        node: reportItem,
                        loc: {
                            line: commaToken.loc.end.line,
                            column: commaToken.loc.end.column
                        },
                        messageId: "expectedCommaLast",
                        fix: getFixerFunction(style, previousItemToken, commaToken, currentItemToken)
                    });
                }
            }
            /**
             * Checks the comma placement with regards to a declaration/property/element
             * @param {ASTNode} node The binary expression node to check
             * @param {string} property The property of the node containing child nodes.
             * @private
             * @returns {void}
             */
            function validateComma(node, property) {
                const items = node[property], arrayLiteral = (node.type === "ArrayExpression" || node.type === "ArrayPattern");
                if (items.length > 1 || arrayLiteral) {
                    // seed as opening [
                    let previousItemToken = sourceCode.getFirstToken(node);
                    items.forEach(item => {
                        const commaToken = item ? sourceCode.getTokenBefore(item) : previousItemToken, currentItemToken = item ? sourceCode.getFirstToken(item) : sourceCode.getTokenAfter(commaToken), reportItem = item || currentItemToken;
                        /*
                         * This works by comparing three token locations:
                         * - previousItemToken is the last token of the previous item
                         * - commaToken is the location of the comma before the current item
                         * - currentItemToken is the first token of the current item
                         *
                         * These values get switched around if item is undefined.
                         * previousItemToken will refer to the last token not belonging
                         * to the current item, which could be a comma or an opening
                         * square bracket. currentItemToken could be a comma.
                         *
                         * All comparisons are done based on these tokens directly, so
                         * they are always valid regardless of an undefined item.
                         */
                        if (astUtils.isCommaToken(commaToken)) {
                            validateCommaItemSpacing(previousItemToken, commaToken, currentItemToken, reportItem);
                        }
                        if (item) {
                            const tokenAfterItem = sourceCode.getTokenAfter(item, astUtils.isNotClosingParenToken);
                            previousItemToken = tokenAfterItem
                                ? sourceCode.getTokenBefore(tokenAfterItem)
                                : sourceCode.ast.tokens[sourceCode.ast.tokens.length - 1];
                        }
                    });
                    /*
                     * Special case for array literals that have empty last items, such
                     * as [ 1, 2, ]. These arrays only have two items show up in the
                     * AST, so we need to look at the token to verify that there's no
                     * dangling comma.
                     */
                    if (arrayLiteral) {
                        const lastToken = sourceCode.getLastToken(node), nextToLastToken = sourceCode.getTokenBefore(lastToken);
                        if (astUtils.isCommaToken(nextToLastToken)) {
                            validateCommaItemSpacing(sourceCode.getTokenBefore(nextToLastToken), nextToLastToken, lastToken, lastToken);
                        }
                    }
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            const nodes = {};
            if (!exceptions.VariableDeclaration) {
                nodes.VariableDeclaration = function (node) {
                    validateComma(node, "declarations");
                };
            }
            if (!exceptions.ObjectExpression) {
                nodes.ObjectExpression = function (node) {
                    validateComma(node, "properties");
                };
            }
            if (!exceptions.ObjectPattern) {
                nodes.ObjectPattern = function (node) {
                    validateComma(node, "properties");
                };
            }
            if (!exceptions.ArrayExpression) {
                nodes.ArrayExpression = function (node) {
                    validateComma(node, "elements");
                };
            }
            if (!exceptions.ArrayPattern) {
                nodes.ArrayPattern = function (node) {
                    validateComma(node, "elements");
                };
            }
            if (!exceptions.FunctionDeclaration) {
                nodes.FunctionDeclaration = function (node) {
                    validateComma(node, "params");
                };
            }
            if (!exceptions.FunctionExpression) {
                nodes.FunctionExpression = function (node) {
                    validateComma(node, "params");
                };
            }
            if (!exceptions.ArrowFunctionExpression) {
                nodes.ArrowFunctionExpression = function (node) {
                    validateComma(node, "params");
                };
            }
            if (!exceptions.CallExpression) {
                nodes.CallExpression = function (node) {
                    validateComma(node, "arguments");
                };
            }
            if (!exceptions.ImportDeclaration) {
                nodes.ImportDeclaration = function (node) {
                    validateComma(node, "specifiers");
                };
            }
            if (!exceptions.NewExpression) {
                nodes.NewExpression = function (node) {
                    validateComma(node, "arguments");
                };
            }
            return nodes;
        }
    };
},
/* eslint/lib/rules/complexity.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce a maximum cyclomatic complexity allowed in a program",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/complexity"
            },
            schema: [
                {
                    oneOf: [
                        {
                            type: "integer",
                            minimum: 0
                        },
                        {
                            type: "object",
                            properties: {
                                maximum: {
                                    type: "integer",
                                    minimum: 0
                                },
                                max: {
                                    type: "integer",
                                    minimum: 0
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                complex: "{{name}} has a complexity of {{complexity}}. Maximum allowed is {{max}}."
            }
        },
        create(context) {
            const option = context.options[0];
            let THRESHOLD = 20;
            if (typeof option === "object" &&
                (Object.prototype.hasOwnProperty.call(option, "maximum") || Object.prototype.hasOwnProperty.call(option, "max"))) {
                THRESHOLD = option.maximum || option.max;
            }
            else if (typeof option === "number") {
                THRESHOLD = option;
            }
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            // Using a stack to store complexity (handling nested functions)
            const fns = [];
            /**
             * When parsing a new function, store it in our function stack
             * @returns {void}
             * @private
             */
            function startFunction() {
                fns.push(1);
            }
            /**
             * Evaluate the node at the end of function
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function endFunction(node) {
                const name = lodash.upperFirst(astUtils.getFunctionNameWithKind(node));
                const complexity = fns.pop();
                if (complexity > THRESHOLD) {
                    context.report({
                        node,
                        messageId: "complex",
                        data: { name, complexity, max: THRESHOLD }
                    });
                }
            }
            /**
             * Increase the complexity of the function in context
             * @returns {void}
             * @private
             */
            function increaseComplexity() {
                if (fns.length) {
                    fns[fns.length - 1]++;
                }
            }
            /**
             * Increase the switch complexity in context
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function increaseSwitchComplexity(node) {
                // Avoiding `default`
                if (node.test) {
                    increaseComplexity();
                }
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                FunctionDeclaration: startFunction,
                FunctionExpression: startFunction,
                ArrowFunctionExpression: startFunction,
                "FunctionDeclaration:exit": endFunction,
                "FunctionExpression:exit": endFunction,
                "ArrowFunctionExpression:exit": endFunction,
                CatchClause: increaseComplexity,
                ConditionalExpression: increaseComplexity,
                LogicalExpression: increaseComplexity,
                ForStatement: increaseComplexity,
                ForInStatement: increaseComplexity,
                ForOfStatement: increaseComplexity,
                IfStatement: increaseComplexity,
                SwitchCase: increaseSwitchComplexity,
                WhileStatement: increaseComplexity,
                DoWhileStatement: increaseComplexity
            };
        }
    };
},
/* eslint/lib/rules/computed-property-spacing.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing inside computed property brackets",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/computed-property-spacing"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["always", "never"]
                },
                {
                    type: "object",
                    properties: {
                        enforceForClassMembers: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpectedSpaceBefore: "There should be no space before '{{tokenValue}}'.",
                unexpectedSpaceAfter: "There should be no space after '{{tokenValue}}'.",
                missingSpaceBefore: "A space is required before '{{tokenValue}}'.",
                missingSpaceAfter: "A space is required after '{{tokenValue}}'."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const propertyNameMustBeSpaced = context.options[0] === "always"; // default is "never"
            const enforceForClassMembers = context.options[1] && context.options[1].enforceForClassMembers;
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Reports that there shouldn't be a space after the first token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @param {Token} tokenAfter The token after `token`.
             * @returns {void}
             */
            function reportNoBeginningSpace(node, token, tokenAfter) {
                context.report({
                    node,
                    loc: token.loc.start,
                    messageId: "unexpectedSpaceAfter",
                    data: {
                        tokenValue: token.value
                    },
                    fix(fixer) {
                        return fixer.removeRange([token.range[1], tokenAfter.range[0]]);
                    }
                });
            }
            /**
             * Reports that there shouldn't be a space before the last token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @param {Token} tokenBefore The token before `token`.
             * @returns {void}
             */
            function reportNoEndingSpace(node, token, tokenBefore) {
                context.report({
                    node,
                    loc: token.loc.start,
                    messageId: "unexpectedSpaceBefore",
                    data: {
                        tokenValue: token.value
                    },
                    fix(fixer) {
                        return fixer.removeRange([tokenBefore.range[1], token.range[0]]);
                    }
                });
            }
            /**
             * Reports that there should be a space after the first token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportRequiredBeginningSpace(node, token) {
                context.report({
                    node,
                    loc: token.loc.start,
                    messageId: "missingSpaceAfter",
                    data: {
                        tokenValue: token.value
                    },
                    fix(fixer) {
                        return fixer.insertTextAfter(token, " ");
                    }
                });
            }
            /**
             * Reports that there should be a space before the last token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportRequiredEndingSpace(node, token) {
                context.report({
                    node,
                    loc: token.loc.start,
                    messageId: "missingSpaceBefore",
                    data: {
                        tokenValue: token.value
                    },
                    fix(fixer) {
                        return fixer.insertTextBefore(token, " ");
                    }
                });
            }
            /**
             * Returns a function that checks the spacing of a node on the property name
             * that was passed in.
             * @param {string} propertyName The property on the node to check for spacing
             * @returns {Function} A function that will check spacing on a node
             */
            function checkSpacing(propertyName) {
                return function (node) {
                    if (!node.computed) {
                        return;
                    }
                    const property = node[propertyName];
                    const before = sourceCode.getTokenBefore(property, astUtils.isOpeningBracketToken), first = sourceCode.getTokenAfter(before, { includeComments: true }), after = sourceCode.getTokenAfter(property, astUtils.isClosingBracketToken), last = sourceCode.getTokenBefore(after, { includeComments: true });
                    if (astUtils.isTokenOnSameLine(before, first)) {
                        if (propertyNameMustBeSpaced) {
                            if (!sourceCode.isSpaceBetweenTokens(before, first) && astUtils.isTokenOnSameLine(before, first)) {
                                reportRequiredBeginningSpace(node, before);
                            }
                        }
                        else {
                            if (sourceCode.isSpaceBetweenTokens(before, first)) {
                                reportNoBeginningSpace(node, before, first);
                            }
                        }
                    }
                    if (astUtils.isTokenOnSameLine(last, after)) {
                        if (propertyNameMustBeSpaced) {
                            if (!sourceCode.isSpaceBetweenTokens(last, after) && astUtils.isTokenOnSameLine(last, after)) {
                                reportRequiredEndingSpace(node, after);
                            }
                        }
                        else {
                            if (sourceCode.isSpaceBetweenTokens(last, after)) {
                                reportNoEndingSpace(node, after, last);
                            }
                        }
                    }
                };
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            const listeners = {
                Property: checkSpacing("key"),
                MemberExpression: checkSpacing("property")
            };
            if (enforceForClassMembers) {
                listeners.MethodDefinition = checkSpacing("key");
            }
            return listeners;
        }
    };
},
/* eslint/lib/rules/consistent-return.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a given node is an `Identifier` node which was named a given name.
     * @param {ASTNode} node A node to check.
     * @param {string} name An expected name of the node.
     * @returns {boolean} `true` if the node is an `Identifier` node which was named as expected.
     */
    function isIdentifier(node, name) {
        return node.type === "Identifier" && node.name === name;
    }
    /**
     * Checks whether or not a given code path segment is unreachable.
     * @param {CodePathSegment} segment A CodePathSegment to check.
     * @returns {boolean} `true` if the segment is unreachable.
     */
    function isUnreachable(segment) {
        return !segment.reachable;
    }
    /**
     * Checks whether a given node is a `constructor` method in an ES6 class
     * @param {ASTNode} node A node to check
     * @returns {boolean} `true` if the node is a `constructor` method
     */
    function isClassConstructor(node) {
        return node.type === "FunctionExpression" &&
            node.parent &&
            node.parent.type === "MethodDefinition" &&
            node.parent.kind === "constructor";
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require `return` statements to either always or never specify values",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/consistent-return"
            },
            schema: [{
                    type: "object",
                    properties: {
                        treatUndefinedAsUnspecified: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }],
            messages: {
                missingReturn: "Expected to return a value at the end of {{name}}.",
                missingReturnValue: "{{name}} expected a return value.",
                unexpectedReturnValue: "{{name}} expected no return value."
            }
        },
        create(context) {
            const options = context.options[0] || {};
            const treatUndefinedAsUnspecified = options.treatUndefinedAsUnspecified === true;
            let funcInfo = null;
            /**
             * Checks whether of not the implicit returning is consistent if the last
             * code path segment is reachable.
             * @param {ASTNode} node A program/function node to check.
             * @returns {void}
             */
            function checkLastSegment(node) {
                let loc, name;
                /*
                 * Skip if it expected no return value or unreachable.
                 * When unreachable, all paths are returned or thrown.
                 */
                if (!funcInfo.hasReturnValue ||
                    funcInfo.codePath.currentSegments.every(isUnreachable) ||
                    astUtils.isES5Constructor(node) ||
                    isClassConstructor(node)) {
                    return;
                }
                // Adjust a location and a message.
                if (node.type === "Program") {
                    // The head of program.
                    loc = { line: 1, column: 0 };
                    name = "program";
                }
                else if (node.type === "ArrowFunctionExpression") {
                    // `=>` token
                    loc = context.getSourceCode().getTokenBefore(node.body, astUtils.isArrowToken).loc.start;
                }
                else if (node.parent.type === "MethodDefinition" ||
                    (node.parent.type === "Property" && node.parent.method)) {
                    // Method name.
                    loc = node.parent.key.loc.start;
                }
                else {
                    // Function name or `function` keyword.
                    loc = (node.id || node).loc.start;
                }
                if (!name) {
                    name = astUtils.getFunctionNameWithKind(node);
                }
                // Reports.
                context.report({
                    node,
                    loc,
                    messageId: "missingReturn",
                    data: { name }
                });
            }
            return {
                // Initializes/Disposes state of each code path.
                onCodePathStart(codePath, node) {
                    funcInfo = {
                        upper: funcInfo,
                        codePath,
                        hasReturn: false,
                        hasReturnValue: false,
                        messageId: "",
                        node
                    };
                },
                onCodePathEnd() {
                    funcInfo = funcInfo.upper;
                },
                // Reports a given return statement if it's inconsistent.
                ReturnStatement(node) {
                    const argument = node.argument;
                    let hasReturnValue = Boolean(argument);
                    if (treatUndefinedAsUnspecified && hasReturnValue) {
                        hasReturnValue = !isIdentifier(argument, "undefined") && argument.operator !== "void";
                    }
                    if (!funcInfo.hasReturn) {
                        funcInfo.hasReturn = true;
                        funcInfo.hasReturnValue = hasReturnValue;
                        funcInfo.messageId = hasReturnValue ? "missingReturnValue" : "unexpectedReturnValue";
                        funcInfo.data = {
                            name: funcInfo.node.type === "Program"
                                ? "Program"
                                : lodash.upperFirst(astUtils.getFunctionNameWithKind(funcInfo.node))
                        };
                    }
                    else if (funcInfo.hasReturnValue !== hasReturnValue) {
                        context.report({
                            node,
                            messageId: funcInfo.messageId,
                            data: funcInfo.data
                        });
                    }
                },
                // Reports a given program/function if the implicit returning is not consistent.
                "Program:exit": checkLastSegment,
                "FunctionDeclaration:exit": checkLastSegment,
                "FunctionExpression:exit": checkLastSegment,
                "ArrowFunctionExpression:exit": checkLastSegment
            };
        }
    };
},
/* eslint/lib/rules/consistent-this.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce consistent naming when capturing the current execution context",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/consistent-this"
            },
            schema: {
                type: "array",
                items: {
                    type: "string",
                    minLength: 1
                },
                uniqueItems: true
            },
            messages: {
                aliasNotAssignedToThis: "Designated alias '{{name}}' is not assigned to 'this'.",
                unexpectedAlias: "Unexpected alias '{{name}}' for 'this'."
            }
        },
        create(context) {
            let aliases = [];
            if (context.options.length === 0) {
                aliases.push("that");
            }
            else {
                aliases = context.options;
            }
            /**
             * Reports that a variable declarator or assignment expression is assigning
             * a non-'this' value to the specified alias.
             * @param {ASTNode} node The assigning node.
             * @param {string}  name the name of the alias that was incorrectly used.
             * @returns {void}
             */
            function reportBadAssignment(node, name) {
                context.report({ node, messageId: "aliasNotAssignedToThis", data: { name } });
            }
            /**
             * Checks that an assignment to an identifier only assigns 'this' to the
             * appropriate alias, and the alias is only assigned to 'this'.
             * @param {ASTNode} node The assigning node.
             * @param {Identifier} name The name of the variable assigned to.
             * @param {Expression} value The value of the assignment.
             * @returns {void}
             */
            function checkAssignment(node, name, value) {
                const isThis = value.type === "ThisExpression";
                if (aliases.indexOf(name) !== -1) {
                    if (!isThis || node.operator && node.operator !== "=") {
                        reportBadAssignment(node, name);
                    }
                }
                else if (isThis) {
                    context.report({ node, messageId: "unexpectedAlias", data: { name } });
                }
            }
            /**
             * Ensures that a variable declaration of the alias in a program or function
             * is assigned to the correct value.
             * @param {string} alias alias the check the assignment of.
             * @param {Object} scope scope of the current code we are checking.
             * @private
             * @returns {void}
             */
            function checkWasAssigned(alias, scope) {
                const variable = scope.set.get(alias);
                if (!variable) {
                    return;
                }
                if (variable.defs.some(def => def.node.type === "VariableDeclarator" &&
                    def.node.init !== null)) {
                    return;
                }
                /*
                 * The alias has been declared and not assigned: check it was
                 * assigned later in the same scope.
                 */
                if (!variable.references.some(reference => {
                    const write = reference.writeExpr;
                    return (reference.from === scope &&
                        write && write.type === "ThisExpression" &&
                        write.parent.operator === "=");
                })) {
                    variable.defs.map(def => def.node).forEach(node => {
                        reportBadAssignment(node, alias);
                    });
                }
            }
            /**
             * Check each alias to ensure that is was assinged to the correct value.
             * @returns {void}
             */
            function ensureWasAssigned() {
                const scope = context.getScope();
                aliases.forEach(alias => {
                    checkWasAssigned(alias, scope);
                });
            }
            return {
                "Program:exit": ensureWasAssigned,
                "FunctionExpression:exit": ensureWasAssigned,
                "FunctionDeclaration:exit": ensureWasAssigned,
                VariableDeclarator(node) {
                    const id = node.id;
                    const isDestructuring = id.type === "ArrayPattern" || id.type === "ObjectPattern";
                    if (node.init !== null && !isDestructuring) {
                        checkAssignment(node, id.name, node.init);
                    }
                },
                AssignmentExpression(node) {
                    if (node.left.type === "Identifier") {
                        checkAssignment(node, node.left.name, node.right);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/constructor-super.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether a given code path segment is reachable or not.
     * @param {CodePathSegment} segment A code path segment to check.
     * @returns {boolean} `true` if the segment is reachable.
     */
    function isReachable(segment) {
        return segment.reachable;
    }
    /**
     * Checks whether or not a given node is a constructor.
     * @param {ASTNode} node A node to check. This node type is one of
     *   `Program`, `FunctionDeclaration`, `FunctionExpression`, and
     *   `ArrowFunctionExpression`.
     * @returns {boolean} `true` if the node is a constructor.
     */
    function isConstructorFunction(node) {
        return (node.type === "FunctionExpression" &&
            node.parent.type === "MethodDefinition" &&
            node.parent.kind === "constructor");
    }
    /**
     * Checks whether a given node can be a constructor or not.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node can be a constructor.
     */
    function isPossibleConstructor(node) {
        if (!node) {
            return false;
        }
        switch (node.type) {
            case "ClassExpression":
            case "FunctionExpression":
            case "ThisExpression":
            case "MemberExpression":
            case "CallExpression":
            case "NewExpression":
            case "YieldExpression":
            case "TaggedTemplateExpression":
            case "MetaProperty":
                return true;
            case "Identifier":
                return node.name !== "undefined";
            case "AssignmentExpression":
                return isPossibleConstructor(node.right);
            case "LogicalExpression":
                return (isPossibleConstructor(node.left) ||
                    isPossibleConstructor(node.right));
            case "ConditionalExpression":
                return (isPossibleConstructor(node.alternate) ||
                    isPossibleConstructor(node.consequent));
            case "SequenceExpression": {
                const lastExpression = node.expressions[node.expressions.length - 1];
                return isPossibleConstructor(lastExpression);
            }
            default:
                return false;
        }
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "require `super()` calls in constructors",
                category: "ECMAScript 6",
                recommended: true,
                url: "https://eslint.org/docs/rules/constructor-super"
            },
            schema: [],
            messages: {
                missingSome: "Lacked a call of 'super()' in some code paths.",
                missingAll: "Expected to call 'super()'.",
                duplicate: "Unexpected duplicate 'super()'.",
                badSuper: "Unexpected 'super()' because 'super' is not a constructor.",
                unexpected: "Unexpected 'super()'."
            }
        },
        create(context) {
            /*
             * {{hasExtends: boolean, scope: Scope, codePath: CodePath}[]}
             * Information for each constructor.
             * - upper:      Information of the upper constructor.
             * - hasExtends: A flag which shows whether own class has a valid `extends`
             *               part.
             * - scope:      The scope of own class.
             * - codePath:   The code path object of the constructor.
             */
            let funcInfo = null;
            /*
             * {Map<string, {calledInSomePaths: boolean, calledInEveryPaths: boolean}>}
             * Information for each code path segment.
             * - calledInSomePaths:  A flag of be called `super()` in some code paths.
             * - calledInEveryPaths: A flag of be called `super()` in all code paths.
             * - validNodes:
             */
            let segInfoMap = Object.create(null);
            /**
             * Gets the flag which shows `super()` is called in some paths.
             * @param {CodePathSegment} segment A code path segment to get.
             * @returns {boolean} The flag which shows `super()` is called in some paths
             */
            function isCalledInSomePath(segment) {
                return segment.reachable && segInfoMap[segment.id].calledInSomePaths;
            }
            /**
             * Gets the flag which shows `super()` is called in all paths.
             * @param {CodePathSegment} segment A code path segment to get.
             * @returns {boolean} The flag which shows `super()` is called in all paths.
             */
            function isCalledInEveryPath(segment) {
                /*
                 * If specific segment is the looped segment of the current segment,
                 * skip the segment.
                 * If not skipped, this never becomes true after a loop.
                 */
                if (segment.nextSegments.length === 1 &&
                    segment.nextSegments[0].isLoopedPrevSegment(segment)) {
                    return true;
                }
                return segment.reachable && segInfoMap[segment.id].calledInEveryPaths;
            }
            return {
                /**
                 * Stacks a constructor information.
                 * @param {CodePath} codePath A code path which was started.
                 * @param {ASTNode} node The current node.
                 * @returns {void}
                 */
                onCodePathStart(codePath, node) {
                    if (isConstructorFunction(node)) {
                        // Class > ClassBody > MethodDefinition > FunctionExpression
                        const classNode = node.parent.parent.parent;
                        const superClass = classNode.superClass;
                        funcInfo = {
                            upper: funcInfo,
                            isConstructor: true,
                            hasExtends: Boolean(superClass),
                            superIsConstructor: isPossibleConstructor(superClass),
                            codePath
                        };
                    }
                    else {
                        funcInfo = {
                            upper: funcInfo,
                            isConstructor: false,
                            hasExtends: false,
                            superIsConstructor: false,
                            codePath
                        };
                    }
                },
                /**
                 * Pops a constructor information.
                 * And reports if `super()` lacked.
                 * @param {CodePath} codePath A code path which was ended.
                 * @param {ASTNode} node The current node.
                 * @returns {void}
                 */
                onCodePathEnd(codePath, node) {
                    const hasExtends = funcInfo.hasExtends;
                    // Pop.
                    funcInfo = funcInfo.upper;
                    if (!hasExtends) {
                        return;
                    }
                    // Reports if `super()` lacked.
                    const segments = codePath.returnedSegments;
                    const calledInEveryPaths = segments.every(isCalledInEveryPath);
                    const calledInSomePaths = segments.some(isCalledInSomePath);
                    if (!calledInEveryPaths) {
                        context.report({
                            messageId: calledInSomePaths
                                ? "missingSome"
                                : "missingAll",
                            node: node.parent
                        });
                    }
                },
                /**
                 * Initialize information of a given code path segment.
                 * @param {CodePathSegment} segment A code path segment to initialize.
                 * @returns {void}
                 */
                onCodePathSegmentStart(segment) {
                    if (!(funcInfo && funcInfo.isConstructor && funcInfo.hasExtends)) {
                        return;
                    }
                    // Initialize info.
                    const info = segInfoMap[segment.id] = {
                        calledInSomePaths: false,
                        calledInEveryPaths: false,
                        validNodes: []
                    };
                    // When there are previous segments, aggregates these.
                    const prevSegments = segment.prevSegments;
                    if (prevSegments.length > 0) {
                        info.calledInSomePaths = prevSegments.some(isCalledInSomePath);
                        info.calledInEveryPaths = prevSegments.every(isCalledInEveryPath);
                    }
                },
                /**
                 * Update information of the code path segment when a code path was
                 * looped.
                 * @param {CodePathSegment} fromSegment The code path segment of the
                 *      end of a loop.
                 * @param {CodePathSegment} toSegment A code path segment of the head
                 *      of a loop.
                 * @returns {void}
                 */
                onCodePathSegmentLoop(fromSegment, toSegment) {
                    if (!(funcInfo && funcInfo.isConstructor && funcInfo.hasExtends)) {
                        return;
                    }
                    // Update information inside of the loop.
                    const isRealLoop = toSegment.prevSegments.length >= 2;
                    funcInfo.codePath.traverseSegments({ first: toSegment, last: fromSegment }, segment => {
                        const info = segInfoMap[segment.id];
                        const prevSegments = segment.prevSegments;
                        // Updates flags.
                        info.calledInSomePaths = prevSegments.some(isCalledInSomePath);
                        info.calledInEveryPaths = prevSegments.every(isCalledInEveryPath);
                        // If flags become true anew, reports the valid nodes.
                        if (info.calledInSomePaths || isRealLoop) {
                            const nodes = info.validNodes;
                            info.validNodes = [];
                            for (let i = 0; i < nodes.length; ++i) {
                                const node = nodes[i];
                                context.report({
                                    messageId: "duplicate",
                                    node
                                });
                            }
                        }
                    });
                },
                /**
                 * Checks for a call of `super()`.
                 * @param {ASTNode} node A CallExpression node to check.
                 * @returns {void}
                 */
                "CallExpression:exit"(node) {
                    if (!(funcInfo && funcInfo.isConstructor)) {
                        return;
                    }
                    // Skips except `super()`.
                    if (node.callee.type !== "Super") {
                        return;
                    }
                    // Reports if needed.
                    if (funcInfo.hasExtends) {
                        const segments = funcInfo.codePath.currentSegments;
                        let duplicate = false;
                        let info = null;
                        for (let i = 0; i < segments.length; ++i) {
                            const segment = segments[i];
                            if (segment.reachable) {
                                info = segInfoMap[segment.id];
                                duplicate = duplicate || info.calledInSomePaths;
                                info.calledInSomePaths = info.calledInEveryPaths = true;
                            }
                        }
                        if (info) {
                            if (duplicate) {
                                context.report({
                                    messageId: "duplicate",
                                    node
                                });
                            }
                            else if (!funcInfo.superIsConstructor) {
                                context.report({
                                    messageId: "badSuper",
                                    node
                                });
                            }
                            else {
                                info.validNodes.push(node);
                            }
                        }
                    }
                    else if (funcInfo.codePath.currentSegments.some(isReachable)) {
                        context.report({
                            messageId: "unexpected",
                            node
                        });
                    }
                },
                /**
                 * Set the mark to the returned path as `super()` was called.
                 * @param {ASTNode} node A ReturnStatement node to check.
                 * @returns {void}
                 */
                ReturnStatement(node) {
                    if (!(funcInfo && funcInfo.isConstructor && funcInfo.hasExtends)) {
                        return;
                    }
                    // Skips if no argument.
                    if (!node.argument) {
                        return;
                    }
                    // Returning argument is a substitute of 'super()'.
                    const segments = funcInfo.codePath.currentSegments;
                    for (let i = 0; i < segments.length; ++i) {
                        const segment = segments[i];
                        if (segment.reachable) {
                            const info = segInfoMap[segment.id];
                            info.calledInSomePaths = info.calledInEveryPaths = true;
                        }
                    }
                },
                /**
                 * Resets state.
                 * @returns {void}
                 */
                "Program:exit"() {
                    segInfoMap = Object.create(null);
                }
            };
        }
    };
},
/* eslint/lib/rules/curly.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce consistent brace style for all control statements",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/curly"
            },
            schema: {
                anyOf: [
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["all"]
                            }
                        ],
                        minItems: 0,
                        maxItems: 1
                    },
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["multi", "multi-line", "multi-or-nest"]
                            },
                            {
                                enum: ["consistent"]
                            }
                        ],
                        minItems: 0,
                        maxItems: 2
                    }
                ]
            },
            fixable: "code",
            messages: {
                missingCurlyAfter: "Expected { after '{{name}}'.",
                missingCurlyAfterCondition: "Expected { after '{{name}}' condition.",
                unexpectedCurlyAfter: "Unnecessary { after '{{name}}'.",
                unexpectedCurlyAfterCondition: "Unnecessary { after '{{name}}' condition."
            }
        },
        create(context) {
            const multiOnly = (context.options[0] === "multi");
            const multiLine = (context.options[0] === "multi-line");
            const multiOrNest = (context.options[0] === "multi-or-nest");
            const consistent = (context.options[1] === "consistent");
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Determines if a given node is a one-liner that's on the same line as it's preceding code.
             * @param {ASTNode} node The node to check.
             * @returns {boolean} True if the node is a one-liner that's on the same line as it's preceding code.
             * @private
             */
            function isCollapsedOneLiner(node) {
                const before = sourceCode.getTokenBefore(node);
                const last = sourceCode.getLastToken(node);
                const lastExcludingSemicolon = astUtils.isSemicolonToken(last) ? sourceCode.getTokenBefore(last) : last;
                return before.loc.start.line === lastExcludingSemicolon.loc.end.line;
            }
            /**
             * Determines if a given node is a one-liner.
             * @param {ASTNode} node The node to check.
             * @returns {boolean} True if the node is a one-liner.
             * @private
             */
            function isOneLiner(node) {
                if (node.type === "EmptyStatement") {
                    return true;
                }
                const first = sourceCode.getFirstToken(node);
                const last = sourceCode.getLastToken(node);
                const lastExcludingSemicolon = astUtils.isSemicolonToken(last) ? sourceCode.getTokenBefore(last) : last;
                return first.loc.start.line === lastExcludingSemicolon.loc.end.line;
            }
            /**
             * Determines if the given node is a lexical declaration (let, const, function, or class)
             * @param {ASTNode} node The node to check
             * @returns {boolean} True if the node is a lexical declaration
             * @private
             */
            function isLexicalDeclaration(node) {
                if (node.type === "VariableDeclaration") {
                    return node.kind === "const" || node.kind === "let";
                }
                return node.type === "FunctionDeclaration" || node.type === "ClassDeclaration";
            }
            /**
             * Checks if the given token is an `else` token or not.
             * @param {Token} token The token to check.
             * @returns {boolean} `true` if the token is an `else` token.
             */
            function isElseKeywordToken(token) {
                return token.value === "else" && token.type === "Keyword";
            }
            /**
             * Gets the `else` keyword token of a given `IfStatement` node.
             * @param {ASTNode} node A `IfStatement` node to get.
             * @returns {Token} The `else` keyword token.
             */
            function getElseKeyword(node) {
                return node.alternate && sourceCode.getFirstTokenBetween(node.consequent, node.alternate, isElseKeywordToken);
            }
            /**
             * Checks a given IfStatement node requires braces of the consequent chunk.
             * This returns `true` when below:
             *
             * 1. The given node has the `alternate` node.
             * 2. There is a `IfStatement` which doesn't have `alternate` node in the
             *    trailing statement chain of the `consequent` node.
             * @param {ASTNode} node A IfStatement node to check.
             * @returns {boolean} `true` if the node requires braces of the consequent chunk.
             */
            function requiresBraceOfConsequent(node) {
                if (node.alternate && node.consequent.type === "BlockStatement") {
                    if (node.consequent.body.length >= 2) {
                        return true;
                    }
                    for (let currentNode = node.consequent.body[0]; currentNode; currentNode = astUtils.getTrailingStatement(currentNode)) {
                        if (currentNode.type === "IfStatement" && !currentNode.alternate) {
                            return true;
                        }
                    }
                }
                return false;
            }
            /**
             * Determines if a semicolon needs to be inserted after removing a set of curly brackets, in order to avoid a SyntaxError.
             * @param {Token} closingBracket The } token
             * @returns {boolean} `true` if a semicolon needs to be inserted after the last statement in the block.
             */
            function needsSemicolon(closingBracket) {
                const tokenBefore = sourceCode.getTokenBefore(closingBracket);
                const tokenAfter = sourceCode.getTokenAfter(closingBracket);
                const lastBlockNode = sourceCode.getNodeByRangeIndex(tokenBefore.range[0]);
                if (astUtils.isSemicolonToken(tokenBefore)) {
                    // If the last statement already has a semicolon, don't add another one.
                    return false;
                }
                if (!tokenAfter) {
                    // If there are no statements after this block, there is no need to add a semicolon.
                    return false;
                }
                if (lastBlockNode.type === "BlockStatement" && lastBlockNode.parent.type !== "FunctionExpression" && lastBlockNode.parent.type !== "ArrowFunctionExpression") {
                    /*
                     * If the last node surrounded by curly brackets is a BlockStatement (other than a FunctionExpression or an ArrowFunctionExpression),
                     * don't insert a semicolon. Otherwise, the semicolon would be parsed as a separate statement, which would cause
                     * a SyntaxError if it was followed by `else`.
                     */
                    return false;
                }
                if (tokenBefore.loc.end.line === tokenAfter.loc.start.line) {
                    // If the next token is on the same line, insert a semicolon.
                    return true;
                }
                if (/^[([/`+-]/u.test(tokenAfter.value)) {
                    // If the next token starts with a character that would disrupt ASI, insert a semicolon.
                    return true;
                }
                if (tokenBefore.type === "Punctuator" && (tokenBefore.value === "++" || tokenBefore.value === "--")) {
                    // If the last token is ++ or --, insert a semicolon to avoid disrupting ASI.
                    return true;
                }
                // Otherwise, do not insert a semicolon.
                return false;
            }
            /**
             * Prepares to check the body of a node to see if it's a block statement.
             * @param {ASTNode} node The node to report if there's a problem.
             * @param {ASTNode} body The body node to check for blocks.
             * @param {string} name The name to report if there's a problem.
             * @param {{ condition: boolean }} opts Options to pass to the report functions
             * @returns {Object} a prepared check object, with "actual", "expected", "check" properties.
             *   "actual" will be `true` or `false` whether the body is already a block statement.
             *   "expected" will be `true` or `false` if the body should be a block statement or not, or
             *   `null` if it doesn't matter, depending on the rule options. It can be modified to change
             *   the final behavior of "check".
             *   "check" will be a function reporting appropriate problems depending on the other
             *   properties.
             */
            function prepareCheck(node, body, name, opts) {
                const hasBlock = (body.type === "BlockStatement");
                let expected = null;
                if (node.type === "IfStatement" && node.consequent === body && requiresBraceOfConsequent(node)) {
                    expected = true;
                }
                else if (multiOnly) {
                    if (hasBlock && body.body.length === 1 && !isLexicalDeclaration(body.body[0])) {
                        expected = false;
                    }
                }
                else if (multiLine) {
                    if (!isCollapsedOneLiner(body)) {
                        expected = true;
                    }
                }
                else if (multiOrNest) {
                    if (hasBlock && body.body.length === 1 && isOneLiner(body.body[0])) {
                        const leadingComments = sourceCode.getCommentsBefore(body.body[0]);
                        const isLexDef = isLexicalDeclaration(body.body[0]);
                        if (isLexDef) {
                            expected = true;
                        }
                        else {
                            expected = leadingComments.length > 0;
                        }
                    }
                    else if (!isOneLiner(body)) {
                        expected = true;
                    }
                }
                else {
                    expected = true;
                }
                return {
                    actual: hasBlock,
                    expected,
                    check() {
                        if (this.expected !== null && this.expected !== this.actual) {
                            if (this.expected) {
                                context.report({
                                    node,
                                    loc: (name !== "else" ? node : getElseKeyword(node)).loc.start,
                                    messageId: opts && opts.condition ? "missingCurlyAfterCondition" : "missingCurlyAfter",
                                    data: {
                                        name
                                    },
                                    fix: fixer => fixer.replaceText(body, `{${sourceCode.getText(body)}}`)
                                });
                            }
                            else {
                                context.report({
                                    node,
                                    loc: (name !== "else" ? node : getElseKeyword(node)).loc.start,
                                    messageId: opts && opts.condition ? "unexpectedCurlyAfterCondition" : "unexpectedCurlyAfter",
                                    data: {
                                        name
                                    },
                                    fix(fixer) {
                                        /*
                                         * `do while` expressions sometimes need a space to be inserted after `do`.
                                         * e.g. `do{foo()} while (bar)` should be corrected to `do foo() while (bar)`
                                         */
                                        const needsPrecedingSpace = node.type === "DoWhileStatement" &&
                                            sourceCode.getTokenBefore(body).range[1] === body.range[0] &&
                                            !astUtils.canTokensBeAdjacent("do", sourceCode.getFirstToken(body, { skip: 1 }));
                                        const openingBracket = sourceCode.getFirstToken(body);
                                        const closingBracket = sourceCode.getLastToken(body);
                                        const lastTokenInBlock = sourceCode.getTokenBefore(closingBracket);
                                        if (needsSemicolon(closingBracket)) {
                                            /*
                                             * If removing braces would cause a SyntaxError due to multiple statements on the same line (or
                                             * change the semantics of the code due to ASI), don't perform a fix.
                                             */
                                            return null;
                                        }
                                        const resultingBodyText = sourceCode.getText().slice(openingBracket.range[1], lastTokenInBlock.range[0]) +
                                            sourceCode.getText(lastTokenInBlock) +
                                            sourceCode.getText().slice(lastTokenInBlock.range[1], closingBracket.range[0]);
                                        return fixer.replaceText(body, (needsPrecedingSpace ? " " : "") + resultingBodyText);
                                    }
                                });
                            }
                        }
                    }
                };
            }
            /**
             * Prepares to check the bodies of a "if", "else if" and "else" chain.
             * @param {ASTNode} node The first IfStatement node of the chain.
             * @returns {Object[]} prepared checks for each body of the chain. See `prepareCheck` for more
             *   information.
             */
            function prepareIfChecks(node) {
                const preparedChecks = [];
                for (let currentNode = node; currentNode; currentNode = currentNode.alternate) {
                    preparedChecks.push(prepareCheck(currentNode, currentNode.consequent, "if", { condition: true }));
                    if (currentNode.alternate && currentNode.alternate.type !== "IfStatement") {
                        preparedChecks.push(prepareCheck(currentNode, currentNode.alternate, "else"));
                        break;
                    }
                }
                if (consistent) {
                    /*
                     * If any node should have or already have braces, make sure they
                     * all have braces.
                     * If all nodes shouldn't have braces, make sure they don't.
                     */
                    const expected = preparedChecks.some(preparedCheck => {
                        if (preparedCheck.expected !== null) {
                            return preparedCheck.expected;
                        }
                        return preparedCheck.actual;
                    });
                    preparedChecks.forEach(preparedCheck => {
                        preparedCheck.expected = expected;
                    });
                }
                return preparedChecks;
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                IfStatement(node) {
                    if (node.parent.type !== "IfStatement") {
                        prepareIfChecks(node).forEach(preparedCheck => {
                            preparedCheck.check();
                        });
                    }
                },
                WhileStatement(node) {
                    prepareCheck(node, node.body, "while", { condition: true }).check();
                },
                DoWhileStatement(node) {
                    prepareCheck(node, node.body, "do").check();
                },
                ForStatement(node) {
                    prepareCheck(node, node.body, "for", { condition: true }).check();
                },
                ForInStatement(node) {
                    prepareCheck(node, node.body, "for-in").check();
                },
                ForOfStatement(node) {
                    prepareCheck(node, node.body, "for-of").check();
                }
            };
        }
    };
},
/* eslint/lib/rules/default-case.js */ function _(require, module, exports) {
    const DEFAULT_COMMENT_PATTERN = /^no default$/iu;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require `default` cases in `switch` statements",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/default-case"
            },
            schema: [{
                    type: "object",
                    properties: {
                        commentPattern: {
                            type: "string"
                        }
                    },
                    additionalProperties: false
                }],
            messages: {
                missingDefaultCase: "Expected a default case."
            }
        },
        create(context) {
            const options = context.options[0] || {};
            const commentPattern = options.commentPattern
                ? new RegExp(options.commentPattern, "u")
                : DEFAULT_COMMENT_PATTERN;
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Shortcut to get last element of array
             * @param  {*[]} collection Array
             * @returns {*} Last element
             */
            function last(collection) {
                return collection[collection.length - 1];
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                SwitchStatement(node) {
                    if (!node.cases.length) {
                        /*
                         * skip check of empty switch because there is no easy way
                         * to extract comments inside it now
                         */
                        return;
                    }
                    const hasDefault = node.cases.some(v => v.test === null);
                    if (!hasDefault) {
                        let comment;
                        const lastCase = last(node.cases);
                        const comments = sourceCode.getCommentsAfter(lastCase);
                        if (comments.length) {
                            comment = last(comments);
                        }
                        if (!comment || !commentPattern.test(comment.value.trim())) {
                            context.report({ node, messageId: "missingDefaultCase" });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/default-param-last.js */ function _(require, module, exports) {
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce default parameters to be last",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/default-param-last"
            },
            schema: [],
            messages: {
                shouldBeLast: "Default parameters should be last."
            }
        },
        create(context) {
            // eslint-disable-next-line jsdoc/require-description
            /**
             * @param {ASTNode} node function node
             * @returns {void}
             */
            function handleFunction(node) {
                let hasSeenPlainParam = false;
                for (let i = node.params.length - 1; i >= 0; i -= 1) {
                    const param = node.params[i];
                    if (param.type !== "AssignmentPattern" &&
                        param.type !== "RestElement") {
                        hasSeenPlainParam = true;
                        continue;
                    }
                    if (hasSeenPlainParam && param.type === "AssignmentPattern") {
                        context.report({
                            node: param,
                            messageId: "shouldBeLast"
                        });
                    }
                }
            }
            return {
                FunctionDeclaration: handleFunction,
                FunctionExpression: handleFunction,
                ArrowFunctionExpression: handleFunction
            };
        }
    };
},
/* eslint/lib/rules/dot-location.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent newlines before and after dots",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/dot-location"
            },
            schema: [
                {
                    enum: ["object", "property"]
                }
            ],
            fixable: "code",
            messages: {
                expectedDotAfterObject: "Expected dot to be on same line as object.",
                expectedDotBeforeProperty: "Expected dot to be on same line as property."
            }
        },
        create(context) {
            const config = context.options[0];
            // default to onObject if no preference is passed
            const onObject = config === "object" || !config;
            const sourceCode = context.getSourceCode();
            /**
             * Reports if the dot between object and property is on the correct loccation.
             * @param {ASTNode} node The `MemberExpression` node.
             * @returns {void}
             */
            function checkDotLocation(node) {
                const property = node.property;
                const dot = sourceCode.getTokenBefore(property);
                // `obj` expression can be parenthesized, but those paren tokens are not a part of the `obj` node.
                const tokenBeforeDot = sourceCode.getTokenBefore(dot);
                const textBeforeDot = sourceCode.getText().slice(tokenBeforeDot.range[1], dot.range[0]);
                const textAfterDot = sourceCode.getText().slice(dot.range[1], property.range[0]);
                if (onObject) {
                    if (!astUtils.isTokenOnSameLine(tokenBeforeDot, dot)) {
                        const neededTextAfterToken = astUtils.isDecimalIntegerNumericToken(tokenBeforeDot) ? " " : "";
                        context.report({
                            node,
                            loc: dot.loc,
                            messageId: "expectedDotAfterObject",
                            fix: fixer => fixer.replaceTextRange([tokenBeforeDot.range[1], property.range[0]], `${neededTextAfterToken}.${textBeforeDot}${textAfterDot}`)
                        });
                    }
                }
                else if (!astUtils.isTokenOnSameLine(dot, property)) {
                    context.report({
                        node,
                        loc: dot.loc,
                        messageId: "expectedDotBeforeProperty",
                        fix: fixer => fixer.replaceTextRange([tokenBeforeDot.range[1], property.range[0]], `${textBeforeDot}${textAfterDot}.`)
                    });
                }
            }
            /**
             * Checks the spacing of the dot within a member expression.
             * @param {ASTNode} node The node to check.
             * @returns {void}
             */
            function checkNode(node) {
                if (!node.computed) {
                    checkDotLocation(node);
                }
            }
            return {
                MemberExpression: checkNode
            };
        }
    };
},
/* eslint/lib/rules/dot-notation.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    const keywords = require(159) /* ./utils/keywords */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const validIdentifier = /^[a-zA-Z_$][a-zA-Z0-9_$]*$/u;
    // `null` literal must be handled separately.
    const literalTypesToCheck = new Set(["string", "boolean"]);
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce dot notation whenever possible",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/dot-notation"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allowKeywords: {
                            type: "boolean",
                            default: true
                        },
                        allowPattern: {
                            type: "string",
                            default: ""
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "code",
            messages: {
                useDot: "[{{key}}] is better written in dot notation.",
                useBrackets: ".{{key}} is a syntax error."
            }
        },
        create(context) {
            const options = context.options[0] || {};
            const allowKeywords = options.allowKeywords === void 0 || options.allowKeywords;
            const sourceCode = context.getSourceCode();
            let allowPattern;
            if (options.allowPattern) {
                allowPattern = new RegExp(options.allowPattern, "u");
            }
            /**
             * Check if the property is valid dot notation
             * @param {ASTNode} node The dot notation node
             * @param {string} value Value which is to be checked
             * @returns {void}
             */
            function checkComputedProperty(node, value) {
                if (validIdentifier.test(value) &&
                    (allowKeywords || keywords.indexOf(String(value)) === -1) &&
                    !(allowPattern && allowPattern.test(value))) {
                    const formattedValue = node.property.type === "Literal" ? JSON.stringify(value) : `\`${value}\``;
                    context.report({
                        node: node.property,
                        messageId: "useDot",
                        data: {
                            key: formattedValue
                        },
                        fix(fixer) {
                            const leftBracket = sourceCode.getTokenAfter(node.object, astUtils.isOpeningBracketToken);
                            const rightBracket = sourceCode.getLastToken(node);
                            if (sourceCode.getFirstTokenBetween(leftBracket, rightBracket, { includeComments: true, filter: astUtils.isCommentToken })) {
                                // Don't perform any fixes if there are comments inside the brackets.
                                return null;
                            }
                            const tokenAfterProperty = sourceCode.getTokenAfter(rightBracket);
                            const needsSpaceAfterProperty = tokenAfterProperty &&
                                rightBracket.range[1] === tokenAfterProperty.range[0] &&
                                !astUtils.canTokensBeAdjacent(String(value), tokenAfterProperty);
                            const textBeforeDot = astUtils.isDecimalInteger(node.object) ? " " : "";
                            const textAfterProperty = needsSpaceAfterProperty ? " " : "";
                            return fixer.replaceTextRange([leftBracket.range[0], rightBracket.range[1]], `${textBeforeDot}.${value}${textAfterProperty}`);
                        }
                    });
                }
            }
            return {
                MemberExpression(node) {
                    if (node.computed &&
                        node.property.type === "Literal" &&
                        (literalTypesToCheck.has(typeof node.property.value) || astUtils.isNullLiteral(node.property))) {
                        checkComputedProperty(node, node.property.value);
                    }
                    if (node.computed &&
                        node.property.type === "TemplateLiteral" &&
                        node.property.expressions.length === 0) {
                        checkComputedProperty(node, node.property.quasis[0].value.cooked);
                    }
                    if (!allowKeywords &&
                        !node.computed &&
                        keywords.indexOf(String(node.property.name)) !== -1) {
                        context.report({
                            node: node.property,
                            messageId: "useBrackets",
                            data: {
                                key: node.property.name
                            },
                            fix(fixer) {
                                const dot = sourceCode.getTokenBefore(node.property);
                                const textAfterDot = sourceCode.text.slice(dot.range[1], node.property.range[0]);
                                if (textAfterDot.trim()) {
                                    // Don't perform any fixes if there are comments between the dot and the property name.
                                    return null;
                                }
                                if (node.object.type === "Identifier" && node.object.name === "let") {
                                    /*
                                     * A statement that starts with `let[` is parsed as a destructuring variable declaration, not
                                     * a MemberExpression.
                                     */
                                    return null;
                                }
                                return fixer.replaceTextRange([dot.range[0], node.property.range[1]], `[${textAfterDot}"${node.property.name}"]`);
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/utils/keywords.js */ function _(require, module, exports) {
    module.exports = [
        "abstract",
        "boolean",
        "break",
        "byte",
        "case",
        "catch",
        "char",
        "class",
        "const",
        "continue",
        "debugger",
        "default",
        "delete",
        "do",
        "double",
        "else",
        "enum",
        "export",
        "extends",
        "false",
        "final",
        "finally",
        "float",
        "for",
        "function",
        "goto",
        "if",
        "implements",
        "import",
        "in",
        "instanceof",
        "int",
        "interface",
        "long",
        "native",
        "new",
        "null",
        "package",
        "private",
        "protected",
        "public",
        "return",
        "short",
        "static",
        "super",
        "switch",
        "synchronized",
        "this",
        "throw",
        "throws",
        "transient",
        "true",
        "try",
        "typeof",
        "var",
        "void",
        "volatile",
        "while",
        "with"
    ];
},
/* eslint/lib/rules/eol-last.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow newline at the end of files",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/eol-last"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["always", "never", "unix", "windows"]
                }
            ],
            messages: {
                missing: "Newline required at end of file but not found.",
                unexpected: "Newline not allowed at end of file."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                Program: function checkBadEOF(node) {
                    const sourceCode = context.getSourceCode(), src = sourceCode.getText(), location = {
                        column: lodash.last(sourceCode.lines).length,
                        line: sourceCode.lines.length
                    }, LF = "\n", CRLF = `\r${LF}`, endsWithNewline = lodash.endsWith(src, LF);
                    /*
                     * Empty source is always valid: No content in file so we don't
                     * need to lint for a newline on the last line of content.
                     */
                    if (!src.length) {
                        return;
                    }
                    let mode = context.options[0] || "always", appendCRLF = false;
                    if (mode === "unix") {
                        // `"unix"` should behave exactly as `"always"`
                        mode = "always";
                    }
                    if (mode === "windows") {
                        // `"windows"` should behave exactly as `"always"`, but append CRLF in the fixer for backwards compatibility
                        mode = "always";
                        appendCRLF = true;
                    }
                    if (mode === "always" && !endsWithNewline) {
                        // File is not newline-terminated, but should be
                        context.report({
                            node,
                            loc: location,
                            messageId: "missing",
                            fix(fixer) {
                                return fixer.insertTextAfterRange([0, src.length], appendCRLF ? CRLF : LF);
                            }
                        });
                    }
                    else if (mode === "never" && endsWithNewline) {
                        // File is newline-terminated, but shouldn't be
                        context.report({
                            node,
                            loc: location,
                            messageId: "unexpected",
                            fix(fixer) {
                                const finalEOLs = /(?:\r?\n)+$/u, match = finalEOLs.exec(sourceCode.text), start = match.index, end = sourceCode.text.length;
                                return fixer.replaceTextRange([start, end], "");
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/eqeqeq.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require the use of `===` and `!==`",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/eqeqeq"
            },
            schema: {
                anyOf: [
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["always"]
                            },
                            {
                                type: "object",
                                properties: {
                                    null: {
                                        enum: ["always", "never", "ignore"]
                                    }
                                },
                                additionalProperties: false
                            }
                        ],
                        additionalItems: false
                    },
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["smart", "allow-null"]
                            }
                        ],
                        additionalItems: false
                    }
                ]
            },
            fixable: "code",
            messages: {
                unexpected: "Expected '{{expectedOperator}}' and instead saw '{{actualOperator}}'."
            }
        },
        create(context) {
            const config = context.options[0] || "always";
            const options = context.options[1] || {};
            const sourceCode = context.getSourceCode();
            const nullOption = (config === "always")
                ? options.null || "always"
                : "ignore";
            const enforceRuleForNull = (nullOption === "always");
            const enforceInverseRuleForNull = (nullOption === "never");
            /**
             * Checks if an expression is a typeof expression
             * @param  {ASTNode} node The node to check
             * @returns {boolean} if the node is a typeof expression
             */
            function isTypeOf(node) {
                return node.type === "UnaryExpression" && node.operator === "typeof";
            }
            /**
             * Checks if either operand of a binary expression is a typeof operation
             * @param {ASTNode} node The node to check
             * @returns {boolean} if one of the operands is typeof
             * @private
             */
            function isTypeOfBinary(node) {
                return isTypeOf(node.left) || isTypeOf(node.right);
            }
            /**
             * Checks if operands are literals of the same type (via typeof)
             * @param {ASTNode} node The node to check
             * @returns {boolean} if operands are of same type
             * @private
             */
            function areLiteralsAndSameType(node) {
                return node.left.type === "Literal" && node.right.type === "Literal" &&
                    typeof node.left.value === typeof node.right.value;
            }
            /**
             * Checks if one of the operands is a literal null
             * @param {ASTNode} node The node to check
             * @returns {boolean} if operands are null
             * @private
             */
            function isNullCheck(node) {
                return astUtils.isNullLiteral(node.right) || astUtils.isNullLiteral(node.left);
            }
            /**
             * Reports a message for this rule.
             * @param {ASTNode} node The binary expression node that was checked
             * @param {string} expectedOperator The operator that was expected (either '==', '!=', '===', or '!==')
             * @returns {void}
             * @private
             */
            function report(node, expectedOperator) {
                const operatorToken = sourceCode.getFirstTokenBetween(node.left, node.right, token => token.value === node.operator);
                context.report({
                    node,
                    loc: operatorToken.loc,
                    messageId: "unexpected",
                    data: { expectedOperator, actualOperator: node.operator },
                    fix(fixer) {
                        // If the comparison is a `typeof` comparison or both sides are literals with the same type, then it's safe to fix.
                        if (isTypeOfBinary(node) || areLiteralsAndSameType(node)) {
                            return fixer.replaceText(operatorToken, expectedOperator);
                        }
                        return null;
                    }
                });
            }
            return {
                BinaryExpression(node) {
                    const isNull = isNullCheck(node);
                    if (node.operator !== "==" && node.operator !== "!=") {
                        if (enforceInverseRuleForNull && isNull) {
                            report(node, node.operator.slice(0, -1));
                        }
                        return;
                    }
                    if (config === "smart" && (isTypeOfBinary(node) ||
                        areLiteralsAndSameType(node) || isNull)) {
                        return;
                    }
                    if (!enforceRuleForNull && isNull) {
                        return;
                    }
                    report(node, `${node.operator}=`);
                }
            };
        }
    };
},
/* eslint/lib/rules/for-direction.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "enforce \"for\" loop update clause moving the counter in the right direction.",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/for-direction"
            },
            fixable: null,
            schema: [],
            messages: {
                incorrectDirection: "The update clause in this loop moves the variable in the wrong direction."
            }
        },
        create(context) {
            /**
             * report an error.
             * @param {ASTNode} node the node to report.
             * @returns {void}
             */
            function report(node) {
                context.report({
                    node,
                    messageId: "incorrectDirection"
                });
            }
            /**
             * check the right side of the assignment
             * @param {ASTNode} update UpdateExpression to check
             * @param {int} dir expected direction that could either be turned around or invalidated
             * @returns {int} return dir, the negated dir or zero if it's not clear for identifiers
             */
            function getRightDirection(update, dir) {
                if (update.right.type === "UnaryExpression") {
                    if (update.right.operator === "-") {
                        return -dir;
                    }
                }
                else if (update.right.type === "Identifier") {
                    return 0;
                }
                return dir;
            }
            /**
             * check UpdateExpression add/sub the counter
             * @param {ASTNode} update UpdateExpression to check
             * @param {string} counter variable name to check
             * @returns {int} if add return 1, if sub return -1, if nochange, return 0
             */
            function getUpdateDirection(update, counter) {
                if (update.argument.type === "Identifier" && update.argument.name === counter) {
                    if (update.operator === "++") {
                        return 1;
                    }
                    if (update.operator === "--") {
                        return -1;
                    }
                }
                return 0;
            }
            /**
             * check AssignmentExpression add/sub the counter
             * @param {ASTNode} update AssignmentExpression to check
             * @param {string} counter variable name to check
             * @returns {int} if add return 1, if sub return -1, if nochange, return 0
             */
            function getAssignmentDirection(update, counter) {
                if (update.left.name === counter) {
                    if (update.operator === "+=") {
                        return getRightDirection(update, 1);
                    }
                    if (update.operator === "-=") {
                        return getRightDirection(update, -1);
                    }
                }
                return 0;
            }
            return {
                ForStatement(node) {
                    if (node.test && node.test.type === "BinaryExpression" && node.test.left.type === "Identifier" && node.update) {
                        const counter = node.test.left.name;
                        const operator = node.test.operator;
                        const update = node.update;
                        let wrongDirection;
                        if (operator === "<" || operator === "<=") {
                            wrongDirection = -1;
                        }
                        else if (operator === ">" || operator === ">=") {
                            wrongDirection = 1;
                        }
                        else {
                            return;
                        }
                        if (update.type === "UpdateExpression") {
                            if (getUpdateDirection(update, counter) === wrongDirection) {
                                report(node);
                            }
                        }
                        else if (update.type === "AssignmentExpression" && getAssignmentDirection(update, counter) === wrongDirection) {
                            report(node);
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/func-call-spacing.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow spacing between function identifiers and their invocations",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/func-call-spacing"
            },
            fixable: "whitespace",
            schema: {
                anyOf: [
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["never"]
                            }
                        ],
                        minItems: 0,
                        maxItems: 1
                    },
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["always"]
                            },
                            {
                                type: "object",
                                properties: {
                                    allowNewlines: {
                                        type: "boolean"
                                    }
                                },
                                additionalProperties: false
                            }
                        ],
                        minItems: 0,
                        maxItems: 2
                    }
                ]
            },
            messages: {
                unexpected: "Unexpected newline between function name and paren.",
                missing: "Missing space between function name and paren."
            }
        },
        create(context) {
            const never = context.options[0] !== "always";
            const allowNewlines = !never && context.options[1] && context.options[1].allowNewlines;
            const sourceCode = context.getSourceCode();
            const text = sourceCode.getText();
            /**
             * Check if open space is present in a function name
             * @param {ASTNode} node node to evaluate
             * @param {Token} leftToken The last token of the callee. This may be the closing parenthesis that encloses the callee.
             * @param {Token} rightToken Tha first token of the arguments. this is the opening parenthesis that encloses the arguments.
             * @returns {void}
             * @private
             */
            function checkSpacing(node, leftToken, rightToken) {
                const textBetweenTokens = text.slice(leftToken.range[1], rightToken.range[0]).replace(/\/\*.*?\*\//gu, "");
                const hasWhitespace = /\s/u.test(textBetweenTokens);
                const hasNewline = hasWhitespace && astUtils.LINEBREAK_MATCHER.test(textBetweenTokens);
                /*
                 * never allowNewlines hasWhitespace hasNewline message
                 * F     F             F             F          Missing space between function name and paren.
                 * F     F             F             T          (Invalid `!hasWhitespace && hasNewline`)
                 * F     F             T             T          Unexpected newline between function name and paren.
                 * F     F             T             F          (OK)
                 * F     T             T             F          (OK)
                 * F     T             T             T          (OK)
                 * F     T             F             T          (Invalid `!hasWhitespace && hasNewline`)
                 * F     T             F             F          Missing space between function name and paren.
                 * T     T             F             F          (Invalid `never && allowNewlines`)
                 * T     T             F             T          (Invalid `!hasWhitespace && hasNewline`)
                 * T     T             T             T          (Invalid `never && allowNewlines`)
                 * T     T             T             F          (Invalid `never && allowNewlines`)
                 * T     F             T             F          Unexpected space between function name and paren.
                 * T     F             T             T          Unexpected space between function name and paren.
                 * T     F             F             T          (Invalid `!hasWhitespace && hasNewline`)
                 * T     F             F             F          (OK)
                 *
                 * T                   T                        Unexpected space between function name and paren.
                 * F                   F                        Missing space between function name and paren.
                 * F     F                           T          Unexpected newline between function name and paren.
                 */
                if (never && hasWhitespace) {
                    context.report({
                        node,
                        loc: leftToken.loc.start,
                        messageId: "unexpected",
                        fix(fixer) {
                            /*
                             * Only autofix if there is no newline
                             * https://github.com/eslint/eslint/issues/7787
                             */
                            if (!hasNewline) {
                                return fixer.removeRange([leftToken.range[1], rightToken.range[0]]);
                            }
                            return null;
                        }
                    });
                }
                else if (!never && !hasWhitespace) {
                    context.report({
                        node,
                        loc: leftToken.loc.start,
                        messageId: "missing",
                        fix(fixer) {
                            return fixer.insertTextBefore(rightToken, " ");
                        }
                    });
                }
                else if (!never && !allowNewlines && hasNewline) {
                    context.report({
                        node,
                        loc: leftToken.loc.start,
                        messageId: "unexpected",
                        fix(fixer) {
                            return fixer.replaceTextRange([leftToken.range[1], rightToken.range[0]], " ");
                        }
                    });
                }
            }
            return {
                "CallExpression, NewExpression"(node) {
                    const lastToken = sourceCode.getLastToken(node);
                    const lastCalleeToken = sourceCode.getLastToken(node.callee);
                    const parenToken = sourceCode.getFirstTokenBetween(lastCalleeToken, lastToken, astUtils.isOpeningParenToken);
                    const prevToken = parenToken && sourceCode.getTokenBefore(parenToken);
                    // Parens in NewExpression are optional
                    if (!(parenToken && parenToken.range[1] < node.range[1])) {
                        return;
                    }
                    checkSpacing(node, prevToken, parenToken);
                },
                ImportExpression(node) {
                    const leftToken = sourceCode.getFirstToken(node);
                    const rightToken = sourceCode.getTokenAfter(leftToken);
                    checkSpacing(node, leftToken, rightToken);
                }
            };
        }
    };
},
/* eslint/lib/rules/func-name-matching.js */ function _(require, module, exports) {
    //--------------------------------------------------------------------------
    // Requirements
    //--------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    const esutils = require(127) /* esutils */;
    //--------------------------------------------------------------------------
    // Helpers
    //--------------------------------------------------------------------------
    /**
     * Determines if a pattern is `module.exports` or `module["exports"]`
     * @param {ASTNode} pattern The left side of the AssignmentExpression
     * @returns {boolean} True if the pattern is `module.exports` or `module["exports"]`
     */
    function isModuleExports(pattern) {
        if (pattern.type === "MemberExpression" && pattern.object.type === "Identifier" && pattern.object.name === "module") {
            // module.exports
            if (pattern.property.type === "Identifier" && pattern.property.name === "exports") {
                return true;
            }
            // module["exports"]
            if (pattern.property.type === "Literal" && pattern.property.value === "exports") {
                return true;
            }
        }
        return false;
    }
    /**
     * Determines if a string name is a valid identifier
     * @param {string} name The string to be checked
     * @param {int} ecmaVersion The ECMAScript version if specified in the parserOptions config
     * @returns {boolean} True if the string is a valid identifier
     */
    function isIdentifier(name, ecmaVersion) {
        if (ecmaVersion >= 6) {
            return esutils.keyword.isIdentifierES6(name);
        }
        return esutils.keyword.isIdentifierES5(name);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const alwaysOrNever = { enum: ["always", "never"] };
    const optionsObject = {
        type: "object",
        properties: {
            considerPropertyDescriptor: {
                type: "boolean"
            },
            includeCommonJSModuleExports: {
                type: "boolean"
            }
        },
        additionalProperties: false
    };
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require function names to match the name of the variable or property to which they are assigned",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/func-name-matching"
            },
            schema: {
                anyOf: [{
                        type: "array",
                        additionalItems: false,
                        items: [alwaysOrNever, optionsObject]
                    }, {
                        type: "array",
                        additionalItems: false,
                        items: [optionsObject]
                    }]
            },
            messages: {
                matchProperty: "Function name `{{funcName}}` should match property name `{{name}}`.",
                matchVariable: "Function name `{{funcName}}` should match variable name `{{name}}`.",
                notMatchProperty: "Function name `{{funcName}}` should not match property name `{{name}}`.",
                notMatchVariable: "Function name `{{funcName}}` should not match variable name `{{name}}`."
            }
        },
        create(context) {
            const options = (typeof context.options[0] === "object" ? context.options[0] : context.options[1]) || {};
            const nameMatches = typeof context.options[0] === "string" ? context.options[0] : "always";
            const considerPropertyDescriptor = options.considerPropertyDescriptor;
            const includeModuleExports = options.includeCommonJSModuleExports;
            const ecmaVersion = context.parserOptions && context.parserOptions.ecmaVersion ? context.parserOptions.ecmaVersion : 5;
            /**
             * Check whether node is a certain CallExpression.
             * @param {string} objName object name
             * @param {string} funcName function name
             * @param {ASTNode} node The node to check
             * @returns {boolean} `true` if node matches CallExpression
             */
            function isPropertyCall(objName, funcName, node) {
                if (!node) {
                    return false;
                }
                return node.type === "CallExpression" &&
                    node.callee.type === "MemberExpression" &&
                    node.callee.object.name === objName &&
                    node.callee.property.name === funcName;
            }
            /**
             * Compares identifiers based on the nameMatches option
             * @param {string} x the first identifier
             * @param {string} y the second identifier
             * @returns {boolean} whether the two identifiers should warn.
             */
            function shouldWarn(x, y) {
                return (nameMatches === "always" && x !== y) || (nameMatches === "never" && x === y);
            }
            /**
             * Reports
             * @param {ASTNode} node The node to report
             * @param {string} name The variable or property name
             * @param {string} funcName The function name
             * @param {boolean} isProp True if the reported node is a property assignment
             * @returns {void}
             */
            function report(node, name, funcName, isProp) {
                let messageId;
                if (nameMatches === "always" && isProp) {
                    messageId = "matchProperty";
                }
                else if (nameMatches === "always") {
                    messageId = "matchVariable";
                }
                else if (isProp) {
                    messageId = "notMatchProperty";
                }
                else {
                    messageId = "notMatchVariable";
                }
                context.report({
                    node,
                    messageId,
                    data: {
                        name,
                        funcName
                    }
                });
            }
            /**
             * Determines whether a given node is a string literal
             * @param {ASTNode} node The node to check
             * @returns {boolean} `true` if the node is a string literal
             */
            function isStringLiteral(node) {
                return node.type === "Literal" && typeof node.value === "string";
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                VariableDeclarator(node) {
                    if (!node.init || node.init.type !== "FunctionExpression" || node.id.type !== "Identifier") {
                        return;
                    }
                    if (node.init.id && shouldWarn(node.id.name, node.init.id.name)) {
                        report(node, node.id.name, node.init.id.name, false);
                    }
                },
                AssignmentExpression(node) {
                    if (node.right.type !== "FunctionExpression" ||
                        (node.left.computed && node.left.property.type !== "Literal") ||
                        (!includeModuleExports && isModuleExports(node.left)) ||
                        (node.left.type !== "Identifier" && node.left.type !== "MemberExpression")) {
                        return;
                    }
                    const isProp = node.left.type === "MemberExpression";
                    const name = isProp ? astUtils.getStaticPropertyName(node.left) : node.left.name;
                    if (node.right.id && isIdentifier(name) && shouldWarn(name, node.right.id.name)) {
                        report(node, name, node.right.id.name, isProp);
                    }
                },
                Property(node) {
                    if (node.value.type !== "FunctionExpression" || !node.value.id || node.computed && !isStringLiteral(node.key)) {
                        return;
                    }
                    if (node.key.type === "Identifier") {
                        const functionName = node.value.id.name;
                        let propertyName = node.key.name;
                        if (considerPropertyDescriptor && propertyName === "value") {
                            if (isPropertyCall("Object", "defineProperty", node.parent.parent) || isPropertyCall("Reflect", "defineProperty", node.parent.parent)) {
                                const property = node.parent.parent.arguments[1];
                                if (isStringLiteral(property) && shouldWarn(property.value, functionName)) {
                                    report(node, property.value, functionName, true);
                                }
                            }
                            else if (isPropertyCall("Object", "defineProperties", node.parent.parent.parent.parent)) {
                                propertyName = node.parent.parent.key.name;
                                if (!node.parent.parent.computed && shouldWarn(propertyName, functionName)) {
                                    report(node, propertyName, functionName, true);
                                }
                            }
                            else if (isPropertyCall("Object", "create", node.parent.parent.parent.parent)) {
                                propertyName = node.parent.parent.key.name;
                                if (!node.parent.parent.computed && shouldWarn(propertyName, functionName)) {
                                    report(node, propertyName, functionName, true);
                                }
                            }
                            else if (shouldWarn(propertyName, functionName)) {
                                report(node, propertyName, functionName, true);
                            }
                        }
                        else if (shouldWarn(propertyName, functionName)) {
                            report(node, propertyName, functionName, true);
                        }
                        return;
                    }
                    if (isStringLiteral(node.key) &&
                        isIdentifier(node.key.value, ecmaVersion) &&
                        shouldWarn(node.key.value, node.value.id.name)) {
                        report(node, node.key.value, node.value.id.name, true);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/func-names.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    /**
     * Checks whether or not a given variable is a function name.
     * @param {eslint-scope.Variable} variable A variable to check.
     * @returns {boolean} `true` if the variable is a function name.
     */
    function isFunctionName(variable) {
        return variable && variable.defs[0].type === "FunctionName";
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require or disallow named `function` expressions",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/func-names"
            },
            schema: {
                definitions: {
                    value: {
                        enum: [
                            "always",
                            "as-needed",
                            "never"
                        ]
                    }
                },
                items: [
                    {
                        $ref: "#/definitions/value"
                    },
                    {
                        type: "object",
                        properties: {
                            generators: {
                                $ref: "#/definitions/value"
                            }
                        },
                        additionalProperties: false
                    }
                ]
            },
            messages: {
                unnamed: "Unexpected unnamed {{name}}.",
                named: "Unexpected named {{name}}."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Returns the config option for the given node.
             * @param {ASTNode} node A node to get the config for.
             * @returns {string} The config option.
             */
            function getConfigForNode(node) {
                if (node.generator &&
                    context.options.length > 1 &&
                    context.options[1].generators) {
                    return context.options[1].generators;
                }
                return context.options[0] || "always";
            }
            /**
             * Determines whether the current FunctionExpression node is a get, set, or
             * shorthand method in an object literal or a class.
             * @param {ASTNode} node A node to check.
             * @returns {boolean} True if the node is a get, set, or shorthand method.
             */
            function isObjectOrClassMethod(node) {
                const parent = node.parent;
                return (parent.type === "MethodDefinition" || (parent.type === "Property" && (parent.method ||
                    parent.kind === "get" ||
                    parent.kind === "set")));
            }
            /**
             * Determines whether the current FunctionExpression node has a name that would be
             * inferred from context in a conforming ES6 environment.
             * @param {ASTNode} node A node to check.
             * @returns {boolean} True if the node would have a name assigned automatically.
             */
            function hasInferredName(node) {
                const parent = node.parent;
                return isObjectOrClassMethod(node) ||
                    (parent.type === "VariableDeclarator" && parent.id.type === "Identifier" && parent.init === node) ||
                    (parent.type === "Property" && parent.value === node) ||
                    (parent.type === "AssignmentExpression" && parent.left.type === "Identifier" && parent.right === node) ||
                    (parent.type === "ExportDefaultDeclaration" && parent.declaration === node) ||
                    (parent.type === "AssignmentPattern" && parent.right === node);
            }
            /**
             * Reports that an unnamed function should be named
             * @param {ASTNode} node The node to report in the event of an error.
             * @returns {void}
             */
            function reportUnexpectedUnnamedFunction(node) {
                context.report({
                    node,
                    messageId: "unnamed",
                    loc: astUtils.getFunctionHeadLoc(node, sourceCode),
                    data: { name: astUtils.getFunctionNameWithKind(node) }
                });
            }
            /**
             * Reports that a named function should be unnamed
             * @param {ASTNode} node The node to report in the event of an error.
             * @returns {void}
             */
            function reportUnexpectedNamedFunction(node) {
                context.report({
                    node,
                    messageId: "named",
                    loc: astUtils.getFunctionHeadLoc(node, sourceCode),
                    data: { name: astUtils.getFunctionNameWithKind(node) }
                });
            }
            return {
                "FunctionExpression:exit"(node) {
                    // Skip recursive functions.
                    const nameVar = context.getDeclaredVariables(node)[0];
                    if (isFunctionName(nameVar) && nameVar.references.length > 0) {
                        return;
                    }
                    const hasName = Boolean(node.id && node.id.name);
                    const config = getConfigForNode(node);
                    if (config === "never") {
                        if (hasName) {
                            reportUnexpectedNamedFunction(node);
                        }
                    }
                    else if (config === "as-needed") {
                        if (!hasName && !hasInferredName(node)) {
                            reportUnexpectedUnnamedFunction(node);
                        }
                    }
                    else {
                        if (!hasName && !isObjectOrClassMethod(node)) {
                            reportUnexpectedUnnamedFunction(node);
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/func-style.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce the consistent use of either `function` declarations or expressions",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/func-style"
            },
            schema: [
                {
                    enum: ["declaration", "expression"]
                },
                {
                    type: "object",
                    properties: {
                        allowArrowFunctions: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                expression: "Expected a function expression.",
                declaration: "Expected a function declaration."
            }
        },
        create(context) {
            const style = context.options[0], allowArrowFunctions = context.options[1] && context.options[1].allowArrowFunctions, enforceDeclarations = (style === "declaration"), stack = [];
            const nodesToCheck = {
                FunctionDeclaration(node) {
                    stack.push(false);
                    if (!enforceDeclarations && node.parent.type !== "ExportDefaultDeclaration") {
                        context.report({ node, messageId: "expression" });
                    }
                },
                "FunctionDeclaration:exit"() {
                    stack.pop();
                },
                FunctionExpression(node) {
                    stack.push(false);
                    if (enforceDeclarations && node.parent.type === "VariableDeclarator") {
                        context.report({ node: node.parent, messageId: "declaration" });
                    }
                },
                "FunctionExpression:exit"() {
                    stack.pop();
                },
                ThisExpression() {
                    if (stack.length > 0) {
                        stack[stack.length - 1] = true;
                    }
                }
            };
            if (!allowArrowFunctions) {
                nodesToCheck.ArrowFunctionExpression = function () {
                    stack.push(false);
                };
                nodesToCheck["ArrowFunctionExpression:exit"] = function (node) {
                    const hasThisExpr = stack.pop();
                    if (enforceDeclarations && !hasThisExpr && node.parent.type === "VariableDeclarator") {
                        context.report({ node: node.parent, messageId: "declaration" });
                    }
                };
            }
            return nodesToCheck;
        }
    };
},
/* eslint/lib/rules/function-call-argument-newline.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce line breaks between arguments of a function call",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/function-call-argument-newline"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["always", "never", "consistent"]
                }
            ],
            messages: {
                unexpectedLineBreak: "There should be no line break here.",
                missingLineBreak: "There should be a line break after this argument."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const checkers = {
                unexpected: {
                    messageId: "unexpectedLineBreak",
                    check: (prevToken, currentToken) => prevToken.loc.end.line !== currentToken.loc.start.line,
                    createFix: (token, tokenBefore) => fixer => fixer.replaceTextRange([tokenBefore.range[1], token.range[0]], " ")
                },
                missing: {
                    messageId: "missingLineBreak",
                    check: (prevToken, currentToken) => prevToken.loc.end.line === currentToken.loc.start.line,
                    createFix: (token, tokenBefore) => fixer => fixer.replaceTextRange([tokenBefore.range[1], token.range[0]], "\n")
                }
            };
            /**
             * Check all arguments for line breaks in the CallExpression
             * @param {CallExpression} node node to evaluate
             * @param {{ messageId: string, check: Function }} checker selected checker
             * @returns {void}
             * @private
             */
            function checkArguments(node, checker) {
                for (let i = 1; i < node.arguments.length; i++) {
                    const prevArgToken = sourceCode.getLastToken(node.arguments[i - 1]);
                    const currentArgToken = sourceCode.getFirstToken(node.arguments[i]);
                    if (checker.check(prevArgToken, currentArgToken)) {
                        const tokenBefore = sourceCode.getTokenBefore(currentArgToken, { includeComments: true });
                        const hasLineCommentBefore = tokenBefore.type === "Line";
                        context.report({
                            node,
                            loc: {
                                start: tokenBefore.loc.end,
                                end: currentArgToken.loc.start
                            },
                            messageId: checker.messageId,
                            fix: hasLineCommentBefore ? null : checker.createFix(currentArgToken, tokenBefore)
                        });
                    }
                }
            }
            /**
             * Check if open space is present in a function name
             * @param {CallExpression} node node to evaluate
             * @returns {void}
             * @private
             */
            function check(node) {
                if (node.arguments.length < 2) {
                    return;
                }
                const option = context.options[0] || "always";
                if (option === "never") {
                    checkArguments(node, checkers.unexpected);
                }
                else if (option === "always") {
                    checkArguments(node, checkers.missing);
                }
                else if (option === "consistent") {
                    const firstArgToken = sourceCode.getLastToken(node.arguments[0]);
                    const secondArgToken = sourceCode.getFirstToken(node.arguments[1]);
                    if (firstArgToken.loc.end.line === secondArgToken.loc.start.line) {
                        checkArguments(node, checkers.unexpected);
                    }
                    else {
                        checkArguments(node, checkers.missing);
                    }
                }
            }
            return {
                CallExpression: check,
                NewExpression: check
            };
        }
    };
},
/* eslint/lib/rules/function-paren-newline.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent line breaks inside function parentheses",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/function-paren-newline"
            },
            fixable: "whitespace",
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["always", "never", "consistent", "multiline", "multiline-arguments"]
                        },
                        {
                            type: "object",
                            properties: {
                                minItems: {
                                    type: "integer",
                                    minimum: 0
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                expectedBefore: "Expected newline before ')'.",
                expectedAfter: "Expected newline after '('.",
                expectedBetween: "Expected newline between arguments/params.",
                unexpectedBefore: "Unexpected newline before ')'.",
                unexpectedAfter: "Unexpected newline after '('."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const rawOption = context.options[0] || "multiline";
            const multilineOption = rawOption === "multiline";
            const multilineArgumentsOption = rawOption === "multiline-arguments";
            const consistentOption = rawOption === "consistent";
            let minItems;
            if (typeof rawOption === "object") {
                minItems = rawOption.minItems;
            }
            else if (rawOption === "always") {
                minItems = 0;
            }
            else if (rawOption === "never") {
                minItems = Infinity;
            }
            else {
                minItems = null;
            }
            //----------------------------------------------------------------------
            // Helpers
            //----------------------------------------------------------------------
            /**
             * Determines whether there should be newlines inside function parens
             * @param {ASTNode[]} elements The arguments or parameters in the list
             * @param {boolean} hasLeftNewline `true` if the left paren has a newline in the current code.
             * @returns {boolean} `true` if there should be newlines inside the function parens
             */
            function shouldHaveNewlines(elements, hasLeftNewline) {
                if (multilineArgumentsOption && elements.length === 1) {
                    return hasLeftNewline;
                }
                if (multilineOption || multilineArgumentsOption) {
                    return elements.some((element, index) => index !== elements.length - 1 && element.loc.end.line !== elements[index + 1].loc.start.line);
                }
                if (consistentOption) {
                    return hasLeftNewline;
                }
                return elements.length >= minItems;
            }
            /**
             * Validates parens
             * @param {Object} parens An object with keys `leftParen` for the left paren token, and `rightParen` for the right paren token
             * @param {ASTNode[]} elements The arguments or parameters in the list
             * @returns {void}
             */
            function validateParens(parens, elements) {
                const leftParen = parens.leftParen;
                const rightParen = parens.rightParen;
                const tokenAfterLeftParen = sourceCode.getTokenAfter(leftParen);
                const tokenBeforeRightParen = sourceCode.getTokenBefore(rightParen);
                const hasLeftNewline = !astUtils.isTokenOnSameLine(leftParen, tokenAfterLeftParen);
                const hasRightNewline = !astUtils.isTokenOnSameLine(tokenBeforeRightParen, rightParen);
                const needsNewlines = shouldHaveNewlines(elements, hasLeftNewline);
                if (hasLeftNewline && !needsNewlines) {
                    context.report({
                        node: leftParen,
                        messageId: "unexpectedAfter",
                        fix(fixer) {
                            return sourceCode.getText().slice(leftParen.range[1], tokenAfterLeftParen.range[0]).trim()
                                // If there is a comment between the ( and the first element, don't do a fix.
                                ? null
                                : fixer.removeRange([leftParen.range[1], tokenAfterLeftParen.range[0]]);
                        }
                    });
                }
                else if (!hasLeftNewline && needsNewlines) {
                    context.report({
                        node: leftParen,
                        messageId: "expectedAfter",
                        fix: fixer => fixer.insertTextAfter(leftParen, "\n")
                    });
                }
                if (hasRightNewline && !needsNewlines) {
                    context.report({
                        node: rightParen,
                        messageId: "unexpectedBefore",
                        fix(fixer) {
                            return sourceCode.getText().slice(tokenBeforeRightParen.range[1], rightParen.range[0]).trim()
                                // If there is a comment between the last element and the ), don't do a fix.
                                ? null
                                : fixer.removeRange([tokenBeforeRightParen.range[1], rightParen.range[0]]);
                        }
                    });
                }
                else if (!hasRightNewline && needsNewlines) {
                    context.report({
                        node: rightParen,
                        messageId: "expectedBefore",
                        fix: fixer => fixer.insertTextBefore(rightParen, "\n")
                    });
                }
            }
            /**
             * Validates a list of arguments or parameters
             * @param {Object} parens An object with keys `leftParen` for the left paren token, and `rightParen` for the right paren token
             * @param {ASTNode[]} elements The arguments or parameters in the list
             * @returns {void}
             */
            function validateArguments(parens, elements) {
                const leftParen = parens.leftParen;
                const tokenAfterLeftParen = sourceCode.getTokenAfter(leftParen);
                const hasLeftNewline = !astUtils.isTokenOnSameLine(leftParen, tokenAfterLeftParen);
                const needsNewlines = shouldHaveNewlines(elements, hasLeftNewline);
                for (let i = 0; i <= elements.length - 2; i++) {
                    const currentElement = elements[i];
                    const nextElement = elements[i + 1];
                    const hasNewLine = currentElement.loc.end.line !== nextElement.loc.start.line;
                    if (!hasNewLine && needsNewlines) {
                        context.report({
                            node: currentElement,
                            messageId: "expectedBetween",
                            fix: fixer => fixer.insertTextBefore(nextElement, "\n")
                        });
                    }
                }
            }
            /**
             * Gets the left paren and right paren tokens of a node.
             * @param {ASTNode} node The node with parens
             * @returns {Object} An object with keys `leftParen` for the left paren token, and `rightParen` for the right paren token.
             * Can also return `null` if an expression has no parens (e.g. a NewExpression with no arguments, or an ArrowFunctionExpression
             * with a single parameter)
             */
            function getParenTokens(node) {
                switch (node.type) {
                    case "NewExpression":
                        if (!node.arguments.length && !(astUtils.isOpeningParenToken(sourceCode.getLastToken(node, { skip: 1 })) &&
                            astUtils.isClosingParenToken(sourceCode.getLastToken(node)))) {
                            // If the NewExpression does not have parens (e.g. `new Foo`), return null.
                            return null;
                        }
                    // falls through
                    case "CallExpression":
                        return {
                            leftParen: sourceCode.getTokenAfter(node.callee, astUtils.isOpeningParenToken),
                            rightParen: sourceCode.getLastToken(node)
                        };
                    case "FunctionDeclaration":
                    case "FunctionExpression": {
                        const leftParen = sourceCode.getFirstToken(node, astUtils.isOpeningParenToken);
                        const rightParen = node.params.length
                            ? sourceCode.getTokenAfter(node.params[node.params.length - 1], astUtils.isClosingParenToken)
                            : sourceCode.getTokenAfter(leftParen);
                        return { leftParen, rightParen };
                    }
                    case "ArrowFunctionExpression": {
                        const firstToken = sourceCode.getFirstToken(node);
                        if (!astUtils.isOpeningParenToken(firstToken)) {
                            // If the ArrowFunctionExpression has a single param without parens, return null.
                            return null;
                        }
                        return {
                            leftParen: firstToken,
                            rightParen: sourceCode.getTokenBefore(node.body, astUtils.isClosingParenToken)
                        };
                    }
                    case "ImportExpression": {
                        const leftParen = sourceCode.getFirstToken(node, 1);
                        const rightParen = sourceCode.getLastToken(node);
                        return { leftParen, rightParen };
                    }
                    default:
                        throw new TypeError(`unexpected node with type ${node.type}`);
                }
            }
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                [[
                    "ArrowFunctionExpression",
                    "CallExpression",
                    "FunctionDeclaration",
                    "FunctionExpression",
                    "ImportExpression",
                    "NewExpression"
                ]](node) {
                    const parens = getParenTokens(node);
                    let params;
                    if (node.type === "ImportExpression") {
                        params = [node.source];
                    }
                    else if (astUtils.isFunction(node)) {
                        params = node.params;
                    }
                    else {
                        params = node.arguments;
                    }
                    if (parens) {
                        validateParens(parens, params);
                        if (multilineArgumentsOption) {
                            validateArguments(parens, params);
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/generator-star-spacing.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const OVERRIDE_SCHEMA = {
        oneOf: [
            {
                enum: ["before", "after", "both", "neither"]
            },
            {
                type: "object",
                properties: {
                    before: { type: "boolean" },
                    after: { type: "boolean" }
                },
                additionalProperties: false
            }
        ]
    };
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing around `*` operators in generator functions",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/generator-star-spacing"
            },
            fixable: "whitespace",
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["before", "after", "both", "neither"]
                        },
                        {
                            type: "object",
                            properties: {
                                before: { type: "boolean" },
                                after: { type: "boolean" },
                                named: OVERRIDE_SCHEMA,
                                anonymous: OVERRIDE_SCHEMA,
                                method: OVERRIDE_SCHEMA
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                missingBefore: "Missing space before *.",
                missingAfter: "Missing space after *.",
                unexpectedBefore: "Unexpected space before *.",
                unexpectedAfter: "Unexpected space after *."
            }
        },
        create(context) {
            const optionDefinitions = {
                before: { before: true, after: false },
                after: { before: false, after: true },
                both: { before: true, after: true },
                neither: { before: false, after: false }
            };
            /**
             * Returns resolved option definitions based on an option and defaults
             * @param {any} option The option object or string value
             * @param {Object} defaults The defaults to use if options are not present
             * @returns {Object} the resolved object definition
             */
            function optionToDefinition(option, defaults) {
                if (!option) {
                    return defaults;
                }
                return typeof option === "string"
                    ? optionDefinitions[option]
                    : Object.assign({}, defaults, option);
            }
            const modes = (function (option) {
                const defaults = optionToDefinition(option, optionDefinitions.before);
                return {
                    named: optionToDefinition(option.named, defaults),
                    anonymous: optionToDefinition(option.anonymous, defaults),
                    method: optionToDefinition(option.method, defaults)
                };
            }(context.options[0] || {}));
            const sourceCode = context.getSourceCode();
            /**
             * Checks if the given token is a star token or not.
             * @param {Token} token The token to check.
             * @returns {boolean} `true` if the token is a star token.
             */
            function isStarToken(token) {
                return token.value === "*" && token.type === "Punctuator";
            }
            /**
             * Gets the generator star token of the given function node.
             * @param {ASTNode} node The function node to get.
             * @returns {Token} Found star token.
             */
            function getStarToken(node) {
                return sourceCode.getFirstToken((node.parent.method || node.parent.type === "MethodDefinition") ? node.parent : node, isStarToken);
            }
            /**
             * capitalize a given string.
             * @param {string} str the given string.
             * @returns {string} the capitalized string.
             */
            function capitalize(str) {
                return str[0].toUpperCase() + str.slice(1);
            }
            /**
             * Checks the spacing between two tokens before or after the star token.
             * @param {string} kind Either "named", "anonymous", or "method"
             * @param {string} side Either "before" or "after".
             * @param {Token} leftToken `function` keyword token if side is "before", or
             *     star token if side is "after".
             * @param {Token} rightToken Star token if side is "before", or identifier
             *     token if side is "after".
             * @returns {void}
             */
            function checkSpacing(kind, side, leftToken, rightToken) {
                if (!!(rightToken.range[0] - leftToken.range[1]) !== modes[kind][side]) {
                    const after = leftToken.value === "*";
                    const spaceRequired = modes[kind][side];
                    const node = after ? leftToken : rightToken;
                    const messageId = `${spaceRequired ? "missing" : "unexpected"}${capitalize(side)}`;
                    context.report({
                        node,
                        messageId,
                        fix(fixer) {
                            if (spaceRequired) {
                                if (after) {
                                    return fixer.insertTextAfter(node, " ");
                                }
                                return fixer.insertTextBefore(node, " ");
                            }
                            return fixer.removeRange([leftToken.range[1], rightToken.range[0]]);
                        }
                    });
                }
            }
            /**
             * Enforces the spacing around the star if node is a generator function.
             * @param {ASTNode} node A function expression or declaration node.
             * @returns {void}
             */
            function checkFunction(node) {
                if (!node.generator) {
                    return;
                }
                const starToken = getStarToken(node);
                const prevToken = sourceCode.getTokenBefore(starToken);
                const nextToken = sourceCode.getTokenAfter(starToken);
                let kind = "named";
                if (node.parent.type === "MethodDefinition" || (node.parent.type === "Property" && node.parent.method)) {
                    kind = "method";
                }
                else if (!node.id) {
                    kind = "anonymous";
                }
                // Only check before when preceded by `function`|`static` keyword
                if (!(kind === "method" && starToken === sourceCode.getFirstToken(node.parent))) {
                    checkSpacing(kind, "before", prevToken, starToken);
                }
                checkSpacing(kind, "after", starToken, nextToken);
            }
            return {
                FunctionDeclaration: checkFunction,
                FunctionExpression: checkFunction
            };
        }
    };
},
/* eslint/lib/rules/getter-return.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const TARGET_NODE_TYPE = /^(?:Arrow)?FunctionExpression$/u;
    /**
     * Checks a given code path segment is reachable.
     * @param {CodePathSegment} segment A segment to check.
     * @returns {boolean} `true` if the segment is reachable.
     */
    function isReachable(segment) {
        return segment.reachable;
    }
    /**
     * Gets a readable location.
     *
     * - FunctionExpression -> the function name or `function` keyword.
     * @param {ASTNode} node A function node to get.
     * @returns {ASTNode|Token} The node or the token of a location.
     */
    function getId(node) {
        return node.id || node;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "enforce `return` statements in getters",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/getter-return"
            },
            fixable: null,
            schema: [
                {
                    type: "object",
                    properties: {
                        allowImplicit: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                expected: "Expected to return a value in {{name}}.",
                expectedAlways: "Expected {{name}} to always return a value."
            }
        },
        create(context) {
            const options = context.options[0] || { allowImplicit: false };
            let funcInfo = {
                upper: null,
                codePath: null,
                hasReturn: false,
                shouldCheck: false,
                node: null
            };
            /**
             * Checks whether or not the last code path segment is reachable.
             * Then reports this function if the segment is reachable.
             *
             * If the last code path segment is reachable, there are paths which are not
             * returned or thrown.
             * @param {ASTNode} node A node to check.
             * @returns {void}
             */
            function checkLastSegment(node) {
                if (funcInfo.shouldCheck &&
                    funcInfo.codePath.currentSegments.some(isReachable)) {
                    context.report({
                        node,
                        loc: getId(node).loc.start,
                        messageId: funcInfo.hasReturn ? "expectedAlways" : "expected",
                        data: {
                            name: astUtils.getFunctionNameWithKind(funcInfo.node)
                        }
                    });
                }
            }
            /**
             * Checks whether a node means a getter function.
             * @param {ASTNode} node a node to check.
             * @returns {boolean} if node means a getter, return true; else return false.
             */
            function isGetter(node) {
                const parent = node.parent;
                if (TARGET_NODE_TYPE.test(node.type) && node.body.type === "BlockStatement") {
                    if (parent.kind === "get") {
                        return true;
                    }
                    if (parent.type === "Property" && astUtils.getStaticPropertyName(parent) === "get" && parent.parent.type === "ObjectExpression") {
                        // Object.defineProperty()
                        if (parent.parent.parent.type === "CallExpression" &&
                            astUtils.getStaticPropertyName(parent.parent.parent.callee) === "defineProperty") {
                            return true;
                        }
                        // Object.defineProperties()
                        if (parent.parent.parent.type === "Property" &&
                            parent.parent.parent.parent.type === "ObjectExpression" &&
                            parent.parent.parent.parent.parent.type === "CallExpression" &&
                            astUtils.getStaticPropertyName(parent.parent.parent.parent.parent.callee) === "defineProperties") {
                            return true;
                        }
                    }
                }
                return false;
            }
            return {
                // Stacks this function's information.
                onCodePathStart(codePath, node) {
                    funcInfo = {
                        upper: funcInfo,
                        codePath,
                        hasReturn: false,
                        shouldCheck: isGetter(node),
                        node
                    };
                },
                // Pops this function's information.
                onCodePathEnd() {
                    funcInfo = funcInfo.upper;
                },
                // Checks the return statement is valid.
                ReturnStatement(node) {
                    if (funcInfo.shouldCheck) {
                        funcInfo.hasReturn = true;
                        // if allowImplicit: false, should also check node.argument
                        if (!options.allowImplicit && !node.argument) {
                            context.report({
                                node,
                                messageId: "expected",
                                data: {
                                    name: astUtils.getFunctionNameWithKind(funcInfo.node)
                                }
                            });
                        }
                    }
                },
                // Reports a given function if the last path is reachable.
                "FunctionExpression:exit": checkLastSegment,
                "ArrowFunctionExpression:exit": checkLastSegment
            };
        }
    };
},
/* eslint/lib/rules/global-require.js */ function _(require, module, exports) {
    const ACCEPTABLE_PARENTS = [
        "AssignmentExpression",
        "VariableDeclarator",
        "MemberExpression",
        "ExpressionStatement",
        "CallExpression",
        "ConditionalExpression",
        "Program",
        "VariableDeclaration"
    ];
    /**
     * Finds the eslint-scope reference in the given scope.
     * @param {Object} scope The scope to search.
     * @param {ASTNode} node The identifier node.
     * @returns {Reference|null} Returns the found reference or null if none were found.
     */
    function findReference(scope, node) {
        const references = scope.references.filter(reference => reference.identifier.range[0] === node.range[0] &&
            reference.identifier.range[1] === node.range[1]);
        /* istanbul ignore else: correctly returns null */
        if (references.length === 1) {
            return references[0];
        }
        return null;
    }
    /**
     * Checks if the given identifier node is shadowed in the given scope.
     * @param {Object} scope The current scope.
     * @param {ASTNode} node The identifier node to check.
     * @returns {boolean} Whether or not the name is shadowed.
     */
    function isShadowed(scope, node) {
        const reference = findReference(scope, node);
        return reference && reference.resolved && reference.resolved.defs.length > 0;
    }
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require `require()` calls to be placed at top-level module scope",
                category: "Node.js and CommonJS",
                recommended: false,
                url: "https://eslint.org/docs/rules/global-require"
            },
            schema: [],
            messages: {
                unexpected: "Unexpected require()."
            }
        },
        create(context) {
            return {
                CallExpression(node) {
                    const currentScope = context.getScope();
                    if (node.callee.name === "require" && !isShadowed(currentScope, node.callee)) {
                        const isGoodRequire = context.getAncestors().every(parent => ACCEPTABLE_PARENTS.indexOf(parent.type) > -1);
                        if (!isGoodRequire) {
                            context.report({ node, messageId: "unexpected" });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/grouped-accessor-pairs.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Typedefs
    //------------------------------------------------------------------------------
    /**
     * Property name if it can be computed statically, otherwise the list of the tokens of the key node.
     * @typedef {string|Token[]} Key
     */
    /**
     * Accessor nodes with the same key.
     * @typedef {Object} AccessorData
     * @property {Key} key Accessor's key
     * @property {ASTNode[]} getters List of getter nodes.
     * @property {ASTNode[]} setters List of setter nodes.
     */
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not the given lists represent the equal tokens in the same order.
     * Tokens are compared by their properties, not by instance.
     * @param {Token[]} left First list of tokens.
     * @param {Token[]} right Second list of tokens.
     * @returns {boolean} `true` if the lists have same tokens.
     */
    function areEqualTokenLists(left, right) {
        if (left.length !== right.length) {
            return false;
        }
        for (let i = 0; i < left.length; i++) {
            const leftToken = left[i], rightToken = right[i];
            if (leftToken.type !== rightToken.type || leftToken.value !== rightToken.value) {
                return false;
            }
        }
        return true;
    }
    /**
     * Checks whether or not the given keys are equal.
     * @param {Key} left First key.
     * @param {Key} right Second key.
     * @returns {boolean} `true` if the keys are equal.
     */
    function areEqualKeys(left, right) {
        if (typeof left === "string" && typeof right === "string") {
            // Statically computed names.
            return left === right;
        }
        if (Array.isArray(left) && Array.isArray(right)) {
            // Token lists.
            return areEqualTokenLists(left, right);
        }
        return false;
    }
    /**
     * Checks whether or not a given node is of an accessor kind ('get' or 'set').
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node is of an accessor kind.
     */
    function isAccessorKind(node) {
        return node.kind === "get" || node.kind === "set";
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require grouped accessor pairs in object literals and classes",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/grouped-accessor-pairs"
            },
            schema: [
                {
                    enum: ["anyOrder", "getBeforeSet", "setBeforeGet"]
                }
            ],
            messages: {
                notGrouped: "Accessor pair {{ formerName }} and {{ latterName }} should be grouped.",
                invalidOrder: "Expected {{ latterName }} to be before {{ formerName }}."
            }
        },
        create(context) {
            const order = context.options[0] || "anyOrder";
            const sourceCode = context.getSourceCode();
            /**
             * Reports the given accessor pair.
             * @param {string} messageId messageId to report.
             * @param {ASTNode} formerNode getter/setter node that is defined before `latterNode`.
             * @param {ASTNode} latterNode getter/setter node that is defined after `formerNode`.
             * @returns {void}
             * @private
             */
            function report(messageId, formerNode, latterNode) {
                context.report({
                    node: latterNode,
                    messageId,
                    loc: astUtils.getFunctionHeadLoc(latterNode.value, sourceCode),
                    data: {
                        formerName: astUtils.getFunctionNameWithKind(formerNode.value),
                        latterName: astUtils.getFunctionNameWithKind(latterNode.value)
                    }
                });
            }
            /**
             * Creates a new `AccessorData` object for the given getter or setter node.
             * @param {ASTNode} node A getter or setter node.
             * @returns {AccessorData} New `AccessorData` object that contains the given node.
             * @private
             */
            function createAccessorData(node) {
                const name = astUtils.getStaticPropertyName(node);
                const key = (name !== null) ? name : sourceCode.getTokens(node.key);
                return {
                    key,
                    getters: node.kind === "get" ? [node] : [],
                    setters: node.kind === "set" ? [node] : []
                };
            }
            /**
             * Merges the given `AccessorData` object into the given accessors list.
             * @param {AccessorData[]} accessors The list to merge into.
             * @param {AccessorData} accessorData The object to merge.
             * @returns {AccessorData[]} The same instance with the merged object.
             * @private
             */
            function mergeAccessorData(accessors, accessorData) {
                const equalKeyElement = accessors.find(a => areEqualKeys(a.key, accessorData.key));
                if (equalKeyElement) {
                    equalKeyElement.getters.push(...accessorData.getters);
                    equalKeyElement.setters.push(...accessorData.setters);
                }
                else {
                    accessors.push(accessorData);
                }
                return accessors;
            }
            /**
             * Checks accessor pairs in the given list of nodes.
             * @param {ASTNode[]} nodes The list to check.
             * @param {Function} shouldCheck – Predicate that returns `true` if the node should be checked.
             * @returns {void}
             * @private
             */
            function checkList(nodes, shouldCheck) {
                const accessors = nodes
                    .filter(shouldCheck)
                    .filter(isAccessorKind)
                    .map(createAccessorData)
                    .reduce(mergeAccessorData, []);
                for (const { getters, setters } of accessors) {
                    // Don't report accessor properties that have duplicate getters or setters.
                    if (getters.length === 1 && setters.length === 1) {
                        const [getter] = getters, [setter] = setters, getterIndex = nodes.indexOf(getter), setterIndex = nodes.indexOf(setter), formerNode = getterIndex < setterIndex ? getter : setter, latterNode = getterIndex < setterIndex ? setter : getter;
                        if (Math.abs(getterIndex - setterIndex) > 1) {
                            report("notGrouped", formerNode, latterNode);
                        }
                        else if ((order === "getBeforeSet" && getterIndex > setterIndex) ||
                            (order === "setBeforeGet" && getterIndex < setterIndex)) {
                            report("invalidOrder", formerNode, latterNode);
                        }
                    }
                }
            }
            return {
                ObjectExpression(node) {
                    checkList(node.properties, n => n.type === "Property");
                },
                ClassBody(node) {
                    checkList(node.body, n => n.type === "MethodDefinition" && !n.static);
                    checkList(node.body, n => n.type === "MethodDefinition" && n.static);
                }
            };
        }
    };
},
/* eslint/lib/rules/guard-for-in.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require `for-in` loops to include an `if` statement",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/guard-for-in"
            },
            schema: [],
            messages: {
                wrap: "The body of a for-in should be wrapped in an if statement to filter unwanted properties from the prototype."
            }
        },
        create(context) {
            return {
                ForInStatement(node) {
                    const body = node.body;
                    // empty statement
                    if (body.type === "EmptyStatement") {
                        return;
                    }
                    // if statement
                    if (body.type === "IfStatement") {
                        return;
                    }
                    // empty block
                    if (body.type === "BlockStatement" && body.body.length === 0) {
                        return;
                    }
                    // block with just if statement
                    if (body.type === "BlockStatement" && body.body.length === 1 && body.body[0].type === "IfStatement") {
                        return;
                    }
                    // block that starts with if statement
                    if (body.type === "BlockStatement" && body.body.length >= 1 && body.body[0].type === "IfStatement") {
                        const i = body.body[0];
                        // ... whose consequent is a continue
                        if (i.consequent.type === "ContinueStatement") {
                            return;
                        }
                        // ... whose consequent is a block that contains only a continue
                        if (i.consequent.type === "BlockStatement" && i.consequent.body.length === 1 && i.consequent.body[0].type === "ContinueStatement") {
                            return;
                        }
                    }
                    context.report({ node, messageId: "wrap" });
                }
            };
        }
    };
},
/* eslint/lib/rules/handle-callback-err.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require error handling in callbacks",
                category: "Node.js and CommonJS",
                recommended: false,
                url: "https://eslint.org/docs/rules/handle-callback-err"
            },
            schema: [
                {
                    type: "string"
                }
            ],
            messages: {
                expected: "Expected error to be handled."
            }
        },
        create(context) {
            const errorArgument = context.options[0] || "err";
            /**
             * Checks if the given argument should be interpreted as a regexp pattern.
             * @param {string} stringToCheck The string which should be checked.
             * @returns {boolean} Whether or not the string should be interpreted as a pattern.
             */
            function isPattern(stringToCheck) {
                const firstChar = stringToCheck[0];
                return firstChar === "^";
            }
            /**
             * Checks if the given name matches the configured error argument.
             * @param {string} name The name which should be compared.
             * @returns {boolean} Whether or not the given name matches the configured error variable name.
             */
            function matchesConfiguredErrorName(name) {
                if (isPattern(errorArgument)) {
                    const regexp = new RegExp(errorArgument, "u");
                    return regexp.test(name);
                }
                return name === errorArgument;
            }
            /**
             * Get the parameters of a given function scope.
             * @param {Object} scope The function scope.
             * @returns {Array} All parameters of the given scope.
             */
            function getParameters(scope) {
                return scope.variables.filter(variable => variable.defs[0] && variable.defs[0].type === "Parameter");
            }
            /**
             * Check to see if we're handling the error object properly.
             * @param {ASTNode} node The AST node to check.
             * @returns {void}
             */
            function checkForError(node) {
                const scope = context.getScope(), parameters = getParameters(scope), firstParameter = parameters[0];
                if (firstParameter && matchesConfiguredErrorName(firstParameter.name)) {
                    if (firstParameter.references.length === 0) {
                        context.report({ node, messageId: "expected" });
                    }
                }
            }
            return {
                FunctionDeclaration: checkForError,
                FunctionExpression: checkForError,
                ArrowFunctionExpression: checkForError
            };
        }
    };
},
/* eslint/lib/rules/id-blacklist.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow specified identifiers",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/id-blacklist"
            },
            schema: {
                type: "array",
                items: {
                    type: "string"
                },
                uniqueItems: true
            },
            messages: {
                blacklisted: "Identifier '{{name}}' is blacklisted."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            const blacklist = context.options;
            /**
             * Checks if a string matches the provided pattern
             * @param {string} name The string to check.
             * @returns {boolean} if the string is a match
             * @private
             */
            function isInvalid(name) {
                return blacklist.indexOf(name) !== -1;
            }
            /**
             * Verifies if we should report an error or not based on the effective
             * parent node and the identifier name.
             * @param {ASTNode} effectiveParent The effective parent node of the node to be reported
             * @param {string} name The identifier name of the identifier node
             * @returns {boolean} whether an error should be reported or not
             */
            function shouldReport(effectiveParent, name) {
                return effectiveParent.type !== "CallExpression" &&
                    effectiveParent.type !== "NewExpression" &&
                    isInvalid(name);
            }
            /**
             * Reports an AST node as a rule violation.
             * @param {ASTNode} node The node to report.
             * @returns {void}
             * @private
             */
            function report(node) {
                context.report({
                    node,
                    messageId: "blacklisted",
                    data: {
                        name: node.name
                    }
                });
            }
            return {
                Identifier(node) {
                    const name = node.name, effectiveParent = (node.parent.type === "MemberExpression") ? node.parent.parent : node.parent;
                    // MemberExpressions get special rules
                    if (node.parent.type === "MemberExpression") {
                        // Always check object names
                        if (node.parent.object.type === "Identifier" &&
                            node.parent.object.name === node.name) {
                            if (isInvalid(name)) {
                                report(node);
                            }
                            // Report AssignmentExpressions only if they are the left side of the assignment
                        }
                        else if (effectiveParent.type === "AssignmentExpression" &&
                            (effectiveParent.right.type !== "MemberExpression" ||
                                effectiveParent.left.type === "MemberExpression" &&
                                    effectiveParent.left.property.name === node.name)) {
                            if (isInvalid(name)) {
                                report(node);
                            }
                        }
                        // Properties have their own rules
                    }
                    else if (node.parent.type === "Property") {
                        if (shouldReport(effectiveParent, name)) {
                            report(node);
                        }
                        // Report anything that is a match and not a CallExpression
                    }
                    else if (shouldReport(effectiveParent, name)) {
                        report(node);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/id-length.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce minimum and maximum identifier lengths",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/id-length"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        min: {
                            type: "integer",
                            default: 2
                        },
                        max: {
                            type: "integer"
                        },
                        exceptions: {
                            type: "array",
                            uniqueItems: true,
                            items: {
                                type: "string"
                            }
                        },
                        properties: {
                            enum: ["always", "never"]
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                tooShort: "Identifier name '{{name}}' is too short (< {{min}}).",
                tooLong: "Identifier name '{{name}}' is too long (> {{max}})."
            }
        },
        create(context) {
            const options = context.options[0] || {};
            const minLength = typeof options.min !== "undefined" ? options.min : 2;
            const maxLength = typeof options.max !== "undefined" ? options.max : Infinity;
            const properties = options.properties !== "never";
            const exceptions = (options.exceptions ? options.exceptions : [])
                .reduce((obj, item) => {
                obj[item] = true;
                return obj;
            }, {});
            const SUPPORTED_EXPRESSIONS = {
                MemberExpression: properties && function (parent) {
                    return !parent.computed && (
                    // regular property assignment
                    (parent.parent.left === parent && parent.parent.type === "AssignmentExpression" ||
                        // or the last identifier in an ObjectPattern destructuring
                        parent.parent.type === "Property" && parent.parent.value === parent &&
                            parent.parent.parent.type === "ObjectPattern" && parent.parent.parent.parent.left === parent.parent.parent));
                },
                AssignmentPattern(parent, node) {
                    return parent.left === node;
                },
                VariableDeclarator(parent, node) {
                    return parent.id === node;
                },
                Property: properties && function (parent, node) {
                    return parent.key === node;
                },
                ImportDefaultSpecifier: true,
                RestElement: true,
                FunctionExpression: true,
                ArrowFunctionExpression: true,
                ClassDeclaration: true,
                FunctionDeclaration: true,
                MethodDefinition: true,
                CatchClause: true
            };
            return {
                Identifier(node) {
                    const name = node.name;
                    const parent = node.parent;
                    const isShort = name.length < minLength;
                    const isLong = name.length > maxLength;
                    if (!(isShort || isLong) || exceptions[name]) {
                        return; // Nothing to report
                    }
                    const isValidExpression = SUPPORTED_EXPRESSIONS[parent.type];
                    if (isValidExpression && (isValidExpression === true || isValidExpression(parent, node))) {
                        context.report({
                            node,
                            messageId: isShort ? "tooShort" : "tooLong",
                            data: { name, min: minLength, max: maxLength }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/id-match.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require identifiers to match a specified regular expression",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/id-match"
            },
            schema: [
                {
                    type: "string"
                },
                {
                    type: "object",
                    properties: {
                        properties: {
                            type: "boolean",
                            default: false
                        },
                        onlyDeclarations: {
                            type: "boolean",
                            default: false
                        },
                        ignoreDestructuring: {
                            type: "boolean",
                            default: false
                        }
                    }
                }
            ],
            messages: {
                notMatch: "Identifier '{{name}}' does not match the pattern '{{pattern}}'."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Options
            //--------------------------------------------------------------------------
            const pattern = context.options[0] || "^.+$", regexp = new RegExp(pattern, "u");
            const options = context.options[1] || {}, properties = !!options.properties, onlyDeclarations = !!options.onlyDeclarations, ignoreDestructuring = !!options.ignoreDestructuring;
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            // contains reported nodes to avoid reporting twice on destructuring with shorthand notation
            const reported = new Map();
            const ALLOWED_PARENT_TYPES = new Set(["CallExpression", "NewExpression"]);
            const DECLARATION_TYPES = new Set(["FunctionDeclaration", "VariableDeclarator"]);
            const IMPORT_TYPES = new Set(["ImportSpecifier", "ImportNamespaceSpecifier", "ImportDefaultSpecifier"]);
            /**
             * Checks if a string matches the provided pattern
             * @param {string} name The string to check.
             * @returns {boolean} if the string is a match
             * @private
             */
            function isInvalid(name) {
                return !regexp.test(name);
            }
            /**
             * Checks if a parent of a node is an ObjectPattern.
             * @param {ASTNode} node The node to check.
             * @returns {boolean} if the node is inside an ObjectPattern
             * @private
             */
            function isInsideObjectPattern(node) {
                let { parent } = node;
                while (parent) {
                    if (parent.type === "ObjectPattern") {
                        return true;
                    }
                    parent = parent.parent;
                }
                return false;
            }
            /**
             * Verifies if we should report an error or not based on the effective
             * parent node and the identifier name.
             * @param {ASTNode} effectiveParent The effective parent node of the node to be reported
             * @param {string} name The identifier name of the identifier node
             * @returns {boolean} whether an error should be reported or not
             */
            function shouldReport(effectiveParent, name) {
                return (!onlyDeclarations || DECLARATION_TYPES.has(effectiveParent.type)) &&
                    !ALLOWED_PARENT_TYPES.has(effectiveParent.type) && isInvalid(name);
            }
            /**
             * Reports an AST node as a rule violation.
             * @param {ASTNode} node The node to report.
             * @returns {void}
             * @private
             */
            function report(node) {
                if (!reported.has(node)) {
                    context.report({
                        node,
                        messageId: "notMatch",
                        data: {
                            name: node.name,
                            pattern
                        }
                    });
                    reported.set(node, true);
                }
            }
            return {
                Identifier(node) {
                    const name = node.name, parent = node.parent, effectiveParent = (parent.type === "MemberExpression") ? parent.parent : parent;
                    if (parent.type === "MemberExpression") {
                        if (!properties) {
                            return;
                        }
                        // Always check object names
                        if (parent.object.type === "Identifier" &&
                            parent.object.name === name) {
                            if (isInvalid(name)) {
                                report(node);
                            }
                            // Report AssignmentExpressions left side's assigned variable id
                        }
                        else if (effectiveParent.type === "AssignmentExpression" &&
                            effectiveParent.left.type === "MemberExpression" &&
                            effectiveParent.left.property.name === node.name) {
                            if (isInvalid(name)) {
                                report(node);
                            }
                            // Report AssignmentExpressions only if they are the left side of the assignment
                        }
                        else if (effectiveParent.type === "AssignmentExpression" && effectiveParent.right.type !== "MemberExpression") {
                            if (isInvalid(name)) {
                                report(node);
                            }
                        }
                        /*
                         * Properties have their own rules, and
                         * AssignmentPattern nodes can be treated like Properties:
                         * e.g.: const { no_camelcased = false } = bar;
                         */
                    }
                    else if (parent.type === "Property" || parent.type === "AssignmentPattern") {
                        if (parent.parent && parent.parent.type === "ObjectPattern") {
                            if (parent.shorthand && parent.value.left && isInvalid(name)) {
                                report(node);
                            }
                            const assignmentKeyEqualsValue = parent.key.name === parent.value.name;
                            // prevent checking righthand side of destructured object
                            if (!assignmentKeyEqualsValue && parent.key === node) {
                                return;
                            }
                            const valueIsInvalid = parent.value.name && isInvalid(name);
                            // ignore destructuring if the option is set, unless a new identifier is created
                            if (valueIsInvalid && !(assignmentKeyEqualsValue && ignoreDestructuring)) {
                                report(node);
                            }
                        }
                        // never check properties or always ignore destructuring
                        if (!properties || (ignoreDestructuring && isInsideObjectPattern(node))) {
                            return;
                        }
                        // don't check right hand side of AssignmentExpression to prevent duplicate warnings
                        if (parent.right !== node && shouldReport(effectiveParent, name)) {
                            report(node);
                        }
                        // Check if it's an import specifier
                    }
                    else if (IMPORT_TYPES.has(parent.type)) {
                        // Report only if the local imported identifier is invalid
                        if (parent.local && parent.local.name === node.name && isInvalid(name)) {
                            report(node);
                        }
                        // Report anything that is invalid that isn't a CallExpression
                    }
                    else if (shouldReport(effectiveParent, name)) {
                        report(node);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/implicit-arrow-linebreak.js */ function _(require, module, exports) {
    const { isCommentToken, isNotOpeningParenToken } = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce the location of arrow function bodies",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/implicit-arrow-linebreak"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["beside", "below"]
                }
            ],
            messages: {
                expected: "Expected a linebreak before this expression.",
                unexpected: "Expected no linebreak before this expression."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const option = context.options[0] || "beside";
            /**
             * Validates the location of an arrow function body
             * @param {ASTNode} node The arrow function body
             * @returns {void}
             */
            function validateExpression(node) {
                if (node.body.type === "BlockStatement") {
                    return;
                }
                const arrowToken = sourceCode.getTokenBefore(node.body, isNotOpeningParenToken);
                const firstTokenOfBody = sourceCode.getTokenAfter(arrowToken);
                if (arrowToken.loc.end.line === firstTokenOfBody.loc.start.line && option === "below") {
                    context.report({
                        node: firstTokenOfBody,
                        messageId: "expected",
                        fix: fixer => fixer.insertTextBefore(firstTokenOfBody, "\n")
                    });
                }
                else if (arrowToken.loc.end.line !== firstTokenOfBody.loc.start.line && option === "beside") {
                    context.report({
                        node: firstTokenOfBody,
                        messageId: "unexpected",
                        fix(fixer) {
                            if (sourceCode.getFirstTokenBetween(arrowToken, firstTokenOfBody, { includeComments: true, filter: isCommentToken })) {
                                return null;
                            }
                            return fixer.replaceTextRange([arrowToken.range[1], firstTokenOfBody.range[0]], " ");
                        }
                    });
                }
            }
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                ArrowFunctionExpression: node => validateExpression(node)
            };
        }
    };
},
/* eslint/lib/rules/indent.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    const createTree = require(180) /* functional-red-black-tree */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const KNOWN_NODES = new Set([
        "AssignmentExpression",
        "AssignmentPattern",
        "ArrayExpression",
        "ArrayPattern",
        "ArrowFunctionExpression",
        "AwaitExpression",
        "BlockStatement",
        "BinaryExpression",
        "BreakStatement",
        "CallExpression",
        "CatchClause",
        "ClassBody",
        "ClassDeclaration",
        "ClassExpression",
        "ConditionalExpression",
        "ContinueStatement",
        "DoWhileStatement",
        "DebuggerStatement",
        "EmptyStatement",
        "ExperimentalRestProperty",
        "ExperimentalSpreadProperty",
        "ExpressionStatement",
        "ForStatement",
        "ForInStatement",
        "ForOfStatement",
        "FunctionDeclaration",
        "FunctionExpression",
        "Identifier",
        "IfStatement",
        "Literal",
        "LabeledStatement",
        "LogicalExpression",
        "MemberExpression",
        "MetaProperty",
        "MethodDefinition",
        "NewExpression",
        "ObjectExpression",
        "ObjectPattern",
        "Program",
        "Property",
        "RestElement",
        "ReturnStatement",
        "SequenceExpression",
        "SpreadElement",
        "Super",
        "SwitchCase",
        "SwitchStatement",
        "TaggedTemplateExpression",
        "TemplateElement",
        "TemplateLiteral",
        "ThisExpression",
        "ThrowStatement",
        "TryStatement",
        "UnaryExpression",
        "UpdateExpression",
        "VariableDeclaration",
        "VariableDeclarator",
        "WhileStatement",
        "WithStatement",
        "YieldExpression",
        "JSXFragment",
        "JSXOpeningFragment",
        "JSXClosingFragment",
        "JSXIdentifier",
        "JSXNamespacedName",
        "JSXMemberExpression",
        "JSXEmptyExpression",
        "JSXExpressionContainer",
        "JSXElement",
        "JSXClosingElement",
        "JSXOpeningElement",
        "JSXAttribute",
        "JSXSpreadAttribute",
        "JSXText",
        "ExportDefaultDeclaration",
        "ExportNamedDeclaration",
        "ExportAllDeclaration",
        "ExportSpecifier",
        "ImportDeclaration",
        "ImportSpecifier",
        "ImportDefaultSpecifier",
        "ImportNamespaceSpecifier",
        "ImportExpression"
    ]);
    /*
     * General rule strategy:
     * 1. An OffsetStorage instance stores a map of desired offsets, where each token has a specified offset from another
     *    specified token or to the first column.
     * 2. As the AST is traversed, modify the desired offsets of tokens accordingly. For example, when entering a
     *    BlockStatement, offset all of the tokens in the BlockStatement by 1 indent level from the opening curly
     *    brace of the BlockStatement.
     * 3. After traversing the AST, calculate the expected indentation levels of every token according to the
     *    OffsetStorage container.
     * 4. For each line, compare the expected indentation of the first token to the actual indentation in the file,
     *    and report the token if the two values are not equal.
     */
    /**
     * A mutable balanced binary search tree that stores (key, value) pairs. The keys are numeric, and must be unique.
     * This is intended to be a generic wrapper around a balanced binary search tree library, so that the underlying implementation
     * can easily be swapped out.
     */
    class BinarySearchTree {
        /**
         * Creates an empty tree
         */
        constructor() {
            this._rbTree = createTree();
        }
        /**
         * Inserts an entry into the tree.
         * @param {number} key The entry's key
         * @param {*} value The entry's value
         * @returns {void}
         */
        insert(key, value) {
            const iterator = this._rbTree.find(key);
            if (iterator.valid) {
                this._rbTree = iterator.update(value);
            }
            else {
                this._rbTree = this._rbTree.insert(key, value);
            }
        }
        /**
         * Finds the entry with the largest key less than or equal to the provided key
         * @param {number} key The provided key
         * @returns {{key: number, value: *}|null} The found entry, or null if no such entry exists.
         */
        findLe(key) {
            const iterator = this._rbTree.le(key);
            return iterator && { key: iterator.key, value: iterator.value };
        }
        /**
         * Deletes all of the keys in the interval [start, end)
         * @param {number} start The start of the range
         * @param {number} end The end of the range
         * @returns {void}
         */
        deleteRange(start, end) {
            // Exit without traversing the tree if the range has zero size.
            if (start === end) {
                return;
            }
            const iterator = this._rbTree.ge(start);
            while (iterator.valid && iterator.key < end) {
                this._rbTree = this._rbTree.remove(iterator.key);
                iterator.next();
            }
        }
    }
    /**
     * A helper class to get token-based info related to indentation
     */
    class TokenInfo {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {SourceCode} sourceCode A SourceCode object
         */
        constructor(sourceCode) {
            this.sourceCode = sourceCode;
            this.firstTokensByLineNumber = sourceCode.tokensAndComments.reduce((map, token) => {
                if (!map.has(token.loc.start.line)) {
                    map.set(token.loc.start.line, token);
                }
                if (!map.has(token.loc.end.line) && sourceCode.text.slice(token.range[1] - token.loc.end.column, token.range[1]).trim()) {
                    map.set(token.loc.end.line, token);
                }
                return map;
            }, new Map());
        }
        /**
         * Gets the first token on a given token's line
         * @param {Token|ASTNode} token a node or token
         * @returns {Token} The first token on the given line
         */
        getFirstTokenOfLine(token) {
            return this.firstTokensByLineNumber.get(token.loc.start.line);
        }
        /**
         * Determines whether a token is the first token in its line
         * @param {Token} token The token
         * @returns {boolean} `true` if the token is the first on its line
         */
        isFirstTokenOfLine(token) {
            return this.getFirstTokenOfLine(token) === token;
        }
        /**
         * Get the actual indent of a token
         * @param {Token} token Token to examine. This should be the first token on its line.
         * @returns {string} The indentation characters that precede the token
         */
        getTokenIndent(token) {
            return this.sourceCode.text.slice(token.range[0] - token.loc.start.column, token.range[0]);
        }
    }
    /**
     * A class to store information on desired offsets of tokens from each other
     */
    class OffsetStorage {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {TokenInfo} tokenInfo a TokenInfo instance
         * @param {number} indentSize The desired size of each indentation level
         * @param {string} indentType The indentation character
         */
        constructor(tokenInfo, indentSize, indentType) {
            this._tokenInfo = tokenInfo;
            this._indentSize = indentSize;
            this._indentType = indentType;
            this._tree = new BinarySearchTree();
            this._tree.insert(0, { offset: 0, from: null, force: false });
            this._lockedFirstTokens = new WeakMap();
            this._desiredIndentCache = new WeakMap();
            this._ignoredTokens = new WeakSet();
        }
        _getOffsetDescriptor(token) {
            return this._tree.findLe(token.range[0]).value;
        }
        /**
         * Sets the offset column of token B to match the offset column of token A.
         * **WARNING**: This matches a *column*, even if baseToken is not the first token on its line. In
         * most cases, `setDesiredOffset` should be used instead.
         * @param {Token} baseToken The first token
         * @param {Token} offsetToken The second token, whose offset should be matched to the first token
         * @returns {void}
         */
        matchOffsetOf(baseToken, offsetToken) {
            /*
             * lockedFirstTokens is a map from a token whose indentation is controlled by the "first" option to
             * the token that it depends on. For example, with the `ArrayExpression: first` option, the first
             * token of each element in the array after the first will be mapped to the first token of the first
             * element. The desired indentation of each of these tokens is computed based on the desired indentation
             * of the "first" element, rather than through the normal offset mechanism.
             */
            this._lockedFirstTokens.set(offsetToken, baseToken);
        }
        /**
         * Sets the desired offset of a token.
         *
         * This uses a line-based offset collapsing behavior to handle tokens on the same line.
         * For example, consider the following two cases:
         *
         * (
         *     [
         *         bar
         *     ]
         * )
         *
         * ([
         *     bar
         * ])
         *
         * Based on the first case, it's clear that the `bar` token needs to have an offset of 1 indent level (4 spaces) from
         * the `[` token, and the `[` token has to have an offset of 1 indent level from the `(` token. Since the `(` token is
         * the first on its line (with an indent of 0 spaces), the `bar` token needs to be offset by 2 indent levels (8 spaces)
         * from the start of its line.
         *
         * However, in the second case `bar` should only be indented by 4 spaces. This is because the offset of 1 indent level
         * between the `(` and the `[` tokens gets "collapsed" because the two tokens are on the same line. As a result, the
         * `(` token is mapped to the `[` token with an offset of 0, and the rule correctly decides that `bar` should be indented
         * by 1 indent level from the start of the line.
         *
         * This is useful because rule listeners can usually just call `setDesiredOffset` for all the tokens in the node,
         * without needing to check which lines those tokens are on.
         *
         * Note that since collapsing only occurs when two tokens are on the same line, there are a few cases where non-intuitive
         * behavior can occur. For example, consider the following cases:
         *
         * foo(
         * ).
         *     bar(
         *         baz
         *     )
         *
         * foo(
         * ).bar(
         *     baz
         * )
         *
         * Based on the first example, it would seem that `bar` should be offset by 1 indent level from `foo`, and `baz`
         * should be offset by 1 indent level from `bar`. However, this is not correct, because it would result in `baz`
         * being indented by 2 indent levels in the second case (since `foo`, `bar`, and `baz` are all on separate lines, no
         * collapsing would occur).
         *
         * Instead, the correct way would be to offset `baz` by 1 level from `bar`, offset `bar` by 1 level from the `)`, and
         * offset the `)` by 0 levels from `foo`. This ensures that the offset between `bar` and the `)` are correctly collapsed
         * in the second case.
         * @param {Token} token The token
         * @param {Token} fromToken The token that `token` should be offset from
         * @param {number} offset The desired indent level
         * @returns {void}
         */
        setDesiredOffset(token, fromToken, offset) {
            return this.setDesiredOffsets(token.range, fromToken, offset);
        }
        /**
         * Sets the desired offset of all tokens in a range
         * It's common for node listeners in this file to need to apply the same offset to a large, contiguous range of tokens.
         * Moreover, the offset of any given token is usually updated multiple times (roughly once for each node that contains
         * it). This means that the offset of each token is updated O(AST depth) times.
         * It would not be performant to store and update the offsets for each token independently, because the rule would end
         * up having a time complexity of O(number of tokens * AST depth), which is quite slow for large files.
         *
         * Instead, the offset tree is represented as a collection of contiguous offset ranges in a file. For example, the following
         * list could represent the state of the offset tree at a given point:
         *
         * * Tokens starting in the interval [0, 15) are aligned with the beginning of the file
         * * Tokens starting in the interval [15, 30) are offset by 1 indent level from the `bar` token
         * * Tokens starting in the interval [30, 43) are offset by 1 indent level from the `foo` token
         * * Tokens starting in the interval [43, 820) are offset by 2 indent levels from the `bar` token
         * * Tokens starting in the interval [820, ∞) are offset by 1 indent level from the `baz` token
         *
         * The `setDesiredOffsets` methods inserts ranges like the ones above. The third line above would be inserted by using:
         * `setDesiredOffsets([30, 43], fooToken, 1);`
         * @param {[number, number]} range A [start, end] pair. All tokens with range[0] <= token.start < range[1] will have the offset applied.
         * @param {Token} fromToken The token that this is offset from
         * @param {number} offset The desired indent level
         * @param {boolean} force `true` if this offset should not use the normal collapsing behavior. This should almost always be false.
         * @returns {void}
         */
        setDesiredOffsets(range, fromToken, offset, force) {
            /*
             * Offset ranges are stored as a collection of nodes, where each node maps a numeric key to an offset
             * descriptor. The tree for the example above would have the following nodes:
             *
             * * key: 0, value: { offset: 0, from: null }
             * * key: 15, value: { offset: 1, from: barToken }
             * * key: 30, value: { offset: 1, from: fooToken }
             * * key: 43, value: { offset: 2, from: barToken }
             * * key: 820, value: { offset: 1, from: bazToken }
             *
             * To find the offset descriptor for any given token, one needs to find the node with the largest key
             * which is <= token.start. To make this operation fast, the nodes are stored in a balanced binary
             * search tree indexed by key.
             */
            const descriptorToInsert = { offset, from: fromToken, force };
            const descriptorAfterRange = this._tree.findLe(range[1]).value;
            const fromTokenIsInRange = fromToken && fromToken.range[0] >= range[0] && fromToken.range[1] <= range[1];
            const fromTokenDescriptor = fromTokenIsInRange && this._getOffsetDescriptor(fromToken);
            // First, remove any existing nodes in the range from the tree.
            this._tree.deleteRange(range[0] + 1, range[1]);
            // Insert a new node into the tree for this range
            this._tree.insert(range[0], descriptorToInsert);
            /*
             * To avoid circular offset dependencies, keep the `fromToken` token mapped to whatever it was mapped to previously,
             * even if it's in the current range.
             */
            if (fromTokenIsInRange) {
                this._tree.insert(fromToken.range[0], fromTokenDescriptor);
                this._tree.insert(fromToken.range[1], descriptorToInsert);
            }
            /*
             * To avoid modifying the offset of tokens after the range, insert another node to keep the offset of the following
             * tokens the same as it was before.
             */
            this._tree.insert(range[1], descriptorAfterRange);
        }
        /**
         * Gets the desired indent of a token
         * @param {Token} token The token
         * @returns {string} The desired indent of the token
         */
        getDesiredIndent(token) {
            if (!this._desiredIndentCache.has(token)) {
                if (this._ignoredTokens.has(token)) {
                    /*
                     * If the token is ignored, use the actual indent of the token as the desired indent.
                     * This ensures that no errors are reported for this token.
                     */
                    this._desiredIndentCache.set(token, this._tokenInfo.getTokenIndent(token));
                }
                else if (this._lockedFirstTokens.has(token)) {
                    const firstToken = this._lockedFirstTokens.get(token);
                    this._desiredIndentCache.set(token, 
                    // (indentation for the first element's line)
                    this.getDesiredIndent(this._tokenInfo.getFirstTokenOfLine(firstToken)) +
                        // (space between the start of the first element's line and the first element)
                        this._indentType.repeat(firstToken.loc.start.column - this._tokenInfo.getFirstTokenOfLine(firstToken).loc.start.column));
                }
                else {
                    const offsetInfo = this._getOffsetDescriptor(token);
                    const offset = (offsetInfo.from &&
                        offsetInfo.from.loc.start.line === token.loc.start.line &&
                        !/^\s*?\n/u.test(token.value) &&
                        !offsetInfo.force) ? 0 : offsetInfo.offset * this._indentSize;
                    this._desiredIndentCache.set(token, (offsetInfo.from ? this.getDesiredIndent(offsetInfo.from) : "") + this._indentType.repeat(offset));
                }
            }
            return this._desiredIndentCache.get(token);
        }
        /**
         * Ignores a token, preventing it from being reported.
         * @param {Token} token The token
         * @returns {void}
         */
        ignoreToken(token) {
            if (this._tokenInfo.isFirstTokenOfLine(token)) {
                this._ignoredTokens.add(token);
            }
        }
        /**
         * Gets the first token that the given token's indentation is dependent on
         * @param {Token} token The token
         * @returns {Token} The token that the given token depends on, or `null` if the given token is at the top level
         */
        getFirstDependency(token) {
            return this._getOffsetDescriptor(token).from;
        }
    }
    const ELEMENT_LIST_SCHEMA = {
        oneOf: [
            {
                type: "integer",
                minimum: 0
            },
            {
                enum: ["first", "off"]
            }
        ]
    };
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent indentation",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/indent"
            },
            fixable: "whitespace",
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["tab"]
                        },
                        {
                            type: "integer",
                            minimum: 0
                        }
                    ]
                },
                {
                    type: "object",
                    properties: {
                        SwitchCase: {
                            type: "integer",
                            minimum: 0,
                            default: 0
                        },
                        VariableDeclarator: {
                            oneOf: [
                                ELEMENT_LIST_SCHEMA,
                                {
                                    type: "object",
                                    properties: {
                                        var: ELEMENT_LIST_SCHEMA,
                                        let: ELEMENT_LIST_SCHEMA,
                                        const: ELEMENT_LIST_SCHEMA
                                    },
                                    additionalProperties: false
                                }
                            ]
                        },
                        outerIIFEBody: {
                            type: "integer",
                            minimum: 0
                        },
                        MemberExpression: {
                            oneOf: [
                                {
                                    type: "integer",
                                    minimum: 0
                                },
                                {
                                    enum: ["off"]
                                }
                            ]
                        },
                        FunctionDeclaration: {
                            type: "object",
                            properties: {
                                parameters: ELEMENT_LIST_SCHEMA,
                                body: {
                                    type: "integer",
                                    minimum: 0
                                }
                            },
                            additionalProperties: false
                        },
                        FunctionExpression: {
                            type: "object",
                            properties: {
                                parameters: ELEMENT_LIST_SCHEMA,
                                body: {
                                    type: "integer",
                                    minimum: 0
                                }
                            },
                            additionalProperties: false
                        },
                        CallExpression: {
                            type: "object",
                            properties: {
                                arguments: ELEMENT_LIST_SCHEMA
                            },
                            additionalProperties: false
                        },
                        ArrayExpression: ELEMENT_LIST_SCHEMA,
                        ObjectExpression: ELEMENT_LIST_SCHEMA,
                        ImportDeclaration: ELEMENT_LIST_SCHEMA,
                        flatTernaryExpressions: {
                            type: "boolean",
                            default: false
                        },
                        ignoredNodes: {
                            type: "array",
                            items: {
                                type: "string",
                                not: {
                                    pattern: ":exit$"
                                }
                            }
                        },
                        ignoreComments: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                wrongIndentation: "Expected indentation of {{expected}} but found {{actual}}."
            }
        },
        create(context) {
            const DEFAULT_VARIABLE_INDENT = 1;
            const DEFAULT_PARAMETER_INDENT = 1;
            const DEFAULT_FUNCTION_BODY_INDENT = 1;
            let indentType = "space";
            let indentSize = 4;
            const options = {
                SwitchCase: 0,
                VariableDeclarator: {
                    var: DEFAULT_VARIABLE_INDENT,
                    let: DEFAULT_VARIABLE_INDENT,
                    const: DEFAULT_VARIABLE_INDENT
                },
                outerIIFEBody: 1,
                FunctionDeclaration: {
                    parameters: DEFAULT_PARAMETER_INDENT,
                    body: DEFAULT_FUNCTION_BODY_INDENT
                },
                FunctionExpression: {
                    parameters: DEFAULT_PARAMETER_INDENT,
                    body: DEFAULT_FUNCTION_BODY_INDENT
                },
                CallExpression: {
                    arguments: DEFAULT_PARAMETER_INDENT
                },
                MemberExpression: 1,
                ArrayExpression: 1,
                ObjectExpression: 1,
                ImportDeclaration: 1,
                flatTernaryExpressions: false,
                ignoredNodes: [],
                ignoreComments: false
            };
            if (context.options.length) {
                if (context.options[0] === "tab") {
                    indentSize = 1;
                    indentType = "tab";
                }
                else {
                    indentSize = context.options[0];
                    indentType = "space";
                }
                if (context.options[1]) {
                    Object.assign(options, context.options[1]);
                    if (typeof options.VariableDeclarator === "number" || options.VariableDeclarator === "first") {
                        options.VariableDeclarator = {
                            var: options.VariableDeclarator,
                            let: options.VariableDeclarator,
                            const: options.VariableDeclarator
                        };
                    }
                }
            }
            const sourceCode = context.getSourceCode();
            const tokenInfo = new TokenInfo(sourceCode);
            const offsets = new OffsetStorage(tokenInfo, indentSize, indentType === "space" ? " " : "\t");
            const parameterParens = new WeakSet();
            /**
             * Creates an error message for a line, given the expected/actual indentation.
             * @param {int} expectedAmount The expected amount of indentation characters for this line
             * @param {int} actualSpaces The actual number of indentation spaces that were found on this line
             * @param {int} actualTabs The actual number of indentation tabs that were found on this line
             * @returns {string} An error message for this line
             */
            function createErrorMessageData(expectedAmount, actualSpaces, actualTabs) {
                const expectedStatement = `${expectedAmount} ${indentType}${expectedAmount === 1 ? "" : "s"}`; // e.g. "2 tabs"
                const foundSpacesWord = `space${actualSpaces === 1 ? "" : "s"}`; // e.g. "space"
                const foundTabsWord = `tab${actualTabs === 1 ? "" : "s"}`; // e.g. "tabs"
                let foundStatement;
                if (actualSpaces > 0) {
                    /*
                     * Abbreviate the message if the expected indentation is also spaces.
                     * e.g. 'Expected 4 spaces but found 2' rather than 'Expected 4 spaces but found 2 spaces'
                     */
                    foundStatement = indentType === "space" ? actualSpaces : `${actualSpaces} ${foundSpacesWord}`;
                }
                else if (actualTabs > 0) {
                    foundStatement = indentType === "tab" ? actualTabs : `${actualTabs} ${foundTabsWord}`;
                }
                else {
                    foundStatement = "0";
                }
                return {
                    expected: expectedStatement,
                    actual: foundStatement
                };
            }
            /**
             * Reports a given indent violation
             * @param {Token} token Token violating the indent rule
             * @param {string} neededIndent Expected indentation string
             * @returns {void}
             */
            function report(token, neededIndent) {
                const actualIndent = Array.from(tokenInfo.getTokenIndent(token));
                const numSpaces = actualIndent.filter(char => char === " ").length;
                const numTabs = actualIndent.filter(char => char === "\t").length;
                context.report({
                    node: token,
                    messageId: "wrongIndentation",
                    data: createErrorMessageData(neededIndent.length, numSpaces, numTabs),
                    loc: {
                        start: { line: token.loc.start.line, column: 0 },
                        end: { line: token.loc.start.line, column: token.loc.start.column }
                    },
                    fix(fixer) {
                        const range = [token.range[0] - token.loc.start.column, token.range[0]];
                        const newText = neededIndent;
                        return fixer.replaceTextRange(range, newText);
                    }
                });
            }
            /**
             * Checks if a token's indentation is correct
             * @param {Token} token Token to examine
             * @param {string} desiredIndent Desired indentation of the string
             * @returns {boolean} `true` if the token's indentation is correct
             */
            function validateTokenIndent(token, desiredIndent) {
                const indentation = tokenInfo.getTokenIndent(token);
                return indentation === desiredIndent ||
                    // To avoid conflicts with no-mixed-spaces-and-tabs, don't report mixed spaces and tabs.
                    indentation.includes(" ") && indentation.includes("\t");
            }
            /**
             * Check to see if the node is a file level IIFE
             * @param {ASTNode} node The function node to check.
             * @returns {boolean} True if the node is the outer IIFE
             */
            function isOuterIIFE(node) {
                /*
                 * Verify that the node is an IIFE
                 */
                if (!node.parent || node.parent.type !== "CallExpression" || node.parent.callee !== node) {
                    return false;
                }
                /*
                 * Navigate legal ancestors to determine whether this IIFE is outer.
                 * A "legal ancestor" is an expression or statement that causes the function to get executed immediately.
                 * For example, `!(function(){})()` is an outer IIFE even though it is preceded by a ! operator.
                 */
                let statement = node.parent && node.parent.parent;
                while (statement.type === "UnaryExpression" && ["!", "~", "+", "-"].indexOf(statement.operator) > -1 ||
                    statement.type === "AssignmentExpression" ||
                    statement.type === "LogicalExpression" ||
                    statement.type === "SequenceExpression" ||
                    statement.type === "VariableDeclarator") {
                    statement = statement.parent;
                }
                return (statement.type === "ExpressionStatement" || statement.type === "VariableDeclaration") && statement.parent.type === "Program";
            }
            /**
             * Counts the number of linebreaks that follow the last non-whitespace character in a string
             * @param {string} string The string to check
             * @returns {number} The number of JavaScript linebreaks that follow the last non-whitespace character,
             * or the total number of linebreaks if the string is all whitespace.
             */
            function countTrailingLinebreaks(string) {
                const trailingWhitespace = string.match(/\s*$/u)[0];
                const linebreakMatches = trailingWhitespace.match(astUtils.createGlobalLinebreakMatcher());
                return linebreakMatches === null ? 0 : linebreakMatches.length;
            }
            /**
             * Check indentation for lists of elements (arrays, objects, function params)
             * @param {ASTNode[]} elements List of elements that should be offset
             * @param {Token} startToken The start token of the list that element should be aligned against, e.g. '['
             * @param {Token} endToken The end token of the list, e.g. ']'
             * @param {number|string} offset The amount that the elements should be offset
             * @returns {void}
             */
            function addElementListIndent(elements, startToken, endToken, offset) {
                /**
                 * Gets the first token of a given element, including surrounding parentheses.
                 * @param {ASTNode} element A node in the `elements` list
                 * @returns {Token} The first token of this element
                 */
                function getFirstToken(element) {
                    let token = sourceCode.getTokenBefore(element);
                    while (astUtils.isOpeningParenToken(token) && token !== startToken) {
                        token = sourceCode.getTokenBefore(token);
                    }
                    return sourceCode.getTokenAfter(token);
                }
                // Run through all the tokens in the list, and offset them by one indent level (mainly for comments, other things will end up overridden)
                offsets.setDesiredOffsets([startToken.range[1], endToken.range[0]], startToken, typeof offset === "number" ? offset : 1);
                offsets.setDesiredOffset(endToken, startToken, 0);
                // If the preference is "first" but there is no first element (e.g. sparse arrays w/ empty first slot), fall back to 1 level.
                if (offset === "first" && elements.length && !elements[0]) {
                    return;
                }
                elements.forEach((element, index) => {
                    if (!element) {
                        // Skip holes in arrays
                        return;
                    }
                    if (offset === "off") {
                        // Ignore the first token of every element if the "off" option is used
                        offsets.ignoreToken(getFirstToken(element));
                    }
                    // Offset the following elements correctly relative to the first element
                    if (index === 0) {
                        return;
                    }
                    if (offset === "first" && tokenInfo.isFirstTokenOfLine(getFirstToken(element))) {
                        offsets.matchOffsetOf(getFirstToken(elements[0]), getFirstToken(element));
                    }
                    else {
                        const previousElement = elements[index - 1];
                        const firstTokenOfPreviousElement = previousElement && getFirstToken(previousElement);
                        const previousElementLastToken = previousElement && sourceCode.getLastToken(previousElement);
                        if (previousElement &&
                            previousElementLastToken.loc.end.line - countTrailingLinebreaks(previousElementLastToken.value) > startToken.loc.end.line) {
                            offsets.setDesiredOffsets([previousElement.range[1], element.range[1]], firstTokenOfPreviousElement, 0);
                        }
                    }
                });
            }
            /**
             * Check and decide whether to check for indentation for blockless nodes
             * Scenarios are for or while statements without braces around them
             * @param {ASTNode} node node to examine
             * @returns {void}
             */
            function addBlocklessNodeIndent(node) {
                if (node.type !== "BlockStatement") {
                    const lastParentToken = sourceCode.getTokenBefore(node, astUtils.isNotOpeningParenToken);
                    let firstBodyToken = sourceCode.getFirstToken(node);
                    let lastBodyToken = sourceCode.getLastToken(node);
                    while (astUtils.isOpeningParenToken(sourceCode.getTokenBefore(firstBodyToken)) &&
                        astUtils.isClosingParenToken(sourceCode.getTokenAfter(lastBodyToken))) {
                        firstBodyToken = sourceCode.getTokenBefore(firstBodyToken);
                        lastBodyToken = sourceCode.getTokenAfter(lastBodyToken);
                    }
                    offsets.setDesiredOffsets([firstBodyToken.range[0], lastBodyToken.range[1]], lastParentToken, 1);
                    /*
                     * For blockless nodes with semicolon-first style, don't indent the semicolon.
                     * e.g.
                     * if (foo) bar()
                     * ; [1, 2, 3].map(foo)
                     */
                    const lastToken = sourceCode.getLastToken(node);
                    if (node.type !== "EmptyStatement" && astUtils.isSemicolonToken(lastToken)) {
                        offsets.setDesiredOffset(lastToken, lastParentToken, 0);
                    }
                }
            }
            /**
             * Checks the indentation for nodes that are like function calls (`CallExpression` and `NewExpression`)
             * @param {ASTNode} node A CallExpression or NewExpression node
             * @returns {void}
             */
            function addFunctionCallIndent(node) {
                let openingParen;
                if (node.arguments.length) {
                    openingParen = sourceCode.getFirstTokenBetween(node.callee, node.arguments[0], astUtils.isOpeningParenToken);
                }
                else {
                    openingParen = sourceCode.getLastToken(node, 1);
                }
                const closingParen = sourceCode.getLastToken(node);
                parameterParens.add(openingParen);
                parameterParens.add(closingParen);
                offsets.setDesiredOffset(openingParen, sourceCode.getTokenBefore(openingParen), 0);
                addElementListIndent(node.arguments, openingParen, closingParen, options.CallExpression.arguments);
            }
            /**
             * Checks the indentation of parenthesized values, given a list of tokens in a program
             * @param {Token[]} tokens A list of tokens
             * @returns {void}
             */
            function addParensIndent(tokens) {
                const parenStack = [];
                const parenPairs = [];
                tokens.forEach(nextToken => {
                    // Accumulate a list of parenthesis pairs
                    if (astUtils.isOpeningParenToken(nextToken)) {
                        parenStack.push(nextToken);
                    }
                    else if (astUtils.isClosingParenToken(nextToken)) {
                        parenPairs.unshift({ left: parenStack.pop(), right: nextToken });
                    }
                });
                parenPairs.forEach(pair => {
                    const leftParen = pair.left;
                    const rightParen = pair.right;
                    // We only want to handle parens around expressions, so exclude parentheses that are in function parameters and function call arguments.
                    if (!parameterParens.has(leftParen) && !parameterParens.has(rightParen)) {
                        const parenthesizedTokens = new Set(sourceCode.getTokensBetween(leftParen, rightParen));
                        parenthesizedTokens.forEach(token => {
                            if (!parenthesizedTokens.has(offsets.getFirstDependency(token))) {
                                offsets.setDesiredOffset(token, leftParen, 1);
                            }
                        });
                    }
                    offsets.setDesiredOffset(rightParen, leftParen, 0);
                });
            }
            /**
             * Ignore all tokens within an unknown node whose offset do not depend
             * on another token's offset within the unknown node
             * @param {ASTNode} node Unknown Node
             * @returns {void}
             */
            function ignoreNode(node) {
                const unknownNodeTokens = new Set(sourceCode.getTokens(node, { includeComments: true }));
                unknownNodeTokens.forEach(token => {
                    if (!unknownNodeTokens.has(offsets.getFirstDependency(token))) {
                        const firstTokenOfLine = tokenInfo.getFirstTokenOfLine(token);
                        if (token === firstTokenOfLine) {
                            offsets.ignoreToken(token);
                        }
                        else {
                            offsets.setDesiredOffset(token, firstTokenOfLine, 0);
                        }
                    }
                });
            }
            /**
             * Check whether the given token is on the first line of a statement.
             * @param {Token} token The token to check.
             * @param {ASTNode} leafNode The expression node that the token belongs directly.
             * @returns {boolean} `true` if the token is on the first line of a statement.
             */
            function isOnFirstLineOfStatement(token, leafNode) {
                let node = leafNode;
                while (node.parent && !node.parent.type.endsWith("Statement") && !node.parent.type.endsWith("Declaration")) {
                    node = node.parent;
                }
                node = node.parent;
                return !node || node.loc.start.line === token.loc.start.line;
            }
            /**
             * Check whether there are any blank (whitespace-only) lines between
             * two tokens on separate lines.
             * @param {Token} firstToken The first token.
             * @param {Token} secondToken The second token.
             * @returns {boolean} `true` if the tokens are on separate lines and
             *   there exists a blank line between them, `false` otherwise.
             */
            function hasBlankLinesBetween(firstToken, secondToken) {
                const firstTokenLine = firstToken.loc.end.line;
                const secondTokenLine = secondToken.loc.start.line;
                if (firstTokenLine === secondTokenLine || firstTokenLine === secondTokenLine - 1) {
                    return false;
                }
                for (let line = firstTokenLine + 1; line < secondTokenLine; ++line) {
                    if (!tokenInfo.firstTokensByLineNumber.has(line)) {
                        return true;
                    }
                }
                return false;
            }
            const ignoredNodeFirstTokens = new Set();
            const baseOffsetListeners = {
                "ArrayExpression, ArrayPattern"(node) {
                    const openingBracket = sourceCode.getFirstToken(node);
                    const closingBracket = sourceCode.getTokenAfter(lodash.findLast(node.elements) || openingBracket, astUtils.isClosingBracketToken);
                    addElementListIndent(node.elements, openingBracket, closingBracket, options.ArrayExpression);
                },
                "ObjectExpression, ObjectPattern"(node) {
                    const openingCurly = sourceCode.getFirstToken(node);
                    const closingCurly = sourceCode.getTokenAfter(node.properties.length ? node.properties[node.properties.length - 1] : openingCurly, astUtils.isClosingBraceToken);
                    addElementListIndent(node.properties, openingCurly, closingCurly, options.ObjectExpression);
                },
                ArrowFunctionExpression(node) {
                    const firstToken = sourceCode.getFirstToken(node);
                    if (astUtils.isOpeningParenToken(firstToken)) {
                        const openingParen = firstToken;
                        const closingParen = sourceCode.getTokenBefore(node.body, astUtils.isClosingParenToken);
                        parameterParens.add(openingParen);
                        parameterParens.add(closingParen);
                        addElementListIndent(node.params, openingParen, closingParen, options.FunctionExpression.parameters);
                    }
                    addBlocklessNodeIndent(node.body);
                },
                AssignmentExpression(node) {
                    const operator = sourceCode.getFirstTokenBetween(node.left, node.right, token => token.value === node.operator);
                    offsets.setDesiredOffsets([operator.range[0], node.range[1]], sourceCode.getLastToken(node.left), 1);
                    offsets.ignoreToken(operator);
                    offsets.ignoreToken(sourceCode.getTokenAfter(operator));
                },
                "BinaryExpression, LogicalExpression"(node) {
                    const operator = sourceCode.getFirstTokenBetween(node.left, node.right, token => token.value === node.operator);
                    /*
                     * For backwards compatibility, don't check BinaryExpression indents, e.g.
                     * var foo = bar &&
                     *                   baz;
                     */
                    const tokenAfterOperator = sourceCode.getTokenAfter(operator);
                    offsets.ignoreToken(operator);
                    offsets.ignoreToken(tokenAfterOperator);
                    offsets.setDesiredOffset(tokenAfterOperator, operator, 0);
                },
                "BlockStatement, ClassBody"(node) {
                    let blockIndentLevel;
                    if (node.parent && isOuterIIFE(node.parent)) {
                        blockIndentLevel = options.outerIIFEBody;
                    }
                    else if (node.parent && (node.parent.type === "FunctionExpression" || node.parent.type === "ArrowFunctionExpression")) {
                        blockIndentLevel = options.FunctionExpression.body;
                    }
                    else if (node.parent && node.parent.type === "FunctionDeclaration") {
                        blockIndentLevel = options.FunctionDeclaration.body;
                    }
                    else {
                        blockIndentLevel = 1;
                    }
                    /*
                     * For blocks that aren't lone statements, ensure that the opening curly brace
                     * is aligned with the parent.
                     */
                    if (!astUtils.STATEMENT_LIST_PARENTS.has(node.parent.type)) {
                        offsets.setDesiredOffset(sourceCode.getFirstToken(node), sourceCode.getFirstToken(node.parent), 0);
                    }
                    addElementListIndent(node.body, sourceCode.getFirstToken(node), sourceCode.getLastToken(node), blockIndentLevel);
                },
                CallExpression: addFunctionCallIndent,
                "ClassDeclaration[superClass], ClassExpression[superClass]"(node) {
                    const classToken = sourceCode.getFirstToken(node);
                    const extendsToken = sourceCode.getTokenBefore(node.superClass, astUtils.isNotOpeningParenToken);
                    offsets.setDesiredOffsets([extendsToken.range[0], node.body.range[0]], classToken, 1);
                },
                ConditionalExpression(node) {
                    const firstToken = sourceCode.getFirstToken(node);
                    // `flatTernaryExpressions` option is for the following style:
                    // var a =
                    //     foo > 0 ? bar :
                    //     foo < 0 ? baz :
                    //     /*else*/ qiz ;
                    if (!options.flatTernaryExpressions ||
                        !astUtils.isTokenOnSameLine(node.test, node.consequent) ||
                        isOnFirstLineOfStatement(firstToken, node)) {
                        const questionMarkToken = sourceCode.getFirstTokenBetween(node.test, node.consequent, token => token.type === "Punctuator" && token.value === "?");
                        const colonToken = sourceCode.getFirstTokenBetween(node.consequent, node.alternate, token => token.type === "Punctuator" && token.value === ":");
                        const firstConsequentToken = sourceCode.getTokenAfter(questionMarkToken);
                        const lastConsequentToken = sourceCode.getTokenBefore(colonToken);
                        const firstAlternateToken = sourceCode.getTokenAfter(colonToken);
                        offsets.setDesiredOffset(questionMarkToken, firstToken, 1);
                        offsets.setDesiredOffset(colonToken, firstToken, 1);
                        offsets.setDesiredOffset(firstConsequentToken, firstToken, 1);
                        /*
                         * The alternate and the consequent should usually have the same indentation.
                         * If they share part of a line, align the alternate against the first token of the consequent.
                         * This allows the alternate to be indented correctly in cases like this:
                         * foo ? (
                         *   bar
                         * ) : ( // this '(' is aligned with the '(' above, so it's considered to be aligned with `foo`
                         *   baz // as a result, `baz` is offset by 1 rather than 2
                         * )
                         */
                        if (lastConsequentToken.loc.end.line === firstAlternateToken.loc.start.line) {
                            offsets.setDesiredOffset(firstAlternateToken, firstConsequentToken, 0);
                        }
                        else {
                            /**
                             * If the alternate and consequent do not share part of a line, offset the alternate from the first
                             * token of the conditional expression. For example:
                             * foo ? bar
                             *   : baz
                             *
                             * If `baz` were aligned with `bar` rather than being offset by 1 from `foo`, `baz` would end up
                             * having no expected indentation.
                             */
                            offsets.setDesiredOffset(firstAlternateToken, firstToken, 1);
                        }
                    }
                },
                "DoWhileStatement, WhileStatement, ForInStatement, ForOfStatement": node => addBlocklessNodeIndent(node.body),
                ExportNamedDeclaration(node) {
                    if (node.declaration === null) {
                        const closingCurly = sourceCode.getLastToken(node, astUtils.isClosingBraceToken);
                        // Indent the specifiers in `export {foo, bar, baz}`
                        addElementListIndent(node.specifiers, sourceCode.getFirstToken(node, { skip: 1 }), closingCurly, 1);
                        if (node.source) {
                            // Indent everything after and including the `from` token in `export {foo, bar, baz} from 'qux'`
                            offsets.setDesiredOffsets([closingCurly.range[1], node.range[1]], sourceCode.getFirstToken(node), 1);
                        }
                    }
                },
                ForStatement(node) {
                    const forOpeningParen = sourceCode.getFirstToken(node, 1);
                    if (node.init) {
                        offsets.setDesiredOffsets(node.init.range, forOpeningParen, 1);
                    }
                    if (node.test) {
                        offsets.setDesiredOffsets(node.test.range, forOpeningParen, 1);
                    }
                    if (node.update) {
                        offsets.setDesiredOffsets(node.update.range, forOpeningParen, 1);
                    }
                    addBlocklessNodeIndent(node.body);
                },
                "FunctionDeclaration, FunctionExpression"(node) {
                    const closingParen = sourceCode.getTokenBefore(node.body);
                    const openingParen = sourceCode.getTokenBefore(node.params.length ? node.params[0] : closingParen);
                    parameterParens.add(openingParen);
                    parameterParens.add(closingParen);
                    addElementListIndent(node.params, openingParen, closingParen, options[node.type].parameters);
                },
                IfStatement(node) {
                    addBlocklessNodeIndent(node.consequent);
                    if (node.alternate && node.alternate.type !== "IfStatement") {
                        addBlocklessNodeIndent(node.alternate);
                    }
                },
                ImportDeclaration(node) {
                    if (node.specifiers.some(specifier => specifier.type === "ImportSpecifier")) {
                        const openingCurly = sourceCode.getFirstToken(node, astUtils.isOpeningBraceToken);
                        const closingCurly = sourceCode.getLastToken(node, astUtils.isClosingBraceToken);
                        addElementListIndent(node.specifiers.filter(specifier => specifier.type === "ImportSpecifier"), openingCurly, closingCurly, options.ImportDeclaration);
                    }
                    const fromToken = sourceCode.getLastToken(node, token => token.type === "Identifier" && token.value === "from");
                    const sourceToken = sourceCode.getLastToken(node, token => token.type === "String");
                    const semiToken = sourceCode.getLastToken(node, token => token.type === "Punctuator" && token.value === ";");
                    if (fromToken) {
                        const end = semiToken && semiToken.range[1] === sourceToken.range[1] ? node.range[1] : sourceToken.range[1];
                        offsets.setDesiredOffsets([fromToken.range[0], end], sourceCode.getFirstToken(node), 1);
                    }
                },
                ImportExpression(node) {
                    const openingParen = sourceCode.getFirstToken(node, 1);
                    const closingParen = sourceCode.getLastToken(node);
                    parameterParens.add(openingParen);
                    parameterParens.add(closingParen);
                    offsets.setDesiredOffset(openingParen, sourceCode.getTokenBefore(openingParen), 0);
                    addElementListIndent([node.source], openingParen, closingParen, options.CallExpression.arguments);
                },
                "MemberExpression, JSXMemberExpression, MetaProperty"(node) {
                    const object = node.type === "MetaProperty" ? node.meta : node.object;
                    const firstNonObjectToken = sourceCode.getFirstTokenBetween(object, node.property, astUtils.isNotClosingParenToken);
                    const secondNonObjectToken = sourceCode.getTokenAfter(firstNonObjectToken);
                    const objectParenCount = sourceCode.getTokensBetween(object, node.property, { filter: astUtils.isClosingParenToken }).length;
                    const firstObjectToken = objectParenCount
                        ? sourceCode.getTokenBefore(object, { skip: objectParenCount - 1 })
                        : sourceCode.getFirstToken(object);
                    const lastObjectToken = sourceCode.getTokenBefore(firstNonObjectToken);
                    const firstPropertyToken = node.computed ? firstNonObjectToken : secondNonObjectToken;
                    if (node.computed) {
                        // For computed MemberExpressions, match the closing bracket with the opening bracket.
                        offsets.setDesiredOffset(sourceCode.getLastToken(node), firstNonObjectToken, 0);
                        offsets.setDesiredOffsets(node.property.range, firstNonObjectToken, 1);
                    }
                    /*
                     * If the object ends on the same line that the property starts, match against the last token
                     * of the object, to ensure that the MemberExpression is not indented.
                     *
                     * Otherwise, match against the first token of the object, e.g.
                     * foo
                     *   .bar
                     *   .baz // <-- offset by 1 from `foo`
                     */
                    const offsetBase = lastObjectToken.loc.end.line === firstPropertyToken.loc.start.line
                        ? lastObjectToken
                        : firstObjectToken;
                    if (typeof options.MemberExpression === "number") {
                        // Match the dot (for non-computed properties) or the opening bracket (for computed properties) against the object.
                        offsets.setDesiredOffset(firstNonObjectToken, offsetBase, options.MemberExpression);
                        /*
                         * For computed MemberExpressions, match the first token of the property against the opening bracket.
                         * Otherwise, match the first token of the property against the object.
                         */
                        offsets.setDesiredOffset(secondNonObjectToken, node.computed ? firstNonObjectToken : offsetBase, options.MemberExpression);
                    }
                    else {
                        // If the MemberExpression option is off, ignore the dot and the first token of the property.
                        offsets.ignoreToken(firstNonObjectToken);
                        offsets.ignoreToken(secondNonObjectToken);
                        // To ignore the property indentation, ensure that the property tokens depend on the ignored tokens.
                        offsets.setDesiredOffset(firstNonObjectToken, offsetBase, 0);
                        offsets.setDesiredOffset(secondNonObjectToken, firstNonObjectToken, 0);
                    }
                },
                NewExpression(node) {
                    // Only indent the arguments if the NewExpression has parens (e.g. `new Foo(bar)` or `new Foo()`, but not `new Foo`
                    if (node.arguments.length > 0 ||
                        astUtils.isClosingParenToken(sourceCode.getLastToken(node)) &&
                            astUtils.isOpeningParenToken(sourceCode.getLastToken(node, 1))) {
                        addFunctionCallIndent(node);
                    }
                },
                Property(node) {
                    if (!node.shorthand && !node.method && node.kind === "init") {
                        const colon = sourceCode.getFirstTokenBetween(node.key, node.value, astUtils.isColonToken);
                        offsets.ignoreToken(sourceCode.getTokenAfter(colon));
                    }
                },
                SwitchStatement(node) {
                    const openingCurly = sourceCode.getTokenAfter(node.discriminant, astUtils.isOpeningBraceToken);
                    const closingCurly = sourceCode.getLastToken(node);
                    offsets.setDesiredOffsets([openingCurly.range[1], closingCurly.range[0]], openingCurly, options.SwitchCase);
                    if (node.cases.length) {
                        sourceCode.getTokensBetween(node.cases[node.cases.length - 1], closingCurly, { includeComments: true, filter: astUtils.isCommentToken }).forEach(token => offsets.ignoreToken(token));
                    }
                },
                SwitchCase(node) {
                    if (!(node.consequent.length === 1 && node.consequent[0].type === "BlockStatement")) {
                        const caseKeyword = sourceCode.getFirstToken(node);
                        const tokenAfterCurrentCase = sourceCode.getTokenAfter(node);
                        offsets.setDesiredOffsets([caseKeyword.range[1], tokenAfterCurrentCase.range[0]], caseKeyword, 1);
                    }
                },
                TemplateLiteral(node) {
                    node.expressions.forEach((expression, index) => {
                        const previousQuasi = node.quasis[index];
                        const nextQuasi = node.quasis[index + 1];
                        const tokenToAlignFrom = previousQuasi.loc.start.line === previousQuasi.loc.end.line
                            ? sourceCode.getFirstToken(previousQuasi)
                            : null;
                        offsets.setDesiredOffsets([previousQuasi.range[1], nextQuasi.range[0]], tokenToAlignFrom, 1);
                        offsets.setDesiredOffset(sourceCode.getFirstToken(nextQuasi), tokenToAlignFrom, 0);
                    });
                },
                VariableDeclaration(node) {
                    let variableIndent = Object.prototype.hasOwnProperty.call(options.VariableDeclarator, node.kind)
                        ? options.VariableDeclarator[node.kind]
                        : DEFAULT_VARIABLE_INDENT;
                    const firstToken = sourceCode.getFirstToken(node), lastToken = sourceCode.getLastToken(node);
                    if (options.VariableDeclarator[node.kind] === "first") {
                        if (node.declarations.length > 1) {
                            addElementListIndent(node.declarations, firstToken, lastToken, "first");
                            return;
                        }
                        variableIndent = DEFAULT_VARIABLE_INDENT;
                    }
                    if (node.declarations[node.declarations.length - 1].loc.start.line > node.loc.start.line) {
                        /*
                         * VariableDeclarator indentation is a bit different from other forms of indentation, in that the
                         * indentation of an opening bracket sometimes won't match that of a closing bracket. For example,
                         * the following indentations are correct:
                         *
                         * var foo = {
                         *   ok: true
                         * };
                         *
                         * var foo = {
                         *     ok: true,
                         *   },
                         *   bar = 1;
                         *
                         * Account for when exiting the AST (after indentations have already been set for the nodes in
                         * the declaration) by manually increasing the indentation level of the tokens in this declarator
                         * on the same line as the start of the declaration, provided that there are declarators that
                         * follow this one.
                         */
                        offsets.setDesiredOffsets(node.range, firstToken, variableIndent, true);
                    }
                    else {
                        offsets.setDesiredOffsets(node.range, firstToken, variableIndent);
                    }
                    if (astUtils.isSemicolonToken(lastToken)) {
                        offsets.ignoreToken(lastToken);
                    }
                },
                VariableDeclarator(node) {
                    if (node.init) {
                        const equalOperator = sourceCode.getTokenBefore(node.init, astUtils.isNotOpeningParenToken);
                        const tokenAfterOperator = sourceCode.getTokenAfter(equalOperator);
                        offsets.ignoreToken(equalOperator);
                        offsets.ignoreToken(tokenAfterOperator);
                        offsets.setDesiredOffsets([tokenAfterOperator.range[0], node.range[1]], equalOperator, 1);
                        offsets.setDesiredOffset(equalOperator, sourceCode.getLastToken(node.id), 0);
                    }
                },
                "JSXAttribute[value]"(node) {
                    const equalsToken = sourceCode.getFirstTokenBetween(node.name, node.value, token => token.type === "Punctuator" && token.value === "=");
                    offsets.setDesiredOffsets([equalsToken.range[0], node.value.range[1]], sourceCode.getFirstToken(node.name), 1);
                },
                JSXElement(node) {
                    if (node.closingElement) {
                        addElementListIndent(node.children, sourceCode.getFirstToken(node.openingElement), sourceCode.getFirstToken(node.closingElement), 1);
                    }
                },
                JSXOpeningElement(node) {
                    const firstToken = sourceCode.getFirstToken(node);
                    let closingToken;
                    if (node.selfClosing) {
                        closingToken = sourceCode.getLastToken(node, { skip: 1 });
                        offsets.setDesiredOffset(sourceCode.getLastToken(node), closingToken, 0);
                    }
                    else {
                        closingToken = sourceCode.getLastToken(node);
                    }
                    offsets.setDesiredOffsets(node.name.range, sourceCode.getFirstToken(node));
                    addElementListIndent(node.attributes, firstToken, closingToken, 1);
                },
                JSXClosingElement(node) {
                    const firstToken = sourceCode.getFirstToken(node);
                    offsets.setDesiredOffsets(node.name.range, firstToken, 1);
                },
                JSXFragment(node) {
                    const firstOpeningToken = sourceCode.getFirstToken(node.openingFragment);
                    const firstClosingToken = sourceCode.getFirstToken(node.closingFragment);
                    addElementListIndent(node.children, firstOpeningToken, firstClosingToken, 1);
                },
                JSXOpeningFragment(node) {
                    const firstToken = sourceCode.getFirstToken(node);
                    const closingToken = sourceCode.getLastToken(node);
                    offsets.setDesiredOffsets(node.range, firstToken, 1);
                    offsets.matchOffsetOf(firstToken, closingToken);
                },
                JSXClosingFragment(node) {
                    const firstToken = sourceCode.getFirstToken(node);
                    const slashToken = sourceCode.getLastToken(node, { skip: 1 });
                    const closingToken = sourceCode.getLastToken(node);
                    const tokenToMatch = astUtils.isTokenOnSameLine(slashToken, closingToken) ? slashToken : closingToken;
                    offsets.setDesiredOffsets(node.range, firstToken, 1);
                    offsets.matchOffsetOf(firstToken, tokenToMatch);
                },
                JSXExpressionContainer(node) {
                    const openingCurly = sourceCode.getFirstToken(node);
                    const closingCurly = sourceCode.getLastToken(node);
                    offsets.setDesiredOffsets([openingCurly.range[1], closingCurly.range[0]], openingCurly, 1);
                },
                JSXSpreadAttribute(node) {
                    const openingCurly = sourceCode.getFirstToken(node);
                    const closingCurly = sourceCode.getLastToken(node);
                    offsets.setDesiredOffsets([openingCurly.range[1], closingCurly.range[0]], openingCurly, 1);
                },
                "*"(node) {
                    const firstToken = sourceCode.getFirstToken(node);
                    // Ensure that the children of every node are indented at least as much as the first token.
                    if (firstToken && !ignoredNodeFirstTokens.has(firstToken)) {
                        offsets.setDesiredOffsets(node.range, firstToken, 0);
                    }
                }
            };
            const listenerCallQueue = [];
            /*
             * To ignore the indentation of a node:
             * 1. Don't call the node's listener when entering it (if it has a listener)
             * 2. Don't set any offsets against the first token of the node.
             * 3. Call `ignoreNode` on the node sometime after exiting it and before validating offsets.
             */
            const offsetListeners = lodash.mapValues(baseOffsetListeners, 
            /*
             * Offset listener calls are deferred until traversal is finished, and are called as
             * part of the final `Program:exit` listener. This is necessary because a node might
             * be matched by multiple selectors.
             *
             * Example: Suppose there is an offset listener for `Identifier`, and the user has
             * specified in configuration that `MemberExpression > Identifier` should be ignored.
             * Due to selector specificity rules, the `Identifier` listener will get called first. However,
             * if a given Identifier node is supposed to be ignored, then the `Identifier` offset listener
             * should not have been called at all. Without doing extra selector matching, we don't know
             * whether the Identifier matches the `MemberExpression > Identifier` selector until the
             * `MemberExpression > Identifier` listener is called.
             *
             * To avoid this, the `Identifier` listener isn't called until traversal finishes and all
             * ignored nodes are known.
             */
            listener => node => listenerCallQueue.push({ listener, node }));
            // For each ignored node selector, set up a listener to collect it into the `ignoredNodes` set.
            const ignoredNodes = new Set();
            /**
             * Ignores a node
             * @param {ASTNode} node The node to ignore
             * @returns {void}
             */
            function addToIgnoredNodes(node) {
                ignoredNodes.add(node);
                ignoredNodeFirstTokens.add(sourceCode.getFirstToken(node));
            }
            const ignoredNodeListeners = options.ignoredNodes.reduce((listeners, ignoredSelector) => Object.assign(listeners, { [ignoredSelector]: addToIgnoredNodes }), {});
            /*
             * Join the listeners, and add a listener to verify that all tokens actually have the correct indentation
             * at the end.
             *
             * Using Object.assign will cause some offset listeners to be overwritten if the same selector also appears
             * in `ignoredNodeListeners`. This isn't a problem because all of the matching nodes will be ignored,
             * so those listeners wouldn't be called anyway.
             */
            return Object.assign(offsetListeners, ignoredNodeListeners, {
                "*:exit"(node) {
                    // If a node's type is nonstandard, we can't tell how its children should be offset, so ignore it.
                    if (!KNOWN_NODES.has(node.type)) {
                        addToIgnoredNodes(node);
                    }
                },
                "Program:exit"() {
                    // If ignoreComments option is enabled, ignore all comment tokens.
                    if (options.ignoreComments) {
                        sourceCode.getAllComments()
                            .forEach(comment => offsets.ignoreToken(comment));
                    }
                    // Invoke the queued offset listeners for the nodes that aren't ignored.
                    listenerCallQueue
                        .filter(nodeInfo => !ignoredNodes.has(nodeInfo.node))
                        .forEach(nodeInfo => nodeInfo.listener(nodeInfo.node));
                    // Update the offsets for ignored nodes to prevent their child tokens from being reported.
                    ignoredNodes.forEach(ignoreNode);
                    addParensIndent(sourceCode.ast.tokens);
                    /*
                     * Create a Map from (tokenOrComment) => (precedingToken).
                     * This is necessary because sourceCode.getTokenBefore does not handle a comment as an argument correctly.
                     */
                    const precedingTokens = sourceCode.ast.comments.reduce((commentMap, comment) => {
                        const tokenOrCommentBefore = sourceCode.getTokenBefore(comment, { includeComments: true });
                        return commentMap.set(comment, commentMap.has(tokenOrCommentBefore) ? commentMap.get(tokenOrCommentBefore) : tokenOrCommentBefore);
                    }, new WeakMap());
                    sourceCode.lines.forEach((line, lineIndex) => {
                        const lineNumber = lineIndex + 1;
                        if (!tokenInfo.firstTokensByLineNumber.has(lineNumber)) {
                            // Don't check indentation on blank lines
                            return;
                        }
                        const firstTokenOfLine = tokenInfo.firstTokensByLineNumber.get(lineNumber);
                        if (firstTokenOfLine.loc.start.line !== lineNumber) {
                            // Don't check the indentation of multi-line tokens (e.g. template literals or block comments) twice.
                            return;
                        }
                        if (astUtils.isCommentToken(firstTokenOfLine)) {
                            const tokenBefore = precedingTokens.get(firstTokenOfLine);
                            const tokenAfter = tokenBefore ? sourceCode.getTokenAfter(tokenBefore) : sourceCode.ast.tokens[0];
                            const mayAlignWithBefore = tokenBefore && !hasBlankLinesBetween(tokenBefore, firstTokenOfLine);
                            const mayAlignWithAfter = tokenAfter && !hasBlankLinesBetween(firstTokenOfLine, tokenAfter);
                            /*
                             * If a comment precedes a line that begins with a semicolon token, align to that token, i.e.
                             *
                             * let foo
                             * // comment
                             * ;(async () => {})()
                             */
                            if (tokenAfter && astUtils.isSemicolonToken(tokenAfter) && !astUtils.isTokenOnSameLine(firstTokenOfLine, tokenAfter)) {
                                offsets.setDesiredOffset(firstTokenOfLine, tokenAfter, 0);
                            }
                            // If a comment matches the expected indentation of the token immediately before or after, don't report it.
                            if (mayAlignWithBefore && validateTokenIndent(firstTokenOfLine, offsets.getDesiredIndent(tokenBefore)) ||
                                mayAlignWithAfter && validateTokenIndent(firstTokenOfLine, offsets.getDesiredIndent(tokenAfter))) {
                                return;
                            }
                        }
                        // If the token matches the expected indentation, don't report it.
                        if (validateTokenIndent(firstTokenOfLine, offsets.getDesiredIndent(firstTokenOfLine))) {
                            return;
                        }
                        // Otherwise, report the token/comment.
                        report(firstTokenOfLine, offsets.getDesiredIndent(firstTokenOfLine));
                    });
                }
            });
        }
    };
},
/* functional-red-black-tree/rbtree.js */ function _(require, module, exports) {
    module.exports = createRBTree;
    var RED = 0;
    var BLACK = 1;
    function RBNode(color, key, value, left, right, count) {
        this._color = color;
        this.key = key;
        this.value = value;
        this.left = left;
        this.right = right;
        this._count = count;
    }
    function cloneNode(node) {
        return new RBNode(node._color, node.key, node.value, node.left, node.right, node._count);
    }
    function repaint(color, node) {
        return new RBNode(color, node.key, node.value, node.left, node.right, node._count);
    }
    function recount(node) {
        node._count = 1 + (node.left ? node.left._count : 0) + (node.right ? node.right._count : 0);
    }
    function RedBlackTree(compare, root) {
        this._compare = compare;
        this.root = root;
    }
    var proto = RedBlackTree.prototype;
    Object.defineProperty(proto, "keys", {
        get: function () {
            var result = [];
            this.forEach(function (k, v) {
                result.push(k);
            });
            return result;
        }
    });
    Object.defineProperty(proto, "values", {
        get: function () {
            var result = [];
            this.forEach(function (k, v) {
                result.push(v);
            });
            return result;
        }
    });
    //Returns the number of nodes in the tree
    Object.defineProperty(proto, "length", {
        get: function () {
            if (this.root) {
                return this.root._count;
            }
            return 0;
        }
    });
    //Insert a new item into the tree
    proto.insert = function (key, value) {
        var cmp = this._compare;
        //Find point to insert new node at
        var n = this.root;
        var n_stack = [];
        var d_stack = [];
        while (n) {
            var d = cmp(key, n.key);
            n_stack.push(n);
            d_stack.push(d);
            if (d <= 0) {
                n = n.left;
            }
            else {
                n = n.right;
            }
        }
        //Rebuild path to leaf node
        n_stack.push(new RBNode(RED, key, value, null, null, 1));
        for (var s = n_stack.length - 2; s >= 0; --s) {
            var n = n_stack[s];
            if (d_stack[s] <= 0) {
                n_stack[s] = new RBNode(n._color, n.key, n.value, n_stack[s + 1], n.right, n._count + 1);
            }
            else {
                n_stack[s] = new RBNode(n._color, n.key, n.value, n.left, n_stack[s + 1], n._count + 1);
            }
        }
        //Rebalance tree using rotations
        //console.log("start insert", key, d_stack)
        for (var s = n_stack.length - 1; s > 1; --s) {
            var p = n_stack[s - 1];
            var n = n_stack[s];
            if (p._color === BLACK || n._color === BLACK) {
                break;
            }
            var pp = n_stack[s - 2];
            if (pp.left === p) {
                if (p.left === n) {
                    var y = pp.right;
                    if (y && y._color === RED) {
                        //console.log("LLr")
                        p._color = BLACK;
                        pp.right = repaint(BLACK, y);
                        pp._color = RED;
                        s -= 1;
                    }
                    else {
                        //console.log("LLb")
                        pp._color = RED;
                        pp.left = p.right;
                        p._color = BLACK;
                        p.right = pp;
                        n_stack[s - 2] = p;
                        n_stack[s - 1] = n;
                        recount(pp);
                        recount(p);
                        if (s >= 3) {
                            var ppp = n_stack[s - 3];
                            if (ppp.left === pp) {
                                ppp.left = p;
                            }
                            else {
                                ppp.right = p;
                            }
                        }
                        break;
                    }
                }
                else {
                    var y = pp.right;
                    if (y && y._color === RED) {
                        //console.log("LRr")
                        p._color = BLACK;
                        pp.right = repaint(BLACK, y);
                        pp._color = RED;
                        s -= 1;
                    }
                    else {
                        //console.log("LRb")
                        p.right = n.left;
                        pp._color = RED;
                        pp.left = n.right;
                        n._color = BLACK;
                        n.left = p;
                        n.right = pp;
                        n_stack[s - 2] = n;
                        n_stack[s - 1] = p;
                        recount(pp);
                        recount(p);
                        recount(n);
                        if (s >= 3) {
                            var ppp = n_stack[s - 3];
                            if (ppp.left === pp) {
                                ppp.left = n;
                            }
                            else {
                                ppp.right = n;
                            }
                        }
                        break;
                    }
                }
            }
            else {
                if (p.right === n) {
                    var y = pp.left;
                    if (y && y._color === RED) {
                        //console.log("RRr", y.key)
                        p._color = BLACK;
                        pp.left = repaint(BLACK, y);
                        pp._color = RED;
                        s -= 1;
                    }
                    else {
                        //console.log("RRb")
                        pp._color = RED;
                        pp.right = p.left;
                        p._color = BLACK;
                        p.left = pp;
                        n_stack[s - 2] = p;
                        n_stack[s - 1] = n;
                        recount(pp);
                        recount(p);
                        if (s >= 3) {
                            var ppp = n_stack[s - 3];
                            if (ppp.right === pp) {
                                ppp.right = p;
                            }
                            else {
                                ppp.left = p;
                            }
                        }
                        break;
                    }
                }
                else {
                    var y = pp.left;
                    if (y && y._color === RED) {
                        //console.log("RLr")
                        p._color = BLACK;
                        pp.left = repaint(BLACK, y);
                        pp._color = RED;
                        s -= 1;
                    }
                    else {
                        //console.log("RLb")
                        p.left = n.right;
                        pp._color = RED;
                        pp.right = n.left;
                        n._color = BLACK;
                        n.right = p;
                        n.left = pp;
                        n_stack[s - 2] = n;
                        n_stack[s - 1] = p;
                        recount(pp);
                        recount(p);
                        recount(n);
                        if (s >= 3) {
                            var ppp = n_stack[s - 3];
                            if (ppp.right === pp) {
                                ppp.right = n;
                            }
                            else {
                                ppp.left = n;
                            }
                        }
                        break;
                    }
                }
            }
        }
        //Return new tree
        n_stack[0]._color = BLACK;
        return new RedBlackTree(cmp, n_stack[0]);
    };
    //Visit all nodes inorder
    function doVisitFull(visit, node) {
        if (node.left) {
            var v = doVisitFull(visit, node.left);
            if (v) {
                return v;
            }
        }
        var v = visit(node.key, node.value);
        if (v) {
            return v;
        }
        if (node.right) {
            return doVisitFull(visit, node.right);
        }
    }
    //Visit half nodes in order
    function doVisitHalf(lo, compare, visit, node) {
        var l = compare(lo, node.key);
        if (l <= 0) {
            if (node.left) {
                var v = doVisitHalf(lo, compare, visit, node.left);
                if (v) {
                    return v;
                }
            }
            var v = visit(node.key, node.value);
            if (v) {
                return v;
            }
        }
        if (node.right) {
            return doVisitHalf(lo, compare, visit, node.right);
        }
    }
    //Visit all nodes within a range
    function doVisit(lo, hi, compare, visit, node) {
        var l = compare(lo, node.key);
        var h = compare(hi, node.key);
        var v;
        if (l <= 0) {
            if (node.left) {
                v = doVisit(lo, hi, compare, visit, node.left);
                if (v) {
                    return v;
                }
            }
            if (h > 0) {
                v = visit(node.key, node.value);
                if (v) {
                    return v;
                }
            }
        }
        if (h > 0 && node.right) {
            return doVisit(lo, hi, compare, visit, node.right);
        }
    }
    proto.forEach = function rbTreeForEach(visit, lo, hi) {
        if (!this.root) {
            return;
        }
        switch (arguments.length) {
            case 1:
                return doVisitFull(visit, this.root);
                break;
            case 2:
                return doVisitHalf(lo, this._compare, visit, this.root);
                break;
            case 3:
                if (this._compare(lo, hi) >= 0) {
                    return;
                }
                return doVisit(lo, hi, this._compare, visit, this.root);
                break;
        }
    };
    //First item in list
    Object.defineProperty(proto, "begin", {
        get: function () {
            var stack = [];
            var n = this.root;
            while (n) {
                stack.push(n);
                n = n.left;
            }
            return new RedBlackTreeIterator(this, stack);
        }
    });
    //Last item in list
    Object.defineProperty(proto, "end", {
        get: function () {
            var stack = [];
            var n = this.root;
            while (n) {
                stack.push(n);
                n = n.right;
            }
            return new RedBlackTreeIterator(this, stack);
        }
    });
    //Find the ith item in the tree
    proto.at = function (idx) {
        if (idx < 0) {
            return new RedBlackTreeIterator(this, []);
        }
        var n = this.root;
        var stack = [];
        while (true) {
            stack.push(n);
            if (n.left) {
                if (idx < n.left._count) {
                    n = n.left;
                    continue;
                }
                idx -= n.left._count;
            }
            if (!idx) {
                return new RedBlackTreeIterator(this, stack);
            }
            idx -= 1;
            if (n.right) {
                if (idx >= n.right._count) {
                    break;
                }
                n = n.right;
            }
            else {
                break;
            }
        }
        return new RedBlackTreeIterator(this, []);
    };
    proto.ge = function (key) {
        var cmp = this._compare;
        var n = this.root;
        var stack = [];
        var last_ptr = 0;
        while (n) {
            var d = cmp(key, n.key);
            stack.push(n);
            if (d <= 0) {
                last_ptr = stack.length;
            }
            if (d <= 0) {
                n = n.left;
            }
            else {
                n = n.right;
            }
        }
        stack.length = last_ptr;
        return new RedBlackTreeIterator(this, stack);
    };
    proto.gt = function (key) {
        var cmp = this._compare;
        var n = this.root;
        var stack = [];
        var last_ptr = 0;
        while (n) {
            var d = cmp(key, n.key);
            stack.push(n);
            if (d < 0) {
                last_ptr = stack.length;
            }
            if (d < 0) {
                n = n.left;
            }
            else {
                n = n.right;
            }
        }
        stack.length = last_ptr;
        return new RedBlackTreeIterator(this, stack);
    };
    proto.lt = function (key) {
        var cmp = this._compare;
        var n = this.root;
        var stack = [];
        var last_ptr = 0;
        while (n) {
            var d = cmp(key, n.key);
            stack.push(n);
            if (d > 0) {
                last_ptr = stack.length;
            }
            if (d <= 0) {
                n = n.left;
            }
            else {
                n = n.right;
            }
        }
        stack.length = last_ptr;
        return new RedBlackTreeIterator(this, stack);
    };
    proto.le = function (key) {
        var cmp = this._compare;
        var n = this.root;
        var stack = [];
        var last_ptr = 0;
        while (n) {
            var d = cmp(key, n.key);
            stack.push(n);
            if (d >= 0) {
                last_ptr = stack.length;
            }
            if (d < 0) {
                n = n.left;
            }
            else {
                n = n.right;
            }
        }
        stack.length = last_ptr;
        return new RedBlackTreeIterator(this, stack);
    };
    //Finds the item with key if it exists
    proto.find = function (key) {
        var cmp = this._compare;
        var n = this.root;
        var stack = [];
        while (n) {
            var d = cmp(key, n.key);
            stack.push(n);
            if (d === 0) {
                return new RedBlackTreeIterator(this, stack);
            }
            if (d <= 0) {
                n = n.left;
            }
            else {
                n = n.right;
            }
        }
        return new RedBlackTreeIterator(this, []);
    };
    //Removes item with key from tree
    proto.remove = function (key) {
        var iter = this.find(key);
        if (iter) {
            return iter.remove();
        }
        return this;
    };
    //Returns the item at `key`
    proto.get = function (key) {
        var cmp = this._compare;
        var n = this.root;
        while (n) {
            var d = cmp(key, n.key);
            if (d === 0) {
                return n.value;
            }
            if (d <= 0) {
                n = n.left;
            }
            else {
                n = n.right;
            }
        }
        return;
    };
    //Iterator for red black tree
    function RedBlackTreeIterator(tree, stack) {
        this.tree = tree;
        this._stack = stack;
    }
    var iproto = RedBlackTreeIterator.prototype;
    //Test if iterator is valid
    Object.defineProperty(iproto, "valid", {
        get: function () {
            return this._stack.length > 0;
        }
    });
    //Node of the iterator
    Object.defineProperty(iproto, "node", {
        get: function () {
            if (this._stack.length > 0) {
                return this._stack[this._stack.length - 1];
            }
            return null;
        },
        enumerable: true
    });
    //Makes a copy of an iterator
    iproto.clone = function () {
        return new RedBlackTreeIterator(this.tree, this._stack.slice());
    };
    //Swaps two nodes
    function swapNode(n, v) {
        n.key = v.key;
        n.value = v.value;
        n.left = v.left;
        n.right = v.right;
        n._color = v._color;
        n._count = v._count;
    }
    //Fix up a double black node in a tree
    function fixDoubleBlack(stack) {
        var n, p, s, z;
        for (var i = stack.length - 1; i >= 0; --i) {
            n = stack[i];
            if (i === 0) {
                n._color = BLACK;
                return;
            }
            //console.log("visit node:", n.key, i, stack[i].key, stack[i-1].key)
            p = stack[i - 1];
            if (p.left === n) {
                //console.log("left child")
                s = p.right;
                if (s.right && s.right._color === RED) {
                    //console.log("case 1: right sibling child red")
                    s = p.right = cloneNode(s);
                    z = s.right = cloneNode(s.right);
                    p.right = s.left;
                    s.left = p;
                    s.right = z;
                    s._color = p._color;
                    n._color = BLACK;
                    p._color = BLACK;
                    z._color = BLACK;
                    recount(p);
                    recount(s);
                    if (i > 1) {
                        var pp = stack[i - 2];
                        if (pp.left === p) {
                            pp.left = s;
                        }
                        else {
                            pp.right = s;
                        }
                    }
                    stack[i - 1] = s;
                    return;
                }
                else if (s.left && s.left._color === RED) {
                    //console.log("case 1: left sibling child red")
                    s = p.right = cloneNode(s);
                    z = s.left = cloneNode(s.left);
                    p.right = z.left;
                    s.left = z.right;
                    z.left = p;
                    z.right = s;
                    z._color = p._color;
                    p._color = BLACK;
                    s._color = BLACK;
                    n._color = BLACK;
                    recount(p);
                    recount(s);
                    recount(z);
                    if (i > 1) {
                        var pp = stack[i - 2];
                        if (pp.left === p) {
                            pp.left = z;
                        }
                        else {
                            pp.right = z;
                        }
                    }
                    stack[i - 1] = z;
                    return;
                }
                if (s._color === BLACK) {
                    if (p._color === RED) {
                        //console.log("case 2: black sibling, red parent", p.right.value)
                        p._color = BLACK;
                        p.right = repaint(RED, s);
                        return;
                    }
                    else {
                        //console.log("case 2: black sibling, black parent", p.right.value)
                        p.right = repaint(RED, s);
                        continue;
                    }
                }
                else {
                    //console.log("case 3: red sibling")
                    s = cloneNode(s);
                    p.right = s.left;
                    s.left = p;
                    s._color = p._color;
                    p._color = RED;
                    recount(p);
                    recount(s);
                    if (i > 1) {
                        var pp = stack[i - 2];
                        if (pp.left === p) {
                            pp.left = s;
                        }
                        else {
                            pp.right = s;
                        }
                    }
                    stack[i - 1] = s;
                    stack[i] = p;
                    if (i + 1 < stack.length) {
                        stack[i + 1] = n;
                    }
                    else {
                        stack.push(n);
                    }
                    i = i + 2;
                }
            }
            else {
                //console.log("right child")
                s = p.left;
                if (s.left && s.left._color === RED) {
                    //console.log("case 1: left sibling child red", p.value, p._color)
                    s = p.left = cloneNode(s);
                    z = s.left = cloneNode(s.left);
                    p.left = s.right;
                    s.right = p;
                    s.left = z;
                    s._color = p._color;
                    n._color = BLACK;
                    p._color = BLACK;
                    z._color = BLACK;
                    recount(p);
                    recount(s);
                    if (i > 1) {
                        var pp = stack[i - 2];
                        if (pp.right === p) {
                            pp.right = s;
                        }
                        else {
                            pp.left = s;
                        }
                    }
                    stack[i - 1] = s;
                    return;
                }
                else if (s.right && s.right._color === RED) {
                    //console.log("case 1: right sibling child red")
                    s = p.left = cloneNode(s);
                    z = s.right = cloneNode(s.right);
                    p.left = z.right;
                    s.right = z.left;
                    z.right = p;
                    z.left = s;
                    z._color = p._color;
                    p._color = BLACK;
                    s._color = BLACK;
                    n._color = BLACK;
                    recount(p);
                    recount(s);
                    recount(z);
                    if (i > 1) {
                        var pp = stack[i - 2];
                        if (pp.right === p) {
                            pp.right = z;
                        }
                        else {
                            pp.left = z;
                        }
                    }
                    stack[i - 1] = z;
                    return;
                }
                if (s._color === BLACK) {
                    if (p._color === RED) {
                        //console.log("case 2: black sibling, red parent")
                        p._color = BLACK;
                        p.left = repaint(RED, s);
                        return;
                    }
                    else {
                        //console.log("case 2: black sibling, black parent")
                        p.left = repaint(RED, s);
                        continue;
                    }
                }
                else {
                    //console.log("case 3: red sibling")
                    s = cloneNode(s);
                    p.left = s.right;
                    s.right = p;
                    s._color = p._color;
                    p._color = RED;
                    recount(p);
                    recount(s);
                    if (i > 1) {
                        var pp = stack[i - 2];
                        if (pp.right === p) {
                            pp.right = s;
                        }
                        else {
                            pp.left = s;
                        }
                    }
                    stack[i - 1] = s;
                    stack[i] = p;
                    if (i + 1 < stack.length) {
                        stack[i + 1] = n;
                    }
                    else {
                        stack.push(n);
                    }
                    i = i + 2;
                }
            }
        }
    }
    //Removes item at iterator from tree
    iproto.remove = function () {
        var stack = this._stack;
        if (stack.length === 0) {
            return this.tree;
        }
        //First copy path to node
        var cstack = new Array(stack.length);
        var n = stack[stack.length - 1];
        cstack[cstack.length - 1] = new RBNode(n._color, n.key, n.value, n.left, n.right, n._count);
        for (var i = stack.length - 2; i >= 0; --i) {
            var n = stack[i];
            if (n.left === stack[i + 1]) {
                cstack[i] = new RBNode(n._color, n.key, n.value, cstack[i + 1], n.right, n._count);
            }
            else {
                cstack[i] = new RBNode(n._color, n.key, n.value, n.left, cstack[i + 1], n._count);
            }
        }
        //Get node
        n = cstack[cstack.length - 1];
        //console.log("start remove: ", n.value)
        //If not leaf, then swap with previous node
        if (n.left && n.right) {
            //console.log("moving to leaf")
            //First walk to previous leaf
            var split = cstack.length;
            n = n.left;
            while (n.right) {
                cstack.push(n);
                n = n.right;
            }
            //Copy path to leaf
            var v = cstack[split - 1];
            cstack.push(new RBNode(n._color, v.key, v.value, n.left, n.right, n._count));
            cstack[split - 1].key = n.key;
            cstack[split - 1].value = n.value;
            //Fix up stack
            for (var i = cstack.length - 2; i >= split; --i) {
                n = cstack[i];
                cstack[i] = new RBNode(n._color, n.key, n.value, n.left, cstack[i + 1], n._count);
            }
            cstack[split - 1].left = cstack[split];
        }
        //console.log("stack=", cstack.map(function(v) { return v.value }))
        //Remove leaf node
        n = cstack[cstack.length - 1];
        if (n._color === RED) {
            //Easy case: removing red leaf
            //console.log("RED leaf")
            var p = cstack[cstack.length - 2];
            if (p.left === n) {
                p.left = null;
            }
            else if (p.right === n) {
                p.right = null;
            }
            cstack.pop();
            for (var i = 0; i < cstack.length; ++i) {
                cstack[i]._count--;
            }
            return new RedBlackTree(this.tree._compare, cstack[0]);
        }
        else {
            if (n.left || n.right) {
                //Second easy case:  Single child black parent
                //console.log("BLACK single child")
                if (n.left) {
                    swapNode(n, n.left);
                }
                else if (n.right) {
                    swapNode(n, n.right);
                }
                //Child must be red, so repaint it black to balance color
                n._color = BLACK;
                for (var i = 0; i < cstack.length - 1; ++i) {
                    cstack[i]._count--;
                }
                return new RedBlackTree(this.tree._compare, cstack[0]);
            }
            else if (cstack.length === 1) {
                //Third easy case: root
                //console.log("ROOT")
                return new RedBlackTree(this.tree._compare, null);
            }
            else {
                //Hard case: Repaint n, and then do some nasty stuff
                //console.log("BLACK leaf no children")
                for (var i = 0; i < cstack.length; ++i) {
                    cstack[i]._count--;
                }
                var parent = cstack[cstack.length - 2];
                fixDoubleBlack(cstack);
                //Fix up links
                if (parent.left === n) {
                    parent.left = null;
                }
                else {
                    parent.right = null;
                }
            }
        }
        return new RedBlackTree(this.tree._compare, cstack[0]);
    };
    //Returns key
    Object.defineProperty(iproto, "key", {
        get: function () {
            if (this._stack.length > 0) {
                return this._stack[this._stack.length - 1].key;
            }
            return;
        },
        enumerable: true
    });
    //Returns value
    Object.defineProperty(iproto, "value", {
        get: function () {
            if (this._stack.length > 0) {
                return this._stack[this._stack.length - 1].value;
            }
            return;
        },
        enumerable: true
    });
    //Returns the position of this iterator in the sorted list
    Object.defineProperty(iproto, "index", {
        get: function () {
            var idx = 0;
            var stack = this._stack;
            if (stack.length === 0) {
                var r = this.tree.root;
                if (r) {
                    return r._count;
                }
                return 0;
            }
            else if (stack[stack.length - 1].left) {
                idx = stack[stack.length - 1].left._count;
            }
            for (var s = stack.length - 2; s >= 0; --s) {
                if (stack[s + 1] === stack[s].right) {
                    ++idx;
                    if (stack[s].left) {
                        idx += stack[s].left._count;
                    }
                }
            }
            return idx;
        },
        enumerable: true
    });
    //Advances iterator to next element in list
    iproto.next = function () {
        var stack = this._stack;
        if (stack.length === 0) {
            return;
        }
        var n = stack[stack.length - 1];
        if (n.right) {
            n = n.right;
            while (n) {
                stack.push(n);
                n = n.left;
            }
        }
        else {
            stack.pop();
            while (stack.length > 0 && stack[stack.length - 1].right === n) {
                n = stack[stack.length - 1];
                stack.pop();
            }
        }
    };
    //Checks if iterator is at end of tree
    Object.defineProperty(iproto, "hasNext", {
        get: function () {
            var stack = this._stack;
            if (stack.length === 0) {
                return false;
            }
            if (stack[stack.length - 1].right) {
                return true;
            }
            for (var s = stack.length - 1; s > 0; --s) {
                if (stack[s - 1].left === stack[s]) {
                    return true;
                }
            }
            return false;
        }
    });
    //Update value
    iproto.update = function (value) {
        var stack = this._stack;
        if (stack.length === 0) {
            throw new Error("Can't update empty node!");
        }
        var cstack = new Array(stack.length);
        var n = stack[stack.length - 1];
        cstack[cstack.length - 1] = new RBNode(n._color, n.key, value, n.left, n.right, n._count);
        for (var i = stack.length - 2; i >= 0; --i) {
            n = stack[i];
            if (n.left === stack[i + 1]) {
                cstack[i] = new RBNode(n._color, n.key, n.value, cstack[i + 1], n.right, n._count);
            }
            else {
                cstack[i] = new RBNode(n._color, n.key, n.value, n.left, cstack[i + 1], n._count);
            }
        }
        return new RedBlackTree(this.tree._compare, cstack[0]);
    };
    //Moves iterator backward one element
    iproto.prev = function () {
        var stack = this._stack;
        if (stack.length === 0) {
            return;
        }
        var n = stack[stack.length - 1];
        if (n.left) {
            n = n.left;
            while (n) {
                stack.push(n);
                n = n.right;
            }
        }
        else {
            stack.pop();
            while (stack.length > 0 && stack[stack.length - 1].left === n) {
                n = stack[stack.length - 1];
                stack.pop();
            }
        }
    };
    //Checks if iterator is at start of tree
    Object.defineProperty(iproto, "hasPrev", {
        get: function () {
            var stack = this._stack;
            if (stack.length === 0) {
                return false;
            }
            if (stack[stack.length - 1].left) {
                return true;
            }
            for (var s = stack.length - 1; s > 0; --s) {
                if (stack[s - 1].right === stack[s]) {
                    return true;
                }
            }
            return false;
        }
    });
    //Default comparison function
    function defaultCompare(a, b) {
        if (a < b) {
            return -1;
        }
        if (a > b) {
            return 1;
        }
        return 0;
    }
    //Build a tree
    function createRBTree(compare) {
        return new RedBlackTree(compare || defaultCompare, null);
    }
},
/* eslint/lib/rules/indent-legacy.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    /* istanbul ignore next: this rule has known coverage issues, but it's deprecated and shouldn't be updated in the future anyway. */
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent indentation",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/indent-legacy"
            },
            deprecated: true,
            replacedBy: ["indent"],
            fixable: "whitespace",
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["tab"]
                        },
                        {
                            type: "integer",
                            minimum: 0
                        }
                    ]
                },
                {
                    type: "object",
                    properties: {
                        SwitchCase: {
                            type: "integer",
                            minimum: 0
                        },
                        VariableDeclarator: {
                            oneOf: [
                                {
                                    type: "integer",
                                    minimum: 0
                                },
                                {
                                    type: "object",
                                    properties: {
                                        var: {
                                            type: "integer",
                                            minimum: 0
                                        },
                                        let: {
                                            type: "integer",
                                            minimum: 0
                                        },
                                        const: {
                                            type: "integer",
                                            minimum: 0
                                        }
                                    }
                                }
                            ]
                        },
                        outerIIFEBody: {
                            type: "integer",
                            minimum: 0
                        },
                        MemberExpression: {
                            type: "integer",
                            minimum: 0
                        },
                        FunctionDeclaration: {
                            type: "object",
                            properties: {
                                parameters: {
                                    oneOf: [
                                        {
                                            type: "integer",
                                            minimum: 0
                                        },
                                        {
                                            enum: ["first"]
                                        }
                                    ]
                                },
                                body: {
                                    type: "integer",
                                    minimum: 0
                                }
                            }
                        },
                        FunctionExpression: {
                            type: "object",
                            properties: {
                                parameters: {
                                    oneOf: [
                                        {
                                            type: "integer",
                                            minimum: 0
                                        },
                                        {
                                            enum: ["first"]
                                        }
                                    ]
                                },
                                body: {
                                    type: "integer",
                                    minimum: 0
                                }
                            }
                        },
                        CallExpression: {
                            type: "object",
                            properties: {
                                parameters: {
                                    oneOf: [
                                        {
                                            type: "integer",
                                            minimum: 0
                                        },
                                        {
                                            enum: ["first"]
                                        }
                                    ]
                                }
                            }
                        },
                        ArrayExpression: {
                            oneOf: [
                                {
                                    type: "integer",
                                    minimum: 0
                                },
                                {
                                    enum: ["first"]
                                }
                            ]
                        },
                        ObjectExpression: {
                            oneOf: [
                                {
                                    type: "integer",
                                    minimum: 0
                                },
                                {
                                    enum: ["first"]
                                }
                            ]
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                expected: "Expected indentation of {{expected}} but found {{actual}}."
            }
        },
        create(context) {
            const DEFAULT_VARIABLE_INDENT = 1;
            const DEFAULT_PARAMETER_INDENT = null; // For backwards compatibility, don't check parameter indentation unless specified in the config
            const DEFAULT_FUNCTION_BODY_INDENT = 1;
            let indentType = "space";
            let indentSize = 4;
            const options = {
                SwitchCase: 0,
                VariableDeclarator: {
                    var: DEFAULT_VARIABLE_INDENT,
                    let: DEFAULT_VARIABLE_INDENT,
                    const: DEFAULT_VARIABLE_INDENT
                },
                outerIIFEBody: null,
                FunctionDeclaration: {
                    parameters: DEFAULT_PARAMETER_INDENT,
                    body: DEFAULT_FUNCTION_BODY_INDENT
                },
                FunctionExpression: {
                    parameters: DEFAULT_PARAMETER_INDENT,
                    body: DEFAULT_FUNCTION_BODY_INDENT
                },
                CallExpression: {
                    arguments: DEFAULT_PARAMETER_INDENT
                },
                ArrayExpression: 1,
                ObjectExpression: 1
            };
            const sourceCode = context.getSourceCode();
            if (context.options.length) {
                if (context.options[0] === "tab") {
                    indentSize = 1;
                    indentType = "tab";
                }
                else /* istanbul ignore else : this will be caught by options validation */ if (typeof context.options[0] === "number") {
                    indentSize = context.options[0];
                    indentType = "space";
                }
                if (context.options[1]) {
                    const opts = context.options[1];
                    options.SwitchCase = opts.SwitchCase || 0;
                    const variableDeclaratorRules = opts.VariableDeclarator;
                    if (typeof variableDeclaratorRules === "number") {
                        options.VariableDeclarator = {
                            var: variableDeclaratorRules,
                            let: variableDeclaratorRules,
                            const: variableDeclaratorRules
                        };
                    }
                    else if (typeof variableDeclaratorRules === "object") {
                        Object.assign(options.VariableDeclarator, variableDeclaratorRules);
                    }
                    if (typeof opts.outerIIFEBody === "number") {
                        options.outerIIFEBody = opts.outerIIFEBody;
                    }
                    if (typeof opts.MemberExpression === "number") {
                        options.MemberExpression = opts.MemberExpression;
                    }
                    if (typeof opts.FunctionDeclaration === "object") {
                        Object.assign(options.FunctionDeclaration, opts.FunctionDeclaration);
                    }
                    if (typeof opts.FunctionExpression === "object") {
                        Object.assign(options.FunctionExpression, opts.FunctionExpression);
                    }
                    if (typeof opts.CallExpression === "object") {
                        Object.assign(options.CallExpression, opts.CallExpression);
                    }
                    if (typeof opts.ArrayExpression === "number" || typeof opts.ArrayExpression === "string") {
                        options.ArrayExpression = opts.ArrayExpression;
                    }
                    if (typeof opts.ObjectExpression === "number" || typeof opts.ObjectExpression === "string") {
                        options.ObjectExpression = opts.ObjectExpression;
                    }
                }
            }
            const caseIndentStore = {};
            /**
             * Creates an error message for a line, given the expected/actual indentation.
             * @param {int} expectedAmount The expected amount of indentation characters for this line
             * @param {int} actualSpaces The actual number of indentation spaces that were found on this line
             * @param {int} actualTabs The actual number of indentation tabs that were found on this line
             * @returns {string} An error message for this line
             */
            function createErrorMessageData(expectedAmount, actualSpaces, actualTabs) {
                const expectedStatement = `${expectedAmount} ${indentType}${expectedAmount === 1 ? "" : "s"}`; // e.g. "2 tabs"
                const foundSpacesWord = `space${actualSpaces === 1 ? "" : "s"}`; // e.g. "space"
                const foundTabsWord = `tab${actualTabs === 1 ? "" : "s"}`; // e.g. "tabs"
                let foundStatement;
                if (actualSpaces > 0 && actualTabs > 0) {
                    foundStatement = `${actualSpaces} ${foundSpacesWord} and ${actualTabs} ${foundTabsWord}`; // e.g. "1 space and 2 tabs"
                }
                else if (actualSpaces > 0) {
                    /*
                     * Abbreviate the message if the expected indentation is also spaces.
                     * e.g. 'Expected 4 spaces but found 2' rather than 'Expected 4 spaces but found 2 spaces'
                     */
                    foundStatement = indentType === "space" ? actualSpaces : `${actualSpaces} ${foundSpacesWord}`;
                }
                else if (actualTabs > 0) {
                    foundStatement = indentType === "tab" ? actualTabs : `${actualTabs} ${foundTabsWord}`;
                }
                else {
                    foundStatement = "0";
                }
                return {
                    expected: expectedStatement,
                    actual: foundStatement
                };
            }
            /**
             * Reports a given indent violation
             * @param {ASTNode} node Node violating the indent rule
             * @param {int} needed Expected indentation character count
             * @param {int} gottenSpaces Indentation space count in the actual node/code
             * @param {int} gottenTabs Indentation tab count in the actual node/code
             * @param {Object} [loc] Error line and column location
             * @param {boolean} isLastNodeCheck Is the error for last node check
             * @returns {void}
             */
            function report(node, needed, gottenSpaces, gottenTabs, loc, isLastNodeCheck) {
                if (gottenSpaces && gottenTabs) {
                    // To avoid conflicts with `no-mixed-spaces-and-tabs`, don't report lines that have both spaces and tabs.
                    return;
                }
                const desiredIndent = (indentType === "space" ? " " : "\t").repeat(needed);
                const textRange = isLastNodeCheck
                    ? [node.range[1] - node.loc.end.column, node.range[1] - node.loc.end.column + gottenSpaces + gottenTabs]
                    : [node.range[0] - node.loc.start.column, node.range[0] - node.loc.start.column + gottenSpaces + gottenTabs];
                context.report({
                    node,
                    loc,
                    messageId: "expected",
                    data: createErrorMessageData(needed, gottenSpaces, gottenTabs),
                    fix: fixer => fixer.replaceTextRange(textRange, desiredIndent)
                });
            }
            /**
             * Get the actual indent of node
             * @param {ASTNode|Token} node Node to examine
             * @param {boolean} [byLastLine=false] get indent of node's last line
             * @returns {Object} The node's indent. Contains keys `space` and `tab`, representing the indent of each character. Also
             * contains keys `goodChar` and `badChar`, where `goodChar` is the amount of the user's desired indentation character, and
             * `badChar` is the amount of the other indentation character.
             */
            function getNodeIndent(node, byLastLine) {
                const token = byLastLine ? sourceCode.getLastToken(node) : sourceCode.getFirstToken(node);
                const srcCharsBeforeNode = sourceCode.getText(token, token.loc.start.column).split("");
                const indentChars = srcCharsBeforeNode.slice(0, srcCharsBeforeNode.findIndex(char => char !== " " && char !== "\t"));
                const spaces = indentChars.filter(char => char === " ").length;
                const tabs = indentChars.filter(char => char === "\t").length;
                return {
                    space: spaces,
                    tab: tabs,
                    goodChar: indentType === "space" ? spaces : tabs,
                    badChar: indentType === "space" ? tabs : spaces
                };
            }
            /**
             * Checks node is the first in its own start line. By default it looks by start line.
             * @param {ASTNode} node The node to check
             * @param {boolean} [byEndLocation=false] Lookup based on start position or end
             * @returns {boolean} true if its the first in the its start line
             */
            function isNodeFirstInLine(node, byEndLocation) {
                const firstToken = byEndLocation === true ? sourceCode.getLastToken(node, 1) : sourceCode.getTokenBefore(node), startLine = byEndLocation === true ? node.loc.end.line : node.loc.start.line, endLine = firstToken ? firstToken.loc.end.line : -1;
                return startLine !== endLine;
            }
            /**
             * Check indent for node
             * @param {ASTNode} node Node to check
             * @param {int} neededIndent needed indent
             * @returns {void}
             */
            function checkNodeIndent(node, neededIndent) {
                const actualIndent = getNodeIndent(node, false);
                if (node.type !== "ArrayExpression" &&
                    node.type !== "ObjectExpression" &&
                    (actualIndent.goodChar !== neededIndent || actualIndent.badChar !== 0) &&
                    isNodeFirstInLine(node)) {
                    report(node, neededIndent, actualIndent.space, actualIndent.tab);
                }
                if (node.type === "IfStatement" && node.alternate) {
                    const elseToken = sourceCode.getTokenBefore(node.alternate);
                    checkNodeIndent(elseToken, neededIndent);
                    if (!isNodeFirstInLine(node.alternate)) {
                        checkNodeIndent(node.alternate, neededIndent);
                    }
                }
                if (node.type === "TryStatement" && node.handler) {
                    const catchToken = sourceCode.getFirstToken(node.handler);
                    checkNodeIndent(catchToken, neededIndent);
                }
                if (node.type === "TryStatement" && node.finalizer) {
                    const finallyToken = sourceCode.getTokenBefore(node.finalizer);
                    checkNodeIndent(finallyToken, neededIndent);
                }
                if (node.type === "DoWhileStatement") {
                    const whileToken = sourceCode.getTokenAfter(node.body);
                    checkNodeIndent(whileToken, neededIndent);
                }
            }
            /**
             * Check indent for nodes list
             * @param {ASTNode[]} nodes list of node objects
             * @param {int} indent needed indent
             * @returns {void}
             */
            function checkNodesIndent(nodes, indent) {
                nodes.forEach(node => checkNodeIndent(node, indent));
            }
            /**
             * Check last node line indent this detects, that block closed correctly
             * @param {ASTNode} node Node to examine
             * @param {int} lastLineIndent needed indent
             * @returns {void}
             */
            function checkLastNodeLineIndent(node, lastLineIndent) {
                const lastToken = sourceCode.getLastToken(node);
                const endIndent = getNodeIndent(lastToken, true);
                if ((endIndent.goodChar !== lastLineIndent || endIndent.badChar !== 0) && isNodeFirstInLine(node, true)) {
                    report(node, lastLineIndent, endIndent.space, endIndent.tab, { line: lastToken.loc.start.line, column: lastToken.loc.start.column }, true);
                }
            }
            /**
             * Check last node line indent this detects, that block closed correctly
             * This function for more complicated return statement case, where closing parenthesis may be followed by ';'
             * @param {ASTNode} node Node to examine
             * @param {int} firstLineIndent first line needed indent
             * @returns {void}
             */
            function checkLastReturnStatementLineIndent(node, firstLineIndent) {
                /*
                 * in case if return statement ends with ');' we have traverse back to ')'
                 * otherwise we'll measure indent for ';' and replace ')'
                 */
                const lastToken = sourceCode.getLastToken(node, astUtils.isClosingParenToken);
                const textBeforeClosingParenthesis = sourceCode.getText(lastToken, lastToken.loc.start.column).slice(0, -1);
                if (textBeforeClosingParenthesis.trim()) {
                    // There are tokens before the closing paren, don't report this case
                    return;
                }
                const endIndent = getNodeIndent(lastToken, true);
                if (endIndent.goodChar !== firstLineIndent) {
                    report(node, firstLineIndent, endIndent.space, endIndent.tab, { line: lastToken.loc.start.line, column: lastToken.loc.start.column }, true);
                }
            }
            /**
             * Check first node line indent is correct
             * @param {ASTNode} node Node to examine
             * @param {int} firstLineIndent needed indent
             * @returns {void}
             */
            function checkFirstNodeLineIndent(node, firstLineIndent) {
                const startIndent = getNodeIndent(node, false);
                if ((startIndent.goodChar !== firstLineIndent || startIndent.badChar !== 0) && isNodeFirstInLine(node)) {
                    report(node, firstLineIndent, startIndent.space, startIndent.tab, { line: node.loc.start.line, column: node.loc.start.column });
                }
            }
            /**
             * Returns a parent node of given node based on a specified type
             * if not present then return null
             * @param {ASTNode} node node to examine
             * @param {string} type type that is being looked for
             * @param {string} stopAtList end points for the evaluating code
             * @returns {ASTNode|void} if found then node otherwise null
             */
            function getParentNodeByType(node, type, stopAtList) {
                let parent = node.parent;
                const stopAtSet = new Set(stopAtList || ["Program"]);
                while (parent.type !== type && !stopAtSet.has(parent.type) && parent.type !== "Program") {
                    parent = parent.parent;
                }
                return parent.type === type ? parent : null;
            }
            /**
             * Returns the VariableDeclarator based on the current node
             * if not present then return null
             * @param {ASTNode} node node to examine
             * @returns {ASTNode|void} if found then node otherwise null
             */
            function getVariableDeclaratorNode(node) {
                return getParentNodeByType(node, "VariableDeclarator");
            }
            /**
             * Check to see if the node is part of the multi-line variable declaration.
             * Also if its on the same line as the varNode
             * @param {ASTNode} node node to check
             * @param {ASTNode} varNode variable declaration node to check against
             * @returns {boolean} True if all the above condition satisfy
             */
            function isNodeInVarOnTop(node, varNode) {
                return varNode &&
                    varNode.parent.loc.start.line === node.loc.start.line &&
                    varNode.parent.declarations.length > 1;
            }
            /**
             * Check to see if the argument before the callee node is multi-line and
             * there should only be 1 argument before the callee node
             * @param {ASTNode} node node to check
             * @returns {boolean} True if arguments are multi-line
             */
            function isArgBeforeCalleeNodeMultiline(node) {
                const parent = node.parent;
                if (parent.arguments.length >= 2 && parent.arguments[1] === node) {
                    return parent.arguments[0].loc.end.line > parent.arguments[0].loc.start.line;
                }
                return false;
            }
            /**
             * Check to see if the node is a file level IIFE
             * @param {ASTNode} node The function node to check.
             * @returns {boolean} True if the node is the outer IIFE
             */
            function isOuterIIFE(node) {
                const parent = node.parent;
                let stmt = parent.parent;
                /*
                 * Verify that the node is an IIEF
                 */
                if (parent.type !== "CallExpression" ||
                    parent.callee !== node) {
                    return false;
                }
                /*
                 * Navigate legal ancestors to determine whether this IIEF is outer
                 */
                while (stmt.type === "UnaryExpression" && (stmt.operator === "!" ||
                    stmt.operator === "~" ||
                    stmt.operator === "+" ||
                    stmt.operator === "-") ||
                    stmt.type === "AssignmentExpression" ||
                    stmt.type === "LogicalExpression" ||
                    stmt.type === "SequenceExpression" ||
                    stmt.type === "VariableDeclarator") {
                    stmt = stmt.parent;
                }
                return ((stmt.type === "ExpressionStatement" ||
                    stmt.type === "VariableDeclaration") &&
                    stmt.parent && stmt.parent.type === "Program");
            }
            /**
             * Check indent for function block content
             * @param {ASTNode} node A BlockStatement node that is inside of a function.
             * @returns {void}
             */
            function checkIndentInFunctionBlock(node) {
                /*
                 * Search first caller in chain.
                 * Ex.:
                 *
                 * Models <- Identifier
                 *   .User
                 *   .find()
                 *   .exec(function() {
                 *   // function body
                 * });
                 *
                 * Looks for 'Models'
                 */
                const calleeNode = node.parent; // FunctionExpression
                let indent;
                if (calleeNode.parent &&
                    (calleeNode.parent.type === "Property" ||
                        calleeNode.parent.type === "ArrayExpression")) {
                    // If function is part of array or object, comma can be put at left
                    indent = getNodeIndent(calleeNode, false).goodChar;
                }
                else {
                    // If function is standalone, simple calculate indent
                    indent = getNodeIndent(calleeNode).goodChar;
                }
                if (calleeNode.parent.type === "CallExpression") {
                    const calleeParent = calleeNode.parent;
                    if (calleeNode.type !== "FunctionExpression" && calleeNode.type !== "ArrowFunctionExpression") {
                        if (calleeParent && calleeParent.loc.start.line < node.loc.start.line) {
                            indent = getNodeIndent(calleeParent).goodChar;
                        }
                    }
                    else {
                        if (isArgBeforeCalleeNodeMultiline(calleeNode) &&
                            calleeParent.callee.loc.start.line === calleeParent.callee.loc.end.line &&
                            !isNodeFirstInLine(calleeNode)) {
                            indent = getNodeIndent(calleeParent).goodChar;
                        }
                    }
                }
                /*
                 * function body indent should be indent + indent size, unless this
                 * is a FunctionDeclaration, FunctionExpression, or outer IIFE and the corresponding options are enabled.
                 */
                let functionOffset = indentSize;
                if (options.outerIIFEBody !== null && isOuterIIFE(calleeNode)) {
                    functionOffset = options.outerIIFEBody * indentSize;
                }
                else if (calleeNode.type === "FunctionExpression") {
                    functionOffset = options.FunctionExpression.body * indentSize;
                }
                else if (calleeNode.type === "FunctionDeclaration") {
                    functionOffset = options.FunctionDeclaration.body * indentSize;
                }
                indent += functionOffset;
                // check if the node is inside a variable
                const parentVarNode = getVariableDeclaratorNode(node);
                if (parentVarNode && isNodeInVarOnTop(node, parentVarNode)) {
                    indent += indentSize * options.VariableDeclarator[parentVarNode.parent.kind];
                }
                if (node.body.length > 0) {
                    checkNodesIndent(node.body, indent);
                }
                checkLastNodeLineIndent(node, indent - functionOffset);
            }
            /**
             * Checks if the given node starts and ends on the same line
             * @param {ASTNode} node The node to check
             * @returns {boolean} Whether or not the block starts and ends on the same line.
             */
            function isSingleLineNode(node) {
                const lastToken = sourceCode.getLastToken(node), startLine = node.loc.start.line, endLine = lastToken.loc.end.line;
                return startLine === endLine;
            }
            /**
             * Check to see if the first element inside an array is an object and on the same line as the node
             * If the node is not an array then it will return false.
             * @param {ASTNode} node node to check
             * @returns {boolean} success/failure
             */
            function isFirstArrayElementOnSameLine(node) {
                if (node.type === "ArrayExpression" && node.elements[0]) {
                    return node.elements[0].loc.start.line === node.loc.start.line && node.elements[0].type === "ObjectExpression";
                }
                return false;
            }
            /**
             * Check indent for array block content or object block content
             * @param {ASTNode} node node to examine
             * @returns {void}
             */
            function checkIndentInArrayOrObjectBlock(node) {
                // Skip inline
                if (isSingleLineNode(node)) {
                    return;
                }
                let elements = (node.type === "ArrayExpression") ? node.elements : node.properties;
                // filter out empty elements example would be [ , 2] so remove first element as espree considers it as null
                elements = elements.filter(elem => elem !== null);
                let nodeIndent;
                let elementsIndent;
                const parentVarNode = getVariableDeclaratorNode(node);
                // TODO - come up with a better strategy in future
                if (isNodeFirstInLine(node)) {
                    const parent = node.parent;
                    nodeIndent = getNodeIndent(parent).goodChar;
                    if (!parentVarNode || parentVarNode.loc.start.line !== node.loc.start.line) {
                        if (parent.type !== "VariableDeclarator" || parentVarNode === parentVarNode.parent.declarations[0]) {
                            if (parent.type === "VariableDeclarator" && parentVarNode.loc.start.line === parent.loc.start.line) {
                                nodeIndent += (indentSize * options.VariableDeclarator[parentVarNode.parent.kind]);
                            }
                            else if (parent.type === "ObjectExpression" || parent.type === "ArrayExpression") {
                                const parentElements = node.parent.type === "ObjectExpression" ? node.parent.properties : node.parent.elements;
                                if (parentElements[0] &&
                                    parentElements[0].loc.start.line === parent.loc.start.line &&
                                    parentElements[0].loc.end.line !== parent.loc.start.line) {
                                    /*
                                     * If the first element of the array spans multiple lines, don't increase the expected indentation of the rest.
                                     * e.g. [{
                                     *        foo: 1
                                     *      },
                                     *      {
                                     *        bar: 1
                                     *      }]
                                     * the second object is not indented.
                                     */
                                }
                                else if (typeof options[parent.type] === "number") {
                                    nodeIndent += options[parent.type] * indentSize;
                                }
                                else {
                                    nodeIndent = parentElements[0].loc.start.column;
                                }
                            }
                            else if (parent.type === "CallExpression" || parent.type === "NewExpression") {
                                if (typeof options.CallExpression.arguments === "number") {
                                    nodeIndent += options.CallExpression.arguments * indentSize;
                                }
                                else if (options.CallExpression.arguments === "first") {
                                    if (parent.arguments.indexOf(node) !== -1) {
                                        nodeIndent = parent.arguments[0].loc.start.column;
                                    }
                                }
                                else {
                                    nodeIndent += indentSize;
                                }
                            }
                            else if (parent.type === "LogicalExpression" || parent.type === "ArrowFunctionExpression") {
                                nodeIndent += indentSize;
                            }
                        }
                    }
                    else if (!parentVarNode && !isFirstArrayElementOnSameLine(parent) && parent.type !== "MemberExpression" && parent.type !== "ExpressionStatement" && parent.type !== "AssignmentExpression" && parent.type !== "Property") {
                        nodeIndent += indentSize;
                    }
                    checkFirstNodeLineIndent(node, nodeIndent);
                }
                else {
                    nodeIndent = getNodeIndent(node).goodChar;
                }
                if (options[node.type] === "first") {
                    elementsIndent = elements.length ? elements[0].loc.start.column : 0; // If there are no elements, elementsIndent doesn't matter.
                }
                else {
                    elementsIndent = nodeIndent + indentSize * options[node.type];
                }
                /*
                 * Check if the node is a multiple variable declaration; if so, then
                 * make sure indentation takes that into account.
                 */
                if (isNodeInVarOnTop(node, parentVarNode)) {
                    elementsIndent += indentSize * options.VariableDeclarator[parentVarNode.parent.kind];
                }
                checkNodesIndent(elements, elementsIndent);
                if (elements.length > 0) {
                    // Skip last block line check if last item in same line
                    if (elements[elements.length - 1].loc.end.line === node.loc.end.line) {
                        return;
                    }
                }
                checkLastNodeLineIndent(node, nodeIndent +
                    (isNodeInVarOnTop(node, parentVarNode) ? options.VariableDeclarator[parentVarNode.parent.kind] * indentSize : 0));
            }
            /**
             * Check if the node or node body is a BlockStatement or not
             * @param {ASTNode} node node to test
             * @returns {boolean} True if it or its body is a block statement
             */
            function isNodeBodyBlock(node) {
                return node.type === "BlockStatement" || node.type === "ClassBody" || (node.body && node.body.type === "BlockStatement") ||
                    (node.consequent && node.consequent.type === "BlockStatement");
            }
            /**
             * Check indentation for blocks
             * @param {ASTNode} node node to check
             * @returns {void}
             */
            function blockIndentationCheck(node) {
                // Skip inline blocks
                if (isSingleLineNode(node)) {
                    return;
                }
                if (node.parent && (node.parent.type === "FunctionExpression" ||
                    node.parent.type === "FunctionDeclaration" ||
                    node.parent.type === "ArrowFunctionExpression")) {
                    checkIndentInFunctionBlock(node);
                    return;
                }
                let indent;
                let nodesToCheck = [];
                /*
                 * For this statements we should check indent from statement beginning,
                 * not from the beginning of the block.
                 */
                const statementsWithProperties = [
                    "IfStatement", "WhileStatement", "ForStatement", "ForInStatement", "ForOfStatement", "DoWhileStatement", "ClassDeclaration", "TryStatement"
                ];
                if (node.parent && statementsWithProperties.indexOf(node.parent.type) !== -1 && isNodeBodyBlock(node)) {
                    indent = getNodeIndent(node.parent).goodChar;
                }
                else if (node.parent && node.parent.type === "CatchClause") {
                    indent = getNodeIndent(node.parent.parent).goodChar;
                }
                else {
                    indent = getNodeIndent(node).goodChar;
                }
                if (node.type === "IfStatement" && node.consequent.type !== "BlockStatement") {
                    nodesToCheck = [node.consequent];
                }
                else if (Array.isArray(node.body)) {
                    nodesToCheck = node.body;
                }
                else {
                    nodesToCheck = [node.body];
                }
                if (nodesToCheck.length > 0) {
                    checkNodesIndent(nodesToCheck, indent + indentSize);
                }
                if (node.type === "BlockStatement") {
                    checkLastNodeLineIndent(node, indent);
                }
            }
            /**
             * Filter out the elements which are on the same line of each other or the node.
             * basically have only 1 elements from each line except the variable declaration line.
             * @param {ASTNode} node Variable declaration node
             * @returns {ASTNode[]} Filtered elements
             */
            function filterOutSameLineVars(node) {
                return node.declarations.reduce((finalCollection, elem) => {
                    const lastElem = finalCollection[finalCollection.length - 1];
                    if ((elem.loc.start.line !== node.loc.start.line && !lastElem) ||
                        (lastElem && lastElem.loc.start.line !== elem.loc.start.line)) {
                        finalCollection.push(elem);
                    }
                    return finalCollection;
                }, []);
            }
            /**
             * Check indentation for variable declarations
             * @param {ASTNode} node node to examine
             * @returns {void}
             */
            function checkIndentInVariableDeclarations(node) {
                const elements = filterOutSameLineVars(node);
                const nodeIndent = getNodeIndent(node).goodChar;
                const lastElement = elements[elements.length - 1];
                const elementsIndent = nodeIndent + indentSize * options.VariableDeclarator[node.kind];
                checkNodesIndent(elements, elementsIndent);
                // Only check the last line if there is any token after the last item
                if (sourceCode.getLastToken(node).loc.end.line <= lastElement.loc.end.line) {
                    return;
                }
                const tokenBeforeLastElement = sourceCode.getTokenBefore(lastElement);
                if (tokenBeforeLastElement.value === ",") {
                    // Special case for comma-first syntax where the semicolon is indented
                    checkLastNodeLineIndent(node, getNodeIndent(tokenBeforeLastElement).goodChar);
                }
                else {
                    checkLastNodeLineIndent(node, elementsIndent - indentSize);
                }
            }
            /**
             * Check and decide whether to check for indentation for blockless nodes
             * Scenarios are for or while statements without braces around them
             * @param {ASTNode} node node to examine
             * @returns {void}
             */
            function blockLessNodes(node) {
                if (node.body.type !== "BlockStatement") {
                    blockIndentationCheck(node);
                }
            }
            /**
             * Returns the expected indentation for the case statement
             * @param {ASTNode} node node to examine
             * @param {int} [providedSwitchIndent] indent for switch statement
             * @returns {int} indent size
             */
            function expectedCaseIndent(node, providedSwitchIndent) {
                const switchNode = (node.type === "SwitchStatement") ? node : node.parent;
                const switchIndent = typeof providedSwitchIndent === "undefined"
                    ? getNodeIndent(switchNode).goodChar
                    : providedSwitchIndent;
                let caseIndent;
                if (caseIndentStore[switchNode.loc.start.line]) {
                    return caseIndentStore[switchNode.loc.start.line];
                }
                if (switchNode.cases.length > 0 && options.SwitchCase === 0) {
                    caseIndent = switchIndent;
                }
                else {
                    caseIndent = switchIndent + (indentSize * options.SwitchCase);
                }
                caseIndentStore[switchNode.loc.start.line] = caseIndent;
                return caseIndent;
            }
            /**
             * Checks wether a return statement is wrapped in ()
             * @param {ASTNode} node node to examine
             * @returns {boolean} the result
             */
            function isWrappedInParenthesis(node) {
                const regex = /^return\s*?\(\s*?\);*?/u;
                const statementWithoutArgument = sourceCode.getText(node).replace(sourceCode.getText(node.argument), "");
                return regex.test(statementWithoutArgument);
            }
            return {
                Program(node) {
                    if (node.body.length > 0) {
                        // Root nodes should have no indent
                        checkNodesIndent(node.body, getNodeIndent(node).goodChar);
                    }
                },
                ClassBody: blockIndentationCheck,
                BlockStatement: blockIndentationCheck,
                WhileStatement: blockLessNodes,
                ForStatement: blockLessNodes,
                ForInStatement: blockLessNodes,
                ForOfStatement: blockLessNodes,
                DoWhileStatement: blockLessNodes,
                IfStatement(node) {
                    if (node.consequent.type !== "BlockStatement" && node.consequent.loc.start.line > node.loc.start.line) {
                        blockIndentationCheck(node);
                    }
                },
                VariableDeclaration(node) {
                    if (node.declarations[node.declarations.length - 1].loc.start.line > node.declarations[0].loc.start.line) {
                        checkIndentInVariableDeclarations(node);
                    }
                },
                ObjectExpression(node) {
                    checkIndentInArrayOrObjectBlock(node);
                },
                ArrayExpression(node) {
                    checkIndentInArrayOrObjectBlock(node);
                },
                MemberExpression(node) {
                    if (typeof options.MemberExpression === "undefined") {
                        return;
                    }
                    if (isSingleLineNode(node)) {
                        return;
                    }
                    /*
                     * The typical layout of variable declarations and assignments
                     * alter the expectation of correct indentation. Skip them.
                     * TODO: Add appropriate configuration options for variable
                     * declarations and assignments.
                     */
                    if (getParentNodeByType(node, "VariableDeclarator", ["FunctionExpression", "ArrowFunctionExpression"])) {
                        return;
                    }
                    if (getParentNodeByType(node, "AssignmentExpression", ["FunctionExpression"])) {
                        return;
                    }
                    const propertyIndent = getNodeIndent(node).goodChar + indentSize * options.MemberExpression;
                    const checkNodes = [node.property];
                    const dot = sourceCode.getTokenBefore(node.property);
                    if (dot.type === "Punctuator" && dot.value === ".") {
                        checkNodes.push(dot);
                    }
                    checkNodesIndent(checkNodes, propertyIndent);
                },
                SwitchStatement(node) {
                    // Switch is not a 'BlockStatement'
                    const switchIndent = getNodeIndent(node).goodChar;
                    const caseIndent = expectedCaseIndent(node, switchIndent);
                    checkNodesIndent(node.cases, caseIndent);
                    checkLastNodeLineIndent(node, switchIndent);
                },
                SwitchCase(node) {
                    // Skip inline cases
                    if (isSingleLineNode(node)) {
                        return;
                    }
                    const caseIndent = expectedCaseIndent(node);
                    checkNodesIndent(node.consequent, caseIndent + indentSize);
                },
                FunctionDeclaration(node) {
                    if (isSingleLineNode(node)) {
                        return;
                    }
                    if (options.FunctionDeclaration.parameters === "first" && node.params.length) {
                        checkNodesIndent(node.params.slice(1), node.params[0].loc.start.column);
                    }
                    else if (options.FunctionDeclaration.parameters !== null) {
                        checkNodesIndent(node.params, getNodeIndent(node).goodChar + indentSize * options.FunctionDeclaration.parameters);
                    }
                },
                FunctionExpression(node) {
                    if (isSingleLineNode(node)) {
                        return;
                    }
                    if (options.FunctionExpression.parameters === "first" && node.params.length) {
                        checkNodesIndent(node.params.slice(1), node.params[0].loc.start.column);
                    }
                    else if (options.FunctionExpression.parameters !== null) {
                        checkNodesIndent(node.params, getNodeIndent(node).goodChar + indentSize * options.FunctionExpression.parameters);
                    }
                },
                ReturnStatement(node) {
                    if (isSingleLineNode(node)) {
                        return;
                    }
                    const firstLineIndent = getNodeIndent(node).goodChar;
                    // in case if return statement is wrapped in parenthesis
                    if (isWrappedInParenthesis(node)) {
                        checkLastReturnStatementLineIndent(node, firstLineIndent);
                    }
                    else {
                        checkNodeIndent(node, firstLineIndent);
                    }
                },
                CallExpression(node) {
                    if (isSingleLineNode(node)) {
                        return;
                    }
                    if (options.CallExpression.arguments === "first" && node.arguments.length) {
                        checkNodesIndent(node.arguments.slice(1), node.arguments[0].loc.start.column);
                    }
                    else if (options.CallExpression.arguments !== null) {
                        checkNodesIndent(node.arguments, getNodeIndent(node).goodChar + indentSize * options.CallExpression.arguments);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/init-declarations.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a given node is a for loop.
     * @param {ASTNode} block A node to check.
     * @returns {boolean} `true` when the node is a for loop.
     */
    function isForLoop(block) {
        return block.type === "ForInStatement" ||
            block.type === "ForOfStatement" ||
            block.type === "ForStatement";
    }
    /**
     * Checks whether or not a given declarator node has its initializer.
     * @param {ASTNode} node A declarator node to check.
     * @returns {boolean} `true` when the node has its initializer.
     */
    function isInitialized(node) {
        const declaration = node.parent;
        const block = declaration.parent;
        if (isForLoop(block)) {
            if (block.type === "ForStatement") {
                return block.init === declaration;
            }
            return block.left === declaration;
        }
        return Boolean(node.init);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require or disallow initialization in variable declarations",
                category: "Variables",
                recommended: false,
                url: "https://eslint.org/docs/rules/init-declarations"
            },
            schema: {
                anyOf: [
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["always"]
                            }
                        ],
                        minItems: 0,
                        maxItems: 1
                    },
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["never"]
                            },
                            {
                                type: "object",
                                properties: {
                                    ignoreForLoopInit: {
                                        type: "boolean"
                                    }
                                },
                                additionalProperties: false
                            }
                        ],
                        minItems: 0,
                        maxItems: 2
                    }
                ]
            },
            messages: {
                initialized: "Variable '{{idName}}' should be initialized on declaration.",
                notInitialized: "Variable '{{idName}}' should not be initialized on declaration."
            }
        },
        create(context) {
            const MODE_ALWAYS = "always", MODE_NEVER = "never";
            const mode = context.options[0] || MODE_ALWAYS;
            const params = context.options[1] || {};
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                "VariableDeclaration:exit"(node) {
                    const kind = node.kind, declarations = node.declarations;
                    for (let i = 0; i < declarations.length; ++i) {
                        const declaration = declarations[i], id = declaration.id, initialized = isInitialized(declaration), isIgnoredForLoop = params.ignoreForLoopInit && isForLoop(node.parent);
                        let messageId = "";
                        if (mode === MODE_ALWAYS && !initialized) {
                            messageId = "initialized";
                        }
                        else if (mode === MODE_NEVER && kind !== "const" && initialized && !isIgnoredForLoop) {
                            messageId = "notInitialized";
                        }
                        if (id.type === "Identifier" && messageId) {
                            context.report({
                                node: declaration,
                                messageId,
                                data: {
                                    idName: id.name
                                }
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/jsx-quotes.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Constants
    //------------------------------------------------------------------------------
    const QUOTE_SETTINGS = {
        "prefer-double": {
            quote: "\"",
            description: "singlequote",
            convert(str) {
                return str.replace(/'/gu, "\"");
            }
        },
        "prefer-single": {
            quote: "'",
            description: "doublequote",
            convert(str) {
                return str.replace(/"/gu, "'");
            }
        }
    };
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce the consistent use of either double or single quotes in JSX attributes",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/jsx-quotes"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["prefer-single", "prefer-double"]
                }
            ],
            messages: {
                unexpected: "Unexpected usage of {{description}}."
            }
        },
        create(context) {
            const quoteOption = context.options[0] || "prefer-double", setting = QUOTE_SETTINGS[quoteOption];
            /**
             * Checks if the given string literal node uses the expected quotes
             * @param {ASTNode} node A string literal node.
             * @returns {boolean} Whether or not the string literal used the expected quotes.
             * @public
             */
            function usesExpectedQuotes(node) {
                return node.value.indexOf(setting.quote) !== -1 || astUtils.isSurroundedBy(node.raw, setting.quote);
            }
            return {
                JSXAttribute(node) {
                    const attributeValue = node.value;
                    if (attributeValue && astUtils.isStringLiteral(attributeValue) && !usesExpectedQuotes(attributeValue)) {
                        context.report({
                            node: attributeValue,
                            messageId: "unexpected",
                            data: {
                                description: setting.description
                            },
                            fix(fixer) {
                                return fixer.replaceText(attributeValue, setting.convert(attributeValue.raw));
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/key-spacing.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether a string contains a line terminator as defined in
     * http://www.ecma-international.org/ecma-262/5.1/#sec-7.3
     * @param {string} str String to test.
     * @returns {boolean} True if str contains a line terminator.
     */
    function containsLineTerminator(str) {
        return astUtils.LINEBREAK_MATCHER.test(str);
    }
    /**
     * Gets the last element of an array.
     * @param {Array} arr An array.
     * @returns {any} Last element of arr.
     */
    function last(arr) {
        return arr[arr.length - 1];
    }
    /**
     * Checks whether a node is contained on a single line.
     * @param {ASTNode} node AST Node being evaluated.
     * @returns {boolean} True if the node is a single line.
     */
    function isSingleLine(node) {
        return (node.loc.end.line === node.loc.start.line);
    }
    /**
     * Checks whether the properties on a single line.
     * @param {ASTNode[]} properties List of Property AST nodes.
     * @returns {boolean} True if all properies is on a single line.
     */
    function isSingleLineProperties(properties) {
        const [firstProp] = properties, lastProp = last(properties);
        return firstProp.loc.start.line === lastProp.loc.end.line;
    }
    /**
     * Initializes a single option property from the configuration with defaults for undefined values
     * @param {Object} toOptions Object to be initialized
     * @param {Object} fromOptions Object to be initialized from
     * @returns {Object} The object with correctly initialized options and values
     */
    function initOptionProperty(toOptions, fromOptions) {
        toOptions.mode = fromOptions.mode || "strict";
        // Set value of beforeColon
        if (typeof fromOptions.beforeColon !== "undefined") {
            toOptions.beforeColon = +fromOptions.beforeColon;
        }
        else {
            toOptions.beforeColon = 0;
        }
        // Set value of afterColon
        if (typeof fromOptions.afterColon !== "undefined") {
            toOptions.afterColon = +fromOptions.afterColon;
        }
        else {
            toOptions.afterColon = 1;
        }
        // Set align if exists
        if (typeof fromOptions.align !== "undefined") {
            if (typeof fromOptions.align === "object") {
                toOptions.align = fromOptions.align;
            }
            else { // "string"
                toOptions.align = {
                    on: fromOptions.align,
                    mode: toOptions.mode,
                    beforeColon: toOptions.beforeColon,
                    afterColon: toOptions.afterColon
                };
            }
        }
        return toOptions;
    }
    /**
     * Initializes all the option values (singleLine, multiLine and align) from the configuration with defaults for undefined values
     * @param {Object} toOptions Object to be initialized
     * @param {Object} fromOptions Object to be initialized from
     * @returns {Object} The object with correctly initialized options and values
     */
    function initOptions(toOptions, fromOptions) {
        if (typeof fromOptions.align === "object") {
            // Initialize the alignment configuration
            toOptions.align = initOptionProperty({}, fromOptions.align);
            toOptions.align.on = fromOptions.align.on || "colon";
            toOptions.align.mode = fromOptions.align.mode || "strict";
            toOptions.multiLine = initOptionProperty({}, (fromOptions.multiLine || fromOptions));
            toOptions.singleLine = initOptionProperty({}, (fromOptions.singleLine || fromOptions));
        }
        else { // string or undefined
            toOptions.multiLine = initOptionProperty({}, (fromOptions.multiLine || fromOptions));
            toOptions.singleLine = initOptionProperty({}, (fromOptions.singleLine || fromOptions));
            // If alignment options are defined in multiLine, pull them out into the general align configuration
            if (toOptions.multiLine.align) {
                toOptions.align = {
                    on: toOptions.multiLine.align.on,
                    mode: toOptions.multiLine.align.mode || toOptions.multiLine.mode,
                    beforeColon: toOptions.multiLine.align.beforeColon,
                    afterColon: toOptions.multiLine.align.afterColon
                };
            }
        }
        return toOptions;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing between keys and values in object literal properties",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/key-spacing"
            },
            fixable: "whitespace",
            schema: [{
                    anyOf: [
                        {
                            type: "object",
                            properties: {
                                align: {
                                    anyOf: [
                                        {
                                            enum: ["colon", "value"]
                                        },
                                        {
                                            type: "object",
                                            properties: {
                                                mode: {
                                                    enum: ["strict", "minimum"]
                                                },
                                                on: {
                                                    enum: ["colon", "value"]
                                                },
                                                beforeColon: {
                                                    type: "boolean"
                                                },
                                                afterColon: {
                                                    type: "boolean"
                                                }
                                            },
                                            additionalProperties: false
                                        }
                                    ]
                                },
                                mode: {
                                    enum: ["strict", "minimum"]
                                },
                                beforeColon: {
                                    type: "boolean"
                                },
                                afterColon: {
                                    type: "boolean"
                                }
                            },
                            additionalProperties: false
                        },
                        {
                            type: "object",
                            properties: {
                                singleLine: {
                                    type: "object",
                                    properties: {
                                        mode: {
                                            enum: ["strict", "minimum"]
                                        },
                                        beforeColon: {
                                            type: "boolean"
                                        },
                                        afterColon: {
                                            type: "boolean"
                                        }
                                    },
                                    additionalProperties: false
                                },
                                multiLine: {
                                    type: "object",
                                    properties: {
                                        align: {
                                            anyOf: [
                                                {
                                                    enum: ["colon", "value"]
                                                },
                                                {
                                                    type: "object",
                                                    properties: {
                                                        mode: {
                                                            enum: ["strict", "minimum"]
                                                        },
                                                        on: {
                                                            enum: ["colon", "value"]
                                                        },
                                                        beforeColon: {
                                                            type: "boolean"
                                                        },
                                                        afterColon: {
                                                            type: "boolean"
                                                        }
                                                    },
                                                    additionalProperties: false
                                                }
                                            ]
                                        },
                                        mode: {
                                            enum: ["strict", "minimum"]
                                        },
                                        beforeColon: {
                                            type: "boolean"
                                        },
                                        afterColon: {
                                            type: "boolean"
                                        }
                                    },
                                    additionalProperties: false
                                }
                            },
                            additionalProperties: false
                        },
                        {
                            type: "object",
                            properties: {
                                singleLine: {
                                    type: "object",
                                    properties: {
                                        mode: {
                                            enum: ["strict", "minimum"]
                                        },
                                        beforeColon: {
                                            type: "boolean"
                                        },
                                        afterColon: {
                                            type: "boolean"
                                        }
                                    },
                                    additionalProperties: false
                                },
                                multiLine: {
                                    type: "object",
                                    properties: {
                                        mode: {
                                            enum: ["strict", "minimum"]
                                        },
                                        beforeColon: {
                                            type: "boolean"
                                        },
                                        afterColon: {
                                            type: "boolean"
                                        }
                                    },
                                    additionalProperties: false
                                },
                                align: {
                                    type: "object",
                                    properties: {
                                        mode: {
                                            enum: ["strict", "minimum"]
                                        },
                                        on: {
                                            enum: ["colon", "value"]
                                        },
                                        beforeColon: {
                                            type: "boolean"
                                        },
                                        afterColon: {
                                            type: "boolean"
                                        }
                                    },
                                    additionalProperties: false
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }],
            messages: {
                extraKey: "Extra space after {{computed}}key '{{key}}'.",
                extraValue: "Extra space before value for {{computed}}key '{{key}}'.",
                missingKey: "Missing space after {{computed}}key '{{key}}'.",
                missingValue: "Missing space before value for {{computed}}key '{{key}}'."
            }
        },
        create(context) {
            /**
             * OPTIONS
             * "key-spacing": [2, {
             *     beforeColon: false,
             *     afterColon: true,
             *     align: "colon" // Optional, or "value"
             * }
             */
            const options = context.options[0] || {}, ruleOptions = initOptions({}, options), multiLineOptions = ruleOptions.multiLine, singleLineOptions = ruleOptions.singleLine, alignmentOptions = ruleOptions.align || null;
            const sourceCode = context.getSourceCode();
            /**
             * Checks whether a property is a member of the property group it follows.
             * @param {ASTNode} lastMember The last Property known to be in the group.
             * @param {ASTNode} candidate The next Property that might be in the group.
             * @returns {boolean} True if the candidate property is part of the group.
             */
            function continuesPropertyGroup(lastMember, candidate) {
                const groupEndLine = lastMember.loc.start.line, candidateStartLine = candidate.loc.start.line;
                if (candidateStartLine - groupEndLine <= 1) {
                    return true;
                }
                /*
                 * Check that the first comment is adjacent to the end of the group, the
                 * last comment is adjacent to the candidate property, and that successive
                 * comments are adjacent to each other.
                 */
                const leadingComments = sourceCode.getCommentsBefore(candidate);
                if (leadingComments.length &&
                    leadingComments[0].loc.start.line - groupEndLine <= 1 &&
                    candidateStartLine - last(leadingComments).loc.end.line <= 1) {
                    for (let i = 1; i < leadingComments.length; i++) {
                        if (leadingComments[i].loc.start.line - leadingComments[i - 1].loc.end.line > 1) {
                            return false;
                        }
                    }
                    return true;
                }
                return false;
            }
            /**
             * Determines if the given property is key-value property.
             * @param {ASTNode} property Property node to check.
             * @returns {boolean} Whether the property is a key-value property.
             */
            function isKeyValueProperty(property) {
                return !((property.method ||
                    property.shorthand ||
                    property.kind !== "init" || property.type !== "Property") // Could be "ExperimentalSpreadProperty" or "SpreadElement"
                );
            }
            /**
             * Starting from the given a node (a property.key node here) looks forward
             * until it finds the last token before a colon punctuator and returns it.
             * @param {ASTNode} node The node to start looking from.
             * @returns {ASTNode} The last token before a colon punctuator.
             */
            function getLastTokenBeforeColon(node) {
                const colonToken = sourceCode.getTokenAfter(node, astUtils.isColonToken);
                return sourceCode.getTokenBefore(colonToken);
            }
            /**
             * Starting from the given a node (a property.key node here) looks forward
             * until it finds the colon punctuator and returns it.
             * @param {ASTNode} node The node to start looking from.
             * @returns {ASTNode} The colon punctuator.
             */
            function getNextColon(node) {
                return sourceCode.getTokenAfter(node, astUtils.isColonToken);
            }
            /**
             * Gets an object literal property's key as the identifier name or string value.
             * @param {ASTNode} property Property node whose key to retrieve.
             * @returns {string} The property's key.
             */
            function getKey(property) {
                const key = property.key;
                if (property.computed) {
                    return sourceCode.getText().slice(key.range[0], key.range[1]);
                }
                return property.key.name || property.key.value;
            }
            /**
             * Reports an appropriately-formatted error if spacing is incorrect on one
             * side of the colon.
             * @param {ASTNode} property Key-value pair in an object literal.
             * @param {string} side Side being verified - either "key" or "value".
             * @param {string} whitespace Actual whitespace string.
             * @param {int} expected Expected whitespace length.
             * @param {string} mode Value of the mode as "strict" or "minimum"
             * @returns {void}
             */
            function report(property, side, whitespace, expected, mode) {
                const diff = whitespace.length - expected, nextColon = getNextColon(property.key), tokenBeforeColon = sourceCode.getTokenBefore(nextColon, { includeComments: true }), tokenAfterColon = sourceCode.getTokenAfter(nextColon, { includeComments: true }), isKeySide = side === "key", locStart = isKeySide ? tokenBeforeColon.loc.start : tokenAfterColon.loc.start, isExtra = diff > 0, diffAbs = Math.abs(diff), spaces = Array(diffAbs + 1).join(" ");
                if ((diff && mode === "strict" ||
                    diff < 0 && mode === "minimum" ||
                    diff > 0 && !expected && mode === "minimum") &&
                    !(expected && containsLineTerminator(whitespace))) {
                    let fix;
                    if (isExtra) {
                        let range;
                        // Remove whitespace
                        if (isKeySide) {
                            range = [tokenBeforeColon.range[1], tokenBeforeColon.range[1] + diffAbs];
                        }
                        else {
                            range = [tokenAfterColon.range[0] - diffAbs, tokenAfterColon.range[0]];
                        }
                        fix = function (fixer) {
                            return fixer.removeRange(range);
                        };
                    }
                    else {
                        // Add whitespace
                        if (isKeySide) {
                            fix = function (fixer) {
                                return fixer.insertTextAfter(tokenBeforeColon, spaces);
                            };
                        }
                        else {
                            fix = function (fixer) {
                                return fixer.insertTextBefore(tokenAfterColon, spaces);
                            };
                        }
                    }
                    let messageId = "";
                    if (isExtra) {
                        messageId = side === "key" ? "extraKey" : "extraValue";
                    }
                    else {
                        messageId = side === "key" ? "missingKey" : "missingValue";
                    }
                    context.report({
                        node: property[side],
                        loc: locStart,
                        messageId,
                        data: {
                            computed: property.computed ? "computed " : "",
                            key: getKey(property)
                        },
                        fix
                    });
                }
            }
            /**
             * Gets the number of characters in a key, including quotes around string
             * keys and braces around computed property keys.
             * @param {ASTNode} property Property of on object literal.
             * @returns {int} Width of the key.
             */
            function getKeyWidth(property) {
                const startToken = sourceCode.getFirstToken(property);
                const endToken = getLastTokenBeforeColon(property.key);
                return endToken.range[1] - startToken.range[0];
            }
            /**
             * Gets the whitespace around the colon in an object literal property.
             * @param {ASTNode} property Property node from an object literal.
             * @returns {Object} Whitespace before and after the property's colon.
             */
            function getPropertyWhitespace(property) {
                const whitespace = /(\s*):(\s*)/u.exec(sourceCode.getText().slice(property.key.range[1], property.value.range[0]));
                if (whitespace) {
                    return {
                        beforeColon: whitespace[1],
                        afterColon: whitespace[2]
                    };
                }
                return null;
            }
            /**
             * Creates groups of properties.
             * @param  {ASTNode} node ObjectExpression node being evaluated.
             * @returns {Array.<ASTNode[]>} Groups of property AST node lists.
             */
            function createGroups(node) {
                if (node.properties.length === 1) {
                    return [node.properties];
                }
                return node.properties.reduce((groups, property) => {
                    const currentGroup = last(groups), prev = last(currentGroup);
                    if (!prev || continuesPropertyGroup(prev, property)) {
                        currentGroup.push(property);
                    }
                    else {
                        groups.push([property]);
                    }
                    return groups;
                }, [
                    []
                ]);
            }
            /**
             * Verifies correct vertical alignment of a group of properties.
             * @param {ASTNode[]} properties List of Property AST nodes.
             * @returns {void}
             */
            function verifyGroupAlignment(properties) {
                const length = properties.length, widths = properties.map(getKeyWidth), // Width of keys, including quotes
                align = alignmentOptions.on; // "value" or "colon"
                let targetWidth = Math.max(...widths), beforeColon, afterColon, mode;
                if (alignmentOptions && length > 1) { // When aligning values within a group, use the alignment configuration.
                    beforeColon = alignmentOptions.beforeColon;
                    afterColon = alignmentOptions.afterColon;
                    mode = alignmentOptions.mode;
                }
                else {
                    beforeColon = multiLineOptions.beforeColon;
                    afterColon = multiLineOptions.afterColon;
                    mode = alignmentOptions.mode;
                }
                // Conditionally include one space before or after colon
                targetWidth += (align === "colon" ? beforeColon : afterColon);
                for (let i = 0; i < length; i++) {
                    const property = properties[i];
                    const whitespace = getPropertyWhitespace(property);
                    if (whitespace) { // Object literal getters/setters lack a colon
                        const width = widths[i];
                        if (align === "value") {
                            report(property, "key", whitespace.beforeColon, beforeColon, mode);
                            report(property, "value", whitespace.afterColon, targetWidth - width, mode);
                        }
                        else { // align = "colon"
                            report(property, "key", whitespace.beforeColon, targetWidth - width, mode);
                            report(property, "value", whitespace.afterColon, afterColon, mode);
                        }
                    }
                }
            }
            /**
             * Verifies spacing of property conforms to specified options.
             * @param  {ASTNode} node Property node being evaluated.
             * @param {Object} lineOptions Configured singleLine or multiLine options
             * @returns {void}
             */
            function verifySpacing(node, lineOptions) {
                const actual = getPropertyWhitespace(node);
                if (actual) { // Object literal getters/setters lack colons
                    report(node, "key", actual.beforeColon, lineOptions.beforeColon, lineOptions.mode);
                    report(node, "value", actual.afterColon, lineOptions.afterColon, lineOptions.mode);
                }
            }
            /**
             * Verifies spacing of each property in a list.
             * @param {ASTNode[]} properties List of Property AST nodes.
             * @param {Object} lineOptions Configured singleLine or multiLine options
             * @returns {void}
             */
            function verifyListSpacing(properties, lineOptions) {
                const length = properties.length;
                for (let i = 0; i < length; i++) {
                    verifySpacing(properties[i], lineOptions);
                }
            }
            /**
             * Verifies vertical alignment, taking into account groups of properties.
             * @param  {ASTNode} node ObjectExpression node being evaluated.
             * @returns {void}
             */
            function verifyAlignment(node) {
                createGroups(node).forEach(group => {
                    const properties = group.filter(isKeyValueProperty);
                    if (properties.length > 0 && isSingleLineProperties(properties)) {
                        verifyListSpacing(properties, multiLineOptions);
                    }
                    else {
                        verifyGroupAlignment(properties);
                    }
                });
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            if (alignmentOptions) { // Verify vertical alignment
                return {
                    ObjectExpression(node) {
                        if (isSingleLine(node)) {
                            verifyListSpacing(node.properties.filter(isKeyValueProperty), singleLineOptions);
                        }
                        else {
                            verifyAlignment(node);
                        }
                    }
                };
            }
            // Obey beforeColon and afterColon in each property as configured
            return {
                Property(node) {
                    verifySpacing(node, isSingleLine(node.parent) ? singleLineOptions : multiLineOptions);
                }
            };
        }
    };
},
/* eslint/lib/rules/keyword-spacing.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */, keywords = require(159) /* ./utils/keywords */;
    //------------------------------------------------------------------------------
    // Constants
    //------------------------------------------------------------------------------
    const PREV_TOKEN = /^[)\]}>]$/u;
    const NEXT_TOKEN = /^(?:[([{<~!]|\+\+?|--?)$/u;
    const PREV_TOKEN_M = /^[)\]}>*]$/u;
    const NEXT_TOKEN_M = /^[{*]$/u;
    const TEMPLATE_OPEN_PAREN = /\$\{$/u;
    const TEMPLATE_CLOSE_PAREN = /^\}/u;
    const CHECK_TYPE = /^(?:JSXElement|RegularExpression|String|Template)$/u;
    const KEYS = keywords.concat(["as", "async", "await", "from", "get", "let", "of", "set", "yield"]);
    // check duplications.
    (function () {
        KEYS.sort();
        for (let i = 1; i < KEYS.length; ++i) {
            if (KEYS[i] === KEYS[i - 1]) {
                throw new Error(`Duplication was found in the keyword list: ${KEYS[i]}`);
            }
        }
    }());
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a given token is a "Template" token ends with "${".
     * @param {Token} token A token to check.
     * @returns {boolean} `true` if the token is a "Template" token ends with "${".
     */
    function isOpenParenOfTemplate(token) {
        return token.type === "Template" && TEMPLATE_OPEN_PAREN.test(token.value);
    }
    /**
     * Checks whether or not a given token is a "Template" token starts with "}".
     * @param {Token} token A token to check.
     * @returns {boolean} `true` if the token is a "Template" token starts with "}".
     */
    function isCloseParenOfTemplate(token) {
        return token.type === "Template" && TEMPLATE_CLOSE_PAREN.test(token.value);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing before and after keywords",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/keyword-spacing"
            },
            fixable: "whitespace",
            schema: [
                {
                    type: "object",
                    properties: {
                        before: { type: "boolean", default: true },
                        after: { type: "boolean", default: true },
                        overrides: {
                            type: "object",
                            properties: KEYS.reduce((retv, key) => {
                                retv[key] = {
                                    type: "object",
                                    properties: {
                                        before: { type: "boolean" },
                                        after: { type: "boolean" }
                                    },
                                    additionalProperties: false
                                };
                                return retv;
                            }, {}),
                            additionalProperties: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                expectedBefore: "Expected space(s) before \"{{value}}\".",
                expectedAfter: "Expected space(s) after \"{{value}}\".",
                unexpectedBefore: "Unexpected space(s) before \"{{value}}\".",
                unexpectedAfter: "Unexpected space(s) after \"{{value}}\"."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Reports a given token if there are not space(s) before the token.
             * @param {Token} token A token to report.
             * @param {RegExp} pattern A pattern of the previous token to check.
             * @returns {void}
             */
            function expectSpaceBefore(token, pattern) {
                const prevToken = sourceCode.getTokenBefore(token);
                if (prevToken &&
                    (CHECK_TYPE.test(prevToken.type) || pattern.test(prevToken.value)) &&
                    !isOpenParenOfTemplate(prevToken) &&
                    astUtils.isTokenOnSameLine(prevToken, token) &&
                    !sourceCode.isSpaceBetweenTokens(prevToken, token)) {
                    context.report({
                        loc: token.loc.start,
                        messageId: "expectedBefore",
                        data: token,
                        fix(fixer) {
                            return fixer.insertTextBefore(token, " ");
                        }
                    });
                }
            }
            /**
             * Reports a given token if there are space(s) before the token.
             * @param {Token} token A token to report.
             * @param {RegExp} pattern A pattern of the previous token to check.
             * @returns {void}
             */
            function unexpectSpaceBefore(token, pattern) {
                const prevToken = sourceCode.getTokenBefore(token);
                if (prevToken &&
                    (CHECK_TYPE.test(prevToken.type) || pattern.test(prevToken.value)) &&
                    !isOpenParenOfTemplate(prevToken) &&
                    astUtils.isTokenOnSameLine(prevToken, token) &&
                    sourceCode.isSpaceBetweenTokens(prevToken, token)) {
                    context.report({
                        loc: token.loc.start,
                        messageId: "unexpectedBefore",
                        data: token,
                        fix(fixer) {
                            return fixer.removeRange([prevToken.range[1], token.range[0]]);
                        }
                    });
                }
            }
            /**
             * Reports a given token if there are not space(s) after the token.
             * @param {Token} token A token to report.
             * @param {RegExp} pattern A pattern of the next token to check.
             * @returns {void}
             */
            function expectSpaceAfter(token, pattern) {
                const nextToken = sourceCode.getTokenAfter(token);
                if (nextToken &&
                    (CHECK_TYPE.test(nextToken.type) || pattern.test(nextToken.value)) &&
                    !isCloseParenOfTemplate(nextToken) &&
                    astUtils.isTokenOnSameLine(token, nextToken) &&
                    !sourceCode.isSpaceBetweenTokens(token, nextToken)) {
                    context.report({
                        loc: token.loc.start,
                        messageId: "expectedAfter",
                        data: token,
                        fix(fixer) {
                            return fixer.insertTextAfter(token, " ");
                        }
                    });
                }
            }
            /**
             * Reports a given token if there are space(s) after the token.
             * @param {Token} token A token to report.
             * @param {RegExp} pattern A pattern of the next token to check.
             * @returns {void}
             */
            function unexpectSpaceAfter(token, pattern) {
                const nextToken = sourceCode.getTokenAfter(token);
                if (nextToken &&
                    (CHECK_TYPE.test(nextToken.type) || pattern.test(nextToken.value)) &&
                    !isCloseParenOfTemplate(nextToken) &&
                    astUtils.isTokenOnSameLine(token, nextToken) &&
                    sourceCode.isSpaceBetweenTokens(token, nextToken)) {
                    context.report({
                        loc: token.loc.start,
                        messageId: "unexpectedAfter",
                        data: token,
                        fix(fixer) {
                            return fixer.removeRange([token.range[1], nextToken.range[0]]);
                        }
                    });
                }
            }
            /**
             * Parses the option object and determines check methods for each keyword.
             * @param {Object|undefined} options The option object to parse.
             * @returns {Object} - Normalized option object.
             *      Keys are keywords (there are for every keyword).
             *      Values are instances of `{"before": function, "after": function}`.
             */
            function parseOptions(options = {}) {
                const before = options.before !== false;
                const after = options.after !== false;
                const defaultValue = {
                    before: before ? expectSpaceBefore : unexpectSpaceBefore,
                    after: after ? expectSpaceAfter : unexpectSpaceAfter
                };
                const overrides = (options && options.overrides) || {};
                const retv = Object.create(null);
                for (let i = 0; i < KEYS.length; ++i) {
                    const key = KEYS[i];
                    const override = overrides[key];
                    if (override) {
                        const thisBefore = ("before" in override) ? override.before : before;
                        const thisAfter = ("after" in override) ? override.after : after;
                        retv[key] = {
                            before: thisBefore ? expectSpaceBefore : unexpectSpaceBefore,
                            after: thisAfter ? expectSpaceAfter : unexpectSpaceAfter
                        };
                    }
                    else {
                        retv[key] = defaultValue;
                    }
                }
                return retv;
            }
            const checkMethodMap = parseOptions(context.options[0]);
            /**
             * Reports a given token if usage of spacing followed by the token is
             * invalid.
             * @param {Token} token A token to report.
             * @param {RegExp} [pattern] Optional. A pattern of the previous
             *      token to check.
             * @returns {void}
             */
            function checkSpacingBefore(token, pattern) {
                checkMethodMap[token.value].before(token, pattern || PREV_TOKEN);
            }
            /**
             * Reports a given token if usage of spacing preceded by the token is
             * invalid.
             * @param {Token} token A token to report.
             * @param {RegExp} [pattern] Optional. A pattern of the next
             *      token to check.
             * @returns {void}
             */
            function checkSpacingAfter(token, pattern) {
                checkMethodMap[token.value].after(token, pattern || NEXT_TOKEN);
            }
            /**
             * Reports a given token if usage of spacing around the token is invalid.
             * @param {Token} token A token to report.
             * @returns {void}
             */
            function checkSpacingAround(token) {
                checkSpacingBefore(token);
                checkSpacingAfter(token);
            }
            /**
             * Reports the first token of a given node if the first token is a keyword
             * and usage of spacing around the token is invalid.
             * @param {ASTNode|null} node A node to report.
             * @returns {void}
             */
            function checkSpacingAroundFirstToken(node) {
                const firstToken = node && sourceCode.getFirstToken(node);
                if (firstToken && firstToken.type === "Keyword") {
                    checkSpacingAround(firstToken);
                }
            }
            /**
             * Reports the first token of a given node if the first token is a keyword
             * and usage of spacing followed by the token is invalid.
             *
             * This is used for unary operators (e.g. `typeof`), `function`, and `super`.
             * Other rules are handling usage of spacing preceded by those keywords.
             * @param {ASTNode|null} node A node to report.
             * @returns {void}
             */
            function checkSpacingBeforeFirstToken(node) {
                const firstToken = node && sourceCode.getFirstToken(node);
                if (firstToken && firstToken.type === "Keyword") {
                    checkSpacingBefore(firstToken);
                }
            }
            /**
             * Reports the previous token of a given node if the token is a keyword and
             * usage of spacing around the token is invalid.
             * @param {ASTNode|null} node A node to report.
             * @returns {void}
             */
            function checkSpacingAroundTokenBefore(node) {
                if (node) {
                    const token = sourceCode.getTokenBefore(node, astUtils.isKeywordToken);
                    checkSpacingAround(token);
                }
            }
            /**
             * Reports `async` or `function` keywords of a given node if usage of
             * spacing around those keywords is invalid.
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function checkSpacingForFunction(node) {
                const firstToken = node && sourceCode.getFirstToken(node);
                if (firstToken &&
                    ((firstToken.type === "Keyword" && firstToken.value === "function") ||
                        firstToken.value === "async")) {
                    checkSpacingBefore(firstToken);
                }
            }
            /**
             * Reports `class` and `extends` keywords of a given node if usage of
             * spacing around those keywords is invalid.
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function checkSpacingForClass(node) {
                checkSpacingAroundFirstToken(node);
                checkSpacingAroundTokenBefore(node.superClass);
            }
            /**
             * Reports `if` and `else` keywords of a given node if usage of spacing
             * around those keywords is invalid.
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function checkSpacingForIfStatement(node) {
                checkSpacingAroundFirstToken(node);
                checkSpacingAroundTokenBefore(node.alternate);
            }
            /**
             * Reports `try`, `catch`, and `finally` keywords of a given node if usage
             * of spacing around those keywords is invalid.
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function checkSpacingForTryStatement(node) {
                checkSpacingAroundFirstToken(node);
                checkSpacingAroundFirstToken(node.handler);
                checkSpacingAroundTokenBefore(node.finalizer);
            }
            /**
             * Reports `do` and `while` keywords of a given node if usage of spacing
             * around those keywords is invalid.
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function checkSpacingForDoWhileStatement(node) {
                checkSpacingAroundFirstToken(node);
                checkSpacingAroundTokenBefore(node.test);
            }
            /**
             * Reports `for` and `in` keywords of a given node if usage of spacing
             * around those keywords is invalid.
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function checkSpacingForForInStatement(node) {
                checkSpacingAroundFirstToken(node);
                checkSpacingAroundTokenBefore(node.right);
            }
            /**
             * Reports `for` and `of` keywords of a given node if usage of spacing
             * around those keywords is invalid.
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function checkSpacingForForOfStatement(node) {
                if (node.await) {
                    checkSpacingBefore(sourceCode.getFirstToken(node, 0));
                    checkSpacingAfter(sourceCode.getFirstToken(node, 1));
                }
                else {
                    checkSpacingAroundFirstToken(node);
                }
                checkSpacingAround(sourceCode.getTokenBefore(node.right, astUtils.isNotOpeningParenToken));
            }
            /**
             * Reports `import`, `export`, `as`, and `from` keywords of a given node if
             * usage of spacing around those keywords is invalid.
             *
             * This rule handles the `*` token in module declarations.
             *
             *     import*as A from "./a"; /*error Expected space(s) after "import".
             *                               error Expected space(s) before "as".
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function checkSpacingForModuleDeclaration(node) {
                const firstToken = sourceCode.getFirstToken(node);
                checkSpacingBefore(firstToken, PREV_TOKEN_M);
                checkSpacingAfter(firstToken, NEXT_TOKEN_M);
                if (node.type === "ExportDefaultDeclaration") {
                    checkSpacingAround(sourceCode.getTokenAfter(firstToken));
                }
                if (node.source) {
                    const fromToken = sourceCode.getTokenBefore(node.source);
                    checkSpacingBefore(fromToken, PREV_TOKEN_M);
                    checkSpacingAfter(fromToken, NEXT_TOKEN_M);
                }
            }
            /**
             * Reports `as` keyword of a given node if usage of spacing around this
             * keyword is invalid.
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function checkSpacingForImportNamespaceSpecifier(node) {
                const asToken = sourceCode.getFirstToken(node, 1);
                checkSpacingBefore(asToken, PREV_TOKEN_M);
            }
            /**
             * Reports `static`, `get`, and `set` keywords of a given node if usage of
             * spacing around those keywords is invalid.
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function checkSpacingForProperty(node) {
                if (node.static) {
                    checkSpacingAroundFirstToken(node);
                }
                if (node.kind === "get" ||
                    node.kind === "set" ||
                    ((node.method || node.type === "MethodDefinition") &&
                        node.value.async)) {
                    const token = sourceCode.getTokenBefore(node.key, tok => {
                        switch (tok.value) {
                            case "get":
                            case "set":
                            case "async":
                                return true;
                            default:
                                return false;
                        }
                    });
                    if (!token) {
                        throw new Error("Failed to find token get, set, or async beside method name");
                    }
                    checkSpacingAround(token);
                }
            }
            /**
             * Reports `await` keyword of a given node if usage of spacing before
             * this keyword is invalid.
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function checkSpacingForAwaitExpression(node) {
                checkSpacingBefore(sourceCode.getFirstToken(node));
            }
            return {
                // Statements
                DebuggerStatement: checkSpacingAroundFirstToken,
                WithStatement: checkSpacingAroundFirstToken,
                // Statements - Control flow
                BreakStatement: checkSpacingAroundFirstToken,
                ContinueStatement: checkSpacingAroundFirstToken,
                ReturnStatement: checkSpacingAroundFirstToken,
                ThrowStatement: checkSpacingAroundFirstToken,
                TryStatement: checkSpacingForTryStatement,
                // Statements - Choice
                IfStatement: checkSpacingForIfStatement,
                SwitchStatement: checkSpacingAroundFirstToken,
                SwitchCase: checkSpacingAroundFirstToken,
                // Statements - Loops
                DoWhileStatement: checkSpacingForDoWhileStatement,
                ForInStatement: checkSpacingForForInStatement,
                ForOfStatement: checkSpacingForForOfStatement,
                ForStatement: checkSpacingAroundFirstToken,
                WhileStatement: checkSpacingAroundFirstToken,
                // Statements - Declarations
                ClassDeclaration: checkSpacingForClass,
                ExportNamedDeclaration: checkSpacingForModuleDeclaration,
                ExportDefaultDeclaration: checkSpacingForModuleDeclaration,
                ExportAllDeclaration: checkSpacingForModuleDeclaration,
                FunctionDeclaration: checkSpacingForFunction,
                ImportDeclaration: checkSpacingForModuleDeclaration,
                VariableDeclaration: checkSpacingAroundFirstToken,
                // Expressions
                ArrowFunctionExpression: checkSpacingForFunction,
                AwaitExpression: checkSpacingForAwaitExpression,
                ClassExpression: checkSpacingForClass,
                FunctionExpression: checkSpacingForFunction,
                NewExpression: checkSpacingBeforeFirstToken,
                Super: checkSpacingBeforeFirstToken,
                ThisExpression: checkSpacingBeforeFirstToken,
                UnaryExpression: checkSpacingBeforeFirstToken,
                YieldExpression: checkSpacingBeforeFirstToken,
                // Others
                ImportNamespaceSpecifier: checkSpacingForImportNamespaceSpecifier,
                MethodDefinition: checkSpacingForProperty,
                Property: checkSpacingForProperty
            };
        }
    };
},
/* eslint/lib/rules/line-comment-position.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce position of line comments",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/line-comment-position"
            },
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["above", "beside"]
                        },
                        {
                            type: "object",
                            properties: {
                                position: {
                                    enum: ["above", "beside"]
                                },
                                ignorePattern: {
                                    type: "string"
                                },
                                applyDefaultPatterns: {
                                    type: "boolean"
                                },
                                applyDefaultIgnorePatterns: {
                                    type: "boolean"
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                above: "Expected comment to be above code.",
                beside: "Expected comment to be beside code."
            }
        },
        create(context) {
            const options = context.options[0];
            let above, ignorePattern, applyDefaultIgnorePatterns = true;
            if (!options || typeof options === "string") {
                above = !options || options === "above";
            }
            else {
                above = !options.position || options.position === "above";
                ignorePattern = options.ignorePattern;
                if (Object.prototype.hasOwnProperty.call(options, "applyDefaultIgnorePatterns")) {
                    applyDefaultIgnorePatterns = options.applyDefaultIgnorePatterns;
                }
                else {
                    applyDefaultIgnorePatterns = options.applyDefaultPatterns !== false;
                }
            }
            const defaultIgnoreRegExp = astUtils.COMMENTS_IGNORE_PATTERN;
            const fallThroughRegExp = /^\s*falls?\s?through/u;
            const customIgnoreRegExp = new RegExp(ignorePattern, "u");
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                Program() {
                    const comments = sourceCode.getAllComments();
                    comments.filter(token => token.type === "Line").forEach(node => {
                        if (applyDefaultIgnorePatterns && (defaultIgnoreRegExp.test(node.value) || fallThroughRegExp.test(node.value))) {
                            return;
                        }
                        if (ignorePattern && customIgnoreRegExp.test(node.value)) {
                            return;
                        }
                        const previous = sourceCode.getTokenBefore(node, { includeComments: true });
                        const isOnSameLine = previous && previous.loc.end.line === node.loc.start.line;
                        if (above) {
                            if (isOnSameLine) {
                                context.report({
                                    node,
                                    messageId: "above"
                                });
                            }
                        }
                        else {
                            if (!isOnSameLine) {
                                context.report({
                                    node,
                                    messageId: "beside"
                                });
                            }
                        }
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/linebreak-style.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent linebreak style",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/linebreak-style"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["unix", "windows"]
                }
            ],
            messages: {
                expectedLF: "Expected linebreaks to be 'LF' but found 'CRLF'.",
                expectedCRLF: "Expected linebreaks to be 'CRLF' but found 'LF'."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Builds a fix function that replaces text at the specified range in the source text.
             * @param {int[]} range The range to replace
             * @param {string} text The text to insert.
             * @returns {Function} Fixer function
             * @private
             */
            function createFix(range, text) {
                return function (fixer) {
                    return fixer.replaceTextRange(range, text);
                };
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                Program: function checkForlinebreakStyle(node) {
                    const linebreakStyle = context.options[0] || "unix", expectedLF = linebreakStyle === "unix", expectedLFChars = expectedLF ? "\n" : "\r\n", source = sourceCode.getText(), pattern = astUtils.createGlobalLinebreakMatcher();
                    let match;
                    let i = 0;
                    while ((match = pattern.exec(source)) !== null) {
                        i++;
                        if (match[0] === expectedLFChars) {
                            continue;
                        }
                        const index = match.index;
                        const range = [index, index + match[0].length];
                        context.report({
                            node,
                            loc: {
                                line: i,
                                column: sourceCode.lines[i - 1].length
                            },
                            messageId: expectedLF ? "expectedLF" : "expectedCRLF",
                            fix: createFix(range, expectedLFChars)
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/lines-around-comment.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */, astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Return an array with with any line numbers that are empty.
     * @param {Array} lines An array of each line of the file.
     * @returns {Array} An array of line numbers.
     */
    function getEmptyLineNums(lines) {
        const emptyLines = lines.map((line, i) => ({
            code: line.trim(),
            num: i + 1
        })).filter(line => !line.code).map(line => line.num);
        return emptyLines;
    }
    /**
     * Return an array with with any line numbers that contain comments.
     * @param {Array} comments An array of comment tokens.
     * @returns {Array} An array of line numbers.
     */
    function getCommentLineNums(comments) {
        const lines = [];
        comments.forEach(token => {
            const start = token.loc.start.line;
            const end = token.loc.end.line;
            lines.push(start, end);
        });
        return lines;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require empty lines around comments",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/lines-around-comment"
            },
            fixable: "whitespace",
            schema: [
                {
                    type: "object",
                    properties: {
                        beforeBlockComment: {
                            type: "boolean",
                            default: true
                        },
                        afterBlockComment: {
                            type: "boolean",
                            default: false
                        },
                        beforeLineComment: {
                            type: "boolean",
                            default: false
                        },
                        afterLineComment: {
                            type: "boolean",
                            default: false
                        },
                        allowBlockStart: {
                            type: "boolean",
                            default: false
                        },
                        allowBlockEnd: {
                            type: "boolean",
                            default: false
                        },
                        allowClassStart: {
                            type: "boolean"
                        },
                        allowClassEnd: {
                            type: "boolean"
                        },
                        allowObjectStart: {
                            type: "boolean"
                        },
                        allowObjectEnd: {
                            type: "boolean"
                        },
                        allowArrayStart: {
                            type: "boolean"
                        },
                        allowArrayEnd: {
                            type: "boolean"
                        },
                        ignorePattern: {
                            type: "string"
                        },
                        applyDefaultIgnorePatterns: {
                            type: "boolean"
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                after: "Expected line after comment.",
                before: "Expected line before comment."
            }
        },
        create(context) {
            const options = Object.assign({}, context.options[0]);
            const ignorePattern = options.ignorePattern;
            const defaultIgnoreRegExp = astUtils.COMMENTS_IGNORE_PATTERN;
            const customIgnoreRegExp = new RegExp(ignorePattern, "u");
            const applyDefaultIgnorePatterns = options.applyDefaultIgnorePatterns !== false;
            options.beforeBlockComment = typeof options.beforeBlockComment !== "undefined" ? options.beforeBlockComment : true;
            const sourceCode = context.getSourceCode();
            const lines = sourceCode.lines, numLines = lines.length + 1, comments = sourceCode.getAllComments(), commentLines = getCommentLineNums(comments), emptyLines = getEmptyLineNums(lines), commentAndEmptyLines = commentLines.concat(emptyLines);
            /**
             * Returns whether or not comments are on lines starting with or ending with code
             * @param {token} token The comment token to check.
             * @returns {boolean} True if the comment is not alone.
             */
            function codeAroundComment(token) {
                let currentToken = token;
                do {
                    currentToken = sourceCode.getTokenBefore(currentToken, { includeComments: true });
                } while (currentToken && astUtils.isCommentToken(currentToken));
                if (currentToken && astUtils.isTokenOnSameLine(currentToken, token)) {
                    return true;
                }
                currentToken = token;
                do {
                    currentToken = sourceCode.getTokenAfter(currentToken, { includeComments: true });
                } while (currentToken && astUtils.isCommentToken(currentToken));
                if (currentToken && astUtils.isTokenOnSameLine(token, currentToken)) {
                    return true;
                }
                return false;
            }
            /**
             * Returns whether or not comments are inside a node type or not.
             * @param {ASTNode} parent The Comment parent node.
             * @param {string} nodeType The parent type to check against.
             * @returns {boolean} True if the comment is inside nodeType.
             */
            function isParentNodeType(parent, nodeType) {
                return parent.type === nodeType ||
                    (parent.body && parent.body.type === nodeType) ||
                    (parent.consequent && parent.consequent.type === nodeType);
            }
            /**
             * Returns the parent node that contains the given token.
             * @param {token} token The token to check.
             * @returns {ASTNode} The parent node that contains the given token.
             */
            function getParentNodeOfToken(token) {
                return sourceCode.getNodeByRangeIndex(token.range[0]);
            }
            /**
             * Returns whether or not comments are at the parent start or not.
             * @param {token} token The Comment token.
             * @param {string} nodeType The parent type to check against.
             * @returns {boolean} True if the comment is at parent start.
             */
            function isCommentAtParentStart(token, nodeType) {
                const parent = getParentNodeOfToken(token);
                return parent && isParentNodeType(parent, nodeType) &&
                    token.loc.start.line - parent.loc.start.line === 1;
            }
            /**
             * Returns whether or not comments are at the parent end or not.
             * @param {token} token The Comment token.
             * @param {string} nodeType The parent type to check against.
             * @returns {boolean} True if the comment is at parent end.
             */
            function isCommentAtParentEnd(token, nodeType) {
                const parent = getParentNodeOfToken(token);
                return parent && isParentNodeType(parent, nodeType) &&
                    parent.loc.end.line - token.loc.end.line === 1;
            }
            /**
             * Returns whether or not comments are at the block start or not.
             * @param {token} token The Comment token.
             * @returns {boolean} True if the comment is at block start.
             */
            function isCommentAtBlockStart(token) {
                return isCommentAtParentStart(token, "ClassBody") || isCommentAtParentStart(token, "BlockStatement") || isCommentAtParentStart(token, "SwitchCase");
            }
            /**
             * Returns whether or not comments are at the block end or not.
             * @param {token} token The Comment token.
             * @returns {boolean} True if the comment is at block end.
             */
            function isCommentAtBlockEnd(token) {
                return isCommentAtParentEnd(token, "ClassBody") || isCommentAtParentEnd(token, "BlockStatement") || isCommentAtParentEnd(token, "SwitchCase") || isCommentAtParentEnd(token, "SwitchStatement");
            }
            /**
             * Returns whether or not comments are at the class start or not.
             * @param {token} token The Comment token.
             * @returns {boolean} True if the comment is at class start.
             */
            function isCommentAtClassStart(token) {
                return isCommentAtParentStart(token, "ClassBody");
            }
            /**
             * Returns whether or not comments are at the class end or not.
             * @param {token} token The Comment token.
             * @returns {boolean} True if the comment is at class end.
             */
            function isCommentAtClassEnd(token) {
                return isCommentAtParentEnd(token, "ClassBody");
            }
            /**
             * Returns whether or not comments are at the object start or not.
             * @param {token} token The Comment token.
             * @returns {boolean} True if the comment is at object start.
             */
            function isCommentAtObjectStart(token) {
                return isCommentAtParentStart(token, "ObjectExpression") || isCommentAtParentStart(token, "ObjectPattern");
            }
            /**
             * Returns whether or not comments are at the object end or not.
             * @param {token} token The Comment token.
             * @returns {boolean} True if the comment is at object end.
             */
            function isCommentAtObjectEnd(token) {
                return isCommentAtParentEnd(token, "ObjectExpression") || isCommentAtParentEnd(token, "ObjectPattern");
            }
            /**
             * Returns whether or not comments are at the array start or not.
             * @param {token} token The Comment token.
             * @returns {boolean} True if the comment is at array start.
             */
            function isCommentAtArrayStart(token) {
                return isCommentAtParentStart(token, "ArrayExpression") || isCommentAtParentStart(token, "ArrayPattern");
            }
            /**
             * Returns whether or not comments are at the array end or not.
             * @param {token} token The Comment token.
             * @returns {boolean} True if the comment is at array end.
             */
            function isCommentAtArrayEnd(token) {
                return isCommentAtParentEnd(token, "ArrayExpression") || isCommentAtParentEnd(token, "ArrayPattern");
            }
            /**
             * Checks if a comment token has lines around it (ignores inline comments)
             * @param {token} token The Comment token.
             * @param {Object} opts Options to determine the newline.
             * @param {boolean} opts.after Should have a newline after this line.
             * @param {boolean} opts.before Should have a newline before this line.
             * @returns {void}
             */
            function checkForEmptyLine(token, opts) {
                if (applyDefaultIgnorePatterns && defaultIgnoreRegExp.test(token.value)) {
                    return;
                }
                if (ignorePattern && customIgnoreRegExp.test(token.value)) {
                    return;
                }
                let after = opts.after, before = opts.before;
                const prevLineNum = token.loc.start.line - 1, nextLineNum = token.loc.end.line + 1, commentIsNotAlone = codeAroundComment(token);
                const blockStartAllowed = options.allowBlockStart &&
                    isCommentAtBlockStart(token) &&
                    !(options.allowClassStart === false &&
                        isCommentAtClassStart(token)), blockEndAllowed = options.allowBlockEnd && isCommentAtBlockEnd(token) && !(options.allowClassEnd === false && isCommentAtClassEnd(token)), classStartAllowed = options.allowClassStart && isCommentAtClassStart(token), classEndAllowed = options.allowClassEnd && isCommentAtClassEnd(token), objectStartAllowed = options.allowObjectStart && isCommentAtObjectStart(token), objectEndAllowed = options.allowObjectEnd && isCommentAtObjectEnd(token), arrayStartAllowed = options.allowArrayStart && isCommentAtArrayStart(token), arrayEndAllowed = options.allowArrayEnd && isCommentAtArrayEnd(token);
                const exceptionStartAllowed = blockStartAllowed || classStartAllowed || objectStartAllowed || arrayStartAllowed;
                const exceptionEndAllowed = blockEndAllowed || classEndAllowed || objectEndAllowed || arrayEndAllowed;
                // ignore top of the file and bottom of the file
                if (prevLineNum < 1) {
                    before = false;
                }
                if (nextLineNum >= numLines) {
                    after = false;
                }
                // we ignore all inline comments
                if (commentIsNotAlone) {
                    return;
                }
                const previousTokenOrComment = sourceCode.getTokenBefore(token, { includeComments: true });
                const nextTokenOrComment = sourceCode.getTokenAfter(token, { includeComments: true });
                // check for newline before
                if (!exceptionStartAllowed && before && !lodash.includes(commentAndEmptyLines, prevLineNum) &&
                    !(astUtils.isCommentToken(previousTokenOrComment) && astUtils.isTokenOnSameLine(previousTokenOrComment, token))) {
                    const lineStart = token.range[0] - token.loc.start.column;
                    const range = [lineStart, lineStart];
                    context.report({
                        node: token,
                        messageId: "before",
                        fix(fixer) {
                            return fixer.insertTextBeforeRange(range, "\n");
                        }
                    });
                }
                // check for newline after
                if (!exceptionEndAllowed && after && !lodash.includes(commentAndEmptyLines, nextLineNum) &&
                    !(astUtils.isCommentToken(nextTokenOrComment) && astUtils.isTokenOnSameLine(token, nextTokenOrComment))) {
                    context.report({
                        node: token,
                        messageId: "after",
                        fix(fixer) {
                            return fixer.insertTextAfter(token, "\n");
                        }
                    });
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                Program() {
                    comments.forEach(token => {
                        if (token.type === "Line") {
                            if (options.beforeLineComment || options.afterLineComment) {
                                checkForEmptyLine(token, {
                                    after: options.afterLineComment,
                                    before: options.beforeLineComment
                                });
                            }
                        }
                        else if (token.type === "Block") {
                            if (options.beforeBlockComment || options.afterBlockComment) {
                                checkForEmptyLine(token, {
                                    after: options.afterBlockComment,
                                    before: options.beforeBlockComment
                                });
                            }
                        }
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/lines-around-directive.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow newlines around directives",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/lines-around-directive"
            },
            schema: [{
                    oneOf: [
                        {
                            enum: ["always", "never"]
                        },
                        {
                            type: "object",
                            properties: {
                                before: {
                                    enum: ["always", "never"]
                                },
                                after: {
                                    enum: ["always", "never"]
                                }
                            },
                            additionalProperties: false,
                            minProperties: 2
                        }
                    ]
                }],
            fixable: "whitespace",
            messages: {
                expected: "Expected newline {{location}} \"{{value}}\" directive.",
                unexpected: "Unexpected newline {{location}} \"{{value}}\" directive."
            },
            deprecated: true,
            replacedBy: ["padding-line-between-statements"]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const config = context.options[0] || "always";
            const expectLineBefore = typeof config === "string" ? config : config.before;
            const expectLineAfter = typeof config === "string" ? config : config.after;
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Check if node is preceded by a blank newline.
             * @param {ASTNode} node Node to check.
             * @returns {boolean} Whether or not the passed in node is preceded by a blank newline.
             */
            function hasNewlineBefore(node) {
                const tokenBefore = sourceCode.getTokenBefore(node, { includeComments: true });
                const tokenLineBefore = tokenBefore ? tokenBefore.loc.end.line : 0;
                return node.loc.start.line - tokenLineBefore >= 2;
            }
            /**
             * Gets the last token of a node that is on the same line as the rest of the node.
             * This will usually be the last token of the node, but it will be the second-to-last token if the node has a trailing
             * semicolon on a different line.
             * @param {ASTNode} node A directive node
             * @returns {Token} The last token of the node on the line
             */
            function getLastTokenOnLine(node) {
                const lastToken = sourceCode.getLastToken(node);
                const secondToLastToken = sourceCode.getTokenBefore(lastToken);
                return astUtils.isSemicolonToken(lastToken) && lastToken.loc.start.line > secondToLastToken.loc.end.line
                    ? secondToLastToken
                    : lastToken;
            }
            /**
             * Check if node is followed by a blank newline.
             * @param {ASTNode} node Node to check.
             * @returns {boolean} Whether or not the passed in node is followed by a blank newline.
             */
            function hasNewlineAfter(node) {
                const lastToken = getLastTokenOnLine(node);
                const tokenAfter = sourceCode.getTokenAfter(lastToken, { includeComments: true });
                return tokenAfter.loc.start.line - lastToken.loc.end.line >= 2;
            }
            /**
             * Report errors for newlines around directives.
             * @param {ASTNode} node Node to check.
             * @param {string} location Whether the error was found before or after the directive.
             * @param {boolean} expected Whether or not a newline was expected or unexpected.
             * @returns {void}
             */
            function reportError(node, location, expected) {
                context.report({
                    node,
                    messageId: expected ? "expected" : "unexpected",
                    data: {
                        value: node.expression.value,
                        location
                    },
                    fix(fixer) {
                        const lastToken = getLastTokenOnLine(node);
                        if (expected) {
                            return location === "before" ? fixer.insertTextBefore(node, "\n") : fixer.insertTextAfter(lastToken, "\n");
                        }
                        return fixer.removeRange(location === "before" ? [node.range[0] - 1, node.range[0]] : [lastToken.range[1], lastToken.range[1] + 1]);
                    }
                });
            }
            /**
             * Check lines around directives in node
             * @param {ASTNode} node node to check
             * @returns {void}
             */
            function checkDirectives(node) {
                const directives = astUtils.getDirectivePrologue(node);
                if (!directives.length) {
                    return;
                }
                const firstDirective = directives[0];
                const leadingComments = sourceCode.getCommentsBefore(firstDirective);
                /*
                 * Only check before the first directive if it is preceded by a comment or if it is at the top of
                 * the file and expectLineBefore is set to "never". This is to not force a newline at the top of
                 * the file if there are no comments as well as for compatibility with padded-blocks.
                 */
                if (leadingComments.length) {
                    if (expectLineBefore === "always" && !hasNewlineBefore(firstDirective)) {
                        reportError(firstDirective, "before", true);
                    }
                    if (expectLineBefore === "never" && hasNewlineBefore(firstDirective)) {
                        reportError(firstDirective, "before", false);
                    }
                }
                else if (node.type === "Program" &&
                    expectLineBefore === "never" &&
                    !leadingComments.length &&
                    hasNewlineBefore(firstDirective)) {
                    reportError(firstDirective, "before", false);
                }
                const lastDirective = directives[directives.length - 1];
                const statements = node.type === "Program" ? node.body : node.body.body;
                /*
                 * Do not check after the last directive if the body only
                 * contains a directive prologue and isn't followed by a comment to ensure
                 * this rule behaves well with padded-blocks.
                 */
                if (lastDirective === statements[statements.length - 1] && !lastDirective.trailingComments) {
                    return;
                }
                if (expectLineAfter === "always" && !hasNewlineAfter(lastDirective)) {
                    reportError(lastDirective, "after", true);
                }
                if (expectLineAfter === "never" && hasNewlineAfter(lastDirective)) {
                    reportError(lastDirective, "after", false);
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                Program: checkDirectives,
                FunctionDeclaration: checkDirectives,
                FunctionExpression: checkDirectives,
                ArrowFunctionExpression: checkDirectives
            };
        }
    };
},
/* eslint/lib/rules/lines-between-class-members.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow an empty line between class members",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/lines-between-class-members"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["always", "never"]
                },
                {
                    type: "object",
                    properties: {
                        exceptAfterSingleLine: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                never: "Unexpected blank line between class members.",
                always: "Expected blank line between class members."
            }
        },
        create(context) {
            const options = [];
            options[0] = context.options[0] || "always";
            options[1] = context.options[1] || { exceptAfterSingleLine: false };
            const sourceCode = context.getSourceCode();
            /**
             * Return the last token among the consecutive tokens that have no exceed max line difference in between, before the first token in the next member.
             * @param {Token} prevLastToken The last token in the previous member node.
             * @param {Token} nextFirstToken The first token in the next member node.
             * @param {number} maxLine The maximum number of allowed line difference between consecutive tokens.
             * @returns {Token} The last token among the consecutive tokens.
             */
            function findLastConsecutiveTokenAfter(prevLastToken, nextFirstToken, maxLine) {
                const after = sourceCode.getTokenAfter(prevLastToken, { includeComments: true });
                if (after !== nextFirstToken && after.loc.start.line - prevLastToken.loc.end.line <= maxLine) {
                    return findLastConsecutiveTokenAfter(after, nextFirstToken, maxLine);
                }
                return prevLastToken;
            }
            /**
             * Return the first token among the consecutive tokens that have no exceed max line difference in between, after the last token in the previous member.
             * @param {Token} nextFirstToken The first token in the next member node.
             * @param {Token} prevLastToken The last token in the previous member node.
             * @param {number} maxLine The maximum number of allowed line difference between consecutive tokens.
             * @returns {Token} The first token among the consecutive tokens.
             */
            function findFirstConsecutiveTokenBefore(nextFirstToken, prevLastToken, maxLine) {
                const before = sourceCode.getTokenBefore(nextFirstToken, { includeComments: true });
                if (before !== prevLastToken && nextFirstToken.loc.start.line - before.loc.end.line <= maxLine) {
                    return findFirstConsecutiveTokenBefore(before, prevLastToken, maxLine);
                }
                return nextFirstToken;
            }
            /**
             * Checks if there is a token or comment between two tokens.
             * @param {Token} before The token before.
             * @param {Token} after The token after.
             * @returns {boolean} True if there is a token or comment between two tokens.
             */
            function hasTokenOrCommentBetween(before, after) {
                return sourceCode.getTokensBetween(before, after, { includeComments: true }).length !== 0;
            }
            return {
                ClassBody(node) {
                    const body = node.body;
                    for (let i = 0; i < body.length - 1; i++) {
                        const curFirst = sourceCode.getFirstToken(body[i]);
                        const curLast = sourceCode.getLastToken(body[i]);
                        const nextFirst = sourceCode.getFirstToken(body[i + 1]);
                        const isMulti = !astUtils.isTokenOnSameLine(curFirst, curLast);
                        const skip = !isMulti && options[1].exceptAfterSingleLine;
                        const beforePadding = findLastConsecutiveTokenAfter(curLast, nextFirst, 1);
                        const afterPadding = findFirstConsecutiveTokenBefore(nextFirst, curLast, 1);
                        const isPadded = afterPadding.loc.start.line - beforePadding.loc.end.line > 1;
                        const hasTokenInPadding = hasTokenOrCommentBetween(beforePadding, afterPadding);
                        const curLineLastToken = findLastConsecutiveTokenAfter(curLast, nextFirst, 0);
                        if ((options[0] === "always" && !skip && !isPadded) ||
                            (options[0] === "never" && isPadded)) {
                            context.report({
                                node: body[i + 1],
                                messageId: isPadded ? "never" : "always",
                                fix(fixer) {
                                    if (hasTokenInPadding) {
                                        return null;
                                    }
                                    return isPadded
                                        ? fixer.replaceTextRange([beforePadding.range[1], afterPadding.range[0]], "\n")
                                        : fixer.insertTextAfter(curLineLastToken, "\n");
                                }
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/max-classes-per-file.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce a maximum number of classes per file",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/max-classes-per-file"
            },
            schema: [
                {
                    type: "integer",
                    minimum: 1
                }
            ],
            messages: {
                maximumExceeded: "File has too many classes ({{ classCount }}). Maximum allowed is {{ max }}."
            }
        },
        create(context) {
            const maxClasses = context.options[0] || 1;
            let classCount = 0;
            return {
                Program() {
                    classCount = 0;
                },
                "Program:exit"(node) {
                    if (classCount > maxClasses) {
                        context.report({
                            node,
                            messageId: "maximumExceeded",
                            data: {
                                classCount,
                                max: maxClasses
                            }
                        });
                    }
                },
                "ClassDeclaration, ClassExpression"() {
                    classCount++;
                }
            };
        }
    };
},
/* eslint/lib/rules/max-depth.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce a maximum depth that blocks can be nested",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/max-depth"
            },
            schema: [
                {
                    oneOf: [
                        {
                            type: "integer",
                            minimum: 0
                        },
                        {
                            type: "object",
                            properties: {
                                maximum: {
                                    type: "integer",
                                    minimum: 0
                                },
                                max: {
                                    type: "integer",
                                    minimum: 0
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                tooDeeply: "Blocks are nested too deeply ({{depth}}). Maximum allowed is {{maxDepth}}."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            const functionStack = [], option = context.options[0];
            let maxDepth = 4;
            if (typeof option === "object" &&
                (Object.prototype.hasOwnProperty.call(option, "maximum") || Object.prototype.hasOwnProperty.call(option, "max"))) {
                maxDepth = option.maximum || option.max;
            }
            if (typeof option === "number") {
                maxDepth = option;
            }
            /**
             * When parsing a new function, store it in our function stack
             * @returns {void}
             * @private
             */
            function startFunction() {
                functionStack.push(0);
            }
            /**
             * When parsing is done then pop out the reference
             * @returns {void}
             * @private
             */
            function endFunction() {
                functionStack.pop();
            }
            /**
             * Save the block and Evaluate the node
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function pushBlock(node) {
                const len = ++functionStack[functionStack.length - 1];
                if (len > maxDepth) {
                    context.report({ node, messageId: "tooDeeply", data: { depth: len, maxDepth } });
                }
            }
            /**
             * Pop the saved block
             * @returns {void}
             * @private
             */
            function popBlock() {
                functionStack[functionStack.length - 1]--;
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                Program: startFunction,
                FunctionDeclaration: startFunction,
                FunctionExpression: startFunction,
                ArrowFunctionExpression: startFunction,
                IfStatement(node) {
                    if (node.parent.type !== "IfStatement") {
                        pushBlock(node);
                    }
                },
                SwitchStatement: pushBlock,
                TryStatement: pushBlock,
                DoWhileStatement: pushBlock,
                WhileStatement: pushBlock,
                WithStatement: pushBlock,
                ForStatement: pushBlock,
                ForInStatement: pushBlock,
                ForOfStatement: pushBlock,
                "IfStatement:exit": popBlock,
                "SwitchStatement:exit": popBlock,
                "TryStatement:exit": popBlock,
                "DoWhileStatement:exit": popBlock,
                "WhileStatement:exit": popBlock,
                "WithStatement:exit": popBlock,
                "ForStatement:exit": popBlock,
                "ForInStatement:exit": popBlock,
                "ForOfStatement:exit": popBlock,
                "FunctionDeclaration:exit": endFunction,
                "FunctionExpression:exit": endFunction,
                "ArrowFunctionExpression:exit": endFunction,
                "Program:exit": endFunction
            };
        }
    };
},
/* eslint/lib/rules/max-len.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Constants
    //------------------------------------------------------------------------------
    const OPTIONS_SCHEMA = {
        type: "object",
        properties: {
            code: {
                type: "integer",
                minimum: 0
            },
            comments: {
                type: "integer",
                minimum: 0
            },
            tabWidth: {
                type: "integer",
                minimum: 0
            },
            ignorePattern: {
                type: "string"
            },
            ignoreComments: {
                type: "boolean"
            },
            ignoreStrings: {
                type: "boolean"
            },
            ignoreUrls: {
                type: "boolean"
            },
            ignoreTemplateLiterals: {
                type: "boolean"
            },
            ignoreRegExpLiterals: {
                type: "boolean"
            },
            ignoreTrailingComments: {
                type: "boolean"
            }
        },
        additionalProperties: false
    };
    const OPTIONS_OR_INTEGER_SCHEMA = {
        anyOf: [
            OPTIONS_SCHEMA,
            {
                type: "integer",
                minimum: 0
            }
        ]
    };
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce a maximum line length",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/max-len"
            },
            schema: [
                OPTIONS_OR_INTEGER_SCHEMA,
                OPTIONS_OR_INTEGER_SCHEMA,
                OPTIONS_SCHEMA
            ],
            messages: {
                max: "This line has a length of {{lineLength}}. Maximum allowed is {{maxLength}}.",
                maxComment: "This line has a comment length of {{lineLength}}. Maximum allowed is {{maxCommentLength}}."
            }
        },
        create(context) {
            /*
             * Inspired by http://tools.ietf.org/html/rfc3986#appendix-B, however:
             * - They're matching an entire string that we know is a URI
             * - We're matching part of a string where we think there *might* be a URL
             * - We're only concerned about URLs, as picking out any URI would cause
             *   too many false positives
             * - We don't care about matching the entire URL, any small segment is fine
             */
            const URL_REGEXP = /[^:/?#]:\/\/[^?#]/u;
            const sourceCode = context.getSourceCode();
            /**
             * Computes the length of a line that may contain tabs. The width of each
             * tab will be the number of spaces to the next tab stop.
             * @param {string} line The line.
             * @param {int} tabWidth The width of each tab stop in spaces.
             * @returns {int} The computed line length.
             * @private
             */
            function computeLineLength(line, tabWidth) {
                let extraCharacterCount = 0;
                line.replace(/\t/gu, (match, offset) => {
                    const totalOffset = offset + extraCharacterCount, previousTabStopOffset = tabWidth ? totalOffset % tabWidth : 0, spaceCount = tabWidth - previousTabStopOffset;
                    extraCharacterCount += spaceCount - 1; // -1 for the replaced tab
                });
                return Array.from(line).length + extraCharacterCount;
            }
            // The options object must be the last option specified…
            const options = Object.assign({}, context.options[context.options.length - 1]);
            // …but max code length…
            if (typeof context.options[0] === "number") {
                options.code = context.options[0];
            }
            // …and tabWidth can be optionally specified directly as integers.
            if (typeof context.options[1] === "number") {
                options.tabWidth = context.options[1];
            }
            const maxLength = typeof options.code === "number" ? options.code : 80, tabWidth = typeof options.tabWidth === "number" ? options.tabWidth : 4, ignoreComments = !!options.ignoreComments, ignoreStrings = !!options.ignoreStrings, ignoreTemplateLiterals = !!options.ignoreTemplateLiterals, ignoreRegExpLiterals = !!options.ignoreRegExpLiterals, ignoreTrailingComments = !!options.ignoreTrailingComments || !!options.ignoreComments, ignoreUrls = !!options.ignoreUrls, maxCommentLength = options.comments;
            let ignorePattern = options.ignorePattern || null;
            if (ignorePattern) {
                ignorePattern = new RegExp(ignorePattern, "u");
            }
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Tells if a given comment is trailing: it starts on the current line and
             * extends to or past the end of the current line.
             * @param {string} line The source line we want to check for a trailing comment on
             * @param {number} lineNumber The one-indexed line number for line
             * @param {ASTNode} comment The comment to inspect
             * @returns {boolean} If the comment is trailing on the given line
             */
            function isTrailingComment(line, lineNumber, comment) {
                return comment &&
                    (comment.loc.start.line === lineNumber && lineNumber <= comment.loc.end.line) &&
                    (comment.loc.end.line > lineNumber || comment.loc.end.column === line.length);
            }
            /**
             * Tells if a comment encompasses the entire line.
             * @param {string} line The source line with a trailing comment
             * @param {number} lineNumber The one-indexed line number this is on
             * @param {ASTNode} comment The comment to remove
             * @returns {boolean} If the comment covers the entire line
             */
            function isFullLineComment(line, lineNumber, comment) {
                const start = comment.loc.start, end = comment.loc.end, isFirstTokenOnLine = !line.slice(0, comment.loc.start.column).trim();
                return comment &&
                    (start.line < lineNumber || (start.line === lineNumber && isFirstTokenOnLine)) &&
                    (end.line > lineNumber || (end.line === lineNumber && end.column === line.length));
            }
            /**
             * Gets the line after the comment and any remaining trailing whitespace is
             * stripped.
             * @param {string} line The source line with a trailing comment
             * @param {ASTNode} comment The comment to remove
             * @returns {string} Line without comment and trailing whitepace
             */
            function stripTrailingComment(line, comment) {
                // loc.column is zero-indexed
                return line.slice(0, comment.loc.start.column).replace(/\s+$/u, "");
            }
            /**
             * Ensure that an array exists at [key] on `object`, and add `value` to it.
             * @param {Object} object the object to mutate
             * @param {string} key the object's key
             * @param {*} value the value to add
             * @returns {void}
             * @private
             */
            function ensureArrayAndPush(object, key, value) {
                if (!Array.isArray(object[key])) {
                    object[key] = [];
                }
                object[key].push(value);
            }
            /**
             * Retrieves an array containing all strings (" or ') in the source code.
             * @returns {ASTNode[]} An array of string nodes.
             */
            function getAllStrings() {
                return sourceCode.ast.tokens.filter(token => (token.type === "String" ||
                    (token.type === "JSXText" && sourceCode.getNodeByRangeIndex(token.range[0] - 1).type === "JSXAttribute")));
            }
            /**
             * Retrieves an array containing all template literals in the source code.
             * @returns {ASTNode[]} An array of template literal nodes.
             */
            function getAllTemplateLiterals() {
                return sourceCode.ast.tokens.filter(token => token.type === "Template");
            }
            /**
             * Retrieves an array containing all RegExp literals in the source code.
             * @returns {ASTNode[]} An array of RegExp literal nodes.
             */
            function getAllRegExpLiterals() {
                return sourceCode.ast.tokens.filter(token => token.type === "RegularExpression");
            }
            /**
             * A reducer to group an AST node by line number, both start and end.
             * @param {Object} acc the accumulator
             * @param {ASTNode} node the AST node in question
             * @returns {Object} the modified accumulator
             * @private
             */
            function groupByLineNumber(acc, node) {
                for (let i = node.loc.start.line; i <= node.loc.end.line; ++i) {
                    ensureArrayAndPush(acc, i, node);
                }
                return acc;
            }
            /**
             * Check the program for max length
             * @param {ASTNode} node Node to examine
             * @returns {void}
             * @private
             */
            function checkProgramForMaxLength(node) {
                // split (honors line-ending)
                const lines = sourceCode.lines, 
                // list of comments to ignore
                comments = ignoreComments || maxCommentLength || ignoreTrailingComments ? sourceCode.getAllComments() : [];
                // we iterate over comments in parallel with the lines
                let commentsIndex = 0;
                const strings = getAllStrings();
                const stringsByLine = strings.reduce(groupByLineNumber, {});
                const templateLiterals = getAllTemplateLiterals();
                const templateLiteralsByLine = templateLiterals.reduce(groupByLineNumber, {});
                const regExpLiterals = getAllRegExpLiterals();
                const regExpLiteralsByLine = regExpLiterals.reduce(groupByLineNumber, {});
                lines.forEach((line, i) => {
                    // i is zero-indexed, line numbers are one-indexed
                    const lineNumber = i + 1;
                    /*
                     * if we're checking comment length; we need to know whether this
                     * line is a comment
                     */
                    let lineIsComment = false;
                    let textToMeasure;
                    /*
                     * We can short-circuit the comment checks if we're already out of
                     * comments to check.
                     */
                    if (commentsIndex < comments.length) {
                        let comment = null;
                        // iterate over comments until we find one past the current line
                        do {
                            comment = comments[++commentsIndex];
                        } while (comment && comment.loc.start.line <= lineNumber);
                        // and step back by one
                        comment = comments[--commentsIndex];
                        if (isFullLineComment(line, lineNumber, comment)) {
                            lineIsComment = true;
                            textToMeasure = line;
                        }
                        else if (ignoreTrailingComments && isTrailingComment(line, lineNumber, comment)) {
                            textToMeasure = stripTrailingComment(line, comment);
                            // ignore multiple trailing comments in the same line
                            let lastIndex = commentsIndex;
                            while (isTrailingComment(textToMeasure, lineNumber, comments[--lastIndex])) {
                                textToMeasure = stripTrailingComment(textToMeasure, comments[lastIndex]);
                            }
                        }
                        else {
                            textToMeasure = line;
                        }
                    }
                    else {
                        textToMeasure = line;
                    }
                    if (ignorePattern && ignorePattern.test(textToMeasure) ||
                        ignoreUrls && URL_REGEXP.test(textToMeasure) ||
                        ignoreStrings && stringsByLine[lineNumber] ||
                        ignoreTemplateLiterals && templateLiteralsByLine[lineNumber] ||
                        ignoreRegExpLiterals && regExpLiteralsByLine[lineNumber]) {
                        // ignore this line
                        return;
                    }
                    const lineLength = computeLineLength(textToMeasure, tabWidth);
                    const commentLengthApplies = lineIsComment && maxCommentLength;
                    if (lineIsComment && ignoreComments) {
                        return;
                    }
                    if (commentLengthApplies) {
                        if (lineLength > maxCommentLength) {
                            context.report({
                                node,
                                loc: { line: lineNumber, column: 0 },
                                messageId: "maxComment",
                                data: {
                                    lineLength,
                                    maxCommentLength
                                }
                            });
                        }
                    }
                    else if (lineLength > maxLength) {
                        context.report({
                            node,
                            loc: { line: lineNumber, column: 0 },
                            messageId: "max",
                            data: {
                                lineLength,
                                maxLength
                            }
                        });
                    }
                });
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                Program: checkProgramForMaxLength
            };
        }
    };
},
/* eslint/lib/rules/max-lines.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce a maximum number of lines per file",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/max-lines"
            },
            schema: [
                {
                    oneOf: [
                        {
                            type: "integer",
                            minimum: 0
                        },
                        {
                            type: "object",
                            properties: {
                                max: {
                                    type: "integer",
                                    minimum: 0
                                },
                                skipComments: {
                                    type: "boolean"
                                },
                                skipBlankLines: {
                                    type: "boolean"
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                exceed: "File has too many lines ({{actual}}). Maximum allowed is {{max}}."
            }
        },
        create(context) {
            const option = context.options[0];
            let max = 300;
            if (typeof option === "object" && Object.prototype.hasOwnProperty.call(option, "max")) {
                max = option.max;
            }
            else if (typeof option === "number") {
                max = option;
            }
            const skipComments = option && option.skipComments;
            const skipBlankLines = option && option.skipBlankLines;
            const sourceCode = context.getSourceCode();
            /**
             * Returns whether or not a token is a comment node type
             * @param {Token} token The token to check
             * @returns {boolean} True if the token is a comment node
             */
            function isCommentNodeType(token) {
                return token && (token.type === "Block" || token.type === "Line");
            }
            /**
             * Returns the line numbers of a comment that don't have any code on the same line
             * @param {Node} comment The comment node to check
             * @returns {number[]} The line numbers
             */
            function getLinesWithoutCode(comment) {
                let start = comment.loc.start.line;
                let end = comment.loc.end.line;
                let token;
                token = comment;
                do {
                    token = sourceCode.getTokenBefore(token, { includeComments: true });
                } while (isCommentNodeType(token));
                if (token && astUtils.isTokenOnSameLine(token, comment)) {
                    start += 1;
                }
                token = comment;
                do {
                    token = sourceCode.getTokenAfter(token, { includeComments: true });
                } while (isCommentNodeType(token));
                if (token && astUtils.isTokenOnSameLine(comment, token)) {
                    end -= 1;
                }
                if (start <= end) {
                    return lodash.range(start, end + 1);
                }
                return [];
            }
            return {
                "Program:exit"() {
                    let lines = sourceCode.lines.map((text, i) => ({ lineNumber: i + 1, text }));
                    if (skipBlankLines) {
                        lines = lines.filter(l => l.text.trim() !== "");
                    }
                    if (skipComments) {
                        const comments = sourceCode.getAllComments();
                        const commentLines = lodash.flatten(comments.map(comment => getLinesWithoutCode(comment)));
                        lines = lines.filter(l => !lodash.includes(commentLines, l.lineNumber));
                    }
                    if (lines.length > max) {
                        context.report({
                            loc: { line: 1, column: 0 },
                            messageId: "exceed",
                            data: {
                                max,
                                actual: lines.length
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/max-lines-per-function.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    const lodash = require(111) /* lodash */;
    //------------------------------------------------------------------------------
    // Constants
    //------------------------------------------------------------------------------
    const OPTIONS_SCHEMA = {
        type: "object",
        properties: {
            max: {
                type: "integer",
                minimum: 0
            },
            skipComments: {
                type: "boolean"
            },
            skipBlankLines: {
                type: "boolean"
            },
            IIFEs: {
                type: "boolean"
            }
        },
        additionalProperties: false
    };
    const OPTIONS_OR_INTEGER_SCHEMA = {
        oneOf: [
            OPTIONS_SCHEMA,
            {
                type: "integer",
                minimum: 1
            }
        ]
    };
    /**
     * Given a list of comment nodes, return a map with numeric keys (source code line numbers) and comment token values.
     * @param {Array} comments An array of comment nodes.
     * @returns {Map.<string,Node>} A map with numeric keys (source code line numbers) and comment token values.
     */
    function getCommentLineNumbers(comments) {
        const map = new Map();
        if (!comments) {
            return map;
        }
        comments.forEach(comment => {
            for (let i = comment.loc.start.line; i <= comment.loc.end.line; i++) {
                map.set(i, comment);
            }
        });
        return map;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce a maximum number of line of code in a function",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/max-lines-per-function"
            },
            schema: [
                OPTIONS_OR_INTEGER_SCHEMA
            ],
            messages: {
                exceed: "{{name}} has too many lines ({{lineCount}}). Maximum allowed is {{maxLines}}."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const lines = sourceCode.lines;
            const option = context.options[0];
            let maxLines = 50;
            let skipComments = false;
            let skipBlankLines = false;
            let IIFEs = false;
            if (typeof option === "object") {
                maxLines = typeof option.max === "number" ? option.max : 50;
                skipComments = !!option.skipComments;
                skipBlankLines = !!option.skipBlankLines;
                IIFEs = !!option.IIFEs;
            }
            else if (typeof option === "number") {
                maxLines = option;
            }
            const commentLineNumbers = getCommentLineNumbers(sourceCode.getAllComments());
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Tells if a comment encompasses the entire line.
             * @param {string} line The source line with a trailing comment
             * @param {number} lineNumber The one-indexed line number this is on
             * @param {ASTNode} comment The comment to remove
             * @returns {boolean} If the comment covers the entire line
             */
            function isFullLineComment(line, lineNumber, comment) {
                const start = comment.loc.start, end = comment.loc.end, isFirstTokenOnLine = start.line === lineNumber && !line.slice(0, start.column).trim(), isLastTokenOnLine = end.line === lineNumber && !line.slice(end.column).trim();
                return comment &&
                    (start.line < lineNumber || isFirstTokenOnLine) &&
                    (end.line > lineNumber || isLastTokenOnLine);
            }
            /**
             * Identifies is a node is a FunctionExpression which is part of an IIFE
             * @param {ASTNode} node Node to test
             * @returns {boolean} True if it's an IIFE
             */
            function isIIFE(node) {
                return node.type === "FunctionExpression" && node.parent && node.parent.type === "CallExpression" && node.parent.callee === node;
            }
            /**
             * Identifies is a node is a FunctionExpression which is embedded within a MethodDefinition or Property
             * @param {ASTNode} node Node to test
             * @returns {boolean} True if it's a FunctionExpression embedded within a MethodDefinition or Property
             */
            function isEmbedded(node) {
                if (!node.parent) {
                    return false;
                }
                if (node !== node.parent.value) {
                    return false;
                }
                if (node.parent.type === "MethodDefinition") {
                    return true;
                }
                if (node.parent.type === "Property") {
                    return node.parent.method === true || node.parent.kind === "get" || node.parent.kind === "set";
                }
                return false;
            }
            /**
             * Count the lines in the function
             * @param {ASTNode} funcNode Function AST node
             * @returns {void}
             * @private
             */
            function processFunction(funcNode) {
                const node = isEmbedded(funcNode) ? funcNode.parent : funcNode;
                if (!IIFEs && isIIFE(node)) {
                    return;
                }
                let lineCount = 0;
                for (let i = node.loc.start.line - 1; i < node.loc.end.line; ++i) {
                    const line = lines[i];
                    if (skipComments) {
                        if (commentLineNumbers.has(i + 1) && isFullLineComment(line, i + 1, commentLineNumbers.get(i + 1))) {
                            continue;
                        }
                    }
                    if (skipBlankLines) {
                        if (line.match(/^\s*$/u)) {
                            continue;
                        }
                    }
                    lineCount++;
                }
                if (lineCount > maxLines) {
                    const name = lodash.upperFirst(astUtils.getFunctionNameWithKind(funcNode));
                    context.report({
                        node,
                        messageId: "exceed",
                        data: { name, lineCount, maxLines }
                    });
                }
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                FunctionDeclaration: processFunction,
                FunctionExpression: processFunction,
                ArrowFunctionExpression: processFunction
            };
        }
    };
},
/* eslint/lib/rules/max-nested-callbacks.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce a maximum depth that callbacks can be nested",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/max-nested-callbacks"
            },
            schema: [
                {
                    oneOf: [
                        {
                            type: "integer",
                            minimum: 0
                        },
                        {
                            type: "object",
                            properties: {
                                maximum: {
                                    type: "integer",
                                    minimum: 0
                                },
                                max: {
                                    type: "integer",
                                    minimum: 0
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                exceed: "Too many nested callbacks ({{num}}). Maximum allowed is {{max}}."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Constants
            //--------------------------------------------------------------------------
            const option = context.options[0];
            let THRESHOLD = 10;
            if (typeof option === "object" &&
                (Object.prototype.hasOwnProperty.call(option, "maximum") || Object.prototype.hasOwnProperty.call(option, "max"))) {
                THRESHOLD = option.maximum || option.max;
            }
            else if (typeof option === "number") {
                THRESHOLD = option;
            }
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            const callbackStack = [];
            /**
             * Checks a given function node for too many callbacks.
             * @param {ASTNode} node The node to check.
             * @returns {void}
             * @private
             */
            function checkFunction(node) {
                const parent = node.parent;
                if (parent.type === "CallExpression") {
                    callbackStack.push(node);
                }
                if (callbackStack.length > THRESHOLD) {
                    const opts = { num: callbackStack.length, max: THRESHOLD };
                    context.report({ node, messageId: "exceed", data: opts });
                }
            }
            /**
             * Pops the call stack.
             * @returns {void}
             * @private
             */
            function popStack() {
                callbackStack.pop();
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                ArrowFunctionExpression: checkFunction,
                "ArrowFunctionExpression:exit": popStack,
                FunctionExpression: checkFunction,
                "FunctionExpression:exit": popStack
            };
        }
    };
},
/* eslint/lib/rules/max-params.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce a maximum number of parameters in function definitions",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/max-params"
            },
            schema: [
                {
                    oneOf: [
                        {
                            type: "integer",
                            minimum: 0
                        },
                        {
                            type: "object",
                            properties: {
                                maximum: {
                                    type: "integer",
                                    minimum: 0
                                },
                                max: {
                                    type: "integer",
                                    minimum: 0
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                exceed: "{{name}} has too many parameters ({{count}}). Maximum allowed is {{max}}."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const option = context.options[0];
            let numParams = 3;
            if (typeof option === "object" &&
                (Object.prototype.hasOwnProperty.call(option, "maximum") || Object.prototype.hasOwnProperty.call(option, "max"))) {
                numParams = option.maximum || option.max;
            }
            if (typeof option === "number") {
                numParams = option;
            }
            /**
             * Checks a function to see if it has too many parameters.
             * @param {ASTNode} node The node to check.
             * @returns {void}
             * @private
             */
            function checkFunction(node) {
                if (node.params.length > numParams) {
                    context.report({
                        loc: astUtils.getFunctionHeadLoc(node, sourceCode),
                        node,
                        messageId: "exceed",
                        data: {
                            name: lodash.upperFirst(astUtils.getFunctionNameWithKind(node)),
                            count: node.params.length,
                            max: numParams
                        }
                    });
                }
            }
            return {
                FunctionDeclaration: checkFunction,
                ArrowFunctionExpression: checkFunction,
                FunctionExpression: checkFunction
            };
        }
    };
},
/* eslint/lib/rules/max-statements.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce a maximum number of statements allowed in function blocks",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/max-statements"
            },
            schema: [
                {
                    oneOf: [
                        {
                            type: "integer",
                            minimum: 0
                        },
                        {
                            type: "object",
                            properties: {
                                maximum: {
                                    type: "integer",
                                    minimum: 0
                                },
                                max: {
                                    type: "integer",
                                    minimum: 0
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                },
                {
                    type: "object",
                    properties: {
                        ignoreTopLevelFunctions: {
                            type: "boolean"
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                exceed: "{{name}} has too many statements ({{count}}). Maximum allowed is {{max}}."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            const functionStack = [], option = context.options[0], ignoreTopLevelFunctions = context.options[1] && context.options[1].ignoreTopLevelFunctions || false, topLevelFunctions = [];
            let maxStatements = 10;
            if (typeof option === "object" &&
                (Object.prototype.hasOwnProperty.call(option, "maximum") || Object.prototype.hasOwnProperty.call(option, "max"))) {
                maxStatements = option.maximum || option.max;
            }
            else if (typeof option === "number") {
                maxStatements = option;
            }
            /**
             * Reports a node if it has too many statements
             * @param {ASTNode} node node to evaluate
             * @param {int} count Number of statements in node
             * @param {int} max Maximum number of statements allowed
             * @returns {void}
             * @private
             */
            function reportIfTooManyStatements(node, count, max) {
                if (count > max) {
                    const name = lodash.upperFirst(astUtils.getFunctionNameWithKind(node));
                    context.report({
                        node,
                        messageId: "exceed",
                        data: { name, count, max }
                    });
                }
            }
            /**
             * When parsing a new function, store it in our function stack
             * @returns {void}
             * @private
             */
            function startFunction() {
                functionStack.push(0);
            }
            /**
             * Evaluate the node at the end of function
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function endFunction(node) {
                const count = functionStack.pop();
                if (ignoreTopLevelFunctions && functionStack.length === 0) {
                    topLevelFunctions.push({ node, count });
                }
                else {
                    reportIfTooManyStatements(node, count, maxStatements);
                }
            }
            /**
             * Increment the count of the functions
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function countStatements(node) {
                functionStack[functionStack.length - 1] += node.body.length;
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                FunctionDeclaration: startFunction,
                FunctionExpression: startFunction,
                ArrowFunctionExpression: startFunction,
                BlockStatement: countStatements,
                "FunctionDeclaration:exit": endFunction,
                "FunctionExpression:exit": endFunction,
                "ArrowFunctionExpression:exit": endFunction,
                "Program:exit"() {
                    if (topLevelFunctions.length === 1) {
                        return;
                    }
                    topLevelFunctions.forEach(element => {
                        const count = element.count;
                        const node = element.node;
                        reportIfTooManyStatements(node, count, maxStatements);
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/max-statements-per-line.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce a maximum number of statements allowed per line",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/max-statements-per-line"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        max: {
                            type: "integer",
                            minimum: 1,
                            default: 1
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                exceed: "This line has {{numberOfStatementsOnThisLine}} {{statements}}. Maximum allowed is {{maxStatementsPerLine}}."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode(), options = context.options[0] || {}, maxStatementsPerLine = typeof options.max !== "undefined" ? options.max : 1;
            let lastStatementLine = 0, numberOfStatementsOnThisLine = 0, firstExtraStatement;
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            const SINGLE_CHILD_ALLOWED = /^(?:(?:DoWhile|For|ForIn|ForOf|If|Labeled|While)Statement|Export(?:Default|Named)Declaration)$/u;
            /**
             * Reports with the first extra statement, and clears it.
             * @returns {void}
             */
            function reportFirstExtraStatementAndClear() {
                if (firstExtraStatement) {
                    context.report({
                        node: firstExtraStatement,
                        messageId: "exceed",
                        data: {
                            numberOfStatementsOnThisLine,
                            maxStatementsPerLine,
                            statements: numberOfStatementsOnThisLine === 1 ? "statement" : "statements"
                        }
                    });
                }
                firstExtraStatement = null;
            }
            /**
             * Gets the actual last token of a given node.
             * @param {ASTNode} node A node to get. This is a node except EmptyStatement.
             * @returns {Token} The actual last token.
             */
            function getActualLastToken(node) {
                return sourceCode.getLastToken(node, astUtils.isNotSemicolonToken);
            }
            /**
             * Addresses a given node.
             * It updates the state of this rule, then reports the node if the node violated this rule.
             * @param {ASTNode} node A node to check.
             * @returns {void}
             */
            function enterStatement(node) {
                const line = node.loc.start.line;
                /*
                 * Skip to allow non-block statements if this is direct child of control statements.
                 * `if (a) foo();` is counted as 1.
                 * But `if (a) foo(); else foo();` should be counted as 2.
                 */
                if (SINGLE_CHILD_ALLOWED.test(node.parent.type) &&
                    node.parent.alternate !== node) {
                    return;
                }
                // Update state.
                if (line === lastStatementLine) {
                    numberOfStatementsOnThisLine += 1;
                }
                else {
                    reportFirstExtraStatementAndClear();
                    numberOfStatementsOnThisLine = 1;
                    lastStatementLine = line;
                }
                // Reports if the node violated this rule.
                if (numberOfStatementsOnThisLine === maxStatementsPerLine + 1) {
                    firstExtraStatement = firstExtraStatement || node;
                }
            }
            /**
             * Updates the state of this rule with the end line of leaving node to check with the next statement.
             * @param {ASTNode} node A node to check.
             * @returns {void}
             */
            function leaveStatement(node) {
                const line = getActualLastToken(node).loc.end.line;
                // Update state.
                if (line !== lastStatementLine) {
                    reportFirstExtraStatementAndClear();
                    numberOfStatementsOnThisLine = 1;
                    lastStatementLine = line;
                }
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                BreakStatement: enterStatement,
                ClassDeclaration: enterStatement,
                ContinueStatement: enterStatement,
                DebuggerStatement: enterStatement,
                DoWhileStatement: enterStatement,
                ExpressionStatement: enterStatement,
                ForInStatement: enterStatement,
                ForOfStatement: enterStatement,
                ForStatement: enterStatement,
                FunctionDeclaration: enterStatement,
                IfStatement: enterStatement,
                ImportDeclaration: enterStatement,
                LabeledStatement: enterStatement,
                ReturnStatement: enterStatement,
                SwitchStatement: enterStatement,
                ThrowStatement: enterStatement,
                TryStatement: enterStatement,
                VariableDeclaration: enterStatement,
                WhileStatement: enterStatement,
                WithStatement: enterStatement,
                ExportNamedDeclaration: enterStatement,
                ExportDefaultDeclaration: enterStatement,
                ExportAllDeclaration: enterStatement,
                "BreakStatement:exit": leaveStatement,
                "ClassDeclaration:exit": leaveStatement,
                "ContinueStatement:exit": leaveStatement,
                "DebuggerStatement:exit": leaveStatement,
                "DoWhileStatement:exit": leaveStatement,
                "ExpressionStatement:exit": leaveStatement,
                "ForInStatement:exit": leaveStatement,
                "ForOfStatement:exit": leaveStatement,
                "ForStatement:exit": leaveStatement,
                "FunctionDeclaration:exit": leaveStatement,
                "IfStatement:exit": leaveStatement,
                "ImportDeclaration:exit": leaveStatement,
                "LabeledStatement:exit": leaveStatement,
                "ReturnStatement:exit": leaveStatement,
                "SwitchStatement:exit": leaveStatement,
                "ThrowStatement:exit": leaveStatement,
                "TryStatement:exit": leaveStatement,
                "VariableDeclaration:exit": leaveStatement,
                "WhileStatement:exit": leaveStatement,
                "WithStatement:exit": leaveStatement,
                "ExportNamedDeclaration:exit": leaveStatement,
                "ExportDefaultDeclaration:exit": leaveStatement,
                "ExportAllDeclaration:exit": leaveStatement,
                "Program:exit": reportFirstExtraStatementAndClear
            };
        }
    };
},
/* eslint/lib/rules/multiline-comment-style.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce a particular style for multiline comments",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/multiline-comment-style"
            },
            fixable: "whitespace",
            schema: [{ enum: ["starred-block", "separate-lines", "bare-block"] }],
            messages: {
                expectedBlock: "Expected a block comment instead of consecutive line comments.",
                expectedBareBlock: "Expected a block comment without padding stars.",
                startNewline: "Expected a linebreak after '/*'.",
                endNewline: "Expected a linebreak before '*/'.",
                missingStar: "Expected a '*' at the start of this line.",
                alignment: "Expected this line to be aligned with the start of the comment.",
                expectedLines: "Expected multiple line comments instead of a block comment."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const option = context.options[0] || "starred-block";
            //----------------------------------------------------------------------
            // Helpers
            //----------------------------------------------------------------------
            /**
             * Checks if a comment line is starred.
             * @param {string} line A string representing a comment line.
             * @returns {boolean} Whether or not the comment line is starred.
             */
            function isStarredCommentLine(line) {
                return /^\s*\*/u.test(line);
            }
            /**
             * Checks if a comment group is in starred-block form.
             * @param {Token[]} commentGroup A group of comments, containing either multiple line comments or a single block comment.
             * @returns {boolean} Whether or not the comment group is in starred block form.
             */
            function isStarredBlockComment([firstComment]) {
                if (firstComment.type !== "Block") {
                    return false;
                }
                const lines = firstComment.value.split(astUtils.LINEBREAK_MATCHER);
                // The first and last lines can only contain whitespace.
                return lines.length > 0 && lines.every((line, i) => (i === 0 || i === lines.length - 1 ? /^\s*$/u : /^\s*\*/u).test(line));
            }
            /**
             * Checks if a comment group is in JSDoc form.
             * @param {Token[]} commentGroup A group of comments, containing either multiple line comments or a single block comment.
             * @returns {boolean} Whether or not the comment group is in JSDoc form.
             */
            function isJSDocComment([firstComment]) {
                if (firstComment.type !== "Block") {
                    return false;
                }
                const lines = firstComment.value.split(astUtils.LINEBREAK_MATCHER);
                return /^\*\s*$/u.test(lines[0]) &&
                    lines.slice(1, -1).every(line => /^\s* /u.test(line)) &&
                    /^\s*$/u.test(lines[lines.length - 1]);
            }
            /**
             * Processes a comment group that is currently in separate-line form, calculating the offset for each line.
             * @param {Token[]} commentGroup A group of comments containing multiple line comments.
             * @returns {string[]} An array of the processed lines.
             */
            function processSeparateLineComments(commentGroup) {
                const allLinesHaveLeadingSpace = commentGroup
                    .map(({ value }) => value)
                    .filter(line => line.trim().length)
                    .every(line => line.startsWith(" "));
                return commentGroup.map(({ value }) => (allLinesHaveLeadingSpace ? value.replace(/^ /u, "") : value));
            }
            /**
             * Processes a comment group that is currently in starred-block form, calculating the offset for each line.
             * @param {Token} comment A single block comment token in starred-block form.
             * @returns {string[]} An array of the processed lines.
             */
            function processStarredBlockComment(comment) {
                const lines = comment.value.split(astUtils.LINEBREAK_MATCHER)
                    .filter((line, i, linesArr) => !(i === 0 || i === linesArr.length - 1))
                    .map(line => line.replace(/^\s*$/u, ""));
                const allLinesHaveLeadingSpace = lines
                    .map(line => line.replace(/\s*\*/u, ""))
                    .filter(line => line.trim().length)
                    .every(line => line.startsWith(" "));
                return lines.map(line => line.replace(allLinesHaveLeadingSpace ? /\s*\* ?/u : /\s*\*/u, ""));
            }
            /**
             * Processes a comment group that is currently in bare-block form, calculating the offset for each line.
             * @param {Token} comment A single block comment token in bare-block form.
             * @returns {string[]} An array of the processed lines.
             */
            function processBareBlockComment(comment) {
                const lines = comment.value.split(astUtils.LINEBREAK_MATCHER).map(line => line.replace(/^\s*$/u, ""));
                const leadingWhitespace = `${sourceCode.text.slice(comment.range[0] - comment.loc.start.column, comment.range[0])}   `;
                let offset = "";
                /*
                 * Calculate the offset of the least indented line and use that as the basis for offsetting all the lines.
                 * The first line should not be checked because it is inline with the opening block comment delimiter.
                 */
                for (const [i, line] of lines.entries()) {
                    if (!line.trim().length || i === 0) {
                        continue;
                    }
                    const [, lineOffset] = line.match(/^(\s*\*?\s*)/u);
                    if (lineOffset.length < leadingWhitespace.length) {
                        const newOffset = leadingWhitespace.slice(lineOffset.length - leadingWhitespace.length);
                        if (newOffset.length > offset.length) {
                            offset = newOffset;
                        }
                    }
                }
                return lines.map(line => {
                    const match = line.match(/^(\s*\*?\s*)(.*)/u);
                    const [, lineOffset, lineContents] = match;
                    if (lineOffset.length > leadingWhitespace.length) {
                        return `${lineOffset.slice(leadingWhitespace.length - (offset.length + lineOffset.length))}${lineContents}`;
                    }
                    if (lineOffset.length < leadingWhitespace.length) {
                        return `${lineOffset.slice(leadingWhitespace.length)}${lineContents}`;
                    }
                    return lineContents;
                });
            }
            /**
             * Gets a list of comment lines in a group, formatting leading whitespace as necessary.
             * @param {Token[]} commentGroup A group of comments containing either multiple line comments or a single block comment.
             * @returns {string[]} A list of comment lines.
             */
            function getCommentLines(commentGroup) {
                const [firstComment] = commentGroup;
                if (firstComment.type === "Line") {
                    return processSeparateLineComments(commentGroup);
                }
                if (isStarredBlockComment(commentGroup)) {
                    return processStarredBlockComment(firstComment);
                }
                return processBareBlockComment(firstComment);
            }
            /**
             * Gets the initial offset (whitespace) from the beginning of a line to a given comment token.
             * @param {Token} comment The token to check.
             * @returns {string} The offset from the beginning of a line to the token.
             */
            function getInitialOffset(comment) {
                return sourceCode.text.slice(comment.range[0] - comment.loc.start.column, comment.range[0]);
            }
            /**
             * Converts a comment into starred-block form
             * @param {Token} firstComment The first comment of the group being converted
             * @param {string[]} commentLinesList A list of lines to appear in the new starred-block comment
             * @returns {string} A representation of the comment value in starred-block form, excluding start and end markers
             */
            function convertToStarredBlock(firstComment, commentLinesList) {
                const initialOffset = getInitialOffset(firstComment);
                return `/*\n${commentLinesList.map(line => `${initialOffset} * ${line}`).join("\n")}\n${initialOffset} */`;
            }
            /**
             * Converts a comment into separate-line form
             * @param {Token} firstComment The first comment of the group being converted
             * @param {string[]} commentLinesList A list of lines to appear in the new starred-block comment
             * @returns {string} A representation of the comment value in separate-line form
             */
            function convertToSeparateLines(firstComment, commentLinesList) {
                return commentLinesList.map(line => `// ${line}`).join(`\n${getInitialOffset(firstComment)}`);
            }
            /**
             * Converts a comment into bare-block form
             * @param {Token} firstComment The first comment of the group being converted
             * @param {string[]} commentLinesList A list of lines to appear in the new starred-block comment
             * @returns {string} A representation of the comment value in bare-block form
             */
            function convertToBlock(firstComment, commentLinesList) {
                return `/* ${commentLinesList.join(`\n${getInitialOffset(firstComment)}   `)} */`;
            }
            /**
             * Each method checks a group of comments to see if it's valid according to the given option.
             * @param {Token[]} commentGroup A list of comments that appear together. This will either contain a single
             * block comment or multiple line comments.
             * @returns {void}
             */
            const commentGroupCheckers = {
                "starred-block"(commentGroup) {
                    const [firstComment] = commentGroup;
                    const commentLines = getCommentLines(commentGroup);
                    if (commentLines.some(value => value.includes("*/"))) {
                        return;
                    }
                    if (commentGroup.length > 1) {
                        context.report({
                            loc: {
                                start: firstComment.loc.start,
                                end: commentGroup[commentGroup.length - 1].loc.end
                            },
                            messageId: "expectedBlock",
                            fix(fixer) {
                                const range = [firstComment.range[0], commentGroup[commentGroup.length - 1].range[1]];
                                return commentLines.some(value => value.startsWith("/"))
                                    ? null
                                    : fixer.replaceTextRange(range, convertToStarredBlock(firstComment, commentLines));
                            }
                        });
                    }
                    else {
                        const lines = firstComment.value.split(astUtils.LINEBREAK_MATCHER);
                        const expectedLeadingWhitespace = getInitialOffset(firstComment);
                        const expectedLinePrefix = `${expectedLeadingWhitespace} *`;
                        if (!/^\*?\s*$/u.test(lines[0])) {
                            const start = firstComment.value.startsWith("*") ? firstComment.range[0] + 1 : firstComment.range[0];
                            context.report({
                                loc: {
                                    start: firstComment.loc.start,
                                    end: { line: firstComment.loc.start.line, column: firstComment.loc.start.column + 2 }
                                },
                                messageId: "startNewline",
                                fix: fixer => fixer.insertTextAfterRange([start, start + 2], `\n${expectedLinePrefix}`)
                            });
                        }
                        if (!/^\s*$/u.test(lines[lines.length - 1])) {
                            context.report({
                                loc: {
                                    start: { line: firstComment.loc.end.line, column: firstComment.loc.end.column - 2 },
                                    end: firstComment.loc.end
                                },
                                messageId: "endNewline",
                                fix: fixer => fixer.replaceTextRange([firstComment.range[1] - 2, firstComment.range[1]], `\n${expectedLinePrefix}/`)
                            });
                        }
                        for (let lineNumber = firstComment.loc.start.line + 1; lineNumber <= firstComment.loc.end.line; lineNumber++) {
                            const lineText = sourceCode.lines[lineNumber - 1];
                            const errorType = isStarredCommentLine(lineText)
                                ? "alignment"
                                : "missingStar";
                            if (!lineText.startsWith(expectedLinePrefix)) {
                                context.report({
                                    loc: {
                                        start: { line: lineNumber, column: 0 },
                                        end: { line: lineNumber, column: lineText.length }
                                    },
                                    messageId: errorType,
                                    fix(fixer) {
                                        const lineStartIndex = sourceCode.getIndexFromLoc({ line: lineNumber, column: 0 });
                                        if (errorType === "alignment") {
                                            const [, commentTextPrefix = ""] = lineText.match(/^(\s*\*)/u) || [];
                                            const commentTextStartIndex = lineStartIndex + commentTextPrefix.length;
                                            return fixer.replaceTextRange([lineStartIndex, commentTextStartIndex], expectedLinePrefix);
                                        }
                                        const [, commentTextPrefix = ""] = lineText.match(/^(\s*)/u) || [];
                                        const commentTextStartIndex = lineStartIndex + commentTextPrefix.length;
                                        let offset;
                                        for (const [idx, line] of lines.entries()) {
                                            if (!/\S+/u.test(line)) {
                                                continue;
                                            }
                                            const lineTextToAlignWith = sourceCode.lines[firstComment.loc.start.line - 1 + idx];
                                            const [, prefix = "", initialOffset = ""] = lineTextToAlignWith.match(/^(\s*(?:\/?\*)?(\s*))/u) || [];
                                            offset = `${commentTextPrefix.slice(prefix.length)}${initialOffset}`;
                                            break;
                                        }
                                        return fixer.replaceTextRange([lineStartIndex, commentTextStartIndex], `${expectedLinePrefix}${offset}`);
                                    }
                                });
                            }
                        }
                    }
                },
                "separate-lines"(commentGroup) {
                    const [firstComment] = commentGroup;
                    if (firstComment.type !== "Block" || isJSDocComment(commentGroup)) {
                        return;
                    }
                    const commentLines = getCommentLines(commentGroup);
                    const tokenAfter = sourceCode.getTokenAfter(firstComment, { includeComments: true });
                    if (tokenAfter && firstComment.loc.end.line === tokenAfter.loc.start.line) {
                        return;
                    }
                    context.report({
                        loc: {
                            start: firstComment.loc.start,
                            end: { line: firstComment.loc.start.line, column: firstComment.loc.start.column + 2 }
                        },
                        messageId: "expectedLines",
                        fix(fixer) {
                            return fixer.replaceText(firstComment, convertToSeparateLines(firstComment, commentLines));
                        }
                    });
                },
                "bare-block"(commentGroup) {
                    if (isJSDocComment(commentGroup)) {
                        return;
                    }
                    const [firstComment] = commentGroup;
                    const commentLines = getCommentLines(commentGroup);
                    // Disallows consecutive line comments in favor of using a block comment.
                    if (firstComment.type === "Line" && commentLines.length > 1 &&
                        !commentLines.some(value => value.includes("*/"))) {
                        context.report({
                            loc: {
                                start: firstComment.loc.start,
                                end: commentGroup[commentGroup.length - 1].loc.end
                            },
                            messageId: "expectedBlock",
                            fix(fixer) {
                                return fixer.replaceTextRange([firstComment.range[0], commentGroup[commentGroup.length - 1].range[1]], convertToBlock(firstComment, commentLines));
                            }
                        });
                    }
                    // Prohibits block comments from having a * at the beginning of each line.
                    if (isStarredBlockComment(commentGroup)) {
                        context.report({
                            loc: {
                                start: firstComment.loc.start,
                                end: { line: firstComment.loc.start.line, column: firstComment.loc.start.column + 2 }
                            },
                            messageId: "expectedBareBlock",
                            fix(fixer) {
                                return fixer.replaceText(firstComment, convertToBlock(firstComment, commentLines));
                            }
                        });
                    }
                }
            };
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                Program() {
                    return sourceCode.getAllComments()
                        .filter(comment => comment.type !== "Shebang")
                        .filter(comment => !astUtils.COMMENTS_IGNORE_PATTERN.test(comment.value))
                        .filter(comment => {
                        const tokenBefore = sourceCode.getTokenBefore(comment, { includeComments: true });
                        return !tokenBefore || tokenBefore.loc.end.line < comment.loc.start.line;
                    })
                        .reduce((commentGroups, comment, index, commentList) => {
                        const tokenBefore = sourceCode.getTokenBefore(comment, { includeComments: true });
                        if (comment.type === "Line" &&
                            index && commentList[index - 1].type === "Line" &&
                            tokenBefore && tokenBefore.loc.end.line === comment.loc.start.line - 1 &&
                            tokenBefore === commentList[index - 1]) {
                            commentGroups[commentGroups.length - 1].push(comment);
                        }
                        else {
                            commentGroups.push([comment]);
                        }
                        return commentGroups;
                    }, [])
                        .filter(commentGroup => !(commentGroup.length === 1 && commentGroup[0].loc.start.line === commentGroup[0].loc.end.line))
                        .forEach(commentGroupCheckers[option]);
                }
            };
        }
    };
},
/* eslint/lib/rules/multiline-ternary.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce newlines between operands of ternary expressions",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/multiline-ternary"
            },
            schema: [
                {
                    enum: ["always", "always-multiline", "never"]
                }
            ],
            messages: {
                expectedTestCons: "Expected newline between test and consequent of ternary expression.",
                expectedConsAlt: "Expected newline between consequent and alternate of ternary expression.",
                unexpectedTestCons: "Unexpected newline between test and consequent of ternary expression.",
                unexpectedConsAlt: "Unexpected newline between consequent and alternate of ternary expression."
            }
        },
        create(context) {
            const option = context.options[0];
            const multiline = option !== "never";
            const allowSingleLine = option === "always-multiline";
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Tests whether node is preceded by supplied tokens
             * @param {ASTNode} node node to check
             * @param {ASTNode} parentNode parent of node to report
             * @param {boolean} expected whether newline was expected or not
             * @returns {void}
             * @private
             */
            function reportError(node, parentNode, expected) {
                context.report({
                    node,
                    messageId: `${expected ? "expected" : "unexpected"}${node === parentNode.test ? "TestCons" : "ConsAlt"}`
                });
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                ConditionalExpression(node) {
                    const areTestAndConsequentOnSameLine = astUtils.isTokenOnSameLine(node.test, node.consequent);
                    const areConsequentAndAlternateOnSameLine = astUtils.isTokenOnSameLine(node.consequent, node.alternate);
                    if (!multiline) {
                        if (!areTestAndConsequentOnSameLine) {
                            reportError(node.test, node, false);
                        }
                        if (!areConsequentAndAlternateOnSameLine) {
                            reportError(node.consequent, node, false);
                        }
                    }
                    else {
                        if (allowSingleLine && node.loc.start.line === node.loc.end.line) {
                            return;
                        }
                        if (areTestAndConsequentOnSameLine) {
                            reportError(node.test, node, true);
                        }
                        if (areConsequentAndAlternateOnSameLine) {
                            reportError(node.consequent, node, true);
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/new-cap.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const CAPS_ALLOWED = [
        "Array",
        "Boolean",
        "Date",
        "Error",
        "Function",
        "Number",
        "Object",
        "RegExp",
        "String",
        "Symbol",
        "BigInt"
    ];
    /**
     * Ensure that if the key is provided, it must be an array.
     * @param {Object} obj Object to check with `key`.
     * @param {string} key Object key to check on `obj`.
     * @param {*} fallback If obj[key] is not present, this will be returned.
     * @returns {string[]} Returns obj[key] if it's an Array, otherwise `fallback`
     */
    function checkArray(obj, key, fallback) {
        /* istanbul ignore if */
        if (Object.prototype.hasOwnProperty.call(obj, key) && !Array.isArray(obj[key])) {
            throw new TypeError(`${key}, if provided, must be an Array`);
        }
        return obj[key] || fallback;
    }
    /**
     * A reducer function to invert an array to an Object mapping the string form of the key, to `true`.
     * @param {Object} map Accumulator object for the reduce.
     * @param {string} key Object key to set to `true`.
     * @returns {Object} Returns the updated Object for further reduction.
     */
    function invert(map, key) {
        map[key] = true;
        return map;
    }
    /**
     * Creates an object with the cap is new exceptions as its keys and true as their values.
     * @param {Object} config Rule configuration
     * @returns {Object} Object with cap is new exceptions.
     */
    function calculateCapIsNewExceptions(config) {
        let capIsNewExceptions = checkArray(config, "capIsNewExceptions", CAPS_ALLOWED);
        if (capIsNewExceptions !== CAPS_ALLOWED) {
            capIsNewExceptions = capIsNewExceptions.concat(CAPS_ALLOWED);
        }
        return capIsNewExceptions.reduce(invert, {});
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require constructor names to begin with a capital letter",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/new-cap"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        newIsCap: {
                            type: "boolean",
                            default: true
                        },
                        capIsNew: {
                            type: "boolean",
                            default: true
                        },
                        newIsCapExceptions: {
                            type: "array",
                            items: {
                                type: "string"
                            }
                        },
                        newIsCapExceptionPattern: {
                            type: "string"
                        },
                        capIsNewExceptions: {
                            type: "array",
                            items: {
                                type: "string"
                            }
                        },
                        capIsNewExceptionPattern: {
                            type: "string"
                        },
                        properties: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                upper: "A function with a name starting with an uppercase letter should only be used as a constructor.",
                lower: "A constructor name should not start with a lowercase letter."
            }
        },
        create(context) {
            const config = Object.assign({}, context.options[0]);
            config.newIsCap = config.newIsCap !== false;
            config.capIsNew = config.capIsNew !== false;
            const skipProperties = config.properties === false;
            const newIsCapExceptions = checkArray(config, "newIsCapExceptions", []).reduce(invert, {});
            const newIsCapExceptionPattern = config.newIsCapExceptionPattern ? new RegExp(config.newIsCapExceptionPattern, "u") : null;
            const capIsNewExceptions = calculateCapIsNewExceptions(config);
            const capIsNewExceptionPattern = config.capIsNewExceptionPattern ? new RegExp(config.capIsNewExceptionPattern, "u") : null;
            const listeners = {};
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Get exact callee name from expression
             * @param {ASTNode} node CallExpression or NewExpression node
             * @returns {string} name
             */
            function extractNameFromExpression(node) {
                let name = "";
                if (node.callee.type === "MemberExpression") {
                    const property = node.callee.property;
                    if (property.type === "Literal" && (typeof property.value === "string")) {
                        name = property.value;
                    }
                    else if (property.type === "Identifier" && !node.callee.computed) {
                        name = property.name;
                    }
                }
                else {
                    name = node.callee.name;
                }
                return name;
            }
            /**
             * Returns the capitalization state of the string -
             * Whether the first character is uppercase, lowercase, or non-alphabetic
             * @param {string} str String
             * @returns {string} capitalization state: "non-alpha", "lower", or "upper"
             */
            function getCap(str) {
                const firstChar = str.charAt(0);
                const firstCharLower = firstChar.toLowerCase();
                const firstCharUpper = firstChar.toUpperCase();
                if (firstCharLower === firstCharUpper) {
                    // char has no uppercase variant, so it's non-alphabetic
                    return "non-alpha";
                }
                if (firstChar === firstCharLower) {
                    return "lower";
                }
                return "upper";
            }
            /**
             * Check if capitalization is allowed for a CallExpression
             * @param {Object} allowedMap Object mapping calleeName to a Boolean
             * @param {ASTNode} node CallExpression node
             * @param {string} calleeName Capitalized callee name from a CallExpression
             * @param {Object} pattern RegExp object from options pattern
             * @returns {boolean} Returns true if the callee may be capitalized
             */
            function isCapAllowed(allowedMap, node, calleeName, pattern) {
                const sourceText = sourceCode.getText(node.callee);
                if (allowedMap[calleeName] || allowedMap[sourceText]) {
                    return true;
                }
                if (pattern && pattern.test(sourceText)) {
                    return true;
                }
                if (calleeName === "UTC" && node.callee.type === "MemberExpression") {
                    // allow if callee is Date.UTC
                    return node.callee.object.type === "Identifier" &&
                        node.callee.object.name === "Date";
                }
                return skipProperties && node.callee.type === "MemberExpression";
            }
            /**
             * Reports the given messageId for the given node. The location will be the start of the property or the callee.
             * @param {ASTNode} node CallExpression or NewExpression node.
             * @param {string} messageId The messageId to report.
             * @returns {void}
             */
            function report(node, messageId) {
                let callee = node.callee;
                if (callee.type === "MemberExpression") {
                    callee = callee.property;
                }
                context.report({ node, loc: callee.loc.start, messageId });
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            if (config.newIsCap) {
                listeners.NewExpression = function (node) {
                    const constructorName = extractNameFromExpression(node);
                    if (constructorName) {
                        const capitalization = getCap(constructorName);
                        const isAllowed = capitalization !== "lower" || isCapAllowed(newIsCapExceptions, node, constructorName, newIsCapExceptionPattern);
                        if (!isAllowed) {
                            report(node, "lower");
                        }
                    }
                };
            }
            if (config.capIsNew) {
                listeners.CallExpression = function (node) {
                    const calleeName = extractNameFromExpression(node);
                    if (calleeName) {
                        const capitalization = getCap(calleeName);
                        const isAllowed = capitalization !== "upper" || isCapAllowed(capIsNewExceptions, node, calleeName, capIsNewExceptionPattern);
                        if (!isAllowed) {
                            report(node, "upper");
                        }
                    }
                };
            }
            return listeners;
        }
    };
},
/* eslint/lib/rules/new-parens.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce or disallow parentheses when invoking a constructor with no arguments",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/new-parens"
            },
            fixable: "code",
            schema: {
                anyOf: [
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["always", "never"]
                            }
                        ],
                        minItems: 0,
                        maxItems: 1
                    }
                ]
            },
            messages: {
                missing: "Missing '()' invoking a constructor.",
                unnecessary: "Unnecessary '()' invoking a constructor with no arguments."
            }
        },
        create(context) {
            const options = context.options;
            const always = options[0] !== "never"; // Default is always
            const sourceCode = context.getSourceCode();
            return {
                NewExpression(node) {
                    if (node.arguments.length !== 0) {
                        return; // if there are arguments, there have to be parens
                    }
                    const lastToken = sourceCode.getLastToken(node);
                    const hasLastParen = lastToken && astUtils.isClosingParenToken(lastToken);
                    // `hasParens` is true only if the new expression ends with its own parens, e.g., new new foo() does not end with its own parens
                    const hasParens = hasLastParen &&
                        astUtils.isOpeningParenToken(sourceCode.getTokenBefore(lastToken)) &&
                        node.callee.range[1] < node.range[1];
                    if (always) {
                        if (!hasParens) {
                            context.report({
                                node,
                                messageId: "missing",
                                fix: fixer => fixer.insertTextAfter(node, "()")
                            });
                        }
                    }
                    else {
                        if (hasParens) {
                            context.report({
                                node,
                                messageId: "unnecessary",
                                fix: fixer => [
                                    fixer.remove(sourceCode.getTokenBefore(lastToken)),
                                    fixer.remove(lastToken),
                                    fixer.insertTextBefore(node, "("),
                                    fixer.insertTextAfter(node, ")")
                                ]
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/newline-after-var.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow an empty line after variable declarations",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/newline-after-var"
            },
            schema: [
                {
                    enum: ["never", "always"]
                }
            ],
            fixable: "whitespace",
            messages: {
                expected: "Expected blank line after variable declarations.",
                unexpected: "Unexpected blank line after variable declarations."
            },
            deprecated: true,
            replacedBy: ["padding-line-between-statements"]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            // Default `mode` to "always".
            const mode = context.options[0] === "never" ? "never" : "always";
            // Cache starting and ending line numbers of comments for faster lookup
            const commentEndLine = sourceCode.getAllComments().reduce((result, token) => {
                result[token.loc.start.line] = token.loc.end.line;
                return result;
            }, {});
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Gets a token from the given node to compare line to the next statement.
             *
             * In general, the token is the last token of the node. However, the token is the second last token if the following conditions satisfy.
             *
             * - The last token is semicolon.
             * - The semicolon is on a different line from the previous token of the semicolon.
             *
             * This behavior would address semicolon-less style code. e.g.:
             *
             *     var foo = 1
             *
             *     ;(a || b).doSomething()
             * @param {ASTNode} node The node to get.
             * @returns {Token} The token to compare line to the next statement.
             */
            function getLastToken(node) {
                const lastToken = sourceCode.getLastToken(node);
                if (lastToken.type === "Punctuator" && lastToken.value === ";") {
                    const prevToken = sourceCode.getTokenBefore(lastToken);
                    if (prevToken.loc.end.line !== lastToken.loc.start.line) {
                        return prevToken;
                    }
                }
                return lastToken;
            }
            /**
             * Determine if provided keyword is a variable declaration
             * @private
             * @param {string} keyword keyword to test
             * @returns {boolean} True if `keyword` is a type of var
             */
            function isVar(keyword) {
                return keyword === "var" || keyword === "let" || keyword === "const";
            }
            /**
             * Determine if provided keyword is a variant of for specifiers
             * @private
             * @param {string} keyword keyword to test
             * @returns {boolean} True if `keyword` is a variant of for specifier
             */
            function isForTypeSpecifier(keyword) {
                return keyword === "ForStatement" || keyword === "ForInStatement" || keyword === "ForOfStatement";
            }
            /**
             * Determine if provided keyword is an export specifiers
             * @private
             * @param {string} nodeType nodeType to test
             * @returns {boolean} True if `nodeType` is an export specifier
             */
            function isExportSpecifier(nodeType) {
                return nodeType === "ExportNamedDeclaration" || nodeType === "ExportSpecifier" ||
                    nodeType === "ExportDefaultDeclaration" || nodeType === "ExportAllDeclaration";
            }
            /**
             * Determine if provided node is the last of their parent block.
             * @private
             * @param {ASTNode} node node to test
             * @returns {boolean} True if `node` is last of their parent block.
             */
            function isLastNode(node) {
                const token = sourceCode.getTokenAfter(node);
                return !token || (token.type === "Punctuator" && token.value === "}");
            }
            /**
             * Gets the last line of a group of consecutive comments
             * @param {number} commentStartLine The starting line of the group
             * @returns {number} The number of the last comment line of the group
             */
            function getLastCommentLineOfBlock(commentStartLine) {
                const currentCommentEnd = commentEndLine[commentStartLine];
                return commentEndLine[currentCommentEnd + 1] ? getLastCommentLineOfBlock(currentCommentEnd + 1) : currentCommentEnd;
            }
            /**
             * Determine if a token starts more than one line after a comment ends
             * @param  {token}   token            The token being checked
             * @param {integer}  commentStartLine The line number on which the comment starts
             * @returns {boolean}                 True if `token` does not start immediately after a comment
             */
            function hasBlankLineAfterComment(token, commentStartLine) {
                return token.loc.start.line > getLastCommentLineOfBlock(commentStartLine) + 1;
            }
            /**
             * Checks that a blank line exists after a variable declaration when mode is
             * set to "always", or checks that there is no blank line when mode is set
             * to "never"
             * @private
             * @param {ASTNode} node `VariableDeclaration` node to test
             * @returns {void}
             */
            function checkForBlankLine(node) {
                /*
                 * lastToken is the last token on the node's line. It will usually also be the last token of the node, but it will
                 * sometimes be second-last if there is a semicolon on a different line.
                 */
                const lastToken = getLastToken(node), 
                /*
                 * If lastToken is the last token of the node, nextToken should be the token after the node. Otherwise, nextToken
                 * is the last token of the node.
                 */
                nextToken = lastToken === sourceCode.getLastToken(node) ? sourceCode.getTokenAfter(node) : sourceCode.getLastToken(node), nextLineNum = lastToken.loc.end.line + 1;
                // Ignore if there is no following statement
                if (!nextToken) {
                    return;
                }
                // Ignore if parent of node is a for variant
                if (isForTypeSpecifier(node.parent.type)) {
                    return;
                }
                // Ignore if parent of node is an export specifier
                if (isExportSpecifier(node.parent.type)) {
                    return;
                }
                /*
                 * Some coding styles use multiple `var` statements, so do nothing if
                 * the next token is a `var` statement.
                 */
                if (nextToken.type === "Keyword" && isVar(nextToken.value)) {
                    return;
                }
                // Ignore if it is last statement in a block
                if (isLastNode(node)) {
                    return;
                }
                // Next statement is not a `var`...
                const noNextLineToken = nextToken.loc.start.line > nextLineNum;
                const hasNextLineComment = (typeof commentEndLine[nextLineNum] !== "undefined");
                if (mode === "never" && noNextLineToken && !hasNextLineComment) {
                    context.report({
                        node,
                        messageId: "unexpected",
                        data: { identifier: node.name },
                        fix(fixer) {
                            const linesBetween = sourceCode.getText().slice(lastToken.range[1], nextToken.range[0]).split(astUtils.LINEBREAK_MATCHER);
                            return fixer.replaceTextRange([lastToken.range[1], nextToken.range[0]], `${linesBetween.slice(0, -1).join("")}\n${linesBetween[linesBetween.length - 1]}`);
                        }
                    });
                }
                // Token on the next line, or comment without blank line
                if (mode === "always" && (!noNextLineToken ||
                    hasNextLineComment && !hasBlankLineAfterComment(nextToken, nextLineNum))) {
                    context.report({
                        node,
                        messageId: "expected",
                        data: { identifier: node.name },
                        fix(fixer) {
                            if ((noNextLineToken ? getLastCommentLineOfBlock(nextLineNum) : lastToken.loc.end.line) === nextToken.loc.start.line) {
                                return fixer.insertTextBefore(nextToken, "\n\n");
                            }
                            return fixer.insertTextBeforeRange([nextToken.range[0] - nextToken.loc.start.column, nextToken.range[1]], "\n");
                        }
                    });
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                VariableDeclaration: checkForBlankLine
            };
        }
    };
},
/* eslint/lib/rules/newline-before-return.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require an empty line before `return` statements",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/newline-before-return"
            },
            fixable: "whitespace",
            schema: [],
            messages: {
                expected: "Expected newline before return statement."
            },
            deprecated: true,
            replacedBy: ["padding-line-between-statements"]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Tests whether node is preceded by supplied tokens
             * @param {ASTNode} node node to check
             * @param {Array} testTokens array of tokens to test against
             * @returns {boolean} Whether or not the node is preceded by one of the supplied tokens
             * @private
             */
            function isPrecededByTokens(node, testTokens) {
                const tokenBefore = sourceCode.getTokenBefore(node);
                return testTokens.some(token => tokenBefore.value === token);
            }
            /**
             * Checks whether node is the first node after statement or in block
             * @param {ASTNode} node node to check
             * @returns {boolean} Whether or not the node is the first node after statement or in block
             * @private
             */
            function isFirstNode(node) {
                const parentType = node.parent.type;
                if (node.parent.body) {
                    return Array.isArray(node.parent.body)
                        ? node.parent.body[0] === node
                        : node.parent.body === node;
                }
                if (parentType === "IfStatement") {
                    return isPrecededByTokens(node, ["else", ")"]);
                }
                if (parentType === "DoWhileStatement") {
                    return isPrecededByTokens(node, ["do"]);
                }
                if (parentType === "SwitchCase") {
                    return isPrecededByTokens(node, [":"]);
                }
                return isPrecededByTokens(node, [")"]);
            }
            /**
             * Returns the number of lines of comments that precede the node
             * @param {ASTNode} node node to check for overlapping comments
             * @param {number} lineNumTokenBefore line number of previous token, to check for overlapping comments
             * @returns {number} Number of lines of comments that precede the node
             * @private
             */
            function calcCommentLines(node, lineNumTokenBefore) {
                const comments = sourceCode.getCommentsBefore(node);
                let numLinesComments = 0;
                if (!comments.length) {
                    return numLinesComments;
                }
                comments.forEach(comment => {
                    numLinesComments++;
                    if (comment.type === "Block") {
                        numLinesComments += comment.loc.end.line - comment.loc.start.line;
                    }
                    // avoid counting lines with inline comments twice
                    if (comment.loc.start.line === lineNumTokenBefore) {
                        numLinesComments--;
                    }
                    if (comment.loc.end.line === node.loc.start.line) {
                        numLinesComments--;
                    }
                });
                return numLinesComments;
            }
            /**
             * Returns the line number of the token before the node that is passed in as an argument
             * @param {ASTNode} node The node to use as the start of the calculation
             * @returns {number} Line number of the token before `node`
             * @private
             */
            function getLineNumberOfTokenBefore(node) {
                const tokenBefore = sourceCode.getTokenBefore(node);
                let lineNumTokenBefore;
                /**
                 * Global return (at the beginning of a script) is a special case.
                 * If there is no token before `return`, then we expect no line
                 * break before the return. Comments are allowed to occupy lines
                 * before the global return, just no blank lines.
                 * Setting lineNumTokenBefore to zero in that case results in the
                 * desired behavior.
                 */
                if (tokenBefore) {
                    lineNumTokenBefore = tokenBefore.loc.end.line;
                }
                else {
                    lineNumTokenBefore = 0; // global return at beginning of script
                }
                return lineNumTokenBefore;
            }
            /**
             * Checks whether node is preceded by a newline
             * @param {ASTNode} node node to check
             * @returns {boolean} Whether or not the node is preceded by a newline
             * @private
             */
            function hasNewlineBefore(node) {
                const lineNumNode = node.loc.start.line;
                const lineNumTokenBefore = getLineNumberOfTokenBefore(node);
                const commentLines = calcCommentLines(node, lineNumTokenBefore);
                return (lineNumNode - lineNumTokenBefore - commentLines) > 1;
            }
            /**
             * Checks whether it is safe to apply a fix to a given return statement.
             *
             * The fix is not considered safe if the given return statement has leading comments,
             * as we cannot safely determine if the newline should be added before or after the comments.
             * For more information, see: https://github.com/eslint/eslint/issues/5958#issuecomment-222767211
             * @param {ASTNode} node The return statement node to check.
             * @returns {boolean} `true` if it can fix the node.
             * @private
             */
            function canFix(node) {
                const leadingComments = sourceCode.getCommentsBefore(node);
                const lastLeadingComment = leadingComments[leadingComments.length - 1];
                const tokenBefore = sourceCode.getTokenBefore(node);
                if (leadingComments.length === 0) {
                    return true;
                }
                /*
                 * if the last leading comment ends in the same line as the previous token and
                 * does not share a line with the `return` node, we can consider it safe to fix.
                 * Example:
                 * function a() {
                 *     var b; //comment
                 *     return;
                 * }
                 */
                if (lastLeadingComment.loc.end.line === tokenBefore.loc.end.line &&
                    lastLeadingComment.loc.end.line !== node.loc.start.line) {
                    return true;
                }
                return false;
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                ReturnStatement(node) {
                    if (!isFirstNode(node) && !hasNewlineBefore(node)) {
                        context.report({
                            node,
                            messageId: "expected",
                            fix(fixer) {
                                if (canFix(node)) {
                                    const tokenBefore = sourceCode.getTokenBefore(node);
                                    const newlines = node.loc.start.line === tokenBefore.loc.end.line ? "\n\n" : "\n";
                                    return fixer.insertTextBefore(node, newlines);
                                }
                                return null;
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/newline-per-chained-call.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require a newline after each call in a method chain",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/newline-per-chained-call"
            },
            fixable: "whitespace",
            schema: [{
                    type: "object",
                    properties: {
                        ignoreChainWithDepth: {
                            type: "integer",
                            minimum: 1,
                            maximum: 10,
                            default: 2
                        }
                    },
                    additionalProperties: false
                }],
            messages: {
                expected: "Expected line break before `{{callee}}`."
            }
        },
        create(context) {
            const options = context.options[0] || {}, ignoreChainWithDepth = options.ignoreChainWithDepth || 2;
            const sourceCode = context.getSourceCode();
            /**
             * Get the prefix of a given MemberExpression node.
             * If the MemberExpression node is a computed value it returns a
             * left bracket. If not it returns a period.
             * @param  {ASTNode} node A MemberExpression node to get
             * @returns {string} The prefix of the node.
             */
            function getPrefix(node) {
                return node.computed ? "[" : ".";
            }
            /**
             * Gets the property text of a given MemberExpression node.
             * If the text is multiline, this returns only the first line.
             * @param {ASTNode} node A MemberExpression node to get.
             * @returns {string} The property text of the node.
             */
            function getPropertyText(node) {
                const prefix = getPrefix(node);
                const lines = sourceCode.getText(node.property).split(astUtils.LINEBREAK_MATCHER);
                const suffix = node.computed && lines.length === 1 ? "]" : "";
                return prefix + lines[0] + suffix;
            }
            return {
                "CallExpression:exit"(node) {
                    if (!node.callee || node.callee.type !== "MemberExpression") {
                        return;
                    }
                    const callee = node.callee;
                    let parent = callee.object;
                    let depth = 1;
                    while (parent && parent.callee) {
                        depth += 1;
                        parent = parent.callee.object;
                    }
                    if (depth > ignoreChainWithDepth && astUtils.isTokenOnSameLine(callee.object, callee.property)) {
                        context.report({
                            node: callee.property,
                            loc: callee.property.loc.start,
                            messageId: "expected",
                            data: {
                                callee: getPropertyText(callee)
                            },
                            fix(fixer) {
                                const firstTokenAfterObject = sourceCode.getTokenAfter(callee.object, astUtils.isNotClosingParenToken);
                                return fixer.insertTextBefore(firstTokenAfterObject, "\n");
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-alert.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const getPropertyName = require(126) /* ./utils/ast-utils */.getStaticPropertyName;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks if the given name is a prohibited identifier.
     * @param {string} name The name to check
     * @returns {boolean} Whether or not the name is prohibited.
     */
    function isProhibitedIdentifier(name) {
        return /^(alert|confirm|prompt)$/u.test(name);
    }
    /**
     * Finds the eslint-scope reference in the given scope.
     * @param {Object} scope The scope to search.
     * @param {ASTNode} node The identifier node.
     * @returns {Reference|null} Returns the found reference or null if none were found.
     */
    function findReference(scope, node) {
        const references = scope.references.filter(reference => reference.identifier.range[0] === node.range[0] &&
            reference.identifier.range[1] === node.range[1]);
        if (references.length === 1) {
            return references[0];
        }
        return null;
    }
    /**
     * Checks if the given identifier node is shadowed in the given scope.
     * @param {Object} scope The current scope.
     * @param {string} node The identifier node to check
     * @returns {boolean} Whether or not the name is shadowed.
     */
    function isShadowed(scope, node) {
        const reference = findReference(scope, node);
        return reference && reference.resolved && reference.resolved.defs.length > 0;
    }
    /**
     * Checks if the given identifier node is a ThisExpression in the global scope or the global window property.
     * @param {Object} scope The current scope.
     * @param {string} node The identifier node to check
     * @returns {boolean} Whether or not the node is a reference to the global object.
     */
    function isGlobalThisReferenceOrGlobalWindow(scope, node) {
        if (scope.type === "global" && node.type === "ThisExpression") {
            return true;
        }
        if (node.name === "window") {
            return !isShadowed(scope, node);
        }
        return false;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the use of `alert`, `confirm`, and `prompt`",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-alert"
            },
            schema: [],
            messages: {
                unexpected: "Unexpected {{name}}."
            }
        },
        create(context) {
            return {
                CallExpression(node) {
                    const callee = node.callee, currentScope = context.getScope();
                    // without window.
                    if (callee.type === "Identifier") {
                        const name = callee.name;
                        if (!isShadowed(currentScope, callee) && isProhibitedIdentifier(callee.name)) {
                            context.report({
                                node,
                                messageId: "unexpected",
                                data: { name }
                            });
                        }
                    }
                    else if (callee.type === "MemberExpression" && isGlobalThisReferenceOrGlobalWindow(currentScope, callee.object)) {
                        const name = getPropertyName(callee);
                        if (isProhibitedIdentifier(name)) {
                            context.report({
                                node,
                                messageId: "unexpected",
                                data: { name }
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-array-constructor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `Array` constructors",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-array-constructor"
            },
            schema: [],
            messages: {
                preferLiteral: "The array literal notation [] is preferable."
            }
        },
        create(context) {
            /**
             * Disallow construction of dense arrays using the Array constructor
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function check(node) {
                if (node.arguments.length !== 1 &&
                    node.callee.type === "Identifier" &&
                    node.callee.name === "Array") {
                    context.report({ node, messageId: "preferLiteral" });
                }
            }
            return {
                CallExpression: check,
                NewExpression: check
            };
        }
    };
},
/* eslint/lib/rules/no-async-promise-executor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow using an async function as a Promise executor",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-async-promise-executor"
            },
            fixable: null,
            schema: [],
            messages: {
                async: "Promise executor functions should not be async."
            }
        },
        create(context) {
            return {
                "NewExpression[callee.name='Promise'][arguments.0.async=true]"(node) {
                    context.report({
                        node: context.getSourceCode().getFirstToken(node.arguments[0], token => token.value === "async"),
                        messageId: "async"
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-await-in-loop.js */ function _(require, module, exports) {
    /**
     * Check whether it should stop traversing ancestors at the given node.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if it should stop traversing.
     */
    function isBoundary(node) {
        const t = node.type;
        return (t === "FunctionDeclaration" ||
            t === "FunctionExpression" ||
            t === "ArrowFunctionExpression" ||
            /*
             * Don't report the await expressions on for-await-of loop since it's
             * asynchronous iteration intentionally.
             */
            (t === "ForOfStatement" && node.await === true));
    }
    /**
     * Check whether the given node is in loop.
     * @param {ASTNode} node A node to check.
     * @param {ASTNode} parent A parent node to check.
     * @returns {boolean} `true` if the node is in loop.
     */
    function isLooped(node, parent) {
        switch (parent.type) {
            case "ForStatement":
                return (node === parent.test ||
                    node === parent.update ||
                    node === parent.body);
            case "ForOfStatement":
            case "ForInStatement":
                return node === parent.body;
            case "WhileStatement":
            case "DoWhileStatement":
                return node === parent.test || node === parent.body;
            default:
                return false;
        }
    }
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow `await` inside of loops",
                category: "Possible Errors",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-await-in-loop"
            },
            schema: [],
            messages: {
                unexpectedAwait: "Unexpected `await` inside a loop."
            }
        },
        create(context) {
            /**
             * Validate an await expression.
             * @param {ASTNode} awaitNode An AwaitExpression or ForOfStatement node to validate.
             * @returns {void}
             */
            function validate(awaitNode) {
                if (awaitNode.type === "ForOfStatement" && !awaitNode.await) {
                    return;
                }
                let node = awaitNode;
                let parent = node.parent;
                while (parent && !isBoundary(parent)) {
                    if (isLooped(node, parent)) {
                        context.report({
                            node: awaitNode,
                            messageId: "unexpectedAwait"
                        });
                        return;
                    }
                    node = parent;
                    parent = parent.parent;
                }
            }
            return {
                AwaitExpression: validate,
                ForOfStatement: validate
            };
        }
    };
},
/* eslint/lib/rules/no-bitwise.js */ function _(require, module, exports) {
    /*
     *
     * Set of bitwise operators.
     *
     */
    const BITWISE_OPERATORS = [
        "^", "|", "&", "<<", ">>", ">>>",
        "^=", "|=", "&=", "<<=", ">>=", ">>>=",
        "~"
    ];
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow bitwise operators",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-bitwise"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allow: {
                            type: "array",
                            items: {
                                enum: BITWISE_OPERATORS
                            },
                            uniqueItems: true
                        },
                        int32Hint: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpected: "Unexpected use of '{{operator}}'."
            }
        },
        create(context) {
            const options = context.options[0] || {};
            const allowed = options.allow || [];
            const int32Hint = options.int32Hint === true;
            /**
             * Reports an unexpected use of a bitwise operator.
             * @param   {ASTNode} node Node which contains the bitwise operator.
             * @returns {void}
             */
            function report(node) {
                context.report({ node, messageId: "unexpected", data: { operator: node.operator } });
            }
            /**
             * Checks if the given node has a bitwise operator.
             * @param   {ASTNode} node The node to check.
             * @returns {boolean} Whether or not the node has a bitwise operator.
             */
            function hasBitwiseOperator(node) {
                return BITWISE_OPERATORS.indexOf(node.operator) !== -1;
            }
            /**
             * Checks if exceptions were provided, e.g. `{ allow: ['~', '|'] }`.
             * @param   {ASTNode} node The node to check.
             * @returns {boolean} Whether or not the node has a bitwise operator.
             */
            function allowedOperator(node) {
                return allowed.indexOf(node.operator) !== -1;
            }
            /**
             * Checks if the given bitwise operator is used for integer typecasting, i.e. "|0"
             * @param   {ASTNode} node The node to check.
             * @returns {boolean} whether the node is used in integer typecasting.
             */
            function isInt32Hint(node) {
                return int32Hint && node.operator === "|" && node.right &&
                    node.right.type === "Literal" && node.right.value === 0;
            }
            /**
             * Report if the given node contains a bitwise operator.
             * @param   {ASTNode} node The node to check.
             * @returns {void}
             */
            function checkNodeForBitwiseOperator(node) {
                if (hasBitwiseOperator(node) && !allowedOperator(node) && !isInt32Hint(node)) {
                    report(node);
                }
            }
            return {
                AssignmentExpression: checkNodeForBitwiseOperator,
                BinaryExpression: checkNodeForBitwiseOperator,
                UnaryExpression: checkNodeForBitwiseOperator
            };
        }
    };
},
/* eslint/lib/rules/no-buffer-constructor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow use of the `Buffer()` constructor",
                category: "Node.js and CommonJS",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-buffer-constructor"
            },
            schema: [],
            messages: {
                deprecated: "{{expr}} is deprecated. Use Buffer.from(), Buffer.alloc(), or Buffer.allocUnsafe() instead."
            }
        },
        create(context) {
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                "CallExpression[callee.name='Buffer'], NewExpression[callee.name='Buffer']"(node) {
                    context.report({
                        node,
                        messageId: "deprecated",
                        data: { expr: node.type === "CallExpression" ? "Buffer()" : "new Buffer()" }
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-caller.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the use of `arguments.caller` or `arguments.callee`",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-caller"
            },
            schema: [],
            messages: {
                unexpected: "Avoid arguments.{{prop}}."
            }
        },
        create(context) {
            return {
                MemberExpression(node) {
                    const objectName = node.object.name, propertyName = node.property.name;
                    if (objectName === "arguments" && !node.computed && propertyName && propertyName.match(/^calle[er]$/u)) {
                        context.report({ node, messageId: "unexpected", data: { prop: propertyName } });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-case-declarations.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow lexical declarations in case clauses",
                category: "Best Practices",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-case-declarations"
            },
            schema: [],
            messages: {
                unexpected: "Unexpected lexical declaration in case block."
            }
        },
        create(context) {
            /**
             * Checks whether or not a node is a lexical declaration.
             * @param {ASTNode} node A direct child statement of a switch case.
             * @returns {boolean} Whether or not the node is a lexical declaration.
             */
            function isLexicalDeclaration(node) {
                switch (node.type) {
                    case "FunctionDeclaration":
                    case "ClassDeclaration":
                        return true;
                    case "VariableDeclaration":
                        return node.kind !== "var";
                    default:
                        return false;
                }
            }
            return {
                SwitchCase(node) {
                    for (let i = 0; i < node.consequent.length; i++) {
                        const statement = node.consequent[i];
                        if (isLexicalDeclaration(statement)) {
                            context.report({
                                node: statement,
                                messageId: "unexpected"
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-catch-shadow.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `catch` clause parameters from shadowing variables in the outer scope",
                category: "Variables",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-catch-shadow"
            },
            replacedBy: ["no-shadow"],
            deprecated: true,
            schema: [],
            messages: {
                mutable: "Value of '{{name}}' may be overwritten in IE 8 and earlier."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Check if the parameters are been shadowed
             * @param {Object} scope current scope
             * @param {string} name parameter name
             * @returns {boolean} True is its been shadowed
             */
            function paramIsShadowing(scope, name) {
                return astUtils.getVariableByName(scope, name) !== null;
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                "CatchClause[param!=null]"(node) {
                    let scope = context.getScope();
                    /*
                     * When ecmaVersion >= 6, CatchClause creates its own scope
                     * so start from one upper scope to exclude the current node
                     */
                    if (scope.block === node) {
                        scope = scope.upper;
                    }
                    if (paramIsShadowing(scope, node.param.name)) {
                        context.report({ node, messageId: "mutable", data: { name: node.param.name } });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-class-assign.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow reassigning class members",
                category: "ECMAScript 6",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-class-assign"
            },
            schema: [],
            messages: {
                class: "'{{name}}' is a class."
            }
        },
        create(context) {
            /**
             * Finds and reports references that are non initializer and writable.
             * @param {Variable} variable A variable to check.
             * @returns {void}
             */
            function checkVariable(variable) {
                astUtils.getModifyingReferences(variable.references).forEach(reference => {
                    context.report({ node: reference.identifier, messageId: "class", data: { name: reference.identifier.name } });
                });
            }
            /**
             * Finds and reports references that are non initializer and writable.
             * @param {ASTNode} node A ClassDeclaration/ClassExpression node to check.
             * @returns {void}
             */
            function checkForClass(node) {
                context.getDeclaredVariables(node).forEach(checkVariable);
            }
            return {
                ClassDeclaration: checkForClass,
                ClassExpression: checkForClass
            };
        }
    };
},
/* eslint/lib/rules/no-compare-neg-zero.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow comparing against -0",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-compare-neg-zero"
            },
            fixable: null,
            schema: [],
            messages: {
                unexpected: "Do not use the '{{operator}}' operator to compare against -0."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Checks a given node is -0
             * @param {ASTNode} node A node to check.
             * @returns {boolean} `true` if the node is -0.
             */
            function isNegZero(node) {
                return node.type === "UnaryExpression" && node.operator === "-" && node.argument.type === "Literal" && node.argument.value === 0;
            }
            const OPERATORS_TO_CHECK = new Set([">", ">=", "<", "<=", "==", "===", "!=", "!=="]);
            return {
                BinaryExpression(node) {
                    if (OPERATORS_TO_CHECK.has(node.operator)) {
                        if (isNegZero(node.left) || isNegZero(node.right)) {
                            context.report({
                                node,
                                messageId: "unexpected",
                                data: { operator: node.operator }
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-cond-assign.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const TEST_CONDITION_PARENT_TYPES = new Set(["IfStatement", "WhileStatement", "DoWhileStatement", "ForStatement", "ConditionalExpression"]);
    const NODE_DESCRIPTIONS = {
        DoWhileStatement: "a 'do...while' statement",
        ForStatement: "a 'for' statement",
        IfStatement: "an 'if' statement",
        WhileStatement: "a 'while' statement"
    };
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow assignment operators in conditional expressions",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-cond-assign"
            },
            schema: [
                {
                    enum: ["except-parens", "always"]
                }
            ],
            messages: {
                unexpected: "Unexpected assignment within {{type}}.",
                // must match JSHint's error message
                missing: "Expected a conditional expression and instead saw an assignment."
            }
        },
        create(context) {
            const prohibitAssign = (context.options[0] || "except-parens");
            const sourceCode = context.getSourceCode();
            /**
             * Check whether an AST node is the test expression for a conditional statement.
             * @param {!Object} node The node to test.
             * @returns {boolean} `true` if the node is the text expression for a conditional statement; otherwise, `false`.
             */
            function isConditionalTestExpression(node) {
                return node.parent &&
                    TEST_CONDITION_PARENT_TYPES.has(node.parent.type) &&
                    node === node.parent.test;
            }
            /**
             * Given an AST node, perform a bottom-up search for the first ancestor that represents a conditional statement.
             * @param {!Object} node The node to use at the start of the search.
             * @returns {?Object} The closest ancestor node that represents a conditional statement.
             */
            function findConditionalAncestor(node) {
                let currentAncestor = node;
                do {
                    if (isConditionalTestExpression(currentAncestor)) {
                        return currentAncestor.parent;
                    }
                } while ((currentAncestor = currentAncestor.parent) && !astUtils.isFunction(currentAncestor));
                return null;
            }
            /**
             * Check whether the code represented by an AST node is enclosed in two sets of parentheses.
             * @param {!Object} node The node to test.
             * @returns {boolean} `true` if the code is enclosed in two sets of parentheses; otherwise, `false`.
             */
            function isParenthesisedTwice(node) {
                const previousToken = sourceCode.getTokenBefore(node, 1), nextToken = sourceCode.getTokenAfter(node, 1);
                return astUtils.isParenthesised(sourceCode, node) &&
                    previousToken && astUtils.isOpeningParenToken(previousToken) && previousToken.range[1] <= node.range[0] &&
                    astUtils.isClosingParenToken(nextToken) && nextToken.range[0] >= node.range[1];
            }
            /**
             * Check a conditional statement's test expression for top-level assignments that are not enclosed in parentheses.
             * @param {!Object} node The node for the conditional statement.
             * @returns {void}
             */
            function testForAssign(node) {
                if (node.test &&
                    (node.test.type === "AssignmentExpression") &&
                    (node.type === "ForStatement"
                        ? !astUtils.isParenthesised(sourceCode, node.test)
                        : !isParenthesisedTwice(node.test))) {
                    context.report({
                        node: node.test,
                        messageId: "missing"
                    });
                }
            }
            /**
             * Check whether an assignment expression is descended from a conditional statement's test expression.
             * @param {!Object} node The node for the assignment expression.
             * @returns {void}
             */
            function testForConditionalAncestor(node) {
                const ancestor = findConditionalAncestor(node);
                if (ancestor) {
                    context.report({
                        node,
                        messageId: "unexpected",
                        data: {
                            type: NODE_DESCRIPTIONS[ancestor.type] || ancestor.type
                        }
                    });
                }
            }
            if (prohibitAssign === "always") {
                return {
                    AssignmentExpression: testForConditionalAncestor
                };
            }
            return {
                DoWhileStatement: testForAssign,
                ForStatement: testForAssign,
                IfStatement: testForAssign,
                WhileStatement: testForAssign,
                ConditionalExpression: testForAssign
            };
        }
    };
},
/* eslint/lib/rules/no-confusing-arrow.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils.js */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a node is a conditional expression.
     * @param {ASTNode} node node to test
     * @returns {boolean} `true` if the node is a conditional expression.
     */
    function isConditional(node) {
        return node && node.type === "ConditionalExpression";
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow arrow functions where they could be confused with comparisons",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-confusing-arrow"
            },
            fixable: "code",
            schema: [{
                    type: "object",
                    properties: {
                        allowParens: { type: "boolean", default: true }
                    },
                    additionalProperties: false
                }],
            messages: {
                confusing: "Arrow function used ambiguously with a conditional expression."
            }
        },
        create(context) {
            const config = context.options[0] || {};
            const allowParens = config.allowParens || (config.allowParens === void 0);
            const sourceCode = context.getSourceCode();
            /**
             * Reports if an arrow function contains an ambiguous conditional.
             * @param {ASTNode} node A node to check and report.
             * @returns {void}
             */
            function checkArrowFunc(node) {
                const body = node.body;
                if (isConditional(body) && !(allowParens && astUtils.isParenthesised(sourceCode, body))) {
                    context.report({
                        node,
                        messageId: "confusing",
                        fix(fixer) {
                            // if `allowParens` is not set to true dont bother wrapping in parens
                            return allowParens && fixer.replaceText(node.body, `(${sourceCode.getText(node.body)})`);
                        }
                    });
                }
            }
            return {
                ArrowFunctionExpression: checkArrowFunc
            };
        }
    };
},
/* eslint/lib/rules/no-console.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the use of `console`",
                category: "Possible Errors",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-console"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allow: {
                            type: "array",
                            items: {
                                type: "string"
                            },
                            minItems: 1,
                            uniqueItems: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpected: "Unexpected console statement."
            }
        },
        create(context) {
            const options = context.options[0] || {};
            const allowed = options.allow || [];
            /**
             * Checks whether the given reference is 'console' or not.
             * @param {eslint-scope.Reference} reference The reference to check.
             * @returns {boolean} `true` if the reference is 'console'.
             */
            function isConsole(reference) {
                const id = reference.identifier;
                return id && id.name === "console";
            }
            /**
             * Checks whether the property name of the given MemberExpression node
             * is allowed by options or not.
             * @param {ASTNode} node The MemberExpression node to check.
             * @returns {boolean} `true` if the property name of the node is allowed.
             */
            function isAllowed(node) {
                const propertyName = astUtils.getStaticPropertyName(node);
                return propertyName && allowed.indexOf(propertyName) !== -1;
            }
            /**
             * Checks whether the given reference is a member access which is not
             * allowed by options or not.
             * @param {eslint-scope.Reference} reference The reference to check.
             * @returns {boolean} `true` if the reference is a member access which
             *      is not allowed by options.
             */
            function isMemberAccessExceptAllowed(reference) {
                const node = reference.identifier;
                const parent = node.parent;
                return (parent.type === "MemberExpression" &&
                    parent.object === node &&
                    !isAllowed(parent));
            }
            /**
             * Reports the given reference as a violation.
             * @param {eslint-scope.Reference} reference The reference to report.
             * @returns {void}
             */
            function report(reference) {
                const node = reference.identifier.parent;
                context.report({
                    node,
                    loc: node.loc,
                    messageId: "unexpected"
                });
            }
            return {
                "Program:exit"() {
                    const scope = context.getScope();
                    const consoleVar = astUtils.getVariableByName(scope, "console");
                    const shadowed = consoleVar && consoleVar.defs.length > 0;
                    /*
                     * 'scope.through' includes all references to undefined
                     * variables. If the variable 'console' is not defined, it uses
                     * 'scope.through'.
                     */
                    const references = consoleVar
                        ? consoleVar.references
                        : scope.through.filter(isConsole);
                    if (!shadowed) {
                        references
                            .filter(isMemberAccessExceptAllowed)
                            .forEach(report);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-const-assign.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow reassigning `const` variables",
                category: "ECMAScript 6",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-const-assign"
            },
            schema: [],
            messages: {
                const: "'{{name}}' is constant."
            }
        },
        create(context) {
            /**
             * Finds and reports references that are non initializer and writable.
             * @param {Variable} variable A variable to check.
             * @returns {void}
             */
            function checkVariable(variable) {
                astUtils.getModifyingReferences(variable.references).forEach(reference => {
                    context.report({ node: reference.identifier, messageId: "const", data: { name: reference.identifier.name } });
                });
            }
            return {
                VariableDeclaration(node) {
                    if (node.kind === "const") {
                        context.getDeclaredVariables(node).forEach(checkVariable);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-constant-condition.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const EQUALITY_OPERATORS = ["===", "!==", "==", "!="];
    const RELATIONAL_OPERATORS = [">", "<", ">=", "<=", "in", "instanceof"];
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow constant expressions in conditions",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-constant-condition"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        checkLoops: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpected: "Unexpected constant condition."
            }
        },
        create(context) {
            const options = context.options[0] || {}, checkLoops = options.checkLoops !== false, loopSetStack = [];
            let loopsInCurrentScope = new Set();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Checks if a branch node of LogicalExpression short circuits the whole condition
             * @param {ASTNode} node The branch of main condition which needs to be checked
             * @param {string} operator The operator of the main LogicalExpression.
             * @returns {boolean} true when condition short circuits whole condition
             */
            function isLogicalIdentity(node, operator) {
                switch (node.type) {
                    case "Literal":
                        return (operator === "||" && node.value === true) ||
                            (operator === "&&" && node.value === false);
                    case "UnaryExpression":
                        return (operator === "&&" && node.operator === "void");
                    case "LogicalExpression":
                        return isLogicalIdentity(node.left, node.operator) ||
                            isLogicalIdentity(node.right, node.operator);
                    // no default
                }
                return false;
            }
            /**
             * Checks if a node has a constant truthiness value.
             * @param {ASTNode} node The AST node to check.
             * @param {boolean} inBooleanPosition `false` if checking branch of a condition.
             *  `true` in all other cases
             * @returns {Bool} true when node's truthiness is constant
             * @private
             */
            function isConstant(node, inBooleanPosition) {
                switch (node.type) {
                    case "Literal":
                    case "ArrowFunctionExpression":
                    case "FunctionExpression":
                    case "ObjectExpression":
                    case "ArrayExpression":
                        return true;
                    case "UnaryExpression":
                        if (node.operator === "void") {
                            return true;
                        }
                        return (node.operator === "typeof" && inBooleanPosition) ||
                            isConstant(node.argument, true);
                    case "BinaryExpression":
                        return isConstant(node.left, false) &&
                            isConstant(node.right, false) &&
                            node.operator !== "in";
                    case "LogicalExpression": {
                        const isLeftConstant = isConstant(node.left, inBooleanPosition);
                        const isRightConstant = isConstant(node.right, inBooleanPosition);
                        const isLeftShortCircuit = (isLeftConstant && isLogicalIdentity(node.left, node.operator));
                        const isRightShortCircuit = (isRightConstant && isLogicalIdentity(node.right, node.operator));
                        return (isLeftConstant && isRightConstant) ||
                            (
                            // in the case of an "OR", we need to know if the right constant value is truthy
                            node.operator === "||" &&
                                isRightConstant &&
                                node.right.value &&
                                (!node.parent ||
                                    node.parent.type !== "BinaryExpression" ||
                                    !(EQUALITY_OPERATORS.includes(node.parent.operator) || RELATIONAL_OPERATORS.includes(node.parent.operator)))) ||
                            isLeftShortCircuit ||
                            isRightShortCircuit;
                    }
                    case "AssignmentExpression":
                        return (node.operator === "=") && isConstant(node.right, inBooleanPosition);
                    case "SequenceExpression":
                        return isConstant(node.expressions[node.expressions.length - 1], inBooleanPosition);
                    // no default
                }
                return false;
            }
            /**
             * Tracks when the given node contains a constant condition.
             * @param {ASTNode} node The AST node to check.
             * @returns {void}
             * @private
             */
            function trackConstantConditionLoop(node) {
                if (node.test && isConstant(node.test, true)) {
                    loopsInCurrentScope.add(node);
                }
            }
            /**
             * Reports when the set contains the given constant condition node
             * @param {ASTNode} node The AST node to check.
             * @returns {void}
             * @private
             */
            function checkConstantConditionLoopInSet(node) {
                if (loopsInCurrentScope.has(node)) {
                    loopsInCurrentScope.delete(node);
                    context.report({ node: node.test, messageId: "unexpected" });
                }
            }
            /**
             * Reports when the given node contains a constant condition.
             * @param {ASTNode} node The AST node to check.
             * @returns {void}
             * @private
             */
            function reportIfConstant(node) {
                if (node.test && isConstant(node.test, true)) {
                    context.report({ node: node.test, messageId: "unexpected" });
                }
            }
            /**
             * Stores current set of constant loops in loopSetStack temporarily
             * and uses a new set to track constant loops
             * @returns {void}
             * @private
             */
            function enterFunction() {
                loopSetStack.push(loopsInCurrentScope);
                loopsInCurrentScope = new Set();
            }
            /**
             * Reports when the set still contains stored constant conditions
             * @returns {void}
             * @private
             */
            function exitFunction() {
                loopsInCurrentScope = loopSetStack.pop();
            }
            /**
             * Checks node when checkLoops option is enabled
             * @param {ASTNode} node The AST node to check.
             * @returns {void}
             * @private
             */
            function checkLoop(node) {
                if (checkLoops) {
                    trackConstantConditionLoop(node);
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                ConditionalExpression: reportIfConstant,
                IfStatement: reportIfConstant,
                WhileStatement: checkLoop,
                "WhileStatement:exit": checkConstantConditionLoopInSet,
                DoWhileStatement: checkLoop,
                "DoWhileStatement:exit": checkConstantConditionLoopInSet,
                ForStatement: checkLoop,
                "ForStatement > .test": node => checkLoop(node.parent),
                "ForStatement:exit": checkConstantConditionLoopInSet,
                FunctionDeclaration: enterFunction,
                "FunctionDeclaration:exit": exitFunction,
                FunctionExpression: enterFunction,
                "FunctionExpression:exit": exitFunction,
                YieldExpression: () => loopsInCurrentScope.clear()
            };
        }
    };
},
/* eslint/lib/rules/no-constructor-return.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow returning value from constructor",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-constructor-return"
            },
            schema: {},
            fixable: null,
            messages: {
                unexpected: "Unexpected return statement in constructor."
            }
        },
        create(context) {
            const stack = [];
            return {
                onCodePathStart(_, node) {
                    stack.push(node);
                },
                onCodePathEnd() {
                    stack.pop();
                },
                ReturnStatement(node) {
                    const last = stack[stack.length - 1];
                    if (!last.parent) {
                        return;
                    }
                    if (last.parent.type === "MethodDefinition" &&
                        last.parent.kind === "constructor" &&
                        (node.parent.parent === last || node.argument)) {
                        context.report({
                            node,
                            messageId: "unexpected"
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-continue.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `continue` statements",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-continue"
            },
            schema: [],
            messages: {
                unexpected: "Unexpected use of continue statement."
            }
        },
        create(context) {
            return {
                ContinueStatement(node) {
                    context.report({ node, messageId: "unexpected" });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-control-regex.js */ function _(require, module, exports) {
    const RegExpValidator = require(226) /* regexpp */.RegExpValidator;
    const collector = new (class {
        constructor() {
            this.ecmaVersion = 2018;
            this._source = "";
            this._controlChars = [];
            this._validator = new RegExpValidator(this);
        }
        onPatternEnter() {
            this._controlChars = [];
        }
        onCharacter(start, end, cp) {
            if (cp >= 0x00 &&
                cp <= 0x1F &&
                (this._source.codePointAt(start) === cp ||
                    this._source.slice(start, end).startsWith("\\x") ||
                    this._source.slice(start, end).startsWith("\\u"))) {
                this._controlChars.push(`\\x${`0${cp.toString(16)}`.slice(-2)}`);
            }
        }
        collectControlChars(regexpStr) {
            try {
                this._source = regexpStr;
                this._validator.validatePattern(regexpStr); // Call onCharacter hook
            }
            catch (err) {
                // Ignore syntax errors in RegExp.
            }
            return this._controlChars;
        }
    })();
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow control characters in regular expressions",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-control-regex"
            },
            schema: [],
            messages: {
                unexpected: "Unexpected control character(s) in regular expression: {{controlChars}}."
            }
        },
        create(context) {
            /**
             * Get the regex expression
             * @param {ASTNode} node node to evaluate
             * @returns {RegExp|null} Regex if found else null
             * @private
             */
            function getRegExpPattern(node) {
                if (node.regex) {
                    return node.regex.pattern;
                }
                if (typeof node.value === "string" &&
                    (node.parent.type === "NewExpression" || node.parent.type === "CallExpression") &&
                    node.parent.callee.type === "Identifier" &&
                    node.parent.callee.name === "RegExp" &&
                    node.parent.arguments[0] === node) {
                    return node.value;
                }
                return null;
            }
            return {
                Literal(node) {
                    const pattern = getRegExpPattern(node);
                    if (pattern) {
                        const controlCharacters = collector.collectControlChars(pattern);
                        if (controlCharacters.length > 0) {
                            context.report({
                                node,
                                messageId: "unexpected",
                                data: {
                                    controlChars: controlCharacters.join(", ")
                                }
                            });
                        }
                    }
                }
            };
        }
    };
},
/* regexpp/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, '__esModule', { value: true });
    var ast = /*#__PURE__*/ Object.freeze({});
    let largeIdStartPattern = null;
    let largeIdContinuePattern = null;
    function isIdStart(cp) {
        if (cp < 0x41)
            return false;
        if (cp < 0x5b)
            return true;
        if (cp < 0x61)
            return false;
        if (cp < 0x7b)
            return true;
        return isLargeIdStart(cp);
    }
    function isIdContinue(cp) {
        if (cp < 0x30)
            return false;
        if (cp < 0x3a)
            return true;
        if (cp < 0x41)
            return false;
        if (cp < 0x5b)
            return true;
        if (cp === 0x5f)
            return true;
        if (cp < 0x61)
            return false;
        if (cp < 0x7b)
            return true;
        return isLargeIdStart(cp) || isLargeIdContinue(cp);
    }
    function isLargeIdStart(cp) {
        if (!largeIdStartPattern) {
            largeIdStartPattern = new RegExp("^[\xaa\xb5\xba\xc0-\xd6\xd8-\xf6\xf8-\u02c1\u02c6-\u02d1" +
                "\u02e0-\u02e4\u02ec\u02ee\u0370-\u0374\u0376\u0377" +
                "\u037a-\u037d\u037f\u0386\u0388-\u038a\u038c\u038e-\u03a1" +
                "\u03a3-\u03f5\u03f7-\u0481\u048a-\u052f\u0531-\u0556\u0559" +
                "\u0560-\u0588\u05d0-\u05ea\u05ef-\u05f2\u0620-\u064a" +
                "\u066e\u066f\u0671-\u06d3\u06d5\u06e5\u06e6\u06ee\u06ef" +
                "\u06fa-\u06fc\u06ff\u0710\u0712-\u072f\u074d-\u07a5\u07b1" +
                "\u07ca-\u07ea\u07f4\u07f5\u07fa\u0800-\u0815\u081a\u0824" +
                "\u0828\u0840-\u0858\u0860-\u086a\u08a0-\u08b4\u08b6-\u08bd" +
                "\u0904-\u0939\u093d\u0950\u0958-\u0961\u0971-\u0980" +
                "\u0985-\u098c\u098f\u0990\u0993-\u09a8\u09aa-\u09b0\u09b2" +
                "\u09b6-\u09b9\u09bd\u09ce\u09dc\u09dd\u09df-\u09e1" +
                "\u09f0\u09f1\u09fc\u0a05-\u0a0a\u0a0f\u0a10\u0a13-\u0a28" +
                "\u0a2a-\u0a30\u0a32\u0a33\u0a35\u0a36\u0a38\u0a39" +
                "\u0a59-\u0a5c\u0a5e\u0a72-\u0a74\u0a85-\u0a8d\u0a8f-\u0a91" +
                "\u0a93-\u0aa8\u0aaa-\u0ab0\u0ab2\u0ab3\u0ab5-\u0ab9\u0abd" +
                "\u0ad0\u0ae0\u0ae1\u0af9\u0b05-\u0b0c\u0b0f\u0b10" +
                "\u0b13-\u0b28\u0b2a-\u0b30\u0b32\u0b33\u0b35-\u0b39\u0b3d" +
                "\u0b5c\u0b5d\u0b5f-\u0b61\u0b71\u0b83\u0b85-\u0b8a" +
                "\u0b8e-\u0b90\u0b92-\u0b95\u0b99\u0b9a\u0b9c\u0b9e\u0b9f" +
                "\u0ba3\u0ba4\u0ba8-\u0baa\u0bae-\u0bb9\u0bd0\u0c05-\u0c0c" +
                "\u0c0e-\u0c10\u0c12-\u0c28\u0c2a-\u0c39\u0c3d\u0c58-\u0c5a" +
                "\u0c60\u0c61\u0c80\u0c85-\u0c8c\u0c8e-\u0c90\u0c92-\u0ca8" +
                "\u0caa-\u0cb3\u0cb5-\u0cb9\u0cbd\u0cde\u0ce0\u0ce1" +
                "\u0cf1\u0cf2\u0d05-\u0d0c\u0d0e-\u0d10\u0d12-\u0d3a\u0d3d" +
                "\u0d4e\u0d54-\u0d56\u0d5f-\u0d61\u0d7a-\u0d7f\u0d85-\u0d96" +
                "\u0d9a-\u0db1\u0db3-\u0dbb\u0dbd\u0dc0-\u0dc6\u0e01-\u0e30" +
                "\u0e32\u0e33\u0e40-\u0e46\u0e81\u0e82\u0e84\u0e86-\u0e8a" +
                "\u0e8c-\u0ea3\u0ea5\u0ea7-\u0eb0\u0eb2\u0eb3\u0ebd" +
                "\u0ec0-\u0ec4\u0ec6\u0edc-\u0edf\u0f00\u0f40-\u0f47" +
                "\u0f49-\u0f6c\u0f88-\u0f8c\u1000-\u102a\u103f\u1050-\u1055" +
                "\u105a-\u105d\u1061\u1065\u1066\u106e-\u1070\u1075-\u1081" +
                "\u108e\u10a0-\u10c5\u10c7\u10cd\u10d0-\u10fa\u10fc-\u1248" +
                "\u124a-\u124d\u1250-\u1256\u1258\u125a-\u125d\u1260-\u1288" +
                "\u128a-\u128d\u1290-\u12b0\u12b2-\u12b5\u12b8-\u12be\u12c0" +
                "\u12c2-\u12c5\u12c8-\u12d6\u12d8-\u1310\u1312-\u1315" +
                "\u1318-\u135a\u1380-\u138f\u13a0-\u13f5\u13f8-\u13fd" +
                "\u1401-\u166c\u166f-\u167f\u1681-\u169a\u16a0-\u16ea" +
                "\u16ee-\u16f8\u1700-\u170c\u170e-\u1711\u1720-\u1731" +
                "\u1740-\u1751\u1760-\u176c\u176e-\u1770\u1780-\u17b3\u17d7" +
                "\u17dc\u1820-\u1878\u1880-\u18a8\u18aa\u18b0-\u18f5" +
                "\u1900-\u191e\u1950-\u196d\u1970-\u1974\u1980-\u19ab" +
                "\u19b0-\u19c9\u1a00-\u1a16\u1a20-\u1a54\u1aa7\u1b05-\u1b33" +
                "\u1b45-\u1b4b\u1b83-\u1ba0\u1bae\u1baf\u1bba-\u1be5" +
                "\u1c00-\u1c23\u1c4d-\u1c4f\u1c5a-\u1c7d\u1c80-\u1c88" +
                "\u1c90-\u1cba\u1cbd-\u1cbf\u1ce9-\u1cec\u1cee-\u1cf3" +
                "\u1cf5\u1cf6\u1cfa\u1d00-\u1dbf\u1e00-\u1f15\u1f18-\u1f1d" +
                "\u1f20-\u1f45\u1f48-\u1f4d\u1f50-\u1f57\u1f59\u1f5b\u1f5d" +
                "\u1f5f-\u1f7d\u1f80-\u1fb4\u1fb6-\u1fbc\u1fbe\u1fc2-\u1fc4" +
                "\u1fc6-\u1fcc\u1fd0-\u1fd3\u1fd6-\u1fdb\u1fe0-\u1fec" +
                "\u1ff2-\u1ff4\u1ff6-\u1ffc\u2071\u207f\u2090-\u209c\u2102" +
                "\u2107\u210a-\u2113\u2115\u2118-\u211d\u2124\u2126\u2128" +
                "\u212a-\u2139\u213c-\u213f\u2145-\u2149\u214e\u2160-\u2188" +
                "\u2c00-\u2c2e\u2c30-\u2c5e\u2c60-\u2ce4\u2ceb-\u2cee" +
                "\u2cf2\u2cf3\u2d00-\u2d25\u2d27\u2d2d\u2d30-\u2d67\u2d6f" +
                "\u2d80-\u2d96\u2da0-\u2da6\u2da8-\u2dae\u2db0-\u2db6" +
                "\u2db8-\u2dbe\u2dc0-\u2dc6\u2dc8-\u2dce\u2dd0-\u2dd6" +
                "\u2dd8-\u2dde\u3005-\u3007\u3021-\u3029\u3031-\u3035" +
                "\u3038-\u303c\u3041-\u3096\u309b-\u309f\u30a1-\u30fa" +
                "\u30fc-\u30ff\u3105-\u312f\u3131-\u318e\u31a0-\u31ba" +
                "\u31f0-\u31ff\u3400-\u4db5\u4e00-\u9fef\ua000-\ua48c" +
                "\ua4d0-\ua4fd\ua500-\ua60c\ua610-\ua61f\ua62a\ua62b" +
                "\ua640-\ua66e\ua67f-\ua69d\ua6a0-\ua6ef\ua717-\ua71f" +
                "\ua722-\ua788\ua78b-\ua7bf\ua7c2-\ua7c6\ua7f7-\ua801" +
                "\ua803-\ua805\ua807-\ua80a\ua80c-\ua822\ua840-\ua873" +
                "\ua882-\ua8b3\ua8f2-\ua8f7\ua8fb\ua8fd\ua8fe\ua90a-\ua925" +
                "\ua930-\ua946\ua960-\ua97c\ua984-\ua9b2\ua9cf\ua9e0-\ua9e4" +
                "\ua9e6-\ua9ef\ua9fa-\ua9fe\uaa00-\uaa28\uaa40-\uaa42" +
                "\uaa44-\uaa4b\uaa60-\uaa76\uaa7a\uaa7e-\uaaaf\uaab1" +
                "\uaab5\uaab6\uaab9-\uaabd\uaac0\uaac2\uaadb-\uaadd" +
                "\uaae0-\uaaea\uaaf2-\uaaf4\uab01-\uab06\uab09-\uab0e" +
                "\uab11-\uab16\uab20-\uab26\uab28-\uab2e\uab30-\uab5a" +
                "\uab5c-\uab67\uab70-\uabe2\uac00-\ud7a3\ud7b0-\ud7c6" +
                "\ud7cb-\ud7fb\uf900-\ufa6d\ufa70-\ufad9\ufb00-\ufb06" +
                "\ufb13-\ufb17\ufb1d\ufb1f-\ufb28\ufb2a-\ufb36\ufb38-\ufb3c" +
                "\ufb3e\ufb40\ufb41\ufb43\ufb44\ufb46-\ufbb1\ufbd3-\ufd3d" +
                "\ufd50-\ufd8f\ufd92-\ufdc7\ufdf0-\ufdfb\ufe70-\ufe74" +
                "\ufe76-\ufefc\uff21-\uff3a\uff41-\uff5a\uff66-\uffbe" +
                "\uffc2-\uffc7\uffca-\uffcf\uffd2-\uffd7\uffda-\uffdc" +
                "\u{10000}-\u{1000b}\u{1000d}-\u{10026}\u{10028}-\u{1003a}" +
                "\u{1003c}\u{1003d}\u{1003f}-\u{1004d}\u{10050}-\u{1005d}" +
                "\u{10080}-\u{100fa}\u{10140}-\u{10174}\u{10280}-\u{1029c}" +
                "\u{102a0}-\u{102d0}\u{10300}-\u{1031f}\u{1032d}-\u{1034a}" +
                "\u{10350}-\u{10375}\u{10380}-\u{1039d}\u{103a0}-\u{103c3}" +
                "\u{103c8}-\u{103cf}\u{103d1}-\u{103d5}\u{10400}-\u{1049d}" +
                "\u{104b0}-\u{104d3}\u{104d8}-\u{104fb}\u{10500}-\u{10527}" +
                "\u{10530}-\u{10563}\u{10600}-\u{10736}\u{10740}-\u{10755}" +
                "\u{10760}-\u{10767}\u{10800}-\u{10805}\u{10808}" +
                "\u{1080a}-\u{10835}\u{10837}\u{10838}\u{1083c}" +
                "\u{1083f}-\u{10855}\u{10860}-\u{10876}\u{10880}-\u{1089e}" +
                "\u{108e0}-\u{108f2}\u{108f4}\u{108f5}\u{10900}-\u{10915}" +
                "\u{10920}-\u{10939}\u{10980}-\u{109b7}\u{109be}\u{109bf}" +
                "\u{10a00}\u{10a10}-\u{10a13}\u{10a15}-\u{10a17}" +
                "\u{10a19}-\u{10a35}\u{10a60}-\u{10a7c}\u{10a80}-\u{10a9c}" +
                "\u{10ac0}-\u{10ac7}\u{10ac9}-\u{10ae4}\u{10b00}-\u{10b35}" +
                "\u{10b40}-\u{10b55}\u{10b60}-\u{10b72}\u{10b80}-\u{10b91}" +
                "\u{10c00}-\u{10c48}\u{10c80}-\u{10cb2}\u{10cc0}-\u{10cf2}" +
                "\u{10d00}-\u{10d23}\u{10f00}-\u{10f1c}\u{10f27}" +
                "\u{10f30}-\u{10f45}\u{10fe0}-\u{10ff6}\u{11003}-\u{11037}" +
                "\u{11083}-\u{110af}\u{110d0}-\u{110e8}\u{11103}-\u{11126}" +
                "\u{11144}\u{11150}-\u{11172}\u{11176}\u{11183}-\u{111b2}" +
                "\u{111c1}-\u{111c4}\u{111da}\u{111dc}\u{11200}-\u{11211}" +
                "\u{11213}-\u{1122b}\u{11280}-\u{11286}\u{11288}" +
                "\u{1128a}-\u{1128d}\u{1128f}-\u{1129d}\u{1129f}-\u{112a8}" +
                "\u{112b0}-\u{112de}\u{11305}-\u{1130c}\u{1130f}\u{11310}" +
                "\u{11313}-\u{11328}\u{1132a}-\u{11330}\u{11332}\u{11333}" +
                "\u{11335}-\u{11339}\u{1133d}\u{11350}\u{1135d}-\u{11361}" +
                "\u{11400}-\u{11434}\u{11447}-\u{1144a}\u{1145f}" +
                "\u{11480}-\u{114af}\u{114c4}\u{114c5}\u{114c7}" +
                "\u{11580}-\u{115ae}\u{115d8}-\u{115db}\u{11600}-\u{1162f}" +
                "\u{11644}\u{11680}-\u{116aa}\u{116b8}\u{11700}-\u{1171a}" +
                "\u{11800}-\u{1182b}\u{118a0}-\u{118df}\u{118ff}" +
                "\u{119a0}-\u{119a7}\u{119aa}-\u{119d0}\u{119e1}\u{119e3}" +
                "\u{11a00}\u{11a0b}-\u{11a32}\u{11a3a}\u{11a50}" +
                "\u{11a5c}-\u{11a89}\u{11a9d}\u{11ac0}-\u{11af8}" +
                "\u{11c00}-\u{11c08}\u{11c0a}-\u{11c2e}\u{11c40}" +
                "\u{11c72}-\u{11c8f}\u{11d00}-\u{11d06}\u{11d08}\u{11d09}" +
                "\u{11d0b}-\u{11d30}\u{11d46}\u{11d60}-\u{11d65}" +
                "\u{11d67}\u{11d68}\u{11d6a}-\u{11d89}\u{11d98}" +
                "\u{11ee0}-\u{11ef2}\u{12000}-\u{12399}\u{12400}-\u{1246e}" +
                "\u{12480}-\u{12543}\u{13000}-\u{1342e}\u{14400}-\u{14646}" +
                "\u{16800}-\u{16a38}\u{16a40}-\u{16a5e}\u{16ad0}-\u{16aed}" +
                "\u{16b00}-\u{16b2f}\u{16b40}-\u{16b43}\u{16b63}-\u{16b77}" +
                "\u{16b7d}-\u{16b8f}\u{16e40}-\u{16e7f}\u{16f00}-\u{16f4a}" +
                "\u{16f50}\u{16f93}-\u{16f9f}\u{16fe0}\u{16fe1}\u{16fe3}" +
                "\u{17000}-\u{187f7}\u{18800}-\u{18af2}\u{1b000}-\u{1b11e}" +
                "\u{1b150}-\u{1b152}\u{1b164}-\u{1b167}\u{1b170}-\u{1b2fb}" +
                "\u{1bc00}-\u{1bc6a}\u{1bc70}-\u{1bc7c}\u{1bc80}-\u{1bc88}" +
                "\u{1bc90}-\u{1bc99}\u{1d400}-\u{1d454}\u{1d456}-\u{1d49c}" +
                "\u{1d49e}\u{1d49f}\u{1d4a2}\u{1d4a5}\u{1d4a6}" +
                "\u{1d4a9}-\u{1d4ac}\u{1d4ae}-\u{1d4b9}\u{1d4bb}" +
                "\u{1d4bd}-\u{1d4c3}\u{1d4c5}-\u{1d505}\u{1d507}-\u{1d50a}" +
                "\u{1d50d}-\u{1d514}\u{1d516}-\u{1d51c}\u{1d51e}-\u{1d539}" +
                "\u{1d53b}-\u{1d53e}\u{1d540}-\u{1d544}\u{1d546}" +
                "\u{1d54a}-\u{1d550}\u{1d552}-\u{1d6a5}\u{1d6a8}-\u{1d6c0}" +
                "\u{1d6c2}-\u{1d6da}\u{1d6dc}-\u{1d6fa}\u{1d6fc}-\u{1d714}" +
                "\u{1d716}-\u{1d734}\u{1d736}-\u{1d74e}\u{1d750}-\u{1d76e}" +
                "\u{1d770}-\u{1d788}\u{1d78a}-\u{1d7a8}\u{1d7aa}-\u{1d7c2}" +
                "\u{1d7c4}-\u{1d7cb}\u{1e100}-\u{1e12c}\u{1e137}-\u{1e13d}" +
                "\u{1e14e}\u{1e2c0}-\u{1e2eb}\u{1e800}-\u{1e8c4}" +
                "\u{1e900}-\u{1e943}\u{1e94b}\u{1ee00}-\u{1ee03}" +
                "\u{1ee05}-\u{1ee1f}\u{1ee21}\u{1ee22}\u{1ee24}\u{1ee27}" +
                "\u{1ee29}-\u{1ee32}\u{1ee34}-\u{1ee37}\u{1ee39}\u{1ee3b}" +
                "\u{1ee42}\u{1ee47}\u{1ee49}\u{1ee4b}\u{1ee4d}-\u{1ee4f}" +
                "\u{1ee51}\u{1ee52}\u{1ee54}\u{1ee57}\u{1ee59}\u{1ee5b}" +
                "\u{1ee5d}\u{1ee5f}\u{1ee61}\u{1ee62}\u{1ee64}" +
                "\u{1ee67}-\u{1ee6a}\u{1ee6c}-\u{1ee72}\u{1ee74}-\u{1ee77}" +
                "\u{1ee79}-\u{1ee7c}\u{1ee7e}\u{1ee80}-\u{1ee89}" +
                "\u{1ee8b}-\u{1ee9b}\u{1eea1}-\u{1eea3}\u{1eea5}-\u{1eea9}" +
                "\u{1eeab}-\u{1eebb}\u{20000}-\u{2a6d6}\u{2a700}-\u{2b734}" +
                "\u{2b740}-\u{2b81d}\u{2b820}-\u{2cea1}\u{2ceb0}-\u{2ebe0}" +
                "\u{2f800}-\u{2fa1d}]$", "u");
        }
        return largeIdStartPattern.test(String.fromCodePoint(cp));
    }
    function isLargeIdContinue(cp) {
        if (!largeIdContinuePattern) {
            largeIdContinuePattern = new RegExp("^[\xb7\u0300-\u036f\u0387\u0483-\u0487\u0591-\u05bd\u05bf" +
                "\u05c1\u05c2\u05c4\u05c5\u05c7\u0610-\u061a\u064b-\u0669" +
                "\u0670\u06d6-\u06dc\u06df-\u06e4\u06e7\u06e8\u06ea-\u06ed" +
                "\u06f0-\u06f9\u0711\u0730-\u074a\u07a6-\u07b0\u07c0-\u07c9" +
                "\u07eb-\u07f3\u07fd\u0816-\u0819\u081b-\u0823\u0825-\u0827" +
                "\u0829-\u082d\u0859-\u085b\u08d3-\u08e1\u08e3-\u0903" +
                "\u093a-\u093c\u093e-\u094f\u0951-\u0957\u0962\u0963" +
                "\u0966-\u096f\u0981-\u0983\u09bc\u09be-\u09c4\u09c7\u09c8" +
                "\u09cb-\u09cd\u09d7\u09e2\u09e3\u09e6-\u09ef\u09fe" +
                "\u0a01-\u0a03\u0a3c\u0a3e-\u0a42\u0a47\u0a48\u0a4b-\u0a4d" +
                "\u0a51\u0a66-\u0a71\u0a75\u0a81-\u0a83\u0abc\u0abe-\u0ac5" +
                "\u0ac7-\u0ac9\u0acb-\u0acd\u0ae2\u0ae3\u0ae6-\u0aef" +
                "\u0afa-\u0aff\u0b01-\u0b03\u0b3c\u0b3e-\u0b44\u0b47\u0b48" +
                "\u0b4b-\u0b4d\u0b56\u0b57\u0b62\u0b63\u0b66-\u0b6f\u0b82" +
                "\u0bbe-\u0bc2\u0bc6-\u0bc8\u0bca-\u0bcd\u0bd7\u0be6-\u0bef" +
                "\u0c00-\u0c04\u0c3e-\u0c44\u0c46-\u0c48\u0c4a-\u0c4d" +
                "\u0c55\u0c56\u0c62\u0c63\u0c66-\u0c6f\u0c81-\u0c83\u0cbc" +
                "\u0cbe-\u0cc4\u0cc6-\u0cc8\u0cca-\u0ccd\u0cd5\u0cd6" +
                "\u0ce2\u0ce3\u0ce6-\u0cef\u0d00-\u0d03\u0d3b\u0d3c" +
                "\u0d3e-\u0d44\u0d46-\u0d48\u0d4a-\u0d4d\u0d57\u0d62\u0d63" +
                "\u0d66-\u0d6f\u0d82\u0d83\u0dca\u0dcf-\u0dd4\u0dd6" +
                "\u0dd8-\u0ddf\u0de6-\u0def\u0df2\u0df3\u0e31\u0e34-\u0e3a" +
                "\u0e47-\u0e4e\u0e50-\u0e59\u0eb1\u0eb4-\u0ebc\u0ec8-\u0ecd" +
                "\u0ed0-\u0ed9\u0f18\u0f19\u0f20-\u0f29\u0f35\u0f37\u0f39" +
                "\u0f3e\u0f3f\u0f71-\u0f84\u0f86\u0f87\u0f8d-\u0f97" +
                "\u0f99-\u0fbc\u0fc6\u102b-\u103e\u1040-\u1049\u1056-\u1059" +
                "\u105e-\u1060\u1062-\u1064\u1067-\u106d\u1071-\u1074" +
                "\u1082-\u108d\u108f-\u109d\u135d-\u135f\u1369-\u1371" +
                "\u1712-\u1714\u1732-\u1734\u1752\u1753\u1772\u1773" +
                "\u17b4-\u17d3\u17dd\u17e0-\u17e9\u180b-\u180d\u1810-\u1819" +
                "\u18a9\u1920-\u192b\u1930-\u193b\u1946-\u194f\u19d0-\u19da" +
                "\u1a17-\u1a1b\u1a55-\u1a5e\u1a60-\u1a7c\u1a7f-\u1a89" +
                "\u1a90-\u1a99\u1ab0-\u1abd\u1b00-\u1b04\u1b34-\u1b44" +
                "\u1b50-\u1b59\u1b6b-\u1b73\u1b80-\u1b82\u1ba1-\u1bad" +
                "\u1bb0-\u1bb9\u1be6-\u1bf3\u1c24-\u1c37\u1c40-\u1c49" +
                "\u1c50-\u1c59\u1cd0-\u1cd2\u1cd4-\u1ce8\u1ced\u1cf4" +
                "\u1cf7-\u1cf9\u1dc0-\u1df9\u1dfb-\u1dff\u203f\u2040\u2054" +
                "\u20d0-\u20dc\u20e1\u20e5-\u20f0\u2cef-\u2cf1\u2d7f" +
                "\u2de0-\u2dff\u302a-\u302f\u3099\u309a\ua620-\ua629\ua66f" +
                "\ua674-\ua67d\ua69e\ua69f\ua6f0\ua6f1\ua802\ua806\ua80b" +
                "\ua823-\ua827\ua880\ua881\ua8b4-\ua8c5\ua8d0-\ua8d9" +
                "\ua8e0-\ua8f1\ua8ff-\ua909\ua926-\ua92d\ua947-\ua953" +
                "\ua980-\ua983\ua9b3-\ua9c0\ua9d0-\ua9d9\ua9e5\ua9f0-\ua9f9" +
                "\uaa29-\uaa36\uaa43\uaa4c\uaa4d\uaa50-\uaa59\uaa7b-\uaa7d" +
                "\uaab0\uaab2-\uaab4\uaab7\uaab8\uaabe\uaabf\uaac1" +
                "\uaaeb-\uaaef\uaaf5\uaaf6\uabe3-\uabea\uabec\uabed" +
                "\uabf0-\uabf9\ufb1e\ufe00-\ufe0f\ufe20-\ufe2f\ufe33\ufe34" +
                "\ufe4d-\ufe4f\uff10-\uff19\uff3f\u{101fd}\u{102e0}" +
                "\u{10376}-\u{1037a}\u{104a0}-\u{104a9}\u{10a01}-\u{10a03}" +
                "\u{10a05}\u{10a06}\u{10a0c}-\u{10a0f}\u{10a38}-\u{10a3a}" +
                "\u{10a3f}\u{10ae5}\u{10ae6}\u{10d24}-\u{10d27}" +
                "\u{10d30}-\u{10d39}\u{10f46}-\u{10f50}\u{11000}-\u{11002}" +
                "\u{11038}-\u{11046}\u{11066}-\u{1106f}\u{1107f}-\u{11082}" +
                "\u{110b0}-\u{110ba}\u{110f0}-\u{110f9}\u{11100}-\u{11102}" +
                "\u{11127}-\u{11134}\u{11136}-\u{1113f}\u{11145}\u{11146}" +
                "\u{11173}\u{11180}-\u{11182}\u{111b3}-\u{111c0}" +
                "\u{111c9}-\u{111cc}\u{111d0}-\u{111d9}\u{1122c}-\u{11237}" +
                "\u{1123e}\u{112df}-\u{112ea}\u{112f0}-\u{112f9}" +
                "\u{11300}-\u{11303}\u{1133b}\u{1133c}\u{1133e}-\u{11344}" +
                "\u{11347}\u{11348}\u{1134b}-\u{1134d}\u{11357}" +
                "\u{11362}\u{11363}\u{11366}-\u{1136c}\u{11370}-\u{11374}" +
                "\u{11435}-\u{11446}\u{11450}-\u{11459}\u{1145e}" +
                "\u{114b0}-\u{114c3}\u{114d0}-\u{114d9}\u{115af}-\u{115b5}" +
                "\u{115b8}-\u{115c0}\u{115dc}\u{115dd}\u{11630}-\u{11640}" +
                "\u{11650}-\u{11659}\u{116ab}-\u{116b7}\u{116c0}-\u{116c9}" +
                "\u{1171d}-\u{1172b}\u{11730}-\u{11739}\u{1182c}-\u{1183a}" +
                "\u{118e0}-\u{118e9}\u{119d1}-\u{119d7}\u{119da}-\u{119e0}" +
                "\u{119e4}\u{11a01}-\u{11a0a}\u{11a33}-\u{11a39}" +
                "\u{11a3b}-\u{11a3e}\u{11a47}\u{11a51}-\u{11a5b}" +
                "\u{11a8a}-\u{11a99}\u{11c2f}-\u{11c36}\u{11c38}-\u{11c3f}" +
                "\u{11c50}-\u{11c59}\u{11c92}-\u{11ca7}\u{11ca9}-\u{11cb6}" +
                "\u{11d31}-\u{11d36}\u{11d3a}\u{11d3c}\u{11d3d}" +
                "\u{11d3f}-\u{11d45}\u{11d47}\u{11d50}-\u{11d59}" +
                "\u{11d8a}-\u{11d8e}\u{11d90}\u{11d91}\u{11d93}-\u{11d97}" +
                "\u{11da0}-\u{11da9}\u{11ef3}-\u{11ef6}\u{16a60}-\u{16a69}" +
                "\u{16af0}-\u{16af4}\u{16b30}-\u{16b36}\u{16b50}-\u{16b59}" +
                "\u{16f4f}\u{16f51}-\u{16f87}\u{16f8f}-\u{16f92}" +
                "\u{1bc9d}\u{1bc9e}\u{1d165}-\u{1d169}\u{1d16d}-\u{1d172}" +
                "\u{1d17b}-\u{1d182}\u{1d185}-\u{1d18b}\u{1d1aa}-\u{1d1ad}" +
                "\u{1d242}-\u{1d244}\u{1d7ce}-\u{1d7ff}\u{1da00}-\u{1da36}" +
                "\u{1da3b}-\u{1da6c}\u{1da75}\u{1da84}\u{1da9b}-\u{1da9f}" +
                "\u{1daa1}-\u{1daaf}\u{1e000}-\u{1e006}\u{1e008}-\u{1e018}" +
                "\u{1e01b}-\u{1e021}\u{1e023}\u{1e024}\u{1e026}-\u{1e02a}" +
                "\u{1e130}-\u{1e136}\u{1e140}-\u{1e149}\u{1e2ec}-\u{1e2f9}" +
                "\u{1e8d0}-\u{1e8d6}\u{1e944}-\u{1e94a}\u{1e950}-\u{1e959}" +
                "\u{e0100}-\u{e01ef}]$", "u");
        }
        return largeIdContinuePattern.test(String.fromCodePoint(cp));
    }
    const gcNamePattern = /^(?:General_Category|gc)$/u;
    const scNamePattern = /^(?:Script(?:_Extensions)?|scx?)$/u;
    const gcValuePatterns = {
        es2018: null,
        es2019: null,
        es2020: null,
    };
    const scValuePatterns = {
        es2018: null,
        es2019: null,
        es2020: null,
    };
    const binPropertyPatterns = {
        es2018: null,
        es2019: null,
        es2020: null,
    };
    function isValidUnicodeProperty(version, name, value) {
        if (gcNamePattern.test(name)) {
            if (version >= 2018) {
                if (!gcValuePatterns.es2018) {
                    gcValuePatterns.es2018 = new RegExp("^(?:C|Cased_Letter|Cc|Cf|Close_Punctuation|Cn|Co|" +
                        "Combining_Mark|Connector_Punctuation|Control|Cs|" +
                        "Currency_Symbol|Dash_Punctuation|Decimal_Number|" +
                        "Enclosing_Mark|Final_Punctuation|Format|" +
                        "Initial_Punctuation|L|LC|Letter|Letter_Number|" +
                        "Line_Separator|Ll|Lm|Lo|Lowercase_Letter|Lt|Lu|M|" +
                        "Mark|Math_Symbol|Mc|Me|Mn|Modifier_Letter|" +
                        "Modifier_Symbol|N|Nd|Nl|No|Nonspacing_Mark|Number|" +
                        "Open_Punctuation|Other|Other_Letter|Other_Number|" +
                        "Other_Punctuation|Other_Symbol|P|" +
                        "Paragraph_Separator|Pc|Pd|Pe|Pf|Pi|Po|Private_Use|" +
                        "Ps|Punctuation|S|Sc|Separator|Sk|Sm|So|" +
                        "Space_Separator|Spacing_Mark|Surrogate|Symbol|" +
                        "Titlecase_Letter|Unassigned|Uppercase_Letter|Z|Zl|" +
                        "Zp|Zs|cntrl|digit|punct)$", "u");
                }
                if (gcValuePatterns.es2018.test(value)) {
                    return true;
                }
            }
        }
        if (scNamePattern.test(name)) {
            if (version >= 2018) {
                if (!scValuePatterns.es2018) {
                    scValuePatterns.es2018 = new RegExp("^(?:Adlam|Adlm|Aghb|Ahom|Anatolian_Hieroglyphs|Arab|" +
                        "Arabic|Armenian|Armi|Armn|Avestan|Avst|Bali|" +
                        "Balinese|Bamu|Bamum|Bass|Bassa_Vah|Batak|Batk|Beng|" +
                        "Bengali|Bhaiksuki|Bhks|Bopo|Bopomofo|Brah|Brahmi|" +
                        "Brai|Braille|Bugi|Buginese|Buhd|Buhid|Cakm|" +
                        "Canadian_Aboriginal|Cans|Cari|Carian|" +
                        "Caucasian_Albanian|Chakma|Cham|Cher|Cherokee|Common|" +
                        "Copt|Coptic|Cprt|Cuneiform|Cypriot|Cyrillic|Cyrl|" +
                        "Deseret|Deva|Devanagari|Dsrt|Dupl|Duployan|Egyp|" +
                        "Egyptian_Hieroglyphs|Elba|Elbasan|Ethi|Ethiopic|" +
                        "Geor|Georgian|Glag|Glagolitic|Gonm|Goth|Gothic|Gran|" +
                        "Grantha|Greek|Grek|Gujarati|Gujr|Gurmukhi|Guru|Han|" +
                        "Hang|Hangul|Hani|Hano|Hanunoo|Hatr|Hatran|Hebr|" +
                        "Hebrew|Hira|Hiragana|Hluw|Hmng|Hung|" +
                        "Imperial_Aramaic|Inherited|Inscriptional_Pahlavi|" +
                        "Inscriptional_Parthian|Ital|Java|Javanese|Kaithi|" +
                        "Kali|Kana|Kannada|Katakana|Kayah_Li|Khar|Kharoshthi|" +
                        "Khmer|Khmr|Khoj|Khojki|Khudawadi|Knda|Kthi|Lana|Lao|" +
                        "Laoo|Latin|Latn|Lepc|Lepcha|Limb|Limbu|Lina|Linb|" +
                        "Linear_A|Linear_B|Lisu|Lyci|Lycian|Lydi|Lydian|" +
                        "Mahajani|Mahj|Malayalam|Mand|Mandaic|Mani|" +
                        "Manichaean|Marc|Marchen|Masaram_Gondi|Meetei_Mayek|" +
                        "Mend|Mende_Kikakui|Merc|Mero|Meroitic_Cursive|" +
                        "Meroitic_Hieroglyphs|Miao|Mlym|Modi|Mong|Mongolian|" +
                        "Mro|Mroo|Mtei|Mult|Multani|Myanmar|Mymr|Nabataean|" +
                        "Narb|Nbat|New_Tai_Lue|Newa|Nko|Nkoo|Nshu|Nushu|Ogam|" +
                        "Ogham|Ol_Chiki|Olck|Old_Hungarian|Old_Italic|" +
                        "Old_North_Arabian|Old_Permic|Old_Persian|" +
                        "Old_South_Arabian|Old_Turkic|Oriya|Orkh|Orya|Osage|" +
                        "Osge|Osma|Osmanya|Pahawh_Hmong|Palm|Palmyrene|" +
                        "Pau_Cin_Hau|Pauc|Perm|Phag|Phags_Pa|Phli|Phlp|Phnx|" +
                        "Phoenician|Plrd|Prti|Psalter_Pahlavi|Qaac|Qaai|" +
                        "Rejang|Rjng|Runic|Runr|Samaritan|Samr|Sarb|Saur|" +
                        "Saurashtra|Sgnw|Sharada|Shavian|Shaw|Shrd|Sidd|" +
                        "Siddham|SignWriting|Sind|Sinh|Sinhala|Sora|" +
                        "Sora_Sompeng|Soyo|Soyombo|Sund|Sundanese|Sylo|" +
                        "Syloti_Nagri|Syrc|Syriac|Tagalog|Tagb|Tagbanwa|" +
                        "Tai_Le|Tai_Tham|Tai_Viet|Takr|Takri|Tale|Talu|Tamil|" +
                        "Taml|Tang|Tangut|Tavt|Telu|Telugu|Tfng|Tglg|Thaa|" +
                        "Thaana|Thai|Tibetan|Tibt|Tifinagh|Tirh|Tirhuta|Ugar|" +
                        "Ugaritic|Vai|Vaii|Wara|Warang_Citi|Xpeo|Xsux|Yi|" +
                        "Yiii|Zanabazar_Square|Zanb|Zinh|Zyyy)$", "u");
                }
                if (scValuePatterns.es2018.test(value)) {
                    return true;
                }
            }
            if (version >= 2019) {
                if (!scValuePatterns.es2019) {
                    scValuePatterns.es2019 = new RegExp("^(?:Dogr|Dogra|Gong|Gunjala_Gondi|Hanifi_Rohingya|" +
                        "Maka|Makasar|Medefaidrin|Medf|Old_Sogdian|Rohg|Sogd|" +
                        "Sogdian|Sogo)$", "u");
                }
                if (scValuePatterns.es2019.test(value)) {
                    return true;
                }
            }
            if (version >= 2020) {
                if (!scValuePatterns.es2020) {
                    scValuePatterns.es2020 = new RegExp("^(?:Hmnp|Nand|Nandinagari|Nyiakeng_Puachue_Hmong|" +
                        "Wancho|Wcho)$", "u");
                }
                if (scValuePatterns.es2020.test(value)) {
                    return true;
                }
            }
        }
        return false;
    }
    function isValidLoneUnicodeProperty(version, value) {
        if (version >= 2018) {
            if (!binPropertyPatterns.es2018) {
                binPropertyPatterns.es2018 = new RegExp("^(?:AHex|ASCII|ASCII_Hex_Digit|Alpha|Alphabetic|Any|" +
                    "Assigned|Bidi_C|Bidi_Control|Bidi_M|Bidi_Mirrored|CI|" +
                    "CWCF|CWCM|CWKCF|CWL|CWT|CWU|Case_Ignorable|Cased|" +
                    "Changes_When_Casefolded|Changes_When_Casemapped|" +
                    "Changes_When_Lowercased|Changes_When_NFKC_Casefolded|" +
                    "Changes_When_Titlecased|Changes_When_Uppercased|DI|Dash|" +
                    "Default_Ignorable_Code_Point|Dep|Deprecated|Dia|" +
                    "Diacritic|Emoji|Emoji_Component|Emoji_Modifier|" +
                    "Emoji_Modifier_Base|Emoji_Presentation|Ext|Extender|" +
                    "Gr_Base|Gr_Ext|Grapheme_Base|Grapheme_Extend|Hex|" +
                    "Hex_Digit|IDC|IDS|IDSB|IDST|IDS_Binary_Operator|" +
                    "IDS_Trinary_Operator|ID_Continue|ID_Start|Ideo|" +
                    "Ideographic|Join_C|Join_Control|LOE|" +
                    "Logical_Order_Exception|Lower|Lowercase|Math|NChar|" +
                    "Noncharacter_Code_Point|Pat_Syn|Pat_WS|Pattern_Syntax|" +
                    "Pattern_White_Space|QMark|Quotation_Mark|RI|Radical|" +
                    "Regional_Indicator|SD|STerm|Sentence_Terminal|" +
                    "Soft_Dotted|Term|Terminal_Punctuation|UIdeo|" +
                    "Unified_Ideograph|Upper|Uppercase|VS|Variation_Selector|" +
                    "White_Space|XIDC|XIDS|XID_Continue|XID_Start|space)$", "u");
            }
            if (binPropertyPatterns.es2018.test(value)) {
                return true;
            }
        }
        if (version >= 2019) {
            if (!binPropertyPatterns.es2019) {
                binPropertyPatterns.es2019 = new RegExp("^(?:Extended_Pictographic)$", "u");
            }
            if (binPropertyPatterns.es2019.test(value)) {
                return true;
            }
        }
        return false;
    }
    const Backspace = 0x08;
    const CharacterTabulation = 0x09;
    const LineFeed = 0x0a;
    const LineTabulation = 0x0b;
    const FormFeed = 0x0c;
    const CarriageReturn = 0x0d;
    const ExclamationMark = 0x21;
    const DollarSign = 0x24;
    const LeftParenthesis = 0x28;
    const RightParenthesis = 0x29;
    const Asterisk = 0x2a;
    const PlusSign = 0x2b;
    const Comma = 0x2c;
    const HyphenMinus = 0x2d;
    const FullStop = 0x2e;
    const Solidus = 0x2f;
    const DigitZero = 0x30;
    const DigitOne = 0x31;
    const DigitSeven = 0x37;
    const DigitNine = 0x39;
    const Colon = 0x3a;
    const LessThanSign = 0x3c;
    const EqualsSign = 0x3d;
    const GreaterThanSign = 0x3e;
    const QuestionMark = 0x3f;
    const LatinCapitalLetterA = 0x41;
    const LatinCapitalLetterB = 0x42;
    const LatinCapitalLetterD = 0x44;
    const LatinCapitalLetterF = 0x46;
    const LatinCapitalLetterP = 0x50;
    const LatinCapitalLetterS = 0x53;
    const LatinCapitalLetterW = 0x57;
    const LatinCapitalLetterZ = 0x5a;
    const LowLine = 0x5f;
    const LatinSmallLetterA = 0x61;
    const LatinSmallLetterB = 0x62;
    const LatinSmallLetterC = 0x63;
    const LatinSmallLetterD = 0x64;
    const LatinSmallLetterF = 0x66;
    const LatinSmallLetterG = 0x67;
    const LatinSmallLetterI = 0x69;
    const LatinSmallLetterK = 0x6b;
    const LatinSmallLetterM = 0x6d;
    const LatinSmallLetterN = 0x6e;
    const LatinSmallLetterP = 0x70;
    const LatinSmallLetterR = 0x72;
    const LatinSmallLetterS = 0x73;
    const LatinSmallLetterT = 0x74;
    const LatinSmallLetterU = 0x75;
    const LatinSmallLetterV = 0x76;
    const LatinSmallLetterW = 0x77;
    const LatinSmallLetterX = 0x78;
    const LatinSmallLetterY = 0x79;
    const LatinSmallLetterZ = 0x7a;
    const LeftSquareBracket = 0x5b;
    const ReverseSolidus = 0x5c;
    const RightSquareBracket = 0x5d;
    const CircumflexAccent = 0x5e;
    const LeftCurlyBracket = 0x7b;
    const VerticalLine = 0x7c;
    const RightCurlyBracket = 0x7d;
    const ZeroWidthNonJoiner = 0x200c;
    const ZeroWidthJoiner = 0x200d;
    const LineSeparator = 0x2028;
    const ParagraphSeparator = 0x2029;
    const MinCodePoint = 0x00;
    const MaxCodePoint = 0x10ffff;
    function isLatinLetter(code) {
        return ((code >= LatinCapitalLetterA && code <= LatinCapitalLetterZ) ||
            (code >= LatinSmallLetterA && code <= LatinSmallLetterZ));
    }
    function isDecimalDigit(code) {
        return code >= DigitZero && code <= DigitNine;
    }
    function isOctalDigit(code) {
        return code >= DigitZero && code <= DigitSeven;
    }
    function isHexDigit(code) {
        return ((code >= DigitZero && code <= DigitNine) ||
            (code >= LatinCapitalLetterA && code <= LatinCapitalLetterF) ||
            (code >= LatinSmallLetterA && code <= LatinSmallLetterF));
    }
    function isLineTerminator(code) {
        return (code === LineFeed ||
            code === CarriageReturn ||
            code === LineSeparator ||
            code === ParagraphSeparator);
    }
    function isValidUnicode(code) {
        return code >= MinCodePoint && code <= MaxCodePoint;
    }
    function digitToInt(code) {
        if (code >= LatinSmallLetterA && code <= LatinSmallLetterF) {
            return code - LatinSmallLetterA + 10;
        }
        if (code >= LatinCapitalLetterA && code <= LatinCapitalLetterF) {
            return code - LatinCapitalLetterA + 10;
        }
        return code - DigitZero;
    }
    const legacyImpl = {
        at(s, end, i) {
            return i < end ? s.charCodeAt(i) : -1;
        },
        width(c) {
            return 1;
        },
    };
    const unicodeImpl = {
        at(s, end, i) {
            return i < end ? s.codePointAt(i) : -1;
        },
        width(c) {
            return c > 0xffff ? 2 : 1;
        },
    };
    class Reader {
        constructor() {
            this._impl = legacyImpl;
            this._s = "";
            this._i = 0;
            this._end = 0;
            this._cp1 = -1;
            this._w1 = 1;
            this._cp2 = -1;
            this._w2 = 1;
            this._cp3 = -1;
            this._w3 = 1;
            this._cp4 = -1;
        }
        get source() {
            return this._s;
        }
        get index() {
            return this._i;
        }
        get currentCodePoint() {
            return this._cp1;
        }
        get nextCodePoint() {
            return this._cp2;
        }
        get nextCodePoint2() {
            return this._cp3;
        }
        get nextCodePoint3() {
            return this._cp4;
        }
        reset(source, start, end, uFlag) {
            this._impl = uFlag ? unicodeImpl : legacyImpl;
            this._s = source;
            this._end = end;
            this.rewind(start);
        }
        rewind(index) {
            const impl = this._impl;
            this._i = index;
            this._cp1 = impl.at(this._s, this._end, index);
            this._w1 = impl.width(this._cp1);
            this._cp2 = impl.at(this._s, this._end, index + this._w1);
            this._w2 = impl.width(this._cp2);
            this._cp3 = impl.at(this._s, this._end, index + this._w1 + this._w2);
            this._w3 = impl.width(this._cp3);
            this._cp4 = impl.at(this._s, this._end, index + this._w1 + this._w2 + this._w3);
        }
        advance() {
            if (this._cp1 !== -1) {
                const impl = this._impl;
                this._i += this._w1;
                this._cp1 = this._cp2;
                this._w1 = this._w2;
                this._cp2 = this._cp3;
                this._w2 = impl.width(this._cp2);
                this._cp3 = this._cp4;
                this._w3 = impl.width(this._cp3);
                this._cp4 = impl.at(this._s, this._end, this._i + this._w1 + this._w2 + this._w3);
            }
        }
        eat(cp) {
            if (this._cp1 === cp) {
                this.advance();
                return true;
            }
            return false;
        }
        eat2(cp1, cp2) {
            if (this._cp1 === cp1 && this._cp2 === cp2) {
                this.advance();
                this.advance();
                return true;
            }
            return false;
        }
        eat3(cp1, cp2, cp3) {
            if (this._cp1 === cp1 && this._cp2 === cp2 && this._cp3 === cp3) {
                this.advance();
                this.advance();
                this.advance();
                return true;
            }
            return false;
        }
    }
    class RegExpSyntaxError extends SyntaxError {
        constructor(source, uFlag, index, message) {
            if (source) {
                if (!source.startsWith("/")) {
                    source = `/${source}/${uFlag ? "u" : ""}`;
                }
                source = `: ${source}`;
            }
            super(`Invalid regular expression${source}: ${message}`);
            this.index = index;
        }
    }
    function isSyntaxCharacter(cp) {
        return (cp === CircumflexAccent ||
            cp === DollarSign ||
            cp === ReverseSolidus ||
            cp === FullStop ||
            cp === Asterisk ||
            cp === PlusSign ||
            cp === QuestionMark ||
            cp === LeftParenthesis ||
            cp === RightParenthesis ||
            cp === LeftSquareBracket ||
            cp === RightSquareBracket ||
            cp === LeftCurlyBracket ||
            cp === RightCurlyBracket ||
            cp === VerticalLine);
    }
    function isRegExpIdentifierStart(cp) {
        return isIdStart(cp) || cp === DollarSign || cp === LowLine;
    }
    function isRegExpIdentifierPart(cp) {
        return (isIdContinue(cp) ||
            cp === DollarSign ||
            cp === LowLine ||
            cp === ZeroWidthNonJoiner ||
            cp === ZeroWidthJoiner);
    }
    function isUnicodePropertyNameCharacter(cp) {
        return isLatinLetter(cp) || cp === LowLine;
    }
    function isUnicodePropertyValueCharacter(cp) {
        return isUnicodePropertyNameCharacter(cp) || isDecimalDigit(cp);
    }
    class RegExpValidator {
        constructor(options) {
            this._reader = new Reader();
            this._uFlag = false;
            this._nFlag = false;
            this._lastIntValue = 0;
            this._lastMinValue = 0;
            this._lastMaxValue = 0;
            this._lastStrValue = "";
            this._lastKeyValue = "";
            this._lastValValue = "";
            this._lastAssertionIsQuantifiable = false;
            this._numCapturingParens = 0;
            this._groupNames = new Set();
            this._backreferenceNames = new Set();
            this._options = options || {};
        }
        validateLiteral(source, start = 0, end = source.length) {
            this._uFlag = this._nFlag = false;
            this.reset(source, start, end);
            this.onLiteralEnter(start);
            if (this.eat(Solidus) && this.eatRegExpBody() && this.eat(Solidus)) {
                const flagStart = this.index;
                const uFlag = source.includes("u", flagStart);
                this.validateFlags(source, flagStart, end);
                this.validatePattern(source, start + 1, flagStart - 1, uFlag);
            }
            else if (start >= end) {
                this.raise("Empty");
            }
            else {
                const c = String.fromCodePoint(this.currentCodePoint);
                this.raise(`Unexpected character '${c}'`);
            }
            this.onLiteralLeave(start, end);
        }
        validateFlags(source, start = 0, end = source.length) {
            const existingFlags = new Set();
            let global = false;
            let ignoreCase = false;
            let multiline = false;
            let sticky = false;
            let unicode = false;
            let dotAll = false;
            for (let i = start; i < end; ++i) {
                const flag = source.charCodeAt(i);
                if (existingFlags.has(flag)) {
                    this.raise(`Duplicated flag '${source[i]}'`);
                }
                existingFlags.add(flag);
                if (flag === LatinSmallLetterG) {
                    global = true;
                }
                else if (flag === LatinSmallLetterI) {
                    ignoreCase = true;
                }
                else if (flag === LatinSmallLetterM) {
                    multiline = true;
                }
                else if (flag === LatinSmallLetterU && this.ecmaVersion >= 2015) {
                    unicode = true;
                }
                else if (flag === LatinSmallLetterY && this.ecmaVersion >= 2015) {
                    sticky = true;
                }
                else if (flag === LatinSmallLetterS && this.ecmaVersion >= 2018) {
                    dotAll = true;
                }
                else {
                    this.raise(`Invalid flag '${source[i]}'`);
                }
            }
            this.onFlags(start, end, global, ignoreCase, multiline, unicode, sticky, dotAll);
        }
        validatePattern(source, start = 0, end = source.length, uFlag = false) {
            this._uFlag = uFlag && this.ecmaVersion >= 2015;
            this._nFlag = uFlag && this.ecmaVersion >= 2018;
            this.reset(source, start, end);
            this.pattern();
            if (!this._nFlag &&
                this.ecmaVersion >= 2018 &&
                this._groupNames.size > 0) {
                this._nFlag = true;
                this.rewind(start);
                this.pattern();
            }
        }
        get strict() {
            return Boolean(this._options.strict || this._uFlag);
        }
        get ecmaVersion() {
            return this._options.ecmaVersion || 2020;
        }
        onLiteralEnter(start) {
            if (this._options.onLiteralEnter) {
                this._options.onLiteralEnter(start);
            }
        }
        onLiteralLeave(start, end) {
            if (this._options.onLiteralLeave) {
                this._options.onLiteralLeave(start, end);
            }
        }
        onFlags(start, end, global, ignoreCase, multiline, unicode, sticky, dotAll) {
            if (this._options.onFlags) {
                this._options.onFlags(start, end, global, ignoreCase, multiline, unicode, sticky, dotAll);
            }
        }
        onPatternEnter(start) {
            if (this._options.onPatternEnter) {
                this._options.onPatternEnter(start);
            }
        }
        onPatternLeave(start, end) {
            if (this._options.onPatternLeave) {
                this._options.onPatternLeave(start, end);
            }
        }
        onDisjunctionEnter(start) {
            if (this._options.onDisjunctionEnter) {
                this._options.onDisjunctionEnter(start);
            }
        }
        onDisjunctionLeave(start, end) {
            if (this._options.onDisjunctionLeave) {
                this._options.onDisjunctionLeave(start, end);
            }
        }
        onAlternativeEnter(start, index) {
            if (this._options.onAlternativeEnter) {
                this._options.onAlternativeEnter(start, index);
            }
        }
        onAlternativeLeave(start, end, index) {
            if (this._options.onAlternativeLeave) {
                this._options.onAlternativeLeave(start, end, index);
            }
        }
        onGroupEnter(start) {
            if (this._options.onGroupEnter) {
                this._options.onGroupEnter(start);
            }
        }
        onGroupLeave(start, end) {
            if (this._options.onGroupLeave) {
                this._options.onGroupLeave(start, end);
            }
        }
        onCapturingGroupEnter(start, name) {
            if (this._options.onCapturingGroupEnter) {
                this._options.onCapturingGroupEnter(start, name);
            }
        }
        onCapturingGroupLeave(start, end, name) {
            if (this._options.onCapturingGroupLeave) {
                this._options.onCapturingGroupLeave(start, end, name);
            }
        }
        onQuantifier(start, end, min, max, greedy) {
            if (this._options.onQuantifier) {
                this._options.onQuantifier(start, end, min, max, greedy);
            }
        }
        onLookaroundAssertionEnter(start, kind, negate) {
            if (this._options.onLookaroundAssertionEnter) {
                this._options.onLookaroundAssertionEnter(start, kind, negate);
            }
        }
        onLookaroundAssertionLeave(start, end, kind, negate) {
            if (this._options.onLookaroundAssertionLeave) {
                this._options.onLookaroundAssertionLeave(start, end, kind, negate);
            }
        }
        onEdgeAssertion(start, end, kind) {
            if (this._options.onEdgeAssertion) {
                this._options.onEdgeAssertion(start, end, kind);
            }
        }
        onWordBoundaryAssertion(start, end, kind, negate) {
            if (this._options.onWordBoundaryAssertion) {
                this._options.onWordBoundaryAssertion(start, end, kind, negate);
            }
        }
        onAnyCharacterSet(start, end, kind) {
            if (this._options.onAnyCharacterSet) {
                this._options.onAnyCharacterSet(start, end, kind);
            }
        }
        onEscapeCharacterSet(start, end, kind, negate) {
            if (this._options.onEscapeCharacterSet) {
                this._options.onEscapeCharacterSet(start, end, kind, negate);
            }
        }
        onUnicodePropertyCharacterSet(start, end, kind, key, value, negate) {
            if (this._options.onUnicodePropertyCharacterSet) {
                this._options.onUnicodePropertyCharacterSet(start, end, kind, key, value, negate);
            }
        }
        onCharacter(start, end, value) {
            if (this._options.onCharacter) {
                this._options.onCharacter(start, end, value);
            }
        }
        onBackreference(start, end, ref) {
            if (this._options.onBackreference) {
                this._options.onBackreference(start, end, ref);
            }
        }
        onCharacterClassEnter(start, negate) {
            if (this._options.onCharacterClassEnter) {
                this._options.onCharacterClassEnter(start, negate);
            }
        }
        onCharacterClassLeave(start, end, negate) {
            if (this._options.onCharacterClassLeave) {
                this._options.onCharacterClassLeave(start, end, negate);
            }
        }
        onCharacterClassRange(start, end, min, max) {
            if (this._options.onCharacterClassRange) {
                this._options.onCharacterClassRange(start, end, min, max);
            }
        }
        get source() {
            return this._reader.source;
        }
        get index() {
            return this._reader.index;
        }
        get currentCodePoint() {
            return this._reader.currentCodePoint;
        }
        get nextCodePoint() {
            return this._reader.nextCodePoint;
        }
        get nextCodePoint2() {
            return this._reader.nextCodePoint2;
        }
        get nextCodePoint3() {
            return this._reader.nextCodePoint3;
        }
        reset(source, start, end) {
            this._reader.reset(source, start, end, this._uFlag);
        }
        rewind(index) {
            this._reader.rewind(index);
        }
        advance() {
            this._reader.advance();
        }
        eat(cp) {
            return this._reader.eat(cp);
        }
        eat2(cp1, cp2) {
            return this._reader.eat2(cp1, cp2);
        }
        eat3(cp1, cp2, cp3) {
            return this._reader.eat3(cp1, cp2, cp3);
        }
        raise(message) {
            throw new RegExpSyntaxError(this.source, this._uFlag, this.index, message);
        }
        eatRegExpBody() {
            const start = this.index;
            let inClass = false;
            let escaped = false;
            for (;;) {
                const cp = this.currentCodePoint;
                if (cp === -1 || isLineTerminator(cp)) {
                    const kind = inClass ? "character class" : "regular expression";
                    this.raise(`Unterminated ${kind}`);
                }
                if (escaped) {
                    escaped = false;
                }
                else if (cp === ReverseSolidus) {
                    escaped = true;
                }
                else if (cp === LeftSquareBracket) {
                    inClass = true;
                }
                else if (cp === RightSquareBracket) {
                    inClass = false;
                }
                else if ((cp === Solidus && !inClass) ||
                    (cp === Asterisk && this.index === start)) {
                    break;
                }
                this.advance();
            }
            return this.index !== start;
        }
        pattern() {
            const start = this.index;
            this._numCapturingParens = this.countCapturingParens();
            this._groupNames.clear();
            this._backreferenceNames.clear();
            this.onPatternEnter(start);
            this.disjunction();
            const cp = this.currentCodePoint;
            if (this.currentCodePoint !== -1) {
                if (cp === RightParenthesis) {
                    this.raise("Unmatched ')'");
                }
                if (cp === ReverseSolidus) {
                    this.raise("\\ at end of pattern");
                }
                if (cp === RightSquareBracket || cp === RightCurlyBracket) {
                    this.raise("Lone quantifier brackets");
                }
                const c = String.fromCodePoint(cp);
                this.raise(`Unexpected character '${c}'`);
            }
            for (const name of this._backreferenceNames) {
                if (!this._groupNames.has(name)) {
                    this.raise("Invalid named capture referenced");
                }
            }
            this.onPatternLeave(start, this.index);
        }
        countCapturingParens() {
            const start = this.index;
            let inClass = false;
            let escaped = false;
            let count = 0;
            let cp = 0;
            while ((cp = this.currentCodePoint) !== -1) {
                if (escaped) {
                    escaped = false;
                }
                else if (cp === ReverseSolidus) {
                    escaped = true;
                }
                else if (cp === LeftSquareBracket) {
                    inClass = true;
                }
                else if (cp === RightSquareBracket) {
                    inClass = false;
                }
                else if (cp === LeftParenthesis &&
                    !inClass &&
                    (this.nextCodePoint !== QuestionMark ||
                        (this.nextCodePoint2 === LessThanSign &&
                            this.nextCodePoint3 !== EqualsSign &&
                            this.nextCodePoint3 !== ExclamationMark))) {
                    count += 1;
                }
                this.advance();
            }
            this.rewind(start);
            return count;
        }
        disjunction() {
            const start = this.index;
            let i = 0;
            this.onDisjunctionEnter(start);
            this.alternative(i++);
            while (this.eat(VerticalLine)) {
                this.alternative(i++);
            }
            if (this.eatQuantifier(true)) {
                this.raise("Nothing to repeat");
            }
            if (this.eat(LeftCurlyBracket)) {
                this.raise("Lone quantifier brackets");
            }
            this.onDisjunctionLeave(start, this.index);
        }
        alternative(i) {
            const start = this.index;
            this.onAlternativeEnter(start, i);
            while (this.currentCodePoint !== -1 && this.eatTerm()) {
            }
            this.onAlternativeLeave(start, this.index, i);
        }
        eatTerm() {
            if (this.eatAssertion()) {
                if (this._lastAssertionIsQuantifiable) {
                    this.eatQuantifier();
                }
                return true;
            }
            if (this.strict ? this.eatAtom() : this.eatExtendedAtom()) {
                this.eatQuantifier();
                return true;
            }
            return false;
        }
        eatAssertion() {
            const start = this.index;
            this._lastAssertionIsQuantifiable = false;
            if (this.eat(CircumflexAccent)) {
                this.onEdgeAssertion(start, this.index, "start");
                return true;
            }
            if (this.eat(DollarSign)) {
                this.onEdgeAssertion(start, this.index, "end");
                return true;
            }
            if (this.eat2(ReverseSolidus, LatinCapitalLetterB)) {
                this.onWordBoundaryAssertion(start, this.index, "word", true);
                return true;
            }
            if (this.eat2(ReverseSolidus, LatinSmallLetterB)) {
                this.onWordBoundaryAssertion(start, this.index, "word", false);
                return true;
            }
            if (this.eat2(LeftParenthesis, QuestionMark)) {
                const lookbehind = this.ecmaVersion >= 2018 && this.eat(LessThanSign);
                let negate = false;
                if (this.eat(EqualsSign) || (negate = this.eat(ExclamationMark))) {
                    const kind = lookbehind ? "lookbehind" : "lookahead";
                    this.onLookaroundAssertionEnter(start, kind, negate);
                    this.disjunction();
                    if (!this.eat(RightParenthesis)) {
                        this.raise("Unterminated group");
                    }
                    this._lastAssertionIsQuantifiable = !lookbehind && !this.strict;
                    this.onLookaroundAssertionLeave(start, this.index, kind, negate);
                    return true;
                }
                this.rewind(start);
            }
            return false;
        }
        eatQuantifier(noError = false) {
            const start = this.index;
            let min = 0;
            let max = 0;
            let greedy = false;
            if (this.eat(Asterisk)) {
                min = 0;
                max = Number.POSITIVE_INFINITY;
            }
            else if (this.eat(PlusSign)) {
                min = 1;
                max = Number.POSITIVE_INFINITY;
            }
            else if (this.eat(QuestionMark)) {
                min = 0;
                max = 1;
            }
            else if (this.eatBracedQuantifier(noError)) {
                min = this._lastMinValue;
                max = this._lastMaxValue;
            }
            else {
                return false;
            }
            greedy = !this.eat(QuestionMark);
            if (!noError) {
                this.onQuantifier(start, this.index, min, max, greedy);
            }
            return true;
        }
        eatBracedQuantifier(noError) {
            const start = this.index;
            if (this.eat(LeftCurlyBracket)) {
                this._lastMinValue = 0;
                this._lastMaxValue = Number.POSITIVE_INFINITY;
                if (this.eatDecimalDigits()) {
                    this._lastMinValue = this._lastMaxValue = this._lastIntValue;
                    if (this.eat(Comma)) {
                        this._lastMaxValue = this.eatDecimalDigits()
                            ? this._lastIntValue
                            : Number.POSITIVE_INFINITY;
                    }
                    if (this.eat(RightCurlyBracket)) {
                        if (!noError && this._lastMaxValue < this._lastMinValue) {
                            this.raise("numbers out of order in {} quantifier");
                        }
                        return true;
                    }
                }
                if (!noError && this.strict) {
                    this.raise("Incomplete quantifier");
                }
                this.rewind(start);
            }
            return false;
        }
        eatAtom() {
            return (this.eatPatternCharacter() ||
                this.eatDot() ||
                this.eatReverseSolidusAtomEscape() ||
                this.eatCharacterClass() ||
                this.eatUncapturingGroup() ||
                this.eatCapturingGroup());
        }
        eatDot() {
            if (this.eat(FullStop)) {
                this.onAnyCharacterSet(this.index - 1, this.index, "any");
                return true;
            }
            return false;
        }
        eatReverseSolidusAtomEscape() {
            const start = this.index;
            if (this.eat(ReverseSolidus)) {
                if (this.eatAtomEscape()) {
                    return true;
                }
                this.rewind(start);
            }
            return false;
        }
        eatUncapturingGroup() {
            const start = this.index;
            if (this.eat3(LeftParenthesis, QuestionMark, Colon)) {
                this.onGroupEnter(start);
                this.disjunction();
                if (!this.eat(RightParenthesis)) {
                    this.raise("Unterminated group");
                }
                this.onGroupLeave(start, this.index);
                return true;
            }
            return false;
        }
        eatCapturingGroup() {
            const start = this.index;
            if (this.eat(LeftParenthesis)) {
                this._lastStrValue = "";
                if (this.ecmaVersion >= 2018) {
                    this.groupSpecifier();
                }
                else if (this.currentCodePoint === QuestionMark) {
                    this.raise("Invalid group");
                }
                const name = this._lastStrValue || null;
                this.onCapturingGroupEnter(start, name);
                this.disjunction();
                if (!this.eat(RightParenthesis)) {
                    this.raise("Unterminated group");
                }
                this.onCapturingGroupLeave(start, this.index, name);
                return true;
            }
            return false;
        }
        eatExtendedAtom() {
            return (this.eatDot() ||
                this.eatReverseSolidusAtomEscape() ||
                this.eatReverseSolidusFollowedByC() ||
                this.eatCharacterClass() ||
                this.eatUncapturingGroup() ||
                this.eatCapturingGroup() ||
                this.eatInvalidBracedQuantifier() ||
                this.eatExtendedPatternCharacter());
        }
        eatReverseSolidusFollowedByC() {
            if (this.currentCodePoint === ReverseSolidus &&
                this.nextCodePoint === LatinSmallLetterC) {
                this._lastIntValue = this.currentCodePoint;
                this.advance();
                this.onCharacter(this.index - 1, this.index, ReverseSolidus);
                return true;
            }
            return false;
        }
        eatInvalidBracedQuantifier() {
            if (this.eatBracedQuantifier(true)) {
                this.raise("Nothing to repeat");
            }
            return false;
        }
        eatSyntaxCharacter() {
            if (isSyntaxCharacter(this.currentCodePoint)) {
                this._lastIntValue = this.currentCodePoint;
                this.advance();
                return true;
            }
            return false;
        }
        eatPatternCharacter() {
            const start = this.index;
            const cp = this.currentCodePoint;
            if (cp !== -1 && !isSyntaxCharacter(cp)) {
                this.advance();
                this.onCharacter(start, this.index, cp);
                return true;
            }
            return false;
        }
        eatExtendedPatternCharacter() {
            const start = this.index;
            const cp = this.currentCodePoint;
            if (cp !== -1 &&
                cp !== CircumflexAccent &&
                cp !== DollarSign &&
                cp !== ReverseSolidus &&
                cp !== FullStop &&
                cp !== Asterisk &&
                cp !== PlusSign &&
                cp !== QuestionMark &&
                cp !== LeftParenthesis &&
                cp !== RightParenthesis &&
                cp !== LeftSquareBracket &&
                cp !== VerticalLine) {
                this.advance();
                this.onCharacter(start, this.index, cp);
                return true;
            }
            return false;
        }
        groupSpecifier() {
            this._lastStrValue = "";
            if (this.eat(QuestionMark)) {
                if (this.eatGroupName()) {
                    if (!this._groupNames.has(this._lastStrValue)) {
                        this._groupNames.add(this._lastStrValue);
                        return;
                    }
                    this.raise("Duplicate capture group name");
                }
                this.raise("Invalid group");
            }
        }
        eatGroupName() {
            this._lastStrValue = "";
            if (this.eat(LessThanSign)) {
                if (this.eatRegExpIdentifierName() && this.eat(GreaterThanSign)) {
                    return true;
                }
                this.raise("Invalid capture group name");
            }
            return false;
        }
        eatRegExpIdentifierName() {
            this._lastStrValue = "";
            if (this.eatRegExpIdentifierStart()) {
                this._lastStrValue += String.fromCodePoint(this._lastIntValue);
                while (this.eatRegExpIdentifierPart()) {
                    this._lastStrValue += String.fromCodePoint(this._lastIntValue);
                }
                return true;
            }
            return false;
        }
        eatRegExpIdentifierStart() {
            const start = this.index;
            let cp = this.currentCodePoint;
            this.advance();
            if (cp === ReverseSolidus && this.eatRegExpUnicodeEscapeSequence()) {
                cp = this._lastIntValue;
            }
            if (isRegExpIdentifierStart(cp)) {
                this._lastIntValue = cp;
                return true;
            }
            if (this.index !== start) {
                this.rewind(start);
            }
            return false;
        }
        eatRegExpIdentifierPart() {
            const start = this.index;
            let cp = this.currentCodePoint;
            this.advance();
            if (cp === ReverseSolidus && this.eatRegExpUnicodeEscapeSequence()) {
                cp = this._lastIntValue;
            }
            if (isRegExpIdentifierPart(cp)) {
                this._lastIntValue = cp;
                return true;
            }
            if (this.index !== start) {
                this.rewind(start);
            }
            return false;
        }
        eatAtomEscape() {
            if (this.eatBackreference() ||
                this.eatCharacterClassEscape() ||
                this.eatCharacterEscape() ||
                (this._nFlag && this.eatKGroupName())) {
                return true;
            }
            if (this.strict || this._uFlag) {
                this.raise("Invalid escape");
            }
            return false;
        }
        eatBackreference() {
            const start = this.index;
            if (this.eatDecimalEscape()) {
                const n = this._lastIntValue;
                if (n <= this._numCapturingParens) {
                    this.onBackreference(start - 1, this.index, n);
                    return true;
                }
                if (this.strict) {
                    this.raise("Invalid escape");
                }
                this.rewind(start);
            }
            return false;
        }
        eatKGroupName() {
            const start = this.index;
            if (this.eat(LatinSmallLetterK)) {
                if (this.eatGroupName()) {
                    const groupName = this._lastStrValue;
                    this._backreferenceNames.add(groupName);
                    this.onBackreference(start - 1, this.index, groupName);
                    return true;
                }
                this.raise("Invalid named reference");
            }
            return false;
        }
        eatCharacterEscape() {
            const start = this.index;
            if (this.eatControlEscape() ||
                this.eatCControlLetter() ||
                this.eatZero() ||
                this.eatHexEscapeSequence() ||
                this.eatRegExpUnicodeEscapeSequence() ||
                (!this.strict && this.eatLegacyOctalEscapeSequence()) ||
                this.eatIdentityEscape()) {
                this.onCharacter(start - 1, this.index, this._lastIntValue);
                return true;
            }
            return false;
        }
        eatCControlLetter() {
            const start = this.index;
            if (this.eat(LatinSmallLetterC)) {
                if (this.eatControlLetter()) {
                    return true;
                }
                this.rewind(start);
            }
            return false;
        }
        eatZero() {
            if (this.currentCodePoint === DigitZero &&
                !isDecimalDigit(this.nextCodePoint)) {
                this._lastIntValue = 0;
                this.advance();
                return true;
            }
            return false;
        }
        eatControlEscape() {
            if (this.eat(LatinSmallLetterT)) {
                this._lastIntValue = CharacterTabulation;
                return true;
            }
            if (this.eat(LatinSmallLetterN)) {
                this._lastIntValue = LineFeed;
                return true;
            }
            if (this.eat(LatinSmallLetterV)) {
                this._lastIntValue = LineTabulation;
                return true;
            }
            if (this.eat(LatinSmallLetterF)) {
                this._lastIntValue = FormFeed;
                return true;
            }
            if (this.eat(LatinSmallLetterR)) {
                this._lastIntValue = CarriageReturn;
                return true;
            }
            return false;
        }
        eatControlLetter() {
            const cp = this.currentCodePoint;
            if (isLatinLetter(cp)) {
                this.advance();
                this._lastIntValue = cp % 0x20;
                return true;
            }
            return false;
        }
        eatRegExpUnicodeEscapeSequence() {
            const start = this.index;
            if (this.eat(LatinSmallLetterU)) {
                if (this.eatFixedHexDigits(4)) {
                    const lead = this._lastIntValue;
                    if (this._uFlag && lead >= 0xd800 && lead <= 0xdbff) {
                        const leadSurrogateEnd = this.index;
                        if (this.eat(ReverseSolidus) &&
                            this.eat(LatinSmallLetterU) &&
                            this.eatFixedHexDigits(4)) {
                            const trail = this._lastIntValue;
                            if (trail >= 0xdc00 && trail <= 0xdfff) {
                                this._lastIntValue =
                                    (lead - 0xd800) * 0x400 +
                                        (trail - 0xdc00) +
                                        0x10000;
                                return true;
                            }
                        }
                        this.rewind(leadSurrogateEnd);
                        this._lastIntValue = lead;
                    }
                    return true;
                }
                if (this._uFlag &&
                    this.eat(LeftCurlyBracket) &&
                    this.eatHexDigits() &&
                    this.eat(RightCurlyBracket) &&
                    isValidUnicode(this._lastIntValue)) {
                    return true;
                }
                if (this.strict || this._uFlag) {
                    this.raise("Invalid unicode escape");
                }
                this.rewind(start);
            }
            return false;
        }
        eatIdentityEscape() {
            if (this._uFlag) {
                if (this.eatSyntaxCharacter()) {
                    return true;
                }
                if (this.eat(Solidus)) {
                    this._lastIntValue = Solidus;
                    return true;
                }
                return false;
            }
            if (this.isValidIdentityEscape(this.currentCodePoint)) {
                this._lastIntValue = this.currentCodePoint;
                this.advance();
                return true;
            }
            return false;
        }
        isValidIdentityEscape(cp) {
            if (cp === -1) {
                return false;
            }
            if (this.strict) {
                return !isIdContinue(cp);
            }
            return (cp !== LatinSmallLetterC &&
                (!this._nFlag || cp !== LatinSmallLetterK));
        }
        eatDecimalEscape() {
            this._lastIntValue = 0;
            let cp = this.currentCodePoint;
            if (cp >= DigitOne && cp <= DigitNine) {
                do {
                    this._lastIntValue = 10 * this._lastIntValue + (cp - DigitZero);
                    this.advance();
                } while ((cp = this.currentCodePoint) >= DigitZero &&
                    cp <= DigitNine);
                return true;
            }
            return false;
        }
        eatCharacterClassEscape() {
            const start = this.index;
            if (this.eat(LatinSmallLetterD)) {
                this._lastIntValue = -1;
                this.onEscapeCharacterSet(start - 1, this.index, "digit", false);
                return true;
            }
            if (this.eat(LatinCapitalLetterD)) {
                this._lastIntValue = -1;
                this.onEscapeCharacterSet(start - 1, this.index, "digit", true);
                return true;
            }
            if (this.eat(LatinSmallLetterS)) {
                this._lastIntValue = -1;
                this.onEscapeCharacterSet(start - 1, this.index, "space", false);
                return true;
            }
            if (this.eat(LatinCapitalLetterS)) {
                this._lastIntValue = -1;
                this.onEscapeCharacterSet(start - 1, this.index, "space", true);
                return true;
            }
            if (this.eat(LatinSmallLetterW)) {
                this._lastIntValue = -1;
                this.onEscapeCharacterSet(start - 1, this.index, "word", false);
                return true;
            }
            if (this.eat(LatinCapitalLetterW)) {
                this._lastIntValue = -1;
                this.onEscapeCharacterSet(start - 1, this.index, "word", true);
                return true;
            }
            let negate = false;
            if (this._uFlag &&
                this.ecmaVersion >= 2018 &&
                (this.eat(LatinSmallLetterP) ||
                    (negate = this.eat(LatinCapitalLetterP)))) {
                this._lastIntValue = -1;
                if (this.eat(LeftCurlyBracket) &&
                    this.eatUnicodePropertyValueExpression() &&
                    this.eat(RightCurlyBracket)) {
                    this.onUnicodePropertyCharacterSet(start - 1, this.index, "property", this._lastKeyValue, this._lastValValue || null, negate);
                    return true;
                }
                this.raise("Invalid property name");
            }
            return false;
        }
        eatUnicodePropertyValueExpression() {
            const start = this.index;
            if (this.eatUnicodePropertyName() && this.eat(EqualsSign)) {
                this._lastKeyValue = this._lastStrValue;
                if (this.eatUnicodePropertyValue()) {
                    this._lastValValue = this._lastStrValue;
                    if (isValidUnicodeProperty(this.ecmaVersion, this._lastKeyValue, this._lastValValue)) {
                        return true;
                    }
                    this.raise("Invalid property name");
                }
            }
            this.rewind(start);
            if (this.eatLoneUnicodePropertyNameOrValue()) {
                const nameOrValue = this._lastStrValue;
                if (isValidUnicodeProperty(this.ecmaVersion, "General_Category", nameOrValue)) {
                    this._lastKeyValue = "General_Category";
                    this._lastValValue = nameOrValue;
                    return true;
                }
                if (isValidLoneUnicodeProperty(this.ecmaVersion, nameOrValue)) {
                    this._lastKeyValue = nameOrValue;
                    this._lastValValue = "";
                    return true;
                }
                this.raise("Invalid property name");
            }
            return false;
        }
        eatUnicodePropertyName() {
            this._lastStrValue = "";
            while (isUnicodePropertyNameCharacter(this.currentCodePoint)) {
                this._lastStrValue += String.fromCodePoint(this.currentCodePoint);
                this.advance();
            }
            return this._lastStrValue !== "";
        }
        eatUnicodePropertyValue() {
            this._lastStrValue = "";
            while (isUnicodePropertyValueCharacter(this.currentCodePoint)) {
                this._lastStrValue += String.fromCodePoint(this.currentCodePoint);
                this.advance();
            }
            return this._lastStrValue !== "";
        }
        eatLoneUnicodePropertyNameOrValue() {
            return this.eatUnicodePropertyValue();
        }
        eatCharacterClass() {
            const start = this.index;
            if (this.eat(LeftSquareBracket)) {
                const negate = this.eat(CircumflexAccent);
                this.onCharacterClassEnter(start, negate);
                this.classRanges();
                if (!this.eat(RightSquareBracket)) {
                    this.raise("Unterminated character class");
                }
                this.onCharacterClassLeave(start, this.index, negate);
                return true;
            }
            return false;
        }
        classRanges() {
            let start = this.index;
            while (this.eatClassAtom()) {
                const left = this._lastIntValue;
                const hyphenStart = this.index;
                if (this.eat(HyphenMinus)) {
                    this.onCharacter(hyphenStart, this.index, HyphenMinus);
                    if (this.eatClassAtom()) {
                        const right = this._lastIntValue;
                        if (left === -1 || right === -1) {
                            if (this.strict) {
                                this.raise("Invalid character class");
                            }
                        }
                        else if (left > right) {
                            this.raise("Range out of order in character class");
                        }
                        else {
                            this.onCharacterClassRange(start, this.index, left, right);
                        }
                    }
                }
                start = this.index;
            }
        }
        eatClassAtom() {
            const start = this.index;
            if (this.eat(ReverseSolidus)) {
                if (this.eatClassEscape()) {
                    return true;
                }
                if (this._uFlag) {
                    this.raise("Invalid escape");
                }
                this.rewind(start);
            }
            const cp = this.currentCodePoint;
            if (cp !== -1 && cp !== RightSquareBracket) {
                this.advance();
                this._lastIntValue = cp;
                this.onCharacter(start, this.index, cp);
                return true;
            }
            return false;
        }
        eatClassEscape() {
            const start = this.index;
            if (this.eat(LatinSmallLetterB)) {
                this._lastIntValue = Backspace;
                this.onCharacter(start - 1, this.index, Backspace);
                return true;
            }
            if (this._uFlag && this.eat(HyphenMinus)) {
                this._lastIntValue = HyphenMinus;
                this.onCharacter(start - 1, this.index, HyphenMinus);
                return true;
            }
            if (!this._uFlag && this.eat(LatinSmallLetterC)) {
                if (this.eatClassControlLetter()) {
                    this.onCharacter(start - 1, this.index, this._lastIntValue);
                    return true;
                }
                this.rewind(start);
            }
            return this.eatCharacterClassEscape() || this.eatCharacterEscape();
        }
        eatClassControlLetter() {
            const cp = this.currentCodePoint;
            if (isDecimalDigit(cp) || cp === LowLine) {
                this.advance();
                this._lastIntValue = cp % 0x20;
                return true;
            }
            return false;
        }
        eatHexEscapeSequence() {
            const start = this.index;
            if (this.eat(LatinSmallLetterX)) {
                if (this.eatFixedHexDigits(2)) {
                    return true;
                }
                if (this._uFlag) {
                    this.raise("Invalid escape");
                }
                this.rewind(start);
            }
            return false;
        }
        eatDecimalDigits() {
            const start = this.index;
            this._lastIntValue = 0;
            while (isDecimalDigit(this.currentCodePoint)) {
                this._lastIntValue =
                    10 * this._lastIntValue + digitToInt(this.currentCodePoint);
                this.advance();
            }
            return this.index !== start;
        }
        eatHexDigits() {
            const start = this.index;
            this._lastIntValue = 0;
            while (isHexDigit(this.currentCodePoint)) {
                this._lastIntValue =
                    16 * this._lastIntValue + digitToInt(this.currentCodePoint);
                this.advance();
            }
            return this.index !== start;
        }
        eatLegacyOctalEscapeSequence() {
            if (this.eatOctalDigit()) {
                const n1 = this._lastIntValue;
                if (this.eatOctalDigit()) {
                    const n2 = this._lastIntValue;
                    if (n1 <= 3 && this.eatOctalDigit()) {
                        this._lastIntValue = n1 * 64 + n2 * 8 + this._lastIntValue;
                    }
                    else {
                        this._lastIntValue = n1 * 8 + n2;
                    }
                }
                else {
                    this._lastIntValue = n1;
                }
                return true;
            }
            return false;
        }
        eatOctalDigit() {
            const cp = this.currentCodePoint;
            if (isOctalDigit(cp)) {
                this.advance();
                this._lastIntValue = cp - DigitZero;
                return true;
            }
            this._lastIntValue = 0;
            return false;
        }
        eatFixedHexDigits(length) {
            const start = this.index;
            this._lastIntValue = 0;
            for (let i = 0; i < length; ++i) {
                const cp = this.currentCodePoint;
                if (!isHexDigit(cp)) {
                    this.rewind(start);
                    return false;
                }
                this._lastIntValue = 16 * this._lastIntValue + digitToInt(cp);
                this.advance();
            }
            return true;
        }
    }
    const DummyPattern = {};
    const DummyFlags = {};
    const DummyCapturingGroup = {};
    class RegExpParserState {
        constructor(options) {
            this._node = DummyPattern;
            this._flags = DummyFlags;
            this._backreferences = [];
            this._capturingGroups = [];
            this.source = "";
            this.strict = Boolean(options && options.strict);
            this.ecmaVersion = (options && options.ecmaVersion) || 2020;
        }
        get pattern() {
            if (this._node.type !== "Pattern") {
                throw new Error("UnknownError");
            }
            return this._node;
        }
        get flags() {
            if (this._flags.type !== "Flags") {
                throw new Error("UnknownError");
            }
            return this._flags;
        }
        onFlags(start, end, global, ignoreCase, multiline, unicode, sticky, dotAll) {
            this._flags = {
                type: "Flags",
                parent: null,
                start,
                end,
                raw: this.source.slice(start, end),
                global,
                ignoreCase,
                multiline,
                unicode,
                sticky,
                dotAll,
            };
        }
        onPatternEnter(start) {
            this._node = {
                type: "Pattern",
                parent: null,
                start,
                end: start,
                raw: "",
                alternatives: [],
            };
            this._backreferences.length = 0;
            this._capturingGroups.length = 0;
        }
        onPatternLeave(start, end) {
            this._node.end = end;
            this._node.raw = this.source.slice(start, end);
            for (const reference of this._backreferences) {
                const ref = reference.ref;
                const group = typeof ref === "number"
                    ? this._capturingGroups[ref - 1]
                    : this._capturingGroups.find(g => g.name === ref);
                reference.resolved = group;
                group.references.push(reference);
            }
        }
        onAlternativeEnter(start) {
            const parent = this._node;
            if (parent.type !== "Assertion" &&
                parent.type !== "CapturingGroup" &&
                parent.type !== "Group" &&
                parent.type !== "Pattern") {
                throw new Error("UnknownError");
            }
            this._node = {
                type: "Alternative",
                parent,
                start,
                end: start,
                raw: "",
                elements: [],
            };
            parent.alternatives.push(this._node);
        }
        onAlternativeLeave(start, end) {
            const node = this._node;
            if (node.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            node.end = end;
            node.raw = this.source.slice(start, end);
            this._node = node.parent;
        }
        onGroupEnter(start) {
            const parent = this._node;
            if (parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            this._node = {
                type: "Group",
                parent,
                start,
                end: start,
                raw: "",
                alternatives: [],
            };
            parent.elements.push(this._node);
        }
        onGroupLeave(start, end) {
            const node = this._node;
            if (node.type !== "Group" || node.parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            node.end = end;
            node.raw = this.source.slice(start, end);
            this._node = node.parent;
        }
        onCapturingGroupEnter(start, name) {
            const parent = this._node;
            if (parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            this._node = {
                type: "CapturingGroup",
                parent,
                start,
                end: start,
                raw: "",
                name,
                alternatives: [],
                references: [],
            };
            parent.elements.push(this._node);
            this._capturingGroups.push(this._node);
        }
        onCapturingGroupLeave(start, end) {
            const node = this._node;
            if (node.type !== "CapturingGroup" ||
                node.parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            node.end = end;
            node.raw = this.source.slice(start, end);
            this._node = node.parent;
        }
        onQuantifier(start, end, min, max, greedy) {
            const parent = this._node;
            if (parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            const element = parent.elements.pop();
            if (element == null ||
                element.type === "Quantifier" ||
                (element.type === "Assertion" && element.kind !== "lookahead")) {
                throw new Error("UnknownError");
            }
            const node = {
                type: "Quantifier",
                parent,
                start: element.start,
                end,
                raw: this.source.slice(element.start, end),
                min,
                max,
                greedy,
                element,
            };
            parent.elements.push(node);
            element.parent = node;
        }
        onLookaroundAssertionEnter(start, kind, negate) {
            const parent = this._node;
            if (parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            const node = (this._node = {
                type: "Assertion",
                parent,
                start,
                end: start,
                raw: "",
                kind,
                negate,
                alternatives: [],
            });
            parent.elements.push(node);
        }
        onLookaroundAssertionLeave(start, end) {
            const node = this._node;
            if (node.type !== "Assertion" || node.parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            node.end = end;
            node.raw = this.source.slice(start, end);
            this._node = node.parent;
        }
        onEdgeAssertion(start, end, kind) {
            const parent = this._node;
            if (parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            parent.elements.push({
                type: "Assertion",
                parent,
                start,
                end,
                raw: this.source.slice(start, end),
                kind,
            });
        }
        onWordBoundaryAssertion(start, end, kind, negate) {
            const parent = this._node;
            if (parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            parent.elements.push({
                type: "Assertion",
                parent,
                start,
                end,
                raw: this.source.slice(start, end),
                kind,
                negate,
            });
        }
        onAnyCharacterSet(start, end, kind) {
            const parent = this._node;
            if (parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            parent.elements.push({
                type: "CharacterSet",
                parent,
                start,
                end,
                raw: this.source.slice(start, end),
                kind,
            });
        }
        onEscapeCharacterSet(start, end, kind, negate) {
            const parent = this._node;
            if (parent.type !== "Alternative" && parent.type !== "CharacterClass") {
                throw new Error("UnknownError");
            }
            parent.elements.push({
                type: "CharacterSet",
                parent,
                start,
                end,
                raw: this.source.slice(start, end),
                kind,
                negate,
            });
        }
        onUnicodePropertyCharacterSet(start, end, kind, key, value, negate) {
            const parent = this._node;
            if (parent.type !== "Alternative" && parent.type !== "CharacterClass") {
                throw new Error("UnknownError");
            }
            parent.elements.push({
                type: "CharacterSet",
                parent,
                start,
                end,
                raw: this.source.slice(start, end),
                kind,
                key,
                value,
                negate,
            });
        }
        onCharacter(start, end, value) {
            const parent = this._node;
            if (parent.type !== "Alternative" && parent.type !== "CharacterClass") {
                throw new Error("UnknownError");
            }
            parent.elements.push({
                type: "Character",
                parent,
                start,
                end,
                raw: this.source.slice(start, end),
                value,
            });
        }
        onBackreference(start, end, ref) {
            const parent = this._node;
            if (parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            const node = {
                type: "Backreference",
                parent,
                start,
                end,
                raw: this.source.slice(start, end),
                ref,
                resolved: DummyCapturingGroup,
            };
            parent.elements.push(node);
            this._backreferences.push(node);
        }
        onCharacterClassEnter(start, negate) {
            const parent = this._node;
            if (parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            this._node = {
                type: "CharacterClass",
                parent,
                start,
                end: start,
                raw: "",
                negate,
                elements: [],
            };
            parent.elements.push(this._node);
        }
        onCharacterClassLeave(start, end) {
            const node = this._node;
            if (node.type !== "CharacterClass" ||
                node.parent.type !== "Alternative") {
                throw new Error("UnknownError");
            }
            node.end = end;
            node.raw = this.source.slice(start, end);
            this._node = node.parent;
        }
        onCharacterClassRange(start, end) {
            const parent = this._node;
            if (parent.type !== "CharacterClass") {
                throw new Error("UnknownError");
            }
            const elements = parent.elements;
            const max = elements.pop();
            const hyphen = elements.pop();
            const min = elements.pop();
            if (!min ||
                !max ||
                !hyphen ||
                min.type !== "Character" ||
                max.type !== "Character" ||
                hyphen.type !== "Character" ||
                hyphen.value !== HyphenMinus) {
                throw new Error("UnknownError");
            }
            const node = {
                type: "CharacterClassRange",
                parent,
                start,
                end,
                raw: this.source.slice(start, end),
                min,
                max,
            };
            min.parent = node;
            max.parent = node;
            elements.push(node);
        }
    }
    class RegExpParser {
        constructor(options) {
            this._state = new RegExpParserState(options);
            this._validator = new RegExpValidator(this._state);
        }
        parseLiteral(source, start = 0, end = source.length) {
            this._state.source = source;
            this._validator.validateLiteral(source, start, end);
            const pattern = this._state.pattern;
            const flags = this._state.flags;
            const literal = {
                type: "RegExpLiteral",
                parent: null,
                start,
                end,
                raw: source,
                pattern,
                flags,
            };
            pattern.parent = literal;
            flags.parent = literal;
            return literal;
        }
        parseFlags(source, start = 0, end = source.length) {
            this._state.source = source;
            this._validator.validateFlags(source, start, end);
            return this._state.flags;
        }
        parsePattern(source, start = 0, end = source.length, uFlag = false) {
            this._state.source = source;
            this._validator.validatePattern(source, start, end, uFlag);
            return this._state.pattern;
        }
    }
    class RegExpVisitor {
        constructor(handlers) {
            this._handlers = handlers;
        }
        visit(node) {
            switch (node.type) {
                case "Alternative":
                    this.visitAlternative(node);
                    break;
                case "Assertion":
                    this.visitAssertion(node);
                    break;
                case "Backreference":
                    this.visitBackreference(node);
                    break;
                case "CapturingGroup":
                    this.visitCapturingGroup(node);
                    break;
                case "Character":
                    this.visitCharacter(node);
                    break;
                case "CharacterClass":
                    this.visitCharacterClass(node);
                    break;
                case "CharacterClassRange":
                    this.visitCharacterClassRange(node);
                    break;
                case "CharacterSet":
                    this.visitCharacterSet(node);
                    break;
                case "Flags":
                    this.visitFlags(node);
                    break;
                case "Group":
                    this.visitGroup(node);
                    break;
                case "Pattern":
                    this.visitPattern(node);
                    break;
                case "Quantifier":
                    this.visitQuantifier(node);
                    break;
                case "RegExpLiteral":
                    this.visitRegExpLiteral(node);
                    break;
                default:
                    throw new Error(`Unknown type: ${node.type}`);
            }
        }
        visitAlternative(node) {
            if (this._handlers.onAlternativeEnter) {
                this._handlers.onAlternativeEnter(node);
            }
            node.elements.forEach(this.visit, this);
            if (this._handlers.onAlternativeLeave) {
                this._handlers.onAlternativeLeave(node);
            }
        }
        visitAssertion(node) {
            if (this._handlers.onAssertionEnter) {
                this._handlers.onAssertionEnter(node);
            }
            if (node.kind === "lookahead" || node.kind === "lookbehind") {
                node.alternatives.forEach(this.visit, this);
            }
            if (this._handlers.onAssertionLeave) {
                this._handlers.onAssertionLeave(node);
            }
        }
        visitBackreference(node) {
            if (this._handlers.onBackreferenceEnter) {
                this._handlers.onBackreferenceEnter(node);
            }
            if (this._handlers.onBackreferenceLeave) {
                this._handlers.onBackreferenceLeave(node);
            }
        }
        visitCapturingGroup(node) {
            if (this._handlers.onCapturingGroupEnter) {
                this._handlers.onCapturingGroupEnter(node);
            }
            node.alternatives.forEach(this.visit, this);
            if (this._handlers.onCapturingGroupLeave) {
                this._handlers.onCapturingGroupLeave(node);
            }
        }
        visitCharacter(node) {
            if (this._handlers.onCharacterEnter) {
                this._handlers.onCharacterEnter(node);
            }
            if (this._handlers.onCharacterLeave) {
                this._handlers.onCharacterLeave(node);
            }
        }
        visitCharacterClass(node) {
            if (this._handlers.onCharacterClassEnter) {
                this._handlers.onCharacterClassEnter(node);
            }
            node.elements.forEach(this.visit, this);
            if (this._handlers.onCharacterClassLeave) {
                this._handlers.onCharacterClassLeave(node);
            }
        }
        visitCharacterClassRange(node) {
            if (this._handlers.onCharacterClassRangeEnter) {
                this._handlers.onCharacterClassRangeEnter(node);
            }
            this.visitCharacter(node.min);
            this.visitCharacter(node.max);
            if (this._handlers.onCharacterClassRangeLeave) {
                this._handlers.onCharacterClassRangeLeave(node);
            }
        }
        visitCharacterSet(node) {
            if (this._handlers.onCharacterSetEnter) {
                this._handlers.onCharacterSetEnter(node);
            }
            if (this._handlers.onCharacterSetLeave) {
                this._handlers.onCharacterSetLeave(node);
            }
        }
        visitFlags(node) {
            if (this._handlers.onFlagsEnter) {
                this._handlers.onFlagsEnter(node);
            }
            if (this._handlers.onFlagsLeave) {
                this._handlers.onFlagsLeave(node);
            }
        }
        visitGroup(node) {
            if (this._handlers.onGroupEnter) {
                this._handlers.onGroupEnter(node);
            }
            node.alternatives.forEach(this.visit, this);
            if (this._handlers.onGroupLeave) {
                this._handlers.onGroupLeave(node);
            }
        }
        visitPattern(node) {
            if (this._handlers.onPatternEnter) {
                this._handlers.onPatternEnter(node);
            }
            node.alternatives.forEach(this.visit, this);
            if (this._handlers.onPatternLeave) {
                this._handlers.onPatternLeave(node);
            }
        }
        visitQuantifier(node) {
            if (this._handlers.onQuantifierEnter) {
                this._handlers.onQuantifierEnter(node);
            }
            this.visit(node.element);
            if (this._handlers.onQuantifierLeave) {
                this._handlers.onQuantifierLeave(node);
            }
        }
        visitRegExpLiteral(node) {
            if (this._handlers.onRegExpLiteralEnter) {
                this._handlers.onRegExpLiteralEnter(node);
            }
            this.visitPattern(node.pattern);
            this.visitFlags(node.flags);
            if (this._handlers.onRegExpLiteralLeave) {
                this._handlers.onRegExpLiteralLeave(node);
            }
        }
    }
    function parseRegExpLiteral(source, options) {
        return new RegExpParser(options).parseLiteral(String(source));
    }
    function validateRegExpLiteral(source, options) {
        return new RegExpValidator(options).validateLiteral(source);
    }
    function visitRegExpAST(node, handlers) {
        new RegExpVisitor(handlers).visit(node);
    }
    exports.AST = ast;
    exports.RegExpParser = RegExpParser;
    exports.RegExpValidator = RegExpValidator;
    exports.parseRegExpLiteral = parseRegExpLiteral;
    exports.validateRegExpLiteral = validateRegExpLiteral;
    exports.visitRegExpAST = visitRegExpAST;
},
/* eslint/lib/rules/no-debugger.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow the use of `debugger`",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-debugger"
            },
            fixable: null,
            schema: [],
            messages: {
                unexpected: "Unexpected 'debugger' statement."
            }
        },
        create(context) {
            return {
                DebuggerStatement(node) {
                    context.report({
                        node,
                        messageId: "unexpected"
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-delete-var.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow deleting variables",
                category: "Variables",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-delete-var"
            },
            schema: [],
            messages: {
                unexpected: "Variables should not be deleted."
            }
        },
        create(context) {
            return {
                UnaryExpression(node) {
                    if (node.operator === "delete" && node.argument.type === "Identifier") {
                        context.report({ node, messageId: "unexpected" });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-div-regex.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow division operators explicitly at the beginning of regular expressions",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-div-regex"
            },
            fixable: "code",
            schema: [],
            messages: {
                unexpected: "A regular expression literal can be confused with '/='."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                Literal(node) {
                    const token = sourceCode.getFirstToken(node);
                    if (token.type === "RegularExpression" && token.value[1] === "=") {
                        context.report({
                            node,
                            messageId: "unexpected",
                            fix(fixer) {
                                return fixer.replaceTextRange([token.range[0] + 1, token.range[0] + 2], "[=]");
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-dupe-args.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow duplicate arguments in `function` definitions",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-dupe-args"
            },
            schema: [],
            messages: {
                unexpected: "Duplicate param '{{name}}'."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Checks whether or not a given definition is a parameter's.
             * @param {eslint-scope.DefEntry} def A definition to check.
             * @returns {boolean} `true` if the definition is a parameter's.
             */
            function isParameter(def) {
                return def.type === "Parameter";
            }
            /**
             * Determines if a given node has duplicate parameters.
             * @param {ASTNode} node The node to check.
             * @returns {void}
             * @private
             */
            function checkParams(node) {
                const variables = context.getDeclaredVariables(node);
                for (let i = 0; i < variables.length; ++i) {
                    const variable = variables[i];
                    // Checks and reports duplications.
                    const defs = variable.defs.filter(isParameter);
                    if (defs.length >= 2) {
                        context.report({
                            node,
                            messageId: "unexpected",
                            data: { name: variable.name }
                        });
                    }
                }
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                FunctionDeclaration: checkParams,
                FunctionExpression: checkParams
            };
        }
    };
},
/* eslint/lib/rules/no-dupe-class-members.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow duplicate class members",
                category: "ECMAScript 6",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-dupe-class-members"
            },
            schema: [],
            messages: {
                unexpected: "Duplicate name '{{name}}'."
            }
        },
        create(context) {
            let stack = [];
            /**
             * Gets state of a given member name.
             * @param {string} name A name of a member.
             * @param {boolean} isStatic A flag which specifies that is a static member.
             * @returns {Object} A state of a given member name.
             *   - retv.init {boolean} A flag which shows the name is declared as normal member.
             *   - retv.get {boolean} A flag which shows the name is declared as getter.
             *   - retv.set {boolean} A flag which shows the name is declared as setter.
             */
            function getState(name, isStatic) {
                const stateMap = stack[stack.length - 1];
                const key = `$${name}`; // to avoid "__proto__".
                if (!stateMap[key]) {
                    stateMap[key] = {
                        nonStatic: { init: false, get: false, set: false },
                        static: { init: false, get: false, set: false }
                    };
                }
                return stateMap[key][isStatic ? "static" : "nonStatic"];
            }
            /**
             * Gets the name text of a given node.
             * @param {ASTNode} node A node to get the name.
             * @returns {string} The name text of the node.
             */
            function getName(node) {
                switch (node.type) {
                    case "Identifier": return node.name;
                    case "Literal": return String(node.value);
                    /* istanbul ignore next: syntax error */
                    default: return "";
                }
            }
            return {
                // Initializes the stack of state of member declarations.
                Program() {
                    stack = [];
                },
                // Initializes state of member declarations for the class.
                ClassBody() {
                    stack.push(Object.create(null));
                },
                // Disposes the state for the class.
                "ClassBody:exit"() {
                    stack.pop();
                },
                // Reports the node if its name has been declared already.
                MethodDefinition(node) {
                    if (node.computed) {
                        return;
                    }
                    const name = getName(node.key);
                    const state = getState(name, node.static);
                    let isDuplicate = false;
                    if (node.kind === "get") {
                        isDuplicate = (state.init || state.get);
                        state.get = true;
                    }
                    else if (node.kind === "set") {
                        isDuplicate = (state.init || state.set);
                        state.set = true;
                    }
                    else {
                        isDuplicate = (state.init || state.get || state.set);
                        state.init = true;
                    }
                    if (isDuplicate) {
                        context.report({ node, messageId: "unexpected", data: { name } });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-dupe-else-if.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Determines whether the first given array is a subset of the second given array.
     * @param {Function} comparator A function to compare two elements, should return `true` if they are equal.
     * @param {Array} arrA The array to compare from.
     * @param {Array} arrB The array to compare against.
     * @returns {boolean} `true` if the array `arrA` is a subset of the array `arrB`.
     */
    function isSubsetByComparator(comparator, arrA, arrB) {
        return arrA.every(a => arrB.some(b => comparator(a, b)));
    }
    /**
     * Splits the given node by the given logical operator.
     * @param {string} operator Logical operator `||` or `&&`.
     * @param {ASTNode} node The node to split.
     * @returns {ASTNode[]} Array of conditions that makes the node when joined by the operator.
     */
    function splitByLogicalOperator(operator, node) {
        if (node.type === "LogicalExpression" && node.operator === operator) {
            return [...splitByLogicalOperator(operator, node.left), ...splitByLogicalOperator(operator, node.right)];
        }
        return [node];
    }
    const splitByOr = splitByLogicalOperator.bind(null, "||");
    const splitByAnd = splitByLogicalOperator.bind(null, "&&");
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow duplicate conditions in if-else-if chains",
                category: "Possible Errors",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-dupe-else-if"
            },
            schema: [],
            messages: {
                unexpected: "This branch can never execute. Its condition is a duplicate or covered by previous conditions in the if-else-if chain."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Determines whether the two given nodes are considered to be equal. In particular, given that the nodes
             * represent expressions in a boolean context, `||` and `&&` can be considered as commutative operators.
             * @param {ASTNode} a First node.
             * @param {ASTNode} b Second node.
             * @returns {boolean} `true` if the nodes are considered to be equal.
             */
            function equal(a, b) {
                if (a.type !== b.type) {
                    return false;
                }
                if (a.type === "LogicalExpression" &&
                    (a.operator === "||" || a.operator === "&&") &&
                    a.operator === b.operator) {
                    return equal(a.left, b.left) && equal(a.right, b.right) ||
                        equal(a.left, b.right) && equal(a.right, b.left);
                }
                return astUtils.equalTokens(a, b, sourceCode);
            }
            const isSubset = isSubsetByComparator.bind(null, equal);
            return {
                IfStatement(node) {
                    const test = node.test, conditionsToCheck = test.type === "LogicalExpression" && test.operator === "&&"
                        ? [test, ...splitByAnd(test)]
                        : [test];
                    let current = node, listToCheck = conditionsToCheck.map(c => splitByOr(c).map(splitByAnd));
                    while (current.parent && current.parent.type === "IfStatement" && current.parent.alternate === current) {
                        current = current.parent;
                        const currentOrOperands = splitByOr(current.test).map(splitByAnd);
                        listToCheck = listToCheck.map(orOperands => orOperands.filter(orOperand => !currentOrOperands.some(currentOrOperand => isSubset(currentOrOperand, orOperand))));
                        if (listToCheck.some(orOperands => orOperands.length === 0)) {
                            context.report({ node: test, messageId: "unexpected" });
                            break;
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-dupe-keys.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const GET_KIND = /^(?:init|get)$/u;
    const SET_KIND = /^(?:init|set)$/u;
    /**
     * The class which stores properties' information of an object.
     */
    class ObjectInfo {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {ObjectInfo|null} upper The information of the outer object.
         * @param {ASTNode} node The ObjectExpression node of this information.
         */
        constructor(upper, node) {
            this.upper = upper;
            this.node = node;
            this.properties = new Map();
        }
        /**
         * Gets the information of the given Property node.
         * @param {ASTNode} node The Property node to get.
         * @returns {{get: boolean, set: boolean}} The information of the property.
         */
        getPropertyInfo(node) {
            const name = astUtils.getStaticPropertyName(node);
            if (!this.properties.has(name)) {
                this.properties.set(name, { get: false, set: false });
            }
            return this.properties.get(name);
        }
        /**
         * Checks whether the given property has been defined already or not.
         * @param {ASTNode} node The Property node to check.
         * @returns {boolean} `true` if the property has been defined.
         */
        isPropertyDefined(node) {
            const entry = this.getPropertyInfo(node);
            return ((GET_KIND.test(node.kind) && entry.get) ||
                (SET_KIND.test(node.kind) && entry.set));
        }
        /**
         * Defines the given property.
         * @param {ASTNode} node The Property node to define.
         * @returns {void}
         */
        defineProperty(node) {
            const entry = this.getPropertyInfo(node);
            if (GET_KIND.test(node.kind)) {
                entry.get = true;
            }
            if (SET_KIND.test(node.kind)) {
                entry.set = true;
            }
        }
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow duplicate keys in object literals",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-dupe-keys"
            },
            schema: [],
            messages: {
                unexpected: "Duplicate key '{{name}}'."
            }
        },
        create(context) {
            let info = null;
            return {
                ObjectExpression(node) {
                    info = new ObjectInfo(info, node);
                },
                "ObjectExpression:exit"() {
                    info = info.upper;
                },
                Property(node) {
                    const name = astUtils.getStaticPropertyName(node);
                    // Skip destructuring.
                    if (node.parent.type !== "ObjectExpression") {
                        return;
                    }
                    // Skip if the name is not static.
                    if (name === null) {
                        return;
                    }
                    // Reports if the name is defined already.
                    if (info.isPropertyDefined(node)) {
                        context.report({
                            node: info.node,
                            loc: node.key.loc,
                            messageId: "unexpected",
                            data: { name }
                        });
                    }
                    // Update info.
                    info.defineProperty(node);
                }
            };
        }
    };
},
/* eslint/lib/rules/no-duplicate-case.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow duplicate case labels",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-duplicate-case"
            },
            schema: [],
            messages: {
                unexpected: "Duplicate case label."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                SwitchStatement(node) {
                    const previousKeys = new Set();
                    for (const switchCase of node.cases) {
                        if (switchCase.test) {
                            const key = sourceCode.getText(switchCase.test);
                            if (previousKeys.has(key)) {
                                context.report({ node: switchCase, messageId: "unexpected" });
                            }
                            else {
                                previousKeys.add(key);
                            }
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-duplicate-imports.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    /**
     * Returns the name of the module imported or re-exported.
     * @param {ASTNode} node A node to get.
     * @returns {string} the name of the module, or empty string if no name.
     */
    function getValue(node) {
        if (node && node.source && node.source.value) {
            return node.source.value.trim();
        }
        return "";
    }
    /**
     * Checks if the name of the import or export exists in the given array, and reports if so.
     * @param {RuleContext} context The ESLint rule context object.
     * @param {ASTNode} node A node to get.
     * @param {string} value The name of the imported or exported module.
     * @param {string[]} array The array containing other imports or exports in the file.
     * @param {string} messageId A messageId to be reported after the name of the module
     *
     * @returns {void} No return value
     */
    function checkAndReport(context, node, value, array, messageId) {
        if (array.indexOf(value) !== -1) {
            context.report({
                node,
                messageId,
                data: {
                    module: value
                }
            });
        }
    }
    /**
     * @callback nodeCallback
     * @param {ASTNode} node A node to handle.
     */
    /**
     * Returns a function handling the imports of a given file
     * @param {RuleContext} context The ESLint rule context object.
     * @param {boolean} includeExports Whether or not to check for exports in addition to imports.
     * @param {string[]} importsInFile The array containing other imports in the file.
     * @param {string[]} exportsInFile The array containing other exports in the file.
     *
     * @returns {nodeCallback} A function passed to ESLint to handle the statement.
     */
    function handleImports(context, includeExports, importsInFile, exportsInFile) {
        return function (node) {
            const value = getValue(node);
            if (value) {
                checkAndReport(context, node, value, importsInFile, "import");
                if (includeExports) {
                    checkAndReport(context, node, value, exportsInFile, "importAs");
                }
                importsInFile.push(value);
            }
        };
    }
    /**
     * Returns a function handling the exports of a given file
     * @param {RuleContext} context The ESLint rule context object.
     * @param {string[]} importsInFile The array containing other imports in the file.
     * @param {string[]} exportsInFile The array containing other exports in the file.
     *
     * @returns {nodeCallback} A function passed to ESLint to handle the statement.
     */
    function handleExports(context, importsInFile, exportsInFile) {
        return function (node) {
            const value = getValue(node);
            if (value) {
                checkAndReport(context, node, value, exportsInFile, "export");
                checkAndReport(context, node, value, importsInFile, "exportAs");
                exportsInFile.push(value);
            }
        };
    }
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow duplicate module imports",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-duplicate-imports"
            },
            schema: [{
                    type: "object",
                    properties: {
                        includeExports: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }],
            messages: {
                import: "'{{module}}' import is duplicated.",
                importAs: "'{{module}}' import is duplicated as export.",
                export: "'{{module}}' export is duplicated.",
                exportAs: "'{{module}}' export is duplicated as import."
            }
        },
        create(context) {
            const includeExports = (context.options[0] || {}).includeExports, importsInFile = [], exportsInFile = [];
            const handlers = {
                ImportDeclaration: handleImports(context, includeExports, importsInFile, exportsInFile)
            };
            if (includeExports) {
                handlers.ExportNamedDeclaration = handleExports(context, importsInFile, exportsInFile);
                handlers.ExportAllDeclaration = handleExports(context, importsInFile, exportsInFile);
            }
            return handlers;
        }
    };
},
/* eslint/lib/rules/no-else-return.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    const FixTracker = require(237) /* ./utils/fix-tracker */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `else` blocks after `return` statements in `if` statements",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-else-return"
            },
            schema: [{
                    type: "object",
                    properties: {
                        allowElseIf: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }],
            fixable: "code",
            messages: {
                unexpected: "Unnecessary 'else' after 'return'."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Checks whether the given names can be safely used to declare block-scoped variables
             * in the given scope. Name collisions can produce redeclaration syntax errors,
             * or silently change references and modify behavior of the original code.
             *
             * This is not a generic function. In particular, it is assumed that the scope is a function scope or
             * a function's inner scope, and that the names can be valid identifiers in the given scope.
             * @param {string[]} names Array of variable names.
             * @param {eslint-scope.Scope} scope Function scope or a function's inner scope.
             * @returns {boolean} True if all names can be safely declared, false otherwise.
             */
            function isSafeToDeclare(names, scope) {
                if (names.length === 0) {
                    return true;
                }
                const functionScope = scope.variableScope;
                /*
                 * If this is a function scope, scope.variables will contain parameters, implicit variables such as "arguments",
                 * all function-scoped variables ('var'), and block-scoped variables defined in the scope.
                 * If this is an inner scope, scope.variables will contain block-scoped variables defined in the scope.
                 *
                 * Redeclaring any of these would cause a syntax error, except for the implicit variables.
                 */
                const declaredVariables = scope.variables.filter(({ defs }) => defs.length > 0);
                if (declaredVariables.some(({ name }) => names.includes(name))) {
                    return false;
                }
                // Redeclaring a catch variable would also cause a syntax error.
                if (scope !== functionScope && scope.upper.type === "catch") {
                    if (scope.upper.variables.some(({ name }) => names.includes(name))) {
                        return false;
                    }
                }
                /*
                 * Redeclaring an implicit variable, such as "arguments", would not cause a syntax error.
                 * However, if the variable was used, declaring a new one with the same name would change references
                 * and modify behavior.
                 */
                const usedImplicitVariables = scope.variables.filter(({ defs, references }) => defs.length === 0 && references.length > 0);
                if (usedImplicitVariables.some(({ name }) => names.includes(name))) {
                    return false;
                }
                /*
                 * Declaring a variable with a name that was already used to reference a variable from an upper scope
                 * would change references and modify behavior.
                 */
                if (scope.through.some(t => names.includes(t.identifier.name))) {
                    return false;
                }
                /*
                 * If the scope is an inner scope (not the function scope), an uninitialized `var` variable declared inside
                 * the scope node (directly or in one of its descendants) is neither declared nor 'through' in the scope.
                 *
                 * For example, this would be a syntax error "Identifier 'a' has already been declared":
                 * function foo() { if (bar) { let a; if (baz) { var a; } } }
                 */
                if (scope !== functionScope) {
                    const scopeNodeRange = scope.block.range;
                    const variablesToCheck = functionScope.variables.filter(({ name }) => names.includes(name));
                    if (variablesToCheck.some(v => v.defs.some(({ node: { range } }) => scopeNodeRange[0] <= range[0] && range[1] <= scopeNodeRange[1]))) {
                        return false;
                    }
                }
                return true;
            }
            /**
             * Checks whether the removal of `else` and its braces is safe from variable name collisions.
             * @param {Node} node The 'else' node.
             * @param {eslint-scope.Scope} scope The scope in which the node and the whole 'if' statement is.
             * @returns {boolean} True if it is safe, false otherwise.
             */
            function isSafeFromNameCollisions(node, scope) {
                if (node.type === "FunctionDeclaration") {
                    // Conditional function declaration. Scope and hoisting are unpredictable, different engines work differently.
                    return false;
                }
                if (node.type !== "BlockStatement") {
                    return true;
                }
                const elseBlockScope = scope.childScopes.find(({ block }) => block === node);
                if (!elseBlockScope) {
                    // ecmaVersion < 6, `else` block statement cannot have its own scope, no possible collisions.
                    return true;
                }
                /*
                 * elseBlockScope is supposed to merge into its upper scope. elseBlockScope.variables array contains
                 * only block-scoped variables (such as let and const variables or class and function declarations)
                 * defined directly in the elseBlockScope. These are exactly the only names that could cause collisions.
                 */
                const namesToCheck = elseBlockScope.variables.map(({ name }) => name);
                return isSafeToDeclare(namesToCheck, scope);
            }
            /**
             * Display the context report if rule is violated
             * @param {Node} node The 'else' node
             * @returns {void}
             */
            function displayReport(node) {
                const currentScope = context.getScope();
                context.report({
                    node,
                    messageId: "unexpected",
                    fix: fixer => {
                        if (!isSafeFromNameCollisions(node, currentScope)) {
                            return null;
                        }
                        const sourceCode = context.getSourceCode();
                        const startToken = sourceCode.getFirstToken(node);
                        const elseToken = sourceCode.getTokenBefore(startToken);
                        const source = sourceCode.getText(node);
                        const lastIfToken = sourceCode.getTokenBefore(elseToken);
                        let fixedSource, firstTokenOfElseBlock;
                        if (startToken.type === "Punctuator" && startToken.value === "{") {
                            firstTokenOfElseBlock = sourceCode.getTokenAfter(startToken);
                        }
                        else {
                            firstTokenOfElseBlock = startToken;
                        }
                        /*
                         * If the if block does not have curly braces and does not end in a semicolon
                         * and the else block starts with (, [, /, +, ` or -, then it is not
                         * safe to remove the else keyword, because ASI will not add a semicolon
                         * after the if block
                         */
                        const ifBlockMaybeUnsafe = node.parent.consequent.type !== "BlockStatement" && lastIfToken.value !== ";";
                        const elseBlockUnsafe = /^[([/+`-]/u.test(firstTokenOfElseBlock.value);
                        if (ifBlockMaybeUnsafe && elseBlockUnsafe) {
                            return null;
                        }
                        const endToken = sourceCode.getLastToken(node);
                        const lastTokenOfElseBlock = sourceCode.getTokenBefore(endToken);
                        if (lastTokenOfElseBlock.value !== ";") {
                            const nextToken = sourceCode.getTokenAfter(endToken);
                            const nextTokenUnsafe = nextToken && /^[([/+`-]/u.test(nextToken.value);
                            const nextTokenOnSameLine = nextToken && nextToken.loc.start.line === lastTokenOfElseBlock.loc.start.line;
                            /*
                             * If the else block contents does not end in a semicolon,
                             * and the else block starts with (, [, /, +, ` or -, then it is not
                             * safe to remove the else block, because ASI will not add a semicolon
                             * after the remaining else block contents
                             */
                            if (nextTokenUnsafe || (nextTokenOnSameLine && nextToken.value !== "}")) {
                                return null;
                            }
                        }
                        if (startToken.type === "Punctuator" && startToken.value === "{") {
                            fixedSource = source.slice(1, -1);
                        }
                        else {
                            fixedSource = source;
                        }
                        /*
                         * Extend the replacement range to include the entire
                         * function to avoid conflicting with no-useless-return.
                         * https://github.com/eslint/eslint/issues/8026
                         *
                         * Also, to avoid name collisions between two else blocks.
                         */
                        return new FixTracker(fixer, sourceCode)
                            .retainEnclosingFunction(node)
                            .replaceTextRange([elseToken.range[0], node.range[1]], fixedSource);
                    }
                });
            }
            /**
             * Check to see if the node is a ReturnStatement
             * @param {Node} node The node being evaluated
             * @returns {boolean} True if node is a return
             */
            function checkForReturn(node) {
                return node.type === "ReturnStatement";
            }
            /**
             * Naive return checking, does not iterate through the whole
             * BlockStatement because we make the assumption that the ReturnStatement
             * will be the last node in the body of the BlockStatement.
             * @param {Node} node The consequent/alternate node
             * @returns {boolean} True if it has a return
             */
            function naiveHasReturn(node) {
                if (node.type === "BlockStatement") {
                    const body = node.body, lastChildNode = body[body.length - 1];
                    return lastChildNode && checkForReturn(lastChildNode);
                }
                return checkForReturn(node);
            }
            /**
             * Check to see if the node is valid for evaluation,
             * meaning it has an else.
             * @param {Node} node The node being evaluated
             * @returns {boolean} True if the node is valid
             */
            function hasElse(node) {
                return node.alternate && node.consequent;
            }
            /**
             * If the consequent is an IfStatement, check to see if it has an else
             * and both its consequent and alternate path return, meaning this is
             * a nested case of rule violation.  If-Else not considered currently.
             * @param {Node} node The consequent node
             * @returns {boolean} True if this is a nested rule violation
             */
            function checkForIf(node) {
                return node.type === "IfStatement" && hasElse(node) &&
                    naiveHasReturn(node.alternate) && naiveHasReturn(node.consequent);
            }
            /**
             * Check the consequent/body node to make sure it is not
             * a ReturnStatement or an IfStatement that returns on both
             * code paths.
             * @param {Node} node The consequent or body node
             * @returns {boolean} `true` if it is a Return/If node that always returns.
             */
            function checkForReturnOrIf(node) {
                return checkForReturn(node) || checkForIf(node);
            }
            /**
             * Check whether a node returns in every codepath.
             * @param {Node} node The node to be checked
             * @returns {boolean} `true` if it returns on every codepath.
             */
            function alwaysReturns(node) {
                if (node.type === "BlockStatement") {
                    // If we have a BlockStatement, check each consequent body node.
                    return node.body.some(checkForReturnOrIf);
                }
                /*
                 * If not a block statement, make sure the consequent isn't a
                 * ReturnStatement or an IfStatement with returns on both paths.
                 */
                return checkForReturnOrIf(node);
            }
            /**
             * Check the if statement, but don't catch else-if blocks.
             * @returns {void}
             * @param {Node} node The node for the if statement to check
             * @private
             */
            function checkIfWithoutElse(node) {
                const parent = node.parent;
                /*
                 * Fixing this would require splitting one statement into two, so no error should
                 * be reported if this node is in a position where only one statement is allowed.
                 */
                if (!astUtils.STATEMENT_LIST_PARENTS.has(parent.type)) {
                    return;
                }
                const consequents = [];
                let alternate;
                for (let currentNode = node; currentNode.type === "IfStatement"; currentNode = currentNode.alternate) {
                    if (!currentNode.alternate) {
                        return;
                    }
                    consequents.push(currentNode.consequent);
                    alternate = currentNode.alternate;
                }
                if (consequents.every(alwaysReturns)) {
                    displayReport(alternate);
                }
            }
            /**
             * Check the if statement
             * @returns {void}
             * @param {Node} node The node for the if statement to check
             * @private
             */
            function checkIfWithElse(node) {
                const parent = node.parent;
                /*
                 * Fixing this would require splitting one statement into two, so no error should
                 * be reported if this node is in a position where only one statement is allowed.
                 */
                if (!astUtils.STATEMENT_LIST_PARENTS.has(parent.type)) {
                    return;
                }
                const alternate = node.alternate;
                if (alternate && alwaysReturns(node.consequent)) {
                    displayReport(alternate);
                }
            }
            const allowElseIf = !(context.options[0] && context.options[0].allowElseIf === false);
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                "IfStatement:exit": allowElseIf ? checkIfWithoutElse : checkIfWithElse
            };
        }
    };
},
/* eslint/lib/rules/utils/fix-tracker.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./ast-utils */;
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * A helper class to combine fix options into a fix command. Currently, it
     * exposes some "retain" methods that extend the range of the text being
     * replaced so that other fixes won't touch that region in the same pass.
     */
    class FixTracker {
        /**
         * Create a new FixTracker.
         * @param {ruleFixer} fixer A ruleFixer instance.
         * @param {SourceCode} sourceCode A SourceCode object for the current code.
         */
        constructor(fixer, sourceCode) {
            this.fixer = fixer;
            this.sourceCode = sourceCode;
            this.retainedRange = null;
        }
        /**
         * Mark the given range as "retained", meaning that other fixes may not
         * may not modify this region in the same pass.
         * @param {int[]} range The range to retain.
         * @returns {FixTracker} The same RuleFixer, for chained calls.
         */
        retainRange(range) {
            this.retainedRange = range;
            return this;
        }
        /**
         * Given a node, find the function containing it (or the entire program) and
         * mark it as retained, meaning that other fixes may not modify it in this
         * pass. This is useful for avoiding conflicts in fixes that modify control
         * flow.
         * @param {ASTNode} node The node to use as a starting point.
         * @returns {FixTracker} The same RuleFixer, for chained calls.
         */
        retainEnclosingFunction(node) {
            const functionNode = astUtils.getUpperFunction(node);
            return this.retainRange(functionNode ? functionNode.range : this.sourceCode.ast.range);
        }
        /**
         * Given a node or token, find the token before and afterward, and mark that
         * range as retained, meaning that other fixes may not modify it in this
         * pass. This is useful for avoiding conflicts in fixes that make a small
         * change to the code where the AST should not be changed.
         * @param {ASTNode|Token} nodeOrToken The node or token to use as a starting
         *      point. The token to the left and right are use in the range.
         * @returns {FixTracker} The same RuleFixer, for chained calls.
         */
        retainSurroundingTokens(nodeOrToken) {
            const tokenBefore = this.sourceCode.getTokenBefore(nodeOrToken) || nodeOrToken;
            const tokenAfter = this.sourceCode.getTokenAfter(nodeOrToken) || nodeOrToken;
            return this.retainRange([tokenBefore.range[0], tokenAfter.range[1]]);
        }
        /**
         * Create a fix command that replaces the given range with the given text,
         * accounting for any retained ranges.
         * @param {int[]} range The range to remove in the fix.
         * @param {string} text The text to insert in place of the range.
         * @returns {Object} The fix command.
         */
        replaceTextRange(range, text) {
            let actualRange;
            if (this.retainedRange) {
                actualRange = [
                    Math.min(this.retainedRange[0], range[0]),
                    Math.max(this.retainedRange[1], range[1])
                ];
            }
            else {
                actualRange = range;
            }
            return this.fixer.replaceTextRange(actualRange, this.sourceCode.text.slice(actualRange[0], range[0]) +
                text +
                this.sourceCode.text.slice(range[1], actualRange[1]));
        }
        /**
         * Create a fix command that removes the given node or token, accounting for
         * any retained ranges.
         * @param {ASTNode|Token} nodeOrToken The node or token to remove.
         * @returns {Object} The fix command.
         */
        remove(nodeOrToken) {
            return this.replaceTextRange(nodeOrToken.range, "");
        }
    }
    module.exports = FixTracker;
},
/* eslint/lib/rules/no-empty.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow empty block statements",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-empty"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allowEmptyCatch: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpected: "Empty {{type}} statement."
            }
        },
        create(context) {
            const options = context.options[0] || {}, allowEmptyCatch = options.allowEmptyCatch || false;
            const sourceCode = context.getSourceCode();
            return {
                BlockStatement(node) {
                    // if the body is not empty, we can just return immediately
                    if (node.body.length !== 0) {
                        return;
                    }
                    // a function is generally allowed to be empty
                    if (astUtils.isFunction(node.parent)) {
                        return;
                    }
                    if (allowEmptyCatch && node.parent.type === "CatchClause") {
                        return;
                    }
                    // any other block is only allowed to be empty, if it contains a comment
                    if (sourceCode.getCommentsInside(node).length > 0) {
                        return;
                    }
                    context.report({ node, messageId: "unexpected", data: { type: "block" } });
                },
                SwitchStatement(node) {
                    if (typeof node.cases === "undefined" || node.cases.length === 0) {
                        context.report({ node, messageId: "unexpected", data: { type: "switch" } });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-empty-character-class.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /*
     * plain-English description of the following regexp:
     * 0. `^` fix the match at the beginning of the string
     * 1. `\/`: the `/` that begins the regexp
     * 2. `([^\\[]|\\.|\[([^\\\]]|\\.)+\])*`: regexp contents; 0 or more of the following
     * 2.0. `[^\\[]`: any character that's not a `\` or a `[` (anything but escape sequences and character classes)
     * 2.1. `\\.`: an escape sequence
     * 2.2. `\[([^\\\]]|\\.)+\]`: a character class that isn't empty
     * 3. `\/` the `/` that ends the regexp
     * 4. `[gimuy]*`: optional regexp flags
     * 5. `$`: fix the match at the end of the string
     */
    const regex = /^\/([^\\[]|\\.|\[([^\\\]]|\\.)+\])*\/[gimuys]*$/u;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow empty character classes in regular expressions",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-empty-character-class"
            },
            schema: [],
            messages: {
                unexpected: "Empty class."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                Literal(node) {
                    const token = sourceCode.getFirstToken(node);
                    if (token.type === "RegularExpression" && !regex.test(token.value)) {
                        context.report({ node, messageId: "unexpected" });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-empty-function.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const ALLOW_OPTIONS = Object.freeze([
        "functions",
        "arrowFunctions",
        "generatorFunctions",
        "methods",
        "generatorMethods",
        "getters",
        "setters",
        "constructors"
    ]);
    /**
     * Gets the kind of a given function node.
     * @param {ASTNode} node A function node to get. This is one of
     *      an ArrowFunctionExpression, a FunctionDeclaration, or a
     *      FunctionExpression.
     * @returns {string} The kind of the function. This is one of "functions",
     *      "arrowFunctions", "generatorFunctions", "asyncFunctions", "methods",
     *      "generatorMethods", "asyncMethods", "getters", "setters", and
     *      "constructors".
     */
    function getKind(node) {
        const parent = node.parent;
        let kind = "";
        if (node.type === "ArrowFunctionExpression") {
            return "arrowFunctions";
        }
        // Detects main kind.
        if (parent.type === "Property") {
            if (parent.kind === "get") {
                return "getters";
            }
            if (parent.kind === "set") {
                return "setters";
            }
            kind = parent.method ? "methods" : "functions";
        }
        else if (parent.type === "MethodDefinition") {
            if (parent.kind === "get") {
                return "getters";
            }
            if (parent.kind === "set") {
                return "setters";
            }
            if (parent.kind === "constructor") {
                return "constructors";
            }
            kind = "methods";
        }
        else {
            kind = "functions";
        }
        // Detects prefix.
        let prefix = "";
        if (node.generator) {
            prefix = "generator";
        }
        else if (node.async) {
            prefix = "async";
        }
        else {
            return kind;
        }
        return prefix + kind[0].toUpperCase() + kind.slice(1);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow empty functions",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-empty-function"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allow: {
                            type: "array",
                            items: { enum: ALLOW_OPTIONS },
                            uniqueItems: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpected: "Unexpected empty {{name}}."
            }
        },
        create(context) {
            const options = context.options[0] || {};
            const allowed = options.allow || [];
            const sourceCode = context.getSourceCode();
            /**
             * Reports a given function node if the node matches the following patterns.
             *
             * - Not allowed by options.
             * - The body is empty.
             * - The body doesn't have any comments.
             * @param {ASTNode} node A function node to report. This is one of
             *      an ArrowFunctionExpression, a FunctionDeclaration, or a
             *      FunctionExpression.
             * @returns {void}
             */
            function reportIfEmpty(node) {
                const kind = getKind(node);
                const name = astUtils.getFunctionNameWithKind(node);
                const innerComments = sourceCode.getTokens(node.body, {
                    includeComments: true,
                    filter: astUtils.isCommentToken
                });
                if (allowed.indexOf(kind) === -1 &&
                    node.body.type === "BlockStatement" &&
                    node.body.body.length === 0 &&
                    innerComments.length === 0) {
                    context.report({
                        node,
                        loc: node.body.loc.start,
                        messageId: "unexpected",
                        data: { name }
                    });
                }
            }
            return {
                ArrowFunctionExpression: reportIfEmpty,
                FunctionDeclaration: reportIfEmpty,
                FunctionExpression: reportIfEmpty
            };
        }
    };
},
/* eslint/lib/rules/no-empty-pattern.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow empty destructuring patterns",
                category: "Best Practices",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-empty-pattern"
            },
            schema: [],
            messages: {
                unexpected: "Unexpected empty {{type}} pattern."
            }
        },
        create(context) {
            return {
                ObjectPattern(node) {
                    if (node.properties.length === 0) {
                        context.report({ node, messageId: "unexpected", data: { type: "object" } });
                    }
                },
                ArrayPattern(node) {
                    if (node.elements.length === 0) {
                        context.report({ node, messageId: "unexpected", data: { type: "array" } });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-eq-null.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `null` comparisons without type-checking operators",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-eq-null"
            },
            schema: [],
            messages: {
                unexpected: "Use '===' to compare with null."
            }
        },
        create(context) {
            return {
                BinaryExpression(node) {
                    const badOperator = node.operator === "==" || node.operator === "!=";
                    if (node.right.type === "Literal" && node.right.raw === "null" && badOperator ||
                        node.left.type === "Literal" && node.left.raw === "null" && badOperator) {
                        context.report({ node, messageId: "unexpected" });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-eval.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const candidatesOfGlobalObject = Object.freeze([
        "global",
        "window"
    ]);
    /**
     * Checks a given node is a Identifier node of the specified name.
     * @param {ASTNode} node A node to check.
     * @param {string} name A name to check.
     * @returns {boolean} `true` if the node is a Identifier node of the name.
     */
    function isIdentifier(node, name) {
        return node.type === "Identifier" && node.name === name;
    }
    /**
     * Checks a given node is a Literal node of the specified string value.
     * @param {ASTNode} node A node to check.
     * @param {string} name A name to check.
     * @returns {boolean} `true` if the node is a Literal node of the name.
     */
    function isConstant(node, name) {
        switch (node.type) {
            case "Literal":
                return node.value === name;
            case "TemplateLiteral":
                return (node.expressions.length === 0 &&
                    node.quasis[0].value.cooked === name);
            default:
                return false;
        }
    }
    /**
     * Checks a given node is a MemberExpression node which has the specified name's
     * property.
     * @param {ASTNode} node A node to check.
     * @param {string} name A name to check.
     * @returns {boolean} `true` if the node is a MemberExpression node which has
     *      the specified name's property
     */
    function isMember(node, name) {
        return (node.type === "MemberExpression" &&
            (node.computed ? isConstant : isIdentifier)(node.property, name));
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the use of `eval()`",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-eval"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allowIndirect: { type: "boolean", default: false }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpected: "eval can be harmful."
            }
        },
        create(context) {
            const allowIndirect = Boolean(context.options[0] &&
                context.options[0].allowIndirect);
            const sourceCode = context.getSourceCode();
            let funcInfo = null;
            /**
             * Pushs a variable scope (Program or Function) information to the stack.
             *
             * This is used in order to check whether or not `this` binding is a
             * reference to the global object.
             * @param {ASTNode} node A node of the scope. This is one of Program,
             *      FunctionDeclaration, FunctionExpression, and ArrowFunctionExpression.
             * @returns {void}
             */
            function enterVarScope(node) {
                const strict = context.getScope().isStrict;
                funcInfo = {
                    upper: funcInfo,
                    node,
                    strict,
                    defaultThis: false,
                    initialized: strict
                };
            }
            /**
             * Pops a variable scope from the stack.
             * @returns {void}
             */
            function exitVarScope() {
                funcInfo = funcInfo.upper;
            }
            /**
             * Reports a given node.
             *
             * `node` is `Identifier` or `MemberExpression`.
             * The parent of `node` might be `CallExpression`.
             *
             * The location of the report is always `eval` `Identifier` (or possibly
             * `Literal`). The type of the report is `CallExpression` if the parent is
             * `CallExpression`. Otherwise, it's the given node type.
             * @param {ASTNode} node A node to report.
             * @returns {void}
             */
            function report(node) {
                const parent = node.parent;
                const locationNode = node.type === "MemberExpression"
                    ? node.property
                    : node;
                const reportNode = parent.type === "CallExpression" && parent.callee === node
                    ? parent
                    : node;
                context.report({
                    node: reportNode,
                    loc: locationNode.loc.start,
                    messageId: "unexpected"
                });
            }
            /**
             * Reports accesses of `eval` via the global object.
             * @param {eslint-scope.Scope} globalScope The global scope.
             * @returns {void}
             */
            function reportAccessingEvalViaGlobalObject(globalScope) {
                for (let i = 0; i < candidatesOfGlobalObject.length; ++i) {
                    const name = candidatesOfGlobalObject[i];
                    const variable = astUtils.getVariableByName(globalScope, name);
                    if (!variable) {
                        continue;
                    }
                    const references = variable.references;
                    for (let j = 0; j < references.length; ++j) {
                        const identifier = references[j].identifier;
                        let node = identifier.parent;
                        // To detect code like `window.window.eval`.
                        while (isMember(node, name)) {
                            node = node.parent;
                        }
                        // Reports.
                        if (isMember(node, "eval")) {
                            report(node);
                        }
                    }
                }
            }
            /**
             * Reports all accesses of `eval` (excludes direct calls to eval).
             * @param {eslint-scope.Scope} globalScope The global scope.
             * @returns {void}
             */
            function reportAccessingEval(globalScope) {
                const variable = astUtils.getVariableByName(globalScope, "eval");
                if (!variable) {
                    return;
                }
                const references = variable.references;
                for (let i = 0; i < references.length; ++i) {
                    const reference = references[i];
                    const id = reference.identifier;
                    if (id.name === "eval" && !astUtils.isCallee(id)) {
                        // Is accessing to eval (excludes direct calls to eval)
                        report(id);
                    }
                }
            }
            if (allowIndirect) {
                // Checks only direct calls to eval. It's simple!
                return {
                    "CallExpression:exit"(node) {
                        const callee = node.callee;
                        if (isIdentifier(callee, "eval")) {
                            report(callee);
                        }
                    }
                };
            }
            return {
                "CallExpression:exit"(node) {
                    const callee = node.callee;
                    if (isIdentifier(callee, "eval")) {
                        report(callee);
                    }
                },
                Program(node) {
                    const scope = context.getScope(), features = context.parserOptions.ecmaFeatures || {}, strict = scope.isStrict ||
                        node.sourceType === "module" ||
                        (features.globalReturn && scope.childScopes[0].isStrict);
                    funcInfo = {
                        upper: null,
                        node,
                        strict,
                        defaultThis: true,
                        initialized: true
                    };
                },
                "Program:exit"() {
                    const globalScope = context.getScope();
                    exitVarScope();
                    reportAccessingEval(globalScope);
                    reportAccessingEvalViaGlobalObject(globalScope);
                },
                FunctionDeclaration: enterVarScope,
                "FunctionDeclaration:exit": exitVarScope,
                FunctionExpression: enterVarScope,
                "FunctionExpression:exit": exitVarScope,
                ArrowFunctionExpression: enterVarScope,
                "ArrowFunctionExpression:exit": exitVarScope,
                ThisExpression(node) {
                    if (!isMember(node.parent, "eval")) {
                        return;
                    }
                    /*
                     * `this.eval` is found.
                     * Checks whether or not the value of `this` is the global object.
                     */
                    if (!funcInfo.initialized) {
                        funcInfo.initialized = true;
                        funcInfo.defaultThis = astUtils.isDefaultThisBinding(funcInfo.node, sourceCode);
                    }
                    if (!funcInfo.strict && funcInfo.defaultThis) {
                        // `this.eval` is possible built-in `eval`.
                        report(node.parent);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-ex-assign.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow reassigning exceptions in `catch` clauses",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-ex-assign"
            },
            schema: [],
            messages: {
                unexpected: "Do not assign to the exception parameter."
            }
        },
        create(context) {
            /**
             * Finds and reports references that are non initializer and writable.
             * @param {Variable} variable A variable to check.
             * @returns {void}
             */
            function checkVariable(variable) {
                astUtils.getModifyingReferences(variable.references).forEach(reference => {
                    context.report({ node: reference.identifier, messageId: "unexpected" });
                });
            }
            return {
                CatchClause(node) {
                    context.getDeclaredVariables(node).forEach(checkVariable);
                }
            };
        }
    };
},
/* eslint/lib/rules/no-extend-native.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    const globals = require(113) /* globals */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const propertyDefinitionMethods = new Set(["defineProperty", "defineProperties"]);
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow extending native types",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-extend-native"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        exceptions: {
                            type: "array",
                            items: {
                                type: "string"
                            },
                            uniqueItems: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpected: "{{builtin}} prototype is read only, properties should not be added."
            }
        },
        create(context) {
            const config = context.options[0] || {};
            const exceptions = new Set(config.exceptions || []);
            const modifiedBuiltins = new Set(Object.keys(globals.builtin)
                .filter(builtin => builtin[0].toUpperCase() === builtin[0])
                .filter(builtin => !exceptions.has(builtin)));
            /**
             * Reports a lint error for the given node.
             * @param {ASTNode} node The node to report.
             * @param {string} builtin The name of the native builtin being extended.
             * @returns {void}
             */
            function reportNode(node, builtin) {
                context.report({
                    node,
                    messageId: "unexpected",
                    data: {
                        builtin
                    }
                });
            }
            /**
             * Check to see if the `prototype` property of the given object
             * identifier node is being accessed.
             * @param {ASTNode} identifierNode The Identifier representing the object
             * to check.
             * @returns {boolean} True if the identifier is the object of a
             * MemberExpression and its `prototype` property is being accessed,
             * false otherwise.
             */
            function isPrototypePropertyAccessed(identifierNode) {
                return Boolean(identifierNode &&
                    identifierNode.parent &&
                    identifierNode.parent.type === "MemberExpression" &&
                    identifierNode.parent.object === identifierNode &&
                    astUtils.getStaticPropertyName(identifierNode.parent) === "prototype");
            }
            /**
             * Checks that an identifier is an object of a prototype whose member
             * is being assigned in an AssignmentExpression.
             * Example: Object.prototype.foo = "bar"
             * @param {ASTNode} identifierNode The identifier to check.
             * @returns {boolean} True if the identifier's prototype is modified.
             */
            function isInPrototypePropertyAssignment(identifierNode) {
                return Boolean(isPrototypePropertyAccessed(identifierNode) &&
                    identifierNode.parent.parent.type === "MemberExpression" &&
                    identifierNode.parent.parent.parent.type === "AssignmentExpression" &&
                    identifierNode.parent.parent.parent.left === identifierNode.parent.parent);
            }
            /**
             * Checks that an identifier is an object of a prototype whose member
             * is being extended via the Object.defineProperty() or
             * Object.defineProperties() methods.
             * Example: Object.defineProperty(Array.prototype, "foo", ...)
             * Example: Object.defineProperties(Array.prototype, ...)
             * @param {ASTNode} identifierNode The identifier to check.
             * @returns {boolean} True if the identifier's prototype is modified.
             */
            function isInDefinePropertyCall(identifierNode) {
                return Boolean(isPrototypePropertyAccessed(identifierNode) &&
                    identifierNode.parent.parent.type === "CallExpression" &&
                    identifierNode.parent.parent.arguments[0] === identifierNode.parent &&
                    identifierNode.parent.parent.callee.type === "MemberExpression" &&
                    identifierNode.parent.parent.callee.object.type === "Identifier" &&
                    identifierNode.parent.parent.callee.object.name === "Object" &&
                    identifierNode.parent.parent.callee.property.type === "Identifier" &&
                    propertyDefinitionMethods.has(identifierNode.parent.parent.callee.property.name));
            }
            /**
             * Check to see if object prototype access is part of a prototype
             * extension. There are three ways a prototype can be extended:
             * 1. Assignment to prototype property (Object.prototype.foo = 1)
             * 2. Object.defineProperty()/Object.defineProperties() on a prototype
             * If prototype extension is detected, report the AssignmentExpression
             * or CallExpression node.
             * @param {ASTNode} identifierNode The Identifier representing the object
             * which prototype is being accessed and possibly extended.
             * @returns {void}
             */
            function checkAndReportPrototypeExtension(identifierNode) {
                if (isInPrototypePropertyAssignment(identifierNode)) {
                    // Identifier --> MemberExpression --> MemberExpression --> AssignmentExpression
                    reportNode(identifierNode.parent.parent.parent, identifierNode.name);
                }
                else if (isInDefinePropertyCall(identifierNode)) {
                    // Identifier --> MemberExpression --> CallExpression
                    reportNode(identifierNode.parent.parent, identifierNode.name);
                }
            }
            return {
                "Program:exit"() {
                    const globalScope = context.getScope();
                    modifiedBuiltins.forEach(builtin => {
                        const builtinVar = globalScope.set.get(builtin);
                        if (builtinVar && builtinVar.references) {
                            builtinVar.references
                                .map(ref => ref.identifier)
                                .forEach(checkAndReportPrototypeExtension);
                        }
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-extra-bind.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const SIDE_EFFECT_FREE_NODE_TYPES = new Set(["Literal", "Identifier", "ThisExpression", "FunctionExpression"]);
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary calls to `.bind()`",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-extra-bind"
            },
            schema: [],
            fixable: "code",
            messages: {
                unexpected: "The function binding is unnecessary."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            let scopeInfo = null;
            /**
             * Checks if a node is free of side effects.
             *
             * This check is stricter than it needs to be, in order to keep the implementation simple.
             * @param {ASTNode} node A node to check.
             * @returns {boolean} True if the node is known to be side-effect free, false otherwise.
             */
            function isSideEffectFree(node) {
                return SIDE_EFFECT_FREE_NODE_TYPES.has(node.type);
            }
            /**
             * Reports a given function node.
             * @param {ASTNode} node A node to report. This is a FunctionExpression or
             *      an ArrowFunctionExpression.
             * @returns {void}
             */
            function report(node) {
                context.report({
                    node: node.parent.parent,
                    messageId: "unexpected",
                    loc: node.parent.property.loc.start,
                    fix(fixer) {
                        if (node.parent.parent.arguments.length && !isSideEffectFree(node.parent.parent.arguments[0])) {
                            return null;
                        }
                        const firstTokenToRemove = sourceCode
                            .getFirstTokenBetween(node.parent.object, node.parent.property, astUtils.isNotClosingParenToken);
                        const lastTokenToRemove = sourceCode.getLastToken(node.parent.parent);
                        if (sourceCode.commentsExistBetween(firstTokenToRemove, lastTokenToRemove)) {
                            return null;
                        }
                        return fixer.removeRange([firstTokenToRemove.range[0], node.parent.parent.range[1]]);
                    }
                });
            }
            /**
             * Checks whether or not a given function node is the callee of `.bind()`
             * method.
             *
             * e.g. `(function() {}.bind(foo))`
             * @param {ASTNode} node A node to report. This is a FunctionExpression or
             *      an ArrowFunctionExpression.
             * @returns {boolean} `true` if the node is the callee of `.bind()` method.
             */
            function isCalleeOfBindMethod(node) {
                const parent = node.parent;
                const grandparent = parent.parent;
                return (grandparent &&
                    grandparent.type === "CallExpression" &&
                    grandparent.callee === parent &&
                    grandparent.arguments.length === 1 &&
                    grandparent.arguments[0].type !== "SpreadElement" &&
                    parent.type === "MemberExpression" &&
                    parent.object === node &&
                    astUtils.getStaticPropertyName(parent) === "bind");
            }
            /**
             * Adds a scope information object to the stack.
             * @param {ASTNode} node A node to add. This node is a FunctionExpression
             *      or a FunctionDeclaration node.
             * @returns {void}
             */
            function enterFunction(node) {
                scopeInfo = {
                    isBound: isCalleeOfBindMethod(node),
                    thisFound: false,
                    upper: scopeInfo
                };
            }
            /**
             * Removes the scope information object from the top of the stack.
             * At the same time, this reports the function node if the function has
             * `.bind()` and the `this` keywords found.
             * @param {ASTNode} node A node to remove. This node is a
             *      FunctionExpression or a FunctionDeclaration node.
             * @returns {void}
             */
            function exitFunction(node) {
                if (scopeInfo.isBound && !scopeInfo.thisFound) {
                    report(node);
                }
                scopeInfo = scopeInfo.upper;
            }
            /**
             * Reports a given arrow function if the function is callee of `.bind()`
             * method.
             * @param {ASTNode} node A node to report. This node is an
             *      ArrowFunctionExpression.
             * @returns {void}
             */
            function exitArrowFunction(node) {
                if (isCalleeOfBindMethod(node)) {
                    report(node);
                }
            }
            /**
             * Set the mark as the `this` keyword was found in this scope.
             * @returns {void}
             */
            function markAsThisFound() {
                if (scopeInfo) {
                    scopeInfo.thisFound = true;
                }
            }
            return {
                "ArrowFunctionExpression:exit": exitArrowFunction,
                FunctionDeclaration: enterFunction,
                "FunctionDeclaration:exit": exitFunction,
                FunctionExpression: enterFunction,
                "FunctionExpression:exit": exitFunction,
                ThisExpression: markAsThisFound
            };
        }
    };
},
/* eslint/lib/rules/no-extra-boolean-cast.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary boolean casts",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-extra-boolean-cast"
            },
            schema: [],
            fixable: "code",
            messages: {
                unexpectedCall: "Redundant Boolean call.",
                unexpectedNegation: "Redundant double negation."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            // Node types which have a test which will coerce values to booleans.
            const BOOLEAN_NODE_TYPES = [
                "IfStatement",
                "DoWhileStatement",
                "WhileStatement",
                "ConditionalExpression",
                "ForStatement"
            ];
            /**
             * Check if a node is in a context where its value would be coerced to a boolean at runtime.
             * @param {ASTNode} node The node
             * @param {ASTNode} parent Its parent
             * @returns {boolean} If it is in a boolean context
             */
            function isInBooleanContext(node, parent) {
                return ((BOOLEAN_NODE_TYPES.indexOf(parent.type) !== -1 &&
                    node === parent.test) ||
                    // !<bool>
                    (parent.type === "UnaryExpression" &&
                        parent.operator === "!"));
            }
            /**
             * Check if a node has comments inside.
             * @param {ASTNode} node The node to check.
             * @returns {boolean} `true` if it has comments inside.
             */
            function hasCommentsInside(node) {
                return Boolean(sourceCode.getCommentsInside(node).length);
            }
            return {
                UnaryExpression(node) {
                    const ancestors = context.getAncestors(), parent = ancestors.pop(), grandparent = ancestors.pop();
                    // Exit early if it's guaranteed not to match
                    if (node.operator !== "!" ||
                        parent.type !== "UnaryExpression" ||
                        parent.operator !== "!") {
                        return;
                    }
                    if (isInBooleanContext(parent, grandparent) ||
                        // Boolean(<bool>) and new Boolean(<bool>)
                        ((grandparent.type === "CallExpression" || grandparent.type === "NewExpression") &&
                            grandparent.callee.type === "Identifier" &&
                            grandparent.callee.name === "Boolean")) {
                        context.report({
                            node: parent,
                            messageId: "unexpectedNegation",
                            fix: fixer => {
                                if (hasCommentsInside(parent)) {
                                    return null;
                                }
                                let prefix = "";
                                const tokenBefore = sourceCode.getTokenBefore(parent);
                                const firstReplacementToken = sourceCode.getFirstToken(node.argument);
                                if (tokenBefore && tokenBefore.range[1] === parent.range[0] &&
                                    !astUtils.canTokensBeAdjacent(tokenBefore, firstReplacementToken)) {
                                    prefix = " ";
                                }
                                return fixer.replaceText(parent, prefix + sourceCode.getText(node.argument));
                            }
                        });
                    }
                },
                CallExpression(node) {
                    const parent = node.parent;
                    if (node.callee.type !== "Identifier" || node.callee.name !== "Boolean") {
                        return;
                    }
                    if (isInBooleanContext(node, parent)) {
                        context.report({
                            node,
                            messageId: "unexpectedCall",
                            fix: fixer => {
                                if (!node.arguments.length) {
                                    if (parent.type === "UnaryExpression" && parent.operator === "!") {
                                        // !Boolean() -> true
                                        if (hasCommentsInside(parent)) {
                                            return null;
                                        }
                                        const replacement = "true";
                                        let prefix = "";
                                        const tokenBefore = sourceCode.getTokenBefore(parent);
                                        if (tokenBefore && tokenBefore.range[1] === parent.range[0] &&
                                            !astUtils.canTokensBeAdjacent(tokenBefore, replacement)) {
                                            prefix = " ";
                                        }
                                        return fixer.replaceText(parent, prefix + replacement);
                                    }
                                    // Boolean() -> false
                                    if (hasCommentsInside(node)) {
                                        return null;
                                    }
                                    return fixer.replaceText(node, "false");
                                }
                                if (node.arguments.length > 1 || node.arguments[0].type === "SpreadElement" ||
                                    hasCommentsInside(node)) {
                                    return null;
                                }
                                const argument = node.arguments[0];
                                if (astUtils.getPrecedence(argument) < astUtils.getPrecedence(node.parent)) {
                                    return fixer.replaceText(node, `(${sourceCode.getText(argument)})`);
                                }
                                return fixer.replaceText(node, sourceCode.getText(argument));
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-extra-label.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary labels",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-extra-label"
            },
            schema: [],
            fixable: "code",
            messages: {
                unexpected: "This label '{{name}}' is unnecessary."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            let scopeInfo = null;
            /**
             * Creates a new scope with a breakable statement.
             * @param {ASTNode} node A node to create. This is a BreakableStatement.
             * @returns {void}
             */
            function enterBreakableStatement(node) {
                scopeInfo = {
                    label: node.parent.type === "LabeledStatement" ? node.parent.label : null,
                    breakable: true,
                    upper: scopeInfo
                };
            }
            /**
             * Removes the top scope of the stack.
             * @returns {void}
             */
            function exitBreakableStatement() {
                scopeInfo = scopeInfo.upper;
            }
            /**
             * Creates a new scope with a labeled statement.
             *
             * This ignores it if the body is a breakable statement.
             * In this case it's handled in the `enterBreakableStatement` function.
             * @param {ASTNode} node A node to create. This is a LabeledStatement.
             * @returns {void}
             */
            function enterLabeledStatement(node) {
                if (!astUtils.isBreakableStatement(node.body)) {
                    scopeInfo = {
                        label: node.label,
                        breakable: false,
                        upper: scopeInfo
                    };
                }
            }
            /**
             * Removes the top scope of the stack.
             *
             * This ignores it if the body is a breakable statement.
             * In this case it's handled in the `exitBreakableStatement` function.
             * @param {ASTNode} node A node. This is a LabeledStatement.
             * @returns {void}
             */
            function exitLabeledStatement(node) {
                if (!astUtils.isBreakableStatement(node.body)) {
                    scopeInfo = scopeInfo.upper;
                }
            }
            /**
             * Reports a given control node if it's unnecessary.
             * @param {ASTNode} node A node. This is a BreakStatement or a
             *      ContinueStatement.
             * @returns {void}
             */
            function reportIfUnnecessary(node) {
                if (!node.label) {
                    return;
                }
                const labelNode = node.label;
                for (let info = scopeInfo; info !== null; info = info.upper) {
                    if (info.breakable || info.label && info.label.name === labelNode.name) {
                        if (info.breakable && info.label && info.label.name === labelNode.name) {
                            context.report({
                                node: labelNode,
                                messageId: "unexpected",
                                data: labelNode,
                                fix(fixer) {
                                    const breakOrContinueToken = sourceCode.getFirstToken(node);
                                    if (sourceCode.commentsExistBetween(breakOrContinueToken, labelNode)) {
                                        return null;
                                    }
                                    return fixer.removeRange([breakOrContinueToken.range[1], labelNode.range[1]]);
                                }
                            });
                        }
                        return;
                    }
                }
            }
            return {
                WhileStatement: enterBreakableStatement,
                "WhileStatement:exit": exitBreakableStatement,
                DoWhileStatement: enterBreakableStatement,
                "DoWhileStatement:exit": exitBreakableStatement,
                ForStatement: enterBreakableStatement,
                "ForStatement:exit": exitBreakableStatement,
                ForInStatement: enterBreakableStatement,
                "ForInStatement:exit": exitBreakableStatement,
                ForOfStatement: enterBreakableStatement,
                "ForOfStatement:exit": exitBreakableStatement,
                SwitchStatement: enterBreakableStatement,
                "SwitchStatement:exit": exitBreakableStatement,
                LabeledStatement: enterLabeledStatement,
                "LabeledStatement:exit": exitLabeledStatement,
                BreakStatement: reportIfUnnecessary,
                ContinueStatement: reportIfUnnecessary
            };
        }
    };
},
/* eslint/lib/rules/no-extra-parens.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const { isParenthesized: isParenthesizedRaw } = require(250) /* eslint-utils */;
    const astUtils = require(126) /* ./utils/ast-utils.js */;
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "disallow unnecessary parentheses",
                category: "Possible Errors",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-extra-parens"
            },
            fixable: "code",
            schema: {
                anyOf: [
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["functions"]
                            }
                        ],
                        minItems: 0,
                        maxItems: 1
                    },
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["all"]
                            },
                            {
                                type: "object",
                                properties: {
                                    conditionalAssign: { type: "boolean" },
                                    nestedBinaryExpressions: { type: "boolean" },
                                    returnAssign: { type: "boolean" },
                                    ignoreJSX: { enum: ["none", "all", "single-line", "multi-line"] },
                                    enforceForArrowConditionals: { type: "boolean" },
                                    enforceForSequenceExpressions: { type: "boolean" },
                                    enforceForNewInMemberExpressions: { type: "boolean" }
                                },
                                additionalProperties: false
                            }
                        ],
                        minItems: 0,
                        maxItems: 2
                    }
                ]
            },
            messages: {
                unexpected: "Unnecessary parentheses around expression."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const tokensToIgnore = new WeakSet();
            const precedence = astUtils.getPrecedence;
            const ALL_NODES = context.options[0] !== "functions";
            const EXCEPT_COND_ASSIGN = ALL_NODES && context.options[1] && context.options[1].conditionalAssign === false;
            const NESTED_BINARY = ALL_NODES && context.options[1] && context.options[1].nestedBinaryExpressions === false;
            const EXCEPT_RETURN_ASSIGN = ALL_NODES && context.options[1] && context.options[1].returnAssign === false;
            const IGNORE_JSX = ALL_NODES && context.options[1] && context.options[1].ignoreJSX;
            const IGNORE_ARROW_CONDITIONALS = ALL_NODES && context.options[1] &&
                context.options[1].enforceForArrowConditionals === false;
            const IGNORE_SEQUENCE_EXPRESSIONS = ALL_NODES && context.options[1] &&
                context.options[1].enforceForSequenceExpressions === false;
            const IGNORE_NEW_IN_MEMBER_EXPR = ALL_NODES && context.options[1] &&
                context.options[1].enforceForNewInMemberExpressions === false;
            const PRECEDENCE_OF_ASSIGNMENT_EXPR = precedence({ type: "AssignmentExpression" });
            const PRECEDENCE_OF_UPDATE_EXPR = precedence({ type: "UpdateExpression" });
            let reportsBuffer;
            /**
             * Determines if this rule should be enforced for a node given the current configuration.
             * @param {ASTNode} node The node to be checked.
             * @returns {boolean} True if the rule should be enforced for this node.
             * @private
             */
            function ruleApplies(node) {
                if (node.type === "JSXElement" || node.type === "JSXFragment") {
                    const isSingleLine = node.loc.start.line === node.loc.end.line;
                    switch (IGNORE_JSX) {
                        // Exclude this JSX element from linting
                        case "all":
                            return false;
                        // Exclude this JSX element if it is multi-line element
                        case "multi-line":
                            return isSingleLine;
                        // Exclude this JSX element if it is single-line element
                        case "single-line":
                            return !isSingleLine;
                        // Nothing special to be done for JSX elements
                        case "none":
                            break;
                        // no default
                    }
                }
                if (node.type === "SequenceExpression" && IGNORE_SEQUENCE_EXPRESSIONS) {
                    return false;
                }
                return ALL_NODES || node.type === "FunctionExpression" || node.type === "ArrowFunctionExpression";
            }
            /**
             * Determines if a node is surrounded by parentheses.
             * @param {ASTNode} node The node to be checked.
             * @returns {boolean} True if the node is parenthesised.
             * @private
             */
            function isParenthesised(node) {
                return isParenthesizedRaw(1, node, sourceCode);
            }
            /**
             * Determines if a node is surrounded by parentheses twice.
             * @param {ASTNode} node The node to be checked.
             * @returns {boolean} True if the node is doubly parenthesised.
             * @private
             */
            function isParenthesisedTwice(node) {
                return isParenthesizedRaw(2, node, sourceCode);
            }
            /**
             * Determines if a node is surrounded by (potentially) invalid parentheses.
             * @param {ASTNode} node The node to be checked.
             * @returns {boolean} True if the node is incorrectly parenthesised.
             * @private
             */
            function hasExcessParens(node) {
                return ruleApplies(node) && isParenthesised(node);
            }
            /**
             * Determines if a node that is expected to be parenthesised is surrounded by
             * (potentially) invalid extra parentheses.
             * @param {ASTNode} node The node to be checked.
             * @returns {boolean} True if the node is has an unexpected extra pair of parentheses.
             * @private
             */
            function hasDoubleExcessParens(node) {
                return ruleApplies(node) && isParenthesisedTwice(node);
            }
            /**
             * Determines if a node test expression is allowed to have a parenthesised assignment
             * @param {ASTNode} node The node to be checked.
             * @returns {boolean} True if the assignment can be parenthesised.
             * @private
             */
            function isCondAssignException(node) {
                return EXCEPT_COND_ASSIGN && node.test.type === "AssignmentExpression";
            }
            /**
             * Determines if a node is in a return statement
             * @param {ASTNode} node The node to be checked.
             * @returns {boolean} True if the node is in a return statement.
             * @private
             */
            function isInReturnStatement(node) {
                for (let currentNode = node; currentNode; currentNode = currentNode.parent) {
                    if (currentNode.type === "ReturnStatement" ||
                        (currentNode.type === "ArrowFunctionExpression" && currentNode.body.type !== "BlockStatement")) {
                        return true;
                    }
                }
                return false;
            }
            /**
             * Determines if a constructor function is newed-up with parens
             * @param {ASTNode} newExpression The NewExpression node to be checked.
             * @returns {boolean} True if the constructor is called with parens.
             * @private
             */
            function isNewExpressionWithParens(newExpression) {
                const lastToken = sourceCode.getLastToken(newExpression);
                const penultimateToken = sourceCode.getTokenBefore(lastToken);
                return newExpression.arguments.length > 0 ||
                    (
                    // The expression should end with its own parens, e.g., new new foo() is not a new expression with parens
                    astUtils.isOpeningParenToken(penultimateToken) &&
                        astUtils.isClosingParenToken(lastToken) &&
                        newExpression.callee.range[1] < newExpression.range[1]);
            }
            /**
             * Determines if a node is or contains an assignment expression
             * @param {ASTNode} node The node to be checked.
             * @returns {boolean} True if the node is or contains an assignment expression.
             * @private
             */
            function containsAssignment(node) {
                if (node.type === "AssignmentExpression") {
                    return true;
                }
                if (node.type === "ConditionalExpression" &&
                    (node.consequent.type === "AssignmentExpression" || node.alternate.type === "AssignmentExpression")) {
                    return true;
                }
                if ((node.left && node.left.type === "AssignmentExpression") ||
                    (node.right && node.right.type === "AssignmentExpression")) {
                    return true;
                }
                return false;
            }
            /**
             * Determines if a node is contained by or is itself a return statement and is allowed to have a parenthesised assignment
             * @param {ASTNode} node The node to be checked.
             * @returns {boolean} True if the assignment can be parenthesised.
             * @private
             */
            function isReturnAssignException(node) {
                if (!EXCEPT_RETURN_ASSIGN || !isInReturnStatement(node)) {
                    return false;
                }
                if (node.type === "ReturnStatement") {
                    return node.argument && containsAssignment(node.argument);
                }
                if (node.type === "ArrowFunctionExpression" && node.body.type !== "BlockStatement") {
                    return containsAssignment(node.body);
                }
                return containsAssignment(node);
            }
            /**
             * Determines if a node following a [no LineTerminator here] restriction is
             * surrounded by (potentially) invalid extra parentheses.
             * @param {Token} token The token preceding the [no LineTerminator here] restriction.
             * @param {ASTNode} node The node to be checked.
             * @returns {boolean} True if the node is incorrectly parenthesised.
             * @private
             */
            function hasExcessParensNoLineTerminator(token, node) {
                if (token.loc.end.line === node.loc.start.line) {
                    return hasExcessParens(node);
                }
                return hasDoubleExcessParens(node);
            }
            /**
             * Determines whether a node should be preceded by an additional space when removing parens
             * @param {ASTNode} node node to evaluate; must be surrounded by parentheses
             * @returns {boolean} `true` if a space should be inserted before the node
             * @private
             */
            function requiresLeadingSpace(node) {
                const leftParenToken = sourceCode.getTokenBefore(node);
                const tokenBeforeLeftParen = sourceCode.getTokenBefore(node, 1);
                const firstToken = sourceCode.getFirstToken(node);
                return tokenBeforeLeftParen &&
                    tokenBeforeLeftParen.range[1] === leftParenToken.range[0] &&
                    leftParenToken.range[1] === firstToken.range[0] &&
                    !astUtils.canTokensBeAdjacent(tokenBeforeLeftParen, firstToken);
            }
            /**
             * Determines whether a node should be followed by an additional space when removing parens
             * @param {ASTNode} node node to evaluate; must be surrounded by parentheses
             * @returns {boolean} `true` if a space should be inserted after the node
             * @private
             */
            function requiresTrailingSpace(node) {
                const nextTwoTokens = sourceCode.getTokensAfter(node, { count: 2 });
                const rightParenToken = nextTwoTokens[0];
                const tokenAfterRightParen = nextTwoTokens[1];
                const tokenBeforeRightParen = sourceCode.getLastToken(node);
                return rightParenToken && tokenAfterRightParen &&
                    !sourceCode.isSpaceBetweenTokens(rightParenToken, tokenAfterRightParen) &&
                    !astUtils.canTokensBeAdjacent(tokenBeforeRightParen, tokenAfterRightParen);
            }
            /**
             * Determines if a given expression node is an IIFE
             * @param {ASTNode} node The node to check
             * @returns {boolean} `true` if the given node is an IIFE
             */
            function isIIFE(node) {
                return node.type === "CallExpression" && node.callee.type === "FunctionExpression";
            }
            /**
             * Report the node
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function report(node) {
                const leftParenToken = sourceCode.getTokenBefore(node);
                const rightParenToken = sourceCode.getTokenAfter(node);
                if (!isParenthesisedTwice(node)) {
                    if (tokensToIgnore.has(sourceCode.getFirstToken(node))) {
                        return;
                    }
                    if (isIIFE(node) && !isParenthesised(node.callee)) {
                        return;
                    }
                }
                /**
                 * Finishes reporting
                 * @returns {void}
                 * @private
                 */
                function finishReport() {
                    context.report({
                        node,
                        loc: leftParenToken.loc,
                        messageId: "unexpected",
                        fix(fixer) {
                            const parenthesizedSource = sourceCode.text.slice(leftParenToken.range[1], rightParenToken.range[0]);
                            return fixer.replaceTextRange([
                                leftParenToken.range[0],
                                rightParenToken.range[1]
                            ], (requiresLeadingSpace(node) ? " " : "") + parenthesizedSource + (requiresTrailingSpace(node) ? " " : ""));
                        }
                    });
                }
                if (reportsBuffer) {
                    reportsBuffer.reports.push({ node, finishReport });
                    return;
                }
                finishReport();
            }
            /**
             * Evaluate Unary update
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkUnaryUpdate(node) {
                if (node.type === "UnaryExpression" && node.argument.type === "BinaryExpression" && node.argument.operator === "**") {
                    return;
                }
                if (hasExcessParens(node.argument) && precedence(node.argument) >= precedence(node)) {
                    report(node.argument);
                }
            }
            /**
             * Check if a member expression contains a call expression
             * @param {ASTNode} node MemberExpression node to evaluate
             * @returns {boolean} true if found, false if not
             */
            function doesMemberExpressionContainCallExpression(node) {
                let currentNode = node.object;
                let currentNodeType = node.object.type;
                while (currentNodeType === "MemberExpression") {
                    currentNode = currentNode.object;
                    currentNodeType = currentNode.type;
                }
                return currentNodeType === "CallExpression";
            }
            /**
             * Evaluate a new call
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkCallNew(node) {
                const callee = node.callee;
                if (hasExcessParens(callee) && precedence(callee) >= precedence(node)) {
                    const hasNewParensException = callee.type === "NewExpression" && !isNewExpressionWithParens(callee);
                    if (hasDoubleExcessParens(callee) ||
                        !isIIFE(node) && !hasNewParensException && !(
                        /*
                         * Allow extra parens around a new expression if
                         * there are intervening parentheses.
                         */
                        (callee.type === "MemberExpression" && doesMemberExpressionContainCallExpression(callee)))) {
                        report(node.callee);
                    }
                }
                node.arguments
                    .filter(arg => hasExcessParens(arg) && precedence(arg) >= PRECEDENCE_OF_ASSIGNMENT_EXPR)
                    .forEach(report);
            }
            /**
             * Evaluate binary logicals
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkBinaryLogical(node) {
                const prec = precedence(node);
                const leftPrecedence = precedence(node.left);
                const rightPrecedence = precedence(node.right);
                const isExponentiation = node.operator === "**";
                const shouldSkipLeft = (NESTED_BINARY && (node.left.type === "BinaryExpression" || node.left.type === "LogicalExpression")) ||
                    node.left.type === "UnaryExpression" && isExponentiation;
                const shouldSkipRight = NESTED_BINARY && (node.right.type === "BinaryExpression" || node.right.type === "LogicalExpression");
                if (!shouldSkipLeft && hasExcessParens(node.left) && (leftPrecedence > prec || (leftPrecedence === prec && !isExponentiation))) {
                    report(node.left);
                }
                if (!shouldSkipRight && hasExcessParens(node.right) && (rightPrecedence > prec || (rightPrecedence === prec && isExponentiation))) {
                    report(node.right);
                }
            }
            /**
             * Check the parentheses around the super class of the given class definition.
             * @param {ASTNode} node The node of class declarations to check.
             * @returns {void}
             */
            function checkClass(node) {
                if (!node.superClass) {
                    return;
                }
                /*
                 * If `node.superClass` is a LeftHandSideExpression, parentheses are extra.
                 * Otherwise, parentheses are needed.
                 */
                const hasExtraParens = precedence(node.superClass) > PRECEDENCE_OF_UPDATE_EXPR
                    ? hasExcessParens(node.superClass)
                    : hasDoubleExcessParens(node.superClass);
                if (hasExtraParens) {
                    report(node.superClass);
                }
            }
            /**
             * Check the parentheses around the argument of the given spread operator.
             * @param {ASTNode} node The node of spread elements/properties to check.
             * @returns {void}
             */
            function checkSpreadOperator(node) {
                const hasExtraParens = precedence(node.argument) >= PRECEDENCE_OF_ASSIGNMENT_EXPR
                    ? hasExcessParens(node.argument)
                    : hasDoubleExcessParens(node.argument);
                if (hasExtraParens) {
                    report(node.argument);
                }
            }
            /**
             * Checks the parentheses for an ExpressionStatement or ExportDefaultDeclaration
             * @param {ASTNode} node The ExpressionStatement.expression or ExportDefaultDeclaration.declaration node
             * @returns {void}
             */
            function checkExpressionOrExportStatement(node) {
                const firstToken = isParenthesised(node) ? sourceCode.getTokenBefore(node) : sourceCode.getFirstToken(node);
                const secondToken = sourceCode.getTokenAfter(firstToken, astUtils.isNotOpeningParenToken);
                const thirdToken = secondToken ? sourceCode.getTokenAfter(secondToken) : null;
                const tokenAfterClosingParens = secondToken ? sourceCode.getTokenAfter(secondToken, astUtils.isNotClosingParenToken) : null;
                if (astUtils.isOpeningParenToken(firstToken) &&
                    (astUtils.isOpeningBraceToken(secondToken) ||
                        secondToken.type === "Keyword" && (secondToken.value === "function" ||
                            secondToken.value === "class" ||
                            secondToken.value === "let" &&
                                tokenAfterClosingParens &&
                                (astUtils.isOpeningBracketToken(tokenAfterClosingParens) ||
                                    tokenAfterClosingParens.type === "Identifier")) ||
                        secondToken && secondToken.type === "Identifier" && secondToken.value === "async" && thirdToken && thirdToken.type === "Keyword" && thirdToken.value === "function")) {
                    tokensToIgnore.add(secondToken);
                }
                if (hasExcessParens(node)) {
                    report(node);
                }
            }
            /**
             * Finds the path from the given node to the specified ancestor.
             * @param {ASTNode} node First node in the path.
             * @param {ASTNode} ancestor Last node in the path.
             * @returns {ASTNode[]} Path, including both nodes.
             * @throws {Error} If the given node does not have the specified ancestor.
             */
            function pathToAncestor(node, ancestor) {
                const path = [node];
                let currentNode = node;
                while (currentNode !== ancestor) {
                    currentNode = currentNode.parent;
                    /* istanbul ignore if */
                    if (currentNode === null) {
                        throw new Error("Nodes are not in the ancestor-descendant relationship.");
                    }
                    path.push(currentNode);
                }
                return path;
            }
            /**
             * Finds the path from the given node to the specified descendant.
             * @param {ASTNode} node First node in the path.
             * @param {ASTNode} descendant Last node in the path.
             * @returns {ASTNode[]} Path, including both nodes.
             * @throws {Error} If the given node does not have the specified descendant.
             */
            function pathToDescendant(node, descendant) {
                return pathToAncestor(descendant, node).reverse();
            }
            /**
             * Checks whether the syntax of the given ancestor of an 'in' expression inside a for-loop initializer
             * is preventing the 'in' keyword from being interpreted as a part of an ill-formed for-in loop.
             * @param {ASTNode} node Ancestor of an 'in' expression.
             * @param {ASTNode} child Child of the node, ancestor of the same 'in' expression or the 'in' expression itself.
             * @returns {boolean} True if the keyword 'in' would be interpreted as the 'in' operator, without any parenthesis.
             */
            function isSafelyEnclosingInExpression(node, child) {
                switch (node.type) {
                    case "ArrayExpression":
                    case "ArrayPattern":
                    case "BlockStatement":
                    case "ObjectExpression":
                    case "ObjectPattern":
                    case "TemplateLiteral":
                        return true;
                    case "ArrowFunctionExpression":
                    case "FunctionExpression":
                        return node.params.includes(child);
                    case "CallExpression":
                    case "NewExpression":
                        return node.arguments.includes(child);
                    case "MemberExpression":
                        return node.computed && node.property === child;
                    case "ConditionalExpression":
                        return node.consequent === child;
                    default:
                        return false;
                }
            }
            /**
             * Starts a new reports buffering. Warnings will be stored in a buffer instead of being reported immediately.
             * An additional logic that requires multiple nodes (e.g. a whole subtree) may dismiss some of the stored warnings.
             * @returns {void}
             */
            function startNewReportsBuffering() {
                reportsBuffer = {
                    upper: reportsBuffer,
                    inExpressionNodes: [],
                    reports: []
                };
            }
            /**
             * Ends the current reports buffering.
             * @returns {void}
             */
            function endCurrentReportsBuffering() {
                const { upper, inExpressionNodes, reports } = reportsBuffer;
                if (upper) {
                    upper.inExpressionNodes.push(...inExpressionNodes);
                    upper.reports.push(...reports);
                }
                else {
                    // flush remaining reports
                    reports.forEach(({ finishReport }) => finishReport());
                }
                reportsBuffer = upper;
            }
            /**
             * Checks whether the given node is in the current reports buffer.
             * @param {ASTNode} node Node to check.
             * @returns {boolean} True if the node is in the current buffer, false otherwise.
             */
            function isInCurrentReportsBuffer(node) {
                return reportsBuffer.reports.some(r => r.node === node);
            }
            /**
             * Removes the given node from the current reports buffer.
             * @param {ASTNode} node Node to remove.
             * @returns {void}
             */
            function removeFromCurrentReportsBuffer(node) {
                reportsBuffer.reports = reportsBuffer.reports.filter(r => r.node !== node);
            }
            return {
                ArrayExpression(node) {
                    node.elements
                        .filter(e => e && hasExcessParens(e) && precedence(e) >= PRECEDENCE_OF_ASSIGNMENT_EXPR)
                        .forEach(report);
                },
                ArrowFunctionExpression(node) {
                    if (isReturnAssignException(node)) {
                        return;
                    }
                    if (node.body.type === "ConditionalExpression" &&
                        IGNORE_ARROW_CONDITIONALS &&
                        !isParenthesisedTwice(node.body)) {
                        return;
                    }
                    if (node.body.type !== "BlockStatement") {
                        const firstBodyToken = sourceCode.getFirstToken(node.body, astUtils.isNotOpeningParenToken);
                        const tokenBeforeFirst = sourceCode.getTokenBefore(firstBodyToken);
                        if (astUtils.isOpeningParenToken(tokenBeforeFirst) && astUtils.isOpeningBraceToken(firstBodyToken)) {
                            tokensToIgnore.add(firstBodyToken);
                        }
                        if (hasExcessParens(node.body) && precedence(node.body) >= PRECEDENCE_OF_ASSIGNMENT_EXPR) {
                            report(node.body);
                        }
                    }
                },
                AssignmentExpression(node) {
                    if (isReturnAssignException(node)) {
                        return;
                    }
                    if (hasExcessParens(node.right) && precedence(node.right) >= precedence(node)) {
                        report(node.right);
                    }
                },
                BinaryExpression(node) {
                    if (reportsBuffer && node.operator === "in") {
                        reportsBuffer.inExpressionNodes.push(node);
                    }
                    checkBinaryLogical(node);
                },
                CallExpression: checkCallNew,
                ClassBody(node) {
                    node.body
                        .filter(member => member.type === "MethodDefinition" && member.computed &&
                        member.key && hasExcessParens(member.key) && precedence(member.key) >= PRECEDENCE_OF_ASSIGNMENT_EXPR)
                        .forEach(member => report(member.key));
                },
                ConditionalExpression(node) {
                    if (isReturnAssignException(node)) {
                        return;
                    }
                    if (hasExcessParens(node.test) && precedence(node.test) >= precedence({ type: "LogicalExpression", operator: "||" })) {
                        report(node.test);
                    }
                    if (hasExcessParens(node.consequent) && precedence(node.consequent) >= PRECEDENCE_OF_ASSIGNMENT_EXPR) {
                        report(node.consequent);
                    }
                    if (hasExcessParens(node.alternate) && precedence(node.alternate) >= PRECEDENCE_OF_ASSIGNMENT_EXPR) {
                        report(node.alternate);
                    }
                },
                DoWhileStatement(node) {
                    if (hasExcessParens(node.test) && !isCondAssignException(node)) {
                        report(node.test);
                    }
                },
                ExportDefaultDeclaration: node => checkExpressionOrExportStatement(node.declaration),
                ExpressionStatement: node => checkExpressionOrExportStatement(node.expression),
                "ForInStatement, ForOfStatement"(node) {
                    if (node.left.type !== "VariableDeclarator") {
                        const firstLeftToken = sourceCode.getFirstToken(node.left, astUtils.isNotOpeningParenToken);
                        if (firstLeftToken.value === "let" && (
                        /*
                         * If `let` is the only thing on the left side of the loop, it's the loop variable: `for ((let) of foo);`
                         * Removing it will cause a syntax error, because it will be parsed as the start of a VariableDeclarator.
                         */
                        (firstLeftToken.range[1] === node.left.range[1] || /*
                         * If `let` is followed by a `[` token, it's a property access on the `let` value: `for ((let[foo]) of bar);`
                         * Removing it will cause the property access to be parsed as a destructuring declaration of `foo` instead.
                         */
                            astUtils.isOpeningBracketToken(sourceCode.getTokenAfter(firstLeftToken, astUtils.isNotClosingParenToken))))) {
                            tokensToIgnore.add(firstLeftToken);
                        }
                    }
                    if (!(node.type === "ForOfStatement" && node.right.type === "SequenceExpression") && hasExcessParens(node.right)) {
                        report(node.right);
                    }
                    if (hasExcessParens(node.left)) {
                        report(node.left);
                    }
                },
                ForStatement(node) {
                    if (node.test && hasExcessParens(node.test) && !isCondAssignException(node)) {
                        report(node.test);
                    }
                    if (node.update && hasExcessParens(node.update)) {
                        report(node.update);
                    }
                    if (node.init) {
                        startNewReportsBuffering();
                        if (hasExcessParens(node.init)) {
                            report(node.init);
                        }
                    }
                },
                "ForStatement > *.init:exit"(node) {
                    /*
                     * Removing parentheses around `in` expressions might change semantics and cause errors.
                     *
                     * For example, this valid for loop:
                     *      for (let a = (b in c); ;);
                     * after removing parentheses would be treated as an invalid for-in loop:
                     *      for (let a = b in c; ;);
                     */
                    if (reportsBuffer.reports.length) {
                        reportsBuffer.inExpressionNodes.forEach(inExpressionNode => {
                            const path = pathToDescendant(node, inExpressionNode);
                            let nodeToExclude;
                            for (let i = 0; i < path.length; i++) {
                                const pathNode = path[i];
                                if (i < path.length - 1) {
                                    const nextPathNode = path[i + 1];
                                    if (isSafelyEnclosingInExpression(pathNode, nextPathNode)) {
                                        // The 'in' expression in safely enclosed by the syntax of its ancestor nodes (e.g. by '{}' or '[]').
                                        return;
                                    }
                                }
                                if (isParenthesised(pathNode)) {
                                    if (isInCurrentReportsBuffer(pathNode)) {
                                        // This node was supposed to be reported, but parentheses might be necessary.
                                        if (isParenthesisedTwice(pathNode)) {
                                            /*
                                             * This node is parenthesised twice, it certainly has at least one pair of `extra` parentheses.
                                             * If the --fix option is on, the current fixing iteration will remove only one pair of parentheses.
                                             * The remaining pair is safely enclosing the 'in' expression.
                                             */
                                            return;
                                        }
                                        // Exclude the outermost node only.
                                        if (!nodeToExclude) {
                                            nodeToExclude = pathNode;
                                        }
                                        // Don't break the loop here, there might be some safe nodes or parentheses that will stay inside.
                                    }
                                    else {
                                        // This node will stay parenthesised, the 'in' expression in safely enclosed by '()'.
                                        return;
                                    }
                                }
                            }
                            // Exclude the node from the list (i.e. treat parentheses as necessary)
                            removeFromCurrentReportsBuffer(nodeToExclude);
                        });
                    }
                    endCurrentReportsBuffering();
                },
                IfStatement(node) {
                    if (hasExcessParens(node.test) && !isCondAssignException(node)) {
                        report(node.test);
                    }
                },
                ImportExpression(node) {
                    const { source } = node;
                    if (source.type === "SequenceExpression") {
                        if (hasDoubleExcessParens(source)) {
                            report(source);
                        }
                    }
                    else if (hasExcessParens(source)) {
                        report(source);
                    }
                },
                LogicalExpression: checkBinaryLogical,
                MemberExpression(node) {
                    const nodeObjHasExcessParens = hasExcessParens(node.object);
                    if (nodeObjHasExcessParens &&
                        precedence(node.object) >= precedence(node) &&
                        (node.computed ||
                            !(astUtils.isDecimalInteger(node.object) ||
                                // RegExp literal is allowed to have parens (#1589)
                                (node.object.type === "Literal" && node.object.regex)))) {
                        report(node.object);
                    }
                    if (nodeObjHasExcessParens &&
                        node.object.type === "CallExpression" &&
                        node.parent.type !== "NewExpression") {
                        report(node.object);
                    }
                    if (nodeObjHasExcessParens &&
                        !IGNORE_NEW_IN_MEMBER_EXPR &&
                        node.object.type === "NewExpression" &&
                        isNewExpressionWithParens(node.object)) {
                        report(node.object);
                    }
                    if (node.computed && hasExcessParens(node.property)) {
                        report(node.property);
                    }
                },
                NewExpression: checkCallNew,
                ObjectExpression(node) {
                    node.properties
                        .filter(property => {
                        const value = property.value;
                        return value && hasExcessParens(value) && precedence(value) >= PRECEDENCE_OF_ASSIGNMENT_EXPR;
                    }).forEach(property => report(property.value));
                },
                Property(node) {
                    if (node.computed) {
                        const { key } = node;
                        if (key && hasExcessParens(key) && precedence(key) >= PRECEDENCE_OF_ASSIGNMENT_EXPR) {
                            report(key);
                        }
                    }
                },
                ReturnStatement(node) {
                    const returnToken = sourceCode.getFirstToken(node);
                    if (isReturnAssignException(node)) {
                        return;
                    }
                    if (node.argument &&
                        hasExcessParensNoLineTerminator(returnToken, node.argument) &&
                        // RegExp literal is allowed to have parens (#1589)
                        !(node.argument.type === "Literal" && node.argument.regex)) {
                        report(node.argument);
                    }
                },
                SequenceExpression(node) {
                    node.expressions
                        .filter(e => hasExcessParens(e) && precedence(e) >= precedence(node))
                        .forEach(report);
                },
                SwitchCase(node) {
                    if (node.test && hasExcessParens(node.test)) {
                        report(node.test);
                    }
                },
                SwitchStatement(node) {
                    if (hasExcessParens(node.discriminant)) {
                        report(node.discriminant);
                    }
                },
                ThrowStatement(node) {
                    const throwToken = sourceCode.getFirstToken(node);
                    if (hasExcessParensNoLineTerminator(throwToken, node.argument)) {
                        report(node.argument);
                    }
                },
                UnaryExpression: checkUnaryUpdate,
                UpdateExpression: checkUnaryUpdate,
                AwaitExpression: checkUnaryUpdate,
                VariableDeclarator(node) {
                    if (node.init && hasExcessParens(node.init) &&
                        precedence(node.init) >= PRECEDENCE_OF_ASSIGNMENT_EXPR &&
                        // RegExp literal is allowed to have parens (#1589)
                        !(node.init.type === "Literal" && node.init.regex)) {
                        report(node.init);
                    }
                },
                WhileStatement(node) {
                    if (hasExcessParens(node.test) && !isCondAssignException(node)) {
                        report(node.test);
                    }
                },
                WithStatement(node) {
                    if (hasExcessParens(node.object)) {
                        report(node.object);
                    }
                },
                YieldExpression(node) {
                    if (node.argument) {
                        const yieldToken = sourceCode.getFirstToken(node);
                        if ((precedence(node.argument) >= precedence(node) &&
                            hasExcessParensNoLineTerminator(yieldToken, node.argument)) ||
                            hasDoubleExcessParens(node.argument)) {
                            report(node.argument);
                        }
                    }
                },
                ClassDeclaration: checkClass,
                ClassExpression: checkClass,
                SpreadElement: checkSpreadOperator,
                SpreadProperty: checkSpreadOperator,
                ExperimentalSpreadProperty: checkSpreadOperator,
                TemplateLiteral(node) {
                    node.expressions
                        .filter(e => e && hasExcessParens(e))
                        .forEach(report);
                },
                AssignmentPattern(node) {
                    const { right } = node;
                    if (right && hasExcessParens(right) && precedence(right) >= PRECEDENCE_OF_ASSIGNMENT_EXPR) {
                        report(right);
                    }
                }
            };
        }
    };
},
/* eslint-utils/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, '__esModule', { value: true });
    function _interopDefault(ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }
    var evk = _interopDefault(require(100) /* eslint-visitor-keys */);
    /**
     * Get the innermost scope which contains a given location.
     * @param {Scope} initialScope The initial scope to search.
     * @param {Node} node The location to search.
     * @returns {Scope} The innermost scope.
     */
    function getInnermostScope(initialScope, node) {
        const location = node.range[0];
        let scope = initialScope;
        let found = false;
        do {
            found = false;
            for (const childScope of scope.childScopes) {
                const range = childScope.block.range;
                if (range[0] <= location && location < range[1]) {
                    scope = childScope;
                    found = true;
                    break;
                }
            }
        } while (found);
        return scope;
    }
    /**
     * Find the variable of a given name.
     * @param {Scope} initialScope The scope to start finding.
     * @param {string|Node} nameOrNode The variable name to find. If this is a Node object then it should be an Identifier node.
     * @returns {Variable|null} The found variable or null.
     */
    function findVariable(initialScope, nameOrNode) {
        let name = "";
        let scope = initialScope;
        if (typeof nameOrNode === "string") {
            name = nameOrNode;
        }
        else {
            name = nameOrNode.name;
            scope = getInnermostScope(scope, nameOrNode);
        }
        while (scope != null) {
            const variable = scope.set.get(name);
            if (variable != null) {
                return variable;
            }
            scope = scope.upper;
        }
        return null;
    }
    /**
     * Negate the result of `this` calling.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the result of `this(token)` is `false`.
     */
    function negate0(token) {
        return !this(token); //eslint-disable-line no-invalid-this
    }
    /**
     * Creates the negate function of the given function.
     * @param {function(Token):boolean} f - The function to negate.
     * @returns {function(Token):boolean} Negated function.
     */
    function negate(f) {
        return negate0.bind(f);
    }
    /**
     * Checks if the given token is an arrow token or not.
     * @param {Token} token - The token to check.
     * @returns {boolean} `true` if the token is an arrow token.
     */
    function isArrowToken(token) {
        return token.value === "=>" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a comma token or not.
     * @param {Token} token - The token to check.
     * @returns {boolean} `true` if the token is a comma token.
     */
    function isCommaToken(token) {
        return token.value === "," && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a semicolon token or not.
     * @param {Token} token - The token to check.
     * @returns {boolean} `true` if the token is a semicolon token.
     */
    function isSemicolonToken(token) {
        return token.value === ";" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a colon token or not.
     * @param {Token} token - The token to check.
     * @returns {boolean} `true` if the token is a colon token.
     */
    function isColonToken(token) {
        return token.value === ":" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is an opening parenthesis token or not.
     * @param {Token} token - The token to check.
     * @returns {boolean} `true` if the token is an opening parenthesis token.
     */
    function isOpeningParenToken(token) {
        return token.value === "(" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a closing parenthesis token or not.
     * @param {Token} token - The token to check.
     * @returns {boolean} `true` if the token is a closing parenthesis token.
     */
    function isClosingParenToken(token) {
        return token.value === ")" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is an opening square bracket token or not.
     * @param {Token} token - The token to check.
     * @returns {boolean} `true` if the token is an opening square bracket token.
     */
    function isOpeningBracketToken(token) {
        return token.value === "[" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a closing square bracket token or not.
     * @param {Token} token - The token to check.
     * @returns {boolean} `true` if the token is a closing square bracket token.
     */
    function isClosingBracketToken(token) {
        return token.value === "]" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is an opening brace token or not.
     * @param {Token} token - The token to check.
     * @returns {boolean} `true` if the token is an opening brace token.
     */
    function isOpeningBraceToken(token) {
        return token.value === "{" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a closing brace token or not.
     * @param {Token} token - The token to check.
     * @returns {boolean} `true` if the token is a closing brace token.
     */
    function isClosingBraceToken(token) {
        return token.value === "}" && token.type === "Punctuator";
    }
    /**
     * Checks if the given token is a comment token or not.
     * @param {Token} token - The token to check.
     * @returns {boolean} `true` if the token is a comment token.
     */
    function isCommentToken(token) {
        return (token.type === "Line" ||
            token.type === "Block" ||
            token.type === "Shebang");
    }
    const isNotArrowToken = negate(isArrowToken);
    const isNotCommaToken = negate(isCommaToken);
    const isNotSemicolonToken = negate(isSemicolonToken);
    const isNotColonToken = negate(isColonToken);
    const isNotOpeningParenToken = negate(isOpeningParenToken);
    const isNotClosingParenToken = negate(isClosingParenToken);
    const isNotOpeningBracketToken = negate(isOpeningBracketToken);
    const isNotClosingBracketToken = negate(isClosingBracketToken);
    const isNotOpeningBraceToken = negate(isOpeningBraceToken);
    const isNotClosingBraceToken = negate(isClosingBraceToken);
    const isNotCommentToken = negate(isCommentToken);
    /**
     * Get the `(` token of the given function node.
     * @param {Node} node - The function node to get.
     * @param {SourceCode} sourceCode - The source code object to get tokens.
     * @returns {Token} `(` token.
     */
    function getOpeningParenOfParams(node, sourceCode) {
        return node.id
            ? sourceCode.getTokenAfter(node.id, isOpeningParenToken)
            : sourceCode.getFirstToken(node, isOpeningParenToken);
    }
    /**
     * Get the location of the given function node for reporting.
     * @param {Node} node - The function node to get.
     * @param {SourceCode} sourceCode - The source code object to get tokens.
     * @returns {string} The location of the function node for reporting.
     */
    function getFunctionHeadLocation(node, sourceCode) {
        const parent = node.parent;
        let start = null;
        let end = null;
        if (node.type === "ArrowFunctionExpression") {
            const arrowToken = sourceCode.getTokenBefore(node.body, isArrowToken);
            start = arrowToken.loc.start;
            end = arrowToken.loc.end;
        }
        else if (parent.type === "Property" ||
            parent.type === "MethodDefinition") {
            start = parent.loc.start;
            end = getOpeningParenOfParams(node, sourceCode).loc.start;
        }
        else {
            start = node.loc.start;
            end = getOpeningParenOfParams(node, sourceCode).loc.start;
        }
        return {
            start: Object.assign({}, start),
            end: Object.assign({}, end),
        };
    }
    /* globals BigInt, globalThis, global, self, window */
    const globalObject = typeof globalThis !== "undefined"
        ? globalThis
        : typeof self !== "undefined"
            ? self
            : typeof window !== "undefined"
                ? window
                : typeof global !== "undefined"
                    ? global
                    : {};
    const builtinNames = Object.freeze(new Set([
        "Array",
        "ArrayBuffer",
        "BigInt",
        "BigInt64Array",
        "BigUint64Array",
        "Boolean",
        "DataView",
        "Date",
        "decodeURI",
        "decodeURIComponent",
        "encodeURI",
        "encodeURIComponent",
        "escape",
        "Float32Array",
        "Float64Array",
        "Function",
        "Infinity",
        "Int16Array",
        "Int32Array",
        "Int8Array",
        "isFinite",
        "isNaN",
        "isPrototypeOf",
        "JSON",
        "Map",
        "Math",
        "NaN",
        "Number",
        "Object",
        "parseFloat",
        "parseInt",
        "Promise",
        "Proxy",
        "Reflect",
        "RegExp",
        "Set",
        "String",
        "Symbol",
        "Uint16Array",
        "Uint32Array",
        "Uint8Array",
        "Uint8ClampedArray",
        "undefined",
        "unescape",
        "WeakMap",
        "WeakSet",
    ]));
    const callAllowed = new Set([
        Array.isArray,
        typeof BigInt === "function" ? BigInt : undefined,
        Boolean,
        Date,
        Date.parse,
        decodeURI,
        decodeURIComponent,
        encodeURI,
        encodeURIComponent,
        escape,
        isFinite,
        isNaN,
        isPrototypeOf,
        ...Object.getOwnPropertyNames(Math)
            .map(k => Math[k])
            .filter(f => typeof f === "function"),
        Number,
        Number.isFinite,
        Number.isNaN,
        Number.parseFloat,
        Number.parseInt,
        Object,
        Object.entries,
        Object.is,
        Object.isExtensible,
        Object.isFrozen,
        Object.isSealed,
        Object.keys,
        Object.values,
        parseFloat,
        parseInt,
        RegExp,
        String,
        String.fromCharCode,
        String.fromCodePoint,
        String.raw,
        Symbol,
        Symbol.for,
        Symbol.keyFor,
        unescape,
    ].filter(f => typeof f === "function"));
    const callPassThrough = new Set([
        Object.freeze,
        Object.preventExtensions,
        Object.seal,
    ]);
    /**
     * Get the property descriptor.
     * @param {object} object The object to get.
     * @param {string|number|symbol} name The property name to get.
     */
    function getPropertyDescriptor(object, name) {
        let x = object;
        while ((typeof x === "object" || typeof x === "function") && x !== null) {
            const d = Object.getOwnPropertyDescriptor(x, name);
            if (d) {
                return d;
            }
            x = Object.getPrototypeOf(x);
        }
        return null;
    }
    /**
     * Check if a property is getter or not.
     * @param {object} object The object to check.
     * @param {string|number|symbol} name The property name to check.
     */
    function isGetter(object, name) {
        const d = getPropertyDescriptor(object, name);
        return d != null && d.get != null;
    }
    /**
     * Get the element values of a given node list.
     * @param {Node[]} nodeList The node list to get values.
     * @param {Scope|undefined} initialScope The initial scope to find variables.
     * @returns {any[]|null} The value list if all nodes are constant. Otherwise, null.
     */
    function getElementValues(nodeList, initialScope) {
        const valueList = [];
        for (let i = 0; i < nodeList.length; ++i) {
            const elementNode = nodeList[i];
            if (elementNode == null) {
                valueList.length = i + 1;
            }
            else if (elementNode.type === "SpreadElement") {
                const argument = getStaticValueR(elementNode.argument, initialScope);
                if (argument == null) {
                    return null;
                }
                valueList.push(...argument.value);
            }
            else {
                const element = getStaticValueR(elementNode, initialScope);
                if (element == null) {
                    return null;
                }
                valueList.push(element.value);
            }
        }
        return valueList;
    }
    const operations = Object.freeze({
        ArrayExpression(node, initialScope) {
            const elements = getElementValues(node.elements, initialScope);
            return elements != null ? { value: elements } : null;
        },
        AssignmentExpression(node, initialScope) {
            if (node.operator === "=") {
                return getStaticValueR(node.right, initialScope);
            }
            return null;
        },
        //eslint-disable-next-line complexity
        BinaryExpression(node, initialScope) {
            if (node.operator === "in" || node.operator === "instanceof") {
                // Not supported.
                return null;
            }
            const left = getStaticValueR(node.left, initialScope);
            const right = getStaticValueR(node.right, initialScope);
            if (left != null && right != null) {
                switch (node.operator) {
                    case "==":
                        return { value: left.value == right.value }; //eslint-disable-line eqeqeq
                    case "!=":
                        return { value: left.value != right.value }; //eslint-disable-line eqeqeq
                    case "===":
                        return { value: left.value === right.value };
                    case "!==":
                        return { value: left.value !== right.value };
                    case "<":
                        return { value: left.value < right.value };
                    case "<=":
                        return { value: left.value <= right.value };
                    case ">":
                        return { value: left.value > right.value };
                    case ">=":
                        return { value: left.value >= right.value };
                    case "<<":
                        return { value: left.value << right.value };
                    case ">>":
                        return { value: left.value >> right.value };
                    case ">>>":
                        return { value: left.value >>> right.value };
                    case "+":
                        return { value: left.value + right.value };
                    case "-":
                        return { value: left.value - right.value };
                    case "*":
                        return { value: left.value * right.value };
                    case "/":
                        return { value: left.value / right.value };
                    case "%":
                        return { value: left.value % right.value };
                    case "**":
                        return { value: Math.pow(left.value, right.value) };
                    case "|":
                        return { value: left.value | right.value };
                    case "^":
                        return { value: left.value ^ right.value };
                    case "&":
                        return { value: left.value & right.value };
                    // no default
                }
            }
            return null;
        },
        CallExpression(node, initialScope) {
            const calleeNode = node.callee;
            const args = getElementValues(node.arguments, initialScope);
            if (args != null) {
                if (calleeNode.type === "MemberExpression") {
                    const object = getStaticValueR(calleeNode.object, initialScope);
                    const property = calleeNode.computed
                        ? getStaticValueR(calleeNode.property, initialScope)
                        : { value: calleeNode.property.name };
                    if (object != null && property != null) {
                        const receiver = object.value;
                        const methodName = property.value;
                        if (callAllowed.has(receiver[methodName])) {
                            return { value: receiver[methodName](...args) };
                        }
                        if (callPassThrough.has(receiver[methodName])) {
                            return { value: args[0] };
                        }
                    }
                }
                else {
                    const callee = getStaticValueR(calleeNode, initialScope);
                    if (callee != null) {
                        const func = callee.value;
                        if (callAllowed.has(func)) {
                            return { value: func(...args) };
                        }
                        if (callPassThrough.has(func)) {
                            return { value: args[0] };
                        }
                    }
                }
            }
            return null;
        },
        ConditionalExpression(node, initialScope) {
            const test = getStaticValueR(node.test, initialScope);
            if (test != null) {
                return test.value
                    ? getStaticValueR(node.consequent, initialScope)
                    : getStaticValueR(node.alternate, initialScope);
            }
            return null;
        },
        ExpressionStatement(node, initialScope) {
            return getStaticValueR(node.expression, initialScope);
        },
        Identifier(node, initialScope) {
            if (initialScope != null) {
                const variable = findVariable(initialScope, node);
                // Built-in globals.
                if (variable != null &&
                    variable.defs.length === 0 &&
                    builtinNames.has(variable.name) &&
                    variable.name in globalObject) {
                    return { value: globalObject[variable.name] };
                }
                // Constants.
                if (variable != null && variable.defs.length === 1) {
                    const def = variable.defs[0];
                    if (def.parent &&
                        def.parent.kind === "const" &&
                        // TODO(mysticatea): don't support destructuring here.
                        def.node.id.type === "Identifier") {
                        return getStaticValueR(def.node.init, initialScope);
                    }
                }
            }
            return null;
        },
        Literal(node) {
            //istanbul ignore if : this is implementation-specific behavior.
            if ((node.regex != null || node.bigint != null) && node.value == null) {
                // It was a RegExp/BigInt literal, but Node.js didn't support it.
                return null;
            }
            return { value: node.value };
        },
        LogicalExpression(node, initialScope) {
            const left = getStaticValueR(node.left, initialScope);
            if (left != null) {
                if ((node.operator === "||" && Boolean(left.value) === true) ||
                    (node.operator === "&&" && Boolean(left.value) === false)) {
                    return left;
                }
                const right = getStaticValueR(node.right, initialScope);
                if (right != null) {
                    return right;
                }
            }
            return null;
        },
        MemberExpression(node, initialScope) {
            const object = getStaticValueR(node.object, initialScope);
            const property = node.computed
                ? getStaticValueR(node.property, initialScope)
                : { value: node.property.name };
            if (object != null &&
                property != null &&
                !isGetter(object.value, property.value)) {
                return { value: object.value[property.value] };
            }
            return null;
        },
        NewExpression(node, initialScope) {
            const callee = getStaticValueR(node.callee, initialScope);
            const args = getElementValues(node.arguments, initialScope);
            if (callee != null && args != null) {
                const Func = callee.value;
                if (callAllowed.has(Func)) {
                    return { value: new Func(...args) };
                }
            }
            return null;
        },
        ObjectExpression(node, initialScope) {
            const object = {};
            for (const propertyNode of node.properties) {
                if (propertyNode.type === "Property") {
                    if (propertyNode.kind !== "init") {
                        return null;
                    }
                    const key = propertyNode.computed
                        ? getStaticValueR(propertyNode.key, initialScope)
                        : { value: propertyNode.key.name };
                    const value = getStaticValueR(propertyNode.value, initialScope);
                    if (key == null || value == null) {
                        return null;
                    }
                    object[key.value] = value.value;
                }
                else if (propertyNode.type === "SpreadElement" ||
                    propertyNode.type === "ExperimentalSpreadProperty") {
                    const argument = getStaticValueR(propertyNode.argument, initialScope);
                    if (argument == null) {
                        return null;
                    }
                    Object.assign(object, argument.value);
                }
                else {
                    return null;
                }
            }
            return { value: object };
        },
        SequenceExpression(node, initialScope) {
            const last = node.expressions[node.expressions.length - 1];
            return getStaticValueR(last, initialScope);
        },
        TaggedTemplateExpression(node, initialScope) {
            const tag = getStaticValueR(node.tag, initialScope);
            const expressions = getElementValues(node.quasi.expressions, initialScope);
            if (tag != null && expressions != null) {
                const func = tag.value;
                const strings = node.quasi.quasis.map(q => q.value.cooked);
                strings.raw = node.quasi.quasis.map(q => q.value.raw);
                if (func === String.raw) {
                    return { value: func(strings, ...expressions) };
                }
            }
            return null;
        },
        TemplateLiteral(node, initialScope) {
            const expressions = getElementValues(node.expressions, initialScope);
            if (expressions != null) {
                let value = node.quasis[0].value.cooked;
                for (let i = 0; i < expressions.length; ++i) {
                    value += expressions[i];
                    value += node.quasis[i + 1].value.cooked;
                }
                return { value };
            }
            return null;
        },
        UnaryExpression(node, initialScope) {
            if (node.operator === "delete") {
                // Not supported.
                return null;
            }
            if (node.operator === "void") {
                return { value: undefined };
            }
            const arg = getStaticValueR(node.argument, initialScope);
            if (arg != null) {
                switch (node.operator) {
                    case "-":
                        return { value: -arg.value };
                    case "+":
                        return { value: +arg.value }; //eslint-disable-line no-implicit-coercion
                    case "!":
                        return { value: !arg.value };
                    case "~":
                        return { value: ~arg.value };
                    case "typeof":
                        return { value: typeof arg.value };
                    // no default
                }
            }
            return null;
        },
    });
    /**
     * Get the value of a given node if it's a static value.
     * @param {Node} node The node to get.
     * @param {Scope|undefined} initialScope The scope to start finding variable.
     * @returns {{value:any}|null} The static value of the node, or `null`.
     */
    function getStaticValueR(node, initialScope) {
        if (node != null && Object.hasOwnProperty.call(operations, node.type)) {
            return operations[node.type](node, initialScope);
        }
        return null;
    }
    /**
     * Get the value of a given node if it's a static value.
     * @param {Node} node The node to get.
     * @param {Scope} [initialScope] The scope to start finding variable. Optional. If this scope was given, this tries to resolve identifier references which are in the given node as much as possible.
     * @returns {{value:any}|null} The static value of the node, or `null`.
     */
    function getStaticValue(node, initialScope = null) {
        try {
            return getStaticValueR(node, initialScope);
        }
        catch (_error) {
            return null;
        }
    }
    /**
     * Get the value of a given node if it's a literal or a template literal.
     * @param {Node} node The node to get.
     * @param {Scope} [initialScope] The scope to start finding variable. Optional. If the node is an Identifier node and this scope was given, this checks the variable of the identifier, and returns the value of it if the variable is a constant.
     * @returns {string|null} The value of the node, or `null`.
     */
    function getStringIfConstant(node, initialScope = null) {
        // Handle the literals that the platform doesn't support natively.
        if (node && node.type === "Literal" && node.value === null) {
            if (node.regex) {
                return `/${node.regex.pattern}/${node.regex.flags}`;
            }
            if (node.bigint) {
                return node.bigint;
            }
        }
        const evaluated = getStaticValue(node, initialScope);
        return evaluated && String(evaluated.value);
    }
    /**
     * Get the property name from a MemberExpression node or a Property node.
     * @param {Node} node The node to get.
     * @param {Scope} [initialScope] The scope to start finding variable. Optional. If the node is a computed property node and this scope was given, this checks the computed property name by the `getStringIfConstant` function with the scope, and returns the value of it.
     * @returns {string|null} The property name of the node.
     */
    function getPropertyName(node, initialScope) {
        switch (node.type) {
            case "MemberExpression":
                if (node.computed) {
                    return getStringIfConstant(node.property, initialScope);
                }
                return node.property.name;
            case "Property":
            case "MethodDefinition":
                if (node.computed) {
                    return getStringIfConstant(node.key, initialScope);
                }
                if (node.key.type === "Literal") {
                    return String(node.key.value);
                }
                return node.key.name;
            // no default
        }
        return null;
    }
    /**
     * Get the name and kind of the given function node.
     * @param {ASTNode} node - The function node to get.
     * @returns {string} The name and kind of the function node.
     */
    function getFunctionNameWithKind(node) {
        const parent = node.parent;
        const tokens = [];
        if (parent.type === "MethodDefinition" && parent.static) {
            tokens.push("static");
        }
        if (node.async) {
            tokens.push("async");
        }
        if (node.generator) {
            tokens.push("generator");
        }
        if (node.type === "ArrowFunctionExpression") {
            tokens.push("arrow", "function");
        }
        else if (parent.type === "Property" ||
            parent.type === "MethodDefinition") {
            if (parent.kind === "constructor") {
                return "constructor";
            }
            if (parent.kind === "get") {
                tokens.push("getter");
            }
            else if (parent.kind === "set") {
                tokens.push("setter");
            }
            else {
                tokens.push("method");
            }
        }
        else {
            tokens.push("function");
        }
        if (node.id) {
            tokens.push(`'${node.id.name}'`);
        }
        else {
            const name = getPropertyName(parent);
            if (name) {
                tokens.push(`'${name}'`);
            }
        }
        return tokens.join(" ");
    }
    const typeConversionBinaryOps = Object.freeze(new Set([
        "==",
        "!=",
        "<",
        "<=",
        ">",
        ">=",
        "<<",
        ">>",
        ">>>",
        "+",
        "-",
        "*",
        "/",
        "%",
        "|",
        "^",
        "&",
        "in",
    ]));
    const typeConversionUnaryOps = Object.freeze(new Set(["-", "+", "!", "~"]));
    const visitor = Object.freeze(Object.assign(Object.create(null), {
        $visit(node, options, visitorKeys) {
            const { type } = node;
            if (typeof this[type] === "function") {
                return this[type](node, options, visitorKeys);
            }
            return this.$visitChildren(node, options, visitorKeys);
        },
        $visitChildren(node, options, visitorKeys) {
            const { type } = node;
            for (const key of visitorKeys[type] || evk.getKeys(node)) {
                const value = node[key];
                if (Array.isArray(value)) {
                    for (const element of value) {
                        if (element &&
                            this.$visit(element, options, visitorKeys)) {
                            return true;
                        }
                    }
                }
                else if (value && this.$visit(value, options, visitorKeys)) {
                    return true;
                }
            }
            return false;
        },
        ArrowFunctionExpression() {
            return false;
        },
        AssignmentExpression() {
            return true;
        },
        AwaitExpression() {
            return true;
        },
        BinaryExpression(node, options, visitorKeys) {
            if (options.considerImplicitTypeConversion &&
                typeConversionBinaryOps.has(node.operator) &&
                (node.left.type !== "Literal" || node.right.type !== "Literal")) {
                return true;
            }
            return this.$visitChildren(node, options, visitorKeys);
        },
        CallExpression() {
            return true;
        },
        FunctionExpression() {
            return false;
        },
        ImportExpression() {
            return true;
        },
        MemberExpression(node, options, visitorKeys) {
            if (options.considerGetters) {
                return true;
            }
            if (options.considerImplicitTypeConversion &&
                node.computed &&
                node.property.type !== "Literal") {
                return true;
            }
            return this.$visitChildren(node, options, visitorKeys);
        },
        MethodDefinition(node, options, visitorKeys) {
            if (options.considerImplicitTypeConversion &&
                node.computed &&
                node.key.type !== "Literal") {
                return true;
            }
            return this.$visitChildren(node, options, visitorKeys);
        },
        NewExpression() {
            return true;
        },
        Property(node, options, visitorKeys) {
            if (options.considerImplicitTypeConversion &&
                node.computed &&
                node.key.type !== "Literal") {
                return true;
            }
            return this.$visitChildren(node, options, visitorKeys);
        },
        UnaryExpression(node, options, visitorKeys) {
            if (node.operator === "delete") {
                return true;
            }
            if (options.considerImplicitTypeConversion &&
                typeConversionUnaryOps.has(node.operator) &&
                node.argument.type !== "Literal") {
                return true;
            }
            return this.$visitChildren(node, options, visitorKeys);
        },
        UpdateExpression() {
            return true;
        },
        YieldExpression() {
            return true;
        },
    }));
    /**
     * Check whether a given node has any side effect or not.
     * @param {Node} node The node to get.
     * @param {SourceCode} sourceCode The source code object.
     * @param {object} [options] The option object.
     * @param {boolean} [options.considerGetters=false] If `true` then it considers member accesses as the node which has side effects.
     * @param {boolean} [options.considerImplicitTypeConversion=false] If `true` then it considers implicit type conversion as the node which has side effects.
     * @param {object} [options.visitorKeys=evk.KEYS] The keys to traverse nodes. Use `context.getSourceCode().visitorKeys`.
     * @returns {boolean} `true` if the node has a certain side effect.
     */
    function hasSideEffect(node, sourceCode, { considerGetters = false, considerImplicitTypeConversion = false } = {}) {
        return visitor.$visit(node, { considerGetters, considerImplicitTypeConversion }, sourceCode.visitorKeys || evk.KEYS);
    }
    /**
     * Get the left parenthesis of the parent node syntax if it exists.
     * E.g., `if (a) {}` then the `(`.
     * @param {Node} node The AST node to check.
     * @param {SourceCode} sourceCode The source code object to get tokens.
     * @returns {Token|null} The left parenthesis of the parent node syntax
     */
    function getParentSyntaxParen(node, sourceCode) {
        const parent = node.parent;
        switch (parent.type) {
            case "CallExpression":
            case "NewExpression":
                if (parent.arguments.length === 1 && parent.arguments[0] === node) {
                    return sourceCode.getTokenAfter(parent.callee, isOpeningParenToken);
                }
                return null;
            case "DoWhileStatement":
                if (parent.test === node) {
                    return sourceCode.getTokenAfter(parent.body, isOpeningParenToken);
                }
                return null;
            case "IfStatement":
            case "WhileStatement":
                if (parent.test === node) {
                    return sourceCode.getFirstToken(parent, 1);
                }
                return null;
            case "ImportExpression":
                if (parent.source === node) {
                    return sourceCode.getFirstToken(parent, 1);
                }
                return null;
            case "SwitchStatement":
                if (parent.discriminant === node) {
                    return sourceCode.getFirstToken(parent, 1);
                }
                return null;
            case "WithStatement":
                if (parent.object === node) {
                    return sourceCode.getFirstToken(parent, 1);
                }
                return null;
            default:
                return null;
        }
    }
    /**
     * Check whether a given node is parenthesized or not.
     * @param {number} times The number of parantheses.
     * @param {Node} node The AST node to check.
     * @param {SourceCode} sourceCode The source code object to get tokens.
     * @returns {boolean} `true` if the node is parenthesized the given times.
     */
    /**
     * Check whether a given node is parenthesized or not.
     * @param {Node} node The AST node to check.
     * @param {SourceCode} sourceCode The source code object to get tokens.
     * @returns {boolean} `true` if the node is parenthesized.
     */
    function isParenthesized(timesOrNode, nodeOrSourceCode, optionalSourceCode) {
        let times, node, sourceCode, maybeLeftParen, maybeRightParen;
        if (typeof timesOrNode === "number") {
            times = timesOrNode | 0;
            node = nodeOrSourceCode;
            sourceCode = optionalSourceCode;
            if (!(times >= 1)) {
                throw new TypeError("'times' should be a positive integer.");
            }
        }
        else {
            times = 1;
            node = timesOrNode;
            sourceCode = nodeOrSourceCode;
        }
        if (node == null) {
            return false;
        }
        maybeLeftParen = maybeRightParen = node;
        do {
            maybeLeftParen = sourceCode.getTokenBefore(maybeLeftParen);
            maybeRightParen = sourceCode.getTokenAfter(maybeRightParen);
        } while (maybeLeftParen != null &&
            maybeRightParen != null &&
            isOpeningParenToken(maybeLeftParen) &&
            isClosingParenToken(maybeRightParen) &&
            // Avoid false positive such as `if (a) {}`
            maybeLeftParen !== getParentSyntaxParen(node, sourceCode) &&
            --times > 0);
        return times === 0;
    }
    /**
     * @author Toru Nagashima <https://github.com/mysticatea>
     * See LICENSE file in root directory for full license.
     */
    const placeholder = /\$(?:[$&`']|[1-9][0-9]?)/gu;
    /** @type {WeakMap<PatternMatcher, {pattern:RegExp,escaped:boolean}>} */
    const internal = new WeakMap();
    /**
     * Check whether a given character is escaped or not.
     * @param {string} str The string to check.
     * @param {number} index The location of the character to check.
     * @returns {boolean} `true` if the character is escaped.
     */
    function isEscaped(str, index) {
        let escaped = false;
        for (let i = index - 1; i >= 0 && str.charCodeAt(i) === 0x5c; --i) {
            escaped = !escaped;
        }
        return escaped;
    }
    /**
     * Replace a given string by a given matcher.
     * @param {PatternMatcher} matcher The pattern matcher.
     * @param {string} str The string to be replaced.
     * @param {string} replacement The new substring to replace each matched part.
     * @returns {string} The replaced string.
     */
    function replaceS(matcher, str, replacement) {
        const chunks = [];
        let index = 0;
        /** @type {RegExpExecArray} */
        let match = null;
        /**
         * @param {string} key The placeholder.
         * @returns {string} The replaced string.
         */
        function replacer(key) {
            switch (key) {
                case "$$":
                    return "$";
                case "$&":
                    return match[0];
                case "$`":
                    return str.slice(0, match.index);
                case "$'":
                    return str.slice(match.index + match[0].length);
                default: {
                    const i = key.slice(1);
                    if (i in match) {
                        return match[i];
                    }
                    return key;
                }
            }
        }
        for (match of matcher.execAll(str)) {
            chunks.push(str.slice(index, match.index));
            chunks.push(replacement.replace(placeholder, replacer));
            index = match.index + match[0].length;
        }
        chunks.push(str.slice(index));
        return chunks.join("");
    }
    /**
     * Replace a given string by a given matcher.
     * @param {PatternMatcher} matcher The pattern matcher.
     * @param {string} str The string to be replaced.
     * @param {(...strs[])=>string} replace The function to replace each matched part.
     * @returns {string} The replaced string.
     */
    function replaceF(matcher, str, replace) {
        const chunks = [];
        let index = 0;
        for (const match of matcher.execAll(str)) {
            chunks.push(str.slice(index, match.index));
            chunks.push(String(replace(...match, match.index, match.input)));
            index = match.index + match[0].length;
        }
        chunks.push(str.slice(index));
        return chunks.join("");
    }
    /**
     * The class to find patterns as considering escape sequences.
     */
    class PatternMatcher {
        /**
         * Initialize this matcher.
         * @param {RegExp} pattern The pattern to match.
         * @param {{escaped:boolean}} options The options.
         */
        constructor(pattern, { escaped = false } = {}) {
            if (!(pattern instanceof RegExp)) {
                throw new TypeError("'pattern' should be a RegExp instance.");
            }
            if (!pattern.flags.includes("g")) {
                throw new Error("'pattern' should contains 'g' flag.");
            }
            internal.set(this, {
                pattern: new RegExp(pattern.source, pattern.flags),
                escaped: Boolean(escaped),
            });
        }
        /**
         * Find the pattern in a given string.
         * @param {string} str The string to find.
         * @returns {IterableIterator<RegExpExecArray>} The iterator which iterate the matched information.
         */
        *execAll(str) {
            const { pattern, escaped } = internal.get(this);
            let match = null;
            let lastIndex = 0;
            pattern.lastIndex = 0;
            while ((match = pattern.exec(str)) != null) {
                if (escaped || !isEscaped(str, match.index)) {
                    lastIndex = pattern.lastIndex;
                    yield match;
                    pattern.lastIndex = lastIndex;
                }
            }
        }
        /**
         * Check whether the pattern is found in a given string.
         * @param {string} str The string to check.
         * @returns {boolean} `true` if the pattern was found in the string.
         */
        test(str) {
            const it = this.execAll(str);
            const ret = it.next();
            return !ret.done;
        }
        /**
         * Replace a given string.
         * @param {string} str The string to be replaced.
         * @param {(string|((...strs:string[])=>string))} replacer The string or function to replace. This is the same as the 2nd argument of `String.prototype.replace`.
         * @returns {string} The replaced string.
         */
        [Symbol.replace](str, replacer) {
            return typeof replacer === "function"
                ? replaceF(this, String(str), replacer)
                : replaceS(this, String(str), String(replacer));
        }
    }
    const IMPORT_TYPE = /^(?:Import|Export(?:All|Default|Named))Declaration$/u;
    const has = Function.call.bind(Object.hasOwnProperty);
    const READ = Symbol("read");
    const CALL = Symbol("call");
    const CONSTRUCT = Symbol("construct");
    const ESM = Symbol("esm");
    const requireCall = { require: { [CALL]: true } };
    /**
     * Check whether a given variable is modified or not.
     * @param {Variable} variable The variable to check.
     * @returns {boolean} `true` if the variable is modified.
     */
    function isModifiedGlobal(variable) {
        return (variable == null ||
            variable.defs.length !== 0 ||
            variable.references.some(r => r.isWrite()));
    }
    /**
     * Check if the value of a given node is passed through to the parent syntax as-is.
     * For example, `a` and `b` in (`a || b` and `c ? a : b`) are passed through.
     * @param {Node} node A node to check.
     * @returns {boolean} `true` if the node is passed through.
     */
    function isPassThrough(node) {
        const parent = node.parent;
        switch (parent && parent.type) {
            case "ConditionalExpression":
                return parent.consequent === node || parent.alternate === node;
            case "LogicalExpression":
                return true;
            case "SequenceExpression":
                return parent.expressions[parent.expressions.length - 1] === node;
            default:
                return false;
        }
    }
    /**
     * The reference tracker.
     */
    class ReferenceTracker {
        /**
         * Initialize this tracker.
         * @param {Scope} globalScope The global scope.
         * @param {object} [options] The options.
         * @param {"legacy"|"strict"} [options.mode="strict"] The mode to determine the ImportDeclaration's behavior for CJS modules.
         * @param {string[]} [options.globalObjectNames=["global","self","window"]] The variable names for Global Object.
         */
        constructor(globalScope, { mode = "strict", globalObjectNames = ["global", "self", "window"], } = {}) {
            this.variableStack = [];
            this.globalScope = globalScope;
            this.mode = mode;
            this.globalObjectNames = globalObjectNames.slice(0);
        }
        /**
         * Iterate the references of global variables.
         * @param {object} traceMap The trace map.
         * @returns {IterableIterator<{node:Node,path:string[],type:symbol,info:any}>} The iterator to iterate references.
         */
        *iterateGlobalReferences(traceMap) {
            for (const key of Object.keys(traceMap)) {
                const nextTraceMap = traceMap[key];
                const path = [key];
                const variable = this.globalScope.set.get(key);
                if (isModifiedGlobal(variable)) {
                    continue;
                }
                yield* this._iterateVariableReferences(variable, path, nextTraceMap, true);
            }
            for (const key of this.globalObjectNames) {
                const path = [];
                const variable = this.globalScope.set.get(key);
                if (isModifiedGlobal(variable)) {
                    continue;
                }
                yield* this._iterateVariableReferences(variable, path, traceMap, false);
            }
        }
        /**
         * Iterate the references of CommonJS modules.
         * @param {object} traceMap The trace map.
         * @returns {IterableIterator<{node:Node,path:string[],type:symbol,info:any}>} The iterator to iterate references.
         */
        *iterateCjsReferences(traceMap) {
            for (const { node } of this.iterateGlobalReferences(requireCall)) {
                const key = getStringIfConstant(node.arguments[0]);
                if (key == null || !has(traceMap, key)) {
                    continue;
                }
                const nextTraceMap = traceMap[key];
                const path = [key];
                if (nextTraceMap[READ]) {
                    yield {
                        node,
                        path,
                        type: READ,
                        info: nextTraceMap[READ],
                    };
                }
                yield* this._iteratePropertyReferences(node, path, nextTraceMap);
            }
        }
        /**
         * Iterate the references of ES modules.
         * @param {object} traceMap The trace map.
         * @returns {IterableIterator<{node:Node,path:string[],type:symbol,info:any}>} The iterator to iterate references.
         */
        *iterateEsmReferences(traceMap) {
            const programNode = this.globalScope.block;
            for (const node of programNode.body) {
                if (!IMPORT_TYPE.test(node.type) || node.source == null) {
                    continue;
                }
                const moduleId = node.source.value;
                if (!has(traceMap, moduleId)) {
                    continue;
                }
                const nextTraceMap = traceMap[moduleId];
                const path = [moduleId];
                if (nextTraceMap[READ]) {
                    yield { node, path, type: READ, info: nextTraceMap[READ] };
                }
                if (node.type === "ExportAllDeclaration") {
                    for (const key of Object.keys(nextTraceMap)) {
                        const exportTraceMap = nextTraceMap[key];
                        if (exportTraceMap[READ]) {
                            yield {
                                node,
                                path: path.concat(key),
                                type: READ,
                                info: exportTraceMap[READ],
                            };
                        }
                    }
                }
                else {
                    for (const specifier of node.specifiers) {
                        const esm = has(nextTraceMap, ESM);
                        const it = this._iterateImportReferences(specifier, path, esm
                            ? nextTraceMap
                            : this.mode === "legacy"
                                ? Object.assign({ default: nextTraceMap }, nextTraceMap)
                                : { default: nextTraceMap });
                        if (esm) {
                            yield* it;
                        }
                        else {
                            for (const report of it) {
                                report.path = report.path.filter(exceptDefault);
                                if (report.path.length >= 2 ||
                                    report.type !== READ) {
                                    yield report;
                                }
                            }
                        }
                    }
                }
            }
        }
        /**
         * Iterate the references for a given variable.
         * @param {Variable} variable The variable to iterate that references.
         * @param {string[]} path The current path.
         * @param {object} traceMap The trace map.
         * @param {boolean} shouldReport = The flag to report those references.
         * @returns {IterableIterator<{node:Node,path:string[],type:symbol,info:any}>} The iterator to iterate references.
         */
        *_iterateVariableReferences(variable, path, traceMap, shouldReport) {
            if (this.variableStack.includes(variable)) {
                return;
            }
            this.variableStack.push(variable);
            try {
                for (const reference of variable.references) {
                    if (!reference.isRead()) {
                        continue;
                    }
                    const node = reference.identifier;
                    if (shouldReport && traceMap[READ]) {
                        yield { node, path, type: READ, info: traceMap[READ] };
                    }
                    yield* this._iteratePropertyReferences(node, path, traceMap);
                }
            }
            finally {
                this.variableStack.pop();
            }
        }
        /**
         * Iterate the references for a given AST node.
         * @param rootNode The AST node to iterate references.
         * @param {string[]} path The current path.
         * @param {object} traceMap The trace map.
         * @returns {IterableIterator<{node:Node,path:string[],type:symbol,info:any}>} The iterator to iterate references.
         */
        //eslint-disable-next-line complexity
        *_iteratePropertyReferences(rootNode, path, traceMap) {
            let node = rootNode;
            while (isPassThrough(node)) {
                node = node.parent;
            }
            const parent = node.parent;
            if (parent.type === "MemberExpression") {
                if (parent.object === node) {
                    const key = getPropertyName(parent);
                    if (key == null || !has(traceMap, key)) {
                        return;
                    }
                    path = path.concat(key); //eslint-disable-line no-param-reassign
                    const nextTraceMap = traceMap[key];
                    if (nextTraceMap[READ]) {
                        yield {
                            node: parent,
                            path,
                            type: READ,
                            info: nextTraceMap[READ],
                        };
                    }
                    yield* this._iteratePropertyReferences(parent, path, nextTraceMap);
                }
                return;
            }
            if (parent.type === "CallExpression") {
                if (parent.callee === node && traceMap[CALL]) {
                    yield { node: parent, path, type: CALL, info: traceMap[CALL] };
                }
                return;
            }
            if (parent.type === "NewExpression") {
                if (parent.callee === node && traceMap[CONSTRUCT]) {
                    yield {
                        node: parent,
                        path,
                        type: CONSTRUCT,
                        info: traceMap[CONSTRUCT],
                    };
                }
                return;
            }
            if (parent.type === "AssignmentExpression") {
                if (parent.right === node) {
                    yield* this._iterateLhsReferences(parent.left, path, traceMap);
                    yield* this._iteratePropertyReferences(parent, path, traceMap);
                }
                return;
            }
            if (parent.type === "AssignmentPattern") {
                if (parent.right === node) {
                    yield* this._iterateLhsReferences(parent.left, path, traceMap);
                }
                return;
            }
            if (parent.type === "VariableDeclarator") {
                if (parent.init === node) {
                    yield* this._iterateLhsReferences(parent.id, path, traceMap);
                }
            }
        }
        /**
         * Iterate the references for a given Pattern node.
         * @param {Node} patternNode The Pattern node to iterate references.
         * @param {string[]} path The current path.
         * @param {object} traceMap The trace map.
         * @returns {IterableIterator<{node:Node,path:string[],type:symbol,info:any}>} The iterator to iterate references.
         */
        *_iterateLhsReferences(patternNode, path, traceMap) {
            if (patternNode.type === "Identifier") {
                const variable = findVariable(this.globalScope, patternNode);
                if (variable != null) {
                    yield* this._iterateVariableReferences(variable, path, traceMap, false);
                }
                return;
            }
            if (patternNode.type === "ObjectPattern") {
                for (const property of patternNode.properties) {
                    const key = getPropertyName(property);
                    if (key == null || !has(traceMap, key)) {
                        continue;
                    }
                    const nextPath = path.concat(key);
                    const nextTraceMap = traceMap[key];
                    if (nextTraceMap[READ]) {
                        yield {
                            node: property,
                            path: nextPath,
                            type: READ,
                            info: nextTraceMap[READ],
                        };
                    }
                    yield* this._iterateLhsReferences(property.value, nextPath, nextTraceMap);
                }
                return;
            }
            if (patternNode.type === "AssignmentPattern") {
                yield* this._iterateLhsReferences(patternNode.left, path, traceMap);
            }
        }
        /**
         * Iterate the references for a given ModuleSpecifier node.
         * @param {Node} specifierNode The ModuleSpecifier node to iterate references.
         * @param {string[]} path The current path.
         * @param {object} traceMap The trace map.
         * @returns {IterableIterator<{node:Node,path:string[],type:symbol,info:any}>} The iterator to iterate references.
         */
        *_iterateImportReferences(specifierNode, path, traceMap) {
            const type = specifierNode.type;
            if (type === "ImportSpecifier" || type === "ImportDefaultSpecifier") {
                const key = type === "ImportDefaultSpecifier"
                    ? "default"
                    : specifierNode.imported.name;
                if (!has(traceMap, key)) {
                    return;
                }
                path = path.concat(key); //eslint-disable-line no-param-reassign
                const nextTraceMap = traceMap[key];
                if (nextTraceMap[READ]) {
                    yield {
                        node: specifierNode,
                        path,
                        type: READ,
                        info: nextTraceMap[READ],
                    };
                }
                yield* this._iterateVariableReferences(findVariable(this.globalScope, specifierNode.local), path, nextTraceMap, false);
                return;
            }
            if (type === "ImportNamespaceSpecifier") {
                yield* this._iterateVariableReferences(findVariable(this.globalScope, specifierNode.local), path, traceMap, false);
                return;
            }
            if (type === "ExportSpecifier") {
                const key = specifierNode.local.name;
                if (!has(traceMap, key)) {
                    return;
                }
                path = path.concat(key); //eslint-disable-line no-param-reassign
                const nextTraceMap = traceMap[key];
                if (nextTraceMap[READ]) {
                    yield {
                        node: specifierNode,
                        path,
                        type: READ,
                        info: nextTraceMap[READ],
                    };
                }
            }
        }
    }
    ReferenceTracker.READ = READ;
    ReferenceTracker.CALL = CALL;
    ReferenceTracker.CONSTRUCT = CONSTRUCT;
    ReferenceTracker.ESM = ESM;
    /**
     * This is a predicate function for Array#filter.
     * @param {string} name A name part.
     * @param {number} index The index of the name.
     * @returns {boolean} `false` if it's default.
     */
    function exceptDefault(name, index) {
        return !(index === 1 && name === "default");
    }
    var index = {
        CALL,
        CONSTRUCT,
        ESM,
        findVariable,
        getFunctionHeadLocation,
        getFunctionNameWithKind,
        getInnermostScope,
        getPropertyName,
        getStaticValue,
        getStringIfConstant,
        hasSideEffect,
        isArrowToken,
        isClosingBraceToken,
        isClosingBracketToken,
        isClosingParenToken,
        isColonToken,
        isCommaToken,
        isCommentToken,
        isNotArrowToken,
        isNotClosingBraceToken,
        isNotClosingBracketToken,
        isNotClosingParenToken,
        isNotColonToken,
        isNotCommaToken,
        isNotCommentToken,
        isNotOpeningBraceToken,
        isNotOpeningBracketToken,
        isNotOpeningParenToken,
        isNotSemicolonToken,
        isOpeningBraceToken,
        isOpeningBracketToken,
        isOpeningParenToken,
        isParenthesized,
        isSemicolonToken,
        PatternMatcher,
        READ,
        ReferenceTracker,
    };
    exports.CALL = CALL;
    exports.CONSTRUCT = CONSTRUCT;
    exports.ESM = ESM;
    exports.PatternMatcher = PatternMatcher;
    exports.READ = READ;
    exports.ReferenceTracker = ReferenceTracker;
    exports.default = index;
    exports.findVariable = findVariable;
    exports.getFunctionHeadLocation = getFunctionHeadLocation;
    exports.getFunctionNameWithKind = getFunctionNameWithKind;
    exports.getInnermostScope = getInnermostScope;
    exports.getPropertyName = getPropertyName;
    exports.getStaticValue = getStaticValue;
    exports.getStringIfConstant = getStringIfConstant;
    exports.hasSideEffect = hasSideEffect;
    exports.isArrowToken = isArrowToken;
    exports.isClosingBraceToken = isClosingBraceToken;
    exports.isClosingBracketToken = isClosingBracketToken;
    exports.isClosingParenToken = isClosingParenToken;
    exports.isColonToken = isColonToken;
    exports.isCommaToken = isCommaToken;
    exports.isCommentToken = isCommentToken;
    exports.isNotArrowToken = isNotArrowToken;
    exports.isNotClosingBraceToken = isNotClosingBraceToken;
    exports.isNotClosingBracketToken = isNotClosingBracketToken;
    exports.isNotClosingParenToken = isNotClosingParenToken;
    exports.isNotColonToken = isNotColonToken;
    exports.isNotCommaToken = isNotCommaToken;
    exports.isNotCommentToken = isNotCommentToken;
    exports.isNotOpeningBraceToken = isNotOpeningBraceToken;
    exports.isNotOpeningBracketToken = isNotOpeningBracketToken;
    exports.isNotOpeningParenToken = isNotOpeningParenToken;
    exports.isNotSemicolonToken = isNotSemicolonToken;
    exports.isOpeningBraceToken = isOpeningBraceToken;
    exports.isOpeningBracketToken = isOpeningBracketToken;
    exports.isOpeningParenToken = isOpeningParenToken;
    exports.isParenthesized = isParenthesized;
    exports.isSemicolonToken = isSemicolonToken;
},
/* eslint/lib/rules/no-extra-semi.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const FixTracker = require(237) /* ./utils/fix-tracker */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary semicolons",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-extra-semi"
            },
            fixable: "code",
            schema: [],
            messages: {
                unexpected: "Unnecessary semicolon."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Reports an unnecessary semicolon error.
             * @param {Node|Token} nodeOrToken A node or a token to be reported.
             * @returns {void}
             */
            function report(nodeOrToken) {
                context.report({
                    node: nodeOrToken,
                    messageId: "unexpected",
                    fix(fixer) {
                        /*
                         * Expand the replacement range to include the surrounding
                         * tokens to avoid conflicting with semi.
                         * https://github.com/eslint/eslint/issues/7928
                         */
                        return new FixTracker(fixer, context.getSourceCode())
                            .retainSurroundingTokens(nodeOrToken)
                            .remove(nodeOrToken);
                    }
                });
            }
            /**
             * Checks for a part of a class body.
             * This checks tokens from a specified token to a next MethodDefinition or the end of class body.
             * @param {Token} firstToken The first token to check.
             * @returns {void}
             */
            function checkForPartOfClassBody(firstToken) {
                for (let token = firstToken; token.type === "Punctuator" && !astUtils.isClosingBraceToken(token); token = sourceCode.getTokenAfter(token)) {
                    if (astUtils.isSemicolonToken(token)) {
                        report(token);
                    }
                }
            }
            return {
                /**
                 * Reports this empty statement, except if the parent node is a loop.
                 * @param {Node} node A EmptyStatement node to be reported.
                 * @returns {void}
                 */
                EmptyStatement(node) {
                    const parent = node.parent, allowedParentTypes = [
                        "ForStatement",
                        "ForInStatement",
                        "ForOfStatement",
                        "WhileStatement",
                        "DoWhileStatement",
                        "IfStatement",
                        "LabeledStatement",
                        "WithStatement"
                    ];
                    if (allowedParentTypes.indexOf(parent.type) === -1) {
                        report(node);
                    }
                },
                /**
                 * Checks tokens from the head of this class body to the first MethodDefinition or the end of this class body.
                 * @param {Node} node A ClassBody node to check.
                 * @returns {void}
                 */
                ClassBody(node) {
                    checkForPartOfClassBody(sourceCode.getFirstToken(node, 1)); // 0 is `{`.
                },
                /**
                 * Checks tokens from this MethodDefinition to the next MethodDefinition or the end of this class body.
                 * @param {Node} node A MethodDefinition node of the start point.
                 * @returns {void}
                 */
                MethodDefinition(node) {
                    checkForPartOfClassBody(sourceCode.getTokenAfter(node));
                }
            };
        }
    };
},
/* eslint/lib/rules/no-fallthrough.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const DEFAULT_FALLTHROUGH_COMMENT = /falls?\s?through/iu;
    /**
     * Checks whether or not a given node has a fallthrough comment.
     * @param {ASTNode} node A SwitchCase node to get comments.
     * @param {RuleContext} context A rule context which stores comments.
     * @param {RegExp} fallthroughCommentPattern A pattern to match comment to.
     * @returns {boolean} `true` if the node has a valid fallthrough comment.
     */
    function hasFallthroughComment(node, context, fallthroughCommentPattern) {
        const sourceCode = context.getSourceCode();
        const comment = lodash.last(sourceCode.getCommentsBefore(node));
        return Boolean(comment && fallthroughCommentPattern.test(comment.value));
    }
    /**
     * Checks whether or not a given code path segment is reachable.
     * @param {CodePathSegment} segment A CodePathSegment to check.
     * @returns {boolean} `true` if the segment is reachable.
     */
    function isReachable(segment) {
        return segment.reachable;
    }
    /**
     * Checks whether a node and a token are separated by blank lines
     * @param {ASTNode} node The node to check
     * @param {Token} token The token to compare against
     * @returns {boolean} `true` if there are blank lines between node and token
     */
    function hasBlankLinesBetween(node, token) {
        return token.loc.start.line > node.loc.end.line + 1;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow fallthrough of `case` statements",
                category: "Best Practices",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-fallthrough"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        commentPattern: {
                            type: "string",
                            default: ""
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                case: "Expected a 'break' statement before 'case'.",
                default: "Expected a 'break' statement before 'default'."
            }
        },
        create(context) {
            const options = context.options[0] || {};
            let currentCodePath = null;
            const sourceCode = context.getSourceCode();
            /*
             * We need to use leading comments of the next SwitchCase node because
             * trailing comments is wrong if semicolons are omitted.
             */
            let fallthroughCase = null;
            let fallthroughCommentPattern = null;
            if (options.commentPattern) {
                fallthroughCommentPattern = new RegExp(options.commentPattern, "u");
            }
            else {
                fallthroughCommentPattern = DEFAULT_FALLTHROUGH_COMMENT;
            }
            return {
                onCodePathStart(codePath) {
                    currentCodePath = codePath;
                },
                onCodePathEnd() {
                    currentCodePath = currentCodePath.upper;
                },
                SwitchCase(node) {
                    /*
                     * Checks whether or not there is a fallthrough comment.
                     * And reports the previous fallthrough node if that does not exist.
                     */
                    if (fallthroughCase && !hasFallthroughComment(node, context, fallthroughCommentPattern)) {
                        context.report({
                            messageId: node.test ? "case" : "default",
                            node
                        });
                    }
                    fallthroughCase = null;
                },
                "SwitchCase:exit"(node) {
                    const nextToken = sourceCode.getTokenAfter(node);
                    /*
                     * `reachable` meant fall through because statements preceded by
                     * `break`, `return`, or `throw` are unreachable.
                     * And allows empty cases and the last case.
                     */
                    if (currentCodePath.currentSegments.some(isReachable) &&
                        (node.consequent.length > 0 || hasBlankLinesBetween(node, nextToken)) &&
                        lodash.last(node.parent.cases) !== node) {
                        fallthroughCase = node;
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-floating-decimal.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow leading or trailing decimal points in numeric literals",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-floating-decimal"
            },
            schema: [],
            fixable: "code",
            messages: {
                leading: "A leading decimal point can be confused with a dot.",
                trailing: "A trailing decimal point can be confused with a dot."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                Literal(node) {
                    if (typeof node.value === "number") {
                        if (node.raw.startsWith(".")) {
                            context.report({
                                node,
                                messageId: "leading",
                                fix(fixer) {
                                    const tokenBefore = sourceCode.getTokenBefore(node);
                                    const needsSpaceBefore = tokenBefore &&
                                        tokenBefore.range[1] === node.range[0] &&
                                        !astUtils.canTokensBeAdjacent(tokenBefore, `0${node.raw}`);
                                    return fixer.insertTextBefore(node, needsSpaceBefore ? " 0" : "0");
                                }
                            });
                        }
                        if (node.raw.indexOf(".") === node.raw.length - 1) {
                            context.report({
                                node,
                                messageId: "trailing",
                                fix: fixer => fixer.insertTextAfter(node, "0")
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-func-assign.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow reassigning `function` declarations",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-func-assign"
            },
            schema: []
        },
        create(context) {
            /**
             * Reports a reference if is non initializer and writable.
             * @param {References} references Collection of reference to check.
             * @returns {void}
             */
            function checkReference(references) {
                astUtils.getModifyingReferences(references).forEach(reference => {
                    context.report({ node: reference.identifier, message: "'{{name}}' is a function.", data: { name: reference.identifier.name } });
                });
            }
            /**
             * Finds and reports references that are non initializer and writable.
             * @param {Variable} variable A variable to check.
             * @returns {void}
             */
            function checkVariable(variable) {
                if (variable.defs[0].type === "FunctionName") {
                    checkReference(variable.references);
                }
            }
            /**
             * Checks parameters of a given function node.
             * @param {ASTNode} node A function node to check.
             * @returns {void}
             */
            function checkForFunction(node) {
                context.getDeclaredVariables(node).forEach(checkVariable);
            }
            return {
                FunctionDeclaration: checkForFunction,
                FunctionExpression: checkForFunction
            };
        }
    };
},
/* eslint/lib/rules/no-global-assign.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow assignments to native objects or read-only global variables",
                category: "Best Practices",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-global-assign"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        exceptions: {
                            type: "array",
                            items: { type: "string" },
                            uniqueItems: true
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const config = context.options[0];
            const exceptions = (config && config.exceptions) || [];
            /**
             * Reports write references.
             * @param {Reference} reference A reference to check.
             * @param {int} index The index of the reference in the references.
             * @param {Reference[]} references The array that the reference belongs to.
             * @returns {void}
             */
            function checkReference(reference, index, references) {
                const identifier = reference.identifier;
                if (reference.init === false &&
                    reference.isWrite() &&
                    /*
                     * Destructuring assignments can have multiple default value,
                     * so possibly there are multiple writeable references for the same identifier.
                     */
                    (index === 0 || references[index - 1].identifier !== identifier)) {
                    context.report({
                        node: identifier,
                        message: "Read-only global '{{name}}' should not be modified.",
                        data: identifier
                    });
                }
            }
            /**
             * Reports write references if a given variable is read-only builtin.
             * @param {Variable} variable A variable to check.
             * @returns {void}
             */
            function checkVariable(variable) {
                if (variable.writeable === false && exceptions.indexOf(variable.name) === -1) {
                    variable.references.forEach(checkReference);
                }
            }
            return {
                Program() {
                    const globalScope = context.getScope();
                    globalScope.variables.forEach(checkVariable);
                }
            };
        }
    };
},
/* eslint/lib/rules/no-implicit-coercion.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const INDEX_OF_PATTERN = /^(?:i|lastI)ndexOf$/u;
    const ALLOWABLE_OPERATORS = ["~", "!!", "+", "*"];
    /**
     * Parses and normalizes an option object.
     * @param {Object} options An option object to parse.
     * @returns {Object} The parsed and normalized option object.
     */
    function parseOptions(options) {
        return {
            boolean: "boolean" in options ? options.boolean : true,
            number: "number" in options ? options.number : true,
            string: "string" in options ? options.string : true,
            allow: options.allow || []
        };
    }
    /**
     * Checks whether or not a node is a double logical nigating.
     * @param {ASTNode} node An UnaryExpression node to check.
     * @returns {boolean} Whether or not the node is a double logical nigating.
     */
    function isDoubleLogicalNegating(node) {
        return (node.operator === "!" &&
            node.argument.type === "UnaryExpression" &&
            node.argument.operator === "!");
    }
    /**
     * Checks whether or not a node is a binary negating of `.indexOf()` method calling.
     * @param {ASTNode} node An UnaryExpression node to check.
     * @returns {boolean} Whether or not the node is a binary negating of `.indexOf()` method calling.
     */
    function isBinaryNegatingOfIndexOf(node) {
        return (node.operator === "~" &&
            node.argument.type === "CallExpression" &&
            node.argument.callee.type === "MemberExpression" &&
            node.argument.callee.property.type === "Identifier" &&
            INDEX_OF_PATTERN.test(node.argument.callee.property.name));
    }
    /**
     * Checks whether or not a node is a multiplying by one.
     * @param {BinaryExpression} node A BinaryExpression node to check.
     * @returns {boolean} Whether or not the node is a multiplying by one.
     */
    function isMultiplyByOne(node) {
        return node.operator === "*" && (node.left.type === "Literal" && node.left.value === 1 ||
            node.right.type === "Literal" && node.right.value === 1);
    }
    /**
     * Checks whether the result of a node is numeric or not
     * @param {ASTNode} node The node to test
     * @returns {boolean} true if the node is a number literal or a `Number()`, `parseInt` or `parseFloat` call
     */
    function isNumeric(node) {
        return (node.type === "Literal" && typeof node.value === "number" ||
            node.type === "CallExpression" && (node.callee.name === "Number" ||
                node.callee.name === "parseInt" ||
                node.callee.name === "parseFloat"));
    }
    /**
     * Returns the first non-numeric operand in a BinaryExpression. Designed to be
     * used from bottom to up since it walks up the BinaryExpression trees using
     * node.parent to find the result.
     * @param {BinaryExpression} node The BinaryExpression node to be walked up on
     * @returns {ASTNode|null} The first non-numeric item in the BinaryExpression tree or null
     */
    function getNonNumericOperand(node) {
        const left = node.left, right = node.right;
        if (right.type !== "BinaryExpression" && !isNumeric(right)) {
            return right;
        }
        if (left.type !== "BinaryExpression" && !isNumeric(left)) {
            return left;
        }
        return null;
    }
    /**
     * Checks whether a node is an empty string literal or not.
     * @param {ASTNode} node The node to check.
     * @returns {boolean} Whether or not the passed in node is an
     * empty string literal or not.
     */
    function isEmptyString(node) {
        return astUtils.isStringLiteral(node) && (node.value === "" || (node.type === "TemplateLiteral" && node.quasis.length === 1 && node.quasis[0].value.cooked === ""));
    }
    /**
     * Checks whether or not a node is a concatenating with an empty string.
     * @param {ASTNode} node A BinaryExpression node to check.
     * @returns {boolean} Whether or not the node is a concatenating with an empty string.
     */
    function isConcatWithEmptyString(node) {
        return node.operator === "+" && ((isEmptyString(node.left) && !astUtils.isStringLiteral(node.right)) ||
            (isEmptyString(node.right) && !astUtils.isStringLiteral(node.left)));
    }
    /**
     * Checks whether or not a node is appended with an empty string.
     * @param {ASTNode} node An AssignmentExpression node to check.
     * @returns {boolean} Whether or not the node is appended with an empty string.
     */
    function isAppendEmptyString(node) {
        return node.operator === "+=" && isEmptyString(node.right);
    }
    /**
     * Returns the operand that is not an empty string from a flagged BinaryExpression.
     * @param {ASTNode} node The flagged BinaryExpression node to check.
     * @returns {ASTNode} The operand that is not an empty string from a flagged BinaryExpression.
     */
    function getNonEmptyOperand(node) {
        return isEmptyString(node.left) ? node.right : node.left;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow shorthand type conversions",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-implicit-coercion"
            },
            fixable: "code",
            schema: [{
                    type: "object",
                    properties: {
                        boolean: {
                            type: "boolean",
                            default: true
                        },
                        number: {
                            type: "boolean",
                            default: true
                        },
                        string: {
                            type: "boolean",
                            default: true
                        },
                        allow: {
                            type: "array",
                            items: {
                                enum: ALLOWABLE_OPERATORS
                            },
                            uniqueItems: true
                        }
                    },
                    additionalProperties: false
                }]
        },
        create(context) {
            const options = parseOptions(context.options[0] || {});
            const sourceCode = context.getSourceCode();
            /**
             * Reports an error and autofixes the node
             * @param {ASTNode} node An ast node to report the error on.
             * @param {string} recommendation The recommended code for the issue
             * @param {bool} shouldFix Whether this report should fix the node
             * @returns {void}
             */
            function report(node, recommendation, shouldFix) {
                context.report({
                    node,
                    message: "use `{{recommendation}}` instead.",
                    data: {
                        recommendation
                    },
                    fix(fixer) {
                        if (!shouldFix) {
                            return null;
                        }
                        const tokenBefore = sourceCode.getTokenBefore(node);
                        if (tokenBefore &&
                            tokenBefore.range[1] === node.range[0] &&
                            !astUtils.canTokensBeAdjacent(tokenBefore, recommendation)) {
                            return fixer.replaceText(node, ` ${recommendation}`);
                        }
                        return fixer.replaceText(node, recommendation);
                    }
                });
            }
            return {
                UnaryExpression(node) {
                    let operatorAllowed;
                    // !!foo
                    operatorAllowed = options.allow.indexOf("!!") >= 0;
                    if (!operatorAllowed && options.boolean && isDoubleLogicalNegating(node)) {
                        const recommendation = `Boolean(${sourceCode.getText(node.argument.argument)})`;
                        report(node, recommendation, true);
                    }
                    // ~foo.indexOf(bar)
                    operatorAllowed = options.allow.indexOf("~") >= 0;
                    if (!operatorAllowed && options.boolean && isBinaryNegatingOfIndexOf(node)) {
                        const recommendation = `${sourceCode.getText(node.argument)} !== -1`;
                        report(node, recommendation, false);
                    }
                    // +foo
                    operatorAllowed = options.allow.indexOf("+") >= 0;
                    if (!operatorAllowed && options.number && node.operator === "+" && !isNumeric(node.argument)) {
                        const recommendation = `Number(${sourceCode.getText(node.argument)})`;
                        report(node, recommendation, true);
                    }
                },
                // Use `:exit` to prevent double reporting
                "BinaryExpression:exit"(node) {
                    let operatorAllowed;
                    // 1 * foo
                    operatorAllowed = options.allow.indexOf("*") >= 0;
                    const nonNumericOperand = !operatorAllowed && options.number && isMultiplyByOne(node) && getNonNumericOperand(node);
                    if (nonNumericOperand) {
                        const recommendation = `Number(${sourceCode.getText(nonNumericOperand)})`;
                        report(node, recommendation, true);
                    }
                    // "" + foo
                    operatorAllowed = options.allow.indexOf("+") >= 0;
                    if (!operatorAllowed && options.string && isConcatWithEmptyString(node)) {
                        const recommendation = `String(${sourceCode.getText(getNonEmptyOperand(node))})`;
                        report(node, recommendation, true);
                    }
                },
                AssignmentExpression(node) {
                    // foo += ""
                    const operatorAllowed = options.allow.indexOf("+") >= 0;
                    if (!operatorAllowed && options.string && isAppendEmptyString(node)) {
                        const code = sourceCode.getText(getNonEmptyOperand(node));
                        const recommendation = `${code} = String(${code})`;
                        report(node, recommendation, true);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-implicit-globals.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow declarations in the global scope",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-implicit-globals"
            },
            schema: [{
                    type: "object",
                    properties: {
                        lexicalBindings: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }],
            messages: {
                globalNonLexicalBinding: "Unexpected {{kind}} declaration in the global scope, wrap in an IIFE for a local variable, assign as global property for a global variable.",
                globalLexicalBinding: "Unexpected {{kind}} declaration in the global scope, wrap in a block or in an IIFE.",
                globalVariableLeak: "Global variable leak, declare the variable if it is intended to be local.",
                assignmentToReadonlyGlobal: "Unexpected assignment to read-only global variable.",
                redeclarationOfReadonlyGlobal: "Unexpected redeclaration of read-only global variable."
            }
        },
        create(context) {
            const checkLexicalBindings = context.options[0] && context.options[0].lexicalBindings === true;
            /**
             * Reports the node.
             * @param {ASTNode} node Node to report.
             * @param {string} messageId Id of the message to report.
             * @param {string|undefined} kind Declaration kind, can be 'var', 'const', 'let', function or class.
             * @returns {void}
             */
            function report(node, messageId, kind) {
                context.report({
                    node,
                    messageId,
                    data: {
                        kind
                    }
                });
            }
            return {
                Program() {
                    const scope = context.getScope();
                    scope.variables.forEach(variable => {
                        // Only ESLint global variables have the `writable` key.
                        const isReadonlyEslintGlobalVariable = variable.writeable === false;
                        const isWritableEslintGlobalVariable = variable.writeable === true;
                        if (isWritableEslintGlobalVariable) {
                            // Everything is allowed with writable ESLint global variables.
                            return;
                        }
                        variable.defs.forEach(def => {
                            const defNode = def.node;
                            if (def.type === "FunctionName" || (def.type === "Variable" && def.parent.kind === "var")) {
                                if (isReadonlyEslintGlobalVariable) {
                                    report(defNode, "redeclarationOfReadonlyGlobal");
                                }
                                else {
                                    report(defNode, "globalNonLexicalBinding", def.type === "FunctionName" ? "function" : `'${def.parent.kind}'`);
                                }
                            }
                            if (checkLexicalBindings) {
                                if (def.type === "ClassName" ||
                                    (def.type === "Variable" && (def.parent.kind === "let" || def.parent.kind === "const"))) {
                                    if (isReadonlyEslintGlobalVariable) {
                                        report(defNode, "redeclarationOfReadonlyGlobal");
                                    }
                                    else {
                                        report(defNode, "globalLexicalBinding", def.type === "ClassName" ? "class" : `'${def.parent.kind}'`);
                                    }
                                }
                            }
                        });
                    });
                    // Undeclared assigned variables.
                    scope.implicit.variables.forEach(variable => {
                        const scopeVariable = scope.set.get(variable.name);
                        let messageId;
                        if (scopeVariable) {
                            // ESLint global variable
                            if (scopeVariable.writeable) {
                                return;
                            }
                            messageId = "assignmentToReadonlyGlobal";
                        }
                        else {
                            // Reference to an unknown variable, possible global leak.
                            messageId = "globalVariableLeak";
                        }
                        // def.node is an AssignmentExpression, ForInStatement or ForOfStatement.
                        variable.defs.forEach(def => {
                            report(def.node, messageId);
                        });
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-implied-eval.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the use of `eval()`-like methods",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-implied-eval"
            },
            schema: []
        },
        create(context) {
            const CALLEE_RE = /^(setTimeout|setInterval|execScript)$/u;
            /*
             * Figures out if we should inspect a given binary expression. Is a stack
             * of stacks, where the first element in each substack is a CallExpression.
             */
            const impliedEvalAncestorsStack = [];
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Get the last element of an array, without modifying arr, like pop(), but non-destructive.
             * @param {Array} arr What to inspect
             * @returns {*} The last element of arr
             * @private
             */
            function last(arr) {
                return arr ? arr[arr.length - 1] : null;
            }
            /**
             * Checks if the given MemberExpression node is a potentially implied eval identifier on window.
             * @param {ASTNode} node The MemberExpression node to check.
             * @returns {boolean} Whether or not the given node is potentially an implied eval.
             * @private
             */
            function isImpliedEvalMemberExpression(node) {
                const object = node.object, property = node.property, hasImpliedEvalName = CALLEE_RE.test(property.name) || CALLEE_RE.test(property.value);
                return object.name === "window" && hasImpliedEvalName;
            }
            /**
             * Determines if a node represents a call to a potentially implied eval.
             *
             * This checks the callee name and that there's an argument, but not the type of the argument.
             * @param {ASTNode} node The CallExpression to check.
             * @returns {boolean} True if the node matches, false if not.
             * @private
             */
            function isImpliedEvalCallExpression(node) {
                const isMemberExpression = (node.callee.type === "MemberExpression"), isIdentifier = (node.callee.type === "Identifier"), isImpliedEvalCallee = (isIdentifier && CALLEE_RE.test(node.callee.name)) ||
                    (isMemberExpression && isImpliedEvalMemberExpression(node.callee));
                return isImpliedEvalCallee && node.arguments.length;
            }
            /**
             * Checks that the parent is a direct descendent of an potential implied eval CallExpression, and if the parent is a CallExpression, that we're the first argument.
             * @param {ASTNode} node The node to inspect the parent of.
             * @returns {boolean} Was the parent a direct descendent, and is the child therefore potentially part of a dangerous argument?
             * @private
             */
            function hasImpliedEvalParent(node) {
                // make sure our parent is marked
                return node.parent === last(last(impliedEvalAncestorsStack)) &&
                    // if our parent is a CallExpression, make sure we're the first argument
                    (node.parent.type !== "CallExpression" || node === node.parent.arguments[0]);
            }
            /**
             * Checks if our parent is marked as part of an implied eval argument. If
             * so, collapses the top of impliedEvalAncestorsStack and reports on the
             * original CallExpression.
             * @param {ASTNode} node The CallExpression to check.
             * @returns {boolean} True if the node matches, false if not.
             * @private
             */
            function checkString(node) {
                if (hasImpliedEvalParent(node)) {
                    // remove the entire substack, to avoid duplicate reports
                    const substack = impliedEvalAncestorsStack.pop();
                    context.report({ node: substack[0], message: "Implied eval. Consider passing a function instead of a string." });
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                CallExpression(node) {
                    if (isImpliedEvalCallExpression(node)) {
                        // call expressions create a new substack
                        impliedEvalAncestorsStack.push([node]);
                    }
                },
                "CallExpression:exit"(node) {
                    if (node === last(last(impliedEvalAncestorsStack))) {
                        /*
                         * Destroys the entire sub-stack, rather than just using
                         * last(impliedEvalAncestorsStack).pop(), as a CallExpression is
                         * always the bottom of a impliedEvalAncestorsStack substack.
                         */
                        impliedEvalAncestorsStack.pop();
                    }
                },
                BinaryExpression(node) {
                    if (node.operator === "+" && hasImpliedEvalParent(node)) {
                        last(impliedEvalAncestorsStack).push(node);
                    }
                },
                "BinaryExpression:exit"(node) {
                    if (node === last(last(impliedEvalAncestorsStack))) {
                        last(impliedEvalAncestorsStack).pop();
                    }
                },
                Literal(node) {
                    if (typeof node.value === "string") {
                        checkString(node);
                    }
                },
                TemplateLiteral(node) {
                    checkString(node);
                }
            };
        }
    };
},
/* eslint/lib/rules/no-import-assign.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const { findVariable, getPropertyName } = require(250) /* eslint-utils */;
    const MutationMethods = {
        Object: new Set([
            "assign", "defineProperties", "defineProperty", "freeze",
            "setPrototypeOf"
        ]),
        Reflect: new Set([
            "defineProperty", "deleteProperty", "set", "setPrototypeOf"
        ])
    };
    /**
     * Check if a given node is LHS of an assignment node.
     * @param {ASTNode} node The node to check.
     * @returns {boolean} `true` if the node is LHS.
     */
    function isAssignmentLeft(node) {
        const { parent } = node;
        return ((parent.type === "AssignmentExpression" &&
            parent.left === node) ||
            // Destructuring assignments
            parent.type === "ArrayPattern" ||
            (parent.type === "Property" &&
                parent.value === node &&
                parent.parent.type === "ObjectPattern") ||
            parent.type === "RestElement" ||
            (parent.type === "AssignmentPattern" &&
                parent.left === node));
    }
    /**
     * Check if a given node is the operand of mutation unary operator.
     * @param {ASTNode} node The node to check.
     * @returns {boolean} `true` if the node is the operand of mutation unary operator.
     */
    function isOperandOfMutationUnaryOperator(node) {
        const { parent } = node;
        return ((parent.type === "UpdateExpression" &&
            parent.argument === node) ||
            (parent.type === "UnaryExpression" &&
                parent.operator === "delete" &&
                parent.argument === node));
    }
    /**
     * Check if a given node is the iteration variable of `for-in`/`for-of` syntax.
     * @param {ASTNode} node The node to check.
     * @returns {boolean} `true` if the node is the iteration variable.
     */
    function isIterationVariable(node) {
        const { parent } = node;
        return ((parent.type === "ForInStatement" &&
            parent.left === node) ||
            (parent.type === "ForOfStatement" &&
                parent.left === node));
    }
    /**
     * Check if a given node is the iteration variable of `for-in`/`for-of` syntax.
     * @param {ASTNode} node The node to check.
     * @param {Scope} scope A `escope.Scope` object to find variable (whichever).
     * @returns {boolean} `true` if the node is the iteration variable.
     */
    function isArgumentOfWellKnownMutationFunction(node, scope) {
        const { parent } = node;
        if (parent.type === "CallExpression" &&
            parent.arguments[0] === node &&
            parent.callee.type === "MemberExpression" &&
            parent.callee.object.type === "Identifier") {
            const { callee } = parent;
            const { object } = callee;
            if (Object.keys(MutationMethods).includes(object.name)) {
                const variable = findVariable(scope, object);
                return (variable !== null &&
                    variable.scope.type === "global" &&
                    MutationMethods[object.name].has(getPropertyName(callee, scope)));
            }
        }
        return false;
    }
    /**
     * Check if the identifier node is placed at to update members.
     * @param {ASTNode} id The Identifier node to check.
     * @param {Scope} scope A `escope.Scope` object to find variable (whichever).
     * @returns {boolean} `true` if the member of `id` was updated.
     */
    function isMemberWrite(id, scope) {
        const { parent } = id;
        return ((parent.type === "MemberExpression" &&
            parent.object === id &&
            (isAssignmentLeft(parent) ||
                isOperandOfMutationUnaryOperator(parent) ||
                isIterationVariable(parent))) ||
            isArgumentOfWellKnownMutationFunction(id, scope));
    }
    /**
     * Get the mutation node.
     * @param {ASTNode} id The Identifier node to get.
     * @returns {ASTNode} The mutation node.
     */
    function getWriteNode(id) {
        let node = id.parent;
        while (node &&
            node.type !== "AssignmentExpression" &&
            node.type !== "UpdateExpression" &&
            node.type !== "UnaryExpression" &&
            node.type !== "CallExpression" &&
            node.type !== "ForInStatement" &&
            node.type !== "ForOfStatement") {
            node = node.parent;
        }
        return node || id;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow assigning to imported bindings",
                category: "Possible Errors",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-import-assign"
            },
            schema: [],
            messages: {
                readonly: "'{{name}}' is read-only.",
                readonlyMember: "The members of '{{name}}' are read-only."
            }
        },
        create(context) {
            return {
                ImportDeclaration(node) {
                    const scope = context.getScope();
                    for (const variable of context.getDeclaredVariables(node)) {
                        const shouldCheckMembers = variable.defs.some(d => d.node.type === "ImportNamespaceSpecifier");
                        let prevIdNode = null;
                        for (const reference of variable.references) {
                            const idNode = reference.identifier;
                            /*
                             * AssignmentPattern (e.g. `[a = 0] = b`) makes two write
                             * references for the same identifier. This should skip
                             * the one of the two in order to prevent redundant reports.
                             */
                            if (idNode === prevIdNode) {
                                continue;
                            }
                            prevIdNode = idNode;
                            if (reference.isWrite()) {
                                context.report({
                                    node: getWriteNode(idNode),
                                    messageId: "readonly",
                                    data: { name: idNode.name }
                                });
                            }
                            else if (shouldCheckMembers && isMemberWrite(idNode, scope)) {
                                context.report({
                                    node: getWriteNode(idNode),
                                    messageId: "readonlyMember",
                                    data: { name: idNode.name }
                                });
                            }
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-inline-comments.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow inline comments after code",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-inline-comments"
            },
            schema: []
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Will check that comments are not on lines starting with or ending with code
             * @param {ASTNode} node The comment node to check
             * @private
             * @returns {void}
             */
            function testCodeAroundComment(node) {
                const startLine = String(sourceCode.lines[node.loc.start.line - 1]), endLine = String(sourceCode.lines[node.loc.end.line - 1]), preamble = startLine.slice(0, node.loc.start.column).trim(), postamble = endLine.slice(node.loc.end.column).trim(), isPreambleEmpty = !preamble, isPostambleEmpty = !postamble;
                // Nothing on both sides
                if (isPreambleEmpty && isPostambleEmpty) {
                    return;
                }
                // JSX Exception
                if ((isPreambleEmpty || preamble === "{") &&
                    (isPostambleEmpty || postamble === "}")) {
                    const enclosingNode = sourceCode.getNodeByRangeIndex(node.range[0]);
                    if (enclosingNode && enclosingNode.type === "JSXEmptyExpression") {
                        return;
                    }
                }
                // Don't report ESLint directive comments
                if (astUtils.isDirectiveComment(node)) {
                    return;
                }
                context.report({ node, message: "Unexpected comment inline with code." });
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                Program() {
                    const comments = sourceCode.getAllComments();
                    comments.filter(token => token.type !== "Shebang").forEach(testCodeAroundComment);
                }
            };
        }
    };
},
/* eslint/lib/rules/no-inner-declarations.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow variable or `function` declarations in nested blocks",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-inner-declarations"
            },
            schema: [
                {
                    enum: ["functions", "both"]
                }
            ]
        },
        create(context) {
            /**
             * Find the nearest Program or Function ancestor node.
             * @returns {Object} Ancestor's type and distance from node.
             */
            function nearestBody() {
                const ancestors = context.getAncestors();
                let ancestor = ancestors.pop(), generation = 1;
                while (ancestor && ["Program", "FunctionDeclaration",
                    "FunctionExpression", "ArrowFunctionExpression"
                ].indexOf(ancestor.type) < 0) {
                    generation += 1;
                    ancestor = ancestors.pop();
                }
                return {
                    // Type of containing ancestor
                    type: ancestor.type,
                    // Separation between ancestor and node
                    distance: generation
                };
            }
            /**
             * Ensure that a given node is at a program or function body's root.
             * @param {ASTNode} node Declaration node to check.
             * @returns {void}
             */
            function check(node) {
                const body = nearestBody(), valid = ((body.type === "Program" && body.distance === 1) ||
                    body.distance === 2);
                if (!valid) {
                    context.report({
                        node,
                        message: "Move {{type}} declaration to {{body}} root.",
                        data: {
                            type: (node.type === "FunctionDeclaration" ? "function" : "variable"),
                            body: (body.type === "Program" ? "program" : "function body")
                        }
                    });
                }
            }
            return {
                FunctionDeclaration: check,
                VariableDeclaration(node) {
                    if (context.options[0] === "both" && node.kind === "var") {
                        check(node);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-invalid-regexp.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const RegExpValidator = require(226) /* regexpp */.RegExpValidator;
    const validator = new RegExpValidator({ ecmaVersion: 2018 });
    const validFlags = /[gimuys]/gu;
    const undefined1 = void 0;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow invalid regular expression strings in `RegExp` constructors",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-invalid-regexp"
            },
            schema: [{
                    type: "object",
                    properties: {
                        allowConstructorFlags: {
                            type: "array",
                            items: {
                                type: "string"
                            }
                        }
                    },
                    additionalProperties: false
                }]
        },
        create(context) {
            const options = context.options[0];
            let allowedFlags = null;
            if (options && options.allowConstructorFlags) {
                const temp = options.allowConstructorFlags.join("").replace(validFlags, "");
                if (temp) {
                    allowedFlags = new RegExp(`[${temp}]`, "giu");
                }
            }
            /**
             * Check if node is a string
             * @param {ASTNode} node node to evaluate
             * @returns {boolean} True if its a string
             * @private
             */
            function isString(node) {
                return node && node.type === "Literal" && typeof node.value === "string";
            }
            /**
             * Check syntax error in a given pattern.
             * @param {string} pattern The RegExp pattern to validate.
             * @param {boolean} uFlag The Unicode flag.
             * @returns {string|null} The syntax error.
             */
            function validateRegExpPattern(pattern, uFlag) {
                try {
                    validator.validatePattern(pattern, undefined1, undefined1, uFlag);
                    return null;
                }
                catch (err) {
                    return err.message;
                }
            }
            /**
             * Check syntax error in a given flags.
             * @param {string} flags The RegExp flags to validate.
             * @returns {string|null} The syntax error.
             */
            function validateRegExpFlags(flags) {
                try {
                    validator.validateFlags(flags);
                    return null;
                }
                catch (err) {
                    return `Invalid flags supplied to RegExp constructor '${flags}'`;
                }
            }
            return {
                "CallExpression, NewExpression"(node) {
                    if (node.callee.type !== "Identifier" || node.callee.name !== "RegExp" || !isString(node.arguments[0])) {
                        return;
                    }
                    const pattern = node.arguments[0].value;
                    let flags = isString(node.arguments[1]) ? node.arguments[1].value : "";
                    if (allowedFlags) {
                        flags = flags.replace(allowedFlags, "");
                    }
                    // If flags are unknown, check both are errored or not.
                    const message = validateRegExpFlags(flags) || (flags
                        ? validateRegExpPattern(pattern, flags.indexOf("u") !== -1)
                        : validateRegExpPattern(pattern, true) && validateRegExpPattern(pattern, false));
                    if (message) {
                        context.report({
                            node,
                            message: "{{message}}.",
                            data: { message }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-invalid-this.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `this` keywords outside of classes or class-like objects",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-invalid-this"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        capIsConstructor: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const options = context.options[0] || {};
            const capIsConstructor = options.capIsConstructor !== false;
            const stack = [], sourceCode = context.getSourceCode();
            /**
             * Gets the current checking context.
             *
             * The return value has a flag that whether or not `this` keyword is valid.
             * The flag is initialized when got at the first time.
             * @returns {{valid: boolean}}
             *   an object which has a flag that whether or not `this` keyword is valid.
             */
            stack.getCurrent = function () {
                const current = this[this.length - 1];
                if (!current.init) {
                    current.init = true;
                    current.valid = !astUtils.isDefaultThisBinding(current.node, sourceCode, { capIsConstructor });
                }
                return current;
            };
            /**
             * Pushs new checking context into the stack.
             *
             * The checking context is not initialized yet.
             * Because most functions don't have `this` keyword.
             * When `this` keyword was found, the checking context is initialized.
             * @param {ASTNode} node A function node that was entered.
             * @returns {void}
             */
            function enterFunction(node) {
                // `this` can be invalid only under strict mode.
                stack.push({
                    init: !context.getScope().isStrict,
                    node,
                    valid: true
                });
            }
            /**
             * Pops the current checking context from the stack.
             * @returns {void}
             */
            function exitFunction() {
                stack.pop();
            }
            return {
                /*
                 * `this` is invalid only under strict mode.
                 * Modules is always strict mode.
                 */
                Program(node) {
                    const scope = context.getScope(), features = context.parserOptions.ecmaFeatures || {};
                    stack.push({
                        init: true,
                        node,
                        valid: !(scope.isStrict ||
                            node.sourceType === "module" ||
                            (features.globalReturn && scope.childScopes[0].isStrict))
                    });
                },
                "Program:exit"() {
                    stack.pop();
                },
                FunctionDeclaration: enterFunction,
                "FunctionDeclaration:exit": exitFunction,
                FunctionExpression: enterFunction,
                "FunctionExpression:exit": exitFunction,
                // Reports if `this` of the current context is invalid.
                ThisExpression(node) {
                    const current = stack.getCurrent();
                    if (current && !current.valid) {
                        context.report({ node, message: "Unexpected 'this'." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-irregular-whitespace.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Constants
    //------------------------------------------------------------------------------
    const ALL_IRREGULARS = /[\f\v\u0085\ufeff\u00a0\u1680\u180e\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200a\u200b\u202f\u205f\u3000\u2028\u2029]/u;
    const IRREGULAR_WHITESPACE = /[\f\v\u0085\ufeff\u00a0\u1680\u180e\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200a\u200b\u202f\u205f\u3000]+/mgu;
    const IRREGULAR_LINE_TERMINATORS = /[\u2028\u2029]/mgu;
    const LINE_BREAK = astUtils.createGlobalLinebreakMatcher();
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow irregular whitespace",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-irregular-whitespace"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        skipComments: {
                            type: "boolean",
                            default: false
                        },
                        skipStrings: {
                            type: "boolean",
                            default: true
                        },
                        skipTemplates: {
                            type: "boolean",
                            default: false
                        },
                        skipRegExps: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            // Module store of errors that we have found
            let errors = [];
            // Lookup the `skipComments` option, which defaults to `false`.
            const options = context.options[0] || {};
            const skipComments = !!options.skipComments;
            const skipStrings = options.skipStrings !== false;
            const skipRegExps = !!options.skipRegExps;
            const skipTemplates = !!options.skipTemplates;
            const sourceCode = context.getSourceCode();
            const commentNodes = sourceCode.getAllComments();
            /**
             * Removes errors that occur inside a string node
             * @param {ASTNode} node to check for matching errors.
             * @returns {void}
             * @private
             */
            function removeWhitespaceError(node) {
                const locStart = node.loc.start;
                const locEnd = node.loc.end;
                errors = errors.filter(({ loc: errorLoc }) => {
                    if (errorLoc.line >= locStart.line && errorLoc.line <= locEnd.line) {
                        if (errorLoc.column >= locStart.column && (errorLoc.column <= locEnd.column || errorLoc.line < locEnd.line)) {
                            return false;
                        }
                    }
                    return true;
                });
            }
            /**
             * Checks identifier or literal nodes for errors that we are choosing to ignore and calls the relevant methods to remove the errors
             * @param {ASTNode} node to check for matching errors.
             * @returns {void}
             * @private
             */
            function removeInvalidNodeErrorsInIdentifierOrLiteral(node) {
                const shouldCheckStrings = skipStrings && (typeof node.value === "string");
                const shouldCheckRegExps = skipRegExps && Boolean(node.regex);
                if (shouldCheckStrings || shouldCheckRegExps) {
                    // If we have irregular characters remove them from the errors list
                    if (ALL_IRREGULARS.test(node.raw)) {
                        removeWhitespaceError(node);
                    }
                }
            }
            /**
             * Checks template string literal nodes for errors that we are choosing to ignore and calls the relevant methods to remove the errors
             * @param {ASTNode} node to check for matching errors.
             * @returns {void}
             * @private
             */
            function removeInvalidNodeErrorsInTemplateLiteral(node) {
                if (typeof node.value.raw === "string") {
                    if (ALL_IRREGULARS.test(node.value.raw)) {
                        removeWhitespaceError(node);
                    }
                }
            }
            /**
             * Checks comment nodes for errors that we are choosing to ignore and calls the relevant methods to remove the errors
             * @param {ASTNode} node to check for matching errors.
             * @returns {void}
             * @private
             */
            function removeInvalidNodeErrorsInComment(node) {
                if (ALL_IRREGULARS.test(node.value)) {
                    removeWhitespaceError(node);
                }
            }
            /**
             * Checks the program source for irregular whitespace
             * @param {ASTNode} node The program node
             * @returns {void}
             * @private
             */
            function checkForIrregularWhitespace(node) {
                const sourceLines = sourceCode.lines;
                sourceLines.forEach((sourceLine, lineIndex) => {
                    const lineNumber = lineIndex + 1;
                    let match;
                    while ((match = IRREGULAR_WHITESPACE.exec(sourceLine)) !== null) {
                        const location = {
                            line: lineNumber,
                            column: match.index
                        };
                        errors.push({ node, message: "Irregular whitespace not allowed.", loc: location });
                    }
                });
            }
            /**
             * Checks the program source for irregular line terminators
             * @param {ASTNode} node The program node
             * @returns {void}
             * @private
             */
            function checkForIrregularLineTerminators(node) {
                const source = sourceCode.getText(), sourceLines = sourceCode.lines, linebreaks = source.match(LINE_BREAK);
                let lastLineIndex = -1, match;
                while ((match = IRREGULAR_LINE_TERMINATORS.exec(source)) !== null) {
                    const lineIndex = linebreaks.indexOf(match[0], lastLineIndex + 1) || 0;
                    const location = {
                        line: lineIndex + 1,
                        column: sourceLines[lineIndex].length
                    };
                    errors.push({ node, message: "Irregular whitespace not allowed.", loc: location });
                    lastLineIndex = lineIndex;
                }
            }
            /**
             * A no-op function to act as placeholder for comment accumulation when the `skipComments` option is `false`.
             * @returns {void}
             * @private
             */
            function noop() { }
            const nodes = {};
            if (ALL_IRREGULARS.test(sourceCode.getText())) {
                nodes.Program = function (node) {
                    /*
                     * As we can easily fire warnings for all white space issues with
                     * all the source its simpler to fire them here.
                     * This means we can check all the application code without having
                     * to worry about issues caused in the parser tokens.
                     * When writing this code also evaluating per node was missing out
                     * connecting tokens in some cases.
                     * We can later filter the errors when they are found to be not an
                     * issue in nodes we don't care about.
                     */
                    checkForIrregularWhitespace(node);
                    checkForIrregularLineTerminators(node);
                };
                nodes.Identifier = removeInvalidNodeErrorsInIdentifierOrLiteral;
                nodes.Literal = removeInvalidNodeErrorsInIdentifierOrLiteral;
                nodes.TemplateElement = skipTemplates ? removeInvalidNodeErrorsInTemplateLiteral : noop;
                nodes["Program:exit"] = function () {
                    if (skipComments) {
                        // First strip errors occurring in comment nodes.
                        commentNodes.forEach(removeInvalidNodeErrorsInComment);
                    }
                    // If we have any errors remaining report on them
                    errors.forEach(error => context.report(error));
                };
            }
            else {
                nodes.Program = noop;
            }
            return nodes;
        }
    };
},
/* eslint/lib/rules/no-iterator.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the use of the `__iterator__` property",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-iterator"
            },
            schema: []
        },
        create(context) {
            return {
                MemberExpression(node) {
                    if (node.property &&
                        (node.property.type === "Identifier" && node.property.name === "__iterator__" && !node.computed) ||
                        (node.property.type === "Literal" && node.property.value === "__iterator__")) {
                        context.report({ node, message: "Reserved name '__iterator__'." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-label-var.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow labels that share a name with a variable",
                category: "Variables",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-label-var"
            },
            schema: []
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Check if the identifier is present inside current scope
             * @param {Object} scope current scope
             * @param {string} name To evaluate
             * @returns {boolean} True if its present
             * @private
             */
            function findIdentifier(scope, name) {
                return astUtils.getVariableByName(scope, name) !== null;
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                LabeledStatement(node) {
                    // Fetch the innermost scope.
                    const scope = context.getScope();
                    /*
                     * Recursively find the identifier walking up the scope, starting
                     * with the innermost scope.
                     */
                    if (findIdentifier(scope, node.label.name)) {
                        context.report({ node, message: "Found identifier with same name as label." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-labels.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow labeled statements",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-labels"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allowLoop: {
                            type: "boolean",
                            default: false
                        },
                        allowSwitch: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const options = context.options[0];
            const allowLoop = options && options.allowLoop;
            const allowSwitch = options && options.allowSwitch;
            let scopeInfo = null;
            /**
             * Gets the kind of a given node.
             * @param {ASTNode} node A node to get.
             * @returns {string} The kind of the node.
             */
            function getBodyKind(node) {
                if (astUtils.isLoop(node)) {
                    return "loop";
                }
                if (node.type === "SwitchStatement") {
                    return "switch";
                }
                return "other";
            }
            /**
             * Checks whether the label of a given kind is allowed or not.
             * @param {string} kind A kind to check.
             * @returns {boolean} `true` if the kind is allowed.
             */
            function isAllowed(kind) {
                switch (kind) {
                    case "loop": return allowLoop;
                    case "switch": return allowSwitch;
                    default: return false;
                }
            }
            /**
             * Checks whether a given name is a label of a loop or not.
             * @param {string} label A name of a label to check.
             * @returns {boolean} `true` if the name is a label of a loop.
             */
            function getKind(label) {
                let info = scopeInfo;
                while (info) {
                    if (info.label === label) {
                        return info.kind;
                    }
                    info = info.upper;
                }
                /* istanbul ignore next: syntax error */
                return "other";
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                LabeledStatement(node) {
                    scopeInfo = {
                        label: node.label.name,
                        kind: getBodyKind(node.body),
                        upper: scopeInfo
                    };
                },
                "LabeledStatement:exit"(node) {
                    if (!isAllowed(scopeInfo.kind)) {
                        context.report({
                            node,
                            message: "Unexpected labeled statement."
                        });
                    }
                    scopeInfo = scopeInfo.upper;
                },
                BreakStatement(node) {
                    if (node.label && !isAllowed(getKind(node.label.name))) {
                        context.report({
                            node,
                            message: "Unexpected label in break statement."
                        });
                    }
                },
                ContinueStatement(node) {
                    if (node.label && !isAllowed(getKind(node.label.name))) {
                        context.report({
                            node,
                            message: "Unexpected label in continue statement."
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-lone-blocks.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary nested blocks",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-lone-blocks"
            },
            schema: []
        },
        create(context) {
            // A stack of lone blocks to be checked for block-level bindings
            const loneBlocks = [];
            let ruleDef;
            /**
             * Reports a node as invalid.
             * @param {ASTNode} node The node to be reported.
             * @returns {void}
             */
            function report(node) {
                const message = node.parent.type === "BlockStatement" ? "Nested block is redundant." : "Block is redundant.";
                context.report({ node, message });
            }
            /**
             * Checks for any ocurrence of a BlockStatement in a place where lists of statements can appear
             * @param {ASTNode} node The node to check
             * @returns {boolean} True if the node is a lone block.
             */
            function isLoneBlock(node) {
                return node.parent.type === "BlockStatement" ||
                    node.parent.type === "Program" ||
                    // Don't report blocks in switch cases if the block is the only statement of the case.
                    node.parent.type === "SwitchCase" && !(node.parent.consequent[0] === node && node.parent.consequent.length === 1);
            }
            /**
             * Checks the enclosing block of the current node for block-level bindings,
             * and "marks it" as valid if any.
             * @returns {void}
             */
            function markLoneBlock() {
                if (loneBlocks.length === 0) {
                    return;
                }
                const block = context.getAncestors().pop();
                if (loneBlocks[loneBlocks.length - 1] === block) {
                    loneBlocks.pop();
                }
            }
            // Default rule definition: report all lone blocks
            ruleDef = {
                BlockStatement(node) {
                    if (isLoneBlock(node)) {
                        report(node);
                    }
                }
            };
            // ES6: report blocks without block-level bindings, or that's only child of another block
            if (context.parserOptions.ecmaVersion >= 6) {
                ruleDef = {
                    BlockStatement(node) {
                        if (isLoneBlock(node)) {
                            loneBlocks.push(node);
                        }
                    },
                    "BlockStatement:exit"(node) {
                        if (loneBlocks.length > 0 && loneBlocks[loneBlocks.length - 1] === node) {
                            loneBlocks.pop();
                            report(node);
                        }
                        else if (node.parent.type === "BlockStatement" &&
                            node.parent.body.length === 1) {
                            report(node);
                        }
                    }
                };
                ruleDef.VariableDeclaration = function (node) {
                    if (node.kind === "let" || node.kind === "const") {
                        markLoneBlock();
                    }
                };
                ruleDef.FunctionDeclaration = function () {
                    if (context.getScope().isStrict) {
                        markLoneBlock();
                    }
                };
                ruleDef.ClassDeclaration = markLoneBlock;
            }
            return ruleDef;
        }
    };
},
/* eslint/lib/rules/no-lonely-if.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `if` statements as the only statement in `else` blocks",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-lonely-if"
            },
            schema: [],
            fixable: "code"
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                IfStatement(node) {
                    const ancestors = context.getAncestors(), parent = ancestors.pop(), grandparent = ancestors.pop();
                    if (parent && parent.type === "BlockStatement" &&
                        parent.body.length === 1 && grandparent &&
                        grandparent.type === "IfStatement" &&
                        parent === grandparent.alternate) {
                        context.report({
                            node,
                            message: "Unexpected if as the only statement in an else block.",
                            fix(fixer) {
                                const openingElseCurly = sourceCode.getFirstToken(parent);
                                const closingElseCurly = sourceCode.getLastToken(parent);
                                const elseKeyword = sourceCode.getTokenBefore(openingElseCurly);
                                const tokenAfterElseBlock = sourceCode.getTokenAfter(closingElseCurly);
                                const lastIfToken = sourceCode.getLastToken(node.consequent);
                                const sourceText = sourceCode.getText();
                                if (sourceText.slice(openingElseCurly.range[1], node.range[0]).trim() || sourceText.slice(node.range[1], closingElseCurly.range[0]).trim()) {
                                    // Don't fix if there are any non-whitespace characters interfering (e.g. comments)
                                    return null;
                                }
                                if (node.consequent.type !== "BlockStatement" && lastIfToken.value !== ";" && tokenAfterElseBlock &&
                                    (node.consequent.loc.end.line === tokenAfterElseBlock.loc.start.line ||
                                        /^[([/+`-]/u.test(tokenAfterElseBlock.value) ||
                                        lastIfToken.value === "++" ||
                                        lastIfToken.value === "--")) {
                                    /*
                                     * If the `if` statement has no block, and is not followed by a semicolon, make sure that fixing
                                     * the issue would not change semantics due to ASI. If this would happen, don't do a fix.
                                     */
                                    return null;
                                }
                                return fixer.replaceTextRange([openingElseCurly.range[0], closingElseCurly.range[1]], (elseKeyword.range[1] === openingElseCurly.range[0] ? " " : "") + sourceCode.getText(node));
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-loop-func.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Gets the containing loop node of a specified node.
     *
     * We don't need to check nested functions, so this ignores those.
     * `Scope.through` contains references of nested functions.
     * @param {ASTNode} node An AST node to get.
     * @returns {ASTNode|null} The containing loop node of the specified node, or
     *      `null`.
     */
    function getContainingLoopNode(node) {
        for (let currentNode = node; currentNode.parent; currentNode = currentNode.parent) {
            const parent = currentNode.parent;
            switch (parent.type) {
                case "WhileStatement":
                case "DoWhileStatement":
                    return parent;
                case "ForStatement":
                    // `init` is outside of the loop.
                    if (parent.init !== currentNode) {
                        return parent;
                    }
                    break;
                case "ForInStatement":
                case "ForOfStatement":
                    // `right` is outside of the loop.
                    if (parent.right !== currentNode) {
                        return parent;
                    }
                    break;
                case "ArrowFunctionExpression":
                case "FunctionExpression":
                case "FunctionDeclaration":
                    // We don't need to check nested functions.
                    return null;
                default:
                    break;
            }
        }
        return null;
    }
    /**
     * Gets the containing loop node of a given node.
     * If the loop was nested, this returns the most outer loop.
     * @param {ASTNode} node A node to get. This is a loop node.
     * @param {ASTNode|null} excludedNode A node that the result node should not
     *      include.
     * @returns {ASTNode} The most outer loop node.
     */
    function getTopLoopNode(node, excludedNode) {
        const border = excludedNode ? excludedNode.range[1] : 0;
        let retv = node;
        let containingLoopNode = node;
        while (containingLoopNode && containingLoopNode.range[0] >= border) {
            retv = containingLoopNode;
            containingLoopNode = getContainingLoopNode(containingLoopNode);
        }
        return retv;
    }
    /**
     * Checks whether a given reference which refers to an upper scope's variable is
     * safe or not.
     * @param {ASTNode} loopNode A containing loop node.
     * @param {eslint-scope.Reference} reference A reference to check.
     * @returns {boolean} `true` if the reference is safe or not.
     */
    function isSafe(loopNode, reference) {
        const variable = reference.resolved;
        const definition = variable && variable.defs[0];
        const declaration = definition && definition.parent;
        const kind = (declaration && declaration.type === "VariableDeclaration")
            ? declaration.kind
            : "";
        // Variables which are declared by `const` is safe.
        if (kind === "const") {
            return true;
        }
        /*
         * Variables which are declared by `let` in the loop is safe.
         * It's a different instance from the next loop step's.
         */
        if (kind === "let" &&
            declaration.range[0] > loopNode.range[0] &&
            declaration.range[1] < loopNode.range[1]) {
            return true;
        }
        /*
         * WriteReferences which exist after this border are unsafe because those
         * can modify the variable.
         */
        const border = getTopLoopNode(loopNode, (kind === "let") ? declaration : null).range[0];
        /**
         * Checks whether a given reference is safe or not.
         * The reference is every reference of the upper scope's variable we are
         * looking now.
         *
         * It's safeafe if the reference matches one of the following condition.
         * - is readonly.
         * - doesn't exist inside a local function and after the border.
         * @param {eslint-scope.Reference} upperRef A reference to check.
         * @returns {boolean} `true` if the reference is safe.
         */
        function isSafeReference(upperRef) {
            const id = upperRef.identifier;
            return (!upperRef.isWrite() ||
                variable.scope.variableScope === upperRef.from.variableScope &&
                    id.range[0] < border);
        }
        return Boolean(variable) && variable.references.every(isSafeReference);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow function declarations that contain unsafe references inside loop statements",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-loop-func"
            },
            schema: [],
            messages: {
                unsafeRefs: "Function declared in a loop contains unsafe references to variable(s) {{ varNames }}."
            }
        },
        create(context) {
            /**
             * Reports functions which match the following condition:
             *
             * - has a loop node in ancestors.
             * - has any references which refers to an unsafe variable.
             * @param {ASTNode} node The AST node to check.
             * @returns {boolean} Whether or not the node is within a loop.
             */
            function checkForLoops(node) {
                const loopNode = getContainingLoopNode(node);
                if (!loopNode) {
                    return;
                }
                const references = context.getScope().through;
                const unsafeRefs = references.filter(r => !isSafe(loopNode, r)).map(r => r.identifier.name);
                if (unsafeRefs.length > 0) {
                    context.report({
                        node,
                        messageId: "unsafeRefs",
                        data: { varNames: `'${unsafeRefs.join("', '")}'` }
                    });
                }
            }
            return {
                ArrowFunctionExpression: checkForLoops,
                FunctionExpression: checkForLoops,
                FunctionDeclaration: checkForLoops
            };
        }
    };
},
/* eslint/lib/rules/no-magic-numbers.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow magic numbers",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-magic-numbers"
            },
            schema: [{
                    type: "object",
                    properties: {
                        detectObjects: {
                            type: "boolean",
                            default: false
                        },
                        enforceConst: {
                            type: "boolean",
                            default: false
                        },
                        ignore: {
                            type: "array",
                            items: {
                                type: "number"
                            },
                            uniqueItems: true
                        },
                        ignoreArrayIndexes: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }],
            messages: {
                useConst: "Number constants declarations must use 'const'.",
                noMagic: "No magic number: {{raw}}."
            }
        },
        create(context) {
            const config = context.options[0] || {}, detectObjects = !!config.detectObjects, enforceConst = !!config.enforceConst, ignore = config.ignore || [], ignoreArrayIndexes = !!config.ignoreArrayIndexes;
            /**
             * Returns whether the node is number literal
             * @param {Node} node the node literal being evaluated
             * @returns {boolean} true if the node is a number literal
             */
            function isNumber(node) {
                return typeof node.value === "number";
            }
            /**
             * Returns whether the number should be ignored
             * @param {number} num the number
             * @returns {boolean} true if the number should be ignored
             */
            function shouldIgnoreNumber(num) {
                return ignore.indexOf(num) !== -1;
            }
            /**
             * Returns whether the number should be ignored when used as a radix within parseInt() or Number.parseInt()
             * @param {ASTNode} parent the non-"UnaryExpression" parent
             * @param {ASTNode} node the node literal being evaluated
             * @returns {boolean} true if the number should be ignored
             */
            function shouldIgnoreParseInt(parent, node) {
                return parent.type === "CallExpression" && node === parent.arguments[1] &&
                    (parent.callee.name === "parseInt" ||
                        parent.callee.type === "MemberExpression" &&
                            parent.callee.object.name === "Number" &&
                            parent.callee.property.name === "parseInt");
            }
            /**
             * Returns whether the number should be ignored when used to define a JSX prop
             * @param {ASTNode} parent the non-"UnaryExpression" parent
             * @returns {boolean} true if the number should be ignored
             */
            function shouldIgnoreJSXNumbers(parent) {
                return parent.type.indexOf("JSX") === 0;
            }
            /**
             * Returns whether the number should be ignored when used as an array index with enabled 'ignoreArrayIndexes' option.
             * @param {ASTNode} parent the non-"UnaryExpression" parent.
             * @returns {boolean} true if the number should be ignored
             */
            function shouldIgnoreArrayIndexes(parent) {
                return parent.type === "MemberExpression" && ignoreArrayIndexes;
            }
            return {
                Literal(node) {
                    const okTypes = detectObjects ? [] : ["ObjectExpression", "Property", "AssignmentExpression"];
                    if (!isNumber(node)) {
                        return;
                    }
                    let fullNumberNode;
                    let parent;
                    let value;
                    let raw;
                    // For negative magic numbers: update the value and parent node
                    if (node.parent.type === "UnaryExpression" && node.parent.operator === "-") {
                        fullNumberNode = node.parent;
                        parent = fullNumberNode.parent;
                        value = -node.value;
                        raw = `-${node.raw}`;
                    }
                    else {
                        fullNumberNode = node;
                        parent = node.parent;
                        value = node.value;
                        raw = node.raw;
                    }
                    if (shouldIgnoreNumber(value) ||
                        shouldIgnoreParseInt(parent, fullNumberNode) ||
                        shouldIgnoreArrayIndexes(parent) ||
                        shouldIgnoreJSXNumbers(parent)) {
                        return;
                    }
                    if (parent.type === "VariableDeclarator") {
                        if (enforceConst && parent.parent.kind !== "const") {
                            context.report({
                                node: fullNumberNode,
                                messageId: "useConst"
                            });
                        }
                    }
                    else if (okTypes.indexOf(parent.type) === -1 ||
                        (parent.type === "AssignmentExpression" && parent.left.type === "Identifier")) {
                        context.report({
                            node: fullNumberNode,
                            messageId: "noMagic",
                            data: {
                                raw
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-misleading-character-class.js */ function _(require, module, exports) {
    const { CALL, CONSTRUCT, ReferenceTracker, getStringIfConstant } = require(250) /* eslint-utils */;
    const { RegExpParser, visitRegExpAST } = require(226) /* regexpp */;
    const { isCombiningCharacter, isEmojiModifier, isRegionalIndicatorSymbol, isSurrogatePair } = require(273) /* ./utils/unicode */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Iterate character sequences of a given nodes.
     *
     * CharacterClassRange syntax can steal a part of character sequence,
     * so this function reverts CharacterClassRange syntax and restore the sequence.
     * @param {regexpp.AST.CharacterClassElement[]} nodes The node list to iterate character sequences.
     * @returns {IterableIterator<number[]>} The list of character sequences.
     */
    function* iterateCharacterSequence(nodes) {
        let seq = [];
        for (const node of nodes) {
            switch (node.type) {
                case "Character":
                    seq.push(node.value);
                    break;
                case "CharacterClassRange":
                    seq.push(node.min.value);
                    yield seq;
                    seq = [node.max.value];
                    break;
                case "CharacterSet":
                    if (seq.length > 0) {
                        yield seq;
                        seq = [];
                    }
                    break;
                // no default
            }
        }
        if (seq.length > 0) {
            yield seq;
        }
    }
    const hasCharacterSequence = {
        surrogatePairWithoutUFlag(chars) {
            return chars.some((c, i) => i !== 0 && isSurrogatePair(chars[i - 1], c));
        },
        combiningClass(chars) {
            return chars.some((c, i) => (i !== 0 &&
                isCombiningCharacter(c) &&
                !isCombiningCharacter(chars[i - 1])));
        },
        emojiModifier(chars) {
            return chars.some((c, i) => (i !== 0 &&
                isEmojiModifier(c) &&
                !isEmojiModifier(chars[i - 1])));
        },
        regionalIndicatorSymbol(chars) {
            return chars.some((c, i) => (i !== 0 &&
                isRegionalIndicatorSymbol(c) &&
                isRegionalIndicatorSymbol(chars[i - 1])));
        },
        zwj(chars) {
            const lastIndex = chars.length - 1;
            return chars.some((c, i) => (i !== 0 &&
                i !== lastIndex &&
                c === 0x200d &&
                chars[i - 1] !== 0x200d &&
                chars[i + 1] !== 0x200d));
        }
    };
    const kinds = Object.keys(hasCharacterSequence);
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow characters which are made with multiple code points in character class syntax",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-misleading-character-class"
            },
            schema: [],
            messages: {
                surrogatePairWithoutUFlag: "Unexpected surrogate pair in character class. Use 'u' flag.",
                combiningClass: "Unexpected combined character in character class.",
                emojiModifier: "Unexpected modified Emoji in character class.",
                regionalIndicatorSymbol: "Unexpected national flag in character class.",
                zwj: "Unexpected joined character sequence in character class."
            }
        },
        create(context) {
            const parser = new RegExpParser();
            /**
             * Verify a given regular expression.
             * @param {Node} node The node to report.
             * @param {string} pattern The regular expression pattern to verify.
             * @param {string} flags The flags of the regular expression.
             * @returns {void}
             */
            function verify(node, pattern, flags) {
                const has = {
                    surrogatePairWithoutUFlag: false,
                    combiningClass: false,
                    variationSelector: false,
                    emojiModifier: false,
                    regionalIndicatorSymbol: false,
                    zwj: false
                };
                let patternNode;
                try {
                    patternNode = parser.parsePattern(pattern, 0, pattern.length, flags.includes("u"));
                }
                catch (e) {
                    // Ignore regular expressions with syntax errors
                    return;
                }
                visitRegExpAST(patternNode, {
                    onCharacterClassEnter(ccNode) {
                        for (const chars of iterateCharacterSequence(ccNode.elements)) {
                            for (const kind of kinds) {
                                has[kind] = has[kind] || hasCharacterSequence[kind](chars);
                            }
                        }
                    }
                });
                for (const kind of kinds) {
                    if (has[kind]) {
                        context.report({ node, messageId: kind });
                    }
                }
            }
            return {
                "Literal[regex]"(node) {
                    verify(node, node.regex.pattern, node.regex.flags);
                },
                "Program"() {
                    const scope = context.getScope();
                    const tracker = new ReferenceTracker(scope);
                    /*
                     * Iterate calls of RegExp.
                     * E.g., `new RegExp()`, `RegExp()`, `new window.RegExp()`,
                     *       `const {RegExp: a} = window; new a()`, etc...
                     */
                    for (const { node } of tracker.iterateGlobalReferences({
                        RegExp: { [CALL]: true, [CONSTRUCT]: true }
                    })) {
                        const [patternNode, flagsNode] = node.arguments;
                        const pattern = getStringIfConstant(patternNode, scope);
                        const flags = getStringIfConstant(flagsNode, scope);
                        if (typeof pattern === "string") {
                            verify(node, pattern, flags || "");
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/utils/unicode/index.js */ function _(require, module, exports) {
    module.exports = {
        isCombiningCharacter: require(274) /* ./is-combining-character */,
        isEmojiModifier: require(275) /* ./is-emoji-modifier */,
        isRegionalIndicatorSymbol: require(276) /* ./is-regional-indicator-symbol */,
        isSurrogatePair: require(277) /* ./is-surrogate-pair */
    };
},
/* eslint/lib/rules/utils/unicode/is-combining-character.js */ function _(require, module, exports) {
    const combiningChars = new Set([768, 769, 770, 771, 772, 773, 774, 775, 776, 777, 778, 779, 780, 781, 782, 783, 784, 785, 786, 787, 788, 789, 790, 791, 792, 793, 794, 795, 796, 797, 798, 799, 800, 801, 802, 803, 804, 805, 806, 807, 808, 809, 810, 811, 812, 813, 814, 815, 816, 817, 818, 819, 820, 821, 822, 823, 824, 825, 826, 827, 828, 829, 830, 831, 832, 833, 834, 835, 836, 837, 838, 839, 840, 841, 842, 843, 844, 845, 846, 847, 848, 849, 850, 851, 852, 853, 854, 855, 856, 857, 858, 859, 860, 861, 862, 863, 864, 865, 866, 867, 868, 869, 870, 871, 872, 873, 874, 875, 876, 877, 878, 879, 1155, 1156, 1157, 1158, 1159, 1160, 1161, 1425, 1426, 1427, 1428, 1429, 1430, 1431, 1432, 1433, 1434, 1435, 1436, 1437, 1438, 1439, 1440, 1441, 1442, 1443, 1444, 1445, 1446, 1447, 1448, 1449, 1450, 1451, 1452, 1453, 1454, 1455, 1456, 1457, 1458, 1459, 1460, 1461, 1462, 1463, 1464, 1465, 1466, 1467, 1468, 1469, 1471, 1473, 1474, 1476, 1477, 1479, 1552, 1553, 1554, 1555, 1556, 1557, 1558, 1559, 1560, 1561, 1562, 1611, 1612, 1613, 1614, 1615, 1616, 1617, 1618, 1619, 1620, 1621, 1622, 1623, 1624, 1625, 1626, 1627, 1628, 1629, 1630, 1631, 1648, 1750, 1751, 1752, 1753, 1754, 1755, 1756, 1759, 1760, 1761, 1762, 1763, 1764, 1767, 1768, 1770, 1771, 1772, 1773, 1809, 1840, 1841, 1842, 1843, 1844, 1845, 1846, 1847, 1848, 1849, 1850, 1851, 1852, 1853, 1854, 1855, 1856, 1857, 1858, 1859, 1860, 1861, 1862, 1863, 1864, 1865, 1866, 1958, 1959, 1960, 1961, 1962, 1963, 1964, 1965, 1966, 1967, 1968, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034, 2035, 2070, 2071, 2072, 2073, 2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2085, 2086, 2087, 2089, 2090, 2091, 2092, 2093, 2137, 2138, 2139, 2260, 2261, 2262, 2263, 2264, 2265, 2266, 2267, 2268, 2269, 2270, 2271, 2272, 2273, 2275, 2276, 2277, 2278, 2279, 2280, 2281, 2282, 2283, 2284, 2285, 2286, 2287, 2288, 2289, 2290, 2291, 2292, 2293, 2294, 2295, 2296, 2297, 2298, 2299, 2300, 2301, 2302, 2303, 2304, 2305, 2306, 2307, 2362, 2363, 2364, 2366, 2367, 2368, 2369, 2370, 2371, 2372, 2373, 2374, 2375, 2376, 2377, 2378, 2379, 2380, 2381, 2382, 2383, 2385, 2386, 2387, 2388, 2389, 2390, 2391, 2402, 2403, 2433, 2434, 2435, 2492, 2494, 2495, 2496, 2497, 2498, 2499, 2500, 2503, 2504, 2507, 2508, 2509, 2519, 2530, 2531, 2561, 2562, 2563, 2620, 2622, 2623, 2624, 2625, 2626, 2631, 2632, 2635, 2636, 2637, 2641, 2672, 2673, 2677, 2689, 2690, 2691, 2748, 2750, 2751, 2752, 2753, 2754, 2755, 2756, 2757, 2759, 2760, 2761, 2763, 2764, 2765, 2786, 2787, 2810, 2811, 2812, 2813, 2814, 2815, 2817, 2818, 2819, 2876, 2878, 2879, 2880, 2881, 2882, 2883, 2884, 2887, 2888, 2891, 2892, 2893, 2902, 2903, 2914, 2915, 2946, 3006, 3007, 3008, 3009, 3010, 3014, 3015, 3016, 3018, 3019, 3020, 3021, 3031, 3072, 3073, 3074, 3075, 3134, 3135, 3136, 3137, 3138, 3139, 3140, 3142, 3143, 3144, 3146, 3147, 3148, 3149, 3157, 3158, 3170, 3171, 3201, 3202, 3203, 3260, 3262, 3263, 3264, 3265, 3266, 3267, 3268, 3270, 3271, 3272, 3274, 3275, 3276, 3277, 3285, 3286, 3298, 3299, 3328, 3329, 3330, 3331, 3387, 3388, 3390, 3391, 3392, 3393, 3394, 3395, 3396, 3398, 3399, 3400, 3402, 3403, 3404, 3405, 3415, 3426, 3427, 3458, 3459, 3530, 3535, 3536, 3537, 3538, 3539, 3540, 3542, 3544, 3545, 3546, 3547, 3548, 3549, 3550, 3551, 3570, 3571, 3633, 3636, 3637, 3638, 3639, 3640, 3641, 3642, 3655, 3656, 3657, 3658, 3659, 3660, 3661, 3662, 3761, 3764, 3765, 3766, 3767, 3768, 3769, 3771, 3772, 3784, 3785, 3786, 3787, 3788, 3789, 3864, 3865, 3893, 3895, 3897, 3902, 3903, 3953, 3954, 3955, 3956, 3957, 3958, 3959, 3960, 3961, 3962, 3963, 3964, 3965, 3966, 3967, 3968, 3969, 3970, 3971, 3972, 3974, 3975, 3981, 3982, 3983, 3984, 3985, 3986, 3987, 3988, 3989, 3990, 3991, 3993, 3994, 3995, 3996, 3997, 3998, 3999, 4000, 4001, 4002, 4003, 4004, 4005, 4006, 4007, 4008, 4009, 4010, 4011, 4012, 4013, 4014, 4015, 4016, 4017, 4018, 4019, 4020, 4021, 4022, 4023, 4024, 4025, 4026, 4027, 4028, 4038, 4139, 4140, 4141, 4142, 4143, 4144, 4145, 4146, 4147, 4148, 4149, 4150, 4151, 4152, 4153, 4154, 4155, 4156, 4157, 4158, 4182, 4183, 4184, 4185, 4190, 4191, 4192, 4194, 4195, 4196, 4199, 4200, 4201, 4202, 4203, 4204, 4205, 4209, 4210, 4211, 4212, 4226, 4227, 4228, 4229, 4230, 4231, 4232, 4233, 4234, 4235, 4236, 4237, 4239, 4250, 4251, 4252, 4253, 4957, 4958, 4959, 5906, 5907, 5908, 5938, 5939, 5940, 5970, 5971, 6002, 6003, 6068, 6069, 6070, 6071, 6072, 6073, 6074, 6075, 6076, 6077, 6078, 6079, 6080, 6081, 6082, 6083, 6084, 6085, 6086, 6087, 6088, 6089, 6090, 6091, 6092, 6093, 6094, 6095, 6096, 6097, 6098, 6099, 6109, 6155, 6156, 6157, 6277, 6278, 6313, 6432, 6433, 6434, 6435, 6436, 6437, 6438, 6439, 6440, 6441, 6442, 6443, 6448, 6449, 6450, 6451, 6452, 6453, 6454, 6455, 6456, 6457, 6458, 6459, 6679, 6680, 6681, 6682, 6683, 6741, 6742, 6743, 6744, 6745, 6746, 6747, 6748, 6749, 6750, 6752, 6753, 6754, 6755, 6756, 6757, 6758, 6759, 6760, 6761, 6762, 6763, 6764, 6765, 6766, 6767, 6768, 6769, 6770, 6771, 6772, 6773, 6774, 6775, 6776, 6777, 6778, 6779, 6780, 6783, 6832, 6833, 6834, 6835, 6836, 6837, 6838, 6839, 6840, 6841, 6842, 6843, 6844, 6845, 6846, 6912, 6913, 6914, 6915, 6916, 6964, 6965, 6966, 6967, 6968, 6969, 6970, 6971, 6972, 6973, 6974, 6975, 6976, 6977, 6978, 6979, 6980, 7019, 7020, 7021, 7022, 7023, 7024, 7025, 7026, 7027, 7040, 7041, 7042, 7073, 7074, 7075, 7076, 7077, 7078, 7079, 7080, 7081, 7082, 7083, 7084, 7085, 7142, 7143, 7144, 7145, 7146, 7147, 7148, 7149, 7150, 7151, 7152, 7153, 7154, 7155, 7204, 7205, 7206, 7207, 7208, 7209, 7210, 7211, 7212, 7213, 7214, 7215, 7216, 7217, 7218, 7219, 7220, 7221, 7222, 7223, 7376, 7377, 7378, 7380, 7381, 7382, 7383, 7384, 7385, 7386, 7387, 7388, 7389, 7390, 7391, 7392, 7393, 7394, 7395, 7396, 7397, 7398, 7399, 7400, 7405, 7410, 7411, 7412, 7415, 7416, 7417, 7616, 7617, 7618, 7619, 7620, 7621, 7622, 7623, 7624, 7625, 7626, 7627, 7628, 7629, 7630, 7631, 7632, 7633, 7634, 7635, 7636, 7637, 7638, 7639, 7640, 7641, 7642, 7643, 7644, 7645, 7646, 7647, 7648, 7649, 7650, 7651, 7652, 7653, 7654, 7655, 7656, 7657, 7658, 7659, 7660, 7661, 7662, 7663, 7664, 7665, 7666, 7667, 7668, 7669, 7670, 7671, 7672, 7673, 7675, 7676, 7677, 7678, 7679, 8400, 8401, 8402, 8403, 8404, 8405, 8406, 8407, 8408, 8409, 8410, 8411, 8412, 8413, 8414, 8415, 8416, 8417, 8418, 8419, 8420, 8421, 8422, 8423, 8424, 8425, 8426, 8427, 8428, 8429, 8430, 8431, 8432, 11503, 11504, 11505, 11647, 11744, 11745, 11746, 11747, 11748, 11749, 11750, 11751, 11752, 11753, 11754, 11755, 11756, 11757, 11758, 11759, 11760, 11761, 11762, 11763, 11764, 11765, 11766, 11767, 11768, 11769, 11770, 11771, 11772, 11773, 11774, 11775, 12330, 12331, 12332, 12333, 12334, 12335, 12441, 12442, 42607, 42608, 42609, 42610, 42612, 42613, 42614, 42615, 42616, 42617, 42618, 42619, 42620, 42621, 42654, 42655, 42736, 42737, 43010, 43014, 43019, 43043, 43044, 43045, 43046, 43047, 43136, 43137, 43188, 43189, 43190, 43191, 43192, 43193, 43194, 43195, 43196, 43197, 43198, 43199, 43200, 43201, 43202, 43203, 43204, 43205, 43232, 43233, 43234, 43235, 43236, 43237, 43238, 43239, 43240, 43241, 43242, 43243, 43244, 43245, 43246, 43247, 43248, 43249, 43302, 43303, 43304, 43305, 43306, 43307, 43308, 43309, 43335, 43336, 43337, 43338, 43339, 43340, 43341, 43342, 43343, 43344, 43345, 43346, 43347, 43392, 43393, 43394, 43395, 43443, 43444, 43445, 43446, 43447, 43448, 43449, 43450, 43451, 43452, 43453, 43454, 43455, 43456, 43493, 43561, 43562, 43563, 43564, 43565, 43566, 43567, 43568, 43569, 43570, 43571, 43572, 43573, 43574, 43587, 43596, 43597, 43643, 43644, 43645, 43696, 43698, 43699, 43700, 43703, 43704, 43710, 43711, 43713, 43755, 43756, 43757, 43758, 43759, 43765, 43766, 44003, 44004, 44005, 44006, 44007, 44008, 44009, 44010, 44012, 44013, 64286, 65024, 65025, 65026, 65027, 65028, 65029, 65030, 65031, 65032, 65033, 65034, 65035, 65036, 65037, 65038, 65039, 65056, 65057, 65058, 65059, 65060, 65061, 65062, 65063, 65064, 65065, 65066, 65067, 65068, 65069, 65070, 65071, 66045, 66272, 66422, 66423, 66424, 66425, 66426, 68097, 68098, 68099, 68101, 68102, 68108, 68109, 68110, 68111, 68152, 68153, 68154, 68159, 68325, 68326, 69632, 69633, 69634, 69688, 69689, 69690, 69691, 69692, 69693, 69694, 69695, 69696, 69697, 69698, 69699, 69700, 69701, 69702, 69759, 69760, 69761, 69762, 69808, 69809, 69810, 69811, 69812, 69813, 69814, 69815, 69816, 69817, 69818, 69888, 69889, 69890, 69927, 69928, 69929, 69930, 69931, 69932, 69933, 69934, 69935, 69936, 69937, 69938, 69939, 69940, 70003, 70016, 70017, 70018, 70067, 70068, 70069, 70070, 70071, 70072, 70073, 70074, 70075, 70076, 70077, 70078, 70079, 70080, 70090, 70091, 70092, 70188, 70189, 70190, 70191, 70192, 70193, 70194, 70195, 70196, 70197, 70198, 70199, 70206, 70367, 70368, 70369, 70370, 70371, 70372, 70373, 70374, 70375, 70376, 70377, 70378, 70400, 70401, 70402, 70403, 70460, 70462, 70463, 70464, 70465, 70466, 70467, 70468, 70471, 70472, 70475, 70476, 70477, 70487, 70498, 70499, 70502, 70503, 70504, 70505, 70506, 70507, 70508, 70512, 70513, 70514, 70515, 70516, 70709, 70710, 70711, 70712, 70713, 70714, 70715, 70716, 70717, 70718, 70719, 70720, 70721, 70722, 70723, 70724, 70725, 70726, 70832, 70833, 70834, 70835, 70836, 70837, 70838, 70839, 70840, 70841, 70842, 70843, 70844, 70845, 70846, 70847, 70848, 70849, 70850, 70851, 71087, 71088, 71089, 71090, 71091, 71092, 71093, 71096, 71097, 71098, 71099, 71100, 71101, 71102, 71103, 71104, 71132, 71133, 71216, 71217, 71218, 71219, 71220, 71221, 71222, 71223, 71224, 71225, 71226, 71227, 71228, 71229, 71230, 71231, 71232, 71339, 71340, 71341, 71342, 71343, 71344, 71345, 71346, 71347, 71348, 71349, 71350, 71351, 71453, 71454, 71455, 71456, 71457, 71458, 71459, 71460, 71461, 71462, 71463, 71464, 71465, 71466, 71467, 72193, 72194, 72195, 72196, 72197, 72198, 72199, 72200, 72201, 72202, 72243, 72244, 72245, 72246, 72247, 72248, 72249, 72251, 72252, 72253, 72254, 72263, 72273, 72274, 72275, 72276, 72277, 72278, 72279, 72280, 72281, 72282, 72283, 72330, 72331, 72332, 72333, 72334, 72335, 72336, 72337, 72338, 72339, 72340, 72341, 72342, 72343, 72344, 72345, 72751, 72752, 72753, 72754, 72755, 72756, 72757, 72758, 72760, 72761, 72762, 72763, 72764, 72765, 72766, 72767, 72850, 72851, 72852, 72853, 72854, 72855, 72856, 72857, 72858, 72859, 72860, 72861, 72862, 72863, 72864, 72865, 72866, 72867, 72868, 72869, 72870, 72871, 72873, 72874, 72875, 72876, 72877, 72878, 72879, 72880, 72881, 72882, 72883, 72884, 72885, 72886, 73009, 73010, 73011, 73012, 73013, 73014, 73018, 73020, 73021, 73023, 73024, 73025, 73026, 73027, 73028, 73029, 73031, 92912, 92913, 92914, 92915, 92916, 92976, 92977, 92978, 92979, 92980, 92981, 92982, 94033, 94034, 94035, 94036, 94037, 94038, 94039, 94040, 94041, 94042, 94043, 94044, 94045, 94046, 94047, 94048, 94049, 94050, 94051, 94052, 94053, 94054, 94055, 94056, 94057, 94058, 94059, 94060, 94061, 94062, 94063, 94064, 94065, 94066, 94067, 94068, 94069, 94070, 94071, 94072, 94073, 94074, 94075, 94076, 94077, 94078, 94095, 94096, 94097, 94098, 113821, 113822, 119141, 119142, 119143, 119144, 119145, 119149, 119150, 119151, 119152, 119153, 119154, 119163, 119164, 119165, 119166, 119167, 119168, 119169, 119170, 119173, 119174, 119175, 119176, 119177, 119178, 119179, 119210, 119211, 119212, 119213, 119362, 119363, 119364, 121344, 121345, 121346, 121347, 121348, 121349, 121350, 121351, 121352, 121353, 121354, 121355, 121356, 121357, 121358, 121359, 121360, 121361, 121362, 121363, 121364, 121365, 121366, 121367, 121368, 121369, 121370, 121371, 121372, 121373, 121374, 121375, 121376, 121377, 121378, 121379, 121380, 121381, 121382, 121383, 121384, 121385, 121386, 121387, 121388, 121389, 121390, 121391, 121392, 121393, 121394, 121395, 121396, 121397, 121398, 121403, 121404, 121405, 121406, 121407, 121408, 121409, 121410, 121411, 121412, 121413, 121414, 121415, 121416, 121417, 121418, 121419, 121420, 121421, 121422, 121423, 121424, 121425, 121426, 121427, 121428, 121429, 121430, 121431, 121432, 121433, 121434, 121435, 121436, 121437, 121438, 121439, 121440, 121441, 121442, 121443, 121444, 121445, 121446, 121447, 121448, 121449, 121450, 121451, 121452, 121461, 121476, 121499, 121500, 121501, 121502, 121503, 121505, 121506, 121507, 121508, 121509, 121510, 121511, 121512, 121513, 121514, 121515, 121516, 121517, 121518, 121519, 122880, 122881, 122882, 122883, 122884, 122885, 122886, 122888, 122889, 122890, 122891, 122892, 122893, 122894, 122895, 122896, 122897, 122898, 122899, 122900, 122901, 122902, 122903, 122904, 122907, 122908, 122909, 122910, 122911, 122912, 122913, 122915, 122916, 122918, 122919, 122920, 122921, 122922, 125136, 125137, 125138, 125139, 125140, 125141, 125142, 125252, 125253, 125254, 125255, 125256, 125257, 125258, 917760, 917761, 917762, 917763, 917764, 917765, 917766, 917767, 917768, 917769, 917770, 917771, 917772, 917773, 917774, 917775, 917776, 917777, 917778, 917779, 917780, 917781, 917782, 917783, 917784, 917785, 917786, 917787, 917788, 917789, 917790, 917791, 917792, 917793, 917794, 917795, 917796, 917797, 917798, 917799, 917800, 917801, 917802, 917803, 917804, 917805, 917806, 917807, 917808, 917809, 917810, 917811, 917812, 917813, 917814, 917815, 917816, 917817, 917818, 917819, 917820, 917821, 917822, 917823, 917824, 917825, 917826, 917827, 917828, 917829, 917830, 917831, 917832, 917833, 917834, 917835, 917836, 917837, 917838, 917839, 917840, 917841, 917842, 917843, 917844, 917845, 917846, 917847, 917848, 917849, 917850, 917851, 917852, 917853, 917854, 917855, 917856, 917857, 917858, 917859, 917860, 917861, 917862, 917863, 917864, 917865, 917866, 917867, 917868, 917869, 917870, 917871, 917872, 917873, 917874, 917875, 917876, 917877, 917878, 917879, 917880, 917881, 917882, 917883, 917884, 917885, 917886, 917887, 917888, 917889, 917890, 917891, 917892, 917893, 917894, 917895, 917896, 917897, 917898, 917899, 917900, 917901, 917902, 917903, 917904, 917905, 917906, 917907, 917908, 917909, 917910, 917911, 917912, 917913, 917914, 917915, 917916, 917917, 917918, 917919, 917920, 917921, 917922, 917923, 917924, 917925, 917926, 917927, 917928, 917929, 917930, 917931, 917932, 917933, 917934, 917935, 917936, 917937, 917938, 917939, 917940, 917941, 917942, 917943, 917944, 917945, 917946, 917947, 917948, 917949, 917950, 917951, 917952, 917953, 917954, 917955, 917956, 917957, 917958, 917959, 917960, 917961, 917962, 917963, 917964, 917965, 917966, 917967, 917968, 917969, 917970, 917971, 917972, 917973, 917974, 917975, 917976, 917977, 917978, 917979, 917980, 917981, 917982, 917983, 917984, 917985, 917986, 917987, 917988, 917989, 917990, 917991, 917992, 917993, 917994, 917995, 917996, 917997, 917998, 917999]);
    /**
     * Check whether a given character is a combining mark or not.
     * @param {number} c The character code to check.
     * @returns {boolean} `true` if the character belongs to the category, one of `Mc`, `Me`, and `Mn`.
     */
    module.exports = function isCombiningCharacter(c) {
        return combiningChars.has(c);
    };
},
/* eslint/lib/rules/utils/unicode/is-emoji-modifier.js */ function _(require, module, exports) {
    /**
     * Check whether a given character is an emoji modifier.
     * @param {number} code The character code to check.
     * @returns {boolean} `true` if the character is an emoji modifier.
     */
    module.exports = function isEmojiModifier(code) {
        return code >= 0x1F3FB && code <= 0x1F3FF;
    };
},
/* eslint/lib/rules/utils/unicode/is-regional-indicator-symbol.js */ function _(require, module, exports) {
    /**
     * Check whether a given character is a regional indicator symbol.
     * @param {number} code The character code to check.
     * @returns {boolean} `true` if the character is a regional indicator symbol.
     */
    module.exports = function isRegionalIndicatorSymbol(code) {
        return code >= 0x1F1E6 && code <= 0x1F1FF;
    };
},
/* eslint/lib/rules/utils/unicode/is-surrogate-pair.js */ function _(require, module, exports) {
    /**
     * Check whether given two characters are a surrogate pair.
     * @param {number} lead The code of the lead character.
     * @param {number} tail The code of the tail character.
     * @returns {boolean} `true` if the character pair is a surrogate pair.
     */
    module.exports = function isSurrogatePair(lead, tail) {
        return lead >= 0xD800 && lead < 0xDC00 && tail >= 0xDC00 && tail < 0xE000;
    };
},
/* eslint/lib/rules/no-mixed-operators.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils.js */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const ARITHMETIC_OPERATORS = ["+", "-", "*", "/", "%", "**"];
    const BITWISE_OPERATORS = ["&", "|", "^", "~", "<<", ">>", ">>>"];
    const COMPARISON_OPERATORS = ["==", "!=", "===", "!==", ">", ">=", "<", "<="];
    const LOGICAL_OPERATORS = ["&&", "||"];
    const RELATIONAL_OPERATORS = ["in", "instanceof"];
    const TERNARY_OPERATOR = ["?:"];
    const ALL_OPERATORS = [].concat(ARITHMETIC_OPERATORS, BITWISE_OPERATORS, COMPARISON_OPERATORS, LOGICAL_OPERATORS, RELATIONAL_OPERATORS, TERNARY_OPERATOR);
    const DEFAULT_GROUPS = [
        ARITHMETIC_OPERATORS,
        BITWISE_OPERATORS,
        COMPARISON_OPERATORS,
        LOGICAL_OPERATORS,
        RELATIONAL_OPERATORS
    ];
    const TARGET_NODE_TYPE = /^(?:Binary|Logical|Conditional)Expression$/u;
    /**
     * Normalizes options.
     * @param {Object|undefined} options A options object to normalize.
     * @returns {Object} Normalized option object.
     */
    function normalizeOptions(options = {}) {
        const hasGroups = options.groups && options.groups.length > 0;
        const groups = hasGroups ? options.groups : DEFAULT_GROUPS;
        const allowSamePrecedence = options.allowSamePrecedence !== false;
        return {
            groups,
            allowSamePrecedence
        };
    }
    /**
     * Checks whether any group which includes both given operator exists or not.
     * @param {Array.<string[]>} groups A list of groups to check.
     * @param {string} left An operator.
     * @param {string} right Another operator.
     * @returns {boolean} `true` if such group existed.
     */
    function includesBothInAGroup(groups, left, right) {
        return groups.some(group => group.indexOf(left) !== -1 && group.indexOf(right) !== -1);
    }
    /**
     * Checks whether the given node is a conditional expression and returns the test node else the left node.
     * @param {ASTNode} node A node which can be a BinaryExpression or a LogicalExpression node.
     * This parent node can be BinaryExpression, LogicalExpression
     *      , or a ConditionalExpression node
     * @returns {ASTNode} node the appropriate node(left or test).
     */
    function getChildNode(node) {
        return node.type === "ConditionalExpression" ? node.test : node.left;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow mixed binary operators",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-mixed-operators"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        groups: {
                            type: "array",
                            items: {
                                type: "array",
                                items: { enum: ALL_OPERATORS },
                                minItems: 2,
                                uniqueItems: true
                            },
                            uniqueItems: true
                        },
                        allowSamePrecedence: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const options = normalizeOptions(context.options[0]);
            /**
             * Checks whether a given node should be ignored by options or not.
             * @param {ASTNode} node A node to check. This is a BinaryExpression
             *      node or a LogicalExpression node. This parent node is one of
             *      them, too.
             * @returns {boolean} `true` if the node should be ignored.
             */
            function shouldIgnore(node) {
                const a = node;
                const b = node.parent;
                return (!includesBothInAGroup(options.groups, a.operator, b.type === "ConditionalExpression" ? "?:" : b.operator) ||
                    (options.allowSamePrecedence &&
                        astUtils.getPrecedence(a) === astUtils.getPrecedence(b)));
            }
            /**
             * Checks whether the operator of a given node is mixed with parent
             * node's operator or not.
             * @param {ASTNode} node A node to check. This is a BinaryExpression
             *      node or a LogicalExpression node. This parent node is one of
             *      them, too.
             * @returns {boolean} `true` if the node was mixed.
             */
            function isMixedWithParent(node) {
                return (node.operator !== node.parent.operator &&
                    !astUtils.isParenthesised(sourceCode, node));
            }
            /**
             * Checks whether the operator of a given node is mixed with a
             * conditional expression.
             * @param {ASTNode} node A node to check. This is a conditional
             *      expression node
             * @returns {boolean} `true` if the node was mixed.
             */
            function isMixedWithConditionalParent(node) {
                return !astUtils.isParenthesised(sourceCode, node) && !astUtils.isParenthesised(sourceCode, node.test);
            }
            /**
             * Gets the operator token of a given node.
             * @param {ASTNode} node A node to check. This is a BinaryExpression
             *      node or a LogicalExpression node.
             * @returns {Token} The operator token of the node.
             */
            function getOperatorToken(node) {
                return sourceCode.getTokenAfter(getChildNode(node), astUtils.isNotClosingParenToken);
            }
            /**
             * Reports both the operator of a given node and the operator of the
             * parent node.
             * @param {ASTNode} node A node to check. This is a BinaryExpression
             *      node or a LogicalExpression node. This parent node is one of
             *      them, too.
             * @returns {void}
             */
            function reportBothOperators(node) {
                const parent = node.parent;
                const left = (getChildNode(parent) === node) ? node : parent;
                const right = (getChildNode(parent) !== node) ? node : parent;
                const message = "Unexpected mix of '{{leftOperator}}' and '{{rightOperator}}'.";
                const data = {
                    leftOperator: left.operator || "?:",
                    rightOperator: right.operator || "?:"
                };
                context.report({
                    node: left,
                    loc: getOperatorToken(left).loc,
                    message,
                    data
                });
                context.report({
                    node: right,
                    loc: getOperatorToken(right).loc,
                    message,
                    data
                });
            }
            /**
             * Checks between the operator of this node and the operator of the
             * parent node.
             * @param {ASTNode} node A node to check.
             * @returns {void}
             */
            function check(node) {
                if (TARGET_NODE_TYPE.test(node.parent.type)) {
                    if (node.parent.type === "ConditionalExpression" && !shouldIgnore(node) && isMixedWithConditionalParent(node.parent)) {
                        reportBothOperators(node);
                    }
                    else {
                        if (TARGET_NODE_TYPE.test(node.parent.type) &&
                            isMixedWithParent(node) &&
                            !shouldIgnore(node)) {
                            reportBothOperators(node);
                        }
                    }
                }
            }
            return {
                BinaryExpression: check,
                LogicalExpression: check
            };
        }
    };
},
/* eslint/lib/rules/no-mixed-requires.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `require` calls to be mixed with regular variable declarations",
                category: "Node.js and CommonJS",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-mixed-requires"
            },
            schema: [
                {
                    oneOf: [
                        {
                            type: "boolean"
                        },
                        {
                            type: "object",
                            properties: {
                                grouping: {
                                    type: "boolean"
                                },
                                allowCall: {
                                    type: "boolean"
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ]
        },
        create(context) {
            const options = context.options[0];
            let grouping = false, allowCall = false;
            if (typeof options === "object") {
                grouping = options.grouping;
                allowCall = options.allowCall;
            }
            else {
                grouping = !!options;
            }
            /**
             * Returns the list of built-in modules.
             * @returns {string[]} An array of built-in Node.js modules.
             */
            function getBuiltinModules() {
                /*
                 * This list is generated using:
                 * `require("repl")._builtinLibs.concat('repl').sort()`
                 * This particular list is as per nodejs v0.12.2 and iojs v0.7.1
                 */
                return [
                    "assert", "buffer", "child_process", "cluster", "crypto",
                    "dgram", "dns", "domain", "events", "fs", "http", "https",
                    "net", "os", "path", "punycode", "querystring", "readline",
                    "repl", "smalloc", "stream", "string_decoder", "tls", "tty",
                    "url", "util", "v8", "vm", "zlib"
                ];
            }
            const BUILTIN_MODULES = getBuiltinModules();
            const DECL_REQUIRE = "require", DECL_UNINITIALIZED = "uninitialized", DECL_OTHER = "other";
            const REQ_CORE = "core", REQ_FILE = "file", REQ_MODULE = "module", REQ_COMPUTED = "computed";
            /**
             * Determines the type of a declaration statement.
             * @param {ASTNode} initExpression The init node of the VariableDeclarator.
             * @returns {string} The type of declaration represented by the expression.
             */
            function getDeclarationType(initExpression) {
                if (!initExpression) {
                    // "var x;"
                    return DECL_UNINITIALIZED;
                }
                if (initExpression.type === "CallExpression" &&
                    initExpression.callee.type === "Identifier" &&
                    initExpression.callee.name === "require") {
                    // "var x = require('util');"
                    return DECL_REQUIRE;
                }
                if (allowCall &&
                    initExpression.type === "CallExpression" &&
                    initExpression.callee.type === "CallExpression") {
                    // "var x = require('diagnose')('sub-module');"
                    return getDeclarationType(initExpression.callee);
                }
                if (initExpression.type === "MemberExpression") {
                    // "var x = require('glob').Glob;"
                    return getDeclarationType(initExpression.object);
                }
                // "var x = 42;"
                return DECL_OTHER;
            }
            /**
             * Determines the type of module that is loaded via require.
             * @param {ASTNode} initExpression The init node of the VariableDeclarator.
             * @returns {string} The module type.
             */
            function inferModuleType(initExpression) {
                if (initExpression.type === "MemberExpression") {
                    // "var x = require('glob').Glob;"
                    return inferModuleType(initExpression.object);
                }
                if (initExpression.arguments.length === 0) {
                    // "var x = require();"
                    return REQ_COMPUTED;
                }
                const arg = initExpression.arguments[0];
                if (arg.type !== "Literal" || typeof arg.value !== "string") {
                    // "var x = require(42);"
                    return REQ_COMPUTED;
                }
                if (BUILTIN_MODULES.indexOf(arg.value) !== -1) {
                    // "var fs = require('fs');"
                    return REQ_CORE;
                }
                if (/^\.{0,2}\//u.test(arg.value)) {
                    // "var utils = require('./utils');"
                    return REQ_FILE;
                }
                // "var async = require('async');"
                return REQ_MODULE;
            }
            /**
             * Check if the list of variable declarations is mixed, i.e. whether it
             * contains both require and other declarations.
             * @param {ASTNode} declarations The list of VariableDeclarators.
             * @returns {boolean} True if the declarations are mixed, false if not.
             */
            function isMixed(declarations) {
                const contains = {};
                declarations.forEach(declaration => {
                    const type = getDeclarationType(declaration.init);
                    contains[type] = true;
                });
                return !!(contains[DECL_REQUIRE] &&
                    (contains[DECL_UNINITIALIZED] || contains[DECL_OTHER]));
            }
            /**
             * Check if all require declarations in the given list are of the same
             * type.
             * @param {ASTNode} declarations The list of VariableDeclarators.
             * @returns {boolean} True if the declarations are grouped, false if not.
             */
            function isGrouped(declarations) {
                const found = {};
                declarations.forEach(declaration => {
                    if (getDeclarationType(declaration.init) === DECL_REQUIRE) {
                        found[inferModuleType(declaration.init)] = true;
                    }
                });
                return Object.keys(found).length <= 1;
            }
            return {
                VariableDeclaration(node) {
                    if (isMixed(node.declarations)) {
                        context.report({ node, message: "Do not mix 'require' and other declarations." });
                    }
                    else if (grouping && !isGrouped(node.declarations)) {
                        context.report({ node, message: "Do not mix core, module, file and computed requires." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-mixed-spaces-and-tabs.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "disallow mixed spaces and tabs for indentation",
                category: "Stylistic Issues",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-mixed-spaces-and-tabs"
            },
            schema: [
                {
                    enum: ["smart-tabs", true, false]
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            let smartTabs;
            const ignoredLocs = [];
            switch (context.options[0]) {
                case true: // Support old syntax, maybe add deprecation warning here
                case "smart-tabs":
                    smartTabs = true;
                    break;
                default:
                    smartTabs = false;
            }
            /**
             * Determines if a given line and column are before a location.
             * @param {Location} loc The location object from an AST node.
             * @param {int} line The line to check.
             * @param {int} column The column to check.
             * @returns {boolean} True if the line and column are before the location, false if not.
             * @private
             */
            function beforeLoc(loc, line, column) {
                if (line < loc.start.line) {
                    return true;
                }
                return line === loc.start.line && column < loc.start.column;
            }
            /**
             * Determines if a given line and column are after a location.
             * @param {Location} loc The location object from an AST node.
             * @param {int} line The line to check.
             * @param {int} column The column to check.
             * @returns {boolean} True if the line and column are after the location, false if not.
             * @private
             */
            function afterLoc(loc, line, column) {
                if (line > loc.end.line) {
                    return true;
                }
                return line === loc.end.line && column > loc.end.column;
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                TemplateElement(node) {
                    ignoredLocs.push(node.loc);
                },
                "Program:exit"(node) {
                    /*
                     * At least one space followed by a tab
                     * or the reverse before non-tab/-space
                     * characters begin.
                     */
                    let regex = /^(?=[\t ]*(\t | \t))/u;
                    const lines = sourceCode.lines, comments = sourceCode.getAllComments();
                    comments.forEach(comment => {
                        ignoredLocs.push(comment.loc);
                    });
                    ignoredLocs.sort((first, second) => {
                        if (beforeLoc(first, second.start.line, second.start.column)) {
                            return 1;
                        }
                        if (beforeLoc(second, first.start.line, second.start.column)) {
                            return -1;
                        }
                        return 0;
                    });
                    if (smartTabs) {
                        /*
                         * At least one space followed by a tab
                         * before non-tab/-space characters begin.
                         */
                        regex = /^(?=[\t ]* \t)/u;
                    }
                    lines.forEach((line, i) => {
                        const match = regex.exec(line);
                        if (match) {
                            const lineNumber = i + 1, column = match.index + 1;
                            for (let j = 0; j < ignoredLocs.length; j++) {
                                if (beforeLoc(ignoredLocs[j], lineNumber, column)) {
                                    continue;
                                }
                                if (afterLoc(ignoredLocs[j], lineNumber, column)) {
                                    continue;
                                }
                                return;
                            }
                            context.report({ node, loc: { line: lineNumber, column }, message: "Mixed spaces and tabs." });
                        }
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-multi-assign.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow use of chained assignment expressions",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-multi-assign"
            },
            schema: []
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                AssignmentExpression(node) {
                    if (["AssignmentExpression", "VariableDeclarator"].indexOf(node.parent.type) !== -1) {
                        context.report({
                            node,
                            message: "Unexpected chained assignment."
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-multi-spaces.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "disallow multiple spaces",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-multi-spaces"
            },
            fixable: "whitespace",
            schema: [
                {
                    type: "object",
                    properties: {
                        exceptions: {
                            type: "object",
                            patternProperties: {
                                "^([A-Z][a-z]*)+$": {
                                    type: "boolean"
                                }
                            },
                            additionalProperties: false
                        },
                        ignoreEOLComments: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const options = context.options[0] || {};
            const ignoreEOLComments = options.ignoreEOLComments;
            const exceptions = Object.assign({ Property: true }, options.exceptions);
            const hasExceptions = Object.keys(exceptions).filter(key => exceptions[key]).length > 0;
            /**
             * Formats value of given comment token for error message by truncating its length.
             * @param {Token} token comment token
             * @returns {string} formatted value
             * @private
             */
            function formatReportedCommentValue(token) {
                const valueLines = token.value.split("\n");
                const value = valueLines[0];
                const formattedValue = `${value.slice(0, 12)}...`;
                return valueLines.length === 1 && value.length <= 12 ? value : formattedValue;
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                Program() {
                    sourceCode.tokensAndComments.forEach((leftToken, leftIndex, tokensAndComments) => {
                        if (leftIndex === tokensAndComments.length - 1) {
                            return;
                        }
                        const rightToken = tokensAndComments[leftIndex + 1];
                        // Ignore tokens that don't have 2 spaces between them or are on different lines
                        if (!sourceCode.text.slice(leftToken.range[1], rightToken.range[0]).includes("  ") ||
                            leftToken.loc.end.line < rightToken.loc.start.line) {
                            return;
                        }
                        // Ignore comments that are the last token on their line if `ignoreEOLComments` is active.
                        if (ignoreEOLComments &&
                            astUtils.isCommentToken(rightToken) &&
                            (leftIndex === tokensAndComments.length - 2 ||
                                rightToken.loc.end.line < tokensAndComments[leftIndex + 2].loc.start.line)) {
                            return;
                        }
                        // Ignore tokens that are in a node in the "exceptions" object
                        if (hasExceptions) {
                            const parentNode = sourceCode.getNodeByRangeIndex(rightToken.range[0] - 1);
                            if (parentNode && exceptions[parentNode.type]) {
                                return;
                            }
                        }
                        let displayValue;
                        if (rightToken.type === "Block") {
                            displayValue = `/*${formatReportedCommentValue(rightToken)}*/`;
                        }
                        else if (rightToken.type === "Line") {
                            displayValue = `//${formatReportedCommentValue(rightToken)}`;
                        }
                        else {
                            displayValue = rightToken.value;
                        }
                        context.report({
                            node: rightToken,
                            loc: { start: leftToken.loc.end, end: rightToken.loc.start },
                            message: "Multiple spaces found before '{{displayValue}}'.",
                            data: { displayValue },
                            fix: fixer => fixer.replaceTextRange([leftToken.range[1], rightToken.range[0]], " ")
                        });
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-multi-str.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow multiline strings",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-multi-str"
            },
            schema: []
        },
        create(context) {
            /**
             * Determines if a given node is part of JSX syntax.
             * @param {ASTNode} node The node to check.
             * @returns {boolean} True if the node is a JSX node, false if not.
             * @private
             */
            function isJSXElement(node) {
                return node.type.indexOf("JSX") === 0;
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                Literal(node) {
                    if (astUtils.LINEBREAK_MATCHER.test(node.raw) && !isJSXElement(node.parent)) {
                        context.report({ node, message: "Multiline support is limited to browsers supporting ES5 only." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-multiple-empty-lines.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "disallow multiple empty lines",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-multiple-empty-lines"
            },
            fixable: "whitespace",
            schema: [
                {
                    type: "object",
                    properties: {
                        max: {
                            type: "integer",
                            minimum: 0
                        },
                        maxEOF: {
                            type: "integer",
                            minimum: 0
                        },
                        maxBOF: {
                            type: "integer",
                            minimum: 0
                        }
                    },
                    required: ["max"],
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            // Use options.max or 2 as default
            let max = 2, maxEOF = max, maxBOF = max;
            if (context.options.length) {
                max = context.options[0].max;
                maxEOF = typeof context.options[0].maxEOF !== "undefined" ? context.options[0].maxEOF : max;
                maxBOF = typeof context.options[0].maxBOF !== "undefined" ? context.options[0].maxBOF : max;
            }
            const sourceCode = context.getSourceCode();
            // Swallow the final newline, as some editors add it automatically and we don't want it to cause an issue
            const allLines = sourceCode.lines[sourceCode.lines.length - 1] === "" ? sourceCode.lines.slice(0, -1) : sourceCode.lines;
            const templateLiteralLines = new Set();
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                TemplateLiteral(node) {
                    node.quasis.forEach(literalPart => {
                        // Empty lines have a semantic meaning if they're inside template literals. Don't count these as empty lines.
                        for (let ignoredLine = literalPart.loc.start.line; ignoredLine < literalPart.loc.end.line; ignoredLine++) {
                            templateLiteralLines.add(ignoredLine);
                        }
                    });
                },
                "Program:exit"(node) {
                    return allLines
                        // Given a list of lines, first get a list of line numbers that are non-empty.
                        .reduce((nonEmptyLineNumbers, line, index) => {
                        if (line.trim() || templateLiteralLines.has(index + 1)) {
                            nonEmptyLineNumbers.push(index + 1);
                        }
                        return nonEmptyLineNumbers;
                    }, [])
                        // Add a value at the end to allow trailing empty lines to be checked.
                        .concat(allLines.length + 1)
                        // Given two line numbers of non-empty lines, report the lines between if the difference is too large.
                        .reduce((lastLineNumber, lineNumber) => {
                        let message, maxAllowed;
                        if (lastLineNumber === 0) {
                            message = "Too many blank lines at the beginning of file. Max of {{max}} allowed.";
                            maxAllowed = maxBOF;
                        }
                        else if (lineNumber === allLines.length + 1) {
                            message = "Too many blank lines at the end of file. Max of {{max}} allowed.";
                            maxAllowed = maxEOF;
                        }
                        else {
                            message = "More than {{max}} blank {{pluralizedLines}} not allowed.";
                            maxAllowed = max;
                        }
                        if (lineNumber - lastLineNumber - 1 > maxAllowed) {
                            context.report({
                                node,
                                loc: { start: { line: lastLineNumber + maxAllowed + 1, column: 0 }, end: { line: lineNumber, column: 0 } },
                                message,
                                data: { max: maxAllowed, pluralizedLines: maxAllowed === 1 ? "line" : "lines" },
                                fix(fixer) {
                                    const rangeStart = sourceCode.getIndexFromLoc({ line: lastLineNumber + 1, column: 0 });
                                    /*
                                     * The end of the removal range is usually the start index of the next line.
                                     * However, at the end of the file there is no next line, so the end of the
                                     * range is just the length of the text.
                                     */
                                    const lineNumberAfterRemovedLines = lineNumber - maxAllowed;
                                    const rangeEnd = lineNumberAfterRemovedLines <= allLines.length
                                        ? sourceCode.getIndexFromLoc({ line: lineNumberAfterRemovedLines, column: 0 })
                                        : sourceCode.text.length;
                                    return fixer.removeRange([rangeStart, rangeEnd]);
                                }
                            });
                        }
                        return lineNumber;
                    }, 0);
                }
            };
        }
    };
},
/* eslint/lib/rules/no-native-reassign.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow assignments to native objects or read-only global variables",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-native-reassign"
            },
            deprecated: true,
            replacedBy: ["no-global-assign"],
            schema: [
                {
                    type: "object",
                    properties: {
                        exceptions: {
                            type: "array",
                            items: { type: "string" },
                            uniqueItems: true
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const config = context.options[0];
            const exceptions = (config && config.exceptions) || [];
            /**
             * Reports write references.
             * @param {Reference} reference A reference to check.
             * @param {int} index The index of the reference in the references.
             * @param {Reference[]} references The array that the reference belongs to.
             * @returns {void}
             */
            function checkReference(reference, index, references) {
                const identifier = reference.identifier;
                if (reference.init === false &&
                    reference.isWrite() &&
                    /*
                     * Destructuring assignments can have multiple default value,
                     * so possibly there are multiple writeable references for the same identifier.
                     */
                    (index === 0 || references[index - 1].identifier !== identifier)) {
                    context.report({
                        node: identifier,
                        message: "Read-only global '{{name}}' should not be modified.",
                        data: identifier
                    });
                }
            }
            /**
             * Reports write references if a given variable is read-only builtin.
             * @param {Variable} variable A variable to check.
             * @returns {void}
             */
            function checkVariable(variable) {
                if (variable.writeable === false && exceptions.indexOf(variable.name) === -1) {
                    variable.references.forEach(checkReference);
                }
            }
            return {
                Program() {
                    const globalScope = context.getScope();
                    globalScope.variables.forEach(checkVariable);
                }
            };
        }
    };
},
/* eslint/lib/rules/no-negated-condition.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow negated conditions",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-negated-condition"
            },
            schema: []
        },
        create(context) {
            /**
             * Determines if a given node is an if-else without a condition on the else
             * @param {ASTNode} node The node to check.
             * @returns {boolean} True if the node has an else without an if.
             * @private
             */
            function hasElseWithoutCondition(node) {
                return node.alternate && node.alternate.type !== "IfStatement";
            }
            /**
             * Determines if a given node is a negated unary expression
             * @param {Object} test The test object to check.
             * @returns {boolean} True if the node is a negated unary expression.
             * @private
             */
            function isNegatedUnaryExpression(test) {
                return test.type === "UnaryExpression" && test.operator === "!";
            }
            /**
             * Determines if a given node is a negated binary expression
             * @param {Test} test The test to check.
             * @returns {boolean} True if the node is a negated binary expression.
             * @private
             */
            function isNegatedBinaryExpression(test) {
                return test.type === "BinaryExpression" &&
                    (test.operator === "!=" || test.operator === "!==");
            }
            /**
             * Determines if a given node has a negated if expression
             * @param {ASTNode} node The node to check.
             * @returns {boolean} True if the node has a negated if expression.
             * @private
             */
            function isNegatedIf(node) {
                return isNegatedUnaryExpression(node.test) || isNegatedBinaryExpression(node.test);
            }
            return {
                IfStatement(node) {
                    if (!hasElseWithoutCondition(node)) {
                        return;
                    }
                    if (isNegatedIf(node)) {
                        context.report({ node, message: "Unexpected negated condition." });
                    }
                },
                ConditionalExpression(node) {
                    if (isNegatedIf(node)) {
                        context.report({ node, message: "Unexpected negated condition." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-negated-in-lhs.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow negating the left operand in `in` expressions",
                category: "Possible Errors",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-negated-in-lhs"
            },
            replacedBy: ["no-unsafe-negation"],
            deprecated: true,
            schema: []
        },
        create(context) {
            return {
                BinaryExpression(node) {
                    if (node.operator === "in" && node.left.type === "UnaryExpression" && node.left.operator === "!") {
                        context.report({ node, message: "The 'in' expression's left operand is negated." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-nested-ternary.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow nested ternary expressions",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-nested-ternary"
            },
            schema: []
        },
        create(context) {
            return {
                ConditionalExpression(node) {
                    if (node.alternate.type === "ConditionalExpression" ||
                        node.consequent.type === "ConditionalExpression") {
                        context.report({ node, message: "Do not nest ternary expressions." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-new.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `new` operators outside of assignments or comparisons",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-new"
            },
            schema: []
        },
        create(context) {
            return {
                "ExpressionStatement > NewExpression"(node) {
                    context.report({ node: node.parent, message: "Do not use 'new' for side effects." });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-new-func.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `new` operators with the `Function` object",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-new-func"
            },
            schema: []
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Reports a node.
             * @param {ASTNode} node The node to report
             * @returns {void}
             * @private
             */
            function report(node) {
                context.report({ node, message: "The Function constructor is eval." });
            }
            return {
                "NewExpression[callee.name = 'Function']": report,
                "CallExpression[callee.name = 'Function']": report
            };
        }
    };
},
/* eslint/lib/rules/no-new-object.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `Object` constructors",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-new-object"
            },
            schema: []
        },
        create(context) {
            return {
                NewExpression(node) {
                    if (node.callee.name === "Object") {
                        context.report({ node, message: "The object literal notation {} is preferrable." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-new-require.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `new` operators with calls to `require`",
                category: "Node.js and CommonJS",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-new-require"
            },
            schema: []
        },
        create(context) {
            return {
                NewExpression(node) {
                    if (node.callee.type === "Identifier" && node.callee.name === "require") {
                        context.report({ node, message: "Unexpected use of new with require." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-new-symbol.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow `new` operators with the `Symbol` object",
                category: "ECMAScript 6",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-new-symbol"
            },
            schema: []
        },
        create(context) {
            return {
                "Program:exit"() {
                    const globalScope = context.getScope();
                    const variable = globalScope.set.get("Symbol");
                    if (variable && variable.defs.length === 0) {
                        variable.references.forEach(ref => {
                            const node = ref.identifier;
                            if (node.parent && node.parent.type === "NewExpression") {
                                context.report({ node, message: "`Symbol` cannot be called as a constructor." });
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-new-wrappers.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `new` operators with the `String`, `Number`, and `Boolean` objects",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-new-wrappers"
            },
            schema: []
        },
        create(context) {
            return {
                NewExpression(node) {
                    const wrapperObjects = ["String", "Number", "Boolean", "Math", "JSON"];
                    if (wrapperObjects.indexOf(node.callee.name) > -1) {
                        context.report({ node, message: "Do not use {{fn}} as a constructor.", data: { fn: node.callee.name } });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-obj-calls.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const { CALL, ReferenceTracker } = require(250) /* eslint-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const nonCallableGlobals = ["Atomics", "JSON", "Math", "Reflect"];
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow calling global object properties as functions",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-obj-calls"
            },
            schema: [],
            messages: {
                unexpectedCall: "'{{name}}' is not a function."
            }
        },
        create(context) {
            return {
                Program() {
                    const scope = context.getScope();
                    const tracker = new ReferenceTracker(scope);
                    const traceMap = {};
                    for (const global of nonCallableGlobals) {
                        traceMap[global] = {
                            [CALL]: true
                        };
                    }
                    for (const { node } of tracker.iterateGlobalReferences(traceMap)) {
                        context.report({ node, messageId: "unexpectedCall", data: { name: node.callee.name } });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-octal.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow octal literals",
                category: "Best Practices",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-octal"
            },
            schema: []
        },
        create(context) {
            return {
                Literal(node) {
                    if (typeof node.value === "number" && /^0[0-9]/u.test(node.raw)) {
                        context.report({ node, message: "Octal literals should not be used." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-octal-escape.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow octal escape sequences in string literals",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-octal-escape"
            },
            schema: [],
            messages: {
                octalEscapeSequence: "Don't use octal: '\\{{sequence}}'. Use '\\u....' instead."
            }
        },
        create(context) {
            return {
                Literal(node) {
                    if (typeof node.value !== "string") {
                        return;
                    }
                    // \0 represents a valid NULL character if it isn't followed by a digit.
                    const match = node.raw.match(/^(?:[^\\]|\\.)*?\\([0-3][0-7]{1,2}|[4-7][0-7]|0(?=[89])|[1-7])/su);
                    if (match) {
                        context.report({
                            node,
                            messageId: "octalEscapeSequence",
                            data: { sequence: match[1] }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-param-reassign.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const stopNodePattern = /(?:Statement|Declaration|Function(?:Expression)?|Program)$/u;
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow reassigning `function` parameters",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-param-reassign"
            },
            schema: [
                {
                    oneOf: [
                        {
                            type: "object",
                            properties: {
                                props: {
                                    enum: [false]
                                }
                            },
                            additionalProperties: false
                        },
                        {
                            type: "object",
                            properties: {
                                props: {
                                    enum: [true]
                                },
                                ignorePropertyModificationsFor: {
                                    type: "array",
                                    items: {
                                        type: "string"
                                    },
                                    uniqueItems: true
                                },
                                ignorePropertyModificationsForRegex: {
                                    type: "array",
                                    items: {
                                        type: "string"
                                    },
                                    uniqueItems: true
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ]
        },
        create(context) {
            const props = context.options[0] && context.options[0].props;
            const ignoredPropertyAssignmentsFor = context.options[0] && context.options[0].ignorePropertyModificationsFor || [];
            const ignoredPropertyAssignmentsForRegex = context.options[0] && context.options[0].ignorePropertyModificationsForRegex || [];
            /**
             * Checks whether or not the reference modifies properties of its variable.
             * @param {Reference} reference A reference to check.
             * @returns {boolean} Whether or not the reference modifies properties of its variable.
             */
            function isModifyingProp(reference) {
                let node = reference.identifier;
                let parent = node.parent;
                while (parent && (!stopNodePattern.test(parent.type) ||
                    parent.type === "ForInStatement" || parent.type === "ForOfStatement")) {
                    switch (parent.type) {
                        // e.g. foo.a = 0;
                        case "AssignmentExpression":
                            return parent.left === node;
                        // e.g. ++foo.a;
                        case "UpdateExpression":
                            return true;
                        // e.g. delete foo.a;
                        case "UnaryExpression":
                            if (parent.operator === "delete") {
                                return true;
                            }
                            break;
                        // e.g. for (foo.a in b) {}
                        case "ForInStatement":
                        case "ForOfStatement":
                            if (parent.left === node) {
                                return true;
                            }
                            // this is a stop node for parent.right and parent.body
                            return false;
                        // EXCLUDES: e.g. cache.get(foo.a).b = 0;
                        case "CallExpression":
                            if (parent.callee !== node) {
                                return false;
                            }
                            break;
                        // EXCLUDES: e.g. cache[foo.a] = 0;
                        case "MemberExpression":
                            if (parent.property === node) {
                                return false;
                            }
                            break;
                        // EXCLUDES: e.g. ({ [foo]: a }) = bar;
                        case "Property":
                            if (parent.key === node) {
                                return false;
                            }
                            break;
                        // EXCLUDES: e.g. (foo ? a : b).c = bar;
                        case "ConditionalExpression":
                            if (parent.test === node) {
                                return false;
                            }
                            break;
                        // no default
                    }
                    node = parent;
                    parent = node.parent;
                }
                return false;
            }
            /**
             * Tests that an identifier name matches any of the ignored property assignments.
             * First we test strings in ignoredPropertyAssignmentsFor.
             * Then we instantiate and test RegExp objects from ignoredPropertyAssignmentsForRegex strings.
             * @param {string} identifierName A string that describes the name of an identifier to
             * ignore property assignments for.
             * @returns {boolean} Whether the string matches an ignored property assignment regular expression or not.
             */
            function isIgnoredPropertyAssignment(identifierName) {
                return ignoredPropertyAssignmentsFor.includes(identifierName) ||
                    ignoredPropertyAssignmentsForRegex.some(ignored => new RegExp(ignored, "u").test(identifierName));
            }
            /**
             * Reports a reference if is non initializer and writable.
             * @param {Reference} reference A reference to check.
             * @param {int} index The index of the reference in the references.
             * @param {Reference[]} references The array that the reference belongs to.
             * @returns {void}
             */
            function checkReference(reference, index, references) {
                const identifier = reference.identifier;
                if (identifier &&
                    !reference.init &&
                    /*
                     * Destructuring assignments can have multiple default value,
                     * so possibly there are multiple writeable references for the same identifier.
                     */
                    (index === 0 || references[index - 1].identifier !== identifier)) {
                    if (reference.isWrite()) {
                        context.report({ node: identifier, message: "Assignment to function parameter '{{name}}'.", data: { name: identifier.name } });
                    }
                    else if (props && isModifyingProp(reference) && !isIgnoredPropertyAssignment(identifier.name)) {
                        context.report({ node: identifier, message: "Assignment to property of function parameter '{{name}}'.", data: { name: identifier.name } });
                    }
                }
            }
            /**
             * Finds and reports references that are non initializer and writable.
             * @param {Variable} variable A variable to check.
             * @returns {void}
             */
            function checkVariable(variable) {
                if (variable.defs[0].type === "Parameter") {
                    variable.references.forEach(checkReference);
                }
            }
            /**
             * Checks parameters of a given function node.
             * @param {ASTNode} node A function node to check.
             * @returns {void}
             */
            function checkForFunction(node) {
                context.getDeclaredVariables(node).forEach(checkVariable);
            }
            return {
                // `:exit` is needed for the `node.parent` property of identifier nodes.
                "FunctionDeclaration:exit": checkForFunction,
                "FunctionExpression:exit": checkForFunction,
                "ArrowFunctionExpression:exit": checkForFunction
            };
        }
    };
},
/* eslint/lib/rules/no-path-concat.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow string concatenation with `__dirname` and `__filename`",
                category: "Node.js and CommonJS",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-path-concat"
            },
            schema: []
        },
        create(context) {
            const MATCHER = /^__(?:dir|file)name$/u;
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                BinaryExpression(node) {
                    const left = node.left, right = node.right;
                    if (node.operator === "+" &&
                        ((left.type === "Identifier" && MATCHER.test(left.name)) ||
                            (right.type === "Identifier" && MATCHER.test(right.name)))) {
                        context.report({ node, message: "Use path.join() or path.resolve() instead of + to create paths." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-plusplus.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the unary operators `++` and `--`",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-plusplus"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allowForLoopAfterthoughts: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const config = context.options[0];
            let allowInForAfterthought = false;
            if (typeof config === "object") {
                allowInForAfterthought = config.allowForLoopAfterthoughts === true;
            }
            return {
                UpdateExpression(node) {
                    if (allowInForAfterthought && node.parent.type === "ForStatement") {
                        return;
                    }
                    context.report({
                        node,
                        message: "Unary operator '{{operator}}' used.",
                        data: {
                            operator: node.operator
                        }
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-process-env.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the use of `process.env`",
                category: "Node.js and CommonJS",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-process-env"
            },
            schema: []
        },
        create(context) {
            return {
                MemberExpression(node) {
                    const objectName = node.object.name, propertyName = node.property.name;
                    if (objectName === "process" && !node.computed && propertyName && propertyName === "env") {
                        context.report({ node, message: "Unexpected use of process.env." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-process-exit.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the use of `process.exit()`",
                category: "Node.js and CommonJS",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-process-exit"
            },
            schema: []
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                "CallExpression > MemberExpression.callee[object.name = 'process'][property.name = 'exit']"(node) {
                    context.report({ node: node.parent, message: "Don't use process.exit(); throw an error instead." });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-proto.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the use of the `__proto__` property",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-proto"
            },
            schema: []
        },
        create(context) {
            return {
                MemberExpression(node) {
                    if (node.property &&
                        (node.property.type === "Identifier" && node.property.name === "__proto__" && !node.computed) ||
                        (node.property.type === "Literal" && node.property.value === "__proto__")) {
                        context.report({ node, message: "The '__proto__' property is deprecated." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-prototype-builtins.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow calling some `Object.prototype` methods directly on objects",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-prototype-builtins"
            },
            schema: []
        },
        create(context) {
            const DISALLOWED_PROPS = [
                "hasOwnProperty",
                "isPrototypeOf",
                "propertyIsEnumerable"
            ];
            /**
             * Reports if a disallowed property is used in a CallExpression
             * @param {ASTNode} node The CallExpression node.
             * @returns {void}
             */
            function disallowBuiltIns(node) {
                if (node.callee.type !== "MemberExpression" || node.callee.computed) {
                    return;
                }
                const propName = node.callee.property.name;
                if (DISALLOWED_PROPS.indexOf(propName) > -1) {
                    context.report({
                        message: "Do not access Object.prototype method '{{prop}}' from target object.",
                        loc: node.callee.property.loc.start,
                        data: { prop: propName },
                        node
                    });
                }
            }
            return {
                CallExpression: disallowBuiltIns
            };
        }
    };
},
/* eslint/lib/rules/no-redeclare.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow variable redeclaration",
                category: "Best Practices",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-redeclare"
            },
            messages: {
                redeclared: "'{{id}}' is already defined.",
                redeclaredAsBuiltin: "'{{id}}' is already defined as a built-in global variable.",
                redeclaredBySyntax: "'{{id}}' is already defined by a variable declaration."
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        builtinGlobals: { type: "boolean", default: true }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const options = {
                builtinGlobals: Boolean(context.options.length === 0 ||
                    context.options[0].builtinGlobals)
            };
            const sourceCode = context.getSourceCode();
            /**
             * Iterate declarations of a given variable.
             * @param {escope.variable} variable The variable object to iterate declarations.
             * @returns {IterableIterator<{type:string,node:ASTNode,loc:SourceLocation}>} The declarations.
             */
            function* iterateDeclarations(variable) {
                if (options.builtinGlobals && (variable.eslintImplicitGlobalSetting === "readonly" ||
                    variable.eslintImplicitGlobalSetting === "writable")) {
                    yield { type: "builtin" };
                }
                for (const id of variable.identifiers) {
                    yield { type: "syntax", node: id, loc: id.loc };
                }
                if (variable.eslintExplicitGlobalComments) {
                    for (const comment of variable.eslintExplicitGlobalComments) {
                        yield {
                            type: "comment",
                            node: comment,
                            loc: astUtils.getNameLocationInGlobalDirectiveComment(sourceCode, comment, variable.name)
                        };
                    }
                }
            }
            /**
             * Find variables in a given scope and flag redeclared ones.
             * @param {Scope} scope An eslint-scope scope object.
             * @returns {void}
             * @private
             */
            function findVariablesInScope(scope) {
                for (const variable of scope.variables) {
                    const [declaration, ...extraDeclarations] = iterateDeclarations(variable);
                    if (extraDeclarations.length === 0) {
                        continue;
                    }
                    /*
                     * If the type of a declaration is different from the type of
                     * the first declaration, it shows the location of the first
                     * declaration.
                     */
                    const detailMessageId = declaration.type === "builtin"
                        ? "redeclaredAsBuiltin"
                        : "redeclaredBySyntax";
                    const data = { id: variable.name };
                    // Report extra declarations.
                    for (const { type, node, loc } of extraDeclarations) {
                        const messageId = type === declaration.type
                            ? "redeclared"
                            : detailMessageId;
                        context.report({ node, loc, messageId, data });
                    }
                }
            }
            /**
             * Find variables in the current scope.
             * @param {ASTNode} node The node of the current scope.
             * @returns {void}
             * @private
             */
            function checkForBlock(node) {
                const scope = context.getScope();
                /*
                 * In ES5, some node type such as `BlockStatement` doesn't have that scope.
                 * `scope.block` is a different node in such a case.
                 */
                if (scope.block === node) {
                    findVariablesInScope(scope);
                }
            }
            return {
                Program() {
                    const scope = context.getScope();
                    findVariablesInScope(scope);
                    // Node.js or ES modules has a special scope.
                    if (scope.type === "global" &&
                        scope.childScopes[0] &&
                        // The special scope's block is the Program node.
                        scope.block === scope.childScopes[0].block) {
                        findVariablesInScope(scope.childScopes[0]);
                    }
                },
                FunctionDeclaration: checkForBlock,
                FunctionExpression: checkForBlock,
                ArrowFunctionExpression: checkForBlock,
                BlockStatement: checkForBlock,
                ForStatement: checkForBlock,
                ForInStatement: checkForBlock,
                ForOfStatement: checkForBlock,
                SwitchStatement: checkForBlock
            };
        }
    };
},
/* eslint/lib/rules/no-regex-spaces.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    const regexpp = require(226) /* regexpp */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const regExpParser = new regexpp.RegExpParser();
    const DOUBLE_SPACE = / {2}/u;
    /**
     * Check if node is a string
     * @param {ASTNode} node node to evaluate
     * @returns {boolean} True if its a string
     * @private
     */
    function isString(node) {
        return node && node.type === "Literal" && typeof node.value === "string";
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow multiple spaces in regular expressions",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-regex-spaces"
            },
            schema: [],
            fixable: "code"
        },
        create(context) {
            /**
             * Validate regular expression
             * @param {ASTNode} nodeToReport Node to report.
             * @param {string} pattern Regular expression pattern to validate.
             * @param {string} rawPattern Raw representation of the pattern in the source code.
             * @param {number} rawPatternStartRange Start range of the pattern in the source code.
             * @param {string} flags Regular expression flags.
             * @returns {void}
             * @private
             */
            function checkRegex(nodeToReport, pattern, rawPattern, rawPatternStartRange, flags) {
                // Skip if there are no consecutive spaces in the source code, to avoid reporting e.g., RegExp(' \ ').
                if (!DOUBLE_SPACE.test(rawPattern)) {
                    return;
                }
                const characterClassNodes = [];
                let regExpAST;
                try {
                    regExpAST = regExpParser.parsePattern(pattern, 0, pattern.length, flags.includes("u"));
                }
                catch (e) {
                    // Ignore regular expressions with syntax errors
                    return;
                }
                regexpp.visitRegExpAST(regExpAST, {
                    onCharacterClassEnter(ccNode) {
                        characterClassNodes.push(ccNode);
                    }
                });
                const spacesPattern = /( {2,})(?: [+*{?]|[^+*{?]|$)/gu;
                let match;
                while ((match = spacesPattern.exec(pattern))) {
                    const { 1: { length }, index } = match;
                    // Report only consecutive spaces that are not in character classes.
                    if (characterClassNodes.every(({ start, end }) => index < start || end <= index)) {
                        context.report({
                            node: nodeToReport,
                            message: "Spaces are hard to count. Use {{{length}}}.",
                            data: { length },
                            fix(fixer) {
                                if (pattern !== rawPattern) {
                                    return null;
                                }
                                return fixer.replaceTextRange([rawPatternStartRange + index, rawPatternStartRange + index + length], ` {${length}}`);
                            }
                        });
                        // Report only the first occurence of consecutive spaces
                        return;
                    }
                }
            }
            /**
             * Validate regular expression literals
             * @param {ASTNode} node node to validate
             * @returns {void}
             * @private
             */
            function checkLiteral(node) {
                if (node.regex) {
                    const pattern = node.regex.pattern;
                    const rawPattern = node.raw.slice(1, node.raw.lastIndexOf("/"));
                    const rawPatternStartRange = node.range[0] + 1;
                    const flags = node.regex.flags;
                    checkRegex(node, pattern, rawPattern, rawPatternStartRange, flags);
                }
            }
            /**
             * Validate strings passed to the RegExp constructor
             * @param {ASTNode} node node to validate
             * @returns {void}
             * @private
             */
            function checkFunction(node) {
                const scope = context.getScope();
                const regExpVar = astUtils.getVariableByName(scope, "RegExp");
                const shadowed = regExpVar && regExpVar.defs.length > 0;
                const patternNode = node.arguments[0];
                const flagsNode = node.arguments[1];
                if (node.callee.type === "Identifier" && node.callee.name === "RegExp" && isString(patternNode) && !shadowed) {
                    const pattern = patternNode.value;
                    const rawPattern = patternNode.raw.slice(1, -1);
                    const rawPatternStartRange = patternNode.range[0] + 1;
                    const flags = isString(flagsNode) ? flagsNode.value : "";
                    checkRegex(node, pattern, rawPattern, rawPatternStartRange, flags);
                }
            }
            return {
                Literal: checkLiteral,
                CallExpression: checkFunction,
                NewExpression: checkFunction
            };
        }
    };
},
/* eslint/lib/rules/no-restricted-globals.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const DEFAULT_MESSAGE_TEMPLATE = "Unexpected use of '{{name}}'.", CUSTOM_MESSAGE_TEMPLATE = "Unexpected use of '{{name}}'. {{customMessage}}";
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow specified global variables",
                category: "Variables",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-restricted-globals"
            },
            schema: {
                type: "array",
                items: {
                    oneOf: [
                        {
                            type: "string"
                        },
                        {
                            type: "object",
                            properties: {
                                name: { type: "string" },
                                message: { type: "string" }
                            },
                            required: ["name"],
                            additionalProperties: false
                        }
                    ]
                },
                uniqueItems: true,
                minItems: 0
            }
        },
        create(context) {
            // If no globals are restricted, we don't need to do anything
            if (context.options.length === 0) {
                return {};
            }
            const restrictedGlobalMessages = context.options.reduce((memo, option) => {
                if (typeof option === "string") {
                    memo[option] = null;
                }
                else {
                    memo[option.name] = option.message;
                }
                return memo;
            }, {});
            /**
             * Report a variable to be used as a restricted global.
             * @param {Reference} reference the variable reference
             * @returns {void}
             * @private
             */
            function reportReference(reference) {
                const name = reference.identifier.name, customMessage = restrictedGlobalMessages[name], message = customMessage
                    ? CUSTOM_MESSAGE_TEMPLATE
                    : DEFAULT_MESSAGE_TEMPLATE;
                context.report({
                    node: reference.identifier,
                    message,
                    data: {
                        name,
                        customMessage
                    }
                });
            }
            /**
             * Check if the given name is a restricted global name.
             * @param {string} name name of a variable
             * @returns {boolean} whether the variable is a restricted global or not
             * @private
             */
            function isRestricted(name) {
                return Object.prototype.hasOwnProperty.call(restrictedGlobalMessages, name);
            }
            return {
                Program() {
                    const scope = context.getScope();
                    // Report variables declared elsewhere (ex: variables defined as "global" by eslint)
                    scope.variables.forEach(variable => {
                        if (!variable.defs.length && isRestricted(variable.name)) {
                            variable.references.forEach(reportReference);
                        }
                    });
                    // Report variables not declared at all
                    scope.through.forEach(reference => {
                        if (isRestricted(reference.identifier.name)) {
                            reportReference(reference);
                        }
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-restricted-imports.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const ignore = require(309) /* ignore */;
    const arrayOfStrings = {
        type: "array",
        items: { type: "string" },
        uniqueItems: true
    };
    const arrayOfStringsOrObjects = {
        type: "array",
        items: {
            anyOf: [
                { type: "string" },
                {
                    type: "object",
                    properties: {
                        name: { type: "string" },
                        message: {
                            type: "string",
                            minLength: 1
                        },
                        importNames: {
                            type: "array",
                            items: {
                                type: "string"
                            }
                        }
                    },
                    additionalProperties: false,
                    required: ["name"]
                }
            ]
        },
        uniqueItems: true
    };
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow specified modules when loaded by `import`",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-restricted-imports"
            },
            messages: {
                path: "'{{importSource}}' import is restricted from being used.",
                // eslint-disable-next-line eslint-plugin/report-message-format
                pathWithCustomMessage: "'{{importSource}}' import is restricted from being used. {{customMessage}}",
                patterns: "'{{importSource}}' import is restricted from being used by a pattern.",
                everything: "* import is invalid because '{{importNames}}' from '{{importSource}}' is restricted.",
                // eslint-disable-next-line eslint-plugin/report-message-format
                everythingWithCustomMessage: "* import is invalid because '{{importNames}}' from '{{importSource}}' is restricted. {{customMessage}}"
            },
            schema: {
                anyOf: [
                    arrayOfStringsOrObjects,
                    {
                        type: "array",
                        items: [{
                                type: "object",
                                properties: {
                                    paths: arrayOfStringsOrObjects,
                                    patterns: arrayOfStrings
                                },
                                additionalProperties: false
                            }],
                        additionalItems: false
                    }
                ]
            }
        },
        create(context) {
            const options = Array.isArray(context.options) ? context.options : [];
            const isPathAndPatternsObject = typeof options[0] === "object" &&
                (Object.prototype.hasOwnProperty.call(options[0], "paths") || Object.prototype.hasOwnProperty.call(options[0], "patterns"));
            const restrictedPaths = (isPathAndPatternsObject ? options[0].paths : context.options) || [];
            const restrictedPatterns = (isPathAndPatternsObject ? options[0].patterns : []) || [];
            const restrictedPathMessages = restrictedPaths.reduce((memo, importSource) => {
                if (typeof importSource === "string") {
                    memo[importSource] = { message: null };
                }
                else {
                    memo[importSource.name] = {
                        message: importSource.message,
                        importNames: importSource.importNames
                    };
                }
                return memo;
            }, {});
            // if no imports are restricted we don"t need to check
            if (Object.keys(restrictedPaths).length === 0 && restrictedPatterns.length === 0) {
                return {};
            }
            const restrictedPatternsMatcher = ignore().add(restrictedPatterns);
            /**
             * Checks to see if "*" is being used to import everything.
             * @param {Set.<string>} importNames Set of import names that are being imported
             * @returns {boolean} whether everything is imported or not
             */
            function isEverythingImported(importNames) {
                return importNames.has("*");
            }
            /**
             * Report a restricted path.
             * @param {node} node representing the restricted path reference
             * @returns {void}
             * @private
             */
            function reportPath(node) {
                const importSource = node.source.value.trim();
                const customMessage = restrictedPathMessages[importSource] && restrictedPathMessages[importSource].message;
                context.report({
                    node,
                    messageId: customMessage ? "pathWithCustomMessage" : "path",
                    data: {
                        importSource,
                        customMessage
                    }
                });
            }
            /**
             * Report a restricted path specifically for patterns.
             * @param {node} node representing the restricted path reference
             * @returns {void}
             * @private
             */
            function reportPathForPatterns(node) {
                const importSource = node.source.value.trim();
                context.report({
                    node,
                    messageId: "patterns",
                    data: {
                        importSource
                    }
                });
            }
            /**
             * Report a restricted path specifically when using the '*' import.
             * @param {string} importSource path of the import
             * @param {node} node representing the restricted path reference
             * @returns {void}
             * @private
             */
            function reportPathForEverythingImported(importSource, node) {
                const importNames = restrictedPathMessages[importSource].importNames;
                const customMessage = restrictedPathMessages[importSource] && restrictedPathMessages[importSource].message;
                context.report({
                    node,
                    messageId: customMessage ? "everythingWithCustomMessage" : "everything",
                    data: {
                        importSource,
                        importNames,
                        customMessage
                    }
                });
            }
            /**
             * Check if the given importSource is restricted because '*' is being imported.
             * @param {string} importSource path of the import
             * @param {Set.<string>} importNames Set of import names that are being imported
             * @returns {boolean} whether the path is restricted
             * @private
             */
            function isRestrictedForEverythingImported(importSource, importNames) {
                return Object.prototype.hasOwnProperty.call(restrictedPathMessages, importSource) &&
                    restrictedPathMessages[importSource].importNames &&
                    isEverythingImported(importNames);
            }
            /**
             * Check if the given importNames are restricted given a list of restrictedImportNames.
             * @param {Set.<string>} importNames Set of import names that are being imported
             * @param {string[]} restrictedImportNames array of import names that are restricted for this import
             * @returns {boolean} whether the objectName is restricted
             * @private
             */
            function isRestrictedObject(importNames, restrictedImportNames) {
                return restrictedImportNames.some(restrictedObjectName => (importNames.has(restrictedObjectName)));
            }
            /**
             * Check if the given importSource is a restricted path.
             * @param {string} importSource path of the import
             * @param {Set.<string>} importNames Set of import names that are being imported
             * @returns {boolean} whether the variable is a restricted path or not
             * @private
             */
            function isRestrictedPath(importSource, importNames) {
                let isRestricted = false;
                if (Object.prototype.hasOwnProperty.call(restrictedPathMessages, importSource)) {
                    if (restrictedPathMessages[importSource].importNames) {
                        isRestricted = isRestrictedObject(importNames, restrictedPathMessages[importSource].importNames);
                    }
                    else {
                        isRestricted = true;
                    }
                }
                return isRestricted;
            }
            /**
             * Check if the given importSource is restricted by a pattern.
             * @param {string} importSource path of the import
             * @returns {boolean} whether the variable is a restricted pattern or not
             * @private
             */
            function isRestrictedPattern(importSource) {
                return restrictedPatterns.length > 0 && restrictedPatternsMatcher.ignores(importSource);
            }
            /**
             * Checks a node to see if any problems should be reported.
             * @param {ASTNode} node The node to check.
             * @returns {void}
             * @private
             */
            function checkNode(node) {
                const importSource = node.source.value.trim();
                const importNames = node.specifiers ? node.specifiers.reduce((set, specifier) => {
                    if (specifier.type === "ImportDefaultSpecifier") {
                        set.add("default");
                    }
                    else if (specifier.type === "ImportNamespaceSpecifier") {
                        set.add("*");
                    }
                    else if (specifier.imported) {
                        set.add(specifier.imported.name);
                    }
                    else if (specifier.local) {
                        set.add(specifier.local.name);
                    }
                    return set;
                }, new Set()) : new Set();
                if (isRestrictedForEverythingImported(importSource, importNames)) {
                    reportPathForEverythingImported(importSource, node);
                }
                if (isRestrictedPath(importSource, importNames)) {
                    reportPath(node);
                }
                if (isRestrictedPattern(importSource)) {
                    reportPathForPatterns(node);
                }
            }
            return {
                ImportDeclaration: checkNode,
                ExportNamedDeclaration(node) {
                    if (node.source) {
                        checkNode(node);
                    }
                },
                ExportAllDeclaration: checkNode
            };
        }
    };
},
/* ignore/index.js */ function _(require, module, exports) {
    // A simple implementation of make-array
    function makeArray(subject) {
        return Array.isArray(subject)
            ? subject
            : [subject];
    }
    const REGEX_TEST_BLANK_LINE = /^\s+$/;
    const REGEX_REPLACE_LEADING_EXCAPED_EXCLAMATION = /^\\!/;
    const REGEX_REPLACE_LEADING_EXCAPED_HASH = /^\\#/;
    const REGEX_SPLITALL_CRLF = /\r?\n/g;
    // /foo,
    // ./foo,
    // ../foo,
    // .
    // ..
    const REGEX_TEST_INVALID_PATH = /^\.*\/|^\.+$/;
    const SLASH = '/';
    const KEY_IGNORE = typeof Symbol !== 'undefined'
        ? Symbol.for('node-ignore')
        /* istanbul ignore next */
        : 'node-ignore';
    const define = (object, key, value) => Object.defineProperty(object, key, { value });
    const REGEX_REGEXP_RANGE = /([0-z])-([0-z])/g;
    // Sanitize the range of a regular expression
    // The cases are complicated, see test cases for details
    const sanitizeRange = range => range.replace(REGEX_REGEXP_RANGE, (match, from, to) => from.charCodeAt(0) <= to.charCodeAt(0)
        ? match
        // Invalid range (out of order) which is ok for gitignore rules but
        //   fatal for JavaScript regular expression, so eliminate it.
        : '');
    // > If the pattern ends with a slash,
    // > it is removed for the purpose of the following description,
    // > but it would only find a match with a directory.
    // > In other words, foo/ will match a directory foo and paths underneath it,
    // > but will not match a regular file or a symbolic link foo
    // >  (this is consistent with the way how pathspec works in general in Git).
    // '`foo/`' will not match regular file '`foo`' or symbolic link '`foo`'
    // -> ignore-rules will not deal with it, because it costs extra `fs.stat` call
    //      you could use option `mark: true` with `glob`
    // '`foo/`' should not continue with the '`..`'
    const REPLACERS = [
        // > Trailing spaces are ignored unless they are quoted with backslash ("\")
        [
            // (a\ ) -> (a )
            // (a  ) -> (a)
            // (a \ ) -> (a  )
            /\\?\s+$/,
            match => match.indexOf('\\') === 0
                ? ' '
                : ''
        ],
        // replace (\ ) with ' '
        [
            /\\\s/g,
            () => ' '
        ],
        // Escape metacharacters
        // which is written down by users but means special for regular expressions.
        // > There are 12 characters with special meanings:
        // > - the backslash \,
        // > - the caret ^,
        // > - the dollar sign $,
        // > - the period or dot .,
        // > - the vertical bar or pipe symbol |,
        // > - the question mark ?,
        // > - the asterisk or star *,
        // > - the plus sign +,
        // > - the opening parenthesis (,
        // > - the closing parenthesis ),
        // > - and the opening square bracket [,
        // > - the opening curly brace {,
        // > These special characters are often called "metacharacters".
        [
            /[\\^$.|*+(){]/g,
            match => `\\${match}`
        ],
        [
            // > [abc] matches any character inside the brackets
            // >    (in this case a, b, or c);
            /\[([^\]/]*)($|\])/g,
            (match, p1, p2) => p2 === ']'
                ? `[${sanitizeRange(p1)}]`
                : `\\${match}`
        ],
        [
            // > a question mark (?) matches a single character
            /(?!\\)\?/g,
            () => '[^/]'
        ],
        // leading slash
        [
            // > A leading slash matches the beginning of the pathname.
            // > For example, "/*.c" matches "cat-file.c" but not "mozilla-sha1/sha1.c".
            // A leading slash matches the beginning of the pathname
            /^\//,
            () => '^'
        ],
        // replace special metacharacter slash after the leading slash
        [
            /\//g,
            () => '\\/'
        ],
        [
            // > A leading "**" followed by a slash means match in all directories.
            // > For example, "**/foo" matches file or directory "foo" anywhere,
            // > the same as pattern "foo".
            // > "**/foo/bar" matches file or directory "bar" anywhere that is directly
            // >   under directory "foo".
            // Notice that the '*'s have been replaced as '\\*'
            /^\^*\\\*\\\*\\\//,
            // '**/foo' <-> 'foo'
            () => '^(?:.*\\/)?'
        ],
        // ending
        [
            // 'js' will not match 'js.'
            // 'ab' will not match 'abc'
            /(?:[^*])$/,
            // WTF!
            // https://git-scm.com/docs/gitignore
            // changes in [2.22.1](https://git-scm.com/docs/gitignore/2.22.1)
            // which re-fixes #24, #38
            // > If there is a separator at the end of the pattern then the pattern
            // > will only match directories, otherwise the pattern can match both
            // > files and directories.
            // 'js*' will not match 'a.js'
            // 'js/' will not match 'a.js'
            // 'js' will match 'a.js' and 'a.js/'
            match => /\/$/.test(match)
                // foo/ will not match 'foo'
                ? `${match}$`
                // foo matches 'foo' and 'foo/'
                : `${match}(?=$|\\/$)`
        ],
        // starting
        [
            // there will be no leading '/'
            //   (which has been replaced by section "leading slash")
            // If starts with '**', adding a '^' to the regular expression also works
            /^(?=[^^])/,
            function startingReplacer() {
                // If has a slash `/` at the beginning or middle
                return !/\/(?!$)/.test(this)
                    // > Prior to 2.22.1
                    // > If the pattern does not contain a slash /,
                    // >   Git treats it as a shell glob pattern
                    // Actually, if there is only a trailing slash,
                    //   git also treats it as a shell glob pattern
                    // After 2.22.1 (compatible but clearer)
                    // > If there is a separator at the beginning or middle (or both)
                    // > of the pattern, then the pattern is relative to the directory
                    // > level of the particular .gitignore file itself.
                    // > Otherwise the pattern may also match at any level below
                    // > the .gitignore level.
                    ? '(?:^|\\/)'
                    // > Otherwise, Git treats the pattern as a shell glob suitable for
                    // >   consumption by fnmatch(3)
                    : '^';
            }
        ],
        // two globstars
        [
            // Use lookahead assertions so that we could match more than one `'/**'`
            /\\\/\\\*\\\*(?=\\\/|$)/g,
            // Zero, one or several directories
            // should not use '*', or it will be replaced by the next replacer
            // Check if it is not the last `'/**'`
            (_, index, str) => index + 6 < str.length
                // case: /**/
                // > A slash followed by two consecutive asterisks then a slash matches
                // >   zero or more directories.
                // > For example, "a/**/b" matches "a/b", "a/x/b", "a/x/y/b" and so on.
                // '/**/'
                ? '(?:\\/[^\\/]+)*'
                // case: /**
                // > A trailing `"/**"` matches everything inside.
                // #21: everything inside but it should not include the current folder
                : '\\/.+'
        ],
        // intermediate wildcards
        [
            // Never replace escaped '*'
            // ignore rule '\*' will match the path '*'
            // 'abc.*/' -> go
            // 'abc.*'  -> skip this rule
            /(^|[^\\]+)\\\*(?=.+)/g,
            // '*.js' matches '.js'
            // '*.js' doesn't match 'abc'
            (_, p1) => `${p1}[^\\/]*`
        ],
        // trailing wildcard
        [
            /(\^|\\\/)?\\\*$/,
            (_, p1) => {
                const prefix = p1
                    // '\^':
                    // '/*' does not match ''
                    // '/*' does not match everything
                    // '\\\/':
                    // 'abc/*' does not match 'abc/'
                    ? `${p1}[^/]+`
                    // 'a*' matches 'a'
                    // 'a*' matches 'aa'
                    : '[^/]*';
                return `${prefix}(?=$|\\/$)`;
            }
        ],
        [
            // unescape
            /\\\\\\/g,
            () => '\\'
        ]
    ];
    // A simple cache, because an ignore rule only has only one certain meaning
    const regexCache = Object.create(null);
    // @param {pattern}
    const makeRegex = (pattern, negative, ignorecase) => {
        const r = regexCache[pattern];
        if (r) {
            return r;
        }
        // const replacers = negative
        //   ? NEGATIVE_REPLACERS
        //   : POSITIVE_REPLACERS
        const source = REPLACERS.reduce((prev, current) => prev.replace(current[0], current[1].bind(pattern)), pattern);
        return regexCache[pattern] = ignorecase
            ? new RegExp(source, 'i')
            : new RegExp(source);
    };
    const isString = subject => typeof subject === 'string';
    // > A blank line matches no files, so it can serve as a separator for readability.
    const checkPattern = pattern => pattern
        && isString(pattern)
        && !REGEX_TEST_BLANK_LINE.test(pattern)
        // > A line starting with # serves as a comment.
        && pattern.indexOf('#') !== 0;
    const splitPattern = pattern => pattern.split(REGEX_SPLITALL_CRLF);
    class IgnoreRule {
        constructor(origin, pattern, negative, regex) {
            this.origin = origin;
            this.pattern = pattern;
            this.negative = negative;
            this.regex = regex;
        }
    }
    const createRule = (pattern, ignorecase) => {
        const origin = pattern;
        let negative = false;
        // > An optional prefix "!" which negates the pattern;
        if (pattern.indexOf('!') === 0) {
            negative = true;
            pattern = pattern.substr(1);
        }
        pattern = pattern
            // > Put a backslash ("\") in front of the first "!" for patterns that
            // >   begin with a literal "!", for example, `"\!important!.txt"`.
            .replace(REGEX_REPLACE_LEADING_EXCAPED_EXCLAMATION, '!')
            // > Put a backslash ("\") in front of the first hash for patterns that
            // >   begin with a hash.
            .replace(REGEX_REPLACE_LEADING_EXCAPED_HASH, '#');
        const regex = makeRegex(pattern, negative, ignorecase);
        return new IgnoreRule(origin, pattern, negative, regex);
    };
    const throwError = (message, Ctor) => {
        throw new Ctor(message);
    };
    const checkPath = (path, originalPath, doThrow) => {
        if (!isString(path)) {
            return doThrow(`path must be a string, but got \`${originalPath}\``, TypeError);
        }
        // We don't know if we should ignore '', so throw
        if (!path) {
            return doThrow(`path must not be empty`, TypeError);
        }
        // Check if it is a relative path
        if (checkPath.isNotRelative(path)) {
            const r = '`path.relative()`d';
            return doThrow(`path should be a ${r} string, but got "${originalPath}"`, RangeError);
        }
        return true;
    };
    const isNotRelative = path => REGEX_TEST_INVALID_PATH.test(path);
    checkPath.isNotRelative = isNotRelative;
    checkPath.convert = p => p;
    class Ignore {
        constructor({ ignorecase = true } = {}) {
            this._rules = [];
            this._ignorecase = ignorecase;
            define(this, KEY_IGNORE, true);
            this._initCache();
        }
        _initCache() {
            this._ignoreCache = Object.create(null);
            this._testCache = Object.create(null);
        }
        _addPattern(pattern) {
            // #32
            if (pattern && pattern[KEY_IGNORE]) {
                this._rules = this._rules.concat(pattern._rules);
                this._added = true;
                return;
            }
            if (checkPattern(pattern)) {
                const rule = createRule(pattern, this._ignorecase);
                this._added = true;
                this._rules.push(rule);
            }
        }
        // @param {Array<string> | string | Ignore} pattern
        add(pattern) {
            this._added = false;
            makeArray(isString(pattern)
                ? splitPattern(pattern)
                : pattern).forEach(this._addPattern, this);
            // Some rules have just added to the ignore,
            // making the behavior changed.
            if (this._added) {
                this._initCache();
            }
            return this;
        }
        // legacy
        addPattern(pattern) {
            return this.add(pattern);
        }
        //          |           ignored : unignored
        // negative |   0:0   |   0:1   |   1:0   |   1:1
        // -------- | ------- | ------- | ------- | --------
        //     0    |  TEST   |  TEST   |  SKIP   |    X
        //     1    |  TESTIF |  SKIP   |  TEST   |    X
        // - SKIP: always skip
        // - TEST: always test
        // - TESTIF: only test if checkUnignored
        // - X: that never happen
        // @param {boolean} whether should check if the path is unignored,
        //   setting `checkUnignored` to `false` could reduce additional
        //   path matching.
        // @returns {TestResult} true if a file is ignored
        _testOne(path, checkUnignored) {
            let ignored = false;
            let unignored = false;
            this._rules.forEach(rule => {
                const { negative } = rule;
                if (unignored === negative && ignored !== unignored
                    || negative && !ignored && !unignored && !checkUnignored) {
                    return;
                }
                const matched = rule.regex.test(path);
                if (matched) {
                    ignored = !negative;
                    unignored = negative;
                }
            });
            return {
                ignored,
                unignored
            };
        }
        // @returns {TestResult}
        _test(originalPath, cache, checkUnignored, slices) {
            const path = originalPath
                // Supports nullable path
                && checkPath.convert(originalPath);
            checkPath(path, originalPath, throwError);
            return this._t(path, cache, checkUnignored, slices);
        }
        _t(path, cache, checkUnignored, slices) {
            if (path in cache) {
                return cache[path];
            }
            if (!slices) {
                // path/to/a.js
                // ['path', 'to', 'a.js']
                slices = path.split(SLASH);
            }
            slices.pop();
            // If the path has no parent directory, just test it
            if (!slices.length) {
                return cache[path] = this._testOne(path, checkUnignored);
            }
            const parent = this._t(slices.join(SLASH) + SLASH, cache, checkUnignored, slices);
            // If the path contains a parent directory, check the parent first
            return cache[path] = parent.ignored
                // > It is not possible to re-include a file if a parent directory of
                // >   that file is excluded.
                ? parent
                : this._testOne(path, checkUnignored);
        }
        ignores(path) {
            return this._test(path, this._ignoreCache, false).ignored;
        }
        createFilter() {
            return path => !this.ignores(path);
        }
        filter(paths) {
            return makeArray(paths).filter(this.createFilter());
        }
        // @returns {TestResult}
        test(path) {
            return this._test(path, this._testCache, true);
        }
    }
    const factory = options => new Ignore(options);
    const returnFalse = () => false;
    const isPathValid = path => checkPath(path && checkPath.convert(path), path, returnFalse);
    factory.isPathValid = isPathValid;
    // Fixes typescript
    factory.default = factory;
    module.exports = factory;
    // Windows
    // --------------------------------------------------------------
    /* istanbul ignore if  */
    if (
    // Detect `process` so that it can run in browsers.
    typeof process !== 'undefined'
        && (process.env && process.env.IGNORE_TEST_WIN32
            || process.platform === 'win32')) {
        /* eslint no-control-regex: "off" */
        const makePosix = str => /^\\\\\?\\/.test(str)
            || /["<>|\u0000-\u001F]+/u.test(str)
            ? str
            : str.replace(/\\/g, '/');
        checkPath.convert = makePosix;
        // 'C:\\foo'     <- 'C:\\foo' has been converted to 'C:/'
        // 'd:\\foo'
        const REGIX_IS_WINDOWS_PATH_ABSOLUTE = /^[a-z]:\//i;
        checkPath.isNotRelative = path => REGIX_IS_WINDOWS_PATH_ABSOLUTE.test(path)
            || isNotRelative(path);
    }
},
/* eslint/lib/rules/no-restricted-modules.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const DEFAULT_MESSAGE_TEMPLATE = "'{{moduleName}}' module is restricted from being used.";
    const CUSTOM_MESSAGE_TEMPLATE = "'{{moduleName}}' module is restricted from being used. {{customMessage}}";
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const ignore = require(309) /* ignore */;
    const arrayOfStrings = {
        type: "array",
        items: { type: "string" },
        uniqueItems: true
    };
    const arrayOfStringsOrObjects = {
        type: "array",
        items: {
            anyOf: [
                { type: "string" },
                {
                    type: "object",
                    properties: {
                        name: { type: "string" },
                        message: {
                            type: "string",
                            minLength: 1
                        }
                    },
                    additionalProperties: false,
                    required: ["name"]
                }
            ]
        },
        uniqueItems: true
    };
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow specified modules when loaded by `require`",
                category: "Node.js and CommonJS",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-restricted-modules"
            },
            schema: {
                anyOf: [
                    arrayOfStringsOrObjects,
                    {
                        type: "array",
                        items: {
                            type: "object",
                            properties: {
                                paths: arrayOfStringsOrObjects,
                                patterns: arrayOfStrings
                            },
                            additionalProperties: false
                        },
                        additionalItems: false
                    }
                ]
            }
        },
        create(context) {
            const options = Array.isArray(context.options) ? context.options : [];
            const isPathAndPatternsObject = typeof options[0] === "object" &&
                (Object.prototype.hasOwnProperty.call(options[0], "paths") || Object.prototype.hasOwnProperty.call(options[0], "patterns"));
            const restrictedPaths = (isPathAndPatternsObject ? options[0].paths : context.options) || [];
            const restrictedPatterns = (isPathAndPatternsObject ? options[0].patterns : []) || [];
            const restrictedPathMessages = restrictedPaths.reduce((memo, importName) => {
                if (typeof importName === "string") {
                    memo[importName] = null;
                }
                else {
                    memo[importName.name] = importName.message;
                }
                return memo;
            }, {});
            // if no imports are restricted we don"t need to check
            if (Object.keys(restrictedPaths).length === 0 && restrictedPatterns.length === 0) {
                return {};
            }
            const ig = ignore().add(restrictedPatterns);
            /**
             * Function to check if a node is a string literal.
             * @param {ASTNode} node The node to check.
             * @returns {boolean} If the node is a string literal.
             */
            function isString(node) {
                return node && node.type === "Literal" && typeof node.value === "string";
            }
            /**
             * Function to check if a node is a require call.
             * @param {ASTNode} node The node to check.
             * @returns {boolean} If the node is a require call.
             */
            function isRequireCall(node) {
                return node.callee.type === "Identifier" && node.callee.name === "require";
            }
            /**
             * Report a restricted path.
             * @param {node} node representing the restricted path reference
             * @returns {void}
             * @private
             */
            function reportPath(node) {
                const moduleName = node.arguments[0].value.trim();
                const customMessage = restrictedPathMessages[moduleName];
                const message = customMessage
                    ? CUSTOM_MESSAGE_TEMPLATE
                    : DEFAULT_MESSAGE_TEMPLATE;
                context.report({
                    node,
                    message,
                    data: {
                        moduleName,
                        customMessage
                    }
                });
            }
            /**
             * Check if the given name is a restricted path name
             * @param {string} name name of a variable
             * @returns {boolean} whether the variable is a restricted path or not
             * @private
             */
            function isRestrictedPath(name) {
                return Object.prototype.hasOwnProperty.call(restrictedPathMessages, name);
            }
            return {
                CallExpression(node) {
                    if (isRequireCall(node)) {
                        // node has arguments and first argument is string
                        if (node.arguments.length && isString(node.arguments[0])) {
                            const moduleName = node.arguments[0].value.trim();
                            // check if argument value is in restricted modules array
                            if (isRestrictedPath(moduleName)) {
                                reportPath(node);
                            }
                            if (restrictedPatterns.length > 0 && ig.ignores(moduleName)) {
                                context.report({
                                    node,
                                    message: "'{{moduleName}}' module is restricted from being used by a pattern.",
                                    data: { moduleName }
                                });
                            }
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-restricted-properties.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow certain properties on certain objects",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-restricted-properties"
            },
            schema: {
                type: "array",
                items: {
                    anyOf: [
                        {
                            type: "object",
                            properties: {
                                object: {
                                    type: "string"
                                },
                                property: {
                                    type: "string"
                                },
                                message: {
                                    type: "string"
                                }
                            },
                            additionalProperties: false,
                            required: ["object"]
                        },
                        {
                            type: "object",
                            properties: {
                                object: {
                                    type: "string"
                                },
                                property: {
                                    type: "string"
                                },
                                message: {
                                    type: "string"
                                }
                            },
                            additionalProperties: false,
                            required: ["property"]
                        }
                    ]
                },
                uniqueItems: true
            }
        },
        create(context) {
            const restrictedCalls = context.options;
            if (restrictedCalls.length === 0) {
                return {};
            }
            const restrictedProperties = new Map();
            const globallyRestrictedObjects = new Map();
            const globallyRestrictedProperties = new Map();
            restrictedCalls.forEach(option => {
                const objectName = option.object;
                const propertyName = option.property;
                if (typeof objectName === "undefined") {
                    globallyRestrictedProperties.set(propertyName, { message: option.message });
                }
                else if (typeof propertyName === "undefined") {
                    globallyRestrictedObjects.set(objectName, { message: option.message });
                }
                else {
                    if (!restrictedProperties.has(objectName)) {
                        restrictedProperties.set(objectName, new Map());
                    }
                    restrictedProperties.get(objectName).set(propertyName, {
                        message: option.message
                    });
                }
            });
            /**
             * Checks to see whether a property access is restricted, and reports it if so.
             * @param {ASTNode} node The node to report
             * @param {string} objectName The name of the object
             * @param {string} propertyName The name of the property
             * @returns {undefined}
             */
            function checkPropertyAccess(node, objectName, propertyName) {
                if (propertyName === null) {
                    return;
                }
                const matchedObject = restrictedProperties.get(objectName);
                const matchedObjectProperty = matchedObject ? matchedObject.get(propertyName) : globallyRestrictedObjects.get(objectName);
                const globalMatchedProperty = globallyRestrictedProperties.get(propertyName);
                if (matchedObjectProperty) {
                    const message = matchedObjectProperty.message ? ` ${matchedObjectProperty.message}` : "";
                    context.report({
                        node,
                        // eslint-disable-next-line eslint-plugin/report-message-format
                        message: "'{{objectName}}.{{propertyName}}' is restricted from being used.{{message}}",
                        data: {
                            objectName,
                            propertyName,
                            message
                        }
                    });
                }
                else if (globalMatchedProperty) {
                    const message = globalMatchedProperty.message ? ` ${globalMatchedProperty.message}` : "";
                    context.report({
                        node,
                        // eslint-disable-next-line eslint-plugin/report-message-format
                        message: "'{{propertyName}}' is restricted from being used.{{message}}",
                        data: {
                            propertyName,
                            message
                        }
                    });
                }
            }
            /**
             * Checks property accesses in a destructuring assignment expression, e.g. `var foo; ({foo} = bar);`
             * @param {ASTNode} node An AssignmentExpression or AssignmentPattern node
             * @returns {undefined}
             */
            function checkDestructuringAssignment(node) {
                if (node.right.type === "Identifier") {
                    const objectName = node.right.name;
                    if (node.left.type === "ObjectPattern") {
                        node.left.properties.forEach(property => {
                            checkPropertyAccess(node.left, objectName, astUtils.getStaticPropertyName(property));
                        });
                    }
                }
            }
            return {
                MemberExpression(node) {
                    checkPropertyAccess(node, node.object && node.object.name, astUtils.getStaticPropertyName(node));
                },
                VariableDeclarator(node) {
                    if (node.init && node.init.type === "Identifier") {
                        const objectName = node.init.name;
                        if (node.id.type === "ObjectPattern") {
                            node.id.properties.forEach(property => {
                                checkPropertyAccess(node.id, objectName, astUtils.getStaticPropertyName(property));
                            });
                        }
                    }
                },
                AssignmentExpression: checkDestructuringAssignment,
                AssignmentPattern: checkDestructuringAssignment
            };
        }
    };
},
/* eslint/lib/rules/no-restricted-syntax.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow specified syntax",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-restricted-syntax"
            },
            schema: {
                type: "array",
                items: {
                    oneOf: [
                        {
                            type: "string"
                        },
                        {
                            type: "object",
                            properties: {
                                selector: { type: "string" },
                                message: { type: "string" }
                            },
                            required: ["selector"],
                            additionalProperties: false
                        }
                    ]
                },
                uniqueItems: true,
                minItems: 0
            }
        },
        create(context) {
            return context.options.reduce((result, selectorOrObject) => {
                const isStringFormat = (typeof selectorOrObject === "string");
                const hasCustomMessage = !isStringFormat && Boolean(selectorOrObject.message);
                const selector = isStringFormat ? selectorOrObject : selectorOrObject.selector;
                const message = hasCustomMessage ? selectorOrObject.message : "Using '{{selector}}' is not allowed.";
                return Object.assign(result, {
                    [selector](node) {
                        context.report({
                            node,
                            message,
                            data: hasCustomMessage ? {} : { selector }
                        });
                    }
                });
            }, {});
        }
    };
},
/* eslint/lib/rules/no-return-assign.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const SENTINEL_TYPE = /^(?:[a-zA-Z]+?Statement|ArrowFunctionExpression|FunctionExpression|ClassExpression)$/u;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow assignment operators in `return` statements",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-return-assign"
            },
            schema: [
                {
                    enum: ["except-parens", "always"]
                }
            ]
        },
        create(context) {
            const always = (context.options[0] || "except-parens") !== "except-parens";
            const sourceCode = context.getSourceCode();
            return {
                AssignmentExpression(node) {
                    if (!always && astUtils.isParenthesised(sourceCode, node)) {
                        return;
                    }
                    let currentChild = node;
                    let parent = currentChild.parent;
                    // Find ReturnStatement or ArrowFunctionExpression in ancestors.
                    while (parent && !SENTINEL_TYPE.test(parent.type)) {
                        currentChild = parent;
                        parent = parent.parent;
                    }
                    // Reports.
                    if (parent && parent.type === "ReturnStatement") {
                        context.report({
                            node: parent,
                            message: "Return statement should not contain assignment."
                        });
                    }
                    else if (parent && parent.type === "ArrowFunctionExpression" && parent.body === currentChild) {
                        context.report({
                            node: parent,
                            message: "Arrow function should not return assignment."
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-return-await.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const message = "Redundant use of `await` on a return value.";
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary `return await`",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-return-await"
            },
            fixable: null,
            schema: []
        },
        create(context) {
            /**
             * Reports a found unnecessary `await` expression.
             * @param {ASTNode} node The node representing the `await` expression to report
             * @returns {void}
             */
            function reportUnnecessaryAwait(node) {
                context.report({
                    node: context.getSourceCode().getFirstToken(node),
                    loc: node.loc,
                    message
                });
            }
            /**
             * Determines whether a thrown error from this node will be caught/handled within this function rather than immediately halting
             * this function. For example, a statement in a `try` block will always have an error handler. A statement in
             * a `catch` block will only have an error handler if there is also a `finally` block.
             * @param {ASTNode} node A node representing a location where an could be thrown
             * @returns {boolean} `true` if a thrown error will be caught/handled in this function
             */
            function hasErrorHandler(node) {
                let ancestor = node;
                while (!astUtils.isFunction(ancestor) && ancestor.type !== "Program") {
                    if (ancestor.parent.type === "TryStatement" && (ancestor === ancestor.parent.block || ancestor === ancestor.parent.handler && ancestor.parent.finalizer)) {
                        return true;
                    }
                    ancestor = ancestor.parent;
                }
                return false;
            }
            /**
             * Checks if a node is placed in tail call position. Once `return` arguments (or arrow function expressions) can be a complex expression,
             * an `await` expression could or could not be unnecessary by the definition of this rule. So we're looking for `await` expressions that are in tail position.
             * @param {ASTNode} node A node representing the `await` expression to check
             * @returns {boolean} The checking result
             */
            function isInTailCallPosition(node) {
                if (node.parent.type === "ArrowFunctionExpression") {
                    return true;
                }
                if (node.parent.type === "ReturnStatement") {
                    return !hasErrorHandler(node.parent);
                }
                if (node.parent.type === "ConditionalExpression" && (node === node.parent.consequent || node === node.parent.alternate)) {
                    return isInTailCallPosition(node.parent);
                }
                if (node.parent.type === "LogicalExpression" && node === node.parent.right) {
                    return isInTailCallPosition(node.parent);
                }
                if (node.parent.type === "SequenceExpression" && node === node.parent.expressions[node.parent.expressions.length - 1]) {
                    return isInTailCallPosition(node.parent);
                }
                return false;
            }
            return {
                AwaitExpression(node) {
                    if (isInTailCallPosition(node) && !hasErrorHandler(node)) {
                        reportUnnecessaryAwait(node);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-script-url.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `javascript:` urls",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-script-url"
            },
            schema: []
        },
        create(context) {
            return {
                Literal(node) {
                    if (node.value && typeof node.value === "string") {
                        const value = node.value.toLowerCase();
                        if (value.indexOf("javascript:") === 0) {
                            context.report({ node, message: "Script URL is a form of eval." });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-self-assign.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const SPACES = /\s+/gu;
    /**
     * Checks whether the property of 2 given member expression nodes are the same
     * property or not.
     * @param {ASTNode} left A member expression node to check.
     * @param {ASTNode} right Another member expression node to check.
     * @returns {boolean} `true` if the member expressions have the same property.
     */
    function isSameProperty(left, right) {
        if (left.property.type === "Identifier" &&
            left.property.type === right.property.type &&
            left.property.name === right.property.name &&
            left.computed === right.computed) {
            return true;
        }
        const lname = astUtils.getStaticPropertyName(left);
        const rname = astUtils.getStaticPropertyName(right);
        return lname !== null && lname === rname;
    }
    /**
     * Checks whether 2 given member expression nodes are the reference to the same
     * property or not.
     * @param {ASTNode} left A member expression node to check.
     * @param {ASTNode} right Another member expression node to check.
     * @returns {boolean} `true` if the member expressions are the reference to the
     *  same property or not.
     */
    function isSameMember(left, right) {
        if (!isSameProperty(left, right)) {
            return false;
        }
        const lobj = left.object;
        const robj = right.object;
        if (lobj.type !== robj.type) {
            return false;
        }
        if (lobj.type === "MemberExpression") {
            return isSameMember(lobj, robj);
        }
        if (lobj.type === "ThisExpression") {
            return true;
        }
        return lobj.type === "Identifier" && lobj.name === robj.name;
    }
    /**
     * Traverses 2 Pattern nodes in parallel, then reports self-assignments.
     * @param {ASTNode|null} left A left node to traverse. This is a Pattern or
     *      a Property.
     * @param {ASTNode|null} right A right node to traverse. This is a Pattern or
     *      a Property.
     * @param {boolean} props The flag to check member expressions as well.
     * @param {Function} report A callback function to report.
     * @returns {void}
     */
    function eachSelfAssignment(left, right, props, report) {
        if (!left || !right) {
            // do nothing
        }
        else if (left.type === "Identifier" &&
            right.type === "Identifier" &&
            left.name === right.name) {
            report(right);
        }
        else if (left.type === "ArrayPattern" &&
            right.type === "ArrayExpression") {
            const end = Math.min(left.elements.length, right.elements.length);
            for (let i = 0; i < end; ++i) {
                const leftElement = left.elements[i];
                const rightElement = right.elements[i];
                // Avoid cases such as [...a] = [...a, 1]
                if (leftElement &&
                    leftElement.type === "RestElement" &&
                    i < right.elements.length - 1) {
                    break;
                }
                eachSelfAssignment(leftElement, rightElement, props, report);
                // After a spread element, those indices are unknown.
                if (rightElement && rightElement.type === "SpreadElement") {
                    break;
                }
            }
        }
        else if (left.type === "RestElement" &&
            right.type === "SpreadElement") {
            eachSelfAssignment(left.argument, right.argument, props, report);
        }
        else if (left.type === "ObjectPattern" &&
            right.type === "ObjectExpression" &&
            right.properties.length >= 1) {
            /*
             * Gets the index of the last spread property.
             * It's possible to overwrite properties followed by it.
             */
            let startJ = 0;
            for (let i = right.properties.length - 1; i >= 0; --i) {
                const propType = right.properties[i].type;
                if (propType === "SpreadElement" || propType === "ExperimentalSpreadProperty") {
                    startJ = i + 1;
                    break;
                }
            }
            for (let i = 0; i < left.properties.length; ++i) {
                for (let j = startJ; j < right.properties.length; ++j) {
                    eachSelfAssignment(left.properties[i], right.properties[j], props, report);
                }
            }
        }
        else if (left.type === "Property" &&
            right.type === "Property" &&
            right.kind === "init" &&
            !right.method) {
            const leftName = astUtils.getStaticPropertyName(left);
            if (leftName !== null && leftName === astUtils.getStaticPropertyName(right)) {
                eachSelfAssignment(left.value, right.value, props, report);
            }
        }
        else if (props &&
            left.type === "MemberExpression" &&
            right.type === "MemberExpression" &&
            isSameMember(left, right)) {
            report(right);
        }
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow assignments where both sides are exactly the same",
                category: "Best Practices",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-self-assign"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        props: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const [{ props = true } = {}] = context.options;
            /**
             * Reports a given node as self assignments.
             * @param {ASTNode} node A node to report. This is an Identifier node.
             * @returns {void}
             */
            function report(node) {
                context.report({
                    node,
                    message: "'{{name}}' is assigned to itself.",
                    data: {
                        name: sourceCode.getText(node).replace(SPACES, "")
                    }
                });
            }
            return {
                AssignmentExpression(node) {
                    if (node.operator === "=") {
                        eachSelfAssignment(node.left, node.right, props, report);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-self-compare.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow comparisons where both sides are exactly the same",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-self-compare"
            },
            schema: []
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Determines whether two nodes are composed of the same tokens.
             * @param {ASTNode} nodeA The first node
             * @param {ASTNode} nodeB The second node
             * @returns {boolean} true if the nodes have identical token representations
             */
            function hasSameTokens(nodeA, nodeB) {
                const tokensA = sourceCode.getTokens(nodeA);
                const tokensB = sourceCode.getTokens(nodeB);
                return tokensA.length === tokensB.length &&
                    tokensA.every((token, index) => token.type === tokensB[index].type && token.value === tokensB[index].value);
            }
            return {
                BinaryExpression(node) {
                    const operators = new Set(["===", "==", "!==", "!=", ">", "<", ">=", "<="]);
                    if (operators.has(node.operator) && hasSameTokens(node.left, node.right)) {
                        context.report({ node, message: "Comparing to itself is potentially pointless." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-sequences.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow comma operators",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-sequences"
            },
            schema: []
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Parts of the grammar that are required to have parens.
             */
            const parenthesized = {
                DoWhileStatement: "test",
                IfStatement: "test",
                SwitchStatement: "discriminant",
                WhileStatement: "test",
                WithStatement: "object",
                ArrowFunctionExpression: "body"
                /*
                 * Omitting CallExpression - commas are parsed as argument separators
                 * Omitting NewExpression - commas are parsed as argument separators
                 * Omitting ForInStatement - parts aren't individually parenthesised
                 * Omitting ForStatement - parts aren't individually parenthesised
                 */
            };
            /**
             * Determines whether a node is required by the grammar to be wrapped in
             * parens, e.g. the test of an if statement.
             * @param {ASTNode} node The AST node
             * @returns {boolean} True if parens around node belong to parent node.
             */
            function requiresExtraParens(node) {
                return node.parent && parenthesized[node.parent.type] &&
                    node === node.parent[parenthesized[node.parent.type]];
            }
            /**
             * Check if a node is wrapped in parens.
             * @param {ASTNode} node The AST node
             * @returns {boolean} True if the node has a paren on each side.
             */
            function isParenthesised(node) {
                return astUtils.isParenthesised(sourceCode, node);
            }
            /**
             * Check if a node is wrapped in two levels of parens.
             * @param {ASTNode} node The AST node
             * @returns {boolean} True if two parens surround the node on each side.
             */
            function isParenthesisedTwice(node) {
                const previousToken = sourceCode.getTokenBefore(node, 1), nextToken = sourceCode.getTokenAfter(node, 1);
                return isParenthesised(node) && previousToken && nextToken &&
                    astUtils.isOpeningParenToken(previousToken) && previousToken.range[1] <= node.range[0] &&
                    astUtils.isClosingParenToken(nextToken) && nextToken.range[0] >= node.range[1];
            }
            return {
                SequenceExpression(node) {
                    // Always allow sequences in for statement update
                    if (node.parent.type === "ForStatement" &&
                        (node === node.parent.init || node === node.parent.update)) {
                        return;
                    }
                    // Wrapping a sequence in extra parens indicates intent
                    if (requiresExtraParens(node)) {
                        if (isParenthesisedTwice(node)) {
                            return;
                        }
                    }
                    else {
                        if (isParenthesised(node)) {
                            return;
                        }
                    }
                    const firstCommaToken = sourceCode.getTokenAfter(node.expressions[0], astUtils.isCommaToken);
                    context.report({ node, loc: firstCommaToken.loc, message: "Unexpected use of comma operator." });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-setter-return.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    const { findVariable } = require(250) /* eslint-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Determines whether the given identifier node is a reference to a global variable.
     * @param {ASTNode} node `Identifier` node to check.
     * @param {Scope} scope Scope to which the node belongs.
     * @returns {boolean} True if the identifier is a reference to a global variable.
     */
    function isGlobalReference(node, scope) {
        const variable = findVariable(scope, node);
        return variable !== null && variable.scope.type === "global" && variable.defs.length === 0;
    }
    /**
     * Determines whether the given node is an argument of the specified global method call, at the given `index` position.
     * E.g., for given `index === 1`, this function checks for `objectName.methodName(foo, node)`, where objectName is a global variable.
     * @param {ASTNode} node The node to check.
     * @param {Scope} scope Scope to which the node belongs.
     * @param {string} objectName Name of the global object.
     * @param {string} methodName Name of the method.
     * @param {number} index The given position.
     * @returns {boolean} `true` if the node is argument at the given position.
     */
    function isArgumentOfGlobalMethodCall(node, scope, objectName, methodName, index) {
        const parent = node.parent;
        return parent.type === "CallExpression" &&
            parent.arguments[index] === node &&
            parent.callee.type === "MemberExpression" &&
            astUtils.getStaticPropertyName(parent.callee) === methodName &&
            parent.callee.object.type === "Identifier" &&
            parent.callee.object.name === objectName &&
            isGlobalReference(parent.callee.object, scope);
    }
    /**
     * Determines whether the given node is used as a property descriptor.
     * @param {ASTNode} node The node to check.
     * @param {Scope} scope Scope to which the node belongs.
     * @returns {boolean} `true` if the node is a property descriptor.
     */
    function isPropertyDescriptor(node, scope) {
        if (isArgumentOfGlobalMethodCall(node, scope, "Object", "defineProperty", 2) ||
            isArgumentOfGlobalMethodCall(node, scope, "Reflect", "defineProperty", 2)) {
            return true;
        }
        const parent = node.parent;
        if (parent.type === "Property" &&
            parent.value === node) {
            const grandparent = parent.parent;
            if (grandparent.type === "ObjectExpression" &&
                (isArgumentOfGlobalMethodCall(grandparent, scope, "Object", "create", 1) ||
                    isArgumentOfGlobalMethodCall(grandparent, scope, "Object", "defineProperties", 1))) {
                return true;
            }
        }
        return false;
    }
    /**
     * Determines whether the given function node is used as a setter function.
     * @param {ASTNode} node The node to check.
     * @param {Scope} scope Scope to which the node belongs.
     * @returns {boolean} `true` if the node is a setter.
     */
    function isSetter(node, scope) {
        const parent = node.parent;
        if (parent.kind === "set" &&
            parent.value === node) {
            // Setter in an object literal or in a class
            return true;
        }
        if (parent.type === "Property" &&
            parent.value === node &&
            astUtils.getStaticPropertyName(parent) === "set" &&
            parent.parent.type === "ObjectExpression" &&
            isPropertyDescriptor(parent.parent, scope)) {
            // Setter in a property descriptor
            return true;
        }
        return false;
    }
    /**
     * Finds function's outer scope.
     * @param {Scope} scope Function's own scope.
     * @returns {Scope} Function's outer scope.
     */
    function getOuterScope(scope) {
        const upper = scope.upper;
        if (upper.type === "function-expression-name") {
            return upper.upper;
        }
        return upper;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow returning values from setters",
                category: "Possible Errors",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-setter-return"
            },
            schema: [],
            messages: {
                returnsValue: "Setter cannot return a value."
            }
        },
        create(context) {
            let funcInfo = null;
            /**
             * Creates and pushes to the stack a function info object for the given function node.
             * @param {ASTNode} node The function node.
             * @returns {void}
             */
            function enterFunction(node) {
                const outerScope = getOuterScope(context.getScope());
                funcInfo = {
                    upper: funcInfo,
                    isSetter: isSetter(node, outerScope)
                };
            }
            /**
             * Pops the current function info object from the stack.
             * @returns {void}
             */
            function exitFunction() {
                funcInfo = funcInfo.upper;
            }
            /**
             * Reports the given node.
             * @param {ASTNode} node Node to report.
             * @returns {void}
             */
            function report(node) {
                context.report({ node, messageId: "returnsValue" });
            }
            return {
                /*
                 * Function declarations cannot be setters, but we still have to track them in the `funcInfo` stack to avoid
                 * false positives, because a ReturnStatement node can belong to a function declaration inside a setter.
                 *
                 * Note: A previously declared function can be referenced and actually used as a setter in a property descriptor,
                 * but that's out of scope for this rule.
                 */
                FunctionDeclaration: enterFunction,
                FunctionExpression: enterFunction,
                ArrowFunctionExpression(node) {
                    enterFunction(node);
                    if (funcInfo.isSetter && node.expression) {
                        // { set: foo => bar } property descriptor. Report implicit return 'bar' as the equivalent for a return statement.
                        report(node.body);
                    }
                },
                "FunctionDeclaration:exit": exitFunction,
                "FunctionExpression:exit": exitFunction,
                "ArrowFunctionExpression:exit": exitFunction,
                ReturnStatement(node) {
                    // Global returns (e.g., at the top level of a Node module) don't have `funcInfo`.
                    if (funcInfo && funcInfo.isSetter && node.argument) {
                        report(node);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-shadow.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow variable declarations from shadowing variables declared in the outer scope",
                category: "Variables",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-shadow"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        builtinGlobals: { type: "boolean", default: false },
                        hoist: { enum: ["all", "functions", "never"], default: "functions" },
                        allow: {
                            type: "array",
                            items: {
                                type: "string"
                            }
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const options = {
                builtinGlobals: context.options[0] && context.options[0].builtinGlobals,
                hoist: (context.options[0] && context.options[0].hoist) || "functions",
                allow: (context.options[0] && context.options[0].allow) || []
            };
            /**
             * Check if variable name is allowed.
             * @param  {ASTNode} variable The variable to check.
             * @returns {boolean} Whether or not the variable name is allowed.
             */
            function isAllowed(variable) {
                return options.allow.indexOf(variable.name) !== -1;
            }
            /**
             * Checks if a variable of the class name in the class scope of ClassDeclaration.
             *
             * ClassDeclaration creates two variables of its name into its outer scope and its class scope.
             * So we should ignore the variable in the class scope.
             * @param {Object} variable The variable to check.
             * @returns {boolean} Whether or not the variable of the class name in the class scope of ClassDeclaration.
             */
            function isDuplicatedClassNameVariable(variable) {
                const block = variable.scope.block;
                return block.type === "ClassDeclaration" && block.id === variable.identifiers[0];
            }
            /**
             * Checks if a variable is inside the initializer of scopeVar.
             *
             * To avoid reporting at declarations such as `var a = function a() {};`.
             * But it should report `var a = function(a) {};` or `var a = function() { function a() {} };`.
             * @param {Object} variable The variable to check.
             * @param {Object} scopeVar The scope variable to look for.
             * @returns {boolean} Whether or not the variable is inside initializer of scopeVar.
             */
            function isOnInitializer(variable, scopeVar) {
                const outerScope = scopeVar.scope;
                const outerDef = scopeVar.defs[0];
                const outer = outerDef && outerDef.parent && outerDef.parent.range;
                const innerScope = variable.scope;
                const innerDef = variable.defs[0];
                const inner = innerDef && innerDef.name.range;
                return (outer &&
                    inner &&
                    outer[0] < inner[0] &&
                    inner[1] < outer[1] &&
                    ((innerDef.type === "FunctionName" && innerDef.node.type === "FunctionExpression") || innerDef.node.type === "ClassExpression") &&
                    outerScope === innerScope.upper);
            }
            /**
             * Get a range of a variable's identifier node.
             * @param {Object} variable The variable to get.
             * @returns {Array|undefined} The range of the variable's identifier node.
             */
            function getNameRange(variable) {
                const def = variable.defs[0];
                return def && def.name.range;
            }
            /**
             * Checks if a variable is in TDZ of scopeVar.
             * @param {Object} variable The variable to check.
             * @param {Object} scopeVar The variable of TDZ.
             * @returns {boolean} Whether or not the variable is in TDZ of scopeVar.
             */
            function isInTdz(variable, scopeVar) {
                const outerDef = scopeVar.defs[0];
                const inner = getNameRange(variable);
                const outer = getNameRange(scopeVar);
                return (inner &&
                    outer &&
                    inner[1] < outer[0] &&
                    // Excepts FunctionDeclaration if is {"hoist":"function"}.
                    (options.hoist !== "functions" || !outerDef || outerDef.node.type !== "FunctionDeclaration"));
            }
            /**
             * Checks the current context for shadowed variables.
             * @param {Scope} scope Fixme
             * @returns {void}
             */
            function checkForShadows(scope) {
                const variables = scope.variables;
                for (let i = 0; i < variables.length; ++i) {
                    const variable = variables[i];
                    // Skips "arguments" or variables of a class name in the class scope of ClassDeclaration.
                    if (variable.identifiers.length === 0 ||
                        isDuplicatedClassNameVariable(variable) ||
                        isAllowed(variable)) {
                        continue;
                    }
                    // Gets shadowed variable.
                    const shadowed = astUtils.getVariableByName(scope.upper, variable.name);
                    if (shadowed &&
                        (shadowed.identifiers.length > 0 || (options.builtinGlobals && "writeable" in shadowed)) &&
                        !isOnInitializer(variable, shadowed) &&
                        !(options.hoist !== "all" && isInTdz(variable, shadowed))) {
                        context.report({
                            node: variable.identifiers[0],
                            message: "'{{name}}' is already declared in the upper scope.",
                            data: variable
                        });
                    }
                }
            }
            return {
                "Program:exit"() {
                    const globalScope = context.getScope();
                    const stack = globalScope.childScopes.slice();
                    while (stack.length) {
                        const scope = stack.pop();
                        stack.push(...scope.childScopes);
                        checkForShadows(scope);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-shadow-restricted-names.js */ function _(require, module, exports) {
    /**
     * Determines if a variable safely shadows undefined.
     * This is the case when a variable named `undefined` is never assigned to a value (i.e. it always shares the same value
     * as the global).
     * @param {eslintScope.Variable} variable The variable to check
     * @returns {boolean} true if this variable safely shadows `undefined`
     */
    function safelyShadowsUndefined(variable) {
        return variable.name === "undefined" &&
            variable.references.every(ref => !ref.isWrite()) &&
            variable.defs.every(def => def.node.type === "VariableDeclarator" && def.node.init === null);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow identifiers from shadowing restricted names",
                category: "Variables",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-shadow-restricted-names"
            },
            schema: []
        },
        create(context) {
            const RESTRICTED = new Set(["undefined", "NaN", "Infinity", "arguments", "eval"]);
            return {
                "VariableDeclaration, :function, CatchClause"(node) {
                    for (const variable of context.getDeclaredVariables(node)) {
                        if (variable.defs.length > 0 && RESTRICTED.has(variable.name) && !safelyShadowsUndefined(variable)) {
                            context.report({
                                node: variable.defs[0].name,
                                message: "Shadowing of global property '{{idName}}'.",
                                data: {
                                    idName: variable.name
                                }
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-spaced-func.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "disallow spacing between function identifiers and their applications (deprecated)",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-spaced-func"
            },
            deprecated: true,
            replacedBy: ["func-call-spacing"],
            fixable: "whitespace",
            schema: []
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Check if open space is present in a function name
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function detectOpenSpaces(node) {
                const lastCalleeToken = sourceCode.getLastToken(node.callee);
                let prevToken = lastCalleeToken, parenToken = sourceCode.getTokenAfter(lastCalleeToken);
                // advances to an open parenthesis.
                while (parenToken &&
                    parenToken.range[1] < node.range[1] &&
                    parenToken.value !== "(") {
                    prevToken = parenToken;
                    parenToken = sourceCode.getTokenAfter(parenToken);
                }
                // look for a space between the callee and the open paren
                if (parenToken &&
                    parenToken.range[1] < node.range[1] &&
                    sourceCode.isSpaceBetweenTokens(prevToken, parenToken)) {
                    context.report({
                        node,
                        loc: lastCalleeToken.loc.start,
                        message: "Unexpected space between function name and paren.",
                        fix(fixer) {
                            return fixer.removeRange([prevToken.range[1], parenToken.range[0]]);
                        }
                    });
                }
            }
            return {
                CallExpression: detectOpenSpaces,
                NewExpression: detectOpenSpaces
            };
        }
    };
},
/* eslint/lib/rules/no-sparse-arrays.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow sparse arrays",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-sparse-arrays"
            },
            schema: []
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                ArrayExpression(node) {
                    const emptySpot = node.elements.indexOf(null) > -1;
                    if (emptySpot) {
                        context.report({ node, message: "Unexpected comma in middle of array." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-sync.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow synchronous methods",
                category: "Node.js and CommonJS",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-sync"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allowAtRootLevel: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const selector = context.options[0] && context.options[0].allowAtRootLevel
                ? ":function MemberExpression[property.name=/.*Sync$/]"
                : "MemberExpression[property.name=/.*Sync$/]";
            return {
                [selector](node) {
                    context.report({
                        node,
                        message: "Unexpected sync method: '{{propertyName}}'.",
                        data: {
                            propertyName: node.property.name
                        }
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-tabs.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const tabRegex = /\t+/gu;
    const anyNonWhitespaceRegex = /\S/u;
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "disallow all tabs",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-tabs"
            },
            schema: [{
                    type: "object",
                    properties: {
                        allowIndentationTabs: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const allowIndentationTabs = context.options && context.options[0] && context.options[0].allowIndentationTabs;
            return {
                Program(node) {
                    sourceCode.getLines().forEach((line, index) => {
                        let match;
                        while ((match = tabRegex.exec(line)) !== null) {
                            if (allowIndentationTabs && !anyNonWhitespaceRegex.test(line.slice(0, match.index))) {
                                continue;
                            }
                            context.report({
                                node,
                                loc: {
                                    start: {
                                        line: index + 1,
                                        column: match.index
                                    },
                                    end: {
                                        line: index + 1,
                                        column: match.index + match[0].length
                                    }
                                },
                                message: "Unexpected tab character."
                            });
                        }
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-template-curly-in-string.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow template literal placeholder syntax in regular strings",
                category: "Possible Errors",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-template-curly-in-string"
            },
            schema: []
        },
        create(context) {
            const regex = /\$\{[^}]+\}/u;
            return {
                Literal(node) {
                    if (typeof node.value === "string" && regex.test(node.value)) {
                        context.report({
                            node,
                            message: "Unexpected template string expression."
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-ternary.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow ternary operators",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-ternary"
            },
            schema: []
        },
        create(context) {
            return {
                ConditionalExpression(node) {
                    context.report({ node, message: "Ternary operator used." });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-this-before-super.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a given node is a constructor.
     * @param {ASTNode} node A node to check. This node type is one of
     *   `Program`, `FunctionDeclaration`, `FunctionExpression`, and
     *   `ArrowFunctionExpression`.
     * @returns {boolean} `true` if the node is a constructor.
     */
    function isConstructorFunction(node) {
        return (node.type === "FunctionExpression" &&
            node.parent.type === "MethodDefinition" &&
            node.parent.kind === "constructor");
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow `this`/`super` before calling `super()` in constructors",
                category: "ECMAScript 6",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-this-before-super"
            },
            schema: []
        },
        create(context) {
            /*
             * Information for each constructor.
             * - upper:      Information of the upper constructor.
             * - hasExtends: A flag which shows whether the owner class has a valid
             *   `extends` part.
             * - scope:      The scope of the owner class.
             * - codePath:   The code path of this constructor.
             */
            let funcInfo = null;
            /*
             * Information for each code path segment.
             * Each key is the id of a code path segment.
             * Each value is an object:
             * - superCalled:  The flag which shows `super()` called in all code paths.
             * - invalidNodes: The array of invalid ThisExpression and Super nodes.
             */
            let segInfoMap = Object.create(null);
            /**
             * Gets whether or not `super()` is called in a given code path segment.
             * @param {CodePathSegment} segment A code path segment to get.
             * @returns {boolean} `true` if `super()` is called.
             */
            function isCalled(segment) {
                return !segment.reachable || segInfoMap[segment.id].superCalled;
            }
            /**
             * Checks whether or not this is in a constructor.
             * @returns {boolean} `true` if this is in a constructor.
             */
            function isInConstructorOfDerivedClass() {
                return Boolean(funcInfo && funcInfo.isConstructor && funcInfo.hasExtends);
            }
            /**
             * Checks whether or not this is before `super()` is called.
             * @returns {boolean} `true` if this is before `super()` is called.
             */
            function isBeforeCallOfSuper() {
                return (isInConstructorOfDerivedClass() &&
                    !funcInfo.codePath.currentSegments.every(isCalled));
            }
            /**
             * Sets a given node as invalid.
             * @param {ASTNode} node A node to set as invalid. This is one of
             *      a ThisExpression and a Super.
             * @returns {void}
             */
            function setInvalid(node) {
                const segments = funcInfo.codePath.currentSegments;
                for (let i = 0; i < segments.length; ++i) {
                    const segment = segments[i];
                    if (segment.reachable) {
                        segInfoMap[segment.id].invalidNodes.push(node);
                    }
                }
            }
            /**
             * Sets the current segment as `super` was called.
             * @returns {void}
             */
            function setSuperCalled() {
                const segments = funcInfo.codePath.currentSegments;
                for (let i = 0; i < segments.length; ++i) {
                    const segment = segments[i];
                    if (segment.reachable) {
                        segInfoMap[segment.id].superCalled = true;
                    }
                }
            }
            return {
                /**
                 * Adds information of a constructor into the stack.
                 * @param {CodePath} codePath A code path which was started.
                 * @param {ASTNode} node The current node.
                 * @returns {void}
                 */
                onCodePathStart(codePath, node) {
                    if (isConstructorFunction(node)) {
                        // Class > ClassBody > MethodDefinition > FunctionExpression
                        const classNode = node.parent.parent.parent;
                        funcInfo = {
                            upper: funcInfo,
                            isConstructor: true,
                            hasExtends: Boolean(classNode.superClass &&
                                !astUtils.isNullOrUndefined(classNode.superClass)),
                            codePath
                        };
                    }
                    else {
                        funcInfo = {
                            upper: funcInfo,
                            isConstructor: false,
                            hasExtends: false,
                            codePath
                        };
                    }
                },
                /**
                 * Removes the top of stack item.
                 *
                 * And this treverses all segments of this code path then reports every
                 * invalid node.
                 * @param {CodePath} codePath A code path which was ended.
                 * @returns {void}
                 */
                onCodePathEnd(codePath) {
                    const isDerivedClass = funcInfo.hasExtends;
                    funcInfo = funcInfo.upper;
                    if (!isDerivedClass) {
                        return;
                    }
                    codePath.traverseSegments((segment, controller) => {
                        const info = segInfoMap[segment.id];
                        for (let i = 0; i < info.invalidNodes.length; ++i) {
                            const invalidNode = info.invalidNodes[i];
                            context.report({
                                message: "'{{kind}}' is not allowed before 'super()'.",
                                node: invalidNode,
                                data: {
                                    kind: invalidNode.type === "Super" ? "super" : "this"
                                }
                            });
                        }
                        if (info.superCalled) {
                            controller.skip();
                        }
                    });
                },
                /**
                 * Initialize information of a given code path segment.
                 * @param {CodePathSegment} segment A code path segment to initialize.
                 * @returns {void}
                 */
                onCodePathSegmentStart(segment) {
                    if (!isInConstructorOfDerivedClass()) {
                        return;
                    }
                    // Initialize info.
                    segInfoMap[segment.id] = {
                        superCalled: (segment.prevSegments.length > 0 &&
                            segment.prevSegments.every(isCalled)),
                        invalidNodes: []
                    };
                },
                /**
                 * Update information of the code path segment when a code path was
                 * looped.
                 * @param {CodePathSegment} fromSegment The code path segment of the
                 *      end of a loop.
                 * @param {CodePathSegment} toSegment A code path segment of the head
                 *      of a loop.
                 * @returns {void}
                 */
                onCodePathSegmentLoop(fromSegment, toSegment) {
                    if (!isInConstructorOfDerivedClass()) {
                        return;
                    }
                    // Update information inside of the loop.
                    funcInfo.codePath.traverseSegments({ first: toSegment, last: fromSegment }, (segment, controller) => {
                        const info = segInfoMap[segment.id];
                        if (info.superCalled) {
                            info.invalidNodes = [];
                            controller.skip();
                        }
                        else if (segment.prevSegments.length > 0 &&
                            segment.prevSegments.every(isCalled)) {
                            info.superCalled = true;
                            info.invalidNodes = [];
                        }
                    });
                },
                /**
                 * Reports if this is before `super()`.
                 * @param {ASTNode} node A target node.
                 * @returns {void}
                 */
                ThisExpression(node) {
                    if (isBeforeCallOfSuper()) {
                        setInvalid(node);
                    }
                },
                /**
                 * Reports if this is before `super()`.
                 * @param {ASTNode} node A target node.
                 * @returns {void}
                 */
                Super(node) {
                    if (!astUtils.isCallee(node) && isBeforeCallOfSuper()) {
                        setInvalid(node);
                    }
                },
                /**
                 * Marks `super()` called.
                 * @param {ASTNode} node A target node.
                 * @returns {void}
                 */
                "CallExpression:exit"(node) {
                    if (node.callee.type === "Super" && isBeforeCallOfSuper()) {
                        setSuperCalled();
                    }
                },
                /**
                 * Resets state.
                 * @returns {void}
                 */
                "Program:exit"() {
                    segInfoMap = Object.create(null);
                }
            };
        }
    };
},
/* eslint/lib/rules/no-throw-literal.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow throwing literals as exceptions",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-throw-literal"
            },
            schema: [],
            messages: {
                object: "Expected an error object to be thrown.",
                undef: "Do not throw undefined."
            }
        },
        create(context) {
            return {
                ThrowStatement(node) {
                    if (!astUtils.couldBeError(node.argument)) {
                        context.report({ node, messageId: "object" });
                    }
                    else if (node.argument.type === "Identifier") {
                        if (node.argument.name === "undefined") {
                            context.report({ node, messageId: "undef" });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-trailing-spaces.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "disallow trailing whitespace at the end of lines",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-trailing-spaces"
            },
            fixable: "whitespace",
            schema: [
                {
                    type: "object",
                    properties: {
                        skipBlankLines: {
                            type: "boolean",
                            default: false
                        },
                        ignoreComments: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const BLANK_CLASS = "[ \t\u00a0\u2000-\u200b\u3000]", SKIP_BLANK = `^${BLANK_CLASS}*$`, NONBLANK = `${BLANK_CLASS}+$`;
            const options = context.options[0] || {}, skipBlankLines = options.skipBlankLines || false, ignoreComments = options.ignoreComments || false;
            /**
             * Report the error message
             * @param {ASTNode} node node to report
             * @param {int[]} location range information
             * @param {int[]} fixRange Range based on the whole program
             * @returns {void}
             */
            function report(node, location, fixRange) {
                /*
                 * Passing node is a bit dirty, because message data will contain big
                 * text in `source`. But... who cares :) ?
                 * One more kludge will not make worse the bloody wizardry of this
                 * plugin.
                 */
                context.report({
                    node,
                    loc: location,
                    message: "Trailing spaces not allowed.",
                    fix(fixer) {
                        return fixer.removeRange(fixRange);
                    }
                });
            }
            /**
             * Given a list of comment nodes, return the line numbers for those comments.
             * @param {Array} comments An array of comment nodes.
             * @returns {number[]} An array of line numbers containing comments.
             */
            function getCommentLineNumbers(comments) {
                const lines = new Set();
                comments.forEach(comment => {
                    const endLine = comment.type === "Block"
                        ? comment.loc.end.line - 1
                        : comment.loc.end.line;
                    for (let i = comment.loc.start.line; i <= endLine; i++) {
                        lines.add(i);
                    }
                });
                return lines;
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                Program: function checkTrailingSpaces(node) {
                    /*
                     * Let's hack. Since Espree does not return whitespace nodes,
                     * fetch the source code and do matching via regexps.
                     */
                    const re = new RegExp(NONBLANK, "u"), skipMatch = new RegExp(SKIP_BLANK, "u"), lines = sourceCode.lines, linebreaks = sourceCode.getText().match(astUtils.createGlobalLinebreakMatcher()), comments = sourceCode.getAllComments(), commentLineNumbers = getCommentLineNumbers(comments);
                    let totalLength = 0, fixRange = [];
                    for (let i = 0, ii = lines.length; i < ii; i++) {
                        const lineNumber = i + 1;
                        /*
                         * Always add linebreak length to line length to accommodate for line break (\n or \r\n)
                         * Because during the fix time they also reserve one spot in the array.
                         * Usually linebreak length is 2 for \r\n (CRLF) and 1 for \n (LF)
                         */
                        const linebreakLength = linebreaks && linebreaks[i] ? linebreaks[i].length : 1;
                        const lineLength = lines[i].length + linebreakLength;
                        const matches = re.exec(lines[i]);
                        if (matches) {
                            const location = {
                                start: {
                                    line: lineNumber,
                                    column: matches.index
                                },
                                end: {
                                    line: lineNumber,
                                    column: lineLength - linebreakLength
                                }
                            };
                            const rangeStart = totalLength + location.start.column;
                            const rangeEnd = totalLength + location.end.column;
                            const containingNode = sourceCode.getNodeByRangeIndex(rangeStart);
                            if (containingNode && containingNode.type === "TemplateElement" &&
                                rangeStart > containingNode.parent.range[0] &&
                                rangeEnd < containingNode.parent.range[1]) {
                                totalLength += lineLength;
                                continue;
                            }
                            /*
                             * If the line has only whitespace, and skipBlankLines
                             * is true, don't report it
                             */
                            if (skipBlankLines && skipMatch.test(lines[i])) {
                                totalLength += lineLength;
                                continue;
                            }
                            fixRange = [rangeStart, rangeEnd];
                            if (!ignoreComments || !commentLineNumbers.has(lineNumber)) {
                                report(node, location, fixRange);
                            }
                        }
                        totalLength += lineLength;
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-undef.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks if the given node is the argument of a typeof operator.
     * @param {ASTNode} node The AST node being checked.
     * @returns {boolean} Whether or not the node is the argument of a typeof operator.
     */
    function hasTypeOfOperator(node) {
        const parent = node.parent;
        return parent.type === "UnaryExpression" && parent.operator === "typeof";
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow the use of undeclared variables unless mentioned in `/*global */` comments",
                category: "Variables",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-undef"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        typeof: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                undef: "'{{name}}' is not defined."
            }
        },
        create(context) {
            const options = context.options[0];
            const considerTypeOf = options && options.typeof === true || false;
            return {
                "Program:exit"( /* node */) {
                    const globalScope = context.getScope();
                    globalScope.through.forEach(ref => {
                        const identifier = ref.identifier;
                        if (!considerTypeOf && hasTypeOfOperator(identifier)) {
                            return;
                        }
                        context.report({
                            node: identifier,
                            messageId: "undef",
                            data: identifier
                        });
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/no-undef-init.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow initializing variables to `undefined`",
                category: "Variables",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-undef-init"
            },
            schema: [],
            fixable: "code"
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                VariableDeclarator(node) {
                    const name = sourceCode.getText(node.id), init = node.init && node.init.name, scope = context.getScope(), undefinedVar = astUtils.getVariableByName(scope, "undefined"), shadowed = undefinedVar && undefinedVar.defs.length > 0, lastToken = sourceCode.getLastToken(node);
                    if (init === "undefined" && node.parent.kind !== "const" && !shadowed) {
                        context.report({
                            node,
                            message: "It's not necessary to initialize '{{name}}' to undefined.",
                            data: { name },
                            fix(fixer) {
                                if (node.parent.kind === "var") {
                                    return null;
                                }
                                if (node.id.type === "ArrayPattern" || node.id.type === "ObjectPattern") {
                                    // Don't fix destructuring assignment to `undefined`.
                                    return null;
                                }
                                if (sourceCode.commentsExistBetween(node.id, lastToken)) {
                                    return null;
                                }
                                return fixer.removeRange([node.id.range[1], node.range[1]]);
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-undefined.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the use of `undefined` as an identifier",
                category: "Variables",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-undefined"
            },
            schema: []
        },
        create(context) {
            /**
             * Report an invalid "undefined" identifier node.
             * @param {ASTNode} node The node to report.
             * @returns {void}
             */
            function report(node) {
                context.report({
                    node,
                    message: "Unexpected use of undefined."
                });
            }
            /**
             * Checks the given scope for references to `undefined` and reports
             * all references found.
             * @param {eslint-scope.Scope} scope The scope to check.
             * @returns {void}
             */
            function checkScope(scope) {
                const undefinedVar = scope.set.get("undefined");
                if (!undefinedVar) {
                    return;
                }
                const references = undefinedVar.references;
                const defs = undefinedVar.defs;
                // Report non-initializing references (those are covered in defs below)
                references
                    .filter(ref => !ref.init)
                    .forEach(ref => report(ref.identifier));
                defs.forEach(def => report(def.name));
            }
            return {
                "Program:exit"() {
                    const globalScope = context.getScope();
                    const stack = [globalScope];
                    while (stack.length) {
                        const scope = stack.pop();
                        stack.push(...scope.childScopes);
                        checkScope(scope);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-underscore-dangle.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow dangling underscores in identifiers",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-underscore-dangle"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allow: {
                            type: "array",
                            items: {
                                type: "string"
                            }
                        },
                        allowAfterThis: {
                            type: "boolean",
                            default: false
                        },
                        allowAfterSuper: {
                            type: "boolean",
                            default: false
                        },
                        allowAfterThisConstructor: {
                            type: "boolean",
                            default: false
                        },
                        enforceInMethodNames: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const options = context.options[0] || {};
            const ALLOWED_VARIABLES = options.allow ? options.allow : [];
            const allowAfterThis = typeof options.allowAfterThis !== "undefined" ? options.allowAfterThis : false;
            const allowAfterSuper = typeof options.allowAfterSuper !== "undefined" ? options.allowAfterSuper : false;
            const allowAfterThisConstructor = typeof options.allowAfterThisConstructor !== "undefined" ? options.allowAfterThisConstructor : false;
            const enforceInMethodNames = typeof options.enforceInMethodNames !== "undefined" ? options.enforceInMethodNames : false;
            //-------------------------------------------------------------------------
            // Helpers
            //-------------------------------------------------------------------------
            /**
             * Check if identifier is present inside the allowed option
             * @param {string} identifier name of the node
             * @returns {boolean} true if its is present
             * @private
             */
            function isAllowed(identifier) {
                return ALLOWED_VARIABLES.some(ident => ident === identifier);
            }
            /**
             * Check if identifier has a underscore at the end
             * @param {string} identifier name of the node
             * @returns {boolean} true if its is present
             * @private
             */
            function hasTrailingUnderscore(identifier) {
                const len = identifier.length;
                return identifier !== "_" && (identifier[0] === "_" || identifier[len - 1] === "_");
            }
            /**
             * Check if identifier is a special case member expression
             * @param {string} identifier name of the node
             * @returns {boolean} true if its is a special case
             * @private
             */
            function isSpecialCaseIdentifierForMemberExpression(identifier) {
                return identifier === "__proto__";
            }
            /**
             * Check if identifier is a special case variable expression
             * @param {string} identifier name of the node
             * @returns {boolean} true if its is a special case
             * @private
             */
            function isSpecialCaseIdentifierInVariableExpression(identifier) {
                // Checks for the underscore library usage here
                return identifier === "_";
            }
            /**
             * Check if a node is a member reference of this.constructor
             * @param {ASTNode} node node to evaluate
             * @returns {boolean} true if it is a reference on this.constructor
             * @private
             */
            function isThisConstructorReference(node) {
                return node.object.type === "MemberExpression" &&
                    node.object.property.name === "constructor" &&
                    node.object.object.type === "ThisExpression";
            }
            /**
             * Check if function has a underscore at the end
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkForTrailingUnderscoreInFunctionDeclaration(node) {
                if (node.id) {
                    const identifier = node.id.name;
                    if (typeof identifier !== "undefined" && hasTrailingUnderscore(identifier) && !isAllowed(identifier)) {
                        context.report({
                            node,
                            message: "Unexpected dangling '_' in '{{identifier}}'.",
                            data: {
                                identifier
                            }
                        });
                    }
                }
            }
            /**
             * Check if variable expression has a underscore at the end
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkForTrailingUnderscoreInVariableExpression(node) {
                const identifier = node.id.name;
                if (typeof identifier !== "undefined" && hasTrailingUnderscore(identifier) &&
                    !isSpecialCaseIdentifierInVariableExpression(identifier) && !isAllowed(identifier)) {
                    context.report({
                        node,
                        message: "Unexpected dangling '_' in '{{identifier}}'.",
                        data: {
                            identifier
                        }
                    });
                }
            }
            /**
             * Check if member expression has a underscore at the end
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkForTrailingUnderscoreInMemberExpression(node) {
                const identifier = node.property.name, isMemberOfThis = node.object.type === "ThisExpression", isMemberOfSuper = node.object.type === "Super", isMemberOfThisConstructor = isThisConstructorReference(node);
                if (typeof identifier !== "undefined" && hasTrailingUnderscore(identifier) &&
                    !(isMemberOfThis && allowAfterThis) &&
                    !(isMemberOfSuper && allowAfterSuper) &&
                    !(isMemberOfThisConstructor && allowAfterThisConstructor) &&
                    !isSpecialCaseIdentifierForMemberExpression(identifier) && !isAllowed(identifier)) {
                    context.report({
                        node,
                        message: "Unexpected dangling '_' in '{{identifier}}'.",
                        data: {
                            identifier
                        }
                    });
                }
            }
            /**
             * Check if method declaration or method property has a underscore at the end
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkForTrailingUnderscoreInMethod(node) {
                const identifier = node.key.name;
                const isMethod = node.type === "MethodDefinition" || node.type === "Property" && node.method;
                if (typeof identifier !== "undefined" && enforceInMethodNames && isMethod && hasTrailingUnderscore(identifier)) {
                    context.report({
                        node,
                        message: "Unexpected dangling '_' in '{{identifier}}'.",
                        data: {
                            identifier
                        }
                    });
                }
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                FunctionDeclaration: checkForTrailingUnderscoreInFunctionDeclaration,
                VariableDeclarator: checkForTrailingUnderscoreInVariableExpression,
                MemberExpression: checkForTrailingUnderscoreInMemberExpression,
                MethodDefinition: checkForTrailingUnderscoreInMethod,
                Property: checkForTrailingUnderscoreInMethod
            };
        }
    };
},
/* eslint/lib/rules/no-unexpected-multiline.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow confusing multiline expressions",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-unexpected-multiline"
            },
            schema: [],
            messages: {
                function: "Unexpected newline between function and ( of function call.",
                property: "Unexpected newline between object and [ of property access.",
                taggedTemplate: "Unexpected newline between template tag and template literal.",
                division: "Unexpected newline between numerator and division operator."
            }
        },
        create(context) {
            const REGEX_FLAG_MATCHER = /^[gimsuy]+$/u;
            const sourceCode = context.getSourceCode();
            /**
             * Check to see if there is a newline between the node and the following open bracket
             * line's expression
             * @param {ASTNode} node The node to check.
             * @param {string} messageId The error messageId to use.
             * @returns {void}
             * @private
             */
            function checkForBreakAfter(node, messageId) {
                const openParen = sourceCode.getTokenAfter(node, astUtils.isNotClosingParenToken);
                const nodeExpressionEnd = sourceCode.getTokenBefore(openParen);
                if (openParen.loc.start.line !== nodeExpressionEnd.loc.end.line) {
                    context.report({ node, loc: openParen.loc.start, messageId, data: { char: openParen.value } });
                }
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                MemberExpression(node) {
                    if (!node.computed) {
                        return;
                    }
                    checkForBreakAfter(node.object, "property");
                },
                TaggedTemplateExpression(node) {
                    if (node.tag.loc.end.line === node.quasi.loc.start.line) {
                        return;
                    }
                    // handle generics type parameters on template tags
                    const tokenBefore = sourceCode.getTokenBefore(node.quasi);
                    if (tokenBefore.loc.end.line === node.quasi.loc.start.line) {
                        return;
                    }
                    context.report({ node, loc: node.loc.start, messageId: "taggedTemplate" });
                },
                CallExpression(node) {
                    if (node.arguments.length === 0) {
                        return;
                    }
                    checkForBreakAfter(node.callee, "function");
                },
                "BinaryExpression[operator='/'] > BinaryExpression[operator='/'].left"(node) {
                    const secondSlash = sourceCode.getTokenAfter(node, token => token.value === "/");
                    const tokenAfterOperator = sourceCode.getTokenAfter(secondSlash);
                    if (tokenAfterOperator.type === "Identifier" &&
                        REGEX_FLAG_MATCHER.test(tokenAfterOperator.value) &&
                        secondSlash.range[1] === tokenAfterOperator.range[0]) {
                        checkForBreakAfter(node.left, "division");
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-unmodified-loop-condition.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const Traverser = require(337) /* ../shared/traverser */, astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const SENTINEL_PATTERN = /(?:(?:Call|Class|Function|Member|New|Yield)Expression|Statement|Declaration)$/u;
    const LOOP_PATTERN = /^(?:DoWhile|For|While)Statement$/u; // for-in/of statements don't have `test` property.
    const GROUP_PATTERN = /^(?:BinaryExpression|ConditionalExpression)$/u;
    const SKIP_PATTERN = /^(?:ArrowFunction|Class|Function)Expression$/u;
    const DYNAMIC_PATTERN = /^(?:Call|Member|New|TaggedTemplate|Yield)Expression$/u;
    /**
     * @typedef {Object} LoopConditionInfo
     * @property {eslint-scope.Reference} reference - The reference.
     * @property {ASTNode} group - BinaryExpression or ConditionalExpression nodes
     *      that the reference is belonging to.
     * @property {Function} isInLoop - The predicate which checks a given reference
     *      is in this loop.
     * @property {boolean} modified - The flag that the reference is modified in
     *      this loop.
     */
    /**
     * Checks whether or not a given reference is a write reference.
     * @param {eslint-scope.Reference} reference A reference to check.
     * @returns {boolean} `true` if the reference is a write reference.
     */
    function isWriteReference(reference) {
        if (reference.init) {
            const def = reference.resolved && reference.resolved.defs[0];
            if (!def || def.type !== "Variable" || def.parent.kind !== "var") {
                return false;
            }
        }
        return reference.isWrite();
    }
    /**
     * Checks whether or not a given loop condition info does not have the modified
     * flag.
     * @param {LoopConditionInfo} condition A loop condition info to check.
     * @returns {boolean} `true` if the loop condition info is "unmodified".
     */
    function isUnmodified(condition) {
        return !condition.modified;
    }
    /**
     * Checks whether or not a given loop condition info does not have the modified
     * flag and does not have the group this condition belongs to.
     * @param {LoopConditionInfo} condition A loop condition info to check.
     * @returns {boolean} `true` if the loop condition info is "unmodified".
     */
    function isUnmodifiedAndNotBelongToGroup(condition) {
        return !(condition.modified || condition.group);
    }
    /**
     * Checks whether or not a given reference is inside of a given node.
     * @param {ASTNode} node A node to check.
     * @param {eslint-scope.Reference} reference A reference to check.
     * @returns {boolean} `true` if the reference is inside of the node.
     */
    function isInRange(node, reference) {
        const or = node.range;
        const ir = reference.identifier.range;
        return or[0] <= ir[0] && ir[1] <= or[1];
    }
    /**
     * Checks whether or not a given reference is inside of a loop node's condition.
     * @param {ASTNode} node A node to check.
     * @param {eslint-scope.Reference} reference A reference to check.
     * @returns {boolean} `true` if the reference is inside of the loop node's
     *      condition.
     */
    const isInLoop = {
        WhileStatement: isInRange,
        DoWhileStatement: isInRange,
        ForStatement(node, reference) {
            return (isInRange(node, reference) &&
                !(node.init && isInRange(node.init, reference)));
        }
    };
    /**
     * Gets the function which encloses a given reference.
     * This supports only FunctionDeclaration.
     * @param {eslint-scope.Reference} reference A reference to get.
     * @returns {ASTNode|null} The function node or null.
     */
    function getEncloseFunctionDeclaration(reference) {
        let node = reference.identifier;
        while (node) {
            if (node.type === "FunctionDeclaration") {
                return node.id ? node : null;
            }
            node = node.parent;
        }
        return null;
    }
    /**
     * Updates the "modified" flags of given loop conditions with given modifiers.
     * @param {LoopConditionInfo[]} conditions The loop conditions to be updated.
     * @param {eslint-scope.Reference[]} modifiers The references to update.
     * @returns {void}
     */
    function updateModifiedFlag(conditions, modifiers) {
        for (let i = 0; i < conditions.length; ++i) {
            const condition = conditions[i];
            for (let j = 0; !condition.modified && j < modifiers.length; ++j) {
                const modifier = modifiers[j];
                let funcNode, funcVar;
                /*
                 * Besides checking for the condition being in the loop, we want to
                 * check the function that this modifier is belonging to is called
                 * in the loop.
                 * FIXME: This should probably be extracted to a function.
                 */
                const inLoop = condition.isInLoop(modifier) || Boolean((funcNode = getEncloseFunctionDeclaration(modifier)) &&
                    (funcVar = astUtils.getVariableByName(modifier.from.upper, funcNode.id.name)) &&
                    funcVar.references.some(condition.isInLoop));
                condition.modified = inLoop;
            }
        }
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow unmodified loop conditions",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-unmodified-loop-condition"
            },
            schema: []
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            let groupMap = null;
            /**
             * Reports a given condition info.
             * @param {LoopConditionInfo} condition A loop condition info to report.
             * @returns {void}
             */
            function report(condition) {
                const node = condition.reference.identifier;
                context.report({
                    node,
                    message: "'{{name}}' is not modified in this loop.",
                    data: node
                });
            }
            /**
             * Registers given conditions to the group the condition belongs to.
             * @param {LoopConditionInfo[]} conditions A loop condition info to
             *      register.
             * @returns {void}
             */
            function registerConditionsToGroup(conditions) {
                for (let i = 0; i < conditions.length; ++i) {
                    const condition = conditions[i];
                    if (condition.group) {
                        let group = groupMap.get(condition.group);
                        if (!group) {
                            group = [];
                            groupMap.set(condition.group, group);
                        }
                        group.push(condition);
                    }
                }
            }
            /**
             * Reports references which are inside of unmodified groups.
             * @param {LoopConditionInfo[]} conditions A loop condition info to report.
             * @returns {void}
             */
            function checkConditionsInGroup(conditions) {
                if (conditions.every(isUnmodified)) {
                    conditions.forEach(report);
                }
            }
            /**
             * Checks whether or not a given group node has any dynamic elements.
             * @param {ASTNode} root A node to check.
             *      This node is one of BinaryExpression or ConditionalExpression.
             * @returns {boolean} `true` if the node is dynamic.
             */
            function hasDynamicExpressions(root) {
                let retv = false;
                Traverser.traverse(root, {
                    visitorKeys: sourceCode.visitorKeys,
                    enter(node) {
                        if (DYNAMIC_PATTERN.test(node.type)) {
                            retv = true;
                            this.break();
                        }
                        else if (SKIP_PATTERN.test(node.type)) {
                            this.skip();
                        }
                    }
                });
                return retv;
            }
            /**
             * Creates the loop condition information from a given reference.
             * @param {eslint-scope.Reference} reference A reference to create.
             * @returns {LoopConditionInfo|null} Created loop condition info, or null.
             */
            function toLoopCondition(reference) {
                if (reference.init) {
                    return null;
                }
                let group = null;
                let child = reference.identifier;
                let node = child.parent;
                while (node) {
                    if (SENTINEL_PATTERN.test(node.type)) {
                        if (LOOP_PATTERN.test(node.type) && node.test === child) {
                            // This reference is inside of a loop condition.
                            return {
                                reference,
                                group,
                                isInLoop: isInLoop[node.type].bind(null, node),
                                modified: false
                            };
                        }
                        // This reference is outside of a loop condition.
                        break;
                    }
                    /*
                     * If it's inside of a group, OK if either operand is modified.
                     * So stores the group this reference belongs to.
                     */
                    if (GROUP_PATTERN.test(node.type)) {
                        // If this expression is dynamic, no need to check.
                        if (hasDynamicExpressions(node)) {
                            break;
                        }
                        else {
                            group = node;
                        }
                    }
                    child = node;
                    node = node.parent;
                }
                return null;
            }
            /**
             * Finds unmodified references which are inside of a loop condition.
             * Then reports the references which are outside of groups.
             * @param {eslint-scope.Variable} variable A variable to report.
             * @returns {void}
             */
            function checkReferences(variable) {
                // Gets references that exist in loop conditions.
                const conditions = variable
                    .references
                    .map(toLoopCondition)
                    .filter(Boolean);
                if (conditions.length === 0) {
                    return;
                }
                // Registers the conditions to belonging groups.
                registerConditionsToGroup(conditions);
                // Check the conditions are modified.
                const modifiers = variable.references.filter(isWriteReference);
                if (modifiers.length > 0) {
                    updateModifiedFlag(conditions, modifiers);
                }
                /*
                 * Reports the conditions which are not belonging to groups.
                 * Others will be reported after all variables are done.
                 */
                conditions
                    .filter(isUnmodifiedAndNotBelongToGroup)
                    .forEach(report);
            }
            return {
                "Program:exit"() {
                    const queue = [context.getScope()];
                    groupMap = new Map();
                    let scope;
                    while ((scope = queue.pop())) {
                        queue.push(...scope.childScopes);
                        scope.variables.forEach(checkReferences);
                    }
                    groupMap.forEach(checkConditionsInGroup);
                    groupMap = null;
                }
            };
        }
    };
},
/* eslint/lib/shared/traverser.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const vk = require(100) /* eslint-visitor-keys */;
    const debug = require(120) /* debug */("eslint:traverser");
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Do nothing.
     * @returns {void}
     */
    function noop() {
        // do nothing.
    }
    /**
     * Check whether the given value is an ASTNode or not.
     * @param {any} x The value to check.
     * @returns {boolean} `true` if the value is an ASTNode.
     */
    function isNode(x) {
        return x !== null && typeof x === "object" && typeof x.type === "string";
    }
    /**
     * Get the visitor keys of a given node.
     * @param {Object} visitorKeys The map of visitor keys.
     * @param {ASTNode} node The node to get their visitor keys.
     * @returns {string[]} The visitor keys of the node.
     */
    function getVisitorKeys(visitorKeys, node) {
        let keys = visitorKeys[node.type];
        if (!keys) {
            keys = vk.getKeys(node);
            debug("Unknown node type \"%s\": Estimated visitor keys %j", node.type, keys);
        }
        return keys;
    }
    /**
     * The traverser class to traverse AST trees.
     */
    class Traverser {
        constructor() {
            this._current = null;
            this._parents = [];
            this._skipped = false;
            this._broken = false;
            this._visitorKeys = null;
            this._enter = null;
            this._leave = null;
        }
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @returns {ASTNode} The current node.
         */
        current() {
            return this._current;
        }
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @returns {ASTNode[]} The ancestor nodes.
         */
        parents() {
            return this._parents.slice(0);
        }
        /**
         * Break the current traversal.
         * @returns {void}
         */
        break() {
            this._broken = true;
        }
        /**
         * Skip child nodes for the current traversal.
         * @returns {void}
         */
        skip() {
            this._skipped = true;
        }
        /**
         * Traverse the given AST tree.
         * @param {ASTNode} node The root node to traverse.
         * @param {Object} options The option object.
         * @param {Object} [options.visitorKeys=DEFAULT_VISITOR_KEYS] The keys of each node types to traverse child nodes. Default is `./default-visitor-keys.json`.
         * @param {Function} [options.enter=noop] The callback function which is called on entering each node.
         * @param {Function} [options.leave=noop] The callback function which is called on leaving each node.
         * @returns {void}
         */
        traverse(node, options) {
            this._current = null;
            this._parents = [];
            this._skipped = false;
            this._broken = false;
            this._visitorKeys = options.visitorKeys || vk.KEYS;
            this._enter = options.enter || noop;
            this._leave = options.leave || noop;
            this._traverse(node, null);
        }
        /**
         * Traverse the given AST tree recursively.
         * @param {ASTNode} node The current node.
         * @param {ASTNode|null} parent The parent node.
         * @returns {void}
         * @private
         */
        _traverse(node, parent) {
            if (!isNode(node)) {
                return;
            }
            this._current = node;
            this._skipped = false;
            this._enter(node, parent);
            if (!this._skipped && !this._broken) {
                const keys = getVisitorKeys(this._visitorKeys, node);
                if (keys.length >= 1) {
                    this._parents.push(node);
                    for (let i = 0; i < keys.length && !this._broken; ++i) {
                        const child = node[keys[i]];
                        if (Array.isArray(child)) {
                            for (let j = 0; j < child.length && !this._broken; ++j) {
                                this._traverse(child[j], node);
                            }
                        }
                        else {
                            this._traverse(child, node);
                        }
                    }
                    this._parents.pop();
                }
            }
            if (!this._broken) {
                this._leave(node, parent);
            }
            this._current = parent;
        }
        /**
         * Calculates the keys to use for traversal.
         * @param {ASTNode} node The node to read keys from.
         * @returns {string[]} An array of keys to visit on the node.
         * @private
         */
        static getKeys(node) {
            return vk.getKeys(node);
        }
        /**
         * Traverse the given AST tree.
         * @param {ASTNode} node The root node to traverse.
         * @param {Object} options The option object.
         * @param {Object} [options.visitorKeys=DEFAULT_VISITOR_KEYS] The keys of each node types to traverse child nodes. Default is `./default-visitor-keys.json`.
         * @param {Function} [options.enter=noop] The callback function which is called on entering each node.
         * @param {Function} [options.leave=noop] The callback function which is called on leaving each node.
         * @returns {void}
         */
        static traverse(node, options) {
            new Traverser().traverse(node, options);
        }
        /**
         * The default visitor keys.
         * @type {Object}
         */
        static get DEFAULT_VISITOR_KEYS() {
            return vk.KEYS;
        }
    }
    module.exports = Traverser;
},
/* eslint/lib/rules/no-unneeded-ternary.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    // Operators that always result in a boolean value
    const BOOLEAN_OPERATORS = new Set(["==", "===", "!=", "!==", ">", ">=", "<", "<=", "in", "instanceof"]);
    const OPERATOR_INVERSES = {
        "==": "!=",
        "!=": "==",
        "===": "!==",
        "!==": "==="
        // Operators like < and >= are not true inverses, since both will return false with NaN.
    };
    const OR_PRECEDENCE = astUtils.getPrecedence({ type: "LogicalExpression", operator: "||" });
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow ternary operators when simpler alternatives exist",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-unneeded-ternary"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        defaultAssignment: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "code"
        },
        create(context) {
            const options = context.options[0] || {};
            const defaultAssignment = options.defaultAssignment !== false;
            const sourceCode = context.getSourceCode();
            /**
             * Test if the node is a boolean literal
             * @param {ASTNode} node The node to report.
             * @returns {boolean} True if the its a boolean literal
             * @private
             */
            function isBooleanLiteral(node) {
                return node.type === "Literal" && typeof node.value === "boolean";
            }
            /**
             * Creates an expression that represents the boolean inverse of the expression represented by the original node
             * @param {ASTNode} node A node representing an expression
             * @returns {string} A string representing an inverted expression
             */
            function invertExpression(node) {
                if (node.type === "BinaryExpression" && Object.prototype.hasOwnProperty.call(OPERATOR_INVERSES, node.operator)) {
                    const operatorToken = sourceCode.getFirstTokenBetween(node.left, node.right, token => token.value === node.operator);
                    const text = sourceCode.getText();
                    return text.slice(node.range[0], operatorToken.range[0]) + OPERATOR_INVERSES[node.operator] + text.slice(operatorToken.range[1], node.range[1]);
                }
                if (astUtils.getPrecedence(node) < astUtils.getPrecedence({ type: "UnaryExpression" })) {
                    return `!(${astUtils.getParenthesisedText(sourceCode, node)})`;
                }
                return `!${astUtils.getParenthesisedText(sourceCode, node)}`;
            }
            /**
             * Tests if a given node always evaluates to a boolean value
             * @param {ASTNode} node An expression node
             * @returns {boolean} True if it is determined that the node will always evaluate to a boolean value
             */
            function isBooleanExpression(node) {
                return node.type === "BinaryExpression" && BOOLEAN_OPERATORS.has(node.operator) ||
                    node.type === "UnaryExpression" && node.operator === "!";
            }
            /**
             * Test if the node matches the pattern id ? id : expression
             * @param {ASTNode} node The ConditionalExpression to check.
             * @returns {boolean} True if the pattern is matched, and false otherwise
             * @private
             */
            function matchesDefaultAssignment(node) {
                return node.test.type === "Identifier" &&
                    node.consequent.type === "Identifier" &&
                    node.test.name === node.consequent.name;
            }
            return {
                ConditionalExpression(node) {
                    if (isBooleanLiteral(node.alternate) && isBooleanLiteral(node.consequent)) {
                        context.report({
                            node,
                            loc: node.consequent.loc.start,
                            message: "Unnecessary use of boolean literals in conditional expression.",
                            fix(fixer) {
                                if (node.consequent.value === node.alternate.value) {
                                    // Replace `foo ? true : true` with just `true`, but don't replace `foo() ? true : true`
                                    return node.test.type === "Identifier" ? fixer.replaceText(node, node.consequent.value.toString()) : null;
                                }
                                if (node.alternate.value) {
                                    // Replace `foo() ? false : true` with `!(foo())`
                                    return fixer.replaceText(node, invertExpression(node.test));
                                }
                                // Replace `foo ? true : false` with `foo` if `foo` is guaranteed to be a boolean, or `!!foo` otherwise.
                                return fixer.replaceText(node, isBooleanExpression(node.test) ? astUtils.getParenthesisedText(sourceCode, node.test) : `!${invertExpression(node.test)}`);
                            }
                        });
                    }
                    else if (!defaultAssignment && matchesDefaultAssignment(node)) {
                        context.report({
                            node,
                            loc: node.consequent.loc.start,
                            message: "Unnecessary use of conditional expression for default assignment.",
                            fix: fixer => {
                                const shouldParenthesizeAlternate = (astUtils.getPrecedence(node.alternate) < OR_PRECEDENCE &&
                                    !astUtils.isParenthesised(sourceCode, node.alternate));
                                const alternateText = shouldParenthesizeAlternate
                                    ? `(${sourceCode.getText(node.alternate)})`
                                    : astUtils.getParenthesisedText(sourceCode, node.alternate);
                                const testText = astUtils.getParenthesisedText(sourceCode, node.test);
                                return fixer.replaceText(node, `${testText} || ${alternateText}`);
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-unreachable.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a given variable declarator has the initializer.
     * @param {ASTNode} node A VariableDeclarator node to check.
     * @returns {boolean} `true` if the node has the initializer.
     */
    function isInitialized(node) {
        return Boolean(node.init);
    }
    /**
     * Checks whether or not a given code path segment is unreachable.
     * @param {CodePathSegment} segment A CodePathSegment to check.
     * @returns {boolean} `true` if the segment is unreachable.
     */
    function isUnreachable(segment) {
        return !segment.reachable;
    }
    /**
     * The class to distinguish consecutive unreachable statements.
     */
    class ConsecutiveRange {
        constructor(sourceCode) {
            this.sourceCode = sourceCode;
            this.startNode = null;
            this.endNode = null;
        }
        /**
         * The location object of this range.
         * @type {Object}
         */
        get location() {
            return {
                start: this.startNode.loc.start,
                end: this.endNode.loc.end
            };
        }
        /**
         * `true` if this range is empty.
         * @type {boolean}
         */
        get isEmpty() {
            return !(this.startNode && this.endNode);
        }
        /**
         * Checks whether the given node is inside of this range.
         * @param {ASTNode|Token} node The node to check.
         * @returns {boolean} `true` if the node is inside of this range.
         */
        contains(node) {
            return (node.range[0] >= this.startNode.range[0] &&
                node.range[1] <= this.endNode.range[1]);
        }
        /**
         * Checks whether the given node is consecutive to this range.
         * @param {ASTNode} node The node to check.
         * @returns {boolean} `true` if the node is consecutive to this range.
         */
        isConsecutive(node) {
            return this.contains(this.sourceCode.getTokenBefore(node));
        }
        /**
         * Merges the given node to this range.
         * @param {ASTNode} node The node to merge.
         * @returns {void}
         */
        merge(node) {
            this.endNode = node;
        }
        /**
         * Resets this range by the given node or null.
         * @param {ASTNode|null} node The node to reset, or null.
         * @returns {void}
         */
        reset(node) {
            this.startNode = this.endNode = node;
        }
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow unreachable code after `return`, `throw`, `continue`, and `break` statements",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-unreachable"
            },
            schema: []
        },
        create(context) {
            let currentCodePath = null;
            const range = new ConsecutiveRange(context.getSourceCode());
            /**
             * Reports a given node if it's unreachable.
             * @param {ASTNode} node A statement node to report.
             * @returns {void}
             */
            function reportIfUnreachable(node) {
                let nextNode = null;
                if (node && currentCodePath.currentSegments.every(isUnreachable)) {
                    // Store this statement to distinguish consecutive statements.
                    if (range.isEmpty) {
                        range.reset(node);
                        return;
                    }
                    // Skip if this statement is inside of the current range.
                    if (range.contains(node)) {
                        return;
                    }
                    // Merge if this statement is consecutive to the current range.
                    if (range.isConsecutive(node)) {
                        range.merge(node);
                        return;
                    }
                    nextNode = node;
                }
                /*
                 * Report the current range since this statement is reachable or is
                 * not consecutive to the current range.
                 */
                if (!range.isEmpty) {
                    context.report({
                        message: "Unreachable code.",
                        loc: range.location,
                        node: range.startNode
                    });
                }
                // Update the current range.
                range.reset(nextNode);
            }
            return {
                // Manages the current code path.
                onCodePathStart(codePath) {
                    currentCodePath = codePath;
                },
                onCodePathEnd() {
                    currentCodePath = currentCodePath.upper;
                },
                // Registers for all statement nodes (excludes FunctionDeclaration).
                BlockStatement: reportIfUnreachable,
                BreakStatement: reportIfUnreachable,
                ClassDeclaration: reportIfUnreachable,
                ContinueStatement: reportIfUnreachable,
                DebuggerStatement: reportIfUnreachable,
                DoWhileStatement: reportIfUnreachable,
                ExpressionStatement: reportIfUnreachable,
                ForInStatement: reportIfUnreachable,
                ForOfStatement: reportIfUnreachable,
                ForStatement: reportIfUnreachable,
                IfStatement: reportIfUnreachable,
                ImportDeclaration: reportIfUnreachable,
                LabeledStatement: reportIfUnreachable,
                ReturnStatement: reportIfUnreachable,
                SwitchStatement: reportIfUnreachable,
                ThrowStatement: reportIfUnreachable,
                TryStatement: reportIfUnreachable,
                VariableDeclaration(node) {
                    if (node.kind !== "var" || node.declarations.some(isInitialized)) {
                        reportIfUnreachable(node);
                    }
                },
                WhileStatement: reportIfUnreachable,
                WithStatement: reportIfUnreachable,
                ExportNamedDeclaration: reportIfUnreachable,
                ExportDefaultDeclaration: reportIfUnreachable,
                ExportAllDeclaration: reportIfUnreachable,
                "Program:exit"() {
                    reportIfUnreachable();
                }
            };
        }
    };
},
/* eslint/lib/rules/no-unsafe-finally.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const SENTINEL_NODE_TYPE_RETURN_THROW = /^(?:Program|(?:Function|Class)(?:Declaration|Expression)|ArrowFunctionExpression)$/u;
    const SENTINEL_NODE_TYPE_BREAK = /^(?:Program|(?:Function|Class)(?:Declaration|Expression)|ArrowFunctionExpression|DoWhileStatement|WhileStatement|ForOfStatement|ForInStatement|ForStatement|SwitchStatement)$/u;
    const SENTINEL_NODE_TYPE_CONTINUE = /^(?:Program|(?:Function|Class)(?:Declaration|Expression)|ArrowFunctionExpression|DoWhileStatement|WhileStatement|ForOfStatement|ForInStatement|ForStatement)$/u;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow control flow statements in `finally` blocks",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-unsafe-finally"
            },
            schema: []
        },
        create(context) {
            /**
             * Checks if the node is the finalizer of a TryStatement
             * @param {ASTNode} node node to check.
             * @returns {boolean} - true if the node is the finalizer of a TryStatement
             */
            function isFinallyBlock(node) {
                return node.parent.type === "TryStatement" && node.parent.finalizer === node;
            }
            /**
             * Climbs up the tree if the node is not a sentinel node
             * @param {ASTNode} node node to check.
             * @param {string} label label of the break or continue statement
             * @returns {boolean} - return whether the node is a finally block or a sentinel node
             */
            function isInFinallyBlock(node, label) {
                let labelInside = false;
                let sentinelNodeType;
                if (node.type === "BreakStatement" && !node.label) {
                    sentinelNodeType = SENTINEL_NODE_TYPE_BREAK;
                }
                else if (node.type === "ContinueStatement") {
                    sentinelNodeType = SENTINEL_NODE_TYPE_CONTINUE;
                }
                else {
                    sentinelNodeType = SENTINEL_NODE_TYPE_RETURN_THROW;
                }
                for (let currentNode = node; currentNode && !sentinelNodeType.test(currentNode.type); currentNode = currentNode.parent) {
                    if (currentNode.parent.label && label && (currentNode.parent.label.name === label.name)) {
                        labelInside = true;
                    }
                    if (isFinallyBlock(currentNode)) {
                        if (label && labelInside) {
                            return false;
                        }
                        return true;
                    }
                }
                return false;
            }
            /**
             * Checks whether the possibly-unsafe statement is inside a finally block.
             * @param {ASTNode} node node to check.
             * @returns {void}
             */
            function check(node) {
                if (isInFinallyBlock(node, node.label)) {
                    context.report({
                        message: "Unsafe usage of {{nodeType}}.",
                        data: {
                            nodeType: node.type
                        },
                        node,
                        line: node.loc.line,
                        column: node.loc.column
                    });
                }
            }
            return {
                ReturnStatement: check,
                ThrowStatement: check,
                BreakStatement: check,
                ContinueStatement: check
            };
        }
    };
},
/* eslint/lib/rules/no-unsafe-negation.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether the given operator is `in` or `instanceof`
     * @param {string} op The operator type to check.
     * @returns {boolean} `true` if the operator is `in` or `instanceof`
     */
    function isInOrInstanceOfOperator(op) {
        return op === "in" || op === "instanceof";
    }
    /**
     * Checks whether the given operator is an ordering relational operator or not.
     * @param {string} op The operator type to check.
     * @returns {boolean} `true` if the operator is an ordering relational operator.
     */
    function isOrderingRelationalOperator(op) {
        return op === "<" || op === ">" || op === ">=" || op === "<=";
    }
    /**
     * Checks whether the given node is a logical negation expression or not.
     * @param {ASTNode} node The node to check.
     * @returns {boolean} `true` if the node is a logical negation expression.
     */
    function isNegation(node) {
        return node.type === "UnaryExpression" && node.operator === "!";
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow negating the left operand of relational operators",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-unsafe-negation",
                suggestion: true
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        enforceForOrderingRelations: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: null,
            messages: {
                unexpected: "Unexpected negating the left operand of '{{operator}}' operator.",
                suggestNegatedExpression: "Negate '{{operator}}' expression instead of its left operand. This changes the current behavior.",
                suggestParenthesisedNegation: "Wrap negation in '()' to make the intention explicit. This preserves the current behavior."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const options = context.options[0] || {};
            const enforceForOrderingRelations = options.enforceForOrderingRelations === true;
            return {
                BinaryExpression(node) {
                    const operator = node.operator;
                    const orderingRelationRuleApplies = enforceForOrderingRelations && isOrderingRelationalOperator(operator);
                    if ((isInOrInstanceOfOperator(operator) || orderingRelationRuleApplies) &&
                        isNegation(node.left) &&
                        !astUtils.isParenthesised(sourceCode, node.left)) {
                        context.report({
                            node,
                            loc: node.left.loc,
                            messageId: "unexpected",
                            data: { operator },
                            suggest: [
                                {
                                    messageId: "suggestNegatedExpression",
                                    data: { operator },
                                    fix(fixer) {
                                        const negationToken = sourceCode.getFirstToken(node.left);
                                        const fixRange = [negationToken.range[1], node.range[1]];
                                        const text = sourceCode.text.slice(fixRange[0], fixRange[1]);
                                        return fixer.replaceTextRange(fixRange, `(${text})`);
                                    }
                                },
                                {
                                    messageId: "suggestParenthesisedNegation",
                                    fix(fixer) {
                                        return fixer.replaceText(node.left, `(${sourceCode.getText(node.left)})`);
                                    }
                                }
                            ]
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-unused-expressions.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unused expressions",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-unused-expressions"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allowShortCircuit: {
                            type: "boolean",
                            default: false
                        },
                        allowTernary: {
                            type: "boolean",
                            default: false
                        },
                        allowTaggedTemplates: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const config = context.options[0] || {}, allowShortCircuit = config.allowShortCircuit || false, allowTernary = config.allowTernary || false, allowTaggedTemplates = config.allowTaggedTemplates || false;
            // eslint-disable-next-line jsdoc/require-description
            /**
             * @param {ASTNode} node any node
             * @returns {boolean} whether the given node structurally represents a directive
             */
            function looksLikeDirective(node) {
                return node.type === "ExpressionStatement" &&
                    node.expression.type === "Literal" && typeof node.expression.value === "string";
            }
            // eslint-disable-next-line jsdoc/require-description
            /**
             * @param {Function} predicate ([a] -> Boolean) the function used to make the determination
             * @param {a[]} list the input list
             * @returns {a[]} the leading sequence of members in the given list that pass the given predicate
             */
            function takeWhile(predicate, list) {
                for (let i = 0; i < list.length; ++i) {
                    if (!predicate(list[i])) {
                        return list.slice(0, i);
                    }
                }
                return list.slice();
            }
            // eslint-disable-next-line jsdoc/require-description
            /**
             * @param {ASTNode} node a Program or BlockStatement node
             * @returns {ASTNode[]} the leading sequence of directive nodes in the given node's body
             */
            function directives(node) {
                return takeWhile(looksLikeDirective, node.body);
            }
            // eslint-disable-next-line jsdoc/require-description
            /**
             * @param {ASTNode} node any node
             * @param {ASTNode[]} ancestors the given node's ancestors
             * @returns {boolean} whether the given node is considered a directive in its current position
             */
            function isDirective(node, ancestors) {
                const parent = ancestors[ancestors.length - 1], grandparent = ancestors[ancestors.length - 2];
                return (parent.type === "Program" || parent.type === "BlockStatement" &&
                    (/Function/u.test(grandparent.type))) &&
                    directives(parent).indexOf(node) >= 0;
            }
            /**
             * Determines whether or not a given node is a valid expression. Recurses on short circuit eval and ternary nodes if enabled by flags.
             * @param {ASTNode} node any node
             * @returns {boolean} whether the given node is a valid expression
             */
            function isValidExpression(node) {
                if (allowTernary) {
                    // Recursive check for ternary and logical expressions
                    if (node.type === "ConditionalExpression") {
                        return isValidExpression(node.consequent) && isValidExpression(node.alternate);
                    }
                }
                if (allowShortCircuit) {
                    if (node.type === "LogicalExpression") {
                        return isValidExpression(node.right);
                    }
                }
                if (allowTaggedTemplates && node.type === "TaggedTemplateExpression") {
                    return true;
                }
                return /^(?:Assignment|Call|New|Update|Yield|Await)Expression$/u.test(node.type) ||
                    (node.type === "UnaryExpression" && ["delete", "void"].indexOf(node.operator) >= 0);
            }
            return {
                ExpressionStatement(node) {
                    if (!isValidExpression(node.expression) && !isDirective(node, context.getAncestors())) {
                        context.report({ node, message: "Expected an assignment or function call and instead saw an expression." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-unused-labels.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unused labels",
                category: "Best Practices",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-unused-labels"
            },
            schema: [],
            fixable: "code",
            messages: {
                unused: "'{{name}}:' is defined but never used."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            let scopeInfo = null;
            /**
             * Adds a scope info to the stack.
             * @param {ASTNode} node A node to add. This is a LabeledStatement.
             * @returns {void}
             */
            function enterLabeledScope(node) {
                scopeInfo = {
                    label: node.label.name,
                    used: false,
                    upper: scopeInfo
                };
            }
            /**
             * Removes the top of the stack.
             * At the same time, this reports the label if it's never used.
             * @param {ASTNode} node A node to report. This is a LabeledStatement.
             * @returns {void}
             */
            function exitLabeledScope(node) {
                if (!scopeInfo.used) {
                    context.report({
                        node: node.label,
                        messageId: "unused",
                        data: node.label,
                        fix(fixer) {
                            /*
                             * Only perform a fix if there are no comments between the label and the body. This will be the case
                             * when there is exactly one token/comment (the ":") between the label and the body.
                             */
                            if (sourceCode.getTokenAfter(node.label, { includeComments: true }) ===
                                sourceCode.getTokenBefore(node.body, { includeComments: true })) {
                                return fixer.removeRange([node.range[0], node.body.range[0]]);
                            }
                            return null;
                        }
                    });
                }
                scopeInfo = scopeInfo.upper;
            }
            /**
             * Marks the label of a given node as used.
             * @param {ASTNode} node A node to mark. This is a BreakStatement or
             *      ContinueStatement.
             * @returns {void}
             */
            function markAsUsed(node) {
                if (!node.label) {
                    return;
                }
                const label = node.label.name;
                let info = scopeInfo;
                while (info) {
                    if (info.label === label) {
                        info.used = true;
                        break;
                    }
                    info = info.upper;
                }
            }
            return {
                LabeledStatement: enterLabeledScope,
                "LabeledStatement:exit": exitLabeledScope,
                BreakStatement: markAsUsed,
                ContinueStatement: markAsUsed
            };
        }
    };
},
/* eslint/lib/rules/no-unused-vars.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow unused variables",
                category: "Variables",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-unused-vars"
            },
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["all", "local"]
                        },
                        {
                            type: "object",
                            properties: {
                                vars: {
                                    enum: ["all", "local"]
                                },
                                varsIgnorePattern: {
                                    type: "string"
                                },
                                args: {
                                    enum: ["all", "after-used", "none"]
                                },
                                ignoreRestSiblings: {
                                    type: "boolean"
                                },
                                argsIgnorePattern: {
                                    type: "string"
                                },
                                caughtErrors: {
                                    enum: ["all", "none"]
                                },
                                caughtErrorsIgnorePattern: {
                                    type: "string"
                                }
                            }
                        }
                    ]
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const REST_PROPERTY_TYPE = /^(?:RestElement|(?:Experimental)?RestProperty)$/u;
            const config = {
                vars: "all",
                args: "after-used",
                ignoreRestSiblings: false,
                caughtErrors: "none"
            };
            const firstOption = context.options[0];
            if (firstOption) {
                if (typeof firstOption === "string") {
                    config.vars = firstOption;
                }
                else {
                    config.vars = firstOption.vars || config.vars;
                    config.args = firstOption.args || config.args;
                    config.ignoreRestSiblings = firstOption.ignoreRestSiblings || config.ignoreRestSiblings;
                    config.caughtErrors = firstOption.caughtErrors || config.caughtErrors;
                    if (firstOption.varsIgnorePattern) {
                        config.varsIgnorePattern = new RegExp(firstOption.varsIgnorePattern, "u");
                    }
                    if (firstOption.argsIgnorePattern) {
                        config.argsIgnorePattern = new RegExp(firstOption.argsIgnorePattern, "u");
                    }
                    if (firstOption.caughtErrorsIgnorePattern) {
                        config.caughtErrorsIgnorePattern = new RegExp(firstOption.caughtErrorsIgnorePattern, "u");
                    }
                }
            }
            /**
             * Generate the warning message about the variable being
             * defined and unused, including the ignore pattern if configured.
             * @param {Variable} unusedVar eslint-scope variable object.
             * @returns {string} The warning message to be used with this unused variable.
             */
            function getDefinedMessage(unusedVar) {
                const defType = unusedVar.defs && unusedVar.defs[0] && unusedVar.defs[0].type;
                let type;
                let pattern;
                if (defType === "CatchClause" && config.caughtErrorsIgnorePattern) {
                    type = "args";
                    pattern = config.caughtErrorsIgnorePattern.toString();
                }
                else if (defType === "Parameter" && config.argsIgnorePattern) {
                    type = "args";
                    pattern = config.argsIgnorePattern.toString();
                }
                else if (defType !== "Parameter" && config.varsIgnorePattern) {
                    type = "vars";
                    pattern = config.varsIgnorePattern.toString();
                }
                const additional = type ? ` Allowed unused ${type} must match ${pattern}.` : "";
                return `'{{name}}' is defined but never used.${additional}`;
            }
            /**
             * Generate the warning message about the variable being
             * assigned and unused, including the ignore pattern if configured.
             * @returns {string} The warning message to be used with this unused variable.
             */
            function getAssignedMessage() {
                const additional = config.varsIgnorePattern ? ` Allowed unused vars must match ${config.varsIgnorePattern.toString()}.` : "";
                return `'{{name}}' is assigned a value but never used.${additional}`;
            }
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            const STATEMENT_TYPE = /(?:Statement|Declaration)$/u;
            /**
             * Determines if a given variable is being exported from a module.
             * @param {Variable} variable eslint-scope variable object.
             * @returns {boolean} True if the variable is exported, false if not.
             * @private
             */
            function isExported(variable) {
                const definition = variable.defs[0];
                if (definition) {
                    let node = definition.node;
                    if (node.type === "VariableDeclarator") {
                        node = node.parent;
                    }
                    else if (definition.type === "Parameter") {
                        return false;
                    }
                    return node.parent.type.indexOf("Export") === 0;
                }
                return false;
            }
            /**
             * Determines if a variable has a sibling rest property
             * @param {Variable} variable eslint-scope variable object.
             * @returns {boolean} True if the variable is exported, false if not.
             * @private
             */
            function hasRestSpreadSibling(variable) {
                if (config.ignoreRestSiblings) {
                    return variable.defs.some(def => {
                        const propertyNode = def.name.parent;
                        const patternNode = propertyNode.parent;
                        return (propertyNode.type === "Property" &&
                            patternNode.type === "ObjectPattern" &&
                            REST_PROPERTY_TYPE.test(patternNode.properties[patternNode.properties.length - 1].type));
                    });
                }
                return false;
            }
            /**
             * Determines if a reference is a read operation.
             * @param {Reference} ref An eslint-scope Reference
             * @returns {boolean} whether the given reference represents a read operation
             * @private
             */
            function isReadRef(ref) {
                return ref.isRead();
            }
            /**
             * Determine if an identifier is referencing an enclosing function name.
             * @param {Reference} ref The reference to check.
             * @param {ASTNode[]} nodes The candidate function nodes.
             * @returns {boolean} True if it's a self-reference, false if not.
             * @private
             */
            function isSelfReference(ref, nodes) {
                let scope = ref.from;
                while (scope) {
                    if (nodes.indexOf(scope.block) >= 0) {
                        return true;
                    }
                    scope = scope.upper;
                }
                return false;
            }
            /**
             * Gets a list of function definitions for a specified variable.
             * @param {Variable} variable eslint-scope variable object.
             * @returns {ASTNode[]} Function nodes.
             * @private
             */
            function getFunctionDefinitions(variable) {
                const functionDefinitions = [];
                variable.defs.forEach(def => {
                    const { type, node } = def;
                    // FunctionDeclarations
                    if (type === "FunctionName") {
                        functionDefinitions.push(node);
                    }
                    // FunctionExpressions
                    if (type === "Variable" && node.init &&
                        (node.init.type === "FunctionExpression" || node.init.type === "ArrowFunctionExpression")) {
                        functionDefinitions.push(node.init);
                    }
                });
                return functionDefinitions;
            }
            /**
             * Checks the position of given nodes.
             * @param {ASTNode} inner A node which is expected as inside.
             * @param {ASTNode} outer A node which is expected as outside.
             * @returns {boolean} `true` if the `inner` node exists in the `outer` node.
             * @private
             */
            function isInside(inner, outer) {
                return (inner.range[0] >= outer.range[0] &&
                    inner.range[1] <= outer.range[1]);
            }
            /**
             * If a given reference is left-hand side of an assignment, this gets
             * the right-hand side node of the assignment.
             *
             * In the following cases, this returns null.
             *
             * - The reference is not the LHS of an assignment expression.
             * - The reference is inside of a loop.
             * - The reference is inside of a function scope which is different from
             *   the declaration.
             * @param {eslint-scope.Reference} ref A reference to check.
             * @param {ASTNode} prevRhsNode The previous RHS node.
             * @returns {ASTNode|null} The RHS node or null.
             * @private
             */
            function getRhsNode(ref, prevRhsNode) {
                const id = ref.identifier;
                const parent = id.parent;
                const granpa = parent.parent;
                const refScope = ref.from.variableScope;
                const varScope = ref.resolved.scope.variableScope;
                const canBeUsedLater = refScope !== varScope || astUtils.isInLoop(id);
                /*
                 * Inherits the previous node if this reference is in the node.
                 * This is for `a = a + a`-like code.
                 */
                if (prevRhsNode && isInside(id, prevRhsNode)) {
                    return prevRhsNode;
                }
                if (parent.type === "AssignmentExpression" &&
                    granpa.type === "ExpressionStatement" &&
                    id === parent.left &&
                    !canBeUsedLater) {
                    return parent.right;
                }
                return null;
            }
            /**
             * Checks whether a given function node is stored to somewhere or not.
             * If the function node is stored, the function can be used later.
             * @param {ASTNode} funcNode A function node to check.
             * @param {ASTNode} rhsNode The RHS node of the previous assignment.
             * @returns {boolean} `true` if under the following conditions:
             *      - the funcNode is assigned to a variable.
             *      - the funcNode is bound as an argument of a function call.
             *      - the function is bound to a property and the object satisfies above conditions.
             * @private
             */
            function isStorableFunction(funcNode, rhsNode) {
                let node = funcNode;
                let parent = funcNode.parent;
                while (parent && isInside(parent, rhsNode)) {
                    switch (parent.type) {
                        case "SequenceExpression":
                            if (parent.expressions[parent.expressions.length - 1] !== node) {
                                return false;
                            }
                            break;
                        case "CallExpression":
                        case "NewExpression":
                            return parent.callee !== node;
                        case "AssignmentExpression":
                        case "TaggedTemplateExpression":
                        case "YieldExpression":
                            return true;
                        default:
                            if (STATEMENT_TYPE.test(parent.type)) {
                                /*
                                 * If it encountered statements, this is a complex pattern.
                                 * Since analyzeing complex patterns is hard, this returns `true` to avoid false positive.
                                 */
                                return true;
                            }
                    }
                    node = parent;
                    parent = parent.parent;
                }
                return false;
            }
            /**
             * Checks whether a given Identifier node exists inside of a function node which can be used later.
             *
             * "can be used later" means:
             * - the function is assigned to a variable.
             * - the function is bound to a property and the object can be used later.
             * - the function is bound as an argument of a function call.
             *
             * If a reference exists in a function which can be used later, the reference is read when the function is called.
             * @param {ASTNode} id An Identifier node to check.
             * @param {ASTNode} rhsNode The RHS node of the previous assignment.
             * @returns {boolean} `true` if the `id` node exists inside of a function node which can be used later.
             * @private
             */
            function isInsideOfStorableFunction(id, rhsNode) {
                const funcNode = astUtils.getUpperFunction(id);
                return (funcNode &&
                    isInside(funcNode, rhsNode) &&
                    isStorableFunction(funcNode, rhsNode));
            }
            /**
             * Checks whether a given reference is a read to update itself or not.
             * @param {eslint-scope.Reference} ref A reference to check.
             * @param {ASTNode} rhsNode The RHS node of the previous assignment.
             * @returns {boolean} The reference is a read to update itself.
             * @private
             */
            function isReadForItself(ref, rhsNode) {
                const id = ref.identifier;
                const parent = id.parent;
                const granpa = parent.parent;
                return ref.isRead() && (
                // self update. e.g. `a += 1`, `a++`
                ( // in RHS of an assignment for itself. e.g. `a = a + 1`
                ((parent.type === "AssignmentExpression" &&
                    granpa.type === "ExpressionStatement" &&
                    parent.left === id) ||
                    (parent.type === "UpdateExpression" &&
                        granpa.type === "ExpressionStatement") || rhsNode &&
                    isInside(id, rhsNode) &&
                    !isInsideOfStorableFunction(id, rhsNode))));
            }
            /**
             * Determine if an identifier is used either in for-in loops.
             * @param {Reference} ref The reference to check.
             * @returns {boolean} whether reference is used in the for-in loops
             * @private
             */
            function isForInRef(ref) {
                let target = ref.identifier.parent;
                // "for (var ...) { return; }"
                if (target.type === "VariableDeclarator") {
                    target = target.parent.parent;
                }
                if (target.type !== "ForInStatement") {
                    return false;
                }
                // "for (...) { return; }"
                if (target.body.type === "BlockStatement") {
                    target = target.body.body[0];
                    // "for (...) return;"
                }
                else {
                    target = target.body;
                }
                // For empty loop body
                if (!target) {
                    return false;
                }
                return target.type === "ReturnStatement";
            }
            /**
             * Determines if the variable is used.
             * @param {Variable} variable The variable to check.
             * @returns {boolean} True if the variable is used
             * @private
             */
            function isUsedVariable(variable) {
                const functionNodes = getFunctionDefinitions(variable), isFunctionDefinition = functionNodes.length > 0;
                let rhsNode = null;
                return variable.references.some(ref => {
                    if (isForInRef(ref)) {
                        return true;
                    }
                    const forItself = isReadForItself(ref, rhsNode);
                    rhsNode = getRhsNode(ref, rhsNode);
                    return (isReadRef(ref) &&
                        !forItself &&
                        !(isFunctionDefinition && isSelfReference(ref, functionNodes)));
                });
            }
            /**
             * Checks whether the given variable is after the last used parameter.
             * @param {eslint-scope.Variable} variable The variable to check.
             * @returns {boolean} `true` if the variable is defined after the last
             * used parameter.
             */
            function isAfterLastUsedArg(variable) {
                const def = variable.defs[0];
                const params = context.getDeclaredVariables(def.node);
                const posteriorParams = params.slice(params.indexOf(variable) + 1);
                // If any used parameters occur after this parameter, do not report.
                return !posteriorParams.some(v => v.references.length > 0 || v.eslintUsed);
            }
            /**
             * Gets an array of variables without read references.
             * @param {Scope} scope an eslint-scope Scope object.
             * @param {Variable[]} unusedVars an array that saving result.
             * @returns {Variable[]} unused variables of the scope and descendant scopes.
             * @private
             */
            function collectUnusedVariables(scope, unusedVars) {
                const variables = scope.variables;
                const childScopes = scope.childScopes;
                let i, l;
                if (scope.type !== "global" || config.vars === "all") {
                    for (i = 0, l = variables.length; i < l; ++i) {
                        const variable = variables[i];
                        // skip a variable of class itself name in the class scope
                        if (scope.type === "class" && scope.block.id === variable.identifiers[0]) {
                            continue;
                        }
                        // skip function expression names and variables marked with markVariableAsUsed()
                        if (scope.functionExpressionScope || variable.eslintUsed) {
                            continue;
                        }
                        // skip implicit "arguments" variable
                        if (scope.type === "function" && variable.name === "arguments" && variable.identifiers.length === 0) {
                            continue;
                        }
                        // explicit global variables don't have definitions.
                        const def = variable.defs[0];
                        if (def) {
                            const type = def.type;
                            // skip catch variables
                            if (type === "CatchClause") {
                                if (config.caughtErrors === "none") {
                                    continue;
                                }
                                // skip ignored parameters
                                if (config.caughtErrorsIgnorePattern && config.caughtErrorsIgnorePattern.test(def.name.name)) {
                                    continue;
                                }
                            }
                            if (type === "Parameter") {
                                // skip any setter argument
                                if ((def.node.parent.type === "Property" || def.node.parent.type === "MethodDefinition") && def.node.parent.kind === "set") {
                                    continue;
                                }
                                // if "args" option is "none", skip any parameter
                                if (config.args === "none") {
                                    continue;
                                }
                                // skip ignored parameters
                                if (config.argsIgnorePattern && config.argsIgnorePattern.test(def.name.name)) {
                                    continue;
                                }
                                // if "args" option is "after-used", skip used variables
                                if (config.args === "after-used" && astUtils.isFunction(def.name.parent) && !isAfterLastUsedArg(variable)) {
                                    continue;
                                }
                            }
                            else {
                                // skip ignored variables
                                if (config.varsIgnorePattern && config.varsIgnorePattern.test(def.name.name)) {
                                    continue;
                                }
                            }
                        }
                        if (!isUsedVariable(variable) && !isExported(variable) && !hasRestSpreadSibling(variable)) {
                            unusedVars.push(variable);
                        }
                    }
                }
                for (i = 0, l = childScopes.length; i < l; ++i) {
                    collectUnusedVariables(childScopes[i], unusedVars);
                }
                return unusedVars;
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                "Program:exit"(programNode) {
                    const unusedVars = collectUnusedVariables(context.getScope(), []);
                    for (let i = 0, l = unusedVars.length; i < l; ++i) {
                        const unusedVar = unusedVars[i];
                        // Report the first declaration.
                        if (unusedVar.defs.length > 0) {
                            context.report({
                                node: unusedVar.identifiers[0],
                                message: unusedVar.references.some(ref => ref.isWrite())
                                    ? getAssignedMessage()
                                    : getDefinedMessage(unusedVar),
                                data: unusedVar
                            });
                            // If there are no regular declaration, report the first `/*globals*/` comment directive.
                        }
                        else if (unusedVar.eslintExplicitGlobalComments) {
                            const directiveComment = unusedVar.eslintExplicitGlobalComments[0];
                            context.report({
                                node: programNode,
                                loc: astUtils.getNameLocationInGlobalDirectiveComment(sourceCode, directiveComment, unusedVar.name),
                                message: getDefinedMessage(unusedVar),
                                data: unusedVar
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-use-before-define.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const SENTINEL_TYPE = /^(?:(?:Function|Class)(?:Declaration|Expression)|ArrowFunctionExpression|CatchClause|ImportDeclaration|ExportNamedDeclaration)$/u;
    const FOR_IN_OF_TYPE = /^For(?:In|Of)Statement$/u;
    /**
     * Parses a given value as options.
     * @param {any} options A value to parse.
     * @returns {Object} The parsed options.
     */
    function parseOptions(options) {
        let functions = true;
        let classes = true;
        let variables = true;
        if (typeof options === "string") {
            functions = (options !== "nofunc");
        }
        else if (typeof options === "object" && options !== null) {
            functions = options.functions !== false;
            classes = options.classes !== false;
            variables = options.variables !== false;
        }
        return { functions, classes, variables };
    }
    /**
     * Checks whether or not a given variable is a function declaration.
     * @param {eslint-scope.Variable} variable A variable to check.
     * @returns {boolean} `true` if the variable is a function declaration.
     */
    function isFunction(variable) {
        return variable.defs[0].type === "FunctionName";
    }
    /**
     * Checks whether or not a given variable is a class declaration in an upper function scope.
     * @param {eslint-scope.Variable} variable A variable to check.
     * @param {eslint-scope.Reference} reference A reference to check.
     * @returns {boolean} `true` if the variable is a class declaration.
     */
    function isOuterClass(variable, reference) {
        return (variable.defs[0].type === "ClassName" &&
            variable.scope.variableScope !== reference.from.variableScope);
    }
    /**
     * Checks whether or not a given variable is a variable declaration in an upper function scope.
     * @param {eslint-scope.Variable} variable A variable to check.
     * @param {eslint-scope.Reference} reference A reference to check.
     * @returns {boolean} `true` if the variable is a variable declaration.
     */
    function isOuterVariable(variable, reference) {
        return (variable.defs[0].type === "Variable" &&
            variable.scope.variableScope !== reference.from.variableScope);
    }
    /**
     * Checks whether or not a given location is inside of the range of a given node.
     * @param {ASTNode} node An node to check.
     * @param {number} location A location to check.
     * @returns {boolean} `true` if the location is inside of the range of the node.
     */
    function isInRange(node, location) {
        return node && node.range[0] <= location && location <= node.range[1];
    }
    /**
     * Checks whether or not a given reference is inside of the initializers of a given variable.
     *
     * This returns `true` in the following cases:
     *
     *     var a = a
     *     var [a = a] = list
     *     var {a = a} = obj
     *     for (var a in a) {}
     *     for (var a of a) {}
     * @param {Variable} variable A variable to check.
     * @param {Reference} reference A reference to check.
     * @returns {boolean} `true` if the reference is inside of the initializers.
     */
    function isInInitializer(variable, reference) {
        if (variable.scope !== reference.from) {
            return false;
        }
        let node = variable.identifiers[0].parent;
        const location = reference.identifier.range[1];
        while (node) {
            if (node.type === "VariableDeclarator") {
                if (isInRange(node.init, location)) {
                    return true;
                }
                if (FOR_IN_OF_TYPE.test(node.parent.parent.type) &&
                    isInRange(node.parent.parent.right, location)) {
                    return true;
                }
                break;
            }
            else if (node.type === "AssignmentPattern") {
                if (isInRange(node.right, location)) {
                    return true;
                }
            }
            else if (SENTINEL_TYPE.test(node.type)) {
                break;
            }
            node = node.parent;
        }
        return false;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow the use of variables before they are defined",
                category: "Variables",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-use-before-define"
            },
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["nofunc"]
                        },
                        {
                            type: "object",
                            properties: {
                                functions: { type: "boolean" },
                                classes: { type: "boolean" },
                                variables: { type: "boolean" }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ]
        },
        create(context) {
            const options = parseOptions(context.options[0]);
            /**
             * Determines whether a given use-before-define case should be reported according to the options.
             * @param {eslint-scope.Variable} variable The variable that gets used before being defined
             * @param {eslint-scope.Reference} reference The reference to the variable
             * @returns {boolean} `true` if the usage should be reported
             */
            function isForbidden(variable, reference) {
                if (isFunction(variable)) {
                    return options.functions;
                }
                if (isOuterClass(variable, reference)) {
                    return options.classes;
                }
                if (isOuterVariable(variable, reference)) {
                    return options.variables;
                }
                return true;
            }
            /**
             * Finds and validates all variables in a given scope.
             * @param {Scope} scope The scope object.
             * @returns {void}
             * @private
             */
            function findVariablesInScope(scope) {
                scope.references.forEach(reference => {
                    const variable = reference.resolved;
                    /*
                     * Skips when the reference is:
                     * - initialization's.
                     * - referring to an undefined variable.
                     * - referring to a global environment variable (there're no identifiers).
                     * - located preceded by the variable (except in initializers).
                     * - allowed by options.
                     */
                    if (reference.init ||
                        !variable ||
                        variable.identifiers.length === 0 ||
                        (variable.identifiers[0].range[1] < reference.identifier.range[1] && !isInInitializer(variable, reference)) ||
                        !isForbidden(variable, reference)) {
                        return;
                    }
                    // Reports.
                    context.report({
                        node: reference.identifier,
                        message: "'{{name}}' was used before it was defined.",
                        data: reference.identifier
                    });
                });
                scope.childScopes.forEach(findVariablesInScope);
            }
            return {
                Program() {
                    findVariablesInScope(context.getScope());
                }
            };
        }
    };
},
/* eslint/lib/rules/no-useless-call.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a node is a `.call()`/`.apply()`.
     * @param {ASTNode} node A CallExpression node to check.
     * @returns {boolean} Whether or not the node is a `.call()`/`.apply()`.
     */
    function isCallOrNonVariadicApply(node) {
        return (node.callee.type === "MemberExpression" &&
            node.callee.property.type === "Identifier" &&
            node.callee.computed === false &&
            ((node.callee.property.name === "call" && node.arguments.length >= 1) ||
                (node.callee.property.name === "apply" && node.arguments.length === 2 && node.arguments[1].type === "ArrayExpression")));
    }
    /**
     * Checks whether or not `thisArg` is not changed by `.call()`/`.apply()`.
     * @param {ASTNode|null} expectedThis The node that is the owner of the applied function.
     * @param {ASTNode} thisArg The node that is given to the first argument of the `.call()`/`.apply()`.
     * @param {SourceCode} sourceCode The ESLint source code object.
     * @returns {boolean} Whether or not `thisArg` is not changed by `.call()`/`.apply()`.
     */
    function isValidThisArg(expectedThis, thisArg, sourceCode) {
        if (!expectedThis) {
            return astUtils.isNullOrUndefined(thisArg);
        }
        return astUtils.equalTokens(expectedThis, thisArg, sourceCode);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary calls to `.call()` and `.apply()`",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-useless-call"
            },
            schema: []
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                CallExpression(node) {
                    if (!isCallOrNonVariadicApply(node)) {
                        return;
                    }
                    const applied = node.callee.object;
                    const expectedThis = (applied.type === "MemberExpression") ? applied.object : null;
                    const thisArg = node.arguments[0];
                    if (isValidThisArg(expectedThis, thisArg, sourceCode)) {
                        context.report({ node, message: "unnecessary '.{{name}}()'.", data: { name: node.callee.property.name } });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-useless-catch.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary `catch` clauses",
                category: "Best Practices",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-useless-catch"
            },
            schema: []
        },
        create(context) {
            return {
                CatchClause(node) {
                    if (node.param &&
                        node.param.type === "Identifier" &&
                        node.body.body.length &&
                        node.body.body[0].type === "ThrowStatement" &&
                        node.body.body[0].argument.type === "Identifier" &&
                        node.body.body[0].argument.name === node.param.name) {
                        if (node.parent.finalizer) {
                            context.report({
                                node,
                                message: "Unnecessary catch clause."
                            });
                        }
                        else {
                            context.report({
                                node: node.parent,
                                message: "Unnecessary try/catch wrapper."
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-useless-computed-key.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const MESSAGE_UNNECESSARY_COMPUTED = "Unnecessarily computed property [{{property}}] found.";
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary computed property keys in objects and classes",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-useless-computed-key"
            },
            schema: [{
                    type: "object",
                    properties: {
                        enforceForClassMembers: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }],
            fixable: "code"
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const enforceForClassMembers = context.options[0] && context.options[0].enforceForClassMembers;
            /**
             * Reports a given node if it violated this rule.
             * @param {ASTNode} node The node to check.
             * @returns {void}
             */
            function check(node) {
                if (!node.computed) {
                    return;
                }
                const key = node.key, nodeType = typeof key.value;
                let allowedKey;
                if (node.type === "MethodDefinition") {
                    allowedKey = node.static ? "prototype" : "constructor";
                }
                else {
                    allowedKey = "__proto__";
                }
                if (key.type === "Literal" && (nodeType === "string" || nodeType === "number") && key.value !== allowedKey) {
                    context.report({
                        node,
                        message: MESSAGE_UNNECESSARY_COMPUTED,
                        data: { property: sourceCode.getText(key) },
                        fix(fixer) {
                            const leftSquareBracket = sourceCode.getFirstToken(node, astUtils.isOpeningBracketToken);
                            const rightSquareBracket = sourceCode.getFirstTokenBetween(node.key, node.value, astUtils.isClosingBracketToken);
                            const tokensBetween = sourceCode.getTokensBetween(leftSquareBracket, rightSquareBracket, 1);
                            if (tokensBetween.slice(0, -1).some((token, index) => sourceCode.getText().slice(token.range[1], tokensBetween[index + 1].range[0]).trim())) {
                                // If there are comments between the brackets and the property name, don't do a fix.
                                return null;
                            }
                            const tokenBeforeLeftBracket = sourceCode.getTokenBefore(leftSquareBracket);
                            // Insert a space before the key to avoid changing identifiers, e.g. ({ get[2]() {} }) to ({ get2() {} })
                            const needsSpaceBeforeKey = tokenBeforeLeftBracket.range[1] === leftSquareBracket.range[0] &&
                                !astUtils.canTokensBeAdjacent(tokenBeforeLeftBracket, sourceCode.getFirstToken(key));
                            const replacementKey = (needsSpaceBeforeKey ? " " : "") + key.raw;
                            return fixer.replaceTextRange([leftSquareBracket.range[0], rightSquareBracket.range[1]], replacementKey);
                        }
                    });
                }
            }
            return {
                Property: check,
                MethodDefinition: enforceForClassMembers ? check : lodash.noop
            };
        }
    };
},
/* eslint/lib/rules/no-useless-concat.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a given node is a concatenation.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node is a concatenation.
     */
    function isConcatenation(node) {
        return node.type === "BinaryExpression" && node.operator === "+";
    }
    /**
     * Checks if the given token is a `+` token or not.
     * @param {Token} token The token to check.
     * @returns {boolean} `true` if the token is a `+` token.
     */
    function isConcatOperatorToken(token) {
        return token.value === "+" && token.type === "Punctuator";
    }
    /**
     * Get's the right most node on the left side of a BinaryExpression with + operator.
     * @param {ASTNode} node A BinaryExpression node to check.
     * @returns {ASTNode} node
     */
    function getLeft(node) {
        let left = node.left;
        while (isConcatenation(left)) {
            left = left.right;
        }
        return left;
    }
    /**
     * Get's the left most node on the right side of a BinaryExpression with + operator.
     * @param {ASTNode} node A BinaryExpression node to check.
     * @returns {ASTNode} node
     */
    function getRight(node) {
        let right = node.right;
        while (isConcatenation(right)) {
            right = right.left;
        }
        return right;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary concatenation of literals or template literals",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-useless-concat"
            },
            schema: []
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                BinaryExpression(node) {
                    // check if not concatenation
                    if (node.operator !== "+") {
                        return;
                    }
                    // account for the `foo + "a" + "b"` case
                    const left = getLeft(node);
                    const right = getRight(node);
                    if (astUtils.isStringLiteral(left) &&
                        astUtils.isStringLiteral(right) &&
                        astUtils.isTokenOnSameLine(left, right)) {
                        const operatorToken = sourceCode.getFirstTokenBetween(left, right, isConcatOperatorToken);
                        context.report({
                            node,
                            loc: operatorToken.loc.start,
                            message: "Unexpected string concatenation of literals."
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-useless-constructor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether a given array of statements is a single call of `super`.
     * @param {ASTNode[]} body An array of statements to check.
     * @returns {boolean} `true` if the body is a single call of `super`.
     */
    function isSingleSuperCall(body) {
        return (body.length === 1 &&
            body[0].type === "ExpressionStatement" &&
            body[0].expression.type === "CallExpression" &&
            body[0].expression.callee.type === "Super");
    }
    /**
     * Checks whether a given node is a pattern which doesn't have any side effects.
     * Default parameters and Destructuring parameters can have side effects.
     * @param {ASTNode} node A pattern node.
     * @returns {boolean} `true` if the node doesn't have any side effects.
     */
    function isSimple(node) {
        return node.type === "Identifier" || node.type === "RestElement";
    }
    /**
     * Checks whether a given array of expressions is `...arguments` or not.
     * `super(...arguments)` passes all arguments through.
     * @param {ASTNode[]} superArgs An array of expressions to check.
     * @returns {boolean} `true` if the superArgs is `...arguments`.
     */
    function isSpreadArguments(superArgs) {
        return (superArgs.length === 1 &&
            superArgs[0].type === "SpreadElement" &&
            superArgs[0].argument.type === "Identifier" &&
            superArgs[0].argument.name === "arguments");
    }
    /**
     * Checks whether given 2 nodes are identifiers which have the same name or not.
     * @param {ASTNode} ctorParam A node to check.
     * @param {ASTNode} superArg A node to check.
     * @returns {boolean} `true` if the nodes are identifiers which have the same
     *      name.
     */
    function isValidIdentifierPair(ctorParam, superArg) {
        return (ctorParam.type === "Identifier" &&
            superArg.type === "Identifier" &&
            ctorParam.name === superArg.name);
    }
    /**
     * Checks whether given 2 nodes are a rest/spread pair which has the same values.
     * @param {ASTNode} ctorParam A node to check.
     * @param {ASTNode} superArg A node to check.
     * @returns {boolean} `true` if the nodes are a rest/spread pair which has the
     *      same values.
     */
    function isValidRestSpreadPair(ctorParam, superArg) {
        return (ctorParam.type === "RestElement" &&
            superArg.type === "SpreadElement" &&
            isValidIdentifierPair(ctorParam.argument, superArg.argument));
    }
    /**
     * Checks whether given 2 nodes have the same value or not.
     * @param {ASTNode} ctorParam A node to check.
     * @param {ASTNode} superArg A node to check.
     * @returns {boolean} `true` if the nodes have the same value or not.
     */
    function isValidPair(ctorParam, superArg) {
        return (isValidIdentifierPair(ctorParam, superArg) ||
            isValidRestSpreadPair(ctorParam, superArg));
    }
    /**
     * Checks whether the parameters of a constructor and the arguments of `super()`
     * have the same values or not.
     * @param {ASTNode} ctorParams The parameters of a constructor to check.
     * @param {ASTNode} superArgs The arguments of `super()` to check.
     * @returns {boolean} `true` if those have the same values.
     */
    function isPassingThrough(ctorParams, superArgs) {
        if (ctorParams.length !== superArgs.length) {
            return false;
        }
        for (let i = 0; i < ctorParams.length; ++i) {
            if (!isValidPair(ctorParams[i], superArgs[i])) {
                return false;
            }
        }
        return true;
    }
    /**
     * Checks whether the constructor body is a redundant super call.
     * @param {Array} body constructor body content.
     * @param {Array} ctorParams The params to check against super call.
     * @returns {boolean} true if the construtor body is redundant
     */
    function isRedundantSuperCall(body, ctorParams) {
        return (isSingleSuperCall(body) &&
            ctorParams.every(isSimple) &&
            (isSpreadArguments(body[0].expression.arguments) ||
                isPassingThrough(ctorParams, body[0].expression.arguments)));
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary constructors",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-useless-constructor"
            },
            schema: []
        },
        create(context) {
            /**
             * Checks whether a node is a redundant constructor
             * @param {ASTNode} node node to check
             * @returns {void}
             */
            function checkForConstructor(node) {
                if (node.kind !== "constructor") {
                    return;
                }
                const body = node.value.body.body;
                const ctorParams = node.value.params;
                const superClass = node.parent.parent.superClass;
                if (superClass ? isRedundantSuperCall(body, ctorParams) : (body.length === 0)) {
                    context.report({
                        node,
                        message: "Useless constructor."
                    });
                }
            }
            return {
                MethodDefinition: checkForConstructor
            };
        }
    };
},
/* eslint/lib/rules/no-useless-escape.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    /**
     * Returns the union of two sets.
     * @param {Set} setA The first set
     * @param {Set} setB The second set
     * @returns {Set} The union of the two sets
     */
    function union(setA, setB) {
        return new Set(function* () {
            yield* setA;
            yield* setB;
        }());
    }
    const VALID_STRING_ESCAPES = union(new Set("\\nrvtbfux"), astUtils.LINEBREAKS);
    const REGEX_GENERAL_ESCAPES = new Set("\\bcdDfnpPrsStvwWxu0123456789]");
    const REGEX_NON_CHARCLASS_ESCAPES = union(REGEX_GENERAL_ESCAPES, new Set("^/.$*+?[{}|()Bk"));
    /**
     * Parses a regular expression into a list of characters with character class info.
     * @param {string} regExpText The raw text used to create the regular expression
     * @returns {Object[]} A list of characters, each with info on escaping and whether they're in a character class.
     * @example
     *
     * parseRegExp('a\\b[cd-]')
     *
     * returns:
     * [
     *   {text: 'a', index: 0, escaped: false, inCharClass: false, startsCharClass: false, endsCharClass: false},
     *   {text: 'b', index: 2, escaped: true, inCharClass: false, startsCharClass: false, endsCharClass: false},
     *   {text: 'c', index: 4, escaped: false, inCharClass: true, startsCharClass: true, endsCharClass: false},
     *   {text: 'd', index: 5, escaped: false, inCharClass: true, startsCharClass: false, endsCharClass: false},
     *   {text: '-', index: 6, escaped: false, inCharClass: true, startsCharClass: false, endsCharClass: false}
     * ]
     */
    function parseRegExp(regExpText) {
        const charList = [];
        regExpText.split("").reduce((state, char, index) => {
            if (!state.escapeNextChar) {
                if (char === "\\") {
                    return Object.assign(state, { escapeNextChar: true });
                }
                if (char === "[" && !state.inCharClass) {
                    return Object.assign(state, { inCharClass: true, startingCharClass: true });
                }
                if (char === "]" && state.inCharClass) {
                    if (charList.length && charList[charList.length - 1].inCharClass) {
                        charList[charList.length - 1].endsCharClass = true;
                    }
                    return Object.assign(state, { inCharClass: false, startingCharClass: false });
                }
            }
            charList.push({
                text: char,
                index,
                escaped: state.escapeNextChar,
                inCharClass: state.inCharClass,
                startsCharClass: state.startingCharClass,
                endsCharClass: false
            });
            return Object.assign(state, { escapeNextChar: false, startingCharClass: false });
        }, { escapeNextChar: false, inCharClass: false, startingCharClass: false });
        return charList;
    }
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow unnecessary escape characters",
                category: "Best Practices",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-useless-escape",
                suggestion: true
            },
            messages: {
                unnecessaryEscape: "Unnecessary escape character: \\{{character}}.",
                removeEscape: "Remove the `\\`. This maintains the current functionality.",
                escapeBackslash: "Replace the `\\` with `\\\\` to include the actual backslash character."
            },
            schema: []
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Reports a node
             * @param {ASTNode} node The node to report
             * @param {number} startOffset The backslash's offset from the start of the node
             * @param {string} character The uselessly escaped character (not including the backslash)
             * @returns {void}
             */
            function report(node, startOffset, character) {
                const start = sourceCode.getLocFromIndex(sourceCode.getIndexFromLoc(node.loc.start) + startOffset);
                const rangeStart = sourceCode.getIndexFromLoc(node.loc.start) + startOffset;
                const range = [rangeStart, rangeStart + 1];
                context.report({
                    node,
                    loc: {
                        start,
                        end: { line: start.line, column: start.column + 1 }
                    },
                    messageId: "unnecessaryEscape",
                    data: { character },
                    suggest: [
                        {
                            messageId: "removeEscape",
                            fix(fixer) {
                                return fixer.removeRange(range);
                            }
                        },
                        {
                            messageId: "escapeBackslash",
                            fix(fixer) {
                                return fixer.insertTextBeforeRange(range, "\\");
                            }
                        }
                    ]
                });
            }
            /**
             * Checks if the escape character in given string slice is unnecessary.
             * @private
             * @param {ASTNode} node node to validate.
             * @param {string} match string slice to validate.
             * @returns {void}
             */
            function validateString(node, match) {
                const isTemplateElement = node.type === "TemplateElement";
                const escapedChar = match[0][1];
                let isUnnecessaryEscape = !VALID_STRING_ESCAPES.has(escapedChar);
                let isQuoteEscape;
                if (isTemplateElement) {
                    isQuoteEscape = escapedChar === "`";
                    if (escapedChar === "$") {
                        // Warn if `\$` is not followed by `{`
                        isUnnecessaryEscape = match.input[match.index + 2] !== "{";
                    }
                    else if (escapedChar === "{") {
                        /*
                         * Warn if `\{` is not preceded by `$`. If preceded by `$`, escaping
                         * is necessary and the rule should not warn. If preceded by `/$`, the rule
                         * will warn for the `/$` instead, as it is the first unnecessarily escaped character.
                         */
                        isUnnecessaryEscape = match.input[match.index - 1] !== "$";
                    }
                }
                else {
                    isQuoteEscape = escapedChar === node.raw[0];
                }
                if (isUnnecessaryEscape && !isQuoteEscape) {
                    report(node, match.index + 1, match[0].slice(1));
                }
            }
            /**
             * Checks if a node has an escape.
             * @param {ASTNode} node node to check.
             * @returns {void}
             */
            function check(node) {
                const isTemplateElement = node.type === "TemplateElement";
                if (isTemplateElement &&
                    node.parent &&
                    node.parent.parent &&
                    node.parent.parent.type === "TaggedTemplateExpression" &&
                    node.parent === node.parent.parent.quasi) {
                    // Don't report tagged template literals, because the backslash character is accessible to the tag function.
                    return;
                }
                if (typeof node.value === "string" || isTemplateElement) {
                    /*
                     * JSXAttribute doesn't have any escape sequence: https://facebook.github.io/jsx/.
                     * In addition, backticks are not supported by JSX yet: https://github.com/facebook/jsx/issues/25.
                     */
                    if (node.parent.type === "JSXAttribute" || node.parent.type === "JSXElement" || node.parent.type === "JSXFragment") {
                        return;
                    }
                    const value = isTemplateElement ? node.value.raw : node.raw.slice(1, -1);
                    const pattern = /\\[^\d]/gu;
                    let match;
                    while ((match = pattern.exec(value))) {
                        validateString(node, match);
                    }
                }
                else if (node.regex) {
                    parseRegExp(node.regex.pattern)
                        /*
                         * The '-' character is a special case, because it's only valid to escape it if it's in a character
                         * class, and is not at either edge of the character class. To account for this, don't consider '-'
                         * characters to be valid in general, and filter out '-' characters that appear in the middle of a
                         * character class.
                         */
                        .filter(charInfo => !(charInfo.text === "-" && charInfo.inCharClass && !charInfo.startsCharClass && !charInfo.endsCharClass))
                        /*
                         * The '^' character is also a special case; it must always be escaped outside of character classes, but
                         * it only needs to be escaped in character classes if it's at the beginning of the character class. To
                         * account for this, consider it to be a valid escape character outside of character classes, and filter
                         * out '^' characters that appear at the start of a character class.
                         */
                        .filter(charInfo => !(charInfo.text === "^" && charInfo.startsCharClass))
                        // Filter out characters that aren't escaped.
                        .filter(charInfo => charInfo.escaped)
                        // Filter out characters that are valid to escape, based on their position in the regular expression.
                        .filter(charInfo => !(charInfo.inCharClass ? REGEX_GENERAL_ESCAPES : REGEX_NON_CHARCLASS_ESCAPES).has(charInfo.text))
                        // Report all the remaining characters.
                        .forEach(charInfo => report(node, charInfo.index, charInfo.text));
                }
            }
            return {
                Literal: check,
                TemplateElement: check
            };
        }
    };
},
/* eslint/lib/rules/no-useless-rename.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow renaming import, export, and destructured assignments to the same name",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-useless-rename"
            },
            fixable: "code",
            schema: [
                {
                    type: "object",
                    properties: {
                        ignoreDestructuring: { type: "boolean", default: false },
                        ignoreImport: { type: "boolean", default: false },
                        ignoreExport: { type: "boolean", default: false }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode(), options = context.options[0] || {}, ignoreDestructuring = options.ignoreDestructuring === true, ignoreImport = options.ignoreImport === true, ignoreExport = options.ignoreExport === true;
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Reports error for unnecessarily renamed assignments
             * @param {ASTNode} node node to report
             * @param {ASTNode} initial node with initial name value
             * @param {ASTNode} result node with new name value
             * @param {string} type the type of the offending node
             * @returns {void}
             */
            function reportError(node, initial, result, type) {
                const name = initial.type === "Identifier" ? initial.name : initial.value;
                return context.report({
                    node,
                    message: "{{type}} {{name}} unnecessarily renamed.",
                    data: {
                        name,
                        type
                    },
                    fix(fixer) {
                        if (sourceCode.commentsExistBetween(initial, result)) {
                            return null;
                        }
                        const replacementText = result.type === "AssignmentPattern"
                            ? sourceCode.getText(result)
                            : name;
                        return fixer.replaceTextRange([
                            initial.range[0],
                            result.range[1]
                        ], replacementText);
                    }
                });
            }
            /**
             * Checks whether a destructured assignment is unnecessarily renamed
             * @param {ASTNode} node node to check
             * @returns {void}
             */
            function checkDestructured(node) {
                if (ignoreDestructuring) {
                    return;
                }
                for (const property of node.properties) {
                    /*
                     * TODO: Remove after babel-eslint removes ExperimentalRestProperty
                     * https://github.com/eslint/eslint/issues/12335
                     */
                    if (property.type === "ExperimentalRestProperty") {
                        continue;
                    }
                    /**
                     * Properties using shorthand syntax and rest elements can not be renamed.
                     * If the property is computed, we have no idea if a rename is useless or not.
                     */
                    if (property.shorthand || property.type === "RestElement" || property.computed) {
                        continue;
                    }
                    const key = (property.key.type === "Identifier" && property.key.name) || (property.key.type === "Literal" && property.key.value);
                    const renamedKey = property.value.type === "AssignmentPattern" ? property.value.left.name : property.value.name;
                    if (key === renamedKey) {
                        reportError(property, property.key, property.value, "Destructuring assignment");
                    }
                }
            }
            /**
             * Checks whether an import is unnecessarily renamed
             * @param {ASTNode} node node to check
             * @returns {void}
             */
            function checkImport(node) {
                if (ignoreImport) {
                    return;
                }
                if (node.imported.name === node.local.name &&
                    node.imported.range[0] !== node.local.range[0]) {
                    reportError(node, node.imported, node.local, "Import");
                }
            }
            /**
             * Checks whether an export is unnecessarily renamed
             * @param {ASTNode} node node to check
             * @returns {void}
             */
            function checkExport(node) {
                if (ignoreExport) {
                    return;
                }
                if (node.local.name === node.exported.name &&
                    node.local.range[0] !== node.exported.range[0]) {
                    reportError(node, node.local, node.exported, "Export");
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                ObjectPattern: checkDestructured,
                ImportSpecifier: checkImport,
                ExportSpecifier: checkExport
            };
        }
    };
},
/* eslint/lib/rules/no-useless-return.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */, FixTracker = require(237) /* ./utils/fix-tracker */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Removes the given element from the array.
     * @param {Array} array The source array to remove.
     * @param {any} element The target item to remove.
     * @returns {void}
     */
    function remove(array, element) {
        const index = array.indexOf(element);
        if (index !== -1) {
            array.splice(index, 1);
        }
    }
    /**
     * Checks whether it can remove the given return statement or not.
     * @param {ASTNode} node The return statement node to check.
     * @returns {boolean} `true` if the node is removeable.
     */
    function isRemovable(node) {
        return astUtils.STATEMENT_LIST_PARENTS.has(node.parent.type);
    }
    /**
     * Checks whether the given return statement is in a `finally` block or not.
     * @param {ASTNode} node The return statement node to check.
     * @returns {boolean} `true` if the node is in a `finally` block.
     */
    function isInFinally(node) {
        for (let currentNode = node; currentNode && currentNode.parent && !astUtils.isFunction(currentNode); currentNode = currentNode.parent) {
            if (currentNode.parent.type === "TryStatement" && currentNode.parent.finalizer === currentNode) {
                return true;
            }
        }
        return false;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow redundant return statements",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-useless-return"
            },
            fixable: "code",
            schema: []
        },
        create(context) {
            const segmentInfoMap = new WeakMap();
            const usedUnreachableSegments = new WeakSet();
            const sourceCode = context.getSourceCode();
            let scopeInfo = null;
            /**
             * Checks whether the given segment is terminated by a return statement or not.
             * @param {CodePathSegment} segment The segment to check.
             * @returns {boolean} `true` if the segment is terminated by a return statement, or if it's still a part of unreachable.
             */
            function isReturned(segment) {
                const info = segmentInfoMap.get(segment);
                return !info || info.returned;
            }
            /**
             * Collects useless return statements from the given previous segments.
             *
             * A previous segment may be an unreachable segment.
             * In that case, the information object of the unreachable segment is not
             * initialized because `onCodePathSegmentStart` event is not notified for
             * unreachable segments.
             * This goes to the previous segments of the unreachable segment recursively
             * if the unreachable segment was generated by a return statement. Otherwise,
             * this ignores the unreachable segment.
             *
             * This behavior would simulate code paths for the case that the return
             * statement does not exist.
             * @param {ASTNode[]} uselessReturns The collected return statements.
             * @param {CodePathSegment[]} prevSegments The previous segments to traverse.
             * @param {WeakSet<CodePathSegment>} [providedTraversedSegments] A set of segments that have already been traversed in this call
             * @returns {ASTNode[]} `uselessReturns`.
             */
            function getUselessReturns(uselessReturns, prevSegments, providedTraversedSegments) {
                const traversedSegments = providedTraversedSegments || new WeakSet();
                for (const segment of prevSegments) {
                    if (!segment.reachable) {
                        if (!traversedSegments.has(segment)) {
                            traversedSegments.add(segment);
                            getUselessReturns(uselessReturns, segment.allPrevSegments.filter(isReturned), traversedSegments);
                        }
                        continue;
                    }
                    uselessReturns.push(...segmentInfoMap.get(segment).uselessReturns);
                }
                return uselessReturns;
            }
            /**
             * Removes the return statements on the given segment from the useless return
             * statement list.
             *
             * This segment may be an unreachable segment.
             * In that case, the information object of the unreachable segment is not
             * initialized because `onCodePathSegmentStart` event is not notified for
             * unreachable segments.
             * This goes to the previous segments of the unreachable segment recursively
             * if the unreachable segment was generated by a return statement. Otherwise,
             * this ignores the unreachable segment.
             *
             * This behavior would simulate code paths for the case that the return
             * statement does not exist.
             * @param {CodePathSegment} segment The segment to get return statements.
             * @returns {void}
             */
            function markReturnStatementsOnSegmentAsUsed(segment) {
                if (!segment.reachable) {
                    usedUnreachableSegments.add(segment);
                    segment.allPrevSegments
                        .filter(isReturned)
                        .filter(prevSegment => !usedUnreachableSegments.has(prevSegment))
                        .forEach(markReturnStatementsOnSegmentAsUsed);
                    return;
                }
                const info = segmentInfoMap.get(segment);
                for (const node of info.uselessReturns) {
                    remove(scopeInfo.uselessReturns, node);
                }
                info.uselessReturns = [];
            }
            /**
             * Removes the return statements on the current segments from the useless
             * return statement list.
             *
             * This function will be called at every statement except FunctionDeclaration,
             * BlockStatement, and BreakStatement.
             *
             * - FunctionDeclarations are always executed whether it's returned or not.
             * - BlockStatements do nothing.
             * - BreakStatements go the next merely.
             * @returns {void}
             */
            function markReturnStatementsOnCurrentSegmentsAsUsed() {
                scopeInfo
                    .codePath
                    .currentSegments
                    .forEach(markReturnStatementsOnSegmentAsUsed);
            }
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                // Makes and pushs a new scope information.
                onCodePathStart(codePath) {
                    scopeInfo = {
                        upper: scopeInfo,
                        uselessReturns: [],
                        codePath
                    };
                },
                // Reports useless return statements if exist.
                onCodePathEnd() {
                    for (const node of scopeInfo.uselessReturns) {
                        context.report({
                            node,
                            loc: node.loc,
                            message: "Unnecessary return statement.",
                            fix(fixer) {
                                if (isRemovable(node) && !sourceCode.getCommentsInside(node).length) {
                                    /*
                                     * Extend the replacement range to include the
                                     * entire function to avoid conflicting with
                                     * no-else-return.
                                     * https://github.com/eslint/eslint/issues/8026
                                     */
                                    return new FixTracker(fixer, sourceCode)
                                        .retainEnclosingFunction(node)
                                        .remove(node);
                                }
                                return null;
                            }
                        });
                    }
                    scopeInfo = scopeInfo.upper;
                },
                /*
                 * Initializes segments.
                 * NOTE: This event is notified for only reachable segments.
                 */
                onCodePathSegmentStart(segment) {
                    const info = {
                        uselessReturns: getUselessReturns([], segment.allPrevSegments),
                        returned: false
                    };
                    // Stores the info.
                    segmentInfoMap.set(segment, info);
                },
                // Adds ReturnStatement node to check whether it's useless or not.
                ReturnStatement(node) {
                    if (node.argument) {
                        markReturnStatementsOnCurrentSegmentsAsUsed();
                    }
                    if (node.argument ||
                        astUtils.isInLoop(node) ||
                        isInFinally(node) ||
                        // Ignore `return` statements in unreachable places (https://github.com/eslint/eslint/issues/11647).
                        !scopeInfo.codePath.currentSegments.some(s => s.reachable)) {
                        return;
                    }
                    for (const segment of scopeInfo.codePath.currentSegments) {
                        const info = segmentInfoMap.get(segment);
                        if (info) {
                            info.uselessReturns.push(node);
                            info.returned = true;
                        }
                    }
                    scopeInfo.uselessReturns.push(node);
                },
                /*
                 * Registers for all statement nodes except FunctionDeclaration, BlockStatement, BreakStatement.
                 * Removes return statements of the current segments from the useless return statement list.
                 */
                ClassDeclaration: markReturnStatementsOnCurrentSegmentsAsUsed,
                ContinueStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                DebuggerStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                DoWhileStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                EmptyStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                ExpressionStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                ForInStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                ForOfStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                ForStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                IfStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                ImportDeclaration: markReturnStatementsOnCurrentSegmentsAsUsed,
                LabeledStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                SwitchStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                ThrowStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                TryStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                VariableDeclaration: markReturnStatementsOnCurrentSegmentsAsUsed,
                WhileStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                WithStatement: markReturnStatementsOnCurrentSegmentsAsUsed,
                ExportNamedDeclaration: markReturnStatementsOnCurrentSegmentsAsUsed,
                ExportDefaultDeclaration: markReturnStatementsOnCurrentSegmentsAsUsed,
                ExportAllDeclaration: markReturnStatementsOnCurrentSegmentsAsUsed
            };
        }
    };
},
/* eslint/lib/rules/no-var.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Check whether a given variable is a global variable or not.
     * @param {eslint-scope.Variable} variable The variable to check.
     * @returns {boolean} `true` if the variable is a global variable.
     */
    function isGlobal(variable) {
        return Boolean(variable.scope) && variable.scope.type === "global";
    }
    /**
     * Finds the nearest function scope or global scope walking up the scope
     * hierarchy.
     * @param {eslint-scope.Scope} scope The scope to traverse.
     * @returns {eslint-scope.Scope} a function scope or global scope containing the given
     *      scope.
     */
    function getEnclosingFunctionScope(scope) {
        let currentScope = scope;
        while (currentScope.type !== "function" && currentScope.type !== "global") {
            currentScope = currentScope.upper;
        }
        return currentScope;
    }
    /**
     * Checks whether the given variable has any references from a more specific
     * function expression (i.e. a closure).
     * @param {eslint-scope.Variable} variable A variable to check.
     * @returns {boolean} `true` if the variable is used from a closure.
     */
    function isReferencedInClosure(variable) {
        const enclosingFunctionScope = getEnclosingFunctionScope(variable.scope);
        return variable.references.some(reference => getEnclosingFunctionScope(reference.from) !== enclosingFunctionScope);
    }
    /**
     * Checks whether the given node is the assignee of a loop.
     * @param {ASTNode} node A VariableDeclaration node to check.
     * @returns {boolean} `true` if the declaration is assigned as part of loop
     *      iteration.
     */
    function isLoopAssignee(node) {
        return (node.parent.type === "ForOfStatement" || node.parent.type === "ForInStatement") &&
            node === node.parent.left;
    }
    /**
     * Checks whether the given variable declaration is immediately initialized.
     * @param {ASTNode} node A VariableDeclaration node to check.
     * @returns {boolean} `true` if the declaration has an initializer.
     */
    function isDeclarationInitialized(node) {
        return node.declarations.every(declarator => declarator.init !== null);
    }
    const SCOPE_NODE_TYPE = /^(?:Program|BlockStatement|SwitchStatement|ForStatement|ForInStatement|ForOfStatement)$/u;
    /**
     * Gets the scope node which directly contains a given node.
     * @param {ASTNode} node A node to get. This is a `VariableDeclaration` or
     *      an `Identifier`.
     * @returns {ASTNode} A scope node. This is one of `Program`, `BlockStatement`,
     *      `SwitchStatement`, `ForStatement`, `ForInStatement`, and
     *      `ForOfStatement`.
     */
    function getScopeNode(node) {
        for (let currentNode = node; currentNode; currentNode = currentNode.parent) {
            if (SCOPE_NODE_TYPE.test(currentNode.type)) {
                return currentNode;
            }
        }
        /* istanbul ignore next : unreachable */
        return null;
    }
    /**
     * Checks whether a given variable is redeclared or not.
     * @param {eslint-scope.Variable} variable A variable to check.
     * @returns {boolean} `true` if the variable is redeclared.
     */
    function isRedeclared(variable) {
        return variable.defs.length >= 2;
    }
    /**
     * Checks whether a given variable is used from outside of the specified scope.
     * @param {ASTNode} scopeNode A scope node to check.
     * @returns {Function} The predicate function which checks whether a given
     *      variable is used from outside of the specified scope.
     */
    function isUsedFromOutsideOf(scopeNode) {
        /**
         * Checks whether a given reference is inside of the specified scope or not.
         * @param {eslint-scope.Reference} reference A reference to check.
         * @returns {boolean} `true` if the reference is inside of the specified
         *      scope.
         */
        function isOutsideOfScope(reference) {
            const scope = scopeNode.range;
            const id = reference.identifier.range;
            return id[0] < scope[0] || id[1] > scope[1];
        }
        return function (variable) {
            return variable.references.some(isOutsideOfScope);
        };
    }
    /**
     * Creates the predicate function which checks whether a variable has their references in TDZ.
     *
     * The predicate function would return `true`:
     *
     * - if a reference is before the declarator. E.g. (var a = b, b = 1;)(var {a = b, b} = {};)
     * - if a reference is in the expression of their default value.  E.g. (var {a = a} = {};)
     * - if a reference is in the expression of their initializer.  E.g. (var a = a;)
     * @param {ASTNode} node The initializer node of VariableDeclarator.
     * @returns {Function} The predicate function.
     * @private
     */
    function hasReferenceInTDZ(node) {
        const initStart = node.range[0];
        const initEnd = node.range[1];
        return variable => {
            const id = variable.defs[0].name;
            const idStart = id.range[0];
            const defaultValue = (id.parent.type === "AssignmentPattern" ? id.parent.right : null);
            const defaultStart = defaultValue && defaultValue.range[0];
            const defaultEnd = defaultValue && defaultValue.range[1];
            return variable.references.some(reference => {
                const start = reference.identifier.range[0];
                const end = reference.identifier.range[1];
                return !reference.init && (start < idStart ||
                    (defaultValue !== null && start >= defaultStart && end <= defaultEnd) ||
                    (start >= initStart && end <= initEnd));
            });
        };
    }
    /**
     * Checks whether a given variable has name that is allowed for 'var' declarations,
     * but disallowed for `let` declarations.
     * @param {eslint-scope.Variable} variable The variable to check.
     * @returns {boolean} `true` if the variable has a disallowed name.
     */
    function hasNameDisallowedForLetDeclarations(variable) {
        return variable.name === "let";
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require `let` or `const` instead of `var`",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-var"
            },
            schema: [],
            fixable: "code"
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Checks whether the variables which are defined by the given declarator node have their references in TDZ.
             * @param {ASTNode} declarator The VariableDeclarator node to check.
             * @returns {boolean} `true` if one of the variables which are defined by the given declarator node have their references in TDZ.
             */
            function hasSelfReferenceInTDZ(declarator) {
                if (!declarator.init) {
                    return false;
                }
                const variables = context.getDeclaredVariables(declarator);
                return variables.some(hasReferenceInTDZ(declarator.init));
            }
            /**
             * Checks whether it can fix a given variable declaration or not.
             * It cannot fix if the following cases:
             *
             * - A variable is a global variable.
             * - A variable is declared on a SwitchCase node.
             * - A variable is redeclared.
             * - A variable is used from outside the scope.
             * - A variable is used from a closure within a loop.
             * - A variable might be used before it is assigned within a loop.
             * - A variable might be used in TDZ.
             * - A variable is declared in statement position (e.g. a single-line `IfStatement`)
             * - A variable has name that is disallowed for `let` declarations.
             *
             * ## A variable is declared on a SwitchCase node.
             *
             * If this rule modifies 'var' declarations on a SwitchCase node, it
             * would generate the warnings of 'no-case-declarations' rule. And the
             * 'eslint:recommended' preset includes 'no-case-declarations' rule, so
             * this rule doesn't modify those declarations.
             *
             * ## A variable is redeclared.
             *
             * The language spec disallows redeclarations of `let` declarations.
             * Those variables would cause syntax errors.
             *
             * ## A variable is used from outside the scope.
             *
             * The language spec disallows accesses from outside of the scope for
             * `let` declarations. Those variables would cause reference errors.
             *
             * ## A variable is used from a closure within a loop.
             *
             * A `var` declaration within a loop shares the same variable instance
             * across all loop iterations, while a `let` declaration creates a new
             * instance for each iteration. This means if a variable in a loop is
             * referenced by any closure, changing it from `var` to `let` would
             * change the behavior in a way that is generally unsafe.
             *
             * ## A variable might be used before it is assigned within a loop.
             *
             * Within a loop, a `let` declaration without an initializer will be
             * initialized to null, while a `var` declaration will retain its value
             * from the previous iteration, so it is only safe to change `var` to
             * `let` if we can statically determine that the variable is always
             * assigned a value before its first access in the loop body. To keep
             * the implementation simple, we only convert `var` to `let` within
             * loops when the variable is a loop assignee or the declaration has an
             * initializer.
             * @param {ASTNode} node A variable declaration node to check.
             * @returns {boolean} `true` if it can fix the node.
             */
            function canFix(node) {
                const variables = context.getDeclaredVariables(node);
                const scopeNode = getScopeNode(node);
                if (node.parent.type === "SwitchCase" ||
                    node.declarations.some(hasSelfReferenceInTDZ) ||
                    variables.some(isGlobal) ||
                    variables.some(isRedeclared) ||
                    variables.some(isUsedFromOutsideOf(scopeNode)) ||
                    variables.some(hasNameDisallowedForLetDeclarations)) {
                    return false;
                }
                if (astUtils.isInLoop(node)) {
                    if (variables.some(isReferencedInClosure)) {
                        return false;
                    }
                    if (!isLoopAssignee(node) && !isDeclarationInitialized(node)) {
                        return false;
                    }
                }
                if (!isLoopAssignee(node) &&
                    !(node.parent.type === "ForStatement" && node.parent.init === node) &&
                    !astUtils.STATEMENT_LIST_PARENTS.has(node.parent.type)) {
                    // If the declaration is not in a block, e.g. `if (foo) var bar = 1;`, then it can't be fixed.
                    return false;
                }
                return true;
            }
            /**
             * Reports a given variable declaration node.
             * @param {ASTNode} node A variable declaration node to report.
             * @returns {void}
             */
            function report(node) {
                context.report({
                    node,
                    message: "Unexpected var, use let or const instead.",
                    fix(fixer) {
                        const varToken = sourceCode.getFirstToken(node, { filter: t => t.value === "var" });
                        return canFix(node)
                            ? fixer.replaceText(varToken, "let")
                            : null;
                    }
                });
            }
            return {
                "VariableDeclaration:exit"(node) {
                    if (node.kind === "var") {
                        report(node);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-void.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `void` operators",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-void"
            },
            schema: []
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                UnaryExpression(node) {
                    if (node.operator === "void") {
                        context.report({ node, message: "Expected 'undefined' and instead saw 'void'." });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-warning-comments.js */ function _(require, module, exports) {
    const { escapeRegExp } = require(111) /* lodash */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow specified warning terms in comments",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-warning-comments"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        terms: {
                            type: "array",
                            items: {
                                type: "string"
                            }
                        },
                        location: {
                            enum: ["start", "anywhere"]
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode(), configuration = context.options[0] || {}, warningTerms = configuration.terms || ["todo", "fixme", "xxx"], location = configuration.location || "start", selfConfigRegEx = /\bno-warning-comments\b/u;
            /**
             * Convert a warning term into a RegExp which will match a comment containing that whole word in the specified
             * location ("start" or "anywhere"). If the term starts or ends with non word characters, then the match will not
             * require word boundaries on that side.
             * @param {string} term A term to convert to a RegExp
             * @returns {RegExp} The term converted to a RegExp
             */
            function convertToRegExp(term) {
                const escaped = escapeRegExp(term);
                const wordBoundary = "\\b";
                const eitherOrWordBoundary = `|${wordBoundary}`;
                let prefix;
                /*
                 * If the term ends in a word character (a-z0-9_), ensure a word
                 * boundary at the end, so that substrings do not get falsely
                 * matched. eg "todo" in a string such as "mastodon".
                 * If the term ends in a non-word character, then \b won't match on
                 * the boundary to the next non-word character, which would likely
                 * be a space. For example `/\bFIX!\b/.test('FIX! blah') === false`.
                 * In these cases, use no bounding match. Same applies for the
                 * prefix, handled below.
                 */
                const suffix = /\w$/u.test(term) ? "\\b" : "";
                if (location === "start") {
                    /*
                     * When matching at the start, ignore leading whitespace, and
                     * there's no need to worry about word boundaries.
                     */
                    prefix = "^\\s*";
                }
                else if (/^\w/u.test(term)) {
                    prefix = wordBoundary;
                }
                else {
                    prefix = "";
                }
                if (location === "start") {
                    /*
                     * For location "start" the regex should be
                     * ^\s*TERM\b.  This checks the word boundary
                     * at the beginning of the comment.
                     */
                    return new RegExp(prefix + escaped + suffix, "iu");
                }
                /*
                 * For location "anywhere" the regex should be
                 * \bTERM\b|\bTERM\b, this checks the entire comment
                 * for the term.
                 */
                return new RegExp(prefix + escaped + suffix + eitherOrWordBoundary + term + wordBoundary, "iu");
            }
            const warningRegExps = warningTerms.map(convertToRegExp);
            /**
             * Checks the specified comment for matches of the configured warning terms and returns the matches.
             * @param {string} comment The comment which is checked.
             * @returns {Array} All matched warning terms for this comment.
             */
            function commentContainsWarningTerm(comment) {
                const matches = [];
                warningRegExps.forEach((regex, index) => {
                    if (regex.test(comment)) {
                        matches.push(warningTerms[index]);
                    }
                });
                return matches;
            }
            /**
             * Checks the specified node for matching warning comments and reports them.
             * @param {ASTNode} node The AST node being checked.
             * @returns {void} undefined.
             */
            function checkComment(node) {
                if (astUtils.isDirectiveComment(node) && selfConfigRegEx.test(node.value)) {
                    return;
                }
                const matches = commentContainsWarningTerm(node.value);
                matches.forEach(matchedTerm => {
                    context.report({
                        node,
                        message: "Unexpected '{{matchedTerm}}' comment.",
                        data: {
                            matchedTerm
                        }
                    });
                });
            }
            return {
                Program() {
                    const comments = sourceCode.getAllComments();
                    comments.filter(token => token.type !== "Shebang").forEach(checkComment);
                }
            };
        }
    };
},
/* eslint/lib/rules/no-whitespace-before-property.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "disallow whitespace before properties",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/no-whitespace-before-property"
            },
            fixable: "whitespace",
            schema: []
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Reports whitespace before property token
             * @param {ASTNode} node the node to report in the event of an error
             * @param {Token} leftToken the left token
             * @param {Token} rightToken the right token
             * @returns {void}
             * @private
             */
            function reportError(node, leftToken, rightToken) {
                const replacementText = node.computed ? "" : ".";
                context.report({
                    node,
                    message: "Unexpected whitespace before property {{propName}}.",
                    data: {
                        propName: sourceCode.getText(node.property)
                    },
                    fix(fixer) {
                        if (!node.computed && astUtils.isDecimalInteger(node.object)) {
                            /*
                             * If the object is a number literal, fixing it to something like 5.toString() would cause a SyntaxError.
                             * Don't fix this case.
                             */
                            return null;
                        }
                        return fixer.replaceTextRange([leftToken.range[1], rightToken.range[0]], replacementText);
                    }
                });
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                MemberExpression(node) {
                    let rightToken;
                    let leftToken;
                    if (!astUtils.isTokenOnSameLine(node.object, node.property)) {
                        return;
                    }
                    if (node.computed) {
                        rightToken = sourceCode.getTokenBefore(node.property, astUtils.isOpeningBracketToken);
                        leftToken = sourceCode.getTokenBefore(rightToken);
                    }
                    else {
                        rightToken = sourceCode.getFirstToken(node.property);
                        leftToken = sourceCode.getTokenBefore(rightToken, 1);
                    }
                    if (sourceCode.isSpaceBetweenTokens(leftToken, rightToken)) {
                        reportError(node, leftToken, rightToken);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/no-with.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `with` statements",
                category: "Best Practices",
                recommended: true,
                url: "https://eslint.org/docs/rules/no-with"
            },
            schema: []
        },
        create(context) {
            return {
                WithStatement(node) {
                    context.report({ node, message: "Unexpected use of 'with' statement." });
                }
            };
        }
    };
},
/* eslint/lib/rules/nonblock-statement-body-position.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const POSITION_SCHEMA = { enum: ["beside", "below", "any"] };
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce the location of single-line statements",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/nonblock-statement-body-position"
            },
            fixable: "whitespace",
            schema: [
                POSITION_SCHEMA,
                {
                    properties: {
                        overrides: {
                            properties: {
                                if: POSITION_SCHEMA,
                                else: POSITION_SCHEMA,
                                while: POSITION_SCHEMA,
                                do: POSITION_SCHEMA,
                                for: POSITION_SCHEMA
                            },
                            additionalProperties: false
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            //----------------------------------------------------------------------
            // Helpers
            //----------------------------------------------------------------------
            /**
             * Gets the applicable preference for a particular keyword
             * @param {string} keywordName The name of a keyword, e.g. 'if'
             * @returns {string} The applicable option for the keyword, e.g. 'beside'
             */
            function getOption(keywordName) {
                return context.options[1] && context.options[1].overrides && context.options[1].overrides[keywordName] ||
                    context.options[0] ||
                    "beside";
            }
            /**
             * Validates the location of a single-line statement
             * @param {ASTNode} node The single-line statement
             * @param {string} keywordName The applicable keyword name for the single-line statement
             * @returns {void}
             */
            function validateStatement(node, keywordName) {
                const option = getOption(keywordName);
                if (node.type === "BlockStatement" || option === "any") {
                    return;
                }
                const tokenBefore = sourceCode.getTokenBefore(node);
                if (tokenBefore.loc.end.line === node.loc.start.line && option === "below") {
                    context.report({
                        node,
                        message: "Expected a linebreak before this statement.",
                        fix: fixer => fixer.insertTextBefore(node, "\n")
                    });
                }
                else if (tokenBefore.loc.end.line !== node.loc.start.line && option === "beside") {
                    context.report({
                        node,
                        message: "Expected no linebreak before this statement.",
                        fix(fixer) {
                            if (sourceCode.getText().slice(tokenBefore.range[1], node.range[0]).trim()) {
                                return null;
                            }
                            return fixer.replaceTextRange([tokenBefore.range[1], node.range[0]], " ");
                        }
                    });
                }
            }
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                IfStatement(node) {
                    validateStatement(node.consequent, "if");
                    // Check the `else` node, but don't check 'else if' statements.
                    if (node.alternate && node.alternate.type !== "IfStatement") {
                        validateStatement(node.alternate, "else");
                    }
                },
                WhileStatement: node => validateStatement(node.body, "while"),
                DoWhileStatement: node => validateStatement(node.body, "do"),
                ForStatement: node => validateStatement(node.body, "for"),
                ForInStatement: node => validateStatement(node.body, "for"),
                ForOfStatement: node => validateStatement(node.body, "for")
            };
        }
    };
},
/* eslint/lib/rules/object-curly-newline.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    const lodash = require(111) /* lodash */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    // Schema objects.
    const OPTION_VALUE = {
        oneOf: [
            {
                enum: ["always", "never"]
            },
            {
                type: "object",
                properties: {
                    multiline: {
                        type: "boolean"
                    },
                    minProperties: {
                        type: "integer",
                        minimum: 0
                    },
                    consistent: {
                        type: "boolean"
                    }
                },
                additionalProperties: false,
                minProperties: 1
            }
        ]
    };
    /**
     * Normalizes a given option value.
     * @param {string|Object|undefined} value An option value to parse.
     * @returns {{multiline: boolean, minProperties: number, consistent: boolean}} Normalized option object.
     */
    function normalizeOptionValue(value) {
        let multiline = false;
        let minProperties = Number.POSITIVE_INFINITY;
        let consistent = false;
        if (value) {
            if (value === "always") {
                minProperties = 0;
            }
            else if (value === "never") {
                minProperties = Number.POSITIVE_INFINITY;
            }
            else {
                multiline = Boolean(value.multiline);
                minProperties = value.minProperties || Number.POSITIVE_INFINITY;
                consistent = Boolean(value.consistent);
            }
        }
        else {
            consistent = true;
        }
        return { multiline, minProperties, consistent };
    }
    /**
     * Normalizes a given option value.
     * @param {string|Object|undefined} options An option value to parse.
     * @returns {{
     *   ObjectExpression: {multiline: boolean, minProperties: number, consistent: boolean},
     *   ObjectPattern: {multiline: boolean, minProperties: number, consistent: boolean},
     *   ImportDeclaration: {multiline: boolean, minProperties: number, consistent: boolean},
     *   ExportNamedDeclaration : {multiline: boolean, minProperties: number, consistent: boolean}
     * }} Normalized option object.
     */
    function normalizeOptions(options) {
        const isNodeSpecificOption = lodash.overSome([lodash.isPlainObject, lodash.isString]);
        if (lodash.isPlainObject(options) && lodash.some(options, isNodeSpecificOption)) {
            return {
                ObjectExpression: normalizeOptionValue(options.ObjectExpression),
                ObjectPattern: normalizeOptionValue(options.ObjectPattern),
                ImportDeclaration: normalizeOptionValue(options.ImportDeclaration),
                ExportNamedDeclaration: normalizeOptionValue(options.ExportDeclaration)
            };
        }
        const value = normalizeOptionValue(options);
        return { ObjectExpression: value, ObjectPattern: value, ImportDeclaration: value, ExportNamedDeclaration: value };
    }
    /**
     * Determines if ObjectExpression, ObjectPattern, ImportDeclaration or ExportNamedDeclaration
     * node needs to be checked for missing line breaks
     * @param {ASTNode} node Node under inspection
     * @param {Object} options option specific to node type
     * @param {Token} first First object property
     * @param {Token} last Last object property
     * @returns {boolean} `true` if node needs to be checked for missing line breaks
     */
    function areLineBreaksRequired(node, options, first, last) {
        let objectProperties;
        if (node.type === "ObjectExpression" || node.type === "ObjectPattern") {
            objectProperties = node.properties;
        }
        else {
            // is ImportDeclaration or ExportNamedDeclaration
            objectProperties = node.specifiers
                .filter(s => s.type === "ImportSpecifier" || s.type === "ExportSpecifier");
        }
        return objectProperties.length >= options.minProperties ||
            (options.multiline &&
                objectProperties.length > 0 &&
                first.loc.start.line !== last.loc.end.line);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent line breaks inside braces",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/object-curly-newline"
            },
            fixable: "whitespace",
            schema: [
                {
                    oneOf: [
                        OPTION_VALUE,
                        {
                            type: "object",
                            properties: {
                                ObjectExpression: OPTION_VALUE,
                                ObjectPattern: OPTION_VALUE,
                                ImportDeclaration: OPTION_VALUE,
                                ExportDeclaration: OPTION_VALUE
                            },
                            additionalProperties: false,
                            minProperties: 1
                        }
                    ]
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const normalizedOptions = normalizeOptions(context.options[0]);
            /**
             * Reports a given node if it violated this rule.
             * @param {ASTNode} node A node to check. This is an ObjectExpression, ObjectPattern, ImportDeclaration or ExportNamedDeclaration node.
             * @returns {void}
             */
            function check(node) {
                const options = normalizedOptions[node.type];
                if ((node.type === "ImportDeclaration" &&
                    !node.specifiers.some(specifier => specifier.type === "ImportSpecifier")) ||
                    (node.type === "ExportNamedDeclaration" &&
                        !node.specifiers.some(specifier => specifier.type === "ExportSpecifier"))) {
                    return;
                }
                const openBrace = sourceCode.getFirstToken(node, token => token.value === "{");
                let closeBrace;
                if (node.typeAnnotation) {
                    closeBrace = sourceCode.getTokenBefore(node.typeAnnotation);
                }
                else {
                    closeBrace = sourceCode.getLastToken(node, token => token.value === "}");
                }
                let first = sourceCode.getTokenAfter(openBrace, { includeComments: true });
                let last = sourceCode.getTokenBefore(closeBrace, { includeComments: true });
                const needsLineBreaks = areLineBreaksRequired(node, options, first, last);
                const hasCommentsFirstToken = astUtils.isCommentToken(first);
                const hasCommentsLastToken = astUtils.isCommentToken(last);
                /*
                 * Use tokens or comments to check multiline or not.
                 * But use only tokens to check whether line breaks are needed.
                 * This allows:
                 *     var obj = { // eslint-disable-line foo
                 *         a: 1
                 *     }
                 */
                first = sourceCode.getTokenAfter(openBrace);
                last = sourceCode.getTokenBefore(closeBrace);
                if (needsLineBreaks) {
                    if (astUtils.isTokenOnSameLine(openBrace, first)) {
                        context.report({
                            message: "Expected a line break after this opening brace.",
                            node,
                            loc: openBrace.loc.start,
                            fix(fixer) {
                                if (hasCommentsFirstToken) {
                                    return null;
                                }
                                return fixer.insertTextAfter(openBrace, "\n");
                            }
                        });
                    }
                    if (astUtils.isTokenOnSameLine(last, closeBrace)) {
                        context.report({
                            message: "Expected a line break before this closing brace.",
                            node,
                            loc: closeBrace.loc.start,
                            fix(fixer) {
                                if (hasCommentsLastToken) {
                                    return null;
                                }
                                return fixer.insertTextBefore(closeBrace, "\n");
                            }
                        });
                    }
                }
                else {
                    const consistent = options.consistent;
                    const hasLineBreakBetweenOpenBraceAndFirst = !astUtils.isTokenOnSameLine(openBrace, first);
                    const hasLineBreakBetweenCloseBraceAndLast = !astUtils.isTokenOnSameLine(last, closeBrace);
                    if ((!consistent && hasLineBreakBetweenOpenBraceAndFirst) ||
                        (consistent && hasLineBreakBetweenOpenBraceAndFirst && !hasLineBreakBetweenCloseBraceAndLast)) {
                        context.report({
                            message: "Unexpected line break after this opening brace.",
                            node,
                            loc: openBrace.loc.start,
                            fix(fixer) {
                                if (hasCommentsFirstToken) {
                                    return null;
                                }
                                return fixer.removeRange([
                                    openBrace.range[1],
                                    first.range[0]
                                ]);
                            }
                        });
                    }
                    if ((!consistent && hasLineBreakBetweenCloseBraceAndLast) ||
                        (consistent && !hasLineBreakBetweenOpenBraceAndFirst && hasLineBreakBetweenCloseBraceAndLast)) {
                        context.report({
                            message: "Unexpected line break before this closing brace.",
                            node,
                            loc: closeBrace.loc.start,
                            fix(fixer) {
                                if (hasCommentsLastToken) {
                                    return null;
                                }
                                return fixer.removeRange([
                                    last.range[1],
                                    closeBrace.range[0]
                                ]);
                            }
                        });
                    }
                }
            }
            return {
                ObjectExpression: check,
                ObjectPattern: check,
                ImportDeclaration: check,
                ExportNamedDeclaration: check
            };
        }
    };
},
/* eslint/lib/rules/object-curly-spacing.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing inside braces",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/object-curly-spacing"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["always", "never"]
                },
                {
                    type: "object",
                    properties: {
                        arraysInObjects: {
                            type: "boolean"
                        },
                        objectsInObjects: {
                            type: "boolean"
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const spaced = context.options[0] === "always", sourceCode = context.getSourceCode();
            /**
             * Determines whether an option is set, relative to the spacing option.
             * If spaced is "always", then check whether option is set to false.
             * If spaced is "never", then check whether option is set to true.
             * @param {Object} option The option to exclude.
             * @returns {boolean} Whether or not the property is excluded.
             */
            function isOptionSet(option) {
                return context.options[1] ? context.options[1][option] === !spaced : false;
            }
            const options = {
                spaced,
                arraysInObjectsException: isOptionSet("arraysInObjects"),
                objectsInObjectsException: isOptionSet("objectsInObjects")
            };
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Reports that there shouldn't be a space after the first token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportNoBeginningSpace(node, token) {
                const nextToken = context.getSourceCode().getTokenAfter(token, { includeComments: true });
                context.report({
                    node,
                    loc: { start: token.loc.end, end: nextToken.loc.start },
                    message: "There should be no space after '{{token}}'.",
                    data: {
                        token: token.value
                    },
                    fix(fixer) {
                        return fixer.removeRange([token.range[1], nextToken.range[0]]);
                    }
                });
            }
            /**
             * Reports that there shouldn't be a space before the last token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportNoEndingSpace(node, token) {
                const previousToken = context.getSourceCode().getTokenBefore(token, { includeComments: true });
                context.report({
                    node,
                    loc: { start: previousToken.loc.end, end: token.loc.start },
                    message: "There should be no space before '{{token}}'.",
                    data: {
                        token: token.value
                    },
                    fix(fixer) {
                        return fixer.removeRange([previousToken.range[1], token.range[0]]);
                    }
                });
            }
            /**
             * Reports that there should be a space after the first token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportRequiredBeginningSpace(node, token) {
                context.report({
                    node,
                    loc: token.loc,
                    message: "A space is required after '{{token}}'.",
                    data: {
                        token: token.value
                    },
                    fix(fixer) {
                        return fixer.insertTextAfter(token, " ");
                    }
                });
            }
            /**
             * Reports that there should be a space before the last token
             * @param {ASTNode} node The node to report in the event of an error.
             * @param {Token} token The token to use for the report.
             * @returns {void}
             */
            function reportRequiredEndingSpace(node, token) {
                context.report({
                    node,
                    loc: token.loc,
                    message: "A space is required before '{{token}}'.",
                    data: {
                        token: token.value
                    },
                    fix(fixer) {
                        return fixer.insertTextBefore(token, " ");
                    }
                });
            }
            /**
             * Determines if spacing in curly braces is valid.
             * @param {ASTNode} node The AST node to check.
             * @param {Token} first The first token to check (should be the opening brace)
             * @param {Token} second The second token to check (should be first after the opening brace)
             * @param {Token} penultimate The penultimate token to check (should be last before closing brace)
             * @param {Token} last The last token to check (should be closing brace)
             * @returns {void}
             */
            function validateBraceSpacing(node, first, second, penultimate, last) {
                if (astUtils.isTokenOnSameLine(first, second)) {
                    const firstSpaced = sourceCode.isSpaceBetweenTokens(first, second);
                    if (options.spaced && !firstSpaced) {
                        reportRequiredBeginningSpace(node, first);
                    }
                    if (!options.spaced && firstSpaced && second.type !== "Line") {
                        reportNoBeginningSpace(node, first);
                    }
                }
                if (astUtils.isTokenOnSameLine(penultimate, last)) {
                    const shouldCheckPenultimate = (options.arraysInObjectsException && astUtils.isClosingBracketToken(penultimate) ||
                        options.objectsInObjectsException && astUtils.isClosingBraceToken(penultimate));
                    const penultimateType = shouldCheckPenultimate && sourceCode.getNodeByRangeIndex(penultimate.range[0]).type;
                    const closingCurlyBraceMustBeSpaced = (options.arraysInObjectsException && penultimateType === "ArrayExpression" ||
                        options.objectsInObjectsException && (penultimateType === "ObjectExpression" || penultimateType === "ObjectPattern")) ? !options.spaced : options.spaced;
                    const lastSpaced = sourceCode.isSpaceBetweenTokens(penultimate, last);
                    if (closingCurlyBraceMustBeSpaced && !lastSpaced) {
                        reportRequiredEndingSpace(node, last);
                    }
                    if (!closingCurlyBraceMustBeSpaced && lastSpaced) {
                        reportNoEndingSpace(node, last);
                    }
                }
            }
            /**
             * Gets '}' token of an object node.
             *
             * Because the last token of object patterns might be a type annotation,
             * this traverses tokens preceded by the last property, then returns the
             * first '}' token.
             * @param {ASTNode} node The node to get. This node is an
             *      ObjectExpression or an ObjectPattern. And this node has one or
             *      more properties.
             * @returns {Token} '}' token.
             */
            function getClosingBraceOfObject(node) {
                const lastProperty = node.properties[node.properties.length - 1];
                return sourceCode.getTokenAfter(lastProperty, astUtils.isClosingBraceToken);
            }
            /**
             * Reports a given object node if spacing in curly braces is invalid.
             * @param {ASTNode} node An ObjectExpression or ObjectPattern node to check.
             * @returns {void}
             */
            function checkForObject(node) {
                if (node.properties.length === 0) {
                    return;
                }
                const first = sourceCode.getFirstToken(node), last = getClosingBraceOfObject(node), second = sourceCode.getTokenAfter(first, { includeComments: true }), penultimate = sourceCode.getTokenBefore(last, { includeComments: true });
                validateBraceSpacing(node, first, second, penultimate, last);
            }
            /**
             * Reports a given import node if spacing in curly braces is invalid.
             * @param {ASTNode} node An ImportDeclaration node to check.
             * @returns {void}
             */
            function checkForImport(node) {
                if (node.specifiers.length === 0) {
                    return;
                }
                let firstSpecifier = node.specifiers[0];
                const lastSpecifier = node.specifiers[node.specifiers.length - 1];
                if (lastSpecifier.type !== "ImportSpecifier") {
                    return;
                }
                if (firstSpecifier.type !== "ImportSpecifier") {
                    firstSpecifier = node.specifiers[1];
                }
                const first = sourceCode.getTokenBefore(firstSpecifier), last = sourceCode.getTokenAfter(lastSpecifier, astUtils.isNotCommaToken), second = sourceCode.getTokenAfter(first, { includeComments: true }), penultimate = sourceCode.getTokenBefore(last, { includeComments: true });
                validateBraceSpacing(node, first, second, penultimate, last);
            }
            /**
             * Reports a given export node if spacing in curly braces is invalid.
             * @param {ASTNode} node An ExportNamedDeclaration node to check.
             * @returns {void}
             */
            function checkForExport(node) {
                if (node.specifiers.length === 0) {
                    return;
                }
                const firstSpecifier = node.specifiers[0], lastSpecifier = node.specifiers[node.specifiers.length - 1], first = sourceCode.getTokenBefore(firstSpecifier), last = sourceCode.getTokenAfter(lastSpecifier, astUtils.isNotCommaToken), second = sourceCode.getTokenAfter(first, { includeComments: true }), penultimate = sourceCode.getTokenBefore(last, { includeComments: true });
                validateBraceSpacing(node, first, second, penultimate, last);
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                // var {x} = y;
                ObjectPattern: checkForObject,
                // var y = {x: 'y'}
                ObjectExpression: checkForObject,
                // import {y} from 'x';
                ImportDeclaration: checkForImport,
                // export {name} from 'yo';
                ExportNamedDeclaration: checkForExport
            };
        }
    };
},
/* eslint/lib/rules/object-property-newline.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce placing object properties on separate lines",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/object-property-newline"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allowAllPropertiesOnSameLine: {
                            type: "boolean",
                            default: false
                        },
                        allowMultiplePropertiesPerLine: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "whitespace"
        },
        create(context) {
            const allowSameLine = context.options[0] && ((context.options[0].allowAllPropertiesOnSameLine || context.options[0].allowMultiplePropertiesPerLine /* Deprecated */));
            const errorMessage = allowSameLine
                ? "Object properties must go on a new line if they aren't all on the same line."
                : "Object properties must go on a new line.";
            const sourceCode = context.getSourceCode();
            return {
                ObjectExpression(node) {
                    if (allowSameLine) {
                        if (node.properties.length > 1) {
                            const firstTokenOfFirstProperty = sourceCode.getFirstToken(node.properties[0]);
                            const lastTokenOfLastProperty = sourceCode.getLastToken(node.properties[node.properties.length - 1]);
                            if (firstTokenOfFirstProperty.loc.end.line === lastTokenOfLastProperty.loc.start.line) {
                                // All keys and values are on the same line
                                return;
                            }
                        }
                    }
                    for (let i = 1; i < node.properties.length; i++) {
                        const lastTokenOfPreviousProperty = sourceCode.getLastToken(node.properties[i - 1]);
                        const firstTokenOfCurrentProperty = sourceCode.getFirstToken(node.properties[i]);
                        if (lastTokenOfPreviousProperty.loc.end.line === firstTokenOfCurrentProperty.loc.start.line) {
                            context.report({
                                node,
                                loc: firstTokenOfCurrentProperty.loc.start,
                                message: errorMessage,
                                fix(fixer) {
                                    const comma = sourceCode.getTokenBefore(firstTokenOfCurrentProperty);
                                    const rangeAfterComma = [comma.range[1], firstTokenOfCurrentProperty.range[0]];
                                    // Don't perform a fix if there are any comments between the comma and the next property.
                                    if (sourceCode.text.slice(rangeAfterComma[0], rangeAfterComma[1]).trim()) {
                                        return null;
                                    }
                                    return fixer.replaceTextRange(rangeAfterComma, "\n");
                                }
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/object-shorthand.js */ function _(require, module, exports) {
    const OPTIONS = {
        always: "always",
        never: "never",
        methods: "methods",
        properties: "properties",
        consistent: "consistent",
        consistentAsNeeded: "consistent-as-needed"
    };
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require or disallow method and property shorthand syntax for object literals",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/object-shorthand"
            },
            fixable: "code",
            schema: {
                anyOf: [
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["always", "methods", "properties", "never", "consistent", "consistent-as-needed"]
                            }
                        ],
                        minItems: 0,
                        maxItems: 1
                    },
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["always", "methods", "properties"]
                            },
                            {
                                type: "object",
                                properties: {
                                    avoidQuotes: {
                                        type: "boolean"
                                    }
                                },
                                additionalProperties: false
                            }
                        ],
                        minItems: 0,
                        maxItems: 2
                    },
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["always", "methods"]
                            },
                            {
                                type: "object",
                                properties: {
                                    ignoreConstructors: {
                                        type: "boolean"
                                    },
                                    avoidQuotes: {
                                        type: "boolean"
                                    },
                                    avoidExplicitReturnArrows: {
                                        type: "boolean"
                                    }
                                },
                                additionalProperties: false
                            }
                        ],
                        minItems: 0,
                        maxItems: 2
                    }
                ]
            }
        },
        create(context) {
            const APPLY = context.options[0] || OPTIONS.always;
            const APPLY_TO_METHODS = APPLY === OPTIONS.methods || APPLY === OPTIONS.always;
            const APPLY_TO_PROPS = APPLY === OPTIONS.properties || APPLY === OPTIONS.always;
            const APPLY_NEVER = APPLY === OPTIONS.never;
            const APPLY_CONSISTENT = APPLY === OPTIONS.consistent;
            const APPLY_CONSISTENT_AS_NEEDED = APPLY === OPTIONS.consistentAsNeeded;
            const PARAMS = context.options[1] || {};
            const IGNORE_CONSTRUCTORS = PARAMS.ignoreConstructors;
            const AVOID_QUOTES = PARAMS.avoidQuotes;
            const AVOID_EXPLICIT_RETURN_ARROWS = !!PARAMS.avoidExplicitReturnArrows;
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            const CTOR_PREFIX_REGEX = /[^_$0-9]/u;
            /**
             * Determines if the first character of the name is a capital letter.
             * @param {string} name The name of the node to evaluate.
             * @returns {boolean} True if the first character of the property name is a capital letter, false if not.
             * @private
             */
            function isConstructor(name) {
                const match = CTOR_PREFIX_REGEX.exec(name);
                // Not a constructor if name has no characters apart from '_', '$' and digits e.g. '_', '$$', '_8'
                if (!match) {
                    return false;
                }
                const firstChar = name.charAt(match.index);
                return firstChar === firstChar.toUpperCase();
            }
            /**
             * Determines if the property can have a shorthand form.
             * @param {ASTNode} property Property AST node
             * @returns {boolean} True if the property can have a shorthand form
             * @private
             *
             */
            function canHaveShorthand(property) {
                return (property.kind !== "set" && property.kind !== "get" && property.type !== "SpreadElement" && property.type !== "SpreadProperty" && property.type !== "ExperimentalSpreadProperty");
            }
            /**
             * Checks whether a node is a string literal.
             * @param   {ASTNode} node Any AST node.
             * @returns {boolean} `true` if it is a string literal.
             */
            function isStringLiteral(node) {
                return node.type === "Literal" && typeof node.value === "string";
            }
            /**
             * Determines if the property is a shorthand or not.
             * @param {ASTNode} property Property AST node
             * @returns {boolean} True if the property is considered shorthand, false if not.
             * @private
             *
             */
            function isShorthand(property) {
                // property.method is true when `{a(){}}`.
                return (property.shorthand || property.method);
            }
            /**
             * Determines if the property's key and method or value are named equally.
             * @param {ASTNode} property Property AST node
             * @returns {boolean} True if the key and value are named equally, false if not.
             * @private
             *
             */
            function isRedundant(property) {
                const value = property.value;
                if (value.type === "FunctionExpression") {
                    return !value.id; // Only anonymous should be shorthand method.
                }
                if (value.type === "Identifier") {
                    return astUtils.getStaticPropertyName(property) === value.name;
                }
                return false;
            }
            /**
             * Ensures that an object's properties are consistently shorthand, or not shorthand at all.
             * @param   {ASTNode} node Property AST node
             * @param   {boolean} checkRedundancy Whether to check longform redundancy
             * @returns {void}
             *
             */
            function checkConsistency(node, checkRedundancy) {
                // We are excluding getters/setters and spread properties as they are considered neither longform nor shorthand.
                const properties = node.properties.filter(canHaveShorthand);
                // Do we still have properties left after filtering the getters and setters?
                if (properties.length > 0) {
                    const shorthandProperties = properties.filter(isShorthand);
                    /*
                     * If we do not have an equal number of longform properties as
                     * shorthand properties, we are using the annotations inconsistently
                     */
                    if (shorthandProperties.length !== properties.length) {
                        // We have at least 1 shorthand property
                        if (shorthandProperties.length > 0) {
                            context.report({ node, message: "Unexpected mix of shorthand and non-shorthand properties." });
                        }
                        else if (checkRedundancy) {
                            /*
                             * If all properties of the object contain a method or value with a name matching it's key,
                             * all the keys are redundant.
                             */
                            const canAlwaysUseShorthand = properties.every(isRedundant);
                            if (canAlwaysUseShorthand) {
                                context.report({ node, message: "Expected shorthand for all properties." });
                            }
                        }
                    }
                }
            }
            /**
             * Fixes a FunctionExpression node by making it into a shorthand property.
             * @param {SourceCodeFixer} fixer The fixer object
             * @param {ASTNode} node A `Property` node that has a `FunctionExpression` or `ArrowFunctionExpression` as its value
             * @returns {Object} A fix for this node
             */
            function makeFunctionShorthand(fixer, node) {
                const firstKeyToken = node.computed
                    ? sourceCode.getFirstToken(node, astUtils.isOpeningBracketToken)
                    : sourceCode.getFirstToken(node.key);
                const lastKeyToken = node.computed
                    ? sourceCode.getFirstTokenBetween(node.key, node.value, astUtils.isClosingBracketToken)
                    : sourceCode.getLastToken(node.key);
                const keyText = sourceCode.text.slice(firstKeyToken.range[0], lastKeyToken.range[1]);
                let keyPrefix = "";
                // key: /* */ () => {}
                if (sourceCode.commentsExistBetween(lastKeyToken, node.value)) {
                    return null;
                }
                if (node.value.async) {
                    keyPrefix += "async ";
                }
                if (node.value.generator) {
                    keyPrefix += "*";
                }
                const fixRange = [firstKeyToken.range[0], node.range[1]];
                const methodPrefix = keyPrefix + keyText;
                if (node.value.type === "FunctionExpression") {
                    const functionToken = sourceCode.getTokens(node.value).find(token => token.type === "Keyword" && token.value === "function");
                    const tokenBeforeParams = node.value.generator ? sourceCode.getTokenAfter(functionToken) : functionToken;
                    return fixer.replaceTextRange(fixRange, methodPrefix + sourceCode.text.slice(tokenBeforeParams.range[1], node.value.range[1]));
                }
                const arrowToken = sourceCode.getTokenBefore(node.value.body, astUtils.isArrowToken);
                const fnBody = sourceCode.text.slice(arrowToken.range[1], node.value.range[1]);
                let shouldAddParensAroundParameters = false;
                let tokenBeforeParams;
                if (node.value.params.length === 0) {
                    tokenBeforeParams = sourceCode.getFirstToken(node.value, astUtils.isOpeningParenToken);
                }
                else {
                    tokenBeforeParams = sourceCode.getTokenBefore(node.value.params[0]);
                }
                if (node.value.params.length === 1) {
                    const hasParen = astUtils.isOpeningParenToken(tokenBeforeParams);
                    const isTokenOutsideNode = tokenBeforeParams.range[0] < node.range[0];
                    shouldAddParensAroundParameters = !hasParen || isTokenOutsideNode;
                }
                const sliceStart = shouldAddParensAroundParameters
                    ? node.value.params[0].range[0]
                    : tokenBeforeParams.range[0];
                const sliceEnd = sourceCode.getTokenBefore(arrowToken).range[1];
                const oldParamText = sourceCode.text.slice(sliceStart, sliceEnd);
                const newParamText = shouldAddParensAroundParameters ? `(${oldParamText})` : oldParamText;
                return fixer.replaceTextRange(fixRange, methodPrefix + newParamText + fnBody);
            }
            /**
             * Fixes a FunctionExpression node by making it into a longform property.
             * @param {SourceCodeFixer} fixer The fixer object
             * @param {ASTNode} node A `Property` node that has a `FunctionExpression` as its value
             * @returns {Object} A fix for this node
             */
            function makeFunctionLongform(fixer, node) {
                const firstKeyToken = node.computed ? sourceCode.getTokens(node).find(token => token.value === "[") : sourceCode.getFirstToken(node.key);
                const lastKeyToken = node.computed ? sourceCode.getTokensBetween(node.key, node.value).find(token => token.value === "]") : sourceCode.getLastToken(node.key);
                const keyText = sourceCode.text.slice(firstKeyToken.range[0], lastKeyToken.range[1]);
                let functionHeader = "function";
                if (node.value.async) {
                    functionHeader = `async ${functionHeader}`;
                }
                if (node.value.generator) {
                    functionHeader = `${functionHeader}*`;
                }
                return fixer.replaceTextRange([node.range[0], lastKeyToken.range[1]], `${keyText}: ${functionHeader}`);
            }
            /*
             * To determine whether a given arrow function has a lexical identifier (`this`, `arguments`, `super`, or `new.target`),
             * create a stack of functions that define these identifiers (i.e. all functions except arrow functions) as the AST is
             * traversed. Whenever a new function is encountered, create a new entry on the stack (corresponding to a different lexical
             * scope of `this`), and whenever a function is exited, pop that entry off the stack. When an arrow function is entered,
             * keep a reference to it on the current stack entry, and remove that reference when the arrow function is exited.
             * When a lexical identifier is encountered, mark all the arrow functions on the current stack entry by adding them
             * to an `arrowsWithLexicalIdentifiers` set. Any arrow function in that set will not be reported by this rule,
             * because converting it into a method would change the value of one of the lexical identifiers.
             */
            const lexicalScopeStack = [];
            const arrowsWithLexicalIdentifiers = new WeakSet();
            const argumentsIdentifiers = new WeakSet();
            /**
             * Enters a function. This creates a new lexical identifier scope, so a new Set of arrow functions is pushed onto the stack.
             * Also, this marks all `arguments` identifiers so that they can be detected later.
             * @returns {void}
             */
            function enterFunction() {
                lexicalScopeStack.unshift(new Set());
                context.getScope().variables.filter(variable => variable.name === "arguments").forEach(variable => {
                    variable.references.map(ref => ref.identifier).forEach(identifier => argumentsIdentifiers.add(identifier));
                });
            }
            /**
             * Exits a function. This pops the current set of arrow functions off the lexical scope stack.
             * @returns {void}
             */
            function exitFunction() {
                lexicalScopeStack.shift();
            }
            /**
             * Marks the current function as having a lexical keyword. This implies that all arrow functions
             * in the current lexical scope contain a reference to this lexical keyword.
             * @returns {void}
             */
            function reportLexicalIdentifier() {
                lexicalScopeStack[0].forEach(arrowFunction => arrowsWithLexicalIdentifiers.add(arrowFunction));
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                Program: enterFunction,
                FunctionDeclaration: enterFunction,
                FunctionExpression: enterFunction,
                "Program:exit": exitFunction,
                "FunctionDeclaration:exit": exitFunction,
                "FunctionExpression:exit": exitFunction,
                ArrowFunctionExpression(node) {
                    lexicalScopeStack[0].add(node);
                },
                "ArrowFunctionExpression:exit"(node) {
                    lexicalScopeStack[0].delete(node);
                },
                ThisExpression: reportLexicalIdentifier,
                Super: reportLexicalIdentifier,
                MetaProperty(node) {
                    if (node.meta.name === "new" && node.property.name === "target") {
                        reportLexicalIdentifier();
                    }
                },
                Identifier(node) {
                    if (argumentsIdentifiers.has(node)) {
                        reportLexicalIdentifier();
                    }
                },
                ObjectExpression(node) {
                    if (APPLY_CONSISTENT) {
                        checkConsistency(node, false);
                    }
                    else if (APPLY_CONSISTENT_AS_NEEDED) {
                        checkConsistency(node, true);
                    }
                },
                "Property:exit"(node) {
                    const isConciseProperty = node.method || node.shorthand;
                    // Ignore destructuring assignment
                    if (node.parent.type === "ObjectPattern") {
                        return;
                    }
                    // getters and setters are ignored
                    if (node.kind === "get" || node.kind === "set") {
                        return;
                    }
                    // only computed methods can fail the following checks
                    if (node.computed && node.value.type !== "FunctionExpression" && node.value.type !== "ArrowFunctionExpression") {
                        return;
                    }
                    //--------------------------------------------------------------
                    // Checks for property/method shorthand.
                    if (isConciseProperty) {
                        if (node.method && (APPLY_NEVER || AVOID_QUOTES && isStringLiteral(node.key))) {
                            const message = APPLY_NEVER ? "Expected longform method syntax." : "Expected longform method syntax for string literal keys.";
                            // { x() {} } should be written as { x: function() {} }
                            context.report({
                                node,
                                message,
                                fix: fixer => makeFunctionLongform(fixer, node)
                            });
                        }
                        else if (APPLY_NEVER) {
                            // { x } should be written as { x: x }
                            context.report({
                                node,
                                message: "Expected longform property syntax.",
                                fix: fixer => fixer.insertTextAfter(node.key, `: ${node.key.name}`)
                            });
                        }
                    }
                    else if (APPLY_TO_METHODS && !node.value.id && (node.value.type === "FunctionExpression" || node.value.type === "ArrowFunctionExpression")) {
                        if (IGNORE_CONSTRUCTORS && node.key.type === "Identifier" && isConstructor(node.key.name)) {
                            return;
                        }
                        if (AVOID_QUOTES && isStringLiteral(node.key)) {
                            return;
                        }
                        // {[x]: function(){}} should be written as {[x]() {}}
                        if (node.value.type === "FunctionExpression" ||
                            node.value.type === "ArrowFunctionExpression" &&
                                node.value.body.type === "BlockStatement" &&
                                AVOID_EXPLICIT_RETURN_ARROWS &&
                                !arrowsWithLexicalIdentifiers.has(node.value)) {
                            context.report({
                                node,
                                message: "Expected method shorthand.",
                                fix: fixer => makeFunctionShorthand(fixer, node)
                            });
                        }
                    }
                    else if (node.value.type === "Identifier" && node.key.name === node.value.name && APPLY_TO_PROPS) {
                        // {x: x} should be written as {x}
                        context.report({
                            node,
                            message: "Expected property shorthand.",
                            fix(fixer) {
                                return fixer.replaceText(node, node.value.name);
                            }
                        });
                    }
                    else if (node.value.type === "Identifier" && node.key.type === "Literal" && node.key.value === node.value.name && APPLY_TO_PROPS) {
                        if (AVOID_QUOTES) {
                            return;
                        }
                        // {"x": x} should be written as {x}
                        context.report({
                            node,
                            message: "Expected property shorthand.",
                            fix(fixer) {
                                return fixer.replaceText(node, node.value.name);
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/one-var.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce variables to be declared either together or separately in functions",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/one-var"
            },
            fixable: "code",
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["always", "never", "consecutive"]
                        },
                        {
                            type: "object",
                            properties: {
                                separateRequires: {
                                    type: "boolean"
                                },
                                var: {
                                    enum: ["always", "never", "consecutive"]
                                },
                                let: {
                                    enum: ["always", "never", "consecutive"]
                                },
                                const: {
                                    enum: ["always", "never", "consecutive"]
                                }
                            },
                            additionalProperties: false
                        },
                        {
                            type: "object",
                            properties: {
                                initialized: {
                                    enum: ["always", "never", "consecutive"]
                                },
                                uninitialized: {
                                    enum: ["always", "never", "consecutive"]
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ]
        },
        create(context) {
            const MODE_ALWAYS = "always";
            const MODE_NEVER = "never";
            const MODE_CONSECUTIVE = "consecutive";
            const mode = context.options[0] || MODE_ALWAYS;
            const options = {};
            if (typeof mode === "string") { // simple options configuration with just a string
                options.var = { uninitialized: mode, initialized: mode };
                options.let = { uninitialized: mode, initialized: mode };
                options.const = { uninitialized: mode, initialized: mode };
            }
            else if (typeof mode === "object") { // options configuration is an object
                options.separateRequires = !!mode.separateRequires;
                options.var = { uninitialized: mode.var, initialized: mode.var };
                options.let = { uninitialized: mode.let, initialized: mode.let };
                options.const = { uninitialized: mode.const, initialized: mode.const };
                if (Object.prototype.hasOwnProperty.call(mode, "uninitialized")) {
                    options.var.uninitialized = mode.uninitialized;
                    options.let.uninitialized = mode.uninitialized;
                    options.const.uninitialized = mode.uninitialized;
                }
                if (Object.prototype.hasOwnProperty.call(mode, "initialized")) {
                    options.var.initialized = mode.initialized;
                    options.let.initialized = mode.initialized;
                    options.const.initialized = mode.initialized;
                }
            }
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            const functionStack = [];
            const blockStack = [];
            /**
             * Increments the blockStack counter.
             * @returns {void}
             * @private
             */
            function startBlock() {
                blockStack.push({
                    let: { initialized: false, uninitialized: false },
                    const: { initialized: false, uninitialized: false }
                });
            }
            /**
             * Increments the functionStack counter.
             * @returns {void}
             * @private
             */
            function startFunction() {
                functionStack.push({ initialized: false, uninitialized: false });
                startBlock();
            }
            /**
             * Decrements the blockStack counter.
             * @returns {void}
             * @private
             */
            function endBlock() {
                blockStack.pop();
            }
            /**
             * Decrements the functionStack counter.
             * @returns {void}
             * @private
             */
            function endFunction() {
                functionStack.pop();
                endBlock();
            }
            /**
             * Check if a variable declaration is a require.
             * @param {ASTNode} decl variable declaration Node
             * @returns {bool} if decl is a require, return true; else return false.
             * @private
             */
            function isRequire(decl) {
                return decl.init && decl.init.type === "CallExpression" && decl.init.callee.name === "require";
            }
            /**
             * Records whether initialized/uninitialized/required variables are defined in current scope.
             * @param {string} statementType node.kind, one of: "var", "let", or "const"
             * @param {ASTNode[]} declarations List of declarations
             * @param {Object} currentScope The scope being investigated
             * @returns {void}
             * @private
             */
            function recordTypes(statementType, declarations, currentScope) {
                for (let i = 0; i < declarations.length; i++) {
                    if (declarations[i].init === null) {
                        if (options[statementType] && options[statementType].uninitialized === MODE_ALWAYS) {
                            currentScope.uninitialized = true;
                        }
                    }
                    else {
                        if (options[statementType] && options[statementType].initialized === MODE_ALWAYS) {
                            if (options.separateRequires && isRequire(declarations[i])) {
                                currentScope.required = true;
                            }
                            else {
                                currentScope.initialized = true;
                            }
                        }
                    }
                }
            }
            /**
             * Determines the current scope (function or block)
             * @param  {string} statementType node.kind, one of: "var", "let", or "const"
             * @returns {Object} The scope associated with statementType
             */
            function getCurrentScope(statementType) {
                let currentScope;
                if (statementType === "var") {
                    currentScope = functionStack[functionStack.length - 1];
                }
                else if (statementType === "let") {
                    currentScope = blockStack[blockStack.length - 1].let;
                }
                else if (statementType === "const") {
                    currentScope = blockStack[blockStack.length - 1].const;
                }
                return currentScope;
            }
            /**
             * Counts the number of initialized and uninitialized declarations in a list of declarations
             * @param {ASTNode[]} declarations List of declarations
             * @returns {Object} Counts of 'uninitialized' and 'initialized' declarations
             * @private
             */
            function countDeclarations(declarations) {
                const counts = { uninitialized: 0, initialized: 0 };
                for (let i = 0; i < declarations.length; i++) {
                    if (declarations[i].init === null) {
                        counts.uninitialized++;
                    }
                    else {
                        counts.initialized++;
                    }
                }
                return counts;
            }
            /**
             * Determines if there is more than one var statement in the current scope.
             * @param {string} statementType node.kind, one of: "var", "let", or "const"
             * @param {ASTNode[]} declarations List of declarations
             * @returns {boolean} Returns true if it is the first var declaration, false if not.
             * @private
             */
            function hasOnlyOneStatement(statementType, declarations) {
                const declarationCounts = countDeclarations(declarations);
                const currentOptions = options[statementType] || {};
                const currentScope = getCurrentScope(statementType);
                const hasRequires = declarations.some(isRequire);
                if (currentOptions.uninitialized === MODE_ALWAYS && currentOptions.initialized === MODE_ALWAYS) {
                    if (currentScope.uninitialized || currentScope.initialized) {
                        if (!hasRequires) {
                            return false;
                        }
                    }
                }
                if (declarationCounts.uninitialized > 0) {
                    if (currentOptions.uninitialized === MODE_ALWAYS && currentScope.uninitialized) {
                        return false;
                    }
                }
                if (declarationCounts.initialized > 0) {
                    if (currentOptions.initialized === MODE_ALWAYS && currentScope.initialized) {
                        if (!hasRequires) {
                            return false;
                        }
                    }
                }
                if (currentScope.required && hasRequires) {
                    return false;
                }
                recordTypes(statementType, declarations, currentScope);
                return true;
            }
            /**
             * Fixer to join VariableDeclaration's into a single declaration
             * @param   {VariableDeclarator[]} declarations The `VariableDeclaration` to join
             * @returns {Function}                         The fixer function
             */
            function joinDeclarations(declarations) {
                const declaration = declarations[0];
                const body = Array.isArray(declaration.parent.parent.body) ? declaration.parent.parent.body : [];
                const currentIndex = body.findIndex(node => node.range[0] === declaration.parent.range[0]);
                const previousNode = body[currentIndex - 1];
                return fixer => {
                    const type = sourceCode.getTokenBefore(declaration);
                    const prevSemi = sourceCode.getTokenBefore(type);
                    const res = [];
                    if (previousNode && previousNode.kind === sourceCode.getText(type)) {
                        if (prevSemi.value === ";") {
                            res.push(fixer.replaceText(prevSemi, ","));
                        }
                        else {
                            res.push(fixer.insertTextAfter(prevSemi, ","));
                        }
                        res.push(fixer.replaceText(type, ""));
                    }
                    return res;
                };
            }
            /**
             * Fixer to split a VariableDeclaration into individual declarations
             * @param   {VariableDeclaration}   declaration The `VariableDeclaration` to split
             * @returns {Function}                          The fixer function
             */
            function splitDeclarations(declaration) {
                return fixer => declaration.declarations.map(declarator => {
                    const tokenAfterDeclarator = sourceCode.getTokenAfter(declarator);
                    if (tokenAfterDeclarator === null) {
                        return null;
                    }
                    const afterComma = sourceCode.getTokenAfter(tokenAfterDeclarator, { includeComments: true });
                    if (tokenAfterDeclarator.value !== ",") {
                        return null;
                    }
                    /*
                     * `var x,y`
                     * tokenAfterDeclarator ^^ afterComma
                     */
                    if (afterComma.range[0] === tokenAfterDeclarator.range[1]) {
                        return fixer.replaceText(tokenAfterDeclarator, `; ${declaration.kind} `);
                    }
                    /*
                     * `var x,
                     * tokenAfterDeclarator ^
                     *      y`
                     *      ^ afterComma
                     */
                    if (afterComma.loc.start.line > tokenAfterDeclarator.loc.end.line ||
                        afterComma.type === "Line" ||
                        afterComma.type === "Block") {
                        let lastComment = afterComma;
                        while (lastComment.type === "Line" || lastComment.type === "Block") {
                            lastComment = sourceCode.getTokenAfter(lastComment, { includeComments: true });
                        }
                        return fixer.replaceTextRange([tokenAfterDeclarator.range[0], lastComment.range[0]], `;${sourceCode.text.slice(tokenAfterDeclarator.range[1], lastComment.range[0])}${declaration.kind} `);
                    }
                    return fixer.replaceText(tokenAfterDeclarator, `; ${declaration.kind}`);
                }).filter(x => x);
            }
            /**
             * Checks a given VariableDeclaration node for errors.
             * @param {ASTNode} node The VariableDeclaration node to check
             * @returns {void}
             * @private
             */
            function checkVariableDeclaration(node) {
                const parent = node.parent;
                const type = node.kind;
                if (!options[type]) {
                    return;
                }
                const declarations = node.declarations;
                const declarationCounts = countDeclarations(declarations);
                const mixedRequires = declarations.some(isRequire) && !declarations.every(isRequire);
                if (options[type].initialized === MODE_ALWAYS) {
                    if (options.separateRequires && mixedRequires) {
                        context.report({
                            node,
                            message: "Split requires to be separated into a single block."
                        });
                    }
                }
                // consecutive
                const nodeIndex = (parent.body && parent.body.length > 0 && parent.body.indexOf(node)) || 0;
                if (nodeIndex > 0) {
                    const previousNode = parent.body[nodeIndex - 1];
                    const isPreviousNodeDeclaration = previousNode.type === "VariableDeclaration";
                    const declarationsWithPrevious = declarations.concat(previousNode.declarations || []);
                    if (isPreviousNodeDeclaration &&
                        previousNode.kind === type &&
                        !(declarationsWithPrevious.some(isRequire) && !declarationsWithPrevious.every(isRequire))) {
                        const previousDeclCounts = countDeclarations(previousNode.declarations);
                        if (options[type].initialized === MODE_CONSECUTIVE && options[type].uninitialized === MODE_CONSECUTIVE) {
                            context.report({
                                node,
                                message: "Combine this with the previous '{{type}}' statement.",
                                data: {
                                    type
                                },
                                fix: joinDeclarations(declarations)
                            });
                        }
                        else if (options[type].initialized === MODE_CONSECUTIVE && declarationCounts.initialized > 0 && previousDeclCounts.initialized > 0) {
                            context.report({
                                node,
                                message: "Combine this with the previous '{{type}}' statement with initialized variables.",
                                data: {
                                    type
                                },
                                fix: joinDeclarations(declarations)
                            });
                        }
                        else if (options[type].uninitialized === MODE_CONSECUTIVE &&
                            declarationCounts.uninitialized > 0 &&
                            previousDeclCounts.uninitialized > 0) {
                            context.report({
                                node,
                                message: "Combine this with the previous '{{type}}' statement with uninitialized variables.",
                                data: {
                                    type
                                },
                                fix: joinDeclarations(declarations)
                            });
                        }
                    }
                }
                // always
                if (!hasOnlyOneStatement(type, declarations)) {
                    if (options[type].initialized === MODE_ALWAYS && options[type].uninitialized === MODE_ALWAYS) {
                        context.report({
                            node,
                            message: "Combine this with the previous '{{type}}' statement.",
                            data: {
                                type
                            },
                            fix: joinDeclarations(declarations)
                        });
                    }
                    else {
                        if (options[type].initialized === MODE_ALWAYS && declarationCounts.initialized > 0) {
                            context.report({
                                node,
                                message: "Combine this with the previous '{{type}}' statement with initialized variables.",
                                data: {
                                    type
                                },
                                fix: joinDeclarations(declarations)
                            });
                        }
                        if (options[type].uninitialized === MODE_ALWAYS && declarationCounts.uninitialized > 0) {
                            if (node.parent.left === node && (node.parent.type === "ForInStatement" || node.parent.type === "ForOfStatement")) {
                                return;
                            }
                            context.report({
                                node,
                                message: "Combine this with the previous '{{type}}' statement with uninitialized variables.",
                                data: {
                                    type
                                },
                                fix: joinDeclarations(declarations)
                            });
                        }
                    }
                }
                // never
                if (parent.type !== "ForStatement" || parent.init !== node) {
                    const totalDeclarations = declarationCounts.uninitialized + declarationCounts.initialized;
                    if (totalDeclarations > 1) {
                        if (options[type].initialized === MODE_NEVER && options[type].uninitialized === MODE_NEVER) {
                            // both initialized and uninitialized
                            context.report({
                                node,
                                message: "Split '{{type}}' declarations into multiple statements.",
                                data: {
                                    type
                                },
                                fix: splitDeclarations(node)
                            });
                        }
                        else if (options[type].initialized === MODE_NEVER && declarationCounts.initialized > 0) {
                            // initialized
                            context.report({
                                node,
                                message: "Split initialized '{{type}}' declarations into multiple statements.",
                                data: {
                                    type
                                },
                                fix: splitDeclarations(node)
                            });
                        }
                        else if (options[type].uninitialized === MODE_NEVER && declarationCounts.uninitialized > 0) {
                            // uninitialized
                            context.report({
                                node,
                                message: "Split uninitialized '{{type}}' declarations into multiple statements.",
                                data: {
                                    type
                                },
                                fix: splitDeclarations(node)
                            });
                        }
                    }
                }
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                Program: startFunction,
                FunctionDeclaration: startFunction,
                FunctionExpression: startFunction,
                ArrowFunctionExpression: startFunction,
                BlockStatement: startBlock,
                ForStatement: startBlock,
                ForInStatement: startBlock,
                ForOfStatement: startBlock,
                SwitchStatement: startBlock,
                VariableDeclaration: checkVariableDeclaration,
                "ForStatement:exit": endBlock,
                "ForOfStatement:exit": endBlock,
                "ForInStatement:exit": endBlock,
                "SwitchStatement:exit": endBlock,
                "BlockStatement:exit": endBlock,
                "Program:exit": endFunction,
                "FunctionDeclaration:exit": endFunction,
                "FunctionExpression:exit": endFunction,
                "ArrowFunctionExpression:exit": endFunction
            };
        }
    };
},
/* eslint/lib/rules/one-var-declaration-per-line.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require or disallow newlines around variable declarations",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/one-var-declaration-per-line"
            },
            schema: [
                {
                    enum: ["always", "initializations"]
                }
            ],
            fixable: "whitespace"
        },
        create(context) {
            const ERROR_MESSAGE = "Expected variable declaration to be on a new line.";
            const always = context.options[0] === "always";
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Determine if provided keyword is a variant of for specifiers
             * @private
             * @param {string} keyword keyword to test
             * @returns {boolean} True if `keyword` is a variant of for specifier
             */
            function isForTypeSpecifier(keyword) {
                return keyword === "ForStatement" || keyword === "ForInStatement" || keyword === "ForOfStatement";
            }
            /**
             * Checks newlines around variable declarations.
             * @private
             * @param {ASTNode} node `VariableDeclaration` node to test
             * @returns {void}
             */
            function checkForNewLine(node) {
                if (isForTypeSpecifier(node.parent.type)) {
                    return;
                }
                const declarations = node.declarations;
                let prev;
                declarations.forEach(current => {
                    if (prev && prev.loc.end.line === current.loc.start.line) {
                        if (always || prev.init || current.init) {
                            context.report({
                                node,
                                message: ERROR_MESSAGE,
                                loc: current.loc.start,
                                fix: fixer => fixer.insertTextBefore(current, "\n")
                            });
                        }
                    }
                    prev = current;
                });
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                VariableDeclaration: checkForNewLine
            };
        }
    };
},
/* eslint/lib/rules/operator-assignment.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether an operator is commutative and has an operator assignment
     * shorthand form.
     * @param   {string}  operator Operator to check.
     * @returns {boolean}          True if the operator is commutative and has a
     *     shorthand form.
     */
    function isCommutativeOperatorWithShorthand(operator) {
        return ["*", "&", "^", "|"].indexOf(operator) >= 0;
    }
    /**
     * Checks whether an operator is not commuatative and has an operator assignment
     * shorthand form.
     * @param   {string}  operator Operator to check.
     * @returns {boolean}          True if the operator is not commuatative and has
     *     a shorthand form.
     */
    function isNonCommutativeOperatorWithShorthand(operator) {
        return ["+", "-", "/", "%", "<<", ">>", ">>>", "**"].indexOf(operator) >= 0;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    /**
     * Checks whether two expressions reference the same value. For example:
     *     a = a
     *     a.b = a.b
     *     a[0] = a[0]
     *     a['b'] = a['b']
     * @param   {ASTNode} a Left side of the comparison.
     * @param   {ASTNode} b Right side of the comparison.
     * @returns {boolean}   True if both sides match and reference the same value.
     */
    function same(a, b) {
        if (a.type !== b.type) {
            return false;
        }
        switch (a.type) {
            case "Identifier":
                return a.name === b.name;
            case "Literal":
                return a.value === b.value;
            case "MemberExpression":
                /*
                 * x[0] = x[0]
                 * x[y] = x[y]
                 * x.y = x.y
                 */
                return same(a.object, b.object) && same(a.property, b.property);
            case "ThisExpression":
                return true;
            default:
                return false;
        }
    }
    /**
     * Determines if the left side of a node can be safely fixed (i.e. if it activates the same getters/setters and)
     * toString calls regardless of whether assignment shorthand is used)
     * @param {ASTNode} node The node on the left side of the expression
     * @returns {boolean} `true` if the node can be fixed
     */
    function canBeFixed(node) {
        return (node.type === "Identifier" ||
            (node.type === "MemberExpression" &&
                (node.object.type === "Identifier" || node.object.type === "ThisExpression") &&
                (!node.computed || node.property.type === "Literal")));
    }
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require or disallow assignment operator shorthand where possible",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/operator-assignment"
            },
            schema: [
                {
                    enum: ["always", "never"]
                }
            ],
            fixable: "code",
            messages: {
                replaced: "Assignment can be replaced with operator assignment.",
                unexpected: "Unexpected operator assignment shorthand."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Returns the operator token of an AssignmentExpression or BinaryExpression
             * @param {ASTNode} node An AssignmentExpression or BinaryExpression node
             * @returns {Token} The operator token in the node
             */
            function getOperatorToken(node) {
                return sourceCode.getFirstTokenBetween(node.left, node.right, token => token.value === node.operator);
            }
            /**
             * Ensures that an assignment uses the shorthand form where possible.
             * @param   {ASTNode} node An AssignmentExpression node.
             * @returns {void}
             */
            function verify(node) {
                if (node.operator !== "=" || node.right.type !== "BinaryExpression") {
                    return;
                }
                const left = node.left;
                const expr = node.right;
                const operator = expr.operator;
                if (isCommutativeOperatorWithShorthand(operator) || isNonCommutativeOperatorWithShorthand(operator)) {
                    if (same(left, expr.left)) {
                        context.report({
                            node,
                            messageId: "replaced",
                            fix(fixer) {
                                if (canBeFixed(left)) {
                                    const equalsToken = getOperatorToken(node);
                                    const operatorToken = getOperatorToken(expr);
                                    const leftText = sourceCode.getText().slice(node.range[0], equalsToken.range[0]);
                                    const rightText = sourceCode.getText().slice(operatorToken.range[1], node.right.range[1]);
                                    // Check for comments that would be removed.
                                    if (sourceCode.commentsExistBetween(equalsToken, operatorToken)) {
                                        return null;
                                    }
                                    return fixer.replaceText(node, `${leftText}${expr.operator}=${rightText}`);
                                }
                                return null;
                            }
                        });
                    }
                    else if (same(left, expr.right) && isCommutativeOperatorWithShorthand(operator)) {
                        /*
                         * This case can't be fixed safely.
                         * If `a` and `b` both have custom valueOf() behavior, then fixing `a = b * a` to `a *= b` would
                         * change the execution order of the valueOf() functions.
                         */
                        context.report({
                            node,
                            messageId: "replaced"
                        });
                    }
                }
            }
            /**
             * Warns if an assignment expression uses operator assignment shorthand.
             * @param   {ASTNode} node An AssignmentExpression node.
             * @returns {void}
             */
            function prohibit(node) {
                if (node.operator !== "=") {
                    context.report({
                        node,
                        messageId: "unexpected",
                        fix(fixer) {
                            if (canBeFixed(node.left)) {
                                const firstToken = sourceCode.getFirstToken(node);
                                const operatorToken = getOperatorToken(node);
                                const leftText = sourceCode.getText().slice(node.range[0], operatorToken.range[0]);
                                const newOperator = node.operator.slice(0, -1);
                                let rightText;
                                // Check for comments that would be duplicated.
                                if (sourceCode.commentsExistBetween(firstToken, operatorToken)) {
                                    return null;
                                }
                                // If this change would modify precedence (e.g. `foo *= bar + 1` => `foo = foo * (bar + 1)`), parenthesize the right side.
                                if (astUtils.getPrecedence(node.right) <= astUtils.getPrecedence({ type: "BinaryExpression", operator: newOperator }) &&
                                    !astUtils.isParenthesised(sourceCode, node.right)) {
                                    rightText = `${sourceCode.text.slice(operatorToken.range[1], node.right.range[0])}(${sourceCode.getText(node.right)})`;
                                }
                                else {
                                    const firstRightToken = sourceCode.getFirstToken(node.right);
                                    let rightTextPrefix = "";
                                    if (operatorToken.range[1] === firstRightToken.range[0] &&
                                        !astUtils.canTokensBeAdjacent(newOperator, firstRightToken)) {
                                        rightTextPrefix = " "; // foo+=+bar -> foo= foo+ +bar
                                    }
                                    rightText = `${rightTextPrefix}${sourceCode.text.slice(operatorToken.range[1], node.range[1])}`;
                                }
                                return fixer.replaceText(node, `${leftText}= ${leftText}${newOperator}${rightText}`);
                            }
                            return null;
                        }
                    });
                }
            }
            return {
                AssignmentExpression: context.options[0] !== "never" ? verify : prohibit
            };
        }
    };
},
/* eslint/lib/rules/operator-linebreak.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent linebreak style for operators",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/operator-linebreak"
            },
            schema: [
                {
                    enum: ["after", "before", "none", null]
                },
                {
                    type: "object",
                    properties: {
                        overrides: {
                            type: "object",
                            properties: {
                                anyOf: {
                                    type: "string",
                                    enum: ["after", "before", "none", "ignore"]
                                }
                            }
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "code"
        },
        create(context) {
            const usedDefaultGlobal = !context.options[0];
            const globalStyle = context.options[0] || "after";
            const options = context.options[1] || {};
            const styleOverrides = options.overrides ? Object.assign({}, options.overrides) : {};
            if (usedDefaultGlobal && !styleOverrides["?"]) {
                styleOverrides["?"] = "before";
            }
            if (usedDefaultGlobal && !styleOverrides[":"]) {
                styleOverrides[":"] = "before";
            }
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Gets a fixer function to fix rule issues
             * @param {Token} operatorToken The operator token of an expression
             * @param {string} desiredStyle The style for the rule. One of 'before', 'after', 'none'
             * @returns {Function} A fixer function
             */
            function getFixer(operatorToken, desiredStyle) {
                return fixer => {
                    const tokenBefore = sourceCode.getTokenBefore(operatorToken);
                    const tokenAfter = sourceCode.getTokenAfter(operatorToken);
                    const textBefore = sourceCode.text.slice(tokenBefore.range[1], operatorToken.range[0]);
                    const textAfter = sourceCode.text.slice(operatorToken.range[1], tokenAfter.range[0]);
                    const hasLinebreakBefore = !astUtils.isTokenOnSameLine(tokenBefore, operatorToken);
                    const hasLinebreakAfter = !astUtils.isTokenOnSameLine(operatorToken, tokenAfter);
                    let newTextBefore, newTextAfter;
                    if (hasLinebreakBefore !== hasLinebreakAfter && desiredStyle !== "none") {
                        // If there is a comment before and after the operator, don't do a fix.
                        if (sourceCode.getTokenBefore(operatorToken, { includeComments: true }) !== tokenBefore &&
                            sourceCode.getTokenAfter(operatorToken, { includeComments: true }) !== tokenAfter) {
                            return null;
                        }
                        /*
                         * If there is only one linebreak and it's on the wrong side of the operator, swap the text before and after the operator.
                         * foo &&
                         *           bar
                         * would get fixed to
                         * foo
                         *        && bar
                         */
                        newTextBefore = textAfter;
                        newTextAfter = textBefore;
                    }
                    else {
                        const LINEBREAK_REGEX = astUtils.createGlobalLinebreakMatcher();
                        // Otherwise, if no linebreak is desired and no comments interfere, replace the linebreaks with empty strings.
                        newTextBefore = desiredStyle === "before" || textBefore.trim() ? textBefore : textBefore.replace(LINEBREAK_REGEX, "");
                        newTextAfter = desiredStyle === "after" || textAfter.trim() ? textAfter : textAfter.replace(LINEBREAK_REGEX, "");
                        // If there was no change (due to interfering comments), don't output a fix.
                        if (newTextBefore === textBefore && newTextAfter === textAfter) {
                            return null;
                        }
                    }
                    if (newTextAfter === "" && tokenAfter.type === "Punctuator" && "+-".includes(operatorToken.value) && tokenAfter.value === operatorToken.value) {
                        // To avoid accidentally creating a ++ or -- operator, insert a space if the operator is a +/- and the following token is a unary +/-.
                        newTextAfter += " ";
                    }
                    return fixer.replaceTextRange([tokenBefore.range[1], tokenAfter.range[0]], newTextBefore + operatorToken.value + newTextAfter);
                };
            }
            /**
             * Checks the operator placement
             * @param {ASTNode} node The node to check
             * @param {ASTNode} leftSide The node that comes before the operator in `node`
             * @private
             * @returns {void}
             */
            function validateNode(node, leftSide) {
                /*
                 * When the left part of a binary expression is a single expression wrapped in
                 * parentheses (ex: `(a) + b`), leftToken will be the last token of the expression
                 * and operatorToken will be the closing parenthesis.
                 * The leftToken should be the last closing parenthesis, and the operatorToken
                 * should be the token right after that.
                 */
                const operatorToken = sourceCode.getTokenAfter(leftSide, astUtils.isNotClosingParenToken);
                const leftToken = sourceCode.getTokenBefore(operatorToken);
                const rightToken = sourceCode.getTokenAfter(operatorToken);
                const operator = operatorToken.value;
                const operatorStyleOverride = styleOverrides[operator];
                const style = operatorStyleOverride || globalStyle;
                const fix = getFixer(operatorToken, style);
                // if single line
                if (astUtils.isTokenOnSameLine(leftToken, operatorToken) &&
                    astUtils.isTokenOnSameLine(operatorToken, rightToken)) {
                    // do nothing.
                }
                else if (operatorStyleOverride !== "ignore" && !astUtils.isTokenOnSameLine(leftToken, operatorToken) &&
                    !astUtils.isTokenOnSameLine(operatorToken, rightToken)) {
                    // lone operator
                    context.report({
                        node,
                        loc: {
                            line: operatorToken.loc.end.line,
                            column: operatorToken.loc.end.column
                        },
                        message: "Bad line breaking before and after '{{operator}}'.",
                        data: {
                            operator
                        },
                        fix
                    });
                }
                else if (style === "before" && astUtils.isTokenOnSameLine(leftToken, operatorToken)) {
                    context.report({
                        node,
                        loc: {
                            line: operatorToken.loc.end.line,
                            column: operatorToken.loc.end.column
                        },
                        message: "'{{operator}}' should be placed at the beginning of the line.",
                        data: {
                            operator
                        },
                        fix
                    });
                }
                else if (style === "after" && astUtils.isTokenOnSameLine(operatorToken, rightToken)) {
                    context.report({
                        node,
                        loc: {
                            line: operatorToken.loc.end.line,
                            column: operatorToken.loc.end.column
                        },
                        message: "'{{operator}}' should be placed at the end of the line.",
                        data: {
                            operator
                        },
                        fix
                    });
                }
                else if (style === "none") {
                    context.report({
                        node,
                        loc: {
                            line: operatorToken.loc.end.line,
                            column: operatorToken.loc.end.column
                        },
                        message: "There should be no line break before or after '{{operator}}'.",
                        data: {
                            operator
                        },
                        fix
                    });
                }
            }
            /**
             * Validates a binary expression using `validateNode`
             * @param {BinaryExpression|LogicalExpression|AssignmentExpression} node node to be validated
             * @returns {void}
             */
            function validateBinaryExpression(node) {
                validateNode(node, node.left);
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                BinaryExpression: validateBinaryExpression,
                LogicalExpression: validateBinaryExpression,
                AssignmentExpression: validateBinaryExpression,
                VariableDeclarator(node) {
                    if (node.init) {
                        validateNode(node, node.id);
                    }
                },
                ConditionalExpression(node) {
                    validateNode(node, node.test);
                    validateNode(node, node.consequent);
                }
            };
        }
    };
},
/* eslint/lib/rules/padded-blocks.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow padding within blocks",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/padded-blocks"
            },
            fixable: "whitespace",
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["always", "never"]
                        },
                        {
                            type: "object",
                            properties: {
                                blocks: {
                                    enum: ["always", "never"]
                                },
                                switches: {
                                    enum: ["always", "never"]
                                },
                                classes: {
                                    enum: ["always", "never"]
                                }
                            },
                            additionalProperties: false,
                            minProperties: 1
                        }
                    ]
                },
                {
                    type: "object",
                    properties: {
                        allowSingleLineBlocks: {
                            type: "boolean"
                        }
                    }
                }
            ]
        },
        create(context) {
            const options = {};
            const typeOptions = context.options[0] || "always";
            const exceptOptions = context.options[1] || {};
            if (typeof typeOptions === "string") {
                const shouldHavePadding = typeOptions === "always";
                options.blocks = shouldHavePadding;
                options.switches = shouldHavePadding;
                options.classes = shouldHavePadding;
            }
            else {
                if (Object.prototype.hasOwnProperty.call(typeOptions, "blocks")) {
                    options.blocks = typeOptions.blocks === "always";
                }
                if (Object.prototype.hasOwnProperty.call(typeOptions, "switches")) {
                    options.switches = typeOptions.switches === "always";
                }
                if (Object.prototype.hasOwnProperty.call(typeOptions, "classes")) {
                    options.classes = typeOptions.classes === "always";
                }
            }
            if (Object.prototype.hasOwnProperty.call(exceptOptions, "allowSingleLineBlocks")) {
                options.allowSingleLineBlocks = exceptOptions.allowSingleLineBlocks === true;
            }
            const ALWAYS_MESSAGE = "Block must be padded by blank lines.", NEVER_MESSAGE = "Block must not be padded by blank lines.";
            const sourceCode = context.getSourceCode();
            /**
             * Gets the open brace token from a given node.
             * @param {ASTNode} node A BlockStatement or SwitchStatement node from which to get the open brace.
             * @returns {Token} The token of the open brace.
             */
            function getOpenBrace(node) {
                if (node.type === "SwitchStatement") {
                    return sourceCode.getTokenBefore(node.cases[0]);
                }
                return sourceCode.getFirstToken(node);
            }
            /**
             * Checks if the given parameter is a comment node
             * @param {ASTNode|Token} node An AST node or token
             * @returns {boolean} True if node is a comment
             */
            function isComment(node) {
                return node.type === "Line" || node.type === "Block";
            }
            /**
             * Checks if there is padding between two tokens
             * @param {Token} first The first token
             * @param {Token} second The second token
             * @returns {boolean} True if there is at least a line between the tokens
             */
            function isPaddingBetweenTokens(first, second) {
                return second.loc.start.line - first.loc.end.line >= 2;
            }
            /**
             * Checks if the given token has a blank line after it.
             * @param {Token} token The token to check.
             * @returns {boolean} Whether or not the token is followed by a blank line.
             */
            function getFirstBlockToken(token) {
                let prev, first = token;
                do {
                    prev = first;
                    first = sourceCode.getTokenAfter(first, { includeComments: true });
                } while (isComment(first) && first.loc.start.line === prev.loc.end.line);
                return first;
            }
            /**
             * Checks if the given token is preceeded by a blank line.
             * @param {Token} token The token to check
             * @returns {boolean} Whether or not the token is preceeded by a blank line
             */
            function getLastBlockToken(token) {
                let last = token, next;
                do {
                    next = last;
                    last = sourceCode.getTokenBefore(last, { includeComments: true });
                } while (isComment(last) && last.loc.end.line === next.loc.start.line);
                return last;
            }
            /**
             * Checks if a node should be padded, according to the rule config.
             * @param {ASTNode} node The AST node to check.
             * @returns {boolean} True if the node should be padded, false otherwise.
             */
            function requirePaddingFor(node) {
                switch (node.type) {
                    case "BlockStatement":
                        return options.blocks;
                    case "SwitchStatement":
                        return options.switches;
                    case "ClassBody":
                        return options.classes;
                    /* istanbul ignore next */
                    default:
                        throw new Error("unreachable");
                }
            }
            /**
             * Checks the given BlockStatement node to be padded if the block is not empty.
             * @param {ASTNode} node The AST node of a BlockStatement.
             * @returns {void} undefined.
             */
            function checkPadding(node) {
                const openBrace = getOpenBrace(node), firstBlockToken = getFirstBlockToken(openBrace), tokenBeforeFirst = sourceCode.getTokenBefore(firstBlockToken, { includeComments: true }), closeBrace = sourceCode.getLastToken(node), lastBlockToken = getLastBlockToken(closeBrace), tokenAfterLast = sourceCode.getTokenAfter(lastBlockToken, { includeComments: true }), blockHasTopPadding = isPaddingBetweenTokens(tokenBeforeFirst, firstBlockToken), blockHasBottomPadding = isPaddingBetweenTokens(lastBlockToken, tokenAfterLast);
                if (options.allowSingleLineBlocks && astUtils.isTokenOnSameLine(tokenBeforeFirst, tokenAfterLast)) {
                    return;
                }
                if (requirePaddingFor(node)) {
                    if (!blockHasTopPadding) {
                        context.report({
                            node,
                            loc: { line: tokenBeforeFirst.loc.start.line, column: tokenBeforeFirst.loc.start.column },
                            fix(fixer) {
                                return fixer.insertTextAfter(tokenBeforeFirst, "\n");
                            },
                            message: ALWAYS_MESSAGE
                        });
                    }
                    if (!blockHasBottomPadding) {
                        context.report({
                            node,
                            loc: { line: tokenAfterLast.loc.end.line, column: tokenAfterLast.loc.end.column - 1 },
                            fix(fixer) {
                                return fixer.insertTextBefore(tokenAfterLast, "\n");
                            },
                            message: ALWAYS_MESSAGE
                        });
                    }
                }
                else {
                    if (blockHasTopPadding) {
                        context.report({
                            node,
                            loc: { line: tokenBeforeFirst.loc.start.line, column: tokenBeforeFirst.loc.start.column },
                            fix(fixer) {
                                return fixer.replaceTextRange([tokenBeforeFirst.range[1], firstBlockToken.range[0] - firstBlockToken.loc.start.column], "\n");
                            },
                            message: NEVER_MESSAGE
                        });
                    }
                    if (blockHasBottomPadding) {
                        context.report({
                            node,
                            loc: { line: tokenAfterLast.loc.end.line, column: tokenAfterLast.loc.end.column - 1 },
                            message: NEVER_MESSAGE,
                            fix(fixer) {
                                return fixer.replaceTextRange([lastBlockToken.range[1], tokenAfterLast.range[0] - tokenAfterLast.loc.start.column], "\n");
                            }
                        });
                    }
                }
            }
            const rule = {};
            if (Object.prototype.hasOwnProperty.call(options, "switches")) {
                rule.SwitchStatement = function (node) {
                    if (node.cases.length === 0) {
                        return;
                    }
                    checkPadding(node);
                };
            }
            if (Object.prototype.hasOwnProperty.call(options, "blocks")) {
                rule.BlockStatement = function (node) {
                    if (node.body.length === 0) {
                        return;
                    }
                    checkPadding(node);
                };
            }
            if (Object.prototype.hasOwnProperty.call(options, "classes")) {
                rule.ClassBody = function (node) {
                    if (node.body.length === 0) {
                        return;
                    }
                    checkPadding(node);
                };
            }
            return rule;
        }
    };
},
/* eslint/lib/rules/padding-line-between-statements.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const LT = `[${Array.from(astUtils.LINEBREAKS).join("")}]`;
    const PADDING_LINE_SEQUENCE = new RegExp(String.raw `^(\s*?${LT})\s*${LT}(\s*;?)$`, "u");
    const CJS_EXPORT = /^(?:module\s*\.\s*)?exports(?:\s*\.|\s*\[|$)/u;
    const CJS_IMPORT = /^require\(/u;
    /**
     * Creates tester which check if a node starts with specific keyword.
     * @param {string} keyword The keyword to test.
     * @returns {Object} the created tester.
     * @private
     */
    function newKeywordTester(keyword) {
        return {
            test: (node, sourceCode) => sourceCode.getFirstToken(node).value === keyword
        };
    }
    /**
     * Creates tester which check if a node starts with specific keyword and spans a single line.
     * @param {string} keyword The keyword to test.
     * @returns {Object} the created tester.
     * @private
     */
    function newSinglelineKeywordTester(keyword) {
        return {
            test: (node, sourceCode) => node.loc.start.line === node.loc.end.line &&
                sourceCode.getFirstToken(node).value === keyword
        };
    }
    /**
     * Creates tester which check if a node starts with specific keyword and spans multiple lines.
     * @param {string} keyword The keyword to test.
     * @returns {Object} the created tester.
     * @private
     */
    function newMultilineKeywordTester(keyword) {
        return {
            test: (node, sourceCode) => node.loc.start.line !== node.loc.end.line &&
                sourceCode.getFirstToken(node).value === keyword
        };
    }
    /**
     * Creates tester which check if a node is specific type.
     * @param {string} type The node type to test.
     * @returns {Object} the created tester.
     * @private
     */
    function newNodeTypeTester(type) {
        return {
            test: node => node.type === type
        };
    }
    /**
     * Checks the given node is an expression statement of IIFE.
     * @param {ASTNode} node The node to check.
     * @returns {boolean} `true` if the node is an expression statement of IIFE.
     * @private
     */
    function isIIFEStatement(node) {
        if (node.type === "ExpressionStatement") {
            let call = node.expression;
            if (call.type === "UnaryExpression") {
                call = call.argument;
            }
            return call.type === "CallExpression" && astUtils.isFunction(call.callee);
        }
        return false;
    }
    /**
     * Checks whether the given node is a block-like statement.
     * This checks the last token of the node is the closing brace of a block.
     * @param {SourceCode} sourceCode The source code to get tokens.
     * @param {ASTNode} node The node to check.
     * @returns {boolean} `true` if the node is a block-like statement.
     * @private
     */
    function isBlockLikeStatement(sourceCode, node) {
        // do-while with a block is a block-like statement.
        if (node.type === "DoWhileStatement" && node.body.type === "BlockStatement") {
            return true;
        }
        /*
         * IIFE is a block-like statement specially from
         * JSCS#disallowPaddingNewLinesAfterBlocks.
         */
        if (isIIFEStatement(node)) {
            return true;
        }
        // Checks the last token is a closing brace of blocks.
        const lastToken = sourceCode.getLastToken(node, astUtils.isNotSemicolonToken);
        const belongingNode = lastToken && astUtils.isClosingBraceToken(lastToken)
            ? sourceCode.getNodeByRangeIndex(lastToken.range[0])
            : null;
        return Boolean(belongingNode) && (belongingNode.type === "BlockStatement" ||
            belongingNode.type === "SwitchStatement");
    }
    /**
     * Check whether the given node is a directive or not.
     * @param {ASTNode} node The node to check.
     * @param {SourceCode} sourceCode The source code object to get tokens.
     * @returns {boolean} `true` if the node is a directive.
     */
    function isDirective(node, sourceCode) {
        return (node.type === "ExpressionStatement" &&
            (node.parent.type === "Program" ||
                (node.parent.type === "BlockStatement" &&
                    astUtils.isFunction(node.parent.parent))) &&
            node.expression.type === "Literal" &&
            typeof node.expression.value === "string" &&
            !astUtils.isParenthesised(sourceCode, node.expression));
    }
    /**
     * Check whether the given node is a part of directive prologue or not.
     * @param {ASTNode} node The node to check.
     * @param {SourceCode} sourceCode The source code object to get tokens.
     * @returns {boolean} `true` if the node is a part of directive prologue.
     */
    function isDirectivePrologue(node, sourceCode) {
        if (isDirective(node, sourceCode)) {
            for (const sibling of node.parent.body) {
                if (sibling === node) {
                    break;
                }
                if (!isDirective(sibling, sourceCode)) {
                    return false;
                }
            }
            return true;
        }
        return false;
    }
    /**
     * Gets the actual last token.
     *
     * If a semicolon is semicolon-less style's semicolon, this ignores it.
     * For example:
     *
     *     foo()
     *     ;[1, 2, 3].forEach(bar)
     * @param {SourceCode} sourceCode The source code to get tokens.
     * @param {ASTNode} node The node to get.
     * @returns {Token} The actual last token.
     * @private
     */
    function getActualLastToken(sourceCode, node) {
        const semiToken = sourceCode.getLastToken(node);
        const prevToken = sourceCode.getTokenBefore(semiToken);
        const nextToken = sourceCode.getTokenAfter(semiToken);
        const isSemicolonLessStyle = Boolean(prevToken &&
            nextToken &&
            prevToken.range[0] >= node.range[0] &&
            astUtils.isSemicolonToken(semiToken) &&
            semiToken.loc.start.line !== prevToken.loc.end.line &&
            semiToken.loc.end.line === nextToken.loc.start.line);
        return isSemicolonLessStyle ? prevToken : semiToken;
    }
    /**
     * This returns the concatenation of the first 2 captured strings.
     * @param {string} _ Unused. Whole matched string.
     * @param {string} trailingSpaces The trailing spaces of the first line.
     * @param {string} indentSpaces The indentation spaces of the last line.
     * @returns {string} The concatenation of trailingSpaces and indentSpaces.
     * @private
     */
    function replacerToRemovePaddingLines(_, trailingSpaces, indentSpaces) {
        return trailingSpaces + indentSpaces;
    }
    /**
     * Check and report statements for `any` configuration.
     * It does nothing.
     * @returns {void}
     * @private
     */
    function verifyForAny() {
    }
    /**
     * Check and report statements for `never` configuration.
     * This autofix removes blank lines between the given 2 statements.
     * However, if comments exist between 2 blank lines, it does not remove those
     * blank lines automatically.
     * @param {RuleContext} context The rule context to report.
     * @param {ASTNode} _ Unused. The previous node to check.
     * @param {ASTNode} nextNode The next node to check.
     * @param {Array<Token[]>} paddingLines The array of token pairs that blank
     * lines exist between the pair.
     * @returns {void}
     * @private
     */
    function verifyForNever(context, _, nextNode, paddingLines) {
        if (paddingLines.length === 0) {
            return;
        }
        context.report({
            node: nextNode,
            message: "Unexpected blank line before this statement.",
            fix(fixer) {
                if (paddingLines.length >= 2) {
                    return null;
                }
                const prevToken = paddingLines[0][0];
                const nextToken = paddingLines[0][1];
                const start = prevToken.range[1];
                const end = nextToken.range[0];
                const text = context.getSourceCode().text
                    .slice(start, end)
                    .replace(PADDING_LINE_SEQUENCE, replacerToRemovePaddingLines);
                return fixer.replaceTextRange([start, end], text);
            }
        });
    }
    /**
     * Check and report statements for `always` configuration.
     * This autofix inserts a blank line between the given 2 statements.
     * If the `prevNode` has trailing comments, it inserts a blank line after the
     * trailing comments.
     * @param {RuleContext} context The rule context to report.
     * @param {ASTNode} prevNode The previous node to check.
     * @param {ASTNode} nextNode The next node to check.
     * @param {Array<Token[]>} paddingLines The array of token pairs that blank
     * lines exist between the pair.
     * @returns {void}
     * @private
     */
    function verifyForAlways(context, prevNode, nextNode, paddingLines) {
        if (paddingLines.length > 0) {
            return;
        }
        context.report({
            node: nextNode,
            message: "Expected blank line before this statement.",
            fix(fixer) {
                const sourceCode = context.getSourceCode();
                let prevToken = getActualLastToken(sourceCode, prevNode);
                const nextToken = sourceCode.getFirstTokenBetween(prevToken, nextNode, {
                    includeComments: true,
                    /**
                     * Skip the trailing comments of the previous node.
                     * This inserts a blank line after the last trailing comment.
                     *
                     * For example:
                     *
                     *     foo(); // trailing comment.
                     *     // comment.
                     *     bar();
                     *
                     * Get fixed to:
                     *
                     *     foo(); // trailing comment.
                     *
                     *     // comment.
                     *     bar();
                     * @param {Token} token The token to check.
                     * @returns {boolean} `true` if the token is not a trailing comment.
                     * @private
                     */
                    filter(token) {
                        if (astUtils.isTokenOnSameLine(prevToken, token)) {
                            prevToken = token;
                            return false;
                        }
                        return true;
                    }
                }) || nextNode;
                const insertText = astUtils.isTokenOnSameLine(prevToken, nextToken)
                    ? "\n\n"
                    : "\n";
                return fixer.insertTextAfter(prevToken, insertText);
            }
        });
    }
    /**
     * Types of blank lines.
     * `any`, `never`, and `always` are defined.
     * Those have `verify` method to check and report statements.
     * @private
     */
    const PaddingTypes = {
        any: { verify: verifyForAny },
        never: { verify: verifyForNever },
        always: { verify: verifyForAlways }
    };
    /**
     * Types of statements.
     * Those have `test` method to check it matches to the given statement.
     * @private
     */
    const StatementTypes = {
        "*": { test: () => true },
        "block-like": {
            test: (node, sourceCode) => isBlockLikeStatement(sourceCode, node)
        },
        "cjs-export": {
            test: (node, sourceCode) => node.type === "ExpressionStatement" &&
                node.expression.type === "AssignmentExpression" &&
                CJS_EXPORT.test(sourceCode.getText(node.expression.left))
        },
        "cjs-import": {
            test: (node, sourceCode) => node.type === "VariableDeclaration" &&
                node.declarations.length > 0 &&
                Boolean(node.declarations[0].init) &&
                CJS_IMPORT.test(sourceCode.getText(node.declarations[0].init))
        },
        directive: {
            test: isDirectivePrologue
        },
        expression: {
            test: (node, sourceCode) => node.type === "ExpressionStatement" &&
                !isDirectivePrologue(node, sourceCode)
        },
        iife: {
            test: isIIFEStatement
        },
        "multiline-block-like": {
            test: (node, sourceCode) => node.loc.start.line !== node.loc.end.line &&
                isBlockLikeStatement(sourceCode, node)
        },
        "multiline-expression": {
            test: (node, sourceCode) => node.loc.start.line !== node.loc.end.line &&
                node.type === "ExpressionStatement" &&
                !isDirectivePrologue(node, sourceCode)
        },
        "multiline-const": newMultilineKeywordTester("const"),
        "multiline-let": newMultilineKeywordTester("let"),
        "multiline-var": newMultilineKeywordTester("var"),
        "singleline-const": newSinglelineKeywordTester("const"),
        "singleline-let": newSinglelineKeywordTester("let"),
        "singleline-var": newSinglelineKeywordTester("var"),
        block: newNodeTypeTester("BlockStatement"),
        empty: newNodeTypeTester("EmptyStatement"),
        function: newNodeTypeTester("FunctionDeclaration"),
        break: newKeywordTester("break"),
        case: newKeywordTester("case"),
        class: newKeywordTester("class"),
        const: newKeywordTester("const"),
        continue: newKeywordTester("continue"),
        debugger: newKeywordTester("debugger"),
        default: newKeywordTester("default"),
        do: newKeywordTester("do"),
        export: newKeywordTester("export"),
        for: newKeywordTester("for"),
        if: newKeywordTester("if"),
        import: newKeywordTester("import"),
        let: newKeywordTester("let"),
        return: newKeywordTester("return"),
        switch: newKeywordTester("switch"),
        throw: newKeywordTester("throw"),
        try: newKeywordTester("try"),
        var: newKeywordTester("var"),
        while: newKeywordTester("while"),
        with: newKeywordTester("with")
    };
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow padding lines between statements",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/padding-line-between-statements"
            },
            fixable: "whitespace",
            schema: {
                definitions: {
                    paddingType: {
                        enum: Object.keys(PaddingTypes)
                    },
                    statementType: {
                        anyOf: [
                            { enum: Object.keys(StatementTypes) },
                            {
                                type: "array",
                                items: { enum: Object.keys(StatementTypes) },
                                minItems: 1,
                                uniqueItems: true,
                                additionalItems: false
                            }
                        ]
                    }
                },
                type: "array",
                items: {
                    type: "object",
                    properties: {
                        blankLine: { $ref: "#/definitions/paddingType" },
                        prev: { $ref: "#/definitions/statementType" },
                        next: { $ref: "#/definitions/statementType" }
                    },
                    additionalProperties: false,
                    required: ["blankLine", "prev", "next"]
                },
                additionalItems: false
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const configureList = context.options || [];
            let scopeInfo = null;
            /**
             * Processes to enter to new scope.
             * This manages the current previous statement.
             * @returns {void}
             * @private
             */
            function enterScope() {
                scopeInfo = {
                    upper: scopeInfo,
                    prevNode: null
                };
            }
            /**
             * Processes to exit from the current scope.
             * @returns {void}
             * @private
             */
            function exitScope() {
                scopeInfo = scopeInfo.upper;
            }
            /**
             * Checks whether the given node matches the given type.
             * @param {ASTNode} node The statement node to check.
             * @param {string|string[]} type The statement type to check.
             * @returns {boolean} `true` if the statement node matched the type.
             * @private
             */
            function match(node, type) {
                let innerStatementNode = node;
                while (innerStatementNode.type === "LabeledStatement") {
                    innerStatementNode = innerStatementNode.body;
                }
                if (Array.isArray(type)) {
                    return type.some(match.bind(null, innerStatementNode));
                }
                return StatementTypes[type].test(innerStatementNode, sourceCode);
            }
            /**
             * Finds the last matched configure from configureList.
             * @param {ASTNode} prevNode The previous statement to match.
             * @param {ASTNode} nextNode The current statement to match.
             * @returns {Object} The tester of the last matched configure.
             * @private
             */
            function getPaddingType(prevNode, nextNode) {
                for (let i = configureList.length - 1; i >= 0; --i) {
                    const configure = configureList[i];
                    const matched = match(prevNode, configure.prev) &&
                        match(nextNode, configure.next);
                    if (matched) {
                        return PaddingTypes[configure.blankLine];
                    }
                }
                return PaddingTypes.any;
            }
            /**
             * Gets padding line sequences between the given 2 statements.
             * Comments are separators of the padding line sequences.
             * @param {ASTNode} prevNode The previous statement to count.
             * @param {ASTNode} nextNode The current statement to count.
             * @returns {Array<Token[]>} The array of token pairs.
             * @private
             */
            function getPaddingLineSequences(prevNode, nextNode) {
                const pairs = [];
                let prevToken = getActualLastToken(sourceCode, prevNode);
                if (nextNode.loc.start.line - prevToken.loc.end.line >= 2) {
                    do {
                        const token = sourceCode.getTokenAfter(prevToken, { includeComments: true });
                        if (token.loc.start.line - prevToken.loc.end.line >= 2) {
                            pairs.push([prevToken, token]);
                        }
                        prevToken = token;
                    } while (prevToken.range[0] < nextNode.range[0]);
                }
                return pairs;
            }
            /**
             * Verify padding lines between the given node and the previous node.
             * @param {ASTNode} node The node to verify.
             * @returns {void}
             * @private
             */
            function verify(node) {
                const parentType = node.parent.type;
                const validParent = astUtils.STATEMENT_LIST_PARENTS.has(parentType) ||
                    parentType === "SwitchStatement";
                if (!validParent) {
                    return;
                }
                // Save this node as the current previous statement.
                const prevNode = scopeInfo.prevNode;
                // Verify.
                if (prevNode) {
                    const type = getPaddingType(prevNode, node);
                    const paddingLines = getPaddingLineSequences(prevNode, node);
                    type.verify(context, prevNode, node, paddingLines);
                }
                scopeInfo.prevNode = node;
            }
            /**
             * Verify padding lines between the given node and the previous node.
             * Then process to enter to new scope.
             * @param {ASTNode} node The node to verify.
             * @returns {void}
             * @private
             */
            function verifyThenEnterScope(node) {
                verify(node);
                enterScope();
            }
            return {
                Program: enterScope,
                BlockStatement: enterScope,
                SwitchStatement: enterScope,
                "Program:exit": exitScope,
                "BlockStatement:exit": exitScope,
                "SwitchStatement:exit": exitScope,
                ":statement": verify,
                SwitchCase: verifyThenEnterScope,
                "SwitchCase:exit": exitScope
            };
        }
    };
},
/* eslint/lib/rules/prefer-arrow-callback.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a given variable is a function name.
     * @param {eslint-scope.Variable} variable A variable to check.
     * @returns {boolean} `true` if the variable is a function name.
     */
    function isFunctionName(variable) {
        return variable && variable.defs[0].type === "FunctionName";
    }
    /**
     * Checks whether or not a given MetaProperty node equals to a given value.
     * @param {ASTNode} node A MetaProperty node to check.
     * @param {string} metaName The name of `MetaProperty.meta`.
     * @param {string} propertyName The name of `MetaProperty.property`.
     * @returns {boolean} `true` if the node is the specific value.
     */
    function checkMetaProperty(node, metaName, propertyName) {
        return node.meta.name === metaName && node.property.name === propertyName;
    }
    /**
     * Gets the variable object of `arguments` which is defined implicitly.
     * @param {eslint-scope.Scope} scope A scope to get.
     * @returns {eslint-scope.Variable} The found variable object.
     */
    function getVariableOfArguments(scope) {
        const variables = scope.variables;
        for (let i = 0; i < variables.length; ++i) {
            const variable = variables[i];
            if (variable.name === "arguments") {
                /*
                 * If there was a parameter which is named "arguments", the
                 * implicit "arguments" is not defined.
                 * So does fast return with null.
                 */
                return (variable.identifiers.length === 0) ? variable : null;
            }
        }
        /* istanbul ignore next */
        return null;
    }
    /**
     * Checkes whether or not a given node is a callback.
     * @param {ASTNode} node A node to check.
     * @returns {Object}
     *   {boolean} retv.isCallback - `true` if the node is a callback.
     *   {boolean} retv.isLexicalThis - `true` if the node is with `.bind(this)`.
     */
    function getCallbackInfo(node) {
        const retv = { isCallback: false, isLexicalThis: false };
        let currentNode = node;
        let parent = node.parent;
        while (currentNode) {
            switch (parent.type) {
                // Checks parents recursively.
                case "LogicalExpression":
                case "ConditionalExpression":
                    break;
                // Checks whether the parent node is `.bind(this)` call.
                case "MemberExpression":
                    if (parent.object === currentNode &&
                        !parent.property.computed &&
                        parent.property.type === "Identifier" &&
                        parent.property.name === "bind" &&
                        parent.parent.type === "CallExpression" &&
                        parent.parent.callee === parent) {
                        retv.isLexicalThis = (parent.parent.arguments.length === 1 &&
                            parent.parent.arguments[0].type === "ThisExpression");
                        parent = parent.parent;
                    }
                    else {
                        return retv;
                    }
                    break;
                // Checks whether the node is a callback.
                case "CallExpression":
                case "NewExpression":
                    if (parent.callee !== currentNode) {
                        retv.isCallback = true;
                    }
                    return retv;
                default:
                    return retv;
            }
            currentNode = parent;
            parent = parent.parent;
        }
        /* istanbul ignore next */
        throw new Error("unreachable");
    }
    /**
     * Checks whether a simple list of parameters contains any duplicates. This does not handle complex
     * parameter lists (e.g. with destructuring), since complex parameter lists are a SyntaxError with duplicate
     * parameter names anyway. Instead, it always returns `false` for complex parameter lists.
     * @param {ASTNode[]} paramsList The list of parameters for a function
     * @returns {boolean} `true` if the list of parameters contains any duplicates
     */
    function hasDuplicateParams(paramsList) {
        return paramsList.every(param => param.type === "Identifier") && paramsList.length !== new Set(paramsList.map(param => param.name)).size;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require using arrow functions for callbacks",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-arrow-callback"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        allowNamedFunctions: {
                            type: "boolean",
                            default: false
                        },
                        allowUnboundThis: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "code"
        },
        create(context) {
            const options = context.options[0] || {};
            const allowUnboundThis = options.allowUnboundThis !== false; // default to true
            const allowNamedFunctions = options.allowNamedFunctions;
            const sourceCode = context.getSourceCode();
            /*
             * {Array<{this: boolean, super: boolean, meta: boolean}>}
             * - this - A flag which shows there are one or more ThisExpression.
             * - super - A flag which shows there are one or more Super.
             * - meta - A flag which shows there are one or more MethProperty.
             */
            let stack = [];
            /**
             * Pushes new function scope with all `false` flags.
             * @returns {void}
             */
            function enterScope() {
                stack.push({ this: false, super: false, meta: false });
            }
            /**
             * Pops a function scope from the stack.
             * @returns {{this: boolean, super: boolean, meta: boolean}} The information of the last scope.
             */
            function exitScope() {
                return stack.pop();
            }
            return {
                // Reset internal state.
                Program() {
                    stack = [];
                },
                // If there are below, it cannot replace with arrow functions merely.
                ThisExpression() {
                    const info = stack[stack.length - 1];
                    if (info) {
                        info.this = true;
                    }
                },
                Super() {
                    const info = stack[stack.length - 1];
                    if (info) {
                        info.super = true;
                    }
                },
                MetaProperty(node) {
                    const info = stack[stack.length - 1];
                    if (info && checkMetaProperty(node, "new", "target")) {
                        info.meta = true;
                    }
                },
                // To skip nested scopes.
                FunctionDeclaration: enterScope,
                "FunctionDeclaration:exit": exitScope,
                // Main.
                FunctionExpression: enterScope,
                "FunctionExpression:exit"(node) {
                    const scopeInfo = exitScope();
                    // Skip named function expressions
                    if (allowNamedFunctions && node.id && node.id.name) {
                        return;
                    }
                    // Skip generators.
                    if (node.generator) {
                        return;
                    }
                    // Skip recursive functions.
                    const nameVar = context.getDeclaredVariables(node)[0];
                    if (isFunctionName(nameVar) && nameVar.references.length > 0) {
                        return;
                    }
                    // Skip if it's using arguments.
                    const variable = getVariableOfArguments(context.getScope());
                    if (variable && variable.references.length > 0) {
                        return;
                    }
                    // Reports if it's a callback which can replace with arrows.
                    const callbackInfo = getCallbackInfo(node);
                    if (callbackInfo.isCallback &&
                        (!allowUnboundThis || !scopeInfo.this || callbackInfo.isLexicalThis) &&
                        !scopeInfo.super &&
                        !scopeInfo.meta) {
                        context.report({
                            node,
                            message: "Unexpected function expression.",
                            fix(fixer) {
                                if ((!callbackInfo.isLexicalThis && scopeInfo.this) || hasDuplicateParams(node.params)) {
                                    /*
                                     * If the callback function does not have .bind(this) and contains a reference to `this`, there
                                     * is no way to determine what `this` should be, so don't perform any fixes.
                                     * If the callback function has duplicates in its list of parameters (possible in sloppy mode),
                                     * don't replace it with an arrow function, because this is a SyntaxError with arrow functions.
                                     */
                                    return null;
                                }
                                const paramsLeftParen = node.params.length ? sourceCode.getTokenBefore(node.params[0]) : sourceCode.getTokenBefore(node.body, 1);
                                const paramsRightParen = sourceCode.getTokenBefore(node.body);
                                const asyncKeyword = node.async ? "async " : "";
                                const paramsFullText = sourceCode.text.slice(paramsLeftParen.range[0], paramsRightParen.range[1]);
                                const arrowFunctionText = `${asyncKeyword}${paramsFullText} => ${sourceCode.getText(node.body)}`;
                                /*
                                 * If the callback function has `.bind(this)`, replace it with an arrow function and remove the binding.
                                 * Otherwise, just replace the arrow function itself.
                                 */
                                const replacedNode = callbackInfo.isLexicalThis ? node.parent.parent : node;
                                /*
                                 * If the replaced node is part of a BinaryExpression, LogicalExpression, or MemberExpression, then
                                 * the arrow function needs to be parenthesized, because `foo || () => {}` is invalid syntax even
                                 * though `foo || function() {}` is valid.
                                 */
                                const needsParens = replacedNode.parent.type !== "CallExpression" && replacedNode.parent.type !== "ConditionalExpression";
                                const replacementText = needsParens ? `(${arrowFunctionText})` : arrowFunctionText;
                                return fixer.replaceText(replacedNode, replacementText);
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/prefer-const.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const PATTERN_TYPE = /^(?:.+?Pattern|RestElement|SpreadProperty|ExperimentalRestProperty|Property)$/u;
    const DECLARATION_HOST_TYPE = /^(?:Program|BlockStatement|SwitchCase)$/u;
    const DESTRUCTURING_HOST_TYPE = /^(?:VariableDeclarator|AssignmentExpression)$/u;
    /**
     * Checks whether a given node is located at `ForStatement.init` or not.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node is located at `ForStatement.init`.
     */
    function isInitOfForStatement(node) {
        return node.parent.type === "ForStatement" && node.parent.init === node;
    }
    /**
     * Checks whether a given Identifier node becomes a VariableDeclaration or not.
     * @param {ASTNode} identifier An Identifier node to check.
     * @returns {boolean} `true` if the node can become a VariableDeclaration.
     */
    function canBecomeVariableDeclaration(identifier) {
        let node = identifier.parent;
        while (PATTERN_TYPE.test(node.type)) {
            node = node.parent;
        }
        return (node.type === "VariableDeclarator" ||
            (node.type === "AssignmentExpression" &&
                node.parent.type === "ExpressionStatement" &&
                DECLARATION_HOST_TYPE.test(node.parent.parent.type)));
    }
    /**
     * Checks if an property or element is from outer scope or function parameters
     * in destructing pattern.
     * @param {string} name A variable name to be checked.
     * @param {eslint-scope.Scope} initScope A scope to start find.
     * @returns {boolean} Indicates if the variable is from outer scope or function parameters.
     */
    function isOuterVariableInDestructing(name, initScope) {
        if (initScope.through.find(ref => ref.resolved && ref.resolved.name === name)) {
            return true;
        }
        const variable = astUtils.getVariableByName(initScope, name);
        if (variable !== null) {
            return variable.defs.some(def => def.type === "Parameter");
        }
        return false;
    }
    /**
     * Gets the VariableDeclarator/AssignmentExpression node that a given reference
     * belongs to.
     * This is used to detect a mix of reassigned and never reassigned in a
     * destructuring.
     * @param {eslint-scope.Reference} reference A reference to get.
     * @returns {ASTNode|null} A VariableDeclarator/AssignmentExpression node or
     *      null.
     */
    function getDestructuringHost(reference) {
        if (!reference.isWrite()) {
            return null;
        }
        let node = reference.identifier.parent;
        while (PATTERN_TYPE.test(node.type)) {
            node = node.parent;
        }
        if (!DESTRUCTURING_HOST_TYPE.test(node.type)) {
            return null;
        }
        return node;
    }
    /**
     * Determines if a destructuring assignment node contains
     * any MemberExpression nodes. This is used to determine if a
     * variable that is only written once using destructuring can be
     * safely converted into a const declaration.
     * @param {ASTNode} node The ObjectPattern or ArrayPattern node to check.
     * @returns {boolean} True if the destructuring pattern contains
     *      a MemberExpression, false if not.
     */
    function hasMemberExpressionAssignment(node) {
        switch (node.type) {
            case "ObjectPattern":
                return node.properties.some(prop => {
                    if (prop) {
                        /*
                         * Spread elements have an argument property while
                         * others have a value property. Because different
                         * parsers use different node types for spread elements,
                         * we just check if there is an argument property.
                         */
                        return hasMemberExpressionAssignment(prop.argument || prop.value);
                    }
                    return false;
                });
            case "ArrayPattern":
                return node.elements.some(element => {
                    if (element) {
                        return hasMemberExpressionAssignment(element);
                    }
                    return false;
                });
            case "AssignmentPattern":
                return hasMemberExpressionAssignment(node.left);
            case "MemberExpression":
                return true;
            // no default
        }
        return false;
    }
    /**
     * Gets an identifier node of a given variable.
     *
     * If the initialization exists or one or more reading references exist before
     * the first assignment, the identifier node is the node of the declaration.
     * Otherwise, the identifier node is the node of the first assignment.
     *
     * If the variable should not change to const, this function returns null.
     * - If the variable is reassigned.
     * - If the variable is never initialized nor assigned.
     * - If the variable is initialized in a different scope from the declaration.
     * - If the unique assignment of the variable cannot change to a declaration.
     *   e.g. `if (a) b = 1` / `return (b = 1)`
     * - If the variable is declared in the global scope and `eslintUsed` is `true`.
     *   `/*exported foo` directive comment makes such variables. This rule does not
     *   warn such variables because this rule cannot distinguish whether the
     *   exported variables are reassigned or not.
     * @param {eslint-scope.Variable} variable A variable to get.
     * @param {boolean} ignoreReadBeforeAssign
     *      The value of `ignoreReadBeforeAssign` option.
     * @returns {ASTNode|null}
     *      An Identifier node if the variable should change to const.
     *      Otherwise, null.
     */
    function getIdentifierIfShouldBeConst(variable, ignoreReadBeforeAssign) {
        if (variable.eslintUsed && variable.scope.type === "global") {
            return null;
        }
        // Finds the unique WriteReference.
        let writer = null;
        let isReadBeforeInit = false;
        const references = variable.references;
        for (let i = 0; i < references.length; ++i) {
            const reference = references[i];
            if (reference.isWrite()) {
                const isReassigned = (writer !== null &&
                    writer.identifier !== reference.identifier);
                if (isReassigned) {
                    return null;
                }
                const destructuringHost = getDestructuringHost(reference);
                if (destructuringHost !== null && destructuringHost.left !== void 0) {
                    const leftNode = destructuringHost.left;
                    let hasOuterVariables = false, hasNonIdentifiers = false;
                    if (leftNode.type === "ObjectPattern") {
                        const properties = leftNode.properties;
                        hasOuterVariables = properties
                            .filter(prop => prop.value)
                            .map(prop => prop.value.name)
                            .some(name => isOuterVariableInDestructing(name, variable.scope));
                        hasNonIdentifiers = hasMemberExpressionAssignment(leftNode);
                    }
                    else if (leftNode.type === "ArrayPattern") {
                        const elements = leftNode.elements;
                        hasOuterVariables = elements
                            .map(element => element && element.name)
                            .some(name => isOuterVariableInDestructing(name, variable.scope));
                        hasNonIdentifiers = hasMemberExpressionAssignment(leftNode);
                    }
                    if (hasOuterVariables || hasNonIdentifiers) {
                        return null;
                    }
                }
                writer = reference;
            }
            else if (reference.isRead() && writer === null) {
                if (ignoreReadBeforeAssign) {
                    return null;
                }
                isReadBeforeInit = true;
            }
        }
        /*
         * If the assignment is from a different scope, ignore it.
         * If the assignment cannot change to a declaration, ignore it.
         */
        const shouldBeConst = (writer !== null &&
            writer.from === variable.scope &&
            canBecomeVariableDeclaration(writer.identifier));
        if (!shouldBeConst) {
            return null;
        }
        if (isReadBeforeInit) {
            return variable.defs[0].name;
        }
        return writer.identifier;
    }
    /**
     * Groups by the VariableDeclarator/AssignmentExpression node that each
     * reference of given variables belongs to.
     * This is used to detect a mix of reassigned and never reassigned in a
     * destructuring.
     * @param {eslint-scope.Variable[]} variables Variables to group by destructuring.
     * @param {boolean} ignoreReadBeforeAssign
     *      The value of `ignoreReadBeforeAssign` option.
     * @returns {Map<ASTNode, ASTNode[]>} Grouped identifier nodes.
     */
    function groupByDestructuring(variables, ignoreReadBeforeAssign) {
        const identifierMap = new Map();
        for (let i = 0; i < variables.length; ++i) {
            const variable = variables[i];
            const references = variable.references;
            const identifier = getIdentifierIfShouldBeConst(variable, ignoreReadBeforeAssign);
            let prevId = null;
            for (let j = 0; j < references.length; ++j) {
                const reference = references[j];
                const id = reference.identifier;
                /*
                 * Avoid counting a reference twice or more for default values of
                 * destructuring.
                 */
                if (id === prevId) {
                    continue;
                }
                prevId = id;
                // Add the identifier node into the destructuring group.
                const group = getDestructuringHost(reference);
                if (group) {
                    if (identifierMap.has(group)) {
                        identifierMap.get(group).push(identifier);
                    }
                    else {
                        identifierMap.set(group, [identifier]);
                    }
                }
            }
        }
        return identifierMap;
    }
    /**
     * Finds the nearest parent of node with a given type.
     * @param {ASTNode} node The node to search from.
     * @param {string} type The type field of the parent node.
     * @param {Function} shouldStop A predicate that returns true if the traversal should stop, and false otherwise.
     * @returns {ASTNode} The closest ancestor with the specified type; null if no such ancestor exists.
     */
    function findUp(node, type, shouldStop) {
        if (!node || shouldStop(node)) {
            return null;
        }
        if (node.type === type) {
            return node;
        }
        return findUp(node.parent, type, shouldStop);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require `const` declarations for variables that are never reassigned after declared",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-const"
            },
            fixable: "code",
            schema: [
                {
                    type: "object",
                    properties: {
                        destructuring: { enum: ["any", "all"], default: "any" },
                        ignoreReadBeforeAssign: { type: "boolean", default: false }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                useConst: "'{{name}}' is never reassigned. Use 'const' instead."
            }
        },
        create(context) {
            const options = context.options[0] || {};
            const sourceCode = context.getSourceCode();
            const shouldMatchAnyDestructuredVariable = options.destructuring !== "all";
            const ignoreReadBeforeAssign = options.ignoreReadBeforeAssign === true;
            const variables = [];
            let reportCount = 0;
            let checkedId = null;
            let checkedName = "";
            /**
             * Reports given identifier nodes if all of the nodes should be declared
             * as const.
             *
             * The argument 'nodes' is an array of Identifier nodes.
             * This node is the result of 'getIdentifierIfShouldBeConst()', so it's
             * nullable. In simple declaration or assignment cases, the length of
             * the array is 1. In destructuring cases, the length of the array can
             * be 2 or more.
             * @param {(eslint-scope.Reference|null)[]} nodes
             *      References which are grouped by destructuring to report.
             * @returns {void}
             */
            function checkGroup(nodes) {
                const nodesToReport = nodes.filter(Boolean);
                if (nodes.length && (shouldMatchAnyDestructuredVariable || nodesToReport.length === nodes.length)) {
                    const varDeclParent = findUp(nodes[0], "VariableDeclaration", parentNode => parentNode.type.endsWith("Statement"));
                    const isVarDecParentNull = varDeclParent === null;
                    if (!isVarDecParentNull && varDeclParent.declarations.length > 0) {
                        const firstDeclaration = varDeclParent.declarations[0];
                        if (firstDeclaration.init) {
                            const firstDecParent = firstDeclaration.init.parent;
                            /*
                             * First we check the declaration type and then depending on
                             * if the type is a "VariableDeclarator" or its an "ObjectPattern"
                             * we compare the name and id from the first identifier, if the names are different
                             * we assign the new name, id and reset the count of reportCount and nodeCount in
                             * order to check each block for the number of reported errors and base our fix
                             * based on comparing nodes.length and nodesToReport.length.
                             */
                            if (firstDecParent.type === "VariableDeclarator") {
                                if (firstDecParent.id.name !== checkedName) {
                                    checkedName = firstDecParent.id.name;
                                    reportCount = 0;
                                }
                                if (firstDecParent.id.type === "ObjectPattern") {
                                    if (firstDecParent.init.name !== checkedName) {
                                        checkedName = firstDecParent.init.name;
                                        reportCount = 0;
                                    }
                                }
                                if (firstDecParent.id !== checkedId) {
                                    checkedId = firstDecParent.id;
                                    reportCount = 0;
                                }
                            }
                        }
                    }
                    let shouldFix = varDeclParent &&
                        // Don't do a fix unless all variables in the declarations are initialized (or it's in a for-in or for-of loop)
                        (varDeclParent.parent.type === "ForInStatement" || varDeclParent.parent.type === "ForOfStatement" ||
                            varDeclParent.declarations.every(declaration => declaration.init)) &&
                        /*
                         * If options.destructuring is "all", then this warning will not occur unless
                         * every assignment in the destructuring should be const. In that case, it's safe
                         * to apply the fix.
                         */
                        nodesToReport.length === nodes.length;
                    if (!isVarDecParentNull && varDeclParent.declarations && varDeclParent.declarations.length !== 1) {
                        if (varDeclParent && varDeclParent.declarations && varDeclParent.declarations.length >= 1) {
                            /*
                             * Add nodesToReport.length to a count, then comparing the count to the length
                             * of the declarations in the current block.
                             */
                            reportCount += nodesToReport.length;
                            shouldFix = shouldFix && (reportCount === varDeclParent.declarations.length);
                        }
                    }
                    nodesToReport.forEach(node => {
                        context.report({
                            node,
                            messageId: "useConst",
                            data: node,
                            fix: shouldFix
                                ? fixer => fixer.replaceText(sourceCode.getFirstToken(varDeclParent, t => t.value === varDeclParent.kind), "const")
                                : null
                        });
                    });
                }
            }
            return {
                "Program:exit"() {
                    groupByDestructuring(variables, ignoreReadBeforeAssign).forEach(checkGroup);
                },
                VariableDeclaration(node) {
                    if (node.kind === "let" && !isInitOfForStatement(node)) {
                        variables.push(...context.getDeclaredVariables(node));
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/prefer-destructuring.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require destructuring from arrays and/or objects",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-destructuring"
            },
            fixable: "code",
            schema: [
                {
                    /*
                     * old support {array: Boolean, object: Boolean}
                     * new support {VariableDeclarator: {}, AssignmentExpression: {}}
                     */
                    oneOf: [
                        {
                            type: "object",
                            properties: {
                                VariableDeclarator: {
                                    type: "object",
                                    properties: {
                                        array: {
                                            type: "boolean"
                                        },
                                        object: {
                                            type: "boolean"
                                        }
                                    },
                                    additionalProperties: false
                                },
                                AssignmentExpression: {
                                    type: "object",
                                    properties: {
                                        array: {
                                            type: "boolean"
                                        },
                                        object: {
                                            type: "boolean"
                                        }
                                    },
                                    additionalProperties: false
                                }
                            },
                            additionalProperties: false
                        },
                        {
                            type: "object",
                            properties: {
                                array: {
                                    type: "boolean"
                                },
                                object: {
                                    type: "boolean"
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                },
                {
                    type: "object",
                    properties: {
                        enforceForRenamedProperties: {
                            type: "boolean"
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const enabledTypes = context.options[0];
            const enforceForRenamedProperties = context.options[1] && context.options[1].enforceForRenamedProperties;
            let normalizedOptions = {
                VariableDeclarator: { array: true, object: true },
                AssignmentExpression: { array: true, object: true }
            };
            if (enabledTypes) {
                normalizedOptions = typeof enabledTypes.array !== "undefined" || typeof enabledTypes.object !== "undefined"
                    ? { VariableDeclarator: enabledTypes, AssignmentExpression: enabledTypes }
                    : enabledTypes;
            }
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            // eslint-disable-next-line jsdoc/require-description
            /**
             * @param {string} nodeType "AssignmentExpression" or "VariableDeclarator"
             * @param {string} destructuringType "array" or "object"
             * @returns {boolean} `true` if the destructuring type should be checked for the given node
             */
            function shouldCheck(nodeType, destructuringType) {
                return normalizedOptions &&
                    normalizedOptions[nodeType] &&
                    normalizedOptions[nodeType][destructuringType];
            }
            /**
             * Determines if the given node is accessing an array index
             *
             * This is used to differentiate array index access from object property
             * access.
             * @param {ASTNode} node the node to evaluate
             * @returns {boolean} whether or not the node is an integer
             */
            function isArrayIndexAccess(node) {
                return Number.isInteger(node.property.value);
            }
            /**
             * Report that the given node should use destructuring
             * @param {ASTNode} reportNode the node to report
             * @param {string} type the type of destructuring that should have been done
             * @param {Function|null} fix the fix function or null to pass to context.report
             * @returns {void}
             */
            function report(reportNode, type, fix) {
                context.report({
                    node: reportNode,
                    message: "Use {{type}} destructuring.",
                    data: { type },
                    fix
                });
            }
            /**
             * Determines if a node should be fixed into object destructuring
             *
             * The fixer only fixes the simplest case of object destructuring,
             * like: `let x = a.x`;
             *
             * Assignment expression is not fixed.
             * Array destructuring is not fixed.
             * Renamed property is not fixed.
             * @param {ASTNode} node the the node to evaluate
             * @returns {boolean} whether or not the node should be fixed
             */
            function shouldFix(node) {
                return node.type === "VariableDeclarator" &&
                    node.id.type === "Identifier" &&
                    node.init.type === "MemberExpression" &&
                    node.id.name === node.init.property.name;
            }
            /**
             * Fix a node into object destructuring.
             * This function only handles the simplest case of object destructuring,
             * see {@link shouldFix}.
             * @param {SourceCodeFixer} fixer the fixer object
             * @param {ASTNode} node the node to be fixed.
             * @returns {Object} a fix for the node
             */
            function fixIntoObjectDestructuring(fixer, node) {
                const rightNode = node.init;
                const sourceCode = context.getSourceCode();
                return fixer.replaceText(node, `{${rightNode.property.name}} = ${sourceCode.getText(rightNode.object)}`);
            }
            /**
             * Check that the `prefer-destructuring` rules are followed based on the
             * given left- and right-hand side of the assignment.
             *
             * Pulled out into a separate method so that VariableDeclarators and
             * AssignmentExpressions can share the same verification logic.
             * @param {ASTNode} leftNode the left-hand side of the assignment
             * @param {ASTNode} rightNode the right-hand side of the assignment
             * @param {ASTNode} reportNode the node to report the error on
             * @returns {void}
             */
            function performCheck(leftNode, rightNode, reportNode) {
                if (rightNode.type !== "MemberExpression" || rightNode.object.type === "Super") {
                    return;
                }
                if (isArrayIndexAccess(rightNode)) {
                    if (shouldCheck(reportNode.type, "array")) {
                        report(reportNode, "array", null);
                    }
                    return;
                }
                const fix = shouldFix(reportNode)
                    ? fixer => fixIntoObjectDestructuring(fixer, reportNode)
                    : null;
                if (shouldCheck(reportNode.type, "object") && enforceForRenamedProperties) {
                    report(reportNode, "object", fix);
                    return;
                }
                if (shouldCheck(reportNode.type, "object")) {
                    const property = rightNode.property;
                    if ((property.type === "Literal" && leftNode.name === property.value) ||
                        (property.type === "Identifier" && leftNode.name === property.name && !rightNode.computed)) {
                        report(reportNode, "object", fix);
                    }
                }
            }
            /**
             * Check if a given variable declarator is coming from an property access
             * that should be using destructuring instead
             * @param {ASTNode} node the variable declarator to check
             * @returns {void}
             */
            function checkVariableDeclarator(node) {
                // Skip if variable is declared without assignment
                if (!node.init) {
                    return;
                }
                // We only care about member expressions past this point
                if (node.init.type !== "MemberExpression") {
                    return;
                }
                performCheck(node.id, node.init, node);
            }
            /**
             * Run the `prefer-destructuring` check on an AssignmentExpression
             * @param {ASTNode} node the AssignmentExpression node
             * @returns {void}
             */
            function checkAssigmentExpression(node) {
                if (node.operator === "=") {
                    performCheck(node.left, node.right, node);
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                VariableDeclarator: checkVariableDeclarator,
                AssignmentExpression: checkAssigmentExpression
            };
        }
    };
},
/* eslint/lib/rules/prefer-exponentiation-operator.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    const { CALL, ReferenceTracker } = require(250) /* eslint-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const PRECEDENCE_OF_EXPONENTIATION_EXPR = astUtils.getPrecedence({ type: "BinaryExpression", operator: "**" });
    /**
     * Determines whether the given node needs parens if used as the base in an exponentiation binary expression.
     * @param {ASTNode} base The node to check.
     * @returns {boolean} `true` if the node needs to be parenthesised.
     */
    function doesBaseNeedParens(base) {
        return (
        // '**' is right-associative, parens are needed when Math.pow(a ** b, c) is converted to (a ** b) ** c
        astUtils.getPrecedence(base) <= PRECEDENCE_OF_EXPONENTIATION_EXPR ||
            // An unary operator cannot be used immediately before an exponentiation expression
            base.type === "UnaryExpression");
    }
    /**
     * Determines whether the given node needs parens if used as the exponent in an exponentiation binary expression.
     * @param {ASTNode} exponent The node to check.
     * @returns {boolean} `true` if the node needs to be parenthesised.
     */
    function doesExponentNeedParens(exponent) {
        // '**' is right-associative, there is no need for parens when Math.pow(a, b ** c) is converted to a ** b ** c
        return astUtils.getPrecedence(exponent) < PRECEDENCE_OF_EXPONENTIATION_EXPR;
    }
    /**
     * Determines whether an exponentiation binary expression at the place of the given node would need parens.
     * @param {ASTNode} node A node that would be replaced by an exponentiation binary expression.
     * @param {SourceCode} sourceCode A SourceCode object.
     * @returns {boolean} `true` if the expression needs to be parenthesised.
     */
    function doesExponentiationExpressionNeedParens(node, sourceCode) {
        const parent = node.parent;
        const needsParens = (parent.type === "ClassDeclaration" ||
            (parent.type.endsWith("Expression") &&
                astUtils.getPrecedence(parent) >= PRECEDENCE_OF_EXPONENTIATION_EXPR &&
                !(parent.type === "BinaryExpression" && parent.operator === "**" && parent.right === node) &&
                !((parent.type === "CallExpression" || parent.type === "NewExpression") && parent.arguments.includes(node)) &&
                !(parent.type === "MemberExpression" && parent.computed && parent.property === node) &&
                !(parent.type === "ArrayExpression")));
        return needsParens && !astUtils.isParenthesised(sourceCode, node);
    }
    /**
     * Optionally parenthesizes given text.
     * @param {string} text The text to parenthesize.
     * @param {boolean} shouldParenthesize If `true`, the text will be parenthesised.
     * @returns {string} parenthesised or unchanged text.
     */
    function parenthesizeIfShould(text, shouldParenthesize) {
        return shouldParenthesize ? `(${text})` : text;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow the use of `Math.pow` in favor of the `**` operator",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-exponentiation-operator"
            },
            schema: [],
            fixable: "code",
            messages: {
                useExponentiation: "Use the '**' operator instead of 'Math.pow'."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Reports the given node.
             * @param {ASTNode} node 'Math.pow()' node to report.
             * @returns {void}
             */
            function report(node) {
                context.report({
                    node,
                    messageId: "useExponentiation",
                    fix(fixer) {
                        if (node.arguments.length !== 2 ||
                            node.arguments.some(arg => arg.type === "SpreadElement") ||
                            sourceCode.getCommentsInside(node).length > 0) {
                            return null;
                        }
                        const base = node.arguments[0], exponent = node.arguments[1], baseText = sourceCode.getText(base), exponentText = sourceCode.getText(exponent), shouldParenthesizeBase = doesBaseNeedParens(base), shouldParenthesizeExponent = doesExponentNeedParens(exponent), shouldParenthesizeAll = doesExponentiationExpressionNeedParens(node, sourceCode);
                        let prefix = "", suffix = "";
                        if (!shouldParenthesizeAll) {
                            if (!shouldParenthesizeBase) {
                                const firstReplacementToken = sourceCode.getFirstToken(base), tokenBefore = sourceCode.getTokenBefore(node);
                                if (tokenBefore &&
                                    tokenBefore.range[1] === node.range[0] &&
                                    !astUtils.canTokensBeAdjacent(tokenBefore, firstReplacementToken)) {
                                    prefix = " "; // a+Math.pow(++b, c) -> a+ ++b**c
                                }
                            }
                            if (!shouldParenthesizeExponent) {
                                const lastReplacementToken = sourceCode.getLastToken(exponent), tokenAfter = sourceCode.getTokenAfter(node);
                                if (tokenAfter &&
                                    node.range[1] === tokenAfter.range[0] &&
                                    !astUtils.canTokensBeAdjacent(lastReplacementToken, tokenAfter)) {
                                    suffix = " "; // Math.pow(a, b)in c -> a**b in c
                                }
                            }
                        }
                        const baseReplacement = parenthesizeIfShould(baseText, shouldParenthesizeBase), exponentReplacement = parenthesizeIfShould(exponentText, shouldParenthesizeExponent), replacement = parenthesizeIfShould(`${baseReplacement}**${exponentReplacement}`, shouldParenthesizeAll);
                        return fixer.replaceText(node, `${prefix}${replacement}${suffix}`);
                    }
                });
            }
            return {
                Program() {
                    const scope = context.getScope();
                    const tracker = new ReferenceTracker(scope);
                    const trackMap = {
                        Math: {
                            pow: { [CALL]: true }
                        }
                    };
                    for (const { node } of tracker.iterateGlobalReferences(trackMap)) {
                        report(node);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/prefer-named-capture-group.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const { CALL, CONSTRUCT, ReferenceTracker, getStringIfConstant } = require(250) /* eslint-utils */;
    const regexpp = require(226) /* regexpp */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const parser = new regexpp.RegExpParser();
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce using named capture group in regular expression",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-named-capture-group"
            },
            schema: [],
            messages: {
                required: "Capture group '{{group}}' should be converted to a named or non-capturing group."
            }
        },
        create(context) {
            /**
             * Function to check regular expression.
             * @param {string} pattern The regular expression pattern to be check.
             * @param {ASTNode} node AST node which contains regular expression.
             * @param {boolean} uFlag Flag indicates whether unicode mode is enabled or not.
             * @returns {void}
             */
            function checkRegex(pattern, node, uFlag) {
                let ast;
                try {
                    ast = parser.parsePattern(pattern, 0, pattern.length, uFlag);
                }
                catch (_) {
                    // ignore regex syntax errors
                    return;
                }
                regexpp.visitRegExpAST(ast, {
                    onCapturingGroupEnter(group) {
                        if (!group.name) {
                            context.report({
                                node,
                                messageId: "required",
                                data: {
                                    group: group.raw
                                }
                            });
                        }
                    }
                });
            }
            return {
                Literal(node) {
                    if (node.regex) {
                        checkRegex(node.regex.pattern, node, node.regex.flags.includes("u"));
                    }
                },
                Program() {
                    const scope = context.getScope();
                    const tracker = new ReferenceTracker(scope);
                    const traceMap = {
                        RegExp: {
                            [CALL]: true,
                            [CONSTRUCT]: true
                        }
                    };
                    for (const { node } of tracker.iterateGlobalReferences(traceMap)) {
                        const regex = getStringIfConstant(node.arguments[0]);
                        const flags = getStringIfConstant(node.arguments[1]);
                        if (regex) {
                            checkRegex(regex, node, flags && flags.includes("u"));
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/prefer-numeric-literals.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const radixMap = new Map([
        [2, { system: "binary", literalPrefix: "0b" }],
        [8, { system: "octal", literalPrefix: "0o" }],
        [16, { system: "hexadecimal", literalPrefix: "0x" }]
    ]);
    /**
     * Checks to see if a CallExpression's callee node is `parseInt` or
     * `Number.parseInt`.
     * @param {ASTNode} calleeNode The callee node to evaluate.
     * @returns {boolean} True if the callee is `parseInt` or `Number.parseInt`,
     * false otherwise.
     */
    function isParseInt(calleeNode) {
        switch (calleeNode.type) {
            case "Identifier":
                return calleeNode.name === "parseInt";
            case "MemberExpression":
                return calleeNode.object.type === "Identifier" &&
                    calleeNode.object.name === "Number" &&
                    calleeNode.property.type === "Identifier" &&
                    calleeNode.property.name === "parseInt";
            // no default
        }
        return false;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow `parseInt()` and `Number.parseInt()` in favor of binary, octal, and hexadecimal literals",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-numeric-literals"
            },
            schema: [],
            messages: {
                useLiteral: "Use {{system}} literals instead of {{functionName}}()."
            },
            fixable: "code"
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                "CallExpression[arguments.length=2]"(node) {
                    const [strNode, radixNode] = node.arguments, str = strNode.value, radix = radixNode.value;
                    if (strNode.type === "Literal" &&
                        radixNode.type === "Literal" &&
                        typeof str === "string" &&
                        typeof radix === "number" &&
                        radixMap.has(radix) &&
                        isParseInt(node.callee)) {
                        const { system, literalPrefix } = radixMap.get(radix);
                        context.report({
                            node,
                            messageId: "useLiteral",
                            data: {
                                system,
                                functionName: sourceCode.getText(node.callee)
                            },
                            fix(fixer) {
                                if (sourceCode.getCommentsInside(node).length) {
                                    return null;
                                }
                                const replacement = `${literalPrefix}${str}`;
                                if (+replacement !== parseInt(str, radix)) {
                                    /*
                                     * If the newly-produced literal would be invalid, (e.g. 0b1234),
                                     * or it would yield an incorrect parseInt result for some other reason, don't make a fix.
                                     */
                                    return null;
                                }
                                const tokenBefore = sourceCode.getTokenBefore(node), tokenAfter = sourceCode.getTokenAfter(node);
                                let prefix = "", suffix = "";
                                if (tokenBefore &&
                                    tokenBefore.range[1] === node.range[0] &&
                                    !astUtils.canTokensBeAdjacent(tokenBefore, replacement)) {
                                    prefix = " ";
                                }
                                if (tokenAfter &&
                                    node.range[1] === tokenAfter.range[0] &&
                                    !astUtils.canTokensBeAdjacent(replacement, tokenAfter)) {
                                    suffix = " ";
                                }
                                return fixer.replaceText(node, `${prefix}${replacement}${suffix}`);
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/prefer-object-spread.js */ function _(require, module, exports) {
    const { CALL, ReferenceTracker } = require(250) /* eslint-utils */;
    const { isCommaToken, isOpeningParenToken, isClosingParenToken, isParenthesised } = require(126) /* ./utils/ast-utils */;
    const ANY_SPACE = /\s/u;
    /**
     * Helper that checks if the Object.assign call has array spread
     * @param {ASTNode} node The node that the rule warns on
     * @returns {boolean} - Returns true if the Object.assign call has array spread
     */
    function hasArraySpread(node) {
        return node.arguments.some(arg => arg.type === "SpreadElement");
    }
    /**
     * Helper that checks if the node needs parentheses to be valid JS.
     * The default is to wrap the node in parentheses to avoid parsing errors.
     * @param {ASTNode} node The node that the rule warns on
     * @param {Object} sourceCode in context sourcecode object
     * @returns {boolean} - Returns true if the node needs parentheses
     */
    function needsParens(node, sourceCode) {
        const parent = node.parent;
        switch (parent.type) {
            case "VariableDeclarator":
            case "ArrayExpression":
            case "ReturnStatement":
            case "CallExpression":
            case "Property":
                return false;
            case "AssignmentExpression":
                return parent.left === node && !isParenthesised(sourceCode, node);
            default:
                return !isParenthesised(sourceCode, node);
        }
    }
    /**
     * Determines if an argument needs parentheses. The default is to not add parens.
     * @param {ASTNode} node The node to be checked.
     * @param {Object} sourceCode in context sourcecode object
     * @returns {boolean} True if the node needs parentheses
     */
    function argNeedsParens(node, sourceCode) {
        switch (node.type) {
            case "AssignmentExpression":
            case "ArrowFunctionExpression":
            case "ConditionalExpression":
                return !isParenthesised(sourceCode, node);
            default:
                return false;
        }
    }
    /**
     * Get the parenthesis tokens of a given ObjectExpression node.
     * This incldues the braces of the object literal and enclosing parentheses.
     * @param {ASTNode} node The node to get.
     * @param {Token} leftArgumentListParen The opening paren token of the argument list.
     * @param {SourceCode} sourceCode The source code object to get tokens.
     * @returns {Token[]} The parenthesis tokens of the node. This is sorted by the location.
     */
    function getParenTokens(node, leftArgumentListParen, sourceCode) {
        const parens = [sourceCode.getFirstToken(node), sourceCode.getLastToken(node)];
        let leftNext = sourceCode.getTokenBefore(node);
        let rightNext = sourceCode.getTokenAfter(node);
        // Note: don't include the parens of the argument list.
        while (leftNext &&
            rightNext &&
            leftNext.range[0] > leftArgumentListParen.range[0] &&
            isOpeningParenToken(leftNext) &&
            isClosingParenToken(rightNext)) {
            parens.push(leftNext, rightNext);
            leftNext = sourceCode.getTokenBefore(leftNext);
            rightNext = sourceCode.getTokenAfter(rightNext);
        }
        return parens.sort((a, b) => a.range[0] - b.range[0]);
    }
    /**
     * Get the range of a given token and around whitespaces.
     * @param {Token} token The token to get range.
     * @param {SourceCode} sourceCode The source code object to get tokens.
     * @returns {number} The end of the range of the token and around whitespaces.
     */
    function getStartWithSpaces(token, sourceCode) {
        const text = sourceCode.text;
        let start = token.range[0];
        // If the previous token is a line comment then skip this step to avoid commenting this token out.
        {
            const prevToken = sourceCode.getTokenBefore(token, { includeComments: true });
            if (prevToken && prevToken.type === "Line") {
                return start;
            }
        }
        // Detect spaces before the token.
        while (ANY_SPACE.test(text[start - 1] || "")) {
            start -= 1;
        }
        return start;
    }
    /**
     * Get the range of a given token and around whitespaces.
     * @param {Token} token The token to get range.
     * @param {SourceCode} sourceCode The source code object to get tokens.
     * @returns {number} The start of the range of the token and around whitespaces.
     */
    function getEndWithSpaces(token, sourceCode) {
        const text = sourceCode.text;
        let end = token.range[1];
        // Detect spaces after the token.
        while (ANY_SPACE.test(text[end] || "")) {
            end += 1;
        }
        return end;
    }
    /**
     * Autofixes the Object.assign call to use an object spread instead.
     * @param {ASTNode|null} node The node that the rule warns on, i.e. the Object.assign call
     * @param {string} sourceCode sourceCode of the Object.assign call
     * @returns {Function} autofixer - replaces the Object.assign with a spread object.
     */
    function defineFixer(node, sourceCode) {
        return function* (fixer) {
            const leftParen = sourceCode.getTokenAfter(node.callee, isOpeningParenToken);
            const rightParen = sourceCode.getLastToken(node);
            // Remove the callee `Object.assign`
            yield fixer.remove(node.callee);
            // Replace the parens of argument list to braces.
            if (needsParens(node, sourceCode)) {
                yield fixer.replaceText(leftParen, "({");
                yield fixer.replaceText(rightParen, "})");
            }
            else {
                yield fixer.replaceText(leftParen, "{");
                yield fixer.replaceText(rightParen, "}");
            }
            // Process arguments.
            for (const argNode of node.arguments) {
                const innerParens = getParenTokens(argNode, leftParen, sourceCode);
                const left = innerParens.shift();
                const right = innerParens.pop();
                if (argNode.type === "ObjectExpression") {
                    const maybeTrailingComma = sourceCode.getLastToken(argNode, 1);
                    const maybeArgumentComma = sourceCode.getTokenAfter(right);
                    /*
                     * Make bare this object literal.
                     * And remove spaces inside of the braces for better formatting.
                     */
                    for (const innerParen of innerParens) {
                        yield fixer.remove(innerParen);
                    }
                    const leftRange = [left.range[0], getEndWithSpaces(left, sourceCode)];
                    const rightRange = [
                        Math.max(getStartWithSpaces(right, sourceCode), leftRange[1]),
                        right.range[1]
                    ];
                    yield fixer.removeRange(leftRange);
                    yield fixer.removeRange(rightRange);
                    // Remove the comma of this argument if it's duplication.
                    if ((argNode.properties.length === 0 || isCommaToken(maybeTrailingComma)) &&
                        isCommaToken(maybeArgumentComma)) {
                        yield fixer.remove(maybeArgumentComma);
                    }
                }
                else {
                    // Make spread.
                    if (argNeedsParens(argNode, sourceCode)) {
                        yield fixer.insertTextBefore(left, "...(");
                        yield fixer.insertTextAfter(right, ")");
                    }
                    else {
                        yield fixer.insertTextBefore(left, "...");
                    }
                }
            }
        };
    }
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow using Object.assign with an object literal as the first argument and prefer the use of object spread instead.",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-object-spread"
            },
            schema: [],
            fixable: "code",
            messages: {
                useSpreadMessage: "Use an object spread instead of `Object.assign` eg: `{ ...foo }`.",
                useLiteralMessage: "Use an object literal instead of `Object.assign`. eg: `{ foo: bar }`."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                Program() {
                    const scope = context.getScope();
                    const tracker = new ReferenceTracker(scope);
                    const trackMap = {
                        Object: {
                            assign: { [CALL]: true }
                        }
                    };
                    // Iterate all calls of `Object.assign` (only of the global variable `Object`).
                    for (const { node } of tracker.iterateGlobalReferences(trackMap)) {
                        if (node.arguments.length >= 1 &&
                            node.arguments[0].type === "ObjectExpression" &&
                            !hasArraySpread(node)) {
                            const messageId = node.arguments.length === 1
                                ? "useLiteralMessage"
                                : "useSpreadMessage";
                            const fix = defineFixer(node, sourceCode);
                            context.report({ node, messageId, fix });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/prefer-promise-reject-errors.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require using Error objects as Promise rejection reasons",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-promise-reject-errors"
            },
            fixable: null,
            schema: [
                {
                    type: "object",
                    properties: {
                        allowEmptyReject: { type: "boolean", default: false }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const ALLOW_EMPTY_REJECT = context.options.length && context.options[0].allowEmptyReject;
            //----------------------------------------------------------------------
            // Helpers
            //----------------------------------------------------------------------
            /**
             * Checks the argument of a reject() or Promise.reject() CallExpression, and reports it if it can't be an Error
             * @param {ASTNode} callExpression A CallExpression node which is used to reject a Promise
             * @returns {void}
             */
            function checkRejectCall(callExpression) {
                if (!callExpression.arguments.length && ALLOW_EMPTY_REJECT) {
                    return;
                }
                if (!callExpression.arguments.length ||
                    !astUtils.couldBeError(callExpression.arguments[0]) ||
                    callExpression.arguments[0].type === "Identifier" && callExpression.arguments[0].name === "undefined") {
                    context.report({
                        node: callExpression,
                        message: "Expected the Promise rejection reason to be an Error."
                    });
                }
            }
            /**
             * Determines whether a function call is a Promise.reject() call
             * @param {ASTNode} node A CallExpression node
             * @returns {boolean} `true` if the call is a Promise.reject() call
             */
            function isPromiseRejectCall(node) {
                return node.callee.type === "MemberExpression" &&
                    node.callee.object.type === "Identifier" && node.callee.object.name === "Promise" &&
                    node.callee.property.type === "Identifier" && node.callee.property.name === "reject";
            }
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                // Check `Promise.reject(value)` calls.
                CallExpression(node) {
                    if (isPromiseRejectCall(node)) {
                        checkRejectCall(node);
                    }
                },
                /*
                 * Check for `new Promise((resolve, reject) => {})`, and check for reject() calls.
                 * This function is run on "NewExpression:exit" instead of "NewExpression" to ensure that
                 * the nodes in the expression already have the `parent` property.
                 */
                "NewExpression:exit"(node) {
                    if (node.callee.type === "Identifier" && node.callee.name === "Promise" &&
                        node.arguments.length && astUtils.isFunction(node.arguments[0]) &&
                        node.arguments[0].params.length > 1 && node.arguments[0].params[1].type === "Identifier") {
                        context.getDeclaredVariables(node.arguments[0])
                            /*
                             * Find the first variable that matches the second parameter's name.
                             * If the first parameter has the same name as the second parameter, then the variable will actually
                             * be "declared" when the first parameter is evaluated, but then it will be immediately overwritten
                             * by the second parameter. It's not possible for an expression with the variable to be evaluated before
                             * the variable is overwritten, because functions with duplicate parameters cannot have destructuring or
                             * default assignments in their parameter lists. Therefore, it's not necessary to explicitly account for
                             * this case.
                             */
                            .find(variable => variable.name === node.arguments[0].params[1].name)
                            // Get the references to that variable.
                            .references
                            // Only check the references that read the parameter's value.
                            .filter(ref => ref.isRead())
                            // Only check the references that are used as the callee in a function call, e.g. `reject(foo)`.
                            .filter(ref => ref.identifier.parent.type === "CallExpression" && ref.identifier === ref.identifier.parent.callee)
                            // Check the argument of the function call to determine whether it's an Error.
                            .forEach(ref => checkRejectCall(ref.identifier.parent));
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/prefer-reflect.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require `Reflect` methods where applicable",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-reflect"
            },
            deprecated: true,
            replacedBy: [],
            schema: [
                {
                    type: "object",
                    properties: {
                        exceptions: {
                            type: "array",
                            items: {
                                enum: [
                                    "apply",
                                    "call",
                                    "delete",
                                    "defineProperty",
                                    "getOwnPropertyDescriptor",
                                    "getPrototypeOf",
                                    "setPrototypeOf",
                                    "isExtensible",
                                    "getOwnPropertyNames",
                                    "preventExtensions"
                                ]
                            },
                            uniqueItems: true
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const existingNames = {
                apply: "Function.prototype.apply",
                call: "Function.prototype.call",
                defineProperty: "Object.defineProperty",
                getOwnPropertyDescriptor: "Object.getOwnPropertyDescriptor",
                getPrototypeOf: "Object.getPrototypeOf",
                setPrototypeOf: "Object.setPrototypeOf",
                isExtensible: "Object.isExtensible",
                getOwnPropertyNames: "Object.getOwnPropertyNames",
                preventExtensions: "Object.preventExtensions"
            };
            const reflectSubsitutes = {
                apply: "Reflect.apply",
                call: "Reflect.apply",
                defineProperty: "Reflect.defineProperty",
                getOwnPropertyDescriptor: "Reflect.getOwnPropertyDescriptor",
                getPrototypeOf: "Reflect.getPrototypeOf",
                setPrototypeOf: "Reflect.setPrototypeOf",
                isExtensible: "Reflect.isExtensible",
                getOwnPropertyNames: "Reflect.getOwnPropertyNames",
                preventExtensions: "Reflect.preventExtensions"
            };
            const exceptions = (context.options[0] || {}).exceptions || [];
            /**
             * Reports the Reflect violation based on the `existing` and `substitute`
             * @param {Object} node The node that violates the rule.
             * @param {string} existing The existing method name that has been used.
             * @param {string} substitute The Reflect substitute that should be used.
             * @returns {void}
             */
            function report(node, existing, substitute) {
                context.report({
                    node,
                    message: "Avoid using {{existing}}, instead use {{substitute}}.",
                    data: {
                        existing,
                        substitute
                    }
                });
            }
            return {
                CallExpression(node) {
                    const methodName = (node.callee.property || {}).name;
                    const isReflectCall = (node.callee.object || {}).name === "Reflect";
                    const hasReflectSubsitute = Object.prototype.hasOwnProperty.call(reflectSubsitutes, methodName);
                    const userConfiguredException = exceptions.indexOf(methodName) !== -1;
                    if (hasReflectSubsitute && !isReflectCall && !userConfiguredException) {
                        report(node, existingNames[methodName], reflectSubsitutes[methodName]);
                    }
                },
                UnaryExpression(node) {
                    const isDeleteOperator = node.operator === "delete";
                    const targetsIdentifier = node.argument.type === "Identifier";
                    const userConfiguredException = exceptions.indexOf("delete") !== -1;
                    if (isDeleteOperator && !targetsIdentifier && !userConfiguredException) {
                        report(node, "the delete keyword", "Reflect.deleteProperty");
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/prefer-regex-literals.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    const { CALL, CONSTRUCT, ReferenceTracker, findVariable } = require(250) /* eslint-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Determines whether the given node is a string literal.
     * @param {ASTNode} node Node to check.
     * @returns {boolean} True if the node is a string literal.
     */
    function isStringLiteral(node) {
        return node.type === "Literal" && typeof node.value === "string";
    }
    /**
     * Determines whether the given node is a template literal without expressions.
     * @param {ASTNode} node Node to check.
     * @returns {boolean} True if the node is a template literal without expressions.
     */
    function isStaticTemplateLiteral(node) {
        return node.type === "TemplateLiteral" && node.expressions.length === 0;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow use of the `RegExp` constructor in favor of regular expression literals",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-regex-literals"
            },
            schema: [],
            messages: {
                unexpectedRegExp: "Use a regular expression literal instead of the 'RegExp' constructor."
            }
        },
        create(context) {
            /**
             * Determines whether the given identifier node is a reference to a global variable.
             * @param {ASTNode} node `Identifier` node to check.
             * @returns {boolean} True if the identifier is a reference to a global variable.
             */
            function isGlobalReference(node) {
                const scope = context.getScope();
                const variable = findVariable(scope, node);
                return variable !== null && variable.scope.type === "global" && variable.defs.length === 0;
            }
            /**
             * Determines whether the given node is a String.raw`` tagged template expression
             * with a static template literal.
             * @param {ASTNode} node Node to check.
             * @returns {boolean} True if the node is String.raw`` with a static template.
             */
            function isStringRawTaggedStaticTemplateLiteral(node) {
                return node.type === "TaggedTemplateExpression" &&
                    node.tag.type === "MemberExpression" &&
                    node.tag.object.type === "Identifier" &&
                    node.tag.object.name === "String" &&
                    isGlobalReference(node.tag.object) &&
                    astUtils.getStaticPropertyName(node.tag) === "raw" &&
                    isStaticTemplateLiteral(node.quasi);
            }
            /**
             * Determines whether the given node is considered to be a static string by the logic of this rule.
             * @param {ASTNode} node Node to check.
             * @returns {boolean} True if the node is a static string.
             */
            function isStaticString(node) {
                return isStringLiteral(node) ||
                    isStaticTemplateLiteral(node) ||
                    isStringRawTaggedStaticTemplateLiteral(node);
            }
            return {
                Program() {
                    const scope = context.getScope();
                    const tracker = new ReferenceTracker(scope);
                    const traceMap = {
                        RegExp: {
                            [CALL]: true,
                            [CONSTRUCT]: true
                        }
                    };
                    for (const { node } of tracker.iterateGlobalReferences(traceMap)) {
                        const args = node.arguments;
                        if ((args.length === 1 || args.length === 2) &&
                            args.every(isStaticString)) {
                            context.report({ node, messageId: "unexpectedRegExp" });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/prefer-rest-params.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Gets the variable object of `arguments` which is defined implicitly.
     * @param {eslint-scope.Scope} scope A scope to get.
     * @returns {eslint-scope.Variable} The found variable object.
     */
    function getVariableOfArguments(scope) {
        const variables = scope.variables;
        for (let i = 0; i < variables.length; ++i) {
            const variable = variables[i];
            if (variable.name === "arguments") {
                /*
                 * If there was a parameter which is named "arguments", the implicit "arguments" is not defined.
                 * So does fast return with null.
                 */
                return (variable.identifiers.length === 0) ? variable : null;
            }
        }
        /* istanbul ignore next : unreachable */
        return null;
    }
    /**
     * Checks if the given reference is not normal member access.
     *
     * - arguments         .... true    // not member access
     * - arguments[i]      .... true    // computed member access
     * - arguments[0]      .... true    // computed member access
     * - arguments.length  .... false   // normal member access
     * @param {eslint-scope.Reference} reference The reference to check.
     * @returns {boolean} `true` if the reference is not normal member access.
     */
    function isNotNormalMemberAccess(reference) {
        const id = reference.identifier;
        const parent = id.parent;
        return !(parent.type === "MemberExpression" &&
            parent.object === id &&
            !parent.computed);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require rest parameters instead of `arguments`",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-rest-params"
            },
            schema: []
        },
        create(context) {
            /**
             * Reports a given reference.
             * @param {eslint-scope.Reference} reference A reference to report.
             * @returns {void}
             */
            function report(reference) {
                context.report({
                    node: reference.identifier,
                    loc: reference.identifier.loc,
                    message: "Use the rest parameters instead of 'arguments'."
                });
            }
            /**
             * Reports references of the implicit `arguments` variable if exist.
             * @returns {void}
             */
            function checkForArguments() {
                const argumentsVar = getVariableOfArguments(context.getScope());
                if (argumentsVar) {
                    argumentsVar
                        .references
                        .filter(isNotNormalMemberAccess)
                        .forEach(report);
                }
            }
            return {
                "FunctionDeclaration:exit": checkForArguments,
                "FunctionExpression:exit": checkForArguments
            };
        }
    };
},
/* eslint/lib/rules/prefer-spread.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a node is a `.apply()` for variadic.
     * @param {ASTNode} node A CallExpression node to check.
     * @returns {boolean} Whether or not the node is a `.apply()` for variadic.
     */
    function isVariadicApplyCalling(node) {
        return (node.callee.type === "MemberExpression" &&
            node.callee.property.type === "Identifier" &&
            node.callee.property.name === "apply" &&
            node.callee.computed === false &&
            node.arguments.length === 2 &&
            node.arguments[1].type !== "ArrayExpression" &&
            node.arguments[1].type !== "SpreadElement");
    }
    /**
     * Checks whether or not `thisArg` is not changed by `.apply()`.
     * @param {ASTNode|null} expectedThis The node that is the owner of the applied function.
     * @param {ASTNode} thisArg The node that is given to the first argument of the `.apply()`.
     * @param {RuleContext} context The ESLint rule context object.
     * @returns {boolean} Whether or not `thisArg` is not changed by `.apply()`.
     */
    function isValidThisArg(expectedThis, thisArg, context) {
        if (!expectedThis) {
            return astUtils.isNullOrUndefined(thisArg);
        }
        return astUtils.equalTokens(expectedThis, thisArg, context);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require spread operators instead of `.apply()`",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-spread"
            },
            schema: [],
            fixable: null
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                CallExpression(node) {
                    if (!isVariadicApplyCalling(node)) {
                        return;
                    }
                    const applied = node.callee.object;
                    const expectedThis = (applied.type === "MemberExpression") ? applied.object : null;
                    const thisArg = node.arguments[0];
                    if (isValidThisArg(expectedThis, thisArg, sourceCode)) {
                        context.report({
                            node,
                            message: "Use the spread operator instead of '.apply()'."
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/prefer-template.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a given node is a concatenation.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node is a concatenation.
     */
    function isConcatenation(node) {
        return node.type === "BinaryExpression" && node.operator === "+";
    }
    /**
     * Gets the top binary expression node for concatenation in parents of a given node.
     * @param {ASTNode} node A node to get.
     * @returns {ASTNode} the top binary expression node in parents of a given node.
     */
    function getTopConcatBinaryExpression(node) {
        let currentNode = node;
        while (isConcatenation(currentNode.parent)) {
            currentNode = currentNode.parent;
        }
        return currentNode;
    }
    /**
     * Determines whether a given node is a octal escape sequence
     * @param {ASTNode} node A node to check
     * @returns {boolean} `true` if the node is an octal escape sequence
     */
    function isOctalEscapeSequence(node) {
        // No need to check TemplateLiterals – would throw error with octal escape
        const isStringLiteral = node.type === "Literal" && typeof node.value === "string";
        if (!isStringLiteral) {
            return false;
        }
        return astUtils.hasOctalEscapeSequence(node.raw);
    }
    /**
     * Checks whether or not a node contains a octal escape sequence
     * @param {ASTNode} node A node to check
     * @returns {boolean} `true` if the node contains an octal escape sequence
     */
    function hasOctalEscapeSequence(node) {
        if (isConcatenation(node)) {
            return hasOctalEscapeSequence(node.left) || hasOctalEscapeSequence(node.right);
        }
        return isOctalEscapeSequence(node);
    }
    /**
     * Checks whether or not a given binary expression has string literals.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node has string literals.
     */
    function hasStringLiteral(node) {
        if (isConcatenation(node)) {
            // `left` is deeper than `right` normally.
            return hasStringLiteral(node.right) || hasStringLiteral(node.left);
        }
        return astUtils.isStringLiteral(node);
    }
    /**
     * Checks whether or not a given binary expression has non string literals.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node has non string literals.
     */
    function hasNonStringLiteral(node) {
        if (isConcatenation(node)) {
            // `left` is deeper than `right` normally.
            return hasNonStringLiteral(node.right) || hasNonStringLiteral(node.left);
        }
        return !astUtils.isStringLiteral(node);
    }
    /**
     * Determines whether a given node will start with a template curly expression (`${}`) when being converted to a template literal.
     * @param {ASTNode} node The node that will be fixed to a template literal
     * @returns {boolean} `true` if the node will start with a template curly.
     */
    function startsWithTemplateCurly(node) {
        if (node.type === "BinaryExpression") {
            return startsWithTemplateCurly(node.left);
        }
        if (node.type === "TemplateLiteral") {
            return node.expressions.length && node.quasis.length && node.quasis[0].range[0] === node.quasis[0].range[1];
        }
        return node.type !== "Literal" || typeof node.value !== "string";
    }
    /**
     * Determines whether a given node end with a template curly expression (`${}`) when being converted to a template literal.
     * @param {ASTNode} node The node that will be fixed to a template literal
     * @returns {boolean} `true` if the node will end with a template curly.
     */
    function endsWithTemplateCurly(node) {
        if (node.type === "BinaryExpression") {
            return startsWithTemplateCurly(node.right);
        }
        if (node.type === "TemplateLiteral") {
            return node.expressions.length && node.quasis.length && node.quasis[node.quasis.length - 1].range[0] === node.quasis[node.quasis.length - 1].range[1];
        }
        return node.type !== "Literal" || typeof node.value !== "string";
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require template literals instead of string concatenation",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/prefer-template"
            },
            schema: [],
            fixable: "code"
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            let done = Object.create(null);
            /**
             * Gets the non-token text between two nodes, ignoring any other tokens that appear between the two tokens.
             * @param {ASTNode} node1 The first node
             * @param {ASTNode} node2 The second node
             * @returns {string} The text between the nodes, excluding other tokens
             */
            function getTextBetween(node1, node2) {
                const allTokens = [node1].concat(sourceCode.getTokensBetween(node1, node2)).concat(node2);
                const sourceText = sourceCode.getText();
                return allTokens.slice(0, -1).reduce((accumulator, token, index) => accumulator + sourceText.slice(token.range[1], allTokens[index + 1].range[0]), "");
            }
            /**
             * Returns a template literal form of the given node.
             * @param {ASTNode} currentNode A node that should be converted to a template literal
             * @param {string} textBeforeNode Text that should appear before the node
             * @param {string} textAfterNode Text that should appear after the node
             * @returns {string} A string form of this node, represented as a template literal
             */
            function getTemplateLiteral(currentNode, textBeforeNode, textAfterNode) {
                if (currentNode.type === "Literal" && typeof currentNode.value === "string") {
                    /*
                     * If the current node is a string literal, escape any instances of ${ or ` to prevent them from being interpreted
                     * as a template placeholder. However, if the code already contains a backslash before the ${ or `
                     * for some reason, don't add another backslash, because that would change the meaning of the code (it would cause
                     * an actual backslash character to appear before the dollar sign).
                     */
                    return `\`${currentNode.raw.slice(1, -1).replace(/\\*(\$\{|`)/gu, matched => {
                        if (matched.lastIndexOf("\\") % 2) {
                            return `\\${matched}`;
                        }
                        return matched;
                        // Unescape any quotes that appear in the original Literal that no longer need to be escaped.
                    }).replace(new RegExp(`\\\\${currentNode.raw[0]}`, "gu"), currentNode.raw[0])}\``;
                }
                if (currentNode.type === "TemplateLiteral") {
                    return sourceCode.getText(currentNode);
                }
                if (isConcatenation(currentNode) && hasStringLiteral(currentNode) && hasNonStringLiteral(currentNode)) {
                    const plusSign = sourceCode.getFirstTokenBetween(currentNode.left, currentNode.right, token => token.value === "+");
                    const textBeforePlus = getTextBetween(currentNode.left, plusSign);
                    const textAfterPlus = getTextBetween(plusSign, currentNode.right);
                    const leftEndsWithCurly = endsWithTemplateCurly(currentNode.left);
                    const rightStartsWithCurly = startsWithTemplateCurly(currentNode.right);
                    if (leftEndsWithCurly) {
                        // If the left side of the expression ends with a template curly, add the extra text to the end of the curly bracket.
                        // `foo${bar}` /* comment */ + 'baz' --> `foo${bar /* comment */  }${baz}`
                        return getTemplateLiteral(currentNode.left, textBeforeNode, textBeforePlus + textAfterPlus).slice(0, -1) +
                            getTemplateLiteral(currentNode.right, null, textAfterNode).slice(1);
                    }
                    if (rightStartsWithCurly) {
                        // Otherwise, if the right side of the expression starts with a template curly, add the text there.
                        // 'foo' /* comment */ + `${bar}baz` --> `foo${ /* comment */  bar}baz`
                        return getTemplateLiteral(currentNode.left, textBeforeNode, null).slice(0, -1) +
                            getTemplateLiteral(currentNode.right, textBeforePlus + textAfterPlus, textAfterNode).slice(1);
                    }
                    /*
                     * Otherwise, these nodes should not be combined into a template curly, since there is nowhere to put
                     * the text between them.
                     */
                    return `${getTemplateLiteral(currentNode.left, textBeforeNode, null)}${textBeforePlus}+${textAfterPlus}${getTemplateLiteral(currentNode.right, textAfterNode, null)}`;
                }
                return `\`\${${textBeforeNode || ""}${sourceCode.getText(currentNode)}${textAfterNode || ""}}\``;
            }
            /**
             * Returns a fixer object that converts a non-string binary expression to a template literal
             * @param {SourceCodeFixer} fixer The fixer object
             * @param {ASTNode} node A node that should be converted to a template literal
             * @returns {Object} A fix for this binary expression
             */
            function fixNonStringBinaryExpression(fixer, node) {
                const topBinaryExpr = getTopConcatBinaryExpression(node.parent);
                if (hasOctalEscapeSequence(topBinaryExpr)) {
                    return null;
                }
                return fixer.replaceText(topBinaryExpr, getTemplateLiteral(topBinaryExpr, null, null));
            }
            /**
             * Reports if a given node is string concatenation with non string literals.
             * @param {ASTNode} node A node to check.
             * @returns {void}
             */
            function checkForStringConcat(node) {
                if (!astUtils.isStringLiteral(node) || !isConcatenation(node.parent)) {
                    return;
                }
                const topBinaryExpr = getTopConcatBinaryExpression(node.parent);
                // Checks whether or not this node had been checked already.
                if (done[topBinaryExpr.range[0]]) {
                    return;
                }
                done[topBinaryExpr.range[0]] = true;
                if (hasNonStringLiteral(topBinaryExpr)) {
                    context.report({
                        node: topBinaryExpr,
                        message: "Unexpected string concatenation.",
                        fix: fixer => fixNonStringBinaryExpression(fixer, node)
                    });
                }
            }
            return {
                Program() {
                    done = Object.create(null);
                },
                Literal: checkForStringConcat,
                TemplateLiteral: checkForStringConcat
            };
        }
    };
},
/* eslint/lib/rules/quote-props.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const espree = require(102) /* espree */, keywords = require(159) /* ./utils/keywords */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require quotes around object literal property names",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/quote-props"
            },
            schema: {
                anyOf: [
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["always", "as-needed", "consistent", "consistent-as-needed"]
                            }
                        ],
                        minItems: 0,
                        maxItems: 1
                    },
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["always", "as-needed", "consistent", "consistent-as-needed"]
                            },
                            {
                                type: "object",
                                properties: {
                                    keywords: {
                                        type: "boolean"
                                    },
                                    unnecessary: {
                                        type: "boolean"
                                    },
                                    numbers: {
                                        type: "boolean"
                                    }
                                },
                                additionalProperties: false
                            }
                        ],
                        minItems: 0,
                        maxItems: 2
                    }
                ]
            },
            fixable: "code"
        },
        create(context) {
            const MODE = context.options[0], KEYWORDS = context.options[1] && context.options[1].keywords, CHECK_UNNECESSARY = !context.options[1] || context.options[1].unnecessary !== false, NUMBERS = context.options[1] && context.options[1].numbers, MESSAGE_UNNECESSARY = "Unnecessarily quoted property '{{property}}' found.", MESSAGE_UNQUOTED = "Unquoted property '{{property}}' found.", MESSAGE_NUMERIC = "Unquoted number literal '{{property}}' used as key.", MESSAGE_RESERVED = "Unquoted reserved word '{{property}}' used as key.", sourceCode = context.getSourceCode();
            /**
             * Checks whether a certain string constitutes an ES3 token
             * @param   {string} tokenStr The string to be checked.
             * @returns {boolean} `true` if it is an ES3 token.
             */
            function isKeyword(tokenStr) {
                return keywords.indexOf(tokenStr) >= 0;
            }
            /**
             * Checks if an espree-tokenized key has redundant quotes (i.e. whether quotes are unnecessary)
             * @param   {string} rawKey The raw key value from the source
             * @param   {espreeTokens} tokens The espree-tokenized node key
             * @param   {boolean} [skipNumberLiterals=false] Indicates whether number literals should be checked
             * @returns {boolean} Whether or not a key has redundant quotes.
             * @private
             */
            function areQuotesRedundant(rawKey, tokens, skipNumberLiterals) {
                return tokens.length === 1 && tokens[0].start === 0 && tokens[0].end === rawKey.length &&
                    (["Identifier", "Keyword", "Null", "Boolean"].indexOf(tokens[0].type) >= 0 ||
                        (tokens[0].type === "Numeric" && !skipNumberLiterals && String(+tokens[0].value) === tokens[0].value));
            }
            /**
             * Returns a string representation of a property node with quotes removed
             * @param {ASTNode} key Key AST Node, which may or may not be quoted
             * @returns {string} A replacement string for this property
             */
            function getUnquotedKey(key) {
                return key.type === "Identifier" ? key.name : key.value;
            }
            /**
             * Returns a string representation of a property node with quotes added
             * @param {ASTNode} key Key AST Node, which may or may not be quoted
             * @returns {string} A replacement string for this property
             */
            function getQuotedKey(key) {
                if (key.type === "Literal" && typeof key.value === "string") {
                    // If the key is already a string literal, don't replace the quotes with double quotes.
                    return sourceCode.getText(key);
                }
                // Otherwise, the key is either an identifier or a number literal.
                return `"${key.type === "Identifier" ? key.name : key.value}"`;
            }
            /**
             * Ensures that a property's key is quoted only when necessary
             * @param   {ASTNode} node Property AST node
             * @returns {void}
             */
            function checkUnnecessaryQuotes(node) {
                const key = node.key;
                if (node.method || node.computed || node.shorthand) {
                    return;
                }
                if (key.type === "Literal" && typeof key.value === "string") {
                    let tokens;
                    try {
                        tokens = espree.tokenize(key.value);
                    }
                    catch (e) {
                        return;
                    }
                    if (tokens.length !== 1) {
                        return;
                    }
                    const isKeywordToken = isKeyword(tokens[0].value);
                    if (isKeywordToken && KEYWORDS) {
                        return;
                    }
                    if (CHECK_UNNECESSARY && areQuotesRedundant(key.value, tokens, NUMBERS)) {
                        context.report({
                            node,
                            message: MESSAGE_UNNECESSARY,
                            data: { property: key.value },
                            fix: fixer => fixer.replaceText(key, getUnquotedKey(key))
                        });
                    }
                }
                else if (KEYWORDS && key.type === "Identifier" && isKeyword(key.name)) {
                    context.report({
                        node,
                        message: MESSAGE_RESERVED,
                        data: { property: key.name },
                        fix: fixer => fixer.replaceText(key, getQuotedKey(key))
                    });
                }
                else if (NUMBERS && key.type === "Literal" && typeof key.value === "number") {
                    context.report({
                        node,
                        message: MESSAGE_NUMERIC,
                        data: { property: key.value },
                        fix: fixer => fixer.replaceText(key, getQuotedKey(key))
                    });
                }
            }
            /**
             * Ensures that a property's key is quoted
             * @param   {ASTNode} node Property AST node
             * @returns {void}
             */
            function checkOmittedQuotes(node) {
                const key = node.key;
                if (!node.method && !node.computed && !node.shorthand && !(key.type === "Literal" && typeof key.value === "string")) {
                    context.report({
                        node,
                        message: MESSAGE_UNQUOTED,
                        data: { property: key.name || key.value },
                        fix: fixer => fixer.replaceText(key, getQuotedKey(key))
                    });
                }
            }
            /**
             * Ensures that an object's keys are consistently quoted, optionally checks for redundancy of quotes
             * @param   {ASTNode} node Property AST node
             * @param   {boolean} checkQuotesRedundancy Whether to check quotes' redundancy
             * @returns {void}
             */
            function checkConsistency(node, checkQuotesRedundancy) {
                const quotedProps = [], unquotedProps = [];
                let keywordKeyName = null, necessaryQuotes = false;
                node.properties.forEach(property => {
                    const key = property.key;
                    if (!key || property.method || property.computed || property.shorthand) {
                        return;
                    }
                    if (key.type === "Literal" && typeof key.value === "string") {
                        quotedProps.push(property);
                        if (checkQuotesRedundancy) {
                            let tokens;
                            try {
                                tokens = espree.tokenize(key.value);
                            }
                            catch (e) {
                                necessaryQuotes = true;
                                return;
                            }
                            necessaryQuotes = necessaryQuotes || !areQuotesRedundant(key.value, tokens) || KEYWORDS && isKeyword(tokens[0].value);
                        }
                    }
                    else if (KEYWORDS && checkQuotesRedundancy && key.type === "Identifier" && isKeyword(key.name)) {
                        unquotedProps.push(property);
                        necessaryQuotes = true;
                        keywordKeyName = key.name;
                    }
                    else {
                        unquotedProps.push(property);
                    }
                });
                if (checkQuotesRedundancy && quotedProps.length && !necessaryQuotes) {
                    quotedProps.forEach(property => {
                        context.report({
                            node: property,
                            message: "Properties shouldn't be quoted as all quotes are redundant.",
                            fix: fixer => fixer.replaceText(property.key, getUnquotedKey(property.key))
                        });
                    });
                }
                else if (unquotedProps.length && keywordKeyName) {
                    unquotedProps.forEach(property => {
                        context.report({
                            node: property,
                            message: "Properties should be quoted as '{{property}}' is a reserved word.",
                            data: { property: keywordKeyName },
                            fix: fixer => fixer.replaceText(property.key, getQuotedKey(property.key))
                        });
                    });
                }
                else if (quotedProps.length && unquotedProps.length) {
                    unquotedProps.forEach(property => {
                        context.report({
                            node: property,
                            message: "Inconsistently quoted property '{{key}}' found.",
                            data: { key: property.key.name || property.key.value },
                            fix: fixer => fixer.replaceText(property.key, getQuotedKey(property.key))
                        });
                    });
                }
            }
            return {
                Property(node) {
                    if (MODE === "always" || !MODE) {
                        checkOmittedQuotes(node);
                    }
                    if (MODE === "as-needed") {
                        checkUnnecessaryQuotes(node);
                    }
                },
                ObjectExpression(node) {
                    if (MODE === "consistent") {
                        checkConsistency(node, false);
                    }
                    if (MODE === "consistent-as-needed") {
                        checkConsistency(node, true);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/quotes.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Constants
    //------------------------------------------------------------------------------
    const QUOTE_SETTINGS = {
        double: {
            quote: "\"",
            alternateQuote: "'",
            description: "doublequote"
        },
        single: {
            quote: "'",
            alternateQuote: "\"",
            description: "singlequote"
        },
        backtick: {
            quote: "`",
            alternateQuote: "\"",
            description: "backtick"
        }
    };
    // An unescaped newline is a newline preceded by an even number of backslashes.
    const UNESCAPED_LINEBREAK_PATTERN = new RegExp(String.raw `(^|[^\\])(\\\\)*[${Array.from(astUtils.LINEBREAKS).join("")}]`, "u");
    /**
     * Switches quoting of javascript string between ' " and `
     * escaping and unescaping as necessary.
     * Only escaping of the minimal set of characters is changed.
     * Note: escaping of newlines when switching from backtick to other quotes is not handled.
     * @param {string} str A string to convert.
     * @returns {string} The string with changed quotes.
     * @private
     */
    QUOTE_SETTINGS.double.convert =
        QUOTE_SETTINGS.single.convert =
            QUOTE_SETTINGS.backtick.convert = function (str) {
                const newQuote = this.quote;
                const oldQuote = str[0];
                if (newQuote === oldQuote) {
                    return str;
                }
                return newQuote + str.slice(1, -1).replace(/\\(\$\{|\r\n?|\n|.)|["'`]|\$\{|(\r\n?|\n)/gu, (match, escaped, newline) => {
                    if (escaped === oldQuote || oldQuote === "`" && escaped === "${") {
                        return escaped; // unescape
                    }
                    if (match === newQuote || newQuote === "`" && match === "${") {
                        return `\\${match}`; // escape
                    }
                    if (newline && oldQuote === "`") {
                        return "\\n"; // escape newlines
                    }
                    return match;
                }) + newQuote;
            };
    const AVOID_ESCAPE = "avoid-escape";
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce the consistent use of either backticks, double, or single quotes",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/quotes"
            },
            fixable: "code",
            schema: [
                {
                    enum: ["single", "double", "backtick"]
                },
                {
                    anyOf: [
                        {
                            enum: ["avoid-escape"]
                        },
                        {
                            type: "object",
                            properties: {
                                avoidEscape: {
                                    type: "boolean"
                                },
                                allowTemplateLiterals: {
                                    type: "boolean"
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ]
        },
        create(context) {
            const quoteOption = context.options[0], settings = QUOTE_SETTINGS[quoteOption || "double"], options = context.options[1], allowTemplateLiterals = options && options.allowTemplateLiterals === true, sourceCode = context.getSourceCode();
            let avoidEscape = options && options.avoidEscape === true;
            // deprecated
            if (options === AVOID_ESCAPE) {
                avoidEscape = true;
            }
            /**
             * Determines if a given node is part of JSX syntax.
             *
             * This function returns `true` in the following cases:
             *
             * - `<div className="foo"></div>` ... If the literal is an attribute value, the parent of the literal is `JSXAttribute`.
             * - `<div>foo</div>` ... If the literal is a text content, the parent of the literal is `JSXElement`.
             * - `<>foo</>` ... If the literal is a text content, the parent of the literal is `JSXFragment`.
             *
             * In particular, this function returns `false` in the following cases:
             *
             * - `<div className={"foo"}></div>`
             * - `<div>{"foo"}</div>`
             *
             * In both cases, inside of the braces is handled as normal JavaScript.
             * The braces are `JSXExpressionContainer` nodes.
             * @param {ASTNode} node The Literal node to check.
             * @returns {boolean} True if the node is a part of JSX, false if not.
             * @private
             */
            function isJSXLiteral(node) {
                return node.parent.type === "JSXAttribute" || node.parent.type === "JSXElement" || node.parent.type === "JSXFragment";
            }
            /**
             * Checks whether or not a given node is a directive.
             * The directive is a `ExpressionStatement` which has only a string literal.
             * @param {ASTNode} node A node to check.
             * @returns {boolean} Whether or not the node is a directive.
             * @private
             */
            function isDirective(node) {
                return (node.type === "ExpressionStatement" &&
                    node.expression.type === "Literal" &&
                    typeof node.expression.value === "string");
            }
            /**
             * Checks whether or not a given node is a part of directive prologues.
             * See also: http://www.ecma-international.org/ecma-262/6.0/#sec-directive-prologues-and-the-use-strict-directive
             * @param {ASTNode} node A node to check.
             * @returns {boolean} Whether or not the node is a part of directive prologues.
             * @private
             */
            function isPartOfDirectivePrologue(node) {
                const block = node.parent.parent;
                if (block.type !== "Program" && (block.type !== "BlockStatement" || !astUtils.isFunction(block.parent))) {
                    return false;
                }
                // Check the node is at a prologue.
                for (let i = 0; i < block.body.length; ++i) {
                    const statement = block.body[i];
                    if (statement === node.parent) {
                        return true;
                    }
                    if (!isDirective(statement)) {
                        break;
                    }
                }
                return false;
            }
            /**
             * Checks whether or not a given node is allowed as non backtick.
             * @param {ASTNode} node A node to check.
             * @returns {boolean} Whether or not the node is allowed as non backtick.
             * @private
             */
            function isAllowedAsNonBacktick(node) {
                const parent = node.parent;
                switch (parent.type) {
                    // Directive Prologues.
                    case "ExpressionStatement":
                        return isPartOfDirectivePrologue(node);
                    // LiteralPropertyName.
                    case "Property":
                    case "MethodDefinition":
                        return parent.key === node && !parent.computed;
                    // ModuleSpecifier.
                    case "ImportDeclaration":
                    case "ExportNamedDeclaration":
                    case "ExportAllDeclaration":
                        return parent.source === node;
                    // Others don't allow.
                    default:
                        return false;
                }
            }
            /**
             * Checks whether or not a given TemplateLiteral node is actually using any of the special features provided by template literal strings.
             * @param {ASTNode} node A TemplateLiteral node to check.
             * @returns {boolean} Whether or not the TemplateLiteral node is using any of the special features provided by template literal strings.
             * @private
             */
            function isUsingFeatureOfTemplateLiteral(node) {
                const hasTag = node.parent.type === "TaggedTemplateExpression" && node === node.parent.quasi;
                if (hasTag) {
                    return true;
                }
                const hasStringInterpolation = node.expressions.length > 0;
                if (hasStringInterpolation) {
                    return true;
                }
                const isMultilineString = node.quasis.length >= 1 && UNESCAPED_LINEBREAK_PATTERN.test(node.quasis[0].value.raw);
                if (isMultilineString) {
                    return true;
                }
                return false;
            }
            return {
                Literal(node) {
                    const val = node.value, rawVal = node.raw;
                    if (settings && typeof val === "string") {
                        let isValid = (quoteOption === "backtick" && isAllowedAsNonBacktick(node)) ||
                            isJSXLiteral(node) ||
                            astUtils.isSurroundedBy(rawVal, settings.quote);
                        if (!isValid && avoidEscape) {
                            isValid = astUtils.isSurroundedBy(rawVal, settings.alternateQuote) && rawVal.indexOf(settings.quote) >= 0;
                        }
                        if (!isValid) {
                            context.report({
                                node,
                                message: "Strings must use {{description}}.",
                                data: {
                                    description: settings.description
                                },
                                fix(fixer) {
                                    if (quoteOption === "backtick" && astUtils.hasOctalEscapeSequence(rawVal)) {
                                        // An octal escape sequence in a template literal would produce syntax error, even in non-strict mode.
                                        return null;
                                    }
                                    return fixer.replaceText(node, settings.convert(node.raw));
                                }
                            });
                        }
                    }
                },
                TemplateLiteral(node) {
                    // Don't throw an error if backticks are expected or a template literal feature is in use.
                    if (allowTemplateLiterals ||
                        quoteOption === "backtick" ||
                        isUsingFeatureOfTemplateLiteral(node)) {
                        return;
                    }
                    context.report({
                        node,
                        message: "Strings must use {{description}}.",
                        data: {
                            description: settings.description
                        },
                        fix(fixer) {
                            if (isPartOfDirectivePrologue(node)) {
                                /*
                                 * TemplateLiterals in a directive prologue aren't actually directives, but if they're
                                 * in the directive prologue, then fixing them might turn them into directives and change
                                 * the behavior of the code.
                                 */
                                return null;
                            }
                            return fixer.replaceText(node, settings.convert(sourceCode.getText(node)));
                        }
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/radix.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const MODE_ALWAYS = "always", MODE_AS_NEEDED = "as-needed";
    /**
     * Checks whether a given variable is shadowed or not.
     * @param {eslint-scope.Variable} variable A variable to check.
     * @returns {boolean} `true` if the variable is shadowed.
     */
    function isShadowed(variable) {
        return variable.defs.length >= 1;
    }
    /**
     * Checks whether a given node is a MemberExpression of `parseInt` method or not.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node is a MemberExpression of `parseInt`
     *      method.
     */
    function isParseIntMethod(node) {
        return (node.type === "MemberExpression" &&
            !node.computed &&
            node.property.type === "Identifier" &&
            node.property.name === "parseInt");
    }
    /**
     * Checks whether a given node is a valid value of radix or not.
     *
     * The following values are invalid.
     *
     * - A literal except numbers.
     * - undefined.
     * @param {ASTNode} radix A node of radix to check.
     * @returns {boolean} `true` if the node is valid.
     */
    function isValidRadix(radix) {
        return !((radix.type === "Literal" && typeof radix.value !== "number") ||
            (radix.type === "Identifier" && radix.name === "undefined"));
    }
    /**
     * Checks whether a given node is a default value of radix or not.
     * @param {ASTNode} radix A node of radix to check.
     * @returns {boolean} `true` if the node is the literal node of `10`.
     */
    function isDefaultRadix(radix) {
        return radix.type === "Literal" && radix.value === 10;
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce the consistent use of the radix argument when using `parseInt()`",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/radix"
            },
            schema: [
                {
                    enum: ["always", "as-needed"]
                }
            ]
        },
        create(context) {
            const mode = context.options[0] || MODE_ALWAYS;
            /**
             * Checks the arguments of a given CallExpression node and reports it if it
             * offends this rule.
             * @param {ASTNode} node A CallExpression node to check.
             * @returns {void}
             */
            function checkArguments(node) {
                const args = node.arguments;
                switch (args.length) {
                    case 0:
                        context.report({
                            node,
                            message: "Missing parameters."
                        });
                        break;
                    case 1:
                        if (mode === MODE_ALWAYS) {
                            context.report({
                                node,
                                message: "Missing radix parameter."
                            });
                        }
                        break;
                    default:
                        if (mode === MODE_AS_NEEDED && isDefaultRadix(args[1])) {
                            context.report({
                                node,
                                message: "Redundant radix parameter."
                            });
                        }
                        else if (!isValidRadix(args[1])) {
                            context.report({
                                node,
                                message: "Invalid radix parameter."
                            });
                        }
                        break;
                }
            }
            return {
                "Program:exit"() {
                    const scope = context.getScope();
                    let variable;
                    // Check `parseInt()`
                    variable = astUtils.getVariableByName(scope, "parseInt");
                    if (!isShadowed(variable)) {
                        variable.references.forEach(reference => {
                            const node = reference.identifier;
                            if (astUtils.isCallee(node)) {
                                checkArguments(node.parent);
                            }
                        });
                    }
                    // Check `Number.parseInt()`
                    variable = astUtils.getVariableByName(scope, "Number");
                    if (!isShadowed(variable)) {
                        variable.references.forEach(reference => {
                            const node = reference.identifier.parent;
                            if (isParseIntMethod(node) && astUtils.isCallee(node)) {
                                checkArguments(node.parent);
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/require-atomic-updates.js */ function _(require, module, exports) {
    /**
     * Make the map from identifiers to each reference.
     * @param {escope.Scope} scope The scope to get references.
     * @param {Map<Identifier, escope.Reference>} [outReferenceMap] The map from identifier nodes to each reference object.
     * @returns {Map<Identifier, escope.Reference>} `referenceMap`.
     */
    function createReferenceMap(scope, outReferenceMap = new Map()) {
        for (const reference of scope.references) {
            outReferenceMap.set(reference.identifier, reference);
        }
        for (const childScope of scope.childScopes) {
            if (childScope.type !== "function") {
                createReferenceMap(childScope, outReferenceMap);
            }
        }
        return outReferenceMap;
    }
    /**
     * Get `reference.writeExpr` of a given reference.
     * If it's the read reference of MemberExpression in LHS, returns RHS in order to address `a.b = await a`
     * @param {escope.Reference} reference The reference to get.
     * @returns {Expression|null} The `reference.writeExpr`.
     */
    function getWriteExpr(reference) {
        if (reference.writeExpr) {
            return reference.writeExpr;
        }
        let node = reference.identifier;
        while (node) {
            const t = node.parent.type;
            if (t === "AssignmentExpression" && node.parent.left === node) {
                return node.parent.right;
            }
            if (t === "MemberExpression" && node.parent.object === node) {
                node = node.parent;
                continue;
            }
            break;
        }
        return null;
    }
    /**
     * Checks if an expression is a variable that can only be observed within the given function.
     * @param {Variable|null} variable The variable to check
     * @param {boolean} isMemberAccess If `true` then this is a member access.
     * @returns {boolean} `true` if the variable is local to the given function, and is never referenced in a closure.
     */
    function isLocalVariableWithoutEscape(variable, isMemberAccess) {
        if (!variable) {
            return false; // A global variable which was not defined.
        }
        // If the reference is a property access and the variable is a parameter, it handles the variable is not local.
        if (isMemberAccess && variable.defs.some(d => d.type === "Parameter")) {
            return false;
        }
        const functionScope = variable.scope.variableScope;
        return variable.references.every(reference => reference.from.variableScope === functionScope);
    }
    class SegmentInfo {
        constructor() {
            this.info = new WeakMap();
        }
        /**
         * Initialize the segment information.
         * @param {PathSegment} segment The segment to initialize.
         * @returns {void}
         */
        initialize(segment) {
            const outdatedReadVariableNames = new Set();
            const freshReadVariableNames = new Set();
            for (const prevSegment of segment.prevSegments) {
                const info = this.info.get(prevSegment);
                if (info) {
                    info.outdatedReadVariableNames.forEach(Set.prototype.add, outdatedReadVariableNames);
                    info.freshReadVariableNames.forEach(Set.prototype.add, freshReadVariableNames);
                }
            }
            this.info.set(segment, { outdatedReadVariableNames, freshReadVariableNames });
        }
        /**
         * Mark a given variable as read on given segments.
         * @param {PathSegment[]} segments The segments that it read the variable on.
         * @param {string} variableName The variable name to be read.
         * @returns {void}
         */
        markAsRead(segments, variableName) {
            for (const segment of segments) {
                const info = this.info.get(segment);
                if (info) {
                    info.freshReadVariableNames.add(variableName);
                }
            }
        }
        /**
         * Move `freshReadVariableNames` to `outdatedReadVariableNames`.
         * @param {PathSegment[]} segments The segments to process.
         * @returns {void}
         */
        makeOutdated(segments) {
            for (const segment of segments) {
                const info = this.info.get(segment);
                if (info) {
                    info.freshReadVariableNames.forEach(Set.prototype.add, info.outdatedReadVariableNames);
                    info.freshReadVariableNames.clear();
                }
            }
        }
        /**
         * Check if a given variable is outdated on the current segments.
         * @param {PathSegment[]} segments The current segments.
         * @param {string} variableName The variable name to check.
         * @returns {boolean} `true` if the variable is outdated on the segments.
         */
        isOutdated(segments, variableName) {
            for (const segment of segments) {
                const info = this.info.get(segment);
                if (info && info.outdatedReadVariableNames.has(variableName)) {
                    return true;
                }
            }
            return false;
        }
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "disallow assignments that can lead to race conditions due to usage of `await` or `yield`",
                category: "Possible Errors",
                recommended: false,
                url: "https://eslint.org/docs/rules/require-atomic-updates"
            },
            fixable: null,
            schema: [],
            messages: {
                nonAtomicUpdate: "Possible race condition: `{{value}}` might be reassigned based on an outdated value of `{{value}}`."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const assignmentReferences = new Map();
            const segmentInfo = new SegmentInfo();
            let stack = null;
            return {
                onCodePathStart(codePath) {
                    const scope = context.getScope();
                    const shouldVerify = scope.type === "function" &&
                        (scope.block.async || scope.block.generator);
                    stack = {
                        upper: stack,
                        codePath,
                        referenceMap: shouldVerify ? createReferenceMap(scope) : null
                    };
                },
                onCodePathEnd() {
                    stack = stack.upper;
                },
                // Initialize the segment information.
                onCodePathSegmentStart(segment) {
                    segmentInfo.initialize(segment);
                },
                // Handle references to prepare verification.
                Identifier(node) {
                    const { codePath, referenceMap } = stack;
                    const reference = referenceMap && referenceMap.get(node);
                    // Ignore if this is not a valid variable reference.
                    if (!reference) {
                        return;
                    }
                    const name = reference.identifier.name;
                    const variable = reference.resolved;
                    const writeExpr = getWriteExpr(reference);
                    const isMemberAccess = reference.identifier.parent.type === "MemberExpression";
                    // Add a fresh read variable.
                    if (reference.isRead() && !(writeExpr && writeExpr.parent.operator === "=")) {
                        segmentInfo.markAsRead(codePath.currentSegments, name);
                    }
                    /*
                     * Register the variable to verify after ESLint traversed the `writeExpr` node
                     * if this reference is an assignment to a variable which is referred from other clausure.
                     */
                    if (writeExpr &&
                        writeExpr.parent.right === writeExpr && // ← exclude variable declarations.
                        !isLocalVariableWithoutEscape(variable, isMemberAccess)) {
                        let refs = assignmentReferences.get(writeExpr);
                        if (!refs) {
                            refs = [];
                            assignmentReferences.set(writeExpr, refs);
                        }
                        refs.push(reference);
                    }
                },
                /*
                 * Verify assignments.
                 * If the reference exists in `outdatedReadVariableNames` list, report it.
                 */
                ":expression:exit"(node) {
                    const { codePath, referenceMap } = stack;
                    // referenceMap exists if this is in a resumable function scope.
                    if (!referenceMap) {
                        return;
                    }
                    // Mark the read variables on this code path as outdated.
                    if (node.type === "AwaitExpression" || node.type === "YieldExpression") {
                        segmentInfo.makeOutdated(codePath.currentSegments);
                    }
                    // Verify.
                    const references = assignmentReferences.get(node);
                    if (references) {
                        assignmentReferences.delete(node);
                        for (const reference of references) {
                            const name = reference.identifier.name;
                            if (segmentInfo.isOutdated(codePath.currentSegments, name)) {
                                context.report({
                                    node: node.parent,
                                    messageId: "nonAtomicUpdate",
                                    data: {
                                        value: sourceCode.getText(node.parent.left)
                                    }
                                });
                            }
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/require-await.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Capitalize the 1st letter of the given text.
     * @param {string} text The text to capitalize.
     * @returns {string} The text that the 1st letter was capitalized.
     */
    function capitalizeFirstLetter(text) {
        return text[0].toUpperCase() + text.slice(1);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "disallow async functions which have no `await` expression",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/require-await"
            },
            schema: []
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            let scopeInfo = null;
            /**
             * Push the scope info object to the stack.
             * @returns {void}
             */
            function enterFunction() {
                scopeInfo = {
                    upper: scopeInfo,
                    hasAwait: false
                };
            }
            /**
             * Pop the top scope info object from the stack.
             * Also, it reports the function if needed.
             * @param {ASTNode} node The node to report.
             * @returns {void}
             */
            function exitFunction(node) {
                if (node.async && !scopeInfo.hasAwait && !astUtils.isEmptyFunction(node)) {
                    context.report({
                        node,
                        loc: astUtils.getFunctionHeadLoc(node, sourceCode),
                        message: "{{name}} has no 'await' expression.",
                        data: {
                            name: capitalizeFirstLetter(astUtils.getFunctionNameWithKind(node))
                        }
                    });
                }
                scopeInfo = scopeInfo.upper;
            }
            return {
                FunctionDeclaration: enterFunction,
                FunctionExpression: enterFunction,
                ArrowFunctionExpression: enterFunction,
                "FunctionDeclaration:exit": exitFunction,
                "FunctionExpression:exit": exitFunction,
                "ArrowFunctionExpression:exit": exitFunction,
                AwaitExpression() {
                    if (!scopeInfo) {
                        return;
                    }
                    scopeInfo.hasAwait = true;
                },
                ForOfStatement(node) {
                    if (!scopeInfo) {
                        return;
                    }
                    if (node.await) {
                        scopeInfo.hasAwait = true;
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/require-jsdoc.js */ function _(require, module, exports) {
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require JSDoc comments",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/require-jsdoc"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        require: {
                            type: "object",
                            properties: {
                                ClassDeclaration: {
                                    type: "boolean",
                                    default: false
                                },
                                MethodDefinition: {
                                    type: "boolean",
                                    default: false
                                },
                                FunctionDeclaration: {
                                    type: "boolean",
                                    default: true
                                },
                                ArrowFunctionExpression: {
                                    type: "boolean",
                                    default: false
                                },
                                FunctionExpression: {
                                    type: "boolean",
                                    default: false
                                }
                            },
                            additionalProperties: false,
                            default: {}
                        }
                    },
                    additionalProperties: false
                }
            ],
            deprecated: true,
            replacedBy: []
        },
        create(context) {
            const source = context.getSourceCode();
            const DEFAULT_OPTIONS = {
                FunctionDeclaration: true,
                MethodDefinition: false,
                ClassDeclaration: false,
                ArrowFunctionExpression: false,
                FunctionExpression: false
            };
            const options = Object.assign(DEFAULT_OPTIONS, context.options[0] && context.options[0].require);
            /**
             * Report the error message
             * @param {ASTNode} node node to report
             * @returns {void}
             */
            function report(node) {
                context.report({ node, message: "Missing JSDoc comment." });
            }
            /**
             * Check if the jsdoc comment is present or not.
             * @param {ASTNode} node node to examine
             * @returns {void}
             */
            function checkJsDoc(node) {
                const jsdocComment = source.getJSDocComment(node);
                if (!jsdocComment) {
                    report(node);
                }
            }
            return {
                FunctionDeclaration(node) {
                    if (options.FunctionDeclaration) {
                        checkJsDoc(node);
                    }
                },
                FunctionExpression(node) {
                    if ((options.MethodDefinition && node.parent.type === "MethodDefinition") ||
                        (options.FunctionExpression && (node.parent.type === "VariableDeclarator" || (node.parent.type === "Property" && node === node.parent.value)))) {
                        checkJsDoc(node);
                    }
                },
                ClassDeclaration(node) {
                    if (options.ClassDeclaration) {
                        checkJsDoc(node);
                    }
                },
                ArrowFunctionExpression(node) {
                    if (options.ArrowFunctionExpression && node.parent.type === "VariableDeclarator") {
                        checkJsDoc(node);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/require-unicode-regexp.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const { CALL, CONSTRUCT, ReferenceTracker, getStringIfConstant } = require(250) /* eslint-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce the use of `u` flag on RegExp",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/require-unicode-regexp"
            },
            messages: {
                requireUFlag: "Use the 'u' flag."
            },
            schema: []
        },
        create(context) {
            return {
                "Literal[regex]"(node) {
                    const flags = node.regex.flags || "";
                    if (!flags.includes("u")) {
                        context.report({ node, messageId: "requireUFlag" });
                    }
                },
                Program() {
                    const scope = context.getScope();
                    const tracker = new ReferenceTracker(scope);
                    const trackMap = {
                        RegExp: { [CALL]: true, [CONSTRUCT]: true }
                    };
                    for (const { node } of tracker.iterateGlobalReferences(trackMap)) {
                        const flagsNode = node.arguments[1];
                        const flags = getStringIfConstant(flagsNode, scope);
                        if (!flagsNode || (typeof flags === "string" && !flags.includes("u"))) {
                            context.report({ node, messageId: "requireUFlag" });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/require-yield.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require generator functions to contain `yield`",
                category: "ECMAScript 6",
                recommended: true,
                url: "https://eslint.org/docs/rules/require-yield"
            },
            schema: []
        },
        create(context) {
            const stack = [];
            /**
             * If the node is a generator function, start counting `yield` keywords.
             * @param {Node} node A function node to check.
             * @returns {void}
             */
            function beginChecking(node) {
                if (node.generator) {
                    stack.push(0);
                }
            }
            /**
             * If the node is a generator function, end counting `yield` keywords, then
             * reports result.
             * @param {Node} node A function node to check.
             * @returns {void}
             */
            function endChecking(node) {
                if (!node.generator) {
                    return;
                }
                const countYield = stack.pop();
                if (countYield === 0 && node.body.body.length > 0) {
                    context.report({ node, message: "This generator function does not have 'yield'." });
                }
            }
            return {
                FunctionDeclaration: beginChecking,
                "FunctionDeclaration:exit": endChecking,
                FunctionExpression: beginChecking,
                "FunctionExpression:exit": endChecking,
                // Increases the count of `yield` keyword.
                YieldExpression() {
                    /* istanbul ignore else */
                    if (stack.length > 0) {
                        stack[stack.length - 1] += 1;
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/rest-spread-spacing.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce spacing between rest and spread operators and their expressions",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/rest-spread-spacing"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["always", "never"]
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode(), alwaysSpace = context.options[0] === "always";
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Checks whitespace between rest/spread operators and their expressions
             * @param {ASTNode} node The node to check
             * @returns {void}
             */
            function checkWhiteSpace(node) {
                const operator = sourceCode.getFirstToken(node), nextToken = sourceCode.getTokenAfter(operator), hasWhitespace = sourceCode.isSpaceBetweenTokens(operator, nextToken);
                let type;
                switch (node.type) {
                    case "SpreadElement":
                        type = "spread";
                        if (node.parent.type === "ObjectExpression") {
                            type += " property";
                        }
                        break;
                    case "RestElement":
                        type = "rest";
                        if (node.parent.type === "ObjectPattern") {
                            type += " property";
                        }
                        break;
                    case "ExperimentalSpreadProperty":
                        type = "spread property";
                        break;
                    case "ExperimentalRestProperty":
                        type = "rest property";
                        break;
                    default:
                        return;
                }
                if (alwaysSpace && !hasWhitespace) {
                    context.report({
                        node,
                        loc: {
                            line: operator.loc.end.line,
                            column: operator.loc.end.column
                        },
                        message: "Expected whitespace after {{type}} operator.",
                        data: {
                            type
                        },
                        fix(fixer) {
                            return fixer.replaceTextRange([operator.range[1], nextToken.range[0]], " ");
                        }
                    });
                }
                else if (!alwaysSpace && hasWhitespace) {
                    context.report({
                        node,
                        loc: {
                            line: operator.loc.end.line,
                            column: operator.loc.end.column
                        },
                        message: "Unexpected whitespace after {{type}} operator.",
                        data: {
                            type
                        },
                        fix(fixer) {
                            return fixer.removeRange([operator.range[1], nextToken.range[0]]);
                        }
                    });
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                SpreadElement: checkWhiteSpace,
                RestElement: checkWhiteSpace,
                ExperimentalSpreadProperty: checkWhiteSpace,
                ExperimentalRestProperty: checkWhiteSpace
            };
        }
    };
},
/* eslint/lib/rules/semi.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const FixTracker = require(237) /* ./utils/fix-tracker */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow semicolons instead of ASI",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/semi"
            },
            fixable: "code",
            schema: {
                anyOf: [
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["never"]
                            },
                            {
                                type: "object",
                                properties: {
                                    beforeStatementContinuationChars: {
                                        enum: ["always", "any", "never"]
                                    }
                                },
                                additionalProperties: false
                            }
                        ],
                        minItems: 0,
                        maxItems: 2
                    },
                    {
                        type: "array",
                        items: [
                            {
                                enum: ["always"]
                            },
                            {
                                type: "object",
                                properties: {
                                    omitLastInOneLineBlock: { type: "boolean" }
                                },
                                additionalProperties: false
                            }
                        ],
                        minItems: 0,
                        maxItems: 2
                    }
                ]
            }
        },
        create(context) {
            const OPT_OUT_PATTERN = /^[-[(/+`]/u; // One of [(/+-`
            const options = context.options[1];
            const never = context.options[0] === "never";
            const exceptOneLine = Boolean(options && options.omitLastInOneLineBlock);
            const beforeStatementContinuationChars = options && options.beforeStatementContinuationChars || "any";
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Reports a semicolon error with appropriate location and message.
             * @param {ASTNode} node The node with an extra or missing semicolon.
             * @param {boolean} missing True if the semicolon is missing.
             * @returns {void}
             */
            function report(node, missing) {
                const lastToken = sourceCode.getLastToken(node);
                let message, fix, loc;
                if (!missing) {
                    message = "Missing semicolon.";
                    loc = {
                        start: lastToken.loc.end,
                        end: astUtils.getNextLocation(sourceCode, lastToken.loc.end)
                    };
                    fix = function (fixer) {
                        return fixer.insertTextAfter(lastToken, ";");
                    };
                }
                else {
                    message = "Extra semicolon.";
                    loc = lastToken.loc;
                    fix = function (fixer) {
                        /*
                         * Expand the replacement range to include the surrounding
                         * tokens to avoid conflicting with no-extra-semi.
                         * https://github.com/eslint/eslint/issues/7928
                         */
                        return new FixTracker(fixer, sourceCode)
                            .retainSurroundingTokens(lastToken)
                            .remove(lastToken);
                    };
                }
                context.report({
                    node,
                    loc,
                    message,
                    fix
                });
            }
            /**
             * Check whether a given semicolon token is redandant.
             * @param {Token} semiToken A semicolon token to check.
             * @returns {boolean} `true` if the next token is `;` or `}`.
             */
            function isRedundantSemi(semiToken) {
                const nextToken = sourceCode.getTokenAfter(semiToken);
                return (!nextToken ||
                    astUtils.isClosingBraceToken(nextToken) ||
                    astUtils.isSemicolonToken(nextToken));
            }
            /**
             * Check whether a given token is the closing brace of an arrow function.
             * @param {Token} lastToken A token to check.
             * @returns {boolean} `true` if the token is the closing brace of an arrow function.
             */
            function isEndOfArrowBlock(lastToken) {
                if (!astUtils.isClosingBraceToken(lastToken)) {
                    return false;
                }
                const node = sourceCode.getNodeByRangeIndex(lastToken.range[0]);
                return (node.type === "BlockStatement" &&
                    node.parent.type === "ArrowFunctionExpression");
            }
            /**
             * Check whether a given node is on the same line with the next token.
             * @param {Node} node A statement node to check.
             * @returns {boolean} `true` if the node is on the same line with the next token.
             */
            function isOnSameLineWithNextToken(node) {
                const prevToken = sourceCode.getLastToken(node, 1);
                const nextToken = sourceCode.getTokenAfter(node);
                return !!nextToken && astUtils.isTokenOnSameLine(prevToken, nextToken);
            }
            /**
             * Check whether a given node can connect the next line if the next line is unreliable.
             * @param {Node} node A statement node to check.
             * @returns {boolean} `true` if the node can connect the next line.
             */
            function maybeAsiHazardAfter(node) {
                const t = node.type;
                if (t === "DoWhileStatement" ||
                    t === "BreakStatement" ||
                    t === "ContinueStatement" ||
                    t === "DebuggerStatement" ||
                    t === "ImportDeclaration" ||
                    t === "ExportAllDeclaration") {
                    return false;
                }
                if (t === "ReturnStatement") {
                    return Boolean(node.argument);
                }
                if (t === "ExportNamedDeclaration") {
                    return Boolean(node.declaration);
                }
                if (isEndOfArrowBlock(sourceCode.getLastToken(node, 1))) {
                    return false;
                }
                return true;
            }
            /**
             * Check whether a given token can connect the previous statement.
             * @param {Token} token A token to check.
             * @returns {boolean} `true` if the token is one of `[`, `(`, `/`, `+`, `-`, ```, `++`, and `--`.
             */
            function maybeAsiHazardBefore(token) {
                return (Boolean(token) &&
                    OPT_OUT_PATTERN.test(token.value) &&
                    token.value !== "++" &&
                    token.value !== "--");
            }
            /**
             * Check if the semicolon of a given node is unnecessary, only true if:
             *   - next token is a valid statement divider (`;` or `}`).
             *   - next token is on a new line and the node is not connectable to the new line.
             * @param {Node} node A statement node to check.
             * @returns {boolean} whether the semicolon is unnecessary.
             */
            function canRemoveSemicolon(node) {
                if (isRedundantSemi(sourceCode.getLastToken(node))) {
                    return true; // `;;` or `;}`
                }
                if (isOnSameLineWithNextToken(node)) {
                    return false; // One liner.
                }
                if (beforeStatementContinuationChars === "never" && !maybeAsiHazardAfter(node)) {
                    return true; // ASI works. This statement doesn't connect to the next.
                }
                if (!maybeAsiHazardBefore(sourceCode.getTokenAfter(node))) {
                    return true; // ASI works. The next token doesn't connect to this statement.
                }
                return false;
            }
            /**
             * Checks a node to see if it's in a one-liner block statement.
             * @param {ASTNode} node The node to check.
             * @returns {boolean} whether the node is in a one-liner block statement.
             */
            function isOneLinerBlock(node) {
                const parent = node.parent;
                const nextToken = sourceCode.getTokenAfter(node);
                if (!nextToken || nextToken.value !== "}") {
                    return false;
                }
                return (!!parent &&
                    parent.type === "BlockStatement" &&
                    parent.loc.start.line === parent.loc.end.line);
            }
            /**
             * Checks a node to see if it's followed by a semicolon.
             * @param {ASTNode} node The node to check.
             * @returns {void}
             */
            function checkForSemicolon(node) {
                const isSemi = astUtils.isSemicolonToken(sourceCode.getLastToken(node));
                if (never) {
                    if (isSemi && canRemoveSemicolon(node)) {
                        report(node, true);
                    }
                    else if (!isSemi && beforeStatementContinuationChars === "always" && maybeAsiHazardBefore(sourceCode.getTokenAfter(node))) {
                        report(node);
                    }
                }
                else {
                    const oneLinerBlock = (exceptOneLine && isOneLinerBlock(node));
                    if (isSemi && oneLinerBlock) {
                        report(node, true);
                    }
                    else if (!isSemi && !oneLinerBlock) {
                        report(node);
                    }
                }
            }
            /**
             * Checks to see if there's a semicolon after a variable declaration.
             * @param {ASTNode} node The node to check.
             * @returns {void}
             */
            function checkForSemicolonForVariableDeclaration(node) {
                const parent = node.parent;
                if ((parent.type !== "ForStatement" || parent.init !== node) &&
                    (!/^For(?:In|Of)Statement/u.test(parent.type) || parent.left !== node)) {
                    checkForSemicolon(node);
                }
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                VariableDeclaration: checkForSemicolonForVariableDeclaration,
                ExpressionStatement: checkForSemicolon,
                ReturnStatement: checkForSemicolon,
                ThrowStatement: checkForSemicolon,
                DoWhileStatement: checkForSemicolon,
                DebuggerStatement: checkForSemicolon,
                BreakStatement: checkForSemicolon,
                ContinueStatement: checkForSemicolon,
                ImportDeclaration: checkForSemicolon,
                ExportAllDeclaration: checkForSemicolon,
                ExportNamedDeclaration(node) {
                    if (!node.declaration) {
                        checkForSemicolon(node);
                    }
                },
                ExportDefaultDeclaration(node) {
                    if (!/(?:Class|Function)Declaration/u.test(node.declaration.type)) {
                        checkForSemicolon(node);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/semi-spacing.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing before and after semicolons",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/semi-spacing"
            },
            fixable: "whitespace",
            schema: [
                {
                    type: "object",
                    properties: {
                        before: {
                            type: "boolean",
                            default: false
                        },
                        after: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const config = context.options[0], sourceCode = context.getSourceCode();
            let requireSpaceBefore = false, requireSpaceAfter = true;
            if (typeof config === "object") {
                requireSpaceBefore = config.before;
                requireSpaceAfter = config.after;
            }
            /**
             * Checks if a given token has leading whitespace.
             * @param {Object} token The token to check.
             * @returns {boolean} True if the given token has leading space, false if not.
             */
            function hasLeadingSpace(token) {
                const tokenBefore = sourceCode.getTokenBefore(token);
                return tokenBefore && astUtils.isTokenOnSameLine(tokenBefore, token) && sourceCode.isSpaceBetweenTokens(tokenBefore, token);
            }
            /**
             * Checks if a given token has trailing whitespace.
             * @param {Object} token The token to check.
             * @returns {boolean} True if the given token has trailing space, false if not.
             */
            function hasTrailingSpace(token) {
                const tokenAfter = sourceCode.getTokenAfter(token);
                return tokenAfter && astUtils.isTokenOnSameLine(token, tokenAfter) && sourceCode.isSpaceBetweenTokens(token, tokenAfter);
            }
            /**
             * Checks if the given token is the last token in its line.
             * @param {Token} token The token to check.
             * @returns {boolean} Whether or not the token is the last in its line.
             */
            function isLastTokenInCurrentLine(token) {
                const tokenAfter = sourceCode.getTokenAfter(token);
                return !(tokenAfter && astUtils.isTokenOnSameLine(token, tokenAfter));
            }
            /**
             * Checks if the given token is the first token in its line
             * @param {Token} token The token to check.
             * @returns {boolean} Whether or not the token is the first in its line.
             */
            function isFirstTokenInCurrentLine(token) {
                const tokenBefore = sourceCode.getTokenBefore(token);
                return !(tokenBefore && astUtils.isTokenOnSameLine(token, tokenBefore));
            }
            /**
             * Checks if the next token of a given token is a closing parenthesis.
             * @param {Token} token The token to check.
             * @returns {boolean} Whether or not the next token of a given token is a closing parenthesis.
             */
            function isBeforeClosingParen(token) {
                const nextToken = sourceCode.getTokenAfter(token);
                return (nextToken && astUtils.isClosingBraceToken(nextToken) || astUtils.isClosingParenToken(nextToken));
            }
            /**
             * Reports if the given token has invalid spacing.
             * @param {Token} token The semicolon token to check.
             * @param {ASTNode} node The corresponding node of the token.
             * @returns {void}
             */
            function checkSemicolonSpacing(token, node) {
                if (astUtils.isSemicolonToken(token)) {
                    const location = token.loc.start;
                    if (hasLeadingSpace(token)) {
                        if (!requireSpaceBefore) {
                            context.report({
                                node,
                                loc: location,
                                message: "Unexpected whitespace before semicolon.",
                                fix(fixer) {
                                    const tokenBefore = sourceCode.getTokenBefore(token);
                                    return fixer.removeRange([tokenBefore.range[1], token.range[0]]);
                                }
                            });
                        }
                    }
                    else {
                        if (requireSpaceBefore) {
                            context.report({
                                node,
                                loc: location,
                                message: "Missing whitespace before semicolon.",
                                fix(fixer) {
                                    return fixer.insertTextBefore(token, " ");
                                }
                            });
                        }
                    }
                    if (!isFirstTokenInCurrentLine(token) && !isLastTokenInCurrentLine(token) && !isBeforeClosingParen(token)) {
                        if (hasTrailingSpace(token)) {
                            if (!requireSpaceAfter) {
                                context.report({
                                    node,
                                    loc: location,
                                    message: "Unexpected whitespace after semicolon.",
                                    fix(fixer) {
                                        const tokenAfter = sourceCode.getTokenAfter(token);
                                        return fixer.removeRange([token.range[1], tokenAfter.range[0]]);
                                    }
                                });
                            }
                        }
                        else {
                            if (requireSpaceAfter) {
                                context.report({
                                    node,
                                    loc: location,
                                    message: "Missing whitespace after semicolon.",
                                    fix(fixer) {
                                        return fixer.insertTextAfter(token, " ");
                                    }
                                });
                            }
                        }
                    }
                }
            }
            /**
             * Checks the spacing of the semicolon with the assumption that the last token is the semicolon.
             * @param {ASTNode} node The node to check.
             * @returns {void}
             */
            function checkNode(node) {
                const token = sourceCode.getLastToken(node);
                checkSemicolonSpacing(token, node);
            }
            return {
                VariableDeclaration: checkNode,
                ExpressionStatement: checkNode,
                BreakStatement: checkNode,
                ContinueStatement: checkNode,
                DebuggerStatement: checkNode,
                ReturnStatement: checkNode,
                ThrowStatement: checkNode,
                ImportDeclaration: checkNode,
                ExportNamedDeclaration: checkNode,
                ExportAllDeclaration: checkNode,
                ExportDefaultDeclaration: checkNode,
                ForStatement(node) {
                    if (node.init) {
                        checkSemicolonSpacing(sourceCode.getTokenAfter(node.init), node);
                    }
                    if (node.test) {
                        checkSemicolonSpacing(sourceCode.getTokenAfter(node.test), node);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/semi-style.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    const SELECTOR = `:matches(${[
        "BreakStatement", "ContinueStatement", "DebuggerStatement",
        "DoWhileStatement", "ExportAllDeclaration",
        "ExportDefaultDeclaration", "ExportNamedDeclaration",
        "ExpressionStatement", "ImportDeclaration", "ReturnStatement",
        "ThrowStatement", "VariableDeclaration"
    ].join(",")})`;
    /**
     * Get the child node list of a given node.
     * This returns `Program#body`, `BlockStatement#body`, or `SwitchCase#consequent`.
     * This is used to check whether a node is the first/last child.
     * @param {Node} node A node to get child node list.
     * @returns {Node[]|null} The child node list.
     */
    function getChildren(node) {
        const t = node.type;
        if (t === "BlockStatement" || t === "Program") {
            return node.body;
        }
        if (t === "SwitchCase") {
            return node.consequent;
        }
        return null;
    }
    /**
     * Check whether a given node is the last statement in the parent block.
     * @param {Node} node A node to check.
     * @returns {boolean} `true` if the node is the last statement in the parent block.
     */
    function isLastChild(node) {
        const t = node.parent.type;
        if (t === "IfStatement" && node.parent.consequent === node && node.parent.alternate) { // before `else` keyword.
            return true;
        }
        if (t === "DoWhileStatement") { // before `while` keyword.
            return true;
        }
        const nodeList = getChildren(node.parent);
        return nodeList !== null && nodeList[nodeList.length - 1] === node; // before `}` or etc.
    }
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce location of semicolons",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/semi-style"
            },
            schema: [{ enum: ["last", "first"] }],
            fixable: "whitespace"
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const option = context.options[0] || "last";
            /**
             * Check the given semicolon token.
             * @param {Token} semiToken The semicolon token to check.
             * @param {"first"|"last"} expected The expected location to check.
             * @returns {void}
             */
            function check(semiToken, expected) {
                const prevToken = sourceCode.getTokenBefore(semiToken);
                const nextToken = sourceCode.getTokenAfter(semiToken);
                const prevIsSameLine = !prevToken || astUtils.isTokenOnSameLine(prevToken, semiToken);
                const nextIsSameLine = !nextToken || astUtils.isTokenOnSameLine(semiToken, nextToken);
                if ((expected === "last" && !prevIsSameLine) || (expected === "first" && !nextIsSameLine)) {
                    context.report({
                        loc: semiToken.loc,
                        message: "Expected this semicolon to be at {{pos}}.",
                        data: {
                            pos: (expected === "last")
                                ? "the end of the previous line"
                                : "the beginning of the next line"
                        },
                        fix(fixer) {
                            if (prevToken && nextToken && sourceCode.commentsExistBetween(prevToken, nextToken)) {
                                return null;
                            }
                            const start = prevToken ? prevToken.range[1] : semiToken.range[0];
                            const end = nextToken ? nextToken.range[0] : semiToken.range[1];
                            const text = (expected === "last") ? ";\n" : "\n;";
                            return fixer.replaceTextRange([start, end], text);
                        }
                    });
                }
            }
            return {
                [SELECTOR](node) {
                    if (option === "first" && isLastChild(node)) {
                        return;
                    }
                    const lastToken = sourceCode.getLastToken(node);
                    if (astUtils.isSemicolonToken(lastToken)) {
                        check(lastToken, option);
                    }
                },
                ForStatement(node) {
                    const firstSemi = node.init && sourceCode.getTokenAfter(node.init, astUtils.isSemicolonToken);
                    const secondSemi = node.test && sourceCode.getTokenAfter(node.test, astUtils.isSemicolonToken);
                    if (firstSemi) {
                        check(firstSemi, "last");
                    }
                    if (secondSemi) {
                        check(secondSemi, "last");
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/sort-imports.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce sorted import declarations within modules",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/sort-imports"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        ignoreCase: {
                            type: "boolean",
                            default: false
                        },
                        memberSyntaxSortOrder: {
                            type: "array",
                            items: {
                                enum: ["none", "all", "multiple", "single"]
                            },
                            uniqueItems: true,
                            minItems: 4,
                            maxItems: 4
                        },
                        ignoreDeclarationSort: {
                            type: "boolean",
                            default: false
                        },
                        ignoreMemberSort: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "code"
        },
        create(context) {
            const configuration = context.options[0] || {}, ignoreCase = configuration.ignoreCase || false, ignoreDeclarationSort = configuration.ignoreDeclarationSort || false, ignoreMemberSort = configuration.ignoreMemberSort || false, memberSyntaxSortOrder = configuration.memberSyntaxSortOrder || ["none", "all", "multiple", "single"], sourceCode = context.getSourceCode();
            let previousDeclaration = null;
            /**
             * Gets the used member syntax style.
             *
             * import "my-module.js" --> none
             * import * as myModule from "my-module.js" --> all
             * import {myMember} from "my-module.js" --> single
             * import {foo, bar} from  "my-module.js" --> multiple
             * @param {ASTNode} node the ImportDeclaration node.
             * @returns {string} used member parameter style, ["all", "multiple", "single"]
             */
            function usedMemberSyntax(node) {
                if (node.specifiers.length === 0) {
                    return "none";
                }
                if (node.specifiers[0].type === "ImportNamespaceSpecifier") {
                    return "all";
                }
                if (node.specifiers.length === 1) {
                    return "single";
                }
                return "multiple";
            }
            /**
             * Gets the group by member parameter index for given declaration.
             * @param {ASTNode} node the ImportDeclaration node.
             * @returns {number} the declaration group by member index.
             */
            function getMemberParameterGroupIndex(node) {
                return memberSyntaxSortOrder.indexOf(usedMemberSyntax(node));
            }
            /**
             * Gets the local name of the first imported module.
             * @param {ASTNode} node the ImportDeclaration node.
             * @returns {?string} the local name of the first imported module.
             */
            function getFirstLocalMemberName(node) {
                if (node.specifiers[0]) {
                    return node.specifiers[0].local.name;
                }
                return null;
            }
            return {
                ImportDeclaration(node) {
                    if (!ignoreDeclarationSort) {
                        if (previousDeclaration) {
                            const currentMemberSyntaxGroupIndex = getMemberParameterGroupIndex(node), previousMemberSyntaxGroupIndex = getMemberParameterGroupIndex(previousDeclaration);
                            let currentLocalMemberName = getFirstLocalMemberName(node), previousLocalMemberName = getFirstLocalMemberName(previousDeclaration);
                            if (ignoreCase) {
                                previousLocalMemberName = previousLocalMemberName && previousLocalMemberName.toLowerCase();
                                currentLocalMemberName = currentLocalMemberName && currentLocalMemberName.toLowerCase();
                            }
                            /*
                             * When the current declaration uses a different member syntax,
                             * then check if the ordering is correct.
                             * Otherwise, make a default string compare (like rule sort-vars to be consistent) of the first used local member name.
                             */
                            if (currentMemberSyntaxGroupIndex !== previousMemberSyntaxGroupIndex) {
                                if (currentMemberSyntaxGroupIndex < previousMemberSyntaxGroupIndex) {
                                    context.report({
                                        node,
                                        message: "Expected '{{syntaxA}}' syntax before '{{syntaxB}}' syntax.",
                                        data: {
                                            syntaxA: memberSyntaxSortOrder[currentMemberSyntaxGroupIndex],
                                            syntaxB: memberSyntaxSortOrder[previousMemberSyntaxGroupIndex]
                                        }
                                    });
                                }
                            }
                            else {
                                if (previousLocalMemberName &&
                                    currentLocalMemberName &&
                                    currentLocalMemberName < previousLocalMemberName) {
                                    context.report({
                                        node,
                                        message: "Imports should be sorted alphabetically."
                                    });
                                }
                            }
                        }
                        previousDeclaration = node;
                    }
                    if (!ignoreMemberSort) {
                        const importSpecifiers = node.specifiers.filter(specifier => specifier.type === "ImportSpecifier");
                        const getSortableName = ignoreCase ? specifier => specifier.local.name.toLowerCase() : specifier => specifier.local.name;
                        const firstUnsortedIndex = importSpecifiers.map(getSortableName).findIndex((name, index, array) => array[index - 1] > name);
                        if (firstUnsortedIndex !== -1) {
                            context.report({
                                node: importSpecifiers[firstUnsortedIndex],
                                message: "Member '{{memberName}}' of the import declaration should be sorted alphabetically.",
                                data: { memberName: importSpecifiers[firstUnsortedIndex].local.name },
                                fix(fixer) {
                                    if (importSpecifiers.some(specifier => sourceCode.getCommentsBefore(specifier).length || sourceCode.getCommentsAfter(specifier).length)) {
                                        // If there are comments in the ImportSpecifier list, don't rearrange the specifiers.
                                        return null;
                                    }
                                    return fixer.replaceTextRange([importSpecifiers[0].range[0], importSpecifiers[importSpecifiers.length - 1].range[1]], importSpecifiers
                                        // Clone the importSpecifiers array to avoid mutating it
                                        .slice()
                                        // Sort the array into the desired order
                                        .sort((specifierA, specifierB) => {
                                        const aName = getSortableName(specifierA);
                                        const bName = getSortableName(specifierB);
                                        return aName > bName ? 1 : -1;
                                    })
                                        // Build a string out of the sorted list of import specifiers and the text between the originals
                                        .reduce((sourceText, specifier, index) => {
                                        const textAfterSpecifier = index === importSpecifiers.length - 1
                                            ? ""
                                            : sourceCode.getText().slice(importSpecifiers[index].range[1], importSpecifiers[index + 1].range[0]);
                                        return sourceText + sourceCode.getText(specifier) + textAfterSpecifier;
                                    }, ""));
                                }
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/sort-keys.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */, naturalCompare = require(397) /* natural-compare */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Gets the property name of the given `Property` node.
     *
     * - If the property's key is an `Identifier` node, this returns the key's name
     *   whether it's a computed property or not.
     * - If the property has a static name, this returns the static name.
     * - Otherwise, this returns null.
     * @param {ASTNode} node The `Property` node to get.
     * @returns {string|null} The property name or null.
     * @private
     */
    function getPropertyName(node) {
        const staticName = astUtils.getStaticPropertyName(node);
        if (staticName !== null) {
            return staticName;
        }
        return node.key.name || null;
    }
    /**
     * Functions which check that the given 2 names are in specific order.
     *
     * Postfix `I` is meant insensitive.
     * Postfix `N` is meant natual.
     * @private
     */
    const isValidOrders = {
        asc(a, b) {
            return a <= b;
        },
        ascI(a, b) {
            return a.toLowerCase() <= b.toLowerCase();
        },
        ascN(a, b) {
            return naturalCompare(a, b) <= 0;
        },
        ascIN(a, b) {
            return naturalCompare(a.toLowerCase(), b.toLowerCase()) <= 0;
        },
        desc(a, b) {
            return isValidOrders.asc(b, a);
        },
        descI(a, b) {
            return isValidOrders.ascI(b, a);
        },
        descN(a, b) {
            return isValidOrders.ascN(b, a);
        },
        descIN(a, b) {
            return isValidOrders.ascIN(b, a);
        }
    };
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require object keys to be sorted",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/sort-keys"
            },
            schema: [
                {
                    enum: ["asc", "desc"]
                },
                {
                    type: "object",
                    properties: {
                        caseSensitive: {
                            type: "boolean",
                            default: true
                        },
                        natural: {
                            type: "boolean",
                            default: false
                        },
                        minKeys: {
                            type: "integer",
                            minimum: 2,
                            default: 2
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            // Parse options.
            const order = context.options[0] || "asc";
            const options = context.options[1];
            const insensitive = options && options.caseSensitive === false;
            const natual = options && options.natural;
            const minKeys = options && options.minKeys;
            const isValidOrder = isValidOrders[order + (insensitive ? "I" : "") + (natual ? "N" : "")];
            // The stack to save the previous property's name for each object literals.
            let stack = null;
            return {
                ObjectExpression(node) {
                    stack = {
                        upper: stack,
                        prevName: null,
                        numKeys: node.properties.length
                    };
                },
                "ObjectExpression:exit"() {
                    stack = stack.upper;
                },
                SpreadElement(node) {
                    if (node.parent.type === "ObjectExpression") {
                        stack.prevName = null;
                    }
                },
                Property(node) {
                    if (node.parent.type === "ObjectPattern") {
                        return;
                    }
                    const prevName = stack.prevName;
                    const numKeys = stack.numKeys;
                    const thisName = getPropertyName(node);
                    if (thisName !== null) {
                        stack.prevName = thisName;
                    }
                    if (prevName === null || thisName === null || numKeys < minKeys) {
                        return;
                    }
                    if (!isValidOrder(prevName, thisName)) {
                        context.report({
                            node,
                            loc: node.key.loc,
                            message: "Expected object keys to be in {{natual}}{{insensitive}}{{order}}ending order. '{{thisName}}' should be before '{{prevName}}'.",
                            data: {
                                thisName,
                                prevName,
                                order,
                                insensitive: insensitive ? "insensitive " : "",
                                natual: natual ? "natural " : ""
                            }
                        });
                    }
                }
            };
        }
    };
},
/* natural-compare/index.js */ function _(require, module, exports) {
    /*
     * @version    1.4.0
     * @date       2015-10-26
     * @stability  3 - Stable
     * @author     Lauri Rooden (https://github.com/litejs/natural-compare-lite)
     * @license    MIT License
     */
    var naturalCompare = function (a, b) {
        var i, codeA, codeB = 1, posA = 0, posB = 0, alphabet = String.alphabet;
        function getCode(str, pos, code) {
            if (code) {
                for (i = pos; code = getCode(str, i), code < 76 && code > 65;)
                    ++i;
                return +str.slice(pos - 1, i);
            }
            code = alphabet && alphabet.indexOf(str.charAt(pos));
            return code > -1 ? code + 76 : ((code = str.charCodeAt(pos) || 0), code < 45 || code > 127) ? code
                : code < 46 ? 65 // -
                    : code < 48 ? code - 1
                        : code < 58 ? code + 18 // 0-9
                            : code < 65 ? code - 11
                                : code < 91 ? code + 11 // A-Z
                                    : code < 97 ? code - 37
                                        : code < 123 ? code + 5 // a-z
                                            : code - 63;
        }
        if ((a += "") != (b += ""))
            for (; codeB;) {
                codeA = getCode(a, posA++);
                codeB = getCode(b, posB++);
                if (codeA < 76 && codeB < 76 && codeA > 66 && codeB > 66) {
                    codeA = getCode(a, posA, posA);
                    codeB = getCode(b, posB, posA = i);
                    posB = i;
                }
                if (codeA != codeB)
                    return (codeA < codeB) ? -1 : 1;
            }
        return 0;
    };
    try {
        module.exports = naturalCompare;
    }
    catch (e) {
        String.naturalCompare = naturalCompare;
    }
},
/* eslint/lib/rules/sort-vars.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require variables within the same declaration block to be sorted",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/sort-vars"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        ignoreCase: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "code"
        },
        create(context) {
            const configuration = context.options[0] || {}, ignoreCase = configuration.ignoreCase || false, sourceCode = context.getSourceCode();
            return {
                VariableDeclaration(node) {
                    const idDeclarations = node.declarations.filter(decl => decl.id.type === "Identifier");
                    const getSortableName = ignoreCase ? decl => decl.id.name.toLowerCase() : decl => decl.id.name;
                    const unfixable = idDeclarations.some(decl => decl.init !== null && decl.init.type !== "Literal");
                    let fixed = false;
                    idDeclarations.slice(1).reduce((memo, decl) => {
                        const lastVariableName = getSortableName(memo), currentVariableName = getSortableName(decl);
                        if (currentVariableName < lastVariableName) {
                            context.report({
                                node: decl,
                                message: "Variables within the same declaration block should be sorted alphabetically.",
                                fix(fixer) {
                                    if (unfixable || fixed) {
                                        return null;
                                    }
                                    return fixer.replaceTextRange([idDeclarations[0].range[0], idDeclarations[idDeclarations.length - 1].range[1]], idDeclarations
                                        // Clone the idDeclarations array to avoid mutating it
                                        .slice()
                                        // Sort the array into the desired order
                                        .sort((declA, declB) => {
                                        const aName = getSortableName(declA);
                                        const bName = getSortableName(declB);
                                        return aName > bName ? 1 : -1;
                                    })
                                        // Build a string out of the sorted list of identifier declarations and the text between the originals
                                        .reduce((sourceText, identifier, index) => {
                                        const textAfterIdentifier = index === idDeclarations.length - 1
                                            ? ""
                                            : sourceCode.getText().slice(idDeclarations[index].range[1], idDeclarations[index + 1].range[0]);
                                        return sourceText + sourceCode.getText(identifier) + textAfterIdentifier;
                                    }, ""));
                                }
                            });
                            fixed = true;
                            return memo;
                        }
                        return decl;
                    }, idDeclarations[0]);
                }
            };
        }
    };
},
/* eslint/lib/rules/space-before-blocks.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing before blocks",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/space-before-blocks"
            },
            fixable: "whitespace",
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["always", "never"]
                        },
                        {
                            type: "object",
                            properties: {
                                keywords: {
                                    enum: ["always", "never", "off"]
                                },
                                functions: {
                                    enum: ["always", "never", "off"]
                                },
                                classes: {
                                    enum: ["always", "never", "off"]
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ]
        },
        create(context) {
            const config = context.options[0], sourceCode = context.getSourceCode();
            let alwaysFunctions = true, alwaysKeywords = true, alwaysClasses = true, neverFunctions = false, neverKeywords = false, neverClasses = false;
            if (typeof config === "object") {
                alwaysFunctions = config.functions === "always";
                alwaysKeywords = config.keywords === "always";
                alwaysClasses = config.classes === "always";
                neverFunctions = config.functions === "never";
                neverKeywords = config.keywords === "never";
                neverClasses = config.classes === "never";
            }
            else if (config === "never") {
                alwaysFunctions = false;
                alwaysKeywords = false;
                alwaysClasses = false;
                neverFunctions = true;
                neverKeywords = true;
                neverClasses = true;
            }
            /**
             * Checks whether or not a given token is an arrow operator (=>) or a keyword
             * in order to avoid to conflict with `arrow-spacing` and `keyword-spacing`.
             * @param {Token} token A token to check.
             * @returns {boolean} `true` if the token is an arrow operator.
             */
            function isConflicted(token) {
                return (token.type === "Punctuator" && token.value === "=>") || token.type === "Keyword";
            }
            /**
             * Checks the given BlockStatement node has a preceding space if it doesn’t start on a new line.
             * @param {ASTNode|Token} node The AST node of a BlockStatement.
             * @returns {void} undefined.
             */
            function checkPrecedingSpace(node) {
                const precedingToken = sourceCode.getTokenBefore(node);
                if (precedingToken && !isConflicted(precedingToken) && astUtils.isTokenOnSameLine(precedingToken, node)) {
                    const hasSpace = sourceCode.isSpaceBetweenTokens(precedingToken, node);
                    const parent = context.getAncestors().pop();
                    let requireSpace;
                    let requireNoSpace;
                    if (parent.type === "FunctionExpression" || parent.type === "FunctionDeclaration") {
                        requireSpace = alwaysFunctions;
                        requireNoSpace = neverFunctions;
                    }
                    else if (node.type === "ClassBody") {
                        requireSpace = alwaysClasses;
                        requireNoSpace = neverClasses;
                    }
                    else {
                        requireSpace = alwaysKeywords;
                        requireNoSpace = neverKeywords;
                    }
                    if (requireSpace && !hasSpace) {
                        context.report({
                            node,
                            message: "Missing space before opening brace.",
                            fix(fixer) {
                                return fixer.insertTextBefore(node, " ");
                            }
                        });
                    }
                    else if (requireNoSpace && hasSpace) {
                        context.report({
                            node,
                            message: "Unexpected space before opening brace.",
                            fix(fixer) {
                                return fixer.removeRange([precedingToken.range[1], node.range[0]]);
                            }
                        });
                    }
                }
            }
            /**
             * Checks if the CaseBlock of an given SwitchStatement node has a preceding space.
             * @param {ASTNode} node The node of a SwitchStatement.
             * @returns {void} undefined.
             */
            function checkSpaceBeforeCaseBlock(node) {
                const cases = node.cases;
                let openingBrace;
                if (cases.length > 0) {
                    openingBrace = sourceCode.getTokenBefore(cases[0]);
                }
                else {
                    openingBrace = sourceCode.getLastToken(node, 1);
                }
                checkPrecedingSpace(openingBrace);
            }
            return {
                BlockStatement: checkPrecedingSpace,
                ClassBody: checkPrecedingSpace,
                SwitchStatement: checkSpaceBeforeCaseBlock
            };
        }
    };
},
/* eslint/lib/rules/space-before-function-paren.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing before `function` definition opening parenthesis",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/space-before-function-paren"
            },
            fixable: "whitespace",
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["always", "never"]
                        },
                        {
                            type: "object",
                            properties: {
                                anonymous: {
                                    enum: ["always", "never", "ignore"]
                                },
                                named: {
                                    enum: ["always", "never", "ignore"]
                                },
                                asyncArrow: {
                                    enum: ["always", "never", "ignore"]
                                }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const baseConfig = typeof context.options[0] === "string" ? context.options[0] : "always";
            const overrideConfig = typeof context.options[0] === "object" ? context.options[0] : {};
            /**
             * Determines whether a function has a name.
             * @param {ASTNode} node The function node.
             * @returns {boolean} Whether the function has a name.
             */
            function isNamedFunction(node) {
                if (node.id) {
                    return true;
                }
                const parent = node.parent;
                return parent.type === "MethodDefinition" ||
                    (parent.type === "Property" &&
                        (parent.kind === "get" ||
                            parent.kind === "set" ||
                            parent.method));
            }
            /**
             * Gets the config for a given function
             * @param {ASTNode} node The function node
             * @returns {string} "always", "never", or "ignore"
             */
            function getConfigForFunction(node) {
                if (node.type === "ArrowFunctionExpression") {
                    // Always ignore non-async functions and arrow functions without parens, e.g. async foo => bar
                    if (node.async && astUtils.isOpeningParenToken(sourceCode.getFirstToken(node, { skip: 1 }))) {
                        return overrideConfig.asyncArrow || baseConfig;
                    }
                }
                else if (isNamedFunction(node)) {
                    return overrideConfig.named || baseConfig;
                    // `generator-star-spacing` should warn anonymous generators. E.g. `function* () {}`
                }
                else if (!node.generator) {
                    return overrideConfig.anonymous || baseConfig;
                }
                return "ignore";
            }
            /**
             * Checks the parens of a function node
             * @param {ASTNode} node A function node
             * @returns {void}
             */
            function checkFunction(node) {
                const functionConfig = getConfigForFunction(node);
                if (functionConfig === "ignore") {
                    return;
                }
                const rightToken = sourceCode.getFirstToken(node, astUtils.isOpeningParenToken);
                const leftToken = sourceCode.getTokenBefore(rightToken);
                const hasSpacing = sourceCode.isSpaceBetweenTokens(leftToken, rightToken);
                if (hasSpacing && functionConfig === "never") {
                    context.report({
                        node,
                        loc: leftToken.loc.end,
                        message: "Unexpected space before function parentheses.",
                        fix(fixer) {
                            const comments = sourceCode.getCommentsBefore(rightToken);
                            // Don't fix anything if there's a single line comment between the left and the right token
                            if (comments.some(comment => comment.type === "Line")) {
                                return null;
                            }
                            return fixer.replaceTextRange([leftToken.range[1], rightToken.range[0]], comments.reduce((text, comment) => text + sourceCode.getText(comment), ""));
                        }
                    });
                }
                else if (!hasSpacing && functionConfig === "always") {
                    context.report({
                        node,
                        loc: leftToken.loc.end,
                        message: "Missing space before function parentheses.",
                        fix: fixer => fixer.insertTextAfter(leftToken, " ")
                    });
                }
            }
            return {
                ArrowFunctionExpression: checkFunction,
                FunctionDeclaration: checkFunction,
                FunctionExpression: checkFunction
            };
        }
    };
},
/* eslint/lib/rules/space-in-parens.js */ function _(require, module, exports) {
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing inside parentheses",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/space-in-parens"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["always", "never"]
                },
                {
                    type: "object",
                    properties: {
                        exceptions: {
                            type: "array",
                            items: {
                                enum: ["{}", "[]", "()", "empty"]
                            },
                            uniqueItems: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                missingOpeningSpace: "There must be a space after this paren.",
                missingClosingSpace: "There must be a space before this paren.",
                rejectedOpeningSpace: "There should be no space after this paren.",
                rejectedClosingSpace: "There should be no space before this paren."
            }
        },
        create(context) {
            const ALWAYS = context.options[0] === "always", exceptionsArrayOptions = (context.options[1] && context.options[1].exceptions) || [], options = {};
            let exceptions;
            if (exceptionsArrayOptions.length) {
                options.braceException = exceptionsArrayOptions.includes("{}");
                options.bracketException = exceptionsArrayOptions.includes("[]");
                options.parenException = exceptionsArrayOptions.includes("()");
                options.empty = exceptionsArrayOptions.includes("empty");
            }
            /**
             * Produces an object with the opener and closer exception values
             * @returns {Object} `openers` and `closers` exception values
             * @private
             */
            function getExceptions() {
                const openers = [], closers = [];
                if (options.braceException) {
                    openers.push("{");
                    closers.push("}");
                }
                if (options.bracketException) {
                    openers.push("[");
                    closers.push("]");
                }
                if (options.parenException) {
                    openers.push("(");
                    closers.push(")");
                }
                if (options.empty) {
                    openers.push(")");
                    closers.push("(");
                }
                return {
                    openers,
                    closers
                };
            }
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            const sourceCode = context.getSourceCode();
            /**
             * Determines if a token is one of the exceptions for the opener paren
             * @param {Object} token The token to check
             * @returns {boolean} True if the token is one of the exceptions for the opener paren
             */
            function isOpenerException(token) {
                return exceptions.openers.includes(token.value);
            }
            /**
             * Determines if a token is one of the exceptions for the closer paren
             * @param {Object} token The token to check
             * @returns {boolean} True if the token is one of the exceptions for the closer paren
             */
            function isCloserException(token) {
                return exceptions.closers.includes(token.value);
            }
            /**
             * Determines if an opening paren is immediately followed by a required space
             * @param {Object} openingParenToken The paren token
             * @param {Object} tokenAfterOpeningParen The token after it
             * @returns {boolean} True if the opening paren is missing a required space
             */
            function openerMissingSpace(openingParenToken, tokenAfterOpeningParen) {
                if (sourceCode.isSpaceBetweenTokens(openingParenToken, tokenAfterOpeningParen)) {
                    return false;
                }
                if (!options.empty && astUtils.isClosingParenToken(tokenAfterOpeningParen)) {
                    return false;
                }
                if (ALWAYS) {
                    return !isOpenerException(tokenAfterOpeningParen);
                }
                return isOpenerException(tokenAfterOpeningParen);
            }
            /**
             * Determines if an opening paren is immediately followed by a disallowed space
             * @param {Object} openingParenToken The paren token
             * @param {Object} tokenAfterOpeningParen The token after it
             * @returns {boolean} True if the opening paren has a disallowed space
             */
            function openerRejectsSpace(openingParenToken, tokenAfterOpeningParen) {
                if (!astUtils.isTokenOnSameLine(openingParenToken, tokenAfterOpeningParen)) {
                    return false;
                }
                if (tokenAfterOpeningParen.type === "Line") {
                    return false;
                }
                if (!sourceCode.isSpaceBetweenTokens(openingParenToken, tokenAfterOpeningParen)) {
                    return false;
                }
                if (ALWAYS) {
                    return isOpenerException(tokenAfterOpeningParen);
                }
                return !isOpenerException(tokenAfterOpeningParen);
            }
            /**
             * Determines if a closing paren is immediately preceeded by a required space
             * @param {Object} tokenBeforeClosingParen The token before the paren
             * @param {Object} closingParenToken The paren token
             * @returns {boolean} True if the closing paren is missing a required space
             */
            function closerMissingSpace(tokenBeforeClosingParen, closingParenToken) {
                if (sourceCode.isSpaceBetweenTokens(tokenBeforeClosingParen, closingParenToken)) {
                    return false;
                }
                if (!options.empty && astUtils.isOpeningParenToken(tokenBeforeClosingParen)) {
                    return false;
                }
                if (ALWAYS) {
                    return !isCloserException(tokenBeforeClosingParen);
                }
                return isCloserException(tokenBeforeClosingParen);
            }
            /**
             * Determines if a closer paren is immediately preceeded by a disallowed space
             * @param {Object} tokenBeforeClosingParen The token before the paren
             * @param {Object} closingParenToken The paren token
             * @returns {boolean} True if the closing paren has a disallowed space
             */
            function closerRejectsSpace(tokenBeforeClosingParen, closingParenToken) {
                if (!astUtils.isTokenOnSameLine(tokenBeforeClosingParen, closingParenToken)) {
                    return false;
                }
                if (!sourceCode.isSpaceBetweenTokens(tokenBeforeClosingParen, closingParenToken)) {
                    return false;
                }
                if (ALWAYS) {
                    return isCloserException(tokenBeforeClosingParen);
                }
                return !isCloserException(tokenBeforeClosingParen);
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                Program: function checkParenSpaces(node) {
                    exceptions = getExceptions();
                    const tokens = sourceCode.tokensAndComments;
                    tokens.forEach((token, i) => {
                        const prevToken = tokens[i - 1];
                        const nextToken = tokens[i + 1];
                        // if token is not an opening or closing paren token, do nothing
                        if (!astUtils.isOpeningParenToken(token) && !astUtils.isClosingParenToken(token)) {
                            return;
                        }
                        // if token is an opening paren and is not followed by a required space
                        if (token.value === "(" && openerMissingSpace(token, nextToken)) {
                            context.report({
                                node,
                                loc: token.loc,
                                messageId: "missingOpeningSpace",
                                fix(fixer) {
                                    return fixer.insertTextAfter(token, " ");
                                }
                            });
                        }
                        // if token is an opening paren and is followed by a disallowed space
                        if (token.value === "(" && openerRejectsSpace(token, nextToken)) {
                            context.report({
                                node,
                                loc: { start: token.loc.end, end: nextToken.loc.start },
                                messageId: "rejectedOpeningSpace",
                                fix(fixer) {
                                    return fixer.removeRange([token.range[1], nextToken.range[0]]);
                                }
                            });
                        }
                        // if token is a closing paren and is not preceded by a required space
                        if (token.value === ")" && closerMissingSpace(prevToken, token)) {
                            context.report({
                                node,
                                loc: token.loc,
                                messageId: "missingClosingSpace",
                                fix(fixer) {
                                    return fixer.insertTextBefore(token, " ");
                                }
                            });
                        }
                        // if token is a closing paren and is preceded by a disallowed space
                        if (token.value === ")" && closerRejectsSpace(prevToken, token)) {
                            context.report({
                                node,
                                loc: { start: prevToken.loc.end, end: token.loc.start },
                                messageId: "rejectedClosingSpace",
                                fix(fixer) {
                                    return fixer.removeRange([prevToken.range[1], token.range[0]]);
                                }
                            });
                        }
                    });
                }
            };
        }
    };
},
/* eslint/lib/rules/space-infix-ops.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require spacing around infix operators",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/space-infix-ops"
            },
            fixable: "whitespace",
            schema: [
                {
                    type: "object",
                    properties: {
                        int32Hint: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const int32Hint = context.options[0] ? context.options[0].int32Hint === true : false;
            const sourceCode = context.getSourceCode();
            /**
             * Returns the first token which violates the rule
             * @param {ASTNode} left The left node of the main node
             * @param {ASTNode} right The right node of the main node
             * @param {string} op The operator of the main node
             * @returns {Object} The violator token or null
             * @private
             */
            function getFirstNonSpacedToken(left, right, op) {
                const operator = sourceCode.getFirstTokenBetween(left, right, token => token.value === op);
                const prev = sourceCode.getTokenBefore(operator);
                const next = sourceCode.getTokenAfter(operator);
                if (!sourceCode.isSpaceBetweenTokens(prev, operator) || !sourceCode.isSpaceBetweenTokens(operator, next)) {
                    return operator;
                }
                return null;
            }
            /**
             * Reports an AST node as a rule violation
             * @param {ASTNode} mainNode The node to report
             * @param {Object} culpritToken The token which has a problem
             * @returns {void}
             * @private
             */
            function report(mainNode, culpritToken) {
                context.report({
                    node: mainNode,
                    loc: culpritToken.loc,
                    message: "Operator '{{operator}}' must be spaced.",
                    data: {
                        operator: culpritToken.value
                    },
                    fix(fixer) {
                        const previousToken = sourceCode.getTokenBefore(culpritToken);
                        const afterToken = sourceCode.getTokenAfter(culpritToken);
                        let fixString = "";
                        if (culpritToken.range[0] - previousToken.range[1] === 0) {
                            fixString = " ";
                        }
                        fixString += culpritToken.value;
                        if (afterToken.range[0] - culpritToken.range[1] === 0) {
                            fixString += " ";
                        }
                        return fixer.replaceText(culpritToken, fixString);
                    }
                });
            }
            /**
             * Check if the node is binary then report
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkBinary(node) {
                const leftNode = (node.left.typeAnnotation) ? node.left.typeAnnotation : node.left;
                const rightNode = node.right;
                // search for = in AssignmentPattern nodes
                const operator = node.operator || "=";
                const nonSpacedNode = getFirstNonSpacedToken(leftNode, rightNode, operator);
                if (nonSpacedNode) {
                    if (!(int32Hint && sourceCode.getText(node).endsWith("|0"))) {
                        report(node, nonSpacedNode);
                    }
                }
            }
            /**
             * Check if the node is conditional
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkConditional(node) {
                const nonSpacedConsequesntNode = getFirstNonSpacedToken(node.test, node.consequent, "?");
                const nonSpacedAlternateNode = getFirstNonSpacedToken(node.consequent, node.alternate, ":");
                if (nonSpacedConsequesntNode) {
                    report(node, nonSpacedConsequesntNode);
                }
                else if (nonSpacedAlternateNode) {
                    report(node, nonSpacedAlternateNode);
                }
            }
            /**
             * Check if the node is a variable
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkVar(node) {
                const leftNode = (node.id.typeAnnotation) ? node.id.typeAnnotation : node.id;
                const rightNode = node.init;
                if (rightNode) {
                    const nonSpacedNode = getFirstNonSpacedToken(leftNode, rightNode, "=");
                    if (nonSpacedNode) {
                        report(node, nonSpacedNode);
                    }
                }
            }
            return {
                AssignmentExpression: checkBinary,
                AssignmentPattern: checkBinary,
                BinaryExpression: checkBinary,
                LogicalExpression: checkBinary,
                ConditionalExpression: checkConditional,
                VariableDeclarator: checkVar
            };
        }
    };
},
/* eslint/lib/rules/space-unary-ops.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce consistent spacing before or after unary operators",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/space-unary-ops"
            },
            fixable: "whitespace",
            schema: [
                {
                    type: "object",
                    properties: {
                        words: {
                            type: "boolean",
                            default: true
                        },
                        nonwords: {
                            type: "boolean",
                            default: false
                        },
                        overrides: {
                            type: "object",
                            additionalProperties: {
                                type: "boolean"
                            }
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                unexpectedBefore: "Unexpected space before unary operator '{{operator}}'.",
                unexpectedAfter: "Unexpected space after unary operator '{{operator}}'.",
                unexpectedAfterWord: "Unexpected space after unary word operator '{{word}}'.",
                wordOperator: "Unary word operator '{{word}}' must be followed by whitespace.",
                operator: "Unary operator '{{operator}}' must be followed by whitespace.",
                beforeUnaryExpressions: "Space is required before unary expressions '{{token}}'."
            }
        },
        create(context) {
            const options = context.options[0] || { words: true, nonwords: false };
            const sourceCode = context.getSourceCode();
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            /**
             * Check if the node is the first "!" in a "!!" convert to Boolean expression
             * @param {ASTnode} node AST node
             * @returns {boolean} Whether or not the node is first "!" in "!!"
             */
            function isFirstBangInBangBangExpression(node) {
                return node && node.type === "UnaryExpression" && node.argument.operator === "!" &&
                    node.argument && node.argument.type === "UnaryExpression" && node.argument.operator === "!";
            }
            /**
             * Checks if an override exists for a given operator.
             * @param {string} operator Operator
             * @returns {boolean} Whether or not an override has been provided for the operator
             */
            function overrideExistsForOperator(operator) {
                return options.overrides && Object.prototype.hasOwnProperty.call(options.overrides, operator);
            }
            /**
             * Gets the value that the override was set to for this operator
             * @param {string} operator Operator
             * @returns {boolean} Whether or not an override enforces a space with this operator
             */
            function overrideEnforcesSpaces(operator) {
                return options.overrides[operator];
            }
            /**
             * Verify Unary Word Operator has spaces after the word operator
             * @param {ASTnode} node AST node
             * @param {Object} firstToken first token from the AST node
             * @param {Object} secondToken second token from the AST node
             * @param {string} word The word to be used for reporting
             * @returns {void}
             */
            function verifyWordHasSpaces(node, firstToken, secondToken, word) {
                if (secondToken.range[0] === firstToken.range[1]) {
                    context.report({
                        node,
                        messageId: "wordOperator",
                        data: {
                            word
                        },
                        fix(fixer) {
                            return fixer.insertTextAfter(firstToken, " ");
                        }
                    });
                }
            }
            /**
             * Verify Unary Word Operator doesn't have spaces after the word operator
             * @param {ASTnode} node AST node
             * @param {Object} firstToken first token from the AST node
             * @param {Object} secondToken second token from the AST node
             * @param {string} word The word to be used for reporting
             * @returns {void}
             */
            function verifyWordDoesntHaveSpaces(node, firstToken, secondToken, word) {
                if (astUtils.canTokensBeAdjacent(firstToken, secondToken)) {
                    if (secondToken.range[0] > firstToken.range[1]) {
                        context.report({
                            node,
                            messageId: "unexpectedAfterWord",
                            data: {
                                word
                            },
                            fix(fixer) {
                                return fixer.removeRange([firstToken.range[1], secondToken.range[0]]);
                            }
                        });
                    }
                }
            }
            /**
             * Check Unary Word Operators for spaces after the word operator
             * @param {ASTnode} node AST node
             * @param {Object} firstToken first token from the AST node
             * @param {Object} secondToken second token from the AST node
             * @param {string} word The word to be used for reporting
             * @returns {void}
             */
            function checkUnaryWordOperatorForSpaces(node, firstToken, secondToken, word) {
                if (overrideExistsForOperator(word)) {
                    if (overrideEnforcesSpaces(word)) {
                        verifyWordHasSpaces(node, firstToken, secondToken, word);
                    }
                    else {
                        verifyWordDoesntHaveSpaces(node, firstToken, secondToken, word);
                    }
                }
                else if (options.words) {
                    verifyWordHasSpaces(node, firstToken, secondToken, word);
                }
                else {
                    verifyWordDoesntHaveSpaces(node, firstToken, secondToken, word);
                }
            }
            /**
             * Verifies YieldExpressions satisfy spacing requirements
             * @param {ASTnode} node AST node
             * @returns {void}
             */
            function checkForSpacesAfterYield(node) {
                const tokens = sourceCode.getFirstTokens(node, 3), word = "yield";
                if (!node.argument || node.delegate) {
                    return;
                }
                checkUnaryWordOperatorForSpaces(node, tokens[0], tokens[1], word);
            }
            /**
             * Verifies AwaitExpressions satisfy spacing requirements
             * @param {ASTNode} node AwaitExpression AST node
             * @returns {void}
             */
            function checkForSpacesAfterAwait(node) {
                const tokens = sourceCode.getFirstTokens(node, 3);
                checkUnaryWordOperatorForSpaces(node, tokens[0], tokens[1], "await");
            }
            /**
             * Verifies UnaryExpression, UpdateExpression and NewExpression have spaces before or after the operator
             * @param {ASTnode} node AST node
             * @param {Object} firstToken First token in the expression
             * @param {Object} secondToken Second token in the expression
             * @returns {void}
             */
            function verifyNonWordsHaveSpaces(node, firstToken, secondToken) {
                if (node.prefix) {
                    if (isFirstBangInBangBangExpression(node)) {
                        return;
                    }
                    if (firstToken.range[1] === secondToken.range[0]) {
                        context.report({
                            node,
                            messageId: "operator",
                            data: {
                                operator: firstToken.value
                            },
                            fix(fixer) {
                                return fixer.insertTextAfter(firstToken, " ");
                            }
                        });
                    }
                }
                else {
                    if (firstToken.range[1] === secondToken.range[0]) {
                        context.report({
                            node,
                            messageId: "beforeUnaryExpressions",
                            data: {
                                token: secondToken.value
                            },
                            fix(fixer) {
                                return fixer.insertTextBefore(secondToken, " ");
                            }
                        });
                    }
                }
            }
            /**
             * Verifies UnaryExpression, UpdateExpression and NewExpression don't have spaces before or after the operator
             * @param {ASTnode} node AST node
             * @param {Object} firstToken First token in the expression
             * @param {Object} secondToken Second token in the expression
             * @returns {void}
             */
            function verifyNonWordsDontHaveSpaces(node, firstToken, secondToken) {
                if (node.prefix) {
                    if (secondToken.range[0] > firstToken.range[1]) {
                        context.report({
                            node,
                            messageId: "unexpectedAfter",
                            data: {
                                operator: firstToken.value
                            },
                            fix(fixer) {
                                if (astUtils.canTokensBeAdjacent(firstToken, secondToken)) {
                                    return fixer.removeRange([firstToken.range[1], secondToken.range[0]]);
                                }
                                return null;
                            }
                        });
                    }
                }
                else {
                    if (secondToken.range[0] > firstToken.range[1]) {
                        context.report({
                            node,
                            messageId: "unexpectedBefore",
                            data: {
                                operator: secondToken.value
                            },
                            fix(fixer) {
                                return fixer.removeRange([firstToken.range[1], secondToken.range[0]]);
                            }
                        });
                    }
                }
            }
            /**
             * Verifies UnaryExpression, UpdateExpression and NewExpression satisfy spacing requirements
             * @param {ASTnode} node AST node
             * @returns {void}
             */
            function checkForSpaces(node) {
                const tokens = node.type === "UpdateExpression" && !node.prefix
                    ? sourceCode.getLastTokens(node, 2)
                    : sourceCode.getFirstTokens(node, 2);
                const firstToken = tokens[0];
                const secondToken = tokens[1];
                if ((node.type === "NewExpression" || node.prefix) && firstToken.type === "Keyword") {
                    checkUnaryWordOperatorForSpaces(node, firstToken, secondToken, firstToken.value);
                    return;
                }
                const operator = node.prefix ? tokens[0].value : tokens[1].value;
                if (overrideExistsForOperator(operator)) {
                    if (overrideEnforcesSpaces(operator)) {
                        verifyNonWordsHaveSpaces(node, firstToken, secondToken);
                    }
                    else {
                        verifyNonWordsDontHaveSpaces(node, firstToken, secondToken);
                    }
                }
                else if (options.nonwords) {
                    verifyNonWordsHaveSpaces(node, firstToken, secondToken);
                }
                else {
                    verifyNonWordsDontHaveSpaces(node, firstToken, secondToken);
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                UnaryExpression: checkForSpaces,
                UpdateExpression: checkForSpaces,
                NewExpression: checkForSpaces,
                YieldExpression: checkForSpacesAfterYield,
                AwaitExpression: checkForSpacesAfterAwait
            };
        }
    };
},
/* eslint/lib/rules/spaced-comment.js */ function _(require, module, exports) {
    const lodash = require(111) /* lodash */;
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Escapes the control characters of a given string.
     * @param {string} s A string to escape.
     * @returns {string} An escaped string.
     */
    function escape(s) {
        return `(?:${lodash.escapeRegExp(s)})`;
    }
    /**
     * Escapes the control characters of a given string.
     * And adds a repeat flag.
     * @param {string} s A string to escape.
     * @returns {string} An escaped string.
     */
    function escapeAndRepeat(s) {
        return `${escape(s)}+`;
    }
    /**
     * Parses `markers` option.
     * If markers don't include `"*"`, this adds `"*"` to allow JSDoc comments.
     * @param {string[]} [markers] A marker list.
     * @returns {string[]} A marker list.
     */
    function parseMarkersOption(markers) {
        // `*` is a marker for JSDoc comments.
        if (markers.indexOf("*") === -1) {
            return markers.concat("*");
        }
        return markers;
    }
    /**
     * Creates string pattern for exceptions.
     * Generated pattern:
     *
     * 1. A space or an exception pattern sequence.
     * @param {string[]} exceptions An exception pattern list.
     * @returns {string} A regular expression string for exceptions.
     */
    function createExceptionsPattern(exceptions) {
        let pattern = "";
        /*
         * A space or an exception pattern sequence.
         * []                 ==> "\s"
         * ["-"]              ==> "(?:\s|\-+$)"
         * ["-", "="]         ==> "(?:\s|(?:\-+|=+)$)"
         * ["-", "=", "--=="] ==> "(?:\s|(?:\-+|=+|(?:\-\-==)+)$)" ==> https://jex.im/regulex/#!embed=false&flags=&re=(%3F%3A%5Cs%7C(%3F%3A%5C-%2B%7C%3D%2B%7C(%3F%3A%5C-%5C-%3D%3D)%2B)%24)
         */
        if (exceptions.length === 0) {
            // a space.
            pattern += "\\s";
        }
        else {
            // a space or...
            pattern += "(?:\\s|";
            if (exceptions.length === 1) {
                // a sequence of the exception pattern.
                pattern += escapeAndRepeat(exceptions[0]);
            }
            else {
                // a sequence of one of the exception patterns.
                pattern += "(?:";
                pattern += exceptions.map(escapeAndRepeat).join("|");
                pattern += ")";
            }
            pattern += `(?:$|[${Array.from(astUtils.LINEBREAKS).join("")}]))`;
        }
        return pattern;
    }
    /**
     * Creates RegExp object for `always` mode.
     * Generated pattern for beginning of comment:
     *
     * 1. First, a marker or nothing.
     * 2. Next, a space or an exception pattern sequence.
     * @param {string[]} markers A marker list.
     * @param {string[]} exceptions An exception pattern list.
     * @returns {RegExp} A RegExp object for the beginning of a comment in `always` mode.
     */
    function createAlwaysStylePattern(markers, exceptions) {
        let pattern = "^";
        /*
         * A marker or nothing.
         * ["*"]            ==> "\*?"
         * ["*", "!"]       ==> "(?:\*|!)?"
         * ["*", "/", "!<"] ==> "(?:\*|\/|(?:!<))?" ==> https://jex.im/regulex/#!embed=false&flags=&re=(%3F%3A%5C*%7C%5C%2F%7C(%3F%3A!%3C))%3F
         */
        if (markers.length === 1) {
            // the marker.
            pattern += escape(markers[0]);
        }
        else {
            // one of markers.
            pattern += "(?:";
            pattern += markers.map(escape).join("|");
            pattern += ")";
        }
        pattern += "?"; // or nothing.
        pattern += createExceptionsPattern(exceptions);
        return new RegExp(pattern, "u");
    }
    /**
     * Creates RegExp object for `never` mode.
     * Generated pattern for beginning of comment:
     *
     * 1. First, a marker or nothing (captured).
     * 2. Next, a space or a tab.
     * @param {string[]} markers A marker list.
     * @returns {RegExp} A RegExp object for `never` mode.
     */
    function createNeverStylePattern(markers) {
        const pattern = `^(${markers.map(escape).join("|")})?[ \t]+`;
        return new RegExp(pattern, "u");
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce consistent spacing after the `//` or `/*` in a comment",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/spaced-comment"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["always", "never"]
                },
                {
                    type: "object",
                    properties: {
                        exceptions: {
                            type: "array",
                            items: {
                                type: "string"
                            }
                        },
                        markers: {
                            type: "array",
                            items: {
                                type: "string"
                            }
                        },
                        line: {
                            type: "object",
                            properties: {
                                exceptions: {
                                    type: "array",
                                    items: {
                                        type: "string"
                                    }
                                },
                                markers: {
                                    type: "array",
                                    items: {
                                        type: "string"
                                    }
                                }
                            },
                            additionalProperties: false
                        },
                        block: {
                            type: "object",
                            properties: {
                                exceptions: {
                                    type: "array",
                                    items: {
                                        type: "string"
                                    }
                                },
                                markers: {
                                    type: "array",
                                    items: {
                                        type: "string"
                                    }
                                },
                                balanced: {
                                    type: "boolean",
                                    default: false
                                }
                            },
                            additionalProperties: false
                        }
                    },
                    additionalProperties: false
                }
            ]
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            // Unless the first option is never, require a space
            const requireSpace = context.options[0] !== "never";
            /*
             * Parse the second options.
             * If markers don't include `"*"`, it's added automatically for JSDoc
             * comments.
             */
            const config = context.options[1] || {};
            const balanced = config.block && config.block.balanced;
            const styleRules = ["block", "line"].reduce((rule, type) => {
                const markers = parseMarkersOption(config[type] && config[type].markers || config.markers || []);
                const exceptions = config[type] && config[type].exceptions || config.exceptions || [];
                const endNeverPattern = "[ \t]+$";
                // Create RegExp object for valid patterns.
                rule[type] = {
                    beginRegex: requireSpace ? createAlwaysStylePattern(markers, exceptions) : createNeverStylePattern(markers),
                    endRegex: balanced && requireSpace ? new RegExp(`${createExceptionsPattern(exceptions)}$`, "u") : new RegExp(endNeverPattern, "u"),
                    hasExceptions: exceptions.length > 0,
                    captureMarker: new RegExp(`^(${markers.map(escape).join("|")})`, "u"),
                    markers: new Set(markers)
                };
                return rule;
            }, {});
            /**
             * Reports a beginning spacing error with an appropriate message.
             * @param {ASTNode} node A comment node to check.
             * @param {string} message An error message to report.
             * @param {Array} match An array of match results for markers.
             * @param {string} refChar Character used for reference in the error message.
             * @returns {void}
             */
            function reportBegin(node, message, match, refChar) {
                const type = node.type.toLowerCase(), commentIdentifier = type === "block" ? "/*" : "//";
                context.report({
                    node,
                    fix(fixer) {
                        const start = node.range[0];
                        let end = start + 2;
                        if (requireSpace) {
                            if (match) {
                                end += match[0].length;
                            }
                            return fixer.insertTextAfterRange([start, end], " ");
                        }
                        end += match[0].length;
                        return fixer.replaceTextRange([start, end], commentIdentifier + (match[1] ? match[1] : ""));
                    },
                    message,
                    data: { refChar }
                });
            }
            /**
             * Reports an ending spacing error with an appropriate message.
             * @param {ASTNode} node A comment node to check.
             * @param {string} message An error message to report.
             * @param {string} match An array of the matched whitespace characters.
             * @returns {void}
             */
            function reportEnd(node, message, match) {
                context.report({
                    node,
                    fix(fixer) {
                        if (requireSpace) {
                            return fixer.insertTextAfterRange([node.range[0], node.range[1] - 2], " ");
                        }
                        const end = node.range[1] - 2, start = end - match[0].length;
                        return fixer.replaceTextRange([start, end], "");
                    },
                    message
                });
            }
            /**
             * Reports a given comment if it's invalid.
             * @param {ASTNode} node a comment node to check.
             * @returns {void}
             */
            function checkCommentForSpace(node) {
                const type = node.type.toLowerCase(), rule = styleRules[type], commentIdentifier = type === "block" ? "/*" : "//";
                // Ignores empty comments and comments that consist only of a marker.
                if (node.value.length === 0 || rule.markers.has(node.value)) {
                    return;
                }
                const beginMatch = rule.beginRegex.exec(node.value);
                const endMatch = rule.endRegex.exec(node.value);
                // Checks.
                if (requireSpace) {
                    if (!beginMatch) {
                        const hasMarker = rule.captureMarker.exec(node.value);
                        const marker = hasMarker ? commentIdentifier + hasMarker[0] : commentIdentifier;
                        if (rule.hasExceptions) {
                            reportBegin(node, "Expected exception block, space or tab after '{{refChar}}' in comment.", hasMarker, marker);
                        }
                        else {
                            reportBegin(node, "Expected space or tab after '{{refChar}}' in comment.", hasMarker, marker);
                        }
                    }
                    if (balanced && type === "block" && !endMatch) {
                        reportEnd(node, "Expected space or tab before '*/' in comment.");
                    }
                }
                else {
                    if (beginMatch) {
                        if (!beginMatch[1]) {
                            reportBegin(node, "Unexpected space or tab after '{{refChar}}' in comment.", beginMatch, commentIdentifier);
                        }
                        else {
                            reportBegin(node, "Unexpected space or tab after marker ({{refChar}}) in comment.", beginMatch, beginMatch[1]);
                        }
                    }
                    if (balanced && type === "block" && endMatch) {
                        reportEnd(node, "Unexpected space or tab before '*/' in comment.", endMatch);
                    }
                }
            }
            return {
                Program() {
                    const comments = sourceCode.getAllComments();
                    comments.filter(token => token.type !== "Shebang").forEach(checkCommentForSpace);
                }
            };
        }
    };
},
/* eslint/lib/rules/strict.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Gets all of the Use Strict Directives in the Directive Prologue of a group of
     * statements.
     * @param {ASTNode[]} statements Statements in the program or function body.
     * @returns {ASTNode[]} All of the Use Strict Directives.
     */
    function getUseStrictDirectives(statements) {
        const directives = [];
        for (let i = 0; i < statements.length; i++) {
            const statement = statements[i];
            if (statement.type === "ExpressionStatement" &&
                statement.expression.type === "Literal" &&
                statement.expression.value === "use strict") {
                directives[i] = statement;
            }
            else {
                break;
            }
        }
        return directives;
    }
    /**
     * Checks whether a given parameter is a simple parameter.
     * @param {ASTNode} node A pattern node to check.
     * @returns {boolean} `true` if the node is an Identifier node.
     */
    function isSimpleParameter(node) {
        return node.type === "Identifier";
    }
    /**
     * Checks whether a given parameter list is a simple parameter list.
     * @param {ASTNode[]} params A parameter list to check.
     * @returns {boolean} `true` if the every parameter is an Identifier node.
     */
    function isSimpleParameterList(params) {
        return params.every(isSimpleParameter);
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require or disallow strict mode directives",
                category: "Strict Mode",
                recommended: false,
                url: "https://eslint.org/docs/rules/strict"
            },
            schema: [
                {
                    enum: ["never", "global", "function", "safe"]
                }
            ],
            fixable: "code",
            messages: {
                function: "Use the function form of 'use strict'.",
                global: "Use the global form of 'use strict'.",
                multiple: "Multiple 'use strict' directives.",
                never: "Strict mode is not permitted.",
                unnecessary: "Unnecessary 'use strict' directive.",
                module: "'use strict' is unnecessary inside of modules.",
                implied: "'use strict' is unnecessary when implied strict mode is enabled.",
                unnecessaryInClasses: "'use strict' is unnecessary inside of classes.",
                nonSimpleParameterList: "'use strict' directive inside a function with non-simple parameter list throws a syntax error since ES2016.",
                wrap: "Wrap {{name}} in a function with 'use strict' directive."
            }
        },
        create(context) {
            const ecmaFeatures = context.parserOptions.ecmaFeatures || {}, scopes = [], classScopes = [];
            let mode = context.options[0] || "safe";
            if (ecmaFeatures.impliedStrict) {
                mode = "implied";
            }
            else if (mode === "safe") {
                mode = ecmaFeatures.globalReturn ? "global" : "function";
            }
            /**
             * Determines whether a reported error should be fixed, depending on the error type.
             * @param {string} errorType The type of error
             * @returns {boolean} `true` if the reported error should be fixed
             */
            function shouldFix(errorType) {
                return errorType === "multiple" || errorType === "unnecessary" || errorType === "module" || errorType === "implied" || errorType === "unnecessaryInClasses";
            }
            /**
             * Gets a fixer function to remove a given 'use strict' directive.
             * @param {ASTNode} node The directive that should be removed
             * @returns {Function} A fixer function
             */
            function getFixFunction(node) {
                return fixer => fixer.remove(node);
            }
            /**
             * Report a slice of an array of nodes with a given message.
             * @param {ASTNode[]} nodes Nodes.
             * @param {string} start Index to start from.
             * @param {string} end Index to end before.
             * @param {string} messageId Message to display.
             * @param {boolean} fix `true` if the directive should be fixed (i.e. removed)
             * @returns {void}
             */
            function reportSlice(nodes, start, end, messageId, fix) {
                nodes.slice(start, end).forEach(node => {
                    context.report({ node, messageId, fix: fix ? getFixFunction(node) : null });
                });
            }
            /**
             * Report all nodes in an array with a given message.
             * @param {ASTNode[]} nodes Nodes.
             * @param {string} messageId Message id to display.
             * @param {boolean} fix `true` if the directive should be fixed (i.e. removed)
             * @returns {void}
             */
            function reportAll(nodes, messageId, fix) {
                reportSlice(nodes, 0, nodes.length, messageId, fix);
            }
            /**
             * Report all nodes in an array, except the first, with a given message.
             * @param {ASTNode[]} nodes Nodes.
             * @param {string} messageId Message id to display.
             * @param {boolean} fix `true` if the directive should be fixed (i.e. removed)
             * @returns {void}
             */
            function reportAllExceptFirst(nodes, messageId, fix) {
                reportSlice(nodes, 1, nodes.length, messageId, fix);
            }
            /**
             * Entering a function in 'function' mode pushes a new nested scope onto the
             * stack. The new scope is true if the nested function is strict mode code.
             * @param {ASTNode} node The function declaration or expression.
             * @param {ASTNode[]} useStrictDirectives The Use Strict Directives of the node.
             * @returns {void}
             */
            function enterFunctionInFunctionMode(node, useStrictDirectives) {
                const isInClass = classScopes.length > 0, isParentGlobal = scopes.length === 0 && classScopes.length === 0, isParentStrict = scopes.length > 0 && scopes[scopes.length - 1], isStrict = useStrictDirectives.length > 0;
                if (isStrict) {
                    if (!isSimpleParameterList(node.params)) {
                        context.report({ node: useStrictDirectives[0], messageId: "nonSimpleParameterList" });
                    }
                    else if (isParentStrict) {
                        context.report({ node: useStrictDirectives[0], messageId: "unnecessary", fix: getFixFunction(useStrictDirectives[0]) });
                    }
                    else if (isInClass) {
                        context.report({ node: useStrictDirectives[0], messageId: "unnecessaryInClasses", fix: getFixFunction(useStrictDirectives[0]) });
                    }
                    reportAllExceptFirst(useStrictDirectives, "multiple", true);
                }
                else if (isParentGlobal) {
                    if (isSimpleParameterList(node.params)) {
                        context.report({ node, messageId: "function" });
                    }
                    else {
                        context.report({
                            node,
                            messageId: "wrap",
                            data: { name: astUtils.getFunctionNameWithKind(node) }
                        });
                    }
                }
                scopes.push(isParentStrict || isStrict);
            }
            /**
             * Exiting a function in 'function' mode pops its scope off the stack.
             * @returns {void}
             */
            function exitFunctionInFunctionMode() {
                scopes.pop();
            }
            /**
             * Enter a function and either:
             * - Push a new nested scope onto the stack (in 'function' mode).
             * - Report all the Use Strict Directives (in the other modes).
             * @param {ASTNode} node The function declaration or expression.
             * @returns {void}
             */
            function enterFunction(node) {
                const isBlock = node.body.type === "BlockStatement", useStrictDirectives = isBlock
                    ? getUseStrictDirectives(node.body.body) : [];
                if (mode === "function") {
                    enterFunctionInFunctionMode(node, useStrictDirectives);
                }
                else if (useStrictDirectives.length > 0) {
                    if (isSimpleParameterList(node.params)) {
                        reportAll(useStrictDirectives, mode, shouldFix(mode));
                    }
                    else {
                        context.report({ node: useStrictDirectives[0], messageId: "nonSimpleParameterList" });
                        reportAllExceptFirst(useStrictDirectives, "multiple", true);
                    }
                }
            }
            const rule = {
                Program(node) {
                    const useStrictDirectives = getUseStrictDirectives(node.body);
                    if (node.sourceType === "module") {
                        mode = "module";
                    }
                    if (mode === "global") {
                        if (node.body.length > 0 && useStrictDirectives.length === 0) {
                            context.report({ node, messageId: "global" });
                        }
                        reportAllExceptFirst(useStrictDirectives, "multiple", true);
                    }
                    else {
                        reportAll(useStrictDirectives, mode, shouldFix(mode));
                    }
                },
                FunctionDeclaration: enterFunction,
                FunctionExpression: enterFunction,
                ArrowFunctionExpression: enterFunction
            };
            if (mode === "function") {
                Object.assign(rule, {
                    // Inside of class bodies are always strict mode.
                    ClassBody() {
                        classScopes.push(true);
                    },
                    "ClassBody:exit"() {
                        classScopes.pop();
                    },
                    "FunctionDeclaration:exit": exitFunctionInFunctionMode,
                    "FunctionExpression:exit": exitFunctionInFunctionMode,
                    "ArrowFunctionExpression:exit": exitFunctionInFunctionMode
                });
            }
            return rule;
        }
    };
},
/* eslint/lib/rules/switch-colon-spacing.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "enforce spacing around colons of switch statements",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/switch-colon-spacing"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        before: { type: "boolean", default: false },
                        after: { type: "boolean", default: true }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "whitespace",
            messages: {
                expectedBefore: "Expected space(s) before this colon.",
                expectedAfter: "Expected space(s) after this colon.",
                unexpectedBefore: "Unexpected space(s) before this colon.",
                unexpectedAfter: "Unexpected space(s) after this colon."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const options = context.options[0] || {};
            const beforeSpacing = options.before === true; // false by default
            const afterSpacing = options.after !== false; // true by default
            /**
             * Get the colon token of the given SwitchCase node.
             * @param {ASTNode} node The SwitchCase node to get.
             * @returns {Token} The colon token of the node.
             */
            function getColonToken(node) {
                if (node.test) {
                    return sourceCode.getTokenAfter(node.test, astUtils.isColonToken);
                }
                return sourceCode.getFirstToken(node, 1);
            }
            /**
             * Check whether the spacing between the given 2 tokens is valid or not.
             * @param {Token} left The left token to check.
             * @param {Token} right The right token to check.
             * @param {boolean} expected The expected spacing to check. `true` if there should be a space.
             * @returns {boolean} `true` if the spacing between the tokens is valid.
             */
            function isValidSpacing(left, right, expected) {
                return (astUtils.isClosingBraceToken(right) ||
                    !astUtils.isTokenOnSameLine(left, right) ||
                    sourceCode.isSpaceBetweenTokens(left, right) === expected);
            }
            /**
             * Check whether comments exist between the given 2 tokens.
             * @param {Token} left The left token to check.
             * @param {Token} right The right token to check.
             * @returns {boolean} `true` if comments exist between the given 2 tokens.
             */
            function commentsExistBetween(left, right) {
                return sourceCode.getFirstTokenBetween(left, right, {
                    includeComments: true,
                    filter: astUtils.isCommentToken
                }) !== null;
            }
            /**
             * Fix the spacing between the given 2 tokens.
             * @param {RuleFixer} fixer The fixer to fix.
             * @param {Token} left The left token of fix range.
             * @param {Token} right The right token of fix range.
             * @param {boolean} spacing The spacing style. `true` if there should be a space.
             * @returns {Fix|null} The fix object.
             */
            function fix(fixer, left, right, spacing) {
                if (commentsExistBetween(left, right)) {
                    return null;
                }
                if (spacing) {
                    return fixer.insertTextAfter(left, " ");
                }
                return fixer.removeRange([left.range[1], right.range[0]]);
            }
            return {
                SwitchCase(node) {
                    const colonToken = getColonToken(node);
                    const beforeToken = sourceCode.getTokenBefore(colonToken);
                    const afterToken = sourceCode.getTokenAfter(colonToken);
                    if (!isValidSpacing(beforeToken, colonToken, beforeSpacing)) {
                        context.report({
                            node,
                            loc: colonToken.loc,
                            messageId: beforeSpacing ? "expectedBefore" : "unexpectedBefore",
                            fix: fixer => fix(fixer, beforeToken, colonToken, beforeSpacing)
                        });
                    }
                    if (!isValidSpacing(colonToken, afterToken, afterSpacing)) {
                        context.report({
                            node,
                            loc: colonToken.loc,
                            messageId: afterSpacing ? "expectedAfter" : "unexpectedAfter",
                            fix: fixer => fix(fixer, colonToken, afterToken, afterSpacing)
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/symbol-description.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require symbol descriptions",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/symbol-description"
            },
            fixable: null,
            schema: [],
            messages: {
                expected: "Expected Symbol to have a description."
            }
        },
        create(context) {
            /**
             * Reports if node does not conform the rule in case rule is set to
             * report missing description
             * @param {ASTNode} node A CallExpression node to check.
             * @returns {void}
             */
            function checkArgument(node) {
                if (node.arguments.length === 0) {
                    context.report({
                        node,
                        messageId: "expected"
                    });
                }
            }
            return {
                "Program:exit"() {
                    const scope = context.getScope();
                    const variable = astUtils.getVariableByName(scope, "Symbol");
                    if (variable && variable.defs.length === 0) {
                        variable.references.forEach(reference => {
                            const node = reference.identifier;
                            if (astUtils.isCallee(node)) {
                                checkArgument(node.parent);
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/template-curly-spacing.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const OPEN_PAREN = /\$\{$/u;
    const CLOSE_PAREN = /^\}/u;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow spacing around embedded expressions of template strings",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/template-curly-spacing"
            },
            fixable: "whitespace",
            schema: [
                { enum: ["always", "never"] }
            ],
            messages: {
                expectedBefore: "Expected space(s) before '}'.",
                expectedAfter: "Expected space(s) after '${'.",
                unexpectedBefore: "Unexpected space(s) before '}'.",
                unexpectedAfter: "Unexpected space(s) after '${'."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const always = context.options[0] === "always";
            const prefix = always ? "expected" : "unexpected";
            /**
             * Checks spacing before `}` of a given token.
             * @param {Token} token A token to check. This is a Template token.
             * @returns {void}
             */
            function checkSpacingBefore(token) {
                const prevToken = sourceCode.getTokenBefore(token);
                if (prevToken &&
                    CLOSE_PAREN.test(token.value) &&
                    astUtils.isTokenOnSameLine(prevToken, token) &&
                    sourceCode.isSpaceBetweenTokens(prevToken, token) !== always) {
                    context.report({
                        loc: token.loc.start,
                        messageId: `${prefix}Before`,
                        fix(fixer) {
                            if (always) {
                                return fixer.insertTextBefore(token, " ");
                            }
                            return fixer.removeRange([
                                prevToken.range[1],
                                token.range[0]
                            ]);
                        }
                    });
                }
            }
            /**
             * Checks spacing after `${` of a given token.
             * @param {Token} token A token to check. This is a Template token.
             * @returns {void}
             */
            function checkSpacingAfter(token) {
                const nextToken = sourceCode.getTokenAfter(token);
                if (nextToken &&
                    OPEN_PAREN.test(token.value) &&
                    astUtils.isTokenOnSameLine(token, nextToken) &&
                    sourceCode.isSpaceBetweenTokens(token, nextToken) !== always) {
                    context.report({
                        loc: {
                            line: token.loc.end.line,
                            column: token.loc.end.column - 2
                        },
                        messageId: `${prefix}After`,
                        fix(fixer) {
                            if (always) {
                                return fixer.insertTextAfter(token, " ");
                            }
                            return fixer.removeRange([
                                token.range[1],
                                nextToken.range[0]
                            ]);
                        }
                    });
                }
            }
            return {
                TemplateElement(node) {
                    const token = sourceCode.getFirstToken(node);
                    checkSpacingBefore(token);
                    checkSpacingAfter(token);
                }
            };
        }
    };
},
/* eslint/lib/rules/template-tag-spacing.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow spacing between template tags and their literals",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/template-tag-spacing"
            },
            fixable: "whitespace",
            schema: [
                { enum: ["always", "never"] }
            ],
            messages: {
                unexpected: "Unexpected space between template tag and template literal.",
                missing: "Missing space between template tag and template literal."
            }
        },
        create(context) {
            const never = context.options[0] !== "always";
            const sourceCode = context.getSourceCode();
            /**
             * Check if a space is present between a template tag and its literal
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkSpacing(node) {
                const tagToken = sourceCode.getTokenBefore(node.quasi);
                const literalToken = sourceCode.getFirstToken(node.quasi);
                const hasWhitespace = sourceCode.isSpaceBetweenTokens(tagToken, literalToken);
                if (never && hasWhitespace) {
                    context.report({
                        node,
                        loc: tagToken.loc.start,
                        messageId: "unexpected",
                        fix(fixer) {
                            const comments = sourceCode.getCommentsBefore(node.quasi);
                            // Don't fix anything if there's a single line comment after the template tag
                            if (comments.some(comment => comment.type === "Line")) {
                                return null;
                            }
                            return fixer.replaceTextRange([tagToken.range[1], literalToken.range[0]], comments.reduce((text, comment) => text + sourceCode.getText(comment), ""));
                        }
                    });
                }
                else if (!never && !hasWhitespace) {
                    context.report({
                        node,
                        loc: tagToken.loc.start,
                        messageId: "missing",
                        fix(fixer) {
                            return fixer.insertTextAfter(tagToken, " ");
                        }
                    });
                }
            }
            return {
                TaggedTemplateExpression: checkSpacing
            };
        }
    };
},
/* eslint/lib/rules/unicode-bom.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow Unicode byte order mark (BOM)",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/unicode-bom"
            },
            fixable: "whitespace",
            schema: [
                {
                    enum: ["always", "never"]
                }
            ],
            messages: {
                expected: "Expected Unicode BOM (Byte Order Mark).",
                unexpected: "Unexpected Unicode BOM (Byte Order Mark)."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                Program: function checkUnicodeBOM(node) {
                    const sourceCode = context.getSourceCode(), location = { column: 0, line: 1 }, requireBOM = context.options[0] || "never";
                    if (!sourceCode.hasBOM && (requireBOM === "always")) {
                        context.report({
                            node,
                            loc: location,
                            messageId: "expected",
                            fix(fixer) {
                                return fixer.insertTextBeforeRange([0, 1], "\uFEFF");
                            }
                        });
                    }
                    else if (sourceCode.hasBOM && (requireBOM === "never")) {
                        context.report({
                            node,
                            loc: location,
                            messageId: "unexpected",
                            fix(fixer) {
                                return fixer.removeRange([-1, 0]);
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/use-isnan.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Determines if the given node is a NaN `Identifier` node.
     * @param {ASTNode|null} node The node to check.
     * @returns {boolean} `true` if the node is 'NaN' identifier.
     */
    function isNaNIdentifier(node) {
        return Boolean(node) && node.type === "Identifier" && node.name === "NaN";
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "require calls to `isNaN()` when checking for `NaN`",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/use-isnan"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        enforceForSwitchCase: {
                            type: "boolean",
                            default: false
                        },
                        enforceForIndexOf: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                comparisonWithNaN: "Use the isNaN function to compare with NaN.",
                switchNaN: "'switch(NaN)' can never match a case clause. Use Number.isNaN instead of the switch.",
                caseNaN: "'case NaN' can never match. Use Number.isNaN before the switch.",
                indexOfNaN: "Array prototype method '{{ methodName }}' cannot find NaN."
            }
        },
        create(context) {
            const enforceForSwitchCase = context.options[0] && context.options[0].enforceForSwitchCase;
            const enforceForIndexOf = context.options[0] && context.options[0].enforceForIndexOf;
            /**
             * Checks the given `BinaryExpression` node for `foo === NaN` and other comparisons.
             * @param {ASTNode} node The node to check.
             * @returns {void}
             */
            function checkBinaryExpression(node) {
                if (/^(?:[<>]|[!=]=)=?$/u.test(node.operator) &&
                    (isNaNIdentifier(node.left) || isNaNIdentifier(node.right))) {
                    context.report({ node, messageId: "comparisonWithNaN" });
                }
            }
            /**
             * Checks the discriminant and all case clauses of the given `SwitchStatement` node for `switch(NaN)` and `case NaN:`
             * @param {ASTNode} node The node to check.
             * @returns {void}
             */
            function checkSwitchStatement(node) {
                if (isNaNIdentifier(node.discriminant)) {
                    context.report({ node, messageId: "switchNaN" });
                }
                for (const switchCase of node.cases) {
                    if (isNaNIdentifier(switchCase.test)) {
                        context.report({ node: switchCase, messageId: "caseNaN" });
                    }
                }
            }
            /**
             * Checks the the given `CallExpression` node for `.indexOf(NaN)` and `.lastIndexOf(NaN)`.
             * @param {ASTNode} node The node to check.
             * @returns {void}
             */
            function checkCallExpression(node) {
                const callee = node.callee;
                if (callee.type === "MemberExpression") {
                    const methodName = astUtils.getStaticPropertyName(callee);
                    if ((methodName === "indexOf" || methodName === "lastIndexOf") &&
                        node.arguments.length === 1 &&
                        isNaNIdentifier(node.arguments[0])) {
                        context.report({ node, messageId: "indexOfNaN", data: { methodName } });
                    }
                }
            }
            const listeners = {
                BinaryExpression: checkBinaryExpression
            };
            if (enforceForSwitchCase) {
                listeners.SwitchStatement = checkSwitchStatement;
            }
            if (enforceForIndexOf) {
                listeners.CallExpression = checkCallExpression;
            }
            return listeners;
        }
    };
},
/* eslint/lib/rules/valid-jsdoc.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const doctrine = require(413) /* doctrine */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "enforce valid JSDoc comments",
                category: "Possible Errors",
                recommended: false,
                url: "https://eslint.org/docs/rules/valid-jsdoc"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        prefer: {
                            type: "object",
                            additionalProperties: {
                                type: "string"
                            }
                        },
                        preferType: {
                            type: "object",
                            additionalProperties: {
                                type: "string"
                            }
                        },
                        requireReturn: {
                            type: "boolean",
                            default: true
                        },
                        requireParamDescription: {
                            type: "boolean",
                            default: true
                        },
                        requireReturnDescription: {
                            type: "boolean",
                            default: true
                        },
                        matchDescription: {
                            type: "string"
                        },
                        requireReturnType: {
                            type: "boolean",
                            default: true
                        },
                        requireParamType: {
                            type: "boolean",
                            default: true
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "code",
            messages: {
                unexpectedTag: "Unexpected @{{title}} tag; function has no return statement.",
                expected: "Expected JSDoc for '{{name}}' but found '{{jsdocName}}'.",
                use: "Use @{{name}} instead.",
                useType: "Use '{{expectedTypeName}}' instead of '{{currentTypeName}}'.",
                syntaxError: "JSDoc syntax error.",
                missingBrace: "JSDoc type missing brace.",
                missingParamDesc: "Missing JSDoc parameter description for '{{name}}'.",
                missingParamType: "Missing JSDoc parameter type for '{{name}}'.",
                missingReturnType: "Missing JSDoc return type.",
                missingReturnDesc: "Missing JSDoc return description.",
                missingReturn: "Missing JSDoc @{{returns}} for function.",
                missingParam: "Missing JSDoc for parameter '{{name}}'.",
                duplicateParam: "Duplicate JSDoc parameter '{{name}}'.",
                unsatisfiedDesc: "JSDoc description does not satisfy the regex pattern."
            },
            deprecated: true,
            replacedBy: []
        },
        create(context) {
            const options = context.options[0] || {}, prefer = options.prefer || {}, sourceCode = context.getSourceCode(), 
            // these both default to true, so you have to explicitly make them false
            requireReturn = options.requireReturn !== false, requireParamDescription = options.requireParamDescription !== false, requireReturnDescription = options.requireReturnDescription !== false, requireReturnType = options.requireReturnType !== false, requireParamType = options.requireParamType !== false, preferType = options.preferType || {}, checkPreferType = Object.keys(preferType).length !== 0;
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            // Using a stack to store if a function returns or not (handling nested functions)
            const fns = [];
            /**
             * Check if node type is a Class
             * @param {ASTNode} node node to check.
             * @returns {boolean} True is its a class
             * @private
             */
            function isTypeClass(node) {
                return node.type === "ClassExpression" || node.type === "ClassDeclaration";
            }
            /**
             * When parsing a new function, store it in our function stack.
             * @param {ASTNode} node A function node to check.
             * @returns {void}
             * @private
             */
            function startFunction(node) {
                fns.push({
                    returnPresent: (node.type === "ArrowFunctionExpression" && node.body.type !== "BlockStatement") ||
                        isTypeClass(node) || node.async
                });
            }
            /**
             * Indicate that return has been found in the current function.
             * @param {ASTNode} node The return node.
             * @returns {void}
             * @private
             */
            function addReturn(node) {
                const functionState = fns[fns.length - 1];
                if (functionState && node.argument !== null) {
                    functionState.returnPresent = true;
                }
            }
            /**
             * Check if return tag type is void or undefined
             * @param {Object} tag JSDoc tag
             * @returns {boolean} True if its of type void or undefined
             * @private
             */
            function isValidReturnType(tag) {
                return tag.type === null || tag.type.name === "void" || tag.type.type === "UndefinedLiteral";
            }
            /**
             * Check if type should be validated based on some exceptions
             * @param {Object} type JSDoc tag
             * @returns {boolean} True if it can be validated
             * @private
             */
            function canTypeBeValidated(type) {
                return type !== "UndefinedLiteral" && // {undefined} as there is no name property available.
                    type !== "NullLiteral" && // {null}
                    type !== "NullableLiteral" && // {?}
                    type !== "FunctionType" && // {function(a)}
                    type !== "AllLiteral"; // {*}
            }
            /**
             * Extract the current and expected type based on the input type object
             * @param {Object} type JSDoc tag
             * @returns {{currentType: Doctrine.Type, expectedTypeName: string}} The current type annotation and
             * the expected name of the annotation
             * @private
             */
            function getCurrentExpectedTypes(type) {
                let currentType;
                if (type.name) {
                    currentType = type;
                }
                else if (type.expression) {
                    currentType = type.expression;
                }
                return {
                    currentType,
                    expectedTypeName: currentType && preferType[currentType.name]
                };
            }
            /**
             * Gets the location of a JSDoc node in a file
             * @param {Token} jsdocComment The comment that this node is parsed from
             * @param {{range: number[]}} parsedJsdocNode A tag or other node which was parsed from this comment
             * @returns {{start: SourceLocation, end: SourceLocation}} The 0-based source location for the tag
             */
            function getAbsoluteRange(jsdocComment, parsedJsdocNode) {
                return {
                    start: sourceCode.getLocFromIndex(jsdocComment.range[0] + 2 + parsedJsdocNode.range[0]),
                    end: sourceCode.getLocFromIndex(jsdocComment.range[0] + 2 + parsedJsdocNode.range[1])
                };
            }
            /**
             * Validate type for a given JSDoc node
             * @param {Object} jsdocNode JSDoc node
             * @param {Object} type JSDoc tag
             * @returns {void}
             * @private
             */
            function validateType(jsdocNode, type) {
                if (!type || !canTypeBeValidated(type.type)) {
                    return;
                }
                const typesToCheck = [];
                let elements = [];
                switch (type.type) {
                    case "TypeApplication": // {Array.<String>}
                        elements = type.applications[0].type === "UnionType" ? type.applications[0].elements : type.applications;
                        typesToCheck.push(getCurrentExpectedTypes(type));
                        break;
                    case "RecordType": // {{20:String}}
                        elements = type.fields;
                        break;
                    case "UnionType": // {String|number|Test}
                    case "ArrayType": // {[String, number, Test]}
                        elements = type.elements;
                        break;
                    case "FieldType": // Array.<{count: number, votes: number}>
                        if (type.value) {
                            typesToCheck.push(getCurrentExpectedTypes(type.value));
                        }
                        break;
                    default:
                        typesToCheck.push(getCurrentExpectedTypes(type));
                }
                elements.forEach(validateType.bind(null, jsdocNode));
                typesToCheck.forEach(typeToCheck => {
                    if (typeToCheck.expectedTypeName &&
                        typeToCheck.expectedTypeName !== typeToCheck.currentType.name) {
                        context.report({
                            node: jsdocNode,
                            messageId: "useType",
                            loc: getAbsoluteRange(jsdocNode, typeToCheck.currentType),
                            data: {
                                currentTypeName: typeToCheck.currentType.name,
                                expectedTypeName: typeToCheck.expectedTypeName
                            },
                            fix(fixer) {
                                return fixer.replaceTextRange(typeToCheck.currentType.range.map(indexInComment => jsdocNode.range[0] + 2 + indexInComment), typeToCheck.expectedTypeName);
                            }
                        });
                    }
                });
            }
            /**
             * Validate the JSDoc node and output warnings if anything is wrong.
             * @param {ASTNode} node The AST node to check.
             * @returns {void}
             * @private
             */
            function checkJSDoc(node) {
                const jsdocNode = sourceCode.getJSDocComment(node), functionData = fns.pop(), paramTagsByName = Object.create(null), paramTags = [];
                let hasReturns = false, returnsTag, hasConstructor = false, isInterface = false, isOverride = false, isAbstract = false;
                // make sure only to validate JSDoc comments
                if (jsdocNode) {
                    let jsdoc;
                    try {
                        jsdoc = doctrine.parse(jsdocNode.value, {
                            strict: true,
                            unwrap: true,
                            sloppy: true,
                            range: true
                        });
                    }
                    catch (ex) {
                        if (/braces/iu.test(ex.message)) {
                            context.report({ node: jsdocNode, messageId: "missingBrace" });
                        }
                        else {
                            context.report({ node: jsdocNode, messageId: "syntaxError" });
                        }
                        return;
                    }
                    jsdoc.tags.forEach(tag => {
                        switch (tag.title.toLowerCase()) {
                            case "param":
                            case "arg":
                            case "argument":
                                paramTags.push(tag);
                                break;
                            case "return":
                            case "returns":
                                hasReturns = true;
                                returnsTag = tag;
                                break;
                            case "constructor":
                            case "class":
                                hasConstructor = true;
                                break;
                            case "override":
                            case "inheritdoc":
                                isOverride = true;
                                break;
                            case "abstract":
                            case "virtual":
                                isAbstract = true;
                                break;
                            case "interface":
                                isInterface = true;
                                break;
                            // no default
                        }
                        // check tag preferences
                        if (Object.prototype.hasOwnProperty.call(prefer, tag.title) && tag.title !== prefer[tag.title]) {
                            const entireTagRange = getAbsoluteRange(jsdocNode, tag);
                            context.report({
                                node: jsdocNode,
                                messageId: "use",
                                loc: {
                                    start: entireTagRange.start,
                                    end: {
                                        line: entireTagRange.start.line,
                                        column: entireTagRange.start.column + `@${tag.title}`.length
                                    }
                                },
                                data: { name: prefer[tag.title] },
                                fix(fixer) {
                                    return fixer.replaceTextRange([
                                        jsdocNode.range[0] + tag.range[0] + 3,
                                        jsdocNode.range[0] + tag.range[0] + tag.title.length + 3
                                    ], prefer[tag.title]);
                                }
                            });
                        }
                        // validate the types
                        if (checkPreferType && tag.type) {
                            validateType(jsdocNode, tag.type);
                        }
                    });
                    paramTags.forEach(param => {
                        if (requireParamType && !param.type) {
                            context.report({
                                node: jsdocNode,
                                messageId: "missingParamType",
                                loc: getAbsoluteRange(jsdocNode, param),
                                data: { name: param.name }
                            });
                        }
                        if (!param.description && requireParamDescription) {
                            context.report({
                                node: jsdocNode,
                                messageId: "missingParamDesc",
                                loc: getAbsoluteRange(jsdocNode, param),
                                data: { name: param.name }
                            });
                        }
                        if (paramTagsByName[param.name]) {
                            context.report({
                                node: jsdocNode,
                                messageId: "duplicateParam",
                                loc: getAbsoluteRange(jsdocNode, param),
                                data: { name: param.name }
                            });
                        }
                        else if (param.name.indexOf(".") === -1) {
                            paramTagsByName[param.name] = param;
                        }
                    });
                    if (hasReturns) {
                        if (!requireReturn && !functionData.returnPresent && (returnsTag.type === null || !isValidReturnType(returnsTag)) && !isAbstract) {
                            context.report({
                                node: jsdocNode,
                                messageId: "unexpectedTag",
                                loc: getAbsoluteRange(jsdocNode, returnsTag),
                                data: {
                                    title: returnsTag.title
                                }
                            });
                        }
                        else {
                            if (requireReturnType && !returnsTag.type) {
                                context.report({ node: jsdocNode, messageId: "missingReturnType" });
                            }
                            if (!isValidReturnType(returnsTag) && !returnsTag.description && requireReturnDescription) {
                                context.report({ node: jsdocNode, messageId: "missingReturnDesc" });
                            }
                        }
                    }
                    // check for functions missing @returns
                    if (!isOverride && !hasReturns && !hasConstructor && !isInterface &&
                        node.parent.kind !== "get" && node.parent.kind !== "constructor" &&
                        node.parent.kind !== "set" && !isTypeClass(node)) {
                        if (requireReturn || (functionData.returnPresent && !node.async)) {
                            context.report({
                                node: jsdocNode,
                                messageId: "missingReturn",
                                data: {
                                    returns: prefer.returns || "returns"
                                }
                            });
                        }
                    }
                    // check the parameters
                    const jsdocParamNames = Object.keys(paramTagsByName);
                    if (node.params) {
                        node.params.forEach((param, paramsIndex) => {
                            const bindingParam = param.type === "AssignmentPattern"
                                ? param.left
                                : param;
                            // TODO(nzakas): Figure out logical things to do with destructured, default, rest params
                            if (bindingParam.type === "Identifier") {
                                const name = bindingParam.name;
                                if (jsdocParamNames[paramsIndex] && (name !== jsdocParamNames[paramsIndex])) {
                                    context.report({
                                        node: jsdocNode,
                                        messageId: "expected",
                                        loc: getAbsoluteRange(jsdocNode, paramTagsByName[jsdocParamNames[paramsIndex]]),
                                        data: {
                                            name,
                                            jsdocName: jsdocParamNames[paramsIndex]
                                        }
                                    });
                                }
                                else if (!paramTagsByName[name] && !isOverride) {
                                    context.report({
                                        node: jsdocNode,
                                        messageId: "missingParam",
                                        data: {
                                            name
                                        }
                                    });
                                }
                            }
                        });
                    }
                    if (options.matchDescription) {
                        const regex = new RegExp(options.matchDescription, "u");
                        if (!regex.test(jsdoc.description)) {
                            context.report({ node: jsdocNode, messageId: "unsatisfiedDesc" });
                        }
                    }
                }
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                ArrowFunctionExpression: startFunction,
                FunctionExpression: startFunction,
                FunctionDeclaration: startFunction,
                ClassExpression: startFunction,
                ClassDeclaration: startFunction,
                "ArrowFunctionExpression:exit": checkJSDoc,
                "FunctionExpression:exit": checkJSDoc,
                "FunctionDeclaration:exit": checkJSDoc,
                "ClassExpression:exit": checkJSDoc,
                "ClassDeclaration:exit": checkJSDoc,
                ReturnStatement: addReturn
            };
        }
    };
},
/* doctrine/lib/doctrine.js */ function _(require, module, exports) {
    /*
     * @fileoverview Main Doctrine object
     * @author Yusuke Suzuki <utatane.tea@gmail.com>
     * @author Dan Tao <daniel.tao@gmail.com>
     * @author Andrew Eisenberg <andrew@eisenberg.as>
     */
    (function () {
        'use strict';
        var typed, utility, jsdoc, esutils, hasOwnProperty;
        esutils = require(127) /* esutils */;
        typed = require(414) /* ./typed */;
        utility = require(415) /* ./utility */;
        function sliceSource(source, index, last) {
            return source.slice(index, last);
        }
        hasOwnProperty = (function () {
            var func = Object.prototype.hasOwnProperty;
            return function hasOwnProperty(obj, name) {
                return func.call(obj, name);
            };
        }());
        function shallowCopy(obj) {
            var ret = {}, key;
            for (key in obj) {
                if (obj.hasOwnProperty(key)) {
                    ret[key] = obj[key];
                }
            }
            return ret;
        }
        function isASCIIAlphanumeric(ch) {
            return (ch >= 0x61 /* 'a' */ && ch <= 0x7A /* 'z' */) ||
                (ch >= 0x41 /* 'A' */ && ch <= 0x5A /* 'Z' */) ||
                (ch >= 0x30 /* '0' */ && ch <= 0x39 /* '9' */);
        }
        function isParamTitle(title) {
            return title === 'param' || title === 'argument' || title === 'arg';
        }
        function isReturnTitle(title) {
            return title === 'return' || title === 'returns';
        }
        function isProperty(title) {
            return title === 'property' || title === 'prop';
        }
        function isNameParameterRequired(title) {
            return isParamTitle(title) || isProperty(title) ||
                title === 'alias' || title === 'this' || title === 'mixes' || title === 'requires';
        }
        function isAllowedName(title) {
            return isNameParameterRequired(title) || title === 'const' || title === 'constant';
        }
        function isAllowedNested(title) {
            return isProperty(title) || isParamTitle(title);
        }
        function isAllowedOptional(title) {
            return isProperty(title) || isParamTitle(title);
        }
        function isTypeParameterRequired(title) {
            return isParamTitle(title) || isReturnTitle(title) ||
                title === 'define' || title === 'enum' ||
                title === 'implements' || title === 'this' ||
                title === 'type' || title === 'typedef' || isProperty(title);
        }
        // Consider deprecation instead using 'isTypeParameterRequired' and 'Rules' declaration to pick when a type is optional/required
        // This would require changes to 'parseType'
        function isAllowedType(title) {
            return isTypeParameterRequired(title) || title === 'throws' || title === 'const' || title === 'constant' ||
                title === 'namespace' || title === 'member' || title === 'var' || title === 'module' ||
                title === 'constructor' || title === 'class' || title === 'extends' || title === 'augments' ||
                title === 'public' || title === 'private' || title === 'protected';
        }
        // A regex character class that contains all whitespace except linebreak characters (\r, \n, \u2028, \u2029)
        var WHITESPACE = '[ \\f\\t\\v\\u00a0\\u1680\\u180e\\u2000-\\u200a\\u202f\\u205f\\u3000\\ufeff]';
        var STAR_MATCHER = '(' + WHITESPACE + '*(?:\\*' + WHITESPACE + '?)?)(.+|[\r\n\u2028\u2029])';
        function unwrapComment(doc) {
            // JSDoc comment is following form
            //   /**
            //    * .......
            //    */
            return doc.
                // remove /**
                replace(/^\/\*\*?/, '').
                // remove */
                replace(/\*\/$/, '').
                // remove ' * ' at the beginning of a line
                replace(new RegExp(STAR_MATCHER, 'g'), '$2').
                // remove trailing whitespace
                replace(/\s*$/, '');
        }
        /**
         * Converts an index in an "unwrapped" JSDoc comment to the corresponding index in the original "wrapped" version
         * @param {string} originalSource The original wrapped comment
         * @param {number} unwrappedIndex The index of a character in the unwrapped string
         * @returns {number} The index of the corresponding character in the original wrapped string
         */
        function convertUnwrappedCommentIndex(originalSource, unwrappedIndex) {
            var replacedSource = originalSource.replace(/^\/\*\*?/, '');
            var numSkippedChars = 0;
            var matcher = new RegExp(STAR_MATCHER, 'g');
            var match;
            while ((match = matcher.exec(replacedSource))) {
                numSkippedChars += match[1].length;
                if (match.index + match[0].length > unwrappedIndex + numSkippedChars) {
                    return unwrappedIndex + numSkippedChars + originalSource.length - replacedSource.length;
                }
            }
            return originalSource.replace(/\*\/$/, '').replace(/\s*$/, '').length;
        }
        // JSDoc Tag Parser
        (function (exports) {
            var Rules, index, lineNumber, length, source, originalSource, recoverable, sloppy, strict;
            function advance() {
                var ch = source.charCodeAt(index);
                index += 1;
                if (esutils.code.isLineTerminator(ch) && !(ch === 0x0D /* '\r' */ && source.charCodeAt(index) === 0x0A /* '\n' */)) {
                    lineNumber += 1;
                }
                return String.fromCharCode(ch);
            }
            function scanTitle() {
                var title = '';
                // waste '@'
                advance();
                while (index < length && isASCIIAlphanumeric(source.charCodeAt(index))) {
                    title += advance();
                }
                return title;
            }
            function seekContent() {
                var ch, waiting, last = index;
                waiting = false;
                while (last < length) {
                    ch = source.charCodeAt(last);
                    if (esutils.code.isLineTerminator(ch) && !(ch === 0x0D /* '\r' */ && source.charCodeAt(last + 1) === 0x0A /* '\n' */)) {
                        waiting = true;
                    }
                    else if (waiting) {
                        if (ch === 0x40 /* '@' */) {
                            break;
                        }
                        if (!esutils.code.isWhiteSpace(ch)) {
                            waiting = false;
                        }
                    }
                    last += 1;
                }
                return last;
            }
            // type expression may have nest brace, such as,
            // { { ok: string } }
            //
            // therefore, scanning type expression with balancing braces.
            function parseType(title, last, addRange) {
                var ch, brace, type, startIndex, direct = false;
                // search '{'
                while (index < last) {
                    ch = source.charCodeAt(index);
                    if (esutils.code.isWhiteSpace(ch)) {
                        advance();
                    }
                    else if (ch === 0x7B /* '{' */) {
                        advance();
                        break;
                    }
                    else {
                        // this is direct pattern
                        direct = true;
                        break;
                    }
                }
                if (direct) {
                    return null;
                }
                // type expression { is found
                brace = 1;
                type = '';
                while (index < last) {
                    ch = source.charCodeAt(index);
                    if (esutils.code.isLineTerminator(ch)) {
                        advance();
                    }
                    else {
                        if (ch === 0x7D /* '}' */) {
                            brace -= 1;
                            if (brace === 0) {
                                advance();
                                break;
                            }
                        }
                        else if (ch === 0x7B /* '{' */) {
                            brace += 1;
                        }
                        if (type === '') {
                            startIndex = index;
                        }
                        type += advance();
                    }
                }
                if (brace !== 0) {
                    // braces is not balanced
                    return utility.throwError('Braces are not balanced');
                }
                if (isAllowedOptional(title)) {
                    return typed.parseParamType(type, { startIndex: convertIndex(startIndex), range: addRange });
                }
                return typed.parseType(type, { startIndex: convertIndex(startIndex), range: addRange });
            }
            function scanIdentifier(last) {
                var identifier;
                if (!esutils.code.isIdentifierStartES5(source.charCodeAt(index)) && !source[index].match(/[0-9]/)) {
                    return null;
                }
                identifier = advance();
                while (index < last && esutils.code.isIdentifierPartES5(source.charCodeAt(index))) {
                    identifier += advance();
                }
                return identifier;
            }
            function skipWhiteSpace(last) {
                while (index < last && (esutils.code.isWhiteSpace(source.charCodeAt(index)) || esutils.code.isLineTerminator(source.charCodeAt(index)))) {
                    advance();
                }
            }
            function parseName(last, allowBrackets, allowNestedParams) {
                var name = '', useBrackets, insideString;
                skipWhiteSpace(last);
                if (index >= last) {
                    return null;
                }
                if (source.charCodeAt(index) === 0x5B /* '[' */) {
                    if (allowBrackets) {
                        useBrackets = true;
                        name = advance();
                    }
                    else {
                        return null;
                    }
                }
                name += scanIdentifier(last);
                if (allowNestedParams) {
                    if (source.charCodeAt(index) === 0x3A /* ':' */ && (name === 'module' ||
                        name === 'external' ||
                        name === 'event')) {
                        name += advance();
                        name += scanIdentifier(last);
                    }
                    if (source.charCodeAt(index) === 0x5B /* '[' */ && source.charCodeAt(index + 1) === 0x5D /* ']' */) {
                        name += advance();
                        name += advance();
                    }
                    while (source.charCodeAt(index) === 0x2E /* '.' */ ||
                        source.charCodeAt(index) === 0x2F /* '/' */ ||
                        source.charCodeAt(index) === 0x23 /* '#' */ ||
                        source.charCodeAt(index) === 0x2D /* '-' */ ||
                        source.charCodeAt(index) === 0x7E /* '~' */) {
                        name += advance();
                        name += scanIdentifier(last);
                    }
                }
                if (useBrackets) {
                    skipWhiteSpace(last);
                    // do we have a default value for this?
                    if (source.charCodeAt(index) === 0x3D /* '=' */) {
                        // consume the '='' symbol
                        name += advance();
                        skipWhiteSpace(last);
                        var ch;
                        var bracketDepth = 1;
                        // scan in the default value
                        while (index < last) {
                            ch = source.charCodeAt(index);
                            if (esutils.code.isWhiteSpace(ch)) {
                                if (!insideString) {
                                    skipWhiteSpace(last);
                                    ch = source.charCodeAt(index);
                                }
                            }
                            if (ch === 0x27 /* ''' */) {
                                if (!insideString) {
                                    insideString = '\'';
                                }
                                else {
                                    if (insideString === '\'') {
                                        insideString = '';
                                    }
                                }
                            }
                            if (ch === 0x22 /* '"' */) {
                                if (!insideString) {
                                    insideString = '"';
                                }
                                else {
                                    if (insideString === '"') {
                                        insideString = '';
                                    }
                                }
                            }
                            if (ch === 0x5B /* '[' */) {
                                bracketDepth++;
                            }
                            else if (ch === 0x5D /* ']' */ &&
                                --bracketDepth === 0) {
                                break;
                            }
                            name += advance();
                        }
                    }
                    skipWhiteSpace(last);
                    if (index >= last || source.charCodeAt(index) !== 0x5D /* ']' */) {
                        // we never found a closing ']'
                        return null;
                    }
                    // collect the last ']'
                    name += advance();
                }
                return name;
            }
            function skipToTag() {
                while (index < length && source.charCodeAt(index) !== 0x40 /* '@' */) {
                    advance();
                }
                if (index >= length) {
                    return false;
                }
                utility.assert(source.charCodeAt(index) === 0x40 /* '@' */);
                return true;
            }
            function convertIndex(rangeIndex) {
                if (source === originalSource) {
                    return rangeIndex;
                }
                return convertUnwrappedCommentIndex(originalSource, rangeIndex);
            }
            function TagParser(options, title) {
                this._options = options;
                this._title = title.toLowerCase();
                this._tag = {
                    title: title,
                    description: null
                };
                if (this._options.lineNumbers) {
                    this._tag.lineNumber = lineNumber;
                }
                this._first = index - title.length - 1;
                this._last = 0;
                // space to save special information for title parsers.
                this._extra = {};
            }
            // addError(err, ...)
            TagParser.prototype.addError = function addError(errorText) {
                var args = Array.prototype.slice.call(arguments, 1), msg = errorText.replace(/%(\d)/g, function (whole, index) {
                    utility.assert(index < args.length, 'Message reference must be in range');
                    return args[index];
                });
                if (!this._tag.errors) {
                    this._tag.errors = [];
                }
                if (strict) {
                    utility.throwError(msg);
                }
                this._tag.errors.push(msg);
                return recoverable;
            };
            TagParser.prototype.parseType = function () {
                // type required titles
                if (isTypeParameterRequired(this._title)) {
                    try {
                        this._tag.type = parseType(this._title, this._last, this._options.range);
                        if (!this._tag.type) {
                            if (!isParamTitle(this._title) && !isReturnTitle(this._title)) {
                                if (!this.addError('Missing or invalid tag type')) {
                                    return false;
                                }
                            }
                        }
                    }
                    catch (error) {
                        this._tag.type = null;
                        if (!this.addError(error.message)) {
                            return false;
                        }
                    }
                }
                else if (isAllowedType(this._title)) {
                    // optional types
                    try {
                        this._tag.type = parseType(this._title, this._last, this._options.range);
                    }
                    catch (e) {
                        //For optional types, lets drop the thrown error when we hit the end of the file
                    }
                }
                return true;
            };
            TagParser.prototype._parseNamePath = function (optional) {
                var name;
                name = parseName(this._last, sloppy && isAllowedOptional(this._title), true);
                if (!name) {
                    if (!optional) {
                        if (!this.addError('Missing or invalid tag name')) {
                            return false;
                        }
                    }
                }
                this._tag.name = name;
                return true;
            };
            TagParser.prototype.parseNamePath = function () {
                return this._parseNamePath(false);
            };
            TagParser.prototype.parseNamePathOptional = function () {
                return this._parseNamePath(true);
            };
            TagParser.prototype.parseName = function () {
                var assign, name;
                // param, property requires name
                if (isAllowedName(this._title)) {
                    this._tag.name = parseName(this._last, sloppy && isAllowedOptional(this._title), isAllowedNested(this._title));
                    if (!this._tag.name) {
                        if (!isNameParameterRequired(this._title)) {
                            return true;
                        }
                        // it's possible the name has already been parsed but interpreted as a type
                        // it's also possible this is a sloppy declaration, in which case it will be
                        // fixed at the end
                        if (isParamTitle(this._title) && this._tag.type && this._tag.type.name) {
                            this._extra.name = this._tag.type;
                            this._tag.name = this._tag.type.name;
                            this._tag.type = null;
                        }
                        else {
                            if (!this.addError('Missing or invalid tag name')) {
                                return false;
                            }
                        }
                    }
                    else {
                        name = this._tag.name;
                        if (name.charAt(0) === '[' && name.charAt(name.length - 1) === ']') {
                            // extract the default value if there is one
                            // example: @param {string} [somebody=John Doe] description
                            assign = name.substring(1, name.length - 1).split('=');
                            if (assign.length > 1) {
                                this._tag['default'] = assign.slice(1).join('=');
                            }
                            this._tag.name = assign[0];
                            // convert to an optional type
                            if (this._tag.type && this._tag.type.type !== 'OptionalType') {
                                this._tag.type = {
                                    type: 'OptionalType',
                                    expression: this._tag.type
                                };
                            }
                        }
                    }
                }
                return true;
            };
            TagParser.prototype.parseDescription = function parseDescription() {
                var description = sliceSource(source, index, this._last).trim();
                if (description) {
                    if ((/^-\s+/).test(description)) {
                        description = description.substring(2);
                    }
                    this._tag.description = description;
                }
                return true;
            };
            TagParser.prototype.parseCaption = function parseDescription() {
                var description = sliceSource(source, index, this._last).trim();
                var captionStartTag = '<caption>';
                var captionEndTag = '</caption>';
                var captionStart = description.indexOf(captionStartTag);
                var captionEnd = description.indexOf(captionEndTag);
                if (captionStart >= 0 && captionEnd >= 0) {
                    this._tag.caption = description.substring(captionStart + captionStartTag.length, captionEnd).trim();
                    this._tag.description = description.substring(captionEnd + captionEndTag.length).trim();
                }
                else {
                    this._tag.description = description;
                }
                return true;
            };
            TagParser.prototype.parseKind = function parseKind() {
                var kind, kinds;
                kinds = {
                    'class': true,
                    'constant': true,
                    'event': true,
                    'external': true,
                    'file': true,
                    'function': true,
                    'member': true,
                    'mixin': true,
                    'module': true,
                    'namespace': true,
                    'typedef': true
                };
                kind = sliceSource(source, index, this._last).trim();
                this._tag.kind = kind;
                if (!hasOwnProperty(kinds, kind)) {
                    if (!this.addError('Invalid kind name \'%0\'', kind)) {
                        return false;
                    }
                }
                return true;
            };
            TagParser.prototype.parseAccess = function parseAccess() {
                var access;
                access = sliceSource(source, index, this._last).trim();
                this._tag.access = access;
                if (access !== 'private' && access !== 'protected' && access !== 'public') {
                    if (!this.addError('Invalid access name \'%0\'', access)) {
                        return false;
                    }
                }
                return true;
            };
            TagParser.prototype.parseThis = function parseThis() {
                // this name may be a name expression (e.g. {foo.bar}),
                // an union (e.g. {foo.bar|foo.baz}) or a name path (e.g. foo.bar)
                var value = sliceSource(source, index, this._last).trim();
                if (value && value.charAt(0) === '{') {
                    var gotType = this.parseType();
                    if (gotType && this._tag.type.type === 'NameExpression' || this._tag.type.type === 'UnionType') {
                        this._tag.name = this._tag.type.name;
                        return true;
                    }
                    else {
                        return this.addError('Invalid name for this');
                    }
                }
                else {
                    return this.parseNamePath();
                }
            };
            TagParser.prototype.parseVariation = function parseVariation() {
                var variation, text;
                text = sliceSource(source, index, this._last).trim();
                variation = parseFloat(text, 10);
                this._tag.variation = variation;
                if (isNaN(variation)) {
                    if (!this.addError('Invalid variation \'%0\'', text)) {
                        return false;
                    }
                }
                return true;
            };
            TagParser.prototype.ensureEnd = function () {
                var shouldBeEmpty = sliceSource(source, index, this._last).trim();
                if (shouldBeEmpty) {
                    if (!this.addError('Unknown content \'%0\'', shouldBeEmpty)) {
                        return false;
                    }
                }
                return true;
            };
            TagParser.prototype.epilogue = function epilogue() {
                var description;
                description = this._tag.description;
                // un-fix potentially sloppy declaration
                if (isAllowedOptional(this._title) && !this._tag.type && description && description.charAt(0) === '[') {
                    this._tag.type = this._extra.name;
                    if (!this._tag.name) {
                        this._tag.name = undefined;
                    }
                    if (!sloppy) {
                        if (!this.addError('Missing or invalid tag name')) {
                            return false;
                        }
                    }
                }
                return true;
            };
            Rules = {
                // http://usejsdoc.org/tags-access.html
                'access': ['parseAccess'],
                // http://usejsdoc.org/tags-alias.html
                'alias': ['parseNamePath', 'ensureEnd'],
                // http://usejsdoc.org/tags-augments.html
                'augments': ['parseType', 'parseNamePathOptional', 'ensureEnd'],
                // http://usejsdoc.org/tags-constructor.html
                'constructor': ['parseType', 'parseNamePathOptional', 'ensureEnd'],
                // Synonym: http://usejsdoc.org/tags-constructor.html
                'class': ['parseType', 'parseNamePathOptional', 'ensureEnd'],
                // Synonym: http://usejsdoc.org/tags-extends.html
                'extends': ['parseType', 'parseNamePathOptional', 'ensureEnd'],
                // http://usejsdoc.org/tags-example.html
                'example': ['parseCaption'],
                // http://usejsdoc.org/tags-deprecated.html
                'deprecated': ['parseDescription'],
                // http://usejsdoc.org/tags-global.html
                'global': ['ensureEnd'],
                // http://usejsdoc.org/tags-inner.html
                'inner': ['ensureEnd'],
                // http://usejsdoc.org/tags-instance.html
                'instance': ['ensureEnd'],
                // http://usejsdoc.org/tags-kind.html
                'kind': ['parseKind'],
                // http://usejsdoc.org/tags-mixes.html
                'mixes': ['parseNamePath', 'ensureEnd'],
                // http://usejsdoc.org/tags-mixin.html
                'mixin': ['parseNamePathOptional', 'ensureEnd'],
                // http://usejsdoc.org/tags-member.html
                'member': ['parseType', 'parseNamePathOptional', 'ensureEnd'],
                // http://usejsdoc.org/tags-method.html
                'method': ['parseNamePathOptional', 'ensureEnd'],
                // http://usejsdoc.org/tags-module.html
                'module': ['parseType', 'parseNamePathOptional', 'ensureEnd'],
                // Synonym: http://usejsdoc.org/tags-method.html
                'func': ['parseNamePathOptional', 'ensureEnd'],
                // Synonym: http://usejsdoc.org/tags-method.html
                'function': ['parseNamePathOptional', 'ensureEnd'],
                // Synonym: http://usejsdoc.org/tags-member.html
                'var': ['parseType', 'parseNamePathOptional', 'ensureEnd'],
                // http://usejsdoc.org/tags-name.html
                'name': ['parseNamePath', 'ensureEnd'],
                // http://usejsdoc.org/tags-namespace.html
                'namespace': ['parseType', 'parseNamePathOptional', 'ensureEnd'],
                // http://usejsdoc.org/tags-private.html
                'private': ['parseType', 'parseDescription'],
                // http://usejsdoc.org/tags-protected.html
                'protected': ['parseType', 'parseDescription'],
                // http://usejsdoc.org/tags-public.html
                'public': ['parseType', 'parseDescription'],
                // http://usejsdoc.org/tags-readonly.html
                'readonly': ['ensureEnd'],
                // http://usejsdoc.org/tags-requires.html
                'requires': ['parseNamePath', 'ensureEnd'],
                // http://usejsdoc.org/tags-since.html
                'since': ['parseDescription'],
                // http://usejsdoc.org/tags-static.html
                'static': ['ensureEnd'],
                // http://usejsdoc.org/tags-summary.html
                'summary': ['parseDescription'],
                // http://usejsdoc.org/tags-this.html
                'this': ['parseThis', 'ensureEnd'],
                // http://usejsdoc.org/tags-todo.html
                'todo': ['parseDescription'],
                // http://usejsdoc.org/tags-typedef.html
                'typedef': ['parseType', 'parseNamePathOptional'],
                // http://usejsdoc.org/tags-variation.html
                'variation': ['parseVariation'],
                // http://usejsdoc.org/tags-version.html
                'version': ['parseDescription']
            };
            TagParser.prototype.parse = function parse() {
                var i, iz, sequences, method;
                // empty title
                if (!this._title) {
                    if (!this.addError('Missing or invalid title')) {
                        return null;
                    }
                }
                // Seek to content last index.
                this._last = seekContent(this._title);
                if (this._options.range) {
                    this._tag.range = [this._first, source.slice(0, this._last).replace(/\s*$/, '').length].map(convertIndex);
                }
                if (hasOwnProperty(Rules, this._title)) {
                    sequences = Rules[this._title];
                }
                else {
                    // default sequences
                    sequences = ['parseType', 'parseName', 'parseDescription', 'epilogue'];
                }
                for (i = 0, iz = sequences.length; i < iz; ++i) {
                    method = sequences[i];
                    if (!this[method]()) {
                        return null;
                    }
                }
                return this._tag;
            };
            function parseTag(options) {
                var title, parser, tag;
                // skip to tag
                if (!skipToTag()) {
                    return null;
                }
                // scan title
                title = scanTitle();
                // construct tag parser
                parser = new TagParser(options, title);
                tag = parser.parse();
                // Seek global index to end of this tag.
                while (index < parser._last) {
                    advance();
                }
                return tag;
            }
            //
            // Parse JSDoc
            //
            function scanJSDocDescription(preserveWhitespace) {
                var description = '', ch, atAllowed;
                atAllowed = true;
                while (index < length) {
                    ch = source.charCodeAt(index);
                    if (atAllowed && ch === 0x40 /* '@' */) {
                        break;
                    }
                    if (esutils.code.isLineTerminator(ch)) {
                        atAllowed = true;
                    }
                    else if (atAllowed && !esutils.code.isWhiteSpace(ch)) {
                        atAllowed = false;
                    }
                    description += advance();
                }
                return preserveWhitespace ? description : description.trim();
            }
            function parse(comment, options) {
                var tags = [], tag, description, interestingTags, i, iz;
                if (options === undefined) {
                    options = {};
                }
                if (typeof options.unwrap === 'boolean' && options.unwrap) {
                    source = unwrapComment(comment);
                }
                else {
                    source = comment;
                }
                originalSource = comment;
                // array of relevant tags
                if (options.tags) {
                    if (Array.isArray(options.tags)) {
                        interestingTags = {};
                        for (i = 0, iz = options.tags.length; i < iz; i++) {
                            if (typeof options.tags[i] === 'string') {
                                interestingTags[options.tags[i]] = true;
                            }
                            else {
                                utility.throwError('Invalid "tags" parameter: ' + options.tags);
                            }
                        }
                    }
                    else {
                        utility.throwError('Invalid "tags" parameter: ' + options.tags);
                    }
                }
                length = source.length;
                index = 0;
                lineNumber = 0;
                recoverable = options.recoverable;
                sloppy = options.sloppy;
                strict = options.strict;
                description = scanJSDocDescription(options.preserveWhitespace);
                while (true) {
                    tag = parseTag(options);
                    if (!tag) {
                        break;
                    }
                    if (!interestingTags || interestingTags.hasOwnProperty(tag.title)) {
                        tags.push(tag);
                    }
                }
                return {
                    description: description,
                    tags: tags
                };
            }
            exports.parse = parse;
        }(jsdoc = {}));
        exports.version = utility.VERSION;
        exports.parse = jsdoc.parse;
        exports.parseType = typed.parseType;
        exports.parseParamType = typed.parseParamType;
        exports.unwrapComment = unwrapComment;
        exports.Syntax = shallowCopy(typed.Syntax);
        exports.Error = utility.DoctrineError;
        exports.type = {
            Syntax: exports.Syntax,
            parseType: typed.parseType,
            parseParamType: typed.parseParamType,
            stringify: typed.stringify
        };
    }());
},
/* doctrine/lib/typed.js */ function _(require, module, exports) {
    /*
     * @fileoverview Type expression parser.
     * @author Yusuke Suzuki <utatane.tea@gmail.com>
     * @author Dan Tao <daniel.tao@gmail.com>
     * @author Andrew Eisenberg <andrew@eisenberg.as>
     */
    // "typed", the Type Expression Parser for doctrine.
    (function () {
        'use strict';
        var Syntax, Token, source, length, index, previous, token, value, esutils, utility, rangeOffset, addRange;
        esutils = require(127) /* esutils */;
        utility = require(415) /* ./utility */;
        Syntax = {
            NullableLiteral: 'NullableLiteral',
            AllLiteral: 'AllLiteral',
            NullLiteral: 'NullLiteral',
            UndefinedLiteral: 'UndefinedLiteral',
            VoidLiteral: 'VoidLiteral',
            UnionType: 'UnionType',
            ArrayType: 'ArrayType',
            RecordType: 'RecordType',
            FieldType: 'FieldType',
            FunctionType: 'FunctionType',
            ParameterType: 'ParameterType',
            RestType: 'RestType',
            NonNullableType: 'NonNullableType',
            OptionalType: 'OptionalType',
            NullableType: 'NullableType',
            NameExpression: 'NameExpression',
            TypeApplication: 'TypeApplication',
            StringLiteralType: 'StringLiteralType',
            NumericLiteralType: 'NumericLiteralType',
            BooleanLiteralType: 'BooleanLiteralType'
        };
        Token = {
            ILLEGAL: 0,
            DOT_LT: 1,
            REST: 2,
            LT: 3,
            GT: 4,
            LPAREN: 5,
            RPAREN: 6,
            LBRACE: 7,
            RBRACE: 8,
            LBRACK: 9,
            RBRACK: 10,
            COMMA: 11,
            COLON: 12,
            STAR: 13,
            PIPE: 14,
            QUESTION: 15,
            BANG: 16,
            EQUAL: 17,
            NAME: 18,
            STRING: 19,
            NUMBER: 20,
            EOF: 21
        };
        function isTypeName(ch) {
            return '><(){}[],:*|?!='.indexOf(String.fromCharCode(ch)) === -1 && !esutils.code.isWhiteSpace(ch) && !esutils.code.isLineTerminator(ch);
        }
        function Context(previous, index, token, value) {
            this._previous = previous;
            this._index = index;
            this._token = token;
            this._value = value;
        }
        Context.prototype.restore = function () {
            previous = this._previous;
            index = this._index;
            token = this._token;
            value = this._value;
        };
        Context.save = function () {
            return new Context(previous, index, token, value);
        };
        function maybeAddRange(node, range) {
            if (addRange) {
                node.range = [range[0] + rangeOffset, range[1] + rangeOffset];
            }
            return node;
        }
        function advance() {
            var ch = source.charAt(index);
            index += 1;
            return ch;
        }
        function scanHexEscape(prefix) {
            var i, len, ch, code = 0;
            len = (prefix === 'u') ? 4 : 2;
            for (i = 0; i < len; ++i) {
                if (index < length && esutils.code.isHexDigit(source.charCodeAt(index))) {
                    ch = advance();
                    code = code * 16 + '0123456789abcdef'.indexOf(ch.toLowerCase());
                }
                else {
                    return '';
                }
            }
            return String.fromCharCode(code);
        }
        function scanString() {
            var str = '', quote, ch, code, unescaped, restore; //TODO review removal octal = false
            quote = source.charAt(index);
            ++index;
            while (index < length) {
                ch = advance();
                if (ch === quote) {
                    quote = '';
                    break;
                }
                else if (ch === '\\') {
                    ch = advance();
                    if (!esutils.code.isLineTerminator(ch.charCodeAt(0))) {
                        switch (ch) {
                            case 'n':
                                str += '\n';
                                break;
                            case 'r':
                                str += '\r';
                                break;
                            case 't':
                                str += '\t';
                                break;
                            case 'u':
                            case 'x':
                                restore = index;
                                unescaped = scanHexEscape(ch);
                                if (unescaped) {
                                    str += unescaped;
                                }
                                else {
                                    index = restore;
                                    str += ch;
                                }
                                break;
                            case 'b':
                                str += '\b';
                                break;
                            case 'f':
                                str += '\f';
                                break;
                            case 'v':
                                str += '\v';
                                break;
                            default:
                                if (esutils.code.isOctalDigit(ch.charCodeAt(0))) {
                                    code = '01234567'.indexOf(ch);
                                    // \0 is not octal escape sequence
                                    // Deprecating unused code. TODO review removal
                                    //if (code !== 0) {
                                    //    octal = true;
                                    //}
                                    if (index < length && esutils.code.isOctalDigit(source.charCodeAt(index))) {
                                        //TODO Review Removal octal = true;
                                        code = code * 8 + '01234567'.indexOf(advance());
                                        // 3 digits are only allowed when string starts
                                        // with 0, 1, 2, 3
                                        if ('0123'.indexOf(ch) >= 0 &&
                                            index < length &&
                                            esutils.code.isOctalDigit(source.charCodeAt(index))) {
                                            code = code * 8 + '01234567'.indexOf(advance());
                                        }
                                    }
                                    str += String.fromCharCode(code);
                                }
                                else {
                                    str += ch;
                                }
                                break;
                        }
                    }
                    else {
                        if (ch === '\r' && source.charCodeAt(index) === 0x0A /* '\n' */) {
                            ++index;
                        }
                    }
                }
                else if (esutils.code.isLineTerminator(ch.charCodeAt(0))) {
                    break;
                }
                else {
                    str += ch;
                }
            }
            if (quote !== '') {
                utility.throwError('unexpected quote');
            }
            value = str;
            return Token.STRING;
        }
        function scanNumber() {
            var number, ch;
            number = '';
            ch = source.charCodeAt(index);
            if (ch !== 0x2E /* '.' */) {
                number = advance();
                ch = source.charCodeAt(index);
                if (number === '0') {
                    if (ch === 0x78 /* 'x' */ || ch === 0x58 /* 'X' */) {
                        number += advance();
                        while (index < length) {
                            ch = source.charCodeAt(index);
                            if (!esutils.code.isHexDigit(ch)) {
                                break;
                            }
                            number += advance();
                        }
                        if (number.length <= 2) {
                            // only 0x
                            utility.throwError('unexpected token');
                        }
                        if (index < length) {
                            ch = source.charCodeAt(index);
                            if (esutils.code.isIdentifierStartES5(ch)) {
                                utility.throwError('unexpected token');
                            }
                        }
                        value = parseInt(number, 16);
                        return Token.NUMBER;
                    }
                    if (esutils.code.isOctalDigit(ch)) {
                        number += advance();
                        while (index < length) {
                            ch = source.charCodeAt(index);
                            if (!esutils.code.isOctalDigit(ch)) {
                                break;
                            }
                            number += advance();
                        }
                        if (index < length) {
                            ch = source.charCodeAt(index);
                            if (esutils.code.isIdentifierStartES5(ch) || esutils.code.isDecimalDigit(ch)) {
                                utility.throwError('unexpected token');
                            }
                        }
                        value = parseInt(number, 8);
                        return Token.NUMBER;
                    }
                    if (esutils.code.isDecimalDigit(ch)) {
                        utility.throwError('unexpected token');
                    }
                }
                while (index < length) {
                    ch = source.charCodeAt(index);
                    if (!esutils.code.isDecimalDigit(ch)) {
                        break;
                    }
                    number += advance();
                }
            }
            if (ch === 0x2E /* '.' */) {
                number += advance();
                while (index < length) {
                    ch = source.charCodeAt(index);
                    if (!esutils.code.isDecimalDigit(ch)) {
                        break;
                    }
                    number += advance();
                }
            }
            if (ch === 0x65 /* 'e' */ || ch === 0x45 /* 'E' */) {
                number += advance();
                ch = source.charCodeAt(index);
                if (ch === 0x2B /* '+' */ || ch === 0x2D /* '-' */) {
                    number += advance();
                }
                ch = source.charCodeAt(index);
                if (esutils.code.isDecimalDigit(ch)) {
                    number += advance();
                    while (index < length) {
                        ch = source.charCodeAt(index);
                        if (!esutils.code.isDecimalDigit(ch)) {
                            break;
                        }
                        number += advance();
                    }
                }
                else {
                    utility.throwError('unexpected token');
                }
            }
            if (index < length) {
                ch = source.charCodeAt(index);
                if (esutils.code.isIdentifierStartES5(ch)) {
                    utility.throwError('unexpected token');
                }
            }
            value = parseFloat(number);
            return Token.NUMBER;
        }
        function scanTypeName() {
            var ch, ch2;
            value = advance();
            while (index < length && isTypeName(source.charCodeAt(index))) {
                ch = source.charCodeAt(index);
                if (ch === 0x2E /* '.' */) {
                    if ((index + 1) >= length) {
                        return Token.ILLEGAL;
                    }
                    ch2 = source.charCodeAt(index + 1);
                    if (ch2 === 0x3C /* '<' */) {
                        break;
                    }
                }
                value += advance();
            }
            return Token.NAME;
        }
        function next() {
            var ch;
            previous = index;
            while (index < length && esutils.code.isWhiteSpace(source.charCodeAt(index))) {
                advance();
            }
            if (index >= length) {
                token = Token.EOF;
                return token;
            }
            ch = source.charCodeAt(index);
            switch (ch) {
                case 0x27: /* ''' */
                case 0x22: /* '"' */
                    token = scanString();
                    return token;
                case 0x3A: /* ':' */
                    advance();
                    token = Token.COLON;
                    return token;
                case 0x2C: /* ',' */
                    advance();
                    token = Token.COMMA;
                    return token;
                case 0x28: /* '(' */
                    advance();
                    token = Token.LPAREN;
                    return token;
                case 0x29: /* ')' */
                    advance();
                    token = Token.RPAREN;
                    return token;
                case 0x5B: /* '[' */
                    advance();
                    token = Token.LBRACK;
                    return token;
                case 0x5D: /* ']' */
                    advance();
                    token = Token.RBRACK;
                    return token;
                case 0x7B: /* '{' */
                    advance();
                    token = Token.LBRACE;
                    return token;
                case 0x7D: /* '}' */
                    advance();
                    token = Token.RBRACE;
                    return token;
                case 0x2E: /* '.' */
                    if (index + 1 < length) {
                        ch = source.charCodeAt(index + 1);
                        if (ch === 0x3C /* '<' */) {
                            advance(); // '.'
                            advance(); // '<'
                            token = Token.DOT_LT;
                            return token;
                        }
                        if (ch === 0x2E /* '.' */ && index + 2 < length && source.charCodeAt(index + 2) === 0x2E /* '.' */) {
                            advance(); // '.'
                            advance(); // '.'
                            advance(); // '.'
                            token = Token.REST;
                            return token;
                        }
                        if (esutils.code.isDecimalDigit(ch)) {
                            token = scanNumber();
                            return token;
                        }
                    }
                    token = Token.ILLEGAL;
                    return token;
                case 0x3C: /* '<' */
                    advance();
                    token = Token.LT;
                    return token;
                case 0x3E: /* '>' */
                    advance();
                    token = Token.GT;
                    return token;
                case 0x2A: /* '*' */
                    advance();
                    token = Token.STAR;
                    return token;
                case 0x7C: /* '|' */
                    advance();
                    token = Token.PIPE;
                    return token;
                case 0x3F: /* '?' */
                    advance();
                    token = Token.QUESTION;
                    return token;
                case 0x21: /* '!' */
                    advance();
                    token = Token.BANG;
                    return token;
                case 0x3D: /* '=' */
                    advance();
                    token = Token.EQUAL;
                    return token;
                case 0x2D: /* '-' */
                    token = scanNumber();
                    return token;
                default:
                    if (esutils.code.isDecimalDigit(ch)) {
                        token = scanNumber();
                        return token;
                    }
                    // type string permits following case,
                    //
                    // namespace.module.MyClass
                    //
                    // this reduced 1 token TK_NAME
                    utility.assert(isTypeName(ch));
                    token = scanTypeName();
                    return token;
            }
        }
        function consume(target, text) {
            utility.assert(token === target, text || 'consumed token not matched');
            next();
        }
        function expect(target, message) {
            if (token !== target) {
                utility.throwError(message || 'unexpected token');
            }
            next();
        }
        // UnionType := '(' TypeUnionList ')'
        //
        // TypeUnionList :=
        //     <<empty>>
        //   | NonemptyTypeUnionList
        //
        // NonemptyTypeUnionList :=
        //     TypeExpression
        //   | TypeExpression '|' NonemptyTypeUnionList
        function parseUnionType() {
            var elements, startIndex = index - 1;
            consume(Token.LPAREN, 'UnionType should start with (');
            elements = [];
            if (token !== Token.RPAREN) {
                while (true) {
                    elements.push(parseTypeExpression());
                    if (token === Token.RPAREN) {
                        break;
                    }
                    expect(Token.PIPE);
                }
            }
            consume(Token.RPAREN, 'UnionType should end with )');
            return maybeAddRange({
                type: Syntax.UnionType,
                elements: elements
            }, [startIndex, previous]);
        }
        // ArrayType := '[' ElementTypeList ']'
        //
        // ElementTypeList :=
        //     <<empty>>
        //  | TypeExpression
        //  | '...' TypeExpression
        //  | TypeExpression ',' ElementTypeList
        function parseArrayType() {
            var elements, startIndex = index - 1, restStartIndex;
            consume(Token.LBRACK, 'ArrayType should start with [');
            elements = [];
            while (token !== Token.RBRACK) {
                if (token === Token.REST) {
                    restStartIndex = index - 3;
                    consume(Token.REST);
                    elements.push(maybeAddRange({
                        type: Syntax.RestType,
                        expression: parseTypeExpression()
                    }, [restStartIndex, previous]));
                    break;
                }
                else {
                    elements.push(parseTypeExpression());
                }
                if (token !== Token.RBRACK) {
                    expect(Token.COMMA);
                }
            }
            expect(Token.RBRACK);
            return maybeAddRange({
                type: Syntax.ArrayType,
                elements: elements
            }, [startIndex, previous]);
        }
        function parseFieldName() {
            var v = value;
            if (token === Token.NAME || token === Token.STRING) {
                next();
                return v;
            }
            if (token === Token.NUMBER) {
                consume(Token.NUMBER);
                return String(v);
            }
            utility.throwError('unexpected token');
        }
        // FieldType :=
        //     FieldName
        //   | FieldName ':' TypeExpression
        //
        // FieldName :=
        //     NameExpression
        //   | StringLiteral
        //   | NumberLiteral
        //   | ReservedIdentifier
        function parseFieldType() {
            var key, rangeStart = previous;
            key = parseFieldName();
            if (token === Token.COLON) {
                consume(Token.COLON);
                return maybeAddRange({
                    type: Syntax.FieldType,
                    key: key,
                    value: parseTypeExpression()
                }, [rangeStart, previous]);
            }
            return maybeAddRange({
                type: Syntax.FieldType,
                key: key,
                value: null
            }, [rangeStart, previous]);
        }
        // RecordType := '{' FieldTypeList '}'
        //
        // FieldTypeList :=
        //     <<empty>>
        //   | FieldType
        //   | FieldType ',' FieldTypeList
        function parseRecordType() {
            var fields, rangeStart = index - 1, rangeEnd;
            consume(Token.LBRACE, 'RecordType should start with {');
            fields = [];
            if (token === Token.COMMA) {
                consume(Token.COMMA);
            }
            else {
                while (token !== Token.RBRACE) {
                    fields.push(parseFieldType());
                    if (token !== Token.RBRACE) {
                        expect(Token.COMMA);
                    }
                }
            }
            rangeEnd = index;
            expect(Token.RBRACE);
            return maybeAddRange({
                type: Syntax.RecordType,
                fields: fields
            }, [rangeStart, rangeEnd]);
        }
        // NameExpression :=
        //    Identifier
        //  | TagIdentifier ':' Identifier
        //
        // Tag identifier is one of "module", "external" or "event"
        // Identifier is the same as Token.NAME, including any dots, something like
        // namespace.module.MyClass
        function parseNameExpression() {
            var name = value, rangeStart = index - name.length;
            expect(Token.NAME);
            if (token === Token.COLON && (name === 'module' ||
                name === 'external' ||
                name === 'event')) {
                consume(Token.COLON);
                name += ':' + value;
                expect(Token.NAME);
            }
            return maybeAddRange({
                type: Syntax.NameExpression,
                name: name
            }, [rangeStart, previous]);
        }
        // TypeExpressionList :=
        //     TopLevelTypeExpression
        //   | TopLevelTypeExpression ',' TypeExpressionList
        function parseTypeExpressionList() {
            var elements = [];
            elements.push(parseTop());
            while (token === Token.COMMA) {
                consume(Token.COMMA);
                elements.push(parseTop());
            }
            return elements;
        }
        // TypeName :=
        //     NameExpression
        //   | NameExpression TypeApplication
        //
        // TypeApplication :=
        //     '.<' TypeExpressionList '>'
        //   | '<' TypeExpressionList '>'   // this is extension of doctrine
        function parseTypeName() {
            var expr, applications, startIndex = index - value.length;
            expr = parseNameExpression();
            if (token === Token.DOT_LT || token === Token.LT) {
                next();
                applications = parseTypeExpressionList();
                expect(Token.GT);
                return maybeAddRange({
                    type: Syntax.TypeApplication,
                    expression: expr,
                    applications: applications
                }, [startIndex, previous]);
            }
            return expr;
        }
        // ResultType :=
        //     <<empty>>
        //   | ':' void
        //   | ':' TypeExpression
        //
        // BNF is above
        // but, we remove <<empty>> pattern, so token is always TypeToken::COLON
        function parseResultType() {
            consume(Token.COLON, 'ResultType should start with :');
            if (token === Token.NAME && value === 'void') {
                consume(Token.NAME);
                return {
                    type: Syntax.VoidLiteral
                };
            }
            return parseTypeExpression();
        }
        // ParametersType :=
        //     RestParameterType
        //   | NonRestParametersType
        //   | NonRestParametersType ',' RestParameterType
        //
        // RestParameterType :=
        //     '...'
        //     '...' Identifier
        //
        // NonRestParametersType :=
        //     ParameterType ',' NonRestParametersType
        //   | ParameterType
        //   | OptionalParametersType
        //
        // OptionalParametersType :=
        //     OptionalParameterType
        //   | OptionalParameterType, OptionalParametersType
        //
        // OptionalParameterType := ParameterType=
        //
        // ParameterType := TypeExpression | Identifier ':' TypeExpression
        //
        // Identifier is "new" or "this"
        function parseParametersType() {
            var params = [], optionalSequence = false, expr, rest = false, startIndex, restStartIndex = index - 3, nameStartIndex;
            while (token !== Token.RPAREN) {
                if (token === Token.REST) {
                    // RestParameterType
                    consume(Token.REST);
                    rest = true;
                }
                startIndex = previous;
                expr = parseTypeExpression();
                if (expr.type === Syntax.NameExpression && token === Token.COLON) {
                    nameStartIndex = previous - expr.name.length;
                    // Identifier ':' TypeExpression
                    consume(Token.COLON);
                    expr = maybeAddRange({
                        type: Syntax.ParameterType,
                        name: expr.name,
                        expression: parseTypeExpression()
                    }, [nameStartIndex, previous]);
                }
                if (token === Token.EQUAL) {
                    consume(Token.EQUAL);
                    expr = maybeAddRange({
                        type: Syntax.OptionalType,
                        expression: expr
                    }, [startIndex, previous]);
                    optionalSequence = true;
                }
                else {
                    if (optionalSequence) {
                        utility.throwError('unexpected token');
                    }
                }
                if (rest) {
                    expr = maybeAddRange({
                        type: Syntax.RestType,
                        expression: expr
                    }, [restStartIndex, previous]);
                }
                params.push(expr);
                if (token !== Token.RPAREN) {
                    expect(Token.COMMA);
                }
            }
            return params;
        }
        // FunctionType := 'function' FunctionSignatureType
        //
        // FunctionSignatureType :=
        //   | TypeParameters '(' ')' ResultType
        //   | TypeParameters '(' ParametersType ')' ResultType
        //   | TypeParameters '(' 'this' ':' TypeName ')' ResultType
        //   | TypeParameters '(' 'this' ':' TypeName ',' ParametersType ')' ResultType
        function parseFunctionType() {
            var isNew, thisBinding, params, result, fnType, startIndex = index - value.length;
            utility.assert(token === Token.NAME && value === 'function', 'FunctionType should start with \'function\'');
            consume(Token.NAME);
            // Google Closure Compiler is not implementing TypeParameters.
            // So we do not. if we don't get '(', we see it as error.
            expect(Token.LPAREN);
            isNew = false;
            params = [];
            thisBinding = null;
            if (token !== Token.RPAREN) {
                // ParametersType or 'this'
                if (token === Token.NAME &&
                    (value === 'this' || value === 'new')) {
                    // 'this' or 'new'
                    // 'new' is Closure Compiler extension
                    isNew = value === 'new';
                    consume(Token.NAME);
                    expect(Token.COLON);
                    thisBinding = parseTypeName();
                    if (token === Token.COMMA) {
                        consume(Token.COMMA);
                        params = parseParametersType();
                    }
                }
                else {
                    params = parseParametersType();
                }
            }
            expect(Token.RPAREN);
            result = null;
            if (token === Token.COLON) {
                result = parseResultType();
            }
            fnType = maybeAddRange({
                type: Syntax.FunctionType,
                params: params,
                result: result
            }, [startIndex, previous]);
            if (thisBinding) {
                // avoid adding null 'new' and 'this' properties
                fnType['this'] = thisBinding;
                if (isNew) {
                    fnType['new'] = true;
                }
            }
            return fnType;
        }
        // BasicTypeExpression :=
        //     '*'
        //   | 'null'
        //   | 'undefined'
        //   | TypeName
        //   | FunctionType
        //   | UnionType
        //   | RecordType
        //   | ArrayType
        function parseBasicTypeExpression() {
            var context, startIndex;
            switch (token) {
                case Token.STAR:
                    consume(Token.STAR);
                    return maybeAddRange({
                        type: Syntax.AllLiteral
                    }, [previous - 1, previous]);
                case Token.LPAREN:
                    return parseUnionType();
                case Token.LBRACK:
                    return parseArrayType();
                case Token.LBRACE:
                    return parseRecordType();
                case Token.NAME:
                    startIndex = index - value.length;
                    if (value === 'null') {
                        consume(Token.NAME);
                        return maybeAddRange({
                            type: Syntax.NullLiteral
                        }, [startIndex, previous]);
                    }
                    if (value === 'undefined') {
                        consume(Token.NAME);
                        return maybeAddRange({
                            type: Syntax.UndefinedLiteral
                        }, [startIndex, previous]);
                    }
                    if (value === 'true' || value === 'false') {
                        consume(Token.NAME);
                        return maybeAddRange({
                            type: Syntax.BooleanLiteralType,
                            value: value === 'true'
                        }, [startIndex, previous]);
                    }
                    context = Context.save();
                    if (value === 'function') {
                        try {
                            return parseFunctionType();
                        }
                        catch (e) {
                            context.restore();
                        }
                    }
                    return parseTypeName();
                case Token.STRING:
                    next();
                    return maybeAddRange({
                        type: Syntax.StringLiteralType,
                        value: value
                    }, [previous - value.length - 2, previous]);
                case Token.NUMBER:
                    next();
                    return maybeAddRange({
                        type: Syntax.NumericLiteralType,
                        value: value
                    }, [previous - String(value).length, previous]);
                default:
                    utility.throwError('unexpected token');
            }
        }
        // TypeExpression :=
        //     BasicTypeExpression
        //   | '?' BasicTypeExpression
        //   | '!' BasicTypeExpression
        //   | BasicTypeExpression '?'
        //   | BasicTypeExpression '!'
        //   | '?'
        //   | BasicTypeExpression '[]'
        function parseTypeExpression() {
            var expr, rangeStart;
            if (token === Token.QUESTION) {
                rangeStart = index - 1;
                consume(Token.QUESTION);
                if (token === Token.COMMA || token === Token.EQUAL || token === Token.RBRACE ||
                    token === Token.RPAREN || token === Token.PIPE || token === Token.EOF ||
                    token === Token.RBRACK || token === Token.GT) {
                    return maybeAddRange({
                        type: Syntax.NullableLiteral
                    }, [rangeStart, previous]);
                }
                return maybeAddRange({
                    type: Syntax.NullableType,
                    expression: parseBasicTypeExpression(),
                    prefix: true
                }, [rangeStart, previous]);
            }
            else if (token === Token.BANG) {
                rangeStart = index - 1;
                consume(Token.BANG);
                return maybeAddRange({
                    type: Syntax.NonNullableType,
                    expression: parseBasicTypeExpression(),
                    prefix: true
                }, [rangeStart, previous]);
            }
            else {
                rangeStart = previous;
            }
            expr = parseBasicTypeExpression();
            if (token === Token.BANG) {
                consume(Token.BANG);
                return maybeAddRange({
                    type: Syntax.NonNullableType,
                    expression: expr,
                    prefix: false
                }, [rangeStart, previous]);
            }
            if (token === Token.QUESTION) {
                consume(Token.QUESTION);
                return maybeAddRange({
                    type: Syntax.NullableType,
                    expression: expr,
                    prefix: false
                }, [rangeStart, previous]);
            }
            if (token === Token.LBRACK) {
                consume(Token.LBRACK);
                expect(Token.RBRACK, 'expected an array-style type declaration (' + value + '[])');
                return maybeAddRange({
                    type: Syntax.TypeApplication,
                    expression: maybeAddRange({
                        type: Syntax.NameExpression,
                        name: 'Array'
                    }, [rangeStart, previous]),
                    applications: [expr]
                }, [rangeStart, previous]);
            }
            return expr;
        }
        // TopLevelTypeExpression :=
        //      TypeExpression
        //    | TypeUnionList
        //
        // This rule is Google Closure Compiler extension, not ES4
        // like,
        //   { number | string }
        // If strict to ES4, we should write it as
        //   { (number|string) }
        function parseTop() {
            var expr, elements;
            expr = parseTypeExpression();
            if (token !== Token.PIPE) {
                return expr;
            }
            elements = [expr];
            consume(Token.PIPE);
            while (true) {
                elements.push(parseTypeExpression());
                if (token !== Token.PIPE) {
                    break;
                }
                consume(Token.PIPE);
            }
            return maybeAddRange({
                type: Syntax.UnionType,
                elements: elements
            }, [0, index]);
        }
        function parseTopParamType() {
            var expr;
            if (token === Token.REST) {
                consume(Token.REST);
                return maybeAddRange({
                    type: Syntax.RestType,
                    expression: parseTop()
                }, [0, index]);
            }
            expr = parseTop();
            if (token === Token.EQUAL) {
                consume(Token.EQUAL);
                return maybeAddRange({
                    type: Syntax.OptionalType,
                    expression: expr
                }, [0, index]);
            }
            return expr;
        }
        function parseType(src, opt) {
            var expr;
            source = src;
            length = source.length;
            index = 0;
            previous = 0;
            addRange = opt && opt.range;
            rangeOffset = opt && opt.startIndex || 0;
            next();
            expr = parseTop();
            if (opt && opt.midstream) {
                return {
                    expression: expr,
                    index: previous
                };
            }
            if (token !== Token.EOF) {
                utility.throwError('not reach to EOF');
            }
            return expr;
        }
        function parseParamType(src, opt) {
            var expr;
            source = src;
            length = source.length;
            index = 0;
            previous = 0;
            addRange = opt && opt.range;
            rangeOffset = opt && opt.startIndex || 0;
            next();
            expr = parseTopParamType();
            if (opt && opt.midstream) {
                return {
                    expression: expr,
                    index: previous
                };
            }
            if (token !== Token.EOF) {
                utility.throwError('not reach to EOF');
            }
            return expr;
        }
        function stringifyImpl(node, compact, topLevel) {
            var result, i, iz;
            switch (node.type) {
                case Syntax.NullableLiteral:
                    result = '?';
                    break;
                case Syntax.AllLiteral:
                    result = '*';
                    break;
                case Syntax.NullLiteral:
                    result = 'null';
                    break;
                case Syntax.UndefinedLiteral:
                    result = 'undefined';
                    break;
                case Syntax.VoidLiteral:
                    result = 'void';
                    break;
                case Syntax.UnionType:
                    if (!topLevel) {
                        result = '(';
                    }
                    else {
                        result = '';
                    }
                    for (i = 0, iz = node.elements.length; i < iz; ++i) {
                        result += stringifyImpl(node.elements[i], compact);
                        if ((i + 1) !== iz) {
                            result += compact ? '|' : ' | ';
                        }
                    }
                    if (!topLevel) {
                        result += ')';
                    }
                    break;
                case Syntax.ArrayType:
                    result = '[';
                    for (i = 0, iz = node.elements.length; i < iz; ++i) {
                        result += stringifyImpl(node.elements[i], compact);
                        if ((i + 1) !== iz) {
                            result += compact ? ',' : ', ';
                        }
                    }
                    result += ']';
                    break;
                case Syntax.RecordType:
                    result = '{';
                    for (i = 0, iz = node.fields.length; i < iz; ++i) {
                        result += stringifyImpl(node.fields[i], compact);
                        if ((i + 1) !== iz) {
                            result += compact ? ',' : ', ';
                        }
                    }
                    result += '}';
                    break;
                case Syntax.FieldType:
                    if (node.value) {
                        result = node.key + (compact ? ':' : ': ') + stringifyImpl(node.value, compact);
                    }
                    else {
                        result = node.key;
                    }
                    break;
                case Syntax.FunctionType:
                    result = compact ? 'function(' : 'function (';
                    if (node['this']) {
                        if (node['new']) {
                            result += (compact ? 'new:' : 'new: ');
                        }
                        else {
                            result += (compact ? 'this:' : 'this: ');
                        }
                        result += stringifyImpl(node['this'], compact);
                        if (node.params.length !== 0) {
                            result += compact ? ',' : ', ';
                        }
                    }
                    for (i = 0, iz = node.params.length; i < iz; ++i) {
                        result += stringifyImpl(node.params[i], compact);
                        if ((i + 1) !== iz) {
                            result += compact ? ',' : ', ';
                        }
                    }
                    result += ')';
                    if (node.result) {
                        result += (compact ? ':' : ': ') + stringifyImpl(node.result, compact);
                    }
                    break;
                case Syntax.ParameterType:
                    result = node.name + (compact ? ':' : ': ') + stringifyImpl(node.expression, compact);
                    break;
                case Syntax.RestType:
                    result = '...';
                    if (node.expression) {
                        result += stringifyImpl(node.expression, compact);
                    }
                    break;
                case Syntax.NonNullableType:
                    if (node.prefix) {
                        result = '!' + stringifyImpl(node.expression, compact);
                    }
                    else {
                        result = stringifyImpl(node.expression, compact) + '!';
                    }
                    break;
                case Syntax.OptionalType:
                    result = stringifyImpl(node.expression, compact) + '=';
                    break;
                case Syntax.NullableType:
                    if (node.prefix) {
                        result = '?' + stringifyImpl(node.expression, compact);
                    }
                    else {
                        result = stringifyImpl(node.expression, compact) + '?';
                    }
                    break;
                case Syntax.NameExpression:
                    result = node.name;
                    break;
                case Syntax.TypeApplication:
                    result = stringifyImpl(node.expression, compact) + '.<';
                    for (i = 0, iz = node.applications.length; i < iz; ++i) {
                        result += stringifyImpl(node.applications[i], compact);
                        if ((i + 1) !== iz) {
                            result += compact ? ',' : ', ';
                        }
                    }
                    result += '>';
                    break;
                case Syntax.StringLiteralType:
                    result = '"' + node.value + '"';
                    break;
                case Syntax.NumericLiteralType:
                    result = String(node.value);
                    break;
                case Syntax.BooleanLiteralType:
                    result = String(node.value);
                    break;
                default:
                    utility.throwError('Unknown type ' + node.type);
            }
            return result;
        }
        function stringify(node, options) {
            if (options == null) {
                options = {};
            }
            return stringifyImpl(node, options.compact, options.topLevel);
        }
        exports.parseType = parseType;
        exports.parseParamType = parseParamType;
        exports.stringify = stringify;
        exports.Syntax = Syntax;
    }());
},
/* doctrine/lib/utility.js */ function _(require, module, exports) {
    /*
     * @fileoverview Utilities for Doctrine
     * @author Yusuke Suzuki <utatane.tea@gmail.com>
     */
    (function () {
        'use strict';
        var VERSION;
        VERSION = require(416) /* ../package.json */.version;
        exports.VERSION = VERSION;
        function DoctrineError(message) {
            this.name = 'DoctrineError';
            this.message = message;
        }
        DoctrineError.prototype = (function () {
            var Middle = function () { };
            Middle.prototype = Error.prototype;
            return new Middle();
        }());
        DoctrineError.prototype.constructor = DoctrineError;
        exports.DoctrineError = DoctrineError;
        function throwError(message) {
            throw new DoctrineError(message);
        }
        exports.throwError = throwError;
        exports.assert = require('assert');
    }());
},
/* doctrine/package.json */ function _(require, module, exports) {
    module.exports = {
        "_args": [
            [
                "doctrine@3.0.0",
                "/Users/bryan/work/bokeh/bokehjs"
            ]
        ],
        "_development": true,
        "_from": "doctrine@3.0.0",
        "_id": "doctrine@3.0.0",
        "_inBundle": false,
        "_integrity": "sha512-yS+Q5i3hBf7GBkd4KG8a7eBNNWNGLTaEwwYWUijIYM7zrlYDM0BFXHjjPWlWZ1Rg7UaddZeIDmi9jF3HmqiQ2w==",
        "_location": "/doctrine",
        "_phantomChildren": {},
        "_requested": {
            "type": "version",
            "registry": true,
            "raw": "doctrine@3.0.0",
            "name": "doctrine",
            "escapedName": "doctrine",
            "rawSpec": "3.0.0",
            "saveSpec": null,
            "fetchSpec": "3.0.0"
        },
        "_requiredBy": [
            "/eslint"
        ],
        "_resolved": "https://registry.npmjs.org/doctrine/-/doctrine-3.0.0.tgz",
        "_spec": "3.0.0",
        "_where": "/Users/bryan/work/bokeh/bokehjs",
        "bugs": {
            "url": "https://github.com/eslint/doctrine/issues"
        },
        "dependencies": {
            "esutils": "^2.0.2"
        },
        "description": "JSDoc parser",
        "devDependencies": {
            "coveralls": "^3.0.1",
            "dateformat": "^1.0.11",
            "eslint": "^1.10.3",
            "eslint-release": "^1.0.0",
            "linefix": "^0.1.1",
            "mocha": "^3.4.2",
            "npm-license": "^0.3.1",
            "nyc": "^10.3.2",
            "semver": "^5.0.3",
            "shelljs": "^0.5.3",
            "shelljs-nodecli": "^0.1.1",
            "should": "^5.0.1"
        },
        "directories": {
            "lib": "./lib"
        },
        "engines": {
            "node": ">=6.0.0"
        },
        "files": [
            "lib"
        ],
        "homepage": "https://github.com/eslint/doctrine",
        "license": "Apache-2.0",
        "main": "lib/doctrine.js",
        "maintainers": [
            {
                "name": "Nicholas C. Zakas",
                "email": "nicholas+npm@nczconsulting.com",
                "url": "https://www.nczonline.net"
            },
            {
                "name": "Yusuke Suzuki",
                "email": "utatane.tea@gmail.com",
                "url": "https://github.com/Constellation"
            }
        ],
        "name": "doctrine",
        "repository": {
            "type": "git",
            "url": "git+https://github.com/eslint/doctrine.git"
        },
        "scripts": {
            "coveralls": "nyc report --reporter=text-lcov | coveralls",
            "generate-alpharelease": "eslint-generate-prerelease alpha",
            "generate-betarelease": "eslint-generate-prerelease beta",
            "generate-rcrelease": "eslint-generate-prerelease rc",
            "generate-release": "eslint-generate-release",
            "lint": "eslint lib/",
            "pretest": "npm run lint",
            "publish-release": "eslint-publish-release",
            "test": "nyc mocha"
        },
        "version": "3.0.0"
    };
},
/* eslint/lib/rules/valid-typeof.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "problem",
            docs: {
                description: "enforce comparing `typeof` expressions against valid strings",
                category: "Possible Errors",
                recommended: true,
                url: "https://eslint.org/docs/rules/valid-typeof"
            },
            schema: [
                {
                    type: "object",
                    properties: {
                        requireStringLiterals: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            messages: {
                invalidValue: "Invalid typeof comparison value.",
                notString: "Typeof comparisons should be to string literals."
            }
        },
        create(context) {
            const VALID_TYPES = ["symbol", "undefined", "object", "boolean", "number", "string", "function", "bigint"], OPERATORS = ["==", "===", "!=", "!=="];
            const requireStringLiterals = context.options[0] && context.options[0].requireStringLiterals;
            /**
             * Determines whether a node is a typeof expression.
             * @param {ASTNode} node The node
             * @returns {boolean} `true` if the node is a typeof expression
             */
            function isTypeofExpression(node) {
                return node.type === "UnaryExpression" && node.operator === "typeof";
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                UnaryExpression(node) {
                    if (isTypeofExpression(node)) {
                        const parent = context.getAncestors().pop();
                        if (parent.type === "BinaryExpression" && OPERATORS.indexOf(parent.operator) !== -1) {
                            const sibling = parent.left === node ? parent.right : parent.left;
                            if (sibling.type === "Literal" || sibling.type === "TemplateLiteral" && !sibling.expressions.length) {
                                const value = sibling.type === "Literal" ? sibling.value : sibling.quasis[0].value.cooked;
                                if (VALID_TYPES.indexOf(value) === -1) {
                                    context.report({ node: sibling, messageId: "invalidValue" });
                                }
                            }
                            else if (requireStringLiterals && !isTypeofExpression(sibling)) {
                                context.report({ node: sibling, messageId: "notString" });
                            }
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/vars-on-top.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require `var` declarations be placed at the top of their containing scope",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/vars-on-top"
            },
            schema: [],
            messages: {
                top: "All 'var' declarations must be at the top of the function scope."
            }
        },
        create(context) {
            //--------------------------------------------------------------------------
            // Helpers
            //--------------------------------------------------------------------------
            // eslint-disable-next-line jsdoc/require-description
            /**
             * @param {ASTNode} node any node
             * @returns {boolean} whether the given node structurally represents a directive
             */
            function looksLikeDirective(node) {
                return node.type === "ExpressionStatement" &&
                    node.expression.type === "Literal" && typeof node.expression.value === "string";
            }
            /**
             * Check to see if its a ES6 import declaration
             * @param {ASTNode} node any node
             * @returns {boolean} whether the given node represents a import declaration
             */
            function looksLikeImport(node) {
                return node.type === "ImportDeclaration" || node.type === "ImportSpecifier" ||
                    node.type === "ImportDefaultSpecifier" || node.type === "ImportNamespaceSpecifier";
            }
            /**
             * Checks whether a given node is a variable declaration or not.
             * @param {ASTNode} node any node
             * @returns {boolean} `true` if the node is a variable declaration.
             */
            function isVariableDeclaration(node) {
                return (node.type === "VariableDeclaration" ||
                    (node.type === "ExportNamedDeclaration" &&
                        node.declaration &&
                        node.declaration.type === "VariableDeclaration"));
            }
            /**
             * Checks whether this variable is on top of the block body
             * @param {ASTNode} node The node to check
             * @param {ASTNode[]} statements collection of ASTNodes for the parent node block
             * @returns {boolean} True if var is on top otherwise false
             */
            function isVarOnTop(node, statements) {
                const l = statements.length;
                let i = 0;
                // skip over directives
                for (; i < l; ++i) {
                    if (!looksLikeDirective(statements[i]) && !looksLikeImport(statements[i])) {
                        break;
                    }
                }
                for (; i < l; ++i) {
                    if (!isVariableDeclaration(statements[i])) {
                        return false;
                    }
                    if (statements[i] === node) {
                        return true;
                    }
                }
                return false;
            }
            /**
             * Checks whether variable is on top at the global level
             * @param {ASTNode} node The node to check
             * @param {ASTNode} parent Parent of the node
             * @returns {void}
             */
            function globalVarCheck(node, parent) {
                if (!isVarOnTop(node, parent.body)) {
                    context.report({ node, messageId: "top" });
                }
            }
            /**
             * Checks whether variable is on top at functional block scope level
             * @param {ASTNode} node The node to check
             * @param {ASTNode} parent Parent of the node
             * @param {ASTNode} grandParent Parent of the node's parent
             * @returns {void}
             */
            function blockScopeVarCheck(node, parent, grandParent) {
                if (!(/Function/u.test(grandParent.type) &&
                    parent.type === "BlockStatement" &&
                    isVarOnTop(node, parent.body))) {
                    context.report({ node, messageId: "top" });
                }
            }
            //--------------------------------------------------------------------------
            // Public API
            //--------------------------------------------------------------------------
            return {
                "VariableDeclaration[kind='var']"(node) {
                    if (node.parent.type === "ExportNamedDeclaration") {
                        globalVarCheck(node.parent, node.parent.parent);
                    }
                    else if (node.parent.type === "Program") {
                        globalVarCheck(node, node.parent);
                    }
                    else {
                        blockScopeVarCheck(node, node.parent, node.parent.parent);
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/wrap-iife.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require parentheses around immediate `function` invocations",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/wrap-iife"
            },
            schema: [
                {
                    enum: ["outside", "inside", "any"]
                },
                {
                    type: "object",
                    properties: {
                        functionPrototypeMethods: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "code",
            messages: {
                wrapInvocation: "Wrap an immediate function invocation in parentheses.",
                wrapExpression: "Wrap only the function expression in parens.",
                moveInvocation: "Move the invocation into the parens that contain the function."
            }
        },
        create(context) {
            const style = context.options[0] || "outside";
            const includeFunctionPrototypeMethods = context.options[1] && context.options[1].functionPrototypeMethods;
            const sourceCode = context.getSourceCode();
            /**
             * Check if the node is wrapped in ()
             * @param {ASTNode} node node to evaluate
             * @returns {boolean} True if it is wrapped
             * @private
             */
            function wrapped(node) {
                return astUtils.isParenthesised(sourceCode, node);
            }
            /**
             * Get the function node from an IIFE
             * @param {ASTNode} node node to evaluate
             * @returns {ASTNode} node that is the function expression of the given IIFE, or null if none exist
             */
            function getFunctionNodeFromIIFE(node) {
                const callee = node.callee;
                if (callee.type === "FunctionExpression") {
                    return callee;
                }
                if (includeFunctionPrototypeMethods &&
                    callee.type === "MemberExpression" &&
                    callee.object.type === "FunctionExpression" &&
                    (astUtils.getStaticPropertyName(callee) === "call" || astUtils.getStaticPropertyName(callee) === "apply")) {
                    return callee.object;
                }
                return null;
            }
            return {
                CallExpression(node) {
                    const innerNode = getFunctionNodeFromIIFE(node);
                    if (!innerNode) {
                        return;
                    }
                    const callExpressionWrapped = wrapped(node), functionExpressionWrapped = wrapped(innerNode);
                    if (!callExpressionWrapped && !functionExpressionWrapped) {
                        context.report({
                            node,
                            messageId: "wrapInvocation",
                            fix(fixer) {
                                const nodeToSurround = style === "inside" ? innerNode : node;
                                return fixer.replaceText(nodeToSurround, `(${sourceCode.getText(nodeToSurround)})`);
                            }
                        });
                    }
                    else if (style === "inside" && !functionExpressionWrapped) {
                        context.report({
                            node,
                            messageId: "wrapExpression",
                            fix(fixer) {
                                /*
                                 * The outer call expression will always be wrapped at this point.
                                 * Replace the range between the end of the function expression and the end of the call expression.
                                 * for example, in `(function(foo) {}(bar))`, the range `(bar))` should get replaced with `)(bar)`.
                                 * Replace the parens from the outer expression, and parenthesize the function expression.
                                 */
                                const parenAfter = sourceCode.getTokenAfter(node);
                                return fixer.replaceTextRange([innerNode.range[1], parenAfter.range[1]], `)${sourceCode.getText().slice(innerNode.range[1], parenAfter.range[0])}`);
                            }
                        });
                    }
                    else if (style === "outside" && !callExpressionWrapped) {
                        context.report({
                            node,
                            messageId: "moveInvocation",
                            fix(fixer) {
                                /*
                                 * The inner function expression will always be wrapped at this point.
                                 * It's only necessary to replace the range between the end of the function expression
                                 * and the call expression. For example, in `(function(foo) {})(bar)`, the range `)(bar)`
                                 * should get replaced with `(bar))`.
                                 */
                                const parenAfter = sourceCode.getTokenAfter(innerNode);
                                return fixer.replaceTextRange([parenAfter.range[0], node.range[1]], `${sourceCode.getText().slice(parenAfter.range[1], node.range[1])})`);
                            }
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/wrap-regex.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require parenthesis around regex literals",
                category: "Stylistic Issues",
                recommended: false,
                url: "https://eslint.org/docs/rules/wrap-regex"
            },
            schema: [],
            fixable: "code",
            messages: {
                requireParens: "Wrap the regexp literal in parens to disambiguate the slash."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                Literal(node) {
                    const token = sourceCode.getFirstToken(node), nodeType = token.type;
                    if (nodeType === "RegularExpression") {
                        const beforeToken = sourceCode.getTokenBefore(node);
                        const afterToken = sourceCode.getTokenAfter(node);
                        const ancestors = context.getAncestors();
                        const grandparent = ancestors[ancestors.length - 1];
                        if (grandparent.type === "MemberExpression" && grandparent.object === node &&
                            !(beforeToken && beforeToken.value === "(" && afterToken && afterToken.value === ")")) {
                            context.report({
                                node,
                                messageId: "requireParens",
                                fix: fixer => fixer.replaceText(node, `(${sourceCode.getText(node)})`)
                            });
                        }
                    }
                }
            };
        }
    };
},
/* eslint/lib/rules/yield-star-spacing.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "layout",
            docs: {
                description: "require or disallow spacing around the `*` in `yield*` expressions",
                category: "ECMAScript 6",
                recommended: false,
                url: "https://eslint.org/docs/rules/yield-star-spacing"
            },
            fixable: "whitespace",
            schema: [
                {
                    oneOf: [
                        {
                            enum: ["before", "after", "both", "neither"]
                        },
                        {
                            type: "object",
                            properties: {
                                before: { type: "boolean" },
                                after: { type: "boolean" }
                            },
                            additionalProperties: false
                        }
                    ]
                }
            ],
            messages: {
                missingBefore: "Missing space before *.",
                missingAfter: "Missing space after *.",
                unexpectedBefore: "Unexpected space before *.",
                unexpectedAfter: "Unexpected space after *."
            }
        },
        create(context) {
            const sourceCode = context.getSourceCode();
            const mode = (function (option) {
                if (!option || typeof option === "string") {
                    return {
                        before: { before: true, after: false },
                        after: { before: false, after: true },
                        both: { before: true, after: true },
                        neither: { before: false, after: false }
                    }[option || "after"];
                }
                return option;
            }(context.options[0]));
            /**
             * Checks the spacing between two tokens before or after the star token.
             * @param {string} side Either "before" or "after".
             * @param {Token} leftToken `function` keyword token if side is "before", or
             *     star token if side is "after".
             * @param {Token} rightToken Star token if side is "before", or identifier
             *     token if side is "after".
             * @returns {void}
             */
            function checkSpacing(side, leftToken, rightToken) {
                if (sourceCode.isSpaceBetweenTokens(leftToken, rightToken) !== mode[side]) {
                    const after = leftToken.value === "*";
                    const spaceRequired = mode[side];
                    const node = after ? leftToken : rightToken;
                    let messageId = "";
                    if (spaceRequired) {
                        messageId = side === "before" ? "missingBefore" : "missingAfter";
                    }
                    else {
                        messageId = side === "before" ? "unexpectedBefore" : "unexpectedAfter";
                    }
                    context.report({
                        node,
                        messageId,
                        fix(fixer) {
                            if (spaceRequired) {
                                if (after) {
                                    return fixer.insertTextAfter(node, " ");
                                }
                                return fixer.insertTextBefore(node, " ");
                            }
                            return fixer.removeRange([leftToken.range[1], rightToken.range[0]]);
                        }
                    });
                }
            }
            /**
             * Enforces the spacing around the star if node is a yield* expression.
             * @param {ASTNode} node A yield expression node.
             * @returns {void}
             */
            function checkExpression(node) {
                if (!node.delegate) {
                    return;
                }
                const tokens = sourceCode.getFirstTokens(node, 3);
                const yieldToken = tokens[0];
                const starToken = tokens[1];
                const nextToken = tokens[2];
                checkSpacing("before", yieldToken, starToken);
                checkSpacing("after", starToken, nextToken);
            }
            return {
                YieldExpression: checkExpression
            };
        }
    };
},
/* eslint/lib/rules/yoda.js */ function _(require, module, exports) {
    //--------------------------------------------------------------------------
    // Requirements
    //--------------------------------------------------------------------------
    const astUtils = require(126) /* ./utils/ast-utils */;
    //--------------------------------------------------------------------------
    // Helpers
    //--------------------------------------------------------------------------
    /**
     * Determines whether an operator is a comparison operator.
     * @param {string} operator The operator to check.
     * @returns {boolean} Whether or not it is a comparison operator.
     */
    function isComparisonOperator(operator) {
        return (/^(==|===|!=|!==|<|>|<=|>=)$/u).test(operator);
    }
    /**
     * Determines whether an operator is an equality operator.
     * @param {string} operator The operator to check.
     * @returns {boolean} Whether or not it is an equality operator.
     */
    function isEqualityOperator(operator) {
        return (/^(==|===)$/u).test(operator);
    }
    /**
     * Determines whether an operator is one used in a range test.
     * Allowed operators are `<` and `<=`.
     * @param {string} operator The operator to check.
     * @returns {boolean} Whether the operator is used in range tests.
     */
    function isRangeTestOperator(operator) {
        return ["<", "<="].indexOf(operator) >= 0;
    }
    /**
     * Determines whether a non-Literal node is a negative number that should be
     * treated as if it were a single Literal node.
     * @param {ASTNode} node Node to test.
     * @returns {boolean} True if the node is a negative number that looks like a
     *                    real literal and should be treated as such.
     */
    function looksLikeLiteral(node) {
        return (node.type === "UnaryExpression" &&
            node.operator === "-" &&
            node.prefix &&
            node.argument.type === "Literal" &&
            typeof node.argument.value === "number");
    }
    /**
     * Attempts to derive a Literal node from nodes that are treated like literals.
     * @param {ASTNode} node Node to normalize.
     * @param {number} [defaultValue] The default value to be returned if the node
     *                                is not a Literal.
     * @returns {ASTNode} One of the following options.
     *  1. The original node if the node is already a Literal
     *  2. A normalized Literal node with the negative number as the value if the
     *     node represents a negative number literal.
     *  3. The Literal node which has the `defaultValue` argument if it exists.
     *  4. Otherwise `null`.
     */
    function getNormalizedLiteral(node, defaultValue) {
        if (node.type === "Literal") {
            return node;
        }
        if (looksLikeLiteral(node)) {
            return {
                type: "Literal",
                value: -node.argument.value,
                raw: `-${node.argument.value}`
            };
        }
        if (defaultValue) {
            return {
                type: "Literal",
                value: defaultValue,
                raw: String(defaultValue)
            };
        }
        return null;
    }
    /**
     * Checks whether two expressions reference the same value. For example:
     *     a = a
     *     a.b = a.b
     *     a[0] = a[0]
     *     a['b'] = a['b']
     * @param   {ASTNode} a Left side of the comparison.
     * @param   {ASTNode} b Right side of the comparison.
     * @returns {boolean}   True if both sides match and reference the same value.
     */
    function same(a, b) {
        if (a.type !== b.type) {
            return false;
        }
        switch (a.type) {
            case "Identifier":
                return a.name === b.name;
            case "Literal":
                return a.value === b.value;
            case "MemberExpression": {
                const nameA = astUtils.getStaticPropertyName(a);
                // x.y = x["y"]
                if (nameA !== null) {
                    return (same(a.object, b.object) &&
                        nameA === astUtils.getStaticPropertyName(b));
                }
                /*
                 * x[0] = x[0]
                 * x[y] = x[y]
                 * x.y = x.y
                 */
                return (a.computed === b.computed &&
                    same(a.object, b.object) &&
                    same(a.property, b.property));
            }
            case "ThisExpression":
                return true;
            default:
                return false;
        }
    }
    //------------------------------------------------------------------------------
    // Rule Definition
    //------------------------------------------------------------------------------
    module.exports = {
        meta: {
            type: "suggestion",
            docs: {
                description: "require or disallow \"Yoda\" conditions",
                category: "Best Practices",
                recommended: false,
                url: "https://eslint.org/docs/rules/yoda"
            },
            schema: [
                {
                    enum: ["always", "never"]
                },
                {
                    type: "object",
                    properties: {
                        exceptRange: {
                            type: "boolean",
                            default: false
                        },
                        onlyEquality: {
                            type: "boolean",
                            default: false
                        }
                    },
                    additionalProperties: false
                }
            ],
            fixable: "code",
            messages: {
                expected: "Expected literal to be on the {{expectedSide}} side of {{operator}}."
            }
        },
        create(context) {
            // Default to "never" (!always) if no option
            const always = (context.options[0] === "always");
            const exceptRange = (context.options[1] && context.options[1].exceptRange);
            const onlyEquality = (context.options[1] && context.options[1].onlyEquality);
            const sourceCode = context.getSourceCode();
            /**
             * Determines whether node represents a range test.
             * A range test is a "between" test like `(0 <= x && x < 1)` or an "outside"
             * test like `(x < 0 || 1 <= x)`. It must be wrapped in parentheses, and
             * both operators must be `<` or `<=`. Finally, the literal on the left side
             * must be less than or equal to the literal on the right side so that the
             * test makes any sense.
             * @param {ASTNode} node LogicalExpression node to test.
             * @returns {boolean} Whether node is a range test.
             */
            function isRangeTest(node) {
                const left = node.left, right = node.right;
                /**
                 * Determines whether node is of the form `0 <= x && x < 1`.
                 * @returns {boolean} Whether node is a "between" range test.
                 */
                function isBetweenTest() {
                    let leftLiteral, rightLiteral;
                    return (node.operator === "&&" &&
                        (leftLiteral = getNormalizedLiteral(left.left)) &&
                        (rightLiteral = getNormalizedLiteral(right.right, Number.POSITIVE_INFINITY)) &&
                        leftLiteral.value <= rightLiteral.value &&
                        same(left.right, right.left));
                }
                /**
                 * Determines whether node is of the form `x < 0 || 1 <= x`.
                 * @returns {boolean} Whether node is an "outside" range test.
                 */
                function isOutsideTest() {
                    let leftLiteral, rightLiteral;
                    return (node.operator === "||" &&
                        (leftLiteral = getNormalizedLiteral(left.right, Number.NEGATIVE_INFINITY)) &&
                        (rightLiteral = getNormalizedLiteral(right.left)) &&
                        leftLiteral.value <= rightLiteral.value &&
                        same(left.left, right.right));
                }
                /**
                 * Determines whether node is wrapped in parentheses.
                 * @returns {boolean} Whether node is preceded immediately by an open
                 *                    paren token and followed immediately by a close
                 *                    paren token.
                 */
                function isParenWrapped() {
                    return astUtils.isParenthesised(sourceCode, node);
                }
                return (node.type === "LogicalExpression" &&
                    left.type === "BinaryExpression" &&
                    right.type === "BinaryExpression" &&
                    isRangeTestOperator(left.operator) &&
                    isRangeTestOperator(right.operator) &&
                    (isBetweenTest() || isOutsideTest()) &&
                    isParenWrapped());
            }
            const OPERATOR_FLIP_MAP = {
                "===": "===",
                "!==": "!==",
                "==": "==",
                "!=": "!=",
                "<": ">",
                ">": "<",
                "<=": ">=",
                ">=": "<="
            };
            /**
             * Returns a string representation of a BinaryExpression node with its sides/operator flipped around.
             * @param {ASTNode} node The BinaryExpression node
             * @returns {string} A string representation of the node with the sides and operator flipped
             */
            function getFlippedString(node) {
                const tokenBefore = sourceCode.getTokenBefore(node);
                const operatorToken = sourceCode.getFirstTokenBetween(node.left, node.right, token => token.value === node.operator);
                const textBeforeOperator = sourceCode.getText().slice(sourceCode.getTokenBefore(operatorToken).range[1], operatorToken.range[0]);
                const textAfterOperator = sourceCode.getText().slice(operatorToken.range[1], sourceCode.getTokenAfter(operatorToken).range[0]);
                const leftText = sourceCode.getText().slice(node.range[0], sourceCode.getTokenBefore(operatorToken).range[1]);
                const firstRightToken = sourceCode.getTokenAfter(operatorToken);
                const rightText = sourceCode.getText().slice(firstRightToken.range[0], node.range[1]);
                let prefix = "";
                if (tokenBefore && tokenBefore.range[1] === node.range[0] &&
                    !astUtils.canTokensBeAdjacent(tokenBefore, firstRightToken)) {
                    prefix = " ";
                }
                return prefix + rightText + textBeforeOperator + OPERATOR_FLIP_MAP[operatorToken.value] + textAfterOperator + leftText;
            }
            //--------------------------------------------------------------------------
            // Public
            //--------------------------------------------------------------------------
            return {
                BinaryExpression(node) {
                    const expectedLiteral = always ? node.left : node.right;
                    const expectedNonLiteral = always ? node.right : node.left;
                    // If `expectedLiteral` is not a literal, and `expectedNonLiteral` is a literal, raise an error.
                    if ((expectedNonLiteral.type === "Literal" || looksLikeLiteral(expectedNonLiteral)) &&
                        !(expectedLiteral.type === "Literal" || looksLikeLiteral(expectedLiteral)) &&
                        !(!isEqualityOperator(node.operator) && onlyEquality) &&
                        isComparisonOperator(node.operator) &&
                        !(exceptRange && isRangeTest(context.getAncestors().pop()))) {
                        context.report({
                            node,
                            messageId: "expected",
                            data: {
                                operator: node.operator,
                                expectedSide: always ? "left" : "right"
                            },
                            fix: fixer => fixer.replaceText(node, getFlippedString(node))
                        });
                    }
                }
            };
        }
    };
},
/* eslint/lib/shared/ajv.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const Ajv = require(424) /* ajv */, metaSchema = require(470) /* ajv/lib/refs/json-schema-draft-04.json */;
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    module.exports = (additionalOptions = {}) => {
        const ajv = new Ajv({
            meta: false,
            useDefaults: true,
            validateSchema: false,
            missingRefs: "ignore",
            verbose: true,
            schemaId: "auto",
            ...additionalOptions
        });
        ajv.addMetaSchema(metaSchema);
        // eslint-disable-next-line no-underscore-dangle
        ajv._opts.defaultMeta = metaSchema.id;
        return ajv;
    };
},
/* ajv/lib/ajv.js */ function _(require, module, exports) {
    var compileSchema = require(425) /* ./compile */, resolve = require(426) /* ./compile/resolve */, Cache = require(436) /* ./cache */, SchemaObject = require(431) /* ./compile/schema_obj */, stableStringify = require(434) /* fast-json-stable-stringify */, formats = require(437) /* ./compile/formats */, rules = require(438) /* ./compile/rules */, $dataMetaSchema = require(463) /* ./data */, util = require(429) /* ./compile/util */;
    module.exports = Ajv;
    Ajv.prototype.validate = validate;
    Ajv.prototype.compile = compile;
    Ajv.prototype.addSchema = addSchema;
    Ajv.prototype.addMetaSchema = addMetaSchema;
    Ajv.prototype.validateSchema = validateSchema;
    Ajv.prototype.getSchema = getSchema;
    Ajv.prototype.removeSchema = removeSchema;
    Ajv.prototype.addFormat = addFormat;
    Ajv.prototype.errorsText = errorsText;
    Ajv.prototype._addSchema = _addSchema;
    Ajv.prototype._compile = _compile;
    Ajv.prototype.compileAsync = require(464) /* ./compile/async */;
    var customKeyword = require(465) /* ./keyword */;
    Ajv.prototype.addKeyword = customKeyword.add;
    Ajv.prototype.getKeyword = customKeyword.get;
    Ajv.prototype.removeKeyword = customKeyword.remove;
    Ajv.prototype.validateKeyword = customKeyword.validate;
    var errorClasses = require(433) /* ./compile/error_classes */;
    Ajv.ValidationError = errorClasses.Validation;
    Ajv.MissingRefError = errorClasses.MissingRef;
    Ajv.$dataMetaSchema = $dataMetaSchema;
    var META_SCHEMA_ID = 'http://json-schema.org/draft-07/schema';
    var META_IGNORE_OPTIONS = ['removeAdditional', 'useDefaults', 'coerceTypes', 'strictDefaults'];
    var META_SUPPORT_DATA = ['/properties'];
    /**
     * Creates validator instance.
     * Usage: `Ajv(opts)`
     * @param {Object} opts optional options
     * @return {Object} ajv instance
     */
    function Ajv(opts) {
        if (!(this instanceof Ajv))
            return new Ajv(opts);
        opts = this._opts = util.copy(opts) || {};
        setLogger(this);
        this._schemas = {};
        this._refs = {};
        this._fragments = {};
        this._formats = formats(opts.format);
        this._cache = opts.cache || new Cache;
        this._loadingSchemas = {};
        this._compilations = [];
        this.RULES = rules();
        this._getId = chooseGetId(opts);
        opts.loopRequired = opts.loopRequired || Infinity;
        if (opts.errorDataPath == 'property')
            opts._errorDataPathProperty = true;
        if (opts.serialize === undefined)
            opts.serialize = stableStringify;
        this._metaOpts = getMetaSchemaOptions(this);
        if (opts.formats)
            addInitialFormats(this);
        if (opts.keywords)
            addInitialKeywords(this);
        addDefaultMetaSchema(this);
        if (typeof opts.meta == 'object')
            this.addMetaSchema(opts.meta);
        if (opts.nullable)
            this.addKeyword('nullable', { metaSchema: { type: 'boolean' } });
        addInitialSchemas(this);
    }
    /**
     * Validate data using schema
     * Schema will be compiled and cached (using serialized JSON as key. [fast-json-stable-stringify](https://github.com/epoberezkin/fast-json-stable-stringify) is used to serialize.
     * @this   Ajv
     * @param  {String|Object} schemaKeyRef key, ref or schema object
     * @param  {Any} data to be validated
     * @return {Boolean} validation result. Errors from the last validation will be available in `ajv.errors` (and also in compiled schema: `schema.errors`).
     */
    function validate(schemaKeyRef, data) {
        var v;
        if (typeof schemaKeyRef == 'string') {
            v = this.getSchema(schemaKeyRef);
            if (!v)
                throw new Error('no schema with key or ref "' + schemaKeyRef + '"');
        }
        else {
            var schemaObj = this._addSchema(schemaKeyRef);
            v = schemaObj.validate || this._compile(schemaObj);
        }
        var valid = v(data);
        if (v.$async !== true)
            this.errors = v.errors;
        return valid;
    }
    /**
     * Create validating function for passed schema.
     * @this   Ajv
     * @param  {Object} schema schema object
     * @param  {Boolean} _meta true if schema is a meta-schema. Used internally to compile meta schemas of custom keywords.
     * @return {Function} validating function
     */
    function compile(schema, _meta) {
        var schemaObj = this._addSchema(schema, undefined, _meta);
        return schemaObj.validate || this._compile(schemaObj);
    }
    /**
     * Adds schema to the instance.
     * @this   Ajv
     * @param {Object|Array} schema schema or array of schemas. If array is passed, `key` and other parameters will be ignored.
     * @param {String} key Optional schema key. Can be passed to `validate` method instead of schema object or id/ref. One schema per instance can have empty `id` and `key`.
     * @param {Boolean} _skipValidation true to skip schema validation. Used internally, option validateSchema should be used instead.
     * @param {Boolean} _meta true if schema is a meta-schema. Used internally, addMetaSchema should be used instead.
     * @return {Ajv} this for method chaining
     */
    function addSchema(schema, key, _skipValidation, _meta) {
        if (Array.isArray(schema)) {
            for (var i = 0; i < schema.length; i++)
                this.addSchema(schema[i], undefined, _skipValidation, _meta);
            return this;
        }
        var id = this._getId(schema);
        if (id !== undefined && typeof id != 'string')
            throw new Error('schema id must be string');
        key = resolve.normalizeId(key || id);
        checkUnique(this, key);
        this._schemas[key] = this._addSchema(schema, _skipValidation, _meta, true);
        return this;
    }
    /**
     * Add schema that will be used to validate other schemas
     * options in META_IGNORE_OPTIONS are alway set to false
     * @this   Ajv
     * @param {Object} schema schema object
     * @param {String} key optional schema key
     * @param {Boolean} skipValidation true to skip schema validation, can be used to override validateSchema option for meta-schema
     * @return {Ajv} this for method chaining
     */
    function addMetaSchema(schema, key, skipValidation) {
        this.addSchema(schema, key, skipValidation, true);
        return this;
    }
    /**
     * Validate schema
     * @this   Ajv
     * @param {Object} schema schema to validate
     * @param {Boolean} throwOrLogError pass true to throw (or log) an error if invalid
     * @return {Boolean} true if schema is valid
     */
    function validateSchema(schema, throwOrLogError) {
        var $schema = schema.$schema;
        if ($schema !== undefined && typeof $schema != 'string')
            throw new Error('$schema must be a string');
        $schema = $schema || this._opts.defaultMeta || defaultMeta(this);
        if (!$schema) {
            this.logger.warn('meta-schema not available');
            this.errors = null;
            return true;
        }
        var valid = this.validate($schema, schema);
        if (!valid && throwOrLogError) {
            var message = 'schema is invalid: ' + this.errorsText();
            if (this._opts.validateSchema == 'log')
                this.logger.error(message);
            else
                throw new Error(message);
        }
        return valid;
    }
    function defaultMeta(self) {
        var meta = self._opts.meta;
        self._opts.defaultMeta = typeof meta == 'object'
            ? self._getId(meta) || meta
            : self.getSchema(META_SCHEMA_ID)
                ? META_SCHEMA_ID
                : undefined;
        return self._opts.defaultMeta;
    }
    /**
     * Get compiled schema from the instance by `key` or `ref`.
     * @this   Ajv
     * @param  {String} keyRef `key` that was passed to `addSchema` or full schema reference (`schema.id` or resolved id).
     * @return {Function} schema validating function (with property `schema`).
     */
    function getSchema(keyRef) {
        var schemaObj = _getSchemaObj(this, keyRef);
        switch (typeof schemaObj) {
            case 'object': return schemaObj.validate || this._compile(schemaObj);
            case 'string': return this.getSchema(schemaObj);
            case 'undefined': return _getSchemaFragment(this, keyRef);
        }
    }
    function _getSchemaFragment(self, ref) {
        var res = resolve.schema.call(self, { schema: {} }, ref);
        if (res) {
            var schema = res.schema, root = res.root, baseId = res.baseId;
            var v = compileSchema.call(self, schema, root, undefined, baseId);
            self._fragments[ref] = new SchemaObject({
                ref: ref,
                fragment: true,
                schema: schema,
                root: root,
                baseId: baseId,
                validate: v
            });
            return v;
        }
    }
    function _getSchemaObj(self, keyRef) {
        keyRef = resolve.normalizeId(keyRef);
        return self._schemas[keyRef] || self._refs[keyRef] || self._fragments[keyRef];
    }
    /**
     * Remove cached schema(s).
     * If no parameter is passed all schemas but meta-schemas are removed.
     * If RegExp is passed all schemas with key/id matching pattern but meta-schemas are removed.
     * Even if schema is referenced by other schemas it still can be removed as other schemas have local references.
     * @this   Ajv
     * @param  {String|Object|RegExp} schemaKeyRef key, ref, pattern to match key/ref or schema object
     * @return {Ajv} this for method chaining
     */
    function removeSchema(schemaKeyRef) {
        if (schemaKeyRef instanceof RegExp) {
            _removeAllSchemas(this, this._schemas, schemaKeyRef);
            _removeAllSchemas(this, this._refs, schemaKeyRef);
            return this;
        }
        switch (typeof schemaKeyRef) {
            case 'undefined':
                _removeAllSchemas(this, this._schemas);
                _removeAllSchemas(this, this._refs);
                this._cache.clear();
                return this;
            case 'string':
                var schemaObj = _getSchemaObj(this, schemaKeyRef);
                if (schemaObj)
                    this._cache.del(schemaObj.cacheKey);
                delete this._schemas[schemaKeyRef];
                delete this._refs[schemaKeyRef];
                return this;
            case 'object':
                var serialize = this._opts.serialize;
                var cacheKey = serialize ? serialize(schemaKeyRef) : schemaKeyRef;
                this._cache.del(cacheKey);
                var id = this._getId(schemaKeyRef);
                if (id) {
                    id = resolve.normalizeId(id);
                    delete this._schemas[id];
                    delete this._refs[id];
                }
        }
        return this;
    }
    function _removeAllSchemas(self, schemas, regex) {
        for (var keyRef in schemas) {
            var schemaObj = schemas[keyRef];
            if (!schemaObj.meta && (!regex || regex.test(keyRef))) {
                self._cache.del(schemaObj.cacheKey);
                delete schemas[keyRef];
            }
        }
    }
    /* @this   Ajv */
    function _addSchema(schema, skipValidation, meta, shouldAddSchema) {
        if (typeof schema != 'object' && typeof schema != 'boolean')
            throw new Error('schema should be object or boolean');
        var serialize = this._opts.serialize;
        var cacheKey = serialize ? serialize(schema) : schema;
        var cached = this._cache.get(cacheKey);
        if (cached)
            return cached;
        shouldAddSchema = shouldAddSchema || this._opts.addUsedSchema !== false;
        var id = resolve.normalizeId(this._getId(schema));
        if (id && shouldAddSchema)
            checkUnique(this, id);
        var willValidate = this._opts.validateSchema !== false && !skipValidation;
        var recursiveMeta;
        if (willValidate && !(recursiveMeta = id && id == resolve.normalizeId(schema.$schema)))
            this.validateSchema(schema, true);
        var localRefs = resolve.ids.call(this, schema);
        var schemaObj = new SchemaObject({
            id: id,
            schema: schema,
            localRefs: localRefs,
            cacheKey: cacheKey,
            meta: meta
        });
        if (id[0] != '#' && shouldAddSchema)
            this._refs[id] = schemaObj;
        this._cache.put(cacheKey, schemaObj);
        if (willValidate && recursiveMeta)
            this.validateSchema(schema, true);
        return schemaObj;
    }
    /* @this   Ajv */
    function _compile(schemaObj, root) {
        if (schemaObj.compiling) {
            schemaObj.validate = callValidate;
            callValidate.schema = schemaObj.schema;
            callValidate.errors = null;
            callValidate.root = root ? root : callValidate;
            if (schemaObj.schema.$async === true)
                callValidate.$async = true;
            return callValidate;
        }
        schemaObj.compiling = true;
        var currentOpts;
        if (schemaObj.meta) {
            currentOpts = this._opts;
            this._opts = this._metaOpts;
        }
        var v;
        try {
            v = compileSchema.call(this, schemaObj.schema, root, schemaObj.localRefs);
        }
        catch (e) {
            delete schemaObj.validate;
            throw e;
        }
        finally {
            schemaObj.compiling = false;
            if (schemaObj.meta)
                this._opts = currentOpts;
        }
        schemaObj.validate = v;
        schemaObj.refs = v.refs;
        schemaObj.refVal = v.refVal;
        schemaObj.root = v.root;
        return v;
        /* @this   {*} - custom context, see passContext option */
        function callValidate() {
            /* jshint validthis: true */
            var _validate = schemaObj.validate;
            var result = _validate.apply(this, arguments);
            callValidate.errors = _validate.errors;
            return result;
        }
    }
    function chooseGetId(opts) {
        switch (opts.schemaId) {
            case 'auto': return _get$IdOrId;
            case 'id': return _getId;
            default: return _get$Id;
        }
    }
    /* @this   Ajv */
    function _getId(schema) {
        if (schema.$id)
            this.logger.warn('schema $id ignored', schema.$id);
        return schema.id;
    }
    /* @this   Ajv */
    function _get$Id(schema) {
        if (schema.id)
            this.logger.warn('schema id ignored', schema.id);
        return schema.$id;
    }
    function _get$IdOrId(schema) {
        if (schema.$id && schema.id && schema.$id != schema.id)
            throw new Error('schema $id is different from id');
        return schema.$id || schema.id;
    }
    /**
     * Convert array of error message objects to string
     * @this   Ajv
     * @param  {Array<Object>} errors optional array of validation errors, if not passed errors from the instance are used.
     * @param  {Object} options optional options with properties `separator` and `dataVar`.
     * @return {String} human readable string with all errors descriptions
     */
    function errorsText(errors, options) {
        errors = errors || this.errors;
        if (!errors)
            return 'No errors';
        options = options || {};
        var separator = options.separator === undefined ? ', ' : options.separator;
        var dataVar = options.dataVar === undefined ? 'data' : options.dataVar;
        var text = '';
        for (var i = 0; i < errors.length; i++) {
            var e = errors[i];
            if (e)
                text += dataVar + e.dataPath + ' ' + e.message + separator;
        }
        return text.slice(0, -separator.length);
    }
    /**
     * Add custom format
     * @this   Ajv
     * @param {String} name format name
     * @param {String|RegExp|Function} format string is converted to RegExp; function should return boolean (true when valid)
     * @return {Ajv} this for method chaining
     */
    function addFormat(name, format) {
        if (typeof format == 'string')
            format = new RegExp(format);
        this._formats[name] = format;
        return this;
    }
    function addDefaultMetaSchema(self) {
        var $dataSchema;
        if (self._opts.$data) {
            $dataSchema = require(469) /* ./refs/data.json */;
            self.addMetaSchema($dataSchema, $dataSchema.$id, true);
        }
        if (self._opts.meta === false)
            return;
        var metaSchema = require(468) /* ./refs/json-schema-draft-07.json */;
        if (self._opts.$data)
            metaSchema = $dataMetaSchema(metaSchema, META_SUPPORT_DATA);
        self.addMetaSchema(metaSchema, META_SCHEMA_ID, true);
        self._refs['http://json-schema.org/schema'] = META_SCHEMA_ID;
    }
    function addInitialSchemas(self) {
        var optsSchemas = self._opts.schemas;
        if (!optsSchemas)
            return;
        if (Array.isArray(optsSchemas))
            self.addSchema(optsSchemas);
        else
            for (var key in optsSchemas)
                self.addSchema(optsSchemas[key], key);
    }
    function addInitialFormats(self) {
        for (var name in self._opts.formats) {
            var format = self._opts.formats[name];
            self.addFormat(name, format);
        }
    }
    function addInitialKeywords(self) {
        for (var name in self._opts.keywords) {
            var keyword = self._opts.keywords[name];
            self.addKeyword(name, keyword);
        }
    }
    function checkUnique(self, id) {
        if (self._schemas[id] || self._refs[id])
            throw new Error('schema with key or id "' + id + '" already exists');
    }
    function getMetaSchemaOptions(self) {
        var metaOpts = util.copy(self._opts);
        for (var i = 0; i < META_IGNORE_OPTIONS.length; i++)
            delete metaOpts[META_IGNORE_OPTIONS[i]];
        return metaOpts;
    }
    function setLogger(self) {
        var logger = self._opts.logger;
        if (logger === false) {
            self.logger = { log: noop, warn: noop, error: noop };
        }
        else {
            if (logger === undefined)
                logger = console;
            if (!(typeof logger == 'object' && logger.log && logger.warn && logger.error))
                throw new Error('logger must implement log, warn and error methods');
            self.logger = logger;
        }
    }
    function noop() { }
},
/* ajv/lib/compile/index.js */ function _(require, module, exports) {
    var resolve = require(426) /* ./resolve */, util = require(429) /* ./util */, errorClasses = require(433) /* ./error_classes */, stableStringify = require(434) /* fast-json-stable-stringify */;
    var validateGenerator = require(435) /* ../dotjs/validate */;
    /**
     * Functions below are used inside compiled validations function
     */
    var ucs2length = util.ucs2length;
    var equal = require(428) /* fast-deep-equal */;
    // this error is thrown by async schemas to return validation errors via exception
    var ValidationError = errorClasses.Validation;
    module.exports = compile;
    /**
     * Compiles schema to validation function
     * @this   Ajv
     * @param  {Object} schema schema object
     * @param  {Object} root object with information about the root schema for this schema
     * @param  {Object} localRefs the hash of local references inside the schema (created by resolve.id), used for inline resolution
     * @param  {String} baseId base ID for IDs in the schema
     * @return {Function} validation function
     */
    function compile(schema, root, localRefs, baseId) {
        /* jshint validthis: true, evil: true */
        /* eslint no-shadow: 0 */
        var self = this, opts = this._opts, refVal = [undefined], refs = {}, patterns = [], patternsHash = {}, defaults = [], defaultsHash = {}, customRules = [];
        root = root || { schema: schema, refVal: refVal, refs: refs };
        var c = checkCompiling.call(this, schema, root, baseId);
        var compilation = this._compilations[c.index];
        if (c.compiling)
            return (compilation.callValidate = callValidate);
        var formats = this._formats;
        var RULES = this.RULES;
        try {
            var v = localCompile(schema, root, localRefs, baseId);
            compilation.validate = v;
            var cv = compilation.callValidate;
            if (cv) {
                cv.schema = v.schema;
                cv.errors = null;
                cv.refs = v.refs;
                cv.refVal = v.refVal;
                cv.root = v.root;
                cv.$async = v.$async;
                if (opts.sourceCode)
                    cv.source = v.source;
            }
            return v;
        }
        finally {
            endCompiling.call(this, schema, root, baseId);
        }
        /* @this   {*} - custom context, see passContext option */
        function callValidate() {
            /* jshint validthis: true */
            var validate = compilation.validate;
            var result = validate.apply(this, arguments);
            callValidate.errors = validate.errors;
            return result;
        }
        function localCompile(_schema, _root, localRefs, baseId) {
            var isRoot = !_root || (_root && _root.schema == _schema);
            if (_root.schema != root.schema)
                return compile.call(self, _schema, _root, localRefs, baseId);
            var $async = _schema.$async === true;
            var sourceCode = validateGenerator({
                isTop: true,
                schema: _schema,
                isRoot: isRoot,
                baseId: baseId,
                root: _root,
                schemaPath: '',
                errSchemaPath: '#',
                errorPath: '""',
                MissingRefError: errorClasses.MissingRef,
                RULES: RULES,
                validate: validateGenerator,
                util: util,
                resolve: resolve,
                resolveRef: resolveRef,
                usePattern: usePattern,
                useDefault: useDefault,
                useCustomRule: useCustomRule,
                opts: opts,
                formats: formats,
                logger: self.logger,
                self: self
            });
            sourceCode = vars(refVal, refValCode) + vars(patterns, patternCode)
                + vars(defaults, defaultCode) + vars(customRules, customRuleCode)
                + sourceCode;
            if (opts.processCode)
                sourceCode = opts.processCode(sourceCode);
            // console.log('\n\n\n *** \n', JSON.stringify(sourceCode));
            var validate;
            try {
                var makeValidate = new Function('self', 'RULES', 'formats', 'root', 'refVal', 'defaults', 'customRules', 'equal', 'ucs2length', 'ValidationError', sourceCode);
                validate = makeValidate(self, RULES, formats, root, refVal, defaults, customRules, equal, ucs2length, ValidationError);
                refVal[0] = validate;
            }
            catch (e) {
                self.logger.error('Error compiling schema, function code:', sourceCode);
                throw e;
            }
            validate.schema = _schema;
            validate.errors = null;
            validate.refs = refs;
            validate.refVal = refVal;
            validate.root = isRoot ? validate : _root;
            if ($async)
                validate.$async = true;
            if (opts.sourceCode === true) {
                validate.source = {
                    code: sourceCode,
                    patterns: patterns,
                    defaults: defaults
                };
            }
            return validate;
        }
        function resolveRef(baseId, ref, isRoot) {
            ref = resolve.url(baseId, ref);
            var refIndex = refs[ref];
            var _refVal, refCode;
            if (refIndex !== undefined) {
                _refVal = refVal[refIndex];
                refCode = 'refVal[' + refIndex + ']';
                return resolvedRef(_refVal, refCode);
            }
            if (!isRoot && root.refs) {
                var rootRefId = root.refs[ref];
                if (rootRefId !== undefined) {
                    _refVal = root.refVal[rootRefId];
                    refCode = addLocalRef(ref, _refVal);
                    return resolvedRef(_refVal, refCode);
                }
            }
            refCode = addLocalRef(ref);
            var v = resolve.call(self, localCompile, root, ref);
            if (v === undefined) {
                var localSchema = localRefs && localRefs[ref];
                if (localSchema) {
                    v = resolve.inlineRef(localSchema, opts.inlineRefs)
                        ? localSchema
                        : compile.call(self, localSchema, root, localRefs, baseId);
                }
            }
            if (v === undefined) {
                removeLocalRef(ref);
            }
            else {
                replaceLocalRef(ref, v);
                return resolvedRef(v, refCode);
            }
        }
        function addLocalRef(ref, v) {
            var refId = refVal.length;
            refVal[refId] = v;
            refs[ref] = refId;
            return 'refVal' + refId;
        }
        function removeLocalRef(ref) {
            delete refs[ref];
        }
        function replaceLocalRef(ref, v) {
            var refId = refs[ref];
            refVal[refId] = v;
        }
        function resolvedRef(refVal, code) {
            return typeof refVal == 'object' || typeof refVal == 'boolean'
                ? { code: code, schema: refVal, inline: true }
                : { code: code, $async: refVal && !!refVal.$async };
        }
        function usePattern(regexStr) {
            var index = patternsHash[regexStr];
            if (index === undefined) {
                index = patternsHash[regexStr] = patterns.length;
                patterns[index] = regexStr;
            }
            return 'pattern' + index;
        }
        function useDefault(value) {
            switch (typeof value) {
                case 'boolean':
                case 'number':
                    return '' + value;
                case 'string':
                    return util.toQuotedString(value);
                case 'object':
                    if (value === null)
                        return 'null';
                    var valueStr = stableStringify(value);
                    var index = defaultsHash[valueStr];
                    if (index === undefined) {
                        index = defaultsHash[valueStr] = defaults.length;
                        defaults[index] = value;
                    }
                    return 'default' + index;
            }
        }
        function useCustomRule(rule, schema, parentSchema, it) {
            if (self._opts.validateSchema !== false) {
                var deps = rule.definition.dependencies;
                if (deps && !deps.every(function (keyword) {
                    return Object.prototype.hasOwnProperty.call(parentSchema, keyword);
                }))
                    throw new Error('parent schema must have all required keywords: ' + deps.join(','));
                var validateSchema = rule.definition.validateSchema;
                if (validateSchema) {
                    var valid = validateSchema(schema);
                    if (!valid) {
                        var message = 'keyword schema is invalid: ' + self.errorsText(validateSchema.errors);
                        if (self._opts.validateSchema == 'log')
                            self.logger.error(message);
                        else
                            throw new Error(message);
                    }
                }
            }
            var compile = rule.definition.compile, inline = rule.definition.inline, macro = rule.definition.macro;
            var validate;
            if (compile) {
                validate = compile.call(self, schema, parentSchema, it);
            }
            else if (macro) {
                validate = macro.call(self, schema, parentSchema, it);
                if (opts.validateSchema !== false)
                    self.validateSchema(validate, true);
            }
            else if (inline) {
                validate = inline.call(self, it, rule.keyword, schema, parentSchema);
            }
            else {
                validate = rule.definition.validate;
                if (!validate)
                    return;
            }
            if (validate === undefined)
                throw new Error('custom keyword "' + rule.keyword + '"failed to compile');
            var index = customRules.length;
            customRules[index] = validate;
            return {
                code: 'customRule' + index,
                validate: validate
            };
        }
    }
    /**
     * Checks if the schema is currently compiled
     * @this   Ajv
     * @param  {Object} schema schema to compile
     * @param  {Object} root root object
     * @param  {String} baseId base schema ID
     * @return {Object} object with properties "index" (compilation index) and "compiling" (boolean)
     */
    function checkCompiling(schema, root, baseId) {
        /* jshint validthis: true */
        var index = compIndex.call(this, schema, root, baseId);
        if (index >= 0)
            return { index: index, compiling: true };
        index = this._compilations.length;
        this._compilations[index] = {
            schema: schema,
            root: root,
            baseId: baseId
        };
        return { index: index, compiling: false };
    }
    /**
     * Removes the schema from the currently compiled list
     * @this   Ajv
     * @param  {Object} schema schema to compile
     * @param  {Object} root root object
     * @param  {String} baseId base schema ID
     */
    function endCompiling(schema, root, baseId) {
        /* jshint validthis: true */
        var i = compIndex.call(this, schema, root, baseId);
        if (i >= 0)
            this._compilations.splice(i, 1);
    }
    /**
     * Index of schema compilation in the currently compiled list
     * @this   Ajv
     * @param  {Object} schema schema to compile
     * @param  {Object} root root object
     * @param  {String} baseId base schema ID
     * @return {Integer} compilation index
     */
    function compIndex(schema, root, baseId) {
        /* jshint validthis: true */
        for (var i = 0; i < this._compilations.length; i++) {
            var c = this._compilations[i];
            if (c.schema == schema && c.root == root && c.baseId == baseId)
                return i;
        }
        return -1;
    }
    function patternCode(i, patterns) {
        return 'var pattern' + i + ' = new RegExp(' + util.toQuotedString(patterns[i]) + ');';
    }
    function defaultCode(i) {
        return 'var default' + i + ' = defaults[' + i + '];';
    }
    function refValCode(i, refVal) {
        return refVal[i] === undefined ? '' : 'var refVal' + i + ' = refVal[' + i + '];';
    }
    function customRuleCode(i) {
        return 'var customRule' + i + ' = customRules[' + i + '];';
    }
    function vars(arr, statement) {
        if (!arr.length)
            return '';
        var code = '';
        for (var i = 0; i < arr.length; i++)
            code += statement(i, arr);
        return code;
    }
},
/* ajv/lib/compile/resolve.js */ function _(require, module, exports) {
    var URI = require(427) /* uri-js */, equal = require(428) /* fast-deep-equal */, util = require(429) /* ./util */, SchemaObject = require(431) /* ./schema_obj */, traverse = require(432) /* json-schema-traverse */;
    module.exports = resolve;
    resolve.normalizeId = normalizeId;
    resolve.fullPath = getFullPath;
    resolve.url = resolveUrl;
    resolve.ids = resolveIds;
    resolve.inlineRef = inlineRef;
    resolve.schema = resolveSchema;
    /**
     * [resolve and compile the references ($ref)]
     * @this   Ajv
     * @param  {Function} compile reference to schema compilation funciton (localCompile)
     * @param  {Object} root object with information about the root schema for the current schema
     * @param  {String} ref reference to resolve
     * @return {Object|Function} schema object (if the schema can be inlined) or validation function
     */
    function resolve(compile, root, ref) {
        /* jshint validthis: true */
        var refVal = this._refs[ref];
        if (typeof refVal == 'string') {
            if (this._refs[refVal])
                refVal = this._refs[refVal];
            else
                return resolve.call(this, compile, root, refVal);
        }
        refVal = refVal || this._schemas[ref];
        if (refVal instanceof SchemaObject) {
            return inlineRef(refVal.schema, this._opts.inlineRefs)
                ? refVal.schema
                : refVal.validate || this._compile(refVal);
        }
        var res = resolveSchema.call(this, root, ref);
        var schema, v, baseId;
        if (res) {
            schema = res.schema;
            root = res.root;
            baseId = res.baseId;
        }
        if (schema instanceof SchemaObject) {
            v = schema.validate || compile.call(this, schema.schema, root, undefined, baseId);
        }
        else if (schema !== undefined) {
            v = inlineRef(schema, this._opts.inlineRefs)
                ? schema
                : compile.call(this, schema, root, undefined, baseId);
        }
        return v;
    }
    /**
     * Resolve schema, its root and baseId
     * @this Ajv
     * @param  {Object} root root object with properties schema, refVal, refs
     * @param  {String} ref  reference to resolve
     * @return {Object} object with properties schema, root, baseId
     */
    function resolveSchema(root, ref) {
        /* jshint validthis: true */
        var p = URI.parse(ref), refPath = _getFullPath(p), baseId = getFullPath(this._getId(root.schema));
        if (Object.keys(root.schema).length === 0 || refPath !== baseId) {
            var id = normalizeId(refPath);
            var refVal = this._refs[id];
            if (typeof refVal == 'string') {
                return resolveRecursive.call(this, root, refVal, p);
            }
            else if (refVal instanceof SchemaObject) {
                if (!refVal.validate)
                    this._compile(refVal);
                root = refVal;
            }
            else {
                refVal = this._schemas[id];
                if (refVal instanceof SchemaObject) {
                    if (!refVal.validate)
                        this._compile(refVal);
                    if (id == normalizeId(ref))
                        return { schema: refVal, root: root, baseId: baseId };
                    root = refVal;
                }
                else {
                    return;
                }
            }
            if (!root.schema)
                return;
            baseId = getFullPath(this._getId(root.schema));
        }
        return getJsonPointer.call(this, p, baseId, root.schema, root);
    }
    /* @this Ajv */
    function resolveRecursive(root, ref, parsedRef) {
        /* jshint validthis: true */
        var res = resolveSchema.call(this, root, ref);
        if (res) {
            var schema = res.schema;
            var baseId = res.baseId;
            root = res.root;
            var id = this._getId(schema);
            if (id)
                baseId = resolveUrl(baseId, id);
            return getJsonPointer.call(this, parsedRef, baseId, schema, root);
        }
    }
    var PREVENT_SCOPE_CHANGE = util.toHash(['properties', 'patternProperties', 'enum', 'dependencies', 'definitions']);
    /* @this Ajv */
    function getJsonPointer(parsedRef, baseId, schema, root) {
        /* jshint validthis: true */
        parsedRef.fragment = parsedRef.fragment || '';
        if (parsedRef.fragment.slice(0, 1) != '/')
            return;
        var parts = parsedRef.fragment.split('/');
        for (var i = 1; i < parts.length; i++) {
            var part = parts[i];
            if (part) {
                part = util.unescapeFragment(part);
                schema = schema[part];
                if (schema === undefined)
                    break;
                var id;
                if (!PREVENT_SCOPE_CHANGE[part]) {
                    id = this._getId(schema);
                    if (id)
                        baseId = resolveUrl(baseId, id);
                    if (schema.$ref) {
                        var $ref = resolveUrl(baseId, schema.$ref);
                        var res = resolveSchema.call(this, root, $ref);
                        if (res) {
                            schema = res.schema;
                            root = res.root;
                            baseId = res.baseId;
                        }
                    }
                }
            }
        }
        if (schema !== undefined && schema !== root.schema)
            return { schema: schema, root: root, baseId: baseId };
    }
    var SIMPLE_INLINED = util.toHash([
        'type', 'format', 'pattern',
        'maxLength', 'minLength',
        'maxProperties', 'minProperties',
        'maxItems', 'minItems',
        'maximum', 'minimum',
        'uniqueItems', 'multipleOf',
        'required', 'enum'
    ]);
    function inlineRef(schema, limit) {
        if (limit === false)
            return false;
        if (limit === undefined || limit === true)
            return checkNoRef(schema);
        else if (limit)
            return countKeys(schema) <= limit;
    }
    function checkNoRef(schema) {
        var item;
        if (Array.isArray(schema)) {
            for (var i = 0; i < schema.length; i++) {
                item = schema[i];
                if (typeof item == 'object' && !checkNoRef(item))
                    return false;
            }
        }
        else {
            for (var key in schema) {
                if (key == '$ref')
                    return false;
                item = schema[key];
                if (typeof item == 'object' && !checkNoRef(item))
                    return false;
            }
        }
        return true;
    }
    function countKeys(schema) {
        var count = 0, item;
        if (Array.isArray(schema)) {
            for (var i = 0; i < schema.length; i++) {
                item = schema[i];
                if (typeof item == 'object')
                    count += countKeys(item);
                if (count == Infinity)
                    return Infinity;
            }
        }
        else {
            for (var key in schema) {
                if (key == '$ref')
                    return Infinity;
                if (SIMPLE_INLINED[key]) {
                    count++;
                }
                else {
                    item = schema[key];
                    if (typeof item == 'object')
                        count += countKeys(item) + 1;
                    if (count == Infinity)
                        return Infinity;
                }
            }
        }
        return count;
    }
    function getFullPath(id, normalize) {
        if (normalize !== false)
            id = normalizeId(id);
        var p = URI.parse(id);
        return _getFullPath(p);
    }
    function _getFullPath(p) {
        return URI.serialize(p).split('#')[0] + '#';
    }
    var TRAILING_SLASH_HASH = /#\/?$/;
    function normalizeId(id) {
        return id ? id.replace(TRAILING_SLASH_HASH, '') : '';
    }
    function resolveUrl(baseId, id) {
        id = normalizeId(id);
        return URI.resolve(baseId, id);
    }
    /* @this Ajv */
    function resolveIds(schema) {
        var schemaId = normalizeId(this._getId(schema));
        var baseIds = { '': schemaId };
        var fullPaths = { '': getFullPath(schemaId, false) };
        var localRefs = {};
        var self = this;
        traverse(schema, { allKeys: true }, function (sch, jsonPtr, rootSchema, parentJsonPtr, parentKeyword, parentSchema, keyIndex) {
            if (jsonPtr === '')
                return;
            var id = self._getId(sch);
            var baseId = baseIds[parentJsonPtr];
            var fullPath = fullPaths[parentJsonPtr] + '/' + parentKeyword;
            if (keyIndex !== undefined)
                fullPath += '/' + (typeof keyIndex == 'number' ? keyIndex : util.escapeFragment(keyIndex));
            if (typeof id == 'string') {
                id = baseId = normalizeId(baseId ? URI.resolve(baseId, id) : id);
                var refVal = self._refs[id];
                if (typeof refVal == 'string')
                    refVal = self._refs[refVal];
                if (refVal && refVal.schema) {
                    if (!equal(sch, refVal.schema))
                        throw new Error('id "' + id + '" resolves to more than one schema');
                }
                else if (id != normalizeId(fullPath)) {
                    if (id[0] == '#') {
                        if (localRefs[id] && !equal(sch, localRefs[id]))
                            throw new Error('id "' + id + '" resolves to more than one schema');
                        localRefs[id] = sch;
                    }
                    else {
                        self._refs[id] = fullPath;
                    }
                }
            }
            baseIds[jsonPtr] = baseId;
            fullPaths[jsonPtr] = fullPath;
        });
        return localRefs;
    }
},
/* uri-js/dist/es5/uri.all.js */ function _(require, module, exports) {
    /** @license URI.js v4.2.1 (c) 2011 Gary Court. License: http://github.com/garycourt/uri-js */
    (function (global, factory) {
        typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
            typeof define === 'function' && define.amd ? define(['exports'], factory) :
                (factory((global.URI = global.URI || {})));
    }(this, (function (exports) {
        'use strict';
        function merge() {
            for (var _len = arguments.length, sets = Array(_len), _key = 0; _key < _len; _key++) {
                sets[_key] = arguments[_key];
            }
            if (sets.length > 1) {
                sets[0] = sets[0].slice(0, -1);
                var xl = sets.length - 1;
                for (var x = 1; x < xl; ++x) {
                    sets[x] = sets[x].slice(1, -1);
                }
                sets[xl] = sets[xl].slice(1);
                return sets.join('');
            }
            else {
                return sets[0];
            }
        }
        function subexp(str) {
            return "(?:" + str + ")";
        }
        function typeOf(o) {
            return o === undefined ? "undefined" : o === null ? "null" : Object.prototype.toString.call(o).split(" ").pop().split("]").shift().toLowerCase();
        }
        function toUpperCase(str) {
            return str.toUpperCase();
        }
        function toArray(obj) {
            return obj !== undefined && obj !== null ? obj instanceof Array ? obj : typeof obj.length !== "number" || obj.split || obj.setInterval || obj.call ? [obj] : Array.prototype.slice.call(obj) : [];
        }
        function assign(target, source) {
            var obj = target;
            if (source) {
                for (var key in source) {
                    obj[key] = source[key];
                }
            }
            return obj;
        }
        function buildExps(isIRI) {
            var ALPHA$$ = "[A-Za-z]", CR$ = "[\\x0D]", DIGIT$$ = "[0-9]", DQUOTE$$ = "[\\x22]", HEXDIG$$ = merge(DIGIT$$, "[A-Fa-f]"), 
            //case-insensitive
            LF$$ = "[\\x0A]", SP$$ = "[\\x20]", PCT_ENCODED$ = subexp(subexp("%[EFef]" + HEXDIG$$ + "%" + HEXDIG$$ + HEXDIG$$ + "%" + HEXDIG$$ + HEXDIG$$) + "|" + subexp("%[89A-Fa-f]" + HEXDIG$$ + "%" + HEXDIG$$ + HEXDIG$$) + "|" + subexp("%" + HEXDIG$$ + HEXDIG$$)), 
            //expanded
            GEN_DELIMS$$ = "[\\:\\/\\?\\#\\[\\]\\@]", SUB_DELIMS$$ = "[\\!\\$\\&\\'\\(\\)\\*\\+\\,\\;\\=]", RESERVED$$ = merge(GEN_DELIMS$$, SUB_DELIMS$$), UCSCHAR$$ = isIRI ? "[\\xA0-\\u200D\\u2010-\\u2029\\u202F-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFEF]" : "[]", 
            //subset, excludes bidi control characters
            IPRIVATE$$ = isIRI ? "[\\uE000-\\uF8FF]" : "[]", 
            //subset
            UNRESERVED$$ = merge(ALPHA$$, DIGIT$$, "[\\-\\.\\_\\~]", UCSCHAR$$), SCHEME$ = subexp(ALPHA$$ + merge(ALPHA$$, DIGIT$$, "[\\+\\-\\.]") + "*"), USERINFO$ = subexp(subexp(PCT_ENCODED$ + "|" + merge(UNRESERVED$$, SUB_DELIMS$$, "[\\:]")) + "*"), DEC_OCTET$ = subexp(subexp("25[0-5]") + "|" + subexp("2[0-4]" + DIGIT$$) + "|" + subexp("1" + DIGIT$$ + DIGIT$$) + "|" + subexp("[1-9]" + DIGIT$$) + "|" + DIGIT$$), DEC_OCTET_RELAXED$ = subexp(subexp("25[0-5]") + "|" + subexp("2[0-4]" + DIGIT$$) + "|" + subexp("1" + DIGIT$$ + DIGIT$$) + "|" + subexp("0?[1-9]" + DIGIT$$) + "|0?0?" + DIGIT$$), 
            //relaxed parsing rules
            IPV4ADDRESS$ = subexp(DEC_OCTET_RELAXED$ + "\\." + DEC_OCTET_RELAXED$ + "\\." + DEC_OCTET_RELAXED$ + "\\." + DEC_OCTET_RELAXED$), H16$ = subexp(HEXDIG$$ + "{1,4}"), LS32$ = subexp(subexp(H16$ + "\\:" + H16$) + "|" + IPV4ADDRESS$), IPV6ADDRESS1$ = subexp(subexp(H16$ + "\\:") + "{6}" + LS32$), 
            //                           6( h16 ":" ) ls32
            IPV6ADDRESS2$ = subexp("\\:\\:" + subexp(H16$ + "\\:") + "{5}" + LS32$), 
            //                      "::" 5( h16 ":" ) ls32
            IPV6ADDRESS3$ = subexp(subexp(H16$) + "?\\:\\:" + subexp(H16$ + "\\:") + "{4}" + LS32$), 
            //[               h16 ] "::" 4( h16 ":" ) ls32
            IPV6ADDRESS4$ = subexp(subexp(subexp(H16$ + "\\:") + "{0,1}" + H16$) + "?\\:\\:" + subexp(H16$ + "\\:") + "{3}" + LS32$), 
            //[ *1( h16 ":" ) h16 ] "::" 3( h16 ":" ) ls32
            IPV6ADDRESS5$ = subexp(subexp(subexp(H16$ + "\\:") + "{0,2}" + H16$) + "?\\:\\:" + subexp(H16$ + "\\:") + "{2}" + LS32$), 
            //[ *2( h16 ":" ) h16 ] "::" 2( h16 ":" ) ls32
            IPV6ADDRESS6$ = subexp(subexp(subexp(H16$ + "\\:") + "{0,3}" + H16$) + "?\\:\\:" + H16$ + "\\:" + LS32$), 
            //[ *3( h16 ":" ) h16 ] "::"    h16 ":"   ls32
            IPV6ADDRESS7$ = subexp(subexp(subexp(H16$ + "\\:") + "{0,4}" + H16$) + "?\\:\\:" + LS32$), 
            //[ *4( h16 ":" ) h16 ] "::"              ls32
            IPV6ADDRESS8$ = subexp(subexp(subexp(H16$ + "\\:") + "{0,5}" + H16$) + "?\\:\\:" + H16$), 
            //[ *5( h16 ":" ) h16 ] "::"              h16
            IPV6ADDRESS9$ = subexp(subexp(subexp(H16$ + "\\:") + "{0,6}" + H16$) + "?\\:\\:"), 
            //[ *6( h16 ":" ) h16 ] "::"
            IPV6ADDRESS$ = subexp([IPV6ADDRESS1$, IPV6ADDRESS2$, IPV6ADDRESS3$, IPV6ADDRESS4$, IPV6ADDRESS5$, IPV6ADDRESS6$, IPV6ADDRESS7$, IPV6ADDRESS8$, IPV6ADDRESS9$].join("|")), ZONEID$ = subexp(subexp(UNRESERVED$$ + "|" + PCT_ENCODED$) + "+"), 
            //RFC 6874
            IPV6ADDRZ$ = subexp(IPV6ADDRESS$ + "\\%25" + ZONEID$), 
            //RFC 6874
            IPV6ADDRZ_RELAXED$ = subexp(IPV6ADDRESS$ + subexp("\\%25|\\%(?!" + HEXDIG$$ + "{2})") + ZONEID$), 
            //RFC 6874, with relaxed parsing rules
            IPVFUTURE$ = subexp("[vV]" + HEXDIG$$ + "+\\." + merge(UNRESERVED$$, SUB_DELIMS$$, "[\\:]") + "+"), IP_LITERAL$ = subexp("\\[" + subexp(IPV6ADDRZ_RELAXED$ + "|" + IPV6ADDRESS$ + "|" + IPVFUTURE$) + "\\]"), 
            //RFC 6874
            REG_NAME$ = subexp(subexp(PCT_ENCODED$ + "|" + merge(UNRESERVED$$, SUB_DELIMS$$)) + "*"), HOST$ = subexp(IP_LITERAL$ + "|" + IPV4ADDRESS$ + "(?!" + REG_NAME$ + ")" + "|" + REG_NAME$), PORT$ = subexp(DIGIT$$ + "*"), AUTHORITY$ = subexp(subexp(USERINFO$ + "@") + "?" + HOST$ + subexp("\\:" + PORT$) + "?"), PCHAR$ = subexp(PCT_ENCODED$ + "|" + merge(UNRESERVED$$, SUB_DELIMS$$, "[\\:\\@]")), SEGMENT$ = subexp(PCHAR$ + "*"), SEGMENT_NZ$ = subexp(PCHAR$ + "+"), SEGMENT_NZ_NC$ = subexp(subexp(PCT_ENCODED$ + "|" + merge(UNRESERVED$$, SUB_DELIMS$$, "[\\@]")) + "+"), PATH_ABEMPTY$ = subexp(subexp("\\/" + SEGMENT$) + "*"), PATH_ABSOLUTE$ = subexp("\\/" + subexp(SEGMENT_NZ$ + PATH_ABEMPTY$) + "?"), 
            //simplified
            PATH_NOSCHEME$ = subexp(SEGMENT_NZ_NC$ + PATH_ABEMPTY$), 
            //simplified
            PATH_ROOTLESS$ = subexp(SEGMENT_NZ$ + PATH_ABEMPTY$), 
            //simplified
            PATH_EMPTY$ = "(?!" + PCHAR$ + ")", PATH$ = subexp(PATH_ABEMPTY$ + "|" + PATH_ABSOLUTE$ + "|" + PATH_NOSCHEME$ + "|" + PATH_ROOTLESS$ + "|" + PATH_EMPTY$), QUERY$ = subexp(subexp(PCHAR$ + "|" + merge("[\\/\\?]", IPRIVATE$$)) + "*"), FRAGMENT$ = subexp(subexp(PCHAR$ + "|[\\/\\?]") + "*"), HIER_PART$ = subexp(subexp("\\/\\/" + AUTHORITY$ + PATH_ABEMPTY$) + "|" + PATH_ABSOLUTE$ + "|" + PATH_ROOTLESS$ + "|" + PATH_EMPTY$), URI$ = subexp(SCHEME$ + "\\:" + HIER_PART$ + subexp("\\?" + QUERY$) + "?" + subexp("\\#" + FRAGMENT$) + "?"), RELATIVE_PART$ = subexp(subexp("\\/\\/" + AUTHORITY$ + PATH_ABEMPTY$) + "|" + PATH_ABSOLUTE$ + "|" + PATH_NOSCHEME$ + "|" + PATH_EMPTY$), RELATIVE$ = subexp(RELATIVE_PART$ + subexp("\\?" + QUERY$) + "?" + subexp("\\#" + FRAGMENT$) + "?"), URI_REFERENCE$ = subexp(URI$ + "|" + RELATIVE$), ABSOLUTE_URI$ = subexp(SCHEME$ + "\\:" + HIER_PART$ + subexp("\\?" + QUERY$) + "?"), GENERIC_REF$ = "^(" + SCHEME$ + ")\\:" + subexp(subexp("\\/\\/(" + subexp("(" + USERINFO$ + ")@") + "?(" + HOST$ + ")" + subexp("\\:(" + PORT$ + ")") + "?)") + "?(" + PATH_ABEMPTY$ + "|" + PATH_ABSOLUTE$ + "|" + PATH_ROOTLESS$ + "|" + PATH_EMPTY$ + ")") + subexp("\\?(" + QUERY$ + ")") + "?" + subexp("\\#(" + FRAGMENT$ + ")") + "?$", RELATIVE_REF$ = "^(){0}" + subexp(subexp("\\/\\/(" + subexp("(" + USERINFO$ + ")@") + "?(" + HOST$ + ")" + subexp("\\:(" + PORT$ + ")") + "?)") + "?(" + PATH_ABEMPTY$ + "|" + PATH_ABSOLUTE$ + "|" + PATH_NOSCHEME$ + "|" + PATH_EMPTY$ + ")") + subexp("\\?(" + QUERY$ + ")") + "?" + subexp("\\#(" + FRAGMENT$ + ")") + "?$", ABSOLUTE_REF$ = "^(" + SCHEME$ + ")\\:" + subexp(subexp("\\/\\/(" + subexp("(" + USERINFO$ + ")@") + "?(" + HOST$ + ")" + subexp("\\:(" + PORT$ + ")") + "?)") + "?(" + PATH_ABEMPTY$ + "|" + PATH_ABSOLUTE$ + "|" + PATH_ROOTLESS$ + "|" + PATH_EMPTY$ + ")") + subexp("\\?(" + QUERY$ + ")") + "?$", SAMEDOC_REF$ = "^" + subexp("\\#(" + FRAGMENT$ + ")") + "?$", AUTHORITY_REF$ = "^" + subexp("(" + USERINFO$ + ")@") + "?(" + HOST$ + ")" + subexp("\\:(" + PORT$ + ")") + "?$";
            return {
                NOT_SCHEME: new RegExp(merge("[^]", ALPHA$$, DIGIT$$, "[\\+\\-\\.]"), "g"),
                NOT_USERINFO: new RegExp(merge("[^\\%\\:]", UNRESERVED$$, SUB_DELIMS$$), "g"),
                NOT_HOST: new RegExp(merge("[^\\%\\[\\]\\:]", UNRESERVED$$, SUB_DELIMS$$), "g"),
                NOT_PATH: new RegExp(merge("[^\\%\\/\\:\\@]", UNRESERVED$$, SUB_DELIMS$$), "g"),
                NOT_PATH_NOSCHEME: new RegExp(merge("[^\\%\\/\\@]", UNRESERVED$$, SUB_DELIMS$$), "g"),
                NOT_QUERY: new RegExp(merge("[^\\%]", UNRESERVED$$, SUB_DELIMS$$, "[\\:\\@\\/\\?]", IPRIVATE$$), "g"),
                NOT_FRAGMENT: new RegExp(merge("[^\\%]", UNRESERVED$$, SUB_DELIMS$$, "[\\:\\@\\/\\?]"), "g"),
                ESCAPE: new RegExp(merge("[^]", UNRESERVED$$, SUB_DELIMS$$), "g"),
                UNRESERVED: new RegExp(UNRESERVED$$, "g"),
                OTHER_CHARS: new RegExp(merge("[^\\%]", UNRESERVED$$, RESERVED$$), "g"),
                PCT_ENCODED: new RegExp(PCT_ENCODED$, "g"),
                IPV4ADDRESS: new RegExp("^(" + IPV4ADDRESS$ + ")$"),
                IPV6ADDRESS: new RegExp("^\\[?(" + IPV6ADDRESS$ + ")" + subexp(subexp("\\%25|\\%(?!" + HEXDIG$$ + "{2})") + "(" + ZONEID$ + ")") + "?\\]?$") //RFC 6874, with relaxed parsing rules
            };
        }
        var URI_PROTOCOL = buildExps(false);
        var IRI_PROTOCOL = buildExps(true);
        var slicedToArray = function () {
            function sliceIterator(arr, i) {
                var _arr = [];
                var _n = true;
                var _d = false;
                var _e = undefined;
                try {
                    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
                        _arr.push(_s.value);
                        if (i && _arr.length === i)
                            break;
                    }
                }
                catch (err) {
                    _d = true;
                    _e = err;
                }
                finally {
                    try {
                        if (!_n && _i["return"])
                            _i["return"]();
                    }
                    finally {
                        if (_d)
                            throw _e;
                    }
                }
                return _arr;
            }
            return function (arr, i) {
                if (Array.isArray(arr)) {
                    return arr;
                }
                else if (Symbol.iterator in Object(arr)) {
                    return sliceIterator(arr, i);
                }
                else {
                    throw new TypeError("Invalid attempt to destructure non-iterable instance");
                }
            };
        }();
        var toConsumableArray = function (arr) {
            if (Array.isArray(arr)) {
                for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++)
                    arr2[i] = arr[i];
                return arr2;
            }
            else {
                return Array.from(arr);
            }
        };
        /** Highest positive signed 32-bit float value */
        var maxInt = 2147483647; // aka. 0x7FFFFFFF or 2^31-1
        /** Bootstring parameters */
        var base = 36;
        var tMin = 1;
        var tMax = 26;
        var skew = 38;
        var damp = 700;
        var initialBias = 72;
        var initialN = 128; // 0x80
        var delimiter = '-'; // '\x2D'
        /** Regular expressions */
        var regexPunycode = /^xn--/;
        var regexNonASCII = /[^\0-\x7E]/; // non-ASCII chars
        var regexSeparators = /[\x2E\u3002\uFF0E\uFF61]/g; // RFC 3490 separators
        /** Error messages */
        var errors = {
            'overflow': 'Overflow: input needs wider integers to process',
            'not-basic': 'Illegal input >= 0x80 (not a basic code point)',
            'invalid-input': 'Invalid input'
        };
        /** Convenience shortcuts */
        var baseMinusTMin = base - tMin;
        var floor = Math.floor;
        var stringFromCharCode = String.fromCharCode;
        /*--------------------------------------------------------------------------*/
        /**
         * A generic error utility function.
         * @private
         * @param {String} type The error type.
         * @returns {Error} Throws a `RangeError` with the applicable error message.
         */
        function error$1(type) {
            throw new RangeError(errors[type]);
        }
        /**
         * A generic `Array#map` utility function.
         * @private
         * @param {Array} array The array to iterate over.
         * @param {Function} callback The function that gets called for every array
         * item.
         * @returns {Array} A new array of values returned by the callback function.
         */
        function map(array, fn) {
            var result = [];
            var length = array.length;
            while (length--) {
                result[length] = fn(array[length]);
            }
            return result;
        }
        /**
         * A simple `Array#map`-like wrapper to work with domain name strings or email
         * addresses.
         * @private
         * @param {String} domain The domain name or email address.
         * @param {Function} callback The function that gets called for every
         * character.
         * @returns {Array} A new string of characters returned by the callback
         * function.
         */
        function mapDomain(string, fn) {
            var parts = string.split('@');
            var result = '';
            if (parts.length > 1) {
                // In email addresses, only the domain name should be punycoded. Leave
                // the local part (i.e. everything up to `@`) intact.
                result = parts[0] + '@';
                string = parts[1];
            }
            // Avoid `split(regex)` for IE8 compatibility. See #17.
            string = string.replace(regexSeparators, '\x2E');
            var labels = string.split('.');
            var encoded = map(labels, fn).join('.');
            return result + encoded;
        }
        /**
         * Creates an array containing the numeric code points of each Unicode
         * character in the string. While JavaScript uses UCS-2 internally,
         * this function will convert a pair of surrogate halves (each of which
         * UCS-2 exposes as separate characters) into a single code point,
         * matching UTF-16.
         * @see `punycode.ucs2.encode`
         * @see <https://mathiasbynens.be/notes/javascript-encoding>
         * @memberOf punycode.ucs2
         * @name decode
         * @param {String} string The Unicode input string (UCS-2).
         * @returns {Array} The new array of code points.
         */
        function ucs2decode(string) {
            var output = [];
            var counter = 0;
            var length = string.length;
            while (counter < length) {
                var value = string.charCodeAt(counter++);
                if (value >= 0xD800 && value <= 0xDBFF && counter < length) {
                    // It's a high surrogate, and there is a next character.
                    var extra = string.charCodeAt(counter++);
                    if ((extra & 0xFC00) == 0xDC00) {
                        // Low surrogate.
                        output.push(((value & 0x3FF) << 10) + (extra & 0x3FF) + 0x10000);
                    }
                    else {
                        // It's an unmatched surrogate; only append this code unit, in case the
                        // next code unit is the high surrogate of a surrogate pair.
                        output.push(value);
                        counter--;
                    }
                }
                else {
                    output.push(value);
                }
            }
            return output;
        }
        /**
         * Creates a string based on an array of numeric code points.
         * @see `punycode.ucs2.decode`
         * @memberOf punycode.ucs2
         * @name encode
         * @param {Array} codePoints The array of numeric code points.
         * @returns {String} The new Unicode string (UCS-2).
         */
        var ucs2encode = function ucs2encode(array) {
            return String.fromCodePoint.apply(String, toConsumableArray(array));
        };
        /**
         * Converts a basic code point into a digit/integer.
         * @see `digitToBasic()`
         * @private
         * @param {Number} codePoint The basic numeric code point value.
         * @returns {Number} The numeric value of a basic code point (for use in
         * representing integers) in the range `0` to `base - 1`, or `base` if
         * the code point does not represent a value.
         */
        var basicToDigit = function basicToDigit(codePoint) {
            if (codePoint - 0x30 < 0x0A) {
                return codePoint - 0x16;
            }
            if (codePoint - 0x41 < 0x1A) {
                return codePoint - 0x41;
            }
            if (codePoint - 0x61 < 0x1A) {
                return codePoint - 0x61;
            }
            return base;
        };
        /**
         * Converts a digit/integer into a basic code point.
         * @see `basicToDigit()`
         * @private
         * @param {Number} digit The numeric value of a basic code point.
         * @returns {Number} The basic code point whose value (when used for
         * representing integers) is `digit`, which needs to be in the range
         * `0` to `base - 1`. If `flag` is non-zero, the uppercase form is
         * used; else, the lowercase form is used. The behavior is undefined
         * if `flag` is non-zero and `digit` has no uppercase form.
         */
        var digitToBasic = function digitToBasic(digit, flag) {
            //  0..25 map to ASCII a..z or A..Z
            // 26..35 map to ASCII 0..9
            return digit + 22 + 75 * (digit < 26) - ((flag != 0) << 5);
        };
        /**
         * Bias adaptation function as per section 3.4 of RFC 3492.
         * https://tools.ietf.org/html/rfc3492#section-3.4
         * @private
         */
        var adapt = function adapt(delta, numPoints, firstTime) {
            var k = 0;
            delta = firstTime ? floor(delta / damp) : delta >> 1;
            delta += floor(delta / numPoints);
            for (; /* no initialization */ delta > baseMinusTMin * tMax >> 1; k += base) {
                delta = floor(delta / baseMinusTMin);
            }
            return floor(k + (baseMinusTMin + 1) * delta / (delta + skew));
        };
        /**
         * Converts a Punycode string of ASCII-only symbols to a string of Unicode
         * symbols.
         * @memberOf punycode
         * @param {String} input The Punycode string of ASCII-only symbols.
         * @returns {String} The resulting string of Unicode symbols.
         */
        var decode = function decode(input) {
            // Don't use UCS-2.
            var output = [];
            var inputLength = input.length;
            var i = 0;
            var n = initialN;
            var bias = initialBias;
            // Handle the basic code points: let `basic` be the number of input code
            // points before the last delimiter, or `0` if there is none, then copy
            // the first basic code points to the output.
            var basic = input.lastIndexOf(delimiter);
            if (basic < 0) {
                basic = 0;
            }
            for (var j = 0; j < basic; ++j) {
                // if it's not a basic code point
                if (input.charCodeAt(j) >= 0x80) {
                    error$1('not-basic');
                }
                output.push(input.charCodeAt(j));
            }
            // Main decoding loop: start just after the last delimiter if any basic code
            // points were copied; start at the beginning otherwise.
            for (var index = basic > 0 ? basic + 1 : 0; index < inputLength;) /* no final expression */ {
                // `index` is the index of the next character to be consumed.
                // Decode a generalized variable-length integer into `delta`,
                // which gets added to `i`. The overflow checking is easier
                // if we increase `i` as we go, then subtract off its starting
                // value at the end to obtain `delta`.
                var oldi = i;
                for (var w = 1, k = base;; /* no condition */ k += base) {
                    if (index >= inputLength) {
                        error$1('invalid-input');
                    }
                    var digit = basicToDigit(input.charCodeAt(index++));
                    if (digit >= base || digit > floor((maxInt - i) / w)) {
                        error$1('overflow');
                    }
                    i += digit * w;
                    var t = k <= bias ? tMin : k >= bias + tMax ? tMax : k - bias;
                    if (digit < t) {
                        break;
                    }
                    var baseMinusT = base - t;
                    if (w > floor(maxInt / baseMinusT)) {
                        error$1('overflow');
                    }
                    w *= baseMinusT;
                }
                var out = output.length + 1;
                bias = adapt(i - oldi, out, oldi == 0);
                // `i` was supposed to wrap around from `out` to `0`,
                // incrementing `n` each time, so we'll fix that now:
                if (floor(i / out) > maxInt - n) {
                    error$1('overflow');
                }
                n += floor(i / out);
                i %= out;
                // Insert `n` at position `i` of the output.
                output.splice(i++, 0, n);
            }
            return String.fromCodePoint.apply(String, output);
        };
        /**
         * Converts a string of Unicode symbols (e.g. a domain name label) to a
         * Punycode string of ASCII-only symbols.
         * @memberOf punycode
         * @param {String} input The string of Unicode symbols.
         * @returns {String} The resulting Punycode string of ASCII-only symbols.
         */
        var encode = function encode(input) {
            var output = [];
            // Convert the input in UCS-2 to an array of Unicode code points.
            input = ucs2decode(input);
            // Cache the length.
            var inputLength = input.length;
            // Initialize the state.
            var n = initialN;
            var delta = 0;
            var bias = initialBias;
            // Handle the basic code points.
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;
            try {
                for (var _iterator = input[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var _currentValue2 = _step.value;
                    if (_currentValue2 < 0x80) {
                        output.push(stringFromCharCode(_currentValue2));
                    }
                }
            }
            catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            }
            finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                }
                finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
            var basicLength = output.length;
            var handledCPCount = basicLength;
            // `handledCPCount` is the number of code points that have been handled;
            // `basicLength` is the number of basic code points.
            // Finish the basic string with a delimiter unless it's empty.
            if (basicLength) {
                output.push(delimiter);
            }
            // Main encoding loop:
            while (handledCPCount < inputLength) {
                // All non-basic code points < n have been handled already. Find the next
                // larger one:
                var m = maxInt;
                var _iteratorNormalCompletion2 = true;
                var _didIteratorError2 = false;
                var _iteratorError2 = undefined;
                try {
                    for (var _iterator2 = input[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                        var currentValue = _step2.value;
                        if (currentValue >= n && currentValue < m) {
                            m = currentValue;
                        }
                    }
                    // Increase `delta` enough to advance the decoder's <n,i> state to <m,0>,
                    // but guard against overflow.
                }
                catch (err) {
                    _didIteratorError2 = true;
                    _iteratorError2 = err;
                }
                finally {
                    try {
                        if (!_iteratorNormalCompletion2 && _iterator2.return) {
                            _iterator2.return();
                        }
                    }
                    finally {
                        if (_didIteratorError2) {
                            throw _iteratorError2;
                        }
                    }
                }
                var handledCPCountPlusOne = handledCPCount + 1;
                if (m - n > floor((maxInt - delta) / handledCPCountPlusOne)) {
                    error$1('overflow');
                }
                delta += (m - n) * handledCPCountPlusOne;
                n = m;
                var _iteratorNormalCompletion3 = true;
                var _didIteratorError3 = false;
                var _iteratorError3 = undefined;
                try {
                    for (var _iterator3 = input[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                        var _currentValue = _step3.value;
                        if (_currentValue < n && ++delta > maxInt) {
                            error$1('overflow');
                        }
                        if (_currentValue == n) {
                            // Represent delta as a generalized variable-length integer.
                            var q = delta;
                            for (var k = base;; /* no condition */ k += base) {
                                var t = k <= bias ? tMin : k >= bias + tMax ? tMax : k - bias;
                                if (q < t) {
                                    break;
                                }
                                var qMinusT = q - t;
                                var baseMinusT = base - t;
                                output.push(stringFromCharCode(digitToBasic(t + qMinusT % baseMinusT, 0)));
                                q = floor(qMinusT / baseMinusT);
                            }
                            output.push(stringFromCharCode(digitToBasic(q, 0)));
                            bias = adapt(delta, handledCPCountPlusOne, handledCPCount == basicLength);
                            delta = 0;
                            ++handledCPCount;
                        }
                    }
                }
                catch (err) {
                    _didIteratorError3 = true;
                    _iteratorError3 = err;
                }
                finally {
                    try {
                        if (!_iteratorNormalCompletion3 && _iterator3.return) {
                            _iterator3.return();
                        }
                    }
                    finally {
                        if (_didIteratorError3) {
                            throw _iteratorError3;
                        }
                    }
                }
                ++delta;
                ++n;
            }
            return output.join('');
        };
        /**
         * Converts a Punycode string representing a domain name or an email address
         * to Unicode. Only the Punycoded parts of the input will be converted, i.e.
         * it doesn't matter if you call it on a string that has already been
         * converted to Unicode.
         * @memberOf punycode
         * @param {String} input The Punycoded domain name or email address to
         * convert to Unicode.
         * @returns {String} The Unicode representation of the given Punycode
         * string.
         */
        var toUnicode = function toUnicode(input) {
            return mapDomain(input, function (string) {
                return regexPunycode.test(string) ? decode(string.slice(4).toLowerCase()) : string;
            });
        };
        /**
         * Converts a Unicode string representing a domain name or an email address to
         * Punycode. Only the non-ASCII parts of the domain name will be converted,
         * i.e. it doesn't matter if you call it with a domain that's already in
         * ASCII.
         * @memberOf punycode
         * @param {String} input The domain name or email address to convert, as a
         * Unicode string.
         * @returns {String} The Punycode representation of the given domain name or
         * email address.
         */
        var toASCII = function toASCII(input) {
            return mapDomain(input, function (string) {
                return regexNonASCII.test(string) ? 'xn--' + encode(string) : string;
            });
        };
        /*--------------------------------------------------------------------------*/
        /** Define the public API */
        var punycode = {
            /**
          * A string representing the current Punycode.js version number.
          * @memberOf punycode
          * @type String
          */
            'version': '2.1.0',
            /**
          * An object of methods to convert from JavaScript's internal character
          * representation (UCS-2) to Unicode code points, and back.
          * @see <https://mathiasbynens.be/notes/javascript-encoding>
          * @memberOf punycode
          * @type Object
          */
            'ucs2': {
                'decode': ucs2decode,
                'encode': ucs2encode
            },
            'decode': decode,
            'encode': encode,
            'toASCII': toASCII,
            'toUnicode': toUnicode
        };
        /**
         * URI.js
         *
         * @fileoverview An RFC 3986 compliant, scheme extendable URI parsing/validating/resolving library for JavaScript.
         * @author <a href="mailto:gary.court@gmail.com">Gary Court</a>
         * @see http://github.com/garycourt/uri-js
         */
        /**
         * Copyright 2011 Gary Court. All rights reserved.
         *
         * Redistribution and use in source and binary forms, with or without modification, are
         * permitted provided that the following conditions are met:
         *
         *    1. Redistributions of source code must retain the above copyright notice, this list of
         *       conditions and the following disclaimer.
         *
         *    2. Redistributions in binary form must reproduce the above copyright notice, this list
         *       of conditions and the following disclaimer in the documentation and/or other materials
         *       provided with the distribution.
         *
         * THIS SOFTWARE IS PROVIDED BY GARY COURT ``AS IS'' AND ANY EXPRESS OR IMPLIED
         * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
         * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GARY COURT OR
         * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
         * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
         * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
         * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
         * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
         * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
         *
         * The views and conclusions contained in the software and documentation are those of the
         * authors and should not be interpreted as representing official policies, either expressed
         * or implied, of Gary Court.
         */
        var SCHEMES = {};
        function pctEncChar(chr) {
            var c = chr.charCodeAt(0);
            var e = void 0;
            if (c < 16)
                e = "%0" + c.toString(16).toUpperCase();
            else if (c < 128)
                e = "%" + c.toString(16).toUpperCase();
            else if (c < 2048)
                e = "%" + (c >> 6 | 192).toString(16).toUpperCase() + "%" + (c & 63 | 128).toString(16).toUpperCase();
            else
                e = "%" + (c >> 12 | 224).toString(16).toUpperCase() + "%" + (c >> 6 & 63 | 128).toString(16).toUpperCase() + "%" + (c & 63 | 128).toString(16).toUpperCase();
            return e;
        }
        function pctDecChars(str) {
            var newStr = "";
            var i = 0;
            var il = str.length;
            while (i < il) {
                var c = parseInt(str.substr(i + 1, 2), 16);
                if (c < 128) {
                    newStr += String.fromCharCode(c);
                    i += 3;
                }
                else if (c >= 194 && c < 224) {
                    if (il - i >= 6) {
                        var c2 = parseInt(str.substr(i + 4, 2), 16);
                        newStr += String.fromCharCode((c & 31) << 6 | c2 & 63);
                    }
                    else {
                        newStr += str.substr(i, 6);
                    }
                    i += 6;
                }
                else if (c >= 224) {
                    if (il - i >= 9) {
                        var _c = parseInt(str.substr(i + 4, 2), 16);
                        var c3 = parseInt(str.substr(i + 7, 2), 16);
                        newStr += String.fromCharCode((c & 15) << 12 | (_c & 63) << 6 | c3 & 63);
                    }
                    else {
                        newStr += str.substr(i, 9);
                    }
                    i += 9;
                }
                else {
                    newStr += str.substr(i, 3);
                    i += 3;
                }
            }
            return newStr;
        }
        function _normalizeComponentEncoding(components, protocol) {
            function decodeUnreserved(str) {
                var decStr = pctDecChars(str);
                return !decStr.match(protocol.UNRESERVED) ? str : decStr;
            }
            if (components.scheme)
                components.scheme = String(components.scheme).replace(protocol.PCT_ENCODED, decodeUnreserved).toLowerCase().replace(protocol.NOT_SCHEME, "");
            if (components.userinfo !== undefined)
                components.userinfo = String(components.userinfo).replace(protocol.PCT_ENCODED, decodeUnreserved).replace(protocol.NOT_USERINFO, pctEncChar).replace(protocol.PCT_ENCODED, toUpperCase);
            if (components.host !== undefined)
                components.host = String(components.host).replace(protocol.PCT_ENCODED, decodeUnreserved).toLowerCase().replace(protocol.NOT_HOST, pctEncChar).replace(protocol.PCT_ENCODED, toUpperCase);
            if (components.path !== undefined)
                components.path = String(components.path).replace(protocol.PCT_ENCODED, decodeUnreserved).replace(components.scheme ? protocol.NOT_PATH : protocol.NOT_PATH_NOSCHEME, pctEncChar).replace(protocol.PCT_ENCODED, toUpperCase);
            if (components.query !== undefined)
                components.query = String(components.query).replace(protocol.PCT_ENCODED, decodeUnreserved).replace(protocol.NOT_QUERY, pctEncChar).replace(protocol.PCT_ENCODED, toUpperCase);
            if (components.fragment !== undefined)
                components.fragment = String(components.fragment).replace(protocol.PCT_ENCODED, decodeUnreserved).replace(protocol.NOT_FRAGMENT, pctEncChar).replace(protocol.PCT_ENCODED, toUpperCase);
            return components;
        }
        function _stripLeadingZeros(str) {
            return str.replace(/^0*(.*)/, "$1") || "0";
        }
        function _normalizeIPv4(host, protocol) {
            var matches = host.match(protocol.IPV4ADDRESS) || [];
            var _matches = slicedToArray(matches, 2), address = _matches[1];
            if (address) {
                return address.split(".").map(_stripLeadingZeros).join(".");
            }
            else {
                return host;
            }
        }
        function _normalizeIPv6(host, protocol) {
            var matches = host.match(protocol.IPV6ADDRESS) || [];
            var _matches2 = slicedToArray(matches, 3), address = _matches2[1], zone = _matches2[2];
            if (address) {
                var _address$toLowerCase$ = address.toLowerCase().split('::').reverse(), _address$toLowerCase$2 = slicedToArray(_address$toLowerCase$, 2), last = _address$toLowerCase$2[0], first = _address$toLowerCase$2[1];
                var firstFields = first ? first.split(":").map(_stripLeadingZeros) : [];
                var lastFields = last.split(":").map(_stripLeadingZeros);
                var isLastFieldIPv4Address = protocol.IPV4ADDRESS.test(lastFields[lastFields.length - 1]);
                var fieldCount = isLastFieldIPv4Address ? 7 : 8;
                var lastFieldsStart = lastFields.length - fieldCount;
                var fields = Array(fieldCount);
                for (var x = 0; x < fieldCount; ++x) {
                    fields[x] = firstFields[x] || lastFields[lastFieldsStart + x] || '';
                }
                if (isLastFieldIPv4Address) {
                    fields[fieldCount - 1] = _normalizeIPv4(fields[fieldCount - 1], protocol);
                }
                var allZeroFields = fields.reduce(function (acc, field, index) {
                    if (!field || field === "0") {
                        var lastLongest = acc[acc.length - 1];
                        if (lastLongest && lastLongest.index + lastLongest.length === index) {
                            lastLongest.length++;
                        }
                        else {
                            acc.push({ index: index, length: 1 });
                        }
                    }
                    return acc;
                }, []);
                var longestZeroFields = allZeroFields.sort(function (a, b) {
                    return b.length - a.length;
                })[0];
                var newHost = void 0;
                if (longestZeroFields && longestZeroFields.length > 1) {
                    var newFirst = fields.slice(0, longestZeroFields.index);
                    var newLast = fields.slice(longestZeroFields.index + longestZeroFields.length);
                    newHost = newFirst.join(":") + "::" + newLast.join(":");
                }
                else {
                    newHost = fields.join(":");
                }
                if (zone) {
                    newHost += "%" + zone;
                }
                return newHost;
            }
            else {
                return host;
            }
        }
        var URI_PARSE = /^(?:([^:\/?#]+):)?(?:\/\/((?:([^\/?#@]*)@)?(\[[^\/?#\]]+\]|[^\/?#:]*)(?:\:(\d*))?))?([^?#]*)(?:\?([^#]*))?(?:#((?:.|\n|\r)*))?/i;
        var NO_MATCH_IS_UNDEFINED = "".match(/(){0}/)[1] === undefined;
        function parse(uriString) {
            var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
            var components = {};
            var protocol = options.iri !== false ? IRI_PROTOCOL : URI_PROTOCOL;
            if (options.reference === "suffix")
                uriString = (options.scheme ? options.scheme + ":" : "") + "//" + uriString;
            var matches = uriString.match(URI_PARSE);
            if (matches) {
                if (NO_MATCH_IS_UNDEFINED) {
                    //store each component
                    components.scheme = matches[1];
                    components.userinfo = matches[3];
                    components.host = matches[4];
                    components.port = parseInt(matches[5], 10);
                    components.path = matches[6] || "";
                    components.query = matches[7];
                    components.fragment = matches[8];
                    //fix port number
                    if (isNaN(components.port)) {
                        components.port = matches[5];
                    }
                }
                else {
                    //IE FIX for improper RegExp matching
                    //store each component
                    components.scheme = matches[1] || undefined;
                    components.userinfo = uriString.indexOf("@") !== -1 ? matches[3] : undefined;
                    components.host = uriString.indexOf("//") !== -1 ? matches[4] : undefined;
                    components.port = parseInt(matches[5], 10);
                    components.path = matches[6] || "";
                    components.query = uriString.indexOf("?") !== -1 ? matches[7] : undefined;
                    components.fragment = uriString.indexOf("#") !== -1 ? matches[8] : undefined;
                    //fix port number
                    if (isNaN(components.port)) {
                        components.port = uriString.match(/\/\/(?:.|\n)*\:(?:\/|\?|\#|$)/) ? matches[4] : undefined;
                    }
                }
                if (components.host) {
                    //normalize IP hosts
                    components.host = _normalizeIPv6(_normalizeIPv4(components.host, protocol), protocol);
                }
                //determine reference type
                if (components.scheme === undefined && components.userinfo === undefined && components.host === undefined && components.port === undefined && !components.path && components.query === undefined) {
                    components.reference = "same-document";
                }
                else if (components.scheme === undefined) {
                    components.reference = "relative";
                }
                else if (components.fragment === undefined) {
                    components.reference = "absolute";
                }
                else {
                    components.reference = "uri";
                }
                //check for reference errors
                if (options.reference && options.reference !== "suffix" && options.reference !== components.reference) {
                    components.error = components.error || "URI is not a " + options.reference + " reference.";
                }
                //find scheme handler
                var schemeHandler = SCHEMES[(options.scheme || components.scheme || "").toLowerCase()];
                //check if scheme can't handle IRIs
                if (!options.unicodeSupport && (!schemeHandler || !schemeHandler.unicodeSupport)) {
                    //if host component is a domain name
                    if (components.host && (options.domainHost || schemeHandler && schemeHandler.domainHost)) {
                        //convert Unicode IDN -> ASCII IDN
                        try {
                            components.host = punycode.toASCII(components.host.replace(protocol.PCT_ENCODED, pctDecChars).toLowerCase());
                        }
                        catch (e) {
                            components.error = components.error || "Host's domain name can not be converted to ASCII via punycode: " + e;
                        }
                    }
                    //convert IRI -> URI
                    _normalizeComponentEncoding(components, URI_PROTOCOL);
                }
                else {
                    //normalize encodings
                    _normalizeComponentEncoding(components, protocol);
                }
                //perform scheme specific parsing
                if (schemeHandler && schemeHandler.parse) {
                    schemeHandler.parse(components, options);
                }
            }
            else {
                components.error = components.error || "URI can not be parsed.";
            }
            return components;
        }
        function _recomposeAuthority(components, options) {
            var protocol = options.iri !== false ? IRI_PROTOCOL : URI_PROTOCOL;
            var uriTokens = [];
            if (components.userinfo !== undefined) {
                uriTokens.push(components.userinfo);
                uriTokens.push("@");
            }
            if (components.host !== undefined) {
                //normalize IP hosts, add brackets and escape zone separator for IPv6
                uriTokens.push(_normalizeIPv6(_normalizeIPv4(String(components.host), protocol), protocol).replace(protocol.IPV6ADDRESS, function (_, $1, $2) {
                    return "[" + $1 + ($2 ? "%25" + $2 : "") + "]";
                }));
            }
            if (typeof components.port === "number") {
                uriTokens.push(":");
                uriTokens.push(components.port.toString(10));
            }
            return uriTokens.length ? uriTokens.join("") : undefined;
        }
        var RDS1 = /^\.\.?\//;
        var RDS2 = /^\/\.(\/|$)/;
        var RDS3 = /^\/\.\.(\/|$)/;
        var RDS5 = /^\/?(?:.|\n)*?(?=\/|$)/;
        function removeDotSegments(input) {
            var output = [];
            while (input.length) {
                if (input.match(RDS1)) {
                    input = input.replace(RDS1, "");
                }
                else if (input.match(RDS2)) {
                    input = input.replace(RDS2, "/");
                }
                else if (input.match(RDS3)) {
                    input = input.replace(RDS3, "/");
                    output.pop();
                }
                else if (input === "." || input === "..") {
                    input = "";
                }
                else {
                    var im = input.match(RDS5);
                    if (im) {
                        var s = im[0];
                        input = input.slice(s.length);
                        output.push(s);
                    }
                    else {
                        throw new Error("Unexpected dot segment condition");
                    }
                }
            }
            return output.join("");
        }
        function serialize(components) {
            var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
            var protocol = options.iri ? IRI_PROTOCOL : URI_PROTOCOL;
            var uriTokens = [];
            //find scheme handler
            var schemeHandler = SCHEMES[(options.scheme || components.scheme || "").toLowerCase()];
            //perform scheme specific serialization
            if (schemeHandler && schemeHandler.serialize)
                schemeHandler.serialize(components, options);
            if (components.host) {
                //if host component is an IPv6 address
                if (protocol.IPV6ADDRESS.test(components.host)) { }
                //TODO: normalize IPv6 address as per RFC 5952
                //if host component is a domain name
                else if (options.domainHost || schemeHandler && schemeHandler.domainHost) {
                    //convert IDN via punycode
                    try {
                        components.host = !options.iri ? punycode.toASCII(components.host.replace(protocol.PCT_ENCODED, pctDecChars).toLowerCase()) : punycode.toUnicode(components.host);
                    }
                    catch (e) {
                        components.error = components.error || "Host's domain name can not be converted to " + (!options.iri ? "ASCII" : "Unicode") + " via punycode: " + e;
                    }
                }
            }
            //normalize encoding
            _normalizeComponentEncoding(components, protocol);
            if (options.reference !== "suffix" && components.scheme) {
                uriTokens.push(components.scheme);
                uriTokens.push(":");
            }
            var authority = _recomposeAuthority(components, options);
            if (authority !== undefined) {
                if (options.reference !== "suffix") {
                    uriTokens.push("//");
                }
                uriTokens.push(authority);
                if (components.path && components.path.charAt(0) !== "/") {
                    uriTokens.push("/");
                }
            }
            if (components.path !== undefined) {
                var s = components.path;
                if (!options.absolutePath && (!schemeHandler || !schemeHandler.absolutePath)) {
                    s = removeDotSegments(s);
                }
                if (authority === undefined) {
                    s = s.replace(/^\/\//, "/%2F"); //don't allow the path to start with "//"
                }
                uriTokens.push(s);
            }
            if (components.query !== undefined) {
                uriTokens.push("?");
                uriTokens.push(components.query);
            }
            if (components.fragment !== undefined) {
                uriTokens.push("#");
                uriTokens.push(components.fragment);
            }
            return uriTokens.join(""); //merge tokens into a string
        }
        function resolveComponents(base, relative) {
            var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
            var skipNormalization = arguments[3];
            var target = {};
            if (!skipNormalization) {
                base = parse(serialize(base, options), options); //normalize base components
                relative = parse(serialize(relative, options), options); //normalize relative components
            }
            options = options || {};
            if (!options.tolerant && relative.scheme) {
                target.scheme = relative.scheme;
                //target.authority = relative.authority;
                target.userinfo = relative.userinfo;
                target.host = relative.host;
                target.port = relative.port;
                target.path = removeDotSegments(relative.path || "");
                target.query = relative.query;
            }
            else {
                if (relative.userinfo !== undefined || relative.host !== undefined || relative.port !== undefined) {
                    //target.authority = relative.authority;
                    target.userinfo = relative.userinfo;
                    target.host = relative.host;
                    target.port = relative.port;
                    target.path = removeDotSegments(relative.path || "");
                    target.query = relative.query;
                }
                else {
                    if (!relative.path) {
                        target.path = base.path;
                        if (relative.query !== undefined) {
                            target.query = relative.query;
                        }
                        else {
                            target.query = base.query;
                        }
                    }
                    else {
                        if (relative.path.charAt(0) === "/") {
                            target.path = removeDotSegments(relative.path);
                        }
                        else {
                            if ((base.userinfo !== undefined || base.host !== undefined || base.port !== undefined) && !base.path) {
                                target.path = "/" + relative.path;
                            }
                            else if (!base.path) {
                                target.path = relative.path;
                            }
                            else {
                                target.path = base.path.slice(0, base.path.lastIndexOf("/") + 1) + relative.path;
                            }
                            target.path = removeDotSegments(target.path);
                        }
                        target.query = relative.query;
                    }
                    //target.authority = base.authority;
                    target.userinfo = base.userinfo;
                    target.host = base.host;
                    target.port = base.port;
                }
                target.scheme = base.scheme;
            }
            target.fragment = relative.fragment;
            return target;
        }
        function resolve(baseURI, relativeURI, options) {
            var schemelessOptions = assign({ scheme: 'null' }, options);
            return serialize(resolveComponents(parse(baseURI, schemelessOptions), parse(relativeURI, schemelessOptions), schemelessOptions, true), schemelessOptions);
        }
        function normalize(uri, options) {
            if (typeof uri === "string") {
                uri = serialize(parse(uri, options), options);
            }
            else if (typeOf(uri) === "object") {
                uri = parse(serialize(uri, options), options);
            }
            return uri;
        }
        function equal(uriA, uriB, options) {
            if (typeof uriA === "string") {
                uriA = serialize(parse(uriA, options), options);
            }
            else if (typeOf(uriA) === "object") {
                uriA = serialize(uriA, options);
            }
            if (typeof uriB === "string") {
                uriB = serialize(parse(uriB, options), options);
            }
            else if (typeOf(uriB) === "object") {
                uriB = serialize(uriB, options);
            }
            return uriA === uriB;
        }
        function escapeComponent(str, options) {
            return str && str.toString().replace(!options || !options.iri ? URI_PROTOCOL.ESCAPE : IRI_PROTOCOL.ESCAPE, pctEncChar);
        }
        function unescapeComponent(str, options) {
            return str && str.toString().replace(!options || !options.iri ? URI_PROTOCOL.PCT_ENCODED : IRI_PROTOCOL.PCT_ENCODED, pctDecChars);
        }
        var handler = {
            scheme: "http",
            domainHost: true,
            parse: function parse(components, options) {
                //report missing host
                if (!components.host) {
                    components.error = components.error || "HTTP URIs must have a host.";
                }
                return components;
            },
            serialize: function serialize(components, options) {
                //normalize the default port
                if (components.port === (String(components.scheme).toLowerCase() !== "https" ? 80 : 443) || components.port === "") {
                    components.port = undefined;
                }
                //normalize the empty path
                if (!components.path) {
                    components.path = "/";
                }
                //NOTE: We do not parse query strings for HTTP URIs
                //as WWW Form Url Encoded query strings are part of the HTML4+ spec,
                //and not the HTTP spec.
                return components;
            }
        };
        var handler$1 = {
            scheme: "https",
            domainHost: handler.domainHost,
            parse: handler.parse,
            serialize: handler.serialize
        };
        var O = {};
        var isIRI = true;
        //RFC 3986
        var UNRESERVED$$ = "[A-Za-z0-9\\-\\.\\_\\~" + (isIRI ? "\\xA0-\\u200D\\u2010-\\u2029\\u202F-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFEF" : "") + "]";
        var HEXDIG$$ = "[0-9A-Fa-f]"; //case-insensitive
        var PCT_ENCODED$ = subexp(subexp("%[EFef]" + HEXDIG$$ + "%" + HEXDIG$$ + HEXDIG$$ + "%" + HEXDIG$$ + HEXDIG$$) + "|" + subexp("%[89A-Fa-f]" + HEXDIG$$ + "%" + HEXDIG$$ + HEXDIG$$) + "|" + subexp("%" + HEXDIG$$ + HEXDIG$$)); //expanded
        //RFC 5322, except these symbols as per RFC 6068: @ : / ? # [ ] & ; =
        //const ATEXT$$ = "[A-Za-z0-9\\!\\#\\$\\%\\&\\'\\*\\+\\-\\/\\=\\?\\^\\_\\`\\{\\|\\}\\~]";
        //const WSP$$ = "[\\x20\\x09]";
        //const OBS_QTEXT$$ = "[\\x01-\\x08\\x0B\\x0C\\x0E-\\x1F\\x7F]";  //(%d1-8 / %d11-12 / %d14-31 / %d127)
        //const QTEXT$$ = merge("[\\x21\\x23-\\x5B\\x5D-\\x7E]", OBS_QTEXT$$);  //%d33 / %d35-91 / %d93-126 / obs-qtext
        //const VCHAR$$ = "[\\x21-\\x7E]";
        //const WSP$$ = "[\\x20\\x09]";
        //const OBS_QP$ = subexp("\\\\" + merge("[\\x00\\x0D\\x0A]", OBS_QTEXT$$));  //%d0 / CR / LF / obs-qtext
        //const FWS$ = subexp(subexp(WSP$$ + "*" + "\\x0D\\x0A") + "?" + WSP$$ + "+");
        //const QUOTED_PAIR$ = subexp(subexp("\\\\" + subexp(VCHAR$$ + "|" + WSP$$)) + "|" + OBS_QP$);
        //const QUOTED_STRING$ = subexp('\\"' + subexp(FWS$ + "?" + QCONTENT$) + "*" + FWS$ + "?" + '\\"');
        var ATEXT$$ = "[A-Za-z0-9\\!\\$\\%\\'\\*\\+\\-\\^\\_\\`\\{\\|\\}\\~]";
        var QTEXT$$ = "[\\!\\$\\%\\'\\(\\)\\*\\+\\,\\-\\.0-9\\<\\>A-Z\\x5E-\\x7E]";
        var VCHAR$$ = merge(QTEXT$$, "[\\\"\\\\]");
        var SOME_DELIMS$$ = "[\\!\\$\\'\\(\\)\\*\\+\\,\\;\\:\\@]";
        var UNRESERVED = new RegExp(UNRESERVED$$, "g");
        var PCT_ENCODED = new RegExp(PCT_ENCODED$, "g");
        var NOT_LOCAL_PART = new RegExp(merge("[^]", ATEXT$$, "[\\.]", '[\\"]', VCHAR$$), "g");
        var NOT_HFNAME = new RegExp(merge("[^]", UNRESERVED$$, SOME_DELIMS$$), "g");
        var NOT_HFVALUE = NOT_HFNAME;
        function decodeUnreserved(str) {
            var decStr = pctDecChars(str);
            return !decStr.match(UNRESERVED) ? str : decStr;
        }
        var handler$2 = {
            scheme: "mailto",
            parse: function parse$$1(components, options) {
                var mailtoComponents = components;
                var to = mailtoComponents.to = mailtoComponents.path ? mailtoComponents.path.split(",") : [];
                mailtoComponents.path = undefined;
                if (mailtoComponents.query) {
                    var unknownHeaders = false;
                    var headers = {};
                    var hfields = mailtoComponents.query.split("&");
                    for (var x = 0, xl = hfields.length; x < xl; ++x) {
                        var hfield = hfields[x].split("=");
                        switch (hfield[0]) {
                            case "to":
                                var toAddrs = hfield[1].split(",");
                                for (var _x = 0, _xl = toAddrs.length; _x < _xl; ++_x) {
                                    to.push(toAddrs[_x]);
                                }
                                break;
                            case "subject":
                                mailtoComponents.subject = unescapeComponent(hfield[1], options);
                                break;
                            case "body":
                                mailtoComponents.body = unescapeComponent(hfield[1], options);
                                break;
                            default:
                                unknownHeaders = true;
                                headers[unescapeComponent(hfield[0], options)] = unescapeComponent(hfield[1], options);
                                break;
                        }
                    }
                    if (unknownHeaders)
                        mailtoComponents.headers = headers;
                }
                mailtoComponents.query = undefined;
                for (var _x2 = 0, _xl2 = to.length; _x2 < _xl2; ++_x2) {
                    var addr = to[_x2].split("@");
                    addr[0] = unescapeComponent(addr[0]);
                    if (!options.unicodeSupport) {
                        //convert Unicode IDN -> ASCII IDN
                        try {
                            addr[1] = punycode.toASCII(unescapeComponent(addr[1], options).toLowerCase());
                        }
                        catch (e) {
                            mailtoComponents.error = mailtoComponents.error || "Email address's domain name can not be converted to ASCII via punycode: " + e;
                        }
                    }
                    else {
                        addr[1] = unescapeComponent(addr[1], options).toLowerCase();
                    }
                    to[_x2] = addr.join("@");
                }
                return mailtoComponents;
            },
            serialize: function serialize$$1(mailtoComponents, options) {
                var components = mailtoComponents;
                var to = toArray(mailtoComponents.to);
                if (to) {
                    for (var x = 0, xl = to.length; x < xl; ++x) {
                        var toAddr = String(to[x]);
                        var atIdx = toAddr.lastIndexOf("@");
                        var localPart = toAddr.slice(0, atIdx).replace(PCT_ENCODED, decodeUnreserved).replace(PCT_ENCODED, toUpperCase).replace(NOT_LOCAL_PART, pctEncChar);
                        var domain = toAddr.slice(atIdx + 1);
                        //convert IDN via punycode
                        try {
                            domain = !options.iri ? punycode.toASCII(unescapeComponent(domain, options).toLowerCase()) : punycode.toUnicode(domain);
                        }
                        catch (e) {
                            components.error = components.error || "Email address's domain name can not be converted to " + (!options.iri ? "ASCII" : "Unicode") + " via punycode: " + e;
                        }
                        to[x] = localPart + "@" + domain;
                    }
                    components.path = to.join(",");
                }
                var headers = mailtoComponents.headers = mailtoComponents.headers || {};
                if (mailtoComponents.subject)
                    headers["subject"] = mailtoComponents.subject;
                if (mailtoComponents.body)
                    headers["body"] = mailtoComponents.body;
                var fields = [];
                for (var name in headers) {
                    if (headers[name] !== O[name]) {
                        fields.push(name.replace(PCT_ENCODED, decodeUnreserved).replace(PCT_ENCODED, toUpperCase).replace(NOT_HFNAME, pctEncChar) + "=" + headers[name].replace(PCT_ENCODED, decodeUnreserved).replace(PCT_ENCODED, toUpperCase).replace(NOT_HFVALUE, pctEncChar));
                    }
                }
                if (fields.length) {
                    components.query = fields.join("&");
                }
                return components;
            }
        };
        var URN_PARSE = /^([^\:]+)\:(.*)/;
        //RFC 2141
        var handler$3 = {
            scheme: "urn",
            parse: function parse$$1(components, options) {
                var matches = components.path && components.path.match(URN_PARSE);
                var urnComponents = components;
                if (matches) {
                    var scheme = options.scheme || urnComponents.scheme || "urn";
                    var nid = matches[1].toLowerCase();
                    var nss = matches[2];
                    var urnScheme = scheme + ":" + (options.nid || nid);
                    var schemeHandler = SCHEMES[urnScheme];
                    urnComponents.nid = nid;
                    urnComponents.nss = nss;
                    urnComponents.path = undefined;
                    if (schemeHandler) {
                        urnComponents = schemeHandler.parse(urnComponents, options);
                    }
                }
                else {
                    urnComponents.error = urnComponents.error || "URN can not be parsed.";
                }
                return urnComponents;
            },
            serialize: function serialize$$1(urnComponents, options) {
                var scheme = options.scheme || urnComponents.scheme || "urn";
                var nid = urnComponents.nid;
                var urnScheme = scheme + ":" + (options.nid || nid);
                var schemeHandler = SCHEMES[urnScheme];
                if (schemeHandler) {
                    urnComponents = schemeHandler.serialize(urnComponents, options);
                }
                var uriComponents = urnComponents;
                var nss = urnComponents.nss;
                uriComponents.path = (nid || options.nid) + ":" + nss;
                return uriComponents;
            }
        };
        var UUID = /^[0-9A-Fa-f]{8}(?:\-[0-9A-Fa-f]{4}){3}\-[0-9A-Fa-f]{12}$/;
        //RFC 4122
        var handler$4 = {
            scheme: "urn:uuid",
            parse: function parse(urnComponents, options) {
                var uuidComponents = urnComponents;
                uuidComponents.uuid = uuidComponents.nss;
                uuidComponents.nss = undefined;
                if (!options.tolerant && (!uuidComponents.uuid || !uuidComponents.uuid.match(UUID))) {
                    uuidComponents.error = uuidComponents.error || "UUID is not valid.";
                }
                return uuidComponents;
            },
            serialize: function serialize(uuidComponents, options) {
                var urnComponents = uuidComponents;
                //normalize UUID
                urnComponents.nss = (uuidComponents.uuid || "").toLowerCase();
                return urnComponents;
            }
        };
        SCHEMES[handler.scheme] = handler;
        SCHEMES[handler$1.scheme] = handler$1;
        SCHEMES[handler$2.scheme] = handler$2;
        SCHEMES[handler$3.scheme] = handler$3;
        SCHEMES[handler$4.scheme] = handler$4;
        exports.SCHEMES = SCHEMES;
        exports.pctEncChar = pctEncChar;
        exports.pctDecChars = pctDecChars;
        exports.parse = parse;
        exports.removeDotSegments = removeDotSegments;
        exports.serialize = serialize;
        exports.resolveComponents = resolveComponents;
        exports.resolve = resolve;
        exports.normalize = normalize;
        exports.equal = equal;
        exports.escapeComponent = escapeComponent;
        exports.unescapeComponent = unescapeComponent;
        Object.defineProperty(exports, '__esModule', { value: true });
    })));
},
/* fast-deep-equal/index.js */ function _(require, module, exports) {
    // do not edit .js files directly - edit src/index.jst
    module.exports = function equal(a, b) {
        if (a === b)
            return true;
        if (a && b && typeof a == 'object' && typeof b == 'object') {
            if (a.constructor !== b.constructor)
                return false;
            var length, i, keys;
            if (Array.isArray(a)) {
                length = a.length;
                if (length != b.length)
                    return false;
                for (i = length; i-- !== 0;)
                    if (!equal(a[i], b[i]))
                        return false;
                return true;
            }
            if (a.constructor === RegExp)
                return a.source === b.source && a.flags === b.flags;
            if (a.valueOf !== Object.prototype.valueOf)
                return a.valueOf() === b.valueOf();
            if (a.toString !== Object.prototype.toString)
                return a.toString() === b.toString();
            keys = Object.keys(a);
            length = keys.length;
            if (length !== Object.keys(b).length)
                return false;
            for (i = length; i-- !== 0;)
                if (!Object.prototype.hasOwnProperty.call(b, keys[i]))
                    return false;
            for (i = length; i-- !== 0;) {
                var key = keys[i];
                if (!equal(a[key], b[key]))
                    return false;
            }
            return true;
        }
        // true if both NaN, false otherwise
        return a !== a && b !== b;
    };
},
/* ajv/lib/compile/util.js */ function _(require, module, exports) {
    module.exports = {
        copy: copy,
        checkDataType: checkDataType,
        checkDataTypes: checkDataTypes,
        coerceToTypes: coerceToTypes,
        toHash: toHash,
        getProperty: getProperty,
        escapeQuotes: escapeQuotes,
        equal: require(428) /* fast-deep-equal */,
        ucs2length: require(430) /* ./ucs2length */,
        varOccurences: varOccurences,
        varReplace: varReplace,
        cleanUpCode: cleanUpCode,
        finalCleanUpCode: finalCleanUpCode,
        schemaHasRules: schemaHasRules,
        schemaHasRulesExcept: schemaHasRulesExcept,
        schemaUnknownRules: schemaUnknownRules,
        toQuotedString: toQuotedString,
        getPathExpr: getPathExpr,
        getPath: getPath,
        getData: getData,
        unescapeFragment: unescapeFragment,
        unescapeJsonPointer: unescapeJsonPointer,
        escapeFragment: escapeFragment,
        escapeJsonPointer: escapeJsonPointer
    };
    function copy(o, to) {
        to = to || {};
        for (var key in o)
            to[key] = o[key];
        return to;
    }
    function checkDataType(dataType, data, negate) {
        var EQUAL = negate ? ' !== ' : ' === ', AND = negate ? ' || ' : ' && ', OK = negate ? '!' : '', NOT = negate ? '' : '!';
        switch (dataType) {
            case 'null': return data + EQUAL + 'null';
            case 'array': return OK + 'Array.isArray(' + data + ')';
            case 'object': return '(' + OK + data + AND +
                'typeof ' + data + EQUAL + '"object"' + AND +
                NOT + 'Array.isArray(' + data + '))';
            case 'integer': return '(typeof ' + data + EQUAL + '"number"' + AND +
                NOT + '(' + data + ' % 1)' +
                AND + data + EQUAL + data + ')';
            default: return 'typeof ' + data + EQUAL + '"' + dataType + '"';
        }
    }
    function checkDataTypes(dataTypes, data) {
        switch (dataTypes.length) {
            case 1: return checkDataType(dataTypes[0], data, true);
            default:
                var code = '';
                var types = toHash(dataTypes);
                if (types.array && types.object) {
                    code = types.null ? '(' : '(!' + data + ' || ';
                    code += 'typeof ' + data + ' !== "object")';
                    delete types.null;
                    delete types.array;
                    delete types.object;
                }
                if (types.number)
                    delete types.integer;
                for (var t in types)
                    code += (code ? ' && ' : '') + checkDataType(t, data, true);
                return code;
        }
    }
    var COERCE_TO_TYPES = toHash(['string', 'number', 'integer', 'boolean', 'null']);
    function coerceToTypes(optionCoerceTypes, dataTypes) {
        if (Array.isArray(dataTypes)) {
            var types = [];
            for (var i = 0; i < dataTypes.length; i++) {
                var t = dataTypes[i];
                if (COERCE_TO_TYPES[t])
                    types[types.length] = t;
                else if (optionCoerceTypes === 'array' && t === 'array')
                    types[types.length] = t;
            }
            if (types.length)
                return types;
        }
        else if (COERCE_TO_TYPES[dataTypes]) {
            return [dataTypes];
        }
        else if (optionCoerceTypes === 'array' && dataTypes === 'array') {
            return ['array'];
        }
    }
    function toHash(arr) {
        var hash = {};
        for (var i = 0; i < arr.length; i++)
            hash[arr[i]] = true;
        return hash;
    }
    var IDENTIFIER = /^[a-z$_][a-z$_0-9]*$/i;
    var SINGLE_QUOTE = /'|\\/g;
    function getProperty(key) {
        return typeof key == 'number'
            ? '[' + key + ']'
            : IDENTIFIER.test(key)
                ? '.' + key
                : "['" + escapeQuotes(key) + "']";
    }
    function escapeQuotes(str) {
        return str.replace(SINGLE_QUOTE, '\\$&')
            .replace(/\n/g, '\\n')
            .replace(/\r/g, '\\r')
            .replace(/\f/g, '\\f')
            .replace(/\t/g, '\\t');
    }
    function varOccurences(str, dataVar) {
        dataVar += '[^0-9]';
        var matches = str.match(new RegExp(dataVar, 'g'));
        return matches ? matches.length : 0;
    }
    function varReplace(str, dataVar, expr) {
        dataVar += '([^0-9])';
        expr = expr.replace(/\$/g, '$$$$');
        return str.replace(new RegExp(dataVar, 'g'), expr + '$1');
    }
    var EMPTY_ELSE = /else\s*{\s*}/g, EMPTY_IF_NO_ELSE = /if\s*\([^)]+\)\s*\{\s*\}(?!\s*else)/g, EMPTY_IF_WITH_ELSE = /if\s*\(([^)]+)\)\s*\{\s*\}\s*else(?!\s*if)/g;
    function cleanUpCode(out) {
        return out.replace(EMPTY_ELSE, '')
            .replace(EMPTY_IF_NO_ELSE, '')
            .replace(EMPTY_IF_WITH_ELSE, 'if (!($1))');
    }
    var ERRORS_REGEXP = /[^v.]errors/g, REMOVE_ERRORS = /var errors = 0;|var vErrors = null;|validate.errors = vErrors;/g, REMOVE_ERRORS_ASYNC = /var errors = 0;|var vErrors = null;/g, RETURN_VALID = 'return errors === 0;', RETURN_TRUE = 'validate.errors = null; return true;', RETURN_ASYNC = /if \(errors === 0\) return data;\s*else throw new ValidationError\(vErrors\);/, RETURN_DATA_ASYNC = 'return data;', ROOTDATA_REGEXP = /[^A-Za-z_$]rootData[^A-Za-z0-9_$]/g, REMOVE_ROOTDATA = /if \(rootData === undefined\) rootData = data;/;
    function finalCleanUpCode(out, async) {
        var matches = out.match(ERRORS_REGEXP);
        if (matches && matches.length == 2) {
            out = async
                ? out.replace(REMOVE_ERRORS_ASYNC, '')
                    .replace(RETURN_ASYNC, RETURN_DATA_ASYNC)
                : out.replace(REMOVE_ERRORS, '')
                    .replace(RETURN_VALID, RETURN_TRUE);
        }
        matches = out.match(ROOTDATA_REGEXP);
        if (!matches || matches.length !== 3)
            return out;
        return out.replace(REMOVE_ROOTDATA, '');
    }
    function schemaHasRules(schema, rules) {
        if (typeof schema == 'boolean')
            return !schema;
        for (var key in schema)
            if (rules[key])
                return true;
    }
    function schemaHasRulesExcept(schema, rules, exceptKeyword) {
        if (typeof schema == 'boolean')
            return !schema && exceptKeyword != 'not';
        for (var key in schema)
            if (key != exceptKeyword && rules[key])
                return true;
    }
    function schemaUnknownRules(schema, rules) {
        if (typeof schema == 'boolean')
            return;
        for (var key in schema)
            if (!rules[key])
                return key;
    }
    function toQuotedString(str) {
        return '\'' + escapeQuotes(str) + '\'';
    }
    function getPathExpr(currentPath, expr, jsonPointers, isNumber) {
        var path = jsonPointers // false by default
            ? '\'/\' + ' + expr + (isNumber ? '' : '.replace(/~/g, \'~0\').replace(/\\//g, \'~1\')')
            : (isNumber ? '\'[\' + ' + expr + ' + \']\'' : '\'[\\\'\' + ' + expr + ' + \'\\\']\'');
        return joinPaths(currentPath, path);
    }
    function getPath(currentPath, prop, jsonPointers) {
        var path = jsonPointers // false by default
            ? toQuotedString('/' + escapeJsonPointer(prop))
            : toQuotedString(getProperty(prop));
        return joinPaths(currentPath, path);
    }
    var JSON_POINTER = /^\/(?:[^~]|~0|~1)*$/;
    var RELATIVE_JSON_POINTER = /^([0-9]+)(#|\/(?:[^~]|~0|~1)*)?$/;
    function getData($data, lvl, paths) {
        var up, jsonPointer, data, matches;
        if ($data === '')
            return 'rootData';
        if ($data[0] == '/') {
            if (!JSON_POINTER.test($data))
                throw new Error('Invalid JSON-pointer: ' + $data);
            jsonPointer = $data;
            data = 'rootData';
        }
        else {
            matches = $data.match(RELATIVE_JSON_POINTER);
            if (!matches)
                throw new Error('Invalid JSON-pointer: ' + $data);
            up = +matches[1];
            jsonPointer = matches[2];
            if (jsonPointer == '#') {
                if (up >= lvl)
                    throw new Error('Cannot access property/index ' + up + ' levels up, current level is ' + lvl);
                return paths[lvl - up];
            }
            if (up > lvl)
                throw new Error('Cannot access data ' + up + ' levels up, current level is ' + lvl);
            data = 'data' + ((lvl - up) || '');
            if (!jsonPointer)
                return data;
        }
        var expr = data;
        var segments = jsonPointer.split('/');
        for (var i = 0; i < segments.length; i++) {
            var segment = segments[i];
            if (segment) {
                data += getProperty(unescapeJsonPointer(segment));
                expr += ' && ' + data;
            }
        }
        return expr;
    }
    function joinPaths(a, b) {
        if (a == '""')
            return b;
        return (a + ' + ' + b).replace(/' \+ '/g, '');
    }
    function unescapeFragment(str) {
        return unescapeJsonPointer(decodeURIComponent(str));
    }
    function escapeFragment(str) {
        return encodeURIComponent(escapeJsonPointer(str));
    }
    function escapeJsonPointer(str) {
        return str.replace(/~/g, '~0').replace(/\//g, '~1');
    }
    function unescapeJsonPointer(str) {
        return str.replace(/~1/g, '/').replace(/~0/g, '~');
    }
},
/* ajv/lib/compile/ucs2length.js */ function _(require, module, exports) {
    // https://mathiasbynens.be/notes/javascript-encoding
    // https://github.com/bestiejs/punycode.js - punycode.ucs2.decode
    module.exports = function ucs2length(str) {
        var length = 0, len = str.length, pos = 0, value;
        while (pos < len) {
            length++;
            value = str.charCodeAt(pos++);
            if (value >= 0xD800 && value <= 0xDBFF && pos < len) {
                // high surrogate, and there is a next character
                value = str.charCodeAt(pos);
                if ((value & 0xFC00) == 0xDC00)
                    pos++; // low surrogate
            }
        }
        return length;
    };
},
/* ajv/lib/compile/schema_obj.js */ function _(require, module, exports) {
    var util = require(429) /* ./util */;
    module.exports = SchemaObject;
    function SchemaObject(obj) {
        util.copy(obj, this);
    }
},
/* json-schema-traverse/index.js */ function _(require, module, exports) {
    var traverse = module.exports = function (schema, opts, cb) {
        // Legacy support for v0.3.1 and earlier.
        if (typeof opts == 'function') {
            cb = opts;
            opts = {};
        }
        cb = opts.cb || cb;
        var pre = (typeof cb == 'function') ? cb : cb.pre || function () { };
        var post = cb.post || function () { };
        _traverse(opts, pre, post, schema, '', schema);
    };
    traverse.keywords = {
        additionalItems: true,
        items: true,
        contains: true,
        additionalProperties: true,
        propertyNames: true,
        not: true
    };
    traverse.arrayKeywords = {
        items: true,
        allOf: true,
        anyOf: true,
        oneOf: true
    };
    traverse.propsKeywords = {
        definitions: true,
        properties: true,
        patternProperties: true,
        dependencies: true
    };
    traverse.skipKeywords = {
        default: true,
        enum: true,
        const: true,
        required: true,
        maximum: true,
        minimum: true,
        exclusiveMaximum: true,
        exclusiveMinimum: true,
        multipleOf: true,
        maxLength: true,
        minLength: true,
        pattern: true,
        format: true,
        maxItems: true,
        minItems: true,
        uniqueItems: true,
        maxProperties: true,
        minProperties: true
    };
    function _traverse(opts, pre, post, schema, jsonPtr, rootSchema, parentJsonPtr, parentKeyword, parentSchema, keyIndex) {
        if (schema && typeof schema == 'object' && !Array.isArray(schema)) {
            pre(schema, jsonPtr, rootSchema, parentJsonPtr, parentKeyword, parentSchema, keyIndex);
            for (var key in schema) {
                var sch = schema[key];
                if (Array.isArray(sch)) {
                    if (key in traverse.arrayKeywords) {
                        for (var i = 0; i < sch.length; i++)
                            _traverse(opts, pre, post, sch[i], jsonPtr + '/' + key + '/' + i, rootSchema, jsonPtr, key, schema, i);
                    }
                }
                else if (key in traverse.propsKeywords) {
                    if (sch && typeof sch == 'object') {
                        for (var prop in sch)
                            _traverse(opts, pre, post, sch[prop], jsonPtr + '/' + key + '/' + escapeJsonPtr(prop), rootSchema, jsonPtr, key, schema, prop);
                    }
                }
                else if (key in traverse.keywords || (opts.allKeys && !(key in traverse.skipKeywords))) {
                    _traverse(opts, pre, post, sch, jsonPtr + '/' + key, rootSchema, jsonPtr, key, schema);
                }
            }
            post(schema, jsonPtr, rootSchema, parentJsonPtr, parentKeyword, parentSchema, keyIndex);
        }
    }
    function escapeJsonPtr(str) {
        return str.replace(/~/g, '~0').replace(/\//g, '~1');
    }
},
/* ajv/lib/compile/error_classes.js */ function _(require, module, exports) {
    var resolve = require(426) /* ./resolve */;
    module.exports = {
        Validation: errorSubclass(ValidationError),
        MissingRef: errorSubclass(MissingRefError)
    };
    function ValidationError(errors) {
        this.message = 'validation failed';
        this.errors = errors;
        this.ajv = this.validation = true;
    }
    MissingRefError.message = function (baseId, ref) {
        return 'can\'t resolve reference ' + ref + ' from id ' + baseId;
    };
    function MissingRefError(baseId, ref, message) {
        this.message = message || MissingRefError.message(baseId, ref);
        this.missingRef = resolve.url(baseId, ref);
        this.missingSchema = resolve.normalizeId(resolve.fullPath(this.missingRef));
    }
    function errorSubclass(Subclass) {
        Subclass.prototype = Object.create(Error.prototype);
        Subclass.prototype.constructor = Subclass;
        return Subclass;
    }
},
/* fast-json-stable-stringify/index.js */ function _(require, module, exports) {
    module.exports = function (data, opts) {
        if (!opts)
            opts = {};
        if (typeof opts === 'function')
            opts = { cmp: opts };
        var cycles = (typeof opts.cycles === 'boolean') ? opts.cycles : false;
        var cmp = opts.cmp && (function (f) {
            return function (node) {
                return function (a, b) {
                    var aobj = { key: a, value: node[a] };
                    var bobj = { key: b, value: node[b] };
                    return f(aobj, bobj);
                };
            };
        })(opts.cmp);
        var seen = [];
        return (function stringify(node) {
            if (node && node.toJSON && typeof node.toJSON === 'function') {
                node = node.toJSON();
            }
            if (node === undefined)
                return;
            if (typeof node == 'number')
                return isFinite(node) ? '' + node : 'null';
            if (typeof node !== 'object')
                return JSON.stringify(node);
            var i, out;
            if (Array.isArray(node)) {
                out = '[';
                for (i = 0; i < node.length; i++) {
                    if (i)
                        out += ',';
                    out += stringify(node[i]) || 'null';
                }
                return out + ']';
            }
            if (node === null)
                return 'null';
            if (seen.indexOf(node) !== -1) {
                if (cycles)
                    return JSON.stringify('__cycle__');
                throw new TypeError('Converting circular structure to JSON');
            }
            var seenIndex = seen.push(node) - 1;
            var keys = Object.keys(node).sort(cmp && cmp(node));
            out = '';
            for (i = 0; i < keys.length; i++) {
                var key = keys[i];
                var value = stringify(node[key]);
                if (!value)
                    continue;
                if (out)
                    out += ',';
                out += JSON.stringify(key) + ':' + value;
            }
            seen.splice(seenIndex, 1);
            return '{' + out + '}';
        })(data);
    };
},
/* ajv/lib/dotjs/validate.js */ function _(require, module, exports) {
    module.exports = function generate_validate(it, $keyword, $ruleType) {
        var out = '';
        var $async = it.schema.$async === true, $refKeywords = it.util.schemaHasRulesExcept(it.schema, it.RULES.all, '$ref'), $id = it.self._getId(it.schema);
        if (it.opts.strictKeywords) {
            var $unknownKwd = it.util.schemaUnknownRules(it.schema, it.RULES.keywords);
            if ($unknownKwd) {
                var $keywordsMsg = 'unknown keyword: ' + $unknownKwd;
                if (it.opts.strictKeywords === 'log')
                    it.logger.warn($keywordsMsg);
                else
                    throw new Error($keywordsMsg);
            }
        }
        if (it.isTop) {
            out += ' var validate = ';
            if ($async) {
                it.async = true;
                out += 'async ';
            }
            out += 'function(data, dataPath, parentData, parentDataProperty, rootData) { \'use strict\'; ';
            if ($id && (it.opts.sourceCode || it.opts.processCode)) {
                out += ' ' + ('/\*# sourceURL=' + $id + ' */') + ' ';
            }
        }
        if (typeof it.schema == 'boolean' || !($refKeywords || it.schema.$ref)) {
            var $keyword = 'false schema';
            var $lvl = it.level;
            var $dataLvl = it.dataLevel;
            var $schema = it.schema[$keyword];
            var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
            var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
            var $breakOnError = !it.opts.allErrors;
            var $errorKeyword;
            var $data = 'data' + ($dataLvl || '');
            var $valid = 'valid' + $lvl;
            if (it.schema === false) {
                if (it.isTop) {
                    $breakOnError = true;
                }
                else {
                    out += ' var ' + ($valid) + ' = false; ';
                }
                var $$outStack = $$outStack || [];
                $$outStack.push(out);
                out = ''; /* istanbul ignore else */
                if (it.createErrors !== false) {
                    out += ' { keyword: \'' + ($errorKeyword || 'false schema') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: {} ';
                    if (it.opts.messages !== false) {
                        out += ' , message: \'boolean schema is false\' ';
                    }
                    if (it.opts.verbose) {
                        out += ' , schema: false , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                    }
                    out += ' } ';
                }
                else {
                    out += ' {} ';
                }
                var __err = out;
                out = $$outStack.pop();
                if (!it.compositeRule && $breakOnError) {
                    /* istanbul ignore if */
                    if (it.async) {
                        out += ' throw new ValidationError([' + (__err) + ']); ';
                    }
                    else {
                        out += ' validate.errors = [' + (__err) + ']; return false; ';
                    }
                }
                else {
                    out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                }
            }
            else {
                if (it.isTop) {
                    if ($async) {
                        out += ' return data; ';
                    }
                    else {
                        out += ' validate.errors = null; return true; ';
                    }
                }
                else {
                    out += ' var ' + ($valid) + ' = true; ';
                }
            }
            if (it.isTop) {
                out += ' }; return validate; ';
            }
            return out;
        }
        if (it.isTop) {
            var $top = it.isTop, $lvl = it.level = 0, $dataLvl = it.dataLevel = 0, $data = 'data';
            it.rootId = it.resolve.fullPath(it.self._getId(it.root.schema));
            it.baseId = it.baseId || it.rootId;
            delete it.isTop;
            it.dataPathArr = [undefined];
            if (it.schema.default !== undefined && it.opts.useDefaults && it.opts.strictDefaults) {
                var $defaultMsg = 'default is ignored in the schema root';
                if (it.opts.strictDefaults === 'log')
                    it.logger.warn($defaultMsg);
                else
                    throw new Error($defaultMsg);
            }
            out += ' var vErrors = null; ';
            out += ' var errors = 0;     ';
            out += ' if (rootData === undefined) rootData = data; ';
        }
        else {
            var $lvl = it.level, $dataLvl = it.dataLevel, $data = 'data' + ($dataLvl || '');
            if ($id)
                it.baseId = it.resolve.url(it.baseId, $id);
            if ($async && !it.async)
                throw new Error('async schema in sync schema');
            out += ' var errs_' + ($lvl) + ' = errors;';
        }
        var $valid = 'valid' + $lvl, $breakOnError = !it.opts.allErrors, $closingBraces1 = '', $closingBraces2 = '';
        var $errorKeyword;
        var $typeSchema = it.schema.type, $typeIsArray = Array.isArray($typeSchema);
        if ($typeSchema && it.opts.nullable && it.schema.nullable === true) {
            if ($typeIsArray) {
                if ($typeSchema.indexOf('null') == -1)
                    $typeSchema = $typeSchema.concat('null');
            }
            else if ($typeSchema != 'null') {
                $typeSchema = [$typeSchema, 'null'];
                $typeIsArray = true;
            }
        }
        if ($typeIsArray && $typeSchema.length == 1) {
            $typeSchema = $typeSchema[0];
            $typeIsArray = false;
        }
        if (it.schema.$ref && $refKeywords) {
            if (it.opts.extendRefs == 'fail') {
                throw new Error('$ref: validation keywords used in schema at path "' + it.errSchemaPath + '" (see option extendRefs)');
            }
            else if (it.opts.extendRefs !== true) {
                $refKeywords = false;
                it.logger.warn('$ref: keywords ignored in schema at path "' + it.errSchemaPath + '"');
            }
        }
        if (it.schema.$comment && it.opts.$comment) {
            out += ' ' + (it.RULES.all.$comment.code(it, '$comment'));
        }
        if ($typeSchema) {
            if (it.opts.coerceTypes) {
                var $coerceToTypes = it.util.coerceToTypes(it.opts.coerceTypes, $typeSchema);
            }
            var $rulesGroup = it.RULES.types[$typeSchema];
            if ($coerceToTypes || $typeIsArray || $rulesGroup === true || ($rulesGroup && !$shouldUseGroup($rulesGroup))) {
                var $schemaPath = it.schemaPath + '.type', $errSchemaPath = it.errSchemaPath + '/type';
                var $schemaPath = it.schemaPath + '.type', $errSchemaPath = it.errSchemaPath + '/type', $method = $typeIsArray ? 'checkDataTypes' : 'checkDataType';
                out += ' if (' + (it.util[$method]($typeSchema, $data, true)) + ') { ';
                if ($coerceToTypes) {
                    var $dataType = 'dataType' + $lvl, $coerced = 'coerced' + $lvl;
                    out += ' var ' + ($dataType) + ' = typeof ' + ($data) + '; ';
                    if (it.opts.coerceTypes == 'array') {
                        out += ' if (' + ($dataType) + ' == \'object\' && Array.isArray(' + ($data) + ')) ' + ($dataType) + ' = \'array\'; ';
                    }
                    out += ' var ' + ($coerced) + ' = undefined; ';
                    var $bracesCoercion = '';
                    var arr1 = $coerceToTypes;
                    if (arr1) {
                        var $type, $i = -1, l1 = arr1.length - 1;
                        while ($i < l1) {
                            $type = arr1[$i += 1];
                            if ($i) {
                                out += ' if (' + ($coerced) + ' === undefined) { ';
                                $bracesCoercion += '}';
                            }
                            if (it.opts.coerceTypes == 'array' && $type != 'array') {
                                out += ' if (' + ($dataType) + ' == \'array\' && ' + ($data) + '.length == 1) { ' + ($coerced) + ' = ' + ($data) + ' = ' + ($data) + '[0]; ' + ($dataType) + ' = typeof ' + ($data) + ';  } ';
                            }
                            if ($type == 'string') {
                                out += ' if (' + ($dataType) + ' == \'number\' || ' + ($dataType) + ' == \'boolean\') ' + ($coerced) + ' = \'\' + ' + ($data) + '; else if (' + ($data) + ' === null) ' + ($coerced) + ' = \'\'; ';
                            }
                            else if ($type == 'number' || $type == 'integer') {
                                out += ' if (' + ($dataType) + ' == \'boolean\' || ' + ($data) + ' === null || (' + ($dataType) + ' == \'string\' && ' + ($data) + ' && ' + ($data) + ' == +' + ($data) + ' ';
                                if ($type == 'integer') {
                                    out += ' && !(' + ($data) + ' % 1)';
                                }
                                out += ')) ' + ($coerced) + ' = +' + ($data) + '; ';
                            }
                            else if ($type == 'boolean') {
                                out += ' if (' + ($data) + ' === \'false\' || ' + ($data) + ' === 0 || ' + ($data) + ' === null) ' + ($coerced) + ' = false; else if (' + ($data) + ' === \'true\' || ' + ($data) + ' === 1) ' + ($coerced) + ' = true; ';
                            }
                            else if ($type == 'null') {
                                out += ' if (' + ($data) + ' === \'\' || ' + ($data) + ' === 0 || ' + ($data) + ' === false) ' + ($coerced) + ' = null; ';
                            }
                            else if (it.opts.coerceTypes == 'array' && $type == 'array') {
                                out += ' if (' + ($dataType) + ' == \'string\' || ' + ($dataType) + ' == \'number\' || ' + ($dataType) + ' == \'boolean\' || ' + ($data) + ' == null) ' + ($coerced) + ' = [' + ($data) + ']; ';
                            }
                        }
                    }
                    out += ' ' + ($bracesCoercion) + ' if (' + ($coerced) + ' === undefined) {   ';
                    var $$outStack = $$outStack || [];
                    $$outStack.push(out);
                    out = ''; /* istanbul ignore else */
                    if (it.createErrors !== false) {
                        out += ' { keyword: \'' + ($errorKeyword || 'type') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { type: \'';
                        if ($typeIsArray) {
                            out += '' + ($typeSchema.join(","));
                        }
                        else {
                            out += '' + ($typeSchema);
                        }
                        out += '\' } ';
                        if (it.opts.messages !== false) {
                            out += ' , message: \'should be ';
                            if ($typeIsArray) {
                                out += '' + ($typeSchema.join(","));
                            }
                            else {
                                out += '' + ($typeSchema);
                            }
                            out += '\' ';
                        }
                        if (it.opts.verbose) {
                            out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                        }
                        out += ' } ';
                    }
                    else {
                        out += ' {} ';
                    }
                    var __err = out;
                    out = $$outStack.pop();
                    if (!it.compositeRule && $breakOnError) {
                        /* istanbul ignore if */
                        if (it.async) {
                            out += ' throw new ValidationError([' + (__err) + ']); ';
                        }
                        else {
                            out += ' validate.errors = [' + (__err) + ']; return false; ';
                        }
                    }
                    else {
                        out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                    }
                    out += ' } else {  ';
                    var $parentData = $dataLvl ? 'data' + (($dataLvl - 1) || '') : 'parentData', $parentDataProperty = $dataLvl ? it.dataPathArr[$dataLvl] : 'parentDataProperty';
                    out += ' ' + ($data) + ' = ' + ($coerced) + '; ';
                    if (!$dataLvl) {
                        out += 'if (' + ($parentData) + ' !== undefined)';
                    }
                    out += ' ' + ($parentData) + '[' + ($parentDataProperty) + '] = ' + ($coerced) + '; } ';
                }
                else {
                    var $$outStack = $$outStack || [];
                    $$outStack.push(out);
                    out = ''; /* istanbul ignore else */
                    if (it.createErrors !== false) {
                        out += ' { keyword: \'' + ($errorKeyword || 'type') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { type: \'';
                        if ($typeIsArray) {
                            out += '' + ($typeSchema.join(","));
                        }
                        else {
                            out += '' + ($typeSchema);
                        }
                        out += '\' } ';
                        if (it.opts.messages !== false) {
                            out += ' , message: \'should be ';
                            if ($typeIsArray) {
                                out += '' + ($typeSchema.join(","));
                            }
                            else {
                                out += '' + ($typeSchema);
                            }
                            out += '\' ';
                        }
                        if (it.opts.verbose) {
                            out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                        }
                        out += ' } ';
                    }
                    else {
                        out += ' {} ';
                    }
                    var __err = out;
                    out = $$outStack.pop();
                    if (!it.compositeRule && $breakOnError) {
                        /* istanbul ignore if */
                        if (it.async) {
                            out += ' throw new ValidationError([' + (__err) + ']); ';
                        }
                        else {
                            out += ' validate.errors = [' + (__err) + ']; return false; ';
                        }
                    }
                    else {
                        out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                    }
                }
                out += ' } ';
            }
        }
        if (it.schema.$ref && !$refKeywords) {
            out += ' ' + (it.RULES.all.$ref.code(it, '$ref')) + ' ';
            if ($breakOnError) {
                out += ' } if (errors === ';
                if ($top) {
                    out += '0';
                }
                else {
                    out += 'errs_' + ($lvl);
                }
                out += ') { ';
                $closingBraces2 += '}';
            }
        }
        else {
            var arr2 = it.RULES;
            if (arr2) {
                var $rulesGroup, i2 = -1, l2 = arr2.length - 1;
                while (i2 < l2) {
                    $rulesGroup = arr2[i2 += 1];
                    if ($shouldUseGroup($rulesGroup)) {
                        if ($rulesGroup.type) {
                            out += ' if (' + (it.util.checkDataType($rulesGroup.type, $data)) + ') { ';
                        }
                        if (it.opts.useDefaults) {
                            if ($rulesGroup.type == 'object' && it.schema.properties) {
                                var $schema = it.schema.properties, $schemaKeys = Object.keys($schema);
                                var arr3 = $schemaKeys;
                                if (arr3) {
                                    var $propertyKey, i3 = -1, l3 = arr3.length - 1;
                                    while (i3 < l3) {
                                        $propertyKey = arr3[i3 += 1];
                                        var $sch = $schema[$propertyKey];
                                        if ($sch.default !== undefined) {
                                            var $passData = $data + it.util.getProperty($propertyKey);
                                            if (it.compositeRule) {
                                                if (it.opts.strictDefaults) {
                                                    var $defaultMsg = 'default is ignored for: ' + $passData;
                                                    if (it.opts.strictDefaults === 'log')
                                                        it.logger.warn($defaultMsg);
                                                    else
                                                        throw new Error($defaultMsg);
                                                }
                                            }
                                            else {
                                                out += ' if (' + ($passData) + ' === undefined ';
                                                if (it.opts.useDefaults == 'empty') {
                                                    out += ' || ' + ($passData) + ' === null || ' + ($passData) + ' === \'\' ';
                                                }
                                                out += ' ) ' + ($passData) + ' = ';
                                                if (it.opts.useDefaults == 'shared') {
                                                    out += ' ' + (it.useDefault($sch.default)) + ' ';
                                                }
                                                else {
                                                    out += ' ' + (JSON.stringify($sch.default)) + ' ';
                                                }
                                                out += '; ';
                                            }
                                        }
                                    }
                                }
                            }
                            else if ($rulesGroup.type == 'array' && Array.isArray(it.schema.items)) {
                                var arr4 = it.schema.items;
                                if (arr4) {
                                    var $sch, $i = -1, l4 = arr4.length - 1;
                                    while ($i < l4) {
                                        $sch = arr4[$i += 1];
                                        if ($sch.default !== undefined) {
                                            var $passData = $data + '[' + $i + ']';
                                            if (it.compositeRule) {
                                                if (it.opts.strictDefaults) {
                                                    var $defaultMsg = 'default is ignored for: ' + $passData;
                                                    if (it.opts.strictDefaults === 'log')
                                                        it.logger.warn($defaultMsg);
                                                    else
                                                        throw new Error($defaultMsg);
                                                }
                                            }
                                            else {
                                                out += ' if (' + ($passData) + ' === undefined ';
                                                if (it.opts.useDefaults == 'empty') {
                                                    out += ' || ' + ($passData) + ' === null || ' + ($passData) + ' === \'\' ';
                                                }
                                                out += ' ) ' + ($passData) + ' = ';
                                                if (it.opts.useDefaults == 'shared') {
                                                    out += ' ' + (it.useDefault($sch.default)) + ' ';
                                                }
                                                else {
                                                    out += ' ' + (JSON.stringify($sch.default)) + ' ';
                                                }
                                                out += '; ';
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        var arr5 = $rulesGroup.rules;
                        if (arr5) {
                            var $rule, i5 = -1, l5 = arr5.length - 1;
                            while (i5 < l5) {
                                $rule = arr5[i5 += 1];
                                if ($shouldUseRule($rule)) {
                                    var $code = $rule.code(it, $rule.keyword, $rulesGroup.type);
                                    if ($code) {
                                        out += ' ' + ($code) + ' ';
                                        if ($breakOnError) {
                                            $closingBraces1 += '}';
                                        }
                                    }
                                }
                            }
                        }
                        if ($breakOnError) {
                            out += ' ' + ($closingBraces1) + ' ';
                            $closingBraces1 = '';
                        }
                        if ($rulesGroup.type) {
                            out += ' } ';
                            if ($typeSchema && $typeSchema === $rulesGroup.type && !$coerceToTypes) {
                                out += ' else { ';
                                var $schemaPath = it.schemaPath + '.type', $errSchemaPath = it.errSchemaPath + '/type';
                                var $$outStack = $$outStack || [];
                                $$outStack.push(out);
                                out = ''; /* istanbul ignore else */
                                if (it.createErrors !== false) {
                                    out += ' { keyword: \'' + ($errorKeyword || 'type') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { type: \'';
                                    if ($typeIsArray) {
                                        out += '' + ($typeSchema.join(","));
                                    }
                                    else {
                                        out += '' + ($typeSchema);
                                    }
                                    out += '\' } ';
                                    if (it.opts.messages !== false) {
                                        out += ' , message: \'should be ';
                                        if ($typeIsArray) {
                                            out += '' + ($typeSchema.join(","));
                                        }
                                        else {
                                            out += '' + ($typeSchema);
                                        }
                                        out += '\' ';
                                    }
                                    if (it.opts.verbose) {
                                        out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                                    }
                                    out += ' } ';
                                }
                                else {
                                    out += ' {} ';
                                }
                                var __err = out;
                                out = $$outStack.pop();
                                if (!it.compositeRule && $breakOnError) {
                                    /* istanbul ignore if */
                                    if (it.async) {
                                        out += ' throw new ValidationError([' + (__err) + ']); ';
                                    }
                                    else {
                                        out += ' validate.errors = [' + (__err) + ']; return false; ';
                                    }
                                }
                                else {
                                    out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                                }
                                out += ' } ';
                            }
                        }
                        if ($breakOnError) {
                            out += ' if (errors === ';
                            if ($top) {
                                out += '0';
                            }
                            else {
                                out += 'errs_' + ($lvl);
                            }
                            out += ') { ';
                            $closingBraces2 += '}';
                        }
                    }
                }
            }
        }
        if ($breakOnError) {
            out += ' ' + ($closingBraces2) + ' ';
        }
        if ($top) {
            if ($async) {
                out += ' if (errors === 0) return data;           ';
                out += ' else throw new ValidationError(vErrors); ';
            }
            else {
                out += ' validate.errors = vErrors; ';
                out += ' return errors === 0;       ';
            }
            out += ' }; return validate;';
        }
        else {
            out += ' var ' + ($valid) + ' = errors === errs_' + ($lvl) + ';';
        }
        out = it.util.cleanUpCode(out);
        if ($top) {
            out = it.util.finalCleanUpCode(out, $async);
        }
        function $shouldUseGroup($rulesGroup) {
            var rules = $rulesGroup.rules;
            for (var i = 0; i < rules.length; i++)
                if ($shouldUseRule(rules[i]))
                    return true;
        }
        function $shouldUseRule($rule) {
            return it.schema[$rule.keyword] !== undefined || ($rule.implements && $ruleImplementsSomeKeyword($rule));
        }
        function $ruleImplementsSomeKeyword($rule) {
            var impl = $rule.implements;
            for (var i = 0; i < impl.length; i++)
                if (it.schema[impl[i]] !== undefined)
                    return true;
        }
        return out;
    };
},
/* ajv/lib/cache.js */ function _(require, module, exports) {
    var Cache = module.exports = function Cache() {
        this._cache = {};
    };
    Cache.prototype.put = function Cache_put(key, value) {
        this._cache[key] = value;
    };
    Cache.prototype.get = function Cache_get(key) {
        return this._cache[key];
    };
    Cache.prototype.del = function Cache_del(key) {
        delete this._cache[key];
    };
    Cache.prototype.clear = function Cache_clear() {
        this._cache = {};
    };
},
/* ajv/lib/compile/formats.js */ function _(require, module, exports) {
    var util = require(429) /* ./util */;
    var DATE = /^(\d\d\d\d)-(\d\d)-(\d\d)$/;
    var DAYS = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    var TIME = /^(\d\d):(\d\d):(\d\d)(\.\d+)?(z|[+-]\d\d(?::?\d\d)?)?$/i;
    var HOSTNAME = /^(?=.{1,253}\.?$)[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?(?:\.[a-z0-9](?:[-0-9a-z]{0,61}[0-9a-z])?)*\.?$/i;
    var URI = /^(?:[a-z][a-z0-9+\-.]*:)(?:\/?\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:]|%[0-9a-f]{2})*@)?(?:\[(?:(?:(?:(?:[0-9a-f]{1,4}:){6}|::(?:[0-9a-f]{1,4}:){5}|(?:[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){4}|(?:(?:[0-9a-f]{1,4}:){0,1}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){3}|(?:(?:[0-9a-f]{1,4}:){0,2}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){2}|(?:(?:[0-9a-f]{1,4}:){0,3}[0-9a-f]{1,4})?::[0-9a-f]{1,4}:|(?:(?:[0-9a-f]{1,4}:){0,4}[0-9a-f]{1,4})?::)(?:[0-9a-f]{1,4}:[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))|(?:(?:[0-9a-f]{1,4}:){0,5}[0-9a-f]{1,4})?::[0-9a-f]{1,4}|(?:(?:[0-9a-f]{1,4}:){0,6}[0-9a-f]{1,4})?::)|[Vv][0-9a-f]+\.[a-z0-9\-._~!$&'()*+,;=:]+)\]|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|(?:[a-z0-9\-._~!$&'()*+,;=]|%[0-9a-f]{2})*)(?::\d*)?(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*|\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*)?|(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'()*+,;=:@]|%[0-9a-f]{2})*)*)(?:\?(?:[a-z0-9\-._~!$&'()*+,;=:@/?]|%[0-9a-f]{2})*)?(?:#(?:[a-z0-9\-._~!$&'()*+,;=:@/?]|%[0-9a-f]{2})*)?$/i;
    var URIREF = /^(?:[a-z][a-z0-9+\-.]*:)?(?:\/?\/(?:(?:[a-z0-9\-._~!$&'()*+,;=:]|%[0-9a-f]{2})*@)?(?:\[(?:(?:(?:(?:[0-9a-f]{1,4}:){6}|::(?:[0-9a-f]{1,4}:){5}|(?:[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){4}|(?:(?:[0-9a-f]{1,4}:){0,1}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){3}|(?:(?:[0-9a-f]{1,4}:){0,2}[0-9a-f]{1,4})?::(?:[0-9a-f]{1,4}:){2}|(?:(?:[0-9a-f]{1,4}:){0,3}[0-9a-f]{1,4})?::[0-9a-f]{1,4}:|(?:(?:[0-9a-f]{1,4}:){0,4}[0-9a-f]{1,4})?::)(?:[0-9a-f]{1,4}:[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?))|(?:(?:[0-9a-f]{1,4}:){0,5}[0-9a-f]{1,4})?::[0-9a-f]{1,4}|(?:(?:[0-9a-f]{1,4}:){0,6}[0-9a-f]{1,4})?::)|[Vv][0-9a-f]+\.[a-z0-9\-._~!$&'()*+,;=:]+)\]|(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|(?:[a-z0-9\-._~!$&'"()*+,;=]|%[0-9a-f]{2})*)(?::\d*)?(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*|\/(?:(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*)?|(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})+(?:\/(?:[a-z0-9\-._~!$&'"()*+,;=:@]|%[0-9a-f]{2})*)*)?(?:\?(?:[a-z0-9\-._~!$&'"()*+,;=:@/?]|%[0-9a-f]{2})*)?(?:#(?:[a-z0-9\-._~!$&'"()*+,;=:@/?]|%[0-9a-f]{2})*)?$/i;
    // uri-template: https://tools.ietf.org/html/rfc6570
    var URITEMPLATE = /^(?:(?:[^\x00-\x20"'<>%\\^`{|}]|%[0-9a-f]{2})|\{[+#./;?&=,!@|]?(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?(?:,(?:[a-z0-9_]|%[0-9a-f]{2})+(?::[1-9][0-9]{0,3}|\*)?)*\})*$/i;
    // For the source: https://gist.github.com/dperini/729294
    // For test cases: https://mathiasbynens.be/demo/url-regex
    // @todo Delete current URL in favour of the commented out URL rule when this issue is fixed https://github.com/eslint/eslint/issues/7983.
    // var URL = /^(?:(?:https?|ftp):\/\/)(?:\S+(?::\S*)?@)?(?:(?!10(?:\.\d{1,3}){3})(?!127(?:\.\d{1,3}){3})(?!169\.254(?:\.\d{1,3}){2})(?!192\.168(?:\.\d{1,3}){2})(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))|(?:(?:[a-z\u{00a1}-\u{ffff}0-9]+-?)*[a-z\u{00a1}-\u{ffff}0-9]+)(?:\.(?:[a-z\u{00a1}-\u{ffff}0-9]+-?)*[a-z\u{00a1}-\u{ffff}0-9]+)*(?:\.(?:[a-z\u{00a1}-\u{ffff}]{2,})))(?::\d{2,5})?(?:\/[^\s]*)?$/iu;
    var URL = /^(?:(?:http[s\u017F]?|ftp):\/\/)(?:(?:[\0-\x08\x0E-\x1F!-\x9F\xA1-\u167F\u1681-\u1FFF\u200B-\u2027\u202A-\u202E\u2030-\u205E\u2060-\u2FFF\u3001-\uD7FF\uE000-\uFEFE\uFF00-\uFFFF]|[\uD800-\uDBFF][\uDC00-\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+(?::(?:[\0-\x08\x0E-\x1F!-\x9F\xA1-\u167F\u1681-\u1FFF\u200B-\u2027\u202A-\u202E\u2030-\u205E\u2060-\u2FFF\u3001-\uD7FF\uE000-\uFEFE\uFF00-\uFFFF]|[\uD800-\uDBFF][\uDC00-\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])*)?@)?(?:(?!10(?:\.[0-9]{1,3}){3})(?!127(?:\.[0-9]{1,3}){3})(?!169\.254(?:\.[0-9]{1,3}){2})(?!192\.168(?:\.[0-9]{1,3}){2})(?!172\.(?:1[6-9]|2[0-9]|3[01])(?:\.[0-9]{1,3}){2})(?:[1-9][0-9]?|1[0-9][0-9]|2[01][0-9]|22[0-3])(?:\.(?:1?[0-9]{1,2}|2[0-4][0-9]|25[0-5])){2}(?:\.(?:[1-9][0-9]?|1[0-9][0-9]|2[0-4][0-9]|25[0-4]))|(?:(?:(?:[0-9KSa-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+-?)*(?:[0-9KSa-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+)(?:\.(?:(?:[0-9KSa-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+-?)*(?:[0-9KSa-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])+)*(?:\.(?:(?:[KSa-z\xA1-\uD7FF\uE000-\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]){2,})))(?::[0-9]{2,5})?(?:\/(?:[\0-\x08\x0E-\x1F!-\x9F\xA1-\u167F\u1681-\u1FFF\u200B-\u2027\u202A-\u202E\u2030-\u205E\u2060-\u2FFF\u3001-\uD7FF\uE000-\uFEFE\uFF00-\uFFFF]|[\uD800-\uDBFF][\uDC00-\uDFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF])*)?$/i;
    var UUID = /^(?:urn:uuid:)?[0-9a-f]{8}-(?:[0-9a-f]{4}-){3}[0-9a-f]{12}$/i;
    var JSON_POINTER = /^(?:\/(?:[^~/]|~0|~1)*)*$/;
    var JSON_POINTER_URI_FRAGMENT = /^#(?:\/(?:[a-z0-9_\-.!$&'()*+,;:=@]|%[0-9a-f]{2}|~0|~1)*)*$/i;
    var RELATIVE_JSON_POINTER = /^(?:0|[1-9][0-9]*)(?:#|(?:\/(?:[^~/]|~0|~1)*)*)$/;
    module.exports = formats;
    function formats(mode) {
        mode = mode == 'full' ? 'full' : 'fast';
        return util.copy(formats[mode]);
    }
    formats.fast = {
        // date: http://tools.ietf.org/html/rfc3339#section-5.6
        date: /^\d\d\d\d-[0-1]\d-[0-3]\d$/,
        // date-time: http://tools.ietf.org/html/rfc3339#section-5.6
        time: /^(?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d(?::?\d\d)?)?$/i,
        'date-time': /^\d\d\d\d-[0-1]\d-[0-3]\d[t\s](?:[0-2]\d:[0-5]\d:[0-5]\d|23:59:60)(?:\.\d+)?(?:z|[+-]\d\d(?::?\d\d)?)$/i,
        // uri: https://github.com/mafintosh/is-my-json-valid/blob/master/formats.js
        uri: /^(?:[a-z][a-z0-9+-.]*:)(?:\/?\/)?[^\s]*$/i,
        'uri-reference': /^(?:(?:[a-z][a-z0-9+-.]*:)?\/?\/)?(?:[^\\\s#][^\s#]*)?(?:#[^\\\s]*)?$/i,
        'uri-template': URITEMPLATE,
        url: URL,
        // email (sources from jsen validator):
        // http://stackoverflow.com/questions/201323/using-a-regular-expression-to-validate-an-email-address#answer-8829363
        // http://www.w3.org/TR/html5/forms.html#valid-e-mail-address (search for 'willful violation')
        email: /^[a-z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?(?:\.[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?)*$/i,
        hostname: HOSTNAME,
        // optimized https://www.safaribooksonline.com/library/view/regular-expressions-cookbook/9780596802837/ch07s16.html
        ipv4: /^(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)$/,
        // optimized http://stackoverflow.com/questions/53497/regular-expression-that-matches-valid-ipv6-addresses
        ipv6: /^\s*(?:(?:(?:[0-9a-f]{1,4}:){7}(?:[0-9a-f]{1,4}|:))|(?:(?:[0-9a-f]{1,4}:){6}(?::[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){5}(?:(?:(?::[0-9a-f]{1,4}){1,2})|:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){4}(?:(?:(?::[0-9a-f]{1,4}){1,3})|(?:(?::[0-9a-f]{1,4})?:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){3}(?:(?:(?::[0-9a-f]{1,4}){1,4})|(?:(?::[0-9a-f]{1,4}){0,2}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){2}(?:(?:(?::[0-9a-f]{1,4}){1,5})|(?:(?::[0-9a-f]{1,4}){0,3}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){1}(?:(?:(?::[0-9a-f]{1,4}){1,6})|(?:(?::[0-9a-f]{1,4}){0,4}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?::(?:(?:(?::[0-9a-f]{1,4}){1,7})|(?:(?::[0-9a-f]{1,4}){0,5}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(?:%.+)?\s*$/i,
        regex: regex,
        // uuid: http://tools.ietf.org/html/rfc4122
        uuid: UUID,
        // JSON-pointer: https://tools.ietf.org/html/rfc6901
        // uri fragment: https://tools.ietf.org/html/rfc3986#appendix-A
        'json-pointer': JSON_POINTER,
        'json-pointer-uri-fragment': JSON_POINTER_URI_FRAGMENT,
        // relative JSON-pointer: http://tools.ietf.org/html/draft-luff-relative-json-pointer-00
        'relative-json-pointer': RELATIVE_JSON_POINTER
    };
    formats.full = {
        date: date,
        time: time,
        'date-time': date_time,
        uri: uri,
        'uri-reference': URIREF,
        'uri-template': URITEMPLATE,
        url: URL,
        email: /^[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$/i,
        hostname: HOSTNAME,
        ipv4: /^(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d\d?)$/,
        ipv6: /^\s*(?:(?:(?:[0-9a-f]{1,4}:){7}(?:[0-9a-f]{1,4}|:))|(?:(?:[0-9a-f]{1,4}:){6}(?::[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){5}(?:(?:(?::[0-9a-f]{1,4}){1,2})|:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){4}(?:(?:(?::[0-9a-f]{1,4}){1,3})|(?:(?::[0-9a-f]{1,4})?:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){3}(?:(?:(?::[0-9a-f]{1,4}){1,4})|(?:(?::[0-9a-f]{1,4}){0,2}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){2}(?:(?:(?::[0-9a-f]{1,4}){1,5})|(?:(?::[0-9a-f]{1,4}){0,3}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){1}(?:(?:(?::[0-9a-f]{1,4}){1,6})|(?:(?::[0-9a-f]{1,4}){0,4}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?::(?:(?:(?::[0-9a-f]{1,4}){1,7})|(?:(?::[0-9a-f]{1,4}){0,5}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(?:%.+)?\s*$/i,
        regex: regex,
        uuid: UUID,
        'json-pointer': JSON_POINTER,
        'json-pointer-uri-fragment': JSON_POINTER_URI_FRAGMENT,
        'relative-json-pointer': RELATIVE_JSON_POINTER
    };
    function isLeapYear(year) {
        // https://tools.ietf.org/html/rfc3339#appendix-C
        return year % 4 === 0 && (year % 100 !== 0 || year % 400 === 0);
    }
    function date(str) {
        // full-date from http://tools.ietf.org/html/rfc3339#section-5.6
        var matches = str.match(DATE);
        if (!matches)
            return false;
        var year = +matches[1];
        var month = +matches[2];
        var day = +matches[3];
        return month >= 1 && month <= 12 && day >= 1 &&
            day <= (month == 2 && isLeapYear(year) ? 29 : DAYS[month]);
    }
    function time(str, full) {
        var matches = str.match(TIME);
        if (!matches)
            return false;
        var hour = matches[1];
        var minute = matches[2];
        var second = matches[3];
        var timeZone = matches[5];
        return ((hour <= 23 && minute <= 59 && second <= 59) ||
            (hour == 23 && minute == 59 && second == 60)) &&
            (!full || timeZone);
    }
    var DATE_TIME_SEPARATOR = /t|\s/i;
    function date_time(str) {
        // http://tools.ietf.org/html/rfc3339#section-5.6
        var dateTime = str.split(DATE_TIME_SEPARATOR);
        return dateTime.length == 2 && date(dateTime[0]) && time(dateTime[1], true);
    }
    var NOT_URI_FRAGMENT = /\/|:/;
    function uri(str) {
        // http://jmrware.com/articles/2009/uri_regexp/URI_regex.html + optional protocol + required "."
        return NOT_URI_FRAGMENT.test(str) && URI.test(str);
    }
    var Z_ANCHOR = /[^\\]\\Z/;
    function regex(str) {
        if (Z_ANCHOR.test(str))
            return false;
        try {
            new RegExp(str);
            return true;
        }
        catch (e) {
            return false;
        }
    }
},
/* ajv/lib/compile/rules.js */ function _(require, module, exports) {
    var ruleModules = require(439) /* ../dotjs */, toHash = require(429) /* ./util */.toHash;
    module.exports = function rules() {
        var RULES = [
            { type: 'number',
                rules: [{ 'maximum': ['exclusiveMaximum'] },
                    { 'minimum': ['exclusiveMinimum'] }, 'multipleOf', 'format'] },
            { type: 'string',
                rules: ['maxLength', 'minLength', 'pattern', 'format'] },
            { type: 'array',
                rules: ['maxItems', 'minItems', 'items', 'contains', 'uniqueItems'] },
            { type: 'object',
                rules: ['maxProperties', 'minProperties', 'required', 'dependencies', 'propertyNames',
                    { 'properties': ['additionalProperties', 'patternProperties'] }] },
            { rules: ['$ref', 'const', 'enum', 'not', 'anyOf', 'oneOf', 'allOf', 'if'] }
        ];
        var ALL = ['type', '$comment'];
        var KEYWORDS = [
            '$schema', '$id', 'id', '$data', '$async', 'title',
            'description', 'default', 'definitions',
            'examples', 'readOnly', 'writeOnly',
            'contentMediaType', 'contentEncoding',
            'additionalItems', 'then', 'else'
        ];
        var TYPES = ['number', 'integer', 'string', 'array', 'object', 'boolean', 'null'];
        RULES.all = toHash(ALL);
        RULES.types = toHash(TYPES);
        RULES.forEach(function (group) {
            group.rules = group.rules.map(function (keyword) {
                var implKeywords;
                if (typeof keyword == 'object') {
                    var key = Object.keys(keyword)[0];
                    implKeywords = keyword[key];
                    keyword = key;
                    implKeywords.forEach(function (k) {
                        ALL.push(k);
                        RULES.all[k] = true;
                    });
                }
                ALL.push(keyword);
                var rule = RULES.all[keyword] = {
                    keyword: keyword,
                    code: ruleModules[keyword],
                    implements: implKeywords
                };
                return rule;
            });
            RULES.all.$comment = {
                keyword: '$comment',
                code: ruleModules.$comment
            };
            if (group.type)
                RULES.types[group.type] = group;
        });
        RULES.keywords = toHash(ALL.concat(KEYWORDS));
        RULES.custom = {};
        return RULES;
    };
},
/* ajv/lib/dotjs/index.js */ function _(require, module, exports) {
    //all requires must be explicit because browserify won't work with dynamic requires
    module.exports = {
        '$ref': require(440) /* ./ref */,
        allOf: require(441) /* ./allOf */,
        anyOf: require(442) /* ./anyOf */,
        '$comment': require(443) /* ./comment */,
        const: require(444) /* ./const */,
        contains: require(445) /* ./contains */,
        dependencies: require(446) /* ./dependencies */,
        'enum': require(447) /* ./enum */,
        format: require(448) /* ./format */,
        'if': require(449) /* ./if */,
        items: require(450) /* ./items */,
        maximum: require(451) /* ./_limit */,
        minimum: require(451) /* ./_limit */,
        maxItems: require(452) /* ./_limitItems */,
        minItems: require(452) /* ./_limitItems */,
        maxLength: require(453) /* ./_limitLength */,
        minLength: require(453) /* ./_limitLength */,
        maxProperties: require(454) /* ./_limitProperties */,
        minProperties: require(454) /* ./_limitProperties */,
        multipleOf: require(455) /* ./multipleOf */,
        not: require(456) /* ./not */,
        oneOf: require(457) /* ./oneOf */,
        pattern: require(458) /* ./pattern */,
        properties: require(459) /* ./properties */,
        propertyNames: require(460) /* ./propertyNames */,
        required: require(461) /* ./required */,
        uniqueItems: require(462) /* ./uniqueItems */,
        validate: require(435) /* ./validate */
    };
},
/* ajv/lib/dotjs/ref.js */ function _(require, module, exports) {
    module.exports = function generate_ref(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $valid = 'valid' + $lvl;
        var $async, $refCode;
        if ($schema == '#' || $schema == '#/') {
            if (it.isRoot) {
                $async = it.async;
                $refCode = 'validate';
            }
            else {
                $async = it.root.schema.$async === true;
                $refCode = 'root.refVal[0]';
            }
        }
        else {
            var $refVal = it.resolveRef(it.baseId, $schema, it.isRoot);
            if ($refVal === undefined) {
                var $message = it.MissingRefError.message(it.baseId, $schema);
                if (it.opts.missingRefs == 'fail') {
                    it.logger.error($message);
                    var $$outStack = $$outStack || [];
                    $$outStack.push(out);
                    out = ''; /* istanbul ignore else */
                    if (it.createErrors !== false) {
                        out += ' { keyword: \'' + ('$ref') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { ref: \'' + (it.util.escapeQuotes($schema)) + '\' } ';
                        if (it.opts.messages !== false) {
                            out += ' , message: \'can\\\'t resolve reference ' + (it.util.escapeQuotes($schema)) + '\' ';
                        }
                        if (it.opts.verbose) {
                            out += ' , schema: ' + (it.util.toQuotedString($schema)) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                        }
                        out += ' } ';
                    }
                    else {
                        out += ' {} ';
                    }
                    var __err = out;
                    out = $$outStack.pop();
                    if (!it.compositeRule && $breakOnError) {
                        /* istanbul ignore if */
                        if (it.async) {
                            out += ' throw new ValidationError([' + (__err) + ']); ';
                        }
                        else {
                            out += ' validate.errors = [' + (__err) + ']; return false; ';
                        }
                    }
                    else {
                        out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                    }
                    if ($breakOnError) {
                        out += ' if (false) { ';
                    }
                }
                else if (it.opts.missingRefs == 'ignore') {
                    it.logger.warn($message);
                    if ($breakOnError) {
                        out += ' if (true) { ';
                    }
                }
                else {
                    throw new it.MissingRefError(it.baseId, $schema, $message);
                }
            }
            else if ($refVal.inline) {
                var $it = it.util.copy(it);
                $it.level++;
                var $nextValid = 'valid' + $it.level;
                $it.schema = $refVal.schema;
                $it.schemaPath = '';
                $it.errSchemaPath = $schema;
                var $code = it.validate($it).replace(/validate\.schema/g, $refVal.code);
                out += ' ' + ($code) + ' ';
                if ($breakOnError) {
                    out += ' if (' + ($nextValid) + ') { ';
                }
            }
            else {
                $async = $refVal.$async === true || (it.async && $refVal.$async !== false);
                $refCode = $refVal.code;
            }
        }
        if ($refCode) {
            var $$outStack = $$outStack || [];
            $$outStack.push(out);
            out = '';
            if (it.opts.passContext) {
                out += ' ' + ($refCode) + '.call(this, ';
            }
            else {
                out += ' ' + ($refCode) + '( ';
            }
            out += ' ' + ($data) + ', (dataPath || \'\')';
            if (it.errorPath != '""') {
                out += ' + ' + (it.errorPath);
            }
            var $parentData = $dataLvl ? 'data' + (($dataLvl - 1) || '') : 'parentData', $parentDataProperty = $dataLvl ? it.dataPathArr[$dataLvl] : 'parentDataProperty';
            out += ' , ' + ($parentData) + ' , ' + ($parentDataProperty) + ', rootData)  ';
            var __callValidate = out;
            out = $$outStack.pop();
            if ($async) {
                if (!it.async)
                    throw new Error('async schema referenced by sync schema');
                if ($breakOnError) {
                    out += ' var ' + ($valid) + '; ';
                }
                out += ' try { await ' + (__callValidate) + '; ';
                if ($breakOnError) {
                    out += ' ' + ($valid) + ' = true; ';
                }
                out += ' } catch (e) { if (!(e instanceof ValidationError)) throw e; if (vErrors === null) vErrors = e.errors; else vErrors = vErrors.concat(e.errors); errors = vErrors.length; ';
                if ($breakOnError) {
                    out += ' ' + ($valid) + ' = false; ';
                }
                out += ' } ';
                if ($breakOnError) {
                    out += ' if (' + ($valid) + ') { ';
                }
            }
            else {
                out += ' if (!' + (__callValidate) + ') { if (vErrors === null) vErrors = ' + ($refCode) + '.errors; else vErrors = vErrors.concat(' + ($refCode) + '.errors); errors = vErrors.length; } ';
                if ($breakOnError) {
                    out += ' else { ';
                }
            }
        }
        return out;
    };
},
/* ajv/lib/dotjs/allOf.js */ function _(require, module, exports) {
    module.exports = function generate_allOf(it, $keyword, $ruleType) {
        var out = ' ';
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $it = it.util.copy(it);
        var $closingBraces = '';
        $it.level++;
        var $nextValid = 'valid' + $it.level;
        var $currentBaseId = $it.baseId, $allSchemasEmpty = true;
        var arr1 = $schema;
        if (arr1) {
            var $sch, $i = -1, l1 = arr1.length - 1;
            while ($i < l1) {
                $sch = arr1[$i += 1];
                if ((it.opts.strictKeywords ? typeof $sch == 'object' && Object.keys($sch).length > 0 : it.util.schemaHasRules($sch, it.RULES.all))) {
                    $allSchemasEmpty = false;
                    $it.schema = $sch;
                    $it.schemaPath = $schemaPath + '[' + $i + ']';
                    $it.errSchemaPath = $errSchemaPath + '/' + $i;
                    out += '  ' + (it.validate($it)) + ' ';
                    $it.baseId = $currentBaseId;
                    if ($breakOnError) {
                        out += ' if (' + ($nextValid) + ') { ';
                        $closingBraces += '}';
                    }
                }
            }
        }
        if ($breakOnError) {
            if ($allSchemasEmpty) {
                out += ' if (true) { ';
            }
            else {
                out += ' ' + ($closingBraces.slice(0, -1)) + ' ';
            }
        }
        out = it.util.cleanUpCode(out);
        return out;
    };
},
/* ajv/lib/dotjs/anyOf.js */ function _(require, module, exports) {
    module.exports = function generate_anyOf(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $valid = 'valid' + $lvl;
        var $errs = 'errs__' + $lvl;
        var $it = it.util.copy(it);
        var $closingBraces = '';
        $it.level++;
        var $nextValid = 'valid' + $it.level;
        var $noEmptySchema = $schema.every(function ($sch) {
            return (it.opts.strictKeywords ? typeof $sch == 'object' && Object.keys($sch).length > 0 : it.util.schemaHasRules($sch, it.RULES.all));
        });
        if ($noEmptySchema) {
            var $currentBaseId = $it.baseId;
            out += ' var ' + ($errs) + ' = errors; var ' + ($valid) + ' = false;  ';
            var $wasComposite = it.compositeRule;
            it.compositeRule = $it.compositeRule = true;
            var arr1 = $schema;
            if (arr1) {
                var $sch, $i = -1, l1 = arr1.length - 1;
                while ($i < l1) {
                    $sch = arr1[$i += 1];
                    $it.schema = $sch;
                    $it.schemaPath = $schemaPath + '[' + $i + ']';
                    $it.errSchemaPath = $errSchemaPath + '/' + $i;
                    out += '  ' + (it.validate($it)) + ' ';
                    $it.baseId = $currentBaseId;
                    out += ' ' + ($valid) + ' = ' + ($valid) + ' || ' + ($nextValid) + '; if (!' + ($valid) + ') { ';
                    $closingBraces += '}';
                }
            }
            it.compositeRule = $it.compositeRule = $wasComposite;
            out += ' ' + ($closingBraces) + ' if (!' + ($valid) + ') {   var err =   '; /* istanbul ignore else */
            if (it.createErrors !== false) {
                out += ' { keyword: \'' + ('anyOf') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: {} ';
                if (it.opts.messages !== false) {
                    out += ' , message: \'should match some schema in anyOf\' ';
                }
                if (it.opts.verbose) {
                    out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                }
                out += ' } ';
            }
            else {
                out += ' {} ';
            }
            out += ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
            if (!it.compositeRule && $breakOnError) {
                /* istanbul ignore if */
                if (it.async) {
                    out += ' throw new ValidationError(vErrors); ';
                }
                else {
                    out += ' validate.errors = vErrors; return false; ';
                }
            }
            out += ' } else {  errors = ' + ($errs) + '; if (vErrors !== null) { if (' + ($errs) + ') vErrors.length = ' + ($errs) + '; else vErrors = null; } ';
            if (it.opts.allErrors) {
                out += ' } ';
            }
            out = it.util.cleanUpCode(out);
        }
        else {
            if ($breakOnError) {
                out += ' if (true) { ';
            }
        }
        return out;
    };
},
/* ajv/lib/dotjs/comment.js */ function _(require, module, exports) {
    module.exports = function generate_comment(it, $keyword, $ruleType) {
        var out = ' ';
        var $schema = it.schema[$keyword];
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $comment = it.util.toQuotedString($schema);
        if (it.opts.$comment === true) {
            out += ' console.log(' + ($comment) + ');';
        }
        else if (typeof it.opts.$comment == 'function') {
            out += ' self._opts.$comment(' + ($comment) + ', ' + (it.util.toQuotedString($errSchemaPath)) + ', validate.root.schema);';
        }
        return out;
    };
},
/* ajv/lib/dotjs/const.js */ function _(require, module, exports) {
    module.exports = function generate_const(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $valid = 'valid' + $lvl;
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        if (!$isData) {
            out += ' var schema' + ($lvl) + ' = validate.schema' + ($schemaPath) + ';';
        }
        out += 'var ' + ($valid) + ' = equal(' + ($data) + ', schema' + ($lvl) + '); if (!' + ($valid) + ') {   ';
        var $$outStack = $$outStack || [];
        $$outStack.push(out);
        out = ''; /* istanbul ignore else */
        if (it.createErrors !== false) {
            out += ' { keyword: \'' + ('const') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { allowedValue: schema' + ($lvl) + ' } ';
            if (it.opts.messages !== false) {
                out += ' , message: \'should be equal to constant\' ';
            }
            if (it.opts.verbose) {
                out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
            }
            out += ' } ';
        }
        else {
            out += ' {} ';
        }
        var __err = out;
        out = $$outStack.pop();
        if (!it.compositeRule && $breakOnError) {
            /* istanbul ignore if */
            if (it.async) {
                out += ' throw new ValidationError([' + (__err) + ']); ';
            }
            else {
                out += ' validate.errors = [' + (__err) + ']; return false; ';
            }
        }
        else {
            out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
        }
        out += ' }';
        if ($breakOnError) {
            out += ' else { ';
        }
        return out;
    };
},
/* ajv/lib/dotjs/contains.js */ function _(require, module, exports) {
    module.exports = function generate_contains(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $valid = 'valid' + $lvl;
        var $errs = 'errs__' + $lvl;
        var $it = it.util.copy(it);
        var $closingBraces = '';
        $it.level++;
        var $nextValid = 'valid' + $it.level;
        var $idx = 'i' + $lvl, $dataNxt = $it.dataLevel = it.dataLevel + 1, $nextData = 'data' + $dataNxt, $currentBaseId = it.baseId, $nonEmptySchema = (it.opts.strictKeywords ? typeof $schema == 'object' && Object.keys($schema).length > 0 : it.util.schemaHasRules($schema, it.RULES.all));
        out += 'var ' + ($errs) + ' = errors;var ' + ($valid) + ';';
        if ($nonEmptySchema) {
            var $wasComposite = it.compositeRule;
            it.compositeRule = $it.compositeRule = true;
            $it.schema = $schema;
            $it.schemaPath = $schemaPath;
            $it.errSchemaPath = $errSchemaPath;
            out += ' var ' + ($nextValid) + ' = false; for (var ' + ($idx) + ' = 0; ' + ($idx) + ' < ' + ($data) + '.length; ' + ($idx) + '++) { ';
            $it.errorPath = it.util.getPathExpr(it.errorPath, $idx, it.opts.jsonPointers, true);
            var $passData = $data + '[' + $idx + ']';
            $it.dataPathArr[$dataNxt] = $idx;
            var $code = it.validate($it);
            $it.baseId = $currentBaseId;
            if (it.util.varOccurences($code, $nextData) < 2) {
                out += ' ' + (it.util.varReplace($code, $nextData, $passData)) + ' ';
            }
            else {
                out += ' var ' + ($nextData) + ' = ' + ($passData) + '; ' + ($code) + ' ';
            }
            out += ' if (' + ($nextValid) + ') break; }  ';
            it.compositeRule = $it.compositeRule = $wasComposite;
            out += ' ' + ($closingBraces) + ' if (!' + ($nextValid) + ') {';
        }
        else {
            out += ' if (' + ($data) + '.length == 0) {';
        }
        var $$outStack = $$outStack || [];
        $$outStack.push(out);
        out = ''; /* istanbul ignore else */
        if (it.createErrors !== false) {
            out += ' { keyword: \'' + ('contains') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: {} ';
            if (it.opts.messages !== false) {
                out += ' , message: \'should contain a valid item\' ';
            }
            if (it.opts.verbose) {
                out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
            }
            out += ' } ';
        }
        else {
            out += ' {} ';
        }
        var __err = out;
        out = $$outStack.pop();
        if (!it.compositeRule && $breakOnError) {
            /* istanbul ignore if */
            if (it.async) {
                out += ' throw new ValidationError([' + (__err) + ']); ';
            }
            else {
                out += ' validate.errors = [' + (__err) + ']; return false; ';
            }
        }
        else {
            out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
        }
        out += ' } else { ';
        if ($nonEmptySchema) {
            out += '  errors = ' + ($errs) + '; if (vErrors !== null) { if (' + ($errs) + ') vErrors.length = ' + ($errs) + '; else vErrors = null; } ';
        }
        if (it.opts.allErrors) {
            out += ' } ';
        }
        out = it.util.cleanUpCode(out);
        return out;
    };
},
/* ajv/lib/dotjs/dependencies.js */ function _(require, module, exports) {
    module.exports = function generate_dependencies(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $errs = 'errs__' + $lvl;
        var $it = it.util.copy(it);
        var $closingBraces = '';
        $it.level++;
        var $nextValid = 'valid' + $it.level;
        var $schemaDeps = {}, $propertyDeps = {}, $ownProperties = it.opts.ownProperties;
        for ($property in $schema) {
            var $sch = $schema[$property];
            var $deps = Array.isArray($sch) ? $propertyDeps : $schemaDeps;
            $deps[$property] = $sch;
        }
        out += 'var ' + ($errs) + ' = errors;';
        var $currentErrorPath = it.errorPath;
        out += 'var missing' + ($lvl) + ';';
        for (var $property in $propertyDeps) {
            $deps = $propertyDeps[$property];
            if ($deps.length) {
                out += ' if ( ' + ($data) + (it.util.getProperty($property)) + ' !== undefined ';
                if ($ownProperties) {
                    out += ' && Object.prototype.hasOwnProperty.call(' + ($data) + ', \'' + (it.util.escapeQuotes($property)) + '\') ';
                }
                if ($breakOnError) {
                    out += ' && ( ';
                    var arr1 = $deps;
                    if (arr1) {
                        var $propertyKey, $i = -1, l1 = arr1.length - 1;
                        while ($i < l1) {
                            $propertyKey = arr1[$i += 1];
                            if ($i) {
                                out += ' || ';
                            }
                            var $prop = it.util.getProperty($propertyKey), $useData = $data + $prop;
                            out += ' ( ( ' + ($useData) + ' === undefined ';
                            if ($ownProperties) {
                                out += ' || ! Object.prototype.hasOwnProperty.call(' + ($data) + ', \'' + (it.util.escapeQuotes($propertyKey)) + '\') ';
                            }
                            out += ') && (missing' + ($lvl) + ' = ' + (it.util.toQuotedString(it.opts.jsonPointers ? $propertyKey : $prop)) + ') ) ';
                        }
                    }
                    out += ')) {  ';
                    var $propertyPath = 'missing' + $lvl, $missingProperty = '\' + ' + $propertyPath + ' + \'';
                    if (it.opts._errorDataPathProperty) {
                        it.errorPath = it.opts.jsonPointers ? it.util.getPathExpr($currentErrorPath, $propertyPath, true) : $currentErrorPath + ' + ' + $propertyPath;
                    }
                    var $$outStack = $$outStack || [];
                    $$outStack.push(out);
                    out = ''; /* istanbul ignore else */
                    if (it.createErrors !== false) {
                        out += ' { keyword: \'' + ('dependencies') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { property: \'' + (it.util.escapeQuotes($property)) + '\', missingProperty: \'' + ($missingProperty) + '\', depsCount: ' + ($deps.length) + ', deps: \'' + (it.util.escapeQuotes($deps.length == 1 ? $deps[0] : $deps.join(", "))) + '\' } ';
                        if (it.opts.messages !== false) {
                            out += ' , message: \'should have ';
                            if ($deps.length == 1) {
                                out += 'property ' + (it.util.escapeQuotes($deps[0]));
                            }
                            else {
                                out += 'properties ' + (it.util.escapeQuotes($deps.join(", ")));
                            }
                            out += ' when property ' + (it.util.escapeQuotes($property)) + ' is present\' ';
                        }
                        if (it.opts.verbose) {
                            out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                        }
                        out += ' } ';
                    }
                    else {
                        out += ' {} ';
                    }
                    var __err = out;
                    out = $$outStack.pop();
                    if (!it.compositeRule && $breakOnError) {
                        /* istanbul ignore if */
                        if (it.async) {
                            out += ' throw new ValidationError([' + (__err) + ']); ';
                        }
                        else {
                            out += ' validate.errors = [' + (__err) + ']; return false; ';
                        }
                    }
                    else {
                        out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                    }
                }
                else {
                    out += ' ) { ';
                    var arr2 = $deps;
                    if (arr2) {
                        var $propertyKey, i2 = -1, l2 = arr2.length - 1;
                        while (i2 < l2) {
                            $propertyKey = arr2[i2 += 1];
                            var $prop = it.util.getProperty($propertyKey), $missingProperty = it.util.escapeQuotes($propertyKey), $useData = $data + $prop;
                            if (it.opts._errorDataPathProperty) {
                                it.errorPath = it.util.getPath($currentErrorPath, $propertyKey, it.opts.jsonPointers);
                            }
                            out += ' if ( ' + ($useData) + ' === undefined ';
                            if ($ownProperties) {
                                out += ' || ! Object.prototype.hasOwnProperty.call(' + ($data) + ', \'' + (it.util.escapeQuotes($propertyKey)) + '\') ';
                            }
                            out += ') {  var err =   '; /* istanbul ignore else */
                            if (it.createErrors !== false) {
                                out += ' { keyword: \'' + ('dependencies') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { property: \'' + (it.util.escapeQuotes($property)) + '\', missingProperty: \'' + ($missingProperty) + '\', depsCount: ' + ($deps.length) + ', deps: \'' + (it.util.escapeQuotes($deps.length == 1 ? $deps[0] : $deps.join(", "))) + '\' } ';
                                if (it.opts.messages !== false) {
                                    out += ' , message: \'should have ';
                                    if ($deps.length == 1) {
                                        out += 'property ' + (it.util.escapeQuotes($deps[0]));
                                    }
                                    else {
                                        out += 'properties ' + (it.util.escapeQuotes($deps.join(", ")));
                                    }
                                    out += ' when property ' + (it.util.escapeQuotes($property)) + ' is present\' ';
                                }
                                if (it.opts.verbose) {
                                    out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                                }
                                out += ' } ';
                            }
                            else {
                                out += ' {} ';
                            }
                            out += ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } ';
                        }
                    }
                }
                out += ' }   ';
                if ($breakOnError) {
                    $closingBraces += '}';
                    out += ' else { ';
                }
            }
        }
        it.errorPath = $currentErrorPath;
        var $currentBaseId = $it.baseId;
        for (var $property in $schemaDeps) {
            var $sch = $schemaDeps[$property];
            if ((it.opts.strictKeywords ? typeof $sch == 'object' && Object.keys($sch).length > 0 : it.util.schemaHasRules($sch, it.RULES.all))) {
                out += ' ' + ($nextValid) + ' = true; if ( ' + ($data) + (it.util.getProperty($property)) + ' !== undefined ';
                if ($ownProperties) {
                    out += ' && Object.prototype.hasOwnProperty.call(' + ($data) + ', \'' + (it.util.escapeQuotes($property)) + '\') ';
                }
                out += ') { ';
                $it.schema = $sch;
                $it.schemaPath = $schemaPath + it.util.getProperty($property);
                $it.errSchemaPath = $errSchemaPath + '/' + it.util.escapeFragment($property);
                out += '  ' + (it.validate($it)) + ' ';
                $it.baseId = $currentBaseId;
                out += ' }  ';
                if ($breakOnError) {
                    out += ' if (' + ($nextValid) + ') { ';
                    $closingBraces += '}';
                }
            }
        }
        if ($breakOnError) {
            out += '   ' + ($closingBraces) + ' if (' + ($errs) + ' == errors) {';
        }
        out = it.util.cleanUpCode(out);
        return out;
    };
},
/* ajv/lib/dotjs/enum.js */ function _(require, module, exports) {
    module.exports = function generate_enum(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $valid = 'valid' + $lvl;
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        var $i = 'i' + $lvl, $vSchema = 'schema' + $lvl;
        if (!$isData) {
            out += ' var ' + ($vSchema) + ' = validate.schema' + ($schemaPath) + ';';
        }
        out += 'var ' + ($valid) + ';';
        if ($isData) {
            out += ' if (schema' + ($lvl) + ' === undefined) ' + ($valid) + ' = true; else if (!Array.isArray(schema' + ($lvl) + ')) ' + ($valid) + ' = false; else {';
        }
        out += '' + ($valid) + ' = false;for (var ' + ($i) + '=0; ' + ($i) + '<' + ($vSchema) + '.length; ' + ($i) + '++) if (equal(' + ($data) + ', ' + ($vSchema) + '[' + ($i) + '])) { ' + ($valid) + ' = true; break; }';
        if ($isData) {
            out += '  }  ';
        }
        out += ' if (!' + ($valid) + ') {   ';
        var $$outStack = $$outStack || [];
        $$outStack.push(out);
        out = ''; /* istanbul ignore else */
        if (it.createErrors !== false) {
            out += ' { keyword: \'' + ('enum') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { allowedValues: schema' + ($lvl) + ' } ';
            if (it.opts.messages !== false) {
                out += ' , message: \'should be equal to one of the allowed values\' ';
            }
            if (it.opts.verbose) {
                out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
            }
            out += ' } ';
        }
        else {
            out += ' {} ';
        }
        var __err = out;
        out = $$outStack.pop();
        if (!it.compositeRule && $breakOnError) {
            /* istanbul ignore if */
            if (it.async) {
                out += ' throw new ValidationError([' + (__err) + ']); ';
            }
            else {
                out += ' validate.errors = [' + (__err) + ']; return false; ';
            }
        }
        else {
            out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
        }
        out += ' }';
        if ($breakOnError) {
            out += ' else { ';
        }
        return out;
    };
},
/* ajv/lib/dotjs/format.js */ function _(require, module, exports) {
    module.exports = function generate_format(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        if (it.opts.format === false) {
            if ($breakOnError) {
                out += ' if (true) { ';
            }
            return out;
        }
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        var $unknownFormats = it.opts.unknownFormats, $allowUnknown = Array.isArray($unknownFormats);
        if ($isData) {
            var $format = 'format' + $lvl, $isObject = 'isObject' + $lvl, $formatType = 'formatType' + $lvl;
            out += ' var ' + ($format) + ' = formats[' + ($schemaValue) + ']; var ' + ($isObject) + ' = typeof ' + ($format) + ' == \'object\' && !(' + ($format) + ' instanceof RegExp) && ' + ($format) + '.validate; var ' + ($formatType) + ' = ' + ($isObject) + ' && ' + ($format) + '.type || \'string\'; if (' + ($isObject) + ') { ';
            if (it.async) {
                out += ' var async' + ($lvl) + ' = ' + ($format) + '.async; ';
            }
            out += ' ' + ($format) + ' = ' + ($format) + '.validate; } if (  ';
            if ($isData) {
                out += ' (' + ($schemaValue) + ' !== undefined && typeof ' + ($schemaValue) + ' != \'string\') || ';
            }
            out += ' (';
            if ($unknownFormats != 'ignore') {
                out += ' (' + ($schemaValue) + ' && !' + ($format) + ' ';
                if ($allowUnknown) {
                    out += ' && self._opts.unknownFormats.indexOf(' + ($schemaValue) + ') == -1 ';
                }
                out += ') || ';
            }
            out += ' (' + ($format) + ' && ' + ($formatType) + ' == \'' + ($ruleType) + '\' && !(typeof ' + ($format) + ' == \'function\' ? ';
            if (it.async) {
                out += ' (async' + ($lvl) + ' ? await ' + ($format) + '(' + ($data) + ') : ' + ($format) + '(' + ($data) + ')) ';
            }
            else {
                out += ' ' + ($format) + '(' + ($data) + ') ';
            }
            out += ' : ' + ($format) + '.test(' + ($data) + '))))) {';
        }
        else {
            var $format = it.formats[$schema];
            if (!$format) {
                if ($unknownFormats == 'ignore') {
                    it.logger.warn('unknown format "' + $schema + '" ignored in schema at path "' + it.errSchemaPath + '"');
                    if ($breakOnError) {
                        out += ' if (true) { ';
                    }
                    return out;
                }
                else if ($allowUnknown && $unknownFormats.indexOf($schema) >= 0) {
                    if ($breakOnError) {
                        out += ' if (true) { ';
                    }
                    return out;
                }
                else {
                    throw new Error('unknown format "' + $schema + '" is used in schema at path "' + it.errSchemaPath + '"');
                }
            }
            var $isObject = typeof $format == 'object' && !($format instanceof RegExp) && $format.validate;
            var $formatType = $isObject && $format.type || 'string';
            if ($isObject) {
                var $async = $format.async === true;
                $format = $format.validate;
            }
            if ($formatType != $ruleType) {
                if ($breakOnError) {
                    out += ' if (true) { ';
                }
                return out;
            }
            if ($async) {
                if (!it.async)
                    throw new Error('async format in sync schema');
                var $formatRef = 'formats' + it.util.getProperty($schema) + '.validate';
                out += ' if (!(await ' + ($formatRef) + '(' + ($data) + '))) { ';
            }
            else {
                out += ' if (! ';
                var $formatRef = 'formats' + it.util.getProperty($schema);
                if ($isObject)
                    $formatRef += '.validate';
                if (typeof $format == 'function') {
                    out += ' ' + ($formatRef) + '(' + ($data) + ') ';
                }
                else {
                    out += ' ' + ($formatRef) + '.test(' + ($data) + ') ';
                }
                out += ') { ';
            }
        }
        var $$outStack = $$outStack || [];
        $$outStack.push(out);
        out = ''; /* istanbul ignore else */
        if (it.createErrors !== false) {
            out += ' { keyword: \'' + ('format') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { format:  ';
            if ($isData) {
                out += '' + ($schemaValue);
            }
            else {
                out += '' + (it.util.toQuotedString($schema));
            }
            out += '  } ';
            if (it.opts.messages !== false) {
                out += ' , message: \'should match format "';
                if ($isData) {
                    out += '\' + ' + ($schemaValue) + ' + \'';
                }
                else {
                    out += '' + (it.util.escapeQuotes($schema));
                }
                out += '"\' ';
            }
            if (it.opts.verbose) {
                out += ' , schema:  ';
                if ($isData) {
                    out += 'validate.schema' + ($schemaPath);
                }
                else {
                    out += '' + (it.util.toQuotedString($schema));
                }
                out += '         , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
            }
            out += ' } ';
        }
        else {
            out += ' {} ';
        }
        var __err = out;
        out = $$outStack.pop();
        if (!it.compositeRule && $breakOnError) {
            /* istanbul ignore if */
            if (it.async) {
                out += ' throw new ValidationError([' + (__err) + ']); ';
            }
            else {
                out += ' validate.errors = [' + (__err) + ']; return false; ';
            }
        }
        else {
            out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
        }
        out += ' } ';
        if ($breakOnError) {
            out += ' else { ';
        }
        return out;
    };
},
/* ajv/lib/dotjs/if.js */ function _(require, module, exports) {
    module.exports = function generate_if(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $valid = 'valid' + $lvl;
        var $errs = 'errs__' + $lvl;
        var $it = it.util.copy(it);
        $it.level++;
        var $nextValid = 'valid' + $it.level;
        var $thenSch = it.schema['then'], $elseSch = it.schema['else'], $thenPresent = $thenSch !== undefined && (it.opts.strictKeywords ? typeof $thenSch == 'object' && Object.keys($thenSch).length > 0 : it.util.schemaHasRules($thenSch, it.RULES.all)), $elsePresent = $elseSch !== undefined && (it.opts.strictKeywords ? typeof $elseSch == 'object' && Object.keys($elseSch).length > 0 : it.util.schemaHasRules($elseSch, it.RULES.all)), $currentBaseId = $it.baseId;
        if ($thenPresent || $elsePresent) {
            var $ifClause;
            $it.createErrors = false;
            $it.schema = $schema;
            $it.schemaPath = $schemaPath;
            $it.errSchemaPath = $errSchemaPath;
            out += ' var ' + ($errs) + ' = errors; var ' + ($valid) + ' = true;  ';
            var $wasComposite = it.compositeRule;
            it.compositeRule = $it.compositeRule = true;
            out += '  ' + (it.validate($it)) + ' ';
            $it.baseId = $currentBaseId;
            $it.createErrors = true;
            out += '  errors = ' + ($errs) + '; if (vErrors !== null) { if (' + ($errs) + ') vErrors.length = ' + ($errs) + '; else vErrors = null; }  ';
            it.compositeRule = $it.compositeRule = $wasComposite;
            if ($thenPresent) {
                out += ' if (' + ($nextValid) + ') {  ';
                $it.schema = it.schema['then'];
                $it.schemaPath = it.schemaPath + '.then';
                $it.errSchemaPath = it.errSchemaPath + '/then';
                out += '  ' + (it.validate($it)) + ' ';
                $it.baseId = $currentBaseId;
                out += ' ' + ($valid) + ' = ' + ($nextValid) + '; ';
                if ($thenPresent && $elsePresent) {
                    $ifClause = 'ifClause' + $lvl;
                    out += ' var ' + ($ifClause) + ' = \'then\'; ';
                }
                else {
                    $ifClause = '\'then\'';
                }
                out += ' } ';
                if ($elsePresent) {
                    out += ' else { ';
                }
            }
            else {
                out += ' if (!' + ($nextValid) + ') { ';
            }
            if ($elsePresent) {
                $it.schema = it.schema['else'];
                $it.schemaPath = it.schemaPath + '.else';
                $it.errSchemaPath = it.errSchemaPath + '/else';
                out += '  ' + (it.validate($it)) + ' ';
                $it.baseId = $currentBaseId;
                out += ' ' + ($valid) + ' = ' + ($nextValid) + '; ';
                if ($thenPresent && $elsePresent) {
                    $ifClause = 'ifClause' + $lvl;
                    out += ' var ' + ($ifClause) + ' = \'else\'; ';
                }
                else {
                    $ifClause = '\'else\'';
                }
                out += ' } ';
            }
            out += ' if (!' + ($valid) + ') {   var err =   '; /* istanbul ignore else */
            if (it.createErrors !== false) {
                out += ' { keyword: \'' + ('if') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { failingKeyword: ' + ($ifClause) + ' } ';
                if (it.opts.messages !== false) {
                    out += ' , message: \'should match "\' + ' + ($ifClause) + ' + \'" schema\' ';
                }
                if (it.opts.verbose) {
                    out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                }
                out += ' } ';
            }
            else {
                out += ' {} ';
            }
            out += ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
            if (!it.compositeRule && $breakOnError) {
                /* istanbul ignore if */
                if (it.async) {
                    out += ' throw new ValidationError(vErrors); ';
                }
                else {
                    out += ' validate.errors = vErrors; return false; ';
                }
            }
            out += ' }   ';
            if ($breakOnError) {
                out += ' else { ';
            }
            out = it.util.cleanUpCode(out);
        }
        else {
            if ($breakOnError) {
                out += ' if (true) { ';
            }
        }
        return out;
    };
},
/* ajv/lib/dotjs/items.js */ function _(require, module, exports) {
    module.exports = function generate_items(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $valid = 'valid' + $lvl;
        var $errs = 'errs__' + $lvl;
        var $it = it.util.copy(it);
        var $closingBraces = '';
        $it.level++;
        var $nextValid = 'valid' + $it.level;
        var $idx = 'i' + $lvl, $dataNxt = $it.dataLevel = it.dataLevel + 1, $nextData = 'data' + $dataNxt, $currentBaseId = it.baseId;
        out += 'var ' + ($errs) + ' = errors;var ' + ($valid) + ';';
        if (Array.isArray($schema)) {
            var $additionalItems = it.schema.additionalItems;
            if ($additionalItems === false) {
                out += ' ' + ($valid) + ' = ' + ($data) + '.length <= ' + ($schema.length) + '; ';
                var $currErrSchemaPath = $errSchemaPath;
                $errSchemaPath = it.errSchemaPath + '/additionalItems';
                out += '  if (!' + ($valid) + ') {   ';
                var $$outStack = $$outStack || [];
                $$outStack.push(out);
                out = ''; /* istanbul ignore else */
                if (it.createErrors !== false) {
                    out += ' { keyword: \'' + ('additionalItems') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { limit: ' + ($schema.length) + ' } ';
                    if (it.opts.messages !== false) {
                        out += ' , message: \'should NOT have more than ' + ($schema.length) + ' items\' ';
                    }
                    if (it.opts.verbose) {
                        out += ' , schema: false , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                    }
                    out += ' } ';
                }
                else {
                    out += ' {} ';
                }
                var __err = out;
                out = $$outStack.pop();
                if (!it.compositeRule && $breakOnError) {
                    /* istanbul ignore if */
                    if (it.async) {
                        out += ' throw new ValidationError([' + (__err) + ']); ';
                    }
                    else {
                        out += ' validate.errors = [' + (__err) + ']; return false; ';
                    }
                }
                else {
                    out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                }
                out += ' } ';
                $errSchemaPath = $currErrSchemaPath;
                if ($breakOnError) {
                    $closingBraces += '}';
                    out += ' else { ';
                }
            }
            var arr1 = $schema;
            if (arr1) {
                var $sch, $i = -1, l1 = arr1.length - 1;
                while ($i < l1) {
                    $sch = arr1[$i += 1];
                    if ((it.opts.strictKeywords ? typeof $sch == 'object' && Object.keys($sch).length > 0 : it.util.schemaHasRules($sch, it.RULES.all))) {
                        out += ' ' + ($nextValid) + ' = true; if (' + ($data) + '.length > ' + ($i) + ') { ';
                        var $passData = $data + '[' + $i + ']';
                        $it.schema = $sch;
                        $it.schemaPath = $schemaPath + '[' + $i + ']';
                        $it.errSchemaPath = $errSchemaPath + '/' + $i;
                        $it.errorPath = it.util.getPathExpr(it.errorPath, $i, it.opts.jsonPointers, true);
                        $it.dataPathArr[$dataNxt] = $i;
                        var $code = it.validate($it);
                        $it.baseId = $currentBaseId;
                        if (it.util.varOccurences($code, $nextData) < 2) {
                            out += ' ' + (it.util.varReplace($code, $nextData, $passData)) + ' ';
                        }
                        else {
                            out += ' var ' + ($nextData) + ' = ' + ($passData) + '; ' + ($code) + ' ';
                        }
                        out += ' }  ';
                        if ($breakOnError) {
                            out += ' if (' + ($nextValid) + ') { ';
                            $closingBraces += '}';
                        }
                    }
                }
            }
            if (typeof $additionalItems == 'object' && (it.opts.strictKeywords ? typeof $additionalItems == 'object' && Object.keys($additionalItems).length > 0 : it.util.schemaHasRules($additionalItems, it.RULES.all))) {
                $it.schema = $additionalItems;
                $it.schemaPath = it.schemaPath + '.additionalItems';
                $it.errSchemaPath = it.errSchemaPath + '/additionalItems';
                out += ' ' + ($nextValid) + ' = true; if (' + ($data) + '.length > ' + ($schema.length) + ') {  for (var ' + ($idx) + ' = ' + ($schema.length) + '; ' + ($idx) + ' < ' + ($data) + '.length; ' + ($idx) + '++) { ';
                $it.errorPath = it.util.getPathExpr(it.errorPath, $idx, it.opts.jsonPointers, true);
                var $passData = $data + '[' + $idx + ']';
                $it.dataPathArr[$dataNxt] = $idx;
                var $code = it.validate($it);
                $it.baseId = $currentBaseId;
                if (it.util.varOccurences($code, $nextData) < 2) {
                    out += ' ' + (it.util.varReplace($code, $nextData, $passData)) + ' ';
                }
                else {
                    out += ' var ' + ($nextData) + ' = ' + ($passData) + '; ' + ($code) + ' ';
                }
                if ($breakOnError) {
                    out += ' if (!' + ($nextValid) + ') break; ';
                }
                out += ' } }  ';
                if ($breakOnError) {
                    out += ' if (' + ($nextValid) + ') { ';
                    $closingBraces += '}';
                }
            }
        }
        else if ((it.opts.strictKeywords ? typeof $schema == 'object' && Object.keys($schema).length > 0 : it.util.schemaHasRules($schema, it.RULES.all))) {
            $it.schema = $schema;
            $it.schemaPath = $schemaPath;
            $it.errSchemaPath = $errSchemaPath;
            out += '  for (var ' + ($idx) + ' = ' + (0) + '; ' + ($idx) + ' < ' + ($data) + '.length; ' + ($idx) + '++) { ';
            $it.errorPath = it.util.getPathExpr(it.errorPath, $idx, it.opts.jsonPointers, true);
            var $passData = $data + '[' + $idx + ']';
            $it.dataPathArr[$dataNxt] = $idx;
            var $code = it.validate($it);
            $it.baseId = $currentBaseId;
            if (it.util.varOccurences($code, $nextData) < 2) {
                out += ' ' + (it.util.varReplace($code, $nextData, $passData)) + ' ';
            }
            else {
                out += ' var ' + ($nextData) + ' = ' + ($passData) + '; ' + ($code) + ' ';
            }
            if ($breakOnError) {
                out += ' if (!' + ($nextValid) + ') break; ';
            }
            out += ' }';
        }
        if ($breakOnError) {
            out += ' ' + ($closingBraces) + ' if (' + ($errs) + ' == errors) {';
        }
        out = it.util.cleanUpCode(out);
        return out;
    };
},
/* ajv/lib/dotjs/_limit.js */ function _(require, module, exports) {
    module.exports = function generate__limit(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $errorKeyword;
        var $data = 'data' + ($dataLvl || '');
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        var $isMax = $keyword == 'maximum', $exclusiveKeyword = $isMax ? 'exclusiveMaximum' : 'exclusiveMinimum', $schemaExcl = it.schema[$exclusiveKeyword], $isDataExcl = it.opts.$data && $schemaExcl && $schemaExcl.$data, $op = $isMax ? '<' : '>', $notOp = $isMax ? '>' : '<', $errorKeyword = undefined;
        if ($isDataExcl) {
            var $schemaValueExcl = it.util.getData($schemaExcl.$data, $dataLvl, it.dataPathArr), $exclusive = 'exclusive' + $lvl, $exclType = 'exclType' + $lvl, $exclIsNumber = 'exclIsNumber' + $lvl, $opExpr = 'op' + $lvl, $opStr = '\' + ' + $opExpr + ' + \'';
            out += ' var schemaExcl' + ($lvl) + ' = ' + ($schemaValueExcl) + '; ';
            $schemaValueExcl = 'schemaExcl' + $lvl;
            out += ' var ' + ($exclusive) + '; var ' + ($exclType) + ' = typeof ' + ($schemaValueExcl) + '; if (' + ($exclType) + ' != \'boolean\' && ' + ($exclType) + ' != \'undefined\' && ' + ($exclType) + ' != \'number\') { ';
            var $errorKeyword = $exclusiveKeyword;
            var $$outStack = $$outStack || [];
            $$outStack.push(out);
            out = ''; /* istanbul ignore else */
            if (it.createErrors !== false) {
                out += ' { keyword: \'' + ($errorKeyword || '_exclusiveLimit') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: {} ';
                if (it.opts.messages !== false) {
                    out += ' , message: \'' + ($exclusiveKeyword) + ' should be boolean\' ';
                }
                if (it.opts.verbose) {
                    out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                }
                out += ' } ';
            }
            else {
                out += ' {} ';
            }
            var __err = out;
            out = $$outStack.pop();
            if (!it.compositeRule && $breakOnError) {
                /* istanbul ignore if */
                if (it.async) {
                    out += ' throw new ValidationError([' + (__err) + ']); ';
                }
                else {
                    out += ' validate.errors = [' + (__err) + ']; return false; ';
                }
            }
            else {
                out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
            }
            out += ' } else if ( ';
            if ($isData) {
                out += ' (' + ($schemaValue) + ' !== undefined && typeof ' + ($schemaValue) + ' != \'number\') || ';
            }
            out += ' ' + ($exclType) + ' == \'number\' ? ( (' + ($exclusive) + ' = ' + ($schemaValue) + ' === undefined || ' + ($schemaValueExcl) + ' ' + ($op) + '= ' + ($schemaValue) + ') ? ' + ($data) + ' ' + ($notOp) + '= ' + ($schemaValueExcl) + ' : ' + ($data) + ' ' + ($notOp) + ' ' + ($schemaValue) + ' ) : ( (' + ($exclusive) + ' = ' + ($schemaValueExcl) + ' === true) ? ' + ($data) + ' ' + ($notOp) + '= ' + ($schemaValue) + ' : ' + ($data) + ' ' + ($notOp) + ' ' + ($schemaValue) + ' ) || ' + ($data) + ' !== ' + ($data) + ') { var op' + ($lvl) + ' = ' + ($exclusive) + ' ? \'' + ($op) + '\' : \'' + ($op) + '=\'; ';
            if ($schema === undefined) {
                $errorKeyword = $exclusiveKeyword;
                $errSchemaPath = it.errSchemaPath + '/' + $exclusiveKeyword;
                $schemaValue = $schemaValueExcl;
                $isData = $isDataExcl;
            }
        }
        else {
            var $exclIsNumber = typeof $schemaExcl == 'number', $opStr = $op;
            if ($exclIsNumber && $isData) {
                var $opExpr = '\'' + $opStr + '\'';
                out += ' if ( ';
                if ($isData) {
                    out += ' (' + ($schemaValue) + ' !== undefined && typeof ' + ($schemaValue) + ' != \'number\') || ';
                }
                out += ' ( ' + ($schemaValue) + ' === undefined || ' + ($schemaExcl) + ' ' + ($op) + '= ' + ($schemaValue) + ' ? ' + ($data) + ' ' + ($notOp) + '= ' + ($schemaExcl) + ' : ' + ($data) + ' ' + ($notOp) + ' ' + ($schemaValue) + ' ) || ' + ($data) + ' !== ' + ($data) + ') { ';
            }
            else {
                if ($exclIsNumber && $schema === undefined) {
                    $exclusive = true;
                    $errorKeyword = $exclusiveKeyword;
                    $errSchemaPath = it.errSchemaPath + '/' + $exclusiveKeyword;
                    $schemaValue = $schemaExcl;
                    $notOp += '=';
                }
                else {
                    if ($exclIsNumber)
                        $schemaValue = Math[$isMax ? 'min' : 'max']($schemaExcl, $schema);
                    if ($schemaExcl === ($exclIsNumber ? $schemaValue : true)) {
                        $exclusive = true;
                        $errorKeyword = $exclusiveKeyword;
                        $errSchemaPath = it.errSchemaPath + '/' + $exclusiveKeyword;
                        $notOp += '=';
                    }
                    else {
                        $exclusive = false;
                        $opStr += '=';
                    }
                }
                var $opExpr = '\'' + $opStr + '\'';
                out += ' if ( ';
                if ($isData) {
                    out += ' (' + ($schemaValue) + ' !== undefined && typeof ' + ($schemaValue) + ' != \'number\') || ';
                }
                out += ' ' + ($data) + ' ' + ($notOp) + ' ' + ($schemaValue) + ' || ' + ($data) + ' !== ' + ($data) + ') { ';
            }
        }
        $errorKeyword = $errorKeyword || $keyword;
        var $$outStack = $$outStack || [];
        $$outStack.push(out);
        out = ''; /* istanbul ignore else */
        if (it.createErrors !== false) {
            out += ' { keyword: \'' + ($errorKeyword || '_limit') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { comparison: ' + ($opExpr) + ', limit: ' + ($schemaValue) + ', exclusive: ' + ($exclusive) + ' } ';
            if (it.opts.messages !== false) {
                out += ' , message: \'should be ' + ($opStr) + ' ';
                if ($isData) {
                    out += '\' + ' + ($schemaValue);
                }
                else {
                    out += '' + ($schemaValue) + '\'';
                }
            }
            if (it.opts.verbose) {
                out += ' , schema:  ';
                if ($isData) {
                    out += 'validate.schema' + ($schemaPath);
                }
                else {
                    out += '' + ($schema);
                }
                out += '         , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
            }
            out += ' } ';
        }
        else {
            out += ' {} ';
        }
        var __err = out;
        out = $$outStack.pop();
        if (!it.compositeRule && $breakOnError) {
            /* istanbul ignore if */
            if (it.async) {
                out += ' throw new ValidationError([' + (__err) + ']); ';
            }
            else {
                out += ' validate.errors = [' + (__err) + ']; return false; ';
            }
        }
        else {
            out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
        }
        out += ' } ';
        if ($breakOnError) {
            out += ' else { ';
        }
        return out;
    };
},
/* ajv/lib/dotjs/_limitItems.js */ function _(require, module, exports) {
    module.exports = function generate__limitItems(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $errorKeyword;
        var $data = 'data' + ($dataLvl || '');
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        var $op = $keyword == 'maxItems' ? '>' : '<';
        out += 'if ( ';
        if ($isData) {
            out += ' (' + ($schemaValue) + ' !== undefined && typeof ' + ($schemaValue) + ' != \'number\') || ';
        }
        out += ' ' + ($data) + '.length ' + ($op) + ' ' + ($schemaValue) + ') { ';
        var $errorKeyword = $keyword;
        var $$outStack = $$outStack || [];
        $$outStack.push(out);
        out = ''; /* istanbul ignore else */
        if (it.createErrors !== false) {
            out += ' { keyword: \'' + ($errorKeyword || '_limitItems') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { limit: ' + ($schemaValue) + ' } ';
            if (it.opts.messages !== false) {
                out += ' , message: \'should NOT have ';
                if ($keyword == 'maxItems') {
                    out += 'more';
                }
                else {
                    out += 'fewer';
                }
                out += ' than ';
                if ($isData) {
                    out += '\' + ' + ($schemaValue) + ' + \'';
                }
                else {
                    out += '' + ($schema);
                }
                out += ' items\' ';
            }
            if (it.opts.verbose) {
                out += ' , schema:  ';
                if ($isData) {
                    out += 'validate.schema' + ($schemaPath);
                }
                else {
                    out += '' + ($schema);
                }
                out += '         , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
            }
            out += ' } ';
        }
        else {
            out += ' {} ';
        }
        var __err = out;
        out = $$outStack.pop();
        if (!it.compositeRule && $breakOnError) {
            /* istanbul ignore if */
            if (it.async) {
                out += ' throw new ValidationError([' + (__err) + ']); ';
            }
            else {
                out += ' validate.errors = [' + (__err) + ']; return false; ';
            }
        }
        else {
            out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
        }
        out += '} ';
        if ($breakOnError) {
            out += ' else { ';
        }
        return out;
    };
},
/* ajv/lib/dotjs/_limitLength.js */ function _(require, module, exports) {
    module.exports = function generate__limitLength(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $errorKeyword;
        var $data = 'data' + ($dataLvl || '');
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        var $op = $keyword == 'maxLength' ? '>' : '<';
        out += 'if ( ';
        if ($isData) {
            out += ' (' + ($schemaValue) + ' !== undefined && typeof ' + ($schemaValue) + ' != \'number\') || ';
        }
        if (it.opts.unicode === false) {
            out += ' ' + ($data) + '.length ';
        }
        else {
            out += ' ucs2length(' + ($data) + ') ';
        }
        out += ' ' + ($op) + ' ' + ($schemaValue) + ') { ';
        var $errorKeyword = $keyword;
        var $$outStack = $$outStack || [];
        $$outStack.push(out);
        out = ''; /* istanbul ignore else */
        if (it.createErrors !== false) {
            out += ' { keyword: \'' + ($errorKeyword || '_limitLength') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { limit: ' + ($schemaValue) + ' } ';
            if (it.opts.messages !== false) {
                out += ' , message: \'should NOT be ';
                if ($keyword == 'maxLength') {
                    out += 'longer';
                }
                else {
                    out += 'shorter';
                }
                out += ' than ';
                if ($isData) {
                    out += '\' + ' + ($schemaValue) + ' + \'';
                }
                else {
                    out += '' + ($schema);
                }
                out += ' characters\' ';
            }
            if (it.opts.verbose) {
                out += ' , schema:  ';
                if ($isData) {
                    out += 'validate.schema' + ($schemaPath);
                }
                else {
                    out += '' + ($schema);
                }
                out += '         , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
            }
            out += ' } ';
        }
        else {
            out += ' {} ';
        }
        var __err = out;
        out = $$outStack.pop();
        if (!it.compositeRule && $breakOnError) {
            /* istanbul ignore if */
            if (it.async) {
                out += ' throw new ValidationError([' + (__err) + ']); ';
            }
            else {
                out += ' validate.errors = [' + (__err) + ']; return false; ';
            }
        }
        else {
            out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
        }
        out += '} ';
        if ($breakOnError) {
            out += ' else { ';
        }
        return out;
    };
},
/* ajv/lib/dotjs/_limitProperties.js */ function _(require, module, exports) {
    module.exports = function generate__limitProperties(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $errorKeyword;
        var $data = 'data' + ($dataLvl || '');
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        var $op = $keyword == 'maxProperties' ? '>' : '<';
        out += 'if ( ';
        if ($isData) {
            out += ' (' + ($schemaValue) + ' !== undefined && typeof ' + ($schemaValue) + ' != \'number\') || ';
        }
        out += ' Object.keys(' + ($data) + ').length ' + ($op) + ' ' + ($schemaValue) + ') { ';
        var $errorKeyword = $keyword;
        var $$outStack = $$outStack || [];
        $$outStack.push(out);
        out = ''; /* istanbul ignore else */
        if (it.createErrors !== false) {
            out += ' { keyword: \'' + ($errorKeyword || '_limitProperties') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { limit: ' + ($schemaValue) + ' } ';
            if (it.opts.messages !== false) {
                out += ' , message: \'should NOT have ';
                if ($keyword == 'maxProperties') {
                    out += 'more';
                }
                else {
                    out += 'fewer';
                }
                out += ' than ';
                if ($isData) {
                    out += '\' + ' + ($schemaValue) + ' + \'';
                }
                else {
                    out += '' + ($schema);
                }
                out += ' properties\' ';
            }
            if (it.opts.verbose) {
                out += ' , schema:  ';
                if ($isData) {
                    out += 'validate.schema' + ($schemaPath);
                }
                else {
                    out += '' + ($schema);
                }
                out += '         , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
            }
            out += ' } ';
        }
        else {
            out += ' {} ';
        }
        var __err = out;
        out = $$outStack.pop();
        if (!it.compositeRule && $breakOnError) {
            /* istanbul ignore if */
            if (it.async) {
                out += ' throw new ValidationError([' + (__err) + ']); ';
            }
            else {
                out += ' validate.errors = [' + (__err) + ']; return false; ';
            }
        }
        else {
            out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
        }
        out += '} ';
        if ($breakOnError) {
            out += ' else { ';
        }
        return out;
    };
},
/* ajv/lib/dotjs/multipleOf.js */ function _(require, module, exports) {
    module.exports = function generate_multipleOf(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        out += 'var division' + ($lvl) + ';if (';
        if ($isData) {
            out += ' ' + ($schemaValue) + ' !== undefined && ( typeof ' + ($schemaValue) + ' != \'number\' || ';
        }
        out += ' (division' + ($lvl) + ' = ' + ($data) + ' / ' + ($schemaValue) + ', ';
        if (it.opts.multipleOfPrecision) {
            out += ' Math.abs(Math.round(division' + ($lvl) + ') - division' + ($lvl) + ') > 1e-' + (it.opts.multipleOfPrecision) + ' ';
        }
        else {
            out += ' division' + ($lvl) + ' !== parseInt(division' + ($lvl) + ') ';
        }
        out += ' ) ';
        if ($isData) {
            out += '  )  ';
        }
        out += ' ) {   ';
        var $$outStack = $$outStack || [];
        $$outStack.push(out);
        out = ''; /* istanbul ignore else */
        if (it.createErrors !== false) {
            out += ' { keyword: \'' + ('multipleOf') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { multipleOf: ' + ($schemaValue) + ' } ';
            if (it.opts.messages !== false) {
                out += ' , message: \'should be multiple of ';
                if ($isData) {
                    out += '\' + ' + ($schemaValue);
                }
                else {
                    out += '' + ($schemaValue) + '\'';
                }
            }
            if (it.opts.verbose) {
                out += ' , schema:  ';
                if ($isData) {
                    out += 'validate.schema' + ($schemaPath);
                }
                else {
                    out += '' + ($schema);
                }
                out += '         , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
            }
            out += ' } ';
        }
        else {
            out += ' {} ';
        }
        var __err = out;
        out = $$outStack.pop();
        if (!it.compositeRule && $breakOnError) {
            /* istanbul ignore if */
            if (it.async) {
                out += ' throw new ValidationError([' + (__err) + ']); ';
            }
            else {
                out += ' validate.errors = [' + (__err) + ']; return false; ';
            }
        }
        else {
            out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
        }
        out += '} ';
        if ($breakOnError) {
            out += ' else { ';
        }
        return out;
    };
},
/* ajv/lib/dotjs/not.js */ function _(require, module, exports) {
    module.exports = function generate_not(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $errs = 'errs__' + $lvl;
        var $it = it.util.copy(it);
        $it.level++;
        var $nextValid = 'valid' + $it.level;
        if ((it.opts.strictKeywords ? typeof $schema == 'object' && Object.keys($schema).length > 0 : it.util.schemaHasRules($schema, it.RULES.all))) {
            $it.schema = $schema;
            $it.schemaPath = $schemaPath;
            $it.errSchemaPath = $errSchemaPath;
            out += ' var ' + ($errs) + ' = errors;  ';
            var $wasComposite = it.compositeRule;
            it.compositeRule = $it.compositeRule = true;
            $it.createErrors = false;
            var $allErrorsOption;
            if ($it.opts.allErrors) {
                $allErrorsOption = $it.opts.allErrors;
                $it.opts.allErrors = false;
            }
            out += ' ' + (it.validate($it)) + ' ';
            $it.createErrors = true;
            if ($allErrorsOption)
                $it.opts.allErrors = $allErrorsOption;
            it.compositeRule = $it.compositeRule = $wasComposite;
            out += ' if (' + ($nextValid) + ') {   ';
            var $$outStack = $$outStack || [];
            $$outStack.push(out);
            out = ''; /* istanbul ignore else */
            if (it.createErrors !== false) {
                out += ' { keyword: \'' + ('not') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: {} ';
                if (it.opts.messages !== false) {
                    out += ' , message: \'should NOT be valid\' ';
                }
                if (it.opts.verbose) {
                    out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                }
                out += ' } ';
            }
            else {
                out += ' {} ';
            }
            var __err = out;
            out = $$outStack.pop();
            if (!it.compositeRule && $breakOnError) {
                /* istanbul ignore if */
                if (it.async) {
                    out += ' throw new ValidationError([' + (__err) + ']); ';
                }
                else {
                    out += ' validate.errors = [' + (__err) + ']; return false; ';
                }
            }
            else {
                out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
            }
            out += ' } else {  errors = ' + ($errs) + '; if (vErrors !== null) { if (' + ($errs) + ') vErrors.length = ' + ($errs) + '; else vErrors = null; } ';
            if (it.opts.allErrors) {
                out += ' } ';
            }
        }
        else {
            out += '  var err =   '; /* istanbul ignore else */
            if (it.createErrors !== false) {
                out += ' { keyword: \'' + ('not') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: {} ';
                if (it.opts.messages !== false) {
                    out += ' , message: \'should NOT be valid\' ';
                }
                if (it.opts.verbose) {
                    out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                }
                out += ' } ';
            }
            else {
                out += ' {} ';
            }
            out += ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
            if ($breakOnError) {
                out += ' if (false) { ';
            }
        }
        return out;
    };
},
/* ajv/lib/dotjs/oneOf.js */ function _(require, module, exports) {
    module.exports = function generate_oneOf(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $valid = 'valid' + $lvl;
        var $errs = 'errs__' + $lvl;
        var $it = it.util.copy(it);
        var $closingBraces = '';
        $it.level++;
        var $nextValid = 'valid' + $it.level;
        var $currentBaseId = $it.baseId, $prevValid = 'prevValid' + $lvl, $passingSchemas = 'passingSchemas' + $lvl;
        out += 'var ' + ($errs) + ' = errors , ' + ($prevValid) + ' = false , ' + ($valid) + ' = false , ' + ($passingSchemas) + ' = null; ';
        var $wasComposite = it.compositeRule;
        it.compositeRule = $it.compositeRule = true;
        var arr1 = $schema;
        if (arr1) {
            var $sch, $i = -1, l1 = arr1.length - 1;
            while ($i < l1) {
                $sch = arr1[$i += 1];
                if ((it.opts.strictKeywords ? typeof $sch == 'object' && Object.keys($sch).length > 0 : it.util.schemaHasRules($sch, it.RULES.all))) {
                    $it.schema = $sch;
                    $it.schemaPath = $schemaPath + '[' + $i + ']';
                    $it.errSchemaPath = $errSchemaPath + '/' + $i;
                    out += '  ' + (it.validate($it)) + ' ';
                    $it.baseId = $currentBaseId;
                }
                else {
                    out += ' var ' + ($nextValid) + ' = true; ';
                }
                if ($i) {
                    out += ' if (' + ($nextValid) + ' && ' + ($prevValid) + ') { ' + ($valid) + ' = false; ' + ($passingSchemas) + ' = [' + ($passingSchemas) + ', ' + ($i) + ']; } else { ';
                    $closingBraces += '}';
                }
                out += ' if (' + ($nextValid) + ') { ' + ($valid) + ' = ' + ($prevValid) + ' = true; ' + ($passingSchemas) + ' = ' + ($i) + '; }';
            }
        }
        it.compositeRule = $it.compositeRule = $wasComposite;
        out += '' + ($closingBraces) + 'if (!' + ($valid) + ') {   var err =   '; /* istanbul ignore else */
        if (it.createErrors !== false) {
            out += ' { keyword: \'' + ('oneOf') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { passingSchemas: ' + ($passingSchemas) + ' } ';
            if (it.opts.messages !== false) {
                out += ' , message: \'should match exactly one schema in oneOf\' ';
            }
            if (it.opts.verbose) {
                out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
            }
            out += ' } ';
        }
        else {
            out += ' {} ';
        }
        out += ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
        if (!it.compositeRule && $breakOnError) {
            /* istanbul ignore if */
            if (it.async) {
                out += ' throw new ValidationError(vErrors); ';
            }
            else {
                out += ' validate.errors = vErrors; return false; ';
            }
        }
        out += '} else {  errors = ' + ($errs) + '; if (vErrors !== null) { if (' + ($errs) + ') vErrors.length = ' + ($errs) + '; else vErrors = null; }';
        if (it.opts.allErrors) {
            out += ' } ';
        }
        return out;
    };
},
/* ajv/lib/dotjs/pattern.js */ function _(require, module, exports) {
    module.exports = function generate_pattern(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        var $regexp = $isData ? '(new RegExp(' + $schemaValue + '))' : it.usePattern($schema);
        out += 'if ( ';
        if ($isData) {
            out += ' (' + ($schemaValue) + ' !== undefined && typeof ' + ($schemaValue) + ' != \'string\') || ';
        }
        out += ' !' + ($regexp) + '.test(' + ($data) + ') ) {   ';
        var $$outStack = $$outStack || [];
        $$outStack.push(out);
        out = ''; /* istanbul ignore else */
        if (it.createErrors !== false) {
            out += ' { keyword: \'' + ('pattern') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { pattern:  ';
            if ($isData) {
                out += '' + ($schemaValue);
            }
            else {
                out += '' + (it.util.toQuotedString($schema));
            }
            out += '  } ';
            if (it.opts.messages !== false) {
                out += ' , message: \'should match pattern "';
                if ($isData) {
                    out += '\' + ' + ($schemaValue) + ' + \'';
                }
                else {
                    out += '' + (it.util.escapeQuotes($schema));
                }
                out += '"\' ';
            }
            if (it.opts.verbose) {
                out += ' , schema:  ';
                if ($isData) {
                    out += 'validate.schema' + ($schemaPath);
                }
                else {
                    out += '' + (it.util.toQuotedString($schema));
                }
                out += '         , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
            }
            out += ' } ';
        }
        else {
            out += ' {} ';
        }
        var __err = out;
        out = $$outStack.pop();
        if (!it.compositeRule && $breakOnError) {
            /* istanbul ignore if */
            if (it.async) {
                out += ' throw new ValidationError([' + (__err) + ']); ';
            }
            else {
                out += ' validate.errors = [' + (__err) + ']; return false; ';
            }
        }
        else {
            out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
        }
        out += '} ';
        if ($breakOnError) {
            out += ' else { ';
        }
        return out;
    };
},
/* ajv/lib/dotjs/properties.js */ function _(require, module, exports) {
    module.exports = function generate_properties(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $errs = 'errs__' + $lvl;
        var $it = it.util.copy(it);
        var $closingBraces = '';
        $it.level++;
        var $nextValid = 'valid' + $it.level;
        var $key = 'key' + $lvl, $idx = 'idx' + $lvl, $dataNxt = $it.dataLevel = it.dataLevel + 1, $nextData = 'data' + $dataNxt, $dataProperties = 'dataProperties' + $lvl;
        var $schemaKeys = Object.keys($schema || {}), $pProperties = it.schema.patternProperties || {}, $pPropertyKeys = Object.keys($pProperties), $aProperties = it.schema.additionalProperties, $someProperties = $schemaKeys.length || $pPropertyKeys.length, $noAdditional = $aProperties === false, $additionalIsSchema = typeof $aProperties == 'object' && Object.keys($aProperties).length, $removeAdditional = it.opts.removeAdditional, $checkAdditional = $noAdditional || $additionalIsSchema || $removeAdditional, $ownProperties = it.opts.ownProperties, $currentBaseId = it.baseId;
        var $required = it.schema.required;
        if ($required && !(it.opts.$data && $required.$data) && $required.length < it.opts.loopRequired)
            var $requiredHash = it.util.toHash($required);
        out += 'var ' + ($errs) + ' = errors;var ' + ($nextValid) + ' = true;';
        if ($ownProperties) {
            out += ' var ' + ($dataProperties) + ' = undefined;';
        }
        if ($checkAdditional) {
            if ($ownProperties) {
                out += ' ' + ($dataProperties) + ' = ' + ($dataProperties) + ' || Object.keys(' + ($data) + '); for (var ' + ($idx) + '=0; ' + ($idx) + '<' + ($dataProperties) + '.length; ' + ($idx) + '++) { var ' + ($key) + ' = ' + ($dataProperties) + '[' + ($idx) + ']; ';
            }
            else {
                out += ' for (var ' + ($key) + ' in ' + ($data) + ') { ';
            }
            if ($someProperties) {
                out += ' var isAdditional' + ($lvl) + ' = !(false ';
                if ($schemaKeys.length) {
                    if ($schemaKeys.length > 8) {
                        out += ' || validate.schema' + ($schemaPath) + '.hasOwnProperty(' + ($key) + ') ';
                    }
                    else {
                        var arr1 = $schemaKeys;
                        if (arr1) {
                            var $propertyKey, i1 = -1, l1 = arr1.length - 1;
                            while (i1 < l1) {
                                $propertyKey = arr1[i1 += 1];
                                out += ' || ' + ($key) + ' == ' + (it.util.toQuotedString($propertyKey)) + ' ';
                            }
                        }
                    }
                }
                if ($pPropertyKeys.length) {
                    var arr2 = $pPropertyKeys;
                    if (arr2) {
                        var $pProperty, $i = -1, l2 = arr2.length - 1;
                        while ($i < l2) {
                            $pProperty = arr2[$i += 1];
                            out += ' || ' + (it.usePattern($pProperty)) + '.test(' + ($key) + ') ';
                        }
                    }
                }
                out += ' ); if (isAdditional' + ($lvl) + ') { ';
            }
            if ($removeAdditional == 'all') {
                out += ' delete ' + ($data) + '[' + ($key) + ']; ';
            }
            else {
                var $currentErrorPath = it.errorPath;
                var $additionalProperty = '\' + ' + $key + ' + \'';
                if (it.opts._errorDataPathProperty) {
                    it.errorPath = it.util.getPathExpr(it.errorPath, $key, it.opts.jsonPointers);
                }
                if ($noAdditional) {
                    if ($removeAdditional) {
                        out += ' delete ' + ($data) + '[' + ($key) + ']; ';
                    }
                    else {
                        out += ' ' + ($nextValid) + ' = false; ';
                        var $currErrSchemaPath = $errSchemaPath;
                        $errSchemaPath = it.errSchemaPath + '/additionalProperties';
                        var $$outStack = $$outStack || [];
                        $$outStack.push(out);
                        out = ''; /* istanbul ignore else */
                        if (it.createErrors !== false) {
                            out += ' { keyword: \'' + ('additionalProperties') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { additionalProperty: \'' + ($additionalProperty) + '\' } ';
                            if (it.opts.messages !== false) {
                                out += ' , message: \'';
                                if (it.opts._errorDataPathProperty) {
                                    out += 'is an invalid additional property';
                                }
                                else {
                                    out += 'should NOT have additional properties';
                                }
                                out += '\' ';
                            }
                            if (it.opts.verbose) {
                                out += ' , schema: false , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                            }
                            out += ' } ';
                        }
                        else {
                            out += ' {} ';
                        }
                        var __err = out;
                        out = $$outStack.pop();
                        if (!it.compositeRule && $breakOnError) {
                            /* istanbul ignore if */
                            if (it.async) {
                                out += ' throw new ValidationError([' + (__err) + ']); ';
                            }
                            else {
                                out += ' validate.errors = [' + (__err) + ']; return false; ';
                            }
                        }
                        else {
                            out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                        }
                        $errSchemaPath = $currErrSchemaPath;
                        if ($breakOnError) {
                            out += ' break; ';
                        }
                    }
                }
                else if ($additionalIsSchema) {
                    if ($removeAdditional == 'failing') {
                        out += ' var ' + ($errs) + ' = errors;  ';
                        var $wasComposite = it.compositeRule;
                        it.compositeRule = $it.compositeRule = true;
                        $it.schema = $aProperties;
                        $it.schemaPath = it.schemaPath + '.additionalProperties';
                        $it.errSchemaPath = it.errSchemaPath + '/additionalProperties';
                        $it.errorPath = it.opts._errorDataPathProperty ? it.errorPath : it.util.getPathExpr(it.errorPath, $key, it.opts.jsonPointers);
                        var $passData = $data + '[' + $key + ']';
                        $it.dataPathArr[$dataNxt] = $key;
                        var $code = it.validate($it);
                        $it.baseId = $currentBaseId;
                        if (it.util.varOccurences($code, $nextData) < 2) {
                            out += ' ' + (it.util.varReplace($code, $nextData, $passData)) + ' ';
                        }
                        else {
                            out += ' var ' + ($nextData) + ' = ' + ($passData) + '; ' + ($code) + ' ';
                        }
                        out += ' if (!' + ($nextValid) + ') { errors = ' + ($errs) + '; if (validate.errors !== null) { if (errors) validate.errors.length = errors; else validate.errors = null; } delete ' + ($data) + '[' + ($key) + ']; }  ';
                        it.compositeRule = $it.compositeRule = $wasComposite;
                    }
                    else {
                        $it.schema = $aProperties;
                        $it.schemaPath = it.schemaPath + '.additionalProperties';
                        $it.errSchemaPath = it.errSchemaPath + '/additionalProperties';
                        $it.errorPath = it.opts._errorDataPathProperty ? it.errorPath : it.util.getPathExpr(it.errorPath, $key, it.opts.jsonPointers);
                        var $passData = $data + '[' + $key + ']';
                        $it.dataPathArr[$dataNxt] = $key;
                        var $code = it.validate($it);
                        $it.baseId = $currentBaseId;
                        if (it.util.varOccurences($code, $nextData) < 2) {
                            out += ' ' + (it.util.varReplace($code, $nextData, $passData)) + ' ';
                        }
                        else {
                            out += ' var ' + ($nextData) + ' = ' + ($passData) + '; ' + ($code) + ' ';
                        }
                        if ($breakOnError) {
                            out += ' if (!' + ($nextValid) + ') break; ';
                        }
                    }
                }
                it.errorPath = $currentErrorPath;
            }
            if ($someProperties) {
                out += ' } ';
            }
            out += ' }  ';
            if ($breakOnError) {
                out += ' if (' + ($nextValid) + ') { ';
                $closingBraces += '}';
            }
        }
        var $useDefaults = it.opts.useDefaults && !it.compositeRule;
        if ($schemaKeys.length) {
            var arr3 = $schemaKeys;
            if (arr3) {
                var $propertyKey, i3 = -1, l3 = arr3.length - 1;
                while (i3 < l3) {
                    $propertyKey = arr3[i3 += 1];
                    var $sch = $schema[$propertyKey];
                    if ((it.opts.strictKeywords ? typeof $sch == 'object' && Object.keys($sch).length > 0 : it.util.schemaHasRules($sch, it.RULES.all))) {
                        var $prop = it.util.getProperty($propertyKey), $passData = $data + $prop, $hasDefault = $useDefaults && $sch.default !== undefined;
                        $it.schema = $sch;
                        $it.schemaPath = $schemaPath + $prop;
                        $it.errSchemaPath = $errSchemaPath + '/' + it.util.escapeFragment($propertyKey);
                        $it.errorPath = it.util.getPath(it.errorPath, $propertyKey, it.opts.jsonPointers);
                        $it.dataPathArr[$dataNxt] = it.util.toQuotedString($propertyKey);
                        var $code = it.validate($it);
                        $it.baseId = $currentBaseId;
                        if (it.util.varOccurences($code, $nextData) < 2) {
                            $code = it.util.varReplace($code, $nextData, $passData);
                            var $useData = $passData;
                        }
                        else {
                            var $useData = $nextData;
                            out += ' var ' + ($nextData) + ' = ' + ($passData) + '; ';
                        }
                        if ($hasDefault) {
                            out += ' ' + ($code) + ' ';
                        }
                        else {
                            if ($requiredHash && $requiredHash[$propertyKey]) {
                                out += ' if ( ' + ($useData) + ' === undefined ';
                                if ($ownProperties) {
                                    out += ' || ! Object.prototype.hasOwnProperty.call(' + ($data) + ', \'' + (it.util.escapeQuotes($propertyKey)) + '\') ';
                                }
                                out += ') { ' + ($nextValid) + ' = false; ';
                                var $currentErrorPath = it.errorPath, $currErrSchemaPath = $errSchemaPath, $missingProperty = it.util.escapeQuotes($propertyKey);
                                if (it.opts._errorDataPathProperty) {
                                    it.errorPath = it.util.getPath($currentErrorPath, $propertyKey, it.opts.jsonPointers);
                                }
                                $errSchemaPath = it.errSchemaPath + '/required';
                                var $$outStack = $$outStack || [];
                                $$outStack.push(out);
                                out = ''; /* istanbul ignore else */
                                if (it.createErrors !== false) {
                                    out += ' { keyword: \'' + ('required') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { missingProperty: \'' + ($missingProperty) + '\' } ';
                                    if (it.opts.messages !== false) {
                                        out += ' , message: \'';
                                        if (it.opts._errorDataPathProperty) {
                                            out += 'is a required property';
                                        }
                                        else {
                                            out += 'should have required property \\\'' + ($missingProperty) + '\\\'';
                                        }
                                        out += '\' ';
                                    }
                                    if (it.opts.verbose) {
                                        out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                                    }
                                    out += ' } ';
                                }
                                else {
                                    out += ' {} ';
                                }
                                var __err = out;
                                out = $$outStack.pop();
                                if (!it.compositeRule && $breakOnError) {
                                    /* istanbul ignore if */
                                    if (it.async) {
                                        out += ' throw new ValidationError([' + (__err) + ']); ';
                                    }
                                    else {
                                        out += ' validate.errors = [' + (__err) + ']; return false; ';
                                    }
                                }
                                else {
                                    out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                                }
                                $errSchemaPath = $currErrSchemaPath;
                                it.errorPath = $currentErrorPath;
                                out += ' } else { ';
                            }
                            else {
                                if ($breakOnError) {
                                    out += ' if ( ' + ($useData) + ' === undefined ';
                                    if ($ownProperties) {
                                        out += ' || ! Object.prototype.hasOwnProperty.call(' + ($data) + ', \'' + (it.util.escapeQuotes($propertyKey)) + '\') ';
                                    }
                                    out += ') { ' + ($nextValid) + ' = true; } else { ';
                                }
                                else {
                                    out += ' if (' + ($useData) + ' !== undefined ';
                                    if ($ownProperties) {
                                        out += ' &&   Object.prototype.hasOwnProperty.call(' + ($data) + ', \'' + (it.util.escapeQuotes($propertyKey)) + '\') ';
                                    }
                                    out += ' ) { ';
                                }
                            }
                            out += ' ' + ($code) + ' } ';
                        }
                    }
                    if ($breakOnError) {
                        out += ' if (' + ($nextValid) + ') { ';
                        $closingBraces += '}';
                    }
                }
            }
        }
        if ($pPropertyKeys.length) {
            var arr4 = $pPropertyKeys;
            if (arr4) {
                var $pProperty, i4 = -1, l4 = arr4.length - 1;
                while (i4 < l4) {
                    $pProperty = arr4[i4 += 1];
                    var $sch = $pProperties[$pProperty];
                    if ((it.opts.strictKeywords ? typeof $sch == 'object' && Object.keys($sch).length > 0 : it.util.schemaHasRules($sch, it.RULES.all))) {
                        $it.schema = $sch;
                        $it.schemaPath = it.schemaPath + '.patternProperties' + it.util.getProperty($pProperty);
                        $it.errSchemaPath = it.errSchemaPath + '/patternProperties/' + it.util.escapeFragment($pProperty);
                        if ($ownProperties) {
                            out += ' ' + ($dataProperties) + ' = ' + ($dataProperties) + ' || Object.keys(' + ($data) + '); for (var ' + ($idx) + '=0; ' + ($idx) + '<' + ($dataProperties) + '.length; ' + ($idx) + '++) { var ' + ($key) + ' = ' + ($dataProperties) + '[' + ($idx) + ']; ';
                        }
                        else {
                            out += ' for (var ' + ($key) + ' in ' + ($data) + ') { ';
                        }
                        out += ' if (' + (it.usePattern($pProperty)) + '.test(' + ($key) + ')) { ';
                        $it.errorPath = it.util.getPathExpr(it.errorPath, $key, it.opts.jsonPointers);
                        var $passData = $data + '[' + $key + ']';
                        $it.dataPathArr[$dataNxt] = $key;
                        var $code = it.validate($it);
                        $it.baseId = $currentBaseId;
                        if (it.util.varOccurences($code, $nextData) < 2) {
                            out += ' ' + (it.util.varReplace($code, $nextData, $passData)) + ' ';
                        }
                        else {
                            out += ' var ' + ($nextData) + ' = ' + ($passData) + '; ' + ($code) + ' ';
                        }
                        if ($breakOnError) {
                            out += ' if (!' + ($nextValid) + ') break; ';
                        }
                        out += ' } ';
                        if ($breakOnError) {
                            out += ' else ' + ($nextValid) + ' = true; ';
                        }
                        out += ' }  ';
                        if ($breakOnError) {
                            out += ' if (' + ($nextValid) + ') { ';
                            $closingBraces += '}';
                        }
                    }
                }
            }
        }
        if ($breakOnError) {
            out += ' ' + ($closingBraces) + ' if (' + ($errs) + ' == errors) {';
        }
        out = it.util.cleanUpCode(out);
        return out;
    };
},
/* ajv/lib/dotjs/propertyNames.js */ function _(require, module, exports) {
    module.exports = function generate_propertyNames(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $errs = 'errs__' + $lvl;
        var $it = it.util.copy(it);
        var $closingBraces = '';
        $it.level++;
        var $nextValid = 'valid' + $it.level;
        out += 'var ' + ($errs) + ' = errors;';
        if ((it.opts.strictKeywords ? typeof $schema == 'object' && Object.keys($schema).length > 0 : it.util.schemaHasRules($schema, it.RULES.all))) {
            $it.schema = $schema;
            $it.schemaPath = $schemaPath;
            $it.errSchemaPath = $errSchemaPath;
            var $key = 'key' + $lvl, $idx = 'idx' + $lvl, $i = 'i' + $lvl, $invalidName = '\' + ' + $key + ' + \'', $dataNxt = $it.dataLevel = it.dataLevel + 1, $nextData = 'data' + $dataNxt, $dataProperties = 'dataProperties' + $lvl, $ownProperties = it.opts.ownProperties, $currentBaseId = it.baseId;
            if ($ownProperties) {
                out += ' var ' + ($dataProperties) + ' = undefined; ';
            }
            if ($ownProperties) {
                out += ' ' + ($dataProperties) + ' = ' + ($dataProperties) + ' || Object.keys(' + ($data) + '); for (var ' + ($idx) + '=0; ' + ($idx) + '<' + ($dataProperties) + '.length; ' + ($idx) + '++) { var ' + ($key) + ' = ' + ($dataProperties) + '[' + ($idx) + ']; ';
            }
            else {
                out += ' for (var ' + ($key) + ' in ' + ($data) + ') { ';
            }
            out += ' var startErrs' + ($lvl) + ' = errors; ';
            var $passData = $key;
            var $wasComposite = it.compositeRule;
            it.compositeRule = $it.compositeRule = true;
            var $code = it.validate($it);
            $it.baseId = $currentBaseId;
            if (it.util.varOccurences($code, $nextData) < 2) {
                out += ' ' + (it.util.varReplace($code, $nextData, $passData)) + ' ';
            }
            else {
                out += ' var ' + ($nextData) + ' = ' + ($passData) + '; ' + ($code) + ' ';
            }
            it.compositeRule = $it.compositeRule = $wasComposite;
            out += ' if (!' + ($nextValid) + ') { for (var ' + ($i) + '=startErrs' + ($lvl) + '; ' + ($i) + '<errors; ' + ($i) + '++) { vErrors[' + ($i) + '].propertyName = ' + ($key) + '; }   var err =   '; /* istanbul ignore else */
            if (it.createErrors !== false) {
                out += ' { keyword: \'' + ('propertyNames') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { propertyName: \'' + ($invalidName) + '\' } ';
                if (it.opts.messages !== false) {
                    out += ' , message: \'property name \\\'' + ($invalidName) + '\\\' is invalid\' ';
                }
                if (it.opts.verbose) {
                    out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                }
                out += ' } ';
            }
            else {
                out += ' {} ';
            }
            out += ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
            if (!it.compositeRule && $breakOnError) {
                /* istanbul ignore if */
                if (it.async) {
                    out += ' throw new ValidationError(vErrors); ';
                }
                else {
                    out += ' validate.errors = vErrors; return false; ';
                }
            }
            if ($breakOnError) {
                out += ' break; ';
            }
            out += ' } }';
        }
        if ($breakOnError) {
            out += ' ' + ($closingBraces) + ' if (' + ($errs) + ' == errors) {';
        }
        out = it.util.cleanUpCode(out);
        return out;
    };
},
/* ajv/lib/dotjs/required.js */ function _(require, module, exports) {
    module.exports = function generate_required(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $valid = 'valid' + $lvl;
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        var $vSchema = 'schema' + $lvl;
        if (!$isData) {
            if ($schema.length < it.opts.loopRequired && it.schema.properties && Object.keys(it.schema.properties).length) {
                var $required = [];
                var arr1 = $schema;
                if (arr1) {
                    var $property, i1 = -1, l1 = arr1.length - 1;
                    while (i1 < l1) {
                        $property = arr1[i1 += 1];
                        var $propertySch = it.schema.properties[$property];
                        if (!($propertySch && (it.opts.strictKeywords ? typeof $propertySch == 'object' && Object.keys($propertySch).length > 0 : it.util.schemaHasRules($propertySch, it.RULES.all)))) {
                            $required[$required.length] = $property;
                        }
                    }
                }
            }
            else {
                var $required = $schema;
            }
        }
        if ($isData || $required.length) {
            var $currentErrorPath = it.errorPath, $loopRequired = $isData || $required.length >= it.opts.loopRequired, $ownProperties = it.opts.ownProperties;
            if ($breakOnError) {
                out += ' var missing' + ($lvl) + '; ';
                if ($loopRequired) {
                    if (!$isData) {
                        out += ' var ' + ($vSchema) + ' = validate.schema' + ($schemaPath) + '; ';
                    }
                    var $i = 'i' + $lvl, $propertyPath = 'schema' + $lvl + '[' + $i + ']', $missingProperty = '\' + ' + $propertyPath + ' + \'';
                    if (it.opts._errorDataPathProperty) {
                        it.errorPath = it.util.getPathExpr($currentErrorPath, $propertyPath, it.opts.jsonPointers);
                    }
                    out += ' var ' + ($valid) + ' = true; ';
                    if ($isData) {
                        out += ' if (schema' + ($lvl) + ' === undefined) ' + ($valid) + ' = true; else if (!Array.isArray(schema' + ($lvl) + ')) ' + ($valid) + ' = false; else {';
                    }
                    out += ' for (var ' + ($i) + ' = 0; ' + ($i) + ' < ' + ($vSchema) + '.length; ' + ($i) + '++) { ' + ($valid) + ' = ' + ($data) + '[' + ($vSchema) + '[' + ($i) + ']] !== undefined ';
                    if ($ownProperties) {
                        out += ' &&   Object.prototype.hasOwnProperty.call(' + ($data) + ', ' + ($vSchema) + '[' + ($i) + ']) ';
                    }
                    out += '; if (!' + ($valid) + ') break; } ';
                    if ($isData) {
                        out += '  }  ';
                    }
                    out += '  if (!' + ($valid) + ') {   ';
                    var $$outStack = $$outStack || [];
                    $$outStack.push(out);
                    out = ''; /* istanbul ignore else */
                    if (it.createErrors !== false) {
                        out += ' { keyword: \'' + ('required') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { missingProperty: \'' + ($missingProperty) + '\' } ';
                        if (it.opts.messages !== false) {
                            out += ' , message: \'';
                            if (it.opts._errorDataPathProperty) {
                                out += 'is a required property';
                            }
                            else {
                                out += 'should have required property \\\'' + ($missingProperty) + '\\\'';
                            }
                            out += '\' ';
                        }
                        if (it.opts.verbose) {
                            out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                        }
                        out += ' } ';
                    }
                    else {
                        out += ' {} ';
                    }
                    var __err = out;
                    out = $$outStack.pop();
                    if (!it.compositeRule && $breakOnError) {
                        /* istanbul ignore if */
                        if (it.async) {
                            out += ' throw new ValidationError([' + (__err) + ']); ';
                        }
                        else {
                            out += ' validate.errors = [' + (__err) + ']; return false; ';
                        }
                    }
                    else {
                        out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                    }
                    out += ' } else { ';
                }
                else {
                    out += ' if ( ';
                    var arr2 = $required;
                    if (arr2) {
                        var $propertyKey, $i = -1, l2 = arr2.length - 1;
                        while ($i < l2) {
                            $propertyKey = arr2[$i += 1];
                            if ($i) {
                                out += ' || ';
                            }
                            var $prop = it.util.getProperty($propertyKey), $useData = $data + $prop;
                            out += ' ( ( ' + ($useData) + ' === undefined ';
                            if ($ownProperties) {
                                out += ' || ! Object.prototype.hasOwnProperty.call(' + ($data) + ', \'' + (it.util.escapeQuotes($propertyKey)) + '\') ';
                            }
                            out += ') && (missing' + ($lvl) + ' = ' + (it.util.toQuotedString(it.opts.jsonPointers ? $propertyKey : $prop)) + ') ) ';
                        }
                    }
                    out += ') {  ';
                    var $propertyPath = 'missing' + $lvl, $missingProperty = '\' + ' + $propertyPath + ' + \'';
                    if (it.opts._errorDataPathProperty) {
                        it.errorPath = it.opts.jsonPointers ? it.util.getPathExpr($currentErrorPath, $propertyPath, true) : $currentErrorPath + ' + ' + $propertyPath;
                    }
                    var $$outStack = $$outStack || [];
                    $$outStack.push(out);
                    out = ''; /* istanbul ignore else */
                    if (it.createErrors !== false) {
                        out += ' { keyword: \'' + ('required') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { missingProperty: \'' + ($missingProperty) + '\' } ';
                        if (it.opts.messages !== false) {
                            out += ' , message: \'';
                            if (it.opts._errorDataPathProperty) {
                                out += 'is a required property';
                            }
                            else {
                                out += 'should have required property \\\'' + ($missingProperty) + '\\\'';
                            }
                            out += '\' ';
                        }
                        if (it.opts.verbose) {
                            out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                        }
                        out += ' } ';
                    }
                    else {
                        out += ' {} ';
                    }
                    var __err = out;
                    out = $$outStack.pop();
                    if (!it.compositeRule && $breakOnError) {
                        /* istanbul ignore if */
                        if (it.async) {
                            out += ' throw new ValidationError([' + (__err) + ']); ';
                        }
                        else {
                            out += ' validate.errors = [' + (__err) + ']; return false; ';
                        }
                    }
                    else {
                        out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                    }
                    out += ' } else { ';
                }
            }
            else {
                if ($loopRequired) {
                    if (!$isData) {
                        out += ' var ' + ($vSchema) + ' = validate.schema' + ($schemaPath) + '; ';
                    }
                    var $i = 'i' + $lvl, $propertyPath = 'schema' + $lvl + '[' + $i + ']', $missingProperty = '\' + ' + $propertyPath + ' + \'';
                    if (it.opts._errorDataPathProperty) {
                        it.errorPath = it.util.getPathExpr($currentErrorPath, $propertyPath, it.opts.jsonPointers);
                    }
                    if ($isData) {
                        out += ' if (' + ($vSchema) + ' && !Array.isArray(' + ($vSchema) + ')) {  var err =   '; /* istanbul ignore else */
                        if (it.createErrors !== false) {
                            out += ' { keyword: \'' + ('required') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { missingProperty: \'' + ($missingProperty) + '\' } ';
                            if (it.opts.messages !== false) {
                                out += ' , message: \'';
                                if (it.opts._errorDataPathProperty) {
                                    out += 'is a required property';
                                }
                                else {
                                    out += 'should have required property \\\'' + ($missingProperty) + '\\\'';
                                }
                                out += '\' ';
                            }
                            if (it.opts.verbose) {
                                out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                            }
                            out += ' } ';
                        }
                        else {
                            out += ' {} ';
                        }
                        out += ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } else if (' + ($vSchema) + ' !== undefined) { ';
                    }
                    out += ' for (var ' + ($i) + ' = 0; ' + ($i) + ' < ' + ($vSchema) + '.length; ' + ($i) + '++) { if (' + ($data) + '[' + ($vSchema) + '[' + ($i) + ']] === undefined ';
                    if ($ownProperties) {
                        out += ' || ! Object.prototype.hasOwnProperty.call(' + ($data) + ', ' + ($vSchema) + '[' + ($i) + ']) ';
                    }
                    out += ') {  var err =   '; /* istanbul ignore else */
                    if (it.createErrors !== false) {
                        out += ' { keyword: \'' + ('required') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { missingProperty: \'' + ($missingProperty) + '\' } ';
                        if (it.opts.messages !== false) {
                            out += ' , message: \'';
                            if (it.opts._errorDataPathProperty) {
                                out += 'is a required property';
                            }
                            else {
                                out += 'should have required property \\\'' + ($missingProperty) + '\\\'';
                            }
                            out += '\' ';
                        }
                        if (it.opts.verbose) {
                            out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                        }
                        out += ' } ';
                    }
                    else {
                        out += ' {} ';
                    }
                    out += ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } } ';
                    if ($isData) {
                        out += '  }  ';
                    }
                }
                else {
                    var arr3 = $required;
                    if (arr3) {
                        var $propertyKey, i3 = -1, l3 = arr3.length - 1;
                        while (i3 < l3) {
                            $propertyKey = arr3[i3 += 1];
                            var $prop = it.util.getProperty($propertyKey), $missingProperty = it.util.escapeQuotes($propertyKey), $useData = $data + $prop;
                            if (it.opts._errorDataPathProperty) {
                                it.errorPath = it.util.getPath($currentErrorPath, $propertyKey, it.opts.jsonPointers);
                            }
                            out += ' if ( ' + ($useData) + ' === undefined ';
                            if ($ownProperties) {
                                out += ' || ! Object.prototype.hasOwnProperty.call(' + ($data) + ', \'' + (it.util.escapeQuotes($propertyKey)) + '\') ';
                            }
                            out += ') {  var err =   '; /* istanbul ignore else */
                            if (it.createErrors !== false) {
                                out += ' { keyword: \'' + ('required') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { missingProperty: \'' + ($missingProperty) + '\' } ';
                                if (it.opts.messages !== false) {
                                    out += ' , message: \'';
                                    if (it.opts._errorDataPathProperty) {
                                        out += 'is a required property';
                                    }
                                    else {
                                        out += 'should have required property \\\'' + ($missingProperty) + '\\\'';
                                    }
                                    out += '\' ';
                                }
                                if (it.opts.verbose) {
                                    out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                                }
                                out += ' } ';
                            }
                            else {
                                out += ' {} ';
                            }
                            out += ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; } ';
                        }
                    }
                }
            }
            it.errorPath = $currentErrorPath;
        }
        else if ($breakOnError) {
            out += ' if (true) {';
        }
        return out;
    };
},
/* ajv/lib/dotjs/uniqueItems.js */ function _(require, module, exports) {
    module.exports = function generate_uniqueItems(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $data = 'data' + ($dataLvl || '');
        var $valid = 'valid' + $lvl;
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        if (($schema || $isData) && it.opts.uniqueItems !== false) {
            if ($isData) {
                out += ' var ' + ($valid) + '; if (' + ($schemaValue) + ' === false || ' + ($schemaValue) + ' === undefined) ' + ($valid) + ' = true; else if (typeof ' + ($schemaValue) + ' != \'boolean\') ' + ($valid) + ' = false; else { ';
            }
            out += ' var i = ' + ($data) + '.length , ' + ($valid) + ' = true , j; if (i > 1) { ';
            var $itemType = it.schema.items && it.schema.items.type, $typeIsArray = Array.isArray($itemType);
            if (!$itemType || $itemType == 'object' || $itemType == 'array' || ($typeIsArray && ($itemType.indexOf('object') >= 0 || $itemType.indexOf('array') >= 0))) {
                out += ' outer: for (;i--;) { for (j = i; j--;) { if (equal(' + ($data) + '[i], ' + ($data) + '[j])) { ' + ($valid) + ' = false; break outer; } } } ';
            }
            else {
                out += ' var itemIndices = {}, item; for (;i--;) { var item = ' + ($data) + '[i]; ';
                var $method = 'checkDataType' + ($typeIsArray ? 's' : '');
                out += ' if (' + (it.util[$method]($itemType, 'item', true)) + ') continue; ';
                if ($typeIsArray) {
                    out += ' if (typeof item == \'string\') item = \'"\' + item; ';
                }
                out += ' if (typeof itemIndices[item] == \'number\') { ' + ($valid) + ' = false; j = itemIndices[item]; break; } itemIndices[item] = i; } ';
            }
            out += ' } ';
            if ($isData) {
                out += '  }  ';
            }
            out += ' if (!' + ($valid) + ') {   ';
            var $$outStack = $$outStack || [];
            $$outStack.push(out);
            out = ''; /* istanbul ignore else */
            if (it.createErrors !== false) {
                out += ' { keyword: \'' + ('uniqueItems') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { i: i, j: j } ';
                if (it.opts.messages !== false) {
                    out += ' , message: \'should NOT have duplicate items (items ## \' + j + \' and \' + i + \' are identical)\' ';
                }
                if (it.opts.verbose) {
                    out += ' , schema:  ';
                    if ($isData) {
                        out += 'validate.schema' + ($schemaPath);
                    }
                    else {
                        out += '' + ($schema);
                    }
                    out += '         , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                }
                out += ' } ';
            }
            else {
                out += ' {} ';
            }
            var __err = out;
            out = $$outStack.pop();
            if (!it.compositeRule && $breakOnError) {
                /* istanbul ignore if */
                if (it.async) {
                    out += ' throw new ValidationError([' + (__err) + ']); ';
                }
                else {
                    out += ' validate.errors = [' + (__err) + ']; return false; ';
                }
            }
            else {
                out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
            }
            out += ' } ';
            if ($breakOnError) {
                out += ' else { ';
            }
        }
        else {
            if ($breakOnError) {
                out += ' if (true) { ';
            }
        }
        return out;
    };
},
/* ajv/lib/data.js */ function _(require, module, exports) {
    var KEYWORDS = [
        'multipleOf',
        'maximum',
        'exclusiveMaximum',
        'minimum',
        'exclusiveMinimum',
        'maxLength',
        'minLength',
        'pattern',
        'additionalItems',
        'maxItems',
        'minItems',
        'uniqueItems',
        'maxProperties',
        'minProperties',
        'required',
        'additionalProperties',
        'enum',
        'format',
        'const'
    ];
    module.exports = function (metaSchema, keywordsJsonPointers) {
        for (var i = 0; i < keywordsJsonPointers.length; i++) {
            metaSchema = JSON.parse(JSON.stringify(metaSchema));
            var segments = keywordsJsonPointers[i].split('/');
            var keywords = metaSchema;
            var j;
            for (j = 1; j < segments.length; j++)
                keywords = keywords[segments[j]];
            for (j = 0; j < KEYWORDS.length; j++) {
                var key = KEYWORDS[j];
                var schema = keywords[key];
                if (schema) {
                    keywords[key] = {
                        anyOf: [
                            schema,
                            { $ref: 'https://raw.githubusercontent.com/epoberezkin/ajv/master/lib/refs/data.json#' }
                        ]
                    };
                }
            }
        }
        return metaSchema;
    };
},
/* ajv/lib/compile/async.js */ function _(require, module, exports) {
    var MissingRefError = require(433) /* ./error_classes */.MissingRef;
    module.exports = compileAsync;
    /**
     * Creates validating function for passed schema with asynchronous loading of missing schemas.
     * `loadSchema` option should be a function that accepts schema uri and returns promise that resolves with the schema.
     * @this  Ajv
     * @param {Object}   schema schema object
     * @param {Boolean}  meta optional true to compile meta-schema; this parameter can be skipped
     * @param {Function} callback an optional node-style callback, it is called with 2 parameters: error (or null) and validating function.
     * @return {Promise} promise that resolves with a validating function.
     */
    function compileAsync(schema, meta, callback) {
        /* eslint no-shadow: 0 */
        /* global Promise */
        /* jshint validthis: true */
        var self = this;
        if (typeof this._opts.loadSchema != 'function')
            throw new Error('options.loadSchema should be a function');
        if (typeof meta == 'function') {
            callback = meta;
            meta = undefined;
        }
        var p = loadMetaSchemaOf(schema).then(function () {
            var schemaObj = self._addSchema(schema, undefined, meta);
            return schemaObj.validate || _compileAsync(schemaObj);
        });
        if (callback) {
            p.then(function (v) { callback(null, v); }, callback);
        }
        return p;
        function loadMetaSchemaOf(sch) {
            var $schema = sch.$schema;
            return $schema && !self.getSchema($schema)
                ? compileAsync.call(self, { $ref: $schema }, true)
                : Promise.resolve();
        }
        function _compileAsync(schemaObj) {
            try {
                return self._compile(schemaObj);
            }
            catch (e) {
                if (e instanceof MissingRefError)
                    return loadMissingSchema(e);
                throw e;
            }
            function loadMissingSchema(e) {
                var ref = e.missingSchema;
                if (added(ref))
                    throw new Error('Schema ' + ref + ' is loaded but ' + e.missingRef + ' cannot be resolved');
                var schemaPromise = self._loadingSchemas[ref];
                if (!schemaPromise) {
                    schemaPromise = self._loadingSchemas[ref] = self._opts.loadSchema(ref);
                    schemaPromise.then(removePromise, removePromise);
                }
                return schemaPromise.then(function (sch) {
                    if (!added(ref)) {
                        return loadMetaSchemaOf(sch).then(function () {
                            if (!added(ref))
                                self.addSchema(sch, ref, undefined, meta);
                        });
                    }
                }).then(function () {
                    return _compileAsync(schemaObj);
                });
                function removePromise() {
                    delete self._loadingSchemas[ref];
                }
                function added(ref) {
                    return self._refs[ref] || self._schemas[ref];
                }
            }
        }
    }
},
/* ajv/lib/keyword.js */ function _(require, module, exports) {
    var IDENTIFIER = /^[a-z_$][a-z0-9_$-]*$/i;
    var customRuleCode = require(466) /* ./dotjs/custom */;
    var definitionSchema = require(467) /* ./definition_schema */;
    module.exports = {
        add: addKeyword,
        get: getKeyword,
        remove: removeKeyword,
        validate: validateKeyword
    };
    /**
     * Define custom keyword
     * @this  Ajv
     * @param {String} keyword custom keyword, should be unique (including different from all standard, custom and macro keywords).
     * @param {Object} definition keyword definition object with properties `type` (type(s) which the keyword applies to), `validate` or `compile`.
     * @return {Ajv} this for method chaining
     */
    function addKeyword(keyword, definition) {
        /* jshint validthis: true */
        /* eslint no-shadow: 0 */
        var RULES = this.RULES;
        if (RULES.keywords[keyword])
            throw new Error('Keyword ' + keyword + ' is already defined');
        if (!IDENTIFIER.test(keyword))
            throw new Error('Keyword ' + keyword + ' is not a valid identifier');
        if (definition) {
            this.validateKeyword(definition, true);
            var dataType = definition.type;
            if (Array.isArray(dataType)) {
                for (var i = 0; i < dataType.length; i++)
                    _addRule(keyword, dataType[i], definition);
            }
            else {
                _addRule(keyword, dataType, definition);
            }
            var metaSchema = definition.metaSchema;
            if (metaSchema) {
                if (definition.$data && this._opts.$data) {
                    metaSchema = {
                        anyOf: [
                            metaSchema,
                            { '$ref': 'https://raw.githubusercontent.com/epoberezkin/ajv/master/lib/refs/data.json#' }
                        ]
                    };
                }
                definition.validateSchema = this.compile(metaSchema, true);
            }
        }
        RULES.keywords[keyword] = RULES.all[keyword] = true;
        function _addRule(keyword, dataType, definition) {
            var ruleGroup;
            for (var i = 0; i < RULES.length; i++) {
                var rg = RULES[i];
                if (rg.type == dataType) {
                    ruleGroup = rg;
                    break;
                }
            }
            if (!ruleGroup) {
                ruleGroup = { type: dataType, rules: [] };
                RULES.push(ruleGroup);
            }
            var rule = {
                keyword: keyword,
                definition: definition,
                custom: true,
                code: customRuleCode,
                implements: definition.implements
            };
            ruleGroup.rules.push(rule);
            RULES.custom[keyword] = rule;
        }
        return this;
    }
    /**
     * Get keyword
     * @this  Ajv
     * @param {String} keyword pre-defined or custom keyword.
     * @return {Object|Boolean} custom keyword definition, `true` if it is a predefined keyword, `false` otherwise.
     */
    function getKeyword(keyword) {
        /* jshint validthis: true */
        var rule = this.RULES.custom[keyword];
        return rule ? rule.definition : this.RULES.keywords[keyword] || false;
    }
    /**
     * Remove keyword
     * @this  Ajv
     * @param {String} keyword pre-defined or custom keyword.
     * @return {Ajv} this for method chaining
     */
    function removeKeyword(keyword) {
        /* jshint validthis: true */
        var RULES = this.RULES;
        delete RULES.keywords[keyword];
        delete RULES.all[keyword];
        delete RULES.custom[keyword];
        for (var i = 0; i < RULES.length; i++) {
            var rules = RULES[i].rules;
            for (var j = 0; j < rules.length; j++) {
                if (rules[j].keyword == keyword) {
                    rules.splice(j, 1);
                    break;
                }
            }
        }
        return this;
    }
    /**
     * Validate keyword definition
     * @this  Ajv
     * @param {Object} definition keyword definition object.
     * @param {Boolean} throwError true to throw exception if definition is invalid
     * @return {boolean} validation result
     */
    function validateKeyword(definition, throwError) {
        validateKeyword.errors = null;
        var v = this._validateKeyword = this._validateKeyword
            || this.compile(definitionSchema, true);
        if (v(definition))
            return true;
        validateKeyword.errors = v.errors;
        if (throwError)
            throw new Error('custom keyword definition is invalid: ' + this.errorsText(v.errors));
        else
            return false;
    }
},
/* ajv/lib/dotjs/custom.js */ function _(require, module, exports) {
    module.exports = function generate_custom(it, $keyword, $ruleType) {
        var out = ' ';
        var $lvl = it.level;
        var $dataLvl = it.dataLevel;
        var $schema = it.schema[$keyword];
        var $schemaPath = it.schemaPath + it.util.getProperty($keyword);
        var $errSchemaPath = it.errSchemaPath + '/' + $keyword;
        var $breakOnError = !it.opts.allErrors;
        var $errorKeyword;
        var $data = 'data' + ($dataLvl || '');
        var $valid = 'valid' + $lvl;
        var $errs = 'errs__' + $lvl;
        var $isData = it.opts.$data && $schema && $schema.$data, $schemaValue;
        if ($isData) {
            out += ' var schema' + ($lvl) + ' = ' + (it.util.getData($schema.$data, $dataLvl, it.dataPathArr)) + '; ';
            $schemaValue = 'schema' + $lvl;
        }
        else {
            $schemaValue = $schema;
        }
        var $rule = this, $definition = 'definition' + $lvl, $rDef = $rule.definition, $closingBraces = '';
        var $compile, $inline, $macro, $ruleValidate, $validateCode;
        if ($isData && $rDef.$data) {
            $validateCode = 'keywordValidate' + $lvl;
            var $validateSchema = $rDef.validateSchema;
            out += ' var ' + ($definition) + ' = RULES.custom[\'' + ($keyword) + '\'].definition; var ' + ($validateCode) + ' = ' + ($definition) + '.validate;';
        }
        else {
            $ruleValidate = it.useCustomRule($rule, $schema, it.schema, it);
            if (!$ruleValidate)
                return;
            $schemaValue = 'validate.schema' + $schemaPath;
            $validateCode = $ruleValidate.code;
            $compile = $rDef.compile;
            $inline = $rDef.inline;
            $macro = $rDef.macro;
        }
        var $ruleErrs = $validateCode + '.errors', $i = 'i' + $lvl, $ruleErr = 'ruleErr' + $lvl, $asyncKeyword = $rDef.async;
        if ($asyncKeyword && !it.async)
            throw new Error('async keyword in sync schema');
        if (!($inline || $macro)) {
            out += '' + ($ruleErrs) + ' = null;';
        }
        out += 'var ' + ($errs) + ' = errors;var ' + ($valid) + ';';
        if ($isData && $rDef.$data) {
            $closingBraces += '}';
            out += ' if (' + ($schemaValue) + ' === undefined) { ' + ($valid) + ' = true; } else { ';
            if ($validateSchema) {
                $closingBraces += '}';
                out += ' ' + ($valid) + ' = ' + ($definition) + '.validateSchema(' + ($schemaValue) + '); if (' + ($valid) + ') { ';
            }
        }
        if ($inline) {
            if ($rDef.statements) {
                out += ' ' + ($ruleValidate.validate) + ' ';
            }
            else {
                out += ' ' + ($valid) + ' = ' + ($ruleValidate.validate) + '; ';
            }
        }
        else if ($macro) {
            var $it = it.util.copy(it);
            var $closingBraces = '';
            $it.level++;
            var $nextValid = 'valid' + $it.level;
            $it.schema = $ruleValidate.validate;
            $it.schemaPath = '';
            var $wasComposite = it.compositeRule;
            it.compositeRule = $it.compositeRule = true;
            var $code = it.validate($it).replace(/validate\.schema/g, $validateCode);
            it.compositeRule = $it.compositeRule = $wasComposite;
            out += ' ' + ($code);
        }
        else {
            var $$outStack = $$outStack || [];
            $$outStack.push(out);
            out = '';
            out += '  ' + ($validateCode) + '.call( ';
            if (it.opts.passContext) {
                out += 'this';
            }
            else {
                out += 'self';
            }
            if ($compile || $rDef.schema === false) {
                out += ' , ' + ($data) + ' ';
            }
            else {
                out += ' , ' + ($schemaValue) + ' , ' + ($data) + ' , validate.schema' + (it.schemaPath) + ' ';
            }
            out += ' , (dataPath || \'\')';
            if (it.errorPath != '""') {
                out += ' + ' + (it.errorPath);
            }
            var $parentData = $dataLvl ? 'data' + (($dataLvl - 1) || '') : 'parentData', $parentDataProperty = $dataLvl ? it.dataPathArr[$dataLvl] : 'parentDataProperty';
            out += ' , ' + ($parentData) + ' , ' + ($parentDataProperty) + ' , rootData )  ';
            var def_callRuleValidate = out;
            out = $$outStack.pop();
            if ($rDef.errors === false) {
                out += ' ' + ($valid) + ' = ';
                if ($asyncKeyword) {
                    out += 'await ';
                }
                out += '' + (def_callRuleValidate) + '; ';
            }
            else {
                if ($asyncKeyword) {
                    $ruleErrs = 'customErrors' + $lvl;
                    out += ' var ' + ($ruleErrs) + ' = null; try { ' + ($valid) + ' = await ' + (def_callRuleValidate) + '; } catch (e) { ' + ($valid) + ' = false; if (e instanceof ValidationError) ' + ($ruleErrs) + ' = e.errors; else throw e; } ';
                }
                else {
                    out += ' ' + ($ruleErrs) + ' = null; ' + ($valid) + ' = ' + (def_callRuleValidate) + '; ';
                }
            }
        }
        if ($rDef.modifying) {
            out += ' if (' + ($parentData) + ') ' + ($data) + ' = ' + ($parentData) + '[' + ($parentDataProperty) + '];';
        }
        out += '' + ($closingBraces);
        if ($rDef.valid) {
            if ($breakOnError) {
                out += ' if (true) { ';
            }
        }
        else {
            out += ' if ( ';
            if ($rDef.valid === undefined) {
                out += ' !';
                if ($macro) {
                    out += '' + ($nextValid);
                }
                else {
                    out += '' + ($valid);
                }
            }
            else {
                out += ' ' + (!$rDef.valid) + ' ';
            }
            out += ') { ';
            $errorKeyword = $rule.keyword;
            var $$outStack = $$outStack || [];
            $$outStack.push(out);
            out = '';
            var $$outStack = $$outStack || [];
            $$outStack.push(out);
            out = ''; /* istanbul ignore else */
            if (it.createErrors !== false) {
                out += ' { keyword: \'' + ($errorKeyword || 'custom') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { keyword: \'' + ($rule.keyword) + '\' } ';
                if (it.opts.messages !== false) {
                    out += ' , message: \'should pass "' + ($rule.keyword) + '" keyword validation\' ';
                }
                if (it.opts.verbose) {
                    out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                }
                out += ' } ';
            }
            else {
                out += ' {} ';
            }
            var __err = out;
            out = $$outStack.pop();
            if (!it.compositeRule && $breakOnError) {
                /* istanbul ignore if */
                if (it.async) {
                    out += ' throw new ValidationError([' + (__err) + ']); ';
                }
                else {
                    out += ' validate.errors = [' + (__err) + ']; return false; ';
                }
            }
            else {
                out += ' var err = ' + (__err) + ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
            }
            var def_customError = out;
            out = $$outStack.pop();
            if ($inline) {
                if ($rDef.errors) {
                    if ($rDef.errors != 'full') {
                        out += '  for (var ' + ($i) + '=' + ($errs) + '; ' + ($i) + '<errors; ' + ($i) + '++) { var ' + ($ruleErr) + ' = vErrors[' + ($i) + ']; if (' + ($ruleErr) + '.dataPath === undefined) ' + ($ruleErr) + '.dataPath = (dataPath || \'\') + ' + (it.errorPath) + '; if (' + ($ruleErr) + '.schemaPath === undefined) { ' + ($ruleErr) + '.schemaPath = "' + ($errSchemaPath) + '"; } ';
                        if (it.opts.verbose) {
                            out += ' ' + ($ruleErr) + '.schema = ' + ($schemaValue) + '; ' + ($ruleErr) + '.data = ' + ($data) + '; ';
                        }
                        out += ' } ';
                    }
                }
                else {
                    if ($rDef.errors === false) {
                        out += ' ' + (def_customError) + ' ';
                    }
                    else {
                        out += ' if (' + ($errs) + ' == errors) { ' + (def_customError) + ' } else {  for (var ' + ($i) + '=' + ($errs) + '; ' + ($i) + '<errors; ' + ($i) + '++) { var ' + ($ruleErr) + ' = vErrors[' + ($i) + ']; if (' + ($ruleErr) + '.dataPath === undefined) ' + ($ruleErr) + '.dataPath = (dataPath || \'\') + ' + (it.errorPath) + '; if (' + ($ruleErr) + '.schemaPath === undefined) { ' + ($ruleErr) + '.schemaPath = "' + ($errSchemaPath) + '"; } ';
                        if (it.opts.verbose) {
                            out += ' ' + ($ruleErr) + '.schema = ' + ($schemaValue) + '; ' + ($ruleErr) + '.data = ' + ($data) + '; ';
                        }
                        out += ' } } ';
                    }
                }
            }
            else if ($macro) {
                out += '   var err =   '; /* istanbul ignore else */
                if (it.createErrors !== false) {
                    out += ' { keyword: \'' + ($errorKeyword || 'custom') + '\' , dataPath: (dataPath || \'\') + ' + (it.errorPath) + ' , schemaPath: ' + (it.util.toQuotedString($errSchemaPath)) + ' , params: { keyword: \'' + ($rule.keyword) + '\' } ';
                    if (it.opts.messages !== false) {
                        out += ' , message: \'should pass "' + ($rule.keyword) + '" keyword validation\' ';
                    }
                    if (it.opts.verbose) {
                        out += ' , schema: validate.schema' + ($schemaPath) + ' , parentSchema: validate.schema' + (it.schemaPath) + ' , data: ' + ($data) + ' ';
                    }
                    out += ' } ';
                }
                else {
                    out += ' {} ';
                }
                out += ';  if (vErrors === null) vErrors = [err]; else vErrors.push(err); errors++; ';
                if (!it.compositeRule && $breakOnError) {
                    /* istanbul ignore if */
                    if (it.async) {
                        out += ' throw new ValidationError(vErrors); ';
                    }
                    else {
                        out += ' validate.errors = vErrors; return false; ';
                    }
                }
            }
            else {
                if ($rDef.errors === false) {
                    out += ' ' + (def_customError) + ' ';
                }
                else {
                    out += ' if (Array.isArray(' + ($ruleErrs) + ')) { if (vErrors === null) vErrors = ' + ($ruleErrs) + '; else vErrors = vErrors.concat(' + ($ruleErrs) + '); errors = vErrors.length;  for (var ' + ($i) + '=' + ($errs) + '; ' + ($i) + '<errors; ' + ($i) + '++) { var ' + ($ruleErr) + ' = vErrors[' + ($i) + ']; if (' + ($ruleErr) + '.dataPath === undefined) ' + ($ruleErr) + '.dataPath = (dataPath || \'\') + ' + (it.errorPath) + ';  ' + ($ruleErr) + '.schemaPath = "' + ($errSchemaPath) + '";  ';
                    if (it.opts.verbose) {
                        out += ' ' + ($ruleErr) + '.schema = ' + ($schemaValue) + '; ' + ($ruleErr) + '.data = ' + ($data) + '; ';
                    }
                    out += ' } } else { ' + (def_customError) + ' } ';
                }
            }
            out += ' } ';
            if ($breakOnError) {
                out += ' else { ';
            }
        }
        return out;
    };
},
/* ajv/lib/definition_schema.js */ function _(require, module, exports) {
    var metaSchema = require(468) /* ./refs/json-schema-draft-07.json */;
    module.exports = {
        $id: 'https://github.com/epoberezkin/ajv/blob/master/lib/definition_schema.js',
        definitions: {
            simpleTypes: metaSchema.definitions.simpleTypes
        },
        type: 'object',
        dependencies: {
            schema: ['validate'],
            $data: ['validate'],
            statements: ['inline'],
            valid: { not: { required: ['macro'] } }
        },
        properties: {
            type: metaSchema.properties.type,
            schema: { type: 'boolean' },
            statements: { type: 'boolean' },
            dependencies: {
                type: 'array',
                items: { type: 'string' }
            },
            metaSchema: { type: 'object' },
            modifying: { type: 'boolean' },
            valid: { type: 'boolean' },
            $data: { type: 'boolean' },
            async: { type: 'boolean' },
            errors: {
                anyOf: [
                    { type: 'boolean' },
                    { const: 'full' }
                ]
            }
        }
    };
},
/* ajv/lib/refs/json-schema-draft-07.json */ function _(require, module, exports) {
    module.exports = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "$id": "http://json-schema.org/draft-07/schema#",
        "title": "Core schema meta-schema",
        "definitions": {
            "schemaArray": {
                "type": "array",
                "minItems": 1,
                "items": { "$ref": "#" }
            },
            "nonNegativeInteger": {
                "type": "integer",
                "minimum": 0
            },
            "nonNegativeIntegerDefault0": {
                "allOf": [
                    { "$ref": "#/definitions/nonNegativeInteger" },
                    { "default": 0 }
                ]
            },
            "simpleTypes": {
                "enum": [
                    "array",
                    "boolean",
                    "integer",
                    "null",
                    "number",
                    "object",
                    "string"
                ]
            },
            "stringArray": {
                "type": "array",
                "items": { "type": "string" },
                "uniqueItems": true,
                "default": []
            }
        },
        "type": ["object", "boolean"],
        "properties": {
            "$id": {
                "type": "string",
                "format": "uri-reference"
            },
            "$schema": {
                "type": "string",
                "format": "uri"
            },
            "$ref": {
                "type": "string",
                "format": "uri-reference"
            },
            "$comment": {
                "type": "string"
            },
            "title": {
                "type": "string"
            },
            "description": {
                "type": "string"
            },
            "default": true,
            "readOnly": {
                "type": "boolean",
                "default": false
            },
            "examples": {
                "type": "array",
                "items": true
            },
            "multipleOf": {
                "type": "number",
                "exclusiveMinimum": 0
            },
            "maximum": {
                "type": "number"
            },
            "exclusiveMaximum": {
                "type": "number"
            },
            "minimum": {
                "type": "number"
            },
            "exclusiveMinimum": {
                "type": "number"
            },
            "maxLength": { "$ref": "#/definitions/nonNegativeInteger" },
            "minLength": { "$ref": "#/definitions/nonNegativeIntegerDefault0" },
            "pattern": {
                "type": "string",
                "format": "regex"
            },
            "additionalItems": { "$ref": "#" },
            "items": {
                "anyOf": [
                    { "$ref": "#" },
                    { "$ref": "#/definitions/schemaArray" }
                ],
                "default": true
            },
            "maxItems": { "$ref": "#/definitions/nonNegativeInteger" },
            "minItems": { "$ref": "#/definitions/nonNegativeIntegerDefault0" },
            "uniqueItems": {
                "type": "boolean",
                "default": false
            },
            "contains": { "$ref": "#" },
            "maxProperties": { "$ref": "#/definitions/nonNegativeInteger" },
            "minProperties": { "$ref": "#/definitions/nonNegativeIntegerDefault0" },
            "required": { "$ref": "#/definitions/stringArray" },
            "additionalProperties": { "$ref": "#" },
            "definitions": {
                "type": "object",
                "additionalProperties": { "$ref": "#" },
                "default": {}
            },
            "properties": {
                "type": "object",
                "additionalProperties": { "$ref": "#" },
                "default": {}
            },
            "patternProperties": {
                "type": "object",
                "additionalProperties": { "$ref": "#" },
                "propertyNames": { "format": "regex" },
                "default": {}
            },
            "dependencies": {
                "type": "object",
                "additionalProperties": {
                    "anyOf": [
                        { "$ref": "#" },
                        { "$ref": "#/definitions/stringArray" }
                    ]
                }
            },
            "propertyNames": { "$ref": "#" },
            "const": true,
            "enum": {
                "type": "array",
                "items": true,
                "minItems": 1,
                "uniqueItems": true
            },
            "type": {
                "anyOf": [
                    { "$ref": "#/definitions/simpleTypes" },
                    {
                        "type": "array",
                        "items": { "$ref": "#/definitions/simpleTypes" },
                        "minItems": 1,
                        "uniqueItems": true
                    }
                ]
            },
            "format": { "type": "string" },
            "contentMediaType": { "type": "string" },
            "contentEncoding": { "type": "string" },
            "if": { "$ref": "#" },
            "then": { "$ref": "#" },
            "else": { "$ref": "#" },
            "allOf": { "$ref": "#/definitions/schemaArray" },
            "anyOf": { "$ref": "#/definitions/schemaArray" },
            "oneOf": { "$ref": "#/definitions/schemaArray" },
            "not": { "$ref": "#" }
        },
        "default": true
    };
},
/* ajv/lib/refs/data.json */ function _(require, module, exports) {
    module.exports = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "$id": "https://raw.githubusercontent.com/epoberezkin/ajv/master/lib/refs/data.json#",
        "description": "Meta-schema for $data reference (JSON Schema extension proposal)",
        "type": "object",
        "required": ["$data"],
        "properties": {
            "$data": {
                "type": "string",
                "anyOf": [
                    { "format": "relative-json-pointer" },
                    { "format": "json-pointer" }
                ]
            }
        },
        "additionalProperties": false
    };
},
/* ajv/lib/refs/json-schema-draft-04.json */ function _(require, module, exports) {
    module.exports = {
        "id": "http://json-schema.org/draft-04/schema#",
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Core schema meta-schema",
        "definitions": {
            "schemaArray": {
                "type": "array",
                "minItems": 1,
                "items": { "$ref": "#" }
            },
            "positiveInteger": {
                "type": "integer",
                "minimum": 0
            },
            "positiveIntegerDefault0": {
                "allOf": [{ "$ref": "#/definitions/positiveInteger" }, { "default": 0 }]
            },
            "simpleTypes": {
                "enum": ["array", "boolean", "integer", "null", "number", "object", "string"]
            },
            "stringArray": {
                "type": "array",
                "items": { "type": "string" },
                "minItems": 1,
                "uniqueItems": true
            }
        },
        "type": "object",
        "properties": {
            "id": {
                "type": "string"
            },
            "$schema": {
                "type": "string"
            },
            "title": {
                "type": "string"
            },
            "description": {
                "type": "string"
            },
            "default": {},
            "multipleOf": {
                "type": "number",
                "minimum": 0,
                "exclusiveMinimum": true
            },
            "maximum": {
                "type": "number"
            },
            "exclusiveMaximum": {
                "type": "boolean",
                "default": false
            },
            "minimum": {
                "type": "number"
            },
            "exclusiveMinimum": {
                "type": "boolean",
                "default": false
            },
            "maxLength": { "$ref": "#/definitions/positiveInteger" },
            "minLength": { "$ref": "#/definitions/positiveIntegerDefault0" },
            "pattern": {
                "type": "string",
                "format": "regex"
            },
            "additionalItems": {
                "anyOf": [
                    { "type": "boolean" },
                    { "$ref": "#" }
                ],
                "default": {}
            },
            "items": {
                "anyOf": [
                    { "$ref": "#" },
                    { "$ref": "#/definitions/schemaArray" }
                ],
                "default": {}
            },
            "maxItems": { "$ref": "#/definitions/positiveInteger" },
            "minItems": { "$ref": "#/definitions/positiveIntegerDefault0" },
            "uniqueItems": {
                "type": "boolean",
                "default": false
            },
            "maxProperties": { "$ref": "#/definitions/positiveInteger" },
            "minProperties": { "$ref": "#/definitions/positiveIntegerDefault0" },
            "required": { "$ref": "#/definitions/stringArray" },
            "additionalProperties": {
                "anyOf": [
                    { "type": "boolean" },
                    { "$ref": "#" }
                ],
                "default": {}
            },
            "definitions": {
                "type": "object",
                "additionalProperties": { "$ref": "#" },
                "default": {}
            },
            "properties": {
                "type": "object",
                "additionalProperties": { "$ref": "#" },
                "default": {}
            },
            "patternProperties": {
                "type": "object",
                "additionalProperties": { "$ref": "#" },
                "default": {}
            },
            "dependencies": {
                "type": "object",
                "additionalProperties": {
                    "anyOf": [
                        { "$ref": "#" },
                        { "$ref": "#/definitions/stringArray" }
                    ]
                }
            },
            "enum": {
                "type": "array",
                "minItems": 1,
                "uniqueItems": true
            },
            "type": {
                "anyOf": [
                    { "$ref": "#/definitions/simpleTypes" },
                    {
                        "type": "array",
                        "items": { "$ref": "#/definitions/simpleTypes" },
                        "minItems": 1,
                        "uniqueItems": true
                    }
                ]
            },
            "format": { "type": "string" },
            "allOf": { "$ref": "#/definitions/schemaArray" },
            "anyOf": { "$ref": "#/definitions/schemaArray" },
            "oneOf": { "$ref": "#/definitions/schemaArray" },
            "not": { "$ref": "#" }
        },
        "dependencies": {
            "exclusiveMaximum": ["maximum"],
            "exclusiveMinimum": ["minimum"]
        },
        "default": {}
    };
},
/* eslint/lib/source-code/index.js */ function _(require, module, exports) {
    module.exports = {
        SourceCode: require(472) /* ./source-code */
    };
},
/* eslint/lib/source-code/source-code.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const { isCommentToken } = require(250) /* eslint-utils */, TokenStore = require(473) /* ./token-store */, astUtils = require(115) /* ../shared/ast-utils */, Traverser = require(337) /* ../shared/traverser */, lodash = require(111) /* lodash */;
    //------------------------------------------------------------------------------
    // Private
    //------------------------------------------------------------------------------
    /**
     * Validates that the given AST has the required information.
     * @param {ASTNode} ast The Program node of the AST to check.
     * @throws {Error} If the AST doesn't contain the correct information.
     * @returns {void}
     * @private
     */
    function validate(ast) {
        if (!ast.tokens) {
            throw new Error("AST is missing the tokens array.");
        }
        if (!ast.comments) {
            throw new Error("AST is missing the comments array.");
        }
        if (!ast.loc) {
            throw new Error("AST is missing location information.");
        }
        if (!ast.range) {
            throw new Error("AST is missing range information");
        }
    }
    /**
     * Check to see if its a ES6 export declaration.
     * @param {ASTNode} astNode An AST node.
     * @returns {boolean} whether the given node represents an export declaration.
     * @private
     */
    function looksLikeExport(astNode) {
        return astNode.type === "ExportDefaultDeclaration" || astNode.type === "ExportNamedDeclaration" ||
            astNode.type === "ExportAllDeclaration" || astNode.type === "ExportSpecifier";
    }
    /**
     * Merges two sorted lists into a larger sorted list in O(n) time.
     * @param {Token[]} tokens The list of tokens.
     * @param {Token[]} comments The list of comments.
     * @returns {Token[]} A sorted list of tokens and comments.
     * @private
     */
    function sortedMerge(tokens, comments) {
        const result = [];
        let tokenIndex = 0;
        let commentIndex = 0;
        while (tokenIndex < tokens.length || commentIndex < comments.length) {
            if (commentIndex >= comments.length || tokenIndex < tokens.length && tokens[tokenIndex].range[0] < comments[commentIndex].range[0]) {
                result.push(tokens[tokenIndex++]);
            }
            else {
                result.push(comments[commentIndex++]);
            }
        }
        return result;
    }
    /**
     * Determines if two nodes or tokens overlap.
     * @param {ASTNode|Token} first The first node or token to check.
     * @param {ASTNode|Token} second The second node or token to check.
     * @returns {boolean} True if the two nodes or tokens overlap.
     * @private
     */
    function nodesOrTokensOverlap(first, second) {
        return (first.range[0] <= second.range[0] && first.range[1] >= second.range[0]) ||
            (second.range[0] <= first.range[0] && second.range[1] >= first.range[0]);
    }
    /**
     * Determines if two nodes or tokens have at least one whitespace character
     * between them. Order does not matter. Returns false if the given nodes or
     * tokens overlap.
     * @param {SourceCode} sourceCode The source code object.
     * @param {ASTNode|Token} first The first node or token to check between.
     * @param {ASTNode|Token} second The second node or token to check between.
     * @param {boolean} checkInsideOfJSXText If `true` is present, check inside of JSXText tokens for backward compatibility.
     * @returns {boolean} True if there is a whitespace character between
     * any of the tokens found between the two given nodes or tokens.
     * @public
     */
    function isSpaceBetween(sourceCode, first, second, checkInsideOfJSXText) {
        if (nodesOrTokensOverlap(first, second)) {
            return false;
        }
        const [startingNodeOrToken, endingNodeOrToken] = first.range[1] <= second.range[0]
            ? [first, second]
            : [second, first];
        const firstToken = sourceCode.getLastToken(startingNodeOrToken) || startingNodeOrToken;
        const finalToken = sourceCode.getFirstToken(endingNodeOrToken) || endingNodeOrToken;
        let currentToken = firstToken;
        while (currentToken !== finalToken) {
            const nextToken = sourceCode.getTokenAfter(currentToken, { includeComments: true });
            if (currentToken.range[1] !== nextToken.range[0] ||
                /*
                 * For backward compatibility, check speces in JSXText.
                 * https://github.com/eslint/eslint/issues/12614
                 */
                (checkInsideOfJSXText &&
                    nextToken !== finalToken &&
                    nextToken.type === "JSXText" &&
                    /\s/u.test(nextToken.value))) {
                return true;
            }
            currentToken = nextToken;
        }
        return false;
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    class SourceCode extends TokenStore {
        /**
         * Represents parsed source code.
         * @param {string|Object} textOrConfig The source code text or config object.
         * @param {string} textOrConfig.text The source code text.
         * @param {ASTNode} textOrConfig.ast The Program node of the AST representing the code. This AST should be created from the text that BOM was stripped.
         * @param {Object|null} textOrConfig.parserServices The parser services.
         * @param {ScopeManager|null} textOrConfig.scopeManager The scope of this source code.
         * @param {Object|null} textOrConfig.visitorKeys The visitor keys to traverse AST.
         * @param {ASTNode} [astIfNoConfig] The Program node of the AST representing the code. This AST should be created from the text that BOM was stripped.
         */
        constructor(textOrConfig, astIfNoConfig) {
            let text, ast, parserServices, scopeManager, visitorKeys;
            // Process overloading.
            if (typeof textOrConfig === "string") {
                text = textOrConfig;
                ast = astIfNoConfig;
            }
            else if (typeof textOrConfig === "object" && textOrConfig !== null) {
                text = textOrConfig.text;
                ast = textOrConfig.ast;
                parserServices = textOrConfig.parserServices;
                scopeManager = textOrConfig.scopeManager;
                visitorKeys = textOrConfig.visitorKeys;
            }
            validate(ast);
            super(ast.tokens, ast.comments);
            /**
             * The flag to indicate that the source code has Unicode BOM.
             * @type boolean
             */
            this.hasBOM = (text.charCodeAt(0) === 0xFEFF);
            /**
             * The original text source code.
             * BOM was stripped from this text.
             * @type string
             */
            this.text = (this.hasBOM ? text.slice(1) : text);
            /**
             * The parsed AST for the source code.
             * @type ASTNode
             */
            this.ast = ast;
            /**
             * The parser services of this source code.
             * @type {Object}
             */
            this.parserServices = parserServices || {};
            /**
             * The scope of this source code.
             * @type {ScopeManager|null}
             */
            this.scopeManager = scopeManager || null;
            /**
             * The visitor keys to traverse AST.
             * @type {Object}
             */
            this.visitorKeys = visitorKeys || Traverser.DEFAULT_VISITOR_KEYS;
            // Check the source text for the presence of a shebang since it is parsed as a standard line comment.
            const shebangMatched = this.text.match(astUtils.shebangPattern);
            const hasShebang = shebangMatched && ast.comments.length && ast.comments[0].value === shebangMatched[1];
            if (hasShebang) {
                ast.comments[0].type = "Shebang";
            }
            this.tokensAndComments = sortedMerge(ast.tokens, ast.comments);
            /**
             * The source code split into lines according to ECMA-262 specification.
             * This is done to avoid each rule needing to do so separately.
             * @type string[]
             */
            this.lines = [];
            this.lineStartIndices = [0];
            const lineEndingPattern = astUtils.createGlobalLinebreakMatcher();
            let match;
            /*
             * Previously, this was implemented using a regex that
             * matched a sequence of non-linebreak characters followed by a
             * linebreak, then adding the lengths of the matches. However,
             * this caused a catastrophic backtracking issue when the end
             * of a file contained a large number of non-newline characters.
             * To avoid this, the current implementation just matches newlines
             * and uses match.index to get the correct line start indices.
             */
            while ((match = lineEndingPattern.exec(this.text))) {
                this.lines.push(this.text.slice(this.lineStartIndices[this.lineStartIndices.length - 1], match.index));
                this.lineStartIndices.push(match.index + match[0].length);
            }
            this.lines.push(this.text.slice(this.lineStartIndices[this.lineStartIndices.length - 1]));
            // Cache for comments found using getComments().
            this._commentCache = new WeakMap();
            // don't allow modification of this object
            Object.freeze(this);
            Object.freeze(this.lines);
        }
        /**
         * Split the source code into multiple lines based on the line delimiters.
         * @param {string} text Source code as a string.
         * @returns {string[]} Array of source code lines.
         * @public
         */
        static splitLines(text) {
            return text.split(astUtils.createGlobalLinebreakMatcher());
        }
        /**
         * Gets the source code for the given node.
         * @param {ASTNode} [node] The AST node to get the text for.
         * @param {int} [beforeCount] The number of characters before the node to retrieve.
         * @param {int} [afterCount] The number of characters after the node to retrieve.
         * @returns {string} The text representing the AST node.
         * @public
         */
        getText(node, beforeCount, afterCount) {
            if (node) {
                return this.text.slice(Math.max(node.range[0] - (beforeCount || 0), 0), node.range[1] + (afterCount || 0));
            }
            return this.text;
        }
        /**
         * Gets the entire source text split into an array of lines.
         * @returns {Array} The source text as an array of lines.
         * @public
         */
        getLines() {
            return this.lines;
        }
        /**
         * Retrieves an array containing all comments in the source code.
         * @returns {ASTNode[]} An array of comment nodes.
         * @public
         */
        getAllComments() {
            return this.ast.comments;
        }
        /**
         * Gets all comments for the given node.
         * @param {ASTNode} node The AST node to get the comments for.
         * @returns {Object} An object containing a leading and trailing array
         *      of comments indexed by their position.
         * @public
         */
        getComments(node) {
            if (this._commentCache.has(node)) {
                return this._commentCache.get(node);
            }
            const comments = {
                leading: [],
                trailing: []
            };
            /*
             * Return all comments as leading comments of the Program node when
             * there is no executable code.
             */
            if (node.type === "Program") {
                if (node.body.length === 0) {
                    comments.leading = node.comments;
                }
            }
            else {
                /*
                 * Return comments as trailing comments of nodes that only contain
                 * comments (to mimic the comment attachment behavior present in Espree).
                 */
                if ((node.type === "BlockStatement" || node.type === "ClassBody") && node.body.length === 0 ||
                    node.type === "ObjectExpression" && node.properties.length === 0 ||
                    node.type === "ArrayExpression" && node.elements.length === 0 ||
                    node.type === "SwitchStatement" && node.cases.length === 0) {
                    comments.trailing = this.getTokens(node, {
                        includeComments: true,
                        filter: isCommentToken
                    });
                }
                /*
                 * Iterate over tokens before and after node and collect comment tokens.
                 * Do not include comments that exist outside of the parent node
                 * to avoid duplication.
                 */
                let currentToken = this.getTokenBefore(node, { includeComments: true });
                while (currentToken && isCommentToken(currentToken)) {
                    if (node.parent && (currentToken.start < node.parent.start)) {
                        break;
                    }
                    comments.leading.push(currentToken);
                    currentToken = this.getTokenBefore(currentToken, { includeComments: true });
                }
                comments.leading.reverse();
                currentToken = this.getTokenAfter(node, { includeComments: true });
                while (currentToken && isCommentToken(currentToken)) {
                    if (node.parent && (currentToken.end > node.parent.end)) {
                        break;
                    }
                    comments.trailing.push(currentToken);
                    currentToken = this.getTokenAfter(currentToken, { includeComments: true });
                }
            }
            this._commentCache.set(node, comments);
            return comments;
        }
        /**
         * Retrieves the JSDoc comment for a given node.
         * @param {ASTNode} node The AST node to get the comment for.
         * @returns {Token|null} The Block comment token containing the JSDoc comment
         *      for the given node or null if not found.
         * @public
         * @deprecated
         */
        getJSDocComment(node) {
            /**
             * Checks for the presence of a JSDoc comment for the given node and returns it.
             * @param {ASTNode} astNode The AST node to get the comment for.
             * @returns {Token|null} The Block comment token containing the JSDoc comment
             *      for the given node or null if not found.
             * @private
             */
            const findJSDocComment = astNode => {
                const tokenBefore = this.getTokenBefore(astNode, { includeComments: true });
                if (tokenBefore &&
                    isCommentToken(tokenBefore) &&
                    tokenBefore.type === "Block" &&
                    tokenBefore.value.charAt(0) === "*" &&
                    astNode.loc.start.line - tokenBefore.loc.end.line <= 1) {
                    return tokenBefore;
                }
                return null;
            };
            let parent = node.parent;
            switch (node.type) {
                case "ClassDeclaration":
                case "FunctionDeclaration":
                    return findJSDocComment(looksLikeExport(parent) ? parent : node);
                case "ClassExpression":
                    return findJSDocComment(parent.parent);
                case "ArrowFunctionExpression":
                case "FunctionExpression":
                    if (parent.type !== "CallExpression" && parent.type !== "NewExpression") {
                        while (!this.getCommentsBefore(parent).length &&
                            !/Function/u.test(parent.type) &&
                            parent.type !== "MethodDefinition" &&
                            parent.type !== "Property") {
                            parent = parent.parent;
                            if (!parent) {
                                break;
                            }
                        }
                        if (parent && parent.type !== "FunctionDeclaration" && parent.type !== "Program") {
                            return findJSDocComment(parent);
                        }
                    }
                    return findJSDocComment(node);
                // falls through
                default:
                    return null;
            }
        }
        /**
         * Gets the deepest node containing a range index.
         * @param {int} index Range index of the desired node.
         * @returns {ASTNode} The node if found or null if not found.
         * @public
         */
        getNodeByRangeIndex(index) {
            let result = null;
            Traverser.traverse(this.ast, {
                visitorKeys: this.visitorKeys,
                enter(node) {
                    if (node.range[0] <= index && index < node.range[1]) {
                        result = node;
                    }
                    else {
                        this.skip();
                    }
                },
                leave(node) {
                    if (node === result) {
                        this.break();
                    }
                }
            });
            return result;
        }
        /**
         * Determines if two nodes or tokens have at least one whitespace character
         * between them. Order does not matter. Returns false if the given nodes or
         * tokens overlap.
         * @param {ASTNode|Token} first The first node or token to check between.
         * @param {ASTNode|Token} second The second node or token to check between.
         * @returns {boolean} True if there is a whitespace character between
         * any of the tokens found between the two given nodes or tokens.
         * @public
         */
        isSpaceBetween(first, second) {
            return isSpaceBetween(this, first, second, false);
        }
        /**
         * Determines if two nodes or tokens have at least one whitespace character
         * between them. Order does not matter. Returns false if the given nodes or
         * tokens overlap.
         * For backward compatibility, this method returns true if there are
         * `JSXText` tokens that contain whitespaces between the two.
         * @param {ASTNode|Token} first The first node or token to check between.
         * @param {ASTNode|Token} second The second node or token to check between.
         * @returns {boolean} True if there is a whitespace character between
         * any of the tokens found between the two given nodes or tokens.
         * @deprecated in favor of isSpaceBetween().
         * @public
         */
        isSpaceBetweenTokens(first, second) {
            return isSpaceBetween(this, first, second, true);
        }
        /**
         * Converts a source text index into a (line, column) pair.
         * @param {number} index The index of a character in a file
         * @returns {Object} A {line, column} location object with a 0-indexed column
         * @public
         */
        getLocFromIndex(index) {
            if (typeof index !== "number") {
                throw new TypeError("Expected `index` to be a number.");
            }
            if (index < 0 || index > this.text.length) {
                throw new RangeError(`Index out of range (requested index ${index}, but source text has length ${this.text.length}).`);
            }
            /*
             * For an argument of this.text.length, return the location one "spot" past the last character
             * of the file. If the last character is a linebreak, the location will be column 0 of the next
             * line; otherwise, the location will be in the next column on the same line.
             *
             * See getIndexFromLoc for the motivation for this special case.
             */
            if (index === this.text.length) {
                return { line: this.lines.length, column: this.lines[this.lines.length - 1].length };
            }
            /*
             * To figure out which line rangeIndex is on, determine the last index at which rangeIndex could
             * be inserted into lineIndices to keep the list sorted.
             */
            const lineNumber = lodash.sortedLastIndex(this.lineStartIndices, index);
            return { line: lineNumber, column: index - this.lineStartIndices[lineNumber - 1] };
        }
        /**
         * Converts a (line, column) pair into a range index.
         * @param {Object} loc A line/column location
         * @param {number} loc.line The line number of the location (1-indexed)
         * @param {number} loc.column The column number of the location (0-indexed)
         * @returns {number} The range index of the location in the file.
         * @public
         */
        getIndexFromLoc(loc) {
            if (typeof loc !== "object" || typeof loc.line !== "number" || typeof loc.column !== "number") {
                throw new TypeError("Expected `loc` to be an object with numeric `line` and `column` properties.");
            }
            if (loc.line <= 0) {
                throw new RangeError(`Line number out of range (line ${loc.line} requested). Line numbers should be 1-based.`);
            }
            if (loc.line > this.lineStartIndices.length) {
                throw new RangeError(`Line number out of range (line ${loc.line} requested, but only ${this.lineStartIndices.length} lines present).`);
            }
            const lineStartIndex = this.lineStartIndices[loc.line - 1];
            const lineEndIndex = loc.line === this.lineStartIndices.length ? this.text.length : this.lineStartIndices[loc.line];
            const positionIndex = lineStartIndex + loc.column;
            /*
             * By design, getIndexFromLoc({ line: lineNum, column: 0 }) should return the start index of
             * the given line, provided that the line number is valid element of this.lines. Since the
             * last element of this.lines is an empty string for files with trailing newlines, add a
             * special case where getting the index for the first location after the end of the file
             * will return the length of the file, rather than throwing an error. This allows rules to
             * use getIndexFromLoc consistently without worrying about edge cases at the end of a file.
             */
            if (loc.line === this.lineStartIndices.length && positionIndex > lineEndIndex ||
                loc.line < this.lineStartIndices.length && positionIndex >= lineEndIndex) {
                throw new RangeError(`Column number out of range (column ${loc.column} requested, but the length of line ${loc.line} is ${lineEndIndex - lineStartIndex}).`);
            }
            return positionIndex;
        }
    }
    module.exports = SourceCode;
},
/* eslint/lib/source-code/token-store/index.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const assert = require("assert");
    const { isCommentToken } = require(250) /* eslint-utils */;
    const cursors = require(474) /* ./cursors */;
    const ForwardTokenCursor = require(482) /* ./forward-token-cursor */;
    const PaddedTokenCursor = require(485) /* ./padded-token-cursor */;
    const utils = require(477) /* ./utils */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const TOKENS = Symbol("tokens");
    const COMMENTS = Symbol("comments");
    const INDEX_MAP = Symbol("indexMap");
    /**
     * Creates the map from locations to indices in `tokens`.
     *
     * The first/last location of tokens is mapped to the index of the token.
     * The first/last location of comments is mapped to the index of the next token of each comment.
     * @param {Token[]} tokens The array of tokens.
     * @param {Comment[]} comments The array of comments.
     * @returns {Object} The map from locations to indices in `tokens`.
     * @private
     */
    function createIndexMap(tokens, comments) {
        const map = Object.create(null);
        let tokenIndex = 0;
        let commentIndex = 0;
        let nextStart = 0;
        let range = null;
        while (tokenIndex < tokens.length || commentIndex < comments.length) {
            nextStart = (commentIndex < comments.length) ? comments[commentIndex].range[0] : Number.MAX_SAFE_INTEGER;
            while (tokenIndex < tokens.length && (range = tokens[tokenIndex].range)[0] < nextStart) {
                map[range[0]] = tokenIndex;
                map[range[1] - 1] = tokenIndex;
                tokenIndex += 1;
            }
            nextStart = (tokenIndex < tokens.length) ? tokens[tokenIndex].range[0] : Number.MAX_SAFE_INTEGER;
            while (commentIndex < comments.length && (range = comments[commentIndex].range)[0] < nextStart) {
                map[range[0]] = tokenIndex;
                map[range[1] - 1] = tokenIndex;
                commentIndex += 1;
            }
        }
        return map;
    }
    /**
     * Creates the cursor iterates tokens with options.
     * @param {CursorFactory} factory The cursor factory to initialize cursor.
     * @param {Token[]} tokens The array of tokens.
     * @param {Comment[]} comments The array of comments.
     * @param {Object} indexMap The map from locations to indices in `tokens`.
     * @param {number} startLoc The start location of the iteration range.
     * @param {number} endLoc The end location of the iteration range.
     * @param {number|Function|Object} [opts=0] The option object. If this is a number then it's `opts.skip`. If this is a function then it's `opts.filter`.
     * @param {boolean} [opts.includeComments=false] The flag to iterate comments as well.
     * @param {Function|null} [opts.filter=null] The predicate function to choose tokens.
     * @param {number} [opts.skip=0] The count of tokens the cursor skips.
     * @returns {Cursor} The created cursor.
     * @private
     */
    function createCursorWithSkip(factory, tokens, comments, indexMap, startLoc, endLoc, opts) {
        let includeComments = false;
        let skip = 0;
        let filter = null;
        if (typeof opts === "number") {
            skip = opts | 0;
        }
        else if (typeof opts === "function") {
            filter = opts;
        }
        else if (opts) {
            includeComments = !!opts.includeComments;
            skip = opts.skip | 0;
            filter = opts.filter || null;
        }
        assert(skip >= 0, "options.skip should be zero or a positive integer.");
        assert(!filter || typeof filter === "function", "options.filter should be a function.");
        return factory.createCursor(tokens, comments, indexMap, startLoc, endLoc, includeComments, filter, skip, -1);
    }
    /**
     * Creates the cursor iterates tokens with options.
     * @param {CursorFactory} factory The cursor factory to initialize cursor.
     * @param {Token[]} tokens The array of tokens.
     * @param {Comment[]} comments The array of comments.
     * @param {Object} indexMap The map from locations to indices in `tokens`.
     * @param {number} startLoc The start location of the iteration range.
     * @param {number} endLoc The end location of the iteration range.
     * @param {number|Function|Object} [opts=0] The option object. If this is a number then it's `opts.count`. If this is a function then it's `opts.filter`.
     * @param {boolean} [opts.includeComments] The flag to iterate comments as well.
     * @param {Function|null} [opts.filter=null] The predicate function to choose tokens.
     * @param {number} [opts.count=0] The maximum count of tokens the cursor iterates. Zero is no iteration for backward compatibility.
     * @returns {Cursor} The created cursor.
     * @private
     */
    function createCursorWithCount(factory, tokens, comments, indexMap, startLoc, endLoc, opts) {
        let includeComments = false;
        let count = 0;
        let countExists = false;
        let filter = null;
        if (typeof opts === "number") {
            count = opts | 0;
            countExists = true;
        }
        else if (typeof opts === "function") {
            filter = opts;
        }
        else if (opts) {
            includeComments = !!opts.includeComments;
            count = opts.count | 0;
            countExists = typeof opts.count === "number";
            filter = opts.filter || null;
        }
        assert(count >= 0, "options.count should be zero or a positive integer.");
        assert(!filter || typeof filter === "function", "options.filter should be a function.");
        return factory.createCursor(tokens, comments, indexMap, startLoc, endLoc, includeComments, filter, 0, countExists ? count : -1);
    }
    /**
     * Creates the cursor iterates tokens with options.
     * This is overload function of the below.
     * @param {Token[]} tokens The array of tokens.
     * @param {Comment[]} comments The array of comments.
     * @param {Object} indexMap The map from locations to indices in `tokens`.
     * @param {number} startLoc The start location of the iteration range.
     * @param {number} endLoc The end location of the iteration range.
     * @param {Function|Object} opts The option object. If this is a function then it's `opts.filter`.
     * @param {boolean} [opts.includeComments] The flag to iterate comments as well.
     * @param {Function|null} [opts.filter=null] The predicate function to choose tokens.
     * @param {number} [opts.count=0] The maximum count of tokens the cursor iterates. Zero is no iteration for backward compatibility.
     * @returns {Cursor} The created cursor.
     * @private
     */
    /**
     * Creates the cursor iterates tokens with options.
     * @param {Token[]} tokens The array of tokens.
     * @param {Comment[]} comments The array of comments.
     * @param {Object} indexMap The map from locations to indices in `tokens`.
     * @param {number} startLoc The start location of the iteration range.
     * @param {number} endLoc The end location of the iteration range.
     * @param {number} [beforeCount=0] The number of tokens before the node to retrieve.
     * @param {boolean} [afterCount=0] The number of tokens after the node to retrieve.
     * @returns {Cursor} The created cursor.
     * @private
     */
    function createCursorWithPadding(tokens, comments, indexMap, startLoc, endLoc, beforeCount, afterCount) {
        if (typeof beforeCount === "undefined" && typeof afterCount === "undefined") {
            return new ForwardTokenCursor(tokens, comments, indexMap, startLoc, endLoc);
        }
        if (typeof beforeCount === "number" || typeof beforeCount === "undefined") {
            return new PaddedTokenCursor(tokens, comments, indexMap, startLoc, endLoc, beforeCount | 0, afterCount | 0);
        }
        return createCursorWithCount(cursors.forward, tokens, comments, indexMap, startLoc, endLoc, beforeCount);
    }
    /**
     * Gets comment tokens that are adjacent to the current cursor position.
     * @param {Cursor} cursor A cursor instance.
     * @returns {Array} An array of comment tokens adjacent to the current cursor position.
     * @private
     */
    function getAdjacentCommentTokensFromCursor(cursor) {
        const tokens = [];
        let currentToken = cursor.getOneToken();
        while (currentToken && isCommentToken(currentToken)) {
            tokens.push(currentToken);
            currentToken = cursor.getOneToken();
        }
        return tokens;
    }
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * The token store.
     *
     * This class provides methods to get tokens by locations as fast as possible.
     * The methods are a part of public API, so we should be careful if it changes this class.
     *
     * People can get tokens in O(1) by the hash map which is mapping from the location of tokens/comments to tokens.
     * Also people can get a mix of tokens and comments in O(log k), the k is the number of comments.
     * Assuming that comments to be much fewer than tokens, this does not make hash map from token's locations to comments to reduce memory cost.
     * This uses binary-searching instead for comments.
     */
    module.exports = class TokenStore {
        /**
         * Initializes this token store.
         * @param {Token[]} tokens The array of tokens.
         * @param {Comment[]} comments The array of comments.
         */
        constructor(tokens, comments) {
            this[TOKENS] = tokens;
            this[COMMENTS] = comments;
            this[INDEX_MAP] = createIndexMap(tokens, comments);
        }
        //--------------------------------------------------------------------------
        // Gets single token.
        //--------------------------------------------------------------------------
        /**
         * Gets the token starting at the specified index.
         * @param {number} offset Index of the start of the token's range.
         * @param {Object} [options=0] The option object.
         * @param {boolean} [options.includeComments=false] The flag to iterate comments as well.
         * @returns {Token|null} The token starting at index, or null if no such token.
         */
        getTokenByRangeStart(offset, options) {
            const includeComments = options && options.includeComments;
            const token = cursors.forward.createBaseCursor(this[TOKENS], this[COMMENTS], this[INDEX_MAP], offset, -1, includeComments).getOneToken();
            if (token && token.range[0] === offset) {
                return token;
            }
            return null;
        }
        /**
         * Gets the first token of the given node.
         * @param {ASTNode} node The AST node.
         * @param {number|Function|Object} [options=0] The option object. If this is a number then it's `options.skip`. If this is a function then it's `options.filter`.
         * @param {boolean} [options.includeComments=false] The flag to iterate comments as well.
         * @param {Function|null} [options.filter=null] The predicate function to choose tokens.
         * @param {number} [options.skip=0] The count of tokens the cursor skips.
         * @returns {Token|null} An object representing the token.
         */
        getFirstToken(node, options) {
            return createCursorWithSkip(cursors.forward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], node.range[0], node.range[1], options).getOneToken();
        }
        /**
         * Gets the last token of the given node.
         * @param {ASTNode} node The AST node.
         * @param {number|Function|Object} [options=0] The option object. Same options as getFirstToken()
         * @returns {Token|null} An object representing the token.
         */
        getLastToken(node, options) {
            return createCursorWithSkip(cursors.backward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], node.range[0], node.range[1], options).getOneToken();
        }
        /**
         * Gets the token that precedes a given node or token.
         * @param {ASTNode|Token|Comment} node The AST node or token.
         * @param {number|Function|Object} [options=0] The option object. Same options as getFirstToken()
         * @returns {Token|null} An object representing the token.
         */
        getTokenBefore(node, options) {
            return createCursorWithSkip(cursors.backward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], -1, node.range[0], options).getOneToken();
        }
        /**
         * Gets the token that follows a given node or token.
         * @param {ASTNode|Token|Comment} node The AST node or token.
         * @param {number|Function|Object} [options=0] The option object. Same options as getFirstToken()
         * @returns {Token|null} An object representing the token.
         */
        getTokenAfter(node, options) {
            return createCursorWithSkip(cursors.forward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], node.range[1], -1, options).getOneToken();
        }
        /**
         * Gets the first token between two non-overlapping nodes.
         * @param {ASTNode|Token|Comment} left Node before the desired token range.
         * @param {ASTNode|Token|Comment} right Node after the desired token range.
         * @param {number|Function|Object} [options=0] The option object. Same options as getFirstToken()
         * @returns {Token|null} An object representing the token.
         */
        getFirstTokenBetween(left, right, options) {
            return createCursorWithSkip(cursors.forward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], left.range[1], right.range[0], options).getOneToken();
        }
        /**
         * Gets the last token between two non-overlapping nodes.
         * @param {ASTNode|Token|Comment} left Node before the desired token range.
         * @param {ASTNode|Token|Comment} right Node after the desired token range.
         * @param {number|Function|Object} [options=0] The option object. Same options as getFirstToken()
         * @returns {Token|null} An object representing the token.
         */
        getLastTokenBetween(left, right, options) {
            return createCursorWithSkip(cursors.backward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], left.range[1], right.range[0], options).getOneToken();
        }
        /**
         * Gets the token that precedes a given node or token in the token stream.
         * This is defined for backward compatibility. Use `includeComments` option instead.
         * TODO: We have a plan to remove this in a future major version.
         * @param {ASTNode|Token|Comment} node The AST node or token.
         * @param {number} [skip=0] A number of tokens to skip.
         * @returns {Token|null} An object representing the token.
         * @deprecated
         */
        getTokenOrCommentBefore(node, skip) {
            return this.getTokenBefore(node, { includeComments: true, skip });
        }
        /**
         * Gets the token that follows a given node or token in the token stream.
         * This is defined for backward compatibility. Use `includeComments` option instead.
         * TODO: We have a plan to remove this in a future major version.
         * @param {ASTNode|Token|Comment} node The AST node or token.
         * @param {number} [skip=0] A number of tokens to skip.
         * @returns {Token|null} An object representing the token.
         * @deprecated
         */
        getTokenOrCommentAfter(node, skip) {
            return this.getTokenAfter(node, { includeComments: true, skip });
        }
        //--------------------------------------------------------------------------
        // Gets multiple tokens.
        //--------------------------------------------------------------------------
        /**
         * Gets the first `count` tokens of the given node.
         * @param {ASTNode} node The AST node.
         * @param {number|Function|Object} [options=0] The option object. If this is a number then it's `options.count`. If this is a function then it's `options.filter`.
         * @param {boolean} [options.includeComments=false] The flag to iterate comments as well.
         * @param {Function|null} [options.filter=null] The predicate function to choose tokens.
         * @param {number} [options.count=0] The maximum count of tokens the cursor iterates.
         * @returns {Token[]} Tokens.
         */
        getFirstTokens(node, options) {
            return createCursorWithCount(cursors.forward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], node.range[0], node.range[1], options).getAllTokens();
        }
        /**
         * Gets the last `count` tokens of the given node.
         * @param {ASTNode} node The AST node.
         * @param {number|Function|Object} [options=0] The option object. Same options as getFirstTokens()
         * @returns {Token[]} Tokens.
         */
        getLastTokens(node, options) {
            return createCursorWithCount(cursors.backward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], node.range[0], node.range[1], options).getAllTokens().reverse();
        }
        /**
         * Gets the `count` tokens that precedes a given node or token.
         * @param {ASTNode|Token|Comment} node The AST node or token.
         * @param {number|Function|Object} [options=0] The option object. Same options as getFirstTokens()
         * @returns {Token[]} Tokens.
         */
        getTokensBefore(node, options) {
            return createCursorWithCount(cursors.backward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], -1, node.range[0], options).getAllTokens().reverse();
        }
        /**
         * Gets the `count` tokens that follows a given node or token.
         * @param {ASTNode|Token|Comment} node The AST node or token.
         * @param {number|Function|Object} [options=0] The option object. Same options as getFirstTokens()
         * @returns {Token[]} Tokens.
         */
        getTokensAfter(node, options) {
            return createCursorWithCount(cursors.forward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], node.range[1], -1, options).getAllTokens();
        }
        /**
         * Gets the first `count` tokens between two non-overlapping nodes.
         * @param {ASTNode|Token|Comment} left Node before the desired token range.
         * @param {ASTNode|Token|Comment} right Node after the desired token range.
         * @param {number|Function|Object} [options=0] The option object. Same options as getFirstTokens()
         * @returns {Token[]} Tokens between left and right.
         */
        getFirstTokensBetween(left, right, options) {
            return createCursorWithCount(cursors.forward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], left.range[1], right.range[0], options).getAllTokens();
        }
        /**
         * Gets the last `count` tokens between two non-overlapping nodes.
         * @param {ASTNode|Token|Comment} left Node before the desired token range.
         * @param {ASTNode|Token|Comment} right Node after the desired token range.
         * @param {number|Function|Object} [options=0] The option object. Same options as getFirstTokens()
         * @returns {Token[]} Tokens between left and right.
         */
        getLastTokensBetween(left, right, options) {
            return createCursorWithCount(cursors.backward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], left.range[1], right.range[0], options).getAllTokens().reverse();
        }
        /**
         * Gets all tokens that are related to the given node.
         * @param {ASTNode} node The AST node.
         * @param {Function|Object} options The option object. If this is a function then it's `options.filter`.
         * @param {boolean} [options.includeComments=false] The flag to iterate comments as well.
         * @param {Function|null} [options.filter=null] The predicate function to choose tokens.
         * @param {number} [options.count=0] The maximum count of tokens the cursor iterates.
         * @returns {Token[]} Array of objects representing tokens.
         */
        /**
         * Gets all tokens that are related to the given node.
         * @param {ASTNode} node The AST node.
         * @param {int} [beforeCount=0] The number of tokens before the node to retrieve.
         * @param {int} [afterCount=0] The number of tokens after the node to retrieve.
         * @returns {Token[]} Array of objects representing tokens.
         */
        getTokens(node, beforeCount, afterCount) {
            return createCursorWithPadding(this[TOKENS], this[COMMENTS], this[INDEX_MAP], node.range[0], node.range[1], beforeCount, afterCount).getAllTokens();
        }
        /**
         * Gets all of the tokens between two non-overlapping nodes.
         * @param {ASTNode|Token|Comment} left Node before the desired token range.
         * @param {ASTNode|Token|Comment} right Node after the desired token range.
         * @param {Function|Object} options The option object. If this is a function then it's `options.filter`.
         * @param {boolean} [options.includeComments=false] The flag to iterate comments as well.
         * @param {Function|null} [options.filter=null] The predicate function to choose tokens.
         * @param {number} [options.count=0] The maximum count of tokens the cursor iterates.
         * @returns {Token[]} Tokens between left and right.
         */
        /**
         * Gets all of the tokens between two non-overlapping nodes.
         * @param {ASTNode|Token|Comment} left Node before the desired token range.
         * @param {ASTNode|Token|Comment} right Node after the desired token range.
         * @param {int} [padding=0] Number of extra tokens on either side of center.
         * @returns {Token[]} Tokens between left and right.
         */
        getTokensBetween(left, right, padding) {
            return createCursorWithPadding(this[TOKENS], this[COMMENTS], this[INDEX_MAP], left.range[1], right.range[0], padding, padding).getAllTokens();
        }
        //--------------------------------------------------------------------------
        // Others.
        //--------------------------------------------------------------------------
        /**
         * Checks whether any comments exist or not between the given 2 nodes.
         * @param {ASTNode} left The node to check.
         * @param {ASTNode} right The node to check.
         * @returns {boolean} `true` if one or more comments exist.
         */
        commentsExistBetween(left, right) {
            const index = utils.search(this[COMMENTS], left.range[1]);
            return (index < this[COMMENTS].length &&
                this[COMMENTS][index].range[1] <= right.range[0]);
        }
        /**
         * Gets all comment tokens directly before the given node or token.
         * @param {ASTNode|token} nodeOrToken The AST node or token to check for adjacent comment tokens.
         * @returns {Array} An array of comments in occurrence order.
         */
        getCommentsBefore(nodeOrToken) {
            const cursor = createCursorWithCount(cursors.backward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], -1, nodeOrToken.range[0], { includeComments: true });
            return getAdjacentCommentTokensFromCursor(cursor).reverse();
        }
        /**
         * Gets all comment tokens directly after the given node or token.
         * @param {ASTNode|token} nodeOrToken The AST node or token to check for adjacent comment tokens.
         * @returns {Array} An array of comments in occurrence order.
         */
        getCommentsAfter(nodeOrToken) {
            const cursor = createCursorWithCount(cursors.forward, this[TOKENS], this[COMMENTS], this[INDEX_MAP], nodeOrToken.range[1], -1, { includeComments: true });
            return getAdjacentCommentTokensFromCursor(cursor);
        }
        /**
         * Gets all comment tokens inside the given node.
         * @param {ASTNode} node The AST node to get the comments for.
         * @returns {Array} An array of comments in occurrence order.
         */
        getCommentsInside(node) {
            return this.getTokens(node, {
                includeComments: true,
                filter: isCommentToken
            });
        }
    };
},
/* eslint/lib/source-code/token-store/cursors.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const BackwardTokenCommentCursor = require(475) /* ./backward-token-comment-cursor */;
    const BackwardTokenCursor = require(478) /* ./backward-token-cursor */;
    const FilterCursor = require(479) /* ./filter-cursor */;
    const ForwardTokenCommentCursor = require(481) /* ./forward-token-comment-cursor */;
    const ForwardTokenCursor = require(482) /* ./forward-token-cursor */;
    const LimitCursor = require(483) /* ./limit-cursor */;
    const SkipCursor = require(484) /* ./skip-cursor */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * The cursor factory.
     * @private
     */
    class CursorFactory {
        /**
         * Initializes this cursor.
         * @param {Function} TokenCursor The class of the cursor which iterates tokens only.
         * @param {Function} TokenCommentCursor The class of the cursor which iterates the mix of tokens and comments.
         */
        constructor(TokenCursor, TokenCommentCursor) {
            this.TokenCursor = TokenCursor;
            this.TokenCommentCursor = TokenCommentCursor;
        }
        /**
         * Creates a base cursor instance that can be decorated by createCursor.
         * @param {Token[]} tokens The array of tokens.
         * @param {Comment[]} comments The array of comments.
         * @param {Object} indexMap The map from locations to indices in `tokens`.
         * @param {number} startLoc The start location of the iteration range.
         * @param {number} endLoc The end location of the iteration range.
         * @param {boolean} includeComments The flag to iterate comments as well.
         * @returns {Cursor} The created base cursor.
         */
        createBaseCursor(tokens, comments, indexMap, startLoc, endLoc, includeComments) {
            const Cursor = includeComments ? this.TokenCommentCursor : this.TokenCursor;
            return new Cursor(tokens, comments, indexMap, startLoc, endLoc);
        }
        /**
         * Creates a cursor that iterates tokens with normalized options.
         * @param {Token[]} tokens The array of tokens.
         * @param {Comment[]} comments The array of comments.
         * @param {Object} indexMap The map from locations to indices in `tokens`.
         * @param {number} startLoc The start location of the iteration range.
         * @param {number} endLoc The end location of the iteration range.
         * @param {boolean} includeComments The flag to iterate comments as well.
         * @param {Function|null} filter The predicate function to choose tokens.
         * @param {number} skip The count of tokens the cursor skips.
         * @param {number} count The maximum count of tokens the cursor iterates. Zero is no iteration for backward compatibility.
         * @returns {Cursor} The created cursor.
         */
        createCursor(tokens, comments, indexMap, startLoc, endLoc, includeComments, filter, skip, count) {
            let cursor = this.createBaseCursor(tokens, comments, indexMap, startLoc, endLoc, includeComments);
            if (filter) {
                cursor = new FilterCursor(cursor, filter);
            }
            if (skip >= 1) {
                cursor = new SkipCursor(cursor, skip);
            }
            if (count >= 0) {
                cursor = new LimitCursor(cursor, count);
            }
            return cursor;
        }
    }
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    exports.forward = new CursorFactory(ForwardTokenCursor, ForwardTokenCommentCursor);
    exports.backward = new CursorFactory(BackwardTokenCursor, BackwardTokenCommentCursor);
},
/* eslint/lib/source-code/token-store/backward-token-comment-cursor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const Cursor = require(476) /* ./cursor */;
    const utils = require(477) /* ./utils */;
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * The cursor which iterates tokens and comments in reverse.
     */
    module.exports = class BackwardTokenCommentCursor extends Cursor {
        /**
         * Initializes this cursor.
         * @param {Token[]} tokens The array of tokens.
         * @param {Comment[]} comments The array of comments.
         * @param {Object} indexMap The map from locations to indices in `tokens`.
         * @param {number} startLoc The start location of the iteration range.
         * @param {number} endLoc The end location of the iteration range.
         */
        constructor(tokens, comments, indexMap, startLoc, endLoc) {
            super();
            this.tokens = tokens;
            this.comments = comments;
            this.tokenIndex = utils.getLastIndex(tokens, indexMap, endLoc);
            this.commentIndex = utils.search(comments, endLoc) - 1;
            this.border = startLoc;
        }
        /** @inheritdoc */
        moveNext() {
            const token = (this.tokenIndex >= 0) ? this.tokens[this.tokenIndex] : null;
            const comment = (this.commentIndex >= 0) ? this.comments[this.commentIndex] : null;
            if (token && (!comment || token.range[1] > comment.range[1])) {
                this.current = token;
                this.tokenIndex -= 1;
            }
            else if (comment) {
                this.current = comment;
                this.commentIndex -= 1;
            }
            else {
                this.current = null;
            }
            return Boolean(this.current) && (this.border === -1 || this.current.range[0] >= this.border);
        }
    };
},
/* eslint/lib/source-code/token-store/cursor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * The abstract class about cursors which iterate tokens.
     *
     * This class has 2 abstract methods.
     *
     * - `current: Token | Comment | null` ... The current token.
     * - `moveNext(): boolean` ... Moves this cursor to the next token. If the next token didn't exist, it returns `false`.
     *
     * This is similar to ES2015 Iterators.
     * However, Iterators were slow (at 2017-01), so I created this class as similar to C# IEnumerable.
     *
     * There are the following known sub classes.
     *
     * - ForwardTokenCursor .......... The cursor which iterates tokens only.
     * - BackwardTokenCursor ......... The cursor which iterates tokens only in reverse.
     * - ForwardTokenCommentCursor ... The cursor which iterates tokens and comments.
     * - BackwardTokenCommentCursor .. The cursor which iterates tokens and comments in reverse.
     * - DecorativeCursor
     *     - FilterCursor ............ The cursor which ignores the specified tokens.
     *     - SkipCursor .............. The cursor which ignores the first few tokens.
     *     - LimitCursor ............. The cursor which limits the count of tokens.
     *
     */
    module.exports = class Cursor {
        /**
         * Initializes this cursor.
         */
        constructor() {
            this.current = null;
        }
        /**
         * Gets the first token.
         * This consumes this cursor.
         * @returns {Token|Comment} The first token or null.
         */
        getOneToken() {
            return this.moveNext() ? this.current : null;
        }
        /**
         * Gets the first tokens.
         * This consumes this cursor.
         * @returns {(Token|Comment)[]} All tokens.
         */
        getAllTokens() {
            const tokens = [];
            while (this.moveNext()) {
                tokens.push(this.current);
            }
            return tokens;
        }
        /**
         * Moves this cursor to the next token.
         * @returns {boolean} `true` if the next token exists.
         * @abstract
         */
        /* istanbul ignore next */
        moveNext() {
            throw new Error("Not implemented.");
        }
    };
},
/* eslint/lib/source-code/token-store/utils.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const lodash = require(111) /* lodash */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Gets `token.range[0]` from the given token.
     * @param {Node|Token|Comment} token The token to get.
     * @returns {number} The start location.
     * @private
     */
    function getStartLocation(token) {
        return token.range[0];
    }
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * Binary-searches the index of the first token which is after the given location.
     * If it was not found, this returns `tokens.length`.
     * @param {(Token|Comment)[]} tokens It searches the token in this list.
     * @param {number} location The location to search.
     * @returns {number} The found index or `tokens.length`.
     */
    exports.search = function search(tokens, location) {
        return lodash.sortedIndexBy(tokens, { range: [location] }, getStartLocation);
    };
    /**
     * Gets the index of the `startLoc` in `tokens`.
     * `startLoc` can be the value of `node.range[1]`, so this checks about `startLoc - 1` as well.
     * @param {(Token|Comment)[]} tokens The tokens to find an index.
     * @param {Object} indexMap The map from locations to indices.
     * @param {number} startLoc The location to get an index.
     * @returns {number} The index.
     */
    exports.getFirstIndex = function getFirstIndex(tokens, indexMap, startLoc) {
        if (startLoc in indexMap) {
            return indexMap[startLoc];
        }
        if ((startLoc - 1) in indexMap) {
            const index = indexMap[startLoc - 1];
            const token = (index >= 0 && index < tokens.length) ? tokens[index] : null;
            /*
             * For the map of "comment's location -> token's index", it points the next token of a comment.
             * In that case, +1 is unnecessary.
             */
            if (token && token.range[0] >= startLoc) {
                return index;
            }
            return index + 1;
        }
        return 0;
    };
    /**
     * Gets the index of the `endLoc` in `tokens`.
     * The information of end locations are recorded at `endLoc - 1` in `indexMap`, so this checks about `endLoc - 1` as well.
     * @param {(Token|Comment)[]} tokens The tokens to find an index.
     * @param {Object} indexMap The map from locations to indices.
     * @param {number} endLoc The location to get an index.
     * @returns {number} The index.
     */
    exports.getLastIndex = function getLastIndex(tokens, indexMap, endLoc) {
        if (endLoc in indexMap) {
            return indexMap[endLoc] - 1;
        }
        if ((endLoc - 1) in indexMap) {
            const index = indexMap[endLoc - 1];
            const token = (index >= 0 && index < tokens.length) ? tokens[index] : null;
            /*
             * For the map of "comment's location -> token's index", it points the next token of a comment.
             * In that case, -1 is necessary.
             */
            if (token && token.range[1] > endLoc) {
                return index - 1;
            }
            return index;
        }
        return tokens.length - 1;
    };
},
/* eslint/lib/source-code/token-store/backward-token-cursor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const Cursor = require(476) /* ./cursor */;
    const utils = require(477) /* ./utils */;
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * The cursor which iterates tokens only in reverse.
     */
    module.exports = class BackwardTokenCursor extends Cursor {
        /**
         * Initializes this cursor.
         * @param {Token[]} tokens The array of tokens.
         * @param {Comment[]} comments The array of comments.
         * @param {Object} indexMap The map from locations to indices in `tokens`.
         * @param {number} startLoc The start location of the iteration range.
         * @param {number} endLoc The end location of the iteration range.
         */
        constructor(tokens, comments, indexMap, startLoc, endLoc) {
            super();
            this.tokens = tokens;
            this.index = utils.getLastIndex(tokens, indexMap, endLoc);
            this.indexEnd = utils.getFirstIndex(tokens, indexMap, startLoc);
        }
        /** @inheritdoc */
        moveNext() {
            if (this.index >= this.indexEnd) {
                this.current = this.tokens[this.index];
                this.index -= 1;
                return true;
            }
            return false;
        }
        /*
         *
         * Shorthand for performance.
         *
         */
        /** @inheritdoc */
        getOneToken() {
            return (this.index >= this.indexEnd) ? this.tokens[this.index] : null;
        }
    };
},
/* eslint/lib/source-code/token-store/filter-cursor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const DecorativeCursor = require(480) /* ./decorative-cursor */;
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * The decorative cursor which ignores specified tokens.
     */
    module.exports = class FilterCursor extends DecorativeCursor {
        /**
         * Initializes this cursor.
         * @param {Cursor} cursor The cursor to be decorated.
         * @param {Function} predicate The predicate function to decide tokens this cursor iterates.
         */
        constructor(cursor, predicate) {
            super(cursor);
            this.predicate = predicate;
        }
        /** @inheritdoc */
        moveNext() {
            const predicate = this.predicate;
            while (super.moveNext()) {
                if (predicate(this.current)) {
                    return true;
                }
            }
            return false;
        }
    };
},
/* eslint/lib/source-code/token-store/decorative-cursor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const Cursor = require(476) /* ./cursor */;
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * The abstract class about cursors which manipulate another cursor.
     */
    module.exports = class DecorativeCursor extends Cursor {
        /**
         * Initializes this cursor.
         * @param {Cursor} cursor The cursor to be decorated.
         */
        constructor(cursor) {
            super();
            this.cursor = cursor;
        }
        /** @inheritdoc */
        moveNext() {
            const retv = this.cursor.moveNext();
            this.current = this.cursor.current;
            return retv;
        }
    };
},
/* eslint/lib/source-code/token-store/forward-token-comment-cursor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const Cursor = require(476) /* ./cursor */;
    const utils = require(477) /* ./utils */;
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * The cursor which iterates tokens and comments.
     */
    module.exports = class ForwardTokenCommentCursor extends Cursor {
        /**
         * Initializes this cursor.
         * @param {Token[]} tokens The array of tokens.
         * @param {Comment[]} comments The array of comments.
         * @param {Object} indexMap The map from locations to indices in `tokens`.
         * @param {number} startLoc The start location of the iteration range.
         * @param {number} endLoc The end location of the iteration range.
         */
        constructor(tokens, comments, indexMap, startLoc, endLoc) {
            super();
            this.tokens = tokens;
            this.comments = comments;
            this.tokenIndex = utils.getFirstIndex(tokens, indexMap, startLoc);
            this.commentIndex = utils.search(comments, startLoc);
            this.border = endLoc;
        }
        /** @inheritdoc */
        moveNext() {
            const token = (this.tokenIndex < this.tokens.length) ? this.tokens[this.tokenIndex] : null;
            const comment = (this.commentIndex < this.comments.length) ? this.comments[this.commentIndex] : null;
            if (token && (!comment || token.range[0] < comment.range[0])) {
                this.current = token;
                this.tokenIndex += 1;
            }
            else if (comment) {
                this.current = comment;
                this.commentIndex += 1;
            }
            else {
                this.current = null;
            }
            return Boolean(this.current) && (this.border === -1 || this.current.range[1] <= this.border);
        }
    };
},
/* eslint/lib/source-code/token-store/forward-token-cursor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const Cursor = require(476) /* ./cursor */;
    const utils = require(477) /* ./utils */;
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * The cursor which iterates tokens only.
     */
    module.exports = class ForwardTokenCursor extends Cursor {
        /**
         * Initializes this cursor.
         * @param {Token[]} tokens The array of tokens.
         * @param {Comment[]} comments The array of comments.
         * @param {Object} indexMap The map from locations to indices in `tokens`.
         * @param {number} startLoc The start location of the iteration range.
         * @param {number} endLoc The end location of the iteration range.
         */
        constructor(tokens, comments, indexMap, startLoc, endLoc) {
            super();
            this.tokens = tokens;
            this.index = utils.getFirstIndex(tokens, indexMap, startLoc);
            this.indexEnd = utils.getLastIndex(tokens, indexMap, endLoc);
        }
        /** @inheritdoc */
        moveNext() {
            if (this.index <= this.indexEnd) {
                this.current = this.tokens[this.index];
                this.index += 1;
                return true;
            }
            return false;
        }
        /*
         *
         * Shorthand for performance.
         *
         */
        /** @inheritdoc */
        getOneToken() {
            return (this.index <= this.indexEnd) ? this.tokens[this.index] : null;
        }
        /** @inheritdoc */
        getAllTokens() {
            return this.tokens.slice(this.index, this.indexEnd + 1);
        }
    };
},
/* eslint/lib/source-code/token-store/limit-cursor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const DecorativeCursor = require(480) /* ./decorative-cursor */;
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * The decorative cursor which limits the number of tokens.
     */
    module.exports = class LimitCursor extends DecorativeCursor {
        /**
         * Initializes this cursor.
         * @param {Cursor} cursor The cursor to be decorated.
         * @param {number} count The count of tokens this cursor iterates.
         */
        constructor(cursor, count) {
            super(cursor);
            this.count = count;
        }
        /** @inheritdoc */
        moveNext() {
            if (this.count > 0) {
                this.count -= 1;
                return super.moveNext();
            }
            return false;
        }
    };
},
/* eslint/lib/source-code/token-store/skip-cursor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const DecorativeCursor = require(480) /* ./decorative-cursor */;
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * The decorative cursor which ignores the first few tokens.
     */
    module.exports = class SkipCursor extends DecorativeCursor {
        /**
         * Initializes this cursor.
         * @param {Cursor} cursor The cursor to be decorated.
         * @param {number} count The count of tokens this cursor skips.
         */
        constructor(cursor, count) {
            super(cursor);
            this.count = count;
        }
        /** @inheritdoc */
        moveNext() {
            while (this.count > 0) {
                this.count -= 1;
                if (!super.moveNext()) {
                    return false;
                }
            }
            return super.moveNext();
        }
    };
},
/* eslint/lib/source-code/token-store/padded-token-cursor.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const ForwardTokenCursor = require(482) /* ./forward-token-cursor */;
    //------------------------------------------------------------------------------
    // Exports
    //------------------------------------------------------------------------------
    /**
     * The cursor which iterates tokens only, with inflated range.
     * This is for the backward compatibility of padding options.
     */
    module.exports = class PaddedTokenCursor extends ForwardTokenCursor {
        /**
         * Initializes this cursor.
         * @param {Token[]} tokens The array of tokens.
         * @param {Comment[]} comments The array of comments.
         * @param {Object} indexMap The map from locations to indices in `tokens`.
         * @param {number} startLoc The start location of the iteration range.
         * @param {number} endLoc The end location of the iteration range.
         * @param {number} beforeCount The number of tokens this cursor iterates before start.
         * @param {number} afterCount The number of tokens this cursor iterates after end.
         */
        constructor(tokens, comments, indexMap, startLoc, endLoc, beforeCount, afterCount) {
            super(tokens, comments, indexMap, startLoc, endLoc);
            this.index = Math.max(0, this.index - beforeCount);
            this.indexEnd = Math.min(tokens.length - 1, this.indexEnd + afterCount);
        }
    };
},
/* eslint/lib/linter/code-path-analysis/code-path-analyzer.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const assert = require("assert"), { breakableTypePattern } = require(115) /* ../../shared/ast-utils */, CodePath = require(487) /* ./code-path */, CodePathSegment = require(489) /* ./code-path-segment */, IdGenerator = require(492) /* ./id-generator */, debug = require(490) /* ./debug-helpers */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a given node is a `case` node (not `default` node).
     * @param {ASTNode} node A `SwitchCase` node to check.
     * @returns {boolean} `true` if the node is a `case` node (not `default` node).
     */
    function isCaseNode(node) {
        return Boolean(node.test);
    }
    /**
     * Checks whether the given logical operator is taken into account for the code
     * path analysis.
     * @param {string} operator The operator found in the LogicalExpression node
     * @returns {boolean} `true` if the operator is "&&" or "||"
     */
    function isHandledLogicalOperator(operator) {
        return operator === "&&" || operator === "||";
    }
    /**
     * Gets the label if the parent node of a given node is a LabeledStatement.
     * @param {ASTNode} node A node to get.
     * @returns {string|null} The label or `null`.
     */
    function getLabel(node) {
        if (node.parent.type === "LabeledStatement") {
            return node.parent.label.name;
        }
        return null;
    }
    /**
     * Checks whether or not a given logical expression node goes different path
     * between the `true` case and the `false` case.
     * @param {ASTNode} node A node to check.
     * @returns {boolean} `true` if the node is a test of a choice statement.
     */
    function isForkingByTrueOrFalse(node) {
        const parent = node.parent;
        switch (parent.type) {
            case "ConditionalExpression":
            case "IfStatement":
            case "WhileStatement":
            case "DoWhileStatement":
            case "ForStatement":
                return parent.test === node;
            case "LogicalExpression":
                return isHandledLogicalOperator(parent.operator);
            default:
                return false;
        }
    }
    /**
     * Gets the boolean value of a given literal node.
     *
     * This is used to detect infinity loops (e.g. `while (true) {}`).
     * Statements preceded by an infinity loop are unreachable if the loop didn't
     * have any `break` statement.
     * @param {ASTNode} node A node to get.
     * @returns {boolean|undefined} a boolean value if the node is a Literal node,
     *   otherwise `undefined`.
     */
    function getBooleanValueIfSimpleConstant(node) {
        if (node.type === "Literal") {
            return Boolean(node.value);
        }
        return void 0;
    }
    /**
     * Checks that a given identifier node is a reference or not.
     *
     * This is used to detect the first throwable node in a `try` block.
     * @param {ASTNode} node An Identifier node to check.
     * @returns {boolean} `true` if the node is a reference.
     */
    function isIdentifierReference(node) {
        const parent = node.parent;
        switch (parent.type) {
            case "LabeledStatement":
            case "BreakStatement":
            case "ContinueStatement":
            case "ArrayPattern":
            case "RestElement":
            case "ImportSpecifier":
            case "ImportDefaultSpecifier":
            case "ImportNamespaceSpecifier":
            case "CatchClause":
                return false;
            case "FunctionDeclaration":
            case "FunctionExpression":
            case "ArrowFunctionExpression":
            case "ClassDeclaration":
            case "ClassExpression":
            case "VariableDeclarator":
                return parent.id !== node;
            case "Property":
            case "MethodDefinition":
                return (parent.key !== node ||
                    parent.computed ||
                    parent.shorthand);
            case "AssignmentPattern":
                return parent.key !== node;
            default:
                return true;
        }
    }
    /**
     * Updates the current segment with the head segment.
     * This is similar to local branches and tracking branches of git.
     *
     * To separate the current and the head is in order to not make useless segments.
     *
     * In this process, both "onCodePathSegmentStart" and "onCodePathSegmentEnd"
     * events are fired.
     * @param {CodePathAnalyzer} analyzer The instance.
     * @param {ASTNode} node The current AST node.
     * @returns {void}
     */
    function forwardCurrentToHead(analyzer, node) {
        const codePath = analyzer.codePath;
        const state = CodePath.getState(codePath);
        const currentSegments = state.currentSegments;
        const headSegments = state.headSegments;
        const end = Math.max(currentSegments.length, headSegments.length);
        let i, currentSegment, headSegment;
        // Fires leaving events.
        for (i = 0; i < end; ++i) {
            currentSegment = currentSegments[i];
            headSegment = headSegments[i];
            if (currentSegment !== headSegment && currentSegment) {
                debug.dump(`onCodePathSegmentEnd ${currentSegment.id}`);
                if (currentSegment.reachable) {
                    analyzer.emitter.emit("onCodePathSegmentEnd", currentSegment, node);
                }
            }
        }
        // Update state.
        state.currentSegments = headSegments;
        // Fires entering events.
        for (i = 0; i < end; ++i) {
            currentSegment = currentSegments[i];
            headSegment = headSegments[i];
            if (currentSegment !== headSegment && headSegment) {
                debug.dump(`onCodePathSegmentStart ${headSegment.id}`);
                CodePathSegment.markUsed(headSegment);
                if (headSegment.reachable) {
                    analyzer.emitter.emit("onCodePathSegmentStart", headSegment, node);
                }
            }
        }
    }
    /**
     * Updates the current segment with empty.
     * This is called at the last of functions or the program.
     * @param {CodePathAnalyzer} analyzer The instance.
     * @param {ASTNode} node The current AST node.
     * @returns {void}
     */
    function leaveFromCurrentSegment(analyzer, node) {
        const state = CodePath.getState(analyzer.codePath);
        const currentSegments = state.currentSegments;
        for (let i = 0; i < currentSegments.length; ++i) {
            const currentSegment = currentSegments[i];
            debug.dump(`onCodePathSegmentEnd ${currentSegment.id}`);
            if (currentSegment.reachable) {
                analyzer.emitter.emit("onCodePathSegmentEnd", currentSegment, node);
            }
        }
        state.currentSegments = [];
    }
    /**
     * Updates the code path due to the position of a given node in the parent node
     * thereof.
     *
     * For example, if the node is `parent.consequent`, this creates a fork from the
     * current path.
     * @param {CodePathAnalyzer} analyzer The instance.
     * @param {ASTNode} node The current AST node.
     * @returns {void}
     */
    function preprocess(analyzer, node) {
        const codePath = analyzer.codePath;
        const state = CodePath.getState(codePath);
        const parent = node.parent;
        switch (parent.type) {
            case "LogicalExpression":
                if (parent.right === node &&
                    isHandledLogicalOperator(parent.operator)) {
                    state.makeLogicalRight();
                }
                break;
            case "ConditionalExpression":
            case "IfStatement":
                /*
                 * Fork if this node is at `consequent`/`alternate`.
                 * `popForkContext()` exists at `IfStatement:exit` and
                 * `ConditionalExpression:exit`.
                 */
                if (parent.consequent === node) {
                    state.makeIfConsequent();
                }
                else if (parent.alternate === node) {
                    state.makeIfAlternate();
                }
                break;
            case "SwitchCase":
                if (parent.consequent[0] === node) {
                    state.makeSwitchCaseBody(false, !parent.test);
                }
                break;
            case "TryStatement":
                if (parent.handler === node) {
                    state.makeCatchBlock();
                }
                else if (parent.finalizer === node) {
                    state.makeFinallyBlock();
                }
                break;
            case "WhileStatement":
                if (parent.test === node) {
                    state.makeWhileTest(getBooleanValueIfSimpleConstant(node));
                }
                else {
                    assert(parent.body === node);
                    state.makeWhileBody();
                }
                break;
            case "DoWhileStatement":
                if (parent.body === node) {
                    state.makeDoWhileBody();
                }
                else {
                    assert(parent.test === node);
                    state.makeDoWhileTest(getBooleanValueIfSimpleConstant(node));
                }
                break;
            case "ForStatement":
                if (parent.test === node) {
                    state.makeForTest(getBooleanValueIfSimpleConstant(node));
                }
                else if (parent.update === node) {
                    state.makeForUpdate();
                }
                else if (parent.body === node) {
                    state.makeForBody();
                }
                break;
            case "ForInStatement":
            case "ForOfStatement":
                if (parent.left === node) {
                    state.makeForInOfLeft();
                }
                else if (parent.right === node) {
                    state.makeForInOfRight();
                }
                else {
                    assert(parent.body === node);
                    state.makeForInOfBody();
                }
                break;
            case "AssignmentPattern":
                /*
                 * Fork if this node is at `right`.
                 * `left` is executed always, so it uses the current path.
                 * `popForkContext()` exists at `AssignmentPattern:exit`.
                 */
                if (parent.right === node) {
                    state.pushForkContext();
                    state.forkBypassPath();
                    state.forkPath();
                }
                break;
            default:
                break;
        }
    }
    /**
     * Updates the code path due to the type of a given node in entering.
     * @param {CodePathAnalyzer} analyzer The instance.
     * @param {ASTNode} node The current AST node.
     * @returns {void}
     */
    function processCodePathToEnter(analyzer, node) {
        let codePath = analyzer.codePath;
        let state = codePath && CodePath.getState(codePath);
        const parent = node.parent;
        switch (node.type) {
            case "Program":
            case "FunctionDeclaration":
            case "FunctionExpression":
            case "ArrowFunctionExpression":
                if (codePath) {
                    // Emits onCodePathSegmentStart events if updated.
                    forwardCurrentToHead(analyzer, node);
                    debug.dumpState(node, state, false);
                }
                // Create the code path of this scope.
                codePath = analyzer.codePath = new CodePath(analyzer.idGenerator.next(), codePath, analyzer.onLooped);
                state = CodePath.getState(codePath);
                // Emits onCodePathStart events.
                debug.dump(`onCodePathStart ${codePath.id}`);
                analyzer.emitter.emit("onCodePathStart", codePath, node);
                break;
            case "LogicalExpression":
                if (isHandledLogicalOperator(node.operator)) {
                    state.pushChoiceContext(node.operator, isForkingByTrueOrFalse(node));
                }
                break;
            case "ConditionalExpression":
            case "IfStatement":
                state.pushChoiceContext("test", false);
                break;
            case "SwitchStatement":
                state.pushSwitchContext(node.cases.some(isCaseNode), getLabel(node));
                break;
            case "TryStatement":
                state.pushTryContext(Boolean(node.finalizer));
                break;
            case "SwitchCase":
                /*
                 * Fork if this node is after the 2st node in `cases`.
                 * It's similar to `else` blocks.
                 * The next `test` node is processed in this path.
                 */
                if (parent.discriminant !== node && parent.cases[0] !== node) {
                    state.forkPath();
                }
                break;
            case "WhileStatement":
            case "DoWhileStatement":
            case "ForStatement":
            case "ForInStatement":
            case "ForOfStatement":
                state.pushLoopContext(node.type, getLabel(node));
                break;
            case "LabeledStatement":
                if (!breakableTypePattern.test(node.body.type)) {
                    state.pushBreakContext(false, node.label.name);
                }
                break;
            default:
                break;
        }
        // Emits onCodePathSegmentStart events if updated.
        forwardCurrentToHead(analyzer, node);
        debug.dumpState(node, state, false);
    }
    /**
     * Updates the code path due to the type of a given node in leaving.
     * @param {CodePathAnalyzer} analyzer The instance.
     * @param {ASTNode} node The current AST node.
     * @returns {void}
     */
    function processCodePathToExit(analyzer, node) {
        const codePath = analyzer.codePath;
        const state = CodePath.getState(codePath);
        let dontForward = false;
        switch (node.type) {
            case "IfStatement":
            case "ConditionalExpression":
                state.popChoiceContext();
                break;
            case "LogicalExpression":
                if (isHandledLogicalOperator(node.operator)) {
                    state.popChoiceContext();
                }
                break;
            case "SwitchStatement":
                state.popSwitchContext();
                break;
            case "SwitchCase":
                /*
                 * This is the same as the process at the 1st `consequent` node in
                 * `preprocess` function.
                 * Must do if this `consequent` is empty.
                 */
                if (node.consequent.length === 0) {
                    state.makeSwitchCaseBody(true, !node.test);
                }
                if (state.forkContext.reachable) {
                    dontForward = true;
                }
                break;
            case "TryStatement":
                state.popTryContext();
                break;
            case "BreakStatement":
                forwardCurrentToHead(analyzer, node);
                state.makeBreak(node.label && node.label.name);
                dontForward = true;
                break;
            case "ContinueStatement":
                forwardCurrentToHead(analyzer, node);
                state.makeContinue(node.label && node.label.name);
                dontForward = true;
                break;
            case "ReturnStatement":
                forwardCurrentToHead(analyzer, node);
                state.makeReturn();
                dontForward = true;
                break;
            case "ThrowStatement":
                forwardCurrentToHead(analyzer, node);
                state.makeThrow();
                dontForward = true;
                break;
            case "Identifier":
                if (isIdentifierReference(node)) {
                    state.makeFirstThrowablePathInTryBlock();
                    dontForward = true;
                }
                break;
            case "CallExpression":
            case "ImportExpression":
            case "MemberExpression":
            case "NewExpression":
                state.makeFirstThrowablePathInTryBlock();
                break;
            case "WhileStatement":
            case "DoWhileStatement":
            case "ForStatement":
            case "ForInStatement":
            case "ForOfStatement":
                state.popLoopContext();
                break;
            case "AssignmentPattern":
                state.popForkContext();
                break;
            case "LabeledStatement":
                if (!breakableTypePattern.test(node.body.type)) {
                    state.popBreakContext();
                }
                break;
            default:
                break;
        }
        // Emits onCodePathSegmentStart events if updated.
        if (!dontForward) {
            forwardCurrentToHead(analyzer, node);
        }
        debug.dumpState(node, state, true);
    }
    /**
     * Updates the code path to finalize the current code path.
     * @param {CodePathAnalyzer} analyzer The instance.
     * @param {ASTNode} node The current AST node.
     * @returns {void}
     */
    function postprocess(analyzer, node) {
        switch (node.type) {
            case "Program":
            case "FunctionDeclaration":
            case "FunctionExpression":
            case "ArrowFunctionExpression": {
                let codePath = analyzer.codePath;
                // Mark the current path as the final node.
                CodePath.getState(codePath).makeFinal();
                // Emits onCodePathSegmentEnd event of the current segments.
                leaveFromCurrentSegment(analyzer, node);
                // Emits onCodePathEnd event of this code path.
                debug.dump(`onCodePathEnd ${codePath.id}`);
                analyzer.emitter.emit("onCodePathEnd", codePath, node);
                debug.dumpDot(codePath);
                codePath = analyzer.codePath = analyzer.codePath.upper;
                if (codePath) {
                    debug.dumpState(node, CodePath.getState(codePath), true);
                }
                break;
            }
            default:
                break;
        }
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * The class to analyze code paths.
     * This class implements the EventGenerator interface.
     */
    class CodePathAnalyzer {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {EventGenerator} eventGenerator An event generator to wrap.
         */
        constructor(eventGenerator) {
            this.original = eventGenerator;
            this.emitter = eventGenerator.emitter;
            this.codePath = null;
            this.idGenerator = new IdGenerator("s");
            this.currentNode = null;
            this.onLooped = this.onLooped.bind(this);
        }
        /**
         * Does the process to enter a given AST node.
         * This updates state of analysis and calls `enterNode` of the wrapped.
         * @param {ASTNode} node A node which is entering.
         * @returns {void}
         */
        enterNode(node) {
            this.currentNode = node;
            // Updates the code path due to node's position in its parent node.
            if (node.parent) {
                preprocess(this, node);
            }
            /*
             * Updates the code path.
             * And emits onCodePathStart/onCodePathSegmentStart events.
             */
            processCodePathToEnter(this, node);
            // Emits node events.
            this.original.enterNode(node);
            this.currentNode = null;
        }
        /**
         * Does the process to leave a given AST node.
         * This updates state of analysis and calls `leaveNode` of the wrapped.
         * @param {ASTNode} node A node which is leaving.
         * @returns {void}
         */
        leaveNode(node) {
            this.currentNode = node;
            /*
             * Updates the code path.
             * And emits onCodePathStart/onCodePathSegmentStart events.
             */
            processCodePathToExit(this, node);
            // Emits node events.
            this.original.leaveNode(node);
            // Emits the last onCodePathStart/onCodePathSegmentStart events.
            postprocess(this, node);
            this.currentNode = null;
        }
        /**
         * This is called on a code path looped.
         * Then this raises a looped event.
         * @param {CodePathSegment} fromSegment A segment of prev.
         * @param {CodePathSegment} toSegment A segment of next.
         * @returns {void}
         */
        onLooped(fromSegment, toSegment) {
            if (fromSegment.reachable && toSegment.reachable) {
                debug.dump(`onCodePathSegmentLoop ${fromSegment.id} -> ${toSegment.id}`);
                this.emitter.emit("onCodePathSegmentLoop", fromSegment, toSegment, this.currentNode);
            }
        }
    }
    module.exports = CodePathAnalyzer;
},
/* eslint/lib/linter/code-path-analysis/code-path.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const CodePathState = require(488) /* ./code-path-state */;
    const IdGenerator = require(492) /* ./id-generator */;
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * A code path.
     */
    class CodePath {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {string} id An identifier.
         * @param {CodePath|null} upper The code path of the upper function scope.
         * @param {Function} onLooped A callback function to notify looping.
         */
        constructor(id, upper, onLooped) {
            /**
             * The identifier of this code path.
             * Rules use it to store additional information of each rule.
             * @type {string}
             */
            this.id = id;
            /**
             * The code path of the upper function scope.
             * @type {CodePath|null}
             */
            this.upper = upper;
            /**
             * The code paths of nested function scopes.
             * @type {CodePath[]}
             */
            this.childCodePaths = [];
            // Initializes internal state.
            Object.defineProperty(this, "internal", { value: new CodePathState(new IdGenerator(`${id}_`), onLooped) });
            // Adds this into `childCodePaths` of `upper`.
            if (upper) {
                upper.childCodePaths.push(this);
            }
        }
        /**
         * Gets the state of a given code path.
         * @param {CodePath} codePath A code path to get.
         * @returns {CodePathState} The state of the code path.
         */
        static getState(codePath) {
            return codePath.internal;
        }
        /**
         * The initial code path segment.
         * @type {CodePathSegment}
         */
        get initialSegment() {
            return this.internal.initialSegment;
        }
        /**
         * Final code path segments.
         * This array is a mix of `returnedSegments` and `thrownSegments`.
         * @type {CodePathSegment[]}
         */
        get finalSegments() {
            return this.internal.finalSegments;
        }
        /**
         * Final code path segments which is with `return` statements.
         * This array contains the last path segment if it's reachable.
         * Since the reachable last path returns `undefined`.
         * @type {CodePathSegment[]}
         */
        get returnedSegments() {
            return this.internal.returnedForkContext;
        }
        /**
         * Final code path segments which is with `throw` statements.
         * @type {CodePathSegment[]}
         */
        get thrownSegments() {
            return this.internal.thrownForkContext;
        }
        /**
         * Current code path segments.
         * @type {CodePathSegment[]}
         */
        get currentSegments() {
            return this.internal.currentSegments;
        }
        /**
         * Traverses all segments in this code path.
         *
         *     codePath.traverseSegments(function(segment, controller) {
         *         // do something.
         *     });
         *
         * This method enumerates segments in order from the head.
         *
         * The `controller` object has two methods.
         *
         * - `controller.skip()` - Skip the following segments in this branch.
         * - `controller.break()` - Skip all following segments.
         * @param {Object} [options] Omittable.
         * @param {CodePathSegment} [options.first] The first segment to traverse.
         * @param {CodePathSegment} [options.last] The last segment to traverse.
         * @param {Function} callback A callback function.
         * @returns {void}
         */
        traverseSegments(options, callback) {
            let resolvedOptions;
            let resolvedCallback;
            if (typeof options === "function") {
                resolvedCallback = options;
                resolvedOptions = {};
            }
            else {
                resolvedOptions = options || {};
                resolvedCallback = callback;
            }
            const startSegment = resolvedOptions.first || this.internal.initialSegment;
            const lastSegment = resolvedOptions.last;
            let item = null;
            let index = 0;
            let end = 0;
            let segment = null;
            const visited = Object.create(null);
            const stack = [[startSegment, 0]];
            let skippedSegment = null;
            let broken = false;
            const controller = {
                skip() {
                    if (stack.length <= 1) {
                        broken = true;
                    }
                    else {
                        skippedSegment = stack[stack.length - 2][0];
                    }
                },
                break() {
                    broken = true;
                }
            };
            /**
             * Checks a given previous segment has been visited.
             * @param {CodePathSegment} prevSegment A previous segment to check.
             * @returns {boolean} `true` if the segment has been visited.
             */
            function isVisited(prevSegment) {
                return (visited[prevSegment.id] ||
                    segment.isLoopedPrevSegment(prevSegment));
            }
            while (stack.length > 0) {
                item = stack[stack.length - 1];
                segment = item[0];
                index = item[1];
                if (index === 0) {
                    // Skip if this segment has been visited already.
                    if (visited[segment.id]) {
                        stack.pop();
                        continue;
                    }
                    // Skip if all previous segments have not been visited.
                    if (segment !== startSegment &&
                        segment.prevSegments.length > 0 &&
                        !segment.prevSegments.every(isVisited)) {
                        stack.pop();
                        continue;
                    }
                    // Reset the flag of skipping if all branches have been skipped.
                    if (skippedSegment && segment.prevSegments.indexOf(skippedSegment) !== -1) {
                        skippedSegment = null;
                    }
                    visited[segment.id] = true;
                    // Call the callback when the first time.
                    if (!skippedSegment) {
                        resolvedCallback.call(this, segment, controller);
                        if (segment === lastSegment) {
                            controller.skip();
                        }
                        if (broken) {
                            break;
                        }
                    }
                }
                // Update the stack.
                end = segment.nextSegments.length - 1;
                if (index < end) {
                    item[1] += 1;
                    stack.push([segment.nextSegments[index], 0]);
                }
                else if (index === end) {
                    item[0] = segment.nextSegments[index];
                    item[1] = 0;
                }
                else {
                    stack.pop();
                }
            }
        }
    }
    module.exports = CodePath;
},
/* eslint/lib/linter/code-path-analysis/code-path-state.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const CodePathSegment = require(489) /* ./code-path-segment */, ForkContext = require(491) /* ./fork-context */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Adds given segments into the `dest` array.
     * If the `others` array does not includes the given segments, adds to the `all`
     * array as well.
     *
     * This adds only reachable and used segments.
     * @param {CodePathSegment[]} dest A destination array (`returnedSegments` or `thrownSegments`).
     * @param {CodePathSegment[]} others Another destination array (`returnedSegments` or `thrownSegments`).
     * @param {CodePathSegment[]} all The unified destination array (`finalSegments`).
     * @param {CodePathSegment[]} segments Segments to add.
     * @returns {void}
     */
    function addToReturnedOrThrown(dest, others, all, segments) {
        for (let i = 0; i < segments.length; ++i) {
            const segment = segments[i];
            dest.push(segment);
            if (others.indexOf(segment) === -1) {
                all.push(segment);
            }
        }
    }
    /**
     * Gets a loop-context for a `continue` statement.
     * @param {CodePathState} state A state to get.
     * @param {string} label The label of a `continue` statement.
     * @returns {LoopContext} A loop-context for a `continue` statement.
     */
    function getContinueContext(state, label) {
        if (!label) {
            return state.loopContext;
        }
        let context = state.loopContext;
        while (context) {
            if (context.label === label) {
                return context;
            }
            context = context.upper;
        }
        /* istanbul ignore next: foolproof (syntax error) */
        return null;
    }
    /**
     * Gets a context for a `break` statement.
     * @param {CodePathState} state A state to get.
     * @param {string} label The label of a `break` statement.
     * @returns {LoopContext|SwitchContext} A context for a `break` statement.
     */
    function getBreakContext(state, label) {
        let context = state.breakContext;
        while (context) {
            if (label ? context.label === label : context.breakable) {
                return context;
            }
            context = context.upper;
        }
        /* istanbul ignore next: foolproof (syntax error) */
        return null;
    }
    /**
     * Gets a context for a `return` statement.
     * @param {CodePathState} state A state to get.
     * @returns {TryContext|CodePathState} A context for a `return` statement.
     */
    function getReturnContext(state) {
        let context = state.tryContext;
        while (context) {
            if (context.hasFinalizer && context.position !== "finally") {
                return context;
            }
            context = context.upper;
        }
        return state;
    }
    /**
     * Gets a context for a `throw` statement.
     * @param {CodePathState} state A state to get.
     * @returns {TryContext|CodePathState} A context for a `throw` statement.
     */
    function getThrowContext(state) {
        let context = state.tryContext;
        while (context) {
            if (context.position === "try" ||
                (context.hasFinalizer && context.position === "catch")) {
                return context;
            }
            context = context.upper;
        }
        return state;
    }
    /**
     * Removes a given element from a given array.
     * @param {any[]} xs An array to remove the specific element.
     * @param {any} x An element to be removed.
     * @returns {void}
     */
    function remove(xs, x) {
        xs.splice(xs.indexOf(x), 1);
    }
    /**
     * Disconnect given segments.
     *
     * This is used in a process for switch statements.
     * If there is the "default" chunk before other cases, the order is different
     * between node's and running's.
     * @param {CodePathSegment[]} prevSegments Forward segments to disconnect.
     * @param {CodePathSegment[]} nextSegments Backward segments to disconnect.
     * @returns {void}
     */
    function removeConnection(prevSegments, nextSegments) {
        for (let i = 0; i < prevSegments.length; ++i) {
            const prevSegment = prevSegments[i];
            const nextSegment = nextSegments[i];
            remove(prevSegment.nextSegments, nextSegment);
            remove(prevSegment.allNextSegments, nextSegment);
            remove(nextSegment.prevSegments, prevSegment);
            remove(nextSegment.allPrevSegments, prevSegment);
        }
    }
    /**
     * Creates looping path.
     * @param {CodePathState} state The instance.
     * @param {CodePathSegment[]} unflattenedFromSegments Segments which are source.
     * @param {CodePathSegment[]} unflattenedToSegments Segments which are destination.
     * @returns {void}
     */
    function makeLooped(state, unflattenedFromSegments, unflattenedToSegments) {
        const fromSegments = CodePathSegment.flattenUnusedSegments(unflattenedFromSegments);
        const toSegments = CodePathSegment.flattenUnusedSegments(unflattenedToSegments);
        const end = Math.min(fromSegments.length, toSegments.length);
        for (let i = 0; i < end; ++i) {
            const fromSegment = fromSegments[i];
            const toSegment = toSegments[i];
            if (toSegment.reachable) {
                fromSegment.nextSegments.push(toSegment);
            }
            if (fromSegment.reachable) {
                toSegment.prevSegments.push(fromSegment);
            }
            fromSegment.allNextSegments.push(toSegment);
            toSegment.allPrevSegments.push(fromSegment);
            if (toSegment.allPrevSegments.length >= 2) {
                CodePathSegment.markPrevSegmentAsLooped(toSegment, fromSegment);
            }
            state.notifyLooped(fromSegment, toSegment);
        }
    }
    /**
     * Finalizes segments of `test` chunk of a ForStatement.
     *
     * - Adds `false` paths to paths which are leaving from the loop.
     * - Sets `true` paths to paths which go to the body.
     * @param {LoopContext} context A loop context to modify.
     * @param {ChoiceContext} choiceContext A choice context of this loop.
     * @param {CodePathSegment[]} head The current head paths.
     * @returns {void}
     */
    function finalizeTestSegmentsOfFor(context, choiceContext, head) {
        if (!choiceContext.processed) {
            choiceContext.trueForkContext.add(head);
            choiceContext.falseForkContext.add(head);
        }
        if (context.test !== true) {
            context.brokenForkContext.addAll(choiceContext.falseForkContext);
        }
        context.endOfTestSegments = choiceContext.trueForkContext.makeNext(0, -1);
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * A class which manages state to analyze code paths.
     */
    class CodePathState {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {IdGenerator} idGenerator An id generator to generate id for code
         *   path segments.
         * @param {Function} onLooped A callback function to notify looping.
         */
        constructor(idGenerator, onLooped) {
            this.idGenerator = idGenerator;
            this.notifyLooped = onLooped;
            this.forkContext = ForkContext.newRoot(idGenerator);
            this.choiceContext = null;
            this.switchContext = null;
            this.tryContext = null;
            this.loopContext = null;
            this.breakContext = null;
            this.currentSegments = [];
            this.initialSegment = this.forkContext.head[0];
            // returnedSegments and thrownSegments push elements into finalSegments also.
            const final = this.finalSegments = [];
            const returned = this.returnedForkContext = [];
            const thrown = this.thrownForkContext = [];
            returned.add = addToReturnedOrThrown.bind(null, returned, thrown, final);
            thrown.add = addToReturnedOrThrown.bind(null, thrown, returned, final);
        }
        /**
         * The head segments.
         * @type {CodePathSegment[]}
         */
        get headSegments() {
            return this.forkContext.head;
        }
        /**
         * The parent forking context.
         * This is used for the root of new forks.
         * @type {ForkContext}
         */
        get parentForkContext() {
            const current = this.forkContext;
            return current && current.upper;
        }
        /**
         * Creates and stacks new forking context.
         * @param {boolean} forkLeavingPath A flag which shows being in a
         *   "finally" block.
         * @returns {ForkContext} The created context.
         */
        pushForkContext(forkLeavingPath) {
            this.forkContext = ForkContext.newEmpty(this.forkContext, forkLeavingPath);
            return this.forkContext;
        }
        /**
         * Pops and merges the last forking context.
         * @returns {ForkContext} The last context.
         */
        popForkContext() {
            const lastContext = this.forkContext;
            this.forkContext = lastContext.upper;
            this.forkContext.replaceHead(lastContext.makeNext(0, -1));
            return lastContext;
        }
        /**
         * Creates a new path.
         * @returns {void}
         */
        forkPath() {
            this.forkContext.add(this.parentForkContext.makeNext(-1, -1));
        }
        /**
         * Creates a bypass path.
         * This is used for such as IfStatement which does not have "else" chunk.
         * @returns {void}
         */
        forkBypassPath() {
            this.forkContext.add(this.parentForkContext.head);
        }
        //--------------------------------------------------------------------------
        // ConditionalExpression, LogicalExpression, IfStatement
        //--------------------------------------------------------------------------
        /**
         * Creates a context for ConditionalExpression, LogicalExpression,
         * IfStatement, WhileStatement, DoWhileStatement, or ForStatement.
         *
         * LogicalExpressions have cases that it goes different paths between the
         * `true` case and the `false` case.
         *
         * For Example:
         *
         *     if (a || b) {
         *         foo();
         *     } else {
         *         bar();
         *     }
         *
         * In this case, `b` is evaluated always in the code path of the `else`
         * block, but it's not so in the code path of the `if` block.
         * So there are 3 paths.
         *
         *     a -> foo();
         *     a -> b -> foo();
         *     a -> b -> bar();
         * @param {string} kind A kind string.
         *   If the new context is LogicalExpression's, this is `"&&"` or `"||"`.
         *   If it's IfStatement's or ConditionalExpression's, this is `"test"`.
         *   Otherwise, this is `"loop"`.
         * @param {boolean} isForkingAsResult A flag that shows that goes different
         *   paths between `true` and `false`.
         * @returns {void}
         */
        pushChoiceContext(kind, isForkingAsResult) {
            this.choiceContext = {
                upper: this.choiceContext,
                kind,
                isForkingAsResult,
                trueForkContext: ForkContext.newEmpty(this.forkContext),
                falseForkContext: ForkContext.newEmpty(this.forkContext),
                processed: false
            };
        }
        /**
         * Pops the last choice context and finalizes it.
         * @returns {ChoiceContext} The popped context.
         */
        popChoiceContext() {
            const context = this.choiceContext;
            this.choiceContext = context.upper;
            const forkContext = this.forkContext;
            const headSegments = forkContext.head;
            switch (context.kind) {
                case "&&":
                case "||":
                    /*
                     * If any result were not transferred from child contexts,
                     * this sets the head segments to both cases.
                     * The head segments are the path of the right-hand operand.
                     */
                    if (!context.processed) {
                        context.trueForkContext.add(headSegments);
                        context.falseForkContext.add(headSegments);
                    }
                    /*
                     * Transfers results to upper context if this context is in
                     * test chunk.
                     */
                    if (context.isForkingAsResult) {
                        const parentContext = this.choiceContext;
                        parentContext.trueForkContext.addAll(context.trueForkContext);
                        parentContext.falseForkContext.addAll(context.falseForkContext);
                        parentContext.processed = true;
                        return context;
                    }
                    break;
                case "test":
                    if (!context.processed) {
                        /*
                         * The head segments are the path of the `if` block here.
                         * Updates the `true` path with the end of the `if` block.
                         */
                        context.trueForkContext.clear();
                        context.trueForkContext.add(headSegments);
                    }
                    else {
                        /*
                         * The head segments are the path of the `else` block here.
                         * Updates the `false` path with the end of the `else`
                         * block.
                         */
                        context.falseForkContext.clear();
                        context.falseForkContext.add(headSegments);
                    }
                    break;
                case "loop":
                    /*
                     * Loops are addressed in popLoopContext().
                     * This is called from popLoopContext().
                     */
                    return context;
                /* istanbul ignore next */
                default:
                    throw new Error("unreachable");
            }
            // Merges all paths.
            const prevForkContext = context.trueForkContext;
            prevForkContext.addAll(context.falseForkContext);
            forkContext.replaceHead(prevForkContext.makeNext(0, -1));
            return context;
        }
        /**
         * Makes a code path segment of the right-hand operand of a logical
         * expression.
         * @returns {void}
         */
        makeLogicalRight() {
            const context = this.choiceContext;
            const forkContext = this.forkContext;
            if (context.processed) {
                /*
                 * This got segments already from the child choice context.
                 * Creates the next path from own true/false fork context.
                 */
                const prevForkContext = context.kind === "&&" ? context.trueForkContext
                    /* kind === "||" */ : context.falseForkContext;
                forkContext.replaceHead(prevForkContext.makeNext(0, -1));
                prevForkContext.clear();
                context.processed = false;
            }
            else {
                /*
                 * This did not get segments from the child choice context.
                 * So addresses the head segments.
                 * The head segments are the path of the left-hand operand.
                 */
                if (context.kind === "&&") {
                    // The path does short-circuit if false.
                    context.falseForkContext.add(forkContext.head);
                }
                else {
                    // The path does short-circuit if true.
                    context.trueForkContext.add(forkContext.head);
                }
                forkContext.replaceHead(forkContext.makeNext(-1, -1));
            }
        }
        /**
         * Makes a code path segment of the `if` block.
         * @returns {void}
         */
        makeIfConsequent() {
            const context = this.choiceContext;
            const forkContext = this.forkContext;
            /*
             * If any result were not transferred from child contexts,
             * this sets the head segments to both cases.
             * The head segments are the path of the test expression.
             */
            if (!context.processed) {
                context.trueForkContext.add(forkContext.head);
                context.falseForkContext.add(forkContext.head);
            }
            context.processed = false;
            // Creates new path from the `true` case.
            forkContext.replaceHead(context.trueForkContext.makeNext(0, -1));
        }
        /**
         * Makes a code path segment of the `else` block.
         * @returns {void}
         */
        makeIfAlternate() {
            const context = this.choiceContext;
            const forkContext = this.forkContext;
            /*
             * The head segments are the path of the `if` block.
             * Updates the `true` path with the end of the `if` block.
             */
            context.trueForkContext.clear();
            context.trueForkContext.add(forkContext.head);
            context.processed = true;
            // Creates new path from the `false` case.
            forkContext.replaceHead(context.falseForkContext.makeNext(0, -1));
        }
        //--------------------------------------------------------------------------
        // SwitchStatement
        //--------------------------------------------------------------------------
        /**
         * Creates a context object of SwitchStatement and stacks it.
         * @param {boolean} hasCase `true` if the switch statement has one or more
         *   case parts.
         * @param {string|null} label The label text.
         * @returns {void}
         */
        pushSwitchContext(hasCase, label) {
            this.switchContext = {
                upper: this.switchContext,
                hasCase,
                defaultSegments: null,
                defaultBodySegments: null,
                foundDefault: false,
                lastIsDefault: false,
                countForks: 0
            };
            this.pushBreakContext(true, label);
        }
        /**
         * Pops the last context of SwitchStatement and finalizes it.
         *
         * - Disposes all forking stack for `case` and `default`.
         * - Creates the next code path segment from `context.brokenForkContext`.
         * - If the last `SwitchCase` node is not a `default` part, creates a path
         *   to the `default` body.
         * @returns {void}
         */
        popSwitchContext() {
            const context = this.switchContext;
            this.switchContext = context.upper;
            const forkContext = this.forkContext;
            const brokenForkContext = this.popBreakContext().brokenForkContext;
            if (context.countForks === 0) {
                /*
                 * When there is only one `default` chunk and there is one or more
                 * `break` statements, even if forks are nothing, it needs to merge
                 * those.
                 */
                if (!brokenForkContext.empty) {
                    brokenForkContext.add(forkContext.makeNext(-1, -1));
                    forkContext.replaceHead(brokenForkContext.makeNext(0, -1));
                }
                return;
            }
            const lastSegments = forkContext.head;
            this.forkBypassPath();
            const lastCaseSegments = forkContext.head;
            /*
             * `brokenForkContext` is used to make the next segment.
             * It must add the last segment into `brokenForkContext`.
             */
            brokenForkContext.add(lastSegments);
            /*
             * A path which is failed in all case test should be connected to path
             * of `default` chunk.
             */
            if (!context.lastIsDefault) {
                if (context.defaultBodySegments) {
                    /*
                     * Remove a link from `default` label to its chunk.
                     * It's false route.
                     */
                    removeConnection(context.defaultSegments, context.defaultBodySegments);
                    makeLooped(this, lastCaseSegments, context.defaultBodySegments);
                }
                else {
                    /*
                     * It handles the last case body as broken if `default` chunk
                     * does not exist.
                     */
                    brokenForkContext.add(lastCaseSegments);
                }
            }
            // Pops the segment context stack until the entry segment.
            for (let i = 0; i < context.countForks; ++i) {
                this.forkContext = this.forkContext.upper;
            }
            /*
             * Creates a path from all brokenForkContext paths.
             * This is a path after switch statement.
             */
            this.forkContext.replaceHead(brokenForkContext.makeNext(0, -1));
        }
        /**
         * Makes a code path segment for a `SwitchCase` node.
         * @param {boolean} isEmpty `true` if the body is empty.
         * @param {boolean} isDefault `true` if the body is the default case.
         * @returns {void}
         */
        makeSwitchCaseBody(isEmpty, isDefault) {
            const context = this.switchContext;
            if (!context.hasCase) {
                return;
            }
            /*
             * Merge forks.
             * The parent fork context has two segments.
             * Those are from the current case and the body of the previous case.
             */
            const parentForkContext = this.forkContext;
            const forkContext = this.pushForkContext();
            forkContext.add(parentForkContext.makeNext(0, -1));
            /*
             * Save `default` chunk info.
             * If the `default` label is not at the last, we must make a path from
             * the last `case` to the `default` chunk.
             */
            if (isDefault) {
                context.defaultSegments = parentForkContext.head;
                if (isEmpty) {
                    context.foundDefault = true;
                }
                else {
                    context.defaultBodySegments = forkContext.head;
                }
            }
            else {
                if (!isEmpty && context.foundDefault) {
                    context.foundDefault = false;
                    context.defaultBodySegments = forkContext.head;
                }
            }
            context.lastIsDefault = isDefault;
            context.countForks += 1;
        }
        //--------------------------------------------------------------------------
        // TryStatement
        //--------------------------------------------------------------------------
        /**
         * Creates a context object of TryStatement and stacks it.
         * @param {boolean} hasFinalizer `true` if the try statement has a
         *   `finally` block.
         * @returns {void}
         */
        pushTryContext(hasFinalizer) {
            this.tryContext = {
                upper: this.tryContext,
                position: "try",
                hasFinalizer,
                returnedForkContext: hasFinalizer
                    ? ForkContext.newEmpty(this.forkContext)
                    : null,
                thrownForkContext: ForkContext.newEmpty(this.forkContext),
                lastOfTryIsReachable: false,
                lastOfCatchIsReachable: false
            };
        }
        /**
         * Pops the last context of TryStatement and finalizes it.
         * @returns {void}
         */
        popTryContext() {
            const context = this.tryContext;
            this.tryContext = context.upper;
            if (context.position === "catch") {
                // Merges two paths from the `try` block and `catch` block merely.
                this.popForkContext();
                return;
            }
            /*
             * The following process is executed only when there is the `finally`
             * block.
             */
            const returned = context.returnedForkContext;
            const thrown = context.thrownForkContext;
            if (returned.empty && thrown.empty) {
                return;
            }
            // Separate head to normal paths and leaving paths.
            const headSegments = this.forkContext.head;
            this.forkContext = this.forkContext.upper;
            const normalSegments = headSegments.slice(0, headSegments.length / 2 | 0);
            const leavingSegments = headSegments.slice(headSegments.length / 2 | 0);
            // Forwards the leaving path to upper contexts.
            if (!returned.empty) {
                getReturnContext(this).returnedForkContext.add(leavingSegments);
            }
            if (!thrown.empty) {
                getThrowContext(this).thrownForkContext.add(leavingSegments);
            }
            // Sets the normal path as the next.
            this.forkContext.replaceHead(normalSegments);
            /*
             * If both paths of the `try` block and the `catch` block are
             * unreachable, the next path becomes unreachable as well.
             */
            if (!context.lastOfTryIsReachable && !context.lastOfCatchIsReachable) {
                this.forkContext.makeUnreachable();
            }
        }
        /**
         * Makes a code path segment for a `catch` block.
         * @returns {void}
         */
        makeCatchBlock() {
            const context = this.tryContext;
            const forkContext = this.forkContext;
            const thrown = context.thrownForkContext;
            // Update state.
            context.position = "catch";
            context.thrownForkContext = ForkContext.newEmpty(forkContext);
            context.lastOfTryIsReachable = forkContext.reachable;
            // Merge thrown paths.
            thrown.add(forkContext.head);
            const thrownSegments = thrown.makeNext(0, -1);
            // Fork to a bypass and the merged thrown path.
            this.pushForkContext();
            this.forkBypassPath();
            this.forkContext.add(thrownSegments);
        }
        /**
         * Makes a code path segment for a `finally` block.
         *
         * In the `finally` block, parallel paths are created. The parallel paths
         * are used as leaving-paths. The leaving-paths are paths from `return`
         * statements and `throw` statements in a `try` block or a `catch` block.
         * @returns {void}
         */
        makeFinallyBlock() {
            const context = this.tryContext;
            let forkContext = this.forkContext;
            const returned = context.returnedForkContext;
            const thrown = context.thrownForkContext;
            const headOfLeavingSegments = forkContext.head;
            // Update state.
            if (context.position === "catch") {
                // Merges two paths from the `try` block and `catch` block.
                this.popForkContext();
                forkContext = this.forkContext;
                context.lastOfCatchIsReachable = forkContext.reachable;
            }
            else {
                context.lastOfTryIsReachable = forkContext.reachable;
            }
            context.position = "finally";
            if (returned.empty && thrown.empty) {
                // This path does not leave.
                return;
            }
            /*
             * Create a parallel segment from merging returned and thrown.
             * This segment will leave at the end of this finally block.
             */
            const segments = forkContext.makeNext(-1, -1);
            for (let i = 0; i < forkContext.count; ++i) {
                const prevSegsOfLeavingSegment = [headOfLeavingSegments[i]];
                for (let j = 0; j < returned.segmentsList.length; ++j) {
                    prevSegsOfLeavingSegment.push(returned.segmentsList[j][i]);
                }
                for (let j = 0; j < thrown.segmentsList.length; ++j) {
                    prevSegsOfLeavingSegment.push(thrown.segmentsList[j][i]);
                }
                segments.push(CodePathSegment.newNext(this.idGenerator.next(), prevSegsOfLeavingSegment));
            }
            this.pushForkContext(true);
            this.forkContext.add(segments);
        }
        /**
         * Makes a code path segment from the first throwable node to the `catch`
         * block or the `finally` block.
         * @returns {void}
         */
        makeFirstThrowablePathInTryBlock() {
            const forkContext = this.forkContext;
            if (!forkContext.reachable) {
                return;
            }
            const context = getThrowContext(this);
            if (context === this ||
                context.position !== "try" ||
                !context.thrownForkContext.empty) {
                return;
            }
            context.thrownForkContext.add(forkContext.head);
            forkContext.replaceHead(forkContext.makeNext(-1, -1));
        }
        //--------------------------------------------------------------------------
        // Loop Statements
        //--------------------------------------------------------------------------
        /**
         * Creates a context object of a loop statement and stacks it.
         * @param {string} type The type of the node which was triggered. One of
         *   `WhileStatement`, `DoWhileStatement`, `ForStatement`, `ForInStatement`,
         *   and `ForStatement`.
         * @param {string|null} label A label of the node which was triggered.
         * @returns {void}
         */
        pushLoopContext(type, label) {
            const forkContext = this.forkContext;
            const breakContext = this.pushBreakContext(true, label);
            switch (type) {
                case "WhileStatement":
                    this.pushChoiceContext("loop", false);
                    this.loopContext = {
                        upper: this.loopContext,
                        type,
                        label,
                        test: void 0,
                        continueDestSegments: null,
                        brokenForkContext: breakContext.brokenForkContext
                    };
                    break;
                case "DoWhileStatement":
                    this.pushChoiceContext("loop", false);
                    this.loopContext = {
                        upper: this.loopContext,
                        type,
                        label,
                        test: void 0,
                        entrySegments: null,
                        continueForkContext: ForkContext.newEmpty(forkContext),
                        brokenForkContext: breakContext.brokenForkContext
                    };
                    break;
                case "ForStatement":
                    this.pushChoiceContext("loop", false);
                    this.loopContext = {
                        upper: this.loopContext,
                        type,
                        label,
                        test: void 0,
                        endOfInitSegments: null,
                        testSegments: null,
                        endOfTestSegments: null,
                        updateSegments: null,
                        endOfUpdateSegments: null,
                        continueDestSegments: null,
                        brokenForkContext: breakContext.brokenForkContext
                    };
                    break;
                case "ForInStatement":
                case "ForOfStatement":
                    this.loopContext = {
                        upper: this.loopContext,
                        type,
                        label,
                        prevSegments: null,
                        leftSegments: null,
                        endOfLeftSegments: null,
                        continueDestSegments: null,
                        brokenForkContext: breakContext.brokenForkContext
                    };
                    break;
                /* istanbul ignore next */
                default:
                    throw new Error(`unknown type: "${type}"`);
            }
        }
        /**
         * Pops the last context of a loop statement and finalizes it.
         * @returns {void}
         */
        popLoopContext() {
            const context = this.loopContext;
            this.loopContext = context.upper;
            const forkContext = this.forkContext;
            const brokenForkContext = this.popBreakContext().brokenForkContext;
            // Creates a looped path.
            switch (context.type) {
                case "WhileStatement":
                case "ForStatement":
                    this.popChoiceContext();
                    makeLooped(this, forkContext.head, context.continueDestSegments);
                    break;
                case "DoWhileStatement": {
                    const choiceContext = this.popChoiceContext();
                    if (!choiceContext.processed) {
                        choiceContext.trueForkContext.add(forkContext.head);
                        choiceContext.falseForkContext.add(forkContext.head);
                    }
                    if (context.test !== true) {
                        brokenForkContext.addAll(choiceContext.falseForkContext);
                    }
                    // `true` paths go to looping.
                    const segmentsList = choiceContext.trueForkContext.segmentsList;
                    for (let i = 0; i < segmentsList.length; ++i) {
                        makeLooped(this, segmentsList[i], context.entrySegments);
                    }
                    break;
                }
                case "ForInStatement":
                case "ForOfStatement":
                    brokenForkContext.add(forkContext.head);
                    makeLooped(this, forkContext.head, context.leftSegments);
                    break;
                /* istanbul ignore next */
                default:
                    throw new Error("unreachable");
            }
            // Go next.
            if (brokenForkContext.empty) {
                forkContext.replaceHead(forkContext.makeUnreachable(-1, -1));
            }
            else {
                forkContext.replaceHead(brokenForkContext.makeNext(0, -1));
            }
        }
        /**
         * Makes a code path segment for the test part of a WhileStatement.
         * @param {boolean|undefined} test The test value (only when constant).
         * @returns {void}
         */
        makeWhileTest(test) {
            const context = this.loopContext;
            const forkContext = this.forkContext;
            const testSegments = forkContext.makeNext(0, -1);
            // Update state.
            context.test = test;
            context.continueDestSegments = testSegments;
            forkContext.replaceHead(testSegments);
        }
        /**
         * Makes a code path segment for the body part of a WhileStatement.
         * @returns {void}
         */
        makeWhileBody() {
            const context = this.loopContext;
            const choiceContext = this.choiceContext;
            const forkContext = this.forkContext;
            if (!choiceContext.processed) {
                choiceContext.trueForkContext.add(forkContext.head);
                choiceContext.falseForkContext.add(forkContext.head);
            }
            // Update state.
            if (context.test !== true) {
                context.brokenForkContext.addAll(choiceContext.falseForkContext);
            }
            forkContext.replaceHead(choiceContext.trueForkContext.makeNext(0, -1));
        }
        /**
         * Makes a code path segment for the body part of a DoWhileStatement.
         * @returns {void}
         */
        makeDoWhileBody() {
            const context = this.loopContext;
            const forkContext = this.forkContext;
            const bodySegments = forkContext.makeNext(-1, -1);
            // Update state.
            context.entrySegments = bodySegments;
            forkContext.replaceHead(bodySegments);
        }
        /**
         * Makes a code path segment for the test part of a DoWhileStatement.
         * @param {boolean|undefined} test The test value (only when constant).
         * @returns {void}
         */
        makeDoWhileTest(test) {
            const context = this.loopContext;
            const forkContext = this.forkContext;
            context.test = test;
            // Creates paths of `continue` statements.
            if (!context.continueForkContext.empty) {
                context.continueForkContext.add(forkContext.head);
                const testSegments = context.continueForkContext.makeNext(0, -1);
                forkContext.replaceHead(testSegments);
            }
        }
        /**
         * Makes a code path segment for the test part of a ForStatement.
         * @param {boolean|undefined} test The test value (only when constant).
         * @returns {void}
         */
        makeForTest(test) {
            const context = this.loopContext;
            const forkContext = this.forkContext;
            const endOfInitSegments = forkContext.head;
            const testSegments = forkContext.makeNext(-1, -1);
            // Update state.
            context.test = test;
            context.endOfInitSegments = endOfInitSegments;
            context.continueDestSegments = context.testSegments = testSegments;
            forkContext.replaceHead(testSegments);
        }
        /**
         * Makes a code path segment for the update part of a ForStatement.
         * @returns {void}
         */
        makeForUpdate() {
            const context = this.loopContext;
            const choiceContext = this.choiceContext;
            const forkContext = this.forkContext;
            // Make the next paths of the test.
            if (context.testSegments) {
                finalizeTestSegmentsOfFor(context, choiceContext, forkContext.head);
            }
            else {
                context.endOfInitSegments = forkContext.head;
            }
            // Update state.
            const updateSegments = forkContext.makeDisconnected(-1, -1);
            context.continueDestSegments = context.updateSegments = updateSegments;
            forkContext.replaceHead(updateSegments);
        }
        /**
         * Makes a code path segment for the body part of a ForStatement.
         * @returns {void}
         */
        makeForBody() {
            const context = this.loopContext;
            const choiceContext = this.choiceContext;
            const forkContext = this.forkContext;
            // Update state.
            if (context.updateSegments) {
                context.endOfUpdateSegments = forkContext.head;
                // `update` -> `test`
                if (context.testSegments) {
                    makeLooped(this, context.endOfUpdateSegments, context.testSegments);
                }
            }
            else if (context.testSegments) {
                finalizeTestSegmentsOfFor(context, choiceContext, forkContext.head);
            }
            else {
                context.endOfInitSegments = forkContext.head;
            }
            let bodySegments = context.endOfTestSegments;
            if (!bodySegments) {
                /*
                 * If there is not the `test` part, the `body` path comes from the
                 * `init` part and the `update` part.
                 */
                const prevForkContext = ForkContext.newEmpty(forkContext);
                prevForkContext.add(context.endOfInitSegments);
                if (context.endOfUpdateSegments) {
                    prevForkContext.add(context.endOfUpdateSegments);
                }
                bodySegments = prevForkContext.makeNext(0, -1);
            }
            context.continueDestSegments = context.continueDestSegments || bodySegments;
            forkContext.replaceHead(bodySegments);
        }
        /**
         * Makes a code path segment for the left part of a ForInStatement and a
         * ForOfStatement.
         * @returns {void}
         */
        makeForInOfLeft() {
            const context = this.loopContext;
            const forkContext = this.forkContext;
            const leftSegments = forkContext.makeDisconnected(-1, -1);
            // Update state.
            context.prevSegments = forkContext.head;
            context.leftSegments = context.continueDestSegments = leftSegments;
            forkContext.replaceHead(leftSegments);
        }
        /**
         * Makes a code path segment for the right part of a ForInStatement and a
         * ForOfStatement.
         * @returns {void}
         */
        makeForInOfRight() {
            const context = this.loopContext;
            const forkContext = this.forkContext;
            const temp = ForkContext.newEmpty(forkContext);
            temp.add(context.prevSegments);
            const rightSegments = temp.makeNext(-1, -1);
            // Update state.
            context.endOfLeftSegments = forkContext.head;
            forkContext.replaceHead(rightSegments);
        }
        /**
         * Makes a code path segment for the body part of a ForInStatement and a
         * ForOfStatement.
         * @returns {void}
         */
        makeForInOfBody() {
            const context = this.loopContext;
            const forkContext = this.forkContext;
            const temp = ForkContext.newEmpty(forkContext);
            temp.add(context.endOfLeftSegments);
            const bodySegments = temp.makeNext(-1, -1);
            // Make a path: `right` -> `left`.
            makeLooped(this, forkContext.head, context.leftSegments);
            // Update state.
            context.brokenForkContext.add(forkContext.head);
            forkContext.replaceHead(bodySegments);
        }
        //--------------------------------------------------------------------------
        // Control Statements
        //--------------------------------------------------------------------------
        /**
         * Creates new context for BreakStatement.
         * @param {boolean} breakable The flag to indicate it can break by
         *      an unlabeled BreakStatement.
         * @param {string|null} label The label of this context.
         * @returns {Object} The new context.
         */
        pushBreakContext(breakable, label) {
            this.breakContext = {
                upper: this.breakContext,
                breakable,
                label,
                brokenForkContext: ForkContext.newEmpty(this.forkContext)
            };
            return this.breakContext;
        }
        /**
         * Removes the top item of the break context stack.
         * @returns {Object} The removed context.
         */
        popBreakContext() {
            const context = this.breakContext;
            const forkContext = this.forkContext;
            this.breakContext = context.upper;
            // Process this context here for other than switches and loops.
            if (!context.breakable) {
                const brokenForkContext = context.brokenForkContext;
                if (!brokenForkContext.empty) {
                    brokenForkContext.add(forkContext.head);
                    forkContext.replaceHead(brokenForkContext.makeNext(0, -1));
                }
            }
            return context;
        }
        /**
         * Makes a path for a `break` statement.
         *
         * It registers the head segment to a context of `break`.
         * It makes new unreachable segment, then it set the head with the segment.
         * @param {string} label A label of the break statement.
         * @returns {void}
         */
        makeBreak(label) {
            const forkContext = this.forkContext;
            if (!forkContext.reachable) {
                return;
            }
            const context = getBreakContext(this, label);
            /* istanbul ignore else: foolproof (syntax error) */
            if (context) {
                context.brokenForkContext.add(forkContext.head);
            }
            forkContext.replaceHead(forkContext.makeUnreachable(-1, -1));
        }
        /**
         * Makes a path for a `continue` statement.
         *
         * It makes a looping path.
         * It makes new unreachable segment, then it set the head with the segment.
         * @param {string} label A label of the continue statement.
         * @returns {void}
         */
        makeContinue(label) {
            const forkContext = this.forkContext;
            if (!forkContext.reachable) {
                return;
            }
            const context = getContinueContext(this, label);
            /* istanbul ignore else: foolproof (syntax error) */
            if (context) {
                if (context.continueDestSegments) {
                    makeLooped(this, forkContext.head, context.continueDestSegments);
                    // If the context is a for-in/of loop, this effects a break also.
                    if (context.type === "ForInStatement" ||
                        context.type === "ForOfStatement") {
                        context.brokenForkContext.add(forkContext.head);
                    }
                }
                else {
                    context.continueForkContext.add(forkContext.head);
                }
            }
            forkContext.replaceHead(forkContext.makeUnreachable(-1, -1));
        }
        /**
         * Makes a path for a `return` statement.
         *
         * It registers the head segment to a context of `return`.
         * It makes new unreachable segment, then it set the head with the segment.
         * @returns {void}
         */
        makeReturn() {
            const forkContext = this.forkContext;
            if (forkContext.reachable) {
                getReturnContext(this).returnedForkContext.add(forkContext.head);
                forkContext.replaceHead(forkContext.makeUnreachable(-1, -1));
            }
        }
        /**
         * Makes a path for a `throw` statement.
         *
         * It registers the head segment to a context of `throw`.
         * It makes new unreachable segment, then it set the head with the segment.
         * @returns {void}
         */
        makeThrow() {
            const forkContext = this.forkContext;
            if (forkContext.reachable) {
                getThrowContext(this).thrownForkContext.add(forkContext.head);
                forkContext.replaceHead(forkContext.makeUnreachable(-1, -1));
            }
        }
        /**
         * Makes the final path.
         * @returns {void}
         */
        makeFinal() {
            const segments = this.currentSegments;
            if (segments.length > 0 && segments[0].reachable) {
                this.returnedForkContext.add(segments);
            }
        }
    }
    module.exports = CodePathState;
},
/* eslint/lib/linter/code-path-analysis/code-path-segment.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const debug = require(490) /* ./debug-helpers */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Checks whether or not a given segment is reachable.
     * @param {CodePathSegment} segment A segment to check.
     * @returns {boolean} `true` if the segment is reachable.
     */
    function isReachable(segment) {
        return segment.reachable;
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * A code path segment.
     */
    class CodePathSegment {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {string} id An identifier.
         * @param {CodePathSegment[]} allPrevSegments An array of the previous segments.
         *   This array includes unreachable segments.
         * @param {boolean} reachable A flag which shows this is reachable.
         */
        constructor(id, allPrevSegments, reachable) {
            /**
             * The identifier of this code path.
             * Rules use it to store additional information of each rule.
             * @type {string}
             */
            this.id = id;
            /**
             * An array of the next segments.
             * @type {CodePathSegment[]}
             */
            this.nextSegments = [];
            /**
             * An array of the previous segments.
             * @type {CodePathSegment[]}
             */
            this.prevSegments = allPrevSegments.filter(isReachable);
            /**
             * An array of the next segments.
             * This array includes unreachable segments.
             * @type {CodePathSegment[]}
             */
            this.allNextSegments = [];
            /**
             * An array of the previous segments.
             * This array includes unreachable segments.
             * @type {CodePathSegment[]}
             */
            this.allPrevSegments = allPrevSegments;
            /**
             * A flag which shows this is reachable.
             * @type {boolean}
             */
            this.reachable = reachable;
            // Internal data.
            Object.defineProperty(this, "internal", {
                value: {
                    used: false,
                    loopedPrevSegments: []
                }
            });
            /* istanbul ignore if */
            if (debug.enabled) {
                this.internal.nodes = [];
                this.internal.exitNodes = [];
            }
        }
        /**
         * Checks a given previous segment is coming from the end of a loop.
         * @param {CodePathSegment} segment A previous segment to check.
         * @returns {boolean} `true` if the segment is coming from the end of a loop.
         */
        isLoopedPrevSegment(segment) {
            return this.internal.loopedPrevSegments.indexOf(segment) !== -1;
        }
        /**
         * Creates the root segment.
         * @param {string} id An identifier.
         * @returns {CodePathSegment} The created segment.
         */
        static newRoot(id) {
            return new CodePathSegment(id, [], true);
        }
        /**
         * Creates a segment that follows given segments.
         * @param {string} id An identifier.
         * @param {CodePathSegment[]} allPrevSegments An array of the previous segments.
         * @returns {CodePathSegment} The created segment.
         */
        static newNext(id, allPrevSegments) {
            return new CodePathSegment(id, CodePathSegment.flattenUnusedSegments(allPrevSegments), allPrevSegments.some(isReachable));
        }
        /**
         * Creates an unreachable segment that follows given segments.
         * @param {string} id An identifier.
         * @param {CodePathSegment[]} allPrevSegments An array of the previous segments.
         * @returns {CodePathSegment} The created segment.
         */
        static newUnreachable(id, allPrevSegments) {
            const segment = new CodePathSegment(id, CodePathSegment.flattenUnusedSegments(allPrevSegments), false);
            /*
             * In `if (a) return a; foo();` case, the unreachable segment preceded by
             * the return statement is not used but must not be remove.
             */
            CodePathSegment.markUsed(segment);
            return segment;
        }
        /**
         * Creates a segment that follows given segments.
         * This factory method does not connect with `allPrevSegments`.
         * But this inherits `reachable` flag.
         * @param {string} id An identifier.
         * @param {CodePathSegment[]} allPrevSegments An array of the previous segments.
         * @returns {CodePathSegment} The created segment.
         */
        static newDisconnected(id, allPrevSegments) {
            return new CodePathSegment(id, [], allPrevSegments.some(isReachable));
        }
        /**
         * Makes a given segment being used.
         *
         * And this function registers the segment into the previous segments as a next.
         * @param {CodePathSegment} segment A segment to mark.
         * @returns {void}
         */
        static markUsed(segment) {
            if (segment.internal.used) {
                return;
            }
            segment.internal.used = true;
            let i;
            if (segment.reachable) {
                for (i = 0; i < segment.allPrevSegments.length; ++i) {
                    const prevSegment = segment.allPrevSegments[i];
                    prevSegment.allNextSegments.push(segment);
                    prevSegment.nextSegments.push(segment);
                }
            }
            else {
                for (i = 0; i < segment.allPrevSegments.length; ++i) {
                    segment.allPrevSegments[i].allNextSegments.push(segment);
                }
            }
        }
        /**
         * Marks a previous segment as looped.
         * @param {CodePathSegment} segment A segment.
         * @param {CodePathSegment} prevSegment A previous segment to mark.
         * @returns {void}
         */
        static markPrevSegmentAsLooped(segment, prevSegment) {
            segment.internal.loopedPrevSegments.push(prevSegment);
        }
        /**
         * Replaces unused segments with the previous segments of each unused segment.
         * @param {CodePathSegment[]} segments An array of segments to replace.
         * @returns {CodePathSegment[]} The replaced array.
         */
        static flattenUnusedSegments(segments) {
            const done = Object.create(null);
            const retv = [];
            for (let i = 0; i < segments.length; ++i) {
                const segment = segments[i];
                // Ignores duplicated.
                if (done[segment.id]) {
                    continue;
                }
                // Use previous segments if unused.
                if (!segment.internal.used) {
                    for (let j = 0; j < segment.allPrevSegments.length; ++j) {
                        const prevSegment = segment.allPrevSegments[j];
                        if (!done[prevSegment.id]) {
                            done[prevSegment.id] = true;
                            retv.push(prevSegment);
                        }
                    }
                }
                else {
                    done[segment.id] = true;
                    retv.push(segment);
                }
            }
            return retv;
        }
    }
    module.exports = CodePathSegment;
},
/* eslint/lib/linter/code-path-analysis/debug-helpers.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const debug = require(120) /* debug */("eslint:code-path");
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Gets id of a given segment.
     * @param {CodePathSegment} segment A segment to get.
     * @returns {string} Id of the segment.
     */
    /* istanbul ignore next */
    function getId(segment) {
        return segment.id + (segment.reachable ? "" : "!");
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    module.exports = {
        /**
         * A flag that debug dumping is enabled or not.
         * @type {boolean}
         */
        enabled: debug.enabled,
        /**
         * Dumps given objects.
         * @param {...any} args objects to dump.
         * @returns {void}
         */
        dump: debug,
        /**
         * Dumps the current analyzing state.
         * @param {ASTNode} node A node to dump.
         * @param {CodePathState} state A state to dump.
         * @param {boolean} leaving A flag whether or not it's leaving
         * @returns {void}
         */
        dumpState: !debug.enabled ? debug : /* istanbul ignore next */ function (node, state, leaving) {
            for (let i = 0; i < state.currentSegments.length; ++i) {
                const segInternal = state.currentSegments[i].internal;
                if (leaving) {
                    segInternal.exitNodes.push(node);
                }
                else {
                    segInternal.nodes.push(node);
                }
            }
            debug([
                `${state.currentSegments.map(getId).join(",")})`,
                `${node.type}${leaving ? ":exit" : ""}`
            ].join(" "));
        },
        /**
         * Dumps a DOT code of a given code path.
         * The DOT code can be visialized with Graphvis.
         * @param {CodePath} codePath A code path to dump.
         * @returns {void}
         * @see http://www.graphviz.org
         * @see http://www.webgraphviz.com
         */
        dumpDot: !debug.enabled ? debug : /* istanbul ignore next */ function (codePath) {
            let text = "\n" +
                "digraph {\n" +
                "node[shape=box,style=\"rounded,filled\",fillcolor=white];\n" +
                "initial[label=\"\",shape=circle,style=filled,fillcolor=black,width=0.25,height=0.25];\n";
            if (codePath.returnedSegments.length > 0) {
                text += "final[label=\"\",shape=doublecircle,style=filled,fillcolor=black,width=0.25,height=0.25];\n";
            }
            if (codePath.thrownSegments.length > 0) {
                text += "thrown[label=\"✘\",shape=circle,width=0.3,height=0.3,fixedsize];\n";
            }
            const traceMap = Object.create(null);
            const arrows = this.makeDotArrows(codePath, traceMap);
            for (const id in traceMap) { // eslint-disable-line guard-for-in
                const segment = traceMap[id];
                text += `${id}[`;
                if (segment.reachable) {
                    text += "label=\"";
                }
                else {
                    text += "style=\"rounded,dashed,filled\",fillcolor=\"#FF9800\",label=\"<<unreachable>>\\n";
                }
                if (segment.internal.nodes.length > 0 || segment.internal.exitNodes.length > 0) {
                    text += [].concat(segment.internal.nodes.map(node => {
                        switch (node.type) {
                            case "Identifier": return `${node.type} (${node.name})`;
                            case "Literal": return `${node.type} (${node.value})`;
                            default: return node.type;
                        }
                    }), segment.internal.exitNodes.map(node => {
                        switch (node.type) {
                            case "Identifier": return `${node.type}:exit (${node.name})`;
                            case "Literal": return `${node.type}:exit (${node.value})`;
                            default: return `${node.type}:exit`;
                        }
                    })).join("\\n");
                }
                else {
                    text += "????";
                }
                text += "\"];\n";
            }
            text += `${arrows}\n`;
            text += "}";
            debug("DOT", text);
        },
        /**
         * Makes a DOT code of a given code path.
         * The DOT code can be visialized with Graphvis.
         * @param {CodePath} codePath A code path to make DOT.
         * @param {Object} traceMap Optional. A map to check whether or not segments had been done.
         * @returns {string} A DOT code of the code path.
         */
        makeDotArrows(codePath, traceMap) {
            const stack = [[codePath.initialSegment, 0]];
            const done = traceMap || Object.create(null);
            let lastId = codePath.initialSegment.id;
            let text = `initial->${codePath.initialSegment.id}`;
            while (stack.length > 0) {
                const item = stack.pop();
                const segment = item[0];
                const index = item[1];
                if (done[segment.id] && index === 0) {
                    continue;
                }
                done[segment.id] = segment;
                const nextSegment = segment.allNextSegments[index];
                if (!nextSegment) {
                    continue;
                }
                if (lastId === segment.id) {
                    text += `->${nextSegment.id}`;
                }
                else {
                    text += `;\n${segment.id}->${nextSegment.id}`;
                }
                lastId = nextSegment.id;
                stack.unshift([segment, 1 + index]);
                stack.push([nextSegment, 0]);
            }
            codePath.returnedSegments.forEach(finalSegment => {
                if (lastId === finalSegment.id) {
                    text += "->final";
                }
                else {
                    text += `;\n${finalSegment.id}->final`;
                }
                lastId = null;
            });
            codePath.thrownSegments.forEach(finalSegment => {
                if (lastId === finalSegment.id) {
                    text += "->thrown";
                }
                else {
                    text += `;\n${finalSegment.id}->thrown`;
                }
                lastId = null;
            });
            return `${text};`;
        }
    };
},
/* eslint/lib/linter/code-path-analysis/fork-context.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const assert = require("assert"), CodePathSegment = require(489) /* ./code-path-segment */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Gets whether or not a given segment is reachable.
     * @param {CodePathSegment} segment A segment to get.
     * @returns {boolean} `true` if the segment is reachable.
     */
    function isReachable(segment) {
        return segment.reachable;
    }
    /**
     * Creates new segments from the specific range of `context.segmentsList`.
     *
     * When `context.segmentsList` is `[[a, b], [c, d], [e, f]]`, `begin` is `0`, and
     * `end` is `-1`, this creates `[g, h]`. This `g` is from `a`, `c`, and `e`.
     * This `h` is from `b`, `d`, and `f`.
     * @param {ForkContext} context An instance.
     * @param {number} begin The first index of the previous segments.
     * @param {number} end The last index of the previous segments.
     * @param {Function} create A factory function of new segments.
     * @returns {CodePathSegment[]} New segments.
     */
    function makeSegments(context, begin, end, create) {
        const list = context.segmentsList;
        const normalizedBegin = begin >= 0 ? begin : list.length + begin;
        const normalizedEnd = end >= 0 ? end : list.length + end;
        const segments = [];
        for (let i = 0; i < context.count; ++i) {
            const allPrevSegments = [];
            for (let j = normalizedBegin; j <= normalizedEnd; ++j) {
                allPrevSegments.push(list[j][i]);
            }
            segments.push(create(context.idGenerator.next(), allPrevSegments));
        }
        return segments;
    }
    /**
     * `segments` becomes doubly in a `finally` block. Then if a code path exits by a
     * control statement (such as `break`, `continue`) from the `finally` block, the
     * destination's segments may be half of the source segments. In that case, this
     * merges segments.
     * @param {ForkContext} context An instance.
     * @param {CodePathSegment[]} segments Segments to merge.
     * @returns {CodePathSegment[]} The merged segments.
     */
    function mergeExtraSegments(context, segments) {
        let currentSegments = segments;
        while (currentSegments.length > context.count) {
            const merged = [];
            for (let i = 0, length = currentSegments.length / 2 | 0; i < length; ++i) {
                merged.push(CodePathSegment.newNext(context.idGenerator.next(), [currentSegments[i], currentSegments[i + length]]));
            }
            currentSegments = merged;
        }
        return currentSegments;
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * A class to manage forking.
     */
    class ForkContext {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {IdGenerator} idGenerator An identifier generator for segments.
         * @param {ForkContext|null} upper An upper fork context.
         * @param {number} count A number of parallel segments.
         */
        constructor(idGenerator, upper, count) {
            this.idGenerator = idGenerator;
            this.upper = upper;
            this.count = count;
            this.segmentsList = [];
        }
        /**
         * The head segments.
         * @type {CodePathSegment[]}
         */
        get head() {
            const list = this.segmentsList;
            return list.length === 0 ? [] : list[list.length - 1];
        }
        /**
         * A flag which shows empty.
         * @type {boolean}
         */
        get empty() {
            return this.segmentsList.length === 0;
        }
        /**
         * A flag which shows reachable.
         * @type {boolean}
         */
        get reachable() {
            const segments = this.head;
            return segments.length > 0 && segments.some(isReachable);
        }
        /**
         * Creates new segments from this context.
         * @param {number} begin The first index of previous segments.
         * @param {number} end The last index of previous segments.
         * @returns {CodePathSegment[]} New segments.
         */
        makeNext(begin, end) {
            return makeSegments(this, begin, end, CodePathSegment.newNext);
        }
        /**
         * Creates new segments from this context.
         * The new segments is always unreachable.
         * @param {number} begin The first index of previous segments.
         * @param {number} end The last index of previous segments.
         * @returns {CodePathSegment[]} New segments.
         */
        makeUnreachable(begin, end) {
            return makeSegments(this, begin, end, CodePathSegment.newUnreachable);
        }
        /**
         * Creates new segments from this context.
         * The new segments don't have connections for previous segments.
         * But these inherit the reachable flag from this context.
         * @param {number} begin The first index of previous segments.
         * @param {number} end The last index of previous segments.
         * @returns {CodePathSegment[]} New segments.
         */
        makeDisconnected(begin, end) {
            return makeSegments(this, begin, end, CodePathSegment.newDisconnected);
        }
        /**
         * Adds segments into this context.
         * The added segments become the head.
         * @param {CodePathSegment[]} segments Segments to add.
         * @returns {void}
         */
        add(segments) {
            assert(segments.length >= this.count, `${segments.length} >= ${this.count}`);
            this.segmentsList.push(mergeExtraSegments(this, segments));
        }
        /**
         * Replaces the head segments with given segments.
         * The current head segments are removed.
         * @param {CodePathSegment[]} segments Segments to add.
         * @returns {void}
         */
        replaceHead(segments) {
            assert(segments.length >= this.count, `${segments.length} >= ${this.count}`);
            this.segmentsList.splice(-1, 1, mergeExtraSegments(this, segments));
        }
        /**
         * Adds all segments of a given fork context into this context.
         * @param {ForkContext} context A fork context to add.
         * @returns {void}
         */
        addAll(context) {
            assert(context.count === this.count);
            const source = context.segmentsList;
            for (let i = 0; i < source.length; ++i) {
                this.segmentsList.push(source[i]);
            }
        }
        /**
         * Clears all secments in this context.
         * @returns {void}
         */
        clear() {
            this.segmentsList = [];
        }
        /**
         * Creates the root fork context.
         * @param {IdGenerator} idGenerator An identifier generator for segments.
         * @returns {ForkContext} New fork context.
         */
        static newRoot(idGenerator) {
            const context = new ForkContext(idGenerator, null, 1);
            context.add([CodePathSegment.newRoot(idGenerator.next())]);
            return context;
        }
        /**
         * Creates an empty fork context preceded by a given context.
         * @param {ForkContext} parentContext The parent fork context.
         * @param {boolean} forkLeavingPath A flag which shows inside of `finally` block.
         * @returns {ForkContext} New fork context.
         */
        static newEmpty(parentContext, forkLeavingPath) {
            return new ForkContext(parentContext.idGenerator, parentContext, (forkLeavingPath ? 2 : 1) * parentContext.count);
        }
    }
    module.exports = ForkContext;
},
/* eslint/lib/linter/code-path-analysis/id-generator.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * A generator for unique ids.
     */
    class IdGenerator {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {string} prefix Optional. A prefix of generated ids.
         */
        constructor(prefix) {
            this.prefix = String(prefix);
            this.n = 0;
        }
        /**
         * Generates id.
         * @returns {string} A generated id.
         */
        next() {
            this.n = 1 + this.n | 0;
            /* istanbul ignore if */
            if (this.n < 0) {
                this.n = 1;
            }
            return this.prefix + this.n;
        }
    }
    module.exports = IdGenerator;
},
/* eslint/lib/linter/apply-disable-directives.js */ function _(require, module, exports) {
    const lodash = require(111) /* lodash */;
    /**
     * Compares the locations of two objects in a source file
     * @param {{line: number, column: number}} itemA The first object
     * @param {{line: number, column: number}} itemB The second object
     * @returns {number} A value less than 1 if itemA appears before itemB in the source file, greater than 1 if
     * itemA appears after itemB in the source file, or 0 if itemA and itemB have the same location.
     */
    function compareLocations(itemA, itemB) {
        return itemA.line - itemB.line || itemA.column - itemB.column;
    }
    /**
     * This is the same as the exported function, except that it
     * doesn't handle disable-line and disable-next-line directives, and it always reports unused
     * disable directives.
     * @param {Object} options options for applying directives. This is the same as the options
     * for the exported function, except that `reportUnusedDisableDirectives` is not supported
     * (this function always reports unused disable directives).
     * @returns {{problems: Problem[], unusedDisableDirectives: Problem[]}} An object with a list
     * of filtered problems and unused eslint-disable directives
     */
    function applyDirectives(options) {
        const problems = [];
        let nextDirectiveIndex = 0;
        let currentGlobalDisableDirective = null;
        const disabledRuleMap = new Map();
        // enabledRules is only used when there is a current global disable directive.
        const enabledRules = new Set();
        const usedDisableDirectives = new Set();
        for (const problem of options.problems) {
            while (nextDirectiveIndex < options.directives.length &&
                compareLocations(options.directives[nextDirectiveIndex], problem) <= 0) {
                const directive = options.directives[nextDirectiveIndex++];
                switch (directive.type) {
                    case "disable":
                        if (directive.ruleId === null) {
                            currentGlobalDisableDirective = directive;
                            disabledRuleMap.clear();
                            enabledRules.clear();
                        }
                        else if (currentGlobalDisableDirective) {
                            enabledRules.delete(directive.ruleId);
                            disabledRuleMap.set(directive.ruleId, directive);
                        }
                        else {
                            disabledRuleMap.set(directive.ruleId, directive);
                        }
                        break;
                    case "enable":
                        if (directive.ruleId === null) {
                            currentGlobalDisableDirective = null;
                            disabledRuleMap.clear();
                        }
                        else if (currentGlobalDisableDirective) {
                            enabledRules.add(directive.ruleId);
                            disabledRuleMap.delete(directive.ruleId);
                        }
                        else {
                            disabledRuleMap.delete(directive.ruleId);
                        }
                        break;
                    // no default
                }
            }
            if (disabledRuleMap.has(problem.ruleId)) {
                usedDisableDirectives.add(disabledRuleMap.get(problem.ruleId));
            }
            else if (currentGlobalDisableDirective && !enabledRules.has(problem.ruleId)) {
                usedDisableDirectives.add(currentGlobalDisableDirective);
            }
            else {
                problems.push(problem);
            }
        }
        const unusedDisableDirectives = options.directives
            .filter(directive => directive.type === "disable" && !usedDisableDirectives.has(directive))
            .map(directive => ({
            ruleId: null,
            message: directive.ruleId
                ? `Unused eslint-disable directive (no problems were reported from '${directive.ruleId}').`
                : "Unused eslint-disable directive (no problems were reported).",
            line: directive.unprocessedDirective.line,
            column: directive.unprocessedDirective.column,
            severity: options.reportUnusedDisableDirectives === "warn" ? 1 : 2,
            nodeType: null
        }));
        return { problems, unusedDisableDirectives };
    }
    /**
     * Given a list of directive comments (i.e. metadata about eslint-disable and eslint-enable comments) and a list
     * of reported problems, determines which problems should be reported.
     * @param {Object} options Information about directives and problems
     * @param {{
     *      type: ("disable"|"enable"|"disable-line"|"disable-next-line"),
     *      ruleId: (string|null),
     *      line: number,
     *      column: number
     * }} options.directives Directive comments found in the file, with one-based columns.
     * Two directive comments can only have the same location if they also have the same type (e.g. a single eslint-disable
     * comment for two different rules is represented as two directives).
     * @param {{ruleId: (string|null), line: number, column: number}[]} options.problems
     * A list of problems reported by rules, sorted by increasing location in the file, with one-based columns.
     * @param {"off" | "warn" | "error"} options.reportUnusedDisableDirectives If `"warn"` or `"error"`, adds additional problems for unused directives
     * @returns {{ruleId: (string|null), line: number, column: number}[]}
     * A list of reported problems that were not disabled by the directive comments.
     */
    module.exports = ({ directives, problems, reportUnusedDisableDirectives = "off" }) => {
        const blockDirectives = directives
            .filter(directive => directive.type === "disable" || directive.type === "enable")
            .map(directive => Object.assign({}, directive, { unprocessedDirective: directive }))
            .sort(compareLocations);
        const lineDirectives = lodash.flatMap(directives, directive => {
            switch (directive.type) {
                case "disable":
                case "enable":
                    return [];
                case "disable-line":
                    return [
                        { type: "disable", line: directive.line, column: 1, ruleId: directive.ruleId, unprocessedDirective: directive },
                        { type: "enable", line: directive.line + 1, column: 0, ruleId: directive.ruleId, unprocessedDirective: directive }
                    ];
                case "disable-next-line":
                    return [
                        { type: "disable", line: directive.line + 1, column: 1, ruleId: directive.ruleId, unprocessedDirective: directive },
                        { type: "enable", line: directive.line + 2, column: 0, ruleId: directive.ruleId, unprocessedDirective: directive }
                    ];
                default:
                    throw new TypeError(`Unrecognized directive type '${directive.type}'`);
            }
        }).sort(compareLocations);
        const blockDirectivesResult = applyDirectives({
            problems,
            directives: blockDirectives,
            reportUnusedDisableDirectives
        });
        const lineDirectivesResult = applyDirectives({
            problems: blockDirectivesResult.problems,
            directives: lineDirectives,
            reportUnusedDisableDirectives
        });
        return reportUnusedDisableDirectives !== "off"
            ? lineDirectivesResult.problems
                .concat(blockDirectivesResult.unusedDisableDirectives)
                .concat(lineDirectivesResult.unusedDisableDirectives)
                .sort(compareLocations)
            : lineDirectivesResult.problems;
    };
},
/* eslint/lib/linter/config-comment-parser.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const levn = require(495) /* levn */, ConfigOps = require(82) /* ../shared/config-ops */;
    const debug = require(120) /* debug */("eslint:config-comment-parser");
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * Object to parse ESLint configuration comments inside JavaScript files.
     * @name ConfigCommentParser
     */
    module.exports = class ConfigCommentParser {
        /**
         * Parses a list of "name:string_value" or/and "name" options divided by comma or
         * whitespace. Used for "global" and "exported" comments.
         * @param {string} string The string to parse.
         * @param {Comment} comment The comment node which has the string.
         * @returns {Object} Result map object of names and string values, or null values if no value was provided
         */
        parseStringConfig(string, comment) {
            debug("Parsing String config");
            const items = {};
            // Collapse whitespace around `:` and `,` to make parsing easier
            const trimmedString = string.replace(/\s*([:,])\s*/gu, "$1");
            trimmedString.split(/\s|,+/u).forEach(name => {
                if (!name) {
                    return;
                }
                // value defaults to null (if not provided), e.g: "foo" => ["foo", null]
                const [key, value = null] = name.split(":");
                items[key] = { value, comment };
            });
            return items;
        }
        /**
         * Parses a JSON-like config.
         * @param {string} string The string to parse.
         * @param {Object} location Start line and column of comments for potential error message.
         * @returns {({success: true, config: Object}|{success: false, error: Problem})} Result map object
         */
        parseJsonConfig(string, location) {
            debug("Parsing JSON config");
            let items = {};
            // Parses a JSON-like comment by the same way as parsing CLI option.
            try {
                items = levn.parse("Object", string) || {};
                // Some tests say that it should ignore invalid comments such as `/*eslint no-alert:abc*/`.
                // Also, commaless notations have invalid severity:
                //     "no-alert: 2 no-console: 2" --> {"no-alert": "2 no-console: 2"}
                // Should ignore that case as well.
                if (ConfigOps.isEverySeverityValid(items)) {
                    return {
                        success: true,
                        config: items
                    };
                }
            }
            catch (ex) {
                debug("Levn parsing failed; falling back to manual parsing.");
                // ignore to parse the string by a fallback.
            }
            /*
             * Optionator cannot parse commaless notations.
             * But we are supporting that. So this is a fallback for that.
             */
            items = {};
            const normalizedString = string.replace(/([-a-zA-Z0-9/]+):/gu, "\"$1\":").replace(/(\]|[0-9])\s+(?=")/u, "$1,");
            try {
                items = JSON.parse(`{${normalizedString}}`);
            }
            catch (ex) {
                debug("Manual parsing failed.");
                return {
                    success: false,
                    error: {
                        ruleId: null,
                        fatal: true,
                        severity: 2,
                        message: `Failed to parse JSON from '${normalizedString}': ${ex.message}`,
                        line: location.start.line,
                        column: location.start.column + 1
                    }
                };
            }
            return {
                success: true,
                config: items
            };
        }
        /**
         * Parses a config of values separated by comma.
         * @param {string} string The string to parse.
         * @returns {Object} Result map of values and true values
         */
        parseListConfig(string) {
            debug("Parsing list config");
            const items = {};
            // Collapse whitespace around commas
            string.replace(/\s*,\s*/gu, ",").split(/,+/u).forEach(name => {
                const trimmedName = name.trim();
                if (trimmedName) {
                    items[trimmedName] = true;
                }
            });
            return items;
        }
    };
},
/* levn/lib/index.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    (function () {
        var parseString, cast, parseType, VERSION, parsedTypeParse, parse;
        parseString = require(496) /* ./parse-string */;
        cast = require(503) /* ./cast */;
        parseType = require(504) /* type-check */.parseType;
        VERSION = '0.3.0';
        parsedTypeParse = function (parsedType, string, options) {
            options == null && (options = {});
            options.explicit == null && (options.explicit = false);
            options.customTypes == null && (options.customTypes = {});
            return cast(parseString(parsedType, string, options), parsedType, options);
        };
        parse = function (type, string, options) {
            return parsedTypeParse(parseType(type), string, options);
        };
        module.exports = {
            VERSION: VERSION,
            parse: parse,
            parsedTypeParse: parsedTypeParse
        };
    }).call(this);
},
/* levn/lib/parse-string.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    (function () {
        var reject, special, tokenRegex;
        reject = require(497) /* prelude-ls */.reject;
        function consumeOp(tokens, op) {
            if (tokens[0] === op) {
                return tokens.shift();
            }
            else {
                throw new Error("Expected '" + op + "', but got '" + tokens[0] + "' instead in " + JSON.stringify(tokens) + ".");
            }
        }
        function maybeConsumeOp(tokens, op) {
            if (tokens[0] === op) {
                return tokens.shift();
            }
        }
        function consumeList(tokens, arg$, hasDelimiters) {
            var open, close, result, untilTest;
            open = arg$[0], close = arg$[1];
            if (hasDelimiters) {
                consumeOp(tokens, open);
            }
            result = [];
            untilTest = "," + (hasDelimiters ? close : '');
            while (tokens.length && (hasDelimiters && tokens[0] !== close)) {
                result.push(consumeElement(tokens, untilTest));
                maybeConsumeOp(tokens, ',');
            }
            if (hasDelimiters) {
                consumeOp(tokens, close);
            }
            return result;
        }
        function consumeArray(tokens, hasDelimiters) {
            return consumeList(tokens, ['[', ']'], hasDelimiters);
        }
        function consumeTuple(tokens, hasDelimiters) {
            return consumeList(tokens, ['(', ')'], hasDelimiters);
        }
        function consumeFields(tokens, hasDelimiters) {
            var result, untilTest, key;
            if (hasDelimiters) {
                consumeOp(tokens, '{');
            }
            result = {};
            untilTest = "," + (hasDelimiters ? '}' : '');
            while (tokens.length && (!hasDelimiters || tokens[0] !== '}')) {
                key = consumeValue(tokens, ':');
                consumeOp(tokens, ':');
                result[key] = consumeElement(tokens, untilTest);
                maybeConsumeOp(tokens, ',');
            }
            if (hasDelimiters) {
                consumeOp(tokens, '}');
            }
            return result;
        }
        function consumeValue(tokens, untilTest) {
            var out;
            untilTest == null && (untilTest = '');
            out = '';
            while (tokens.length && -1 === untilTest.indexOf(tokens[0])) {
                out += tokens.shift();
            }
            return out;
        }
        function consumeElement(tokens, untilTest) {
            switch (tokens[0]) {
                case '[':
                    return consumeArray(tokens, true);
                case '(':
                    return consumeTuple(tokens, true);
                case '{':
                    return consumeFields(tokens, true);
                default:
                    return consumeValue(tokens, untilTest);
            }
        }
        function consumeTopLevel(tokens, types, options) {
            var ref$, type, structure, origTokens, result, finalResult, x$, y$;
            ref$ = types[0], type = ref$.type, structure = ref$.structure;
            origTokens = tokens.concat();
            if (!options.explicit && types.length === 1 && ((!type && structure) || (type === 'Array' || type === 'Object'))) {
                result = structure === 'array' || type === 'Array'
                    ? consumeArray(tokens, tokens[0] === '[')
                    : structure === 'tuple'
                        ? consumeTuple(tokens, tokens[0] === '(')
                        : consumeFields(tokens, tokens[0] === '{');
                finalResult = tokens.length ? consumeElement(structure === 'array' || type === 'Array'
                    ? (x$ = origTokens, x$.unshift('['), x$.push(']'), x$)
                    : (y$ = origTokens, y$.unshift('('), y$.push(')'), y$)) : result;
            }
            else {
                finalResult = consumeElement(tokens);
            }
            return finalResult;
        }
        special = /\[\]\(\)}{:,/.source;
        tokenRegex = RegExp('("(?:\\\\"|[^"])*")|(\'(?:\\\\\'|[^\'])*\')|(/(?:\\\\/|[^/])*/[a-zA-Z]*)|(#.*#)|([' + special + '])|([^\\s' + special + '](?:\\s*[^\\s' + special + ']+)*)|\\s*');
        module.exports = function (types, string, options) {
            var tokens, node;
            options == null && (options = {});
            if (!options.explicit && types.length === 1 && types[0].type === 'String') {
                return "'" + string.replace(/\\'/g, "\\\\'") + "'";
            }
            tokens = reject(not$, string.split(tokenRegex));
            node = consumeTopLevel(tokens, types, options);
            if (!node) {
                throw new Error("Error parsing '" + string + "'.");
            }
            return node;
        };
        function not$(x) { return !x; }
    }).call(this);
},
/* prelude-ls/lib/index.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    var Func, List, Obj, Str, Num, id, isType, replicate, prelude, toString$ = {}.toString;
    Func = require(498) /* ./Func.js */;
    List = require(499) /* ./List.js */;
    Obj = require(500) /* ./Obj.js */;
    Str = require(501) /* ./Str.js */;
    Num = require(502) /* ./Num.js */;
    id = function (x) {
        return x;
    };
    isType = curry$(function (type, x) {
        return toString$.call(x).slice(8, -1) === type;
    });
    replicate = curry$(function (n, x) {
        var i$, results$ = [];
        for (i$ = 0; i$ < n; ++i$) {
            results$.push(x);
        }
        return results$;
    });
    Str.empty = List.empty;
    Str.slice = List.slice;
    Str.take = List.take;
    Str.drop = List.drop;
    Str.splitAt = List.splitAt;
    Str.takeWhile = List.takeWhile;
    Str.dropWhile = List.dropWhile;
    Str.span = List.span;
    Str.breakStr = List.breakList;
    prelude = {
        Func: Func,
        List: List,
        Obj: Obj,
        Str: Str,
        Num: Num,
        id: id,
        isType: isType,
        replicate: replicate
    };
    prelude.each = List.each;
    prelude.map = List.map;
    prelude.filter = List.filter;
    prelude.compact = List.compact;
    prelude.reject = List.reject;
    prelude.partition = List.partition;
    prelude.find = List.find;
    prelude.head = List.head;
    prelude.first = List.first;
    prelude.tail = List.tail;
    prelude.last = List.last;
    prelude.initial = List.initial;
    prelude.empty = List.empty;
    prelude.reverse = List.reverse;
    prelude.difference = List.difference;
    prelude.intersection = List.intersection;
    prelude.union = List.union;
    prelude.countBy = List.countBy;
    prelude.groupBy = List.groupBy;
    prelude.fold = List.fold;
    prelude.foldl = List.foldl;
    prelude.fold1 = List.fold1;
    prelude.foldl1 = List.foldl1;
    prelude.foldr = List.foldr;
    prelude.foldr1 = List.foldr1;
    prelude.unfoldr = List.unfoldr;
    prelude.andList = List.andList;
    prelude.orList = List.orList;
    prelude.any = List.any;
    prelude.all = List.all;
    prelude.unique = List.unique;
    prelude.uniqueBy = List.uniqueBy;
    prelude.sort = List.sort;
    prelude.sortWith = List.sortWith;
    prelude.sortBy = List.sortBy;
    prelude.sum = List.sum;
    prelude.product = List.product;
    prelude.mean = List.mean;
    prelude.average = List.average;
    prelude.concat = List.concat;
    prelude.concatMap = List.concatMap;
    prelude.flatten = List.flatten;
    prelude.maximum = List.maximum;
    prelude.minimum = List.minimum;
    prelude.maximumBy = List.maximumBy;
    prelude.minimumBy = List.minimumBy;
    prelude.scan = List.scan;
    prelude.scanl = List.scanl;
    prelude.scan1 = List.scan1;
    prelude.scanl1 = List.scanl1;
    prelude.scanr = List.scanr;
    prelude.scanr1 = List.scanr1;
    prelude.slice = List.slice;
    prelude.take = List.take;
    prelude.drop = List.drop;
    prelude.splitAt = List.splitAt;
    prelude.takeWhile = List.takeWhile;
    prelude.dropWhile = List.dropWhile;
    prelude.span = List.span;
    prelude.breakList = List.breakList;
    prelude.zip = List.zip;
    prelude.zipWith = List.zipWith;
    prelude.zipAll = List.zipAll;
    prelude.zipAllWith = List.zipAllWith;
    prelude.at = List.at;
    prelude.elemIndex = List.elemIndex;
    prelude.elemIndices = List.elemIndices;
    prelude.findIndex = List.findIndex;
    prelude.findIndices = List.findIndices;
    prelude.apply = Func.apply;
    prelude.curry = Func.curry;
    prelude.flip = Func.flip;
    prelude.fix = Func.fix;
    prelude.over = Func.over;
    prelude.split = Str.split;
    prelude.join = Str.join;
    prelude.lines = Str.lines;
    prelude.unlines = Str.unlines;
    prelude.words = Str.words;
    prelude.unwords = Str.unwords;
    prelude.chars = Str.chars;
    prelude.unchars = Str.unchars;
    prelude.repeat = Str.repeat;
    prelude.capitalize = Str.capitalize;
    prelude.camelize = Str.camelize;
    prelude.dasherize = Str.dasherize;
    prelude.values = Obj.values;
    prelude.keys = Obj.keys;
    prelude.pairsToObj = Obj.pairsToObj;
    prelude.objToPairs = Obj.objToPairs;
    prelude.listsToObj = Obj.listsToObj;
    prelude.objToLists = Obj.objToLists;
    prelude.max = Num.max;
    prelude.min = Num.min;
    prelude.negate = Num.negate;
    prelude.abs = Num.abs;
    prelude.signum = Num.signum;
    prelude.quot = Num.quot;
    prelude.rem = Num.rem;
    prelude.div = Num.div;
    prelude.mod = Num.mod;
    prelude.recip = Num.recip;
    prelude.pi = Num.pi;
    prelude.tau = Num.tau;
    prelude.exp = Num.exp;
    prelude.sqrt = Num.sqrt;
    prelude.ln = Num.ln;
    prelude.pow = Num.pow;
    prelude.sin = Num.sin;
    prelude.tan = Num.tan;
    prelude.cos = Num.cos;
    prelude.acos = Num.acos;
    prelude.asin = Num.asin;
    prelude.atan = Num.atan;
    prelude.atan2 = Num.atan2;
    prelude.truncate = Num.truncate;
    prelude.round = Num.round;
    prelude.ceiling = Num.ceiling;
    prelude.floor = Num.floor;
    prelude.isItNaN = Num.isItNaN;
    prelude.even = Num.even;
    prelude.odd = Num.odd;
    prelude.gcd = Num.gcd;
    prelude.lcm = Num.lcm;
    prelude.VERSION = '1.1.2';
    module.exports = prelude;
    function curry$(f, bound) {
        var context, _curry = function (args) {
            return f.length > 1 ? function () {
                var params = args ? args.concat() : [];
                context = bound ? context || this : this;
                return params.push.apply(params, arguments) <
                    f.length && arguments.length ?
                    _curry.call(context, params) : f.apply(context, params);
            } : f;
        };
        return _curry();
    }
},
/* prelude-ls/lib/Func.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    var apply, curry, flip, fix, over, memoize, slice$ = [].slice, toString$ = {}.toString;
    apply = curry$(function (f, list) {
        return f.apply(null, list);
    });
    curry = function (f) {
        return curry$(f);
    };
    flip = curry$(function (f, x, y) {
        return f(y, x);
    });
    fix = function (f) {
        return function (g) {
            return function () {
                return f(g(g)).apply(null, arguments);
            };
        }(function (g) {
            return function () {
                return f(g(g)).apply(null, arguments);
            };
        });
    };
    over = curry$(function (f, g, x, y) {
        return f(g(x), g(y));
    });
    memoize = function (f) {
        var memo;
        memo = {};
        return function () {
            var args, key, arg;
            args = slice$.call(arguments);
            key = (function () {
                var i$, ref$, len$, results$ = [];
                for (i$ = 0, len$ = (ref$ = args).length; i$ < len$; ++i$) {
                    arg = ref$[i$];
                    results$.push(arg + toString$.call(arg).slice(8, -1));
                }
                return results$;
            }()).join('');
            return memo[key] = key in memo
                ? memo[key]
                : f.apply(null, args);
        };
    };
    module.exports = {
        curry: curry,
        flip: flip,
        fix: fix,
        apply: apply,
        over: over,
        memoize: memoize
    };
    function curry$(f, bound) {
        var context, _curry = function (args) {
            return f.length > 1 ? function () {
                var params = args ? args.concat() : [];
                context = bound ? context || this : this;
                return params.push.apply(params, arguments) <
                    f.length && arguments.length ?
                    _curry.call(context, params) : f.apply(context, params);
            } : f;
        };
        return _curry();
    }
},
/* prelude-ls/lib/List.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    var each, map, compact, filter, reject, partition, find, head, first, tail, last, initial, empty, reverse, unique, uniqueBy, fold, foldl, fold1, foldl1, foldr, foldr1, unfoldr, concat, concatMap, flatten, difference, intersection, union, countBy, groupBy, andList, orList, any, all, sort, sortWith, sortBy, sum, product, mean, average, maximum, minimum, maximumBy, minimumBy, scan, scanl, scan1, scanl1, scanr, scanr1, slice, take, drop, splitAt, takeWhile, dropWhile, span, breakList, zip, zipWith, zipAll, zipAllWith, at, elemIndex, elemIndices, findIndex, findIndices, toString$ = {}.toString, slice$ = [].slice;
    each = curry$(function (f, xs) {
        var i$, len$, x;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            f(x);
        }
        return xs;
    });
    map = curry$(function (f, xs) {
        var i$, len$, x, results$ = [];
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            results$.push(f(x));
        }
        return results$;
    });
    compact = function (xs) {
        var i$, len$, x, results$ = [];
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            if (x) {
                results$.push(x);
            }
        }
        return results$;
    };
    filter = curry$(function (f, xs) {
        var i$, len$, x, results$ = [];
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            if (f(x)) {
                results$.push(x);
            }
        }
        return results$;
    });
    reject = curry$(function (f, xs) {
        var i$, len$, x, results$ = [];
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            if (!f(x)) {
                results$.push(x);
            }
        }
        return results$;
    });
    partition = curry$(function (f, xs) {
        var passed, failed, i$, len$, x;
        passed = [];
        failed = [];
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            (f(x) ? passed : failed).push(x);
        }
        return [passed, failed];
    });
    find = curry$(function (f, xs) {
        var i$, len$, x;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            if (f(x)) {
                return x;
            }
        }
    });
    head = first = function (xs) {
        return xs[0];
    };
    tail = function (xs) {
        if (!xs.length) {
            return;
        }
        return xs.slice(1);
    };
    last = function (xs) {
        return xs[xs.length - 1];
    };
    initial = function (xs) {
        if (!xs.length) {
            return;
        }
        return xs.slice(0, -1);
    };
    empty = function (xs) {
        return !xs.length;
    };
    reverse = function (xs) {
        return xs.concat().reverse();
    };
    unique = function (xs) {
        var result, i$, len$, x;
        result = [];
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            if (!in$(x, result)) {
                result.push(x);
            }
        }
        return result;
    };
    uniqueBy = curry$(function (f, xs) {
        var seen, i$, len$, x, val, results$ = [];
        seen = [];
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            val = f(x);
            if (in$(val, seen)) {
                continue;
            }
            seen.push(val);
            results$.push(x);
        }
        return results$;
    });
    fold = foldl = curry$(function (f, memo, xs) {
        var i$, len$, x;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            memo = f(memo, x);
        }
        return memo;
    });
    fold1 = foldl1 = curry$(function (f, xs) {
        return fold(f, xs[0], xs.slice(1));
    });
    foldr = curry$(function (f, memo, xs) {
        var i$, x;
        for (i$ = xs.length - 1; i$ >= 0; --i$) {
            x = xs[i$];
            memo = f(x, memo);
        }
        return memo;
    });
    foldr1 = curry$(function (f, xs) {
        return foldr(f, xs[xs.length - 1], xs.slice(0, -1));
    });
    unfoldr = curry$(function (f, b) {
        var result, x, that;
        result = [];
        x = b;
        while ((that = f(x)) != null) {
            result.push(that[0]);
            x = that[1];
        }
        return result;
    });
    concat = function (xss) {
        return [].concat.apply([], xss);
    };
    concatMap = curry$(function (f, xs) {
        var x;
        return [].concat.apply([], (function () {
            var i$, ref$, len$, results$ = [];
            for (i$ = 0, len$ = (ref$ = xs).length; i$ < len$; ++i$) {
                x = ref$[i$];
                results$.push(f(x));
            }
            return results$;
        }()));
    });
    flatten = function (xs) {
        var x;
        return [].concat.apply([], (function () {
            var i$, ref$, len$, results$ = [];
            for (i$ = 0, len$ = (ref$ = xs).length; i$ < len$; ++i$) {
                x = ref$[i$];
                if (toString$.call(x).slice(8, -1) === 'Array') {
                    results$.push(flatten(x));
                }
                else {
                    results$.push(x);
                }
            }
            return results$;
        }()));
    };
    difference = function (xs) {
        var yss, results, i$, len$, x, j$, len1$, ys;
        yss = slice$.call(arguments, 1);
        results = [];
        outer: for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            for (j$ = 0, len1$ = yss.length; j$ < len1$; ++j$) {
                ys = yss[j$];
                if (in$(x, ys)) {
                    continue outer;
                }
            }
            results.push(x);
        }
        return results;
    };
    intersection = function (xs) {
        var yss, results, i$, len$, x, j$, len1$, ys;
        yss = slice$.call(arguments, 1);
        results = [];
        outer: for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            for (j$ = 0, len1$ = yss.length; j$ < len1$; ++j$) {
                ys = yss[j$];
                if (!in$(x, ys)) {
                    continue outer;
                }
            }
            results.push(x);
        }
        return results;
    };
    union = function () {
        var xss, results, i$, len$, xs, j$, len1$, x;
        xss = slice$.call(arguments);
        results = [];
        for (i$ = 0, len$ = xss.length; i$ < len$; ++i$) {
            xs = xss[i$];
            for (j$ = 0, len1$ = xs.length; j$ < len1$; ++j$) {
                x = xs[j$];
                if (!in$(x, results)) {
                    results.push(x);
                }
            }
        }
        return results;
    };
    countBy = curry$(function (f, xs) {
        var results, i$, len$, x, key;
        results = {};
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            key = f(x);
            if (key in results) {
                results[key] += 1;
            }
            else {
                results[key] = 1;
            }
        }
        return results;
    });
    groupBy = curry$(function (f, xs) {
        var results, i$, len$, x, key;
        results = {};
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            key = f(x);
            if (key in results) {
                results[key].push(x);
            }
            else {
                results[key] = [x];
            }
        }
        return results;
    });
    andList = function (xs) {
        var i$, len$, x;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            if (!x) {
                return false;
            }
        }
        return true;
    };
    orList = function (xs) {
        var i$, len$, x;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            if (x) {
                return true;
            }
        }
        return false;
    };
    any = curry$(function (f, xs) {
        var i$, len$, x;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            if (f(x)) {
                return true;
            }
        }
        return false;
    });
    all = curry$(function (f, xs) {
        var i$, len$, x;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            if (!f(x)) {
                return false;
            }
        }
        return true;
    });
    sort = function (xs) {
        return xs.concat().sort(function (x, y) {
            if (x > y) {
                return 1;
            }
            else if (x < y) {
                return -1;
            }
            else {
                return 0;
            }
        });
    };
    sortWith = curry$(function (f, xs) {
        return xs.concat().sort(f);
    });
    sortBy = curry$(function (f, xs) {
        return xs.concat().sort(function (x, y) {
            if (f(x) > f(y)) {
                return 1;
            }
            else if (f(x) < f(y)) {
                return -1;
            }
            else {
                return 0;
            }
        });
    });
    sum = function (xs) {
        var result, i$, len$, x;
        result = 0;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            result += x;
        }
        return result;
    };
    product = function (xs) {
        var result, i$, len$, x;
        result = 1;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            result *= x;
        }
        return result;
    };
    mean = average = function (xs) {
        var sum, i$, len$, x;
        sum = 0;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            x = xs[i$];
            sum += x;
        }
        return sum / xs.length;
    };
    maximum = function (xs) {
        var max, i$, ref$, len$, x;
        max = xs[0];
        for (i$ = 0, len$ = (ref$ = xs.slice(1)).length; i$ < len$; ++i$) {
            x = ref$[i$];
            if (x > max) {
                max = x;
            }
        }
        return max;
    };
    minimum = function (xs) {
        var min, i$, ref$, len$, x;
        min = xs[0];
        for (i$ = 0, len$ = (ref$ = xs.slice(1)).length; i$ < len$; ++i$) {
            x = ref$[i$];
            if (x < min) {
                min = x;
            }
        }
        return min;
    };
    maximumBy = curry$(function (f, xs) {
        var max, i$, ref$, len$, x;
        max = xs[0];
        for (i$ = 0, len$ = (ref$ = xs.slice(1)).length; i$ < len$; ++i$) {
            x = ref$[i$];
            if (f(x) > f(max)) {
                max = x;
            }
        }
        return max;
    });
    minimumBy = curry$(function (f, xs) {
        var min, i$, ref$, len$, x;
        min = xs[0];
        for (i$ = 0, len$ = (ref$ = xs.slice(1)).length; i$ < len$; ++i$) {
            x = ref$[i$];
            if (f(x) < f(min)) {
                min = x;
            }
        }
        return min;
    });
    scan = scanl = curry$(function (f, memo, xs) {
        var last, x;
        last = memo;
        return [memo].concat((function () {
            var i$, ref$, len$, results$ = [];
            for (i$ = 0, len$ = (ref$ = xs).length; i$ < len$; ++i$) {
                x = ref$[i$];
                results$.push(last = f(last, x));
            }
            return results$;
        }()));
    });
    scan1 = scanl1 = curry$(function (f, xs) {
        if (!xs.length) {
            return;
        }
        return scan(f, xs[0], xs.slice(1));
    });
    scanr = curry$(function (f, memo, xs) {
        xs = xs.concat().reverse();
        return scan(f, memo, xs).reverse();
    });
    scanr1 = curry$(function (f, xs) {
        if (!xs.length) {
            return;
        }
        xs = xs.concat().reverse();
        return scan(f, xs[0], xs.slice(1)).reverse();
    });
    slice = curry$(function (x, y, xs) {
        return xs.slice(x, y);
    });
    take = curry$(function (n, xs) {
        if (n <= 0) {
            return xs.slice(0, 0);
        }
        else {
            return xs.slice(0, n);
        }
    });
    drop = curry$(function (n, xs) {
        if (n <= 0) {
            return xs;
        }
        else {
            return xs.slice(n);
        }
    });
    splitAt = curry$(function (n, xs) {
        return [take(n, xs), drop(n, xs)];
    });
    takeWhile = curry$(function (p, xs) {
        var len, i;
        len = xs.length;
        if (!len) {
            return xs;
        }
        i = 0;
        while (i < len && p(xs[i])) {
            i += 1;
        }
        return xs.slice(0, i);
    });
    dropWhile = curry$(function (p, xs) {
        var len, i;
        len = xs.length;
        if (!len) {
            return xs;
        }
        i = 0;
        while (i < len && p(xs[i])) {
            i += 1;
        }
        return xs.slice(i);
    });
    span = curry$(function (p, xs) {
        return [takeWhile(p, xs), dropWhile(p, xs)];
    });
    breakList = curry$(function (p, xs) {
        return span(compose$(p, not$), xs);
    });
    zip = curry$(function (xs, ys) {
        var result, len, i$, len$, i, x;
        result = [];
        len = ys.length;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            i = i$;
            x = xs[i$];
            if (i === len) {
                break;
            }
            result.push([x, ys[i]]);
        }
        return result;
    });
    zipWith = curry$(function (f, xs, ys) {
        var result, len, i$, len$, i, x;
        result = [];
        len = ys.length;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            i = i$;
            x = xs[i$];
            if (i === len) {
                break;
            }
            result.push(f(x, ys[i]));
        }
        return result;
    });
    zipAll = function () {
        var xss, minLength, i$, len$, xs, ref$, i, lresult$, j$, results$ = [];
        xss = slice$.call(arguments);
        minLength = undefined;
        for (i$ = 0, len$ = xss.length; i$ < len$; ++i$) {
            xs = xss[i$];
            minLength <= (ref$ = xs.length) || (minLength = ref$);
        }
        for (i$ = 0; i$ < minLength; ++i$) {
            i = i$;
            lresult$ = [];
            for (j$ = 0, len$ = xss.length; j$ < len$; ++j$) {
                xs = xss[j$];
                lresult$.push(xs[i]);
            }
            results$.push(lresult$);
        }
        return results$;
    };
    zipAllWith = function (f) {
        var xss, minLength, i$, len$, xs, ref$, i, results$ = [];
        xss = slice$.call(arguments, 1);
        minLength = undefined;
        for (i$ = 0, len$ = xss.length; i$ < len$; ++i$) {
            xs = xss[i$];
            minLength <= (ref$ = xs.length) || (minLength = ref$);
        }
        for (i$ = 0; i$ < minLength; ++i$) {
            i = i$;
            results$.push(f.apply(null, (fn$())));
        }
        return results$;
        function fn$() {
            var i$, ref$, len$, results$ = [];
            for (i$ = 0, len$ = (ref$ = xss).length; i$ < len$; ++i$) {
                xs = ref$[i$];
                results$.push(xs[i]);
            }
            return results$;
        }
    };
    at = curry$(function (n, xs) {
        if (n < 0) {
            return xs[xs.length + n];
        }
        else {
            return xs[n];
        }
    });
    elemIndex = curry$(function (el, xs) {
        var i$, len$, i, x;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            i = i$;
            x = xs[i$];
            if (x === el) {
                return i;
            }
        }
    });
    elemIndices = curry$(function (el, xs) {
        var i$, len$, i, x, results$ = [];
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            i = i$;
            x = xs[i$];
            if (x === el) {
                results$.push(i);
            }
        }
        return results$;
    });
    findIndex = curry$(function (f, xs) {
        var i$, len$, i, x;
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            i = i$;
            x = xs[i$];
            if (f(x)) {
                return i;
            }
        }
    });
    findIndices = curry$(function (f, xs) {
        var i$, len$, i, x, results$ = [];
        for (i$ = 0, len$ = xs.length; i$ < len$; ++i$) {
            i = i$;
            x = xs[i$];
            if (f(x)) {
                results$.push(i);
            }
        }
        return results$;
    });
    module.exports = {
        each: each,
        map: map,
        filter: filter,
        compact: compact,
        reject: reject,
        partition: partition,
        find: find,
        head: head,
        first: first,
        tail: tail,
        last: last,
        initial: initial,
        empty: empty,
        reverse: reverse,
        difference: difference,
        intersection: intersection,
        union: union,
        countBy: countBy,
        groupBy: groupBy,
        fold: fold,
        fold1: fold1,
        foldl: foldl,
        foldl1: foldl1,
        foldr: foldr,
        foldr1: foldr1,
        unfoldr: unfoldr,
        andList: andList,
        orList: orList,
        any: any,
        all: all,
        unique: unique,
        uniqueBy: uniqueBy,
        sort: sort,
        sortWith: sortWith,
        sortBy: sortBy,
        sum: sum,
        product: product,
        mean: mean,
        average: average,
        concat: concat,
        concatMap: concatMap,
        flatten: flatten,
        maximum: maximum,
        minimum: minimum,
        maximumBy: maximumBy,
        minimumBy: minimumBy,
        scan: scan,
        scan1: scan1,
        scanl: scanl,
        scanl1: scanl1,
        scanr: scanr,
        scanr1: scanr1,
        slice: slice,
        take: take,
        drop: drop,
        splitAt: splitAt,
        takeWhile: takeWhile,
        dropWhile: dropWhile,
        span: span,
        breakList: breakList,
        zip: zip,
        zipWith: zipWith,
        zipAll: zipAll,
        zipAllWith: zipAllWith,
        at: at,
        elemIndex: elemIndex,
        elemIndices: elemIndices,
        findIndex: findIndex,
        findIndices: findIndices
    };
    function curry$(f, bound) {
        var context, _curry = function (args) {
            return f.length > 1 ? function () {
                var params = args ? args.concat() : [];
                context = bound ? context || this : this;
                return params.push.apply(params, arguments) <
                    f.length && arguments.length ?
                    _curry.call(context, params) : f.apply(context, params);
            } : f;
        };
        return _curry();
    }
    function in$(x, xs) {
        var i = -1, l = xs.length >>> 0;
        while (++i < l)
            if (x === xs[i])
                return true;
        return false;
    }
    function compose$() {
        var functions = arguments;
        return function () {
            var i, result;
            result = functions[0].apply(this, arguments);
            for (i = 1; i < functions.length; ++i) {
                result = functions[i](result);
            }
            return result;
        };
    }
    function not$(x) { return !x; }
},
/* prelude-ls/lib/Obj.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    var values, keys, pairsToObj, objToPairs, listsToObj, objToLists, empty, each, map, compact, filter, reject, partition, find;
    values = function (object) {
        var i$, x, results$ = [];
        for (i$ in object) {
            x = object[i$];
            results$.push(x);
        }
        return results$;
    };
    keys = function (object) {
        var x, results$ = [];
        for (x in object) {
            results$.push(x);
        }
        return results$;
    };
    pairsToObj = function (object) {
        var i$, len$, x, resultObj$ = {};
        for (i$ = 0, len$ = object.length; i$ < len$; ++i$) {
            x = object[i$];
            resultObj$[x[0]] = x[1];
        }
        return resultObj$;
    };
    objToPairs = function (object) {
        var key, value, results$ = [];
        for (key in object) {
            value = object[key];
            results$.push([key, value]);
        }
        return results$;
    };
    listsToObj = curry$(function (keys, values) {
        var i$, len$, i, key, resultObj$ = {};
        for (i$ = 0, len$ = keys.length; i$ < len$; ++i$) {
            i = i$;
            key = keys[i$];
            resultObj$[key] = values[i];
        }
        return resultObj$;
    });
    objToLists = function (object) {
        var keys, values, key, value;
        keys = [];
        values = [];
        for (key in object) {
            value = object[key];
            keys.push(key);
            values.push(value);
        }
        return [keys, values];
    };
    empty = function (object) {
        var x;
        for (x in object) {
            return false;
        }
        return true;
    };
    each = curry$(function (f, object) {
        var i$, x;
        for (i$ in object) {
            x = object[i$];
            f(x);
        }
        return object;
    });
    map = curry$(function (f, object) {
        var k, x, resultObj$ = {};
        for (k in object) {
            x = object[k];
            resultObj$[k] = f(x);
        }
        return resultObj$;
    });
    compact = function (object) {
        var k, x, resultObj$ = {};
        for (k in object) {
            x = object[k];
            if (x) {
                resultObj$[k] = x;
            }
        }
        return resultObj$;
    };
    filter = curry$(function (f, object) {
        var k, x, resultObj$ = {};
        for (k in object) {
            x = object[k];
            if (f(x)) {
                resultObj$[k] = x;
            }
        }
        return resultObj$;
    });
    reject = curry$(function (f, object) {
        var k, x, resultObj$ = {};
        for (k in object) {
            x = object[k];
            if (!f(x)) {
                resultObj$[k] = x;
            }
        }
        return resultObj$;
    });
    partition = curry$(function (f, object) {
        var passed, failed, k, x;
        passed = {};
        failed = {};
        for (k in object) {
            x = object[k];
            (f(x) ? passed : failed)[k] = x;
        }
        return [passed, failed];
    });
    find = curry$(function (f, object) {
        var i$, x;
        for (i$ in object) {
            x = object[i$];
            if (f(x)) {
                return x;
            }
        }
    });
    module.exports = {
        values: values,
        keys: keys,
        pairsToObj: pairsToObj,
        objToPairs: objToPairs,
        listsToObj: listsToObj,
        objToLists: objToLists,
        empty: empty,
        each: each,
        map: map,
        filter: filter,
        compact: compact,
        reject: reject,
        partition: partition,
        find: find
    };
    function curry$(f, bound) {
        var context, _curry = function (args) {
            return f.length > 1 ? function () {
                var params = args ? args.concat() : [];
                context = bound ? context || this : this;
                return params.push.apply(params, arguments) <
                    f.length && arguments.length ?
                    _curry.call(context, params) : f.apply(context, params);
            } : f;
        };
        return _curry();
    }
},
/* prelude-ls/lib/Str.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    var split, join, lines, unlines, words, unwords, chars, unchars, reverse, repeat, capitalize, camelize, dasherize;
    split = curry$(function (sep, str) {
        return str.split(sep);
    });
    join = curry$(function (sep, xs) {
        return xs.join(sep);
    });
    lines = function (str) {
        if (!str.length) {
            return [];
        }
        return str.split('\n');
    };
    unlines = function (it) {
        return it.join('\n');
    };
    words = function (str) {
        if (!str.length) {
            return [];
        }
        return str.split(/[ ]+/);
    };
    unwords = function (it) {
        return it.join(' ');
    };
    chars = function (it) {
        return it.split('');
    };
    unchars = function (it) {
        return it.join('');
    };
    reverse = function (str) {
        return str.split('').reverse().join('');
    };
    repeat = curry$(function (n, str) {
        var result, i$;
        result = '';
        for (i$ = 0; i$ < n; ++i$) {
            result += str;
        }
        return result;
    });
    capitalize = function (str) {
        return str.charAt(0).toUpperCase() + str.slice(1);
    };
    camelize = function (it) {
        return it.replace(/[-_]+(.)?/g, function (arg$, c) {
            return (c != null ? c : '').toUpperCase();
        });
    };
    dasherize = function (str) {
        return str.replace(/([^-A-Z])([A-Z]+)/g, function (arg$, lower, upper) {
            return lower + "-" + (upper.length > 1
                ? upper
                : upper.toLowerCase());
        }).replace(/^([A-Z]+)/, function (arg$, upper) {
            if (upper.length > 1) {
                return upper + "-";
            }
            else {
                return upper.toLowerCase();
            }
        });
    };
    module.exports = {
        split: split,
        join: join,
        lines: lines,
        unlines: unlines,
        words: words,
        unwords: unwords,
        chars: chars,
        unchars: unchars,
        reverse: reverse,
        repeat: repeat,
        capitalize: capitalize,
        camelize: camelize,
        dasherize: dasherize
    };
    function curry$(f, bound) {
        var context, _curry = function (args) {
            return f.length > 1 ? function () {
                var params = args ? args.concat() : [];
                context = bound ? context || this : this;
                return params.push.apply(params, arguments) <
                    f.length && arguments.length ?
                    _curry.call(context, params) : f.apply(context, params);
            } : f;
        };
        return _curry();
    }
},
/* prelude-ls/lib/Num.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    var max, min, negate, abs, signum, quot, rem, div, mod, recip, pi, tau, exp, sqrt, ln, pow, sin, tan, cos, asin, acos, atan, atan2, truncate, round, ceiling, floor, isItNaN, even, odd, gcd, lcm;
    max = curry$(function (x$, y$) {
        return x$ > y$ ? x$ : y$;
    });
    min = curry$(function (x$, y$) {
        return x$ < y$ ? x$ : y$;
    });
    negate = function (x) {
        return -x;
    };
    abs = Math.abs;
    signum = function (x) {
        if (x < 0) {
            return -1;
        }
        else if (x > 0) {
            return 1;
        }
        else {
            return 0;
        }
    };
    quot = curry$(function (x, y) {
        return ~~(x / y);
    });
    rem = curry$(function (x$, y$) {
        return x$ % y$;
    });
    div = curry$(function (x, y) {
        return Math.floor(x / y);
    });
    mod = curry$(function (x$, y$) {
        var ref$;
        return (((x$) % (ref$ = y$) + ref$) % ref$);
    });
    recip = (function (it) {
        return 1 / it;
    });
    pi = Math.PI;
    tau = pi * 2;
    exp = Math.exp;
    sqrt = Math.sqrt;
    ln = Math.log;
    pow = curry$(function (x$, y$) {
        return Math.pow(x$, y$);
    });
    sin = Math.sin;
    tan = Math.tan;
    cos = Math.cos;
    asin = Math.asin;
    acos = Math.acos;
    atan = Math.atan;
    atan2 = curry$(function (x, y) {
        return Math.atan2(x, y);
    });
    truncate = function (x) {
        return ~~x;
    };
    round = Math.round;
    ceiling = Math.ceil;
    floor = Math.floor;
    isItNaN = function (x) {
        return x !== x;
    };
    even = function (x) {
        return x % 2 === 0;
    };
    odd = function (x) {
        return x % 2 !== 0;
    };
    gcd = curry$(function (x, y) {
        var z;
        x = Math.abs(x);
        y = Math.abs(y);
        while (y !== 0) {
            z = x % y;
            x = y;
            y = z;
        }
        return x;
    });
    lcm = curry$(function (x, y) {
        return Math.abs(Math.floor(x / gcd(x, y) * y));
    });
    module.exports = {
        max: max,
        min: min,
        negate: negate,
        abs: abs,
        signum: signum,
        quot: quot,
        rem: rem,
        div: div,
        mod: mod,
        recip: recip,
        pi: pi,
        tau: tau,
        exp: exp,
        sqrt: sqrt,
        ln: ln,
        pow: pow,
        sin: sin,
        tan: tan,
        cos: cos,
        acos: acos,
        asin: asin,
        atan: atan,
        atan2: atan2,
        truncate: truncate,
        round: round,
        ceiling: ceiling,
        floor: floor,
        isItNaN: isItNaN,
        even: even,
        odd: odd,
        gcd: gcd,
        lcm: lcm
    };
    function curry$(f, bound) {
        var context, _curry = function (args) {
            return f.length > 1 ? function () {
                var params = args ? args.concat() : [];
                context = bound ? context || this : this;
                return params.push.apply(params, arguments) <
                    f.length && arguments.length ?
                    _curry.call(context, params) : f.apply(context, params);
            } : f;
        };
        return _curry();
    }
},
/* levn/lib/cast.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    (function () {
        var parsedTypeCheck, types, toString$ = {}.toString;
        parsedTypeCheck = require(504) /* type-check */.parsedTypeCheck;
        types = {
            '*': function (value, options) {
                switch (toString$.call(value).slice(8, -1)) {
                    case 'Array':
                        return typeCast(value, {
                            type: 'Array'
                        }, options);
                    case 'Object':
                        return typeCast(value, {
                            type: 'Object'
                        }, options);
                    default:
                        return {
                            type: 'Just',
                            value: typesCast(value, [
                                {
                                    type: 'Undefined'
                                }, {
                                    type: 'Null'
                                }, {
                                    type: 'NaN'
                                }, {
                                    type: 'Boolean'
                                }, {
                                    type: 'Number'
                                }, {
                                    type: 'Date'
                                }, {
                                    type: 'RegExp'
                                }, {
                                    type: 'Array'
                                }, {
                                    type: 'Object'
                                }, {
                                    type: 'String'
                                }
                            ], (options.explicit = true, options))
                        };
                }
            },
            Undefined: function (it) {
                if (it === 'undefined' || it === void 8) {
                    return {
                        type: 'Just',
                        value: void 8
                    };
                }
                else {
                    return {
                        type: 'Nothing'
                    };
                }
            },
            Null: function (it) {
                if (it === 'null') {
                    return {
                        type: 'Just',
                        value: null
                    };
                }
                else {
                    return {
                        type: 'Nothing'
                    };
                }
            },
            NaN: function (it) {
                if (it === 'NaN') {
                    return {
                        type: 'Just',
                        value: NaN
                    };
                }
                else {
                    return {
                        type: 'Nothing'
                    };
                }
            },
            Boolean: function (it) {
                if (it === 'true') {
                    return {
                        type: 'Just',
                        value: true
                    };
                }
                else if (it === 'false') {
                    return {
                        type: 'Just',
                        value: false
                    };
                }
                else {
                    return {
                        type: 'Nothing'
                    };
                }
            },
            Number: function (it) {
                return {
                    type: 'Just',
                    value: +it
                };
            },
            Int: function (it) {
                return {
                    type: 'Just',
                    value: +it
                };
            },
            Float: function (it) {
                return {
                    type: 'Just',
                    value: +it
                };
            },
            Date: function (value, options) {
                var that;
                if (that = /^\#([\s\S]*)\#$/.exec(value)) {
                    return {
                        type: 'Just',
                        value: new Date(+that[1] || that[1])
                    };
                }
                else if (options.explicit) {
                    return {
                        type: 'Nothing'
                    };
                }
                else {
                    return {
                        type: 'Just',
                        value: new Date(+value || value)
                    };
                }
            },
            RegExp: function (value, options) {
                var that;
                if (that = /^\/([\s\S]*)\/([gimy]*)$/.exec(value)) {
                    return {
                        type: 'Just',
                        value: new RegExp(that[1], that[2])
                    };
                }
                else if (options.explicit) {
                    return {
                        type: 'Nothing'
                    };
                }
                else {
                    return {
                        type: 'Just',
                        value: new RegExp(value)
                    };
                }
            },
            Array: function (value, options) {
                return castArray(value, {
                    of: [{
                            type: '*'
                        }]
                }, options);
            },
            Object: function (value, options) {
                return castFields(value, {
                    of: {}
                }, options);
            },
            String: function (it) {
                var that;
                if (toString$.call(it).slice(8, -1) !== 'String') {
                    return {
                        type: 'Nothing'
                    };
                }
                if (that = it.match(/^'([\s\S]*)'$/)) {
                    return {
                        type: 'Just',
                        value: that[1].replace(/\\'/g, "'")
                    };
                }
                else if (that = it.match(/^"([\s\S]*)"$/)) {
                    return {
                        type: 'Just',
                        value: that[1].replace(/\\"/g, '"')
                    };
                }
                else {
                    return {
                        type: 'Just',
                        value: it
                    };
                }
            }
        };
        function castArray(node, type, options) {
            var typeOf, element;
            if (toString$.call(node).slice(8, -1) !== 'Array') {
                return {
                    type: 'Nothing'
                };
            }
            typeOf = type.of;
            return {
                type: 'Just',
                value: (function () {
                    var i$, ref$, len$, results$ = [];
                    for (i$ = 0, len$ = (ref$ = node).length; i$ < len$; ++i$) {
                        element = ref$[i$];
                        results$.push(typesCast(element, typeOf, options));
                    }
                    return results$;
                }())
            };
        }
        function castTuple(node, type, options) {
            var result, i, i$, ref$, len$, types, cast;
            if (toString$.call(node).slice(8, -1) !== 'Array') {
                return {
                    type: 'Nothing'
                };
            }
            result = [];
            i = 0;
            for (i$ = 0, len$ = (ref$ = type.of).length; i$ < len$; ++i$) {
                types = ref$[i$];
                cast = typesCast(node[i], types, options);
                if (toString$.call(cast).slice(8, -1) !== 'Undefined') {
                    result.push(cast);
                }
                i++;
            }
            if (node.length <= i) {
                return {
                    type: 'Just',
                    value: result
                };
            }
            else {
                return {
                    type: 'Nothing'
                };
            }
        }
        function castFields(node, type, options) {
            var typeOf, key, value;
            if (toString$.call(node).slice(8, -1) !== 'Object') {
                return {
                    type: 'Nothing'
                };
            }
            typeOf = type.of;
            return {
                type: 'Just',
                value: (function () {
                    var ref$, resultObj$ = {};
                    for (key in ref$ = node) {
                        value = ref$[key];
                        resultObj$[typesCast(key, [{
                                type: 'String'
                            }], options)] = typesCast(value, typeOf[key] || [{
                                type: '*'
                            }], options);
                    }
                    return resultObj$;
                }())
            };
        }
        function typeCast(node, typeObj, options) {
            var type, structure, castFunc, ref$;
            type = typeObj.type, structure = typeObj.structure;
            if (type) {
                castFunc = ((ref$ = options.customTypes[type]) != null ? ref$.cast : void 8) || types[type];
                if (!castFunc) {
                    throw new Error("Type not defined: " + type + ".");
                }
                return castFunc(node, options, typesCast);
            }
            else {
                switch (structure) {
                    case 'array':
                        return castArray(node, typeObj, options);
                    case 'tuple':
                        return castTuple(node, typeObj, options);
                    case 'fields':
                        return castFields(node, typeObj, options);
                }
            }
        }
        function typesCast(node, types, options) {
            var i$, len$, type, ref$, valueType, value;
            for (i$ = 0, len$ = types.length; i$ < len$; ++i$) {
                type = types[i$];
                ref$ = typeCast(node, type, options), valueType = ref$.type, value = ref$.value;
                if (valueType === 'Nothing') {
                    continue;
                }
                if (parsedTypeCheck([type], value, {
                    customTypes: options.customTypes
                })) {
                    return value;
                }
            }
            throw new Error("Value " + JSON.stringify(node) + " does not type check against " + JSON.stringify(types) + ".");
        }
        module.exports = typesCast;
    }).call(this);
},
/* type-check/lib/index.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    (function () {
        var VERSION, parseType, parsedTypeCheck, typeCheck;
        VERSION = '0.3.2';
        parseType = require(505) /* ./parse-type */;
        parsedTypeCheck = require(506) /* ./check */;
        typeCheck = function (type, input, options) {
            return parsedTypeCheck(parseType(type), input, options);
        };
        module.exports = {
            VERSION: VERSION,
            typeCheck: typeCheck,
            parsedTypeCheck: parsedTypeCheck,
            parseType: parseType
        };
    }).call(this);
},
/* type-check/lib/parse-type.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    (function () {
        var identifierRegex, tokenRegex;
        identifierRegex = /[\$\w]+/;
        function peek(tokens) {
            var token;
            token = tokens[0];
            if (token == null) {
                throw new Error('Unexpected end of input.');
            }
            return token;
        }
        function consumeIdent(tokens) {
            var token;
            token = peek(tokens);
            if (!identifierRegex.test(token)) {
                throw new Error("Expected text, got '" + token + "' instead.");
            }
            return tokens.shift();
        }
        function consumeOp(tokens, op) {
            var token;
            token = peek(tokens);
            if (token !== op) {
                throw new Error("Expected '" + op + "', got '" + token + "' instead.");
            }
            return tokens.shift();
        }
        function maybeConsumeOp(tokens, op) {
            var token;
            token = tokens[0];
            if (token === op) {
                return tokens.shift();
            }
            else {
                return null;
            }
        }
        function consumeArray(tokens) {
            var types;
            consumeOp(tokens, '[');
            if (peek(tokens) === ']') {
                throw new Error("Must specify type of Array - eg. [Type], got [] instead.");
            }
            types = consumeTypes(tokens);
            consumeOp(tokens, ']');
            return {
                structure: 'array',
                of: types
            };
        }
        function consumeTuple(tokens) {
            var components;
            components = [];
            consumeOp(tokens, '(');
            if (peek(tokens) === ')') {
                throw new Error("Tuple must be of at least length 1 - eg. (Type), got () instead.");
            }
            for (;;) {
                components.push(consumeTypes(tokens));
                maybeConsumeOp(tokens, ',');
                if (')' === peek(tokens)) {
                    break;
                }
            }
            consumeOp(tokens, ')');
            return {
                structure: 'tuple',
                of: components
            };
        }
        function consumeFields(tokens) {
            var fields, subset, ref$, key, types;
            fields = {};
            consumeOp(tokens, '{');
            subset = false;
            for (;;) {
                if (maybeConsumeOp(tokens, '...')) {
                    subset = true;
                    break;
                }
                ref$ = consumeField(tokens), key = ref$[0], types = ref$[1];
                fields[key] = types;
                maybeConsumeOp(tokens, ',');
                if ('}' === peek(tokens)) {
                    break;
                }
            }
            consumeOp(tokens, '}');
            return {
                structure: 'fields',
                of: fields,
                subset: subset
            };
        }
        function consumeField(tokens) {
            var key, types;
            key = consumeIdent(tokens);
            consumeOp(tokens, ':');
            types = consumeTypes(tokens);
            return [key, types];
        }
        function maybeConsumeStructure(tokens) {
            switch (tokens[0]) {
                case '[':
                    return consumeArray(tokens);
                case '(':
                    return consumeTuple(tokens);
                case '{':
                    return consumeFields(tokens);
            }
        }
        function consumeType(tokens) {
            var token, wildcard, type, structure;
            token = peek(tokens);
            wildcard = token === '*';
            if (wildcard || identifierRegex.test(token)) {
                type = wildcard
                    ? consumeOp(tokens, '*')
                    : consumeIdent(tokens);
                structure = maybeConsumeStructure(tokens);
                if (structure) {
                    return structure.type = type, structure;
                }
                else {
                    return {
                        type: type
                    };
                }
            }
            else {
                structure = maybeConsumeStructure(tokens);
                if (!structure) {
                    throw new Error("Unexpected character: " + token);
                }
                return structure;
            }
        }
        function consumeTypes(tokens) {
            var lookahead, types, typesSoFar, typeObj, type;
            if ('::' === peek(tokens)) {
                throw new Error("No comment before comment separator '::' found.");
            }
            lookahead = tokens[1];
            if (lookahead != null && lookahead === '::') {
                tokens.shift();
                tokens.shift();
            }
            types = [];
            typesSoFar = {};
            if ('Maybe' === peek(tokens)) {
                tokens.shift();
                types = [
                    {
                        type: 'Undefined'
                    }, {
                        type: 'Null'
                    }
                ];
                typesSoFar = {
                    Undefined: true,
                    Null: true
                };
            }
            for (;;) {
                typeObj = consumeType(tokens), type = typeObj.type;
                if (!typesSoFar[type]) {
                    types.push(typeObj);
                }
                typesSoFar[type] = true;
                if (!maybeConsumeOp(tokens, '|')) {
                    break;
                }
            }
            return types;
        }
        tokenRegex = RegExp('\\.\\.\\.|::|->|' + identifierRegex.source + '|\\S', 'g');
        module.exports = function (input) {
            var tokens, e;
            if (!input.length) {
                throw new Error('No type specified.');
            }
            tokens = input.match(tokenRegex) || [];
            if (in$('->', tokens)) {
                throw new Error("Function types are not supported.\ To validate that something is a function, you may use 'Function'.");
            }
            try {
                return consumeTypes(tokens);
            }
            catch (e$) {
                e = e$;
                throw new Error(e.message + " - Remaining tokens: " + JSON.stringify(tokens) + " - Initial input: '" + input + "'");
            }
        };
        function in$(x, xs) {
            var i = -1, l = xs.length >>> 0;
            while (++i < l)
                if (x === xs[i])
                    return true;
            return false;
        }
    }).call(this);
},
/* type-check/lib/check.js */ function _(require, module, exports) {
    // Generated by LiveScript 1.4.0
    (function () {
        var ref$, any, all, isItNaN, types, defaultType, customTypes, toString$ = {}.toString;
        ref$ = require(497) /* prelude-ls */, any = ref$.any, all = ref$.all, isItNaN = ref$.isItNaN;
        types = {
            Number: {
                typeOf: 'Number',
                validate: function (it) {
                    return !isItNaN(it);
                }
            },
            NaN: {
                typeOf: 'Number',
                validate: isItNaN
            },
            Int: {
                typeOf: 'Number',
                validate: function (it) {
                    return !isItNaN(it) && it % 1 === 0;
                }
            },
            Float: {
                typeOf: 'Number',
                validate: function (it) {
                    return !isItNaN(it);
                }
            },
            Date: {
                typeOf: 'Date',
                validate: function (it) {
                    return !isItNaN(it.getTime());
                }
            }
        };
        defaultType = {
            array: 'Array',
            tuple: 'Array'
        };
        function checkArray(input, type) {
            return all(function (it) {
                return checkMultiple(it, type.of);
            }, input);
        }
        function checkTuple(input, type) {
            var i, i$, ref$, len$, types;
            i = 0;
            for (i$ = 0, len$ = (ref$ = type.of).length; i$ < len$; ++i$) {
                types = ref$[i$];
                if (!checkMultiple(input[i], types)) {
                    return false;
                }
                i++;
            }
            return input.length <= i;
        }
        function checkFields(input, type) {
            var inputKeys, numInputKeys, k, numOfKeys, key, ref$, types;
            inputKeys = {};
            numInputKeys = 0;
            for (k in input) {
                inputKeys[k] = true;
                numInputKeys++;
            }
            numOfKeys = 0;
            for (key in ref$ = type.of) {
                types = ref$[key];
                if (!checkMultiple(input[key], types)) {
                    return false;
                }
                if (inputKeys[key]) {
                    numOfKeys++;
                }
            }
            return type.subset || numInputKeys === numOfKeys;
        }
        function checkStructure(input, type) {
            if (!(input instanceof Object)) {
                return false;
            }
            switch (type.structure) {
                case 'fields':
                    return checkFields(input, type);
                case 'array':
                    return checkArray(input, type);
                case 'tuple':
                    return checkTuple(input, type);
            }
        }
        function check(input, typeObj) {
            var type, structure, setting, that;
            type = typeObj.type, structure = typeObj.structure;
            if (type) {
                if (type === '*') {
                    return true;
                }
                setting = customTypes[type] || types[type];
                if (setting) {
                    return setting.typeOf === toString$.call(input).slice(8, -1) && setting.validate(input);
                }
                else {
                    return type === toString$.call(input).slice(8, -1) && (!structure || checkStructure(input, typeObj));
                }
            }
            else if (structure) {
                if (that = defaultType[structure]) {
                    if (that !== toString$.call(input).slice(8, -1)) {
                        return false;
                    }
                }
                return checkStructure(input, typeObj);
            }
            else {
                throw new Error("No type defined. Input: " + input + ".");
            }
        }
        function checkMultiple(input, types) {
            if (toString$.call(types).slice(8, -1) !== 'Array') {
                throw new Error("Types must be in an array. Input: " + input + ".");
            }
            return any(function (it) {
                return check(input, it);
            }, types);
        }
        module.exports = function (parsedType, input, options) {
            options == null && (options = {});
            customTypes = options.customTypes || {};
            return checkMultiple(input, parsedType);
        };
    }).call(this);
},
/* eslint/lib/linter/node-event-generator.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const esquery = require(508) /* esquery */;
    const lodash = require(111) /* lodash */;
    //------------------------------------------------------------------------------
    // Typedefs
    //------------------------------------------------------------------------------
    /**
     * An object describing an AST selector
     * @typedef {Object} ASTSelector
     * @property {string} rawSelector The string that was parsed into this selector
     * @property {boolean} isExit `true` if this should be emitted when exiting the node rather than when entering
     * @property {Object} parsedSelector An object (from esquery) describing the matching behavior of the selector
     * @property {string[]|null} listenerTypes A list of node types that could possibly cause the selector to match,
     * or `null` if all node types could cause a match
     * @property {number} attributeCount The total number of classes, pseudo-classes, and attribute queries in this selector
     * @property {number} identifierCount The total number of identifier queries in this selector
     */
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Gets the possible types of a selector
     * @param {Object} parsedSelector An object (from esquery) describing the matching behavior of the selector
     * @returns {string[]|null} The node types that could possibly trigger this selector, or `null` if all node types could trigger it
     */
    function getPossibleTypes(parsedSelector) {
        switch (parsedSelector.type) {
            case "identifier":
                return [parsedSelector.value];
            case "matches": {
                const typesForComponents = parsedSelector.selectors.map(getPossibleTypes);
                if (typesForComponents.every(Boolean)) {
                    return lodash.union(...typesForComponents);
                }
                return null;
            }
            case "compound": {
                const typesForComponents = parsedSelector.selectors.map(getPossibleTypes).filter(typesForComponent => typesForComponent);
                // If all of the components could match any type, then the compound could also match any type.
                if (!typesForComponents.length) {
                    return null;
                }
                /*
                 * If at least one of the components could only match a particular type, the compound could only match
                 * the intersection of those types.
                 */
                return lodash.intersection(...typesForComponents);
            }
            case "child":
            case "descendant":
            case "sibling":
            case "adjacent":
                return getPossibleTypes(parsedSelector.right);
            default:
                return null;
        }
    }
    /**
     * Counts the number of class, pseudo-class, and attribute queries in this selector
     * @param {Object} parsedSelector An object (from esquery) describing the selector's matching behavior
     * @returns {number} The number of class, pseudo-class, and attribute queries in this selector
     */
    function countClassAttributes(parsedSelector) {
        switch (parsedSelector.type) {
            case "child":
            case "descendant":
            case "sibling":
            case "adjacent":
                return countClassAttributes(parsedSelector.left) + countClassAttributes(parsedSelector.right);
            case "compound":
            case "not":
            case "matches":
                return parsedSelector.selectors.reduce((sum, childSelector) => sum + countClassAttributes(childSelector), 0);
            case "attribute":
            case "field":
            case "nth-child":
            case "nth-last-child":
                return 1;
            default:
                return 0;
        }
    }
    /**
     * Counts the number of identifier queries in this selector
     * @param {Object} parsedSelector An object (from esquery) describing the selector's matching behavior
     * @returns {number} The number of identifier queries
     */
    function countIdentifiers(parsedSelector) {
        switch (parsedSelector.type) {
            case "child":
            case "descendant":
            case "sibling":
            case "adjacent":
                return countIdentifiers(parsedSelector.left) + countIdentifiers(parsedSelector.right);
            case "compound":
            case "not":
            case "matches":
                return parsedSelector.selectors.reduce((sum, childSelector) => sum + countIdentifiers(childSelector), 0);
            case "identifier":
                return 1;
            default:
                return 0;
        }
    }
    /**
     * Compares the specificity of two selector objects, with CSS-like rules.
     * @param {ASTSelector} selectorA An AST selector descriptor
     * @param {ASTSelector} selectorB Another AST selector descriptor
     * @returns {number}
     * a value less than 0 if selectorA is less specific than selectorB
     * a value greater than 0 if selectorA is more specific than selectorB
     * a value less than 0 if selectorA and selectorB have the same specificity, and selectorA <= selectorB alphabetically
     * a value greater than 0 if selectorA and selectorB have the same specificity, and selectorA > selectorB alphabetically
     */
    function compareSpecificity(selectorA, selectorB) {
        return selectorA.attributeCount - selectorB.attributeCount ||
            selectorA.identifierCount - selectorB.identifierCount ||
            (selectorA.rawSelector <= selectorB.rawSelector ? -1 : 1);
    }
    /**
     * Parses a raw selector string, and throws a useful error if parsing fails.
     * @param {string} rawSelector A raw AST selector
     * @returns {Object} An object (from esquery) describing the matching behavior of this selector
     * @throws {Error} An error if the selector is invalid
     */
    function tryParseSelector(rawSelector) {
        try {
            return esquery.parse(rawSelector.replace(/:exit$/u, ""));
        }
        catch (err) {
            if (typeof err.offset === "number") {
                throw new SyntaxError(`Syntax error in selector "${rawSelector}" at position ${err.offset}: ${err.message}`);
            }
            throw err;
        }
    }
    /**
     * Parses a raw selector string, and returns the parsed selector along with specificity and type information.
     * @param {string} rawSelector A raw AST selector
     * @returns {ASTSelector} A selector descriptor
     */
    const parseSelector = lodash.memoize(rawSelector => {
        const parsedSelector = tryParseSelector(rawSelector);
        return {
            rawSelector,
            isExit: rawSelector.endsWith(":exit"),
            parsedSelector,
            listenerTypes: getPossibleTypes(parsedSelector),
            attributeCount: countClassAttributes(parsedSelector),
            identifierCount: countIdentifiers(parsedSelector)
        };
    });
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * The event generator for AST nodes.
     * This implements below interface.
     *
     * ```ts
     * interface EventGenerator {
     *     emitter: SafeEmitter;
     *     enterNode(node: ASTNode): void;
     *     leaveNode(node: ASTNode): void;
     * }
     * ```
     */
    class NodeEventGenerator {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {SafeEmitter} emitter
         * An SafeEmitter which is the destination of events. This emitter must already
         * have registered listeners for all of the events that it needs to listen for.
         * (See lib/linter/safe-emitter.js for more details on `SafeEmitter`.)
         * @returns {NodeEventGenerator} new instance
         */
        constructor(emitter) {
            this.emitter = emitter;
            this.currentAncestry = [];
            this.enterSelectorsByNodeType = new Map();
            this.exitSelectorsByNodeType = new Map();
            this.anyTypeEnterSelectors = [];
            this.anyTypeExitSelectors = [];
            emitter.eventNames().forEach(rawSelector => {
                const selector = parseSelector(rawSelector);
                if (selector.listenerTypes) {
                    const typeMap = selector.isExit ? this.exitSelectorsByNodeType : this.enterSelectorsByNodeType;
                    selector.listenerTypes.forEach(nodeType => {
                        if (!typeMap.has(nodeType)) {
                            typeMap.set(nodeType, []);
                        }
                        typeMap.get(nodeType).push(selector);
                    });
                    return;
                }
                const selectors = selector.isExit ? this.anyTypeExitSelectors : this.anyTypeEnterSelectors;
                selectors.push(selector);
            });
            this.anyTypeEnterSelectors.sort(compareSpecificity);
            this.anyTypeExitSelectors.sort(compareSpecificity);
            this.enterSelectorsByNodeType.forEach(selectorList => selectorList.sort(compareSpecificity));
            this.exitSelectorsByNodeType.forEach(selectorList => selectorList.sort(compareSpecificity));
        }
        /**
         * Checks a selector against a node, and emits it if it matches
         * @param {ASTNode} node The node to check
         * @param {ASTSelector} selector An AST selector descriptor
         * @returns {void}
         */
        applySelector(node, selector) {
            if (esquery.matches(node, selector.parsedSelector, this.currentAncestry)) {
                this.emitter.emit(selector.rawSelector, node);
            }
        }
        /**
         * Applies all appropriate selectors to a node, in specificity order
         * @param {ASTNode} node The node to check
         * @param {boolean} isExit `false` if the node is currently being entered, `true` if it's currently being exited
         * @returns {void}
         */
        applySelectors(node, isExit) {
            const selectorsByNodeType = (isExit ? this.exitSelectorsByNodeType : this.enterSelectorsByNodeType).get(node.type) || [];
            const anyTypeSelectors = isExit ? this.anyTypeExitSelectors : this.anyTypeEnterSelectors;
            /*
             * selectorsByNodeType and anyTypeSelectors were already sorted by specificity in the constructor.
             * Iterate through each of them, applying selectors in the right order.
             */
            let selectorsByTypeIndex = 0;
            let anyTypeSelectorsIndex = 0;
            while (selectorsByTypeIndex < selectorsByNodeType.length || anyTypeSelectorsIndex < anyTypeSelectors.length) {
                if (selectorsByTypeIndex >= selectorsByNodeType.length ||
                    anyTypeSelectorsIndex < anyTypeSelectors.length &&
                        compareSpecificity(anyTypeSelectors[anyTypeSelectorsIndex], selectorsByNodeType[selectorsByTypeIndex]) < 0) {
                    this.applySelector(node, anyTypeSelectors[anyTypeSelectorsIndex++]);
                }
                else {
                    this.applySelector(node, selectorsByNodeType[selectorsByTypeIndex++]);
                }
            }
        }
        /**
         * Emits an event of entering AST node.
         * @param {ASTNode} node A node which was entered.
         * @returns {void}
         */
        enterNode(node) {
            if (node.parent) {
                this.currentAncestry.unshift(node.parent);
            }
            this.applySelectors(node, false);
        }
        /**
         * Emits an event of leaving AST node.
         * @param {ASTNode} node A node which was left.
         * @returns {void}
         */
        leaveNode(node) {
            this.applySelectors(node, true);
            this.currentAncestry.shift();
        }
    }
    module.exports = NodeEventGenerator;
},
/* esquery/esquery.js */ function _(require, module, exports) {
    /* vim: set sw=4 sts=4 : */
    (function () {
        var estraverse = require(90) /* estraverse */;
        var parser = require(509) /* ./parser */;
        var isArray = Array.isArray || function isArray(array) {
            return {}.toString.call(array) === '[object Array]';
        };
        var LEFT_SIDE = {};
        var RIGHT_SIDE = {};
        function esqueryModule() {
            /**
             * Get the value of a property which may be multiple levels down in the object.
             */
            function getPath(obj, key) {
                var i, keys = key.split(".");
                for (i = 0; i < keys.length; i++) {
                    if (obj == null) {
                        return obj;
                    }
                    obj = obj[keys[i]];
                }
                return obj;
            }
            /**
             * Determine whether `node` can be reached by following `path`, starting at `ancestor`.
             */
            function inPath(node, ancestor, path) {
                var field, remainingPath, i;
                if (path.length === 0) {
                    return node === ancestor;
                }
                if (ancestor == null) {
                    return false;
                }
                field = ancestor[path[0]];
                remainingPath = path.slice(1);
                if (isArray(field)) {
                    for (i = 0, l = field.length; i < l; ++i) {
                        if (inPath(node, field[i], remainingPath)) {
                            return true;
                        }
                    }
                    return false;
                }
                else {
                    return inPath(node, field, remainingPath);
                }
            }
            /**
             * Given a `node` and its ancestors, determine if `node` is matched by `selector`.
             */
            function matches(node, selector, ancestry) {
                var path, ancestor, i, l, p;
                if (!selector) {
                    return true;
                }
                if (!node) {
                    return false;
                }
                if (!ancestry) {
                    ancestry = [];
                }
                switch (selector.type) {
                    case 'wildcard':
                        return true;
                    case 'identifier':
                        return selector.value.toLowerCase() === node.type.toLowerCase();
                    case 'field':
                        path = selector.name.split('.');
                        ancestor = ancestry[path.length - 1];
                        return inPath(node, ancestor, path);
                    case 'matches':
                        for (i = 0, l = selector.selectors.length; i < l; ++i) {
                            if (matches(node, selector.selectors[i], ancestry)) {
                                return true;
                            }
                        }
                        return false;
                    case 'compound':
                        for (i = 0, l = selector.selectors.length; i < l; ++i) {
                            if (!matches(node, selector.selectors[i], ancestry)) {
                                return false;
                            }
                        }
                        return true;
                    case 'not':
                        for (i = 0, l = selector.selectors.length; i < l; ++i) {
                            if (matches(node, selector.selectors[i], ancestry)) {
                                return false;
                            }
                        }
                        return true;
                    case 'has':
                        var a, collector = [];
                        for (i = 0, l = selector.selectors.length; i < l; ++i) {
                            a = [];
                            estraverse.traverse(node, {
                                enter: function (node, parent) {
                                    if (parent != null) {
                                        a.unshift(parent);
                                    }
                                    if (matches(node, selector.selectors[i], a)) {
                                        collector.push(node);
                                    }
                                },
                                leave: function () { a.shift(); },
                                fallback: 'iteration'
                            });
                        }
                        return collector.length !== 0;
                    case 'child':
                        if (matches(node, selector.right, ancestry)) {
                            return matches(ancestry[0], selector.left, ancestry.slice(1));
                        }
                        return false;
                    case 'descendant':
                        if (matches(node, selector.right, ancestry)) {
                            for (i = 0, l = ancestry.length; i < l; ++i) {
                                if (matches(ancestry[i], selector.left, ancestry.slice(i + 1))) {
                                    return true;
                                }
                            }
                        }
                        return false;
                    case 'attribute':
                        p = getPath(node, selector.name);
                        switch (selector.operator) {
                            case null:
                            case void 0:
                                return p != null;
                            case '=':
                                switch (selector.value.type) {
                                    case 'regexp': return typeof p === 'string' && selector.value.value.test(p);
                                    case 'literal': return '' + selector.value.value === '' + p;
                                    case 'type': return selector.value.value === typeof p;
                                }
                            case '!=':
                                switch (selector.value.type) {
                                    case 'regexp': return !selector.value.value.test(p);
                                    case 'literal': return '' + selector.value.value !== '' + p;
                                    case 'type': return selector.value.value !== typeof p;
                                }
                            case '<=': return p <= selector.value.value;
                            case '<': return p < selector.value.value;
                            case '>': return p > selector.value.value;
                            case '>=': return p >= selector.value.value;
                        }
                    case 'sibling':
                        return matches(node, selector.right, ancestry) &&
                            sibling(node, selector.left, ancestry, LEFT_SIDE) ||
                            selector.left.subject &&
                                matches(node, selector.left, ancestry) &&
                                sibling(node, selector.right, ancestry, RIGHT_SIDE);
                    case 'adjacent':
                        return matches(node, selector.right, ancestry) &&
                            adjacent(node, selector.left, ancestry, LEFT_SIDE) ||
                            selector.right.subject &&
                                matches(node, selector.left, ancestry) &&
                                adjacent(node, selector.right, ancestry, RIGHT_SIDE);
                    case 'nth-child':
                        return matches(node, selector.right, ancestry) &&
                            nthChild(node, ancestry, function (length) {
                                return selector.index.value - 1;
                            });
                    case 'nth-last-child':
                        return matches(node, selector.right, ancestry) &&
                            nthChild(node, ancestry, function (length) {
                                return length - selector.index.value;
                            });
                    case 'class':
                        if (!node.type)
                            return false;
                        switch (selector.name.toLowerCase()) {
                            case 'statement':
                                if (node.type.slice(-9) === 'Statement')
                                    return true;
                            // fallthrough: interface Declaration <: Statement { }
                            case 'declaration':
                                return node.type.slice(-11) === 'Declaration';
                            case 'pattern':
                                if (node.type.slice(-7) === 'Pattern')
                                    return true;
                            // fallthrough: interface Expression <: Node, Pattern { }
                            case 'expression':
                                return node.type.slice(-10) === 'Expression' ||
                                    node.type.slice(-7) === 'Literal' ||
                                    (node.type === 'Identifier' &&
                                        (ancestry.length === 0 || ancestry[0].type !== 'MetaProperty')) ||
                                    node.type === 'MetaProperty';
                            case 'function':
                                return node.type.slice(0, 8) === 'Function' ||
                                    node.type === 'ArrowFunctionExpression';
                        }
                        throw new Error('Unknown class name: ' + selector.name);
                }
                throw new Error('Unknown selector type: ' + selector.type);
            }
            /*
             * Determines if the given node has a sibling that matches the given selector.
             */
            function sibling(node, selector, ancestry, side) {
                var parent = ancestry[0], listProp, startIndex, keys, i, l, k, lowerBound, upperBound;
                if (!parent) {
                    return false;
                }
                keys = estraverse.VisitorKeys[parent.type];
                for (i = 0, l = keys.length; i < l; ++i) {
                    listProp = parent[keys[i]];
                    if (isArray(listProp)) {
                        startIndex = listProp.indexOf(node);
                        if (startIndex < 0) {
                            continue;
                        }
                        if (side === LEFT_SIDE) {
                            lowerBound = 0;
                            upperBound = startIndex;
                        }
                        else {
                            lowerBound = startIndex + 1;
                            upperBound = listProp.length;
                        }
                        for (k = lowerBound; k < upperBound; ++k) {
                            if (matches(listProp[k], selector, ancestry)) {
                                return true;
                            }
                        }
                    }
                }
                return false;
            }
            /*
             * Determines if the given node has an adjacent sibling that matches the given selector.
             */
            function adjacent(node, selector, ancestry, side) {
                var parent = ancestry[0], listProp, keys, i, l, idx;
                if (!parent) {
                    return false;
                }
                keys = estraverse.VisitorKeys[parent.type];
                for (i = 0, l = keys.length; i < l; ++i) {
                    listProp = parent[keys[i]];
                    if (isArray(listProp)) {
                        idx = listProp.indexOf(node);
                        if (idx < 0) {
                            continue;
                        }
                        if (side === LEFT_SIDE && idx > 0 && matches(listProp[idx - 1], selector, ancestry)) {
                            return true;
                        }
                        if (side === RIGHT_SIDE && idx < listProp.length - 1 && matches(listProp[idx + 1], selector, ancestry)) {
                            return true;
                        }
                    }
                }
                return false;
            }
            /*
             * Determines if the given node is the nth child, determined by idxFn, which is given the containing list's length.
             */
            function nthChild(node, ancestry, idxFn) {
                var parent = ancestry[0], listProp, keys, i, l, idx;
                if (!parent) {
                    return false;
                }
                keys = estraverse.VisitorKeys[parent.type];
                for (i = 0, l = keys.length; i < l; ++i) {
                    listProp = parent[keys[i]];
                    if (isArray(listProp)) {
                        idx = listProp.indexOf(node);
                        if (idx >= 0 && idx === idxFn(listProp.length)) {
                            return true;
                        }
                    }
                }
                return false;
            }
            /*
             * For each selector node marked as a subject, find the portion of the selector that the subject must match.
             */
            function subjects(selector, ancestor) {
                var results, p;
                if (selector == null || typeof selector != 'object') {
                    return [];
                }
                if (ancestor == null) {
                    ancestor = selector;
                }
                results = selector.subject ? [ancestor] : [];
                for (p in selector) {
                    if (!{}.hasOwnProperty.call(selector, p)) {
                        continue;
                    }
                    [].push.apply(results, subjects(selector[p], p === 'left' ? selector[p] : ancestor));
                }
                return results;
            }
            /**
             * From a JS AST and a selector AST, collect all JS AST nodes that match the selector.
             */
            function match(ast, selector) {
                var ancestry = [], results = [], altSubjects, i, l, k, m;
                if (!selector) {
                    return results;
                }
                altSubjects = subjects(selector);
                estraverse.traverse(ast, {
                    enter: function (node, parent) {
                        if (parent != null) {
                            ancestry.unshift(parent);
                        }
                        if (matches(node, selector, ancestry)) {
                            if (altSubjects.length) {
                                for (i = 0, l = altSubjects.length; i < l; ++i) {
                                    if (matches(node, altSubjects[i], ancestry)) {
                                        results.push(node);
                                    }
                                    for (k = 0, m = ancestry.length; k < m; ++k) {
                                        if (matches(ancestry[k], altSubjects[i], ancestry.slice(k + 1))) {
                                            results.push(ancestry[k]);
                                        }
                                    }
                                }
                            }
                            else {
                                results.push(node);
                            }
                        }
                    },
                    leave: function () { ancestry.shift(); },
                    fallback: 'iteration'
                });
                return results;
            }
            /**
             * Parse a selector string and return its AST.
             */
            function parse(selector) {
                return parser.parse(selector);
            }
            /**
             * Query the code AST using the selector string.
             */
            function query(ast, selector) {
                return match(ast, parse(selector));
            }
            query.parse = parse;
            query.match = match;
            query.matches = matches;
            return query.query = query;
        }
        if (typeof define === "function" && define.amd) {
            define(esqueryModule);
        }
        else if (typeof module !== 'undefined' && module.exports) {
            module.exports = esqueryModule();
        }
        else {
            this.esquery = esqueryModule();
        }
    })();
},
/* esquery/parser.js */ function _(require, module, exports) {
    var result = (function () {
        /*
         * Generated by PEG.js 0.7.0.
         *
         * http://pegjs.majda.cz/
         */
        function quote(s) {
            /*
             * ECMA-262, 5th ed., 7.8.4: All characters may appear literally in a
             * string literal except for the closing quote character, backslash,
             * carriage return, line separator, paragraph separator, and line feed.
             * Any character may appear in the form of an escape sequence.
             *
             * For portability, we also escape escape all control and non-ASCII
             * characters. Note that "\0" and "\v" escape sequences are not used
             * because JSHint does not like the first and IE the second.
             */
            return '"' + s
                .replace(/\\/g, '\\\\') // backslash
                .replace(/"/g, '\\"') // closing quote character
                .replace(/\x08/g, '\\b') // backspace
                .replace(/\t/g, '\\t') // horizontal tab
                .replace(/\n/g, '\\n') // line feed
                .replace(/\f/g, '\\f') // form feed
                .replace(/\r/g, '\\r') // carriage return
                .replace(/[\x00-\x07\x0B\x0E-\x1F\x80-\uFFFF]/g, escape)
                + '"';
        }
        var result = {
            /*
             * Parses the input with a generated parser. If the parsing is successful,
             * returns a value explicitly or implicitly specified by the grammar from
             * which the parser was generated (see |PEG.buildParser|). If the parsing is
             * unsuccessful, throws |PEG.parser.SyntaxError| describing the error.
             */
            parse: function (input, startRule) {
                var parseFunctions = {
                    "start": parse_start,
                    "_": parse__,
                    "identifierName": parse_identifierName,
                    "binaryOp": parse_binaryOp,
                    "selectors": parse_selectors,
                    "selector": parse_selector,
                    "sequence": parse_sequence,
                    "atom": parse_atom,
                    "wildcard": parse_wildcard,
                    "identifier": parse_identifier,
                    "attr": parse_attr,
                    "attrOps": parse_attrOps,
                    "attrEqOps": parse_attrEqOps,
                    "attrName": parse_attrName,
                    "attrValue": parse_attrValue,
                    "string": parse_string,
                    "number": parse_number,
                    "path": parse_path,
                    "type": parse_type,
                    "regex": parse_regex,
                    "field": parse_field,
                    "negation": parse_negation,
                    "matches": parse_matches,
                    "has": parse_has,
                    "firstChild": parse_firstChild,
                    "lastChild": parse_lastChild,
                    "nthChild": parse_nthChild,
                    "nthLastChild": parse_nthLastChild,
                    "class": parse_class
                };
                if (startRule !== undefined) {
                    if (parseFunctions[startRule] === undefined) {
                        throw new Error("Invalid rule name: " + quote(startRule) + ".");
                    }
                }
                else {
                    startRule = "start";
                }
                var pos = 0;
                var reportFailures = 0;
                var rightmostFailuresPos = 0;
                var rightmostFailuresExpected = [];
                var cache = {};
                function padLeft(input, padding, length) {
                    var result = input;
                    var padLength = length - input.length;
                    for (var i = 0; i < padLength; i++) {
                        result = padding + result;
                    }
                    return result;
                }
                function escape(ch) {
                    var charCode = ch.charCodeAt(0);
                    var escapeChar;
                    var length;
                    if (charCode <= 0xFF) {
                        escapeChar = 'x';
                        length = 2;
                    }
                    else {
                        escapeChar = 'u';
                        length = 4;
                    }
                    return '\\' + escapeChar + padLeft(charCode.toString(16).toUpperCase(), '0', length);
                }
                function matchFailed(failure) {
                    if (pos < rightmostFailuresPos) {
                        return;
                    }
                    if (pos > rightmostFailuresPos) {
                        rightmostFailuresPos = pos;
                        rightmostFailuresExpected = [];
                    }
                    rightmostFailuresExpected.push(failure);
                }
                function parse_start() {
                    var cacheKey = "start@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    result0 = parse__();
                    if (result0 !== null) {
                        result1 = parse_selectors();
                        if (result1 !== null) {
                            result2 = parse__();
                            if (result2 !== null) {
                                result0 = [result0, result1, result2];
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, ss) { return ss.length === 1 ? ss[0] : { type: 'matches', selectors: ss }; })(pos0, result0[1]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    if (result0 === null) {
                        pos0 = pos;
                        result0 = parse__();
                        if (result0 !== null) {
                            result0 = (function (offset) { return void 0; })(pos0);
                        }
                        if (result0 === null) {
                            pos = pos0;
                        }
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse__() {
                    var cacheKey = "_@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1;
                    result0 = [];
                    if (input.charCodeAt(pos) === 32) {
                        result1 = " ";
                        pos++;
                    }
                    else {
                        result1 = null;
                        if (reportFailures === 0) {
                            matchFailed("\" \"");
                        }
                    }
                    while (result1 !== null) {
                        result0.push(result1);
                        if (input.charCodeAt(pos) === 32) {
                            result1 = " ";
                            pos++;
                        }
                        else {
                            result1 = null;
                            if (reportFailures === 0) {
                                matchFailed("\" \"");
                            }
                        }
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_identifierName() {
                    var cacheKey = "identifierName@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1;
                    var pos0;
                    pos0 = pos;
                    if (/^[^ [\],():#!=><~+.]/.test(input.charAt(pos))) {
                        result1 = input.charAt(pos);
                        pos++;
                    }
                    else {
                        result1 = null;
                        if (reportFailures === 0) {
                            matchFailed("[^ [\\],():#!=><~+.]");
                        }
                    }
                    if (result1 !== null) {
                        result0 = [];
                        while (result1 !== null) {
                            result0.push(result1);
                            if (/^[^ [\],():#!=><~+.]/.test(input.charAt(pos))) {
                                result1 = input.charAt(pos);
                                pos++;
                            }
                            else {
                                result1 = null;
                                if (reportFailures === 0) {
                                    matchFailed("[^ [\\],():#!=><~+.]");
                                }
                            }
                        }
                    }
                    else {
                        result0 = null;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, i) { return i.join(''); })(pos0, result0);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_binaryOp() {
                    var cacheKey = "binaryOp@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    result0 = parse__();
                    if (result0 !== null) {
                        if (input.charCodeAt(pos) === 62) {
                            result1 = ">";
                            pos++;
                        }
                        else {
                            result1 = null;
                            if (reportFailures === 0) {
                                matchFailed("\">\"");
                            }
                        }
                        if (result1 !== null) {
                            result2 = parse__();
                            if (result2 !== null) {
                                result0 = [result0, result1, result2];
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset) { return 'child'; })(pos0);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    if (result0 === null) {
                        pos0 = pos;
                        pos1 = pos;
                        result0 = parse__();
                        if (result0 !== null) {
                            if (input.charCodeAt(pos) === 126) {
                                result1 = "~";
                                pos++;
                            }
                            else {
                                result1 = null;
                                if (reportFailures === 0) {
                                    matchFailed("\"~\"");
                                }
                            }
                            if (result1 !== null) {
                                result2 = parse__();
                                if (result2 !== null) {
                                    result0 = [result0, result1, result2];
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                        if (result0 !== null) {
                            result0 = (function (offset) { return 'sibling'; })(pos0);
                        }
                        if (result0 === null) {
                            pos = pos0;
                        }
                        if (result0 === null) {
                            pos0 = pos;
                            pos1 = pos;
                            result0 = parse__();
                            if (result0 !== null) {
                                if (input.charCodeAt(pos) === 43) {
                                    result1 = "+";
                                    pos++;
                                }
                                else {
                                    result1 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("\"+\"");
                                    }
                                }
                                if (result1 !== null) {
                                    result2 = parse__();
                                    if (result2 !== null) {
                                        result0 = [result0, result1, result2];
                                    }
                                    else {
                                        result0 = null;
                                        pos = pos1;
                                    }
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                            if (result0 !== null) {
                                result0 = (function (offset) { return 'adjacent'; })(pos0);
                            }
                            if (result0 === null) {
                                pos = pos0;
                            }
                            if (result0 === null) {
                                pos0 = pos;
                                pos1 = pos;
                                if (input.charCodeAt(pos) === 32) {
                                    result0 = " ";
                                    pos++;
                                }
                                else {
                                    result0 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("\" \"");
                                    }
                                }
                                if (result0 !== null) {
                                    result1 = parse__();
                                    if (result1 !== null) {
                                        result0 = [result0, result1];
                                    }
                                    else {
                                        result0 = null;
                                        pos = pos1;
                                    }
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                                if (result0 !== null) {
                                    result0 = (function (offset) { return 'descendant'; })(pos0);
                                }
                                if (result0 === null) {
                                    pos = pos0;
                                }
                            }
                        }
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_selectors() {
                    var cacheKey = "selectors@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3, result4, result5;
                    var pos0, pos1, pos2;
                    pos0 = pos;
                    pos1 = pos;
                    result0 = parse_selector();
                    if (result0 !== null) {
                        result1 = [];
                        pos2 = pos;
                        result2 = parse__();
                        if (result2 !== null) {
                            if (input.charCodeAt(pos) === 44) {
                                result3 = ",";
                                pos++;
                            }
                            else {
                                result3 = null;
                                if (reportFailures === 0) {
                                    matchFailed("\",\"");
                                }
                            }
                            if (result3 !== null) {
                                result4 = parse__();
                                if (result4 !== null) {
                                    result5 = parse_selector();
                                    if (result5 !== null) {
                                        result2 = [result2, result3, result4, result5];
                                    }
                                    else {
                                        result2 = null;
                                        pos = pos2;
                                    }
                                }
                                else {
                                    result2 = null;
                                    pos = pos2;
                                }
                            }
                            else {
                                result2 = null;
                                pos = pos2;
                            }
                        }
                        else {
                            result2 = null;
                            pos = pos2;
                        }
                        while (result2 !== null) {
                            result1.push(result2);
                            pos2 = pos;
                            result2 = parse__();
                            if (result2 !== null) {
                                if (input.charCodeAt(pos) === 44) {
                                    result3 = ",";
                                    pos++;
                                }
                                else {
                                    result3 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("\",\"");
                                    }
                                }
                                if (result3 !== null) {
                                    result4 = parse__();
                                    if (result4 !== null) {
                                        result5 = parse_selector();
                                        if (result5 !== null) {
                                            result2 = [result2, result3, result4, result5];
                                        }
                                        else {
                                            result2 = null;
                                            pos = pos2;
                                        }
                                    }
                                    else {
                                        result2 = null;
                                        pos = pos2;
                                    }
                                }
                                else {
                                    result2 = null;
                                    pos = pos2;
                                }
                            }
                            else {
                                result2 = null;
                                pos = pos2;
                            }
                        }
                        if (result1 !== null) {
                            result0 = [result0, result1];
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, s, ss) {
                            return [s].concat(ss.map(function (s) { return s[3]; }));
                        })(pos0, result0[0], result0[1]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_selector() {
                    var cacheKey = "selector@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3;
                    var pos0, pos1, pos2;
                    pos0 = pos;
                    pos1 = pos;
                    result0 = parse_sequence();
                    if (result0 !== null) {
                        result1 = [];
                        pos2 = pos;
                        result2 = parse_binaryOp();
                        if (result2 !== null) {
                            result3 = parse_sequence();
                            if (result3 !== null) {
                                result2 = [result2, result3];
                            }
                            else {
                                result2 = null;
                                pos = pos2;
                            }
                        }
                        else {
                            result2 = null;
                            pos = pos2;
                        }
                        while (result2 !== null) {
                            result1.push(result2);
                            pos2 = pos;
                            result2 = parse_binaryOp();
                            if (result2 !== null) {
                                result3 = parse_sequence();
                                if (result3 !== null) {
                                    result2 = [result2, result3];
                                }
                                else {
                                    result2 = null;
                                    pos = pos2;
                                }
                            }
                            else {
                                result2 = null;
                                pos = pos2;
                            }
                        }
                        if (result1 !== null) {
                            result0 = [result0, result1];
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, a, ops) {
                            return ops.reduce(function (memo, rhs) {
                                return { type: rhs[0], left: memo, right: rhs[1] };
                            }, a);
                        })(pos0, result0[0], result0[1]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_sequence() {
                    var cacheKey = "sequence@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.charCodeAt(pos) === 33) {
                        result0 = "!";
                        pos++;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\"!\"");
                        }
                    }
                    result0 = result0 !== null ? result0 : "";
                    if (result0 !== null) {
                        result2 = parse_atom();
                        if (result2 !== null) {
                            result1 = [];
                            while (result2 !== null) {
                                result1.push(result2);
                                result2 = parse_atom();
                            }
                        }
                        else {
                            result1 = null;
                        }
                        if (result1 !== null) {
                            result0 = [result0, result1];
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, subject, as) {
                            var b = as.length === 1 ? as[0] : { type: 'compound', selectors: as };
                            if (subject)
                                b.subject = true;
                            return b;
                        })(pos0, result0[0], result0[1]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_atom() {
                    var cacheKey = "atom@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0;
                    result0 = parse_wildcard();
                    if (result0 === null) {
                        result0 = parse_identifier();
                        if (result0 === null) {
                            result0 = parse_attr();
                            if (result0 === null) {
                                result0 = parse_field();
                                if (result0 === null) {
                                    result0 = parse_negation();
                                    if (result0 === null) {
                                        result0 = parse_matches();
                                        if (result0 === null) {
                                            result0 = parse_has();
                                            if (result0 === null) {
                                                result0 = parse_firstChild();
                                                if (result0 === null) {
                                                    result0 = parse_lastChild();
                                                    if (result0 === null) {
                                                        result0 = parse_nthChild();
                                                        if (result0 === null) {
                                                            result0 = parse_nthLastChild();
                                                            if (result0 === null) {
                                                                result0 = parse_class();
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_wildcard() {
                    var cacheKey = "wildcard@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0;
                    var pos0;
                    pos0 = pos;
                    if (input.charCodeAt(pos) === 42) {
                        result0 = "*";
                        pos++;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\"*\"");
                        }
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, a) { return { type: 'wildcard', value: a }; })(pos0, result0);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_identifier() {
                    var cacheKey = "identifier@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.charCodeAt(pos) === 35) {
                        result0 = "#";
                        pos++;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\"#\"");
                        }
                    }
                    result0 = result0 !== null ? result0 : "";
                    if (result0 !== null) {
                        result1 = parse_identifierName();
                        if (result1 !== null) {
                            result0 = [result0, result1];
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, i) { return { type: 'identifier', value: i }; })(pos0, result0[1]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_attr() {
                    var cacheKey = "attr@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3, result4;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.charCodeAt(pos) === 91) {
                        result0 = "[";
                        pos++;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\"[\"");
                        }
                    }
                    if (result0 !== null) {
                        result1 = parse__();
                        if (result1 !== null) {
                            result2 = parse_attrValue();
                            if (result2 !== null) {
                                result3 = parse__();
                                if (result3 !== null) {
                                    if (input.charCodeAt(pos) === 93) {
                                        result4 = "]";
                                        pos++;
                                    }
                                    else {
                                        result4 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("\"]\"");
                                        }
                                    }
                                    if (result4 !== null) {
                                        result0 = [result0, result1, result2, result3, result4];
                                    }
                                    else {
                                        result0 = null;
                                        pos = pos1;
                                    }
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, v) { return v; })(pos0, result0[2]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_attrOps() {
                    var cacheKey = "attrOps@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (/^[><!]/.test(input.charAt(pos))) {
                        result0 = input.charAt(pos);
                        pos++;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("[><!]");
                        }
                    }
                    result0 = result0 !== null ? result0 : "";
                    if (result0 !== null) {
                        if (input.charCodeAt(pos) === 61) {
                            result1 = "=";
                            pos++;
                        }
                        else {
                            result1 = null;
                            if (reportFailures === 0) {
                                matchFailed("\"=\"");
                            }
                        }
                        if (result1 !== null) {
                            result0 = [result0, result1];
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, a) { return a + '='; })(pos0, result0[0]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    if (result0 === null) {
                        if (/^[><]/.test(input.charAt(pos))) {
                            result0 = input.charAt(pos);
                            pos++;
                        }
                        else {
                            result0 = null;
                            if (reportFailures === 0) {
                                matchFailed("[><]");
                            }
                        }
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_attrEqOps() {
                    var cacheKey = "attrEqOps@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.charCodeAt(pos) === 33) {
                        result0 = "!";
                        pos++;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\"!\"");
                        }
                    }
                    result0 = result0 !== null ? result0 : "";
                    if (result0 !== null) {
                        if (input.charCodeAt(pos) === 61) {
                            result1 = "=";
                            pos++;
                        }
                        else {
                            result1 = null;
                            if (reportFailures === 0) {
                                matchFailed("\"=\"");
                            }
                        }
                        if (result1 !== null) {
                            result0 = [result0, result1];
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, a) { return a + '='; })(pos0, result0[0]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_attrName() {
                    var cacheKey = "attrName@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1;
                    var pos0;
                    pos0 = pos;
                    result1 = parse_identifierName();
                    if (result1 === null) {
                        if (input.charCodeAt(pos) === 46) {
                            result1 = ".";
                            pos++;
                        }
                        else {
                            result1 = null;
                            if (reportFailures === 0) {
                                matchFailed("\".\"");
                            }
                        }
                    }
                    if (result1 !== null) {
                        result0 = [];
                        while (result1 !== null) {
                            result0.push(result1);
                            result1 = parse_identifierName();
                            if (result1 === null) {
                                if (input.charCodeAt(pos) === 46) {
                                    result1 = ".";
                                    pos++;
                                }
                                else {
                                    result1 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("\".\"");
                                    }
                                }
                            }
                        }
                    }
                    else {
                        result0 = null;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, i) { return i.join(''); })(pos0, result0);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_attrValue() {
                    var cacheKey = "attrValue@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3, result4;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    result0 = parse_attrName();
                    if (result0 !== null) {
                        result1 = parse__();
                        if (result1 !== null) {
                            result2 = parse_attrEqOps();
                            if (result2 !== null) {
                                result3 = parse__();
                                if (result3 !== null) {
                                    result4 = parse_type();
                                    if (result4 === null) {
                                        result4 = parse_regex();
                                    }
                                    if (result4 !== null) {
                                        result0 = [result0, result1, result2, result3, result4];
                                    }
                                    else {
                                        result0 = null;
                                        pos = pos1;
                                    }
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, name, op, value) {
                            return { type: 'attribute', name: name, operator: op, value: value };
                        })(pos0, result0[0], result0[2], result0[4]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    if (result0 === null) {
                        pos0 = pos;
                        pos1 = pos;
                        result0 = parse_attrName();
                        if (result0 !== null) {
                            result1 = parse__();
                            if (result1 !== null) {
                                result2 = parse_attrOps();
                                if (result2 !== null) {
                                    result3 = parse__();
                                    if (result3 !== null) {
                                        result4 = parse_string();
                                        if (result4 === null) {
                                            result4 = parse_number();
                                            if (result4 === null) {
                                                result4 = parse_path();
                                            }
                                        }
                                        if (result4 !== null) {
                                            result0 = [result0, result1, result2, result3, result4];
                                        }
                                        else {
                                            result0 = null;
                                            pos = pos1;
                                        }
                                    }
                                    else {
                                        result0 = null;
                                        pos = pos1;
                                    }
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                        if (result0 !== null) {
                            result0 = (function (offset, name, op, value) {
                                return { type: 'attribute', name: name, operator: op, value: value };
                            })(pos0, result0[0], result0[2], result0[4]);
                        }
                        if (result0 === null) {
                            pos = pos0;
                        }
                        if (result0 === null) {
                            pos0 = pos;
                            result0 = parse_attrName();
                            if (result0 !== null) {
                                result0 = (function (offset, name) { return { type: 'attribute', name: name }; })(pos0, result0);
                            }
                            if (result0 === null) {
                                pos = pos0;
                            }
                        }
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_string() {
                    var cacheKey = "string@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3;
                    var pos0, pos1, pos2, pos3;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.charCodeAt(pos) === 34) {
                        result0 = "\"";
                        pos++;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\"\\\"\"");
                        }
                    }
                    if (result0 !== null) {
                        result1 = [];
                        if (/^[^\\"]/.test(input.charAt(pos))) {
                            result2 = input.charAt(pos);
                            pos++;
                        }
                        else {
                            result2 = null;
                            if (reportFailures === 0) {
                                matchFailed("[^\\\\\"]");
                            }
                        }
                        if (result2 === null) {
                            pos2 = pos;
                            pos3 = pos;
                            if (input.charCodeAt(pos) === 92) {
                                result2 = "\\";
                                pos++;
                            }
                            else {
                                result2 = null;
                                if (reportFailures === 0) {
                                    matchFailed("\"\\\\\"");
                                }
                            }
                            if (result2 !== null) {
                                if (input.length > pos) {
                                    result3 = input.charAt(pos);
                                    pos++;
                                }
                                else {
                                    result3 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("any character");
                                    }
                                }
                                if (result3 !== null) {
                                    result2 = [result2, result3];
                                }
                                else {
                                    result2 = null;
                                    pos = pos3;
                                }
                            }
                            else {
                                result2 = null;
                                pos = pos3;
                            }
                            if (result2 !== null) {
                                result2 = (function (offset, a, b) { return a + b; })(pos2, result2[0], result2[1]);
                            }
                            if (result2 === null) {
                                pos = pos2;
                            }
                        }
                        while (result2 !== null) {
                            result1.push(result2);
                            if (/^[^\\"]/.test(input.charAt(pos))) {
                                result2 = input.charAt(pos);
                                pos++;
                            }
                            else {
                                result2 = null;
                                if (reportFailures === 0) {
                                    matchFailed("[^\\\\\"]");
                                }
                            }
                            if (result2 === null) {
                                pos2 = pos;
                                pos3 = pos;
                                if (input.charCodeAt(pos) === 92) {
                                    result2 = "\\";
                                    pos++;
                                }
                                else {
                                    result2 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("\"\\\\\"");
                                    }
                                }
                                if (result2 !== null) {
                                    if (input.length > pos) {
                                        result3 = input.charAt(pos);
                                        pos++;
                                    }
                                    else {
                                        result3 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("any character");
                                        }
                                    }
                                    if (result3 !== null) {
                                        result2 = [result2, result3];
                                    }
                                    else {
                                        result2 = null;
                                        pos = pos3;
                                    }
                                }
                                else {
                                    result2 = null;
                                    pos = pos3;
                                }
                                if (result2 !== null) {
                                    result2 = (function (offset, a, b) { return a + b; })(pos2, result2[0], result2[1]);
                                }
                                if (result2 === null) {
                                    pos = pos2;
                                }
                            }
                        }
                        if (result1 !== null) {
                            if (input.charCodeAt(pos) === 34) {
                                result2 = "\"";
                                pos++;
                            }
                            else {
                                result2 = null;
                                if (reportFailures === 0) {
                                    matchFailed("\"\\\"\"");
                                }
                            }
                            if (result2 !== null) {
                                result0 = [result0, result1, result2];
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, d) {
                            return { type: 'literal', value: strUnescape(d.join('')) };
                        })(pos0, result0[1]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    if (result0 === null) {
                        pos0 = pos;
                        pos1 = pos;
                        if (input.charCodeAt(pos) === 39) {
                            result0 = "'";
                            pos++;
                        }
                        else {
                            result0 = null;
                            if (reportFailures === 0) {
                                matchFailed("\"'\"");
                            }
                        }
                        if (result0 !== null) {
                            result1 = [];
                            if (/^[^\\']/.test(input.charAt(pos))) {
                                result2 = input.charAt(pos);
                                pos++;
                            }
                            else {
                                result2 = null;
                                if (reportFailures === 0) {
                                    matchFailed("[^\\\\']");
                                }
                            }
                            if (result2 === null) {
                                pos2 = pos;
                                pos3 = pos;
                                if (input.charCodeAt(pos) === 92) {
                                    result2 = "\\";
                                    pos++;
                                }
                                else {
                                    result2 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("\"\\\\\"");
                                    }
                                }
                                if (result2 !== null) {
                                    if (input.length > pos) {
                                        result3 = input.charAt(pos);
                                        pos++;
                                    }
                                    else {
                                        result3 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("any character");
                                        }
                                    }
                                    if (result3 !== null) {
                                        result2 = [result2, result3];
                                    }
                                    else {
                                        result2 = null;
                                        pos = pos3;
                                    }
                                }
                                else {
                                    result2 = null;
                                    pos = pos3;
                                }
                                if (result2 !== null) {
                                    result2 = (function (offset, a, b) { return a + b; })(pos2, result2[0], result2[1]);
                                }
                                if (result2 === null) {
                                    pos = pos2;
                                }
                            }
                            while (result2 !== null) {
                                result1.push(result2);
                                if (/^[^\\']/.test(input.charAt(pos))) {
                                    result2 = input.charAt(pos);
                                    pos++;
                                }
                                else {
                                    result2 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("[^\\\\']");
                                    }
                                }
                                if (result2 === null) {
                                    pos2 = pos;
                                    pos3 = pos;
                                    if (input.charCodeAt(pos) === 92) {
                                        result2 = "\\";
                                        pos++;
                                    }
                                    else {
                                        result2 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("\"\\\\\"");
                                        }
                                    }
                                    if (result2 !== null) {
                                        if (input.length > pos) {
                                            result3 = input.charAt(pos);
                                            pos++;
                                        }
                                        else {
                                            result3 = null;
                                            if (reportFailures === 0) {
                                                matchFailed("any character");
                                            }
                                        }
                                        if (result3 !== null) {
                                            result2 = [result2, result3];
                                        }
                                        else {
                                            result2 = null;
                                            pos = pos3;
                                        }
                                    }
                                    else {
                                        result2 = null;
                                        pos = pos3;
                                    }
                                    if (result2 !== null) {
                                        result2 = (function (offset, a, b) { return a + b; })(pos2, result2[0], result2[1]);
                                    }
                                    if (result2 === null) {
                                        pos = pos2;
                                    }
                                }
                            }
                            if (result1 !== null) {
                                if (input.charCodeAt(pos) === 39) {
                                    result2 = "'";
                                    pos++;
                                }
                                else {
                                    result2 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("\"'\"");
                                    }
                                }
                                if (result2 !== null) {
                                    result0 = [result0, result1, result2];
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                        if (result0 !== null) {
                            result0 = (function (offset, d) {
                                return { type: 'literal', value: strUnescape(d.join('')) };
                            })(pos0, result0[1]);
                        }
                        if (result0 === null) {
                            pos = pos0;
                        }
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_number() {
                    var cacheKey = "number@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2;
                    var pos0, pos1, pos2;
                    pos0 = pos;
                    pos1 = pos;
                    pos2 = pos;
                    result0 = [];
                    if (/^[0-9]/.test(input.charAt(pos))) {
                        result1 = input.charAt(pos);
                        pos++;
                    }
                    else {
                        result1 = null;
                        if (reportFailures === 0) {
                            matchFailed("[0-9]");
                        }
                    }
                    while (result1 !== null) {
                        result0.push(result1);
                        if (/^[0-9]/.test(input.charAt(pos))) {
                            result1 = input.charAt(pos);
                            pos++;
                        }
                        else {
                            result1 = null;
                            if (reportFailures === 0) {
                                matchFailed("[0-9]");
                            }
                        }
                    }
                    if (result0 !== null) {
                        if (input.charCodeAt(pos) === 46) {
                            result1 = ".";
                            pos++;
                        }
                        else {
                            result1 = null;
                            if (reportFailures === 0) {
                                matchFailed("\".\"");
                            }
                        }
                        if (result1 !== null) {
                            result0 = [result0, result1];
                        }
                        else {
                            result0 = null;
                            pos = pos2;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos2;
                    }
                    result0 = result0 !== null ? result0 : "";
                    if (result0 !== null) {
                        if (/^[0-9]/.test(input.charAt(pos))) {
                            result2 = input.charAt(pos);
                            pos++;
                        }
                        else {
                            result2 = null;
                            if (reportFailures === 0) {
                                matchFailed("[0-9]");
                            }
                        }
                        if (result2 !== null) {
                            result1 = [];
                            while (result2 !== null) {
                                result1.push(result2);
                                if (/^[0-9]/.test(input.charAt(pos))) {
                                    result2 = input.charAt(pos);
                                    pos++;
                                }
                                else {
                                    result2 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("[0-9]");
                                    }
                                }
                            }
                        }
                        else {
                            result1 = null;
                        }
                        if (result1 !== null) {
                            result0 = [result0, result1];
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, a, b) {
                            return { type: 'literal', value: parseFloat((a ? a.join('') : '') + b.join('')) };
                        })(pos0, result0[0], result0[1]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_path() {
                    var cacheKey = "path@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0;
                    var pos0;
                    pos0 = pos;
                    result0 = parse_identifierName();
                    if (result0 !== null) {
                        result0 = (function (offset, i) { return { type: 'literal', value: i }; })(pos0, result0);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_type() {
                    var cacheKey = "type@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3, result4;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.substr(pos, 5) === "type(") {
                        result0 = "type(";
                        pos += 5;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\"type(\"");
                        }
                    }
                    if (result0 !== null) {
                        result1 = parse__();
                        if (result1 !== null) {
                            if (/^[^ )]/.test(input.charAt(pos))) {
                                result3 = input.charAt(pos);
                                pos++;
                            }
                            else {
                                result3 = null;
                                if (reportFailures === 0) {
                                    matchFailed("[^ )]");
                                }
                            }
                            if (result3 !== null) {
                                result2 = [];
                                while (result3 !== null) {
                                    result2.push(result3);
                                    if (/^[^ )]/.test(input.charAt(pos))) {
                                        result3 = input.charAt(pos);
                                        pos++;
                                    }
                                    else {
                                        result3 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("[^ )]");
                                        }
                                    }
                                }
                            }
                            else {
                                result2 = null;
                            }
                            if (result2 !== null) {
                                result3 = parse__();
                                if (result3 !== null) {
                                    if (input.charCodeAt(pos) === 41) {
                                        result4 = ")";
                                        pos++;
                                    }
                                    else {
                                        result4 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("\")\"");
                                        }
                                    }
                                    if (result4 !== null) {
                                        result0 = [result0, result1, result2, result3, result4];
                                    }
                                    else {
                                        result0 = null;
                                        pos = pos1;
                                    }
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, t) { return { type: 'type', value: t.join('') }; })(pos0, result0[2]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_regex() {
                    var cacheKey = "regex@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.charCodeAt(pos) === 47) {
                        result0 = "/";
                        pos++;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\"/\"");
                        }
                    }
                    if (result0 !== null) {
                        if (/^[^\/]/.test(input.charAt(pos))) {
                            result2 = input.charAt(pos);
                            pos++;
                        }
                        else {
                            result2 = null;
                            if (reportFailures === 0) {
                                matchFailed("[^\\/]");
                            }
                        }
                        if (result2 !== null) {
                            result1 = [];
                            while (result2 !== null) {
                                result1.push(result2);
                                if (/^[^\/]/.test(input.charAt(pos))) {
                                    result2 = input.charAt(pos);
                                    pos++;
                                }
                                else {
                                    result2 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("[^\\/]");
                                    }
                                }
                            }
                        }
                        else {
                            result1 = null;
                        }
                        if (result1 !== null) {
                            if (input.charCodeAt(pos) === 47) {
                                result2 = "/";
                                pos++;
                            }
                            else {
                                result2 = null;
                                if (reportFailures === 0) {
                                    matchFailed("\"/\"");
                                }
                            }
                            if (result2 !== null) {
                                result0 = [result0, result1, result2];
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, d) { return { type: 'regexp', value: new RegExp(d.join('')) }; })(pos0, result0[1]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_field() {
                    var cacheKey = "field@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3, result4;
                    var pos0, pos1, pos2;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.charCodeAt(pos) === 46) {
                        result0 = ".";
                        pos++;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\".\"");
                        }
                    }
                    if (result0 !== null) {
                        result1 = parse_identifierName();
                        if (result1 !== null) {
                            result2 = [];
                            pos2 = pos;
                            if (input.charCodeAt(pos) === 46) {
                                result3 = ".";
                                pos++;
                            }
                            else {
                                result3 = null;
                                if (reportFailures === 0) {
                                    matchFailed("\".\"");
                                }
                            }
                            if (result3 !== null) {
                                result4 = parse_identifierName();
                                if (result4 !== null) {
                                    result3 = [result3, result4];
                                }
                                else {
                                    result3 = null;
                                    pos = pos2;
                                }
                            }
                            else {
                                result3 = null;
                                pos = pos2;
                            }
                            while (result3 !== null) {
                                result2.push(result3);
                                pos2 = pos;
                                if (input.charCodeAt(pos) === 46) {
                                    result3 = ".";
                                    pos++;
                                }
                                else {
                                    result3 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("\".\"");
                                    }
                                }
                                if (result3 !== null) {
                                    result4 = parse_identifierName();
                                    if (result4 !== null) {
                                        result3 = [result3, result4];
                                    }
                                    else {
                                        result3 = null;
                                        pos = pos2;
                                    }
                                }
                                else {
                                    result3 = null;
                                    pos = pos2;
                                }
                            }
                            if (result2 !== null) {
                                result0 = [result0, result1, result2];
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, i, is) {
                            return { type: 'field', name: is.reduce(function (memo, p) { return memo + p[0] + p[1]; }, i) };
                        })(pos0, result0[1], result0[2]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_negation() {
                    var cacheKey = "negation@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3, result4;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.substr(pos, 5) === ":not(") {
                        result0 = ":not(";
                        pos += 5;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\":not(\"");
                        }
                    }
                    if (result0 !== null) {
                        result1 = parse__();
                        if (result1 !== null) {
                            result2 = parse_selectors();
                            if (result2 !== null) {
                                result3 = parse__();
                                if (result3 !== null) {
                                    if (input.charCodeAt(pos) === 41) {
                                        result4 = ")";
                                        pos++;
                                    }
                                    else {
                                        result4 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("\")\"");
                                        }
                                    }
                                    if (result4 !== null) {
                                        result0 = [result0, result1, result2, result3, result4];
                                    }
                                    else {
                                        result0 = null;
                                        pos = pos1;
                                    }
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, ss) { return { type: 'not', selectors: ss }; })(pos0, result0[2]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_matches() {
                    var cacheKey = "matches@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3, result4;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.substr(pos, 9) === ":matches(") {
                        result0 = ":matches(";
                        pos += 9;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\":matches(\"");
                        }
                    }
                    if (result0 !== null) {
                        result1 = parse__();
                        if (result1 !== null) {
                            result2 = parse_selectors();
                            if (result2 !== null) {
                                result3 = parse__();
                                if (result3 !== null) {
                                    if (input.charCodeAt(pos) === 41) {
                                        result4 = ")";
                                        pos++;
                                    }
                                    else {
                                        result4 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("\")\"");
                                        }
                                    }
                                    if (result4 !== null) {
                                        result0 = [result0, result1, result2, result3, result4];
                                    }
                                    else {
                                        result0 = null;
                                        pos = pos1;
                                    }
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, ss) { return { type: 'matches', selectors: ss }; })(pos0, result0[2]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_has() {
                    var cacheKey = "has@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3, result4;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.substr(pos, 5) === ":has(") {
                        result0 = ":has(";
                        pos += 5;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\":has(\"");
                        }
                    }
                    if (result0 !== null) {
                        result1 = parse__();
                        if (result1 !== null) {
                            result2 = parse_selectors();
                            if (result2 !== null) {
                                result3 = parse__();
                                if (result3 !== null) {
                                    if (input.charCodeAt(pos) === 41) {
                                        result4 = ")";
                                        pos++;
                                    }
                                    else {
                                        result4 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("\")\"");
                                        }
                                    }
                                    if (result4 !== null) {
                                        result0 = [result0, result1, result2, result3, result4];
                                    }
                                    else {
                                        result0 = null;
                                        pos = pos1;
                                    }
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, ss) { return { type: 'has', selectors: ss }; })(pos0, result0[2]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_firstChild() {
                    var cacheKey = "firstChild@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0;
                    var pos0;
                    pos0 = pos;
                    if (input.substr(pos, 12) === ":first-child") {
                        result0 = ":first-child";
                        pos += 12;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\":first-child\"");
                        }
                    }
                    if (result0 !== null) {
                        result0 = (function (offset) { return nth(1); })(pos0);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_lastChild() {
                    var cacheKey = "lastChild@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0;
                    var pos0;
                    pos0 = pos;
                    if (input.substr(pos, 11) === ":last-child") {
                        result0 = ":last-child";
                        pos += 11;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\":last-child\"");
                        }
                    }
                    if (result0 !== null) {
                        result0 = (function (offset) { return nthLast(1); })(pos0);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_nthChild() {
                    var cacheKey = "nthChild@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3, result4;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.substr(pos, 11) === ":nth-child(") {
                        result0 = ":nth-child(";
                        pos += 11;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\":nth-child(\"");
                        }
                    }
                    if (result0 !== null) {
                        result1 = parse__();
                        if (result1 !== null) {
                            if (/^[0-9]/.test(input.charAt(pos))) {
                                result3 = input.charAt(pos);
                                pos++;
                            }
                            else {
                                result3 = null;
                                if (reportFailures === 0) {
                                    matchFailed("[0-9]");
                                }
                            }
                            if (result3 !== null) {
                                result2 = [];
                                while (result3 !== null) {
                                    result2.push(result3);
                                    if (/^[0-9]/.test(input.charAt(pos))) {
                                        result3 = input.charAt(pos);
                                        pos++;
                                    }
                                    else {
                                        result3 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("[0-9]");
                                        }
                                    }
                                }
                            }
                            else {
                                result2 = null;
                            }
                            if (result2 !== null) {
                                result3 = parse__();
                                if (result3 !== null) {
                                    if (input.charCodeAt(pos) === 41) {
                                        result4 = ")";
                                        pos++;
                                    }
                                    else {
                                        result4 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("\")\"");
                                        }
                                    }
                                    if (result4 !== null) {
                                        result0 = [result0, result1, result2, result3, result4];
                                    }
                                    else {
                                        result0 = null;
                                        pos = pos1;
                                    }
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, n) { return nth(parseInt(n.join(''), 10)); })(pos0, result0[2]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_nthLastChild() {
                    var cacheKey = "nthLastChild@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1, result2, result3, result4;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.substr(pos, 16) === ":nth-last-child(") {
                        result0 = ":nth-last-child(";
                        pos += 16;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\":nth-last-child(\"");
                        }
                    }
                    if (result0 !== null) {
                        result1 = parse__();
                        if (result1 !== null) {
                            if (/^[0-9]/.test(input.charAt(pos))) {
                                result3 = input.charAt(pos);
                                pos++;
                            }
                            else {
                                result3 = null;
                                if (reportFailures === 0) {
                                    matchFailed("[0-9]");
                                }
                            }
                            if (result3 !== null) {
                                result2 = [];
                                while (result3 !== null) {
                                    result2.push(result3);
                                    if (/^[0-9]/.test(input.charAt(pos))) {
                                        result3 = input.charAt(pos);
                                        pos++;
                                    }
                                    else {
                                        result3 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("[0-9]");
                                        }
                                    }
                                }
                            }
                            else {
                                result2 = null;
                            }
                            if (result2 !== null) {
                                result3 = parse__();
                                if (result3 !== null) {
                                    if (input.charCodeAt(pos) === 41) {
                                        result4 = ")";
                                        pos++;
                                    }
                                    else {
                                        result4 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("\")\"");
                                        }
                                    }
                                    if (result4 !== null) {
                                        result0 = [result0, result1, result2, result3, result4];
                                    }
                                    else {
                                        result0 = null;
                                        pos = pos1;
                                    }
                                }
                                else {
                                    result0 = null;
                                    pos = pos1;
                                }
                            }
                            else {
                                result0 = null;
                                pos = pos1;
                            }
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, n) { return nthLast(parseInt(n.join(''), 10)); })(pos0, result0[2]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function parse_class() {
                    var cacheKey = "class@" + pos;
                    var cachedResult = cache[cacheKey];
                    if (cachedResult) {
                        pos = cachedResult.nextPos;
                        return cachedResult.result;
                    }
                    var result0, result1;
                    var pos0, pos1;
                    pos0 = pos;
                    pos1 = pos;
                    if (input.charCodeAt(pos) === 58) {
                        result0 = ":";
                        pos++;
                    }
                    else {
                        result0 = null;
                        if (reportFailures === 0) {
                            matchFailed("\":\"");
                        }
                    }
                    if (result0 !== null) {
                        if (input.substr(pos, 9).toLowerCase() === "statement") {
                            result1 = input.substr(pos, 9);
                            pos += 9;
                        }
                        else {
                            result1 = null;
                            if (reportFailures === 0) {
                                matchFailed("\"statement\"");
                            }
                        }
                        if (result1 === null) {
                            if (input.substr(pos, 10).toLowerCase() === "expression") {
                                result1 = input.substr(pos, 10);
                                pos += 10;
                            }
                            else {
                                result1 = null;
                                if (reportFailures === 0) {
                                    matchFailed("\"expression\"");
                                }
                            }
                            if (result1 === null) {
                                if (input.substr(pos, 11).toLowerCase() === "declaration") {
                                    result1 = input.substr(pos, 11);
                                    pos += 11;
                                }
                                else {
                                    result1 = null;
                                    if (reportFailures === 0) {
                                        matchFailed("\"declaration\"");
                                    }
                                }
                                if (result1 === null) {
                                    if (input.substr(pos, 8).toLowerCase() === "function") {
                                        result1 = input.substr(pos, 8);
                                        pos += 8;
                                    }
                                    else {
                                        result1 = null;
                                        if (reportFailures === 0) {
                                            matchFailed("\"function\"");
                                        }
                                    }
                                    if (result1 === null) {
                                        if (input.substr(pos, 7).toLowerCase() === "pattern") {
                                            result1 = input.substr(pos, 7);
                                            pos += 7;
                                        }
                                        else {
                                            result1 = null;
                                            if (reportFailures === 0) {
                                                matchFailed("\"pattern\"");
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (result1 !== null) {
                            result0 = [result0, result1];
                        }
                        else {
                            result0 = null;
                            pos = pos1;
                        }
                    }
                    else {
                        result0 = null;
                        pos = pos1;
                    }
                    if (result0 !== null) {
                        result0 = (function (offset, c) {
                            return { type: 'class', name: c };
                        })(pos0, result0[1]);
                    }
                    if (result0 === null) {
                        pos = pos0;
                    }
                    cache[cacheKey] = {
                        nextPos: pos,
                        result: result0
                    };
                    return result0;
                }
                function cleanupExpected(expected) {
                    expected.sort();
                    var lastExpected = null;
                    var cleanExpected = [];
                    for (var i = 0; i < expected.length; i++) {
                        if (expected[i] !== lastExpected) {
                            cleanExpected.push(expected[i]);
                            lastExpected = expected[i];
                        }
                    }
                    return cleanExpected;
                }
                function computeErrorPosition() {
                    /*
                     * The first idea was to use |String.split| to break the input up to the
                     * error position along newlines and derive the line and column from
                     * there. However IE's |split| implementation is so broken that it was
                     * enough to prevent it.
                     */
                    var line = 1;
                    var column = 1;
                    var seenCR = false;
                    for (var i = 0; i < Math.max(pos, rightmostFailuresPos); i++) {
                        var ch = input.charAt(i);
                        if (ch === "\n") {
                            if (!seenCR) {
                                line++;
                            }
                            column = 1;
                            seenCR = false;
                        }
                        else if (ch === "\r" || ch === "\u2028" || ch === "\u2029") {
                            line++;
                            column = 1;
                            seenCR = true;
                        }
                        else {
                            column++;
                            seenCR = false;
                        }
                    }
                    return { line: line, column: column };
                }
                function nth(n) { return { type: 'nth-child', index: { type: 'literal', value: n } }; }
                function nthLast(n) { return { type: 'nth-last-child', index: { type: 'literal', value: n } }; }
                function strUnescape(s) {
                    return s.replace(/\\(.)/g, function (match, ch) {
                        switch (ch) {
                            case 'a': return '\a';
                            case 'b': return '\b';
                            case 'f': return '\f';
                            case 'n': return '\n';
                            case 'r': return '\r';
                            case 't': return '\t';
                            case 'v': return '\v';
                            default: return ch;
                        }
                    });
                }
                var result = parseFunctions[startRule]();
                /*
                 * The parser is now in one of the following three states:
                 *
                 * 1. The parser successfully parsed the whole input.
                 *
                 *    - |result !== null|
                 *    - |pos === input.length|
                 *    - |rightmostFailuresExpected| may or may not contain something
                 *
                 * 2. The parser successfully parsed only a part of the input.
                 *
                 *    - |result !== null|
                 *    - |pos < input.length|
                 *    - |rightmostFailuresExpected| may or may not contain something
                 *
                 * 3. The parser did not successfully parse any part of the input.
                 *
                 *   - |result === null|
                 *   - |pos === 0|
                 *   - |rightmostFailuresExpected| contains at least one failure
                 *
                 * All code following this comment (including called functions) must
                 * handle these states.
                 */
                if (result === null || pos !== input.length) {
                    var offset = Math.max(pos, rightmostFailuresPos);
                    var found = offset < input.length ? input.charAt(offset) : null;
                    var errorPosition = computeErrorPosition();
                    throw new this.SyntaxError(cleanupExpected(rightmostFailuresExpected), found, offset, errorPosition.line, errorPosition.column);
                }
                return result;
            },
            /* Returns the parser source code. */
            toSource: function () { return this._source; }
        };
        /* Thrown when a parser encounters a syntax error. */
        result.SyntaxError = function (expected, found, offset, line, column) {
            function buildMessage(expected, found) {
                var expectedHumanized, foundHumanized;
                switch (expected.length) {
                    case 0:
                        expectedHumanized = "end of input";
                        break;
                    case 1:
                        expectedHumanized = expected[0];
                        break;
                    default:
                        expectedHumanized = expected.slice(0, expected.length - 1).join(", ")
                            + " or "
                            + expected[expected.length - 1];
                }
                foundHumanized = found ? quote(found) : "end of input";
                return "Expected " + expectedHumanized + " but " + foundHumanized + " found.";
            }
            this.name = "SyntaxError";
            this.expected = expected;
            this.found = found;
            this.message = buildMessage(expected, found);
            this.offset = offset;
            this.line = line;
            this.column = column;
        };
        result.SyntaxError.prototype = Error.prototype;
        return result;
    })();
    if (typeof define === "function" && define.amd) {
        define(function () { return result; });
    }
    else if (typeof module !== "undefined" && module.exports) {
        module.exports = result;
    }
    else {
        this.esquery = result;
    }
},
/* eslint/lib/linter/report-translator.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const assert = require("assert");
    const ruleFixer = require(511) /* ./rule-fixer */;
    const interpolate = require(512) /* ./interpolate */;
    //------------------------------------------------------------------------------
    // Typedefs
    //------------------------------------------------------------------------------
    /**
     * An error message description
     * @typedef {Object} MessageDescriptor
     * @property {ASTNode} [node] The reported node
     * @property {Location} loc The location of the problem.
     * @property {string} message The problem message.
     * @property {Object} [data] Optional data to use to fill in placeholders in the
     *      message.
     * @property {Function} [fix] The function to call that creates a fix command.
     * @property {Array<{desc?: string, messageId?: string, fix: Function}>} suggest Suggestion descriptions and functions to create a the associated fixes.
     */
    /**
     * Information about the report
     * @typedef {Object} ReportInfo
     * @property {string} ruleId
     * @property {(0|1|2)} severity
     * @property {(string|undefined)} message
     * @property {(string|undefined)} [messageId]
     * @property {number} line
     * @property {number} column
     * @property {(number|undefined)} [endLine]
     * @property {(number|undefined)} [endColumn]
     * @property {(string|null)} nodeType
     * @property {string} source
     * @property {({text: string, range: (number[]|null)}|null)} [fix]
     * @property {Array<{text: string, range: (number[]|null)}|null>} [suggestions]
     */
    //------------------------------------------------------------------------------
    // Module Definition
    //------------------------------------------------------------------------------
    /**
     * Translates a multi-argument context.report() call into a single object argument call
     * @param {...*} args A list of arguments passed to `context.report`
     * @returns {MessageDescriptor} A normalized object containing report information
     */
    function normalizeMultiArgReportCall(...args) {
        // If there is one argument, it is considered to be a new-style call already.
        if (args.length === 1) {
            // Shallow clone the object to avoid surprises if reusing the descriptor
            return Object.assign({}, args[0]);
        }
        // If the second argument is a string, the arguments are interpreted as [node, message, data, fix].
        if (typeof args[1] === "string") {
            return {
                node: args[0],
                message: args[1],
                data: args[2],
                fix: args[3]
            };
        }
        // Otherwise, the arguments are interpreted as [node, loc, message, data, fix].
        return {
            node: args[0],
            loc: args[1],
            message: args[2],
            data: args[3],
            fix: args[4]
        };
    }
    /**
     * Asserts that either a loc or a node was provided, and the node is valid if it was provided.
     * @param {MessageDescriptor} descriptor A descriptor to validate
     * @returns {void}
     * @throws AssertionError if neither a node nor a loc was provided, or if the node is not an object
     */
    function assertValidNodeInfo(descriptor) {
        if (descriptor.node) {
            assert(typeof descriptor.node === "object", "Node must be an object");
        }
        else {
            assert(descriptor.loc, "Node must be provided when reporting error if location is not provided");
        }
    }
    /**
     * Normalizes a MessageDescriptor to always have a `loc` with `start` and `end` properties
     * @param {MessageDescriptor} descriptor A descriptor for the report from a rule.
     * @returns {{start: Location, end: (Location|null)}} An updated location that infers the `start` and `end` properties
     * from the `node` of the original descriptor, or infers the `start` from the `loc` of the original descriptor.
     */
    function normalizeReportLoc(descriptor) {
        if (descriptor.loc) {
            if (descriptor.loc.start) {
                return descriptor.loc;
            }
            return { start: descriptor.loc, end: null };
        }
        return descriptor.node.loc;
    }
    /**
     * Compares items in a fixes array by range.
     * @param {Fix} a The first message.
     * @param {Fix} b The second message.
     * @returns {int} -1 if a comes before b, 1 if a comes after b, 0 if equal.
     * @private
     */
    function compareFixesByRange(a, b) {
        return a.range[0] - b.range[0] || a.range[1] - b.range[1];
    }
    /**
     * Merges the given fixes array into one.
     * @param {Fix[]} fixes The fixes to merge.
     * @param {SourceCode} sourceCode The source code object to get the text between fixes.
     * @returns {{text: string, range: number[]}} The merged fixes
     */
    function mergeFixes(fixes, sourceCode) {
        if (fixes.length === 0) {
            return null;
        }
        if (fixes.length === 1) {
            return fixes[0];
        }
        fixes.sort(compareFixesByRange);
        const originalText = sourceCode.text;
        const start = fixes[0].range[0];
        const end = fixes[fixes.length - 1].range[1];
        let text = "";
        let lastPos = Number.MIN_SAFE_INTEGER;
        for (const fix of fixes) {
            assert(fix.range[0] >= lastPos, "Fix objects must not be overlapped in a report.");
            if (fix.range[0] >= 0) {
                text += originalText.slice(Math.max(0, start, lastPos), fix.range[0]);
            }
            text += fix.text;
            lastPos = fix.range[1];
        }
        text += originalText.slice(Math.max(0, start, lastPos), end);
        return { range: [start, end], text };
    }
    /**
     * Gets one fix object from the given descriptor.
     * If the descriptor retrieves multiple fixes, this merges those to one.
     * @param {MessageDescriptor} descriptor The report descriptor.
     * @param {SourceCode} sourceCode The source code object to get text between fixes.
     * @returns {({text: string, range: number[]}|null)} The fix for the descriptor
     */
    function normalizeFixes(descriptor, sourceCode) {
        if (typeof descriptor.fix !== "function") {
            return null;
        }
        // @type {null | Fix | Fix[] | IterableIterator<Fix>}
        const fix = descriptor.fix(ruleFixer);
        // Merge to one.
        if (fix && Symbol.iterator in fix) {
            return mergeFixes(Array.from(fix), sourceCode);
        }
        return fix;
    }
    /**
     * Gets an array of suggestion objects from the given descriptor.
     * @param {MessageDescriptor} descriptor The report descriptor.
     * @param {SourceCode} sourceCode The source code object to get text between fixes.
     * @param {Object} messages Object of meta messages for the rule.
     * @returns {Array<SuggestionResult>} The suggestions for the descriptor
     */
    function mapSuggestions(descriptor, sourceCode, messages) {
        if (!descriptor.suggest || !Array.isArray(descriptor.suggest)) {
            return [];
        }
        return descriptor.suggest.map(suggestInfo => {
            const computedDesc = suggestInfo.desc || messages[suggestInfo.messageId];
            return {
                ...suggestInfo,
                desc: interpolate(computedDesc, suggestInfo.data),
                fix: normalizeFixes(suggestInfo, sourceCode)
            };
        });
    }
    /**
     * Creates information about the report from a descriptor
     * @param {Object} options Information about the problem
     * @param {string} options.ruleId Rule ID
     * @param {(0|1|2)} options.severity Rule severity
     * @param {(ASTNode|null)} options.node Node
     * @param {string} options.message Error message
     * @param {string} [options.messageId] The error message ID.
     * @param {{start: SourceLocation, end: (SourceLocation|null)}} options.loc Start and end location
     * @param {{text: string, range: (number[]|null)}} options.fix The fix object
     * @param {Array<{text: string, range: (number[]|null)}>} options.suggestions The array of suggestions objects
     * @returns {function(...args): ReportInfo} Function that returns information about the report
     */
    function createProblem(options) {
        const problem = {
            ruleId: options.ruleId,
            severity: options.severity,
            message: options.message,
            line: options.loc.start.line,
            column: options.loc.start.column + 1,
            nodeType: options.node && options.node.type || null
        };
        /*
         * If this isn’t in the conditional, some of the tests fail
         * because `messageId` is present in the problem object
         */
        if (options.messageId) {
            problem.messageId = options.messageId;
        }
        if (options.loc.end) {
            problem.endLine = options.loc.end.line;
            problem.endColumn = options.loc.end.column + 1;
        }
        if (options.fix) {
            problem.fix = options.fix;
        }
        if (options.suggestions && options.suggestions.length > 0) {
            problem.suggestions = options.suggestions;
        }
        return problem;
    }
    /**
     * Validates that suggestions are properly defined. Throws if an error is detected.
     * @param {Array<{ desc?: string, messageId?: string }>} suggest The incoming suggest data.
     * @param {Object} messages Object of meta messages for the rule.
     * @returns {void}
     */
    function validateSuggestions(suggest, messages) {
        if (suggest && Array.isArray(suggest)) {
            suggest.forEach(suggestion => {
                if (suggestion.messageId) {
                    const { messageId } = suggestion;
                    if (!messages) {
                        throw new TypeError(`context.report() called with a suggest option with a messageId '${messageId}', but no messages were present in the rule metadata.`);
                    }
                    if (!messages[messageId]) {
                        throw new TypeError(`context.report() called with a suggest option with a messageId '${messageId}' which is not present in the 'messages' config: ${JSON.stringify(messages, null, 2)}`);
                    }
                    if (suggestion.desc) {
                        throw new TypeError("context.report() called with a suggest option that defines both a 'messageId' and an 'desc'. Please only pass one.");
                    }
                }
                else if (!suggestion.desc) {
                    throw new TypeError("context.report() called with a suggest option that doesn't have either a `desc` or `messageId`");
                }
                if (typeof suggestion.fix !== "function") {
                    throw new TypeError(`context.report() called with a suggest option without a fix function. See: ${suggestion}`);
                }
            });
        }
    }
    /**
     * Returns a function that converts the arguments of a `context.report` call from a rule into a reported
     * problem for the Node.js API.
     * @param {{ruleId: string, severity: number, sourceCode: SourceCode, messageIds: Object, disableFixes: boolean}} metadata Metadata for the reported problem
     * @param {SourceCode} sourceCode The `SourceCode` instance for the text being linted
     * @returns {function(...args): ReportInfo} Function that returns information about the report
     */
    module.exports = function createReportTranslator(metadata) {
        /*
         * `createReportTranslator` gets called once per enabled rule per file. It needs to be very performant.
         * The report translator itself (i.e. the function that `createReportTranslator` returns) gets
         * called every time a rule reports a problem, which happens much less frequently (usually, the vast
         * majority of rules don't report any problems for a given file).
         */
        return (...args) => {
            const descriptor = normalizeMultiArgReportCall(...args);
            const messages = metadata.messageIds;
            assertValidNodeInfo(descriptor);
            let computedMessage;
            if (descriptor.messageId) {
                if (!messages) {
                    throw new TypeError("context.report() called with a messageId, but no messages were present in the rule metadata.");
                }
                const id = descriptor.messageId;
                if (descriptor.message) {
                    throw new TypeError("context.report() called with a message and a messageId. Please only pass one.");
                }
                if (!messages || !Object.prototype.hasOwnProperty.call(messages, id)) {
                    throw new TypeError(`context.report() called with a messageId of '${id}' which is not present in the 'messages' config: ${JSON.stringify(messages, null, 2)}`);
                }
                computedMessage = messages[id];
            }
            else if (descriptor.message) {
                computedMessage = descriptor.message;
            }
            else {
                throw new TypeError("Missing `message` property in report() call; add a message that describes the linting problem.");
            }
            validateSuggestions(descriptor.suggest, messages);
            return createProblem({
                ruleId: metadata.ruleId,
                severity: metadata.severity,
                node: descriptor.node,
                message: interpolate(computedMessage, descriptor.data),
                messageId: descriptor.messageId,
                loc: normalizeReportLoc(descriptor),
                fix: metadata.disableFixes ? null : normalizeFixes(descriptor, metadata.sourceCode),
                suggestions: metadata.disableFixes ? [] : mapSuggestions(descriptor, metadata.sourceCode, messages)
            });
        };
    };
},
/* eslint/lib/linter/rule-fixer.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    // none!
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Creates a fix command that inserts text at the specified index in the source text.
     * @param {int} index The 0-based index at which to insert the new text.
     * @param {string} text The text to insert.
     * @returns {Object} The fix command.
     * @private
     */
    function insertTextAt(index, text) {
        return {
            range: [index, index],
            text
        };
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * Creates code fixing commands for rules.
     */
    const ruleFixer = Object.freeze({
        /**
         * Creates a fix command that inserts text after the given node or token.
         * The fix is not applied until applyFixes() is called.
         * @param {ASTNode|Token} nodeOrToken The node or token to insert after.
         * @param {string} text The text to insert.
         * @returns {Object} The fix command.
         */
        insertTextAfter(nodeOrToken, text) {
            return this.insertTextAfterRange(nodeOrToken.range, text);
        },
        /**
         * Creates a fix command that inserts text after the specified range in the source text.
         * The fix is not applied until applyFixes() is called.
         * @param {int[]} range The range to replace, first item is start of range, second
         *      is end of range.
         * @param {string} text The text to insert.
         * @returns {Object} The fix command.
         */
        insertTextAfterRange(range, text) {
            return insertTextAt(range[1], text);
        },
        /**
         * Creates a fix command that inserts text before the given node or token.
         * The fix is not applied until applyFixes() is called.
         * @param {ASTNode|Token} nodeOrToken The node or token to insert before.
         * @param {string} text The text to insert.
         * @returns {Object} The fix command.
         */
        insertTextBefore(nodeOrToken, text) {
            return this.insertTextBeforeRange(nodeOrToken.range, text);
        },
        /**
         * Creates a fix command that inserts text before the specified range in the source text.
         * The fix is not applied until applyFixes() is called.
         * @param {int[]} range The range to replace, first item is start of range, second
         *      is end of range.
         * @param {string} text The text to insert.
         * @returns {Object} The fix command.
         */
        insertTextBeforeRange(range, text) {
            return insertTextAt(range[0], text);
        },
        /**
         * Creates a fix command that replaces text at the node or token.
         * The fix is not applied until applyFixes() is called.
         * @param {ASTNode|Token} nodeOrToken The node or token to remove.
         * @param {string} text The text to insert.
         * @returns {Object} The fix command.
         */
        replaceText(nodeOrToken, text) {
            return this.replaceTextRange(nodeOrToken.range, text);
        },
        /**
         * Creates a fix command that replaces text at the specified range in the source text.
         * The fix is not applied until applyFixes() is called.
         * @param {int[]} range The range to replace, first item is start of range, second
         *      is end of range.
         * @param {string} text The text to insert.
         * @returns {Object} The fix command.
         */
        replaceTextRange(range, text) {
            return {
                range,
                text
            };
        },
        /**
         * Creates a fix command that removes the node or token from the source.
         * The fix is not applied until applyFixes() is called.
         * @param {ASTNode|Token} nodeOrToken The node or token to remove.
         * @returns {Object} The fix command.
         */
        remove(nodeOrToken) {
            return this.removeRange(nodeOrToken.range);
        },
        /**
         * Creates a fix command that removes the specified range of text from the source.
         * The fix is not applied until applyFixes() is called.
         * @param {int[]} range The range to remove, first item is start of range, second
         *      is end of range.
         * @returns {Object} The fix command.
         */
        removeRange(range) {
            return {
                range,
                text: ""
            };
        }
    });
    module.exports = ruleFixer;
},
/* eslint/lib/linter/interpolate.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    module.exports = (text, data) => {
        if (!data) {
            return text;
        }
        // Substitution content for any {{ }} markers.
        return text.replace(/\{\{([^{}]+?)\}\}/gu, (fullMatch, termWithWhitespace) => {
            const term = termWithWhitespace.trim();
            if (term in data) {
                return data[term];
            }
            // Preserve old behavior: If parameter name not provided, don't replace it.
            return fullMatch;
        });
    };
},
/* eslint/lib/linter/rules.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const builtInRules = require(118) /* ../rules */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Normalizes a rule module to the new-style API
     * @param {(Function|{create: Function})} rule A rule object, which can either be a function
     * ("old-style") or an object with a `create` method ("new-style")
     * @returns {{create: Function}} A new-style rule.
     */
    function normalizeRule(rule) {
        return typeof rule === "function" ? Object.assign({ create: rule }, rule) : rule;
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    class Rules {
        constructor() {
            this._rules = Object.create(null);
        }
        /**
         * Registers a rule module for rule id in storage.
         * @param {string} ruleId Rule id (file name).
         * @param {Function} ruleModule Rule handler.
         * @returns {void}
         */
        define(ruleId, ruleModule) {
            this._rules[ruleId] = normalizeRule(ruleModule);
        }
        /**
         * Access rule handler by id (file name).
         * @param {string} ruleId Rule id (file name).
         * @returns {{create: Function, schema: JsonSchema[]}}
         * A rule. This is normalized to always have the new-style shape with a `create` method.
         */
        get(ruleId) {
            if (typeof this._rules[ruleId] === "string") {
                this.define(ruleId, require(this._rules[ruleId]));
            }
            if (this._rules[ruleId]) {
                return this._rules[ruleId];
            }
            if (builtInRules.has(ruleId)) {
                return builtInRules.get(ruleId);
            }
            return null;
        }
        *[Symbol.iterator]() {
            yield* builtInRules;
            for (const ruleId of Object.keys(this._rules)) {
                yield [ruleId, this.get(ruleId)];
            }
        }
    }
    module.exports = Rules;
},
/* eslint/lib/linter/safe-emitter.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Typedefs
    //------------------------------------------------------------------------------
    /**
     * An event emitter
     * @typedef {Object} SafeEmitter
     * @property {function(eventName: string, listenerFunc: Function): void} on Adds a listener for a given event name
     * @property {function(eventName: string, arg1?: any, arg2?: any, arg3?: any)} emit Emits an event with a given name.
     * This calls all the listeners that were listening for that name, with `arg1`, `arg2`, and `arg3` as arguments.
     * @property {function(): string[]} eventNames Gets the list of event names that have registered listeners.
     */
    /**
     * Creates an object which can listen for and emit events.
     * This is similar to the EventEmitter API in Node's standard library, but it has a few differences.
     * The goal is to allow multiple modules to attach arbitrary listeners to the same emitter, without
     * letting the modules know about each other at all.
     * 1. It has no special keys like `error` and `newListener`, which would allow modules to detect when
     * another module throws an error or registers a listener.
     * 2. It calls listener functions without any `this` value. (`EventEmitter` calls listeners with a
     * `this` value of the emitter instance, which would give listeners access to other listeners.)
     * @returns {SafeEmitter} An emitter
     */
    module.exports = () => {
        const listeners = Object.create(null);
        return Object.freeze({
            on(eventName, listener) {
                if (eventName in listeners) {
                    listeners[eventName].push(listener);
                }
                else {
                    listeners[eventName] = [listener];
                }
            },
            emit(eventName, ...args) {
                if (eventName in listeners) {
                    listeners[eventName].forEach(listener => listener(...args));
                }
            },
            eventNames() {
                return Object.keys(listeners);
            }
        });
    };
},
/* eslint/lib/linter/source-code-fixer.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const debug = require(120) /* debug */("eslint:source-code-fixer");
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const BOM = "\uFEFF";
    /**
     * Compares items in a messages array by range.
     * @param {Message} a The first message.
     * @param {Message} b The second message.
     * @returns {int} -1 if a comes before b, 1 if a comes after b, 0 if equal.
     * @private
     */
    function compareMessagesByFixRange(a, b) {
        return a.fix.range[0] - b.fix.range[0] || a.fix.range[1] - b.fix.range[1];
    }
    /**
     * Compares items in a messages array by line and column.
     * @param {Message} a The first message.
     * @param {Message} b The second message.
     * @returns {int} -1 if a comes before b, 1 if a comes after b, 0 if equal.
     * @private
     */
    function compareMessagesByLocation(a, b) {
        return a.line - b.line || a.column - b.column;
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * Utility for apply fixes to source code.
     * @constructor
     */
    function SourceCodeFixer() {
        Object.freeze(this);
    }
    /**
     * Applies the fixes specified by the messages to the given text. Tries to be
     * smart about the fixes and won't apply fixes over the same area in the text.
     * @param {string} sourceText The text to apply the changes to.
     * @param {Message[]} messages The array of messages reported by ESLint.
     * @param {boolean|Function} [shouldFix=true] Determines whether each message should be fixed
     * @returns {Object} An object containing the fixed text and any unfixed messages.
     */
    SourceCodeFixer.applyFixes = function (sourceText, messages, shouldFix) {
        debug("Applying fixes");
        if (shouldFix === false) {
            debug("shouldFix parameter was false, not attempting fixes");
            return {
                fixed: false,
                messages,
                output: sourceText
            };
        }
        // clone the array
        const remainingMessages = [], fixes = [], bom = sourceText.startsWith(BOM) ? BOM : "", text = bom ? sourceText.slice(1) : sourceText;
        let lastPos = Number.NEGATIVE_INFINITY, output = bom;
        /**
         * Try to use the 'fix' from a problem.
         * @param   {Message} problem The message object to apply fixes from
         * @returns {boolean}         Whether fix was successfully applied
         */
        function attemptFix(problem) {
            const fix = problem.fix;
            const start = fix.range[0];
            const end = fix.range[1];
            // Remain it as a problem if it's overlapped or it's a negative range
            if (lastPos >= start || start > end) {
                remainingMessages.push(problem);
                return false;
            }
            // Remove BOM.
            if ((start < 0 && end >= 0) || (start === 0 && fix.text.startsWith(BOM))) {
                output = "";
            }
            // Make output to this fix.
            output += text.slice(Math.max(0, lastPos), Math.max(0, start));
            output += fix.text;
            lastPos = end;
            return true;
        }
        messages.forEach(problem => {
            if (Object.prototype.hasOwnProperty.call(problem, "fix")) {
                fixes.push(problem);
            }
            else {
                remainingMessages.push(problem);
            }
        });
        if (fixes.length) {
            debug("Found fixes to apply");
            let fixesWereApplied = false;
            for (const problem of fixes.sort(compareMessagesByFixRange)) {
                if (typeof shouldFix !== "function" || shouldFix(problem)) {
                    attemptFix(problem);
                    /*
                     * The only time attemptFix will fail is if a previous fix was
                     * applied which conflicts with it.  So we can mark this as true.
                     */
                    fixesWereApplied = true;
                }
                else {
                    remainingMessages.push(problem);
                }
            }
            output += text.slice(Math.max(0, lastPos));
            return {
                fixed: fixesWereApplied,
                messages: remainingMessages.sort(compareMessagesByLocation),
                output
            };
        }
        debug("No fixes to apply");
        return {
            fixed: false,
            messages,
            output: bom + text
        };
    };
    module.exports = SourceCodeFixer;
},
/* eslint/lib/linter/timing.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /* istanbul ignore next */
    /**
     * Align the string to left
     * @param {string} str string to evaluate
     * @param {int} len length of the string
     * @param {string} ch delimiter character
     * @returns {string} modified string
     * @private
     */
    function alignLeft(str, len, ch) {
        return str + new Array(len - str.length + 1).join(ch || " ");
    }
    /* istanbul ignore next */
    /**
     * Align the string to right
     * @param {string} str string to evaluate
     * @param {int} len length of the string
     * @param {string} ch delimiter character
     * @returns {string} modified string
     * @private
     */
    function alignRight(str, len, ch) {
        return new Array(len - str.length + 1).join(ch || " ") + str;
    }
    //------------------------------------------------------------------------------
    // Module definition
    //------------------------------------------------------------------------------
    const enabled = !!process.env.TIMING;
    const HEADERS = ["Rule", "Time (ms)", "Relative"];
    const ALIGN = [alignLeft, alignRight, alignRight];
    /* istanbul ignore next */
    /**
     * display the data
     * @param {Object} data Data object to be displayed
     * @returns {void} prints modified string with console.log
     * @private
     */
    function display(data) {
        let total = 0;
        const rows = Object.keys(data)
            .map(key => {
            const time = data[key];
            total += time;
            return [key, time];
        })
            .sort((a, b) => b[1] - a[1])
            .slice(0, 10);
        rows.forEach(row => {
            row.push(`${(row[1] * 100 / total).toFixed(1)}%`);
            row[1] = row[1].toFixed(3);
        });
        rows.unshift(HEADERS);
        const widths = [];
        rows.forEach(row => {
            const len = row.length;
            for (let i = 0; i < len; i++) {
                const n = row[i].length;
                if (!widths[i] || n > widths[i]) {
                    widths[i] = n;
                }
            }
        });
        const table = rows.map(row => (row
            .map((cell, index) => ALIGN[index](cell, widths[index]))
            .join(" | ")));
        table.splice(1, 0, widths.map((width, index) => {
            const extraAlignment = index !== 0 && index !== widths.length - 1 ? 2 : 1;
            return ALIGN[index](":", width + extraAlignment, "-");
        }).join("|"));
        console.log(table.join("\n")); // eslint-disable-line no-console
    }
    /* istanbul ignore next */
    module.exports = (function () {
        const data = Object.create(null);
        /**
         * Time the run
         * @param {*} key key from the data object
         * @param {Function} fn function to be called
         * @returns {Function} function to be executed
         * @private
         */
        function time(key, fn) {
            if (typeof data[key] === "undefined") {
                data[key] = 0;
            }
            return function (...args) {
                let t = process.hrtime();
                fn(...args);
                t = process.hrtime(t);
                data[key] += t[0] * 1e3 + t[1] / 1e6;
            };
        }
        if (enabled) {
            process.on("exit", () => {
                display(data);
            });
        }
        return {
            time,
            enabled
        };
    }());
},
/* eslint/conf/replacements.json */ function _(require, module, exports) {
    module.exports = {
        "rules": {
            "generator-star": ["generator-star-spacing"],
            "global-strict": ["strict"],
            "no-arrow-condition": ["no-confusing-arrow", "no-constant-condition"],
            "no-comma-dangle": ["comma-dangle"],
            "no-empty-class": ["no-empty-character-class"],
            "no-empty-label": ["no-labels"],
            "no-extra-strict": ["strict"],
            "no-reserved-keys": ["quote-props"],
            "no-space-before-semi": ["semi-spacing"],
            "no-wrap-func": ["no-extra-parens"],
            "space-after-function-name": ["space-before-function-paren"],
            "space-after-keywords": ["keyword-spacing"],
            "space-before-function-parentheses": ["space-before-function-paren"],
            "space-before-keywords": ["keyword-spacing"],
            "space-in-brackets": ["object-curly-spacing", "array-bracket-spacing", "computed-property-spacing"],
            "space-return-throw-case": ["keyword-spacing"],
            "space-unary-word-ops": ["space-unary-ops"],
            "spaced-line-comment": ["spaced-comment"]
        }
    };
},
/* eslint/lib/cli-engine/cascading-config-array-factory.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const os = require("os");
    const path = require("path");
    const { validateConfigArray } = require(116) /* ../shared/config-validator */;
    const { ConfigArrayFactory } = require(519) /* ./config-array-factory */;
    const { ConfigArray, ConfigDependency, IgnorePattern } = require(525) /* ./config-array */;
    const loadRules = require(565) /* ./load-rules */;
    const debug = require(120) /* debug */("eslint:cascading-config-array-factory");
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    // Define types for VSCode IntelliSense.
    /** @typedef {import("../shared/types").ConfigData} ConfigData */
    /** @typedef {import("../shared/types").Parser} Parser */
    /** @typedef {import("../shared/types").Plugin} Plugin */
    /** @typedef {ReturnType<ConfigArrayFactory["create"]>} ConfigArray */
    /**
     * @typedef {Object} CascadingConfigArrayFactoryOptions
     * @property {Map<string,Plugin>} [additionalPluginPool] The map for additional plugins.
     * @property {ConfigData} [baseConfig] The config by `baseConfig` option.
     * @property {ConfigData} [cliConfig] The config by CLI options (`--env`, `--global`, `--ignore-pattern`, `--parser`, `--parser-options`, `--plugin`, and `--rule`). CLI options overwrite the setting in config files.
     * @property {string} [cwd] The base directory to start lookup.
     * @property {string} [ignorePath] The path to the alternative file of `.eslintignore`.
     * @property {string[]} [rulePaths] The value of `--rulesdir` option.
     * @property {string} [specificConfigPath] The value of `--config` option.
     * @property {boolean} [useEslintrc] if `false` then it doesn't load config files.
     */
    /**
     * @typedef {Object} CascadingConfigArrayFactoryInternalSlots
     * @property {ConfigArray} baseConfigArray The config array of `baseConfig` option.
     * @property {ConfigData} baseConfigData The config data of `baseConfig` option. This is used to reset `baseConfigArray`.
     * @property {ConfigArray} cliConfigArray The config array of CLI options.
     * @property {ConfigData} cliConfigData The config data of CLI options. This is used to reset `cliConfigArray`.
     * @property {ConfigArrayFactory} configArrayFactory The factory for config arrays.
     * @property {Map<string, ConfigArray>} configCache The cache from directory paths to config arrays.
     * @property {string} cwd The base directory to start lookup.
     * @property {WeakMap<ConfigArray, ConfigArray>} finalizeCache The cache from config arrays to finalized config arrays.
     * @property {string} [ignorePath] The path to the alternative file of `.eslintignore`.
     * @property {string[]|null} rulePaths The value of `--rulesdir` option. This is used to reset `baseConfigArray`.
     * @property {string|null} specificConfigPath The value of `--config` option. This is used to reset `cliConfigArray`.
     * @property {boolean} useEslintrc if `false` then it doesn't load config files.
     */
    /** @type {WeakMap<CascadingConfigArrayFactory, CascadingConfigArrayFactoryInternalSlots>} */
    const internalSlotsMap = new WeakMap();
    /**
     * Create the config array from `baseConfig` and `rulePaths`.
     * @param {CascadingConfigArrayFactoryInternalSlots} slots The slots.
     * @returns {ConfigArray} The config array of the base configs.
     */
    function createBaseConfigArray({ configArrayFactory, baseConfigData, rulePaths, cwd }) {
        const baseConfigArray = configArrayFactory.create(baseConfigData, { name: "BaseConfig" });
        /*
         * Create the config array element for the default ignore patterns.
         * This element has `ignorePattern` property that ignores the default
         * patterns in the current working directory.
         */
        baseConfigArray.unshift(configArrayFactory.create({ ignorePatterns: IgnorePattern.DefaultPatterns }, { name: "DefaultIgnorePattern" })[0]);
        /*
         * Load rules `--rulesdir` option as a pseudo plugin.
         * Use a pseudo plugin to define rules of `--rulesdir`, so we can validate
         * the rule's options with only information in the config array.
         */
        if (rulePaths && rulePaths.length > 0) {
            baseConfigArray.push({
                name: "--rulesdir",
                filePath: "",
                plugins: {
                    "": new ConfigDependency({
                        definition: {
                            rules: rulePaths.reduce((map, rulesPath) => Object.assign(map, loadRules(rulesPath, cwd)), {})
                        },
                        filePath: "",
                        id: "",
                        importerName: "--rulesdir",
                        importerPath: ""
                    })
                }
            });
        }
        return baseConfigArray;
    }
    /**
     * Create the config array from CLI options.
     * @param {CascadingConfigArrayFactoryInternalSlots} slots The slots.
     * @returns {ConfigArray} The config array of the base configs.
     */
    function createCLIConfigArray({ cliConfigData, configArrayFactory, ignorePath, specificConfigPath }) {
        const cliConfigArray = configArrayFactory.create(cliConfigData, { name: "CLIOptions" });
        cliConfigArray.unshift(...(ignorePath
            ? configArrayFactory.loadESLintIgnore(ignorePath)
            : configArrayFactory.loadDefaultESLintIgnore()));
        if (specificConfigPath) {
            cliConfigArray.unshift(...configArrayFactory.loadFile(specificConfigPath, { name: "--config" }));
        }
        return cliConfigArray;
    }
    /**
     * The error type when there are files matched by a glob, but all of them have been ignored.
     */
    class ConfigurationNotFoundError extends Error {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {string} directoryPath The directory path.
         */
        constructor(directoryPath) {
            super(`No ESLint configuration found in ${directoryPath}.`);
            this.messageTemplate = "no-config-found";
            this.messageData = { directoryPath };
        }
    }
    /**
     * This class provides the functionality that enumerates every file which is
     * matched by given glob patterns and that configuration.
     */
    class CascadingConfigArrayFactory {
        /**
         * Initialize this enumerator.
         * @param {CascadingConfigArrayFactoryOptions} options The options.
         */
        constructor({ additionalPluginPool = new Map(), baseConfig: baseConfigData = null, cliConfig: cliConfigData = null, cwd = process.cwd(), ignorePath, resolvePluginsRelativeTo = cwd, rulePaths = [], specificConfigPath = null, useEslintrc = true } = {}) {
            const configArrayFactory = new ConfigArrayFactory({
                additionalPluginPool,
                cwd,
                resolvePluginsRelativeTo
            });
            internalSlotsMap.set(this, {
                baseConfigArray: createBaseConfigArray({
                    baseConfigData,
                    configArrayFactory,
                    cwd,
                    rulePaths
                }),
                baseConfigData,
                cliConfigArray: createCLIConfigArray({
                    cliConfigData,
                    configArrayFactory,
                    ignorePath,
                    specificConfigPath
                }),
                cliConfigData,
                configArrayFactory,
                configCache: new Map(),
                cwd,
                finalizeCache: new WeakMap(),
                ignorePath,
                rulePaths,
                specificConfigPath,
                useEslintrc
            });
        }
        /**
         * The path to the current working directory.
         * This is used by tests.
         * @type {string}
         */
        get cwd() {
            const { cwd } = internalSlotsMap.get(this);
            return cwd;
        }
        /**
         * Get the config array of a given file.
         * If `filePath` was not given, it returns the config which contains only
         * `baseConfigData` and `cliConfigData`.
         * @param {string} [filePath] The file path to a file.
         * @param {Object} [options] The options.
         * @param {boolean} [options.ignoreNotFoundError] If `true` then it doesn't throw `ConfigurationNotFoundError`.
         * @returns {ConfigArray} The config array of the file.
         */
        getConfigArrayForFile(filePath, { ignoreNotFoundError = false } = {}) {
            const { baseConfigArray, cliConfigArray, cwd } = internalSlotsMap.get(this);
            if (!filePath) {
                return new ConfigArray(...baseConfigArray, ...cliConfigArray);
            }
            const directoryPath = path.dirname(path.resolve(cwd, filePath));
            debug(`Load config files for ${directoryPath}.`);
            return this._finalizeConfigArray(this._loadConfigInAncestors(directoryPath), directoryPath, ignoreNotFoundError);
        }
        /**
         * Clear config cache.
         * @returns {void}
         */
        clearCache() {
            const slots = internalSlotsMap.get(this);
            slots.baseConfigArray = createBaseConfigArray(slots);
            slots.cliConfigArray = createCLIConfigArray(slots);
            slots.configCache.clear();
        }
        /**
         * Load and normalize config files from the ancestor directories.
         * @param {string} directoryPath The path to a leaf directory.
         * @returns {ConfigArray} The loaded config.
         * @private
         */
        _loadConfigInAncestors(directoryPath) {
            const { baseConfigArray, configArrayFactory, configCache, cwd, useEslintrc } = internalSlotsMap.get(this);
            if (!useEslintrc) {
                return baseConfigArray;
            }
            let configArray = configCache.get(directoryPath);
            // Hit cache.
            if (configArray) {
                debug(`Cache hit: ${directoryPath}.`);
                return configArray;
            }
            debug(`No cache found: ${directoryPath}.`);
            const homePath = os.homedir();
            // Consider this is root.
            if (directoryPath === homePath && cwd !== homePath) {
                debug("Stop traversing because of considered root.");
                return this._cacheConfig(directoryPath, baseConfigArray);
            }
            // Load the config on this directory.
            try {
                configArray = configArrayFactory.loadInDirectory(directoryPath);
            }
            catch (error) {
                /* istanbul ignore next */
                if (error.code === "EACCES") {
                    debug("Stop traversing because of 'EACCES' error.");
                    return this._cacheConfig(directoryPath, baseConfigArray);
                }
                throw error;
            }
            if (configArray.length > 0 && configArray.isRoot()) {
                debug("Stop traversing because of 'root:true'.");
                configArray.unshift(...baseConfigArray);
                return this._cacheConfig(directoryPath, configArray);
            }
            // Load from the ancestors and merge it.
            const parentPath = path.dirname(directoryPath);
            const parentConfigArray = parentPath && parentPath !== directoryPath
                ? this._loadConfigInAncestors(parentPath)
                : baseConfigArray;
            if (configArray.length > 0) {
                configArray.unshift(...parentConfigArray);
            }
            else {
                configArray = parentConfigArray;
            }
            // Cache and return.
            return this._cacheConfig(directoryPath, configArray);
        }
        /**
         * Freeze and cache a given config.
         * @param {string} directoryPath The path to a directory as a cache key.
         * @param {ConfigArray} configArray The config array as a cache value.
         * @returns {ConfigArray} The `configArray` (frozen).
         */
        _cacheConfig(directoryPath, configArray) {
            const { configCache } = internalSlotsMap.get(this);
            Object.freeze(configArray);
            configCache.set(directoryPath, configArray);
            return configArray;
        }
        /**
         * Finalize a given config array.
         * Concatenate `--config` and other CLI options.
         * @param {ConfigArray} configArray The parent config array.
         * @param {string} directoryPath The path to the leaf directory to find config files.
         * @param {boolean} ignoreNotFoundError If `true` then it doesn't throw `ConfigurationNotFoundError`.
         * @returns {ConfigArray} The loaded config.
         * @private
         */
        _finalizeConfigArray(configArray, directoryPath, ignoreNotFoundError) {
            const { cliConfigArray, configArrayFactory, finalizeCache, useEslintrc } = internalSlotsMap.get(this);
            let finalConfigArray = finalizeCache.get(configArray);
            if (!finalConfigArray) {
                finalConfigArray = configArray;
                // Load the personal config if there are no regular config files.
                if (useEslintrc &&
                    configArray.every(c => !c.filePath) &&
                    cliConfigArray.every(c => !c.filePath) // `--config` option can be a file.
                ) {
                    debug("Loading the config file of the home directory.");
                    finalConfigArray = configArrayFactory.loadInDirectory(os.homedir(), { name: "PersonalConfig", parent: finalConfigArray });
                }
                // Apply CLI options.
                if (cliConfigArray.length > 0) {
                    finalConfigArray = finalConfigArray.concat(cliConfigArray);
                }
                // Validate rule settings and environments.
                validateConfigArray(finalConfigArray);
                // Cache it.
                Object.freeze(finalConfigArray);
                finalizeCache.set(configArray, finalConfigArray);
                debug("Configuration was determined: %o on %s", finalConfigArray, directoryPath);
            }
            // At least one element (the default ignore patterns) exists.
            if (!ignoreNotFoundError && useEslintrc && finalConfigArray.length <= 1) {
                throw new ConfigurationNotFoundError(directoryPath);
            }
            return finalConfigArray;
        }
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    module.exports = { CascadingConfigArrayFactory };
},
/* eslint/lib/cli-engine/config-array-factory.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const fs = require("fs");
    const path = require("path");
    const importFresh = require(520) /* import-fresh */;
    const stripComments = require(524) /* strip-json-comments */;
    const { validateConfigSchema } = require(116) /* ../shared/config-validator */;
    const naming = require(83) /* ../shared/naming */;
    const ModuleResolver = require(84) /* ../shared/relative-module-resolver */;
    const { ConfigArray, ConfigDependency, IgnorePattern, OverrideTester } = require(525) /* ./config-array */;
    const debug = require(120) /* debug */("eslint:config-array-factory");
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const eslintRecommendedPath = path.resolve(__dirname, "../../conf/eslint-recommended.js");
    const eslintAllPath = path.resolve(__dirname, "../../conf/eslint-all.js");
    const configFilenames = [
        ".eslintrc.js",
        ".eslintrc.cjs",
        ".eslintrc.yaml",
        ".eslintrc.yml",
        ".eslintrc.json",
        ".eslintrc",
        "package.json"
    ];
    // Define types for VSCode IntelliSense.
    /** @typedef {import("../shared/types").ConfigData} ConfigData */
    /** @typedef {import("../shared/types").OverrideConfigData} OverrideConfigData */
    /** @typedef {import("../shared/types").Parser} Parser */
    /** @typedef {import("../shared/types").Plugin} Plugin */
    /** @typedef {import("./config-array/config-dependency").DependentParser} DependentParser */
    /** @typedef {import("./config-array/config-dependency").DependentPlugin} DependentPlugin */
    /** @typedef {ConfigArray[0]} ConfigArrayElement */
    /**
     * @typedef {Object} ConfigArrayFactoryOptions
     * @property {Map<string,Plugin>} [additionalPluginPool] The map for additional plugins.
     * @property {string} [cwd] The path to the current working directory.
     * @property {string} [resolvePluginsRelativeTo] A path to the directory that plugins should be resolved from. Defaults to `cwd`.
     */
    /**
     * @typedef {Object} ConfigArrayFactoryInternalSlots
     * @property {Map<string,Plugin>} additionalPluginPool The map for additional plugins.
     * @property {string} cwd The path to the current working directory.
     * @property {string} resolvePluginsRelativeTo An absolute path the the directory that plugins should be resolved from.
     */
    /** @type {WeakMap<ConfigArrayFactory, ConfigArrayFactoryInternalSlots>} */
    const internalSlotsMap = new WeakMap();
    /**
     * Check if a given string is a file path.
     * @param {string} nameOrPath A module name or file path.
     * @returns {boolean} `true` if the `nameOrPath` is a file path.
     */
    function isFilePath(nameOrPath) {
        return (/^\.{1,2}[/\\]/u.test(nameOrPath) ||
            path.isAbsolute(nameOrPath));
    }
    /**
     * Convenience wrapper for synchronously reading file contents.
     * @param {string} filePath The filename to read.
     * @returns {string} The file contents, with the BOM removed.
     * @private
     */
    function readFile(filePath) {
        return fs.readFileSync(filePath, "utf8").replace(/^\ufeff/u, "");
    }
    /**
     * Loads a YAML configuration from a file.
     * @param {string} filePath The filename to load.
     * @returns {ConfigData} The configuration object from the file.
     * @throws {Error} If the file cannot be read.
     * @private
     */
    function loadYAMLConfigFile(filePath) {
        debug(`Loading YAML config file: ${filePath}`);
        // lazy load YAML to improve performance when not used
        const yaml = require(535) /* js-yaml */;
        try {
            // empty YAML file can be null, so always use
            return yaml.safeLoad(readFile(filePath)) || {};
        }
        catch (e) {
            debug(`Error reading YAML file: ${filePath}`);
            e.message = `Cannot read config file: ${filePath}\nError: ${e.message}`;
            throw e;
        }
    }
    /**
     * Loads a JSON configuration from a file.
     * @param {string} filePath The filename to load.
     * @returns {ConfigData} The configuration object from the file.
     * @throws {Error} If the file cannot be read.
     * @private
     */
    function loadJSONConfigFile(filePath) {
        debug(`Loading JSON config file: ${filePath}`);
        try {
            return JSON.parse(stripComments(readFile(filePath)));
        }
        catch (e) {
            debug(`Error reading JSON file: ${filePath}`);
            e.message = `Cannot read config file: ${filePath}\nError: ${e.message}`;
            e.messageTemplate = "failed-to-read-json";
            e.messageData = {
                path: filePath,
                message: e.message
            };
            throw e;
        }
    }
    /**
     * Loads a legacy (.eslintrc) configuration from a file.
     * @param {string} filePath The filename to load.
     * @returns {ConfigData} The configuration object from the file.
     * @throws {Error} If the file cannot be read.
     * @private
     */
    function loadLegacyConfigFile(filePath) {
        debug(`Loading legacy config file: ${filePath}`);
        // lazy load YAML to improve performance when not used
        const yaml = require(535) /* js-yaml */;
        try {
            return yaml.safeLoad(stripComments(readFile(filePath))) || /* istanbul ignore next */ {};
        }
        catch (e) {
            debug("Error reading YAML file: %s\n%o", filePath, e);
            e.message = `Cannot read config file: ${filePath}\nError: ${e.message}`;
            throw e;
        }
    }
    /**
     * Loads a JavaScript configuration from a file.
     * @param {string} filePath The filename to load.
     * @returns {ConfigData} The configuration object from the file.
     * @throws {Error} If the file cannot be read.
     * @private
     */
    function loadJSConfigFile(filePath) {
        debug(`Loading JS config file: ${filePath}`);
        try {
            return importFresh(filePath);
        }
        catch (e) {
            debug(`Error reading JavaScript file: ${filePath}`);
            e.message = `Cannot read config file: ${filePath}\nError: ${e.message}`;
            throw e;
        }
    }
    /**
     * Loads a configuration from a package.json file.
     * @param {string} filePath The filename to load.
     * @returns {ConfigData} The configuration object from the file.
     * @throws {Error} If the file cannot be read.
     * @private
     */
    function loadPackageJSONConfigFile(filePath) {
        debug(`Loading package.json config file: ${filePath}`);
        try {
            const packageData = loadJSONConfigFile(filePath);
            if (!Object.hasOwnProperty.call(packageData, "eslintConfig")) {
                throw Object.assign(new Error("package.json file doesn't have 'eslintConfig' field."), { code: "ESLINT_CONFIG_FIELD_NOT_FOUND" });
            }
            return packageData.eslintConfig;
        }
        catch (e) {
            debug(`Error reading package.json file: ${filePath}`);
            e.message = `Cannot read config file: ${filePath}\nError: ${e.message}`;
            throw e;
        }
    }
    /**
     * Loads a `.eslintignore` from a file.
     * @param {string} filePath The filename to load.
     * @returns {string[]} The ignore patterns from the file.
     * @private
     */
    function loadESLintIgnoreFile(filePath) {
        debug(`Loading .eslintignore file: ${filePath}`);
        try {
            return readFile(filePath)
                .split(/\r?\n/gu)
                .filter(line => line.trim() !== "" && !line.startsWith("#"));
        }
        catch (e) {
            debug(`Error reading .eslintignore file: ${filePath}`);
            e.message = `Cannot read .eslintignore file: ${filePath}\nError: ${e.message}`;
            throw e;
        }
    }
    /**
     * Creates an error to notify about a missing config to extend from.
     * @param {string} configName The name of the missing config.
     * @param {string} importerName The name of the config that imported the missing config
     * @returns {Error} The error object to throw
     * @private
     */
    function configMissingError(configName, importerName) {
        return Object.assign(new Error(`Failed to load config "${configName}" to extend from.`), {
            messageTemplate: "extend-config-missing",
            messageData: { configName, importerName }
        });
    }
    /**
     * Loads a configuration file regardless of the source. Inspects the file path
     * to determine the correctly way to load the config file.
     * @param {string} filePath The path to the configuration.
     * @returns {ConfigData|null} The configuration information.
     * @private
     */
    function loadConfigFile(filePath) {
        switch (path.extname(filePath)) {
            case ".js":
            case ".cjs":
                return loadJSConfigFile(filePath);
            case ".json":
                if (path.basename(filePath) === "package.json") {
                    return loadPackageJSONConfigFile(filePath);
                }
                return loadJSONConfigFile(filePath);
            case ".yaml":
            case ".yml":
                return loadYAMLConfigFile(filePath);
            default:
                return loadLegacyConfigFile(filePath);
        }
    }
    /**
     * Write debug log.
     * @param {string} request The requested module name.
     * @param {string} relativeTo The file path to resolve the request relative to.
     * @param {string} filePath The resolved file path.
     * @returns {void}
     */
    function writeDebugLogForLoading(request, relativeTo, filePath) {
        /* istanbul ignore next */
        if (debug.enabled) {
            let nameAndVersion = null;
            try {
                const packageJsonPath = ModuleResolver.resolve(`${request}/package.json`, relativeTo);
                const { version = "unknown" } = require(packageJsonPath);
                nameAndVersion = `${request}@${version}`;
            }
            catch (error) {
                debug("package.json was not found:", error.message);
                nameAndVersion = request;
            }
            debug("Loaded: %s (%s)", nameAndVersion, filePath);
        }
    }
    /**
     * Concatenate two config data.
     * @param {IterableIterator<ConfigArrayElement>|null} elements The config elements.
     * @param {ConfigArray|null} parentConfigArray The parent config array.
     * @returns {ConfigArray} The concatenated config array.
     */
    function createConfigArray(elements, parentConfigArray) {
        if (!elements) {
            return parentConfigArray || new ConfigArray();
        }
        const configArray = new ConfigArray(...elements);
        if (parentConfigArray && !configArray.isRoot()) {
            configArray.unshift(...parentConfigArray);
        }
        return configArray;
    }
    /**
     * Normalize a given plugin.
     * - Ensure the object to have four properties: configs, environments, processors, and rules.
     * - Ensure the object to not have other properties.
     * @param {Plugin} plugin The plugin to normalize.
     * @returns {Plugin} The normalized plugin.
     */
    function normalizePlugin(plugin) {
        return {
            configs: plugin.configs || {},
            environments: plugin.environments || {},
            processors: plugin.processors || {},
            rules: plugin.rules || {}
        };
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * The factory of `ConfigArray` objects.
     */
    class ConfigArrayFactory {
        /**
         * Initialize this instance.
         * @param {ConfigArrayFactoryOptions} [options] The map for additional plugins.
         */
        constructor({ additionalPluginPool = new Map(), cwd = process.cwd(), resolvePluginsRelativeTo = cwd } = {}) {
            internalSlotsMap.set(this, { additionalPluginPool, cwd, resolvePluginsRelativeTo: path.resolve(cwd, resolvePluginsRelativeTo) });
        }
        /**
         * Create `ConfigArray` instance from a config data.
         * @param {ConfigData|null} configData The config data to create.
         * @param {Object} [options] The options.
         * @param {string} [options.filePath] The path to this config data.
         * @param {string} [options.name] The config name.
         * @param {ConfigArray} [options.parent] The parent config array.
         * @returns {ConfigArray} Loaded config.
         */
        create(configData, { filePath, name, parent } = {}) {
            return createConfigArray(configData
                ? this._normalizeConfigData(configData, filePath, name)
                : null, parent);
        }
        /**
         * Load a config file.
         * @param {string} filePath The path to a config file.
         * @param {Object} [options] The options.
         * @param {string} [options.name] The config name.
         * @param {ConfigArray} [options.parent] The parent config array.
         * @returns {ConfigArray} Loaded config.
         */
        loadFile(filePath, { name, parent } = {}) {
            const { cwd } = internalSlotsMap.get(this);
            const absolutePath = path.resolve(cwd, filePath);
            return createConfigArray(this._loadConfigData(absolutePath, name), parent);
        }
        /**
         * Load the config file on a given directory if exists.
         * @param {string} directoryPath The path to a directory.
         * @param {Object} [options] The options.
         * @param {string} [options.name] The config name.
         * @param {ConfigArray} [options.parent] The parent config array.
         * @returns {ConfigArray} Loaded config. An empty `ConfigArray` if any config doesn't exist.
         */
        loadInDirectory(directoryPath, { name, parent } = {}) {
            const { cwd } = internalSlotsMap.get(this);
            const absolutePath = path.resolve(cwd, directoryPath);
            return createConfigArray(this._loadConfigDataInDirectory(absolutePath, name), parent);
        }
        /**
         * Load `.eslintignore` file.
         * @param {string} filePath The path to a `.eslintignore` file to load.
         * @returns {ConfigArray} Loaded config. An empty `ConfigArray` if any config doesn't exist.
         */
        loadESLintIgnore(filePath) {
            const { cwd } = internalSlotsMap.get(this);
            const absolutePath = path.resolve(cwd, filePath);
            const name = path.relative(cwd, absolutePath);
            const ignorePatterns = loadESLintIgnoreFile(absolutePath);
            return createConfigArray(this._normalizeESLintIgnoreData(ignorePatterns, absolutePath, name));
        }
        /**
         * Load `.eslintignore` file in the current working directory.
         * @returns {ConfigArray} Loaded config. An empty `ConfigArray` if any config doesn't exist.
         */
        loadDefaultESLintIgnore() {
            const { cwd } = internalSlotsMap.get(this);
            const eslintIgnorePath = path.resolve(cwd, ".eslintignore");
            const packageJsonPath = path.resolve(cwd, "package.json");
            if (fs.existsSync(eslintIgnorePath)) {
                return this.loadESLintIgnore(eslintIgnorePath);
            }
            if (fs.existsSync(packageJsonPath)) {
                const data = loadJSONConfigFile(packageJsonPath);
                if (Object.hasOwnProperty.call(data, "eslintIgnore")) {
                    if (!Array.isArray(data.eslintIgnore)) {
                        throw new Error("Package.json eslintIgnore property requires an array of paths");
                    }
                    return createConfigArray(this._normalizeESLintIgnoreData(data.eslintIgnore, packageJsonPath, "eslintIgnore in package.json"));
                }
            }
            return new ConfigArray();
        }
        /**
         * Load a given config file.
         * @param {string} filePath The path to a config file.
         * @param {string} name The config name.
         * @returns {IterableIterator<ConfigArrayElement>} Loaded config.
         * @private
         */
        _loadConfigData(filePath, name) {
            return this._normalizeConfigData(loadConfigFile(filePath), filePath, name);
        }
        /**
         * Load the config file in a given directory if exists.
         * @param {string} directoryPath The path to a directory.
         * @param {string} name The config name.
         * @returns {IterableIterator<ConfigArrayElement> | null} Loaded config. `null` if any config doesn't exist.
         * @private
         */
        _loadConfigDataInDirectory(directoryPath, name) {
            for (const filename of configFilenames) {
                const filePath = path.join(directoryPath, filename);
                if (fs.existsSync(filePath)) {
                    let configData;
                    try {
                        configData = loadConfigFile(filePath);
                    }
                    catch (error) {
                        if (!error || error.code !== "ESLINT_CONFIG_FIELD_NOT_FOUND") {
                            throw error;
                        }
                    }
                    if (configData) {
                        debug(`Config file found: ${filePath}`);
                        return this._normalizeConfigData(configData, filePath, name);
                    }
                }
            }
            debug(`Config file not found on ${directoryPath}`);
            return null;
        }
        /**
         * Normalize a given `.eslintignore` data to config array elements.
         * @param {string[]} ignorePatterns The patterns to ignore files.
         * @param {string|undefined} filePath The file path of this config.
         * @param {string|undefined} name The name of this config.
         * @returns {IterableIterator<ConfigArrayElement>} The normalized config.
         * @private
         */
        *_normalizeESLintIgnoreData(ignorePatterns, filePath, name) {
            const elements = this._normalizeObjectConfigData({ ignorePatterns }, filePath, name);
            // Set `ignorePattern.loose` flag for backward compatibility.
            for (const element of elements) {
                if (element.ignorePattern) {
                    element.ignorePattern.loose = true;
                }
                yield element;
            }
        }
        /**
         * Normalize a given config to an array.
         * @param {ConfigData} configData The config data to normalize.
         * @param {string|undefined} providedFilePath The file path of this config.
         * @param {string|undefined} providedName The name of this config.
         * @returns {IterableIterator<ConfigArrayElement>} The normalized config.
         * @private
         */
        _normalizeConfigData(configData, providedFilePath, providedName) {
            const { cwd } = internalSlotsMap.get(this);
            const filePath = providedFilePath
                ? path.resolve(cwd, providedFilePath)
                : "";
            const name = providedName || (filePath && path.relative(cwd, filePath));
            validateConfigSchema(configData, name || filePath);
            return this._normalizeObjectConfigData(configData, filePath, name);
        }
        /**
         * Normalize a given config to an array.
         * @param {ConfigData|OverrideConfigData} configData The config data to normalize.
         * @param {string} filePath The file path of this config.
         * @param {string} name The name of this config.
         * @returns {IterableIterator<ConfigArrayElement>} The normalized config.
         * @private
         */
        *_normalizeObjectConfigData(configData, filePath, name) {
            const { cwd } = internalSlotsMap.get(this);
            const { files, excludedFiles, ...configBody } = configData;
            const basePath = filePath ? path.dirname(filePath) : cwd;
            const criteria = OverrideTester.create(files, excludedFiles, basePath);
            const elements = this._normalizeObjectConfigDataBody(configBody, filePath, name);
            // Apply the criteria to every element.
            for (const element of elements) {
                // Adopt the base path of the entry file (the outermost base path).
                if (element.criteria) {
                    element.criteria.basePath = basePath;
                }
                if (element.ignorePattern) {
                    element.ignorePattern.basePath = basePath;
                }
                /*
                 * Merge the criteria; this is for only file extension processors in
                 * `overrides` section for now.
                 */
                element.criteria = OverrideTester.and(criteria, element.criteria);
                /*
                 * Remove `root` property to ignore `root` settings which came from
                 * `extends` in `overrides`.
                 */
                if (element.criteria) {
                    element.root = void 0;
                }
                yield element;
            }
        }
        /**
         * Normalize a given config to an array.
         * @param {ConfigData} configData The config data to normalize.
         * @param {string} filePath The file path of this config.
         * @param {string} name The name of this config.
         * @returns {IterableIterator<ConfigArrayElement>} The normalized config.
         * @private
         */
        *_normalizeObjectConfigDataBody({ env, extends: extend, globals, ignorePatterns, noInlineConfig, parser: parserName, parserOptions, plugins: pluginList, processor, reportUnusedDisableDirectives, root, rules, settings, overrides: overrideList = [] }, filePath, name) {
            const extendList = Array.isArray(extend) ? extend : [extend];
            const ignorePattern = ignorePatterns && new IgnorePattern(Array.isArray(ignorePatterns) ? ignorePatterns : [ignorePatterns], filePath ? path.dirname(filePath) : internalSlotsMap.get(this).cwd);
            // Flatten `extends`.
            for (const extendName of extendList.filter(Boolean)) {
                yield* this._loadExtends(extendName, filePath, name);
            }
            // Load parser & plugins.
            const parser = parserName && this._loadParser(parserName, filePath, name);
            const plugins = pluginList && this._loadPlugins(pluginList, filePath, name);
            // Yield pseudo config data for file extension processors.
            if (plugins) {
                yield* this._takeFileExtensionProcessors(plugins, filePath, name);
            }
            // Yield the config data except `extends` and `overrides`.
            yield {
                // Debug information.
                name,
                filePath,
                // Config data.
                criteria: null,
                env,
                globals,
                ignorePattern,
                noInlineConfig,
                parser,
                parserOptions,
                plugins,
                processor,
                reportUnusedDisableDirectives,
                root,
                rules,
                settings
            };
            // Flatten `overries`.
            for (let i = 0; i < overrideList.length; ++i) {
                yield* this._normalizeObjectConfigData(overrideList[i], filePath, `${name}#overrides[${i}]`);
            }
        }
        /**
         * Load configs of an element in `extends`.
         * @param {string} extendName The name of a base config.
         * @param {string} importerPath The file path which has the `extends` property.
         * @param {string} importerName The name of the config which has the `extends` property.
         * @returns {IterableIterator<ConfigArrayElement>} The normalized config.
         * @private
         */
        _loadExtends(extendName, importerPath, importerName) {
            debug("Loading {extends:%j} relative to %s", extendName, importerPath);
            try {
                if (extendName.startsWith("eslint:")) {
                    return this._loadExtendedBuiltInConfig(extendName, importerName);
                }
                if (extendName.startsWith("plugin:")) {
                    return this._loadExtendedPluginConfig(extendName, importerPath, importerName);
                }
                return this._loadExtendedShareableConfig(extendName, importerPath, importerName);
            }
            catch (error) {
                error.message += `\nReferenced from: ${importerPath || importerName}`;
                throw error;
            }
        }
        /**
         * Load configs of an element in `extends`.
         * @param {string} extendName The name of a base config.
         * @param {string} importerName The name of the config which has the `extends` property.
         * @returns {IterableIterator<ConfigArrayElement>} The normalized config.
         * @private
         */
        _loadExtendedBuiltInConfig(extendName, importerName) {
            const name = `${importerName} » ${extendName}`;
            if (extendName === "eslint:recommended") {
                return this._loadConfigData(eslintRecommendedPath, name);
            }
            if (extendName === "eslint:all") {
                return this._loadConfigData(eslintAllPath, name);
            }
            throw configMissingError(extendName, importerName);
        }
        /**
         * Load configs of an element in `extends`.
         * @param {string} extendName The name of a base config.
         * @param {string} importerPath The file path which has the `extends` property.
         * @param {string} importerName The name of the config which has the `extends` property.
         * @returns {IterableIterator<ConfigArrayElement>} The normalized config.
         * @private
         */
        _loadExtendedPluginConfig(extendName, importerPath, importerName) {
            const slashIndex = extendName.lastIndexOf("/");
            const pluginName = extendName.slice("plugin:".length, slashIndex);
            const configName = extendName.slice(slashIndex + 1);
            if (isFilePath(pluginName)) {
                throw new Error("'extends' cannot use a file path for plugins.");
            }
            const plugin = this._loadPlugin(pluginName, importerPath, importerName);
            const configData = plugin.definition &&
                plugin.definition.configs[configName];
            if (configData) {
                return this._normalizeConfigData(configData, plugin.filePath, `${importerName} » plugin:${plugin.id}/${configName}`);
            }
            throw plugin.error || configMissingError(extendName, importerPath);
        }
        /**
         * Load configs of an element in `extends`.
         * @param {string} extendName The name of a base config.
         * @param {string} importerPath The file path which has the `extends` property.
         * @param {string} importerName The name of the config which has the `extends` property.
         * @returns {IterableIterator<ConfigArrayElement>} The normalized config.
         * @private
         */
        _loadExtendedShareableConfig(extendName, importerPath, importerName) {
            const { cwd } = internalSlotsMap.get(this);
            const relativeTo = importerPath || path.join(cwd, "__placeholder__.js");
            let request;
            if (isFilePath(extendName)) {
                request = extendName;
            }
            else if (extendName.startsWith(".")) {
                request = `./${extendName}`; // For backward compatibility. A ton of tests depended on this behavior.
            }
            else {
                request = naming.normalizePackageName(extendName, "eslint-config");
            }
            let filePath;
            try {
                filePath = ModuleResolver.resolve(request, relativeTo);
            }
            catch (error) {
                /* istanbul ignore else */
                if (error && error.code === "MODULE_NOT_FOUND") {
                    throw configMissingError(extendName, importerPath);
                }
                throw error;
            }
            writeDebugLogForLoading(request, relativeTo, filePath);
            return this._loadConfigData(filePath, `${importerName} » ${request}`);
        }
        /**
         * Load given plugins.
         * @param {string[]} names The plugin names to load.
         * @param {string} importerPath The path to a config file that imports it. This is just a debug info.
         * @param {string} importerName The name of a config file that imports it. This is just a debug info.
         * @returns {Record<string,DependentPlugin>} The loaded parser.
         * @private
         */
        _loadPlugins(names, importerPath, importerName) {
            return names.reduce((map, name) => {
                if (isFilePath(name)) {
                    throw new Error("Plugins array cannot includes file paths.");
                }
                const plugin = this._loadPlugin(name, importerPath, importerName);
                map[plugin.id] = plugin;
                return map;
            }, {});
        }
        /**
         * Load a given parser.
         * @param {string} nameOrPath The package name or the path to a parser file.
         * @param {string} importerPath The path to a config file that imports it.
         * @param {string} importerName The name of a config file that imports it. This is just a debug info.
         * @returns {DependentParser} The loaded parser.
         */
        _loadParser(nameOrPath, importerPath, importerName) {
            debug("Loading parser %j from %s", nameOrPath, importerPath);
            const { cwd } = internalSlotsMap.get(this);
            const relativeTo = importerPath || path.join(cwd, "__placeholder__.js");
            try {
                const filePath = ModuleResolver.resolve(nameOrPath, relativeTo);
                writeDebugLogForLoading(nameOrPath, relativeTo, filePath);
                return new ConfigDependency({
                    definition: require(filePath),
                    filePath,
                    id: nameOrPath,
                    importerName,
                    importerPath
                });
            }
            catch (error) {
                // If the parser name is "espree", load the espree of ESLint.
                if (nameOrPath === "espree") {
                    debug("Fallback espree.");
                    return new ConfigDependency({
                        definition: require(102) /* espree */,
                        filePath: require.resolve("espree"),
                        id: nameOrPath,
                        importerName,
                        importerPath
                    });
                }
                debug("Failed to load parser '%s' declared in '%s'.", nameOrPath, importerName);
                error.message = `Failed to load parser '${nameOrPath}' declared in '${importerName}': ${error.message}`;
                return new ConfigDependency({
                    error,
                    id: nameOrPath,
                    importerName,
                    importerPath
                });
            }
        }
        /**
         * Load a given plugin.
         * @param {string} name The plugin name to load.
         * @param {string} importerPath The path to a config file that imports it. This is just a debug info.
         * @param {string} importerName The name of a config file that imports it. This is just a debug info.
         * @returns {DependentPlugin} The loaded plugin.
         * @private
         */
        _loadPlugin(name, importerPath, importerName) {
            debug("Loading plugin %j from %s", name, importerPath);
            const { additionalPluginPool, resolvePluginsRelativeTo } = internalSlotsMap.get(this);
            const request = naming.normalizePackageName(name, "eslint-plugin");
            const id = naming.getShorthandName(request, "eslint-plugin");
            const relativeTo = path.join(resolvePluginsRelativeTo, "__placeholder__.js");
            if (name.match(/\s+/u)) {
                const error = Object.assign(new Error(`Whitespace found in plugin name '${name}'`), {
                    messageTemplate: "whitespace-found",
                    messageData: { pluginName: request }
                });
                return new ConfigDependency({
                    error,
                    id,
                    importerName,
                    importerPath
                });
            }
            // Check for additional pool.
            const plugin = additionalPluginPool.get(request) ||
                additionalPluginPool.get(id);
            if (plugin) {
                return new ConfigDependency({
                    definition: normalizePlugin(plugin),
                    filePath: importerPath,
                    id,
                    importerName,
                    importerPath
                });
            }
            let filePath;
            let error;
            try {
                filePath = ModuleResolver.resolve(request, relativeTo);
            }
            catch (resolveError) {
                error = resolveError;
                /* istanbul ignore else */
                if (error && error.code === "MODULE_NOT_FOUND") {
                    error.messageTemplate = "plugin-missing";
                    error.messageData = {
                        pluginName: request,
                        resolvePluginsRelativeTo,
                        importerName
                    };
                }
            }
            if (filePath) {
                try {
                    writeDebugLogForLoading(request, relativeTo, filePath);
                    const startTime = Date.now();
                    const pluginDefinition = require(filePath);
                    debug(`Plugin ${filePath} loaded in: ${Date.now() - startTime}ms`);
                    return new ConfigDependency({
                        definition: normalizePlugin(pluginDefinition),
                        filePath,
                        id,
                        importerName,
                        importerPath
                    });
                }
                catch (loadError) {
                    error = loadError;
                }
            }
            debug("Failed to load plugin '%s' declared in '%s'.", name, importerName);
            error.message = `Failed to load plugin '${name}' declared in '${importerName}': ${error.message}`;
            return new ConfigDependency({
                error,
                id,
                importerName,
                importerPath
            });
        }
        /**
         * Take file expression processors as config array elements.
         * @param {Record<string,DependentPlugin>} plugins The plugin definitions.
         * @param {string} filePath The file path of this config.
         * @param {string} name The name of this config.
         * @returns {IterableIterator<ConfigArrayElement>} The config array elements of file expression processors.
         * @private
         */
        *_takeFileExtensionProcessors(plugins, filePath, name) {
            for (const pluginId of Object.keys(plugins)) {
                const processors = plugins[pluginId] &&
                    plugins[pluginId].definition &&
                    plugins[pluginId].definition.processors;
                if (!processors) {
                    continue;
                }
                for (const processorId of Object.keys(processors)) {
                    if (processorId.startsWith(".")) {
                        yield* this._normalizeObjectConfigData({
                            files: [`*${processorId}`],
                            processor: `${pluginId}/${processorId}`
                        }, filePath, `${name}#processors["${pluginId}/${processorId}"]`);
                    }
                }
            }
        }
    }
    module.exports = { ConfigArrayFactory };
},
/* import-fresh/index.js */ function _(require, module, exports) {
    const path = require('path');
    const resolveFrom = require(521) /* resolve-from */;
    const parentModule = require(522) /* parent-module */;
    module.exports = moduleId => {
        if (typeof moduleId !== 'string') {
            throw new TypeError('Expected a string');
        }
        const parentPath = parentModule(__filename);
        const filePath = resolveFrom(path.dirname(parentPath), moduleId);
        const oldModule = require.cache[filePath];
        // Delete itself from module parent
        if (oldModule && oldModule.parent) {
            let i = oldModule.parent.children.length;
            while (i--) {
                if (oldModule.parent.children[i].id === filePath) {
                    oldModule.parent.children.splice(i, 1);
                }
            }
        }
        delete require.cache[filePath]; // Delete module from cache
        const parent = require.cache[parentPath]; // If `filePath` and `parentPath` are the same, cache will already be deleted so we won't get a memory leak in next step
        return parent === undefined ? require(filePath) : parent.require(filePath); // In case cache doesn't have parent, fall back to normal require
    };
},
/* resolve-from/index.js */ function _(require, module, exports) {
    const path = require('path');
    const Module = require('module');
    const fs = require('fs');
    const resolveFrom = (fromDir, moduleId, silent) => {
        if (typeof fromDir !== 'string') {
            throw new TypeError(`Expected \`fromDir\` to be of type \`string\`, got \`${typeof fromDir}\``);
        }
        if (typeof moduleId !== 'string') {
            throw new TypeError(`Expected \`moduleId\` to be of type \`string\`, got \`${typeof moduleId}\``);
        }
        try {
            fromDir = fs.realpathSync(fromDir);
        }
        catch (err) {
            if (err.code === 'ENOENT') {
                fromDir = path.resolve(fromDir);
            }
            else if (silent) {
                return null;
            }
            else {
                throw err;
            }
        }
        const fromFile = path.join(fromDir, 'noop.js');
        const resolveFileName = () => Module._resolveFilename(moduleId, {
            id: fromFile,
            filename: fromFile,
            paths: Module._nodeModulePaths(fromDir)
        });
        if (silent) {
            try {
                return resolveFileName();
            }
            catch (err) {
                return null;
            }
        }
        return resolveFileName();
    };
    module.exports = (fromDir, moduleId) => resolveFrom(fromDir, moduleId);
    module.exports.silent = (fromDir, moduleId) => resolveFrom(fromDir, moduleId, true);
},
/* parent-module/index.js */ function _(require, module, exports) {
    const callsites = require(523) /* callsites */;
    module.exports = filepath => {
        const stacks = callsites();
        if (!filepath) {
            return stacks[2].getFileName();
        }
        let seenVal = false;
        // Skip the first stack as it's this function
        stacks.shift();
        for (const stack of stacks) {
            const parentFilepath = stack.getFileName();
            if (typeof parentFilepath !== 'string') {
                continue;
            }
            if (parentFilepath === filepath) {
                seenVal = true;
                continue;
            }
            // Skip native modules
            if (parentFilepath === 'module.js') {
                continue;
            }
            if (seenVal && parentFilepath !== filepath) {
                return parentFilepath;
            }
        }
    };
},
/* callsites/index.js */ function _(require, module, exports) {
    const callsites = () => {
        const _prepareStackTrace = Error.prepareStackTrace;
        Error.prepareStackTrace = (_, stack) => stack;
        const stack = new Error().stack.slice(1);
        Error.prepareStackTrace = _prepareStackTrace;
        return stack;
    };
    module.exports = callsites;
    // TODO: Remove this for the next major release
    module.exports.default = callsites;
},
/* strip-json-comments/index.js */ function _(require, module, exports) {
    const singleComment = Symbol('singleComment');
    const multiComment = Symbol('multiComment');
    const stripWithoutWhitespace = () => '';
    const stripWithWhitespace = (string, start, end) => string.slice(start, end).replace(/\S/g, ' ');
    const isEscaped = (jsonString, quotePosition) => {
        let index = quotePosition - 1;
        let backslashCount = 0;
        while (jsonString[index] === '\\') {
            index -= 1;
            backslashCount += 1;
        }
        return Boolean(backslashCount % 2);
    };
    module.exports = (jsonString, options = {}) => {
        const strip = options.whitespace === false ? stripWithoutWhitespace : stripWithWhitespace;
        let insideString = false;
        let insideComment = false;
        let offset = 0;
        let result = '';
        for (let i = 0; i < jsonString.length; i++) {
            const currentCharacter = jsonString[i];
            const nextCharacter = jsonString[i + 1];
            if (!insideComment && currentCharacter === '"') {
                const escaped = isEscaped(jsonString, i);
                if (!escaped) {
                    insideString = !insideString;
                }
            }
            if (insideString) {
                continue;
            }
            if (!insideComment && currentCharacter + nextCharacter === '//') {
                result += jsonString.slice(offset, i);
                offset = i;
                insideComment = singleComment;
                i++;
            }
            else if (insideComment === singleComment && currentCharacter + nextCharacter === '\r\n') {
                i++;
                insideComment = false;
                result += strip(jsonString, offset, i);
                offset = i;
                continue;
            }
            else if (insideComment === singleComment && currentCharacter === '\n') {
                insideComment = false;
                result += strip(jsonString, offset, i);
                offset = i;
            }
            else if (!insideComment && currentCharacter + nextCharacter === '/*') {
                result += jsonString.slice(offset, i);
                offset = i;
                insideComment = multiComment;
                i++;
                continue;
            }
            else if (insideComment === multiComment && currentCharacter + nextCharacter === '*/') {
                i++;
                insideComment = false;
                result += strip(jsonString, offset, i + 1);
                offset = i + 1;
                continue;
            }
        }
        return result + (insideComment ? strip(jsonString.slice(offset)) : jsonString.slice(offset));
    };
},
/* eslint/lib/cli-engine/config-array/index.js */ function _(require, module, exports) {
    const { ConfigArray, getUsedExtractedConfigs } = require(526) /* ./config-array */;
    const { ConfigDependency } = require(529) /* ./config-dependency */;
    const { ExtractedConfig } = require(527) /* ./extracted-config */;
    const { IgnorePattern } = require(528) /* ./ignore-pattern */;
    const { OverrideTester } = require(530) /* ./override-tester */;
    module.exports = {
        ConfigArray,
        ConfigDependency,
        ExtractedConfig,
        IgnorePattern,
        OverrideTester,
        getUsedExtractedConfigs
    };
},
/* eslint/lib/cli-engine/config-array/config-array.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const { ExtractedConfig } = require(527) /* ./extracted-config */;
    const { IgnorePattern } = require(528) /* ./ignore-pattern */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    // Define types for VSCode IntelliSense.
    /** @typedef {import("../../shared/types").Environment} Environment */
    /** @typedef {import("../../shared/types").GlobalConf} GlobalConf */
    /** @typedef {import("../../shared/types").RuleConf} RuleConf */
    /** @typedef {import("../../shared/types").Rule} Rule */
    /** @typedef {import("../../shared/types").Plugin} Plugin */
    /** @typedef {import("../../shared/types").Processor} Processor */
    /** @typedef {import("./config-dependency").DependentParser} DependentParser */
    /** @typedef {import("./config-dependency").DependentPlugin} DependentPlugin */
    /** @typedef {import("./override-tester")["OverrideTester"]} OverrideTester */
    /**
     * @typedef {Object} ConfigArrayElement
     * @property {string} name The name of this config element.
     * @property {string} filePath The path to the source file of this config element.
     * @property {InstanceType<OverrideTester>|null} criteria The tester for the `files` and `excludedFiles` of this config element.
     * @property {Record<string, boolean>|undefined} env The environment settings.
     * @property {Record<string, GlobalConf>|undefined} globals The global variable settings.
     * @property {IgnorePattern|undefined} ignorePattern The ignore patterns.
     * @property {boolean|undefined} noInlineConfig The flag that disables directive comments.
     * @property {DependentParser|undefined} parser The parser loader.
     * @property {Object|undefined} parserOptions The parser options.
     * @property {Record<string, DependentPlugin>|undefined} plugins The plugin loaders.
     * @property {string|undefined} processor The processor name to refer plugin's processor.
     * @property {boolean|undefined} reportUnusedDisableDirectives The flag to report unused `eslint-disable` comments.
     * @property {boolean|undefined} root The flag to express root.
     * @property {Record<string, RuleConf>|undefined} rules The rule settings
     * @property {Object|undefined} settings The shared settings.
     */
    /**
     * @typedef {Object} ConfigArrayInternalSlots
     * @property {Map<string, ExtractedConfig>} cache The cache to extract configs.
     * @property {ReadonlyMap<string, Environment>|null} envMap The map from environment ID to environment definition.
     * @property {ReadonlyMap<string, Processor>|null} processorMap The map from processor ID to environment definition.
     * @property {ReadonlyMap<string, Rule>|null} ruleMap The map from rule ID to rule definition.
     */
    /** @type {WeakMap<ConfigArray, ConfigArrayInternalSlots>} */
    const internalSlotsMap = new class extends WeakMap {
        get(key) {
            let value = super.get(key);
            if (!value) {
                value = {
                    cache: new Map(),
                    envMap: null,
                    processorMap: null,
                    ruleMap: null
                };
                super.set(key, value);
            }
            return value;
        }
    }();
    /**
     * Get the indices which are matched to a given file.
     * @param {ConfigArrayElement[]} elements The elements.
     * @param {string} filePath The path to a target file.
     * @returns {number[]} The indices.
     */
    function getMatchedIndices(elements, filePath) {
        const indices = [];
        for (let i = elements.length - 1; i >= 0; --i) {
            const element = elements[i];
            if (!element.criteria || element.criteria.test(filePath)) {
                indices.push(i);
            }
        }
        return indices;
    }
    /**
     * Check if a value is a non-null object.
     * @param {any} x The value to check.
     * @returns {boolean} `true` if the value is a non-null object.
     */
    function isNonNullObject(x) {
        return typeof x === "object" && x !== null;
    }
    /**
     * Merge two objects.
     *
     * Assign every property values of `y` to `x` if `x` doesn't have the property.
     * If `x`'s property value is an object, it does recursive.
     * @param {Object} target The destination to merge
     * @param {Object|undefined} source The source to merge.
     * @returns {void}
     */
    function mergeWithoutOverwrite(target, source) {
        if (!isNonNullObject(source)) {
            return;
        }
        for (const key of Object.keys(source)) {
            if (key === "__proto__") {
                continue;
            }
            if (isNonNullObject(target[key])) {
                mergeWithoutOverwrite(target[key], source[key]);
            }
            else if (target[key] === void 0) {
                if (isNonNullObject(source[key])) {
                    target[key] = Array.isArray(source[key]) ? [] : {};
                    mergeWithoutOverwrite(target[key], source[key]);
                }
                else if (source[key] !== void 0) {
                    target[key] = source[key];
                }
            }
        }
    }
    /**
     * Merge plugins.
     * `target`'s definition is prior to `source`'s.
     * @param {Record<string, DependentPlugin>} target The destination to merge
     * @param {Record<string, DependentPlugin>|undefined} source The source to merge.
     * @returns {void}
     */
    function mergePlugins(target, source) {
        if (!isNonNullObject(source)) {
            return;
        }
        for (const key of Object.keys(source)) {
            if (key === "__proto__") {
                continue;
            }
            const targetValue = target[key];
            const sourceValue = source[key];
            // Adopt the plugin which was found at first.
            if (targetValue === void 0) {
                if (sourceValue.error) {
                    throw sourceValue.error;
                }
                target[key] = sourceValue;
            }
        }
    }
    /**
     * Merge rule configs.
     * `target`'s definition is prior to `source`'s.
     * @param {Record<string, Array>} target The destination to merge
     * @param {Record<string, RuleConf>|undefined} source The source to merge.
     * @returns {void}
     */
    function mergeRuleConfigs(target, source) {
        if (!isNonNullObject(source)) {
            return;
        }
        for (const key of Object.keys(source)) {
            if (key === "__proto__") {
                continue;
            }
            const targetDef = target[key];
            const sourceDef = source[key];
            // Adopt the rule config which was found at first.
            if (targetDef === void 0) {
                if (Array.isArray(sourceDef)) {
                    target[key] = [...sourceDef];
                }
                else {
                    target[key] = [sourceDef];
                }
                /*
                 * If the first found rule config is severity only and the current rule
                 * config has options, merge the severity and the options.
                 */
            }
            else if (targetDef.length === 1 &&
                Array.isArray(sourceDef) &&
                sourceDef.length >= 2) {
                targetDef.push(...sourceDef.slice(1));
            }
        }
    }
    /**
     * Create the extracted config.
     * @param {ConfigArray} instance The config elements.
     * @param {number[]} indices The indices to use.
     * @returns {ExtractedConfig} The extracted config.
     */
    function createConfig(instance, indices) {
        const config = new ExtractedConfig();
        const ignorePatterns = [];
        // Merge elements.
        for (const index of indices) {
            const element = instance[index];
            // Adopt the parser which was found at first.
            if (!config.parser && element.parser) {
                if (element.parser.error) {
                    throw element.parser.error;
                }
                config.parser = element.parser;
            }
            // Adopt the processor which was found at first.
            if (!config.processor && element.processor) {
                config.processor = element.processor;
            }
            // Adopt the noInlineConfig which was found at first.
            if (config.noInlineConfig === void 0 && element.noInlineConfig !== void 0) {
                config.noInlineConfig = element.noInlineConfig;
                config.configNameOfNoInlineConfig = element.name;
            }
            // Adopt the reportUnusedDisableDirectives which was found at first.
            if (config.reportUnusedDisableDirectives === void 0 && element.reportUnusedDisableDirectives !== void 0) {
                config.reportUnusedDisableDirectives = element.reportUnusedDisableDirectives;
            }
            // Collect ignorePatterns
            if (element.ignorePattern) {
                ignorePatterns.push(element.ignorePattern);
            }
            // Merge others.
            mergeWithoutOverwrite(config.env, element.env);
            mergeWithoutOverwrite(config.globals, element.globals);
            mergeWithoutOverwrite(config.parserOptions, element.parserOptions);
            mergeWithoutOverwrite(config.settings, element.settings);
            mergePlugins(config.plugins, element.plugins);
            mergeRuleConfigs(config.rules, element.rules);
        }
        // Create the predicate function for ignore patterns.
        if (ignorePatterns.length > 0) {
            config.ignores = IgnorePattern.createIgnore(ignorePatterns.reverse());
        }
        return config;
    }
    /**
     * Collect definitions.
     * @template T, U
     * @param {string} pluginId The plugin ID for prefix.
     * @param {Record<string,T>} defs The definitions to collect.
     * @param {Map<string, U>} map The map to output.
     * @param {function(T): U} [normalize] The normalize function for each value.
     * @returns {void}
     */
    function collect(pluginId, defs, map, normalize) {
        if (defs) {
            const prefix = pluginId && `${pluginId}/`;
            for (const [key, value] of Object.entries(defs)) {
                map.set(`${prefix}${key}`, normalize ? normalize(value) : value);
            }
        }
    }
    /**
     * Normalize a rule definition.
     * @param {Function|Rule} rule The rule definition to normalize.
     * @returns {Rule} The normalized rule definition.
     */
    function normalizePluginRule(rule) {
        return typeof rule === "function" ? { create: rule } : rule;
    }
    /**
     * Delete the mutation methods from a given map.
     * @param {Map<any, any>} map The map object to delete.
     * @returns {void}
     */
    function deleteMutationMethods(map) {
        Object.defineProperties(map, {
            clear: { configurable: true, value: void 0 },
            delete: { configurable: true, value: void 0 },
            set: { configurable: true, value: void 0 }
        });
    }
    /**
     * Create `envMap`, `processorMap`, `ruleMap` with the plugins in the config array.
     * @param {ConfigArrayElement[]} elements The config elements.
     * @param {ConfigArrayInternalSlots} slots The internal slots.
     * @returns {void}
     */
    function initPluginMemberMaps(elements, slots) {
        const processed = new Set();
        slots.envMap = new Map();
        slots.processorMap = new Map();
        slots.ruleMap = new Map();
        for (const element of elements) {
            if (!element.plugins) {
                continue;
            }
            for (const [pluginId, value] of Object.entries(element.plugins)) {
                const plugin = value.definition;
                if (!plugin || processed.has(pluginId)) {
                    continue;
                }
                processed.add(pluginId);
                collect(pluginId, plugin.environments, slots.envMap);
                collect(pluginId, plugin.processors, slots.processorMap);
                collect(pluginId, plugin.rules, slots.ruleMap, normalizePluginRule);
            }
        }
        deleteMutationMethods(slots.envMap);
        deleteMutationMethods(slots.processorMap);
        deleteMutationMethods(slots.ruleMap);
    }
    /**
     * Create `envMap`, `processorMap`, `ruleMap` with the plugins in the config array.
     * @param {ConfigArray} instance The config elements.
     * @returns {ConfigArrayInternalSlots} The extracted config.
     */
    function ensurePluginMemberMaps(instance) {
        const slots = internalSlotsMap.get(instance);
        if (!slots.ruleMap) {
            initPluginMemberMaps(instance, slots);
        }
        return slots;
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * The Config Array.
     *
     * `ConfigArray` instance contains all settings, parsers, and plugins.
     * You need to call `ConfigArray#extractConfig(filePath)` method in order to
     * extract, merge and get only the config data which is related to an arbitrary
     * file.
     * @extends {Array<ConfigArrayElement>}
     */
    class ConfigArray extends Array {
        /**
         * Get the plugin environments.
         * The returned map cannot be mutated.
         * @type {ReadonlyMap<string, Environment>} The plugin environments.
         */
        get pluginEnvironments() {
            return ensurePluginMemberMaps(this).envMap;
        }
        /**
         * Get the plugin processors.
         * The returned map cannot be mutated.
         * @type {ReadonlyMap<string, Processor>} The plugin processors.
         */
        get pluginProcessors() {
            return ensurePluginMemberMaps(this).processorMap;
        }
        /**
         * Get the plugin rules.
         * The returned map cannot be mutated.
         * @returns {ReadonlyMap<string, Rule>} The plugin rules.
         */
        get pluginRules() {
            return ensurePluginMemberMaps(this).ruleMap;
        }
        /**
         * Check if this config has `root` flag.
         * @returns {boolean} `true` if this config array is root.
         */
        isRoot() {
            for (let i = this.length - 1; i >= 0; --i) {
                const root = this[i].root;
                if (typeof root === "boolean") {
                    return root;
                }
            }
            return false;
        }
        /**
         * Extract the config data which is related to a given file.
         * @param {string} filePath The absolute path to the target file.
         * @returns {ExtractedConfig} The extracted config data.
         */
        extractConfig(filePath) {
            const { cache } = internalSlotsMap.get(this);
            const indices = getMatchedIndices(this, filePath);
            const cacheKey = indices.join(",");
            if (!cache.has(cacheKey)) {
                cache.set(cacheKey, createConfig(this, indices));
            }
            return cache.get(cacheKey);
        }
    }
    const exportObject = {
        ConfigArray,
        /**
         * Get the used extracted configs.
         * CLIEngine will use this method to collect used deprecated rules.
         * @param {ConfigArray} instance The config array object to get.
         * @returns {ExtractedConfig[]} The used extracted configs.
         * @private
         */
        getUsedExtractedConfigs(instance) {
            const { cache } = internalSlotsMap.get(instance);
            return Array.from(cache.values());
        }
    };
    module.exports = exportObject;
},
/* eslint/lib/cli-engine/config-array/extracted-config.js */ function _(require, module, exports) {
    const { IgnorePattern } = require(528) /* ./ignore-pattern */;
    // For VSCode intellisense
    /** @typedef {import("../../shared/types").ConfigData} ConfigData */
    /** @typedef {import("../../shared/types").GlobalConf} GlobalConf */
    /** @typedef {import("../../shared/types").SeverityConf} SeverityConf */
    /** @typedef {import("./config-dependency").DependentParser} DependentParser */
    /** @typedef {import("./config-dependency").DependentPlugin} DependentPlugin */
    /**
     * Check if `xs` starts with `ys`.
     * @template T
     * @param {T[]} xs The array to check.
     * @param {T[]} ys The array that may be the first part of `xs`.
     * @returns {boolean} `true` if `xs` starts with `ys`.
     */
    function startsWith(xs, ys) {
        return xs.length >= ys.length && ys.every((y, i) => y === xs[i]);
    }
    /**
     * The class for extracted config data.
     */
    class ExtractedConfig {
        constructor() {
            /**
             * The config name what `noInlineConfig` setting came from.
             * @type {string}
             */
            this.configNameOfNoInlineConfig = "";
            /**
             * Environments.
             * @type {Record<string, boolean>}
             */
            this.env = {};
            /**
             * Global variables.
             * @type {Record<string, GlobalConf>}
             */
            this.globals = {};
            /**
             * The glob patterns that ignore to lint.
             * @type {(((filePath:string, dot?:boolean) => boolean) & { basePath:string; patterns:string[] }) | undefined}
             */
            this.ignores = void 0;
            /**
             * The flag that disables directive comments.
             * @type {boolean|undefined}
             */
            this.noInlineConfig = void 0;
            /**
             * Parser definition.
             * @type {DependentParser|null}
             */
            this.parser = null;
            /**
             * Options for the parser.
             * @type {Object}
             */
            this.parserOptions = {};
            /**
             * Plugin definitions.
             * @type {Record<string, DependentPlugin>}
             */
            this.plugins = {};
            /**
             * Processor ID.
             * @type {string|null}
             */
            this.processor = null;
            /**
             * The flag that reports unused `eslint-disable` directive comments.
             * @type {boolean|undefined}
             */
            this.reportUnusedDisableDirectives = void 0;
            /**
             * Rule settings.
             * @type {Record<string, [SeverityConf, ...any[]]>}
             */
            this.rules = {};
            /**
             * Shared settings.
             * @type {Object}
             */
            this.settings = {};
        }
        /**
         * Convert this config to the compatible object as a config file content.
         * @returns {ConfigData} The converted object.
         */
        toCompatibleObjectAsConfigFileContent() {
            const { 
            /* eslint-disable no-unused-vars */
            configNameOfNoInlineConfig: _ignore1, processor: _ignore2, 
            /* eslint-enable no-unused-vars */
            ignores, ...config } = this;
            config.parser = config.parser && config.parser.filePath;
            config.plugins = Object.keys(config.plugins).filter(Boolean).reverse();
            config.ignorePatterns = ignores ? ignores.patterns : [];
            // Strip the default patterns from `ignorePatterns`.
            if (startsWith(config.ignorePatterns, IgnorePattern.DefaultPatterns)) {
                config.ignorePatterns =
                    config.ignorePatterns.slice(IgnorePattern.DefaultPatterns.length);
            }
            return config;
        }
    }
    module.exports = { ExtractedConfig };
},
/* eslint/lib/cli-engine/config-array/ignore-pattern.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const assert = require("assert");
    const path = require("path");
    const ignore = require(309) /* ignore */;
    const debug = require(120) /* debug */("eslint:ignore-pattern");
    /** @typedef {ReturnType<import("ignore").default>} Ignore */
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    /**
     * Get the path to the common ancestor directory of given paths.
     * @param {string[]} sourcePaths The paths to calculate the common ancestor.
     * @returns {string} The path to the common ancestor directory.
     */
    function getCommonAncestorPath(sourcePaths) {
        let result = sourcePaths[0];
        for (let i = 1; i < sourcePaths.length; ++i) {
            const a = result;
            const b = sourcePaths[i];
            // Set the shorter one (it's the common ancestor if one includes the other).
            result = a.length < b.length ? a : b;
            // Set the common ancestor.
            for (let j = 0, lastSepPos = 0; j < a.length && j < b.length; ++j) {
                if (a[j] !== b[j]) {
                    result = a.slice(0, lastSepPos);
                    break;
                }
                if (a[j] === path.sep) {
                    lastSepPos = j;
                }
            }
        }
        return result || path.sep;
    }
    /**
     * Make relative path.
     * @param {string} from The source path to get relative path.
     * @param {string} to The destination path to get relative path.
     * @returns {string} The relative path.
     */
    function relative(from, to) {
        const relPath = path.relative(from, to);
        if (path.sep === "/") {
            return relPath;
        }
        return relPath.split(path.sep).join("/");
    }
    /**
     * Get the trailing slash if existed.
     * @param {string} filePath The path to check.
     * @returns {string} The trailing slash if existed.
     */
    function dirSuffix(filePath) {
        const isDir = (filePath.endsWith(path.sep) ||
            (process.platform === "win32" && filePath.endsWith("/")));
        return isDir ? "/" : "";
    }
    const DefaultPatterns = Object.freeze(["/node_modules/*", "/bower_components/*"]);
    const DotPatterns = Object.freeze([".*", "!../"]);
    //------------------------------------------------------------------------------
    // Public
    //------------------------------------------------------------------------------
    class IgnorePattern {
        /**
         * The default patterns.
         * @type {string[]}
         */
        static get DefaultPatterns() {
            return DefaultPatterns;
        }
        /**
         * Create the default predicate function.
         * @param {string} cwd The current working directory.
         * @returns {((filePath:string, dot:boolean) => boolean) & {basePath:string; patterns:string[]}}
         * The preficate function.
         * The first argument is an absolute path that is checked.
         * The second argument is the flag to not ignore dotfiles.
         * If the predicate function returned `true`, it means the path should be ignored.
         */
        static createDefaultIgnore(cwd) {
            return this.createIgnore([new IgnorePattern(DefaultPatterns, cwd)]);
        }
        /**
         * Create the predicate function from multiple `IgnorePattern` objects.
         * @param {IgnorePattern[]} ignorePatterns The list of ignore patterns.
         * @returns {((filePath:string, dot?:boolean) => boolean) & {basePath:string; patterns:string[]}}
         * The preficate function.
         * The first argument is an absolute path that is checked.
         * The second argument is the flag to not ignore dotfiles.
         * If the predicate function returned `true`, it means the path should be ignored.
         */
        static createIgnore(ignorePatterns) {
            debug("Create with: %o", ignorePatterns);
            const basePath = getCommonAncestorPath(ignorePatterns.map(p => p.basePath));
            const patterns = [].concat(...ignorePatterns.map(p => p.getPatternsRelativeTo(basePath)));
            const ig = ignore().add([...DotPatterns, ...patterns]);
            const dotIg = ignore().add(patterns);
            debug("  processed: %o", { basePath, patterns });
            return Object.assign((filePath, dot = false) => {
                assert(path.isAbsolute(filePath), "'filePath' should be an absolute path.");
                const relPathRaw = relative(basePath, filePath);
                const relPath = relPathRaw && (relPathRaw + dirSuffix(filePath));
                const adoptedIg = dot ? dotIg : ig;
                const result = relPath !== "" && adoptedIg.ignores(relPath);
                debug("Check", { filePath, dot, relativePath: relPath, result });
                return result;
            }, { basePath, patterns });
        }
        /**
         * Initialize a new `IgnorePattern` instance.
         * @param {string[]} patterns The glob patterns that ignore to lint.
         * @param {string} basePath The base path of `patterns`.
         */
        constructor(patterns, basePath) {
            assert(path.isAbsolute(basePath), "'basePath' should be an absolute path.");
            /**
             * The glob patterns that ignore to lint.
             * @type {string[]}
             */
            this.patterns = patterns;
            /**
             * The base path of `patterns`.
             * @type {string}
             */
            this.basePath = basePath;
            /**
             * If `true` then patterns which don't start with `/` will match the paths to the outside of `basePath`. Defaults to `false`.
             *
             * It's set `true` for `.eslintignore`, `package.json`, and `--ignore-path` for backward compatibility.
             * It's `false` as-is for `ignorePatterns` property in config files.
             * @type {boolean}
             */
            this.loose = false;
        }
        /**
         * Get `patterns` as modified for a given base path. It modifies the
         * absolute paths in the patterns as prepending the difference of two base
         * paths.
         * @param {string} newBasePath The base path.
         * @returns {string[]} Modifired patterns.
         */
        getPatternsRelativeTo(newBasePath) {
            assert(path.isAbsolute(newBasePath), "'newBasePath' should be an absolute path.");
            const { basePath, loose, patterns } = this;
            if (newBasePath === basePath) {
                return patterns;
            }
            const prefix = `/${relative(newBasePath, basePath)}`;
            return patterns.map(pattern => {
                const negative = pattern.startsWith("!");
                const head = negative ? "!" : "";
                const body = negative ? pattern.slice(1) : pattern;
                if (body.startsWith("/") || body.startsWith("../")) {
                    return `${head}${prefix}${body}`;
                }
                return loose ? pattern : `${head}${prefix}/**/${body}`;
            });
        }
    }
    module.exports = { IgnorePattern };
},
/* eslint/lib/cli-engine/config-array/config-dependency.js */ function _(require, module, exports) {
    const util = require("util");
    /**
     * The class is to store parsers or plugins.
     * This class hides the loaded object from `JSON.stringify()` and `console.log`.
     * @template T
     */
    class ConfigDependency {
        /**
         * Initialize this instance.
         * @param {Object} data The dependency data.
         * @param {T} [data.definition] The dependency if the loading succeeded.
         * @param {Error} [data.error] The error object if the loading failed.
         * @param {string} [data.filePath] The actual path to the dependency if the loading succeeded.
         * @param {string} data.id The ID of this dependency.
         * @param {string} data.importerName The name of the config file which loads this dependency.
         * @param {string} data.importerPath The path to the config file which loads this dependency.
         */
        constructor({ definition = null, error = null, filePath = null, id, importerName, importerPath }) {
            /**
             * The loaded dependency if the loading succeeded.
             * @type {T|null}
             */
            this.definition = definition;
            /**
             * The error object if the loading failed.
             * @type {Error|null}
             */
            this.error = error;
            /**
             * The loaded dependency if the loading succeeded.
             * @type {string|null}
             */
            this.filePath = filePath;
            /**
             * The ID of this dependency.
             * @type {string}
             */
            this.id = id;
            /**
             * The name of the config file which loads this dependency.
             * @type {string}
             */
            this.importerName = importerName;
            /**
             * The path to the config file which loads this dependency.
             * @type {string}
             */
            this.importerPath = importerPath;
        }
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @returns {Object} a JSON compatible object.
         */
        toJSON() {
            const obj = this[util.inspect.custom]();
            // Display `error.message` (`Error#message` is unenumerable).
            if (obj.error instanceof Error) {
                obj.error = { ...obj.error, message: obj.error.message };
            }
            return obj;
        }
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @returns {Object} an object to display by `console.log()`.
         */
        [util.inspect.custom]() {
            const { definition: _ignore, // eslint-disable-line no-unused-vars
            ...obj } = this;
            return obj;
        }
    }
    /** @typedef {ConfigDependency<import("../../shared/types").Parser>} DependentParser */
    /** @typedef {ConfigDependency<import("../../shared/types").Plugin>} DependentPlugin */
    module.exports = { ConfigDependency };
},
/* eslint/lib/cli-engine/config-array/override-tester.js */ function _(require, module, exports) {
    const assert = require("assert");
    const path = require("path");
    const util = require("util");
    const { Minimatch } = require(531) /* minimatch */;
    const minimatchOpts = { dot: true, matchBase: true };
    /**
     * @typedef {Object} Pattern
     * @property {InstanceType<Minimatch>[] | null} includes The positive matchers.
     * @property {InstanceType<Minimatch>[] | null} excludes The negative matchers.
     */
    /**
     * Normalize a given pattern to an array.
     * @param {string|string[]|undefined} patterns A glob pattern or an array of glob patterns.
     * @returns {string[]|null} Normalized patterns.
     * @private
     */
    function normalizePatterns(patterns) {
        if (Array.isArray(patterns)) {
            return patterns.filter(Boolean);
        }
        if (typeof patterns === "string" && patterns) {
            return [patterns];
        }
        return [];
    }
    /**
     * Create the matchers of given patterns.
     * @param {string[]} patterns The patterns.
     * @returns {InstanceType<Minimatch>[] | null} The matchers.
     */
    function toMatcher(patterns) {
        if (patterns.length === 0) {
            return null;
        }
        return patterns.map(pattern => {
            if (/^\.[/\\]/u.test(pattern)) {
                return new Minimatch(pattern.slice(2), 
                // `./*.js` should not match with `subdir/foo.js`
                { ...minimatchOpts, matchBase: false });
            }
            return new Minimatch(pattern, minimatchOpts);
        });
    }
    /**
     * Convert a given matcher to string.
     * @param {Pattern} matchers The matchers.
     * @returns {string} The string expression of the matcher.
     */
    function patternToJson({ includes, excludes }) {
        return {
            includes: includes && includes.map(m => m.pattern),
            excludes: excludes && excludes.map(m => m.pattern)
        };
    }
    /**
     * The class to test given paths are matched by the patterns.
     */
    class OverrideTester {
        /**
         * Create a tester with given criteria.
         * If there are no criteria, returns `null`.
         * @param {string|string[]} files The glob patterns for included files.
         * @param {string|string[]} excludedFiles The glob patterns for excluded files.
         * @param {string} basePath The path to the base directory to test paths.
         * @returns {OverrideTester|null} The created instance or `null`.
         */
        static create(files, excludedFiles, basePath) {
            const includePatterns = normalizePatterns(files);
            const excludePatterns = normalizePatterns(excludedFiles);
            const allPatterns = includePatterns.concat(excludePatterns);
            if (allPatterns.length === 0) {
                return null;
            }
            // Rejects absolute paths or relative paths to parents.
            for (const pattern of allPatterns) {
                if (path.isAbsolute(pattern) || pattern.includes("..")) {
                    throw new Error(`Invalid override pattern (expected relative path not containing '..'): ${pattern}`);
                }
            }
            const includes = toMatcher(includePatterns);
            const excludes = toMatcher(excludePatterns);
            return new OverrideTester([{ includes, excludes }], basePath);
        }
        /**
         * Combine two testers by logical and.
         * If either of the testers was `null`, returns the other tester.
         * The `basePath` property of the two must be the same value.
         * @param {OverrideTester|null} a A tester.
         * @param {OverrideTester|null} b Another tester.
         * @returns {OverrideTester|null} Combined tester.
         */
        static and(a, b) {
            if (!b) {
                return a && new OverrideTester(a.patterns, a.basePath);
            }
            if (!a) {
                return new OverrideTester(b.patterns, b.basePath);
            }
            assert.strictEqual(a.basePath, b.basePath);
            return new OverrideTester(a.patterns.concat(b.patterns), a.basePath);
        }
        /**
         * Initialize this instance.
         * @param {Pattern[]} patterns The matchers.
         * @param {string} basePath The base path.
         */
        constructor(patterns, basePath) {
            /** @type {Pattern[]} */
            this.patterns = patterns;
            /** @type {string} */
            this.basePath = basePath;
        }
        /**
         * Test if a given path is matched or not.
         * @param {string} filePath The absolute path to the target file.
         * @returns {boolean} `true` if the path was matched.
         */
        test(filePath) {
            if (typeof filePath !== "string" || !path.isAbsolute(filePath)) {
                throw new Error(`'filePath' should be an absolute path, but got ${filePath}.`);
            }
            const relativePath = path.relative(this.basePath, filePath);
            return this.patterns.every(({ includes, excludes }) => ((!includes || includes.some(m => m.match(relativePath))) &&
                (!excludes || !excludes.some(m => m.match(relativePath)))));
        }
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @returns {Object} a JSON compatible object.
         */
        toJSON() {
            if (this.patterns.length === 1) {
                return {
                    ...patternToJson(this.patterns[0]),
                    basePath: this.basePath
                };
            }
            return {
                AND: this.patterns.map(patternToJson),
                basePath: this.basePath
            };
        }
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @returns {Object} an object to display by `console.log()`.
         */
        [util.inspect.custom]() {
            return this.toJSON();
        }
    }
    module.exports = { OverrideTester };
},
/* minimatch/minimatch.js */ function _(require, module, exports) {
    module.exports = minimatch;
    minimatch.Minimatch = Minimatch;
    var path = { sep: '/' };
    try {
        path = require('path');
    }
    catch (er) { }
    var GLOBSTAR = minimatch.GLOBSTAR = Minimatch.GLOBSTAR = {};
    var expand = require(532) /* brace-expansion */;
    var plTypes = {
        '!': { open: '(?:(?!(?:', close: '))[^/]*?)' },
        '?': { open: '(?:', close: ')?' },
        '+': { open: '(?:', close: ')+' },
        '*': { open: '(?:', close: ')*' },
        '@': { open: '(?:', close: ')' }
    };
    // any single thing other than /
    // don't need to escape / when using new RegExp()
    var qmark = '[^/]';
    // * => any number of characters
    var star = qmark + '*?';
    // ** when dots are allowed.  Anything goes, except .. and .
    // not (^ or / followed by one or two dots followed by $ or /),
    // followed by anything, any number of times.
    var twoStarDot = '(?:(?!(?:\\\/|^)(?:\\.{1,2})($|\\\/)).)*?';
    // not a ^ or / followed by a dot,
    // followed by anything, any number of times.
    var twoStarNoDot = '(?:(?!(?:\\\/|^)\\.).)*?';
    // characters that need to be escaped in RegExp.
    var reSpecials = charSet('().*{}+?[]^$\\!');
    // "abc" -> { a:true, b:true, c:true }
    function charSet(s) {
        return s.split('').reduce(function (set, c) {
            set[c] = true;
            return set;
        }, {});
    }
    // normalizes slashes.
    var slashSplit = /\/+/;
    minimatch.filter = filter;
    function filter(pattern, options) {
        options = options || {};
        return function (p, i, list) {
            return minimatch(p, pattern, options);
        };
    }
    function ext(a, b) {
        a = a || {};
        b = b || {};
        var t = {};
        Object.keys(b).forEach(function (k) {
            t[k] = b[k];
        });
        Object.keys(a).forEach(function (k) {
            t[k] = a[k];
        });
        return t;
    }
    minimatch.defaults = function (def) {
        if (!def || !Object.keys(def).length)
            return minimatch;
        var orig = minimatch;
        var m = function minimatch(p, pattern, options) {
            return orig.minimatch(p, pattern, ext(def, options));
        };
        m.Minimatch = function Minimatch(pattern, options) {
            return new orig.Minimatch(pattern, ext(def, options));
        };
        return m;
    };
    Minimatch.defaults = function (def) {
        if (!def || !Object.keys(def).length)
            return Minimatch;
        return minimatch.defaults(def).Minimatch;
    };
    function minimatch(p, pattern, options) {
        if (typeof pattern !== 'string') {
            throw new TypeError('glob pattern string required');
        }
        if (!options)
            options = {};
        // shortcut: comments match nothing.
        if (!options.nocomment && pattern.charAt(0) === '#') {
            return false;
        }
        // "" only matches ""
        if (pattern.trim() === '')
            return p === '';
        return new Minimatch(pattern, options).match(p);
    }
    function Minimatch(pattern, options) {
        if (!(this instanceof Minimatch)) {
            return new Minimatch(pattern, options);
        }
        if (typeof pattern !== 'string') {
            throw new TypeError('glob pattern string required');
        }
        if (!options)
            options = {};
        pattern = pattern.trim();
        // windows support: need to use /, not \
        if (path.sep !== '/') {
            pattern = pattern.split(path.sep).join('/');
        }
        this.options = options;
        this.set = [];
        this.pattern = pattern;
        this.regexp = null;
        this.negate = false;
        this.comment = false;
        this.empty = false;
        // make the set of regexps etc.
        this.make();
    }
    Minimatch.prototype.debug = function () { };
    Minimatch.prototype.make = make;
    function make() {
        // don't do it more than once.
        if (this._made)
            return;
        var pattern = this.pattern;
        var options = this.options;
        // empty patterns and comments match nothing.
        if (!options.nocomment && pattern.charAt(0) === '#') {
            this.comment = true;
            return;
        }
        if (!pattern) {
            this.empty = true;
            return;
        }
        // step 1: figure out negation, etc.
        this.parseNegate();
        // step 2: expand braces
        var set = this.globSet = this.braceExpand();
        if (options.debug)
            this.debug = console.error;
        this.debug(this.pattern, set);
        // step 3: now we have a set, so turn each one into a series of path-portion
        // matching patterns.
        // These will be regexps, except in the case of "**", which is
        // set to the GLOBSTAR object for globstar behavior,
        // and will not contain any / characters
        set = this.globParts = set.map(function (s) {
            return s.split(slashSplit);
        });
        this.debug(this.pattern, set);
        // glob --> regexps
        set = set.map(function (s, si, set) {
            return s.map(this.parse, this);
        }, this);
        this.debug(this.pattern, set);
        // filter out everything that didn't compile properly.
        set = set.filter(function (s) {
            return s.indexOf(false) === -1;
        });
        this.debug(this.pattern, set);
        this.set = set;
    }
    Minimatch.prototype.parseNegate = parseNegate;
    function parseNegate() {
        var pattern = this.pattern;
        var negate = false;
        var options = this.options;
        var negateOffset = 0;
        if (options.nonegate)
            return;
        for (var i = 0, l = pattern.length; i < l && pattern.charAt(i) === '!'; i++) {
            negate = !negate;
            negateOffset++;
        }
        if (negateOffset)
            this.pattern = pattern.substr(negateOffset);
        this.negate = negate;
    }
    // Brace expansion:
    // a{b,c}d -> abd acd
    // a{b,}c -> abc ac
    // a{0..3}d -> a0d a1d a2d a3d
    // a{b,c{d,e}f}g -> abg acdfg acefg
    // a{b,c}d{e,f}g -> abdeg acdeg abdeg abdfg
    //
    // Invalid sets are not expanded.
    // a{2..}b -> a{2..}b
    // a{b}c -> a{b}c
    minimatch.braceExpand = function (pattern, options) {
        return braceExpand(pattern, options);
    };
    Minimatch.prototype.braceExpand = braceExpand;
    function braceExpand(pattern, options) {
        if (!options) {
            if (this instanceof Minimatch) {
                options = this.options;
            }
            else {
                options = {};
            }
        }
        pattern = typeof pattern === 'undefined'
            ? this.pattern : pattern;
        if (typeof pattern === 'undefined') {
            throw new TypeError('undefined pattern');
        }
        if (options.nobrace ||
            !pattern.match(/\{.*\}/)) {
            // shortcut. no need to expand.
            return [pattern];
        }
        return expand(pattern);
    }
    // parse a component of the expanded set.
    // At this point, no pattern may contain "/" in it
    // so we're going to return a 2d array, where each entry is the full
    // pattern, split on '/', and then turned into a regular expression.
    // A regexp is made at the end which joins each array with an
    // escaped /, and another full one which joins each regexp with |.
    //
    // Following the lead of Bash 4.1, note that "**" only has special meaning
    // when it is the *only* thing in a path portion.  Otherwise, any series
    // of * is equivalent to a single *.  Globstar behavior is enabled by
    // default, and can be disabled by setting options.noglobstar.
    Minimatch.prototype.parse = parse;
    var SUBPARSE = {};
    function parse(pattern, isSub) {
        if (pattern.length > 1024 * 64) {
            throw new TypeError('pattern is too long');
        }
        var options = this.options;
        // shortcuts
        if (!options.noglobstar && pattern === '**')
            return GLOBSTAR;
        if (pattern === '')
            return '';
        var re = '';
        var hasMagic = !!options.nocase;
        var escaping = false;
        // ? => one single character
        var patternListStack = [];
        var negativeLists = [];
        var stateChar;
        var inClass = false;
        var reClassStart = -1;
        var classStart = -1;
        // . and .. never match anything that doesn't start with .,
        // even when options.dot is set.
        var patternStart = pattern.charAt(0) === '.' ? '' // anything
            // not (start or / followed by . or .. followed by / or end)
            : options.dot ? '(?!(?:^|\\\/)\\.{1,2}(?:$|\\\/))'
                : '(?!\\.)';
        var self = this;
        function clearStateChar() {
            if (stateChar) {
                // we had some state-tracking character
                // that wasn't consumed by this pass.
                switch (stateChar) {
                    case '*':
                        re += star;
                        hasMagic = true;
                        break;
                    case '?':
                        re += qmark;
                        hasMagic = true;
                        break;
                    default:
                        re += '\\' + stateChar;
                        break;
                }
                self.debug('clearStateChar %j %j', stateChar, re);
                stateChar = false;
            }
        }
        for (var i = 0, len = pattern.length, c; (i < len) && (c = pattern.charAt(i)); i++) {
            this.debug('%s\t%s %s %j', pattern, i, re, c);
            // skip over any that are escaped.
            if (escaping && reSpecials[c]) {
                re += '\\' + c;
                escaping = false;
                continue;
            }
            switch (c) {
                case '/':
                    // completely not allowed, even escaped.
                    // Should already be path-split by now.
                    return false;
                case '\\':
                    clearStateChar();
                    escaping = true;
                    continue;
                // the various stateChar values
                // for the "extglob" stuff.
                case '?':
                case '*':
                case '+':
                case '@':
                case '!':
                    this.debug('%s\t%s %s %j <-- stateChar', pattern, i, re, c);
                    // all of those are literals inside a class, except that
                    // the glob [!a] means [^a] in regexp
                    if (inClass) {
                        this.debug('  in class');
                        if (c === '!' && i === classStart + 1)
                            c = '^';
                        re += c;
                        continue;
                    }
                    // if we already have a stateChar, then it means
                    // that there was something like ** or +? in there.
                    // Handle the stateChar, then proceed with this one.
                    self.debug('call clearStateChar %j', stateChar);
                    clearStateChar();
                    stateChar = c;
                    // if extglob is disabled, then +(asdf|foo) isn't a thing.
                    // just clear the statechar *now*, rather than even diving into
                    // the patternList stuff.
                    if (options.noext)
                        clearStateChar();
                    continue;
                case '(':
                    if (inClass) {
                        re += '(';
                        continue;
                    }
                    if (!stateChar) {
                        re += '\\(';
                        continue;
                    }
                    patternListStack.push({
                        type: stateChar,
                        start: i - 1,
                        reStart: re.length,
                        open: plTypes[stateChar].open,
                        close: plTypes[stateChar].close
                    });
                    // negation is (?:(?!js)[^/]*)
                    re += stateChar === '!' ? '(?:(?!(?:' : '(?:';
                    this.debug('plType %j %j', stateChar, re);
                    stateChar = false;
                    continue;
                case ')':
                    if (inClass || !patternListStack.length) {
                        re += '\\)';
                        continue;
                    }
                    clearStateChar();
                    hasMagic = true;
                    var pl = patternListStack.pop();
                    // negation is (?:(?!js)[^/]*)
                    // The others are (?:<pattern>)<type>
                    re += pl.close;
                    if (pl.type === '!') {
                        negativeLists.push(pl);
                    }
                    pl.reEnd = re.length;
                    continue;
                case '|':
                    if (inClass || !patternListStack.length || escaping) {
                        re += '\\|';
                        escaping = false;
                        continue;
                    }
                    clearStateChar();
                    re += '|';
                    continue;
                // these are mostly the same in regexp and glob
                case '[':
                    // swallow any state-tracking char before the [
                    clearStateChar();
                    if (inClass) {
                        re += '\\' + c;
                        continue;
                    }
                    inClass = true;
                    classStart = i;
                    reClassStart = re.length;
                    re += c;
                    continue;
                case ']':
                    //  a right bracket shall lose its special
                    //  meaning and represent itself in
                    //  a bracket expression if it occurs
                    //  first in the list.  -- POSIX.2 2.8.3.2
                    if (i === classStart + 1 || !inClass) {
                        re += '\\' + c;
                        escaping = false;
                        continue;
                    }
                    // handle the case where we left a class open.
                    // "[z-a]" is valid, equivalent to "\[z-a\]"
                    if (inClass) {
                        // split where the last [ was, make sure we don't have
                        // an invalid re. if so, re-walk the contents of the
                        // would-be class to re-translate any characters that
                        // were passed through as-is
                        // TODO: It would probably be faster to determine this
                        // without a try/catch and a new RegExp, but it's tricky
                        // to do safely.  For now, this is safe and works.
                        var cs = pattern.substring(classStart + 1, i);
                        try {
                            RegExp('[' + cs + ']');
                        }
                        catch (er) {
                            // not a valid class!
                            var sp = this.parse(cs, SUBPARSE);
                            re = re.substr(0, reClassStart) + '\\[' + sp[0] + '\\]';
                            hasMagic = hasMagic || sp[1];
                            inClass = false;
                            continue;
                        }
                    }
                    // finish up the class.
                    hasMagic = true;
                    inClass = false;
                    re += c;
                    continue;
                default:
                    // swallow any state char that wasn't consumed
                    clearStateChar();
                    if (escaping) {
                        // no need
                        escaping = false;
                    }
                    else if (reSpecials[c]
                        && !(c === '^' && inClass)) {
                        re += '\\';
                    }
                    re += c;
            } // switch
        } // for
        // handle the case where we left a class open.
        // "[abc" is valid, equivalent to "\[abc"
        if (inClass) {
            // split where the last [ was, and escape it
            // this is a huge pita.  We now have to re-walk
            // the contents of the would-be class to re-translate
            // any characters that were passed through as-is
            cs = pattern.substr(classStart + 1);
            sp = this.parse(cs, SUBPARSE);
            re = re.substr(0, reClassStart) + '\\[' + sp[0];
            hasMagic = hasMagic || sp[1];
        }
        // handle the case where we had a +( thing at the *end*
        // of the pattern.
        // each pattern list stack adds 3 chars, and we need to go through
        // and escape any | chars that were passed through as-is for the regexp.
        // Go through and escape them, taking care not to double-escape any
        // | chars that were already escaped.
        for (pl = patternListStack.pop(); pl; pl = patternListStack.pop()) {
            var tail = re.slice(pl.reStart + pl.open.length);
            this.debug('setting tail', re, pl);
            // maybe some even number of \, then maybe 1 \, followed by a |
            tail = tail.replace(/((?:\\{2}){0,64})(\\?)\|/g, function (_, $1, $2) {
                if (!$2) {
                    // the | isn't already escaped, so escape it.
                    $2 = '\\';
                }
                // need to escape all those slashes *again*, without escaping the
                // one that we need for escaping the | character.  As it works out,
                // escaping an even number of slashes can be done by simply repeating
                // it exactly after itself.  That's why this trick works.
                //
                // I am sorry that you have to see this.
                return $1 + $1 + $2 + '|';
            });
            this.debug('tail=%j\n   %s', tail, tail, pl, re);
            var t = pl.type === '*' ? star
                : pl.type === '?' ? qmark
                    : '\\' + pl.type;
            hasMagic = true;
            re = re.slice(0, pl.reStart) + t + '\\(' + tail;
        }
        // handle trailing things that only matter at the very end.
        clearStateChar();
        if (escaping) {
            // trailing \\
            re += '\\\\';
        }
        // only need to apply the nodot start if the re starts with
        // something that could conceivably capture a dot
        var addPatternStart = false;
        switch (re.charAt(0)) {
            case '.':
            case '[':
            case '(': addPatternStart = true;
        }
        // Hack to work around lack of negative lookbehind in JS
        // A pattern like: *.!(x).!(y|z) needs to ensure that a name
        // like 'a.xyz.yz' doesn't match.  So, the first negative
        // lookahead, has to look ALL the way ahead, to the end of
        // the pattern.
        for (var n = negativeLists.length - 1; n > -1; n--) {
            var nl = negativeLists[n];
            var nlBefore = re.slice(0, nl.reStart);
            var nlFirst = re.slice(nl.reStart, nl.reEnd - 8);
            var nlLast = re.slice(nl.reEnd - 8, nl.reEnd);
            var nlAfter = re.slice(nl.reEnd);
            nlLast += nlAfter;
            // Handle nested stuff like *(*.js|!(*.json)), where open parens
            // mean that we should *not* include the ) in the bit that is considered
            // "after" the negated section.
            var openParensBefore = nlBefore.split('(').length - 1;
            var cleanAfter = nlAfter;
            for (i = 0; i < openParensBefore; i++) {
                cleanAfter = cleanAfter.replace(/\)[+*?]?/, '');
            }
            nlAfter = cleanAfter;
            var dollar = '';
            if (nlAfter === '' && isSub !== SUBPARSE) {
                dollar = '$';
            }
            var newRe = nlBefore + nlFirst + nlAfter + dollar + nlLast;
            re = newRe;
        }
        // if the re is not "" at this point, then we need to make sure
        // it doesn't match against an empty path part.
        // Otherwise a/* will match a/, which it should not.
        if (re !== '' && hasMagic) {
            re = '(?=.)' + re;
        }
        if (addPatternStart) {
            re = patternStart + re;
        }
        // parsing just a piece of a larger pattern.
        if (isSub === SUBPARSE) {
            return [re, hasMagic];
        }
        // skip the regexp for non-magical patterns
        // unescape anything in it, though, so that it'll be
        // an exact match against a file etc.
        if (!hasMagic) {
            return globUnescape(pattern);
        }
        var flags = options.nocase ? 'i' : '';
        try {
            var regExp = new RegExp('^' + re + '$', flags);
        }
        catch (er) {
            // If it was an invalid regular expression, then it can't match
            // anything.  This trick looks for a character after the end of
            // the string, which is of course impossible, except in multi-line
            // mode, but it's not a /m regex.
            return new RegExp('$.');
        }
        regExp._glob = pattern;
        regExp._src = re;
        return regExp;
    }
    minimatch.makeRe = function (pattern, options) {
        return new Minimatch(pattern, options || {}).makeRe();
    };
    Minimatch.prototype.makeRe = makeRe;
    function makeRe() {
        if (this.regexp || this.regexp === false)
            return this.regexp;
        // at this point, this.set is a 2d array of partial
        // pattern strings, or "**".
        //
        // It's better to use .match().  This function shouldn't
        // be used, really, but it's pretty convenient sometimes,
        // when you just want to work with a regex.
        var set = this.set;
        if (!set.length) {
            this.regexp = false;
            return this.regexp;
        }
        var options = this.options;
        var twoStar = options.noglobstar ? star
            : options.dot ? twoStarDot
                : twoStarNoDot;
        var flags = options.nocase ? 'i' : '';
        var re = set.map(function (pattern) {
            return pattern.map(function (p) {
                return (p === GLOBSTAR) ? twoStar
                    : (typeof p === 'string') ? regExpEscape(p)
                        : p._src;
            }).join('\\\/');
        }).join('|');
        // must match entire pattern
        // ending in a * or ** will make it less strict.
        re = '^(?:' + re + ')$';
        // can match anything, as long as it's not this.
        if (this.negate)
            re = '^(?!' + re + ').*$';
        try {
            this.regexp = new RegExp(re, flags);
        }
        catch (ex) {
            this.regexp = false;
        }
        return this.regexp;
    }
    minimatch.match = function (list, pattern, options) {
        options = options || {};
        var mm = new Minimatch(pattern, options);
        list = list.filter(function (f) {
            return mm.match(f);
        });
        if (mm.options.nonull && !list.length) {
            list.push(pattern);
        }
        return list;
    };
    Minimatch.prototype.match = match;
    function match(f, partial) {
        this.debug('match', f, this.pattern);
        // short-circuit in the case of busted things.
        // comments, etc.
        if (this.comment)
            return false;
        if (this.empty)
            return f === '';
        if (f === '/' && partial)
            return true;
        var options = this.options;
        // windows: need to use /, not \
        if (path.sep !== '/') {
            f = f.split(path.sep).join('/');
        }
        // treat the test path as a set of pathparts.
        f = f.split(slashSplit);
        this.debug(this.pattern, 'split', f);
        // just ONE of the pattern sets in this.set needs to match
        // in order for it to be valid.  If negating, then just one
        // match means that we have failed.
        // Either way, return on the first hit.
        var set = this.set;
        this.debug(this.pattern, 'set', set);
        // Find the basename of the path by looking for the last non-empty segment
        var filename;
        var i;
        for (i = f.length - 1; i >= 0; i--) {
            filename = f[i];
            if (filename)
                break;
        }
        for (i = 0; i < set.length; i++) {
            var pattern = set[i];
            var file = f;
            if (options.matchBase && pattern.length === 1) {
                file = [filename];
            }
            var hit = this.matchOne(file, pattern, partial);
            if (hit) {
                if (options.flipNegate)
                    return true;
                return !this.negate;
            }
        }
        // didn't get any hits.  this is success if it's a negative
        // pattern, failure otherwise.
        if (options.flipNegate)
            return false;
        return this.negate;
    }
    // set partial to true to test if, for example,
    // "/a/b" matches the start of "/*/b/*/d"
    // Partial means, if you run out of file before you run
    // out of pattern, then that's fine, as long as all
    // the parts match.
    Minimatch.prototype.matchOne = function (file, pattern, partial) {
        var options = this.options;
        this.debug('matchOne', { 'this': this, file: file, pattern: pattern });
        this.debug('matchOne', file.length, pattern.length);
        for (var fi = 0, pi = 0, fl = file.length, pl = pattern.length; (fi < fl) && (pi < pl); fi++, pi++) {
            this.debug('matchOne loop');
            var p = pattern[pi];
            var f = file[fi];
            this.debug(pattern, p, f);
            // should be impossible.
            // some invalid regexp stuff in the set.
            if (p === false)
                return false;
            if (p === GLOBSTAR) {
                this.debug('GLOBSTAR', [pattern, p, f]);
                // "**"
                // a/**/b/**/c would match the following:
                // a/b/x/y/z/c
                // a/x/y/z/b/c
                // a/b/x/b/x/c
                // a/b/c
                // To do this, take the rest of the pattern after
                // the **, and see if it would match the file remainder.
                // If so, return success.
                // If not, the ** "swallows" a segment, and try again.
                // This is recursively awful.
                //
                // a/**/b/**/c matching a/b/x/y/z/c
                // - a matches a
                // - doublestar
                //   - matchOne(b/x/y/z/c, b/**/c)
                //     - b matches b
                //     - doublestar
                //       - matchOne(x/y/z/c, c) -> no
                //       - matchOne(y/z/c, c) -> no
                //       - matchOne(z/c, c) -> no
                //       - matchOne(c, c) yes, hit
                var fr = fi;
                var pr = pi + 1;
                if (pr === pl) {
                    this.debug('** at the end');
                    // a ** at the end will just swallow the rest.
                    // We have found a match.
                    // however, it will not swallow /.x, unless
                    // options.dot is set.
                    // . and .. are *never* matched by **, for explosively
                    // exponential reasons.
                    for (; fi < fl; fi++) {
                        if (file[fi] === '.' || file[fi] === '..' ||
                            (!options.dot && file[fi].charAt(0) === '.'))
                            return false;
                    }
                    return true;
                }
                // ok, let's see if we can swallow whatever we can.
                while (fr < fl) {
                    var swallowee = file[fr];
                    this.debug('\nglobstar while', file, fr, pattern, pr, swallowee);
                    // XXX remove this slice.  Just pass the start index.
                    if (this.matchOne(file.slice(fr), pattern.slice(pr), partial)) {
                        this.debug('globstar found match!', fr, fl, swallowee);
                        // found a match.
                        return true;
                    }
                    else {
                        // can't swallow "." or ".." ever.
                        // can only swallow ".foo" when explicitly asked.
                        if (swallowee === '.' || swallowee === '..' ||
                            (!options.dot && swallowee.charAt(0) === '.')) {
                            this.debug('dot detected!', file, fr, pattern, pr);
                            break;
                        }
                        // ** swallows a segment, and continue.
                        this.debug('globstar swallow a segment, and continue');
                        fr++;
                    }
                }
                // no match was found.
                // However, in partial mode, we can't say this is necessarily over.
                // If there's more *pattern* left, then
                if (partial) {
                    // ran out of file
                    this.debug('\n>>> no match, partial?', file, fr, pattern, pr);
                    if (fr === fl)
                        return true;
                }
                return false;
            }
            // something other than **
            // non-magic patterns just have to match exactly
            // patterns with magic have been turned into regexps.
            var hit;
            if (typeof p === 'string') {
                if (options.nocase) {
                    hit = f.toLowerCase() === p.toLowerCase();
                }
                else {
                    hit = f === p;
                }
                this.debug('string match', p, f, hit);
            }
            else {
                hit = f.match(p);
                this.debug('pattern match', p, f, hit);
            }
            if (!hit)
                return false;
        }
        // Note: ending in / means that we'll get a final ""
        // at the end of the pattern.  This can only match a
        // corresponding "" at the end of the file.
        // If the file ends in /, then it can only match a
        // a pattern that ends in /, unless the pattern just
        // doesn't have any more for it. But, a/b/ should *not*
        // match "a/b/*", even though "" matches against the
        // [^/]*? pattern, except in partial mode, where it might
        // simply not be reached yet.
        // However, a/b/ should still satisfy a/*
        // now either we fell off the end of the pattern, or we're done.
        if (fi === fl && pi === pl) {
            // ran out of pattern and filename at the same time.
            // an exact hit!
            return true;
        }
        else if (fi === fl) {
            // ran out of file, but still had pattern left.
            // this is ok if we're doing the match as part of
            // a glob fs traversal.
            return partial;
        }
        else if (pi === pl) {
            // ran out of pattern, still have file left.
            // this is only acceptable if we're on the very last
            // empty segment of a file with a trailing slash.
            // a/* should match a/b/
            var emptyFileEnd = (fi === fl - 1) && (file[fi] === '');
            return emptyFileEnd;
        }
        // should be unreachable.
        throw new Error('wtf?');
    };
    // replace stuff like \* with *
    function globUnescape(s) {
        return s.replace(/\\(.)/g, '$1');
    }
    function regExpEscape(s) {
        return s.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, '\\$&');
    }
},
/* brace-expansion/index.js */ function _(require, module, exports) {
    var concatMap = require(533) /* concat-map */;
    var balanced = require(534) /* balanced-match */;
    module.exports = expandTop;
    var escSlash = '\0SLASH' + Math.random() + '\0';
    var escOpen = '\0OPEN' + Math.random() + '\0';
    var escClose = '\0CLOSE' + Math.random() + '\0';
    var escComma = '\0COMMA' + Math.random() + '\0';
    var escPeriod = '\0PERIOD' + Math.random() + '\0';
    function numeric(str) {
        return parseInt(str, 10) == str
            ? parseInt(str, 10)
            : str.charCodeAt(0);
    }
    function escapeBraces(str) {
        return str.split('\\\\').join(escSlash)
            .split('\\{').join(escOpen)
            .split('\\}').join(escClose)
            .split('\\,').join(escComma)
            .split('\\.').join(escPeriod);
    }
    function unescapeBraces(str) {
        return str.split(escSlash).join('\\')
            .split(escOpen).join('{')
            .split(escClose).join('}')
            .split(escComma).join(',')
            .split(escPeriod).join('.');
    }
    // Basically just str.split(","), but handling cases
    // where we have nested braced sections, which should be
    // treated as individual members, like {a,{b,c},d}
    function parseCommaParts(str) {
        if (!str)
            return [''];
        var parts = [];
        var m = balanced('{', '}', str);
        if (!m)
            return str.split(',');
        var pre = m.pre;
        var body = m.body;
        var post = m.post;
        var p = pre.split(',');
        p[p.length - 1] += '{' + body + '}';
        var postParts = parseCommaParts(post);
        if (post.length) {
            p[p.length - 1] += postParts.shift();
            p.push.apply(p, postParts);
        }
        parts.push.apply(parts, p);
        return parts;
    }
    function expandTop(str) {
        if (!str)
            return [];
        // I don't know why Bash 4.3 does this, but it does.
        // Anything starting with {} will have the first two bytes preserved
        // but *only* at the top level, so {},a}b will not expand to anything,
        // but a{},b}c will be expanded to [a}c,abc].
        // One could argue that this is a bug in Bash, but since the goal of
        // this module is to match Bash's rules, we escape a leading {}
        if (str.substr(0, 2) === '{}') {
            str = '\\{\\}' + str.substr(2);
        }
        return expand(escapeBraces(str), true).map(unescapeBraces);
    }
    function identity(e) {
        return e;
    }
    function embrace(str) {
        return '{' + str + '}';
    }
    function isPadded(el) {
        return /^-?0\d/.test(el);
    }
    function lte(i, y) {
        return i <= y;
    }
    function gte(i, y) {
        return i >= y;
    }
    function expand(str, isTop) {
        var expansions = [];
        var m = balanced('{', '}', str);
        if (!m || /\$$/.test(m.pre))
            return [str];
        var isNumericSequence = /^-?\d+\.\.-?\d+(?:\.\.-?\d+)?$/.test(m.body);
        var isAlphaSequence = /^[a-zA-Z]\.\.[a-zA-Z](?:\.\.-?\d+)?$/.test(m.body);
        var isSequence = isNumericSequence || isAlphaSequence;
        var isOptions = m.body.indexOf(',') >= 0;
        if (!isSequence && !isOptions) {
            // {a},b}
            if (m.post.match(/,.*\}/)) {
                str = m.pre + '{' + m.body + escClose + m.post;
                return expand(str);
            }
            return [str];
        }
        var n;
        if (isSequence) {
            n = m.body.split(/\.\./);
        }
        else {
            n = parseCommaParts(m.body);
            if (n.length === 1) {
                // x{{a,b}}y ==> x{a}y x{b}y
                n = expand(n[0], false).map(embrace);
                if (n.length === 1) {
                    var post = m.post.length
                        ? expand(m.post, false)
                        : [''];
                    return post.map(function (p) {
                        return m.pre + n[0] + p;
                    });
                }
            }
        }
        // at this point, n is the parts, and we know it's not a comma set
        // with a single entry.
        // no need to expand pre, since it is guaranteed to be free of brace-sets
        var pre = m.pre;
        var post = m.post.length
            ? expand(m.post, false)
            : [''];
        var N;
        if (isSequence) {
            var x = numeric(n[0]);
            var y = numeric(n[1]);
            var width = Math.max(n[0].length, n[1].length);
            var incr = n.length == 3
                ? Math.abs(numeric(n[2]))
                : 1;
            var test = lte;
            var reverse = y < x;
            if (reverse) {
                incr *= -1;
                test = gte;
            }
            var pad = n.some(isPadded);
            N = [];
            for (var i = x; test(i, y); i += incr) {
                var c;
                if (isAlphaSequence) {
                    c = String.fromCharCode(i);
                    if (c === '\\')
                        c = '';
                }
                else {
                    c = String(i);
                    if (pad) {
                        var need = width - c.length;
                        if (need > 0) {
                            var z = new Array(need + 1).join('0');
                            if (i < 0)
                                c = '-' + z + c.slice(1);
                            else
                                c = z + c;
                        }
                    }
                }
                N.push(c);
            }
        }
        else {
            N = concatMap(n, function (el) { return expand(el, false); });
        }
        for (var j = 0; j < N.length; j++) {
            for (var k = 0; k < post.length; k++) {
                var expansion = pre + N[j] + post[k];
                if (!isTop || isSequence || expansion)
                    expansions.push(expansion);
            }
        }
        return expansions;
    }
},
/* concat-map/index.js */ function _(require, module, exports) {
    module.exports = function (xs, fn) {
        var res = [];
        for (var i = 0; i < xs.length; i++) {
            var x = fn(xs[i], i);
            if (isArray(x))
                res.push.apply(res, x);
            else
                res.push(x);
        }
        return res;
    };
    var isArray = Array.isArray || function (xs) {
        return Object.prototype.toString.call(xs) === '[object Array]';
    };
},
/* balanced-match/index.js */ function _(require, module, exports) {
    module.exports = balanced;
    function balanced(a, b, str) {
        if (a instanceof RegExp)
            a = maybeMatch(a, str);
        if (b instanceof RegExp)
            b = maybeMatch(b, str);
        var r = range(a, b, str);
        return r && {
            start: r[0],
            end: r[1],
            pre: str.slice(0, r[0]),
            body: str.slice(r[0] + a.length, r[1]),
            post: str.slice(r[1] + b.length)
        };
    }
    function maybeMatch(reg, str) {
        var m = str.match(reg);
        return m ? m[0] : null;
    }
    balanced.range = range;
    function range(a, b, str) {
        var begs, beg, left, right, result;
        var ai = str.indexOf(a);
        var bi = str.indexOf(b, ai + 1);
        var i = ai;
        if (ai >= 0 && bi > 0) {
            begs = [];
            left = str.length;
            while (i >= 0 && !result) {
                if (i == ai) {
                    begs.push(i);
                    ai = str.indexOf(a, i + 1);
                }
                else if (begs.length == 1) {
                    result = [begs.pop(), bi];
                }
                else {
                    beg = begs.pop();
                    if (beg < left) {
                        left = beg;
                        right = bi;
                    }
                    bi = str.indexOf(b, i + 1);
                }
                i = ai < bi && ai >= 0 ? ai : bi;
            }
            if (begs.length) {
                result = [left, right];
            }
        }
        return result;
    }
},
/* js-yaml/index.js */ function _(require, module, exports) {
    var yaml = require(536) /* ./lib/js-yaml.js */;
    module.exports = yaml;
},
/* js-yaml/lib/js-yaml.js */ function _(require, module, exports) {
    var loader = require(537) /* ./js-yaml/loader */;
    var dumper = require(564) /* ./js-yaml/dumper */;
    function deprecated(name) {
        return function () {
            throw new Error('Function ' + name + ' is deprecated and cannot be used.');
        };
    }
    module.exports.Type = require(543) /* ./js-yaml/type */;
    module.exports.Schema = require(542) /* ./js-yaml/schema */;
    module.exports.FAILSAFE_SCHEMA = require(546) /* ./js-yaml/schema/failsafe */;
    module.exports.JSON_SCHEMA = require(545) /* ./js-yaml/schema/json */;
    module.exports.CORE_SCHEMA = require(544) /* ./js-yaml/schema/core */;
    module.exports.DEFAULT_SAFE_SCHEMA = require(541) /* ./js-yaml/schema/default_safe */;
    module.exports.DEFAULT_FULL_SCHEMA = require(560) /* ./js-yaml/schema/default_full */;
    module.exports.load = loader.load;
    module.exports.loadAll = loader.loadAll;
    module.exports.safeLoad = loader.safeLoad;
    module.exports.safeLoadAll = loader.safeLoadAll;
    module.exports.dump = dumper.dump;
    module.exports.safeDump = dumper.safeDump;
    module.exports.YAMLException = require(539) /* ./js-yaml/exception */;
    // Deprecated schema names from JS-YAML 2.0.x
    module.exports.MINIMAL_SCHEMA = require(546) /* ./js-yaml/schema/failsafe */;
    module.exports.SAFE_SCHEMA = require(541) /* ./js-yaml/schema/default_safe */;
    module.exports.DEFAULT_SCHEMA = require(560) /* ./js-yaml/schema/default_full */;
    // Deprecated functions from JS-YAML 1.x.x
    module.exports.scan = deprecated('scan');
    module.exports.parse = deprecated('parse');
    module.exports.compose = deprecated('compose');
    module.exports.addConstructor = deprecated('addConstructor');
},
/* js-yaml/lib/js-yaml/loader.js */ function _(require, module, exports) {
    /*eslint-disable max-len,no-use-before-define*/
    var common = require(538) /* ./common */;
    var YAMLException = require(539) /* ./exception */;
    var Mark = require(540) /* ./mark */;
    var DEFAULT_SAFE_SCHEMA = require(541) /* ./schema/default_safe */;
    var DEFAULT_FULL_SCHEMA = require(560) /* ./schema/default_full */;
    var _hasOwnProperty = Object.prototype.hasOwnProperty;
    var CONTEXT_FLOW_IN = 1;
    var CONTEXT_FLOW_OUT = 2;
    var CONTEXT_BLOCK_IN = 3;
    var CONTEXT_BLOCK_OUT = 4;
    var CHOMPING_CLIP = 1;
    var CHOMPING_STRIP = 2;
    var CHOMPING_KEEP = 3;
    var PATTERN_NON_PRINTABLE = /[\x00-\x08\x0B\x0C\x0E-\x1F\x7F-\x84\x86-\x9F\uFFFE\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]/;
    var PATTERN_NON_ASCII_LINE_BREAKS = /[\x85\u2028\u2029]/;
    var PATTERN_FLOW_INDICATORS = /[,\[\]\{\}]/;
    var PATTERN_TAG_HANDLE = /^(?:!|!!|![a-z\-]+!)$/i;
    var PATTERN_TAG_URI = /^(?:!|[^,\[\]\{\}])(?:%[0-9a-f]{2}|[0-9a-z\-#;\/\?:@&=\+\$,_\.!~\*'\(\)\[\]])*$/i;
    function _class(obj) { return Object.prototype.toString.call(obj); }
    function is_EOL(c) {
        return (c === 0x0A /* LF */) || (c === 0x0D /* CR */);
    }
    function is_WHITE_SPACE(c) {
        return (c === 0x09 /* Tab */) || (c === 0x20 /* Space */);
    }
    function is_WS_OR_EOL(c) {
        return (c === 0x09 /* Tab */) ||
            (c === 0x20 /* Space */) ||
            (c === 0x0A /* LF */) ||
            (c === 0x0D /* CR */);
    }
    function is_FLOW_INDICATOR(c) {
        return c === 0x2C /* , */ ||
            c === 0x5B /* [ */ ||
            c === 0x5D /* ] */ ||
            c === 0x7B /* { */ ||
            c === 0x7D /* } */;
    }
    function fromHexCode(c) {
        var lc;
        if ((0x30 /* 0 */ <= c) && (c <= 0x39 /* 9 */)) {
            return c - 0x30;
        }
        /*eslint-disable no-bitwise*/
        lc = c | 0x20;
        if ((0x61 /* a */ <= lc) && (lc <= 0x66 /* f */)) {
            return lc - 0x61 + 10;
        }
        return -1;
    }
    function escapedHexLen(c) {
        if (c === 0x78 /* x */) {
            return 2;
        }
        if (c === 0x75 /* u */) {
            return 4;
        }
        if (c === 0x55 /* U */) {
            return 8;
        }
        return 0;
    }
    function fromDecimalCode(c) {
        if ((0x30 /* 0 */ <= c) && (c <= 0x39 /* 9 */)) {
            return c - 0x30;
        }
        return -1;
    }
    function simpleEscapeSequence(c) {
        /* eslint-disable indent */
        return (c === 0x30 /* 0 */) ? '\x00' :
            (c === 0x61 /* a */) ? '\x07' :
                (c === 0x62 /* b */) ? '\x08' :
                    (c === 0x74 /* t */) ? '\x09' :
                        (c === 0x09 /* Tab */) ? '\x09' :
                            (c === 0x6E /* n */) ? '\x0A' :
                                (c === 0x76 /* v */) ? '\x0B' :
                                    (c === 0x66 /* f */) ? '\x0C' :
                                        (c === 0x72 /* r */) ? '\x0D' :
                                            (c === 0x65 /* e */) ? '\x1B' :
                                                (c === 0x20 /* Space */) ? ' ' :
                                                    (c === 0x22 /* " */) ? '\x22' :
                                                        (c === 0x2F /* / */) ? '/' :
                                                            (c === 0x5C /* \ */) ? '\x5C' :
                                                                (c === 0x4E /* N */) ? '\x85' :
                                                                    (c === 0x5F /* _ */) ? '\xA0' :
                                                                        (c === 0x4C /* L */) ? '\u2028' :
                                                                            (c === 0x50 /* P */) ? '\u2029' : '';
    }
    function charFromCodepoint(c) {
        if (c <= 0xFFFF) {
            return String.fromCharCode(c);
        }
        // Encode UTF-16 surrogate pair
        // https://en.wikipedia.org/wiki/UTF-16#Code_points_U.2B010000_to_U.2B10FFFF
        return String.fromCharCode(((c - 0x010000) >> 10) + 0xD800, ((c - 0x010000) & 0x03FF) + 0xDC00);
    }
    var simpleEscapeCheck = new Array(256); // integer, for fast access
    var simpleEscapeMap = new Array(256);
    for (var i = 0; i < 256; i++) {
        simpleEscapeCheck[i] = simpleEscapeSequence(i) ? 1 : 0;
        simpleEscapeMap[i] = simpleEscapeSequence(i);
    }
    function State(input, options) {
        this.input = input;
        this.filename = options['filename'] || null;
        this.schema = options['schema'] || DEFAULT_FULL_SCHEMA;
        this.onWarning = options['onWarning'] || null;
        this.legacy = options['legacy'] || false;
        this.json = options['json'] || false;
        this.listener = options['listener'] || null;
        this.implicitTypes = this.schema.compiledImplicit;
        this.typeMap = this.schema.compiledTypeMap;
        this.length = input.length;
        this.position = 0;
        this.line = 0;
        this.lineStart = 0;
        this.lineIndent = 0;
        this.documents = [];
        /*
        this.version;
        this.checkLineBreaks;
        this.tagMap;
        this.anchorMap;
        this.tag;
        this.anchor;
        this.kind;
        this.result;*/
    }
    function generateError(state, message) {
        return new YAMLException(message, new Mark(state.filename, state.input, state.position, state.line, (state.position - state.lineStart)));
    }
    function throwError(state, message) {
        throw generateError(state, message);
    }
    function throwWarning(state, message) {
        if (state.onWarning) {
            state.onWarning.call(null, generateError(state, message));
        }
    }
    var directiveHandlers = {
        YAML: function handleYamlDirective(state, name, args) {
            var match, major, minor;
            if (state.version !== null) {
                throwError(state, 'duplication of %YAML directive');
            }
            if (args.length !== 1) {
                throwError(state, 'YAML directive accepts exactly one argument');
            }
            match = /^([0-9]+)\.([0-9]+)$/.exec(args[0]);
            if (match === null) {
                throwError(state, 'ill-formed argument of the YAML directive');
            }
            major = parseInt(match[1], 10);
            minor = parseInt(match[2], 10);
            if (major !== 1) {
                throwError(state, 'unacceptable YAML version of the document');
            }
            state.version = args[0];
            state.checkLineBreaks = (minor < 2);
            if (minor !== 1 && minor !== 2) {
                throwWarning(state, 'unsupported YAML version of the document');
            }
        },
        TAG: function handleTagDirective(state, name, args) {
            var handle, prefix;
            if (args.length !== 2) {
                throwError(state, 'TAG directive accepts exactly two arguments');
            }
            handle = args[0];
            prefix = args[1];
            if (!PATTERN_TAG_HANDLE.test(handle)) {
                throwError(state, 'ill-formed tag handle (first argument) of the TAG directive');
            }
            if (_hasOwnProperty.call(state.tagMap, handle)) {
                throwError(state, 'there is a previously declared suffix for "' + handle + '" tag handle');
            }
            if (!PATTERN_TAG_URI.test(prefix)) {
                throwError(state, 'ill-formed tag prefix (second argument) of the TAG directive');
            }
            state.tagMap[handle] = prefix;
        }
    };
    function captureSegment(state, start, end, checkJson) {
        var _position, _length, _character, _result;
        if (start < end) {
            _result = state.input.slice(start, end);
            if (checkJson) {
                for (_position = 0, _length = _result.length; _position < _length; _position += 1) {
                    _character = _result.charCodeAt(_position);
                    if (!(_character === 0x09 ||
                        (0x20 <= _character && _character <= 0x10FFFF))) {
                        throwError(state, 'expected valid JSON character');
                    }
                }
            }
            else if (PATTERN_NON_PRINTABLE.test(_result)) {
                throwError(state, 'the stream contains non-printable characters');
            }
            state.result += _result;
        }
    }
    function mergeMappings(state, destination, source, overridableKeys) {
        var sourceKeys, key, index, quantity;
        if (!common.isObject(source)) {
            throwError(state, 'cannot merge mappings; the provided source object is unacceptable');
        }
        sourceKeys = Object.keys(source);
        for (index = 0, quantity = sourceKeys.length; index < quantity; index += 1) {
            key = sourceKeys[index];
            if (!_hasOwnProperty.call(destination, key)) {
                destination[key] = source[key];
                overridableKeys[key] = true;
            }
        }
    }
    function storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode, startLine, startPos) {
        var index, quantity;
        // The output is a plain object here, so keys can only be strings.
        // We need to convert keyNode to a string, but doing so can hang the process
        // (deeply nested arrays that explode exponentially using aliases).
        if (Array.isArray(keyNode)) {
            keyNode = Array.prototype.slice.call(keyNode);
            for (index = 0, quantity = keyNode.length; index < quantity; index += 1) {
                if (Array.isArray(keyNode[index])) {
                    throwError(state, 'nested arrays are not supported inside keys');
                }
                if (typeof keyNode === 'object' && _class(keyNode[index]) === '[object Object]') {
                    keyNode[index] = '[object Object]';
                }
            }
        }
        // Avoid code execution in load() via toString property
        // (still use its own toString for arrays, timestamps,
        // and whatever user schema extensions happen to have @@toStringTag)
        if (typeof keyNode === 'object' && _class(keyNode) === '[object Object]') {
            keyNode = '[object Object]';
        }
        keyNode = String(keyNode);
        if (_result === null) {
            _result = {};
        }
        if (keyTag === 'tag:yaml.org,2002:merge') {
            if (Array.isArray(valueNode)) {
                for (index = 0, quantity = valueNode.length; index < quantity; index += 1) {
                    mergeMappings(state, _result, valueNode[index], overridableKeys);
                }
            }
            else {
                mergeMappings(state, _result, valueNode, overridableKeys);
            }
        }
        else {
            if (!state.json &&
                !_hasOwnProperty.call(overridableKeys, keyNode) &&
                _hasOwnProperty.call(_result, keyNode)) {
                state.line = startLine || state.line;
                state.position = startPos || state.position;
                throwError(state, 'duplicated mapping key');
            }
            _result[keyNode] = valueNode;
            delete overridableKeys[keyNode];
        }
        return _result;
    }
    function readLineBreak(state) {
        var ch;
        ch = state.input.charCodeAt(state.position);
        if (ch === 0x0A /* LF */) {
            state.position++;
        }
        else if (ch === 0x0D /* CR */) {
            state.position++;
            if (state.input.charCodeAt(state.position) === 0x0A /* LF */) {
                state.position++;
            }
        }
        else {
            throwError(state, 'a line break is expected');
        }
        state.line += 1;
        state.lineStart = state.position;
    }
    function skipSeparationSpace(state, allowComments, checkIndent) {
        var lineBreaks = 0, ch = state.input.charCodeAt(state.position);
        while (ch !== 0) {
            while (is_WHITE_SPACE(ch)) {
                ch = state.input.charCodeAt(++state.position);
            }
            if (allowComments && ch === 0x23 /* # */) {
                do {
                    ch = state.input.charCodeAt(++state.position);
                } while (ch !== 0x0A /* LF */ && ch !== 0x0D /* CR */ && ch !== 0);
            }
            if (is_EOL(ch)) {
                readLineBreak(state);
                ch = state.input.charCodeAt(state.position);
                lineBreaks++;
                state.lineIndent = 0;
                while (ch === 0x20 /* Space */) {
                    state.lineIndent++;
                    ch = state.input.charCodeAt(++state.position);
                }
            }
            else {
                break;
            }
        }
        if (checkIndent !== -1 && lineBreaks !== 0 && state.lineIndent < checkIndent) {
            throwWarning(state, 'deficient indentation');
        }
        return lineBreaks;
    }
    function testDocumentSeparator(state) {
        var _position = state.position, ch;
        ch = state.input.charCodeAt(_position);
        // Condition state.position === state.lineStart is tested
        // in parent on each call, for efficiency. No needs to test here again.
        if ((ch === 0x2D /* - */ || ch === 0x2E /* . */) &&
            ch === state.input.charCodeAt(_position + 1) &&
            ch === state.input.charCodeAt(_position + 2)) {
            _position += 3;
            ch = state.input.charCodeAt(_position);
            if (ch === 0 || is_WS_OR_EOL(ch)) {
                return true;
            }
        }
        return false;
    }
    function writeFoldedLines(state, count) {
        if (count === 1) {
            state.result += ' ';
        }
        else if (count > 1) {
            state.result += common.repeat('\n', count - 1);
        }
    }
    function readPlainScalar(state, nodeIndent, withinFlowCollection) {
        var preceding, following, captureStart, captureEnd, hasPendingContent, _line, _lineStart, _lineIndent, _kind = state.kind, _result = state.result, ch;
        ch = state.input.charCodeAt(state.position);
        if (is_WS_OR_EOL(ch) ||
            is_FLOW_INDICATOR(ch) ||
            ch === 0x23 /* # */ ||
            ch === 0x26 /* & */ ||
            ch === 0x2A /* * */ ||
            ch === 0x21 /* ! */ ||
            ch === 0x7C /* | */ ||
            ch === 0x3E /* > */ ||
            ch === 0x27 /* ' */ ||
            ch === 0x22 /* " */ ||
            ch === 0x25 /* % */ ||
            ch === 0x40 /* @ */ ||
            ch === 0x60 /* ` */) {
            return false;
        }
        if (ch === 0x3F /* ? */ || ch === 0x2D /* - */) {
            following = state.input.charCodeAt(state.position + 1);
            if (is_WS_OR_EOL(following) ||
                withinFlowCollection && is_FLOW_INDICATOR(following)) {
                return false;
            }
        }
        state.kind = 'scalar';
        state.result = '';
        captureStart = captureEnd = state.position;
        hasPendingContent = false;
        while (ch !== 0) {
            if (ch === 0x3A /* : */) {
                following = state.input.charCodeAt(state.position + 1);
                if (is_WS_OR_EOL(following) ||
                    withinFlowCollection && is_FLOW_INDICATOR(following)) {
                    break;
                }
            }
            else if (ch === 0x23 /* # */) {
                preceding = state.input.charCodeAt(state.position - 1);
                if (is_WS_OR_EOL(preceding)) {
                    break;
                }
            }
            else if ((state.position === state.lineStart && testDocumentSeparator(state)) ||
                withinFlowCollection && is_FLOW_INDICATOR(ch)) {
                break;
            }
            else if (is_EOL(ch)) {
                _line = state.line;
                _lineStart = state.lineStart;
                _lineIndent = state.lineIndent;
                skipSeparationSpace(state, false, -1);
                if (state.lineIndent >= nodeIndent) {
                    hasPendingContent = true;
                    ch = state.input.charCodeAt(state.position);
                    continue;
                }
                else {
                    state.position = captureEnd;
                    state.line = _line;
                    state.lineStart = _lineStart;
                    state.lineIndent = _lineIndent;
                    break;
                }
            }
            if (hasPendingContent) {
                captureSegment(state, captureStart, captureEnd, false);
                writeFoldedLines(state, state.line - _line);
                captureStart = captureEnd = state.position;
                hasPendingContent = false;
            }
            if (!is_WHITE_SPACE(ch)) {
                captureEnd = state.position + 1;
            }
            ch = state.input.charCodeAt(++state.position);
        }
        captureSegment(state, captureStart, captureEnd, false);
        if (state.result) {
            return true;
        }
        state.kind = _kind;
        state.result = _result;
        return false;
    }
    function readSingleQuotedScalar(state, nodeIndent) {
        var ch, captureStart, captureEnd;
        ch = state.input.charCodeAt(state.position);
        if (ch !== 0x27 /* ' */) {
            return false;
        }
        state.kind = 'scalar';
        state.result = '';
        state.position++;
        captureStart = captureEnd = state.position;
        while ((ch = state.input.charCodeAt(state.position)) !== 0) {
            if (ch === 0x27 /* ' */) {
                captureSegment(state, captureStart, state.position, true);
                ch = state.input.charCodeAt(++state.position);
                if (ch === 0x27 /* ' */) {
                    captureStart = state.position;
                    state.position++;
                    captureEnd = state.position;
                }
                else {
                    return true;
                }
            }
            else if (is_EOL(ch)) {
                captureSegment(state, captureStart, captureEnd, true);
                writeFoldedLines(state, skipSeparationSpace(state, false, nodeIndent));
                captureStart = captureEnd = state.position;
            }
            else if (state.position === state.lineStart && testDocumentSeparator(state)) {
                throwError(state, 'unexpected end of the document within a single quoted scalar');
            }
            else {
                state.position++;
                captureEnd = state.position;
            }
        }
        throwError(state, 'unexpected end of the stream within a single quoted scalar');
    }
    function readDoubleQuotedScalar(state, nodeIndent) {
        var captureStart, captureEnd, hexLength, hexResult, tmp, ch;
        ch = state.input.charCodeAt(state.position);
        if (ch !== 0x22 /* " */) {
            return false;
        }
        state.kind = 'scalar';
        state.result = '';
        state.position++;
        captureStart = captureEnd = state.position;
        while ((ch = state.input.charCodeAt(state.position)) !== 0) {
            if (ch === 0x22 /* " */) {
                captureSegment(state, captureStart, state.position, true);
                state.position++;
                return true;
            }
            else if (ch === 0x5C /* \ */) {
                captureSegment(state, captureStart, state.position, true);
                ch = state.input.charCodeAt(++state.position);
                if (is_EOL(ch)) {
                    skipSeparationSpace(state, false, nodeIndent);
                    // TODO: rework to inline fn with no type cast?
                }
                else if (ch < 256 && simpleEscapeCheck[ch]) {
                    state.result += simpleEscapeMap[ch];
                    state.position++;
                }
                else if ((tmp = escapedHexLen(ch)) > 0) {
                    hexLength = tmp;
                    hexResult = 0;
                    for (; hexLength > 0; hexLength--) {
                        ch = state.input.charCodeAt(++state.position);
                        if ((tmp = fromHexCode(ch)) >= 0) {
                            hexResult = (hexResult << 4) + tmp;
                        }
                        else {
                            throwError(state, 'expected hexadecimal character');
                        }
                    }
                    state.result += charFromCodepoint(hexResult);
                    state.position++;
                }
                else {
                    throwError(state, 'unknown escape sequence');
                }
                captureStart = captureEnd = state.position;
            }
            else if (is_EOL(ch)) {
                captureSegment(state, captureStart, captureEnd, true);
                writeFoldedLines(state, skipSeparationSpace(state, false, nodeIndent));
                captureStart = captureEnd = state.position;
            }
            else if (state.position === state.lineStart && testDocumentSeparator(state)) {
                throwError(state, 'unexpected end of the document within a double quoted scalar');
            }
            else {
                state.position++;
                captureEnd = state.position;
            }
        }
        throwError(state, 'unexpected end of the stream within a double quoted scalar');
    }
    function readFlowCollection(state, nodeIndent) {
        var readNext = true, _line, _tag = state.tag, _result, _anchor = state.anchor, following, terminator, isPair, isExplicitPair, isMapping, overridableKeys = {}, keyNode, keyTag, valueNode, ch;
        ch = state.input.charCodeAt(state.position);
        if (ch === 0x5B /* [ */) {
            terminator = 0x5D; /* ] */
            isMapping = false;
            _result = [];
        }
        else if (ch === 0x7B /* { */) {
            terminator = 0x7D; /* } */
            isMapping = true;
            _result = {};
        }
        else {
            return false;
        }
        if (state.anchor !== null) {
            state.anchorMap[state.anchor] = _result;
        }
        ch = state.input.charCodeAt(++state.position);
        while (ch !== 0) {
            skipSeparationSpace(state, true, nodeIndent);
            ch = state.input.charCodeAt(state.position);
            if (ch === terminator) {
                state.position++;
                state.tag = _tag;
                state.anchor = _anchor;
                state.kind = isMapping ? 'mapping' : 'sequence';
                state.result = _result;
                return true;
            }
            else if (!readNext) {
                throwError(state, 'missed comma between flow collection entries');
            }
            keyTag = keyNode = valueNode = null;
            isPair = isExplicitPair = false;
            if (ch === 0x3F /* ? */) {
                following = state.input.charCodeAt(state.position + 1);
                if (is_WS_OR_EOL(following)) {
                    isPair = isExplicitPair = true;
                    state.position++;
                    skipSeparationSpace(state, true, nodeIndent);
                }
            }
            _line = state.line;
            composeNode(state, nodeIndent, CONTEXT_FLOW_IN, false, true);
            keyTag = state.tag;
            keyNode = state.result;
            skipSeparationSpace(state, true, nodeIndent);
            ch = state.input.charCodeAt(state.position);
            if ((isExplicitPair || state.line === _line) && ch === 0x3A /* : */) {
                isPair = true;
                ch = state.input.charCodeAt(++state.position);
                skipSeparationSpace(state, true, nodeIndent);
                composeNode(state, nodeIndent, CONTEXT_FLOW_IN, false, true);
                valueNode = state.result;
            }
            if (isMapping) {
                storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode);
            }
            else if (isPair) {
                _result.push(storeMappingPair(state, null, overridableKeys, keyTag, keyNode, valueNode));
            }
            else {
                _result.push(keyNode);
            }
            skipSeparationSpace(state, true, nodeIndent);
            ch = state.input.charCodeAt(state.position);
            if (ch === 0x2C /* , */) {
                readNext = true;
                ch = state.input.charCodeAt(++state.position);
            }
            else {
                readNext = false;
            }
        }
        throwError(state, 'unexpected end of the stream within a flow collection');
    }
    function readBlockScalar(state, nodeIndent) {
        var captureStart, folding, chomping = CHOMPING_CLIP, didReadContent = false, detectedIndent = false, textIndent = nodeIndent, emptyLines = 0, atMoreIndented = false, tmp, ch;
        ch = state.input.charCodeAt(state.position);
        if (ch === 0x7C /* | */) {
            folding = false;
        }
        else if (ch === 0x3E /* > */) {
            folding = true;
        }
        else {
            return false;
        }
        state.kind = 'scalar';
        state.result = '';
        while (ch !== 0) {
            ch = state.input.charCodeAt(++state.position);
            if (ch === 0x2B /* + */ || ch === 0x2D /* - */) {
                if (CHOMPING_CLIP === chomping) {
                    chomping = (ch === 0x2B /* + */) ? CHOMPING_KEEP : CHOMPING_STRIP;
                }
                else {
                    throwError(state, 'repeat of a chomping mode identifier');
                }
            }
            else if ((tmp = fromDecimalCode(ch)) >= 0) {
                if (tmp === 0) {
                    throwError(state, 'bad explicit indentation width of a block scalar; it cannot be less than one');
                }
                else if (!detectedIndent) {
                    textIndent = nodeIndent + tmp - 1;
                    detectedIndent = true;
                }
                else {
                    throwError(state, 'repeat of an indentation width identifier');
                }
            }
            else {
                break;
            }
        }
        if (is_WHITE_SPACE(ch)) {
            do {
                ch = state.input.charCodeAt(++state.position);
            } while (is_WHITE_SPACE(ch));
            if (ch === 0x23 /* # */) {
                do {
                    ch = state.input.charCodeAt(++state.position);
                } while (!is_EOL(ch) && (ch !== 0));
            }
        }
        while (ch !== 0) {
            readLineBreak(state);
            state.lineIndent = 0;
            ch = state.input.charCodeAt(state.position);
            while ((!detectedIndent || state.lineIndent < textIndent) &&
                (ch === 0x20 /* Space */)) {
                state.lineIndent++;
                ch = state.input.charCodeAt(++state.position);
            }
            if (!detectedIndent && state.lineIndent > textIndent) {
                textIndent = state.lineIndent;
            }
            if (is_EOL(ch)) {
                emptyLines++;
                continue;
            }
            // End of the scalar.
            if (state.lineIndent < textIndent) {
                // Perform the chomping.
                if (chomping === CHOMPING_KEEP) {
                    state.result += common.repeat('\n', didReadContent ? 1 + emptyLines : emptyLines);
                }
                else if (chomping === CHOMPING_CLIP) {
                    if (didReadContent) { // i.e. only if the scalar is not empty.
                        state.result += '\n';
                    }
                }
                // Break this `while` cycle and go to the funciton's epilogue.
                break;
            }
            // Folded style: use fancy rules to handle line breaks.
            if (folding) {
                // Lines starting with white space characters (more-indented lines) are not folded.
                if (is_WHITE_SPACE(ch)) {
                    atMoreIndented = true;
                    // except for the first content line (cf. Example 8.1)
                    state.result += common.repeat('\n', didReadContent ? 1 + emptyLines : emptyLines);
                    // End of more-indented block.
                }
                else if (atMoreIndented) {
                    atMoreIndented = false;
                    state.result += common.repeat('\n', emptyLines + 1);
                    // Just one line break - perceive as the same line.
                }
                else if (emptyLines === 0) {
                    if (didReadContent) { // i.e. only if we have already read some scalar content.
                        state.result += ' ';
                    }
                    // Several line breaks - perceive as different lines.
                }
                else {
                    state.result += common.repeat('\n', emptyLines);
                }
                // Literal style: just add exact number of line breaks between content lines.
            }
            else {
                // Keep all line breaks except the header line break.
                state.result += common.repeat('\n', didReadContent ? 1 + emptyLines : emptyLines);
            }
            didReadContent = true;
            detectedIndent = true;
            emptyLines = 0;
            captureStart = state.position;
            while (!is_EOL(ch) && (ch !== 0)) {
                ch = state.input.charCodeAt(++state.position);
            }
            captureSegment(state, captureStart, state.position, false);
        }
        return true;
    }
    function readBlockSequence(state, nodeIndent) {
        var _line, _tag = state.tag, _anchor = state.anchor, _result = [], following, detected = false, ch;
        if (state.anchor !== null) {
            state.anchorMap[state.anchor] = _result;
        }
        ch = state.input.charCodeAt(state.position);
        while (ch !== 0) {
            if (ch !== 0x2D /* - */) {
                break;
            }
            following = state.input.charCodeAt(state.position + 1);
            if (!is_WS_OR_EOL(following)) {
                break;
            }
            detected = true;
            state.position++;
            if (skipSeparationSpace(state, true, -1)) {
                if (state.lineIndent <= nodeIndent) {
                    _result.push(null);
                    ch = state.input.charCodeAt(state.position);
                    continue;
                }
            }
            _line = state.line;
            composeNode(state, nodeIndent, CONTEXT_BLOCK_IN, false, true);
            _result.push(state.result);
            skipSeparationSpace(state, true, -1);
            ch = state.input.charCodeAt(state.position);
            if ((state.line === _line || state.lineIndent > nodeIndent) && (ch !== 0)) {
                throwError(state, 'bad indentation of a sequence entry');
            }
            else if (state.lineIndent < nodeIndent) {
                break;
            }
        }
        if (detected) {
            state.tag = _tag;
            state.anchor = _anchor;
            state.kind = 'sequence';
            state.result = _result;
            return true;
        }
        return false;
    }
    function readBlockMapping(state, nodeIndent, flowIndent) {
        var following, allowCompact, _line, _pos, _tag = state.tag, _anchor = state.anchor, _result = {}, overridableKeys = {}, keyTag = null, keyNode = null, valueNode = null, atExplicitKey = false, detected = false, ch;
        if (state.anchor !== null) {
            state.anchorMap[state.anchor] = _result;
        }
        ch = state.input.charCodeAt(state.position);
        while (ch !== 0) {
            following = state.input.charCodeAt(state.position + 1);
            _line = state.line; // Save the current line.
            _pos = state.position;
            //
            // Explicit notation case. There are two separate blocks:
            // first for the key (denoted by "?") and second for the value (denoted by ":")
            //
            if ((ch === 0x3F /* ? */ || ch === 0x3A /* : */) && is_WS_OR_EOL(following)) {
                if (ch === 0x3F /* ? */) {
                    if (atExplicitKey) {
                        storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null);
                        keyTag = keyNode = valueNode = null;
                    }
                    detected = true;
                    atExplicitKey = true;
                    allowCompact = true;
                }
                else if (atExplicitKey) {
                    // i.e. 0x3A/* : */ === character after the explicit key.
                    atExplicitKey = false;
                    allowCompact = true;
                }
                else {
                    throwError(state, 'incomplete explicit mapping pair; a key node is missed; or followed by a non-tabulated empty line');
                }
                state.position += 1;
                ch = following;
                //
                // Implicit notation case. Flow-style node as the key first, then ":", and the value.
                //
            }
            else if (composeNode(state, flowIndent, CONTEXT_FLOW_OUT, false, true)) {
                if (state.line === _line) {
                    ch = state.input.charCodeAt(state.position);
                    while (is_WHITE_SPACE(ch)) {
                        ch = state.input.charCodeAt(++state.position);
                    }
                    if (ch === 0x3A /* : */) {
                        ch = state.input.charCodeAt(++state.position);
                        if (!is_WS_OR_EOL(ch)) {
                            throwError(state, 'a whitespace character is expected after the key-value separator within a block mapping');
                        }
                        if (atExplicitKey) {
                            storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null);
                            keyTag = keyNode = valueNode = null;
                        }
                        detected = true;
                        atExplicitKey = false;
                        allowCompact = false;
                        keyTag = state.tag;
                        keyNode = state.result;
                    }
                    else if (detected) {
                        throwError(state, 'can not read an implicit mapping pair; a colon is missed');
                    }
                    else {
                        state.tag = _tag;
                        state.anchor = _anchor;
                        return true; // Keep the result of `composeNode`.
                    }
                }
                else if (detected) {
                    throwError(state, 'can not read a block mapping entry; a multiline key may not be an implicit key');
                }
                else {
                    state.tag = _tag;
                    state.anchor = _anchor;
                    return true; // Keep the result of `composeNode`.
                }
            }
            else {
                break; // Reading is done. Go to the epilogue.
            }
            //
            // Common reading code for both explicit and implicit notations.
            //
            if (state.line === _line || state.lineIndent > nodeIndent) {
                if (composeNode(state, nodeIndent, CONTEXT_BLOCK_OUT, true, allowCompact)) {
                    if (atExplicitKey) {
                        keyNode = state.result;
                    }
                    else {
                        valueNode = state.result;
                    }
                }
                if (!atExplicitKey) {
                    storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode, _line, _pos);
                    keyTag = keyNode = valueNode = null;
                }
                skipSeparationSpace(state, true, -1);
                ch = state.input.charCodeAt(state.position);
            }
            if (state.lineIndent > nodeIndent && (ch !== 0)) {
                throwError(state, 'bad indentation of a mapping entry');
            }
            else if (state.lineIndent < nodeIndent) {
                break;
            }
        }
        //
        // Epilogue.
        //
        // Special case: last mapping's node contains only the key in explicit notation.
        if (atExplicitKey) {
            storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null);
        }
        // Expose the resulting mapping.
        if (detected) {
            state.tag = _tag;
            state.anchor = _anchor;
            state.kind = 'mapping';
            state.result = _result;
        }
        return detected;
    }
    function readTagProperty(state) {
        var _position, isVerbatim = false, isNamed = false, tagHandle, tagName, ch;
        ch = state.input.charCodeAt(state.position);
        if (ch !== 0x21 /* ! */)
            return false;
        if (state.tag !== null) {
            throwError(state, 'duplication of a tag property');
        }
        ch = state.input.charCodeAt(++state.position);
        if (ch === 0x3C /* < */) {
            isVerbatim = true;
            ch = state.input.charCodeAt(++state.position);
        }
        else if (ch === 0x21 /* ! */) {
            isNamed = true;
            tagHandle = '!!';
            ch = state.input.charCodeAt(++state.position);
        }
        else {
            tagHandle = '!';
        }
        _position = state.position;
        if (isVerbatim) {
            do {
                ch = state.input.charCodeAt(++state.position);
            } while (ch !== 0 && ch !== 0x3E /* > */);
            if (state.position < state.length) {
                tagName = state.input.slice(_position, state.position);
                ch = state.input.charCodeAt(++state.position);
            }
            else {
                throwError(state, 'unexpected end of the stream within a verbatim tag');
            }
        }
        else {
            while (ch !== 0 && !is_WS_OR_EOL(ch)) {
                if (ch === 0x21 /* ! */) {
                    if (!isNamed) {
                        tagHandle = state.input.slice(_position - 1, state.position + 1);
                        if (!PATTERN_TAG_HANDLE.test(tagHandle)) {
                            throwError(state, 'named tag handle cannot contain such characters');
                        }
                        isNamed = true;
                        _position = state.position + 1;
                    }
                    else {
                        throwError(state, 'tag suffix cannot contain exclamation marks');
                    }
                }
                ch = state.input.charCodeAt(++state.position);
            }
            tagName = state.input.slice(_position, state.position);
            if (PATTERN_FLOW_INDICATORS.test(tagName)) {
                throwError(state, 'tag suffix cannot contain flow indicator characters');
            }
        }
        if (tagName && !PATTERN_TAG_URI.test(tagName)) {
            throwError(state, 'tag name cannot contain such characters: ' + tagName);
        }
        if (isVerbatim) {
            state.tag = tagName;
        }
        else if (_hasOwnProperty.call(state.tagMap, tagHandle)) {
            state.tag = state.tagMap[tagHandle] + tagName;
        }
        else if (tagHandle === '!') {
            state.tag = '!' + tagName;
        }
        else if (tagHandle === '!!') {
            state.tag = 'tag:yaml.org,2002:' + tagName;
        }
        else {
            throwError(state, 'undeclared tag handle "' + tagHandle + '"');
        }
        return true;
    }
    function readAnchorProperty(state) {
        var _position, ch;
        ch = state.input.charCodeAt(state.position);
        if (ch !== 0x26 /* & */)
            return false;
        if (state.anchor !== null) {
            throwError(state, 'duplication of an anchor property');
        }
        ch = state.input.charCodeAt(++state.position);
        _position = state.position;
        while (ch !== 0 && !is_WS_OR_EOL(ch) && !is_FLOW_INDICATOR(ch)) {
            ch = state.input.charCodeAt(++state.position);
        }
        if (state.position === _position) {
            throwError(state, 'name of an anchor node must contain at least one character');
        }
        state.anchor = state.input.slice(_position, state.position);
        return true;
    }
    function readAlias(state) {
        var _position, alias, ch;
        ch = state.input.charCodeAt(state.position);
        if (ch !== 0x2A /* * */)
            return false;
        ch = state.input.charCodeAt(++state.position);
        _position = state.position;
        while (ch !== 0 && !is_WS_OR_EOL(ch) && !is_FLOW_INDICATOR(ch)) {
            ch = state.input.charCodeAt(++state.position);
        }
        if (state.position === _position) {
            throwError(state, 'name of an alias node must contain at least one character');
        }
        alias = state.input.slice(_position, state.position);
        if (!state.anchorMap.hasOwnProperty(alias)) {
            throwError(state, 'unidentified alias "' + alias + '"');
        }
        state.result = state.anchorMap[alias];
        skipSeparationSpace(state, true, -1);
        return true;
    }
    function composeNode(state, parentIndent, nodeContext, allowToSeek, allowCompact) {
        var allowBlockStyles, allowBlockScalars, allowBlockCollections, indentStatus = 1, // 1: this>parent, 0: this=parent, -1: this<parent
        atNewLine = false, hasContent = false, typeIndex, typeQuantity, type, flowIndent, blockIndent;
        if (state.listener !== null) {
            state.listener('open', state);
        }
        state.tag = null;
        state.anchor = null;
        state.kind = null;
        state.result = null;
        allowBlockStyles = allowBlockScalars = allowBlockCollections =
            CONTEXT_BLOCK_OUT === nodeContext ||
                CONTEXT_BLOCK_IN === nodeContext;
        if (allowToSeek) {
            if (skipSeparationSpace(state, true, -1)) {
                atNewLine = true;
                if (state.lineIndent > parentIndent) {
                    indentStatus = 1;
                }
                else if (state.lineIndent === parentIndent) {
                    indentStatus = 0;
                }
                else if (state.lineIndent < parentIndent) {
                    indentStatus = -1;
                }
            }
        }
        if (indentStatus === 1) {
            while (readTagProperty(state) || readAnchorProperty(state)) {
                if (skipSeparationSpace(state, true, -1)) {
                    atNewLine = true;
                    allowBlockCollections = allowBlockStyles;
                    if (state.lineIndent > parentIndent) {
                        indentStatus = 1;
                    }
                    else if (state.lineIndent === parentIndent) {
                        indentStatus = 0;
                    }
                    else if (state.lineIndent < parentIndent) {
                        indentStatus = -1;
                    }
                }
                else {
                    allowBlockCollections = false;
                }
            }
        }
        if (allowBlockCollections) {
            allowBlockCollections = atNewLine || allowCompact;
        }
        if (indentStatus === 1 || CONTEXT_BLOCK_OUT === nodeContext) {
            if (CONTEXT_FLOW_IN === nodeContext || CONTEXT_FLOW_OUT === nodeContext) {
                flowIndent = parentIndent;
            }
            else {
                flowIndent = parentIndent + 1;
            }
            blockIndent = state.position - state.lineStart;
            if (indentStatus === 1) {
                if (allowBlockCollections &&
                    (readBlockSequence(state, blockIndent) ||
                        readBlockMapping(state, blockIndent, flowIndent)) ||
                    readFlowCollection(state, flowIndent)) {
                    hasContent = true;
                }
                else {
                    if ((allowBlockScalars && readBlockScalar(state, flowIndent)) ||
                        readSingleQuotedScalar(state, flowIndent) ||
                        readDoubleQuotedScalar(state, flowIndent)) {
                        hasContent = true;
                    }
                    else if (readAlias(state)) {
                        hasContent = true;
                        if (state.tag !== null || state.anchor !== null) {
                            throwError(state, 'alias node should not have any properties');
                        }
                    }
                    else if (readPlainScalar(state, flowIndent, CONTEXT_FLOW_IN === nodeContext)) {
                        hasContent = true;
                        if (state.tag === null) {
                            state.tag = '?';
                        }
                    }
                    if (state.anchor !== null) {
                        state.anchorMap[state.anchor] = state.result;
                    }
                }
            }
            else if (indentStatus === 0) {
                // Special case: block sequences are allowed to have same indentation level as the parent.
                // http://www.yaml.org/spec/1.2/spec.html#id2799784
                hasContent = allowBlockCollections && readBlockSequence(state, blockIndent);
            }
        }
        if (state.tag !== null && state.tag !== '!') {
            if (state.tag === '?') {
                for (typeIndex = 0, typeQuantity = state.implicitTypes.length; typeIndex < typeQuantity; typeIndex += 1) {
                    type = state.implicitTypes[typeIndex];
                    // Implicit resolving is not allowed for non-scalar types, and '?'
                    // non-specific tag is only assigned to plain scalars. So, it isn't
                    // needed to check for 'kind' conformity.
                    if (type.resolve(state.result)) { // `state.result` updated in resolver if matched
                        state.result = type.construct(state.result);
                        state.tag = type.tag;
                        if (state.anchor !== null) {
                            state.anchorMap[state.anchor] = state.result;
                        }
                        break;
                    }
                }
            }
            else if (_hasOwnProperty.call(state.typeMap[state.kind || 'fallback'], state.tag)) {
                type = state.typeMap[state.kind || 'fallback'][state.tag];
                if (state.result !== null && type.kind !== state.kind) {
                    throwError(state, 'unacceptable node kind for !<' + state.tag + '> tag; it should be "' + type.kind + '", not "' + state.kind + '"');
                }
                if (!type.resolve(state.result)) { // `state.result` updated in resolver if matched
                    throwError(state, 'cannot resolve a node with !<' + state.tag + '> explicit tag');
                }
                else {
                    state.result = type.construct(state.result);
                    if (state.anchor !== null) {
                        state.anchorMap[state.anchor] = state.result;
                    }
                }
            }
            else {
                throwError(state, 'unknown tag !<' + state.tag + '>');
            }
        }
        if (state.listener !== null) {
            state.listener('close', state);
        }
        return state.tag !== null || state.anchor !== null || hasContent;
    }
    function readDocument(state) {
        var documentStart = state.position, _position, directiveName, directiveArgs, hasDirectives = false, ch;
        state.version = null;
        state.checkLineBreaks = state.legacy;
        state.tagMap = {};
        state.anchorMap = {};
        while ((ch = state.input.charCodeAt(state.position)) !== 0) {
            skipSeparationSpace(state, true, -1);
            ch = state.input.charCodeAt(state.position);
            if (state.lineIndent > 0 || ch !== 0x25 /* % */) {
                break;
            }
            hasDirectives = true;
            ch = state.input.charCodeAt(++state.position);
            _position = state.position;
            while (ch !== 0 && !is_WS_OR_EOL(ch)) {
                ch = state.input.charCodeAt(++state.position);
            }
            directiveName = state.input.slice(_position, state.position);
            directiveArgs = [];
            if (directiveName.length < 1) {
                throwError(state, 'directive name must not be less than one character in length');
            }
            while (ch !== 0) {
                while (is_WHITE_SPACE(ch)) {
                    ch = state.input.charCodeAt(++state.position);
                }
                if (ch === 0x23 /* # */) {
                    do {
                        ch = state.input.charCodeAt(++state.position);
                    } while (ch !== 0 && !is_EOL(ch));
                    break;
                }
                if (is_EOL(ch))
                    break;
                _position = state.position;
                while (ch !== 0 && !is_WS_OR_EOL(ch)) {
                    ch = state.input.charCodeAt(++state.position);
                }
                directiveArgs.push(state.input.slice(_position, state.position));
            }
            if (ch !== 0)
                readLineBreak(state);
            if (_hasOwnProperty.call(directiveHandlers, directiveName)) {
                directiveHandlers[directiveName](state, directiveName, directiveArgs);
            }
            else {
                throwWarning(state, 'unknown document directive "' + directiveName + '"');
            }
        }
        skipSeparationSpace(state, true, -1);
        if (state.lineIndent === 0 &&
            state.input.charCodeAt(state.position) === 0x2D /* - */ &&
            state.input.charCodeAt(state.position + 1) === 0x2D /* - */ &&
            state.input.charCodeAt(state.position + 2) === 0x2D /* - */) {
            state.position += 3;
            skipSeparationSpace(state, true, -1);
        }
        else if (hasDirectives) {
            throwError(state, 'directives end mark is expected');
        }
        composeNode(state, state.lineIndent - 1, CONTEXT_BLOCK_OUT, false, true);
        skipSeparationSpace(state, true, -1);
        if (state.checkLineBreaks &&
            PATTERN_NON_ASCII_LINE_BREAKS.test(state.input.slice(documentStart, state.position))) {
            throwWarning(state, 'non-ASCII line breaks are interpreted as content');
        }
        state.documents.push(state.result);
        if (state.position === state.lineStart && testDocumentSeparator(state)) {
            if (state.input.charCodeAt(state.position) === 0x2E /* . */) {
                state.position += 3;
                skipSeparationSpace(state, true, -1);
            }
            return;
        }
        if (state.position < (state.length - 1)) {
            throwError(state, 'end of the stream or a document separator is expected');
        }
        else {
            return;
        }
    }
    function loadDocuments(input, options) {
        input = String(input);
        options = options || {};
        if (input.length !== 0) {
            // Add tailing `\n` if not exists
            if (input.charCodeAt(input.length - 1) !== 0x0A /* LF */ &&
                input.charCodeAt(input.length - 1) !== 0x0D /* CR */) {
                input += '\n';
            }
            // Strip BOM
            if (input.charCodeAt(0) === 0xFEFF) {
                input = input.slice(1);
            }
        }
        var state = new State(input, options);
        // Use 0 as string terminator. That significantly simplifies bounds check.
        state.input += '\0';
        while (state.input.charCodeAt(state.position) === 0x20 /* Space */) {
            state.lineIndent += 1;
            state.position += 1;
        }
        while (state.position < (state.length - 1)) {
            readDocument(state);
        }
        return state.documents;
    }
    function loadAll(input, iterator, options) {
        var documents = loadDocuments(input, options), index, length;
        if (typeof iterator !== 'function') {
            return documents;
        }
        for (index = 0, length = documents.length; index < length; index += 1) {
            iterator(documents[index]);
        }
    }
    function load(input, options) {
        var documents = loadDocuments(input, options);
        if (documents.length === 0) {
            /*eslint-disable no-undefined*/
            return undefined;
        }
        else if (documents.length === 1) {
            return documents[0];
        }
        throw new YAMLException('expected a single document in the stream, but found more');
    }
    function safeLoadAll(input, output, options) {
        if (typeof output === 'function') {
            loadAll(input, output, common.extend({ schema: DEFAULT_SAFE_SCHEMA }, options));
        }
        else {
            return loadAll(input, common.extend({ schema: DEFAULT_SAFE_SCHEMA }, options));
        }
    }
    function safeLoad(input, options) {
        return load(input, common.extend({ schema: DEFAULT_SAFE_SCHEMA }, options));
    }
    module.exports.loadAll = loadAll;
    module.exports.load = load;
    module.exports.safeLoadAll = safeLoadAll;
    module.exports.safeLoad = safeLoad;
},
/* js-yaml/lib/js-yaml/common.js */ function _(require, module, exports) {
    function isNothing(subject) {
        return (typeof subject === 'undefined') || (subject === null);
    }
    function isObject(subject) {
        return (typeof subject === 'object') && (subject !== null);
    }
    function toArray(sequence) {
        if (Array.isArray(sequence))
            return sequence;
        else if (isNothing(sequence))
            return [];
        return [sequence];
    }
    function extend(target, source) {
        var index, length, key, sourceKeys;
        if (source) {
            sourceKeys = Object.keys(source);
            for (index = 0, length = sourceKeys.length; index < length; index += 1) {
                key = sourceKeys[index];
                target[key] = source[key];
            }
        }
        return target;
    }
    function repeat(string, count) {
        var result = '', cycle;
        for (cycle = 0; cycle < count; cycle += 1) {
            result += string;
        }
        return result;
    }
    function isNegativeZero(number) {
        return (number === 0) && (Number.NEGATIVE_INFINITY === 1 / number);
    }
    module.exports.isNothing = isNothing;
    module.exports.isObject = isObject;
    module.exports.toArray = toArray;
    module.exports.repeat = repeat;
    module.exports.isNegativeZero = isNegativeZero;
    module.exports.extend = extend;
},
/* js-yaml/lib/js-yaml/exception.js */ function _(require, module, exports) {
    function YAMLException(reason, mark) {
        // Super constructor
        Error.call(this);
        this.name = 'YAMLException';
        this.reason = reason;
        this.mark = mark;
        this.message = (this.reason || '(unknown reason)') + (this.mark ? ' ' + this.mark.toString() : '');
        // Include stack trace in error object
        if (Error.captureStackTrace) {
            // Chrome and NodeJS
            Error.captureStackTrace(this, this.constructor);
        }
        else {
            // FF, IE 10+ and Safari 6+. Fallback for others
            this.stack = (new Error()).stack || '';
        }
    }
    // Inherit from Error
    YAMLException.prototype = Object.create(Error.prototype);
    YAMLException.prototype.constructor = YAMLException;
    YAMLException.prototype.toString = function toString(compact) {
        var result = this.name + ': ';
        result += this.reason || '(unknown reason)';
        if (!compact && this.mark) {
            result += ' ' + this.mark.toString();
        }
        return result;
    };
    module.exports = YAMLException;
},
/* js-yaml/lib/js-yaml/mark.js */ function _(require, module, exports) {
    var common = require(538) /* ./common */;
    function Mark(name, buffer, position, line, column) {
        this.name = name;
        this.buffer = buffer;
        this.position = position;
        this.line = line;
        this.column = column;
    }
    Mark.prototype.getSnippet = function getSnippet(indent, maxLength) {
        var head, start, tail, end, snippet;
        if (!this.buffer)
            return null;
        indent = indent || 4;
        maxLength = maxLength || 75;
        head = '';
        start = this.position;
        while (start > 0 && '\x00\r\n\x85\u2028\u2029'.indexOf(this.buffer.charAt(start - 1)) === -1) {
            start -= 1;
            if (this.position - start > (maxLength / 2 - 1)) {
                head = ' ... ';
                start += 5;
                break;
            }
        }
        tail = '';
        end = this.position;
        while (end < this.buffer.length && '\x00\r\n\x85\u2028\u2029'.indexOf(this.buffer.charAt(end)) === -1) {
            end += 1;
            if (end - this.position > (maxLength / 2 - 1)) {
                tail = ' ... ';
                end -= 5;
                break;
            }
        }
        snippet = this.buffer.slice(start, end);
        return common.repeat(' ', indent) + head + snippet + tail + '\n' +
            common.repeat(' ', indent + this.position - start + head.length) + '^';
    };
    Mark.prototype.toString = function toString(compact) {
        var snippet, where = '';
        if (this.name) {
            where += 'in "' + this.name + '" ';
        }
        where += 'at line ' + (this.line + 1) + ', column ' + (this.column + 1);
        if (!compact) {
            snippet = this.getSnippet();
            if (snippet) {
                where += ':\n' + snippet;
            }
        }
        return where;
    };
    module.exports = Mark;
},
/* js-yaml/lib/js-yaml/schema/default_safe.js */ function _(require, module, exports) {
    var Schema = require(542) /* ../schema */;
    module.exports = new Schema({
        include: [
            require(544) /* ./core */
        ],
        implicit: [
            require(554) /* ../type/timestamp */,
            require(555) /* ../type/merge */
        ],
        explicit: [
            require(556) /* ../type/binary */,
            require(557) /* ../type/omap */,
            require(558) /* ../type/pairs */,
            require(559) /* ../type/set */
        ]
    });
},
/* js-yaml/lib/js-yaml/schema.js */ function _(require, module, exports) {
    /*eslint-disable max-len*/
    var common = require(538) /* ./common */;
    var YAMLException = require(539) /* ./exception */;
    var Type = require(543) /* ./type */;
    function compileList(schema, name, result) {
        var exclude = [];
        schema.include.forEach(function (includedSchema) {
            result = compileList(includedSchema, name, result);
        });
        schema[name].forEach(function (currentType) {
            result.forEach(function (previousType, previousIndex) {
                if (previousType.tag === currentType.tag && previousType.kind === currentType.kind) {
                    exclude.push(previousIndex);
                }
            });
            result.push(currentType);
        });
        return result.filter(function (type, index) {
            return exclude.indexOf(index) === -1;
        });
    }
    function compileMap( /* lists... */) {
        var result = {
            scalar: {},
            sequence: {},
            mapping: {},
            fallback: {}
        }, index, length;
        function collectType(type) {
            result[type.kind][type.tag] = result['fallback'][type.tag] = type;
        }
        for (index = 0, length = arguments.length; index < length; index += 1) {
            arguments[index].forEach(collectType);
        }
        return result;
    }
    function Schema(definition) {
        this.include = definition.include || [];
        this.implicit = definition.implicit || [];
        this.explicit = definition.explicit || [];
        this.implicit.forEach(function (type) {
            if (type.loadKind && type.loadKind !== 'scalar') {
                throw new YAMLException('There is a non-scalar type in the implicit list of a schema. Implicit resolving of such types is not supported.');
            }
        });
        this.compiledImplicit = compileList(this, 'implicit', []);
        this.compiledExplicit = compileList(this, 'explicit', []);
        this.compiledTypeMap = compileMap(this.compiledImplicit, this.compiledExplicit);
    }
    Schema.DEFAULT = null;
    Schema.create = function createSchema() {
        var schemas, types;
        switch (arguments.length) {
            case 1:
                schemas = Schema.DEFAULT;
                types = arguments[0];
                break;
            case 2:
                schemas = arguments[0];
                types = arguments[1];
                break;
            default:
                throw new YAMLException('Wrong number of arguments for Schema.create function');
        }
        schemas = common.toArray(schemas);
        types = common.toArray(types);
        if (!schemas.every(function (schema) { return schema instanceof Schema; })) {
            throw new YAMLException('Specified list of super schemas (or a single Schema object) contains a non-Schema object.');
        }
        if (!types.every(function (type) { return type instanceof Type; })) {
            throw new YAMLException('Specified list of YAML types (or a single Type object) contains a non-Type object.');
        }
        return new Schema({
            include: schemas,
            explicit: types
        });
    };
    module.exports = Schema;
},
/* js-yaml/lib/js-yaml/type.js */ function _(require, module, exports) {
    var YAMLException = require(539) /* ./exception */;
    var TYPE_CONSTRUCTOR_OPTIONS = [
        'kind',
        'resolve',
        'construct',
        'instanceOf',
        'predicate',
        'represent',
        'defaultStyle',
        'styleAliases'
    ];
    var YAML_NODE_KINDS = [
        'scalar',
        'sequence',
        'mapping'
    ];
    function compileStyleAliases(map) {
        var result = {};
        if (map !== null) {
            Object.keys(map).forEach(function (style) {
                map[style].forEach(function (alias) {
                    result[String(alias)] = style;
                });
            });
        }
        return result;
    }
    function Type(tag, options) {
        options = options || {};
        Object.keys(options).forEach(function (name) {
            if (TYPE_CONSTRUCTOR_OPTIONS.indexOf(name) === -1) {
                throw new YAMLException('Unknown option "' + name + '" is met in definition of "' + tag + '" YAML type.');
            }
        });
        // TODO: Add tag format check.
        this.tag = tag;
        this.kind = options['kind'] || null;
        this.resolve = options['resolve'] || function () { return true; };
        this.construct = options['construct'] || function (data) { return data; };
        this.instanceOf = options['instanceOf'] || null;
        this.predicate = options['predicate'] || null;
        this.represent = options['represent'] || null;
        this.defaultStyle = options['defaultStyle'] || null;
        this.styleAliases = compileStyleAliases(options['styleAliases'] || null);
        if (YAML_NODE_KINDS.indexOf(this.kind) === -1) {
            throw new YAMLException('Unknown kind "' + this.kind + '" is specified for "' + tag + '" YAML type.');
        }
    }
    module.exports = Type;
},
/* js-yaml/lib/js-yaml/schema/core.js */ function _(require, module, exports) {
    var Schema = require(542) /* ../schema */;
    module.exports = new Schema({
        include: [
            require(545) /* ./json */
        ]
    });
},
/* js-yaml/lib/js-yaml/schema/json.js */ function _(require, module, exports) {
    var Schema = require(542) /* ../schema */;
    module.exports = new Schema({
        include: [
            require(546) /* ./failsafe */
        ],
        implicit: [
            require(550) /* ../type/null */,
            require(551) /* ../type/bool */,
            require(552) /* ../type/int */,
            require(553) /* ../type/float */
        ]
    });
},
/* js-yaml/lib/js-yaml/schema/failsafe.js */ function _(require, module, exports) {
    var Schema = require(542) /* ../schema */;
    module.exports = new Schema({
        explicit: [
            require(547) /* ../type/str */,
            require(548) /* ../type/seq */,
            require(549) /* ../type/map */
        ]
    });
},
/* js-yaml/lib/js-yaml/type/str.js */ function _(require, module, exports) {
    var Type = require(543) /* ../type */;
    module.exports = new Type('tag:yaml.org,2002:str', {
        kind: 'scalar',
        construct: function (data) { return data !== null ? data : ''; }
    });
},
/* js-yaml/lib/js-yaml/type/seq.js */ function _(require, module, exports) {
    var Type = require(543) /* ../type */;
    module.exports = new Type('tag:yaml.org,2002:seq', {
        kind: 'sequence',
        construct: function (data) { return data !== null ? data : []; }
    });
},
/* js-yaml/lib/js-yaml/type/map.js */ function _(require, module, exports) {
    var Type = require(543) /* ../type */;
    module.exports = new Type('tag:yaml.org,2002:map', {
        kind: 'mapping',
        construct: function (data) { return data !== null ? data : {}; }
    });
},
/* js-yaml/lib/js-yaml/type/null.js */ function _(require, module, exports) {
    var Type = require(543) /* ../type */;
    function resolveYamlNull(data) {
        if (data === null)
            return true;
        var max = data.length;
        return (max === 1 && data === '~') ||
            (max === 4 && (data === 'null' || data === 'Null' || data === 'NULL'));
    }
    function constructYamlNull() {
        return null;
    }
    function isNull(object) {
        return object === null;
    }
    module.exports = new Type('tag:yaml.org,2002:null', {
        kind: 'scalar',
        resolve: resolveYamlNull,
        construct: constructYamlNull,
        predicate: isNull,
        represent: {
            canonical: function () { return '~'; },
            lowercase: function () { return 'null'; },
            uppercase: function () { return 'NULL'; },
            camelcase: function () { return 'Null'; }
        },
        defaultStyle: 'lowercase'
    });
},
/* js-yaml/lib/js-yaml/type/bool.js */ function _(require, module, exports) {
    var Type = require(543) /* ../type */;
    function resolveYamlBoolean(data) {
        if (data === null)
            return false;
        var max = data.length;
        return (max === 4 && (data === 'true' || data === 'True' || data === 'TRUE')) ||
            (max === 5 && (data === 'false' || data === 'False' || data === 'FALSE'));
    }
    function constructYamlBoolean(data) {
        return data === 'true' ||
            data === 'True' ||
            data === 'TRUE';
    }
    function isBoolean(object) {
        return Object.prototype.toString.call(object) === '[object Boolean]';
    }
    module.exports = new Type('tag:yaml.org,2002:bool', {
        kind: 'scalar',
        resolve: resolveYamlBoolean,
        construct: constructYamlBoolean,
        predicate: isBoolean,
        represent: {
            lowercase: function (object) { return object ? 'true' : 'false'; },
            uppercase: function (object) { return object ? 'TRUE' : 'FALSE'; },
            camelcase: function (object) { return object ? 'True' : 'False'; }
        },
        defaultStyle: 'lowercase'
    });
},
/* js-yaml/lib/js-yaml/type/int.js */ function _(require, module, exports) {
    var common = require(538) /* ../common */;
    var Type = require(543) /* ../type */;
    function isHexCode(c) {
        return ((0x30 /* 0 */ <= c) && (c <= 0x39 /* 9 */)) ||
            ((0x41 /* A */ <= c) && (c <= 0x46 /* F */)) ||
            ((0x61 /* a */ <= c) && (c <= 0x66 /* f */));
    }
    function isOctCode(c) {
        return ((0x30 /* 0 */ <= c) && (c <= 0x37 /* 7 */));
    }
    function isDecCode(c) {
        return ((0x30 /* 0 */ <= c) && (c <= 0x39 /* 9 */));
    }
    function resolveYamlInteger(data) {
        if (data === null)
            return false;
        var max = data.length, index = 0, hasDigits = false, ch;
        if (!max)
            return false;
        ch = data[index];
        // sign
        if (ch === '-' || ch === '+') {
            ch = data[++index];
        }
        if (ch === '0') {
            // 0
            if (index + 1 === max)
                return true;
            ch = data[++index];
            // base 2, base 8, base 16
            if (ch === 'b') {
                // base 2
                index++;
                for (; index < max; index++) {
                    ch = data[index];
                    if (ch === '_')
                        continue;
                    if (ch !== '0' && ch !== '1')
                        return false;
                    hasDigits = true;
                }
                return hasDigits && ch !== '_';
            }
            if (ch === 'x') {
                // base 16
                index++;
                for (; index < max; index++) {
                    ch = data[index];
                    if (ch === '_')
                        continue;
                    if (!isHexCode(data.charCodeAt(index)))
                        return false;
                    hasDigits = true;
                }
                return hasDigits && ch !== '_';
            }
            // base 8
            for (; index < max; index++) {
                ch = data[index];
                if (ch === '_')
                    continue;
                if (!isOctCode(data.charCodeAt(index)))
                    return false;
                hasDigits = true;
            }
            return hasDigits && ch !== '_';
        }
        // base 10 (except 0) or base 60
        // value should not start with `_`;
        if (ch === '_')
            return false;
        for (; index < max; index++) {
            ch = data[index];
            if (ch === '_')
                continue;
            if (ch === ':')
                break;
            if (!isDecCode(data.charCodeAt(index))) {
                return false;
            }
            hasDigits = true;
        }
        // Should have digits and should not end with `_`
        if (!hasDigits || ch === '_')
            return false;
        // if !base60 - done;
        if (ch !== ':')
            return true;
        // base60 almost not used, no needs to optimize
        return /^(:[0-5]?[0-9])+$/.test(data.slice(index));
    }
    function constructYamlInteger(data) {
        var value = data, sign = 1, ch, base, digits = [];
        if (value.indexOf('_') !== -1) {
            value = value.replace(/_/g, '');
        }
        ch = value[0];
        if (ch === '-' || ch === '+') {
            if (ch === '-')
                sign = -1;
            value = value.slice(1);
            ch = value[0];
        }
        if (value === '0')
            return 0;
        if (ch === '0') {
            if (value[1] === 'b')
                return sign * parseInt(value.slice(2), 2);
            if (value[1] === 'x')
                return sign * parseInt(value, 16);
            return sign * parseInt(value, 8);
        }
        if (value.indexOf(':') !== -1) {
            value.split(':').forEach(function (v) {
                digits.unshift(parseInt(v, 10));
            });
            value = 0;
            base = 1;
            digits.forEach(function (d) {
                value += (d * base);
                base *= 60;
            });
            return sign * value;
        }
        return sign * parseInt(value, 10);
    }
    function isInteger(object) {
        return (Object.prototype.toString.call(object)) === '[object Number]' &&
            (object % 1 === 0 && !common.isNegativeZero(object));
    }
    module.exports = new Type('tag:yaml.org,2002:int', {
        kind: 'scalar',
        resolve: resolveYamlInteger,
        construct: constructYamlInteger,
        predicate: isInteger,
        represent: {
            binary: function (obj) { return obj >= 0 ? '0b' + obj.toString(2) : '-0b' + obj.toString(2).slice(1); },
            octal: function (obj) { return obj >= 0 ? '0' + obj.toString(8) : '-0' + obj.toString(8).slice(1); },
            decimal: function (obj) { return obj.toString(10); },
            /* eslint-disable max-len */
            hexadecimal: function (obj) { return obj >= 0 ? '0x' + obj.toString(16).toUpperCase() : '-0x' + obj.toString(16).toUpperCase().slice(1); }
        },
        defaultStyle: 'decimal',
        styleAliases: {
            binary: [2, 'bin'],
            octal: [8, 'oct'],
            decimal: [10, 'dec'],
            hexadecimal: [16, 'hex']
        }
    });
},
/* js-yaml/lib/js-yaml/type/float.js */ function _(require, module, exports) {
    var common = require(538) /* ../common */;
    var Type = require(543) /* ../type */;
    var YAML_FLOAT_PATTERN = new RegExp(
    // 2.5e4, 2.5 and integers
    '^(?:[-+]?(?:0|[1-9][0-9_]*)(?:\\.[0-9_]*)?(?:[eE][-+]?[0-9]+)?' +
        // .2e4, .2
        // special case, seems not from spec
        '|\\.[0-9_]+(?:[eE][-+]?[0-9]+)?' +
        // 20:59
        '|[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+\\.[0-9_]*' +
        // .inf
        '|[-+]?\\.(?:inf|Inf|INF)' +
        // .nan
        '|\\.(?:nan|NaN|NAN))$');
    function resolveYamlFloat(data) {
        if (data === null)
            return false;
        if (!YAML_FLOAT_PATTERN.test(data) ||
            // Quick hack to not allow integers end with `_`
            // Probably should update regexp & check speed
            data[data.length - 1] === '_') {
            return false;
        }
        return true;
    }
    function constructYamlFloat(data) {
        var value, sign, base, digits;
        value = data.replace(/_/g, '').toLowerCase();
        sign = value[0] === '-' ? -1 : 1;
        digits = [];
        if ('+-'.indexOf(value[0]) >= 0) {
            value = value.slice(1);
        }
        if (value === '.inf') {
            return (sign === 1) ? Number.POSITIVE_INFINITY : Number.NEGATIVE_INFINITY;
        }
        else if (value === '.nan') {
            return NaN;
        }
        else if (value.indexOf(':') >= 0) {
            value.split(':').forEach(function (v) {
                digits.unshift(parseFloat(v, 10));
            });
            value = 0.0;
            base = 1;
            digits.forEach(function (d) {
                value += d * base;
                base *= 60;
            });
            return sign * value;
        }
        return sign * parseFloat(value, 10);
    }
    var SCIENTIFIC_WITHOUT_DOT = /^[-+]?[0-9]+e/;
    function representYamlFloat(object, style) {
        var res;
        if (isNaN(object)) {
            switch (style) {
                case 'lowercase': return '.nan';
                case 'uppercase': return '.NAN';
                case 'camelcase': return '.NaN';
            }
        }
        else if (Number.POSITIVE_INFINITY === object) {
            switch (style) {
                case 'lowercase': return '.inf';
                case 'uppercase': return '.INF';
                case 'camelcase': return '.Inf';
            }
        }
        else if (Number.NEGATIVE_INFINITY === object) {
            switch (style) {
                case 'lowercase': return '-.inf';
                case 'uppercase': return '-.INF';
                case 'camelcase': return '-.Inf';
            }
        }
        else if (common.isNegativeZero(object)) {
            return '-0.0';
        }
        res = object.toString(10);
        // JS stringifier can build scientific format without dots: 5e-100,
        // while YAML requres dot: 5.e-100. Fix it with simple hack
        return SCIENTIFIC_WITHOUT_DOT.test(res) ? res.replace('e', '.e') : res;
    }
    function isFloat(object) {
        return (Object.prototype.toString.call(object) === '[object Number]') &&
            (object % 1 !== 0 || common.isNegativeZero(object));
    }
    module.exports = new Type('tag:yaml.org,2002:float', {
        kind: 'scalar',
        resolve: resolveYamlFloat,
        construct: constructYamlFloat,
        predicate: isFloat,
        represent: representYamlFloat,
        defaultStyle: 'lowercase'
    });
},
/* js-yaml/lib/js-yaml/type/timestamp.js */ function _(require, module, exports) {
    var Type = require(543) /* ../type */;
    var YAML_DATE_REGEXP = new RegExp('^([0-9][0-9][0-9][0-9])' + // [1] year
        '-([0-9][0-9])' + // [2] month
        '-([0-9][0-9])$'); // [3] day
    var YAML_TIMESTAMP_REGEXP = new RegExp('^([0-9][0-9][0-9][0-9])' + // [1] year
        '-([0-9][0-9]?)' + // [2] month
        '-([0-9][0-9]?)' + // [3] day
        '(?:[Tt]|[ \\t]+)' + // ...
        '([0-9][0-9]?)' + // [4] hour
        ':([0-9][0-9])' + // [5] minute
        ':([0-9][0-9])' + // [6] second
        '(?:\\.([0-9]*))?' + // [7] fraction
        '(?:[ \\t]*(Z|([-+])([0-9][0-9]?)' + // [8] tz [9] tz_sign [10] tz_hour
        '(?::([0-9][0-9]))?))?$'); // [11] tz_minute
    function resolveYamlTimestamp(data) {
        if (data === null)
            return false;
        if (YAML_DATE_REGEXP.exec(data) !== null)
            return true;
        if (YAML_TIMESTAMP_REGEXP.exec(data) !== null)
            return true;
        return false;
    }
    function constructYamlTimestamp(data) {
        var match, year, month, day, hour, minute, second, fraction = 0, delta = null, tz_hour, tz_minute, date;
        match = YAML_DATE_REGEXP.exec(data);
        if (match === null)
            match = YAML_TIMESTAMP_REGEXP.exec(data);
        if (match === null)
            throw new Error('Date resolve error');
        // match: [1] year [2] month [3] day
        year = +(match[1]);
        month = +(match[2]) - 1; // JS month starts with 0
        day = +(match[3]);
        if (!match[4]) { // no hour
            return new Date(Date.UTC(year, month, day));
        }
        // match: [4] hour [5] minute [6] second [7] fraction
        hour = +(match[4]);
        minute = +(match[5]);
        second = +(match[6]);
        if (match[7]) {
            fraction = match[7].slice(0, 3);
            while (fraction.length < 3) { // milli-seconds
                fraction += '0';
            }
            fraction = +fraction;
        }
        // match: [8] tz [9] tz_sign [10] tz_hour [11] tz_minute
        if (match[9]) {
            tz_hour = +(match[10]);
            tz_minute = +(match[11] || 0);
            delta = (tz_hour * 60 + tz_minute) * 60000; // delta in mili-seconds
            if (match[9] === '-')
                delta = -delta;
        }
        date = new Date(Date.UTC(year, month, day, hour, minute, second, fraction));
        if (delta)
            date.setTime(date.getTime() - delta);
        return date;
    }
    function representYamlTimestamp(object /*, style*/) {
        return object.toISOString();
    }
    module.exports = new Type('tag:yaml.org,2002:timestamp', {
        kind: 'scalar',
        resolve: resolveYamlTimestamp,
        construct: constructYamlTimestamp,
        instanceOf: Date,
        represent: representYamlTimestamp
    });
},
/* js-yaml/lib/js-yaml/type/merge.js */ function _(require, module, exports) {
    var Type = require(543) /* ../type */;
    function resolveYamlMerge(data) {
        return data === '<<' || data === null;
    }
    module.exports = new Type('tag:yaml.org,2002:merge', {
        kind: 'scalar',
        resolve: resolveYamlMerge
    });
},
/* js-yaml/lib/js-yaml/type/binary.js */ function _(require, module, exports) {
    /*eslint-disable no-bitwise*/
    var NodeBuffer;
    try {
        // A trick for browserified version, to not include `Buffer` shim
        var _require = require;
        NodeBuffer = _require('buffer').Buffer;
    }
    catch (__) { }
    var Type = require(543) /* ../type */;
    // [ 64, 65, 66 ] -> [ padding, CR, LF ]
    var BASE64_MAP = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=\n\r';
    function resolveYamlBinary(data) {
        if (data === null)
            return false;
        var code, idx, bitlen = 0, max = data.length, map = BASE64_MAP;
        // Convert one by one.
        for (idx = 0; idx < max; idx++) {
            code = map.indexOf(data.charAt(idx));
            // Skip CR/LF
            if (code > 64)
                continue;
            // Fail on illegal characters
            if (code < 0)
                return false;
            bitlen += 6;
        }
        // If there are any bits left, source was corrupted
        return (bitlen % 8) === 0;
    }
    function constructYamlBinary(data) {
        var idx, tailbits, input = data.replace(/[\r\n=]/g, ''), // remove CR/LF & padding to simplify scan
        max = input.length, map = BASE64_MAP, bits = 0, result = [];
        // Collect by 6*4 bits (3 bytes)
        for (idx = 0; idx < max; idx++) {
            if ((idx % 4 === 0) && idx) {
                result.push((bits >> 16) & 0xFF);
                result.push((bits >> 8) & 0xFF);
                result.push(bits & 0xFF);
            }
            bits = (bits << 6) | map.indexOf(input.charAt(idx));
        }
        // Dump tail
        tailbits = (max % 4) * 6;
        if (tailbits === 0) {
            result.push((bits >> 16) & 0xFF);
            result.push((bits >> 8) & 0xFF);
            result.push(bits & 0xFF);
        }
        else if (tailbits === 18) {
            result.push((bits >> 10) & 0xFF);
            result.push((bits >> 2) & 0xFF);
        }
        else if (tailbits === 12) {
            result.push((bits >> 4) & 0xFF);
        }
        // Wrap into Buffer for NodeJS and leave Array for browser
        if (NodeBuffer) {
            // Support node 6.+ Buffer API when available
            return NodeBuffer.from ? NodeBuffer.from(result) : new NodeBuffer(result);
        }
        return result;
    }
    function representYamlBinary(object /*, style*/) {
        var result = '', bits = 0, idx, tail, max = object.length, map = BASE64_MAP;
        // Convert every three bytes to 4 ASCII characters.
        for (idx = 0; idx < max; idx++) {
            if ((idx % 3 === 0) && idx) {
                result += map[(bits >> 18) & 0x3F];
                result += map[(bits >> 12) & 0x3F];
                result += map[(bits >> 6) & 0x3F];
                result += map[bits & 0x3F];
            }
            bits = (bits << 8) + object[idx];
        }
        // Dump tail
        tail = max % 3;
        if (tail === 0) {
            result += map[(bits >> 18) & 0x3F];
            result += map[(bits >> 12) & 0x3F];
            result += map[(bits >> 6) & 0x3F];
            result += map[bits & 0x3F];
        }
        else if (tail === 2) {
            result += map[(bits >> 10) & 0x3F];
            result += map[(bits >> 4) & 0x3F];
            result += map[(bits << 2) & 0x3F];
            result += map[64];
        }
        else if (tail === 1) {
            result += map[(bits >> 2) & 0x3F];
            result += map[(bits << 4) & 0x3F];
            result += map[64];
            result += map[64];
        }
        return result;
    }
    function isBinary(object) {
        return NodeBuffer && NodeBuffer.isBuffer(object);
    }
    module.exports = new Type('tag:yaml.org,2002:binary', {
        kind: 'scalar',
        resolve: resolveYamlBinary,
        construct: constructYamlBinary,
        predicate: isBinary,
        represent: representYamlBinary
    });
},
/* js-yaml/lib/js-yaml/type/omap.js */ function _(require, module, exports) {
    var Type = require(543) /* ../type */;
    var _hasOwnProperty = Object.prototype.hasOwnProperty;
    var _toString = Object.prototype.toString;
    function resolveYamlOmap(data) {
        if (data === null)
            return true;
        var objectKeys = [], index, length, pair, pairKey, pairHasKey, object = data;
        for (index = 0, length = object.length; index < length; index += 1) {
            pair = object[index];
            pairHasKey = false;
            if (_toString.call(pair) !== '[object Object]')
                return false;
            for (pairKey in pair) {
                if (_hasOwnProperty.call(pair, pairKey)) {
                    if (!pairHasKey)
                        pairHasKey = true;
                    else
                        return false;
                }
            }
            if (!pairHasKey)
                return false;
            if (objectKeys.indexOf(pairKey) === -1)
                objectKeys.push(pairKey);
            else
                return false;
        }
        return true;
    }
    function constructYamlOmap(data) {
        return data !== null ? data : [];
    }
    module.exports = new Type('tag:yaml.org,2002:omap', {
        kind: 'sequence',
        resolve: resolveYamlOmap,
        construct: constructYamlOmap
    });
},
/* js-yaml/lib/js-yaml/type/pairs.js */ function _(require, module, exports) {
    var Type = require(543) /* ../type */;
    var _toString = Object.prototype.toString;
    function resolveYamlPairs(data) {
        if (data === null)
            return true;
        var index, length, pair, keys, result, object = data;
        result = new Array(object.length);
        for (index = 0, length = object.length; index < length; index += 1) {
            pair = object[index];
            if (_toString.call(pair) !== '[object Object]')
                return false;
            keys = Object.keys(pair);
            if (keys.length !== 1)
                return false;
            result[index] = [keys[0], pair[keys[0]]];
        }
        return true;
    }
    function constructYamlPairs(data) {
        if (data === null)
            return [];
        var index, length, pair, keys, result, object = data;
        result = new Array(object.length);
        for (index = 0, length = object.length; index < length; index += 1) {
            pair = object[index];
            keys = Object.keys(pair);
            result[index] = [keys[0], pair[keys[0]]];
        }
        return result;
    }
    module.exports = new Type('tag:yaml.org,2002:pairs', {
        kind: 'sequence',
        resolve: resolveYamlPairs,
        construct: constructYamlPairs
    });
},
/* js-yaml/lib/js-yaml/type/set.js */ function _(require, module, exports) {
    var Type = require(543) /* ../type */;
    var _hasOwnProperty = Object.prototype.hasOwnProperty;
    function resolveYamlSet(data) {
        if (data === null)
            return true;
        var key, object = data;
        for (key in object) {
            if (_hasOwnProperty.call(object, key)) {
                if (object[key] !== null)
                    return false;
            }
        }
        return true;
    }
    function constructYamlSet(data) {
        return data !== null ? data : {};
    }
    module.exports = new Type('tag:yaml.org,2002:set', {
        kind: 'mapping',
        resolve: resolveYamlSet,
        construct: constructYamlSet
    });
},
/* js-yaml/lib/js-yaml/schema/default_full.js */ function _(require, module, exports) {
    var Schema = require(542) /* ../schema */;
    module.exports = Schema.DEFAULT = new Schema({
        include: [
            require(541) /* ./default_safe */
        ],
        explicit: [
            require(561) /* ../type/js/undefined */,
            require(562) /* ../type/js/regexp */,
            require(563) /* ../type/js/function */
        ]
    });
},
/* js-yaml/lib/js-yaml/type/js/undefined.js */ function _(require, module, exports) {
    var Type = require(543) /* ../../type */;
    function resolveJavascriptUndefined() {
        return true;
    }
    function constructJavascriptUndefined() {
        /*eslint-disable no-undefined*/
        return undefined;
    }
    function representJavascriptUndefined() {
        return '';
    }
    function isUndefined(object) {
        return typeof object === 'undefined';
    }
    module.exports = new Type('tag:yaml.org,2002:js/undefined', {
        kind: 'scalar',
        resolve: resolveJavascriptUndefined,
        construct: constructJavascriptUndefined,
        predicate: isUndefined,
        represent: representJavascriptUndefined
    });
},
/* js-yaml/lib/js-yaml/type/js/regexp.js */ function _(require, module, exports) {
    var Type = require(543) /* ../../type */;
    function resolveJavascriptRegExp(data) {
        if (data === null)
            return false;
        if (data.length === 0)
            return false;
        var regexp = data, tail = /\/([gim]*)$/.exec(data), modifiers = '';
        // if regexp starts with '/' it can have modifiers and must be properly closed
        // `/foo/gim` - modifiers tail can be maximum 3 chars
        if (regexp[0] === '/') {
            if (tail)
                modifiers = tail[1];
            if (modifiers.length > 3)
                return false;
            // if expression starts with /, is should be properly terminated
            if (regexp[regexp.length - modifiers.length - 1] !== '/')
                return false;
        }
        return true;
    }
    function constructJavascriptRegExp(data) {
        var regexp = data, tail = /\/([gim]*)$/.exec(data), modifiers = '';
        // `/foo/gim` - tail can be maximum 4 chars
        if (regexp[0] === '/') {
            if (tail)
                modifiers = tail[1];
            regexp = regexp.slice(1, regexp.length - modifiers.length - 1);
        }
        return new RegExp(regexp, modifiers);
    }
    function representJavascriptRegExp(object /*, style*/) {
        var result = '/' + object.source + '/';
        if (object.global)
            result += 'g';
        if (object.multiline)
            result += 'm';
        if (object.ignoreCase)
            result += 'i';
        return result;
    }
    function isRegExp(object) {
        return Object.prototype.toString.call(object) === '[object RegExp]';
    }
    module.exports = new Type('tag:yaml.org,2002:js/regexp', {
        kind: 'scalar',
        resolve: resolveJavascriptRegExp,
        construct: constructJavascriptRegExp,
        predicate: isRegExp,
        represent: representJavascriptRegExp
    });
},
/* js-yaml/lib/js-yaml/type/js/function.js */ function _(require, module, exports) {
    var esprima;
    // Browserified version does not have esprima
    //
    // 1. For node.js just require module as deps
    // 2. For browser try to require mudule via external AMD system.
    //    If not found - try to fallback to window.esprima. If not
    //    found too - then fail to parse.
    //
    try {
        // workaround to exclude package from browserify list.
        var _require = require;
        esprima = _require('esprima');
    }
    catch (_) {
        /*global window */
        if (typeof window !== 'undefined')
            esprima = window.esprima;
    }
    var Type = require(543) /* ../../type */;
    function resolveJavascriptFunction(data) {
        if (data === null)
            return false;
        try {
            var source = '(' + data + ')', ast = esprima.parse(source, { range: true });
            if (ast.type !== 'Program' ||
                ast.body.length !== 1 ||
                ast.body[0].type !== 'ExpressionStatement' ||
                (ast.body[0].expression.type !== 'ArrowFunctionExpression' &&
                    ast.body[0].expression.type !== 'FunctionExpression')) {
                return false;
            }
            return true;
        }
        catch (err) {
            return false;
        }
    }
    function constructJavascriptFunction(data) {
        /*jslint evil:true*/
        var source = '(' + data + ')', ast = esprima.parse(source, { range: true }), params = [], body;
        if (ast.type !== 'Program' ||
            ast.body.length !== 1 ||
            ast.body[0].type !== 'ExpressionStatement' ||
            (ast.body[0].expression.type !== 'ArrowFunctionExpression' &&
                ast.body[0].expression.type !== 'FunctionExpression')) {
            throw new Error('Failed to resolve function');
        }
        ast.body[0].expression.params.forEach(function (param) {
            params.push(param.name);
        });
        body = ast.body[0].expression.body.range;
        // Esprima's ranges include the first '{' and the last '}' characters on
        // function expressions. So cut them out.
        if (ast.body[0].expression.body.type === 'BlockStatement') {
            /*eslint-disable no-new-func*/
            return new Function(params, source.slice(body[0] + 1, body[1] - 1));
        }
        // ES6 arrow functions can omit the BlockStatement. In that case, just return
        // the body.
        /*eslint-disable no-new-func*/
        return new Function(params, 'return ' + source.slice(body[0], body[1]));
    }
    function representJavascriptFunction(object /*, style*/) {
        return object.toString();
    }
    function isFunction(object) {
        return Object.prototype.toString.call(object) === '[object Function]';
    }
    module.exports = new Type('tag:yaml.org,2002:js/function', {
        kind: 'scalar',
        resolve: resolveJavascriptFunction,
        construct: constructJavascriptFunction,
        predicate: isFunction,
        represent: representJavascriptFunction
    });
},
/* js-yaml/lib/js-yaml/dumper.js */ function _(require, module, exports) {
    /*eslint-disable no-use-before-define*/
    var common = require(538) /* ./common */;
    var YAMLException = require(539) /* ./exception */;
    var DEFAULT_FULL_SCHEMA = require(560) /* ./schema/default_full */;
    var DEFAULT_SAFE_SCHEMA = require(541) /* ./schema/default_safe */;
    var _toString = Object.prototype.toString;
    var _hasOwnProperty = Object.prototype.hasOwnProperty;
    var CHAR_TAB = 0x09; /* Tab */
    var CHAR_LINE_FEED = 0x0A; /* LF */
    var CHAR_SPACE = 0x20; /* Space */
    var CHAR_EXCLAMATION = 0x21; /* ! */
    var CHAR_DOUBLE_QUOTE = 0x22; /* " */
    var CHAR_SHARP = 0x23; /* # */
    var CHAR_PERCENT = 0x25; /* % */
    var CHAR_AMPERSAND = 0x26; /* & */
    var CHAR_SINGLE_QUOTE = 0x27; /* ' */
    var CHAR_ASTERISK = 0x2A; /* * */
    var CHAR_COMMA = 0x2C; /* , */
    var CHAR_MINUS = 0x2D; /* - */
    var CHAR_COLON = 0x3A; /* : */
    var CHAR_GREATER_THAN = 0x3E; /* > */
    var CHAR_QUESTION = 0x3F; /* ? */
    var CHAR_COMMERCIAL_AT = 0x40; /* @ */
    var CHAR_LEFT_SQUARE_BRACKET = 0x5B; /* [ */
    var CHAR_RIGHT_SQUARE_BRACKET = 0x5D; /* ] */
    var CHAR_GRAVE_ACCENT = 0x60; /* ` */
    var CHAR_LEFT_CURLY_BRACKET = 0x7B; /* { */
    var CHAR_VERTICAL_LINE = 0x7C; /* | */
    var CHAR_RIGHT_CURLY_BRACKET = 0x7D; /* } */
    var ESCAPE_SEQUENCES = {};
    ESCAPE_SEQUENCES[0x00] = '\\0';
    ESCAPE_SEQUENCES[0x07] = '\\a';
    ESCAPE_SEQUENCES[0x08] = '\\b';
    ESCAPE_SEQUENCES[0x09] = '\\t';
    ESCAPE_SEQUENCES[0x0A] = '\\n';
    ESCAPE_SEQUENCES[0x0B] = '\\v';
    ESCAPE_SEQUENCES[0x0C] = '\\f';
    ESCAPE_SEQUENCES[0x0D] = '\\r';
    ESCAPE_SEQUENCES[0x1B] = '\\e';
    ESCAPE_SEQUENCES[0x22] = '\\"';
    ESCAPE_SEQUENCES[0x5C] = '\\\\';
    ESCAPE_SEQUENCES[0x85] = '\\N';
    ESCAPE_SEQUENCES[0xA0] = '\\_';
    ESCAPE_SEQUENCES[0x2028] = '\\L';
    ESCAPE_SEQUENCES[0x2029] = '\\P';
    var DEPRECATED_BOOLEANS_SYNTAX = [
        'y', 'Y', 'yes', 'Yes', 'YES', 'on', 'On', 'ON',
        'n', 'N', 'no', 'No', 'NO', 'off', 'Off', 'OFF'
    ];
    function compileStyleMap(schema, map) {
        var result, keys, index, length, tag, style, type;
        if (map === null)
            return {};
        result = {};
        keys = Object.keys(map);
        for (index = 0, length = keys.length; index < length; index += 1) {
            tag = keys[index];
            style = String(map[tag]);
            if (tag.slice(0, 2) === '!!') {
                tag = 'tag:yaml.org,2002:' + tag.slice(2);
            }
            type = schema.compiledTypeMap['fallback'][tag];
            if (type && _hasOwnProperty.call(type.styleAliases, style)) {
                style = type.styleAliases[style];
            }
            result[tag] = style;
        }
        return result;
    }
    function encodeHex(character) {
        var string, handle, length;
        string = character.toString(16).toUpperCase();
        if (character <= 0xFF) {
            handle = 'x';
            length = 2;
        }
        else if (character <= 0xFFFF) {
            handle = 'u';
            length = 4;
        }
        else if (character <= 0xFFFFFFFF) {
            handle = 'U';
            length = 8;
        }
        else {
            throw new YAMLException('code point within a string may not be greater than 0xFFFFFFFF');
        }
        return '\\' + handle + common.repeat('0', length - string.length) + string;
    }
    function State(options) {
        this.schema = options['schema'] || DEFAULT_FULL_SCHEMA;
        this.indent = Math.max(1, (options['indent'] || 2));
        this.noArrayIndent = options['noArrayIndent'] || false;
        this.skipInvalid = options['skipInvalid'] || false;
        this.flowLevel = (common.isNothing(options['flowLevel']) ? -1 : options['flowLevel']);
        this.styleMap = compileStyleMap(this.schema, options['styles'] || null);
        this.sortKeys = options['sortKeys'] || false;
        this.lineWidth = options['lineWidth'] || 80;
        this.noRefs = options['noRefs'] || false;
        this.noCompatMode = options['noCompatMode'] || false;
        this.condenseFlow = options['condenseFlow'] || false;
        this.implicitTypes = this.schema.compiledImplicit;
        this.explicitTypes = this.schema.compiledExplicit;
        this.tag = null;
        this.result = '';
        this.duplicates = [];
        this.usedDuplicates = null;
    }
    // Indents every line in a string. Empty lines (\n only) are not indented.
    function indentString(string, spaces) {
        var ind = common.repeat(' ', spaces), position = 0, next = -1, result = '', line, length = string.length;
        while (position < length) {
            next = string.indexOf('\n', position);
            if (next === -1) {
                line = string.slice(position);
                position = length;
            }
            else {
                line = string.slice(position, next + 1);
                position = next + 1;
            }
            if (line.length && line !== '\n')
                result += ind;
            result += line;
        }
        return result;
    }
    function generateNextLine(state, level) {
        return '\n' + common.repeat(' ', state.indent * level);
    }
    function testImplicitResolving(state, str) {
        var index, length, type;
        for (index = 0, length = state.implicitTypes.length; index < length; index += 1) {
            type = state.implicitTypes[index];
            if (type.resolve(str)) {
                return true;
            }
        }
        return false;
    }
    // [33] s-white ::= s-space | s-tab
    function isWhitespace(c) {
        return c === CHAR_SPACE || c === CHAR_TAB;
    }
    // Returns true if the character can be printed without escaping.
    // From YAML 1.2: "any allowed characters known to be non-printable
    // should also be escaped. [However,] This isn’t mandatory"
    // Derived from nb-char - \t - #x85 - #xA0 - #x2028 - #x2029.
    function isPrintable(c) {
        return (0x00020 <= c && c <= 0x00007E)
            || ((0x000A1 <= c && c <= 0x00D7FF) && c !== 0x2028 && c !== 0x2029)
            || ((0x0E000 <= c && c <= 0x00FFFD) && c !== 0xFEFF /* BOM */)
            || (0x10000 <= c && c <= 0x10FFFF);
    }
    // Simplified test for values allowed after the first character in plain style.
    function isPlainSafe(c) {
        // Uses a subset of nb-char - c-flow-indicator - ":" - "#"
        // where nb-char ::= c-printable - b-char - c-byte-order-mark.
        return isPrintable(c) && c !== 0xFEFF
            // - c-flow-indicator
            && c !== CHAR_COMMA
            && c !== CHAR_LEFT_SQUARE_BRACKET
            && c !== CHAR_RIGHT_SQUARE_BRACKET
            && c !== CHAR_LEFT_CURLY_BRACKET
            && c !== CHAR_RIGHT_CURLY_BRACKET
            // - ":" - "#"
            && c !== CHAR_COLON
            && c !== CHAR_SHARP;
    }
    // Simplified test for values allowed as the first character in plain style.
    function isPlainSafeFirst(c) {
        // Uses a subset of ns-char - c-indicator
        // where ns-char = nb-char - s-white.
        return isPrintable(c) && c !== 0xFEFF
            && !isWhitespace(c) // - s-white
            // - (c-indicator ::=
            // “-” | “?” | “:” | “,” | “[” | “]” | “{” | “}”
            && c !== CHAR_MINUS
            && c !== CHAR_QUESTION
            && c !== CHAR_COLON
            && c !== CHAR_COMMA
            && c !== CHAR_LEFT_SQUARE_BRACKET
            && c !== CHAR_RIGHT_SQUARE_BRACKET
            && c !== CHAR_LEFT_CURLY_BRACKET
            && c !== CHAR_RIGHT_CURLY_BRACKET
            // | “#” | “&” | “*” | “!” | “|” | “>” | “'” | “"”
            && c !== CHAR_SHARP
            && c !== CHAR_AMPERSAND
            && c !== CHAR_ASTERISK
            && c !== CHAR_EXCLAMATION
            && c !== CHAR_VERTICAL_LINE
            && c !== CHAR_GREATER_THAN
            && c !== CHAR_SINGLE_QUOTE
            && c !== CHAR_DOUBLE_QUOTE
            // | “%” | “@” | “`”)
            && c !== CHAR_PERCENT
            && c !== CHAR_COMMERCIAL_AT
            && c !== CHAR_GRAVE_ACCENT;
    }
    // Determines whether block indentation indicator is required.
    function needIndentIndicator(string) {
        var leadingSpaceRe = /^\n* /;
        return leadingSpaceRe.test(string);
    }
    var STYLE_PLAIN = 1, STYLE_SINGLE = 2, STYLE_LITERAL = 3, STYLE_FOLDED = 4, STYLE_DOUBLE = 5;
    // Determines which scalar styles are possible and returns the preferred style.
    // lineWidth = -1 => no limit.
    // Pre-conditions: str.length > 0.
    // Post-conditions:
    //    STYLE_PLAIN or STYLE_SINGLE => no \n are in the string.
    //    STYLE_LITERAL => no lines are suitable for folding (or lineWidth is -1).
    //    STYLE_FOLDED => a line > lineWidth and can be folded (and lineWidth != -1).
    function chooseScalarStyle(string, singleLineOnly, indentPerLevel, lineWidth, testAmbiguousType) {
        var i;
        var char;
        var hasLineBreak = false;
        var hasFoldableLine = false; // only checked if shouldTrackWidth
        var shouldTrackWidth = lineWidth !== -1;
        var previousLineBreak = -1; // count the first line correctly
        var plain = isPlainSafeFirst(string.charCodeAt(0))
            && !isWhitespace(string.charCodeAt(string.length - 1));
        if (singleLineOnly) {
            // Case: no block styles.
            // Check for disallowed characters to rule out plain and single.
            for (i = 0; i < string.length; i++) {
                char = string.charCodeAt(i);
                if (!isPrintable(char)) {
                    return STYLE_DOUBLE;
                }
                plain = plain && isPlainSafe(char);
            }
        }
        else {
            // Case: block styles permitted.
            for (i = 0; i < string.length; i++) {
                char = string.charCodeAt(i);
                if (char === CHAR_LINE_FEED) {
                    hasLineBreak = true;
                    // Check if any line can be folded.
                    if (shouldTrackWidth) {
                        hasFoldableLine = hasFoldableLine ||
                            // Foldable line = too long, and not more-indented.
                            (i - previousLineBreak - 1 > lineWidth &&
                                string[previousLineBreak + 1] !== ' ');
                        previousLineBreak = i;
                    }
                }
                else if (!isPrintable(char)) {
                    return STYLE_DOUBLE;
                }
                plain = plain && isPlainSafe(char);
            }
            // in case the end is missing a \n
            hasFoldableLine = hasFoldableLine || (shouldTrackWidth &&
                (i - previousLineBreak - 1 > lineWidth &&
                    string[previousLineBreak + 1] !== ' '));
        }
        // Although every style can represent \n without escaping, prefer block styles
        // for multiline, since they're more readable and they don't add empty lines.
        // Also prefer folding a super-long line.
        if (!hasLineBreak && !hasFoldableLine) {
            // Strings interpretable as another type have to be quoted;
            // e.g. the string 'true' vs. the boolean true.
            return plain && !testAmbiguousType(string)
                ? STYLE_PLAIN : STYLE_SINGLE;
        }
        // Edge case: block indentation indicator can only have one digit.
        if (indentPerLevel > 9 && needIndentIndicator(string)) {
            return STYLE_DOUBLE;
        }
        // At this point we know block styles are valid.
        // Prefer literal style unless we want to fold.
        return hasFoldableLine ? STYLE_FOLDED : STYLE_LITERAL;
    }
    // Note: line breaking/folding is implemented for only the folded style.
    // NB. We drop the last trailing newline (if any) of a returned block scalar
    //  since the dumper adds its own newline. This always works:
    //    • No ending newline => unaffected; already using strip "-" chomping.
    //    • Ending newline    => removed then restored.
    //  Importantly, this keeps the "+" chomp indicator from gaining an extra line.
    function writeScalar(state, string, level, iskey) {
        state.dump = (function () {
            if (string.length === 0) {
                return "''";
            }
            if (!state.noCompatMode &&
                DEPRECATED_BOOLEANS_SYNTAX.indexOf(string) !== -1) {
                return "'" + string + "'";
            }
            var indent = state.indent * Math.max(1, level); // no 0-indent scalars
            // As indentation gets deeper, let the width decrease monotonically
            // to the lower bound min(state.lineWidth, 40).
            // Note that this implies
            //  state.lineWidth ≤ 40 + state.indent: width is fixed at the lower bound.
            //  state.lineWidth > 40 + state.indent: width decreases until the lower bound.
            // This behaves better than a constant minimum width which disallows narrower options,
            // or an indent threshold which causes the width to suddenly increase.
            var lineWidth = state.lineWidth === -1
                ? -1 : Math.max(Math.min(state.lineWidth, 40), state.lineWidth - indent);
            // Without knowing if keys are implicit/explicit, assume implicit for safety.
            var singleLineOnly = iskey
                // No block styles in flow mode.
                || (state.flowLevel > -1 && level >= state.flowLevel);
            function testAmbiguity(string) {
                return testImplicitResolving(state, string);
            }
            switch (chooseScalarStyle(string, singleLineOnly, state.indent, lineWidth, testAmbiguity)) {
                case STYLE_PLAIN:
                    return string;
                case STYLE_SINGLE:
                    return "'" + string.replace(/'/g, "''") + "'";
                case STYLE_LITERAL:
                    return '|' + blockHeader(string, state.indent)
                        + dropEndingNewline(indentString(string, indent));
                case STYLE_FOLDED:
                    return '>' + blockHeader(string, state.indent)
                        + dropEndingNewline(indentString(foldString(string, lineWidth), indent));
                case STYLE_DOUBLE:
                    return '"' + escapeString(string, lineWidth) + '"';
                default:
                    throw new YAMLException('impossible error: invalid scalar style');
            }
        }());
    }
    // Pre-conditions: string is valid for a block scalar, 1 <= indentPerLevel <= 9.
    function blockHeader(string, indentPerLevel) {
        var indentIndicator = needIndentIndicator(string) ? String(indentPerLevel) : '';
        // note the special case: the string '\n' counts as a "trailing" empty line.
        var clip = string[string.length - 1] === '\n';
        var keep = clip && (string[string.length - 2] === '\n' || string === '\n');
        var chomp = keep ? '+' : (clip ? '' : '-');
        return indentIndicator + chomp + '\n';
    }
    // (See the note for writeScalar.)
    function dropEndingNewline(string) {
        return string[string.length - 1] === '\n' ? string.slice(0, -1) : string;
    }
    // Note: a long line without a suitable break point will exceed the width limit.
    // Pre-conditions: every char in str isPrintable, str.length > 0, width > 0.
    function foldString(string, width) {
        // In folded style, $k$ consecutive newlines output as $k+1$ newlines—
        // unless they're before or after a more-indented line, or at the very
        // beginning or end, in which case $k$ maps to $k$.
        // Therefore, parse each chunk as newline(s) followed by a content line.
        var lineRe = /(\n+)([^\n]*)/g;
        // first line (possibly an empty line)
        var result = (function () {
            var nextLF = string.indexOf('\n');
            nextLF = nextLF !== -1 ? nextLF : string.length;
            lineRe.lastIndex = nextLF;
            return foldLine(string.slice(0, nextLF), width);
        }());
        // If we haven't reached the first content line yet, don't add an extra \n.
        var prevMoreIndented = string[0] === '\n' || string[0] === ' ';
        var moreIndented;
        // rest of the lines
        var match;
        while ((match = lineRe.exec(string))) {
            var prefix = match[1], line = match[2];
            moreIndented = (line[0] === ' ');
            result += prefix
                + (!prevMoreIndented && !moreIndented && line !== ''
                    ? '\n' : '')
                + foldLine(line, width);
            prevMoreIndented = moreIndented;
        }
        return result;
    }
    // Greedy line breaking.
    // Picks the longest line under the limit each time,
    // otherwise settles for the shortest line over the limit.
    // NB. More-indented lines *cannot* be folded, as that would add an extra \n.
    function foldLine(line, width) {
        if (line === '' || line[0] === ' ')
            return line;
        // Since a more-indented line adds a \n, breaks can't be followed by a space.
        var breakRe = / [^ ]/g; // note: the match index will always be <= length-2.
        var match;
        // start is an inclusive index. end, curr, and next are exclusive.
        var start = 0, end, curr = 0, next = 0;
        var result = '';
        // Invariants: 0 <= start <= length-1.
        //   0 <= curr <= next <= max(0, length-2). curr - start <= width.
        // Inside the loop:
        //   A match implies length >= 2, so curr and next are <= length-2.
        while ((match = breakRe.exec(line))) {
            next = match.index;
            // maintain invariant: curr - start <= width
            if (next - start > width) {
                end = (curr > start) ? curr : next; // derive end <= length-2
                result += '\n' + line.slice(start, end);
                // skip the space that was output as \n
                start = end + 1; // derive start <= length-1
            }
            curr = next;
        }
        // By the invariants, start <= length-1, so there is something left over.
        // It is either the whole string or a part starting from non-whitespace.
        result += '\n';
        // Insert a break if the remainder is too long and there is a break available.
        if (line.length - start > width && curr > start) {
            result += line.slice(start, curr) + '\n' + line.slice(curr + 1);
        }
        else {
            result += line.slice(start);
        }
        return result.slice(1); // drop extra \n joiner
    }
    // Escapes a double-quoted string.
    function escapeString(string) {
        var result = '';
        var char, nextChar;
        var escapeSeq;
        for (var i = 0; i < string.length; i++) {
            char = string.charCodeAt(i);
            // Check for surrogate pairs (reference Unicode 3.0 section "3.7 Surrogates").
            if (char >= 0xD800 && char <= 0xDBFF /* high surrogate */) {
                nextChar = string.charCodeAt(i + 1);
                if (nextChar >= 0xDC00 && nextChar <= 0xDFFF /* low surrogate */) {
                    // Combine the surrogate pair and store it escaped.
                    result += encodeHex((char - 0xD800) * 0x400 + nextChar - 0xDC00 + 0x10000);
                    // Advance index one extra since we already used that char here.
                    i++;
                    continue;
                }
            }
            escapeSeq = ESCAPE_SEQUENCES[char];
            result += !escapeSeq && isPrintable(char)
                ? string[i]
                : escapeSeq || encodeHex(char);
        }
        return result;
    }
    function writeFlowSequence(state, level, object) {
        var _result = '', _tag = state.tag, index, length;
        for (index = 0, length = object.length; index < length; index += 1) {
            // Write only valid elements.
            if (writeNode(state, level, object[index], false, false)) {
                if (index !== 0)
                    _result += ',' + (!state.condenseFlow ? ' ' : '');
                _result += state.dump;
            }
        }
        state.tag = _tag;
        state.dump = '[' + _result + ']';
    }
    function writeBlockSequence(state, level, object, compact) {
        var _result = '', _tag = state.tag, index, length;
        for (index = 0, length = object.length; index < length; index += 1) {
            // Write only valid elements.
            if (writeNode(state, level + 1, object[index], true, true)) {
                if (!compact || index !== 0) {
                    _result += generateNextLine(state, level);
                }
                if (state.dump && CHAR_LINE_FEED === state.dump.charCodeAt(0)) {
                    _result += '-';
                }
                else {
                    _result += '- ';
                }
                _result += state.dump;
            }
        }
        state.tag = _tag;
        state.dump = _result || '[]'; // Empty sequence if no valid values.
    }
    function writeFlowMapping(state, level, object) {
        var _result = '', _tag = state.tag, objectKeyList = Object.keys(object), index, length, objectKey, objectValue, pairBuffer;
        for (index = 0, length = objectKeyList.length; index < length; index += 1) {
            pairBuffer = state.condenseFlow ? '"' : '';
            if (index !== 0)
                pairBuffer += ', ';
            objectKey = objectKeyList[index];
            objectValue = object[objectKey];
            if (!writeNode(state, level, objectKey, false, false)) {
                continue; // Skip this pair because of invalid key;
            }
            if (state.dump.length > 1024)
                pairBuffer += '? ';
            pairBuffer += state.dump + (state.condenseFlow ? '"' : '') + ':' + (state.condenseFlow ? '' : ' ');
            if (!writeNode(state, level, objectValue, false, false)) {
                continue; // Skip this pair because of invalid value.
            }
            pairBuffer += state.dump;
            // Both key and value are valid.
            _result += pairBuffer;
        }
        state.tag = _tag;
        state.dump = '{' + _result + '}';
    }
    function writeBlockMapping(state, level, object, compact) {
        var _result = '', _tag = state.tag, objectKeyList = Object.keys(object), index, length, objectKey, objectValue, explicitPair, pairBuffer;
        // Allow sorting keys so that the output file is deterministic
        if (state.sortKeys === true) {
            // Default sorting
            objectKeyList.sort();
        }
        else if (typeof state.sortKeys === 'function') {
            // Custom sort function
            objectKeyList.sort(state.sortKeys);
        }
        else if (state.sortKeys) {
            // Something is wrong
            throw new YAMLException('sortKeys must be a boolean or a function');
        }
        for (index = 0, length = objectKeyList.length; index < length; index += 1) {
            pairBuffer = '';
            if (!compact || index !== 0) {
                pairBuffer += generateNextLine(state, level);
            }
            objectKey = objectKeyList[index];
            objectValue = object[objectKey];
            if (!writeNode(state, level + 1, objectKey, true, true, true)) {
                continue; // Skip this pair because of invalid key.
            }
            explicitPair = (state.tag !== null && state.tag !== '?') ||
                (state.dump && state.dump.length > 1024);
            if (explicitPair) {
                if (state.dump && CHAR_LINE_FEED === state.dump.charCodeAt(0)) {
                    pairBuffer += '?';
                }
                else {
                    pairBuffer += '? ';
                }
            }
            pairBuffer += state.dump;
            if (explicitPair) {
                pairBuffer += generateNextLine(state, level);
            }
            if (!writeNode(state, level + 1, objectValue, true, explicitPair)) {
                continue; // Skip this pair because of invalid value.
            }
            if (state.dump && CHAR_LINE_FEED === state.dump.charCodeAt(0)) {
                pairBuffer += ':';
            }
            else {
                pairBuffer += ': ';
            }
            pairBuffer += state.dump;
            // Both key and value are valid.
            _result += pairBuffer;
        }
        state.tag = _tag;
        state.dump = _result || '{}'; // Empty mapping if no valid pairs.
    }
    function detectType(state, object, explicit) {
        var _result, typeList, index, length, type, style;
        typeList = explicit ? state.explicitTypes : state.implicitTypes;
        for (index = 0, length = typeList.length; index < length; index += 1) {
            type = typeList[index];
            if ((type.instanceOf || type.predicate) &&
                (!type.instanceOf || ((typeof object === 'object') && (object instanceof type.instanceOf))) &&
                (!type.predicate || type.predicate(object))) {
                state.tag = explicit ? type.tag : '?';
                if (type.represent) {
                    style = state.styleMap[type.tag] || type.defaultStyle;
                    if (_toString.call(type.represent) === '[object Function]') {
                        _result = type.represent(object, style);
                    }
                    else if (_hasOwnProperty.call(type.represent, style)) {
                        _result = type.represent[style](object, style);
                    }
                    else {
                        throw new YAMLException('!<' + type.tag + '> tag resolver accepts not "' + style + '" style');
                    }
                    state.dump = _result;
                }
                return true;
            }
        }
        return false;
    }
    // Serializes `object` and writes it to global `result`.
    // Returns true on success, or false on invalid object.
    //
    function writeNode(state, level, object, block, compact, iskey) {
        state.tag = null;
        state.dump = object;
        if (!detectType(state, object, false)) {
            detectType(state, object, true);
        }
        var type = _toString.call(state.dump);
        if (block) {
            block = (state.flowLevel < 0 || state.flowLevel > level);
        }
        var objectOrArray = type === '[object Object]' || type === '[object Array]', duplicateIndex, duplicate;
        if (objectOrArray) {
            duplicateIndex = state.duplicates.indexOf(object);
            duplicate = duplicateIndex !== -1;
        }
        if ((state.tag !== null && state.tag !== '?') || duplicate || (state.indent !== 2 && level > 0)) {
            compact = false;
        }
        if (duplicate && state.usedDuplicates[duplicateIndex]) {
            state.dump = '*ref_' + duplicateIndex;
        }
        else {
            if (objectOrArray && duplicate && !state.usedDuplicates[duplicateIndex]) {
                state.usedDuplicates[duplicateIndex] = true;
            }
            if (type === '[object Object]') {
                if (block && (Object.keys(state.dump).length !== 0)) {
                    writeBlockMapping(state, level, state.dump, compact);
                    if (duplicate) {
                        state.dump = '&ref_' + duplicateIndex + state.dump;
                    }
                }
                else {
                    writeFlowMapping(state, level, state.dump);
                    if (duplicate) {
                        state.dump = '&ref_' + duplicateIndex + ' ' + state.dump;
                    }
                }
            }
            else if (type === '[object Array]') {
                var arrayLevel = (state.noArrayIndent && (level > 0)) ? level - 1 : level;
                if (block && (state.dump.length !== 0)) {
                    writeBlockSequence(state, arrayLevel, state.dump, compact);
                    if (duplicate) {
                        state.dump = '&ref_' + duplicateIndex + state.dump;
                    }
                }
                else {
                    writeFlowSequence(state, arrayLevel, state.dump);
                    if (duplicate) {
                        state.dump = '&ref_' + duplicateIndex + ' ' + state.dump;
                    }
                }
            }
            else if (type === '[object String]') {
                if (state.tag !== '?') {
                    writeScalar(state, state.dump, level, iskey);
                }
            }
            else {
                if (state.skipInvalid)
                    return false;
                throw new YAMLException('unacceptable kind of an object to dump ' + type);
            }
            if (state.tag !== null && state.tag !== '?') {
                state.dump = '!<' + state.tag + '> ' + state.dump;
            }
        }
        return true;
    }
    function getDuplicateReferences(object, state) {
        var objects = [], duplicatesIndexes = [], index, length;
        inspectNode(object, objects, duplicatesIndexes);
        for (index = 0, length = duplicatesIndexes.length; index < length; index += 1) {
            state.duplicates.push(objects[duplicatesIndexes[index]]);
        }
        state.usedDuplicates = new Array(length);
    }
    function inspectNode(object, objects, duplicatesIndexes) {
        var objectKeyList, index, length;
        if (object !== null && typeof object === 'object') {
            index = objects.indexOf(object);
            if (index !== -1) {
                if (duplicatesIndexes.indexOf(index) === -1) {
                    duplicatesIndexes.push(index);
                }
            }
            else {
                objects.push(object);
                if (Array.isArray(object)) {
                    for (index = 0, length = object.length; index < length; index += 1) {
                        inspectNode(object[index], objects, duplicatesIndexes);
                    }
                }
                else {
                    objectKeyList = Object.keys(object);
                    for (index = 0, length = objectKeyList.length; index < length; index += 1) {
                        inspectNode(object[objectKeyList[index]], objects, duplicatesIndexes);
                    }
                }
            }
        }
    }
    function dump(input, options) {
        options = options || {};
        var state = new State(options);
        if (!state.noRefs)
            getDuplicateReferences(input, state);
        if (writeNode(state, 0, input, true, true))
            return state.dump + '\n';
        return '';
    }
    function safeDump(input, options) {
        return dump(input, common.extend({ schema: DEFAULT_SAFE_SCHEMA }, options));
    }
    module.exports.dump = dump;
    module.exports.safeDump = safeDump;
},
/* eslint/lib/cli-engine/load-rules.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const fs = require("fs"), path = require("path");
    const rulesDirCache = {};
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    /**
     * Load all rule modules from specified directory.
     * @param {string} relativeRulesDir Path to rules directory, may be relative.
     * @param {string} cwd Current working directory
     * @returns {Object} Loaded rule modules.
     */
    module.exports = function (relativeRulesDir, cwd) {
        const rulesDir = path.resolve(cwd, relativeRulesDir);
        // cache will help performance as IO operation are expensive
        if (rulesDirCache[rulesDir]) {
            return rulesDirCache[rulesDir];
        }
        const rules = Object.create(null);
        fs.readdirSync(rulesDir).forEach(file => {
            if (path.extname(file) !== ".js") {
                return;
            }
            rules[file.slice(0, -3)] = require(path.join(rulesDir, file));
        });
        rulesDirCache[rulesDir] = rules;
        return rules;
    };
},
/* eslint/lib/cli-engine/file-enumerator.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const fs = require("fs");
    const path = require("path");
    const getGlobParent = require(567) /* glob-parent */;
    const isGlob = require(568) /* is-glob */;
    const { escapeRegExp } = require(111) /* lodash */;
    const { Minimatch } = require(531) /* minimatch */;
    const { IgnorePattern } = require(525) /* ./config-array */;
    const { CascadingConfigArrayFactory } = require(518) /* ./cascading-config-array-factory */;
    const debug = require(120) /* debug */("eslint:file-enumerator");
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    const minimatchOpts = { dot: true, matchBase: true };
    const dotfilesPattern = /(?:(?:^\.)|(?:[/\\]\.))[^/\\.].*/u;
    const NONE = 0;
    const IGNORED_SILENTLY = 1;
    const IGNORED = 2;
    // For VSCode intellisense
    /** @typedef {ReturnType<CascadingConfigArrayFactory["getConfigArrayForFile"]>} ConfigArray */
    /**
     * @typedef {Object} FileEnumeratorOptions
     * @property {CascadingConfigArrayFactory} [configArrayFactory] The factory for config arrays.
     * @property {string} [cwd] The base directory to start lookup.
     * @property {string[]} [extensions] The extensions to match files for directory patterns.
     * @property {boolean} [globInputPaths] Set to false to skip glob resolution of input file paths to lint (default: true). If false, each input file paths is assumed to be a non-glob path to an existing file.
     * @property {boolean} [ignore] The flag to check ignored files.
     * @property {string[]} [rulePaths] The value of `--rulesdir` option.
     */
    /**
     * @typedef {Object} FileAndConfig
     * @property {string} filePath The path to a target file.
     * @property {ConfigArray} config The config entries of that file.
     * @property {boolean} ignored If `true` then this file should be ignored and warned because it was directly specified.
     */
    /**
     * @typedef {Object} FileEntry
     * @property {string} filePath The path to a target file.
     * @property {ConfigArray} config The config entries of that file.
     * @property {NONE|IGNORED_SILENTLY|IGNORED} flag The flag.
     * - `NONE` means the file is a target file.
     * - `IGNORED_SILENTLY` means the file should be ignored silently.
     * - `IGNORED` means the file should be ignored and warned because it was directly specified.
     */
    /**
     * @typedef {Object} FileEnumeratorInternalSlots
     * @property {CascadingConfigArrayFactory} configArrayFactory The factory for config arrays.
     * @property {string} cwd The base directory to start lookup.
     * @property {RegExp} extensionRegExp The RegExp to test if a string ends with specific file extensions.
     * @property {boolean} globInputPaths Set to false to skip glob resolution of input file paths to lint (default: true). If false, each input file paths is assumed to be a non-glob path to an existing file.
     * @property {boolean} ignoreFlag The flag to check ignored files.
     * @property {(filePath:string, dot:boolean) => boolean} defaultIgnores The default predicate function to ignore files.
     */
    /** @type {WeakMap<FileEnumerator, FileEnumeratorInternalSlots>} */
    const internalSlotsMap = new WeakMap();
    /**
     * Check if a string is a glob pattern or not.
     * @param {string} pattern A glob pattern.
     * @returns {boolean} `true` if the string is a glob pattern.
     */
    function isGlobPattern(pattern) {
        return isGlob(path.sep === "\\" ? pattern.replace(/\\/gu, "/") : pattern);
    }
    /**
     * Get stats of a given path.
     * @param {string} filePath The path to target file.
     * @returns {fs.Stats|null} The stats.
     * @private
     */
    function statSafeSync(filePath) {
        try {
            return fs.statSync(filePath);
        }
        catch (error) {
            /* istanbul ignore next */
            if (error.code !== "ENOENT") {
                throw error;
            }
            return null;
        }
    }
    /**
     * Get filenames in a given path to a directory.
     * @param {string} directoryPath The path to target directory.
     * @returns {string[]} The filenames.
     * @private
     */
    function readdirSafeSync(directoryPath) {
        try {
            return fs.readdirSync(directoryPath);
        }
        catch (error) {
            /* istanbul ignore next */
            if (error.code !== "ENOENT") {
                throw error;
            }
            return [];
        }
    }
    /**
     * The error type when no files match a glob.
     */
    class NoFilesFoundError extends Error {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {string} pattern The glob pattern which was not found.
         * @param {boolean} globDisabled If `true` then the pattern was a glob pattern, but glob was disabled.
         */
        constructor(pattern, globDisabled) {
            super(`No files matching '${pattern}' were found${globDisabled ? " (glob was disabled)" : ""}.`);
            this.messageTemplate = "file-not-found";
            this.messageData = { pattern, globDisabled };
        }
    }
    /**
     * The error type when there are files matched by a glob, but all of them have been ignored.
     */
    class AllFilesIgnoredError extends Error {
        // eslint-disable-next-line jsdoc/require-description
        /**
         * @param {string} pattern The glob pattern which was not found.
         */
        constructor(pattern) {
            super(`All files matched by '${pattern}' are ignored.`);
            this.messageTemplate = "all-files-ignored";
            this.messageData = { pattern };
        }
    }
    /**
     * This class provides the functionality that enumerates every file which is
     * matched by given glob patterns and that configuration.
     */
    class FileEnumerator {
        /**
         * Initialize this enumerator.
         * @param {FileEnumeratorOptions} options The options.
         */
        constructor({ cwd = process.cwd(), configArrayFactory = new CascadingConfigArrayFactory({ cwd }), extensions = [".js"], globInputPaths = true, errorOnUnmatchedPattern = true, ignore = true } = {}) {
            internalSlotsMap.set(this, {
                configArrayFactory,
                cwd,
                defaultIgnores: IgnorePattern.createDefaultIgnore(cwd),
                extensionRegExp: new RegExp(`.\\.(?:${extensions
                    .map(ext => escapeRegExp(ext.startsWith(".")
                    ? ext.slice(1)
                    : ext))
                    .join("|")})$`, "u"),
                globInputPaths,
                errorOnUnmatchedPattern,
                ignoreFlag: ignore
            });
        }
        /**
         * The `RegExp` object that tests if a file path has the allowed file extensions.
         * @type {RegExp}
         */
        get extensionRegExp() {
            return internalSlotsMap.get(this).extensionRegExp;
        }
        /**
         * Iterate files which are matched by given glob patterns.
         * @param {string|string[]} patternOrPatterns The glob patterns to iterate files.
         * @returns {IterableIterator<FileAndConfig>} The found files.
         */
        *iterateFiles(patternOrPatterns) {
            const { globInputPaths, errorOnUnmatchedPattern } = internalSlotsMap.get(this);
            const patterns = Array.isArray(patternOrPatterns)
                ? patternOrPatterns
                : [patternOrPatterns];
            debug("Start to iterate files: %o", patterns);
            // The set of paths to remove duplicate.
            const set = new Set();
            for (const pattern of patterns) {
                let foundRegardlessOfIgnored = false;
                let found = false;
                // Skip empty string.
                if (!pattern) {
                    continue;
                }
                // Iterate files of this pttern.
                for (const { config, filePath, flag } of this._iterateFiles(pattern)) {
                    foundRegardlessOfIgnored = true;
                    if (flag === IGNORED_SILENTLY) {
                        continue;
                    }
                    found = true;
                    // Remove duplicate paths while yielding paths.
                    if (!set.has(filePath)) {
                        set.add(filePath);
                        yield {
                            config,
                            filePath,
                            ignored: flag === IGNORED
                        };
                    }
                }
                // Raise an error if any files were not found.
                if (errorOnUnmatchedPattern) {
                    if (!foundRegardlessOfIgnored) {
                        throw new NoFilesFoundError(pattern, !globInputPaths && isGlob(pattern));
                    }
                    if (!found) {
                        throw new AllFilesIgnoredError(pattern);
                    }
                }
            }
            debug(`Complete iterating files: ${JSON.stringify(patterns)}`);
        }
        /**
         * Iterate files which are matched by a given glob pattern.
         * @param {string} pattern The glob pattern to iterate files.
         * @returns {IterableIterator<FileEntry>} The found files.
         */
        _iterateFiles(pattern) {
            const { cwd, globInputPaths } = internalSlotsMap.get(this);
            const absolutePath = path.resolve(cwd, pattern);
            const isDot = dotfilesPattern.test(pattern);
            const stat = statSafeSync(absolutePath);
            if (stat && stat.isDirectory()) {
                return this._iterateFilesWithDirectory(absolutePath, isDot);
            }
            if (stat && stat.isFile()) {
                return this._iterateFilesWithFile(absolutePath);
            }
            if (globInputPaths && isGlobPattern(pattern)) {
                return this._iterateFilesWithGlob(absolutePath, isDot);
            }
            return [];
        }
        /**
         * Iterate a file which is matched by a given path.
         * @param {string} filePath The path to the target file.
         * @returns {IterableIterator<FileEntry>} The found files.
         * @private
         */
        _iterateFilesWithFile(filePath) {
            debug(`File: ${filePath}`);
            const { configArrayFactory } = internalSlotsMap.get(this);
            const config = configArrayFactory.getConfigArrayForFile(filePath);
            const ignored = this._isIgnoredFile(filePath, { config, direct: true });
            const flag = ignored ? IGNORED : NONE;
            return [{ config, filePath, flag }];
        }
        /**
         * Iterate files in a given path.
         * @param {string} directoryPath The path to the target directory.
         * @param {boolean} dotfiles If `true` then it doesn't skip dot files by default.
         * @returns {IterableIterator<FileEntry>} The found files.
         * @private
         */
        _iterateFilesWithDirectory(directoryPath, dotfiles) {
            debug(`Directory: ${directoryPath}`);
            return this._iterateFilesRecursive(directoryPath, { dotfiles, recursive: true, selector: null });
        }
        /**
         * Iterate files which are matched by a given glob pattern.
         * @param {string} pattern The glob pattern to iterate files.
         * @param {boolean} dotfiles If `true` then it doesn't skip dot files by default.
         * @returns {IterableIterator<FileEntry>} The found files.
         * @private
         */
        _iterateFilesWithGlob(pattern, dotfiles) {
            debug(`Glob: ${pattern}`);
            const directoryPath = path.resolve(getGlobParent(pattern));
            const globPart = pattern.slice(directoryPath.length + 1);
            /*
             * recursive if there are `**` or path separators in the glob part.
             * Otherwise, patterns such as `src/*.js`, it doesn't need recursive.
             */
            const recursive = /\*\*|\/|\\/u.test(globPart);
            const selector = new Minimatch(pattern, minimatchOpts);
            debug(`recursive? ${recursive}`);
            return this._iterateFilesRecursive(directoryPath, { dotfiles, recursive, selector });
        }
        /**
         * Iterate files in a given path.
         * @param {string} directoryPath The path to the target directory.
         * @param {Object} options The options to iterate files.
         * @param {boolean} [options.dotfiles] If `true` then it doesn't skip dot files by default.
         * @param {boolean} [options.recursive] If `true` then it dives into sub directories.
         * @param {InstanceType<Minimatch>} [options.selector] The matcher to choose files.
         * @returns {IterableIterator<FileEntry>} The found files.
         * @private
         */
        *_iterateFilesRecursive(directoryPath, options) {
            debug(`Enter the directory: ${directoryPath}`);
            const { configArrayFactory, extensionRegExp } = internalSlotsMap.get(this);
            /** @type {ConfigArray|null} */
            let config = null;
            // Enumerate the files of this directory.
            for (const filename of readdirSafeSync(directoryPath)) {
                const filePath = path.join(directoryPath, filename);
                const stat = statSafeSync(filePath); // TODO: Use `withFileTypes` in the future.
                // Check if the file is matched.
                if (stat && stat.isFile()) {
                    if (!config) {
                        config = configArrayFactory.getConfigArrayForFile(filePath, 
                        /*
                         * We must ignore `ConfigurationNotFoundError` at this
                         * point because we don't know if target files exist in
                         * this directory.
                         */
                        { ignoreNotFoundError: true });
                    }
                    const ignored = this._isIgnoredFile(filePath, { ...options, config });
                    const flag = ignored ? IGNORED_SILENTLY : NONE;
                    const matched = options.selector
                        // Started with a glob pattern; choose by the pattern.
                        ? options.selector.match(filePath)
                        // Started with a directory path; choose by file extensions.
                        : extensionRegExp.test(filePath);
                    if (matched) {
                        debug(`Yield: ${filename}${ignored ? " but ignored" : ""}`);
                        yield {
                            config: configArrayFactory.getConfigArrayForFile(filePath),
                            filePath,
                            flag
                        };
                    }
                    else {
                        debug(`Didn't match: ${filename}`);
                    }
                    // Dive into the sub directory.
                }
                else if (options.recursive && stat && stat.isDirectory()) {
                    if (!config) {
                        config = configArrayFactory.getConfigArrayForFile(filePath, { ignoreNotFoundError: true });
                    }
                    const ignored = this._isIgnoredFile(filePath + path.sep, { ...options, config });
                    if (!ignored) {
                        yield* this._iterateFilesRecursive(filePath, options);
                    }
                }
            }
            debug(`Leave the directory: ${directoryPath}`);
        }
        /**
         * Check if a given file should be ignored.
         * @param {string} filePath The path to a file to check.
         * @param {Object} options Options
         * @param {ConfigArray} [options.config] The config for this file.
         * @param {boolean} [options.dotfiles] If `true` then this is not ignore dot files by default.
         * @param {boolean} [options.direct] If `true` then this is a direct specified file.
         * @returns {boolean} `true` if the file should be ignored.
         * @private
         */
        _isIgnoredFile(filePath, { config: providedConfig, dotfiles = false, direct = false }) {
            const { configArrayFactory, defaultIgnores, ignoreFlag } = internalSlotsMap.get(this);
            if (ignoreFlag) {
                const config = providedConfig ||
                    configArrayFactory.getConfigArrayForFile(filePath, { ignoreNotFoundError: true });
                const ignores = config.extractConfig(filePath).ignores || defaultIgnores;
                return ignores(filePath, dotfiles);
            }
            return !direct && defaultIgnores(filePath, dotfiles);
        }
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    module.exports = { FileEnumerator };
},
/* glob-parent/index.js */ function _(require, module, exports) {
    var isGlob = require(568) /* is-glob */;
    var pathPosixDirname = require('path').posix.dirname;
    var isWin32 = require('os').platform() === 'win32';
    var slash = '/';
    var backslash = /\\/g;
    var enclosure = /[\{\[].*[\/]*.*[\}\]]$/;
    var globby = /(^|[^\\])([\{\[]|\([^\)]+$)/;
    var escaped = /\\([\*\?\|\[\]\(\)\{\}])/g;
    /**
     * @param {string} str
     * @param {Object} opts
     * @param {boolean} [opts.flipBackslashes=true]
     */
    module.exports = function globParent(str, opts) {
        var options = Object.assign({ flipBackslashes: true }, opts);
        // flip windows path separators
        if (options.flipBackslashes && isWin32 && str.indexOf(slash) < 0) {
            str = str.replace(backslash, slash);
        }
        // special case for strings ending in enclosure containing path separator
        if (enclosure.test(str)) {
            str += slash;
        }
        // preserves full path in case of trailing path separator
        str += 'a';
        // remove path parts that are globby
        do {
            str = pathPosixDirname(str);
        } while (isGlob(str) || globby.test(str));
        // remove escape chars and return result
        return str.replace(escaped, '$1');
    };
},
/* is-glob/index.js */ function _(require, module, exports) {
    /*!
     * is-glob <https://github.com/jonschlinkert/is-glob>
     *
     * Copyright (c) 2014-2017, Jon Schlinkert.
     * Released under the MIT License.
     */
    var isExtglob = require(569) /* is-extglob */;
    var chars = { '{': '}', '(': ')', '[': ']' };
    var strictRegex = /\\(.)|(^!|\*|[\].+)]\?|\[[^\\\]]+\]|\{[^\\}]+\}|\(\?[:!=][^\\)]+\)|\([^|]+\|[^\\)]+\))/;
    var relaxedRegex = /\\(.)|(^!|[*?{}()[\]]|\(\?)/;
    module.exports = function isGlob(str, options) {
        if (typeof str !== 'string' || str === '') {
            return false;
        }
        if (isExtglob(str)) {
            return true;
        }
        var regex = strictRegex;
        var match;
        // optionally relax regex
        if (options && options.strict === false) {
            regex = relaxedRegex;
        }
        while ((match = regex.exec(str))) {
            if (match[2])
                return true;
            var idx = match.index + match[0].length;
            // if an open bracket/brace/paren is escaped,
            // set the index to the next closing character
            var open = match[1];
            var close = open ? chars[open] : null;
            if (open && close) {
                var n = str.indexOf(close, idx);
                if (n !== -1) {
                    idx = n + 1;
                }
            }
            str = str.slice(idx);
        }
        return false;
    };
},
/* is-extglob/index.js */ function _(require, module, exports) {
    /*!
     * is-extglob <https://github.com/jonschlinkert/is-extglob>
     *
     * Copyright (c) 2014-2016, Jon Schlinkert.
     * Licensed under the MIT License.
     */
    module.exports = function isExtglob(str) {
        if (typeof str !== 'string' || str === '') {
            return false;
        }
        var match;
        while ((match = /(\\).|([@?!+*]\(.*\))/g.exec(str))) {
            if (match[2])
                return true;
            str = str.slice(match.index + match[0].length);
        }
        return false;
    };
},
/* eslint/lib/cli-engine/hash.js */ function _(require, module, exports) {
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const murmur = require(571) /* imurmurhash */;
    //------------------------------------------------------------------------------
    // Helpers
    //------------------------------------------------------------------------------
    //------------------------------------------------------------------------------
    // Private
    //------------------------------------------------------------------------------
    /**
     * hash the given string
     * @param  {string} str the string to hash
     * @returns {string}    the hash
     */
    function hash(str) {
        return murmur(str).result().toString(36);
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    module.exports = hash;
},
/* imurmurhash/imurmurhash.js */ function _(require, module, exports) {
    /**
     * @preserve
     * JS Implementation of incremental MurmurHash3 (r150) (as of May 10, 2013)
     *
     * @author <a href="mailto:jensyt@gmail.com">Jens Taylor</a>
     * @see http://github.com/homebrewing/brauhaus-diff
     * @author <a href="mailto:gary.court@gmail.com">Gary Court</a>
     * @see http://github.com/garycourt/murmurhash-js
     * @author <a href="mailto:aappleby@gmail.com">Austin Appleby</a>
     * @see http://sites.google.com/site/murmurhash/
     */
    (function () {
        var cache;
        // Call this function without `new` to use the cached object (good for
        // single-threaded environments), or with `new` to create a new object.
        //
        // @param {string} key A UTF-16 or ASCII string
        // @param {number} seed An optional positive integer
        // @return {object} A MurmurHash3 object for incremental hashing
        function MurmurHash3(key, seed) {
            var m = this instanceof MurmurHash3 ? this : cache;
            m.reset(seed);
            if (typeof key === 'string' && key.length > 0) {
                m.hash(key);
            }
            if (m !== this) {
                return m;
            }
        }
        ;
        // Incrementally add a string to this hash
        //
        // @param {string} key A UTF-16 or ASCII string
        // @return {object} this
        MurmurHash3.prototype.hash = function (key) {
            var h1, k1, i, top, len;
            len = key.length;
            this.len += len;
            k1 = this.k1;
            i = 0;
            switch (this.rem) {
                case 0: k1 ^= len > i ? (key.charCodeAt(i++) & 0xffff) : 0;
                case 1: k1 ^= len > i ? (key.charCodeAt(i++) & 0xffff) << 8 : 0;
                case 2: k1 ^= len > i ? (key.charCodeAt(i++) & 0xffff) << 16 : 0;
                case 3:
                    k1 ^= len > i ? (key.charCodeAt(i) & 0xff) << 24 : 0;
                    k1 ^= len > i ? (key.charCodeAt(i++) & 0xff00) >> 8 : 0;
            }
            this.rem = (len + this.rem) & 3; // & 3 is same as % 4
            len -= this.rem;
            if (len > 0) {
                h1 = this.h1;
                while (1) {
                    k1 = (k1 * 0x2d51 + (k1 & 0xffff) * 0xcc9e0000) & 0xffffffff;
                    k1 = (k1 << 15) | (k1 >>> 17);
                    k1 = (k1 * 0x3593 + (k1 & 0xffff) * 0x1b870000) & 0xffffffff;
                    h1 ^= k1;
                    h1 = (h1 << 13) | (h1 >>> 19);
                    h1 = (h1 * 5 + 0xe6546b64) & 0xffffffff;
                    if (i >= len) {
                        break;
                    }
                    k1 = ((key.charCodeAt(i++) & 0xffff)) ^
                        ((key.charCodeAt(i++) & 0xffff) << 8) ^
                        ((key.charCodeAt(i++) & 0xffff) << 16);
                    top = key.charCodeAt(i++);
                    k1 ^= ((top & 0xff) << 24) ^
                        ((top & 0xff00) >> 8);
                }
                k1 = 0;
                switch (this.rem) {
                    case 3: k1 ^= (key.charCodeAt(i + 2) & 0xffff) << 16;
                    case 2: k1 ^= (key.charCodeAt(i + 1) & 0xffff) << 8;
                    case 1: k1 ^= (key.charCodeAt(i) & 0xffff);
                }
                this.h1 = h1;
            }
            this.k1 = k1;
            return this;
        };
        // Get the result of this hash
        //
        // @return {number} The 32-bit hash
        MurmurHash3.prototype.result = function () {
            var k1, h1;
            k1 = this.k1;
            h1 = this.h1;
            if (k1 > 0) {
                k1 = (k1 * 0x2d51 + (k1 & 0xffff) * 0xcc9e0000) & 0xffffffff;
                k1 = (k1 << 15) | (k1 >>> 17);
                k1 = (k1 * 0x3593 + (k1 & 0xffff) * 0x1b870000) & 0xffffffff;
                h1 ^= k1;
            }
            h1 ^= this.len;
            h1 ^= h1 >>> 16;
            h1 = (h1 * 0xca6b + (h1 & 0xffff) * 0x85eb0000) & 0xffffffff;
            h1 ^= h1 >>> 13;
            h1 = (h1 * 0xae35 + (h1 & 0xffff) * 0xc2b20000) & 0xffffffff;
            h1 ^= h1 >>> 16;
            return h1 >>> 0;
        };
        // Reset the hash object for reuse
        //
        // @param {number} seed An optional positive integer
        MurmurHash3.prototype.reset = function (seed) {
            this.h1 = typeof seed === 'number' ? seed : 0;
            this.rem = this.k1 = this.len = 0;
            return this;
        };
        // A cached object to use. This can be safely used if you're in a single-
        // threaded environment, otherwise you need to create new hashes to use.
        cache = new MurmurHash3();
        if (typeof (module) != 'undefined') {
            module.exports = MurmurHash3;
        }
        else {
            this.MurmurHash3 = MurmurHash3;
        }
    }());
},
/* eslint/lib/cli-engine/lint-result-cache.js */ function _(require, module, exports) {
    //-----------------------------------------------------------------------------
    // Requirements
    //-----------------------------------------------------------------------------
    const assert = require("assert");
    const fs = require("fs");
    const fileEntryCache = require(573) /* file-entry-cache */;
    const stringify = require(592) /* json-stable-stringify-without-jsonify */;
    const pkg = require(81) /* ../../package.json */;
    const hash = require(570) /* ./hash */;
    //-----------------------------------------------------------------------------
    // Helpers
    //-----------------------------------------------------------------------------
    const configHashCache = new WeakMap();
    const nodeVersion = process && process.version;
    /**
     * Calculates the hash of the config
     * @param {ConfigArray} config The config.
     * @returns {string} The hash of the config
     */
    function hashOfConfigFor(config) {
        if (!configHashCache.has(config)) {
            configHashCache.set(config, hash(`${pkg.version}_${nodeVersion}_${stringify(config)}`));
        }
        return configHashCache.get(config);
    }
    //-----------------------------------------------------------------------------
    // Public Interface
    //-----------------------------------------------------------------------------
    /**
     * Lint result cache. This wraps around the file-entry-cache module,
     * transparently removing properties that are difficult or expensive to
     * serialize and adding them back in on retrieval.
     */
    class LintResultCache {
        /**
         * Creates a new LintResultCache instance.
         * @param {string} cacheFileLocation The cache file location.
         *   configuration lookup by file path).
         */
        constructor(cacheFileLocation) {
            assert(cacheFileLocation, "Cache file location is required");
            this.fileEntryCache = fileEntryCache.create(cacheFileLocation);
        }
        /**
         * Retrieve cached lint results for a given file path, if present in the
         * cache. If the file is present and has not been changed, rebuild any
         * missing result information.
         * @param {string} filePath The file for which to retrieve lint results.
         * @param {ConfigArray} config The config of the file.
         * @returns {Object|null} The rebuilt lint results, or null if the file is
         *   changed or not in the filesystem.
         */
        getCachedLintResults(filePath, config) {
            /*
             * Cached lint results are valid if and only if:
             * 1. The file is present in the filesystem
             * 2. The file has not changed since the time it was previously linted
             * 3. The ESLint configuration has not changed since the time the file
             *    was previously linted
             * If any of these are not true, we will not reuse the lint results.
             */
            const fileDescriptor = this.fileEntryCache.getFileDescriptor(filePath);
            const hashOfConfig = hashOfConfigFor(config);
            const changed = fileDescriptor.changed || fileDescriptor.meta.hashOfConfig !== hashOfConfig;
            if (fileDescriptor.notFound || changed) {
                return null;
            }
            // If source is present but null, need to reread the file from the filesystem.
            if (fileDescriptor.meta.results && fileDescriptor.meta.results.source === null) {
                fileDescriptor.meta.results.source = fs.readFileSync(filePath, "utf-8");
            }
            return fileDescriptor.meta.results;
        }
        /**
         * Set the cached lint results for a given file path, after removing any
         * information that will be both unnecessary and difficult to serialize.
         * Avoids caching results with an "output" property (meaning fixes were
         * applied), to prevent potentially incorrect results if fixes are not
         * written to disk.
         * @param {string} filePath The file for which to set lint results.
         * @param {ConfigArray} config The config of the file.
         * @param {Object} result The lint result to be set for the file.
         * @returns {void}
         */
        setCachedLintResults(filePath, config, result) {
            if (result && Object.prototype.hasOwnProperty.call(result, "output")) {
                return;
            }
            const fileDescriptor = this.fileEntryCache.getFileDescriptor(filePath);
            if (fileDescriptor && !fileDescriptor.notFound) {
                // Serialize the result, except that we want to remove the file source if present.
                const resultToSerialize = Object.assign({}, result);
                /*
                 * Set result.source to null.
                 * In `getCachedLintResults`, if source is explicitly null, we will
                 * read the file from the filesystem to set the value again.
                 */
                if (Object.prototype.hasOwnProperty.call(resultToSerialize, "source")) {
                    resultToSerialize.source = null;
                }
                fileDescriptor.meta.results = resultToSerialize;
                fileDescriptor.meta.hashOfConfig = hashOfConfigFor(config);
            }
        }
        /**
         * Persists the in-memory cache to disk.
         * @returns {void}
         */
        reconcile() {
            this.fileEntryCache.reconcile();
        }
    }
    module.exports = LintResultCache;
},
/* file-entry-cache/cache.js */ function _(require, module, exports) {
    var path = require('path');
    var crypto = require('crypto');
    module.exports = {
        createFromFile: function (filePath, useChecksum) {
            var fname = path.basename(filePath);
            var dir = path.dirname(filePath);
            return this.create(fname, dir, useChecksum);
        },
        create: function (cacheId, _path, useChecksum) {
            var fs = require('fs');
            var flatCache = require(574) /* flat-cache */;
            var cache = flatCache.load(cacheId, _path);
            var normalizedEntries = {};
            var removeNotFoundFiles = function removeNotFoundFiles() {
                const cachedEntries = cache.keys();
                // remove not found entries
                cachedEntries.forEach(function remover(fPath) {
                    try {
                        fs.statSync(fPath);
                    }
                    catch (err) {
                        if (err.code === 'ENOENT') {
                            cache.removeKey(fPath);
                        }
                    }
                });
            };
            removeNotFoundFiles();
            return {
                /**
                 * the flat cache storage used to persist the metadata of the `files
                 * @type {Object}
                 */
                cache: cache,
                /**
                 * Given a buffer, calculate md5 hash of its content.
                 * @method getHash
                 * @param  {Buffer} buffer   buffer to calculate hash on
                 * @return {String}          content hash digest
                 */
                getHash: function (buffer) {
                    return crypto
                        .createHash('md5')
                        .update(buffer)
                        .digest('hex');
                },
                /**
                 * Return whether or not a file has changed since last time reconcile was called.
                 * @method hasFileChanged
                 * @param  {String}  file  the filepath to check
                 * @return {Boolean}       wheter or not the file has changed
                 */
                hasFileChanged: function (file) {
                    return this.getFileDescriptor(file).changed;
                },
                /**
                 * given an array of file paths it return and object with three arrays:
                 *  - changedFiles: Files that changed since previous run
                 *  - notChangedFiles: Files that haven't change
                 *  - notFoundFiles: Files that were not found, probably deleted
                 *
                 * @param  {Array} files the files to analyze and compare to the previous seen files
                 * @return {[type]}       [description]
                 */
                analyzeFiles: function (files) {
                    var me = this;
                    files = files || [];
                    var res = {
                        changedFiles: [],
                        notFoundFiles: [],
                        notChangedFiles: []
                    };
                    me.normalizeEntries(files).forEach(function (entry) {
                        if (entry.changed) {
                            res.changedFiles.push(entry.key);
                            return;
                        }
                        if (entry.notFound) {
                            res.notFoundFiles.push(entry.key);
                            return;
                        }
                        res.notChangedFiles.push(entry.key);
                    });
                    return res;
                },
                getFileDescriptor: function (file) {
                    var fstat;
                    try {
                        fstat = fs.statSync(file);
                    }
                    catch (ex) {
                        this.removeEntry(file);
                        return { key: file, notFound: true, err: ex };
                    }
                    if (useChecksum) {
                        return this._getFileDescriptorUsingChecksum(file);
                    }
                    return this._getFileDescriptorUsingMtimeAndSize(file, fstat);
                },
                _getFileDescriptorUsingMtimeAndSize: function (file, fstat) {
                    var meta = cache.getKey(file);
                    var cacheExists = !!meta;
                    var cSize = fstat.size;
                    var cTime = fstat.mtime.getTime();
                    var isDifferentDate;
                    var isDifferentSize;
                    if (!meta) {
                        meta = { size: cSize, mtime: cTime };
                    }
                    else {
                        isDifferentDate = cTime !== meta.mtime;
                        isDifferentSize = cSize !== meta.size;
                    }
                    var nEntry = normalizedEntries[file] = {
                        key: file,
                        changed: !cacheExists || isDifferentDate || isDifferentSize,
                        meta: meta
                    };
                    return nEntry;
                },
                _getFileDescriptorUsingChecksum: function (file) {
                    var meta = cache.getKey(file);
                    var cacheExists = !!meta;
                    var contentBuffer;
                    try {
                        contentBuffer = fs.readFileSync(file);
                    }
                    catch (ex) {
                        contentBuffer = '';
                    }
                    var isDifferent = true;
                    var hash = this.getHash(contentBuffer);
                    if (!meta) {
                        meta = { hash: hash };
                    }
                    else {
                        isDifferent = hash !== meta.hash;
                    }
                    var nEntry = normalizedEntries[file] = {
                        key: file,
                        changed: !cacheExists || isDifferent,
                        meta: meta
                    };
                    return nEntry;
                },
                /**
                 * Return the list o the files that changed compared
                 * against the ones stored in the cache
                 *
                 * @method getUpdated
                 * @param files {Array} the array of files to compare against the ones in the cache
                 * @returns {Array}
                 */
                getUpdatedFiles: function (files) {
                    var me = this;
                    files = files || [];
                    return me.normalizeEntries(files).filter(function (entry) {
                        return entry.changed;
                    }).map(function (entry) {
                        return entry.key;
                    });
                },
                /**
                 * return the list of files
                 * @method normalizeEntries
                 * @param files
                 * @returns {*}
                 */
                normalizeEntries: function (files) {
                    files = files || [];
                    var me = this;
                    var nEntries = files.map(function (file) {
                        return me.getFileDescriptor(file);
                    });
                    //normalizeEntries = nEntries;
                    return nEntries;
                },
                /**
                 * Remove an entry from the file-entry-cache. Useful to force the file to still be considered
                 * modified the next time the process is run
                 *
                 * @method removeEntry
                 * @param entryName
                 */
                removeEntry: function (entryName) {
                    delete normalizedEntries[entryName];
                    cache.removeKey(entryName);
                },
                /**
                 * Delete the cache file from the disk
                 * @method deleteCacheFile
                 */
                deleteCacheFile: function () {
                    cache.removeCacheFile();
                },
                /**
                 * remove the cache from the file and clear the memory cache
                 */
                destroy: function () {
                    normalizedEntries = {};
                    cache.destroy();
                },
                _getMetaForFileUsingCheckSum: function (cacheEntry) {
                    var contentBuffer = fs.readFileSync(cacheEntry.key);
                    var hash = this.getHash(contentBuffer);
                    var meta = Object.assign(cacheEntry.meta, { hash: hash });
                    return meta;
                },
                _getMetaForFileUsingMtimeAndSize: function (cacheEntry) {
                    var stat = fs.statSync(cacheEntry.key);
                    var meta = Object.assign(cacheEntry.meta, {
                        size: stat.size,
                        mtime: stat.mtime.getTime()
                    });
                    return meta;
                },
                /**
                 * Sync the files and persist them to the cache
                 * @method reconcile
                 */
                reconcile: function (noPrune) {
                    removeNotFoundFiles();
                    noPrune = typeof noPrune === 'undefined' ? true : noPrune;
                    var entries = normalizedEntries;
                    var keys = Object.keys(entries);
                    if (keys.length === 0) {
                        return;
                    }
                    var me = this;
                    keys.forEach(function (entryName) {
                        var cacheEntry = entries[entryName];
                        try {
                            var meta = useChecksum ? me._getMetaForFileUsingCheckSum(cacheEntry) : me._getMetaForFileUsingMtimeAndSize(cacheEntry);
                            cache.setKey(entryName, meta);
                        }
                        catch (err) {
                            // if the file does not exists we don't save it
                            // other errors are just thrown
                            if (err.code !== 'ENOENT') {
                                throw err;
                            }
                        }
                    });
                    cache.save(noPrune);
                }
            };
        }
    };
},
/* flat-cache/cache.js */ function _(require, module, exports) {
    var path = require('path');
    var fs = require('fs');
    var utils = require(575) /* ./utils */;
    var del = require(579) /* ./del */;
    var writeJSON = utils.writeJSON;
    var cache = {
        /**
         * Load a cache identified by the given Id. If the element does not exists, then initialize an empty
         * cache storage. If specified `cacheDir` will be used as the directory to persist the data to. If omitted
         * then the cache module directory `./cache` will be used instead
         *
         * @method load
         * @param docId {String} the id of the cache, would also be used as the name of the file cache
         * @param [cacheDir] {String} directory for the cache entry
         */
        load: function (docId, cacheDir) {
            var me = this;
            me._visited = {};
            me._persisted = {};
            me._pathToFile = cacheDir ? path.resolve(cacheDir, docId) : path.resolve(__dirname, './.cache/', docId);
            if (fs.existsSync(me._pathToFile)) {
                me._persisted = utils.tryParse(me._pathToFile, {});
            }
        },
        /**
         * Load the cache from the provided file
         * @method loadFile
         * @param  {String} pathToFile the path to the file containing the info for the cache
         */
        loadFile: function (pathToFile) {
            var me = this;
            var dir = path.dirname(pathToFile);
            var fName = path.basename(pathToFile);
            me.load(fName, dir);
        },
        /**
         * Returns the entire persisted object
         * @method all
         * @returns {*}
         */
        all: function () {
            return this._persisted;
        },
        keys: function () {
            return Object.keys(this._persisted);
        },
        /**
         * sets a key to a given value
         * @method setKey
         * @param key {string} the key to set
         * @param value {object} the value of the key. Could be any object that can be serialized with JSON.stringify
         */
        setKey: function (key, value) {
            this._visited[key] = true;
            this._persisted[key] = value;
        },
        /**
         * remove a given key from the cache
         * @method removeKey
         * @param key {String} the key to remove from the object
         */
        removeKey: function (key) {
            delete this._visited[key]; // esfmt-ignore-line
            delete this._persisted[key]; // esfmt-ignore-line
        },
        /**
         * Return the value of the provided key
         * @method getKey
         * @param key {String} the name of the key to retrieve
         * @returns {*} the value from the key
         */
        getKey: function (key) {
            this._visited[key] = true;
            return this._persisted[key];
        },
        /**
         * Remove keys that were not accessed/set since the
         * last time the `prune` method was called.
         * @method _prune
         * @private
         */
        _prune: function () {
            var me = this;
            var obj = {};
            var keys = Object.keys(me._visited);
            // no keys visited for either get or set value
            if (keys.length === 0) {
                return;
            }
            keys.forEach(function (key) {
                obj[key] = me._persisted[key];
            });
            me._visited = {};
            me._persisted = obj;
        },
        /**
         * Save the state of the cache identified by the docId to disk
         * as a JSON structure
         * @param [noPrune=false] {Boolean} whether to remove from cache the non visited files
         * @method save
         */
        save: function (noPrune) {
            var me = this;
            (!noPrune) && me._prune();
            writeJSON(me._pathToFile, me._persisted);
        },
        /**
         * remove the file where the cache is persisted
         * @method removeCacheFile
         * @return {Boolean} true or false if the file was successfully deleted
         */
        removeCacheFile: function () {
            return del(this._pathToFile);
        },
        /**
         * Destroy the file cache and cache content.
         * @method destroy
         */
        destroy: function () {
            var me = this;
            me._visited = {};
            me._persisted = {};
            me.removeCacheFile();
        }
    };
    module.exports = {
        /**
         * Alias for create. Should be considered depreacted. Will be removed in next releases
         *
         * @method load
         * @param docId {String} the id of the cache, would also be used as the name of the file cache
         * @param [cacheDir] {String} directory for the cache entry
         * @returns {cache} cache instance
         */
        load: function (docId, cacheDir) {
            return this.create(docId, cacheDir);
        },
        /**
        * Load a cache identified by the given Id. If the element does not exists, then initialize an empty
        * cache storage.
        *
        * @method create
        * @param docId {String} the id of the cache, would also be used as the name of the file cache
        * @param [cacheDir] {String} directory for the cache entry
        * @returns {cache} cache instance
        */
        create: function (docId, cacheDir) {
            var obj = Object.create(cache);
            obj.load(docId, cacheDir);
            return obj;
        },
        createFromFile: function (filePath) {
            var obj = Object.create(cache);
            obj.loadFile(filePath);
            return obj;
        },
        /**
         * Clear the cache identified by the given id. Caches stored in a different cache directory can be deleted directly
         *
         * @method clearCache
         * @param docId {String} the id of the cache, would also be used as the name of the file cache
         * @param cacheDir {String} the directory where the cache file was written
         * @returns {Boolean} true if the cache folder was deleted. False otherwise
         */
        clearCacheById: function (docId, cacheDir) {
            var filePath = cacheDir ? path.resolve(cacheDir, docId) : path.resolve(__dirname, './.cache/', docId);
            return del(filePath);
        },
        /**
         * Remove all cache stored in the cache directory
         * @method clearAll
         * @returns {Boolean} true if the cache folder was deleted. False otherwise
         */
        clearAll: function (cacheDir) {
            var filePath = cacheDir ? path.resolve(cacheDir) : path.resolve(__dirname, './.cache/');
            return del(filePath);
        }
    };
},
/* flat-cache/utils.js */ function _(require, module, exports) {
    var fs = require('fs');
    var write = require(576) /* write */;
    var flatted = require(578) /* flatted */;
    module.exports = {
        tryParse: function (filePath, defaultValue) {
            var result;
            try {
                result = this.readJSON(filePath);
            }
            catch (ex) {
                result = defaultValue;
            }
            return result;
        },
        /**
         * Read json file synchronously using flatted
         *
         * @method readJSON
         * @param  {String} filePath Json filepath
         * @returns {*} parse result
         */
        readJSON: function (filePath) {
            return flatted.parse(fs.readFileSync(filePath, {
                encoding: 'utf8'
            }));
        },
        /**
         * Write json file synchronously using circular-json
         *
         * @method writeJSON
         * @param  {String} filePath Json filepath
         * @param  {*} data Object to serialize
         */
        writeJSON: function (filePath, data) {
            write.sync(filePath, flatted.stringify(data));
        }
    };
},
/* write/index.js */ function _(require, module, exports) {
    var fs = require('fs');
    var path = require('path');
    var mkdirp = require(577) /* mkdirp */;
    /**
     * Asynchronously writes data to a file, replacing the file if it already
     * exists and creating any intermediate directories if they don't already
     * exist. Data can be a string or a buffer. Returns a promise if a callback
     * function is not passed.
     *
     * ```js
     * var writeFile = require('write');
     * writeFile('foo.txt', 'This is content...', function(err) {
     *   if (err) console.log(err);
     * });
     *
     * // promise
     * writeFile('foo.txt', 'This is content...')
     *   .then(function() {
     *     // do stuff
     *   });
     * ```
     * @name writeFile
     * @param {string|Buffer|integer} `filepath` filepath or file descriptor.
     * @param {string|Buffer|Uint8Array} `data` String to write to disk.
     * @param {object} `options` Options to pass to [fs.writeFile][fs]{#fs_fs_writefile_file_data_options_callback} and/or [mkdirp][]
     * @param {Function} `callback` (optional) If no callback is provided, a promise is returned.
     * @api public
     */
    function writeFile(filepath, data, options, cb) {
        if (typeof options === 'function') {
            cb = options;
            options = {};
        }
        if (typeof cb !== 'function') {
            return writeFile.promise.apply(null, arguments);
        }
        if (typeof filepath !== 'string') {
            cb(new TypeError('expected filepath to be a string'));
            return;
        }
        mkdirp(path.dirname(filepath), options, function (err) {
            if (err) {
                cb(err);
                return;
            }
            fs.writeFile(filepath, data, options, cb);
        });
    }
    ;
    /**
     * The promise version of [writeFile](#writefile). Returns a promise.
     *
     * ```js
     * var writeFile = require('write');
     * writeFile.promise('foo.txt', 'This is content...')
     *   .then(function() {
     *     // do stuff
     *   });
     * ```
     * @name .promise
     * @param {string|Buffer|integer} `filepath` filepath or file descriptor.
     * @param {string|Buffer|Uint8Array} `val` String or buffer to write to disk.
     * @param {object} `options` Options to pass to [fs.writeFile][fs]{#fs_fs_writefile_file_data_options_callback} and/or [mkdirp][]
     * @return {Promise}
     * @api public
     */
    writeFile.promise = function (filepath, val, options) {
        if (typeof filepath !== 'string') {
            return Promise.reject(new TypeError('expected filepath to be a string'));
        }
        return new Promise(function (resolve, reject) {
            mkdirp(path.dirname(filepath), options, function (err) {
                if (err) {
                    reject(err);
                    return;
                }
                fs.writeFile(filepath, val, options, function (err) {
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve(val);
                });
            });
        });
    };
    /**
     * The synchronous version of [writeFile](#writefile). Returns undefined.
     *
     * ```js
     * var writeFile = require('write');
     * writeFile.sync('foo.txt', 'This is content...');
     * ```
     * @name .sync
     * @param {string|Buffer|integer} `filepath` filepath or file descriptor.
     * @param {string|Buffer|Uint8Array} `data` String or buffer to write to disk.
     * @param {object} `options` Options to pass to [fs.writeFileSync][fs]{#fs_fs_writefilesync_file_data_options} and/or [mkdirp][]
     * @return {undefined}
     * @api public
     */
    writeFile.sync = function (filepath, data, options) {
        if (typeof filepath !== 'string') {
            throw new TypeError('expected filepath to be a string');
        }
        mkdirp.sync(path.dirname(filepath), options);
        fs.writeFileSync(filepath, data, options);
    };
    /**
     * Uses `fs.createWriteStream` to write data to a file, replacing the
     * file if it already exists and creating any intermediate directories
     * if they don't already exist. Data can be a string or a buffer. Returns
     * a new [WriteStream](https://nodejs.org/api/fs.html#fs_class_fs_writestream)
     * object.
     *
     * ```js
     * var fs = require('fs');
     * var writeFile = require('write');
     * fs.createReadStream('README.md')
     *   .pipe(writeFile.stream('a/b/c/other-file.md'))
     *   .on('close', function() {
     *     // do stuff
     *   });
     * ```
     * @name .stream
     * @param {string|Buffer|integer} `filepath` filepath or file descriptor.
     * @param {object} `options` Options to pass to [mkdirp][] and [fs.createWriteStream][fs]{#fs_fs_createwritestream_path_options}
     * @return {Stream} Returns a new [WriteStream](https://nodejs.org/api/fs.html#fs_class_fs_writestream) object. (See [Writable Stream](https://nodejs.org/api/stream.html#stream_class_stream_writable)).
     * @api public
     */
    writeFile.stream = function (filepath, options) {
        mkdirp.sync(path.dirname(filepath), options);
        return fs.createWriteStream(filepath, options);
    };
    /**
     * Expose `writeFile`
     */
    module.exports = writeFile;
},
/* mkdirp/index.js */ function _(require, module, exports) {
    var path = require('path');
    var fs = require('fs');
    var _0777 = parseInt('0777', 8);
    module.exports = mkdirP.mkdirp = mkdirP.mkdirP = mkdirP;
    function mkdirP(p, opts, f, made) {
        if (typeof opts === 'function') {
            f = opts;
            opts = {};
        }
        else if (!opts || typeof opts !== 'object') {
            opts = { mode: opts };
        }
        var mode = opts.mode;
        var xfs = opts.fs || fs;
        if (mode === undefined) {
            mode = _0777 & (~process.umask());
        }
        if (!made)
            made = null;
        var cb = f || function () { };
        p = path.resolve(p);
        xfs.mkdir(p, mode, function (er) {
            if (!er) {
                made = made || p;
                return cb(null, made);
            }
            switch (er.code) {
                case 'ENOENT':
                    mkdirP(path.dirname(p), opts, function (er, made) {
                        if (er)
                            cb(er, made);
                        else
                            mkdirP(p, opts, cb, made);
                    });
                    break;
                // In the case of any other error, just see if there's a dir
                // there already.  If so, then hooray!  If not, then something
                // is borked.
                default:
                    xfs.stat(p, function (er2, stat) {
                        // if the stat fails, then that's super weird.
                        // let the original error be the failure reason.
                        if (er2 || !stat.isDirectory())
                            cb(er, made);
                        else
                            cb(null, made);
                    });
                    break;
            }
        });
    }
    mkdirP.sync = function sync(p, opts, made) {
        if (!opts || typeof opts !== 'object') {
            opts = { mode: opts };
        }
        var mode = opts.mode;
        var xfs = opts.fs || fs;
        if (mode === undefined) {
            mode = _0777 & (~process.umask());
        }
        if (!made)
            made = null;
        p = path.resolve(p);
        try {
            xfs.mkdirSync(p, mode);
            made = made || p;
        }
        catch (err0) {
            switch (err0.code) {
                case 'ENOENT':
                    made = sync(path.dirname(p), opts, made);
                    sync(p, opts, made);
                    break;
                // In the case of any other error, just see if there's a dir
                // there already.  If so, then hooray!  If not, then something
                // is borked.
                default:
                    var stat;
                    try {
                        stat = xfs.statSync(p);
                    }
                    catch (err1) {
                        throw err0;
                    }
                    if (!stat.isDirectory())
                        throw err0;
                    break;
            }
        }
        return made;
    };
},
/* flatted/cjs/index.js */ function _(require, module, exports) {
    var Flatted = (function (Primitive, primitive) {
        /*!
         * ISC License
         *
         * Copyright (c) 2018, Andrea Giammarchi, @WebReflection
         *
         * Permission to use, copy, modify, and/or distribute this software for any
         * purpose with or without fee is hereby granted, provided that the above
         * copyright notice and this permission notice appear in all copies.
         *
         * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
         * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
         * AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
         * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
         * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
         * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
         * PERFORMANCE OF THIS SOFTWARE.
         */
        var Flatted = {
            parse: function parse(text, reviver) {
                var input = JSON.parse(text, Primitives).map(primitives);
                var value = input[0];
                var $ = reviver || noop;
                var tmp = typeof value === 'object' && value ?
                    revive(input, new Set, value, $) :
                    value;
                return $.call({ '': tmp }, '', tmp);
            },
            stringify: function stringify(value, replacer, space) {
                for (var firstRun, known = new Map, input = [], output = [], $ = replacer && typeof replacer === typeof input ?
                    function (k, v) {
                        if (k === '' || -1 < replacer.indexOf(k))
                            return v;
                    } :
                    (replacer || noop), i = +set(known, input, $.call({ '': value }, '', value)), replace = function (key, value) {
                    if (firstRun) {
                        firstRun = !firstRun;
                        return value;
                        // this was invoking twice each root object
                        // return i < 1 ? value : $.call(this, key, value);
                    }
                    var after = $.call(this, key, value);
                    switch (typeof after) {
                        case 'object':
                            if (after === null)
                                return after;
                        case primitive:
                            return known.get(after) || set(known, input, after);
                    }
                    return after;
                }; i < input.length; i++) {
                    firstRun = true;
                    output[i] = JSON.stringify(input[i], replace, space);
                }
                return '[' + output.join(',') + ']';
            }
        };
        return Flatted;
        function noop(key, value) {
            return value;
        }
        function revive(input, parsed, output, $) {
            return Object.keys(output).reduce(function (output, key) {
                var value = output[key];
                if (value instanceof Primitive) {
                    var tmp = input[value];
                    if (typeof tmp === 'object' && !parsed.has(tmp)) {
                        parsed.add(tmp);
                        output[key] = $.call(output, key, revive(input, parsed, tmp, $));
                    }
                    else {
                        output[key] = $.call(output, key, tmp);
                    }
                }
                else
                    output[key] = $.call(output, key, value);
                return output;
            }, output);
        }
        function set(known, input, value) {
            var index = Primitive(input.push(value) - 1);
            known.set(value, index);
            return index;
        }
        // the two kinds of primitives
        //  1. the real one
        //  2. the wrapped one
        function primitives(value) {
            return value instanceof Primitive ? Primitive(value) : value;
        }
        function Primitives(key, value) {
            return typeof value === primitive ? new Primitive(value) : value;
        }
    }(String, 'string'));
    module.exports = Flatted;
},
/* flat-cache/del.js */ function _(require, module, exports) {
    var rimraf = require(580) /* rimraf */.sync;
    var fs = require('fs');
    module.exports = function del(file) {
        if (fs.existsSync(file)) {
            //if rimraf doesn't throw then the file has been deleted or didn't exist
            rimraf(file, {
                glob: false
            });
            return true;
        }
        return false;
    };
},
/* rimraf/rimraf.js */ function _(require, module, exports) {
    const assert = require("assert");
    const path = require("path");
    const fs = require("fs");
    let glob = undefined;
    try {
        glob = require(581) /* glob */;
    }
    catch (_err) {
        // treat glob as optional.
    }
    const defaultGlobOpts = {
        nosort: true,
        silent: true
    };
    // for EMFILE handling
    let timeout = 0;
    const isWindows = (process.platform === "win32");
    const defaults = options => {
        const methods = [
            'unlink',
            'chmod',
            'stat',
            'lstat',
            'rmdir',
            'readdir'
        ];
        methods.forEach(m => {
            options[m] = options[m] || fs[m];
            m = m + 'Sync';
            options[m] = options[m] || fs[m];
        });
        options.maxBusyTries = options.maxBusyTries || 3;
        options.emfileWait = options.emfileWait || 1000;
        if (options.glob === false) {
            options.disableGlob = true;
        }
        if (options.disableGlob !== true && glob === undefined) {
            throw Error('glob dependency not found, set `options.disableGlob = true` if intentional');
        }
        options.disableGlob = options.disableGlob || false;
        options.glob = options.glob || defaultGlobOpts;
    };
    const rimraf = (p, options, cb) => {
        if (typeof options === 'function') {
            cb = options;
            options = {};
        }
        assert(p, 'rimraf: missing path');
        assert.equal(typeof p, 'string', 'rimraf: path should be a string');
        assert.equal(typeof cb, 'function', 'rimraf: callback function required');
        assert(options, 'rimraf: invalid options argument provided');
        assert.equal(typeof options, 'object', 'rimraf: options should be object');
        defaults(options);
        let busyTries = 0;
        let errState = null;
        let n = 0;
        const next = (er) => {
            errState = errState || er;
            if (--n === 0)
                cb(errState);
        };
        const afterGlob = (er, results) => {
            if (er)
                return cb(er);
            n = results.length;
            if (n === 0)
                return cb();
            results.forEach(p => {
                const CB = (er) => {
                    if (er) {
                        if ((er.code === "EBUSY" || er.code === "ENOTEMPTY" || er.code === "EPERM") &&
                            busyTries < options.maxBusyTries) {
                            busyTries++;
                            // try again, with the same exact callback as this one.
                            return setTimeout(() => rimraf_(p, options, CB), busyTries * 100);
                        }
                        // this one won't happen if graceful-fs is used.
                        if (er.code === "EMFILE" && timeout < options.emfileWait) {
                            return setTimeout(() => rimraf_(p, options, CB), timeout++);
                        }
                        // already gone
                        if (er.code === "ENOENT")
                            er = null;
                    }
                    timeout = 0;
                    next(er);
                };
                rimraf_(p, options, CB);
            });
        };
        if (options.disableGlob || !glob.hasMagic(p))
            return afterGlob(null, [p]);
        options.lstat(p, (er, stat) => {
            if (!er)
                return afterGlob(null, [p]);
            glob(p, options.glob, afterGlob);
        });
    };
    // Two possible strategies.
    // 1. Assume it's a file.  unlink it, then do the dir stuff on EPERM or EISDIR
    // 2. Assume it's a directory.  readdir, then do the file stuff on ENOTDIR
    //
    // Both result in an extra syscall when you guess wrong.  However, there
    // are likely far more normal files in the world than directories.  This
    // is based on the assumption that a the average number of files per
    // directory is >= 1.
    //
    // If anyone ever complains about this, then I guess the strategy could
    // be made configurable somehow.  But until then, YAGNI.
    const rimraf_ = (p, options, cb) => {
        assert(p);
        assert(options);
        assert(typeof cb === 'function');
        // sunos lets the root user unlink directories, which is... weird.
        // so we have to lstat here and make sure it's not a dir.
        options.lstat(p, (er, st) => {
            if (er && er.code === "ENOENT")
                return cb(null);
            // Windows can EPERM on stat.  Life is suffering.
            if (er && er.code === "EPERM" && isWindows)
                fixWinEPERM(p, options, er, cb);
            if (st && st.isDirectory())
                return rmdir(p, options, er, cb);
            options.unlink(p, er => {
                if (er) {
                    if (er.code === "ENOENT")
                        return cb(null);
                    if (er.code === "EPERM")
                        return (isWindows)
                            ? fixWinEPERM(p, options, er, cb)
                            : rmdir(p, options, er, cb);
                    if (er.code === "EISDIR")
                        return rmdir(p, options, er, cb);
                }
                return cb(er);
            });
        });
    };
    const fixWinEPERM = (p, options, er, cb) => {
        assert(p);
        assert(options);
        assert(typeof cb === 'function');
        options.chmod(p, 0o666, er2 => {
            if (er2)
                cb(er2.code === "ENOENT" ? null : er);
            else
                options.stat(p, (er3, stats) => {
                    if (er3)
                        cb(er3.code === "ENOENT" ? null : er);
                    else if (stats.isDirectory())
                        rmdir(p, options, er, cb);
                    else
                        options.unlink(p, cb);
                });
        });
    };
    const fixWinEPERMSync = (p, options, er) => {
        assert(p);
        assert(options);
        try {
            options.chmodSync(p, 0o666);
        }
        catch (er2) {
            if (er2.code === "ENOENT")
                return;
            else
                throw er;
        }
        let stats;
        try {
            stats = options.statSync(p);
        }
        catch (er3) {
            if (er3.code === "ENOENT")
                return;
            else
                throw er;
        }
        if (stats.isDirectory())
            rmdirSync(p, options, er);
        else
            options.unlinkSync(p);
    };
    const rmdir = (p, options, originalEr, cb) => {
        assert(p);
        assert(options);
        assert(typeof cb === 'function');
        // try to rmdir first, and only readdir on ENOTEMPTY or EEXIST (SunOS)
        // if we guessed wrong, and it's not a directory, then
        // raise the original error.
        options.rmdir(p, er => {
            if (er && (er.code === "ENOTEMPTY" || er.code === "EEXIST" || er.code === "EPERM"))
                rmkids(p, options, cb);
            else if (er && er.code === "ENOTDIR")
                cb(originalEr);
            else
                cb(er);
        });
    };
    const rmkids = (p, options, cb) => {
        assert(p);
        assert(options);
        assert(typeof cb === 'function');
        options.readdir(p, (er, files) => {
            if (er)
                return cb(er);
            let n = files.length;
            if (n === 0)
                return options.rmdir(p, cb);
            let errState;
            files.forEach(f => {
                rimraf(path.join(p, f), options, er => {
                    if (errState)
                        return;
                    if (er)
                        return cb(errState = er);
                    if (--n === 0)
                        options.rmdir(p, cb);
                });
            });
        });
    };
    // this looks simpler, and is strictly *faster*, but will
    // tie up the JavaScript thread and fail on excessively
    // deep directory trees.
    const rimrafSync = (p, options) => {
        options = options || {};
        defaults(options);
        assert(p, 'rimraf: missing path');
        assert.equal(typeof p, 'string', 'rimraf: path should be a string');
        assert(options, 'rimraf: missing options');
        assert.equal(typeof options, 'object', 'rimraf: options should be object');
        let results;
        if (options.disableGlob || !glob.hasMagic(p)) {
            results = [p];
        }
        else {
            try {
                options.lstatSync(p);
                results = [p];
            }
            catch (er) {
                results = glob.sync(p, options.glob);
            }
        }
        if (!results.length)
            return;
        for (let i = 0; i < results.length; i++) {
            const p = results[i];
            let st;
            try {
                st = options.lstatSync(p);
            }
            catch (er) {
                if (er.code === "ENOENT")
                    return;
                // Windows can EPERM on stat.  Life is suffering.
                if (er.code === "EPERM" && isWindows)
                    fixWinEPERMSync(p, options, er);
            }
            try {
                // sunos lets the root user unlink directories, which is... weird.
                if (st && st.isDirectory())
                    rmdirSync(p, options, null);
                else
                    options.unlinkSync(p);
            }
            catch (er) {
                if (er.code === "ENOENT")
                    return;
                if (er.code === "EPERM")
                    return isWindows ? fixWinEPERMSync(p, options, er) : rmdirSync(p, options, er);
                if (er.code !== "EISDIR")
                    throw er;
                rmdirSync(p, options, er);
            }
        }
    };
    const rmdirSync = (p, options, originalEr) => {
        assert(p);
        assert(options);
        try {
            options.rmdirSync(p);
        }
        catch (er) {
            if (er.code === "ENOENT")
                return;
            if (er.code === "ENOTDIR")
                throw originalEr;
            if (er.code === "ENOTEMPTY" || er.code === "EEXIST" || er.code === "EPERM")
                rmkidsSync(p, options);
        }
    };
    const rmkidsSync = (p, options) => {
        assert(p);
        assert(options);
        options.readdirSync(p).forEach(f => rimrafSync(path.join(p, f), options));
        // We only end up here once we got ENOTEMPTY at least once, and
        // at this point, we are guaranteed to have removed all the kids.
        // So, we know that it won't be ENOENT or ENOTDIR or anything else.
        // try really hard to delete stuff on windows, because it has a
        // PROFOUNDLY annoying habit of not closing handles promptly when
        // files are deleted, resulting in spurious ENOTEMPTY errors.
        const retries = isWindows ? 100 : 1;
        let i = 0;
        do {
            let threw = true;
            try {
                const ret = options.rmdirSync(p, options);
                threw = false;
                return ret;
            }
            finally {
                if (++i < retries && threw)
                    continue;
            }
        } while (true);
    };
    module.exports = rimraf;
    rimraf.sync = rimrafSync;
},
/* glob/glob.js */ function _(require, module, exports) {
    // Approach:
    //
    // 1. Get the minimatch set
    // 2. For each pattern in the set, PROCESS(pattern, false)
    // 3. Store matches per-set, then uniq them
    //
    // PROCESS(pattern, inGlobStar)
    // Get the first [n] items from pattern that are all strings
    // Join these together.  This is PREFIX.
    //   If there is no more remaining, then stat(PREFIX) and
    //   add to matches if it succeeds.  END.
    //
    // If inGlobStar and PREFIX is symlink and points to dir
    //   set ENTRIES = []
    // else readdir(PREFIX) as ENTRIES
    //   If fail, END
    //
    // with ENTRIES
    //   If pattern[n] is GLOBSTAR
    //     // handle the case where the globstar match is empty
    //     // by pruning it out, and testing the resulting pattern
    //     PROCESS(pattern[0..n] + pattern[n+1 .. $], false)
    //     // handle other cases.
    //     for ENTRY in ENTRIES (not dotfiles)
    //       // attach globstar + tail onto the entry
    //       // Mark that this entry is a globstar match
    //       PROCESS(pattern[0..n] + ENTRY + pattern[n .. $], true)
    //
    //   else // not globstar
    //     for ENTRY in ENTRIES (not dotfiles, unless pattern[n] is dot)
    //       Test ENTRY against pattern[n]
    //       If fails, continue
    //       If passes, PROCESS(pattern[0..n] + item + pattern[n+1 .. $])
    //
    // Caveat:
    //   Cache all stats and readdirs results to minimize syscall.  Since all
    //   we ever care about is existence and directory-ness, we can just keep
    //   `true` for files, and [children,...] for directories, or `false` for
    //   things that don't exist.
    module.exports = glob;
    var fs = require('fs');
    var rp = require(582) /* fs.realpath */;
    var minimatch = require(531) /* minimatch */;
    var Minimatch = minimatch.Minimatch;
    var inherits = require(584) /* inherits */;
    var EE = require('events').EventEmitter;
    var path = require('path');
    var assert = require('assert');
    var isAbsolute = require(586) /* path-is-absolute */;
    var globSync = require(587) /* ./sync.js */;
    var common = require(588) /* ./common.js */;
    var alphasort = common.alphasort;
    var alphasorti = common.alphasorti;
    var setopts = common.setopts;
    var ownProp = common.ownProp;
    var inflight = require(589) /* inflight */;
    var util = require('util');
    var childrenIgnored = common.childrenIgnored;
    var isIgnored = common.isIgnored;
    var once = require(591) /* once */;
    function glob(pattern, options, cb) {
        if (typeof options === 'function')
            cb = options, options = {};
        if (!options)
            options = {};
        if (options.sync) {
            if (cb)
                throw new TypeError('callback provided to sync glob');
            return globSync(pattern, options);
        }
        return new Glob(pattern, options, cb);
    }
    glob.sync = globSync;
    var GlobSync = glob.GlobSync = globSync.GlobSync;
    // old api surface
    glob.glob = glob;
    function extend(origin, add) {
        if (add === null || typeof add !== 'object') {
            return origin;
        }
        var keys = Object.keys(add);
        var i = keys.length;
        while (i--) {
            origin[keys[i]] = add[keys[i]];
        }
        return origin;
    }
    glob.hasMagic = function (pattern, options_) {
        var options = extend({}, options_);
        options.noprocess = true;
        var g = new Glob(pattern, options);
        var set = g.minimatch.set;
        if (!pattern)
            return false;
        if (set.length > 1)
            return true;
        for (var j = 0; j < set[0].length; j++) {
            if (typeof set[0][j] !== 'string')
                return true;
        }
        return false;
    };
    glob.Glob = Glob;
    inherits(Glob, EE);
    function Glob(pattern, options, cb) {
        if (typeof options === 'function') {
            cb = options;
            options = null;
        }
        if (options && options.sync) {
            if (cb)
                throw new TypeError('callback provided to sync glob');
            return new GlobSync(pattern, options);
        }
        if (!(this instanceof Glob))
            return new Glob(pattern, options, cb);
        setopts(this, pattern, options);
        this._didRealPath = false;
        // process each pattern in the minimatch set
        var n = this.minimatch.set.length;
        // The matches are stored as {<filename>: true,...} so that
        // duplicates are automagically pruned.
        // Later, we do an Object.keys() on these.
        // Keep them as a list so we can fill in when nonull is set.
        this.matches = new Array(n);
        if (typeof cb === 'function') {
            cb = once(cb);
            this.on('error', cb);
            this.on('end', function (matches) {
                cb(null, matches);
            });
        }
        var self = this;
        this._processing = 0;
        this._emitQueue = [];
        this._processQueue = [];
        this.paused = false;
        if (this.noprocess)
            return this;
        if (n === 0)
            return done();
        var sync = true;
        for (var i = 0; i < n; i++) {
            this._process(this.minimatch.set[i], i, false, done);
        }
        sync = false;
        function done() {
            --self._processing;
            if (self._processing <= 0) {
                if (sync) {
                    process.nextTick(function () {
                        self._finish();
                    });
                }
                else {
                    self._finish();
                }
            }
        }
    }
    Glob.prototype._finish = function () {
        assert(this instanceof Glob);
        if (this.aborted)
            return;
        if (this.realpath && !this._didRealpath)
            return this._realpath();
        common.finish(this);
        this.emit('end', this.found);
    };
    Glob.prototype._realpath = function () {
        if (this._didRealpath)
            return;
        this._didRealpath = true;
        var n = this.matches.length;
        if (n === 0)
            return this._finish();
        var self = this;
        for (var i = 0; i < this.matches.length; i++)
            this._realpathSet(i, next);
        function next() {
            if (--n === 0)
                self._finish();
        }
    };
    Glob.prototype._realpathSet = function (index, cb) {
        var matchset = this.matches[index];
        if (!matchset)
            return cb();
        var found = Object.keys(matchset);
        var self = this;
        var n = found.length;
        if (n === 0)
            return cb();
        var set = this.matches[index] = Object.create(null);
        found.forEach(function (p, i) {
            // If there's a problem with the stat, then it means that
            // one or more of the links in the realpath couldn't be
            // resolved.  just return the abs value in that case.
            p = self._makeAbs(p);
            rp.realpath(p, self.realpathCache, function (er, real) {
                if (!er)
                    set[real] = true;
                else if (er.syscall === 'stat')
                    set[p] = true;
                else
                    self.emit('error', er); // srsly wtf right here
                if (--n === 0) {
                    self.matches[index] = set;
                    cb();
                }
            });
        });
    };
    Glob.prototype._mark = function (p) {
        return common.mark(this, p);
    };
    Glob.prototype._makeAbs = function (f) {
        return common.makeAbs(this, f);
    };
    Glob.prototype.abort = function () {
        this.aborted = true;
        this.emit('abort');
    };
    Glob.prototype.pause = function () {
        if (!this.paused) {
            this.paused = true;
            this.emit('pause');
        }
    };
    Glob.prototype.resume = function () {
        if (this.paused) {
            this.emit('resume');
            this.paused = false;
            if (this._emitQueue.length) {
                var eq = this._emitQueue.slice(0);
                this._emitQueue.length = 0;
                for (var i = 0; i < eq.length; i++) {
                    var e = eq[i];
                    this._emitMatch(e[0], e[1]);
                }
            }
            if (this._processQueue.length) {
                var pq = this._processQueue.slice(0);
                this._processQueue.length = 0;
                for (var i = 0; i < pq.length; i++) {
                    var p = pq[i];
                    this._processing--;
                    this._process(p[0], p[1], p[2], p[3]);
                }
            }
        }
    };
    Glob.prototype._process = function (pattern, index, inGlobStar, cb) {
        assert(this instanceof Glob);
        assert(typeof cb === 'function');
        if (this.aborted)
            return;
        this._processing++;
        if (this.paused) {
            this._processQueue.push([pattern, index, inGlobStar, cb]);
            return;
        }
        //console.error('PROCESS %d', this._processing, pattern)
        // Get the first [n] parts of pattern that are all strings.
        var n = 0;
        while (typeof pattern[n] === 'string') {
            n++;
        }
        // now n is the index of the first one that is *not* a string.
        // see if there's anything else
        var prefix;
        switch (n) {
            // if not, then this is rather simple
            case pattern.length:
                this._processSimple(pattern.join('/'), index, cb);
                return;
            case 0:
                // pattern *starts* with some non-trivial item.
                // going to readdir(cwd), but not include the prefix in matches.
                prefix = null;
                break;
            default:
                // pattern has some string bits in the front.
                // whatever it starts with, whether that's 'absolute' like /foo/bar,
                // or 'relative' like '../baz'
                prefix = pattern.slice(0, n).join('/');
                break;
        }
        var remain = pattern.slice(n);
        // get the list of entries.
        var read;
        if (prefix === null)
            read = '.';
        else if (isAbsolute(prefix) || isAbsolute(pattern.join('/'))) {
            if (!prefix || !isAbsolute(prefix))
                prefix = '/' + prefix;
            read = prefix;
        }
        else
            read = prefix;
        var abs = this._makeAbs(read);
        //if ignored, skip _processing
        if (childrenIgnored(this, read))
            return cb();
        var isGlobStar = remain[0] === minimatch.GLOBSTAR;
        if (isGlobStar)
            this._processGlobStar(prefix, read, abs, remain, index, inGlobStar, cb);
        else
            this._processReaddir(prefix, read, abs, remain, index, inGlobStar, cb);
    };
    Glob.prototype._processReaddir = function (prefix, read, abs, remain, index, inGlobStar, cb) {
        var self = this;
        this._readdir(abs, inGlobStar, function (er, entries) {
            return self._processReaddir2(prefix, read, abs, remain, index, inGlobStar, entries, cb);
        });
    };
    Glob.prototype._processReaddir2 = function (prefix, read, abs, remain, index, inGlobStar, entries, cb) {
        // if the abs isn't a dir, then nothing can match!
        if (!entries)
            return cb();
        // It will only match dot entries if it starts with a dot, or if
        // dot is set.  Stuff like @(.foo|.bar) isn't allowed.
        var pn = remain[0];
        var negate = !!this.minimatch.negate;
        var rawGlob = pn._glob;
        var dotOk = this.dot || rawGlob.charAt(0) === '.';
        var matchedEntries = [];
        for (var i = 0; i < entries.length; i++) {
            var e = entries[i];
            if (e.charAt(0) !== '.' || dotOk) {
                var m;
                if (negate && !prefix) {
                    m = !e.match(pn);
                }
                else {
                    m = e.match(pn);
                }
                if (m)
                    matchedEntries.push(e);
            }
        }
        //console.error('prd2', prefix, entries, remain[0]._glob, matchedEntries)
        var len = matchedEntries.length;
        // If there are no matched entries, then nothing matches.
        if (len === 0)
            return cb();
        // if this is the last remaining pattern bit, then no need for
        // an additional stat *unless* the user has specified mark or
        // stat explicitly.  We know they exist, since readdir returned
        // them.
        if (remain.length === 1 && !this.mark && !this.stat) {
            if (!this.matches[index])
                this.matches[index] = Object.create(null);
            for (var i = 0; i < len; i++) {
                var e = matchedEntries[i];
                if (prefix) {
                    if (prefix !== '/')
                        e = prefix + '/' + e;
                    else
                        e = prefix + e;
                }
                if (e.charAt(0) === '/' && !this.nomount) {
                    e = path.join(this.root, e);
                }
                this._emitMatch(index, e);
            }
            // This was the last one, and no stats were needed
            return cb();
        }
        // now test all matched entries as stand-ins for that part
        // of the pattern.
        remain.shift();
        for (var i = 0; i < len; i++) {
            var e = matchedEntries[i];
            var newPattern;
            if (prefix) {
                if (prefix !== '/')
                    e = prefix + '/' + e;
                else
                    e = prefix + e;
            }
            this._process([e].concat(remain), index, inGlobStar, cb);
        }
        cb();
    };
    Glob.prototype._emitMatch = function (index, e) {
        if (this.aborted)
            return;
        if (isIgnored(this, e))
            return;
        if (this.paused) {
            this._emitQueue.push([index, e]);
            return;
        }
        var abs = isAbsolute(e) ? e : this._makeAbs(e);
        if (this.mark)
            e = this._mark(e);
        if (this.absolute)
            e = abs;
        if (this.matches[index][e])
            return;
        if (this.nodir) {
            var c = this.cache[abs];
            if (c === 'DIR' || Array.isArray(c))
                return;
        }
        this.matches[index][e] = true;
        var st = this.statCache[abs];
        if (st)
            this.emit('stat', e, st);
        this.emit('match', e);
    };
    Glob.prototype._readdirInGlobStar = function (abs, cb) {
        if (this.aborted)
            return;
        // follow all symlinked directories forever
        // just proceed as if this is a non-globstar situation
        if (this.follow)
            return this._readdir(abs, false, cb);
        var lstatkey = 'lstat\0' + abs;
        var self = this;
        var lstatcb = inflight(lstatkey, lstatcb_);
        if (lstatcb)
            fs.lstat(abs, lstatcb);
        function lstatcb_(er, lstat) {
            if (er && er.code === 'ENOENT')
                return cb();
            var isSym = lstat && lstat.isSymbolicLink();
            self.symlinks[abs] = isSym;
            // If it's not a symlink or a dir, then it's definitely a regular file.
            // don't bother doing a readdir in that case.
            if (!isSym && lstat && !lstat.isDirectory()) {
                self.cache[abs] = 'FILE';
                cb();
            }
            else
                self._readdir(abs, false, cb);
        }
    };
    Glob.prototype._readdir = function (abs, inGlobStar, cb) {
        if (this.aborted)
            return;
        cb = inflight('readdir\0' + abs + '\0' + inGlobStar, cb);
        if (!cb)
            return;
        //console.error('RD %j %j', +inGlobStar, abs)
        if (inGlobStar && !ownProp(this.symlinks, abs))
            return this._readdirInGlobStar(abs, cb);
        if (ownProp(this.cache, abs)) {
            var c = this.cache[abs];
            if (!c || c === 'FILE')
                return cb();
            if (Array.isArray(c))
                return cb(null, c);
        }
        var self = this;
        fs.readdir(abs, readdirCb(this, abs, cb));
    };
    function readdirCb(self, abs, cb) {
        return function (er, entries) {
            if (er)
                self._readdirError(abs, er, cb);
            else
                self._readdirEntries(abs, entries, cb);
        };
    }
    Glob.prototype._readdirEntries = function (abs, entries, cb) {
        if (this.aborted)
            return;
        // if we haven't asked to stat everything, then just
        // assume that everything in there exists, so we can avoid
        // having to stat it a second time.
        if (!this.mark && !this.stat) {
            for (var i = 0; i < entries.length; i++) {
                var e = entries[i];
                if (abs === '/')
                    e = abs + e;
                else
                    e = abs + '/' + e;
                this.cache[e] = true;
            }
        }
        this.cache[abs] = entries;
        return cb(null, entries);
    };
    Glob.prototype._readdirError = function (f, er, cb) {
        if (this.aborted)
            return;
        // handle errors, and cache the information
        switch (er.code) {
            case 'ENOTSUP': // https://github.com/isaacs/node-glob/issues/205
            case 'ENOTDIR': // totally normal. means it *does* exist.
                var abs = this._makeAbs(f);
                this.cache[abs] = 'FILE';
                if (abs === this.cwdAbs) {
                    var error = new Error(er.code + ' invalid cwd ' + this.cwd);
                    error.path = this.cwd;
                    error.code = er.code;
                    this.emit('error', error);
                    this.abort();
                }
                break;
            case 'ENOENT': // not terribly unusual
            case 'ELOOP':
            case 'ENAMETOOLONG':
            case 'UNKNOWN':
                this.cache[this._makeAbs(f)] = false;
                break;
            default: // some unusual error.  Treat as failure.
                this.cache[this._makeAbs(f)] = false;
                if (this.strict) {
                    this.emit('error', er);
                    // If the error is handled, then we abort
                    // if not, we threw out of here
                    this.abort();
                }
                if (!this.silent)
                    console.error('glob error', er);
                break;
        }
        return cb();
    };
    Glob.prototype._processGlobStar = function (prefix, read, abs, remain, index, inGlobStar, cb) {
        var self = this;
        this._readdir(abs, inGlobStar, function (er, entries) {
            self._processGlobStar2(prefix, read, abs, remain, index, inGlobStar, entries, cb);
        });
    };
    Glob.prototype._processGlobStar2 = function (prefix, read, abs, remain, index, inGlobStar, entries, cb) {
        //console.error('pgs2', prefix, remain[0], entries)
        // no entries means not a dir, so it can never have matches
        // foo.txt/** doesn't match foo.txt
        if (!entries)
            return cb();
        // test without the globstar, and with every child both below
        // and replacing the globstar.
        var remainWithoutGlobStar = remain.slice(1);
        var gspref = prefix ? [prefix] : [];
        var noGlobStar = gspref.concat(remainWithoutGlobStar);
        // the noGlobStar pattern exits the inGlobStar state
        this._process(noGlobStar, index, false, cb);
        var isSym = this.symlinks[abs];
        var len = entries.length;
        // If it's a symlink, and we're in a globstar, then stop
        if (isSym && inGlobStar)
            return cb();
        for (var i = 0; i < len; i++) {
            var e = entries[i];
            if (e.charAt(0) === '.' && !this.dot)
                continue;
            // these two cases enter the inGlobStar state
            var instead = gspref.concat(entries[i], remainWithoutGlobStar);
            this._process(instead, index, true, cb);
            var below = gspref.concat(entries[i], remain);
            this._process(below, index, true, cb);
        }
        cb();
    };
    Glob.prototype._processSimple = function (prefix, index, cb) {
        // XXX review this.  Shouldn't it be doing the mounting etc
        // before doing stat?  kinda weird?
        var self = this;
        this._stat(prefix, function (er, exists) {
            self._processSimple2(prefix, index, er, exists, cb);
        });
    };
    Glob.prototype._processSimple2 = function (prefix, index, er, exists, cb) {
        //console.error('ps2', prefix, exists)
        if (!this.matches[index])
            this.matches[index] = Object.create(null);
        // If it doesn't exist, then just mark the lack of results
        if (!exists)
            return cb();
        if (prefix && isAbsolute(prefix) && !this.nomount) {
            var trail = /[\/\\]$/.test(prefix);
            if (prefix.charAt(0) === '/') {
                prefix = path.join(this.root, prefix);
            }
            else {
                prefix = path.resolve(this.root, prefix);
                if (trail)
                    prefix += '/';
            }
        }
        if (process.platform === 'win32')
            prefix = prefix.replace(/\\/g, '/');
        // Mark this as a match
        this._emitMatch(index, prefix);
        cb();
    };
    // Returns either 'DIR', 'FILE', or false
    Glob.prototype._stat = function (f, cb) {
        var abs = this._makeAbs(f);
        var needDir = f.slice(-1) === '/';
        if (f.length > this.maxLength)
            return cb();
        if (!this.stat && ownProp(this.cache, abs)) {
            var c = this.cache[abs];
            if (Array.isArray(c))
                c = 'DIR';
            // It exists, but maybe not how we need it
            if (!needDir || c === 'DIR')
                return cb(null, c);
            if (needDir && c === 'FILE')
                return cb();
            // otherwise we have to stat, because maybe c=true
            // if we know it exists, but not what it is.
        }
        var exists;
        var stat = this.statCache[abs];
        if (stat !== undefined) {
            if (stat === false)
                return cb(null, stat);
            else {
                var type = stat.isDirectory() ? 'DIR' : 'FILE';
                if (needDir && type === 'FILE')
                    return cb();
                else
                    return cb(null, type, stat);
            }
        }
        var self = this;
        var statcb = inflight('stat\0' + abs, lstatcb_);
        if (statcb)
            fs.lstat(abs, statcb);
        function lstatcb_(er, lstat) {
            if (lstat && lstat.isSymbolicLink()) {
                // If it's a symlink, then treat it as the target, unless
                // the target does not exist, then treat it as a file.
                return fs.stat(abs, function (er, stat) {
                    if (er)
                        self._stat2(f, abs, null, lstat, cb);
                    else
                        self._stat2(f, abs, er, stat, cb);
                });
            }
            else {
                self._stat2(f, abs, er, lstat, cb);
            }
        }
    };
    Glob.prototype._stat2 = function (f, abs, er, stat, cb) {
        if (er && (er.code === 'ENOENT' || er.code === 'ENOTDIR')) {
            this.statCache[abs] = false;
            return cb();
        }
        var needDir = f.slice(-1) === '/';
        this.statCache[abs] = stat;
        if (abs.slice(-1) === '/' && stat && !stat.isDirectory())
            return cb(null, false, stat);
        var c = true;
        if (stat)
            c = stat.isDirectory() ? 'DIR' : 'FILE';
        this.cache[abs] = this.cache[abs] || c;
        if (needDir && c === 'FILE')
            return cb();
        return cb(null, c, stat);
    };
},
/* fs.realpath/index.js */ function _(require, module, exports) {
    module.exports = realpath;
    realpath.realpath = realpath;
    realpath.sync = realpathSync;
    realpath.realpathSync = realpathSync;
    realpath.monkeypatch = monkeypatch;
    realpath.unmonkeypatch = unmonkeypatch;
    var fs = require('fs');
    var origRealpath = fs.realpath;
    var origRealpathSync = fs.realpathSync;
    var version = process.version;
    var ok = /^v[0-5]\./.test(version);
    var old = require(583) /* ./old.js */;
    function newError(er) {
        return er && er.syscall === 'realpath' && (er.code === 'ELOOP' ||
            er.code === 'ENOMEM' ||
            er.code === 'ENAMETOOLONG');
    }
    function realpath(p, cache, cb) {
        if (ok) {
            return origRealpath(p, cache, cb);
        }
        if (typeof cache === 'function') {
            cb = cache;
            cache = null;
        }
        origRealpath(p, cache, function (er, result) {
            if (newError(er)) {
                old.realpath(p, cache, cb);
            }
            else {
                cb(er, result);
            }
        });
    }
    function realpathSync(p, cache) {
        if (ok) {
            return origRealpathSync(p, cache);
        }
        try {
            return origRealpathSync(p, cache);
        }
        catch (er) {
            if (newError(er)) {
                return old.realpathSync(p, cache);
            }
            else {
                throw er;
            }
        }
    }
    function monkeypatch() {
        fs.realpath = realpath;
        fs.realpathSync = realpathSync;
    }
    function unmonkeypatch() {
        fs.realpath = origRealpath;
        fs.realpathSync = origRealpathSync;
    }
},
/* fs.realpath/old.js */ function _(require, module, exports) {
    // Copyright Joyent, Inc. and other Node contributors.
    //
    // Permission is hereby granted, free of charge, to any person obtaining a
    // copy of this software and associated documentation files (the
    // "Software"), to deal in the Software without restriction, including
    // without limitation the rights to use, copy, modify, merge, publish,
    // distribute, sublicense, and/or sell copies of the Software, and to permit
    // persons to whom the Software is furnished to do so, subject to the
    // following conditions:
    //
    // The above copyright notice and this permission notice shall be included
    // in all copies or substantial portions of the Software.
    //
    // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    // OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
    // NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
    // DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
    // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
    // USE OR OTHER DEALINGS IN THE SOFTWARE.
    var pathModule = require('path');
    var isWindows = process.platform === 'win32';
    var fs = require('fs');
    // JavaScript implementation of realpath, ported from node pre-v6
    var DEBUG = process.env.NODE_DEBUG && /fs/.test(process.env.NODE_DEBUG);
    function rethrow() {
        // Only enable in debug mode. A backtrace uses ~1000 bytes of heap space and
        // is fairly slow to generate.
        var callback;
        if (DEBUG) {
            var backtrace = new Error;
            callback = debugCallback;
        }
        else
            callback = missingCallback;
        return callback;
        function debugCallback(err) {
            if (err) {
                backtrace.message = err.message;
                err = backtrace;
                missingCallback(err);
            }
        }
        function missingCallback(err) {
            if (err) {
                if (process.throwDeprecation)
                    throw err; // Forgot a callback but don't know where? Use NODE_DEBUG=fs
                else if (!process.noDeprecation) {
                    var msg = 'fs: missing callback ' + (err.stack || err.message);
                    if (process.traceDeprecation)
                        console.trace(msg);
                    else
                        console.error(msg);
                }
            }
        }
    }
    function maybeCallback(cb) {
        return typeof cb === 'function' ? cb : rethrow();
    }
    var normalize = pathModule.normalize;
    // Regexp that finds the next partion of a (partial) path
    // result is [base_with_slash, base], e.g. ['somedir/', 'somedir']
    if (isWindows) {
        var nextPartRe = /(.*?)(?:[\/\\]+|$)/g;
    }
    else {
        var nextPartRe = /(.*?)(?:[\/]+|$)/g;
    }
    // Regex to find the device root, including trailing slash. E.g. 'c:\\'.
    if (isWindows) {
        var splitRootRe = /^(?:[a-zA-Z]:|[\\\/]{2}[^\\\/]+[\\\/][^\\\/]+)?[\\\/]*/;
    }
    else {
        var splitRootRe = /^[\/]*/;
    }
    exports.realpathSync = function realpathSync(p, cache) {
        // make p is absolute
        p = pathModule.resolve(p);
        if (cache && Object.prototype.hasOwnProperty.call(cache, p)) {
            return cache[p];
        }
        var original = p, seenLinks = {}, knownHard = {};
        // current character position in p
        var pos;
        // the partial path so far, including a trailing slash if any
        var current;
        // the partial path without a trailing slash (except when pointing at a root)
        var base;
        // the partial path scanned in the previous round, with slash
        var previous;
        start();
        function start() {
            // Skip over roots
            var m = splitRootRe.exec(p);
            pos = m[0].length;
            current = m[0];
            base = m[0];
            previous = '';
            // On windows, check that the root exists. On unix there is no need.
            if (isWindows && !knownHard[base]) {
                fs.lstatSync(base);
                knownHard[base] = true;
            }
        }
        // walk down the path, swapping out linked pathparts for their real
        // values
        // NB: p.length changes.
        while (pos < p.length) {
            // find the next part
            nextPartRe.lastIndex = pos;
            var result = nextPartRe.exec(p);
            previous = current;
            current += result[0];
            base = previous + result[1];
            pos = nextPartRe.lastIndex;
            // continue if not a symlink
            if (knownHard[base] || (cache && cache[base] === base)) {
                continue;
            }
            var resolvedLink;
            if (cache && Object.prototype.hasOwnProperty.call(cache, base)) {
                // some known symbolic link.  no need to stat again.
                resolvedLink = cache[base];
            }
            else {
                var stat = fs.lstatSync(base);
                if (!stat.isSymbolicLink()) {
                    knownHard[base] = true;
                    if (cache)
                        cache[base] = base;
                    continue;
                }
                // read the link if it wasn't read before
                // dev/ino always return 0 on windows, so skip the check.
                var linkTarget = null;
                if (!isWindows) {
                    var id = stat.dev.toString(32) + ':' + stat.ino.toString(32);
                    if (seenLinks.hasOwnProperty(id)) {
                        linkTarget = seenLinks[id];
                    }
                }
                if (linkTarget === null) {
                    fs.statSync(base);
                    linkTarget = fs.readlinkSync(base);
                }
                resolvedLink = pathModule.resolve(previous, linkTarget);
                // track this, if given a cache.
                if (cache)
                    cache[base] = resolvedLink;
                if (!isWindows)
                    seenLinks[id] = linkTarget;
            }
            // resolve the link, then start over
            p = pathModule.resolve(resolvedLink, p.slice(pos));
            start();
        }
        if (cache)
            cache[original] = p;
        return p;
    };
    exports.realpath = function realpath(p, cache, cb) {
        if (typeof cb !== 'function') {
            cb = maybeCallback(cache);
            cache = null;
        }
        // make p is absolute
        p = pathModule.resolve(p);
        if (cache && Object.prototype.hasOwnProperty.call(cache, p)) {
            return process.nextTick(cb.bind(null, null, cache[p]));
        }
        var original = p, seenLinks = {}, knownHard = {};
        // current character position in p
        var pos;
        // the partial path so far, including a trailing slash if any
        var current;
        // the partial path without a trailing slash (except when pointing at a root)
        var base;
        // the partial path scanned in the previous round, with slash
        var previous;
        start();
        function start() {
            // Skip over roots
            var m = splitRootRe.exec(p);
            pos = m[0].length;
            current = m[0];
            base = m[0];
            previous = '';
            // On windows, check that the root exists. On unix there is no need.
            if (isWindows && !knownHard[base]) {
                fs.lstat(base, function (err) {
                    if (err)
                        return cb(err);
                    knownHard[base] = true;
                    LOOP();
                });
            }
            else {
                process.nextTick(LOOP);
            }
        }
        // walk down the path, swapping out linked pathparts for their real
        // values
        function LOOP() {
            // stop if scanned past end of path
            if (pos >= p.length) {
                if (cache)
                    cache[original] = p;
                return cb(null, p);
            }
            // find the next part
            nextPartRe.lastIndex = pos;
            var result = nextPartRe.exec(p);
            previous = current;
            current += result[0];
            base = previous + result[1];
            pos = nextPartRe.lastIndex;
            // continue if not a symlink
            if (knownHard[base] || (cache && cache[base] === base)) {
                return process.nextTick(LOOP);
            }
            if (cache && Object.prototype.hasOwnProperty.call(cache, base)) {
                // known symbolic link.  no need to stat again.
                return gotResolvedLink(cache[base]);
            }
            return fs.lstat(base, gotStat);
        }
        function gotStat(err, stat) {
            if (err)
                return cb(err);
            // if not a symlink, skip to the next path part
            if (!stat.isSymbolicLink()) {
                knownHard[base] = true;
                if (cache)
                    cache[base] = base;
                return process.nextTick(LOOP);
            }
            // stat & read the link if not read before
            // call gotTarget as soon as the link target is known
            // dev/ino always return 0 on windows, so skip the check.
            if (!isWindows) {
                var id = stat.dev.toString(32) + ':' + stat.ino.toString(32);
                if (seenLinks.hasOwnProperty(id)) {
                    return gotTarget(null, seenLinks[id], base);
                }
            }
            fs.stat(base, function (err) {
                if (err)
                    return cb(err);
                fs.readlink(base, function (err, target) {
                    if (!isWindows)
                        seenLinks[id] = target;
                    gotTarget(err, target);
                });
            });
        }
        function gotTarget(err, target, base) {
            if (err)
                return cb(err);
            var resolvedLink = pathModule.resolve(previous, target);
            if (cache)
                cache[base] = resolvedLink;
            gotResolvedLink(resolvedLink);
        }
        function gotResolvedLink(resolvedLink) {
            // resolve the link, then start over
            p = pathModule.resolve(resolvedLink, p.slice(pos));
            start();
        }
    };
},
/* inherits/inherits.js */ function _(require, module, exports) {
    try {
        var util = require('util');
        /* istanbul ignore next */
        if (typeof util.inherits !== 'function')
            throw '';
        module.exports = util.inherits;
    }
    catch (e) {
        /* istanbul ignore next */
        module.exports = require(585) /* ./inherits_browser.js */;
    }
},
/* inherits/inherits_browser.js */ function _(require, module, exports) {
    if (typeof Object.create === 'function') {
        // implementation from standard node.js 'util' module
        module.exports = function inherits(ctor, superCtor) {
            if (superCtor) {
                ctor.super_ = superCtor;
                ctor.prototype = Object.create(superCtor.prototype, {
                    constructor: {
                        value: ctor,
                        enumerable: false,
                        writable: true,
                        configurable: true
                    }
                });
            }
        };
    }
    else {
        // old school shim for old browsers
        module.exports = function inherits(ctor, superCtor) {
            if (superCtor) {
                ctor.super_ = superCtor;
                var TempCtor = function () { };
                TempCtor.prototype = superCtor.prototype;
                ctor.prototype = new TempCtor();
                ctor.prototype.constructor = ctor;
            }
        };
    }
},
/* path-is-absolute/index.js */ function _(require, module, exports) {
    function posix(path) {
        return path.charAt(0) === '/';
    }
    function win32(path) {
        // https://github.com/nodejs/node/blob/b3fcc245fb25539909ef1d5eaa01dbf92e168633/lib/path.js#L56
        var splitDeviceRe = /^([a-zA-Z]:|[\\\/]{2}[^\\\/]+[\\\/]+[^\\\/]+)?([\\\/])?([\s\S]*?)$/;
        var result = splitDeviceRe.exec(path);
        var device = result[1] || '';
        var isUnc = Boolean(device && device.charAt(1) !== ':');
        // UNC paths are always absolute
        return Boolean(result[2] || isUnc);
    }
    module.exports = process.platform === 'win32' ? win32 : posix;
    module.exports.posix = posix;
    module.exports.win32 = win32;
},
/* glob/sync.js */ function _(require, module, exports) {
    module.exports = globSync;
    globSync.GlobSync = GlobSync;
    var fs = require('fs');
    var rp = require(582) /* fs.realpath */;
    var minimatch = require(531) /* minimatch */;
    var Minimatch = minimatch.Minimatch;
    var Glob = require(581) /* ./glob.js */.Glob;
    var util = require('util');
    var path = require('path');
    var assert = require('assert');
    var isAbsolute = require(586) /* path-is-absolute */;
    var common = require(588) /* ./common.js */;
    var alphasort = common.alphasort;
    var alphasorti = common.alphasorti;
    var setopts = common.setopts;
    var ownProp = common.ownProp;
    var childrenIgnored = common.childrenIgnored;
    var isIgnored = common.isIgnored;
    function globSync(pattern, options) {
        if (typeof options === 'function' || arguments.length === 3)
            throw new TypeError('callback provided to sync glob\n' +
                'See: https://github.com/isaacs/node-glob/issues/167');
        return new GlobSync(pattern, options).found;
    }
    function GlobSync(pattern, options) {
        if (!pattern)
            throw new Error('must provide pattern');
        if (typeof options === 'function' || arguments.length === 3)
            throw new TypeError('callback provided to sync glob\n' +
                'See: https://github.com/isaacs/node-glob/issues/167');
        if (!(this instanceof GlobSync))
            return new GlobSync(pattern, options);
        setopts(this, pattern, options);
        if (this.noprocess)
            return this;
        var n = this.minimatch.set.length;
        this.matches = new Array(n);
        for (var i = 0; i < n; i++) {
            this._process(this.minimatch.set[i], i, false);
        }
        this._finish();
    }
    GlobSync.prototype._finish = function () {
        assert(this instanceof GlobSync);
        if (this.realpath) {
            var self = this;
            this.matches.forEach(function (matchset, index) {
                var set = self.matches[index] = Object.create(null);
                for (var p in matchset) {
                    try {
                        p = self._makeAbs(p);
                        var real = rp.realpathSync(p, self.realpathCache);
                        set[real] = true;
                    }
                    catch (er) {
                        if (er.syscall === 'stat')
                            set[self._makeAbs(p)] = true;
                        else
                            throw er;
                    }
                }
            });
        }
        common.finish(this);
    };
    GlobSync.prototype._process = function (pattern, index, inGlobStar) {
        assert(this instanceof GlobSync);
        // Get the first [n] parts of pattern that are all strings.
        var n = 0;
        while (typeof pattern[n] === 'string') {
            n++;
        }
        // now n is the index of the first one that is *not* a string.
        // See if there's anything else
        var prefix;
        switch (n) {
            // if not, then this is rather simple
            case pattern.length:
                this._processSimple(pattern.join('/'), index);
                return;
            case 0:
                // pattern *starts* with some non-trivial item.
                // going to readdir(cwd), but not include the prefix in matches.
                prefix = null;
                break;
            default:
                // pattern has some string bits in the front.
                // whatever it starts with, whether that's 'absolute' like /foo/bar,
                // or 'relative' like '../baz'
                prefix = pattern.slice(0, n).join('/');
                break;
        }
        var remain = pattern.slice(n);
        // get the list of entries.
        var read;
        if (prefix === null)
            read = '.';
        else if (isAbsolute(prefix) || isAbsolute(pattern.join('/'))) {
            if (!prefix || !isAbsolute(prefix))
                prefix = '/' + prefix;
            read = prefix;
        }
        else
            read = prefix;
        var abs = this._makeAbs(read);
        //if ignored, skip processing
        if (childrenIgnored(this, read))
            return;
        var isGlobStar = remain[0] === minimatch.GLOBSTAR;
        if (isGlobStar)
            this._processGlobStar(prefix, read, abs, remain, index, inGlobStar);
        else
            this._processReaddir(prefix, read, abs, remain, index, inGlobStar);
    };
    GlobSync.prototype._processReaddir = function (prefix, read, abs, remain, index, inGlobStar) {
        var entries = this._readdir(abs, inGlobStar);
        // if the abs isn't a dir, then nothing can match!
        if (!entries)
            return;
        // It will only match dot entries if it starts with a dot, or if
        // dot is set.  Stuff like @(.foo|.bar) isn't allowed.
        var pn = remain[0];
        var negate = !!this.minimatch.negate;
        var rawGlob = pn._glob;
        var dotOk = this.dot || rawGlob.charAt(0) === '.';
        var matchedEntries = [];
        for (var i = 0; i < entries.length; i++) {
            var e = entries[i];
            if (e.charAt(0) !== '.' || dotOk) {
                var m;
                if (negate && !prefix) {
                    m = !e.match(pn);
                }
                else {
                    m = e.match(pn);
                }
                if (m)
                    matchedEntries.push(e);
            }
        }
        var len = matchedEntries.length;
        // If there are no matched entries, then nothing matches.
        if (len === 0)
            return;
        // if this is the last remaining pattern bit, then no need for
        // an additional stat *unless* the user has specified mark or
        // stat explicitly.  We know they exist, since readdir returned
        // them.
        if (remain.length === 1 && !this.mark && !this.stat) {
            if (!this.matches[index])
                this.matches[index] = Object.create(null);
            for (var i = 0; i < len; i++) {
                var e = matchedEntries[i];
                if (prefix) {
                    if (prefix.slice(-1) !== '/')
                        e = prefix + '/' + e;
                    else
                        e = prefix + e;
                }
                if (e.charAt(0) === '/' && !this.nomount) {
                    e = path.join(this.root, e);
                }
                this._emitMatch(index, e);
            }
            // This was the last one, and no stats were needed
            return;
        }
        // now test all matched entries as stand-ins for that part
        // of the pattern.
        remain.shift();
        for (var i = 0; i < len; i++) {
            var e = matchedEntries[i];
            var newPattern;
            if (prefix)
                newPattern = [prefix, e];
            else
                newPattern = [e];
            this._process(newPattern.concat(remain), index, inGlobStar);
        }
    };
    GlobSync.prototype._emitMatch = function (index, e) {
        if (isIgnored(this, e))
            return;
        var abs = this._makeAbs(e);
        if (this.mark)
            e = this._mark(e);
        if (this.absolute) {
            e = abs;
        }
        if (this.matches[index][e])
            return;
        if (this.nodir) {
            var c = this.cache[abs];
            if (c === 'DIR' || Array.isArray(c))
                return;
        }
        this.matches[index][e] = true;
        if (this.stat)
            this._stat(e);
    };
    GlobSync.prototype._readdirInGlobStar = function (abs) {
        // follow all symlinked directories forever
        // just proceed as if this is a non-globstar situation
        if (this.follow)
            return this._readdir(abs, false);
        var entries;
        var lstat;
        var stat;
        try {
            lstat = fs.lstatSync(abs);
        }
        catch (er) {
            if (er.code === 'ENOENT') {
                // lstat failed, doesn't exist
                return null;
            }
        }
        var isSym = lstat && lstat.isSymbolicLink();
        this.symlinks[abs] = isSym;
        // If it's not a symlink or a dir, then it's definitely a regular file.
        // don't bother doing a readdir in that case.
        if (!isSym && lstat && !lstat.isDirectory())
            this.cache[abs] = 'FILE';
        else
            entries = this._readdir(abs, false);
        return entries;
    };
    GlobSync.prototype._readdir = function (abs, inGlobStar) {
        var entries;
        if (inGlobStar && !ownProp(this.symlinks, abs))
            return this._readdirInGlobStar(abs);
        if (ownProp(this.cache, abs)) {
            var c = this.cache[abs];
            if (!c || c === 'FILE')
                return null;
            if (Array.isArray(c))
                return c;
        }
        try {
            return this._readdirEntries(abs, fs.readdirSync(abs));
        }
        catch (er) {
            this._readdirError(abs, er);
            return null;
        }
    };
    GlobSync.prototype._readdirEntries = function (abs, entries) {
        // if we haven't asked to stat everything, then just
        // assume that everything in there exists, so we can avoid
        // having to stat it a second time.
        if (!this.mark && !this.stat) {
            for (var i = 0; i < entries.length; i++) {
                var e = entries[i];
                if (abs === '/')
                    e = abs + e;
                else
                    e = abs + '/' + e;
                this.cache[e] = true;
            }
        }
        this.cache[abs] = entries;
        // mark and cache dir-ness
        return entries;
    };
    GlobSync.prototype._readdirError = function (f, er) {
        // handle errors, and cache the information
        switch (er.code) {
            case 'ENOTSUP': // https://github.com/isaacs/node-glob/issues/205
            case 'ENOTDIR': // totally normal. means it *does* exist.
                var abs = this._makeAbs(f);
                this.cache[abs] = 'FILE';
                if (abs === this.cwdAbs) {
                    var error = new Error(er.code + ' invalid cwd ' + this.cwd);
                    error.path = this.cwd;
                    error.code = er.code;
                    throw error;
                }
                break;
            case 'ENOENT': // not terribly unusual
            case 'ELOOP':
            case 'ENAMETOOLONG':
            case 'UNKNOWN':
                this.cache[this._makeAbs(f)] = false;
                break;
            default: // some unusual error.  Treat as failure.
                this.cache[this._makeAbs(f)] = false;
                if (this.strict)
                    throw er;
                if (!this.silent)
                    console.error('glob error', er);
                break;
        }
    };
    GlobSync.prototype._processGlobStar = function (prefix, read, abs, remain, index, inGlobStar) {
        var entries = this._readdir(abs, inGlobStar);
        // no entries means not a dir, so it can never have matches
        // foo.txt/** doesn't match foo.txt
        if (!entries)
            return;
        // test without the globstar, and with every child both below
        // and replacing the globstar.
        var remainWithoutGlobStar = remain.slice(1);
        var gspref = prefix ? [prefix] : [];
        var noGlobStar = gspref.concat(remainWithoutGlobStar);
        // the noGlobStar pattern exits the inGlobStar state
        this._process(noGlobStar, index, false);
        var len = entries.length;
        var isSym = this.symlinks[abs];
        // If it's a symlink, and we're in a globstar, then stop
        if (isSym && inGlobStar)
            return;
        for (var i = 0; i < len; i++) {
            var e = entries[i];
            if (e.charAt(0) === '.' && !this.dot)
                continue;
            // these two cases enter the inGlobStar state
            var instead = gspref.concat(entries[i], remainWithoutGlobStar);
            this._process(instead, index, true);
            var below = gspref.concat(entries[i], remain);
            this._process(below, index, true);
        }
    };
    GlobSync.prototype._processSimple = function (prefix, index) {
        // XXX review this.  Shouldn't it be doing the mounting etc
        // before doing stat?  kinda weird?
        var exists = this._stat(prefix);
        if (!this.matches[index])
            this.matches[index] = Object.create(null);
        // If it doesn't exist, then just mark the lack of results
        if (!exists)
            return;
        if (prefix && isAbsolute(prefix) && !this.nomount) {
            var trail = /[\/\\]$/.test(prefix);
            if (prefix.charAt(0) === '/') {
                prefix = path.join(this.root, prefix);
            }
            else {
                prefix = path.resolve(this.root, prefix);
                if (trail)
                    prefix += '/';
            }
        }
        if (process.platform === 'win32')
            prefix = prefix.replace(/\\/g, '/');
        // Mark this as a match
        this._emitMatch(index, prefix);
    };
    // Returns either 'DIR', 'FILE', or false
    GlobSync.prototype._stat = function (f) {
        var abs = this._makeAbs(f);
        var needDir = f.slice(-1) === '/';
        if (f.length > this.maxLength)
            return false;
        if (!this.stat && ownProp(this.cache, abs)) {
            var c = this.cache[abs];
            if (Array.isArray(c))
                c = 'DIR';
            // It exists, but maybe not how we need it
            if (!needDir || c === 'DIR')
                return c;
            if (needDir && c === 'FILE')
                return false;
            // otherwise we have to stat, because maybe c=true
            // if we know it exists, but not what it is.
        }
        var exists;
        var stat = this.statCache[abs];
        if (!stat) {
            var lstat;
            try {
                lstat = fs.lstatSync(abs);
            }
            catch (er) {
                if (er && (er.code === 'ENOENT' || er.code === 'ENOTDIR')) {
                    this.statCache[abs] = false;
                    return false;
                }
            }
            if (lstat && lstat.isSymbolicLink()) {
                try {
                    stat = fs.statSync(abs);
                }
                catch (er) {
                    stat = lstat;
                }
            }
            else {
                stat = lstat;
            }
        }
        this.statCache[abs] = stat;
        var c = true;
        if (stat)
            c = stat.isDirectory() ? 'DIR' : 'FILE';
        this.cache[abs] = this.cache[abs] || c;
        if (needDir && c === 'FILE')
            return false;
        return c;
    };
    GlobSync.prototype._mark = function (p) {
        return common.mark(this, p);
    };
    GlobSync.prototype._makeAbs = function (f) {
        return common.makeAbs(this, f);
    };
},
/* glob/common.js */ function _(require, module, exports) {
    exports.alphasort = alphasort;
    exports.alphasorti = alphasorti;
    exports.setopts = setopts;
    exports.ownProp = ownProp;
    exports.makeAbs = makeAbs;
    exports.finish = finish;
    exports.mark = mark;
    exports.isIgnored = isIgnored;
    exports.childrenIgnored = childrenIgnored;
    function ownProp(obj, field) {
        return Object.prototype.hasOwnProperty.call(obj, field);
    }
    var path = require("path");
    var minimatch = require(531) /* minimatch */;
    var isAbsolute = require(586) /* path-is-absolute */;
    var Minimatch = minimatch.Minimatch;
    function alphasorti(a, b) {
        return a.toLowerCase().localeCompare(b.toLowerCase());
    }
    function alphasort(a, b) {
        return a.localeCompare(b);
    }
    function setupIgnores(self, options) {
        self.ignore = options.ignore || [];
        if (!Array.isArray(self.ignore))
            self.ignore = [self.ignore];
        if (self.ignore.length) {
            self.ignore = self.ignore.map(ignoreMap);
        }
    }
    // ignore patterns are always in dot:true mode.
    function ignoreMap(pattern) {
        var gmatcher = null;
        if (pattern.slice(-3) === '/**') {
            var gpattern = pattern.replace(/(\/\*\*)+$/, '');
            gmatcher = new Minimatch(gpattern, { dot: true });
        }
        return {
            matcher: new Minimatch(pattern, { dot: true }),
            gmatcher: gmatcher
        };
    }
    function setopts(self, pattern, options) {
        if (!options)
            options = {};
        // base-matching: just use globstar for that.
        if (options.matchBase && -1 === pattern.indexOf("/")) {
            if (options.noglobstar) {
                throw new Error("base matching requires globstar");
            }
            pattern = "**/" + pattern;
        }
        self.silent = !!options.silent;
        self.pattern = pattern;
        self.strict = options.strict !== false;
        self.realpath = !!options.realpath;
        self.realpathCache = options.realpathCache || Object.create(null);
        self.follow = !!options.follow;
        self.dot = !!options.dot;
        self.mark = !!options.mark;
        self.nodir = !!options.nodir;
        if (self.nodir)
            self.mark = true;
        self.sync = !!options.sync;
        self.nounique = !!options.nounique;
        self.nonull = !!options.nonull;
        self.nosort = !!options.nosort;
        self.nocase = !!options.nocase;
        self.stat = !!options.stat;
        self.noprocess = !!options.noprocess;
        self.absolute = !!options.absolute;
        self.maxLength = options.maxLength || Infinity;
        self.cache = options.cache || Object.create(null);
        self.statCache = options.statCache || Object.create(null);
        self.symlinks = options.symlinks || Object.create(null);
        setupIgnores(self, options);
        self.changedCwd = false;
        var cwd = process.cwd();
        if (!ownProp(options, "cwd"))
            self.cwd = cwd;
        else {
            self.cwd = path.resolve(options.cwd);
            self.changedCwd = self.cwd !== cwd;
        }
        self.root = options.root || path.resolve(self.cwd, "/");
        self.root = path.resolve(self.root);
        if (process.platform === "win32")
            self.root = self.root.replace(/\\/g, "/");
        // TODO: is an absolute `cwd` supposed to be resolved against `root`?
        // e.g. { cwd: '/test', root: __dirname } === path.join(__dirname, '/test')
        self.cwdAbs = isAbsolute(self.cwd) ? self.cwd : makeAbs(self, self.cwd);
        if (process.platform === "win32")
            self.cwdAbs = self.cwdAbs.replace(/\\/g, "/");
        self.nomount = !!options.nomount;
        // disable comments and negation in Minimatch.
        // Note that they are not supported in Glob itself anyway.
        options.nonegate = true;
        options.nocomment = true;
        self.minimatch = new Minimatch(pattern, options);
        self.options = self.minimatch.options;
    }
    function finish(self) {
        var nou = self.nounique;
        var all = nou ? [] : Object.create(null);
        for (var i = 0, l = self.matches.length; i < l; i++) {
            var matches = self.matches[i];
            if (!matches || Object.keys(matches).length === 0) {
                if (self.nonull) {
                    // do like the shell, and spit out the literal glob
                    var literal = self.minimatch.globSet[i];
                    if (nou)
                        all.push(literal);
                    else
                        all[literal] = true;
                }
            }
            else {
                // had matches
                var m = Object.keys(matches);
                if (nou)
                    all.push.apply(all, m);
                else
                    m.forEach(function (m) {
                        all[m] = true;
                    });
            }
        }
        if (!nou)
            all = Object.keys(all);
        if (!self.nosort)
            all = all.sort(self.nocase ? alphasorti : alphasort);
        // at *some* point we statted all of these
        if (self.mark) {
            for (var i = 0; i < all.length; i++) {
                all[i] = self._mark(all[i]);
            }
            if (self.nodir) {
                all = all.filter(function (e) {
                    var notDir = !(/\/$/.test(e));
                    var c = self.cache[e] || self.cache[makeAbs(self, e)];
                    if (notDir && c)
                        notDir = c !== 'DIR' && !Array.isArray(c);
                    return notDir;
                });
            }
        }
        if (self.ignore.length)
            all = all.filter(function (m) {
                return !isIgnored(self, m);
            });
        self.found = all;
    }
    function mark(self, p) {
        var abs = makeAbs(self, p);
        var c = self.cache[abs];
        var m = p;
        if (c) {
            var isDir = c === 'DIR' || Array.isArray(c);
            var slash = p.slice(-1) === '/';
            if (isDir && !slash)
                m += '/';
            else if (!isDir && slash)
                m = m.slice(0, -1);
            if (m !== p) {
                var mabs = makeAbs(self, m);
                self.statCache[mabs] = self.statCache[abs];
                self.cache[mabs] = self.cache[abs];
            }
        }
        return m;
    }
    // lotta situps...
    function makeAbs(self, f) {
        var abs = f;
        if (f.charAt(0) === '/') {
            abs = path.join(self.root, f);
        }
        else if (isAbsolute(f) || f === '') {
            abs = f;
        }
        else if (self.changedCwd) {
            abs = path.resolve(self.cwd, f);
        }
        else {
            abs = path.resolve(f);
        }
        if (process.platform === 'win32')
            abs = abs.replace(/\\/g, '/');
        return abs;
    }
    // Return true, if pattern ends with globstar '**', for the accompanying parent directory.
    // Ex:- If node_modules/** is the pattern, add 'node_modules' to ignore list along with it's contents
    function isIgnored(self, path) {
        if (!self.ignore.length)
            return false;
        return self.ignore.some(function (item) {
            return item.matcher.match(path) || !!(item.gmatcher && item.gmatcher.match(path));
        });
    }
    function childrenIgnored(self, path) {
        if (!self.ignore.length)
            return false;
        return self.ignore.some(function (item) {
            return !!(item.gmatcher && item.gmatcher.match(path));
        });
    }
},
/* inflight/inflight.js */ function _(require, module, exports) {
    var wrappy = require(590) /* wrappy */;
    var reqs = Object.create(null);
    var once = require(591) /* once */;
    module.exports = wrappy(inflight);
    function inflight(key, cb) {
        if (reqs[key]) {
            reqs[key].push(cb);
            return null;
        }
        else {
            reqs[key] = [cb];
            return makeres(key);
        }
    }
    function makeres(key) {
        return once(function RES() {
            var cbs = reqs[key];
            var len = cbs.length;
            var args = slice(arguments);
            // XXX It's somewhat ambiguous whether a new callback added in this
            // pass should be queued for later execution if something in the
            // list of callbacks throws, or if it should just be discarded.
            // However, it's such an edge case that it hardly matters, and either
            // choice is likely as surprising as the other.
            // As it happens, we do go ahead and schedule it for later execution.
            try {
                for (var i = 0; i < len; i++) {
                    cbs[i].apply(null, args);
                }
            }
            finally {
                if (cbs.length > len) {
                    // added more in the interim.
                    // de-zalgo, just in case, but don't call again.
                    cbs.splice(0, len);
                    process.nextTick(function () {
                        RES.apply(null, args);
                    });
                }
                else {
                    delete reqs[key];
                }
            }
        });
    }
    function slice(args) {
        var length = args.length;
        var array = [];
        for (var i = 0; i < length; i++)
            array[i] = args[i];
        return array;
    }
},
/* wrappy/wrappy.js */ function _(require, module, exports) {
    // Returns a wrapper function that returns a wrapped callback
    // The wrapper function should do some stuff, and return a
    // presumably different callback function.
    // This makes sure that own properties are retained, so that
    // decorations and such are not lost along the way.
    module.exports = wrappy;
    function wrappy(fn, cb) {
        if (fn && cb)
            return wrappy(fn)(cb);
        if (typeof fn !== 'function')
            throw new TypeError('need wrapper function');
        Object.keys(fn).forEach(function (k) {
            wrapper[k] = fn[k];
        });
        return wrapper;
        function wrapper() {
            var args = new Array(arguments.length);
            for (var i = 0; i < args.length; i++) {
                args[i] = arguments[i];
            }
            var ret = fn.apply(this, args);
            var cb = args[args.length - 1];
            if (typeof ret === 'function' && ret !== cb) {
                Object.keys(cb).forEach(function (k) {
                    ret[k] = cb[k];
                });
            }
            return ret;
        }
    }
},
/* once/once.js */ function _(require, module, exports) {
    var wrappy = require(590) /* wrappy */;
    module.exports = wrappy(once);
    module.exports.strict = wrappy(onceStrict);
    once.proto = once(function () {
        Object.defineProperty(Function.prototype, 'once', {
            value: function () {
                return once(this);
            },
            configurable: true
        });
        Object.defineProperty(Function.prototype, 'onceStrict', {
            value: function () {
                return onceStrict(this);
            },
            configurable: true
        });
    });
    function once(fn) {
        var f = function () {
            if (f.called)
                return f.value;
            f.called = true;
            return f.value = fn.apply(this, arguments);
        };
        f.called = false;
        return f;
    }
    function onceStrict(fn) {
        var f = function () {
            if (f.called)
                throw new Error(f.onceError);
            f.called = true;
            return f.value = fn.apply(this, arguments);
        };
        var name = fn.name || 'Function wrapped with `once`';
        f.onceError = name + " shouldn't be called more than once";
        f.called = false;
        return f;
    }
},
/* json-stable-stringify-without-jsonify/index.js */ function _(require, module, exports) {
    module.exports = function (obj, opts) {
        if (!opts)
            opts = {};
        if (typeof opts === 'function')
            opts = { cmp: opts };
        var space = opts.space || '';
        if (typeof space === 'number')
            space = Array(space + 1).join(' ');
        var cycles = (typeof opts.cycles === 'boolean') ? opts.cycles : false;
        var replacer = opts.replacer || function (key, value) { return value; };
        var cmp = opts.cmp && (function (f) {
            return function (node) {
                return function (a, b) {
                    var aobj = { key: a, value: node[a] };
                    var bobj = { key: b, value: node[b] };
                    return f(aobj, bobj);
                };
            };
        })(opts.cmp);
        var seen = [];
        return (function stringify(parent, key, node, level) {
            var indent = space ? ('\n' + new Array(level + 1).join(space)) : '';
            var colonSeparator = space ? ': ' : ':';
            if (node && node.toJSON && typeof node.toJSON === 'function') {
                node = node.toJSON();
            }
            node = replacer.call(parent, key, node);
            if (node === undefined) {
                return;
            }
            if (typeof node !== 'object' || node === null) {
                return JSON.stringify(node);
            }
            if (isArray(node)) {
                var out = [];
                for (var i = 0; i < node.length; i++) {
                    var item = stringify(node, i, node[i], level + 1) || JSON.stringify(null);
                    out.push(indent + space + item);
                }
                return '[' + out.join(',') + indent + ']';
            }
            else {
                if (seen.indexOf(node) !== -1) {
                    if (cycles)
                        return JSON.stringify('__cycle__');
                    throw new TypeError('Converting circular structure to JSON');
                }
                else
                    seen.push(node);
                var keys = objectKeys(node).sort(cmp && cmp(node));
                var out = [];
                for (var i = 0; i < keys.length; i++) {
                    var key = keys[i];
                    var value = stringify(node, key, node[key], level + 1);
                    if (!value)
                        continue;
                    var keyValue = JSON.stringify(key)
                        + colonSeparator
                        + value;
                    ;
                    out.push(indent + space + keyValue);
                }
                seen.splice(seen.indexOf(node), 1);
                return '{' + out.join(',') + indent + '}';
            }
        })({ '': obj }, '', obj, 0);
    };
    var isArray = Array.isArray || function (x) {
        return {}.toString.call(x) === '[object Array]';
    };
    var objectKeys = Object.keys || function (obj) {
        var has = Object.prototype.hasOwnProperty || function () { return true; };
        var keys = [];
        for (var key in obj) {
            if (has.call(obj, key))
                keys.push(key);
        }
        return keys;
    };
},
/* eslint/lib/rule-tester/index.js */ function _(require, module, exports) {
    module.exports = {
        RuleTester: require(594) /* ./rule-tester */
    };
},
/* eslint/lib/rule-tester/rule-tester.js */ function _(require, module, exports) {
    /* global describe, it */
    /*
     * This is a wrapper around mocha to allow for DRY unittests for eslint
     * Format:
     * RuleTester.run("{ruleName}", {
     *      valid: [
     *          "{code}",
     *          { code: "{code}", options: {options}, globals: {globals}, parser: "{parser}", settings: {settings} }
     *      ],
     *      invalid: [
     *          { code: "{code}", errors: {numErrors} },
     *          { code: "{code}", errors: ["{errorMessage}"] },
     *          { code: "{code}", options: {options}, globals: {globals}, parser: "{parser}", settings: {settings}, errors: [{ message: "{errorMessage}", type: "{errorNodeType}"}] }
     *      ]
     *  });
     *
     * Variables:
     * {code} - String that represents the code to be tested
     * {options} - Arguments that are passed to the configurable rules.
     * {globals} - An object representing a list of variables that are
     *             registered as globals
     * {parser} - String representing the parser to use
     * {settings} - An object representing global settings for all rules
     * {numErrors} - If failing case doesn't need to check error message,
     *               this integer will specify how many errors should be
     *               received
     * {errorMessage} - Message that is returned by the rule on failure
     * {errorNodeType} - AST node type that is returned by they rule as
     *                   a cause of the failure.
     */
    //------------------------------------------------------------------------------
    // Requirements
    //------------------------------------------------------------------------------
    const assert = require("assert"), path = require("path"), util = require("util"), lodash = require(111) /* lodash */, { getRuleOptionsSchema, validate } = require(116) /* ../shared/config-validator */, { Linter, SourceCodeFixer, interpolate } = require(85) /* ../linter */;
    const ajv = require(423) /* ../shared/ajv */({ strictDefaults: true });
    //------------------------------------------------------------------------------
    // Private Members
    //------------------------------------------------------------------------------
    /*
     * testerDefaultConfig must not be modified as it allows to reset the tester to
     * the initial default configuration
     */
    const testerDefaultConfig = { rules: {} };
    let defaultConfig = { rules: {} };
    /*
     * List every parameters possible on a test case that are not related to eslint
     * configuration
     */
    const RuleTesterParameters = [
        "code",
        "filename",
        "options",
        "errors",
        "output"
    ];
    const hasOwnProperty = Function.call.bind(Object.hasOwnProperty);
    /**
     * Clones a given value deeply.
     * Note: This ignores `parent` property.
     * @param {any} x A value to clone.
     * @returns {any} A cloned value.
     */
    function cloneDeeplyExcludesParent(x) {
        if (typeof x === "object" && x !== null) {
            if (Array.isArray(x)) {
                return x.map(cloneDeeplyExcludesParent);
            }
            const retv = {};
            for (const key in x) {
                if (key !== "parent" && hasOwnProperty(x, key)) {
                    retv[key] = cloneDeeplyExcludesParent(x[key]);
                }
            }
            return retv;
        }
        return x;
    }
    /**
     * Freezes a given value deeply.
     * @param {any} x A value to freeze.
     * @returns {void}
     */
    function freezeDeeply(x) {
        if (typeof x === "object" && x !== null) {
            if (Array.isArray(x)) {
                x.forEach(freezeDeeply);
            }
            else {
                for (const key in x) {
                    if (key !== "parent" && hasOwnProperty(x, key)) {
                        freezeDeeply(x[key]);
                    }
                }
            }
            Object.freeze(x);
        }
    }
    /**
     * Replace control characters by `\u00xx` form.
     * @param {string} text The text to sanitize.
     * @returns {string} The sanitized text.
     */
    function sanitize(text) {
        return text.replace(/[\u0000-\u0009|\u000b-\u001a]/gu, // eslint-disable-line no-control-regex
        // eslint-disable-line no-control-regex
        c => `\\u${c.codePointAt(0).toString(16).padStart(4, "0")}`);
    }
    //------------------------------------------------------------------------------
    // Public Interface
    //------------------------------------------------------------------------------
    // default separators for testing
    const DESCRIBE = Symbol("describe");
    const IT = Symbol("it");
    /**
     * This is `it` default handler if `it` don't exist.
     * @this {Mocha}
     * @param {string} text The description of the test case.
     * @param {Function} method The logic of the test case.
     * @returns {any} Returned value of `method`.
     */
    function itDefaultHandler(text, method) {
        try {
            return method.call(this);
        }
        catch (err) {
            if (err instanceof assert.AssertionError) {
                err.message += ` (${util.inspect(err.actual)} ${err.operator} ${util.inspect(err.expected)})`;
            }
            throw err;
        }
    }
    /**
     * This is `describe` default handler if `describe` don't exist.
     * @this {Mocha}
     * @param {string} text The description of the test case.
     * @param {Function} method The logic of the test case.
     * @returns {any} Returned value of `method`.
     */
    function describeDefaultHandler(text, method) {
        return method.call(this);
    }
    class RuleTester {
        /**
         * Creates a new instance of RuleTester.
         * @param {Object} [testerConfig] Optional, extra configuration for the tester
         */
        constructor(testerConfig) {
            /**
             * The configuration to use for this tester. Combination of the tester
             * configuration and the default configuration.
             * @type {Object}
             */
            this.testerConfig = lodash.merge(
            // we have to clone because merge uses the first argument for recipient
            lodash.cloneDeep(defaultConfig), testerConfig, { rules: { "rule-tester/validate-ast": "error" } });
            /**
             * Rule definitions to define before tests.
             * @type {Object}
             */
            this.rules = {};
            this.linter = new Linter();
        }
        /**
         * Set the configuration to use for all future tests
         * @param {Object} config the configuration to use.
         * @returns {void}
         */
        static setDefaultConfig(config) {
            if (typeof config !== "object") {
                throw new TypeError("RuleTester.setDefaultConfig: config must be an object");
            }
            defaultConfig = config;
            // Make sure the rules object exists since it is assumed to exist later
            defaultConfig.rules = defaultConfig.rules || {};
        }
        /**
         * Get the current configuration used for all tests
         * @returns {Object} the current configuration
         */
        static getDefaultConfig() {
            return defaultConfig;
        }
        /**
         * Reset the configuration to the initial configuration of the tester removing
         * any changes made until now.
         * @returns {void}
         */
        static resetDefaultConfig() {
            defaultConfig = lodash.cloneDeep(testerDefaultConfig);
        }
        /*
         * If people use `mocha test.js --watch` command, `describe` and `it` function
         * instances are different for each execution. So `describe` and `it` should get fresh instance
         * always.
         */
        static get describe() {
            return (this[DESCRIBE] ||
                (typeof describe === "function" ? describe : describeDefaultHandler));
        }
        static set describe(value) {
            this[DESCRIBE] = value;
        }
        static get it() {
            return (this[IT] ||
                (typeof it === "function" ? it : itDefaultHandler));
        }
        static set it(value) {
            this[IT] = value;
        }
        /**
         * Define a rule for one particular run of tests.
         * @param {string} name The name of the rule to define.
         * @param {Function} rule The rule definition.
         * @returns {void}
         */
        defineRule(name, rule) {
            this.rules[name] = rule;
        }
        /**
         * Adds a new rule test to execute.
         * @param {string} ruleName The name of the rule to run.
         * @param {Function} rule The rule to test.
         * @param {Object} test The collection of tests to run.
         * @returns {void}
         */
        run(ruleName, rule, test) {
            const testerConfig = this.testerConfig, requiredScenarios = ["valid", "invalid"], scenarioErrors = [], linter = this.linter;
            if (lodash.isNil(test) || typeof test !== "object") {
                throw new TypeError(`Test Scenarios for rule ${ruleName} : Could not find test scenario object`);
            }
            requiredScenarios.forEach(scenarioType => {
                if (lodash.isNil(test[scenarioType])) {
                    scenarioErrors.push(`Could not find any ${scenarioType} test scenarios`);
                }
            });
            if (scenarioErrors.length > 0) {
                throw new Error([
                    `Test Scenarios for rule ${ruleName} is invalid:`
                ].concat(scenarioErrors).join("\n"));
            }
            linter.defineRule(ruleName, Object.assign({}, rule, {
                // Create a wrapper rule that freezes the `context` properties.
                create(context) {
                    freezeDeeply(context.options);
                    freezeDeeply(context.settings);
                    freezeDeeply(context.parserOptions);
                    return (typeof rule === "function" ? rule : rule.create)(context);
                }
            }));
            linter.defineRules(this.rules);
            /**
             * Run the rule for the given item
             * @param {string|Object} item Item to run the rule against
             * @returns {Object} Eslint run result
             * @private
             */
            function runRuleForItem(item) {
                let config = lodash.cloneDeep(testerConfig), code, filename, output, beforeAST, afterAST;
                if (typeof item === "string") {
                    code = item;
                }
                else {
                    code = item.code;
                    /*
                     * Assumes everything on the item is a config except for the
                     * parameters used by this tester
                     */
                    const itemConfig = lodash.omit(item, RuleTesterParameters);
                    /*
                     * Create the config object from the tester config and this item
                     * specific configurations.
                     */
                    config = lodash.merge(config, itemConfig);
                }
                if (item.filename) {
                    filename = item.filename;
                }
                if (hasOwnProperty(item, "options")) {
                    assert(Array.isArray(item.options), "options must be an array");
                    config.rules[ruleName] = [1].concat(item.options);
                }
                else {
                    config.rules[ruleName] = 1;
                }
                const schema = getRuleOptionsSchema(rule);
                /*
                 * Setup AST getters.
                 * The goal is to check whether or not AST was modified when
                 * running the rule under test.
                 */
                linter.defineRule("rule-tester/validate-ast", () => ({
                    Program(node) {
                        beforeAST = cloneDeeplyExcludesParent(node);
                    },
                    "Program:exit"(node) {
                        afterAST = node;
                    }
                }));
                if (typeof config.parser === "string") {
                    assert(path.isAbsolute(config.parser), "Parsers provided as strings to RuleTester must be absolute paths");
                    linter.defineParser(config.parser, require(config.parser));
                }
                if (schema) {
                    ajv.validateSchema(schema);
                    if (ajv.errors) {
                        const errors = ajv.errors.map(error => {
                            const field = error.dataPath[0] === "." ? error.dataPath.slice(1) : error.dataPath;
                            return `\t${field}: ${error.message}`;
                        }).join("\n");
                        throw new Error([`Schema for rule ${ruleName} is invalid:`, errors]);
                    }
                    /*
                     * `ajv.validateSchema` checks for errors in the structure of the schema (by comparing the schema against a "meta-schema"),
                     * and it reports those errors individually. However, there are other types of schema errors that only occur when compiling
                     * the schema (e.g. using invalid defaults in a schema), and only one of these errors can be reported at a time. As a result,
                     * the schema is compiled here separately from checking for `validateSchema` errors.
                     */
                    try {
                        ajv.compile(schema);
                    }
                    catch (err) {
                        throw new Error(`Schema for rule ${ruleName} is invalid: ${err.message}`);
                    }
                }
                validate(config, "rule-tester", id => (id === ruleName ? rule : null));
                // Verify the code.
                const messages = linter.verify(code, config, filename);
                // Ignore syntax errors for backward compatibility if `errors` is a number.
                if (typeof item.errors !== "number") {
                    const errorMessage = messages.find(m => m.fatal);
                    assert(!errorMessage, `A fatal parsing error occurred: ${errorMessage && errorMessage.message}`);
                }
                // Verify if autofix makes a syntax error or not.
                if (messages.some(m => m.fix)) {
                    output = SourceCodeFixer.applyFixes(code, messages).output;
                    const errorMessageInFix = linter.verify(output, config, filename).find(m => m.fatal);
                    assert(!errorMessageInFix, `A fatal parsing error occurred in autofix: ${errorMessageInFix && errorMessageInFix.message}`);
                }
                else {
                    output = code;
                }
                return {
                    messages,
                    output,
                    beforeAST,
                    afterAST: cloneDeeplyExcludesParent(afterAST)
                };
            }
            /**
             * Check if the AST was changed
             * @param {ASTNode} beforeAST AST node before running
             * @param {ASTNode} afterAST AST node after running
             * @returns {void}
             * @private
             */
            function assertASTDidntChange(beforeAST, afterAST) {
                if (!lodash.isEqual(beforeAST, afterAST)) {
                    assert.fail("Rule should not modify AST.");
                }
            }
            /**
             * Check if the template is valid or not
             * all valid cases go through this
             * @param {string|Object} item Item to run the rule against
             * @returns {void}
             * @private
             */
            function testValidTemplate(item) {
                const result = runRuleForItem(item);
                const messages = result.messages;
                assert.strictEqual(messages.length, 0, util.format("Should have no errors but had %d: %s", messages.length, util.inspect(messages)));
                assertASTDidntChange(result.beforeAST, result.afterAST);
            }
            /**
             * Asserts that the message matches its expected value. If the expected
             * value is a regular expression, it is checked against the actual
             * value.
             * @param {string} actual Actual value
             * @param {string|RegExp} expected Expected value
             * @returns {void}
             * @private
             */
            function assertMessageMatches(actual, expected) {
                if (expected instanceof RegExp) {
                    // assert.js doesn't have a built-in RegExp match function
                    assert.ok(expected.test(actual), `Expected '${actual}' to match ${expected}`);
                }
                else {
                    assert.strictEqual(actual, expected);
                }
            }
            /**
             * Check if the template is invalid or not
             * all invalid cases go through this.
             * @param {string|Object} item Item to run the rule against
             * @returns {void}
             * @private
             */
            function testInvalidTemplate(item) {
                assert.ok(item.errors || item.errors === 0, `Did not specify errors for an invalid test of ${ruleName}`);
                const result = runRuleForItem(item);
                const messages = result.messages;
                if (typeof item.errors === "number") {
                    assert.strictEqual(messages.length, item.errors, util.format("Should have %d error%s but had %d: %s", item.errors, item.errors === 1 ? "" : "s", messages.length, util.inspect(messages)));
                }
                else {
                    assert.strictEqual(messages.length, item.errors.length, util.format("Should have %d error%s but had %d: %s", item.errors.length, item.errors.length === 1 ? "" : "s", messages.length, util.inspect(messages)));
                    const hasMessageOfThisRule = messages.some(m => m.ruleId === ruleName);
                    for (let i = 0, l = item.errors.length; i < l; i++) {
                        const error = item.errors[i];
                        const message = messages[i];
                        assert(hasMessageOfThisRule, "Error rule name should be the same as the name of the rule being tested");
                        if (typeof error === "string" || error instanceof RegExp) {
                            // Just an error message.
                            assertMessageMatches(message.message, error);
                        }
                        else if (typeof error === "object") {
                            /*
                             * Error object.
                             * This may have a message, messageId, data, node type, line, and/or
                             * column.
                             */
                            if (hasOwnProperty(error, "message")) {
                                assert.ok(!hasOwnProperty(error, "messageId"), "Error should not specify both 'message' and a 'messageId'.");
                                assert.ok(!hasOwnProperty(error, "data"), "Error should not specify both 'data' and 'message'.");
                                assertMessageMatches(message.message, error.message);
                            }
                            else if (hasOwnProperty(error, "messageId")) {
                                assert.ok(hasOwnProperty(rule, "meta") && hasOwnProperty(rule.meta, "messages"), "Error can not use 'messageId' if rule under test doesn't define 'meta.messages'.");
                                if (!hasOwnProperty(rule.meta.messages, error.messageId)) {
                                    const friendlyIDList = `[${Object.keys(rule.meta.messages).map(key => `'${key}'`).join(", ")}]`;
                                    assert(false, `Invalid messageId '${error.messageId}'. Expected one of ${friendlyIDList}.`);
                                }
                                assert.strictEqual(message.messageId, error.messageId, `messageId '${message.messageId}' does not match expected messageId '${error.messageId}'.`);
                                if (hasOwnProperty(error, "data")) {
                                    /*
                                     *  if data was provided, then directly compare the returned message to a synthetic
                                     *  interpolated message using the same message ID and data provided in the test.
                                     *  See https://github.com/eslint/eslint/issues/9890 for context.
                                     */
                                    const unformattedOriginalMessage = rule.meta.messages[error.messageId];
                                    const rehydratedMessage = interpolate(unformattedOriginalMessage, error.data);
                                    assert.strictEqual(message.message, rehydratedMessage, `Hydrated message "${rehydratedMessage}" does not match "${message.message}"`);
                                }
                            }
                            assert.ok(hasOwnProperty(error, "data") ? hasOwnProperty(error, "messageId") : true, "Error must specify 'messageId' if 'data' is used.");
                            if (error.type) {
                                assert.strictEqual(message.nodeType, error.type, `Error type should be ${error.type}, found ${message.nodeType}`);
                            }
                            if (hasOwnProperty(error, "line")) {
                                assert.strictEqual(message.line, error.line, `Error line should be ${error.line}`);
                            }
                            if (hasOwnProperty(error, "column")) {
                                assert.strictEqual(message.column, error.column, `Error column should be ${error.column}`);
                            }
                            if (hasOwnProperty(error, "endLine")) {
                                assert.strictEqual(message.endLine, error.endLine, `Error endLine should be ${error.endLine}`);
                            }
                            if (hasOwnProperty(error, "endColumn")) {
                                assert.strictEqual(message.endColumn, error.endColumn, `Error endColumn should be ${error.endColumn}`);
                            }
                            if (hasOwnProperty(error, "suggestions")) {
                                // Support asserting there are no suggestions
                                if (!error.suggestions || (Array.isArray(error.suggestions) && error.suggestions.length === 0)) {
                                    if (Array.isArray(message.suggestions) && message.suggestions.length > 0) {
                                        assert.fail(`Error should have no suggestions on error with message: "${message.message}"`);
                                    }
                                }
                                else {
                                    assert.strictEqual(Array.isArray(message.suggestions), true, `Error should have an array of suggestions. Instead received "${message.suggestions}" on error with message: "${message.message}"`);
                                    assert.strictEqual(message.suggestions.length, error.suggestions.length, `Error should have ${error.suggestions.length} suggestions. Instead found ${message.suggestions.length} suggestions`);
                                    error.suggestions.forEach((expectedSuggestion, index) => {
                                        const actualSuggestion = message.suggestions[index];
                                        /**
                                         * Tests equality of a suggestion key if that key is defined in the expected output.
                                         * @param {string} key Key to validate from the suggestion object
                                         * @returns {void}
                                         */
                                        function assertSuggestionKeyEquals(key) {
                                            if (hasOwnProperty(expectedSuggestion, key)) {
                                                assert.deepStrictEqual(actualSuggestion[key], expectedSuggestion[key], `Error suggestion at index: ${index} should have desc of: "${actualSuggestion[key]}"`);
                                            }
                                        }
                                        assertSuggestionKeyEquals("desc");
                                        assertSuggestionKeyEquals("messageId");
                                        if (hasOwnProperty(expectedSuggestion, "output")) {
                                            const codeWithAppliedSuggestion = SourceCodeFixer.applyFixes(item.code, [actualSuggestion]).output;
                                            assert.strictEqual(codeWithAppliedSuggestion, expectedSuggestion.output, `Expected the applied suggestion fix to match the test suggestion output for suggestion at index: ${index} on error with message: "${message.message}"`);
                                        }
                                    });
                                }
                            }
                        }
                        else {
                            // Message was an unexpected type
                            assert.fail(`Error should be a string, object, or RegExp, but found (${util.inspect(message)})`);
                        }
                    }
                }
                if (hasOwnProperty(item, "output")) {
                    if (item.output === null) {
                        assert.strictEqual(result.output, item.code, "Expected no autofixes to be suggested");
                    }
                    else {
                        assert.strictEqual(result.output, item.output, "Output is incorrect.");
                    }
                }
                assertASTDidntChange(result.beforeAST, result.afterAST);
            }
            /*
             * This creates a mocha test suite and pipes all supplied info through
             * one of the templates above.
             */
            RuleTester.describe(ruleName, () => {
                RuleTester.describe("valid", () => {
                    test.valid.forEach(valid => {
                        RuleTester.it(sanitize(typeof valid === "object" ? valid.code : valid), () => {
                            testValidTemplate(valid);
                        });
                    });
                });
                RuleTester.describe("invalid", () => {
                    test.invalid.forEach(invalid => {
                        RuleTester.it(sanitize(invalid.code), () => {
                            testInvalidTemplate(invalid);
                        });
                    });
                });
            });
        }
    }
    RuleTester[DESCRIBE] = RuleTester[IT] = null;
    module.exports = RuleTester;
},
/* @typescript-eslint/eslint-plugin/dist/index.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    const rules_1 = __importDefault(require(596) /* ./rules */);
    const all_json_1 = __importDefault(require(742) /* ./configs/all.json */);
    const base_json_1 = __importDefault(require(743) /* ./configs/base.json */);
    const recommended_json_1 = __importDefault(require(744) /* ./configs/recommended.json */);
    const recommended_requiring_type_checking_json_1 = __importDefault(require(745) /* ./configs/recommended-requiring-type-checking.json */);
    const eslint_recommended_1 = __importDefault(require(746) /* ./configs/eslint-recommended */);
    module.exports = {
        rules: rules_1.default,
        configs: {
            all: all_json_1.default,
            base: base_json_1.default,
            recommended: recommended_json_1.default,
            'eslint-recommended': eslint_recommended_1.default,
            'recommended-requiring-type-checking': recommended_requiring_type_checking_json_1.default,
        },
    };
},
/* @typescript-eslint/eslint-plugin/dist/rules/index.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const adjacent_overload_signatures_1 = __importDefault(require(597) /* ./adjacent-overload-signatures */);
    const array_type_1 = __importDefault(require(650) /* ./array-type */);
    const await_thenable_1 = __importDefault(require(651) /* ./await-thenable */);
    const ban_ts_comment_1 = __importDefault(require(652) /* ./ban-ts-comment */);
    const ban_ts_ignore_1 = __importDefault(require(653) /* ./ban-ts-ignore */);
    const ban_types_1 = __importDefault(require(654) /* ./ban-types */);
    const brace_style_1 = __importDefault(require(655) /* ./brace-style */);
    const camelcase_1 = __importDefault(require(656) /* ./camelcase */);
    const class_name_casing_1 = __importDefault(require(657) /* ./class-name-casing */);
    const comma_spacing_1 = __importDefault(require(658) /* ./comma-spacing */);
    const consistent_type_assertions_1 = __importDefault(require(659) /* ./consistent-type-assertions */);
    const consistent_type_definitions_1 = __importDefault(require(660) /* ./consistent-type-definitions */);
    const default_param_last_1 = __importDefault(require(661) /* ./default-param-last */);
    const explicit_function_return_type_1 = __importDefault(require(662) /* ./explicit-function-return-type */);
    const explicit_member_accessibility_1 = __importDefault(require(664) /* ./explicit-member-accessibility */);
    const explicit_module_boundary_types_1 = __importDefault(require(665) /* ./explicit-module-boundary-types */);
    const func_call_spacing_1 = __importDefault(require(666) /* ./func-call-spacing */);
    const generic_type_naming_1 = __importDefault(require(667) /* ./generic-type-naming */);
    const indent_1 = __importDefault(require(668) /* ./indent */);
    const interface_name_prefix_1 = __importDefault(require(669) /* ./interface-name-prefix */);
    const member_delimiter_style_1 = __importDefault(require(670) /* ./member-delimiter-style */);
    const member_naming_1 = __importDefault(require(671) /* ./member-naming */);
    const member_ordering_1 = __importDefault(require(672) /* ./member-ordering */);
    const naming_convention_1 = __importDefault(require(673) /* ./naming-convention */);
    const no_array_constructor_1 = __importDefault(require(674) /* ./no-array-constructor */);
    const no_base_to_string_1 = __importDefault(require(675) /* ./no-base-to-string */);
    const no_dupe_class_members_1 = __importDefault(require(676) /* ./no-dupe-class-members */);
    const no_dynamic_delete_1 = __importDefault(require(677) /* ./no-dynamic-delete */);
    const no_empty_function_1 = __importDefault(require(678) /* ./no-empty-function */);
    const no_empty_interface_1 = __importDefault(require(679) /* ./no-empty-interface */);
    const no_explicit_any_1 = __importDefault(require(680) /* ./no-explicit-any */);
    const no_extraneous_class_1 = __importDefault(require(681) /* ./no-extraneous-class */);
    const no_extra_non_null_assertion_1 = __importDefault(require(682) /* ./no-extra-non-null-assertion */);
    const no_extra_parens_1 = __importDefault(require(683) /* ./no-extra-parens */);
    const no_extra_semi_1 = __importDefault(require(684) /* ./no-extra-semi */);
    const no_floating_promises_1 = __importDefault(require(685) /* ./no-floating-promises */);
    const no_for_in_array_1 = __importDefault(require(686) /* ./no-for-in-array */);
    const no_implied_eval_1 = __importDefault(require(687) /* ./no-implied-eval */);
    const no_inferrable_types_1 = __importDefault(require(688) /* ./no-inferrable-types */);
    const no_magic_numbers_1 = __importDefault(require(689) /* ./no-magic-numbers */);
    const no_misused_new_1 = __importDefault(require(690) /* ./no-misused-new */);
    const no_misused_promises_1 = __importDefault(require(691) /* ./no-misused-promises */);
    const no_namespace_1 = __importDefault(require(692) /* ./no-namespace */);
    const no_non_null_asserted_optional_chain_1 = __importDefault(require(693) /* ./no-non-null-asserted-optional-chain */);
    const no_non_null_assertion_1 = __importDefault(require(694) /* ./no-non-null-assertion */);
    const no_parameter_properties_1 = __importDefault(require(695) /* ./no-parameter-properties */);
    const no_require_imports_1 = __importDefault(require(696) /* ./no-require-imports */);
    const no_this_alias_1 = __importDefault(require(697) /* ./no-this-alias */);
    const no_throw_literal_1 = __importDefault(require(698) /* ./no-throw-literal */);
    const no_type_alias_1 = __importDefault(require(699) /* ./no-type-alias */);
    const no_unnecessary_boolean_literal_compare_1 = __importDefault(require(700) /* ./no-unnecessary-boolean-literal-compare */);
    const no_unnecessary_condition_1 = __importDefault(require(701) /* ./no-unnecessary-condition */);
    const no_unnecessary_qualifier_1 = __importDefault(require(702) /* ./no-unnecessary-qualifier */);
    const no_unnecessary_type_arguments_1 = __importDefault(require(703) /* ./no-unnecessary-type-arguments */);
    const no_unnecessary_type_assertion_1 = __importDefault(require(704) /* ./no-unnecessary-type-assertion */);
    const no_unsafe_call_1 = __importDefault(require(705) /* ./no-unsafe-call */);
    const no_unsafe_member_access_1 = __importDefault(require(706) /* ./no-unsafe-member-access */);
    const no_unsafe_return_1 = __importDefault(require(707) /* ./no-unsafe-return */);
    const no_untyped_public_signature_1 = __importDefault(require(708) /* ./no-untyped-public-signature */);
    const no_unused_expressions_1 = __importDefault(require(709) /* ./no-unused-expressions */);
    const no_unused_vars_1 = __importDefault(require(710) /* ./no-unused-vars */);
    const no_unused_vars_experimental_1 = __importDefault(require(711) /* ./no-unused-vars-experimental */);
    const no_use_before_define_1 = __importDefault(require(712) /* ./no-use-before-define */);
    const no_useless_constructor_1 = __importDefault(require(713) /* ./no-useless-constructor */);
    const no_var_requires_1 = __importDefault(require(714) /* ./no-var-requires */);
    const prefer_as_const_1 = __importDefault(require(715) /* ./prefer-as-const */);
    const prefer_for_of_1 = __importDefault(require(716) /* ./prefer-for-of */);
    const prefer_function_type_1 = __importDefault(require(717) /* ./prefer-function-type */);
    const prefer_includes_1 = __importDefault(require(718) /* ./prefer-includes */);
    const prefer_namespace_keyword_1 = __importDefault(require(719) /* ./prefer-namespace-keyword */);
    const prefer_nullish_coalescing_1 = __importDefault(require(720) /* ./prefer-nullish-coalescing */);
    const prefer_optional_chain_1 = __importDefault(require(721) /* ./prefer-optional-chain */);
    const prefer_readonly_1 = __importDefault(require(722) /* ./prefer-readonly */);
    const prefer_readonly_parameter_types_1 = __importDefault(require(723) /* ./prefer-readonly-parameter-types */);
    const prefer_regexp_exec_1 = __importDefault(require(724) /* ./prefer-regexp-exec */);
    const prefer_string_starts_ends_with_1 = __importDefault(require(725) /* ./prefer-string-starts-ends-with */);
    const promise_function_async_1 = __importDefault(require(726) /* ./promise-function-async */);
    const quotes_1 = __importDefault(require(727) /* ./quotes */);
    const require_array_sort_compare_1 = __importDefault(require(728) /* ./require-array-sort-compare */);
    const require_await_1 = __importDefault(require(729) /* ./require-await */);
    const restrict_plus_operands_1 = __importDefault(require(730) /* ./restrict-plus-operands */);
    const restrict_template_expressions_1 = __importDefault(require(731) /* ./restrict-template-expressions */);
    const return_await_1 = __importDefault(require(732) /* ./return-await */);
    const semi_1 = __importDefault(require(733) /* ./semi */);
    const space_before_function_paren_1 = __importDefault(require(734) /* ./space-before-function-paren */);
    const strict_boolean_expressions_1 = __importDefault(require(735) /* ./strict-boolean-expressions */);
    const switch_exhaustiveness_check_1 = __importDefault(require(736) /* ./switch-exhaustiveness-check */);
    const triple_slash_reference_1 = __importDefault(require(737) /* ./triple-slash-reference */);
    const type_annotation_spacing_1 = __importDefault(require(738) /* ./type-annotation-spacing */);
    const typedef_1 = __importDefault(require(739) /* ./typedef */);
    const unbound_method_1 = __importDefault(require(740) /* ./unbound-method */);
    const unified_signatures_1 = __importDefault(require(741) /* ./unified-signatures */);
    exports.default = {
        'adjacent-overload-signatures': adjacent_overload_signatures_1.default,
        'array-type': array_type_1.default,
        'await-thenable': await_thenable_1.default,
        'ban-ts-comment': ban_ts_comment_1.default,
        'ban-ts-ignore': ban_ts_ignore_1.default,
        'ban-types': ban_types_1.default,
        'no-base-to-string': no_base_to_string_1.default,
        'brace-style': brace_style_1.default,
        camelcase: camelcase_1.default,
        'class-name-casing': class_name_casing_1.default,
        'comma-spacing': comma_spacing_1.default,
        'consistent-type-assertions': consistent_type_assertions_1.default,
        'consistent-type-definitions': consistent_type_definitions_1.default,
        'default-param-last': default_param_last_1.default,
        'explicit-function-return-type': explicit_function_return_type_1.default,
        'explicit-member-accessibility': explicit_member_accessibility_1.default,
        'explicit-module-boundary-types': explicit_module_boundary_types_1.default,
        'func-call-spacing': func_call_spacing_1.default,
        'generic-type-naming': generic_type_naming_1.default,
        indent: indent_1.default,
        'interface-name-prefix': interface_name_prefix_1.default,
        'member-delimiter-style': member_delimiter_style_1.default,
        'member-naming': member_naming_1.default,
        'member-ordering': member_ordering_1.default,
        'naming-convention': naming_convention_1.default,
        'no-array-constructor': no_array_constructor_1.default,
        'no-dupe-class-members': no_dupe_class_members_1.default,
        'no-dynamic-delete': no_dynamic_delete_1.default,
        'no-empty-function': no_empty_function_1.default,
        'no-empty-interface': no_empty_interface_1.default,
        'no-explicit-any': no_explicit_any_1.default,
        'no-extra-non-null-assertion': no_extra_non_null_assertion_1.default,
        'no-extra-parens': no_extra_parens_1.default,
        'no-extra-semi': no_extra_semi_1.default,
        'no-extraneous-class': no_extraneous_class_1.default,
        'no-floating-promises': no_floating_promises_1.default,
        'no-for-in-array': no_for_in_array_1.default,
        'no-implied-eval': no_implied_eval_1.default,
        'no-inferrable-types': no_inferrable_types_1.default,
        'no-magic-numbers': no_magic_numbers_1.default,
        'no-misused-new': no_misused_new_1.default,
        'no-misused-promises': no_misused_promises_1.default,
        'no-namespace': no_namespace_1.default,
        'no-non-null-asserted-optional-chain': no_non_null_asserted_optional_chain_1.default,
        'no-non-null-assertion': no_non_null_assertion_1.default,
        'no-parameter-properties': no_parameter_properties_1.default,
        'no-require-imports': no_require_imports_1.default,
        'no-this-alias': no_this_alias_1.default,
        'no-throw-literal': no_throw_literal_1.default,
        'no-type-alias': no_type_alias_1.default,
        'no-unnecessary-boolean-literal-compare': no_unnecessary_boolean_literal_compare_1.default,
        'no-unnecessary-condition': no_unnecessary_condition_1.default,
        'no-unnecessary-qualifier': no_unnecessary_qualifier_1.default,
        'no-unnecessary-type-arguments': no_unnecessary_type_arguments_1.default,
        'no-unnecessary-type-assertion': no_unnecessary_type_assertion_1.default,
        'no-unsafe-call': no_unsafe_call_1.default,
        'no-unsafe-member-access': no_unsafe_member_access_1.default,
        'no-unsafe-return': no_unsafe_return_1.default,
        'no-untyped-public-signature': no_untyped_public_signature_1.default,
        'no-unused-expressions': no_unused_expressions_1.default,
        'no-unused-vars-experimental': no_unused_vars_experimental_1.default,
        'no-unused-vars': no_unused_vars_1.default,
        'no-use-before-define': no_use_before_define_1.default,
        'no-useless-constructor': no_useless_constructor_1.default,
        'no-var-requires': no_var_requires_1.default,
        'prefer-as-const': prefer_as_const_1.default,
        'prefer-for-of': prefer_for_of_1.default,
        'prefer-function-type': prefer_function_type_1.default,
        'prefer-includes': prefer_includes_1.default,
        'prefer-namespace-keyword': prefer_namespace_keyword_1.default,
        'prefer-nullish-coalescing': prefer_nullish_coalescing_1.default,
        'prefer-optional-chain': prefer_optional_chain_1.default,
        'prefer-readonly-parameter-types': prefer_readonly_parameter_types_1.default,
        'prefer-readonly': prefer_readonly_1.default,
        'prefer-regexp-exec': prefer_regexp_exec_1.default,
        'prefer-string-starts-ends-with': prefer_string_starts_ends_with_1.default,
        'promise-function-async': promise_function_async_1.default,
        quotes: quotes_1.default,
        'require-array-sort-compare': require_array_sort_compare_1.default,
        'require-await': require_await_1.default,
        'restrict-plus-operands': restrict_plus_operands_1.default,
        'restrict-template-expressions': restrict_template_expressions_1.default,
        'return-await': return_await_1.default,
        semi: semi_1.default,
        'space-before-function-paren': space_before_function_paren_1.default,
        'strict-boolean-expressions': strict_boolean_expressions_1.default,
        'switch-exhaustiveness-check': switch_exhaustiveness_check_1.default,
        'triple-slash-reference': triple_slash_reference_1.default,
        'type-annotation-spacing': type_annotation_spacing_1.default,
        typedef: typedef_1.default,
        'unbound-method': unbound_method_1.default,
        'unified-signatures': unified_signatures_1.default,
    };
},
/* @typescript-eslint/eslint-plugin/dist/rules/adjacent-overload-signatures.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'adjacent-overload-signatures',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Require that member overloads be consecutive',
                category: 'Best Practices',
                recommended: 'error',
            },
            schema: [],
            messages: {
                adjacentSignature: "All '{{name}}' signatures should be adjacent.",
            },
        },
        defaultOptions: [],
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Gets the name of the member being processed.
             * @param member the member being processed.
             * @returns the name of the member or null if it's a member not relevant to the rule.
             */
            function getMemberName(member) {
                if (!member) {
                    return null;
                }
                switch (member.type) {
                    case experimental_utils_1.AST_NODE_TYPES.ExportDefaultDeclaration:
                    case experimental_utils_1.AST_NODE_TYPES.ExportNamedDeclaration: {
                        // export statements (e.g. export { a };)
                        // have no declarations, so ignore them
                        if (!member.declaration) {
                            return null;
                        }
                        return getMemberName(member.declaration);
                    }
                    case experimental_utils_1.AST_NODE_TYPES.TSDeclareFunction:
                    case experimental_utils_1.AST_NODE_TYPES.FunctionDeclaration:
                        return member.id && member.id.name;
                    case experimental_utils_1.AST_NODE_TYPES.TSMethodSignature:
                        return util.getNameFromMember(member, sourceCode);
                    case experimental_utils_1.AST_NODE_TYPES.TSCallSignatureDeclaration:
                        return 'call';
                    case experimental_utils_1.AST_NODE_TYPES.TSConstructSignatureDeclaration:
                        return 'new';
                    case experimental_utils_1.AST_NODE_TYPES.MethodDefinition:
                        return util.getNameFromMember(member, sourceCode);
                }
                return null;
            }
            function isSameMethod(method1, method2) {
                return (!!method2 &&
                    method1.name === method2.name &&
                    method1.static === method2.static);
            }
            function getMembers(node) {
                switch (node.type) {
                    case experimental_utils_1.AST_NODE_TYPES.ClassBody:
                    case experimental_utils_1.AST_NODE_TYPES.Program:
                    case experimental_utils_1.AST_NODE_TYPES.TSModuleBlock:
                    case experimental_utils_1.AST_NODE_TYPES.TSInterfaceBody:
                        return node.body;
                    case experimental_utils_1.AST_NODE_TYPES.TSTypeLiteral:
                        return node.members;
                }
            }
            /**
             * Check the body for overload methods.
             * @param {ASTNode} node the body to be inspected.
             */
            function checkBodyForOverloadMethods(node) {
                const members = getMembers(node);
                if (members) {
                    let lastMethod = null;
                    const seenMethods = [];
                    members.forEach(member => {
                        const name = getMemberName(member);
                        if (name === null) {
                            lastMethod = null;
                            return;
                        }
                        const method = {
                            name,
                            static: 'static' in member && !!member.static,
                        };
                        const index = seenMethods.findIndex(seenMethod => isSameMethod(method, seenMethod));
                        if (index > -1 && !isSameMethod(method, lastMethod)) {
                            context.report({
                                node: member,
                                messageId: 'adjacentSignature',
                                data: {
                                    name: (method.static ? 'static ' : '') + method.name,
                                },
                            });
                        }
                        else if (index === -1) {
                            seenMethods.push(method);
                        }
                        lastMethod = method;
                    });
                }
            }
            return {
                ClassBody: checkBodyForOverloadMethods,
                Program: checkBodyForOverloadMethods,
                TSModuleBlock: checkBodyForOverloadMethods,
                TSTypeLiteral: checkBodyForOverloadMethods,
                TSInterfaceBody: checkBodyForOverloadMethods,
            };
        },
    });
},
/* @typescript-eslint/experimental-utils/dist/index.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ESLintUtils = __importStar(require(599) /* ./eslint-utils */);
    exports.ESLintUtils = ESLintUtils;
    const TSESLint = __importStar(require(605) /* ./ts-eslint */);
    exports.TSESLint = TSESLint;
    const TSESLintScope = __importStar(require(609) /* ./ts-eslint-scope */);
    exports.TSESLintScope = TSESLintScope;
    const JSONSchema = __importStar(require(618) /* ./json-schema */);
    exports.JSONSchema = JSONSchema;
    // for convenience's sake - export the types directly from here so consumers
    // don't need to reference/install both packages in their code
    // NOTE - this uses hard links inside ts-estree to avoid initialization of entire package
    //        via its main file (which imports typescript at runtime).
    //        Not every eslint-plugin written in typescript requires typescript at runtime.
    var ts_estree_1 = require(619) /* @typescript-eslint/typescript-estree/dist/ts-estree */;
    exports.AST_NODE_TYPES = ts_estree_1.AST_NODE_TYPES;
    exports.AST_TOKEN_TYPES = ts_estree_1.AST_TOKEN_TYPES;
    exports.TSESTree = ts_estree_1.TSESTree;
},
/* @typescript-eslint/experimental-utils/dist/eslint-utils/index.js */ function _(require, module, exports) {
    function __export(m) {
        for (var p in m)
            if (!exports.hasOwnProperty(p))
                exports[p] = m[p];
    }
    Object.defineProperty(exports, "__esModule", { value: true });
    __export(require(600) /* ./applyDefault */);
    __export(require(602) /* ./batchedSingleLineTests */);
    __export(require(603) /* ./getParserServices */);
    __export(require(604) /* ./RuleCreator */);
    __export(require(601) /* ./deepMerge */);
},
/* @typescript-eslint/experimental-utils/dist/eslint-utils/applyDefault.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const deepMerge_1 = require(601) /* ./deepMerge */;
    /**
     * Pure function - doesn't mutate either parameter!
     * Uses the default options and overrides with the options provided by the user
     * @param defaultOptions the defaults
     * @param userOptions the user opts
     * @returns the options with defaults
     */
    function applyDefault(defaultOptions, userOptions) {
        // clone defaults
        const options = JSON.parse(JSON.stringify(defaultOptions));
        if (userOptions === null || userOptions === undefined) {
            return options;
        }
        options.forEach((opt, i) => {
            if (userOptions[i] !== undefined) {
                const userOpt = userOptions[i];
                if (deepMerge_1.isObjectNotArray(userOpt) && deepMerge_1.isObjectNotArray(opt)) {
                    options[i] = deepMerge_1.deepMerge(opt, userOpt);
                }
                else {
                    options[i] = userOpt;
                }
            }
        });
        return options;
    }
    exports.applyDefault = applyDefault;
},
/* @typescript-eslint/experimental-utils/dist/eslint-utils/deepMerge.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * Check if the variable contains an object strictly rejecting arrays
     * @param obj an object
     * @returns `true` if obj is an object
     */
    function isObjectNotArray(obj) {
        return typeof obj === 'object' && !Array.isArray(obj);
    }
    exports.isObjectNotArray = isObjectNotArray;
    /**
     * Pure function - doesn't mutate either parameter!
     * Merges two objects together deeply, overwriting the properties in first with the properties in second
     * @param first The first object
     * @param second The second object
     * @returns a new object
     */
    function deepMerge(first = {}, second = {}) {
        // get the unique set of keys across both objects
        const keys = new Set(Object.keys(first).concat(Object.keys(second)));
        return Array.from(keys).reduce((acc, key) => {
            const firstHasKey = key in first;
            const secondHasKey = key in second;
            const firstValue = first[key];
            const secondValue = second[key];
            if (firstHasKey && secondHasKey) {
                if (isObjectNotArray(firstValue) && isObjectNotArray(secondValue)) {
                    // object type
                    acc[key] = deepMerge(firstValue, secondValue);
                }
                else {
                    // value type
                    acc[key] = secondValue;
                }
            }
            else if (firstHasKey) {
                acc[key] = firstValue;
            }
            else {
                acc[key] = secondValue;
            }
            return acc;
        }, {});
    }
    exports.deepMerge = deepMerge;
},
/* @typescript-eslint/experimental-utils/dist/eslint-utils/batchedSingleLineTests.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function batchedSingleLineTests(options) {
        // eslint counts lines from 1
        const lineOffset = options.code.startsWith('\n') ? 2 : 1;
        const output = 'output' in options && options.output
            ? options.output.trim().split('\n')
            : null;
        return options.code
            .trim()
            .split('\n')
            .map((code, i) => {
            const lineNum = i + lineOffset;
            const errors = 'errors' in options
                ? options.errors.filter(e => e.line === lineNum)
                : [];
            const returnVal = Object.assign(Object.assign({}, options), { code, errors: errors.map(e => (Object.assign(Object.assign({}, e), { line: 1 }))) });
            if (output === null || output === void 0 ? void 0 : output[i]) {
                return Object.assign(Object.assign({}, returnVal), { output: output[i] });
            }
            return returnVal;
        });
    }
    exports.batchedSingleLineTests = batchedSingleLineTests;
},
/* @typescript-eslint/experimental-utils/dist/eslint-utils/getParserServices.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const ERROR_MESSAGE = 'You have used a rule which requires parserServices to be generated. You must therefore provide a value for the "parserOptions.project" property for @typescript-eslint/parser.';
    /**
     * Try to retrieve typescript parser service from context
     */
    function getParserServices(context) {
        if (!context.parserServices ||
            !context.parserServices.program ||
            !context.parserServices.esTreeNodeToTSNodeMap) {
            /**
             * The user needs to have configured "project" in their parserOptions
             * for @typescript-eslint/parser
             */
            throw new Error(ERROR_MESSAGE);
        }
        return context.parserServices;
    }
    exports.getParserServices = getParserServices;
},
/* @typescript-eslint/experimental-utils/dist/eslint-utils/RuleCreator.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const applyDefault_1 = require(600) /* ./applyDefault */;
    function RuleCreator(urlCreator) {
        // This function will get much easier to call when this is merged https://github.com/Microsoft/TypeScript/pull/26349
        // TODO - when the above PR lands; add type checking for the context.report `data` property
        return function createRule({ name, meta, defaultOptions, create, }) {
            return {
                meta: Object.assign(Object.assign({}, meta), { docs: Object.assign(Object.assign({}, meta.docs), { url: urlCreator(name) }) }),
                create(context) {
                    const optionsWithDefault = applyDefault_1.applyDefault(defaultOptions, context.options);
                    return create(context, optionsWithDefault);
                },
            };
        };
    }
    exports.RuleCreator = RuleCreator;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint/index.js */ function _(require, module, exports) {
    function __export(m) {
        for (var p in m)
            if (!exports.hasOwnProperty(p))
                exports[p] = m[p];
    }
    Object.defineProperty(exports, "__esModule", { value: true });
    __export(require(606) /* ./Linter */);
    __export(require(607) /* ./RuleTester */);
    __export(require(608) /* ./SourceCode */);
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint/Linter.js */ function _(require, module, exports) {
    /* eslint-disable @typescript-eslint/no-namespace */
    Object.defineProperty(exports, "__esModule", { value: true });
    const eslint_1 = require(77) /* eslint */;
    const Linter = eslint_1.Linter;
    exports.Linter = Linter;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint/RuleTester.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const eslint_1 = require(77) /* eslint */;
    // the cast on the extends is so that we don't want to have the built type defs to attempt to import eslint
    class RuleTester extends eslint_1.RuleTester {
        constructor(config) {
            var _a;
            super(config);
            // nobody will ever need watching in tests
            // so we can give everyone a perf win by disabling watching
            if ((_a = config === null || config === void 0 ? void 0 : config.parserOptions) === null || _a === void 0 ? void 0 : _a.project) {
                config.parserOptions.noWatch =
                    typeof config.parserOptions.noWatch === 'boolean' || true;
            }
        }
        run(name, rule, tests) {
            // this method is only defined here because we lazily type the eslint import with `any`
            super.run(name, rule, tests);
        }
    }
    exports.RuleTester = RuleTester;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint/SourceCode.js */ function _(require, module, exports) {
    /* eslint-disable @typescript-eslint/no-namespace */
    Object.defineProperty(exports, "__esModule", { value: true });
    const eslint_1 = require(77) /* eslint */;
    const SourceCode = eslint_1.SourceCode;
    exports.SourceCode = SourceCode;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint-scope/index.js */ function _(require, module, exports) {
    function __export(m) {
        for (var p in m)
            if (!exports.hasOwnProperty(p))
                exports[p] = m[p];
    }
    Object.defineProperty(exports, "__esModule", { value: true });
    const eslint_scope_1 = require(87) /* eslint-scope */;
    __export(require(610) /* ./analyze */);
    __export(require(611) /* ./Definition */);
    __export(require(612) /* ./PatternVisitor */);
    __export(require(613) /* ./Reference */);
    __export(require(614) /* ./Referencer */);
    __export(require(615) /* ./Scope */);
    __export(require(616) /* ./ScopeManager */);
    __export(require(617) /* ./Variable */);
    exports.version = eslint_scope_1.version;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint-scope/analyze.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const eslint_scope_1 = require(87) /* eslint-scope */;
    const analyze = eslint_scope_1.analyze;
    exports.analyze = analyze;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint-scope/Definition.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const definition_1 = require(94) /* eslint-scope/lib/definition */;
    const Definition = definition_1.Definition;
    exports.Definition = Definition;
    const ParameterDefinition = definition_1.ParameterDefinition;
    exports.ParameterDefinition = ParameterDefinition;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint-scope/PatternVisitor.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const pattern_visitor_1 = __importDefault(require(98) /* eslint-scope/lib/pattern-visitor */);
    const PatternVisitor = pattern_visitor_1.default;
    exports.PatternVisitor = PatternVisitor;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint-scope/Reference.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const reference_1 = __importDefault(require(92) /* eslint-scope/lib/reference */);
    const Reference = reference_1.default;
    exports.Reference = Reference;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint-scope/Referencer.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const referencer_1 = __importDefault(require(95) /* eslint-scope/lib/referencer */);
    const Referencer = referencer_1.default;
    exports.Referencer = Referencer;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint-scope/Scope.js */ function _(require, module, exports) {
    /* eslint-disable @typescript-eslint/no-empty-interface, @typescript-eslint/no-explicit-any */
    Object.defineProperty(exports, "__esModule", { value: true });
    const scope_1 = require(89) /* eslint-scope/lib/scope */;
    const Scope = scope_1.Scope;
    exports.Scope = Scope;
    const GlobalScope = scope_1.GlobalScope;
    exports.GlobalScope = GlobalScope;
    const ModuleScope = scope_1.ModuleScope;
    exports.ModuleScope = ModuleScope;
    const FunctionExpressionNameScope = scope_1.FunctionExpressionNameScope;
    exports.FunctionExpressionNameScope = FunctionExpressionNameScope;
    const CatchScope = scope_1.CatchScope;
    exports.CatchScope = CatchScope;
    const WithScope = scope_1.WithScope;
    exports.WithScope = WithScope;
    const BlockScope = scope_1.BlockScope;
    exports.BlockScope = BlockScope;
    const SwitchScope = scope_1.SwitchScope;
    exports.SwitchScope = SwitchScope;
    const FunctionScope = scope_1.FunctionScope;
    exports.FunctionScope = FunctionScope;
    const ForScope = scope_1.ForScope;
    exports.ForScope = ForScope;
    const ClassScope = scope_1.ClassScope;
    exports.ClassScope = ClassScope;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint-scope/ScopeManager.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const scope_manager_1 = __importDefault(require(88) /* eslint-scope/lib/scope-manager */);
    const ScopeManager = scope_manager_1.default;
    exports.ScopeManager = ScopeManager;
},
/* @typescript-eslint/experimental-utils/dist/ts-eslint-scope/Variable.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const variable_1 = __importDefault(require(93) /* eslint-scope/lib/variable */);
    const Variable = variable_1.default;
    exports.Variable = Variable;
},
/* @typescript-eslint/experimental-utils/dist/json-schema.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
},
/* @typescript-eslint/typescript-estree/dist/ts-estree/index.js */ function _(require, module, exports) {
    function __export(m) {
        for (var p in m)
            if (!exports.hasOwnProperty(p))
                exports[p] = m[p];
    }
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const TSESTree = __importStar(require(620) /* ./ts-estree */);
    exports.TSESTree = TSESTree;
    __export(require(621) /* ./ast-node-types */);
},
/* @typescript-eslint/typescript-estree/dist/ts-estree/ts-estree.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
},
/* @typescript-eslint/typescript-estree/dist/ts-estree/ast-node-types.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var AST_NODE_TYPES;
    (function (AST_NODE_TYPES) {
        AST_NODE_TYPES["ArrayExpression"] = "ArrayExpression";
        AST_NODE_TYPES["ArrayPattern"] = "ArrayPattern";
        AST_NODE_TYPES["ArrowFunctionExpression"] = "ArrowFunctionExpression";
        AST_NODE_TYPES["AssignmentExpression"] = "AssignmentExpression";
        AST_NODE_TYPES["AssignmentPattern"] = "AssignmentPattern";
        AST_NODE_TYPES["AwaitExpression"] = "AwaitExpression";
        AST_NODE_TYPES["BigIntLiteral"] = "BigIntLiteral";
        AST_NODE_TYPES["BinaryExpression"] = "BinaryExpression";
        AST_NODE_TYPES["BlockStatement"] = "BlockStatement";
        AST_NODE_TYPES["BreakStatement"] = "BreakStatement";
        AST_NODE_TYPES["CallExpression"] = "CallExpression";
        AST_NODE_TYPES["CatchClause"] = "CatchClause";
        AST_NODE_TYPES["ClassBody"] = "ClassBody";
        AST_NODE_TYPES["ClassDeclaration"] = "ClassDeclaration";
        AST_NODE_TYPES["ClassExpression"] = "ClassExpression";
        AST_NODE_TYPES["ClassProperty"] = "ClassProperty";
        AST_NODE_TYPES["ConditionalExpression"] = "ConditionalExpression";
        AST_NODE_TYPES["ContinueStatement"] = "ContinueStatement";
        AST_NODE_TYPES["DebuggerStatement"] = "DebuggerStatement";
        AST_NODE_TYPES["Decorator"] = "Decorator";
        AST_NODE_TYPES["DoWhileStatement"] = "DoWhileStatement";
        AST_NODE_TYPES["EmptyStatement"] = "EmptyStatement";
        AST_NODE_TYPES["ExportAllDeclaration"] = "ExportAllDeclaration";
        AST_NODE_TYPES["ExportDefaultDeclaration"] = "ExportDefaultDeclaration";
        AST_NODE_TYPES["ExportNamedDeclaration"] = "ExportNamedDeclaration";
        AST_NODE_TYPES["ExportSpecifier"] = "ExportSpecifier";
        AST_NODE_TYPES["ExpressionStatement"] = "ExpressionStatement";
        AST_NODE_TYPES["ForInStatement"] = "ForInStatement";
        AST_NODE_TYPES["ForOfStatement"] = "ForOfStatement";
        AST_NODE_TYPES["ForStatement"] = "ForStatement";
        AST_NODE_TYPES["FunctionDeclaration"] = "FunctionDeclaration";
        AST_NODE_TYPES["FunctionExpression"] = "FunctionExpression";
        AST_NODE_TYPES["Identifier"] = "Identifier";
        AST_NODE_TYPES["IfStatement"] = "IfStatement";
        AST_NODE_TYPES["Import"] = "Import";
        AST_NODE_TYPES["ImportDeclaration"] = "ImportDeclaration";
        AST_NODE_TYPES["ImportDefaultSpecifier"] = "ImportDefaultSpecifier";
        AST_NODE_TYPES["ImportNamespaceSpecifier"] = "ImportNamespaceSpecifier";
        AST_NODE_TYPES["ImportSpecifier"] = "ImportSpecifier";
        AST_NODE_TYPES["JSXAttribute"] = "JSXAttribute";
        AST_NODE_TYPES["JSXClosingElement"] = "JSXClosingElement";
        AST_NODE_TYPES["JSXClosingFragment"] = "JSXClosingFragment";
        AST_NODE_TYPES["JSXElement"] = "JSXElement";
        AST_NODE_TYPES["JSXEmptyExpression"] = "JSXEmptyExpression";
        AST_NODE_TYPES["JSXExpressionContainer"] = "JSXExpressionContainer";
        AST_NODE_TYPES["JSXFragment"] = "JSXFragment";
        AST_NODE_TYPES["JSXIdentifier"] = "JSXIdentifier";
        AST_NODE_TYPES["JSXMemberExpression"] = "JSXMemberExpression";
        AST_NODE_TYPES["JSXOpeningElement"] = "JSXOpeningElement";
        AST_NODE_TYPES["JSXOpeningFragment"] = "JSXOpeningFragment";
        AST_NODE_TYPES["JSXSpreadAttribute"] = "JSXSpreadAttribute";
        AST_NODE_TYPES["JSXSpreadChild"] = "JSXSpreadChild";
        AST_NODE_TYPES["JSXText"] = "JSXText";
        AST_NODE_TYPES["LabeledStatement"] = "LabeledStatement";
        AST_NODE_TYPES["Literal"] = "Literal";
        AST_NODE_TYPES["LogicalExpression"] = "LogicalExpression";
        AST_NODE_TYPES["MemberExpression"] = "MemberExpression";
        AST_NODE_TYPES["MetaProperty"] = "MetaProperty";
        AST_NODE_TYPES["MethodDefinition"] = "MethodDefinition";
        AST_NODE_TYPES["NewExpression"] = "NewExpression";
        AST_NODE_TYPES["ObjectExpression"] = "ObjectExpression";
        AST_NODE_TYPES["ObjectPattern"] = "ObjectPattern";
        AST_NODE_TYPES["OptionalCallExpression"] = "OptionalCallExpression";
        AST_NODE_TYPES["OptionalMemberExpression"] = "OptionalMemberExpression";
        AST_NODE_TYPES["Program"] = "Program";
        AST_NODE_TYPES["Property"] = "Property";
        AST_NODE_TYPES["RestElement"] = "RestElement";
        AST_NODE_TYPES["ReturnStatement"] = "ReturnStatement";
        AST_NODE_TYPES["SequenceExpression"] = "SequenceExpression";
        AST_NODE_TYPES["SpreadElement"] = "SpreadElement";
        AST_NODE_TYPES["Super"] = "Super";
        AST_NODE_TYPES["SwitchCase"] = "SwitchCase";
        AST_NODE_TYPES["SwitchStatement"] = "SwitchStatement";
        AST_NODE_TYPES["TaggedTemplateExpression"] = "TaggedTemplateExpression";
        AST_NODE_TYPES["TemplateElement"] = "TemplateElement";
        AST_NODE_TYPES["TemplateLiteral"] = "TemplateLiteral";
        AST_NODE_TYPES["ThisExpression"] = "ThisExpression";
        AST_NODE_TYPES["ThrowStatement"] = "ThrowStatement";
        AST_NODE_TYPES["TryStatement"] = "TryStatement";
        AST_NODE_TYPES["UnaryExpression"] = "UnaryExpression";
        AST_NODE_TYPES["UpdateExpression"] = "UpdateExpression";
        AST_NODE_TYPES["VariableDeclaration"] = "VariableDeclaration";
        AST_NODE_TYPES["VariableDeclarator"] = "VariableDeclarator";
        AST_NODE_TYPES["WhileStatement"] = "WhileStatement";
        AST_NODE_TYPES["WithStatement"] = "WithStatement";
        AST_NODE_TYPES["YieldExpression"] = "YieldExpression";
        /**
         * TS-prefixed nodes
         */
        AST_NODE_TYPES["TSAbstractClassProperty"] = "TSAbstractClassProperty";
        AST_NODE_TYPES["TSAbstractKeyword"] = "TSAbstractKeyword";
        AST_NODE_TYPES["TSAbstractMethodDefinition"] = "TSAbstractMethodDefinition";
        AST_NODE_TYPES["TSAnyKeyword"] = "TSAnyKeyword";
        AST_NODE_TYPES["TSArrayType"] = "TSArrayType";
        AST_NODE_TYPES["TSAsExpression"] = "TSAsExpression";
        AST_NODE_TYPES["TSAsyncKeyword"] = "TSAsyncKeyword";
        AST_NODE_TYPES["TSBooleanKeyword"] = "TSBooleanKeyword";
        AST_NODE_TYPES["TSBigIntKeyword"] = "TSBigIntKeyword";
        AST_NODE_TYPES["TSConditionalType"] = "TSConditionalType";
        AST_NODE_TYPES["TSConstructorType"] = "TSConstructorType";
        AST_NODE_TYPES["TSCallSignatureDeclaration"] = "TSCallSignatureDeclaration";
        AST_NODE_TYPES["TSClassImplements"] = "TSClassImplements";
        AST_NODE_TYPES["TSConstructSignatureDeclaration"] = "TSConstructSignatureDeclaration";
        AST_NODE_TYPES["TSDeclareKeyword"] = "TSDeclareKeyword";
        AST_NODE_TYPES["TSDeclareFunction"] = "TSDeclareFunction";
        AST_NODE_TYPES["TSEmptyBodyFunctionExpression"] = "TSEmptyBodyFunctionExpression";
        AST_NODE_TYPES["TSEnumDeclaration"] = "TSEnumDeclaration";
        AST_NODE_TYPES["TSEnumMember"] = "TSEnumMember";
        AST_NODE_TYPES["TSExportAssignment"] = "TSExportAssignment";
        AST_NODE_TYPES["TSExportKeyword"] = "TSExportKeyword";
        AST_NODE_TYPES["TSExternalModuleReference"] = "TSExternalModuleReference";
        AST_NODE_TYPES["TSImportType"] = "TSImportType";
        AST_NODE_TYPES["TSInferType"] = "TSInferType";
        AST_NODE_TYPES["TSLiteralType"] = "TSLiteralType";
        AST_NODE_TYPES["TSIndexedAccessType"] = "TSIndexedAccessType";
        AST_NODE_TYPES["TSIndexSignature"] = "TSIndexSignature";
        AST_NODE_TYPES["TSInterfaceBody"] = "TSInterfaceBody";
        AST_NODE_TYPES["TSInterfaceDeclaration"] = "TSInterfaceDeclaration";
        AST_NODE_TYPES["TSInterfaceHeritage"] = "TSInterfaceHeritage";
        AST_NODE_TYPES["TSImportEqualsDeclaration"] = "TSImportEqualsDeclaration";
        AST_NODE_TYPES["TSFunctionType"] = "TSFunctionType";
        AST_NODE_TYPES["TSMethodSignature"] = "TSMethodSignature";
        AST_NODE_TYPES["TSModuleBlock"] = "TSModuleBlock";
        AST_NODE_TYPES["TSModuleDeclaration"] = "TSModuleDeclaration";
        AST_NODE_TYPES["TSNamespaceExportDeclaration"] = "TSNamespaceExportDeclaration";
        AST_NODE_TYPES["TSNonNullExpression"] = "TSNonNullExpression";
        AST_NODE_TYPES["TSNeverKeyword"] = "TSNeverKeyword";
        AST_NODE_TYPES["TSNullKeyword"] = "TSNullKeyword";
        AST_NODE_TYPES["TSNumberKeyword"] = "TSNumberKeyword";
        AST_NODE_TYPES["TSMappedType"] = "TSMappedType";
        AST_NODE_TYPES["TSObjectKeyword"] = "TSObjectKeyword";
        AST_NODE_TYPES["TSParameterProperty"] = "TSParameterProperty";
        AST_NODE_TYPES["TSPrivateKeyword"] = "TSPrivateKeyword";
        AST_NODE_TYPES["TSPropertySignature"] = "TSPropertySignature";
        AST_NODE_TYPES["TSProtectedKeyword"] = "TSProtectedKeyword";
        AST_NODE_TYPES["TSPublicKeyword"] = "TSPublicKeyword";
        AST_NODE_TYPES["TSQualifiedName"] = "TSQualifiedName";
        AST_NODE_TYPES["TSReadonlyKeyword"] = "TSReadonlyKeyword";
        AST_NODE_TYPES["TSRestType"] = "TSRestType";
        AST_NODE_TYPES["TSStaticKeyword"] = "TSStaticKeyword";
        AST_NODE_TYPES["TSStringKeyword"] = "TSStringKeyword";
        AST_NODE_TYPES["TSSymbolKeyword"] = "TSSymbolKeyword";
        AST_NODE_TYPES["TSThisType"] = "TSThisType";
        AST_NODE_TYPES["TSTypeAnnotation"] = "TSTypeAnnotation";
        AST_NODE_TYPES["TSTypeAliasDeclaration"] = "TSTypeAliasDeclaration";
        AST_NODE_TYPES["TSTypeAssertion"] = "TSTypeAssertion";
        AST_NODE_TYPES["TSTypeLiteral"] = "TSTypeLiteral";
        AST_NODE_TYPES["TSTypeOperator"] = "TSTypeOperator";
        AST_NODE_TYPES["TSTypeParameter"] = "TSTypeParameter";
        AST_NODE_TYPES["TSTypeParameterDeclaration"] = "TSTypeParameterDeclaration";
        AST_NODE_TYPES["TSTypeParameterInstantiation"] = "TSTypeParameterInstantiation";
        AST_NODE_TYPES["TSTypePredicate"] = "TSTypePredicate";
        AST_NODE_TYPES["TSTypeReference"] = "TSTypeReference";
        AST_NODE_TYPES["TSTypeQuery"] = "TSTypeQuery";
        AST_NODE_TYPES["TSIntersectionType"] = "TSIntersectionType";
        AST_NODE_TYPES["TSTupleType"] = "TSTupleType";
        AST_NODE_TYPES["TSOptionalType"] = "TSOptionalType";
        AST_NODE_TYPES["TSParenthesizedType"] = "TSParenthesizedType";
        AST_NODE_TYPES["TSUnionType"] = "TSUnionType";
        AST_NODE_TYPES["TSUndefinedKeyword"] = "TSUndefinedKeyword";
        AST_NODE_TYPES["TSUnknownKeyword"] = "TSUnknownKeyword";
        AST_NODE_TYPES["TSVoidKeyword"] = "TSVoidKeyword";
    })(AST_NODE_TYPES = exports.AST_NODE_TYPES || (exports.AST_NODE_TYPES = {}));
    var AST_TOKEN_TYPES;
    (function (AST_TOKEN_TYPES) {
        AST_TOKEN_TYPES["Boolean"] = "Boolean";
        AST_TOKEN_TYPES["Identifier"] = "Identifier";
        AST_TOKEN_TYPES["JSXIdentifier"] = "JSXIdentifier";
        AST_TOKEN_TYPES["JSXText"] = "JSXText";
        AST_TOKEN_TYPES["Keyword"] = "Keyword";
        AST_TOKEN_TYPES["Null"] = "Null";
        AST_TOKEN_TYPES["Numeric"] = "Numeric";
        AST_TOKEN_TYPES["Punctuator"] = "Punctuator";
        AST_TOKEN_TYPES["RegularExpression"] = "RegularExpression";
        AST_TOKEN_TYPES["String"] = "String";
        AST_TOKEN_TYPES["Template"] = "Template";
        // comment types
        AST_TOKEN_TYPES["Block"] = "Block";
        AST_TOKEN_TYPES["Line"] = "Line";
    })(AST_TOKEN_TYPES = exports.AST_TOKEN_TYPES || (exports.AST_TOKEN_TYPES = {}));
},
/* @typescript-eslint/eslint-plugin/dist/util/index.js */ function _(require, module, exports) {
    function __export(m) {
        for (var p in m)
            if (!exports.hasOwnProperty(p))
                exports[p] = m[p];
    }
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    __export(require(623) /* ./astUtils */);
    __export(require(624) /* ./createRule */);
    __export(require(626) /* ./isTypeReadonly */);
    __export(require(647) /* ./misc */);
    __export(require(648) /* ./nullThrows */);
    __export(require(649) /* ./types */);
    // this is done for convenience - saves migrating all of the old rules
    const { applyDefault, deepMerge, isObjectNotArray, getParserServices, } = experimental_utils_1.ESLintUtils;
    exports.applyDefault = applyDefault;
    exports.deepMerge = deepMerge;
    exports.isObjectNotArray = isObjectNotArray;
    exports.getParserServices = getParserServices;
},
/* @typescript-eslint/eslint-plugin/dist/util/astUtils.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const LINEBREAK_MATCHER = /\r\n|[\r\n\u2028\u2029]/;
    exports.LINEBREAK_MATCHER = LINEBREAK_MATCHER;
    function isOptionalChainPunctuator(token) {
        return token.type === experimental_utils_1.AST_TOKEN_TYPES.Punctuator && token.value === '?.';
    }
    exports.isOptionalChainPunctuator = isOptionalChainPunctuator;
    function isNotOptionalChainPunctuator(token) {
        return !isOptionalChainPunctuator(token);
    }
    exports.isNotOptionalChainPunctuator = isNotOptionalChainPunctuator;
    function isNonNullAssertionPunctuator(token) {
        return token.type === experimental_utils_1.AST_TOKEN_TYPES.Punctuator && token.value === '!';
    }
    exports.isNonNullAssertionPunctuator = isNonNullAssertionPunctuator;
    function isNotNonNullAssertionPunctuator(token) {
        return !isNonNullAssertionPunctuator(token);
    }
    exports.isNotNonNullAssertionPunctuator = isNotNonNullAssertionPunctuator;
    /**
     * Returns true if and only if the node represents: foo?.() or foo.bar?.()
     */
    function isOptionalOptionalChain(node) {
        return (node.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression &&
            // this flag means the call expression itself is option
            // i.e. it is foo.bar?.() and not foo?.bar()
            node.optional);
    }
    exports.isOptionalOptionalChain = isOptionalOptionalChain;
    /**
     * Returns true if and only if the node represents logical OR
     */
    function isLogicalOrOperator(node) {
        return (node.type === experimental_utils_1.AST_NODE_TYPES.LogicalExpression && node.operator === '||');
    }
    exports.isLogicalOrOperator = isLogicalOrOperator;
    /**
     * Determines whether two adjacent tokens are on the same line
     */
    function isTokenOnSameLine(left, right) {
        return left.loc.end.line === right.loc.start.line;
    }
    exports.isTokenOnSameLine = isTokenOnSameLine;
    /**
     * Checks if a node is a type assertion:
     * ```
     * x as foo
     * <foo>x
     * ```
     */
    function isTypeAssertion(node) {
        if (!node) {
            return false;
        }
        return (node.type === experimental_utils_1.AST_NODE_TYPES.TSAsExpression ||
            node.type === experimental_utils_1.AST_NODE_TYPES.TSTypeAssertion);
    }
    exports.isTypeAssertion = isTypeAssertion;
    function isVariableDeclarator(node) {
        return (node === null || node === void 0 ? void 0 : node.type) === experimental_utils_1.AST_NODE_TYPES.VariableDeclarator;
    }
    exports.isVariableDeclarator = isVariableDeclarator;
    function isFunction(node) {
        if (!node) {
            return false;
        }
        return [
            experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression,
            experimental_utils_1.AST_NODE_TYPES.FunctionDeclaration,
            experimental_utils_1.AST_NODE_TYPES.FunctionExpression,
        ].includes(node.type);
    }
    exports.isFunction = isFunction;
    function isFunctionType(node) {
        if (!node) {
            return false;
        }
        return [
            experimental_utils_1.AST_NODE_TYPES.TSCallSignatureDeclaration,
            experimental_utils_1.AST_NODE_TYPES.TSConstructorType,
            experimental_utils_1.AST_NODE_TYPES.TSConstructSignatureDeclaration,
            experimental_utils_1.AST_NODE_TYPES.TSEmptyBodyFunctionExpression,
            experimental_utils_1.AST_NODE_TYPES.TSFunctionType,
            experimental_utils_1.AST_NODE_TYPES.TSMethodSignature,
        ].includes(node.type);
    }
    exports.isFunctionType = isFunctionType;
    function isFunctionOrFunctionType(node) {
        return isFunction(node) || isFunctionType(node);
    }
    exports.isFunctionOrFunctionType = isFunctionOrFunctionType;
    function isTSFunctionType(node) {
        return (node === null || node === void 0 ? void 0 : node.type) === experimental_utils_1.AST_NODE_TYPES.TSFunctionType;
    }
    exports.isTSFunctionType = isTSFunctionType;
    function isTSConstructorType(node) {
        return (node === null || node === void 0 ? void 0 : node.type) === experimental_utils_1.AST_NODE_TYPES.TSConstructorType;
    }
    exports.isTSConstructorType = isTSConstructorType;
    function isClassOrTypeElement(node) {
        if (!node) {
            return false;
        }
        return [
            // ClassElement
            experimental_utils_1.AST_NODE_TYPES.ClassProperty,
            experimental_utils_1.AST_NODE_TYPES.FunctionExpression,
            experimental_utils_1.AST_NODE_TYPES.MethodDefinition,
            experimental_utils_1.AST_NODE_TYPES.TSAbstractClassProperty,
            experimental_utils_1.AST_NODE_TYPES.TSAbstractMethodDefinition,
            experimental_utils_1.AST_NODE_TYPES.TSEmptyBodyFunctionExpression,
            experimental_utils_1.AST_NODE_TYPES.TSIndexSignature,
            // TypeElement
            experimental_utils_1.AST_NODE_TYPES.TSCallSignatureDeclaration,
            experimental_utils_1.AST_NODE_TYPES.TSConstructSignatureDeclaration,
            // AST_NODE_TYPES.TSIndexSignature,
            experimental_utils_1.AST_NODE_TYPES.TSMethodSignature,
            experimental_utils_1.AST_NODE_TYPES.TSPropertySignature,
        ].includes(node.type);
    }
    exports.isClassOrTypeElement = isClassOrTypeElement;
    /**
     * Checks if a node is a constructor method.
     */
    function isConstructor(node) {
        return ((node === null || node === void 0 ? void 0 : node.type) === experimental_utils_1.AST_NODE_TYPES.MethodDefinition &&
            node.kind === 'constructor');
    }
    exports.isConstructor = isConstructor;
    /**
     * Checks if a node is a setter method.
     */
    function isSetter(node) {
        return (!!node &&
            (node.type === experimental_utils_1.AST_NODE_TYPES.MethodDefinition ||
                node.type === experimental_utils_1.AST_NODE_TYPES.Property) &&
            node.kind === 'set');
    }
    exports.isSetter = isSetter;
    function isIdentifier(node) {
        return (node === null || node === void 0 ? void 0 : node.type) === experimental_utils_1.AST_NODE_TYPES.Identifier;
    }
    exports.isIdentifier = isIdentifier;
    /**
     * Checks if a node represents an `await …` expression.
     */
    function isAwaitExpression(node) {
        return (node === null || node === void 0 ? void 0 : node.type) === experimental_utils_1.AST_NODE_TYPES.AwaitExpression;
    }
    exports.isAwaitExpression = isAwaitExpression;
    /**
     * Checks if a possible token is the `await` keyword.
     */
    function isAwaitKeyword(node) {
        return (node === null || node === void 0 ? void 0 : node.type) === experimental_utils_1.AST_TOKEN_TYPES.Identifier && node.value === 'await';
    }
    exports.isAwaitKeyword = isAwaitKeyword;
    function isMemberOrOptionalMemberExpression(node) {
        return (node.type === experimental_utils_1.AST_NODE_TYPES.MemberExpression ||
            node.type === experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression);
    }
    exports.isMemberOrOptionalMemberExpression = isMemberOrOptionalMemberExpression;
},
/* @typescript-eslint/eslint-plugin/dist/util/createRule.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    // note - cannot migrate this to an import statement because it will make TSC copy the package.json to the dist folder
    const version = require(625) /* ../../package.json */.version;
    exports.createRule = experimental_utils_1.ESLintUtils.RuleCreator(name => `https://github.com/typescript-eslint/typescript-eslint/blob/v${version}/packages/eslint-plugin/docs/rules/${name}.md`);
},
/* @typescript-eslint/eslint-plugin/package.json */ function _(require, module, exports) {
    module.exports = {
        "_args": [
            [
                "@typescript-eslint/eslint-plugin@2.24.0",
                "/Users/bryan/work/bokeh/bokehjs"
            ]
        ],
        "_development": true,
        "_from": "@typescript-eslint/eslint-plugin@2.24.0",
        "_id": "@typescript-eslint/eslint-plugin@2.24.0",
        "_inBundle": false,
        "_integrity": "sha512-wJRBeaMeT7RLQ27UQkDFOu25MqFOBus8PtOa9KaT5ZuxC1kAsd7JEHqWt4YXuY9eancX0GK9C68i5OROnlIzBA==",
        "_location": "/@typescript-eslint/eslint-plugin",
        "_phantomChildren": {},
        "_requested": {
            "type": "version",
            "registry": true,
            "raw": "@typescript-eslint/eslint-plugin@2.24.0",
            "name": "@typescript-eslint/eslint-plugin",
            "escapedName": "@typescript-eslint%2feslint-plugin",
            "scope": "@typescript-eslint",
            "rawSpec": "2.24.0",
            "saveSpec": null,
            "fetchSpec": "2.24.0"
        },
        "_requiredBy": [
            "#DEV:/"
        ],
        "_resolved": "https://registry.npmjs.org/@typescript-eslint/eslint-plugin/-/eslint-plugin-2.24.0.tgz",
        "_spec": "2.24.0",
        "_where": "/Users/bryan/work/bokeh/bokehjs",
        "bugs": {
            "url": "https://github.com/typescript-eslint/typescript-eslint/issues"
        },
        "dependencies": {
            "@typescript-eslint/experimental-utils": "2.24.0",
            "eslint-utils": "^1.4.3",
            "functional-red-black-tree": "^1.0.1",
            "regexpp": "^3.0.0",
            "tsutils": "^3.17.1"
        },
        "description": "TypeScript plugin for ESLint",
        "devDependencies": {
            "@types/json-schema": "^7.0.3",
            "@types/marked": "^0.7.1",
            "@types/prettier": "^1.18.2",
            "chalk": "^3.0.0",
            "marked": "^0.7.0",
            "prettier": "*",
            "typescript": "*"
        },
        "engines": {
            "node": "^8.10.0 || ^10.13.0 || >=11.10.1"
        },
        "files": [
            "dist",
            "docs",
            "package.json",
            "README.md",
            "LICENSE"
        ],
        "funding": {
            "type": "opencollective",
            "url": "https://opencollective.com/typescript-eslint"
        },
        "gitHead": "56e1e160a36cecc298dd1187ea035e1482489dd0",
        "homepage": "https://github.com/typescript-eslint/typescript-eslint#readme",
        "keywords": [
            "eslint",
            "eslintplugin",
            "eslint-plugin",
            "typescript"
        ],
        "license": "MIT",
        "main": "dist/index.js",
        "name": "@typescript-eslint/eslint-plugin",
        "peerDependencies": {
            "@typescript-eslint/parser": "^2.0.0",
            "eslint": "^5.0.0 || ^6.0.0"
        },
        "peerDependenciesMeta": {
            "typescript": {
                "optional": true
            }
        },
        "repository": {
            "type": "git",
            "url": "git+https://github.com/typescript-eslint/typescript-eslint.git",
            "directory": "packages/eslint-plugin"
        },
        "scripts": {
            "build": "tsc -b tsconfig.build.json",
            "check:configs": "jest tests/configs.test.ts --runTestsByPath --silent --runInBand",
            "check:docs": "jest tests/docs.test.ts --runTestsByPath --silent --runInBand",
            "clean": "tsc -b tsconfig.build.json --clean",
            "format": "prettier --write \"./**/*.{ts,js,json,md}\" --ignore-path ../../.prettierignore",
            "generate:configs": "../../node_modules/.bin/ts-node --files --transpile-only tools/generate-configs.ts",
            "generate:rules-lists": "../../node_modules/.bin/ts-node --files --transpile-only tools/generate-rules-lists.ts",
            "lint": "eslint . --ext .js,.ts --ignore-path ../../.eslintignore",
            "test": "jest --coverage",
            "typecheck": "tsc -p tsconfig.json --noEmit"
        },
        "version": "2.24.0"
    };
},
/* @typescript-eslint/eslint-plugin/dist/util/isTypeReadonly.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const tsutils_1 = require(627) /* tsutils */;
    const ts = __importStar(require(44) /* typescript */);
    const _1 = require(622) /* . */;
    function isTypeReadonlyArrayOrTuple(checker, type, seenTypes) {
        function checkTypeArguments(arrayType) {
            const typeArguments = checker.getTypeArguments(arrayType);
            // this shouldn't happen in reality as:
            // - tuples require at least 1 type argument
            // - ReadonlyArray requires at least 1 type argument
            /* istanbul ignore if */ if (typeArguments.length === 0) {
                return 3 /* Readonly */;
            }
            // validate the element types are also readonly
            if (typeArguments.some(typeArg => isTypeReadonlyRecurser(checker, typeArg, seenTypes) ===
                2 /* Mutable */)) {
                return 2 /* Mutable */;
            }
            return 3 /* Readonly */;
        }
        if (checker.isArrayType(type)) {
            const symbol = _1.nullThrows(type.getSymbol(), _1.NullThrowsReasons.MissingToken('symbol', 'array type'));
            const escapedName = symbol.getEscapedName();
            if (escapedName === 'Array') {
                return 2 /* Mutable */;
            }
            return checkTypeArguments(type);
        }
        if (checker.isTupleType(type)) {
            if (!type.target.readonly) {
                return 2 /* Mutable */;
            }
            return checkTypeArguments(type);
        }
        return 1 /* UnknownType */;
    }
    function isTypeReadonlyObject(checker, type, seenTypes) {
        function checkIndexSignature(kind) {
            const indexInfo = checker.getIndexInfoOfType(type, kind);
            if (indexInfo) {
                return indexInfo.isReadonly
                    ? 3 /* Readonly */
                    : 2 /* Mutable */;
            }
            return 1 /* UnknownType */;
        }
        const properties = type.getProperties();
        if (properties.length) {
            // ensure the properties are marked as readonly
            for (const property of properties) {
                if (!tsutils_1.isPropertyReadonlyInType(type, property.getEscapedName(), checker)) {
                    return 2 /* Mutable */;
                }
            }
            // all properties were readonly
            // now ensure that all of the values are readonly also.
            // do this after checking property readonly-ness as a perf optimization,
            // as we might be able to bail out early due to a mutable property before
            // doing this deep, potentially expensive check.
            for (const property of properties) {
                const propertyType = _1.nullThrows(checker.getTypeOfPropertyOfType(type, property.getName()), _1.NullThrowsReasons.MissingToken(`property "${property.name}"`, 'type'));
                // handle recursive types.
                // we only need this simple check, because a mutable recursive type will break via the above prop readonly check
                if (seenTypes.has(propertyType)) {
                    continue;
                }
                if (isTypeReadonlyRecurser(checker, propertyType, seenTypes) ===
                    2 /* Mutable */) {
                    return 2 /* Mutable */;
                }
            }
        }
        const isStringIndexSigReadonly = checkIndexSignature(ts.IndexKind.String);
        if (isStringIndexSigReadonly === 2 /* Mutable */) {
            return isStringIndexSigReadonly;
        }
        const isNumberIndexSigReadonly = checkIndexSignature(ts.IndexKind.Number);
        if (isNumberIndexSigReadonly === 2 /* Mutable */) {
            return isNumberIndexSigReadonly;
        }
        return 3 /* Readonly */;
    }
    // a helper function to ensure the seenTypes map is always passed down, except by the external caller
    function isTypeReadonlyRecurser(checker, type, seenTypes) {
        seenTypes.add(type);
        if (tsutils_1.isUnionType(type)) {
            // all types in the union must be readonly
            const result = tsutils_1.unionTypeParts(type).every(t => isTypeReadonlyRecurser(checker, t, seenTypes));
            const readonlyness = result ? 3 /* Readonly */ : 2 /* Mutable */;
            return readonlyness;
        }
        // all non-object, non-intersection types are readonly.
        // this should only be primitive types
        if (!tsutils_1.isObjectType(type) && !tsutils_1.isUnionOrIntersectionType(type)) {
            return 3 /* Readonly */;
        }
        // pure function types are readonly
        if (type.getCallSignatures().length > 0 &&
            type.getProperties().length === 0) {
            return 3 /* Readonly */;
        }
        const isReadonlyArray = isTypeReadonlyArrayOrTuple(checker, type, seenTypes);
        if (isReadonlyArray !== 1 /* UnknownType */) {
            return isReadonlyArray;
        }
        const isReadonlyObject = isTypeReadonlyObject(checker, type, seenTypes);
        /* istanbul ignore else */ if (isReadonlyObject !== 1 /* UnknownType */) {
            return isReadonlyObject;
        }
        throw new Error('Unhandled type');
    }
    /**
     * Checks if the given type is readonly
     */
    function isTypeReadonly(checker, type) {
        return (isTypeReadonlyRecurser(checker, type, new Set()) === 3 /* Readonly */);
    }
    exports.isTypeReadonly = isTypeReadonly;
},
/* tsutils/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(629) /* ./typeguard */, exports);
    tslib_1.__exportStar(require(640) /* ./util */, exports);
},
/* tslib/tslib.js */ function _(require, module, exports) {
    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0
    
    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.
    
    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */
    /* global global, define, System, Reflect, Promise */
    var __extends;
    var __assign;
    var __rest;
    var __decorate;
    var __param;
    var __metadata;
    var __awaiter;
    var __generator;
    var __exportStar;
    var __values;
    var __read;
    var __spread;
    var __spreadArrays;
    var __await;
    var __asyncGenerator;
    var __asyncDelegator;
    var __asyncValues;
    var __makeTemplateObject;
    var __importStar;
    var __importDefault;
    var __classPrivateFieldGet;
    var __classPrivateFieldSet;
    (function (factory) {
        var root = typeof global === "object" ? global : typeof self === "object" ? self : typeof this === "object" ? this : {};
        if (typeof define === "function" && define.amd) {
            define("tslib", ["exports"], function (exports) { factory(createExporter(root, createExporter(exports))); });
        }
        else if (typeof module === "object" && typeof module.exports === "object") {
            factory(createExporter(root, createExporter(module.exports)));
        }
        else {
            factory(createExporter(root));
        }
        function createExporter(exports, previous) {
            if (exports !== root) {
                if (typeof Object.create === "function") {
                    Object.defineProperty(exports, "__esModule", { value: true });
                }
                else {
                    exports.__esModule = true;
                }
            }
            return function (id, v) { return exports[id] = previous ? previous(id, v) : v; };
        }
    })(function (exporter) {
        var extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (b.hasOwnProperty(p))
                    d[p] = b[p]; };
        __extends = function (d, b) {
            extendStatics(d, b);
            function __() { this.constructor = d; }
            d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
        };
        __assign = Object.assign || function (t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        __rest = function (s, e) {
            var t = {};
            for (var p in s)
                if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                    t[p] = s[p];
            if (s != null && typeof Object.getOwnPropertySymbols === "function")
                for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                        t[p[i]] = s[p[i]];
                }
            return t;
        };
        __decorate = function (decorators, target, key, desc) {
            var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
            if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
                r = Reflect.decorate(decorators, target, key, desc);
            else
                for (var i = decorators.length - 1; i >= 0; i--)
                    if (d = decorators[i])
                        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
            return c > 3 && r && Object.defineProperty(target, key, r), r;
        };
        __param = function (paramIndex, decorator) {
            return function (target, key) { decorator(target, key, paramIndex); };
        };
        __metadata = function (metadataKey, metadataValue) {
            if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
                return Reflect.metadata(metadataKey, metadataValue);
        };
        __awaiter = function (thisArg, _arguments, P, generator) {
            function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
            return new (P || (P = Promise))(function (resolve, reject) {
                function fulfilled(value) { try {
                    step(generator.next(value));
                }
                catch (e) {
                    reject(e);
                } }
                function rejected(value) { try {
                    step(generator["throw"](value));
                }
                catch (e) {
                    reject(e);
                } }
                function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
                step((generator = generator.apply(thisArg, _arguments || [])).next());
            });
        };
        __generator = function (thisArg, body) {
            var _ = { label: 0, sent: function () { if (t[0] & 1)
                    throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
            return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
            function verb(n) { return function (v) { return step([n, v]); }; }
            function step(op) {
                if (f)
                    throw new TypeError("Generator is already executing.");
                while (_)
                    try {
                        if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                            return t;
                        if (y = 0, t)
                            op = [op[0] & 2, t.value];
                        switch (op[0]) {
                            case 0:
                            case 1:
                                t = op;
                                break;
                            case 4:
                                _.label++;
                                return { value: op[1], done: false };
                            case 5:
                                _.label++;
                                y = op[1];
                                op = [0];
                                continue;
                            case 7:
                                op = _.ops.pop();
                                _.trys.pop();
                                continue;
                            default:
                                if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                    _ = 0;
                                    continue;
                                }
                                if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                    _.label = op[1];
                                    break;
                                }
                                if (op[0] === 6 && _.label < t[1]) {
                                    _.label = t[1];
                                    t = op;
                                    break;
                                }
                                if (t && _.label < t[2]) {
                                    _.label = t[2];
                                    _.ops.push(op);
                                    break;
                                }
                                if (t[2])
                                    _.ops.pop();
                                _.trys.pop();
                                continue;
                        }
                        op = body.call(thisArg, _);
                    }
                    catch (e) {
                        op = [6, e];
                        y = 0;
                    }
                    finally {
                        f = t = 0;
                    }
                if (op[0] & 5)
                    throw op[1];
                return { value: op[0] ? op[1] : void 0, done: true };
            }
        };
        __exportStar = function (m, exports) {
            for (var p in m)
                if (!exports.hasOwnProperty(p))
                    exports[p] = m[p];
        };
        __values = function (o) {
            var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
            if (m)
                return m.call(o);
            if (o && typeof o.length === "number")
                return {
                    next: function () {
                        if (o && i >= o.length)
                            o = void 0;
                        return { value: o && o[i++], done: !o };
                    }
                };
            throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
        };
        __read = function (o, n) {
            var m = typeof Symbol === "function" && o[Symbol.iterator];
            if (!m)
                return o;
            var i = m.call(o), r, ar = [], e;
            try {
                while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                    ar.push(r.value);
            }
            catch (error) {
                e = { error: error };
            }
            finally {
                try {
                    if (r && !r.done && (m = i["return"]))
                        m.call(i);
                }
                finally {
                    if (e)
                        throw e.error;
                }
            }
            return ar;
        };
        __spread = function () {
            for (var ar = [], i = 0; i < arguments.length; i++)
                ar = ar.concat(__read(arguments[i]));
            return ar;
        };
        __spreadArrays = function () {
            for (var s = 0, i = 0, il = arguments.length; i < il; i++)
                s += arguments[i].length;
            for (var r = Array(s), k = 0, i = 0; i < il; i++)
                for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                    r[k] = a[j];
            return r;
        };
        __await = function (v) {
            return this instanceof __await ? (this.v = v, this) : new __await(v);
        };
        __asyncGenerator = function (thisArg, _arguments, generator) {
            if (!Symbol.asyncIterator)
                throw new TypeError("Symbol.asyncIterator is not defined.");
            var g = generator.apply(thisArg, _arguments || []), i, q = [];
            return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
            function verb(n) { if (g[n])
                i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
            function resume(n, v) { try {
                step(g[n](v));
            }
            catch (e) {
                settle(q[0][3], e);
            } }
            function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
            function fulfill(value) { resume("next", value); }
            function reject(value) { resume("throw", value); }
            function settle(f, v) { if (f(v), q.shift(), q.length)
                resume(q[0][0], q[0][1]); }
        };
        __asyncDelegator = function (o) {
            var i, p;
            return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
            function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
        };
        __asyncValues = function (o) {
            if (!Symbol.asyncIterator)
                throw new TypeError("Symbol.asyncIterator is not defined.");
            var m = o[Symbol.asyncIterator], i;
            return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
            function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
            function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
        };
        __makeTemplateObject = function (cooked, raw) {
            if (Object.defineProperty) {
                Object.defineProperty(cooked, "raw", { value: raw });
            }
            else {
                cooked.raw = raw;
            }
            return cooked;
        };
        __importStar = function (mod) {
            if (mod && mod.__esModule)
                return mod;
            var result = {};
            if (mod != null)
                for (var k in mod)
                    if (Object.hasOwnProperty.call(mod, k))
                        result[k] = mod[k];
            result["default"] = mod;
            return result;
        };
        __importDefault = function (mod) {
            return (mod && mod.__esModule) ? mod : { "default": mod };
        };
        __classPrivateFieldGet = function (receiver, privateMap) {
            if (!privateMap.has(receiver)) {
                throw new TypeError("attempted to get private field on non-instance");
            }
            return privateMap.get(receiver);
        };
        __classPrivateFieldSet = function (receiver, privateMap, value) {
            if (!privateMap.has(receiver)) {
                throw new TypeError("attempted to set private field on non-instance");
            }
            privateMap.set(receiver, value);
            return value;
        };
        exporter("__extends", __extends);
        exporter("__assign", __assign);
        exporter("__rest", __rest);
        exporter("__decorate", __decorate);
        exporter("__param", __param);
        exporter("__metadata", __metadata);
        exporter("__awaiter", __awaiter);
        exporter("__generator", __generator);
        exporter("__exportStar", __exportStar);
        exporter("__values", __values);
        exporter("__read", __read);
        exporter("__spread", __spread);
        exporter("__spreadArrays", __spreadArrays);
        exporter("__await", __await);
        exporter("__asyncGenerator", __asyncGenerator);
        exporter("__asyncDelegator", __asyncDelegator);
        exporter("__asyncValues", __asyncValues);
        exporter("__makeTemplateObject", __makeTemplateObject);
        exporter("__importStar", __importStar);
        exporter("__importDefault", __importDefault);
        exporter("__classPrivateFieldGet", __classPrivateFieldGet);
        exporter("__classPrivateFieldSet", __classPrivateFieldSet);
    });
},
/* tsutils/typeguard/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(630) /* ./node */, exports);
    tslib_1.__exportStar(require(635) /* ./type */, exports);
},
/* tsutils/typeguard/node.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(631) /* ./3.2/node */, exports);
},
/* tsutils/typeguard/3.2/node.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(632) /* ../3.0/node */, exports);
    const ts = require(44) /* typescript */;
    function isBigIntLiteral(node) {
        return node.kind === ts.SyntaxKind.BigIntLiteral;
    }
    exports.isBigIntLiteral = isBigIntLiteral;
},
/* tsutils/typeguard/3.0/node.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(633) /* ../2.9/node */, exports);
    const ts = require(44) /* typescript */;
    function isOptionalTypeNode(node) {
        return node.kind === ts.SyntaxKind.OptionalType;
    }
    exports.isOptionalTypeNode = isOptionalTypeNode;
    function isRestTypeNode(node) {
        return node.kind === ts.SyntaxKind.RestType;
    }
    exports.isRestTypeNode = isRestTypeNode;
    function isSyntheticExpression(node) {
        return node.kind === ts.SyntaxKind.SyntheticExpression;
    }
    exports.isSyntheticExpression = isSyntheticExpression;
},
/* tsutils/typeguard/2.9/node.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(634) /* ../2.8/node */, exports);
    const ts = require(44) /* typescript */;
    function isImportTypeNode(node) {
        return node.kind === ts.SyntaxKind.ImportType;
    }
    exports.isImportTypeNode = isImportTypeNode;
},
/* tsutils/typeguard/2.8/node.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = require(44) /* typescript */;
    function isAccessorDeclaration(node) {
        return node.kind === ts.SyntaxKind.GetAccessor ||
            node.kind === ts.SyntaxKind.SetAccessor;
    }
    exports.isAccessorDeclaration = isAccessorDeclaration;
    function isArrayBindingPattern(node) {
        return node.kind === ts.SyntaxKind.ArrayBindingPattern;
    }
    exports.isArrayBindingPattern = isArrayBindingPattern;
    function isArrayLiteralExpression(node) {
        return node.kind === ts.SyntaxKind.ArrayLiteralExpression;
    }
    exports.isArrayLiteralExpression = isArrayLiteralExpression;
    function isArrayTypeNode(node) {
        return node.kind === ts.SyntaxKind.ArrayType;
    }
    exports.isArrayTypeNode = isArrayTypeNode;
    function isArrowFunction(node) {
        return node.kind === ts.SyntaxKind.ArrowFunction;
    }
    exports.isArrowFunction = isArrowFunction;
    function isAsExpression(node) {
        return node.kind === ts.SyntaxKind.AsExpression;
    }
    exports.isAsExpression = isAsExpression;
    function isAssertionExpression(node) {
        return node.kind === ts.SyntaxKind.AsExpression ||
            node.kind === ts.SyntaxKind.TypeAssertionExpression;
    }
    exports.isAssertionExpression = isAssertionExpression;
    function isAwaitExpression(node) {
        return node.kind === ts.SyntaxKind.AwaitExpression;
    }
    exports.isAwaitExpression = isAwaitExpression;
    function isBinaryExpression(node) {
        return node.kind === ts.SyntaxKind.BinaryExpression;
    }
    exports.isBinaryExpression = isBinaryExpression;
    function isBindingElement(node) {
        return node.kind === ts.SyntaxKind.BindingElement;
    }
    exports.isBindingElement = isBindingElement;
    function isBindingPattern(node) {
        return node.kind === ts.SyntaxKind.ArrayBindingPattern ||
            node.kind === ts.SyntaxKind.ObjectBindingPattern;
    }
    exports.isBindingPattern = isBindingPattern;
    function isBlock(node) {
        return node.kind === ts.SyntaxKind.Block;
    }
    exports.isBlock = isBlock;
    function isBlockLike(node) {
        return node.statements !== undefined;
    }
    exports.isBlockLike = isBlockLike;
    function isBooleanLiteral(node) {
        return node.kind === ts.SyntaxKind.TrueKeyword || node.kind === ts.SyntaxKind.FalseKeyword;
    }
    exports.isBooleanLiteral = isBooleanLiteral;
    function isBreakOrContinueStatement(node) {
        return node.kind === ts.SyntaxKind.BreakStatement ||
            node.kind === ts.SyntaxKind.ContinueStatement;
    }
    exports.isBreakOrContinueStatement = isBreakOrContinueStatement;
    function isBreakStatement(node) {
        return node.kind === ts.SyntaxKind.BreakStatement;
    }
    exports.isBreakStatement = isBreakStatement;
    function isCallExpression(node) {
        return node.kind === ts.SyntaxKind.CallExpression;
    }
    exports.isCallExpression = isCallExpression;
    function isCallLikeExpression(node) {
        switch (node.kind) {
            case ts.SyntaxKind.CallExpression:
            case ts.SyntaxKind.Decorator:
            case ts.SyntaxKind.JsxOpeningElement:
            case ts.SyntaxKind.JsxSelfClosingElement:
            case ts.SyntaxKind.NewExpression:
            case ts.SyntaxKind.TaggedTemplateExpression:
                return true;
            default:
                return false;
        }
    }
    exports.isCallLikeExpression = isCallLikeExpression;
    function isCallSignatureDeclaration(node) {
        return node.kind === ts.SyntaxKind.CallSignature;
    }
    exports.isCallSignatureDeclaration = isCallSignatureDeclaration;
    function isCaseBlock(node) {
        return node.kind === ts.SyntaxKind.CaseBlock;
    }
    exports.isCaseBlock = isCaseBlock;
    function isCaseClause(node) {
        return node.kind === ts.SyntaxKind.CaseClause;
    }
    exports.isCaseClause = isCaseClause;
    function isCaseOrDefaultClause(node) {
        return node.kind === ts.SyntaxKind.CaseClause ||
            node.kind === ts.SyntaxKind.DefaultClause;
    }
    exports.isCaseOrDefaultClause = isCaseOrDefaultClause;
    function isCatchClause(node) {
        return node.kind === ts.SyntaxKind.CatchClause;
    }
    exports.isCatchClause = isCatchClause;
    function isClassDeclaration(node) {
        return node.kind === ts.SyntaxKind.ClassDeclaration;
    }
    exports.isClassDeclaration = isClassDeclaration;
    function isClassExpression(node) {
        return node.kind === ts.SyntaxKind.ClassExpression;
    }
    exports.isClassExpression = isClassExpression;
    function isClassLikeDeclaration(node) {
        return node.kind === ts.SyntaxKind.ClassDeclaration ||
            node.kind === ts.SyntaxKind.ClassExpression;
    }
    exports.isClassLikeDeclaration = isClassLikeDeclaration;
    function isCommaListExpression(node) {
        return node.kind === ts.SyntaxKind.CommaListExpression;
    }
    exports.isCommaListExpression = isCommaListExpression;
    function isConditionalExpression(node) {
        return node.kind === ts.SyntaxKind.ConditionalExpression;
    }
    exports.isConditionalExpression = isConditionalExpression;
    function isConditionalTypeNode(node) {
        return node.kind === ts.SyntaxKind.ConditionalType;
    }
    exports.isConditionalTypeNode = isConditionalTypeNode;
    function isConstructorDeclaration(node) {
        return node.kind === ts.SyntaxKind.Constructor;
    }
    exports.isConstructorDeclaration = isConstructorDeclaration;
    function isConstructorTypeNode(node) {
        return node.kind === ts.SyntaxKind.ConstructorType;
    }
    exports.isConstructorTypeNode = isConstructorTypeNode;
    function isConstructSignatureDeclaration(node) {
        return node.kind === ts.SyntaxKind.ConstructSignature;
    }
    exports.isConstructSignatureDeclaration = isConstructSignatureDeclaration;
    function isContinueStatement(node) {
        return node.kind === ts.SyntaxKind.ContinueStatement;
    }
    exports.isContinueStatement = isContinueStatement;
    function isComputedPropertyName(node) {
        return node.kind === ts.SyntaxKind.ComputedPropertyName;
    }
    exports.isComputedPropertyName = isComputedPropertyName;
    function isDebuggerStatement(node) {
        return node.kind === ts.SyntaxKind.DebuggerStatement;
    }
    exports.isDebuggerStatement = isDebuggerStatement;
    function isDecorator(node) {
        return node.kind === ts.SyntaxKind.Decorator;
    }
    exports.isDecorator = isDecorator;
    function isDefaultClause(node) {
        return node.kind === ts.SyntaxKind.DefaultClause;
    }
    exports.isDefaultClause = isDefaultClause;
    function isDeleteExpression(node) {
        return node.kind === ts.SyntaxKind.DeleteExpression;
    }
    exports.isDeleteExpression = isDeleteExpression;
    function isDoStatement(node) {
        return node.kind === ts.SyntaxKind.DoStatement;
    }
    exports.isDoStatement = isDoStatement;
    function isElementAccessExpression(node) {
        return node.kind === ts.SyntaxKind.ElementAccessExpression;
    }
    exports.isElementAccessExpression = isElementAccessExpression;
    function isEmptyStatement(node) {
        return node.kind === ts.SyntaxKind.EmptyStatement;
    }
    exports.isEmptyStatement = isEmptyStatement;
    function isEntityName(node) {
        return node.kind === ts.SyntaxKind.Identifier || isQualifiedName(node);
    }
    exports.isEntityName = isEntityName;
    function isEntityNameExpression(node) {
        return node.kind === ts.SyntaxKind.Identifier ||
            isPropertyAccessExpression(node) && isEntityNameExpression(node.expression);
    }
    exports.isEntityNameExpression = isEntityNameExpression;
    function isEnumDeclaration(node) {
        return node.kind === ts.SyntaxKind.EnumDeclaration;
    }
    exports.isEnumDeclaration = isEnumDeclaration;
    function isEnumMember(node) {
        return node.kind === ts.SyntaxKind.EnumMember;
    }
    exports.isEnumMember = isEnumMember;
    function isExportAssignment(node) {
        return node.kind === ts.SyntaxKind.ExportAssignment;
    }
    exports.isExportAssignment = isExportAssignment;
    function isExportDeclaration(node) {
        return node.kind === ts.SyntaxKind.ExportDeclaration;
    }
    exports.isExportDeclaration = isExportDeclaration;
    function isExportSpecifier(node) {
        return node.kind === ts.SyntaxKind.ExportSpecifier;
    }
    exports.isExportSpecifier = isExportSpecifier;
    function isExpression(node) {
        switch (node.kind) {
            case ts.SyntaxKind.ArrayLiteralExpression:
            case ts.SyntaxKind.ArrowFunction:
            case ts.SyntaxKind.AsExpression:
            case ts.SyntaxKind.AwaitExpression:
            case ts.SyntaxKind.BinaryExpression:
            case ts.SyntaxKind.CallExpression:
            case ts.SyntaxKind.ClassExpression:
            case ts.SyntaxKind.CommaListExpression:
            case ts.SyntaxKind.ConditionalExpression:
            case ts.SyntaxKind.DeleteExpression:
            case ts.SyntaxKind.ElementAccessExpression:
            case ts.SyntaxKind.FalseKeyword:
            case ts.SyntaxKind.FunctionExpression:
            case ts.SyntaxKind.Identifier:
            case ts.SyntaxKind.JsxElement:
            case ts.SyntaxKind.JsxFragment:
            case ts.SyntaxKind.JsxExpression:
            case ts.SyntaxKind.JsxOpeningElement:
            case ts.SyntaxKind.JsxOpeningFragment:
            case ts.SyntaxKind.JsxSelfClosingElement:
            case ts.SyntaxKind.MetaProperty:
            case ts.SyntaxKind.NewExpression:
            case ts.SyntaxKind.NonNullExpression:
            case ts.SyntaxKind.NoSubstitutionTemplateLiteral:
            case ts.SyntaxKind.NullKeyword:
            case ts.SyntaxKind.NumericLiteral:
            case ts.SyntaxKind.ObjectLiteralExpression:
            case ts.SyntaxKind.OmittedExpression:
            case ts.SyntaxKind.ParenthesizedExpression:
            case ts.SyntaxKind.PostfixUnaryExpression:
            case ts.SyntaxKind.PrefixUnaryExpression:
            case ts.SyntaxKind.PropertyAccessExpression:
            case ts.SyntaxKind.RegularExpressionLiteral:
            case ts.SyntaxKind.SpreadElement:
            case ts.SyntaxKind.StringLiteral:
            case ts.SyntaxKind.SuperKeyword:
            case ts.SyntaxKind.TaggedTemplateExpression:
            case ts.SyntaxKind.TemplateExpression:
            case ts.SyntaxKind.ThisKeyword:
            case ts.SyntaxKind.TrueKeyword:
            case ts.SyntaxKind.TypeAssertionExpression:
            case ts.SyntaxKind.TypeOfExpression:
            case ts.SyntaxKind.VoidExpression:
            case ts.SyntaxKind.YieldExpression:
                return true;
            default:
                return false;
        }
    }
    exports.isExpression = isExpression;
    function isExpressionStatement(node) {
        return node.kind === ts.SyntaxKind.ExpressionStatement;
    }
    exports.isExpressionStatement = isExpressionStatement;
    function isExpressionWithTypeArguments(node) {
        return node.kind === ts.SyntaxKind.ExpressionWithTypeArguments;
    }
    exports.isExpressionWithTypeArguments = isExpressionWithTypeArguments;
    function isExternalModuleReference(node) {
        return node.kind === ts.SyntaxKind.ExternalModuleReference;
    }
    exports.isExternalModuleReference = isExternalModuleReference;
    function isForInStatement(node) {
        return node.kind === ts.SyntaxKind.ForInStatement;
    }
    exports.isForInStatement = isForInStatement;
    function isForInOrOfStatement(node) {
        return node.kind === ts.SyntaxKind.ForOfStatement || node.kind === ts.SyntaxKind.ForInStatement;
    }
    exports.isForInOrOfStatement = isForInOrOfStatement;
    function isForOfStatement(node) {
        return node.kind === ts.SyntaxKind.ForOfStatement;
    }
    exports.isForOfStatement = isForOfStatement;
    function isForStatement(node) {
        return node.kind === ts.SyntaxKind.ForStatement;
    }
    exports.isForStatement = isForStatement;
    function isFunctionDeclaration(node) {
        return node.kind === ts.SyntaxKind.FunctionDeclaration;
    }
    exports.isFunctionDeclaration = isFunctionDeclaration;
    function isFunctionExpression(node) {
        return node.kind === ts.SyntaxKind.FunctionExpression;
    }
    exports.isFunctionExpression = isFunctionExpression;
    function isFunctionTypeNode(node) {
        return node.kind === ts.SyntaxKind.FunctionType;
    }
    exports.isFunctionTypeNode = isFunctionTypeNode;
    function isGetAccessorDeclaration(node) {
        return node.kind === ts.SyntaxKind.GetAccessor;
    }
    exports.isGetAccessorDeclaration = isGetAccessorDeclaration;
    function isIdentifier(node) {
        return node.kind === ts.SyntaxKind.Identifier;
    }
    exports.isIdentifier = isIdentifier;
    function isIfStatement(node) {
        return node.kind === ts.SyntaxKind.IfStatement;
    }
    exports.isIfStatement = isIfStatement;
    function isImportClause(node) {
        return node.kind === ts.SyntaxKind.ImportClause;
    }
    exports.isImportClause = isImportClause;
    function isImportDeclaration(node) {
        return node.kind === ts.SyntaxKind.ImportDeclaration;
    }
    exports.isImportDeclaration = isImportDeclaration;
    function isImportEqualsDeclaration(node) {
        return node.kind === ts.SyntaxKind.ImportEqualsDeclaration;
    }
    exports.isImportEqualsDeclaration = isImportEqualsDeclaration;
    function isImportSpecifier(node) {
        return node.kind === ts.SyntaxKind.ImportSpecifier;
    }
    exports.isImportSpecifier = isImportSpecifier;
    function isIndexedAccessTypeNode(node) {
        return node.kind === ts.SyntaxKind.IndexedAccessType;
    }
    exports.isIndexedAccessTypeNode = isIndexedAccessTypeNode;
    function isIndexSignatureDeclaration(node) {
        return node.kind === ts.SyntaxKind.IndexSignature;
    }
    exports.isIndexSignatureDeclaration = isIndexSignatureDeclaration;
    function isInferTypeNode(node) {
        return node.kind === ts.SyntaxKind.InferType;
    }
    exports.isInferTypeNode = isInferTypeNode;
    function isInterfaceDeclaration(node) {
        return node.kind === ts.SyntaxKind.InterfaceDeclaration;
    }
    exports.isInterfaceDeclaration = isInterfaceDeclaration;
    function isIntersectionTypeNode(node) {
        return node.kind === ts.SyntaxKind.IntersectionType;
    }
    exports.isIntersectionTypeNode = isIntersectionTypeNode;
    function isIterationStatement(node) {
        switch (node.kind) {
            case ts.SyntaxKind.ForStatement:
            case ts.SyntaxKind.ForOfStatement:
            case ts.SyntaxKind.ForInStatement:
            case ts.SyntaxKind.WhileStatement:
            case ts.SyntaxKind.DoStatement:
                return true;
            default:
                return false;
        }
    }
    exports.isIterationStatement = isIterationStatement;
    function isJsDoc(node) {
        return node.kind === ts.SyntaxKind.JSDocComment;
    }
    exports.isJsDoc = isJsDoc;
    function isJsxAttribute(node) {
        return node.kind === ts.SyntaxKind.JsxAttribute;
    }
    exports.isJsxAttribute = isJsxAttribute;
    function isJsxAttributeLike(node) {
        return node.kind === ts.SyntaxKind.JsxAttribute ||
            node.kind === ts.SyntaxKind.JsxSpreadAttribute;
    }
    exports.isJsxAttributeLike = isJsxAttributeLike;
    function isJsxAttributes(node) {
        return node.kind === ts.SyntaxKind.JsxAttributes;
    }
    exports.isJsxAttributes = isJsxAttributes;
    function isJsxClosingElement(node) {
        return node.kind === ts.SyntaxKind.JsxClosingElement;
    }
    exports.isJsxClosingElement = isJsxClosingElement;
    function isJsxClosingFragment(node) {
        return node.kind === ts.SyntaxKind.JsxClosingFragment;
    }
    exports.isJsxClosingFragment = isJsxClosingFragment;
    function isJsxElement(node) {
        return node.kind === ts.SyntaxKind.JsxElement;
    }
    exports.isJsxElement = isJsxElement;
    function isJsxExpression(node) {
        return node.kind === ts.SyntaxKind.JsxExpression;
    }
    exports.isJsxExpression = isJsxExpression;
    function isJsxFragment(node) {
        return node.kind === ts.SyntaxKind.JsxFragment;
    }
    exports.isJsxFragment = isJsxFragment;
    function isJsxOpeningElement(node) {
        return node.kind === ts.SyntaxKind.JsxOpeningElement;
    }
    exports.isJsxOpeningElement = isJsxOpeningElement;
    function isJsxOpeningFragment(node) {
        return node.kind === ts.SyntaxKind.JsxOpeningFragment;
    }
    exports.isJsxOpeningFragment = isJsxOpeningFragment;
    function isJsxOpeningLikeElement(node) {
        return node.kind === ts.SyntaxKind.JsxOpeningElement ||
            node.kind === ts.SyntaxKind.JsxSelfClosingElement;
    }
    exports.isJsxOpeningLikeElement = isJsxOpeningLikeElement;
    function isJsxSelfClosingElement(node) {
        return node.kind === ts.SyntaxKind.JsxSelfClosingElement;
    }
    exports.isJsxSelfClosingElement = isJsxSelfClosingElement;
    function isJsxSpreadAttribute(node) {
        return node.kind === ts.SyntaxKind.JsxSpreadAttribute;
    }
    exports.isJsxSpreadAttribute = isJsxSpreadAttribute;
    function isJsxText(node) {
        return node.kind === ts.SyntaxKind.JsxText;
    }
    exports.isJsxText = isJsxText;
    function isLabeledStatement(node) {
        return node.kind === ts.SyntaxKind.LabeledStatement;
    }
    exports.isLabeledStatement = isLabeledStatement;
    function isLiteralExpression(node) {
        return node.kind >= ts.SyntaxKind.FirstLiteralToken &&
            node.kind <= ts.SyntaxKind.LastLiteralToken;
    }
    exports.isLiteralExpression = isLiteralExpression;
    function isLiteralTypeNode(node) {
        return node.kind === ts.SyntaxKind.LiteralType;
    }
    exports.isLiteralTypeNode = isLiteralTypeNode;
    function isMappedTypeNode(node) {
        return node.kind === ts.SyntaxKind.MappedType;
    }
    exports.isMappedTypeNode = isMappedTypeNode;
    function isMetaProperty(node) {
        return node.kind === ts.SyntaxKind.MetaProperty;
    }
    exports.isMetaProperty = isMetaProperty;
    function isMethodDeclaration(node) {
        return node.kind === ts.SyntaxKind.MethodDeclaration;
    }
    exports.isMethodDeclaration = isMethodDeclaration;
    function isMethodSignature(node) {
        return node.kind === ts.SyntaxKind.MethodSignature;
    }
    exports.isMethodSignature = isMethodSignature;
    function isModuleBlock(node) {
        return node.kind === ts.SyntaxKind.ModuleBlock;
    }
    exports.isModuleBlock = isModuleBlock;
    function isModuleDeclaration(node) {
        return node.kind === ts.SyntaxKind.ModuleDeclaration;
    }
    exports.isModuleDeclaration = isModuleDeclaration;
    function isNamedExports(node) {
        return node.kind === ts.SyntaxKind.NamedExports;
    }
    exports.isNamedExports = isNamedExports;
    function isNamedImports(node) {
        return node.kind === ts.SyntaxKind.NamedImports;
    }
    exports.isNamedImports = isNamedImports;
    function isNamespaceDeclaration(node) {
        return isModuleDeclaration(node) &&
            node.name.kind === ts.SyntaxKind.Identifier &&
            node.body !== undefined &&
            (node.body.kind === ts.SyntaxKind.ModuleBlock ||
                isNamespaceDeclaration(node.body));
    }
    exports.isNamespaceDeclaration = isNamespaceDeclaration;
    function isNamespaceImport(node) {
        return node.kind === ts.SyntaxKind.NamespaceImport;
    }
    exports.isNamespaceImport = isNamespaceImport;
    function isNamespaceExportDeclaration(node) {
        return node.kind === ts.SyntaxKind.NamespaceExportDeclaration;
    }
    exports.isNamespaceExportDeclaration = isNamespaceExportDeclaration;
    function isNewExpression(node) {
        return node.kind === ts.SyntaxKind.NewExpression;
    }
    exports.isNewExpression = isNewExpression;
    function isNonNullExpression(node) {
        return node.kind === ts.SyntaxKind.NonNullExpression;
    }
    exports.isNonNullExpression = isNonNullExpression;
    function isNoSubstitutionTemplateLiteral(node) {
        return node.kind === ts.SyntaxKind.NoSubstitutionTemplateLiteral;
    }
    exports.isNoSubstitutionTemplateLiteral = isNoSubstitutionTemplateLiteral;
    function isNullLiteral(node) {
        return node.kind === ts.SyntaxKind.NullKeyword;
    }
    exports.isNullLiteral = isNullLiteral;
    function isNumericLiteral(node) {
        return node.kind === ts.SyntaxKind.NumericLiteral;
    }
    exports.isNumericLiteral = isNumericLiteral;
    function isNumericOrStringLikeLiteral(node) {
        switch (node.kind) {
            case ts.SyntaxKind.StringLiteral:
            case ts.SyntaxKind.NumericLiteral:
            case ts.SyntaxKind.NoSubstitutionTemplateLiteral:
                return true;
            default:
                return false;
        }
    }
    exports.isNumericOrStringLikeLiteral = isNumericOrStringLikeLiteral;
    function isObjectBindingPattern(node) {
        return node.kind === ts.SyntaxKind.ObjectBindingPattern;
    }
    exports.isObjectBindingPattern = isObjectBindingPattern;
    function isObjectLiteralExpression(node) {
        return node.kind === ts.SyntaxKind.ObjectLiteralExpression;
    }
    exports.isObjectLiteralExpression = isObjectLiteralExpression;
    function isOmittedExpression(node) {
        return node.kind === ts.SyntaxKind.OmittedExpression;
    }
    exports.isOmittedExpression = isOmittedExpression;
    function isParameterDeclaration(node) {
        return node.kind === ts.SyntaxKind.Parameter;
    }
    exports.isParameterDeclaration = isParameterDeclaration;
    function isParenthesizedExpression(node) {
        return node.kind === ts.SyntaxKind.ParenthesizedExpression;
    }
    exports.isParenthesizedExpression = isParenthesizedExpression;
    function isParenthesizedTypeNode(node) {
        return node.kind === ts.SyntaxKind.ParenthesizedType;
    }
    exports.isParenthesizedTypeNode = isParenthesizedTypeNode;
    function isPostfixUnaryExpression(node) {
        return node.kind === ts.SyntaxKind.PostfixUnaryExpression;
    }
    exports.isPostfixUnaryExpression = isPostfixUnaryExpression;
    function isPrefixUnaryExpression(node) {
        return node.kind === ts.SyntaxKind.PrefixUnaryExpression;
    }
    exports.isPrefixUnaryExpression = isPrefixUnaryExpression;
    function isPropertyAccessExpression(node) {
        return node.kind === ts.SyntaxKind.PropertyAccessExpression;
    }
    exports.isPropertyAccessExpression = isPropertyAccessExpression;
    function isPropertyAssignment(node) {
        return node.kind === ts.SyntaxKind.PropertyAssignment;
    }
    exports.isPropertyAssignment = isPropertyAssignment;
    function isPropertyDeclaration(node) {
        return node.kind === ts.SyntaxKind.PropertyDeclaration;
    }
    exports.isPropertyDeclaration = isPropertyDeclaration;
    function isPropertySignature(node) {
        return node.kind === ts.SyntaxKind.PropertySignature;
    }
    exports.isPropertySignature = isPropertySignature;
    function isQualifiedName(node) {
        return node.kind === ts.SyntaxKind.QualifiedName;
    }
    exports.isQualifiedName = isQualifiedName;
    function isRegularExpressionLiteral(node) {
        return node.kind === ts.SyntaxKind.RegularExpressionLiteral;
    }
    exports.isRegularExpressionLiteral = isRegularExpressionLiteral;
    function isReturnStatement(node) {
        return node.kind === ts.SyntaxKind.ReturnStatement;
    }
    exports.isReturnStatement = isReturnStatement;
    function isSetAccessorDeclaration(node) {
        return node.kind === ts.SyntaxKind.SetAccessor;
    }
    exports.isSetAccessorDeclaration = isSetAccessorDeclaration;
    function isShorthandPropertyAssignment(node) {
        return node.kind === ts.SyntaxKind.ShorthandPropertyAssignment;
    }
    exports.isShorthandPropertyAssignment = isShorthandPropertyAssignment;
    function isSignatureDeclaration(node) {
        return node.parameters !== undefined;
    }
    exports.isSignatureDeclaration = isSignatureDeclaration;
    function isSourceFile(node) {
        return node.kind === ts.SyntaxKind.SourceFile;
    }
    exports.isSourceFile = isSourceFile;
    function isSpreadAssignment(node) {
        return node.kind === ts.SyntaxKind.SpreadAssignment;
    }
    exports.isSpreadAssignment = isSpreadAssignment;
    function isSpreadElement(node) {
        return node.kind === ts.SyntaxKind.SpreadElement;
    }
    exports.isSpreadElement = isSpreadElement;
    function isStringLiteral(node) {
        return node.kind === ts.SyntaxKind.StringLiteral;
    }
    exports.isStringLiteral = isStringLiteral;
    function isSwitchStatement(node) {
        return node.kind === ts.SyntaxKind.SwitchStatement;
    }
    exports.isSwitchStatement = isSwitchStatement;
    function isSyntaxList(node) {
        return node.kind === ts.SyntaxKind.SyntaxList;
    }
    exports.isSyntaxList = isSyntaxList;
    function isTaggedTemplateExpression(node) {
        return node.kind === ts.SyntaxKind.TaggedTemplateExpression;
    }
    exports.isTaggedTemplateExpression = isTaggedTemplateExpression;
    function isTemplateExpression(node) {
        return node.kind === ts.SyntaxKind.TemplateExpression;
    }
    exports.isTemplateExpression = isTemplateExpression;
    function isTemplateLiteral(node) {
        return node.kind === ts.SyntaxKind.TemplateExpression ||
            node.kind === ts.SyntaxKind.NoSubstitutionTemplateLiteral;
    }
    exports.isTemplateLiteral = isTemplateLiteral;
    function isTextualLiteral(node) {
        return node.kind === ts.SyntaxKind.StringLiteral ||
            node.kind === ts.SyntaxKind.NoSubstitutionTemplateLiteral;
    }
    exports.isTextualLiteral = isTextualLiteral;
    function isThrowStatement(node) {
        return node.kind === ts.SyntaxKind.ThrowStatement;
    }
    exports.isThrowStatement = isThrowStatement;
    function isTryStatement(node) {
        return node.kind === ts.SyntaxKind.TryStatement;
    }
    exports.isTryStatement = isTryStatement;
    function isTupleTypeNode(node) {
        return node.kind === ts.SyntaxKind.TupleType;
    }
    exports.isTupleTypeNode = isTupleTypeNode;
    function isTypeAliasDeclaration(node) {
        return node.kind === ts.SyntaxKind.TypeAliasDeclaration;
    }
    exports.isTypeAliasDeclaration = isTypeAliasDeclaration;
    function isTypeAssertion(node) {
        return node.kind === ts.SyntaxKind.TypeAssertionExpression;
    }
    exports.isTypeAssertion = isTypeAssertion;
    function isTypeLiteralNode(node) {
        return node.kind === ts.SyntaxKind.TypeLiteral;
    }
    exports.isTypeLiteralNode = isTypeLiteralNode;
    function isTypeOfExpression(node) {
        return node.kind === ts.SyntaxKind.TypeOfExpression;
    }
    exports.isTypeOfExpression = isTypeOfExpression;
    function isTypeOperatorNode(node) {
        return node.kind === ts.SyntaxKind.TypeOperator;
    }
    exports.isTypeOperatorNode = isTypeOperatorNode;
    function isTypeParameterDeclaration(node) {
        return node.kind === ts.SyntaxKind.TypeParameter;
    }
    exports.isTypeParameterDeclaration = isTypeParameterDeclaration;
    function isTypePredicateNode(node) {
        return node.kind === ts.SyntaxKind.TypePredicate;
    }
    exports.isTypePredicateNode = isTypePredicateNode;
    function isTypeReferenceNode(node) {
        return node.kind === ts.SyntaxKind.TypeReference;
    }
    exports.isTypeReferenceNode = isTypeReferenceNode;
    function isTypeQueryNode(node) {
        return node.kind === ts.SyntaxKind.TypeQuery;
    }
    exports.isTypeQueryNode = isTypeQueryNode;
    function isUnionTypeNode(node) {
        return node.kind === ts.SyntaxKind.UnionType;
    }
    exports.isUnionTypeNode = isUnionTypeNode;
    function isVariableDeclaration(node) {
        return node.kind === ts.SyntaxKind.VariableDeclaration;
    }
    exports.isVariableDeclaration = isVariableDeclaration;
    function isVariableStatement(node) {
        return node.kind === ts.SyntaxKind.VariableStatement;
    }
    exports.isVariableStatement = isVariableStatement;
    function isVariableDeclarationList(node) {
        return node.kind === ts.SyntaxKind.VariableDeclarationList;
    }
    exports.isVariableDeclarationList = isVariableDeclarationList;
    function isVoidExpression(node) {
        return node.kind === ts.SyntaxKind.VoidExpression;
    }
    exports.isVoidExpression = isVoidExpression;
    function isWhileStatement(node) {
        return node.kind === ts.SyntaxKind.WhileStatement;
    }
    exports.isWhileStatement = isWhileStatement;
    function isWithStatement(node) {
        return node.kind === ts.SyntaxKind.WithStatement;
    }
    exports.isWithStatement = isWithStatement;
},
/* tsutils/typeguard/type.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(636) /* ./3.2/type */, exports);
},
/* tsutils/typeguard/3.2/type.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(637) /* ../3.0/type */, exports);
},
/* tsutils/typeguard/3.0/type.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(638) /* ../2.9/type */, exports);
    const ts = require(44) /* typescript */;
    const type_1 = require(638) /* ../2.9/type */;
    function isTupleType(type) {
        return (type.flags & ts.TypeFlags.Object && type.objectFlags & ts.ObjectFlags.Tuple) !== 0;
    }
    exports.isTupleType = isTupleType;
    function isTupleTypeReference(type) {
        return type_1.isTypeReference(type) && isTupleType(type.target);
    }
    exports.isTupleTypeReference = isTupleTypeReference;
},
/* tsutils/typeguard/2.9/type.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(639) /* ../2.8/type */, exports);
},
/* tsutils/typeguard/2.8/type.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = require(44) /* typescript */;
    function isConditionalType(type) {
        return (type.flags & ts.TypeFlags.Conditional) !== 0;
    }
    exports.isConditionalType = isConditionalType;
    function isEnumType(type) {
        return (type.flags & ts.TypeFlags.Enum) !== 0;
    }
    exports.isEnumType = isEnumType;
    function isGenericType(type) {
        return (type.flags & ts.TypeFlags.Object) !== 0 &&
            (type.objectFlags & ts.ObjectFlags.ClassOrInterface) !== 0 &&
            (type.objectFlags & ts.ObjectFlags.Reference) !== 0;
    }
    exports.isGenericType = isGenericType;
    function isIndexedAccessType(type) {
        return (type.flags & ts.TypeFlags.IndexedAccess) !== 0;
    }
    exports.isIndexedAccessType = isIndexedAccessType;
    function isIndexedAccessype(type) {
        return (type.flags & ts.TypeFlags.Index) !== 0;
    }
    exports.isIndexedAccessype = isIndexedAccessype;
    function isInstantiableType(type) {
        return (type.flags & ts.TypeFlags.Instantiable) !== 0;
    }
    exports.isInstantiableType = isInstantiableType;
    function isInterfaceType(type) {
        return (type.flags & ts.TypeFlags.Object) !== 0 &&
            (type.objectFlags & ts.ObjectFlags.ClassOrInterface) !== 0;
    }
    exports.isInterfaceType = isInterfaceType;
    function isIntersectionType(type) {
        return (type.flags & ts.TypeFlags.Intersection) !== 0;
    }
    exports.isIntersectionType = isIntersectionType;
    function isLiteralType(type) {
        return (type.flags & (ts.TypeFlags.StringOrNumberLiteral | ts.TypeFlags.BigIntLiteral)) !== 0;
    }
    exports.isLiteralType = isLiteralType;
    function isObjectType(type) {
        return (type.flags & ts.TypeFlags.Object) !== 0;
    }
    exports.isObjectType = isObjectType;
    function isSubstitutionType(type) {
        return (type.flags & ts.TypeFlags.Substitution) !== 0;
    }
    exports.isSubstitutionType = isSubstitutionType;
    function isTypeParameter(type) {
        return (type.flags & ts.TypeFlags.TypeParameter) !== 0;
    }
    exports.isTypeParameter = isTypeParameter;
    function isTypeReference(type) {
        return (type.flags & ts.TypeFlags.Object) !== 0 &&
            (type.objectFlags & ts.ObjectFlags.Reference) !== 0;
    }
    exports.isTypeReference = isTypeReference;
    function isTypeVariable(type) {
        return (type.flags & ts.TypeFlags.TypeVariable) !== 0;
    }
    exports.isTypeVariable = isTypeVariable;
    function isUnionOrIntersectionType(type) {
        return (type.flags & ts.TypeFlags.UnionOrIntersection) !== 0;
    }
    exports.isUnionOrIntersectionType = isUnionOrIntersectionType;
    function isUnionType(type) {
        return (type.flags & ts.TypeFlags.Union) !== 0;
    }
    exports.isUnionType = isUnionType;
    function isUniqueESSymbolType(type) {
        return (type.flags & ts.TypeFlags.UniqueESSymbol) !== 0;
    }
    exports.isUniqueESSymbolType = isUniqueESSymbolType;
},
/* tsutils/util/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(641) /* ./util */, exports);
    tslib_1.__exportStar(require(644) /* ./usage */, exports);
    tslib_1.__exportStar(require(645) /* ./control-flow */, exports);
    tslib_1.__exportStar(require(643) /* ./type */, exports);
    tslib_1.__exportStar(require(646) /* ./convert-ast */, exports);
},
/* tsutils/util/util.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = require(44) /* typescript */;
    const node_1 = require(630) /* ../typeguard/node */;
    const _3_2_1 = require(642) /* ../typeguard/3.2 */;
    const type_1 = require(643) /* ./type */;
    function getChildOfKind(node, kind, sourceFile) {
        for (const child of node.getChildren(sourceFile))
            if (child.kind === kind)
                return child;
    }
    exports.getChildOfKind = getChildOfKind;
    function isTokenKind(kind) {
        return kind >= ts.SyntaxKind.FirstToken && kind <= ts.SyntaxKind.LastToken;
    }
    exports.isTokenKind = isTokenKind;
    function isNodeKind(kind) {
        return kind >= ts.SyntaxKind.FirstNode;
    }
    exports.isNodeKind = isNodeKind;
    function isAssignmentKind(kind) {
        return kind >= ts.SyntaxKind.FirstAssignment && kind <= ts.SyntaxKind.LastAssignment;
    }
    exports.isAssignmentKind = isAssignmentKind;
    function isTypeNodeKind(kind) {
        return kind >= ts.SyntaxKind.FirstTypeNode && kind <= ts.SyntaxKind.LastTypeNode;
    }
    exports.isTypeNodeKind = isTypeNodeKind;
    function isJsDocKind(kind) {
        return kind >= ts.SyntaxKind.FirstJSDocNode && kind <= ts.SyntaxKind.LastJSDocNode;
    }
    exports.isJsDocKind = isJsDocKind;
    function isKeywordKind(kind) {
        return kind >= ts.SyntaxKind.FirstKeyword && kind <= ts.SyntaxKind.LastKeyword;
    }
    exports.isKeywordKind = isKeywordKind;
    function isThisParameter(parameter) {
        return parameter.name.kind === ts.SyntaxKind.Identifier && parameter.name.originalKeywordKind === ts.SyntaxKind.ThisKeyword;
    }
    exports.isThisParameter = isThisParameter;
    function getModifier(node, kind) {
        if (node.modifiers !== undefined)
            for (const modifier of node.modifiers)
                if (modifier.kind === kind)
                    return modifier;
    }
    exports.getModifier = getModifier;
    function hasModifier(modifiers, ...kinds) {
        if (modifiers === undefined)
            return false;
        for (const modifier of modifiers)
            if (kinds.includes(modifier.kind))
                return true;
        return false;
    }
    exports.hasModifier = hasModifier;
    function isParameterProperty(node) {
        return hasModifier(node.modifiers, ts.SyntaxKind.PublicKeyword, ts.SyntaxKind.ProtectedKeyword, ts.SyntaxKind.PrivateKeyword, ts.SyntaxKind.ReadonlyKeyword);
    }
    exports.isParameterProperty = isParameterProperty;
    function hasAccessModifier(node) {
        return hasModifier(node.modifiers, ts.SyntaxKind.PublicKeyword, ts.SyntaxKind.ProtectedKeyword, ts.SyntaxKind.PrivateKeyword);
    }
    exports.hasAccessModifier = hasAccessModifier;
    function isFlagSet(obj, flag) {
        return (obj.flags & flag) !== 0;
    }
    exports.isNodeFlagSet = isFlagSet;
    exports.isTypeFlagSet = isFlagSet;
    exports.isSymbolFlagSet = isFlagSet;
    function isObjectFlagSet(objectType, flag) {
        return (objectType.objectFlags & flag) !== 0;
    }
    exports.isObjectFlagSet = isObjectFlagSet;
    function isModifierFlagSet(node, flag) {
        return (ts.getCombinedModifierFlags(node) & flag) !== 0;
    }
    exports.isModifierFlagSet = isModifierFlagSet;
    function getPreviousStatement(statement) {
        const parent = statement.parent;
        if (node_1.isBlockLike(parent)) {
            const index = parent.statements.indexOf(statement);
            if (index > 0)
                return parent.statements[index - 1];
        }
    }
    exports.getPreviousStatement = getPreviousStatement;
    function getNextStatement(statement) {
        const parent = statement.parent;
        if (node_1.isBlockLike(parent)) {
            const index = parent.statements.indexOf(statement);
            if (index < parent.statements.length)
                return parent.statements[index + 1];
        }
    }
    exports.getNextStatement = getNextStatement;
    function getPreviousToken(node, sourceFile) {
        let parent = node.parent;
        while (parent !== undefined && parent.pos === node.pos)
            parent = parent.parent;
        if (parent === undefined)
            return;
        outer: while (true) {
            const children = parent.getChildren(sourceFile);
            for (let i = children.length - 1; i >= 0; --i) {
                const child = children[i];
                if (child.pos < node.pos && child.kind !== ts.SyntaxKind.JSDocComment) {
                    if (isTokenKind(child.kind))
                        return child;
                    parent = child;
                    continue outer;
                }
            }
            return;
        }
    }
    exports.getPreviousToken = getPreviousToken;
    function getNextToken(node, sourceFile = node.getSourceFile()) {
        if (node.kind === ts.SyntaxKind.SourceFile || node.kind === ts.SyntaxKind.EndOfFileToken)
            return;
        const end = node.end;
        node = node.parent;
        while (node.end === end) {
            if (node.parent === undefined)
                return node.endOfFileToken;
            node = node.parent;
        }
        return getTokenAtPositionWorker(node, end, sourceFile, false);
    }
    exports.getNextToken = getNextToken;
    function getTokenAtPosition(parent, pos, sourceFile, allowJsDoc) {
        if (pos < parent.pos || pos >= parent.end)
            return;
        if (isTokenKind(parent.kind))
            return parent;
        if (sourceFile === undefined)
            sourceFile = parent.getSourceFile();
        return getTokenAtPositionWorker(parent, pos, sourceFile, allowJsDoc === true);
    }
    exports.getTokenAtPosition = getTokenAtPosition;
    function getTokenAtPositionWorker(node, pos, sourceFile, allowJsDoc) {
        outer: while (true) {
            for (const child of node.getChildren(sourceFile)) {
                if (child.end > pos && (allowJsDoc || child.kind !== ts.SyntaxKind.JSDocComment)) {
                    if (isTokenKind(child.kind))
                        return child;
                    node = child;
                    continue outer;
                }
            }
            return;
        }
    }
    function getCommentAtPosition(sourceFile, pos, parent = sourceFile) {
        const token = getTokenAtPosition(parent, pos, sourceFile);
        if (token === undefined || token.kind === ts.SyntaxKind.JsxText || pos >= token.end - (ts.tokenToString(token.kind) || '').length)
            return;
        const startPos = token.pos === 0
            ? (ts.getShebang(sourceFile.text) || '').length
            : token.pos;
        return startPos !== 0 && ts.forEachTrailingCommentRange(sourceFile.text, startPos, commentAtPositionCallback, pos) ||
            ts.forEachLeadingCommentRange(sourceFile.text, startPos, commentAtPositionCallback, pos);
    }
    exports.getCommentAtPosition = getCommentAtPosition;
    function commentAtPositionCallback(pos, end, kind, _nl, at) {
        return at >= pos && at < end ? { pos, end, kind } : undefined;
    }
    function isPositionInComment(sourceFile, pos, parent) {
        return getCommentAtPosition(sourceFile, pos, parent) !== undefined;
    }
    exports.isPositionInComment = isPositionInComment;
    function commentText(sourceText, comment) {
        return sourceText.substring(comment.pos + 2, comment.kind === ts.SyntaxKind.SingleLineCommentTrivia ? comment.end : comment.end - 2);
    }
    exports.commentText = commentText;
    function getWrappedNodeAtPosition(wrap, pos) {
        if (wrap.node.pos > pos || wrap.node.end <= pos)
            return;
        outer: while (true) {
            for (const child of wrap.children) {
                if (child.node.pos > pos)
                    return wrap;
                if (child.node.end > pos) {
                    wrap = child;
                    continue outer;
                }
            }
            return wrap;
        }
    }
    exports.getWrappedNodeAtPosition = getWrappedNodeAtPosition;
    function getPropertyName(propertyName) {
        if (propertyName.kind === ts.SyntaxKind.ComputedPropertyName) {
            if (!node_1.isLiteralExpression(propertyName.expression))
                return;
            if (_3_2_1.isBigIntLiteral(propertyName.expression))
                return propertyName.expression.text.slice(0, -1);
            return propertyName.expression.text;
        }
        return propertyName.text;
    }
    exports.getPropertyName = getPropertyName;
    function forEachDestructuringIdentifier(pattern, fn) {
        for (const element of pattern.elements) {
            if (element.kind !== ts.SyntaxKind.BindingElement)
                continue;
            let result;
            if (element.name.kind === ts.SyntaxKind.Identifier) {
                result = fn(element);
            }
            else {
                result = forEachDestructuringIdentifier(element.name, fn);
            }
            if (result)
                return result;
        }
    }
    exports.forEachDestructuringIdentifier = forEachDestructuringIdentifier;
    function forEachDeclaredVariable(declarationList, cb) {
        for (const declaration of declarationList.declarations) {
            let result;
            if (declaration.name.kind === ts.SyntaxKind.Identifier) {
                result = cb(declaration);
            }
            else {
                result = forEachDestructuringIdentifier(declaration.name, cb);
            }
            if (result)
                return result;
        }
    }
    exports.forEachDeclaredVariable = forEachDeclaredVariable;
    var VariableDeclarationKind;
    (function (VariableDeclarationKind) {
        VariableDeclarationKind[VariableDeclarationKind["Var"] = 0] = "Var";
        VariableDeclarationKind[VariableDeclarationKind["Let"] = 1] = "Let";
        VariableDeclarationKind[VariableDeclarationKind["Const"] = 2] = "Const";
    })(VariableDeclarationKind = exports.VariableDeclarationKind || (exports.VariableDeclarationKind = {}));
    function getVariableDeclarationKind(declarationList) {
        if (declarationList.flags & ts.NodeFlags.Let)
            return 1;
        if (declarationList.flags & ts.NodeFlags.Const)
            return 2;
        return 0;
    }
    exports.getVariableDeclarationKind = getVariableDeclarationKind;
    function isBlockScopedVariableDeclarationList(declarationList) {
        return (declarationList.flags & ts.NodeFlags.BlockScoped) !== 0;
    }
    exports.isBlockScopedVariableDeclarationList = isBlockScopedVariableDeclarationList;
    function isBlockScopedVariableDeclaration(declaration) {
        const parent = declaration.parent;
        return parent.kind === ts.SyntaxKind.CatchClause ||
            isBlockScopedVariableDeclarationList(parent);
    }
    exports.isBlockScopedVariableDeclaration = isBlockScopedVariableDeclaration;
    function isBlockScopedDeclarationStatement(statement) {
        switch (statement.kind) {
            case ts.SyntaxKind.VariableStatement:
                return isBlockScopedVariableDeclarationList(statement.declarationList);
            case ts.SyntaxKind.ClassDeclaration:
            case ts.SyntaxKind.EnumDeclaration:
            case ts.SyntaxKind.InterfaceDeclaration:
            case ts.SyntaxKind.TypeAliasDeclaration:
                return true;
            default:
                return false;
        }
    }
    exports.isBlockScopedDeclarationStatement = isBlockScopedDeclarationStatement;
    function isInSingleStatementContext(statement) {
        switch (statement.parent.kind) {
            case ts.SyntaxKind.ForStatement:
            case ts.SyntaxKind.ForInStatement:
            case ts.SyntaxKind.ForOfStatement:
            case ts.SyntaxKind.WhileStatement:
            case ts.SyntaxKind.DoStatement:
            case ts.SyntaxKind.IfStatement:
            case ts.SyntaxKind.WithStatement:
            case ts.SyntaxKind.LabeledStatement:
                return true;
            default:
                return false;
        }
    }
    exports.isInSingleStatementContext = isInSingleStatementContext;
    var ScopeBoundary;
    (function (ScopeBoundary) {
        ScopeBoundary[ScopeBoundary["None"] = 0] = "None";
        ScopeBoundary[ScopeBoundary["Function"] = 1] = "Function";
        ScopeBoundary[ScopeBoundary["Block"] = 2] = "Block";
        ScopeBoundary[ScopeBoundary["Type"] = 4] = "Type";
        ScopeBoundary[ScopeBoundary["ConditionalType"] = 8] = "ConditionalType";
    })(ScopeBoundary = exports.ScopeBoundary || (exports.ScopeBoundary = {}));
    var ScopeBoundarySelector;
    (function (ScopeBoundarySelector) {
        ScopeBoundarySelector[ScopeBoundarySelector["Function"] = 1] = "Function";
        ScopeBoundarySelector[ScopeBoundarySelector["Block"] = 3] = "Block";
        ScopeBoundarySelector[ScopeBoundarySelector["Type"] = 7] = "Type";
        ScopeBoundarySelector[ScopeBoundarySelector["InferType"] = 8] = "InferType";
    })(ScopeBoundarySelector = exports.ScopeBoundarySelector || (exports.ScopeBoundarySelector = {}));
    function isScopeBoundary(node) {
        return isFunctionScopeBoundary(node) || isBlockScopeBoundary(node) || isTypeScopeBoundary(node);
    }
    exports.isScopeBoundary = isScopeBoundary;
    function isTypeScopeBoundary(node) {
        switch (node.kind) {
            case ts.SyntaxKind.InterfaceDeclaration:
            case ts.SyntaxKind.TypeAliasDeclaration:
            case ts.SyntaxKind.MappedType:
                return 4;
            case ts.SyntaxKind.ConditionalType:
                return 8;
            default:
                return 0;
        }
    }
    exports.isTypeScopeBoundary = isTypeScopeBoundary;
    function isFunctionScopeBoundary(node) {
        switch (node.kind) {
            case ts.SyntaxKind.FunctionExpression:
            case ts.SyntaxKind.ArrowFunction:
            case ts.SyntaxKind.Constructor:
            case ts.SyntaxKind.ModuleDeclaration:
            case ts.SyntaxKind.ClassDeclaration:
            case ts.SyntaxKind.ClassExpression:
            case ts.SyntaxKind.EnumDeclaration:
            case ts.SyntaxKind.MethodDeclaration:
            case ts.SyntaxKind.FunctionDeclaration:
            case ts.SyntaxKind.GetAccessor:
            case ts.SyntaxKind.SetAccessor:
            case ts.SyntaxKind.MethodSignature:
            case ts.SyntaxKind.CallSignature:
            case ts.SyntaxKind.ConstructSignature:
            case ts.SyntaxKind.ConstructorType:
            case ts.SyntaxKind.FunctionType:
                return 1;
            case ts.SyntaxKind.SourceFile:
                return ts.isExternalModule(node) ? 1 : 0;
            default:
                return 0;
        }
    }
    exports.isFunctionScopeBoundary = isFunctionScopeBoundary;
    function isBlockScopeBoundary(node) {
        switch (node.kind) {
            case ts.SyntaxKind.Block:
                const parent = node.parent;
                return parent.kind !== ts.SyntaxKind.CatchClause &&
                    (parent.kind === ts.SyntaxKind.SourceFile ||
                        !isFunctionScopeBoundary(parent))
                    ? 2
                    : 0;
            case ts.SyntaxKind.ForStatement:
            case ts.SyntaxKind.ForInStatement:
            case ts.SyntaxKind.ForOfStatement:
            case ts.SyntaxKind.CaseBlock:
            case ts.SyntaxKind.CatchClause:
            case ts.SyntaxKind.WithStatement:
                return 2;
            default:
                return 0;
        }
    }
    exports.isBlockScopeBoundary = isBlockScopeBoundary;
    function hasOwnThisReference(node) {
        switch (node.kind) {
            case ts.SyntaxKind.ClassDeclaration:
            case ts.SyntaxKind.ClassExpression:
            case ts.SyntaxKind.FunctionExpression:
                return true;
            case ts.SyntaxKind.FunctionDeclaration:
                return node.body !== undefined;
            case ts.SyntaxKind.MethodDeclaration:
            case ts.SyntaxKind.GetAccessor:
            case ts.SyntaxKind.SetAccessor:
                return node.parent.kind === ts.SyntaxKind.ObjectLiteralExpression;
            default:
                return false;
        }
    }
    exports.hasOwnThisReference = hasOwnThisReference;
    function isFunctionWithBody(node) {
        switch (node.kind) {
            case ts.SyntaxKind.GetAccessor:
            case ts.SyntaxKind.SetAccessor:
            case ts.SyntaxKind.FunctionDeclaration:
            case ts.SyntaxKind.MethodDeclaration:
            case ts.SyntaxKind.Constructor:
                return node.body !== undefined;
            case ts.SyntaxKind.FunctionExpression:
            case ts.SyntaxKind.ArrowFunction:
                return true;
            default:
                return false;
        }
    }
    exports.isFunctionWithBody = isFunctionWithBody;
    function forEachToken(node, cb, sourceFile = node.getSourceFile()) {
        return (function iterate(child) {
            if (isTokenKind(child.kind))
                return cb(child);
            if (child.kind !== ts.SyntaxKind.JSDocComment)
                return child.getChildren(sourceFile).forEach(iterate);
        })(node);
    }
    exports.forEachToken = forEachToken;
    function forEachTokenWithTrivia(node, cb, sourceFile = node.getSourceFile()) {
        const fullText = sourceFile.text;
        const scanner = ts.createScanner(sourceFile.languageVersion, false, sourceFile.languageVariant, fullText);
        return forEachToken(node, (token) => {
            const tokenStart = token.kind === ts.SyntaxKind.JsxText || token.pos === token.end ? token.pos : token.getStart(sourceFile);
            if (tokenStart !== token.pos) {
                scanner.setTextPos(token.pos);
                let kind = scanner.scan();
                let pos = scanner.getTokenPos();
                while (pos < tokenStart) {
                    const textPos = scanner.getTextPos();
                    cb(fullText, kind, { pos, end: textPos }, token.parent);
                    if (textPos === tokenStart)
                        break;
                    kind = scanner.scan();
                    pos = scanner.getTokenPos();
                }
            }
            return cb(fullText, token.kind, { end: token.end, pos: tokenStart }, token.parent);
        }, sourceFile);
    }
    exports.forEachTokenWithTrivia = forEachTokenWithTrivia;
    function forEachComment(node, cb, sourceFile = node.getSourceFile()) {
        const fullText = sourceFile.text;
        const notJsx = sourceFile.languageVariant !== ts.LanguageVariant.JSX;
        return forEachToken(node, (token) => {
            if (token.pos === token.end)
                return;
            if (token.kind !== ts.SyntaxKind.JsxText)
                ts.forEachLeadingCommentRange(fullText, token.pos === 0 ? (ts.getShebang(fullText) || '').length : token.pos, commentCallback);
            if (notJsx || canHaveTrailingTrivia(token))
                return ts.forEachTrailingCommentRange(fullText, token.end, commentCallback);
        }, sourceFile);
        function commentCallback(pos, end, kind) {
            cb(fullText, { pos, end, kind });
        }
    }
    exports.forEachComment = forEachComment;
    function canHaveTrailingTrivia(token) {
        switch (token.kind) {
            case ts.SyntaxKind.CloseBraceToken:
                return token.parent.kind !== ts.SyntaxKind.JsxExpression || !isJsxElementOrFragment(token.parent.parent);
            case ts.SyntaxKind.GreaterThanToken:
                switch (token.parent.kind) {
                    case ts.SyntaxKind.JsxOpeningElement:
                        return token.end !== token.parent.end;
                    case ts.SyntaxKind.JsxOpeningFragment:
                        return false;
                    case ts.SyntaxKind.JsxSelfClosingElement:
                        return token.end !== token.parent.end ||
                            !isJsxElementOrFragment(token.parent.parent);
                    case ts.SyntaxKind.JsxClosingElement:
                    case ts.SyntaxKind.JsxClosingFragment:
                        return !isJsxElementOrFragment(token.parent.parent.parent);
                }
        }
        return true;
    }
    function isJsxElementOrFragment(node) {
        return node.kind === ts.SyntaxKind.JsxElement || node.kind === ts.SyntaxKind.JsxFragment;
    }
    function getLineRanges(sourceFile) {
        const lineStarts = sourceFile.getLineStarts();
        const result = [];
        const length = lineStarts.length;
        const sourceText = sourceFile.text;
        let pos = 0;
        for (let i = 1; i < length; ++i) {
            const end = lineStarts[i];
            let lineEnd = end;
            for (; lineEnd > pos; --lineEnd)
                if (!ts.isLineBreak(sourceText.charCodeAt(lineEnd - 1)))
                    break;
            result.push({
                pos,
                end,
                contentLength: lineEnd - pos,
            });
            pos = end;
        }
        result.push({
            pos,
            end: sourceFile.end,
            contentLength: sourceFile.end - pos,
        });
        return result;
    }
    exports.getLineRanges = getLineRanges;
    function getLineBreakStyle(sourceFile) {
        const lineStarts = sourceFile.getLineStarts();
        return lineStarts.length === 1 || lineStarts[1] < 2 || sourceFile.text[lineStarts[1] - 2] !== '\r'
            ? '\n'
            : '\r\n';
    }
    exports.getLineBreakStyle = getLineBreakStyle;
    let cachedScanner;
    function scanToken(text, languageVersion) {
        if (cachedScanner === undefined) {
            cachedScanner = ts.createScanner(languageVersion, false, undefined, text);
        }
        else {
            cachedScanner.setScriptTarget(languageVersion);
            cachedScanner.setText(text);
        }
        cachedScanner.scan();
        return cachedScanner;
    }
    function isValidIdentifier(text, languageVersion = ts.ScriptTarget.Latest) {
        const scan = scanToken(text, languageVersion);
        return scan.isIdentifier() && scan.getTextPos() === text.length && scan.getTokenPos() === 0;
    }
    exports.isValidIdentifier = isValidIdentifier;
    function charSize(ch) {
        return ch >= 0x10000 ? 2 : 1;
    }
    function isValidPropertyAccess(text, languageVersion = ts.ScriptTarget.Latest) {
        if (text.length === 0)
            return false;
        let ch = text.codePointAt(0);
        if (!ts.isIdentifierStart(ch, languageVersion))
            return false;
        for (let i = charSize(ch); i < text.length; i += charSize(ch)) {
            ch = text.codePointAt(i);
            if (!ts.isIdentifierPart(ch, languageVersion))
                return false;
        }
        return true;
    }
    exports.isValidPropertyAccess = isValidPropertyAccess;
    function isValidPropertyName(text, languageVersion = ts.ScriptTarget.Latest) {
        if (isValidPropertyAccess(text, languageVersion))
            return true;
        const scan = scanToken(text, languageVersion);
        return scan.getTextPos() === text.length &&
            scan.getToken() === ts.SyntaxKind.NumericLiteral && scan.getTokenValue() === text;
    }
    exports.isValidPropertyName = isValidPropertyName;
    function isValidNumericLiteral(text, languageVersion = ts.ScriptTarget.Latest) {
        const scan = scanToken(text, languageVersion);
        return scan.getToken() === ts.SyntaxKind.NumericLiteral && scan.getTextPos() === text.length && scan.getTokenPos() === 0;
    }
    exports.isValidNumericLiteral = isValidNumericLiteral;
    function isValidJsxIdentifier(text, languageVersion = ts.ScriptTarget.Latest) {
        if (text.length === 0)
            return false;
        let ch = text.codePointAt(0);
        if (!ts.isIdentifierStart(ch, languageVersion))
            return false;
        for (let i = charSize(ch); i < text.length; i += charSize(ch)) {
            ch = text.codePointAt(i);
            if (!ts.isIdentifierPart(ch, languageVersion) && ch !== 45)
                return false;
        }
        return true;
    }
    exports.isValidJsxIdentifier = isValidJsxIdentifier;
    function isNumericPropertyName(name) {
        return String(+name) === name;
    }
    exports.isNumericPropertyName = isNumericPropertyName;
    function isSameLine(sourceFile, pos1, pos2) {
        return ts.getLineAndCharacterOfPosition(sourceFile, pos1).line === ts.getLineAndCharacterOfPosition(sourceFile, pos2).line;
    }
    exports.isSameLine = isSameLine;
    var SideEffectOptions;
    (function (SideEffectOptions) {
        SideEffectOptions[SideEffectOptions["None"] = 0] = "None";
        SideEffectOptions[SideEffectOptions["TaggedTemplate"] = 1] = "TaggedTemplate";
        SideEffectOptions[SideEffectOptions["Constructor"] = 2] = "Constructor";
        SideEffectOptions[SideEffectOptions["JsxElement"] = 4] = "JsxElement";
    })(SideEffectOptions = exports.SideEffectOptions || (exports.SideEffectOptions = {}));
    function hasSideEffects(node, options) {
        switch (node.kind) {
            case ts.SyntaxKind.CallExpression:
            case ts.SyntaxKind.PostfixUnaryExpression:
            case ts.SyntaxKind.AwaitExpression:
            case ts.SyntaxKind.YieldExpression:
            case ts.SyntaxKind.DeleteExpression:
                return true;
            case ts.SyntaxKind.TypeAssertionExpression:
            case ts.SyntaxKind.AsExpression:
            case ts.SyntaxKind.ParenthesizedExpression:
            case ts.SyntaxKind.NonNullExpression:
            case ts.SyntaxKind.VoidExpression:
            case ts.SyntaxKind.TypeOfExpression:
            case ts.SyntaxKind.PropertyAccessExpression:
            case ts.SyntaxKind.SpreadElement:
            case ts.SyntaxKind.PartiallyEmittedExpression:
                return hasSideEffects(node.expression, options);
            case ts.SyntaxKind.BinaryExpression:
                return isAssignmentKind(node.operatorToken.kind) ||
                    hasSideEffects(node.left, options) ||
                    hasSideEffects(node.right, options);
            case ts.SyntaxKind.PrefixUnaryExpression:
                switch (node.operator) {
                    case ts.SyntaxKind.PlusPlusToken:
                    case ts.SyntaxKind.MinusMinusToken:
                        return true;
                    default:
                        return hasSideEffects(node.operand, options);
                }
            case ts.SyntaxKind.ElementAccessExpression:
                return hasSideEffects(node.expression, options) ||
                    node.argumentExpression !== undefined &&
                        hasSideEffects(node.argumentExpression, options);
            case ts.SyntaxKind.ConditionalExpression:
                return hasSideEffects(node.condition, options) ||
                    hasSideEffects(node.whenTrue, options) ||
                    hasSideEffects(node.whenFalse, options);
            case ts.SyntaxKind.NewExpression:
                if (options & 2 || hasSideEffects(node.expression, options))
                    return true;
                if (node.arguments !== undefined)
                    for (const child of node.arguments)
                        if (hasSideEffects(child, options))
                            return true;
                return false;
            case ts.SyntaxKind.TaggedTemplateExpression:
                if (options & 1 || hasSideEffects(node.tag, options))
                    return true;
                if (node.template.kind === ts.SyntaxKind.NoSubstitutionTemplateLiteral)
                    return false;
                node = node.template;
            case ts.SyntaxKind.TemplateExpression:
                for (const child of node.templateSpans)
                    if (hasSideEffects(child.expression, options))
                        return true;
                return false;
            case ts.SyntaxKind.ClassExpression:
                return classExpressionHasSideEffects(node, options);
            case ts.SyntaxKind.ArrayLiteralExpression:
                for (const child of node.elements)
                    if (hasSideEffects(child, options))
                        return true;
                return false;
            case ts.SyntaxKind.ObjectLiteralExpression:
                for (const child of node.properties) {
                    if (child.name !== undefined && child.name.kind === ts.SyntaxKind.ComputedPropertyName &&
                        hasSideEffects(child.name.expression, options))
                        return true;
                    switch (child.kind) {
                        case ts.SyntaxKind.PropertyAssignment:
                            if (hasSideEffects(child.initializer, options))
                                return true;
                            break;
                        case ts.SyntaxKind.SpreadAssignment:
                            if (hasSideEffects(child.expression, options))
                                return true;
                    }
                }
                return false;
            case ts.SyntaxKind.JsxExpression:
                return node.expression !== undefined && hasSideEffects(node.expression, options);
            case ts.SyntaxKind.JsxElement:
            case ts.SyntaxKind.JsxFragment:
                for (const child of node.children)
                    if (child.kind !== ts.SyntaxKind.JsxText && hasSideEffects(child, options))
                        return true;
                if (node.kind === ts.SyntaxKind.JsxFragment)
                    return false;
                node = node.openingElement;
            case ts.SyntaxKind.JsxSelfClosingElement:
            case ts.SyntaxKind.JsxOpeningElement:
                if (options & 4)
                    return true;
                for (const child of node.attributes.properties) {
                    if (child.kind === ts.SyntaxKind.JsxSpreadAttribute) {
                        if (hasSideEffects(child.expression, options))
                            return true;
                    }
                    else if (child.initializer !== undefined && hasSideEffects(child.initializer, options)) {
                        return true;
                    }
                }
                return false;
            case ts.SyntaxKind.CommaListExpression:
                for (const child of node.elements)
                    if (hasSideEffects(child, options))
                        return true;
                return false;
            default:
                return false;
        }
    }
    exports.hasSideEffects = hasSideEffects;
    function classExpressionHasSideEffects(node, options) {
        if (node.heritageClauses !== undefined && node.heritageClauses[0].token === ts.SyntaxKind.ExtendsKeyword)
            for (const base of node.heritageClauses[0].types)
                if (hasSideEffects(base.expression, options))
                    return true;
        for (const child of node.members)
            if (child.name !== undefined && child.name.kind === ts.SyntaxKind.ComputedPropertyName &&
                hasSideEffects(child.name.expression, options) ||
                node_1.isPropertyDeclaration(child) && child.initializer !== undefined &&
                    hasSideEffects(child.initializer, options))
                return true;
        return false;
    }
    function getDeclarationOfBindingElement(node) {
        let parent = node.parent.parent;
        while (parent.kind === ts.SyntaxKind.BindingElement)
            parent = parent.parent.parent;
        return parent;
    }
    exports.getDeclarationOfBindingElement = getDeclarationOfBindingElement;
    function isExpressionValueUsed(node) {
        while (true) {
            const parent = node.parent;
            switch (parent.kind) {
                case ts.SyntaxKind.CallExpression:
                case ts.SyntaxKind.NewExpression:
                case ts.SyntaxKind.ElementAccessExpression:
                case ts.SyntaxKind.WhileStatement:
                case ts.SyntaxKind.DoStatement:
                case ts.SyntaxKind.WithStatement:
                case ts.SyntaxKind.ThrowStatement:
                case ts.SyntaxKind.ReturnStatement:
                case ts.SyntaxKind.JsxExpression:
                case ts.SyntaxKind.JsxSpreadAttribute:
                case ts.SyntaxKind.JsxElement:
                case ts.SyntaxKind.JsxFragment:
                case ts.SyntaxKind.JsxSelfClosingElement:
                case ts.SyntaxKind.ComputedPropertyName:
                case ts.SyntaxKind.ArrowFunction:
                case ts.SyntaxKind.ExportSpecifier:
                case ts.SyntaxKind.ExportAssignment:
                case ts.SyntaxKind.ImportDeclaration:
                case ts.SyntaxKind.ExternalModuleReference:
                case ts.SyntaxKind.Decorator:
                case ts.SyntaxKind.TaggedTemplateExpression:
                case ts.SyntaxKind.TemplateSpan:
                case ts.SyntaxKind.ExpressionWithTypeArguments:
                case ts.SyntaxKind.TypeOfExpression:
                case ts.SyntaxKind.AwaitExpression:
                case ts.SyntaxKind.YieldExpression:
                case ts.SyntaxKind.LiteralType:
                case ts.SyntaxKind.JsxAttributes:
                case ts.SyntaxKind.JsxOpeningElement:
                case ts.SyntaxKind.JsxClosingElement:
                case ts.SyntaxKind.IfStatement:
                case ts.SyntaxKind.CaseClause:
                case ts.SyntaxKind.SwitchStatement:
                    return true;
                case ts.SyntaxKind.PropertyAccessExpression:
                    return parent.expression === node;
                case ts.SyntaxKind.QualifiedName:
                    return parent.left === node;
                case ts.SyntaxKind.ShorthandPropertyAssignment:
                    return parent.objectAssignmentInitializer === node ||
                        !isInDestructuringAssignment(parent);
                case ts.SyntaxKind.PropertyAssignment:
                    return parent.initializer === node && !isInDestructuringAssignment(parent);
                case ts.SyntaxKind.SpreadAssignment:
                case ts.SyntaxKind.SpreadElement:
                case ts.SyntaxKind.ArrayLiteralExpression:
                    return !isInDestructuringAssignment(parent);
                case ts.SyntaxKind.ParenthesizedExpression:
                case ts.SyntaxKind.AsExpression:
                case ts.SyntaxKind.TypeAssertionExpression:
                case ts.SyntaxKind.PostfixUnaryExpression:
                case ts.SyntaxKind.PrefixUnaryExpression:
                case ts.SyntaxKind.NonNullExpression:
                    node = parent;
                    break;
                case ts.SyntaxKind.ForStatement:
                    return parent.condition === node;
                case ts.SyntaxKind.ForInStatement:
                case ts.SyntaxKind.ForOfStatement:
                    return parent.expression === node;
                case ts.SyntaxKind.ConditionalExpression:
                    if (parent.condition === node)
                        return true;
                    node = parent;
                    break;
                case ts.SyntaxKind.PropertyDeclaration:
                case ts.SyntaxKind.BindingElement:
                case ts.SyntaxKind.VariableDeclaration:
                case ts.SyntaxKind.Parameter:
                case ts.SyntaxKind.EnumMember:
                    return parent.initializer === node;
                case ts.SyntaxKind.ImportEqualsDeclaration:
                    return parent.moduleReference === node;
                case ts.SyntaxKind.CommaListExpression:
                    if (parent.elements[parent.elements.length - 1] !== node)
                        return false;
                    node = parent;
                    break;
                case ts.SyntaxKind.BinaryExpression:
                    if (parent.right === node) {
                        if (parent.operatorToken.kind === ts.SyntaxKind.CommaToken) {
                            node = parent;
                            break;
                        }
                        return true;
                    }
                    switch (parent.operatorToken.kind) {
                        case ts.SyntaxKind.CommaToken:
                        case ts.SyntaxKind.EqualsToken:
                            return false;
                        case ts.SyntaxKind.EqualsEqualsEqualsToken:
                        case ts.SyntaxKind.EqualsEqualsToken:
                        case ts.SyntaxKind.ExclamationEqualsEqualsToken:
                        case ts.SyntaxKind.ExclamationEqualsToken:
                        case ts.SyntaxKind.InstanceOfKeyword:
                        case ts.SyntaxKind.PlusToken:
                        case ts.SyntaxKind.MinusToken:
                        case ts.SyntaxKind.AsteriskToken:
                        case ts.SyntaxKind.SlashToken:
                        case ts.SyntaxKind.PercentToken:
                        case ts.SyntaxKind.AsteriskAsteriskToken:
                        case ts.SyntaxKind.GreaterThanToken:
                        case ts.SyntaxKind.GreaterThanGreaterThanToken:
                        case ts.SyntaxKind.GreaterThanGreaterThanGreaterThanToken:
                        case ts.SyntaxKind.GreaterThanEqualsToken:
                        case ts.SyntaxKind.LessThanToken:
                        case ts.SyntaxKind.LessThanLessThanToken:
                        case ts.SyntaxKind.LessThanEqualsToken:
                        case ts.SyntaxKind.AmpersandToken:
                        case ts.SyntaxKind.BarToken:
                        case ts.SyntaxKind.CaretToken:
                        case ts.SyntaxKind.BarBarToken:
                        case ts.SyntaxKind.AmpersandAmpersandToken:
                        case ts.SyntaxKind.InKeyword:
                            return true;
                        default:
                            node = parent;
                    }
                    break;
                default:
                    return false;
            }
        }
    }
    exports.isExpressionValueUsed = isExpressionValueUsed;
    function isInDestructuringAssignment(node) {
        switch (node.kind) {
            case ts.SyntaxKind.ShorthandPropertyAssignment:
                if (node.objectAssignmentInitializer !== undefined)
                    return true;
            case ts.SyntaxKind.PropertyAssignment:
            case ts.SyntaxKind.SpreadAssignment:
                node = node.parent;
                break;
            case ts.SyntaxKind.SpreadElement:
                if (node.parent.kind !== ts.SyntaxKind.ArrayLiteralExpression)
                    return false;
                node = node.parent;
        }
        while (true) {
            switch (node.parent.kind) {
                case ts.SyntaxKind.BinaryExpression:
                    return node.parent.left === node &&
                        node.parent.operatorToken.kind === ts.SyntaxKind.EqualsToken;
                case ts.SyntaxKind.ForOfStatement:
                    return node.parent.initializer === node;
                case ts.SyntaxKind.ArrayLiteralExpression:
                case ts.SyntaxKind.ObjectLiteralExpression:
                    node = node.parent;
                    break;
                case ts.SyntaxKind.SpreadAssignment:
                case ts.SyntaxKind.PropertyAssignment:
                    node = node.parent.parent;
                    break;
                case ts.SyntaxKind.SpreadElement:
                    if (node.parent.parent.kind !== ts.SyntaxKind.ArrayLiteralExpression)
                        return false;
                    node = node.parent.parent;
                    break;
                default:
                    return false;
            }
        }
    }
    var AccessKind;
    (function (AccessKind) {
        AccessKind[AccessKind["None"] = 0] = "None";
        AccessKind[AccessKind["Read"] = 1] = "Read";
        AccessKind[AccessKind["Write"] = 2] = "Write";
        AccessKind[AccessKind["Delete"] = 4] = "Delete";
        AccessKind[AccessKind["ReadWrite"] = 3] = "ReadWrite";
        AccessKind[AccessKind["Modification"] = 6] = "Modification";
    })(AccessKind = exports.AccessKind || (exports.AccessKind = {}));
    function getAccessKind(node) {
        const parent = node.parent;
        switch (parent.kind) {
            case ts.SyntaxKind.DeleteExpression:
                return 4;
            case ts.SyntaxKind.PostfixUnaryExpression:
                return 3;
            case ts.SyntaxKind.PrefixUnaryExpression:
                return parent.operator === ts.SyntaxKind.PlusPlusToken ||
                    parent.operator === ts.SyntaxKind.MinusMinusToken
                    ? 3
                    : 1;
            case ts.SyntaxKind.BinaryExpression:
                return parent.right === node
                    ? 1
                    : !isAssignmentKind(parent.operatorToken.kind)
                        ? 1
                        : parent.operatorToken.kind === ts.SyntaxKind.EqualsToken
                            ? 2
                            : 3;
            case ts.SyntaxKind.ShorthandPropertyAssignment:
                return parent.objectAssignmentInitializer === node
                    ? 1
                    : isInDestructuringAssignment(parent)
                        ? 2
                        : 1;
            case ts.SyntaxKind.PropertyAssignment:
                return parent.name === node
                    ? 0
                    : isInDestructuringAssignment(parent)
                        ? 2
                        : 1;
            case ts.SyntaxKind.ArrayLiteralExpression:
            case ts.SyntaxKind.SpreadElement:
            case ts.SyntaxKind.SpreadAssignment:
                return isInDestructuringAssignment(parent)
                    ? 2
                    : 1;
            case ts.SyntaxKind.ParenthesizedExpression:
            case ts.SyntaxKind.NonNullExpression:
            case ts.SyntaxKind.TypeAssertionExpression:
            case ts.SyntaxKind.AsExpression:
                return getAccessKind(parent);
            case ts.SyntaxKind.ForOfStatement:
            case ts.SyntaxKind.ForInStatement:
                return parent.initializer === node
                    ? 2
                    : 1;
            case ts.SyntaxKind.ExpressionWithTypeArguments:
                return parent.parent.token === ts.SyntaxKind.ExtendsKeyword &&
                    parent.parent.parent.kind !== ts.SyntaxKind.InterfaceDeclaration
                    ? 1
                    : 0;
            case ts.SyntaxKind.ComputedPropertyName:
            case ts.SyntaxKind.ExpressionStatement:
            case ts.SyntaxKind.TypeOfExpression:
            case ts.SyntaxKind.ElementAccessExpression:
            case ts.SyntaxKind.ForStatement:
            case ts.SyntaxKind.IfStatement:
            case ts.SyntaxKind.DoStatement:
            case ts.SyntaxKind.WhileStatement:
            case ts.SyntaxKind.SwitchStatement:
            case ts.SyntaxKind.WithStatement:
            case ts.SyntaxKind.ThrowStatement:
            case ts.SyntaxKind.CallExpression:
            case ts.SyntaxKind.NewExpression:
            case ts.SyntaxKind.TaggedTemplateExpression:
            case ts.SyntaxKind.JsxExpression:
            case ts.SyntaxKind.Decorator:
            case ts.SyntaxKind.TemplateSpan:
            case ts.SyntaxKind.JsxOpeningElement:
            case ts.SyntaxKind.JsxSelfClosingElement:
            case ts.SyntaxKind.JsxSpreadAttribute:
            case ts.SyntaxKind.VoidExpression:
            case ts.SyntaxKind.ReturnStatement:
            case ts.SyntaxKind.AwaitExpression:
            case ts.SyntaxKind.YieldExpression:
            case ts.SyntaxKind.ConditionalExpression:
            case ts.SyntaxKind.CaseClause:
            case ts.SyntaxKind.JsxElement:
                return 1;
            case ts.SyntaxKind.ArrowFunction:
                return parent.body === node
                    ? 1
                    : 2;
            case ts.SyntaxKind.PropertyDeclaration:
            case ts.SyntaxKind.VariableDeclaration:
            case ts.SyntaxKind.Parameter:
            case ts.SyntaxKind.EnumMember:
            case ts.SyntaxKind.BindingElement:
            case ts.SyntaxKind.JsxAttribute:
                return parent.initializer === node
                    ? 1
                    : 0;
            case ts.SyntaxKind.PropertyAccessExpression:
                return parent.expression === node
                    ? 1
                    : 0;
            case ts.SyntaxKind.ExportAssignment:
                return parent.isExportEquals
                    ? 1
                    : 0;
        }
        return 0;
    }
    exports.getAccessKind = getAccessKind;
    function isReassignmentTarget(node) {
        return (getAccessKind(node) & 2) !== 0;
    }
    exports.isReassignmentTarget = isReassignmentTarget;
    function canHaveJsDoc(node) {
        const kind = node.kind;
        switch (kind) {
            case ts.SyntaxKind.Parameter:
            case ts.SyntaxKind.CallSignature:
            case ts.SyntaxKind.ConstructSignature:
            case ts.SyntaxKind.MethodSignature:
            case ts.SyntaxKind.PropertySignature:
            case ts.SyntaxKind.ArrowFunction:
            case ts.SyntaxKind.ParenthesizedExpression:
            case ts.SyntaxKind.SpreadAssignment:
            case ts.SyntaxKind.ShorthandPropertyAssignment:
            case ts.SyntaxKind.PropertyAssignment:
            case ts.SyntaxKind.FunctionExpression:
            case ts.SyntaxKind.FunctionDeclaration:
            case ts.SyntaxKind.LabeledStatement:
            case ts.SyntaxKind.ExpressionStatement:
            case ts.SyntaxKind.VariableStatement:
            case ts.SyntaxKind.Constructor:
            case ts.SyntaxKind.MethodDeclaration:
            case ts.SyntaxKind.PropertyDeclaration:
            case ts.SyntaxKind.GetAccessor:
            case ts.SyntaxKind.SetAccessor:
            case ts.SyntaxKind.ClassDeclaration:
            case ts.SyntaxKind.ClassExpression:
            case ts.SyntaxKind.InterfaceDeclaration:
            case ts.SyntaxKind.TypeAliasDeclaration:
            case ts.SyntaxKind.EnumMember:
            case ts.SyntaxKind.EnumDeclaration:
            case ts.SyntaxKind.ModuleDeclaration:
            case ts.SyntaxKind.ImportEqualsDeclaration:
            case ts.SyntaxKind.IndexSignature:
            case ts.SyntaxKind.FunctionType:
            case ts.SyntaxKind.ConstructorType:
            case ts.SyntaxKind.JSDocFunctionType:
            case ts.SyntaxKind.EndOfFileToken:
            case ts.SyntaxKind.ExportDeclaration:
                return true;
            default:
                return false;
        }
    }
    exports.canHaveJsDoc = canHaveJsDoc;
    function getJsDoc(node, sourceFile) {
        if (node.kind === ts.SyntaxKind.EndOfFileToken)
            return parseJsDocWorker(node, sourceFile || node.parent);
        const result = [];
        for (const child of node.getChildren(sourceFile)) {
            if (!node_1.isJsDoc(child))
                break;
            result.push(child);
        }
        return result;
    }
    exports.getJsDoc = getJsDoc;
    function parseJsDocOfNode(node, considerTrailingComments, sourceFile = node.getSourceFile()) {
        if (canHaveJsDoc(node) && node.kind !== ts.SyntaxKind.EndOfFileToken) {
            const result = getJsDoc(node, sourceFile);
            if (result.length !== 0 || !considerTrailingComments)
                return result;
        }
        return parseJsDocWorker(node, sourceFile, considerTrailingComments);
    }
    exports.parseJsDocOfNode = parseJsDocOfNode;
    function parseJsDocWorker(node, sourceFile, considerTrailingComments) {
        const nodeStart = node.getStart(sourceFile);
        const start = ts[considerTrailingComments && isSameLine(sourceFile, node.pos, nodeStart)
            ? 'forEachTrailingCommentRange'
            : 'forEachLeadingCommentRange'](sourceFile.text, node.pos, (pos, _end, kind) => kind === ts.SyntaxKind.MultiLineCommentTrivia && sourceFile.text[pos + 2] === '*' ? { pos } : undefined);
        if (start === undefined)
            return [];
        const startPos = start.pos;
        const text = sourceFile.text.slice(startPos, nodeStart);
        const newSourceFile = ts.createSourceFile('jsdoc.ts', `${text}var a;`, sourceFile.languageVersion);
        const result = getJsDoc(newSourceFile.statements[0], newSourceFile);
        for (const doc of result)
            updateNode(doc, node);
        return result;
        function updateNode(n, parent) {
            n.pos += startPos;
            n.end += startPos;
            n.parent = parent;
            return ts.forEachChild(n, (child) => updateNode(child, n), (children) => {
                children.pos += startPos;
                children.end += startPos;
                for (const child of children)
                    updateNode(child, n);
            });
        }
    }
    var ImportKind;
    (function (ImportKind) {
        ImportKind[ImportKind["ImportDeclaration"] = 1] = "ImportDeclaration";
        ImportKind[ImportKind["ImportEquals"] = 2] = "ImportEquals";
        ImportKind[ImportKind["ExportFrom"] = 4] = "ExportFrom";
        ImportKind[ImportKind["DynamicImport"] = 8] = "DynamicImport";
        ImportKind[ImportKind["Require"] = 16] = "Require";
        ImportKind[ImportKind["ImportType"] = 32] = "ImportType";
        ImportKind[ImportKind["All"] = 63] = "All";
        ImportKind[ImportKind["AllImports"] = 59] = "AllImports";
        ImportKind[ImportKind["AllStaticImports"] = 3] = "AllStaticImports";
        ImportKind[ImportKind["AllImportExpressions"] = 24] = "AllImportExpressions";
        ImportKind[ImportKind["AllRequireLike"] = 18] = "AllRequireLike";
        ImportKind[ImportKind["AllNestedImports"] = 56] = "AllNestedImports";
        ImportKind[ImportKind["AllTopLevelImports"] = 7] = "AllTopLevelImports";
    })(ImportKind = exports.ImportKind || (exports.ImportKind = {}));
    function findImports(sourceFile, kinds) {
        const result = [];
        for (const node of findImportLikeNodes(sourceFile, kinds)) {
            switch (node.kind) {
                case ts.SyntaxKind.ImportDeclaration:
                    addIfTextualLiteral(node.moduleSpecifier);
                    break;
                case ts.SyntaxKind.ImportEqualsDeclaration:
                    addIfTextualLiteral(node.moduleReference.expression);
                    break;
                case ts.SyntaxKind.ExportDeclaration:
                    addIfTextualLiteral(node.moduleSpecifier);
                    break;
                case ts.SyntaxKind.CallExpression:
                    addIfTextualLiteral(node.arguments[0]);
                    break;
                case ts.SyntaxKind.ImportType:
                    if (node_1.isLiteralTypeNode(node.argument))
                        addIfTextualLiteral(node.argument.literal);
                    break;
                default:
                    throw new Error('unexpected node');
            }
        }
        return result;
        function addIfTextualLiteral(node) {
            if (node_1.isTextualLiteral(node))
                result.push(node);
        }
    }
    exports.findImports = findImports;
    function findImportLikeNodes(sourceFile, kinds) {
        return new ImportFinder(sourceFile, kinds).find();
    }
    exports.findImportLikeNodes = findImportLikeNodes;
    class ImportFinder {
        constructor(_sourceFile, _options) {
            this._sourceFile = _sourceFile;
            this._options = _options;
            this._result = [];
        }
        find() {
            if (this._sourceFile.isDeclarationFile)
                this._options &= ~24;
            if (this._options & 7)
                this._findImports(this._sourceFile.statements);
            if (this._options & 56)
                this._findNestedImports();
            return this._result;
        }
        _findImports(statements) {
            for (const statement of statements) {
                if (node_1.isImportDeclaration(statement)) {
                    if (this._options & 1)
                        this._result.push(statement);
                }
                else if (node_1.isImportEqualsDeclaration(statement)) {
                    if (this._options & 2 &&
                        statement.moduleReference.kind === ts.SyntaxKind.ExternalModuleReference)
                        this._result.push(statement);
                }
                else if (node_1.isExportDeclaration(statement)) {
                    if (statement.moduleSpecifier !== undefined && this._options & 4)
                        this._result.push(statement);
                }
                else if (node_1.isModuleDeclaration(statement)) {
                    this._findImportsInModule(statement);
                }
            }
        }
        _findImportsInModule(declaration) {
            if (declaration.body === undefined)
                return;
            if (declaration.body.kind === ts.SyntaxKind.ModuleDeclaration)
                return this._findImportsInModule(declaration.body);
            this._findImports(declaration.body.statements);
        }
        _findNestedImports() {
            let re;
            if ((this._options & 56) === 16) {
                re = /\brequire\s*[</(]/g;
            }
            else if (this._options & 16) {
                re = /\b(?:import|require)\s*[</(]/g;
            }
            else {
                re = /\bimport\s*[</(]/g;
            }
            const isJavaScriptFile = (this._sourceFile.flags & ts.NodeFlags.JavaScriptFile) !== 0;
            for (let match = re.exec(this._sourceFile.text); match !== null; match = re.exec(this._sourceFile.text)) {
                const token = getTokenAtPositionWorker(this._sourceFile, match.index, this._sourceFile, match[0][0] === 'i' && isJavaScriptFile);
                if (token.kind === ts.SyntaxKind.ImportKeyword) {
                    if (token.end - 'import'.length !== match.index)
                        continue;
                    switch (token.parent.kind) {
                        case ts.SyntaxKind.ImportType:
                            this._result.push(token.parent);
                            break;
                        case ts.SyntaxKind.CallExpression:
                            if (token.parent.arguments.length === 1)
                                this._result.push(token.parent);
                    }
                }
                else if (token.kind === ts.SyntaxKind.Identifier &&
                    token.end - 'require'.length === match.index &&
                    token.parent.kind === ts.SyntaxKind.CallExpression &&
                    token.parent.expression === token &&
                    token.parent.arguments.length === 1) {
                    this._result.push(token.parent);
                }
            }
        }
    }
    function isStatementInAmbientContext(node) {
        while (node.flags & ts.NodeFlags.NestedNamespace)
            node = node.parent;
        return hasModifier(node.modifiers, ts.SyntaxKind.DeclareKeyword) || isAmbientModuleBlock(node.parent);
    }
    exports.isStatementInAmbientContext = isStatementInAmbientContext;
    function isAmbientModuleBlock(node) {
        while (node.kind === ts.SyntaxKind.ModuleBlock) {
            do
                node = node.parent;
            while (node.flags & ts.NodeFlags.NestedNamespace);
            if (hasModifier(node.modifiers, ts.SyntaxKind.DeclareKeyword))
                return true;
            node = node.parent;
        }
        return false;
    }
    exports.isAmbientModuleBlock = isAmbientModuleBlock;
    function getIIFE(func) {
        let node = func.parent;
        while (node.kind === ts.SyntaxKind.ParenthesizedExpression)
            node = node.parent;
        return node_1.isCallExpression(node) && func.end <= node.expression.end ? node : undefined;
    }
    exports.getIIFE = getIIFE;
    function isStrictCompilerOptionEnabled(options, option) {
        return (options.strict ? options[option] !== false : options[option] === true) &&
            (option !== 'strictPropertyInitialization' || isStrictCompilerOptionEnabled(options, 'strictNullChecks'));
    }
    exports.isStrictCompilerOptionEnabled = isStrictCompilerOptionEnabled;
    function isCompilerOptionEnabled(options, option) {
        switch (option) {
            case 'stripInternal':
                return options.stripInternal === true && isCompilerOptionEnabled(options, 'declaration');
            case 'declaration':
                return options.declaration || isCompilerOptionEnabled(options, 'composite');
            case 'incremental':
                return options.incremental === undefined ? isCompilerOptionEnabled(options, 'composite') : options.incremental;
            case 'skipDefaultLibCheck':
                return options.skipDefaultLibCheck || isCompilerOptionEnabled(options, 'skipLibCheck');
            case 'suppressImplicitAnyIndexErrors':
                return options.suppressImplicitAnyIndexErrors === true && isCompilerOptionEnabled(options, 'noImplicitAny');
            case 'allowSyntheticDefaultImports':
                return options.allowSyntheticDefaultImports !== undefined
                    ? options.allowSyntheticDefaultImports
                    : isCompilerOptionEnabled(options, 'esModuleInterop') || options.module === ts.ModuleKind.System;
            case 'noImplicitAny':
            case 'noImplicitThis':
            case 'strictNullChecks':
            case 'strictFunctionTypes':
            case 'strictPropertyInitialization':
            case 'alwaysStrict':
            case 'strictBindCallApply':
                return isStrictCompilerOptionEnabled(options, option);
        }
        return options[option] === true;
    }
    exports.isCompilerOptionEnabled = isCompilerOptionEnabled;
    function isAmbientModule(node) {
        return node.name.kind === ts.SyntaxKind.StringLiteral || (node.flags & ts.NodeFlags.GlobalAugmentation) !== 0;
    }
    exports.isAmbientModule = isAmbientModule;
    function getCheckJsDirective(source) {
        let directive;
        ts.forEachLeadingCommentRange(source, (ts.getShebang(source) || '').length, (pos, end, kind) => {
            if (kind === ts.SyntaxKind.SingleLineCommentTrivia) {
                const text = source.slice(pos, end);
                const match = /^\/{2,3}\s*@ts-(no)?check(?:\s|$)/i.exec(text);
                if (match !== null)
                    directive = { pos, end, enabled: match[1] === undefined };
            }
        });
        return directive;
    }
    exports.getCheckJsDirective = getCheckJsDirective;
    function isConstAssertion(node) {
        return node_1.isTypeReferenceNode(node.type) &&
            node.type.typeName.kind === ts.SyntaxKind.Identifier &&
            node.type.typeName.escapedText === 'const';
    }
    exports.isConstAssertion = isConstAssertion;
    function isInConstContext(node) {
        let current = node;
        while (true) {
            const parent = current.parent;
            outer: switch (parent.kind) {
                case ts.SyntaxKind.TypeAssertionExpression:
                case ts.SyntaxKind.AsExpression:
                    return isConstAssertion(parent);
                case ts.SyntaxKind.PrefixUnaryExpression:
                    if (current.kind !== ts.SyntaxKind.NumericLiteral)
                        return false;
                    switch (parent.operator) {
                        case ts.SyntaxKind.PlusToken:
                        case ts.SyntaxKind.MinusToken:
                            current = parent;
                            break outer;
                        default:
                            return false;
                    }
                case ts.SyntaxKind.PropertyAssignment:
                    if (parent.initializer !== current)
                        return false;
                    current = parent.parent;
                    break;
                case ts.SyntaxKind.ShorthandPropertyAssignment:
                    current = parent.parent;
                    break;
                case ts.SyntaxKind.ParenthesizedExpression:
                case ts.SyntaxKind.ArrayLiteralExpression:
                case ts.SyntaxKind.ObjectLiteralExpression:
                    current = parent;
                    break;
                default:
                    return false;
            }
        }
    }
    exports.isInConstContext = isInConstContext;
    function isReadonlyAssignmentDeclaration(node, checker) {
        if (!isBindableObjectDefinePropertyCall(node))
            return false;
        const descriptorType = checker.getTypeAtLocation(node.arguments[2]);
        if (descriptorType.getProperty('value') === undefined)
            return descriptorType.getProperty('set') === undefined;
        const writableProp = descriptorType.getProperty('writable');
        if (writableProp === undefined)
            return false;
        const writableType = writableProp.valueDeclaration !== undefined && node_1.isPropertyAssignment(writableProp.valueDeclaration)
            ? checker.getTypeAtLocation(writableProp.valueDeclaration.initializer)
            : checker.getTypeOfSymbolAtLocation(writableProp, node.arguments[2]);
        return type_1.isBooleanLiteralType(writableType, false);
    }
    exports.isReadonlyAssignmentDeclaration = isReadonlyAssignmentDeclaration;
    function isBindableObjectDefinePropertyCall(node) {
        return node.arguments.length === 3 &&
            node_1.isEntityNameExpression(node.arguments[0]) &&
            node_1.isNumericOrStringLikeLiteral(node.arguments[1]) &&
            node_1.isPropertyAccessExpression(node.expression) &&
            node.expression.name.escapedText === 'defineProperty' &&
            node_1.isIdentifier(node.expression.expression) &&
            node.expression.expression.escapedText === 'Object';
    }
    exports.isBindableObjectDefinePropertyCall = isBindableObjectDefinePropertyCall;
    function isWellKnownSymbolLiterally(node) {
        return ts.isPropertyAccessExpression(node) &&
            ts.isIdentifier(node.expression) &&
            node.expression.escapedText === 'Symbol';
    }
    exports.isWellKnownSymbolLiterally = isWellKnownSymbolLiterally;
    function getPropertyNameOfWellKnownSymbol(node) {
        return {
            displayName: `[Symbol.${node.name.text}]`,
            symbolName: ('__@' + node.name.text),
        };
    }
    exports.getPropertyNameOfWellKnownSymbol = getPropertyNameOfWellKnownSymbol;
    function getLateBoundPropertyNames(node, checker) {
        const result = {
            known: true,
            names: [],
        };
        node = unwrapParentheses(node);
        if (isWellKnownSymbolLiterally(node)) {
            result.names.push(getPropertyNameOfWellKnownSymbol(node));
        }
        else {
            const type = checker.getTypeAtLocation(node);
            for (const key of type_1.unionTypeParts(checker.getBaseConstraintOfType(type) || type)) {
                const propertyName = type_1.getPropertyNameFromType(key);
                if (propertyName) {
                    result.names.push(propertyName);
                }
                else {
                    result.known = false;
                }
            }
        }
        return result;
    }
    exports.getLateBoundPropertyNames = getLateBoundPropertyNames;
    function getLateBoundPropertyNamesOfPropertyName(node, checker) {
        const staticName = getPropertyName(node);
        return staticName !== undefined
            ? { known: true, names: [{ displayName: staticName, symbolName: ts.escapeLeadingUnderscores(staticName) }] }
            : getLateBoundPropertyNames(node.expression, checker);
    }
    exports.getLateBoundPropertyNamesOfPropertyName = getLateBoundPropertyNamesOfPropertyName;
    function getSingleLateBoundPropertyNameOfPropertyName(node, checker) {
        const staticName = getPropertyName(node);
        if (staticName !== undefined)
            return { displayName: staticName, symbolName: ts.escapeLeadingUnderscores(staticName) };
        const { expression } = node;
        return isWellKnownSymbolLiterally(expression)
            ? getPropertyNameOfWellKnownSymbol(expression)
            : type_1.getPropertyNameFromType(checker.getTypeAtLocation(expression));
    }
    exports.getSingleLateBoundPropertyNameOfPropertyName = getSingleLateBoundPropertyNameOfPropertyName;
    function unwrapParentheses(node) {
        while (node.kind === ts.SyntaxKind.ParenthesizedExpression)
            node = node.expression;
        return node;
    }
    exports.unwrapParentheses = unwrapParentheses;
},
/* tsutils/typeguard/3.2/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const tslib_1 = require(628) /* tslib */;
    tslib_1.__exportStar(require(631) /* ./node */, exports);
    tslib_1.__exportStar(require(636) /* ./type */, exports);
},
/* tsutils/util/type.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = require(44) /* typescript */;
    const type_1 = require(635) /* ../typeguard/type */;
    const util_1 = require(641) /* ./util */;
    const node_1 = require(630) /* ../typeguard/node */;
    function isEmptyObjectType(type) {
        if (type_1.isObjectType(type) &&
            type.objectFlags & ts.ObjectFlags.Anonymous &&
            type.getProperties().length === 0 &&
            type.getCallSignatures().length === 0 &&
            type.getConstructSignatures().length === 0 &&
            type.getStringIndexType() === undefined &&
            type.getNumberIndexType() === undefined) {
            const baseTypes = type.getBaseTypes();
            return baseTypes === undefined || baseTypes.every(isEmptyObjectType);
        }
        return false;
    }
    exports.isEmptyObjectType = isEmptyObjectType;
    function removeOptionalityFromType(checker, type) {
        if (!containsTypeWithFlag(type, ts.TypeFlags.Undefined))
            return type;
        const allowsNull = containsTypeWithFlag(type, ts.TypeFlags.Null);
        type = checker.getNonNullableType(type);
        return allowsNull ? checker.getNullableType(type, ts.TypeFlags.Null) : type;
    }
    exports.removeOptionalityFromType = removeOptionalityFromType;
    function containsTypeWithFlag(type, flag) {
        for (const t of unionTypeParts(type))
            if (util_1.isTypeFlagSet(t, flag))
                return true;
        return false;
    }
    function isTypeAssignableToNumber(checker, type) {
        return isTypeAssignableTo(checker, type, ts.TypeFlags.NumberLike);
    }
    exports.isTypeAssignableToNumber = isTypeAssignableToNumber;
    function isTypeAssignableToString(checker, type) {
        return isTypeAssignableTo(checker, type, ts.TypeFlags.StringLike);
    }
    exports.isTypeAssignableToString = isTypeAssignableToString;
    function isTypeAssignableTo(checker, type, flags) {
        flags |= ts.TypeFlags.Any;
        let typeParametersSeen;
        return (function check(t) {
            if (type_1.isTypeParameter(t) && t.symbol !== undefined && t.symbol.declarations !== undefined) {
                if (typeParametersSeen === undefined) {
                    typeParametersSeen = new Set([t]);
                }
                else if (!typeParametersSeen.has(t)) {
                    typeParametersSeen.add(t);
                }
                else {
                    return false;
                }
                const declaration = t.symbol.declarations[0];
                if (declaration.constraint === undefined)
                    return true;
                return check(checker.getTypeFromTypeNode(declaration.constraint));
            }
            if (type_1.isUnionType(t))
                return t.types.every(check);
            if (type_1.isIntersectionType(t))
                return t.types.some(check);
            return util_1.isTypeFlagSet(t, flags);
        })(type);
    }
    function getCallSignaturesOfType(type) {
        if (type_1.isUnionType(type)) {
            const signatures = [];
            for (const t of type.types)
                signatures.push(...getCallSignaturesOfType(t));
            return signatures;
        }
        if (type_1.isIntersectionType(type)) {
            let signatures;
            for (const t of type.types) {
                const sig = getCallSignaturesOfType(t);
                if (sig.length !== 0) {
                    if (signatures !== undefined)
                        return [];
                    signatures = sig;
                }
            }
            return signatures === undefined ? [] : signatures;
        }
        return type.getCallSignatures();
    }
    exports.getCallSignaturesOfType = getCallSignaturesOfType;
    function unionTypeParts(type) {
        return type_1.isUnionType(type) ? type.types : [type];
    }
    exports.unionTypeParts = unionTypeParts;
    function intersectionTypeParts(type) {
        return type_1.isIntersectionType(type) ? type.types : [type];
    }
    exports.intersectionTypeParts = intersectionTypeParts;
    function someTypePart(type, predicate, cb) {
        return predicate(type) ? type.types.some(cb) : cb(type);
    }
    exports.someTypePart = someTypePart;
    function isThenableType(checker, node, type = checker.getTypeAtLocation(node)) {
        for (const ty of unionTypeParts(checker.getApparentType(type))) {
            const then = ty.getProperty('then');
            if (then === undefined)
                continue;
            const thenType = checker.getTypeOfSymbolAtLocation(then, node);
            for (const t of unionTypeParts(thenType))
                for (const signature of t.getCallSignatures())
                    if (signature.parameters.length !== 0 && isCallback(checker, signature.parameters[0], node))
                        return true;
        }
        return false;
    }
    exports.isThenableType = isThenableType;
    function isCallback(checker, param, node) {
        let type = checker.getApparentType(checker.getTypeOfSymbolAtLocation(param, node));
        if (param.valueDeclaration.dotDotDotToken) {
            type = type.getNumberIndexType();
            if (type === undefined)
                return false;
        }
        for (const t of unionTypeParts(type))
            if (t.getCallSignatures().length !== 0)
                return true;
        return false;
    }
    function isFalsyType(type) {
        if (type.flags & (ts.TypeFlags.Undefined | ts.TypeFlags.Null | ts.TypeFlags.Void))
            return true;
        if (type_1.isLiteralType(type))
            return !type.value;
        return isBooleanLiteralType(type, false);
    }
    exports.isFalsyType = isFalsyType;
    function isBooleanLiteralType(type, literal) {
        return util_1.isTypeFlagSet(type, ts.TypeFlags.BooleanLiteral) &&
            type.intrinsicName === (literal ? 'true' : 'false');
    }
    exports.isBooleanLiteralType = isBooleanLiteralType;
    function getPropertyOfType(type, name) {
        if (!name.startsWith('__'))
            return type.getProperty(name);
        return type.getProperties().find((s) => s.escapedName === name);
    }
    exports.getPropertyOfType = getPropertyOfType;
    function isPropertyReadonlyInType(type, name, checker) {
        let seenProperty = false;
        let seenReadonlySignature = false;
        for (const t of unionTypeParts(type)) {
            if (getPropertyOfType(t, name) === undefined) {
                const index = (util_1.isNumericPropertyName(name) ? checker.getIndexInfoOfType(t, ts.IndexKind.Number) : undefined) ||
                    checker.getIndexInfoOfType(t, ts.IndexKind.String);
                if (index !== undefined && index.isReadonly) {
                    if (seenProperty)
                        return true;
                    seenReadonlySignature = true;
                }
            }
            else if (seenReadonlySignature || isReadonlyPropertyIntersection(t, name, checker)) {
                return true;
            }
            else {
                seenProperty = true;
            }
        }
        return false;
    }
    exports.isPropertyReadonlyInType = isPropertyReadonlyInType;
    function isReadonlyPropertyIntersection(type, name, checker) {
        return someTypePart(type, type_1.isIntersectionType, (t) => {
            const prop = getPropertyOfType(t, name);
            if (prop === undefined)
                return false;
            if (prop.flags & ts.SymbolFlags.Transient) {
                if (/^(?:[1-9]\d*|0)$/.test(name) && type_1.isTupleTypeReference(t))
                    return t.target.readonly;
                switch (isReadonlyPropertyFromMappedType(t, name, checker)) {
                    case true:
                        return true;
                    case false:
                        return false;
                    default:
                }
            }
            return (util_1.isSymbolFlagSet(prop, ts.SymbolFlags.ValueModule) ||
                symbolHasReadonlyDeclaration(prop, checker));
        });
    }
    function isReadonlyPropertyFromMappedType(type, name, checker) {
        if (!type_1.isObjectType(type) || !util_1.isObjectFlagSet(type, ts.ObjectFlags.Mapped))
            return;
        const declaration = type.symbol.declarations[0];
        if (declaration.readonlyToken !== undefined && !/^__@[^@]+$/.test(name))
            return declaration.readonlyToken.kind !== ts.SyntaxKind.MinusToken;
        return isPropertyReadonlyInType(type.modifiersType, name, checker);
    }
    function symbolHasReadonlyDeclaration(symbol, checker) {
        return (symbol.flags & ts.SymbolFlags.Accessor) === ts.SymbolFlags.GetAccessor ||
            symbol.declarations !== undefined &&
                symbol.declarations.some((node) => util_1.isModifierFlagSet(node, ts.ModifierFlags.Readonly) ||
                    node_1.isVariableDeclaration(node) && util_1.isNodeFlagSet(node.parent, ts.NodeFlags.Const) ||
                    node_1.isCallExpression(node) && util_1.isReadonlyAssignmentDeclaration(node, checker) ||
                    node_1.isEnumMember(node) ||
                    (node_1.isPropertyAssignment(node) || node_1.isShorthandPropertyAssignment(node)) && util_1.isInConstContext(node.parent));
    }
    exports.symbolHasReadonlyDeclaration = symbolHasReadonlyDeclaration;
    function getPropertyNameFromType(type) {
        if (type.flags & (ts.TypeFlags.StringLiteral | ts.TypeFlags.NumberLiteral)) {
            const value = String(type.value);
            return { displayName: value, symbolName: ts.escapeLeadingUnderscores(value) };
        }
        if (type_1.isUniqueESSymbolType(type))
            return {
                displayName: `[${type.symbol ? type.symbol.name : type.escapedName.replace(/^__@|@\d+$/g, '')}]`,
                symbolName: type.escapedName,
            };
    }
    exports.getPropertyNameFromType = getPropertyNameFromType;
    function getConstructorTypeOfClassLikeDeclaration(node, checker) {
        return checker.getDeclaredTypeOfSymbol(node.name !== undefined ? checker.getSymbolAtLocation(node.name) : checker.getTypeAtLocation(node).symbol);
    }
    exports.getConstructorTypeOfClassLikeDeclaration = getConstructorTypeOfClassLikeDeclaration;
    function getInstanceTypeOfClassLikeDeclaration(node, checker) {
        return node.kind === ts.SyntaxKind.ClassDeclaration
            ? checker.getTypeAtLocation(node)
            : checker.getTypeOfSymbolAtLocation(checker.getTypeAtLocation(node).getProperty('prototype'), node);
    }
    exports.getInstanceTypeOfClassLikeDeclaration = getInstanceTypeOfClassLikeDeclaration;
    function getIteratorYieldResultFromIteratorResult(type, node, checker) {
        return type_1.isUnionType(type) && type.types.find((t) => {
            const done = t.getProperty('done');
            return done !== undefined &&
                isBooleanLiteralType(removeOptionalityFromType(checker, checker.getTypeOfSymbolAtLocation(done, node)), false);
        }) || type;
    }
    exports.getIteratorYieldResultFromIteratorResult = getIteratorYieldResultFromIteratorResult;
},
/* tsutils/util/usage.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const util_1 = require(641) /* ./util */;
    const ts = require(44) /* typescript */;
    var DeclarationDomain;
    (function (DeclarationDomain) {
        DeclarationDomain[DeclarationDomain["Namespace"] = 1] = "Namespace";
        DeclarationDomain[DeclarationDomain["Type"] = 2] = "Type";
        DeclarationDomain[DeclarationDomain["Value"] = 4] = "Value";
        DeclarationDomain[DeclarationDomain["Import"] = 8] = "Import";
        DeclarationDomain[DeclarationDomain["Any"] = 7] = "Any";
    })(DeclarationDomain = exports.DeclarationDomain || (exports.DeclarationDomain = {}));
    var UsageDomain;
    (function (UsageDomain) {
        UsageDomain[UsageDomain["Namespace"] = 1] = "Namespace";
        UsageDomain[UsageDomain["Type"] = 2] = "Type";
        UsageDomain[UsageDomain["Value"] = 4] = "Value";
        UsageDomain[UsageDomain["ValueOrNamespace"] = 5] = "ValueOrNamespace";
        UsageDomain[UsageDomain["Any"] = 7] = "Any";
        UsageDomain[UsageDomain["TypeQuery"] = 8] = "TypeQuery";
    })(UsageDomain = exports.UsageDomain || (exports.UsageDomain = {}));
    function getUsageDomain(node) {
        const parent = node.parent;
        switch (parent.kind) {
            case ts.SyntaxKind.TypeReference:
                return node.originalKeywordKind !== ts.SyntaxKind.ConstKeyword ? 2 : undefined;
            case ts.SyntaxKind.ExpressionWithTypeArguments:
                return parent.parent.token === ts.SyntaxKind.ImplementsKeyword ||
                    parent.parent.parent.kind === ts.SyntaxKind.InterfaceDeclaration
                    ? 2
                    : 4;
            case ts.SyntaxKind.TypeQuery:
                return 5 | 8;
            case ts.SyntaxKind.QualifiedName:
                if (parent.left === node) {
                    if (getEntityNameParent(parent).kind === ts.SyntaxKind.TypeQuery)
                        return 1 | 8;
                    return 1;
                }
                break;
            case ts.SyntaxKind.ExportSpecifier:
                if (parent.propertyName === undefined ||
                    parent.propertyName === node)
                    return 7;
                break;
            case ts.SyntaxKind.ExportAssignment:
                return 7;
            case ts.SyntaxKind.BindingElement:
                if (parent.initializer === node)
                    return 5;
                break;
            case ts.SyntaxKind.Parameter:
            case ts.SyntaxKind.EnumMember:
            case ts.SyntaxKind.PropertyDeclaration:
            case ts.SyntaxKind.VariableDeclaration:
            case ts.SyntaxKind.PropertyAssignment:
            case ts.SyntaxKind.PropertyAccessExpression:
            case ts.SyntaxKind.ImportEqualsDeclaration:
                if (parent.name !== node)
                    return 5;
                break;
            case ts.SyntaxKind.JsxAttribute:
            case ts.SyntaxKind.FunctionDeclaration:
            case ts.SyntaxKind.FunctionExpression:
            case ts.SyntaxKind.NamespaceImport:
            case ts.SyntaxKind.ClassDeclaration:
            case ts.SyntaxKind.ClassExpression:
            case ts.SyntaxKind.ModuleDeclaration:
            case ts.SyntaxKind.MethodDeclaration:
            case ts.SyntaxKind.EnumDeclaration:
            case ts.SyntaxKind.GetAccessor:
            case ts.SyntaxKind.SetAccessor:
            case ts.SyntaxKind.LabeledStatement:
            case ts.SyntaxKind.BreakStatement:
            case ts.SyntaxKind.ContinueStatement:
            case ts.SyntaxKind.ImportClause:
            case ts.SyntaxKind.ImportSpecifier:
            case ts.SyntaxKind.TypePredicate:
            case ts.SyntaxKind.MethodSignature:
            case ts.SyntaxKind.PropertySignature:
            case ts.SyntaxKind.NamespaceExportDeclaration:
            case ts.SyntaxKind.InterfaceDeclaration:
            case ts.SyntaxKind.TypeAliasDeclaration:
            case ts.SyntaxKind.TypeParameter:
                break;
            default:
                return 5;
        }
    }
    exports.getUsageDomain = getUsageDomain;
    function getDeclarationDomain(node) {
        switch (node.parent.kind) {
            case ts.SyntaxKind.TypeParameter:
            case ts.SyntaxKind.InterfaceDeclaration:
            case ts.SyntaxKind.TypeAliasDeclaration:
                return 2;
            case ts.SyntaxKind.ClassDeclaration:
            case ts.SyntaxKind.ClassExpression:
                return 2 | 4;
            case ts.SyntaxKind.EnumDeclaration:
                return 7;
            case ts.SyntaxKind.NamespaceImport:
            case ts.SyntaxKind.ImportClause:
                return 7 | 8;
            case ts.SyntaxKind.ImportEqualsDeclaration:
            case ts.SyntaxKind.ImportSpecifier:
                return node.parent.name === node
                    ? 7 | 8
                    : undefined;
            case ts.SyntaxKind.ModuleDeclaration:
                return 1;
            case ts.SyntaxKind.Parameter:
                if (node.parent.parent.kind === ts.SyntaxKind.IndexSignature || node.originalKeywordKind === ts.SyntaxKind.ThisKeyword)
                    return;
            case ts.SyntaxKind.BindingElement:
            case ts.SyntaxKind.VariableDeclaration:
                return node.parent.name === node ? 4 : undefined;
            case ts.SyntaxKind.FunctionDeclaration:
            case ts.SyntaxKind.FunctionExpression:
                return 4;
        }
    }
    exports.getDeclarationDomain = getDeclarationDomain;
    function collectVariableUsage(sourceFile) {
        return new UsageWalker().getUsage(sourceFile);
    }
    exports.collectVariableUsage = collectVariableUsage;
    class AbstractScope {
        constructor(_global) {
            this._global = _global;
            this._variables = new Map();
            this._uses = [];
            this._namespaceScopes = undefined;
            this._enumScopes = undefined;
        }
        addVariable(identifier, name, selector, exported, domain) {
            const variables = this.getDestinationScope(selector).getVariables();
            const declaration = {
                domain,
                exported,
                declaration: name,
            };
            const variable = variables.get(identifier);
            if (variable === undefined) {
                variables.set(identifier, {
                    domain,
                    declarations: [declaration],
                    uses: [],
                });
            }
            else {
                variable.domain |= domain;
                variable.declarations.push(declaration);
            }
        }
        addUse(use) {
            this._uses.push(use);
        }
        getVariables() {
            return this._variables;
        }
        getFunctionScope() {
            return this;
        }
        end(cb) {
            if (this._namespaceScopes !== undefined)
                this._namespaceScopes.forEach((value) => value.finish(cb));
            this._namespaceScopes = this._enumScopes = undefined;
            this._applyUses();
            this._variables.forEach((variable) => {
                for (const declaration of variable.declarations) {
                    const result = {
                        declarations: [],
                        domain: declaration.domain,
                        exported: declaration.exported,
                        inGlobalScope: this._global,
                        uses: [],
                    };
                    for (const other of variable.declarations)
                        if (other.domain & declaration.domain)
                            result.declarations.push(other.declaration);
                    for (const use of variable.uses)
                        if (use.domain & declaration.domain)
                            result.uses.push(use);
                    cb(result, declaration.declaration, this);
                }
            });
        }
        markExported(_name) { }
        createOrReuseNamespaceScope(name, _exported, ambient, hasExportStatement) {
            let scope;
            if (this._namespaceScopes === undefined) {
                this._namespaceScopes = new Map();
            }
            else {
                scope = this._namespaceScopes.get(name);
            }
            if (scope === undefined) {
                scope = new NamespaceScope(ambient, hasExportStatement, this);
                this._namespaceScopes.set(name, scope);
            }
            else {
                scope.refresh(ambient, hasExportStatement);
            }
            return scope;
        }
        createOrReuseEnumScope(name, _exported) {
            let scope;
            if (this._enumScopes === undefined) {
                this._enumScopes = new Map();
            }
            else {
                scope = this._enumScopes.get(name);
            }
            if (scope === undefined) {
                scope = new EnumScope(this);
                this._enumScopes.set(name, scope);
            }
            return scope;
        }
        _applyUses() {
            for (const use of this._uses)
                if (!this._applyUse(use))
                    this._addUseToParent(use);
            this._uses = [];
        }
        _applyUse(use, variables = this._variables) {
            const variable = variables.get(use.location.text);
            if (variable === undefined || (variable.domain & use.domain) === 0)
                return false;
            variable.uses.push(use);
            return true;
        }
        _addUseToParent(_use) { }
    }
    class RootScope extends AbstractScope {
        constructor(_exportAll, global) {
            super(global);
            this._exportAll = _exportAll;
            this._exports = undefined;
            this._innerScope = new NonRootScope(this, 1);
        }
        addVariable(identifier, name, selector, exported, domain) {
            if (domain & 8)
                return super.addVariable(identifier, name, selector, exported, domain);
            return this._innerScope.addVariable(identifier, name, selector, exported, domain);
        }
        addUse(use, origin) {
            if (origin === this._innerScope)
                return super.addUse(use);
            return this._innerScope.addUse(use);
        }
        markExported(id) {
            if (this._exports === undefined) {
                this._exports = [id.text];
            }
            else {
                this._exports.push(id.text);
            }
        }
        end(cb) {
            this._innerScope.end((value, key) => {
                value.exported = value.exported || this._exportAll
                    || this._exports !== undefined && this._exports.includes(key.text);
                value.inGlobalScope = this._global;
                return cb(value, key, this);
            });
            return super.end((value, key, scope) => {
                value.exported = value.exported || scope === this
                    && this._exports !== undefined && this._exports.includes(key.text);
                return cb(value, key, scope);
            });
        }
        getDestinationScope() {
            return this;
        }
    }
    class NonRootScope extends AbstractScope {
        constructor(_parent, _boundary) {
            super(false);
            this._parent = _parent;
            this._boundary = _boundary;
        }
        _addUseToParent(use) {
            return this._parent.addUse(use, this);
        }
        getDestinationScope(selector) {
            return this._boundary & selector
                ? this
                : this._parent.getDestinationScope(selector);
        }
    }
    class EnumScope extends NonRootScope {
        constructor(parent) {
            super(parent, 1);
        }
        end() {
            this._applyUses();
        }
    }
    class ConditionalTypeScope extends NonRootScope {
        constructor(parent) {
            super(parent, 8);
            this._state = 0;
        }
        updateState(newState) {
            this._state = newState;
        }
        addUse(use) {
            if (this._state === 2)
                return void this._uses.push(use);
            return this._parent.addUse(use, this);
        }
    }
    class FunctionScope extends NonRootScope {
        constructor(parent) {
            super(parent, 1);
        }
        beginBody() {
            this._applyUses();
        }
    }
    class AbstractNamedExpressionScope extends NonRootScope {
        constructor(_name, _domain, parent) {
            super(parent, 1);
            this._name = _name;
            this._domain = _domain;
        }
        end(cb) {
            this._innerScope.end(cb);
            return cb({
                declarations: [this._name],
                domain: this._domain,
                exported: false,
                uses: this._uses,
                inGlobalScope: false,
            }, this._name, this);
        }
        addUse(use, source) {
            if (source !== this._innerScope)
                return this._innerScope.addUse(use);
            if (use.domain & this._domain && use.location.text === this._name.text) {
                this._uses.push(use);
            }
            else {
                return this._parent.addUse(use, this);
            }
        }
        getFunctionScope() {
            return this._innerScope;
        }
        getDestinationScope() {
            return this._innerScope;
        }
    }
    class FunctionExpressionScope extends AbstractNamedExpressionScope {
        constructor(name, parent) {
            super(name, 4, parent);
            this._innerScope = new FunctionScope(this);
        }
        beginBody() {
            return this._innerScope.beginBody();
        }
    }
    class ClassExpressionScope extends AbstractNamedExpressionScope {
        constructor(name, parent) {
            super(name, 4 | 2, parent);
            this._innerScope = new NonRootScope(this, 1);
        }
    }
    class BlockScope extends NonRootScope {
        constructor(_functionScope, parent) {
            super(parent, 2);
            this._functionScope = _functionScope;
        }
        getFunctionScope() {
            return this._functionScope;
        }
    }
    function mapDeclaration(declaration) {
        return {
            declaration,
            exported: true,
            domain: getDeclarationDomain(declaration),
        };
    }
    class NamespaceScope extends NonRootScope {
        constructor(_ambient, _hasExport, parent) {
            super(parent, 1);
            this._ambient = _ambient;
            this._hasExport = _hasExport;
            this._innerScope = new NonRootScope(this, 1);
            this._exports = undefined;
        }
        finish(cb) {
            return super.end(cb);
        }
        end(cb) {
            this._innerScope.end((variable, key, scope) => {
                if (scope !== this._innerScope ||
                    !variable.exported && (!this._ambient || this._exports !== undefined && !this._exports.has(key.text)))
                    return cb(variable, key, scope);
                const namespaceVar = this._variables.get(key.text);
                if (namespaceVar === undefined) {
                    this._variables.set(key.text, {
                        declarations: variable.declarations.map(mapDeclaration),
                        domain: variable.domain,
                        uses: [...variable.uses],
                    });
                }
                else {
                    outer: for (const declaration of variable.declarations) {
                        for (const existing of namespaceVar.declarations)
                            if (existing.declaration === declaration)
                                continue outer;
                        namespaceVar.declarations.push(mapDeclaration(declaration));
                    }
                    namespaceVar.domain |= variable.domain;
                    for (const use of variable.uses) {
                        if (namespaceVar.uses.includes(use))
                            continue;
                        namespaceVar.uses.push(use);
                    }
                }
            });
            this._applyUses();
            this._innerScope = new NonRootScope(this, 1);
        }
        createOrReuseNamespaceScope(name, exported, ambient, hasExportStatement) {
            if (!exported && (!this._ambient || this._hasExport))
                return this._innerScope.createOrReuseNamespaceScope(name, exported, ambient || this._ambient, hasExportStatement);
            return super.createOrReuseNamespaceScope(name, exported, ambient || this._ambient, hasExportStatement);
        }
        createOrReuseEnumScope(name, exported) {
            if (!exported && (!this._ambient || this._hasExport))
                return this._innerScope.createOrReuseEnumScope(name, exported);
            return super.createOrReuseEnumScope(name, exported);
        }
        addUse(use, source) {
            if (source !== this._innerScope)
                return this._innerScope.addUse(use);
            this._uses.push(use);
        }
        refresh(ambient, hasExport) {
            this._ambient = ambient;
            this._hasExport = hasExport;
        }
        markExported(name, _as) {
            if (this._exports === undefined)
                this._exports = new Set();
            this._exports.add(name.text);
        }
        getDestinationScope() {
            return this._innerScope;
        }
    }
    function getEntityNameParent(name) {
        let parent = name.parent;
        while (parent.kind === ts.SyntaxKind.QualifiedName)
            parent = parent.parent;
        return parent;
    }
    class UsageWalker {
        constructor() {
            this._result = new Map();
        }
        getUsage(sourceFile) {
            const variableCallback = (variable, key) => {
                this._result.set(key, variable);
            };
            const isModule = ts.isExternalModule(sourceFile);
            this._scope = new RootScope(sourceFile.isDeclarationFile && isModule && !containsExportStatement(sourceFile), !isModule);
            const cb = (node) => {
                if (util_1.isBlockScopeBoundary(node))
                    return continueWithScope(node, new BlockScope(this._scope.getFunctionScope(), this._scope), handleBlockScope);
                switch (node.kind) {
                    case ts.SyntaxKind.ClassExpression:
                        return continueWithScope(node, node.name !== undefined
                            ? new ClassExpressionScope(node.name, this._scope)
                            : new NonRootScope(this._scope, 1));
                    case ts.SyntaxKind.ClassDeclaration:
                        this._handleDeclaration(node, true, 4 | 2);
                        return continueWithScope(node, new NonRootScope(this._scope, 1));
                    case ts.SyntaxKind.InterfaceDeclaration:
                    case ts.SyntaxKind.TypeAliasDeclaration:
                        this._handleDeclaration(node, true, 2);
                        return continueWithScope(node, new NonRootScope(this._scope, 4));
                    case ts.SyntaxKind.EnumDeclaration:
                        this._handleDeclaration(node, true, 7);
                        return continueWithScope(node, this._scope.createOrReuseEnumScope(node.name.text, util_1.hasModifier(node.modifiers, ts.SyntaxKind.ExportKeyword)));
                    case ts.SyntaxKind.ModuleDeclaration:
                        return this._handleModule(node, continueWithScope);
                    case ts.SyntaxKind.MappedType:
                        return continueWithScope(node, new NonRootScope(this._scope, 4));
                    case ts.SyntaxKind.FunctionExpression:
                    case ts.SyntaxKind.ArrowFunction:
                    case ts.SyntaxKind.Constructor:
                    case ts.SyntaxKind.MethodDeclaration:
                    case ts.SyntaxKind.FunctionDeclaration:
                    case ts.SyntaxKind.GetAccessor:
                    case ts.SyntaxKind.SetAccessor:
                    case ts.SyntaxKind.MethodSignature:
                    case ts.SyntaxKind.CallSignature:
                    case ts.SyntaxKind.ConstructSignature:
                    case ts.SyntaxKind.ConstructorType:
                    case ts.SyntaxKind.FunctionType:
                        return this._handleFunctionLikeDeclaration(node, cb, variableCallback);
                    case ts.SyntaxKind.ConditionalType:
                        return this._handleConditionalType(node, cb, variableCallback);
                    case ts.SyntaxKind.VariableDeclarationList:
                        this._handleVariableDeclaration(node);
                        break;
                    case ts.SyntaxKind.Parameter:
                        if (node.parent.kind !== ts.SyntaxKind.IndexSignature &&
                            (node.name.kind !== ts.SyntaxKind.Identifier ||
                                node.name.originalKeywordKind !== ts.SyntaxKind.ThisKeyword))
                            this._handleBindingName(node.name, false, false);
                        break;
                    case ts.SyntaxKind.EnumMember:
                        this._scope.addVariable(util_1.getPropertyName(node.name), node.name, 1, true, 4);
                        break;
                    case ts.SyntaxKind.ImportClause:
                    case ts.SyntaxKind.ImportSpecifier:
                    case ts.SyntaxKind.NamespaceImport:
                    case ts.SyntaxKind.ImportEqualsDeclaration:
                        this._handleDeclaration(node, false, 7 | 8);
                        break;
                    case ts.SyntaxKind.TypeParameter:
                        this._scope.addVariable(node.name.text, node.name, node.parent.kind === ts.SyntaxKind.InferType ? 8 : 7, false, 2);
                        break;
                    case ts.SyntaxKind.ExportSpecifier:
                        if (node.propertyName !== undefined)
                            return this._scope.markExported(node.propertyName, node.name);
                        return this._scope.markExported(node.name);
                    case ts.SyntaxKind.ExportAssignment:
                        if (node.expression.kind === ts.SyntaxKind.Identifier)
                            return this._scope.markExported(node.expression);
                        break;
                    case ts.SyntaxKind.Identifier:
                        const domain = getUsageDomain(node);
                        if (domain !== undefined)
                            this._scope.addUse({ domain, location: node });
                        return;
                }
                return ts.forEachChild(node, cb);
            };
            const continueWithScope = (node, scope, next = forEachChild) => {
                const savedScope = this._scope;
                this._scope = scope;
                next(node);
                this._scope.end(variableCallback);
                this._scope = savedScope;
            };
            const handleBlockScope = (node) => {
                if (node.kind === ts.SyntaxKind.CatchClause && node.variableDeclaration !== undefined)
                    this._handleBindingName(node.variableDeclaration.name, true, false);
                return ts.forEachChild(node, cb);
            };
            ts.forEachChild(sourceFile, cb);
            this._scope.end(variableCallback);
            return this._result;
            function forEachChild(node) {
                return ts.forEachChild(node, cb);
            }
        }
        _handleConditionalType(node, cb, varCb) {
            const savedScope = this._scope;
            const scope = this._scope = new ConditionalTypeScope(savedScope);
            cb(node.checkType);
            scope.updateState(1);
            cb(node.extendsType);
            scope.updateState(2);
            cb(node.trueType);
            scope.updateState(3);
            cb(node.falseType);
            scope.end(varCb);
            this._scope = savedScope;
        }
        _handleFunctionLikeDeclaration(node, cb, varCb) {
            if (node.decorators !== undefined)
                node.decorators.forEach(cb);
            const savedScope = this._scope;
            if (node.kind === ts.SyntaxKind.FunctionDeclaration)
                this._handleDeclaration(node, false, 4);
            const scope = this._scope = node.kind === ts.SyntaxKind.FunctionExpression && node.name !== undefined
                ? new FunctionExpressionScope(node.name, savedScope)
                : new FunctionScope(savedScope);
            if (node.name !== undefined)
                cb(node.name);
            if (node.typeParameters !== undefined)
                node.typeParameters.forEach(cb);
            node.parameters.forEach(cb);
            if (node.type !== undefined)
                cb(node.type);
            if (node.body !== undefined) {
                scope.beginBody();
                cb(node.body);
            }
            scope.end(varCb);
            this._scope = savedScope;
        }
        _handleModule(node, next) {
            if (node.flags & ts.NodeFlags.GlobalAugmentation)
                return next(node, this._scope.createOrReuseNamespaceScope('-global', false, true, false));
            if (node.name.kind === ts.SyntaxKind.Identifier) {
                const exported = isNamespaceExported(node);
                this._scope.addVariable(node.name.text, node.name, 1, exported, 1 | 4);
                const ambient = util_1.hasModifier(node.modifiers, ts.SyntaxKind.DeclareKeyword);
                return next(node, this._scope.createOrReuseNamespaceScope(node.name.text, exported, ambient, ambient && namespaceHasExportStatement(node)));
            }
            return next(node, this._scope.createOrReuseNamespaceScope(`"${node.name.text}"`, false, true, namespaceHasExportStatement(node)));
        }
        _handleDeclaration(node, blockScoped, domain) {
            if (node.name !== undefined)
                this._scope.addVariable(node.name.text, node.name, blockScoped ? 3 : 1, util_1.hasModifier(node.modifiers, ts.SyntaxKind.ExportKeyword), domain);
        }
        _handleBindingName(name, blockScoped, exported) {
            if (name.kind === ts.SyntaxKind.Identifier)
                return this._scope.addVariable(name.text, name, blockScoped ? 3 : 1, exported, 4);
            util_1.forEachDestructuringIdentifier(name, (declaration) => {
                this._scope.addVariable(declaration.name.text, declaration.name, blockScoped ? 3 : 1, exported, 4);
            });
        }
        _handleVariableDeclaration(declarationList) {
            const blockScoped = util_1.isBlockScopedVariableDeclarationList(declarationList);
            const exported = declarationList.parent.kind === ts.SyntaxKind.VariableStatement &&
                util_1.hasModifier(declarationList.parent.modifiers, ts.SyntaxKind.ExportKeyword);
            for (const declaration of declarationList.declarations)
                this._handleBindingName(declaration.name, blockScoped, exported);
        }
    }
    function isNamespaceExported(node) {
        return node.parent.kind === ts.SyntaxKind.ModuleDeclaration || util_1.hasModifier(node.modifiers, ts.SyntaxKind.ExportKeyword);
    }
    function namespaceHasExportStatement(ns) {
        if (ns.body === undefined || ns.body.kind !== ts.SyntaxKind.ModuleBlock)
            return false;
        return containsExportStatement(ns.body);
    }
    function containsExportStatement(block) {
        for (const statement of block.statements)
            if (statement.kind === ts.SyntaxKind.ExportDeclaration || statement.kind === ts.SyntaxKind.ExportAssignment)
                return true;
        return false;
    }
},
/* tsutils/util/control-flow.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = require(44) /* typescript */;
    const node_1 = require(630) /* ../typeguard/node */;
    function endsControlFlow(statement) {
        return getControlFlowEnd(statement).end;
    }
    exports.endsControlFlow = endsControlFlow;
    const defaultControlFlowEnd = { statements: [], end: false };
    function getControlFlowEnd(statement) {
        return node_1.isBlockLike(statement) ? handleBlock(statement) : getControlFlowEndWorker(statement);
    }
    exports.getControlFlowEnd = getControlFlowEnd;
    function getControlFlowEndWorker(statement) {
        switch (statement.kind) {
            case ts.SyntaxKind.ReturnStatement:
            case ts.SyntaxKind.ThrowStatement:
            case ts.SyntaxKind.ContinueStatement:
            case ts.SyntaxKind.BreakStatement:
                return { statements: [statement], end: true };
            case ts.SyntaxKind.Block:
                return handleBlock(statement);
            case ts.SyntaxKind.ForStatement:
            case ts.SyntaxKind.WhileStatement:
                return handleForAndWhileStatement(statement);
            case ts.SyntaxKind.ForOfStatement:
            case ts.SyntaxKind.ForInStatement:
                return handleForInOrOfStatement(statement);
            case ts.SyntaxKind.DoStatement:
                return matchBreakOrContinue(getControlFlowEndWorker(statement.statement), node_1.isBreakOrContinueStatement);
            case ts.SyntaxKind.IfStatement:
                return handleIfStatement(statement);
            case ts.SyntaxKind.SwitchStatement:
                return matchBreakOrContinue(handleSwitchStatement(statement), node_1.isBreakStatement);
            case ts.SyntaxKind.TryStatement:
                return handleTryStatement(statement);
            case ts.SyntaxKind.LabeledStatement:
                return matchLabel(getControlFlowEndWorker(statement.statement), statement.label);
            case ts.SyntaxKind.WithStatement:
                return getControlFlowEndWorker(statement.statement);
            default:
                return defaultControlFlowEnd;
        }
    }
    function handleBlock(statement) {
        const result = { statements: [], end: false };
        for (const s of statement.statements) {
            const current = getControlFlowEndWorker(s);
            result.statements.push(...current.statements);
            if (current.end) {
                result.end = true;
                break;
            }
        }
        return result;
    }
    function handleForInOrOfStatement(statement) {
        const end = matchBreakOrContinue(getControlFlowEndWorker(statement.statement), node_1.isBreakOrContinueStatement);
        end.end = false;
        return end;
    }
    function handleForAndWhileStatement(statement) {
        const constantCondition = statement.kind === ts.SyntaxKind.WhileStatement
            ? getConstantCondition(statement.expression)
            : statement.condition === undefined || getConstantCondition(statement.condition);
        if (constantCondition === false)
            return defaultControlFlowEnd;
        const end = matchBreakOrContinue(getControlFlowEndWorker(statement.statement), node_1.isBreakOrContinueStatement);
        if (constantCondition === undefined)
            end.end = false;
        return end;
    }
    function getConstantCondition(node) {
        switch (node.kind) {
            case ts.SyntaxKind.TrueKeyword:
                return true;
            case ts.SyntaxKind.FalseKeyword:
                return false;
            default:
                return;
        }
    }
    function handleIfStatement(node) {
        switch (getConstantCondition(node.expression)) {
            case true:
                return getControlFlowEndWorker(node.thenStatement);
            case false:
                return node.elseStatement === undefined
                    ? defaultControlFlowEnd
                    : getControlFlowEndWorker(node.elseStatement);
        }
        const then = getControlFlowEndWorker(node.thenStatement);
        if (node.elseStatement === undefined)
            return {
                statements: then.statements,
                end: false,
            };
        const elze = getControlFlowEndWorker(node.elseStatement);
        return {
            statements: [...then.statements, ...elze.statements],
            end: then.end && elze.end,
        };
    }
    function handleSwitchStatement(node) {
        let hasDefault = false;
        const result = {
            statements: [],
            end: false,
        };
        for (const clause of node.caseBlock.clauses) {
            if (clause.kind === ts.SyntaxKind.DefaultClause)
                hasDefault = true;
            const current = handleBlock(clause);
            result.end = current.end;
            result.statements.push(...current.statements);
        }
        if (!hasDefault)
            result.end = false;
        return result;
    }
    function handleTryStatement(node) {
        let finallyResult;
        if (node.finallyBlock !== undefined) {
            finallyResult = handleBlock(node.finallyBlock);
            if (finallyResult.end)
                return finallyResult;
        }
        const tryResult = handleBlock(node.tryBlock);
        if (node.catchClause === undefined)
            return { statements: finallyResult.statements.concat(tryResult.statements), end: tryResult.end };
        const catchResult = handleBlock(node.catchClause.block);
        return {
            statements: tryResult.statements
                .filter((s) => s.kind !== ts.SyntaxKind.ThrowStatement)
                .concat(catchResult.statements, finallyResult === undefined ? [] : finallyResult.statements),
            end: tryResult.end && catchResult.end,
        };
    }
    function matchBreakOrContinue(current, pred) {
        const result = {
            statements: [],
            end: current.end,
        };
        for (const statement of current.statements) {
            if (pred(statement) && statement.label === undefined) {
                result.end = false;
                continue;
            }
            result.statements.push(statement);
        }
        return result;
    }
    function matchLabel(current, label) {
        const result = {
            statements: [],
            end: current.end,
        };
        const labelText = label.text;
        for (const statement of current.statements) {
            switch (statement.kind) {
                case ts.SyntaxKind.BreakStatement:
                case ts.SyntaxKind.ContinueStatement:
                    if (statement.label !== undefined && statement.label.text === labelText) {
                        result.end = false;
                        continue;
                    }
            }
            result.statements.push(statement);
        }
        return result;
    }
},
/* tsutils/util/convert-ast.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = require(44) /* typescript */;
    const util_1 = require(641) /* ./util */;
    function convertAst(sourceFile) {
        const wrapped = {
            node: sourceFile,
            parent: undefined,
            kind: ts.SyntaxKind.SourceFile,
            children: [],
            next: undefined,
            skip: undefined,
        };
        const flat = [];
        let current = wrapped;
        let previous = current;
        ts.forEachChild(sourceFile, function wrap(node) {
            flat.push(node);
            const parent = current;
            previous.next = current = {
                node,
                parent,
                kind: node.kind,
                children: [],
                next: undefined,
                skip: undefined,
            };
            if (previous !== parent)
                setSkip(previous, current);
            previous = current;
            parent.children.push(current);
            if (util_1.isNodeKind(node.kind))
                ts.forEachChild(node, wrap);
            current = parent;
        });
        return {
            wrapped,
            flat,
        };
    }
    exports.convertAst = convertAst;
    function setSkip(node, skip) {
        do {
            node.skip = skip;
            node = node.parent;
        } while (node !== skip.parent);
    }
},
/* @typescript-eslint/eslint-plugin/dist/util/misc.js */ function _(require, module, exports) {
    /**
     * @fileoverview Really small utility functions that didn't deserve their own files
     */
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    /**
     * Check if the context file name is *.d.ts or *.d.tsx
     */
    function isDefinitionFile(fileName) {
        return /\.d\.tsx?$/i.test(fileName || '');
    }
    exports.isDefinitionFile = isDefinitionFile;
    /**
     * Upper cases the first character or the string
     */
    function upperCaseFirst(str) {
        return str[0].toUpperCase() + str.slice(1);
    }
    exports.upperCaseFirst = upperCaseFirst;
    function arraysAreEqual(a, b, eq) {
        return (a === b ||
            (a !== undefined &&
                b !== undefined &&
                a.length === b.length &&
                a.every((x, idx) => eq(x, b[idx]))));
    }
    exports.arraysAreEqual = arraysAreEqual;
    /** Returns the first non-`undefined` result. */
    function findFirstResult(inputs, getResult) {
        for (const element of inputs) {
            const result = getResult(element);
            if (result !== undefined) {
                return result;
            }
        }
        return undefined;
    }
    exports.findFirstResult = findFirstResult;
    /**
     * Gets a string representation of the name of the index signature.
     */
    function getNameFromIndexSignature(node) {
        const propName = node.parameters.find((parameter) => parameter.type === experimental_utils_1.AST_NODE_TYPES.Identifier);
        return propName ? propName.name : '(index signature)';
    }
    exports.getNameFromIndexSignature = getNameFromIndexSignature;
    /**
     * Gets a string name representation of the name of the given MethodDefinition
     * or ClassProperty node, with handling for computed property names.
     */
    function getNameFromMember(member, sourceCode) {
        if (member.key.type === experimental_utils_1.AST_NODE_TYPES.Identifier) {
            return member.key.name;
        }
        if (member.key.type === experimental_utils_1.AST_NODE_TYPES.Literal) {
            return `${member.key.value}`;
        }
        return sourceCode.text.slice(...member.key.range);
    }
    exports.getNameFromMember = getNameFromMember;
    function getEnumNames(myEnum) {
        return Object.keys(myEnum).filter(x => isNaN(parseInt(x)));
    }
    exports.getEnumNames = getEnumNames;
},
/* @typescript-eslint/eslint-plugin/dist/util/nullThrows.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * A set of common reasons for calling nullThrows
     */
    const NullThrowsReasons = {
        MissingParent: 'Expected node to have a parent.',
        MissingToken: (token, thing) => `Expected to find a ${token} for the ${thing}.`,
    };
    exports.NullThrowsReasons = NullThrowsReasons;
    /**
     * Assert that a value must not be null or undefined.
     * This is a nice explicit alternative to the non-null assertion operator.
     */
    function nullThrows(value, message) {
        // this function is primarily used to keep types happy in a safe way
        // i.e. is used when we expect that a value is never nullish
        // this means that it's pretty much impossible to test the below if...
        // so ignore it in coverage metrics.
        /* istanbul ignore if */
        if (value === null || value === undefined) {
            throw new Error(`Non-null Assertion Failed: ${message}`);
        }
        return value;
    }
    exports.nullThrows = nullThrows;
},
/* @typescript-eslint/eslint-plugin/dist/util/types.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const tsutils_1 = require(627) /* tsutils */;
    const ts = __importStar(require(44) /* typescript */);
    /**
     * @param type Type being checked by name.
     * @param allowedNames Symbol names checking on the type.
     * @returns Whether the type is, extends, or contains all of the allowed names.
     */
    function containsAllTypesByName(type, allowAny, allowedNames) {
        if (isTypeFlagSet(type, ts.TypeFlags.Any | ts.TypeFlags.Unknown)) {
            return !allowAny;
        }
        if (tsutils_1.isTypeReference(type)) {
            type = type.target;
        }
        if (typeof type.symbol !== 'undefined' &&
            allowedNames.has(type.symbol.name)) {
            return true;
        }
        if (tsutils_1.isUnionOrIntersectionType(type)) {
            return type.types.every(t => containsAllTypesByName(t, allowAny, allowedNames));
        }
        const bases = type.getBaseTypes();
        return (typeof bases !== 'undefined' &&
            bases.length > 0 &&
            bases.every(t => containsAllTypesByName(t, allowAny, allowedNames)));
    }
    exports.containsAllTypesByName = containsAllTypesByName;
    /**
     * Get the type name of a given type.
     * @param typeChecker The context sensitive TypeScript TypeChecker.
     * @param type The type to get the name of.
     */
    function getTypeName(typeChecker, type) {
        // It handles `string` and string literal types as string.
        if ((type.flags & ts.TypeFlags.StringLike) !== 0) {
            return 'string';
        }
        // If the type is a type parameter which extends primitive string types,
        // but it was not recognized as a string like. So check the constraint
        // type of the type parameter.
        if ((type.flags & ts.TypeFlags.TypeParameter) !== 0) {
            // `type.getConstraint()` method doesn't return the constraint type of
            // the type parameter for some reason. So this gets the constraint type
            // via AST.
            const node = type.symbol.declarations[0];
            if (node.constraint != null) {
                return getTypeName(typeChecker, typeChecker.getTypeFromTypeNode(node.constraint));
            }
        }
        // If the type is a union and all types in the union are string like,
        // return `string`. For example:
        // - `"a" | "b"` is string.
        // - `string | string[]` is not string.
        if (type.isUnion() &&
            type.types
                .map(value => getTypeName(typeChecker, value))
                .every(t => t === 'string')) {
            return 'string';
        }
        // If the type is an intersection and a type in the intersection is string
        // like, return `string`. For example: `string & {__htmlEscaped: void}`
        if (type.isIntersection() &&
            type.types
                .map(value => getTypeName(typeChecker, value))
                .some(t => t === 'string')) {
            return 'string';
        }
        return typeChecker.typeToString(type);
    }
    exports.getTypeName = getTypeName;
    /**
     * Resolves the given node's type. Will resolve to the type's generic constraint, if it has one.
     */
    function getConstrainedTypeAtLocation(checker, node) {
        const nodeType = checker.getTypeAtLocation(node);
        const constrained = checker.getBaseConstraintOfType(nodeType);
        return constrained !== null && constrained !== void 0 ? constrained : nodeType;
    }
    exports.getConstrainedTypeAtLocation = getConstrainedTypeAtLocation;
    /**
     * Checks if the given type is (or accepts) nullable
     * @param isReceiver true if the type is a receiving type (i.e. the type of a called function's parameter)
     */
    function isNullableType(type, { isReceiver = false, allowUndefined = true, } = {}) {
        const flags = getTypeFlags(type);
        if (isReceiver && flags & (ts.TypeFlags.Any | ts.TypeFlags.Unknown)) {
            return true;
        }
        if (allowUndefined) {
            return (flags & (ts.TypeFlags.Null | ts.TypeFlags.Undefined)) !== 0;
        }
        else {
            return (flags & ts.TypeFlags.Null) !== 0;
        }
    }
    exports.isNullableType = isNullableType;
    /**
     * Gets the declaration for the given variable
     */
    function getDeclaration(checker, node) {
        const symbol = checker.getSymbolAtLocation(node);
        if (!symbol) {
            return null;
        }
        const declarations = symbol.declarations;
        if (!declarations) {
            return null;
        }
        return declarations[0];
    }
    exports.getDeclaration = getDeclaration;
    /**
     * Gets all of the type flags in a type, iterating through unions automatically
     */
    function getTypeFlags(type) {
        let flags = 0;
        for (const t of tsutils_1.unionTypeParts(type)) {
            flags |= t.flags;
        }
        return flags;
    }
    exports.getTypeFlags = getTypeFlags;
    /**
     * Checks if the given type is (or accepts) the given flags
     * @param isReceiver true if the type is a receiving type (i.e. the type of a called function's parameter)
     */
    function isTypeFlagSet(type, flagsToCheck, isReceiver) {
        const flags = getTypeFlags(type);
        if (isReceiver && flags & (ts.TypeFlags.Any | ts.TypeFlags.Unknown)) {
            return true;
        }
        return (flags & flagsToCheck) !== 0;
    }
    exports.isTypeFlagSet = isTypeFlagSet;
    /**
     * @returns Whether a type is an instance of the parent type, including for the parent's base types.
     */
    function typeIsOrHasBaseType(type, parentType) {
        if (type.symbol === undefined || parentType.symbol === undefined) {
            return false;
        }
        const typeAndBaseTypes = [type];
        const ancestorTypes = type.getBaseTypes();
        if (ancestorTypes !== undefined) {
            typeAndBaseTypes.push(...ancestorTypes);
        }
        for (const baseType of typeAndBaseTypes) {
            if (baseType.symbol !== undefined &&
                baseType.symbol.name === parentType.symbol.name) {
                return true;
            }
        }
        return false;
    }
    exports.typeIsOrHasBaseType = typeIsOrHasBaseType;
    /**
     * Gets the source file for a given node
     */
    function getSourceFileOfNode(node) {
        while (node && node.kind !== ts.SyntaxKind.SourceFile) {
            node = node.parent;
        }
        return node;
    }
    exports.getSourceFileOfNode = getSourceFileOfNode;
    function getTokenAtPosition(sourceFile, position) {
        const queue = [sourceFile];
        let current;
        while (queue.length > 0) {
            current = queue.shift();
            // find the child that contains 'position'
            for (const child of current.getChildren(sourceFile)) {
                const start = child.getFullStart();
                if (start > position) {
                    // If this child begins after position, then all subsequent children will as well.
                    return current;
                }
                const end = child.getEnd();
                if (position < end ||
                    (position === end && child.kind === ts.SyntaxKind.EndOfFileToken)) {
                    queue.push(child);
                    break;
                }
            }
        }
        return current;
    }
    exports.getTokenAtPosition = getTokenAtPosition;
    function getEqualsKind(operator) {
        switch (operator) {
            case '==':
                return {
                    isPositive: true,
                    isStrict: false,
                };
            case '===':
                return {
                    isPositive: true,
                    isStrict: true,
                };
            case '!=':
                return {
                    isPositive: false,
                    isStrict: false,
                };
            case '!==':
                return {
                    isPositive: true,
                    isStrict: true,
                };
            default:
                return undefined;
        }
    }
    exports.getEqualsKind = getEqualsKind;
    /**
     * @returns true if the type is `any`
     */
    function isTypeAnyType(type) {
        return isTypeFlagSet(type, ts.TypeFlags.Any);
    }
    exports.isTypeAnyType = isTypeAnyType;
    /**
     * @returns `AnyType.Any` if the type is `any`, `AnyType.AnyArray` if the type is `any[]` or `readonly any[]`,
     *          otherwise it returns `AnyType.Safe`.
     */
    function isAnyOrAnyArrayTypeDiscriminated(node, checker) {
        const type = checker.getTypeAtLocation(node);
        if (isTypeAnyType(type)) {
            return 0 /* Any */;
        }
        if (checker.isArrayType(type) &&
            isTypeAnyType(checker.getTypeArguments(type)[0])) {
            return 1 /* AnyArray */;
        }
        return 2 /* Safe */;
    }
    exports.isAnyOrAnyArrayTypeDiscriminated = isAnyOrAnyArrayTypeDiscriminated;
    /**
     * Does a simple check to see if there is an any being assigned to a non-any type.
     *
     * This also checks generic positions to ensure there's no unsafe sub-assignments.
     * Note: in the case of generic positions, it makes the assumption that the two types are the same.
     *
     * @example See tests for examples
     *
     * @returns false if it's safe, or an object with the two types if it's unsafe
     */
    function isUnsafeAssignment(type, receiver, checker) {
        var _a, _b;
        if (tsutils_1.isTypeReference(type) && tsutils_1.isTypeReference(receiver)) {
            // TODO - figure out how to handle cases like this,
            // where the types are assignable, but not the same type
            /*
            function foo(): ReadonlySet<number> { return new Set<any>(); }
        
            // and
        
            type Test<T> = { prop: T }
            type Test2 = { prop: string }
            declare const a: Test<any>;
            const b: Test2 = a;
            */
            if (type.target !== receiver.target) {
                // if the type references are different, assume safe, as we won't know how to compare the two types
                // the generic positions might not be equivalent for both types
                return false;
            }
            const typeArguments = (_a = type.typeArguments) !== null && _a !== void 0 ? _a : [];
            const receiverTypeArguments = (_b = receiver.typeArguments) !== null && _b !== void 0 ? _b : [];
            for (let i = 0; i < typeArguments.length; i += 1) {
                const arg = typeArguments[i];
                const receiverArg = receiverTypeArguments[i];
                const unsafe = isUnsafeAssignment(arg, receiverArg, checker);
                if (unsafe) {
                    return { sender: type, receiver };
                }
            }
            return false;
        }
        if (isTypeAnyType(type) && !isTypeAnyType(receiver)) {
            return { sender: type, receiver };
        }
        return false;
    }
    exports.isUnsafeAssignment = isUnsafeAssignment;
    /**
     * Returns the contextual type of a given node.
     * Contextual type is the type of the target the node is going into.
     * i.e. the type of a called function's parameter, or the defined type of a variable declaration
     */
    function getContextualType(checker, node) {
        const parent = node.parent;
        if (!parent) {
            return;
        }
        if (tsutils_1.isCallExpression(parent) || tsutils_1.isNewExpression(parent)) {
            if (node === parent.expression) {
                // is the callee, so has no contextual type
                return;
            }
        }
        else if (tsutils_1.isVariableDeclaration(parent) ||
            tsutils_1.isPropertyDeclaration(parent) ||
            tsutils_1.isParameterDeclaration(parent)) {
            return parent.type ? checker.getTypeFromTypeNode(parent.type) : undefined;
        }
        else if (tsutils_1.isJsxExpression(parent)) {
            return checker.getContextualType(parent);
        }
        else if (![ts.SyntaxKind.TemplateSpan, ts.SyntaxKind.JsxExpression].includes(parent.kind)) {
            // parent is not something we know we can get the contextual type of
            return;
        }
        // TODO - support return statement checking
        return checker.getContextualType(node);
    }
    exports.getContextualType = getContextualType;
},
/* @typescript-eslint/eslint-plugin/dist/rules/array-type.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    /**
     * Check whatever node can be considered as simple
     * @param node the node to be evaluated.
     */
    function isSimpleType(node) {
        switch (node.type) {
            case experimental_utils_1.AST_NODE_TYPES.Identifier:
            case experimental_utils_1.AST_NODE_TYPES.TSAnyKeyword:
            case experimental_utils_1.AST_NODE_TYPES.TSBooleanKeyword:
            case experimental_utils_1.AST_NODE_TYPES.TSNeverKeyword:
            case experimental_utils_1.AST_NODE_TYPES.TSNumberKeyword:
            case experimental_utils_1.AST_NODE_TYPES.TSObjectKeyword:
            case experimental_utils_1.AST_NODE_TYPES.TSStringKeyword:
            case experimental_utils_1.AST_NODE_TYPES.TSSymbolKeyword:
            case experimental_utils_1.AST_NODE_TYPES.TSUnknownKeyword:
            case experimental_utils_1.AST_NODE_TYPES.TSVoidKeyword:
            case experimental_utils_1.AST_NODE_TYPES.TSNullKeyword:
            case experimental_utils_1.AST_NODE_TYPES.TSArrayType:
            case experimental_utils_1.AST_NODE_TYPES.TSUndefinedKeyword:
            case experimental_utils_1.AST_NODE_TYPES.TSThisType:
            case experimental_utils_1.AST_NODE_TYPES.TSQualifiedName:
                return true;
            case experimental_utils_1.AST_NODE_TYPES.TSTypeReference:
                if (node.typeName &&
                    node.typeName.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                    node.typeName.name === 'Array') {
                    if (!node.typeParameters) {
                        return true;
                    }
                    if (node.typeParameters.params.length === 1) {
                        return isSimpleType(node.typeParameters.params[0]);
                    }
                }
                else {
                    if (node.typeParameters) {
                        return false;
                    }
                    return isSimpleType(node.typeName);
                }
                return false;
            default:
                return false;
        }
    }
    /**
     * Check if node needs parentheses
     * @param node the node to be evaluated.
     */
    function typeNeedsParentheses(node) {
        switch (node.type) {
            case experimental_utils_1.AST_NODE_TYPES.TSTypeReference:
                return typeNeedsParentheses(node.typeName);
            case experimental_utils_1.AST_NODE_TYPES.TSUnionType:
            case experimental_utils_1.AST_NODE_TYPES.TSFunctionType:
            case experimental_utils_1.AST_NODE_TYPES.TSIntersectionType:
            case experimental_utils_1.AST_NODE_TYPES.TSTypeOperator:
            case experimental_utils_1.AST_NODE_TYPES.TSInferType:
                return true;
            case experimental_utils_1.AST_NODE_TYPES.Identifier:
                return node.name === 'ReadonlyArray';
            default:
                return false;
        }
    }
    const arrayOption = { enum: ['array', 'generic', 'array-simple'] };
    exports.default = util.createRule({
        name: 'array-type',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Requires using either `T[]` or `Array<T>` for arrays',
                category: 'Stylistic Issues',
                // too opinionated to be recommended
                recommended: false,
            },
            fixable: 'code',
            messages: {
                errorStringGeneric: "Array type using '{{type}}[]' is forbidden. Use 'Array<{{type}}>' instead.",
                errorStringGenericSimple: "Array type using '{{type}}[]' is forbidden for non-simple types. Use 'Array<{{type}}>' instead.",
                errorStringArray: "Array type using 'Array<{{type}}>' is forbidden. Use '{{type}}[]' instead.",
                errorStringArraySimple: "Array type using 'Array<{{type}}>' is forbidden for simple types. Use '{{type}}[]' instead.",
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        default: arrayOption,
                        readonly: arrayOption,
                    },
                },
            ],
        },
        defaultOptions: [
            {
                default: 'array',
            },
        ],
        create(context, [options]) {
            var _a;
            const sourceCode = context.getSourceCode();
            const defaultOption = options.default;
            const readonlyOption = (_a = options.readonly) !== null && _a !== void 0 ? _a : defaultOption;
            const isArraySimpleOption = defaultOption === 'array-simple' && readonlyOption === 'array-simple';
            const isArrayOption = defaultOption === 'array' && readonlyOption === 'array';
            const isGenericOption = defaultOption === 'generic' && readonlyOption === 'generic';
            /**
             * Check if whitespace is needed before this node
             * @param node the node to be evaluated.
             */
            function requireWhitespaceBefore(node) {
                const prevToken = sourceCode.getTokenBefore(node);
                if (!prevToken) {
                    return false;
                }
                const nextToken = sourceCode.getTokenAfter(prevToken);
                if (nextToken && sourceCode.isSpaceBetweenTokens(prevToken, nextToken)) {
                    return false;
                }
                return prevToken.type === experimental_utils_1.AST_TOKEN_TYPES.Identifier;
            }
            /**
             * @param node the node to be evaluated.
             */
            function getMessageType(node) {
                if (node) {
                    if (node.type === experimental_utils_1.AST_NODE_TYPES.TSParenthesizedType) {
                        return getMessageType(node.typeAnnotation);
                    }
                    if (isSimpleType(node)) {
                        return sourceCode.getText(node);
                    }
                }
                return 'T';
            }
            /**
             * @param node the node to be evaluated
             */
            function getTypeOpNodeRange(node) {
                if (!node) {
                    return undefined;
                }
                const firstToken = sourceCode.getFirstToken(node);
                const nextToken = sourceCode.getTokenAfter(firstToken);
                return [firstToken.range[0], nextToken.range[0]];
            }
            return {
                TSArrayType(node) {
                    if (isArrayOption ||
                        (isArraySimpleOption && isSimpleType(node.elementType))) {
                        return;
                    }
                    const isReadonly = node.parent &&
                        node.parent.type === experimental_utils_1.AST_NODE_TYPES.TSTypeOperator &&
                        node.parent.operator === 'readonly';
                    const isReadonlyGeneric = readonlyOption === 'generic' && defaultOption !== 'generic';
                    const isReadonlyArray = readonlyOption !== 'generic' && defaultOption === 'generic';
                    if ((isReadonlyGeneric && !isReadonly) ||
                        (isReadonlyArray && isReadonly)) {
                        return;
                    }
                    const messageId = defaultOption === 'generic'
                        ? 'errorStringGeneric'
                        : 'errorStringGenericSimple';
                    const typeOpNode = isReadonly ? node.parent : null;
                    context.report({
                        node: isReadonly ? node.parent : node,
                        messageId,
                        data: {
                            type: getMessageType(node.elementType),
                        },
                        fix(fixer) {
                            const toFix = [
                                fixer.replaceTextRange([node.range[1] - 2, node.range[1]], '>'),
                            ];
                            const startText = requireWhitespaceBefore(node);
                            const typeOpNodeRange = getTypeOpNodeRange(typeOpNode);
                            if (typeOpNodeRange) {
                                toFix.unshift(fixer.removeRange(typeOpNodeRange));
                            }
                            else {
                                toFix.push(fixer.insertTextBefore(node, `${startText ? ' ' : ''}`));
                            }
                            toFix.push(fixer.insertTextBefore(node, `${isReadonly ? 'Readonly' : ''}Array<`));
                            if (node.elementType.type === experimental_utils_1.AST_NODE_TYPES.TSParenthesizedType) {
                                const first = sourceCode.getFirstToken(node.elementType);
                                const last = sourceCode.getLastToken(node.elementType);
                                if (!first || !last) {
                                    return null;
                                }
                                toFix.push(fixer.remove(first));
                                toFix.push(fixer.remove(last));
                            }
                            return toFix;
                        },
                    });
                },
                TSTypeReference(node) {
                    if (isGenericOption ||
                        node.typeName.type !== experimental_utils_1.AST_NODE_TYPES.Identifier) {
                        return;
                    }
                    const isReadonlyArrayType = node.typeName.name === 'ReadonlyArray';
                    const isArrayType = node.typeName.name === 'Array';
                    if (!(isArrayType || isReadonlyArrayType) ||
                        (readonlyOption === 'generic' && isReadonlyArrayType) ||
                        (defaultOption === 'generic' && !isReadonlyArrayType)) {
                        return;
                    }
                    const readonlyPrefix = isReadonlyArrayType ? 'readonly ' : '';
                    const typeParams = node.typeParameters && node.typeParameters.params;
                    const messageId = defaultOption === 'array'
                        ? 'errorStringArray'
                        : 'errorStringArraySimple';
                    if (!typeParams || typeParams.length === 0) {
                        // Create an 'any' array
                        context.report({
                            node,
                            messageId,
                            data: {
                                type: 'any',
                            },
                            fix(fixer) {
                                return fixer.replaceText(node, `${readonlyPrefix}any[]`);
                            },
                        });
                        return;
                    }
                    if (typeParams.length !== 1 ||
                        (defaultOption === 'array-simple' && !isSimpleType(typeParams[0]))) {
                        return;
                    }
                    const type = typeParams[0];
                    const parens = typeNeedsParentheses(type);
                    context.report({
                        node,
                        messageId,
                        data: {
                            type: getMessageType(type),
                        },
                        fix(fixer) {
                            return [
                                fixer.replaceTextRange([node.range[0], type.range[0]], `${readonlyPrefix}${parens ? '(' : ''}`),
                                fixer.replaceTextRange([type.range[1], node.range[1]], parens ? ')[]' : '[]'),
                            ];
                        },
                    });
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/await-thenable.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const tsutils = __importStar(require(627) /* tsutils */);
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'await-thenable',
        meta: {
            docs: {
                description: 'Disallows awaiting a value that is not a Thenable',
                category: 'Best Practices',
                recommended: 'error',
                requiresTypeChecking: true,
            },
            messages: {
                await: 'Unexpected `await` of a non-Promise (non-"Thenable") value.',
            },
            schema: [],
            type: 'problem',
        },
        defaultOptions: [],
        create(context) {
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            return {
                AwaitExpression(node) {
                    const originalNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                    const type = checker.getTypeAtLocation(originalNode.expression);
                    if (!tsutils.isTypeFlagSet(type, ts.TypeFlags.Any) &&
                        !tsutils.isTypeFlagSet(type, ts.TypeFlags.Unknown) &&
                        !tsutils.isThenableType(checker, originalNode.expression, type)) {
                        context.report({
                            messageId: 'await',
                            node,
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/ban-ts-comment.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    const defaultOptions = [
        {
            'ts-ignore': true,
            'ts-nocheck': true,
            'ts-check': false,
        },
    ];
    exports.default = util.createRule({
        name: 'ban-ts-comment',
        meta: {
            type: 'problem',
            docs: {
                description: 'Bans `// @ts-<directive>` comments from being used',
                category: 'Best Practices',
                recommended: false,
            },
            messages: {
                tsDirectiveComment: 'Do not use "// @ts-{{directive}}" because it alters compilation errors.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        'ts-ignore': {
                            type: 'boolean',
                            default: true,
                        },
                        'ts-nocheck': {
                            type: 'boolean',
                            default: true,
                        },
                        'ts-check': {
                            type: 'boolean',
                            default: false,
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions,
        create(context, [options]) {
            const tsCommentRegExp = /^\/*\s*@ts-(ignore|check|nocheck)/;
            const sourceCode = context.getSourceCode();
            return {
                Program() {
                    const comments = sourceCode.getAllComments();
                    comments.forEach(comment => {
                        var _a;
                        if (comment.type !== experimental_utils_1.AST_TOKEN_TYPES.Line) {
                            return;
                        }
                        const [, directive] = (_a = tsCommentRegExp.exec(comment.value)) !== null && _a !== void 0 ? _a : [];
                        const fullDirective = `ts-${directive}`;
                        if (options[fullDirective]) {
                            context.report({
                                data: { directive },
                                node: comment,
                                messageId: 'tsDirectiveComment',
                            });
                        }
                    });
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/ban-ts-ignore.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'ban-ts-ignore',
        meta: {
            type: 'problem',
            docs: {
                description: 'Bans “// @ts-ignore” comments from being used',
                category: 'Best Practices',
                recommended: 'error',
            },
            schema: [],
            messages: {
                tsIgnoreComment: 'Do not use "// @ts-ignore" comments because they suppress compilation errors.',
            },
            deprecated: true,
            replacedBy: ['@typescript-eslint/ban-ts-comment'],
        },
        defaultOptions: [],
        create(context) {
            const tsIgnoreRegExp = /^\/*\s*@ts-ignore/;
            const sourceCode = context.getSourceCode();
            return {
                Program() {
                    const comments = sourceCode.getAllComments();
                    comments.forEach(comment => {
                        if (comment.type !== experimental_utils_1.AST_TOKEN_TYPES.Line) {
                            return;
                        }
                        if (tsIgnoreRegExp.test(comment.value)) {
                            context.report({
                                node: comment,
                                messageId: 'tsIgnoreComment',
                            });
                        }
                    });
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/ban-types.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    function removeSpaces(str) {
        return str.replace(/ /g, '');
    }
    function stringifyTypeName(node, sourceCode) {
        return removeSpaces(sourceCode.getText(node));
    }
    function getCustomMessage(bannedType) {
        if (bannedType === null) {
            return '';
        }
        if (typeof bannedType === 'string') {
            return ` ${bannedType}`;
        }
        if (bannedType.message) {
            return ` ${bannedType.message}`;
        }
        return '';
    }
    /*
      Defaults for this rule should be treated as an "all or nothing"
      merge, so we need special handling here.
    
      See: https://github.com/typescript-eslint/typescript-eslint/issues/686
     */
    const defaultTypes = {
        String: {
            message: 'Use string instead',
            fixWith: 'string',
        },
        Boolean: {
            message: 'Use boolean instead',
            fixWith: 'boolean',
        },
        Number: {
            message: 'Use number instead',
            fixWith: 'number',
        },
        Object: {
            message: 'Use Record<string, any> instead',
            fixWith: 'Record<string, any>',
        },
        Symbol: {
            message: 'Use symbol instead',
            fixWith: 'symbol',
        },
    };
    exports.default = util.createRule({
        name: 'ban-types',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Bans specific types from being used',
                category: 'Best Practices',
                recommended: 'error',
            },
            fixable: 'code',
            messages: {
                bannedTypeMessage: "Don't use '{{name}}' as a type.{{customMessage}}",
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        types: {
                            type: 'object',
                            additionalProperties: {
                                oneOf: [
                                    { type: 'null' },
                                    { type: 'string' },
                                    {
                                        type: 'object',
                                        properties: {
                                            message: { type: 'string' },
                                            fixWith: { type: 'string' },
                                        },
                                        additionalProperties: false,
                                    },
                                ],
                            },
                        },
                        extendDefaults: {
                            type: 'boolean',
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [{}],
        create(context, [options]) {
            var _a, _b;
            const extendDefaults = (_a = options.extendDefaults) !== null && _a !== void 0 ? _a : true;
            const customTypes = (_b = options.types) !== null && _b !== void 0 ? _b : {};
            const types = Object.assign(Object.assign({}, (extendDefaults ? defaultTypes : {})), customTypes);
            const bannedTypes = new Map(Object.entries(types).map(([type, data]) => [removeSpaces(type), data]));
            function checkBannedTypes(typeNode, name = stringifyTypeName(typeNode, context.getSourceCode())) {
                const bannedType = bannedTypes.get(name);
                if (bannedType !== undefined) {
                    const customMessage = getCustomMessage(bannedType);
                    const fixWith = bannedType && typeof bannedType === 'object' && bannedType.fixWith;
                    context.report({
                        node: typeNode,
                        messageId: 'bannedTypeMessage',
                        data: {
                            name,
                            customMessage,
                        },
                        fix: fixWith
                            ? (fixer) => fixer.replaceText(typeNode, fixWith)
                            : null,
                    });
                }
            }
            return Object.assign(Object.assign(Object.assign({}, (bannedTypes.has('null') && {
                TSNullKeyword(node) {
                    checkBannedTypes(node, 'null');
                },
            })), (bannedTypes.has('undefined') && {
                TSUndefinedKeyword(node) {
                    checkBannedTypes(node, 'undefined');
                },
            })), { TSTypeLiteral(node) {
                    if (node.members.length) {
                        return;
                    }
                    checkBannedTypes(node);
                },
                TSTypeReference({ typeName }) {
                    checkBannedTypes(typeName);
                } });
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/brace-style.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const brace_style_1 = __importDefault(require(140) /* eslint/lib/rules/brace-style */);
    const util_1 = require(622) /* ../util */;
    exports.default = util_1.createRule({
        name: 'brace-style',
        meta: {
            type: 'layout',
            docs: {
                description: 'Enforce consistent brace style for blocks',
                category: 'Stylistic Issues',
                recommended: false,
                extendsBaseRule: true,
            },
            messages: brace_style_1.default.meta.messages,
            fixable: brace_style_1.default.meta.fixable,
            schema: brace_style_1.default.meta.schema,
        },
        defaultOptions: ['1tbs'],
        create(context) {
            const [style, { allowSingleLine } = { allowSingleLine: false },] = context.options;
            const isAllmanStyle = style === 'allman';
            const sourceCode = context.getSourceCode();
            const rules = brace_style_1.default.create(context);
            /**
             * Checks a pair of curly brackets based on the user's config
             */
            function validateCurlyPair(openingCurlyToken, closingCurlyToken) {
                if (allowSingleLine &&
                    util_1.isTokenOnSameLine(openingCurlyToken, closingCurlyToken)) {
                    return;
                }
                const tokenBeforeOpeningCurly = sourceCode.getTokenBefore(openingCurlyToken);
                const tokenBeforeClosingCurly = sourceCode.getTokenBefore(closingCurlyToken);
                const tokenAfterOpeningCurly = sourceCode.getTokenAfter(openingCurlyToken);
                if (!isAllmanStyle &&
                    !util_1.isTokenOnSameLine(tokenBeforeOpeningCurly, openingCurlyToken)) {
                    context.report({
                        node: openingCurlyToken,
                        messageId: 'nextLineOpen',
                        fix: fixer => {
                            const textRange = [
                                tokenBeforeOpeningCurly.range[1],
                                openingCurlyToken.range[0],
                            ];
                            const textBetween = sourceCode.text.slice(textRange[0], textRange[1]);
                            if (textBetween.trim()) {
                                return null;
                            }
                            return fixer.replaceTextRange(textRange, ' ');
                        },
                    });
                }
                if (isAllmanStyle &&
                    util_1.isTokenOnSameLine(tokenBeforeOpeningCurly, openingCurlyToken)) {
                    context.report({
                        node: openingCurlyToken,
                        messageId: 'sameLineOpen',
                        fix: fixer => fixer.insertTextBefore(openingCurlyToken, '\n'),
                    });
                }
                if (util_1.isTokenOnSameLine(openingCurlyToken, tokenAfterOpeningCurly) &&
                    tokenAfterOpeningCurly !== closingCurlyToken) {
                    context.report({
                        node: openingCurlyToken,
                        messageId: 'blockSameLine',
                        fix: fixer => fixer.insertTextAfter(openingCurlyToken, '\n'),
                    });
                }
                if (util_1.isTokenOnSameLine(tokenBeforeClosingCurly, closingCurlyToken) &&
                    tokenBeforeClosingCurly !== openingCurlyToken) {
                    context.report({
                        node: closingCurlyToken,
                        messageId: 'singleLineClose',
                        fix: fixer => fixer.insertTextBefore(closingCurlyToken, '\n'),
                    });
                }
            }
            return Object.assign(Object.assign({}, rules), { 'TSInterfaceBody, TSModuleBlock'(node) {
                    const openingCurly = sourceCode.getFirstToken(node);
                    const closingCurly = sourceCode.getLastToken(node);
                    validateCurlyPair(openingCurly, closingCurly);
                },
                TSEnumDeclaration(node) {
                    const closingCurly = sourceCode.getLastToken(node);
                    const openingCurly = sourceCode.getTokenBefore(node.members.length ? node.members[0] : closingCurly);
                    validateCurlyPair(openingCurly, closingCurly);
                } });
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/camelcase.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const camelcase_1 = __importDefault(require(142) /* eslint/lib/rules/camelcase */);
    const util = __importStar(require(622) /* ../util */);
    const schema = util.deepMerge(Array.isArray(camelcase_1.default.meta.schema)
        ? camelcase_1.default.meta.schema[0]
        : camelcase_1.default.meta.schema, {
        properties: {
            genericType: {
                enum: ['always', 'never'],
            },
        },
    });
    exports.default = util.createRule({
        name: 'camelcase',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Enforce camelCase naming convention',
                category: 'Stylistic Issues',
                recommended: 'error',
                extendsBaseRule: true,
            },
            deprecated: true,
            replacedBy: ['naming-convention'],
            schema: [schema],
            messages: camelcase_1.default.meta.messages,
        },
        defaultOptions: [
            {
                allow: ['^UNSAFE_'],
                ignoreDestructuring: false,
                properties: 'never',
                genericType: 'never',
            },
        ],
        create(context, [options]) {
            var _a, _b;
            const rules = camelcase_1.default.create(context);
            const TS_PROPERTY_TYPES = [
                experimental_utils_1.AST_NODE_TYPES.TSPropertySignature,
                experimental_utils_1.AST_NODE_TYPES.ClassProperty,
                experimental_utils_1.AST_NODE_TYPES.TSParameterProperty,
                experimental_utils_1.AST_NODE_TYPES.TSAbstractClassProperty,
            ];
            const genericType = options.genericType;
            const properties = options.properties;
            const allow = (_b = (_a = options.allow) === null || _a === void 0 ? void 0 : _a.map(entry => ({
                name: entry,
                regex: new RegExp(entry),
            }))) !== null && _b !== void 0 ? _b : [];
            /**
             * Checks if a string contains an underscore and isn't all upper-case
             * @param  name The string to check.
             */
            function isUnderscored(name) {
                // if there's an underscore, it might be A_CONSTANT, which is okay
                return name.includes('_') && name !== name.toUpperCase();
            }
            /**
             * Checks if a string match the ignore list
             * @param name The string to check.
             * @returns if the string is ignored
             * @private
             */
            function isAllowed(name) {
                return (allow.findIndex(entry => name === entry.name || entry.regex.test(name)) !== -1);
            }
            /**
             * Checks if the the node is a valid TypeScript property type.
             * @param node the node to be validated.
             * @returns true if the node is a TypeScript property type.
             * @private
             */
            function isTSPropertyType(node) {
                if (TS_PROPERTY_TYPES.includes(node.type)) {
                    return true;
                }
                if (node.type === experimental_utils_1.AST_NODE_TYPES.AssignmentPattern) {
                    return (node.parent !== undefined &&
                        TS_PROPERTY_TYPES.includes(node.parent.type));
                }
                return false;
            }
            function report(node) {
                context.report({
                    node,
                    messageId: 'notCamelCase',
                    data: { name: node.name },
                });
            }
            return {
                Identifier(node) {
                    /*
                     * Leading and trailing underscores are commonly used to flag
                     * private/protected identifiers, strip them
                     */
                    const name = node.name.replace(/^_+|_+$/g, '');
                    // First, we ignore the node if it match the ignore list
                    if (isAllowed(name)) {
                        return;
                    }
                    // Check TypeScript specific nodes
                    const parent = node.parent;
                    if (parent && isTSPropertyType(parent)) {
                        if (properties === 'always' && isUnderscored(name)) {
                            report(node);
                        }
                        return;
                    }
                    if (parent && parent.type === experimental_utils_1.AST_NODE_TYPES.TSTypeParameter) {
                        if (genericType === 'always' && isUnderscored(name)) {
                            report(node);
                        }
                        return;
                    }
                    if (parent && parent.type === experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression) {
                        // Report underscored object names
                        if (properties === 'always' &&
                            parent.object.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                            parent.object.name === node.name &&
                            isUnderscored(name)) {
                            report(node);
                        }
                        return;
                    }
                    // Let the base rule deal with the rest
                    rules.Identifier(node);
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/class-name-casing.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'class-name-casing',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Require PascalCased class and interface names',
                category: 'Best Practices',
                recommended: 'error',
            },
            deprecated: true,
            replacedBy: ['naming-convention'],
            messages: {
                notPascalCased: "{{friendlyName}} '{{name}}' must be PascalCased.",
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        allowUnderscorePrefix: {
                            type: 'boolean',
                            default: false,
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [{ allowUnderscorePrefix: false }],
        create(context, [options]) {
            const UNDERSCORE = '_';
            /**
             * Determine if the string is Upper cased
             * @param str
             */
            function isUpperCase(str) {
                return str === str.toUpperCase();
            }
            /**
             * Determine if the identifier name is PascalCased
             * @param name The identifier name
             */
            function isPascalCase(name) {
                const startIndex = options.allowUnderscorePrefix && name.startsWith(UNDERSCORE) ? 1 : 0;
                return (isUpperCase(name.charAt(startIndex)) &&
                    !name.includes(UNDERSCORE, startIndex));
            }
            /**
             * Report a class declaration as invalid
             * @param decl The declaration
             * @param id The name of the declaration
             */
            function report(decl, id) {
                let friendlyName;
                switch (decl.type) {
                    case experimental_utils_1.AST_NODE_TYPES.ClassDeclaration:
                    case experimental_utils_1.AST_NODE_TYPES.ClassExpression:
                        friendlyName = decl.abstract ? 'Abstract class' : 'Class';
                        break;
                    case experimental_utils_1.AST_NODE_TYPES.TSInterfaceDeclaration:
                        friendlyName = 'Interface';
                        break;
                }
                context.report({
                    node: id,
                    messageId: 'notPascalCased',
                    data: {
                        friendlyName,
                        name: id.name,
                    },
                });
            }
            return {
                'ClassDeclaration, TSInterfaceDeclaration, ClassExpression'(node) {
                    // class expressions (i.e. export default class {}) are OK
                    if (node.id && !isPascalCase(node.id.name)) {
                        report(node, node.id);
                    }
                },
                "VariableDeclarator[init.type='ClassExpression']"(node) {
                    if (node.id.type === experimental_utils_1.AST_NODE_TYPES.ArrayPattern ||
                        node.id.type === experimental_utils_1.AST_NODE_TYPES.ObjectPattern) {
                        // TODO - handle the BindingPattern case maybe?
                        /*
                        // this example makes me barf, but it's valid code
                        var { bar } = class {
                          static bar() { return 2 }
                        }
                        */
                    }
                    else {
                        const id = node.id;
                        const nodeInit = node.init;
                        if (id && !nodeInit.id && !isPascalCase(id.name)) {
                            report(nodeInit, id);
                        }
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/comma-spacing.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const eslint_utils_1 = require(250) /* eslint-utils */;
    const util_1 = require(622) /* ../util */;
    exports.default = util_1.createRule({
        name: 'comma-spacing',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Enforces consistent spacing before and after commas',
                category: 'Stylistic Issues',
                recommended: false,
                extendsBaseRule: true,
            },
            fixable: 'whitespace',
            schema: [
                {
                    type: 'object',
                    properties: {
                        before: {
                            type: 'boolean',
                            default: false,
                        },
                        after: {
                            type: 'boolean',
                            default: true,
                        },
                    },
                    additionalProperties: false,
                },
            ],
            messages: {
                unexpected: `There should be no space {{loc}} ','.`,
                missing: `A space is required {{loc}} ','.`,
            },
        },
        defaultOptions: [
            {
                before: false,
                after: true,
            },
        ],
        create(context, [{ before: spaceBefore, after: spaceAfter }]) {
            const sourceCode = context.getSourceCode();
            const tokensAndComments = sourceCode.tokensAndComments;
            const ignoredTokens = new Set();
            /**
             * Adds null elements of the ArrayExpression or ArrayPattern node to the ignore list
             * @param node node to evaluate
             */
            function addNullElementsToIgnoreList(node) {
                let previousToken = sourceCode.getFirstToken(node);
                for (const element of node.elements) {
                    let token;
                    if (element === null) {
                        token = sourceCode.getTokenAfter(previousToken);
                        if (token && eslint_utils_1.isCommaToken(token)) {
                            ignoredTokens.add(token);
                        }
                    }
                    else {
                        token = sourceCode.getTokenAfter(element);
                    }
                    previousToken = token;
                }
            }
            /**
             * Adds type parameters trailing comma token to the ignore list
             * @param node node to evaluate
             */
            function addTypeParametersTrailingCommaToIgnoreList(node) {
                const param = node.params[node.params.length - 1];
                const afterToken = sourceCode.getTokenAfter(param);
                if (afterToken && eslint_utils_1.isCommaToken(afterToken)) {
                    ignoredTokens.add(afterToken);
                }
            }
            /**
             * Validates the spacing around a comma token.
             * @param commaToken The token representing the comma
             * @param prevToken The last token before the comma
             * @param nextToken The first token after the comma
             */
            function validateCommaSpacing(commaToken, prevToken, nextToken) {
                if (prevToken &&
                    util_1.isTokenOnSameLine(prevToken, commaToken) &&
                    spaceBefore !== sourceCode.isSpaceBetween(prevToken, commaToken)) {
                    context.report({
                        node: commaToken,
                        data: {
                            loc: 'before',
                        },
                        messageId: spaceBefore ? 'missing' : 'unexpected',
                        fix: fixer => spaceBefore
                            ? fixer.insertTextBefore(commaToken, ' ')
                            : fixer.replaceTextRange([prevToken.range[1], commaToken.range[0]], ''),
                    });
                }
                if (nextToken && eslint_utils_1.isClosingParenToken(nextToken)) {
                    return;
                }
                if (!spaceAfter && nextToken && nextToken.type === experimental_utils_1.AST_TOKEN_TYPES.Line) {
                    return;
                }
                if (nextToken &&
                    util_1.isTokenOnSameLine(commaToken, nextToken) &&
                    spaceAfter !== sourceCode.isSpaceBetween(commaToken, nextToken)) {
                    context.report({
                        node: commaToken,
                        data: {
                            loc: 'after',
                        },
                        messageId: spaceAfter ? 'missing' : 'unexpected',
                        fix: fixer => spaceAfter
                            ? fixer.insertTextAfter(commaToken, ' ')
                            : fixer.replaceTextRange([commaToken.range[1], nextToken.range[0]], ''),
                    });
                }
            }
            return {
                TSTypeParameterDeclaration: addTypeParametersTrailingCommaToIgnoreList,
                ArrayExpression: addNullElementsToIgnoreList,
                ArrayPattern: addNullElementsToIgnoreList,
                'Program:exit'() {
                    tokensAndComments.forEach((token, i) => {
                        if (!eslint_utils_1.isCommaToken(token)) {
                            return;
                        }
                        const prevToken = tokensAndComments[i - 1];
                        const nextToken = tokensAndComments[i + 1];
                        validateCommaSpacing(token, eslint_utils_1.isCommaToken(prevToken) || ignoredTokens.has(token)
                            ? null
                            : prevToken, eslint_utils_1.isCommaToken(nextToken) || ignoredTokens.has(token)
                            ? null
                            : nextToken);
                    });
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/consistent-type-assertions.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    exports.default = util.createRule({
        name: 'consistent-type-assertions',
        meta: {
            type: 'suggestion',
            docs: {
                category: 'Best Practices',
                description: 'Enforces consistent usage of type assertions',
                recommended: 'error',
            },
            messages: {
                as: "Use 'as {{cast}}' instead of '<{{cast}}>'.",
                'angle-bracket': "Use '<{{cast}}>' instead of 'as {{cast}}'.",
                never: 'Do not use any type assertions.',
                unexpectedObjectTypeAssertion: 'Always prefer const x: T = { ... }.',
            },
            schema: [
                {
                    oneOf: [
                        {
                            type: 'object',
                            properties: {
                                assertionStyle: {
                                    enum: ['never'],
                                },
                            },
                            additionalProperties: false,
                            required: ['assertionStyle'],
                        },
                        {
                            type: 'object',
                            properties: {
                                assertionStyle: {
                                    enum: ['as', 'angle-bracket'],
                                },
                                objectLiteralTypeAssertions: {
                                    enum: ['allow', 'allow-as-parameter', 'never'],
                                },
                            },
                            additionalProperties: false,
                            required: ['assertionStyle'],
                        },
                    ],
                },
            ],
        },
        defaultOptions: [
            {
                assertionStyle: 'as',
                objectLiteralTypeAssertions: 'allow',
            },
        ],
        create(context, [options]) {
            const sourceCode = context.getSourceCode();
            function reportIncorrectAssertionType(node) {
                const messageId = options.assertionStyle;
                context.report({
                    node,
                    messageId,
                    data: messageId !== 'never'
                        ? { cast: sourceCode.getText(node.typeAnnotation) }
                        : {},
                });
            }
            function checkType(node) {
                switch (node.type) {
                    case experimental_utils_1.AST_NODE_TYPES.TSAnyKeyword:
                    case experimental_utils_1.AST_NODE_TYPES.TSUnknownKeyword:
                        return false;
                    case experimental_utils_1.AST_NODE_TYPES.TSTypeReference:
                        return (
                        // Ignore `as const` and `<const>`
                        (node.typeName.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                            node.typeName.name !== 'const') ||
                            // Allow qualified names which have dots between identifiers, `Foo.Bar`
                            node.typeName.type === experimental_utils_1.AST_NODE_TYPES.TSQualifiedName);
                    default:
                        return true;
                }
            }
            function checkExpression(node) {
                if (options.assertionStyle === 'never' ||
                    options.objectLiteralTypeAssertions === 'allow' ||
                    node.expression.type !== experimental_utils_1.AST_NODE_TYPES.ObjectExpression) {
                    return;
                }
                if (options.objectLiteralTypeAssertions === 'allow-as-parameter' &&
                    node.parent &&
                    (node.parent.type === experimental_utils_1.AST_NODE_TYPES.NewExpression ||
                        node.parent.type === experimental_utils_1.AST_NODE_TYPES.CallExpression ||
                        node.parent.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression ||
                        node.parent.type === experimental_utils_1.AST_NODE_TYPES.ThrowStatement ||
                        node.parent.type === experimental_utils_1.AST_NODE_TYPES.AssignmentPattern)) {
                    return;
                }
                if (checkType(node.typeAnnotation) &&
                    node.expression.type === experimental_utils_1.AST_NODE_TYPES.ObjectExpression) {
                    context.report({
                        node,
                        messageId: 'unexpectedObjectTypeAssertion',
                    });
                }
            }
            return {
                TSTypeAssertion(node) {
                    if (options.assertionStyle !== 'angle-bracket') {
                        reportIncorrectAssertionType(node);
                        return;
                    }
                    checkExpression(node);
                },
                TSAsExpression(node) {
                    if (options.assertionStyle !== 'as') {
                        reportIncorrectAssertionType(node);
                        return;
                    }
                    checkExpression(node);
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/consistent-type-definitions.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'consistent-type-definitions',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Consistent with type definition either `interface` or `type`',
                category: 'Stylistic Issues',
                // too opinionated to be recommended
                recommended: false,
            },
            messages: {
                interfaceOverType: 'Use an `interface` instead of a `type`',
                typeOverInterface: 'Use a `type` instead of an `interface`',
            },
            schema: [
                {
                    enum: ['interface', 'type'],
                },
            ],
            fixable: 'code',
        },
        defaultOptions: ['interface'],
        create(context, [option]) {
            const sourceCode = context.getSourceCode();
            return {
                "TSTypeAliasDeclaration[typeAnnotation.type='TSTypeLiteral']"(node) {
                    if (option === 'interface') {
                        context.report({
                            node: node.id,
                            messageId: 'interfaceOverType',
                            fix(fixer) {
                                var _a;
                                const typeNode = (_a = node.typeParameters) !== null && _a !== void 0 ? _a : node.id;
                                const fixes = [];
                                const firstToken = sourceCode.getFirstToken(node);
                                if (firstToken) {
                                    fixes.push(fixer.replaceText(firstToken, 'interface'));
                                    fixes.push(fixer.replaceTextRange([typeNode.range[1], node.typeAnnotation.range[0]], ' '));
                                }
                                const afterToken = sourceCode.getTokenAfter(node.typeAnnotation);
                                if (afterToken &&
                                    afterToken.type === experimental_utils_1.AST_TOKEN_TYPES.Punctuator &&
                                    afterToken.value === ';') {
                                    fixes.push(fixer.remove(afterToken));
                                }
                                return fixes;
                            },
                        });
                    }
                },
                TSInterfaceDeclaration(node) {
                    if (option === 'type') {
                        context.report({
                            node: node.id,
                            messageId: 'typeOverInterface',
                            fix(fixer) {
                                var _a;
                                const typeNode = (_a = node.typeParameters) !== null && _a !== void 0 ? _a : node.id;
                                const fixes = [];
                                const firstToken = sourceCode.getFirstToken(node);
                                if (firstToken) {
                                    fixes.push(fixer.replaceText(firstToken, 'type'));
                                    fixes.push(fixer.replaceTextRange([typeNode.range[1], node.body.range[0]], ' = '));
                                }
                                if (node.extends) {
                                    node.extends.forEach(heritage => {
                                        const typeIdentifier = sourceCode.getText(heritage);
                                        fixes.push(fixer.insertTextAfter(node.body, ` & ${typeIdentifier}`));
                                    });
                                }
                                return fixes;
                            },
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/default-param-last.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const util_1 = require(622) /* ../util */;
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    exports.default = util_1.createRule({
        name: 'default-param-last',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Enforce default parameters to be last',
                category: 'Best Practices',
                recommended: false,
                extendsBaseRule: true,
            },
            schema: [],
            messages: {
                shouldBeLast: 'Default parameters should be last.',
            },
        },
        defaultOptions: [],
        create(context) {
            /**
             * checks if node is optional parameter
             * @param node the node to be evaluated
             * @private
             */
            function isOptionalParam(node) {
                return 'optional' in node && node.optional === true;
            }
            /**
             * checks if node is plain parameter
             * @param node the node to be evaluated
             * @private
             */
            function isPlainParam(node) {
                return !(node.type === experimental_utils_1.AST_NODE_TYPES.AssignmentPattern ||
                    node.type === experimental_utils_1.AST_NODE_TYPES.RestElement ||
                    isOptionalParam(node));
            }
            function checkDefaultParamLast(node) {
                let hasSeenPlainParam = false;
                for (let i = node.params.length - 1; i >= 0; i--) {
                    const current = node.params[i];
                    const param = current.type === experimental_utils_1.AST_NODE_TYPES.TSParameterProperty
                        ? current.parameter
                        : current;
                    if (isPlainParam(param)) {
                        hasSeenPlainParam = true;
                        continue;
                    }
                    if (hasSeenPlainParam &&
                        (isOptionalParam(param) ||
                            param.type === experimental_utils_1.AST_NODE_TYPES.AssignmentPattern)) {
                        context.report({ node: current, messageId: 'shouldBeLast' });
                    }
                }
            }
            return {
                ArrowFunctionExpression: checkDefaultParamLast,
                FunctionDeclaration: checkDefaultParamLast,
                FunctionExpression: checkDefaultParamLast,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/explicit-function-return-type.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    const explicitReturnTypeUtils_1 = require(663) /* ../util/explicitReturnTypeUtils */;
    exports.default = util.createRule({
        name: 'explicit-function-return-type',
        meta: {
            type: 'problem',
            docs: {
                description: 'Require explicit return types on functions and class methods',
                category: 'Stylistic Issues',
                recommended: 'warn',
            },
            messages: {
                missingReturnType: 'Missing return type on function.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        allowExpressions: {
                            type: 'boolean',
                        },
                        allowTypedFunctionExpressions: {
                            type: 'boolean',
                        },
                        allowHigherOrderFunctions: {
                            type: 'boolean',
                        },
                        allowDirectConstAssertionInArrowFunctions: {
                            type: 'boolean',
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            {
                allowExpressions: false,
                allowTypedFunctionExpressions: true,
                allowHigherOrderFunctions: true,
                allowDirectConstAssertionInArrowFunctions: true,
            },
        ],
        create(context, [options]) {
            const sourceCode = context.getSourceCode();
            return {
                'ArrowFunctionExpression, FunctionExpression'(node) {
                    explicitReturnTypeUtils_1.checkFunctionExpressionReturnType(node, options, sourceCode, loc => context.report({
                        node,
                        loc,
                        messageId: 'missingReturnType',
                    }));
                },
                FunctionDeclaration(node) {
                    explicitReturnTypeUtils_1.checkFunctionReturnType(node, options, sourceCode, loc => context.report({
                        node,
                        loc,
                        messageId: 'missingReturnType',
                    }));
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/util/explicitReturnTypeUtils.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const astUtils_1 = require(623) /* ./astUtils */;
    const nullThrows_1 = require(648) /* ./nullThrows */;
    /**
     * Creates a report location for the given function.
     * The location only encompasses the "start" of the function, and not the body
     *
     * eg.
     * function foo(args) {}
     * ^^^^^^^^^^^^^^^^^^
     *
     * get y(args) {}
     * ^^^^^^^^^^^
     *
     * const x = (args) => {}
     *           ^^^^^^^^^
     */
    function getReporLoc(node, sourceCode) {
        /**
         * Returns start column position
         * @param node
         */
        function getLocStart() {
            /* highlight method name */
            const parent = node.parent;
            if (parent &&
                (parent.type === experimental_utils_1.AST_NODE_TYPES.MethodDefinition ||
                    (parent.type === experimental_utils_1.AST_NODE_TYPES.Property && parent.method))) {
                return parent.loc.start;
            }
            return node.loc.start;
        }
        /**
         * Returns end column position
         * @param node
         */
        function getLocEnd() {
            /* highlight `=>` */
            if (node.type === experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression) {
                return sourceCode.getTokenBefore(node.body, token => token.type === experimental_utils_1.AST_TOKEN_TYPES.Punctuator && token.value === '=>').loc.end;
            }
            return sourceCode.getTokenBefore(node.body).loc.end;
        }
        return {
            start: getLocStart(),
            end: getLocEnd(),
        };
    }
    /**
     * Checks if a node is a variable declarator with a type annotation.
     * ```
     * const x: Foo = ...
     * ```
     */
    function isVariableDeclaratorWithTypeAnnotation(node) {
        return (node.type === experimental_utils_1.AST_NODE_TYPES.VariableDeclarator && !!node.id.typeAnnotation);
    }
    /**
     * Checks if a node is a class property with a type annotation.
     * ```
     * public x: Foo = ...
     * ```
     */
    function isClassPropertyWithTypeAnnotation(node) {
        return node.type === experimental_utils_1.AST_NODE_TYPES.ClassProperty && !!node.typeAnnotation;
    }
    /**
     * Checks if a node belongs to:
     * ```
     * new Foo(() => {})
     *         ^^^^^^^^
     * ```
     */
    function isConstructorArgument(node) {
        return node.type === experimental_utils_1.AST_NODE_TYPES.NewExpression;
    }
    /**
     * Checks if a node belongs to:
     * ```
     * const x: Foo = { prop: () => {} }
     * const x = { prop: () => {} } as Foo
     * const x = <Foo>{ prop: () => {} }
     * ```
     */
    function isPropertyOfObjectWithType(property) {
        if (!property || property.type !== experimental_utils_1.AST_NODE_TYPES.Property) {
            return false;
        }
        const objectExpr = property.parent; // this shouldn't happen, checking just in case
        /* istanbul ignore if */ if (!objectExpr ||
            objectExpr.type !== experimental_utils_1.AST_NODE_TYPES.ObjectExpression) {
            return false;
        }
        const parent = objectExpr.parent; // this shouldn't happen, checking just in case
        /* istanbul ignore if */ if (!parent) {
            return false;
        }
        return (astUtils_1.isTypeAssertion(parent) ||
            isClassPropertyWithTypeAnnotation(parent) ||
            isVariableDeclaratorWithTypeAnnotation(parent) ||
            isFunctionArgument(parent));
    }
    /**
     * Checks if a function belongs to:
     * ```
     * () => () => ...
     * () => function () { ... }
     * () => { return () => ... }
     * () => { return function () { ... } }
     * function fn() { return () => ... }
     * function fn() { return function() { ... } }
     * ```
     */
    function doesImmediatelyReturnFunctionExpression({ body, }) {
        // Should always have a body; really checking just in case
        /* istanbul ignore if */ if (!body) {
            return false;
        }
        // Check if body is a block with a single statement
        if (body.type === experimental_utils_1.AST_NODE_TYPES.BlockStatement && body.body.length === 1) {
            const [statement] = body.body;
            // Check if that statement is a return statement with an argument
            if (statement.type === experimental_utils_1.AST_NODE_TYPES.ReturnStatement &&
                !!statement.argument) {
                // If so, check that returned argument as body
                body = statement.argument;
            }
        }
        // Check if the body being returned is a function expression
        return (body.type === experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression ||
            body.type === experimental_utils_1.AST_NODE_TYPES.FunctionExpression);
    }
    /**
     * Checks if a node belongs to:
     * ```
     * foo(() => 1)
     * ```
     */
    function isFunctionArgument(parent, callee) {
        return ((parent.type === experimental_utils_1.AST_NODE_TYPES.CallExpression ||
            parent.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression) &&
            // make sure this isn't an IIFE
            parent.callee !== callee);
    }
    /**
     * Checks if a function belongs to:
     * ```
     * () => ({ action: 'xxx' } as const)
     * ```
     */
    function returnsConstAssertionDirectly(node) {
        const { body } = node;
        if (astUtils_1.isTypeAssertion(body)) {
            const { typeAnnotation } = body;
            if (typeAnnotation.type === experimental_utils_1.AST_NODE_TYPES.TSTypeReference) {
                const { typeName } = typeAnnotation;
                if (typeName.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                    typeName.name === 'const') {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * Checks if a function declaration/expression has a return type.
     */
    function checkFunctionReturnType(node, options, sourceCode, report) {
        if (options.allowHigherOrderFunctions &&
            doesImmediatelyReturnFunctionExpression(node)) {
            return;
        }
        if (node.returnType || astUtils_1.isConstructor(node.parent) || astUtils_1.isSetter(node.parent)) {
            return;
        }
        report(getReporLoc(node, sourceCode));
    }
    exports.checkFunctionReturnType = checkFunctionReturnType;
    function isTypedFunctionExpression(node, options) {
        const parent = nullThrows_1.nullThrows(node.parent, nullThrows_1.NullThrowsReasons.MissingParent);
        if (!options.allowTypedFunctionExpressions) {
            return false;
        }
        return (astUtils_1.isTypeAssertion(parent) ||
            isVariableDeclaratorWithTypeAnnotation(parent) ||
            isClassPropertyWithTypeAnnotation(parent) ||
            isPropertyOfObjectWithType(parent) ||
            isFunctionArgument(parent, node) ||
            isConstructorArgument(parent));
    }
    exports.isTypedFunctionExpression = isTypedFunctionExpression;
    /**
     * Checks if a function declaration/expression has a return type.
     */
    function checkFunctionExpressionReturnType(node, options, sourceCode, report) {
        if (isTypedFunctionExpression(node, options)) {
            return;
        }
        const parent = nullThrows_1.nullThrows(node.parent, nullThrows_1.NullThrowsReasons.MissingParent);
        if (options.allowExpressions &&
            parent.type !== experimental_utils_1.AST_NODE_TYPES.VariableDeclarator &&
            parent.type !== experimental_utils_1.AST_NODE_TYPES.MethodDefinition &&
            parent.type !== experimental_utils_1.AST_NODE_TYPES.ExportDefaultDeclaration &&
            parent.type !== experimental_utils_1.AST_NODE_TYPES.ClassProperty) {
            return;
        }
        // https://github.com/typescript-eslint/typescript-eslint/issues/653
        if (options.allowDirectConstAssertionInArrowFunctions &&
            node.type === experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression &&
            returnsConstAssertionDirectly(node)) {
            return;
        }
        checkFunctionReturnType(node, options, sourceCode, report);
    }
    exports.checkFunctionExpressionReturnType = checkFunctionExpressionReturnType;
},
/* @typescript-eslint/eslint-plugin/dist/rules/explicit-member-accessibility.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    const accessibilityLevel = { enum: ['explicit', 'no-public', 'off'] };
    exports.default = util.createRule({
        name: 'explicit-member-accessibility',
        meta: {
            type: 'problem',
            docs: {
                description: 'Require explicit accessibility modifiers on class properties and methods',
                category: 'Stylistic Issues',
                // too opinionated to be recommended
                recommended: false,
            },
            fixable: 'code',
            messages: {
                missingAccessibility: 'Missing accessibility modifier on {{type}} {{name}}.',
                unwantedPublicAccessibility: 'Public accessibility modifier on {{type}} {{name}}.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        accessibility: accessibilityLevel,
                        overrides: {
                            type: 'object',
                            properties: {
                                accessors: accessibilityLevel,
                                constructors: accessibilityLevel,
                                methods: accessibilityLevel,
                                properties: accessibilityLevel,
                                parameterProperties: accessibilityLevel,
                            },
                            additionalProperties: false,
                        },
                        ignoredMethodNames: {
                            type: 'array',
                            items: {
                                type: 'string',
                            },
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [{ accessibility: 'explicit' }],
        create(context, [option]) {
            var _a, _b, _c, _d, _e, _f, _g, _h;
            const sourceCode = context.getSourceCode();
            const baseCheck = (_a = option.accessibility) !== null && _a !== void 0 ? _a : 'explicit';
            const overrides = (_b = option.overrides) !== null && _b !== void 0 ? _b : {};
            const ctorCheck = (_c = overrides.constructors) !== null && _c !== void 0 ? _c : baseCheck;
            const accessorCheck = (_d = overrides.accessors) !== null && _d !== void 0 ? _d : baseCheck;
            const methodCheck = (_e = overrides.methods) !== null && _e !== void 0 ? _e : baseCheck;
            const propCheck = (_f = overrides.properties) !== null && _f !== void 0 ? _f : baseCheck;
            const paramPropCheck = (_g = overrides.parameterProperties) !== null && _g !== void 0 ? _g : baseCheck;
            const ignoredMethodNames = new Set((_h = option.ignoredMethodNames) !== null && _h !== void 0 ? _h : []);
            /**
             * Generates the report for rule violations
             */
            function reportIssue(messageId, nodeType, node, nodeName, fix = null) {
                context.report({
                    node: node,
                    messageId: messageId,
                    data: {
                        type: nodeType,
                        name: nodeName,
                    },
                    fix: fix,
                });
            }
            /**
             * Checks if a method declaration has an accessibility modifier.
             * @param methodDefinition The node representing a MethodDefinition.
             */
            function checkMethodAccessibilityModifier(methodDefinition) {
                let nodeType = 'method definition';
                let check = baseCheck;
                switch (methodDefinition.kind) {
                    case 'method':
                        check = methodCheck;
                        break;
                    case 'constructor':
                        check = ctorCheck;
                        break;
                    case 'get':
                    case 'set':
                        check = accessorCheck;
                        nodeType = `${methodDefinition.kind} property accessor`;
                        break;
                }
                const methodName = util.getNameFromMember(methodDefinition, sourceCode);
                if (check === 'off' || ignoredMethodNames.has(methodName)) {
                    return;
                }
                if (check === 'no-public' &&
                    methodDefinition.accessibility === 'public') {
                    reportIssue('unwantedPublicAccessibility', nodeType, methodDefinition, methodName, getUnwantedPublicAccessibilityFixer(methodDefinition));
                }
                else if (check === 'explicit' && !methodDefinition.accessibility) {
                    reportIssue('missingAccessibility', nodeType, methodDefinition, methodName);
                }
            }
            /**
             * Creates a fixer that removes a "public" keyword with following spaces
             */
            function getUnwantedPublicAccessibilityFixer(node) {
                return function (fixer) {
                    const tokens = sourceCode.getTokens(node);
                    let rangeToRemove;
                    for (let i = 0; i < tokens.length; i++) {
                        const token = tokens[i];
                        if (token.type === experimental_utils_1.AST_TOKEN_TYPES.Keyword &&
                            token.value === 'public') {
                            const commensAfterPublicKeyword = sourceCode.getCommentsAfter(token);
                            if (commensAfterPublicKeyword.length) {
                                // public /* Hi there! */ static foo()
                                // ^^^^^^^
                                rangeToRemove = [
                                    token.range[0],
                                    commensAfterPublicKeyword[0].range[0],
                                ];
                                break;
                            }
                            else {
                                // public static foo()
                                // ^^^^^^^
                                rangeToRemove = [token.range[0], tokens[i + 1].range[0]];
                                break;
                            }
                        }
                    }
                    return fixer.removeRange(rangeToRemove);
                };
            }
            /**
             * Checks if property has an accessibility modifier.
             * @param classProperty The node representing a ClassProperty.
             */
            function checkPropertyAccessibilityModifier(classProperty) {
                const nodeType = 'class property';
                const propertyName = util.getNameFromMember(classProperty, sourceCode);
                if (propCheck === 'no-public' &&
                    classProperty.accessibility === 'public') {
                    reportIssue('unwantedPublicAccessibility', nodeType, classProperty, propertyName, getUnwantedPublicAccessibilityFixer(classProperty));
                }
                else if (propCheck === 'explicit' && !classProperty.accessibility) {
                    reportIssue('missingAccessibility', nodeType, classProperty, propertyName);
                }
            }
            /**
             * Checks that the parameter property has the desired accessibility modifiers set.
             * @param node The node representing a Parameter Property
             */
            function checkParameterPropertyAccessibilityModifier(node) {
                const nodeType = 'parameter property';
                // HAS to be an identifier or assignment or TSC will throw
                if (node.parameter.type !== experimental_utils_1.AST_NODE_TYPES.Identifier &&
                    node.parameter.type !== experimental_utils_1.AST_NODE_TYPES.AssignmentPattern) {
                    return;
                }
                const nodeName = node.parameter.type === experimental_utils_1.AST_NODE_TYPES.Identifier
                    ? node.parameter.name
                    : // has to be an Identifier or TSC will throw an error
                        node.parameter.left.name;
                switch (paramPropCheck) {
                    case 'explicit': {
                        if (!node.accessibility) {
                            reportIssue('missingAccessibility', nodeType, node, nodeName);
                        }
                        break;
                    }
                    case 'no-public': {
                        if (node.accessibility === 'public' && node.readonly) {
                            reportIssue('unwantedPublicAccessibility', nodeType, node, nodeName, getUnwantedPublicAccessibilityFixer(node));
                        }
                        break;
                    }
                }
            }
            return {
                TSParameterProperty: checkParameterPropertyAccessibilityModifier,
                ClassProperty: checkPropertyAccessibilityModifier,
                MethodDefinition: checkMethodAccessibilityModifier,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/explicit-module-boundary-types.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    const explicitReturnTypeUtils_1 = require(663) /* ../util/explicitReturnTypeUtils */;
    exports.default = util.createRule({
        name: 'explicit-module-boundary-types',
        meta: {
            type: 'problem',
            docs: {
                description: "Require explicit return and argument types on exported functions' and classes' public class methods",
                category: 'Stylistic Issues',
                recommended: false,
            },
            messages: {
                missingReturnType: 'Missing return type on function.',
                missingArgType: "Argument '{{name}}' should be typed.",
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        allowTypedFunctionExpressions: {
                            type: 'boolean',
                        },
                        allowHigherOrderFunctions: {
                            type: 'boolean',
                        },
                        allowDirectConstAssertionInArrowFunctions: {
                            type: 'boolean',
                        },
                        allowedNames: {
                            type: 'array',
                            items: {
                                type: 'string',
                            },
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            {
                allowTypedFunctionExpressions: true,
                allowHigherOrderFunctions: true,
                allowDirectConstAssertionInArrowFunctions: true,
                allowedNames: [],
            },
        ],
        create(context, [options]) {
            const sourceCode = context.getSourceCode();
            function isUnexported(node) {
                let isReturnedValue = false;
                while (node) {
                    if (node.type === experimental_utils_1.AST_NODE_TYPES.ExportDefaultDeclaration ||
                        node.type === experimental_utils_1.AST_NODE_TYPES.ExportNamedDeclaration ||
                        node.type === experimental_utils_1.AST_NODE_TYPES.ExportSpecifier) {
                        return false;
                    }
                    if (node.type === experimental_utils_1.AST_NODE_TYPES.JSXExpressionContainer) {
                        return true;
                    }
                    if (node.type === experimental_utils_1.AST_NODE_TYPES.ReturnStatement) {
                        isReturnedValue = true;
                    }
                    if (node.type === experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression ||
                        node.type === experimental_utils_1.AST_NODE_TYPES.FunctionDeclaration ||
                        node.type === experimental_utils_1.AST_NODE_TYPES.FunctionExpression) {
                        isReturnedValue = false;
                    }
                    if (node.type === experimental_utils_1.AST_NODE_TYPES.BlockStatement && !isReturnedValue) {
                        return true;
                    }
                    node = node.parent;
                }
                return true;
            }
            function isArgumentUntyped(node) {
                return (!node.typeAnnotation ||
                    node.typeAnnotation.typeAnnotation.type === experimental_utils_1.AST_NODE_TYPES.TSAnyKeyword);
            }
            /**
             * Checks if a function declaration/expression has a return type.
             */
            function checkArguments(node) {
                const paramIdentifiers = node.params.filter(util.isIdentifier);
                const untypedArgs = paramIdentifiers.filter(isArgumentUntyped);
                untypedArgs.forEach(untypedArg => context.report({
                    node,
                    messageId: 'missingArgType',
                    data: {
                        name: untypedArg.name,
                    },
                }));
            }
            /**
             * Checks if a function name is allowed and should not be checked.
             */
            function isAllowedName(node) {
                if (!node || !options.allowedNames || !options.allowedNames.length) {
                    return false;
                }
                if (node.type === experimental_utils_1.AST_NODE_TYPES.VariableDeclarator) {
                    return (node.id.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                        options.allowedNames.includes(node.id.name));
                }
                else if (node.type === experimental_utils_1.AST_NODE_TYPES.MethodDefinition ||
                    node.type === experimental_utils_1.AST_NODE_TYPES.TSAbstractMethodDefinition) {
                    if (node.key.type === experimental_utils_1.AST_NODE_TYPES.Literal &&
                        typeof node.key.value === 'string') {
                        return options.allowedNames.includes(node.key.value);
                    }
                    if (node.key.type === experimental_utils_1.AST_NODE_TYPES.TemplateLiteral &&
                        node.key.expressions.length === 0) {
                        return options.allowedNames.includes(node.key.quasis[0].value.raw);
                    }
                    if (!node.computed && node.key.type === experimental_utils_1.AST_NODE_TYPES.Identifier) {
                        return options.allowedNames.includes(node.key.name);
                    }
                }
                return false;
            }
            return {
                'ArrowFunctionExpression, FunctionExpression'(node) {
                    var _a;
                    if (((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type) === experimental_utils_1.AST_NODE_TYPES.MethodDefinition &&
                        node.parent.accessibility === 'private') {
                        // don't check private methods as they aren't part of the public signature
                        return;
                    }
                    if (isAllowedName(node.parent) ||
                        isUnexported(node) ||
                        explicitReturnTypeUtils_1.isTypedFunctionExpression(node, options)) {
                        return;
                    }
                    explicitReturnTypeUtils_1.checkFunctionExpressionReturnType(node, options, sourceCode, loc => context.report({
                        node,
                        loc,
                        messageId: 'missingReturnType',
                    }));
                    checkArguments(node);
                },
                FunctionDeclaration(node) {
                    if (isAllowedName(node.parent) || isUnexported(node)) {
                        return;
                    }
                    explicitReturnTypeUtils_1.checkFunctionReturnType(node, options, sourceCode, loc => context.report({
                        node,
                        loc,
                        messageId: 'missingReturnType',
                    }));
                    checkArguments(node);
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/func-call-spacing.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const eslint_utils_1 = require(250) /* eslint-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'func-call-spacing',
        meta: {
            type: 'layout',
            docs: {
                description: 'Require or disallow spacing between function identifiers and their invocations',
                category: 'Stylistic Issues',
                recommended: false,
                extendsBaseRule: true,
            },
            fixable: 'whitespace',
            schema: {
                anyOf: [
                    {
                        type: 'array',
                        items: [
                            {
                                enum: ['never'],
                            },
                        ],
                        minItems: 0,
                        maxItems: 1,
                    },
                    {
                        type: 'array',
                        items: [
                            {
                                enum: ['always'],
                            },
                            {
                                type: 'object',
                                properties: {
                                    allowNewlines: {
                                        type: 'boolean',
                                    },
                                },
                                additionalProperties: false,
                            },
                        ],
                        minItems: 0,
                        maxItems: 2,
                    },
                ],
            },
            messages: {
                unexpected: 'Unexpected space or newline between function name and paren.',
                missing: 'Missing space between function name and paren.',
            },
        },
        defaultOptions: ['never', {}],
        create(context, [option, config]) {
            const sourceCode = context.getSourceCode();
            const text = sourceCode.getText();
            /**
             * Check if open space is present in a function name
             * @param {ASTNode} node node to evaluate
             * @returns {void}
             * @private
             */
            function checkSpacing(node) {
                var _a;
                const isOptionalCall = util.isOptionalOptionalChain(node);
                const closingParenToken = sourceCode.getLastToken(node);
                const lastCalleeTokenWithoutPossibleParens = sourceCode.getLastToken((_a = node.typeParameters) !== null && _a !== void 0 ? _a : node.callee);
                const openingParenToken = sourceCode.getFirstTokenBetween(lastCalleeTokenWithoutPossibleParens, closingParenToken, eslint_utils_1.isOpeningParenToken);
                if (!openingParenToken || openingParenToken.range[1] >= node.range[1]) {
                    // new expression with no parens...
                    return;
                }
                const lastCalleeToken = sourceCode.getTokenBefore(openingParenToken, util.isNotOptionalChainPunctuator);
                const textBetweenTokens = text
                    .slice(lastCalleeToken.range[1], openingParenToken.range[0])
                    .replace(/\/\*.*?\*\//gu, '');
                const hasWhitespace = /\s/u.test(textBetweenTokens);
                const hasNewline = hasWhitespace && util.LINEBREAK_MATCHER.test(textBetweenTokens);
                if (option === 'never') {
                    if (hasWhitespace) {
                        return context.report({
                            node,
                            loc: lastCalleeToken.loc.start,
                            messageId: 'unexpected',
                            fix(fixer) {
                                /*
                                 * Only autofix if there is no newline
                                 * https://github.com/eslint/eslint/issues/7787
                                 */
                                if (!hasNewline &&
                                    // don't fix optional calls
                                    !isOptionalCall) {
                                    return fixer.removeRange([
                                        lastCalleeToken.range[1],
                                        openingParenToken.range[0],
                                    ]);
                                }
                                return null;
                            },
                        });
                    }
                }
                else if (isOptionalCall) {
                    // disallow:
                    // foo?. ();
                    // foo ?.();
                    // foo ?. ();
                    if (hasWhitespace || hasNewline) {
                        context.report({
                            node,
                            loc: lastCalleeToken.loc.start,
                            messageId: 'unexpected',
                        });
                    }
                }
                else {
                    if (!hasWhitespace) {
                        context.report({
                            node,
                            loc: lastCalleeToken.loc.start,
                            messageId: 'missing',
                            fix(fixer) {
                                return fixer.insertTextBefore(openingParenToken, ' ');
                            },
                        });
                    }
                    else if (!config.allowNewlines && hasNewline) {
                        context.report({
                            node,
                            loc: lastCalleeToken.loc.start,
                            messageId: 'unexpected',
                            fix(fixer) {
                                return fixer.replaceTextRange([lastCalleeToken.range[1], openingParenToken.range[0]], ' ');
                            },
                        });
                    }
                }
            }
            return {
                CallExpression: checkSpacing,
                OptionalCallExpression: checkSpacing,
                NewExpression: checkSpacing,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/generic-type-naming.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'generic-type-naming',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Enforces naming of generic type variables',
                category: 'Stylistic Issues',
                // too opinionated to be recommended
                recommended: false,
            },
            deprecated: true,
            replacedBy: ['naming-convention'],
            messages: {
                paramNotMatchRule: 'Type parameter {{name}} does not match rule {{rule}}.',
            },
            schema: [
                {
                    type: 'string',
                },
            ],
        },
        defaultOptions: [
            // Matches: T , TA , TAbc , TA1Bca , T1 , T2
            '^T([A-Z0-9][a-zA-Z0-9]*){0,1}$',
        ],
        create(context, [rule]) {
            const regex = new RegExp(rule);
            return {
                TSTypeParameter(node) {
                    const name = node.name.name;
                    if (name && !regex.test(name)) {
                        context.report({
                            node,
                            messageId: 'paramNotMatchRule',
                            data: {
                                name,
                                rule,
                            },
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/indent.js */ function _(require, module, exports) {
    /**
     * Note this file is rather type-unsafe in its current state.
     * This is due to some really funky type conversions between different node types.
     * This is done intentionally based on the internal implementation of the base indent rule.
     */
    /* eslint-disable @typescript-eslint/no-explicit-any */
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const indent_1 = __importDefault(require(179) /* eslint/lib/rules/indent */);
    const util = __importStar(require(622) /* ../util */);
    const KNOWN_NODES = new Set([
        // Class properties aren't yet supported by eslint...
        experimental_utils_1.AST_NODE_TYPES.ClassProperty,
        // ts keywords
        experimental_utils_1.AST_NODE_TYPES.TSAbstractKeyword,
        experimental_utils_1.AST_NODE_TYPES.TSAnyKeyword,
        experimental_utils_1.AST_NODE_TYPES.TSBooleanKeyword,
        experimental_utils_1.AST_NODE_TYPES.TSNeverKeyword,
        experimental_utils_1.AST_NODE_TYPES.TSNumberKeyword,
        experimental_utils_1.AST_NODE_TYPES.TSStringKeyword,
        experimental_utils_1.AST_NODE_TYPES.TSSymbolKeyword,
        experimental_utils_1.AST_NODE_TYPES.TSUndefinedKeyword,
        experimental_utils_1.AST_NODE_TYPES.TSUnknownKeyword,
        experimental_utils_1.AST_NODE_TYPES.TSVoidKeyword,
        experimental_utils_1.AST_NODE_TYPES.TSNullKeyword,
        // ts specific nodes we want to support
        experimental_utils_1.AST_NODE_TYPES.TSAbstractClassProperty,
        experimental_utils_1.AST_NODE_TYPES.TSAbstractMethodDefinition,
        experimental_utils_1.AST_NODE_TYPES.TSArrayType,
        experimental_utils_1.AST_NODE_TYPES.TSAsExpression,
        experimental_utils_1.AST_NODE_TYPES.TSCallSignatureDeclaration,
        experimental_utils_1.AST_NODE_TYPES.TSConditionalType,
        experimental_utils_1.AST_NODE_TYPES.TSConstructorType,
        experimental_utils_1.AST_NODE_TYPES.TSConstructSignatureDeclaration,
        experimental_utils_1.AST_NODE_TYPES.TSDeclareFunction,
        experimental_utils_1.AST_NODE_TYPES.TSEmptyBodyFunctionExpression,
        experimental_utils_1.AST_NODE_TYPES.TSEnumDeclaration,
        experimental_utils_1.AST_NODE_TYPES.TSEnumMember,
        experimental_utils_1.AST_NODE_TYPES.TSExportAssignment,
        experimental_utils_1.AST_NODE_TYPES.TSExternalModuleReference,
        experimental_utils_1.AST_NODE_TYPES.TSFunctionType,
        experimental_utils_1.AST_NODE_TYPES.TSImportType,
        experimental_utils_1.AST_NODE_TYPES.TSIndexedAccessType,
        experimental_utils_1.AST_NODE_TYPES.TSIndexSignature,
        experimental_utils_1.AST_NODE_TYPES.TSInferType,
        experimental_utils_1.AST_NODE_TYPES.TSInterfaceBody,
        experimental_utils_1.AST_NODE_TYPES.TSInterfaceDeclaration,
        experimental_utils_1.AST_NODE_TYPES.TSInterfaceHeritage,
        experimental_utils_1.AST_NODE_TYPES.TSIntersectionType,
        experimental_utils_1.AST_NODE_TYPES.TSImportEqualsDeclaration,
        experimental_utils_1.AST_NODE_TYPES.TSLiteralType,
        experimental_utils_1.AST_NODE_TYPES.TSMappedType,
        experimental_utils_1.AST_NODE_TYPES.TSMethodSignature,
        'TSMinusToken',
        experimental_utils_1.AST_NODE_TYPES.TSModuleBlock,
        experimental_utils_1.AST_NODE_TYPES.TSModuleDeclaration,
        experimental_utils_1.AST_NODE_TYPES.TSNonNullExpression,
        experimental_utils_1.AST_NODE_TYPES.TSParameterProperty,
        experimental_utils_1.AST_NODE_TYPES.TSParenthesizedType,
        'TSPlusToken',
        experimental_utils_1.AST_NODE_TYPES.TSPropertySignature,
        experimental_utils_1.AST_NODE_TYPES.TSQualifiedName,
        'TSQuestionToken',
        experimental_utils_1.AST_NODE_TYPES.TSRestType,
        experimental_utils_1.AST_NODE_TYPES.TSThisType,
        experimental_utils_1.AST_NODE_TYPES.TSTupleType,
        experimental_utils_1.AST_NODE_TYPES.TSTypeAnnotation,
        experimental_utils_1.AST_NODE_TYPES.TSTypeLiteral,
        experimental_utils_1.AST_NODE_TYPES.TSTypeOperator,
        experimental_utils_1.AST_NODE_TYPES.TSTypeParameter,
        experimental_utils_1.AST_NODE_TYPES.TSTypeParameterDeclaration,
        experimental_utils_1.AST_NODE_TYPES.TSTypeParameterInstantiation,
        experimental_utils_1.AST_NODE_TYPES.TSTypeReference,
        experimental_utils_1.AST_NODE_TYPES.TSUnionType,
        experimental_utils_1.AST_NODE_TYPES.Decorator,
    ]);
    exports.default = util.createRule({
        name: 'indent',
        meta: {
            type: 'layout',
            docs: {
                description: 'Enforce consistent indentation',
                category: 'Stylistic Issues',
                // too opinionated to be recommended
                recommended: false,
                extendsBaseRule: true,
            },
            fixable: 'whitespace',
            schema: indent_1.default.meta.schema,
            messages: indent_1.default.meta.messages,
        },
        defaultOptions: [
            // typescript docs and playground use 4 space indent
            4,
            {
                // typescript docs indent the case from the switch
                // https://www.typescriptlang.org/docs/handbook/release-notes/typescript-1-8.html#example-4
                SwitchCase: 1,
                flatTernaryExpressions: false,
                ignoredNodes: [],
            },
        ],
        create(context, optionsWithDefaults) {
            // because we extend the base rule, have to update opts on the context
            // the context defines options as readonly though...
            const contextWithDefaults = Object.create(context, {
                options: {
                    writable: false,
                    configurable: false,
                    value: optionsWithDefaults,
                },
            });
            const rules = indent_1.default.create(contextWithDefaults);
            /**
             * Converts from a TSPropertySignature to a Property
             * @param node a TSPropertySignature node
             * @param [type] the type to give the new node
             * @returns a Property node
             */
            function TSPropertySignatureToProperty(node, type = experimental_utils_1.AST_NODE_TYPES.Property) {
                const base = {
                    // indent doesn't actually use these
                    key: null,
                    value: null,
                    // Property flags
                    computed: false,
                    method: false,
                    kind: 'init',
                    // this will stop eslint from interrogating the type literal
                    shorthand: true,
                    // location data
                    parent: node.parent,
                    range: node.range,
                    loc: node.loc,
                };
                if (type === experimental_utils_1.AST_NODE_TYPES.Property) {
                    return Object.assign({ type }, base);
                }
                else {
                    return Object.assign({ type, static: false, readonly: false, declare: false }, base);
                }
            }
            return Object.assign({}, rules, {
                // overwrite the base rule here so we can use our KNOWN_NODES list instead
                '*:exit'(node) {
                    // For nodes we care about, skip the default handling, because it just marks the node as ignored...
                    if (!KNOWN_NODES.has(node.type)) {
                        rules['*:exit'](node);
                    }
                },
                VariableDeclaration(node) {
                    // https://github.com/typescript-eslint/typescript-eslint/issues/441
                    if (node.declarations.length === 0) {
                        return;
                    }
                    return rules.VariableDeclaration(node);
                },
                TSAsExpression(node) {
                    // transform it to a BinaryExpression
                    return rules['BinaryExpression, LogicalExpression']({
                        type: experimental_utils_1.AST_NODE_TYPES.BinaryExpression,
                        operator: 'as',
                        left: node.expression,
                        // the first typeAnnotation includes the as token
                        right: node.typeAnnotation,
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                    });
                },
                TSConditionalType(node) {
                    // transform it to a ConditionalExpression
                    return rules.ConditionalExpression({
                        type: experimental_utils_1.AST_NODE_TYPES.ConditionalExpression,
                        test: {
                            type: experimental_utils_1.AST_NODE_TYPES.BinaryExpression,
                            operator: 'extends',
                            left: node.checkType,
                            right: node.extendsType,
                            // location data
                            range: [node.checkType.range[0], node.extendsType.range[1]],
                            loc: {
                                start: node.checkType.loc.start,
                                end: node.extendsType.loc.end,
                            },
                        },
                        consequent: node.trueType,
                        alternate: node.falseType,
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                    });
                },
                'TSEnumDeclaration, TSTypeLiteral'(node) {
                    // transform it to an ObjectExpression
                    return rules['ObjectExpression, ObjectPattern']({
                        type: experimental_utils_1.AST_NODE_TYPES.ObjectExpression,
                        properties: node.members.map(member => TSPropertySignatureToProperty(member)),
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                    });
                },
                TSImportEqualsDeclaration(node) {
                    // transform it to an VariableDeclaration
                    // use VariableDeclaration instead of ImportDeclaration because it's essentially the same thing
                    const { id, moduleReference } = node;
                    return rules.VariableDeclaration({
                        type: experimental_utils_1.AST_NODE_TYPES.VariableDeclaration,
                        kind: 'const',
                        declarations: [
                            {
                                type: experimental_utils_1.AST_NODE_TYPES.VariableDeclarator,
                                range: [id.range[0], moduleReference.range[1]],
                                loc: {
                                    start: id.loc.start,
                                    end: moduleReference.loc.end,
                                },
                                id: id,
                                init: {
                                    type: experimental_utils_1.AST_NODE_TYPES.CallExpression,
                                    callee: {
                                        type: experimental_utils_1.AST_NODE_TYPES.Identifier,
                                        name: 'require',
                                        range: [
                                            moduleReference.range[0],
                                            moduleReference.range[0] + 'require'.length,
                                        ],
                                        loc: {
                                            start: moduleReference.loc.start,
                                            end: {
                                                line: moduleReference.loc.end.line,
                                                column: moduleReference.loc.start.line + 'require'.length,
                                            },
                                        },
                                    },
                                    arguments: 'expression' in moduleReference
                                        ? [moduleReference.expression]
                                        : [],
                                    // location data
                                    range: moduleReference.range,
                                    loc: moduleReference.loc,
                                },
                            },
                        ],
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                    });
                },
                TSIndexedAccessType(node) {
                    // convert to a MemberExpression
                    return rules['MemberExpression, JSXMemberExpression, MetaProperty']({
                        type: experimental_utils_1.AST_NODE_TYPES.MemberExpression,
                        object: node.objectType,
                        property: node.indexType,
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                        optional: false,
                        computed: true,
                    });
                },
                TSInterfaceBody(node) {
                    // transform it to an ClassBody
                    return rules['BlockStatement, ClassBody']({
                        type: experimental_utils_1.AST_NODE_TYPES.ClassBody,
                        body: node.body.map(p => TSPropertySignatureToProperty(p, experimental_utils_1.AST_NODE_TYPES.ClassProperty)),
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                    });
                },
                'TSInterfaceDeclaration[extends.length > 0]'(node) {
                    // transform it to a ClassDeclaration
                    return rules['ClassDeclaration[superClass], ClassExpression[superClass]']({
                        type: experimental_utils_1.AST_NODE_TYPES.ClassDeclaration,
                        body: node.body,
                        id: null,
                        // TODO: This is invalid, there can be more than one extends in interface
                        superClass: node.extends[0].expression,
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                    });
                },
                TSMappedType(node) {
                    const sourceCode = context.getSourceCode();
                    const squareBracketStart = sourceCode.getTokenBefore(node.typeParameter);
                    // transform it to an ObjectExpression
                    return rules['ObjectExpression, ObjectPattern']({
                        type: experimental_utils_1.AST_NODE_TYPES.ObjectExpression,
                        properties: [
                            {
                                type: experimental_utils_1.AST_NODE_TYPES.Property,
                                key: node.typeParameter,
                                value: node.typeAnnotation,
                                // location data
                                range: [
                                    squareBracketStart.range[0],
                                    node.typeAnnotation
                                        ? node.typeAnnotation.range[1]
                                        : squareBracketStart.range[0],
                                ],
                                loc: {
                                    start: squareBracketStart.loc.start,
                                    end: node.typeAnnotation
                                        ? node.typeAnnotation.loc.end
                                        : squareBracketStart.loc.end,
                                },
                                kind: 'init',
                                computed: false,
                                method: false,
                                shorthand: false,
                            },
                        ],
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                    });
                },
                TSModuleBlock(node) {
                    // transform it to a BlockStatement
                    return rules['BlockStatement, ClassBody']({
                        type: experimental_utils_1.AST_NODE_TYPES.BlockStatement,
                        body: node.body,
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                    });
                },
                TSQualifiedName(node) {
                    return rules['MemberExpression, JSXMemberExpression, MetaProperty']({
                        type: experimental_utils_1.AST_NODE_TYPES.MemberExpression,
                        object: node.left,
                        property: node.right,
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                        optional: false,
                        computed: false,
                    });
                },
                TSTupleType(node) {
                    // transform it to an ArrayExpression
                    return rules['ArrayExpression, ArrayPattern']({
                        type: experimental_utils_1.AST_NODE_TYPES.ArrayExpression,
                        elements: node.elementTypes,
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                    });
                },
                TSTypeParameterDeclaration(node) {
                    if (!node.params.length) {
                        return;
                    }
                    const [name, ...attributes] = node.params;
                    // JSX is about the closest we can get because the angle brackets
                    // it's not perfect but it works!
                    return rules.JSXOpeningElement({
                        type: experimental_utils_1.AST_NODE_TYPES.JSXOpeningElement,
                        selfClosing: false,
                        name: name,
                        attributes: attributes,
                        // location data
                        parent: node.parent,
                        range: node.range,
                        loc: node.loc,
                    });
                },
            });
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/interface-name-prefix.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    /**
     * Parses a given value as options.
     */
    function parseOptions([options]) {
        if (options === 'always') {
            return { prefixWithI: 'always', allowUnderscorePrefix: false };
        }
        if (options !== 'never' && options.prefixWithI === 'always') {
            return {
                prefixWithI: 'always',
                allowUnderscorePrefix: !!options.allowUnderscorePrefix,
            };
        }
        return { prefixWithI: 'never' };
    }
    exports.parseOptions = parseOptions;
    exports.default = util.createRule({
        name: 'interface-name-prefix',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Require that interface names should or should not prefixed with `I`',
                category: 'Stylistic Issues',
                // this will always be recommended as there's no reason to use this convention
                // https://github.com/typescript-eslint/typescript-eslint/issues/374
                recommended: 'error',
            },
            deprecated: true,
            replacedBy: ['naming-convention'],
            messages: {
                noPrefix: 'Interface name must not be prefixed with "I".',
                alwaysPrefix: 'Interface name must be prefixed with "I".',
            },
            schema: [
                {
                    oneOf: [
                        {
                            enum: [
                                // Deprecated, equivalent to: { prefixWithI: 'never' }
                                'never',
                                // Deprecated, equivalent to: { prefixWithI: 'always', allowUnderscorePrefix: false }
                                'always',
                            ],
                        },
                        {
                            type: 'object',
                            properties: {
                                prefixWithI: {
                                    type: 'string',
                                    enum: ['never'],
                                },
                            },
                            additionalProperties: false,
                        },
                        {
                            type: 'object',
                            properties: {
                                prefixWithI: {
                                    type: 'string',
                                    enum: ['always'],
                                },
                                allowUnderscorePrefix: {
                                    type: 'boolean',
                                },
                            },
                            required: ['prefixWithI'],
                            additionalProperties: false,
                        },
                    ],
                },
            ],
        },
        defaultOptions: [{ prefixWithI: 'never' }],
        create(context, [options]) {
            const parsedOptions = parseOptions([options]);
            /**
             * Checks if a string is prefixed with "I".
             * @param name The string to check
             */
            function isPrefixedWithI(name) {
                return /^I[A-Z]/.test(name);
            }
            /**
             * Checks if a string is prefixed with "I" or "_I".
             * @param name The string to check
             */
            function isPrefixedWithIOrUnderscoreI(name) {
                return /^_?I[A-Z]/.test(name);
            }
            return {
                TSInterfaceDeclaration(node) {
                    if (parsedOptions.prefixWithI === 'never') {
                        if (isPrefixedWithIOrUnderscoreI(node.id.name)) {
                            context.report({
                                node: node.id,
                                messageId: 'noPrefix',
                            });
                        }
                    }
                    else {
                        if (parsedOptions.allowUnderscorePrefix) {
                            if (!isPrefixedWithIOrUnderscoreI(node.id.name)) {
                                context.report({
                                    node: node.id,
                                    messageId: 'alwaysPrefix',
                                });
                            }
                        }
                        else {
                            if (!isPrefixedWithI(node.id.name)) {
                                context.report({
                                    node: node.id,
                                    messageId: 'alwaysPrefix',
                                });
                            }
                        }
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/member-delimiter-style.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    const definition = {
        type: 'object',
        properties: {
            multiline: {
                type: 'object',
                properties: {
                    delimiter: { enum: ['none', 'semi', 'comma'] },
                    requireLast: { type: 'boolean' },
                },
                additionalProperties: false,
            },
            singleline: {
                type: 'object',
                properties: {
                    // note can't have "none" for single line delimiter as it's invalid syntax
                    delimiter: { enum: ['semi', 'comma'] },
                    requireLast: { type: 'boolean' },
                },
                additionalProperties: false,
            },
        },
        additionalProperties: false,
    };
    exports.default = util.createRule({
        name: 'member-delimiter-style',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Require a specific member delimiter style for interfaces and type literals',
                category: 'Stylistic Issues',
                recommended: 'error',
            },
            fixable: 'code',
            messages: {
                unexpectedComma: 'Unexpected separator (,).',
                unexpectedSemi: 'Unexpected separator (;).',
                expectedComma: 'Expected a comma.',
                expectedSemi: 'Expected a semicolon.',
            },
            schema: [
                {
                    type: 'object',
                    properties: Object.assign({}, definition.properties, {
                        overrides: {
                            type: 'object',
                            properties: {
                                interface: definition,
                                typeLiteral: definition,
                            },
                            additionalProperties: false,
                        },
                    }),
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            {
                multiline: {
                    delimiter: 'semi',
                    requireLast: true,
                },
                singleline: {
                    delimiter: 'semi',
                    requireLast: false,
                },
            },
        ],
        create(context, [options]) {
            var _a;
            const sourceCode = context.getSourceCode();
            // use the base options as the defaults for the cases
            const baseOptions = options;
            const overrides = (_a = baseOptions.overrides) !== null && _a !== void 0 ? _a : {};
            const interfaceOptions = util.deepMerge(baseOptions, overrides.interface);
            const typeLiteralOptions = util.deepMerge(baseOptions, overrides.typeLiteral);
            /**
             * Check the last token in the given member.
             * @param member the member to be evaluated.
             * @param opts the options to be validated.
             * @param isLast a flag indicating `member` is the last in the interface or type literal.
             */
            function checkLastToken(member, opts, isLast) {
                /**
                 * Resolves the boolean value for the given setting enum value
                 * @param type the option name
                 */
                function getOption(type) {
                    if (isLast && !opts.requireLast) {
                        // only turn the option on if its expecting no delimiter for the last member
                        return type === 'none';
                    }
                    return opts.delimiter === type;
                }
                let messageId = null;
                let missingDelimiter = false;
                const lastToken = sourceCode.getLastToken(member, {
                    includeComments: false,
                });
                if (!lastToken) {
                    return;
                }
                const optsSemi = getOption('semi');
                const optsComma = getOption('comma');
                const optsNone = getOption('none');
                if (lastToken.value === ';') {
                    if (optsComma) {
                        messageId = 'expectedComma';
                    }
                    else if (optsNone) {
                        missingDelimiter = true;
                        messageId = 'unexpectedSemi';
                    }
                }
                else if (lastToken.value === ',') {
                    if (optsSemi) {
                        messageId = 'expectedSemi';
                    }
                    else if (optsNone) {
                        missingDelimiter = true;
                        messageId = 'unexpectedComma';
                    }
                }
                else {
                    if (optsSemi) {
                        missingDelimiter = true;
                        messageId = 'expectedSemi';
                    }
                    else if (optsComma) {
                        missingDelimiter = true;
                        messageId = 'expectedComma';
                    }
                }
                if (messageId) {
                    context.report({
                        node: lastToken,
                        loc: {
                            start: {
                                line: lastToken.loc.end.line,
                                column: lastToken.loc.end.column,
                            },
                            end: {
                                line: lastToken.loc.end.line,
                                column: lastToken.loc.end.column,
                            },
                        },
                        messageId,
                        fix(fixer) {
                            if (optsNone) {
                                // remove the unneeded token
                                return fixer.remove(lastToken);
                            }
                            const token = optsSemi ? ';' : ',';
                            if (missingDelimiter) {
                                // add the missing delimiter
                                return fixer.insertTextAfter(lastToken, token);
                            }
                            // correct the current delimiter
                            return fixer.replaceText(lastToken, token);
                        },
                    });
                }
            }
            /**
             * Check the member separator being used matches the delimiter.
             * @param {ASTNode} node the node to be evaluated.
             */
            function checkMemberSeparatorStyle(node) {
                const isSingleLine = node.loc.start.line === node.loc.end.line;
                const members = node.type === experimental_utils_1.AST_NODE_TYPES.TSInterfaceBody ? node.body : node.members;
                const typeOpts = node.type === experimental_utils_1.AST_NODE_TYPES.TSInterfaceBody
                    ? interfaceOptions
                    : typeLiteralOptions;
                const opts = isSingleLine ? typeOpts.singleline : typeOpts.multiline;
                members.forEach((member, index) => {
                    checkLastToken(member, opts !== null && opts !== void 0 ? opts : {}, index === members.length - 1);
                });
            }
            return {
                TSInterfaceBody: checkMemberSeparatorStyle,
                TSTypeLiteral: checkMemberSeparatorStyle,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/member-naming.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'member-naming',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Enforces naming conventions for class members by visibility',
                category: 'Stylistic Issues',
                recommended: false,
            },
            deprecated: true,
            replacedBy: ['naming-convention'],
            messages: {
                incorrectName: '{{accessibility}} property {{name}} should match {{convention}}.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        public: {
                            type: 'string',
                            minLength: 1,
                            format: 'regex',
                        },
                        protected: {
                            type: 'string',
                            minLength: 1,
                            format: 'regex',
                        },
                        private: {
                            type: 'string',
                            minLength: 1,
                            format: 'regex',
                        },
                    },
                    additionalProperties: false,
                    minProperties: 1,
                },
            ],
        },
        defaultOptions: [{}],
        create(context, [config]) {
            const sourceCode = context.getSourceCode();
            const conventions = Object.keys(config).reduce((acc, accessibility) => {
                acc[accessibility] = new RegExp(config[accessibility]);
                return acc;
            }, {});
            function getParameterNode(node) {
                if (node.parameter.type === experimental_utils_1.AST_NODE_TYPES.AssignmentPattern) {
                    return node.parameter.left;
                }
                if (node.parameter.type === experimental_utils_1.AST_NODE_TYPES.Identifier) {
                    return node.parameter;
                }
                return null;
            }
            function validateParameterName(node) {
                const parameterNode = getParameterNode(node);
                if (!parameterNode) {
                    return;
                }
                validate(parameterNode, parameterNode.name, node.accessibility);
            }
            function validateName(node) {
                if (node.type === experimental_utils_1.AST_NODE_TYPES.MethodDefinition &&
                    node.kind === 'constructor') {
                    return;
                }
                validate(node.key, util.getNameFromMember(node, sourceCode), node.accessibility);
            }
            /**
             * Check that the name matches the convention for its accessibility.
             * @param {ASTNode}   node the named node to evaluate.
             * @param {string}    name
             * @param {Modifiers} accessibility
             * @returns {void}
             * @private
             */
            function validate(node, name, accessibility = 'public') {
                const convention = conventions[accessibility];
                if (!convention || convention.test(name)) {
                    return;
                }
                context.report({
                    node,
                    messageId: 'incorrectName',
                    data: { accessibility, name, convention },
                });
            }
            return {
                TSParameterProperty: validateParameterName,
                MethodDefinition: validateName,
                ClassProperty: validateName,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/member-ordering.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    const allMemberTypes = ['field', 'method', 'constructor'].reduce((all, type) => {
        all.push(type);
        ['public', 'protected', 'private'].forEach(accessibility => {
            all.push(`${accessibility}-${type}`); // e.g. `public-field`
            if (type !== 'constructor') {
                // There is no `static-constructor` or `instance-constructor or `abstract-constructor`
                ['static', 'instance', 'abstract'].forEach(scope => {
                    if (!all.includes(`${scope}-${type}`)) {
                        all.push(`${scope}-${type}`);
                    }
                    all.push(`${accessibility}-${scope}-${type}`);
                });
            }
        });
        return all;
    }, []);
    allMemberTypes.unshift('signature');
    exports.default = util.createRule({
        name: 'member-ordering',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Require a consistent member declaration order',
                category: 'Stylistic Issues',
                recommended: false,
            },
            messages: {
                incorrectOrder: 'Member {{name}} should be declared before all {{rank}} definitions.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        default: {
                            oneOf: [
                                {
                                    enum: ['never'],
                                },
                                {
                                    type: 'array',
                                    items: {
                                        enum: allMemberTypes,
                                    },
                                },
                            ],
                        },
                        classes: {
                            oneOf: [
                                {
                                    enum: ['never'],
                                },
                                {
                                    type: 'array',
                                    items: {
                                        enum: allMemberTypes,
                                    },
                                },
                            ],
                        },
                        classExpressions: {
                            oneOf: [
                                {
                                    enum: ['never'],
                                },
                                {
                                    type: 'array',
                                    items: {
                                        enum: allMemberTypes,
                                    },
                                },
                            ],
                        },
                        interfaces: {
                            oneOf: [
                                {
                                    enum: ['never'],
                                },
                                {
                                    type: 'array',
                                    items: {
                                        enum: ['signature', 'field', 'method', 'constructor'],
                                    },
                                },
                            ],
                        },
                        typeLiterals: {
                            oneOf: [
                                {
                                    enum: ['never'],
                                },
                                {
                                    type: 'array',
                                    items: {
                                        enum: ['signature', 'field', 'method', 'constructor'],
                                    },
                                },
                            ],
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            {
                default: [
                    'signature',
                    'public-static-field',
                    'protected-static-field',
                    'private-static-field',
                    'public-instance-field',
                    'protected-instance-field',
                    'private-instance-field',
                    'public-abstract-field',
                    'protected-abstract-field',
                    'private-abstract-field',
                    'public-field',
                    'protected-field',
                    'private-field',
                    'static-field',
                    'instance-field',
                    'abstract-field',
                    'field',
                    'constructor',
                    'public-static-method',
                    'protected-static-method',
                    'private-static-method',
                    'public-instance-method',
                    'protected-instance-method',
                    'private-instance-method',
                    'public-abstract-method',
                    'protected-abstract-method',
                    'private-abstract-method',
                    'public-method',
                    'protected-method',
                    'private-method',
                    'static-method',
                    'instance-method',
                    'abstract-method',
                    'method',
                ],
            },
        ],
        create(context, [options]) {
            const sourceCode = context.getSourceCode();
            const functionExpressions = [
                experimental_utils_1.AST_NODE_TYPES.FunctionExpression,
                experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression,
            ];
            /**
             * Gets the node type.
             * @param node the node to be evaluated.
             */
            function getNodeType(node) {
                // TODO: add missing TSCallSignatureDeclaration
                switch (node.type) {
                    case experimental_utils_1.AST_NODE_TYPES.TSAbstractMethodDefinition:
                    case experimental_utils_1.AST_NODE_TYPES.MethodDefinition:
                        return node.kind;
                    case experimental_utils_1.AST_NODE_TYPES.TSMethodSignature:
                        return 'method';
                    case experimental_utils_1.AST_NODE_TYPES.TSConstructSignatureDeclaration:
                        return 'constructor';
                    case experimental_utils_1.AST_NODE_TYPES.TSAbstractClassProperty:
                    case experimental_utils_1.AST_NODE_TYPES.ClassProperty:
                        return node.value && functionExpressions.includes(node.value.type)
                            ? 'method'
                            : 'field';
                    case experimental_utils_1.AST_NODE_TYPES.TSPropertySignature:
                        return 'field';
                    case experimental_utils_1.AST_NODE_TYPES.TSIndexSignature:
                        return 'signature';
                    default:
                        return null;
                }
            }
            /**
             * Gets the member name based on the member type.
             * @param node the node to be evaluated.
             */
            function getMemberName(node) {
                switch (node.type) {
                    case experimental_utils_1.AST_NODE_TYPES.TSPropertySignature:
                    case experimental_utils_1.AST_NODE_TYPES.TSMethodSignature:
                    case experimental_utils_1.AST_NODE_TYPES.TSAbstractClassProperty:
                    case experimental_utils_1.AST_NODE_TYPES.ClassProperty:
                        return util.getNameFromMember(node, sourceCode);
                    case experimental_utils_1.AST_NODE_TYPES.TSAbstractMethodDefinition:
                    case experimental_utils_1.AST_NODE_TYPES.MethodDefinition:
                        return node.kind === 'constructor'
                            ? 'constructor'
                            : util.getNameFromMember(node, sourceCode);
                    case experimental_utils_1.AST_NODE_TYPES.TSConstructSignatureDeclaration:
                        return 'new';
                    case experimental_utils_1.AST_NODE_TYPES.TSIndexSignature:
                        return util.getNameFromIndexSignature(node);
                    default:
                        return null;
                }
            }
            /**
             * Gets the calculated rank using the provided method definition.
             * The algorithm is as follows:
             * - Get the rank based on the accessibility-scope-type name, e.g. public-instance-field
             * - If there is no order for accessibility-scope-type, then strip out the accessibility.
             * - If there is no order for scope-type, then strip out the scope.
             * - If there is no order for type, then return -1
             * @param memberTypes the valid names to be validated.
             * @param order the current order to be validated.
             *
             * @return Index of the matching member type in the order configuration.
             */
            function getRankOrder(memberTypes, order) {
                let rank = -1;
                const stack = memberTypes.slice(); // Get a copy of the member types
                while (stack.length > 0 && rank === -1) {
                    rank = order.indexOf(stack.shift());
                }
                return rank;
            }
            /**
             * Gets the rank of the node given the order.
             * @param node the node to be evaluated.
             * @param order the current order to be validated.
             * @param supportsModifiers a flag indicating whether the type supports modifiers (scope or accessibility) or not.
             */
            function getRank(node, order, supportsModifiers) {
                const type = getNodeType(node);
                if (type === null) {
                    // shouldn't happen but just in case, put it on the end
                    return order.length - 1;
                }
                const abstract = node.type === experimental_utils_1.AST_NODE_TYPES.TSAbstractClassProperty ||
                    node.type === experimental_utils_1.AST_NODE_TYPES.TSAbstractMethodDefinition;
                const scope = 'static' in node && node.static
                    ? 'static'
                    : abstract
                        ? 'abstract'
                        : 'instance';
                const accessibility = 'accessibility' in node && node.accessibility
                    ? node.accessibility
                    : 'public';
                const memberTypes = [];
                if (supportsModifiers) {
                    if (type !== 'constructor') {
                        // Constructors have no scope
                        memberTypes.push(`${accessibility}-${scope}-${type}`);
                        memberTypes.push(`${scope}-${type}`);
                    }
                    memberTypes.push(`${accessibility}-${type}`);
                }
                memberTypes.push(type);
                return getRankOrder(memberTypes, order);
            }
            /**
             * Gets the lowest possible rank higher than target.
             * e.g. given the following order:
             *   ...
             *   public-static-method
             *   protected-static-method
             *   private-static-method
             *   public-instance-method
             *   protected-instance-method
             *   private-instance-method
             *   ...
             * and considering that a public-instance-method has already been declared, so ranks contains
             * public-instance-method, then the lowest possible rank for public-static-method is
             * public-instance-method.
             * @param ranks the existing ranks in the object.
             * @param target the target rank.
             * @param order the current order to be validated.
             * @returns the name of the lowest possible rank without dashes (-).
             */
            function getLowestRank(ranks, target, order) {
                let lowest = ranks[ranks.length - 1];
                ranks.forEach(rank => {
                    if (rank > target) {
                        lowest = Math.min(lowest, rank);
                    }
                });
                return order[lowest].replace(/-/g, ' ');
            }
            /**
             * Validates if all members are correctly sorted.
             *
             * @param members Members to be validated.
             * @param order Current order to be validated.
             * @param supportsModifiers A flag indicating whether the type supports modifiers (scope or accessibility) or not.
             */
            function validateMembersOrder(members, order, supportsModifiers) {
                if (members && order !== 'never') {
                    const previousRanks = [];
                    // Find first member which isn't correctly sorted
                    members.forEach(member => {
                        const rank = getRank(member, order, supportsModifiers);
                        if (rank !== -1) {
                            if (rank < previousRanks[previousRanks.length - 1]) {
                                context.report({
                                    node: member,
                                    messageId: 'incorrectOrder',
                                    data: {
                                        name: getMemberName(member),
                                        rank: getLowestRank(previousRanks, rank, order),
                                    },
                                });
                            }
                            else {
                                previousRanks.push(rank);
                            }
                        }
                    });
                }
            }
            return {
                ClassDeclaration(node) {
                    var _a;
                    validateMembersOrder(node.body.body, (_a = options.classes) !== null && _a !== void 0 ? _a : options.default, true);
                },
                ClassExpression(node) {
                    var _a;
                    validateMembersOrder(node.body.body, (_a = options.classExpressions) !== null && _a !== void 0 ? _a : options.default, true);
                },
                TSInterfaceDeclaration(node) {
                    var _a;
                    validateMembersOrder(node.body.body, (_a = options.interfaces) !== null && _a !== void 0 ? _a : options.default, false);
                },
                TSTypeLiteral(node) {
                    var _a;
                    validateMembersOrder(node.members, (_a = options.typeLiterals) !== null && _a !== void 0 ? _a : options.default, false);
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/naming-convention.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    // #region Options Type Config
    var PredefinedFormats;
    (function (PredefinedFormats) {
        PredefinedFormats[PredefinedFormats["camelCase"] = 1] = "camelCase";
        PredefinedFormats[PredefinedFormats["strictCamelCase"] = 2] = "strictCamelCase";
        PredefinedFormats[PredefinedFormats["PascalCase"] = 4] = "PascalCase";
        PredefinedFormats[PredefinedFormats["StrictPascalCase"] = 8] = "StrictPascalCase";
        // eslint-disable-next-line @typescript-eslint/camelcase
        PredefinedFormats[PredefinedFormats["snake_case"] = 16] = "snake_case";
        PredefinedFormats[PredefinedFormats["UPPER_CASE"] = 32] = "UPPER_CASE";
    })(PredefinedFormats || (PredefinedFormats = {}));
    var UnderscoreOptions;
    (function (UnderscoreOptions) {
        UnderscoreOptions[UnderscoreOptions["forbid"] = 1] = "forbid";
        UnderscoreOptions[UnderscoreOptions["allow"] = 2] = "allow";
        UnderscoreOptions[UnderscoreOptions["require"] = 4] = "require";
    })(UnderscoreOptions || (UnderscoreOptions = {}));
    var Selectors;
    (function (Selectors) {
        // variableLike
        Selectors[Selectors["variable"] = 1] = "variable";
        Selectors[Selectors["function"] = 2] = "function";
        Selectors[Selectors["parameter"] = 4] = "parameter";
        // memberLike
        Selectors[Selectors["property"] = 8] = "property";
        Selectors[Selectors["parameterProperty"] = 16] = "parameterProperty";
        Selectors[Selectors["method"] = 32] = "method";
        Selectors[Selectors["accessor"] = 64] = "accessor";
        Selectors[Selectors["enumMember"] = 128] = "enumMember";
        // typeLike
        Selectors[Selectors["class"] = 256] = "class";
        Selectors[Selectors["interface"] = 512] = "interface";
        Selectors[Selectors["typeAlias"] = 1024] = "typeAlias";
        Selectors[Selectors["enum"] = 2048] = "enum";
        Selectors[Selectors["typeParameter"] = 4096] = "typeParameter";
    })(Selectors || (Selectors = {}));
    const SELECTOR_COUNT = util.getEnumNames(Selectors).length;
    var MetaSelectors;
    (function (MetaSelectors) {
        MetaSelectors[MetaSelectors["default"] = -1] = "default";
        MetaSelectors[MetaSelectors["variableLike"] = 7] = "variableLike";
        MetaSelectors[MetaSelectors["memberLike"] = 248] = "memberLike";
        MetaSelectors[MetaSelectors["typeLike"] = 7936] = "typeLike";
    })(MetaSelectors || (MetaSelectors = {}));
    var Modifiers;
    (function (Modifiers) {
        Modifiers[Modifiers["readonly"] = 1] = "readonly";
        Modifiers[Modifiers["static"] = 2] = "static";
        Modifiers[Modifiers["public"] = 4] = "public";
        Modifiers[Modifiers["protected"] = 8] = "protected";
        Modifiers[Modifiers["private"] = 16] = "private";
        Modifiers[Modifiers["abstract"] = 32] = "abstract";
    })(Modifiers || (Modifiers = {}));
    var TypeModifiers;
    (function (TypeModifiers) {
        TypeModifiers[TypeModifiers["boolean"] = 1024] = "boolean";
        TypeModifiers[TypeModifiers["string"] = 2048] = "string";
        TypeModifiers[TypeModifiers["number"] = 4096] = "number";
        TypeModifiers[TypeModifiers["function"] = 8192] = "function";
        TypeModifiers[TypeModifiers["array"] = 16384] = "array";
    })(TypeModifiers || (TypeModifiers = {}));
    // #endregion Options Type Config
    // #region Schema Config
    const UNDERSCORE_SCHEMA = {
        type: 'string',
        enum: util.getEnumNames(UnderscoreOptions),
    };
    const PREFIX_SUFFIX_SCHEMA = {
        type: 'array',
        items: {
            type: 'string',
            minLength: 1,
        },
        additionalItems: false,
    };
    const MATCH_REGEX_SCHEMA = {
        type: 'object',
        properties: {
            match: { type: 'boolean' },
            regex: { type: 'string' },
        },
        required: ['match', 'regex'],
    };
    const FORMAT_OPTIONS_PROPERTIES = {
        format: {
            oneOf: [
                {
                    type: 'array',
                    items: {
                        type: 'string',
                        enum: util.getEnumNames(PredefinedFormats),
                    },
                    additionalItems: false,
                },
                {
                    type: 'null',
                },
            ],
        },
        custom: MATCH_REGEX_SCHEMA,
        leadingUnderscore: UNDERSCORE_SCHEMA,
        trailingUnderscore: UNDERSCORE_SCHEMA,
        prefix: PREFIX_SUFFIX_SCHEMA,
        suffix: PREFIX_SUFFIX_SCHEMA,
    };
    function selectorSchema(selectorString, allowType, modifiers) {
        const selector = {
            filter: {
                oneOf: [
                    {
                        type: 'string',
                        minLength: 1,
                    },
                    MATCH_REGEX_SCHEMA,
                ],
            },
            selector: {
                type: 'string',
                enum: [selectorString],
            },
        };
        if (modifiers && modifiers.length > 0) {
            selector.modifiers = {
                type: 'array',
                items: {
                    type: 'string',
                    enum: modifiers,
                },
                additionalItems: false,
            };
        }
        if (allowType) {
            selector.types = {
                type: 'array',
                items: {
                    type: 'string',
                    enum: util.getEnumNames(TypeModifiers),
                },
                additionalItems: false,
            };
        }
        return [
            {
                type: 'object',
                properties: Object.assign(Object.assign({}, FORMAT_OPTIONS_PROPERTIES), selector),
                required: ['selector', 'format'],
                additionalProperties: false,
            },
        ];
    }
    const SCHEMA = {
        type: 'array',
        items: {
            oneOf: [
                ...selectorSchema('default', false, util.getEnumNames(Modifiers)),
                ...selectorSchema('variableLike', false),
                ...selectorSchema('variable', true),
                ...selectorSchema('function', false),
                ...selectorSchema('parameter', true),
                ...selectorSchema('memberLike', false, [
                    'private',
                    'protected',
                    'public',
                    'static',
                    'readonly',
                    'abstract',
                ]),
                ...selectorSchema('property', true, [
                    'private',
                    'protected',
                    'public',
                    'static',
                    'readonly',
                    'abstract',
                ]),
                ...selectorSchema('parameterProperty', true, [
                    'private',
                    'protected',
                    'public',
                    'readonly',
                ]),
                ...selectorSchema('method', false, [
                    'private',
                    'protected',
                    'public',
                    'static',
                    'abstract',
                ]),
                ...selectorSchema('accessor', true, [
                    'private',
                    'protected',
                    'public',
                    'static',
                    'abstract',
                ]),
                ...selectorSchema('enumMember', false),
                ...selectorSchema('typeLike', false, ['abstract']),
                ...selectorSchema('class', false, ['abstract']),
                ...selectorSchema('interface', false),
                ...selectorSchema('typeAlias', false),
                ...selectorSchema('enum', false),
                ...selectorSchema('typeParameter', false),
            ],
        },
        additionalItems: false,
    };
    // #endregion Schema Config
    // This essentially mirrors ESLint's `camelcase` rule
    // note that that rule ignores leading and trailing underscores and only checks those in the middle of a variable name
    const defaultCamelCaseAllTheThingsConfig = [
        {
            selector: 'default',
            format: ['camelCase'],
            leadingUnderscore: 'allow',
            trailingUnderscore: 'allow',
        },
        {
            selector: 'variable',
            format: ['camelCase', 'UPPER_CASE'],
            leadingUnderscore: 'allow',
            trailingUnderscore: 'allow',
        },
        {
            selector: 'typeLike',
            format: ['PascalCase'],
        },
    ];
    exports.default = util.createRule({
        name: 'naming-convention',
        meta: {
            docs: {
                category: 'Variables',
                description: 'Enforces naming conventions for everything across a codebase',
                recommended: false,
                // technically only requires type checking if the user uses "type" modifiers
                requiresTypeChecking: true,
            },
            type: 'suggestion',
            messages: {
                unexpectedUnderscore: '{{type}} name {{name}} must not have a {{position}} underscore.',
                missingUnderscore: '{{type}} name {{name}} must have a {{position}} underscore',
                missingAffix: '{{type}} name {{name}} must have one of the following {{position}}es: {{affixes}}',
                satisfyCustom: '{{type}} name {{name}} must {{regexMatch}} the RegExp: {{regex}}',
                doesNotMatchFormat: '{{type}} name {{name}} must match one of the following formats: {{formats}}',
            },
            schema: SCHEMA,
        },
        defaultOptions: defaultCamelCaseAllTheThingsConfig,
        create(contextWithoutDefaults) {
            const context = contextWithoutDefaults.options
                ? contextWithoutDefaults
                : // only apply the defaults when the user provides no config
                    Object.setPrototypeOf({
                        options: defaultCamelCaseAllTheThingsConfig,
                    }, contextWithoutDefaults);
            const validators = parseOptions(context);
            function handleMember(validator, node, modifiers) {
                if (!validator) {
                    return;
                }
                const key = node.key;
                validator(key, modifiers);
            }
            function getMemberModifiers(node) {
                const modifiers = new Set();
                if (node.accessibility) {
                    modifiers.add(Modifiers[node.accessibility]);
                }
                else {
                    modifiers.add(Modifiers.public);
                }
                if (node.static) {
                    modifiers.add(Modifiers.static);
                }
                if ('readonly' in node && node.readonly) {
                    modifiers.add(Modifiers.readonly);
                }
                if (node.type === experimental_utils_1.AST_NODE_TYPES.TSAbstractClassProperty ||
                    node.type === experimental_utils_1.AST_NODE_TYPES.TSAbstractMethodDefinition) {
                    modifiers.add(Modifiers.abstract);
                }
                return modifiers;
            }
            return {
                // #region variable
                VariableDeclarator(node) {
                    const validator = validators.variable;
                    if (!validator) {
                        return;
                    }
                    const identifiers = [];
                    getIdentifiersFromPattern(node.id, identifiers);
                    identifiers.forEach(i => {
                        validator(i);
                    });
                },
                // #endregion
                // #region function
                'FunctionDeclaration, TSDeclareFunction, FunctionExpression'(node) {
                    const validator = validators.function;
                    if (!validator || node.id === null) {
                        return;
                    }
                    validator(node.id);
                },
                // #endregion function
                // #region parameter
                'FunctionDeclaration, TSDeclareFunction, FunctionExpression, ArrowFunctionExpression'(node) {
                    const validator = validators.parameter;
                    if (!validator) {
                        return;
                    }
                    node.params.forEach(param => {
                        if (param.type === experimental_utils_1.AST_NODE_TYPES.TSParameterProperty) {
                            return;
                        }
                        const identifiers = [];
                        getIdentifiersFromPattern(param, identifiers);
                        identifiers.forEach(i => {
                            validator(i);
                        });
                    });
                },
                // #endregion parameter
                // #region parameterProperty
                TSParameterProperty(node) {
                    const validator = validators.parameterProperty;
                    if (!validator) {
                        return;
                    }
                    const modifiers = getMemberModifiers(node);
                    const identifiers = [];
                    getIdentifiersFromPattern(node.parameter, identifiers);
                    identifiers.forEach(i => {
                        validator(i, modifiers);
                    });
                },
                // #endregion parameterProperty
                // #region property
                'Property[computed = false][kind = "init"][value.type != "ArrowFunctionExpression"][value.type != "FunctionExpression"][value.type != "TSEmptyBodyFunctionExpression"]'(node) {
                    const modifiers = new Set([Modifiers.public]);
                    handleMember(validators.property, node, modifiers);
                },
                ':matches(ClassProperty, TSAbstractClassProperty)[computed = false][value.type != "ArrowFunctionExpression"][value.type != "FunctionExpression"][value.type != "TSEmptyBodyFunctionExpression"]'(node) {
                    const modifiers = getMemberModifiers(node);
                    handleMember(validators.property, node, modifiers);
                },
                'TSPropertySignature[computed = false]'(node) {
                    const modifiers = new Set([Modifiers.public]);
                    if (node.readonly) {
                        modifiers.add(Modifiers.readonly);
                    }
                    handleMember(validators.property, node, modifiers);
                },
                // #endregion property
                // #region method
                [[
                    'Property[computed = false][kind = "init"][value.type = "ArrowFunctionExpression"]',
                    'Property[computed = false][kind = "init"][value.type = "FunctionExpression"]',
                    'Property[computed = false][kind = "init"][value.type = "TSEmptyBodyFunctionExpression"]',
                    'TSMethodSignature[computed = false]',
                ].join(', ')](node) {
                    const modifiers = new Set([Modifiers.public]);
                    handleMember(validators.method, node, modifiers);
                },
                [[
                    ':matches(ClassProperty, TSAbstractClassProperty)[computed = false][value.type = "ArrowFunctionExpression"]',
                    ':matches(ClassProperty, TSAbstractClassProperty)[computed = false][value.type = "FunctionExpression"]',
                    ':matches(ClassProperty, TSAbstractClassProperty)[computed = false][value.type = "TSEmptyBodyFunctionExpression"]',
                    ':matches(MethodDefinition, TSAbstractMethodDefinition)[computed = false][kind = "method"]',
                ].join(', ')](node) {
                    const modifiers = getMemberModifiers(node);
                    handleMember(validators.method, node, modifiers);
                },
                // #endregion method
                // #region accessor
                'Property[computed = false]:matches([kind = "get"], [kind = "set"])'(node) {
                    const modifiers = new Set([Modifiers.public]);
                    handleMember(validators.accessor, node, modifiers);
                },
                'MethodDefinition[computed = false]:matches([kind = "get"], [kind = "set"])'(node) {
                    const modifiers = getMemberModifiers(node);
                    handleMember(validators.accessor, node, modifiers);
                },
                // #endregion accessor
                // #region enumMember
                // computed is optional, so can't do [computed = false]
                'TSEnumMember[computed != true]'(node) {
                    const validator = validators.enumMember;
                    if (!validator) {
                        return;
                    }
                    const id = node.id;
                    validator(id);
                },
                // #endregion enumMember
                // #region class
                'ClassDeclaration, ClassExpression'(node) {
                    const validator = validators.class;
                    if (!validator) {
                        return;
                    }
                    const id = node.id;
                    if (id === null) {
                        return;
                    }
                    const modifiers = new Set();
                    if (node.abstract) {
                        modifiers.add(Modifiers.abstract);
                    }
                    validator(id, modifiers);
                },
                // #endregion class
                // #region interface
                TSInterfaceDeclaration(node) {
                    const validator = validators.interface;
                    if (!validator) {
                        return;
                    }
                    validator(node.id);
                },
                // #endregion interface
                // #region typeAlias
                TSTypeAliasDeclaration(node) {
                    const validator = validators.typeAlias;
                    if (!validator) {
                        return;
                    }
                    validator(node.id);
                },
                // #endregion typeAlias
                // #region enum
                TSEnumDeclaration(node) {
                    const validator = validators.enum;
                    if (!validator) {
                        return;
                    }
                    validator(node.id);
                },
                // #endregion enum
                // #region typeParameter
                'TSTypeParameterDeclaration > TSTypeParameter'(node) {
                    const validator = validators.typeParameter;
                    if (!validator) {
                        return;
                    }
                    validator(node.name);
                },
            };
        },
    });
    function getIdentifiersFromPattern(pattern, identifiers) {
        switch (pattern.type) {
            case experimental_utils_1.AST_NODE_TYPES.Identifier:
                identifiers.push(pattern);
                break;
            case experimental_utils_1.AST_NODE_TYPES.ArrayPattern:
                pattern.elements.forEach(element => {
                    if (element !== null) {
                        getIdentifiersFromPattern(element, identifiers);
                    }
                });
                break;
            case experimental_utils_1.AST_NODE_TYPES.ObjectPattern:
                pattern.properties.forEach(property => {
                    if (property.type === experimental_utils_1.AST_NODE_TYPES.RestElement) {
                        getIdentifiersFromPattern(property, identifiers);
                    }
                    else {
                        // this is a bit weird, but it's because ESTree doesn't have a new node type
                        // for object destructuring properties - it just reuses Property...
                        // https://github.com/estree/estree/blob/9ae284b71130d53226e7153b42f01bf819e6e657/es2015.md#L206-L211
                        // However, the parser guarantees this is safe (and there is error handling)
                        getIdentifiersFromPattern(property.value, identifiers);
                    }
                });
                break;
            case experimental_utils_1.AST_NODE_TYPES.RestElement:
                getIdentifiersFromPattern(pattern.argument, identifiers);
                break;
            case experimental_utils_1.AST_NODE_TYPES.AssignmentPattern:
                getIdentifiersFromPattern(pattern.left, identifiers);
                break;
            case experimental_utils_1.AST_NODE_TYPES.MemberExpression:
                // ignore member expressions, as the everything must already be defined
                break;
            default:
                // https://github.com/typescript-eslint/typescript-eslint/issues/1282
                // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
                throw new Error(`Unexpected pattern type ${pattern.type}`);
        }
    }
    function parseOptions(context) {
        const normalizedOptions = context.options.map(opt => normalizeOption(opt));
        const parsedOptions = util.getEnumNames(Selectors).reduce((acc, k) => {
            acc[k] = createValidator(k, context, normalizedOptions);
            return acc;
        }, {});
        return parsedOptions;
    }
    function createValidator(type, context, allConfigs) {
        // make sure the "highest priority" configs are checked first
        const selectorType = Selectors[type];
        const configs = allConfigs
            // gather all of the applicable selectors
            .filter(c => (c.selector & selectorType) !== 0 ||
            c.selector === MetaSelectors.default)
            .sort((a, b) => {
            if (a.selector === b.selector) {
                // in the event of the same selector, order by modifier weight
                // sort descending - the type modifiers are "more important"
                return b.modifierWeight - a.modifierWeight;
            }
            /*
            meta selectors will always be larger numbers than the normal selectors they contain, as they are the sum of all
            of the selectors that they contain.
            to give normal selectors a higher priority, shift them all SELECTOR_COUNT bits to the left before comparison, so
            they are instead always guaranteed to be larger than the meta selectors.
            */
            const aSelector = isMetaSelector(a.selector)
                ? a.selector
                : a.selector << SELECTOR_COUNT;
            const bSelector = isMetaSelector(b.selector)
                ? b.selector
                : b.selector << SELECTOR_COUNT;
            // sort descending - the meta selectors are "least important"
            return bSelector - aSelector;
        });
        return (node, modifiers = new Set()) => {
            var _a, _b, _c;
            const originalName = node.type === experimental_utils_1.AST_NODE_TYPES.Identifier ? node.name : `${node.value}`;
            // return will break the loop and stop checking configs
            // it is only used when the name is known to have failed or succeeded a config.
            for (const config of configs) {
                if (((_a = config.filter) === null || _a === void 0 ? void 0 : _a.regex.test(originalName)) !== ((_b = config.filter) === null || _b === void 0 ? void 0 : _b.match)) {
                    // name does not match the filter
                    continue;
                }
                if ((_c = config.modifiers) === null || _c === void 0 ? void 0 : _c.some(modifier => !modifiers.has(modifier))) {
                    // does not have the required modifiers
                    continue;
                }
                if (!isCorrectType(node, config, context)) {
                    // is not the correct type
                    continue;
                }
                let name = originalName;
                name = validateUnderscore('leading', config, name, node, originalName);
                if (name === null) {
                    // fail
                    return;
                }
                name = validateUnderscore('trailing', config, name, node, originalName);
                if (name === null) {
                    // fail
                    return;
                }
                name = validateAffix('prefix', config, name, node, originalName);
                if (name === null) {
                    // fail
                    return;
                }
                name = validateAffix('suffix', config, name, node, originalName);
                if (name === null) {
                    // fail
                    return;
                }
                if (!validateCustom(config, name, node, originalName)) {
                    // fail
                    return;
                }
                if (!validatePredefinedFormat(config, name, node, originalName)) {
                    // fail
                    return;
                }
                // it's valid for this config, so we don't need to check any more configs
                return;
            }
        };
        // centralizes the logic for formatting the report data
        function formatReportData({ affixes, formats, originalName, position, custom, }) {
            var _a;
            return {
                type: selectorTypeToMessageString(type),
                name: originalName,
                position,
                affixes: affixes === null || affixes === void 0 ? void 0 : affixes.join(', '),
                formats: formats === null || formats === void 0 ? void 0 : formats.map(f => PredefinedFormats[f]).join(', '),
                regex: (_a = custom === null || custom === void 0 ? void 0 : custom.regex) === null || _a === void 0 ? void 0 : _a.toString(),
                regexMatch: (custom === null || custom === void 0 ? void 0 : custom.match) === true
                    ? 'match'
                    : (custom === null || custom === void 0 ? void 0 : custom.match) === false
                        ? 'not match'
                        : null,
            };
        }
        /**
         * @returns the name with the underscore removed, if it is valid according to the specified underscore option, null otherwise
         */
        function validateUnderscore(position, config, name, node, originalName) {
            const option = position === 'leading'
                ? config.leadingUnderscore
                : config.trailingUnderscore;
            if (!option) {
                return name;
            }
            const hasUnderscore = position === 'leading' ? name.startsWith('_') : name.endsWith('_');
            const trimUnderscore = position === 'leading'
                ? () => name.slice(1)
                : () => name.slice(0, -1);
            switch (option) {
                case UnderscoreOptions.allow:
                    // no check - the user doesn't care if it's there or not
                    break;
                case UnderscoreOptions.forbid:
                    if (hasUnderscore) {
                        context.report({
                            node,
                            messageId: 'unexpectedUnderscore',
                            data: formatReportData({
                                originalName,
                                position,
                            }),
                        });
                        return null;
                    }
                    break;
                case UnderscoreOptions.require:
                    if (!hasUnderscore) {
                        context.report({
                            node,
                            messageId: 'missingUnderscore',
                            data: formatReportData({
                                originalName,
                                position,
                            }),
                        });
                        return null;
                    }
            }
            return hasUnderscore ? trimUnderscore() : name;
        }
        /**
         * @returns the name with the affix removed, if it is valid according to the specified affix option, null otherwise
         */
        function validateAffix(position, config, name, node, originalName) {
            const affixes = config[position];
            if (!affixes || affixes.length === 0) {
                return name;
            }
            for (const affix of affixes) {
                const hasAffix = position === 'prefix' ? name.startsWith(affix) : name.endsWith(affix);
                const trimAffix = position === 'prefix'
                    ? () => name.slice(affix.length)
                    : () => name.slice(0, -affix.length);
                if (hasAffix) {
                    // matches, so trim it and return
                    return trimAffix();
                }
            }
            context.report({
                node,
                messageId: 'missingAffix',
                data: formatReportData({
                    originalName,
                    position,
                    affixes,
                }),
            });
            return null;
        }
        /**
         * @returns true if the name is valid according to the `regex` option, false otherwise
         */
        function validateCustom(config, name, node, originalName) {
            const custom = config.custom;
            if (!custom) {
                return true;
            }
            const result = custom.regex.test(name);
            if (custom.match && result) {
                return true;
            }
            if (!custom.match && !result) {
                return true;
            }
            context.report({
                node,
                messageId: 'satisfyCustom',
                data: formatReportData({
                    originalName,
                    custom,
                }),
            });
            return false;
        }
        /**
         * @returns true if the name is valid according to the `format` option, false otherwise
         */
        function validatePredefinedFormat(config, name, node, originalName) {
            const formats = config.format;
            if (formats === null || formats.length === 0) {
                return true;
            }
            for (const format of formats) {
                const checker = PredefinedFormatToCheckFunction[format];
                if (checker(name)) {
                    return true;
                }
            }
            context.report({
                node,
                messageId: 'doesNotMatchFormat',
                data: formatReportData({
                    originalName,
                    formats,
                }),
            });
            return false;
        }
    }
    // #region Predefined Format Functions
    /*
    These format functions are taken from `tslint-consistent-codestyle/naming-convention`:
    https://github.com/ajafff/tslint-consistent-codestyle/blob/ab156cc8881bcc401236d999f4ce034b59039e81/rules/namingConventionRule.ts#L603-L645
    
    The licence for the code can be viewed here:
    https://github.com/ajafff/tslint-consistent-codestyle/blob/ab156cc8881bcc401236d999f4ce034b59039e81/LICENSE
    */
    /*
    Why not regex here? Because it's actually really, really difficult to create a regex to handle
    all of the unicode cases, and we have many non-english users that use non-english characters.
    https://gist.github.com/mathiasbynens/6334847
    */
    function isPascalCase(name) {
        return (name.length === 0 ||
            // eslint-disable-next-line @typescript-eslint/prefer-string-starts-ends-with
            (name[0] === name[0].toUpperCase() && !name.includes('_')));
    }
    function isStrictPascalCase(name) {
        return (name.length === 0 ||
            // eslint-disable-next-line @typescript-eslint/prefer-string-starts-ends-with
            (name[0] === name[0].toUpperCase() && hasStrictCamelHumps(name, true)));
    }
    function isCamelCase(name) {
        return (name.length === 0 ||
            // eslint-disable-next-line @typescript-eslint/prefer-string-starts-ends-with
            (name[0] === name[0].toLowerCase() && !name.includes('_')));
    }
    function isStrictCamelCase(name) {
        return (name.length === 0 ||
            // eslint-disable-next-line @typescript-eslint/prefer-string-starts-ends-with
            (name[0] === name[0].toLowerCase() && hasStrictCamelHumps(name, false)));
    }
    function hasStrictCamelHumps(name, isUpper) {
        function isUppercaseChar(char) {
            return char === char.toUpperCase() && char !== char.toLowerCase();
        }
        if (name.startsWith('_')) {
            return false;
        }
        for (let i = 1; i < name.length; ++i) {
            if (name[i] === '_') {
                return false;
            }
            if (isUpper === isUppercaseChar(name[i])) {
                if (isUpper) {
                    return false;
                }
            }
            else {
                isUpper = !isUpper;
            }
        }
        return true;
    }
    function isSnakeCase(name) {
        return (name.length === 0 ||
            (name === name.toLowerCase() && validateUnderscores(name)));
    }
    function isUpperCase(name) {
        return (name.length === 0 ||
            (name === name.toUpperCase() && validateUnderscores(name)));
    }
    /** Check for leading trailing and adjacent underscores */
    function validateUnderscores(name) {
        if (name.startsWith('_')) {
            return false;
        }
        let wasUnderscore = false;
        for (let i = 1; i < name.length; ++i) {
            if (name[i] === '_') {
                if (wasUnderscore) {
                    return false;
                }
                wasUnderscore = true;
            }
            else {
                wasUnderscore = false;
            }
        }
        return !wasUnderscore;
    }
    const PredefinedFormatToCheckFunction = {
        [PredefinedFormats.PascalCase]: isPascalCase,
        [PredefinedFormats.StrictPascalCase]: isStrictPascalCase,
        [PredefinedFormats.camelCase]: isCamelCase,
        [PredefinedFormats.strictCamelCase]: isStrictCamelCase,
        [PredefinedFormats.UPPER_CASE]: isUpperCase,
        [PredefinedFormats.snake_case]: isSnakeCase,
    };
    // #endregion Predefined Format Functions
    function selectorTypeToMessageString(selectorType) {
        const notCamelCase = selectorType.replace(/([A-Z])/g, ' $1');
        return notCamelCase.charAt(0).toUpperCase() + notCamelCase.slice(1);
    }
    exports.selectorTypeToMessageString = selectorTypeToMessageString;
    function isMetaSelector(selector) {
        return selector in MetaSelectors;
    }
    function normalizeOption(option) {
        var _a, _b, _c, _d, _e, _f;
        let weight = 0;
        (_a = option.modifiers) === null || _a === void 0 ? void 0 : _a.forEach(mod => {
            weight |= Modifiers[mod];
        });
        (_b = option.types) === null || _b === void 0 ? void 0 : _b.forEach(mod => {
            weight |= TypeModifiers[mod];
        });
        // give selectors with a filter the _highest_ priority
        if (option.filter) {
            weight |= 1 << 30;
        }
        return {
            // format options
            format: option.format ? option.format.map(f => PredefinedFormats[f]) : null,
            custom: option.custom
                ? {
                    regex: new RegExp(option.custom.regex),
                    match: option.custom.match,
                }
                : null,
            leadingUnderscore: option.leadingUnderscore !== undefined
                ? UnderscoreOptions[option.leadingUnderscore]
                : null,
            trailingUnderscore: option.trailingUnderscore !== undefined
                ? UnderscoreOptions[option.trailingUnderscore]
                : null,
            prefix: option.prefix && option.prefix.length > 0 ? option.prefix : null,
            suffix: option.suffix && option.suffix.length > 0 ? option.suffix : null,
            // selector options
            selector: isMetaSelector(option.selector)
                ? MetaSelectors[option.selector]
                : Selectors[option.selector],
            modifiers: (_d = (_c = option.modifiers) === null || _c === void 0 ? void 0 : _c.map(m => Modifiers[m])) !== null && _d !== void 0 ? _d : null,
            types: (_f = (_e = option.types) === null || _e === void 0 ? void 0 : _e.map(m => TypeModifiers[m])) !== null && _f !== void 0 ? _f : null,
            filter: option.filter !== undefined
                ? typeof option.filter === 'string'
                    ? { regex: new RegExp(option.filter), match: true }
                    : {
                        regex: new RegExp(option.filter.regex),
                        match: option.filter.match,
                    }
                : null,
            // calculated ordering weight based on modifiers
            modifierWeight: weight,
        };
    }
    function isCorrectType(node, config, context) {
        if (config.types === null) {
            return true;
        }
        const { esTreeNodeToTSNodeMap, program } = util.getParserServices(context);
        const checker = program.getTypeChecker();
        const tsNode = esTreeNodeToTSNodeMap.get(node);
        const type = checker
            .getTypeAtLocation(tsNode)
            // remove null and undefined from the type, as we don't care about it here
            .getNonNullableType();
        for (const allowedType of config.types) {
            switch (allowedType) {
                case TypeModifiers.array:
                    if (isAllTypesMatch(type, t => checker.isArrayType(t) || checker.isTupleType(t))) {
                        return true;
                    }
                    break;
                case TypeModifiers.function:
                    if (isAllTypesMatch(type, t => t.getCallSignatures().length > 0)) {
                        return true;
                    }
                    break;
                case TypeModifiers.boolean:
                case TypeModifiers.number:
                case TypeModifiers.string: {
                    const typeString = checker.typeToString(
                    // this will resolve things like true => boolean, 'a' => string and 1 => number
                    checker.getWidenedType(checker.getBaseTypeOfLiteralType(type)));
                    const allowedTypeString = TypeModifiers[allowedType];
                    if (typeString === allowedTypeString) {
                        return true;
                    }
                    break;
                }
            }
        }
        return false;
    }
    /**
     * @returns `true` if the type (or all union types) in the given type return true for the callback
     */
    function isAllTypesMatch(type, cb) {
        if (type.isUnion()) {
            return type.types.every(t => cb(t));
        }
        return cb(type);
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-array-constructor.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-array-constructor',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallow generic `Array` constructors',
                category: 'Stylistic Issues',
                recommended: 'error',
                extendsBaseRule: true,
            },
            fixable: 'code',
            messages: {
                useLiteral: 'The array literal notation [] is preferable.',
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            /**
             * Disallow construction of dense arrays using the Array constructor
             * @param node node to evaluate
             */
            function check(node) {
                if (node.arguments.length !== 1 &&
                    node.callee.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                    node.callee.name === 'Array' &&
                    !node.typeParameters &&
                    !util.isOptionalOptionalChain(node)) {
                    context.report({
                        node,
                        messageId: 'useLiteral',
                        fix(fixer) {
                            if (node.arguments.length === 0) {
                                return fixer.replaceText(node, '[]');
                            }
                            const fullText = context.getSourceCode().getText(node);
                            const preambleLength = node.callee.range[1] - node.range[0];
                            return fixer.replaceText(node, `[${fullText.slice(preambleLength + 1, -1)}]`);
                        },
                    });
                }
            }
            return {
                CallExpression: check,
                OptionalCallExpression: check,
                NewExpression: check,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-base-to-string.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    var Usefulness;
    (function (Usefulness) {
        Usefulness[Usefulness["Always"] = 0] = "Always";
        Usefulness["Never"] = "will";
        Usefulness["Sometimes"] = "may";
    })(Usefulness || (Usefulness = {}));
    exports.default = util.createRule({
        name: 'no-base-to-string',
        meta: {
            docs: {
                description: 'Requires that `.toString()` is only called on objects which provide useful information when stringified',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            messages: {
                baseToString: "'{{name}} {{certainty}} evaluate to '[Object object]' when stringified.",
            },
            schema: [],
            type: 'suggestion',
        },
        defaultOptions: [],
        create(context) {
            const parserServices = util.getParserServices(context);
            const typeChecker = parserServices.program.getTypeChecker();
            function checkExpression(node, type) {
                if (node.type === experimental_utils_1.AST_NODE_TYPES.Literal) {
                    return;
                }
                const certainty = collectToStringCertainty(type !== null && type !== void 0 ? type : typeChecker.getTypeAtLocation(parserServices.esTreeNodeToTSNodeMap.get(node)));
                if (certainty === Usefulness.Always) {
                    return;
                }
                context.report({
                    data: {
                        certainty,
                        name: context.getSourceCode().getText(node),
                    },
                    messageId: 'baseToString',
                    node,
                });
            }
            function collectToStringCertainty(type) {
                const toString = typeChecker.getPropertyOfType(type, 'toString');
                if (toString === undefined || toString.declarations.length === 0) {
                    return Usefulness.Always;
                }
                if (toString.declarations.every(({ parent }) => !ts.isInterfaceDeclaration(parent) || parent.name.text !== 'Object')) {
                    return Usefulness.Always;
                }
                if (!type.isUnion()) {
                    return Usefulness.Never;
                }
                for (const subType of type.types) {
                    if (collectToStringCertainty(subType) !== Usefulness.Never) {
                        return Usefulness.Sometimes;
                    }
                }
                return Usefulness.Never;
            }
            return {
                'AssignmentExpression[operator = "+="], BinaryExpression[operator = "+"]'(node) {
                    const leftType = typeChecker.getTypeAtLocation(parserServices.esTreeNodeToTSNodeMap.get(node.left));
                    const rightType = typeChecker.getTypeAtLocation(parserServices.esTreeNodeToTSNodeMap.get(node.right));
                    if (util.getTypeName(typeChecker, leftType) === 'string') {
                        checkExpression(node.right, rightType);
                    }
                    else if (util.getTypeName(typeChecker, rightType) === 'string') {
                        checkExpression(node.left, leftType);
                    }
                },
                'CallExpression > MemberExpression.callee > Identifier[name = "toString"].property'(node) {
                    const memberExpr = node.parent;
                    checkExpression(memberExpr.object);
                },
                TemplateLiteral(node) {
                    for (const expression of node.expressions) {
                        checkExpression(expression);
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-dupe-class-members.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const no_dupe_class_members_1 = __importDefault(require(231) /* eslint/lib/rules/no-dupe-class-members */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-dupe-class-members',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallow duplicate class members',
                category: 'Possible Errors',
                recommended: false,
                extendsBaseRule: true,
            },
            schema: no_dupe_class_members_1.default.meta.schema,
            messages: no_dupe_class_members_1.default.meta.messages,
        },
        defaultOptions: [],
        create(context) {
            const rules = no_dupe_class_members_1.default.create(context);
            return Object.assign(Object.assign({}, rules), { MethodDefinition(node) {
                    if (node.computed) {
                        return;
                    }
                    if (node.value.type === experimental_utils_1.AST_NODE_TYPES.TSEmptyBodyFunctionExpression) {
                        return;
                    }
                    return rules.MethodDefinition(node);
                } });
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-dynamic-delete.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const tsutils = __importStar(require(627) /* tsutils */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-dynamic-delete',
        meta: {
            docs: {
                category: 'Best Practices',
                description: 'Disallow the delete operator with computed key expressions',
                recommended: false,
            },
            fixable: 'code',
            messages: {
                dynamicDelete: 'Do not delete dynamically computed property keys.',
            },
            schema: [],
            type: 'suggestion',
        },
        defaultOptions: [],
        create(context) {
            function createFixer(member) {
                if (member.property.type === experimental_utils_1.AST_NODE_TYPES.Literal &&
                    typeof member.property.value === 'string') {
                    return createPropertyReplacement(member.property, `.${member.property.value}`);
                }
                return undefined;
            }
            return {
                'UnaryExpression[operator=delete]'(node) {
                    if (node.argument.type !== experimental_utils_1.AST_NODE_TYPES.MemberExpression ||
                        !node.argument.computed ||
                        isNecessaryDynamicAccess(diveIntoWrapperExpressions(node.argument.property))) {
                        return;
                    }
                    context.report({
                        fix: createFixer(node.argument),
                        messageId: 'dynamicDelete',
                        node: node.argument.property,
                    });
                },
            };
            function createPropertyReplacement(property, replacement) {
                return (fixer) => fixer.replaceTextRange(getTokenRange(property), replacement);
            }
            function getTokenRange(property) {
                const sourceCode = context.getSourceCode();
                return [
                    sourceCode.getTokenBefore(property).range[0],
                    sourceCode.getTokenAfter(property).range[1],
                ];
            }
        },
    });
    function diveIntoWrapperExpressions(node) {
        if (node.type === experimental_utils_1.AST_NODE_TYPES.UnaryExpression) {
            return diveIntoWrapperExpressions(node.argument);
        }
        return node;
    }
    function isNecessaryDynamicAccess(property) {
        if (property.type !== experimental_utils_1.AST_NODE_TYPES.Literal) {
            return false;
        }
        if (typeof property.value === 'number') {
            return true;
        }
        return (typeof property.value === 'string' &&
            !tsutils.isValidPropertyAccess(property.value));
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-empty-function.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const no_empty_function_1 = __importDefault(require(240) /* eslint/lib/rules/no-empty-function */);
    const util = __importStar(require(622) /* ../util */);
    const schema = util.deepMerge(Array.isArray(no_empty_function_1.default.meta.schema)
        ? no_empty_function_1.default.meta.schema[0]
        : no_empty_function_1.default.meta.schema, {
        properties: {
            allow: {
                items: {
                    enum: [
                        'functions',
                        'arrowFunctions',
                        'generatorFunctions',
                        'methods',
                        'generatorMethods',
                        'getters',
                        'setters',
                        'constructors',
                        'private-constructors',
                        'protected-constructors',
                    ],
                },
            },
        },
    });
    exports.default = util.createRule({
        name: 'no-empty-function',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallow empty functions',
                category: 'Best Practices',
                recommended: 'error',
                extendsBaseRule: true,
            },
            schema: [schema],
            messages: no_empty_function_1.default.meta.messages,
        },
        defaultOptions: [
            {
                allow: [],
            },
        ],
        create(context, [{ allow = [] }]) {
            const rules = no_empty_function_1.default.create(context);
            const isAllowedProtectedConstructors = allow.includes('protected-constructors');
            const isAllowedPrivateConstructors = allow.includes('private-constructors');
            /**
             * Check if the method body is empty
             * @param node the node to be validated
             * @returns true if the body is empty
             * @private
             */
            function isBodyEmpty(node) {
                return !node.body || node.body.body.length === 0;
            }
            /**
             * Check if method has parameter properties
             * @param node the node to be validated
             * @returns true if the body has parameter properties
             * @private
             */
            function hasParameterProperties(node) {
                return (node.params &&
                    node.params.some(param => param.type === experimental_utils_1.AST_NODE_TYPES.TSParameterProperty));
            }
            /**
             * @param node the node to be validated
             * @returns true if the constructor is allowed to be empty
             * @private
             */
            function isAllowedEmptyConstructor(node) {
                const parent = node.parent;
                if (isBodyEmpty(node) &&
                    (parent === null || parent === void 0 ? void 0 : parent.type) === experimental_utils_1.AST_NODE_TYPES.MethodDefinition &&
                    parent.kind === 'constructor') {
                    const { accessibility } = parent;
                    return (
                    // allow protected constructors
                    (accessibility === 'protected' && isAllowedProtectedConstructors) ||
                        // allow private constructors
                        (accessibility === 'private' && isAllowedPrivateConstructors) ||
                        // allow constructors which have parameter properties
                        hasParameterProperties(node));
                }
                return false;
            }
            return Object.assign(Object.assign({}, rules), { FunctionExpression(node) {
                    if (isAllowedEmptyConstructor(node)) {
                        return;
                    }
                    rules.FunctionExpression(node);
                } });
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-empty-interface.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-empty-interface',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallow the declaration of empty interfaces',
                category: 'Best Practices',
                recommended: 'error',
            },
            fixable: 'code',
            messages: {
                noEmpty: 'An empty interface is equivalent to `{}`.',
                noEmptyWithSuper: 'An interface declaring no members is equivalent to its supertype.',
            },
            schema: [
                {
                    type: 'object',
                    additionalProperties: false,
                    properties: {
                        allowSingleExtends: {
                            type: 'boolean',
                        },
                    },
                },
            ],
        },
        defaultOptions: [
            {
                allowSingleExtends: false,
            },
        ],
        create(context, [{ allowSingleExtends }]) {
            return {
                TSInterfaceDeclaration(node) {
                    const sourceCode = context.getSourceCode();
                    if (node.body.body.length !== 0) {
                        // interface contains members --> Nothing to report
                        return;
                    }
                    const extend = node.extends;
                    if (!extend || extend.length === 0) {
                        context.report({
                            node: node.id,
                            messageId: 'noEmpty',
                        });
                    }
                    else if (extend.length === 1) {
                        // interface extends exactly 1 interface --> Report depending on rule setting
                        if (!allowSingleExtends) {
                            context.report({
                                node: node.id,
                                messageId: 'noEmptyWithSuper',
                                fix: fixer => fixer.replaceText(node, `type ${sourceCode.getText(node.id)} = ${sourceCode.getText(extend[0])}`),
                            });
                        }
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-explicit-any.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-explicit-any',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallow usage of the `any` type',
                category: 'Best Practices',
                recommended: 'warn',
            },
            fixable: 'code',
            messages: {
                unexpectedAny: 'Unexpected any. Specify a different type.',
                suggestUnknown: 'Use `unknown` instead, this will force you to explicitly, and safely assert the type is correct.',
                suggestNever: "Use `never` instead, this is useful when instantiating generic type parameters that you don't need to know the type of.",
            },
            schema: [
                {
                    type: 'object',
                    additionalProperties: false,
                    properties: {
                        fixToUnknown: {
                            type: 'boolean',
                        },
                        ignoreRestArgs: {
                            type: 'boolean',
                        },
                    },
                },
            ],
        },
        defaultOptions: [
            {
                fixToUnknown: false,
                ignoreRestArgs: false,
            },
        ],
        create(context, [{ ignoreRestArgs, fixToUnknown }]) {
            /**
             * Checks if the node is an arrow function, function declaration or function expression
             * @param node the node to be validated.
             * @returns true if the node is an arrow function, function declaration, function expression, function type, or call signature
             * @private
             */
            function isNodeValidFunction(node) {
                return [
                    experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression,
                    experimental_utils_1.AST_NODE_TYPES.FunctionDeclaration,
                    experimental_utils_1.AST_NODE_TYPES.FunctionExpression,
                    experimental_utils_1.AST_NODE_TYPES.TSFunctionType,
                    experimental_utils_1.AST_NODE_TYPES.TSCallSignatureDeclaration,
                ].includes(node.type);
            }
            /**
             * Checks if the node is a rest element child node of a function
             * @param node the node to be validated.
             * @returns true if the node is a rest element child node of a function
             * @private
             */
            function isNodeRestElementInFunction(node) {
                return (node.type === experimental_utils_1.AST_NODE_TYPES.RestElement &&
                    typeof node.parent !== 'undefined' &&
                    isNodeValidFunction(node.parent));
            }
            /**
             * Checks if the node is a TSTypeOperator node with a readonly operator
             * @param node the node to be validated.
             * @returns true if the node is a TSTypeOperator node with a readonly operator
             * @private
             */
            function isNodeReadonlyTSTypeOperator(node) {
                return (node.type === experimental_utils_1.AST_NODE_TYPES.TSTypeOperator &&
                    node.operator === 'readonly');
            }
            /**
             * Checks if the node is a TSTypeReference node with an Array identifier
             * @param node the node to be validated.
             * @returns true if the node is a TSTypeReference node with an Array identifier
             * @private
             */
            function isNodeValidArrayTSTypeReference(node) {
                return (node.type === experimental_utils_1.AST_NODE_TYPES.TSTypeReference &&
                    typeof node.typeName !== 'undefined' &&
                    node.typeName.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                    ['Array', 'ReadonlyArray'].includes(node.typeName.name));
            }
            /**
             * Checks if the node is a valid TSTypeOperator or TSTypeReference node
             * @param node the node to be validated.
             * @returns true if the node is a valid TSTypeOperator or TSTypeReference node
             * @private
             */
            function isNodeValidTSType(node) {
                return (isNodeReadonlyTSTypeOperator(node) ||
                    isNodeValidArrayTSTypeReference(node));
            }
            /**
             * Checks if the great grand-parent node is a RestElement node in a function
             * @param node the node to be validated.
             * @returns true if the great grand-parent node is a RestElement node in a function
             * @private
             */
            function isGreatGrandparentRestElement(node) {
                return (typeof node.parent !== 'undefined' &&
                    typeof node.parent.parent !== 'undefined' &&
                    typeof node.parent.parent.parent !== 'undefined' &&
                    isNodeRestElementInFunction(node.parent.parent.parent));
            }
            /**
             * Checks if the great great grand-parent node is a valid RestElement node in a function
             * @param node the node to be validated.
             * @returns true if the great great grand-parent node is a valid RestElement node in a function
             * @private
             */
            function isGreatGreatGrandparentRestElement(node) {
                return (typeof node.parent !== 'undefined' &&
                    typeof node.parent.parent !== 'undefined' &&
                    isNodeValidTSType(node.parent.parent) &&
                    typeof node.parent.parent.parent !== 'undefined' &&
                    typeof node.parent.parent.parent.parent !== 'undefined' &&
                    isNodeRestElementInFunction(node.parent.parent.parent.parent));
            }
            /**
             * Checks if the great grand-parent or the great great grand-parent node is a RestElement node
             * @param node the node to be validated.
             * @returns true if the great grand-parent or the great great grand-parent node is a RestElement node
             * @private
             */
            function isNodeDescendantOfRestElementInFunction(node) {
                return (isGreatGrandparentRestElement(node) ||
                    isGreatGreatGrandparentRestElement(node));
            }
            return {
                TSAnyKeyword(node) {
                    if (ignoreRestArgs && isNodeDescendantOfRestElementInFunction(node)) {
                        return;
                    }
                    const fixOrSuggest = {
                        fix: null,
                        suggest: [
                            {
                                messageId: 'suggestUnknown',
                                fix(fixer) {
                                    return fixer.replaceText(node, 'unknown');
                                },
                            },
                            {
                                messageId: 'suggestNever',
                                fix(fixer) {
                                    return fixer.replaceText(node, 'never');
                                },
                            },
                        ],
                    };
                    if (fixToUnknown) {
                        fixOrSuggest.fix = (fixer => fixer.replaceText(node, 'unknown'));
                    }
                    context.report(Object.assign({ node, messageId: 'unexpectedAny' }, fixOrSuggest));
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-extraneous-class.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-extraneous-class',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Forbids the use of classes as namespaces',
                category: 'Best Practices',
                recommended: false,
            },
            schema: [
                {
                    type: 'object',
                    additionalProperties: false,
                    properties: {
                        allowConstructorOnly: {
                            type: 'boolean',
                        },
                        allowEmpty: {
                            type: 'boolean',
                        },
                        allowStaticOnly: {
                            type: 'boolean',
                        },
                        allowWithDecorator: {
                            type: 'boolean',
                        },
                    },
                },
            ],
            messages: {
                empty: 'Unexpected empty class.',
                onlyStatic: 'Unexpected class with only static properties.',
                onlyConstructor: 'Unexpected class with only a constructor.',
            },
        },
        defaultOptions: [
            {
                allowConstructorOnly: false,
                allowEmpty: false,
                allowStaticOnly: false,
                allowWithDecorator: false,
            },
        ],
        create(context, [{ allowConstructorOnly, allowEmpty, allowStaticOnly, allowWithDecorator }]) {
            const isAllowWithDecorator = (node) => {
                return !!(allowWithDecorator &&
                    node &&
                    node.decorators &&
                    node.decorators.length);
            };
            return {
                ClassBody(node) {
                    const parent = node.parent;
                    if (!parent || parent.superClass) {
                        return;
                    }
                    const reportNode = 'id' in parent && parent.id ? parent.id : parent;
                    if (node.body.length === 0) {
                        if (allowEmpty || isAllowWithDecorator(parent)) {
                            return;
                        }
                        context.report({
                            node: reportNode,
                            messageId: 'empty',
                        });
                        return;
                    }
                    let onlyStatic = true;
                    let onlyConstructor = true;
                    for (const prop of node.body) {
                        if ('kind' in prop && prop.kind === 'constructor') {
                            if (prop.value.params.some(param => param.type === experimental_utils_1.AST_NODE_TYPES.TSParameterProperty)) {
                                onlyConstructor = false;
                                onlyStatic = false;
                            }
                        }
                        else {
                            onlyConstructor = false;
                            if ('static' in prop && !prop.static) {
                                onlyStatic = false;
                            }
                        }
                        if (!(onlyStatic || onlyConstructor)) {
                            break;
                        }
                    }
                    if (onlyConstructor) {
                        if (!allowConstructorOnly) {
                            context.report({
                                node: reportNode,
                                messageId: 'onlyConstructor',
                            });
                        }
                        return;
                    }
                    if (onlyStatic && !allowStaticOnly) {
                        context.report({
                            node: reportNode,
                            messageId: 'onlyStatic',
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-extra-non-null-assertion.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-extra-non-null-assertion',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallow extra non-null assertion',
                category: 'Stylistic Issues',
                recommended: false,
            },
            fixable: 'code',
            schema: [],
            messages: {
                noExtraNonNullAssertion: 'Forbidden extra non-null assertion.',
            },
        },
        defaultOptions: [],
        create(context) {
            function checkExtraNonNullAssertion(node) {
                context.report({
                    node,
                    messageId: 'noExtraNonNullAssertion',
                    fix(fixer) {
                        return fixer.removeRange([node.range[1] - 1, node.range[1]]);
                    },
                });
            }
            return {
                'TSNonNullExpression > TSNonNullExpression': checkExtraNonNullAssertion,
                'OptionalMemberExpression > TSNonNullExpression': checkExtraNonNullAssertion,
                'OptionalCallExpression > TSNonNullExpression.callee': checkExtraNonNullAssertion,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-extra-parens.js */ function _(require, module, exports) {
    // any is required to work around manipulating the AST in weird ways
    /* eslint-disable @typescript-eslint/no-explicit-any */
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const no_extra_parens_1 = __importDefault(require(249) /* eslint/lib/rules/no-extra-parens */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-extra-parens',
        meta: {
            type: 'layout',
            docs: {
                description: 'Disallow unnecessary parentheses',
                category: 'Possible Errors',
                recommended: false,
                extendsBaseRule: true,
            },
            fixable: 'code',
            schema: no_extra_parens_1.default.meta.schema,
            messages: no_extra_parens_1.default.meta.messages,
        },
        defaultOptions: ['all'],
        create(context) {
            const rules = no_extra_parens_1.default.create(context);
            function binaryExp(node) {
                const rule = rules.BinaryExpression;
                // makes the rule think it should skip the left or right
                const isLeftTypeAssertion = util.isTypeAssertion(node.left);
                const isRightTypeAssertion = util.isTypeAssertion(node.right);
                if (isLeftTypeAssertion && isRightTypeAssertion) {
                    return; // ignore
                }
                if (isLeftTypeAssertion) {
                    return rule(Object.assign(Object.assign({}, node), { left: Object.assign(Object.assign({}, node.left), { type: experimental_utils_1.AST_NODE_TYPES.SequenceExpression }) }));
                }
                if (isRightTypeAssertion) {
                    return rule(Object.assign(Object.assign({}, node), { right: Object.assign(Object.assign({}, node.right), { type: experimental_utils_1.AST_NODE_TYPES.SequenceExpression }) }));
                }
                return rule(node);
            }
            function callExp(node) {
                const rule = rules.CallExpression;
                if (util.isTypeAssertion(node.callee)) {
                    // reduces the precedence of the node so the rule thinks it needs to be wrapped
                    return rule(Object.assign(Object.assign({}, node), { callee: Object.assign(Object.assign({}, node.callee), { type: experimental_utils_1.AST_NODE_TYPES.SequenceExpression }) }));
                }
                return rule(node);
            }
            function unaryUpdateExpression(node) {
                const rule = rules.UnaryExpression;
                if (util.isTypeAssertion(node.argument)) {
                    // reduces the precedence of the node so the rule thinks it needs to be wrapped
                    return rule(Object.assign(Object.assign({}, node), { argument: Object.assign(Object.assign({}, node.argument), { type: experimental_utils_1.AST_NODE_TYPES.SequenceExpression }) }));
                }
                return rule(node);
            }
            const overrides = {
                // ArrayExpression
                ArrowFunctionExpression(node) {
                    if (!util.isTypeAssertion(node.body)) {
                        return rules.ArrowFunctionExpression(node);
                    }
                },
                // AssignmentExpression
                // AwaitExpression
                BinaryExpression: binaryExp,
                CallExpression: callExp,
                // ClassDeclaration
                // ClassExpression
                ConditionalExpression(node) {
                    // reduces the precedence of the node so the rule thinks it needs to be wrapped
                    if (util.isTypeAssertion(node.test)) {
                        return rules.ConditionalExpression(Object.assign(Object.assign({}, node), { test: Object.assign(Object.assign({}, node.test), { type: experimental_utils_1.AST_NODE_TYPES.SequenceExpression }) }));
                    }
                    if (util.isTypeAssertion(node.consequent)) {
                        return rules.ConditionalExpression(Object.assign(Object.assign({}, node), { consequent: Object.assign(Object.assign({}, node.consequent), { type: experimental_utils_1.AST_NODE_TYPES.SequenceExpression }) }));
                    }
                    if (util.isTypeAssertion(node.alternate)) {
                        // reduces the precedence of the node so the rule thinks it needs to be rapped
                        return rules.ConditionalExpression(Object.assign(Object.assign({}, node), { alternate: Object.assign(Object.assign({}, node.alternate), { type: experimental_utils_1.AST_NODE_TYPES.SequenceExpression }) }));
                    }
                    return rules.ConditionalExpression(node);
                },
                // DoWhileStatement
                'ForInStatement, ForOfStatement'(node) {
                    if (util.isTypeAssertion(node.right)) {
                        // makes the rule skip checking of the right
                        return rules['ForInStatement, ForOfStatement'](Object.assign(Object.assign({}, node), { type: experimental_utils_1.AST_NODE_TYPES.ForOfStatement, right: Object.assign(Object.assign({}, node.right), { type: experimental_utils_1.AST_NODE_TYPES.SequenceExpression }) }));
                    }
                    return rules['ForInStatement, ForOfStatement'](node);
                },
                ForStatement(node) {
                    // make the rule skip the piece by removing it entirely
                    if (node.init && util.isTypeAssertion(node.init)) {
                        return rules.ForStatement(Object.assign(Object.assign({}, node), { init: null }));
                    }
                    if (node.test && util.isTypeAssertion(node.test)) {
                        return rules.ForStatement(Object.assign(Object.assign({}, node), { test: null }));
                    }
                    if (node.update && util.isTypeAssertion(node.update)) {
                        return rules.ForStatement(Object.assign(Object.assign({}, node), { update: null }));
                    }
                    return rules.ForStatement(node);
                },
                'ForStatement > *.init:exit'(node) {
                    if (!util.isTypeAssertion(node)) {
                        return rules['ForStatement > *.init:exit'](node);
                    }
                },
                // IfStatement
                LogicalExpression: binaryExp,
                MemberExpression(node) {
                    if (util.isTypeAssertion(node.object)) {
                        // reduces the precedence of the node so the rule thinks it needs to be wrapped
                        return rules.MemberExpression(Object.assign(Object.assign({}, node), { object: Object.assign(Object.assign({}, node.object), { type: experimental_utils_1.AST_NODE_TYPES.SequenceExpression }) }));
                    }
                    return rules.MemberExpression(node);
                },
                NewExpression: callExp,
                // ObjectExpression
                // ReturnStatement
                // SequenceExpression
                SpreadElement(node) {
                    if (!util.isTypeAssertion(node.argument)) {
                        return rules.SpreadElement(node);
                    }
                },
                SwitchCase(node) {
                    if (node.test && !util.isTypeAssertion(node.test)) {
                        return rules.SwitchCase(node);
                    }
                },
                // SwitchStatement
                ThrowStatement(node) {
                    if (node.argument && !util.isTypeAssertion(node.argument)) {
                        return rules.ThrowStatement(node);
                    }
                },
                UnaryExpression: unaryUpdateExpression,
                UpdateExpression: unaryUpdateExpression,
                // VariableDeclarator
                // WhileStatement
                // WithStatement - i'm not going to even bother implementing this terrible and never used feature
                YieldExpression(node) {
                    if (node.argument && !util.isTypeAssertion(node.argument)) {
                        return rules.YieldExpression(node);
                    }
                },
            };
            return Object.assign({}, rules, overrides);
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-extra-semi.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const no_extra_semi_1 = __importDefault(require(251) /* eslint/lib/rules/no-extra-semi */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-extra-semi',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallow unnecessary semicolons',
                category: 'Possible Errors',
                recommended: false,
                extendsBaseRule: true,
            },
            fixable: 'code',
            schema: no_extra_semi_1.default.meta.schema,
            messages: no_extra_semi_1.default.meta.messages,
        },
        defaultOptions: [],
        create(context) {
            const rules = no_extra_semi_1.default.create(context);
            return Object.assign(Object.assign({}, rules), { ClassProperty(node) {
                    rules.MethodDefinition(node);
                } });
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-floating-promises.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const tsutils = __importStar(require(627) /* tsutils */);
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-floating-promises',
        meta: {
            docs: {
                description: 'Requires Promise-like values to be handled appropriately',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            messages: {
                floating: 'Promises must be handled appropriately',
                floatingVoid: 'Promises must be handled appropriately' +
                    ' or explicitly marked as ignored with the `void` operator',
                floatingFixVoid: 'Add void operator to ignore',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        ignoreVoid: { type: 'boolean' },
                    },
                    additionalProperties: false,
                },
            ],
            type: 'problem',
        },
        defaultOptions: [
            {
                ignoreVoid: false,
            },
        ],
        create(context, [options]) {
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            const sourceCode = context.getSourceCode();
            return {
                ExpressionStatement(node) {
                    const { expression } = parserServices.esTreeNodeToTSNodeMap.get(node);
                    if (isUnhandledPromise(checker, expression)) {
                        if (options.ignoreVoid) {
                            context.report({
                                node,
                                messageId: 'floatingVoid',
                                suggest: [
                                    {
                                        messageId: 'floatingFixVoid',
                                        fix(fixer) {
                                            let code = sourceCode.getText(node);
                                            code = `void ${code}`;
                                            return fixer.replaceText(node, code);
                                        },
                                    },
                                ],
                            });
                        }
                        else {
                            context.report({
                                node,
                                messageId: 'floating',
                            });
                        }
                    }
                },
            };
            function isUnhandledPromise(checker, node) {
                // First, check expressions whose resulting types may not be promise-like
                if (ts.isBinaryExpression(node) &&
                    node.operatorToken.kind === ts.SyntaxKind.CommaToken) {
                    // Any child in a comma expression could return a potentially unhandled
                    // promise, so we check them all regardless of whether the final returned
                    // value is promise-like.
                    return (isUnhandledPromise(checker, node.left) ||
                        isUnhandledPromise(checker, node.right));
                }
                if (ts.isVoidExpression(node) && !options.ignoreVoid) {
                    // Similarly, a `void` expression always returns undefined, so we need to
                    // see what's inside it without checking the type of the overall expression.
                    return isUnhandledPromise(checker, node.expression);
                }
                // Check the type. At this point it can't be unhandled if it isn't a promise
                if (!isPromiseLike(checker, node)) {
                    return false;
                }
                if (ts.isCallExpression(node)) {
                    // If the outer expression is a call, it must be either a `.then()` or
                    // `.catch()` that handles the promise.
                    return (!isPromiseCatchCallWithHandler(node) &&
                        !isPromiseThenCallWithRejectionHandler(node) &&
                        !isPromiseFinallyCallWithHandler(node));
                }
                else if (ts.isConditionalExpression(node)) {
                    // We must be getting the promise-like value from one of the branches of the
                    // ternary. Check them directly.
                    return (isUnhandledPromise(checker, node.whenFalse) ||
                        isUnhandledPromise(checker, node.whenTrue));
                }
                else if (ts.isPropertyAccessExpression(node) ||
                    ts.isIdentifier(node) ||
                    ts.isNewExpression(node)) {
                    // If it is just a property access chain or a `new` call (e.g. `foo.bar` or
                    // `new Promise()`), the promise is not handled because it doesn't have the
                    // necessary then/catch call at the end of the chain.
                    return true;
                }
                // We conservatively return false for all other types of expressions because
                // we don't want to accidentally fail if the promise is handled internally but
                // we just can't tell.
                return false;
            }
        },
    });
    // Modified from tsutils.isThenable() to only consider thenables which can be
    // rejected/caught via a second parameter. Original source (MIT licensed):
    //
    //   https://github.com/ajafff/tsutils/blob/49d0d31050b44b81e918eae4fbaf1dfe7b7286af/util/type.ts#L95-L125
    function isPromiseLike(checker, node) {
        const type = checker.getTypeAtLocation(node);
        for (const ty of tsutils.unionTypeParts(checker.getApparentType(type))) {
            const then = ty.getProperty('then');
            if (then === undefined) {
                continue;
            }
            const thenType = checker.getTypeOfSymbolAtLocation(then, node);
            if (hasMatchingSignature(thenType, signature => signature.parameters.length >= 2 &&
                isFunctionParam(checker, signature.parameters[0], node) &&
                isFunctionParam(checker, signature.parameters[1], node))) {
                return true;
            }
        }
        return false;
    }
    function hasMatchingSignature(type, matcher) {
        for (const t of tsutils.unionTypeParts(type)) {
            if (t.getCallSignatures().some(matcher)) {
                return true;
            }
        }
        return false;
    }
    function isFunctionParam(checker, param, node) {
        const type = checker.getApparentType(checker.getTypeOfSymbolAtLocation(param, node));
        for (const t of tsutils.unionTypeParts(type)) {
            if (t.getCallSignatures().length !== 0) {
                return true;
            }
        }
        return false;
    }
    function isPromiseCatchCallWithHandler(expression) {
        return (tsutils.isPropertyAccessExpression(expression.expression) &&
            expression.expression.name.text === 'catch' &&
            expression.arguments.length >= 1);
    }
    function isPromiseThenCallWithRejectionHandler(expression) {
        return (tsutils.isPropertyAccessExpression(expression.expression) &&
            expression.expression.name.text === 'then' &&
            expression.arguments.length >= 2);
    }
    function isPromiseFinallyCallWithHandler(expression) {
        return (tsutils.isPropertyAccessExpression(expression.expression) &&
            expression.expression.name.text === 'finally' &&
            expression.arguments.length >= 1);
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-for-in-array.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-for-in-array',
        meta: {
            docs: {
                description: 'Disallow iterating over an array with a for-in loop',
                category: 'Best Practices',
                recommended: 'error',
                requiresTypeChecking: true,
            },
            messages: {
                forInViolation: 'For-in loops over arrays are forbidden. Use for-of or array.forEach instead.',
            },
            schema: [],
            type: 'problem',
        },
        defaultOptions: [],
        create(context) {
            return {
                ForInStatement(node) {
                    const parserServices = util.getParserServices(context);
                    const checker = parserServices.program.getTypeChecker();
                    const originalNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                    const type = checker.getTypeAtLocation(originalNode.expression);
                    if ((typeof type.symbol !== 'undefined' &&
                        type.symbol.name === 'Array') ||
                        (type.flags & ts.TypeFlags.StringLike) !== 0) {
                        context.report({
                            node,
                            messageId: 'forInViolation',
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-implied-eval.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = __importStar(require(44) /* typescript */);
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const tsutils = __importStar(require(627) /* tsutils */);
    const util = __importStar(require(622) /* ../util */);
    const FUNCTION_CONSTRUCTOR = 'Function';
    const EVAL_LIKE_METHODS = new Set([
        'setImmediate',
        'setInterval',
        'setTimeout',
        'execScript',
    ]);
    exports.default = util.createRule({
        name: 'no-implied-eval',
        meta: {
            docs: {
                description: 'Disallow the use of `eval()`-like methods',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            messages: {
                noImpliedEvalError: 'Implied eval. Consider passing a function.',
                noFunctionConstructor: 'Implied eval. Do not use the Function constructor to create functions.',
            },
            schema: [],
            type: 'suggestion',
        },
        defaultOptions: [],
        create(context) {
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            function getCalleeName(node) {
                if (node.type === experimental_utils_1.AST_NODE_TYPES.Identifier) {
                    return node.name;
                }
                if (node.type === experimental_utils_1.AST_NODE_TYPES.MemberExpression &&
                    node.object.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                    node.object.name === 'window') {
                    if (node.property.type === experimental_utils_1.AST_NODE_TYPES.Identifier) {
                        return node.property.name;
                    }
                    if (node.property.type === experimental_utils_1.AST_NODE_TYPES.Literal &&
                        typeof node.property.value === 'string') {
                        return node.property.value;
                    }
                }
                return null;
            }
            function isFunctionType(node) {
                const tsNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                const type = checker.getTypeAtLocation(tsNode);
                const symbol = type.getSymbol();
                if (symbol &&
                    tsutils.isSymbolFlagSet(symbol, ts.SymbolFlags.Function | ts.SymbolFlags.Method)) {
                    return true;
                }
                const signatures = checker.getSignaturesOfType(type, ts.SignatureKind.Call);
                return signatures.length > 0;
            }
            function isFunction(node) {
                switch (node.type) {
                    case experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression:
                    case experimental_utils_1.AST_NODE_TYPES.FunctionDeclaration:
                    case experimental_utils_1.AST_NODE_TYPES.FunctionExpression:
                        return true;
                    case experimental_utils_1.AST_NODE_TYPES.MemberExpression:
                    case experimental_utils_1.AST_NODE_TYPES.Identifier:
                        return isFunctionType(node);
                    case experimental_utils_1.AST_NODE_TYPES.CallExpression:
                        return ((node.callee.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                            node.callee.name === 'bind') ||
                            isFunctionType(node));
                    default:
                        return false;
                }
            }
            function checkImpliedEval(node) {
                const calleeName = getCalleeName(node.callee);
                if (calleeName === null) {
                    return;
                }
                if (calleeName === FUNCTION_CONSTRUCTOR) {
                    context.report({ node, messageId: 'noFunctionConstructor' });
                    return;
                }
                if (node.arguments.length === 0) {
                    return;
                }
                const [handler] = node.arguments;
                if (EVAL_LIKE_METHODS.has(calleeName) && !isFunction(handler)) {
                    context.report({ node: handler, messageId: 'noImpliedEvalError' });
                }
            }
            return {
                NewExpression: checkImpliedEval,
                CallExpression: checkImpliedEval,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-inferrable-types.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-inferrable-types',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallows explicit type declarations for variables or parameters initialized to a number, string, or boolean',
                category: 'Best Practices',
                recommended: 'error',
            },
            fixable: 'code',
            messages: {
                noInferrableType: 'Type {{type}} trivially inferred from a {{type}} literal, remove type annotation.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        ignoreParameters: {
                            type: 'boolean',
                        },
                        ignoreProperties: {
                            type: 'boolean',
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            {
                ignoreParameters: false,
                ignoreProperties: false,
            },
        ],
        create(context, [{ ignoreParameters, ignoreProperties }]) {
            function isFunctionCall(init, callName) {
                return ((init.type === experimental_utils_1.AST_NODE_TYPES.CallExpression ||
                    init.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression) &&
                    init.callee.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                    init.callee.name === callName);
            }
            function isLiteral(init, typeName) {
                return (init.type === experimental_utils_1.AST_NODE_TYPES.Literal && typeof init.value === typeName);
            }
            function isIdentifier(init, ...names) {
                return (init.type === experimental_utils_1.AST_NODE_TYPES.Identifier && names.includes(init.name));
            }
            function hasUnaryPrefix(init, ...operators) {
                return (init.type === experimental_utils_1.AST_NODE_TYPES.UnaryExpression &&
                    operators.includes(init.operator));
            }
            const keywordMap = {
                [experimental_utils_1.AST_NODE_TYPES.TSBigIntKeyword]: 'bigint',
                [experimental_utils_1.AST_NODE_TYPES.TSBooleanKeyword]: 'boolean',
                [experimental_utils_1.AST_NODE_TYPES.TSNumberKeyword]: 'number',
                [experimental_utils_1.AST_NODE_TYPES.TSNullKeyword]: 'null',
                [experimental_utils_1.AST_NODE_TYPES.TSStringKeyword]: 'string',
                [experimental_utils_1.AST_NODE_TYPES.TSSymbolKeyword]: 'symbol',
                [experimental_utils_1.AST_NODE_TYPES.TSUndefinedKeyword]: 'undefined',
            };
            /**
             * Returns whether a node has an inferrable value or not
             */
            function isInferrable(annotation, init) {
                switch (annotation.type) {
                    case experimental_utils_1.AST_NODE_TYPES.TSBigIntKeyword: {
                        // note that bigint cannot have + prefixed to it
                        const unwrappedInit = hasUnaryPrefix(init, '-')
                            ? init.argument
                            : init;
                        return (isFunctionCall(unwrappedInit, 'BigInt') ||
                            unwrappedInit.type === experimental_utils_1.AST_NODE_TYPES.BigIntLiteral);
                    }
                    case experimental_utils_1.AST_NODE_TYPES.TSBooleanKeyword:
                        return (hasUnaryPrefix(init, '!') ||
                            // eslint-disable-next-line @typescript-eslint/internal/prefer-ast-types-enum
                            isFunctionCall(init, 'Boolean') ||
                            isLiteral(init, 'boolean'));
                    case experimental_utils_1.AST_NODE_TYPES.TSNumberKeyword: {
                        const unwrappedInit = hasUnaryPrefix(init, '+', '-')
                            ? init.argument
                            : init;
                        return (isIdentifier(unwrappedInit, 'Infinity', 'NaN') ||
                            isFunctionCall(unwrappedInit, 'Number') ||
                            isLiteral(unwrappedInit, 'number'));
                    }
                    case experimental_utils_1.AST_NODE_TYPES.TSNullKeyword:
                        return init.type === experimental_utils_1.AST_NODE_TYPES.Literal && init.value === null;
                    case experimental_utils_1.AST_NODE_TYPES.TSStringKeyword:
                        return (
                        // eslint-disable-next-line @typescript-eslint/internal/prefer-ast-types-enum
                        isFunctionCall(init, 'String') ||
                            isLiteral(init, 'string') ||
                            init.type === experimental_utils_1.AST_NODE_TYPES.TemplateLiteral);
                    case experimental_utils_1.AST_NODE_TYPES.TSSymbolKeyword:
                        return isFunctionCall(init, 'Symbol');
                    case experimental_utils_1.AST_NODE_TYPES.TSTypeReference: {
                        if (annotation.typeName.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                            annotation.typeName.name === 'RegExp') {
                            const isRegExpLiteral = init.type === experimental_utils_1.AST_NODE_TYPES.Literal &&
                                init.value instanceof RegExp;
                            const isRegExpNewCall = init.type === experimental_utils_1.AST_NODE_TYPES.NewExpression &&
                                init.callee.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                                init.callee.name === 'RegExp';
                            const isRegExpCall = isFunctionCall(init, 'RegExp');
                            return isRegExpLiteral || isRegExpCall || isRegExpNewCall;
                        }
                        return false;
                    }
                    case experimental_utils_1.AST_NODE_TYPES.TSUndefinedKeyword:
                        return (hasUnaryPrefix(init, 'void') || isIdentifier(init, 'undefined'));
                }
                return false;
            }
            /**
             * Reports an inferrable type declaration, if any
             */
            function reportInferrableType(node, typeNode, initNode) {
                if (!typeNode || !initNode || !typeNode.typeAnnotation) {
                    return;
                }
                if (!isInferrable(typeNode.typeAnnotation, initNode)) {
                    return;
                }
                const type = typeNode.typeAnnotation.type === experimental_utils_1.AST_NODE_TYPES.TSTypeReference
                    ? // TODO - if we add more references
                        'RegExp'
                    : keywordMap[typeNode.typeAnnotation.type];
                context.report({
                    node,
                    messageId: 'noInferrableType',
                    data: {
                        type,
                    },
                    fix: fixer => fixer.remove(typeNode),
                });
            }
            function inferrableVariableVisitor(node) {
                if (!node.id) {
                    return;
                }
                reportInferrableType(node, node.id.typeAnnotation, node.init);
            }
            function inferrableParameterVisitor(node) {
                if (ignoreParameters || !node.params) {
                    return;
                }
                node.params.filter(param => param.type === experimental_utils_1.AST_NODE_TYPES.AssignmentPattern &&
                    param.left &&
                    param.right).forEach(param => {
                    reportInferrableType(param, param.left.typeAnnotation, param.right);
                });
            }
            function inferrablePropertyVisitor(node) {
                // We ignore `readonly` because of Microsoft/TypeScript#14416
                // Essentially a readonly property without a type
                // will result in its value being the type, leading to
                // compile errors if the type is stripped.
                if (ignoreProperties || node.readonly || node.optional) {
                    return;
                }
                reportInferrableType(node, node.typeAnnotation, node.value);
            }
            return {
                VariableDeclarator: inferrableVariableVisitor,
                FunctionExpression: inferrableParameterVisitor,
                FunctionDeclaration: inferrableParameterVisitor,
                ArrowFunctionExpression: inferrableParameterVisitor,
                ClassProperty: inferrablePropertyVisitor,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-magic-numbers.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const no_magic_numbers_1 = __importDefault(require(271) /* eslint/lib/rules/no-magic-numbers */);
    const util = __importStar(require(622) /* ../util */);
    const baseRuleSchema = Array.isArray(no_magic_numbers_1.default.meta.schema)
        ? no_magic_numbers_1.default.meta.schema[0]
        : no_magic_numbers_1.default.meta.schema;
    exports.default = util.createRule({
        name: 'no-magic-numbers',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallow magic numbers',
                category: 'Best Practices',
                recommended: false,
                extendsBaseRule: true,
            },
            // Extend base schema with additional property to ignore TS numeric literal types
            schema: [
                Object.assign(Object.assign({}, baseRuleSchema), { properties: Object.assign(Object.assign({}, baseRuleSchema.properties), { ignoreNumericLiteralTypes: {
                            type: 'boolean',
                        }, ignoreEnums: {
                            type: 'boolean',
                        }, ignoreReadonlyClassProperties: {
                            type: 'boolean',
                        } }) }),
            ],
            messages: no_magic_numbers_1.default.meta.messages,
        },
        defaultOptions: [
            {
                ignore: [],
                ignoreArrayIndexes: false,
                enforceConst: false,
                detectObjects: false,
                ignoreNumericLiteralTypes: false,
                ignoreEnums: false,
                ignoreReadonlyClassProperties: false,
            },
        ],
        create(context, [options]) {
            const rules = no_magic_numbers_1.default.create(context);
            return {
                Literal(node) {
                    var _a;
                    // Check if the node is a TypeScript enum declaration
                    if (options.ignoreEnums && isParentTSEnumDeclaration(node)) {
                        return;
                    }
                    // Check TypeScript specific nodes for Numeric Literal
                    if (options.ignoreNumericLiteralTypes &&
                        typeof node.value === 'number' &&
                        isTSNumericLiteralType(node)) {
                        return;
                    }
                    // Check if the node is a readonly class property
                    if (typeof node.value === 'number' &&
                        isParentTSReadonlyClassProperty(node)) {
                        if (options.ignoreReadonlyClassProperties) {
                            return;
                        }
                        let fullNumberNode = node;
                        let raw = node.raw;
                        if (((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type) === experimental_utils_1.AST_NODE_TYPES.UnaryExpression &&
                            // the base rule only shows the operator for negative numbers
                            // https://github.com/eslint/eslint/blob/9dfc8501fb1956c90dc11e6377b4cb38a6bea65d/lib/rules/no-magic-numbers.js#L126
                            node.parent.operator === '-') {
                            fullNumberNode = node.parent;
                            raw = `${node.parent.operator}${node.raw}`;
                        }
                        context.report({
                            messageId: 'noMagic',
                            node: fullNumberNode,
                            data: { raw },
                        });
                        return;
                    }
                    // Let the base rule deal with the rest
                    rules.Literal(node);
                },
            };
        },
    });
    /**
     * Gets the true parent of the literal, handling prefixed numbers (-1 / +1)
     */
    function getLiteralParent(node) {
        var _a;
        if (((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type) === experimental_utils_1.AST_NODE_TYPES.UnaryExpression &&
            ['-', '+'].includes(node.parent.operator)) {
            return node.parent.parent;
        }
        return node.parent;
    }
    /**
     * Checks if the node grandparent is a Typescript type alias declaration
     * @param node the node to be validated.
     * @returns true if the node grandparent is a Typescript type alias declaration
     * @private
     */
    function isGrandparentTSTypeAliasDeclaration(node) {
        var _a, _b;
        return ((_b = (_a = node.parent) === null || _a === void 0 ? void 0 : _a.parent) === null || _b === void 0 ? void 0 : _b.type) === experimental_utils_1.AST_NODE_TYPES.TSTypeAliasDeclaration;
    }
    /**
     * Checks if the node grandparent is a Typescript union type and its parent is a type alias declaration
     * @param node the node to be validated.
     * @returns true if the node grandparent is a Typescript union type and its parent is a type alias declaration
     * @private
     */
    function isGrandparentTSUnionType(node) {
        var _a, _b;
        if (((_b = (_a = node.parent) === null || _a === void 0 ? void 0 : _a.parent) === null || _b === void 0 ? void 0 : _b.type) === experimental_utils_1.AST_NODE_TYPES.TSUnionType) {
            return isGrandparentTSTypeAliasDeclaration(node.parent);
        }
        return false;
    }
    /**
     * Checks if the node parent is a Typescript enum member
     * @param node the node to be validated.
     * @returns true if the node parent is a Typescript enum member
     * @private
     */
    function isParentTSEnumDeclaration(node) {
        const parent = getLiteralParent(node);
        return (parent === null || parent === void 0 ? void 0 : parent.type) === experimental_utils_1.AST_NODE_TYPES.TSEnumMember;
    }
    /**
     * Checks if the node parent is a Typescript literal type
     * @param node the node to be validated.
     * @returns true if the node parent is a Typescript literal type
     * @private
     */
    function isParentTSLiteralType(node) {
        var _a;
        return ((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type) === experimental_utils_1.AST_NODE_TYPES.TSLiteralType;
    }
    /**
     * Checks if the node is a valid TypeScript numeric literal type.
     * @param node the node to be validated.
     * @returns true if the node is a TypeScript numeric literal type.
     * @private
     */
    function isTSNumericLiteralType(node) {
        var _a;
        // For negative numbers, use the parent node
        if (((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type) === experimental_utils_1.AST_NODE_TYPES.UnaryExpression &&
            node.parent.operator === '-') {
            node = node.parent;
        }
        // If the parent node is not a TSLiteralType, early return
        if (!isParentTSLiteralType(node)) {
            return false;
        }
        // If the grandparent is a TSTypeAliasDeclaration, ignore
        if (isGrandparentTSTypeAliasDeclaration(node)) {
            return true;
        }
        // If the grandparent is a TSUnionType and it's parent is a TSTypeAliasDeclaration, ignore
        if (isGrandparentTSUnionType(node)) {
            return true;
        }
        return false;
    }
    /**
     * Checks if the node parent is a readonly class property
     * @param node the node to be validated.
     * @returns true if the node parent is a readonly class property
     * @private
     */
    function isParentTSReadonlyClassProperty(node) {
        const parent = getLiteralParent(node);
        if ((parent === null || parent === void 0 ? void 0 : parent.type) === experimental_utils_1.AST_NODE_TYPES.ClassProperty && parent.readonly) {
            return true;
        }
        return false;
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-misused-new.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-misused-new',
        meta: {
            type: 'problem',
            docs: {
                description: 'Enforce valid definition of `new` and `constructor`',
                category: 'Best Practices',
                recommended: 'error',
            },
            schema: [],
            messages: {
                errorMessageInterface: 'Interfaces cannot be constructed, only classes.',
                errorMessageClass: 'Class cannot have method named `new`.',
            },
        },
        defaultOptions: [],
        create(context) {
            /**
             * @param node type to be inspected.
             * @returns name of simple type or null
             */
            function getTypeReferenceName(node) {
                if (node) {
                    switch (node.type) {
                        case experimental_utils_1.AST_NODE_TYPES.TSTypeAnnotation:
                            return getTypeReferenceName(node.typeAnnotation);
                        case experimental_utils_1.AST_NODE_TYPES.TSTypeReference:
                            return getTypeReferenceName(node.typeName);
                        case experimental_utils_1.AST_NODE_TYPES.Identifier:
                            return node.name;
                        default:
                            break;
                    }
                }
                return null;
            }
            /**
             * @param parent parent node.
             * @param returnType type to be compared
             */
            function isMatchingParentType(parent, returnType) {
                if (parent &&
                    'id' in parent &&
                    parent.id &&
                    parent.id.type === experimental_utils_1.AST_NODE_TYPES.Identifier) {
                    return getTypeReferenceName(returnType) === parent.id.name;
                }
                return false;
            }
            return {
                'TSInterfaceBody > TSConstructSignatureDeclaration'(node) {
                    if (isMatchingParentType(node.parent.parent, node.returnType)) {
                        // constructor
                        context.report({
                            node,
                            messageId: 'errorMessageInterface',
                        });
                    }
                },
                "TSMethodSignature[key.name='constructor']"(node) {
                    context.report({
                        node,
                        messageId: 'errorMessageInterface',
                    });
                },
                "ClassBody > MethodDefinition[key.name='new']"(node) {
                    if (node.value.type === experimental_utils_1.AST_NODE_TYPES.TSEmptyBodyFunctionExpression) {
                        if (node.parent &&
                            isMatchingParentType(node.parent.parent, node.value.returnType)) {
                            context.report({
                                node,
                                messageId: 'errorMessageClass',
                            });
                        }
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-misused-promises.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const tsutils = __importStar(require(627) /* tsutils */);
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-misused-promises',
        meta: {
            docs: {
                description: 'Avoid using promises in places not designed to handle them',
                category: 'Best Practices',
                recommended: 'error',
                requiresTypeChecking: true,
            },
            messages: {
                voidReturn: 'Promise returned in function argument where a void return was expected.',
                conditional: 'Expected non-Promise value in a boolean conditional.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        checksConditionals: {
                            type: 'boolean',
                        },
                        checksVoidReturn: {
                            type: 'boolean',
                        },
                    },
                },
            ],
            type: 'problem',
        },
        defaultOptions: [
            {
                checksConditionals: true,
                checksVoidReturn: true,
            },
        ],
        create(context, [{ checksConditionals, checksVoidReturn }]) {
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            const conditionalChecks = {
                ConditionalExpression: checkTestConditional,
                DoWhileStatement: checkTestConditional,
                ForStatement: checkTestConditional,
                IfStatement: checkTestConditional,
                LogicalExpression(node) {
                    // We only check the lhs of a logical expression because the rhs might
                    // be the return value of a short circuit expression.
                    checkConditional(node.left);
                },
                UnaryExpression(node) {
                    if (node.operator === '!') {
                        checkConditional(node.argument);
                    }
                },
                WhileStatement: checkTestConditional,
            };
            const voidReturnChecks = {
                CallExpression: checkArguments,
                OptionalCallExpression: checkArguments,
                NewExpression: checkArguments,
            };
            function checkTestConditional(node) {
                if (node.test) {
                    checkConditional(node.test);
                }
            }
            function checkConditional(node) {
                const tsNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                if (isAlwaysThenable(checker, tsNode)) {
                    context.report({
                        messageId: 'conditional',
                        node,
                    });
                }
            }
            function checkArguments(node) {
                const tsNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                const voidParams = voidFunctionParams(checker, tsNode);
                if (voidParams.size === 0) {
                    return;
                }
                for (const [index, argument] of node.arguments.entries()) {
                    if (!voidParams.has(index)) {
                        continue;
                    }
                    const tsNode = parserServices.esTreeNodeToTSNodeMap.get(argument);
                    if (returnsThenable(checker, tsNode)) {
                        context.report({
                            messageId: 'voidReturn',
                            node: argument,
                        });
                    }
                }
            }
            return Object.assign(Object.assign({}, (checksConditionals ? conditionalChecks : {})), (checksVoidReturn ? voidReturnChecks : {}));
        },
    });
    // Variation on the thenable check which requires all forms of the type (read:
    // alternates in a union) to be thenable. Otherwise, you might be trying to
    // check if something is defined or undefined and get caught because one of the
    // branches is thenable.
    function isAlwaysThenable(checker, node) {
        const type = checker.getTypeAtLocation(node);
        for (const subType of tsutils.unionTypeParts(checker.getApparentType(type))) {
            const thenProp = subType.getProperty('then');
            // If one of the alternates has no then property, it is not thenable in all
            // cases.
            if (thenProp === undefined) {
                return false;
            }
            // We walk through each variation of the then property. Since we know it
            // exists at this point, we just need at least one of the alternates to
            // be of the right form to consider it thenable.
            const thenType = checker.getTypeOfSymbolAtLocation(thenProp, node);
            let hasThenableSignature = false;
            for (const subType of tsutils.unionTypeParts(thenType)) {
                for (const signature of subType.getCallSignatures()) {
                    if (signature.parameters.length !== 0 &&
                        isFunctionParam(checker, signature.parameters[0], node)) {
                        hasThenableSignature = true;
                        break;
                    }
                }
                // We only need to find one variant of the then property that has a
                // function signature for it to be thenable.
                if (hasThenableSignature) {
                    break;
                }
            }
            // If no flavors of the then property are thenable, we don't consider the
            // overall type to be thenable
            if (!hasThenableSignature) {
                return false;
            }
        }
        // If all variants are considered thenable (i.e. haven't returned false), we
        // consider the overall type thenable
        return true;
    }
    function isFunctionParam(checker, param, node) {
        const type = checker.getApparentType(checker.getTypeOfSymbolAtLocation(param, node));
        for (const subType of tsutils.unionTypeParts(type)) {
            if (subType.getCallSignatures().length !== 0) {
                return true;
            }
        }
        return false;
    }
    // Get the positions of parameters which are void functions (and not also
    // thenable functions). These are the candidates for the void-return check at
    // the current call site.
    function voidFunctionParams(checker, node) {
        const voidReturnIndices = new Set();
        const thenableReturnIndices = new Set();
        const type = checker.getTypeAtLocation(node.expression);
        for (const subType of tsutils.unionTypeParts(type)) {
            // Standard function calls and `new` have two different types of signatures
            const signatures = ts.isCallExpression(node)
                ? subType.getCallSignatures()
                : subType.getConstructSignatures();
            for (const signature of signatures) {
                for (const [index, parameter] of signature.parameters.entries()) {
                    const type = checker.getTypeOfSymbolAtLocation(parameter, node.expression);
                    for (const subType of tsutils.unionTypeParts(type)) {
                        for (const signature of subType.getCallSignatures()) {
                            const returnType = signature.getReturnType();
                            if (tsutils.isTypeFlagSet(returnType, ts.TypeFlags.Void)) {
                                voidReturnIndices.add(index);
                            }
                            else if (tsutils.isThenableType(checker, node.expression, returnType)) {
                                thenableReturnIndices.add(index);
                            }
                        }
                    }
                }
            }
        }
        // If a certain positional argument accepts both thenable and void returns,
        // a promise-returning function is valid
        for (const thenable of thenableReturnIndices) {
            voidReturnIndices.delete(thenable);
        }
        return voidReturnIndices;
    }
    // Returns true if the expression is a function that returns a thenable
    function returnsThenable(checker, node) {
        const type = checker.getApparentType(checker.getTypeAtLocation(node));
        for (const subType of tsutils.unionTypeParts(type)) {
            for (const signature of subType.getCallSignatures()) {
                const returnType = signature.getReturnType();
                if (tsutils.isThenableType(checker, node, returnType)) {
                    return true;
                }
            }
        }
        return false;
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-namespace.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-namespace',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallow the use of custom TypeScript modules and namespaces',
                category: 'Best Practices',
                recommended: 'error',
            },
            messages: {
                moduleSyntaxIsPreferred: 'ES2015 module syntax is preferred over custom TypeScript modules and namespaces.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        allowDeclarations: {
                            type: 'boolean',
                        },
                        allowDefinitionFiles: {
                            type: 'boolean',
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            {
                allowDeclarations: false,
                allowDefinitionFiles: true,
            },
        ],
        create(context, [{ allowDeclarations, allowDefinitionFiles }]) {
            const filename = context.getFilename();
            return {
                "TSModuleDeclaration[global!=true][id.type='Identifier']"(node) {
                    if ((node.parent &&
                        node.parent.type === experimental_utils_1.AST_NODE_TYPES.TSModuleDeclaration) ||
                        (allowDefinitionFiles && util.isDefinitionFile(filename)) ||
                        (allowDeclarations && node.declare === true)) {
                        return;
                    }
                    context.report({
                        node,
                        messageId: 'moduleSyntaxIsPreferred',
                    });
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-non-null-asserted-optional-chain.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-non-null-asserted-optional-chain',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallows using a non-null assertion after an optional chain expression',
                category: 'Possible Errors',
                recommended: false,
            },
            messages: {
                noNonNullOptionalChain: 'Optional chain expressions can return undefined by design - using a non-null assertion is unsafe and wrong.',
                suggestRemovingNonNull: 'You should remove the non-null assertion.',
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            return {
                'TSNonNullExpression > :matches(OptionalMemberExpression, OptionalCallExpression)'(node) {
                    // selector guarantees this assertion
                    const parent = node.parent;
                    context.report({
                        node,
                        messageId: 'noNonNullOptionalChain',
                        // use a suggestion instead of a fixer, because this can obviously break type checks
                        suggest: [
                            {
                                messageId: 'suggestRemovingNonNull',
                                fix(fixer) {
                                    return fixer.removeRange([
                                        parent.range[1] - 1,
                                        parent.range[1],
                                    ]);
                                },
                            },
                        ],
                    });
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-non-null-assertion.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-non-null-assertion',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallows non-null assertions using the `!` postfix operator',
                category: 'Stylistic Issues',
                recommended: 'warn',
            },
            messages: {
                noNonNull: 'Forbidden non-null assertion.',
                suggestOptionalChain: 'Consider using the optional chain operator `?.` instead. This operator includes runtime checks, so it is safer than the compile-only non-null assertion operator.',
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                TSNonNullExpression(node) {
                    const suggest = [];
                    function convertTokenToOptional(replacement) {
                        return (fixer) => {
                            const operator = sourceCode.getTokenAfter(node.expression, util.isNonNullAssertionPunctuator);
                            if (operator) {
                                return fixer.replaceText(operator, replacement);
                            }
                            return null;
                        };
                    }
                    function removeToken() {
                        return (fixer) => {
                            const operator = sourceCode.getTokenAfter(node.expression, util.isNonNullAssertionPunctuator);
                            if (operator) {
                                return fixer.remove(operator);
                            }
                            return null;
                        };
                    }
                    if (node.parent) {
                        if ((node.parent.type === experimental_utils_1.AST_NODE_TYPES.MemberExpression ||
                            node.parent.type === experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression) &&
                            node.parent.object === node) {
                            if (!node.parent.optional) {
                                if (node.parent.computed) {
                                    // it is x![y]?.z
                                    suggest.push({
                                        messageId: 'suggestOptionalChain',
                                        fix: convertTokenToOptional('?.'),
                                    });
                                }
                                else {
                                    // it is x!.y?.z
                                    suggest.push({
                                        messageId: 'suggestOptionalChain',
                                        fix: convertTokenToOptional('?'),
                                    });
                                }
                            }
                            else {
                                if (node.parent.computed) {
                                    // it is x!?.[y].z
                                    suggest.push({
                                        messageId: 'suggestOptionalChain',
                                        fix: removeToken(),
                                    });
                                }
                                else {
                                    // it is x!?.y.z
                                    suggest.push({
                                        messageId: 'suggestOptionalChain',
                                        fix: removeToken(),
                                    });
                                }
                            }
                        }
                        else if ((node.parent.type === experimental_utils_1.AST_NODE_TYPES.CallExpression ||
                            node.parent.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression) &&
                            node.parent.callee === node) {
                            if (!node.parent.optional) {
                                // it is x.y?.z!()
                                suggest.push({
                                    messageId: 'suggestOptionalChain',
                                    fix: convertTokenToOptional('?.'),
                                });
                            }
                            else {
                                // it is x.y.z!?.()
                                suggest.push({
                                    messageId: 'suggestOptionalChain',
                                    fix: removeToken(),
                                });
                            }
                        }
                    }
                    context.report({
                        node,
                        messageId: 'noNonNull',
                        suggest,
                    });
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-parameter-properties.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-parameter-properties',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallow the use of parameter properties in class constructors',
                category: 'Stylistic Issues',
                // too opinionated to be recommended
                recommended: false,
            },
            messages: {
                noParamProp: 'Property {{parameter}} cannot be declared in the constructor.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        allows: {
                            type: 'array',
                            items: {
                                enum: [
                                    'readonly',
                                    'private',
                                    'protected',
                                    'public',
                                    'private readonly',
                                    'protected readonly',
                                    'public readonly',
                                ],
                            },
                            minItems: 1,
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            {
                allows: [],
            },
        ],
        create(context, [{ allows }]) {
            /**
             * Gets the modifiers of `node`.
             * @param node the node to be inspected.
             */
            function getModifiers(node) {
                const modifiers = [];
                if (node.accessibility) {
                    modifiers.push(node.accessibility);
                }
                if (node.readonly) {
                    modifiers.push('readonly');
                }
                return modifiers.filter(Boolean).join(' ');
            }
            return {
                TSParameterProperty(node) {
                    const modifiers = getModifiers(node);
                    if (!allows.includes(modifiers)) {
                        // HAS to be an identifier or assignment or TSC will throw
                        if (node.parameter.type !== experimental_utils_1.AST_NODE_TYPES.Identifier &&
                            node.parameter.type !== experimental_utils_1.AST_NODE_TYPES.AssignmentPattern) {
                            return;
                        }
                        const name = node.parameter.type === experimental_utils_1.AST_NODE_TYPES.Identifier
                            ? node.parameter.name
                            : // has to be an Identifier or TSC will throw an error
                                node.parameter.left.name;
                        context.report({
                            node,
                            messageId: 'noParamProp',
                            data: {
                                parameter: name,
                            },
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-require-imports.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-require-imports',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallows invocation of `require()`',
                category: 'Best Practices',
                recommended: false,
            },
            schema: [],
            messages: {
                noRequireImports: 'A `require()` style import is forbidden.',
            },
        },
        defaultOptions: [],
        create(context) {
            return {
                ':matches(CallExpression, OptionalCallExpression) > Identifier[name="require"]'(node) {
                    context.report({
                        node: node.parent,
                        messageId: 'noRequireImports',
                    });
                },
                TSExternalModuleReference(node) {
                    context.report({
                        node,
                        messageId: 'noRequireImports',
                    });
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-this-alias.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-this-alias',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallow aliasing `this`',
                category: 'Best Practices',
                recommended: 'error',
            },
            schema: [
                {
                    type: 'object',
                    additionalProperties: false,
                    properties: {
                        allowDestructuring: {
                            type: 'boolean',
                        },
                        allowedNames: {
                            type: 'array',
                            items: {
                                type: 'string',
                            },
                        },
                    },
                },
            ],
            messages: {
                thisAssignment: "Unexpected aliasing of 'this' to local variable.",
                thisDestructure: "Unexpected aliasing of members of 'this' to local variables.",
            },
        },
        defaultOptions: [
            {
                allowDestructuring: true,
                allowedNames: [],
            },
        ],
        create(context, [{ allowDestructuring, allowedNames }]) {
            return {
                "VariableDeclarator[init.type='ThisExpression']"(node) {
                    const { id } = node;
                    if (allowDestructuring && id.type !== experimental_utils_1.AST_NODE_TYPES.Identifier) {
                        return;
                    }
                    const hasAllowedName = id.type === experimental_utils_1.AST_NODE_TYPES.Identifier
                        ? allowedNames.includes(id.name)
                        : false;
                    if (!hasAllowedName) {
                        context.report({
                            node: id,
                            messageId: id.type === experimental_utils_1.AST_NODE_TYPES.Identifier
                                ? 'thisAssignment'
                                : 'thisDestructure',
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-throw-literal.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    exports.default = util.createRule({
        name: 'no-throw-literal',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallow throwing literals as exceptions',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            schema: [],
            messages: {
                object: 'Expected an error object to be thrown.',
                undef: 'Do not throw undefined.',
            },
        },
        defaultOptions: [],
        create(context) {
            const parserServices = util.getParserServices(context);
            const program = parserServices.program;
            const checker = program.getTypeChecker();
            function isErrorLike(type) {
                var _a;
                const symbol = type.getSymbol();
                if (!symbol) {
                    return false;
                }
                if (symbol.getName() === 'Error') {
                    const declarations = (_a = symbol.getDeclarations()) !== null && _a !== void 0 ? _a : [];
                    for (const declaration of declarations) {
                        const sourceFile = declaration.getSourceFile();
                        if (program.isSourceFileDefaultLibrary(sourceFile)) {
                            return true;
                        }
                    }
                }
                const baseTypes = checker.getBaseTypes(type);
                for (const baseType of baseTypes) {
                    if (isErrorLike(baseType)) {
                        return true;
                    }
                }
                return false;
            }
            function tryGetThrowArgumentType(node) {
                switch (node.type) {
                    case experimental_utils_1.AST_NODE_TYPES.Identifier:
                    case experimental_utils_1.AST_NODE_TYPES.CallExpression:
                    case experimental_utils_1.AST_NODE_TYPES.NewExpression:
                    case experimental_utils_1.AST_NODE_TYPES.MemberExpression: {
                        const tsNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                        return checker.getTypeAtLocation(tsNode);
                    }
                    case experimental_utils_1.AST_NODE_TYPES.AssignmentExpression:
                        return tryGetThrowArgumentType(node.right);
                    case experimental_utils_1.AST_NODE_TYPES.SequenceExpression:
                        return tryGetThrowArgumentType(node.expressions[node.expressions.length - 1]);
                    case experimental_utils_1.AST_NODE_TYPES.LogicalExpression: {
                        const left = tryGetThrowArgumentType(node.left);
                        return left !== null && left !== void 0 ? left : tryGetThrowArgumentType(node.right);
                    }
                    case experimental_utils_1.AST_NODE_TYPES.ConditionalExpression: {
                        const consequent = tryGetThrowArgumentType(node.consequent);
                        return consequent !== null && consequent !== void 0 ? consequent : tryGetThrowArgumentType(node.alternate);
                    }
                    default:
                        return null;
                }
            }
            function checkThrowArgument(node) {
                if (node.type === experimental_utils_1.AST_NODE_TYPES.AwaitExpression ||
                    node.type === experimental_utils_1.AST_NODE_TYPES.YieldExpression) {
                    return;
                }
                const type = tryGetThrowArgumentType(node);
                if (type) {
                    if (type.flags & ts.TypeFlags.Undefined) {
                        context.report({ node, messageId: 'undef' });
                        return;
                    }
                    if (type.flags & (ts.TypeFlags.Any | ts.TypeFlags.Unknown) ||
                        isErrorLike(type)) {
                        return;
                    }
                }
                context.report({ node, messageId: 'object' });
            }
            return {
                ThrowStatement(node) {
                    if (node.argument) {
                        checkThrowArgument(node.argument);
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-type-alias.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    const enumValues = [
        'always',
        'never',
        'in-unions',
        'in-intersections',
        'in-unions-and-intersections',
    ];
    exports.default = util.createRule({
        name: 'no-type-alias',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallow the use of type aliases',
                category: 'Stylistic Issues',
                // too opinionated to be recommended
                recommended: false,
            },
            messages: {
                noTypeAlias: 'Type {{alias}} are not allowed.',
                noCompositionAlias: '{{typeName}} in {{compositionType}} types are not allowed.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        allowAliases: {
                            enum: enumValues,
                        },
                        allowCallbacks: {
                            enum: ['always', 'never'],
                        },
                        allowConditionalTypes: {
                            enum: ['always', 'never'],
                        },
                        allowConstructors: {
                            enum: ['always', 'never'],
                        },
                        allowLiterals: {
                            enum: enumValues,
                        },
                        allowMappedTypes: {
                            enum: enumValues,
                        },
                        allowTupleTypes: {
                            enum: enumValues,
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            {
                allowAliases: 'never',
                allowCallbacks: 'never',
                allowConditionalTypes: 'never',
                allowConstructors: 'never',
                allowLiterals: 'never',
                allowMappedTypes: 'never',
                allowTupleTypes: 'never',
            },
        ],
        create(context, [{ allowAliases, allowCallbacks, allowConditionalTypes, allowConstructors, allowLiterals, allowMappedTypes, allowTupleTypes, },]) {
            const unions = ['always', 'in-unions', 'in-unions-and-intersections'];
            const intersections = [
                'always',
                'in-intersections',
                'in-unions-and-intersections',
            ];
            const compositions = [
                'in-unions',
                'in-intersections',
                'in-unions-and-intersections',
            ];
            const aliasTypes = new Set([
                experimental_utils_1.AST_NODE_TYPES.TSArrayType,
                experimental_utils_1.AST_NODE_TYPES.TSTypeReference,
                experimental_utils_1.AST_NODE_TYPES.TSLiteralType,
                experimental_utils_1.AST_NODE_TYPES.TSTypeQuery,
                experimental_utils_1.AST_NODE_TYPES.TSIndexedAccessType,
            ]);
            /**
             * Determines if the composition type is supported by the allowed flags.
             * @param isTopLevel a flag indicating this is the top level node.
             * @param compositionType the composition type (either TSUnionType or TSIntersectionType)
             * @param allowed the currently allowed flags.
             */
            function isSupportedComposition(isTopLevel, compositionType, allowed) {
                return (!compositions.includes(allowed) ||
                    (!isTopLevel &&
                        ((compositionType === experimental_utils_1.AST_NODE_TYPES.TSUnionType &&
                            unions.includes(allowed)) ||
                            (compositionType === experimental_utils_1.AST_NODE_TYPES.TSIntersectionType &&
                                intersections.includes(allowed)))));
            }
            /**
             * Gets the message to be displayed based on the node type and whether the node is a top level declaration.
             * @param node the location
             * @param compositionType the type of composition this alias is part of (undefined if not
             *                                  part of a composition)
             * @param isRoot a flag indicating we are dealing with the top level declaration.
             * @param type the kind of type alias being validated.
             */
            function reportError(node, compositionType, isRoot, type) {
                if (isRoot) {
                    return context.report({
                        node,
                        messageId: 'noTypeAlias',
                        data: {
                            alias: type.toLowerCase(),
                        },
                    });
                }
                return context.report({
                    node,
                    messageId: 'noCompositionAlias',
                    data: {
                        compositionType: compositionType === experimental_utils_1.AST_NODE_TYPES.TSUnionType
                            ? 'union'
                            : 'intersection',
                        typeName: type,
                    },
                });
            }
            const isValidTupleType = (type) => {
                if (type.node.type === experimental_utils_1.AST_NODE_TYPES.TSTupleType) {
                    return true;
                }
                if (type.node.type === experimental_utils_1.AST_NODE_TYPES.TSTypeOperator) {
                    if (['keyof', 'readonly'].includes(type.node.operator) &&
                        type.node.typeAnnotation &&
                        type.node.typeAnnotation.type === experimental_utils_1.AST_NODE_TYPES.TSTupleType) {
                        return true;
                    }
                }
                return false;
            };
            const checkAndReport = (optionValue, isTopLevel, type, label) => {
                if (optionValue === 'never' ||
                    !isSupportedComposition(isTopLevel, type.compositionType, optionValue)) {
                    reportError(type.node, type.compositionType, isTopLevel, label);
                }
            };
            /**
             * Validates the node looking for aliases, callbacks and literals.
             * @param type the type of composition this alias is part of (null if not
             *                                  part of a composition)
             * @param isTopLevel a flag indicating this is the top level node.
             */
            function validateTypeAliases(type, isTopLevel = false) {
                if (type.node.type === experimental_utils_1.AST_NODE_TYPES.TSFunctionType) {
                    // callback
                    if (allowCallbacks === 'never') {
                        reportError(type.node, type.compositionType, isTopLevel, 'Callbacks');
                    }
                }
                else if (type.node.type === experimental_utils_1.AST_NODE_TYPES.TSConditionalType) {
                    // conditional type
                    if (allowConditionalTypes === 'never') {
                        reportError(type.node, type.compositionType, isTopLevel, 'Conditional types');
                    }
                }
                else if (type.node.type === experimental_utils_1.AST_NODE_TYPES.TSConstructorType) {
                    if (allowConstructors === 'never') {
                        reportError(type.node, type.compositionType, isTopLevel, 'Constructors');
                    }
                }
                else if (type.node.type === experimental_utils_1.AST_NODE_TYPES.TSTypeLiteral) {
                    // literal object type
                    checkAndReport(allowLiterals, isTopLevel, type, 'Literals');
                }
                else if (type.node.type === experimental_utils_1.AST_NODE_TYPES.TSMappedType) {
                    // mapped type
                    checkAndReport(allowMappedTypes, isTopLevel, type, 'Mapped types');
                }
                else if (isValidTupleType(type)) {
                    // tuple types
                    checkAndReport(allowTupleTypes, isTopLevel, type, 'Tuple Types');
                }
                else if (
                // eslint-disable-next-line @typescript-eslint/internal/prefer-ast-types-enum
                type.node.type.endsWith('Keyword') ||
                    aliasTypes.has(type.node.type)) {
                    // alias / keyword
                    checkAndReport(allowAliases, isTopLevel, type, 'Aliases');
                }
                else {
                    // unhandled type - shouldn't happen
                    reportError(type.node, type.compositionType, isTopLevel, 'Unhandled');
                }
            }
            /**
             * Flatten the given type into an array of its dependencies
             */
            function getTypes(node, compositionType = null) {
                if (node.type === experimental_utils_1.AST_NODE_TYPES.TSUnionType ||
                    node.type === experimental_utils_1.AST_NODE_TYPES.TSIntersectionType) {
                    return node.types.reduce((acc, type) => {
                        acc.push(...getTypes(type, node.type));
                        return acc;
                    }, []);
                }
                if (node.type === experimental_utils_1.AST_NODE_TYPES.TSParenthesizedType) {
                    return getTypes(node.typeAnnotation, compositionType);
                }
                return [{ node, compositionType }];
            }
            return {
                TSTypeAliasDeclaration(node) {
                    const types = getTypes(node.typeAnnotation);
                    if (types.length === 1) {
                        // is a top level type annotation
                        validateTypeAliases(types[0], true);
                    }
                    else {
                        // is a composition type
                        types.forEach(type => {
                            validateTypeAliases(type);
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-unnecessary-boolean-literal-compare.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const tsutils = __importStar(require(627) /* tsutils */);
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-unnecessary-boolean-literal-compare',
        meta: {
            docs: {
                description: 'Flags unnecessary equality comparisons against boolean literals',
                category: 'Stylistic Issues',
                recommended: false,
                requiresTypeChecking: true,
            },
            fixable: 'code',
            messages: {
                direct: 'This expression unnecessarily compares a boolean value to a boolean instead of using it directly',
                negated: 'This expression unnecessarily compares a boolean value to a boolean instead of negating it.',
            },
            schema: [],
            type: 'suggestion',
        },
        defaultOptions: [],
        create(context) {
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            function getBooleanComparison(node) {
                const comparison = deconstructComparison(node);
                if (!comparison) {
                    return undefined;
                }
                const expressionType = checker.getTypeAtLocation(parserServices.esTreeNodeToTSNodeMap.get(comparison.expression));
                if (!tsutils.isTypeFlagSet(expressionType, ts.TypeFlags.Boolean | ts.TypeFlags.BooleanLiteral)) {
                    return undefined;
                }
                return comparison;
            }
            function deconstructComparison(node) {
                const comparisonType = util.getEqualsKind(node.operator);
                if (!comparisonType) {
                    return undefined;
                }
                for (const [against, expression] of [
                    [node.right, node.left],
                    [node.left, node.right],
                ]) {
                    if (against.type !== experimental_utils_1.AST_NODE_TYPES.Literal ||
                        typeof against.value !== 'boolean') {
                        continue;
                    }
                    const { value } = against;
                    const negated = node.operator.startsWith('!');
                    return {
                        forTruthy: value ? !negated : negated,
                        expression,
                        negated,
                        range: expression.range[0] < against.range[0]
                            ? [expression.range[1], against.range[1]]
                            : [against.range[1], expression.range[1]],
                    };
                }
                return undefined;
            }
            return {
                BinaryExpression(node) {
                    const comparison = getBooleanComparison(node);
                    if (comparison) {
                        context.report({
                            fix: function* (fixer) {
                                yield fixer.removeRange(comparison.range);
                                if (!comparison.forTruthy) {
                                    yield fixer.insertTextBefore(node, '!');
                                }
                            },
                            messageId: comparison.negated ? 'negated' : 'direct',
                            node,
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-unnecessary-condition.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const ts = __importStar(require(44) /* typescript */);
    const tsutils_1 = require(627) /* tsutils */;
    const util_1 = require(622) /* ../util */;
    // Truthiness utilities
    // #region
    const isTruthyLiteral = (type) => tsutils_1.isBooleanLiteralType(type, true) || (tsutils_1.isLiteralType(type) && !!type.value);
    const isPossiblyFalsy = (type) => tsutils_1.unionTypeParts(type)
        // PossiblyFalsy flag includes literal values, so exclude ones that
        // are definitely truthy
        .filter(t => !isTruthyLiteral(t))
        .some(type => tsutils_1.isTypeFlagSet(type, ts.TypeFlags.PossiblyFalsy));
    const isPossiblyTruthy = (type) => tsutils_1.unionTypeParts(type).some(type => !tsutils_1.isFalsyType(type));
    // Nullish utilities
    const nullishFlag = ts.TypeFlags.Undefined | ts.TypeFlags.Null;
    const isNullishType = (type) => tsutils_1.isTypeFlagSet(type, nullishFlag);
    const isPossiblyNullish = (type) => tsutils_1.unionTypeParts(type).some(isNullishType);
    const isAlwaysNullish = (type) => tsutils_1.unionTypeParts(type).every(isNullishType);
    // isLiteralType only covers numbers and strings, this is a more exhaustive check.
    const isLiteral = (type) => tsutils_1.isBooleanLiteralType(type, true) ||
        tsutils_1.isBooleanLiteralType(type, false) ||
        type.flags === ts.TypeFlags.Undefined ||
        type.flags === ts.TypeFlags.Null ||
        type.flags === ts.TypeFlags.Void ||
        tsutils_1.isLiteralType(type);
    exports.default = util_1.createRule({
        name: 'no-unnecessary-condition',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Prevents conditionals where the type is always truthy or always falsy',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        allowConstantLoopConditions: {
                            type: 'boolean',
                        },
                        ignoreRhs: {
                            type: 'boolean',
                        },
                        checkArrayPredicates: {
                            type: 'boolean',
                        },
                    },
                    additionalProperties: false,
                },
            ],
            fixable: 'code',
            messages: {
                alwaysTruthy: 'Unnecessary conditional, value is always truthy.',
                alwaysFalsy: 'Unnecessary conditional, value is always falsy.',
                alwaysTruthyFunc: 'This callback should return a conditional, but return is always truthy',
                alwaysFalsyFunc: 'This callback should return a conditional, but return is always falsy',
                neverNullish: 'Unnecessary conditional, expected left-hand side of `??` operator to be possibly null or undefined.',
                alwaysNullish: 'Unnecessary conditional, left-hand side of `??` operator is always `null` or `undefined`',
                literalBooleanExpression: 'Unnecessary conditional, both sides of the expression are literal values',
                never: 'Unnecessary conditional, value is `never`',
                neverOptionalChain: 'Unnecessary optional chain on a non-nullish value',
            },
        },
        defaultOptions: [
            {
                allowConstantLoopConditions: false,
                ignoreRhs: false,
                checkArrayPredicates: false,
            },
        ],
        create(context, [{ allowConstantLoopConditions, checkArrayPredicates, ignoreRhs }]) {
            const service = util_1.getParserServices(context);
            const checker = service.program.getTypeChecker();
            const sourceCode = context.getSourceCode();
            function getNodeType(node) {
                const tsNode = service.esTreeNodeToTSNodeMap.get(node);
                return util_1.getConstrainedTypeAtLocation(checker, tsNode);
            }
            function nodeIsArrayType(node) {
                const nodeType = getNodeType(node);
                return checker.isArrayType(nodeType) || checker.isTupleType(nodeType);
            }
            /**
             * Checks if a conditional node is necessary:
             * if the type of the node is always true or always false, it's not necessary.
             */
            function checkNode(node) {
                const type = getNodeType(node);
                // Conditional is always necessary if it involves:
                //    `any` or `unknown` or a naked type parameter
                if (tsutils_1.unionTypeParts(type).some(part => tsutils_1.isTypeFlagSet(part, ts.TypeFlags.Any |
                    ts.TypeFlags.Unknown |
                    ts.TypeFlags.TypeParameter))) {
                    return;
                }
                const messageId = tsutils_1.isTypeFlagSet(type, ts.TypeFlags.Never)
                    ? 'never'
                    : !isPossiblyTruthy(type)
                        ? 'alwaysFalsy'
                        : !isPossiblyFalsy(type)
                            ? 'alwaysTruthy'
                            : undefined;
                if (messageId) {
                    context.report({ node, messageId });
                }
            }
            function checkNodeForNullish(node) {
                const type = getNodeType(node);
                // Conditional is always necessary if it involves `any` or `unknown`
                if (tsutils_1.isTypeFlagSet(type, ts.TypeFlags.Any | ts.TypeFlags.Unknown)) {
                    return;
                }
                const messageId = tsutils_1.isTypeFlagSet(type, ts.TypeFlags.Never)
                    ? 'never'
                    : !isPossiblyNullish(type)
                        ? 'neverNullish'
                        : isAlwaysNullish(type)
                            ? 'alwaysNullish'
                            : undefined;
                if (messageId) {
                    context.report({ node, messageId });
                }
            }
            /**
             * Checks that a binary expression is necessarily conditional, reports otherwise.
             * If both sides of the binary expression are literal values, it's not a necessary condition.
             *
             * NOTE: It's also unnecessary if the types that don't overlap at all
             *    but that case is handled by the Typescript compiler itself.
             */
            const BOOL_OPERATORS = new Set([
                '<',
                '>',
                '<=',
                '>=',
                '==',
                '===',
                '!=',
                '!==',
            ]);
            function checkIfBinaryExpressionIsNecessaryConditional(node) {
                if (BOOL_OPERATORS.has(node.operator) &&
                    isLiteral(getNodeType(node.left)) &&
                    isLiteral(getNodeType(node.right))) {
                    context.report({ node, messageId: 'literalBooleanExpression' });
                }
            }
            /**
             * Checks that a testable expression is necessarily conditional, reports otherwise.
             * Filters all LogicalExpressions to prevent some duplicate reports.
             */
            function checkIfTestExpressionIsNecessaryConditional(node) {
                if (node.test.type === experimental_utils_1.AST_NODE_TYPES.LogicalExpression) {
                    return;
                }
                checkNode(node.test);
            }
            /**
             * Checks that a logical expression contains a boolean, reports otherwise.
             */
            function checkLogicalExpressionForUnnecessaryConditionals(node) {
                if (node.operator === '??') {
                    checkNodeForNullish(node.left);
                    return;
                }
                checkNode(node.left);
                if (!ignoreRhs) {
                    checkNode(node.right);
                }
            }
            /**
             * Checks that a testable expression of a loop is necessarily conditional, reports otherwise.
             */
            function checkIfLoopIsNecessaryConditional(node) {
                if (node.test === null ||
                    node.test.type === experimental_utils_1.AST_NODE_TYPES.LogicalExpression) {
                    return;
                }
                /**
                 * Allow:
                 *   while (true) {}
                 *   for (;true;) {}
                 *   do {} while (true)
                 */
                if (allowConstantLoopConditions &&
                    tsutils_1.isBooleanLiteralType(getNodeType(node.test), true)) {
                    return;
                }
                checkNode(node.test);
            }
            const ARRAY_PREDICATE_FUNCTIONS = new Set([
                'filter',
                'find',
                'some',
                'every',
            ]);
            function shouldCheckCallback(node) {
                const { callee } = node;
                return (
                // option is on
                !!checkArrayPredicates &&
                    // looks like `something.filter` or `something.find`
                    callee.type === experimental_utils_1.AST_NODE_TYPES.MemberExpression &&
                    callee.property.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                    ARRAY_PREDICATE_FUNCTIONS.has(callee.property.name) &&
                    // and the left-hand side is an array, according to the types
                    nodeIsArrayType(callee.object));
            }
            function checkCallExpression(node) {
                const { arguments: [callback], } = node;
                if (callback && shouldCheckCallback(node)) {
                    // Inline defined functions
                    if ((callback.type === experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression ||
                        callback.type === experimental_utils_1.AST_NODE_TYPES.FunctionExpression) &&
                        callback.body) {
                        // Two special cases, where we can directly check the node that's returned:
                        // () => something
                        if (callback.body.type !== experimental_utils_1.AST_NODE_TYPES.BlockStatement) {
                            return checkNode(callback.body);
                        }
                        // () => { return something; }
                        const callbackBody = callback.body.body;
                        if (callbackBody.length === 1 &&
                            callbackBody[0].type === experimental_utils_1.AST_NODE_TYPES.ReturnStatement &&
                            callbackBody[0].argument) {
                            return checkNode(callbackBody[0].argument);
                        }
                        // Potential enhancement: could use code-path analysis to check
                        //   any function with a single return statement
                        // (Value to complexity ratio is dubious however)
                    }
                    // Otherwise just do type analysis on the function as a whole.
                    const returnTypes = tsutils_1.getCallSignaturesOfType(getNodeType(callback)).map(sig => sig.getReturnType());
                    /* istanbul ignore if */ if (returnTypes.length === 0) {
                        // Not a callable function
                        return;
                    }
                    if (!returnTypes.some(isPossiblyFalsy)) {
                        return context.report({
                            node: callback,
                            messageId: 'alwaysTruthyFunc',
                        });
                    }
                    if (!returnTypes.some(isPossiblyTruthy)) {
                        return context.report({
                            node: callback,
                            messageId: 'alwaysFalsyFunc',
                        });
                    }
                }
            }
            function checkOptionalChain(node, beforeOperator, fix) {
                // We only care if this step in the chain is optional. If just descend
                // from an optional chain, then that's fine.
                if (!node.optional) {
                    return;
                }
                const type = getNodeType(node);
                if (tsutils_1.isTypeFlagSet(type, ts.TypeFlags.Any) ||
                    tsutils_1.isTypeFlagSet(type, ts.TypeFlags.Unknown) ||
                    util_1.isNullableType(type, { allowUndefined: true })) {
                    return;
                }
                const questionDotOperator = util_1.nullThrows(sourceCode.getTokenAfter(beforeOperator, token => token.type === experimental_utils_1.AST_TOKEN_TYPES.Punctuator && token.value === '?.'), util_1.NullThrowsReasons.MissingToken('operator', node.type));
                context.report({
                    node,
                    loc: questionDotOperator.loc,
                    messageId: 'neverOptionalChain',
                    fix(fixer) {
                        return fixer.replaceText(questionDotOperator, fix);
                    },
                });
            }
            function checkOptionalMemberExpression(node) {
                checkOptionalChain(node, node.object, node.computed ? '' : '.');
            }
            function checkOptionalCallExpression(node) {
                checkOptionalChain(node, node.callee, '');
            }
            return {
                BinaryExpression: checkIfBinaryExpressionIsNecessaryConditional,
                CallExpression: checkCallExpression,
                ConditionalExpression: checkIfTestExpressionIsNecessaryConditional,
                DoWhileStatement: checkIfLoopIsNecessaryConditional,
                ForStatement: checkIfLoopIsNecessaryConditional,
                IfStatement: checkIfTestExpressionIsNecessaryConditional,
                LogicalExpression: checkLogicalExpressionForUnnecessaryConditionals,
                WhileStatement: checkIfLoopIsNecessaryConditional,
                OptionalMemberExpression: checkOptionalMemberExpression,
                OptionalCallExpression: checkOptionalCallExpression,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-unnecessary-qualifier.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const ts = __importStar(require(44) /* typescript */);
    const tsutils = __importStar(require(627) /* tsutils */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-unnecessary-qualifier',
        meta: {
            docs: {
                category: 'Best Practices',
                description: 'Warns when a namespace qualifier is unnecessary',
                recommended: false,
                requiresTypeChecking: true,
            },
            fixable: 'code',
            messages: {
                unnecessaryQualifier: "Qualifier is unnecessary since '{{ name }}' is in scope.",
            },
            schema: [],
            type: 'suggestion',
        },
        defaultOptions: [],
        create(context) {
            const namespacesInScope = [];
            let currentFailedNamespaceExpression = null;
            const parserServices = util.getParserServices(context);
            const esTreeNodeToTSNodeMap = parserServices.esTreeNodeToTSNodeMap;
            const program = parserServices.program;
            const checker = program.getTypeChecker();
            const sourceCode = context.getSourceCode();
            function tryGetAliasedSymbol(symbol, checker) {
                return tsutils.isSymbolFlagSet(symbol, ts.SymbolFlags.Alias)
                    ? checker.getAliasedSymbol(symbol)
                    : null;
            }
            function symbolIsNamespaceInScope(symbol) {
                var _a;
                const symbolDeclarations = (_a = symbol.getDeclarations()) !== null && _a !== void 0 ? _a : [];
                if (symbolDeclarations.some(decl => namespacesInScope.some(ns => ns === decl))) {
                    return true;
                }
                const alias = tryGetAliasedSymbol(symbol, checker);
                return alias !== null && symbolIsNamespaceInScope(alias);
            }
            function getSymbolInScope(node, flags, name) {
                // TODO:PERF `getSymbolsInScope` gets a long list. Is there a better way?
                const scope = checker.getSymbolsInScope(node, flags);
                return scope.find(scopeSymbol => scopeSymbol.name === name);
            }
            function symbolsAreEqual(accessed, inScope) {
                return accessed === checker.getExportSymbolOfSymbol(inScope);
            }
            function qualifierIsUnnecessary(qualifier, name) {
                const tsQualifier = esTreeNodeToTSNodeMap.get(qualifier);
                const tsName = esTreeNodeToTSNodeMap.get(name);
                const namespaceSymbol = checker.getSymbolAtLocation(tsQualifier);
                if (typeof namespaceSymbol === 'undefined' ||
                    !symbolIsNamespaceInScope(namespaceSymbol)) {
                    return false;
                }
                const accessedSymbol = checker.getSymbolAtLocation(tsName);
                if (typeof accessedSymbol === 'undefined') {
                    return false;
                }
                // If the symbol in scope is different, the qualifier is necessary.
                const fromScope = getSymbolInScope(tsQualifier, accessedSymbol.flags, sourceCode.getText(name));
                return (typeof fromScope === 'undefined' ||
                    symbolsAreEqual(accessedSymbol, fromScope));
            }
            function visitNamespaceAccess(node, qualifier, name) {
                // Only look for nested qualifier errors if we didn't already fail on the outer qualifier.
                if (!currentFailedNamespaceExpression &&
                    qualifierIsUnnecessary(qualifier, name)) {
                    currentFailedNamespaceExpression = node;
                    context.report({
                        node: qualifier,
                        messageId: 'unnecessaryQualifier',
                        data: {
                            name: sourceCode.getText(name),
                        },
                        fix(fixer) {
                            return fixer.removeRange([qualifier.range[0], name.range[0]]);
                        },
                    });
                }
            }
            function enterDeclaration(node) {
                namespacesInScope.push(esTreeNodeToTSNodeMap.get(node));
            }
            function exitDeclaration() {
                namespacesInScope.pop();
            }
            function resetCurrentNamespaceExpression(node) {
                if (node === currentFailedNamespaceExpression) {
                    currentFailedNamespaceExpression = null;
                }
            }
            function isPropertyAccessExpression(node) {
                return node.type === experimental_utils_1.AST_NODE_TYPES.MemberExpression && !node.computed;
            }
            function isEntityNameExpression(node) {
                return (node.type === experimental_utils_1.AST_NODE_TYPES.Identifier ||
                    (isPropertyAccessExpression(node) &&
                        isEntityNameExpression(node.object)));
            }
            return {
                TSModuleDeclaration: enterDeclaration,
                TSEnumDeclaration: enterDeclaration,
                'ExportNamedDeclaration[declaration.type="TSModuleDeclaration"]': enterDeclaration,
                'ExportNamedDeclaration[declaration.type="TSEnumDeclaration"]': enterDeclaration,
                'TSModuleDeclaration:exit': exitDeclaration,
                'TSEnumDeclaration:exit': exitDeclaration,
                'ExportNamedDeclaration[declaration.type="TSModuleDeclaration"]:exit': exitDeclaration,
                'ExportNamedDeclaration[declaration.type="TSEnumDeclaration"]:exit': exitDeclaration,
                TSQualifiedName(node) {
                    visitNamespaceAccess(node, node.left, node.right);
                },
                'MemberExpression[computed=false]': function (node) {
                    const property = node.property;
                    if (isEntityNameExpression(node.object)) {
                        visitNamespaceAccess(node, node.object, property);
                    }
                },
                'TSQualifiedName:exit': resetCurrentNamespaceExpression,
                'MemberExpression:exit': resetCurrentNamespaceExpression,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-unnecessary-type-arguments.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const tsutils = __importStar(require(627) /* tsutils */);
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    const util_1 = require(622) /* ../util */;
    exports.default = util.createRule({
        name: 'no-unnecessary-type-arguments',
        meta: {
            docs: {
                description: 'Enforces that type arguments will not be used if not required',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            fixable: 'code',
            messages: {
                unnecessaryTypeParameter: 'This is the default value for this type parameter, so it can be omitted.',
            },
            schema: [],
            type: 'suggestion',
        },
        defaultOptions: [],
        create(context) {
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            const sourceCode = context.getSourceCode();
            function checkTSArgsAndParameters(esParameters, typeParameters) {
                // Just check the last one. Must specify previous type parameters if the last one is specified.
                const i = esParameters.params.length - 1;
                const arg = esParameters.params[i];
                const param = typeParameters[i];
                // TODO: would like checker.areTypesEquivalent. https://github.com/Microsoft/TypeScript/issues/13502
                if (!(param === null || param === void 0 ? void 0 : param.default) ||
                    param.default.getText() !== sourceCode.getText(arg)) {
                    return;
                }
                context.report({
                    node: arg,
                    messageId: 'unnecessaryTypeParameter',
                    fix: fixer => fixer.removeRange(i === 0
                        ? esParameters.range
                        : [esParameters.params[i - 1].range[1], arg.range[1]]),
                });
            }
            return {
                TSTypeParameterInstantiation(node) {
                    const expression = parserServices.esTreeNodeToTSNodeMap.get(node);
                    const typeParameters = getTypeParametersFromNode(expression, checker);
                    if (typeParameters) {
                        checkTSArgsAndParameters(node, typeParameters);
                    }
                },
            };
        },
    });
    function getTypeParametersFromNode(node, checker) {
        if (ts.isExpressionWithTypeArguments(node)) {
            return getTypeParametersFromType(node.expression, checker);
        }
        if (ts.isTypeReferenceNode(node)) {
            return getTypeParametersFromType(node.typeName, checker);
        }
        if (ts.isCallExpression(node) || ts.isNewExpression(node)) {
            return getTypeParametersFromCall(node, checker);
        }
        return undefined;
    }
    function getTypeParametersFromType(type, checker) {
        const symAtLocation = checker.getSymbolAtLocation(type);
        if (!symAtLocation) {
            return undefined;
        }
        const sym = getAliasedSymbol(symAtLocation, checker);
        if (!sym.declarations) {
            return undefined;
        }
        return util_1.findFirstResult(sym.declarations, decl => tsutils.isClassLikeDeclaration(decl) ||
            ts.isTypeAliasDeclaration(decl) ||
            ts.isInterfaceDeclaration(decl)
            ? decl.typeParameters
            : undefined);
    }
    function getTypeParametersFromCall(node, checker) {
        const sig = checker.getResolvedSignature(node);
        const sigDecl = sig === null || sig === void 0 ? void 0 : sig.getDeclaration();
        if (!sigDecl) {
            return ts.isNewExpression(node)
                ? getTypeParametersFromType(node.expression, checker)
                : undefined;
        }
        return sigDecl.typeParameters;
    }
    function getAliasedSymbol(symbol, checker) {
        return tsutils.isSymbolFlagSet(symbol, ts.SymbolFlags.Alias)
            ? checker.getAliasedSymbol(symbol)
            : symbol;
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-unnecessary-type-assertion.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const tsutils_1 = require(627) /* tsutils */;
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-unnecessary-type-assertion',
        meta: {
            docs: {
                description: 'Warns if a type assertion does not change the type of an expression',
                category: 'Best Practices',
                recommended: 'error',
                requiresTypeChecking: true,
            },
            fixable: 'code',
            messages: {
                unnecessaryAssertion: 'This assertion is unnecessary since it does not change the type of the expression.',
                contextuallyUnnecessary: 'This assertion is unnecessary since the receiver accepts the original type of the expression.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        typesToIgnore: {
                            type: 'array',
                            items: {
                                type: 'string',
                            },
                        },
                    },
                },
            ],
            type: 'suggestion',
        },
        defaultOptions: [{}],
        create(context, [options]) {
            const sourceCode = context.getSourceCode();
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            const compilerOptions = parserServices.program.getCompilerOptions();
            /**
             * Sometimes tuple types don't have ObjectFlags.Tuple set, like when they're being matched against an inferred type.
             * So, in addition, check if there are integer properties 0..n and no other numeric keys
             */
            function couldBeTupleType(type) {
                const properties = type.getProperties();
                if (properties.length === 0) {
                    return false;
                }
                let i = 0;
                for (; i < properties.length; ++i) {
                    const name = properties[i].name;
                    if (String(i) !== name) {
                        if (i === 0) {
                            // if there are no integer properties, this is not a tuple
                            return false;
                        }
                        break;
                    }
                }
                for (; i < properties.length; ++i) {
                    if (String(+properties[i].name) === properties[i].name) {
                        return false; // if there are any other numeric properties, this is not a tuple
                    }
                }
                return true;
            }
            /**
             * Returns true if there's a chance the variable has been used before a value has been assigned to it
             */
            function isPossiblyUsedBeforeAssigned(node) {
                const declaration = util.getDeclaration(checker, node);
                if (!declaration) {
                    // don't know what the declaration is for some reason, so just assume the worst
                    return true;
                }
                if (
                // non-strict mode doesn't care about used before assigned errors
                tsutils_1.isStrictCompilerOptionEnabled(compilerOptions, 'strictNullChecks') &&
                    // ignore class properties as they are compile time guarded
                    // also ignore function arguments as they can't be used before defined
                    tsutils_1.isVariableDeclaration(declaration) &&
                    // is it `const x!: number`
                    declaration.initializer === undefined &&
                    declaration.exclamationToken === undefined &&
                    declaration.type !== undefined) {
                    // check if the defined variable type has changed since assignment
                    const declarationType = checker.getTypeFromTypeNode(declaration.type);
                    const type = util.getConstrainedTypeAtLocation(checker, node);
                    if (declarationType === type) {
                        // possibly used before assigned, so just skip it
                        // better to false negative and skip it, than false positive and fix to compile erroring code
                        //
                        // no better way to figure this out right now
                        // https://github.com/Microsoft/TypeScript/issues/31124
                        return true;
                    }
                }
                return false;
            }
            return {
                TSNonNullExpression(node) {
                    const originalNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                    const type = util.getConstrainedTypeAtLocation(checker, originalNode.expression);
                    if (!util.isNullableType(type)) {
                        if (isPossiblyUsedBeforeAssigned(originalNode.expression)) {
                            return;
                        }
                        context.report({
                            node,
                            messageId: 'unnecessaryAssertion',
                            fix(fixer) {
                                return fixer.removeRange([
                                    originalNode.expression.end,
                                    originalNode.end,
                                ]);
                            },
                        });
                    }
                    else {
                        // we know it's a nullable type
                        // so figure out if the variable is used in a place that accepts nullable types
                        const contextualType = util.getContextualType(checker, originalNode);
                        if (contextualType) {
                            // in strict mode you can't assign null to undefined, so we have to make sure that
                            // the two types share a nullable type
                            const typeIncludesUndefined = util.isTypeFlagSet(type, ts.TypeFlags.Undefined);
                            const typeIncludesNull = util.isTypeFlagSet(type, ts.TypeFlags.Null);
                            const contextualTypeIncludesUndefined = util.isTypeFlagSet(contextualType, ts.TypeFlags.Undefined);
                            const contextualTypeIncludesNull = util.isTypeFlagSet(contextualType, ts.TypeFlags.Null);
                            // make sure that the parent accepts the same types
                            // i.e. assigning `string | null | undefined` to `string | undefined` is invalid
                            const isValidUndefined = typeIncludesUndefined
                                ? contextualTypeIncludesUndefined
                                : true;
                            const isValidNull = typeIncludesNull
                                ? contextualTypeIncludesNull
                                : true;
                            if (isValidUndefined && isValidNull) {
                                context.report({
                                    node,
                                    messageId: 'contextuallyUnnecessary',
                                    fix(fixer) {
                                        return fixer.removeRange([
                                            originalNode.expression.end,
                                            originalNode.end,
                                        ]);
                                    },
                                });
                            }
                        }
                    }
                },
                'TSAsExpression, TSTypeAssertion'(node) {
                    var _a;
                    if ((_a = options.typesToIgnore) === null || _a === void 0 ? void 0 : _a.includes(sourceCode.getText(node.typeAnnotation))) {
                        return;
                    }
                    const originalNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                    const castType = checker.getTypeAtLocation(originalNode);
                    if (tsutils_1.isTypeFlagSet(castType, ts.TypeFlags.Literal) ||
                        (tsutils_1.isObjectType(castType) &&
                            (tsutils_1.isObjectFlagSet(castType, ts.ObjectFlags.Tuple) ||
                                couldBeTupleType(castType)))) {
                        // It's not always safe to remove a cast to a literal type or tuple
                        // type, as those types are sometimes widened without the cast.
                        return;
                    }
                    const uncastType = checker.getTypeAtLocation(originalNode.expression);
                    if (uncastType === castType) {
                        context.report({
                            node,
                            messageId: 'unnecessaryAssertion',
                            fix(fixer) {
                                return originalNode.kind === ts.SyntaxKind.TypeAssertionExpression
                                    ? fixer.removeRange([
                                        originalNode.getStart(),
                                        originalNode.expression.getStart(),
                                    ])
                                    : fixer.removeRange([
                                        originalNode.expression.end,
                                        originalNode.end,
                                    ]);
                            },
                        });
                    }
                    // TODO - add contextually unnecessary check for this
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-unsafe-call.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-unsafe-call',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallows calling an any type value',
                category: 'Possible Errors',
                recommended: false,
                requiresTypeChecking: true,
            },
            messages: {
                unsafeCall: 'Unsafe call of an any typed value',
                unsafeNew: 'Unsafe construction of an any type value',
                unsafeTemplateTag: 'Unsafe any typed template tag',
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            const { program, esTreeNodeToTSNodeMap } = util.getParserServices(context);
            const checker = program.getTypeChecker();
            function checkCall(node, reportingNode, messageId) {
                const tsNode = esTreeNodeToTSNodeMap.get(node);
                const type = checker.getTypeAtLocation(tsNode);
                if (util.isTypeAnyType(type)) {
                    context.report({
                        node: reportingNode,
                        messageId: messageId,
                    });
                }
            }
            return {
                'CallExpression, OptionalCallExpression'(node) {
                    checkCall(node.callee, node.callee, 'unsafeCall');
                },
                NewExpression(node) {
                    checkCall(node.callee, node, 'unsafeNew');
                },
                'TaggedTemplateExpression > *.tag'(node) {
                    checkCall(node, node, 'unsafeTemplateTag');
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-unsafe-member-access.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-unsafe-member-access',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallows member access on any typed variables',
                category: 'Possible Errors',
                recommended: false,
                requiresTypeChecking: true,
            },
            messages: {
                unsafeMemberExpression: 'Unsafe member access {{property}} on an any value',
                unsafeComputedMemberAccess: 'Computed name {{property}} resolves to an any value',
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            const { program, esTreeNodeToTSNodeMap } = util.getParserServices(context);
            const checker = program.getTypeChecker();
            const sourceCode = context.getSourceCode();
            const stateCache = new Map();
            function checkMemberExpression(node) {
                const cachedState = stateCache.get(node);
                if (cachedState) {
                    return cachedState;
                }
                if (util.isMemberOrOptionalMemberExpression(node.object)) {
                    const objectState = checkMemberExpression(node.object);
                    if (objectState === 1 /* Unsafe */) {
                        // if the object is unsafe, we know this will be unsafe as well
                        // we don't need to report, as we have already reported on the inner member expr
                        stateCache.set(node, objectState);
                        return objectState;
                    }
                }
                const tsNode = esTreeNodeToTSNodeMap.get(node.object);
                const type = checker.getTypeAtLocation(tsNode);
                const state = util.isTypeAnyType(type) ? 1 /* Unsafe */ : 2 /* Safe */;
                stateCache.set(node, state);
                if (state === 1 /* Unsafe */) {
                    const propertyName = sourceCode.getText(node.property);
                    context.report({
                        node,
                        messageId: 'unsafeMemberExpression',
                        data: {
                            property: node.computed ? `[${propertyName}]` : `.${propertyName}`,
                        },
                    });
                }
                return state;
            }
            return {
                'MemberExpression, OptionalMemberExpression': checkMemberExpression,
                ':matches(MemberExpression, OptionalMemberExpression)[computed = true] > *.property'(node) {
                    if (
                    // x[1]
                    node.type === experimental_utils_1.AST_NODE_TYPES.Literal ||
                        // x[1++] x[++x] etc
                        // FUN FACT - **all** update expressions return type number, regardless of the argument's type,
                        // because JS engines return NaN if there the argument is not a number.
                        node.type === experimental_utils_1.AST_NODE_TYPES.UpdateExpression) {
                        // perf optimizations - literals can obviously never be `any`
                        return;
                    }
                    const tsNode = esTreeNodeToTSNodeMap.get(node);
                    const type = checker.getTypeAtLocation(tsNode);
                    if (util.isTypeAnyType(type)) {
                        const propertyName = sourceCode.getText(node);
                        context.report({
                            node,
                            messageId: 'unsafeComputedMemberAccess',
                            data: {
                                property: `[${propertyName}]`,
                            },
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-unsafe-return.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const tsutils_1 = require(627) /* tsutils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-unsafe-return',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallows returning any from a function',
                category: 'Possible Errors',
                recommended: false,
                requiresTypeChecking: true,
            },
            messages: {
                unsafeReturn: 'Unsafe return of an {{type}} typed value',
                unsafeReturnAssignment: 'Unsafe return of type {{sender}} from function with return type {{receiver}}',
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            const { program, esTreeNodeToTSNodeMap } = util.getParserServices(context);
            const checker = program.getTypeChecker();
            function getParentFunctionNode(node) {
                let current = node.parent;
                while (current) {
                    if (current.type === experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression ||
                        current.type === experimental_utils_1.AST_NODE_TYPES.FunctionDeclaration ||
                        current.type === experimental_utils_1.AST_NODE_TYPES.FunctionExpression) {
                        return current;
                    }
                    current = current.parent;
                }
                // this shouldn't happen in correct code, but someone may attempt to parse bad code
                // the parser won't error, so we shouldn't throw here
                /* istanbul ignore next */ return null;
            }
            function checkReturn(returnNode, reportingNode = returnNode) {
                const tsNode = esTreeNodeToTSNodeMap.get(returnNode);
                const anyType = util.isAnyOrAnyArrayTypeDiscriminated(tsNode, checker);
                if (anyType !== 2 /* Safe */) {
                    return context.report({
                        node: reportingNode,
                        messageId: 'unsafeReturn',
                        data: {
                            type: anyType === 0 /* Any */ ? 'any' : 'any[]',
                        },
                    });
                }
                const functionNode = getParentFunctionNode(returnNode);
                /* istanbul ignore if */ if (!functionNode) {
                    return;
                }
                // function has an explicit return type, so ensure it's a safe return
                const returnNodeType = checker.getTypeAtLocation(esTreeNodeToTSNodeMap.get(returnNode));
                const functionTSNode = esTreeNodeToTSNodeMap.get(functionNode);
                // function expressions will not have their return type modified based on receiver typing
                // so we have to use the contextual typing in these cases, i.e.
                // const foo1: () => Set<string> = () => new Set<any>();
                // the return type of the arrow function is Set<any> even though the variable is typed as Set<string>
                let functionType = tsutils_1.isExpression(functionTSNode)
                    ? util.getContextualType(checker, functionTSNode)
                    : checker.getTypeAtLocation(functionTSNode);
                if (!functionType) {
                    functionType = checker.getTypeAtLocation(functionTSNode);
                }
                for (const signature of functionType.getCallSignatures()) {
                    const functionReturnType = signature.getReturnType();
                    if (returnNodeType === functionReturnType) {
                        // don't bother checking if they're the same
                        // either the function is explicitly declared to return the same type
                        // or there was no declaration, so the return type is implicit
                        return;
                    }
                    const result = util.isUnsafeAssignment(returnNodeType, functionReturnType, checker);
                    if (!result) {
                        return;
                    }
                    const { sender, receiver } = result;
                    return context.report({
                        node: reportingNode,
                        messageId: 'unsafeReturnAssignment',
                        data: {
                            sender: checker.typeToString(sender),
                            receiver: checker.typeToString(receiver),
                        },
                    });
                }
            }
            return {
                ReturnStatement(node) {
                    const argument = node.argument;
                    if (!argument) {
                        return;
                    }
                    checkReturn(argument, node);
                },
                'ArrowFunctionExpression > :not(BlockStatement).body': checkReturn,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-untyped-public-signature.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    exports.default = util.createRule({
        name: 'no-untyped-public-signature',
        meta: {
            deprecated: true,
            replacedBy: ['explicit-module-boundary-types'],
            docs: {
                description: 'Disallow untyped public methods',
                category: 'Best Practices',
                recommended: false,
            },
            messages: {
                noReturnType: 'Public method has no return type',
                untypedParameter: 'Public method parameters should be typed',
            },
            schema: [
                {
                    allowAdditionalProperties: false,
                    properties: {
                        ignoredMethods: {
                            type: 'array',
                            items: {
                                type: 'string',
                            },
                        },
                    },
                    type: 'object',
                },
            ],
            type: 'suggestion',
        },
        defaultOptions: [{ ignoredMethods: [] }],
        create(context, [options]) {
            const ignoredMethods = new Set(options.ignoredMethods);
            function isPublicMethod(node) {
                return node.accessibility === 'public' || !node.accessibility;
            }
            function isIgnoredMethod(node, ignoredMethods) {
                if (node.key.type === experimental_utils_1.AST_NODE_TYPES.Literal &&
                    typeof node.key.value === 'string') {
                    return ignoredMethods.has(node.key.value);
                }
                if (node.key.type === experimental_utils_1.AST_NODE_TYPES.TemplateLiteral &&
                    node.key.expressions.length === 0) {
                    return ignoredMethods.has(node.key.quasis[0].value.raw);
                }
                if (!node.computed && node.key.type === experimental_utils_1.AST_NODE_TYPES.Identifier) {
                    return ignoredMethods.has(node.key.name);
                }
                return false;
            }
            function isParamTyped(node) {
                return (!!node.typeAnnotation &&
                    node.typeAnnotation.typeAnnotation.type !== experimental_utils_1.AST_NODE_TYPES.TSAnyKeyword);
            }
            function isReturnTyped(node) {
                if (!node) {
                    return false;
                }
                return (node.typeAnnotation &&
                    node.typeAnnotation.type !== experimental_utils_1.AST_NODE_TYPES.TSAnyKeyword);
            }
            return {
                'TSAbstractMethodDefinition, MethodDefinition'(node) {
                    if (isPublicMethod(node) && !isIgnoredMethod(node, ignoredMethods)) {
                        const paramIdentifiers = node.value.params.filter(param => param.type === experimental_utils_1.AST_NODE_TYPES.Identifier);
                        const identifiersHaveTypes = paramIdentifiers.every(isParamTyped);
                        if (!identifiersHaveTypes) {
                            context.report({
                                node,
                                messageId: 'untypedParameter',
                                data: {},
                            });
                        }
                        if (node.kind !== 'constructor' &&
                            node.kind !== 'set' &&
                            !isReturnTyped(node.value.returnType)) {
                            context.report({
                                node,
                                messageId: 'noReturnType',
                                data: {},
                            });
                        }
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-unused-expressions.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const no_unused_expressions_1 = __importDefault(require(342) /* eslint/lib/rules/no-unused-expressions */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-unused-expressions',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallow unused expressions',
                category: 'Best Practices',
                recommended: false,
                extendsBaseRule: true,
            },
            schema: no_unused_expressions_1.default.meta.schema,
            messages: {},
        },
        defaultOptions: [],
        create(context) {
            const rules = no_unused_expressions_1.default.create(context);
            return {
                ExpressionStatement(node) {
                    if (node.directive ||
                        node.expression.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression) {
                        return;
                    }
                    rules.ExpressionStatement(node);
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-unused-vars.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const no_unused_vars_1 = __importDefault(require(344) /* eslint/lib/rules/no-unused-vars */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-unused-vars',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallow unused variables',
                category: 'Variables',
                recommended: 'warn',
                extendsBaseRule: true,
            },
            schema: no_unused_vars_1.default.meta.schema,
            messages: no_unused_vars_1.default.meta.messages,
        },
        defaultOptions: [],
        create(context) {
            const rules = no_unused_vars_1.default.create(context);
            /**
             * Mark heritage clause as used
             * @param node The node currently being traversed
             */
            function markHeritageAsUsed(node) {
                switch (node.type) {
                    case experimental_utils_1.AST_NODE_TYPES.Identifier:
                        context.markVariableAsUsed(node.name);
                        break;
                    case experimental_utils_1.AST_NODE_TYPES.MemberExpression:
                        markHeritageAsUsed(node.object);
                        break;
                    case experimental_utils_1.AST_NODE_TYPES.CallExpression:
                        markHeritageAsUsed(node.callee);
                        break;
                }
            }
            return Object.assign({}, rules, {
                'TSTypeReference Identifier'(node) {
                    context.markVariableAsUsed(node.name);
                },
                TSInterfaceHeritage(node) {
                    if (node.expression) {
                        markHeritageAsUsed(node.expression);
                    }
                },
                TSClassImplements(node) {
                    if (node.expression) {
                        markHeritageAsUsed(node.expression);
                    }
                },
                'TSParameterProperty Identifier'(node) {
                    // just assume parameter properties are used
                    context.markVariableAsUsed(node.name);
                },
                'TSEnumMember Identifier'(node) {
                    context.markVariableAsUsed(node.name);
                },
                '*[declare=true] Identifier'(node) {
                    context.markVariableAsUsed(node.name);
                    const scope = context.getScope();
                    const { variableScope } = scope;
                    if (variableScope !== scope) {
                        const superVar = variableScope.set.get(node.name);
                        if (superVar) {
                            superVar.eslintUsed = true;
                        }
                    }
                },
            });
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-unused-vars-experimental.js */ function _(require, module, exports) {
    /* eslint-disable no-fallthrough */
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    exports.DEFAULT_IGNORED_REGEX_STRING = '^_';
    exports.default = util.createRule({
        name: 'no-unused-vars-experimental',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallow unused variables and arguments',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        ignoredNamesRegex: {
                            oneOf: [
                                {
                                    type: 'string',
                                },
                                {
                                    type: 'boolean',
                                    enum: [false],
                                },
                            ],
                        },
                        ignoreArgsIfArgsAfterAreUsed: {
                            type: 'boolean',
                        },
                    },
                    additionalProperties: false,
                },
            ],
            messages: {
                unused: "{{type}} '{{name}}' is declared but its value is never read.",
                unusedWithIgnorePattern: "{{type}} '{{name}}' is declared but its value is never read. Allowed unused names must match {{pattern}}",
                unusedImport: 'All imports in import declaration are unused.',
                unusedTypeParameters: 'All type parameters are unused.',
            },
        },
        defaultOptions: [
            {
                ignoredNamesRegex: exports.DEFAULT_IGNORED_REGEX_STRING,
                ignoreArgsIfArgsAfterAreUsed: false,
            },
        ],
        create(context, [userOptions]) {
            var _a;
            const parserServices = util.getParserServices(context);
            const tsProgram = parserServices.program;
            const afterAllDiagnosticsCallbacks = [];
            const options = {
                ignoredNames: userOptions && typeof userOptions.ignoredNamesRegex === 'string'
                    ? new RegExp(userOptions.ignoredNamesRegex)
                    : null,
                ignoreArgsIfArgsAfterAreUsed: (_a = userOptions.ignoreArgsIfArgsAfterAreUsed) !== null && _a !== void 0 ? _a : false,
            };
            function handleIdentifier(identifier) {
                function report(type) {
                    const node = parserServices.tsNodeToESTreeNodeMap.get(identifier);
                    const regex = options.ignoredNames;
                    const name = identifier.getText();
                    if (regex) {
                        if (!regex.test(name)) {
                            context.report({
                                node,
                                messageId: 'unusedWithIgnorePattern',
                                data: {
                                    name,
                                    type,
                                    pattern: regex.toString(),
                                },
                            });
                        }
                    }
                    else {
                        context.report({
                            node,
                            messageId: 'unused',
                            data: {
                                name,
                                type,
                            },
                        });
                    }
                }
                const parent = identifier.parent;
                // is a single variable diagnostic
                switch (parent.kind) {
                    case ts.SyntaxKind.BindingElement:
                    case ts.SyntaxKind.ObjectBindingPattern:
                        report('Destructured Variable');
                        break;
                    case ts.SyntaxKind.ClassDeclaration:
                        report('Class');
                        break;
                    case ts.SyntaxKind.EnumDeclaration:
                        report('Enum');
                        break;
                    case ts.SyntaxKind.FunctionDeclaration:
                        report('Function');
                        break;
                    // this won't happen because there are specific nodes that wrap up named/default import identifiers
                    // case ts.SyntaxKind.ImportDeclaration:
                    // import equals is always treated as a variable
                    case ts.SyntaxKind.ImportEqualsDeclaration:
                    // the default import is NOT used, but a named import is used
                    case ts.SyntaxKind.ImportClause:
                    // a named import is NOT used, but either another named import, or the default import is used
                    case ts.SyntaxKind.ImportSpecifier:
                    // a namespace import is NOT used, but the default import is used
                    case ts.SyntaxKind.NamespaceImport:
                        // eslint-disable-next-line @typescript-eslint/internal/prefer-ast-types-enum
                        report('Import');
                        break;
                    case ts.SyntaxKind.InterfaceDeclaration:
                        report('Interface');
                        break;
                    case ts.SyntaxKind.MethodDeclaration:
                        report('Method');
                        break;
                    case ts.SyntaxKind.Parameter:
                        handleParameterDeclaration(identifier, parent);
                        break;
                    case ts.SyntaxKind.PropertyDeclaration:
                        // eslint-disable-next-line @typescript-eslint/internal/prefer-ast-types-enum
                        report('Property');
                        break;
                    case ts.SyntaxKind.TypeAliasDeclaration:
                        report('Type');
                        break;
                    case ts.SyntaxKind.TypeParameter:
                        handleTypeParam(identifier);
                        break;
                    case ts.SyntaxKind.VariableDeclaration:
                        report('Variable');
                        break;
                    default:
                        throw new Error(`Unknown node with kind ${parent.kind}.`);
                    // TODO - should we just handle this gracefully?
                    // report('Unknown Node');
                    // break;
                }
            }
            const unusedParameters = new Set();
            function handleParameterDeclaration(identifier, parent) {
                const name = identifier.getText();
                // regardless of if the paramter is ignored, track that it had a diagnostic fired on it
                unusedParameters.add(identifier);
                /*
                NOTE - Typescript will automatically ignore parameters that have a
                       leading underscore in their name. We cannot do anything about this.
                */
                function report() {
                    const node = parserServices.tsNodeToESTreeNodeMap.get(identifier);
                    context.report({
                        node,
                        messageId: 'unused',
                        data: {
                            name,
                            type: 'Parameter',
                        },
                    });
                }
                const isLastParameter = parent.parent.parameters.indexOf(parent) ===
                    parent.parent.parameters.length - 1;
                if (!isLastParameter && options.ignoreArgsIfArgsAfterAreUsed) {
                    // once all diagnostics are processed, we can check if the following args are unused
                    afterAllDiagnosticsCallbacks.push(() => {
                        for (const param of parent.parent.parameters) {
                            if (!unusedParameters.has(param.name)) {
                                return;
                            }
                        }
                        // none of the following params were unused, so report
                        report();
                    });
                }
                else {
                    report();
                }
            }
            function handleImportDeclaration(parent) {
                // the entire import statement is unused
                /*
                NOTE - Typescript will automatically ignore imports that have a
                       leading underscore in their name. We cannot do anything about this.
                */
                context.report({
                    messageId: 'unusedImport',
                    node: parserServices.tsNodeToESTreeNodeMap.get(parent),
                });
            }
            function handleDestructure(parent) {
                // the entire destructure is unused
                // note that this case only ever triggers for simple, single-level destructured objects
                // i.e. these will not trigger it:
                // - const {a:_a, b, c: {d}} = z;
                // - const [a, b] = c;
                parent.elements.forEach(element => {
                    if (element.kind === ts.SyntaxKind.BindingElement) {
                        const name = element.name;
                        if (name.kind === ts.SyntaxKind.Identifier) {
                            handleIdentifier(name);
                        }
                    }
                });
            }
            function handleTypeParamList(node) {
                // the entire generic decl list is unused
                /*
                NOTE - Typescript will automatically ignore generics that have a
                       leading underscore in their name. We cannot do anything about this.
                */
                const parent = parserServices.tsNodeToESTreeNodeMap.get(node);
                context.report({
                    messageId: 'unusedTypeParameters',
                    node: parent.typeParameters,
                });
            }
            function handleTypeParam(identifier) {
                context.report({
                    node: parserServices.tsNodeToESTreeNodeMap.get(identifier),
                    messageId: 'unused',
                    data: {
                        name: identifier.getText(),
                        type: 'Type Parameter',
                    },
                });
            }
            return {
                'Program:exit'(program) {
                    const tsNode = parserServices.esTreeNodeToTSNodeMap.get(program);
                    const sourceFile = util.getSourceFileOfNode(tsNode);
                    const diagnostics = tsProgram.getSemanticDiagnostics(sourceFile);
                    diagnostics.forEach(diag => {
                        if (isUnusedDiagnostic(diag.code)) {
                            if (diag.start !== undefined) {
                                const node = util.getTokenAtPosition(sourceFile, diag.start);
                                const parent = node.parent;
                                if (isIdentifier(node)) {
                                    handleIdentifier(node);
                                }
                                else if (isImport(parent)) {
                                    handleImportDeclaration(parent);
                                }
                                else if (isDestructure(parent)) {
                                    handleDestructure(parent);
                                }
                                else if (isGeneric(node, parent)) {
                                    handleTypeParamList(parent);
                                }
                            }
                        }
                    });
                    // trigger all the checks to be done after all the diagnostics have been evaluated
                    afterAllDiagnosticsCallbacks.forEach(cb => cb());
                },
            };
        },
    });
    /**
     * Checks if the diagnostic code is one of the expected "unused var" codes
     */
    function isUnusedDiagnostic(code) {
        return [
            6133,
            6138,
            6192,
            6196,
            6198,
            6199,
            6205,
        ].includes(code);
    }
    /**
     * Checks if the given node is a destructuring pattern
     */
    function isDestructure(node) {
        return (node.kind === ts.SyntaxKind.ObjectBindingPattern ||
            node.kind === ts.SyntaxKind.ArrayBindingPattern);
    }
    function isImport(node) {
        return node.kind === ts.SyntaxKind.ImportDeclaration;
    }
    function isIdentifier(node) {
        return node.kind === ts.SyntaxKind.Identifier;
    }
    function isGeneric(node, parent) {
        return (node.kind === ts.SyntaxKind.LessThanToken &&
            parent.typeParameters !== undefined);
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-use-before-define.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    const SENTINEL_TYPE = /^(?:(?:Function|Class)(?:Declaration|Expression)|ArrowFunctionExpression|CatchClause|ImportDeclaration|ExportNamedDeclaration)$/;
    /**
     * Parses a given value as options.
     */
    function parseOptions(options) {
        let functions = true;
        let classes = true;
        let enums = true;
        let variables = true;
        let typedefs = true;
        if (typeof options === 'string') {
            functions = options !== 'nofunc';
        }
        else if (typeof options === 'object' && options !== null) {
            functions = options.functions !== false;
            classes = options.classes !== false;
            enums = options.enums !== false;
            variables = options.variables !== false;
            typedefs = options.typedefs !== false;
        }
        return { functions, classes, enums, variables, typedefs };
    }
    /**
     * Checks whether or not a given scope is a top level scope.
     */
    function isTopLevelScope(scope) {
        return scope.type === 'module' || scope.type === 'global';
    }
    /**
     * Checks whether or not a given variable declaration in an upper scope.
     */
    function isOuterScope(variable, reference) {
        if (variable.scope.variableScope === reference.from.variableScope) {
            // allow the same scope only if it's the top level global/module scope
            if (!isTopLevelScope(variable.scope.variableScope)) {
                return false;
            }
        }
        return true;
    }
    /**
     * Checks whether or not a given variable is a function declaration.
     */
    function isFunction(variable) {
        return variable.defs[0].type === 'FunctionName';
    }
    /**
     * Checks whether or not a given variable is a enum declaration in an upper function scope.
     */
    function isOuterEnum(variable, reference) {
        const node = variable.defs[0].node;
        return (node.type === experimental_utils_1.AST_NODE_TYPES.TSEnumDeclaration &&
            isOuterScope(variable, reference));
    }
    /**
     * Checks whether or not a given variable is a class declaration in an upper function scope.
     */
    function isOuterClass(variable, reference) {
        return (variable.defs[0].type === 'ClassName' && isOuterScope(variable, reference));
    }
    /**
     * Checks whether or not a given variable is a variable declaration in an upper function scope.
     */
    function isOuterVariable(variable, reference) {
        return (variable.defs[0].type === 'Variable' && isOuterScope(variable, reference));
    }
    /**
     * Checks whether or not a given location is inside of the range of a given node.
     */
    function isInRange(node, location) {
        return !!node && node.range[0] <= location && location <= node.range[1];
    }
    /**
     * Checks whether or not a given reference is inside of the initializers of a given variable.
     *
     * @returns `true` in the following cases:
     * - var a = a
     * - var [a = a] = list
     * - var {a = a} = obj
     * - for (var a in a) {}
     * - for (var a of a) {}
     */
    function isInInitializer(variable, reference) {
        if (variable.scope !== reference.from) {
            return false;
        }
        let node = variable.identifiers[0].parent;
        const location = reference.identifier.range[1];
        while (node) {
            if (node.type === experimental_utils_1.AST_NODE_TYPES.VariableDeclarator) {
                if (isInRange(node.init, location)) {
                    return true;
                }
                if (node.parent &&
                    node.parent.parent &&
                    (node.parent.parent.type === experimental_utils_1.AST_NODE_TYPES.ForInStatement ||
                        node.parent.parent.type === experimental_utils_1.AST_NODE_TYPES.ForOfStatement) &&
                    isInRange(node.parent.parent.right, location)) {
                    return true;
                }
                break;
            }
            else if (node.type === experimental_utils_1.AST_NODE_TYPES.AssignmentPattern) {
                if (isInRange(node.right, location)) {
                    return true;
                }
            }
            else if (SENTINEL_TYPE.test(node.type)) {
                break;
            }
            node = node.parent;
        }
        return false;
    }
    exports.default = util.createRule({
        name: 'no-use-before-define',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallow the use of variables before they are defined',
                category: 'Variables',
                recommended: 'error',
                extendsBaseRule: true,
            },
            messages: {
                noUseBeforeDefine: "'{{name}}' was used before it was defined.",
            },
            schema: [
                {
                    oneOf: [
                        {
                            enum: ['nofunc'],
                        },
                        {
                            type: 'object',
                            properties: {
                                functions: { type: 'boolean' },
                                classes: { type: 'boolean' },
                                enums: { type: 'boolean' },
                                variables: { type: 'boolean' },
                                typedefs: { type: 'boolean' },
                            },
                            additionalProperties: false,
                        },
                    ],
                },
            ],
        },
        defaultOptions: [
            {
                functions: true,
                classes: true,
                enums: true,
                variables: true,
                typedefs: true,
            },
        ],
        create(context, optionsWithDefault) {
            const options = parseOptions(optionsWithDefault[0]);
            /**
             * Determines whether a given use-before-define case should be reported according to the options.
             * @param variable The variable that gets used before being defined
             * @param reference The reference to the variable
             */
            function isForbidden(variable, reference) {
                if (isFunction(variable)) {
                    return !!options.functions;
                }
                if (isOuterClass(variable, reference)) {
                    return !!options.classes;
                }
                if (isOuterVariable(variable, reference)) {
                    return !!options.variables;
                }
                if (isOuterEnum(variable, reference)) {
                    return !!options.enums;
                }
                return true;
            }
            /**
             * Finds and validates all variables in a given scope.
             */
            function findVariablesInScope(scope) {
                scope.references.forEach(reference => {
                    const variable = reference.resolved;
                    // Skips when the reference is:
                    // - initializations.
                    // - referring to an undefined variable.
                    // - referring to a global environment variable (there're no identifiers).
                    // - located preceded by the variable (except in initializers).
                    // - allowed by options.
                    if (reference.init ||
                        !variable ||
                        variable.identifiers.length === 0 ||
                        (variable.identifiers[0].range[1] < reference.identifier.range[1] &&
                            !isInInitializer(variable, reference)) ||
                        !isForbidden(variable, reference)) {
                        return;
                    }
                    // Reports.
                    context.report({
                        node: reference.identifier,
                        messageId: 'noUseBeforeDefine',
                        data: {
                            name: reference.identifier.name,
                        },
                    });
                });
                scope.childScopes.forEach(findVariablesInScope);
            }
            return {
                Program() {
                    findVariablesInScope(context.getScope());
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-useless-constructor.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const no_useless_constructor_1 = __importDefault(require(350) /* eslint/lib/rules/no-useless-constructor */);
    const util = __importStar(require(622) /* ../util */);
    /**
     * Check if method with accessibility is not useless
     */
    function checkAccessibility(node) {
        switch (node.accessibility) {
            case 'protected':
            case 'private':
                return false;
            case 'public':
                if (node.parent &&
                    node.parent.type === experimental_utils_1.AST_NODE_TYPES.ClassBody &&
                    node.parent.parent &&
                    'superClass' in node.parent.parent &&
                    node.parent.parent.superClass) {
                    return false;
                }
                break;
        }
        return true;
    }
    /**
     * Check if method is not unless due to typescript parameter properties
     */
    function checkParams(node) {
        return (!node.value.params ||
            !node.value.params.some(param => param.type === experimental_utils_1.AST_NODE_TYPES.TSParameterProperty));
    }
    exports.default = util.createRule({
        name: 'no-useless-constructor',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallow unnecessary constructors',
                category: 'Best Practices',
                recommended: false,
                extendsBaseRule: true,
            },
            schema: no_useless_constructor_1.default.meta.schema,
            messages: no_useless_constructor_1.default.meta.messages,
        },
        defaultOptions: [],
        create(context) {
            const rules = no_useless_constructor_1.default.create(context);
            return {
                MethodDefinition(node) {
                    if (node.value &&
                        node.value.type === experimental_utils_1.AST_NODE_TYPES.FunctionExpression &&
                        node.value.body &&
                        checkAccessibility(node) &&
                        checkParams(node)) {
                        rules.MethodDefinition(node);
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/no-var-requires.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'no-var-requires',
        meta: {
            type: 'problem',
            docs: {
                description: 'Disallows the use of require statements except in import statements',
                category: 'Best Practices',
                recommended: 'error',
            },
            messages: {
                noVarReqs: 'Require statement not part of import statement.',
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            return {
                'CallExpression, OptionalCallExpression'(node) {
                    if (node.callee.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                        node.callee.name === 'require' &&
                        node.parent &&
                        (node.parent.type === experimental_utils_1.AST_NODE_TYPES.VariableDeclarator ||
                            node.parent.type === experimental_utils_1.AST_NODE_TYPES.CallExpression ||
                            node.parent.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression)) {
                        context.report({
                            node,
                            messageId: 'noVarReqs',
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/prefer-as-const.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'prefer-as-const',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Prefer usage of `as const` over literal type',
                category: 'Best Practices',
                recommended: false,
            },
            fixable: 'code',
            messages: {
                preferConstAssertion: 'Expected a `const` instead of a literal type assertion',
                variableConstAssertion: 'Expected a `const` assertion instead of a literal type annotation',
                variableSuggest: 'You should use `as const` instead of type annotation.',
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            function compareTypes(valueNode, typeNode, canFix) {
                if (valueNode.type === experimental_utils_1.AST_NODE_TYPES.Literal &&
                    typeNode.type === experimental_utils_1.AST_NODE_TYPES.TSLiteralType &&
                    'raw' in typeNode.literal &&
                    valueNode.raw === typeNode.literal.raw) {
                    if (canFix) {
                        context.report({
                            node: typeNode,
                            messageId: 'preferConstAssertion',
                            fix: fixer => fixer.replaceText(typeNode, 'const'),
                        });
                    }
                    else {
                        context.report({
                            node: typeNode,
                            messageId: 'variableConstAssertion',
                            suggest: [
                                {
                                    messageId: 'variableSuggest',
                                    fix: (fixer) => [
                                        fixer.remove(typeNode.parent),
                                        fixer.insertTextAfter(valueNode, ' as const'),
                                    ],
                                },
                            ],
                        });
                    }
                }
            }
            return {
                TSAsExpression(node) {
                    compareTypes(node.expression, node.typeAnnotation, true);
                },
                TSTypeAssertion(node) {
                    compareTypes(node.expression, node.typeAnnotation, true);
                },
                VariableDeclarator(node) {
                    if (node.init && node.id.typeAnnotation) {
                        compareTypes(node.init, node.id.typeAnnotation.typeAnnotation, false);
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/prefer-for-of.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'prefer-for-of',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Prefer a ‘for-of’ loop over a standard ‘for’ loop if the index is only used to access the array being iterated',
                category: 'Stylistic Issues',
                recommended: false,
            },
            messages: {
                preferForOf: 'Expected a `for-of` loop instead of a `for` loop with this simple iteration',
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            function isSingleVariableDeclaration(node) {
                return (node !== null &&
                    node.type === experimental_utils_1.AST_NODE_TYPES.VariableDeclaration &&
                    node.kind !== 'const' &&
                    node.declarations.length === 1);
            }
            function isLiteral(node, value) {
                return node.type === experimental_utils_1.AST_NODE_TYPES.Literal && node.value === value;
            }
            function isZeroInitialized(node) {
                return node.init !== null && isLiteral(node.init, 0);
            }
            function isMatchingIdentifier(node, name) {
                return node.type === experimental_utils_1.AST_NODE_TYPES.Identifier && node.name === name;
            }
            function isLessThanLengthExpression(node, name) {
                if (node !== null &&
                    node.type === experimental_utils_1.AST_NODE_TYPES.BinaryExpression &&
                    node.operator === '<' &&
                    isMatchingIdentifier(node.left, name) &&
                    (node.right.type === experimental_utils_1.AST_NODE_TYPES.MemberExpression ||
                        node.right.type === experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression) &&
                    isMatchingIdentifier(node.right.property, 'length')) {
                    return node.right.object;
                }
                return null;
            }
            function isIncrement(node, name) {
                if (!node) {
                    return false;
                }
                switch (node.type) {
                    case experimental_utils_1.AST_NODE_TYPES.UpdateExpression:
                        // x++ or ++x
                        return (node.operator === '++' && isMatchingIdentifier(node.argument, name));
                    case experimental_utils_1.AST_NODE_TYPES.AssignmentExpression:
                        if (isMatchingIdentifier(node.left, name)) {
                            if (node.operator === '+=') {
                                // x += 1
                                return isLiteral(node.right, 1);
                            }
                            else if (node.operator === '=') {
                                // x = x + 1 or x = 1 + x
                                const expr = node.right;
                                return (expr.type === experimental_utils_1.AST_NODE_TYPES.BinaryExpression &&
                                    expr.operator === '+' &&
                                    ((isMatchingIdentifier(expr.left, name) &&
                                        isLiteral(expr.right, 1)) ||
                                        (isLiteral(expr.left, 1) &&
                                            isMatchingIdentifier(expr.right, name))));
                            }
                        }
                }
                return false;
            }
            function contains(outer, inner) {
                return (outer.range[0] <= inner.range[0] && outer.range[1] >= inner.range[1]);
            }
            function isAssignee(node) {
                var _a;
                const parent = node.parent;
                if (!parent) {
                    return false;
                }
                // a[i] = 1, a[i] += 1, etc.
                if (parent.type === experimental_utils_1.AST_NODE_TYPES.AssignmentExpression &&
                    parent.left === node) {
                    return true;
                }
                // delete a[i]
                if (parent.type === experimental_utils_1.AST_NODE_TYPES.UnaryExpression &&
                    parent.operator === 'delete' &&
                    parent.argument === node) {
                    return true;
                }
                // a[i]++, --a[i], etc.
                if (parent.type === experimental_utils_1.AST_NODE_TYPES.UpdateExpression &&
                    parent.argument === node) {
                    return true;
                }
                // [a[i]] = [0]
                if (parent.type === experimental_utils_1.AST_NODE_TYPES.ArrayPattern) {
                    return true;
                }
                // [...a[i]] = [0]
                if (parent.type === experimental_utils_1.AST_NODE_TYPES.RestElement) {
                    return true;
                }
                // ({ foo: a[i] }) = { foo: 0 }
                if (parent.type === experimental_utils_1.AST_NODE_TYPES.Property &&
                    parent.value === node &&
                    ((_a = parent.parent) === null || _a === void 0 ? void 0 : _a.type) === experimental_utils_1.AST_NODE_TYPES.ObjectExpression &&
                    isAssignee(parent.parent)) {
                    return true;
                }
                return false;
            }
            function isIndexOnlyUsedWithArray(body, indexVar, arrayExpression) {
                const sourceCode = context.getSourceCode();
                const arrayText = sourceCode.getText(arrayExpression);
                return indexVar.references.every(reference => {
                    const id = reference.identifier;
                    const node = id.parent;
                    return (!contains(body, id) ||
                        (node !== undefined &&
                            (node.type === experimental_utils_1.AST_NODE_TYPES.MemberExpression ||
                                node.type === experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression) &&
                            node.property === id &&
                            sourceCode.getText(node.object) === arrayText &&
                            !isAssignee(node)));
                });
            }
            return {
                'ForStatement:exit'(node) {
                    if (!isSingleVariableDeclaration(node.init)) {
                        return;
                    }
                    const declarator = node.init.declarations[0];
                    if (!declarator ||
                        !isZeroInitialized(declarator) ||
                        declarator.id.type !== experimental_utils_1.AST_NODE_TYPES.Identifier) {
                        return;
                    }
                    const indexName = declarator.id.name;
                    const arrayExpression = isLessThanLengthExpression(node.test, indexName);
                    if (!arrayExpression) {
                        return;
                    }
                    const [indexVar] = context.getDeclaredVariables(node.init);
                    if (isIncrement(node.update, indexName) &&
                        isIndexOnlyUsedWithArray(node.body, indexVar, arrayExpression)) {
                        context.report({
                            node,
                            messageId: 'preferForOf',
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/prefer-function-type.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'prefer-function-type',
        meta: {
            docs: {
                description: 'Use function types instead of interfaces with call signatures',
                category: 'Best Practices',
                recommended: false,
            },
            fixable: 'code',
            messages: {
                functionTypeOverCallableType: "{{ type }} has only a call signature - use '{{ sigSuggestion }}' instead.",
            },
            schema: [],
            type: 'suggestion',
        },
        defaultOptions: [],
        create(context) {
            const sourceCode = context.getSourceCode();
            /**
             * Checks if there the interface has exactly one supertype that isn't named 'Function'
             * @param node The node being checked
             */
            function hasOneSupertype(node) {
                if (!node.extends || node.extends.length === 0) {
                    return false;
                }
                if (node.extends.length !== 1) {
                    return true;
                }
                const expr = node.extends[0].expression;
                return (expr.type !== experimental_utils_1.AST_NODE_TYPES.Identifier || expr.name !== 'Function');
            }
            /**
             * @param parent The parent of the call signature causing the diagnostic
             */
            function shouldWrapSuggestion(parent) {
                if (!parent) {
                    return false;
                }
                switch (parent.type) {
                    case experimental_utils_1.AST_NODE_TYPES.TSUnionType:
                    case experimental_utils_1.AST_NODE_TYPES.TSIntersectionType:
                    case experimental_utils_1.AST_NODE_TYPES.TSArrayType:
                        return true;
                    default:
                        return false;
                }
            }
            /**
             * @param call The call signature causing the diagnostic
             * @param parent The parent of the call
             * @returns The suggestion to report
             */
            function renderSuggestion(call, parent) {
                const start = call.range[0];
                const colonPos = call.returnType.range[0] - start;
                const text = sourceCode.getText().slice(start, call.range[1]);
                let suggestion = `${text.slice(0, colonPos)} =>${text.slice(colonPos + 1)}`;
                if (shouldWrapSuggestion(parent.parent)) {
                    suggestion = `(${suggestion})`;
                }
                if (parent.type === experimental_utils_1.AST_NODE_TYPES.TSInterfaceDeclaration) {
                    if (typeof parent.typeParameters !== 'undefined') {
                        return `type ${sourceCode
                            .getText()
                            .slice(parent.id.range[0], parent.typeParameters.range[1])} = ${suggestion}`;
                    }
                    return `type ${parent.id.name} = ${suggestion}`;
                }
                return suggestion.endsWith(';') ? suggestion.slice(0, -1) : suggestion;
            }
            /**
             * @param member The TypeElement being checked
             * @param node The parent of member being checked
             */
            function checkMember(member, node) {
                if ((member.type === experimental_utils_1.AST_NODE_TYPES.TSCallSignatureDeclaration ||
                    member.type === experimental_utils_1.AST_NODE_TYPES.TSConstructSignatureDeclaration) &&
                    typeof member.returnType !== 'undefined') {
                    const suggestion = renderSuggestion(member, node);
                    const fixStart = node.type === experimental_utils_1.AST_NODE_TYPES.TSTypeLiteral
                        ? node.range[0]
                        : sourceCode
                            .getTokens(node)
                            .filter(token => token.type === experimental_utils_1.AST_TOKEN_TYPES.Keyword &&
                            token.value === 'interface')[0].range[0];
                    context.report({
                        node: member,
                        messageId: 'functionTypeOverCallableType',
                        data: {
                            type: node.type === experimental_utils_1.AST_NODE_TYPES.TSTypeLiteral
                                ? 'Type literal'
                                : 'Interface',
                            sigSuggestion: suggestion,
                        },
                        fix(fixer) {
                            return fixer.replaceTextRange([fixStart, node.range[1]], suggestion);
                        },
                    });
                }
            }
            return {
                TSInterfaceDeclaration(node) {
                    if (!hasOneSupertype(node) && node.body.body.length === 1) {
                        checkMember(node.body.body[0], node);
                    }
                },
                'TSTypeLiteral[members.length = 1]'(node) {
                    checkMember(node.members[0], node);
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/prefer-includes.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const eslint_utils_1 = require(250) /* eslint-utils */;
    const regexpp_1 = require(226) /* regexpp */;
    const ts = __importStar(require(44) /* typescript */);
    const util_1 = require(622) /* ../util */;
    exports.default = util_1.createRule({
        name: 'prefer-includes',
        defaultOptions: [],
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Enforce `includes` method over `indexOf` method',
                category: 'Best Practices',
                recommended: 'error',
                requiresTypeChecking: true,
            },
            fixable: 'code',
            messages: {
                preferIncludes: "Use 'includes()' method instead.",
                preferStringIncludes: 'Use `String#includes()` method with a string instead.',
            },
            schema: [],
        },
        create(context) {
            const globalScope = context.getScope();
            const services = util_1.getParserServices(context);
            const types = services.program.getTypeChecker();
            function isNumber(node, value) {
                const evaluated = eslint_utils_1.getStaticValue(node, globalScope);
                return evaluated !== null && evaluated.value === value;
            }
            function isPositiveCheck(node) {
                switch (node.operator) {
                    case '!==':
                    case '!=':
                    case '>':
                        return isNumber(node.right, -1);
                    case '>=':
                        return isNumber(node.right, 0);
                    default:
                        return false;
                }
            }
            function isNegativeCheck(node) {
                switch (node.operator) {
                    case '===':
                    case '==':
                    case '<=':
                        return isNumber(node.right, -1);
                    case '<':
                        return isNumber(node.right, 0);
                    default:
                        return false;
                }
            }
            function hasSameParameters(nodeA, nodeB) {
                if (!ts.isFunctionLike(nodeA) || !ts.isFunctionLike(nodeB)) {
                    return false;
                }
                const paramsA = nodeA.parameters;
                const paramsB = nodeB.parameters;
                if (paramsA.length !== paramsB.length) {
                    return false;
                }
                for (let i = 0; i < paramsA.length; ++i) {
                    const paramA = paramsA[i];
                    const paramB = paramsB[i];
                    // Check name, type, and question token once.
                    if (paramA.getText() !== paramB.getText()) {
                        return false;
                    }
                }
                return true;
            }
            /**
             * Parse a given node if it's a `RegExp` instance.
             * @param node The node to parse.
             */
            function parseRegExp(node) {
                const evaluated = eslint_utils_1.getStaticValue(node, globalScope);
                if (evaluated == null || !(evaluated.value instanceof RegExp)) {
                    return null;
                }
                const { pattern, flags } = regexpp_1.parseRegExpLiteral(evaluated.value);
                if (pattern.alternatives.length !== 1 ||
                    flags.ignoreCase ||
                    flags.global) {
                    return null;
                }
                // Check if it can determine a unique string.
                const chars = pattern.alternatives[0].elements;
                if (!chars.every(c => c.type === 'Character')) {
                    return null;
                }
                // To string.
                return String.fromCodePoint(...chars.map(c => c.value));
            }
            return {
                "BinaryExpression > :matches(CallExpression, OptionalCallExpression).left > :matches(MemberExpression, OptionalMemberExpression).callee[property.name='indexOf'][computed=false]"(node) {
                    // Check if the comparison is equivalent to `includes()`.
                    const callNode = node.parent;
                    const compareNode = callNode.parent;
                    const negative = isNegativeCheck(compareNode);
                    if (!negative && !isPositiveCheck(compareNode)) {
                        return;
                    }
                    // Get the symbol of `indexOf` method.
                    const tsNode = services.esTreeNodeToTSNodeMap.get(node.property);
                    const indexofMethodSymbol = types.getSymbolAtLocation(tsNode);
                    if (indexofMethodSymbol == null ||
                        indexofMethodSymbol.declarations.length === 0) {
                        return;
                    }
                    // Check if every declaration of `indexOf` method has `includes` method
                    // and the two methods have the same parameters.
                    for (const instanceofMethodDecl of indexofMethodSymbol.declarations) {
                        const typeDecl = instanceofMethodDecl.parent;
                        const type = types.getTypeAtLocation(typeDecl);
                        const includesMethodSymbol = type.getProperty('includes');
                        if (includesMethodSymbol == null ||
                            !includesMethodSymbol.declarations.some(includesMethodDecl => hasSameParameters(includesMethodDecl, instanceofMethodDecl))) {
                            return;
                        }
                    }
                    // Report it.
                    context.report({
                        node: compareNode,
                        messageId: 'preferIncludes',
                        *fix(fixer) {
                            if (negative) {
                                yield fixer.insertTextBefore(callNode, '!');
                            }
                            yield fixer.replaceText(node.property, 'includes');
                            yield fixer.removeRange([callNode.range[1], compareNode.range[1]]);
                        },
                    });
                },
                // /bar/.test(foo)
                ':matches(CallExpression, OptionalCallExpression) > :matches(MemberExpression, OptionalMemberExpression).callee[property.name="test"][computed=false]'(node) {
                    const callNode = node.parent;
                    const text = callNode.arguments.length === 1 ? parseRegExp(node.object) : null;
                    if (text == null) {
                        return;
                    }
                    context.report({
                        node: callNode,
                        messageId: 'preferStringIncludes',
                        *fix(fixer) {
                            const argNode = callNode.arguments[0];
                            const needsParen = argNode.type !== experimental_utils_1.AST_NODE_TYPES.Literal &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.TemplateLiteral &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.Identifier &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.MemberExpression &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.CallExpression &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression;
                            yield fixer.removeRange([callNode.range[0], argNode.range[0]]);
                            if (needsParen) {
                                yield fixer.insertTextBefore(argNode, '(');
                                yield fixer.insertTextAfter(argNode, ')');
                            }
                            yield fixer.insertTextAfter(argNode, `${callNode.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression
                                ? '?.'
                                : '.'}includes(${JSON.stringify(text)}`);
                        },
                    });
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/prefer-namespace-keyword.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'prefer-namespace-keyword',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Require the use of the `namespace` keyword instead of the `module` keyword to declare custom TypeScript modules',
                category: 'Best Practices',
                recommended: 'error',
            },
            fixable: 'code',
            messages: {
                useNamespace: "Use 'namespace' instead of 'module' to declare custom TypeScript modules.",
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                TSModuleDeclaration(node) {
                    // Do nothing if the name is a string.
                    if (!node.id || node.id.type === experimental_utils_1.AST_NODE_TYPES.Literal) {
                        return;
                    }
                    // Get tokens of the declaration header.
                    const moduleType = sourceCode.getTokenBefore(node.id);
                    if (moduleType &&
                        moduleType.type === experimental_utils_1.AST_TOKEN_TYPES.Identifier &&
                        moduleType.value === 'module') {
                        context.report({
                            node,
                            messageId: 'useNamespace',
                            fix(fixer) {
                                return fixer.replaceText(moduleType, 'namespace');
                            },
                        });
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/prefer-nullish-coalescing.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'prefer-nullish-coalescing',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Enforce the usage of the nullish coalescing operator instead of logical chaining',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            fixable: 'code',
            messages: {
                preferNullish: 'Prefer using nullish coalescing operator (`??`) instead of a logical or (`||`), as it is a safer operator.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        ignoreConditionalTests: {
                            type: 'boolean',
                        },
                        ignoreMixedLogicalExpressions: {
                            type: 'boolean',
                        },
                        forceSuggestionFixer: {
                            type: 'boolean',
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            {
                ignoreConditionalTests: true,
                ignoreMixedLogicalExpressions: true,
                forceSuggestionFixer: false,
            },
        ],
        create(context, [{ ignoreConditionalTests, ignoreMixedLogicalExpressions, forceSuggestionFixer, },]) {
            const parserServices = util.getParserServices(context);
            const sourceCode = context.getSourceCode();
            const checker = parserServices.program.getTypeChecker();
            return {
                'LogicalExpression[operator = "||"]'(node) {
                    const tsNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                    const type = checker.getTypeAtLocation(tsNode.left);
                    const isNullish = util.isNullableType(type, { allowUndefined: true });
                    if (!isNullish) {
                        return;
                    }
                    if (ignoreConditionalTests === true && isConditionalTest(node)) {
                        return;
                    }
                    const isMixedLogical = isMixedLogicalExpression(node);
                    if (ignoreMixedLogicalExpressions === true && isMixedLogical) {
                        return;
                    }
                    const barBarOperator = util.nullThrows(sourceCode.getTokenAfter(node.left, token => token.type === experimental_utils_1.AST_TOKEN_TYPES.Punctuator &&
                        token.value === node.operator), util.NullThrowsReasons.MissingToken('operator', node.type));
                    function* fix(fixer) {
                        if (node.parent && util.isLogicalOrOperator(node.parent)) {
                            // '&&' and '??' operations cannot be mixed without parentheses (e.g. a && b ?? c)
                            if (node.left.type === experimental_utils_1.AST_NODE_TYPES.LogicalExpression &&
                                !util.isLogicalOrOperator(node.left.left)) {
                                yield fixer.insertTextBefore(node.left.right, '(');
                            }
                            else {
                                yield fixer.insertTextBefore(node.left, '(');
                            }
                            yield fixer.insertTextAfter(node.right, ')');
                        }
                        yield fixer.replaceText(barBarOperator, '??');
                    }
                    const fixer = isMixedLogical || forceSuggestionFixer
                        ? // suggestion instead for cases where we aren't sure if the fixer is completely safe
                            {
                                suggest: [
                                    {
                                        messageId: 'preferNullish',
                                        fix,
                                    },
                                ],
                            }
                        : { fix };
                    context.report(Object.assign({ node: barBarOperator, messageId: 'preferNullish' }, fixer));
                },
            };
        },
    });
    function isConditionalTest(node) {
        const parents = new Set([node]);
        let current = node.parent;
        while (current) {
            parents.add(current);
            if ((current.type === experimental_utils_1.AST_NODE_TYPES.ConditionalExpression ||
                current.type === experimental_utils_1.AST_NODE_TYPES.DoWhileStatement ||
                current.type === experimental_utils_1.AST_NODE_TYPES.IfStatement ||
                current.type === experimental_utils_1.AST_NODE_TYPES.ForStatement ||
                current.type === experimental_utils_1.AST_NODE_TYPES.WhileStatement) &&
                parents.has(current.test)) {
                return true;
            }
            if ([
                experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression,
                experimental_utils_1.AST_NODE_TYPES.FunctionExpression,
            ].includes(current.type)) {
                /**
                 * This is a weird situation like:
                 * `if (() => a || b) {}`
                 * `if (function () { return a || b }) {}`
                 */
                return false;
            }
            current = current.parent;
        }
        return false;
    }
    function isMixedLogicalExpression(node) {
        const seen = new Set();
        const queue = [node.parent, node.left, node.right];
        for (const current of queue) {
            if (seen.has(current)) {
                continue;
            }
            seen.add(current);
            if (current && current.type === experimental_utils_1.AST_NODE_TYPES.LogicalExpression) {
                if (current.operator === '&&') {
                    return true;
                }
                else if (current.operator === '||') {
                    // check the pieces of the node to catch cases like `a || b || c && d`
                    queue.push(current.parent, current.left, current.right);
                }
            }
        }
        return false;
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/prefer-optional-chain.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const eslint_utils_1 = require(250) /* eslint-utils */;
    const util = __importStar(require(622) /* ../util */);
    /*
    The AST is always constructed such the first element is always the deepest element.
    
    I.e. for this code: `foo && foo.bar && foo.bar.baz && foo.bar.baz.buzz`
    The AST will look like this:
    {
      left: {
        left: {
          left: foo
          right: foo.bar
        }
        right: foo.bar.baz
      }
      right: foo.bar.baz.buzz
    }
    */
    exports.default = util.createRule({
        name: 'prefer-optional-chain',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Prefer using concise optional chain expressions instead of chained logical ands',
                category: 'Best Practices',
                recommended: false,
            },
            fixable: 'code',
            messages: {
                preferOptionalChain: "Prefer using an optional chain expression instead, as it's more concise and easier to read.",
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            const sourceCode = context.getSourceCode();
            return {
                [[
                    'LogicalExpression[operator="&&"] > Identifier',
                    'LogicalExpression[operator="&&"] > BinaryExpression[operator="!=="]',
                    'LogicalExpression[operator="&&"] > BinaryExpression[operator="!="]',
                ].join(',')](initialIdentifierOrNotEqualsExpr) {
                    // selector guarantees this cast
                    const initialExpression = initialIdentifierOrNotEqualsExpr.parent;
                    if (initialExpression.left !== initialIdentifierOrNotEqualsExpr) {
                        // the identifier is not the deepest left node
                        return;
                    }
                    if (!isValidChainTarget(initialIdentifierOrNotEqualsExpr, true)) {
                        return;
                    }
                    // walk up the tree to figure out how many logical expressions we can include
                    let previous = initialExpression;
                    let current = initialExpression;
                    let previousLeftText = getText(initialIdentifierOrNotEqualsExpr);
                    let optionallyChainedCode = previousLeftText;
                    let expressionCount = 1;
                    while (current.type === experimental_utils_1.AST_NODE_TYPES.LogicalExpression) {
                        if (!isValidChainTarget(current.right, 
                        // only allow identifiers for the first chain - foo && foo()
                        expressionCount === 1)) {
                            break;
                        }
                        const leftText = previousLeftText;
                        const rightText = getText(current.right);
                        // can't just use startsWith because of cases like foo && fooBar.baz;
                        const matchRegex = new RegExp(`^${
                        // escape regex characters
                        leftText.replace(/[.*+?^${}()|[\]\\]/g, '\\$&')}[^a-zA-Z0-9_$]`);
                        if (!matchRegex.test(rightText) &&
                            // handle redundant cases like foo.bar && foo.bar
                            leftText !== rightText) {
                            break;
                        }
                        // omit weird doubled up expression that make no sense like foo.bar && foo.bar
                        if (rightText !== leftText) {
                            expressionCount += 1;
                            previousLeftText = rightText;
                            /*
                            Diff the left and right text to construct the fix string
                            There are the following cases:
                
                            1)
                            rightText === 'foo.bar.baz.buzz'
                            leftText === 'foo.bar.baz'
                            diff === '.buzz'
                
                            2)
                            rightText === 'foo.bar.baz.buzz()'
                            leftText === 'foo.bar.baz'
                            diff === '.buzz()'
                
                            3)
                            rightText === 'foo.bar.baz.buzz()'
                            leftText === 'foo.bar.baz.buzz'
                            diff === '()'
                
                            4)
                            rightText === 'foo.bar.baz[buzz]'
                            leftText === 'foo.bar.baz'
                            diff === '[buzz]'
                
                            5)
                            rightText === 'foo.bar.baz?.buzz'
                            leftText === 'foo.bar.baz'
                            diff === '?.buzz'
                            */
                            const diff = rightText.replace(leftText, '');
                            if (diff.startsWith('?')) {
                                // item was "pre optional chained"
                                optionallyChainedCode += diff;
                            }
                            else {
                                const needsDot = diff.startsWith('(') || diff.startsWith('[');
                                optionallyChainedCode += `?${needsDot ? '.' : ''}${diff}`;
                            }
                        }
                        previous = current;
                        current = util.nullThrows(current.parent, util.NullThrowsReasons.MissingParent);
                    }
                    if (expressionCount > 1) {
                        if (previous.right.type === experimental_utils_1.AST_NODE_TYPES.BinaryExpression) {
                            // case like foo && foo.bar !== someValue
                            optionallyChainedCode += ` ${previous.right.operator} ${sourceCode.getText(previous.right.right)}`;
                        }
                        context.report({
                            node: previous,
                            messageId: 'preferOptionalChain',
                            fix(fixer) {
                                return fixer.replaceText(previous, optionallyChainedCode);
                            },
                        });
                    }
                },
            };
            function getText(node) {
                if (node.type === experimental_utils_1.AST_NODE_TYPES.BinaryExpression) {
                    return getText(
                    // isValidChainTarget ensures this is type safe
                    node.left);
                }
                if (node.type === experimental_utils_1.AST_NODE_TYPES.CallExpression ||
                    node.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression) {
                    const calleeText = getText(
                    // isValidChainTarget ensures this is type safe
                    node.callee);
                    // ensure that the call arguments are left untouched, or else we can break cases that _need_ whitespace:
                    // - JSX: <Foo Needs Space Between Attrs />
                    // - Unary Operators: typeof foo, await bar, delete baz
                    const closingParenToken = util.nullThrows(sourceCode.getLastToken(node), util.NullThrowsReasons.MissingToken('closing parenthesis', node.type));
                    const openingParenToken = util.nullThrows(sourceCode.getFirstTokenBetween(node.callee, closingParenToken, eslint_utils_1.isOpeningParenToken), util.NullThrowsReasons.MissingToken('opening parenthesis', node.type));
                    const argumentsText = sourceCode.text.substring(openingParenToken.range[0], closingParenToken.range[1]);
                    return `${calleeText}${argumentsText}`;
                }
                if (node.type === experimental_utils_1.AST_NODE_TYPES.Identifier) {
                    return node.name;
                }
                if (node.type === experimental_utils_1.AST_NODE_TYPES.ThisExpression) {
                    return 'this';
                }
                return getMemberExpressionText(node);
            }
            /**
             * Gets a normalized representation of the given MemberExpression
             */
            function getMemberExpressionText(node) {
                let objectText;
                // cases should match the list in ALLOWED_MEMBER_OBJECT_TYPES
                switch (node.object.type) {
                    case experimental_utils_1.AST_NODE_TYPES.CallExpression:
                    case experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression:
                    case experimental_utils_1.AST_NODE_TYPES.Identifier:
                        objectText = getText(node.object);
                        break;
                    case experimental_utils_1.AST_NODE_TYPES.MemberExpression:
                    case experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression:
                        objectText = getMemberExpressionText(node.object);
                        break;
                    case experimental_utils_1.AST_NODE_TYPES.ThisExpression:
                        objectText = getText(node.object);
                        break;
                    /* istanbul ignore next */
                    default:
                        throw new Error(`Unexpected member object type: ${node.object.type}`);
                }
                let propertyText;
                if (node.computed) {
                    // cases should match the list in ALLOWED_COMPUTED_PROP_TYPES
                    switch (node.property.type) {
                        case experimental_utils_1.AST_NODE_TYPES.Identifier:
                            propertyText = getText(node.property);
                            break;
                        case experimental_utils_1.AST_NODE_TYPES.Literal:
                        case experimental_utils_1.AST_NODE_TYPES.BigIntLiteral:
                        case experimental_utils_1.AST_NODE_TYPES.TemplateLiteral:
                            propertyText = sourceCode.getText(node.property);
                            break;
                        case experimental_utils_1.AST_NODE_TYPES.MemberExpression:
                        case experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression:
                            propertyText = getMemberExpressionText(node.property);
                            break;
                        /* istanbul ignore next */
                        default:
                            throw new Error(`Unexpected member property type: ${node.object.type}`);
                    }
                    return `${objectText}${node.optional ? '?.' : ''}[${propertyText}]`;
                }
                else {
                    // cases should match the list in ALLOWED_NON_COMPUTED_PROP_TYPES
                    switch (node.property.type) {
                        case experimental_utils_1.AST_NODE_TYPES.Identifier:
                            propertyText = getText(node.property);
                            break;
                        /* istanbul ignore next */
                        default:
                            throw new Error(`Unexpected member property type: ${node.object.type}`);
                    }
                    return `${objectText}${node.optional ? '?.' : '.'}${propertyText}`;
                }
            }
        },
    });
    const ALLOWED_MEMBER_OBJECT_TYPES = new Set([
        experimental_utils_1.AST_NODE_TYPES.CallExpression,
        experimental_utils_1.AST_NODE_TYPES.Identifier,
        experimental_utils_1.AST_NODE_TYPES.MemberExpression,
        experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression,
        experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression,
        experimental_utils_1.AST_NODE_TYPES.ThisExpression,
    ]);
    const ALLOWED_COMPUTED_PROP_TYPES = new Set([
        experimental_utils_1.AST_NODE_TYPES.BigIntLiteral,
        experimental_utils_1.AST_NODE_TYPES.Identifier,
        experimental_utils_1.AST_NODE_TYPES.Literal,
        experimental_utils_1.AST_NODE_TYPES.MemberExpression,
        experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression,
        experimental_utils_1.AST_NODE_TYPES.TemplateLiteral,
    ]);
    const ALLOWED_NON_COMPUTED_PROP_TYPES = new Set([
        experimental_utils_1.AST_NODE_TYPES.Identifier,
    ]);
    function isValidChainTarget(node, allowIdentifier) {
        if (node.type === experimental_utils_1.AST_NODE_TYPES.MemberExpression ||
            node.type === experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression) {
            const isObjectValid = ALLOWED_MEMBER_OBJECT_TYPES.has(node.object.type) &&
                // make sure to validate the expression is of our expected structure
                isValidChainTarget(node.object, true);
            const isPropertyValid = node.computed
                ? ALLOWED_COMPUTED_PROP_TYPES.has(node.property.type) &&
                    // make sure to validate the member expression is of our expected structure
                    (node.property.type === experimental_utils_1.AST_NODE_TYPES.MemberExpression ||
                        node.property.type === experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression
                        ? isValidChainTarget(node.property, allowIdentifier)
                        : true)
                : ALLOWED_NON_COMPUTED_PROP_TYPES.has(node.property.type);
            return isObjectValid && isPropertyValid;
        }
        if (node.type === experimental_utils_1.AST_NODE_TYPES.CallExpression ||
            node.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression) {
            return isValidChainTarget(node.callee, allowIdentifier);
        }
        if (allowIdentifier &&
            (node.type === experimental_utils_1.AST_NODE_TYPES.Identifier ||
                node.type === experimental_utils_1.AST_NODE_TYPES.ThisExpression)) {
            return true;
        }
        /*
        special case for the following, where we only want the left
        - foo !== null
        - foo != null
        - foo !== undefined
        - foo != undefined
        */
        if (node.type === experimental_utils_1.AST_NODE_TYPES.BinaryExpression &&
            ['!==', '!='].includes(node.operator) &&
            isValidChainTarget(node.left, allowIdentifier)) {
            if (node.right.type === experimental_utils_1.AST_NODE_TYPES.Identifier &&
                node.right.name === 'undefined') {
                return true;
            }
            if (node.right.type === experimental_utils_1.AST_NODE_TYPES.Literal &&
                node.right.value === null) {
                return true;
            }
        }
        return false;
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/prefer-readonly.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const tsutils = __importStar(require(627) /* tsutils */);
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    const util_1 = require(622) /* ../util */;
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const functionScopeBoundaries = [
        experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression,
        experimental_utils_1.AST_NODE_TYPES.FunctionDeclaration,
        experimental_utils_1.AST_NODE_TYPES.FunctionExpression,
        experimental_utils_1.AST_NODE_TYPES.MethodDefinition,
    ].join(', ');
    exports.default = util.createRule({
        name: 'prefer-readonly',
        meta: {
            docs: {
                description: "Requires that private members are marked as `readonly` if they're never modified outside of the constructor",
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            fixable: 'code',
            messages: {
                preferReadonly: "Member '{{name}}' is never reassigned; mark it as `readonly`.",
            },
            schema: [
                {
                    allowAdditionalProperties: false,
                    properties: {
                        onlyInlineLambdas: {
                            type: 'boolean',
                        },
                    },
                    type: 'object',
                },
            ],
            type: 'suggestion',
        },
        defaultOptions: [{ onlyInlineLambdas: false }],
        create(context, [{ onlyInlineLambdas }]) {
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            const classScopeStack = [];
            function handlePropertyAccessExpression(node, parent, classScope) {
                if (ts.isBinaryExpression(parent)) {
                    handleParentBinaryExpression(node, parent, classScope);
                    return;
                }
                if (ts.isDeleteExpression(parent) || isDestructuringAssignment(node)) {
                    classScope.addVariableModification(node);
                    return;
                }
                if (ts.isPostfixUnaryExpression(parent) ||
                    ts.isPrefixUnaryExpression(parent)) {
                    handleParentPostfixOrPrefixUnaryExpression(parent, classScope);
                }
            }
            function handleParentBinaryExpression(node, parent, classScope) {
                if (parent.left === node &&
                    tsutils.isAssignmentKind(parent.operatorToken.kind)) {
                    classScope.addVariableModification(node);
                }
            }
            function handleParentPostfixOrPrefixUnaryExpression(node, classScope) {
                if (node.operator === ts.SyntaxKind.PlusPlusToken ||
                    node.operator === ts.SyntaxKind.MinusMinusToken) {
                    classScope.addVariableModification(node.operand);
                }
            }
            function isDestructuringAssignment(node) {
                let current = node.parent;
                while (current) {
                    const parent = current.parent;
                    if (ts.isObjectLiteralExpression(parent) ||
                        ts.isArrayLiteralExpression(parent) ||
                        ts.isSpreadAssignment(parent) ||
                        (ts.isSpreadElement(parent) &&
                            ts.isArrayLiteralExpression(parent.parent))) {
                        current = parent;
                    }
                    else if (ts.isBinaryExpression(parent)) {
                        return (parent.left === current &&
                            parent.operatorToken.kind === ts.SyntaxKind.EqualsToken);
                    }
                    else {
                        break;
                    }
                }
                return false;
            }
            function isConstructor(node) {
                return (node.type === experimental_utils_1.AST_NODE_TYPES.MethodDefinition &&
                    node.kind === 'constructor');
            }
            function isFunctionScopeBoundaryInStack(node) {
                if (classScopeStack.length === 0) {
                    return false;
                }
                const tsNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                if (ts.isConstructorDeclaration(tsNode)) {
                    return false;
                }
                return tsutils.isFunctionScopeBoundary(tsNode);
            }
            function getEsNodesFromViolatingNode(violatingNode) {
                if (ts.isParameterPropertyDeclaration(violatingNode, violatingNode.parent)) {
                    return {
                        esNode: parserServices.tsNodeToESTreeNodeMap.get(violatingNode.name),
                        nameNode: parserServices.tsNodeToESTreeNodeMap.get(violatingNode.name),
                    };
                }
                return {
                    esNode: parserServices.tsNodeToESTreeNodeMap.get(violatingNode),
                    nameNode: parserServices.tsNodeToESTreeNodeMap.get(violatingNode.name),
                };
            }
            return {
                'ClassDeclaration, ClassExpression'(node) {
                    classScopeStack.push(new ClassScope(checker, parserServices.esTreeNodeToTSNodeMap.get(node), onlyInlineLambdas));
                },
                'ClassDeclaration, ClassExpression:exit'() {
                    const finalizedClassScope = classScopeStack.pop();
                    const sourceCode = context.getSourceCode();
                    for (const violatingNode of finalizedClassScope.finalizeUnmodifiedPrivateNonReadonlys()) {
                        const { esNode, nameNode } = getEsNodesFromViolatingNode(violatingNode);
                        context.report({
                            data: {
                                name: sourceCode.getText(nameNode),
                            },
                            fix: fixer => fixer.insertTextBefore(nameNode, 'readonly '),
                            messageId: 'preferReadonly',
                            node: esNode,
                        });
                    }
                },
                MemberExpression(node) {
                    if (classScopeStack.length !== 0 && !node.computed) {
                        const tsNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                        handlePropertyAccessExpression(tsNode, tsNode.parent, classScopeStack[classScopeStack.length - 1]);
                    }
                },
                [functionScopeBoundaries](node) {
                    if (isConstructor(node)) {
                        classScopeStack[classScopeStack.length - 1].enterConstructor(parserServices.esTreeNodeToTSNodeMap.get(node));
                    }
                    else if (isFunctionScopeBoundaryInStack(node)) {
                        classScopeStack[classScopeStack.length - 1].enterNonConstructor();
                    }
                },
                [`${functionScopeBoundaries}:exit`](node) {
                    if (isConstructor(node)) {
                        classScopeStack[classScopeStack.length - 1].exitConstructor();
                    }
                    else if (isFunctionScopeBoundaryInStack(node)) {
                        classScopeStack[classScopeStack.length - 1].exitNonConstructor();
                    }
                },
            };
        },
    });
    const OUTSIDE_CONSTRUCTOR = -1;
    const DIRECTLY_INSIDE_CONSTRUCTOR = 0;
    class ClassScope {
        constructor(checker, classNode, onlyInlineLambdas) {
            this.checker = checker;
            this.onlyInlineLambdas = onlyInlineLambdas;
            this.privateModifiableMembers = new Map();
            this.privateModifiableStatics = new Map();
            this.memberVariableModifications = new Set();
            this.staticVariableModifications = new Set();
            this.constructorScopeDepth = OUTSIDE_CONSTRUCTOR;
            this.checker = checker;
            this.classType = checker.getTypeAtLocation(classNode);
            for (const member of classNode.members) {
                if (ts.isPropertyDeclaration(member)) {
                    this.addDeclaredVariable(member);
                }
            }
        }
        addDeclaredVariable(node) {
            if (!tsutils.isModifierFlagSet(node, ts.ModifierFlags.Private) ||
                tsutils.isModifierFlagSet(node, ts.ModifierFlags.Readonly) ||
                ts.isComputedPropertyName(node.name)) {
                return;
            }
            if (this.onlyInlineLambdas &&
                node.initializer !== undefined &&
                !ts.isArrowFunction(node.initializer)) {
                return;
            }
            (tsutils.isModifierFlagSet(node, ts.ModifierFlags.Static)
                ? this.privateModifiableStatics
                : this.privateModifiableMembers).set(node.name.getText(), node);
        }
        addVariableModification(node) {
            const modifierType = this.checker.getTypeAtLocation(node.expression);
            if (modifierType.symbol === undefined ||
                !util_1.typeIsOrHasBaseType(modifierType, this.classType)) {
                return;
            }
            const modifyingStatic = tsutils.isObjectType(modifierType) &&
                tsutils.isObjectFlagSet(modifierType, ts.ObjectFlags.Anonymous);
            if (!modifyingStatic &&
                this.constructorScopeDepth === DIRECTLY_INSIDE_CONSTRUCTOR) {
                return;
            }
            (modifyingStatic
                ? this.staticVariableModifications
                : this.memberVariableModifications).add(node.name.text);
        }
        enterConstructor(node) {
            this.constructorScopeDepth = DIRECTLY_INSIDE_CONSTRUCTOR;
            for (const parameter of node.parameters) {
                if (tsutils.isModifierFlagSet(parameter, ts.ModifierFlags.Private)) {
                    this.addDeclaredVariable(parameter);
                }
            }
        }
        exitConstructor() {
            this.constructorScopeDepth = OUTSIDE_CONSTRUCTOR;
        }
        enterNonConstructor() {
            if (this.constructorScopeDepth !== OUTSIDE_CONSTRUCTOR) {
                this.constructorScopeDepth += 1;
            }
        }
        exitNonConstructor() {
            if (this.constructorScopeDepth !== OUTSIDE_CONSTRUCTOR) {
                this.constructorScopeDepth -= 1;
            }
        }
        finalizeUnmodifiedPrivateNonReadonlys() {
            this.memberVariableModifications.forEach(variableName => {
                this.privateModifiableMembers.delete(variableName);
            });
            this.staticVariableModifications.forEach(variableName => {
                this.privateModifiableStatics.delete(variableName);
            });
            return [
                ...Array.from(this.privateModifiableMembers.values()),
                ...Array.from(this.privateModifiableStatics.values()),
            ];
        }
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/prefer-readonly-parameter-types.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'prefer-readonly-parameter-types',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Requires that function parameters are typed as readonly to prevent accidental mutation of inputs',
                category: 'Possible Errors',
                recommended: false,
                requiresTypeChecking: true,
            },
            schema: [
                {
                    type: 'object',
                    additionalProperties: false,
                    properties: {
                        checkParameterProperties: {
                            type: 'boolean',
                        },
                    },
                },
            ],
            messages: {
                shouldBeReadonly: 'Parameter should be a read only type',
            },
        },
        defaultOptions: [
            {
                checkParameterProperties: true,
            },
        ],
        create(context, [{ checkParameterProperties }]) {
            const { esTreeNodeToTSNodeMap, program } = util.getParserServices(context);
            const checker = program.getTypeChecker();
            return {
                [[
                    experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression,
                    experimental_utils_1.AST_NODE_TYPES.FunctionDeclaration,
                    experimental_utils_1.AST_NODE_TYPES.FunctionExpression,
                    experimental_utils_1.AST_NODE_TYPES.TSCallSignatureDeclaration,
                    experimental_utils_1.AST_NODE_TYPES.TSConstructSignatureDeclaration,
                    experimental_utils_1.AST_NODE_TYPES.TSDeclareFunction,
                    experimental_utils_1.AST_NODE_TYPES.TSEmptyBodyFunctionExpression,
                    experimental_utils_1.AST_NODE_TYPES.TSFunctionType,
                    experimental_utils_1.AST_NODE_TYPES.TSMethodSignature,
                ].join(', ')](node) {
                    for (const param of node.params) {
                        if (!checkParameterProperties &&
                            param.type === experimental_utils_1.AST_NODE_TYPES.TSParameterProperty) {
                            continue;
                        }
                        const actualParam = param.type === experimental_utils_1.AST_NODE_TYPES.TSParameterProperty
                            ? param.parameter
                            : param;
                        const tsNode = esTreeNodeToTSNodeMap.get(actualParam);
                        const type = checker.getTypeAtLocation(tsNode);
                        const isReadOnly = util.isTypeReadonly(checker, type);
                        if (!isReadOnly) {
                            context.report({
                                node: actualParam,
                                messageId: 'shouldBeReadonly',
                            });
                        }
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/prefer-regexp-exec.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const eslint_utils_1 = require(250) /* eslint-utils */;
    const util_1 = require(622) /* ../util */;
    exports.default = util_1.createRule({
        name: 'prefer-regexp-exec',
        defaultOptions: [],
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Enforce that `RegExp#exec` is used instead of `String#match` if no global flag is provided',
                category: 'Best Practices',
                recommended: 'error',
                requiresTypeChecking: true,
            },
            messages: {
                regExpExecOverStringMatch: 'Use the `RegExp#exec()` method instead.',
            },
            schema: [],
        },
        create(context) {
            const globalScope = context.getScope();
            const service = util_1.getParserServices(context);
            const typeChecker = service.program.getTypeChecker();
            /**
             * Check if a given node is a string.
             * @param node The node to check.
             */
            function isStringType(node) {
                const objectType = typeChecker.getTypeAtLocation(service.esTreeNodeToTSNodeMap.get(node));
                return util_1.getTypeName(typeChecker, objectType) === 'string';
            }
            return {
                "CallExpression[arguments.length=1] > MemberExpression.callee[property.name='match'][computed=false]"(node) {
                    const callNode = node.parent;
                    const arg = callNode.arguments[0];
                    const evaluated = eslint_utils_1.getStaticValue(arg, globalScope);
                    // Don't report regular expressions with global flag.
                    if (evaluated &&
                        evaluated.value instanceof RegExp &&
                        evaluated.value.flags.includes('g')) {
                        return;
                    }
                    if (isStringType(node.object)) {
                        context.report({
                            node: callNode,
                            messageId: 'regExpExecOverStringMatch',
                        });
                        return;
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/prefer-string-starts-ends-with.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const eslint_utils_1 = require(250) /* eslint-utils */;
    const regexpp_1 = require(226) /* regexpp */;
    const util_1 = require(622) /* ../util */;
    const EQ_OPERATORS = /^[=!]=/;
    const regexpp = new regexpp_1.RegExpParser();
    exports.default = util_1.createRule({
        name: 'prefer-string-starts-ends-with',
        defaultOptions: [],
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Enforce the use of `String#startsWith` and `String#endsWith` instead of other equivalent methods of checking substrings',
                category: 'Best Practices',
                recommended: 'error',
                requiresTypeChecking: true,
            },
            messages: {
                preferStartsWith: "Use 'String#startsWith' method instead.",
                preferEndsWith: "Use the 'String#endsWith' method instead.",
            },
            schema: [],
            fixable: 'code',
        },
        create(context) {
            const globalScope = context.getScope();
            const sourceCode = context.getSourceCode();
            const service = util_1.getParserServices(context);
            const typeChecker = service.program.getTypeChecker();
            /**
             * Check if a given node is a string.
             * @param node The node to check.
             */
            function isStringType(node) {
                const objectType = typeChecker.getTypeAtLocation(service.esTreeNodeToTSNodeMap.get(node));
                return util_1.getTypeName(typeChecker, objectType) === 'string';
            }
            /**
             * Check if a given node is a `Literal` node that is null.
             * @param node The node to check.
             */
            function isNull(node) {
                const evaluated = eslint_utils_1.getStaticValue(node, globalScope);
                return evaluated != null && evaluated.value === null;
            }
            /**
             * Check if a given node is a `Literal` node that is a given value.
             * @param node The node to check.
             * @param value The expected value of the `Literal` node.
             */
            function isNumber(node, value) {
                const evaluated = eslint_utils_1.getStaticValue(node, globalScope);
                return evaluated != null && evaluated.value === value;
            }
            /**
             * Check if a given node is a `Literal` node that is a character.
             * @param node The node to check.
             * @param kind The method name to get a character.
             */
            function isCharacter(node) {
                const evaluated = eslint_utils_1.getStaticValue(node, globalScope);
                return (evaluated != null &&
                    typeof evaluated.value === 'string' &&
                    // checks if the string is a character long
                    // eslint-disable-next-line @typescript-eslint/prefer-string-starts-ends-with
                    evaluated.value[0] === evaluated.value);
            }
            /**
             * Check if a given node is `==`, `===`, `!=`, or `!==`.
             * @param node The node to check.
             */
            function isEqualityComparison(node) {
                return (node.type === experimental_utils_1.AST_NODE_TYPES.BinaryExpression &&
                    EQ_OPERATORS.test(node.operator));
            }
            /**
             * Check if two given nodes are the same meaning.
             * @param node1 A node to compare.
             * @param node2 Another node to compare.
             */
            function isSameTokens(node1, node2) {
                const tokens1 = sourceCode.getTokens(node1);
                const tokens2 = sourceCode.getTokens(node2);
                if (tokens1.length !== tokens2.length) {
                    return false;
                }
                for (let i = 0; i < tokens1.length; ++i) {
                    const token1 = tokens1[i];
                    const token2 = tokens2[i];
                    if (token1.type !== token2.type || token1.value !== token2.value) {
                        return false;
                    }
                }
                return true;
            }
            /**
             * Check if a given node is the expression of the length of a string.
             *
             * - If `length` property access of `expectedObjectNode`, it's `true`.
             *   E.g., `foo` → `foo.length` / `"foo"` → `"foo".length`
             * - If `expectedObjectNode` is a string literal, `node` can be a number.
             *   E.g., `"foo"` → `3`
             *
             * @param node The node to check.
             * @param expectedObjectNode The node which is expected as the receiver of `length` property.
             */
            function isLengthExpression(node, expectedObjectNode) {
                if (node.type === experimental_utils_1.AST_NODE_TYPES.MemberExpression ||
                    node.type === experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression) {
                    return (eslint_utils_1.getPropertyName(node, globalScope) === 'length' &&
                        isSameTokens(node.object, expectedObjectNode));
                }
                const evaluatedLength = eslint_utils_1.getStaticValue(node, globalScope);
                const evaluatedString = eslint_utils_1.getStaticValue(expectedObjectNode, globalScope);
                return (evaluatedLength != null &&
                    evaluatedString != null &&
                    typeof evaluatedLength.value === 'number' &&
                    typeof evaluatedString.value === 'string' &&
                    evaluatedLength.value === evaluatedString.value.length);
            }
            /**
             * Check if a given node is the expression of the last index.
             *
             * E.g. `foo.length - 1`
             *
             * @param node The node to check.
             * @param expectedObjectNode The node which is expected as the receiver of `length` property.
             */
            function isLastIndexExpression(node, expectedObjectNode) {
                return (node.type === experimental_utils_1.AST_NODE_TYPES.BinaryExpression &&
                    node.operator === '-' &&
                    isLengthExpression(node.left, expectedObjectNode) &&
                    isNumber(node.right, 1));
            }
            /**
             * Get the range of the property of a given `MemberExpression` node.
             *
             * - `obj[foo]` → the range of `[foo]`
             * - `obf.foo` → the range of `.foo`
             * - `(obj).foo` → the range of `.foo`
             *
             * @param node The member expression node to get.
             */
            function getPropertyRange(node) {
                const dotOrOpenBracket = sourceCode.getTokenAfter(node.object, eslint_utils_1.isNotClosingParenToken);
                return [dotOrOpenBracket.range[0], node.range[1]];
            }
            /**
             * Parse a given `RegExp` pattern to that string if it's a static string.
             * @param pattern The RegExp pattern text to parse.
             * @param uFlag The Unicode flag of the RegExp.
             */
            function parseRegExpText(pattern, uFlag) {
                // Parse it.
                const ast = regexpp.parsePattern(pattern, undefined, undefined, uFlag);
                if (ast.alternatives.length !== 1) {
                    return null;
                }
                // Drop `^`/`$` assertion.
                const chars = ast.alternatives[0].elements;
                const first = chars[0];
                if (first.type === 'Assertion' && first.kind === 'start') {
                    chars.shift();
                }
                else {
                    chars.pop();
                }
                // Check if it can determine a unique string.
                if (!chars.every(c => c.type === 'Character')) {
                    return null;
                }
                // To string.
                return String.fromCodePoint(...chars.map(c => c.value));
            }
            /**
             * Parse a given node if it's a `RegExp` instance.
             * @param node The node to parse.
             */
            function parseRegExp(node) {
                const evaluated = eslint_utils_1.getStaticValue(node, globalScope);
                if (evaluated == null || !(evaluated.value instanceof RegExp)) {
                    return null;
                }
                const { source, flags } = evaluated.value;
                const isStartsWith = source.startsWith('^');
                const isEndsWith = source.endsWith('$');
                if (isStartsWith === isEndsWith ||
                    flags.includes('i') ||
                    flags.includes('m')) {
                    return null;
                }
                const text = parseRegExpText(source, flags.includes('u'));
                if (text == null) {
                    return null;
                }
                return { isEndsWith, isStartsWith, text };
            }
            /**
             * Fix code with using the right operand as the search string.
             * For example: `foo.slice(0, 3) === 'bar'` → `foo.startsWith('bar')`
             * @param fixer The rule fixer.
             * @param node The node which was reported.
             * @param kind The kind of the report.
             * @param isNegative The flag to fix to negative condition.
             */
            function* fixWithRightOperand(fixer, node, kind, isNegative, isOptional) {
                // left is CallExpression or MemberExpression.
                const leftNode = (node.left.type === experimental_utils_1.AST_NODE_TYPES.CallExpression ||
                    node.left.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression
                    ? node.left.callee
                    : node.left);
                const propertyRange = getPropertyRange(leftNode);
                if (isNegative) {
                    yield fixer.insertTextBefore(node, '!');
                }
                yield fixer.replaceTextRange([propertyRange[0], node.right.range[0]], `${isOptional ? '?.' : '.'}${kind}sWith(`);
                yield fixer.replaceTextRange([node.right.range[1], node.range[1]], ')');
            }
            /**
             * Fix code with using the first argument as the search string.
             * For example: `foo.indexOf('bar') === 0` → `foo.startsWith('bar')`
             * @param fixer The rule fixer.
             * @param node The node which was reported.
             * @param kind The kind of the report.
             * @param negative The flag to fix to negative condition.
             */
            function* fixWithArgument(fixer, node, kind, negative, isOptional) {
                const callNode = node.left;
                const calleeNode = callNode.callee;
                if (negative) {
                    yield fixer.insertTextBefore(node, '!');
                }
                yield fixer.replaceTextRange(getPropertyRange(calleeNode), `${isOptional ? '?.' : '.'}${kind}sWith`);
                yield fixer.removeRange([callNode.range[1], node.range[1]]);
            }
            return {
                // foo[0] === "a"
                // foo.charAt(0) === "a"
                // foo[foo.length - 1] === "a"
                // foo.charAt(foo.length - 1) === "a"
                [[
                    'BinaryExpression > :matches(MemberExpression, OptionalMemberExpression).left[computed=true]',
                    'BinaryExpression > :matches(CallExpression, OptionalCallExpression).left > :matches(MemberExpression, OptionalMemberExpression).callee[property.name="charAt"][computed=false]',
                ].join(', ')](node) {
                    let parentNode = node.parent;
                    let indexNode = null;
                    if (parentNode.type === experimental_utils_1.AST_NODE_TYPES.CallExpression ||
                        parentNode.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression) {
                        if (parentNode.arguments.length === 1) {
                            indexNode = parentNode.arguments[0];
                        }
                        parentNode = parentNode.parent;
                    }
                    else {
                        indexNode = node.property;
                    }
                    if (indexNode == null ||
                        !isEqualityComparison(parentNode) ||
                        !isStringType(node.object)) {
                        return;
                    }
                    const isEndsWith = isLastIndexExpression(indexNode, node.object);
                    const isStartsWith = !isEndsWith && isNumber(indexNode, 0);
                    if (!isStartsWith && !isEndsWith) {
                        return;
                    }
                    const eqNode = parentNode;
                    context.report({
                        node: parentNode,
                        messageId: isStartsWith ? 'preferStartsWith' : 'preferEndsWith',
                        fix(fixer) {
                            // Don't fix if it can change the behavior.
                            if (!isCharacter(eqNode.right)) {
                                return null;
                            }
                            return fixWithRightOperand(fixer, eqNode, isStartsWith ? 'start' : 'end', eqNode.operator.startsWith('!'), node.optional);
                        },
                    });
                },
                // foo.indexOf('bar') === 0
                'BinaryExpression > :matches(CallExpression, OptionalCallExpression).left > :matches(MemberExpression, OptionalMemberExpression).callee[property.name="indexOf"][computed=false]'(node) {
                    const callNode = node.parent;
                    const parentNode = callNode.parent;
                    if (callNode.arguments.length !== 1 ||
                        !isEqualityComparison(parentNode) ||
                        parentNode.left !== callNode ||
                        !isNumber(parentNode.right, 0) ||
                        !isStringType(node.object)) {
                        return;
                    }
                    context.report({
                        node: parentNode,
                        messageId: 'preferStartsWith',
                        fix(fixer) {
                            return fixWithArgument(fixer, parentNode, 'start', parentNode.operator.startsWith('!'), node.optional);
                        },
                    });
                },
                // foo.lastIndexOf('bar') === foo.length - 3
                // foo.lastIndexOf(bar) === foo.length - bar.length
                'BinaryExpression > :matches(CallExpression, OptionalCallExpression).left > :matches(MemberExpression, OptionalMemberExpression).callee[property.name="lastIndexOf"][computed=false]'(node) {
                    const callNode = node.parent;
                    const parentNode = callNode.parent;
                    if (callNode.arguments.length !== 1 ||
                        !isEqualityComparison(parentNode) ||
                        parentNode.left !== callNode ||
                        parentNode.right.type !== experimental_utils_1.AST_NODE_TYPES.BinaryExpression ||
                        parentNode.right.operator !== '-' ||
                        !isLengthExpression(parentNode.right.left, node.object) ||
                        !isLengthExpression(parentNode.right.right, callNode.arguments[0]) ||
                        !isStringType(node.object)) {
                        return;
                    }
                    context.report({
                        node: parentNode,
                        messageId: 'preferEndsWith',
                        fix(fixer) {
                            return fixWithArgument(fixer, parentNode, 'end', parentNode.operator.startsWith('!'), node.optional);
                        },
                    });
                },
                // foo.match(/^bar/) === null
                // foo.match(/bar$/) === null
                'BinaryExpression > :matches(CallExpression, OptionalCallExpression).left > :matches(MemberExpression, OptionalMemberExpression).callee[property.name="match"][computed=false]'(node) {
                    const callNode = node.parent;
                    const parentNode = callNode.parent;
                    if (!isEqualityComparison(parentNode) ||
                        !isNull(parentNode.right) ||
                        !isStringType(node.object)) {
                        return;
                    }
                    const parsed = callNode.arguments.length === 1
                        ? parseRegExp(callNode.arguments[0])
                        : null;
                    if (parsed == null) {
                        return;
                    }
                    const { isStartsWith, text } = parsed;
                    context.report({
                        node: callNode,
                        messageId: isStartsWith ? 'preferStartsWith' : 'preferEndsWith',
                        *fix(fixer) {
                            if (!parentNode.operator.startsWith('!')) {
                                yield fixer.insertTextBefore(parentNode, '!');
                            }
                            yield fixer.replaceTextRange(getPropertyRange(node), `${node.optional ? '?.' : '.'}${isStartsWith ? 'start' : 'end'}sWith`);
                            yield fixer.replaceText(callNode.arguments[0], JSON.stringify(text));
                            yield fixer.removeRange([callNode.range[1], parentNode.range[1]]);
                        },
                    });
                },
                // foo.slice(0, 3) === 'bar'
                // foo.slice(-3) === 'bar'
                // foo.slice(-3, foo.length) === 'bar'
                // foo.substring(0, 3) === 'bar'
                // foo.substring(foo.length - 3) === 'bar'
                // foo.substring(foo.length - 3, foo.length) === 'bar'
                [[
                    ':matches(CallExpression, OptionalCallExpression) > :matches(MemberExpression, OptionalMemberExpression).callee[property.name="slice"][computed=false]',
                    ':matches(CallExpression, OptionalCallExpression) > :matches(MemberExpression, OptionalMemberExpression).callee[property.name="substring"][computed=false]',
                ].join(', ')](node) {
                    const callNode = node.parent;
                    const parentNode = callNode.parent;
                    if (!isEqualityComparison(parentNode) ||
                        parentNode.left !== callNode ||
                        !isStringType(node.object)) {
                        return;
                    }
                    const isEndsWith = callNode.arguments.length === 1 ||
                        (callNode.arguments.length === 2 &&
                            isLengthExpression(callNode.arguments[1], node.object));
                    const isStartsWith = !isEndsWith &&
                        callNode.arguments.length === 2 &&
                        isNumber(callNode.arguments[0], 0);
                    if (!isStartsWith && !isEndsWith) {
                        return;
                    }
                    const eqNode = parentNode;
                    const negativeIndexSupported = node.property.name === 'slice';
                    context.report({
                        node: parentNode,
                        messageId: isStartsWith ? 'preferStartsWith' : 'preferEndsWith',
                        fix(fixer) {
                            // Don't fix if it can change the behavior.
                            if (eqNode.operator.length === 2 &&
                                (eqNode.right.type !== experimental_utils_1.AST_NODE_TYPES.Literal ||
                                    typeof eqNode.right.value !== 'string')) {
                                return null;
                            }
                            if (isStartsWith) {
                                if (!isLengthExpression(callNode.arguments[1], eqNode.right)) {
                                    return null;
                                }
                            }
                            else {
                                const posNode = callNode.arguments[0];
                                const posNodeIsAbsolutelyValid = (posNode.type === experimental_utils_1.AST_NODE_TYPES.BinaryExpression &&
                                    posNode.operator === '-' &&
                                    isLengthExpression(posNode.left, node.object) &&
                                    isLengthExpression(posNode.right, eqNode.right)) ||
                                    (negativeIndexSupported &&
                                        posNode.type === experimental_utils_1.AST_NODE_TYPES.UnaryExpression &&
                                        posNode.operator === '-' &&
                                        isLengthExpression(posNode.argument, eqNode.right));
                                if (!posNodeIsAbsolutelyValid) {
                                    return null;
                                }
                            }
                            return fixWithRightOperand(fixer, parentNode, isStartsWith ? 'start' : 'end', parentNode.operator.startsWith('!'), node.optional);
                        },
                    });
                },
                // /^bar/.test(foo)
                // /bar$/.test(foo)
                ':matches(CallExpression, OptionalCallExpression) > :matches(MemberExpression, OptionalMemberExpression).callee[property.name="test"][computed=false]'(node) {
                    const callNode = node.parent;
                    const parsed = callNode.arguments.length === 1 ? parseRegExp(node.object) : null;
                    if (parsed == null) {
                        return;
                    }
                    const { isStartsWith, text } = parsed;
                    const messageId = isStartsWith ? 'preferStartsWith' : 'preferEndsWith';
                    const methodName = isStartsWith ? 'startsWith' : 'endsWith';
                    context.report({
                        node: callNode,
                        messageId,
                        *fix(fixer) {
                            const argNode = callNode.arguments[0];
                            const needsParen = argNode.type !== experimental_utils_1.AST_NODE_TYPES.Literal &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.TemplateLiteral &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.Identifier &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.MemberExpression &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.CallExpression &&
                                argNode.type !== experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression;
                            yield fixer.removeRange([callNode.range[0], argNode.range[0]]);
                            if (needsParen) {
                                yield fixer.insertTextBefore(argNode, '(');
                                yield fixer.insertTextAfter(argNode, ')');
                            }
                            yield fixer.insertTextAfter(argNode, `${callNode.type === experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression
                                ? '?.'
                                : '.'}${methodName}(${JSON.stringify(text)}`);
                        },
                    });
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/promise-function-async.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'promise-function-async',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Requires any function or method that returns a Promise to be marked async',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            messages: {
                missingAsync: 'Functions that return promises must be async.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        allowAny: {
                            type: 'boolean',
                        },
                        allowedPromiseNames: {
                            type: 'array',
                            items: {
                                type: 'string',
                            },
                        },
                        checkArrowFunctions: {
                            type: 'boolean',
                        },
                        checkFunctionDeclarations: {
                            type: 'boolean',
                        },
                        checkFunctionExpressions: {
                            type: 'boolean',
                        },
                        checkMethodDeclarations: {
                            type: 'boolean',
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            {
                allowAny: true,
                allowedPromiseNames: [],
                checkArrowFunctions: true,
                checkFunctionDeclarations: true,
                checkFunctionExpressions: true,
                checkMethodDeclarations: true,
            },
        ],
        create(context, [{ allowAny, allowedPromiseNames, checkArrowFunctions, checkFunctionDeclarations, checkFunctionExpressions, checkMethodDeclarations, },]) {
            const allAllowedPromiseNames = new Set([
                'Promise',
                ...allowedPromiseNames,
            ]);
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            function validateNode(node) {
                const originalNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                const signatures = checker
                    .getTypeAtLocation(originalNode)
                    .getCallSignatures();
                if (!signatures.length) {
                    return;
                }
                const returnType = checker.getReturnTypeOfSignature(signatures[0]);
                if (!util.containsAllTypesByName(returnType, allowAny, allAllowedPromiseNames)) {
                    return;
                }
                if (node.parent &&
                    (node.parent.type === experimental_utils_1.AST_NODE_TYPES.Property ||
                        node.parent.type === experimental_utils_1.AST_NODE_TYPES.MethodDefinition) &&
                    (node.parent.kind === 'get' || node.parent.kind === 'set')) {
                    return;
                }
                context.report({
                    messageId: 'missingAsync',
                    node,
                });
            }
            return {
                'ArrowFunctionExpression[async = false]'(node) {
                    if (checkArrowFunctions) {
                        validateNode(node);
                    }
                },
                'FunctionDeclaration[async = false]'(node) {
                    if (checkFunctionDeclarations) {
                        validateNode(node);
                    }
                },
                'FunctionExpression[async = false]'(node) {
                    if (node.parent &&
                        'kind' in node.parent &&
                        node.parent.kind === 'method') {
                        if (checkMethodDeclarations) {
                            validateNode(node.parent);
                        }
                    }
                    else if (checkFunctionExpressions) {
                        validateNode(node);
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/quotes.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const quotes_1 = __importDefault(require(384) /* eslint/lib/rules/quotes */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'quotes',
        meta: {
            type: 'layout',
            docs: {
                description: 'Enforce the consistent use of either backticks, double, or single quotes',
                category: 'Stylistic Issues',
                recommended: false,
                extendsBaseRule: true,
            },
            fixable: 'code',
            messages: quotes_1.default.meta.messages,
            schema: quotes_1.default.meta.schema,
        },
        defaultOptions: [
            'double',
            {
                allowTemplateLiterals: false,
                avoidEscape: false,
            },
        ],
        create(context, [option]) {
            const rules = quotes_1.default.create(context);
            function isAllowedAsNonBacktick(node) {
                const parent = node.parent;
                switch (parent === null || parent === void 0 ? void 0 : parent.type) {
                    case experimental_utils_1.AST_NODE_TYPES.TSAbstractMethodDefinition:
                    case experimental_utils_1.AST_NODE_TYPES.TSMethodSignature:
                    case experimental_utils_1.AST_NODE_TYPES.TSPropertySignature:
                    case experimental_utils_1.AST_NODE_TYPES.TSModuleDeclaration:
                    case experimental_utils_1.AST_NODE_TYPES.TSLiteralType:
                        return true;
                    case experimental_utils_1.AST_NODE_TYPES.TSEnumMember:
                        return node === parent.id;
                    case experimental_utils_1.AST_NODE_TYPES.TSAbstractClassProperty:
                    case experimental_utils_1.AST_NODE_TYPES.ClassProperty:
                        return node === parent.key;
                    default:
                        return false;
                }
            }
            return {
                Literal(node) {
                    if (option === 'backtick' && isAllowedAsNonBacktick(node)) {
                        return;
                    }
                    rules.Literal(node);
                },
                TemplateLiteral(node) {
                    rules.TemplateLiteral(node);
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/require-array-sort-compare.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'require-array-sort-compare',
        defaultOptions: [],
        meta: {
            type: 'problem',
            docs: {
                description: 'Requires `Array#sort` calls to always provide a `compareFunction`',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            messages: {
                requireCompare: "Require 'compare' argument.",
            },
            schema: [],
        },
        create(context) {
            const service = util.getParserServices(context);
            const checker = service.program.getTypeChecker();
            return {
                ":matches(CallExpression, OptionalCallExpression)[arguments.length=0] > :matches(MemberExpression, OptionalMemberExpression)[property.name='sort'][computed=false]"(node) {
                    // Get the symbol of the `sort` method.
                    const tsNode = service.esTreeNodeToTSNodeMap.get(node);
                    const sortSymbol = checker.getSymbolAtLocation(tsNode);
                    if (sortSymbol == null) {
                        return;
                    }
                    // Check the owner type of the `sort` method.
                    for (const methodDecl of sortSymbol.declarations) {
                        const typeDecl = methodDecl.parent;
                        if (ts.isInterfaceDeclaration(typeDecl) &&
                            ts.isSourceFile(typeDecl.parent) &&
                            typeDecl.name.escapedText === 'Array') {
                            context.report({ node: node.parent, messageId: 'requireCompare' });
                            return;
                        }
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/require-await.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const eslint_utils_1 = require(250) /* eslint-utils */;
    const tsutils = __importStar(require(627) /* tsutils */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'require-await',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Disallow async functions which have no `await` expression',
                category: 'Best Practices',
                recommended: 'error',
                requiresTypeChecking: true,
                extendsBaseRule: true,
            },
            schema: [],
            messages: {
                missingAwait: "{{name}} has no 'await' expression.",
            },
        },
        defaultOptions: [],
        create(context) {
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            const sourceCode = context.getSourceCode();
            let scopeInfo = null;
            /**
             * Push the scope info object to the stack.
             */
            function enterFunction(node) {
                scopeInfo = {
                    upper: scopeInfo,
                    hasAwait: false,
                    hasAsync: node.async,
                };
            }
            /**
             * Pop the top scope info object from the stack.
             * Also, it reports the function if needed.
             */
            function exitFunction(node) {
                /* istanbul ignore if */ if (!scopeInfo) {
                    // this shouldn't ever happen, as we have to exit a function after we enter it
                    return;
                }
                if (node.async && !scopeInfo.hasAwait && !isEmptyFunction(node)) {
                    context.report({
                        node,
                        loc: getFunctionHeadLoc(node, sourceCode),
                        messageId: 'missingAwait',
                        data: {
                            name: util.upperCaseFirst(eslint_utils_1.getFunctionNameWithKind(node)),
                        },
                    });
                }
                scopeInfo = scopeInfo.upper;
            }
            /**
             * Checks if the node returns a thenable type
             */
            function isThenableType(node) {
                const type = checker.getTypeAtLocation(node);
                return tsutils.isThenableType(checker, node, type);
            }
            /**
             * Marks the current scope as having an await
             */
            function markAsHasAwait() {
                if (!scopeInfo) {
                    return;
                }
                scopeInfo.hasAwait = true;
            }
            return {
                FunctionDeclaration: enterFunction,
                FunctionExpression: enterFunction,
                ArrowFunctionExpression: enterFunction,
                'FunctionDeclaration:exit': exitFunction,
                'FunctionExpression:exit': exitFunction,
                'ArrowFunctionExpression:exit': exitFunction,
                AwaitExpression: markAsHasAwait,
                'ForOfStatement[await = true]': markAsHasAwait,
                // check body-less async arrow function.
                // ignore `async () => await foo` because it's obviously correct
                'ArrowFunctionExpression[async = true] > :not(BlockStatement, AwaitExpression)'(node) {
                    const expression = parserServices.esTreeNodeToTSNodeMap.get(node);
                    if (expression && isThenableType(expression)) {
                        markAsHasAwait();
                    }
                },
                ReturnStatement(node) {
                    // short circuit early to avoid unnecessary type checks
                    if (!scopeInfo || scopeInfo.hasAwait || !scopeInfo.hasAsync) {
                        return;
                    }
                    const { expression } = parserServices.esTreeNodeToTSNodeMap.get(node);
                    if (expression && isThenableType(expression)) {
                        markAsHasAwait();
                    }
                },
            };
        },
    });
    function isEmptyFunction(node) {
        var _a;
        return (((_a = node.body) === null || _a === void 0 ? void 0 : _a.type) === experimental_utils_1.AST_NODE_TYPES.BlockStatement &&
            node.body.body.length === 0);
    }
    // https://github.com/eslint/eslint/blob/03a69dbe86d5b5768a310105416ae726822e3c1c/lib/rules/utils/ast-utils.js#L382-L392
    /**
     * Gets the `(` token of the given function node.
     */
    function getOpeningParenOfParams(node, sourceCode) {
        return util.nullThrows(node.id
            ? sourceCode.getTokenAfter(node.id, eslint_utils_1.isOpeningParenToken)
            : sourceCode.getFirstToken(node, eslint_utils_1.isOpeningParenToken), util.NullThrowsReasons.MissingToken('(', node.type));
    }
    // https://github.com/eslint/eslint/blob/03a69dbe86d5b5768a310105416ae726822e3c1c/lib/rules/utils/ast-utils.js#L1220-L1242
    /**
     * Gets the location of the given function node for reporting.
     */
    function getFunctionHeadLoc(node, sourceCode) {
        const parent = util.nullThrows(node.parent, util.NullThrowsReasons.MissingParent);
        let start = null;
        let end = null;
        if (node.type === experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression) {
            const arrowToken = util.nullThrows(sourceCode.getTokenBefore(node.body, eslint_utils_1.isArrowToken), util.NullThrowsReasons.MissingToken('=>', node.type));
            start = arrowToken.loc.start;
            end = arrowToken.loc.end;
        }
        else if (parent.type === experimental_utils_1.AST_NODE_TYPES.Property ||
            parent.type === experimental_utils_1.AST_NODE_TYPES.MethodDefinition) {
            start = parent.loc.start;
            end = getOpeningParenOfParams(node, sourceCode).loc.start;
        }
        else {
            start = node.loc.start;
            end = getOpeningParenOfParams(node, sourceCode).loc.start;
        }
        return {
            start,
            end,
        };
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/restrict-plus-operands.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'restrict-plus-operands',
        meta: {
            type: 'problem',
            docs: {
                description: 'When adding two variables, operands must both be of type number or of type string',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            messages: {
                notNumbers: "Operands of '+' operation must either be both strings or both numbers.",
                notStrings: "Operands of '+' operation must either be both strings or both numbers. Consider using a template literal.",
                notBigInts: "Operands of '+' operation must be both bigints.",
            },
            schema: [
                {
                    type: 'object',
                    additionalProperties: false,
                    properties: {
                        checkCompoundAssignments: {
                            type: 'boolean',
                        },
                    },
                },
            ],
        },
        defaultOptions: [
            {
                checkCompoundAssignments: false,
            },
        ],
        create(context, [{ checkCompoundAssignments }]) {
            const service = util.getParserServices(context);
            const typeChecker = service.program.getTypeChecker();
            /**
             * Helper function to get base type of node
             */
            function getBaseTypeOfLiteralType(type) {
                const constraint = type.getConstraint();
                if (constraint &&
                    // for generic types with union constraints, it will return itself from getConstraint
                    // so we have to guard against infinite recursion...
                    constraint !== type) {
                    return getBaseTypeOfLiteralType(constraint);
                }
                if (type.isNumberLiteral()) {
                    return 'number';
                }
                if (type.isStringLiteral()) {
                    return 'string';
                }
                // is BigIntLiteral
                if (type.flags & ts.TypeFlags.BigIntLiteral) {
                    return 'bigint';
                }
                if (type.isUnion()) {
                    const types = type.types.map(getBaseTypeOfLiteralType);
                    return types.every(value => value === types[0]) ? types[0] : 'invalid';
                }
                const stringType = typeChecker.typeToString(type);
                if (stringType === 'number' ||
                    stringType === 'string' ||
                    stringType === 'bigint') {
                    return stringType;
                }
                return 'invalid';
            }
            /**
             * Helper function to get base type of node
             * @param node the node to be evaluated.
             */
            function getNodeType(node) {
                const tsNode = service.esTreeNodeToTSNodeMap.get(node);
                const type = typeChecker.getTypeAtLocation(tsNode);
                return getBaseTypeOfLiteralType(type);
            }
            function checkPlusOperands(node) {
                const leftType = getNodeType(node.left);
                const rightType = getNodeType(node.right);
                if (leftType === 'invalid' ||
                    rightType === 'invalid' ||
                    leftType !== rightType) {
                    if (leftType === 'string' || rightType === 'string') {
                        context.report({
                            node,
                            messageId: 'notStrings',
                        });
                    }
                    else if (leftType === 'bigint' || rightType === 'bigint') {
                        context.report({
                            node,
                            messageId: 'notBigInts',
                        });
                    }
                    else {
                        context.report({
                            node,
                            messageId: 'notNumbers',
                        });
                    }
                }
            }
            return {
                "BinaryExpression[operator='+']": checkPlusOperands,
                "AssignmentExpression[operator='+=']"(node) {
                    if (checkCompoundAssignments) {
                        checkPlusOperands(node);
                    }
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/restrict-template-expressions.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'restrict-template-expressions',
        meta: {
            type: 'problem',
            docs: {
                description: 'Enforce template literal expressions to be of string type',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            messages: {
                invalidType: 'Invalid type of template literal expression.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        allowBoolean: { type: 'boolean' },
                        allowNullable: { type: 'boolean' },
                        allowNumber: { type: 'boolean' },
                    },
                },
            ],
        },
        defaultOptions: [{}],
        create(context, [options]) {
            const service = util.getParserServices(context);
            const typeChecker = service.program.getTypeChecker();
            const allowedTypes = [
                'string',
                ...(options.allowNumber ? ['number', 'bigint'] : []),
                ...(options.allowBoolean ? ['boolean'] : []),
                ...(options.allowNullable ? ['null', 'undefined'] : []),
            ];
            function isAllowedType(types) {
                for (const type of types) {
                    if (!allowedTypes.includes(type)) {
                        return false;
                    }
                }
                return true;
            }
            return {
                TemplateLiteral(node) {
                    // don't check tagged template literals
                    if (node.parent.type === experimental_utils_1.AST_NODE_TYPES.TaggedTemplateExpression) {
                        return;
                    }
                    for (const expr of node.expressions) {
                        const type = getNodeType(expr);
                        if (!isAllowedType(type)) {
                            context.report({
                                node: expr,
                                messageId: 'invalidType',
                            });
                        }
                    }
                },
            };
            /**
             * Helper function to get base type of node
             * @param node the node to be evaluated.
             */
            function getNodeType(node) {
                const tsNode = service.esTreeNodeToTSNodeMap.get(node);
                const type = typeChecker.getTypeAtLocation(tsNode);
                return getBaseType(type);
            }
            function getBaseType(type) {
                const constraint = type.getConstraint();
                if (constraint &&
                    // for generic types with union constraints, it will return itself
                    constraint !== type) {
                    return getBaseType(constraint);
                }
                if (type.isStringLiteral()) {
                    return ['string'];
                }
                if (type.isNumberLiteral()) {
                    return ['number'];
                }
                if (type.flags & ts.TypeFlags.BigIntLiteral) {
                    return ['bigint'];
                }
                if (type.flags & ts.TypeFlags.BooleanLiteral) {
                    return ['boolean'];
                }
                if (type.flags & ts.TypeFlags.Null) {
                    return ['null'];
                }
                if (type.flags & ts.TypeFlags.Undefined) {
                    return ['undefined'];
                }
                if (type.isUnion()) {
                    return type.types
                        .map(getBaseType)
                        .reduce((all, array) => [...all, ...array], []);
                }
                const stringType = typeChecker.typeToString(type);
                if (stringType === 'string' ||
                    stringType === 'number' ||
                    stringType === 'bigint' ||
                    stringType === 'boolean') {
                    return [stringType];
                }
                return ['other'];
            }
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/return-await.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const tsutils = __importStar(require(627) /* tsutils */);
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'return-await',
        meta: {
            docs: {
                description: 'Enforces consistent returning of awaited values',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
                extendsBaseRule: 'no-return-await',
            },
            fixable: 'code',
            type: 'problem',
            messages: {
                nonPromiseAwait: 'returning an awaited value that is not a promise is not allowed',
                disallowedPromiseAwait: 'returning an awaited promise is not allowed in this context',
                requiredPromiseAwait: 'returning an awaited promise is required in this context',
            },
            schema: [
                {
                    enum: ['in-try-catch', 'always', 'never'],
                },
            ],
        },
        defaultOptions: ['in-try-catch'],
        create(context, [option]) {
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            const sourceCode = context.getSourceCode();
            function inTryCatch(node) {
                let ancestor = node.parent;
                while (ancestor && !ts.isFunctionLike(ancestor)) {
                    if (tsutils.isTryStatement(ancestor) ||
                        tsutils.isCatchClause(ancestor)) {
                        return true;
                    }
                    ancestor = ancestor.parent;
                }
                return false;
            }
            // function findTokensToRemove()
            function removeAwait(fixer, node) {
                const awaitNode = node.type === experimental_utils_1.AST_NODE_TYPES.ReturnStatement
                    ? node.argument
                    : node.body;
                // Should always be an await node; but let's be safe.
                /* istanbul ignore if */ if (!util.isAwaitExpression(awaitNode)) {
                    return null;
                }
                const awaitToken = sourceCode.getFirstToken(awaitNode, util.isAwaitKeyword);
                // Should always be the case; but let's be safe.
                /* istanbul ignore if */ if (!awaitToken) {
                    return null;
                }
                const startAt = awaitToken.range[0];
                let endAt = awaitToken.range[1];
                // Also remove any extraneous whitespace after `await`, if there is any.
                const nextToken = sourceCode.getTokenAfter(awaitToken, {
                    includeComments: true,
                });
                if (nextToken) {
                    endAt = nextToken.range[0];
                }
                return fixer.removeRange([startAt, endAt]);
            }
            function insertAwait(fixer, node) {
                const targetNode = node.type === experimental_utils_1.AST_NODE_TYPES.ReturnStatement
                    ? node.argument
                    : node.body;
                // There should always be a target node; but let's be safe.
                /* istanbul ignore if */ if (!targetNode) {
                    return null;
                }
                return fixer.insertTextBefore(targetNode, 'await ');
            }
            function test(node, expression) {
                let child;
                const isAwait = tsutils.isAwaitExpression(expression);
                if (isAwait) {
                    child = expression.getChildAt(1);
                }
                else {
                    child = expression;
                }
                const type = checker.getTypeAtLocation(child);
                const isThenable = tsutils.isThenableType(checker, expression, type);
                if (!isAwait && !isThenable) {
                    return;
                }
                if (isAwait && !isThenable) {
                    context.report({
                        messageId: 'nonPromiseAwait',
                        node,
                        fix: fixer => removeAwait(fixer, node),
                    });
                    return;
                }
                if (option === 'always') {
                    if (!isAwait && isThenable) {
                        context.report({
                            messageId: 'requiredPromiseAwait',
                            node,
                            fix: fixer => insertAwait(fixer, node),
                        });
                    }
                    return;
                }
                if (option === 'never') {
                    if (isAwait) {
                        context.report({
                            messageId: 'disallowedPromiseAwait',
                            node,
                            fix: fixer => removeAwait(fixer, node),
                        });
                    }
                    return;
                }
                if (option === 'in-try-catch') {
                    const isInTryCatch = inTryCatch(expression);
                    if (isAwait && !isInTryCatch) {
                        context.report({
                            messageId: 'disallowedPromiseAwait',
                            node,
                            fix: fixer => removeAwait(fixer, node),
                        });
                    }
                    else if (!isAwait && isInTryCatch) {
                        context.report({
                            messageId: 'requiredPromiseAwait',
                            node,
                            fix: fixer => insertAwait(fixer, node),
                        });
                    }
                    return;
                }
            }
            return {
                'ArrowFunctionExpression[async = true]:exit'(node) {
                    if (node.body.type !== experimental_utils_1.AST_NODE_TYPES.BlockStatement) {
                        const expression = parserServices.esTreeNodeToTSNodeMap.get(node.body);
                        test(node, expression);
                    }
                },
                ReturnStatement(node) {
                    const originalNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                    const { expression } = originalNode;
                    if (!expression) {
                        return;
                    }
                    test(node, expression);
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/semi.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const semi_1 = __importDefault(require(392) /* eslint/lib/rules/semi */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'semi',
        meta: {
            type: 'layout',
            docs: {
                description: 'Require or disallow semicolons instead of ASI',
                category: 'Stylistic Issues',
                // too opinionated to be recommended
                recommended: false,
                extendsBaseRule: true,
            },
            fixable: 'code',
            schema: semi_1.default.meta.schema,
            messages: semi_1.default.meta.messages,
        },
        defaultOptions: [
            'always',
            {
                omitLastInOneLineBlock: false,
                beforeStatementContinuationChars: 'any',
            },
        ],
        create(context) {
            const rules = semi_1.default.create(context);
            const checkForSemicolon = rules.ExpressionStatement;
            /*
              The following nodes are handled by the member-delimiter-style rule
              AST_NODE_TYPES.TSCallSignatureDeclaration,
              AST_NODE_TYPES.TSConstructSignatureDeclaration,
              AST_NODE_TYPES.TSIndexSignature,
              AST_NODE_TYPES.TSMethodSignature,
              AST_NODE_TYPES.TSPropertySignature,
            */
            const nodesToCheck = [
                experimental_utils_1.AST_NODE_TYPES.ClassProperty,
                experimental_utils_1.AST_NODE_TYPES.TSAbstractClassProperty,
                experimental_utils_1.AST_NODE_TYPES.TSAbstractMethodDefinition,
                experimental_utils_1.AST_NODE_TYPES.TSDeclareFunction,
                experimental_utils_1.AST_NODE_TYPES.TSExportAssignment,
                experimental_utils_1.AST_NODE_TYPES.TSImportEqualsDeclaration,
                experimental_utils_1.AST_NODE_TYPES.TSTypeAliasDeclaration,
            ].reduce((acc, node) => {
                acc[node] = checkForSemicolon;
                return acc;
            }, {});
            return Object.assign(Object.assign(Object.assign({}, rules), nodesToCheck), { ExportDefaultDeclaration(node) {
                    if (node.declaration.type !== experimental_utils_1.AST_NODE_TYPES.TSInterfaceDeclaration) {
                        rules.ExportDefaultDeclaration(node);
                    }
                } });
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/space-before-function-paren.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const eslint_utils_1 = require(250) /* eslint-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'space-before-function-paren',
        meta: {
            type: 'layout',
            docs: {
                description: 'Enforces consistent spacing before function parenthesis',
                category: 'Stylistic Issues',
                recommended: false,
                extendsBaseRule: true,
            },
            fixable: 'whitespace',
            schema: [
                {
                    oneOf: [
                        {
                            enum: ['always', 'never'],
                        },
                        {
                            type: 'object',
                            properties: {
                                anonymous: {
                                    enum: ['always', 'never', 'ignore'],
                                },
                                named: {
                                    enum: ['always', 'never', 'ignore'],
                                },
                                asyncArrow: {
                                    enum: ['always', 'never', 'ignore'],
                                },
                            },
                            additionalProperties: false,
                        },
                    ],
                },
            ],
            messages: {
                unexpected: 'Unexpected space before function parentheses.',
                missing: 'Missing space before function parentheses.',
            },
        },
        defaultOptions: ['always'],
        create(context) {
            const sourceCode = context.getSourceCode();
            const baseConfig = typeof context.options[0] === 'string' ? context.options[0] : 'always';
            const overrideConfig = typeof context.options[0] === 'object' ? context.options[0] : {};
            /**
             * Determines whether a function has a name.
             * @param {ASTNode} node The function node.
             * @returns {boolean} Whether the function has a name.
             */
            function isNamedFunction(node) {
                if (node.id) {
                    return true;
                }
                const parent = node.parent;
                return (parent.type === experimental_utils_1.AST_NODE_TYPES.MethodDefinition ||
                    (parent.type === experimental_utils_1.AST_NODE_TYPES.Property &&
                        (parent.kind === 'get' || parent.kind === 'set' || parent.method)));
            }
            /**
             * Gets the config for a given function
             * @param {ASTNode} node The function node
             * @returns {string} "always", "never", or "ignore"
             */
            function getConfigForFunction(node) {
                var _a, _b, _c;
                if (node.type === experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression) {
                    // Always ignore non-async functions and arrow functions without parens, e.g. async foo => bar
                    if (node.async &&
                        eslint_utils_1.isOpeningParenToken(sourceCode.getFirstToken(node, { skip: 1 }))) {
                        return (_a = overrideConfig.asyncArrow) !== null && _a !== void 0 ? _a : baseConfig;
                    }
                }
                else if (isNamedFunction(node)) {
                    return (_b = overrideConfig.named) !== null && _b !== void 0 ? _b : baseConfig;
                    // `generator-star-spacing` should warn anonymous generators. E.g. `function* () {}`
                }
                else if (!node.generator) {
                    return (_c = overrideConfig.anonymous) !== null && _c !== void 0 ? _c : baseConfig;
                }
                return 'ignore';
            }
            /**
             * Checks the parens of a function node
             * @param {ASTNode} node A function node
             * @returns {void}
             */
            function checkFunction(node) {
                const functionConfig = getConfigForFunction(node);
                if (functionConfig === 'ignore') {
                    return;
                }
                let leftToken, rightToken;
                if (node.typeParameters) {
                    leftToken = sourceCode.getLastToken(node.typeParameters);
                    rightToken = sourceCode.getTokenAfter(leftToken);
                }
                else {
                    rightToken = sourceCode.getFirstToken(node, eslint_utils_1.isOpeningParenToken);
                    leftToken = sourceCode.getTokenBefore(rightToken);
                }
                const hasSpacing = sourceCode.isSpaceBetweenTokens(leftToken, rightToken);
                if (hasSpacing && functionConfig === 'never') {
                    context.report({
                        node,
                        loc: leftToken.loc.end,
                        messageId: 'unexpected',
                        fix: fixer => fixer.removeRange([leftToken.range[1], rightToken.range[0]]),
                    });
                }
                else if (!hasSpacing &&
                    functionConfig === 'always' &&
                    (!node.typeParameters || node.id)) {
                    context.report({
                        node,
                        loc: leftToken.loc.end,
                        messageId: 'missing',
                        fix: fixer => fixer.insertTextAfter(leftToken, ' '),
                    });
                }
            }
            return {
                ArrowFunctionExpression: checkFunction,
                FunctionDeclaration: checkFunction,
                FunctionExpression: checkFunction,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/strict-boolean-expressions.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const ts = __importStar(require(44) /* typescript */);
    const tsutils = __importStar(require(627) /* tsutils */);
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'strict-boolean-expressions',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Restricts the types allowed in boolean expressions',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        ignoreRhs: {
                            type: 'boolean',
                        },
                        allowNullable: {
                            type: 'boolean',
                        },
                        allowSafe: {
                            type: 'boolean',
                        },
                    },
                    additionalProperties: false,
                },
            ],
            messages: {
                conditionErrorOther: 'Unexpected value in conditional. ' +
                    'A boolean expression is required.',
                conditionErrorAny: 'Unexpected any value in conditional. ' +
                    'An explicit comparison or type cast is required.',
                conditionErrorNullish: 'Unexpected nullish value in conditional. ' +
                    'The condition is always false.',
                conditionErrorNullableBoolean: 'Unexpected nullable boolean value in conditional. ' +
                    'Please handle the nullish case explicitly.',
                conditionErrorString: 'Unexpected string value in conditional. ' +
                    'An explicit empty string check is required.',
                conditionErrorNullableString: 'Unexpected nullable string value in conditional. ' +
                    'Please handle the nullish/empty cases explicitly.',
                conditionErrorNumber: 'Unexpected number value in conditional. ' +
                    'An explicit zero/NaN check is required.',
                conditionErrorNullableNumber: 'Unexpected nullable number value in conditional. ' +
                    'Please handle the nullish/zero/NaN cases explicitly.',
                conditionErrorObject: 'Unexpected object value in conditional. ' +
                    'The condition is always true.',
                conditionErrorNullableObject: 'Unexpected nullable object value in conditional. ' +
                    'An explicit null check is required.',
            },
        },
        defaultOptions: [
            {
                ignoreRhs: false,
                allowNullable: false,
                allowSafe: false,
            },
        ],
        create(context, [options]) {
            const service = util.getParserServices(context);
            const checker = service.program.getTypeChecker();
            const checkedNodes = new Set();
            return {
                ConditionalExpression: checkTestExpression,
                DoWhileStatement: checkTestExpression,
                ForStatement: checkTestExpression,
                IfStatement: checkTestExpression,
                WhileStatement: checkTestExpression,
                'LogicalExpression[operator!="??"]': checkNode,
                'UnaryExpression[operator="!"]': checkUnaryLogicalExpression,
            };
            function checkTestExpression(node) {
                if (node.test == null) {
                    return;
                }
                checkNode(node.test, true);
            }
            function checkUnaryLogicalExpression(node) {
                checkNode(node.argument, true);
            }
            /**
             * This function analyzes the type of a boolean expression node and checks if it is allowed.
             * It can recurse when checking nested logical operators, so that only the outermost expressions are reported.
             * @param node The AST node to check.
             * @param isRoot Whether it is the root of a logical expression and there was no recursion yet.
             * @returns `true` if there was an error reported.
             */
            function checkNode(node, isRoot = false) {
                // prevent checking the same node multiple times
                if (checkedNodes.has(node)) {
                    return false;
                }
                checkedNodes.add(node);
                // for logical operator, we also check its operands
                if (node.type === experimental_utils_1.AST_NODE_TYPES.LogicalExpression &&
                    node.operator !== '??') {
                    let hasError = false;
                    if (checkNode(node.left)) {
                        hasError = true;
                    }
                    if (!options.ignoreRhs) {
                        if (checkNode(node.right)) {
                            hasError = true;
                        }
                    }
                    // if this logical operator is not the root of a logical expression
                    // we only check its operands and return
                    if (!isRoot) {
                        return hasError;
                    }
                    // if this is the root of a logical expression
                    // we want to check its resulting type too
                    else {
                        // ...unless there already was an error, we exit so we don't double-report
                        if (hasError) {
                            return true;
                        }
                    }
                }
                const tsNode = service.esTreeNodeToTSNodeMap.get(node);
                const type = util.getConstrainedTypeAtLocation(checker, tsNode);
                let messageId;
                const types = inspectVariantTypes(tsutils.unionTypeParts(type));
                const is = (...wantedTypes) => types.size === wantedTypes.length &&
                    wantedTypes.every(type => types.has(type));
                // boolean
                if (is('boolean')) {
                    // boolean is always okay
                    return false;
                }
                // never
                if (is('never')) {
                    // never is always okay
                    return false;
                }
                // nullish
                else if (is('nullish')) {
                    // condition is always false
                    messageId = 'conditionErrorNullish';
                }
                // nullable boolean
                else if (is('nullish', 'boolean')) {
                    if (!options.allowNullable) {
                        messageId = 'conditionErrorNullableBoolean';
                    }
                }
                // string
                else if (is('string')) {
                    messageId = 'conditionErrorString';
                }
                // nullable string
                else if (is('nullish', 'string')) {
                    messageId = 'conditionErrorNullableString';
                }
                // number
                else if (is('number')) {
                    messageId = 'conditionErrorNumber';
                }
                // nullable number
                else if (is('nullish', 'number')) {
                    messageId = 'conditionErrorNullableNumber';
                }
                // object
                else if (is('object')) {
                    // condition is always true
                    if (!options.allowSafe) {
                        messageId = 'conditionErrorObject';
                    }
                }
                // nullable object
                else if (is('nullish', 'object')) {
                    if (!options.allowSafe || !options.allowNullable) {
                        messageId = 'conditionErrorNullableObject';
                    }
                }
                // boolean/object
                else if (is('boolean', 'object')) {
                    if (!options.allowSafe) {
                        messageId = 'conditionErrorOther';
                    }
                }
                // nullable boolean/object
                else if (is('nullish', 'boolean', 'object')) {
                    if (!options.allowSafe || !options.allowNullable) {
                        messageId = 'conditionErrorOther';
                    }
                }
                // any
                else if (is('any')) {
                    messageId = 'conditionErrorAny';
                }
                // other
                else {
                    messageId = 'conditionErrorOther';
                }
                if (messageId != null) {
                    context.report({ node, messageId });
                    return true;
                }
                return false;
            }
            /**
             * Check union variants for the types we care about
             */
            function inspectVariantTypes(types) {
                const variantTypes = new Set();
                if (types.some(type => tsutils.isTypeFlagSet(type, ts.TypeFlags.Null | ts.TypeFlags.Undefined | ts.TypeFlags.VoidLike))) {
                    variantTypes.add('nullish');
                }
                if (types.some(type => tsutils.isTypeFlagSet(type, ts.TypeFlags.BooleanLike))) {
                    variantTypes.add('boolean');
                }
                if (types.some(type => tsutils.isTypeFlagSet(type, ts.TypeFlags.StringLike))) {
                    variantTypes.add('string');
                }
                if (types.some(type => tsutils.isTypeFlagSet(type, ts.TypeFlags.NumberLike))) {
                    variantTypes.add('number');
                }
                if (types.some(type => !tsutils.isTypeFlagSet(type, ts.TypeFlags.Null |
                    ts.TypeFlags.Undefined |
                    ts.TypeFlags.VoidLike |
                    ts.TypeFlags.BooleanLike |
                    ts.TypeFlags.StringLike |
                    ts.TypeFlags.NumberLike |
                    ts.TypeFlags.Any |
                    ts.TypeFlags.Unknown |
                    ts.TypeFlags.Never))) {
                    variantTypes.add('object');
                }
                if (types.some(type => tsutils.isTypeFlagSet(type, ts.TypeFlags.Any | ts.TypeFlags.Unknown))) {
                    variantTypes.add('any');
                }
                if (types.some(type => tsutils.isTypeFlagSet(type, ts.TypeFlags.Never))) {
                    variantTypes.add('never');
                }
                return variantTypes;
            }
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/switch-exhaustiveness-check.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = __importStar(require(44) /* typescript */);
    const util_1 = require(622) /* ../util */;
    const tsutils_1 = require(627) /* tsutils */;
    const eslint_utils_1 = require(250) /* eslint-utils */;
    exports.default = util_1.createRule({
        name: 'switch-exhaustiveness-check',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Exhaustiveness checking in switch with union type',
                category: 'Best Practices',
                recommended: false,
                requiresTypeChecking: true,
            },
            schema: [],
            messages: {
                switchIsNotExhaustive: 'Switch is not exhaustive. Cases not matched: {{missingBranches}}',
                addMissingCases: 'Add branches for missing cases',
            },
        },
        defaultOptions: [],
        create(context) {
            const sourceCode = context.getSourceCode();
            const service = util_1.getParserServices(context);
            const checker = service.program.getTypeChecker();
            function getNodeType(node) {
                const tsNode = service.esTreeNodeToTSNodeMap.get(node);
                return util_1.getConstrainedTypeAtLocation(checker, tsNode);
            }
            function fixSwitch(fixer, node, missingBranchTypes) {
                const lastCase = node.cases.length > 0 ? node.cases[node.cases.length - 1] : null;
                const caseIndent = lastCase
                    ? ' '.repeat(lastCase.loc.start.column)
                    : // if there are no cases, use indentation of the switch statement
                        // and leave it to user to format it correctly
                        ' '.repeat(node.loc.start.column);
                const missingCases = [];
                for (const missingBranchType of missingBranchTypes) {
                    // While running this rule on checker.ts of TypeScript project
                    // the fix introduced a compiler error due to:
                    //
                    // type __String = (string & {
                    //         __escapedIdentifier: void;
                    //     }) | (void & {
                    //         __escapedIdentifier: void;
                    //     }) | InternalSymbolName;
                    //
                    // The following check fixes it.
                    if (missingBranchType.isIntersection()) {
                        continue;
                    }
                    const caseTest = checker.typeToString(missingBranchType);
                    const errorMessage = `Not implemented yet: ${caseTest} case`;
                    missingCases.push(`case ${caseTest}: { throw new Error('${errorMessage}') }`);
                }
                const fixString = missingCases
                    .map(code => `${caseIndent}${code}`)
                    .join('\n');
                if (lastCase) {
                    return fixer.insertTextAfter(lastCase, `\n${fixString}`);
                }
                // there were no existing cases
                const openingBrace = sourceCode.getTokenAfter(node.discriminant, eslint_utils_1.isOpeningBraceToken);
                const closingBrace = sourceCode.getTokenAfter(node.discriminant, eslint_utils_1.isClosingBraceToken);
                return fixer.replaceTextRange([openingBrace.range[0], closingBrace.range[1]], ['{', fixString, `${caseIndent}}`].join('\n'));
            }
            function checkSwitchExhaustive(node) {
                const discriminantType = getNodeType(node.discriminant);
                if (discriminantType.isUnion()) {
                    const unionTypes = tsutils_1.unionTypeParts(discriminantType);
                    const caseTypes = new Set();
                    for (const switchCase of node.cases) {
                        if (switchCase.test === null) {
                            // Switch has 'default' branch - do nothing.
                            return;
                        }
                        caseTypes.add(getNodeType(switchCase.test));
                    }
                    const missingBranchTypes = unionTypes.filter(unionType => !caseTypes.has(unionType));
                    if (missingBranchTypes.length === 0) {
                        // All cases matched - do nothing.
                        return;
                    }
                    context.report({
                        node: node.discriminant,
                        messageId: 'switchIsNotExhaustive',
                        data: {
                            missingBranches: missingBranchTypes
                                .map(missingType => tsutils_1.isTypeFlagSet(missingType, ts.TypeFlags.ESSymbolLike)
                                ? `typeof ${missingType.symbol.escapedName}`
                                : checker.typeToString(missingType))
                                .join(' | '),
                        },
                        suggest: [
                            {
                                messageId: 'addMissingCases',
                                fix(fixer) {
                                    return fixSwitch(fixer, node, missingBranchTypes);
                                },
                            },
                        ],
                    });
                }
            }
            return {
                SwitchStatement: checkSwitchExhaustive,
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/triple-slash-reference.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'triple-slash-reference',
        meta: {
            type: 'suggestion',
            docs: {
                description: 'Sets preference level for triple slash directives versus ES6-style import declarations',
                category: 'Best Practices',
                recommended: 'error',
            },
            messages: {
                tripleSlashReference: 'Do not use a triple slash reference for {{module}}, use `import` style instead.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        lib: {
                            enum: ['always', 'never'],
                        },
                        path: {
                            enum: ['always', 'never'],
                        },
                        types: {
                            enum: ['always', 'never', 'prefer-import'],
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            {
                lib: 'always',
                path: 'never',
                types: 'prefer-import',
            },
        ],
        create(context, [{ lib, path, types }]) {
            let programNode;
            const sourceCode = context.getSourceCode();
            const references = [];
            function hasMatchingReference(source) {
                references.forEach(reference => {
                    if (reference.importName === source.value) {
                        context.report({
                            node: reference.comment,
                            messageId: 'tripleSlashReference',
                            data: {
                                module: reference.importName,
                            },
                        });
                    }
                });
            }
            return {
                ImportDeclaration(node) {
                    if (programNode) {
                        hasMatchingReference(node.source);
                    }
                },
                TSImportEqualsDeclaration(node) {
                    if (programNode) {
                        const source = node.moduleReference
                            .expression;
                        hasMatchingReference(source);
                    }
                },
                Program(node) {
                    if (lib === 'always' && path === 'always' && types == 'always') {
                        return;
                    }
                    programNode = node;
                    const referenceRegExp = /^\/\s*<reference\s*(types|path|lib)\s*=\s*["|'](.*)["|']/;
                    const commentsBefore = sourceCode.getCommentsBefore(programNode);
                    commentsBefore.forEach(comment => {
                        if (comment.type !== experimental_utils_1.AST_TOKEN_TYPES.Line) {
                            return;
                        }
                        const referenceResult = referenceRegExp.exec(comment.value);
                        if (referenceResult) {
                            if ((referenceResult[1] === 'types' && types === 'never') ||
                                (referenceResult[1] === 'path' && path === 'never') ||
                                (referenceResult[1] === 'lib' && lib === 'never')) {
                                context.report({
                                    node: comment,
                                    messageId: 'tripleSlashReference',
                                    data: {
                                        module: referenceResult[2],
                                    },
                                });
                                return;
                            }
                            if (referenceResult[1] === 'types' && types === 'prefer-import') {
                                references.push({ comment, importName: referenceResult[2] });
                            }
                        }
                    });
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/type-annotation-spacing.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const util = __importStar(require(622) /* ../util */);
    const util_1 = require(622) /* ../util */;
    const definition = {
        type: 'object',
        properties: {
            before: { type: 'boolean' },
            after: { type: 'boolean' },
        },
        additionalProperties: false,
    };
    function createRules(options) {
        var _a;
        const globals = Object.assign(Object.assign({}, ((options === null || options === void 0 ? void 0 : options.before) !== undefined ? { before: options.before } : {})), ((options === null || options === void 0 ? void 0 : options.after) !== undefined ? { after: options.after } : {}));
        const override = (_a = options === null || options === void 0 ? void 0 : options.overrides) !== null && _a !== void 0 ? _a : {};
        const colon = Object.assign(Object.assign({ before: false, after: true }, globals), override === null || override === void 0 ? void 0 : override.colon);
        const arrow = Object.assign(Object.assign({ before: true, after: true }, globals), override === null || override === void 0 ? void 0 : override.arrow);
        return {
            colon: colon,
            arrow: arrow,
            variable: Object.assign(Object.assign({}, colon), override === null || override === void 0 ? void 0 : override.variable),
            property: Object.assign(Object.assign({}, colon), override === null || override === void 0 ? void 0 : override.property),
            parameter: Object.assign(Object.assign({}, colon), override === null || override === void 0 ? void 0 : override.parameter),
            returnType: Object.assign(Object.assign({}, colon), override === null || override === void 0 ? void 0 : override.returnType),
        };
    }
    function getIdentifierRules(rules, node) {
        const scope = node === null || node === void 0 ? void 0 : node.parent;
        if (util_1.isVariableDeclarator(scope)) {
            return rules.variable;
        }
        else if (util_1.isFunctionOrFunctionType(scope)) {
            return rules.parameter;
        }
        else {
            return rules.colon;
        }
    }
    function getRules(rules, node) {
        var _a;
        const scope = (_a = node === null || node === void 0 ? void 0 : node.parent) === null || _a === void 0 ? void 0 : _a.parent;
        if (util_1.isTSFunctionType(scope) || util_1.isTSConstructorType(scope)) {
            return rules.arrow;
        }
        else if (util_1.isIdentifier(scope)) {
            return getIdentifierRules(rules, scope);
        }
        else if (util_1.isClassOrTypeElement(scope)) {
            return rules.property;
        }
        else if (util_1.isFunction(scope)) {
            return rules.returnType;
        }
        else {
            return rules.colon;
        }
    }
    exports.default = util.createRule({
        name: 'type-annotation-spacing',
        meta: {
            type: 'layout',
            docs: {
                description: 'Require consistent spacing around type annotations',
                category: 'Stylistic Issues',
                recommended: 'error',
            },
            fixable: 'whitespace',
            messages: {
                expectedSpaceAfter: "Expected a space after the '{{type}}'.",
                expectedSpaceBefore: "Expected a space before the '{{type}}'.",
                unexpectedSpaceAfter: "Unexpected space after the '{{type}}'.",
                unexpectedSpaceBefore: "Unexpected space before the '{{type}}'.",
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        before: { type: 'boolean' },
                        after: { type: 'boolean' },
                        overrides: {
                            type: 'object',
                            properties: {
                                colon: definition,
                                arrow: definition,
                                variable: definition,
                                parameter: definition,
                                property: definition,
                                returnType: definition,
                            },
                            additionalProperties: false,
                        },
                    },
                    additionalProperties: false,
                },
            ],
        },
        defaultOptions: [
            // technically there is a default, but the overrides mean
            // that if we apply them here, it will break the no override case.
            {},
        ],
        create(context, [options]) {
            const punctuators = [':', '=>'];
            const sourceCode = context.getSourceCode();
            const ruleSet = createRules(options);
            /**
             * Checks if there's proper spacing around type annotations (no space
             * before colon, one space after).
             */
            function checkTypeAnnotationSpacing(typeAnnotation) {
                const nextToken = typeAnnotation;
                const punctuatorTokenEnd = sourceCode.getTokenBefore(nextToken);
                let punctuatorTokenStart = punctuatorTokenEnd;
                let previousToken = sourceCode.getTokenBefore(punctuatorTokenEnd);
                let type = punctuatorTokenEnd.value;
                if (!punctuators.includes(type)) {
                    return;
                }
                const { before, after } = getRules(ruleSet, typeAnnotation);
                if (type === ':' && previousToken.value === '?') {
                    // shift the start to the ?
                    type = '?:';
                    punctuatorTokenStart = previousToken;
                    previousToken = sourceCode.getTokenBefore(previousToken);
                    // handle the +/- modifiers for optional modification operators
                    if (previousToken.value === '+' || previousToken.value === '-') {
                        type = `${previousToken.value}?:`;
                        punctuatorTokenStart = previousToken;
                        previousToken = sourceCode.getTokenBefore(previousToken);
                    }
                }
                const previousDelta = punctuatorTokenStart.range[0] - previousToken.range[1];
                const nextDelta = nextToken.range[0] - punctuatorTokenEnd.range[1];
                if (after && nextDelta === 0) {
                    context.report({
                        node: punctuatorTokenEnd,
                        messageId: 'expectedSpaceAfter',
                        data: {
                            type,
                        },
                        fix(fixer) {
                            return fixer.insertTextAfter(punctuatorTokenEnd, ' ');
                        },
                    });
                }
                else if (!after && nextDelta > 0) {
                    context.report({
                        node: punctuatorTokenEnd,
                        messageId: 'unexpectedSpaceAfter',
                        data: {
                            type,
                        },
                        fix(fixer) {
                            return fixer.removeRange([
                                punctuatorTokenEnd.range[1],
                                nextToken.range[0],
                            ]);
                        },
                    });
                }
                if (before && previousDelta === 0) {
                    context.report({
                        node: punctuatorTokenStart,
                        messageId: 'expectedSpaceBefore',
                        data: {
                            type,
                        },
                        fix(fixer) {
                            return fixer.insertTextAfter(previousToken, ' ');
                        },
                    });
                }
                else if (!before && previousDelta > 0) {
                    context.report({
                        node: punctuatorTokenStart,
                        messageId: 'unexpectedSpaceBefore',
                        data: {
                            type,
                        },
                        fix(fixer) {
                            return fixer.removeRange([
                                previousToken.range[1],
                                punctuatorTokenStart.range[0],
                            ]);
                        },
                    });
                }
            }
            return {
                TSMappedType(node) {
                    if (node.typeAnnotation) {
                        checkTypeAnnotationSpacing(node.typeAnnotation);
                    }
                },
                TSTypeAnnotation(node) {
                    checkTypeAnnotationSpacing(node.typeAnnotation);
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/typedef.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'typedef',
        meta: {
            docs: {
                description: 'Requires type annotations to exist',
                category: 'Stylistic Issues',
                recommended: false,
            },
            messages: {
                expectedTypedef: 'expected a type annotation',
                expectedTypedefNamed: 'expected {{name}} to have a type annotation',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        ["arrayDestructuring" /* ArrayDestructuring */]: { type: 'boolean' },
                        ["arrowParameter" /* ArrowParameter */]: { type: 'boolean' },
                        ["memberVariableDeclaration" /* MemberVariableDeclaration */]: { type: 'boolean' },
                        ["objectDestructuring" /* ObjectDestructuring */]: { type: 'boolean' },
                        ["parameter" /* Parameter */]: { type: 'boolean' },
                        ["propertyDeclaration" /* PropertyDeclaration */]: { type: 'boolean' },
                        ["variableDeclaration" /* VariableDeclaration */]: { type: 'boolean' },
                        ["variableDeclarationIgnoreFunction" /* VariableDeclarationIgnoreFunction */]: { type: 'boolean' },
                    },
                },
            ],
            type: 'suggestion',
        },
        defaultOptions: [
            {
                ["arrowParameter" /* ArrowParameter */]: true,
                ["memberVariableDeclaration" /* MemberVariableDeclaration */]: true,
                ["parameter" /* Parameter */]: true,
                ["propertyDeclaration" /* PropertyDeclaration */]: true,
            },
        ],
        create(context, [options]) {
            function report(location, name) {
                context.report({
                    node: location,
                    messageId: name ? 'expectedTypedefNamed' : 'expectedTypedef',
                    data: { name },
                });
            }
            function getNodeName(node) {
                return node.type === experimental_utils_1.AST_NODE_TYPES.Identifier ? node.name : undefined;
            }
            function isForOfStatementContext(node) {
                let current = node.parent;
                while (current) {
                    switch (current.type) {
                        case experimental_utils_1.AST_NODE_TYPES.VariableDeclarator:
                        case experimental_utils_1.AST_NODE_TYPES.VariableDeclaration:
                        case experimental_utils_1.AST_NODE_TYPES.ObjectPattern:
                        case experimental_utils_1.AST_NODE_TYPES.ArrayPattern:
                        case experimental_utils_1.AST_NODE_TYPES.Property:
                            current = current.parent;
                            break;
                        case experimental_utils_1.AST_NODE_TYPES.ForOfStatement:
                            return true;
                        default:
                            current = undefined;
                    }
                }
                return false;
            }
            function checkParameters(params) {
                for (const param of params) {
                    let annotationNode;
                    switch (param.type) {
                        case experimental_utils_1.AST_NODE_TYPES.AssignmentPattern:
                            annotationNode = param.left;
                            break;
                        case experimental_utils_1.AST_NODE_TYPES.TSParameterProperty:
                            annotationNode = param.parameter;
                            // Check TS parameter property with default value like `constructor(private param: string = 'something') {}`
                            if (annotationNode &&
                                annotationNode.type === experimental_utils_1.AST_NODE_TYPES.AssignmentPattern) {
                                annotationNode = annotationNode.left;
                            }
                            break;
                        default:
                            annotationNode = param;
                            break;
                    }
                    if (annotationNode !== undefined && !annotationNode.typeAnnotation) {
                        report(param, getNodeName(param));
                    }
                }
            }
            function isVariableDeclarationIgnoreFunction(node) {
                return (!!options["variableDeclarationIgnoreFunction" /* VariableDeclarationIgnoreFunction */] &&
                    (node.type === experimental_utils_1.AST_NODE_TYPES.FunctionExpression ||
                        node.type === experimental_utils_1.AST_NODE_TYPES.ArrowFunctionExpression));
            }
            return {
                ArrayPattern(node) {
                    if (options["arrayDestructuring" /* ArrayDestructuring */] &&
                        !node.typeAnnotation &&
                        !isForOfStatementContext(node)) {
                        report(node);
                    }
                },
                ArrowFunctionExpression(node) {
                    if (options["arrowParameter" /* ArrowParameter */]) {
                        checkParameters(node.params);
                    }
                },
                ClassProperty(node) {
                    if (node.value && isVariableDeclarationIgnoreFunction(node.value)) {
                        return;
                    }
                    if (options["memberVariableDeclaration" /* MemberVariableDeclaration */] &&
                        !node.typeAnnotation) {
                        report(node, node.key.type === experimental_utils_1.AST_NODE_TYPES.Identifier
                            ? node.key.name
                            : undefined);
                    }
                },
                'FunctionDeclaration, FunctionExpression'(node) {
                    if (options["parameter" /* Parameter */]) {
                        checkParameters(node.params);
                    }
                },
                ObjectPattern(node) {
                    if (options["objectDestructuring" /* ObjectDestructuring */] &&
                        !node.typeAnnotation &&
                        !isForOfStatementContext(node)) {
                        report(node);
                    }
                },
                'TSIndexSignature, TSPropertySignature'(node) {
                    if (options["propertyDeclaration" /* PropertyDeclaration */] && !node.typeAnnotation) {
                        report(node, node.type === experimental_utils_1.AST_NODE_TYPES.TSPropertySignature
                            ? getNodeName(node.key)
                            : undefined);
                    }
                },
                VariableDeclarator(node) {
                    if (!options["variableDeclaration" /* VariableDeclaration */] ||
                        node.id.typeAnnotation ||
                        (node.id.type === experimental_utils_1.AST_NODE_TYPES.ArrayPattern &&
                            !options["arrayDestructuring" /* ArrayDestructuring */]) ||
                        (node.id.type === experimental_utils_1.AST_NODE_TYPES.ObjectPattern &&
                            !options["objectDestructuring" /* ObjectDestructuring */]) ||
                        (node.init && isVariableDeclarationIgnoreFunction(node.init))) {
                        return;
                    }
                    let current = node.parent;
                    while (current) {
                        switch (current.type) {
                            case experimental_utils_1.AST_NODE_TYPES.VariableDeclaration:
                                // Keep looking upwards
                                current = current.parent;
                                break;
                            case experimental_utils_1.AST_NODE_TYPES.ForOfStatement:
                            case experimental_utils_1.AST_NODE_TYPES.ForInStatement:
                                // Stop traversing and don't report an error
                                return;
                            default:
                                // Stop traversing
                                current = undefined;
                                break;
                        }
                    }
                    report(node, getNodeName(node.id));
                },
            };
        },
    });
},
/* @typescript-eslint/eslint-plugin/dist/rules/unbound-method.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const tsutils = __importStar(require(627) /* tsutils */);
    const ts = __importStar(require(44) /* typescript */);
    const util = __importStar(require(622) /* ../util */);
    /**
     * The following is a list of exceptions to the rule
     * Generated via the following script.
     * This is statically defined to save making purposely invalid calls every lint run
     * ```
    SUPPORTED_GLOBALS.flatMap(namespace => {
      const object = window[namespace];
        return Object.getOwnPropertyNames(object)
          .filter(
            name =>
              !name.startsWith('_') &&
              typeof object[name] === 'function',
          )
          .map(name => {
            try {
              const x = object[name];
              x();
            } catch (e) {
              if (e.message.includes("called on non-object")) {
                return `${namespace}.${name}`;
              }
            }
          });
    }).filter(Boolean);
       * ```
     */
    const nativelyNotBoundMembers = new Set([
        'Promise.all',
        'Promise.race',
        'Promise.resolve',
        'Promise.reject',
        'Promise.allSettled',
        'Object.defineProperties',
        'Object.defineProperty',
        'Reflect.defineProperty',
        'Reflect.deleteProperty',
        'Reflect.get',
        'Reflect.getOwnPropertyDescriptor',
        'Reflect.getPrototypeOf',
        'Reflect.has',
        'Reflect.isExtensible',
        'Reflect.ownKeys',
        'Reflect.preventExtensions',
        'Reflect.set',
        'Reflect.setPrototypeOf',
    ]);
    const SUPPORTED_GLOBALS = [
        'Number',
        'Object',
        'String',
        'RegExp',
        'Symbol',
        'Array',
        'Proxy',
        'Date',
        'Infinity',
        'Atomics',
        'Reflect',
        'console',
        'Math',
        'JSON',
        'Intl',
    ];
    const nativelyBoundMembers = SUPPORTED_GLOBALS.map(namespace => {
        const object = global[namespace];
        return Object.getOwnPropertyNames(object)
            .filter(name => !name.startsWith('_') &&
            typeof object[name] === 'function')
            .map(name => `${namespace}.${name}`);
    })
        .reduce((arr, names) => arr.concat(names), [])
        .filter(name => !nativelyNotBoundMembers.has(name));
    const isMemberNotImported = (symbol, currentSourceFile) => {
        const { valueDeclaration } = symbol;
        if (!valueDeclaration) {
            // working around https://github.com/microsoft/TypeScript/issues/31294
            return false;
        }
        return (!!currentSourceFile &&
            currentSourceFile !== valueDeclaration.getSourceFile());
    };
    const getNodeName = (node) => node.type === experimental_utils_1.AST_NODE_TYPES.Identifier ? node.name : null;
    const getMemberFullName = (node) => `${getNodeName(node.object)}.${getNodeName(node.property)}`;
    exports.default = util.createRule({
        name: 'unbound-method',
        meta: {
            docs: {
                category: 'Best Practices',
                description: 'Enforces unbound methods are called with their expected scope',
                recommended: 'error',
                requiresTypeChecking: true,
            },
            messages: {
                unbound: 'Avoid referencing unbound methods which may cause unintentional scoping of `this`.',
            },
            schema: [
                {
                    type: 'object',
                    properties: {
                        ignoreStatic: {
                            type: 'boolean',
                        },
                    },
                    additionalProperties: false,
                },
            ],
            type: 'problem',
        },
        defaultOptions: [
            {
                ignoreStatic: false,
            },
        ],
        create(context, [{ ignoreStatic }]) {
            const parserServices = util.getParserServices(context);
            const checker = parserServices.program.getTypeChecker();
            const currentSourceFile = parserServices.program.getSourceFile(context.getFilename());
            return {
                'MemberExpression, OptionalMemberExpression'(node) {
                    if (isSafeUse(node)) {
                        return;
                    }
                    const objectSymbol = checker.getSymbolAtLocation(parserServices.esTreeNodeToTSNodeMap.get(node.object));
                    if (objectSymbol &&
                        nativelyBoundMembers.includes(getMemberFullName(node)) &&
                        isMemberNotImported(objectSymbol, currentSourceFile)) {
                        return;
                    }
                    const originalNode = parserServices.esTreeNodeToTSNodeMap.get(node);
                    const symbol = checker.getSymbolAtLocation(originalNode);
                    if (symbol && isDangerousMethod(symbol, ignoreStatic)) {
                        context.report({
                            messageId: 'unbound',
                            node,
                        });
                    }
                },
            };
        },
    });
    function isDangerousMethod(symbol, ignoreStatic) {
        const { valueDeclaration } = symbol;
        if (!valueDeclaration) {
            // working around https://github.com/microsoft/TypeScript/issues/31294
            return false;
        }
        switch (valueDeclaration.kind) {
            case ts.SyntaxKind.MethodDeclaration:
            case ts.SyntaxKind.MethodSignature:
                return !(ignoreStatic &&
                    tsutils.hasModifier(valueDeclaration.modifiers, ts.SyntaxKind.StaticKeyword));
        }
        return false;
    }
    function isSafeUse(node) {
        const parent = node.parent;
        switch (parent === null || parent === void 0 ? void 0 : parent.type) {
            case experimental_utils_1.AST_NODE_TYPES.IfStatement:
            case experimental_utils_1.AST_NODE_TYPES.ForStatement:
            case experimental_utils_1.AST_NODE_TYPES.MemberExpression:
            case experimental_utils_1.AST_NODE_TYPES.OptionalMemberExpression:
            case experimental_utils_1.AST_NODE_TYPES.SwitchStatement:
            case experimental_utils_1.AST_NODE_TYPES.UpdateExpression:
            case experimental_utils_1.AST_NODE_TYPES.WhileStatement:
                return true;
            case experimental_utils_1.AST_NODE_TYPES.CallExpression:
            case experimental_utils_1.AST_NODE_TYPES.OptionalCallExpression:
                return parent.callee === node;
            case experimental_utils_1.AST_NODE_TYPES.ConditionalExpression:
                return parent.test === node;
            case experimental_utils_1.AST_NODE_TYPES.TaggedTemplateExpression:
                return parent.tag === node;
            case experimental_utils_1.AST_NODE_TYPES.UnaryExpression:
                return parent.operator === 'typeof';
            case experimental_utils_1.AST_NODE_TYPES.BinaryExpression:
                return ['instanceof', '==', '!=', '===', '!=='].includes(parent.operator);
            case experimental_utils_1.AST_NODE_TYPES.TSNonNullExpression:
            case experimental_utils_1.AST_NODE_TYPES.TSAsExpression:
            case experimental_utils_1.AST_NODE_TYPES.TSTypeAssertion:
                return isSafeUse(parent);
            case experimental_utils_1.AST_NODE_TYPES.LogicalExpression:
                if (parent.operator === '&&' && parent.left === node) {
                    // this is safe, as && will return the left if and only if it's falsy
                    return true;
                }
                // in all other cases, it's likely the logical expression will return the method ref
                // so make sure the parent is a safe usage
                return isSafeUse(parent);
        }
        return false;
    }
},
/* @typescript-eslint/eslint-plugin/dist/rules/unified-signatures.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const util = __importStar(require(622) /* ../util */);
    exports.default = util.createRule({
        name: 'unified-signatures',
        meta: {
            docs: {
                description: 'Warns for any two overloads that could be unified into one by using a union or an optional/rest parameter',
                category: 'Variables',
                // too opinionated to be recommended
                recommended: false,
            },
            type: 'suggestion',
            messages: {
                omittingRestParameter: '{{failureStringStart}} with a rest parameter.',
                omittingSingleParameter: '{{failureStringStart}} with an optional parameter.',
                singleParameterDifference: '{{failureStringStart}} taking `{{type1}} | {{type2}}`.',
            },
            schema: [],
        },
        defaultOptions: [],
        create(context) {
            const sourceCode = context.getSourceCode();
            //----------------------------------------------------------------------
            // Helpers
            //----------------------------------------------------------------------
            function failureStringStart(otherLine) {
                // For only 2 overloads we don't need to specify which is the other one.
                const overloads = otherLine === undefined
                    ? 'These overloads'
                    : `This overload and the one on line ${otherLine}`;
                return `${overloads} can be combined into one signature`;
            }
            function addFailures(failures) {
                for (const failure of failures) {
                    const { unify, only2 } = failure;
                    switch (unify.kind) {
                        case 'single-parameter-difference': {
                            const { p0, p1 } = unify;
                            const lineOfOtherOverload = only2 ? undefined : p0.loc.start.line;
                            const typeAnnotation0 = isTSParameterProperty(p0)
                                ? p0.parameter.typeAnnotation
                                : p0.typeAnnotation;
                            const typeAnnotation1 = isTSParameterProperty(p1)
                                ? p1.parameter.typeAnnotation
                                : p1.typeAnnotation;
                            context.report({
                                loc: p1.loc,
                                messageId: 'singleParameterDifference',
                                data: {
                                    failureStringStart: failureStringStart(lineOfOtherOverload),
                                    type1: sourceCode.getText(typeAnnotation0 === null || typeAnnotation0 === void 0 ? void 0 : typeAnnotation0.typeAnnotation),
                                    type2: sourceCode.getText(typeAnnotation1 === null || typeAnnotation1 === void 0 ? void 0 : typeAnnotation1.typeAnnotation),
                                },
                                node: p1,
                            });
                            break;
                        }
                        case 'extra-parameter': {
                            const { extraParameter, otherSignature } = unify;
                            const lineOfOtherOverload = only2
                                ? undefined
                                : otherSignature.loc.start.line;
                            context.report({
                                loc: extraParameter.loc,
                                messageId: extraParameter.type === experimental_utils_1.AST_NODE_TYPES.RestElement
                                    ? 'omittingRestParameter'
                                    : 'omittingSingleParameter',
                                data: {
                                    failureStringStart: failureStringStart(lineOfOtherOverload),
                                },
                                node: extraParameter,
                            });
                        }
                    }
                }
            }
            function checkOverloads(signatures, typeParameters) {
                const result = [];
                const isTypeParameter = getIsTypeParameter(typeParameters);
                for (const overloads of signatures) {
                    if (overloads.length === 2) {
                        const signature0 = overloads[0].value || overloads[0];
                        const signature1 = overloads[1].value || overloads[1];
                        const unify = compareSignatures(signature0, signature1, isTypeParameter);
                        if (unify !== undefined) {
                            result.push({ unify, only2: true });
                        }
                    }
                    else {
                        forEachPair(overloads, (a, b) => {
                            const signature0 = a.value || a;
                            const signature1 = b.value || b;
                            const unify = compareSignatures(signature0, signature1, isTypeParameter);
                            if (unify !== undefined) {
                                result.push({ unify, only2: false });
                            }
                        });
                    }
                }
                return result;
            }
            function compareSignatures(a, b, isTypeParameter) {
                if (!signaturesCanBeUnified(a, b, isTypeParameter)) {
                    return undefined;
                }
                return a.params.length === b.params.length
                    ? signaturesDifferBySingleParameter(a.params, b.params)
                    : signaturesDifferByOptionalOrRestParameter(a, b);
            }
            function signaturesCanBeUnified(a, b, isTypeParameter) {
                // Must return the same type.
                const aTypeParams = a.typeParameters !== undefined ? a.typeParameters.params : undefined;
                const bTypeParams = b.typeParameters !== undefined ? b.typeParameters.params : undefined;
                return (typesAreEqual(a.returnType, b.returnType) &&
                    // Must take the same type parameters.
                    // If one uses a type parameter (from outside) and the other doesn't, they shouldn't be joined.
                    util.arraysAreEqual(aTypeParams, bTypeParams, typeParametersAreEqual) &&
                    signatureUsesTypeParameter(a, isTypeParameter) ===
                        signatureUsesTypeParameter(b, isTypeParameter));
            }
            /** Detect `a(x: number, y: number, z: number)` and `a(x: number, y: string, z: number)`. */
            function signaturesDifferBySingleParameter(types1, types2) {
                const index = getIndexOfFirstDifference(types1, types2, parametersAreEqual);
                if (index === undefined) {
                    return undefined;
                }
                // If remaining arrays are equal, the signatures differ by just one parameter type
                if (!util.arraysAreEqual(types1.slice(index + 1), types2.slice(index + 1), parametersAreEqual)) {
                    return undefined;
                }
                const a = types1[index];
                const b = types2[index];
                // Can unify `a?: string` and `b?: number`. Can't unify `...args: string[]` and `...args: number[]`.
                // See https://github.com/Microsoft/TypeScript/issues/5077
                return parametersHaveEqualSigils(a, b) &&
                    a.type !== experimental_utils_1.AST_NODE_TYPES.RestElement
                    ? { kind: 'single-parameter-difference', p0: a, p1: b }
                    : undefined;
            }
            /**
             * Detect `a(): void` and `a(x: number): void`.
             * Returns the parameter declaration (`x: number` in this example) that should be optional/rest, and overload it's a part of.
             */
            function signaturesDifferByOptionalOrRestParameter(a, b) {
                const sig1 = a.params;
                const sig2 = b.params;
                const minLength = Math.min(sig1.length, sig2.length);
                const longer = sig1.length < sig2.length ? sig2 : sig1;
                const shorter = sig1.length < sig2.length ? sig1 : sig2;
                const shorterSig = sig1.length < sig2.length ? a : b;
                // If one is has 2+ parameters more than the other, they must all be optional/rest.
                // Differ by optional parameters: f() and f(x), f() and f(x, ?y, ...z)
                // Not allowed: f() and f(x, y)
                for (let i = minLength + 1; i < longer.length; i++) {
                    if (!parameterMayBeMissing(longer[i])) {
                        return undefined;
                    }
                }
                for (let i = 0; i < minLength; i++) {
                    const sig1i = sig1[i];
                    const sig2i = sig2[i];
                    const typeAnnotation1 = isTSParameterProperty(sig1i)
                        ? sig1i.parameter.typeAnnotation
                        : sig1i.typeAnnotation;
                    const typeAnnotation2 = isTSParameterProperty(sig2i)
                        ? sig2i.parameter.typeAnnotation
                        : sig2i.typeAnnotation;
                    if (!typesAreEqual(typeAnnotation1, typeAnnotation2)) {
                        return undefined;
                    }
                }
                if (minLength > 0 &&
                    shorter[minLength - 1].type === experimental_utils_1.AST_NODE_TYPES.RestElement) {
                    return undefined;
                }
                return {
                    extraParameter: longer[longer.length - 1],
                    kind: 'extra-parameter',
                    otherSignature: shorterSig,
                };
            }
            /** Given type parameters, returns a function to test whether a type is one of those parameters. */
            function getIsTypeParameter(typeParameters) {
                if (typeParameters === undefined) {
                    return (() => false);
                }
                const set = new Set();
                for (const t of typeParameters.params) {
                    set.add(t.name.name);
                }
                return (typeName => set.has(typeName));
            }
            /** True if any of the outer type parameters are used in a signature. */
            function signatureUsesTypeParameter(sig, isTypeParameter) {
                return sig.params.some((p) => typeContainsTypeParameter(isTSParameterProperty(p)
                    ? p.parameter.typeAnnotation
                    : p.typeAnnotation));
                function typeContainsTypeParameter(type) {
                    if (!type) {
                        return false;
                    }
                    if (type.type === experimental_utils_1.AST_NODE_TYPES.TSTypeReference) {
                        const typeName = type.typeName;
                        if (isIdentifier(typeName) && isTypeParameter(typeName.name)) {
                            return true;
                        }
                    }
                    return typeContainsTypeParameter(type.typeAnnotation ||
                        type.elementType);
                }
            }
            function isTSParameterProperty(node) {
                return (node.type ===
                    experimental_utils_1.AST_NODE_TYPES.TSParameterProperty);
            }
            function parametersAreEqual(a, b) {
                const typeAnnotationA = isTSParameterProperty(a)
                    ? a.parameter.typeAnnotation
                    : a.typeAnnotation;
                const typeAnnotationB = isTSParameterProperty(b)
                    ? b.parameter.typeAnnotation
                    : b.typeAnnotation;
                return (parametersHaveEqualSigils(a, b) &&
                    typesAreEqual(typeAnnotationA, typeAnnotationB));
            }
            /** True for optional/rest parameters. */
            function parameterMayBeMissing(p) {
                const optional = isTSParameterProperty(p)
                    ? p.parameter.optional
                    : p.optional;
                return p.type === experimental_utils_1.AST_NODE_TYPES.RestElement || optional;
            }
            /** False if one is optional and the other isn't, or one is a rest parameter and the other isn't. */
            function parametersHaveEqualSigils(a, b) {
                const optionalA = isTSParameterProperty(a)
                    ? a.parameter.optional
                    : a.optional;
                const optionalB = isTSParameterProperty(b)
                    ? b.parameter.optional
                    : b.optional;
                return ((a.type === experimental_utils_1.AST_NODE_TYPES.RestElement) ===
                    (b.type === experimental_utils_1.AST_NODE_TYPES.RestElement) &&
                    (optionalA !== undefined) === (optionalB !== undefined));
            }
            function typeParametersAreEqual(a, b) {
                return (a.name.name === b.name.name &&
                    constraintsAreEqual(a.constraint, b.constraint));
            }
            function typesAreEqual(a, b) {
                return (a === b ||
                    (a !== undefined &&
                        b !== undefined &&
                        sourceCode.getText(a.typeAnnotation) ===
                            sourceCode.getText(b.typeAnnotation)));
            }
            function constraintsAreEqual(a, b) {
                return (a === b || (a !== undefined && b !== undefined && a.type === b.type));
            }
            /* Returns the first index where `a` and `b` differ. */
            function getIndexOfFirstDifference(a, b, equal) {
                for (let i = 0; i < a.length && i < b.length; i++) {
                    if (!equal(a[i], b[i])) {
                        return i;
                    }
                }
                return undefined;
            }
            /** Calls `action` for every pair of values in `values`. */
            function forEachPair(values, action) {
                for (let i = 0; i < values.length; i++) {
                    for (let j = i + 1; j < values.length; j++) {
                        action(values[i], values[j]);
                    }
                }
            }
            const scopes = [];
            let currentScope = {
                overloads: new Map(),
            };
            function createScope(parent, typeParameters) {
                currentScope && scopes.push(currentScope);
                currentScope = {
                    overloads: new Map(),
                    parent,
                    typeParameters,
                };
            }
            function checkScope() {
                const failures = checkOverloads(Array.from(currentScope.overloads.values()), currentScope.typeParameters);
                addFailures(failures);
                currentScope = scopes.pop();
            }
            function addOverload(signature, key, containingNode) {
                key = key !== null && key !== void 0 ? key : getOverloadKey(signature);
                if (currentScope &&
                    (containingNode || signature).parent === currentScope.parent) {
                    const overloads = currentScope.overloads.get(key);
                    if (overloads !== undefined) {
                        overloads.push(signature);
                    }
                    else {
                        currentScope.overloads.set(key, [signature]);
                    }
                }
            }
            //----------------------------------------------------------------------
            // Public
            //----------------------------------------------------------------------
            return {
                Program: createScope,
                TSModuleBlock: createScope,
                TSInterfaceDeclaration(node) {
                    createScope(node.body, node.typeParameters);
                },
                ClassDeclaration(node) {
                    createScope(node.body, node.typeParameters);
                },
                TSTypeLiteral: createScope,
                // collect overloads
                TSDeclareFunction(node) {
                    var _a, _b;
                    const exportingNode = getExportingNode(node);
                    addOverload(node, (_b = (_a = node.id) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : exportingNode === null || exportingNode === void 0 ? void 0 : exportingNode.type, exportingNode);
                },
                TSCallSignatureDeclaration: addOverload,
                TSConstructSignatureDeclaration: addOverload,
                TSMethodSignature: addOverload,
                TSAbstractMethodDefinition(node) {
                    if (!node.value.body) {
                        addOverload(node);
                    }
                },
                MethodDefinition(node) {
                    if (!node.value.body) {
                        addOverload(node);
                    }
                },
                // validate scopes
                'Program:exit': checkScope,
                'TSModuleBlock:exit': checkScope,
                'TSInterfaceDeclaration:exit': checkScope,
                'ClassDeclaration:exit': checkScope,
                'TSTypeLiteral:exit': checkScope,
            };
        },
    });
    function getExportingNode(node) {
        return node.parent &&
            (node.parent.type === experimental_utils_1.AST_NODE_TYPES.ExportNamedDeclaration ||
                node.parent.type === experimental_utils_1.AST_NODE_TYPES.ExportDefaultDeclaration)
            ? node.parent
            : undefined;
    }
    function getOverloadKey(node) {
        const info = getOverloadInfo(node);
        return ((node.computed ? '0' : '1') +
            (node.static ? '0' : '1') +
            info);
    }
    function getOverloadInfo(node) {
        switch (node.type) {
            case experimental_utils_1.AST_NODE_TYPES.TSConstructSignatureDeclaration:
                return 'constructor';
            case experimental_utils_1.AST_NODE_TYPES.TSCallSignatureDeclaration:
                return '()';
            default: {
                const { key } = node;
                return isIdentifier(key) ? key.name : key.raw;
            }
        }
    }
    function isIdentifier(node) {
        return node.type === experimental_utils_1.AST_NODE_TYPES.Identifier;
    }
},
/* @typescript-eslint/eslint-plugin/dist/configs/all.json */ function _(require, module, exports) {
    module.exports = {
        "extends": "./configs/base.json",
        "rules": {
            "@typescript-eslint/adjacent-overload-signatures": "error",
            "@typescript-eslint/array-type": "error",
            "@typescript-eslint/await-thenable": "error",
            "@typescript-eslint/ban-ts-comment": "error",
            "@typescript-eslint/ban-types": "error",
            "brace-style": "off",
            "@typescript-eslint/brace-style": "error",
            "comma-spacing": "off",
            "@typescript-eslint/comma-spacing": "error",
            "@typescript-eslint/consistent-type-assertions": "error",
            "@typescript-eslint/consistent-type-definitions": "error",
            "default-param-last": "off",
            "@typescript-eslint/default-param-last": "error",
            "@typescript-eslint/explicit-function-return-type": "error",
            "@typescript-eslint/explicit-member-accessibility": "error",
            "@typescript-eslint/explicit-module-boundary-types": "error",
            "func-call-spacing": "off",
            "@typescript-eslint/func-call-spacing": "error",
            "indent": "off",
            "@typescript-eslint/indent": "error",
            "@typescript-eslint/member-delimiter-style": "error",
            "@typescript-eslint/member-ordering": "error",
            "@typescript-eslint/naming-convention": "error",
            "no-array-constructor": "off",
            "@typescript-eslint/no-array-constructor": "error",
            "@typescript-eslint/no-base-to-string": "error",
            "no-dupe-class-members": "off",
            "@typescript-eslint/no-dupe-class-members": "error",
            "@typescript-eslint/no-dynamic-delete": "error",
            "no-empty-function": "off",
            "@typescript-eslint/no-empty-function": "error",
            "@typescript-eslint/no-empty-interface": "error",
            "@typescript-eslint/no-explicit-any": "error",
            "@typescript-eslint/no-extra-non-null-assertion": "error",
            "no-extra-parens": "off",
            "@typescript-eslint/no-extra-parens": "error",
            "no-extra-semi": "off",
            "@typescript-eslint/no-extra-semi": "error",
            "@typescript-eslint/no-extraneous-class": "error",
            "@typescript-eslint/no-floating-promises": "error",
            "@typescript-eslint/no-for-in-array": "error",
            "@typescript-eslint/no-implied-eval": "error",
            "@typescript-eslint/no-inferrable-types": "error",
            "no-magic-numbers": "off",
            "@typescript-eslint/no-magic-numbers": "error",
            "@typescript-eslint/no-misused-new": "error",
            "@typescript-eslint/no-misused-promises": "error",
            "@typescript-eslint/no-namespace": "error",
            "@typescript-eslint/no-non-null-asserted-optional-chain": "error",
            "@typescript-eslint/no-non-null-assertion": "error",
            "@typescript-eslint/no-parameter-properties": "error",
            "@typescript-eslint/no-require-imports": "error",
            "@typescript-eslint/no-this-alias": "error",
            "@typescript-eslint/no-throw-literal": "error",
            "@typescript-eslint/no-type-alias": "error",
            "@typescript-eslint/no-unnecessary-boolean-literal-compare": "error",
            "@typescript-eslint/no-unnecessary-condition": "error",
            "@typescript-eslint/no-unnecessary-qualifier": "error",
            "@typescript-eslint/no-unnecessary-type-arguments": "error",
            "@typescript-eslint/no-unnecessary-type-assertion": "error",
            "@typescript-eslint/no-unsafe-call": "error",
            "@typescript-eslint/no-unsafe-member-access": "error",
            "@typescript-eslint/no-unsafe-return": "error",
            "no-unused-expressions": "off",
            "@typescript-eslint/no-unused-expressions": "error",
            "no-unused-vars": "off",
            "@typescript-eslint/no-unused-vars": "error",
            "@typescript-eslint/no-unused-vars-experimental": "error",
            "no-use-before-define": "off",
            "@typescript-eslint/no-use-before-define": "error",
            "no-useless-constructor": "off",
            "@typescript-eslint/no-useless-constructor": "error",
            "@typescript-eslint/no-var-requires": "error",
            "@typescript-eslint/prefer-as-const": "error",
            "@typescript-eslint/prefer-for-of": "error",
            "@typescript-eslint/prefer-function-type": "error",
            "@typescript-eslint/prefer-includes": "error",
            "@typescript-eslint/prefer-namespace-keyword": "error",
            "@typescript-eslint/prefer-nullish-coalescing": "error",
            "@typescript-eslint/prefer-optional-chain": "error",
            "@typescript-eslint/prefer-readonly": "error",
            "@typescript-eslint/prefer-readonly-parameter-types": "error",
            "@typescript-eslint/prefer-regexp-exec": "error",
            "@typescript-eslint/prefer-string-starts-ends-with": "error",
            "@typescript-eslint/promise-function-async": "error",
            "quotes": "off",
            "@typescript-eslint/quotes": "error",
            "@typescript-eslint/require-array-sort-compare": "error",
            "require-await": "off",
            "@typescript-eslint/require-await": "error",
            "@typescript-eslint/restrict-plus-operands": "error",
            "@typescript-eslint/restrict-template-expressions": "error",
            "no-return-await": "off",
            "@typescript-eslint/return-await": "error",
            "semi": "off",
            "@typescript-eslint/semi": "error",
            "space-before-function-paren": "off",
            "@typescript-eslint/space-before-function-paren": "error",
            "@typescript-eslint/strict-boolean-expressions": "error",
            "@typescript-eslint/switch-exhaustiveness-check": "error",
            "@typescript-eslint/triple-slash-reference": "error",
            "@typescript-eslint/type-annotation-spacing": "error",
            "@typescript-eslint/typedef": "error",
            "@typescript-eslint/unbound-method": "error",
            "@typescript-eslint/unified-signatures": "error"
        }
    };
},
/* @typescript-eslint/eslint-plugin/dist/configs/base.json */ function _(require, module, exports) {
    module.exports = {
        "parser": "@typescript-eslint/parser",
        "parserOptions": { "sourceType": "module" },
        "plugins": ["@typescript-eslint"]
    };
},
/* @typescript-eslint/eslint-plugin/dist/configs/recommended.json */ function _(require, module, exports) {
    module.exports = {
        "extends": "./configs/base.json",
        "rules": {
            "@typescript-eslint/adjacent-overload-signatures": "error",
            "@typescript-eslint/ban-ts-ignore": "error",
            "@typescript-eslint/ban-types": "error",
            "camelcase": "off",
            "@typescript-eslint/camelcase": "error",
            "@typescript-eslint/class-name-casing": "error",
            "@typescript-eslint/consistent-type-assertions": "error",
            "@typescript-eslint/explicit-function-return-type": "warn",
            "@typescript-eslint/interface-name-prefix": "error",
            "@typescript-eslint/member-delimiter-style": "error",
            "no-array-constructor": "off",
            "@typescript-eslint/no-array-constructor": "error",
            "no-empty-function": "off",
            "@typescript-eslint/no-empty-function": "error",
            "@typescript-eslint/no-empty-interface": "error",
            "@typescript-eslint/no-explicit-any": "warn",
            "@typescript-eslint/no-inferrable-types": "error",
            "@typescript-eslint/no-misused-new": "error",
            "@typescript-eslint/no-namespace": "error",
            "@typescript-eslint/no-non-null-assertion": "warn",
            "@typescript-eslint/no-this-alias": "error",
            "no-unused-vars": "off",
            "@typescript-eslint/no-unused-vars": "warn",
            "no-use-before-define": "off",
            "@typescript-eslint/no-use-before-define": "error",
            "@typescript-eslint/no-var-requires": "error",
            "@typescript-eslint/prefer-namespace-keyword": "error",
            "@typescript-eslint/triple-slash-reference": "error",
            "@typescript-eslint/type-annotation-spacing": "error",
            "no-var": "error",
            "prefer-const": "error",
            "prefer-rest-params": "error",
            "prefer-spread": "error"
        }
    };
},
/* @typescript-eslint/eslint-plugin/dist/configs/recommended-requiring-type-checking.json */ function _(require, module, exports) {
    module.exports = {
        "extends": "./configs/base.json",
        "rules": {
            "@typescript-eslint/await-thenable": "error",
            "@typescript-eslint/no-for-in-array": "error",
            "@typescript-eslint/no-misused-promises": "error",
            "@typescript-eslint/no-unnecessary-type-assertion": "error",
            "@typescript-eslint/prefer-includes": "error",
            "@typescript-eslint/prefer-regexp-exec": "error",
            "@typescript-eslint/prefer-string-starts-ends-with": "error",
            "require-await": "off",
            "@typescript-eslint/require-await": "error",
            "@typescript-eslint/unbound-method": "error",
            "no-var": "error",
            "prefer-const": "error",
            "prefer-rest-params": "error",
            "prefer-spread": "error"
        }
    };
},
/* @typescript-eslint/eslint-plugin/dist/configs/eslint-recommended.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * This is a compatibility ruleset that disables rules from eslint:recommended
     * which are already handled by TypeScript.
     */
    exports.default = {
        overrides: [
            {
                files: ['*.ts', '*.tsx'],
                rules: {
                    // Checked by Typescript - ts(2378)
                    'getter-return': 'off',
                    // Checked by Typescript - ts(2300)
                    'no-dupe-args': 'off',
                    // Checked by Typescript - ts(1117)
                    'no-dupe-keys': 'off',
                    // Checked by Typescript - ts(7027)
                    'no-unreachable': 'off',
                    // Checked by Typescript - ts(2367)
                    'valid-typeof': 'off',
                    // Checked by Typescript - ts(2588)
                    'no-const-assign': 'off',
                    // Checked by Typescript - ts(2588)
                    'no-new-symbol': 'off',
                    // Checked by Typescript - ts(2376)
                    'no-this-before-super': 'off',
                    // This is checked by Typescript using the option `strictNullChecks`.
                    'no-undef': 'off',
                    // This is already checked by Typescript.
                    'no-dupe-class-members': 'off',
                    // This is already checked by Typescript.
                    'no-redeclare': 'off',
                },
            },
        ],
    };
},
/* @typescript-eslint/parser/dist/parser.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const typescript_estree_1 = require(748) /* @typescript-eslint/typescript-estree */;
    const analyze_scope_1 = require(819) /* ./analyze-scope */;
    // note - cannot migrate this to an import statement because it will make TSC copy the package.json to the dist folder
    const packageJSON = require(822) /* ../package.json */;
    function validateBoolean(value, fallback = false) {
        if (typeof value !== 'boolean') {
            return fallback;
        }
        return value;
    }
    //------------------------------------------------------------------------------
    // Public
    //------------------------------------------------------------------------------
    exports.version = packageJSON.version;
    exports.Syntax = Object.freeze(typescript_estree_1.AST_NODE_TYPES);
    function parse(code, options) {
        return parseForESLint(code, options).ast;
    }
    exports.parse = parse;
    function parseForESLint(code, options) {
        if (!options || typeof options !== 'object') {
            options = {};
        }
        // https://eslint.org/docs/user-guide/configuring#specifying-parser-options
        // if sourceType is not provided by default eslint expect that it will be set to "script"
        if (options.sourceType !== 'module' && options.sourceType !== 'script') {
            options.sourceType = 'script';
        }
        if (typeof options.ecmaFeatures !== 'object') {
            options.ecmaFeatures = {};
        }
        const parserOptions = {};
        Object.assign(parserOptions, options, {
            useJSXTextNode: validateBoolean(options.useJSXTextNode, true),
            jsx: validateBoolean(options.ecmaFeatures.jsx),
        });
        if (typeof options.filePath === 'string') {
            const tsx = options.filePath.endsWith('.tsx');
            if (tsx || options.filePath.endsWith('.ts')) {
                parserOptions.jsx = tsx;
            }
        }
        /**
         * Allow the user to suppress the warning from typescript-estree if they are using an unsupported
         * version of TypeScript
         */
        const warnOnUnsupportedTypeScriptVersion = validateBoolean(options.warnOnUnsupportedTypeScriptVersion, true);
        if (!warnOnUnsupportedTypeScriptVersion) {
            parserOptions.loggerFn = false;
        }
        const { ast, services } = typescript_estree_1.parseAndGenerateServices(code, parserOptions);
        ast.sourceType = options.sourceType;
        typescript_estree_1.simpleTraverse(ast, {
            enter(node) {
                switch (node.type) {
                    // Function#body cannot be null in ESTree spec.
                    case typescript_estree_1.AST_NODE_TYPES.FunctionExpression:
                        if (!node.body) {
                            // eslint-disable-next-line @typescript-eslint/no-explicit-any
                            node.type = `TSEmptyBody${node.type}`;
                        }
                        break;
                    // no default
                }
            },
        });
        const scopeManager = analyze_scope_1.analyzeScope(ast, options);
        return { ast, services, scopeManager, visitorKeys: typescript_estree_1.visitorKeys };
    }
    exports.parseForESLint = parseForESLint;
    var typescript_estree_2 = require(748) /* @typescript-eslint/typescript-estree */;
    exports.clearCaches = typescript_estree_2.clearCaches;
},
/* @typescript-eslint/typescript-estree/dist/parser.js */ function _(require, module, exports) {
    function __export(m) {
        for (var p in m)
            if (!exports.hasOwnProperty(p))
                exports[p] = m[p];
    }
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const debug_1 = __importDefault(require(120) /* debug */);
    const glob_1 = require(581) /* glob */;
    const is_glob_1 = __importDefault(require(568) /* is-glob */);
    const semver_1 = __importDefault(require(749) /* semver */);
    const ts = __importStar(require(44) /* typescript */);
    const ast_converter_1 = require(790) /* ./ast-converter */;
    const convert_1 = require(791) /* ./convert */;
    const createDefaultProgram_1 = require(811) /* ./create-program/createDefaultProgram */;
    const createIsolatedProgram_1 = require(813) /* ./create-program/createIsolatedProgram */;
    const createProjectProgram_1 = require(814) /* ./create-program/createProjectProgram */;
    const createSourceFile_1 = require(816) /* ./create-program/createSourceFile */;
    const semantic_or_syntactic_errors_1 = require(817) /* ./semantic-or-syntactic-errors */;
    const shared_1 = require(812) /* ./create-program/shared */;
    /**
     * This needs to be kept in sync with the top-level README.md in the
     * typescript-eslint monorepo
     */
    const SUPPORTED_TYPESCRIPT_VERSIONS = '>=3.2.1 <3.8.0';
    /*
     * The semver package will ignore prerelease ranges, and we don't want to explicitly document every one
     * List them all separately here, so we can automatically create the full string
     */
    const SUPPORTED_PRERELEASE_RANGES = ['>3.7.0-dev.0', '3.7.1-rc'];
    const ACTIVE_TYPESCRIPT_VERSION = ts.version;
    const isRunningSupportedTypeScriptVersion = semver_1.default.satisfies(ACTIVE_TYPESCRIPT_VERSION, [SUPPORTED_TYPESCRIPT_VERSIONS]
        .concat(SUPPORTED_PRERELEASE_RANGES)
        .join(' || '));
    let extra;
    let warnedAboutTSVersion = false;
    function enforceString(code) {
        /**
         * Ensure the source code is a string
         */
        if (typeof code !== 'string') {
            return String(code);
        }
        return code;
    }
    /**
     * @param code The code of the file being linted
     * @param shouldProvideParserServices True if the program should be attempted to be calculated from provided tsconfig files
     * @param shouldCreateDefaultProgram True if the program should be created from compiler host
     * @returns Returns a source file and program corresponding to the linted code
     */
    function getProgramAndAST(code, shouldProvideParserServices, shouldCreateDefaultProgram) {
        return ((shouldProvideParserServices &&
            createProjectProgram_1.createProjectProgram(code, shouldCreateDefaultProgram, extra)) ||
            (shouldProvideParserServices &&
                shouldCreateDefaultProgram &&
                createDefaultProgram_1.createDefaultProgram(code, extra)) ||
            createIsolatedProgram_1.createIsolatedProgram(code, extra));
    }
    /**
     * Compute the filename based on the parser options.
     *
     * Even if jsx option is set in typescript compiler, filename still has to
     * contain .tsx file extension.
     *
     * @param options Parser options
     */
    function getFileName({ jsx } = {}) {
        return jsx ? 'estree.tsx' : 'estree.ts';
    }
    /**
     * Resets the extra config object
     */
    function resetExtra() {
        extra = {
            code: '',
            comment: false,
            comments: [],
            createDefaultProgram: false,
            debugLevel: new Set(),
            errorOnTypeScriptSyntacticAndSemanticIssues: false,
            errorOnUnknownASTType: false,
            extraFileExtensions: [],
            filePath: getFileName(),
            jsx: false,
            loc: false,
            log: console.log,
            preserveNodeMaps: undefined,
            projects: [],
            range: false,
            strict: false,
            tokens: null,
            tsconfigRootDir: process.cwd(),
            useJSXTextNode: false,
        };
    }
    function applyParserOptionsToExtra(options) {
        /**
         * Configure Debug logging
         */
        if (options.debugLevel === true) {
            extra.debugLevel = new Set(['typescript-eslint']);
        }
        else if (Array.isArray(options.debugLevel)) {
            extra.debugLevel = new Set(options.debugLevel);
        }
        if (extra.debugLevel.size > 0) {
            // debug doesn't support multiple `enable` calls, so have to do it all at once
            const namespaces = [];
            if (extra.debugLevel.has('typescript-eslint')) {
                namespaces.push('typescript-eslint:*');
            }
            if (extra.debugLevel.has('eslint') ||
                // make sure we don't turn off the eslint debug if it was enabled via --debug
                debug_1.default.enabled('eslint:*')) {
                // https://github.com/eslint/eslint/blob/9dfc8501fb1956c90dc11e6377b4cb38a6bea65d/bin/eslint.js#L25
                namespaces.push('eslint:*,-eslint:code-path');
            }
            debug_1.default.enable(namespaces.join(','));
        }
        /**
         * Track range information in the AST
         */
        extra.range = typeof options.range === 'boolean' && options.range;
        extra.loc = typeof options.loc === 'boolean' && options.loc;
        /**
         * Track tokens in the AST
         */
        if (typeof options.tokens === 'boolean' && options.tokens) {
            extra.tokens = [];
        }
        /**
         * Track comments in the AST
         */
        if (typeof options.comment === 'boolean' && options.comment) {
            extra.comment = true;
            extra.comments = [];
        }
        /**
         * Enable JSX - note the applicable file extension is still required
         */
        if (typeof options.jsx === 'boolean' && options.jsx) {
            extra.jsx = true;
        }
        /**
         * Get the file extension
         */
        if (typeof options.filePath === 'string' && options.filePath !== '<input>') {
            extra.filePath = options.filePath;
        }
        else {
            extra.filePath = getFileName(extra);
        }
        /**
         * The JSX AST changed the node type for string literals
         * inside a JSX Element from `Literal` to `JSXText`.
         *
         * When value is `true`, these nodes will be parsed as type `JSXText`.
         * When value is `false`, these nodes will be parsed as type `Literal`.
         */
        if (typeof options.useJSXTextNode === 'boolean' && options.useJSXTextNode) {
            extra.useJSXTextNode = true;
        }
        /**
         * Allow the user to cause the parser to error if it encounters an unknown AST Node Type
         * (used in testing)
         */
        if (typeof options.errorOnUnknownASTType === 'boolean' &&
            options.errorOnUnknownASTType) {
            extra.errorOnUnknownASTType = true;
        }
        /**
         * Allow the user to override the function used for logging
         */
        if (typeof options.loggerFn === 'function') {
            extra.log = options.loggerFn;
        }
        else if (options.loggerFn === false) {
            extra.log = Function.prototype;
        }
        if (typeof options.project === 'string') {
            extra.projects = [options.project];
        }
        else if (Array.isArray(options.project) &&
            options.project.every(projectPath => typeof projectPath === 'string')) {
            extra.projects = options.project;
        }
        if (typeof options.tsconfigRootDir === 'string') {
            extra.tsconfigRootDir = options.tsconfigRootDir;
        }
        extra.filePath = shared_1.ensureAbsolutePath(extra.filePath, extra);
        // Transform glob patterns into paths
        if (extra.projects) {
            extra.projects = extra.projects.reduce((projects, project) => projects.concat(is_glob_1.default(project)
                ? glob_1.sync(project, {
                    cwd: extra.tsconfigRootDir || process.cwd(),
                })
                : project), []);
        }
        if (Array.isArray(options.extraFileExtensions) &&
            options.extraFileExtensions.every(ext => typeof ext === 'string')) {
            extra.extraFileExtensions = options.extraFileExtensions;
        }
        /**
         * Allow the user to enable or disable the preservation of the AST node maps
         * during the conversion process.
         *
         * NOTE: For backwards compatibility we also preserve node maps in the case where `project` is set,
         * and `preserveNodeMaps` is not explicitly set to anything.
         */
        extra.preserveNodeMaps =
            typeof options.preserveNodeMaps === 'boolean' && options.preserveNodeMaps;
        if (options.preserveNodeMaps === undefined && extra.projects.length > 0) {
            extra.preserveNodeMaps = true;
        }
        extra.createDefaultProgram =
            typeof options.createDefaultProgram === 'boolean' &&
                options.createDefaultProgram;
    }
    function warnAboutTSVersion() {
        var _a;
        if (!isRunningSupportedTypeScriptVersion && !warnedAboutTSVersion) {
            const isTTY = typeof process === undefined ? false : (_a = process.stdout) === null || _a === void 0 ? void 0 : _a.isTTY;
            if (isTTY) {
                const border = '=============';
                const versionWarning = [
                    border,
                    'WARNING: You are currently running a version of TypeScript which is not officially supported by @typescript-eslint/typescript-estree.',
                    'You may find that it works just fine, or you may not.',
                    `SUPPORTED TYPESCRIPT VERSIONS: ${SUPPORTED_TYPESCRIPT_VERSIONS}`,
                    `YOUR TYPESCRIPT VERSION: ${ACTIVE_TYPESCRIPT_VERSION}`,
                    'Please only submit bug reports when using the officially supported version.',
                    border,
                ];
                extra.log(versionWarning.join('\n\n'));
            }
            warnedAboutTSVersion = true;
        }
    }
    //------------------------------------------------------------------------------
    // Public
    //------------------------------------------------------------------------------
    const version = require(818) /* ../package.json */.version;
    exports.version = version;
    function parse(code, options) {
        /**
         * Reset the parse configuration
         */
        resetExtra();
        /**
         * Ensure users do not attempt to use parse() when they need parseAndGenerateServices()
         */
        if (options === null || options === void 0 ? void 0 : options.errorOnTypeScriptSyntacticAndSemanticIssues) {
            throw new Error(`"errorOnTypeScriptSyntacticAndSemanticIssues" is only supported for parseAndGenerateServices()`);
        }
        /**
         * Ensure the source code is a string, and store a reference to it
         */
        code = enforceString(code);
        extra.code = code;
        /**
         * Apply the given parser options
         */
        if (typeof options !== 'undefined') {
            applyParserOptionsToExtra(options);
        }
        /**
         * Warn if the user is using an unsupported version of TypeScript
         */
        warnAboutTSVersion();
        /**
         * Create a ts.SourceFile directly, no ts.Program is needed for a simple
         * parse
         */
        const ast = createSourceFile_1.createSourceFile(code, extra);
        /**
         * Convert the TypeScript AST to an ESTree-compatible one
         */
        const { estree } = ast_converter_1.astConverter(ast, extra, false);
        return estree;
    }
    exports.parse = parse;
    function parseAndGenerateServices(code, options) {
        /**
         * Reset the parse configuration
         */
        resetExtra();
        /**
         * Ensure the source code is a string, and store a reference to it
         */
        code = enforceString(code);
        extra.code = code;
        /**
         * Apply the given parser options
         */
        if (typeof options !== 'undefined') {
            applyParserOptionsToExtra(options);
            if (typeof options.errorOnTypeScriptSyntacticAndSemanticIssues ===
                'boolean' &&
                options.errorOnTypeScriptSyntacticAndSemanticIssues) {
                extra.errorOnTypeScriptSyntacticAndSemanticIssues = true;
            }
        }
        /**
         * Warn if the user is using an unsupported version of TypeScript
         */
        warnAboutTSVersion();
        /**
         * Generate a full ts.Program in order to be able to provide parser
         * services, such as type-checking
         */
        const shouldProvideParserServices = extra.projects && extra.projects.length > 0;
        const { ast, program } = getProgramAndAST(code, shouldProvideParserServices, extra.createDefaultProgram);
        /**
         * Determine if two-way maps of converted AST nodes should be preserved
         * during the conversion process
         */
        const shouldPreserveNodeMaps = extra.preserveNodeMaps !== undefined
            ? extra.preserveNodeMaps
            : shouldProvideParserServices;
        /**
         * Convert the TypeScript AST to an ESTree-compatible one, and optionally preserve
         * mappings between converted and original AST nodes
         */
        const { estree, astMaps } = ast_converter_1.astConverter(ast, extra, shouldPreserveNodeMaps);
        /**
         * Even if TypeScript parsed the source code ok, and we had no problems converting the AST,
         * there may be other syntactic or semantic issues in the code that we can optionally report on.
         */
        if (program && extra.errorOnTypeScriptSyntacticAndSemanticIssues) {
            const error = semantic_or_syntactic_errors_1.getFirstSemanticOrSyntacticError(program, ast);
            if (error) {
                throw convert_1.convertError(error);
            }
        }
        /**
         * Return the converted AST and additional parser services
         */
        return {
            ast: estree,
            services: {
                program: shouldProvideParserServices ? program : undefined,
                esTreeNodeToTSNodeMap: shouldPreserveNodeMaps && astMaps
                    ? astMaps.esTreeNodeToTSNodeMap
                    : undefined,
                tsNodeToESTreeNodeMap: shouldPreserveNodeMaps && astMaps
                    ? astMaps.tsNodeToESTreeNodeMap
                    : undefined,
            },
        };
    }
    exports.parseAndGenerateServices = parseAndGenerateServices;
    var simple_traverse_1 = require(809) /* ./simple-traverse */;
    exports.simpleTraverse = simple_traverse_1.simpleTraverse;
    var visitor_keys_1 = require(810) /* ./visitor-keys */;
    exports.visitorKeys = visitor_keys_1.visitorKeys;
    __export(require(619) /* ./ts-estree */);
    var createWatchProgram_1 = require(815) /* ./create-program/createWatchProgram */;
    exports.clearCaches = createWatchProgram_1.clearCaches;
},
/* semver/index.js */ function _(require, module, exports) {
    // just pre-load all the stuff that index.js lazily exports
    const internalRe = require(750) /* ./internal/re */;
    module.exports = {
        re: internalRe.re,
        src: internalRe.src,
        tokens: internalRe.t,
        SEMVER_SPEC_VERSION: require(751) /* ./internal/constants */.SEMVER_SPEC_VERSION,
        SemVer: require(753) /* ./classes/semver */,
        compareIdentifiers: require(754) /* ./internal/identifiers */.compareIdentifiers,
        rcompareIdentifiers: require(754) /* ./internal/identifiers */.rcompareIdentifiers,
        parse: require(755) /* ./functions/parse */,
        valid: require(756) /* ./functions/valid */,
        clean: require(757) /* ./functions/clean */,
        inc: require(758) /* ./functions/inc */,
        diff: require(759) /* ./functions/diff */,
        major: require(762) /* ./functions/major */,
        minor: require(763) /* ./functions/minor */,
        patch: require(764) /* ./functions/patch */,
        prerelease: require(765) /* ./functions/prerelease */,
        compare: require(761) /* ./functions/compare */,
        rcompare: require(766) /* ./functions/rcompare */,
        compareLoose: require(767) /* ./functions/compare-loose */,
        compareBuild: require(768) /* ./functions/compare-build */,
        sort: require(769) /* ./functions/sort */,
        rsort: require(770) /* ./functions/rsort */,
        gt: require(771) /* ./functions/gt */,
        lt: require(772) /* ./functions/lt */,
        eq: require(760) /* ./functions/eq */,
        neq: require(773) /* ./functions/neq */,
        gte: require(774) /* ./functions/gte */,
        lte: require(775) /* ./functions/lte */,
        cmp: require(776) /* ./functions/cmp */,
        coerce: require(777) /* ./functions/coerce */,
        Comparator: require(778) /* ./classes/comparator */,
        Range: require(779) /* ./classes/range */,
        satisfies: require(780) /* ./functions/satisfies */,
        toComparators: require(781) /* ./ranges/to-comparators */,
        maxSatisfying: require(782) /* ./ranges/max-satisfying */,
        minSatisfying: require(783) /* ./ranges/min-satisfying */,
        minVersion: require(784) /* ./ranges/min-version */,
        validRange: require(785) /* ./ranges/valid */,
        outside: require(786) /* ./ranges/outside */,
        gtr: require(787) /* ./ranges/gtr */,
        ltr: require(788) /* ./ranges/ltr */,
        intersects: require(789) /* ./ranges/intersects */,
    };
},
/* semver/internal/re.js */ function _(require, module, exports) {
    const { MAX_SAFE_COMPONENT_LENGTH } = require(751) /* ./constants */;
    const debug = require(752) /* ./debug */;
    exports = module.exports = {};
    // The actual regexps go on exports.re
    const re = exports.re = [];
    const src = exports.src = [];
    const t = exports.t = {};
    let R = 0;
    const createToken = (name, value, isGlobal) => {
        const index = R++;
        debug(index, value);
        t[name] = index;
        src[index] = value;
        re[index] = new RegExp(value, isGlobal ? 'g' : undefined);
    };
    // The following Regular Expressions can be used for tokenizing,
    // validating, and parsing SemVer version strings.
    // ## Numeric Identifier
    // A single `0`, or a non-zero digit followed by zero or more digits.
    createToken('NUMERICIDENTIFIER', '0|[1-9]\\d*');
    createToken('NUMERICIDENTIFIERLOOSE', '[0-9]+');
    // ## Non-numeric Identifier
    // Zero or more digits, followed by a letter or hyphen, and then zero or
    // more letters, digits, or hyphens.
    createToken('NONNUMERICIDENTIFIER', '\\d*[a-zA-Z-][a-zA-Z0-9-]*');
    // ## Main Version
    // Three dot-separated numeric identifiers.
    createToken('MAINVERSION', `(${src[t.NUMERICIDENTIFIER]})\\.` +
        `(${src[t.NUMERICIDENTIFIER]})\\.` +
        `(${src[t.NUMERICIDENTIFIER]})`);
    createToken('MAINVERSIONLOOSE', `(${src[t.NUMERICIDENTIFIERLOOSE]})\\.` +
        `(${src[t.NUMERICIDENTIFIERLOOSE]})\\.` +
        `(${src[t.NUMERICIDENTIFIERLOOSE]})`);
    // ## Pre-release Version Identifier
    // A numeric identifier, or a non-numeric identifier.
    createToken('PRERELEASEIDENTIFIER', `(?:${src[t.NUMERICIDENTIFIER]}|${src[t.NONNUMERICIDENTIFIER]})`);
    createToken('PRERELEASEIDENTIFIERLOOSE', `(?:${src[t.NUMERICIDENTIFIERLOOSE]}|${src[t.NONNUMERICIDENTIFIER]})`);
    // ## Pre-release Version
    // Hyphen, followed by one or more dot-separated pre-release version
    // identifiers.
    createToken('PRERELEASE', `(?:-(${src[t.PRERELEASEIDENTIFIER]}(?:\\.${src[t.PRERELEASEIDENTIFIER]})*))`);
    createToken('PRERELEASELOOSE', `(?:-?(${src[t.PRERELEASEIDENTIFIERLOOSE]}(?:\\.${src[t.PRERELEASEIDENTIFIERLOOSE]})*))`);
    // ## Build Metadata Identifier
    // Any combination of digits, letters, or hyphens.
    createToken('BUILDIDENTIFIER', '[0-9A-Za-z-]+');
    // ## Build Metadata
    // Plus sign, followed by one or more period-separated build metadata
    // identifiers.
    createToken('BUILD', `(?:\\+(${src[t.BUILDIDENTIFIER]}(?:\\.${src[t.BUILDIDENTIFIER]})*))`);
    // ## Full Version String
    // A main version, followed optionally by a pre-release version and
    // build metadata.
    // Note that the only major, minor, patch, and pre-release sections of
    // the version string are capturing groups.  The build metadata is not a
    // capturing group, because it should not ever be used in version
    // comparison.
    createToken('FULLPLAIN', `v?${src[t.MAINVERSION]}${src[t.PRERELEASE]}?${src[t.BUILD]}?`);
    createToken('FULL', `^${src[t.FULLPLAIN]}$`);
    // like full, but allows v1.2.3 and =1.2.3, which people do sometimes.
    // also, 1.0.0alpha1 (prerelease without the hyphen) which is pretty
    // common in the npm registry.
    createToken('LOOSEPLAIN', `[v=\\s]*${src[t.MAINVERSIONLOOSE]}${src[t.PRERELEASELOOSE]}?${src[t.BUILD]}?`);
    createToken('LOOSE', `^${src[t.LOOSEPLAIN]}$`);
    createToken('GTLT', '((?:<|>)?=?)');
    // Something like "2.*" or "1.2.x".
    // Note that "x.x" is a valid xRange identifer, meaning "any version"
    // Only the first item is strictly required.
    createToken('XRANGEIDENTIFIERLOOSE', `${src[t.NUMERICIDENTIFIERLOOSE]}|x|X|\\*`);
    createToken('XRANGEIDENTIFIER', `${src[t.NUMERICIDENTIFIER]}|x|X|\\*`);
    createToken('XRANGEPLAIN', `[v=\\s]*(${src[t.XRANGEIDENTIFIER]})` +
        `(?:\\.(${src[t.XRANGEIDENTIFIER]})` +
        `(?:\\.(${src[t.XRANGEIDENTIFIER]})` +
        `(?:${src[t.PRERELEASE]})?${src[t.BUILD]}?` +
        `)?)?`);
    createToken('XRANGEPLAINLOOSE', `[v=\\s]*(${src[t.XRANGEIDENTIFIERLOOSE]})` +
        `(?:\\.(${src[t.XRANGEIDENTIFIERLOOSE]})` +
        `(?:\\.(${src[t.XRANGEIDENTIFIERLOOSE]})` +
        `(?:${src[t.PRERELEASELOOSE]})?${src[t.BUILD]}?` +
        `)?)?`);
    createToken('XRANGE', `^${src[t.GTLT]}\\s*${src[t.XRANGEPLAIN]}$`);
    createToken('XRANGELOOSE', `^${src[t.GTLT]}\\s*${src[t.XRANGEPLAINLOOSE]}$`);
    // Coercion.
    // Extract anything that could conceivably be a part of a valid semver
    createToken('COERCE', `${'(^|[^\\d])' +
        '(\\d{1,'}${MAX_SAFE_COMPONENT_LENGTH}})` +
        `(?:\\.(\\d{1,${MAX_SAFE_COMPONENT_LENGTH}}))?` +
        `(?:\\.(\\d{1,${MAX_SAFE_COMPONENT_LENGTH}}))?` +
        `(?:$|[^\\d])`);
    createToken('COERCERTL', src[t.COERCE], true);
    // Tilde ranges.
    // Meaning is "reasonably at or greater than"
    createToken('LONETILDE', '(?:~>?)');
    createToken('TILDETRIM', `(\\s*)${src[t.LONETILDE]}\\s+`, true);
    exports.tildeTrimReplace = '$1~';
    createToken('TILDE', `^${src[t.LONETILDE]}${src[t.XRANGEPLAIN]}$`);
    createToken('TILDELOOSE', `^${src[t.LONETILDE]}${src[t.XRANGEPLAINLOOSE]}$`);
    // Caret ranges.
    // Meaning is "at least and backwards compatible with"
    createToken('LONECARET', '(?:\\^)');
    createToken('CARETTRIM', `(\\s*)${src[t.LONECARET]}\\s+`, true);
    exports.caretTrimReplace = '$1^';
    createToken('CARET', `^${src[t.LONECARET]}${src[t.XRANGEPLAIN]}$`);
    createToken('CARETLOOSE', `^${src[t.LONECARET]}${src[t.XRANGEPLAINLOOSE]}$`);
    // A simple gt/lt/eq thing, or just "" to indicate "any version"
    createToken('COMPARATORLOOSE', `^${src[t.GTLT]}\\s*(${src[t.LOOSEPLAIN]})$|^$`);
    createToken('COMPARATOR', `^${src[t.GTLT]}\\s*(${src[t.FULLPLAIN]})$|^$`);
    // An expression to strip any whitespace between the gtlt and the thing
    // it modifies, so that `> 1.2.3` ==> `>1.2.3`
    createToken('COMPARATORTRIM', `(\\s*)${src[t.GTLT]}\\s*(${src[t.LOOSEPLAIN]}|${src[t.XRANGEPLAIN]})`, true);
    exports.comparatorTrimReplace = '$1$2$3';
    // Something like `1.2.3 - 1.2.4`
    // Note that these all use the loose form, because they'll be
    // checked against either the strict or loose comparator form
    // later.
    createToken('HYPHENRANGE', `^\\s*(${src[t.XRANGEPLAIN]})` +
        `\\s+-\\s+` +
        `(${src[t.XRANGEPLAIN]})` +
        `\\s*$`);
    createToken('HYPHENRANGELOOSE', `^\\s*(${src[t.XRANGEPLAINLOOSE]})` +
        `\\s+-\\s+` +
        `(${src[t.XRANGEPLAINLOOSE]})` +
        `\\s*$`);
    // Star ranges basically just allow anything at all.
    createToken('STAR', '(<|>)?=?\\s*\\*');
},
/* semver/internal/constants.js */ function _(require, module, exports) {
    // Note: this is the semver.org version of the spec that it implements
    // Not necessarily the package version of this code.
    const SEMVER_SPEC_VERSION = '2.0.0';
    const MAX_LENGTH = 256;
    const MAX_SAFE_INTEGER = Number.MAX_SAFE_INTEGER ||
        /* istanbul ignore next */ 9007199254740991;
    // Max safe segment length for coercion.
    const MAX_SAFE_COMPONENT_LENGTH = 16;
    module.exports = {
        SEMVER_SPEC_VERSION,
        MAX_LENGTH,
        MAX_SAFE_INTEGER,
        MAX_SAFE_COMPONENT_LENGTH
    };
},
/* semver/internal/debug.js */ function _(require, module, exports) {
    const debug = (typeof process === 'object' &&
        process.env &&
        process.env.NODE_DEBUG &&
        /\bsemver\b/i.test(process.env.NODE_DEBUG)) ? (...args) => console.error('SEMVER', ...args)
        : () => { };
    module.exports = debug;
},
/* semver/classes/semver.js */ function _(require, module, exports) {
    const debug = require(752) /* ../internal/debug */;
    const { MAX_LENGTH, MAX_SAFE_INTEGER } = require(751) /* ../internal/constants */;
    const { re, t } = require(750) /* ../internal/re */;
    const { compareIdentifiers } = require(754) /* ../internal/identifiers */;
    class SemVer {
        constructor(version, options) {
            if (!options || typeof options !== 'object') {
                options = {
                    loose: !!options,
                    includePrerelease: false
                };
            }
            if (version instanceof SemVer) {
                if (version.loose === !!options.loose &&
                    version.includePrerelease === !!options.includePrerelease) {
                    return version;
                }
                else {
                    version = version.version;
                }
            }
            else if (typeof version !== 'string') {
                throw new TypeError(`Invalid Version: ${version}`);
            }
            if (version.length > MAX_LENGTH) {
                throw new TypeError(`version is longer than ${MAX_LENGTH} characters`);
            }
            debug('SemVer', version, options);
            this.options = options;
            this.loose = !!options.loose;
            // this isn't actually relevant for versions, but keep it so that we
            // don't run into trouble passing this.options around.
            this.includePrerelease = !!options.includePrerelease;
            const m = version.trim().match(options.loose ? re[t.LOOSE] : re[t.FULL]);
            if (!m) {
                throw new TypeError(`Invalid Version: ${version}`);
            }
            this.raw = version;
            // these are actually numbers
            this.major = +m[1];
            this.minor = +m[2];
            this.patch = +m[3];
            if (this.major > MAX_SAFE_INTEGER || this.major < 0) {
                throw new TypeError('Invalid major version');
            }
            if (this.minor > MAX_SAFE_INTEGER || this.minor < 0) {
                throw new TypeError('Invalid minor version');
            }
            if (this.patch > MAX_SAFE_INTEGER || this.patch < 0) {
                throw new TypeError('Invalid patch version');
            }
            // numberify any prerelease numeric ids
            if (!m[4]) {
                this.prerelease = [];
            }
            else {
                this.prerelease = m[4].split('.').map((id) => {
                    if (/^[0-9]+$/.test(id)) {
                        const num = +id;
                        if (num >= 0 && num < MAX_SAFE_INTEGER) {
                            return num;
                        }
                    }
                    return id;
                });
            }
            this.build = m[5] ? m[5].split('.') : [];
            this.format();
        }
        format() {
            this.version = `${this.major}.${this.minor}.${this.patch}`;
            if (this.prerelease.length) {
                this.version += `-${this.prerelease.join('.')}`;
            }
            return this.version;
        }
        toString() {
            return this.version;
        }
        compare(other) {
            debug('SemVer.compare', this.version, this.options, other);
            if (!(other instanceof SemVer)) {
                if (typeof other === 'string' && other === this.version) {
                    return 0;
                }
                other = new SemVer(other, this.options);
            }
            if (other.version === this.version) {
                return 0;
            }
            return this.compareMain(other) || this.comparePre(other);
        }
        compareMain(other) {
            if (!(other instanceof SemVer)) {
                other = new SemVer(other, this.options);
            }
            return (compareIdentifiers(this.major, other.major) ||
                compareIdentifiers(this.minor, other.minor) ||
                compareIdentifiers(this.patch, other.patch));
        }
        comparePre(other) {
            if (!(other instanceof SemVer)) {
                other = new SemVer(other, this.options);
            }
            // NOT having a prerelease is > having one
            if (this.prerelease.length && !other.prerelease.length) {
                return -1;
            }
            else if (!this.prerelease.length && other.prerelease.length) {
                return 1;
            }
            else if (!this.prerelease.length && !other.prerelease.length) {
                return 0;
            }
            let i = 0;
            do {
                const a = this.prerelease[i];
                const b = other.prerelease[i];
                debug('prerelease compare', i, a, b);
                if (a === undefined && b === undefined) {
                    return 0;
                }
                else if (b === undefined) {
                    return 1;
                }
                else if (a === undefined) {
                    return -1;
                }
                else if (a === b) {
                    continue;
                }
                else {
                    return compareIdentifiers(a, b);
                }
            } while (++i);
        }
        compareBuild(other) {
            if (!(other instanceof SemVer)) {
                other = new SemVer(other, this.options);
            }
            let i = 0;
            do {
                const a = this.build[i];
                const b = other.build[i];
                debug('prerelease compare', i, a, b);
                if (a === undefined && b === undefined) {
                    return 0;
                }
                else if (b === undefined) {
                    return 1;
                }
                else if (a === undefined) {
                    return -1;
                }
                else if (a === b) {
                    continue;
                }
                else {
                    return compareIdentifiers(a, b);
                }
            } while (++i);
        }
        // preminor will bump the version up to the next minor release, and immediately
        // down to pre-release. premajor and prepatch work the same way.
        inc(release, identifier) {
            switch (release) {
                case 'premajor':
                    this.prerelease.length = 0;
                    this.patch = 0;
                    this.minor = 0;
                    this.major++;
                    this.inc('pre', identifier);
                    break;
                case 'preminor':
                    this.prerelease.length = 0;
                    this.patch = 0;
                    this.minor++;
                    this.inc('pre', identifier);
                    break;
                case 'prepatch':
                    // If this is already a prerelease, it will bump to the next version
                    // drop any prereleases that might already exist, since they are not
                    // relevant at this point.
                    this.prerelease.length = 0;
                    this.inc('patch', identifier);
                    this.inc('pre', identifier);
                    break;
                // If the input is a non-prerelease version, this acts the same as
                // prepatch.
                case 'prerelease':
                    if (this.prerelease.length === 0) {
                        this.inc('patch', identifier);
                    }
                    this.inc('pre', identifier);
                    break;
                case 'major':
                    // If this is a pre-major version, bump up to the same major version.
                    // Otherwise increment major.
                    // 1.0.0-5 bumps to 1.0.0
                    // 1.1.0 bumps to 2.0.0
                    if (this.minor !== 0 ||
                        this.patch !== 0 ||
                        this.prerelease.length === 0) {
                        this.major++;
                    }
                    this.minor = 0;
                    this.patch = 0;
                    this.prerelease = [];
                    break;
                case 'minor':
                    // If this is a pre-minor version, bump up to the same minor version.
                    // Otherwise increment minor.
                    // 1.2.0-5 bumps to 1.2.0
                    // 1.2.1 bumps to 1.3.0
                    if (this.patch !== 0 || this.prerelease.length === 0) {
                        this.minor++;
                    }
                    this.patch = 0;
                    this.prerelease = [];
                    break;
                case 'patch':
                    // If this is not a pre-release version, it will increment the patch.
                    // If it is a pre-release it will bump up to the same patch version.
                    // 1.2.0-5 patches to 1.2.0
                    // 1.2.0 patches to 1.2.1
                    if (this.prerelease.length === 0) {
                        this.patch++;
                    }
                    this.prerelease = [];
                    break;
                // This probably shouldn't be used publicly.
                // 1.0.0 'pre' would become 1.0.0-0 which is the wrong direction.
                case 'pre':
                    if (this.prerelease.length === 0) {
                        this.prerelease = [0];
                    }
                    else {
                        let i = this.prerelease.length;
                        while (--i >= 0) {
                            if (typeof this.prerelease[i] === 'number') {
                                this.prerelease[i]++;
                                i = -2;
                            }
                        }
                        if (i === -1) {
                            // didn't increment anything
                            this.prerelease.push(0);
                        }
                    }
                    if (identifier) {
                        // 1.2.0-beta.1 bumps to 1.2.0-beta.2,
                        // 1.2.0-beta.fooblz or 1.2.0-beta bumps to 1.2.0-beta.0
                        if (this.prerelease[0] === identifier) {
                            if (isNaN(this.prerelease[1])) {
                                this.prerelease = [identifier, 0];
                            }
                        }
                        else {
                            this.prerelease = [identifier, 0];
                        }
                    }
                    break;
                default:
                    throw new Error(`invalid increment argument: ${release}`);
            }
            this.format();
            this.raw = this.version;
            return this;
        }
    }
    module.exports = SemVer;
},
/* semver/internal/identifiers.js */ function _(require, module, exports) {
    const numeric = /^[0-9]+$/;
    const compareIdentifiers = (a, b) => {
        const anum = numeric.test(a);
        const bnum = numeric.test(b);
        if (anum && bnum) {
            a = +a;
            b = +b;
        }
        return a === b ? 0
            : (anum && !bnum) ? -1
                : (bnum && !anum) ? 1
                    : a < b ? -1
                        : 1;
    };
    const rcompareIdentifiers = (a, b) => compareIdentifiers(b, a);
    module.exports = {
        compareIdentifiers,
        rcompareIdentifiers
    };
},
/* semver/functions/parse.js */ function _(require, module, exports) {
    const { MAX_LENGTH } = require(751) /* ../internal/constants */;
    const { re, t } = require(750) /* ../internal/re */;
    const SemVer = require(753) /* ../classes/semver */;
    const parse = (version, options) => {
        if (!options || typeof options !== 'object') {
            options = {
                loose: !!options,
                includePrerelease: false
            };
        }
        if (version instanceof SemVer) {
            return version;
        }
        if (typeof version !== 'string') {
            return null;
        }
        if (version.length > MAX_LENGTH) {
            return null;
        }
        const r = options.loose ? re[t.LOOSE] : re[t.FULL];
        if (!r.test(version)) {
            return null;
        }
        try {
            return new SemVer(version, options);
        }
        catch (er) {
            return null;
        }
    };
    module.exports = parse;
},
/* semver/functions/valid.js */ function _(require, module, exports) {
    const parse = require(755) /* ./parse */;
    const valid = (version, options) => {
        const v = parse(version, options);
        return v ? v.version : null;
    };
    module.exports = valid;
},
/* semver/functions/clean.js */ function _(require, module, exports) {
    const parse = require(755) /* ./parse */;
    const clean = (version, options) => {
        const s = parse(version.trim().replace(/^[=v]+/, ''), options);
        return s ? s.version : null;
    };
    module.exports = clean;
},
/* semver/functions/inc.js */ function _(require, module, exports) {
    const SemVer = require(753) /* ../classes/semver */;
    const inc = (version, release, options, identifier) => {
        if (typeof (options) === 'string') {
            identifier = options;
            options = undefined;
        }
        try {
            return new SemVer(version, options).inc(release, identifier).version;
        }
        catch (er) {
            return null;
        }
    };
    module.exports = inc;
},
/* semver/functions/diff.js */ function _(require, module, exports) {
    const parse = require(755) /* ./parse */;
    const eq = require(760) /* ./eq */;
    const diff = (version1, version2) => {
        if (eq(version1, version2)) {
            return null;
        }
        else {
            const v1 = parse(version1);
            const v2 = parse(version2);
            const hasPre = v1.prerelease.length || v2.prerelease.length;
            const prefix = hasPre ? 'pre' : '';
            const defaultResult = hasPre ? 'prerelease' : '';
            for (const key in v1) {
                if (key === 'major' || key === 'minor' || key === 'patch') {
                    if (v1[key] !== v2[key]) {
                        return prefix + key;
                    }
                }
            }
            return defaultResult; // may be undefined
        }
    };
    module.exports = diff;
},
/* semver/functions/eq.js */ function _(require, module, exports) {
    const compare = require(761) /* ./compare */;
    const eq = (a, b, loose) => compare(a, b, loose) === 0;
    module.exports = eq;
},
/* semver/functions/compare.js */ function _(require, module, exports) {
    const SemVer = require(753) /* ../classes/semver */;
    const compare = (a, b, loose) => new SemVer(a, loose).compare(new SemVer(b, loose));
    module.exports = compare;
},
/* semver/functions/major.js */ function _(require, module, exports) {
    const SemVer = require(753) /* ../classes/semver */;
    const major = (a, loose) => new SemVer(a, loose).major;
    module.exports = major;
},
/* semver/functions/minor.js */ function _(require, module, exports) {
    const SemVer = require(753) /* ../classes/semver */;
    const minor = (a, loose) => new SemVer(a, loose).minor;
    module.exports = minor;
},
/* semver/functions/patch.js */ function _(require, module, exports) {
    const SemVer = require(753) /* ../classes/semver */;
    const patch = (a, loose) => new SemVer(a, loose).patch;
    module.exports = patch;
},
/* semver/functions/prerelease.js */ function _(require, module, exports) {
    const parse = require(755) /* ./parse */;
    const prerelease = (version, options) => {
        const parsed = parse(version, options);
        return (parsed && parsed.prerelease.length) ? parsed.prerelease : null;
    };
    module.exports = prerelease;
},
/* semver/functions/rcompare.js */ function _(require, module, exports) {
    const compare = require(761) /* ./compare */;
    const rcompare = (a, b, loose) => compare(b, a, loose);
    module.exports = rcompare;
},
/* semver/functions/compare-loose.js */ function _(require, module, exports) {
    const compare = require(761) /* ./compare */;
    const compareLoose = (a, b) => compare(a, b, true);
    module.exports = compareLoose;
},
/* semver/functions/compare-build.js */ function _(require, module, exports) {
    const SemVer = require(753) /* ../classes/semver */;
    const compareBuild = (a, b, loose) => {
        const versionA = new SemVer(a, loose);
        const versionB = new SemVer(b, loose);
        return versionA.compare(versionB) || versionA.compareBuild(versionB);
    };
    module.exports = compareBuild;
},
/* semver/functions/sort.js */ function _(require, module, exports) {
    const compareBuild = require(768) /* ./compare-build */;
    const sort = (list, loose) => list.sort((a, b) => compareBuild(a, b, loose));
    module.exports = sort;
},
/* semver/functions/rsort.js */ function _(require, module, exports) {
    const compareBuild = require(768) /* ./compare-build */;
    const rsort = (list, loose) => list.sort((a, b) => compareBuild(b, a, loose));
    module.exports = rsort;
},
/* semver/functions/gt.js */ function _(require, module, exports) {
    const compare = require(761) /* ./compare */;
    const gt = (a, b, loose) => compare(a, b, loose) > 0;
    module.exports = gt;
},
/* semver/functions/lt.js */ function _(require, module, exports) {
    const compare = require(761) /* ./compare */;
    const lt = (a, b, loose) => compare(a, b, loose) < 0;
    module.exports = lt;
},
/* semver/functions/neq.js */ function _(require, module, exports) {
    const compare = require(761) /* ./compare */;
    const neq = (a, b, loose) => compare(a, b, loose) !== 0;
    module.exports = neq;
},
/* semver/functions/gte.js */ function _(require, module, exports) {
    const compare = require(761) /* ./compare */;
    const gte = (a, b, loose) => compare(a, b, loose) >= 0;
    module.exports = gte;
},
/* semver/functions/lte.js */ function _(require, module, exports) {
    const compare = require(761) /* ./compare */;
    const lte = (a, b, loose) => compare(a, b, loose) <= 0;
    module.exports = lte;
},
/* semver/functions/cmp.js */ function _(require, module, exports) {
    const eq = require(760) /* ./eq */;
    const neq = require(773) /* ./neq */;
    const gt = require(771) /* ./gt */;
    const gte = require(774) /* ./gte */;
    const lt = require(772) /* ./lt */;
    const lte = require(775) /* ./lte */;
    const cmp = (a, op, b, loose) => {
        switch (op) {
            case '===':
                if (typeof a === 'object')
                    a = a.version;
                if (typeof b === 'object')
                    b = b.version;
                return a === b;
            case '!==':
                if (typeof a === 'object')
                    a = a.version;
                if (typeof b === 'object')
                    b = b.version;
                return a !== b;
            case '':
            case '=':
            case '==':
                return eq(a, b, loose);
            case '!=':
                return neq(a, b, loose);
            case '>':
                return gt(a, b, loose);
            case '>=':
                return gte(a, b, loose);
            case '<':
                return lt(a, b, loose);
            case '<=':
                return lte(a, b, loose);
            default:
                throw new TypeError(`Invalid operator: ${op}`);
        }
    };
    module.exports = cmp;
},
/* semver/functions/coerce.js */ function _(require, module, exports) {
    const SemVer = require(753) /* ../classes/semver */;
    const parse = require(755) /* ./parse */;
    const { re, t } = require(750) /* ../internal/re */;
    const coerce = (version, options) => {
        if (version instanceof SemVer) {
            return version;
        }
        if (typeof version === 'number') {
            version = String(version);
        }
        if (typeof version !== 'string') {
            return null;
        }
        options = options || {};
        let match = null;
        if (!options.rtl) {
            match = version.match(re[t.COERCE]);
        }
        else {
            // Find the right-most coercible string that does not share
            // a terminus with a more left-ward coercible string.
            // Eg, '1.2.3.4' wants to coerce '2.3.4', not '3.4' or '4'
            //
            // Walk through the string checking with a /g regexp
            // Manually set the index so as to pick up overlapping matches.
            // Stop when we get a match that ends at the string end, since no
            // coercible string can be more right-ward without the same terminus.
            let next;
            while ((next = re[t.COERCERTL].exec(version)) &&
                (!match || match.index + match[0].length !== version.length)) {
                if (!match ||
                    next.index + next[0].length !== match.index + match[0].length) {
                    match = next;
                }
                re[t.COERCERTL].lastIndex = next.index + next[1].length + next[2].length;
            }
            // leave it in a clean state
            re[t.COERCERTL].lastIndex = -1;
        }
        if (match === null)
            return null;
        return parse(`${match[2]}.${match[3] || '0'}.${match[4] || '0'}`, options);
    };
    module.exports = coerce;
},
/* semver/classes/comparator.js */ function _(require, module, exports) {
    const ANY = Symbol('SemVer ANY');
    // hoisted class for cyclic dependency
    class Comparator {
        static get ANY() {
            return ANY;
        }
        constructor(comp, options) {
            if (!options || typeof options !== 'object') {
                options = {
                    loose: !!options,
                    includePrerelease: false
                };
            }
            if (comp instanceof Comparator) {
                if (comp.loose === !!options.loose) {
                    return comp;
                }
                else {
                    comp = comp.value;
                }
            }
            debug('comparator', comp, options);
            this.options = options;
            this.loose = !!options.loose;
            this.parse(comp);
            if (this.semver === ANY) {
                this.value = '';
            }
            else {
                this.value = this.operator + this.semver.version;
            }
            debug('comp', this);
        }
        parse(comp) {
            const r = this.options.loose ? re[t.COMPARATORLOOSE] : re[t.COMPARATOR];
            const m = comp.match(r);
            if (!m) {
                throw new TypeError(`Invalid comparator: ${comp}`);
            }
            this.operator = m[1] !== undefined ? m[1] : '';
            if (this.operator === '=') {
                this.operator = '';
            }
            // if it literally is just '>' or '' then allow anything.
            if (!m[2]) {
                this.semver = ANY;
            }
            else {
                this.semver = new SemVer(m[2], this.options.loose);
            }
        }
        toString() {
            return this.value;
        }
        test(version) {
            debug('Comparator.test', version, this.options.loose);
            if (this.semver === ANY || version === ANY) {
                return true;
            }
            if (typeof version === 'string') {
                try {
                    version = new SemVer(version, this.options);
                }
                catch (er) {
                    return false;
                }
            }
            return cmp(version, this.operator, this.semver, this.options);
        }
        intersects(comp, options) {
            if (!(comp instanceof Comparator)) {
                throw new TypeError('a Comparator is required');
            }
            if (!options || typeof options !== 'object') {
                options = {
                    loose: !!options,
                    includePrerelease: false
                };
            }
            if (this.operator === '') {
                if (this.value === '') {
                    return true;
                }
                return new Range(comp.value, options).test(this.value);
            }
            else if (comp.operator === '') {
                if (comp.value === '') {
                    return true;
                }
                return new Range(this.value, options).test(comp.semver);
            }
            const sameDirectionIncreasing = (this.operator === '>=' || this.operator === '>') &&
                (comp.operator === '>=' || comp.operator === '>');
            const sameDirectionDecreasing = (this.operator === '<=' || this.operator === '<') &&
                (comp.operator === '<=' || comp.operator === '<');
            const sameSemVer = this.semver.version === comp.semver.version;
            const differentDirectionsInclusive = (this.operator === '>=' || this.operator === '<=') &&
                (comp.operator === '>=' || comp.operator === '<=');
            const oppositeDirectionsLessThan = cmp(this.semver, '<', comp.semver, options) &&
                (this.operator === '>=' || this.operator === '>') &&
                (comp.operator === '<=' || comp.operator === '<');
            const oppositeDirectionsGreaterThan = cmp(this.semver, '>', comp.semver, options) &&
                (this.operator === '<=' || this.operator === '<') &&
                (comp.operator === '>=' || comp.operator === '>');
            return (sameDirectionIncreasing ||
                sameDirectionDecreasing ||
                (sameSemVer && differentDirectionsInclusive) ||
                oppositeDirectionsLessThan ||
                oppositeDirectionsGreaterThan);
        }
    }
    module.exports = Comparator;
    const { re, t } = require(750) /* ../internal/re */;
    const cmp = require(776) /* ../functions/cmp */;
    const debug = require(752) /* ../internal/debug */;
    const SemVer = require(753) /* ./semver */;
    const Range = require(779) /* ./range */;
},
/* semver/classes/range.js */ function _(require, module, exports) {
    // hoisted class for cyclic dependency
    class Range {
        constructor(range, options) {
            if (!options || typeof options !== 'object') {
                options = {
                    loose: !!options,
                    includePrerelease: false
                };
            }
            if (range instanceof Range) {
                if (range.loose === !!options.loose &&
                    range.includePrerelease === !!options.includePrerelease) {
                    return range;
                }
                else {
                    return new Range(range.raw, options);
                }
            }
            if (range instanceof Comparator) {
                // just put it in the set and return
                this.raw = range.value;
                this.set = [[range]];
                this.format();
                return this;
            }
            this.options = options;
            this.loose = !!options.loose;
            this.includePrerelease = !!options.includePrerelease;
            // First, split based on boolean or ||
            this.raw = range;
            this.set = range
                .split(/\s*\|\|\s*/)
                // map the range to a 2d array of comparators
                .map(range => this.parseRange(range.trim()))
                // throw out any comparator lists that are empty
                // this generally means that it was not a valid range, which is allowed
                // in loose mode, but will still throw if the WHOLE range is invalid.
                .filter(c => c.length);
            if (!this.set.length) {
                throw new TypeError(`Invalid SemVer Range: ${range}`);
            }
            this.format();
        }
        format() {
            this.range = this.set
                .map((comps) => {
                return comps.join(' ').trim();
            })
                .join('||')
                .trim();
            return this.range;
        }
        toString() {
            return this.range;
        }
        parseRange(range) {
            const loose = this.options.loose;
            range = range.trim();
            // `1.2.3 - 1.2.4` => `>=1.2.3 <=1.2.4`
            const hr = loose ? re[t.HYPHENRANGELOOSE] : re[t.HYPHENRANGE];
            range = range.replace(hr, hyphenReplace);
            debug('hyphen replace', range);
            // `> 1.2.3 < 1.2.5` => `>1.2.3 <1.2.5`
            range = range.replace(re[t.COMPARATORTRIM], comparatorTrimReplace);
            debug('comparator trim', range, re[t.COMPARATORTRIM]);
            // `~ 1.2.3` => `~1.2.3`
            range = range.replace(re[t.TILDETRIM], tildeTrimReplace);
            // `^ 1.2.3` => `^1.2.3`
            range = range.replace(re[t.CARETTRIM], caretTrimReplace);
            // normalize spaces
            range = range.split(/\s+/).join(' ');
            // At this point, the range is completely trimmed and
            // ready to be split into comparators.
            const compRe = loose ? re[t.COMPARATORLOOSE] : re[t.COMPARATOR];
            return range
                .split(' ')
                .map(comp => parseComparator(comp, this.options))
                .join(' ')
                .split(/\s+/)
                // in loose mode, throw out any that are not valid comparators
                .filter(this.options.loose ? comp => !!comp.match(compRe) : () => true)
                .map(comp => new Comparator(comp, this.options));
        }
        intersects(range, options) {
            if (!(range instanceof Range)) {
                throw new TypeError('a Range is required');
            }
            return this.set.some((thisComparators) => {
                return (isSatisfiable(thisComparators, options) &&
                    range.set.some((rangeComparators) => {
                        return (isSatisfiable(rangeComparators, options) &&
                            thisComparators.every((thisComparator) => {
                                return rangeComparators.every((rangeComparator) => {
                                    return thisComparator.intersects(rangeComparator, options);
                                });
                            }));
                    }));
            });
        }
        // if ANY of the sets match ALL of its comparators, then pass
        test(version) {
            if (!version) {
                return false;
            }
            if (typeof version === 'string') {
                try {
                    version = new SemVer(version, this.options);
                }
                catch (er) {
                    return false;
                }
            }
            for (let i = 0; i < this.set.length; i++) {
                if (testSet(this.set[i], version, this.options)) {
                    return true;
                }
            }
            return false;
        }
    }
    module.exports = Range;
    const Comparator = require(778) /* ./comparator */;
    const debug = require(752) /* ../internal/debug */;
    const SemVer = require(753) /* ./semver */;
    const { re, t, comparatorTrimReplace, tildeTrimReplace, caretTrimReplace } = require(750) /* ../internal/re */;
    // take a set of comparators and determine whether there
    // exists a version which can satisfy it
    const isSatisfiable = (comparators, options) => {
        let result = true;
        const remainingComparators = comparators.slice();
        let testComparator = remainingComparators.pop();
        while (result && remainingComparators.length) {
            result = remainingComparators.every((otherComparator) => {
                return testComparator.intersects(otherComparator, options);
            });
            testComparator = remainingComparators.pop();
        }
        return result;
    };
    // comprised of xranges, tildes, stars, and gtlt's at this point.
    // already replaced the hyphen ranges
    // turn into a set of JUST comparators.
    const parseComparator = (comp, options) => {
        debug('comp', comp, options);
        comp = replaceCarets(comp, options);
        debug('caret', comp);
        comp = replaceTildes(comp, options);
        debug('tildes', comp);
        comp = replaceXRanges(comp, options);
        debug('xrange', comp);
        comp = replaceStars(comp, options);
        debug('stars', comp);
        return comp;
    };
    const isX = id => !id || id.toLowerCase() === 'x' || id === '*';
    // ~, ~> --> * (any, kinda silly)
    // ~2, ~2.x, ~2.x.x, ~>2, ~>2.x ~>2.x.x --> >=2.0.0 <3.0.0
    // ~2.0, ~2.0.x, ~>2.0, ~>2.0.x --> >=2.0.0 <2.1.0
    // ~1.2, ~1.2.x, ~>1.2, ~>1.2.x --> >=1.2.0 <1.3.0
    // ~1.2.3, ~>1.2.3 --> >=1.2.3 <1.3.0
    // ~1.2.0, ~>1.2.0 --> >=1.2.0 <1.3.0
    const replaceTildes = (comp, options) => comp.trim().split(/\s+/).map((comp) => {
        return replaceTilde(comp, options);
    }).join(' ');
    const replaceTilde = (comp, options) => {
        const r = options.loose ? re[t.TILDELOOSE] : re[t.TILDE];
        return comp.replace(r, (_, M, m, p, pr) => {
            debug('tilde', comp, _, M, m, p, pr);
            let ret;
            if (isX(M)) {
                ret = '';
            }
            else if (isX(m)) {
                ret = `>=${M}.0.0 <${+M + 1}.0.0`;
            }
            else if (isX(p)) {
                // ~1.2 == >=1.2.0 <1.3.0
                ret = `>=${M}.${m}.0 <${M}.${+m + 1}.0`;
            }
            else if (pr) {
                debug('replaceTilde pr', pr);
                ret = `>=${M}.${m}.${p}-${pr} <${M}.${+m + 1}.0`;
            }
            else {
                // ~1.2.3 == >=1.2.3 <1.3.0
                ret = `>=${M}.${m}.${p} <${M}.${+m + 1}.0`;
            }
            debug('tilde return', ret);
            return ret;
        });
    };
    // ^ --> * (any, kinda silly)
    // ^2, ^2.x, ^2.x.x --> >=2.0.0 <3.0.0
    // ^2.0, ^2.0.x --> >=2.0.0 <3.0.0
    // ^1.2, ^1.2.x --> >=1.2.0 <2.0.0
    // ^1.2.3 --> >=1.2.3 <2.0.0
    // ^1.2.0 --> >=1.2.0 <2.0.0
    const replaceCarets = (comp, options) => comp.trim().split(/\s+/).map((comp) => {
        return replaceCaret(comp, options);
    }).join(' ');
    const replaceCaret = (comp, options) => {
        debug('caret', comp, options);
        const r = options.loose ? re[t.CARETLOOSE] : re[t.CARET];
        return comp.replace(r, (_, M, m, p, pr) => {
            debug('caret', comp, _, M, m, p, pr);
            let ret;
            if (isX(M)) {
                ret = '';
            }
            else if (isX(m)) {
                ret = `>=${M}.0.0 <${+M + 1}.0.0`;
            }
            else if (isX(p)) {
                if (M === '0') {
                    ret = `>=${M}.${m}.0 <${M}.${+m + 1}.0`;
                }
                else {
                    ret = `>=${M}.${m}.0 <${+M + 1}.0.0`;
                }
            }
            else if (pr) {
                debug('replaceCaret pr', pr);
                if (M === '0') {
                    if (m === '0') {
                        ret = `>=${M}.${m}.${p}-${pr} <${M}.${m}.${+p + 1}`;
                    }
                    else {
                        ret = `>=${M}.${m}.${p}-${pr} <${M}.${+m + 1}.0`;
                    }
                }
                else {
                    ret = `>=${M}.${m}.${p}-${pr} <${+M + 1}.0.0`;
                }
            }
            else {
                debug('no pr');
                if (M === '0') {
                    if (m === '0') {
                        ret = `>=${M}.${m}.${p} <${M}.${m}.${+p + 1}`;
                    }
                    else {
                        ret = `>=${M}.${m}.${p} <${M}.${+m + 1}.0`;
                    }
                }
                else {
                    ret = `>=${M}.${m}.${p} <${+M + 1}.0.0`;
                }
            }
            debug('caret return', ret);
            return ret;
        });
    };
    const replaceXRanges = (comp, options) => {
        debug('replaceXRanges', comp, options);
        return comp.split(/\s+/).map((comp) => {
            return replaceXRange(comp, options);
        }).join(' ');
    };
    const replaceXRange = (comp, options) => {
        comp = comp.trim();
        const r = options.loose ? re[t.XRANGELOOSE] : re[t.XRANGE];
        return comp.replace(r, (ret, gtlt, M, m, p, pr) => {
            debug('xRange', comp, ret, gtlt, M, m, p, pr);
            const xM = isX(M);
            const xm = xM || isX(m);
            const xp = xm || isX(p);
            const anyX = xp;
            if (gtlt === '=' && anyX) {
                gtlt = '';
            }
            // if we're including prereleases in the match, then we need
            // to fix this to -0, the lowest possible prerelease value
            pr = options.includePrerelease ? '-0' : '';
            if (xM) {
                if (gtlt === '>' || gtlt === '<') {
                    // nothing is allowed
                    ret = '<0.0.0-0';
                }
                else {
                    // nothing is forbidden
                    ret = '*';
                }
            }
            else if (gtlt && anyX) {
                // we know patch is an x, because we have any x at all.
                // replace X with 0
                if (xm) {
                    m = 0;
                }
                p = 0;
                if (gtlt === '>') {
                    // >1 => >=2.0.0
                    // >1.2 => >=1.3.0
                    gtlt = '>=';
                    if (xm) {
                        M = +M + 1;
                        m = 0;
                        p = 0;
                    }
                    else {
                        m = +m + 1;
                        p = 0;
                    }
                }
                else if (gtlt === '<=') {
                    // <=0.7.x is actually <0.8.0, since any 0.7.x should
                    // pass.  Similarly, <=7.x is actually <8.0.0, etc.
                    gtlt = '<';
                    if (xm) {
                        M = +M + 1;
                    }
                    else {
                        m = +m + 1;
                    }
                }
                ret = `${gtlt + M}.${m}.${p}${pr}`;
            }
            else if (xm) {
                ret = `>=${M}.0.0${pr} <${+M + 1}.0.0${pr}`;
            }
            else if (xp) {
                ret = `>=${M}.${m}.0${pr} <${M}.${+m + 1}.0${pr}`;
            }
            debug('xRange return', ret);
            return ret;
        });
    };
    // Because * is AND-ed with everything else in the comparator,
    // and '' means "any version", just remove the *s entirely.
    const replaceStars = (comp, options) => {
        debug('replaceStars', comp, options);
        // Looseness is ignored here.  star is always as loose as it gets!
        return comp.trim().replace(re[t.STAR], '');
    };
    // This function is passed to string.replace(re[t.HYPHENRANGE])
    // M, m, patch, prerelease, build
    // 1.2 - 3.4.5 => >=1.2.0 <=3.4.5
    // 1.2.3 - 3.4 => >=1.2.0 <3.5.0 Any 3.4.x will do
    // 1.2 - 3.4 => >=1.2.0 <3.5.0
    const hyphenReplace = ($0, from, fM, fm, fp, fpr, fb, to, tM, tm, tp, tpr, tb) => {
        if (isX(fM)) {
            from = '';
        }
        else if (isX(fm)) {
            from = `>=${fM}.0.0`;
        }
        else if (isX(fp)) {
            from = `>=${fM}.${fm}.0`;
        }
        else {
            from = `>=${from}`;
        }
        if (isX(tM)) {
            to = '';
        }
        else if (isX(tm)) {
            to = `<${+tM + 1}.0.0`;
        }
        else if (isX(tp)) {
            to = `<${tM}.${+tm + 1}.0`;
        }
        else if (tpr) {
            to = `<=${tM}.${tm}.${tp}-${tpr}`;
        }
        else {
            to = `<=${to}`;
        }
        return (`${from} ${to}`).trim();
    };
    const testSet = (set, version, options) => {
        for (let i = 0; i < set.length; i++) {
            if (!set[i].test(version)) {
                return false;
            }
        }
        if (version.prerelease.length && !options.includePrerelease) {
            // Find the set of versions that are allowed to have prereleases
            // For example, ^1.2.3-pr.1 desugars to >=1.2.3-pr.1 <2.0.0
            // That should allow `1.2.3-pr.2` to pass.
            // However, `1.2.4-alpha.notready` should NOT be allowed,
            // even though it's within the range set by the comparators.
            for (let i = 0; i < set.length; i++) {
                debug(set[i].semver);
                if (set[i].semver === Comparator.ANY) {
                    continue;
                }
                if (set[i].semver.prerelease.length > 0) {
                    const allowed = set[i].semver;
                    if (allowed.major === version.major &&
                        allowed.minor === version.minor &&
                        allowed.patch === version.patch) {
                        return true;
                    }
                }
            }
            // Version has a -pre, but it's not one of the ones we like.
            return false;
        }
        return true;
    };
},
/* semver/functions/satisfies.js */ function _(require, module, exports) {
    const Range = require(779) /* ../classes/range */;
    const satisfies = (version, range, options) => {
        try {
            range = new Range(range, options);
        }
        catch (er) {
            return false;
        }
        return range.test(version);
    };
    module.exports = satisfies;
},
/* semver/ranges/to-comparators.js */ function _(require, module, exports) {
    const Range = require(779) /* ../classes/range */;
    // Mostly just for testing and legacy API reasons
    const toComparators = (range, options) => new Range(range, options).set
        .map(comp => comp.map(c => c.value).join(' ').trim().split(' '));
    module.exports = toComparators;
},
/* semver/ranges/max-satisfying.js */ function _(require, module, exports) {
    const SemVer = require(753) /* ../classes/semver */;
    const Range = require(779) /* ../classes/range */;
    const maxSatisfying = (versions, range, options) => {
        let max = null;
        let maxSV = null;
        let rangeObj = null;
        try {
            rangeObj = new Range(range, options);
        }
        catch (er) {
            return null;
        }
        versions.forEach((v) => {
            if (rangeObj.test(v)) {
                // satisfies(v, range, options)
                if (!max || maxSV.compare(v) === -1) {
                    // compare(max, v, true)
                    max = v;
                    maxSV = new SemVer(max, options);
                }
            }
        });
        return max;
    };
    module.exports = maxSatisfying;
},
/* semver/ranges/min-satisfying.js */ function _(require, module, exports) {
    const SemVer = require(753) /* ../classes/semver */;
    const Range = require(779) /* ../classes/range */;
    const minSatisfying = (versions, range, options) => {
        let min = null;
        let minSV = null;
        let rangeObj = null;
        try {
            rangeObj = new Range(range, options);
        }
        catch (er) {
            return null;
        }
        versions.forEach((v) => {
            if (rangeObj.test(v)) {
                // satisfies(v, range, options)
                if (!min || minSV.compare(v) === 1) {
                    // compare(min, v, true)
                    min = v;
                    minSV = new SemVer(min, options);
                }
            }
        });
        return min;
    };
    module.exports = minSatisfying;
},
/* semver/ranges/min-version.js */ function _(require, module, exports) {
    const SemVer = require(753) /* ../classes/semver */;
    const Range = require(779) /* ../classes/range */;
    const gt = require(771) /* ../functions/gt */;
    const minVersion = (range, loose) => {
        range = new Range(range, loose);
        let minver = new SemVer('0.0.0');
        if (range.test(minver)) {
            return minver;
        }
        minver = new SemVer('0.0.0-0');
        if (range.test(minver)) {
            return minver;
        }
        minver = null;
        for (let i = 0; i < range.set.length; ++i) {
            const comparators = range.set[i];
            comparators.forEach((comparator) => {
                // Clone to avoid manipulating the comparator's semver object.
                const compver = new SemVer(comparator.semver.version);
                switch (comparator.operator) {
                    case '>':
                        if (compver.prerelease.length === 0) {
                            compver.patch++;
                        }
                        else {
                            compver.prerelease.push(0);
                        }
                        compver.raw = compver.format();
                    /* fallthrough */
                    case '':
                    case '>=':
                        if (!minver || gt(minver, compver)) {
                            minver = compver;
                        }
                        break;
                    case '<':
                    case '<=':
                        /* Ignore maximum versions */
                        break;
                    /* istanbul ignore next */
                    default:
                        throw new Error(`Unexpected operation: ${comparator.operator}`);
                }
            });
        }
        if (minver && range.test(minver)) {
            return minver;
        }
        return null;
    };
    module.exports = minVersion;
},
/* semver/ranges/valid.js */ function _(require, module, exports) {
    const Range = require(779) /* ../classes/range */;
    const validRange = (range, options) => {
        try {
            // Return '*' instead of '' so that truthiness works.
            // This will throw if it's invalid anyway
            return new Range(range, options).range || '*';
        }
        catch (er) {
            return null;
        }
    };
    module.exports = validRange;
},
/* semver/ranges/outside.js */ function _(require, module, exports) {
    const SemVer = require(753) /* ../classes/semver */;
    const Comparator = require(778) /* ../classes/comparator */;
    const { ANY } = Comparator;
    const Range = require(779) /* ../classes/range */;
    const satisfies = require(780) /* ../functions/satisfies */;
    const gt = require(771) /* ../functions/gt */;
    const lt = require(772) /* ../functions/lt */;
    const lte = require(775) /* ../functions/lte */;
    const gte = require(774) /* ../functions/gte */;
    const outside = (version, range, hilo, options) => {
        version = new SemVer(version, options);
        range = new Range(range, options);
        let gtfn, ltefn, ltfn, comp, ecomp;
        switch (hilo) {
            case '>':
                gtfn = gt;
                ltefn = lte;
                ltfn = lt;
                comp = '>';
                ecomp = '>=';
                break;
            case '<':
                gtfn = lt;
                ltefn = gte;
                ltfn = gt;
                comp = '<';
                ecomp = '<=';
                break;
            default:
                throw new TypeError('Must provide a hilo val of "<" or ">"');
        }
        // If it satisifes the range it is not outside
        if (satisfies(version, range, options)) {
            return false;
        }
        // From now on, variable terms are as if we're in "gtr" mode.
        // but note that everything is flipped for the "ltr" function.
        for (let i = 0; i < range.set.length; ++i) {
            const comparators = range.set[i];
            let high = null;
            let low = null;
            comparators.forEach((comparator) => {
                if (comparator.semver === ANY) {
                    comparator = new Comparator('>=0.0.0');
                }
                high = high || comparator;
                low = low || comparator;
                if (gtfn(comparator.semver, high.semver, options)) {
                    high = comparator;
                }
                else if (ltfn(comparator.semver, low.semver, options)) {
                    low = comparator;
                }
            });
            // If the edge version comparator has a operator then our version
            // isn't outside it
            if (high.operator === comp || high.operator === ecomp) {
                return false;
            }
            // If the lowest version comparator has an operator and our version
            // is less than it then it isn't higher than the range
            if ((!low.operator || low.operator === comp) &&
                ltefn(version, low.semver)) {
                return false;
            }
            else if (low.operator === ecomp && ltfn(version, low.semver)) {
                return false;
            }
        }
        return true;
    };
    module.exports = outside;
},
/* semver/ranges/gtr.js */ function _(require, module, exports) {
    // Determine if version is greater than all the versions possible in the range.
    const outside = require(786) /* ./outside */;
    const gtr = (version, range, options) => outside(version, range, '>', options);
    module.exports = gtr;
},
/* semver/ranges/ltr.js */ function _(require, module, exports) {
    const outside = require(786) /* ./outside */;
    // Determine if version is less than all the versions possible in the range
    const ltr = (version, range, options) => outside(version, range, '<', options);
    module.exports = ltr;
},
/* semver/ranges/intersects.js */ function _(require, module, exports) {
    const Range = require(779) /* ../classes/range */;
    const intersects = (r1, r2, options) => {
        r1 = new Range(r1, options);
        r2 = new Range(r2, options);
        return r1.intersects(r2);
    };
    module.exports = intersects;
},
/* @typescript-eslint/typescript-estree/dist/ast-converter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const convert_1 = require(791) /* ./convert */;
    const convert_comments_1 = require(808) /* ./convert-comments */;
    const node_utils_1 = require(792) /* ./node-utils */;
    const simple_traverse_1 = require(809) /* ./simple-traverse */;
    function astConverter(ast, extra, shouldPreserveNodeMaps) {
        /**
         * The TypeScript compiler produced fundamental parse errors when parsing the
         * source.
         */
        // internal typescript api...
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const parseDiagnostics = ast.parseDiagnostics;
        if (parseDiagnostics.length) {
            throw convert_1.convertError(parseDiagnostics[0]);
        }
        /**
         * Recursively convert the TypeScript AST into an ESTree-compatible AST
         */
        const instance = new convert_1.Converter(ast, {
            errorOnUnknownASTType: extra.errorOnUnknownASTType || false,
            useJSXTextNode: extra.useJSXTextNode || false,
            shouldPreserveNodeMaps,
        });
        const estree = instance.convertProgram();
        /**
         * Optionally remove range and loc if specified
         */
        if (extra.range || extra.loc) {
            simple_traverse_1.simpleTraverse(estree, {
                enter: node => {
                    if (!extra.range) {
                        delete node.range;
                    }
                    if (!extra.loc) {
                        delete node.loc;
                    }
                },
            });
        }
        /**
         * Optionally convert and include all tokens in the AST
         */
        if (extra.tokens) {
            estree.tokens = node_utils_1.convertTokens(ast);
        }
        /**
         * Optionally convert and include all comments in the AST
         */
        if (extra.comment) {
            estree.comments = convert_comments_1.convertComments(ast, extra.code);
        }
        const astMaps = shouldPreserveNodeMaps ? instance.getASTMaps() : undefined;
        return { estree, astMaps };
    }
    exports.astConverter = astConverter;
},
/* @typescript-eslint/typescript-estree/dist/convert.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    // There's lots of funny stuff due to the typing of ts.Node
    /* eslint-disable @typescript-eslint/no-explicit-any */
    const ts = __importStar(require(44) /* typescript */);
    const node_utils_1 = require(792) /* ./node-utils */;
    const ts_estree_1 = require(619) /* ./ts-estree */;
    const SyntaxKind = ts.SyntaxKind;
    /**
     * Extends and formats a given error object
     * @param error the error object
     * @returns converted error object
     */
    function convertError(error) {
        return node_utils_1.createError(error.file, error.start, error.message || error.messageText);
    }
    exports.convertError = convertError;
    class Converter {
        /**
         * Converts a TypeScript node into an ESTree node
         * @param ast the full TypeScript AST
         * @param options additional options for the conversion
         * @returns the converted ESTreeNode
         */
        constructor(ast, options) {
            this.esTreeNodeToTSNodeMap = new WeakMap();
            this.tsNodeToESTreeNodeMap = new WeakMap();
            this.allowPattern = false;
            this.inTypeMode = false;
            this.ast = ast;
            this.options = Object.assign({}, options);
        }
        getASTMaps() {
            return {
                esTreeNodeToTSNodeMap: this.esTreeNodeToTSNodeMap,
                tsNodeToESTreeNodeMap: this.tsNodeToESTreeNodeMap,
            };
        }
        convertProgram() {
            return this.converter(this.ast);
        }
        /**
         * Converts a TypeScript node into an ESTree node.
         * @param node the child ts.Node
         * @param parent parentNode
         * @param inTypeMode flag to determine if we are in typeMode
         * @param allowPattern flag to determine if patterns are allowed
         * @returns the converted ESTree node
         */
        converter(node, parent, inTypeMode, allowPattern) {
            /**
             * Exit early for null and undefined
             */
            if (!node) {
                return null;
            }
            const typeMode = this.inTypeMode;
            const pattern = this.allowPattern;
            if (inTypeMode !== undefined) {
                this.inTypeMode = inTypeMode;
            }
            if (allowPattern !== undefined) {
                this.allowPattern = allowPattern;
            }
            const result = this.convertNode(node, (parent !== null && parent !== void 0 ? parent : node.parent));
            this.registerTSNodeInNodeMap(node, result);
            this.inTypeMode = typeMode;
            this.allowPattern = pattern;
            return result;
        }
        /**
         * Fixes the exports of the given ts.Node
         * @param node the ts.Node
         * @param result result
         * @returns the ESTreeNode with fixed exports
         */
        fixExports(node, result) {
            // check for exports
            if (node.modifiers && node.modifiers[0].kind === SyntaxKind.ExportKeyword) {
                /**
                 * Make sure that original node is registered instead of export
                 */
                this.registerTSNodeInNodeMap(node, result);
                const exportKeyword = node.modifiers[0];
                const nextModifier = node.modifiers[1];
                const declarationIsDefault = nextModifier && nextModifier.kind === SyntaxKind.DefaultKeyword;
                const varToken = declarationIsDefault
                    ? node_utils_1.findNextToken(nextModifier, this.ast, this.ast)
                    : node_utils_1.findNextToken(exportKeyword, this.ast, this.ast);
                result.range[0] = varToken.getStart(this.ast);
                result.loc = node_utils_1.getLocFor(result.range[0], result.range[1], this.ast);
                if (declarationIsDefault) {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ExportDefaultDeclaration,
                        declaration: result,
                        range: [exportKeyword.getStart(this.ast), result.range[1]],
                    });
                }
                else {
                    const isType = result.type === ts_estree_1.AST_NODE_TYPES.TSInterfaceDeclaration ||
                        result.type === ts_estree_1.AST_NODE_TYPES.TSTypeAliasDeclaration;
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ExportNamedDeclaration,
                        declaration: result,
                        specifiers: [],
                        source: null,
                        exportKind: isType ? 'type' : 'value',
                        range: [exportKeyword.getStart(this.ast), result.range[1]],
                    });
                }
            }
            return result;
        }
        /**
         * Register specific TypeScript node into map with first ESTree node provided
         */
        registerTSNodeInNodeMap(node, result) {
            if (result && this.options.shouldPreserveNodeMaps) {
                if (!this.tsNodeToESTreeNodeMap.has(node)) {
                    this.tsNodeToESTreeNodeMap.set(node, result);
                }
            }
        }
        /**
         * Converts a TypeScript node into an ESTree node.
         * @param child the child ts.Node
         * @param parent parentNode
         * @returns the converted ESTree node
         */
        convertPattern(child, parent) {
            return this.converter(child, parent, this.inTypeMode, true);
        }
        /**
         * Converts a TypeScript node into an ESTree node.
         * @param child the child ts.Node
         * @param parent parentNode
         * @returns the converted ESTree node
         */
        convertChild(child, parent) {
            return this.converter(child, parent, this.inTypeMode, false);
        }
        /**
         * Converts a TypeScript node into an ESTree node.
         * @param child the child ts.Node
         * @param parent parentNode
         * @returns the converted ESTree node
         */
        convertType(child, parent) {
            return this.converter(child, parent, true, false);
        }
        createNode(node, data) {
            const result = data;
            if (!result.range) {
                result.range = node_utils_1.getRange(node, this.ast);
            }
            if (!result.loc) {
                result.loc = node_utils_1.getLocFor(result.range[0], result.range[1], this.ast);
            }
            if (result && this.options.shouldPreserveNodeMaps) {
                this.esTreeNodeToTSNodeMap.set(result, node);
            }
            return result;
        }
        /**
         * Converts a child into a type annotation. This creates an intermediary
         * TypeAnnotation node to match what Flow does.
         * @param child The TypeScript AST node to convert.
         * @param parent parentNode
         * @returns The type annotation node.
         */
        convertTypeAnnotation(child, parent) {
            // in FunctionType and ConstructorType typeAnnotation has 2 characters `=>` and in other places is just colon
            const offset = parent.kind === SyntaxKind.FunctionType ||
                parent.kind === SyntaxKind.ConstructorType
                ? 2
                : 1;
            const annotationStartCol = child.getFullStart() - offset;
            const loc = node_utils_1.getLocFor(annotationStartCol, child.end, this.ast);
            return {
                type: ts_estree_1.AST_NODE_TYPES.TSTypeAnnotation,
                loc,
                range: [annotationStartCol, child.end],
                typeAnnotation: this.convertType(child),
            };
        }
        /**
         * Coverts body Nodes and add a directive field to StringLiterals
         * @param nodes of ts.Node
         * @param parent parentNode
         * @returns Array of body statements
         */
        convertBodyExpressions(nodes, parent) {
            let allowDirectives = node_utils_1.canContainDirective(parent);
            return (nodes
                .map(statement => {
                const child = this.convertChild(statement);
                if (allowDirectives) {
                    if ((child === null || child === void 0 ? void 0 : child.expression) &&
                        ts.isExpressionStatement(statement) &&
                        ts.isStringLiteral(statement.expression)) {
                        const raw = child.expression.raw;
                        child.directive = raw.slice(1, -1);
                        return child; // child can be null, but it's filtered below
                    }
                    else {
                        allowDirectives = false;
                    }
                }
                return child; // child can be null, but it's filtered below
            })
                // filter out unknown nodes for now
                .filter(statement => statement));
        }
        /**
         * Converts a ts.Node's typeArguments to TSTypeParameterInstantiation node
         * @param typeArguments ts.NodeArray typeArguments
         * @param node parent used to create this node
         * @returns TypeParameterInstantiation node
         */
        convertTypeArgumentsToTypeParameters(typeArguments, node) {
            const greaterThanToken = node_utils_1.findNextToken(typeArguments, this.ast, this.ast);
            return this.createNode(node, {
                type: ts_estree_1.AST_NODE_TYPES.TSTypeParameterInstantiation,
                range: [typeArguments.pos - 1, greaterThanToken.end],
                params: typeArguments.map(typeArgument => this.convertType(typeArgument)),
            });
        }
        /**
         * Converts a ts.Node's typeParameters to TSTypeParameterDeclaration node
         * @param typeParameters ts.Node typeParameters
         * @returns TypeParameterDeclaration node
         */
        convertTSTypeParametersToTypeParametersDeclaration(typeParameters) {
            const greaterThanToken = node_utils_1.findNextToken(typeParameters, this.ast, this.ast);
            return {
                type: ts_estree_1.AST_NODE_TYPES.TSTypeParameterDeclaration,
                range: [typeParameters.pos - 1, greaterThanToken.end],
                loc: node_utils_1.getLocFor(typeParameters.pos - 1, greaterThanToken.end, this.ast),
                params: typeParameters.map(typeParameter => this.convertType(typeParameter)),
            };
        }
        /**
         * Converts an array of ts.Node parameters into an array of ESTreeNode params
         * @param parameters An array of ts.Node params to be converted
         * @returns an array of converted ESTreeNode params
         */
        convertParameters(parameters) {
            if (!parameters || !parameters.length) {
                return [];
            }
            return parameters.map(param => {
                const convertedParam = this.convertChild(param);
                if (param.decorators && param.decorators.length) {
                    convertedParam.decorators = param.decorators.map(el => this.convertChild(el));
                }
                return convertedParam;
            });
        }
        /**
         * For nodes that are copied directly from the TypeScript AST into
         * ESTree mostly as-is. The only difference is the addition of a type
         * property instead of a kind property. Recursively copies all children.
         */
        deeplyCopy(node) {
            if (node.kind === ts.SyntaxKind.JSDocFunctionType) {
                throw node_utils_1.createError(this.ast, node.pos, 'JSDoc types can only be used inside documentation comments.');
            }
            const customType = `TS${SyntaxKind[node.kind]}`;
            /**
             * If the "errorOnUnknownASTType" option is set to true, throw an error,
             * otherwise fallback to just including the unknown type as-is.
             */
            if (this.options.errorOnUnknownASTType && !ts_estree_1.AST_NODE_TYPES[customType]) {
                throw new Error(`Unknown AST_NODE_TYPE: "${customType}"`);
            }
            const result = this.createNode(node, {
                type: customType,
            });
            if ('type' in node) {
                result.typeAnnotation =
                    node.type && 'kind' in node.type && ts.isTypeNode(node.type)
                        ? this.convertTypeAnnotation(node.type, node)
                        : null;
            }
            if ('typeArguments' in node) {
                result.typeParameters =
                    node.typeArguments && 'pos' in node.typeArguments
                        ? this.convertTypeArgumentsToTypeParameters(node.typeArguments, node)
                        : null;
            }
            if ('typeParameters' in node) {
                result.typeParameters =
                    node.typeParameters && 'pos' in node.typeParameters
                        ? this.convertTSTypeParametersToTypeParametersDeclaration(node.typeParameters)
                        : null;
            }
            if ('decorators' in node && node.decorators && node.decorators.length) {
                result.decorators = node.decorators.map(el => this.convertChild(el));
            }
            Object.entries(node)
                .filter(([key]) => !/^(?:_children|kind|parent|pos|end|flags|modifierFlagsCache|jsDoc|type|typeArguments|typeParameters|decorators)$/.test(key))
                .forEach(([key, value]) => {
                if (Array.isArray(value)) {
                    result[key] = value.map(el => this.convertChild(el));
                }
                else if (value && typeof value === 'object' && value.kind) {
                    // need to check node[key].kind to ensure we don't try to convert a symbol
                    result[key] = this.convertChild(value);
                }
                else {
                    result[key] = value;
                }
            });
            return result;
        }
        /**
         * Converts a TypeScript JSX node.tagName into an ESTree node.name
         * @param node the tagName object from a JSX ts.Node
         * @param parent
         * @returns the converted ESTree name object
         */
        convertJSXTagName(node, parent) {
            let result;
            switch (node.kind) {
                case SyntaxKind.PropertyAccessExpression:
                    if (node.name.kind === SyntaxKind.PrivateIdentifier) {
                        // This is one of the few times where TS explicitly errors, and doesn't even gracefully handle the syntax.
                        // So we shouldn't ever get into this state to begin with.
                        throw new Error('Non-private identifier expected.');
                    }
                    result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXMemberExpression,
                        object: this.convertJSXTagName(node.expression, parent),
                        property: this.convertJSXTagName(node.name, parent),
                    });
                    break;
                case SyntaxKind.ThisKeyword:
                    result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXIdentifier,
                        name: 'this',
                    });
                    break;
                case SyntaxKind.Identifier:
                default:
                    result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXIdentifier,
                        name: node.text,
                    });
                    break;
            }
            this.registerTSNodeInNodeMap(node, result);
            return result;
        }
        /**
         * Applies the given TS modifiers to the given result object.
         * @param result
         * @param modifiers original ts.Nodes from the node.modifiers array
         * @returns the current result object will be mutated
         * @deprecated This method adds not standardized `modifiers` property in nodes
         */
        applyModifiersToResult(result, modifiers) {
            if (!modifiers || !modifiers.length) {
                return;
            }
            /**
             * Some modifiers are explicitly handled by applying them as
             * boolean values on the result node. As well as adding them
             * to the result, we remove them from the array, so that they
             * are not handled twice.
             */
            const handledModifierIndices = {};
            for (let i = 0; i < modifiers.length; i++) {
                const modifier = modifiers[i];
                switch (modifier.kind) {
                    /**
                     * Ignore ExportKeyword and DefaultKeyword, they are handled
                     * via the fixExports utility function
                     */
                    case SyntaxKind.ExportKeyword:
                    case SyntaxKind.DefaultKeyword:
                        handledModifierIndices[i] = true;
                        break;
                    case SyntaxKind.ConstKeyword:
                        result.const = true;
                        handledModifierIndices[i] = true;
                        break;
                    case SyntaxKind.DeclareKeyword:
                        result.declare = true;
                        handledModifierIndices[i] = true;
                        break;
                    default:
                }
            }
            /**
             * If there are still valid modifiers available which have
             * not been explicitly handled above, we just convert and
             * add the modifiers array to the result node.
             */
            const remainingModifiers = modifiers.filter((_, i) => !handledModifierIndices[i]);
            if (!remainingModifiers || !remainingModifiers.length) {
                return;
            }
            result.modifiers = remainingModifiers.map(el => this.convertChild(el));
        }
        /**
         * Uses the provided range location to adjust the location data of the given Node
         * @param result The node that will have its location data mutated
         * @param childRange The child node range used to expand location
         */
        fixParentLocation(result, childRange) {
            if (childRange[0] < result.range[0]) {
                result.range[0] = childRange[0];
                result.loc.start = node_utils_1.getLineAndCharacterFor(result.range[0], this.ast);
            }
            if (childRange[1] > result.range[1]) {
                result.range[1] = childRange[1];
                result.loc.end = node_utils_1.getLineAndCharacterFor(result.range[1], this.ast);
            }
        }
        /**
         * Converts a TypeScript node into an ESTree node.
         * The core of the conversion logic:
         * Identify and convert each relevant TypeScript SyntaxKind
         * @param node the child ts.Node
         * @param parent parentNode
         * @returns the converted ESTree node
         */
        convertNode(node, parent) {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j;
            switch (node.kind) {
                case SyntaxKind.SourceFile: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Program,
                        body: this.convertBodyExpressions(node.statements, node),
                        // externalModuleIndicator is internal field in TSC
                        sourceType: node.externalModuleIndicator
                            ? 'module'
                            : 'script',
                        range: [node.getStart(this.ast), node.endOfFileToken.end],
                    });
                }
                case SyntaxKind.Block: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.BlockStatement,
                        body: this.convertBodyExpressions(node.statements, node),
                    });
                }
                case SyntaxKind.Identifier: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Identifier,
                        name: node.text,
                    });
                }
                case SyntaxKind.WithStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.WithStatement,
                        object: this.convertChild(node.expression),
                        body: this.convertChild(node.statement),
                    });
                // Control Flow
                case SyntaxKind.ReturnStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ReturnStatement,
                        argument: this.convertChild(node.expression),
                    });
                case SyntaxKind.LabeledStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.LabeledStatement,
                        label: this.convertChild(node.label),
                        body: this.convertChild(node.statement),
                    });
                case SyntaxKind.ContinueStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ContinueStatement,
                        label: this.convertChild(node.label),
                    });
                case SyntaxKind.BreakStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.BreakStatement,
                        label: this.convertChild(node.label),
                    });
                // Choice
                case SyntaxKind.IfStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.IfStatement,
                        test: this.convertChild(node.expression),
                        consequent: this.convertChild(node.thenStatement),
                        alternate: this.convertChild(node.elseStatement),
                    });
                case SyntaxKind.SwitchStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.SwitchStatement,
                        discriminant: this.convertChild(node.expression),
                        cases: node.caseBlock.clauses.map(el => this.convertChild(el)),
                    });
                case SyntaxKind.CaseClause:
                case SyntaxKind.DefaultClause:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.SwitchCase,
                        // expression is present in case only
                        test: node.kind === SyntaxKind.CaseClause
                            ? this.convertChild(node.expression)
                            : null,
                        consequent: node.statements.map(el => this.convertChild(el)),
                    });
                // Exceptions
                case SyntaxKind.ThrowStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ThrowStatement,
                        argument: this.convertChild(node.expression),
                    });
                case SyntaxKind.TryStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TryStatement,
                        block: this.convertChild(node.tryBlock),
                        handler: this.convertChild(node.catchClause),
                        finalizer: this.convertChild(node.finallyBlock),
                    });
                case SyntaxKind.CatchClause:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.CatchClause,
                        param: node.variableDeclaration
                            ? this.convertChild(node.variableDeclaration.name)
                            : null,
                        body: this.convertChild(node.block),
                    });
                // Loops
                case SyntaxKind.WhileStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.WhileStatement,
                        test: this.convertChild(node.expression),
                        body: this.convertChild(node.statement),
                    });
                /**
                 * Unlike other parsers, TypeScript calls a "DoWhileStatement"
                 * a "DoStatement"
                 */
                case SyntaxKind.DoStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.DoWhileStatement,
                        test: this.convertChild(node.expression),
                        body: this.convertChild(node.statement),
                    });
                case SyntaxKind.ForStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ForStatement,
                        init: this.convertChild(node.initializer),
                        test: this.convertChild(node.condition),
                        update: this.convertChild(node.incrementor),
                        body: this.convertChild(node.statement),
                    });
                case SyntaxKind.ForInStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ForInStatement,
                        left: this.convertPattern(node.initializer),
                        right: this.convertChild(node.expression),
                        body: this.convertChild(node.statement),
                    });
                case SyntaxKind.ForOfStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ForOfStatement,
                        left: this.convertPattern(node.initializer),
                        right: this.convertChild(node.expression),
                        body: this.convertChild(node.statement),
                        await: Boolean(node.awaitModifier &&
                            node.awaitModifier.kind === SyntaxKind.AwaitKeyword),
                    });
                // Declarations
                case SyntaxKind.FunctionDeclaration: {
                    const isDeclare = node_utils_1.hasModifier(SyntaxKind.DeclareKeyword, node);
                    const result = this.createNode(node, {
                        type: isDeclare || !node.body
                            ? ts_estree_1.AST_NODE_TYPES.TSDeclareFunction
                            : ts_estree_1.AST_NODE_TYPES.FunctionDeclaration,
                        id: this.convertChild(node.name),
                        generator: !!node.asteriskToken,
                        expression: false,
                        async: node_utils_1.hasModifier(SyntaxKind.AsyncKeyword, node),
                        params: this.convertParameters(node.parameters),
                        body: this.convertChild(node.body) || undefined,
                    });
                    // Process returnType
                    if (node.type) {
                        result.returnType = this.convertTypeAnnotation(node.type, node);
                    }
                    if (isDeclare) {
                        result.declare = true;
                    }
                    // Process typeParameters
                    if (node.typeParameters) {
                        result.typeParameters = this.convertTSTypeParametersToTypeParametersDeclaration(node.typeParameters);
                    }
                    /**
                     * Semantically, decorators are not allowed on function declarations,
                     * but the TypeScript compiler will parse them and produce a valid AST,
                     * so we handle them here too.
                     */
                    if (node.decorators) {
                        result.decorators = node.decorators.map(el => this.convertChild(el));
                    }
                    // check for exports
                    return this.fixExports(node, result);
                }
                case SyntaxKind.VariableDeclaration: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.VariableDeclarator,
                        id: this.convertPattern(node.name),
                        init: this.convertChild(node.initializer),
                    });
                    if (node.exclamationToken) {
                        result.definite = true;
                    }
                    if (node.type) {
                        result.id.typeAnnotation = this.convertTypeAnnotation(node.type, node);
                        this.fixParentLocation(result.id, result.id.typeAnnotation.range);
                    }
                    return result;
                }
                case SyntaxKind.VariableStatement: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.VariableDeclaration,
                        declarations: node.declarationList.declarations.map(el => this.convertChild(el)),
                        kind: node_utils_1.getDeclarationKind(node.declarationList),
                    });
                    /**
                     * Semantically, decorators are not allowed on variable declarations,
                     * but the TypeScript compiler will parse them and produce a valid AST,
                     * so we handle them here too.
                     */
                    if (node.decorators) {
                        result.decorators = node.decorators.map(el => this.convertChild(el));
                    }
                    if (node_utils_1.hasModifier(SyntaxKind.DeclareKeyword, node)) {
                        result.declare = true;
                    }
                    // check for exports
                    return this.fixExports(node, result);
                }
                // mostly for for-of, for-in
                case SyntaxKind.VariableDeclarationList:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.VariableDeclaration,
                        declarations: node.declarations.map(el => this.convertChild(el)),
                        kind: node_utils_1.getDeclarationKind(node),
                    });
                // Expressions
                case SyntaxKind.ExpressionStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ExpressionStatement,
                        expression: this.convertChild(node.expression),
                    });
                case SyntaxKind.ThisKeyword:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ThisExpression,
                    });
                case SyntaxKind.ArrayLiteralExpression: {
                    // TypeScript uses ArrayLiteralExpression in destructuring assignment, too
                    if (this.allowPattern) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.ArrayPattern,
                            elements: node.elements.map(el => this.convertPattern(el)),
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.ArrayExpression,
                            elements: node.elements.map(el => this.convertChild(el)),
                        });
                    }
                }
                case SyntaxKind.ObjectLiteralExpression: {
                    // TypeScript uses ObjectLiteralExpression in destructuring assignment, too
                    if (this.allowPattern) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.ObjectPattern,
                            properties: node.properties.map(el => this.convertPattern(el)),
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.ObjectExpression,
                            properties: node.properties.map(el => this.convertChild(el)),
                        });
                    }
                }
                case SyntaxKind.PropertyAssignment:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Property,
                        key: this.convertChild(node.name),
                        value: this.converter(node.initializer, node, this.inTypeMode, this.allowPattern),
                        computed: node_utils_1.isComputedProperty(node.name),
                        method: false,
                        shorthand: false,
                        kind: 'init',
                    });
                case SyntaxKind.ShorthandPropertyAssignment: {
                    if (node.objectAssignmentInitializer) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.Property,
                            key: this.convertChild(node.name),
                            value: this.createNode(node, {
                                type: ts_estree_1.AST_NODE_TYPES.AssignmentPattern,
                                left: this.convertPattern(node.name),
                                right: this.convertChild(node.objectAssignmentInitializer),
                            }),
                            computed: false,
                            method: false,
                            shorthand: true,
                            kind: 'init',
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.Property,
                            key: this.convertChild(node.name),
                            value: this.convertChild(node.name),
                            computed: false,
                            method: false,
                            shorthand: true,
                            kind: 'init',
                        });
                    }
                }
                case SyntaxKind.ComputedPropertyName:
                    return this.convertChild(node.expression);
                case SyntaxKind.PropertyDeclaration: {
                    const isAbstract = node_utils_1.hasModifier(SyntaxKind.AbstractKeyword, node);
                    const result = this.createNode(node, {
                        type: isAbstract
                            ? ts_estree_1.AST_NODE_TYPES.TSAbstractClassProperty
                            : ts_estree_1.AST_NODE_TYPES.ClassProperty,
                        key: this.convertChild(node.name),
                        value: this.convertChild(node.initializer),
                        computed: node_utils_1.isComputedProperty(node.name),
                        static: node_utils_1.hasModifier(SyntaxKind.StaticKeyword, node),
                        readonly: node_utils_1.hasModifier(SyntaxKind.ReadonlyKeyword, node) || undefined,
                        declare: node_utils_1.hasModifier(SyntaxKind.DeclareKeyword, node),
                    });
                    if (node.type) {
                        result.typeAnnotation = this.convertTypeAnnotation(node.type, node);
                    }
                    if (node.decorators) {
                        result.decorators = node.decorators.map(el => this.convertChild(el));
                    }
                    const accessibility = node_utils_1.getTSNodeAccessibility(node);
                    if (accessibility) {
                        result.accessibility = accessibility;
                    }
                    if ((node.name.kind === SyntaxKind.Identifier ||
                        node.name.kind === SyntaxKind.ComputedPropertyName) &&
                        node.questionToken) {
                        result.optional = true;
                    }
                    if (node.exclamationToken) {
                        result.definite = true;
                    }
                    if (result.key.type === ts_estree_1.AST_NODE_TYPES.Literal && node.questionToken) {
                        result.optional = true;
                    }
                    return result;
                }
                case SyntaxKind.GetAccessor:
                case SyntaxKind.SetAccessor:
                case SyntaxKind.MethodDeclaration: {
                    const method = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.FunctionExpression,
                        id: null,
                        generator: !!node.asteriskToken,
                        expression: false,
                        async: node_utils_1.hasModifier(SyntaxKind.AsyncKeyword, node),
                        body: this.convertChild(node.body),
                        range: [node.parameters.pos - 1, node.end],
                        params: [],
                    });
                    if (node.type) {
                        method.returnType = this.convertTypeAnnotation(node.type, node);
                    }
                    // Process typeParameters
                    if (node.typeParameters) {
                        method.typeParameters = this.convertTSTypeParametersToTypeParametersDeclaration(node.typeParameters);
                        this.fixParentLocation(method, method.typeParameters.range);
                    }
                    let result;
                    if (parent.kind === SyntaxKind.ObjectLiteralExpression) {
                        method.params = node.parameters.map(el => this.convertChild(el));
                        result = this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.Property,
                            key: this.convertChild(node.name),
                            value: method,
                            computed: node_utils_1.isComputedProperty(node.name),
                            method: node.kind === SyntaxKind.MethodDeclaration,
                            shorthand: false,
                            kind: 'init',
                        });
                    }
                    else {
                        // class
                        /**
                         * Unlike in object literal methods, class method params can have decorators
                         */
                        method.params = this.convertParameters(node.parameters);
                        /**
                         * TypeScript class methods can be defined as "abstract"
                         */
                        const methodDefinitionType = node_utils_1.hasModifier(SyntaxKind.AbstractKeyword, node)
                            ? ts_estree_1.AST_NODE_TYPES.TSAbstractMethodDefinition
                            : ts_estree_1.AST_NODE_TYPES.MethodDefinition;
                        result = this.createNode(node, {
                            type: methodDefinitionType,
                            key: this.convertChild(node.name),
                            value: method,
                            computed: node_utils_1.isComputedProperty(node.name),
                            static: node_utils_1.hasModifier(SyntaxKind.StaticKeyword, node),
                            kind: 'method',
                        });
                        if (node.decorators) {
                            result.decorators = node.decorators.map(el => this.convertChild(el));
                        }
                        const accessibility = node_utils_1.getTSNodeAccessibility(node);
                        if (accessibility) {
                            result.accessibility = accessibility;
                        }
                    }
                    if (result.key.type === ts_estree_1.AST_NODE_TYPES.Identifier &&
                        node.questionToken) {
                        result.key.optional = true;
                    }
                    if (node.kind === SyntaxKind.GetAccessor) {
                        result.kind = 'get';
                    }
                    else if (node.kind === SyntaxKind.SetAccessor) {
                        result.kind = 'set';
                    }
                    else if (!result.static &&
                        node.name.kind === SyntaxKind.StringLiteral &&
                        node.name.text === 'constructor' &&
                        result.type !== ts_estree_1.AST_NODE_TYPES.Property) {
                        result.kind = 'constructor';
                    }
                    return result;
                }
                // TypeScript uses this even for static methods named "constructor"
                case SyntaxKind.Constructor: {
                    const lastModifier = node_utils_1.getLastModifier(node);
                    const constructorToken = (lastModifier && node_utils_1.findNextToken(lastModifier, node, this.ast)) ||
                        node.getFirstToken();
                    const constructor = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.FunctionExpression,
                        id: null,
                        params: this.convertParameters(node.parameters),
                        generator: false,
                        expression: false,
                        async: false,
                        body: this.convertChild(node.body),
                        range: [node.parameters.pos - 1, node.end],
                    });
                    // Process typeParameters
                    if (node.typeParameters) {
                        constructor.typeParameters = this.convertTSTypeParametersToTypeParametersDeclaration(node.typeParameters);
                        this.fixParentLocation(constructor, constructor.typeParameters.range);
                    }
                    // Process returnType
                    if (node.type) {
                        constructor.returnType = this.convertTypeAnnotation(node.type, node);
                    }
                    const constructorKey = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Identifier,
                        name: 'constructor',
                        range: [constructorToken.getStart(this.ast), constructorToken.end],
                    });
                    const isStatic = node_utils_1.hasModifier(SyntaxKind.StaticKeyword, node);
                    const result = this.createNode(node, {
                        type: node_utils_1.hasModifier(SyntaxKind.AbstractKeyword, node)
                            ? ts_estree_1.AST_NODE_TYPES.TSAbstractMethodDefinition
                            : ts_estree_1.AST_NODE_TYPES.MethodDefinition,
                        key: constructorKey,
                        value: constructor,
                        computed: false,
                        static: isStatic,
                        kind: isStatic ? 'method' : 'constructor',
                    });
                    const accessibility = node_utils_1.getTSNodeAccessibility(node);
                    if (accessibility) {
                        result.accessibility = accessibility;
                    }
                    return result;
                }
                case SyntaxKind.FunctionExpression: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.FunctionExpression,
                        id: this.convertChild(node.name),
                        generator: !!node.asteriskToken,
                        params: this.convertParameters(node.parameters),
                        body: this.convertChild(node.body),
                        async: node_utils_1.hasModifier(SyntaxKind.AsyncKeyword, node),
                        expression: false,
                    });
                    // Process returnType
                    if (node.type) {
                        result.returnType = this.convertTypeAnnotation(node.type, node);
                    }
                    // Process typeParameters
                    if (node.typeParameters) {
                        result.typeParameters = this.convertTSTypeParametersToTypeParametersDeclaration(node.typeParameters);
                    }
                    return result;
                }
                case SyntaxKind.SuperKeyword:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Super,
                    });
                case SyntaxKind.ArrayBindingPattern:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ArrayPattern,
                        elements: node.elements.map(el => this.convertPattern(el)),
                    });
                // occurs with missing array elements like [,]
                case SyntaxKind.OmittedExpression:
                    return null;
                case SyntaxKind.ObjectBindingPattern:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ObjectPattern,
                        properties: node.elements.map(el => this.convertPattern(el)),
                    });
                case SyntaxKind.BindingElement: {
                    if (parent.kind === SyntaxKind.ArrayBindingPattern) {
                        const arrayItem = this.convertChild(node.name, parent);
                        if (node.initializer) {
                            return this.createNode(node, {
                                type: ts_estree_1.AST_NODE_TYPES.AssignmentPattern,
                                left: arrayItem,
                                right: this.convertChild(node.initializer),
                            });
                        }
                        else if (node.dotDotDotToken) {
                            return this.createNode(node, {
                                type: ts_estree_1.AST_NODE_TYPES.RestElement,
                                argument: arrayItem,
                            });
                        }
                        else {
                            return arrayItem;
                        }
                    }
                    else {
                        let result;
                        if (node.dotDotDotToken) {
                            result = this.createNode(node, {
                                type: ts_estree_1.AST_NODE_TYPES.RestElement,
                                argument: this.convertChild((_a = node.propertyName) !== null && _a !== void 0 ? _a : node.name),
                            });
                        }
                        else {
                            result = this.createNode(node, {
                                type: ts_estree_1.AST_NODE_TYPES.Property,
                                key: this.convertChild((_b = node.propertyName) !== null && _b !== void 0 ? _b : node.name),
                                value: this.convertChild(node.name),
                                computed: Boolean(node.propertyName &&
                                    node.propertyName.kind === SyntaxKind.ComputedPropertyName),
                                method: false,
                                shorthand: !node.propertyName,
                                kind: 'init',
                            });
                        }
                        if (node.initializer) {
                            result.value = this.createNode(node, {
                                type: ts_estree_1.AST_NODE_TYPES.AssignmentPattern,
                                left: this.convertChild(node.name),
                                right: this.convertChild(node.initializer),
                                range: [node.name.getStart(this.ast), node.initializer.end],
                            });
                        }
                        return result;
                    }
                }
                case SyntaxKind.ArrowFunction: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ArrowFunctionExpression,
                        generator: false,
                        id: null,
                        params: this.convertParameters(node.parameters),
                        body: this.convertChild(node.body),
                        async: node_utils_1.hasModifier(SyntaxKind.AsyncKeyword, node),
                        expression: node.body.kind !== SyntaxKind.Block,
                    });
                    // Process returnType
                    if (node.type) {
                        result.returnType = this.convertTypeAnnotation(node.type, node);
                    }
                    // Process typeParameters
                    if (node.typeParameters) {
                        result.typeParameters = this.convertTSTypeParametersToTypeParametersDeclaration(node.typeParameters);
                    }
                    return result;
                }
                case SyntaxKind.YieldExpression:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.YieldExpression,
                        delegate: !!node.asteriskToken,
                        argument: this.convertChild(node.expression),
                    });
                case SyntaxKind.AwaitExpression:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.AwaitExpression,
                        argument: this.convertChild(node.expression),
                    });
                // Template Literals
                case SyntaxKind.NoSubstitutionTemplateLiteral:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TemplateLiteral,
                        quasis: [
                            this.createNode(node, {
                                type: ts_estree_1.AST_NODE_TYPES.TemplateElement,
                                value: {
                                    raw: this.ast.text.slice(node.getStart(this.ast) + 1, node.end - 1),
                                    cooked: node.text,
                                },
                                tail: true,
                            }),
                        ],
                        expressions: [],
                    });
                case SyntaxKind.TemplateExpression: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TemplateLiteral,
                        quasis: [this.convertChild(node.head)],
                        expressions: [],
                    });
                    node.templateSpans.forEach(templateSpan => {
                        result.expressions.push(this.convertChild(templateSpan.expression));
                        result.quasis.push(this.convertChild(templateSpan.literal));
                    });
                    return result;
                }
                case SyntaxKind.TaggedTemplateExpression:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TaggedTemplateExpression,
                        typeParameters: node.typeArguments
                            ? this.convertTypeArgumentsToTypeParameters(node.typeArguments, node)
                            : undefined,
                        tag: this.convertChild(node.tag),
                        quasi: this.convertChild(node.template),
                    });
                case SyntaxKind.TemplateHead:
                case SyntaxKind.TemplateMiddle:
                case SyntaxKind.TemplateTail: {
                    const tail = node.kind === SyntaxKind.TemplateTail;
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TemplateElement,
                        value: {
                            raw: this.ast.text.slice(node.getStart(this.ast) + 1, node.end - (tail ? 1 : 2)),
                            cooked: node.text,
                        },
                        tail,
                    });
                }
                // Patterns
                case SyntaxKind.SpreadAssignment:
                case SyntaxKind.SpreadElement: {
                    if (this.allowPattern) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.RestElement,
                            argument: this.convertPattern(node.expression),
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.SpreadElement,
                            argument: this.convertChild(node.expression),
                        });
                    }
                }
                case SyntaxKind.Parameter: {
                    let parameter;
                    let result;
                    if (node.dotDotDotToken) {
                        parameter = result = this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.RestElement,
                            argument: this.convertChild(node.name),
                        });
                    }
                    else if (node.initializer) {
                        parameter = this.convertChild(node.name);
                        result = this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.AssignmentPattern,
                            left: parameter,
                            right: this.convertChild(node.initializer),
                        });
                        if (node.modifiers) {
                            // AssignmentPattern should not contain modifiers in range
                            result.range[0] = parameter.range[0];
                            result.loc = node_utils_1.getLocFor(result.range[0], result.range[1], this.ast);
                        }
                    }
                    else {
                        parameter = result = this.convertChild(node.name, parent);
                    }
                    if (node.type) {
                        parameter.typeAnnotation = this.convertTypeAnnotation(node.type, node);
                        this.fixParentLocation(parameter, parameter.typeAnnotation.range);
                    }
                    if (node.questionToken) {
                        if (node.questionToken.end > parameter.range[1]) {
                            parameter.range[1] = node.questionToken.end;
                            parameter.loc.end = node_utils_1.getLineAndCharacterFor(parameter.range[1], this.ast);
                        }
                        parameter.optional = true;
                    }
                    if (node.modifiers) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.TSParameterProperty,
                            accessibility: (_c = node_utils_1.getTSNodeAccessibility(node)) !== null && _c !== void 0 ? _c : undefined,
                            readonly: node_utils_1.hasModifier(SyntaxKind.ReadonlyKeyword, node) || undefined,
                            static: node_utils_1.hasModifier(SyntaxKind.StaticKeyword, node) || undefined,
                            export: node_utils_1.hasModifier(SyntaxKind.ExportKeyword, node) || undefined,
                            parameter: result,
                        });
                    }
                    return result;
                }
                // Classes
                case SyntaxKind.ClassDeclaration:
                case SyntaxKind.ClassExpression: {
                    const heritageClauses = (_d = node.heritageClauses) !== null && _d !== void 0 ? _d : [];
                    const classNodeType = node.kind === SyntaxKind.ClassDeclaration
                        ? ts_estree_1.AST_NODE_TYPES.ClassDeclaration
                        : ts_estree_1.AST_NODE_TYPES.ClassExpression;
                    const superClass = heritageClauses.find(clause => clause.token === SyntaxKind.ExtendsKeyword);
                    const implementsClause = heritageClauses.find(clause => clause.token === SyntaxKind.ImplementsKeyword);
                    const result = this.createNode(node, {
                        type: classNodeType,
                        id: this.convertChild(node.name),
                        body: this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.ClassBody,
                            body: [],
                            range: [node.members.pos - 1, node.end],
                        }),
                        superClass: (superClass === null || superClass === void 0 ? void 0 : superClass.types[0]) ? this.convertChild(superClass.types[0].expression)
                            : null,
                    });
                    if (superClass) {
                        if (superClass.types.length > 1) {
                            throw node_utils_1.createError(this.ast, superClass.types[1].pos, 'Classes can only extend a single class.');
                        }
                        if (superClass.types[0] && superClass.types[0].typeArguments) {
                            result.superTypeParameters = this.convertTypeArgumentsToTypeParameters(superClass.types[0].typeArguments, superClass.types[0]);
                        }
                    }
                    if (node.typeParameters) {
                        result.typeParameters = this.convertTSTypeParametersToTypeParametersDeclaration(node.typeParameters);
                    }
                    if (implementsClause) {
                        result.implements = implementsClause.types.map(el => this.convertChild(el));
                    }
                    /**
                     * TypeScript class declarations can be defined as "abstract"
                     */
                    if (node_utils_1.hasModifier(SyntaxKind.AbstractKeyword, node)) {
                        result.abstract = true;
                    }
                    if (node_utils_1.hasModifier(SyntaxKind.DeclareKeyword, node)) {
                        result.declare = true;
                    }
                    if (node.decorators) {
                        result.decorators = node.decorators.map(el => this.convertChild(el));
                    }
                    const filteredMembers = node.members.filter(node_utils_1.isESTreeClassMember);
                    if (filteredMembers.length) {
                        result.body.body = filteredMembers.map(el => this.convertChild(el));
                    }
                    // check for exports
                    return this.fixExports(node, result);
                }
                // Modules
                case SyntaxKind.ModuleBlock:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSModuleBlock,
                        body: this.convertBodyExpressions(node.statements, node),
                    });
                case SyntaxKind.ImportDeclaration: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ImportDeclaration,
                        source: this.convertChild(node.moduleSpecifier),
                        specifiers: [],
                        importKind: 'value',
                    });
                    if (node.importClause) {
                        if (node.importClause.isTypeOnly) {
                            result.importKind = 'type';
                        }
                        if (node.importClause.name) {
                            result.specifiers.push(this.convertChild(node.importClause));
                        }
                        if (node.importClause.namedBindings) {
                            switch (node.importClause.namedBindings.kind) {
                                case SyntaxKind.NamespaceImport:
                                    result.specifiers.push(this.convertChild(node.importClause.namedBindings));
                                    break;
                                case SyntaxKind.NamedImports:
                                    result.specifiers = result.specifiers.concat(node.importClause.namedBindings.elements.map(el => this.convertChild(el)));
                                    break;
                            }
                        }
                    }
                    return result;
                }
                case SyntaxKind.NamespaceImport:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ImportNamespaceSpecifier,
                        local: this.convertChild(node.name),
                    });
                case SyntaxKind.ImportSpecifier:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ImportSpecifier,
                        local: this.convertChild(node.name),
                        imported: this.convertChild((_e = node.propertyName) !== null && _e !== void 0 ? _e : node.name),
                    });
                case SyntaxKind.ImportClause:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ImportDefaultSpecifier,
                        local: this.convertChild(node.name),
                        range: [node.getStart(this.ast), node.name.end],
                    });
                case SyntaxKind.ExportDeclaration:
                    if (((_f = node.exportClause) === null || _f === void 0 ? void 0 : _f.kind) === SyntaxKind.NamedExports) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.ExportNamedDeclaration,
                            source: this.convertChild(node.moduleSpecifier),
                            specifiers: node.exportClause.elements.map(el => this.convertChild(el)),
                            exportKind: node.isTypeOnly ? 'type' : 'value',
                            declaration: null,
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.ExportAllDeclaration,
                            source: this.convertChild(node.moduleSpecifier),
                            exportKind: node.isTypeOnly ? 'type' : 'value',
                            exported: ((_g = node.exportClause) === null || _g === void 0 ? void 0 : _g.kind) === SyntaxKind.NamespaceExport
                                ? this.convertChild(node.exportClause.name)
                                : null,
                        });
                    }
                case SyntaxKind.ExportSpecifier:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ExportSpecifier,
                        local: this.convertChild((_h = node.propertyName) !== null && _h !== void 0 ? _h : node.name),
                        exported: this.convertChild(node.name),
                    });
                case SyntaxKind.ExportAssignment:
                    if (node.isExportEquals) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.TSExportAssignment,
                            expression: this.convertChild(node.expression),
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.ExportDefaultDeclaration,
                            declaration: this.convertChild(node.expression),
                        });
                    }
                // Unary Operations
                case SyntaxKind.PrefixUnaryExpression:
                case SyntaxKind.PostfixUnaryExpression: {
                    const operator = node_utils_1.getTextForTokenKind(node.operator);
                    /**
                     * ESTree uses UpdateExpression for ++/--
                     */
                    if (operator === '++' || operator === '--') {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.UpdateExpression,
                            operator,
                            prefix: node.kind === SyntaxKind.PrefixUnaryExpression,
                            argument: this.convertChild(node.operand),
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.UnaryExpression,
                            operator,
                            prefix: node.kind === SyntaxKind.PrefixUnaryExpression,
                            argument: this.convertChild(node.operand),
                        });
                    }
                }
                case SyntaxKind.DeleteExpression:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.UnaryExpression,
                        operator: 'delete',
                        prefix: true,
                        argument: this.convertChild(node.expression),
                    });
                case SyntaxKind.VoidExpression:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.UnaryExpression,
                        operator: 'void',
                        prefix: true,
                        argument: this.convertChild(node.expression),
                    });
                case SyntaxKind.TypeOfExpression:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.UnaryExpression,
                        operator: 'typeof',
                        prefix: true,
                        argument: this.convertChild(node.expression),
                    });
                case SyntaxKind.TypeOperator:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSTypeOperator,
                        operator: node_utils_1.getTextForTokenKind(node.operator),
                        typeAnnotation: this.convertChild(node.type),
                    });
                // Binary Operations
                case SyntaxKind.BinaryExpression: {
                    // TypeScript uses BinaryExpression for sequences as well
                    if (node_utils_1.isComma(node.operatorToken)) {
                        const result = this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.SequenceExpression,
                            expressions: [],
                        });
                        const left = this.convertChild(node.left);
                        if (left.type === ts_estree_1.AST_NODE_TYPES.SequenceExpression &&
                            node.left.kind !== SyntaxKind.ParenthesizedExpression) {
                            result.expressions = result.expressions.concat(left.expressions);
                        }
                        else {
                            result.expressions.push(left);
                        }
                        result.expressions.push(this.convertChild(node.right));
                        return result;
                    }
                    else {
                        const type = node_utils_1.getBinaryExpressionType(node.operatorToken);
                        if (this.allowPattern &&
                            type === ts_estree_1.AST_NODE_TYPES.AssignmentExpression) {
                            return this.createNode(node, {
                                type: ts_estree_1.AST_NODE_TYPES.AssignmentPattern,
                                left: this.convertPattern(node.left, node),
                                right: this.convertChild(node.right),
                            });
                        }
                        return this.createNode(node, {
                            type,
                            operator: node_utils_1.getTextForTokenKind(node.operatorToken.kind),
                            left: this.converter(node.left, node, this.inTypeMode, type === ts_estree_1.AST_NODE_TYPES.AssignmentExpression),
                            right: this.convertChild(node.right),
                        });
                    }
                }
                case SyntaxKind.PropertyAccessExpression: {
                    const object = this.convertChild(node.expression);
                    const property = this.convertChild(node.name);
                    const computed = false;
                    const isLocallyOptional = node.questionDotToken !== undefined;
                    // the optional expression should propagate up the member expression tree
                    const isChildOptional = (object.type === ts_estree_1.AST_NODE_TYPES.OptionalMemberExpression ||
                        object.type === ts_estree_1.AST_NODE_TYPES.OptionalCallExpression) &&
                        // (x?.y).z is semantically different, and as such .z is no longer optional
                        node.expression.kind !== ts.SyntaxKind.ParenthesizedExpression;
                    if (isLocallyOptional || isChildOptional) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.OptionalMemberExpression,
                            object,
                            property,
                            computed,
                            optional: isLocallyOptional,
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.MemberExpression,
                            object,
                            property,
                            computed,
                            optional: false,
                        });
                    }
                }
                case SyntaxKind.ElementAccessExpression: {
                    const object = this.convertChild(node.expression);
                    const property = this.convertChild(node.argumentExpression);
                    const computed = true;
                    const isLocallyOptional = node.questionDotToken !== undefined;
                    // the optional expression should propagate up the member expression tree
                    const isChildOptional = (object.type === ts_estree_1.AST_NODE_TYPES.OptionalMemberExpression ||
                        object.type === ts_estree_1.AST_NODE_TYPES.OptionalCallExpression) &&
                        // (x?.y).z is semantically different, and as such .z is no longer optional
                        node.expression.kind !== ts.SyntaxKind.ParenthesizedExpression;
                    if (isLocallyOptional || isChildOptional) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.OptionalMemberExpression,
                            object,
                            property,
                            computed,
                            optional: isLocallyOptional,
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.MemberExpression,
                            object,
                            property,
                            computed,
                            optional: false,
                        });
                    }
                }
                case SyntaxKind.CallExpression: {
                    const callee = this.convertChild(node.expression);
                    const args = node.arguments.map(el => this.convertChild(el));
                    let result;
                    const isLocallyOptional = node.questionDotToken !== undefined;
                    // the optional expression should propagate up the member expression tree
                    const isChildOptional = (callee.type === ts_estree_1.AST_NODE_TYPES.OptionalMemberExpression ||
                        callee.type === ts_estree_1.AST_NODE_TYPES.OptionalCallExpression) &&
                        // (x?.y).z() is semantically different, and as such .z() is no longer optional
                        node.expression.kind !== ts.SyntaxKind.ParenthesizedExpression;
                    if (isLocallyOptional || isChildOptional) {
                        result = this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.OptionalCallExpression,
                            callee,
                            arguments: args,
                            optional: isLocallyOptional,
                        });
                    }
                    else {
                        result = this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.CallExpression,
                            callee,
                            arguments: args,
                            optional: false,
                        });
                    }
                    if (node.typeArguments) {
                        result.typeParameters = this.convertTypeArgumentsToTypeParameters(node.typeArguments, node);
                    }
                    return result;
                }
                case SyntaxKind.NewExpression: {
                    // NOTE - NewExpression cannot have an optional chain in it
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.NewExpression,
                        callee: this.convertChild(node.expression),
                        arguments: node.arguments
                            ? node.arguments.map(el => this.convertChild(el))
                            : [],
                    });
                    if (node.typeArguments) {
                        result.typeParameters = this.convertTypeArgumentsToTypeParameters(node.typeArguments, node);
                    }
                    return result;
                }
                case SyntaxKind.ConditionalExpression:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.ConditionalExpression,
                        test: this.convertChild(node.condition),
                        consequent: this.convertChild(node.whenTrue),
                        alternate: this.convertChild(node.whenFalse),
                    });
                case SyntaxKind.MetaProperty: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.MetaProperty,
                        meta: this.createNode(
                        // TODO: do we really want to convert it to Token?
                        node.getFirstToken(), {
                            type: ts_estree_1.AST_NODE_TYPES.Identifier,
                            name: node_utils_1.getTextForTokenKind(node.keywordToken),
                        }),
                        property: this.convertChild(node.name),
                    });
                }
                case SyntaxKind.Decorator: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Decorator,
                        expression: this.convertChild(node.expression),
                    });
                }
                // Literals
                case SyntaxKind.StringLiteral: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Literal,
                        raw: '',
                        value: '',
                    });
                    result.raw = this.ast.text.slice(result.range[0], result.range[1]);
                    if ('name' in parent && parent.name === node) {
                        result.value = node.text;
                    }
                    else {
                        result.value = node_utils_1.unescapeStringLiteralText(node.text);
                    }
                    return result;
                }
                case SyntaxKind.NumericLiteral: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Literal,
                        value: Number(node.text),
                        raw: node.getText(),
                    });
                }
                case SyntaxKind.BigIntLiteral: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.BigIntLiteral,
                        raw: '',
                        value: '',
                    });
                    result.raw = this.ast.text.slice(result.range[0], result.range[1]);
                    result.value = result.raw.slice(0, -1); // remove suffix `n`
                    return result;
                }
                case SyntaxKind.RegularExpressionLiteral: {
                    const pattern = node.text.slice(1, node.text.lastIndexOf('/'));
                    const flags = node.text.slice(node.text.lastIndexOf('/') + 1);
                    let regex = null;
                    try {
                        regex = new RegExp(pattern, flags);
                    }
                    catch (exception) {
                        regex = null;
                    }
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Literal,
                        value: regex,
                        raw: node.text,
                        regex: {
                            pattern,
                            flags,
                        },
                    });
                }
                case SyntaxKind.TrueKeyword:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Literal,
                        value: true,
                        raw: 'true',
                    });
                case SyntaxKind.FalseKeyword:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Literal,
                        value: false,
                        raw: 'false',
                    });
                case SyntaxKind.NullKeyword: {
                    if (this.inTypeMode) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.TSNullKeyword,
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.Literal,
                            value: null,
                            raw: 'null',
                        });
                    }
                }
                case SyntaxKind.ImportKeyword:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.Import,
                    });
                case SyntaxKind.EmptyStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.EmptyStatement,
                    });
                case SyntaxKind.DebuggerStatement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.DebuggerStatement,
                    });
                // JSX
                case SyntaxKind.JsxElement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXElement,
                        openingElement: this.convertChild(node.openingElement),
                        closingElement: this.convertChild(node.closingElement),
                        children: node.children.map(el => this.convertChild(el)),
                    });
                case SyntaxKind.JsxFragment:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXFragment,
                        openingFragment: this.convertChild(node.openingFragment),
                        closingFragment: this.convertChild(node.closingFragment),
                        children: node.children.map(el => this.convertChild(el)),
                    });
                case SyntaxKind.JsxSelfClosingElement: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXElement,
                        /**
                         * Convert SyntaxKind.JsxSelfClosingElement to SyntaxKind.JsxOpeningElement,
                         * TypeScript does not seem to have the idea of openingElement when tag is self-closing
                         */
                        openingElement: this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.JSXOpeningElement,
                            typeParameters: node.typeArguments
                                ? this.convertTypeArgumentsToTypeParameters(node.typeArguments, node)
                                : undefined,
                            selfClosing: true,
                            name: this.convertJSXTagName(node.tagName, node),
                            attributes: node.attributes.properties.map(el => this.convertChild(el)),
                            range: node_utils_1.getRange(node, this.ast),
                        }),
                        closingElement: null,
                        children: [],
                    });
                }
                case SyntaxKind.JsxOpeningElement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXOpeningElement,
                        typeParameters: node.typeArguments
                            ? this.convertTypeArgumentsToTypeParameters(node.typeArguments, node)
                            : undefined,
                        selfClosing: false,
                        name: this.convertJSXTagName(node.tagName, node),
                        attributes: node.attributes.properties.map(el => this.convertChild(el)),
                    });
                case SyntaxKind.JsxClosingElement:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXClosingElement,
                        name: this.convertJSXTagName(node.tagName, node),
                    });
                case SyntaxKind.JsxOpeningFragment:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXOpeningFragment,
                    });
                case SyntaxKind.JsxClosingFragment:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXClosingFragment,
                    });
                case SyntaxKind.JsxExpression: {
                    const expression = node.expression
                        ? this.convertChild(node.expression)
                        : this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.JSXEmptyExpression,
                            range: [node.getStart(this.ast) + 1, node.getEnd() - 1],
                        });
                    if (node.dotDotDotToken) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.JSXSpreadChild,
                            expression,
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.JSXExpressionContainer,
                            expression,
                        });
                    }
                }
                case SyntaxKind.JsxAttribute: {
                    const attributeName = this.convertChild(node.name);
                    attributeName.type = ts_estree_1.AST_NODE_TYPES.JSXIdentifier;
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXAttribute,
                        name: attributeName,
                        value: this.convertChild(node.initializer),
                    });
                }
                /**
                 * The JSX AST changed the node type for string literals
                 * inside a JSX Element from `Literal` to `JSXText`. We
                 * provide a flag to support both types until `Literal`
                 * node type is deprecated in ESLint v5.
                 */
                case SyntaxKind.JsxText: {
                    const start = node.getFullStart();
                    const end = node.getEnd();
                    if (this.options.useJSXTextNode) {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.JSXText,
                            value: this.ast.text.slice(start, end),
                            raw: this.ast.text.slice(start, end),
                            range: [start, end],
                        });
                    }
                    else {
                        return this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.Literal,
                            value: this.ast.text.slice(start, end),
                            raw: this.ast.text.slice(start, end),
                            range: [start, end],
                        });
                    }
                }
                case SyntaxKind.JsxSpreadAttribute:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.JSXSpreadAttribute,
                        argument: this.convertChild(node.expression),
                    });
                case SyntaxKind.QualifiedName: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSQualifiedName,
                        left: this.convertChild(node.left),
                        right: this.convertChild(node.right),
                    });
                }
                // TypeScript specific
                case SyntaxKind.TypeReference: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSTypeReference,
                        typeName: this.convertType(node.typeName),
                        typeParameters: node.typeArguments
                            ? this.convertTypeArgumentsToTypeParameters(node.typeArguments, node)
                            : undefined,
                    });
                }
                case SyntaxKind.TypeParameter: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSTypeParameter,
                        name: this.convertType(node.name),
                        constraint: node.constraint
                            ? this.convertType(node.constraint)
                            : undefined,
                        default: node.default ? this.convertType(node.default) : undefined,
                    });
                }
                case SyntaxKind.ThisType:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSThisType,
                    });
                case SyntaxKind.AnyKeyword:
                case SyntaxKind.BigIntKeyword:
                case SyntaxKind.BooleanKeyword:
                case SyntaxKind.NeverKeyword:
                case SyntaxKind.NumberKeyword:
                case SyntaxKind.ObjectKeyword:
                case SyntaxKind.StringKeyword:
                case SyntaxKind.SymbolKeyword:
                case SyntaxKind.UnknownKeyword:
                case SyntaxKind.VoidKeyword:
                case SyntaxKind.UndefinedKeyword: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES[`TS${SyntaxKind[node.kind]}`],
                    });
                }
                case SyntaxKind.NonNullExpression: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSNonNullExpression,
                        expression: this.convertChild(node.expression),
                    });
                }
                case SyntaxKind.TypeLiteral: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSTypeLiteral,
                        members: node.members.map(el => this.convertChild(el)),
                    });
                }
                case SyntaxKind.ArrayType: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSArrayType,
                        elementType: this.convertType(node.elementType),
                    });
                }
                case SyntaxKind.IndexedAccessType: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSIndexedAccessType,
                        objectType: this.convertType(node.objectType),
                        indexType: this.convertType(node.indexType),
                    });
                }
                case SyntaxKind.ConditionalType: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSConditionalType,
                        checkType: this.convertType(node.checkType),
                        extendsType: this.convertType(node.extendsType),
                        trueType: this.convertType(node.trueType),
                        falseType: this.convertType(node.falseType),
                    });
                }
                case SyntaxKind.TypeQuery: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSTypeQuery,
                        exprName: this.convertType(node.exprName),
                    });
                }
                case SyntaxKind.MappedType: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSMappedType,
                        typeParameter: this.convertType(node.typeParameter),
                    });
                    if (node.readonlyToken) {
                        if (node.readonlyToken.kind === SyntaxKind.ReadonlyKeyword) {
                            result.readonly = true;
                        }
                        else {
                            result.readonly = node_utils_1.getTextForTokenKind(node.readonlyToken.kind);
                        }
                    }
                    if (node.questionToken) {
                        if (node.questionToken.kind === SyntaxKind.QuestionToken) {
                            result.optional = true;
                        }
                        else {
                            result.optional = node_utils_1.getTextForTokenKind(node.questionToken.kind);
                        }
                    }
                    if (node.type) {
                        result.typeAnnotation = this.convertType(node.type);
                    }
                    return result;
                }
                case SyntaxKind.ParenthesizedExpression:
                    return this.convertChild(node.expression, parent);
                case SyntaxKind.TypeAliasDeclaration: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSTypeAliasDeclaration,
                        id: this.convertChild(node.name),
                        typeAnnotation: this.convertType(node.type),
                    });
                    if (node_utils_1.hasModifier(SyntaxKind.DeclareKeyword, node)) {
                        result.declare = true;
                    }
                    // Process typeParameters
                    if (node.typeParameters) {
                        result.typeParameters = this.convertTSTypeParametersToTypeParametersDeclaration(node.typeParameters);
                    }
                    // check for exports
                    return this.fixExports(node, result);
                }
                case SyntaxKind.MethodSignature: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSMethodSignature,
                        computed: node_utils_1.isComputedProperty(node.name),
                        key: this.convertChild(node.name),
                        params: this.convertParameters(node.parameters),
                    });
                    if (node_utils_1.isOptional(node)) {
                        result.optional = true;
                    }
                    if (node.type) {
                        result.returnType = this.convertTypeAnnotation(node.type, node);
                    }
                    if (node_utils_1.hasModifier(SyntaxKind.ReadonlyKeyword, node)) {
                        result.readonly = true;
                    }
                    if (node.typeParameters) {
                        result.typeParameters = this.convertTSTypeParametersToTypeParametersDeclaration(node.typeParameters);
                    }
                    const accessibility = node_utils_1.getTSNodeAccessibility(node);
                    if (accessibility) {
                        result.accessibility = accessibility;
                    }
                    if (node_utils_1.hasModifier(SyntaxKind.ExportKeyword, node)) {
                        result.export = true;
                    }
                    if (node_utils_1.hasModifier(SyntaxKind.StaticKeyword, node)) {
                        result.static = true;
                    }
                    return result;
                }
                case SyntaxKind.PropertySignature: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSPropertySignature,
                        optional: node_utils_1.isOptional(node) || undefined,
                        computed: node_utils_1.isComputedProperty(node.name),
                        key: this.convertChild(node.name),
                        typeAnnotation: node.type
                            ? this.convertTypeAnnotation(node.type, node)
                            : undefined,
                        initializer: this.convertChild(node.initializer) || undefined,
                        readonly: node_utils_1.hasModifier(SyntaxKind.ReadonlyKeyword, node) || undefined,
                        static: node_utils_1.hasModifier(SyntaxKind.StaticKeyword, node) || undefined,
                        export: node_utils_1.hasModifier(SyntaxKind.ExportKeyword, node) || undefined,
                    });
                    const accessibility = node_utils_1.getTSNodeAccessibility(node);
                    if (accessibility) {
                        result.accessibility = accessibility;
                    }
                    return result;
                }
                case SyntaxKind.IndexSignature: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSIndexSignature,
                        parameters: node.parameters.map(el => this.convertChild(el)),
                    });
                    if (node.type) {
                        result.typeAnnotation = this.convertTypeAnnotation(node.type, node);
                    }
                    if (node_utils_1.hasModifier(SyntaxKind.ReadonlyKeyword, node)) {
                        result.readonly = true;
                    }
                    const accessibility = node_utils_1.getTSNodeAccessibility(node);
                    if (accessibility) {
                        result.accessibility = accessibility;
                    }
                    if (node_utils_1.hasModifier(SyntaxKind.ExportKeyword, node)) {
                        result.export = true;
                    }
                    if (node_utils_1.hasModifier(SyntaxKind.StaticKeyword, node)) {
                        result.static = true;
                    }
                    return result;
                }
                case SyntaxKind.ConstructorType:
                case SyntaxKind.FunctionType:
                case SyntaxKind.ConstructSignature:
                case SyntaxKind.CallSignature: {
                    let type;
                    switch (node.kind) {
                        case SyntaxKind.ConstructSignature:
                            type = ts_estree_1.AST_NODE_TYPES.TSConstructSignatureDeclaration;
                            break;
                        case SyntaxKind.CallSignature:
                            type = ts_estree_1.AST_NODE_TYPES.TSCallSignatureDeclaration;
                            break;
                        case SyntaxKind.FunctionType:
                            type = ts_estree_1.AST_NODE_TYPES.TSFunctionType;
                            break;
                        case SyntaxKind.ConstructorType:
                        default:
                            type = ts_estree_1.AST_NODE_TYPES.TSConstructorType;
                            break;
                    }
                    const result = this.createNode(node, {
                        type: type,
                        params: this.convertParameters(node.parameters),
                    });
                    if (node.type) {
                        result.returnType = this.convertTypeAnnotation(node.type, node);
                    }
                    if (node.typeParameters) {
                        result.typeParameters = this.convertTSTypeParametersToTypeParametersDeclaration(node.typeParameters);
                    }
                    return result;
                }
                case SyntaxKind.ExpressionWithTypeArguments: {
                    const result = this.createNode(node, {
                        type: parent && parent.kind === SyntaxKind.InterfaceDeclaration
                            ? ts_estree_1.AST_NODE_TYPES.TSInterfaceHeritage
                            : ts_estree_1.AST_NODE_TYPES.TSClassImplements,
                        expression: this.convertChild(node.expression),
                    });
                    if (node.typeArguments) {
                        result.typeParameters = this.convertTypeArgumentsToTypeParameters(node.typeArguments, node);
                    }
                    return result;
                }
                case SyntaxKind.InterfaceDeclaration: {
                    const interfaceHeritageClauses = (_j = node.heritageClauses) !== null && _j !== void 0 ? _j : [];
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSInterfaceDeclaration,
                        body: this.createNode(node, {
                            type: ts_estree_1.AST_NODE_TYPES.TSInterfaceBody,
                            body: node.members.map(member => this.convertChild(member)),
                            range: [node.members.pos - 1, node.end],
                        }),
                        id: this.convertChild(node.name),
                    });
                    if (node.typeParameters) {
                        result.typeParameters = this.convertTSTypeParametersToTypeParametersDeclaration(node.typeParameters);
                    }
                    if (interfaceHeritageClauses.length > 0) {
                        const interfaceExtends = [];
                        const interfaceImplements = [];
                        for (const heritageClause of interfaceHeritageClauses) {
                            if (heritageClause.token === SyntaxKind.ExtendsKeyword) {
                                for (const n of heritageClause.types) {
                                    interfaceExtends.push(this.convertChild(n, node));
                                }
                            }
                            else {
                                for (const n of heritageClause.types) {
                                    interfaceImplements.push(this.convertChild(n, node));
                                }
                            }
                        }
                        if (interfaceExtends.length) {
                            result.extends = interfaceExtends;
                        }
                        if (interfaceImplements.length) {
                            result.implements = interfaceImplements;
                        }
                    }
                    /**
                     * Semantically, decorators are not allowed on interface declarations,
                     * but the TypeScript compiler will parse them and produce a valid AST,
                     * so we handle them here too.
                     */
                    if (node.decorators) {
                        result.decorators = node.decorators.map(el => this.convertChild(el));
                    }
                    if (node_utils_1.hasModifier(SyntaxKind.AbstractKeyword, node)) {
                        result.abstract = true;
                    }
                    if (node_utils_1.hasModifier(SyntaxKind.DeclareKeyword, node)) {
                        result.declare = true;
                    }
                    // check for exports
                    return this.fixExports(node, result);
                }
                case SyntaxKind.TypePredicate: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSTypePredicate,
                        asserts: node.assertsModifier !== undefined,
                        parameterName: this.convertChild(node.parameterName),
                        typeAnnotation: null,
                    });
                    /**
                     * Specific fix for type-guard location data
                     */
                    if (node.type) {
                        result.typeAnnotation = this.convertTypeAnnotation(node.type, node);
                        result.typeAnnotation.loc = result.typeAnnotation.typeAnnotation.loc;
                        result.typeAnnotation.range =
                            result.typeAnnotation.typeAnnotation.range;
                    }
                    return result;
                }
                case SyntaxKind.ImportType:
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSImportType,
                        isTypeOf: !!node.isTypeOf,
                        parameter: this.convertChild(node.argument),
                        qualifier: this.convertChild(node.qualifier),
                        typeParameters: node.typeArguments
                            ? this.convertTypeArgumentsToTypeParameters(node.typeArguments, node)
                            : null,
                    });
                case SyntaxKind.EnumDeclaration: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSEnumDeclaration,
                        id: this.convertChild(node.name),
                        members: node.members.map(el => this.convertChild(el)),
                    });
                    // apply modifiers first...
                    this.applyModifiersToResult(result, node.modifiers);
                    /**
                     * Semantically, decorators are not allowed on enum declarations,
                     * but the TypeScript compiler will parse them and produce a valid AST,
                     * so we handle them here too.
                     */
                    if (node.decorators) {
                        result.decorators = node.decorators.map(el => this.convertChild(el));
                    }
                    // ...then check for exports
                    return this.fixExports(node, result);
                }
                case SyntaxKind.EnumMember: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSEnumMember,
                        id: this.convertChild(node.name),
                    });
                    if (node.initializer) {
                        result.initializer = this.convertChild(node.initializer);
                    }
                    if (node.name.kind === ts.SyntaxKind.ComputedPropertyName) {
                        result.computed = true;
                    }
                    return result;
                }
                case SyntaxKind.ModuleDeclaration: {
                    const result = this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSModuleDeclaration,
                        id: this.convertChild(node.name),
                    });
                    if (node.body) {
                        result.body = this.convertChild(node.body);
                    }
                    // apply modifiers first...
                    this.applyModifiersToResult(result, node.modifiers);
                    if (node.flags & ts.NodeFlags.GlobalAugmentation) {
                        result.global = true;
                    }
                    // ...then check for exports
                    return this.fixExports(node, result);
                }
                // TypeScript specific types
                case SyntaxKind.OptionalType: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSOptionalType,
                        typeAnnotation: this.convertType(node.type),
                    });
                }
                case SyntaxKind.ParenthesizedType: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSParenthesizedType,
                        typeAnnotation: this.convertType(node.type),
                    });
                }
                case SyntaxKind.TupleType: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSTupleType,
                        elementTypes: node.elementTypes.map(el => this.convertType(el)),
                    });
                }
                case SyntaxKind.UnionType: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSUnionType,
                        types: node.types.map(el => this.convertType(el)),
                    });
                }
                case SyntaxKind.IntersectionType: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSIntersectionType,
                        types: node.types.map(el => this.convertType(el)),
                    });
                }
                case SyntaxKind.RestType: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSRestType,
                        typeAnnotation: this.convertType(node.type),
                    });
                }
                case SyntaxKind.AsExpression: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSAsExpression,
                        expression: this.convertChild(node.expression),
                        typeAnnotation: this.convertType(node.type),
                    });
                }
                case SyntaxKind.InferType: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSInferType,
                        typeParameter: this.convertType(node.typeParameter),
                    });
                }
                case SyntaxKind.LiteralType: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSLiteralType,
                        literal: this.convertType(node.literal),
                    });
                }
                case SyntaxKind.TypeAssertionExpression: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSTypeAssertion,
                        typeAnnotation: this.convertType(node.type),
                        expression: this.convertChild(node.expression),
                    });
                }
                case SyntaxKind.ImportEqualsDeclaration: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSImportEqualsDeclaration,
                        id: this.convertChild(node.name),
                        moduleReference: this.convertChild(node.moduleReference),
                        isExport: node_utils_1.hasModifier(SyntaxKind.ExportKeyword, node),
                    });
                }
                case SyntaxKind.ExternalModuleReference: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSExternalModuleReference,
                        expression: this.convertChild(node.expression),
                    });
                }
                case SyntaxKind.NamespaceExportDeclaration: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSNamespaceExportDeclaration,
                        id: this.convertChild(node.name),
                    });
                }
                case SyntaxKind.AbstractKeyword: {
                    return this.createNode(node, {
                        type: ts_estree_1.AST_NODE_TYPES.TSAbstractKeyword,
                    });
                }
                default:
                    return this.deeplyCopy(node);
            }
        }
    }
    exports.Converter = Converter;
},
/* @typescript-eslint/typescript-estree/dist/node-utils.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const unescape_1 = __importDefault(require(793) /* lodash/unescape */);
    const ts = __importStar(require(44) /* typescript */);
    const ts_estree_1 = require(619) /* ./ts-estree */;
    const SyntaxKind = ts.SyntaxKind;
    const ASSIGNMENT_OPERATORS = [
        SyntaxKind.EqualsToken,
        SyntaxKind.PlusEqualsToken,
        SyntaxKind.MinusEqualsToken,
        SyntaxKind.AsteriskEqualsToken,
        SyntaxKind.AsteriskAsteriskEqualsToken,
        SyntaxKind.SlashEqualsToken,
        SyntaxKind.PercentEqualsToken,
        SyntaxKind.LessThanLessThanEqualsToken,
        SyntaxKind.GreaterThanGreaterThanEqualsToken,
        SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken,
        SyntaxKind.AmpersandEqualsToken,
        SyntaxKind.BarEqualsToken,
        SyntaxKind.CaretEqualsToken,
    ];
    const LOGICAL_OPERATORS = [
        SyntaxKind.BarBarToken,
        SyntaxKind.AmpersandAmpersandToken,
        SyntaxKind.QuestionQuestionToken,
    ];
    const TOKEN_TO_TEXT = {
        [SyntaxKind.OpenBraceToken]: '{',
        [SyntaxKind.CloseBraceToken]: '}',
        [SyntaxKind.OpenParenToken]: '(',
        [SyntaxKind.CloseParenToken]: ')',
        [SyntaxKind.OpenBracketToken]: '[',
        [SyntaxKind.CloseBracketToken]: ']',
        [SyntaxKind.DotToken]: '.',
        [SyntaxKind.DotDotDotToken]: '...',
        [SyntaxKind.SemicolonToken]: ';',
        [SyntaxKind.CommaToken]: ',',
        [SyntaxKind.LessThanToken]: '<',
        [SyntaxKind.GreaterThanToken]: '>',
        [SyntaxKind.LessThanEqualsToken]: '<=',
        [SyntaxKind.GreaterThanEqualsToken]: '>=',
        [SyntaxKind.EqualsEqualsToken]: '==',
        [SyntaxKind.ExclamationEqualsToken]: '!=',
        [SyntaxKind.EqualsEqualsEqualsToken]: '===',
        [SyntaxKind.InstanceOfKeyword]: 'instanceof',
        [SyntaxKind.ExclamationEqualsEqualsToken]: '!==',
        [SyntaxKind.EqualsGreaterThanToken]: '=>',
        [SyntaxKind.PlusToken]: '+',
        [SyntaxKind.MinusToken]: '-',
        [SyntaxKind.AsteriskToken]: '*',
        [SyntaxKind.AsteriskAsteriskToken]: '**',
        [SyntaxKind.SlashToken]: '/',
        [SyntaxKind.PercentToken]: '%',
        [SyntaxKind.PlusPlusToken]: '++',
        [SyntaxKind.MinusMinusToken]: '--',
        [SyntaxKind.LessThanLessThanToken]: '<<',
        [SyntaxKind.LessThanSlashToken]: '</',
        [SyntaxKind.GreaterThanGreaterThanToken]: '>>',
        [SyntaxKind.GreaterThanGreaterThanGreaterThanToken]: '>>>',
        [SyntaxKind.AmpersandToken]: '&',
        [SyntaxKind.BarToken]: '|',
        [SyntaxKind.CaretToken]: '^',
        [SyntaxKind.ExclamationToken]: '!',
        [SyntaxKind.TildeToken]: '~',
        [SyntaxKind.AmpersandAmpersandToken]: '&&',
        [SyntaxKind.BarBarToken]: '||',
        [SyntaxKind.QuestionToken]: '?',
        [SyntaxKind.ColonToken]: ':',
        [SyntaxKind.EqualsToken]: '=',
        [SyntaxKind.PlusEqualsToken]: '+=',
        [SyntaxKind.MinusEqualsToken]: '-=',
        [SyntaxKind.AsteriskEqualsToken]: '*=',
        [SyntaxKind.AsteriskAsteriskEqualsToken]: '**=',
        [SyntaxKind.SlashEqualsToken]: '/=',
        [SyntaxKind.PercentEqualsToken]: '%=',
        [SyntaxKind.LessThanLessThanEqualsToken]: '<<=',
        [SyntaxKind.GreaterThanGreaterThanEqualsToken]: '>>=',
        [SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken]: '>>>=',
        [SyntaxKind.AmpersandEqualsToken]: '&=',
        [SyntaxKind.BarEqualsToken]: '|=',
        [SyntaxKind.CaretEqualsToken]: '^=',
        [SyntaxKind.AtToken]: '@',
        [SyntaxKind.InKeyword]: 'in',
        [SyntaxKind.UniqueKeyword]: 'unique',
        [SyntaxKind.KeyOfKeyword]: 'keyof',
        [SyntaxKind.NewKeyword]: 'new',
        [SyntaxKind.ImportKeyword]: 'import',
        [SyntaxKind.ReadonlyKeyword]: 'readonly',
        [SyntaxKind.QuestionQuestionToken]: '??',
        [SyntaxKind.QuestionDotToken]: '?.',
    };
    /**
     * Returns true if the given ts.Token is the assignment operator
     * @param operator the operator token
     * @returns is assignment
     */
    function isAssignmentOperator(operator) {
        return ASSIGNMENT_OPERATORS.includes(operator.kind);
    }
    exports.isAssignmentOperator = isAssignmentOperator;
    /**
     * Returns true if the given ts.Token is a logical operator
     * @param operator the operator token
     * @returns is a logical operator
     */
    function isLogicalOperator(operator) {
        return LOGICAL_OPERATORS.includes(operator.kind);
    }
    exports.isLogicalOperator = isLogicalOperator;
    /**
     * Returns the string form of the given TSToken SyntaxKind
     * @param kind the token's SyntaxKind
     * @returns the token applicable token as a string
     */
    function getTextForTokenKind(kind) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return kind in TOKEN_TO_TEXT ? TOKEN_TO_TEXT[kind] : undefined;
    }
    exports.getTextForTokenKind = getTextForTokenKind;
    /**
     * Returns true if the given ts.Node is a valid ESTree class member
     * @param node TypeScript AST node
     * @returns is valid ESTree class member
     */
    function isESTreeClassMember(node) {
        return node.kind !== SyntaxKind.SemicolonClassElement;
    }
    exports.isESTreeClassMember = isESTreeClassMember;
    /**
     * Checks if a ts.Node has a modifier
     * @param modifierKind TypeScript SyntaxKind modifier
     * @param node TypeScript AST node
     * @returns has the modifier specified
     */
    function hasModifier(modifierKind, node) {
        return (!!node.modifiers &&
            !!node.modifiers.length &&
            node.modifiers.some(modifier => modifier.kind === modifierKind));
    }
    exports.hasModifier = hasModifier;
    /**
     * Get last last modifier in ast
     * @param node TypeScript AST node
     * @returns returns last modifier if present or null
     */
    function getLastModifier(node) {
        return ((!!node.modifiers &&
            !!node.modifiers.length &&
            node.modifiers[node.modifiers.length - 1]) ||
            null);
    }
    exports.getLastModifier = getLastModifier;
    /**
     * Returns true if the given ts.Token is a comma
     * @param token the TypeScript token
     * @returns is comma
     */
    function isComma(token) {
        return token.kind === SyntaxKind.CommaToken;
    }
    exports.isComma = isComma;
    /**
     * Returns true if the given ts.Node is a comment
     * @param node the TypeScript node
     * @returns is comment
     */
    function isComment(node) {
        return (node.kind === SyntaxKind.SingleLineCommentTrivia ||
            node.kind === SyntaxKind.MultiLineCommentTrivia);
    }
    exports.isComment = isComment;
    /**
     * Returns true if the given ts.Node is a JSDoc comment
     * @param node the TypeScript node
     * @returns is JSDoc comment
     */
    function isJSDocComment(node) {
        return node.kind === SyntaxKind.JSDocComment;
    }
    exports.isJSDocComment = isJSDocComment;
    /**
     * Returns the binary expression type of the given ts.Token
     * @param operator the operator token
     * @returns the binary expression type
     */
    function getBinaryExpressionType(operator) {
        if (isAssignmentOperator(operator)) {
            return ts_estree_1.AST_NODE_TYPES.AssignmentExpression;
        }
        else if (isLogicalOperator(operator)) {
            return ts_estree_1.AST_NODE_TYPES.LogicalExpression;
        }
        return ts_estree_1.AST_NODE_TYPES.BinaryExpression;
    }
    exports.getBinaryExpressionType = getBinaryExpressionType;
    /**
     * Returns line and column data for the given positions,
     * @param pos position to check
     * @param ast the AST object
     * @returns line and column
     */
    function getLineAndCharacterFor(pos, ast) {
        const loc = ast.getLineAndCharacterOfPosition(pos);
        return {
            line: loc.line + 1,
            column: loc.character,
        };
    }
    exports.getLineAndCharacterFor = getLineAndCharacterFor;
    /**
     * Returns line and column data for the given start and end positions,
     * for the given AST
     * @param start start data
     * @param end   end data
     * @param ast   the AST object
     * @returns the loc data
     */
    function getLocFor(start, end, ast) {
        return {
            start: getLineAndCharacterFor(start, ast),
            end: getLineAndCharacterFor(end, ast),
        };
    }
    exports.getLocFor = getLocFor;
    /**
     * Check whatever node can contain directive
     * @param node
     * @returns returns true if node can contain directive
     */
    function canContainDirective(node) {
        if (node.kind === ts.SyntaxKind.Block) {
            switch (node.parent.kind) {
                case ts.SyntaxKind.Constructor:
                case ts.SyntaxKind.GetAccessor:
                case ts.SyntaxKind.SetAccessor:
                case ts.SyntaxKind.ArrowFunction:
                case ts.SyntaxKind.FunctionExpression:
                case ts.SyntaxKind.FunctionDeclaration:
                case ts.SyntaxKind.MethodDeclaration:
                    return true;
                default:
                    return false;
            }
        }
        return true;
    }
    exports.canContainDirective = canContainDirective;
    /**
     * Returns range for the given ts.Node
     * @param node the ts.Node or ts.Token
     * @param ast the AST object
     * @returns the range data
     */
    function getRange(node, ast) {
        return [node.getStart(ast), node.getEnd()];
    }
    exports.getRange = getRange;
    /**
     * Returns true if a given ts.Node is a token
     * @param node the ts.Node
     * @returns is a token
     */
    function isToken(node) {
        return (node.kind >= SyntaxKind.FirstToken && node.kind <= SyntaxKind.LastToken);
    }
    exports.isToken = isToken;
    /**
     * Returns true if a given ts.Node is a JSX token
     * @param node ts.Node to be checked
     * @returns is a JSX token
     */
    function isJSXToken(node) {
        return (node.kind >= SyntaxKind.JsxElement && node.kind <= SyntaxKind.JsxAttribute);
    }
    exports.isJSXToken = isJSXToken;
    /**
     * Returns the declaration kind of the given ts.Node
     * @param node TypeScript AST node
     * @returns declaration kind
     */
    function getDeclarationKind(node) {
        if (node.flags & ts.NodeFlags.Let) {
            return 'let';
        }
        if (node.flags & ts.NodeFlags.Const) {
            return 'const';
        }
        return 'var';
    }
    exports.getDeclarationKind = getDeclarationKind;
    /**
     * Gets a ts.Node's accessibility level
     * @param node The ts.Node
     * @returns accessibility "public", "protected", "private", or null
     */
    function getTSNodeAccessibility(node) {
        const modifiers = node.modifiers;
        if (!modifiers) {
            return null;
        }
        for (let i = 0; i < modifiers.length; i++) {
            const modifier = modifiers[i];
            switch (modifier.kind) {
                case SyntaxKind.PublicKeyword:
                    return 'public';
                case SyntaxKind.ProtectedKeyword:
                    return 'protected';
                case SyntaxKind.PrivateKeyword:
                    return 'private';
                default:
                    break;
            }
        }
        return null;
    }
    exports.getTSNodeAccessibility = getTSNodeAccessibility;
    /**
     * Finds the next token based on the previous one and its parent
     * Had to copy this from TS instead of using TS's version because theirs doesn't pass the ast to getChildren
     * @param previousToken The previous TSToken
     * @param parent The parent TSNode
     * @param ast The TS AST
     * @returns the next TSToken
     */
    function findNextToken(previousToken, parent, ast) {
        return find(parent);
        function find(n) {
            if (ts.isToken(n) && n.pos === previousToken.end) {
                // this is token that starts at the end of previous token - return it
                return n;
            }
            return firstDefined(n.getChildren(ast), (child) => {
                const shouldDiveInChildNode = 
                // previous token is enclosed somewhere in the child
                (child.pos <= previousToken.pos && child.end > previousToken.end) ||
                    // previous token ends exactly at the beginning of child
                    child.pos === previousToken.end;
                return shouldDiveInChildNode && nodeHasTokens(child, ast)
                    ? find(child)
                    : undefined;
            });
        }
    }
    exports.findNextToken = findNextToken;
    /**
     * Find the first matching ancestor based on the given predicate function.
     * @param node The current ts.Node
     * @param predicate The predicate function to apply to each checked ancestor
     * @returns a matching parent ts.Node
     */
    function findFirstMatchingAncestor(node, predicate) {
        while (node) {
            if (predicate(node)) {
                return node;
            }
            node = node.parent;
        }
        return undefined;
    }
    exports.findFirstMatchingAncestor = findFirstMatchingAncestor;
    /**
     * Returns true if a given ts.Node has a JSX token within its hierarchy
     * @param node ts.Node to be checked
     * @returns has JSX ancestor
     */
    function hasJSXAncestor(node) {
        return !!findFirstMatchingAncestor(node, isJSXToken);
    }
    exports.hasJSXAncestor = hasJSXAncestor;
    /**
     * Unescape the text content of string literals, e.g. &amp; -> &
     * @param text The escaped string literal text.
     * @returns The unescaped string literal text.
     */
    function unescapeStringLiteralText(text) {
        return unescape_1.default(text);
    }
    exports.unescapeStringLiteralText = unescapeStringLiteralText;
    /**
     * Returns true if a given ts.Node is a computed property
     * @param node ts.Node to be checked
     * @returns is Computed Property
     */
    function isComputedProperty(node) {
        return node.kind === SyntaxKind.ComputedPropertyName;
    }
    exports.isComputedProperty = isComputedProperty;
    /**
     * Returns true if a given ts.Node is optional (has QuestionToken)
     * @param node ts.Node to be checked
     * @returns is Optional
     */
    function isOptional(node) {
        return node.questionToken
            ? node.questionToken.kind === SyntaxKind.QuestionToken
            : false;
    }
    exports.isOptional = isOptional;
    /**
     * Returns the type of a given ts.Token
     * @param token the ts.Token
     * @returns the token type
     */
    function getTokenType(token) {
        if ('originalKeywordKind' in token && token.originalKeywordKind) {
            if (token.originalKeywordKind === SyntaxKind.NullKeyword) {
                return ts_estree_1.AST_TOKEN_TYPES.Null;
            }
            else if (token.originalKeywordKind >= SyntaxKind.FirstFutureReservedWord &&
                token.originalKeywordKind <= SyntaxKind.LastKeyword) {
                return ts_estree_1.AST_TOKEN_TYPES.Identifier;
            }
            return ts_estree_1.AST_TOKEN_TYPES.Keyword;
        }
        if (token.kind >= SyntaxKind.FirstKeyword &&
            token.kind <= SyntaxKind.LastFutureReservedWord) {
            if (token.kind === SyntaxKind.FalseKeyword ||
                token.kind === SyntaxKind.TrueKeyword) {
                return ts_estree_1.AST_TOKEN_TYPES.Boolean;
            }
            return ts_estree_1.AST_TOKEN_TYPES.Keyword;
        }
        if (token.kind >= SyntaxKind.FirstPunctuation &&
            token.kind <= SyntaxKind.LastBinaryOperator) {
            return ts_estree_1.AST_TOKEN_TYPES.Punctuator;
        }
        if (token.kind >= SyntaxKind.NoSubstitutionTemplateLiteral &&
            token.kind <= SyntaxKind.TemplateTail) {
            return ts_estree_1.AST_TOKEN_TYPES.Template;
        }
        switch (token.kind) {
            case SyntaxKind.NumericLiteral:
                return ts_estree_1.AST_TOKEN_TYPES.Numeric;
            case SyntaxKind.JsxText:
                return ts_estree_1.AST_TOKEN_TYPES.JSXText;
            case SyntaxKind.StringLiteral:
                // A TypeScript-StringLiteral token with a TypeScript-JsxAttribute or TypeScript-JsxElement parent,
                // must actually be an ESTree-JSXText token
                if (token.parent &&
                    (token.parent.kind === SyntaxKind.JsxAttribute ||
                        token.parent.kind === SyntaxKind.JsxElement)) {
                    return ts_estree_1.AST_TOKEN_TYPES.JSXText;
                }
                return ts_estree_1.AST_TOKEN_TYPES.String;
            case SyntaxKind.RegularExpressionLiteral:
                return ts_estree_1.AST_TOKEN_TYPES.RegularExpression;
            case SyntaxKind.Identifier:
            case SyntaxKind.ConstructorKeyword:
            case SyntaxKind.GetKeyword:
            case SyntaxKind.SetKeyword:
            // falls through
            default:
        }
        // Some JSX tokens have to be determined based on their parent
        if (token.parent && token.kind === SyntaxKind.Identifier) {
            if (isJSXToken(token.parent)) {
                return ts_estree_1.AST_TOKEN_TYPES.JSXIdentifier;
            }
            if (token.parent.kind === SyntaxKind.PropertyAccessExpression &&
                hasJSXAncestor(token)) {
                return ts_estree_1.AST_TOKEN_TYPES.JSXIdentifier;
            }
        }
        return ts_estree_1.AST_TOKEN_TYPES.Identifier;
    }
    exports.getTokenType = getTokenType;
    /**
     * Extends and formats a given ts.Token, for a given AST
     * @param token the ts.Token
     * @param ast   the AST object
     * @returns the converted Token
     */
    function convertToken(token, ast) {
        const start = token.kind === SyntaxKind.JsxText
            ? token.getFullStart()
            : token.getStart(ast);
        const end = token.getEnd();
        const value = ast.text.slice(start, end);
        const tokenType = getTokenType(token);
        if (tokenType === ts_estree_1.AST_TOKEN_TYPES.RegularExpression) {
            return {
                type: tokenType,
                value,
                range: [start, end],
                loc: getLocFor(start, end, ast),
                regex: {
                    pattern: value.slice(1, value.lastIndexOf('/')),
                    flags: value.slice(value.lastIndexOf('/') + 1),
                },
            };
        }
        else {
            return {
                type: tokenType,
                value,
                range: [start, end],
                loc: getLocFor(start, end, ast),
            };
        }
    }
    exports.convertToken = convertToken;
    /**
     * Converts all tokens for the given AST
     * @param ast the AST object
     * @returns the converted Tokens
     */
    function convertTokens(ast) {
        const result = [];
        /**
         * @param node the ts.Node
         */
        function walk(node) {
            // TypeScript generates tokens for types in JSDoc blocks. Comment tokens
            // and their children should not be walked or added to the resulting tokens list.
            if (isComment(node) || isJSDocComment(node)) {
                return;
            }
            if (isToken(node) && node.kind !== SyntaxKind.EndOfFileToken) {
                const converted = convertToken(node, ast);
                if (converted) {
                    result.push(converted);
                }
            }
            else {
                node.getChildren(ast).forEach(walk);
            }
        }
        walk(ast);
        return result;
    }
    exports.convertTokens = convertTokens;
    /**
     * @param ast     the AST object
     * @param start      the index at which the error starts
     * @param message the error message
     * @returns converted error object
     */
    function createError(ast, start, message) {
        const loc = ast.getLineAndCharacterOfPosition(start);
        return {
            index: start,
            lineNumber: loc.line + 1,
            column: loc.character,
            message,
        };
    }
    exports.createError = createError;
    /**
     * @param n the TSNode
     * @param ast the TS AST
     */
    function nodeHasTokens(n, ast) {
        // If we have a token or node that has a non-zero width, it must have tokens.
        // Note: getWidth() does not take trivia into account.
        return n.kind === SyntaxKind.EndOfFileToken
            ? // eslint-disable-next-line @typescript-eslint/no-explicit-any
                !!n.jsDoc
            : n.getWidth(ast) !== 0;
    }
    exports.nodeHasTokens = nodeHasTokens;
    /**
     * Like `forEach`, but suitable for use with numbers and strings (which may be falsy).
     * @template T
     * @template U
     * @param array
     * @param callback
     */
    function firstDefined(array, callback) {
        if (array === undefined) {
            return undefined;
        }
        for (let i = 0; i < array.length; i++) {
            const result = callback(array[i], i);
            if (result !== undefined) {
                return result;
            }
        }
        return undefined;
    }
    exports.firstDefined = firstDefined;
},
/* lodash/unescape.js */ function _(require, module, exports) {
    var toString = require(794) /* ./toString */, unescapeHtmlChar = require(806) /* ./_unescapeHtmlChar */;
    /** Used to match HTML entities and HTML characters. */
    var reEscapedHtml = /&(?:amp|lt|gt|quot|#39);/g, reHasEscapedHtml = RegExp(reEscapedHtml.source);
    /**
     * The inverse of `_.escape`; this method converts the HTML entities
     * `&amp;`, `&lt;`, `&gt;`, `&quot;`, and `&#39;` in `string` to
     * their corresponding characters.
     *
     * **Note:** No other HTML entities are unescaped. To unescape additional
     * HTML entities use a third-party library like [_he_](https://mths.be/he).
     *
     * @static
     * @memberOf _
     * @since 0.6.0
     * @category String
     * @param {string} [string=''] The string to unescape.
     * @returns {string} Returns the unescaped string.
     * @example
     *
     * _.unescape('fred, barney, &amp; pebbles');
     * // => 'fred, barney, & pebbles'
     */
    function unescape(string) {
        string = toString(string);
        return (string && reHasEscapedHtml.test(string))
            ? string.replace(reEscapedHtml, unescapeHtmlChar)
            : string;
    }
    module.exports = unescape;
},
/* lodash/toString.js */ function _(require, module, exports) {
    var baseToString = require(795) /* ./_baseToString */;
    /**
     * Converts `value` to a string. An empty string is returned for `null`
     * and `undefined` values. The sign of `-0` is preserved.
     *
     * @static
     * @memberOf _
     * @since 4.0.0
     * @category Lang
     * @param {*} value The value to convert.
     * @returns {string} Returns the converted string.
     * @example
     *
     * _.toString(null);
     * // => ''
     *
     * _.toString(-0);
     * // => '-0'
     *
     * _.toString([1, 2, 3]);
     * // => '1,2,3'
     */
    function toString(value) {
        return value == null ? '' : baseToString(value);
    }
    module.exports = toString;
},
/* lodash/_baseToString.js */ function _(require, module, exports) {
    var Symbol = require(796) /* ./_Symbol */, arrayMap = require(799) /* ./_arrayMap */, isArray = require(800) /* ./isArray */, isSymbol = require(801) /* ./isSymbol */;
    /** Used as references for various `Number` constants. */
    var INFINITY = 1 / 0;
    /** Used to convert symbols to primitives and strings. */
    var symbolProto = Symbol ? Symbol.prototype : undefined, symbolToString = symbolProto ? symbolProto.toString : undefined;
    /**
     * The base implementation of `_.toString` which doesn't convert nullish
     * values to empty strings.
     *
     * @private
     * @param {*} value The value to process.
     * @returns {string} Returns the string.
     */
    function baseToString(value) {
        // Exit early for strings to avoid a performance hit in some environments.
        if (typeof value == 'string') {
            return value;
        }
        if (isArray(value)) {
            // Recursively convert values (susceptible to call stack limits).
            return arrayMap(value, baseToString) + '';
        }
        if (isSymbol(value)) {
            return symbolToString ? symbolToString.call(value) : '';
        }
        var result = (value + '');
        return (result == '0' && (1 / value) == -INFINITY) ? '-0' : result;
    }
    module.exports = baseToString;
},
/* lodash/_Symbol.js */ function _(require, module, exports) {
    var root = require(797) /* ./_root */;
    /** Built-in value references. */
    var Symbol = root.Symbol;
    module.exports = Symbol;
},
/* lodash/_root.js */ function _(require, module, exports) {
    var freeGlobal = require(798) /* ./_freeGlobal */;
    /** Detect free variable `self`. */
    var freeSelf = typeof self == 'object' && self && self.Object === Object && self;
    /** Used as a reference to the global object. */
    var root = freeGlobal || freeSelf || Function('return this')();
    module.exports = root;
},
/* lodash/_freeGlobal.js */ function _(require, module, exports) {
    /** Detect free variable `global` from Node.js. */
    var freeGlobal = typeof global == 'object' && global && global.Object === Object && global;
    module.exports = freeGlobal;
},
/* lodash/_arrayMap.js */ function _(require, module, exports) {
    /**
     * A specialized version of `_.map` for arrays without support for iteratee
     * shorthands.
     *
     * @private
     * @param {Array} [array] The array to iterate over.
     * @param {Function} iteratee The function invoked per iteration.
     * @returns {Array} Returns the new mapped array.
     */
    function arrayMap(array, iteratee) {
        var index = -1, length = array == null ? 0 : array.length, result = Array(length);
        while (++index < length) {
            result[index] = iteratee(array[index], index, array);
        }
        return result;
    }
    module.exports = arrayMap;
},
/* lodash/isArray.js */ function _(require, module, exports) {
    /**
     * Checks if `value` is classified as an `Array` object.
     *
     * @static
     * @memberOf _
     * @since 0.1.0
     * @category Lang
     * @param {*} value The value to check.
     * @returns {boolean} Returns `true` if `value` is an array, else `false`.
     * @example
     *
     * _.isArray([1, 2, 3]);
     * // => true
     *
     * _.isArray(document.body.children);
     * // => false
     *
     * _.isArray('abc');
     * // => false
     *
     * _.isArray(_.noop);
     * // => false
     */
    var isArray = Array.isArray;
    module.exports = isArray;
},
/* lodash/isSymbol.js */ function _(require, module, exports) {
    var baseGetTag = require(802) /* ./_baseGetTag */, isObjectLike = require(805) /* ./isObjectLike */;
    /** `Object#toString` result references. */
    var symbolTag = '[object Symbol]';
    /**
     * Checks if `value` is classified as a `Symbol` primitive or object.
     *
     * @static
     * @memberOf _
     * @since 4.0.0
     * @category Lang
     * @param {*} value The value to check.
     * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
     * @example
     *
     * _.isSymbol(Symbol.iterator);
     * // => true
     *
     * _.isSymbol('abc');
     * // => false
     */
    function isSymbol(value) {
        return typeof value == 'symbol' ||
            (isObjectLike(value) && baseGetTag(value) == symbolTag);
    }
    module.exports = isSymbol;
},
/* lodash/_baseGetTag.js */ function _(require, module, exports) {
    var Symbol = require(796) /* ./_Symbol */, getRawTag = require(803) /* ./_getRawTag */, objectToString = require(804) /* ./_objectToString */;
    /** `Object#toString` result references. */
    var nullTag = '[object Null]', undefinedTag = '[object Undefined]';
    /** Built-in value references. */
    var symToStringTag = Symbol ? Symbol.toStringTag : undefined;
    /**
     * The base implementation of `getTag` without fallbacks for buggy environments.
     *
     * @private
     * @param {*} value The value to query.
     * @returns {string} Returns the `toStringTag`.
     */
    function baseGetTag(value) {
        if (value == null) {
            return value === undefined ? undefinedTag : nullTag;
        }
        return (symToStringTag && symToStringTag in Object(value))
            ? getRawTag(value)
            : objectToString(value);
    }
    module.exports = baseGetTag;
},
/* lodash/_getRawTag.js */ function _(require, module, exports) {
    var Symbol = require(796) /* ./_Symbol */;
    /** Used for built-in method references. */
    var objectProto = Object.prototype;
    /** Used to check objects for own properties. */
    var hasOwnProperty = objectProto.hasOwnProperty;
    /**
     * Used to resolve the
     * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
     * of values.
     */
    var nativeObjectToString = objectProto.toString;
    /** Built-in value references. */
    var symToStringTag = Symbol ? Symbol.toStringTag : undefined;
    /**
     * A specialized version of `baseGetTag` which ignores `Symbol.toStringTag` values.
     *
     * @private
     * @param {*} value The value to query.
     * @returns {string} Returns the raw `toStringTag`.
     */
    function getRawTag(value) {
        var isOwn = hasOwnProperty.call(value, symToStringTag), tag = value[symToStringTag];
        try {
            value[symToStringTag] = undefined;
            var unmasked = true;
        }
        catch (e) { }
        var result = nativeObjectToString.call(value);
        if (unmasked) {
            if (isOwn) {
                value[symToStringTag] = tag;
            }
            else {
                delete value[symToStringTag];
            }
        }
        return result;
    }
    module.exports = getRawTag;
},
/* lodash/_objectToString.js */ function _(require, module, exports) {
    /** Used for built-in method references. */
    var objectProto = Object.prototype;
    /**
     * Used to resolve the
     * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
     * of values.
     */
    var nativeObjectToString = objectProto.toString;
    /**
     * Converts `value` to a string using `Object.prototype.toString`.
     *
     * @private
     * @param {*} value The value to convert.
     * @returns {string} Returns the converted string.
     */
    function objectToString(value) {
        return nativeObjectToString.call(value);
    }
    module.exports = objectToString;
},
/* lodash/isObjectLike.js */ function _(require, module, exports) {
    /**
     * Checks if `value` is object-like. A value is object-like if it's not `null`
     * and has a `typeof` result of "object".
     *
     * @static
     * @memberOf _
     * @since 4.0.0
     * @category Lang
     * @param {*} value The value to check.
     * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
     * @example
     *
     * _.isObjectLike({});
     * // => true
     *
     * _.isObjectLike([1, 2, 3]);
     * // => true
     *
     * _.isObjectLike(_.noop);
     * // => false
     *
     * _.isObjectLike(null);
     * // => false
     */
    function isObjectLike(value) {
        return value != null && typeof value == 'object';
    }
    module.exports = isObjectLike;
},
/* lodash/_unescapeHtmlChar.js */ function _(require, module, exports) {
    var basePropertyOf = require(807) /* ./_basePropertyOf */;
    /** Used to map HTML entities to characters. */
    var htmlUnescapes = {
        '&amp;': '&',
        '&lt;': '<',
        '&gt;': '>',
        '&quot;': '"',
        '&#39;': "'"
    };
    /**
     * Used by `_.unescape` to convert HTML entities to characters.
     *
     * @private
     * @param {string} chr The matched character to unescape.
     * @returns {string} Returns the unescaped character.
     */
    var unescapeHtmlChar = basePropertyOf(htmlUnescapes);
    module.exports = unescapeHtmlChar;
},
/* lodash/_basePropertyOf.js */ function _(require, module, exports) {
    /**
     * The base implementation of `_.propertyOf` without support for deep paths.
     *
     * @private
     * @param {Object} object The object to query.
     * @returns {Function} Returns the new accessor function.
     */
    function basePropertyOf(object) {
        return function (key) {
            return object == null ? undefined : object[key];
        };
    }
    module.exports = basePropertyOf;
},
/* @typescript-eslint/typescript-estree/dist/convert-comments.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = __importStar(require(44) /* typescript */);
    const util_1 = require(641) /* tsutils/util/util */;
    const node_utils_1 = require(792) /* ./node-utils */;
    const ts_estree_1 = require(619) /* ./ts-estree */;
    /**
     * Convert all comments for the given AST.
     * @param ast the AST object
     * @param code the TypeScript code
     * @returns the converted ESTreeComment
     * @private
     */
    function convertComments(ast, code) {
        const comments = [];
        util_1.forEachComment(ast, (_, comment) => {
            const type = comment.kind == ts.SyntaxKind.SingleLineCommentTrivia
                ? ts_estree_1.AST_TOKEN_TYPES.Line
                : ts_estree_1.AST_TOKEN_TYPES.Block;
            const range = [comment.pos, comment.end];
            const loc = node_utils_1.getLocFor(range[0], range[1], ast);
            // both comments start with 2 characters - /* or //
            const textStart = range[0] + 2;
            const textEnd = comment.kind === ts.SyntaxKind.SingleLineCommentTrivia
                ? // single line comments end at the end
                    range[1] - textStart
                : // multiline comments end 2 characters early
                    range[1] - textStart - 2;
            comments.push({
                type,
                value: code.substr(textStart, textEnd),
                range,
                loc,
            });
        }, ast);
        return comments;
    }
    exports.convertComments = convertComments;
},
/* @typescript-eslint/typescript-estree/dist/simple-traverse.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const visitor_keys_1 = require(810) /* ./visitor-keys */;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    function isValidNode(x) {
        return x !== null && typeof x === 'object' && typeof x.type === 'string';
    }
    function getVisitorKeysForNode(allVisitorKeys, node) {
        const keys = allVisitorKeys[node.type];
        return keys !== null && keys !== void 0 ? keys : [];
    }
    class SimpleTraverser {
        constructor({ enter }) {
            this.allVisitorKeys = visitor_keys_1.visitorKeys;
            this.enter = enter;
        }
        traverse(node, parent) {
            if (!isValidNode(node)) {
                return;
            }
            this.enter(node, parent);
            const keys = getVisitorKeysForNode(this.allVisitorKeys, node);
            if (keys.length < 1) {
                return;
            }
            for (const key of keys) {
                const childOrChildren = node[key];
                if (Array.isArray(childOrChildren)) {
                    for (const child of childOrChildren) {
                        this.traverse(child, node);
                    }
                }
                else {
                    this.traverse(childOrChildren, node);
                }
            }
        }
    }
    function simpleTraverse(startingNode, options) {
        new SimpleTraverser(options).traverse(startingNode, undefined);
    }
    exports.simpleTraverse = simpleTraverse;
},
/* @typescript-eslint/typescript-estree/dist/visitor-keys.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const eslintVisitorKeys = __importStar(require(100) /* eslint-visitor-keys */);
    exports.visitorKeys = eslintVisitorKeys.unionWith({
        // Additional estree nodes.
        Import: [],
        // Additional Properties.
        ArrayPattern: ['decorators', 'elements', 'typeAnnotation'],
        ArrowFunctionExpression: ['typeParameters', 'params', 'returnType', 'body'],
        ClassDeclaration: [
            'decorators',
            'id',
            'typeParameters',
            'superClass',
            'superTypeParameters',
            'implements',
            'body',
        ],
        ClassExpression: [
            'decorators',
            'id',
            'typeParameters',
            'superClass',
            'superTypeParameters',
            'implements',
            'body',
        ],
        TaggedTemplateExpression: ['tag', 'typeParameters', 'quasi'],
        FunctionDeclaration: ['id', 'typeParameters', 'params', 'returnType', 'body'],
        FunctionExpression: ['id', 'typeParameters', 'params', 'returnType', 'body'],
        Identifier: ['decorators', 'typeAnnotation'],
        MethodDefinition: ['decorators', 'key', 'value'],
        ObjectPattern: ['decorators', 'properties', 'typeAnnotation'],
        RestElement: ['decorators', 'argument', 'typeAnnotation'],
        NewExpression: ['callee', 'typeParameters', 'arguments'],
        CallExpression: ['callee', 'typeParameters', 'arguments'],
        // JSX
        JSXOpeningElement: ['name', 'typeParameters', 'attributes'],
        JSXClosingFragment: [],
        JSXOpeningFragment: [],
        JSXSpreadChild: ['expression'],
        // Additional Nodes.
        BigIntLiteral: [],
        ClassProperty: ['decorators', 'key', 'typeAnnotation', 'value'],
        Decorator: ['expression'],
        OptionalCallExpression: ['callee', 'typeParameters', 'arguments'],
        OptionalMemberExpression: eslintVisitorKeys.KEYS.MemberExpression,
        TSAbstractClassProperty: ['decorators', 'key', 'typeAnnotation', 'value'],
        TSAbstractKeyword: [],
        TSAbstractMethodDefinition: ['key', 'value'],
        TSAnyKeyword: [],
        TSArrayType: ['elementType'],
        TSAsExpression: ['expression', 'typeAnnotation'],
        TSAsyncKeyword: [],
        TSBigIntKeyword: [],
        TSBooleanKeyword: [],
        TSCallSignatureDeclaration: ['typeParameters', 'params', 'returnType'],
        TSClassImplements: ['expression', 'typeParameters'],
        TSConditionalType: ['checkType', 'extendsType', 'trueType', 'falseType'],
        TSConstructSignatureDeclaration: ['typeParameters', 'params', 'returnType'],
        TSConstructorType: ['typeParameters', 'params', 'returnType'],
        TSDeclareFunction: ['id', 'typeParameters', 'params', 'returnType', 'body'],
        TSDeclareKeyword: [],
        TSEmptyBodyFunctionExpression: [
            'id',
            'typeParameters',
            'params',
            'returnType',
        ],
        TSEnumDeclaration: ['id', 'members'],
        TSEnumMember: ['id', 'initializer'],
        TSExportAssignment: ['expression'],
        TSExportKeyword: [],
        TSExternalModuleReference: ['expression'],
        TSImportType: ['parameter', 'qualifier', 'typeParameters'],
        TSInferType: ['typeParameter'],
        TSLiteralType: ['literal'],
        TSIntersectionType: ['types'],
        TSIndexedAccessType: ['indexType', 'objectType'],
        TSIndexSignature: ['parameters', 'typeAnnotation'],
        TSInterfaceBody: ['body'],
        TSInterfaceDeclaration: ['id', 'typeParameters', 'extends', 'body'],
        TSInterfaceHeritage: ['expression', 'typeParameters'],
        TSImportEqualsDeclaration: ['id', 'moduleReference'],
        TSFunctionType: ['typeParameters', 'params', 'returnType'],
        TSMappedType: ['typeParameter', 'typeAnnotation'],
        TSMethodSignature: ['typeParameters', 'key', 'params', 'returnType'],
        TSModuleBlock: ['body'],
        TSModuleDeclaration: ['id', 'body'],
        TSNamespaceExportDeclaration: ['id'],
        TSNonNullExpression: ['expression'],
        TSNeverKeyword: [],
        TSNullKeyword: [],
        TSNumberKeyword: [],
        TSObjectKeyword: [],
        TSOptionalType: ['typeAnnotation'],
        TSParameterProperty: ['decorators', 'parameter'],
        TSParenthesizedType: ['typeAnnotation'],
        TSPrivateKeyword: [],
        TSPropertySignature: ['typeAnnotation', 'key', 'initializer'],
        TSProtectedKeyword: [],
        TSPublicKeyword: [],
        TSQualifiedName: ['left', 'right'],
        TSReadonlyKeyword: [],
        TSRestType: ['typeAnnotation'],
        TSStaticKeyword: [],
        TSStringKeyword: [],
        TSSymbolKeyword: [],
        TSThisType: [],
        TSTupleType: ['elementTypes'],
        TSTypeAliasDeclaration: ['id', 'typeParameters', 'typeAnnotation'],
        TSTypeAnnotation: ['typeAnnotation'],
        TSTypeAssertion: ['typeAnnotation', 'expression'],
        TSTypeLiteral: ['members'],
        TSTypeOperator: ['typeAnnotation'],
        TSTypeParameter: ['name', 'constraint', 'default'],
        TSTypeParameterDeclaration: ['params'],
        TSTypeParameterInstantiation: ['params'],
        TSTypePredicate: ['typeAnnotation', 'parameterName'],
        TSTypeReference: ['typeName', 'typeParameters'],
        TSTypeQuery: ['exprName'],
        TSUnionType: ['types'],
        TSUndefinedKeyword: [],
        TSUnknownKeyword: [],
        TSVoidKeyword: [],
    });
},
/* @typescript-eslint/typescript-estree/dist/create-program/createDefaultProgram.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const debug_1 = __importDefault(require(120) /* debug */);
    const path_1 = __importDefault(require("path"));
    const ts = __importStar(require(44) /* typescript */);
    const shared_1 = require(812) /* ./shared */;
    const log = debug_1.default('typescript-eslint:typescript-estree:createDefaultProgram');
    /**
     * @param code The code of the file being linted
     * @param extra The config object
     * @param extra.tsconfigRootDir The root directory for relative tsconfig paths
     * @param extra.projects Provided tsconfig paths
     * @returns If found, returns the source file corresponding to the code and the containing program
     */
    function createDefaultProgram(code, extra) {
        log('Getting default program for: %s', extra.filePath || 'unnamed file');
        if (!extra.projects || extra.projects.length !== 1) {
            return undefined;
        }
        const tsconfigPath = shared_1.getTsconfigPath(extra.projects[0], extra);
        const commandLine = ts.getParsedCommandLineOfConfigFile(tsconfigPath, shared_1.createDefaultCompilerOptionsFromExtra(extra), Object.assign(Object.assign({}, ts.sys), { onUnRecoverableConfigFileDiagnostic: () => { } }));
        if (!commandLine) {
            return undefined;
        }
        const compilerHost = ts.createCompilerHost(commandLine.options, 
        /* setParentNodes */ true);
        const oldReadFile = compilerHost.readFile;
        compilerHost.readFile = (fileName) => path_1.default.normalize(fileName) === path_1.default.normalize(extra.filePath)
            ? code
            : oldReadFile(fileName);
        const program = ts.createProgram([extra.filePath], commandLine.options, compilerHost);
        const ast = program.getSourceFile(extra.filePath);
        return ast && { ast, program };
    }
    exports.createDefaultProgram = createDefaultProgram;
},
/* @typescript-eslint/typescript-estree/dist/create-program/shared.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const path_1 = __importDefault(require("path"));
    const ts = __importStar(require(44) /* typescript */);
    /**
     * Default compiler options for program generation from single root file
     */
    const DEFAULT_COMPILER_OPTIONS = {
        allowNonTsExtensions: true,
        allowJs: true,
        checkJs: true,
        noEmit: true,
        // extendedDiagnostics: true,
        noUnusedLocals: true,
        noUnusedParameters: true,
    };
    function createDefaultCompilerOptionsFromExtra(extra) {
        if (extra.debugLevel.has('typescript')) {
            return Object.assign(Object.assign({}, DEFAULT_COMPILER_OPTIONS), { extendedDiagnostics: true });
        }
        return DEFAULT_COMPILER_OPTIONS;
    }
    exports.createDefaultCompilerOptionsFromExtra = createDefaultCompilerOptionsFromExtra;
    // typescript doesn't provide a ts.sys implementation for browser environments
    const useCaseSensitiveFileNames = ts.sys !== undefined ? ts.sys.useCaseSensitiveFileNames : true;
    const correctPathCasing = useCaseSensitiveFileNames
        ? (filePath) => filePath
        : (filePath) => filePath.toLowerCase();
    function getCanonicalFileName(filePath) {
        let normalized = path_1.default.normalize(filePath);
        if (normalized.endsWith(path_1.default.sep)) {
            normalized = normalized.substr(0, normalized.length - 1);
        }
        return correctPathCasing(normalized);
    }
    exports.getCanonicalFileName = getCanonicalFileName;
    function ensureAbsolutePath(p, extra) {
        return path_1.default.isAbsolute(p)
            ? p
            : path_1.default.join(extra.tsconfigRootDir || process.cwd(), p);
    }
    exports.ensureAbsolutePath = ensureAbsolutePath;
    function getTsconfigPath(tsconfigPath, extra) {
        return getCanonicalFileName(ensureAbsolutePath(tsconfigPath, extra));
    }
    exports.getTsconfigPath = getTsconfigPath;
    function canonicalDirname(p) {
        return path_1.default.dirname(p);
    }
    exports.canonicalDirname = canonicalDirname;
    function getScriptKind(extra, filePath = extra.filePath) {
        const extension = path_1.default.extname(filePath).toLowerCase();
        // note - we respect the user's extension when it is known  we could override it and force it to match their
        // jsx setting, but that could create weird situations where we throw parse errors when TSC doesn't
        switch (extension) {
            case '.ts':
                return ts.ScriptKind.TS;
            case '.tsx':
                return ts.ScriptKind.TSX;
            case '.js':
                return ts.ScriptKind.JS;
            case '.jsx':
                return ts.ScriptKind.JSX;
            case '.json':
                return ts.ScriptKind.JSON;
            default:
                // unknown extension, force typescript to ignore the file extension, and respect the user's setting
                return extra.jsx ? ts.ScriptKind.TSX : ts.ScriptKind.TS;
        }
    }
    exports.getScriptKind = getScriptKind;
},
/* @typescript-eslint/typescript-estree/dist/create-program/createIsolatedProgram.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const debug_1 = __importDefault(require(120) /* debug */);
    const ts = __importStar(require(44) /* typescript */);
    const shared_1 = require(812) /* ./shared */;
    const log = debug_1.default('typescript-eslint:typescript-estree:createIsolatedProgram');
    /**
     * @param code The code of the file being linted
     * @returns Returns a new source file and program corresponding to the linted code
     */
    function createIsolatedProgram(code, extra) {
        log('Getting isolated program in %s mode for: %s', extra.jsx ? 'TSX' : 'TS', extra.filePath);
        const compilerHost = {
            fileExists() {
                return true;
            },
            getCanonicalFileName() {
                return extra.filePath;
            },
            getCurrentDirectory() {
                return '';
            },
            getDirectories() {
                return [];
            },
            getDefaultLibFileName() {
                return 'lib.d.ts';
            },
            // TODO: Support Windows CRLF
            getNewLine() {
                return '\n';
            },
            getSourceFile(filename) {
                return ts.createSourceFile(filename, code, ts.ScriptTarget.Latest, 
                /* setParentNodes */ true, shared_1.getScriptKind(extra, filename));
            },
            readFile() {
                return undefined;
            },
            useCaseSensitiveFileNames() {
                return true;
            },
            writeFile() {
                return null;
            },
        };
        const program = ts.createProgram([extra.filePath], Object.assign({ noResolve: true, target: ts.ScriptTarget.Latest, jsx: extra.jsx ? ts.JsxEmit.Preserve : undefined }, shared_1.createDefaultCompilerOptionsFromExtra(extra)), compilerHost);
        const ast = program.getSourceFile(extra.filePath);
        if (!ast) {
            throw new Error('Expected an ast to be returned for the single-file isolated program.');
        }
        return { ast, program };
    }
    exports.createIsolatedProgram = createIsolatedProgram;
},
/* @typescript-eslint/typescript-estree/dist/create-program/createProjectProgram.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const debug_1 = __importDefault(require(120) /* debug */);
    const path_1 = __importDefault(require("path"));
    const createWatchProgram_1 = require(815) /* ./createWatchProgram */;
    const node_utils_1 = require(792) /* ../node-utils */;
    const log = debug_1.default('typescript-eslint:typescript-estree:createProjectProgram');
    const DEFAULT_EXTRA_FILE_EXTENSIONS = ['.ts', '.tsx', '.js', '.jsx'];
    function getExtension(fileName) {
        if (!fileName) {
            return null;
        }
        return fileName.endsWith('.d.ts') ? '.d.ts' : path_1.default.extname(fileName);
    }
    /**
     * @param code The code of the file being linted
     * @param createDefaultProgram True if the default program should be created
     * @param extra The config object
     * @returns If found, returns the source file corresponding to the code and the containing program
     */
    function createProjectProgram(code, createDefaultProgram, extra) {
        log('Creating project program for: %s', extra.filePath);
        const astAndProgram = node_utils_1.firstDefined(createWatchProgram_1.getProgramsForProjects(code, extra.filePath, extra), currentProgram => {
            const ast = currentProgram.getSourceFile(extra.filePath);
            // working around https://github.com/typescript-eslint/typescript-eslint/issues/1573
            const expectedExt = getExtension(extra.filePath);
            const returnedExt = getExtension(ast === null || ast === void 0 ? void 0 : ast.fileName);
            if (expectedExt !== returnedExt) {
                return;
            }
            return ast && { ast, program: currentProgram };
        });
        if (!astAndProgram && !createDefaultProgram) {
            // the file was either not matched within the tsconfig, or the extension wasn't expected
            const errorLines = [
                '"parserOptions.project" has been set for @typescript-eslint/parser.',
                `The file does not match your project config: ${path_1.default.relative(extra.tsconfigRootDir || process.cwd(), extra.filePath)}.`,
            ];
            let hasMatchedAnError = false;
            const extraFileExtensions = extra.extraFileExtensions || [];
            extraFileExtensions.forEach(extraExtension => {
                if (!extraExtension.startsWith('.')) {
                    errorLines.push(`Found unexpected extension "${extraExtension}" specified with the "extraFileExtensions" option. Did you mean ".${extraExtension}"?`);
                }
                if (DEFAULT_EXTRA_FILE_EXTENSIONS.includes(extraExtension)) {
                    errorLines.push(`You unnecessarily included the extension "${extraExtension}" with the "extraFileExtensions" option. This extension is already handled by the parser by default.`);
                }
            });
            const fileExtension = path_1.default.extname(extra.filePath);
            if (!DEFAULT_EXTRA_FILE_EXTENSIONS.includes(fileExtension)) {
                const nonStandardExt = `The extension for the file (${fileExtension}) is non-standard`;
                if (extraFileExtensions.length > 0) {
                    if (!extraFileExtensions.includes(fileExtension)) {
                        errorLines.push(`${nonStandardExt}. It should be added to your existing "parserOptions.extraFileExtensions".`);
                        hasMatchedAnError = true;
                    }
                }
                else {
                    errorLines.push(`${nonStandardExt}. You should add "parserOptions.extraFileExtensions" to your config.`);
                    hasMatchedAnError = true;
                }
            }
            if (!hasMatchedAnError) {
                errorLines.push('The file must be included in at least one of the projects provided.');
                hasMatchedAnError = true;
            }
            throw new Error(errorLines.join('\n'));
        }
        return astAndProgram;
    }
    exports.createProjectProgram = createProjectProgram;
},
/* @typescript-eslint/typescript-estree/dist/create-program/createWatchProgram.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const debug_1 = __importDefault(require(120) /* debug */);
    const fs_1 = __importDefault(require("fs"));
    const semver_1 = __importDefault(require(749) /* semver */);
    const ts = __importStar(require(44) /* typescript */);
    const shared_1 = require(812) /* ./shared */;
    const log = debug_1.default('typescript-eslint:typescript-estree:createWatchProgram');
    /**
     * Maps tsconfig paths to their corresponding file contents and resulting watches
     */
    const knownWatchProgramMap = new Map();
    /**
     * Maps file/folder paths to their set of corresponding watch callbacks
     * There may be more than one per file/folder if a file/folder is shared between projects
     */
    const fileWatchCallbackTrackingMap = new Map();
    const folderWatchCallbackTrackingMap = new Map();
    /**
     * Stores the list of known files for each program
     */
    const programFileListCache = new Map();
    /**
     * Caches the last modified time of the tsconfig files
     */
    const tsconfigLastModifiedTimestampCache = new Map();
    const parsedFilesSeenHash = new Map();
    /**
     * Clear all of the parser caches.
     * This should only be used in testing to ensure the parser is clean between tests.
     */
    function clearCaches() {
        knownWatchProgramMap.clear();
        fileWatchCallbackTrackingMap.clear();
        folderWatchCallbackTrackingMap.clear();
        parsedFilesSeenHash.clear();
        programFileListCache.clear();
        tsconfigLastModifiedTimestampCache.clear();
    }
    exports.clearCaches = clearCaches;
    function saveWatchCallback(trackingMap) {
        return (fileName, callback) => {
            const normalizedFileName = shared_1.getCanonicalFileName(fileName);
            const watchers = (() => {
                let watchers = trackingMap.get(normalizedFileName);
                if (!watchers) {
                    watchers = new Set();
                    trackingMap.set(normalizedFileName, watchers);
                }
                return watchers;
            })();
            watchers.add(callback);
            return {
                close: () => {
                    watchers.delete(callback);
                },
            };
        };
    }
    /**
     * Holds information about the file currently being linted
     */
    const currentLintOperationState = {
        code: '',
        filePath: '',
    };
    /**
     * Appropriately report issues found when reading a config file
     * @param diagnostic The diagnostic raised when creating a program
     */
    function diagnosticReporter(diagnostic) {
        throw new Error(ts.flattenDiagnosticMessageText(diagnostic.messageText, ts.sys.newLine));
    }
    /**
     * Hash content for compare content.
     * @param content hashed contend
     * @returns hashed result
     */
    function createHash(content) {
        // No ts.sys in browser environments.
        if (ts.sys && ts.sys.createHash) {
            return ts.sys.createHash(content);
        }
        return content;
    }
    /**
     * Calculate project environments using options provided by consumer and paths from config
     * @param code The code being linted
     * @param filePathIn The path of the file being parsed
     * @param extra.tsconfigRootDir The root directory for relative tsconfig paths
     * @param extra.projects Provided tsconfig paths
     * @returns The programs corresponding to the supplied tsconfig paths
     */
    function getProgramsForProjects(code, filePathIn, extra) {
        const filePath = shared_1.getCanonicalFileName(filePathIn);
        const results = [];
        // preserve reference to code and file being linted
        currentLintOperationState.code = code;
        currentLintOperationState.filePath = filePath;
        // Update file version if necessary
        const fileWatchCallbacks = fileWatchCallbackTrackingMap.get(filePath);
        const codeHash = createHash(code);
        if (parsedFilesSeenHash.get(filePath) !== codeHash &&
            fileWatchCallbacks &&
            fileWatchCallbacks.size > 0) {
            fileWatchCallbacks.forEach(cb => cb(filePath, ts.FileWatcherEventKind.Changed));
        }
        /*
         * before we go into the process of attempting to find and update every program
         * see if we know of a program that contains this file
         */
        for (const rawTsconfigPath of extra.projects) {
            const tsconfigPath = shared_1.getTsconfigPath(rawTsconfigPath, extra);
            const existingWatch = knownWatchProgramMap.get(tsconfigPath);
            if (!existingWatch) {
                continue;
            }
            let fileList = programFileListCache.get(tsconfigPath);
            let updatedProgram = null;
            if (!fileList) {
                updatedProgram = existingWatch.getProgram().getProgram();
                fileList = new Set(updatedProgram.getRootFileNames().map(f => shared_1.getCanonicalFileName(f)));
                programFileListCache.set(tsconfigPath, fileList);
            }
            if (fileList.has(filePath)) {
                log('Found existing program for file. %s', filePath);
                updatedProgram = updatedProgram !== null && updatedProgram !== void 0 ? updatedProgram : existingWatch.getProgram().getProgram();
                // sets parent pointers in source files
                updatedProgram.getTypeChecker();
                return [updatedProgram];
            }
        }
        log('File did not belong to any existing programs, moving to create/update. %s', filePath);
        /*
         * We don't know of a program that contains the file, this means that either:
         * - the required program hasn't been created yet, or
         * - the file is new/renamed, and the program hasn't been updated.
         */
        for (const rawTsconfigPath of extra.projects) {
            const tsconfigPath = shared_1.getTsconfigPath(rawTsconfigPath, extra);
            const existingWatch = knownWatchProgramMap.get(tsconfigPath);
            if (existingWatch) {
                const updatedProgram = maybeInvalidateProgram(existingWatch, filePath, tsconfigPath);
                if (!updatedProgram) {
                    continue;
                }
                // sets parent pointers in source files
                updatedProgram.getTypeChecker();
                results.push(updatedProgram);
                continue;
            }
            const programWatch = createWatchProgram(tsconfigPath, extra);
            const program = programWatch.getProgram().getProgram();
            // cache watch program and return current program
            knownWatchProgramMap.set(tsconfigPath, programWatch);
            // sets parent pointers in source files
            program.getTypeChecker();
            results.push(program);
        }
        return results;
    }
    exports.getProgramsForProjects = getProgramsForProjects;
    const isRunningNoTimeoutFix = semver_1.default.satisfies(ts.version, '>=3.9.0-beta', {
        includePrerelease: true,
    });
    function createWatchProgram(tsconfigPath, extra) {
        log('Creating watch program for %s.', tsconfigPath);
        // create compiler host
        const watchCompilerHost = ts.createWatchCompilerHost(tsconfigPath, shared_1.createDefaultCompilerOptionsFromExtra(extra), ts.sys, ts.createAbstractBuilder, diagnosticReporter, 
        /*reportWatchStatus*/ () => { });
        // ensure readFile reads the code being linted instead of the copy on disk
        const oldReadFile = watchCompilerHost.readFile;
        watchCompilerHost.readFile = (filePathIn, encoding) => {
            const filePath = shared_1.getCanonicalFileName(filePathIn);
            const fileContent = filePath === currentLintOperationState.filePath
                ? currentLintOperationState.code
                : oldReadFile(filePath, encoding);
            if (fileContent !== undefined) {
                parsedFilesSeenHash.set(filePath, createHash(fileContent));
            }
            return fileContent;
        };
        // ensure process reports error on failure instead of exiting process immediately
        watchCompilerHost.onUnRecoverableConfigFileDiagnostic = diagnosticReporter;
        // ensure process doesn't emit programs
        watchCompilerHost.afterProgramCreate = (program) => {
            // report error if there are any errors in the config file
            const configFileDiagnostics = program
                .getConfigFileParsingDiagnostics()
                .filter(diag => diag.category === ts.DiagnosticCategory.Error && diag.code !== 18003);
            if (configFileDiagnostics.length > 0) {
                diagnosticReporter(configFileDiagnostics[0]);
            }
        };
        /*
         * From the CLI, the file watchers won't matter, as the files will be parsed once and then forgotten.
         * When running from an IDE, these watchers will let us tell typescript about changes.
         *
         * ESLint IDE plugins will send us unfinished file content as the user types (before it's saved to disk).
         * We use the file watchers to tell typescript about this latest file content.
         *
         * When files are created (or renamed), we won't know about them because we have no filesystem watchers attached.
         * We use the folder watchers to tell typescript it needs to go and find new files in the project folders.
         */
        watchCompilerHost.watchFile = saveWatchCallback(fileWatchCallbackTrackingMap);
        watchCompilerHost.watchDirectory = saveWatchCallback(folderWatchCallbackTrackingMap);
        // allow files with custom extensions to be included in program (uses internal ts api)
        const oldOnDirectoryStructureHostCreate = watchCompilerHost.onCachedDirectoryStructureHostCreate;
        watchCompilerHost.onCachedDirectoryStructureHostCreate = (host) => {
            const oldReadDirectory = host.readDirectory;
            host.readDirectory = (path, extensions, exclude, include, depth) => oldReadDirectory(path, !extensions ? undefined : extensions.concat(extra.extraFileExtensions), exclude, include, depth);
            oldOnDirectoryStructureHostCreate(host);
        };
        watchCompilerHost.trace = log;
        // Since we don't want to asynchronously update program we want to disable timeout methods
        // So any changes in the program will be delayed and updated when getProgram is called on watch
        let callback;
        if (isRunningNoTimeoutFix) {
            watchCompilerHost.setTimeout = undefined;
            watchCompilerHost.clearTimeout = undefined;
        }
        else {
            log('Running without timeout fix');
            // But because of https://github.com/microsoft/TypeScript/pull/37308 we cannot just set it to undefined
            // instead save it and call before getProgram is called
            watchCompilerHost.setTimeout = (cb, _ms, ...args) => {
                callback = cb.bind(/*this*/ undefined, ...args);
                return callback;
            };
            watchCompilerHost.clearTimeout = () => {
                callback = undefined;
            };
        }
        const watch = ts.createWatchProgram(watchCompilerHost);
        if (!isRunningNoTimeoutFix) {
            const originalGetProgram = watch.getProgram;
            watch.getProgram = () => {
                if (callback) {
                    callback();
                }
                callback = undefined;
                return originalGetProgram.call(watch);
            };
        }
        return watch;
    }
    exports.createWatchProgram = createWatchProgram;
    function hasTSConfigChanged(tsconfigPath) {
        const stat = fs_1.default.statSync(tsconfigPath);
        const lastModifiedAt = stat.mtimeMs;
        const cachedLastModifiedAt = tsconfigLastModifiedTimestampCache.get(tsconfigPath);
        tsconfigLastModifiedTimestampCache.set(tsconfigPath, lastModifiedAt);
        if (cachedLastModifiedAt === undefined) {
            return false;
        }
        return Math.abs(cachedLastModifiedAt - lastModifiedAt) > Number.EPSILON;
    }
    function maybeInvalidateProgram(existingWatch, filePath, tsconfigPath) {
        /*
         * By calling watchProgram.getProgram(), it will trigger a resync of the program based on
         * whatever new file content we've given it from our input.
         */
        let updatedProgram = existingWatch.getProgram().getProgram();
        // In case this change causes problems in larger real world codebases
        // Provide an escape hatch so people don't _have_ to revert to an older version
        if (process.env.TSESTREE_NO_INVALIDATION === 'true') {
            return updatedProgram;
        }
        if (hasTSConfigChanged(tsconfigPath)) {
            /*
             * If the stat of the tsconfig has changed, that could mean the include/exclude/files lists has changed
             * We need to make sure typescript knows this so it can update appropriately
             */
            log('tsconfig has changed - triggering program update. %s', tsconfigPath);
            fileWatchCallbackTrackingMap
                .get(tsconfigPath)
                .forEach(cb => cb(tsconfigPath, ts.FileWatcherEventKind.Changed));
            // tsconfig change means that the file list more than likely changed, so clear the cache
            programFileListCache.delete(tsconfigPath);
        }
        let sourceFile = updatedProgram.getSourceFile(filePath);
        if (sourceFile) {
            return updatedProgram;
        }
        /*
         * Missing source file means our program's folder structure might be out of date.
         * So we need to tell typescript it needs to update the correct folder.
         */
        log('File was not found in program - triggering folder update. %s', filePath);
        // Find the correct directory callback by climbing the folder tree
        const currentDir = shared_1.canonicalDirname(filePath);
        let current = null;
        let next = currentDir;
        let hasCallback = false;
        while (current !== next) {
            current = next;
            const folderWatchCallbacks = folderWatchCallbackTrackingMap.get(current);
            if (folderWatchCallbacks) {
                folderWatchCallbacks.forEach(cb => {
                    if (currentDir !== current) {
                        cb(currentDir, ts.FileWatcherEventKind.Changed);
                    }
                    cb(current, ts.FileWatcherEventKind.Changed);
                });
                hasCallback = true;
            }
            next = shared_1.canonicalDirname(current);
        }
        if (!hasCallback) {
            /*
             * No callback means the paths don't matchup - so no point returning any program
             * this will signal to the caller to skip this program
             */
            log('No callback found for file, not part of this program. %s', filePath);
            return null;
        }
        // directory update means that the file list more than likely changed, so clear the cache
        programFileListCache.delete(tsconfigPath);
        // force the immediate resync
        updatedProgram = existingWatch.getProgram().getProgram();
        sourceFile = updatedProgram.getSourceFile(filePath);
        if (sourceFile) {
            return updatedProgram;
        }
        /*
         * At this point we're in one of two states:
         * - The file isn't supposed to be in this program due to exclusions
         * - The file is new, and was renamed from an old, included filename
         *
         * For the latter case, we need to tell typescript that the old filename is now deleted
         */
        log('File was still not found in program after directory update - checking file deletions. %s', filePath);
        const rootFilenames = updatedProgram.getRootFileNames();
        // use find because we only need to "delete" one file to cause typescript to do a full resync
        const deletedFile = rootFilenames.find(file => !fs_1.default.existsSync(file));
        if (!deletedFile) {
            // There are no deleted files, so it must be the former case of the file not belonging to this program
            return null;
        }
        const fileWatchCallbacks = fileWatchCallbackTrackingMap.get(shared_1.getCanonicalFileName(deletedFile));
        if (!fileWatchCallbacks) {
            // shouldn't happen, but just in case
            log('Could not find watch callbacks for root file. %s', deletedFile);
            return updatedProgram;
        }
        log('Marking file as deleted. %s', deletedFile);
        fileWatchCallbacks.forEach(cb => cb(deletedFile, ts.FileWatcherEventKind.Deleted));
        // deleted files means that the file list _has_ changed, so clear the cache
        programFileListCache.delete(tsconfigPath);
        updatedProgram = existingWatch.getProgram().getProgram();
        sourceFile = updatedProgram.getSourceFile(filePath);
        if (sourceFile) {
            return updatedProgram;
        }
        log('File was still not found in program after deletion check, assuming it is not part of this program. %s', filePath);
        return null;
    }
},
/* @typescript-eslint/typescript-estree/dist/create-program/createSourceFile.js */ function _(require, module, exports) {
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const debug_1 = __importDefault(require(120) /* debug */);
    const ts = __importStar(require(44) /* typescript */);
    const shared_1 = require(812) /* ./shared */;
    const log = debug_1.default('typescript-eslint:typescript-estree:createSourceFile');
    function createSourceFile(code, extra) {
        log('Getting AST without type information in %s mode for: %s', extra.jsx ? 'TSX' : 'TS', extra.filePath);
        return ts.createSourceFile(extra.filePath, code, ts.ScriptTarget.Latest, 
        /* setParentNodes */ true, shared_1.getScriptKind(extra));
    }
    exports.createSourceFile = createSourceFile;
},
/* @typescript-eslint/typescript-estree/dist/semantic-or-syntactic-errors.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const ts = __importStar(require(44) /* typescript */);
    /**
     * By default, diagnostics from the TypeScript compiler contain all errors - regardless of whether
     * they are related to generic ECMAScript standards, or TypeScript-specific constructs.
     *
     * Therefore, we filter out all diagnostics, except for the ones we explicitly want to consider when
     * the user opts in to throwing errors on semantic issues.
     */
    function getFirstSemanticOrSyntacticError(program, ast) {
        try {
            const supportedSyntacticDiagnostics = whitelistSupportedDiagnostics(program.getSyntacticDiagnostics(ast));
            if (supportedSyntacticDiagnostics.length) {
                return convertDiagnosticToSemanticOrSyntacticError(supportedSyntacticDiagnostics[0]);
            }
            const supportedSemanticDiagnostics = whitelistSupportedDiagnostics(program.getSemanticDiagnostics(ast));
            if (supportedSemanticDiagnostics.length) {
                return convertDiagnosticToSemanticOrSyntacticError(supportedSemanticDiagnostics[0]);
            }
            return undefined;
        }
        catch (e) {
            /**
             * TypeScript compiler has certain Debug.fail() statements in, which will cause the diagnostics
             * retrieval above to throw.
             *
             * E.g. from ast-alignment-tests
             * "Debug Failure. Shouldn't ever directly check a JsxOpeningElement"
             *
             * For our current use-cases this is undesired behavior, so we just suppress it
             * and log a a warning.
             */
            /* istanbul ignore next */
            console.warn(`Warning From TSC: "${e.message}`); // eslint-disable-line no-console
            /* istanbul ignore next */
            return undefined;
        }
    }
    exports.getFirstSemanticOrSyntacticError = getFirstSemanticOrSyntacticError;
    function whitelistSupportedDiagnostics(diagnostics) {
        return diagnostics.filter(diagnostic => {
            switch (diagnostic.code) {
                case 1013: // "A rest parameter or binding pattern may not have a trailing comma."
                case 1014: // "A rest parameter must be last in a parameter list."
                case 1044: // "'{0}' modifier cannot appear on a module or namespace element."
                case 1045: // "A '{0}' modifier cannot be used with an interface declaration."
                case 1048: // "A rest parameter cannot have an initializer."
                case 1049: // "A 'set' accessor must have exactly one parameter."
                case 1070: // "'{0}' modifier cannot appear on a type member."
                case 1071: // "'{0}' modifier cannot appear on an index signature."
                case 1085: // "Octal literals are not available when targeting ECMAScript 5 and higher. Use the syntax '{0}'."
                case 1090: // "'{0}' modifier cannot appear on a parameter."
                case 1096: // "An index signature must have exactly one parameter."
                case 1097: // "'{0}' list cannot be empty."
                case 1098: // "Type parameter list cannot be empty."
                case 1099: // "Type argument list cannot be empty."
                case 1117: // "An object literal cannot have multiple properties with the same name in strict mode."
                case 1121: // "Octal literals are not allowed in strict mode."
                case 1123: //  "Variable declaration list cannot be empty."
                case 1141: // "String literal expected."
                case 1162: // "An object member cannot be declared optional."
                case 1164: // "Computed property names are not allowed in enums."
                case 1172: // "'extends' clause already seen."
                case 1173: // "'extends' clause must precede 'implements' clause."
                case 1175: // "'implements' clause already seen."
                case 1176: // "Interface declaration cannot have 'implements' clause."
                case 1190: // "The variable declaration of a 'for...of' statement cannot have an initializer."
                case 1200: // "Line terminator not permitted before arrow."
                case 1206: // "Decorators are not valid here."
                case 1211: // "A class declaration without the 'default' modifier must have a name."
                case 1242: // "'abstract' modifier can only appear on a class, method, or property declaration."
                case 1246: // "An interface property cannot have an initializer."
                case 1255: // "A definite assignment assertion '!' is not permitted in this context."
                case 1308: // "'await' expression is only allowed within an async function."
                case 2364: // "The left-hand side of an assignment expression must be a variable or a property access."
                case 2369: // "A parameter property is only allowed in a constructor implementation."
                case 2452: // "An enum member cannot have a numeric name."
                case 2462: // "A rest element must be last in a destructuring pattern."
                case 8017: // "Octal literal types must use ES2015 syntax. Use the syntax '{0}'."
                case 17012: // "'{0}' is not a valid meta-property for keyword '{1}'. Did you mean '{2}'?"
                case 17013: // "Meta-property '{0}' is only allowed in the body of a function declaration, function expression, or constructor."
                    return true;
            }
            return false;
        });
    }
    function convertDiagnosticToSemanticOrSyntacticError(diagnostic) {
        return Object.assign(Object.assign({}, diagnostic), { message: ts.flattenDiagnosticMessageText(diagnostic.messageText, ts.sys.newLine) });
    }
},
/* @typescript-eslint/typescript-estree/package.json */ function _(require, module, exports) {
    module.exports = {
        "_args": [
            [
                "@typescript-eslint/typescript-estree@2.24.0",
                "/Users/bryan/work/bokeh/bokehjs"
            ]
        ],
        "_development": true,
        "_from": "@typescript-eslint/typescript-estree@2.24.0",
        "_id": "@typescript-eslint/typescript-estree@2.24.0",
        "_inBundle": false,
        "_integrity": "sha512-RJ0yMe5owMSix55qX7Mi9V6z2FDuuDpN6eR5fzRJrp+8in9UF41IGNQHbg5aMK4/PjVaEQksLvz0IA8n+Mr/FA==",
        "_location": "/@typescript-eslint/typescript-estree",
        "_phantomChildren": {},
        "_requested": {
            "type": "version",
            "registry": true,
            "raw": "@typescript-eslint/typescript-estree@2.24.0",
            "name": "@typescript-eslint/typescript-estree",
            "escapedName": "@typescript-eslint%2ftypescript-estree",
            "scope": "@typescript-eslint",
            "rawSpec": "2.24.0",
            "saveSpec": null,
            "fetchSpec": "2.24.0"
        },
        "_requiredBy": [
            "/@typescript-eslint/experimental-utils",
            "/@typescript-eslint/parser"
        ],
        "_resolved": "https://registry.npmjs.org/@typescript-eslint/typescript-estree/-/typescript-estree-2.24.0.tgz",
        "_spec": "2.24.0",
        "_where": "/Users/bryan/work/bokeh/bokehjs",
        "bugs": {
            "url": "https://github.com/typescript-eslint/typescript-eslint/issues"
        },
        "dependencies": {
            "debug": "^4.1.1",
            "eslint-visitor-keys": "^1.1.0",
            "glob": "^7.1.6",
            "is-glob": "^4.0.1",
            "lodash": "^4.17.15",
            "semver": "^6.3.0",
            "tsutils": "^3.17.1"
        },
        "description": "A parser that converts TypeScript source code into an ESTree compatible form",
        "devDependencies": {
            "@babel/code-frame": "^7.8.3",
            "@babel/parser": "^7.8.3",
            "@babel/types": "^7.8.3",
            "@types/babel__code-frame": "^7.0.1",
            "@types/debug": "^4.1.5",
            "@types/glob": "^7.1.1",
            "@types/is-glob": "^4.0.1",
            "@types/lodash": "^4.14.149",
            "@types/semver": "^6.2.0",
            "@types/tmp": "^0.1.0",
            "@typescript-eslint/shared-fixtures": "2.24.0",
            "tmp": "^0.1.0",
            "typescript": "*"
        },
        "engines": {
            "node": "^8.10.0 || ^10.13.0 || >=11.10.1"
        },
        "files": [
            "dist",
            "README.md",
            "LICENSE"
        ],
        "funding": {
            "type": "opencollective",
            "url": "https://opencollective.com/typescript-eslint"
        },
        "gitHead": "56e1e160a36cecc298dd1187ea035e1482489dd0",
        "homepage": "https://github.com/typescript-eslint/typescript-eslint#readme",
        "keywords": [
            "ast",
            "estree",
            "ecmascript",
            "javascript",
            "typescript",
            "parser",
            "syntax"
        ],
        "license": "BSD-2-Clause",
        "main": "dist/parser.js",
        "name": "@typescript-eslint/typescript-estree",
        "peerDependenciesMeta": {
            "typescript": {
                "optional": true
            }
        },
        "repository": {
            "type": "git",
            "url": "git+https://github.com/typescript-eslint/typescript-eslint.git",
            "directory": "packages/typescript-estree"
        },
        "scripts": {
            "build": "tsc -b tsconfig.build.json",
            "clean": "tsc -b tsconfig.build.json --clean",
            "format": "prettier --write \"./**/*.{ts,js,json,md}\" --ignore-path ../../.prettierignore",
            "lint": "eslint . --ext .js,.ts --ignore-path='../../.eslintignore'",
            "test": "jest --coverage",
            "typecheck": "tsc -p tsconfig.json --noEmit"
        },
        "types": "dist/parser.d.ts",
        "version": "2.24.0"
    };
},
/* @typescript-eslint/parser/dist/analyze-scope.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const eslint_visitor_keys_1 = require(100) /* eslint-visitor-keys */;
    const scope_manager_1 = require(820) /* ./scope/scope-manager */;
    const typescript_estree_1 = require(748) /* @typescript-eslint/typescript-estree */;
    /**
     * Define the override function of `Scope#__define` for global augmentation.
     * @param {Function} define The original Scope#__define method.
     * @returns {Function} The override function.
     */
    function overrideDefine(define) {
        return function (node, definition) {
            define.call(this, node, definition);
            // Set `variable.eslintUsed` to tell ESLint that the variable is exported.
            const variable = 'name' in node &&
                typeof node.name === 'string' &&
                this.set.get(node.name);
            if (variable) {
                variable.eslintUsed = true;
            }
        };
    }
    class PatternVisitor extends experimental_utils_1.TSESLintScope.PatternVisitor {
        constructor(options, rootPattern, callback) {
            super(options, rootPattern, callback);
        }
        Identifier(node) {
            super.Identifier(node);
            if (node.decorators) {
                this.rightHandNodes.push(...node.decorators);
            }
            if (node.typeAnnotation) {
                this.rightHandNodes.push(node.typeAnnotation);
            }
        }
        ArrayPattern(node) {
            node.elements.forEach(this.visit, this);
            if (node.decorators) {
                this.rightHandNodes.push(...node.decorators);
            }
            if (node.typeAnnotation) {
                this.rightHandNodes.push(node.typeAnnotation);
            }
        }
        ObjectPattern(node) {
            node.properties.forEach(this.visit, this);
            if (node.decorators) {
                this.rightHandNodes.push(...node.decorators);
            }
            if (node.typeAnnotation) {
                this.rightHandNodes.push(node.typeAnnotation);
            }
        }
        RestElement(node) {
            super.RestElement(node);
            if (node.decorators) {
                this.rightHandNodes.push(...node.decorators);
            }
            if (node.typeAnnotation) {
                this.rightHandNodes.push(node.typeAnnotation);
            }
        }
        TSParameterProperty(node) {
            this.visit(node.parameter);
            if (node.decorators) {
                this.rightHandNodes.push(...node.decorators);
            }
        }
    }
    class Referencer extends experimental_utils_1.TSESLintScope.Referencer {
        constructor(options, scopeManager) {
            super(options, scopeManager);
            this.typeMode = false;
        }
        /**
         * Override to use PatternVisitor we overrode.
         * @param node The Identifier node to visit.
         * @param [options] The flag to visit right-hand side nodes.
         * @param callback The callback function for left-hand side nodes.
         */
        visitPattern(node, options, callback) {
            if (!node) {
                return;
            }
            if (typeof options === 'function') {
                callback = options;
                options = { processRightHandNodes: false };
            }
            const visitor = new PatternVisitor(this.options, node, callback);
            visitor.visit(node);
            if (options.processRightHandNodes) {
                visitor.rightHandNodes.forEach(this.visit, this);
            }
        }
        /**
         * Override.
         * Visit `node.typeParameters` and `node.returnType` additionally to find `typeof` expressions.
         * @param node The function node to visit.
         */
        visitFunction(node) {
            const { type, id, typeParameters, params, returnType, body } = node;
            const scopeManager = this.scopeManager;
            const upperScope = this.currentScope();
            // Process the name.
            if (type === experimental_utils_1.AST_NODE_TYPES.FunctionDeclaration && id) {
                upperScope.__define(id, new experimental_utils_1.TSESLintScope.Definition('FunctionName', id, node, null, null, null));
                // Remove overload definition to avoid confusion of no-redeclare rule.
                const { defs, identifiers } = upperScope.set.get(id.name);
                for (let i = 0; i < defs.length; ++i) {
                    const def = defs[i];
                    if (def.type === 'FunctionName' &&
                        def.node.type === experimental_utils_1.AST_NODE_TYPES.TSDeclareFunction) {
                        defs.splice(i, 1);
                        identifiers.splice(i, 1);
                        break;
                    }
                }
            }
            else if (type === experimental_utils_1.AST_NODE_TYPES.FunctionExpression && id) {
                scopeManager.__nestFunctionExpressionNameScope(node);
            }
            // Open the function scope.
            scopeManager.__nestFunctionScope(node, this.isInnerMethodDefinition);
            const innerScope = this.currentScope();
            // Process the type parameters
            this.visit(typeParameters);
            // Process parameter declarations.
            for (let i = 0; i < params.length; ++i) {
                this.visitPattern(params[i], { processRightHandNodes: true }, (pattern, info) => {
                    if (pattern.type !== experimental_utils_1.AST_NODE_TYPES.Identifier ||
                        pattern.name !== 'this') {
                        innerScope.__define(pattern, new experimental_utils_1.TSESLintScope.ParameterDefinition(pattern, node, i, info.rest));
                        this.referencingDefaultValue(pattern, info.assignments, null, true);
                    }
                });
            }
            // Process the return type.
            this.visit(returnType);
            // Process the body.
            if (body && body.type === experimental_utils_1.AST_NODE_TYPES.BlockStatement) {
                this.visitChildren(body);
            }
            else {
                this.visit(body);
            }
            // Close the function scope.
            this.close(node);
        }
        /**
         * Override.
         * Visit decorators.
         * @param node The class node to visit.
         */
        visitClass(node) {
            this.visitDecorators(node.decorators);
            const upperTypeMode = this.typeMode;
            this.typeMode = true;
            if (node.superTypeParameters) {
                this.visit(node.superTypeParameters);
            }
            if (node.implements) {
                node.implements.forEach(this.visit, this);
            }
            this.typeMode = upperTypeMode;
            super.visitClass(node);
        }
        /**
         * Visit typeParameters.
         * @param node The node to visit.
         */
        visitTypeParameters(node) {
            if (node.typeParameters) {
                const upperTypeMode = this.typeMode;
                this.typeMode = true;
                this.visit(node.typeParameters);
                this.typeMode = upperTypeMode;
            }
        }
        /**
         * Override.
         */
        JSXOpeningElement(node) {
            this.visit(node.name);
            this.visitTypeParameters(node);
            node.attributes.forEach(this.visit, this);
        }
        /**
         * Override.
         * Don't create the reference object in the type mode.
         * @param node The Identifier node to visit.
         */
        Identifier(node) {
            this.visitDecorators(node.decorators);
            if (!this.typeMode) {
                super.Identifier(node);
            }
            this.visit(node.typeAnnotation);
        }
        /**
         * Override.
         * Visit decorators.
         * @param node The MethodDefinition node to visit.
         */
        MethodDefinition(node) {
            this.visitDecorators(node.decorators);
            super.MethodDefinition(node);
        }
        /**
         * Don't create the reference object for the key if not computed.
         * @param node The ClassProperty node to visit.
         */
        ClassProperty(node) {
            const upperTypeMode = this.typeMode;
            const { computed, decorators, key, typeAnnotation, value } = node;
            this.typeMode = false;
            this.visitDecorators(decorators);
            if (computed) {
                this.visit(key);
            }
            this.typeMode = true;
            this.visit(typeAnnotation);
            this.typeMode = false;
            this.visit(value);
            this.typeMode = upperTypeMode;
        }
        /**
         * Visit new expression.
         * @param node The NewExpression node to visit.
         */
        NewExpression(node) {
            this.visitTypeParameters(node);
            this.visit(node.callee);
            node.arguments.forEach(this.visit, this);
        }
        /**
         * Override.
         * Visit call expression.
         * @param node The CallExpression node to visit.
         */
        CallExpression(node) {
            this.visitTypeParameters(node);
            this.visit(node.callee);
            node.arguments.forEach(this.visit, this);
        }
        /**
         * Visit optional member expression.
         * @param node The OptionalMemberExpression node to visit.
         */
        OptionalMemberExpression(node) {
            this.visit(node.object);
            if (node.computed) {
                this.visit(node.property);
            }
        }
        /**
         * Visit optional call expression.
         * @param node The OptionalMemberExpression node to visit.
         */
        OptionalCallExpression(node) {
            this.visitTypeParameters(node);
            this.visit(node.callee);
            node.arguments.forEach(this.visit, this);
        }
        /**
         * Define the variable of this function declaration only once.
         * Because to avoid confusion of `no-redeclare` rule by overloading.
         * @param node The TSDeclareFunction node to visit.
         */
        TSDeclareFunction(node) {
            const scopeManager = this.scopeManager;
            const upperScope = this.currentScope();
            const { id, typeParameters, params, returnType } = node;
            // Ignore this if other overload have already existed.
            if (id) {
                const variable = upperScope.set.get(id.name);
                const defs = variable === null || variable === void 0 ? void 0 : variable.defs;
                const existed = defs === null || defs === void 0 ? void 0 : defs.some((d) => d.type === 'FunctionName');
                if (!existed) {
                    upperScope.__define(id, new experimental_utils_1.TSESLintScope.Definition('FunctionName', id, node, null, null, null));
                }
            }
            // Open the function scope.
            scopeManager.__nestEmptyFunctionScope(node);
            const innerScope = this.currentScope();
            // Process the type parameters
            this.visit(typeParameters);
            // Process parameter declarations.
            for (let i = 0; i < params.length; ++i) {
                this.visitPattern(params[i], { processRightHandNodes: true }, (pattern, info) => {
                    innerScope.__define(pattern, new experimental_utils_1.TSESLintScope.ParameterDefinition(pattern, node, i, info.rest));
                    // Set `variable.eslintUsed` to tell ESLint that the variable is used.
                    const variable = innerScope.set.get(pattern.name);
                    if (variable) {
                        variable.eslintUsed = true;
                    }
                    this.referencingDefaultValue(pattern, info.assignments, null, true);
                });
            }
            // Process the return type.
            this.visit(returnType);
            // Close the function scope.
            this.close(node);
        }
        /**
         * Create reference objects for the references in parameters and return type.
         * @param node The TSEmptyBodyFunctionExpression node to visit.
         */
        TSEmptyBodyFunctionExpression(node) {
            const upperTypeMode = this.typeMode;
            const { typeParameters, params, returnType } = node;
            this.typeMode = true;
            this.visit(typeParameters);
            params.forEach(this.visit, this);
            this.visit(returnType);
            this.typeMode = upperTypeMode;
        }
        /**
         * Don't make variable because it declares only types.
         * Switch to the type mode and visit child nodes to find `typeof x` expression in type declarations.
         * @param node The TSInterfaceDeclaration node to visit.
         */
        TSInterfaceDeclaration(node) {
            this.visitTypeNodes(node);
        }
        /**
         * Don't make variable because it declares only types.
         * Switch to the type mode and visit child nodes to find `typeof x` expression in type declarations.
         * @param node The TSClassImplements node to visit.
         */
        TSClassImplements(node) {
            this.visitTypeNodes(node);
        }
        /**
         * Don't make variable because it declares only types.
         * Switch to the type mode and visit child nodes to find `typeof x` expression in type declarations.
         * @param node The TSIndexSignature node to visit.
         */
        TSIndexSignature(node) {
            this.visitTypeNodes(node);
        }
        /**
         * Visit type assertion.
         * @param node The TSTypeAssertion node to visit.
         */
        TSTypeAssertion(node) {
            if (this.typeMode) {
                this.visit(node.typeAnnotation);
            }
            else {
                this.typeMode = true;
                this.visit(node.typeAnnotation);
                this.typeMode = false;
            }
            this.visit(node.expression);
        }
        /**
         * Visit as expression.
         * @param node The TSAsExpression node to visit.
         */
        TSAsExpression(node) {
            this.visit(node.expression);
            if (this.typeMode) {
                this.visit(node.typeAnnotation);
            }
            else {
                this.typeMode = true;
                this.visit(node.typeAnnotation);
                this.typeMode = false;
            }
        }
        /**
         * Switch to the type mode and visit child nodes to find `typeof x` expression in type declarations.
         * @param node The TSTypeAnnotation node to visit.
         */
        TSTypeAnnotation(node) {
            this.visitTypeNodes(node);
        }
        /**
         * Switch to the type mode and visit child nodes to find `typeof x` expression in type declarations.
         * @param node The TSTypeParameterDeclaration node to visit.
         */
        TSTypeParameterDeclaration(node) {
            this.visitTypeNodes(node);
        }
        /**
         * Create reference objects for the references in `typeof` expression.
         * @param node The TSTypeQuery node to visit.
         */
        TSTypeQuery(node) {
            if (this.typeMode) {
                this.typeMode = false;
                this.visitChildren(node);
                this.typeMode = true;
            }
            else {
                this.visitChildren(node);
            }
        }
        /**
         * @param node The TSTypeParameter node to visit.
         */
        TSTypeParameter(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSInferType node to visit.
         */
        TSInferType(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSTypeReference node to visit.
         */
        TSTypeReference(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSTypeLiteral node to visit.
         */
        TSTypeLiteral(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSLiteralType node to visit.
         */
        TSLiteralType(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSIntersectionType node to visit.
         */
        TSIntersectionType(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSConditionalType node to visit.
         */
        TSConditionalType(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSIndexedAccessType node to visit.
         */
        TSIndexedAccessType(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSMappedType node to visit.
         */
        TSMappedType(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSOptionalType node to visit.
         */
        TSOptionalType(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSParenthesizedType node to visit.
         */
        TSParenthesizedType(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSRestType node to visit.
         */
        TSRestType(node) {
            this.visitTypeNodes(node);
        }
        /**
         * @param node The TSTupleType node to visit.
         */
        TSTupleType(node) {
            this.visitTypeNodes(node);
        }
        /**
         * Create reference objects for the object part. (This is `obj.prop`)
         * @param node The TSQualifiedName node to visit.
         */
        TSQualifiedName(node) {
            this.visit(node.left);
        }
        /**
         * Create reference objects for the references in computed keys.
         * @param node The TSPropertySignature node to visit.
         */
        TSPropertySignature(node) {
            const upperTypeMode = this.typeMode;
            const { computed, key, typeAnnotation, initializer } = node;
            if (computed) {
                this.typeMode = false;
                this.visit(key);
                this.typeMode = true;
            }
            else {
                this.typeMode = true;
                this.visit(key);
            }
            this.visit(typeAnnotation);
            this.visit(initializer);
            this.typeMode = upperTypeMode;
        }
        /**
         * Create reference objects for the references in computed keys.
         * @param node The TSMethodSignature node to visit.
         */
        TSMethodSignature(node) {
            const upperTypeMode = this.typeMode;
            const { computed, key, typeParameters, params, returnType } = node;
            if (computed) {
                this.typeMode = false;
                this.visit(key);
                this.typeMode = true;
            }
            else {
                this.typeMode = true;
                this.visit(key);
            }
            this.visit(typeParameters);
            params.forEach(this.visit, this);
            this.visit(returnType);
            this.typeMode = upperTypeMode;
        }
        /**
         * Create variable object for the enum.
         * The enum declaration creates a scope for the enum members.
         *
         * enum E {
         *   A,
         *   B,
         *   C = A + B // A and B are references to the enum member.
         * }
         *
         * const a = 0
         * enum E {
         *   A = a // a is above constant.
         * }
         *
         * @param node The TSEnumDeclaration node to visit.
         */
        TSEnumDeclaration(node) {
            const { id, members } = node;
            const scopeManager = this.scopeManager;
            const scope = this.currentScope();
            if (id) {
                scope.__define(id, new experimental_utils_1.TSESLintScope.Definition('EnumName', id, node));
            }
            scopeManager.__nestEnumScope(node);
            for (const member of members) {
                this.visit(member);
            }
            this.close(node);
        }
        /**
         * Create variable object for the enum member and create reference object for the initializer.
         * And visit the initializer.
         *
         * @param node The TSEnumMember node to visit.
         */
        TSEnumMember(node) {
            const { id, initializer } = node;
            const scope = this.currentScope();
            scope.__define(id, new experimental_utils_1.TSESLintScope.Definition('EnumMemberName', id, node));
            if (initializer) {
                scope.__referencing(id, experimental_utils_1.TSESLintScope.Reference.WRITE, initializer, null, false, true);
                this.visit(initializer);
            }
        }
        /**
         * Create the variable object for the module name, and visit children.
         * @param node The TSModuleDeclaration node to visit.
         */
        TSModuleDeclaration(node) {
            const scope = this.currentScope();
            const { id, body } = node;
            if (node.global) {
                this.visitGlobalAugmentation(node);
                return;
            }
            if (id && id.type === experimental_utils_1.AST_NODE_TYPES.Identifier) {
                scope.__define(id, new experimental_utils_1.TSESLintScope.Definition('NamespaceName', id, node, null, null, null));
            }
            this.visit(body);
        }
        TSTypeAliasDeclaration(node) {
            this.typeMode = true;
            this.visitChildren(node);
            this.typeMode = false;
        }
        /**
         * Process the module block.
         * @param node The TSModuleBlock node to visit.
         */
        TSModuleBlock(node) {
            this.scopeManager.__nestBlockScope(node);
            this.visitChildren(node);
            this.close(node);
        }
        TSAbstractClassProperty(node) {
            this.ClassProperty(node);
        }
        TSAbstractMethodDefinition(node) {
            this.MethodDefinition(node);
        }
        /**
         * Process import equal declaration
         * @param node The TSImportEqualsDeclaration node to visit.
         */
        TSImportEqualsDeclaration(node) {
            const { id, moduleReference } = node;
            if (id && id.type === experimental_utils_1.AST_NODE_TYPES.Identifier) {
                this.currentScope().__define(id, new experimental_utils_1.TSESLintScope.Definition('ImportBinding', id, node, null, null, null));
            }
            this.visit(moduleReference);
        }
        /**
         * Process the global augmentation.
         * 1. Set the global scope as the current scope.
         * 2. Configure the global scope to set `variable.eslintUsed = true` for all defined variables. This means `no-unused-vars` doesn't warn those.
         * @param node The TSModuleDeclaration node to visit.
         */
        visitGlobalAugmentation(node) {
            const scopeManager = this.scopeManager;
            const currentScope = this.currentScope();
            const globalScope = scopeManager.globalScope;
            const originalDefine = globalScope.__define;
            globalScope.__define = overrideDefine(originalDefine);
            scopeManager.__currentScope = globalScope;
            // Skip TSModuleBlock to avoid to create that block scope.
            if (node.body && node.body.type === experimental_utils_1.AST_NODE_TYPES.TSModuleBlock) {
                node.body.body.forEach(this.visit, this);
            }
            scopeManager.__currentScope = currentScope;
            globalScope.__define = originalDefine;
        }
        /**
         * Process decorators.
         * @param decorators The decorator nodes to visit.
         */
        visitDecorators(decorators) {
            if (decorators) {
                decorators.forEach(this.visit, this);
            }
        }
        /**
         * Process all child of type nodes
         * @param node node to be processed
         */
        visitTypeNodes(node) {
            if (this.typeMode) {
                this.visitChildren(node);
            }
            else {
                this.typeMode = true;
                this.visitChildren(node);
                this.typeMode = false;
            }
        }
    }
    function analyzeScope(ast, parserOptions) {
        var _a;
        const options = {
            ignoreEval: true,
            optimistic: false,
            directive: false,
            nodejsScope: parserOptions.sourceType === 'script' &&
                (parserOptions.ecmaFeatures &&
                    parserOptions.ecmaFeatures.globalReturn) === true,
            impliedStrict: false,
            sourceType: parserOptions.sourceType,
            ecmaVersion: (_a = parserOptions.ecmaVersion) !== null && _a !== void 0 ? _a : 2018,
            childVisitorKeys: typescript_estree_1.visitorKeys,
            fallback: eslint_visitor_keys_1.getKeys,
        };
        const scopeManager = new scope_manager_1.ScopeManager(options);
        const referencer = new Referencer(options, scopeManager);
        referencer.visit(ast);
        return scopeManager;
    }
    exports.analyzeScope = analyzeScope;
},
/* @typescript-eslint/parser/dist/scope/scope-manager.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    const scopes_1 = require(821) /* ./scopes */;
    /**
     * based on eslint-scope
     */
    class ScopeManager extends experimental_utils_1.TSESLintScope.ScopeManager {
        constructor(options) {
            super(options);
        }
        /** @internal */
        __nestEnumScope(node) {
            return this.__nestScope(new scopes_1.EnumScope(this, this.__currentScope, node));
        }
        /** @internal */
        __nestEmptyFunctionScope(node) {
            return this.__nestScope(new scopes_1.EmptyFunctionScope(this, this.__currentScope, node));
        }
    }
    exports.ScopeManager = ScopeManager;
},
/* @typescript-eslint/parser/dist/scope/scopes.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    const experimental_utils_1 = require(598) /* @typescript-eslint/experimental-utils */;
    /** The scope class for enum. */
    class EnumScope extends experimental_utils_1.TSESLintScope.Scope {
        constructor(scopeManager, upperScope, block) {
            super(scopeManager, 'enum', upperScope, block, false);
        }
    }
    exports.EnumScope = EnumScope;
    /** The scope class for empty functions. */
    class EmptyFunctionScope extends experimental_utils_1.TSESLintScope.Scope {
        constructor(scopeManager, upperScope, block) {
            super(scopeManager, 'empty-function', upperScope, block, false);
        }
    }
    exports.EmptyFunctionScope = EmptyFunctionScope;
},
/* @typescript-eslint/parser/package.json */ function _(require, module, exports) {
    module.exports = {
        "_args": [
            [
                "@typescript-eslint/parser@2.24.0",
                "/Users/bryan/work/bokeh/bokehjs"
            ]
        ],
        "_development": true,
        "_from": "@typescript-eslint/parser@2.24.0",
        "_id": "@typescript-eslint/parser@2.24.0",
        "_inBundle": false,
        "_integrity": "sha512-H2Y7uacwSSg8IbVxdYExSI3T7uM1DzmOn2COGtCahCC3g8YtM1xYAPi2MAHyfPs61VKxP/J/UiSctcRgw4G8aw==",
        "_location": "/@typescript-eslint/parser",
        "_phantomChildren": {},
        "_requested": {
            "type": "version",
            "registry": true,
            "raw": "@typescript-eslint/parser@2.24.0",
            "name": "@typescript-eslint/parser",
            "escapedName": "@typescript-eslint%2fparser",
            "scope": "@typescript-eslint",
            "rawSpec": "2.24.0",
            "saveSpec": null,
            "fetchSpec": "2.24.0"
        },
        "_requiredBy": [
            "#DEV:/"
        ],
        "_resolved": "https://registry.npmjs.org/@typescript-eslint/parser/-/parser-2.24.0.tgz",
        "_spec": "2.24.0",
        "_where": "/Users/bryan/work/bokeh/bokehjs",
        "bugs": {
            "url": "https://github.com/typescript-eslint/typescript-eslint/issues"
        },
        "dependencies": {
            "@types/eslint-visitor-keys": "^1.0.0",
            "@typescript-eslint/experimental-utils": "2.24.0",
            "@typescript-eslint/typescript-estree": "2.24.0",
            "eslint-visitor-keys": "^1.1.0"
        },
        "description": "An ESLint custom parser which leverages TypeScript ESTree",
        "devDependencies": {
            "@types/glob": "^7.1.1",
            "@typescript-eslint/shared-fixtures": "2.24.0",
            "glob": "*"
        },
        "engines": {
            "node": "^8.10.0 || ^10.13.0 || >=11.10.1"
        },
        "files": [
            "dist",
            "README.md",
            "LICENSE"
        ],
        "funding": {
            "type": "opencollective",
            "url": "https://opencollective.com/typescript-eslint"
        },
        "gitHead": "56e1e160a36cecc298dd1187ea035e1482489dd0",
        "homepage": "https://github.com/typescript-eslint/typescript-eslint#readme",
        "keywords": [
            "ast",
            "ecmascript",
            "javascript",
            "typescript",
            "parser",
            "syntax",
            "eslint"
        ],
        "license": "BSD-2-Clause",
        "main": "dist/parser.js",
        "name": "@typescript-eslint/parser",
        "peerDependencies": {
            "eslint": "^5.0.0 || ^6.0.0"
        },
        "peerDependenciesMeta": {
            "typescript": {
                "optional": true
            }
        },
        "repository": {
            "type": "git",
            "url": "git+https://github.com/typescript-eslint/typescript-eslint.git",
            "directory": "packages/parser"
        },
        "scripts": {
            "build": "tsc -b tsconfig.build.json",
            "clean": "tsc -b tsconfig.build.json --clean",
            "format": "prettier --write \"./**/*.{ts,js,json,md}\" --ignore-path ../../.prettierignore",
            "lint": "eslint . --ext .js,.ts --ignore-path='../../.eslintignore'",
            "test": "jest --coverage",
            "typecheck": "tsc -p tsconfig.json --noEmit"
        },
        "types": "dist/parser.d.ts",
        "version": "2.24.0"
    };
},
/* compile.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    var __importDefault = (this && this.__importDefault) || function (mod) {
        return (mod && mod.__esModule) ? mod : { "default": mod };
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    const path = __importStar(require("path"));
    const ts = __importStar(require(44) /* typescript */);
    const less_1 = __importDefault(require(824) /* less */);
    const compiler_1 = require(59) /* ./compiler */;
    const sys_1 = require(43) /* ./sys */;
    const transforms = __importStar(require(65) /* ./transforms */);
    const tsconfig_json = __importStar(require(76) /* ./tsconfig.ext.json */);
    function parse_patched_tsconfig(base_dir, preconfigure) {
        // XXX: silence the config validator. We are providing inputs through `inputs` argument anyway.
        const json = Object.assign(Object.assign({}, tsconfig_json), { include: undefined, files: ["dummy.ts"] });
        return compiler_1.parse_tsconfig(json, base_dir, preconfigure);
    }
    function compile_typescript(base_dir, inputs, bokehjs_dir) {
        const preconfigure = {
            module: ts.ModuleKind.CommonJS,
            paths: {
                "*": [
                    path.join(bokehjs_dir, "js/lib/*"),
                    path.join(bokehjs_dir, "js/types/*"),
                ],
            },
            outDir: undefined,
        };
        const tsconfig = parse_patched_tsconfig(base_dir, preconfigure);
        if (tsconfig.diagnostics != null)
            return { diagnostics: tsconfig.diagnostics };
        const host = compiler_1.compiler_host(inputs, tsconfig.options, bokehjs_dir);
        const transformers = compiler_1.default_transformers(tsconfig.options);
        const outputs = new Map();
        host.writeFile = (name, data) => {
            outputs.set(name, data);
        };
        const files = [...inputs.keys()];
        return Object.assign({ outputs }, compiler_1.compile_files(files, tsconfig.options, transformers, host));
    }
    exports.compile_typescript = compile_typescript;
    function compile_javascript(base_dir, file, code) {
        const tsconfig = parse_patched_tsconfig(base_dir, {});
        if (tsconfig.diagnostics != null)
            return { diagnostics: tsconfig.diagnostics };
        const { outputText, diagnostics } = ts.transpileModule(code, {
            fileName: file,
            reportDiagnostics: true,
            compilerOptions: {
                target: tsconfig.options.target,
                module: ts.ModuleKind.CommonJS,
            },
        });
        return { output: outputText, diagnostics };
    }
    function normalize(path) {
        return path.replace(/\\/g, "/");
    }
    async function compile_and_resolve_deps(input) {
        const { file, lang, bokehjs_dir } = input;
        const { code } = input;
        let output;
        switch (lang) {
            case "typescript":
                const inputs = new Map([[normalize(file), code]]);
                const { outputs, diagnostics } = compile_typescript(".", inputs, bokehjs_dir);
                if (diagnostics != null && diagnostics.length != 0) {
                    const failure = compiler_1.report_diagnostics(diagnostics);
                    return { error: failure.text };
                }
                else {
                    const js_file = normalize(sys_1.rename(file, { ext: ".js" }));
                    output = outputs.get(js_file);
                }
                break;
            case "javascript": {
                const result = compile_javascript(".", file, code);
                if (result.diagnostics != null && result.diagnostics.length != 0) {
                    const failure = compiler_1.report_diagnostics(result.diagnostics);
                    return { error: failure.text };
                }
                else {
                    output = result.output;
                }
                break;
            }
            case "less":
                try {
                    const { css } = await less_1.default.render(code, { filename: file, compress: true });
                    return { code: css };
                }
                catch (error) {
                    return { error: error.toString() };
                }
            default:
                throw new Error(`unsupported input type: ${lang}`);
        }
        const source = ts.createSourceFile(file, output, ts.ScriptTarget.ES5, true, ts.ScriptKind.JS);
        const deps = transforms.collect_deps(source);
        return { code: output, deps };
    }
    exports.compile_and_resolve_deps = compile_and_resolve_deps;
},
/* less/index.js */ function _(require, module, exports) {
    module.exports = require(825) /* ./dist/less.cjs.js */;
},
/* less/dist/less.cjs.js */ function _(require, module, exports) {
    var tslib = require(628) /* tslib */;
    var path = require('path');
    var url = require('url');
    var CloneHelper = require(826) /* clone */;
    var environment = {
        encodeBase64: function encodeBase64(str) {
            // Avoid Buffer constructor on newer versions of Node.js.
            var buffer = (Buffer.from ? Buffer.from(str) : (new Buffer(str)));
            return buffer.toString('base64');
        },
        mimeLookup: function (filename) {
            return require(827) /* mime */.lookup(filename);
        },
        charsetLookup: function (mime) {
            return require(827) /* mime */.charsets.lookup(mime);
        },
        getSourceMapGenerator: function getSourceMapGenerator() {
            return require(46) /* source-map */.SourceMapGenerator;
        }
    };
    var fs;
    try {
        fs = require(829) /* graceful-fs */;
    }
    catch (e) {
        fs = require('fs');
    }
    var fs$1 = fs;
    var AbstractFileManager = /** @class */ (function () {
        function AbstractFileManager() {
        }
        AbstractFileManager.prototype.getPath = function (filename) {
            var j = filename.lastIndexOf('?');
            if (j > 0) {
                filename = filename.slice(0, j);
            }
            j = filename.lastIndexOf('/');
            if (j < 0) {
                j = filename.lastIndexOf('\\');
            }
            if (j < 0) {
                return '';
            }
            return filename.slice(0, j + 1);
        };
        AbstractFileManager.prototype.tryAppendExtension = function (path, ext) {
            return /(\.[a-z]*$)|([\?;].*)$/.test(path) ? path : path + ext;
        };
        AbstractFileManager.prototype.tryAppendLessExtension = function (path) {
            return this.tryAppendExtension(path, '.less');
        };
        AbstractFileManager.prototype.supportsSync = function () { return false; };
        AbstractFileManager.prototype.alwaysMakePathsAbsolute = function () { return false; };
        AbstractFileManager.prototype.isPathAbsolute = function (filename) {
            return (/^(?:[a-z-]+:|\/|\\|#)/i).test(filename);
        };
        // TODO: pull out / replace?
        AbstractFileManager.prototype.join = function (basePath, laterPath) {
            if (!basePath) {
                return laterPath;
            }
            return basePath + laterPath;
        };
        AbstractFileManager.prototype.pathDiff = function (url, baseUrl) {
            // diff between two paths to create a relative path
            var urlParts = this.extractUrlParts(url);
            var baseUrlParts = this.extractUrlParts(baseUrl);
            var i;
            var max;
            var urlDirectories;
            var baseUrlDirectories;
            var diff = '';
            if (urlParts.hostPart !== baseUrlParts.hostPart) {
                return '';
            }
            max = Math.max(baseUrlParts.directories.length, urlParts.directories.length);
            for (i = 0; i < max; i++) {
                if (baseUrlParts.directories[i] !== urlParts.directories[i]) {
                    break;
                }
            }
            baseUrlDirectories = baseUrlParts.directories.slice(i);
            urlDirectories = urlParts.directories.slice(i);
            for (i = 0; i < baseUrlDirectories.length - 1; i++) {
                diff += '../';
            }
            for (i = 0; i < urlDirectories.length - 1; i++) {
                diff += urlDirectories[i] + "/";
            }
            return diff;
        };
        // helper function, not part of API
        AbstractFileManager.prototype.extractUrlParts = function (url, baseUrl) {
            // urlParts[1] = protocol://hostname/ OR /
            // urlParts[2] = / if path relative to host base
            // urlParts[3] = directories
            // urlParts[4] = filename
            // urlParts[5] = parameters
            var urlPartsRegex = /^((?:[a-z-]+:)?\/{2}(?:[^\/\?#]*\/)|([\/\\]))?((?:[^\/\\\?#]*[\/\\])*)([^\/\\\?#]*)([#\?].*)?$/i;
            var urlParts = url.match(urlPartsRegex);
            var returner = {};
            var rawDirectories = [];
            var directories = [];
            var i;
            var baseUrlParts;
            if (!urlParts) {
                throw new Error("Could not parse sheet href - '" + url + "'");
            }
            // Stylesheets in IE don't always return the full path
            if (baseUrl && (!urlParts[1] || urlParts[2])) {
                baseUrlParts = baseUrl.match(urlPartsRegex);
                if (!baseUrlParts) {
                    throw new Error("Could not parse page url - '" + baseUrl + "'");
                }
                urlParts[1] = urlParts[1] || baseUrlParts[1] || '';
                if (!urlParts[2]) {
                    urlParts[3] = baseUrlParts[3] + urlParts[3];
                }
            }
            if (urlParts[3]) {
                rawDirectories = urlParts[3].replace(/\\/g, '/').split('/');
                // collapse '..' and skip '.'
                for (i = 0; i < rawDirectories.length; i++) {
                    if (rawDirectories[i] === '..') {
                        directories.pop();
                    }
                    else if (rawDirectories[i] !== '.') {
                        directories.push(rawDirectories[i]);
                    }
                }
            }
            returner.hostPart = urlParts[1];
            returner.directories = directories;
            returner.rawPath = (urlParts[1] || '') + rawDirectories.join('/');
            returner.path = (urlParts[1] || '') + directories.join('/');
            returner.filename = urlParts[4];
            returner.fileUrl = returner.path + (urlParts[4] || '');
            returner.url = returner.fileUrl + (urlParts[5] || '');
            return returner;
        };
        return AbstractFileManager;
    }());
    var FileManager = /** @class */ (function (_super) {
        tslib.__extends(FileManager, _super);
        function FileManager() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        FileManager.prototype.supports = function () {
            return true;
        };
        FileManager.prototype.supportsSync = function () {
            return true;
        };
        FileManager.prototype.loadFile = function (filename, currentDirectory, options, environment, callback) {
            var fullFilename;
            var isAbsoluteFilename = this.isPathAbsolute(filename);
            var filenamesTried = [];
            var self = this;
            var prefix = filename.slice(0, 1);
            var explicit = prefix === '.' || prefix === '/';
            var result = null;
            var isNodeModule = false;
            var npmPrefix = 'npm://';
            options = options || {};
            var paths = isAbsoluteFilename ? [''] : [currentDirectory];
            if (options.paths) {
                paths.push.apply(paths, options.paths);
            }
            if (!isAbsoluteFilename && paths.indexOf('.') === -1) {
                paths.push('.');
            }
            var prefixes = options.prefixes || [''];
            var fileParts = this.extractUrlParts(filename);
            if (options.syncImport) {
                getFileData(returnData, returnData);
                if (callback) {
                    callback(result.error, result);
                }
                else {
                    return result;
                }
            }
            else {
                // promise is guaranteed to be asyncronous
                // which helps as it allows the file handle
                // to be closed before it continues with the next file
                return new Promise(getFileData);
            }
            function returnData(data) {
                if (!data.filename) {
                    result = { error: data };
                }
                else {
                    result = data;
                }
            }
            function getFileData(fulfill, reject) {
                (function tryPathIndex(i) {
                    if (i < paths.length) {
                        (function tryPrefix(j) {
                            if (j < prefixes.length) {
                                isNodeModule = false;
                                fullFilename = fileParts.rawPath + prefixes[j] + fileParts.filename;
                                if (paths[i]) {
                                    fullFilename = path.join(paths[i], fullFilename);
                                }
                                if (!explicit && paths[i] === '.') {
                                    try {
                                        fullFilename = require.resolve(fullFilename);
                                        isNodeModule = true;
                                    }
                                    catch (e) {
                                        filenamesTried.push(npmPrefix + fullFilename);
                                        tryWithExtension();
                                    }
                                }
                                else {
                                    tryWithExtension();
                                }
                                function tryWithExtension() {
                                    var extFilename = options.ext ? self.tryAppendExtension(fullFilename, options.ext) : fullFilename;
                                    if (extFilename !== fullFilename && !explicit && paths[i] === '.') {
                                        try {
                                            fullFilename = require.resolve(extFilename);
                                            isNodeModule = true;
                                        }
                                        catch (e) {
                                            filenamesTried.push(npmPrefix + extFilename);
                                            fullFilename = extFilename;
                                        }
                                    }
                                    else {
                                        fullFilename = extFilename;
                                    }
                                }
                                var readFileArgs = [fullFilename];
                                if (!options.rawBuffer) {
                                    readFileArgs.push('utf-8');
                                }
                                if (options.syncImport) {
                                    try {
                                        var data = fs$1.readFileSync.apply(this, readFileArgs);
                                        fulfill({ contents: data, filename: fullFilename });
                                    }
                                    catch (e) {
                                        filenamesTried.push(isNodeModule ? npmPrefix + fullFilename : fullFilename);
                                        return tryPrefix(j + 1);
                                    }
                                }
                                else {
                                    readFileArgs.push(function (e, data) {
                                        if (e) {
                                            filenamesTried.push(isNodeModule ? npmPrefix + fullFilename : fullFilename);
                                            return tryPrefix(j + 1);
                                        }
                                        fulfill({ contents: data, filename: fullFilename });
                                    });
                                    fs$1.readFile.apply(this, readFileArgs);
                                }
                            }
                            else {
                                tryPathIndex(i + 1);
                            }
                        })(0);
                    }
                    else {
                        reject({ type: 'File', message: "'" + filename + "' wasn't found. Tried - " + filenamesTried.join(',') });
                    }
                }(0));
            }
        };
        FileManager.prototype.loadFileSync = function (filename, currentDirectory, options, environment) {
            options.syncImport = true;
            return this.loadFile(filename, currentDirectory, options, environment);
        };
        return FileManager;
    }(AbstractFileManager));
    var logger = {
        error: function (msg) {
            this._fireEvent('error', msg);
        },
        warn: function (msg) {
            this._fireEvent('warn', msg);
        },
        info: function (msg) {
            this._fireEvent('info', msg);
        },
        debug: function (msg) {
            this._fireEvent('debug', msg);
        },
        addListener: function (listener) {
            this._listeners.push(listener);
        },
        removeListener: function (listener) {
            for (var i_1 = 0; i_1 < this._listeners.length; i_1++) {
                if (this._listeners[i_1] === listener) {
                    this._listeners.splice(i_1, 1);
                    return;
                }
            }
        },
        _fireEvent: function (type, msg) {
            for (var i_2 = 0; i_2 < this._listeners.length; i_2++) {
                var logFunction = this._listeners[i_2][type];
                if (logFunction) {
                    logFunction(msg);
                }
            }
        },
        _listeners: []
    };
    var isUrlRe = /^(?:https?:)?\/\//i;
    var request;
    var UrlFileManager = /** @class */ (function (_super) {
        tslib.__extends(UrlFileManager, _super);
        function UrlFileManager() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        UrlFileManager.prototype.supports = function (filename, currentDirectory, options, environment) {
            return isUrlRe.test(filename) || isUrlRe.test(currentDirectory);
        };
        UrlFileManager.prototype.loadFile = function (filename, currentDirectory, options, environment) {
            return new Promise(function (fulfill, reject) {
                if (request === undefined) {
                    try {
                        request = require(833) /* request */;
                    }
                    catch (e) {
                        request = null;
                    }
                }
                if (!request) {
                    reject({ type: 'File', message: 'optional dependency \'request\' required to import over http(s)\n' });
                    return;
                }
                var urlStr = isUrlRe.test(filename) ? filename : url.resolve(currentDirectory, filename);
                var urlObj = url.parse(urlStr);
                if (!urlObj.protocol) {
                    urlObj.protocol = 'http';
                    urlStr = urlObj.format();
                }
                request.get({ uri: urlStr, strictSSL: !options.insecure }, function (error, res, body) {
                    if (error) {
                        reject({ type: 'File', message: "resource '" + urlStr + "' gave this Error:\n  " + error + "\n" });
                        return;
                    }
                    if (res && res.statusCode === 404) {
                        reject({ type: 'File', message: "resource '" + urlStr + "' was not found\n" });
                        return;
                    }
                    if (!body) {
                        logger.warn("Warning: Empty body (HTTP " + res.statusCode + ") returned by \"" + urlStr + "\"");
                    }
                    fulfill({ contents: body, filename: urlStr });
                });
            });
        };
        return UrlFileManager;
    }(AbstractFileManager));
    var colors = {
        'aliceblue': '#f0f8ff',
        'antiquewhite': '#faebd7',
        'aqua': '#00ffff',
        'aquamarine': '#7fffd4',
        'azure': '#f0ffff',
        'beige': '#f5f5dc',
        'bisque': '#ffe4c4',
        'black': '#000000',
        'blanchedalmond': '#ffebcd',
        'blue': '#0000ff',
        'blueviolet': '#8a2be2',
        'brown': '#a52a2a',
        'burlywood': '#deb887',
        'cadetblue': '#5f9ea0',
        'chartreuse': '#7fff00',
        'chocolate': '#d2691e',
        'coral': '#ff7f50',
        'cornflowerblue': '#6495ed',
        'cornsilk': '#fff8dc',
        'crimson': '#dc143c',
        'cyan': '#00ffff',
        'darkblue': '#00008b',
        'darkcyan': '#008b8b',
        'darkgoldenrod': '#b8860b',
        'darkgray': '#a9a9a9',
        'darkgrey': '#a9a9a9',
        'darkgreen': '#006400',
        'darkkhaki': '#bdb76b',
        'darkmagenta': '#8b008b',
        'darkolivegreen': '#556b2f',
        'darkorange': '#ff8c00',
        'darkorchid': '#9932cc',
        'darkred': '#8b0000',
        'darksalmon': '#e9967a',
        'darkseagreen': '#8fbc8f',
        'darkslateblue': '#483d8b',
        'darkslategray': '#2f4f4f',
        'darkslategrey': '#2f4f4f',
        'darkturquoise': '#00ced1',
        'darkviolet': '#9400d3',
        'deeppink': '#ff1493',
        'deepskyblue': '#00bfff',
        'dimgray': '#696969',
        'dimgrey': '#696969',
        'dodgerblue': '#1e90ff',
        'firebrick': '#b22222',
        'floralwhite': '#fffaf0',
        'forestgreen': '#228b22',
        'fuchsia': '#ff00ff',
        'gainsboro': '#dcdcdc',
        'ghostwhite': '#f8f8ff',
        'gold': '#ffd700',
        'goldenrod': '#daa520',
        'gray': '#808080',
        'grey': '#808080',
        'green': '#008000',
        'greenyellow': '#adff2f',
        'honeydew': '#f0fff0',
        'hotpink': '#ff69b4',
        'indianred': '#cd5c5c',
        'indigo': '#4b0082',
        'ivory': '#fffff0',
        'khaki': '#f0e68c',
        'lavender': '#e6e6fa',
        'lavenderblush': '#fff0f5',
        'lawngreen': '#7cfc00',
        'lemonchiffon': '#fffacd',
        'lightblue': '#add8e6',
        'lightcoral': '#f08080',
        'lightcyan': '#e0ffff',
        'lightgoldenrodyellow': '#fafad2',
        'lightgray': '#d3d3d3',
        'lightgrey': '#d3d3d3',
        'lightgreen': '#90ee90',
        'lightpink': '#ffb6c1',
        'lightsalmon': '#ffa07a',
        'lightseagreen': '#20b2aa',
        'lightskyblue': '#87cefa',
        'lightslategray': '#778899',
        'lightslategrey': '#778899',
        'lightsteelblue': '#b0c4de',
        'lightyellow': '#ffffe0',
        'lime': '#00ff00',
        'limegreen': '#32cd32',
        'linen': '#faf0e6',
        'magenta': '#ff00ff',
        'maroon': '#800000',
        'mediumaquamarine': '#66cdaa',
        'mediumblue': '#0000cd',
        'mediumorchid': '#ba55d3',
        'mediumpurple': '#9370d8',
        'mediumseagreen': '#3cb371',
        'mediumslateblue': '#7b68ee',
        'mediumspringgreen': '#00fa9a',
        'mediumturquoise': '#48d1cc',
        'mediumvioletred': '#c71585',
        'midnightblue': '#191970',
        'mintcream': '#f5fffa',
        'mistyrose': '#ffe4e1',
        'moccasin': '#ffe4b5',
        'navajowhite': '#ffdead',
        'navy': '#000080',
        'oldlace': '#fdf5e6',
        'olive': '#808000',
        'olivedrab': '#6b8e23',
        'orange': '#ffa500',
        'orangered': '#ff4500',
        'orchid': '#da70d6',
        'palegoldenrod': '#eee8aa',
        'palegreen': '#98fb98',
        'paleturquoise': '#afeeee',
        'palevioletred': '#d87093',
        'papayawhip': '#ffefd5',
        'peachpuff': '#ffdab9',
        'peru': '#cd853f',
        'pink': '#ffc0cb',
        'plum': '#dda0dd',
        'powderblue': '#b0e0e6',
        'purple': '#800080',
        'rebeccapurple': '#663399',
        'red': '#ff0000',
        'rosybrown': '#bc8f8f',
        'royalblue': '#4169e1',
        'saddlebrown': '#8b4513',
        'salmon': '#fa8072',
        'sandybrown': '#f4a460',
        'seagreen': '#2e8b57',
        'seashell': '#fff5ee',
        'sienna': '#a0522d',
        'silver': '#c0c0c0',
        'skyblue': '#87ceeb',
        'slateblue': '#6a5acd',
        'slategray': '#708090',
        'slategrey': '#708090',
        'snow': '#fffafa',
        'springgreen': '#00ff7f',
        'steelblue': '#4682b4',
        'tan': '#d2b48c',
        'teal': '#008080',
        'thistle': '#d8bfd8',
        'tomato': '#ff6347',
        'turquoise': '#40e0d0',
        'violet': '#ee82ee',
        'wheat': '#f5deb3',
        'white': '#ffffff',
        'whitesmoke': '#f5f5f5',
        'yellow': '#ffff00',
        'yellowgreen': '#9acd32'
    };
    var unitConversions = {
        length: {
            'm': 1,
            'cm': 0.01,
            'mm': 0.001,
            'in': 0.0254,
            'px': 0.0254 / 96,
            'pt': 0.0254 / 72,
            'pc': 0.0254 / 72 * 12
        },
        duration: {
            's': 1,
            'ms': 0.001
        },
        angle: {
            'rad': 1 / (2 * Math.PI),
            'deg': 1 / 360,
            'grad': 1 / 400,
            'turn': 1
        }
    };
    var data = { colors: colors, unitConversions: unitConversions };
    var Node = /** @class */ (function () {
        function Node() {
            this.parent = null;
            this.visibilityBlocks = undefined;
            this.nodeVisible = undefined;
            this.rootNode = null;
            this.parsed = null;
            var self = this;
            Object.defineProperty(this, 'currentFileInfo', {
                get: function () { return self.fileInfo(); }
            });
            Object.defineProperty(this, 'index', {
                get: function () { return self.getIndex(); }
            });
        }
        Node.prototype.setParent = function (nodes, parent) {
            function set(node) {
                if (node && node instanceof Node) {
                    node.parent = parent;
                }
            }
            if (Array.isArray(nodes)) {
                nodes.forEach(set);
            }
            else {
                set(nodes);
            }
        };
        Node.prototype.getIndex = function () {
            return this._index || (this.parent && this.parent.getIndex()) || 0;
        };
        Node.prototype.fileInfo = function () {
            return this._fileInfo || (this.parent && this.parent.fileInfo()) || {};
        };
        Node.prototype.isRulesetLike = function () {
            return false;
        };
        Node.prototype.toCSS = function (context) {
            var strs = [];
            this.genCSS(context, {
                add: function (chunk, fileInfo, index) {
                    strs.push(chunk);
                },
                isEmpty: function () {
                    return strs.length === 0;
                }
            });
            return strs.join('');
        };
        Node.prototype.genCSS = function (context, output) {
            output.add(this.value);
        };
        Node.prototype.accept = function (visitor) {
            this.value = visitor.visit(this.value);
        };
        Node.prototype.eval = function () { return this; };
        Node.prototype._operate = function (context, op, a, b) {
            switch (op) {
                case '+': return a + b;
                case '-': return a - b;
                case '*': return a * b;
                case '/': return a / b;
            }
        };
        Node.prototype.fround = function (context, value) {
            var precision = context && context.numPrecision;
            // add "epsilon" to ensure numbers like 1.000000005 (represented as 1.000000004999...) are properly rounded:
            return (precision) ? Number((value + 2e-16).toFixed(precision)) : value;
        };
        // Returns true if this node represents root of ast imported by reference
        Node.prototype.blocksVisibility = function () {
            if (this.visibilityBlocks == null) {
                this.visibilityBlocks = 0;
            }
            return this.visibilityBlocks !== 0;
        };
        Node.prototype.addVisibilityBlock = function () {
            if (this.visibilityBlocks == null) {
                this.visibilityBlocks = 0;
            }
            this.visibilityBlocks = this.visibilityBlocks + 1;
        };
        Node.prototype.removeVisibilityBlock = function () {
            if (this.visibilityBlocks == null) {
                this.visibilityBlocks = 0;
            }
            this.visibilityBlocks = this.visibilityBlocks - 1;
        };
        // Turns on node visibility - if called node will be shown in output regardless
        // of whether it comes from import by reference or not
        Node.prototype.ensureVisibility = function () {
            this.nodeVisible = true;
        };
        // Turns off node visibility - if called node will NOT be shown in output regardless
        // of whether it comes from import by reference or not
        Node.prototype.ensureInvisibility = function () {
            this.nodeVisible = false;
        };
        // return values:
        // false - the node must not be visible
        // true - the node must be visible
        // undefined or null - the node has the same visibility as its parent
        Node.prototype.isVisible = function () {
            return this.nodeVisible;
        };
        Node.prototype.visibilityInfo = function () {
            return {
                visibilityBlocks: this.visibilityBlocks,
                nodeVisible: this.nodeVisible
            };
        };
        Node.prototype.copyVisibilityInfo = function (info) {
            if (!info) {
                return;
            }
            this.visibilityBlocks = info.visibilityBlocks;
            this.nodeVisible = info.nodeVisible;
        };
        return Node;
    }());
    Node.compare = function (a, b) {
        /* returns:
         -1: a < b
         0: a = b
         1: a > b
         and *any* other value for a != b (e.g. undefined, NaN, -2 etc.) */
        if ((a.compare) &&
            // for "symmetric results" force toCSS-based comparison
            // of Quoted or Anonymous if either value is one of those
            !(b.type === 'Quoted' || b.type === 'Anonymous')) {
            return a.compare(b);
        }
        else if (b.compare) {
            return -b.compare(a);
        }
        else if (a.type !== b.type) {
            return undefined;
        }
        a = a.value;
        b = b.value;
        if (!Array.isArray(a)) {
            return a === b ? 0 : undefined;
        }
        if (a.length !== b.length) {
            return undefined;
        }
        for (var i_1 = 0; i_1 < a.length; i_1++) {
            if (Node.compare(a[i_1], b[i_1]) !== 0) {
                return undefined;
            }
        }
        return 0;
    };
    Node.numericCompare = function (a, b) {
        return a < b ? -1
            : a === b ? 0
                : a > b ? 1 : undefined;
    };
    //
    // RGB Colors - #ff0014, #eee
    //
    var Color = /** @class */ (function (_super) {
        tslib.__extends(Color, _super);
        function Color(rgb, a, originalForm) {
            var _this = _super.call(this) || this;
            var self = _this;
            //
            // The end goal here, is to parse the arguments
            // into an integer triplet, such as `128, 255, 0`
            //
            // This facilitates operations and conversions.
            //
            if (Array.isArray(rgb)) {
                _this.rgb = rgb;
            }
            else if (rgb.length >= 6) {
                _this.rgb = [];
                rgb.match(/.{2}/g).map(function (c, i) {
                    if (i < 3) {
                        self.rgb.push(parseInt(c, 16));
                    }
                    else {
                        self.alpha = (parseInt(c, 16)) / 255;
                    }
                });
            }
            else {
                _this.rgb = [];
                rgb.split('').map(function (c, i) {
                    if (i < 3) {
                        self.rgb.push(parseInt(c + c, 16));
                    }
                    else {
                        self.alpha = (parseInt(c + c, 16)) / 255;
                    }
                });
            }
            _this.alpha = _this.alpha || (typeof a === 'number' ? a : 1);
            if (typeof originalForm !== 'undefined') {
                _this.value = originalForm;
            }
            return _this;
        }
        Color.prototype.luma = function () {
            var r = this.rgb[0] / 255;
            var g = this.rgb[1] / 255;
            var b = this.rgb[2] / 255;
            r = (r <= 0.03928) ? r / 12.92 : Math.pow(((r + 0.055) / 1.055), 2.4);
            g = (g <= 0.03928) ? g / 12.92 : Math.pow(((g + 0.055) / 1.055), 2.4);
            b = (b <= 0.03928) ? b / 12.92 : Math.pow(((b + 0.055) / 1.055), 2.4);
            return 0.2126 * r + 0.7152 * g + 0.0722 * b;
        };
        Color.prototype.genCSS = function (context, output) {
            output.add(this.toCSS(context));
        };
        Color.prototype.toCSS = function (context, doNotCompress) {
            var compress = context && context.compress && !doNotCompress;
            var color;
            var alpha;
            var colorFunction;
            var args = [];
            // `value` is set if this color was originally
            // converted from a named color string so we need
            // to respect this and try to output named color too.
            alpha = this.fround(context, this.alpha);
            if (this.value) {
                if (this.value.indexOf('rgb') === 0) {
                    if (alpha < 1) {
                        colorFunction = 'rgba';
                    }
                }
                else if (this.value.indexOf('hsl') === 0) {
                    if (alpha < 1) {
                        colorFunction = 'hsla';
                    }
                    else {
                        colorFunction = 'hsl';
                    }
                }
                else {
                    return this.value;
                }
            }
            else {
                if (alpha < 1) {
                    colorFunction = 'rgba';
                }
            }
            switch (colorFunction) {
                case 'rgba':
                    args = this.rgb.map(function (c) { return clamp(Math.round(c), 255); }).concat(clamp(alpha, 1));
                    break;
                case 'hsla':
                    args.push(clamp(alpha, 1));
                case 'hsl':
                    color = this.toHSL();
                    args = [
                        this.fround(context, color.h),
                        this.fround(context, color.s * 100) + "%",
                        this.fround(context, color.l * 100) + "%"
                    ].concat(args);
            }
            if (colorFunction) {
                // Values are capped between `0` and `255`, rounded and zero-padded.
                return colorFunction + "(" + args.join("," + (compress ? '' : ' ')) + ")";
            }
            color = this.toRGB();
            if (compress) {
                var splitcolor = color.split('');
                // Convert color to short format
                if (splitcolor[1] === splitcolor[2] && splitcolor[3] === splitcolor[4] && splitcolor[5] === splitcolor[6]) {
                    color = "#" + splitcolor[1] + splitcolor[3] + splitcolor[5];
                }
            }
            return color;
        };
        //
        // Operations have to be done per-channel, if not,
        // channels will spill onto each other. Once we have
        // our result, in the form of an integer triplet,
        // we create a new Color node to hold the result.
        //
        Color.prototype.operate = function (context, op, other) {
            var rgb = new Array(3);
            var alpha = this.alpha * (1 - other.alpha) + other.alpha;
            for (var c = 0; c < 3; c++) {
                rgb[c] = this._operate(context, op, this.rgb[c], other.rgb[c]);
            }
            return new Color(rgb, alpha);
        };
        Color.prototype.toRGB = function () {
            return toHex(this.rgb);
        };
        Color.prototype.toHSL = function () {
            var r = this.rgb[0] / 255;
            var g = this.rgb[1] / 255;
            var b = this.rgb[2] / 255;
            var a = this.alpha;
            var max = Math.max(r, g, b);
            var min = Math.min(r, g, b);
            var h;
            var s;
            var l = (max + min) / 2;
            var d = max - min;
            if (max === min) {
                h = s = 0;
            }
            else {
                s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
                switch (max) {
                    case r:
                        h = (g - b) / d + (g < b ? 6 : 0);
                        break;
                    case g:
                        h = (b - r) / d + 2;
                        break;
                    case b:
                        h = (r - g) / d + 4;
                        break;
                }
                h /= 6;
            }
            return { h: h * 360, s: s, l: l, a: a };
        };
        // Adapted from http://mjijackson.com/2008/02/rgb-to-hsl-and-rgb-to-hsv-color-model-conversion-algorithms-in-javascript
        Color.prototype.toHSV = function () {
            var r = this.rgb[0] / 255;
            var g = this.rgb[1] / 255;
            var b = this.rgb[2] / 255;
            var a = this.alpha;
            var max = Math.max(r, g, b);
            var min = Math.min(r, g, b);
            var h;
            var s;
            var v = max;
            var d = max - min;
            if (max === 0) {
                s = 0;
            }
            else {
                s = d / max;
            }
            if (max === min) {
                h = 0;
            }
            else {
                switch (max) {
                    case r:
                        h = (g - b) / d + (g < b ? 6 : 0);
                        break;
                    case g:
                        h = (b - r) / d + 2;
                        break;
                    case b:
                        h = (r - g) / d + 4;
                        break;
                }
                h /= 6;
            }
            return { h: h * 360, s: s, v: v, a: a };
        };
        Color.prototype.toARGB = function () {
            return toHex([this.alpha * 255].concat(this.rgb));
        };
        Color.prototype.compare = function (x) {
            return (x.rgb &&
                x.rgb[0] === this.rgb[0] &&
                x.rgb[1] === this.rgb[1] &&
                x.rgb[2] === this.rgb[2] &&
                x.alpha === this.alpha) ? 0 : undefined;
        };
        return Color;
    }(Node));
    Color.prototype.type = 'Color';
    function clamp(v, max) {
        return Math.min(Math.max(v, 0), max);
    }
    function toHex(v) {
        return "#" + v.map(function (c) {
            c = clamp(Math.round(c), 255);
            return (c < 16 ? '0' : '') + c.toString(16);
        }).join('');
    }
    Color.fromKeyword = function (keyword) {
        var c;
        var key = keyword.toLowerCase();
        if (colors.hasOwnProperty(key)) {
            c = new Color(colors[key].slice(1));
        }
        else if (key === 'transparent') {
            c = new Color([0, 0, 0], 0);
        }
        if (c) {
            c.value = keyword;
            return c;
        }
    };
    var Paren = /** @class */ (function (_super) {
        tslib.__extends(Paren, _super);
        function Paren(node) {
            var _this = _super.call(this) || this;
            _this.value = node;
            return _this;
        }
        Paren.prototype.genCSS = function (context, output) {
            output.add('(');
            this.value.genCSS(context, output);
            output.add(')');
        };
        Paren.prototype.eval = function (context) {
            return new Paren(this.value.eval(context));
        };
        return Paren;
    }(Node));
    Paren.prototype.type = 'Paren';
    var _noSpaceCombinators = {
        '': true,
        ' ': true,
        '|': true
    };
    var Combinator = /** @class */ (function (_super) {
        tslib.__extends(Combinator, _super);
        function Combinator(value) {
            var _this = _super.call(this) || this;
            if (value === ' ') {
                _this.value = ' ';
                _this.emptyOrWhitespace = true;
            }
            else {
                _this.value = value ? value.trim() : '';
                _this.emptyOrWhitespace = _this.value === '';
            }
            return _this;
        }
        Combinator.prototype.genCSS = function (context, output) {
            var spaceOrEmpty = (context.compress || _noSpaceCombinators[this.value]) ? '' : ' ';
            output.add(spaceOrEmpty + this.value + spaceOrEmpty);
        };
        return Combinator;
    }(Node));
    Combinator.prototype.type = 'Combinator';
    var Element = /** @class */ (function (_super) {
        tslib.__extends(Element, _super);
        function Element(combinator, value, isVariable, index, currentFileInfo, visibilityInfo) {
            var _this = _super.call(this) || this;
            _this.combinator = combinator instanceof Combinator ?
                combinator : new Combinator(combinator);
            if (typeof value === 'string') {
                _this.value = value.trim();
            }
            else if (value) {
                _this.value = value;
            }
            else {
                _this.value = '';
            }
            _this.isVariable = isVariable;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.copyVisibilityInfo(visibilityInfo);
            _this.setParent(_this.combinator, _this);
            return _this;
        }
        Element.prototype.accept = function (visitor) {
            var value = this.value;
            this.combinator = visitor.visit(this.combinator);
            if (typeof value === 'object') {
                this.value = visitor.visit(value);
            }
        };
        Element.prototype.eval = function (context) {
            return new Element(this.combinator, this.value.eval ? this.value.eval(context) : this.value, this.isVariable, this.getIndex(), this.fileInfo(), this.visibilityInfo());
        };
        Element.prototype.clone = function () {
            return new Element(this.combinator, this.value, this.isVariable, this.getIndex(), this.fileInfo(), this.visibilityInfo());
        };
        Element.prototype.genCSS = function (context, output) {
            output.add(this.toCSS(context), this.fileInfo(), this.getIndex());
        };
        Element.prototype.toCSS = function (context) {
            if (context === void 0) {
                context = {};
            }
            var value = this.value;
            var firstSelector = context.firstSelector;
            if (value instanceof Paren) {
                // selector in parens should not be affected by outer selector
                // flags (breaks only interpolated selectors - see #1973)
                context.firstSelector = true;
            }
            value = value.toCSS ? value.toCSS(context) : value;
            context.firstSelector = firstSelector;
            if (value === '' && this.combinator.value.charAt(0) === '&') {
                return '';
            }
            else {
                return this.combinator.toCSS(context) + value;
            }
        };
        return Element;
    }(Node));
    Element.prototype.type = 'Element';
    var Math$1 = {
        ALWAYS: 0,
        PARENS_DIVISION: 1,
        PARENS: 2,
        STRICT_LEGACY: 3
    };
    var RewriteUrls = {
        OFF: 0,
        LOCAL: 1,
        ALL: 2
    };
    /* jshint proto: true */
    function getLocation(index, inputStream) {
        var n = index + 1;
        var line = null;
        var column = -1;
        while (--n >= 0 && inputStream.charAt(n) !== '\n') {
            column++;
        }
        if (typeof index === 'number') {
            line = (inputStream.slice(0, index).match(/\n/g) || '').length;
        }
        return {
            line: line,
            column: column
        };
    }
    function copyArray(arr) {
        var i;
        var length = arr.length;
        var copy = new Array(length);
        for (i = 0; i < length; i++) {
            copy[i] = arr[i];
        }
        return copy;
    }
    function clone(obj) {
        var cloned = {};
        for (var prop in obj) {
            if (obj.hasOwnProperty(prop)) {
                cloned[prop] = obj[prop];
            }
        }
        return cloned;
    }
    function defaults(obj1, obj2) {
        var newObj = obj2 || {};
        if (!obj2._defaults) {
            newObj = {};
            var defaults_1 = CloneHelper(obj1);
            newObj._defaults = defaults_1;
            var cloned = obj2 ? CloneHelper(obj2) : {};
            Object.assign(newObj, defaults_1, cloned);
        }
        return newObj;
    }
    function copyOptions(obj1, obj2) {
        if (obj2 && obj2._defaults) {
            return obj2;
        }
        var opts = defaults(obj1, obj2);
        if (opts.strictMath) {
            opts.math = Math$1.STRICT_LEGACY;
        }
        // Back compat with changed relativeUrls option
        if (opts.relativeUrls) {
            opts.rewriteUrls = RewriteUrls.ALL;
        }
        if (typeof opts.math === 'string') {
            switch (opts.math.toLowerCase()) {
                case 'always':
                    opts.math = Math$1.ALWAYS;
                    break;
                case 'parens-division':
                    opts.math = Math$1.PARENS_DIVISION;
                    break;
                case 'strict':
                case 'parens':
                    opts.math = Math$1.PARENS;
                    break;
                case 'strict-legacy':
                    opts.math = Math$1.STRICT_LEGACY;
            }
        }
        if (typeof opts.rewriteUrls === 'string') {
            switch (opts.rewriteUrls.toLowerCase()) {
                case 'off':
                    opts.rewriteUrls = RewriteUrls.OFF;
                    break;
                case 'local':
                    opts.rewriteUrls = RewriteUrls.LOCAL;
                    break;
                case 'all':
                    opts.rewriteUrls = RewriteUrls.ALL;
                    break;
            }
        }
        return opts;
    }
    function merge(obj1, obj2) {
        for (var prop in obj2) {
            if (obj2.hasOwnProperty(prop)) {
                obj1[prop] = obj2[prop];
            }
        }
        return obj1;
    }
    function flattenArray(arr, result) {
        if (result === void 0) {
            result = [];
        }
        for (var i_1 = 0, length_1 = arr.length; i_1 < length_1; i_1++) {
            var value = arr[i_1];
            if (Array.isArray(value)) {
                flattenArray(value, result);
            }
            else {
                if (value !== undefined) {
                    result.push(value);
                }
            }
        }
        return result;
    }
    var utils = /*#__PURE__*/ Object.freeze({
        __proto__: null,
        getLocation: getLocation,
        copyArray: copyArray,
        clone: clone,
        defaults: defaults,
        copyOptions: copyOptions,
        merge: merge,
        flattenArray: flattenArray
    });
    var anonymousFunc = /(<anonymous>|Function):(\d+):(\d+)/;
    /**
     * This is a centralized class of any error that could be thrown internally (mostly by the parser).
     * Besides standard .message it keeps some additional data like a path to the file where the error
     * occurred along with line and column numbers.
     *
     * @class
     * @extends Error
     * @type {module.LessError}
     *
     * @prop {string} type
     * @prop {string} filename
     * @prop {number} index
     * @prop {number} line
     * @prop {number} column
     * @prop {number} callLine
     * @prop {number} callExtract
     * @prop {string[]} extract
     *
     * @param {Object} e              - An error object to wrap around or just a descriptive object
     * @param {Object} fileContentMap - An object with file contents in 'contents' property (like importManager) @todo - move to fileManager?
     * @param {string} [currentFilename]
     */
    var LessError = function LessError(e, fileContentMap, currentFilename) {
        Error.call(this);
        var filename = e.filename || currentFilename;
        this.message = e.message;
        this.stack = e.stack;
        if (fileContentMap && filename) {
            var input = fileContentMap.contents[filename];
            var loc = getLocation(e.index, input);
            var line = loc.line;
            var col = loc.column;
            var callLine = e.call && getLocation(e.call, input).line;
            var lines = input ? input.split('\n') : '';
            this.type = e.type || 'Syntax';
            this.filename = filename;
            this.index = e.index;
            this.line = typeof line === 'number' ? line + 1 : null;
            this.column = col;
            if (!this.line && this.stack) {
                var found = this.stack.match(anonymousFunc);
                /**
                 * We have to figure out how this environment stringifies anonymous functions
                 * so we can correctly map plugin errors.
                 *
                 * Note, in Node 8, the output of anonymous funcs varied based on parameters
                 * being present or not, so we inject dummy params.
                 */
                var func = new Function('a', 'throw new Error()');
                var lineAdjust = 0;
                try {
                    func();
                }
                catch (e) {
                    var match = e.stack.match(anonymousFunc);
                    var line_1 = parseInt(match[2]);
                    lineAdjust = 1 - line_1;
                }
                if (found) {
                    if (found[2]) {
                        this.line = parseInt(found[2]) + lineAdjust;
                    }
                    if (found[3]) {
                        this.column = parseInt(found[3]);
                    }
                }
            }
            this.callLine = callLine + 1;
            this.callExtract = lines[callLine];
            this.extract = [
                lines[this.line - 2],
                lines[this.line - 1],
                lines[this.line]
            ];
        }
    };
    if (typeof Object.create === 'undefined') {
        var F = function () { };
        F.prototype = Error.prototype;
        LessError.prototype = new F();
    }
    else {
        LessError.prototype = Object.create(Error.prototype);
    }
    LessError.prototype.constructor = LessError;
    /**
     * An overridden version of the default Object.prototype.toString
     * which uses additional information to create a helpful message.
     *
     * @param {Object} options
     * @returns {string}
     */
    LessError.prototype.toString = function (options) {
        if (options === void 0) {
            options = {};
        }
        var message = '';
        var extract = this.extract || [];
        var error = [];
        var stylize = function (str) { return str; };
        if (options.stylize) {
            var type = typeof options.stylize;
            if (type !== 'function') {
                throw Error("options.stylize should be a function, got a " + type + "!");
            }
            stylize = options.stylize;
        }
        if (this.line !== null) {
            if (typeof extract[0] === 'string') {
                error.push(stylize(this.line - 1 + " " + extract[0], 'grey'));
            }
            if (typeof extract[1] === 'string') {
                var errorTxt = this.line + " ";
                if (extract[1]) {
                    errorTxt += extract[1].slice(0, this.column) +
                        stylize(stylize(stylize(extract[1].substr(this.column, 1), 'bold') +
                            extract[1].slice(this.column + 1), 'red'), 'inverse');
                }
                error.push(errorTxt);
            }
            if (typeof extract[2] === 'string') {
                error.push(stylize(this.line + 1 + " " + extract[2], 'grey'));
            }
            error = error.join('\n') + stylize('', 'reset') + "\n";
        }
        message += stylize(this.type + "Error: " + this.message, 'red');
        if (this.filename) {
            message += stylize(' in ', 'red') + this.filename;
        }
        if (this.line) {
            message += stylize(" on line " + this.line + ", column " + (this.column + 1) + ":", 'grey');
        }
        message += "\n" + error;
        if (this.callLine) {
            message += stylize('from ', 'red') + (this.filename || '') + "/n";
            message += stylize(this.callLine, 'grey') + " " + this.callExtract + "/n";
        }
        return message;
    };
    var Selector = /** @class */ (function (_super) {
        tslib.__extends(Selector, _super);
        function Selector(elements, extendList, condition, index, currentFileInfo, visibilityInfo) {
            var _this = _super.call(this) || this;
            _this.extendList = extendList;
            _this.condition = condition;
            _this.evaldCondition = !condition;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.elements = _this.getElements(elements);
            _this.mixinElements_ = undefined;
            _this.copyVisibilityInfo(visibilityInfo);
            _this.setParent(_this.elements, _this);
            return _this;
        }
        Selector.prototype.accept = function (visitor) {
            if (this.elements) {
                this.elements = visitor.visitArray(this.elements);
            }
            if (this.extendList) {
                this.extendList = visitor.visitArray(this.extendList);
            }
            if (this.condition) {
                this.condition = visitor.visit(this.condition);
            }
        };
        Selector.prototype.createDerived = function (elements, extendList, evaldCondition) {
            elements = this.getElements(elements);
            var newSelector = new Selector(elements, extendList || this.extendList, null, this.getIndex(), this.fileInfo(), this.visibilityInfo());
            newSelector.evaldCondition = (evaldCondition != null) ? evaldCondition : this.evaldCondition;
            newSelector.mediaEmpty = this.mediaEmpty;
            return newSelector;
        };
        Selector.prototype.getElements = function (els) {
            if (!els) {
                return [new Element('', '&', false, this._index, this._fileInfo)];
            }
            if (typeof els === 'string') {
                this.parse.parseNode(els, ['selector'], this._index, this._fileInfo, function (err, result) {
                    if (err) {
                        throw new LessError({
                            index: err.index,
                            message: err.message
                        }, this.parse.imports, this._fileInfo.filename);
                    }
                    els = result[0].elements;
                });
            }
            return els;
        };
        Selector.prototype.createEmptySelectors = function () {
            var el = new Element('', '&', false, this._index, this._fileInfo);
            var sels = [new Selector([el], null, null, this._index, this._fileInfo)];
            sels[0].mediaEmpty = true;
            return sels;
        };
        Selector.prototype.match = function (other) {
            var elements = this.elements;
            var len = elements.length;
            var olen;
            var i;
            other = other.mixinElements();
            olen = other.length;
            if (olen === 0 || len < olen) {
                return 0;
            }
            else {
                for (i = 0; i < olen; i++) {
                    if (elements[i].value !== other[i]) {
                        return 0;
                    }
                }
            }
            return olen; // return number of matched elements
        };
        Selector.prototype.mixinElements = function () {
            if (this.mixinElements_) {
                return this.mixinElements_;
            }
            var elements = this.elements.map(function (v) { return v.combinator.value + (v.value.value || v.value); }).join('').match(/[,&#\*\.\w-]([\w-]|(\\.))*/g);
            if (elements) {
                if (elements[0] === '&') {
                    elements.shift();
                }
            }
            else {
                elements = [];
            }
            return (this.mixinElements_ = elements);
        };
        Selector.prototype.isJustParentSelector = function () {
            return !this.mediaEmpty &&
                this.elements.length === 1 &&
                this.elements[0].value === '&' &&
                (this.elements[0].combinator.value === ' ' || this.elements[0].combinator.value === '');
        };
        Selector.prototype.eval = function (context) {
            var evaldCondition = this.condition && this.condition.eval(context);
            var elements = this.elements;
            var extendList = this.extendList;
            elements = elements && elements.map(function (e) { return e.eval(context); });
            extendList = extendList && extendList.map(function (extend) { return extend.eval(context); });
            return this.createDerived(elements, extendList, evaldCondition);
        };
        Selector.prototype.genCSS = function (context, output) {
            var i;
            var element;
            if ((!context || !context.firstSelector) && this.elements[0].combinator.value === '') {
                output.add(' ', this.fileInfo(), this.getIndex());
            }
            for (i = 0; i < this.elements.length; i++) {
                element = this.elements[i];
                element.genCSS(context, output);
            }
        };
        Selector.prototype.getIsOutput = function () {
            return this.evaldCondition;
        };
        return Selector;
    }(Node));
    Selector.prototype.type = 'Selector';
    var Value = /** @class */ (function (_super) {
        tslib.__extends(Value, _super);
        function Value(value) {
            var _this = _super.call(this) || this;
            if (!value) {
                throw new Error('Value requires an array argument');
            }
            if (!Array.isArray(value)) {
                _this.value = [value];
            }
            else {
                _this.value = value;
            }
            return _this;
        }
        Value.prototype.accept = function (visitor) {
            if (this.value) {
                this.value = visitor.visitArray(this.value);
            }
        };
        Value.prototype.eval = function (context) {
            if (this.value.length === 1) {
                return this.value[0].eval(context);
            }
            else {
                return new Value(this.value.map(function (v) { return v.eval(context); }));
            }
        };
        Value.prototype.genCSS = function (context, output) {
            var i;
            for (i = 0; i < this.value.length; i++) {
                this.value[i].genCSS(context, output);
                if (i + 1 < this.value.length) {
                    output.add((context && context.compress) ? ',' : ', ');
                }
            }
        };
        return Value;
    }(Node));
    Value.prototype.type = 'Value';
    var Keyword = /** @class */ (function (_super) {
        tslib.__extends(Keyword, _super);
        function Keyword(value) {
            var _this = _super.call(this) || this;
            _this.value = value;
            return _this;
        }
        Keyword.prototype.genCSS = function (context, output) {
            if (this.value === '%') {
                throw { type: 'Syntax', message: 'Invalid % without number' };
            }
            output.add(this.value);
        };
        return Keyword;
    }(Node));
    Keyword.prototype.type = 'Keyword';
    Keyword.True = new Keyword('true');
    Keyword.False = new Keyword('false');
    var Anonymous = /** @class */ (function (_super) {
        tslib.__extends(Anonymous, _super);
        function Anonymous(value, index, currentFileInfo, mapLines, rulesetLike, visibilityInfo) {
            var _this = _super.call(this) || this;
            _this.value = value;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.mapLines = mapLines;
            _this.rulesetLike = (typeof rulesetLike === 'undefined') ? false : rulesetLike;
            _this.allowRoot = true;
            _this.copyVisibilityInfo(visibilityInfo);
            return _this;
        }
        Anonymous.prototype.eval = function () {
            return new Anonymous(this.value, this._index, this._fileInfo, this.mapLines, this.rulesetLike, this.visibilityInfo());
        };
        Anonymous.prototype.compare = function (other) {
            return other.toCSS && this.toCSS() === other.toCSS() ? 0 : undefined;
        };
        Anonymous.prototype.isRulesetLike = function () {
            return this.rulesetLike;
        };
        Anonymous.prototype.genCSS = function (context, output) {
            this.nodeVisible = Boolean(this.value);
            if (this.nodeVisible) {
                output.add(this.value, this._fileInfo, this._index, this.mapLines);
            }
        };
        return Anonymous;
    }(Node));
    Anonymous.prototype.type = 'Anonymous';
    var MATH = Math$1;
    var Declaration = /** @class */ (function (_super) {
        tslib.__extends(Declaration, _super);
        function Declaration(name, value, important, merge, index, currentFileInfo, inline, variable) {
            var _this = _super.call(this) || this;
            _this.name = name;
            _this.value = (value instanceof Node) ? value : new Value([value ? new Anonymous(value) : null]);
            _this.important = important ? " " + important.trim() : '';
            _this.merge = merge;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.inline = inline || false;
            _this.variable = (variable !== undefined) ? variable
                : (name.charAt && (name.charAt(0) === '@'));
            _this.allowRoot = true;
            _this.setParent(_this.value, _this);
            return _this;
        }
        Declaration.prototype.genCSS = function (context, output) {
            output.add(this.name + (context.compress ? ':' : ': '), this.fileInfo(), this.getIndex());
            try {
                this.value.genCSS(context, output);
            }
            catch (e) {
                e.index = this._index;
                e.filename = this._fileInfo.filename;
                throw e;
            }
            output.add(this.important + ((this.inline || (context.lastRule && context.compress)) ? '' : ';'), this._fileInfo, this._index);
        };
        Declaration.prototype.eval = function (context) {
            var mathBypass = false;
            var prevMath;
            var name = this.name;
            var evaldValue;
            var variable = this.variable;
            if (typeof name !== 'string') {
                // expand 'primitive' name directly to get
                // things faster (~10% for benchmark.less):
                name = (name.length === 1) && (name[0] instanceof Keyword) ?
                    name[0].value : evalName(context, name);
                variable = false; // never treat expanded interpolation as new variable name
            }
            // @todo remove when parens-division is default
            if (name === 'font' && context.math === MATH.ALWAYS) {
                mathBypass = true;
                prevMath = context.math;
                context.math = MATH.PARENS_DIVISION;
            }
            try {
                context.importantScope.push({});
                evaldValue = this.value.eval(context);
                if (!this.variable && evaldValue.type === 'DetachedRuleset') {
                    throw { message: 'Rulesets cannot be evaluated on a property.',
                        index: this.getIndex(), filename: this.fileInfo().filename };
                }
                var important = this.important;
                var importantResult = context.importantScope.pop();
                if (!important && importantResult.important) {
                    important = importantResult.important;
                }
                return new Declaration(name, evaldValue, important, this.merge, this.getIndex(), this.fileInfo(), this.inline, variable);
            }
            catch (e) {
                if (typeof e.index !== 'number') {
                    e.index = this.getIndex();
                    e.filename = this.fileInfo().filename;
                }
                throw e;
            }
            finally {
                if (mathBypass) {
                    context.math = prevMath;
                }
            }
        };
        Declaration.prototype.makeImportant = function () {
            return new Declaration(this.name, this.value, '!important', this.merge, this.getIndex(), this.fileInfo(), this.inline);
        };
        return Declaration;
    }(Node));
    function evalName(context, name) {
        var value = '';
        var i;
        var n = name.length;
        var output = { add: function (s) { value += s; } };
        for (i = 0; i < n; i++) {
            name[i].eval(context).genCSS(context, output);
        }
        return value;
    }
    Declaration.prototype.type = 'Declaration';
    var debugInfo = function (context, ctx, lineSeparator) {
        var result = '';
        if (context.dumpLineNumbers && !context.compress) {
            switch (context.dumpLineNumbers) {
                case 'comments':
                    result = debugInfo.asComment(ctx);
                    break;
                case 'mediaquery':
                    result = debugInfo.asMediaQuery(ctx);
                    break;
                case 'all':
                    result = debugInfo.asComment(ctx) + (lineSeparator || '') + debugInfo.asMediaQuery(ctx);
                    break;
            }
        }
        return result;
    };
    debugInfo.asComment = function (ctx) { return "/* line " + ctx.debugInfo.lineNumber + ", " + ctx.debugInfo.fileName + " */\n"; };
    debugInfo.asMediaQuery = function (ctx) {
        var filenameWithProtocol = ctx.debugInfo.fileName;
        if (!/^[a-z]+:\/\//i.test(filenameWithProtocol)) {
            filenameWithProtocol = "file://" + filenameWithProtocol;
        }
        return "@media -sass-debug-info{filename{font-family:" + filenameWithProtocol.replace(/([.:\/\\])/g, function (a) {
            if (a == '\\') {
                a = '\/';
            }
            return "\\" + a;
        }) + "}line{font-family:\\00003" + ctx.debugInfo.lineNumber + "}}\n";
    };
    var Comment = /** @class */ (function (_super) {
        tslib.__extends(Comment, _super);
        function Comment(value, isLineComment, index, currentFileInfo) {
            var _this = _super.call(this) || this;
            _this.value = value;
            _this.isLineComment = isLineComment;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.allowRoot = true;
            return _this;
        }
        Comment.prototype.genCSS = function (context, output) {
            if (this.debugInfo) {
                output.add(debugInfo(context, this), this.fileInfo(), this.getIndex());
            }
            output.add(this.value);
        };
        Comment.prototype.isSilent = function (context) {
            var isCompressed = context.compress && this.value[2] !== '!';
            return this.isLineComment || isCompressed;
        };
        return Comment;
    }(Node));
    Comment.prototype.type = 'Comment';
    var contexts = {};
    var copyFromOriginal = function copyFromOriginal(original, destination, propertiesToCopy) {
        if (!original) {
            return;
        }
        for (var i_1 = 0; i_1 < propertiesToCopy.length; i_1++) {
            if (original.hasOwnProperty(propertiesToCopy[i_1])) {
                destination[propertiesToCopy[i_1]] = original[propertiesToCopy[i_1]];
            }
        }
    };
    /*
     parse is used whilst parsing
     */
    var parseCopyProperties = [
        // options
        'paths',
        'rewriteUrls',
        'rootpath',
        'strictImports',
        'insecure',
        'dumpLineNumbers',
        'compress',
        'syncImport',
        'chunkInput',
        'mime',
        'useFileCache',
        // context
        'processImports',
        // Used by the import manager to stop multiple import visitors being created.
        'pluginManager' // Used as the plugin manager for the session
    ];
    contexts.Parse = function (options) {
        copyFromOriginal(options, this, parseCopyProperties);
        if (typeof this.paths === 'string') {
            this.paths = [this.paths];
        }
    };
    var evalCopyProperties = [
        'paths',
        'compress',
        'math',
        'strictUnits',
        'sourceMap',
        'importMultiple',
        'urlArgs',
        'javascriptEnabled',
        'pluginManager',
        'importantScope',
        'rewriteUrls' // option - whether to adjust URL's to be relative
    ];
    function isPathRelative(path) {
        return !/^(?:[a-z-]+:|\/|#)/i.test(path);
    }
    function isPathLocalRelative(path) {
        return path.charAt(0) === '.';
    }
    contexts.Eval = /** @class */ (function () {
        function Eval(options, frames) {
            copyFromOriginal(options, this, evalCopyProperties);
            if (typeof this.paths === 'string') {
                this.paths = [this.paths];
            }
            this.frames = frames || [];
            this.importantScope = this.importantScope || [];
            this.inCalc = false;
            this.mathOn = true;
        }
        Eval.prototype.enterCalc = function () {
            if (!this.calcStack) {
                this.calcStack = [];
            }
            this.calcStack.push(true);
            this.inCalc = true;
        };
        Eval.prototype.exitCalc = function () {
            this.calcStack.pop();
            if (!this.calcStack) {
                this.inCalc = false;
            }
        };
        Eval.prototype.inParenthesis = function () {
            if (!this.parensStack) {
                this.parensStack = [];
            }
            this.parensStack.push(true);
        };
        Eval.prototype.outOfParenthesis = function () {
            this.parensStack.pop();
        };
        Eval.prototype.isMathOn = function (op) {
            if (!this.mathOn) {
                return false;
            }
            if (op === '/' && this.math !== Math$1.ALWAYS && (!this.parensStack || !this.parensStack.length)) {
                return false;
            }
            if (this.math > Math$1.PARENS_DIVISION) {
                return this.parensStack && this.parensStack.length;
            }
            return true;
        };
        Eval.prototype.pathRequiresRewrite = function (path) {
            var isRelative = this.rewriteUrls === RewriteUrls.LOCAL ? isPathLocalRelative : isPathRelative;
            return isRelative(path);
        };
        Eval.prototype.rewritePath = function (path, rootpath) {
            var newPath;
            rootpath = rootpath || '';
            newPath = this.normalizePath(rootpath + path);
            // If a path was explicit relative and the rootpath was not an absolute path
            // we must ensure that the new path is also explicit relative.
            if (isPathLocalRelative(path) &&
                isPathRelative(rootpath) &&
                isPathLocalRelative(newPath) === false) {
                newPath = "./" + newPath;
            }
            return newPath;
        };
        Eval.prototype.normalizePath = function (path) {
            var segments = path.split('/').reverse();
            var segment;
            path = [];
            while (segments.length !== 0) {
                segment = segments.pop();
                switch (segment) {
                    case '.':
                        break;
                    case '..':
                        if ((path.length === 0) || (path[path.length - 1] === '..')) {
                            path.push(segment);
                        }
                        else {
                            path.pop();
                        }
                        break;
                    default:
                        path.push(segment);
                        break;
                }
            }
            return path.join('/');
        };
        return Eval;
    }());
    function makeRegistry(base) {
        return {
            _data: {},
            add: function (name, func) {
                // precautionary case conversion, as later querying of
                // the registry by function-caller uses lower case as well.
                name = name.toLowerCase();
                if (this._data.hasOwnProperty(name))
                    ;
                this._data[name] = func;
            },
            addMultiple: function (functions) {
                var _this = this;
                Object.keys(functions).forEach(function (name) {
                    _this.add(name, functions[name]);
                });
            },
            get: function (name) {
                return this._data[name] || (base && base.get(name));
            },
            getLocalFunctions: function () {
                return this._data;
            },
            inherit: function () {
                return makeRegistry(this);
            },
            create: function (base) {
                return makeRegistry(base);
            }
        };
    }
    var functionRegistry = makeRegistry(null);
    var defaultFunc = {
        eval: function () {
            var v = this.value_;
            var e = this.error_;
            if (e) {
                throw e;
            }
            if (v != null) {
                return v ? Keyword.True : Keyword.False;
            }
        },
        value: function (v) {
            this.value_ = v;
        },
        error: function (e) {
            this.error_ = e;
        },
        reset: function () {
            this.value_ = this.error_ = null;
        }
    };
    var Ruleset = /** @class */ (function (_super) {
        tslib.__extends(Ruleset, _super);
        function Ruleset(selectors, rules, strictImports, visibilityInfo) {
            var _this = _super.call(this) || this;
            _this.selectors = selectors;
            _this.rules = rules;
            _this._lookups = {};
            _this._variables = null;
            _this._properties = null;
            _this.strictImports = strictImports;
            _this.copyVisibilityInfo(visibilityInfo);
            _this.allowRoot = true;
            _this.setParent(_this.selectors, _this);
            _this.setParent(_this.rules, _this);
            return _this;
        }
        Ruleset.prototype.isRulesetLike = function () {
            return true;
        };
        Ruleset.prototype.accept = function (visitor) {
            if (this.paths) {
                this.paths = visitor.visitArray(this.paths, true);
            }
            else if (this.selectors) {
                this.selectors = visitor.visitArray(this.selectors);
            }
            if (this.rules && this.rules.length) {
                this.rules = visitor.visitArray(this.rules);
            }
        };
        Ruleset.prototype.eval = function (context) {
            var selectors;
            var selCnt;
            var selector;
            var i;
            var hasVariable;
            var hasOnePassingSelector = false;
            if (this.selectors && (selCnt = this.selectors.length)) {
                selectors = new Array(selCnt);
                defaultFunc.error({
                    type: 'Syntax',
                    message: 'it is currently only allowed in parametric mixin guards,'
                });
                for (i = 0; i < selCnt; i++) {
                    selector = this.selectors[i].eval(context);
                    for (var j = 0; j < selector.elements.length; j++) {
                        if (selector.elements[j].isVariable) {
                            hasVariable = true;
                            break;
                        }
                    }
                    selectors[i] = selector;
                    if (selector.evaldCondition) {
                        hasOnePassingSelector = true;
                    }
                }
                if (hasVariable) {
                    var toParseSelectors = new Array(selCnt);
                    for (i = 0; i < selCnt; i++) {
                        selector = selectors[i];
                        toParseSelectors[i] = selector.toCSS(context);
                    }
                    this.parse.parseNode(toParseSelectors.join(','), ["selectors"], selectors[0].getIndex(), selectors[0].fileInfo(), function (err, result) {
                        if (result) {
                            selectors = flattenArray(result);
                        }
                    });
                }
                defaultFunc.reset();
            }
            else {
                hasOnePassingSelector = true;
            }
            var rules = this.rules ? copyArray(this.rules) : null;
            var ruleset = new Ruleset(selectors, rules, this.strictImports, this.visibilityInfo());
            var rule;
            var subRule;
            ruleset.originalRuleset = this;
            ruleset.root = this.root;
            ruleset.firstRoot = this.firstRoot;
            ruleset.allowImports = this.allowImports;
            if (this.debugInfo) {
                ruleset.debugInfo = this.debugInfo;
            }
            if (!hasOnePassingSelector) {
                rules.length = 0;
            }
            // inherit a function registry from the frames stack when possible;
            // otherwise from the global registry
            ruleset.functionRegistry = (function (frames) {
                var i = 0;
                var n = frames.length;
                var found;
                for (; i !== n; ++i) {
                    found = frames[i].functionRegistry;
                    if (found) {
                        return found;
                    }
                }
                return functionRegistry;
            })(context.frames).inherit();
            // push the current ruleset to the frames stack
            var ctxFrames = context.frames;
            ctxFrames.unshift(ruleset);
            // currrent selectors
            var ctxSelectors = context.selectors;
            if (!ctxSelectors) {
                context.selectors = ctxSelectors = [];
            }
            ctxSelectors.unshift(this.selectors);
            // Evaluate imports
            if (ruleset.root || ruleset.allowImports || !ruleset.strictImports) {
                ruleset.evalImports(context);
            }
            // Store the frames around mixin definitions,
            // so they can be evaluated like closures when the time comes.
            var rsRules = ruleset.rules;
            for (i = 0; (rule = rsRules[i]); i++) {
                if (rule.evalFirst) {
                    rsRules[i] = rule.eval(context);
                }
            }
            var mediaBlockCount = (context.mediaBlocks && context.mediaBlocks.length) || 0;
            // Evaluate mixin calls.
            for (i = 0; (rule = rsRules[i]); i++) {
                if (rule.type === 'MixinCall') {
                    /* jshint loopfunc:true */
                    rules = rule.eval(context).filter(function (r) {
                        if ((r instanceof Declaration) && r.variable) {
                            // do not pollute the scope if the variable is
                            // already there. consider returning false here
                            // but we need a way to "return" variable from mixins
                            return !(ruleset.variable(r.name));
                        }
                        return true;
                    });
                    rsRules.splice.apply(rsRules, [i, 1].concat(rules));
                    i += rules.length - 1;
                    ruleset.resetCache();
                }
                else if (rule.type === 'VariableCall') {
                    /* jshint loopfunc:true */
                    rules = rule.eval(context).rules.filter(function (r) {
                        if ((r instanceof Declaration) && r.variable) {
                            // do not pollute the scope at all
                            return false;
                        }
                        return true;
                    });
                    rsRules.splice.apply(rsRules, [i, 1].concat(rules));
                    i += rules.length - 1;
                    ruleset.resetCache();
                }
            }
            // Evaluate everything else
            for (i = 0; (rule = rsRules[i]); i++) {
                if (!rule.evalFirst) {
                    rsRules[i] = rule = rule.eval ? rule.eval(context) : rule;
                }
            }
            // Evaluate everything else
            for (i = 0; (rule = rsRules[i]); i++) {
                // for rulesets, check if it is a css guard and can be removed
                if (rule instanceof Ruleset && rule.selectors && rule.selectors.length === 1) {
                    // check if it can be folded in (e.g. & where)
                    if (rule.selectors[0] && rule.selectors[0].isJustParentSelector()) {
                        rsRules.splice(i--, 1);
                        for (var j = 0; (subRule = rule.rules[j]); j++) {
                            if (subRule instanceof Node) {
                                subRule.copyVisibilityInfo(rule.visibilityInfo());
                                if (!(subRule instanceof Declaration) || !subRule.variable) {
                                    rsRules.splice(++i, 0, subRule);
                                }
                            }
                        }
                    }
                }
            }
            // Pop the stack
            ctxFrames.shift();
            ctxSelectors.shift();
            if (context.mediaBlocks) {
                for (i = mediaBlockCount; i < context.mediaBlocks.length; i++) {
                    context.mediaBlocks[i].bubbleSelectors(selectors);
                }
            }
            return ruleset;
        };
        Ruleset.prototype.evalImports = function (context) {
            var rules = this.rules;
            var i;
            var importRules;
            if (!rules) {
                return;
            }
            for (i = 0; i < rules.length; i++) {
                if (rules[i].type === 'Import') {
                    importRules = rules[i].eval(context);
                    if (importRules && (importRules.length || importRules.length === 0)) {
                        rules.splice.apply(rules, [i, 1].concat(importRules));
                        i += importRules.length - 1;
                    }
                    else {
                        rules.splice(i, 1, importRules);
                    }
                    this.resetCache();
                }
            }
        };
        Ruleset.prototype.makeImportant = function () {
            var result = new Ruleset(this.selectors, this.rules.map(function (r) {
                if (r.makeImportant) {
                    return r.makeImportant();
                }
                else {
                    return r;
                }
            }), this.strictImports, this.visibilityInfo());
            return result;
        };
        Ruleset.prototype.matchArgs = function (args) {
            return !args || args.length === 0;
        };
        // lets you call a css selector with a guard
        Ruleset.prototype.matchCondition = function (args, context) {
            var lastSelector = this.selectors[this.selectors.length - 1];
            if (!lastSelector.evaldCondition) {
                return false;
            }
            if (lastSelector.condition &&
                !lastSelector.condition.eval(new contexts.Eval(context, context.frames))) {
                return false;
            }
            return true;
        };
        Ruleset.prototype.resetCache = function () {
            this._rulesets = null;
            this._variables = null;
            this._properties = null;
            this._lookups = {};
        };
        Ruleset.prototype.variables = function () {
            if (!this._variables) {
                this._variables = !this.rules ? {} : this.rules.reduce(function (hash, r) {
                    if (r instanceof Declaration && r.variable === true) {
                        hash[r.name] = r;
                    }
                    // when evaluating variables in an import statement, imports have not been eval'd
                    // so we need to go inside import statements.
                    // guard against root being a string (in the case of inlined less)
                    if (r.type === 'Import' && r.root && r.root.variables) {
                        var vars = r.root.variables();
                        for (var name_1 in vars) {
                            if (vars.hasOwnProperty(name_1)) {
                                hash[name_1] = r.root.variable(name_1);
                            }
                        }
                    }
                    return hash;
                }, {});
            }
            return this._variables;
        };
        Ruleset.prototype.properties = function () {
            if (!this._properties) {
                this._properties = !this.rules ? {} : this.rules.reduce(function (hash, r) {
                    if (r instanceof Declaration && r.variable !== true) {
                        var name_2 = (r.name.length === 1) && (r.name[0] instanceof Keyword) ?
                            r.name[0].value : r.name;
                        // Properties don't overwrite as they can merge
                        if (!hash["$" + name_2]) {
                            hash["$" + name_2] = [r];
                        }
                        else {
                            hash["$" + name_2].push(r);
                        }
                    }
                    return hash;
                }, {});
            }
            return this._properties;
        };
        Ruleset.prototype.variable = function (name) {
            var decl = this.variables()[name];
            if (decl) {
                return this.parseValue(decl);
            }
        };
        Ruleset.prototype.property = function (name) {
            var decl = this.properties()[name];
            if (decl) {
                return this.parseValue(decl);
            }
        };
        Ruleset.prototype.lastDeclaration = function () {
            for (var i_1 = this.rules.length; i_1 > 0; i_1--) {
                var decl = this.rules[i_1 - 1];
                if (decl instanceof Declaration) {
                    return this.parseValue(decl);
                }
            }
        };
        Ruleset.prototype.parseValue = function (toParse) {
            var self = this;
            function transformDeclaration(decl) {
                if (decl.value instanceof Anonymous && !decl.parsed) {
                    if (typeof decl.value.value === 'string') {
                        this.parse.parseNode(decl.value.value, ['value', 'important'], decl.value.getIndex(), decl.fileInfo(), function (err, result) {
                            if (err) {
                                decl.parsed = true;
                            }
                            if (result) {
                                decl.value = result[0];
                                decl.important = result[1] || '';
                                decl.parsed = true;
                            }
                        });
                    }
                    else {
                        decl.parsed = true;
                    }
                    return decl;
                }
                else {
                    return decl;
                }
            }
            if (!Array.isArray(toParse)) {
                return transformDeclaration.call(self, toParse);
            }
            else {
                var nodes_1 = [];
                toParse.forEach(function (n) {
                    nodes_1.push(transformDeclaration.call(self, n));
                });
                return nodes_1;
            }
        };
        Ruleset.prototype.rulesets = function () {
            if (!this.rules) {
                return [];
            }
            var filtRules = [];
            var rules = this.rules;
            var i;
            var rule;
            for (i = 0; (rule = rules[i]); i++) {
                if (rule.isRuleset) {
                    filtRules.push(rule);
                }
            }
            return filtRules;
        };
        Ruleset.prototype.prependRule = function (rule) {
            var rules = this.rules;
            if (rules) {
                rules.unshift(rule);
            }
            else {
                this.rules = [rule];
            }
            this.setParent(rule, this);
        };
        Ruleset.prototype.find = function (selector, self, filter) {
            if (self === void 0) {
                self = this;
            }
            var rules = [];
            var match;
            var foundMixins;
            var key = selector.toCSS();
            if (key in this._lookups) {
                return this._lookups[key];
            }
            this.rulesets().forEach(function (rule) {
                if (rule !== self) {
                    for (var j = 0; j < rule.selectors.length; j++) {
                        match = selector.match(rule.selectors[j]);
                        if (match) {
                            if (selector.elements.length > match) {
                                if (!filter || filter(rule)) {
                                    foundMixins = rule.find(new Selector(selector.elements.slice(match)), self, filter);
                                    for (var i_2 = 0; i_2 < foundMixins.length; ++i_2) {
                                        foundMixins[i_2].path.push(rule);
                                    }
                                    Array.prototype.push.apply(rules, foundMixins);
                                }
                            }
                            else {
                                rules.push({ rule: rule, path: [] });
                            }
                            break;
                        }
                    }
                }
            });
            this._lookups[key] = rules;
            return rules;
        };
        Ruleset.prototype.genCSS = function (context, output) {
            var i;
            var j;
            var charsetRuleNodes = [];
            var ruleNodes = [];
            var // Line number debugging
            debugInfo$1;
            var rule;
            var path;
            context.tabLevel = (context.tabLevel || 0);
            if (!this.root) {
                context.tabLevel++;
            }
            var tabRuleStr = context.compress ? '' : Array(context.tabLevel + 1).join('  ');
            var tabSetStr = context.compress ? '' : Array(context.tabLevel).join('  ');
            var sep;
            var charsetNodeIndex = 0;
            var importNodeIndex = 0;
            for (i = 0; (rule = this.rules[i]); i++) {
                if (rule instanceof Comment) {
                    if (importNodeIndex === i) {
                        importNodeIndex++;
                    }
                    ruleNodes.push(rule);
                }
                else if (rule.isCharset && rule.isCharset()) {
                    ruleNodes.splice(charsetNodeIndex, 0, rule);
                    charsetNodeIndex++;
                    importNodeIndex++;
                }
                else if (rule.type === 'Import') {
                    ruleNodes.splice(importNodeIndex, 0, rule);
                    importNodeIndex++;
                }
                else {
                    ruleNodes.push(rule);
                }
            }
            ruleNodes = charsetRuleNodes.concat(ruleNodes);
            // If this is the root node, we don't render
            // a selector, or {}.
            if (!this.root) {
                debugInfo$1 = debugInfo(context, this, tabSetStr);
                if (debugInfo$1) {
                    output.add(debugInfo$1);
                    output.add(tabSetStr);
                }
                var paths = this.paths;
                var pathCnt = paths.length;
                var pathSubCnt = void 0;
                sep = context.compress ? ',' : (",\n" + tabSetStr);
                for (i = 0; i < pathCnt; i++) {
                    path = paths[i];
                    if (!(pathSubCnt = path.length)) {
                        continue;
                    }
                    if (i > 0) {
                        output.add(sep);
                    }
                    context.firstSelector = true;
                    path[0].genCSS(context, output);
                    context.firstSelector = false;
                    for (j = 1; j < pathSubCnt; j++) {
                        path[j].genCSS(context, output);
                    }
                }
                output.add((context.compress ? '{' : ' {\n') + tabRuleStr);
            }
            // Compile rules and rulesets
            for (i = 0; (rule = ruleNodes[i]); i++) {
                if (i + 1 === ruleNodes.length) {
                    context.lastRule = true;
                }
                var currentLastRule = context.lastRule;
                if (rule.isRulesetLike(rule)) {
                    context.lastRule = false;
                }
                if (rule.genCSS) {
                    rule.genCSS(context, output);
                }
                else if (rule.value) {
                    output.add(rule.value.toString());
                }
                context.lastRule = currentLastRule;
                if (!context.lastRule && rule.isVisible()) {
                    output.add(context.compress ? '' : ("\n" + tabRuleStr));
                }
                else {
                    context.lastRule = false;
                }
            }
            if (!this.root) {
                output.add((context.compress ? '}' : "\n" + tabSetStr + "}"));
                context.tabLevel--;
            }
            if (!output.isEmpty() && !context.compress && this.firstRoot) {
                output.add('\n');
            }
        };
        Ruleset.prototype.joinSelectors = function (paths, context, selectors) {
            for (var s = 0; s < selectors.length; s++) {
                this.joinSelector(paths, context, selectors[s]);
            }
        };
        Ruleset.prototype.joinSelector = function (paths, context, selector) {
            function createParenthesis(elementsToPak, originalElement) {
                var replacementParen;
                var j;
                if (elementsToPak.length === 0) {
                    replacementParen = new Paren(elementsToPak[0]);
                }
                else {
                    var insideParent = new Array(elementsToPak.length);
                    for (j = 0; j < elementsToPak.length; j++) {
                        insideParent[j] = new Element(null, elementsToPak[j], originalElement.isVariable, originalElement._index, originalElement._fileInfo);
                    }
                    replacementParen = new Paren(new Selector(insideParent));
                }
                return replacementParen;
            }
            function createSelector(containedElement, originalElement) {
                var element;
                var selector;
                element = new Element(null, containedElement, originalElement.isVariable, originalElement._index, originalElement._fileInfo);
                selector = new Selector([element]);
                return selector;
            }
            // joins selector path from `beginningPath` with selector path in `addPath`
            // `replacedElement` contains element that is being replaced by `addPath`
            // returns concatenated path
            function addReplacementIntoPath(beginningPath, addPath, replacedElement, originalSelector) {
                var newSelectorPath;
                var lastSelector;
                var newJoinedSelector;
                // our new selector path
                newSelectorPath = [];
                // construct the joined selector - if & is the first thing this will be empty,
                // if not newJoinedSelector will be the last set of elements in the selector
                if (beginningPath.length > 0) {
                    newSelectorPath = copyArray(beginningPath);
                    lastSelector = newSelectorPath.pop();
                    newJoinedSelector = originalSelector.createDerived(copyArray(lastSelector.elements));
                }
                else {
                    newJoinedSelector = originalSelector.createDerived([]);
                }
                if (addPath.length > 0) {
                    // /deep/ is a CSS4 selector - (removed, so should deprecate)
                    // that is valid without anything in front of it
                    // so if the & does not have a combinator that is "" or " " then
                    // and there is a combinator on the parent, then grab that.
                    // this also allows + a { & .b { .a & { ... though not sure why you would want to do that
                    var combinator = replacedElement.combinator;
                    var parentEl = addPath[0].elements[0];
                    if (combinator.emptyOrWhitespace && !parentEl.combinator.emptyOrWhitespace) {
                        combinator = parentEl.combinator;
                    }
                    // join the elements so far with the first part of the parent
                    newJoinedSelector.elements.push(new Element(combinator, parentEl.value, replacedElement.isVariable, replacedElement._index, replacedElement._fileInfo));
                    newJoinedSelector.elements = newJoinedSelector.elements.concat(addPath[0].elements.slice(1));
                }
                // now add the joined selector - but only if it is not empty
                if (newJoinedSelector.elements.length !== 0) {
                    newSelectorPath.push(newJoinedSelector);
                }
                // put together the parent selectors after the join (e.g. the rest of the parent)
                if (addPath.length > 1) {
                    var restOfPath = addPath.slice(1);
                    restOfPath = restOfPath.map(function (selector) { return selector.createDerived(selector.elements, []); });
                    newSelectorPath = newSelectorPath.concat(restOfPath);
                }
                return newSelectorPath;
            }
            // joins selector path from `beginningPath` with every selector path in `addPaths` array
            // `replacedElement` contains element that is being replaced by `addPath`
            // returns array with all concatenated paths
            function addAllReplacementsIntoPath(beginningPath, addPaths, replacedElement, originalSelector, result) {
                var j;
                for (j = 0; j < beginningPath.length; j++) {
                    var newSelectorPath = addReplacementIntoPath(beginningPath[j], addPaths, replacedElement, originalSelector);
                    result.push(newSelectorPath);
                }
                return result;
            }
            function mergeElementsOnToSelectors(elements, selectors) {
                var i;
                var sel;
                if (elements.length === 0) {
                    return;
                }
                if (selectors.length === 0) {
                    selectors.push([new Selector(elements)]);
                    return;
                }
                for (i = 0; (sel = selectors[i]); i++) {
                    // if the previous thing in sel is a parent this needs to join on to it
                    if (sel.length > 0) {
                        sel[sel.length - 1] = sel[sel.length - 1].createDerived(sel[sel.length - 1].elements.concat(elements));
                    }
                    else {
                        sel.push(new Selector(elements));
                    }
                }
            }
            // replace all parent selectors inside `inSelector` by content of `context` array
            // resulting selectors are returned inside `paths` array
            // returns true if `inSelector` contained at least one parent selector
            function replaceParentSelector(paths, context, inSelector) {
                // The paths are [[Selector]]
                // The first list is a list of comma separated selectors
                // The inner list is a list of inheritance separated selectors
                // e.g.
                // .a, .b {
                //   .c {
                //   }
                // }
                // == [[.a] [.c]] [[.b] [.c]]
                //
                var i;
                var j;
                var k;
                var currentElements;
                var newSelectors;
                var selectorsMultiplied;
                var sel;
                var el;
                var hadParentSelector = false;
                var length;
                var lastSelector;
                function findNestedSelector(element) {
                    var maybeSelector;
                    if (!(element.value instanceof Paren)) {
                        return null;
                    }
                    maybeSelector = element.value.value;
                    if (!(maybeSelector instanceof Selector)) {
                        return null;
                    }
                    return maybeSelector;
                }
                // the elements from the current selector so far
                currentElements = [];
                // the current list of new selectors to add to the path.
                // We will build it up. We initiate it with one empty selector as we "multiply" the new selectors
                // by the parents
                newSelectors = [
                    []
                ];
                for (i = 0; (el = inSelector.elements[i]); i++) {
                    // non parent reference elements just get added
                    if (el.value !== '&') {
                        var nestedSelector = findNestedSelector(el);
                        if (nestedSelector != null) {
                            // merge the current list of non parent selector elements
                            // on to the current list of selectors to add
                            mergeElementsOnToSelectors(currentElements, newSelectors);
                            var nestedPaths = [];
                            var replaced = void 0;
                            var replacedNewSelectors = [];
                            replaced = replaceParentSelector(nestedPaths, context, nestedSelector);
                            hadParentSelector = hadParentSelector || replaced;
                            // the nestedPaths array should have only one member - replaceParentSelector does not multiply selectors
                            for (k = 0; k < nestedPaths.length; k++) {
                                var replacementSelector = createSelector(createParenthesis(nestedPaths[k], el), el);
                                addAllReplacementsIntoPath(newSelectors, [replacementSelector], el, inSelector, replacedNewSelectors);
                            }
                            newSelectors = replacedNewSelectors;
                            currentElements = [];
                        }
                        else {
                            currentElements.push(el);
                        }
                    }
                    else {
                        hadParentSelector = true;
                        // the new list of selectors to add
                        selectorsMultiplied = [];
                        // merge the current list of non parent selector elements
                        // on to the current list of selectors to add
                        mergeElementsOnToSelectors(currentElements, newSelectors);
                        // loop through our current selectors
                        for (j = 0; j < newSelectors.length; j++) {
                            sel = newSelectors[j];
                            // if we don't have any parent paths, the & might be in a mixin so that it can be used
                            // whether there are parents or not
                            if (context.length === 0) {
                                // the combinator used on el should now be applied to the next element instead so that
                                // it is not lost
                                if (sel.length > 0) {
                                    sel[0].elements.push(new Element(el.combinator, '', el.isVariable, el._index, el._fileInfo));
                                }
                                selectorsMultiplied.push(sel);
                            }
                            else {
                                // and the parent selectors
                                for (k = 0; k < context.length; k++) {
                                    // We need to put the current selectors
                                    // then join the last selector's elements on to the parents selectors
                                    var newSelectorPath = addReplacementIntoPath(sel, context[k], el, inSelector);
                                    // add that to our new set of selectors
                                    selectorsMultiplied.push(newSelectorPath);
                                }
                            }
                        }
                        // our new selectors has been multiplied, so reset the state
                        newSelectors = selectorsMultiplied;
                        currentElements = [];
                    }
                }
                // if we have any elements left over (e.g. .a& .b == .b)
                // add them on to all the current selectors
                mergeElementsOnToSelectors(currentElements, newSelectors);
                for (i = 0; i < newSelectors.length; i++) {
                    length = newSelectors[i].length;
                    if (length > 0) {
                        paths.push(newSelectors[i]);
                        lastSelector = newSelectors[i][length - 1];
                        newSelectors[i][length - 1] = lastSelector.createDerived(lastSelector.elements, inSelector.extendList);
                    }
                }
                return hadParentSelector;
            }
            function deriveSelector(visibilityInfo, deriveFrom) {
                var newSelector = deriveFrom.createDerived(deriveFrom.elements, deriveFrom.extendList, deriveFrom.evaldCondition);
                newSelector.copyVisibilityInfo(visibilityInfo);
                return newSelector;
            }
            // joinSelector code follows
            var i;
            var newPaths;
            var hadParentSelector;
            newPaths = [];
            hadParentSelector = replaceParentSelector(newPaths, context, selector);
            if (!hadParentSelector) {
                if (context.length > 0) {
                    newPaths = [];
                    for (i = 0; i < context.length; i++) {
                        var concatenated = context[i].map(deriveSelector.bind(this, selector.visibilityInfo()));
                        concatenated.push(selector);
                        newPaths.push(concatenated);
                    }
                }
                else {
                    newPaths = [[selector]];
                }
            }
            for (i = 0; i < newPaths.length; i++) {
                paths.push(newPaths[i]);
            }
        };
        return Ruleset;
    }(Node));
    Ruleset.prototype.type = 'Ruleset';
    Ruleset.prototype.isRuleset = true;
    var AtRule = /** @class */ (function (_super) {
        tslib.__extends(AtRule, _super);
        function AtRule(name, value, rules, index, currentFileInfo, debugInfo, isRooted, visibilityInfo) {
            var _this = _super.call(this) || this;
            var i;
            _this.name = name;
            _this.value = (value instanceof Node) ? value : (value ? new Anonymous(value) : value);
            if (rules) {
                if (Array.isArray(rules)) {
                    _this.rules = rules;
                }
                else {
                    _this.rules = [rules];
                    _this.rules[0].selectors = (new Selector([], null, null, index, currentFileInfo)).createEmptySelectors();
                }
                for (i = 0; i < _this.rules.length; i++) {
                    _this.rules[i].allowImports = true;
                }
                _this.setParent(_this.rules, _this);
            }
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.debugInfo = debugInfo;
            _this.isRooted = isRooted || false;
            _this.copyVisibilityInfo(visibilityInfo);
            _this.allowRoot = true;
            return _this;
        }
        AtRule.prototype.accept = function (visitor) {
            var value = this.value;
            var rules = this.rules;
            if (rules) {
                this.rules = visitor.visitArray(rules);
            }
            if (value) {
                this.value = visitor.visit(value);
            }
        };
        AtRule.prototype.isRulesetLike = function () {
            return this.rules || !this.isCharset();
        };
        AtRule.prototype.isCharset = function () {
            return '@charset' === this.name;
        };
        AtRule.prototype.genCSS = function (context, output) {
            var value = this.value;
            var rules = this.rules;
            output.add(this.name, this.fileInfo(), this.getIndex());
            if (value) {
                output.add(' ');
                value.genCSS(context, output);
            }
            if (rules) {
                this.outputRuleset(context, output, rules);
            }
            else {
                output.add(';');
            }
        };
        AtRule.prototype.eval = function (context) {
            var mediaPathBackup;
            var mediaBlocksBackup;
            var value = this.value;
            var rules = this.rules;
            // media stored inside other atrule should not bubble over it
            // backpup media bubbling information
            mediaPathBackup = context.mediaPath;
            mediaBlocksBackup = context.mediaBlocks;
            // deleted media bubbling information
            context.mediaPath = [];
            context.mediaBlocks = [];
            if (value) {
                value = value.eval(context);
            }
            if (rules) {
                // assuming that there is only one rule at this point - that is how parser constructs the rule
                rules = [rules[0].eval(context)];
                rules[0].root = true;
            }
            // restore media bubbling information
            context.mediaPath = mediaPathBackup;
            context.mediaBlocks = mediaBlocksBackup;
            return new AtRule(this.name, value, rules, this.getIndex(), this.fileInfo(), this.debugInfo, this.isRooted, this.visibilityInfo());
        };
        AtRule.prototype.variable = function (name) {
            if (this.rules) {
                // assuming that there is only one rule at this point - that is how parser constructs the rule
                return Ruleset.prototype.variable.call(this.rules[0], name);
            }
        };
        AtRule.prototype.find = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (this.rules) {
                // assuming that there is only one rule at this point - that is how parser constructs the rule
                return Ruleset.prototype.find.apply(this.rules[0], args);
            }
        };
        AtRule.prototype.rulesets = function () {
            if (this.rules) {
                // assuming that there is only one rule at this point - that is how parser constructs the rule
                return Ruleset.prototype.rulesets.apply(this.rules[0]);
            }
        };
        AtRule.prototype.outputRuleset = function (context, output, rules) {
            var ruleCnt = rules.length;
            var i;
            context.tabLevel = (context.tabLevel | 0) + 1;
            // Compressed
            if (context.compress) {
                output.add('{');
                for (i = 0; i < ruleCnt; i++) {
                    rules[i].genCSS(context, output);
                }
                output.add('}');
                context.tabLevel--;
                return;
            }
            // Non-compressed
            var tabSetStr = "\n" + Array(context.tabLevel).join('  ');
            var tabRuleStr = tabSetStr + "  ";
            if (!ruleCnt) {
                output.add(" {" + tabSetStr + "}");
            }
            else {
                output.add(" {" + tabRuleStr);
                rules[0].genCSS(context, output);
                for (i = 1; i < ruleCnt; i++) {
                    output.add(tabRuleStr);
                    rules[i].genCSS(context, output);
                }
                output.add(tabSetStr + "}");
            }
            context.tabLevel--;
        };
        return AtRule;
    }(Node));
    AtRule.prototype.type = 'AtRule';
    var DetachedRuleset = /** @class */ (function (_super) {
        tslib.__extends(DetachedRuleset, _super);
        function DetachedRuleset(ruleset, frames) {
            var _this = _super.call(this) || this;
            _this.ruleset = ruleset;
            _this.frames = frames;
            _this.setParent(_this.ruleset, _this);
            return _this;
        }
        DetachedRuleset.prototype.accept = function (visitor) {
            this.ruleset = visitor.visit(this.ruleset);
        };
        DetachedRuleset.prototype.eval = function (context) {
            var frames = this.frames || copyArray(context.frames);
            return new DetachedRuleset(this.ruleset, frames);
        };
        DetachedRuleset.prototype.callEval = function (context) {
            return this.ruleset.eval(this.frames ? new contexts.Eval(context, this.frames.concat(context.frames)) : context);
        };
        return DetachedRuleset;
    }(Node));
    DetachedRuleset.prototype.type = 'DetachedRuleset';
    DetachedRuleset.prototype.evalFirst = true;
    var Unit = /** @class */ (function (_super) {
        tslib.__extends(Unit, _super);
        function Unit(numerator, denominator, backupUnit) {
            var _this = _super.call(this) || this;
            _this.numerator = numerator ? copyArray(numerator).sort() : [];
            _this.denominator = denominator ? copyArray(denominator).sort() : [];
            if (backupUnit) {
                _this.backupUnit = backupUnit;
            }
            else if (numerator && numerator.length) {
                _this.backupUnit = numerator[0];
            }
            return _this;
        }
        Unit.prototype.clone = function () {
            return new Unit(copyArray(this.numerator), copyArray(this.denominator), this.backupUnit);
        };
        Unit.prototype.genCSS = function (context, output) {
            // Dimension checks the unit is singular and throws an error if in strict math mode.
            var strictUnits = context && context.strictUnits;
            if (this.numerator.length === 1) {
                output.add(this.numerator[0]); // the ideal situation
            }
            else if (!strictUnits && this.backupUnit) {
                output.add(this.backupUnit);
            }
            else if (!strictUnits && this.denominator.length) {
                output.add(this.denominator[0]);
            }
        };
        Unit.prototype.toString = function () {
            var i;
            var returnStr = this.numerator.join('*');
            for (i = 0; i < this.denominator.length; i++) {
                returnStr += "/" + this.denominator[i];
            }
            return returnStr;
        };
        Unit.prototype.compare = function (other) {
            return this.is(other.toString()) ? 0 : undefined;
        };
        Unit.prototype.is = function (unitString) {
            return this.toString().toUpperCase() === unitString.toUpperCase();
        };
        Unit.prototype.isLength = function () {
            return RegExp('^(px|em|ex|ch|rem|in|cm|mm|pc|pt|ex|vw|vh|vmin|vmax)$', 'gi').test(this.toCSS());
        };
        Unit.prototype.isEmpty = function () {
            return this.numerator.length === 0 && this.denominator.length === 0;
        };
        Unit.prototype.isSingular = function () {
            return this.numerator.length <= 1 && this.denominator.length === 0;
        };
        Unit.prototype.map = function (callback) {
            var i;
            for (i = 0; i < this.numerator.length; i++) {
                this.numerator[i] = callback(this.numerator[i], false);
            }
            for (i = 0; i < this.denominator.length; i++) {
                this.denominator[i] = callback(this.denominator[i], true);
            }
        };
        Unit.prototype.usedUnits = function () {
            var group;
            var result = {};
            var mapUnit;
            var groupName;
            mapUnit = function (atomicUnit) {
                /* jshint loopfunc:true */
                if (group.hasOwnProperty(atomicUnit) && !result[groupName]) {
                    result[groupName] = atomicUnit;
                }
                return atomicUnit;
            };
            for (groupName in unitConversions) {
                if (unitConversions.hasOwnProperty(groupName)) {
                    group = unitConversions[groupName];
                    this.map(mapUnit);
                }
            }
            return result;
        };
        Unit.prototype.cancel = function () {
            var counter = {};
            var atomicUnit;
            var i;
            for (i = 0; i < this.numerator.length; i++) {
                atomicUnit = this.numerator[i];
                counter[atomicUnit] = (counter[atomicUnit] || 0) + 1;
            }
            for (i = 0; i < this.denominator.length; i++) {
                atomicUnit = this.denominator[i];
                counter[atomicUnit] = (counter[atomicUnit] || 0) - 1;
            }
            this.numerator = [];
            this.denominator = [];
            for (atomicUnit in counter) {
                if (counter.hasOwnProperty(atomicUnit)) {
                    var count = counter[atomicUnit];
                    if (count > 0) {
                        for (i = 0; i < count; i++) {
                            this.numerator.push(atomicUnit);
                        }
                    }
                    else if (count < 0) {
                        for (i = 0; i < -count; i++) {
                            this.denominator.push(atomicUnit);
                        }
                    }
                }
            }
            this.numerator.sort();
            this.denominator.sort();
        };
        return Unit;
    }(Node));
    Unit.prototype.type = 'Unit';
    //
    // A number with a unit
    //
    var Dimension = /** @class */ (function (_super) {
        tslib.__extends(Dimension, _super);
        function Dimension(value, unit) {
            var _this = _super.call(this) || this;
            _this.value = parseFloat(value);
            if (isNaN(_this.value)) {
                throw new Error('Dimension is not a number.');
            }
            _this.unit = (unit && unit instanceof Unit) ? unit :
                new Unit(unit ? [unit] : undefined);
            _this.setParent(_this.unit, _this);
            return _this;
        }
        Dimension.prototype.accept = function (visitor) {
            this.unit = visitor.visit(this.unit);
        };
        Dimension.prototype.eval = function (context) {
            return this;
        };
        Dimension.prototype.toColor = function () {
            return new Color([this.value, this.value, this.value]);
        };
        Dimension.prototype.genCSS = function (context, output) {
            if ((context && context.strictUnits) && !this.unit.isSingular()) {
                throw new Error("Multiple units in dimension. Correct the units or use the unit function. Bad unit: " + this.unit.toString());
            }
            var value = this.fround(context, this.value);
            var strValue = String(value);
            if (value !== 0 && value < 0.000001 && value > -0.000001) {
                // would be output 1e-6 etc.
                strValue = value.toFixed(20).replace(/0+$/, '');
            }
            if (context && context.compress) {
                // Zero values doesn't need a unit
                if (value === 0 && this.unit.isLength()) {
                    output.add(strValue);
                    return;
                }
                // Float values doesn't need a leading zero
                if (value > 0 && value < 1) {
                    strValue = (strValue).substr(1);
                }
            }
            output.add(strValue);
            this.unit.genCSS(context, output);
        };
        // In an operation between two Dimensions,
        // we default to the first Dimension's unit,
        // so `1px + 2` will yield `3px`.
        Dimension.prototype.operate = function (context, op, other) {
            /* jshint noempty:false */
            var value = this._operate(context, op, this.value, other.value);
            var unit = this.unit.clone();
            if (op === '+' || op === '-') {
                if (unit.numerator.length === 0 && unit.denominator.length === 0) {
                    unit = other.unit.clone();
                    if (this.unit.backupUnit) {
                        unit.backupUnit = this.unit.backupUnit;
                    }
                }
                else if (other.unit.numerator.length === 0 && unit.denominator.length === 0)
                    ;
                else {
                    other = other.convertTo(this.unit.usedUnits());
                    if (context.strictUnits && other.unit.toString() !== unit.toString()) {
                        throw new Error("Incompatible units. Change the units or use the unit function. " +
                            ("Bad units: '" + unit.toString() + "' and '" + other.unit.toString() + "'."));
                    }
                    value = this._operate(context, op, this.value, other.value);
                }
            }
            else if (op === '*') {
                unit.numerator = unit.numerator.concat(other.unit.numerator).sort();
                unit.denominator = unit.denominator.concat(other.unit.denominator).sort();
                unit.cancel();
            }
            else if (op === '/') {
                unit.numerator = unit.numerator.concat(other.unit.denominator).sort();
                unit.denominator = unit.denominator.concat(other.unit.numerator).sort();
                unit.cancel();
            }
            return new Dimension(value, unit);
        };
        Dimension.prototype.compare = function (other) {
            var a;
            var b;
            if (!(other instanceof Dimension)) {
                return undefined;
            }
            if (this.unit.isEmpty() || other.unit.isEmpty()) {
                a = this;
                b = other;
            }
            else {
                a = this.unify();
                b = other.unify();
                if (a.unit.compare(b.unit) !== 0) {
                    return undefined;
                }
            }
            return Node.numericCompare(a.value, b.value);
        };
        Dimension.prototype.unify = function () {
            return this.convertTo({ length: 'px', duration: 's', angle: 'rad' });
        };
        Dimension.prototype.convertTo = function (conversions) {
            var value = this.value;
            var unit = this.unit.clone();
            var i;
            var groupName;
            var group;
            var targetUnit;
            var derivedConversions = {};
            var applyUnit;
            if (typeof conversions === 'string') {
                for (i in unitConversions) {
                    if (unitConversions[i].hasOwnProperty(conversions)) {
                        derivedConversions = {};
                        derivedConversions[i] = conversions;
                    }
                }
                conversions = derivedConversions;
            }
            applyUnit = function (atomicUnit, denominator) {
                /* jshint loopfunc:true */
                if (group.hasOwnProperty(atomicUnit)) {
                    if (denominator) {
                        value = value / (group[atomicUnit] / group[targetUnit]);
                    }
                    else {
                        value = value * (group[atomicUnit] / group[targetUnit]);
                    }
                    return targetUnit;
                }
                return atomicUnit;
            };
            for (groupName in conversions) {
                if (conversions.hasOwnProperty(groupName)) {
                    targetUnit = conversions[groupName];
                    group = unitConversions[groupName];
                    unit.map(applyUnit);
                }
            }
            unit.cancel();
            return new Dimension(value, unit);
        };
        return Dimension;
    }(Node));
    Dimension.prototype.type = 'Dimension';
    var MATH$1 = Math$1;
    var Operation = /** @class */ (function (_super) {
        tslib.__extends(Operation, _super);
        function Operation(op, operands, isSpaced) {
            var _this = _super.call(this) || this;
            _this.op = op.trim();
            _this.operands = operands;
            _this.isSpaced = isSpaced;
            return _this;
        }
        Operation.prototype.accept = function (visitor) {
            this.operands = visitor.visitArray(this.operands);
        };
        Operation.prototype.eval = function (context) {
            var a = this.operands[0].eval(context);
            var b = this.operands[1].eval(context);
            var op;
            if (context.isMathOn(this.op)) {
                op = this.op === './' ? '/' : this.op;
                if (a instanceof Dimension && b instanceof Color) {
                    a = a.toColor();
                }
                if (b instanceof Dimension && a instanceof Color) {
                    b = b.toColor();
                }
                if (!a.operate) {
                    if (a instanceof Operation && a.op === '/' && context.math === MATH$1.PARENS_DIVISION) {
                        return new Operation(this.op, [a, b], this.isSpaced);
                    }
                    throw { type: 'Operation',
                        message: 'Operation on an invalid type' };
                }
                return a.operate(context, op, b);
            }
            else {
                return new Operation(this.op, [a, b], this.isSpaced);
            }
        };
        Operation.prototype.genCSS = function (context, output) {
            this.operands[0].genCSS(context, output);
            if (this.isSpaced) {
                output.add(' ');
            }
            output.add(this.op);
            if (this.isSpaced) {
                output.add(' ');
            }
            this.operands[1].genCSS(context, output);
        };
        return Operation;
    }(Node));
    Operation.prototype.type = 'Operation';
    var MATH$2 = Math$1;
    var Expression = /** @class */ (function (_super) {
        tslib.__extends(Expression, _super);
        function Expression(value, noSpacing) {
            var _this = _super.call(this) || this;
            _this.value = value;
            _this.noSpacing = noSpacing;
            if (!value) {
                throw new Error('Expression requires an array parameter');
            }
            return _this;
        }
        Expression.prototype.accept = function (visitor) {
            this.value = visitor.visitArray(this.value);
        };
        Expression.prototype.eval = function (context) {
            var returnValue;
            var mathOn = context.isMathOn();
            var inParenthesis = this.parens &&
                (context.math !== MATH$2.STRICT_LEGACY || !this.parensInOp);
            var doubleParen = false;
            if (inParenthesis) {
                context.inParenthesis();
            }
            if (this.value.length > 1) {
                returnValue = new Expression(this.value.map(function (e) {
                    if (!e.eval) {
                        return e;
                    }
                    return e.eval(context);
                }), this.noSpacing);
            }
            else if (this.value.length === 1) {
                if (this.value[0].parens && !this.value[0].parensInOp && !context.inCalc) {
                    doubleParen = true;
                }
                returnValue = this.value[0].eval(context);
            }
            else {
                returnValue = this;
            }
            if (inParenthesis) {
                context.outOfParenthesis();
            }
            if (this.parens && this.parensInOp && !mathOn && !doubleParen
                && (!(returnValue instanceof Dimension))) {
                returnValue = new Paren(returnValue);
            }
            return returnValue;
        };
        Expression.prototype.genCSS = function (context, output) {
            for (var i_1 = 0; i_1 < this.value.length; i_1++) {
                this.value[i_1].genCSS(context, output);
                if (!this.noSpacing && i_1 + 1 < this.value.length) {
                    output.add(' ');
                }
            }
        };
        Expression.prototype.throwAwayComments = function () {
            this.value = this.value.filter(function (v) { return !(v instanceof Comment); });
        };
        return Expression;
    }(Node));
    Expression.prototype.type = 'Expression';
    var functionCaller = /** @class */ (function () {
        function functionCaller(name, context, index, currentFileInfo) {
            this.name = name.toLowerCase();
            this.index = index;
            this.context = context;
            this.currentFileInfo = currentFileInfo;
            this.func = context.frames[0].functionRegistry.get(this.name);
        }
        functionCaller.prototype.isValid = function () {
            return Boolean(this.func);
        };
        functionCaller.prototype.call = function (args) {
            // This code is terrible and should be replaced as per this issue...
            // https://github.com/less/less.js/issues/2477
            if (Array.isArray(args)) {
                args = args.filter(function (item) {
                    if (item.type === 'Comment') {
                        return false;
                    }
                    return true;
                })
                    .map(function (item) {
                    if (item.type === 'Expression') {
                        var subNodes = item.value.filter(function (item) {
                            if (item.type === 'Comment') {
                                return false;
                            }
                            return true;
                        });
                        if (subNodes.length === 1) {
                            return subNodes[0];
                        }
                        else {
                            return new Expression(subNodes);
                        }
                    }
                    return item;
                });
            }
            return this.func.apply(this, args);
        };
        return functionCaller;
    }());
    //
    // A function call node.
    //
    var Call = /** @class */ (function (_super) {
        tslib.__extends(Call, _super);
        function Call(name, args, index, currentFileInfo) {
            var _this = _super.call(this) || this;
            _this.name = name;
            _this.args = args;
            _this.calc = name === 'calc';
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            return _this;
        }
        Call.prototype.accept = function (visitor) {
            if (this.args) {
                this.args = visitor.visitArray(this.args);
            }
        };
        //
        // When evaluating a function call,
        // we either find the function in the functionRegistry,
        // in which case we call it, passing the  evaluated arguments,
        // if this returns null or we cannot find the function, we
        // simply print it out as it appeared originally [2].
        //
        // The reason why we evaluate the arguments, is in the case where
        // we try to pass a variable to a function, like: `saturate(@color)`.
        // The function should receive the value, not the variable.
        //
        Call.prototype.eval = function (context) {
            /**
             * Turn off math for calc(), and switch back on for evaluating nested functions
             */
            var currentMathContext = context.mathOn;
            context.mathOn = !this.calc;
            if (this.calc || context.inCalc) {
                context.enterCalc();
            }
            var args = this.args.map(function (a) { return a.eval(context); });
            if (this.calc || context.inCalc) {
                context.exitCalc();
            }
            context.mathOn = currentMathContext;
            var result;
            var funcCaller = new functionCaller(this.name, context, this.getIndex(), this.fileInfo());
            if (funcCaller.isValid()) {
                try {
                    result = funcCaller.call(args);
                }
                catch (e) {
                    throw {
                        type: e.type || 'Runtime',
                        message: "error evaluating function `" + this.name + "`" + (e.message ? ": " + e.message : ''),
                        index: this.getIndex(),
                        filename: this.fileInfo().filename,
                        line: e.lineNumber,
                        column: e.columnNumber
                    };
                }
                if (result !== null && result !== undefined) {
                    // Results that that are not nodes are cast as Anonymous nodes
                    // Falsy values or booleans are returned as empty nodes
                    if (!(result instanceof Node)) {
                        if (!result || result === true) {
                            result = new Anonymous(null);
                        }
                        else {
                            result = new Anonymous(result.toString());
                        }
                    }
                    result._index = this._index;
                    result._fileInfo = this._fileInfo;
                    return result;
                }
            }
            return new Call(this.name, args, this.getIndex(), this.fileInfo());
        };
        Call.prototype.genCSS = function (context, output) {
            output.add(this.name + "(", this.fileInfo(), this.getIndex());
            for (var i_1 = 0; i_1 < this.args.length; i_1++) {
                this.args[i_1].genCSS(context, output);
                if (i_1 + 1 < this.args.length) {
                    output.add(', ');
                }
            }
            output.add(')');
        };
        return Call;
    }(Node));
    Call.prototype.type = 'Call';
    var Variable = /** @class */ (function (_super) {
        tslib.__extends(Variable, _super);
        function Variable(name, index, currentFileInfo) {
            var _this = _super.call(this) || this;
            _this.name = name;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            return _this;
        }
        Variable.prototype.eval = function (context) {
            var variable;
            var name = this.name;
            if (name.indexOf('@@') === 0) {
                name = "@" + new Variable(name.slice(1), this.getIndex(), this.fileInfo()).eval(context).value;
            }
            if (this.evaluating) {
                throw { type: 'Name',
                    message: "Recursive variable definition for " + name,
                    filename: this.fileInfo().filename,
                    index: this.getIndex() };
            }
            this.evaluating = true;
            variable = this.find(context.frames, function (frame) {
                var v = frame.variable(name);
                if (v) {
                    if (v.important) {
                        var importantScope = context.importantScope[context.importantScope.length - 1];
                        importantScope.important = v.important;
                    }
                    // If in calc, wrap vars in a function call to cascade evaluate args first
                    if (context.inCalc) {
                        return (new Call('_SELF', [v.value])).eval(context);
                    }
                    else {
                        return v.value.eval(context);
                    }
                }
            });
            if (variable) {
                this.evaluating = false;
                return variable;
            }
            else {
                throw { type: 'Name',
                    message: "variable " + name + " is undefined",
                    filename: this.fileInfo().filename,
                    index: this.getIndex() };
            }
        };
        Variable.prototype.find = function (obj, fun) {
            for (var i_1 = 0, r = void 0; i_1 < obj.length; i_1++) {
                r = fun.call(obj, obj[i_1]);
                if (r) {
                    return r;
                }
            }
            return null;
        };
        return Variable;
    }(Node));
    Variable.prototype.type = 'Variable';
    var Property = /** @class */ (function (_super) {
        tslib.__extends(Property, _super);
        function Property(name, index, currentFileInfo) {
            var _this = _super.call(this) || this;
            _this.name = name;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            return _this;
        }
        Property.prototype.eval = function (context) {
            var property;
            var name = this.name;
            // TODO: shorten this reference
            var mergeRules = context.pluginManager.less.visitors.ToCSSVisitor.prototype._mergeRules;
            if (this.evaluating) {
                throw { type: 'Name',
                    message: "Recursive property reference for " + name,
                    filename: this.fileInfo().filename,
                    index: this.getIndex() };
            }
            this.evaluating = true;
            property = this.find(context.frames, function (frame) {
                var v;
                var vArr = frame.property(name);
                if (vArr) {
                    for (var i_1 = 0; i_1 < vArr.length; i_1++) {
                        v = vArr[i_1];
                        vArr[i_1] = new Declaration(v.name, v.value, v.important, v.merge, v.index, v.currentFileInfo, v.inline, v.variable);
                    }
                    mergeRules(vArr);
                    v = vArr[vArr.length - 1];
                    if (v.important) {
                        var importantScope = context.importantScope[context.importantScope.length - 1];
                        importantScope.important = v.important;
                    }
                    v = v.value.eval(context);
                    return v;
                }
            });
            if (property) {
                this.evaluating = false;
                return property;
            }
            else {
                throw { type: 'Name',
                    message: "Property '" + name + "' is undefined",
                    filename: this.currentFileInfo.filename,
                    index: this.index };
            }
        };
        Property.prototype.find = function (obj, fun) {
            for (var i_2 = 0, r = void 0; i_2 < obj.length; i_2++) {
                r = fun.call(obj, obj[i_2]);
                if (r) {
                    return r;
                }
            }
            return null;
        };
        return Property;
    }(Node));
    Property.prototype.type = 'Property';
    var Attribute = /** @class */ (function (_super) {
        tslib.__extends(Attribute, _super);
        function Attribute(key, op, value) {
            var _this = _super.call(this) || this;
            _this.key = key;
            _this.op = op;
            _this.value = value;
            return _this;
        }
        Attribute.prototype.eval = function (context) {
            return new Attribute(this.key.eval ? this.key.eval(context) : this.key, this.op, (this.value && this.value.eval) ? this.value.eval(context) : this.value);
        };
        Attribute.prototype.genCSS = function (context, output) {
            output.add(this.toCSS(context));
        };
        Attribute.prototype.toCSS = function (context) {
            var value = this.key.toCSS ? this.key.toCSS(context) : this.key;
            if (this.op) {
                value += this.op;
                value += (this.value.toCSS ? this.value.toCSS(context) : this.value);
            }
            return "[" + value + "]";
        };
        return Attribute;
    }(Node));
    Attribute.prototype.type = 'Attribute';
    var Quoted = /** @class */ (function (_super) {
        tslib.__extends(Quoted, _super);
        function Quoted(str, content, escaped, index, currentFileInfo) {
            var _this = _super.call(this) || this;
            _this.escaped = (escaped == null) ? true : escaped;
            _this.value = content || '';
            _this.quote = str.charAt(0);
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.variableRegex = /@\{([\w-]+)\}/g;
            _this.propRegex = /\$\{([\w-]+)\}/g;
            _this.allowRoot = escaped;
            return _this;
        }
        Quoted.prototype.genCSS = function (context, output) {
            if (!this.escaped) {
                output.add(this.quote, this.fileInfo(), this.getIndex());
            }
            output.add(this.value);
            if (!this.escaped) {
                output.add(this.quote);
            }
        };
        Quoted.prototype.containsVariables = function () {
            return this.value.match(this.variableRegex);
        };
        Quoted.prototype.eval = function (context) {
            var that = this;
            var value = this.value;
            var variableReplacement = function (_, name) {
                var v = new Variable("@" + name, that.getIndex(), that.fileInfo()).eval(context, true);
                return (v instanceof Quoted) ? v.value : v.toCSS();
            };
            var propertyReplacement = function (_, name) {
                var v = new Property("$" + name, that.getIndex(), that.fileInfo()).eval(context, true);
                return (v instanceof Quoted) ? v.value : v.toCSS();
            };
            function iterativeReplace(value, regexp, replacementFnc) {
                var evaluatedValue = value;
                do {
                    value = evaluatedValue.toString();
                    evaluatedValue = value.replace(regexp, replacementFnc);
                } while (value !== evaluatedValue);
                return evaluatedValue;
            }
            value = iterativeReplace(value, this.variableRegex, variableReplacement);
            value = iterativeReplace(value, this.propRegex, propertyReplacement);
            return new Quoted(this.quote + value + this.quote, value, this.escaped, this.getIndex(), this.fileInfo());
        };
        Quoted.prototype.compare = function (other) {
            // when comparing quoted strings allow the quote to differ
            if (other.type === 'Quoted' && !this.escaped && !other.escaped) {
                return Node.numericCompare(this.value, other.value);
            }
            else {
                return other.toCSS && this.toCSS() === other.toCSS() ? 0 : undefined;
            }
        };
        return Quoted;
    }(Node));
    Quoted.prototype.type = 'Quoted';
    var URL = /** @class */ (function (_super) {
        tslib.__extends(URL, _super);
        function URL(val, index, currentFileInfo, isEvald) {
            var _this = _super.call(this) || this;
            _this.value = val;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.isEvald = isEvald;
            return _this;
        }
        URL.prototype.accept = function (visitor) {
            this.value = visitor.visit(this.value);
        };
        URL.prototype.genCSS = function (context, output) {
            output.add('url(');
            this.value.genCSS(context, output);
            output.add(')');
        };
        URL.prototype.eval = function (context) {
            var val = this.value.eval(context);
            var rootpath;
            if (!this.isEvald) {
                // Add the rootpath if the URL requires a rewrite
                rootpath = this.fileInfo() && this.fileInfo().rootpath;
                if (typeof rootpath === 'string' &&
                    typeof val.value === 'string' &&
                    context.pathRequiresRewrite(val.value)) {
                    if (!val.quote) {
                        rootpath = escapePath(rootpath);
                    }
                    val.value = context.rewritePath(val.value, rootpath);
                }
                else {
                    val.value = context.normalizePath(val.value);
                }
                // Add url args if enabled
                if (context.urlArgs) {
                    if (!val.value.match(/^\s*data:/)) {
                        var delimiter = val.value.indexOf('?') === -1 ? '?' : '&';
                        var urlArgs = delimiter + context.urlArgs;
                        if (val.value.indexOf('#') !== -1) {
                            val.value = val.value.replace('#', urlArgs + "#");
                        }
                        else {
                            val.value += urlArgs;
                        }
                    }
                }
            }
            return new URL(val, this.getIndex(), this.fileInfo(), true);
        };
        return URL;
    }(Node));
    URL.prototype.type = 'Url';
    function escapePath(path) {
        return path.replace(/[\(\)'"\s]/g, function (match) { return "\\" + match; });
    }
    var Media = /** @class */ (function (_super) {
        tslib.__extends(Media, _super);
        function Media(value, features, index, currentFileInfo, visibilityInfo) {
            var _this = _super.call(this) || this;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            var selectors = (new Selector([], null, null, _this._index, _this._fileInfo)).createEmptySelectors();
            _this.features = new Value(features);
            _this.rules = [new Ruleset(selectors, value)];
            _this.rules[0].allowImports = true;
            _this.copyVisibilityInfo(visibilityInfo);
            _this.allowRoot = true;
            _this.setParent(selectors, _this);
            _this.setParent(_this.features, _this);
            _this.setParent(_this.rules, _this);
            return _this;
        }
        Media.prototype.isRulesetLike = function () {
            return true;
        };
        Media.prototype.accept = function (visitor) {
            if (this.features) {
                this.features = visitor.visit(this.features);
            }
            if (this.rules) {
                this.rules = visitor.visitArray(this.rules);
            }
        };
        Media.prototype.genCSS = function (context, output) {
            output.add('@media ', this._fileInfo, this._index);
            this.features.genCSS(context, output);
            this.outputRuleset(context, output, this.rules);
        };
        Media.prototype.eval = function (context) {
            if (!context.mediaBlocks) {
                context.mediaBlocks = [];
                context.mediaPath = [];
            }
            var media = new Media(null, [], this._index, this._fileInfo, this.visibilityInfo());
            if (this.debugInfo) {
                this.rules[0].debugInfo = this.debugInfo;
                media.debugInfo = this.debugInfo;
            }
            media.features = this.features.eval(context);
            context.mediaPath.push(media);
            context.mediaBlocks.push(media);
            this.rules[0].functionRegistry = context.frames[0].functionRegistry.inherit();
            context.frames.unshift(this.rules[0]);
            media.rules = [this.rules[0].eval(context)];
            context.frames.shift();
            context.mediaPath.pop();
            return context.mediaPath.length === 0 ? media.evalTop(context) :
                media.evalNested(context);
        };
        Media.prototype.evalTop = function (context) {
            var result = this;
            // Render all dependent Media blocks.
            if (context.mediaBlocks.length > 1) {
                var selectors = (new Selector([], null, null, this.getIndex(), this.fileInfo())).createEmptySelectors();
                result = new Ruleset(selectors, context.mediaBlocks);
                result.multiMedia = true;
                result.copyVisibilityInfo(this.visibilityInfo());
                this.setParent(result, this);
            }
            delete context.mediaBlocks;
            delete context.mediaPath;
            return result;
        };
        Media.prototype.evalNested = function (context) {
            var i;
            var value;
            var path = context.mediaPath.concat([this]);
            // Extract the media-query conditions separated with `,` (OR).
            for (i = 0; i < path.length; i++) {
                value = path[i].features instanceof Value ?
                    path[i].features.value : path[i].features;
                path[i] = Array.isArray(value) ? value : [value];
            }
            // Trace all permutations to generate the resulting media-query.
            //
            // (a, b and c) with nested (d, e) ->
            //    a and d
            //    a and e
            //    b and c and d
            //    b and c and e
            this.features = new Value(this.permute(path).map(function (path) {
                path = path.map(function (fragment) { return fragment.toCSS ? fragment : new Anonymous(fragment); });
                for (i = path.length - 1; i > 0; i--) {
                    path.splice(i, 0, new Anonymous('and'));
                }
                return new Expression(path);
            }));
            this.setParent(this.features, this);
            // Fake a tree-node that doesn't output anything.
            return new Ruleset([], []);
        };
        Media.prototype.permute = function (arr) {
            if (arr.length === 0) {
                return [];
            }
            else if (arr.length === 1) {
                return arr[0];
            }
            else {
                var result = [];
                var rest = this.permute(arr.slice(1));
                for (var i_1 = 0; i_1 < rest.length; i_1++) {
                    for (var j = 0; j < arr[0].length; j++) {
                        result.push([arr[0][j]].concat(rest[i_1]));
                    }
                }
                return result;
            }
        };
        Media.prototype.bubbleSelectors = function (selectors) {
            if (!selectors) {
                return;
            }
            this.rules = [new Ruleset(copyArray(selectors), [this.rules[0]])];
            this.setParent(this.rules, this);
        };
        return Media;
    }(AtRule));
    Media.prototype.type = 'Media';
    //
    // CSS @import node
    //
    // The general strategy here is that we don't want to wait
    // for the parsing to be completed, before we start importing
    // the file. That's because in the context of a browser,
    // most of the time will be spent waiting for the server to respond.
    //
    // On creation, we push the import path to our import queue, though
    // `import,push`, we also pass it a callback, which it'll call once
    // the file has been fetched, and parsed.
    //
    var Import = /** @class */ (function (_super) {
        tslib.__extends(Import, _super);
        function Import(path, features, options, index, currentFileInfo, visibilityInfo) {
            var _this = _super.call(this) || this;
            _this.options = options;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.path = path;
            _this.features = features;
            _this.allowRoot = true;
            if (_this.options.less !== undefined || _this.options.inline) {
                _this.css = !_this.options.less || _this.options.inline;
            }
            else {
                var pathValue = _this.getPath();
                if (pathValue && /[#\.\&\?]css([\?;].*)?$/.test(pathValue)) {
                    _this.css = true;
                }
            }
            _this.copyVisibilityInfo(visibilityInfo);
            _this.setParent(_this.features, _this);
            _this.setParent(_this.path, _this);
            return _this;
        }
        Import.prototype.accept = function (visitor) {
            if (this.features) {
                this.features = visitor.visit(this.features);
            }
            this.path = visitor.visit(this.path);
            if (!this.options.isPlugin && !this.options.inline && this.root) {
                this.root = visitor.visit(this.root);
            }
        };
        Import.prototype.genCSS = function (context, output) {
            if (this.css && this.path._fileInfo.reference === undefined) {
                output.add('@import ', this._fileInfo, this._index);
                this.path.genCSS(context, output);
                if (this.features) {
                    output.add(' ');
                    this.features.genCSS(context, output);
                }
                output.add(';');
            }
        };
        Import.prototype.getPath = function () {
            return (this.path instanceof URL) ?
                this.path.value.value : this.path.value;
        };
        Import.prototype.isVariableImport = function () {
            var path = this.path;
            if (path instanceof URL) {
                path = path.value;
            }
            if (path instanceof Quoted) {
                return path.containsVariables();
            }
            return true;
        };
        Import.prototype.evalForImport = function (context) {
            var path = this.path;
            if (path instanceof URL) {
                path = path.value;
            }
            return new Import(path.eval(context), this.features, this.options, this._index, this._fileInfo, this.visibilityInfo());
        };
        Import.prototype.evalPath = function (context) {
            var path = this.path.eval(context);
            var fileInfo = this._fileInfo;
            if (!(path instanceof URL)) {
                // Add the rootpath if the URL requires a rewrite
                var pathValue = path.value;
                if (fileInfo &&
                    pathValue &&
                    context.pathRequiresRewrite(pathValue)) {
                    path.value = context.rewritePath(pathValue, fileInfo.rootpath);
                }
                else {
                    path.value = context.normalizePath(path.value);
                }
            }
            return path;
        };
        Import.prototype.eval = function (context) {
            var result = this.doEval(context);
            if (this.options.reference || this.blocksVisibility()) {
                if (result.length || result.length === 0) {
                    result.forEach(function (node) {
                        node.addVisibilityBlock();
                    });
                }
                else {
                    result.addVisibilityBlock();
                }
            }
            return result;
        };
        Import.prototype.doEval = function (context) {
            var ruleset;
            var registry;
            var features = this.features && this.features.eval(context);
            if (this.options.isPlugin) {
                if (this.root && this.root.eval) {
                    try {
                        this.root.eval(context);
                    }
                    catch (e) {
                        e.message = 'Plugin error during evaluation';
                        throw new LessError(e, this.root.imports, this.root.filename);
                    }
                }
                registry = context.frames[0] && context.frames[0].functionRegistry;
                if (registry && this.root && this.root.functions) {
                    registry.addMultiple(this.root.functions);
                }
                return [];
            }
            if (this.skip) {
                if (typeof this.skip === 'function') {
                    this.skip = this.skip();
                }
                if (this.skip) {
                    return [];
                }
            }
            if (this.options.inline) {
                var contents = new Anonymous(this.root, 0, {
                    filename: this.importedFilename,
                    reference: this.path._fileInfo && this.path._fileInfo.reference
                }, true, true);
                return this.features ? new Media([contents], this.features.value) : [contents];
            }
            else if (this.css) {
                var newImport = new Import(this.evalPath(context), features, this.options, this._index);
                if (!newImport.css && this.error) {
                    throw this.error;
                }
                return newImport;
            }
            else {
                ruleset = new Ruleset(null, copyArray(this.root.rules));
                ruleset.evalImports(context);
                return this.features ? new Media(ruleset.rules, this.features.value) : ruleset.rules;
            }
        };
        return Import;
    }(Node));
    Import.prototype.type = 'Import';
    var JsEvalNode = /** @class */ (function (_super) {
        tslib.__extends(JsEvalNode, _super);
        function JsEvalNode() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        JsEvalNode.prototype.evaluateJavaScript = function (expression, context) {
            var result;
            var that = this;
            var evalContext = {};
            if (!context.javascriptEnabled) {
                throw { message: 'Inline JavaScript is not enabled. Is it set in your options?',
                    filename: this.fileInfo().filename,
                    index: this.getIndex() };
            }
            expression = expression.replace(/@\{([\w-]+)\}/g, function (_, name) { return that.jsify(new Variable("@" + name, that.getIndex(), that.fileInfo()).eval(context)); });
            try {
                expression = new Function("return (" + expression + ")");
            }
            catch (e) {
                throw { message: "JavaScript evaluation error: " + e.message + " from `" + expression + "`",
                    filename: this.fileInfo().filename,
                    index: this.getIndex() };
            }
            var variables = context.frames[0].variables();
            for (var k in variables) {
                if (variables.hasOwnProperty(k)) {
                    /* jshint loopfunc:true */
                    evalContext[k.slice(1)] = {
                        value: variables[k].value,
                        toJS: function () {
                            return this.value.eval(context).toCSS();
                        }
                    };
                }
            }
            try {
                result = expression.call(evalContext);
            }
            catch (e) {
                throw { message: "JavaScript evaluation error: '" + e.name + ": " + e.message.replace(/["]/g, '\'') + "'",
                    filename: this.fileInfo().filename,
                    index: this.getIndex() };
            }
            return result;
        };
        JsEvalNode.prototype.jsify = function (obj) {
            if (Array.isArray(obj.value) && (obj.value.length > 1)) {
                return "[" + obj.value.map(function (v) { return v.toCSS(); }).join(', ') + "]";
            }
            else {
                return obj.toCSS();
            }
        };
        return JsEvalNode;
    }(Node));
    var JavaScript = /** @class */ (function (_super) {
        tslib.__extends(JavaScript, _super);
        function JavaScript(string, escaped, index, currentFileInfo) {
            var _this = _super.call(this) || this;
            _this.escaped = escaped;
            _this.expression = string;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            return _this;
        }
        JavaScript.prototype.eval = function (context) {
            var result = this.evaluateJavaScript(this.expression, context);
            var type = typeof result;
            if (type === 'number' && !isNaN(result)) {
                return new Dimension(result);
            }
            else if (type === 'string') {
                return new Quoted("\"" + result + "\"", result, this.escaped, this._index);
            }
            else if (Array.isArray(result)) {
                return new Anonymous(result.join(', '));
            }
            else {
                return new Anonymous(result);
            }
        };
        return JavaScript;
    }(JsEvalNode));
    JavaScript.prototype.type = 'JavaScript';
    var Assignment = /** @class */ (function (_super) {
        tslib.__extends(Assignment, _super);
        function Assignment(key, val) {
            var _this = _super.call(this) || this;
            _this.key = key;
            _this.value = val;
            return _this;
        }
        Assignment.prototype.accept = function (visitor) {
            this.value = visitor.visit(this.value);
        };
        Assignment.prototype.eval = function (context) {
            if (this.value.eval) {
                return new Assignment(this.key, this.value.eval(context));
            }
            return this;
        };
        Assignment.prototype.genCSS = function (context, output) {
            output.add(this.key + "=");
            if (this.value.genCSS) {
                this.value.genCSS(context, output);
            }
            else {
                output.add(this.value);
            }
        };
        return Assignment;
    }(Node));
    Assignment.prototype.type = 'Assignment';
    var Condition = /** @class */ (function (_super) {
        tslib.__extends(Condition, _super);
        function Condition(op, l, r, i, negate) {
            var _this = _super.call(this) || this;
            _this.op = op.trim();
            _this.lvalue = l;
            _this.rvalue = r;
            _this._index = i;
            _this.negate = negate;
            return _this;
        }
        Condition.prototype.accept = function (visitor) {
            this.lvalue = visitor.visit(this.lvalue);
            this.rvalue = visitor.visit(this.rvalue);
        };
        Condition.prototype.eval = function (context) {
            var result = (function (op, a, b) {
                switch (op) {
                    case 'and': return a && b;
                    case 'or': return a || b;
                    default:
                        switch (Node.compare(a, b)) {
                            case -1:
                                return op === '<' || op === '=<' || op === '<=';
                            case 0:
                                return op === '=' || op === '>=' || op === '=<' || op === '<=';
                            case 1:
                                return op === '>' || op === '>=';
                            default:
                                return false;
                        }
                }
            })(this.op, this.lvalue.eval(context), this.rvalue.eval(context));
            return this.negate ? !result : result;
        };
        return Condition;
    }(Node));
    Condition.prototype.type = 'Condition';
    var UnicodeDescriptor = /** @class */ (function (_super) {
        tslib.__extends(UnicodeDescriptor, _super);
        function UnicodeDescriptor(value) {
            var _this = _super.call(this) || this;
            _this.value = value;
            return _this;
        }
        return UnicodeDescriptor;
    }(Node));
    UnicodeDescriptor.prototype.type = 'UnicodeDescriptor';
    var Negative = /** @class */ (function (_super) {
        tslib.__extends(Negative, _super);
        function Negative(node) {
            var _this = _super.call(this) || this;
            _this.value = node;
            return _this;
        }
        Negative.prototype.genCSS = function (context, output) {
            output.add('-');
            this.value.genCSS(context, output);
        };
        Negative.prototype.eval = function (context) {
            if (context.isMathOn()) {
                return (new Operation('*', [new Dimension(-1), this.value])).eval(context);
            }
            return new Negative(this.value.eval(context));
        };
        return Negative;
    }(Node));
    Negative.prototype.type = 'Negative';
    var Extend = /** @class */ (function (_super) {
        tslib.__extends(Extend, _super);
        function Extend(selector, option, index, currentFileInfo, visibilityInfo) {
            var _this = _super.call(this) || this;
            _this.selector = selector;
            _this.option = option;
            _this.object_id = Extend.next_id++;
            _this.parent_ids = [_this.object_id];
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.copyVisibilityInfo(visibilityInfo);
            _this.allowRoot = true;
            switch (option) {
                case 'all':
                    _this.allowBefore = true;
                    _this.allowAfter = true;
                    break;
                default:
                    _this.allowBefore = false;
                    _this.allowAfter = false;
                    break;
            }
            _this.setParent(_this.selector, _this);
            return _this;
        }
        Extend.prototype.accept = function (visitor) {
            this.selector = visitor.visit(this.selector);
        };
        Extend.prototype.eval = function (context) {
            return new Extend(this.selector.eval(context), this.option, this.getIndex(), this.fileInfo(), this.visibilityInfo());
        };
        Extend.prototype.clone = function (context) {
            return new Extend(this.selector, this.option, this.getIndex(), this.fileInfo(), this.visibilityInfo());
        };
        // it concatenates (joins) all selectors in selector array
        Extend.prototype.findSelfSelectors = function (selectors) {
            var selfElements = [];
            var i;
            var selectorElements;
            for (i = 0; i < selectors.length; i++) {
                selectorElements = selectors[i].elements;
                // duplicate the logic in genCSS function inside the selector node.
                // future TODO - move both logics into the selector joiner visitor
                if (i > 0 && selectorElements.length && selectorElements[0].combinator.value === '') {
                    selectorElements[0].combinator.value = ' ';
                }
                selfElements = selfElements.concat(selectors[i].elements);
            }
            this.selfSelectors = [new Selector(selfElements)];
            this.selfSelectors[0].copyVisibilityInfo(this.visibilityInfo());
        };
        return Extend;
    }(Node));
    Extend.next_id = 0;
    Extend.prototype.type = 'Extend';
    var VariableCall = /** @class */ (function (_super) {
        tslib.__extends(VariableCall, _super);
        function VariableCall(variable, index, currentFileInfo) {
            var _this = _super.call(this) || this;
            _this.variable = variable;
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.allowRoot = true;
            return _this;
        }
        VariableCall.prototype.eval = function (context) {
            var rules;
            var detachedRuleset = new Variable(this.variable, this.getIndex(), this.fileInfo()).eval(context);
            var error = new LessError({ message: "Could not evaluate variable call " + this.variable });
            if (!detachedRuleset.ruleset) {
                if (detachedRuleset.rules) {
                    rules = detachedRuleset;
                }
                else if (Array.isArray(detachedRuleset)) {
                    rules = new Ruleset('', detachedRuleset);
                }
                else if (Array.isArray(detachedRuleset.value)) {
                    rules = new Ruleset('', detachedRuleset.value);
                }
                else {
                    throw error;
                }
                detachedRuleset = new DetachedRuleset(rules);
            }
            if (detachedRuleset.ruleset) {
                return detachedRuleset.callEval(context);
            }
            throw error;
        };
        return VariableCall;
    }(Node));
    VariableCall.prototype.type = 'VariableCall';
    var NamespaceValue = /** @class */ (function (_super) {
        tslib.__extends(NamespaceValue, _super);
        function NamespaceValue(ruleCall, lookups, index, fileInfo) {
            var _this = _super.call(this) || this;
            _this.value = ruleCall;
            _this.lookups = lookups;
            _this._index = index;
            _this._fileInfo = fileInfo;
            return _this;
        }
        NamespaceValue.prototype.eval = function (context) {
            var i;
            var name;
            var rules = this.value.eval(context);
            for (i = 0; i < this.lookups.length; i++) {
                name = this.lookups[i];
                /**
                 * Eval'd DRs return rulesets.
                 * Eval'd mixins return rules, so let's make a ruleset if we need it.
                 * We need to do this because of late parsing of values
                 */
                if (Array.isArray(rules)) {
                    rules = new Ruleset([new Selector()], rules);
                }
                if (name === '') {
                    rules = rules.lastDeclaration();
                }
                else if (name.charAt(0) === '@') {
                    if (name.charAt(1) === '@') {
                        name = "@" + new Variable(name.substr(1)).eval(context).value;
                    }
                    if (rules.variables) {
                        rules = rules.variable(name);
                    }
                    if (!rules) {
                        throw { type: 'Name',
                            message: "variable " + name + " not found",
                            filename: this.fileInfo().filename,
                            index: this.getIndex() };
                    }
                }
                else {
                    if (name.substring(0, 2) === '$@') {
                        name = "$" + new Variable(name.substr(1)).eval(context).value;
                    }
                    else {
                        name = name.charAt(0) === '$' ? name : "$" + name;
                    }
                    if (rules.properties) {
                        rules = rules.property(name);
                    }
                    if (!rules) {
                        throw { type: 'Name',
                            message: "property \"" + name.substr(1) + "\" not found",
                            filename: this.fileInfo().filename,
                            index: this.getIndex() };
                    }
                    // Properties are an array of values, since a ruleset can have multiple props.
                    // We pick the last one (the "cascaded" value)
                    rules = rules[rules.length - 1];
                }
                if (rules.value) {
                    rules = rules.eval(context).value;
                }
                if (rules.ruleset) {
                    rules = rules.ruleset.eval(context);
                }
            }
            return rules;
        };
        return NamespaceValue;
    }(Node));
    NamespaceValue.prototype.type = 'NamespaceValue';
    var Definition = /** @class */ (function (_super) {
        tslib.__extends(Definition, _super);
        function Definition(name, params, rules, condition, variadic, frames, visibilityInfo) {
            var _this = _super.call(this) || this;
            _this.name = name || 'anonymous mixin';
            _this.selectors = [new Selector([new Element(null, name, false, _this._index, _this._fileInfo)])];
            _this.params = params;
            _this.condition = condition;
            _this.variadic = variadic;
            _this.arity = params.length;
            _this.rules = rules;
            _this._lookups = {};
            var optionalParameters = [];
            _this.required = params.reduce(function (count, p) {
                if (!p.name || (p.name && !p.value)) {
                    return count + 1;
                }
                else {
                    optionalParameters.push(p.name);
                    return count;
                }
            }, 0);
            _this.optionalParameters = optionalParameters;
            _this.frames = frames;
            _this.copyVisibilityInfo(visibilityInfo);
            _this.allowRoot = true;
            return _this;
        }
        Definition.prototype.accept = function (visitor) {
            if (this.params && this.params.length) {
                this.params = visitor.visitArray(this.params);
            }
            this.rules = visitor.visitArray(this.rules);
            if (this.condition) {
                this.condition = visitor.visit(this.condition);
            }
        };
        Definition.prototype.evalParams = function (context, mixinEnv, args, evaldArguments) {
            /* jshint boss:true */
            var frame = new Ruleset(null, null);
            var varargs;
            var arg;
            var params = copyArray(this.params);
            var i;
            var j;
            var val;
            var name;
            var isNamedFound;
            var argIndex;
            var argsLength = 0;
            if (mixinEnv.frames && mixinEnv.frames[0] && mixinEnv.frames[0].functionRegistry) {
                frame.functionRegistry = mixinEnv.frames[0].functionRegistry.inherit();
            }
            mixinEnv = new contexts.Eval(mixinEnv, [frame].concat(mixinEnv.frames));
            if (args) {
                args = copyArray(args);
                argsLength = args.length;
                for (i = 0; i < argsLength; i++) {
                    arg = args[i];
                    if (name = (arg && arg.name)) {
                        isNamedFound = false;
                        for (j = 0; j < params.length; j++) {
                            if (!evaldArguments[j] && name === params[j].name) {
                                evaldArguments[j] = arg.value.eval(context);
                                frame.prependRule(new Declaration(name, arg.value.eval(context)));
                                isNamedFound = true;
                                break;
                            }
                        }
                        if (isNamedFound) {
                            args.splice(i, 1);
                            i--;
                            continue;
                        }
                        else {
                            throw { type: 'Runtime', message: "Named argument for " + this.name + " " + args[i].name + " not found" };
                        }
                    }
                }
            }
            argIndex = 0;
            for (i = 0; i < params.length; i++) {
                if (evaldArguments[i]) {
                    continue;
                }
                arg = args && args[argIndex];
                if (name = params[i].name) {
                    if (params[i].variadic) {
                        varargs = [];
                        for (j = argIndex; j < argsLength; j++) {
                            varargs.push(args[j].value.eval(context));
                        }
                        frame.prependRule(new Declaration(name, new Expression(varargs).eval(context)));
                    }
                    else {
                        val = arg && arg.value;
                        if (val) {
                            // This was a mixin call, pass in a detached ruleset of it's eval'd rules
                            if (Array.isArray(val)) {
                                val = new DetachedRuleset(new Ruleset('', val));
                            }
                            else {
                                val = val.eval(context);
                            }
                        }
                        else if (params[i].value) {
                            val = params[i].value.eval(mixinEnv);
                            frame.resetCache();
                        }
                        else {
                            throw { type: 'Runtime', message: "wrong number of arguments for " + this.name + " (" + argsLength + " for " + this.arity + ")" };
                        }
                        frame.prependRule(new Declaration(name, val));
                        evaldArguments[i] = val;
                    }
                }
                if (params[i].variadic && args) {
                    for (j = argIndex; j < argsLength; j++) {
                        evaldArguments[j] = args[j].value.eval(context);
                    }
                }
                argIndex++;
            }
            return frame;
        };
        Definition.prototype.makeImportant = function () {
            var rules = !this.rules ? this.rules : this.rules.map(function (r) {
                if (r.makeImportant) {
                    return r.makeImportant(true);
                }
                else {
                    return r;
                }
            });
            var result = new Definition(this.name, this.params, rules, this.condition, this.variadic, this.frames);
            return result;
        };
        Definition.prototype.eval = function (context) {
            return new Definition(this.name, this.params, this.rules, this.condition, this.variadic, this.frames || copyArray(context.frames));
        };
        Definition.prototype.evalCall = function (context, args, important) {
            var _arguments = [];
            var mixinFrames = this.frames ? this.frames.concat(context.frames) : context.frames;
            var frame = this.evalParams(context, new contexts.Eval(context, mixinFrames), args, _arguments);
            var rules;
            var ruleset;
            frame.prependRule(new Declaration('@arguments', new Expression(_arguments).eval(context)));
            rules = copyArray(this.rules);
            ruleset = new Ruleset(null, rules);
            ruleset.originalRuleset = this;
            ruleset = ruleset.eval(new contexts.Eval(context, [this, frame].concat(mixinFrames)));
            if (important) {
                ruleset = ruleset.makeImportant();
            }
            return ruleset;
        };
        Definition.prototype.matchCondition = function (args, context) {
            if (this.condition && !this.condition.eval(new contexts.Eval(context, [this.evalParams(context, /* the parameter variables */ new contexts.Eval(context, this.frames ? this.frames.concat(context.frames) : context.frames), args, [])]
                .concat(this.frames || []) // the parent namespace/mixin frames
                .concat(context.frames)))) { // the current environment frames
                return false;
            }
            return true;
        };
        Definition.prototype.matchArgs = function (args, context) {
            var allArgsCnt = (args && args.length) || 0;
            var len;
            var optionalParameters = this.optionalParameters;
            var requiredArgsCnt = !args ? 0 : args.reduce(function (count, p) {
                if (optionalParameters.indexOf(p.name) < 0) {
                    return count + 1;
                }
                else {
                    return count;
                }
            }, 0);
            if (!this.variadic) {
                if (requiredArgsCnt < this.required) {
                    return false;
                }
                if (allArgsCnt > this.params.length) {
                    return false;
                }
            }
            else {
                if (requiredArgsCnt < (this.required - 1)) {
                    return false;
                }
            }
            // check patterns
            len = Math.min(requiredArgsCnt, this.arity);
            for (var i_1 = 0; i_1 < len; i_1++) {
                if (!this.params[i_1].name && !this.params[i_1].variadic) {
                    if (args[i_1].value.eval(context).toCSS() != this.params[i_1].value.eval(context).toCSS()) {
                        return false;
                    }
                }
            }
            return true;
        };
        return Definition;
    }(Ruleset));
    Definition.prototype.type = 'MixinDefinition';
    Definition.prototype.evalFirst = true;
    var MixinCall = /** @class */ (function (_super) {
        tslib.__extends(MixinCall, _super);
        function MixinCall(elements, args, index, currentFileInfo, important) {
            var _this = _super.call(this) || this;
            _this.selector = new Selector(elements);
            _this.arguments = args || [];
            _this._index = index;
            _this._fileInfo = currentFileInfo;
            _this.important = important;
            _this.allowRoot = true;
            _this.setParent(_this.selector, _this);
            return _this;
        }
        MixinCall.prototype.accept = function (visitor) {
            if (this.selector) {
                this.selector = visitor.visit(this.selector);
            }
            if (this.arguments.length) {
                this.arguments = visitor.visitArray(this.arguments);
            }
        };
        MixinCall.prototype.eval = function (context) {
            var mixins;
            var mixin;
            var mixinPath;
            var args = [];
            var arg;
            var argValue;
            var rules = [];
            var match = false;
            var i;
            var m;
            var f;
            var isRecursive;
            var isOneFound;
            var candidates = [];
            var candidate;
            var conditionResult = [];
            var defaultResult;
            var defFalseEitherCase = -1;
            var defNone = 0;
            var defTrue = 1;
            var defFalse = 2;
            var count;
            var originalRuleset;
            var noArgumentsFilter;
            this.selector = this.selector.eval(context);
            function calcDefGroup(mixin, mixinPath) {
                var f;
                var p;
                var namespace;
                for (f = 0; f < 2; f++) {
                    conditionResult[f] = true;
                    defaultFunc.value(f);
                    for (p = 0; p < mixinPath.length && conditionResult[f]; p++) {
                        namespace = mixinPath[p];
                        if (namespace.matchCondition) {
                            conditionResult[f] = conditionResult[f] && namespace.matchCondition(null, context);
                        }
                    }
                    if (mixin.matchCondition) {
                        conditionResult[f] = conditionResult[f] && mixin.matchCondition(args, context);
                    }
                }
                if (conditionResult[0] || conditionResult[1]) {
                    if (conditionResult[0] != conditionResult[1]) {
                        return conditionResult[1] ?
                            defTrue : defFalse;
                    }
                    return defNone;
                }
                return defFalseEitherCase;
            }
            for (i = 0; i < this.arguments.length; i++) {
                arg = this.arguments[i];
                argValue = arg.value.eval(context);
                if (arg.expand && Array.isArray(argValue.value)) {
                    argValue = argValue.value;
                    for (m = 0; m < argValue.length; m++) {
                        args.push({ value: argValue[m] });
                    }
                }
                else {
                    args.push({ name: arg.name, value: argValue });
                }
            }
            noArgumentsFilter = function (rule) { return rule.matchArgs(null, context); };
            for (i = 0; i < context.frames.length; i++) {
                if ((mixins = context.frames[i].find(this.selector, null, noArgumentsFilter)).length > 0) {
                    isOneFound = true;
                    // To make `default()` function independent of definition order we have two "subpasses" here.
                    // At first we evaluate each guard *twice* (with `default() == true` and `default() == false`),
                    // and build candidate list with corresponding flags. Then, when we know all possible matches,
                    // we make a final decision.
                    for (m = 0; m < mixins.length; m++) {
                        mixin = mixins[m].rule;
                        mixinPath = mixins[m].path;
                        isRecursive = false;
                        for (f = 0; f < context.frames.length; f++) {
                            if ((!(mixin instanceof Definition)) && mixin === (context.frames[f].originalRuleset || context.frames[f])) {
                                isRecursive = true;
                                break;
                            }
                        }
                        if (isRecursive) {
                            continue;
                        }
                        if (mixin.matchArgs(args, context)) {
                            candidate = { mixin: mixin, group: calcDefGroup(mixin, mixinPath) };
                            if (candidate.group !== defFalseEitherCase) {
                                candidates.push(candidate);
                            }
                            match = true;
                        }
                    }
                    defaultFunc.reset();
                    count = [0, 0, 0];
                    for (m = 0; m < candidates.length; m++) {
                        count[candidates[m].group]++;
                    }
                    if (count[defNone] > 0) {
                        defaultResult = defFalse;
                    }
                    else {
                        defaultResult = defTrue;
                        if ((count[defTrue] + count[defFalse]) > 1) {
                            throw { type: 'Runtime',
                                message: "Ambiguous use of `default()` found when matching for `" + this.format(args) + "`",
                                index: this.getIndex(), filename: this.fileInfo().filename };
                        }
                    }
                    for (m = 0; m < candidates.length; m++) {
                        candidate = candidates[m].group;
                        if ((candidate === defNone) || (candidate === defaultResult)) {
                            try {
                                mixin = candidates[m].mixin;
                                if (!(mixin instanceof Definition)) {
                                    originalRuleset = mixin.originalRuleset || mixin;
                                    mixin = new Definition('', [], mixin.rules, null, false, null, originalRuleset.visibilityInfo());
                                    mixin.originalRuleset = originalRuleset;
                                }
                                var newRules = mixin.evalCall(context, args, this.important).rules;
                                this._setVisibilityToReplacement(newRules);
                                Array.prototype.push.apply(rules, newRules);
                            }
                            catch (e) {
                                throw { message: e.message, index: this.getIndex(), filename: this.fileInfo().filename, stack: e.stack };
                            }
                        }
                    }
                    if (match) {
                        return rules;
                    }
                }
            }
            if (isOneFound) {
                throw { type: 'Runtime',
                    message: "No matching definition was found for `" + this.format(args) + "`",
                    index: this.getIndex(), filename: this.fileInfo().filename };
            }
            else {
                throw { type: 'Name',
                    message: this.selector.toCSS().trim() + " is undefined",
                    index: this.getIndex(), filename: this.fileInfo().filename };
            }
        };
        MixinCall.prototype._setVisibilityToReplacement = function (replacement) {
            var i;
            var rule;
            if (this.blocksVisibility()) {
                for (i = 0; i < replacement.length; i++) {
                    rule = replacement[i];
                    rule.addVisibilityBlock();
                }
            }
        };
        MixinCall.prototype.format = function (args) {
            return this.selector.toCSS().trim() + "(" + (args ? args.map(function (a) {
                var argValue = '';
                if (a.name) {
                    argValue += a.name + ":";
                }
                if (a.value.toCSS) {
                    argValue += a.value.toCSS();
                }
                else {
                    argValue += '???';
                }
                return argValue;
            }).join(', ') : '') + ")";
        };
        return MixinCall;
    }(Node));
    MixinCall.prototype.type = 'MixinCall';
    var tree = {
        Node: Node, Color: Color, AtRule: AtRule, DetachedRuleset: DetachedRuleset, Operation: Operation,
        Dimension: Dimension, Unit: Unit, Keyword: Keyword, Variable: Variable, Property: Property,
        Ruleset: Ruleset, Element: Element, Attribute: Attribute, Combinator: Combinator, Selector: Selector,
        Quoted: Quoted, Expression: Expression, Declaration: Declaration, Call: Call, URL: URL, Import: Import,
        Comment: Comment, Anonymous: Anonymous, Value: Value, JavaScript: JavaScript, Assignment: Assignment,
        Condition: Condition, Paren: Paren, Media: Media, UnicodeDescriptor: UnicodeDescriptor, Negative: Negative,
        Extend: Extend, VariableCall: VariableCall, NamespaceValue: NamespaceValue,
        mixin: {
            Call: MixinCall,
            Definition: Definition
        }
    };
    /**
     * @todo Document why this abstraction exists, and the relationship between
     *       environment, file managers, and plugin manager
     */
    var environment$1 = /** @class */ (function () {
        function environment(externalEnvironment, fileManagers) {
            this.fileManagers = fileManagers || [];
            externalEnvironment = externalEnvironment || {};
            var optionalFunctions = ['encodeBase64', 'mimeLookup', 'charsetLookup', 'getSourceMapGenerator'];
            var requiredFunctions = [];
            var functions = requiredFunctions.concat(optionalFunctions);
            for (var i_1 = 0; i_1 < functions.length; i_1++) {
                var propName = functions[i_1];
                var environmentFunc = externalEnvironment[propName];
                if (environmentFunc) {
                    this[propName] = environmentFunc.bind(externalEnvironment);
                }
                else if (i_1 < requiredFunctions.length) {
                    this.warn("missing required function in environment - " + propName);
                }
            }
        }
        environment.prototype.getFileManager = function (filename, currentDirectory, options, environment, isSync) {
            if (!filename) {
                logger.warn('getFileManager called with no filename.. Please report this issue. continuing.');
            }
            if (currentDirectory == null) {
                logger.warn('getFileManager called with null directory.. Please report this issue. continuing.');
            }
            var fileManagers = this.fileManagers;
            if (options.pluginManager) {
                fileManagers = [].concat(fileManagers).concat(options.pluginManager.getFileManagers());
            }
            for (var i_2 = fileManagers.length - 1; i_2 >= 0; i_2--) {
                var fileManager = fileManagers[i_2];
                if (fileManager[isSync ? 'supportsSync' : 'supports'](filename, currentDirectory, options, environment)) {
                    return fileManager;
                }
            }
            return null;
        };
        environment.prototype.addFileManager = function (fileManager) {
            this.fileManagers.push(fileManager);
        };
        environment.prototype.clearFileManagers = function () {
            this.fileManagers = [];
        };
        return environment;
    }());
    var AbstractPluginLoader = /** @class */ (function () {
        function AbstractPluginLoader() {
            // Implemented by Node.js plugin loader
            this.require = function () { return null; };
        }
        AbstractPluginLoader.prototype.evalPlugin = function (contents, context, imports, pluginOptions, fileInfo) {
            var loader;
            var registry;
            var pluginObj;
            var localModule;
            var pluginManager;
            var filename;
            var result;
            pluginManager = context.pluginManager;
            if (fileInfo) {
                if (typeof fileInfo === 'string') {
                    filename = fileInfo;
                }
                else {
                    filename = fileInfo.filename;
                }
            }
            var shortname = (new this.less.FileManager()).extractUrlParts(filename).filename;
            if (filename) {
                pluginObj = pluginManager.get(filename);
                if (pluginObj) {
                    result = this.trySetOptions(pluginObj, filename, shortname, pluginOptions);
                    if (result) {
                        return result;
                    }
                    try {
                        if (pluginObj.use) {
                            pluginObj.use.call(this.context, pluginObj);
                        }
                    }
                    catch (e) {
                        e.message = e.message || 'Error during @plugin call';
                        return new LessError(e, imports, filename);
                    }
                    return pluginObj;
                }
            }
            localModule = {
                exports: {},
                pluginManager: pluginManager,
                fileInfo: fileInfo
            };
            registry = functionRegistry.create();
            var registerPlugin = function (obj) {
                pluginObj = obj;
            };
            try {
                loader = new Function('module', 'require', 'registerPlugin', 'functions', 'tree', 'less', 'fileInfo', contents);
                loader(localModule, this.require(filename), registerPlugin, registry, this.less.tree, this.less, fileInfo);
            }
            catch (e) {
                return new LessError(e, imports, filename);
            }
            if (!pluginObj) {
                pluginObj = localModule.exports;
            }
            pluginObj = this.validatePlugin(pluginObj, filename, shortname);
            if (pluginObj instanceof LessError) {
                return pluginObj;
            }
            if (pluginObj) {
                pluginObj.imports = imports;
                pluginObj.filename = filename;
                // For < 3.x (or unspecified minVersion) - setOptions() before install()
                if (!pluginObj.minVersion || this.compareVersion('3.0.0', pluginObj.minVersion) < 0) {
                    result = this.trySetOptions(pluginObj, filename, shortname, pluginOptions);
                    if (result) {
                        return result;
                    }
                }
                // Run on first load
                pluginManager.addPlugin(pluginObj, fileInfo.filename, registry);
                pluginObj.functions = registry.getLocalFunctions();
                // Need to call setOptions again because the pluginObj might have functions
                result = this.trySetOptions(pluginObj, filename, shortname, pluginOptions);
                if (result) {
                    return result;
                }
                // Run every @plugin call
                try {
                    if (pluginObj.use) {
                        pluginObj.use.call(this.context, pluginObj);
                    }
                }
                catch (e) {
                    e.message = e.message || 'Error during @plugin call';
                    return new LessError(e, imports, filename);
                }
            }
            else {
                return new LessError({ message: 'Not a valid plugin' }, imports, filename);
            }
            return pluginObj;
        };
        AbstractPluginLoader.prototype.trySetOptions = function (plugin, filename, name, options) {
            if (options && !plugin.setOptions) {
                return new LessError({
                    message: "Options have been provided but the plugin " + name + " does not support any options."
                });
            }
            try {
                plugin.setOptions && plugin.setOptions(options);
            }
            catch (e) {
                return new LessError(e);
            }
        };
        AbstractPluginLoader.prototype.validatePlugin = function (plugin, filename, name) {
            if (plugin) {
                // support plugins being a function
                // so that the plugin can be more usable programmatically
                if (typeof plugin === 'function') {
                    plugin = new plugin();
                }
                if (plugin.minVersion) {
                    if (this.compareVersion(plugin.minVersion, this.less.version) < 0) {
                        return new LessError({
                            message: "Plugin " + name + " requires version " + this.versionToString(plugin.minVersion)
                        });
                    }
                }
                return plugin;
            }
            return null;
        };
        AbstractPluginLoader.prototype.compareVersion = function (aVersion, bVersion) {
            if (typeof aVersion === 'string') {
                aVersion = aVersion.match(/^(\d+)\.?(\d+)?\.?(\d+)?/);
                aVersion.shift();
            }
            for (var i_1 = 0; i_1 < aVersion.length; i_1++) {
                if (aVersion[i_1] !== bVersion[i_1]) {
                    return parseInt(aVersion[i_1]) > parseInt(bVersion[i_1]) ? -1 : 1;
                }
            }
            return 0;
        };
        AbstractPluginLoader.prototype.versionToString = function (version) {
            var versionString = '';
            for (var i_2 = 0; i_2 < version.length; i_2++) {
                versionString += (versionString ? '.' : '') + version[i_2];
            }
            return versionString;
        };
        AbstractPluginLoader.prototype.printUsage = function (plugins) {
            for (var i_3 = 0; i_3 < plugins.length; i_3++) {
                var plugin = plugins[i_3];
                if (plugin.printUsage) {
                    plugin.printUsage();
                }
            }
        };
        return AbstractPluginLoader;
    }());
    var _visitArgs = { visitDeeper: true };
    var _hasIndexed = false;
    function _noop(node) {
        return node;
    }
    function indexNodeTypes(parent, ticker) {
        // add .typeIndex to tree node types for lookup table
        var key;
        var child;
        for (key in parent) {
            /* eslint guard-for-in: 0 */
            child = parent[key];
            switch (typeof child) {
                case 'function':
                    // ignore bound functions directly on tree which do not have a prototype
                    // or aren't nodes
                    if (child.prototype && child.prototype.type) {
                        child.prototype.typeIndex = ticker++;
                    }
                    break;
                case 'object':
                    ticker = indexNodeTypes(child, ticker);
                    break;
            }
        }
        return ticker;
    }
    var Visitor = /** @class */ (function () {
        function Visitor(implementation) {
            this._implementation = implementation;
            this._visitInCache = {};
            this._visitOutCache = {};
            if (!_hasIndexed) {
                indexNodeTypes(tree, 1);
                _hasIndexed = true;
            }
        }
        Visitor.prototype.visit = function (node) {
            if (!node) {
                return node;
            }
            var nodeTypeIndex = node.typeIndex;
            if (!nodeTypeIndex) {
                // MixinCall args aren't a node type?
                if (node.value && node.value.typeIndex) {
                    this.visit(node.value);
                }
                return node;
            }
            var impl = this._implementation;
            var func = this._visitInCache[nodeTypeIndex];
            var funcOut = this._visitOutCache[nodeTypeIndex];
            var visitArgs = _visitArgs;
            var fnName;
            visitArgs.visitDeeper = true;
            if (!func) {
                fnName = "visit" + node.type;
                func = impl[fnName] || _noop;
                funcOut = impl[fnName + "Out"] || _noop;
                this._visitInCache[nodeTypeIndex] = func;
                this._visitOutCache[nodeTypeIndex] = funcOut;
            }
            if (func !== _noop) {
                var newNode = func.call(impl, node, visitArgs);
                if (node && impl.isReplacing) {
                    node = newNode;
                }
            }
            if (visitArgs.visitDeeper && node) {
                if (node.length) {
                    for (var i = 0, cnt = node.length; i < cnt; i++) {
                        if (node[i].accept) {
                            node[i].accept(this);
                        }
                    }
                }
                else if (node.accept) {
                    node.accept(this);
                }
            }
            if (funcOut != _noop) {
                funcOut.call(impl, node);
            }
            return node;
        };
        Visitor.prototype.visitArray = function (nodes, nonReplacing) {
            if (!nodes) {
                return nodes;
            }
            var cnt = nodes.length;
            var i;
            // Non-replacing
            if (nonReplacing || !this._implementation.isReplacing) {
                for (i = 0; i < cnt; i++) {
                    this.visit(nodes[i]);
                }
                return nodes;
            }
            // Replacing
            var out = [];
            for (i = 0; i < cnt; i++) {
                var evald = this.visit(nodes[i]);
                if (evald === undefined) {
                    continue;
                }
                if (!evald.splice) {
                    out.push(evald);
                }
                else if (evald.length) {
                    this.flatten(evald, out);
                }
            }
            return out;
        };
        Visitor.prototype.flatten = function (arr, out) {
            if (!out) {
                out = [];
            }
            var cnt;
            var i;
            var item;
            var nestedCnt;
            var j;
            var nestedItem;
            for (i = 0, cnt = arr.length; i < cnt; i++) {
                item = arr[i];
                if (item === undefined) {
                    continue;
                }
                if (!item.splice) {
                    out.push(item);
                    continue;
                }
                for (j = 0, nestedCnt = item.length; j < nestedCnt; j++) {
                    nestedItem = item[j];
                    if (nestedItem === undefined) {
                        continue;
                    }
                    if (!nestedItem.splice) {
                        out.push(nestedItem);
                    }
                    else if (nestedItem.length) {
                        this.flatten(nestedItem, out);
                    }
                }
            }
            return out;
        };
        return Visitor;
    }());
    var ImportSequencer = /** @class */ (function () {
        function ImportSequencer(onSequencerEmpty) {
            this.imports = [];
            this.variableImports = [];
            this._onSequencerEmpty = onSequencerEmpty;
            this._currentDepth = 0;
        }
        ImportSequencer.prototype.addImport = function (callback) {
            var importSequencer = this;
            var importItem = {
                callback: callback,
                args: null,
                isReady: false
            };
            this.imports.push(importItem);
            return function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                importItem.args = Array.prototype.slice.call(args, 0);
                importItem.isReady = true;
                importSequencer.tryRun();
            };
        };
        ImportSequencer.prototype.addVariableImport = function (callback) {
            this.variableImports.push(callback);
        };
        ImportSequencer.prototype.tryRun = function () {
            this._currentDepth++;
            try {
                while (true) {
                    while (this.imports.length > 0) {
                        var importItem = this.imports[0];
                        if (!importItem.isReady) {
                            return;
                        }
                        this.imports = this.imports.slice(1);
                        importItem.callback.apply(null, importItem.args);
                    }
                    if (this.variableImports.length === 0) {
                        break;
                    }
                    var variableImport = this.variableImports[0];
                    this.variableImports = this.variableImports.slice(1);
                    variableImport();
                }
            }
            finally {
                this._currentDepth--;
            }
            if (this._currentDepth === 0 && this._onSequencerEmpty) {
                this._onSequencerEmpty();
            }
        };
        return ImportSequencer;
    }());
    var ImportVisitor = function (importer, finish) {
        this._visitor = new Visitor(this);
        this._importer = importer;
        this._finish = finish;
        this.context = new contexts.Eval();
        this.importCount = 0;
        this.onceFileDetectionMap = {};
        this.recursionDetector = {};
        this._sequencer = new ImportSequencer(this._onSequencerEmpty.bind(this));
    };
    ImportVisitor.prototype = {
        isReplacing: false,
        run: function (root) {
            try {
                // process the contents
                this._visitor.visit(root);
            }
            catch (e) {
                this.error = e;
            }
            this.isFinished = true;
            this._sequencer.tryRun();
        },
        _onSequencerEmpty: function () {
            if (!this.isFinished) {
                return;
            }
            this._finish(this.error);
        },
        visitImport: function (importNode, visitArgs) {
            var inlineCSS = importNode.options.inline;
            if (!importNode.css || inlineCSS) {
                var context = new contexts.Eval(this.context, copyArray(this.context.frames));
                var importParent = context.frames[0];
                this.importCount++;
                if (importNode.isVariableImport()) {
                    this._sequencer.addVariableImport(this.processImportNode.bind(this, importNode, context, importParent));
                }
                else {
                    this.processImportNode(importNode, context, importParent);
                }
            }
            visitArgs.visitDeeper = false;
        },
        processImportNode: function (importNode, context, importParent) {
            var evaldImportNode;
            var inlineCSS = importNode.options.inline;
            try {
                evaldImportNode = importNode.evalForImport(context);
            }
            catch (e) {
                if (!e.filename) {
                    e.index = importNode.getIndex();
                    e.filename = importNode.fileInfo().filename;
                }
                // attempt to eval properly and treat as css
                importNode.css = true;
                // if that fails, this error will be thrown
                importNode.error = e;
            }
            if (evaldImportNode && (!evaldImportNode.css || inlineCSS)) {
                if (evaldImportNode.options.multiple) {
                    context.importMultiple = true;
                }
                // try appending if we haven't determined if it is css or not
                var tryAppendLessExtension = evaldImportNode.css === undefined;
                for (var i_1 = 0; i_1 < importParent.rules.length; i_1++) {
                    if (importParent.rules[i_1] === importNode) {
                        importParent.rules[i_1] = evaldImportNode;
                        break;
                    }
                }
                var onImported = this.onImported.bind(this, evaldImportNode, context);
                var sequencedOnImported = this._sequencer.addImport(onImported);
                this._importer.push(evaldImportNode.getPath(), tryAppendLessExtension, evaldImportNode.fileInfo(), evaldImportNode.options, sequencedOnImported);
            }
            else {
                this.importCount--;
                if (this.isFinished) {
                    this._sequencer.tryRun();
                }
            }
        },
        onImported: function (importNode, context, e, root, importedAtRoot, fullPath) {
            if (e) {
                if (!e.filename) {
                    e.index = importNode.getIndex();
                    e.filename = importNode.fileInfo().filename;
                }
                this.error = e;
            }
            var importVisitor = this;
            var inlineCSS = importNode.options.inline;
            var isPlugin = importNode.options.isPlugin;
            var isOptional = importNode.options.optional;
            var duplicateImport = importedAtRoot || fullPath in importVisitor.recursionDetector;
            if (!context.importMultiple) {
                if (duplicateImport) {
                    importNode.skip = true;
                }
                else {
                    importNode.skip = function () {
                        if (fullPath in importVisitor.onceFileDetectionMap) {
                            return true;
                        }
                        importVisitor.onceFileDetectionMap[fullPath] = true;
                        return false;
                    };
                }
            }
            if (!fullPath && isOptional) {
                importNode.skip = true;
            }
            if (root) {
                importNode.root = root;
                importNode.importedFilename = fullPath;
                if (!inlineCSS && !isPlugin && (context.importMultiple || !duplicateImport)) {
                    importVisitor.recursionDetector[fullPath] = true;
                    var oldContext = this.context;
                    this.context = context;
                    try {
                        this._visitor.visit(root);
                    }
                    catch (e) {
                        this.error = e;
                    }
                    this.context = oldContext;
                }
            }
            importVisitor.importCount--;
            if (importVisitor.isFinished) {
                importVisitor._sequencer.tryRun();
            }
        },
        visitDeclaration: function (declNode, visitArgs) {
            if (declNode.value.type === 'DetachedRuleset') {
                this.context.frames.unshift(declNode);
            }
            else {
                visitArgs.visitDeeper = false;
            }
        },
        visitDeclarationOut: function (declNode) {
            if (declNode.value.type === 'DetachedRuleset') {
                this.context.frames.shift();
            }
        },
        visitAtRule: function (atRuleNode, visitArgs) {
            this.context.frames.unshift(atRuleNode);
        },
        visitAtRuleOut: function (atRuleNode) {
            this.context.frames.shift();
        },
        visitMixinDefinition: function (mixinDefinitionNode, visitArgs) {
            this.context.frames.unshift(mixinDefinitionNode);
        },
        visitMixinDefinitionOut: function (mixinDefinitionNode) {
            this.context.frames.shift();
        },
        visitRuleset: function (rulesetNode, visitArgs) {
            this.context.frames.unshift(rulesetNode);
        },
        visitRulesetOut: function (rulesetNode) {
            this.context.frames.shift();
        },
        visitMedia: function (mediaNode, visitArgs) {
            this.context.frames.unshift(mediaNode.rules[0]);
        },
        visitMediaOut: function (mediaNode) {
            this.context.frames.shift();
        }
    };
    var SetTreeVisibilityVisitor = /** @class */ (function () {
        function SetTreeVisibilityVisitor(visible) {
            this.visible = visible;
        }
        SetTreeVisibilityVisitor.prototype.run = function (root) {
            this.visit(root);
        };
        SetTreeVisibilityVisitor.prototype.visitArray = function (nodes) {
            if (!nodes) {
                return nodes;
            }
            var cnt = nodes.length;
            var i;
            for (i = 0; i < cnt; i++) {
                this.visit(nodes[i]);
            }
            return nodes;
        };
        SetTreeVisibilityVisitor.prototype.visit = function (node) {
            if (!node) {
                return node;
            }
            if (node.constructor === Array) {
                return this.visitArray(node);
            }
            if (!node.blocksVisibility || node.blocksVisibility()) {
                return node;
            }
            if (this.visible) {
                node.ensureVisibility();
            }
            else {
                node.ensureInvisibility();
            }
            node.accept(this);
            return node;
        };
        return SetTreeVisibilityVisitor;
    }());
    /* jshint loopfunc:true */
    var ExtendFinderVisitor = /** @class */ (function () {
        function ExtendFinderVisitor() {
            this._visitor = new Visitor(this);
            this.contexts = [];
            this.allExtendsStack = [[]];
        }
        ExtendFinderVisitor.prototype.run = function (root) {
            root = this._visitor.visit(root);
            root.allExtends = this.allExtendsStack[0];
            return root;
        };
        ExtendFinderVisitor.prototype.visitDeclaration = function (declNode, visitArgs) {
            visitArgs.visitDeeper = false;
        };
        ExtendFinderVisitor.prototype.visitMixinDefinition = function (mixinDefinitionNode, visitArgs) {
            visitArgs.visitDeeper = false;
        };
        ExtendFinderVisitor.prototype.visitRuleset = function (rulesetNode, visitArgs) {
            if (rulesetNode.root) {
                return;
            }
            var i;
            var j;
            var extend;
            var allSelectorsExtendList = [];
            var extendList;
            // get &:extend(.a); rules which apply to all selectors in this ruleset
            var rules = rulesetNode.rules;
            var ruleCnt = rules ? rules.length : 0;
            for (i = 0; i < ruleCnt; i++) {
                if (rulesetNode.rules[i] instanceof tree.Extend) {
                    allSelectorsExtendList.push(rules[i]);
                    rulesetNode.extendOnEveryPath = true;
                }
            }
            // now find every selector and apply the extends that apply to all extends
            // and the ones which apply to an individual extend
            var paths = rulesetNode.paths;
            for (i = 0; i < paths.length; i++) {
                var selectorPath = paths[i];
                var selector = selectorPath[selectorPath.length - 1];
                var selExtendList = selector.extendList;
                extendList = selExtendList ? copyArray(selExtendList).concat(allSelectorsExtendList)
                    : allSelectorsExtendList;
                if (extendList) {
                    extendList = extendList.map(function (allSelectorsExtend) { return allSelectorsExtend.clone(); });
                }
                for (j = 0; j < extendList.length; j++) {
                    this.foundExtends = true;
                    extend = extendList[j];
                    extend.findSelfSelectors(selectorPath);
                    extend.ruleset = rulesetNode;
                    if (j === 0) {
                        extend.firstExtendOnThisSelectorPath = true;
                    }
                    this.allExtendsStack[this.allExtendsStack.length - 1].push(extend);
                }
            }
            this.contexts.push(rulesetNode.selectors);
        };
        ExtendFinderVisitor.prototype.visitRulesetOut = function (rulesetNode) {
            if (!rulesetNode.root) {
                this.contexts.length = this.contexts.length - 1;
            }
        };
        ExtendFinderVisitor.prototype.visitMedia = function (mediaNode, visitArgs) {
            mediaNode.allExtends = [];
            this.allExtendsStack.push(mediaNode.allExtends);
        };
        ExtendFinderVisitor.prototype.visitMediaOut = function (mediaNode) {
            this.allExtendsStack.length = this.allExtendsStack.length - 1;
        };
        ExtendFinderVisitor.prototype.visitAtRule = function (atRuleNode, visitArgs) {
            atRuleNode.allExtends = [];
            this.allExtendsStack.push(atRuleNode.allExtends);
        };
        ExtendFinderVisitor.prototype.visitAtRuleOut = function (atRuleNode) {
            this.allExtendsStack.length = this.allExtendsStack.length - 1;
        };
        return ExtendFinderVisitor;
    }());
    var ProcessExtendsVisitor = /** @class */ (function () {
        function ProcessExtendsVisitor() {
            this._visitor = new Visitor(this);
        }
        ProcessExtendsVisitor.prototype.run = function (root) {
            var extendFinder = new ExtendFinderVisitor();
            this.extendIndices = {};
            extendFinder.run(root);
            if (!extendFinder.foundExtends) {
                return root;
            }
            root.allExtends = root.allExtends.concat(this.doExtendChaining(root.allExtends, root.allExtends));
            this.allExtendsStack = [root.allExtends];
            var newRoot = this._visitor.visit(root);
            this.checkExtendsForNonMatched(root.allExtends);
            return newRoot;
        };
        ProcessExtendsVisitor.prototype.checkExtendsForNonMatched = function (extendList) {
            var indices = this.extendIndices;
            extendList.filter(function (extend) { return !extend.hasFoundMatches && extend.parent_ids.length == 1; }).forEach(function (extend) {
                var selector = '_unknown_';
                try {
                    selector = extend.selector.toCSS({});
                }
                catch (_) { }
                if (!indices[extend.index + " " + selector]) {
                    indices[extend.index + " " + selector] = true;
                    logger.warn("extend '" + selector + "' has no matches");
                }
            });
        };
        ProcessExtendsVisitor.prototype.doExtendChaining = function (extendsList, extendsListTarget, iterationCount) {
            //
            // chaining is different from normal extension.. if we extend an extend then we are not just copying, altering
            // and pasting the selector we would do normally, but we are also adding an extend with the same target selector
            // this means this new extend can then go and alter other extends
            //
            // this method deals with all the chaining work - without it, extend is flat and doesn't work on other extend selectors
            // this is also the most expensive.. and a match on one selector can cause an extension of a selector we had already
            // processed if we look at each selector at a time, as is done in visitRuleset
            var extendIndex;
            var targetExtendIndex;
            var matches;
            var extendsToAdd = [];
            var newSelector;
            var extendVisitor = this;
            var selectorPath;
            var extend;
            var targetExtend;
            var newExtend;
            iterationCount = iterationCount || 0;
            // loop through comparing every extend with every target extend.
            // a target extend is the one on the ruleset we are looking at copy/edit/pasting in place
            // e.g.  .a:extend(.b) {}  and .b:extend(.c) {} then the first extend extends the second one
            // and the second is the target.
            // the separation into two lists allows us to process a subset of chains with a bigger set, as is the
            // case when processing media queries
            for (extendIndex = 0; extendIndex < extendsList.length; extendIndex++) {
                for (targetExtendIndex = 0; targetExtendIndex < extendsListTarget.length; targetExtendIndex++) {
                    extend = extendsList[extendIndex];
                    targetExtend = extendsListTarget[targetExtendIndex];
                    // look for circular references
                    if (extend.parent_ids.indexOf(targetExtend.object_id) >= 0) {
                        continue;
                    }
                    // find a match in the target extends self selector (the bit before :extend)
                    selectorPath = [targetExtend.selfSelectors[0]];
                    matches = extendVisitor.findMatch(extend, selectorPath);
                    if (matches.length) {
                        extend.hasFoundMatches = true;
                        // we found a match, so for each self selector..
                        extend.selfSelectors.forEach(function (selfSelector) {
                            var info = targetExtend.visibilityInfo();
                            // process the extend as usual
                            newSelector = extendVisitor.extendSelector(matches, selectorPath, selfSelector, extend.isVisible());
                            // but now we create a new extend from it
                            newExtend = new (tree.Extend)(targetExtend.selector, targetExtend.option, 0, targetExtend.fileInfo(), info);
                            newExtend.selfSelectors = newSelector;
                            // add the extend onto the list of extends for that selector
                            newSelector[newSelector.length - 1].extendList = [newExtend];
                            // record that we need to add it.
                            extendsToAdd.push(newExtend);
                            newExtend.ruleset = targetExtend.ruleset;
                            // remember its parents for circular references
                            newExtend.parent_ids = newExtend.parent_ids.concat(targetExtend.parent_ids, extend.parent_ids);
                            // only process the selector once.. if we have :extend(.a,.b) then multiple
                            // extends will look at the same selector path, so when extending
                            // we know that any others will be duplicates in terms of what is added to the css
                            if (targetExtend.firstExtendOnThisSelectorPath) {
                                newExtend.firstExtendOnThisSelectorPath = true;
                                targetExtend.ruleset.paths.push(newSelector);
                            }
                        });
                    }
                }
            }
            if (extendsToAdd.length) {
                // try to detect circular references to stop a stack overflow.
                // may no longer be needed.
                this.extendChainCount++;
                if (iterationCount > 100) {
                    var selectorOne = '{unable to calculate}';
                    var selectorTwo = '{unable to calculate}';
                    try {
                        selectorOne = extendsToAdd[0].selfSelectors[0].toCSS();
                        selectorTwo = extendsToAdd[0].selector.toCSS();
                    }
                    catch (e) { }
                    throw { message: "extend circular reference detected. One of the circular extends is currently:" + selectorOne + ":extend(" + selectorTwo + ")" };
                }
                // now process the new extends on the existing rules so that we can handle a extending b extending c extending
                // d extending e...
                return extendsToAdd.concat(extendVisitor.doExtendChaining(extendsToAdd, extendsListTarget, iterationCount + 1));
            }
            else {
                return extendsToAdd;
            }
        };
        ProcessExtendsVisitor.prototype.visitDeclaration = function (ruleNode, visitArgs) {
            visitArgs.visitDeeper = false;
        };
        ProcessExtendsVisitor.prototype.visitMixinDefinition = function (mixinDefinitionNode, visitArgs) {
            visitArgs.visitDeeper = false;
        };
        ProcessExtendsVisitor.prototype.visitSelector = function (selectorNode, visitArgs) {
            visitArgs.visitDeeper = false;
        };
        ProcessExtendsVisitor.prototype.visitRuleset = function (rulesetNode, visitArgs) {
            if (rulesetNode.root) {
                return;
            }
            var matches;
            var pathIndex;
            var extendIndex;
            var allExtends = this.allExtendsStack[this.allExtendsStack.length - 1];
            var selectorsToAdd = [];
            var extendVisitor = this;
            var selectorPath;
            // look at each selector path in the ruleset, find any extend matches and then copy, find and replace
            for (extendIndex = 0; extendIndex < allExtends.length; extendIndex++) {
                for (pathIndex = 0; pathIndex < rulesetNode.paths.length; pathIndex++) {
                    selectorPath = rulesetNode.paths[pathIndex];
                    // extending extends happens initially, before the main pass
                    if (rulesetNode.extendOnEveryPath) {
                        continue;
                    }
                    var extendList = selectorPath[selectorPath.length - 1].extendList;
                    if (extendList && extendList.length) {
                        continue;
                    }
                    matches = this.findMatch(allExtends[extendIndex], selectorPath);
                    if (matches.length) {
                        allExtends[extendIndex].hasFoundMatches = true;
                        allExtends[extendIndex].selfSelectors.forEach(function (selfSelector) {
                            var extendedSelectors;
                            extendedSelectors = extendVisitor.extendSelector(matches, selectorPath, selfSelector, allExtends[extendIndex].isVisible());
                            selectorsToAdd.push(extendedSelectors);
                        });
                    }
                }
            }
            rulesetNode.paths = rulesetNode.paths.concat(selectorsToAdd);
        };
        ProcessExtendsVisitor.prototype.findMatch = function (extend, haystackSelectorPath) {
            //
            // look through the haystack selector path to try and find the needle - extend.selector
            // returns an array of selector matches that can then be replaced
            //
            var haystackSelectorIndex;
            var hackstackSelector;
            var hackstackElementIndex;
            var haystackElement;
            var targetCombinator;
            var i;
            var extendVisitor = this;
            var needleElements = extend.selector.elements;
            var potentialMatches = [];
            var potentialMatch;
            var matches = [];
            // loop through the haystack elements
            for (haystackSelectorIndex = 0; haystackSelectorIndex < haystackSelectorPath.length; haystackSelectorIndex++) {
                hackstackSelector = haystackSelectorPath[haystackSelectorIndex];
                for (hackstackElementIndex = 0; hackstackElementIndex < hackstackSelector.elements.length; hackstackElementIndex++) {
                    haystackElement = hackstackSelector.elements[hackstackElementIndex];
                    // if we allow elements before our match we can add a potential match every time. otherwise only at the first element.
                    if (extend.allowBefore || (haystackSelectorIndex === 0 && hackstackElementIndex === 0)) {
                        potentialMatches.push({ pathIndex: haystackSelectorIndex, index: hackstackElementIndex, matched: 0,
                            initialCombinator: haystackElement.combinator });
                    }
                    for (i = 0; i < potentialMatches.length; i++) {
                        potentialMatch = potentialMatches[i];
                        // selectors add " " onto the first element. When we use & it joins the selectors together, but if we don't
                        // then each selector in haystackSelectorPath has a space before it added in the toCSS phase. so we need to
                        // work out what the resulting combinator will be
                        targetCombinator = haystackElement.combinator.value;
                        if (targetCombinator === '' && hackstackElementIndex === 0) {
                            targetCombinator = ' ';
                        }
                        // if we don't match, null our match to indicate failure
                        if (!extendVisitor.isElementValuesEqual(needleElements[potentialMatch.matched].value, haystackElement.value) ||
                            (potentialMatch.matched > 0 && needleElements[potentialMatch.matched].combinator.value !== targetCombinator)) {
                            potentialMatch = null;
                        }
                        else {
                            potentialMatch.matched++;
                        }
                        // if we are still valid and have finished, test whether we have elements after and whether these are allowed
                        if (potentialMatch) {
                            potentialMatch.finished = potentialMatch.matched === needleElements.length;
                            if (potentialMatch.finished &&
                                (!extend.allowAfter &&
                                    (hackstackElementIndex + 1 < hackstackSelector.elements.length || haystackSelectorIndex + 1 < haystackSelectorPath.length))) {
                                potentialMatch = null;
                            }
                        }
                        // if null we remove, if not, we are still valid, so either push as a valid match or continue
                        if (potentialMatch) {
                            if (potentialMatch.finished) {
                                potentialMatch.length = needleElements.length;
                                potentialMatch.endPathIndex = haystackSelectorIndex;
                                potentialMatch.endPathElementIndex = hackstackElementIndex + 1; // index after end of match
                                potentialMatches.length = 0; // we don't allow matches to overlap, so start matching again
                                matches.push(potentialMatch);
                            }
                        }
                        else {
                            potentialMatches.splice(i, 1);
                            i--;
                        }
                    }
                }
            }
            return matches;
        };
        ProcessExtendsVisitor.prototype.isElementValuesEqual = function (elementValue1, elementValue2) {
            if (typeof elementValue1 === 'string' || typeof elementValue2 === 'string') {
                return elementValue1 === elementValue2;
            }
            if (elementValue1 instanceof tree.Attribute) {
                if (elementValue1.op !== elementValue2.op || elementValue1.key !== elementValue2.key) {
                    return false;
                }
                if (!elementValue1.value || !elementValue2.value) {
                    if (elementValue1.value || elementValue2.value) {
                        return false;
                    }
                    return true;
                }
                elementValue1 = elementValue1.value.value || elementValue1.value;
                elementValue2 = elementValue2.value.value || elementValue2.value;
                return elementValue1 === elementValue2;
            }
            elementValue1 = elementValue1.value;
            elementValue2 = elementValue2.value;
            if (elementValue1 instanceof tree.Selector) {
                if (!(elementValue2 instanceof tree.Selector) || elementValue1.elements.length !== elementValue2.elements.length) {
                    return false;
                }
                for (var i_1 = 0; i_1 < elementValue1.elements.length; i_1++) {
                    if (elementValue1.elements[i_1].combinator.value !== elementValue2.elements[i_1].combinator.value) {
                        if (i_1 !== 0 || (elementValue1.elements[i_1].combinator.value || ' ') !== (elementValue2.elements[i_1].combinator.value || ' ')) {
                            return false;
                        }
                    }
                    if (!this.isElementValuesEqual(elementValue1.elements[i_1].value, elementValue2.elements[i_1].value)) {
                        return false;
                    }
                }
                return true;
            }
            return false;
        };
        ProcessExtendsVisitor.prototype.extendSelector = function (matches, selectorPath, replacementSelector, isVisible) {
            // for a set of matches, replace each match with the replacement selector
            var currentSelectorPathIndex = 0;
            var currentSelectorPathElementIndex = 0;
            var path = [];
            var matchIndex;
            var selector;
            var firstElement;
            var match;
            var newElements;
            for (matchIndex = 0; matchIndex < matches.length; matchIndex++) {
                match = matches[matchIndex];
                selector = selectorPath[match.pathIndex];
                firstElement = new tree.Element(match.initialCombinator, replacementSelector.elements[0].value, replacementSelector.elements[0].isVariable, replacementSelector.elements[0].getIndex(), replacementSelector.elements[0].fileInfo());
                if (match.pathIndex > currentSelectorPathIndex && currentSelectorPathElementIndex > 0) {
                    path[path.length - 1].elements = path[path.length - 1]
                        .elements.concat(selectorPath[currentSelectorPathIndex].elements.slice(currentSelectorPathElementIndex));
                    currentSelectorPathElementIndex = 0;
                    currentSelectorPathIndex++;
                }
                newElements = selector.elements
                    .slice(currentSelectorPathElementIndex, match.index)
                    .concat([firstElement])
                    .concat(replacementSelector.elements.slice(1));
                if (currentSelectorPathIndex === match.pathIndex && matchIndex > 0) {
                    path[path.length - 1].elements =
                        path[path.length - 1].elements.concat(newElements);
                }
                else {
                    path = path.concat(selectorPath.slice(currentSelectorPathIndex, match.pathIndex));
                    path.push(new tree.Selector(newElements));
                }
                currentSelectorPathIndex = match.endPathIndex;
                currentSelectorPathElementIndex = match.endPathElementIndex;
                if (currentSelectorPathElementIndex >= selectorPath[currentSelectorPathIndex].elements.length) {
                    currentSelectorPathElementIndex = 0;
                    currentSelectorPathIndex++;
                }
            }
            if (currentSelectorPathIndex < selectorPath.length && currentSelectorPathElementIndex > 0) {
                path[path.length - 1].elements = path[path.length - 1]
                    .elements.concat(selectorPath[currentSelectorPathIndex].elements.slice(currentSelectorPathElementIndex));
                currentSelectorPathIndex++;
            }
            path = path.concat(selectorPath.slice(currentSelectorPathIndex, selectorPath.length));
            path = path.map(function (currentValue) {
                // we can re-use elements here, because the visibility property matters only for selectors
                var derived = currentValue.createDerived(currentValue.elements);
                if (isVisible) {
                    derived.ensureVisibility();
                }
                else {
                    derived.ensureInvisibility();
                }
                return derived;
            });
            return path;
        };
        ProcessExtendsVisitor.prototype.visitMedia = function (mediaNode, visitArgs) {
            var newAllExtends = mediaNode.allExtends.concat(this.allExtendsStack[this.allExtendsStack.length - 1]);
            newAllExtends = newAllExtends.concat(this.doExtendChaining(newAllExtends, mediaNode.allExtends));
            this.allExtendsStack.push(newAllExtends);
        };
        ProcessExtendsVisitor.prototype.visitMediaOut = function (mediaNode) {
            var lastIndex = this.allExtendsStack.length - 1;
            this.allExtendsStack.length = lastIndex;
        };
        ProcessExtendsVisitor.prototype.visitAtRule = function (atRuleNode, visitArgs) {
            var newAllExtends = atRuleNode.allExtends.concat(this.allExtendsStack[this.allExtendsStack.length - 1]);
            newAllExtends = newAllExtends.concat(this.doExtendChaining(newAllExtends, atRuleNode.allExtends));
            this.allExtendsStack.push(newAllExtends);
        };
        ProcessExtendsVisitor.prototype.visitAtRuleOut = function (atRuleNode) {
            var lastIndex = this.allExtendsStack.length - 1;
            this.allExtendsStack.length = lastIndex;
        };
        return ProcessExtendsVisitor;
    }());
    var JoinSelectorVisitor = /** @class */ (function () {
        function JoinSelectorVisitor() {
            this.contexts = [[]];
            this._visitor = new Visitor(this);
        }
        JoinSelectorVisitor.prototype.run = function (root) {
            return this._visitor.visit(root);
        };
        JoinSelectorVisitor.prototype.visitDeclaration = function (declNode, visitArgs) {
            visitArgs.visitDeeper = false;
        };
        JoinSelectorVisitor.prototype.visitMixinDefinition = function (mixinDefinitionNode, visitArgs) {
            visitArgs.visitDeeper = false;
        };
        JoinSelectorVisitor.prototype.visitRuleset = function (rulesetNode, visitArgs) {
            var context = this.contexts[this.contexts.length - 1];
            var paths = [];
            var selectors;
            this.contexts.push(paths);
            if (!rulesetNode.root) {
                selectors = rulesetNode.selectors;
                if (selectors) {
                    selectors = selectors.filter(function (selector) { return selector.getIsOutput(); });
                    rulesetNode.selectors = selectors.length ? selectors : (selectors = null);
                    if (selectors) {
                        rulesetNode.joinSelectors(paths, context, selectors);
                    }
                }
                if (!selectors) {
                    rulesetNode.rules = null;
                }
                rulesetNode.paths = paths;
            }
        };
        JoinSelectorVisitor.prototype.visitRulesetOut = function (rulesetNode) {
            this.contexts.length = this.contexts.length - 1;
        };
        JoinSelectorVisitor.prototype.visitMedia = function (mediaNode, visitArgs) {
            var context = this.contexts[this.contexts.length - 1];
            mediaNode.rules[0].root = (context.length === 0 || context[0].multiMedia);
        };
        JoinSelectorVisitor.prototype.visitAtRule = function (atRuleNode, visitArgs) {
            var context = this.contexts[this.contexts.length - 1];
            if (atRuleNode.rules && atRuleNode.rules.length) {
                atRuleNode.rules[0].root = (atRuleNode.isRooted || context.length === 0 || null);
            }
        };
        return JoinSelectorVisitor;
    }());
    var CSSVisitorUtils = /** @class */ (function () {
        function CSSVisitorUtils(context) {
            this._visitor = new Visitor(this);
            this._context = context;
        }
        CSSVisitorUtils.prototype.containsSilentNonBlockedChild = function (bodyRules) {
            var rule;
            if (!bodyRules) {
                return false;
            }
            for (var r = 0; r < bodyRules.length; r++) {
                rule = bodyRules[r];
                if (rule.isSilent && rule.isSilent(this._context) && !rule.blocksVisibility()) {
                    // the atrule contains something that was referenced (likely by extend)
                    // therefore it needs to be shown in output too
                    return true;
                }
            }
            return false;
        };
        CSSVisitorUtils.prototype.keepOnlyVisibleChilds = function (owner) {
            if (owner && owner.rules) {
                owner.rules = owner.rules.filter(function (thing) { return thing.isVisible(); });
            }
        };
        CSSVisitorUtils.prototype.isEmpty = function (owner) {
            return (owner && owner.rules)
                ? (owner.rules.length === 0) : true;
        };
        CSSVisitorUtils.prototype.hasVisibleSelector = function (rulesetNode) {
            return (rulesetNode && rulesetNode.paths)
                ? (rulesetNode.paths.length > 0) : false;
        };
        CSSVisitorUtils.prototype.resolveVisibility = function (node, originalRules) {
            if (!node.blocksVisibility()) {
                if (this.isEmpty(node) && !this.containsSilentNonBlockedChild(originalRules)) {
                    return;
                }
                return node;
            }
            var compiledRulesBody = node.rules[0];
            this.keepOnlyVisibleChilds(compiledRulesBody);
            if (this.isEmpty(compiledRulesBody)) {
                return;
            }
            node.ensureVisibility();
            node.removeVisibilityBlock();
            return node;
        };
        CSSVisitorUtils.prototype.isVisibleRuleset = function (rulesetNode) {
            if (rulesetNode.firstRoot) {
                return true;
            }
            if (this.isEmpty(rulesetNode)) {
                return false;
            }
            if (!rulesetNode.root && !this.hasVisibleSelector(rulesetNode)) {
                return false;
            }
            return true;
        };
        return CSSVisitorUtils;
    }());
    var ToCSSVisitor = function (context) {
        this._visitor = new Visitor(this);
        this._context = context;
        this.utils = new CSSVisitorUtils(context);
    };
    ToCSSVisitor.prototype = {
        isReplacing: true,
        run: function (root) {
            return this._visitor.visit(root);
        },
        visitDeclaration: function (declNode, visitArgs) {
            if (declNode.blocksVisibility() || declNode.variable) {
                return;
            }
            return declNode;
        },
        visitMixinDefinition: function (mixinNode, visitArgs) {
            // mixin definitions do not get eval'd - this means they keep state
            // so we have to clear that state here so it isn't used if toCSS is called twice
            mixinNode.frames = [];
        },
        visitExtend: function (extendNode, visitArgs) {
        },
        visitComment: function (commentNode, visitArgs) {
            if (commentNode.blocksVisibility() || commentNode.isSilent(this._context)) {
                return;
            }
            return commentNode;
        },
        visitMedia: function (mediaNode, visitArgs) {
            var originalRules = mediaNode.rules[0].rules;
            mediaNode.accept(this._visitor);
            visitArgs.visitDeeper = false;
            return this.utils.resolveVisibility(mediaNode, originalRules);
        },
        visitImport: function (importNode, visitArgs) {
            if (importNode.blocksVisibility()) {
                return;
            }
            return importNode;
        },
        visitAtRule: function (atRuleNode, visitArgs) {
            if (atRuleNode.rules && atRuleNode.rules.length) {
                return this.visitAtRuleWithBody(atRuleNode, visitArgs);
            }
            else {
                return this.visitAtRuleWithoutBody(atRuleNode, visitArgs);
            }
        },
        visitAnonymous: function (anonymousNode, visitArgs) {
            if (!anonymousNode.blocksVisibility()) {
                anonymousNode.accept(this._visitor);
                return anonymousNode;
            }
        },
        visitAtRuleWithBody: function (atRuleNode, visitArgs) {
            // if there is only one nested ruleset and that one has no path, then it is
            // just fake ruleset
            function hasFakeRuleset(atRuleNode) {
                var bodyRules = atRuleNode.rules;
                return bodyRules.length === 1 && (!bodyRules[0].paths || bodyRules[0].paths.length === 0);
            }
            function getBodyRules(atRuleNode) {
                var nodeRules = atRuleNode.rules;
                if (hasFakeRuleset(atRuleNode)) {
                    return nodeRules[0].rules;
                }
                return nodeRules;
            }
            // it is still true that it is only one ruleset in array
            // this is last such moment
            // process childs
            var originalRules = getBodyRules(atRuleNode);
            atRuleNode.accept(this._visitor);
            visitArgs.visitDeeper = false;
            if (!this.utils.isEmpty(atRuleNode)) {
                this._mergeRules(atRuleNode.rules[0].rules);
            }
            return this.utils.resolveVisibility(atRuleNode, originalRules);
        },
        visitAtRuleWithoutBody: function (atRuleNode, visitArgs) {
            if (atRuleNode.blocksVisibility()) {
                return;
            }
            if (atRuleNode.name === '@charset') {
                // Only output the debug info together with subsequent @charset definitions
                // a comment (or @media statement) before the actual @charset atrule would
                // be considered illegal css as it has to be on the first line
                if (this.charset) {
                    if (atRuleNode.debugInfo) {
                        var comment = new tree.Comment("/* " + atRuleNode.toCSS(this._context).replace(/\n/g, '') + " */\n");
                        comment.debugInfo = atRuleNode.debugInfo;
                        return this._visitor.visit(comment);
                    }
                    return;
                }
                this.charset = true;
            }
            return atRuleNode;
        },
        checkValidNodes: function (rules, isRoot) {
            if (!rules) {
                return;
            }
            for (var i_1 = 0; i_1 < rules.length; i_1++) {
                var ruleNode = rules[i_1];
                if (isRoot && ruleNode instanceof tree.Declaration && !ruleNode.variable) {
                    throw { message: 'Properties must be inside selector blocks. They cannot be in the root',
                        index: ruleNode.getIndex(), filename: ruleNode.fileInfo() && ruleNode.fileInfo().filename };
                }
                if (ruleNode instanceof tree.Call) {
                    throw { message: "Function '" + ruleNode.name + "' is undefined",
                        index: ruleNode.getIndex(), filename: ruleNode.fileInfo() && ruleNode.fileInfo().filename };
                }
                if (ruleNode.type && !ruleNode.allowRoot) {
                    throw { message: ruleNode.type + " node returned by a function is not valid here",
                        index: ruleNode.getIndex(), filename: ruleNode.fileInfo() && ruleNode.fileInfo().filename };
                }
            }
        },
        visitRuleset: function (rulesetNode, visitArgs) {
            // at this point rulesets are nested into each other
            var rule;
            var rulesets = [];
            this.checkValidNodes(rulesetNode.rules, rulesetNode.firstRoot);
            if (!rulesetNode.root) {
                // remove invisible paths
                this._compileRulesetPaths(rulesetNode);
                // remove rulesets from this ruleset body and compile them separately
                var nodeRules = rulesetNode.rules;
                var nodeRuleCnt = nodeRules ? nodeRules.length : 0;
                for (var i_2 = 0; i_2 < nodeRuleCnt;) {
                    rule = nodeRules[i_2];
                    if (rule && rule.rules) {
                        // visit because we are moving them out from being a child
                        rulesets.push(this._visitor.visit(rule));
                        nodeRules.splice(i_2, 1);
                        nodeRuleCnt--;
                        continue;
                    }
                    i_2++;
                }
                // accept the visitor to remove rules and refactor itself
                // then we can decide nogw whether we want it or not
                // compile body
                if (nodeRuleCnt > 0) {
                    rulesetNode.accept(this._visitor);
                }
                else {
                    rulesetNode.rules = null;
                }
                visitArgs.visitDeeper = false;
            }
            else { // if (! rulesetNode.root) {
                rulesetNode.accept(this._visitor);
                visitArgs.visitDeeper = false;
            }
            if (rulesetNode.rules) {
                this._mergeRules(rulesetNode.rules);
                this._removeDuplicateRules(rulesetNode.rules);
            }
            // now decide whether we keep the ruleset
            if (this.utils.isVisibleRuleset(rulesetNode)) {
                rulesetNode.ensureVisibility();
                rulesets.splice(0, 0, rulesetNode);
            }
            if (rulesets.length === 1) {
                return rulesets[0];
            }
            return rulesets;
        },
        _compileRulesetPaths: function (rulesetNode) {
            if (rulesetNode.paths) {
                rulesetNode.paths = rulesetNode.paths
                    .filter(function (p) {
                    var i;
                    if (p[0].elements[0].combinator.value === ' ') {
                        p[0].elements[0].combinator = new (tree.Combinator)('');
                    }
                    for (i = 0; i < p.length; i++) {
                        if (p[i].isVisible() && p[i].getIsOutput()) {
                            return true;
                        }
                    }
                    return false;
                });
            }
        },
        _removeDuplicateRules: function (rules) {
            if (!rules) {
                return;
            }
            // remove duplicates
            var ruleCache = {};
            var ruleList;
            var rule;
            var i;
            for (i = rules.length - 1; i >= 0; i--) {
                rule = rules[i];
                if (rule instanceof tree.Declaration) {
                    if (!ruleCache[rule.name]) {
                        ruleCache[rule.name] = rule;
                    }
                    else {
                        ruleList = ruleCache[rule.name];
                        if (ruleList instanceof tree.Declaration) {
                            ruleList = ruleCache[rule.name] = [ruleCache[rule.name].toCSS(this._context)];
                        }
                        var ruleCSS = rule.toCSS(this._context);
                        if (ruleList.indexOf(ruleCSS) !== -1) {
                            rules.splice(i, 1);
                        }
                        else {
                            ruleList.push(ruleCSS);
                        }
                    }
                }
            }
        },
        _mergeRules: function (rules) {
            if (!rules) {
                return;
            }
            var groups = {};
            var groupsArr = [];
            for (var i_3 = 0; i_3 < rules.length; i_3++) {
                var rule = rules[i_3];
                if (rule.merge) {
                    var key = rule.name;
                    groups[key] ? rules.splice(i_3--, 1) :
                        groupsArr.push(groups[key] = []);
                    groups[key].push(rule);
                }
            }
            groupsArr.forEach(function (group) {
                if (group.length > 0) {
                    var result_1 = group[0];
                    var space_1 = [];
                    var comma_1 = [new tree.Expression(space_1)];
                    group.forEach(function (rule) {
                        if ((rule.merge === '+') && (space_1.length > 0)) {
                            comma_1.push(new tree.Expression(space_1 = []));
                        }
                        space_1.push(rule.value);
                        result_1.important = result_1.important || rule.important;
                    });
                    result_1.value = new tree.Value(comma_1);
                }
            });
        }
    };
    var visitors = {
        Visitor: Visitor,
        ImportVisitor: ImportVisitor,
        MarkVisibleSelectorsVisitor: SetTreeVisibilityVisitor,
        ExtendVisitor: ProcessExtendsVisitor,
        JoinSelectorVisitor: JoinSelectorVisitor,
        ToCSSVisitor: ToCSSVisitor
    };
    // Split the input into chunks.
    var chunker = (function (input, fail) {
        var len = input.length;
        var level = 0;
        var parenLevel = 0;
        var lastOpening;
        var lastOpeningParen;
        var lastMultiComment;
        var lastMultiCommentEndBrace;
        var chunks = [];
        var emitFrom = 0;
        var chunkerCurrentIndex;
        var currentChunkStartIndex;
        var cc;
        var cc2;
        var matched;
        function emitChunk(force) {
            var len = chunkerCurrentIndex - emitFrom;
            if (((len < 512) && !force) || !len) {
                return;
            }
            chunks.push(input.slice(emitFrom, chunkerCurrentIndex + 1));
            emitFrom = chunkerCurrentIndex + 1;
        }
        for (chunkerCurrentIndex = 0; chunkerCurrentIndex < len; chunkerCurrentIndex++) {
            cc = input.charCodeAt(chunkerCurrentIndex);
            if (((cc >= 97) && (cc <= 122)) || (cc < 34)) {
                // a-z or whitespace
                continue;
            }
            switch (cc) {
                case 40: // (
                    parenLevel++;
                    lastOpeningParen = chunkerCurrentIndex;
                    continue;
                case 41: // )
                    if (--parenLevel < 0) {
                        return fail('missing opening `(`', chunkerCurrentIndex);
                    }
                    continue;
                case 59: // ;
                    if (!parenLevel) {
                        emitChunk();
                    }
                    continue;
                case 123: // {
                    level++;
                    lastOpening = chunkerCurrentIndex;
                    continue;
                case 125: // }
                    if (--level < 0) {
                        return fail('missing opening `{`', chunkerCurrentIndex);
                    }
                    if (!level && !parenLevel) {
                        emitChunk();
                    }
                    continue;
                case 92: // \
                    if (chunkerCurrentIndex < len - 1) {
                        chunkerCurrentIndex++;
                        continue;
                    }
                    return fail('unescaped `\\`', chunkerCurrentIndex);
                case 34:
                case 39:
                case 96: // ", ' and `
                    matched = 0;
                    currentChunkStartIndex = chunkerCurrentIndex;
                    for (chunkerCurrentIndex = chunkerCurrentIndex + 1; chunkerCurrentIndex < len; chunkerCurrentIndex++) {
                        cc2 = input.charCodeAt(chunkerCurrentIndex);
                        if (cc2 > 96) {
                            continue;
                        }
                        if (cc2 == cc) {
                            matched = 1;
                            break;
                        }
                        if (cc2 == 92) { // \
                            if (chunkerCurrentIndex == len - 1) {
                                return fail('unescaped `\\`', chunkerCurrentIndex);
                            }
                            chunkerCurrentIndex++;
                        }
                    }
                    if (matched) {
                        continue;
                    }
                    return fail("unmatched `" + String.fromCharCode(cc) + "`", currentChunkStartIndex);
                case 47: // /, check for comment
                    if (parenLevel || (chunkerCurrentIndex == len - 1)) {
                        continue;
                    }
                    cc2 = input.charCodeAt(chunkerCurrentIndex + 1);
                    if (cc2 == 47) {
                        // //, find lnfeed
                        for (chunkerCurrentIndex = chunkerCurrentIndex + 2; chunkerCurrentIndex < len; chunkerCurrentIndex++) {
                            cc2 = input.charCodeAt(chunkerCurrentIndex);
                            if ((cc2 <= 13) && ((cc2 == 10) || (cc2 == 13))) {
                                break;
                            }
                        }
                    }
                    else if (cc2 == 42) {
                        // /*, find */
                        lastMultiComment = currentChunkStartIndex = chunkerCurrentIndex;
                        for (chunkerCurrentIndex = chunkerCurrentIndex + 2; chunkerCurrentIndex < len - 1; chunkerCurrentIndex++) {
                            cc2 = input.charCodeAt(chunkerCurrentIndex);
                            if (cc2 == 125) {
                                lastMultiCommentEndBrace = chunkerCurrentIndex;
                            }
                            if (cc2 != 42) {
                                continue;
                            }
                            if (input.charCodeAt(chunkerCurrentIndex + 1) == 47) {
                                break;
                            }
                        }
                        if (chunkerCurrentIndex == len - 1) {
                            return fail('missing closing `*/`', currentChunkStartIndex);
                        }
                        chunkerCurrentIndex++;
                    }
                    continue;
                case 42: // *, check for unmatched */
                    if ((chunkerCurrentIndex < len - 1) && (input.charCodeAt(chunkerCurrentIndex + 1) == 47)) {
                        return fail('unmatched `/*`', chunkerCurrentIndex);
                    }
                    continue;
            }
        }
        if (level !== 0) {
            if ((lastMultiComment > lastOpening) && (lastMultiCommentEndBrace > lastMultiComment)) {
                return fail('missing closing `}` or `*/`', lastOpening);
            }
            else {
                return fail('missing closing `}`', lastOpening);
            }
        }
        else if (parenLevel !== 0) {
            return fail('missing closing `)`', lastOpeningParen);
        }
        emitChunk(true);
        return chunks;
    });
    var getParserInput = (function () {
        var // Less input string
        input;
        var // current chunk
        j;
        var // holds state for backtracking
        saveStack = [];
        var // furthest index the parser has gone to
        furthest;
        var // if this is furthest we got to, this is the probably cause
        furthestPossibleErrorMessage;
        var // chunkified input
        chunks;
        var // current chunk
        current;
        var // index of current chunk, in `input`
        currentPos;
        var parserInput = {};
        var CHARCODE_SPACE = 32;
        var CHARCODE_TAB = 9;
        var CHARCODE_LF = 10;
        var CHARCODE_CR = 13;
        var CHARCODE_PLUS = 43;
        var CHARCODE_COMMA = 44;
        var CHARCODE_FORWARD_SLASH = 47;
        var CHARCODE_9 = 57;
        function skipWhitespace(length) {
            var oldi = parserInput.i;
            var oldj = j;
            var curr = parserInput.i - currentPos;
            var endIndex = parserInput.i + current.length - curr;
            var mem = (parserInput.i += length);
            var inp = input;
            var c;
            var nextChar;
            var comment;
            for (; parserInput.i < endIndex; parserInput.i++) {
                c = inp.charCodeAt(parserInput.i);
                if (parserInput.autoCommentAbsorb && c === CHARCODE_FORWARD_SLASH) {
                    nextChar = inp.charAt(parserInput.i + 1);
                    if (nextChar === '/') {
                        comment = { index: parserInput.i, isLineComment: true };
                        var nextNewLine = inp.indexOf('\n', parserInput.i + 2);
                        if (nextNewLine < 0) {
                            nextNewLine = endIndex;
                        }
                        parserInput.i = nextNewLine;
                        comment.text = inp.substr(comment.index, parserInput.i - comment.index);
                        parserInput.commentStore.push(comment);
                        continue;
                    }
                    else if (nextChar === '*') {
                        var nextStarSlash = inp.indexOf('*/', parserInput.i + 2);
                        if (nextStarSlash >= 0) {
                            comment = {
                                index: parserInput.i,
                                text: inp.substr(parserInput.i, nextStarSlash + 2 - parserInput.i),
                                isLineComment: false
                            };
                            parserInput.i += comment.text.length - 1;
                            parserInput.commentStore.push(comment);
                            continue;
                        }
                    }
                    break;
                }
                if ((c !== CHARCODE_SPACE) && (c !== CHARCODE_LF) && (c !== CHARCODE_TAB) && (c !== CHARCODE_CR)) {
                    break;
                }
            }
            current = current.slice(length + parserInput.i - mem + curr);
            currentPos = parserInput.i;
            if (!current.length) {
                if (j < chunks.length - 1) {
                    current = chunks[++j];
                    skipWhitespace(0); // skip space at the beginning of a chunk
                    return true; // things changed
                }
                parserInput.finished = true;
            }
            return oldi !== parserInput.i || oldj !== j;
        }
        parserInput.save = function () {
            currentPos = parserInput.i;
            saveStack.push({ current: current, i: parserInput.i, j: j });
        };
        parserInput.restore = function (possibleErrorMessage) {
            if (parserInput.i > furthest || (parserInput.i === furthest && possibleErrorMessage && !furthestPossibleErrorMessage)) {
                furthest = parserInput.i;
                furthestPossibleErrorMessage = possibleErrorMessage;
            }
            var state = saveStack.pop();
            current = state.current;
            currentPos = parserInput.i = state.i;
            j = state.j;
        };
        parserInput.forget = function () {
            saveStack.pop();
        };
        parserInput.isWhitespace = function (offset) {
            var pos = parserInput.i + (offset || 0);
            var code = input.charCodeAt(pos);
            return (code === CHARCODE_SPACE || code === CHARCODE_CR || code === CHARCODE_TAB || code === CHARCODE_LF);
        };
        // Specialization of $(tok)
        parserInput.$re = function (tok) {
            if (parserInput.i > currentPos) {
                current = current.slice(parserInput.i - currentPos);
                currentPos = parserInput.i;
            }
            var m = tok.exec(current);
            if (!m) {
                return null;
            }
            skipWhitespace(m[0].length);
            if (typeof m === 'string') {
                return m;
            }
            return m.length === 1 ? m[0] : m;
        };
        parserInput.$char = function (tok) {
            if (input.charAt(parserInput.i) !== tok) {
                return null;
            }
            skipWhitespace(1);
            return tok;
        };
        parserInput.$str = function (tok) {
            var tokLength = tok.length;
            // https://jsperf.com/string-startswith/21
            for (var i_1 = 0; i_1 < tokLength; i_1++) {
                if (input.charAt(parserInput.i + i_1) !== tok.charAt(i_1)) {
                    return null;
                }
            }
            skipWhitespace(tokLength);
            return tok;
        };
        parserInput.$quoted = function (loc) {
            var pos = loc || parserInput.i;
            var startChar = input.charAt(pos);
            if (startChar !== '\'' && startChar !== '"') {
                return;
            }
            var length = input.length;
            var currentPosition = pos;
            for (var i_2 = 1; i_2 + currentPosition < length; i_2++) {
                var nextChar = input.charAt(i_2 + currentPosition);
                switch (nextChar) {
                    case '\\':
                        i_2++;
                        continue;
                    case '\r':
                    case '\n':
                        break;
                    case startChar:
                        var str = input.substr(currentPosition, i_2 + 1);
                        if (!loc && loc !== 0) {
                            skipWhitespace(i_2 + 1);
                            return str;
                        }
                        return [startChar, str];
                }
            }
            return null;
        };
        /**
         * Permissive parsing. Ignores everything except matching {} [] () and quotes
         * until matching token (outside of blocks)
         */
        parserInput.$parseUntil = function (tok) {
            var quote = '';
            var returnVal = null;
            var inComment = false;
            var blockDepth = 0;
            var blockStack = [];
            var parseGroups = [];
            var length = input.length;
            var startPos = parserInput.i;
            var lastPos = parserInput.i;
            var i = parserInput.i;
            var loop = true;
            var testChar;
            if (typeof tok === 'string') {
                testChar = function (char) { return char === tok; };
            }
            else {
                testChar = function (char) { return tok.test(char); };
            }
            do {
                var nextChar = input.charAt(i);
                if (blockDepth === 0 && testChar(nextChar)) {
                    returnVal = input.substr(lastPos, i - lastPos);
                    if (returnVal) {
                        parseGroups.push(returnVal);
                    }
                    else {
                        parseGroups.push(' ');
                    }
                    returnVal = parseGroups;
                    skipWhitespace(i - startPos);
                    loop = false;
                }
                else {
                    if (inComment) {
                        if (nextChar === '*' &&
                            input.charAt(i + 1) === '/') {
                            i++;
                            blockDepth--;
                            inComment = false;
                        }
                        i++;
                        continue;
                    }
                    switch (nextChar) {
                        case '\\':
                            i++;
                            nextChar = input.charAt(i);
                            parseGroups.push(input.substr(lastPos, i - lastPos + 1));
                            lastPos = i + 1;
                            break;
                        case '/':
                            if (input.charAt(i + 1) === '*') {
                                i++;
                                inComment = true;
                                blockDepth++;
                            }
                            break;
                        case '\'':
                        case '"':
                            quote = parserInput.$quoted(i);
                            if (quote) {
                                parseGroups.push(input.substr(lastPos, i - lastPos), quote);
                                i += quote[1].length - 1;
                                lastPos = i + 1;
                            }
                            else {
                                skipWhitespace(i - startPos);
                                returnVal = nextChar;
                                loop = false;
                            }
                            break;
                        case '{':
                            blockStack.push('}');
                            blockDepth++;
                            break;
                        case '(':
                            blockStack.push(')');
                            blockDepth++;
                            break;
                        case '[':
                            blockStack.push(']');
                            blockDepth++;
                            break;
                        case '}':
                        case ')':
                        case ']':
                            var expected = blockStack.pop();
                            if (nextChar === expected) {
                                blockDepth--;
                            }
                            else {
                                // move the parser to the error and return expected
                                skipWhitespace(i - startPos);
                                returnVal = expected;
                                loop = false;
                            }
                    }
                    i++;
                    if (i > length) {
                        loop = false;
                    }
                }
            } while (loop);
            return returnVal ? returnVal : null;
        };
        parserInput.autoCommentAbsorb = true;
        parserInput.commentStore = [];
        parserInput.finished = false;
        // Same as $(), but don't change the state of the parser,
        // just return the match.
        parserInput.peek = function (tok) {
            if (typeof tok === 'string') {
                // https://jsperf.com/string-startswith/21
                for (var i_3 = 0; i_3 < tok.length; i_3++) {
                    if (input.charAt(parserInput.i + i_3) !== tok.charAt(i_3)) {
                        return false;
                    }
                }
                return true;
            }
            else {
                return tok.test(current);
            }
        };
        // Specialization of peek()
        // TODO remove or change some currentChar calls to peekChar
        parserInput.peekChar = function (tok) { return input.charAt(parserInput.i) === tok; };
        parserInput.currentChar = function () { return input.charAt(parserInput.i); };
        parserInput.prevChar = function () { return input.charAt(parserInput.i - 1); };
        parserInput.getInput = function () { return input; };
        parserInput.peekNotNumeric = function () {
            var c = input.charCodeAt(parserInput.i);
            // Is the first char of the dimension 0-9, '.', '+' or '-'
            return (c > CHARCODE_9 || c < CHARCODE_PLUS) || c === CHARCODE_FORWARD_SLASH || c === CHARCODE_COMMA;
        };
        parserInput.start = function (str, chunkInput, failFunction) {
            input = str;
            parserInput.i = j = currentPos = furthest = 0;
            // chunking apparently makes things quicker (but my tests indicate
            // it might actually make things slower in node at least)
            // and it is a non-perfect parse - it can't recognise
            // unquoted urls, meaning it can't distinguish comments
            // meaning comments with quotes or {}() in them get 'counted'
            // and then lead to parse errors.
            // In addition if the chunking chunks in the wrong place we might
            // not be able to parse a parser statement in one go
            // this is officially deprecated but can be switched on via an option
            // in the case it causes too much performance issues.
            if (chunkInput) {
                chunks = chunker(str, failFunction);
            }
            else {
                chunks = [str];
            }
            current = chunks[0];
            skipWhitespace(0);
        };
        parserInput.end = function () {
            var message;
            var isFinished = parserInput.i >= input.length;
            if (parserInput.i < furthest) {
                message = furthestPossibleErrorMessage;
                parserInput.i = furthest;
            }
            return {
                isFinished: isFinished,
                furthest: parserInput.i,
                furthestPossibleErrorMessage: message,
                furthestReachedEnd: parserInput.i >= input.length - 1,
                furthestChar: input[parserInput.i]
            };
        };
        return parserInput;
    });
    //
    // less.js - parser
    //
    //    A relatively straight-forward predictive parser.
    //    There is no tokenization/lexing stage, the input is parsed
    //    in one sweep.
    //
    //    To make the parser fast enough to run in the browser, several
    //    optimization had to be made:
    //
    //    - Matching and slicing on a huge input is often cause of slowdowns.
    //      The solution is to chunkify the input into smaller strings.
    //      The chunks are stored in the `chunks` var,
    //      `j` holds the current chunk index, and `currentPos` holds
    //      the index of the current chunk in relation to `input`.
    //      This gives us an almost 4x speed-up.
    //
    //    - In many cases, we don't need to match individual tokens;
    //      for example, if a value doesn't hold any variables, operations
    //      or dynamic references, the parser can effectively 'skip' it,
    //      treating it as a literal.
    //      An example would be '1px solid #000' - which evaluates to itself,
    //      we don't need to know what the individual components are.
    //      The drawback, of course is that you don't get the benefits of
    //      syntax-checking on the CSS. This gives us a 50% speed-up in the parser,
    //      and a smaller speed-up in the code-gen.
    //
    //
    //    Token matching is done with the `$` function, which either takes
    //    a terminal string or regexp, or a non-terminal function to call.
    //    It also takes care of moving all the indices forwards.
    //
    var Parser = function Parser(context, imports, fileInfo) {
        var parsers;
        var parserInput = getParserInput();
        function error(msg, type) {
            throw new LessError({
                index: parserInput.i,
                filename: fileInfo.filename,
                type: type || 'Syntax',
                message: msg
            }, imports);
        }
        function expect(arg, msg) {
            // some older browsers return typeof 'function' for RegExp
            var result = (arg instanceof Function) ? arg.call(parsers) : parserInput.$re(arg);
            if (result) {
                return result;
            }
            error(msg || (typeof arg === 'string'
                ? "expected '" + arg + "' got '" + parserInput.currentChar() + "'"
                : 'unexpected token'));
        }
        // Specialization of expect()
        function expectChar(arg, msg) {
            if (parserInput.$char(arg)) {
                return arg;
            }
            error(msg || "expected '" + arg + "' got '" + parserInput.currentChar() + "'");
        }
        function getDebugInfo(index) {
            var filename = fileInfo.filename;
            return {
                lineNumber: getLocation(index, parserInput.getInput()).line + 1,
                fileName: filename
            };
        }
        /**
         *  Used after initial parsing to create nodes on the fly
         *
         *  @param {String} str          - string to parse
         *  @param {Array}  parseList    - array of parsers to run input through e.g. ["value", "important"]
         *  @param {Number} currentIndex - start number to begin indexing
         *  @param {Object} fileInfo     - fileInfo to attach to created nodes
         */
        function parseNode(str, parseList, currentIndex, fileInfo, callback) {
            var result;
            var returnNodes = [];
            var parser = parserInput;
            try {
                parser.start(str, false, function fail(msg, index) {
                    callback({
                        message: msg,
                        index: index + currentIndex
                    });
                });
                for (var x = 0, p = void 0, i_1; (p = parseList[x]); x++) {
                    i_1 = parser.i;
                    result = parsers[p]();
                    if (result) {
                        try {
                            result._index = i_1 + currentIndex;
                            result._fileInfo = fileInfo;
                        }
                        catch (e) { }
                        returnNodes.push(result);
                    }
                    else {
                        returnNodes.push(null);
                    }
                }
                var endInfo = parser.end();
                if (endInfo.isFinished) {
                    callback(null, returnNodes);
                }
                else {
                    callback(true, null);
                }
            }
            catch (e) {
                throw new LessError({
                    index: e.index + currentIndex,
                    message: e.message
                }, imports, fileInfo.filename);
            }
        }
        //
        // The Parser
        //
        return {
            parserInput: parserInput,
            imports: imports,
            fileInfo: fileInfo,
            parseNode: parseNode,
            //
            // Parse an input string into an abstract syntax tree,
            // @param str A string containing 'less' markup
            // @param callback call `callback` when done.
            // @param [additionalData] An optional map which can contains vars - a map (key, value) of variables to apply
            //
            parse: function (str, callback, additionalData) {
                var root;
                var error = null;
                var globalVars;
                var modifyVars;
                var ignored;
                var preText = '';
                globalVars = (additionalData && additionalData.globalVars) ? Parser.serializeVars(additionalData.globalVars) + "\n" : '';
                modifyVars = (additionalData && additionalData.modifyVars) ? "\n" + Parser.serializeVars(additionalData.modifyVars) : '';
                if (context.pluginManager) {
                    var preProcessors = context.pluginManager.getPreProcessors();
                    for (var i_2 = 0; i_2 < preProcessors.length; i_2++) {
                        str = preProcessors[i_2].process(str, { context: context, imports: imports, fileInfo: fileInfo });
                    }
                }
                if (globalVars || (additionalData && additionalData.banner)) {
                    preText = ((additionalData && additionalData.banner) ? additionalData.banner : '') + globalVars;
                    ignored = imports.contentsIgnoredChars;
                    ignored[fileInfo.filename] = ignored[fileInfo.filename] || 0;
                    ignored[fileInfo.filename] += preText.length;
                }
                str = str.replace(/\r\n?/g, '\n');
                // Remove potential UTF Byte Order Mark
                str = preText + str.replace(/^\uFEFF/, '') + modifyVars;
                imports.contents[fileInfo.filename] = str;
                // Start with the primary rule.
                // The whole syntax tree is held under a Ruleset node,
                // with the `root` property set to true, so no `{}` are
                // output. The callback is called when the input is parsed.
                try {
                    parserInput.start(str, context.chunkInput, function fail(msg, index) {
                        throw new LessError({
                            index: index,
                            type: 'Parse',
                            message: msg,
                            filename: fileInfo.filename
                        }, imports);
                    });
                    tree.Node.prototype.parse = this;
                    root = new tree.Ruleset(null, this.parsers.primary());
                    tree.Node.prototype.rootNode = root;
                    root.root = true;
                    root.firstRoot = true;
                    root.functionRegistry = functionRegistry.inherit();
                }
                catch (e) {
                    return callback(new LessError(e, imports, fileInfo.filename));
                }
                // If `i` is smaller than the `input.length - 1`,
                // it means the parser wasn't able to parse the whole
                // string, so we've got a parsing error.
                //
                // We try to extract a \n delimited string,
                // showing the line where the parse error occurred.
                // We split it up into two parts (the part which parsed,
                // and the part which didn't), so we can color them differently.
                var endInfo = parserInput.end();
                if (!endInfo.isFinished) {
                    var message = endInfo.furthestPossibleErrorMessage;
                    if (!message) {
                        message = 'Unrecognised input';
                        if (endInfo.furthestChar === '}') {
                            message += '. Possibly missing opening \'{\'';
                        }
                        else if (endInfo.furthestChar === ')') {
                            message += '. Possibly missing opening \'(\'';
                        }
                        else if (endInfo.furthestReachedEnd) {
                            message += '. Possibly missing something';
                        }
                    }
                    error = new LessError({
                        type: 'Parse',
                        message: message,
                        index: endInfo.furthest,
                        filename: fileInfo.filename
                    }, imports);
                }
                var finish = function (e) {
                    e = error || e || imports.error;
                    if (e) {
                        if (!(e instanceof LessError)) {
                            e = new LessError(e, imports, fileInfo.filename);
                        }
                        return callback(e);
                    }
                    else {
                        return callback(null, root);
                    }
                };
                if (context.processImports !== false) {
                    new visitors.ImportVisitor(imports, finish)
                        .run(root);
                }
                else {
                    return finish();
                }
            },
            //
            // Here in, the parsing rules/functions
            //
            // The basic structure of the syntax tree generated is as follows:
            //
            //   Ruleset ->  Declaration -> Value -> Expression -> Entity
            //
            // Here's some Less code:
            //
            //    .class {
            //      color: #fff;
            //      border: 1px solid #000;
            //      width: @w + 4px;
            //      > .child {...}
            //    }
            //
            // And here's what the parse tree might look like:
            //
            //     Ruleset (Selector '.class', [
            //         Declaration ("color",  Value ([Expression [Color #fff]]))
            //         Declaration ("border", Value ([Expression [Dimension 1px][Keyword "solid"][Color #000]]))
            //         Declaration ("width",  Value ([Expression [Operation " + " [Variable "@w"][Dimension 4px]]]))
            //         Ruleset (Selector [Element '>', '.child'], [...])
            //     ])
            //
            //  In general, most rules will try to parse a token with the `$re()` function, and if the return
            //  value is truly, will return a new node, of the relevant type. Sometimes, we need to check
            //  first, before parsing, that's when we use `peek()`.
            //
            parsers: parsers = {
                //
                // The `primary` rule is the *entry* and *exit* point of the parser.
                // The rules here can appear at any level of the parse tree.
                //
                // The recursive nature of the grammar is an interplay between the `block`
                // rule, which represents `{ ... }`, the `ruleset` rule, and this `primary` rule,
                // as represented by this simplified grammar:
                //
                //     primary  →  (ruleset | declaration)+
                //     ruleset  →  selector+ block
                //     block    →  '{' primary '}'
                //
                // Only at one point is the primary rule not called from the
                // block rule: at the root level.
                //
                primary: function () {
                    var mixin = this.mixin;
                    var root = [];
                    var node;
                    while (true) {
                        while (true) {
                            node = this.comment();
                            if (!node) {
                                break;
                            }
                            root.push(node);
                        }
                        // always process comments before deciding if finished
                        if (parserInput.finished) {
                            break;
                        }
                        if (parserInput.peek('}')) {
                            break;
                        }
                        node = this.extendRule();
                        if (node) {
                            root = root.concat(node);
                            continue;
                        }
                        node = mixin.definition() || this.declaration() || mixin.call(false, false) ||
                            this.ruleset() || this.variableCall() || this.entities.call() || this.atrule();
                        if (node) {
                            root.push(node);
                        }
                        else {
                            var foundSemiColon = false;
                            while (parserInput.$char(';')) {
                                foundSemiColon = true;
                            }
                            if (!foundSemiColon) {
                                break;
                            }
                        }
                    }
                    return root;
                },
                // comments are collected by the main parsing mechanism and then assigned to nodes
                // where the current structure allows it
                comment: function () {
                    if (parserInput.commentStore.length) {
                        var comment = parserInput.commentStore.shift();
                        return new (tree.Comment)(comment.text, comment.isLineComment, comment.index, fileInfo);
                    }
                },
                //
                // Entities are tokens which can be found inside an Expression
                //
                entities: {
                    mixinLookup: function () {
                        return parsers.mixin.call(true, true);
                    },
                    //
                    // A string, which supports escaping " and '
                    //
                    //     "milky way" 'he\'s the one!'
                    //
                    quoted: function (forceEscaped) {
                        var str;
                        var index = parserInput.i;
                        var isEscaped = false;
                        parserInput.save();
                        if (parserInput.$char('~')) {
                            isEscaped = true;
                        }
                        else if (forceEscaped) {
                            parserInput.restore();
                            return;
                        }
                        str = parserInput.$quoted();
                        if (!str) {
                            parserInput.restore();
                            return;
                        }
                        parserInput.forget();
                        return new (tree.Quoted)(str.charAt(0), str.substr(1, str.length - 2), isEscaped, index, fileInfo);
                    },
                    //
                    // A catch-all word, such as:
                    //
                    //     black border-collapse
                    //
                    keyword: function () {
                        var k = parserInput.$char('%') || parserInput.$re(/^\[?(?:[\w-]|\\(?:[A-Fa-f0-9]{1,6} ?|[^A-Fa-f0-9]))+\]?/);
                        if (k) {
                            return tree.Color.fromKeyword(k) || new (tree.Keyword)(k);
                        }
                    },
                    //
                    // A function call
                    //
                    //     rgb(255, 0, 255)
                    //
                    // The arguments are parsed with the `entities.arguments` parser.
                    //
                    call: function () {
                        var name;
                        var args;
                        var func;
                        var index = parserInput.i;
                        // http://jsperf.com/case-insensitive-regex-vs-strtolower-then-regex/18
                        if (parserInput.peek(/^url\(/i)) {
                            return;
                        }
                        parserInput.save();
                        name = parserInput.$re(/^([\w-]+|%|progid:[\w\.]+)\(/);
                        if (!name) {
                            parserInput.forget();
                            return;
                        }
                        name = name[1];
                        func = this.customFuncCall(name);
                        if (func) {
                            args = func.parse();
                            if (args && func.stop) {
                                parserInput.forget();
                                return args;
                            }
                        }
                        args = this.arguments(args);
                        if (!parserInput.$char(')')) {
                            parserInput.restore('Could not parse call arguments or missing \')\'');
                            return;
                        }
                        parserInput.forget();
                        return new (tree.Call)(name, args, index, fileInfo);
                    },
                    //
                    // Parsing rules for functions with non-standard args, e.g.:
                    //
                    //     boolean(not(2 > 1))
                    //
                    //     This is a quick prototype, to be modified/improved when
                    //     more custom-parsed funcs come (e.g. `selector(...)`)
                    //
                    customFuncCall: function (name) {
                        /* Ideally the table is to be moved out of here for faster perf.,
                           but it's quite tricky since it relies on all these `parsers`
                           and `expect` available only here */
                        return {
                            alpha: f(parsers.ieAlpha, true),
                            boolean: f(condition),
                            'if': f(condition)
                        }[name.toLowerCase()];
                        function f(parse, stop) {
                            return {
                                parse: parse,
                                stop: stop // when true - stop after parse() and return its result, 
                                // otherwise continue for plain args
                            };
                        }
                        function condition() {
                            return [expect(parsers.condition, 'expected condition')];
                        }
                    },
                    arguments: function (prevArgs) {
                        var argsComma = prevArgs || [];
                        var argsSemiColon = [];
                        var isSemiColonSeparated;
                        var value;
                        parserInput.save();
                        while (true) {
                            if (prevArgs) {
                                prevArgs = false;
                            }
                            else {
                                value = parsers.detachedRuleset() || this.assignment() || parsers.expression();
                                if (!value) {
                                    break;
                                }
                                if (value.value && value.value.length == 1) {
                                    value = value.value[0];
                                }
                                argsComma.push(value);
                            }
                            if (parserInput.$char(',')) {
                                continue;
                            }
                            if (parserInput.$char(';') || isSemiColonSeparated) {
                                isSemiColonSeparated = true;
                                value = (argsComma.length < 1) ? argsComma[0]
                                    : new tree.Value(argsComma);
                                argsSemiColon.push(value);
                                argsComma = [];
                            }
                        }
                        parserInput.forget();
                        return isSemiColonSeparated ? argsSemiColon : argsComma;
                    },
                    literal: function () {
                        return this.dimension() ||
                            this.color() ||
                            this.quoted() ||
                            this.unicodeDescriptor();
                    },
                    // Assignments are argument entities for calls.
                    // They are present in ie filter properties as shown below.
                    //
                    //     filter: progid:DXImageTransform.Microsoft.Alpha( *opacity=50* )
                    //
                    assignment: function () {
                        var key;
                        var value;
                        parserInput.save();
                        key = parserInput.$re(/^\w+(?=\s?=)/i);
                        if (!key) {
                            parserInput.restore();
                            return;
                        }
                        if (!parserInput.$char('=')) {
                            parserInput.restore();
                            return;
                        }
                        value = parsers.entity();
                        if (value) {
                            parserInput.forget();
                            return new (tree.Assignment)(key, value);
                        }
                        else {
                            parserInput.restore();
                        }
                    },
                    //
                    // Parse url() tokens
                    //
                    // We use a specific rule for urls, because they don't really behave like
                    // standard function calls. The difference is that the argument doesn't have
                    // to be enclosed within a string, so it can't be parsed as an Expression.
                    //
                    url: function () {
                        var value;
                        var index = parserInput.i;
                        parserInput.autoCommentAbsorb = false;
                        if (!parserInput.$str('url(')) {
                            parserInput.autoCommentAbsorb = true;
                            return;
                        }
                        value = this.quoted() || this.variable() || this.property() ||
                            parserInput.$re(/^(?:(?:\\[\(\)'"])|[^\(\)'"])+/) || '';
                        parserInput.autoCommentAbsorb = true;
                        expectChar(')');
                        return new (tree.URL)((value.value != null ||
                            value instanceof tree.Variable ||
                            value instanceof tree.Property) ?
                            value : new (tree.Anonymous)(value, index), index, fileInfo);
                    },
                    //
                    // A Variable entity, such as `@fink`, in
                    //
                    //     width: @fink + 2px
                    //
                    // We use a different parser for variable definitions,
                    // see `parsers.variable`.
                    //
                    variable: function () {
                        var ch;
                        var name;
                        var index = parserInput.i;
                        parserInput.save();
                        if (parserInput.currentChar() === '@' && (name = parserInput.$re(/^@@?[\w-]+/))) {
                            ch = parserInput.currentChar();
                            if (ch === '(' || ch === '[' && !parserInput.prevChar().match(/^\s/)) {
                                // this may be a VariableCall lookup
                                var result = parsers.variableCall(name);
                                if (result) {
                                    parserInput.forget();
                                    return result;
                                }
                            }
                            parserInput.forget();
                            return new (tree.Variable)(name, index, fileInfo);
                        }
                        parserInput.restore();
                    },
                    // A variable entity using the protective {} e.g. @{var}
                    variableCurly: function () {
                        var curly;
                        var index = parserInput.i;
                        if (parserInput.currentChar() === '@' && (curly = parserInput.$re(/^@\{([\w-]+)\}/))) {
                            return new (tree.Variable)("@" + curly[1], index, fileInfo);
                        }
                    },
                    //
                    // A Property accessor, such as `$color`, in
                    //
                    //     background-color: $color
                    //
                    property: function () {
                        var name;
                        var index = parserInput.i;
                        if (parserInput.currentChar() === '$' && (name = parserInput.$re(/^\$[\w-]+/))) {
                            return new (tree.Property)(name, index, fileInfo);
                        }
                    },
                    // A property entity useing the protective {} e.g. ${prop}
                    propertyCurly: function () {
                        var curly;
                        var index = parserInput.i;
                        if (parserInput.currentChar() === '$' && (curly = parserInput.$re(/^\$\{([\w-]+)\}/))) {
                            return new (tree.Property)("$" + curly[1], index, fileInfo);
                        }
                    },
                    //
                    // A Hexadecimal color
                    //
                    //     #4F3C2F
                    //
                    // `rgb` and `hsl` colors are parsed through the `entities.call` parser.
                    //
                    color: function () {
                        var rgb;
                        parserInput.save();
                        if (parserInput.currentChar() === '#' && (rgb = parserInput.$re(/^#([A-Fa-f0-9]{8}|[A-Fa-f0-9]{6}|[A-Fa-f0-9]{3,4})([\w.#\[])?/))) {
                            if (!rgb[2]) {
                                parserInput.forget();
                                return new (tree.Color)(rgb[1], undefined, rgb[0]);
                            }
                        }
                        parserInput.restore();
                    },
                    colorKeyword: function () {
                        parserInput.save();
                        var autoCommentAbsorb = parserInput.autoCommentAbsorb;
                        parserInput.autoCommentAbsorb = false;
                        var k = parserInput.$re(/^[_A-Za-z-][_A-Za-z0-9-]+/);
                        parserInput.autoCommentAbsorb = autoCommentAbsorb;
                        if (!k) {
                            parserInput.forget();
                            return;
                        }
                        parserInput.restore();
                        var color = tree.Color.fromKeyword(k);
                        if (color) {
                            parserInput.$str(k);
                            return color;
                        }
                    },
                    //
                    // A Dimension, that is, a number and a unit
                    //
                    //     0.5em 95%
                    //
                    dimension: function () {
                        if (parserInput.peekNotNumeric()) {
                            return;
                        }
                        var value = parserInput.$re(/^([+-]?\d*\.?\d+)(%|[a-z_]+)?/i);
                        if (value) {
                            return new (tree.Dimension)(value[1], value[2]);
                        }
                    },
                    //
                    // A unicode descriptor, as is used in unicode-range
                    //
                    // U+0??  or U+00A1-00A9
                    //
                    unicodeDescriptor: function () {
                        var ud;
                        ud = parserInput.$re(/^U\+[0-9a-fA-F?]+(\-[0-9a-fA-F?]+)?/);
                        if (ud) {
                            return new (tree.UnicodeDescriptor)(ud[0]);
                        }
                    },
                    //
                    // JavaScript code to be evaluated
                    //
                    //     `window.location.href`
                    //
                    javascript: function () {
                        var js;
                        var index = parserInput.i;
                        parserInput.save();
                        var escape = parserInput.$char('~');
                        var jsQuote = parserInput.$char('`');
                        if (!jsQuote) {
                            parserInput.restore();
                            return;
                        }
                        js = parserInput.$re(/^[^`]*`/);
                        if (js) {
                            parserInput.forget();
                            return new (tree.JavaScript)(js.substr(0, js.length - 1), Boolean(escape), index, fileInfo);
                        }
                        parserInput.restore('invalid javascript definition');
                    }
                },
                //
                // The variable part of a variable definition. Used in the `rule` parser
                //
                //     @fink:
                //
                variable: function () {
                    var name;
                    if (parserInput.currentChar() === '@' && (name = parserInput.$re(/^(@[\w-]+)\s*:/))) {
                        return name[1];
                    }
                },
                //
                // Call a variable value to retrieve a detached ruleset
                // or a value from a detached ruleset's rules.
                //
                //     @fink();
                //     @fink;
                //     color: @fink[@color];
                //
                variableCall: function (parsedName) {
                    var lookups;
                    var i = parserInput.i;
                    var inValue = !!parsedName;
                    var name = parsedName;
                    parserInput.save();
                    if (name || (parserInput.currentChar() === '@'
                        && (name = parserInput.$re(/^(@[\w-]+)(\(\s*\))?/)))) {
                        lookups = this.mixin.ruleLookups();
                        if (!lookups && ((inValue && parserInput.$str('()') !== '()') || (name[2] !== '()'))) {
                            parserInput.restore('Missing \'[...]\' lookup in variable call');
                            return;
                        }
                        if (!inValue) {
                            name = name[1];
                        }
                        var call = new tree.VariableCall(name, i, fileInfo);
                        if (!inValue && parsers.end()) {
                            parserInput.forget();
                            return call;
                        }
                        else {
                            parserInput.forget();
                            return new tree.NamespaceValue(call, lookups, i, fileInfo);
                        }
                    }
                    parserInput.restore();
                },
                //
                // extend syntax - used to extend selectors
                //
                extend: function (isRule) {
                    var elements;
                    var e;
                    var index = parserInput.i;
                    var option;
                    var extendList;
                    var extend;
                    if (!parserInput.$str(isRule ? '&:extend(' : ':extend(')) {
                        return;
                    }
                    do {
                        option = null;
                        elements = null;
                        while (!(option = parserInput.$re(/^(all)(?=\s*(\)|,))/))) {
                            e = this.element();
                            if (!e) {
                                break;
                            }
                            if (elements) {
                                elements.push(e);
                            }
                            else {
                                elements = [e];
                            }
                        }
                        option = option && option[1];
                        if (!elements) {
                            error('Missing target selector for :extend().');
                        }
                        extend = new (tree.Extend)(new (tree.Selector)(elements), option, index, fileInfo);
                        if (extendList) {
                            extendList.push(extend);
                        }
                        else {
                            extendList = [extend];
                        }
                    } while (parserInput.$char(','));
                    expect(/^\)/);
                    if (isRule) {
                        expect(/^;/);
                    }
                    return extendList;
                },
                //
                // extendRule - used in a rule to extend all the parent selectors
                //
                extendRule: function () {
                    return this.extend(true);
                },
                //
                // Mixins
                //
                mixin: {
                    //
                    // A Mixin call, with an optional argument list
                    //
                    //     #mixins > .square(#fff);
                    //     #mixins.square(#fff);
                    //     .rounded(4px, black);
                    //     .button;
                    //
                    // We can lookup / return a value using the lookup syntax:
                    //
                    //     color: #mixin.square(#fff)[@color];
                    //
                    // The `while` loop is there because mixins can be
                    // namespaced, but we only support the child and descendant
                    // selector for now.
                    //
                    call: function (inValue, getLookup) {
                        var s = parserInput.currentChar();
                        var important = false;
                        var lookups;
                        var index = parserInput.i;
                        var elements;
                        var args;
                        var hasParens;
                        if (s !== '.' && s !== '#') {
                            return;
                        }
                        parserInput.save(); // stop us absorbing part of an invalid selector
                        elements = this.elements();
                        if (elements) {
                            if (parserInput.$char('(')) {
                                args = this.args(true).args;
                                expectChar(')');
                                hasParens = true;
                            }
                            if (getLookup !== false) {
                                lookups = this.ruleLookups();
                            }
                            if (getLookup === true && !lookups) {
                                parserInput.restore();
                                return;
                            }
                            if (inValue && !lookups && !hasParens) {
                                // This isn't a valid in-value mixin call
                                parserInput.restore();
                                return;
                            }
                            if (!inValue && parsers.important()) {
                                important = true;
                            }
                            if (inValue || parsers.end()) {
                                parserInput.forget();
                                var mixin = new (tree.mixin.Call)(elements, args, index, fileInfo, !lookups && important);
                                if (lookups) {
                                    return new tree.NamespaceValue(mixin, lookups);
                                }
                                else {
                                    return mixin;
                                }
                            }
                        }
                        parserInput.restore();
                    },
                    /**
                     * Matching elements for mixins
                     * (Start with . or # and can have > )
                     */
                    elements: function () {
                        var elements;
                        var e;
                        var c;
                        var elem;
                        var elemIndex;
                        var re = /^[#.](?:[\w-]|\\(?:[A-Fa-f0-9]{1,6} ?|[^A-Fa-f0-9]))+/;
                        while (true) {
                            elemIndex = parserInput.i;
                            e = parserInput.$re(re);
                            if (!e) {
                                break;
                            }
                            elem = new (tree.Element)(c, e, false, elemIndex, fileInfo);
                            if (elements) {
                                elements.push(elem);
                            }
                            else {
                                elements = [elem];
                            }
                            c = parserInput.$char('>');
                        }
                        return elements;
                    },
                    args: function (isCall) {
                        var entities = parsers.entities;
                        var returner = { args: null, variadic: false };
                        var expressions = [];
                        var argsSemiColon = [];
                        var argsComma = [];
                        var isSemiColonSeparated;
                        var expressionContainsNamed;
                        var name;
                        var nameLoop;
                        var value;
                        var arg;
                        var expand;
                        var hasSep = true;
                        parserInput.save();
                        while (true) {
                            if (isCall) {
                                arg = parsers.detachedRuleset() || parsers.expression();
                            }
                            else {
                                parserInput.commentStore.length = 0;
                                if (parserInput.$str('...')) {
                                    returner.variadic = true;
                                    if (parserInput.$char(';') && !isSemiColonSeparated) {
                                        isSemiColonSeparated = true;
                                    }
                                    (isSemiColonSeparated ? argsSemiColon : argsComma)
                                        .push({ variadic: true });
                                    break;
                                }
                                arg = entities.variable() || entities.property() || entities.literal() || entities.keyword() || this.call(true);
                            }
                            if (!arg || !hasSep) {
                                break;
                            }
                            nameLoop = null;
                            if (arg.throwAwayComments) {
                                arg.throwAwayComments();
                            }
                            value = arg;
                            var val = null;
                            if (isCall) {
                                // Variable
                                if (arg.value && arg.value.length == 1) {
                                    val = arg.value[0];
                                }
                            }
                            else {
                                val = arg;
                            }
                            if (val && (val instanceof tree.Variable || val instanceof tree.Property)) {
                                if (parserInput.$char(':')) {
                                    if (expressions.length > 0) {
                                        if (isSemiColonSeparated) {
                                            error('Cannot mix ; and , as delimiter types');
                                        }
                                        expressionContainsNamed = true;
                                    }
                                    value = parsers.detachedRuleset() || parsers.expression();
                                    if (!value) {
                                        if (isCall) {
                                            error('could not understand value for named argument');
                                        }
                                        else {
                                            parserInput.restore();
                                            returner.args = [];
                                            return returner;
                                        }
                                    }
                                    nameLoop = (name = val.name);
                                }
                                else if (parserInput.$str('...')) {
                                    if (!isCall) {
                                        returner.variadic = true;
                                        if (parserInput.$char(';') && !isSemiColonSeparated) {
                                            isSemiColonSeparated = true;
                                        }
                                        (isSemiColonSeparated ? argsSemiColon : argsComma)
                                            .push({ name: arg.name, variadic: true });
                                        break;
                                    }
                                    else {
                                        expand = true;
                                    }
                                }
                                else if (!isCall) {
                                    name = nameLoop = val.name;
                                    value = null;
                                }
                            }
                            if (value) {
                                expressions.push(value);
                            }
                            argsComma.push({ name: nameLoop, value: value, expand: expand });
                            if (parserInput.$char(',')) {
                                hasSep = true;
                                continue;
                            }
                            hasSep = parserInput.$char(';') === ';';
                            if (hasSep || isSemiColonSeparated) {
                                if (expressionContainsNamed) {
                                    error('Cannot mix ; and , as delimiter types');
                                }
                                isSemiColonSeparated = true;
                                if (expressions.length > 1) {
                                    value = new (tree.Value)(expressions);
                                }
                                argsSemiColon.push({ name: name, value: value, expand: expand });
                                name = null;
                                expressions = [];
                                expressionContainsNamed = false;
                            }
                        }
                        parserInput.forget();
                        returner.args = isSemiColonSeparated ? argsSemiColon : argsComma;
                        return returner;
                    },
                    //
                    // A Mixin definition, with a list of parameters
                    //
                    //     .rounded (@radius: 2px, @color) {
                    //        ...
                    //     }
                    //
                    // Until we have a finer grained state-machine, we have to
                    // do a look-ahead, to make sure we don't have a mixin call.
                    // See the `rule` function for more information.
                    //
                    // We start by matching `.rounded (`, and then proceed on to
                    // the argument list, which has optional default values.
                    // We store the parameters in `params`, with a `value` key,
                    // if there is a value, such as in the case of `@radius`.
                    //
                    // Once we've got our params list, and a closing `)`, we parse
                    // the `{...}` block.
                    //
                    definition: function () {
                        var name;
                        var params = [];
                        var match;
                        var ruleset;
                        var cond;
                        var variadic = false;
                        if ((parserInput.currentChar() !== '.' && parserInput.currentChar() !== '#') ||
                            parserInput.peek(/^[^{]*\}/)) {
                            return;
                        }
                        parserInput.save();
                        match = parserInput.$re(/^([#.](?:[\w-]|\\(?:[A-Fa-f0-9]{1,6} ?|[^A-Fa-f0-9]))+)\s*\(/);
                        if (match) {
                            name = match[1];
                            var argInfo = this.args(false);
                            params = argInfo.args;
                            variadic = argInfo.variadic;
                            // .mixincall("@{a}");
                            // looks a bit like a mixin definition..
                            // also
                            // .mixincall(@a: {rule: set;});
                            // so we have to be nice and restore
                            if (!parserInput.$char(')')) {
                                parserInput.restore('Missing closing \')\'');
                                return;
                            }
                            parserInput.commentStore.length = 0;
                            if (parserInput.$str('when')) { // Guard
                                cond = expect(parsers.conditions, 'expected condition');
                            }
                            ruleset = parsers.block();
                            if (ruleset) {
                                parserInput.forget();
                                return new (tree.mixin.Definition)(name, params, ruleset, cond, variadic);
                            }
                            else {
                                parserInput.restore();
                            }
                        }
                        else {
                            parserInput.restore();
                        }
                    },
                    ruleLookups: function () {
                        var rule;
                        var lookups = [];
                        if (parserInput.currentChar() !== '[') {
                            return;
                        }
                        while (true) {
                            parserInput.save();
                            rule = this.lookupValue();
                            if (!rule && rule !== '') {
                                parserInput.restore();
                                break;
                            }
                            lookups.push(rule);
                            parserInput.forget();
                        }
                        if (lookups.length > 0) {
                            return lookups;
                        }
                    },
                    lookupValue: function () {
                        parserInput.save();
                        if (!parserInput.$char('[')) {
                            parserInput.restore();
                            return;
                        }
                        var name = parserInput.$re(/^(?:[@$]{0,2})[_a-zA-Z0-9-]*/);
                        if (!parserInput.$char(']')) {
                            parserInput.restore();
                            return;
                        }
                        if (name || name === '') {
                            parserInput.forget();
                            return name;
                        }
                        parserInput.restore();
                    }
                },
                //
                // Entities are the smallest recognized token,
                // and can be found inside a rule's value.
                //
                entity: function () {
                    var entities = this.entities;
                    return this.comment() || entities.literal() || entities.variable() || entities.url() ||
                        entities.property() || entities.call() || entities.keyword() || this.mixin.call(true) ||
                        entities.javascript();
                },
                //
                // A Declaration terminator. Note that we use `peek()` to check for '}',
                // because the `block` rule will be expecting it, but we still need to make sure
                // it's there, if ';' was omitted.
                //
                end: function () {
                    return parserInput.$char(';') || parserInput.peek('}');
                },
                //
                // IE's alpha function
                //
                //     alpha(opacity=88)
                //
                ieAlpha: function () {
                    var value;
                    // http://jsperf.com/case-insensitive-regex-vs-strtolower-then-regex/18
                    if (!parserInput.$re(/^opacity=/i)) {
                        return;
                    }
                    value = parserInput.$re(/^\d+/);
                    if (!value) {
                        value = expect(parsers.entities.variable, 'Could not parse alpha');
                        value = "@{" + value.name.slice(1) + "}";
                    }
                    expectChar(')');
                    return new tree.Quoted('', "alpha(opacity=" + value + ")");
                },
                //
                // A Selector Element
                //
                //     div
                //     + h1
                //     #socks
                //     input[type="text"]
                //
                // Elements are the building blocks for Selectors,
                // they are made out of a `Combinator` (see combinator rule),
                // and an element name, such as a tag a class, or `*`.
                //
                element: function () {
                    var e;
                    var c;
                    var v;
                    var index = parserInput.i;
                    c = this.combinator();
                    e = parserInput.$re(/^(?:\d+\.\d+|\d+)%/) ||
                        parserInput.$re(/^(?:[.#]?|:*)(?:[\w-]|[^\x00-\x9f]|\\(?:[A-Fa-f0-9]{1,6} ?|[^A-Fa-f0-9]))+/) ||
                        parserInput.$char('*') || parserInput.$char('&') || this.attribute() ||
                        parserInput.$re(/^\([^&()@]+\)/) || parserInput.$re(/^[\.#:](?=@)/) ||
                        this.entities.variableCurly();
                    if (!e) {
                        parserInput.save();
                        if (parserInput.$char('(')) {
                            if ((v = this.selector(false)) && parserInput.$char(')')) {
                                e = new (tree.Paren)(v);
                                parserInput.forget();
                            }
                            else {
                                parserInput.restore('Missing closing \')\'');
                            }
                        }
                        else {
                            parserInput.forget();
                        }
                    }
                    if (e) {
                        return new (tree.Element)(c, e, e instanceof tree.Variable, index, fileInfo);
                    }
                },
                //
                // Combinators combine elements together, in a Selector.
                //
                // Because our parser isn't white-space sensitive, special care
                // has to be taken, when parsing the descendant combinator, ` `,
                // as it's an empty space. We have to check the previous character
                // in the input, to see if it's a ` ` character. More info on how
                // we deal with this in *combinator.js*.
                //
                combinator: function () {
                    var c = parserInput.currentChar();
                    if (c === '/') {
                        parserInput.save();
                        var slashedCombinator = parserInput.$re(/^\/[a-z]+\//i);
                        if (slashedCombinator) {
                            parserInput.forget();
                            return new (tree.Combinator)(slashedCombinator);
                        }
                        parserInput.restore();
                    }
                    if (c === '>' || c === '+' || c === '~' || c === '|' || c === '^') {
                        parserInput.i++;
                        if (c === '^' && parserInput.currentChar() === '^') {
                            c = '^^';
                            parserInput.i++;
                        }
                        while (parserInput.isWhitespace()) {
                            parserInput.i++;
                        }
                        return new (tree.Combinator)(c);
                    }
                    else if (parserInput.isWhitespace(-1)) {
                        return new (tree.Combinator)(' ');
                    }
                    else {
                        return new (tree.Combinator)(null);
                    }
                },
                //
                // A CSS Selector
                // with less extensions e.g. the ability to extend and guard
                //
                //     .class > div + h1
                //     li a:hover
                //
                // Selectors are made out of one or more Elements, see above.
                //
                selector: function (isLess) {
                    var index = parserInput.i;
                    var elements;
                    var extendList;
                    var c;
                    var e;
                    var allExtends;
                    var when;
                    var condition;
                    isLess = isLess !== false;
                    while ((isLess && (extendList = this.extend())) || (isLess && (when = parserInput.$str('when'))) || (e = this.element())) {
                        if (when) {
                            condition = expect(this.conditions, 'expected condition');
                        }
                        else if (condition) {
                            error('CSS guard can only be used at the end of selector');
                        }
                        else if (extendList) {
                            if (allExtends) {
                                allExtends = allExtends.concat(extendList);
                            }
                            else {
                                allExtends = extendList;
                            }
                        }
                        else {
                            if (allExtends) {
                                error('Extend can only be used at the end of selector');
                            }
                            c = parserInput.currentChar();
                            if (elements) {
                                elements.push(e);
                            }
                            else {
                                elements = [e];
                            }
                            e = null;
                        }
                        if (c === '{' || c === '}' || c === ';' || c === ',' || c === ')') {
                            break;
                        }
                    }
                    if (elements) {
                        return new (tree.Selector)(elements, allExtends, condition, index, fileInfo);
                    }
                    if (allExtends) {
                        error('Extend must be used to extend a selector, it cannot be used on its own');
                    }
                },
                selectors: function () {
                    var s;
                    var selectors;
                    while (true) {
                        s = this.selector();
                        if (!s) {
                            break;
                        }
                        if (selectors) {
                            selectors.push(s);
                        }
                        else {
                            selectors = [s];
                        }
                        parserInput.commentStore.length = 0;
                        if (s.condition && selectors.length > 1) {
                            error("Guards are only currently allowed on a single selector.");
                        }
                        if (!parserInput.$char(',')) {
                            break;
                        }
                        if (s.condition) {
                            error("Guards are only currently allowed on a single selector.");
                        }
                        parserInput.commentStore.length = 0;
                    }
                    return selectors;
                },
                attribute: function () {
                    if (!parserInput.$char('[')) {
                        return;
                    }
                    var entities = this.entities;
                    var key;
                    var val;
                    var op;
                    if (!(key = entities.variableCurly())) {
                        key = expect(/^(?:[_A-Za-z0-9-\*]*\|)?(?:[_A-Za-z0-9-]|\\.)+/);
                    }
                    op = parserInput.$re(/^[|~*$^]?=/);
                    if (op) {
                        val = entities.quoted() || parserInput.$re(/^[0-9]+%/) || parserInput.$re(/^[\w-]+/) || entities.variableCurly();
                    }
                    expectChar(']');
                    return new (tree.Attribute)(key, op, val);
                },
                //
                // The `block` rule is used by `ruleset` and `mixin.definition`.
                // It's a wrapper around the `primary` rule, with added `{}`.
                //
                block: function () {
                    var content;
                    if (parserInput.$char('{') && (content = this.primary()) && parserInput.$char('}')) {
                        return content;
                    }
                },
                blockRuleset: function () {
                    var block = this.block();
                    if (block) {
                        block = new tree.Ruleset(null, block);
                    }
                    return block;
                },
                detachedRuleset: function () {
                    var argInfo;
                    var params;
                    var variadic;
                    parserInput.save();
                    if (parserInput.$re(/^[.#]\(/)) {
                        /**
                         * DR args currently only implemented for each() function, and not
                         * yet settable as `@dr: #(@arg) {}`
                         * This should be done when DRs are merged with mixins.
                         * See: https://github.com/less/less-meta/issues/16
                         */
                        argInfo = this.mixin.args(false);
                        params = argInfo.args;
                        variadic = argInfo.variadic;
                        if (!parserInput.$char(')')) {
                            parserInput.restore();
                            return;
                        }
                    }
                    var blockRuleset = this.blockRuleset();
                    if (blockRuleset) {
                        parserInput.forget();
                        if (params) {
                            return new tree.mixin.Definition(null, params, blockRuleset, null, variadic);
                        }
                        return new tree.DetachedRuleset(blockRuleset);
                    }
                    parserInput.restore();
                },
                //
                // div, .class, body > p {...}
                //
                ruleset: function () {
                    var selectors;
                    var rules;
                    var debugInfo;
                    parserInput.save();
                    if (context.dumpLineNumbers) {
                        debugInfo = getDebugInfo(parserInput.i);
                    }
                    selectors = this.selectors();
                    if (selectors && (rules = this.block())) {
                        parserInput.forget();
                        var ruleset = new (tree.Ruleset)(selectors, rules, context.strictImports);
                        if (context.dumpLineNumbers) {
                            ruleset.debugInfo = debugInfo;
                        }
                        return ruleset;
                    }
                    else {
                        parserInput.restore();
                    }
                },
                declaration: function () {
                    var name;
                    var value;
                    var index = parserInput.i;
                    var hasDR;
                    var c = parserInput.currentChar();
                    var important;
                    var merge;
                    var isVariable;
                    if (c === '.' || c === '#' || c === '&' || c === ':') {
                        return;
                    }
                    parserInput.save();
                    name = this.variable() || this.ruleProperty();
                    if (name) {
                        isVariable = typeof name === 'string';
                        if (isVariable) {
                            value = this.detachedRuleset();
                            if (value) {
                                hasDR = true;
                            }
                        }
                        parserInput.commentStore.length = 0;
                        if (!value) {
                            // a name returned by this.ruleProperty() is always an array of the form:
                            // [string-1, ..., string-n, ""] or [string-1, ..., string-n, "+"]
                            // where each item is a tree.Keyword or tree.Variable
                            merge = !isVariable && name.length > 1 && name.pop().value;
                            // Custom property values get permissive parsing
                            if (name[0].value && name[0].value.slice(0, 2) === '--') {
                                value = this.permissiveValue();
                            }
                            // Try to store values as anonymous
                            // If we need the value later we'll re-parse it in ruleset.parseValue
                            else {
                                value = this.anonymousValue();
                            }
                            if (value) {
                                parserInput.forget();
                                // anonymous values absorb the end ';' which is required for them to work
                                return new (tree.Declaration)(name, value, false, merge, index, fileInfo);
                            }
                            if (!value) {
                                value = this.value();
                            }
                            if (value) {
                                important = this.important();
                            }
                            else if (isVariable) {
                                // As a last resort, try permissiveValue
                                value = this.permissiveValue();
                            }
                        }
                        if (value && (this.end() || hasDR)) {
                            parserInput.forget();
                            return new (tree.Declaration)(name, value, important, merge, index, fileInfo);
                        }
                        else {
                            parserInput.restore();
                        }
                    }
                    else {
                        parserInput.restore();
                    }
                },
                anonymousValue: function () {
                    var index = parserInput.i;
                    var match = parserInput.$re(/^([^.#@\$+\/'"*`(;{}-]*);/);
                    if (match) {
                        return new (tree.Anonymous)(match[1], index);
                    }
                },
                /**
                 * Used for custom properties, at-rules, and variables (as fallback)
                 * Parses almost anything inside of {} [] () "" blocks
                 * until it reaches outer-most tokens.
                 *
                 * First, it will try to parse comments and entities to reach
                 * the end. This is mostly like the Expression parser except no
                 * math is allowed.
                 */
                permissiveValue: function (untilTokens) {
                    var i;
                    var e;
                    var done;
                    var value;
                    var tok = untilTokens || ';';
                    var index = parserInput.i;
                    var result = [];
                    function testCurrentChar() {
                        var char = parserInput.currentChar();
                        if (typeof tok === 'string') {
                            return char === tok;
                        }
                        else {
                            return tok.test(char);
                        }
                    }
                    if (testCurrentChar()) {
                        return;
                    }
                    value = [];
                    do {
                        e = this.comment();
                        if (e) {
                            value.push(e);
                            continue;
                        }
                        e = this.entity();
                        if (e) {
                            value.push(e);
                        }
                    } while (e);
                    done = testCurrentChar();
                    if (value.length > 0) {
                        value = new (tree.Expression)(value);
                        if (done) {
                            return value;
                        }
                        else {
                            result.push(value);
                        }
                        // Preserve space before $parseUntil as it will not
                        if (parserInput.prevChar() === ' ') {
                            result.push(new tree.Anonymous(' ', index));
                        }
                    }
                    parserInput.save();
                    value = parserInput.$parseUntil(tok);
                    if (value) {
                        if (typeof value === 'string') {
                            error("Expected '" + value + "'", 'Parse');
                        }
                        if (value.length === 1 && value[0] === ' ') {
                            parserInput.forget();
                            return new tree.Anonymous('', index);
                        }
                        var item = void 0;
                        for (i = 0; i < value.length; i++) {
                            item = value[i];
                            if (Array.isArray(item)) {
                                // Treat actual quotes as normal quoted values
                                result.push(new tree.Quoted(item[0], item[1], true, index, fileInfo));
                            }
                            else {
                                if (i === value.length - 1) {
                                    item = item.trim();
                                }
                                // Treat like quoted values, but replace vars like unquoted expressions
                                var quote = new tree.Quoted('\'', item, true, index, fileInfo);
                                quote.variableRegex = /@([\w-]+)/g;
                                quote.propRegex = /\$([\w-]+)/g;
                                result.push(quote);
                            }
                        }
                        parserInput.forget();
                        return new tree.Expression(result, true);
                    }
                    parserInput.restore();
                },
                //
                // An @import atrule
                //
                //     @import "lib";
                //
                // Depending on our environment, importing is done differently:
                // In the browser, it's an XHR request, in Node, it would be a
                // file-system operation. The function used for importing is
                // stored in `import`, which we pass to the Import constructor.
                //
                'import': function () {
                    var path;
                    var features;
                    var index = parserInput.i;
                    var dir = parserInput.$re(/^@import?\s+/);
                    if (dir) {
                        var options_1 = (dir ? this.importOptions() : null) || {};
                        if ((path = this.entities.quoted() || this.entities.url())) {
                            features = this.mediaFeatures();
                            if (!parserInput.$char(';')) {
                                parserInput.i = index;
                                error('missing semi-colon or unrecognised media features on import');
                            }
                            features = features && new (tree.Value)(features);
                            return new (tree.Import)(path, features, options_1, index, fileInfo);
                        }
                        else {
                            parserInput.i = index;
                            error('malformed import statement');
                        }
                    }
                },
                importOptions: function () {
                    var o;
                    var options = {};
                    var optionName;
                    var value;
                    // list of options, surrounded by parens
                    if (!parserInput.$char('(')) {
                        return null;
                    }
                    do {
                        o = this.importOption();
                        if (o) {
                            optionName = o;
                            value = true;
                            switch (optionName) {
                                case 'css':
                                    optionName = 'less';
                                    value = false;
                                    break;
                                case 'once':
                                    optionName = 'multiple';
                                    value = false;
                                    break;
                            }
                            options[optionName] = value;
                            if (!parserInput.$char(',')) {
                                break;
                            }
                        }
                    } while (o);
                    expectChar(')');
                    return options;
                },
                importOption: function () {
                    var opt = parserInput.$re(/^(less|css|multiple|once|inline|reference|optional)/);
                    if (opt) {
                        return opt[1];
                    }
                },
                mediaFeature: function () {
                    var entities = this.entities;
                    var nodes = [];
                    var e;
                    var p;
                    parserInput.save();
                    do {
                        e = entities.keyword() || entities.variable() || entities.mixinLookup();
                        if (e) {
                            nodes.push(e);
                        }
                        else if (parserInput.$char('(')) {
                            p = this.property();
                            e = this.value();
                            if (parserInput.$char(')')) {
                                if (p && e) {
                                    nodes.push(new (tree.Paren)(new (tree.Declaration)(p, e, null, null, parserInput.i, fileInfo, true)));
                                }
                                else if (e) {
                                    nodes.push(new (tree.Paren)(e));
                                }
                                else {
                                    error('badly formed media feature definition');
                                }
                            }
                            else {
                                error('Missing closing \')\'', 'Parse');
                            }
                        }
                    } while (e);
                    parserInput.forget();
                    if (nodes.length > 0) {
                        return new (tree.Expression)(nodes);
                    }
                },
                mediaFeatures: function () {
                    var entities = this.entities;
                    var features = [];
                    var e;
                    do {
                        e = this.mediaFeature();
                        if (e) {
                            features.push(e);
                            if (!parserInput.$char(',')) {
                                break;
                            }
                        }
                        else {
                            e = entities.variable() || entities.mixinLookup();
                            if (e) {
                                features.push(e);
                                if (!parserInput.$char(',')) {
                                    break;
                                }
                            }
                        }
                    } while (e);
                    return features.length > 0 ? features : null;
                },
                media: function () {
                    var features;
                    var rules;
                    var media;
                    var debugInfo;
                    var index = parserInput.i;
                    if (context.dumpLineNumbers) {
                        debugInfo = getDebugInfo(index);
                    }
                    parserInput.save();
                    if (parserInput.$str('@media')) {
                        features = this.mediaFeatures();
                        rules = this.block();
                        if (!rules) {
                            error('media definitions require block statements after any features');
                        }
                        parserInput.forget();
                        media = new (tree.Media)(rules, features, index, fileInfo);
                        if (context.dumpLineNumbers) {
                            media.debugInfo = debugInfo;
                        }
                        return media;
                    }
                    parserInput.restore();
                },
                //
                // A @plugin directive, used to import plugins dynamically.
                //
                //     @plugin (args) "lib";
                //
                plugin: function () {
                    var path;
                    var args;
                    var options;
                    var index = parserInput.i;
                    var dir = parserInput.$re(/^@plugin?\s+/);
                    if (dir) {
                        args = this.pluginArgs();
                        if (args) {
                            options = {
                                pluginArgs: args,
                                isPlugin: true
                            };
                        }
                        else {
                            options = { isPlugin: true };
                        }
                        if ((path = this.entities.quoted() || this.entities.url())) {
                            if (!parserInput.$char(';')) {
                                parserInput.i = index;
                                error('missing semi-colon on @plugin');
                            }
                            return new (tree.Import)(path, null, options, index, fileInfo);
                        }
                        else {
                            parserInput.i = index;
                            error('malformed @plugin statement');
                        }
                    }
                },
                pluginArgs: function () {
                    // list of options, surrounded by parens
                    parserInput.save();
                    if (!parserInput.$char('(')) {
                        parserInput.restore();
                        return null;
                    }
                    var args = parserInput.$re(/^\s*([^\);]+)\)\s*/);
                    if (args[1]) {
                        parserInput.forget();
                        return args[1].trim();
                    }
                    else {
                        parserInput.restore();
                        return null;
                    }
                },
                //
                // A CSS AtRule
                //
                //     @charset "utf-8";
                //
                atrule: function () {
                    var index = parserInput.i;
                    var name;
                    var value;
                    var rules;
                    var nonVendorSpecificName;
                    var hasIdentifier;
                    var hasExpression;
                    var hasUnknown;
                    var hasBlock = true;
                    var isRooted = true;
                    if (parserInput.currentChar() !== '@') {
                        return;
                    }
                    value = this['import']() || this.plugin() || this.media();
                    if (value) {
                        return value;
                    }
                    parserInput.save();
                    name = parserInput.$re(/^@[a-z-]+/);
                    if (!name) {
                        return;
                    }
                    nonVendorSpecificName = name;
                    if (name.charAt(1) == '-' && name.indexOf('-', 2) > 0) {
                        nonVendorSpecificName = "@" + name.slice(name.indexOf('-', 2) + 1);
                    }
                    switch (nonVendorSpecificName) {
                        case '@charset':
                            hasIdentifier = true;
                            hasBlock = false;
                            break;
                        case '@namespace':
                            hasExpression = true;
                            hasBlock = false;
                            break;
                        case '@keyframes':
                        case '@counter-style':
                            hasIdentifier = true;
                            break;
                        case '@document':
                        case '@supports':
                            hasUnknown = true;
                            isRooted = false;
                            break;
                        default:
                            hasUnknown = true;
                            break;
                    }
                    parserInput.commentStore.length = 0;
                    if (hasIdentifier) {
                        value = this.entity();
                        if (!value) {
                            error("expected " + name + " identifier");
                        }
                    }
                    else if (hasExpression) {
                        value = this.expression();
                        if (!value) {
                            error("expected " + name + " expression");
                        }
                    }
                    else if (hasUnknown) {
                        value = this.permissiveValue(/^[{;]/);
                        hasBlock = (parserInput.currentChar() === '{');
                        if (!value) {
                            if (!hasBlock && parserInput.currentChar() !== ';') {
                                error(name + " rule is missing block or ending semi-colon");
                            }
                        }
                        else if (!value.value) {
                            value = null;
                        }
                    }
                    if (hasBlock) {
                        rules = this.blockRuleset();
                    }
                    if (rules || (!hasBlock && value && parserInput.$char(';'))) {
                        parserInput.forget();
                        return new (tree.AtRule)(name, value, rules, index, fileInfo, context.dumpLineNumbers ? getDebugInfo(index) : null, isRooted);
                    }
                    parserInput.restore('at-rule options not recognised');
                },
                //
                // A Value is a comma-delimited list of Expressions
                //
                //     font-family: Baskerville, Georgia, serif;
                //
                // In a Rule, a Value represents everything after the `:`,
                // and before the `;`.
                //
                value: function () {
                    var e;
                    var expressions = [];
                    var index = parserInput.i;
                    do {
                        e = this.expression();
                        if (e) {
                            expressions.push(e);
                            if (!parserInput.$char(',')) {
                                break;
                            }
                        }
                    } while (e);
                    if (expressions.length > 0) {
                        return new (tree.Value)(expressions, index);
                    }
                },
                important: function () {
                    if (parserInput.currentChar() === '!') {
                        return parserInput.$re(/^! *important/);
                    }
                },
                sub: function () {
                    var a;
                    var e;
                    parserInput.save();
                    if (parserInput.$char('(')) {
                        a = this.addition();
                        if (a && parserInput.$char(')')) {
                            parserInput.forget();
                            e = new (tree.Expression)([a]);
                            e.parens = true;
                            return e;
                        }
                        parserInput.restore('Expected \')\'');
                        return;
                    }
                    parserInput.restore();
                },
                multiplication: function () {
                    var m;
                    var a;
                    var op;
                    var operation;
                    var isSpaced;
                    m = this.operand();
                    if (m) {
                        isSpaced = parserInput.isWhitespace(-1);
                        while (true) {
                            if (parserInput.peek(/^\/[*\/]/)) {
                                break;
                            }
                            parserInput.save();
                            op = parserInput.$char('/') || parserInput.$char('*') || parserInput.$str('./');
                            if (!op) {
                                parserInput.forget();
                                break;
                            }
                            a = this.operand();
                            if (!a) {
                                parserInput.restore();
                                break;
                            }
                            parserInput.forget();
                            m.parensInOp = true;
                            a.parensInOp = true;
                            operation = new (tree.Operation)(op, [operation || m, a], isSpaced);
                            isSpaced = parserInput.isWhitespace(-1);
                        }
                        return operation || m;
                    }
                },
                addition: function () {
                    var m;
                    var a;
                    var op;
                    var operation;
                    var isSpaced;
                    m = this.multiplication();
                    if (m) {
                        isSpaced = parserInput.isWhitespace(-1);
                        while (true) {
                            op = parserInput.$re(/^[-+]\s+/) || (!isSpaced && (parserInput.$char('+') || parserInput.$char('-')));
                            if (!op) {
                                break;
                            }
                            a = this.multiplication();
                            if (!a) {
                                break;
                            }
                            m.parensInOp = true;
                            a.parensInOp = true;
                            operation = new (tree.Operation)(op, [operation || m, a], isSpaced);
                            isSpaced = parserInput.isWhitespace(-1);
                        }
                        return operation || m;
                    }
                },
                conditions: function () {
                    var a;
                    var b;
                    var index = parserInput.i;
                    var condition;
                    a = this.condition(true);
                    if (a) {
                        while (true) {
                            if (!parserInput.peek(/^,\s*(not\s*)?\(/) || !parserInput.$char(',')) {
                                break;
                            }
                            b = this.condition(true);
                            if (!b) {
                                break;
                            }
                            condition = new (tree.Condition)('or', condition || a, b, index);
                        }
                        return condition || a;
                    }
                },
                condition: function (needsParens) {
                    var result;
                    var logical;
                    var next;
                    function or() {
                        return parserInput.$str('or');
                    }
                    result = this.conditionAnd(needsParens);
                    if (!result) {
                        return;
                    }
                    logical = or();
                    if (logical) {
                        next = this.condition(needsParens);
                        if (next) {
                            result = new (tree.Condition)(logical, result, next);
                        }
                        else {
                            return;
                        }
                    }
                    return result;
                },
                conditionAnd: function (needsParens) {
                    var result;
                    var logical;
                    var next;
                    var self = this;
                    function insideCondition() {
                        var cond = self.negatedCondition(needsParens) || self.parenthesisCondition(needsParens);
                        if (!cond && !needsParens) {
                            return self.atomicCondition(needsParens);
                        }
                        return cond;
                    }
                    function and() {
                        return parserInput.$str('and');
                    }
                    result = insideCondition();
                    if (!result) {
                        return;
                    }
                    logical = and();
                    if (logical) {
                        next = this.conditionAnd(needsParens);
                        if (next) {
                            result = new (tree.Condition)(logical, result, next);
                        }
                        else {
                            return;
                        }
                    }
                    return result;
                },
                negatedCondition: function (needsParens) {
                    if (parserInput.$str('not')) {
                        var result = this.parenthesisCondition(needsParens);
                        if (result) {
                            result.negate = !result.negate;
                        }
                        return result;
                    }
                },
                parenthesisCondition: function (needsParens) {
                    function tryConditionFollowedByParenthesis(me) {
                        var body;
                        parserInput.save();
                        body = me.condition(needsParens);
                        if (!body) {
                            parserInput.restore();
                            return;
                        }
                        if (!parserInput.$char(')')) {
                            parserInput.restore();
                            return;
                        }
                        parserInput.forget();
                        return body;
                    }
                    var body;
                    parserInput.save();
                    if (!parserInput.$str('(')) {
                        parserInput.restore();
                        return;
                    }
                    body = tryConditionFollowedByParenthesis(this);
                    if (body) {
                        parserInput.forget();
                        return body;
                    }
                    body = this.atomicCondition(needsParens);
                    if (!body) {
                        parserInput.restore();
                        return;
                    }
                    if (!parserInput.$char(')')) {
                        parserInput.restore("expected ')' got '" + parserInput.currentChar() + "'");
                        return;
                    }
                    parserInput.forget();
                    return body;
                },
                atomicCondition: function (needsParens) {
                    var entities = this.entities;
                    var index = parserInput.i;
                    var a;
                    var b;
                    var c;
                    var op;
                    function cond() {
                        return this.addition() || entities.keyword() || entities.quoted() || entities.mixinLookup();
                    }
                    cond = cond.bind(this);
                    a = cond();
                    if (a) {
                        if (parserInput.$char('>')) {
                            if (parserInput.$char('=')) {
                                op = '>=';
                            }
                            else {
                                op = '>';
                            }
                        }
                        else if (parserInput.$char('<')) {
                            if (parserInput.$char('=')) {
                                op = '<=';
                            }
                            else {
                                op = '<';
                            }
                        }
                        else if (parserInput.$char('=')) {
                            if (parserInput.$char('>')) {
                                op = '=>';
                            }
                            else if (parserInput.$char('<')) {
                                op = '=<';
                            }
                            else {
                                op = '=';
                            }
                        }
                        if (op) {
                            b = cond();
                            if (b) {
                                c = new (tree.Condition)(op, a, b, index, false);
                            }
                            else {
                                error('expected expression');
                            }
                        }
                        else {
                            c = new (tree.Condition)('=', a, new (tree.Keyword)('true'), index, false);
                        }
                        return c;
                    }
                },
                //
                // An operand is anything that can be part of an operation,
                // such as a Color, or a Variable
                //
                operand: function () {
                    var entities = this.entities;
                    var negate;
                    if (parserInput.peek(/^-[@\$\(]/)) {
                        negate = parserInput.$char('-');
                    }
                    var o = this.sub() || entities.dimension() ||
                        entities.color() || entities.variable() ||
                        entities.property() || entities.call() ||
                        entities.quoted(true) || entities.colorKeyword() ||
                        entities.mixinLookup();
                    if (negate) {
                        o.parensInOp = true;
                        o = new (tree.Negative)(o);
                    }
                    return o;
                },
                //
                // Expressions either represent mathematical operations,
                // or white-space delimited Entities.
                //
                //     1px solid black
                //     @var * 2
                //
                expression: function () {
                    var entities = [];
                    var e;
                    var delim;
                    var index = parserInput.i;
                    do {
                        e = this.comment();
                        if (e) {
                            entities.push(e);
                            continue;
                        }
                        e = this.addition() || this.entity();
                        if (e) {
                            entities.push(e);
                            // operations do not allow keyword "/" dimension (e.g. small/20px) so we support that here
                            if (!parserInput.peek(/^\/[\/*]/)) {
                                delim = parserInput.$char('/');
                                if (delim) {
                                    entities.push(new (tree.Anonymous)(delim, index));
                                }
                            }
                        }
                    } while (e);
                    if (entities.length > 0) {
                        return new (tree.Expression)(entities);
                    }
                },
                property: function () {
                    var name = parserInput.$re(/^(\*?-?[_a-zA-Z0-9-]+)\s*:/);
                    if (name) {
                        return name[1];
                    }
                },
                ruleProperty: function () {
                    var name = [];
                    var index = [];
                    var s;
                    var k;
                    parserInput.save();
                    var simpleProperty = parserInput.$re(/^([_a-zA-Z0-9-]+)\s*:/);
                    if (simpleProperty) {
                        name = [new (tree.Keyword)(simpleProperty[1])];
                        parserInput.forget();
                        return name;
                    }
                    function match(re) {
                        var i = parserInput.i;
                        var chunk = parserInput.$re(re);
                        if (chunk) {
                            index.push(i);
                            return name.push(chunk[1]);
                        }
                    }
                    match(/^(\*?)/);
                    while (true) {
                        if (!match(/^((?:[\w-]+)|(?:[@\$]\{[\w-]+\}))/)) {
                            break;
                        }
                    }
                    if ((name.length > 1) && match(/^((?:\+_|\+)?)\s*:/)) {
                        parserInput.forget();
                        // at last, we have the complete match now. move forward,
                        // convert name particles to tree objects and return:
                        if (name[0] === '') {
                            name.shift();
                            index.shift();
                        }
                        for (k = 0; k < name.length; k++) {
                            s = name[k];
                            name[k] = (s.charAt(0) !== '@' && s.charAt(0) !== '$') ?
                                new (tree.Keyword)(s) :
                                (s.charAt(0) === '@' ?
                                    new (tree.Variable)("@" + s.slice(2, -1), index[k], fileInfo) :
                                    new (tree.Property)("$" + s.slice(2, -1), index[k], fileInfo));
                        }
                        return name;
                    }
                    parserInput.restore();
                }
            }
        };
    };
    Parser.serializeVars = function (vars) {
        var s = '';
        for (var name_1 in vars) {
            if (Object.hasOwnProperty.call(vars, name_1)) {
                var value = vars[name_1];
                s += ((name_1[0] === '@') ? '' : '@') + name_1 + ": " + value + ((String(value).slice(-1) === ';') ? '' : ';');
            }
        }
        return s;
    };
    function boolean(condition) {
        return condition ? Keyword.True : Keyword.False;
    }
    function If(condition, trueValue, falseValue) {
        return condition ? trueValue
            : (falseValue || new Anonymous);
    }
    var boolean$1 = { boolean: boolean, 'if': If };
    var colorFunctions;
    function clamp$1(val) {
        return Math.min(1, Math.max(0, val));
    }
    function hsla(origColor, hsl) {
        var color = colorFunctions.hsla(hsl.h, hsl.s, hsl.l, hsl.a);
        if (color) {
            if (origColor.value &&
                /^(rgb|hsl)/.test(origColor.value)) {
                color.value = origColor.value;
            }
            else {
                color.value = 'rgb';
            }
            return color;
        }
    }
    function toHSL(color) {
        if (color.toHSL) {
            return color.toHSL();
        }
        else {
            throw new Error('Argument cannot be evaluated to a color');
        }
    }
    function toHSV(color) {
        if (color.toHSV) {
            return color.toHSV();
        }
        else {
            throw new Error('Argument cannot be evaluated to a color');
        }
    }
    function number(n) {
        if (n instanceof Dimension) {
            return parseFloat(n.unit.is('%') ? n.value / 100 : n.value);
        }
        else if (typeof n === 'number') {
            return n;
        }
        else {
            throw {
                type: 'Argument',
                message: 'color functions take numbers as parameters'
            };
        }
    }
    function scaled(n, size) {
        if (n instanceof Dimension && n.unit.is('%')) {
            return parseFloat(n.value * size / 100);
        }
        else {
            return number(n);
        }
    }
    colorFunctions = {
        rgb: function (r, g, b) {
            var color = colorFunctions.rgba(r, g, b, 1.0);
            if (color) {
                color.value = 'rgb';
                return color;
            }
        },
        rgba: function (r, g, b, a) {
            try {
                if (r instanceof Color) {
                    if (g) {
                        a = number(g);
                    }
                    else {
                        a = r.alpha;
                    }
                    return new Color(r.rgb, a, 'rgba');
                }
                var rgb = [r, g, b].map(function (c) { return scaled(c, 255); });
                a = number(a);
                return new Color(rgb, a, 'rgba');
            }
            catch (e) { }
        },
        hsl: function (h, s, l) {
            var color = colorFunctions.hsla(h, s, l, 1.0);
            if (color) {
                color.value = 'hsl';
                return color;
            }
        },
        hsla: function (h, s, l, a) {
            try {
                if (h instanceof Color) {
                    if (s) {
                        a = number(s);
                    }
                    else {
                        a = h.alpha;
                    }
                    return new Color(h.rgb, a, 'hsla');
                }
                var m1_1;
                var m2_1;
                function hue(h) {
                    h = h < 0 ? h + 1 : (h > 1 ? h - 1 : h);
                    if (h * 6 < 1) {
                        return m1_1 + (m2_1 - m1_1) * h * 6;
                    }
                    else if (h * 2 < 1) {
                        return m2_1;
                    }
                    else if (h * 3 < 2) {
                        return m1_1 + (m2_1 - m1_1) * (2 / 3 - h) * 6;
                    }
                    else {
                        return m1_1;
                    }
                }
                h = (number(h) % 360) / 360;
                s = clamp$1(number(s));
                l = clamp$1(number(l));
                a = clamp$1(number(a));
                m2_1 = l <= 0.5 ? l * (s + 1) : l + s - l * s;
                m1_1 = l * 2 - m2_1;
                var rgb = [
                    hue(h + 1 / 3) * 255,
                    hue(h) * 255,
                    hue(h - 1 / 3) * 255
                ];
                a = number(a);
                return new Color(rgb, a, 'hsla');
            }
            catch (e) { }
        },
        hsv: function (h, s, v) {
            return colorFunctions.hsva(h, s, v, 1.0);
        },
        hsva: function (h, s, v, a) {
            h = ((number(h) % 360) / 360) * 360;
            s = number(s);
            v = number(v);
            a = number(a);
            var i;
            var f;
            i = Math.floor((h / 60) % 6);
            f = (h / 60) - i;
            var vs = [v,
                v * (1 - s),
                v * (1 - f * s),
                v * (1 - (1 - f) * s)];
            var perm = [[0, 3, 1],
                [2, 0, 1],
                [1, 0, 3],
                [1, 2, 0],
                [3, 1, 0],
                [0, 1, 2]];
            return colorFunctions.rgba(vs[perm[i][0]] * 255, vs[perm[i][1]] * 255, vs[perm[i][2]] * 255, a);
        },
        hue: function (color) {
            return new Dimension(toHSL(color).h);
        },
        saturation: function (color) {
            return new Dimension(toHSL(color).s * 100, '%');
        },
        lightness: function (color) {
            return new Dimension(toHSL(color).l * 100, '%');
        },
        hsvhue: function (color) {
            return new Dimension(toHSV(color).h);
        },
        hsvsaturation: function (color) {
            return new Dimension(toHSV(color).s * 100, '%');
        },
        hsvvalue: function (color) {
            return new Dimension(toHSV(color).v * 100, '%');
        },
        red: function (color) {
            return new Dimension(color.rgb[0]);
        },
        green: function (color) {
            return new Dimension(color.rgb[1]);
        },
        blue: function (color) {
            return new Dimension(color.rgb[2]);
        },
        alpha: function (color) {
            return new Dimension(toHSL(color).a);
        },
        luma: function (color) {
            return new Dimension(color.luma() * color.alpha * 100, '%');
        },
        luminance: function (color) {
            var luminance = (0.2126 * color.rgb[0] / 255) +
                (0.7152 * color.rgb[1] / 255) +
                (0.0722 * color.rgb[2] / 255);
            return new Dimension(luminance * color.alpha * 100, '%');
        },
        saturate: function (color, amount, method) {
            // filter: saturate(3.2);
            // should be kept as is, so check for color
            if (!color.rgb) {
                return null;
            }
            var hsl = toHSL(color);
            if (typeof method !== 'undefined' && method.value === 'relative') {
                hsl.s += hsl.s * amount.value / 100;
            }
            else {
                hsl.s += amount.value / 100;
            }
            hsl.s = clamp$1(hsl.s);
            return hsla(color, hsl);
        },
        desaturate: function (color, amount, method) {
            var hsl = toHSL(color);
            if (typeof method !== 'undefined' && method.value === 'relative') {
                hsl.s -= hsl.s * amount.value / 100;
            }
            else {
                hsl.s -= amount.value / 100;
            }
            hsl.s = clamp$1(hsl.s);
            return hsla(color, hsl);
        },
        lighten: function (color, amount, method) {
            var hsl = toHSL(color);
            if (typeof method !== 'undefined' && method.value === 'relative') {
                hsl.l += hsl.l * amount.value / 100;
            }
            else {
                hsl.l += amount.value / 100;
            }
            hsl.l = clamp$1(hsl.l);
            return hsla(color, hsl);
        },
        darken: function (color, amount, method) {
            var hsl = toHSL(color);
            if (typeof method !== 'undefined' && method.value === 'relative') {
                hsl.l -= hsl.l * amount.value / 100;
            }
            else {
                hsl.l -= amount.value / 100;
            }
            hsl.l = clamp$1(hsl.l);
            return hsla(color, hsl);
        },
        fadein: function (color, amount, method) {
            var hsl = toHSL(color);
            if (typeof method !== 'undefined' && method.value === 'relative') {
                hsl.a += hsl.a * amount.value / 100;
            }
            else {
                hsl.a += amount.value / 100;
            }
            hsl.a = clamp$1(hsl.a);
            return hsla(color, hsl);
        },
        fadeout: function (color, amount, method) {
            var hsl = toHSL(color);
            if (typeof method !== 'undefined' && method.value === 'relative') {
                hsl.a -= hsl.a * amount.value / 100;
            }
            else {
                hsl.a -= amount.value / 100;
            }
            hsl.a = clamp$1(hsl.a);
            return hsla(color, hsl);
        },
        fade: function (color, amount) {
            var hsl = toHSL(color);
            hsl.a = amount.value / 100;
            hsl.a = clamp$1(hsl.a);
            return hsla(color, hsl);
        },
        spin: function (color, amount) {
            var hsl = toHSL(color);
            var hue = (hsl.h + amount.value) % 360;
            hsl.h = hue < 0 ? 360 + hue : hue;
            return hsla(color, hsl);
        },
        //
        // Copyright (c) 2006-2009 Hampton Catlin, Natalie Weizenbaum, and Chris Eppstein
        // http://sass-lang.com
        //
        mix: function (color1, color2, weight) {
            if (!weight) {
                weight = new Dimension(50);
            }
            var p = weight.value / 100.0;
            var w = p * 2 - 1;
            var a = toHSL(color1).a - toHSL(color2).a;
            var w1 = (((w * a == -1) ? w : (w + a) / (1 + w * a)) + 1) / 2.0;
            var w2 = 1 - w1;
            var rgb = [color1.rgb[0] * w1 + color2.rgb[0] * w2,
                color1.rgb[1] * w1 + color2.rgb[1] * w2,
                color1.rgb[2] * w1 + color2.rgb[2] * w2];
            var alpha = color1.alpha * p + color2.alpha * (1 - p);
            return new Color(rgb, alpha);
        },
        greyscale: function (color) {
            return colorFunctions.desaturate(color, new Dimension(100));
        },
        contrast: function (color, dark, light, threshold) {
            // filter: contrast(3.2);
            // should be kept as is, so check for color
            if (!color.rgb) {
                return null;
            }
            if (typeof light === 'undefined') {
                light = colorFunctions.rgba(255, 255, 255, 1.0);
            }
            if (typeof dark === 'undefined') {
                dark = colorFunctions.rgba(0, 0, 0, 1.0);
            }
            // Figure out which is actually light and dark:
            if (dark.luma() > light.luma()) {
                var t = light;
                light = dark;
                dark = t;
            }
            if (typeof threshold === 'undefined') {
                threshold = 0.43;
            }
            else {
                threshold = number(threshold);
            }
            if (color.luma() < threshold) {
                return light;
            }
            else {
                return dark;
            }
        },
        // Changes made in 2.7.0 - Reverted in 3.0.0
        // contrast: function (color, color1, color2, threshold) {
        //     // Return which of `color1` and `color2` has the greatest contrast with `color`
        //     // according to the standard WCAG contrast ratio calculation.
        //     // http://www.w3.org/TR/WCAG20/#contrast-ratiodef
        //     // The threshold param is no longer used, in line with SASS.
        //     // filter: contrast(3.2);
        //     // should be kept as is, so check for color
        //     if (!color.rgb) {
        //         return null;
        //     }
        //     if (typeof color1 === 'undefined') {
        //         color1 = colorFunctions.rgba(0, 0, 0, 1.0);
        //     }
        //     if (typeof color2 === 'undefined') {
        //         color2 = colorFunctions.rgba(255, 255, 255, 1.0);
        //     }
        //     var contrast1, contrast2;
        //     var luma = color.luma();
        //     var luma1 = color1.luma();
        //     var luma2 = color2.luma();
        //     // Calculate contrast ratios for each color
        //     if (luma > luma1) {
        //         contrast1 = (luma + 0.05) / (luma1 + 0.05);
        //     } else {
        //         contrast1 = (luma1 + 0.05) / (luma + 0.05);
        //     }
        //     if (luma > luma2) {
        //         contrast2 = (luma + 0.05) / (luma2 + 0.05);
        //     } else {
        //         contrast2 = (luma2 + 0.05) / (luma + 0.05);
        //     }
        //     if (contrast1 > contrast2) {
        //         return color1;
        //     } else {
        //         return color2;
        //     }
        // },
        argb: function (color) {
            return new Anonymous(color.toARGB());
        },
        color: function (c) {
            if ((c instanceof Quoted) &&
                (/^#([A-Fa-f0-9]{8}|[A-Fa-f0-9]{6}|[A-Fa-f0-9]{3,4})$/i.test(c.value))) {
                var val = c.value.slice(1);
                return new Color(val, undefined, "#" + val);
            }
            if ((c instanceof Color) || (c = Color.fromKeyword(c.value))) {
                c.value = undefined;
                return c;
            }
            throw {
                type: 'Argument',
                message: 'argument must be a color keyword or 3|4|6|8 digit hex e.g. #FFF'
            };
        },
        tint: function (color, amount) {
            return colorFunctions.mix(colorFunctions.rgb(255, 255, 255), color, amount);
        },
        shade: function (color, amount) {
            return colorFunctions.mix(colorFunctions.rgb(0, 0, 0), color, amount);
        }
    };
    var color = colorFunctions;
    // Color Blending
    // ref: http://www.w3.org/TR/compositing-1
    function colorBlend(mode, color1, color2) {
        var ab = color1.alpha; // result
        var // backdrop
        cb;
        var as = color2.alpha;
        var // source
        cs;
        var ar;
        var cr;
        var r = [];
        ar = as + ab * (1 - as);
        for (var i_1 = 0; i_1 < 3; i_1++) {
            cb = color1.rgb[i_1] / 255;
            cs = color2.rgb[i_1] / 255;
            cr = mode(cb, cs);
            if (ar) {
                cr = (as * cs + ab * (cb -
                    as * (cb + cs - cr))) / ar;
            }
            r[i_1] = cr * 255;
        }
        return new Color(r, ar);
    }
    var colorBlendModeFunctions = {
        multiply: function (cb, cs) {
            return cb * cs;
        },
        screen: function (cb, cs) {
            return cb + cs - cb * cs;
        },
        overlay: function (cb, cs) {
            cb *= 2;
            return (cb <= 1) ?
                colorBlendModeFunctions.multiply(cb, cs) :
                colorBlendModeFunctions.screen(cb - 1, cs);
        },
        softlight: function (cb, cs) {
            var d = 1;
            var e = cb;
            if (cs > 0.5) {
                e = 1;
                d = (cb > 0.25) ? Math.sqrt(cb)
                    : ((16 * cb - 12) * cb + 4) * cb;
            }
            return cb - (1 - 2 * cs) * e * (d - cb);
        },
        hardlight: function (cb, cs) {
            return colorBlendModeFunctions.overlay(cs, cb);
        },
        difference: function (cb, cs) {
            return Math.abs(cb - cs);
        },
        exclusion: function (cb, cs) {
            return cb + cs - 2 * cb * cs;
        },
        // non-w3c functions:
        average: function (cb, cs) {
            return (cb + cs) / 2;
        },
        negation: function (cb, cs) {
            return 1 - Math.abs(cb + cs - 1);
        }
    };
    for (var f in colorBlendModeFunctions) {
        if (colorBlendModeFunctions.hasOwnProperty(f)) {
            colorBlend[f] = colorBlend.bind(null, colorBlendModeFunctions[f]);
        }
    }
    var dataUri = (function (environment) {
        var fallback = function (functionThis, node) { return new URL(node, functionThis.index, functionThis.currentFileInfo).eval(functionThis.context); };
        return { 'data-uri': function (mimetypeNode, filePathNode) {
                if (!filePathNode) {
                    filePathNode = mimetypeNode;
                    mimetypeNode = null;
                }
                var mimetype = mimetypeNode && mimetypeNode.value;
                var filePath = filePathNode.value;
                var currentFileInfo = this.currentFileInfo;
                var currentDirectory = currentFileInfo.rewriteUrls ?
                    currentFileInfo.currentDirectory : currentFileInfo.entryPath;
                var fragmentStart = filePath.indexOf('#');
                var fragment = '';
                if (fragmentStart !== -1) {
                    fragment = filePath.slice(fragmentStart);
                    filePath = filePath.slice(0, fragmentStart);
                }
                var context = clone(this.context);
                context.rawBuffer = true;
                var fileManager = environment.getFileManager(filePath, currentDirectory, context, environment, true);
                if (!fileManager) {
                    return fallback(this, filePathNode);
                }
                var useBase64 = false;
                // detect the mimetype if not given
                if (!mimetypeNode) {
                    mimetype = environment.mimeLookup(filePath);
                    if (mimetype === 'image/svg+xml') {
                        useBase64 = false;
                    }
                    else {
                        // use base 64 unless it's an ASCII or UTF-8 format
                        var charset = environment.charsetLookup(mimetype);
                        useBase64 = ['US-ASCII', 'UTF-8'].indexOf(charset) < 0;
                    }
                    if (useBase64) {
                        mimetype += ';base64';
                    }
                }
                else {
                    useBase64 = /;base64$/.test(mimetype);
                }
                var fileSync = fileManager.loadFileSync(filePath, currentDirectory, context, environment);
                if (!fileSync.contents) {
                    logger.warn("Skipped data-uri embedding of " + filePath + " because file not found");
                    return fallback(this, filePathNode || mimetypeNode);
                }
                var buf = fileSync.contents;
                if (useBase64 && !environment.encodeBase64) {
                    return fallback(this, filePathNode);
                }
                buf = useBase64 ? environment.encodeBase64(buf) : encodeURIComponent(buf);
                var uri = "data:" + mimetype + "," + buf + fragment;
                return new URL(new Quoted("\"" + uri + "\"", uri, false, this.index, this.currentFileInfo), this.index, this.currentFileInfo);
            } };
    });
    var getItemsFromNode = function (node) {
        // handle non-array values as an array of length 1
        // return 'undefined' if index is invalid
        var items = Array.isArray(node.value) ?
            node.value : Array(node);
        return items;
    };
    var list = {
        _SELF: function (n) {
            return n;
        },
        extract: function (values, index) {
            // (1-based index)
            index = index.value - 1;
            return getItemsFromNode(values)[index];
        },
        length: function (values) {
            return new Dimension(getItemsFromNode(values).length);
        },
        /**
         * Creates a Less list of incremental values.
         * Modeled after Lodash's range function, also exists natively in PHP
         *
         * @param {Dimension} [start=1]
         * @param {Dimension} end  - e.g. 10 or 10px - unit is added to output
         * @param {Dimension} [step=1]
         */
        range: function (start, end, step) {
            var from;
            var to;
            var stepValue = 1;
            var list = [];
            if (end) {
                to = end;
                from = start.value;
                if (step) {
                    stepValue = step.value;
                }
            }
            else {
                from = 1;
                to = start;
            }
            for (var i_1 = from; i_1 <= to.value; i_1 += stepValue) {
                list.push(new Dimension(i_1, to.unit));
            }
            return new Expression(list);
        },
        each: function (list, rs) {
            var rules = [];
            var newRules;
            var iterator;
            if (list.value && !(list instanceof Quoted)) {
                if (Array.isArray(list.value)) {
                    iterator = list.value;
                }
                else {
                    iterator = [list.value];
                }
            }
            else if (list.ruleset) {
                iterator = list.ruleset.rules;
            }
            else if (list.rules) {
                iterator = list.rules;
            }
            else if (Array.isArray(list)) {
                iterator = list;
            }
            else {
                iterator = [list];
            }
            var valueName = '@value';
            var keyName = '@key';
            var indexName = '@index';
            if (rs.params) {
                valueName = rs.params[0] && rs.params[0].name;
                keyName = rs.params[1] && rs.params[1].name;
                indexName = rs.params[2] && rs.params[2].name;
                rs = rs.rules;
            }
            else {
                rs = rs.ruleset;
            }
            for (var i_2 = 0; i_2 < iterator.length; i_2++) {
                var key = void 0;
                var value = void 0;
                var item = iterator[i_2];
                if (item instanceof Declaration) {
                    key = typeof item.name === 'string' ? item.name : item.name[0].value;
                    value = item.value;
                }
                else {
                    key = new Dimension(i_2 + 1);
                    value = item;
                }
                if (item instanceof Comment) {
                    continue;
                }
                newRules = rs.rules.slice(0);
                if (valueName) {
                    newRules.push(new Declaration(valueName, value, false, false, this.index, this.currentFileInfo));
                }
                if (indexName) {
                    newRules.push(new Declaration(indexName, new Dimension(i_2 + 1), false, false, this.index, this.currentFileInfo));
                }
                if (keyName) {
                    newRules.push(new Declaration(keyName, key, false, false, this.index, this.currentFileInfo));
                }
                rules.push(new Ruleset([new (Selector)([new Element("", '&')])], newRules, rs.strictImports, rs.visibilityInfo()));
            }
            return new Ruleset([new (Selector)([new Element("", '&')])], rules, rs.strictImports, rs.visibilityInfo()).eval(this.context);
        }
    };
    var MathHelper = function (fn, unit, n) {
        if (!(n instanceof Dimension)) {
            throw { type: 'Argument', message: 'argument must be a number' };
        }
        if (unit == null) {
            unit = n.unit;
        }
        else {
            n = n.unify();
        }
        return new Dimension(fn(parseFloat(n.value)), unit);
    };
    var mathFunctions = {
        // name,  unit
        ceil: null,
        floor: null,
        sqrt: null,
        abs: null,
        tan: '',
        sin: '',
        cos: '',
        atan: 'rad',
        asin: 'rad',
        acos: 'rad'
    };
    for (var f$1 in mathFunctions) {
        if (mathFunctions.hasOwnProperty(f$1)) {
            mathFunctions[f$1] = MathHelper.bind(null, Math[f$1], mathFunctions[f$1]);
        }
    }
    mathFunctions.round = function (n, f) {
        var fraction = typeof f === 'undefined' ? 0 : f.value;
        return MathHelper(function (num) { return num.toFixed(fraction); }, null, n);
    };
    var minMax = function (isMin, args) {
        args = Array.prototype.slice.call(args);
        switch (args.length) {
            case 0: throw { type: 'Argument', message: 'one or more arguments required' };
        }
        var i; // key is the unit.toString() for unified Dimension values,
        var j;
        var current;
        var currentUnified;
        var referenceUnified;
        var unit;
        var unitStatic;
        var unitClone;
        var // elems only contains original argument values.
        order = [];
        var values = {};
        // value is the index into the order array.
        for (i = 0; i < args.length; i++) {
            current = args[i];
            if (!(current instanceof Dimension)) {
                if (Array.isArray(args[i].value)) {
                    Array.prototype.push.apply(args, Array.prototype.slice.call(args[i].value));
                }
                continue;
            }
            currentUnified = current.unit.toString() === '' && unitClone !== undefined ? new Dimension(current.value, unitClone).unify() : current.unify();
            unit = currentUnified.unit.toString() === '' && unitStatic !== undefined ? unitStatic : currentUnified.unit.toString();
            unitStatic = unit !== '' && unitStatic === undefined || unit !== '' && order[0].unify().unit.toString() === '' ? unit : unitStatic;
            unitClone = unit !== '' && unitClone === undefined ? current.unit.toString() : unitClone;
            j = values[''] !== undefined && unit !== '' && unit === unitStatic ? values[''] : values[unit];
            if (j === undefined) {
                if (unitStatic !== undefined && unit !== unitStatic) {
                    throw { type: 'Argument', message: 'incompatible types' };
                }
                values[unit] = order.length;
                order.push(current);
                continue;
            }
            referenceUnified = order[j].unit.toString() === '' && unitClone !== undefined ? new Dimension(order[j].value, unitClone).unify() : order[j].unify();
            if (isMin && currentUnified.value < referenceUnified.value ||
                !isMin && currentUnified.value > referenceUnified.value) {
                order[j] = current;
            }
        }
        if (order.length == 1) {
            return order[0];
        }
        args = order.map(function (a) { return a.toCSS(this.context); }).join(this.context.compress ? ',' : ', ');
        return new Anonymous((isMin ? 'min' : 'max') + "(" + args + ")");
    };
    var number$1 = {
        min: function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return minMax(true, args);
        },
        max: function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return minMax(false, args);
        },
        convert: function (val, unit) {
            return val.convertTo(unit.value);
        },
        pi: function () {
            return new Dimension(Math.PI);
        },
        mod: function (a, b) {
            return new Dimension(a.value % b.value, a.unit);
        },
        pow: function (x, y) {
            if (typeof x === 'number' && typeof y === 'number') {
                x = new Dimension(x);
                y = new Dimension(y);
            }
            else if (!(x instanceof Dimension) || !(y instanceof Dimension)) {
                throw { type: 'Argument', message: 'arguments must be numbers' };
            }
            return new Dimension(Math.pow(x.value, y.value), x.unit);
        },
        percentage: function (n) {
            var result = MathHelper(function (num) { return num * 100; }, '%', n);
            return result;
        }
    };
    var string = {
        e: function (str) {
            return new Quoted('"', str instanceof JavaScript ? str.evaluated : str.value, true);
        },
        escape: function (str) {
            return new Anonymous(encodeURI(str.value).replace(/=/g, '%3D').replace(/:/g, '%3A').replace(/#/g, '%23').replace(/;/g, '%3B')
                .replace(/\(/g, '%28').replace(/\)/g, '%29'));
        },
        replace: function (string, pattern, replacement, flags) {
            var result = string.value;
            replacement = (replacement.type === 'Quoted') ?
                replacement.value : replacement.toCSS();
            result = result.replace(new RegExp(pattern.value, flags ? flags.value : ''), replacement);
            return new Quoted(string.quote || '', result, string.escaped);
        },
        '%': function (string /* arg, arg, ... */) {
            var args = Array.prototype.slice.call(arguments, 1);
            var result = string.value;
            var _loop_1 = function (i_1) {
                /* jshint loopfunc:true */
                result = result.replace(/%[sda]/i, function (token) {
                    var value = ((args[i_1].type === 'Quoted') &&
                        token.match(/s/i)) ? args[i_1].value : args[i_1].toCSS();
                    return token.match(/[A-Z]$/) ? encodeURIComponent(value) : value;
                });
            };
            for (var i_1 = 0; i_1 < args.length; i_1++) {
                _loop_1(i_1);
            }
            result = result.replace(/%%/g, '%');
            return new Quoted(string.quote || '', result, string.escaped);
        }
    };
    var svg = (function (environment) {
        return { 'svg-gradient': function (direction) {
                var stops;
                var gradientDirectionSvg;
                var gradientType = 'linear';
                var rectangleDimension = 'x="0" y="0" width="1" height="1"';
                var renderEnv = { compress: false };
                var returner;
                var directionValue = direction.toCSS(renderEnv);
                var i;
                var color;
                var position;
                var positionValue;
                var alpha;
                function throwArgumentDescriptor() {
                    throw { type: 'Argument',
                        message: 'svg-gradient expects direction, start_color [start_position], [color position,]...,' +
                            ' end_color [end_position] or direction, color list' };
                }
                if (arguments.length == 2) {
                    if (arguments[1].value.length < 2) {
                        throwArgumentDescriptor();
                    }
                    stops = arguments[1].value;
                }
                else if (arguments.length < 3) {
                    throwArgumentDescriptor();
                }
                else {
                    stops = Array.prototype.slice.call(arguments, 1);
                }
                switch (directionValue) {
                    case 'to bottom':
                        gradientDirectionSvg = 'x1="0%" y1="0%" x2="0%" y2="100%"';
                        break;
                    case 'to right':
                        gradientDirectionSvg = 'x1="0%" y1="0%" x2="100%" y2="0%"';
                        break;
                    case 'to bottom right':
                        gradientDirectionSvg = 'x1="0%" y1="0%" x2="100%" y2="100%"';
                        break;
                    case 'to top right':
                        gradientDirectionSvg = 'x1="0%" y1="100%" x2="100%" y2="0%"';
                        break;
                    case 'ellipse':
                    case 'ellipse at center':
                        gradientType = 'radial';
                        gradientDirectionSvg = 'cx="50%" cy="50%" r="75%"';
                        rectangleDimension = 'x="-50" y="-50" width="101" height="101"';
                        break;
                    default:
                        throw { type: 'Argument', message: 'svg-gradient direction must be \'to bottom\', \'to right\',' +
                                ' \'to bottom right\', \'to top right\' or \'ellipse at center\'' };
                }
                returner = "<svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 1 1\"><" + gradientType + "Gradient id=\"g\" " + gradientDirectionSvg + ">";
                for (i = 0; i < stops.length; i += 1) {
                    if (stops[i] instanceof Expression) {
                        color = stops[i].value[0];
                        position = stops[i].value[1];
                    }
                    else {
                        color = stops[i];
                        position = undefined;
                    }
                    if (!(color instanceof Color) || (!((i === 0 || i + 1 === stops.length) && position === undefined) && !(position instanceof Dimension))) {
                        throwArgumentDescriptor();
                    }
                    positionValue = position ? position.toCSS(renderEnv) : i === 0 ? '0%' : '100%';
                    alpha = color.alpha;
                    returner += "<stop offset=\"" + positionValue + "\" stop-color=\"" + color.toRGB() + "\"" + (alpha < 1 ? " stop-opacity=\"" + alpha + "\"" : '') + "/>";
                }
                returner += "</" + gradientType + "Gradient><rect " + rectangleDimension + " fill=\"url(#g)\" /></svg>";
                returner = encodeURIComponent(returner);
                returner = "data:image/svg+xml," + returner;
                return new URL(new Quoted("'" + returner + "'", returner, false, this.index, this.currentFileInfo), this.index, this.currentFileInfo);
            } };
    });
    var isa = function (n, Type) { return (n instanceof Type) ? Keyword.True : Keyword.False; };
    var isunit = function (n, unit) {
        if (unit === undefined) {
            throw { type: 'Argument', message: 'missing the required second argument to isunit.' };
        }
        unit = typeof unit.value === 'string' ? unit.value : unit;
        if (typeof unit !== 'string') {
            throw { type: 'Argument', message: 'Second argument to isunit should be a unit or a string.' };
        }
        return (n instanceof Dimension) && n.unit.is(unit) ? Keyword.True : Keyword.False;
    };
    var types = {
        isruleset: function (n) {
            return isa(n, DetachedRuleset);
        },
        iscolor: function (n) {
            return isa(n, Color);
        },
        isnumber: function (n) {
            return isa(n, Dimension);
        },
        isstring: function (n) {
            return isa(n, Quoted);
        },
        iskeyword: function (n) {
            return isa(n, Keyword);
        },
        isurl: function (n) {
            return isa(n, URL);
        },
        ispixel: function (n) {
            return isunit(n, 'px');
        },
        ispercentage: function (n) {
            return isunit(n, '%');
        },
        isem: function (n) {
            return isunit(n, 'em');
        },
        isunit: isunit,
        unit: function (val, unit) {
            if (!(val instanceof Dimension)) {
                throw { type: 'Argument',
                    message: "the first argument to unit must be a number" + (val instanceof Operation ? '. Have you forgotten parenthesis?' : '') };
            }
            if (unit) {
                if (unit instanceof Keyword) {
                    unit = unit.value;
                }
                else {
                    unit = unit.toCSS();
                }
            }
            else {
                unit = '';
            }
            return new Dimension(val.value, unit);
        },
        'get-unit': function (n) {
            return new Anonymous(n.unit);
        }
    };
    var Functions = (function (environment) {
        var functions = { functionRegistry: functionRegistry, functionCaller: functionCaller };
        // register functions
        functionRegistry.addMultiple(boolean$1);
        functionRegistry.add('default', defaultFunc.eval.bind(defaultFunc));
        functionRegistry.addMultiple(color);
        functionRegistry.addMultiple(colorBlend);
        functionRegistry.addMultiple(dataUri(environment));
        functionRegistry.addMultiple(list);
        functionRegistry.addMultiple(mathFunctions);
        functionRegistry.addMultiple(number$1);
        functionRegistry.addMultiple(string);
        functionRegistry.addMultiple(svg());
        functionRegistry.addMultiple(types);
        return functions;
    });
    var sourceMapOutput = (function (environment) {
        var SourceMapOutput = /** @class */ (function () {
            function SourceMapOutput(options) {
                this._css = [];
                this._rootNode = options.rootNode;
                this._contentsMap = options.contentsMap;
                this._contentsIgnoredCharsMap = options.contentsIgnoredCharsMap;
                if (options.sourceMapFilename) {
                    this._sourceMapFilename = options.sourceMapFilename.replace(/\\/g, '/');
                }
                this._outputFilename = options.outputFilename;
                this.sourceMapURL = options.sourceMapURL;
                if (options.sourceMapBasepath) {
                    this._sourceMapBasepath = options.sourceMapBasepath.replace(/\\/g, '/');
                }
                if (options.sourceMapRootpath) {
                    this._sourceMapRootpath = options.sourceMapRootpath.replace(/\\/g, '/');
                    if (this._sourceMapRootpath.charAt(this._sourceMapRootpath.length - 1) !== '/') {
                        this._sourceMapRootpath += '/';
                    }
                }
                else {
                    this._sourceMapRootpath = '';
                }
                this._outputSourceFiles = options.outputSourceFiles;
                this._sourceMapGeneratorConstructor = environment.getSourceMapGenerator();
                this._lineNumber = 0;
                this._column = 0;
            }
            SourceMapOutput.prototype.removeBasepath = function (path) {
                if (this._sourceMapBasepath && path.indexOf(this._sourceMapBasepath) === 0) {
                    path = path.substring(this._sourceMapBasepath.length);
                    if (path.charAt(0) === '\\' || path.charAt(0) === '/') {
                        path = path.substring(1);
                    }
                }
                return path;
            };
            SourceMapOutput.prototype.normalizeFilename = function (filename) {
                filename = filename.replace(/\\/g, '/');
                filename = this.removeBasepath(filename);
                return (this._sourceMapRootpath || '') + filename;
            };
            SourceMapOutput.prototype.add = function (chunk, fileInfo, index, mapLines) {
                // ignore adding empty strings
                if (!chunk) {
                    return;
                }
                var lines;
                var sourceLines;
                var columns;
                var sourceColumns;
                var i;
                if (fileInfo && fileInfo.filename) {
                    var inputSource = this._contentsMap[fileInfo.filename];
                    // remove vars/banner added to the top of the file
                    if (this._contentsIgnoredCharsMap[fileInfo.filename]) {
                        // adjust the index
                        index -= this._contentsIgnoredCharsMap[fileInfo.filename];
                        if (index < 0) {
                            index = 0;
                        }
                        // adjust the source
                        inputSource = inputSource.slice(this._contentsIgnoredCharsMap[fileInfo.filename]);
                    }
                    // ignore empty content
                    if (inputSource === undefined) {
                        return;
                    }
                    inputSource = inputSource.substring(0, index);
                    sourceLines = inputSource.split('\n');
                    sourceColumns = sourceLines[sourceLines.length - 1];
                }
                lines = chunk.split('\n');
                columns = lines[lines.length - 1];
                if (fileInfo && fileInfo.filename) {
                    if (!mapLines) {
                        this._sourceMapGenerator.addMapping({ generated: { line: this._lineNumber + 1, column: this._column },
                            original: { line: sourceLines.length, column: sourceColumns.length },
                            source: this.normalizeFilename(fileInfo.filename) });
                    }
                    else {
                        for (i = 0; i < lines.length; i++) {
                            this._sourceMapGenerator.addMapping({ generated: { line: this._lineNumber + i + 1, column: i === 0 ? this._column : 0 },
                                original: { line: sourceLines.length + i, column: i === 0 ? sourceColumns.length : 0 },
                                source: this.normalizeFilename(fileInfo.filename) });
                        }
                    }
                }
                if (lines.length === 1) {
                    this._column += columns.length;
                }
                else {
                    this._lineNumber += lines.length - 1;
                    this._column = columns.length;
                }
                this._css.push(chunk);
            };
            SourceMapOutput.prototype.isEmpty = function () {
                return this._css.length === 0;
            };
            SourceMapOutput.prototype.toCSS = function (context) {
                this._sourceMapGenerator = new this._sourceMapGeneratorConstructor({ file: this._outputFilename, sourceRoot: null });
                if (this._outputSourceFiles) {
                    for (var filename in this._contentsMap) {
                        if (this._contentsMap.hasOwnProperty(filename)) {
                            var source = this._contentsMap[filename];
                            if (this._contentsIgnoredCharsMap[filename]) {
                                source = source.slice(this._contentsIgnoredCharsMap[filename]);
                            }
                            this._sourceMapGenerator.setSourceContent(this.normalizeFilename(filename), source);
                        }
                    }
                }
                this._rootNode.genCSS(context, this);
                if (this._css.length > 0) {
                    var sourceMapURL = void 0;
                    var sourceMapContent = JSON.stringify(this._sourceMapGenerator.toJSON());
                    if (this.sourceMapURL) {
                        sourceMapURL = this.sourceMapURL;
                    }
                    else if (this._sourceMapFilename) {
                        sourceMapURL = this._sourceMapFilename;
                    }
                    this.sourceMapURL = sourceMapURL;
                    this.sourceMap = sourceMapContent;
                }
                return this._css.join('');
            };
            return SourceMapOutput;
        }());
        return SourceMapOutput;
    });
    var sourceMapBuilder = (function (SourceMapOutput, environment) {
        var SourceMapBuilder = /** @class */ (function () {
            function SourceMapBuilder(options) {
                this.options = options;
            }
            SourceMapBuilder.prototype.toCSS = function (rootNode, options, imports) {
                var sourceMapOutput = new SourceMapOutput({
                    contentsIgnoredCharsMap: imports.contentsIgnoredChars,
                    rootNode: rootNode,
                    contentsMap: imports.contents,
                    sourceMapFilename: this.options.sourceMapFilename,
                    sourceMapURL: this.options.sourceMapURL,
                    outputFilename: this.options.sourceMapOutputFilename,
                    sourceMapBasepath: this.options.sourceMapBasepath,
                    sourceMapRootpath: this.options.sourceMapRootpath,
                    outputSourceFiles: this.options.outputSourceFiles,
                    sourceMapGenerator: this.options.sourceMapGenerator,
                    sourceMapFileInline: this.options.sourceMapFileInline
                });
                var css = sourceMapOutput.toCSS(options);
                this.sourceMap = sourceMapOutput.sourceMap;
                this.sourceMapURL = sourceMapOutput.sourceMapURL;
                if (this.options.sourceMapInputFilename) {
                    this.sourceMapInputFilename = sourceMapOutput.normalizeFilename(this.options.sourceMapInputFilename);
                }
                if (this.options.sourceMapBasepath !== undefined && this.sourceMapURL !== undefined) {
                    this.sourceMapURL = sourceMapOutput.removeBasepath(this.sourceMapURL);
                }
                return css + this.getCSSAppendage();
            };
            SourceMapBuilder.prototype.getCSSAppendage = function () {
                var sourceMapURL = this.sourceMapURL;
                if (this.options.sourceMapFileInline) {
                    if (this.sourceMap === undefined) {
                        return '';
                    }
                    sourceMapURL = "data:application/json;base64," + environment.encodeBase64(this.sourceMap);
                }
                if (sourceMapURL) {
                    return "/*# sourceMappingURL=" + sourceMapURL + " */";
                }
                return '';
            };
            SourceMapBuilder.prototype.getExternalSourceMap = function () {
                return this.sourceMap;
            };
            SourceMapBuilder.prototype.setExternalSourceMap = function (sourceMap) {
                this.sourceMap = sourceMap;
            };
            SourceMapBuilder.prototype.isInline = function () {
                return this.options.sourceMapFileInline;
            };
            SourceMapBuilder.prototype.getSourceMapURL = function () {
                return this.sourceMapURL;
            };
            SourceMapBuilder.prototype.getOutputFilename = function () {
                return this.options.sourceMapOutputFilename;
            };
            SourceMapBuilder.prototype.getInputFilename = function () {
                return this.sourceMapInputFilename;
            };
            return SourceMapBuilder;
        }());
        return SourceMapBuilder;
    });
    var transformTree = (function (root, options) {
        if (options === void 0) {
            options = {};
        }
        var evaldRoot;
        var variables = options.variables;
        var evalEnv = new contexts.Eval(options);
        //
        // Allows setting variables with a hash, so:
        //
        //   `{ color: new tree.Color('#f01') }` will become:
        //
        //   new tree.Declaration('@color',
        //     new tree.Value([
        //       new tree.Expression([
        //         new tree.Color('#f01')
        //       ])
        //     ])
        //   )
        //
        if (typeof variables === 'object' && !Array.isArray(variables)) {
            variables = Object.keys(variables).map(function (k) {
                var value = variables[k];
                if (!(value instanceof tree.Value)) {
                    if (!(value instanceof tree.Expression)) {
                        value = new tree.Expression([value]);
                    }
                    value = new tree.Value([value]);
                }
                return new tree.Declaration("@" + k, value, false, null, 0);
            });
            evalEnv.frames = [new tree.Ruleset(null, variables)];
        }
        var visitors$1 = [
            new visitors.JoinSelectorVisitor(),
            new visitors.MarkVisibleSelectorsVisitor(true),
            new visitors.ExtendVisitor(),
            new visitors.ToCSSVisitor({ compress: Boolean(options.compress) })
        ];
        var preEvalVisitors = [];
        var v;
        var visitorIterator;
        /**
         * first() / get() allows visitors to be added while visiting
         *
         * @todo Add scoping for visitors just like functions for @plugin; right now they're global
         */
        if (options.pluginManager) {
            visitorIterator = options.pluginManager.visitor();
            for (var i = 0; i < 2; i++) {
                visitorIterator.first();
                while ((v = visitorIterator.get())) {
                    if (v.isPreEvalVisitor) {
                        if (i === 0 || preEvalVisitors.indexOf(v) === -1) {
                            preEvalVisitors.push(v);
                            v.run(root);
                        }
                    }
                    else {
                        if (i === 0 || visitors$1.indexOf(v) === -1) {
                            if (v.isPreVisitor) {
                                visitors$1.unshift(v);
                            }
                            else {
                                visitors$1.push(v);
                            }
                        }
                    }
                }
            }
        }
        evaldRoot = root.eval(evalEnv);
        for (var i = 0; i < visitors$1.length; i++) {
            visitors$1[i].run(evaldRoot);
        }
        // Run any remaining visitors added after eval pass
        if (options.pluginManager) {
            visitorIterator.first();
            while ((v = visitorIterator.get())) {
                if (visitors$1.indexOf(v) === -1 && preEvalVisitors.indexOf(v) === -1) {
                    v.run(evaldRoot);
                }
            }
        }
        return evaldRoot;
    });
    var parseTree = (function (SourceMapBuilder) {
        var ParseTree = /** @class */ (function () {
            function ParseTree(root, imports) {
                this.root = root;
                this.imports = imports;
            }
            ParseTree.prototype.toCSS = function (options) {
                var evaldRoot;
                var result = {};
                var sourceMapBuilder;
                try {
                    evaldRoot = transformTree(this.root, options);
                }
                catch (e) {
                    throw new LessError(e, this.imports);
                }
                try {
                    var compress = Boolean(options.compress);
                    if (compress) {
                        logger.warn('The compress option has been deprecated. ' +
                            'We recommend you use a dedicated css minifier, for instance see less-plugin-clean-css.');
                    }
                    var toCSSOptions = {
                        compress: compress,
                        dumpLineNumbers: options.dumpLineNumbers,
                        strictUnits: Boolean(options.strictUnits),
                        numPrecision: 8
                    };
                    if (options.sourceMap) {
                        sourceMapBuilder = new SourceMapBuilder(options.sourceMap);
                        result.css = sourceMapBuilder.toCSS(evaldRoot, toCSSOptions, this.imports);
                    }
                    else {
                        result.css = evaldRoot.toCSS(toCSSOptions);
                    }
                }
                catch (e) {
                    throw new LessError(e, this.imports);
                }
                if (options.pluginManager) {
                    var postProcessors = options.pluginManager.getPostProcessors();
                    for (var i_1 = 0; i_1 < postProcessors.length; i_1++) {
                        result.css = postProcessors[i_1].process(result.css, { sourceMap: sourceMapBuilder, options: options, imports: this.imports });
                    }
                }
                if (options.sourceMap) {
                    result.map = sourceMapBuilder.getExternalSourceMap();
                }
                result.imports = [];
                for (var file_1 in this.imports.files) {
                    if (this.imports.files.hasOwnProperty(file_1) && file_1 !== this.imports.rootFilename) {
                        result.imports.push(file_1);
                    }
                }
                return result;
            };
            return ParseTree;
        }());
        return ParseTree;
    });
    var importManager = (function (environment) {
        // FileInfo = {
        //  'rewriteUrls' - option - whether to adjust URL's to be relative
        //  'filename' - full resolved filename of current file
        //  'rootpath' - path to append to normal URLs for this node
        //  'currentDirectory' - path to the current file, absolute
        //  'rootFilename' - filename of the base file
        //  'entryPath' - absolute path to the entry file
        //  'reference' - whether the file should not be output and only output parts that are referenced
        var ImportManager = /** @class */ (function () {
            function ImportManager(less, context, rootFileInfo) {
                this.less = less;
                this.rootFilename = rootFileInfo.filename;
                this.paths = context.paths || []; // Search paths, when importing
                this.contents = {}; // map - filename to contents of all the files
                this.contentsIgnoredChars = {}; // map - filename to lines at the beginning of each file to ignore
                this.mime = context.mime;
                this.error = null;
                this.context = context;
                // Deprecated? Unused outside of here, could be useful.
                this.queue = []; // Files which haven't been imported yet
                this.files = {}; // Holds the imported parse trees.
            }
            /**
             * Add an import to be imported
             * @param path - the raw path
             * @param tryAppendExtension - whether to try appending a file extension (.less or .js if the path has no extension)
             * @param currentFileInfo - the current file info (used for instance to work out relative paths)
             * @param importOptions - import options
             * @param callback - callback for when it is imported
             */
            ImportManager.prototype.push = function (path, tryAppendExtension, currentFileInfo, importOptions, callback) {
                var importManager = this;
                var pluginLoader = this.context.pluginManager.Loader;
                this.queue.push(path);
                var fileParsedFunc = function (e, root, fullPath) {
                    importManager.queue.splice(importManager.queue.indexOf(path), 1); // Remove the path from the queue
                    var importedEqualsRoot = fullPath === importManager.rootFilename;
                    if (importOptions.optional && e) {
                        callback(null, { rules: [] }, false, null);
                        logger.info("The file " + fullPath + " was skipped because it was not found and the import was marked optional.");
                    }
                    else {
                        // Inline imports aren't cached here.
                        // If we start to cache them, please make sure they won't conflict with non-inline imports of the
                        // same name as they used to do before this comment and the condition below have been added.
                        if (!importManager.files[fullPath] && !importOptions.inline) {
                            importManager.files[fullPath] = { root: root, options: importOptions };
                        }
                        if (e && !importManager.error) {
                            importManager.error = e;
                        }
                        callback(e, root, importedEqualsRoot, fullPath);
                    }
                };
                var newFileInfo = {
                    rewriteUrls: this.context.rewriteUrls,
                    entryPath: currentFileInfo.entryPath,
                    rootpath: currentFileInfo.rootpath,
                    rootFilename: currentFileInfo.rootFilename
                };
                var fileManager = environment.getFileManager(path, currentFileInfo.currentDirectory, this.context, environment);
                if (!fileManager) {
                    fileParsedFunc({ message: "Could not find a file-manager for " + path });
                    return;
                }
                var loadFileCallback = function (loadedFile) {
                    var plugin;
                    var resolvedFilename = loadedFile.filename;
                    var contents = loadedFile.contents.replace(/^\uFEFF/, '');
                    // Pass on an updated rootpath if path of imported file is relative and file
                    // is in a (sub|sup) directory
                    //
                    // Examples:
                    // - If path of imported file is 'module/nav/nav.less' and rootpath is 'less/',
                    //   then rootpath should become 'less/module/nav/'
                    // - If path of imported file is '../mixins.less' and rootpath is 'less/',
                    //   then rootpath should become 'less/../'
                    newFileInfo.currentDirectory = fileManager.getPath(resolvedFilename);
                    if (newFileInfo.rewriteUrls) {
                        newFileInfo.rootpath = fileManager.join((importManager.context.rootpath || ''), fileManager.pathDiff(newFileInfo.currentDirectory, newFileInfo.entryPath));
                        if (!fileManager.isPathAbsolute(newFileInfo.rootpath) && fileManager.alwaysMakePathsAbsolute()) {
                            newFileInfo.rootpath = fileManager.join(newFileInfo.entryPath, newFileInfo.rootpath);
                        }
                    }
                    newFileInfo.filename = resolvedFilename;
                    var newEnv = new contexts.Parse(importManager.context);
                    newEnv.processImports = false;
                    importManager.contents[resolvedFilename] = contents;
                    if (currentFileInfo.reference || importOptions.reference) {
                        newFileInfo.reference = true;
                    }
                    if (importOptions.isPlugin) {
                        plugin = pluginLoader.evalPlugin(contents, newEnv, importManager, importOptions.pluginArgs, newFileInfo);
                        if (plugin instanceof LessError) {
                            fileParsedFunc(plugin, null, resolvedFilename);
                        }
                        else {
                            fileParsedFunc(null, plugin, resolvedFilename);
                        }
                    }
                    else if (importOptions.inline) {
                        fileParsedFunc(null, contents, resolvedFilename);
                    }
                    else {
                        // import (multiple) parse trees apparently get altered and can't be cached.
                        // TODO: investigate why this is
                        if (importManager.files[resolvedFilename]
                            && !importManager.files[resolvedFilename].options.multiple
                            && !importOptions.multiple) {
                            fileParsedFunc(null, importManager.files[resolvedFilename].root, resolvedFilename);
                        }
                        else {
                            new Parser(newEnv, importManager, newFileInfo).parse(contents, function (e, root) {
                                fileParsedFunc(e, root, resolvedFilename);
                            });
                        }
                    }
                };
                var promise;
                var context = clone(this.context);
                if (tryAppendExtension) {
                    context.ext = importOptions.isPlugin ? '.js' : '.less';
                }
                if (importOptions.isPlugin) {
                    context.mime = 'application/javascript';
                    promise = pluginLoader.loadPlugin(path, currentFileInfo.currentDirectory, context, environment, fileManager);
                }
                else {
                    promise = fileManager.loadFile(path, currentFileInfo.currentDirectory, context, environment, function (err, loadedFile) {
                        if (err) {
                            fileParsedFunc(err);
                        }
                        else {
                            loadFileCallback(loadedFile);
                        }
                    });
                }
                if (promise) {
                    promise.then(loadFileCallback, fileParsedFunc);
                }
            };
            return ImportManager;
        }());
        return ImportManager;
    });
    var Render = (function (environment, ParseTree, ImportManager) {
        var render = function (input, options, callback) {
            if (typeof options === 'function') {
                callback = options;
                options = copyOptions(this.options, {});
            }
            else {
                options = copyOptions(this.options, options || {});
            }
            if (!callback) {
                var self_1 = this;
                return new Promise(function (resolve, reject) {
                    render.call(self_1, input, options, function (err, output) {
                        if (err) {
                            reject(err);
                        }
                        else {
                            resolve(output);
                        }
                    });
                });
            }
            else {
                this.parse(input, options, function (err, root, imports, options) {
                    if (err) {
                        return callback(err);
                    }
                    var result;
                    try {
                        var parseTree = new ParseTree(root, imports);
                        result = parseTree.toCSS(options);
                    }
                    catch (err) {
                        return callback(err);
                    }
                    callback(null, result);
                });
            }
        };
        return render;
    });
    /**
     * Plugin Manager
     */
    var PluginManager = /** @class */ (function () {
        function PluginManager(less) {
            this.less = less;
            this.visitors = [];
            this.preProcessors = [];
            this.postProcessors = [];
            this.installedPlugins = [];
            this.fileManagers = [];
            this.iterator = -1;
            this.pluginCache = {};
            this.Loader = new less.PluginLoader(less);
        }
        /**
         * Adds all the plugins in the array
         * @param {Array} plugins
         */
        PluginManager.prototype.addPlugins = function (plugins) {
            if (plugins) {
                for (var i_1 = 0; i_1 < plugins.length; i_1++) {
                    this.addPlugin(plugins[i_1]);
                }
            }
        };
        /**
         *
         * @param plugin
         * @param {String} filename
         */
        PluginManager.prototype.addPlugin = function (plugin, filename, functionRegistry) {
            this.installedPlugins.push(plugin);
            if (filename) {
                this.pluginCache[filename] = plugin;
            }
            if (plugin.install) {
                plugin.install(this.less, this, functionRegistry || this.less.functions.functionRegistry);
            }
        };
        /**
         *
         * @param filename
         */
        PluginManager.prototype.get = function (filename) {
            return this.pluginCache[filename];
        };
        /**
         * Adds a visitor. The visitor object has options on itself to determine
         * when it should run.
         * @param visitor
         */
        PluginManager.prototype.addVisitor = function (visitor) {
            this.visitors.push(visitor);
        };
        /**
         * Adds a pre processor object
         * @param {object} preProcessor
         * @param {number} priority - guidelines 1 = before import, 1000 = import, 2000 = after import
         */
        PluginManager.prototype.addPreProcessor = function (preProcessor, priority) {
            var indexToInsertAt;
            for (indexToInsertAt = 0; indexToInsertAt < this.preProcessors.length; indexToInsertAt++) {
                if (this.preProcessors[indexToInsertAt].priority >= priority) {
                    break;
                }
            }
            this.preProcessors.splice(indexToInsertAt, 0, { preProcessor: preProcessor, priority: priority });
        };
        /**
         * Adds a post processor object
         * @param {object} postProcessor
         * @param {number} priority - guidelines 1 = before compression, 1000 = compression, 2000 = after compression
         */
        PluginManager.prototype.addPostProcessor = function (postProcessor, priority) {
            var indexToInsertAt;
            for (indexToInsertAt = 0; indexToInsertAt < this.postProcessors.length; indexToInsertAt++) {
                if (this.postProcessors[indexToInsertAt].priority >= priority) {
                    break;
                }
            }
            this.postProcessors.splice(indexToInsertAt, 0, { postProcessor: postProcessor, priority: priority });
        };
        /**
         *
         * @param manager
         */
        PluginManager.prototype.addFileManager = function (manager) {
            this.fileManagers.push(manager);
        };
        /**
         *
         * @returns {Array}
         * @private
         */
        PluginManager.prototype.getPreProcessors = function () {
            var preProcessors = [];
            for (var i_2 = 0; i_2 < this.preProcessors.length; i_2++) {
                preProcessors.push(this.preProcessors[i_2].preProcessor);
            }
            return preProcessors;
        };
        /**
         *
         * @returns {Array}
         * @private
         */
        PluginManager.prototype.getPostProcessors = function () {
            var postProcessors = [];
            for (var i_3 = 0; i_3 < this.postProcessors.length; i_3++) {
                postProcessors.push(this.postProcessors[i_3].postProcessor);
            }
            return postProcessors;
        };
        /**
         *
         * @returns {Array}
         * @private
         */
        PluginManager.prototype.getVisitors = function () {
            return this.visitors;
        };
        PluginManager.prototype.visitor = function () {
            var self = this;
            return {
                first: function () {
                    self.iterator = -1;
                    return self.visitors[self.iterator];
                },
                get: function () {
                    self.iterator += 1;
                    return self.visitors[self.iterator];
                }
            };
        };
        /**
         *
         * @returns {Array}
         * @private
         */
        PluginManager.prototype.getFileManagers = function () {
            return this.fileManagers;
        };
        return PluginManager;
    }());
    var pm;
    function PluginManagerFactory(less, newFactory) {
        if (newFactory || !pm) {
            pm = new PluginManager(less);
        }
        return pm;
    }
    var Parse = (function (environment, ParseTree, ImportManager) {
        var parse = function (input, options, callback) {
            if (typeof options === 'function') {
                callback = options;
                options = copyOptions(this.options, {});
            }
            else {
                options = copyOptions(this.options, options || {});
            }
            if (!callback) {
                var self_1 = this;
                return new Promise(function (resolve, reject) {
                    parse.call(self_1, input, options, function (err, output) {
                        if (err) {
                            reject(err);
                        }
                        else {
                            resolve(output);
                        }
                    });
                });
            }
            else {
                var context_1;
                var rootFileInfo = void 0;
                var pluginManager_1 = new PluginManagerFactory(this, !options.reUsePluginManager);
                options.pluginManager = pluginManager_1;
                context_1 = new contexts.Parse(options);
                if (options.rootFileInfo) {
                    rootFileInfo = options.rootFileInfo;
                }
                else {
                    var filename = options.filename || 'input';
                    var entryPath = filename.replace(/[^\/\\]*$/, '');
                    rootFileInfo = {
                        filename: filename,
                        rewriteUrls: context_1.rewriteUrls,
                        rootpath: context_1.rootpath || '',
                        currentDirectory: entryPath,
                        entryPath: entryPath,
                        rootFilename: filename
                    };
                    // add in a missing trailing slash
                    if (rootFileInfo.rootpath && rootFileInfo.rootpath.slice(-1) !== '/') {
                        rootFileInfo.rootpath += '/';
                    }
                }
                var imports_1 = new ImportManager(this, context_1, rootFileInfo);
                this.importManager = imports_1;
                // TODO: allow the plugins to be just a list of paths or names
                // Do an async plugin queue like lessc
                if (options.plugins) {
                    options.plugins.forEach(function (plugin) {
                        var evalResult;
                        var contents;
                        if (plugin.fileContent) {
                            contents = plugin.fileContent.replace(/^\uFEFF/, '');
                            evalResult = pluginManager_1.Loader.evalPlugin(contents, context_1, imports_1, plugin.options, plugin.filename);
                            if (evalResult instanceof LessError) {
                                return callback(evalResult);
                            }
                        }
                        else {
                            pluginManager_1.addPlugin(plugin);
                        }
                    });
                }
                new Parser(context_1, imports_1, rootFileInfo)
                    .parse(input, function (e, root) {
                    if (e) {
                        return callback(e);
                    }
                    callback(null, root, imports_1, options);
                }, options);
            }
        };
        return parse;
    });
    var createFromEnvironment = (function (environment, fileManagers) {
        /**
         * @todo
         * This original code could be improved quite a bit.
         * Many classes / modules currently add side-effects / mutations to passed in objects,
         * which makes it hard to refactor and reason about.
         */
        environment = new environment$1(environment, fileManagers);
        var SourceMapOutput = sourceMapOutput(environment);
        var SourceMapBuilder = sourceMapBuilder(SourceMapOutput, environment);
        var ParseTree = parseTree(SourceMapBuilder);
        var ImportManager = importManager(environment);
        var render = Render(environment, ParseTree);
        var parse = Parse(environment, ParseTree, ImportManager);
        var functions = Functions(environment);
        /**
         * @todo
         * This root properties / methods need to be organized.
         * It's not clear what should / must be public and why.
         */
        var initial = {
            version: [3, 11, 1],
            data: data,
            tree: tree,
            Environment: environment$1,
            AbstractFileManager: AbstractFileManager,
            AbstractPluginLoader: AbstractPluginLoader,
            environment: environment,
            visitors: visitors,
            Parser: Parser,
            functions: functions,
            contexts: contexts,
            SourceMapOutput: SourceMapOutput,
            SourceMapBuilder: SourceMapBuilder,
            ParseTree: ParseTree,
            ImportManager: ImportManager,
            render: render,
            parse: parse,
            LessError: LessError,
            transformTree: transformTree,
            utils: utils,
            PluginManager: PluginManagerFactory,
            logger: logger
        };
        // Create a public API
        var ctor = function (t) {
            return function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return new (t.bind.apply(t, tslib.__spreadArrays([void 0], args)))();
            };
        };
        var t;
        var api = Object.create(initial);
        for (var n in initial.tree) {
            /* eslint guard-for-in: 0 */
            t = initial.tree[n];
            if (typeof t === 'function') {
                api[n.toLowerCase()] = ctor(t);
            }
            else {
                api[n] = Object.create(null);
                for (var o in t) {
                    /* eslint guard-for-in: 0 */
                    api[n][o.toLowerCase()] = ctor(t[o]);
                }
            }
        }
        return api;
    });
    function createCommonjsModule(fn, module) {
        return module = { exports: {} }, fn(module, module.exports), module.exports;
    }
    var lesscHelper = createCommonjsModule(function (module, exports) {
        // lessc_helper.js
        //
        //      helper functions for lessc
        var lessc_helper = {
            // Stylize a string
            stylize: function (str, style) {
                var styles = {
                    'reset': [0, 0],
                    'bold': [1, 22],
                    'inverse': [7, 27],
                    'underline': [4, 24],
                    'yellow': [33, 39],
                    'green': [32, 39],
                    'red': [31, 39],
                    'grey': [90, 39]
                };
                return "\u001B[" + styles[style][0] + "m" + str + "\u001B[" + styles[style][1] + "m";
            },
            // Print command line options
            printUsage: function () {
                console.log('usage: lessc [option option=parameter ...] <source> [destination]');
                console.log('');
                console.log('If source is set to `-\' (dash or hyphen-minus), input is read from stdin.');
                console.log('');
                console.log('options:');
                console.log('  -h, --help                   Prints help (this message) and exit.');
                console.log('  --include-path=PATHS         Sets include paths. Separated by `:\'. `;\' also supported on windows.');
                console.log('  -M, --depends                Outputs a makefile import dependency list to stdout.');
                console.log('  --no-color                   Disables colorized output.');
                console.log('  --ie-compat                  Enables IE8 compatibility checks.');
                console.log('  --js                         Enables inline JavaScript in less files');
                console.log('  -l, --lint                   Syntax check only (lint).');
                console.log('  -s, --silent                 Suppresses output of error messages.');
                console.log('  --strict-imports             Forces evaluation of imports.');
                console.log('  --insecure                   Allows imports from insecure https hosts.');
                console.log('  -v, --version                Prints version number and exit.');
                console.log('  --verbose                    Be verbose.');
                console.log('  --source-map[=FILENAME]      Outputs a v3 sourcemap to the filename (or output filename.map).');
                console.log('  --source-map-rootpath=X      Adds this path onto the sourcemap filename and less file paths.');
                console.log('  --source-map-basepath=X      Sets sourcemap base path, defaults to current working directory.');
                console.log('  --source-map-include-source  Puts the less files into the map instead of referencing them.');
                console.log('  --source-map-inline          Puts the map (and any less files) as a base64 data uri into the output css file.');
                console.log('  --source-map-url=URL         Sets a custom URL to map file, for sourceMappingURL comment');
                console.log('                               in generated CSS file.');
                console.log('  -rp, --rootpath=URL          Sets rootpath for url rewriting in relative imports and urls');
                console.log('                               Works with or without the relative-urls option.');
                console.log('  -ru=, --rewrite-urls=        Rewrites URLs to make them relative to the base less file.');
                console.log('    all|local|off              \'all\' rewrites all URLs, \'local\' just those starting with a \'.\'');
                console.log('');
                console.log('  -m=, --math=');
                console.log('     always                    Less will eagerly perform math operations always.');
                console.log('     parens-division           Math performed except for division (/) operator');
                console.log('     parens | strict           Math only performed inside parentheses');
                console.log('     strict-legacy             Parens required in very strict terms (legacy --strict-math)');
                console.log('');
                console.log('  -su=on|off                   Allows mixed units, e.g. 1px+1em or 1px*1px which have units');
                console.log('  --strict-units=on|off        that cannot be represented.');
                console.log('  --global-var=\'VAR=VALUE\'     Defines a variable that can be referenced by the file.');
                console.log('  --modify-var=\'VAR=VALUE\'     Modifies a variable already declared in the file.');
                console.log('  --url-args=\'QUERYSTRING\'     Adds params into url tokens (e.g. 42, cb=42 or \'a=1&b=2\')');
                console.log('  --plugin=PLUGIN=OPTIONS      Loads a plugin. You can also omit the --plugin= if the plugin begins');
                console.log('                               less-plugin. E.g. the clean css plugin is called less-plugin-clean-css');
                console.log('                               once installed (npm install less-plugin-clean-css), use either with');
                console.log('                               --plugin=less-plugin-clean-css or just --clean-css');
                console.log('                               specify options afterwards e.g. --plugin=less-plugin-clean-css="advanced"');
                console.log('                               or --clean-css="advanced"');
                console.log('');
                console.log('-------------------------- Deprecated ----------------');
                console.log('  -sm=on|off               Legacy parens-only math. Use --math');
                console.log('  --strict-math=on|off     ');
                console.log('');
                console.log('  --line-numbers=TYPE      Outputs filename and line numbers.');
                console.log('                           TYPE can be either \'comments\', which will output');
                console.log('                           the debug info within comments, \'mediaquery\'');
                console.log('                           that will output the information within a fake');
                console.log('                           media query which is compatible with the SASS');
                console.log('                           format, and \'all\' which will do both.');
                console.log('  -x, --compress           Compresses output by removing some whitespaces.');
                console.log('                           We recommend you use a dedicated minifer like less-plugin-clean-css');
                console.log('');
                console.log('Report bugs to: http://github.com/less/less.js/issues');
                console.log('Home page: <http://lesscss.org/>');
            }
        };
        // Exports helper functions
        for (var h in lessc_helper) {
            if (lessc_helper.hasOwnProperty(h)) {
                exports[h] = lessc_helper[h];
            }
        }
    });
    /**
     * Node Plugin Loader
     */
    var PluginLoader = /** @class */ (function (_super) {
        tslib.__extends(PluginLoader, _super);
        function PluginLoader(less) {
            var _this = _super.call(this) || this;
            _this.less = less;
            _this.require = function (prefix) {
                prefix = path.dirname(prefix);
                return function (id) {
                    var str = id.substr(0, 2);
                    if (str === '..' || str === './') {
                        return require(path.join(prefix, id));
                    }
                    else {
                        return require(id);
                    }
                };
            };
            return _this;
        }
        PluginLoader.prototype.loadPlugin = function (filename, basePath, context, environment, fileManager) {
            var prefix = filename.slice(0, 1);
            var explicit = prefix === '.' || prefix === '/' || filename.slice(-3).toLowerCase() === '.js';
            if (!explicit) {
                context.prefixes = ['less-plugin-', ''];
            }
            return new Promise(function (fulfill, reject) {
                fileManager.loadFile(filename, basePath, context, environment).then(function (data) {
                    try {
                        fulfill(data);
                    }
                    catch (e) {
                        console.log(e);
                        reject(e);
                    }
                }).catch(function (err) {
                    reject(err);
                });
            });
        };
        return PluginLoader;
    }(AbstractPluginLoader));
    // Export a new default each time
    var defaultOptions = (function () {
        return ({
            /* Inline Javascript - @plugin still allowed */
            javascriptEnabled: false,
            /* Outputs a makefile import dependency list to stdout. */
            depends: false,
            /* (DEPRECATED) Compress using less built-in compression.
          * This does an okay job but does not utilise all the tricks of
          * dedicated css compression. */
            compress: false,
            /* Runs the less parser and just reports errors without any output. */
            lint: false,
            /* Sets available include paths.
          * If the file in an @import rule does not exist at that exact location,
          * less will look for it at the location(s) passed to this option.
          * You might use this for instance to specify a path to a library which
          * you want to be referenced simply and relatively in the less files. */
            paths: [],
            /* color output in the terminal */
            color: true,
            /* The strictImports controls whether the compiler will allow an @import inside of either
          * @media blocks or (a later addition) other selector blocks.
          * See: https://github.com/less/less.js/issues/656 */
            strictImports: false,
            /* Allow Imports from Insecure HTTPS Hosts */
            insecure: false,
            /* Allows you to add a path to every generated import and url in your css.
          * This does not affect less import statements that are processed, just ones
          * that are left in the output css. */
            rootpath: '',
            /* By default URLs are kept as-is, so if you import a file in a sub-directory
          * that references an image, exactly the same URL will be output in the css.
          * This option allows you to re-write URL's in imported files so that the
          * URL is always relative to the base imported file */
            rewriteUrls: false,
            /* How to process math
          *   0 always           - eagerly try to solve all operations
          *   1 parens-division  - require parens for division "/"
          *   2 parens | strict  - require parens for all operations
          *   3 strict-legacy    - legacy strict behavior (super-strict)
          */
            math: 0,
            /* Without this option, less attempts to guess at the output unit when it does maths. */
            strictUnits: false,
            /* Effectively the declaration is put at the top of your base Less file,
          * meaning it can be used but it also can be overridden if this variable
          * is defined in the file. */
            globalVars: null,
            /* As opposed to the global variable option, this puts the declaration at the
          * end of your base file, meaning it will override anything defined in your Less file. */
            modifyVars: null,
            /* This option allows you to specify a argument to go on to every URL.  */
            urlArgs: ''
        });
    });
    var imageSize = (function (environment) {
        function imageSize(functionContext, filePathNode) {
            var filePath = filePathNode.value;
            var currentFileInfo = functionContext.currentFileInfo;
            var currentDirectory = currentFileInfo.rewriteUrls ?
                currentFileInfo.currentDirectory : currentFileInfo.entryPath;
            var fragmentStart = filePath.indexOf('#');
            var fragment = '';
            if (fragmentStart !== -1) {
                fragment = filePath.slice(fragmentStart);
                filePath = filePath.slice(0, fragmentStart);
            }
            var fileManager = environment.getFileManager(filePath, currentDirectory, functionContext.context, environment, true);
            if (!fileManager) {
                throw {
                    type: 'File',
                    message: "Can not set up FileManager for " + filePathNode
                };
            }
            var fileSync = fileManager.loadFileSync(filePath, currentDirectory, functionContext.context, environment);
            if (fileSync.error) {
                throw fileSync.error;
            }
            var sizeOf = require(963) /* image-size */;
            return sizeOf(fileSync.filename);
        }
        var imageFunctions = {
            'image-size': function (filePathNode) {
                var size = imageSize(this, filePathNode);
                return new Expression([
                    new Dimension(size.width, 'px'),
                    new Dimension(size.height, 'px')
                ]);
            },
            'image-width': function (filePathNode) {
                var size = imageSize(this, filePathNode);
                return new Dimension(size.width, 'px');
            },
            'image-height': function (filePathNode) {
                var size = imageSize(this, filePathNode);
                return new Dimension(size.height, 'px');
            }
        };
        functionRegistry.addMultiple(imageFunctions);
    });
    var less = createFromEnvironment(environment, [new FileManager(), new UrlFileManager()]);
    // allow people to create less with their own environment
    less.createFromEnvironment = createFromEnvironment;
    less.lesscHelper = lesscHelper;
    less.PluginLoader = PluginLoader;
    less.fs = fs$1;
    less.FileManager = FileManager;
    less.UrlFileManager = UrlFileManager;
    // Set up options
    less.options = defaultOptions();
    // provide image-size functionality
    imageSize(less.environment);
    module.exports = less;
},
/* clone/clone.js */ function _(require, module, exports) {
    var clone = (function () {
        'use strict';
        function _instanceof(obj, type) {
            return type != null && obj instanceof type;
        }
        var nativeMap;
        try {
            nativeMap = Map;
        }
        catch (_) {
            // maybe a reference error because no `Map`. Give it a dummy value that no
            // value will ever be an instanceof.
            nativeMap = function () { };
        }
        var nativeSet;
        try {
            nativeSet = Set;
        }
        catch (_) {
            nativeSet = function () { };
        }
        var nativePromise;
        try {
            nativePromise = Promise;
        }
        catch (_) {
            nativePromise = function () { };
        }
        /**
         * Clones (copies) an Object using deep copying.
         *
         * This function supports circular references by default, but if you are certain
         * there are no circular references in your object, you can save some CPU time
         * by calling clone(obj, false).
         *
         * Caution: if `circular` is false and `parent` contains circular references,
         * your program may enter an infinite loop and crash.
         *
         * @param `parent` - the object to be cloned
         * @param `circular` - set to true if the object to be cloned may contain
         *    circular references. (optional - true by default)
         * @param `depth` - set to a number if the object is only to be cloned to
         *    a particular depth. (optional - defaults to Infinity)
         * @param `prototype` - sets the prototype to be used when cloning an object.
         *    (optional - defaults to parent prototype).
         * @param `includeNonEnumerable` - set to true if the non-enumerable properties
         *    should be cloned as well. Non-enumerable properties on the prototype
         *    chain will be ignored. (optional - false by default)
        */
        function clone(parent, circular, depth, prototype, includeNonEnumerable) {
            if (typeof circular === 'object') {
                depth = circular.depth;
                prototype = circular.prototype;
                includeNonEnumerable = circular.includeNonEnumerable;
                circular = circular.circular;
            }
            // maintain two arrays for circular references, where corresponding parents
            // and children have the same index
            var allParents = [];
            var allChildren = [];
            var useBuffer = typeof Buffer != 'undefined';
            if (typeof circular == 'undefined')
                circular = true;
            if (typeof depth == 'undefined')
                depth = Infinity;
            // recurse this function so we don't reset allParents and allChildren
            function _clone(parent, depth) {
                // cloning null always returns null
                if (parent === null)
                    return null;
                if (depth === 0)
                    return parent;
                var child;
                var proto;
                if (typeof parent != 'object') {
                    return parent;
                }
                if (_instanceof(parent, nativeMap)) {
                    child = new nativeMap();
                }
                else if (_instanceof(parent, nativeSet)) {
                    child = new nativeSet();
                }
                else if (_instanceof(parent, nativePromise)) {
                    child = new nativePromise(function (resolve, reject) {
                        parent.then(function (value) {
                            resolve(_clone(value, depth - 1));
                        }, function (err) {
                            reject(_clone(err, depth - 1));
                        });
                    });
                }
                else if (clone.__isArray(parent)) {
                    child = [];
                }
                else if (clone.__isRegExp(parent)) {
                    child = new RegExp(parent.source, __getRegExpFlags(parent));
                    if (parent.lastIndex)
                        child.lastIndex = parent.lastIndex;
                }
                else if (clone.__isDate(parent)) {
                    child = new Date(parent.getTime());
                }
                else if (useBuffer && Buffer.isBuffer(parent)) {
                    if (Buffer.allocUnsafe) {
                        // Node.js >= 4.5.0
                        child = Buffer.allocUnsafe(parent.length);
                    }
                    else {
                        // Older Node.js versions
                        child = new Buffer(parent.length);
                    }
                    parent.copy(child);
                    return child;
                }
                else if (_instanceof(parent, Error)) {
                    child = Object.create(parent);
                }
                else {
                    if (typeof prototype == 'undefined') {
                        proto = Object.getPrototypeOf(parent);
                        child = Object.create(proto);
                    }
                    else {
                        child = Object.create(prototype);
                        proto = prototype;
                    }
                }
                if (circular) {
                    var index = allParents.indexOf(parent);
                    if (index != -1) {
                        return allChildren[index];
                    }
                    allParents.push(parent);
                    allChildren.push(child);
                }
                if (_instanceof(parent, nativeMap)) {
                    parent.forEach(function (value, key) {
                        var keyChild = _clone(key, depth - 1);
                        var valueChild = _clone(value, depth - 1);
                        child.set(keyChild, valueChild);
                    });
                }
                if (_instanceof(parent, nativeSet)) {
                    parent.forEach(function (value) {
                        var entryChild = _clone(value, depth - 1);
                        child.add(entryChild);
                    });
                }
                for (var i in parent) {
                    var attrs;
                    if (proto) {
                        attrs = Object.getOwnPropertyDescriptor(proto, i);
                    }
                    if (attrs && attrs.set == null) {
                        continue;
                    }
                    child[i] = _clone(parent[i], depth - 1);
                }
                if (Object.getOwnPropertySymbols) {
                    var symbols = Object.getOwnPropertySymbols(parent);
                    for (var i = 0; i < symbols.length; i++) {
                        // Don't need to worry about cloning a symbol because it is a primitive,
                        // like a number or string.
                        var symbol = symbols[i];
                        var descriptor = Object.getOwnPropertyDescriptor(parent, symbol);
                        if (descriptor && !descriptor.enumerable && !includeNonEnumerable) {
                            continue;
                        }
                        child[symbol] = _clone(parent[symbol], depth - 1);
                        if (!descriptor.enumerable) {
                            Object.defineProperty(child, symbol, {
                                enumerable: false
                            });
                        }
                    }
                }
                if (includeNonEnumerable) {
                    var allPropertyNames = Object.getOwnPropertyNames(parent);
                    for (var i = 0; i < allPropertyNames.length; i++) {
                        var propertyName = allPropertyNames[i];
                        var descriptor = Object.getOwnPropertyDescriptor(parent, propertyName);
                        if (descriptor && descriptor.enumerable) {
                            continue;
                        }
                        child[propertyName] = _clone(parent[propertyName], depth - 1);
                        Object.defineProperty(child, propertyName, {
                            enumerable: false
                        });
                    }
                }
                return child;
            }
            return _clone(parent, depth);
        }
        /**
         * Simple flat clone using prototype, accepts only objects, usefull for property
         * override on FLAT configuration object (no nested props).
         *
         * USE WITH CAUTION! This may not behave as you wish if you do not know how this
         * works.
         */
        clone.clonePrototype = function clonePrototype(parent) {
            if (parent === null)
                return null;
            var c = function () { };
            c.prototype = parent;
            return new c();
        };
        // private utility functions
        function __objToStr(o) {
            return Object.prototype.toString.call(o);
        }
        clone.__objToStr = __objToStr;
        function __isDate(o) {
            return typeof o === 'object' && __objToStr(o) === '[object Date]';
        }
        clone.__isDate = __isDate;
        function __isArray(o) {
            return typeof o === 'object' && __objToStr(o) === '[object Array]';
        }
        clone.__isArray = __isArray;
        function __isRegExp(o) {
            return typeof o === 'object' && __objToStr(o) === '[object RegExp]';
        }
        clone.__isRegExp = __isRegExp;
        function __getRegExpFlags(re) {
            var flags = '';
            if (re.global)
                flags += 'g';
            if (re.ignoreCase)
                flags += 'i';
            if (re.multiline)
                flags += 'm';
            return flags;
        }
        clone.__getRegExpFlags = __getRegExpFlags;
        return clone;
    })();
    if (typeof module === 'object' && module.exports) {
        module.exports = clone;
    }
},
/* mime/mime.js */ function _(require, module, exports) {
    var path = require('path');
    var fs = require('fs');
    function Mime() {
        // Map of extension -> mime type
        this.types = Object.create(null);
        // Map of mime type -> extension
        this.extensions = Object.create(null);
    }
    /**
     * Define mimetype -> extension mappings.  Each key is a mime-type that maps
     * to an array of extensions associated with the type.  The first extension is
     * used as the default extension for the type.
     *
     * e.g. mime.define({'audio/ogg', ['oga', 'ogg', 'spx']});
     *
     * @param map (Object) type definitions
     */
    Mime.prototype.define = function (map) {
        for (var type in map) {
            var exts = map[type];
            for (var i = 0; i < exts.length; i++) {
                if (process.env.DEBUG_MIME && this.types[exts[i]]) {
                    console.warn((this._loading || "define()").replace(/.*\//, ''), 'changes "' + exts[i] + '" extension type from ' +
                        this.types[exts[i]] + ' to ' + type);
                }
                this.types[exts[i]] = type;
            }
            // Default extension is the first one we encounter
            if (!this.extensions[type]) {
                this.extensions[type] = exts[0];
            }
        }
    };
    /**
     * Load an Apache2-style ".types" file
     *
     * This may be called multiple times (it's expected).  Where files declare
     * overlapping types/extensions, the last file wins.
     *
     * @param file (String) path of file to load.
     */
    Mime.prototype.load = function (file) {
        this._loading = file;
        // Read file and split into lines
        var map = {}, content = fs.readFileSync(file, 'ascii'), lines = content.split(/[\r\n]+/);
        lines.forEach(function (line) {
            // Clean up whitespace/comments, and split into fields
            var fields = line.replace(/\s*#.*|^\s*|\s*$/g, '').split(/\s+/);
            map[fields.shift()] = fields;
        });
        this.define(map);
        this._loading = null;
    };
    /**
     * Lookup a mime type based on extension
     */
    Mime.prototype.lookup = function (path, fallback) {
        var ext = path.replace(/^.*[\.\/\\]/, '').toLowerCase();
        return this.types[ext] || fallback || this.default_type;
    };
    /**
     * Return file extension associated with a mime type
     */
    Mime.prototype.extension = function (mimeType) {
        var type = mimeType.match(/^\s*([^;\s]*)(?:;|\s|$)/)[1].toLowerCase();
        return this.extensions[type];
    };
    // Default instance
    var mime = new Mime();
    // Define built-in types
    mime.define(require(828) /* ./types.json */);
    // Default type
    mime.default_type = mime.lookup('bin');
    //
    // Additional API specific to the default instance
    //
    mime.Mime = Mime;
    /**
     * Lookup a charset based on mime type.
     */
    mime.charsets = {
        lookup: function (mimeType, fallback) {
            // Assume text types are utf8
            return (/^text\/|^application\/(javascript|json)/).test(mimeType) ? 'UTF-8' : fallback;
        }
    };
    module.exports = mime;
},
/* mime/types.json */ function _(require, module, exports) {
    module.exports = { "application/andrew-inset": ["ez"], "application/applixware": ["aw"], "application/atom+xml": ["atom"], "application/atomcat+xml": ["atomcat"], "application/atomsvc+xml": ["atomsvc"], "application/bdoc": ["bdoc"], "application/ccxml+xml": ["ccxml"], "application/cdmi-capability": ["cdmia"], "application/cdmi-container": ["cdmic"], "application/cdmi-domain": ["cdmid"], "application/cdmi-object": ["cdmio"], "application/cdmi-queue": ["cdmiq"], "application/cu-seeme": ["cu"], "application/dash+xml": ["mpd"], "application/davmount+xml": ["davmount"], "application/docbook+xml": ["dbk"], "application/dssc+der": ["dssc"], "application/dssc+xml": ["xdssc"], "application/ecmascript": ["ecma"], "application/emma+xml": ["emma"], "application/epub+zip": ["epub"], "application/exi": ["exi"], "application/font-tdpfr": ["pfr"], "application/font-woff": [], "application/font-woff2": [], "application/geo+json": ["geojson"], "application/gml+xml": ["gml"], "application/gpx+xml": ["gpx"], "application/gxf": ["gxf"], "application/gzip": ["gz"], "application/hyperstudio": ["stk"], "application/inkml+xml": ["ink", "inkml"], "application/ipfix": ["ipfix"], "application/java-archive": ["jar", "war", "ear"], "application/java-serialized-object": ["ser"], "application/java-vm": ["class"], "application/javascript": ["js", "mjs"], "application/json": ["json", "map"], "application/json5": ["json5"], "application/jsonml+json": ["jsonml"], "application/ld+json": ["jsonld"], "application/lost+xml": ["lostxml"], "application/mac-binhex40": ["hqx"], "application/mac-compactpro": ["cpt"], "application/mads+xml": ["mads"], "application/manifest+json": ["webmanifest"], "application/marc": ["mrc"], "application/marcxml+xml": ["mrcx"], "application/mathematica": ["ma", "nb", "mb"], "application/mathml+xml": ["mathml"], "application/mbox": ["mbox"], "application/mediaservercontrol+xml": ["mscml"], "application/metalink+xml": ["metalink"], "application/metalink4+xml": ["meta4"], "application/mets+xml": ["mets"], "application/mods+xml": ["mods"], "application/mp21": ["m21", "mp21"], "application/mp4": ["mp4s", "m4p"], "application/msword": ["doc", "dot"], "application/mxf": ["mxf"], "application/octet-stream": ["bin", "dms", "lrf", "mar", "so", "dist", "distz", "pkg", "bpk", "dump", "elc", "deploy", "exe", "dll", "deb", "dmg", "iso", "img", "msi", "msp", "msm", "buffer"], "application/oda": ["oda"], "application/oebps-package+xml": ["opf"], "application/ogg": ["ogx"], "application/omdoc+xml": ["omdoc"], "application/onenote": ["onetoc", "onetoc2", "onetmp", "onepkg"], "application/oxps": ["oxps"], "application/patch-ops-error+xml": ["xer"], "application/pdf": ["pdf"], "application/pgp-encrypted": ["pgp"], "application/pgp-signature": ["asc", "sig"], "application/pics-rules": ["prf"], "application/pkcs10": ["p10"], "application/pkcs7-mime": ["p7m", "p7c"], "application/pkcs7-signature": ["p7s"], "application/pkcs8": ["p8"], "application/pkix-attr-cert": ["ac"], "application/pkix-cert": ["cer"], "application/pkix-crl": ["crl"], "application/pkix-pkipath": ["pkipath"], "application/pkixcmp": ["pki"], "application/pls+xml": ["pls"], "application/postscript": ["ai", "eps", "ps"], "application/prs.cww": ["cww"], "application/pskc+xml": ["pskcxml"], "application/raml+yaml": ["raml"], "application/rdf+xml": ["rdf"], "application/reginfo+xml": ["rif"], "application/relax-ng-compact-syntax": ["rnc"], "application/resource-lists+xml": ["rl"], "application/resource-lists-diff+xml": ["rld"], "application/rls-services+xml": ["rs"], "application/rpki-ghostbusters": ["gbr"], "application/rpki-manifest": ["mft"], "application/rpki-roa": ["roa"], "application/rsd+xml": ["rsd"], "application/rss+xml": ["rss"], "application/rtf": ["rtf"], "application/sbml+xml": ["sbml"], "application/scvp-cv-request": ["scq"], "application/scvp-cv-response": ["scs"], "application/scvp-vp-request": ["spq"], "application/scvp-vp-response": ["spp"], "application/sdp": ["sdp"], "application/set-payment-initiation": ["setpay"], "application/set-registration-initiation": ["setreg"], "application/shf+xml": ["shf"], "application/smil+xml": ["smi", "smil"], "application/sparql-query": ["rq"], "application/sparql-results+xml": ["srx"], "application/srgs": ["gram"], "application/srgs+xml": ["grxml"], "application/sru+xml": ["sru"], "application/ssdl+xml": ["ssdl"], "application/ssml+xml": ["ssml"], "application/tei+xml": ["tei", "teicorpus"], "application/thraud+xml": ["tfi"], "application/timestamped-data": ["tsd"], "application/vnd.3gpp.pic-bw-large": ["plb"], "application/vnd.3gpp.pic-bw-small": ["psb"], "application/vnd.3gpp.pic-bw-var": ["pvb"], "application/vnd.3gpp2.tcap": ["tcap"], "application/vnd.3m.post-it-notes": ["pwn"], "application/vnd.accpac.simply.aso": ["aso"], "application/vnd.accpac.simply.imp": ["imp"], "application/vnd.acucobol": ["acu"], "application/vnd.acucorp": ["atc", "acutc"], "application/vnd.adobe.air-application-installer-package+zip": ["air"], "application/vnd.adobe.formscentral.fcdt": ["fcdt"], "application/vnd.adobe.fxp": ["fxp", "fxpl"], "application/vnd.adobe.xdp+xml": ["xdp"], "application/vnd.adobe.xfdf": ["xfdf"], "application/vnd.ahead.space": ["ahead"], "application/vnd.airzip.filesecure.azf": ["azf"], "application/vnd.airzip.filesecure.azs": ["azs"], "application/vnd.amazon.ebook": ["azw"], "application/vnd.americandynamics.acc": ["acc"], "application/vnd.amiga.ami": ["ami"], "application/vnd.android.package-archive": ["apk"], "application/vnd.anser-web-certificate-issue-initiation": ["cii"], "application/vnd.anser-web-funds-transfer-initiation": ["fti"], "application/vnd.antix.game-component": ["atx"], "application/vnd.apple.installer+xml": ["mpkg"], "application/vnd.apple.mpegurl": ["m3u8"], "application/vnd.apple.pkpass": ["pkpass"], "application/vnd.aristanetworks.swi": ["swi"], "application/vnd.astraea-software.iota": ["iota"], "application/vnd.audiograph": ["aep"], "application/vnd.blueice.multipass": ["mpm"], "application/vnd.bmi": ["bmi"], "application/vnd.businessobjects": ["rep"], "application/vnd.chemdraw+xml": ["cdxml"], "application/vnd.chipnuts.karaoke-mmd": ["mmd"], "application/vnd.cinderella": ["cdy"], "application/vnd.claymore": ["cla"], "application/vnd.cloanto.rp9": ["rp9"], "application/vnd.clonk.c4group": ["c4g", "c4d", "c4f", "c4p", "c4u"], "application/vnd.cluetrust.cartomobile-config": ["c11amc"], "application/vnd.cluetrust.cartomobile-config-pkg": ["c11amz"], "application/vnd.commonspace": ["csp"], "application/vnd.contact.cmsg": ["cdbcmsg"], "application/vnd.cosmocaller": ["cmc"], "application/vnd.crick.clicker": ["clkx"], "application/vnd.crick.clicker.keyboard": ["clkk"], "application/vnd.crick.clicker.palette": ["clkp"], "application/vnd.crick.clicker.template": ["clkt"], "application/vnd.crick.clicker.wordbank": ["clkw"], "application/vnd.criticaltools.wbs+xml": ["wbs"], "application/vnd.ctc-posml": ["pml"], "application/vnd.cups-ppd": ["ppd"], "application/vnd.curl.car": ["car"], "application/vnd.curl.pcurl": ["pcurl"], "application/vnd.dart": ["dart"], "application/vnd.data-vision.rdz": ["rdz"], "application/vnd.dece.data": ["uvf", "uvvf", "uvd", "uvvd"], "application/vnd.dece.ttml+xml": ["uvt", "uvvt"], "application/vnd.dece.unspecified": ["uvx", "uvvx"], "application/vnd.dece.zip": ["uvz", "uvvz"], "application/vnd.denovo.fcselayout-link": ["fe_launch"], "application/vnd.dna": ["dna"], "application/vnd.dolby.mlp": ["mlp"], "application/vnd.dpgraph": ["dpg"], "application/vnd.dreamfactory": ["dfac"], "application/vnd.ds-keypoint": ["kpxx"], "application/vnd.dvb.ait": ["ait"], "application/vnd.dvb.service": ["svc"], "application/vnd.dynageo": ["geo"], "application/vnd.ecowin.chart": ["mag"], "application/vnd.enliven": ["nml"], "application/vnd.epson.esf": ["esf"], "application/vnd.epson.msf": ["msf"], "application/vnd.epson.quickanime": ["qam"], "application/vnd.epson.salt": ["slt"], "application/vnd.epson.ssf": ["ssf"], "application/vnd.eszigno3+xml": ["es3", "et3"], "application/vnd.ezpix-album": ["ez2"], "application/vnd.ezpix-package": ["ez3"], "application/vnd.fdf": ["fdf"], "application/vnd.fdsn.mseed": ["mseed"], "application/vnd.fdsn.seed": ["seed", "dataless"], "application/vnd.flographit": ["gph"], "application/vnd.fluxtime.clip": ["ftc"], "application/vnd.framemaker": ["fm", "frame", "maker", "book"], "application/vnd.frogans.fnc": ["fnc"], "application/vnd.frogans.ltf": ["ltf"], "application/vnd.fsc.weblaunch": ["fsc"], "application/vnd.fujitsu.oasys": ["oas"], "application/vnd.fujitsu.oasys2": ["oa2"], "application/vnd.fujitsu.oasys3": ["oa3"], "application/vnd.fujitsu.oasysgp": ["fg5"], "application/vnd.fujitsu.oasysprs": ["bh2"], "application/vnd.fujixerox.ddd": ["ddd"], "application/vnd.fujixerox.docuworks": ["xdw"], "application/vnd.fujixerox.docuworks.binder": ["xbd"], "application/vnd.fuzzysheet": ["fzs"], "application/vnd.genomatix.tuxedo": ["txd"], "application/vnd.geogebra.file": ["ggb"], "application/vnd.geogebra.tool": ["ggt"], "application/vnd.geometry-explorer": ["gex", "gre"], "application/vnd.geonext": ["gxt"], "application/vnd.geoplan": ["g2w"], "application/vnd.geospace": ["g3w"], "application/vnd.gmx": ["gmx"], "application/vnd.google-apps.document": ["gdoc"], "application/vnd.google-apps.presentation": ["gslides"], "application/vnd.google-apps.spreadsheet": ["gsheet"], "application/vnd.google-earth.kml+xml": ["kml"], "application/vnd.google-earth.kmz": ["kmz"], "application/vnd.grafeq": ["gqf", "gqs"], "application/vnd.groove-account": ["gac"], "application/vnd.groove-help": ["ghf"], "application/vnd.groove-identity-message": ["gim"], "application/vnd.groove-injector": ["grv"], "application/vnd.groove-tool-message": ["gtm"], "application/vnd.groove-tool-template": ["tpl"], "application/vnd.groove-vcard": ["vcg"], "application/vnd.hal+xml": ["hal"], "application/vnd.handheld-entertainment+xml": ["zmm"], "application/vnd.hbci": ["hbci"], "application/vnd.hhe.lesson-player": ["les"], "application/vnd.hp-hpgl": ["hpgl"], "application/vnd.hp-hpid": ["hpid"], "application/vnd.hp-hps": ["hps"], "application/vnd.hp-jlyt": ["jlt"], "application/vnd.hp-pcl": ["pcl"], "application/vnd.hp-pclxl": ["pclxl"], "application/vnd.hydrostatix.sof-data": ["sfd-hdstx"], "application/vnd.ibm.minipay": ["mpy"], "application/vnd.ibm.modcap": ["afp", "listafp", "list3820"], "application/vnd.ibm.rights-management": ["irm"], "application/vnd.ibm.secure-container": ["sc"], "application/vnd.iccprofile": ["icc", "icm"], "application/vnd.igloader": ["igl"], "application/vnd.immervision-ivp": ["ivp"], "application/vnd.immervision-ivu": ["ivu"], "application/vnd.insors.igm": ["igm"], "application/vnd.intercon.formnet": ["xpw", "xpx"], "application/vnd.intergeo": ["i2g"], "application/vnd.intu.qbo": ["qbo"], "application/vnd.intu.qfx": ["qfx"], "application/vnd.ipunplugged.rcprofile": ["rcprofile"], "application/vnd.irepository.package+xml": ["irp"], "application/vnd.is-xpr": ["xpr"], "application/vnd.isac.fcs": ["fcs"], "application/vnd.jam": ["jam"], "application/vnd.jcp.javame.midlet-rms": ["rms"], "application/vnd.jisp": ["jisp"], "application/vnd.joost.joda-archive": ["joda"], "application/vnd.kahootz": ["ktz", "ktr"], "application/vnd.kde.karbon": ["karbon"], "application/vnd.kde.kchart": ["chrt"], "application/vnd.kde.kformula": ["kfo"], "application/vnd.kde.kivio": ["flw"], "application/vnd.kde.kontour": ["kon"], "application/vnd.kde.kpresenter": ["kpr", "kpt"], "application/vnd.kde.kspread": ["ksp"], "application/vnd.kde.kword": ["kwd", "kwt"], "application/vnd.kenameaapp": ["htke"], "application/vnd.kidspiration": ["kia"], "application/vnd.kinar": ["kne", "knp"], "application/vnd.koan": ["skp", "skd", "skt", "skm"], "application/vnd.kodak-descriptor": ["sse"], "application/vnd.las.las+xml": ["lasxml"], "application/vnd.llamagraphics.life-balance.desktop": ["lbd"], "application/vnd.llamagraphics.life-balance.exchange+xml": ["lbe"], "application/vnd.lotus-1-2-3": ["123"], "application/vnd.lotus-approach": ["apr"], "application/vnd.lotus-freelance": ["pre"], "application/vnd.lotus-notes": ["nsf"], "application/vnd.lotus-organizer": ["org"], "application/vnd.lotus-screencam": ["scm"], "application/vnd.lotus-wordpro": ["lwp"], "application/vnd.macports.portpkg": ["portpkg"], "application/vnd.mcd": ["mcd"], "application/vnd.medcalcdata": ["mc1"], "application/vnd.mediastation.cdkey": ["cdkey"], "application/vnd.mfer": ["mwf"], "application/vnd.mfmp": ["mfm"], "application/vnd.micrografx.flo": ["flo"], "application/vnd.micrografx.igx": ["igx"], "application/vnd.mif": ["mif"], "application/vnd.mobius.daf": ["daf"], "application/vnd.mobius.dis": ["dis"], "application/vnd.mobius.mbk": ["mbk"], "application/vnd.mobius.mqy": ["mqy"], "application/vnd.mobius.msl": ["msl"], "application/vnd.mobius.plc": ["plc"], "application/vnd.mobius.txf": ["txf"], "application/vnd.mophun.application": ["mpn"], "application/vnd.mophun.certificate": ["mpc"], "application/vnd.mozilla.xul+xml": ["xul"], "application/vnd.ms-artgalry": ["cil"], "application/vnd.ms-cab-compressed": ["cab"], "application/vnd.ms-excel": ["xls", "xlm", "xla", "xlc", "xlt", "xlw"], "application/vnd.ms-excel.addin.macroenabled.12": ["xlam"], "application/vnd.ms-excel.sheet.binary.macroenabled.12": ["xlsb"], "application/vnd.ms-excel.sheet.macroenabled.12": ["xlsm"], "application/vnd.ms-excel.template.macroenabled.12": ["xltm"], "application/vnd.ms-fontobject": ["eot"], "application/vnd.ms-htmlhelp": ["chm"], "application/vnd.ms-ims": ["ims"], "application/vnd.ms-lrm": ["lrm"], "application/vnd.ms-officetheme": ["thmx"], "application/vnd.ms-outlook": ["msg"], "application/vnd.ms-pki.seccat": ["cat"], "application/vnd.ms-pki.stl": ["stl"], "application/vnd.ms-powerpoint": ["ppt", "pps", "pot"], "application/vnd.ms-powerpoint.addin.macroenabled.12": ["ppam"], "application/vnd.ms-powerpoint.presentation.macroenabled.12": ["pptm"], "application/vnd.ms-powerpoint.slide.macroenabled.12": ["sldm"], "application/vnd.ms-powerpoint.slideshow.macroenabled.12": ["ppsm"], "application/vnd.ms-powerpoint.template.macroenabled.12": ["potm"], "application/vnd.ms-project": ["mpp", "mpt"], "application/vnd.ms-word.document.macroenabled.12": ["docm"], "application/vnd.ms-word.template.macroenabled.12": ["dotm"], "application/vnd.ms-works": ["wps", "wks", "wcm", "wdb"], "application/vnd.ms-wpl": ["wpl"], "application/vnd.ms-xpsdocument": ["xps"], "application/vnd.mseq": ["mseq"], "application/vnd.musician": ["mus"], "application/vnd.muvee.style": ["msty"], "application/vnd.mynfc": ["taglet"], "application/vnd.neurolanguage.nlu": ["nlu"], "application/vnd.nitf": ["ntf", "nitf"], "application/vnd.noblenet-directory": ["nnd"], "application/vnd.noblenet-sealer": ["nns"], "application/vnd.noblenet-web": ["nnw"], "application/vnd.nokia.n-gage.data": ["ngdat"], "application/vnd.nokia.n-gage.symbian.install": ["n-gage"], "application/vnd.nokia.radio-preset": ["rpst"], "application/vnd.nokia.radio-presets": ["rpss"], "application/vnd.novadigm.edm": ["edm"], "application/vnd.novadigm.edx": ["edx"], "application/vnd.novadigm.ext": ["ext"], "application/vnd.oasis.opendocument.chart": ["odc"], "application/vnd.oasis.opendocument.chart-template": ["otc"], "application/vnd.oasis.opendocument.database": ["odb"], "application/vnd.oasis.opendocument.formula": ["odf"], "application/vnd.oasis.opendocument.formula-template": ["odft"], "application/vnd.oasis.opendocument.graphics": ["odg"], "application/vnd.oasis.opendocument.graphics-template": ["otg"], "application/vnd.oasis.opendocument.image": ["odi"], "application/vnd.oasis.opendocument.image-template": ["oti"], "application/vnd.oasis.opendocument.presentation": ["odp"], "application/vnd.oasis.opendocument.presentation-template": ["otp"], "application/vnd.oasis.opendocument.spreadsheet": ["ods"], "application/vnd.oasis.opendocument.spreadsheet-template": ["ots"], "application/vnd.oasis.opendocument.text": ["odt"], "application/vnd.oasis.opendocument.text-master": ["odm"], "application/vnd.oasis.opendocument.text-template": ["ott"], "application/vnd.oasis.opendocument.text-web": ["oth"], "application/vnd.olpc-sugar": ["xo"], "application/vnd.oma.dd2+xml": ["dd2"], "application/vnd.openofficeorg.extension": ["oxt"], "application/vnd.openxmlformats-officedocument.presentationml.presentation": ["pptx"], "application/vnd.openxmlformats-officedocument.presentationml.slide": ["sldx"], "application/vnd.openxmlformats-officedocument.presentationml.slideshow": ["ppsx"], "application/vnd.openxmlformats-officedocument.presentationml.template": ["potx"], "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet": ["xlsx"], "application/vnd.openxmlformats-officedocument.spreadsheetml.template": ["xltx"], "application/vnd.openxmlformats-officedocument.wordprocessingml.document": ["docx"], "application/vnd.openxmlformats-officedocument.wordprocessingml.template": ["dotx"], "application/vnd.osgeo.mapguide.package": ["mgp"], "application/vnd.osgi.dp": ["dp"], "application/vnd.osgi.subsystem": ["esa"], "application/vnd.palm": ["pdb", "pqa", "oprc"], "application/vnd.pawaafile": ["paw"], "application/vnd.pg.format": ["str"], "application/vnd.pg.osasli": ["ei6"], "application/vnd.picsel": ["efif"], "application/vnd.pmi.widget": ["wg"], "application/vnd.pocketlearn": ["plf"], "application/vnd.powerbuilder6": ["pbd"], "application/vnd.previewsystems.box": ["box"], "application/vnd.proteus.magazine": ["mgz"], "application/vnd.publishare-delta-tree": ["qps"], "application/vnd.pvi.ptid1": ["ptid"], "application/vnd.quark.quarkxpress": ["qxd", "qxt", "qwd", "qwt", "qxl", "qxb"], "application/vnd.realvnc.bed": ["bed"], "application/vnd.recordare.musicxml": ["mxl"], "application/vnd.recordare.musicxml+xml": ["musicxml"], "application/vnd.rig.cryptonote": ["cryptonote"], "application/vnd.rim.cod": ["cod"], "application/vnd.rn-realmedia": ["rm"], "application/vnd.rn-realmedia-vbr": ["rmvb"], "application/vnd.route66.link66+xml": ["link66"], "application/vnd.sailingtracker.track": ["st"], "application/vnd.seemail": ["see"], "application/vnd.sema": ["sema"], "application/vnd.semd": ["semd"], "application/vnd.semf": ["semf"], "application/vnd.shana.informed.formdata": ["ifm"], "application/vnd.shana.informed.formtemplate": ["itp"], "application/vnd.shana.informed.interchange": ["iif"], "application/vnd.shana.informed.package": ["ipk"], "application/vnd.simtech-mindmapper": ["twd", "twds"], "application/vnd.smaf": ["mmf"], "application/vnd.smart.teacher": ["teacher"], "application/vnd.solent.sdkm+xml": ["sdkm", "sdkd"], "application/vnd.spotfire.dxp": ["dxp"], "application/vnd.spotfire.sfs": ["sfs"], "application/vnd.stardivision.calc": ["sdc"], "application/vnd.stardivision.draw": ["sda"], "application/vnd.stardivision.impress": ["sdd"], "application/vnd.stardivision.math": ["smf"], "application/vnd.stardivision.writer": ["sdw", "vor"], "application/vnd.stardivision.writer-global": ["sgl"], "application/vnd.stepmania.package": ["smzip"], "application/vnd.stepmania.stepchart": ["sm"], "application/vnd.sun.wadl+xml": ["wadl"], "application/vnd.sun.xml.calc": ["sxc"], "application/vnd.sun.xml.calc.template": ["stc"], "application/vnd.sun.xml.draw": ["sxd"], "application/vnd.sun.xml.draw.template": ["std"], "application/vnd.sun.xml.impress": ["sxi"], "application/vnd.sun.xml.impress.template": ["sti"], "application/vnd.sun.xml.math": ["sxm"], "application/vnd.sun.xml.writer": ["sxw"], "application/vnd.sun.xml.writer.global": ["sxg"], "application/vnd.sun.xml.writer.template": ["stw"], "application/vnd.sus-calendar": ["sus", "susp"], "application/vnd.svd": ["svd"], "application/vnd.symbian.install": ["sis", "sisx"], "application/vnd.syncml+xml": ["xsm"], "application/vnd.syncml.dm+wbxml": ["bdm"], "application/vnd.syncml.dm+xml": ["xdm"], "application/vnd.tao.intent-module-archive": ["tao"], "application/vnd.tcpdump.pcap": ["pcap", "cap", "dmp"], "application/vnd.tmobile-livetv": ["tmo"], "application/vnd.trid.tpt": ["tpt"], "application/vnd.triscape.mxs": ["mxs"], "application/vnd.trueapp": ["tra"], "application/vnd.ufdl": ["ufd", "ufdl"], "application/vnd.uiq.theme": ["utz"], "application/vnd.umajin": ["umj"], "application/vnd.unity": ["unityweb"], "application/vnd.uoml+xml": ["uoml"], "application/vnd.vcx": ["vcx"], "application/vnd.visio": ["vsd", "vst", "vss", "vsw"], "application/vnd.visionary": ["vis"], "application/vnd.vsf": ["vsf"], "application/vnd.wap.wbxml": ["wbxml"], "application/vnd.wap.wmlc": ["wmlc"], "application/vnd.wap.wmlscriptc": ["wmlsc"], "application/vnd.webturbo": ["wtb"], "application/vnd.wolfram.player": ["nbp"], "application/vnd.wordperfect": ["wpd"], "application/vnd.wqd": ["wqd"], "application/vnd.wt.stf": ["stf"], "application/vnd.xara": ["xar"], "application/vnd.xfdl": ["xfdl"], "application/vnd.yamaha.hv-dic": ["hvd"], "application/vnd.yamaha.hv-script": ["hvs"], "application/vnd.yamaha.hv-voice": ["hvp"], "application/vnd.yamaha.openscoreformat": ["osf"], "application/vnd.yamaha.openscoreformat.osfpvg+xml": ["osfpvg"], "application/vnd.yamaha.smaf-audio": ["saf"], "application/vnd.yamaha.smaf-phrase": ["spf"], "application/vnd.yellowriver-custom-menu": ["cmp"], "application/vnd.zul": ["zir", "zirz"], "application/vnd.zzazz.deck+xml": ["zaz"], "application/voicexml+xml": ["vxml"], "application/wasm": ["wasm"], "application/widget": ["wgt"], "application/winhlp": ["hlp"], "application/wsdl+xml": ["wsdl"], "application/wspolicy+xml": ["wspolicy"], "application/x-7z-compressed": ["7z"], "application/x-abiword": ["abw"], "application/x-ace-compressed": ["ace"], "application/x-apple-diskimage": [], "application/x-arj": ["arj"], "application/x-authorware-bin": ["aab", "x32", "u32", "vox"], "application/x-authorware-map": ["aam"], "application/x-authorware-seg": ["aas"], "application/x-bcpio": ["bcpio"], "application/x-bdoc": [], "application/x-bittorrent": ["torrent"], "application/x-blorb": ["blb", "blorb"], "application/x-bzip": ["bz"], "application/x-bzip2": ["bz2", "boz"], "application/x-cbr": ["cbr", "cba", "cbt", "cbz", "cb7"], "application/x-cdlink": ["vcd"], "application/x-cfs-compressed": ["cfs"], "application/x-chat": ["chat"], "application/x-chess-pgn": ["pgn"], "application/x-chrome-extension": ["crx"], "application/x-cocoa": ["cco"], "application/x-conference": ["nsc"], "application/x-cpio": ["cpio"], "application/x-csh": ["csh"], "application/x-debian-package": ["udeb"], "application/x-dgc-compressed": ["dgc"], "application/x-director": ["dir", "dcr", "dxr", "cst", "cct", "cxt", "w3d", "fgd", "swa"], "application/x-doom": ["wad"], "application/x-dtbncx+xml": ["ncx"], "application/x-dtbook+xml": ["dtb"], "application/x-dtbresource+xml": ["res"], "application/x-dvi": ["dvi"], "application/x-envoy": ["evy"], "application/x-eva": ["eva"], "application/x-font-bdf": ["bdf"], "application/x-font-ghostscript": ["gsf"], "application/x-font-linux-psf": ["psf"], "application/x-font-pcf": ["pcf"], "application/x-font-snf": ["snf"], "application/x-font-type1": ["pfa", "pfb", "pfm", "afm"], "application/x-freearc": ["arc"], "application/x-futuresplash": ["spl"], "application/x-gca-compressed": ["gca"], "application/x-glulx": ["ulx"], "application/x-gnumeric": ["gnumeric"], "application/x-gramps-xml": ["gramps"], "application/x-gtar": ["gtar"], "application/x-hdf": ["hdf"], "application/x-httpd-php": ["php"], "application/x-install-instructions": ["install"], "application/x-iso9660-image": [], "application/x-java-archive-diff": ["jardiff"], "application/x-java-jnlp-file": ["jnlp"], "application/x-latex": ["latex"], "application/x-lua-bytecode": ["luac"], "application/x-lzh-compressed": ["lzh", "lha"], "application/x-makeself": ["run"], "application/x-mie": ["mie"], "application/x-mobipocket-ebook": ["prc", "mobi"], "application/x-ms-application": ["application"], "application/x-ms-shortcut": ["lnk"], "application/x-ms-wmd": ["wmd"], "application/x-ms-wmz": ["wmz"], "application/x-ms-xbap": ["xbap"], "application/x-msaccess": ["mdb"], "application/x-msbinder": ["obd"], "application/x-mscardfile": ["crd"], "application/x-msclip": ["clp"], "application/x-msdos-program": [], "application/x-msdownload": ["com", "bat"], "application/x-msmediaview": ["mvb", "m13", "m14"], "application/x-msmetafile": ["wmf", "emf", "emz"], "application/x-msmoney": ["mny"], "application/x-mspublisher": ["pub"], "application/x-msschedule": ["scd"], "application/x-msterminal": ["trm"], "application/x-mswrite": ["wri"], "application/x-netcdf": ["nc", "cdf"], "application/x-ns-proxy-autoconfig": ["pac"], "application/x-nzb": ["nzb"], "application/x-perl": ["pl", "pm"], "application/x-pilot": [], "application/x-pkcs12": ["p12", "pfx"], "application/x-pkcs7-certificates": ["p7b", "spc"], "application/x-pkcs7-certreqresp": ["p7r"], "application/x-rar-compressed": ["rar"], "application/x-redhat-package-manager": ["rpm"], "application/x-research-info-systems": ["ris"], "application/x-sea": ["sea"], "application/x-sh": ["sh"], "application/x-shar": ["shar"], "application/x-shockwave-flash": ["swf"], "application/x-silverlight-app": ["xap"], "application/x-sql": ["sql"], "application/x-stuffit": ["sit"], "application/x-stuffitx": ["sitx"], "application/x-subrip": ["srt"], "application/x-sv4cpio": ["sv4cpio"], "application/x-sv4crc": ["sv4crc"], "application/x-t3vm-image": ["t3"], "application/x-tads": ["gam"], "application/x-tar": ["tar"], "application/x-tcl": ["tcl", "tk"], "application/x-tex": ["tex"], "application/x-tex-tfm": ["tfm"], "application/x-texinfo": ["texinfo", "texi"], "application/x-tgif": ["obj"], "application/x-ustar": ["ustar"], "application/x-virtualbox-hdd": ["hdd"], "application/x-virtualbox-ova": ["ova"], "application/x-virtualbox-ovf": ["ovf"], "application/x-virtualbox-vbox": ["vbox"], "application/x-virtualbox-vbox-extpack": ["vbox-extpack"], "application/x-virtualbox-vdi": ["vdi"], "application/x-virtualbox-vhd": ["vhd"], "application/x-virtualbox-vmdk": ["vmdk"], "application/x-wais-source": ["src"], "application/x-web-app-manifest+json": ["webapp"], "application/x-x509-ca-cert": ["der", "crt", "pem"], "application/x-xfig": ["fig"], "application/x-xliff+xml": ["xlf"], "application/x-xpinstall": ["xpi"], "application/x-xz": ["xz"], "application/x-zmachine": ["z1", "z2", "z3", "z4", "z5", "z6", "z7", "z8"], "application/xaml+xml": ["xaml"], "application/xcap-diff+xml": ["xdf"], "application/xenc+xml": ["xenc"], "application/xhtml+xml": ["xhtml", "xht"], "application/xml": ["xml", "xsl", "xsd", "rng"], "application/xml-dtd": ["dtd"], "application/xop+xml": ["xop"], "application/xproc+xml": ["xpl"], "application/xslt+xml": ["xslt"], "application/xspf+xml": ["xspf"], "application/xv+xml": ["mxml", "xhvml", "xvml", "xvm"], "application/yang": ["yang"], "application/yin+xml": ["yin"], "application/zip": ["zip"], "audio/3gpp": [], "audio/adpcm": ["adp"], "audio/basic": ["au", "snd"], "audio/midi": ["mid", "midi", "kar", "rmi"], "audio/mp3": [], "audio/mp4": ["m4a", "mp4a"], "audio/mpeg": ["mpga", "mp2", "mp2a", "mp3", "m2a", "m3a"], "audio/ogg": ["oga", "ogg", "spx"], "audio/s3m": ["s3m"], "audio/silk": ["sil"], "audio/vnd.dece.audio": ["uva", "uvva"], "audio/vnd.digital-winds": ["eol"], "audio/vnd.dra": ["dra"], "audio/vnd.dts": ["dts"], "audio/vnd.dts.hd": ["dtshd"], "audio/vnd.lucent.voice": ["lvp"], "audio/vnd.ms-playready.media.pya": ["pya"], "audio/vnd.nuera.ecelp4800": ["ecelp4800"], "audio/vnd.nuera.ecelp7470": ["ecelp7470"], "audio/vnd.nuera.ecelp9600": ["ecelp9600"], "audio/vnd.rip": ["rip"], "audio/wav": ["wav"], "audio/wave": [], "audio/webm": ["weba"], "audio/x-aac": ["aac"], "audio/x-aiff": ["aif", "aiff", "aifc"], "audio/x-caf": ["caf"], "audio/x-flac": ["flac"], "audio/x-m4a": [], "audio/x-matroska": ["mka"], "audio/x-mpegurl": ["m3u"], "audio/x-ms-wax": ["wax"], "audio/x-ms-wma": ["wma"], "audio/x-pn-realaudio": ["ram", "ra"], "audio/x-pn-realaudio-plugin": ["rmp"], "audio/x-realaudio": [], "audio/x-wav": [], "audio/xm": ["xm"], "chemical/x-cdx": ["cdx"], "chemical/x-cif": ["cif"], "chemical/x-cmdf": ["cmdf"], "chemical/x-cml": ["cml"], "chemical/x-csml": ["csml"], "chemical/x-xyz": ["xyz"], "font/collection": ["ttc"], "font/otf": ["otf"], "font/ttf": ["ttf"], "font/woff": ["woff"], "font/woff2": ["woff2"], "image/apng": ["apng"], "image/bmp": ["bmp"], "image/cgm": ["cgm"], "image/g3fax": ["g3"], "image/gif": ["gif"], "image/ief": ["ief"], "image/jp2": ["jp2", "jpg2"], "image/jpeg": ["jpeg", "jpg", "jpe"], "image/jpm": ["jpm"], "image/jpx": ["jpx", "jpf"], "image/ktx": ["ktx"], "image/png": ["png"], "image/prs.btif": ["btif"], "image/sgi": ["sgi"], "image/svg+xml": ["svg", "svgz"], "image/tiff": ["tiff", "tif"], "image/vnd.adobe.photoshop": ["psd"], "image/vnd.dece.graphic": ["uvi", "uvvi", "uvg", "uvvg"], "image/vnd.djvu": ["djvu", "djv"], "image/vnd.dvb.subtitle": [], "image/vnd.dwg": ["dwg"], "image/vnd.dxf": ["dxf"], "image/vnd.fastbidsheet": ["fbs"], "image/vnd.fpx": ["fpx"], "image/vnd.fst": ["fst"], "image/vnd.fujixerox.edmics-mmr": ["mmr"], "image/vnd.fujixerox.edmics-rlc": ["rlc"], "image/vnd.ms-modi": ["mdi"], "image/vnd.ms-photo": ["wdp"], "image/vnd.net-fpx": ["npx"], "image/vnd.wap.wbmp": ["wbmp"], "image/vnd.xiff": ["xif"], "image/webp": ["webp"], "image/x-3ds": ["3ds"], "image/x-cmu-raster": ["ras"], "image/x-cmx": ["cmx"], "image/x-freehand": ["fh", "fhc", "fh4", "fh5", "fh7"], "image/x-icon": ["ico"], "image/x-jng": ["jng"], "image/x-mrsid-image": ["sid"], "image/x-ms-bmp": [], "image/x-pcx": ["pcx"], "image/x-pict": ["pic", "pct"], "image/x-portable-anymap": ["pnm"], "image/x-portable-bitmap": ["pbm"], "image/x-portable-graymap": ["pgm"], "image/x-portable-pixmap": ["ppm"], "image/x-rgb": ["rgb"], "image/x-tga": ["tga"], "image/x-xbitmap": ["xbm"], "image/x-xpixmap": ["xpm"], "image/x-xwindowdump": ["xwd"], "message/rfc822": ["eml", "mime"], "model/gltf+json": ["gltf"], "model/gltf-binary": ["glb"], "model/iges": ["igs", "iges"], "model/mesh": ["msh", "mesh", "silo"], "model/vnd.collada+xml": ["dae"], "model/vnd.dwf": ["dwf"], "model/vnd.gdl": ["gdl"], "model/vnd.gtw": ["gtw"], "model/vnd.mts": ["mts"], "model/vnd.vtu": ["vtu"], "model/vrml": ["wrl", "vrml"], "model/x3d+binary": ["x3db", "x3dbz"], "model/x3d+vrml": ["x3dv", "x3dvz"], "model/x3d+xml": ["x3d", "x3dz"], "text/cache-manifest": ["appcache", "manifest"], "text/calendar": ["ics", "ifb"], "text/coffeescript": ["coffee", "litcoffee"], "text/css": ["css"], "text/csv": ["csv"], "text/hjson": ["hjson"], "text/html": ["html", "htm", "shtml"], "text/jade": ["jade"], "text/jsx": ["jsx"], "text/less": ["less"], "text/markdown": ["markdown", "md"], "text/mathml": ["mml"], "text/n3": ["n3"], "text/plain": ["txt", "text", "conf", "def", "list", "log", "in", "ini"], "text/prs.lines.tag": ["dsc"], "text/richtext": ["rtx"], "text/rtf": [], "text/sgml": ["sgml", "sgm"], "text/slim": ["slim", "slm"], "text/stylus": ["stylus", "styl"], "text/tab-separated-values": ["tsv"], "text/troff": ["t", "tr", "roff", "man", "me", "ms"], "text/turtle": ["ttl"], "text/uri-list": ["uri", "uris", "urls"], "text/vcard": ["vcard"], "text/vnd.curl": ["curl"], "text/vnd.curl.dcurl": ["dcurl"], "text/vnd.curl.mcurl": ["mcurl"], "text/vnd.curl.scurl": ["scurl"], "text/vnd.dvb.subtitle": ["sub"], "text/vnd.fly": ["fly"], "text/vnd.fmi.flexstor": ["flx"], "text/vnd.graphviz": ["gv"], "text/vnd.in3d.3dml": ["3dml"], "text/vnd.in3d.spot": ["spot"], "text/vnd.sun.j2me.app-descriptor": ["jad"], "text/vnd.wap.wml": ["wml"], "text/vnd.wap.wmlscript": ["wmls"], "text/vtt": ["vtt"], "text/x-asm": ["s", "asm"], "text/x-c": ["c", "cc", "cxx", "cpp", "h", "hh", "dic"], "text/x-component": ["htc"], "text/x-fortran": ["f", "for", "f77", "f90"], "text/x-handlebars-template": ["hbs"], "text/x-java-source": ["java"], "text/x-lua": ["lua"], "text/x-markdown": ["mkd"], "text/x-nfo": ["nfo"], "text/x-opml": ["opml"], "text/x-org": [], "text/x-pascal": ["p", "pas"], "text/x-processing": ["pde"], "text/x-sass": ["sass"], "text/x-scss": ["scss"], "text/x-setext": ["etx"], "text/x-sfv": ["sfv"], "text/x-suse-ymp": ["ymp"], "text/x-uuencode": ["uu"], "text/x-vcalendar": ["vcs"], "text/x-vcard": ["vcf"], "text/xml": [], "text/yaml": ["yaml", "yml"], "video/3gpp": ["3gp", "3gpp"], "video/3gpp2": ["3g2"], "video/h261": ["h261"], "video/h263": ["h263"], "video/h264": ["h264"], "video/jpeg": ["jpgv"], "video/jpm": ["jpgm"], "video/mj2": ["mj2", "mjp2"], "video/mp2t": ["ts"], "video/mp4": ["mp4", "mp4v", "mpg4"], "video/mpeg": ["mpeg", "mpg", "mpe", "m1v", "m2v"], "video/ogg": ["ogv"], "video/quicktime": ["qt", "mov"], "video/vnd.dece.hd": ["uvh", "uvvh"], "video/vnd.dece.mobile": ["uvm", "uvvm"], "video/vnd.dece.pd": ["uvp", "uvvp"], "video/vnd.dece.sd": ["uvs", "uvvs"], "video/vnd.dece.video": ["uvv", "uvvv"], "video/vnd.dvb.file": ["dvb"], "video/vnd.fvt": ["fvt"], "video/vnd.mpegurl": ["mxu", "m4u"], "video/vnd.ms-playready.media.pyv": ["pyv"], "video/vnd.uvvu.mp4": ["uvu", "uvvu"], "video/vnd.vivo": ["viv"], "video/webm": ["webm"], "video/x-f4v": ["f4v"], "video/x-fli": ["fli"], "video/x-flv": ["flv"], "video/x-m4v": ["m4v"], "video/x-matroska": ["mkv", "mk3d", "mks"], "video/x-mng": ["mng"], "video/x-ms-asf": ["asf", "asx"], "video/x-ms-vob": ["vob"], "video/x-ms-wm": ["wm"], "video/x-ms-wmv": ["wmv"], "video/x-ms-wmx": ["wmx"], "video/x-ms-wvx": ["wvx"], "video/x-msvideo": ["avi"], "video/x-sgi-movie": ["movie"], "video/x-smv": ["smv"], "x-conference/x-cooltalk": ["ice"] };
},
/* graceful-fs/graceful-fs.js */ function _(require, module, exports) {
    var fs = require('fs');
    var polyfills = require(830) /* ./polyfills.js */;
    var legacy = require(831) /* ./legacy-streams.js */;
    var clone = require(832) /* ./clone.js */;
    var util = require('util');
    /* istanbul ignore next - node 0.x polyfill */
    var gracefulQueue;
    var previousSymbol;
    /* istanbul ignore else - node 0.x polyfill */
    if (typeof Symbol === 'function' && typeof Symbol.for === 'function') {
        gracefulQueue = Symbol.for('graceful-fs.queue');
        // This is used in testing by future versions
        previousSymbol = Symbol.for('graceful-fs.previous');
    }
    else {
        gracefulQueue = '___graceful-fs.queue';
        previousSymbol = '___graceful-fs.previous';
    }
    function noop() { }
    var debug = noop;
    if (util.debuglog)
        debug = util.debuglog('gfs4');
    else if (/\bgfs4\b/i.test(process.env.NODE_DEBUG || ''))
        debug = function () {
            var m = util.format.apply(util, arguments);
            m = 'GFS4: ' + m.split(/\n/).join('\nGFS4: ');
            console.error(m);
        };
    // Once time initialization
    if (!global[gracefulQueue]) {
        // This queue can be shared by multiple loaded instances
        var queue = [];
        Object.defineProperty(global, gracefulQueue, {
            get: function () {
                return queue;
            }
        });
        // Patch fs.close/closeSync to shared queue version, because we need
        // to retry() whenever a close happens *anywhere* in the program.
        // This is essential when multiple graceful-fs instances are
        // in play at the same time.
        fs.close = (function (fs$close) {
            function close(fd, cb) {
                return fs$close.call(fs, fd, function (err) {
                    // This function uses the graceful-fs shared queue
                    if (!err) {
                        retry();
                    }
                    if (typeof cb === 'function')
                        cb.apply(this, arguments);
                });
            }
            Object.defineProperty(close, previousSymbol, {
                value: fs$close
            });
            return close;
        })(fs.close);
        fs.closeSync = (function (fs$closeSync) {
            function closeSync(fd) {
                // This function uses the graceful-fs shared queue
                fs$closeSync.apply(fs, arguments);
                retry();
            }
            Object.defineProperty(closeSync, previousSymbol, {
                value: fs$closeSync
            });
            return closeSync;
        })(fs.closeSync);
        if (/\bgfs4\b/i.test(process.env.NODE_DEBUG || '')) {
            process.on('exit', function () {
                debug(global[gracefulQueue]);
                require('assert').equal(global[gracefulQueue].length, 0);
            });
        }
    }
    module.exports = patch(clone(fs));
    if (process.env.TEST_GRACEFUL_FS_GLOBAL_PATCH && !fs.__patched) {
        module.exports = patch(fs);
        fs.__patched = true;
    }
    function patch(fs) {
        // Everything that references the open() function needs to be in here
        polyfills(fs);
        fs.gracefulify = patch;
        fs.createReadStream = createReadStream;
        fs.createWriteStream = createWriteStream;
        var fs$readFile = fs.readFile;
        fs.readFile = readFile;
        function readFile(path, options, cb) {
            if (typeof options === 'function')
                cb = options, options = null;
            return go$readFile(path, options, cb);
            function go$readFile(path, options, cb) {
                return fs$readFile(path, options, function (err) {
                    if (err && (err.code === 'EMFILE' || err.code === 'ENFILE'))
                        enqueue([go$readFile, [path, options, cb]]);
                    else {
                        if (typeof cb === 'function')
                            cb.apply(this, arguments);
                        retry();
                    }
                });
            }
        }
        var fs$writeFile = fs.writeFile;
        fs.writeFile = writeFile;
        function writeFile(path, data, options, cb) {
            if (typeof options === 'function')
                cb = options, options = null;
            return go$writeFile(path, data, options, cb);
            function go$writeFile(path, data, options, cb) {
                return fs$writeFile(path, data, options, function (err) {
                    if (err && (err.code === 'EMFILE' || err.code === 'ENFILE'))
                        enqueue([go$writeFile, [path, data, options, cb]]);
                    else {
                        if (typeof cb === 'function')
                            cb.apply(this, arguments);
                        retry();
                    }
                });
            }
        }
        var fs$appendFile = fs.appendFile;
        if (fs$appendFile)
            fs.appendFile = appendFile;
        function appendFile(path, data, options, cb) {
            if (typeof options === 'function')
                cb = options, options = null;
            return go$appendFile(path, data, options, cb);
            function go$appendFile(path, data, options, cb) {
                return fs$appendFile(path, data, options, function (err) {
                    if (err && (err.code === 'EMFILE' || err.code === 'ENFILE'))
                        enqueue([go$appendFile, [path, data, options, cb]]);
                    else {
                        if (typeof cb === 'function')
                            cb.apply(this, arguments);
                        retry();
                    }
                });
            }
        }
        var fs$readdir = fs.readdir;
        fs.readdir = readdir;
        function readdir(path, options, cb) {
            var args = [path];
            if (typeof options !== 'function') {
                args.push(options);
            }
            else {
                cb = options;
            }
            args.push(go$readdir$cb);
            return go$readdir(args);
            function go$readdir$cb(err, files) {
                if (files && files.sort)
                    files.sort();
                if (err && (err.code === 'EMFILE' || err.code === 'ENFILE'))
                    enqueue([go$readdir, [args]]);
                else {
                    if (typeof cb === 'function')
                        cb.apply(this, arguments);
                    retry();
                }
            }
        }
        function go$readdir(args) {
            return fs$readdir.apply(fs, args);
        }
        if (process.version.substr(0, 4) === 'v0.8') {
            var legStreams = legacy(fs);
            ReadStream = legStreams.ReadStream;
            WriteStream = legStreams.WriteStream;
        }
        var fs$ReadStream = fs.ReadStream;
        if (fs$ReadStream) {
            ReadStream.prototype = Object.create(fs$ReadStream.prototype);
            ReadStream.prototype.open = ReadStream$open;
        }
        var fs$WriteStream = fs.WriteStream;
        if (fs$WriteStream) {
            WriteStream.prototype = Object.create(fs$WriteStream.prototype);
            WriteStream.prototype.open = WriteStream$open;
        }
        Object.defineProperty(fs, 'ReadStream', {
            get: function () {
                return ReadStream;
            },
            set: function (val) {
                ReadStream = val;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(fs, 'WriteStream', {
            get: function () {
                return WriteStream;
            },
            set: function (val) {
                WriteStream = val;
            },
            enumerable: true,
            configurable: true
        });
        // legacy names
        var FileReadStream = ReadStream;
        Object.defineProperty(fs, 'FileReadStream', {
            get: function () {
                return FileReadStream;
            },
            set: function (val) {
                FileReadStream = val;
            },
            enumerable: true,
            configurable: true
        });
        var FileWriteStream = WriteStream;
        Object.defineProperty(fs, 'FileWriteStream', {
            get: function () {
                return FileWriteStream;
            },
            set: function (val) {
                FileWriteStream = val;
            },
            enumerable: true,
            configurable: true
        });
        function ReadStream(path, options) {
            if (this instanceof ReadStream)
                return fs$ReadStream.apply(this, arguments), this;
            else
                return ReadStream.apply(Object.create(ReadStream.prototype), arguments);
        }
        function ReadStream$open() {
            var that = this;
            open(that.path, that.flags, that.mode, function (err, fd) {
                if (err) {
                    if (that.autoClose)
                        that.destroy();
                    that.emit('error', err);
                }
                else {
                    that.fd = fd;
                    that.emit('open', fd);
                    that.read();
                }
            });
        }
        function WriteStream(path, options) {
            if (this instanceof WriteStream)
                return fs$WriteStream.apply(this, arguments), this;
            else
                return WriteStream.apply(Object.create(WriteStream.prototype), arguments);
        }
        function WriteStream$open() {
            var that = this;
            open(that.path, that.flags, that.mode, function (err, fd) {
                if (err) {
                    that.destroy();
                    that.emit('error', err);
                }
                else {
                    that.fd = fd;
                    that.emit('open', fd);
                }
            });
        }
        function createReadStream(path, options) {
            return new fs.ReadStream(path, options);
        }
        function createWriteStream(path, options) {
            return new fs.WriteStream(path, options);
        }
        var fs$open = fs.open;
        fs.open = open;
        function open(path, flags, mode, cb) {
            if (typeof mode === 'function')
                cb = mode, mode = null;
            return go$open(path, flags, mode, cb);
            function go$open(path, flags, mode, cb) {
                return fs$open(path, flags, mode, function (err, fd) {
                    if (err && (err.code === 'EMFILE' || err.code === 'ENFILE'))
                        enqueue([go$open, [path, flags, mode, cb]]);
                    else {
                        if (typeof cb === 'function')
                            cb.apply(this, arguments);
                        retry();
                    }
                });
            }
        }
        return fs;
    }
    function enqueue(elem) {
        debug('ENQUEUE', elem[0].name, elem[1]);
        global[gracefulQueue].push(elem);
    }
    function retry() {
        var elem = global[gracefulQueue].shift();
        if (elem) {
            debug('RETRY', elem[0].name, elem[1]);
            elem[0].apply(null, elem[1]);
        }
    }
},
/* graceful-fs/polyfills.js */ function _(require, module, exports) {
    var constants = require('constants');
    var origCwd = process.cwd;
    var cwd = null;
    var platform = process.env.GRACEFUL_FS_PLATFORM || process.platform;
    process.cwd = function () {
        if (!cwd)
            cwd = origCwd.call(process);
        return cwd;
    };
    try {
        process.cwd();
    }
    catch (er) { }
    var chdir = process.chdir;
    process.chdir = function (d) {
        cwd = null;
        chdir.call(process, d);
    };
    module.exports = patch;
    function patch(fs) {
        // (re-)implement some things that are known busted or missing.
        // lchmod, broken prior to 0.6.2
        // back-port the fix here.
        if (constants.hasOwnProperty('O_SYMLINK') &&
            process.version.match(/^v0\.6\.[0-2]|^v0\.5\./)) {
            patchLchmod(fs);
        }
        // lutimes implementation, or no-op
        if (!fs.lutimes) {
            patchLutimes(fs);
        }
        // https://github.com/isaacs/node-graceful-fs/issues/4
        // Chown should not fail on einval or eperm if non-root.
        // It should not fail on enosys ever, as this just indicates
        // that a fs doesn't support the intended operation.
        fs.chown = chownFix(fs.chown);
        fs.fchown = chownFix(fs.fchown);
        fs.lchown = chownFix(fs.lchown);
        fs.chmod = chmodFix(fs.chmod);
        fs.fchmod = chmodFix(fs.fchmod);
        fs.lchmod = chmodFix(fs.lchmod);
        fs.chownSync = chownFixSync(fs.chownSync);
        fs.fchownSync = chownFixSync(fs.fchownSync);
        fs.lchownSync = chownFixSync(fs.lchownSync);
        fs.chmodSync = chmodFixSync(fs.chmodSync);
        fs.fchmodSync = chmodFixSync(fs.fchmodSync);
        fs.lchmodSync = chmodFixSync(fs.lchmodSync);
        fs.stat = statFix(fs.stat);
        fs.fstat = statFix(fs.fstat);
        fs.lstat = statFix(fs.lstat);
        fs.statSync = statFixSync(fs.statSync);
        fs.fstatSync = statFixSync(fs.fstatSync);
        fs.lstatSync = statFixSync(fs.lstatSync);
        // if lchmod/lchown do not exist, then make them no-ops
        if (!fs.lchmod) {
            fs.lchmod = function (path, mode, cb) {
                if (cb)
                    process.nextTick(cb);
            };
            fs.lchmodSync = function () { };
        }
        if (!fs.lchown) {
            fs.lchown = function (path, uid, gid, cb) {
                if (cb)
                    process.nextTick(cb);
            };
            fs.lchownSync = function () { };
        }
        // on Windows, A/V software can lock the directory, causing this
        // to fail with an EACCES or EPERM if the directory contains newly
        // created files.  Try again on failure, for up to 60 seconds.
        // Set the timeout this long because some Windows Anti-Virus, such as Parity
        // bit9, may lock files for up to a minute, causing npm package install
        // failures. Also, take care to yield the scheduler. Windows scheduling gives
        // CPU to a busy looping process, which can cause the program causing the lock
        // contention to be starved of CPU by node, so the contention doesn't resolve.
        if (platform === "win32") {
            fs.rename = (function (fs$rename) {
                return function (from, to, cb) {
                    var start = Date.now();
                    var backoff = 0;
                    fs$rename(from, to, function CB(er) {
                        if (er
                            && (er.code === "EACCES" || er.code === "EPERM")
                            && Date.now() - start < 60000) {
                            setTimeout(function () {
                                fs.stat(to, function (stater, st) {
                                    if (stater && stater.code === "ENOENT")
                                        fs$rename(from, to, CB);
                                    else
                                        cb(er);
                                });
                            }, backoff);
                            if (backoff < 100)
                                backoff += 10;
                            return;
                        }
                        if (cb)
                            cb(er);
                    });
                };
            })(fs.rename);
        }
        // if read() returns EAGAIN, then just try it again.
        fs.read = (function (fs$read) {
            function read(fd, buffer, offset, length, position, callback_) {
                var callback;
                if (callback_ && typeof callback_ === 'function') {
                    var eagCounter = 0;
                    callback = function (er, _, __) {
                        if (er && er.code === 'EAGAIN' && eagCounter < 10) {
                            eagCounter++;
                            return fs$read.call(fs, fd, buffer, offset, length, position, callback);
                        }
                        callback_.apply(this, arguments);
                    };
                }
                return fs$read.call(fs, fd, buffer, offset, length, position, callback);
            }
            // This ensures `util.promisify` works as it does for native `fs.read`.
            read.__proto__ = fs$read;
            return read;
        })(fs.read);
        fs.readSync = (function (fs$readSync) {
            return function (fd, buffer, offset, length, position) {
                var eagCounter = 0;
                while (true) {
                    try {
                        return fs$readSync.call(fs, fd, buffer, offset, length, position);
                    }
                    catch (er) {
                        if (er.code === 'EAGAIN' && eagCounter < 10) {
                            eagCounter++;
                            continue;
                        }
                        throw er;
                    }
                }
            };
        })(fs.readSync);
        function patchLchmod(fs) {
            fs.lchmod = function (path, mode, callback) {
                fs.open(path, constants.O_WRONLY | constants.O_SYMLINK, mode, function (err, fd) {
                    if (err) {
                        if (callback)
                            callback(err);
                        return;
                    }
                    // prefer to return the chmod error, if one occurs,
                    // but still try to close, and report closing errors if they occur.
                    fs.fchmod(fd, mode, function (err) {
                        fs.close(fd, function (err2) {
                            if (callback)
                                callback(err || err2);
                        });
                    });
                });
            };
            fs.lchmodSync = function (path, mode) {
                var fd = fs.openSync(path, constants.O_WRONLY | constants.O_SYMLINK, mode);
                // prefer to return the chmod error, if one occurs,
                // but still try to close, and report closing errors if they occur.
                var threw = true;
                var ret;
                try {
                    ret = fs.fchmodSync(fd, mode);
                    threw = false;
                }
                finally {
                    if (threw) {
                        try {
                            fs.closeSync(fd);
                        }
                        catch (er) { }
                    }
                    else {
                        fs.closeSync(fd);
                    }
                }
                return ret;
            };
        }
        function patchLutimes(fs) {
            if (constants.hasOwnProperty("O_SYMLINK")) {
                fs.lutimes = function (path, at, mt, cb) {
                    fs.open(path, constants.O_SYMLINK, function (er, fd) {
                        if (er) {
                            if (cb)
                                cb(er);
                            return;
                        }
                        fs.futimes(fd, at, mt, function (er) {
                            fs.close(fd, function (er2) {
                                if (cb)
                                    cb(er || er2);
                            });
                        });
                    });
                };
                fs.lutimesSync = function (path, at, mt) {
                    var fd = fs.openSync(path, constants.O_SYMLINK);
                    var ret;
                    var threw = true;
                    try {
                        ret = fs.futimesSync(fd, at, mt);
                        threw = false;
                    }
                    finally {
                        if (threw) {
                            try {
                                fs.closeSync(fd);
                            }
                            catch (er) { }
                        }
                        else {
                            fs.closeSync(fd);
                        }
                    }
                    return ret;
                };
            }
            else {
                fs.lutimes = function (_a, _b, _c, cb) { if (cb)
                    process.nextTick(cb); };
                fs.lutimesSync = function () { };
            }
        }
        function chmodFix(orig) {
            if (!orig)
                return orig;
            return function (target, mode, cb) {
                return orig.call(fs, target, mode, function (er) {
                    if (chownErOk(er))
                        er = null;
                    if (cb)
                        cb.apply(this, arguments);
                });
            };
        }
        function chmodFixSync(orig) {
            if (!orig)
                return orig;
            return function (target, mode) {
                try {
                    return orig.call(fs, target, mode);
                }
                catch (er) {
                    if (!chownErOk(er))
                        throw er;
                }
            };
        }
        function chownFix(orig) {
            if (!orig)
                return orig;
            return function (target, uid, gid, cb) {
                return orig.call(fs, target, uid, gid, function (er) {
                    if (chownErOk(er))
                        er = null;
                    if (cb)
                        cb.apply(this, arguments);
                });
            };
        }
        function chownFixSync(orig) {
            if (!orig)
                return orig;
            return function (target, uid, gid) {
                try {
                    return orig.call(fs, target, uid, gid);
                }
                catch (er) {
                    if (!chownErOk(er))
                        throw er;
                }
            };
        }
        function statFix(orig) {
            if (!orig)
                return orig;
            // Older versions of Node erroneously returned signed integers for
            // uid + gid.
            return function (target, options, cb) {
                if (typeof options === 'function') {
                    cb = options;
                    options = null;
                }
                function callback(er, stats) {
                    if (stats) {
                        if (stats.uid < 0)
                            stats.uid += 0x100000000;
                        if (stats.gid < 0)
                            stats.gid += 0x100000000;
                    }
                    if (cb)
                        cb.apply(this, arguments);
                }
                return options ? orig.call(fs, target, options, callback)
                    : orig.call(fs, target, callback);
            };
        }
        function statFixSync(orig) {
            if (!orig)
                return orig;
            // Older versions of Node erroneously returned signed integers for
            // uid + gid.
            return function (target, options) {
                var stats = options ? orig.call(fs, target, options)
                    : orig.call(fs, target);
                if (stats.uid < 0)
                    stats.uid += 0x100000000;
                if (stats.gid < 0)
                    stats.gid += 0x100000000;
                return stats;
            };
        }
        // ENOSYS means that the fs doesn't support the op. Just ignore
        // that, because it doesn't matter.
        //
        // if there's no getuid, or if getuid() is something other
        // than 0, and the error is EINVAL or EPERM, then just ignore
        // it.
        //
        // This specific case is a silent failure in cp, install, tar,
        // and most other unix tools that manage permissions.
        //
        // When running as root, or if other types of errors are
        // encountered, then it's strict.
        function chownErOk(er) {
            if (!er)
                return true;
            if (er.code === "ENOSYS")
                return true;
            var nonroot = !process.getuid || process.getuid() !== 0;
            if (nonroot) {
                if (er.code === "EINVAL" || er.code === "EPERM")
                    return true;
            }
            return false;
        }
    }
},
/* graceful-fs/legacy-streams.js */ function _(require, module, exports) {
    var Stream = require('stream').Stream;
    module.exports = legacy;
    function legacy(fs) {
        return {
            ReadStream: ReadStream,
            WriteStream: WriteStream
        };
        function ReadStream(path, options) {
            if (!(this instanceof ReadStream))
                return new ReadStream(path, options);
            Stream.call(this);
            var self = this;
            this.path = path;
            this.fd = null;
            this.readable = true;
            this.paused = false;
            this.flags = 'r';
            this.mode = 438; /*=0666*/
            this.bufferSize = 64 * 1024;
            options = options || {};
            // Mixin options into this
            var keys = Object.keys(options);
            for (var index = 0, length = keys.length; index < length; index++) {
                var key = keys[index];
                this[key] = options[key];
            }
            if (this.encoding)
                this.setEncoding(this.encoding);
            if (this.start !== undefined) {
                if ('number' !== typeof this.start) {
                    throw TypeError('start must be a Number');
                }
                if (this.end === undefined) {
                    this.end = Infinity;
                }
                else if ('number' !== typeof this.end) {
                    throw TypeError('end must be a Number');
                }
                if (this.start > this.end) {
                    throw new Error('start must be <= end');
                }
                this.pos = this.start;
            }
            if (this.fd !== null) {
                process.nextTick(function () {
                    self._read();
                });
                return;
            }
            fs.open(this.path, this.flags, this.mode, function (err, fd) {
                if (err) {
                    self.emit('error', err);
                    self.readable = false;
                    return;
                }
                self.fd = fd;
                self.emit('open', fd);
                self._read();
            });
        }
        function WriteStream(path, options) {
            if (!(this instanceof WriteStream))
                return new WriteStream(path, options);
            Stream.call(this);
            this.path = path;
            this.fd = null;
            this.writable = true;
            this.flags = 'w';
            this.encoding = 'binary';
            this.mode = 438; /*=0666*/
            this.bytesWritten = 0;
            options = options || {};
            // Mixin options into this
            var keys = Object.keys(options);
            for (var index = 0, length = keys.length; index < length; index++) {
                var key = keys[index];
                this[key] = options[key];
            }
            if (this.start !== undefined) {
                if ('number' !== typeof this.start) {
                    throw TypeError('start must be a Number');
                }
                if (this.start < 0) {
                    throw new Error('start must be >= zero');
                }
                this.pos = this.start;
            }
            this.busy = false;
            this._queue = [];
            if (this.fd === null) {
                this._open = fs.open;
                this._queue.push([this._open, this.path, this.flags, this.mode, undefined]);
                this.flush();
            }
        }
    }
},
/* graceful-fs/clone.js */ function _(require, module, exports) {
    module.exports = clone;
    function clone(obj) {
        if (obj === null || typeof obj !== 'object')
            return obj;
        if (obj instanceof Object)
            var copy = { __proto__: obj.__proto__ };
        else
            var copy = Object.create(null);
        Object.getOwnPropertyNames(obj).forEach(function (key) {
            Object.defineProperty(copy, key, Object.getOwnPropertyDescriptor(obj, key));
        });
        return copy;
    }
},
/* request/index.js */ function _(require, module, exports) {
    var extend = require(834) /* extend */;
    var cookies = require(835) /* ./lib/cookies */;
    var helpers = require(845) /* ./lib/helpers */;
    var paramsHaveRequestBody = helpers.paramsHaveRequestBody;
    // organize params for patch, post, put, head, del
    function initParams(uri, options, callback) {
        if (typeof options === 'function') {
            callback = options;
        }
        var params = {};
        if (options !== null && typeof options === 'object') {
            extend(params, options, { uri: uri });
        }
        else if (typeof uri === 'string') {
            extend(params, { uri: uri });
        }
        else {
            extend(params, uri);
        }
        params.callback = callback || params.callback;
        return params;
    }
    function request(uri, options, callback) {
        if (typeof uri === 'undefined') {
            throw new Error('undefined is not a valid uri or options object.');
        }
        var params = initParams(uri, options, callback);
        if (params.method === 'HEAD' && paramsHaveRequestBody(params)) {
            throw new Error('HTTP HEAD requests MUST NOT include a request body.');
        }
        return new request.Request(params);
    }
    function verbFunc(verb) {
        var method = verb.toUpperCase();
        return function (uri, options, callback) {
            var params = initParams(uri, options, callback);
            params.method = method;
            return request(params, params.callback);
        };
    }
    // define like this to please codeintel/intellisense IDEs
    request.get = verbFunc('get');
    request.head = verbFunc('head');
    request.options = verbFunc('options');
    request.post = verbFunc('post');
    request.put = verbFunc('put');
    request.patch = verbFunc('patch');
    request.del = verbFunc('delete');
    request['delete'] = verbFunc('delete');
    request.jar = function (store) {
        return cookies.jar(store);
    };
    request.cookie = function (str) {
        return cookies.parse(str);
    };
    function wrapRequestMethod(method, options, requester, verb) {
        return function (uri, opts, callback) {
            var params = initParams(uri, opts, callback);
            var target = {};
            extend(true, target, options, params);
            target.pool = params.pool || options.pool;
            if (verb) {
                target.method = verb.toUpperCase();
            }
            if (typeof requester === 'function') {
                method = requester;
            }
            return method(target, target.callback);
        };
    }
    request.defaults = function (options, requester) {
        var self = this;
        options = options || {};
        if (typeof options === 'function') {
            requester = options;
            options = {};
        }
        var defaults = wrapRequestMethod(self, options, requester);
        var verbs = ['get', 'head', 'post', 'put', 'patch', 'del', 'delete'];
        verbs.forEach(function (verb) {
            defaults[verb] = wrapRequestMethod(self[verb], options, requester, verb);
        });
        defaults.cookie = wrapRequestMethod(self.cookie, options, requester);
        defaults.jar = self.jar;
        defaults.defaults = self.defaults;
        return defaults;
    };
    request.forever = function (agentOptions, optionsArg) {
        var options = {};
        if (optionsArg) {
            extend(options, optionsArg);
        }
        if (agentOptions) {
            options.agentOptions = agentOptions;
        }
        options.forever = true;
        return request.defaults(options);
    };
    // Exports
    module.exports = request;
    request.Request = require(847) /* ./request */;
    request.initParams = initParams;
    // Backwards compatibility for request.debug
    Object.defineProperty(request, 'debug', {
        enumerable: true,
        get: function () {
            return request.Request.debug;
        },
        set: function (debug) {
            request.Request.debug = debug;
        }
    });
},
/* extend/index.js */ function _(require, module, exports) {
    var hasOwn = Object.prototype.hasOwnProperty;
    var toStr = Object.prototype.toString;
    var defineProperty = Object.defineProperty;
    var gOPD = Object.getOwnPropertyDescriptor;
    var isArray = function isArray(arr) {
        if (typeof Array.isArray === 'function') {
            return Array.isArray(arr);
        }
        return toStr.call(arr) === '[object Array]';
    };
    var isPlainObject = function isPlainObject(obj) {
        if (!obj || toStr.call(obj) !== '[object Object]') {
            return false;
        }
        var hasOwnConstructor = hasOwn.call(obj, 'constructor');
        var hasIsPrototypeOf = obj.constructor && obj.constructor.prototype && hasOwn.call(obj.constructor.prototype, 'isPrototypeOf');
        // Not own constructor property must be Object
        if (obj.constructor && !hasOwnConstructor && !hasIsPrototypeOf) {
            return false;
        }
        // Own properties are enumerated firstly, so to speed up,
        // if last one is own, then all properties are own.
        var key;
        for (key in obj) { /**/ }
        return typeof key === 'undefined' || hasOwn.call(obj, key);
    };
    // If name is '__proto__', and Object.defineProperty is available, define __proto__ as an own property on target
    var setProperty = function setProperty(target, options) {
        if (defineProperty && options.name === '__proto__') {
            defineProperty(target, options.name, {
                enumerable: true,
                configurable: true,
                value: options.newValue,
                writable: true
            });
        }
        else {
            target[options.name] = options.newValue;
        }
    };
    // Return undefined instead of __proto__ if '__proto__' is not an own property
    var getProperty = function getProperty(obj, name) {
        if (name === '__proto__') {
            if (!hasOwn.call(obj, name)) {
                return void 0;
            }
            else if (gOPD) {
                // In early versions of node, obj['__proto__'] is buggy when obj has
                // __proto__ as an own property. Object.getOwnPropertyDescriptor() works.
                return gOPD(obj, name).value;
            }
        }
        return obj[name];
    };
    module.exports = function extend() {
        var options, name, src, copy, copyIsArray, clone;
        var target = arguments[0];
        var i = 1;
        var length = arguments.length;
        var deep = false;
        // Handle a deep copy situation
        if (typeof target === 'boolean') {
            deep = target;
            target = arguments[1] || {};
            // skip the boolean and the target
            i = 2;
        }
        if (target == null || (typeof target !== 'object' && typeof target !== 'function')) {
            target = {};
        }
        for (; i < length; ++i) {
            options = arguments[i];
            // Only deal with non-null/undefined values
            if (options != null) {
                // Extend the base object
                for (name in options) {
                    src = getProperty(target, name);
                    copy = getProperty(options, name);
                    // Prevent never-ending loop
                    if (target !== copy) {
                        // Recurse if we're merging plain objects or arrays
                        if (deep && copy && (isPlainObject(copy) || (copyIsArray = isArray(copy)))) {
                            if (copyIsArray) {
                                copyIsArray = false;
                                clone = src && isArray(src) ? src : [];
                            }
                            else {
                                clone = src && isPlainObject(src) ? src : {};
                            }
                            // Never move original objects, clone them
                            setProperty(target, { name: name, newValue: extend(deep, clone, copy) });
                            // Don't bring in undefined values
                        }
                        else if (typeof copy !== 'undefined') {
                            setProperty(target, { name: name, newValue: copy });
                        }
                    }
                }
            }
        }
        // Return the modified object
        return target;
    };
},
/* request/lib/cookies.js */ function _(require, module, exports) {
    var tough = require(836) /* tough-cookie */;
    var Cookie = tough.Cookie;
    var CookieJar = tough.CookieJar;
    exports.parse = function (str) {
        if (str && str.uri) {
            str = str.uri;
        }
        if (typeof str !== 'string') {
            throw new Error('The cookie function only accepts STRING as param');
        }
        return Cookie.parse(str, { loose: true });
    };
    // Adapt the sometimes-Async api of tough.CookieJar to our requirements
    function RequestJar(store) {
        var self = this;
        self._jar = new CookieJar(store, { looseMode: true });
    }
    RequestJar.prototype.setCookie = function (cookieOrStr, uri, options) {
        var self = this;
        return self._jar.setCookieSync(cookieOrStr, uri, options || {});
    };
    RequestJar.prototype.getCookieString = function (uri) {
        var self = this;
        return self._jar.getCookieStringSync(uri);
    };
    RequestJar.prototype.getCookies = function (uri) {
        var self = this;
        return self._jar.getCookiesSync(uri);
    };
    exports.jar = function (store) {
        return new RequestJar(store);
    };
},
/* tough-cookie/lib/cookie.js */ function _(require, module, exports) {
    var net = require('net');
    var urlParse = require('url').parse;
    var util = require('util');
    var pubsuffix = require(837) /* ./pubsuffix-psl */;
    var Store = require(840) /* ./store */.Store;
    var MemoryCookieStore = require(841) /* ./memstore */.MemoryCookieStore;
    var pathMatch = require(843) /* ./pathMatch */.pathMatch;
    var VERSION = require(844) /* ./version */;
    var punycode;
    try {
        punycode = require('punycode');
    }
    catch (e) {
        console.warn("tough-cookie: can't load punycode; won't use punycode for domain normalization");
    }
    // From RFC6265 S4.1.1
    // note that it excludes \x3B ";"
    var COOKIE_OCTETS = /^[\x21\x23-\x2B\x2D-\x3A\x3C-\x5B\x5D-\x7E]+$/;
    var CONTROL_CHARS = /[\x00-\x1F]/;
    // From Chromium // '\r', '\n' and '\0' should be treated as a terminator in
    // the "relaxed" mode, see:
    // https://github.com/ChromiumWebApps/chromium/blob/b3d3b4da8bb94c1b2e061600df106d590fda3620/net/cookies/parsed_cookie.cc#L60
    var TERMINATORS = ['\n', '\r', '\0'];
    // RFC6265 S4.1.1 defines path value as 'any CHAR except CTLs or ";"'
    // Note ';' is \x3B
    var PATH_VALUE = /[\x20-\x3A\x3C-\x7E]+/;
    // date-time parsing constants (RFC6265 S5.1.1)
    var DATE_DELIM = /[\x09\x20-\x2F\x3B-\x40\x5B-\x60\x7B-\x7E]/;
    var MONTH_TO_NUM = {
        jan: 0, feb: 1, mar: 2, apr: 3, may: 4, jun: 5,
        jul: 6, aug: 7, sep: 8, oct: 9, nov: 10, dec: 11
    };
    var NUM_TO_MONTH = [
        'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'
    ];
    var NUM_TO_DAY = [
        'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'
    ];
    var MAX_TIME = 2147483647000; // 31-bit max
    var MIN_TIME = 0; // 31-bit min
    /*
     * Parses a Natural number (i.e., non-negative integer) with either the
     *    <min>*<max>DIGIT ( non-digit *OCTET )
     * or
     *    <min>*<max>DIGIT
     * grammar (RFC6265 S5.1.1).
     *
     * The "trailingOK" boolean controls if the grammar accepts a
     * "( non-digit *OCTET )" trailer.
     */
    function parseDigits(token, minDigits, maxDigits, trailingOK) {
        var count = 0;
        while (count < token.length) {
            var c = token.charCodeAt(count);
            // "non-digit = %x00-2F / %x3A-FF"
            if (c <= 0x2F || c >= 0x3A) {
                break;
            }
            count++;
        }
        // constrain to a minimum and maximum number of digits.
        if (count < minDigits || count > maxDigits) {
            return null;
        }
        if (!trailingOK && count != token.length) {
            return null;
        }
        return parseInt(token.substr(0, count), 10);
    }
    function parseTime(token) {
        var parts = token.split(':');
        var result = [0, 0, 0];
        /* RF6256 S5.1.1:
         *      time            = hms-time ( non-digit *OCTET )
         *      hms-time        = time-field ":" time-field ":" time-field
         *      time-field      = 1*2DIGIT
         */
        if (parts.length !== 3) {
            return null;
        }
        for (var i = 0; i < 3; i++) {
            // "time-field" must be strictly "1*2DIGIT", HOWEVER, "hms-time" can be
            // followed by "( non-digit *OCTET )" so therefore the last time-field can
            // have a trailer
            var trailingOK = (i == 2);
            var num = parseDigits(parts[i], 1, 2, trailingOK);
            if (num === null) {
                return null;
            }
            result[i] = num;
        }
        return result;
    }
    function parseMonth(token) {
        token = String(token).substr(0, 3).toLowerCase();
        var num = MONTH_TO_NUM[token];
        return num >= 0 ? num : null;
    }
    /*
     * RFC6265 S5.1.1 date parser (see RFC for full grammar)
     */
    function parseDate(str) {
        if (!str) {
            return;
        }
        /* RFC6265 S5.1.1:
         * 2. Process each date-token sequentially in the order the date-tokens
         * appear in the cookie-date
         */
        var tokens = str.split(DATE_DELIM);
        if (!tokens) {
            return;
        }
        var hour = null;
        var minute = null;
        var second = null;
        var dayOfMonth = null;
        var month = null;
        var year = null;
        for (var i = 0; i < tokens.length; i++) {
            var token = tokens[i].trim();
            if (!token.length) {
                continue;
            }
            var result;
            /* 2.1. If the found-time flag is not set and the token matches the time
             * production, set the found-time flag and set the hour- value,
             * minute-value, and second-value to the numbers denoted by the digits in
             * the date-token, respectively.  Skip the remaining sub-steps and continue
             * to the next date-token.
             */
            if (second === null) {
                result = parseTime(token);
                if (result) {
                    hour = result[0];
                    minute = result[1];
                    second = result[2];
                    continue;
                }
            }
            /* 2.2. If the found-day-of-month flag is not set and the date-token matches
             * the day-of-month production, set the found-day-of- month flag and set
             * the day-of-month-value to the number denoted by the date-token.  Skip
             * the remaining sub-steps and continue to the next date-token.
             */
            if (dayOfMonth === null) {
                // "day-of-month = 1*2DIGIT ( non-digit *OCTET )"
                result = parseDigits(token, 1, 2, true);
                if (result !== null) {
                    dayOfMonth = result;
                    continue;
                }
            }
            /* 2.3. If the found-month flag is not set and the date-token matches the
             * month production, set the found-month flag and set the month-value to
             * the month denoted by the date-token.  Skip the remaining sub-steps and
             * continue to the next date-token.
             */
            if (month === null) {
                result = parseMonth(token);
                if (result !== null) {
                    month = result;
                    continue;
                }
            }
            /* 2.4. If the found-year flag is not set and the date-token matches the
             * year production, set the found-year flag and set the year-value to the
             * number denoted by the date-token.  Skip the remaining sub-steps and
             * continue to the next date-token.
             */
            if (year === null) {
                // "year = 2*4DIGIT ( non-digit *OCTET )"
                result = parseDigits(token, 2, 4, true);
                if (result !== null) {
                    year = result;
                    /* From S5.1.1:
                     * 3.  If the year-value is greater than or equal to 70 and less
                     * than or equal to 99, increment the year-value by 1900.
                     * 4.  If the year-value is greater than or equal to 0 and less
                     * than or equal to 69, increment the year-value by 2000.
                     */
                    if (year >= 70 && year <= 99) {
                        year += 1900;
                    }
                    else if (year >= 0 && year <= 69) {
                        year += 2000;
                    }
                }
            }
        }
        /* RFC 6265 S5.1.1
         * "5. Abort these steps and fail to parse the cookie-date if:
         *     *  at least one of the found-day-of-month, found-month, found-
         *        year, or found-time flags is not set,
         *     *  the day-of-month-value is less than 1 or greater than 31,
         *     *  the year-value is less than 1601,
         *     *  the hour-value is greater than 23,
         *     *  the minute-value is greater than 59, or
         *     *  the second-value is greater than 59.
         *     (Note that leap seconds cannot be represented in this syntax.)"
         *
         * So, in order as above:
         */
        if (dayOfMonth === null || month === null || year === null || second === null ||
            dayOfMonth < 1 || dayOfMonth > 31 ||
            year < 1601 ||
            hour > 23 ||
            minute > 59 ||
            second > 59) {
            return;
        }
        return new Date(Date.UTC(year, month, dayOfMonth, hour, minute, second));
    }
    function formatDate(date) {
        var d = date.getUTCDate();
        d = d >= 10 ? d : '0' + d;
        var h = date.getUTCHours();
        h = h >= 10 ? h : '0' + h;
        var m = date.getUTCMinutes();
        m = m >= 10 ? m : '0' + m;
        var s = date.getUTCSeconds();
        s = s >= 10 ? s : '0' + s;
        return NUM_TO_DAY[date.getUTCDay()] + ', ' +
            d + ' ' + NUM_TO_MONTH[date.getUTCMonth()] + ' ' + date.getUTCFullYear() + ' ' +
            h + ':' + m + ':' + s + ' GMT';
    }
    // S5.1.2 Canonicalized Host Names
    function canonicalDomain(str) {
        if (str == null) {
            return null;
        }
        str = str.trim().replace(/^\./, ''); // S4.1.2.3 & S5.2.3: ignore leading .
        // convert to IDN if any non-ASCII characters
        if (punycode && /[^\u0001-\u007f]/.test(str)) {
            str = punycode.toASCII(str);
        }
        return str.toLowerCase();
    }
    // S5.1.3 Domain Matching
    function domainMatch(str, domStr, canonicalize) {
        if (str == null || domStr == null) {
            return null;
        }
        if (canonicalize !== false) {
            str = canonicalDomain(str);
            domStr = canonicalDomain(domStr);
        }
        /*
         * "The domain string and the string are identical. (Note that both the
         * domain string and the string will have been canonicalized to lower case at
         * this point)"
         */
        if (str == domStr) {
            return true;
        }
        /* "All of the following [three] conditions hold:" (order adjusted from the RFC) */
        /* "* The string is a host name (i.e., not an IP address)." */
        if (net.isIP(str)) {
            return false;
        }
        /* "* The domain string is a suffix of the string" */
        var idx = str.indexOf(domStr);
        if (idx <= 0) {
            return false; // it's a non-match (-1) or prefix (0)
        }
        // e.g "a.b.c".indexOf("b.c") === 2
        // 5 === 3+2
        if (str.length !== domStr.length + idx) { // it's not a suffix
            return false;
        }
        /* "* The last character of the string that is not included in the domain
        * string is a %x2E (".") character." */
        if (str.substr(idx - 1, 1) !== '.') {
            return false;
        }
        return true;
    }
    // RFC6265 S5.1.4 Paths and Path-Match
    /*
     * "The user agent MUST use an algorithm equivalent to the following algorithm
     * to compute the default-path of a cookie:"
     *
     * Assumption: the path (and not query part or absolute uri) is passed in.
     */
    function defaultPath(path) {
        // "2. If the uri-path is empty or if the first character of the uri-path is not
        // a %x2F ("/") character, output %x2F ("/") and skip the remaining steps.
        if (!path || path.substr(0, 1) !== "/") {
            return "/";
        }
        // "3. If the uri-path contains no more than one %x2F ("/") character, output
        // %x2F ("/") and skip the remaining step."
        if (path === "/") {
            return path;
        }
        var rightSlash = path.lastIndexOf("/");
        if (rightSlash === 0) {
            return "/";
        }
        // "4. Output the characters of the uri-path from the first character up to,
        // but not including, the right-most %x2F ("/")."
        return path.slice(0, rightSlash);
    }
    function trimTerminator(str) {
        for (var t = 0; t < TERMINATORS.length; t++) {
            var terminatorIdx = str.indexOf(TERMINATORS[t]);
            if (terminatorIdx !== -1) {
                str = str.substr(0, terminatorIdx);
            }
        }
        return str;
    }
    function parseCookiePair(cookiePair, looseMode) {
        cookiePair = trimTerminator(cookiePair);
        var firstEq = cookiePair.indexOf('=');
        if (looseMode) {
            if (firstEq === 0) { // '=' is immediately at start
                cookiePair = cookiePair.substr(1);
                firstEq = cookiePair.indexOf('='); // might still need to split on '='
            }
        }
        else { // non-loose mode
            if (firstEq <= 0) { // no '=' or is at start
                return; // needs to have non-empty "cookie-name"
            }
        }
        var cookieName, cookieValue;
        if (firstEq <= 0) {
            cookieName = "";
            cookieValue = cookiePair.trim();
        }
        else {
            cookieName = cookiePair.substr(0, firstEq).trim();
            cookieValue = cookiePair.substr(firstEq + 1).trim();
        }
        if (CONTROL_CHARS.test(cookieName) || CONTROL_CHARS.test(cookieValue)) {
            return;
        }
        var c = new Cookie();
        c.key = cookieName;
        c.value = cookieValue;
        return c;
    }
    function parse(str, options) {
        if (!options || typeof options !== 'object') {
            options = {};
        }
        str = str.trim();
        // We use a regex to parse the "name-value-pair" part of S5.2
        var firstSemi = str.indexOf(';'); // S5.2 step 1
        var cookiePair = (firstSemi === -1) ? str : str.substr(0, firstSemi);
        var c = parseCookiePair(cookiePair, !!options.loose);
        if (!c) {
            return;
        }
        if (firstSemi === -1) {
            return c;
        }
        // S5.2.3 "unparsed-attributes consist of the remainder of the set-cookie-string
        // (including the %x3B (";") in question)." plus later on in the same section
        // "discard the first ";" and trim".
        var unparsed = str.slice(firstSemi + 1).trim();
        // "If the unparsed-attributes string is empty, skip the rest of these
        // steps."
        if (unparsed.length === 0) {
            return c;
        }
        /*
         * S5.2 says that when looping over the items "[p]rocess the attribute-name
         * and attribute-value according to the requirements in the following
         * subsections" for every item.  Plus, for many of the individual attributes
         * in S5.3 it says to use the "attribute-value of the last attribute in the
         * cookie-attribute-list".  Therefore, in this implementation, we overwrite
         * the previous value.
         */
        var cookie_avs = unparsed.split(';');
        while (cookie_avs.length) {
            var av = cookie_avs.shift().trim();
            if (av.length === 0) { // happens if ";;" appears
                continue;
            }
            var av_sep = av.indexOf('=');
            var av_key, av_value;
            if (av_sep === -1) {
                av_key = av;
                av_value = null;
            }
            else {
                av_key = av.substr(0, av_sep);
                av_value = av.substr(av_sep + 1);
            }
            av_key = av_key.trim().toLowerCase();
            if (av_value) {
                av_value = av_value.trim();
            }
            switch (av_key) {
                case 'expires': // S5.2.1
                    if (av_value) {
                        var exp = parseDate(av_value);
                        // "If the attribute-value failed to parse as a cookie date, ignore the
                        // cookie-av."
                        if (exp) {
                            // over and underflow not realistically a concern: V8's getTime() seems to
                            // store something larger than a 32-bit time_t (even with 32-bit node)
                            c.expires = exp;
                        }
                    }
                    break;
                case 'max-age': // S5.2.2
                    if (av_value) {
                        // "If the first character of the attribute-value is not a DIGIT or a "-"
                        // character ...[or]... If the remainder of attribute-value contains a
                        // non-DIGIT character, ignore the cookie-av."
                        if (/^-?[0-9]+$/.test(av_value)) {
                            var delta = parseInt(av_value, 10);
                            // "If delta-seconds is less than or equal to zero (0), let expiry-time
                            // be the earliest representable date and time."
                            c.setMaxAge(delta);
                        }
                    }
                    break;
                case 'domain': // S5.2.3
                    // "If the attribute-value is empty, the behavior is undefined.  However,
                    // the user agent SHOULD ignore the cookie-av entirely."
                    if (av_value) {
                        // S5.2.3 "Let cookie-domain be the attribute-value without the leading %x2E
                        // (".") character."
                        var domain = av_value.trim().replace(/^\./, '');
                        if (domain) {
                            // "Convert the cookie-domain to lower case."
                            c.domain = domain.toLowerCase();
                        }
                    }
                    break;
                case 'path': // S5.2.4
                    /*
                     * "If the attribute-value is empty or if the first character of the
                     * attribute-value is not %x2F ("/"):
                     *   Let cookie-path be the default-path.
                     * Otherwise:
                     *   Let cookie-path be the attribute-value."
                     *
                     * We'll represent the default-path as null since it depends on the
                     * context of the parsing.
                     */
                    c.path = av_value && av_value[0] === "/" ? av_value : null;
                    break;
                case 'secure': // S5.2.5
                    /*
                     * "If the attribute-name case-insensitively matches the string "Secure",
                     * the user agent MUST append an attribute to the cookie-attribute-list
                     * with an attribute-name of Secure and an empty attribute-value."
                     */
                    c.secure = true;
                    break;
                case 'httponly': // S5.2.6 -- effectively the same as 'secure'
                    c.httpOnly = true;
                    break;
                default:
                    c.extensions = c.extensions || [];
                    c.extensions.push(av);
                    break;
            }
        }
        return c;
    }
    // avoid the V8 deoptimization monster!
    function jsonParse(str) {
        var obj;
        try {
            obj = JSON.parse(str);
        }
        catch (e) {
            return e;
        }
        return obj;
    }
    function fromJSON(str) {
        if (!str) {
            return null;
        }
        var obj;
        if (typeof str === 'string') {
            obj = jsonParse(str);
            if (obj instanceof Error) {
                return null;
            }
        }
        else {
            // assume it's an Object
            obj = str;
        }
        var c = new Cookie();
        for (var i = 0; i < Cookie.serializableProperties.length; i++) {
            var prop = Cookie.serializableProperties[i];
            if (obj[prop] === undefined ||
                obj[prop] === Cookie.prototype[prop]) {
                continue; // leave as prototype default
            }
            if (prop === 'expires' ||
                prop === 'creation' ||
                prop === 'lastAccessed') {
                if (obj[prop] === null) {
                    c[prop] = null;
                }
                else {
                    c[prop] = obj[prop] == "Infinity" ?
                        "Infinity" : new Date(obj[prop]);
                }
            }
            else {
                c[prop] = obj[prop];
            }
        }
        return c;
    }
    /* Section 5.4 part 2:
     * "*  Cookies with longer paths are listed before cookies with
     *     shorter paths.
     *
     *  *  Among cookies that have equal-length path fields, cookies with
     *     earlier creation-times are listed before cookies with later
     *     creation-times."
     */
    function cookieCompare(a, b) {
        var cmp = 0;
        // descending for length: b CMP a
        var aPathLen = a.path ? a.path.length : 0;
        var bPathLen = b.path ? b.path.length : 0;
        cmp = bPathLen - aPathLen;
        if (cmp !== 0) {
            return cmp;
        }
        // ascending for time: a CMP b
        var aTime = a.creation ? a.creation.getTime() : MAX_TIME;
        var bTime = b.creation ? b.creation.getTime() : MAX_TIME;
        cmp = aTime - bTime;
        if (cmp !== 0) {
            return cmp;
        }
        // break ties for the same millisecond (precision of JavaScript's clock)
        cmp = a.creationIndex - b.creationIndex;
        return cmp;
    }
    // Gives the permutation of all possible pathMatch()es of a given path. The
    // array is in longest-to-shortest order.  Handy for indexing.
    function permutePath(path) {
        if (path === '/') {
            return ['/'];
        }
        if (path.lastIndexOf('/') === path.length - 1) {
            path = path.substr(0, path.length - 1);
        }
        var permutations = [path];
        while (path.length > 1) {
            var lindex = path.lastIndexOf('/');
            if (lindex === 0) {
                break;
            }
            path = path.substr(0, lindex);
            permutations.push(path);
        }
        permutations.push('/');
        return permutations;
    }
    function getCookieContext(url) {
        if (url instanceof Object) {
            return url;
        }
        // NOTE: decodeURI will throw on malformed URIs (see GH-32).
        // Therefore, we will just skip decoding for such URIs.
        try {
            url = decodeURI(url);
        }
        catch (err) {
            // Silently swallow error
        }
        return urlParse(url);
    }
    function Cookie(options) {
        options = options || {};
        Object.keys(options).forEach(function (prop) {
            if (Cookie.prototype.hasOwnProperty(prop) &&
                Cookie.prototype[prop] !== options[prop] &&
                prop.substr(0, 1) !== '_') {
                this[prop] = options[prop];
            }
        }, this);
        this.creation = this.creation || new Date();
        // used to break creation ties in cookieCompare():
        Object.defineProperty(this, 'creationIndex', {
            configurable: false,
            enumerable: false,
            writable: true,
            value: ++Cookie.cookiesCreated
        });
    }
    Cookie.cookiesCreated = 0; // incremented each time a cookie is created
    Cookie.parse = parse;
    Cookie.fromJSON = fromJSON;
    Cookie.prototype.key = "";
    Cookie.prototype.value = "";
    // the order in which the RFC has them:
    Cookie.prototype.expires = "Infinity"; // coerces to literal Infinity
    Cookie.prototype.maxAge = null; // takes precedence over expires for TTL
    Cookie.prototype.domain = null;
    Cookie.prototype.path = null;
    Cookie.prototype.secure = false;
    Cookie.prototype.httpOnly = false;
    Cookie.prototype.extensions = null;
    // set by the CookieJar:
    Cookie.prototype.hostOnly = null; // boolean when set
    Cookie.prototype.pathIsDefault = null; // boolean when set
    Cookie.prototype.creation = null; // Date when set; defaulted by Cookie.parse
    Cookie.prototype.lastAccessed = null; // Date when set
    Object.defineProperty(Cookie.prototype, 'creationIndex', {
        configurable: true,
        enumerable: false,
        writable: true,
        value: 0
    });
    Cookie.serializableProperties = Object.keys(Cookie.prototype)
        .filter(function (prop) {
        return !(Cookie.prototype[prop] instanceof Function ||
            prop === 'creationIndex' ||
            prop.substr(0, 1) === '_');
    });
    Cookie.prototype.inspect = function inspect() {
        var now = Date.now();
        return 'Cookie="' + this.toString() +
            '; hostOnly=' + (this.hostOnly != null ? this.hostOnly : '?') +
            '; aAge=' + (this.lastAccessed ? (now - this.lastAccessed.getTime()) + 'ms' : '?') +
            '; cAge=' + (this.creation ? (now - this.creation.getTime()) + 'ms' : '?') +
            '"';
    };
    // Use the new custom inspection symbol to add the custom inspect function if
    // available.
    if (util.inspect.custom) {
        Cookie.prototype[util.inspect.custom] = Cookie.prototype.inspect;
    }
    Cookie.prototype.toJSON = function () {
        var obj = {};
        var props = Cookie.serializableProperties;
        for (var i = 0; i < props.length; i++) {
            var prop = props[i];
            if (this[prop] === Cookie.prototype[prop]) {
                continue; // leave as prototype default
            }
            if (prop === 'expires' ||
                prop === 'creation' ||
                prop === 'lastAccessed') {
                if (this[prop] === null) {
                    obj[prop] = null;
                }
                else {
                    obj[prop] = this[prop] == "Infinity" ? // intentionally not ===
                        "Infinity" : this[prop].toISOString();
                }
            }
            else if (prop === 'maxAge') {
                if (this[prop] !== null) {
                    // again, intentionally not ===
                    obj[prop] = (this[prop] == Infinity || this[prop] == -Infinity) ?
                        this[prop].toString() : this[prop];
                }
            }
            else {
                if (this[prop] !== Cookie.prototype[prop]) {
                    obj[prop] = this[prop];
                }
            }
        }
        return obj;
    };
    Cookie.prototype.clone = function () {
        return fromJSON(this.toJSON());
    };
    Cookie.prototype.validate = function validate() {
        if (!COOKIE_OCTETS.test(this.value)) {
            return false;
        }
        if (this.expires != Infinity && !(this.expires instanceof Date) && !parseDate(this.expires)) {
            return false;
        }
        if (this.maxAge != null && this.maxAge <= 0) {
            return false; // "Max-Age=" non-zero-digit *DIGIT
        }
        if (this.path != null && !PATH_VALUE.test(this.path)) {
            return false;
        }
        var cdomain = this.cdomain();
        if (cdomain) {
            if (cdomain.match(/\.$/)) {
                return false; // S4.1.2.3 suggests that this is bad. domainMatch() tests confirm this
            }
            var suffix = pubsuffix.getPublicSuffix(cdomain);
            if (suffix == null) { // it's a public suffix
                return false;
            }
        }
        return true;
    };
    Cookie.prototype.setExpires = function setExpires(exp) {
        if (exp instanceof Date) {
            this.expires = exp;
        }
        else {
            this.expires = parseDate(exp) || "Infinity";
        }
    };
    Cookie.prototype.setMaxAge = function setMaxAge(age) {
        if (age === Infinity || age === -Infinity) {
            this.maxAge = age.toString(); // so JSON.stringify() works
        }
        else {
            this.maxAge = age;
        }
    };
    // gives Cookie header format
    Cookie.prototype.cookieString = function cookieString() {
        var val = this.value;
        if (val == null) {
            val = '';
        }
        if (this.key === '') {
            return val;
        }
        return this.key + '=' + val;
    };
    // gives Set-Cookie header format
    Cookie.prototype.toString = function toString() {
        var str = this.cookieString();
        if (this.expires != Infinity) {
            if (this.expires instanceof Date) {
                str += '; Expires=' + formatDate(this.expires);
            }
            else {
                str += '; Expires=' + this.expires;
            }
        }
        if (this.maxAge != null && this.maxAge != Infinity) {
            str += '; Max-Age=' + this.maxAge;
        }
        if (this.domain && !this.hostOnly) {
            str += '; Domain=' + this.domain;
        }
        if (this.path) {
            str += '; Path=' + this.path;
        }
        if (this.secure) {
            str += '; Secure';
        }
        if (this.httpOnly) {
            str += '; HttpOnly';
        }
        if (this.extensions) {
            this.extensions.forEach(function (ext) {
                str += '; ' + ext;
            });
        }
        return str;
    };
    // TTL() partially replaces the "expiry-time" parts of S5.3 step 3 (setCookie()
    // elsewhere)
    // S5.3 says to give the "latest representable date" for which we use Infinity
    // For "expired" we use 0
    Cookie.prototype.TTL = function TTL(now) {
        /* RFC6265 S4.1.2.2 If a cookie has both the Max-Age and the Expires
         * attribute, the Max-Age attribute has precedence and controls the
         * expiration date of the cookie.
         * (Concurs with S5.3 step 3)
         */
        if (this.maxAge != null) {
            return this.maxAge <= 0 ? 0 : this.maxAge * 1000;
        }
        var expires = this.expires;
        if (expires != Infinity) {
            if (!(expires instanceof Date)) {
                expires = parseDate(expires) || Infinity;
            }
            if (expires == Infinity) {
                return Infinity;
            }
            return expires.getTime() - (now || Date.now());
        }
        return Infinity;
    };
    // expiryTime() replaces the "expiry-time" parts of S5.3 step 3 (setCookie()
    // elsewhere)
    Cookie.prototype.expiryTime = function expiryTime(now) {
        if (this.maxAge != null) {
            var relativeTo = now || this.creation || new Date();
            var age = (this.maxAge <= 0) ? -Infinity : this.maxAge * 1000;
            return relativeTo.getTime() + age;
        }
        if (this.expires == Infinity) {
            return Infinity;
        }
        return this.expires.getTime();
    };
    // expiryDate() replaces the "expiry-time" parts of S5.3 step 3 (setCookie()
    // elsewhere), except it returns a Date
    Cookie.prototype.expiryDate = function expiryDate(now) {
        var millisec = this.expiryTime(now);
        if (millisec == Infinity) {
            return new Date(MAX_TIME);
        }
        else if (millisec == -Infinity) {
            return new Date(MIN_TIME);
        }
        else {
            return new Date(millisec);
        }
    };
    // This replaces the "persistent-flag" parts of S5.3 step 3
    Cookie.prototype.isPersistent = function isPersistent() {
        return (this.maxAge != null || this.expires != Infinity);
    };
    // Mostly S5.1.2 and S5.2.3:
    Cookie.prototype.cdomain =
        Cookie.prototype.canonicalizedDomain = function canonicalizedDomain() {
            if (this.domain == null) {
                return null;
            }
            return canonicalDomain(this.domain);
        };
    function CookieJar(store, options) {
        if (typeof options === "boolean") {
            options = { rejectPublicSuffixes: options };
        }
        else if (options == null) {
            options = {};
        }
        if (options.rejectPublicSuffixes != null) {
            this.rejectPublicSuffixes = options.rejectPublicSuffixes;
        }
        if (options.looseMode != null) {
            this.enableLooseMode = options.looseMode;
        }
        if (!store) {
            store = new MemoryCookieStore();
        }
        this.store = store;
    }
    CookieJar.prototype.store = null;
    CookieJar.prototype.rejectPublicSuffixes = true;
    CookieJar.prototype.enableLooseMode = false;
    var CAN_BE_SYNC = [];
    CAN_BE_SYNC.push('setCookie');
    CookieJar.prototype.setCookie = function (cookie, url, options, cb) {
        var err;
        var context = getCookieContext(url);
        if (options instanceof Function) {
            cb = options;
            options = {};
        }
        var host = canonicalDomain(context.hostname);
        var loose = this.enableLooseMode;
        if (options.loose != null) {
            loose = options.loose;
        }
        // S5.3 step 1
        if (!(cookie instanceof Cookie)) {
            cookie = Cookie.parse(cookie, { loose: loose });
        }
        if (!cookie) {
            err = new Error("Cookie failed to parse");
            return cb(options.ignoreError ? null : err);
        }
        // S5.3 step 2
        var now = options.now || new Date(); // will assign later to save effort in the face of errors
        // S5.3 step 3: NOOP; persistent-flag and expiry-time is handled by getCookie()
        // S5.3 step 4: NOOP; domain is null by default
        // S5.3 step 5: public suffixes
        if (this.rejectPublicSuffixes && cookie.domain) {
            var suffix = pubsuffix.getPublicSuffix(cookie.cdomain());
            if (suffix == null) { // e.g. "com"
                err = new Error("Cookie has domain set to a public suffix");
                return cb(options.ignoreError ? null : err);
            }
        }
        // S5.3 step 6:
        if (cookie.domain) {
            if (!domainMatch(host, cookie.cdomain(), false)) {
                err = new Error("Cookie not in this host's domain. Cookie:" + cookie.cdomain() + " Request:" + host);
                return cb(options.ignoreError ? null : err);
            }
            if (cookie.hostOnly == null) { // don't reset if already set
                cookie.hostOnly = false;
            }
        }
        else {
            cookie.hostOnly = true;
            cookie.domain = host;
        }
        //S5.2.4 If the attribute-value is empty or if the first character of the
        //attribute-value is not %x2F ("/"):
        //Let cookie-path be the default-path.
        if (!cookie.path || cookie.path[0] !== '/') {
            cookie.path = defaultPath(context.pathname);
            cookie.pathIsDefault = true;
        }
        // S5.3 step 8: NOOP; secure attribute
        // S5.3 step 9: NOOP; httpOnly attribute
        // S5.3 step 10
        if (options.http === false && cookie.httpOnly) {
            err = new Error("Cookie is HttpOnly and this isn't an HTTP API");
            return cb(options.ignoreError ? null : err);
        }
        var store = this.store;
        if (!store.updateCookie) {
            store.updateCookie = function (oldCookie, newCookie, cb) {
                this.putCookie(newCookie, cb);
            };
        }
        function withCookie(err, oldCookie) {
            if (err) {
                return cb(err);
            }
            var next = function (err) {
                if (err) {
                    return cb(err);
                }
                else {
                    cb(null, cookie);
                }
            };
            if (oldCookie) {
                // S5.3 step 11 - "If the cookie store contains a cookie with the same name,
                // domain, and path as the newly created cookie:"
                if (options.http === false && oldCookie.httpOnly) { // step 11.2
                    err = new Error("old Cookie is HttpOnly and this isn't an HTTP API");
                    return cb(options.ignoreError ? null : err);
                }
                cookie.creation = oldCookie.creation; // step 11.3
                cookie.creationIndex = oldCookie.creationIndex; // preserve tie-breaker
                cookie.lastAccessed = now;
                // Step 11.4 (delete cookie) is implied by just setting the new one:
                store.updateCookie(oldCookie, cookie, next); // step 12
            }
            else {
                cookie.creation = cookie.lastAccessed = now;
                store.putCookie(cookie, next); // step 12
            }
        }
        store.findCookie(cookie.domain, cookie.path, cookie.key, withCookie);
    };
    // RFC6365 S5.4
    CAN_BE_SYNC.push('getCookies');
    CookieJar.prototype.getCookies = function (url, options, cb) {
        var context = getCookieContext(url);
        if (options instanceof Function) {
            cb = options;
            options = {};
        }
        var host = canonicalDomain(context.hostname);
        var path = context.pathname || '/';
        var secure = options.secure;
        if (secure == null && context.protocol &&
            (context.protocol == 'https:' || context.protocol == 'wss:')) {
            secure = true;
        }
        var http = options.http;
        if (http == null) {
            http = true;
        }
        var now = options.now || Date.now();
        var expireCheck = options.expire !== false;
        var allPaths = !!options.allPaths;
        var store = this.store;
        function matchingCookie(c) {
            // "Either:
            //   The cookie's host-only-flag is true and the canonicalized
            //   request-host is identical to the cookie's domain.
            // Or:
            //   The cookie's host-only-flag is false and the canonicalized
            //   request-host domain-matches the cookie's domain."
            if (c.hostOnly) {
                if (c.domain != host) {
                    return false;
                }
            }
            else {
                if (!domainMatch(host, c.domain, false)) {
                    return false;
                }
            }
            // "The request-uri's path path-matches the cookie's path."
            if (!allPaths && !pathMatch(path, c.path)) {
                return false;
            }
            // "If the cookie's secure-only-flag is true, then the request-uri's
            // scheme must denote a "secure" protocol"
            if (c.secure && !secure) {
                return false;
            }
            // "If the cookie's http-only-flag is true, then exclude the cookie if the
            // cookie-string is being generated for a "non-HTTP" API"
            if (c.httpOnly && !http) {
                return false;
            }
            // deferred from S5.3
            // non-RFC: allow retention of expired cookies by choice
            if (expireCheck && c.expiryTime() <= now) {
                store.removeCookie(c.domain, c.path, c.key, function () { }); // result ignored
                return false;
            }
            return true;
        }
        store.findCookies(host, allPaths ? null : path, function (err, cookies) {
            if (err) {
                return cb(err);
            }
            cookies = cookies.filter(matchingCookie);
            // sorting of S5.4 part 2
            if (options.sort !== false) {
                cookies = cookies.sort(cookieCompare);
            }
            // S5.4 part 3
            var now = new Date();
            cookies.forEach(function (c) {
                c.lastAccessed = now;
            });
            // TODO persist lastAccessed
            cb(null, cookies);
        });
    };
    CAN_BE_SYNC.push('getCookieString');
    CookieJar.prototype.getCookieString = function ( /*..., cb*/) {
        var args = Array.prototype.slice.call(arguments, 0);
        var cb = args.pop();
        var next = function (err, cookies) {
            if (err) {
                cb(err);
            }
            else {
                cb(null, cookies
                    .sort(cookieCompare)
                    .map(function (c) {
                    return c.cookieString();
                })
                    .join('; '));
            }
        };
        args.push(next);
        this.getCookies.apply(this, args);
    };
    CAN_BE_SYNC.push('getSetCookieStrings');
    CookieJar.prototype.getSetCookieStrings = function ( /*..., cb*/) {
        var args = Array.prototype.slice.call(arguments, 0);
        var cb = args.pop();
        var next = function (err, cookies) {
            if (err) {
                cb(err);
            }
            else {
                cb(null, cookies.map(function (c) {
                    return c.toString();
                }));
            }
        };
        args.push(next);
        this.getCookies.apply(this, args);
    };
    CAN_BE_SYNC.push('serialize');
    CookieJar.prototype.serialize = function (cb) {
        var type = this.store.constructor.name;
        if (type === 'Object') {
            type = null;
        }
        // update README.md "Serialization Format" if you change this, please!
        var serialized = {
            // The version of tough-cookie that serialized this jar. Generally a good
            // practice since future versions can make data import decisions based on
            // known past behavior. When/if this matters, use `semver`.
            version: 'tough-cookie@' + VERSION,
            // add the store type, to make humans happy:
            storeType: type,
            // CookieJar configuration:
            rejectPublicSuffixes: !!this.rejectPublicSuffixes,
            // this gets filled from getAllCookies:
            cookies: []
        };
        if (!(this.store.getAllCookies &&
            typeof this.store.getAllCookies === 'function')) {
            return cb(new Error('store does not support getAllCookies and cannot be serialized'));
        }
        this.store.getAllCookies(function (err, cookies) {
            if (err) {
                return cb(err);
            }
            serialized.cookies = cookies.map(function (cookie) {
                // convert to serialized 'raw' cookies
                cookie = (cookie instanceof Cookie) ? cookie.toJSON() : cookie;
                // Remove the index so new ones get assigned during deserialization
                delete cookie.creationIndex;
                return cookie;
            });
            return cb(null, serialized);
        });
    };
    // well-known name that JSON.stringify calls
    CookieJar.prototype.toJSON = function () {
        return this.serializeSync();
    };
    // use the class method CookieJar.deserialize instead of calling this directly
    CAN_BE_SYNC.push('_importCookies');
    CookieJar.prototype._importCookies = function (serialized, cb) {
        var jar = this;
        var cookies = serialized.cookies;
        if (!cookies || !Array.isArray(cookies)) {
            return cb(new Error('serialized jar has no cookies array'));
        }
        cookies = cookies.slice(); // do not modify the original
        function putNext(err) {
            if (err) {
                return cb(err);
            }
            if (!cookies.length) {
                return cb(err, jar);
            }
            var cookie;
            try {
                cookie = fromJSON(cookies.shift());
            }
            catch (e) {
                return cb(e);
            }
            if (cookie === null) {
                return putNext(null); // skip this cookie
            }
            jar.store.putCookie(cookie, putNext);
        }
        putNext();
    };
    CookieJar.deserialize = function (strOrObj, store, cb) {
        if (arguments.length !== 3) {
            // store is optional
            cb = store;
            store = null;
        }
        var serialized;
        if (typeof strOrObj === 'string') {
            serialized = jsonParse(strOrObj);
            if (serialized instanceof Error) {
                return cb(serialized);
            }
        }
        else {
            serialized = strOrObj;
        }
        var jar = new CookieJar(store, serialized.rejectPublicSuffixes);
        jar._importCookies(serialized, function (err) {
            if (err) {
                return cb(err);
            }
            cb(null, jar);
        });
    };
    CookieJar.deserializeSync = function (strOrObj, store) {
        var serialized = typeof strOrObj === 'string' ?
            JSON.parse(strOrObj) : strOrObj;
        var jar = new CookieJar(store, serialized.rejectPublicSuffixes);
        // catch this mistake early:
        if (!jar.store.synchronous) {
            throw new Error('CookieJar store is not synchronous; use async API instead.');
        }
        jar._importCookiesSync(serialized);
        return jar;
    };
    CookieJar.fromJSON = CookieJar.deserializeSync;
    CookieJar.prototype.clone = function (newStore, cb) {
        if (arguments.length === 1) {
            cb = newStore;
            newStore = null;
        }
        this.serialize(function (err, serialized) {
            if (err) {
                return cb(err);
            }
            CookieJar.deserialize(serialized, newStore, cb);
        });
    };
    CAN_BE_SYNC.push('removeAllCookies');
    CookieJar.prototype.removeAllCookies = function (cb) {
        var store = this.store;
        // Check that the store implements its own removeAllCookies(). The default
        // implementation in Store will immediately call the callback with a "not
        // implemented" Error.
        if (store.removeAllCookies instanceof Function &&
            store.removeAllCookies !== Store.prototype.removeAllCookies) {
            return store.removeAllCookies(cb);
        }
        store.getAllCookies(function (err, cookies) {
            if (err) {
                return cb(err);
            }
            if (cookies.length === 0) {
                return cb(null);
            }
            var completedCount = 0;
            var removeErrors = [];
            function removeCookieCb(removeErr) {
                if (removeErr) {
                    removeErrors.push(removeErr);
                }
                completedCount++;
                if (completedCount === cookies.length) {
                    return cb(removeErrors.length ? removeErrors[0] : null);
                }
            }
            cookies.forEach(function (cookie) {
                store.removeCookie(cookie.domain, cookie.path, cookie.key, removeCookieCb);
            });
        });
    };
    CookieJar.prototype._cloneSync = syncWrap('clone');
    CookieJar.prototype.cloneSync = function (newStore) {
        if (!newStore.synchronous) {
            throw new Error('CookieJar clone destination store is not synchronous; use async API instead.');
        }
        return this._cloneSync(newStore);
    };
    // Use a closure to provide a true imperative API for synchronous stores.
    function syncWrap(method) {
        return function () {
            if (!this.store.synchronous) {
                throw new Error('CookieJar store is not synchronous; use async API instead.');
            }
            var args = Array.prototype.slice.call(arguments);
            var syncErr, syncResult;
            args.push(function syncCb(err, result) {
                syncErr = err;
                syncResult = result;
            });
            this[method].apply(this, args);
            if (syncErr) {
                throw syncErr;
            }
            return syncResult;
        };
    }
    // wrap all declared CAN_BE_SYNC methods in the sync wrapper
    CAN_BE_SYNC.forEach(function (method) {
        CookieJar.prototype[method + 'Sync'] = syncWrap(method);
    });
    exports.version = VERSION;
    exports.CookieJar = CookieJar;
    exports.Cookie = Cookie;
    exports.Store = Store;
    exports.MemoryCookieStore = MemoryCookieStore;
    exports.parseDate = parseDate;
    exports.formatDate = formatDate;
    exports.parse = parse;
    exports.fromJSON = fromJSON;
    exports.domainMatch = domainMatch;
    exports.defaultPath = defaultPath;
    exports.pathMatch = pathMatch;
    exports.getPublicSuffix = pubsuffix.getPublicSuffix;
    exports.cookieCompare = cookieCompare;
    exports.permuteDomain = require(842) /* ./permuteDomain */.permuteDomain;
    exports.permutePath = permutePath;
    exports.canonicalDomain = canonicalDomain;
},
/* tough-cookie/lib/pubsuffix-psl.js */ function _(require, module, exports) {
    var psl = require(838) /* psl */;
    function getPublicSuffix(domain) {
        return psl.get(domain);
    }
    exports.getPublicSuffix = getPublicSuffix;
},
/* psl/index.js */ function _(require, module, exports) {
    var Punycode = require('punycode');
    var internals = {};
    //
    // Read rules from file.
    //
    internals.rules = require(839) /* ./data/rules.json */.map(function (rule) {
        return {
            rule: rule,
            suffix: rule.replace(/^(\*\.|\!)/, ''),
            punySuffix: -1,
            wildcard: rule.charAt(0) === '*',
            exception: rule.charAt(0) === '!'
        };
    });
    //
    // Check is given string ends with `suffix`.
    //
    internals.endsWith = function (str, suffix) {
        return str.indexOf(suffix, str.length - suffix.length) !== -1;
    };
    //
    // Find rule for a given domain.
    //
    internals.findRule = function (domain) {
        var punyDomain = Punycode.toASCII(domain);
        return internals.rules.reduce(function (memo, rule) {
            if (rule.punySuffix === -1) {
                rule.punySuffix = Punycode.toASCII(rule.suffix);
            }
            if (!internals.endsWith(punyDomain, '.' + rule.punySuffix) && punyDomain !== rule.punySuffix) {
                return memo;
            }
            // This has been commented out as it never seems to run. This is because
            // sub tlds always appear after their parents and we never find a shorter
            // match.
            //if (memo) {
            //  var memoSuffix = Punycode.toASCII(memo.suffix);
            //  if (memoSuffix.length >= punySuffix.length) {
            //    return memo;
            //  }
            //}
            return rule;
        }, null);
    };
    //
    // Error codes and messages.
    //
    exports.errorCodes = {
        DOMAIN_TOO_SHORT: 'Domain name too short.',
        DOMAIN_TOO_LONG: 'Domain name too long. It should be no more than 255 chars.',
        LABEL_STARTS_WITH_DASH: 'Domain name label can not start with a dash.',
        LABEL_ENDS_WITH_DASH: 'Domain name label can not end with a dash.',
        LABEL_TOO_LONG: 'Domain name label should be at most 63 chars long.',
        LABEL_TOO_SHORT: 'Domain name label should be at least 1 character long.',
        LABEL_INVALID_CHARS: 'Domain name label can only contain alphanumeric characters or dashes.'
    };
    //
    // Validate domain name and throw if not valid.
    //
    // From wikipedia:
    //
    // Hostnames are composed of series of labels concatenated with dots, as are all
    // domain names. Each label must be between 1 and 63 characters long, and the
    // entire hostname (including the delimiting dots) has a maximum of 255 chars.
    //
    // Allowed chars:
    //
    // * `a-z`
    // * `0-9`
    // * `-` but not as a starting or ending character
    // * `.` as a separator for the textual portions of a domain name
    //
    // * http://en.wikipedia.org/wiki/Domain_name
    // * http://en.wikipedia.org/wiki/Hostname
    //
    internals.validate = function (input) {
        // Before we can validate we need to take care of IDNs with unicode chars.
        var ascii = Punycode.toASCII(input);
        if (ascii.length < 1) {
            return 'DOMAIN_TOO_SHORT';
        }
        if (ascii.length > 255) {
            return 'DOMAIN_TOO_LONG';
        }
        // Check each part's length and allowed chars.
        var labels = ascii.split('.');
        var label;
        for (var i = 0; i < labels.length; ++i) {
            label = labels[i];
            if (!label.length) {
                return 'LABEL_TOO_SHORT';
            }
            if (label.length > 63) {
                return 'LABEL_TOO_LONG';
            }
            if (label.charAt(0) === '-') {
                return 'LABEL_STARTS_WITH_DASH';
            }
            if (label.charAt(label.length - 1) === '-') {
                return 'LABEL_ENDS_WITH_DASH';
            }
            if (!/^[a-z0-9\-]+$/.test(label)) {
                return 'LABEL_INVALID_CHARS';
            }
        }
    };
    //
    // Public API
    //
    //
    // Parse domain.
    //
    exports.parse = function (input) {
        if (typeof input !== 'string') {
            throw new TypeError('Domain name must be a string.');
        }
        // Force domain to lowercase.
        var domain = input.slice(0).toLowerCase();
        // Handle FQDN.
        // TODO: Simply remove trailing dot?
        if (domain.charAt(domain.length - 1) === '.') {
            domain = domain.slice(0, domain.length - 1);
        }
        // Validate and sanitise input.
        var error = internals.validate(domain);
        if (error) {
            return {
                input: input,
                error: {
                    message: exports.errorCodes[error],
                    code: error
                }
            };
        }
        var parsed = {
            input: input,
            tld: null,
            sld: null,
            domain: null,
            subdomain: null,
            listed: false
        };
        var domainParts = domain.split('.');
        // Non-Internet TLD
        if (domainParts[domainParts.length - 1] === 'local') {
            return parsed;
        }
        var handlePunycode = function () {
            if (!/xn--/.test(domain)) {
                return parsed;
            }
            if (parsed.domain) {
                parsed.domain = Punycode.toASCII(parsed.domain);
            }
            if (parsed.subdomain) {
                parsed.subdomain = Punycode.toASCII(parsed.subdomain);
            }
            return parsed;
        };
        var rule = internals.findRule(domain);
        // Unlisted tld.
        if (!rule) {
            if (domainParts.length < 2) {
                return parsed;
            }
            parsed.tld = domainParts.pop();
            parsed.sld = domainParts.pop();
            parsed.domain = [parsed.sld, parsed.tld].join('.');
            if (domainParts.length) {
                parsed.subdomain = domainParts.pop();
            }
            return handlePunycode();
        }
        // At this point we know the public suffix is listed.
        parsed.listed = true;
        var tldParts = rule.suffix.split('.');
        var privateParts = domainParts.slice(0, domainParts.length - tldParts.length);
        if (rule.exception) {
            privateParts.push(tldParts.shift());
        }
        parsed.tld = tldParts.join('.');
        if (!privateParts.length) {
            return handlePunycode();
        }
        if (rule.wildcard) {
            tldParts.unshift(privateParts.pop());
            parsed.tld = tldParts.join('.');
        }
        if (!privateParts.length) {
            return handlePunycode();
        }
        parsed.sld = privateParts.pop();
        parsed.domain = [parsed.sld, parsed.tld].join('.');
        if (privateParts.length) {
            parsed.subdomain = privateParts.join('.');
        }
        return handlePunycode();
    };
    //
    // Get domain.
    //
    exports.get = function (domain) {
        if (!domain) {
            return null;
        }
        return exports.parse(domain).domain || null;
    };
    //
    // Check whether domain belongs to a known public suffix.
    //
    exports.isValid = function (domain) {
        var parsed = exports.parse(domain);
        return Boolean(parsed.domain && parsed.listed);
    };
},
/* psl/data/rules.json */ function _(require, module, exports) {
    module.exports = [
        "ac",
        "com.ac",
        "edu.ac",
        "gov.ac",
        "net.ac",
        "mil.ac",
        "org.ac",
        "ad",
        "nom.ad",
        "ae",
        "co.ae",
        "net.ae",
        "org.ae",
        "sch.ae",
        "ac.ae",
        "gov.ae",
        "mil.ae",
        "aero",
        "accident-investigation.aero",
        "accident-prevention.aero",
        "aerobatic.aero",
        "aeroclub.aero",
        "aerodrome.aero",
        "agents.aero",
        "aircraft.aero",
        "airline.aero",
        "airport.aero",
        "air-surveillance.aero",
        "airtraffic.aero",
        "air-traffic-control.aero",
        "ambulance.aero",
        "amusement.aero",
        "association.aero",
        "author.aero",
        "ballooning.aero",
        "broker.aero",
        "caa.aero",
        "cargo.aero",
        "catering.aero",
        "certification.aero",
        "championship.aero",
        "charter.aero",
        "civilaviation.aero",
        "club.aero",
        "conference.aero",
        "consultant.aero",
        "consulting.aero",
        "control.aero",
        "council.aero",
        "crew.aero",
        "design.aero",
        "dgca.aero",
        "educator.aero",
        "emergency.aero",
        "engine.aero",
        "engineer.aero",
        "entertainment.aero",
        "equipment.aero",
        "exchange.aero",
        "express.aero",
        "federation.aero",
        "flight.aero",
        "freight.aero",
        "fuel.aero",
        "gliding.aero",
        "government.aero",
        "groundhandling.aero",
        "group.aero",
        "hanggliding.aero",
        "homebuilt.aero",
        "insurance.aero",
        "journal.aero",
        "journalist.aero",
        "leasing.aero",
        "logistics.aero",
        "magazine.aero",
        "maintenance.aero",
        "media.aero",
        "microlight.aero",
        "modelling.aero",
        "navigation.aero",
        "parachuting.aero",
        "paragliding.aero",
        "passenger-association.aero",
        "pilot.aero",
        "press.aero",
        "production.aero",
        "recreation.aero",
        "repbody.aero",
        "res.aero",
        "research.aero",
        "rotorcraft.aero",
        "safety.aero",
        "scientist.aero",
        "services.aero",
        "show.aero",
        "skydiving.aero",
        "software.aero",
        "student.aero",
        "trader.aero",
        "trading.aero",
        "trainer.aero",
        "union.aero",
        "workinggroup.aero",
        "works.aero",
        "af",
        "gov.af",
        "com.af",
        "org.af",
        "net.af",
        "edu.af",
        "ag",
        "com.ag",
        "org.ag",
        "net.ag",
        "co.ag",
        "nom.ag",
        "ai",
        "off.ai",
        "com.ai",
        "net.ai",
        "org.ai",
        "al",
        "com.al",
        "edu.al",
        "gov.al",
        "mil.al",
        "net.al",
        "org.al",
        "am",
        "co.am",
        "com.am",
        "commune.am",
        "net.am",
        "org.am",
        "ao",
        "ed.ao",
        "gv.ao",
        "og.ao",
        "co.ao",
        "pb.ao",
        "it.ao",
        "aq",
        "ar",
        "com.ar",
        "edu.ar",
        "gob.ar",
        "gov.ar",
        "int.ar",
        "mil.ar",
        "musica.ar",
        "net.ar",
        "org.ar",
        "tur.ar",
        "arpa",
        "e164.arpa",
        "in-addr.arpa",
        "ip6.arpa",
        "iris.arpa",
        "uri.arpa",
        "urn.arpa",
        "as",
        "gov.as",
        "asia",
        "at",
        "ac.at",
        "co.at",
        "gv.at",
        "or.at",
        "au",
        "com.au",
        "net.au",
        "org.au",
        "edu.au",
        "gov.au",
        "asn.au",
        "id.au",
        "info.au",
        "conf.au",
        "oz.au",
        "act.au",
        "nsw.au",
        "nt.au",
        "qld.au",
        "sa.au",
        "tas.au",
        "vic.au",
        "wa.au",
        "act.edu.au",
        "catholic.edu.au",
        "nsw.edu.au",
        "nt.edu.au",
        "qld.edu.au",
        "sa.edu.au",
        "tas.edu.au",
        "vic.edu.au",
        "wa.edu.au",
        "qld.gov.au",
        "sa.gov.au",
        "tas.gov.au",
        "vic.gov.au",
        "wa.gov.au",
        "education.tas.edu.au",
        "schools.nsw.edu.au",
        "aw",
        "com.aw",
        "ax",
        "az",
        "com.az",
        "net.az",
        "int.az",
        "gov.az",
        "org.az",
        "edu.az",
        "info.az",
        "pp.az",
        "mil.az",
        "name.az",
        "pro.az",
        "biz.az",
        "ba",
        "com.ba",
        "edu.ba",
        "gov.ba",
        "mil.ba",
        "net.ba",
        "org.ba",
        "bb",
        "biz.bb",
        "co.bb",
        "com.bb",
        "edu.bb",
        "gov.bb",
        "info.bb",
        "net.bb",
        "org.bb",
        "store.bb",
        "tv.bb",
        "*.bd",
        "be",
        "ac.be",
        "bf",
        "gov.bf",
        "bg",
        "a.bg",
        "b.bg",
        "c.bg",
        "d.bg",
        "e.bg",
        "f.bg",
        "g.bg",
        "h.bg",
        "i.bg",
        "j.bg",
        "k.bg",
        "l.bg",
        "m.bg",
        "n.bg",
        "o.bg",
        "p.bg",
        "q.bg",
        "r.bg",
        "s.bg",
        "t.bg",
        "u.bg",
        "v.bg",
        "w.bg",
        "x.bg",
        "y.bg",
        "z.bg",
        "0.bg",
        "1.bg",
        "2.bg",
        "3.bg",
        "4.bg",
        "5.bg",
        "6.bg",
        "7.bg",
        "8.bg",
        "9.bg",
        "bh",
        "com.bh",
        "edu.bh",
        "net.bh",
        "org.bh",
        "gov.bh",
        "bi",
        "co.bi",
        "com.bi",
        "edu.bi",
        "or.bi",
        "org.bi",
        "biz",
        "bj",
        "asso.bj",
        "barreau.bj",
        "gouv.bj",
        "bm",
        "com.bm",
        "edu.bm",
        "gov.bm",
        "net.bm",
        "org.bm",
        "bn",
        "com.bn",
        "edu.bn",
        "gov.bn",
        "net.bn",
        "org.bn",
        "bo",
        "com.bo",
        "edu.bo",
        "gob.bo",
        "int.bo",
        "org.bo",
        "net.bo",
        "mil.bo",
        "tv.bo",
        "web.bo",
        "academia.bo",
        "agro.bo",
        "arte.bo",
        "blog.bo",
        "bolivia.bo",
        "ciencia.bo",
        "cooperativa.bo",
        "democracia.bo",
        "deporte.bo",
        "ecologia.bo",
        "economia.bo",
        "empresa.bo",
        "indigena.bo",
        "industria.bo",
        "info.bo",
        "medicina.bo",
        "movimiento.bo",
        "musica.bo",
        "natural.bo",
        "nombre.bo",
        "noticias.bo",
        "patria.bo",
        "politica.bo",
        "profesional.bo",
        "plurinacional.bo",
        "pueblo.bo",
        "revista.bo",
        "salud.bo",
        "tecnologia.bo",
        "tksat.bo",
        "transporte.bo",
        "wiki.bo",
        "br",
        "9guacu.br",
        "abc.br",
        "adm.br",
        "adv.br",
        "agr.br",
        "aju.br",
        "am.br",
        "anani.br",
        "aparecida.br",
        "arq.br",
        "art.br",
        "ato.br",
        "b.br",
        "barueri.br",
        "belem.br",
        "bhz.br",
        "bio.br",
        "blog.br",
        "bmd.br",
        "boavista.br",
        "bsb.br",
        "campinagrande.br",
        "campinas.br",
        "caxias.br",
        "cim.br",
        "cng.br",
        "cnt.br",
        "com.br",
        "contagem.br",
        "coop.br",
        "cri.br",
        "cuiaba.br",
        "curitiba.br",
        "def.br",
        "ecn.br",
        "eco.br",
        "edu.br",
        "emp.br",
        "eng.br",
        "esp.br",
        "etc.br",
        "eti.br",
        "far.br",
        "feira.br",
        "flog.br",
        "floripa.br",
        "fm.br",
        "fnd.br",
        "fortal.br",
        "fot.br",
        "foz.br",
        "fst.br",
        "g12.br",
        "ggf.br",
        "goiania.br",
        "gov.br",
        "ac.gov.br",
        "al.gov.br",
        "am.gov.br",
        "ap.gov.br",
        "ba.gov.br",
        "ce.gov.br",
        "df.gov.br",
        "es.gov.br",
        "go.gov.br",
        "ma.gov.br",
        "mg.gov.br",
        "ms.gov.br",
        "mt.gov.br",
        "pa.gov.br",
        "pb.gov.br",
        "pe.gov.br",
        "pi.gov.br",
        "pr.gov.br",
        "rj.gov.br",
        "rn.gov.br",
        "ro.gov.br",
        "rr.gov.br",
        "rs.gov.br",
        "sc.gov.br",
        "se.gov.br",
        "sp.gov.br",
        "to.gov.br",
        "gru.br",
        "imb.br",
        "ind.br",
        "inf.br",
        "jab.br",
        "jampa.br",
        "jdf.br",
        "joinville.br",
        "jor.br",
        "jus.br",
        "leg.br",
        "lel.br",
        "londrina.br",
        "macapa.br",
        "maceio.br",
        "manaus.br",
        "maringa.br",
        "mat.br",
        "med.br",
        "mil.br",
        "morena.br",
        "mp.br",
        "mus.br",
        "natal.br",
        "net.br",
        "niteroi.br",
        "*.nom.br",
        "not.br",
        "ntr.br",
        "odo.br",
        "ong.br",
        "org.br",
        "osasco.br",
        "palmas.br",
        "poa.br",
        "ppg.br",
        "pro.br",
        "psc.br",
        "psi.br",
        "pvh.br",
        "qsl.br",
        "radio.br",
        "rec.br",
        "recife.br",
        "ribeirao.br",
        "rio.br",
        "riobranco.br",
        "riopreto.br",
        "salvador.br",
        "sampa.br",
        "santamaria.br",
        "santoandre.br",
        "saobernardo.br",
        "saogonca.br",
        "sjc.br",
        "slg.br",
        "slz.br",
        "sorocaba.br",
        "srv.br",
        "taxi.br",
        "tc.br",
        "teo.br",
        "the.br",
        "tmp.br",
        "trd.br",
        "tur.br",
        "tv.br",
        "udi.br",
        "vet.br",
        "vix.br",
        "vlog.br",
        "wiki.br",
        "zlg.br",
        "bs",
        "com.bs",
        "net.bs",
        "org.bs",
        "edu.bs",
        "gov.bs",
        "bt",
        "com.bt",
        "edu.bt",
        "gov.bt",
        "net.bt",
        "org.bt",
        "bv",
        "bw",
        "co.bw",
        "org.bw",
        "by",
        "gov.by",
        "mil.by",
        "com.by",
        "of.by",
        "bz",
        "com.bz",
        "net.bz",
        "org.bz",
        "edu.bz",
        "gov.bz",
        "ca",
        "ab.ca",
        "bc.ca",
        "mb.ca",
        "nb.ca",
        "nf.ca",
        "nl.ca",
        "ns.ca",
        "nt.ca",
        "nu.ca",
        "on.ca",
        "pe.ca",
        "qc.ca",
        "sk.ca",
        "yk.ca",
        "gc.ca",
        "cat",
        "cc",
        "cd",
        "gov.cd",
        "cf",
        "cg",
        "ch",
        "ci",
        "org.ci",
        "or.ci",
        "com.ci",
        "co.ci",
        "edu.ci",
        "ed.ci",
        "ac.ci",
        "net.ci",
        "go.ci",
        "asso.ci",
        "aéroport.ci",
        "int.ci",
        "presse.ci",
        "md.ci",
        "gouv.ci",
        "*.ck",
        "!www.ck",
        "cl",
        "gov.cl",
        "gob.cl",
        "co.cl",
        "mil.cl",
        "cm",
        "co.cm",
        "com.cm",
        "gov.cm",
        "net.cm",
        "cn",
        "ac.cn",
        "com.cn",
        "edu.cn",
        "gov.cn",
        "net.cn",
        "org.cn",
        "mil.cn",
        "公司.cn",
        "网络.cn",
        "網絡.cn",
        "ah.cn",
        "bj.cn",
        "cq.cn",
        "fj.cn",
        "gd.cn",
        "gs.cn",
        "gz.cn",
        "gx.cn",
        "ha.cn",
        "hb.cn",
        "he.cn",
        "hi.cn",
        "hl.cn",
        "hn.cn",
        "jl.cn",
        "js.cn",
        "jx.cn",
        "ln.cn",
        "nm.cn",
        "nx.cn",
        "qh.cn",
        "sc.cn",
        "sd.cn",
        "sh.cn",
        "sn.cn",
        "sx.cn",
        "tj.cn",
        "xj.cn",
        "xz.cn",
        "yn.cn",
        "zj.cn",
        "hk.cn",
        "mo.cn",
        "tw.cn",
        "co",
        "arts.co",
        "com.co",
        "edu.co",
        "firm.co",
        "gov.co",
        "info.co",
        "int.co",
        "mil.co",
        "net.co",
        "nom.co",
        "org.co",
        "rec.co",
        "web.co",
        "com",
        "coop",
        "cr",
        "ac.cr",
        "co.cr",
        "ed.cr",
        "fi.cr",
        "go.cr",
        "or.cr",
        "sa.cr",
        "cu",
        "com.cu",
        "edu.cu",
        "org.cu",
        "net.cu",
        "gov.cu",
        "inf.cu",
        "cv",
        "cw",
        "com.cw",
        "edu.cw",
        "net.cw",
        "org.cw",
        "cx",
        "gov.cx",
        "cy",
        "ac.cy",
        "biz.cy",
        "com.cy",
        "ekloges.cy",
        "gov.cy",
        "ltd.cy",
        "name.cy",
        "net.cy",
        "org.cy",
        "parliament.cy",
        "press.cy",
        "pro.cy",
        "tm.cy",
        "cz",
        "de",
        "dj",
        "dk",
        "dm",
        "com.dm",
        "net.dm",
        "org.dm",
        "edu.dm",
        "gov.dm",
        "do",
        "art.do",
        "com.do",
        "edu.do",
        "gob.do",
        "gov.do",
        "mil.do",
        "net.do",
        "org.do",
        "sld.do",
        "web.do",
        "dz",
        "com.dz",
        "org.dz",
        "net.dz",
        "gov.dz",
        "edu.dz",
        "asso.dz",
        "pol.dz",
        "art.dz",
        "ec",
        "com.ec",
        "info.ec",
        "net.ec",
        "fin.ec",
        "k12.ec",
        "med.ec",
        "pro.ec",
        "org.ec",
        "edu.ec",
        "gov.ec",
        "gob.ec",
        "mil.ec",
        "edu",
        "ee",
        "edu.ee",
        "gov.ee",
        "riik.ee",
        "lib.ee",
        "med.ee",
        "com.ee",
        "pri.ee",
        "aip.ee",
        "org.ee",
        "fie.ee",
        "eg",
        "com.eg",
        "edu.eg",
        "eun.eg",
        "gov.eg",
        "mil.eg",
        "name.eg",
        "net.eg",
        "org.eg",
        "sci.eg",
        "*.er",
        "es",
        "com.es",
        "nom.es",
        "org.es",
        "gob.es",
        "edu.es",
        "et",
        "com.et",
        "gov.et",
        "org.et",
        "edu.et",
        "biz.et",
        "name.et",
        "info.et",
        "net.et",
        "eu",
        "fi",
        "aland.fi",
        "*.fj",
        "*.fk",
        "fm",
        "fo",
        "fr",
        "asso.fr",
        "com.fr",
        "gouv.fr",
        "nom.fr",
        "prd.fr",
        "tm.fr",
        "aeroport.fr",
        "avocat.fr",
        "avoues.fr",
        "cci.fr",
        "chambagri.fr",
        "chirurgiens-dentistes.fr",
        "experts-comptables.fr",
        "geometre-expert.fr",
        "greta.fr",
        "huissier-justice.fr",
        "medecin.fr",
        "notaires.fr",
        "pharmacien.fr",
        "port.fr",
        "veterinaire.fr",
        "ga",
        "gb",
        "gd",
        "ge",
        "com.ge",
        "edu.ge",
        "gov.ge",
        "org.ge",
        "mil.ge",
        "net.ge",
        "pvt.ge",
        "gf",
        "gg",
        "co.gg",
        "net.gg",
        "org.gg",
        "gh",
        "com.gh",
        "edu.gh",
        "gov.gh",
        "org.gh",
        "mil.gh",
        "gi",
        "com.gi",
        "ltd.gi",
        "gov.gi",
        "mod.gi",
        "edu.gi",
        "org.gi",
        "gl",
        "co.gl",
        "com.gl",
        "edu.gl",
        "net.gl",
        "org.gl",
        "gm",
        "gn",
        "ac.gn",
        "com.gn",
        "edu.gn",
        "gov.gn",
        "org.gn",
        "net.gn",
        "gov",
        "gp",
        "com.gp",
        "net.gp",
        "mobi.gp",
        "edu.gp",
        "org.gp",
        "asso.gp",
        "gq",
        "gr",
        "com.gr",
        "edu.gr",
        "net.gr",
        "org.gr",
        "gov.gr",
        "gs",
        "gt",
        "com.gt",
        "edu.gt",
        "gob.gt",
        "ind.gt",
        "mil.gt",
        "net.gt",
        "org.gt",
        "gu",
        "com.gu",
        "edu.gu",
        "gov.gu",
        "guam.gu",
        "info.gu",
        "net.gu",
        "org.gu",
        "web.gu",
        "gw",
        "gy",
        "co.gy",
        "com.gy",
        "edu.gy",
        "gov.gy",
        "net.gy",
        "org.gy",
        "hk",
        "com.hk",
        "edu.hk",
        "gov.hk",
        "idv.hk",
        "net.hk",
        "org.hk",
        "公司.hk",
        "教育.hk",
        "敎育.hk",
        "政府.hk",
        "個人.hk",
        "个人.hk",
        "箇人.hk",
        "網络.hk",
        "网络.hk",
        "组織.hk",
        "網絡.hk",
        "网絡.hk",
        "组织.hk",
        "組織.hk",
        "組织.hk",
        "hm",
        "hn",
        "com.hn",
        "edu.hn",
        "org.hn",
        "net.hn",
        "mil.hn",
        "gob.hn",
        "hr",
        "iz.hr",
        "from.hr",
        "name.hr",
        "com.hr",
        "ht",
        "com.ht",
        "shop.ht",
        "firm.ht",
        "info.ht",
        "adult.ht",
        "net.ht",
        "pro.ht",
        "org.ht",
        "med.ht",
        "art.ht",
        "coop.ht",
        "pol.ht",
        "asso.ht",
        "edu.ht",
        "rel.ht",
        "gouv.ht",
        "perso.ht",
        "hu",
        "co.hu",
        "info.hu",
        "org.hu",
        "priv.hu",
        "sport.hu",
        "tm.hu",
        "2000.hu",
        "agrar.hu",
        "bolt.hu",
        "casino.hu",
        "city.hu",
        "erotica.hu",
        "erotika.hu",
        "film.hu",
        "forum.hu",
        "games.hu",
        "hotel.hu",
        "ingatlan.hu",
        "jogasz.hu",
        "konyvelo.hu",
        "lakas.hu",
        "media.hu",
        "news.hu",
        "reklam.hu",
        "sex.hu",
        "shop.hu",
        "suli.hu",
        "szex.hu",
        "tozsde.hu",
        "utazas.hu",
        "video.hu",
        "id",
        "ac.id",
        "biz.id",
        "co.id",
        "desa.id",
        "go.id",
        "mil.id",
        "my.id",
        "net.id",
        "or.id",
        "ponpes.id",
        "sch.id",
        "web.id",
        "ie",
        "gov.ie",
        "il",
        "ac.il",
        "co.il",
        "gov.il",
        "idf.il",
        "k12.il",
        "muni.il",
        "net.il",
        "org.il",
        "im",
        "ac.im",
        "co.im",
        "com.im",
        "ltd.co.im",
        "net.im",
        "org.im",
        "plc.co.im",
        "tt.im",
        "tv.im",
        "in",
        "co.in",
        "firm.in",
        "net.in",
        "org.in",
        "gen.in",
        "ind.in",
        "nic.in",
        "ac.in",
        "edu.in",
        "res.in",
        "gov.in",
        "mil.in",
        "info",
        "int",
        "eu.int",
        "io",
        "com.io",
        "iq",
        "gov.iq",
        "edu.iq",
        "mil.iq",
        "com.iq",
        "org.iq",
        "net.iq",
        "ir",
        "ac.ir",
        "co.ir",
        "gov.ir",
        "id.ir",
        "net.ir",
        "org.ir",
        "sch.ir",
        "ایران.ir",
        "ايران.ir",
        "is",
        "net.is",
        "com.is",
        "edu.is",
        "gov.is",
        "org.is",
        "int.is",
        "it",
        "gov.it",
        "edu.it",
        "abr.it",
        "abruzzo.it",
        "aosta-valley.it",
        "aostavalley.it",
        "bas.it",
        "basilicata.it",
        "cal.it",
        "calabria.it",
        "cam.it",
        "campania.it",
        "emilia-romagna.it",
        "emiliaromagna.it",
        "emr.it",
        "friuli-v-giulia.it",
        "friuli-ve-giulia.it",
        "friuli-vegiulia.it",
        "friuli-venezia-giulia.it",
        "friuli-veneziagiulia.it",
        "friuli-vgiulia.it",
        "friuliv-giulia.it",
        "friulive-giulia.it",
        "friulivegiulia.it",
        "friulivenezia-giulia.it",
        "friuliveneziagiulia.it",
        "friulivgiulia.it",
        "fvg.it",
        "laz.it",
        "lazio.it",
        "lig.it",
        "liguria.it",
        "lom.it",
        "lombardia.it",
        "lombardy.it",
        "lucania.it",
        "mar.it",
        "marche.it",
        "mol.it",
        "molise.it",
        "piedmont.it",
        "piemonte.it",
        "pmn.it",
        "pug.it",
        "puglia.it",
        "sar.it",
        "sardegna.it",
        "sardinia.it",
        "sic.it",
        "sicilia.it",
        "sicily.it",
        "taa.it",
        "tos.it",
        "toscana.it",
        "trentin-sud-tirol.it",
        "trentin-süd-tirol.it",
        "trentin-sudtirol.it",
        "trentin-südtirol.it",
        "trentin-sued-tirol.it",
        "trentin-suedtirol.it",
        "trentino-a-adige.it",
        "trentino-aadige.it",
        "trentino-alto-adige.it",
        "trentino-altoadige.it",
        "trentino-s-tirol.it",
        "trentino-stirol.it",
        "trentino-sud-tirol.it",
        "trentino-süd-tirol.it",
        "trentino-sudtirol.it",
        "trentino-südtirol.it",
        "trentino-sued-tirol.it",
        "trentino-suedtirol.it",
        "trentino.it",
        "trentinoa-adige.it",
        "trentinoaadige.it",
        "trentinoalto-adige.it",
        "trentinoaltoadige.it",
        "trentinos-tirol.it",
        "trentinostirol.it",
        "trentinosud-tirol.it",
        "trentinosüd-tirol.it",
        "trentinosudtirol.it",
        "trentinosüdtirol.it",
        "trentinosued-tirol.it",
        "trentinosuedtirol.it",
        "trentinsud-tirol.it",
        "trentinsüd-tirol.it",
        "trentinsudtirol.it",
        "trentinsüdtirol.it",
        "trentinsued-tirol.it",
        "trentinsuedtirol.it",
        "tuscany.it",
        "umb.it",
        "umbria.it",
        "val-d-aosta.it",
        "val-daosta.it",
        "vald-aosta.it",
        "valdaosta.it",
        "valle-aosta.it",
        "valle-d-aosta.it",
        "valle-daosta.it",
        "valleaosta.it",
        "valled-aosta.it",
        "valledaosta.it",
        "vallee-aoste.it",
        "vallée-aoste.it",
        "vallee-d-aoste.it",
        "vallée-d-aoste.it",
        "valleeaoste.it",
        "valléeaoste.it",
        "valleedaoste.it",
        "valléedaoste.it",
        "vao.it",
        "vda.it",
        "ven.it",
        "veneto.it",
        "ag.it",
        "agrigento.it",
        "al.it",
        "alessandria.it",
        "alto-adige.it",
        "altoadige.it",
        "an.it",
        "ancona.it",
        "andria-barletta-trani.it",
        "andria-trani-barletta.it",
        "andriabarlettatrani.it",
        "andriatranibarletta.it",
        "ao.it",
        "aosta.it",
        "aoste.it",
        "ap.it",
        "aq.it",
        "aquila.it",
        "ar.it",
        "arezzo.it",
        "ascoli-piceno.it",
        "ascolipiceno.it",
        "asti.it",
        "at.it",
        "av.it",
        "avellino.it",
        "ba.it",
        "balsan-sudtirol.it",
        "balsan-südtirol.it",
        "balsan-suedtirol.it",
        "balsan.it",
        "bari.it",
        "barletta-trani-andria.it",
        "barlettatraniandria.it",
        "belluno.it",
        "benevento.it",
        "bergamo.it",
        "bg.it",
        "bi.it",
        "biella.it",
        "bl.it",
        "bn.it",
        "bo.it",
        "bologna.it",
        "bolzano-altoadige.it",
        "bolzano.it",
        "bozen-sudtirol.it",
        "bozen-südtirol.it",
        "bozen-suedtirol.it",
        "bozen.it",
        "br.it",
        "brescia.it",
        "brindisi.it",
        "bs.it",
        "bt.it",
        "bulsan-sudtirol.it",
        "bulsan-südtirol.it",
        "bulsan-suedtirol.it",
        "bulsan.it",
        "bz.it",
        "ca.it",
        "cagliari.it",
        "caltanissetta.it",
        "campidano-medio.it",
        "campidanomedio.it",
        "campobasso.it",
        "carbonia-iglesias.it",
        "carboniaiglesias.it",
        "carrara-massa.it",
        "carraramassa.it",
        "caserta.it",
        "catania.it",
        "catanzaro.it",
        "cb.it",
        "ce.it",
        "cesena-forli.it",
        "cesena-forlì.it",
        "cesenaforli.it",
        "cesenaforlì.it",
        "ch.it",
        "chieti.it",
        "ci.it",
        "cl.it",
        "cn.it",
        "co.it",
        "como.it",
        "cosenza.it",
        "cr.it",
        "cremona.it",
        "crotone.it",
        "cs.it",
        "ct.it",
        "cuneo.it",
        "cz.it",
        "dell-ogliastra.it",
        "dellogliastra.it",
        "en.it",
        "enna.it",
        "fc.it",
        "fe.it",
        "fermo.it",
        "ferrara.it",
        "fg.it",
        "fi.it",
        "firenze.it",
        "florence.it",
        "fm.it",
        "foggia.it",
        "forli-cesena.it",
        "forlì-cesena.it",
        "forlicesena.it",
        "forlìcesena.it",
        "fr.it",
        "frosinone.it",
        "ge.it",
        "genoa.it",
        "genova.it",
        "go.it",
        "gorizia.it",
        "gr.it",
        "grosseto.it",
        "iglesias-carbonia.it",
        "iglesiascarbonia.it",
        "im.it",
        "imperia.it",
        "is.it",
        "isernia.it",
        "kr.it",
        "la-spezia.it",
        "laquila.it",
        "laspezia.it",
        "latina.it",
        "lc.it",
        "le.it",
        "lecce.it",
        "lecco.it",
        "li.it",
        "livorno.it",
        "lo.it",
        "lodi.it",
        "lt.it",
        "lu.it",
        "lucca.it",
        "macerata.it",
        "mantova.it",
        "massa-carrara.it",
        "massacarrara.it",
        "matera.it",
        "mb.it",
        "mc.it",
        "me.it",
        "medio-campidano.it",
        "mediocampidano.it",
        "messina.it",
        "mi.it",
        "milan.it",
        "milano.it",
        "mn.it",
        "mo.it",
        "modena.it",
        "monza-brianza.it",
        "monza-e-della-brianza.it",
        "monza.it",
        "monzabrianza.it",
        "monzaebrianza.it",
        "monzaedellabrianza.it",
        "ms.it",
        "mt.it",
        "na.it",
        "naples.it",
        "napoli.it",
        "no.it",
        "novara.it",
        "nu.it",
        "nuoro.it",
        "og.it",
        "ogliastra.it",
        "olbia-tempio.it",
        "olbiatempio.it",
        "or.it",
        "oristano.it",
        "ot.it",
        "pa.it",
        "padova.it",
        "padua.it",
        "palermo.it",
        "parma.it",
        "pavia.it",
        "pc.it",
        "pd.it",
        "pe.it",
        "perugia.it",
        "pesaro-urbino.it",
        "pesarourbino.it",
        "pescara.it",
        "pg.it",
        "pi.it",
        "piacenza.it",
        "pisa.it",
        "pistoia.it",
        "pn.it",
        "po.it",
        "pordenone.it",
        "potenza.it",
        "pr.it",
        "prato.it",
        "pt.it",
        "pu.it",
        "pv.it",
        "pz.it",
        "ra.it",
        "ragusa.it",
        "ravenna.it",
        "rc.it",
        "re.it",
        "reggio-calabria.it",
        "reggio-emilia.it",
        "reggiocalabria.it",
        "reggioemilia.it",
        "rg.it",
        "ri.it",
        "rieti.it",
        "rimini.it",
        "rm.it",
        "rn.it",
        "ro.it",
        "roma.it",
        "rome.it",
        "rovigo.it",
        "sa.it",
        "salerno.it",
        "sassari.it",
        "savona.it",
        "si.it",
        "siena.it",
        "siracusa.it",
        "so.it",
        "sondrio.it",
        "sp.it",
        "sr.it",
        "ss.it",
        "suedtirol.it",
        "südtirol.it",
        "sv.it",
        "ta.it",
        "taranto.it",
        "te.it",
        "tempio-olbia.it",
        "tempioolbia.it",
        "teramo.it",
        "terni.it",
        "tn.it",
        "to.it",
        "torino.it",
        "tp.it",
        "tr.it",
        "trani-andria-barletta.it",
        "trani-barletta-andria.it",
        "traniandriabarletta.it",
        "tranibarlettaandria.it",
        "trapani.it",
        "trento.it",
        "treviso.it",
        "trieste.it",
        "ts.it",
        "turin.it",
        "tv.it",
        "ud.it",
        "udine.it",
        "urbino-pesaro.it",
        "urbinopesaro.it",
        "va.it",
        "varese.it",
        "vb.it",
        "vc.it",
        "ve.it",
        "venezia.it",
        "venice.it",
        "verbania.it",
        "vercelli.it",
        "verona.it",
        "vi.it",
        "vibo-valentia.it",
        "vibovalentia.it",
        "vicenza.it",
        "viterbo.it",
        "vr.it",
        "vs.it",
        "vt.it",
        "vv.it",
        "je",
        "co.je",
        "net.je",
        "org.je",
        "*.jm",
        "jo",
        "com.jo",
        "org.jo",
        "net.jo",
        "edu.jo",
        "sch.jo",
        "gov.jo",
        "mil.jo",
        "name.jo",
        "jobs",
        "jp",
        "ac.jp",
        "ad.jp",
        "co.jp",
        "ed.jp",
        "go.jp",
        "gr.jp",
        "lg.jp",
        "ne.jp",
        "or.jp",
        "aichi.jp",
        "akita.jp",
        "aomori.jp",
        "chiba.jp",
        "ehime.jp",
        "fukui.jp",
        "fukuoka.jp",
        "fukushima.jp",
        "gifu.jp",
        "gunma.jp",
        "hiroshima.jp",
        "hokkaido.jp",
        "hyogo.jp",
        "ibaraki.jp",
        "ishikawa.jp",
        "iwate.jp",
        "kagawa.jp",
        "kagoshima.jp",
        "kanagawa.jp",
        "kochi.jp",
        "kumamoto.jp",
        "kyoto.jp",
        "mie.jp",
        "miyagi.jp",
        "miyazaki.jp",
        "nagano.jp",
        "nagasaki.jp",
        "nara.jp",
        "niigata.jp",
        "oita.jp",
        "okayama.jp",
        "okinawa.jp",
        "osaka.jp",
        "saga.jp",
        "saitama.jp",
        "shiga.jp",
        "shimane.jp",
        "shizuoka.jp",
        "tochigi.jp",
        "tokushima.jp",
        "tokyo.jp",
        "tottori.jp",
        "toyama.jp",
        "wakayama.jp",
        "yamagata.jp",
        "yamaguchi.jp",
        "yamanashi.jp",
        "栃木.jp",
        "愛知.jp",
        "愛媛.jp",
        "兵庫.jp",
        "熊本.jp",
        "茨城.jp",
        "北海道.jp",
        "千葉.jp",
        "和歌山.jp",
        "長崎.jp",
        "長野.jp",
        "新潟.jp",
        "青森.jp",
        "静岡.jp",
        "東京.jp",
        "石川.jp",
        "埼玉.jp",
        "三重.jp",
        "京都.jp",
        "佐賀.jp",
        "大分.jp",
        "大阪.jp",
        "奈良.jp",
        "宮城.jp",
        "宮崎.jp",
        "富山.jp",
        "山口.jp",
        "山形.jp",
        "山梨.jp",
        "岩手.jp",
        "岐阜.jp",
        "岡山.jp",
        "島根.jp",
        "広島.jp",
        "徳島.jp",
        "沖縄.jp",
        "滋賀.jp",
        "神奈川.jp",
        "福井.jp",
        "福岡.jp",
        "福島.jp",
        "秋田.jp",
        "群馬.jp",
        "香川.jp",
        "高知.jp",
        "鳥取.jp",
        "鹿児島.jp",
        "*.kawasaki.jp",
        "*.kitakyushu.jp",
        "*.kobe.jp",
        "*.nagoya.jp",
        "*.sapporo.jp",
        "*.sendai.jp",
        "*.yokohama.jp",
        "!city.kawasaki.jp",
        "!city.kitakyushu.jp",
        "!city.kobe.jp",
        "!city.nagoya.jp",
        "!city.sapporo.jp",
        "!city.sendai.jp",
        "!city.yokohama.jp",
        "aisai.aichi.jp",
        "ama.aichi.jp",
        "anjo.aichi.jp",
        "asuke.aichi.jp",
        "chiryu.aichi.jp",
        "chita.aichi.jp",
        "fuso.aichi.jp",
        "gamagori.aichi.jp",
        "handa.aichi.jp",
        "hazu.aichi.jp",
        "hekinan.aichi.jp",
        "higashiura.aichi.jp",
        "ichinomiya.aichi.jp",
        "inazawa.aichi.jp",
        "inuyama.aichi.jp",
        "isshiki.aichi.jp",
        "iwakura.aichi.jp",
        "kanie.aichi.jp",
        "kariya.aichi.jp",
        "kasugai.aichi.jp",
        "kira.aichi.jp",
        "kiyosu.aichi.jp",
        "komaki.aichi.jp",
        "konan.aichi.jp",
        "kota.aichi.jp",
        "mihama.aichi.jp",
        "miyoshi.aichi.jp",
        "nishio.aichi.jp",
        "nisshin.aichi.jp",
        "obu.aichi.jp",
        "oguchi.aichi.jp",
        "oharu.aichi.jp",
        "okazaki.aichi.jp",
        "owariasahi.aichi.jp",
        "seto.aichi.jp",
        "shikatsu.aichi.jp",
        "shinshiro.aichi.jp",
        "shitara.aichi.jp",
        "tahara.aichi.jp",
        "takahama.aichi.jp",
        "tobishima.aichi.jp",
        "toei.aichi.jp",
        "togo.aichi.jp",
        "tokai.aichi.jp",
        "tokoname.aichi.jp",
        "toyoake.aichi.jp",
        "toyohashi.aichi.jp",
        "toyokawa.aichi.jp",
        "toyone.aichi.jp",
        "toyota.aichi.jp",
        "tsushima.aichi.jp",
        "yatomi.aichi.jp",
        "akita.akita.jp",
        "daisen.akita.jp",
        "fujisato.akita.jp",
        "gojome.akita.jp",
        "hachirogata.akita.jp",
        "happou.akita.jp",
        "higashinaruse.akita.jp",
        "honjo.akita.jp",
        "honjyo.akita.jp",
        "ikawa.akita.jp",
        "kamikoani.akita.jp",
        "kamioka.akita.jp",
        "katagami.akita.jp",
        "kazuno.akita.jp",
        "kitaakita.akita.jp",
        "kosaka.akita.jp",
        "kyowa.akita.jp",
        "misato.akita.jp",
        "mitane.akita.jp",
        "moriyoshi.akita.jp",
        "nikaho.akita.jp",
        "noshiro.akita.jp",
        "odate.akita.jp",
        "oga.akita.jp",
        "ogata.akita.jp",
        "semboku.akita.jp",
        "yokote.akita.jp",
        "yurihonjo.akita.jp",
        "aomori.aomori.jp",
        "gonohe.aomori.jp",
        "hachinohe.aomori.jp",
        "hashikami.aomori.jp",
        "hiranai.aomori.jp",
        "hirosaki.aomori.jp",
        "itayanagi.aomori.jp",
        "kuroishi.aomori.jp",
        "misawa.aomori.jp",
        "mutsu.aomori.jp",
        "nakadomari.aomori.jp",
        "noheji.aomori.jp",
        "oirase.aomori.jp",
        "owani.aomori.jp",
        "rokunohe.aomori.jp",
        "sannohe.aomori.jp",
        "shichinohe.aomori.jp",
        "shingo.aomori.jp",
        "takko.aomori.jp",
        "towada.aomori.jp",
        "tsugaru.aomori.jp",
        "tsuruta.aomori.jp",
        "abiko.chiba.jp",
        "asahi.chiba.jp",
        "chonan.chiba.jp",
        "chosei.chiba.jp",
        "choshi.chiba.jp",
        "chuo.chiba.jp",
        "funabashi.chiba.jp",
        "futtsu.chiba.jp",
        "hanamigawa.chiba.jp",
        "ichihara.chiba.jp",
        "ichikawa.chiba.jp",
        "ichinomiya.chiba.jp",
        "inzai.chiba.jp",
        "isumi.chiba.jp",
        "kamagaya.chiba.jp",
        "kamogawa.chiba.jp",
        "kashiwa.chiba.jp",
        "katori.chiba.jp",
        "katsuura.chiba.jp",
        "kimitsu.chiba.jp",
        "kisarazu.chiba.jp",
        "kozaki.chiba.jp",
        "kujukuri.chiba.jp",
        "kyonan.chiba.jp",
        "matsudo.chiba.jp",
        "midori.chiba.jp",
        "mihama.chiba.jp",
        "minamiboso.chiba.jp",
        "mobara.chiba.jp",
        "mutsuzawa.chiba.jp",
        "nagara.chiba.jp",
        "nagareyama.chiba.jp",
        "narashino.chiba.jp",
        "narita.chiba.jp",
        "noda.chiba.jp",
        "oamishirasato.chiba.jp",
        "omigawa.chiba.jp",
        "onjuku.chiba.jp",
        "otaki.chiba.jp",
        "sakae.chiba.jp",
        "sakura.chiba.jp",
        "shimofusa.chiba.jp",
        "shirako.chiba.jp",
        "shiroi.chiba.jp",
        "shisui.chiba.jp",
        "sodegaura.chiba.jp",
        "sosa.chiba.jp",
        "tako.chiba.jp",
        "tateyama.chiba.jp",
        "togane.chiba.jp",
        "tohnosho.chiba.jp",
        "tomisato.chiba.jp",
        "urayasu.chiba.jp",
        "yachimata.chiba.jp",
        "yachiyo.chiba.jp",
        "yokaichiba.chiba.jp",
        "yokoshibahikari.chiba.jp",
        "yotsukaido.chiba.jp",
        "ainan.ehime.jp",
        "honai.ehime.jp",
        "ikata.ehime.jp",
        "imabari.ehime.jp",
        "iyo.ehime.jp",
        "kamijima.ehime.jp",
        "kihoku.ehime.jp",
        "kumakogen.ehime.jp",
        "masaki.ehime.jp",
        "matsuno.ehime.jp",
        "matsuyama.ehime.jp",
        "namikata.ehime.jp",
        "niihama.ehime.jp",
        "ozu.ehime.jp",
        "saijo.ehime.jp",
        "seiyo.ehime.jp",
        "shikokuchuo.ehime.jp",
        "tobe.ehime.jp",
        "toon.ehime.jp",
        "uchiko.ehime.jp",
        "uwajima.ehime.jp",
        "yawatahama.ehime.jp",
        "echizen.fukui.jp",
        "eiheiji.fukui.jp",
        "fukui.fukui.jp",
        "ikeda.fukui.jp",
        "katsuyama.fukui.jp",
        "mihama.fukui.jp",
        "minamiechizen.fukui.jp",
        "obama.fukui.jp",
        "ohi.fukui.jp",
        "ono.fukui.jp",
        "sabae.fukui.jp",
        "sakai.fukui.jp",
        "takahama.fukui.jp",
        "tsuruga.fukui.jp",
        "wakasa.fukui.jp",
        "ashiya.fukuoka.jp",
        "buzen.fukuoka.jp",
        "chikugo.fukuoka.jp",
        "chikuho.fukuoka.jp",
        "chikujo.fukuoka.jp",
        "chikushino.fukuoka.jp",
        "chikuzen.fukuoka.jp",
        "chuo.fukuoka.jp",
        "dazaifu.fukuoka.jp",
        "fukuchi.fukuoka.jp",
        "hakata.fukuoka.jp",
        "higashi.fukuoka.jp",
        "hirokawa.fukuoka.jp",
        "hisayama.fukuoka.jp",
        "iizuka.fukuoka.jp",
        "inatsuki.fukuoka.jp",
        "kaho.fukuoka.jp",
        "kasuga.fukuoka.jp",
        "kasuya.fukuoka.jp",
        "kawara.fukuoka.jp",
        "keisen.fukuoka.jp",
        "koga.fukuoka.jp",
        "kurate.fukuoka.jp",
        "kurogi.fukuoka.jp",
        "kurume.fukuoka.jp",
        "minami.fukuoka.jp",
        "miyako.fukuoka.jp",
        "miyama.fukuoka.jp",
        "miyawaka.fukuoka.jp",
        "mizumaki.fukuoka.jp",
        "munakata.fukuoka.jp",
        "nakagawa.fukuoka.jp",
        "nakama.fukuoka.jp",
        "nishi.fukuoka.jp",
        "nogata.fukuoka.jp",
        "ogori.fukuoka.jp",
        "okagaki.fukuoka.jp",
        "okawa.fukuoka.jp",
        "oki.fukuoka.jp",
        "omuta.fukuoka.jp",
        "onga.fukuoka.jp",
        "onojo.fukuoka.jp",
        "oto.fukuoka.jp",
        "saigawa.fukuoka.jp",
        "sasaguri.fukuoka.jp",
        "shingu.fukuoka.jp",
        "shinyoshitomi.fukuoka.jp",
        "shonai.fukuoka.jp",
        "soeda.fukuoka.jp",
        "sue.fukuoka.jp",
        "tachiarai.fukuoka.jp",
        "tagawa.fukuoka.jp",
        "takata.fukuoka.jp",
        "toho.fukuoka.jp",
        "toyotsu.fukuoka.jp",
        "tsuiki.fukuoka.jp",
        "ukiha.fukuoka.jp",
        "umi.fukuoka.jp",
        "usui.fukuoka.jp",
        "yamada.fukuoka.jp",
        "yame.fukuoka.jp",
        "yanagawa.fukuoka.jp",
        "yukuhashi.fukuoka.jp",
        "aizubange.fukushima.jp",
        "aizumisato.fukushima.jp",
        "aizuwakamatsu.fukushima.jp",
        "asakawa.fukushima.jp",
        "bandai.fukushima.jp",
        "date.fukushima.jp",
        "fukushima.fukushima.jp",
        "furudono.fukushima.jp",
        "futaba.fukushima.jp",
        "hanawa.fukushima.jp",
        "higashi.fukushima.jp",
        "hirata.fukushima.jp",
        "hirono.fukushima.jp",
        "iitate.fukushima.jp",
        "inawashiro.fukushima.jp",
        "ishikawa.fukushima.jp",
        "iwaki.fukushima.jp",
        "izumizaki.fukushima.jp",
        "kagamiishi.fukushima.jp",
        "kaneyama.fukushima.jp",
        "kawamata.fukushima.jp",
        "kitakata.fukushima.jp",
        "kitashiobara.fukushima.jp",
        "koori.fukushima.jp",
        "koriyama.fukushima.jp",
        "kunimi.fukushima.jp",
        "miharu.fukushima.jp",
        "mishima.fukushima.jp",
        "namie.fukushima.jp",
        "nango.fukushima.jp",
        "nishiaizu.fukushima.jp",
        "nishigo.fukushima.jp",
        "okuma.fukushima.jp",
        "omotego.fukushima.jp",
        "ono.fukushima.jp",
        "otama.fukushima.jp",
        "samegawa.fukushima.jp",
        "shimogo.fukushima.jp",
        "shirakawa.fukushima.jp",
        "showa.fukushima.jp",
        "soma.fukushima.jp",
        "sukagawa.fukushima.jp",
        "taishin.fukushima.jp",
        "tamakawa.fukushima.jp",
        "tanagura.fukushima.jp",
        "tenei.fukushima.jp",
        "yabuki.fukushima.jp",
        "yamato.fukushima.jp",
        "yamatsuri.fukushima.jp",
        "yanaizu.fukushima.jp",
        "yugawa.fukushima.jp",
        "anpachi.gifu.jp",
        "ena.gifu.jp",
        "gifu.gifu.jp",
        "ginan.gifu.jp",
        "godo.gifu.jp",
        "gujo.gifu.jp",
        "hashima.gifu.jp",
        "hichiso.gifu.jp",
        "hida.gifu.jp",
        "higashishirakawa.gifu.jp",
        "ibigawa.gifu.jp",
        "ikeda.gifu.jp",
        "kakamigahara.gifu.jp",
        "kani.gifu.jp",
        "kasahara.gifu.jp",
        "kasamatsu.gifu.jp",
        "kawaue.gifu.jp",
        "kitagata.gifu.jp",
        "mino.gifu.jp",
        "minokamo.gifu.jp",
        "mitake.gifu.jp",
        "mizunami.gifu.jp",
        "motosu.gifu.jp",
        "nakatsugawa.gifu.jp",
        "ogaki.gifu.jp",
        "sakahogi.gifu.jp",
        "seki.gifu.jp",
        "sekigahara.gifu.jp",
        "shirakawa.gifu.jp",
        "tajimi.gifu.jp",
        "takayama.gifu.jp",
        "tarui.gifu.jp",
        "toki.gifu.jp",
        "tomika.gifu.jp",
        "wanouchi.gifu.jp",
        "yamagata.gifu.jp",
        "yaotsu.gifu.jp",
        "yoro.gifu.jp",
        "annaka.gunma.jp",
        "chiyoda.gunma.jp",
        "fujioka.gunma.jp",
        "higashiagatsuma.gunma.jp",
        "isesaki.gunma.jp",
        "itakura.gunma.jp",
        "kanna.gunma.jp",
        "kanra.gunma.jp",
        "katashina.gunma.jp",
        "kawaba.gunma.jp",
        "kiryu.gunma.jp",
        "kusatsu.gunma.jp",
        "maebashi.gunma.jp",
        "meiwa.gunma.jp",
        "midori.gunma.jp",
        "minakami.gunma.jp",
        "naganohara.gunma.jp",
        "nakanojo.gunma.jp",
        "nanmoku.gunma.jp",
        "numata.gunma.jp",
        "oizumi.gunma.jp",
        "ora.gunma.jp",
        "ota.gunma.jp",
        "shibukawa.gunma.jp",
        "shimonita.gunma.jp",
        "shinto.gunma.jp",
        "showa.gunma.jp",
        "takasaki.gunma.jp",
        "takayama.gunma.jp",
        "tamamura.gunma.jp",
        "tatebayashi.gunma.jp",
        "tomioka.gunma.jp",
        "tsukiyono.gunma.jp",
        "tsumagoi.gunma.jp",
        "ueno.gunma.jp",
        "yoshioka.gunma.jp",
        "asaminami.hiroshima.jp",
        "daiwa.hiroshima.jp",
        "etajima.hiroshima.jp",
        "fuchu.hiroshima.jp",
        "fukuyama.hiroshima.jp",
        "hatsukaichi.hiroshima.jp",
        "higashihiroshima.hiroshima.jp",
        "hongo.hiroshima.jp",
        "jinsekikogen.hiroshima.jp",
        "kaita.hiroshima.jp",
        "kui.hiroshima.jp",
        "kumano.hiroshima.jp",
        "kure.hiroshima.jp",
        "mihara.hiroshima.jp",
        "miyoshi.hiroshima.jp",
        "naka.hiroshima.jp",
        "onomichi.hiroshima.jp",
        "osakikamijima.hiroshima.jp",
        "otake.hiroshima.jp",
        "saka.hiroshima.jp",
        "sera.hiroshima.jp",
        "seranishi.hiroshima.jp",
        "shinichi.hiroshima.jp",
        "shobara.hiroshima.jp",
        "takehara.hiroshima.jp",
        "abashiri.hokkaido.jp",
        "abira.hokkaido.jp",
        "aibetsu.hokkaido.jp",
        "akabira.hokkaido.jp",
        "akkeshi.hokkaido.jp",
        "asahikawa.hokkaido.jp",
        "ashibetsu.hokkaido.jp",
        "ashoro.hokkaido.jp",
        "assabu.hokkaido.jp",
        "atsuma.hokkaido.jp",
        "bibai.hokkaido.jp",
        "biei.hokkaido.jp",
        "bifuka.hokkaido.jp",
        "bihoro.hokkaido.jp",
        "biratori.hokkaido.jp",
        "chippubetsu.hokkaido.jp",
        "chitose.hokkaido.jp",
        "date.hokkaido.jp",
        "ebetsu.hokkaido.jp",
        "embetsu.hokkaido.jp",
        "eniwa.hokkaido.jp",
        "erimo.hokkaido.jp",
        "esan.hokkaido.jp",
        "esashi.hokkaido.jp",
        "fukagawa.hokkaido.jp",
        "fukushima.hokkaido.jp",
        "furano.hokkaido.jp",
        "furubira.hokkaido.jp",
        "haboro.hokkaido.jp",
        "hakodate.hokkaido.jp",
        "hamatonbetsu.hokkaido.jp",
        "hidaka.hokkaido.jp",
        "higashikagura.hokkaido.jp",
        "higashikawa.hokkaido.jp",
        "hiroo.hokkaido.jp",
        "hokuryu.hokkaido.jp",
        "hokuto.hokkaido.jp",
        "honbetsu.hokkaido.jp",
        "horokanai.hokkaido.jp",
        "horonobe.hokkaido.jp",
        "ikeda.hokkaido.jp",
        "imakane.hokkaido.jp",
        "ishikari.hokkaido.jp",
        "iwamizawa.hokkaido.jp",
        "iwanai.hokkaido.jp",
        "kamifurano.hokkaido.jp",
        "kamikawa.hokkaido.jp",
        "kamishihoro.hokkaido.jp",
        "kamisunagawa.hokkaido.jp",
        "kamoenai.hokkaido.jp",
        "kayabe.hokkaido.jp",
        "kembuchi.hokkaido.jp",
        "kikonai.hokkaido.jp",
        "kimobetsu.hokkaido.jp",
        "kitahiroshima.hokkaido.jp",
        "kitami.hokkaido.jp",
        "kiyosato.hokkaido.jp",
        "koshimizu.hokkaido.jp",
        "kunneppu.hokkaido.jp",
        "kuriyama.hokkaido.jp",
        "kuromatsunai.hokkaido.jp",
        "kushiro.hokkaido.jp",
        "kutchan.hokkaido.jp",
        "kyowa.hokkaido.jp",
        "mashike.hokkaido.jp",
        "matsumae.hokkaido.jp",
        "mikasa.hokkaido.jp",
        "minamifurano.hokkaido.jp",
        "mombetsu.hokkaido.jp",
        "moseushi.hokkaido.jp",
        "mukawa.hokkaido.jp",
        "muroran.hokkaido.jp",
        "naie.hokkaido.jp",
        "nakagawa.hokkaido.jp",
        "nakasatsunai.hokkaido.jp",
        "nakatombetsu.hokkaido.jp",
        "nanae.hokkaido.jp",
        "nanporo.hokkaido.jp",
        "nayoro.hokkaido.jp",
        "nemuro.hokkaido.jp",
        "niikappu.hokkaido.jp",
        "niki.hokkaido.jp",
        "nishiokoppe.hokkaido.jp",
        "noboribetsu.hokkaido.jp",
        "numata.hokkaido.jp",
        "obihiro.hokkaido.jp",
        "obira.hokkaido.jp",
        "oketo.hokkaido.jp",
        "okoppe.hokkaido.jp",
        "otaru.hokkaido.jp",
        "otobe.hokkaido.jp",
        "otofuke.hokkaido.jp",
        "otoineppu.hokkaido.jp",
        "oumu.hokkaido.jp",
        "ozora.hokkaido.jp",
        "pippu.hokkaido.jp",
        "rankoshi.hokkaido.jp",
        "rebun.hokkaido.jp",
        "rikubetsu.hokkaido.jp",
        "rishiri.hokkaido.jp",
        "rishirifuji.hokkaido.jp",
        "saroma.hokkaido.jp",
        "sarufutsu.hokkaido.jp",
        "shakotan.hokkaido.jp",
        "shari.hokkaido.jp",
        "shibecha.hokkaido.jp",
        "shibetsu.hokkaido.jp",
        "shikabe.hokkaido.jp",
        "shikaoi.hokkaido.jp",
        "shimamaki.hokkaido.jp",
        "shimizu.hokkaido.jp",
        "shimokawa.hokkaido.jp",
        "shinshinotsu.hokkaido.jp",
        "shintoku.hokkaido.jp",
        "shiranuka.hokkaido.jp",
        "shiraoi.hokkaido.jp",
        "shiriuchi.hokkaido.jp",
        "sobetsu.hokkaido.jp",
        "sunagawa.hokkaido.jp",
        "taiki.hokkaido.jp",
        "takasu.hokkaido.jp",
        "takikawa.hokkaido.jp",
        "takinoue.hokkaido.jp",
        "teshikaga.hokkaido.jp",
        "tobetsu.hokkaido.jp",
        "tohma.hokkaido.jp",
        "tomakomai.hokkaido.jp",
        "tomari.hokkaido.jp",
        "toya.hokkaido.jp",
        "toyako.hokkaido.jp",
        "toyotomi.hokkaido.jp",
        "toyoura.hokkaido.jp",
        "tsubetsu.hokkaido.jp",
        "tsukigata.hokkaido.jp",
        "urakawa.hokkaido.jp",
        "urausu.hokkaido.jp",
        "uryu.hokkaido.jp",
        "utashinai.hokkaido.jp",
        "wakkanai.hokkaido.jp",
        "wassamu.hokkaido.jp",
        "yakumo.hokkaido.jp",
        "yoichi.hokkaido.jp",
        "aioi.hyogo.jp",
        "akashi.hyogo.jp",
        "ako.hyogo.jp",
        "amagasaki.hyogo.jp",
        "aogaki.hyogo.jp",
        "asago.hyogo.jp",
        "ashiya.hyogo.jp",
        "awaji.hyogo.jp",
        "fukusaki.hyogo.jp",
        "goshiki.hyogo.jp",
        "harima.hyogo.jp",
        "himeji.hyogo.jp",
        "ichikawa.hyogo.jp",
        "inagawa.hyogo.jp",
        "itami.hyogo.jp",
        "kakogawa.hyogo.jp",
        "kamigori.hyogo.jp",
        "kamikawa.hyogo.jp",
        "kasai.hyogo.jp",
        "kasuga.hyogo.jp",
        "kawanishi.hyogo.jp",
        "miki.hyogo.jp",
        "minamiawaji.hyogo.jp",
        "nishinomiya.hyogo.jp",
        "nishiwaki.hyogo.jp",
        "ono.hyogo.jp",
        "sanda.hyogo.jp",
        "sannan.hyogo.jp",
        "sasayama.hyogo.jp",
        "sayo.hyogo.jp",
        "shingu.hyogo.jp",
        "shinonsen.hyogo.jp",
        "shiso.hyogo.jp",
        "sumoto.hyogo.jp",
        "taishi.hyogo.jp",
        "taka.hyogo.jp",
        "takarazuka.hyogo.jp",
        "takasago.hyogo.jp",
        "takino.hyogo.jp",
        "tamba.hyogo.jp",
        "tatsuno.hyogo.jp",
        "toyooka.hyogo.jp",
        "yabu.hyogo.jp",
        "yashiro.hyogo.jp",
        "yoka.hyogo.jp",
        "yokawa.hyogo.jp",
        "ami.ibaraki.jp",
        "asahi.ibaraki.jp",
        "bando.ibaraki.jp",
        "chikusei.ibaraki.jp",
        "daigo.ibaraki.jp",
        "fujishiro.ibaraki.jp",
        "hitachi.ibaraki.jp",
        "hitachinaka.ibaraki.jp",
        "hitachiomiya.ibaraki.jp",
        "hitachiota.ibaraki.jp",
        "ibaraki.ibaraki.jp",
        "ina.ibaraki.jp",
        "inashiki.ibaraki.jp",
        "itako.ibaraki.jp",
        "iwama.ibaraki.jp",
        "joso.ibaraki.jp",
        "kamisu.ibaraki.jp",
        "kasama.ibaraki.jp",
        "kashima.ibaraki.jp",
        "kasumigaura.ibaraki.jp",
        "koga.ibaraki.jp",
        "miho.ibaraki.jp",
        "mito.ibaraki.jp",
        "moriya.ibaraki.jp",
        "naka.ibaraki.jp",
        "namegata.ibaraki.jp",
        "oarai.ibaraki.jp",
        "ogawa.ibaraki.jp",
        "omitama.ibaraki.jp",
        "ryugasaki.ibaraki.jp",
        "sakai.ibaraki.jp",
        "sakuragawa.ibaraki.jp",
        "shimodate.ibaraki.jp",
        "shimotsuma.ibaraki.jp",
        "shirosato.ibaraki.jp",
        "sowa.ibaraki.jp",
        "suifu.ibaraki.jp",
        "takahagi.ibaraki.jp",
        "tamatsukuri.ibaraki.jp",
        "tokai.ibaraki.jp",
        "tomobe.ibaraki.jp",
        "tone.ibaraki.jp",
        "toride.ibaraki.jp",
        "tsuchiura.ibaraki.jp",
        "tsukuba.ibaraki.jp",
        "uchihara.ibaraki.jp",
        "ushiku.ibaraki.jp",
        "yachiyo.ibaraki.jp",
        "yamagata.ibaraki.jp",
        "yawara.ibaraki.jp",
        "yuki.ibaraki.jp",
        "anamizu.ishikawa.jp",
        "hakui.ishikawa.jp",
        "hakusan.ishikawa.jp",
        "kaga.ishikawa.jp",
        "kahoku.ishikawa.jp",
        "kanazawa.ishikawa.jp",
        "kawakita.ishikawa.jp",
        "komatsu.ishikawa.jp",
        "nakanoto.ishikawa.jp",
        "nanao.ishikawa.jp",
        "nomi.ishikawa.jp",
        "nonoichi.ishikawa.jp",
        "noto.ishikawa.jp",
        "shika.ishikawa.jp",
        "suzu.ishikawa.jp",
        "tsubata.ishikawa.jp",
        "tsurugi.ishikawa.jp",
        "uchinada.ishikawa.jp",
        "wajima.ishikawa.jp",
        "fudai.iwate.jp",
        "fujisawa.iwate.jp",
        "hanamaki.iwate.jp",
        "hiraizumi.iwate.jp",
        "hirono.iwate.jp",
        "ichinohe.iwate.jp",
        "ichinoseki.iwate.jp",
        "iwaizumi.iwate.jp",
        "iwate.iwate.jp",
        "joboji.iwate.jp",
        "kamaishi.iwate.jp",
        "kanegasaki.iwate.jp",
        "karumai.iwate.jp",
        "kawai.iwate.jp",
        "kitakami.iwate.jp",
        "kuji.iwate.jp",
        "kunohe.iwate.jp",
        "kuzumaki.iwate.jp",
        "miyako.iwate.jp",
        "mizusawa.iwate.jp",
        "morioka.iwate.jp",
        "ninohe.iwate.jp",
        "noda.iwate.jp",
        "ofunato.iwate.jp",
        "oshu.iwate.jp",
        "otsuchi.iwate.jp",
        "rikuzentakata.iwate.jp",
        "shiwa.iwate.jp",
        "shizukuishi.iwate.jp",
        "sumita.iwate.jp",
        "tanohata.iwate.jp",
        "tono.iwate.jp",
        "yahaba.iwate.jp",
        "yamada.iwate.jp",
        "ayagawa.kagawa.jp",
        "higashikagawa.kagawa.jp",
        "kanonji.kagawa.jp",
        "kotohira.kagawa.jp",
        "manno.kagawa.jp",
        "marugame.kagawa.jp",
        "mitoyo.kagawa.jp",
        "naoshima.kagawa.jp",
        "sanuki.kagawa.jp",
        "tadotsu.kagawa.jp",
        "takamatsu.kagawa.jp",
        "tonosho.kagawa.jp",
        "uchinomi.kagawa.jp",
        "utazu.kagawa.jp",
        "zentsuji.kagawa.jp",
        "akune.kagoshima.jp",
        "amami.kagoshima.jp",
        "hioki.kagoshima.jp",
        "isa.kagoshima.jp",
        "isen.kagoshima.jp",
        "izumi.kagoshima.jp",
        "kagoshima.kagoshima.jp",
        "kanoya.kagoshima.jp",
        "kawanabe.kagoshima.jp",
        "kinko.kagoshima.jp",
        "kouyama.kagoshima.jp",
        "makurazaki.kagoshima.jp",
        "matsumoto.kagoshima.jp",
        "minamitane.kagoshima.jp",
        "nakatane.kagoshima.jp",
        "nishinoomote.kagoshima.jp",
        "satsumasendai.kagoshima.jp",
        "soo.kagoshima.jp",
        "tarumizu.kagoshima.jp",
        "yusui.kagoshima.jp",
        "aikawa.kanagawa.jp",
        "atsugi.kanagawa.jp",
        "ayase.kanagawa.jp",
        "chigasaki.kanagawa.jp",
        "ebina.kanagawa.jp",
        "fujisawa.kanagawa.jp",
        "hadano.kanagawa.jp",
        "hakone.kanagawa.jp",
        "hiratsuka.kanagawa.jp",
        "isehara.kanagawa.jp",
        "kaisei.kanagawa.jp",
        "kamakura.kanagawa.jp",
        "kiyokawa.kanagawa.jp",
        "matsuda.kanagawa.jp",
        "minamiashigara.kanagawa.jp",
        "miura.kanagawa.jp",
        "nakai.kanagawa.jp",
        "ninomiya.kanagawa.jp",
        "odawara.kanagawa.jp",
        "oi.kanagawa.jp",
        "oiso.kanagawa.jp",
        "sagamihara.kanagawa.jp",
        "samukawa.kanagawa.jp",
        "tsukui.kanagawa.jp",
        "yamakita.kanagawa.jp",
        "yamato.kanagawa.jp",
        "yokosuka.kanagawa.jp",
        "yugawara.kanagawa.jp",
        "zama.kanagawa.jp",
        "zushi.kanagawa.jp",
        "aki.kochi.jp",
        "geisei.kochi.jp",
        "hidaka.kochi.jp",
        "higashitsuno.kochi.jp",
        "ino.kochi.jp",
        "kagami.kochi.jp",
        "kami.kochi.jp",
        "kitagawa.kochi.jp",
        "kochi.kochi.jp",
        "mihara.kochi.jp",
        "motoyama.kochi.jp",
        "muroto.kochi.jp",
        "nahari.kochi.jp",
        "nakamura.kochi.jp",
        "nankoku.kochi.jp",
        "nishitosa.kochi.jp",
        "niyodogawa.kochi.jp",
        "ochi.kochi.jp",
        "okawa.kochi.jp",
        "otoyo.kochi.jp",
        "otsuki.kochi.jp",
        "sakawa.kochi.jp",
        "sukumo.kochi.jp",
        "susaki.kochi.jp",
        "tosa.kochi.jp",
        "tosashimizu.kochi.jp",
        "toyo.kochi.jp",
        "tsuno.kochi.jp",
        "umaji.kochi.jp",
        "yasuda.kochi.jp",
        "yusuhara.kochi.jp",
        "amakusa.kumamoto.jp",
        "arao.kumamoto.jp",
        "aso.kumamoto.jp",
        "choyo.kumamoto.jp",
        "gyokuto.kumamoto.jp",
        "kamiamakusa.kumamoto.jp",
        "kikuchi.kumamoto.jp",
        "kumamoto.kumamoto.jp",
        "mashiki.kumamoto.jp",
        "mifune.kumamoto.jp",
        "minamata.kumamoto.jp",
        "minamioguni.kumamoto.jp",
        "nagasu.kumamoto.jp",
        "nishihara.kumamoto.jp",
        "oguni.kumamoto.jp",
        "ozu.kumamoto.jp",
        "sumoto.kumamoto.jp",
        "takamori.kumamoto.jp",
        "uki.kumamoto.jp",
        "uto.kumamoto.jp",
        "yamaga.kumamoto.jp",
        "yamato.kumamoto.jp",
        "yatsushiro.kumamoto.jp",
        "ayabe.kyoto.jp",
        "fukuchiyama.kyoto.jp",
        "higashiyama.kyoto.jp",
        "ide.kyoto.jp",
        "ine.kyoto.jp",
        "joyo.kyoto.jp",
        "kameoka.kyoto.jp",
        "kamo.kyoto.jp",
        "kita.kyoto.jp",
        "kizu.kyoto.jp",
        "kumiyama.kyoto.jp",
        "kyotamba.kyoto.jp",
        "kyotanabe.kyoto.jp",
        "kyotango.kyoto.jp",
        "maizuru.kyoto.jp",
        "minami.kyoto.jp",
        "minamiyamashiro.kyoto.jp",
        "miyazu.kyoto.jp",
        "muko.kyoto.jp",
        "nagaokakyo.kyoto.jp",
        "nakagyo.kyoto.jp",
        "nantan.kyoto.jp",
        "oyamazaki.kyoto.jp",
        "sakyo.kyoto.jp",
        "seika.kyoto.jp",
        "tanabe.kyoto.jp",
        "uji.kyoto.jp",
        "ujitawara.kyoto.jp",
        "wazuka.kyoto.jp",
        "yamashina.kyoto.jp",
        "yawata.kyoto.jp",
        "asahi.mie.jp",
        "inabe.mie.jp",
        "ise.mie.jp",
        "kameyama.mie.jp",
        "kawagoe.mie.jp",
        "kiho.mie.jp",
        "kisosaki.mie.jp",
        "kiwa.mie.jp",
        "komono.mie.jp",
        "kumano.mie.jp",
        "kuwana.mie.jp",
        "matsusaka.mie.jp",
        "meiwa.mie.jp",
        "mihama.mie.jp",
        "minamiise.mie.jp",
        "misugi.mie.jp",
        "miyama.mie.jp",
        "nabari.mie.jp",
        "shima.mie.jp",
        "suzuka.mie.jp",
        "tado.mie.jp",
        "taiki.mie.jp",
        "taki.mie.jp",
        "tamaki.mie.jp",
        "toba.mie.jp",
        "tsu.mie.jp",
        "udono.mie.jp",
        "ureshino.mie.jp",
        "watarai.mie.jp",
        "yokkaichi.mie.jp",
        "furukawa.miyagi.jp",
        "higashimatsushima.miyagi.jp",
        "ishinomaki.miyagi.jp",
        "iwanuma.miyagi.jp",
        "kakuda.miyagi.jp",
        "kami.miyagi.jp",
        "kawasaki.miyagi.jp",
        "marumori.miyagi.jp",
        "matsushima.miyagi.jp",
        "minamisanriku.miyagi.jp",
        "misato.miyagi.jp",
        "murata.miyagi.jp",
        "natori.miyagi.jp",
        "ogawara.miyagi.jp",
        "ohira.miyagi.jp",
        "onagawa.miyagi.jp",
        "osaki.miyagi.jp",
        "rifu.miyagi.jp",
        "semine.miyagi.jp",
        "shibata.miyagi.jp",
        "shichikashuku.miyagi.jp",
        "shikama.miyagi.jp",
        "shiogama.miyagi.jp",
        "shiroishi.miyagi.jp",
        "tagajo.miyagi.jp",
        "taiwa.miyagi.jp",
        "tome.miyagi.jp",
        "tomiya.miyagi.jp",
        "wakuya.miyagi.jp",
        "watari.miyagi.jp",
        "yamamoto.miyagi.jp",
        "zao.miyagi.jp",
        "aya.miyazaki.jp",
        "ebino.miyazaki.jp",
        "gokase.miyazaki.jp",
        "hyuga.miyazaki.jp",
        "kadogawa.miyazaki.jp",
        "kawaminami.miyazaki.jp",
        "kijo.miyazaki.jp",
        "kitagawa.miyazaki.jp",
        "kitakata.miyazaki.jp",
        "kitaura.miyazaki.jp",
        "kobayashi.miyazaki.jp",
        "kunitomi.miyazaki.jp",
        "kushima.miyazaki.jp",
        "mimata.miyazaki.jp",
        "miyakonojo.miyazaki.jp",
        "miyazaki.miyazaki.jp",
        "morotsuka.miyazaki.jp",
        "nichinan.miyazaki.jp",
        "nishimera.miyazaki.jp",
        "nobeoka.miyazaki.jp",
        "saito.miyazaki.jp",
        "shiiba.miyazaki.jp",
        "shintomi.miyazaki.jp",
        "takaharu.miyazaki.jp",
        "takanabe.miyazaki.jp",
        "takazaki.miyazaki.jp",
        "tsuno.miyazaki.jp",
        "achi.nagano.jp",
        "agematsu.nagano.jp",
        "anan.nagano.jp",
        "aoki.nagano.jp",
        "asahi.nagano.jp",
        "azumino.nagano.jp",
        "chikuhoku.nagano.jp",
        "chikuma.nagano.jp",
        "chino.nagano.jp",
        "fujimi.nagano.jp",
        "hakuba.nagano.jp",
        "hara.nagano.jp",
        "hiraya.nagano.jp",
        "iida.nagano.jp",
        "iijima.nagano.jp",
        "iiyama.nagano.jp",
        "iizuna.nagano.jp",
        "ikeda.nagano.jp",
        "ikusaka.nagano.jp",
        "ina.nagano.jp",
        "karuizawa.nagano.jp",
        "kawakami.nagano.jp",
        "kiso.nagano.jp",
        "kisofukushima.nagano.jp",
        "kitaaiki.nagano.jp",
        "komagane.nagano.jp",
        "komoro.nagano.jp",
        "matsukawa.nagano.jp",
        "matsumoto.nagano.jp",
        "miasa.nagano.jp",
        "minamiaiki.nagano.jp",
        "minamimaki.nagano.jp",
        "minamiminowa.nagano.jp",
        "minowa.nagano.jp",
        "miyada.nagano.jp",
        "miyota.nagano.jp",
        "mochizuki.nagano.jp",
        "nagano.nagano.jp",
        "nagawa.nagano.jp",
        "nagiso.nagano.jp",
        "nakagawa.nagano.jp",
        "nakano.nagano.jp",
        "nozawaonsen.nagano.jp",
        "obuse.nagano.jp",
        "ogawa.nagano.jp",
        "okaya.nagano.jp",
        "omachi.nagano.jp",
        "omi.nagano.jp",
        "ookuwa.nagano.jp",
        "ooshika.nagano.jp",
        "otaki.nagano.jp",
        "otari.nagano.jp",
        "sakae.nagano.jp",
        "sakaki.nagano.jp",
        "saku.nagano.jp",
        "sakuho.nagano.jp",
        "shimosuwa.nagano.jp",
        "shinanomachi.nagano.jp",
        "shiojiri.nagano.jp",
        "suwa.nagano.jp",
        "suzaka.nagano.jp",
        "takagi.nagano.jp",
        "takamori.nagano.jp",
        "takayama.nagano.jp",
        "tateshina.nagano.jp",
        "tatsuno.nagano.jp",
        "togakushi.nagano.jp",
        "togura.nagano.jp",
        "tomi.nagano.jp",
        "ueda.nagano.jp",
        "wada.nagano.jp",
        "yamagata.nagano.jp",
        "yamanouchi.nagano.jp",
        "yasaka.nagano.jp",
        "yasuoka.nagano.jp",
        "chijiwa.nagasaki.jp",
        "futsu.nagasaki.jp",
        "goto.nagasaki.jp",
        "hasami.nagasaki.jp",
        "hirado.nagasaki.jp",
        "iki.nagasaki.jp",
        "isahaya.nagasaki.jp",
        "kawatana.nagasaki.jp",
        "kuchinotsu.nagasaki.jp",
        "matsuura.nagasaki.jp",
        "nagasaki.nagasaki.jp",
        "obama.nagasaki.jp",
        "omura.nagasaki.jp",
        "oseto.nagasaki.jp",
        "saikai.nagasaki.jp",
        "sasebo.nagasaki.jp",
        "seihi.nagasaki.jp",
        "shimabara.nagasaki.jp",
        "shinkamigoto.nagasaki.jp",
        "togitsu.nagasaki.jp",
        "tsushima.nagasaki.jp",
        "unzen.nagasaki.jp",
        "ando.nara.jp",
        "gose.nara.jp",
        "heguri.nara.jp",
        "higashiyoshino.nara.jp",
        "ikaruga.nara.jp",
        "ikoma.nara.jp",
        "kamikitayama.nara.jp",
        "kanmaki.nara.jp",
        "kashiba.nara.jp",
        "kashihara.nara.jp",
        "katsuragi.nara.jp",
        "kawai.nara.jp",
        "kawakami.nara.jp",
        "kawanishi.nara.jp",
        "koryo.nara.jp",
        "kurotaki.nara.jp",
        "mitsue.nara.jp",
        "miyake.nara.jp",
        "nara.nara.jp",
        "nosegawa.nara.jp",
        "oji.nara.jp",
        "ouda.nara.jp",
        "oyodo.nara.jp",
        "sakurai.nara.jp",
        "sango.nara.jp",
        "shimoichi.nara.jp",
        "shimokitayama.nara.jp",
        "shinjo.nara.jp",
        "soni.nara.jp",
        "takatori.nara.jp",
        "tawaramoto.nara.jp",
        "tenkawa.nara.jp",
        "tenri.nara.jp",
        "uda.nara.jp",
        "yamatokoriyama.nara.jp",
        "yamatotakada.nara.jp",
        "yamazoe.nara.jp",
        "yoshino.nara.jp",
        "aga.niigata.jp",
        "agano.niigata.jp",
        "gosen.niigata.jp",
        "itoigawa.niigata.jp",
        "izumozaki.niigata.jp",
        "joetsu.niigata.jp",
        "kamo.niigata.jp",
        "kariwa.niigata.jp",
        "kashiwazaki.niigata.jp",
        "minamiuonuma.niigata.jp",
        "mitsuke.niigata.jp",
        "muika.niigata.jp",
        "murakami.niigata.jp",
        "myoko.niigata.jp",
        "nagaoka.niigata.jp",
        "niigata.niigata.jp",
        "ojiya.niigata.jp",
        "omi.niigata.jp",
        "sado.niigata.jp",
        "sanjo.niigata.jp",
        "seiro.niigata.jp",
        "seirou.niigata.jp",
        "sekikawa.niigata.jp",
        "shibata.niigata.jp",
        "tagami.niigata.jp",
        "tainai.niigata.jp",
        "tochio.niigata.jp",
        "tokamachi.niigata.jp",
        "tsubame.niigata.jp",
        "tsunan.niigata.jp",
        "uonuma.niigata.jp",
        "yahiko.niigata.jp",
        "yoita.niigata.jp",
        "yuzawa.niigata.jp",
        "beppu.oita.jp",
        "bungoono.oita.jp",
        "bungotakada.oita.jp",
        "hasama.oita.jp",
        "hiji.oita.jp",
        "himeshima.oita.jp",
        "hita.oita.jp",
        "kamitsue.oita.jp",
        "kokonoe.oita.jp",
        "kuju.oita.jp",
        "kunisaki.oita.jp",
        "kusu.oita.jp",
        "oita.oita.jp",
        "saiki.oita.jp",
        "taketa.oita.jp",
        "tsukumi.oita.jp",
        "usa.oita.jp",
        "usuki.oita.jp",
        "yufu.oita.jp",
        "akaiwa.okayama.jp",
        "asakuchi.okayama.jp",
        "bizen.okayama.jp",
        "hayashima.okayama.jp",
        "ibara.okayama.jp",
        "kagamino.okayama.jp",
        "kasaoka.okayama.jp",
        "kibichuo.okayama.jp",
        "kumenan.okayama.jp",
        "kurashiki.okayama.jp",
        "maniwa.okayama.jp",
        "misaki.okayama.jp",
        "nagi.okayama.jp",
        "niimi.okayama.jp",
        "nishiawakura.okayama.jp",
        "okayama.okayama.jp",
        "satosho.okayama.jp",
        "setouchi.okayama.jp",
        "shinjo.okayama.jp",
        "shoo.okayama.jp",
        "soja.okayama.jp",
        "takahashi.okayama.jp",
        "tamano.okayama.jp",
        "tsuyama.okayama.jp",
        "wake.okayama.jp",
        "yakage.okayama.jp",
        "aguni.okinawa.jp",
        "ginowan.okinawa.jp",
        "ginoza.okinawa.jp",
        "gushikami.okinawa.jp",
        "haebaru.okinawa.jp",
        "higashi.okinawa.jp",
        "hirara.okinawa.jp",
        "iheya.okinawa.jp",
        "ishigaki.okinawa.jp",
        "ishikawa.okinawa.jp",
        "itoman.okinawa.jp",
        "izena.okinawa.jp",
        "kadena.okinawa.jp",
        "kin.okinawa.jp",
        "kitadaito.okinawa.jp",
        "kitanakagusuku.okinawa.jp",
        "kumejima.okinawa.jp",
        "kunigami.okinawa.jp",
        "minamidaito.okinawa.jp",
        "motobu.okinawa.jp",
        "nago.okinawa.jp",
        "naha.okinawa.jp",
        "nakagusuku.okinawa.jp",
        "nakijin.okinawa.jp",
        "nanjo.okinawa.jp",
        "nishihara.okinawa.jp",
        "ogimi.okinawa.jp",
        "okinawa.okinawa.jp",
        "onna.okinawa.jp",
        "shimoji.okinawa.jp",
        "taketomi.okinawa.jp",
        "tarama.okinawa.jp",
        "tokashiki.okinawa.jp",
        "tomigusuku.okinawa.jp",
        "tonaki.okinawa.jp",
        "urasoe.okinawa.jp",
        "uruma.okinawa.jp",
        "yaese.okinawa.jp",
        "yomitan.okinawa.jp",
        "yonabaru.okinawa.jp",
        "yonaguni.okinawa.jp",
        "zamami.okinawa.jp",
        "abeno.osaka.jp",
        "chihayaakasaka.osaka.jp",
        "chuo.osaka.jp",
        "daito.osaka.jp",
        "fujiidera.osaka.jp",
        "habikino.osaka.jp",
        "hannan.osaka.jp",
        "higashiosaka.osaka.jp",
        "higashisumiyoshi.osaka.jp",
        "higashiyodogawa.osaka.jp",
        "hirakata.osaka.jp",
        "ibaraki.osaka.jp",
        "ikeda.osaka.jp",
        "izumi.osaka.jp",
        "izumiotsu.osaka.jp",
        "izumisano.osaka.jp",
        "kadoma.osaka.jp",
        "kaizuka.osaka.jp",
        "kanan.osaka.jp",
        "kashiwara.osaka.jp",
        "katano.osaka.jp",
        "kawachinagano.osaka.jp",
        "kishiwada.osaka.jp",
        "kita.osaka.jp",
        "kumatori.osaka.jp",
        "matsubara.osaka.jp",
        "minato.osaka.jp",
        "minoh.osaka.jp",
        "misaki.osaka.jp",
        "moriguchi.osaka.jp",
        "neyagawa.osaka.jp",
        "nishi.osaka.jp",
        "nose.osaka.jp",
        "osakasayama.osaka.jp",
        "sakai.osaka.jp",
        "sayama.osaka.jp",
        "sennan.osaka.jp",
        "settsu.osaka.jp",
        "shijonawate.osaka.jp",
        "shimamoto.osaka.jp",
        "suita.osaka.jp",
        "tadaoka.osaka.jp",
        "taishi.osaka.jp",
        "tajiri.osaka.jp",
        "takaishi.osaka.jp",
        "takatsuki.osaka.jp",
        "tondabayashi.osaka.jp",
        "toyonaka.osaka.jp",
        "toyono.osaka.jp",
        "yao.osaka.jp",
        "ariake.saga.jp",
        "arita.saga.jp",
        "fukudomi.saga.jp",
        "genkai.saga.jp",
        "hamatama.saga.jp",
        "hizen.saga.jp",
        "imari.saga.jp",
        "kamimine.saga.jp",
        "kanzaki.saga.jp",
        "karatsu.saga.jp",
        "kashima.saga.jp",
        "kitagata.saga.jp",
        "kitahata.saga.jp",
        "kiyama.saga.jp",
        "kouhoku.saga.jp",
        "kyuragi.saga.jp",
        "nishiarita.saga.jp",
        "ogi.saga.jp",
        "omachi.saga.jp",
        "ouchi.saga.jp",
        "saga.saga.jp",
        "shiroishi.saga.jp",
        "taku.saga.jp",
        "tara.saga.jp",
        "tosu.saga.jp",
        "yoshinogari.saga.jp",
        "arakawa.saitama.jp",
        "asaka.saitama.jp",
        "chichibu.saitama.jp",
        "fujimi.saitama.jp",
        "fujimino.saitama.jp",
        "fukaya.saitama.jp",
        "hanno.saitama.jp",
        "hanyu.saitama.jp",
        "hasuda.saitama.jp",
        "hatogaya.saitama.jp",
        "hatoyama.saitama.jp",
        "hidaka.saitama.jp",
        "higashichichibu.saitama.jp",
        "higashimatsuyama.saitama.jp",
        "honjo.saitama.jp",
        "ina.saitama.jp",
        "iruma.saitama.jp",
        "iwatsuki.saitama.jp",
        "kamiizumi.saitama.jp",
        "kamikawa.saitama.jp",
        "kamisato.saitama.jp",
        "kasukabe.saitama.jp",
        "kawagoe.saitama.jp",
        "kawaguchi.saitama.jp",
        "kawajima.saitama.jp",
        "kazo.saitama.jp",
        "kitamoto.saitama.jp",
        "koshigaya.saitama.jp",
        "kounosu.saitama.jp",
        "kuki.saitama.jp",
        "kumagaya.saitama.jp",
        "matsubushi.saitama.jp",
        "minano.saitama.jp",
        "misato.saitama.jp",
        "miyashiro.saitama.jp",
        "miyoshi.saitama.jp",
        "moroyama.saitama.jp",
        "nagatoro.saitama.jp",
        "namegawa.saitama.jp",
        "niiza.saitama.jp",
        "ogano.saitama.jp",
        "ogawa.saitama.jp",
        "ogose.saitama.jp",
        "okegawa.saitama.jp",
        "omiya.saitama.jp",
        "otaki.saitama.jp",
        "ranzan.saitama.jp",
        "ryokami.saitama.jp",
        "saitama.saitama.jp",
        "sakado.saitama.jp",
        "satte.saitama.jp",
        "sayama.saitama.jp",
        "shiki.saitama.jp",
        "shiraoka.saitama.jp",
        "soka.saitama.jp",
        "sugito.saitama.jp",
        "toda.saitama.jp",
        "tokigawa.saitama.jp",
        "tokorozawa.saitama.jp",
        "tsurugashima.saitama.jp",
        "urawa.saitama.jp",
        "warabi.saitama.jp",
        "yashio.saitama.jp",
        "yokoze.saitama.jp",
        "yono.saitama.jp",
        "yorii.saitama.jp",
        "yoshida.saitama.jp",
        "yoshikawa.saitama.jp",
        "yoshimi.saitama.jp",
        "aisho.shiga.jp",
        "gamo.shiga.jp",
        "higashiomi.shiga.jp",
        "hikone.shiga.jp",
        "koka.shiga.jp",
        "konan.shiga.jp",
        "kosei.shiga.jp",
        "koto.shiga.jp",
        "kusatsu.shiga.jp",
        "maibara.shiga.jp",
        "moriyama.shiga.jp",
        "nagahama.shiga.jp",
        "nishiazai.shiga.jp",
        "notogawa.shiga.jp",
        "omihachiman.shiga.jp",
        "otsu.shiga.jp",
        "ritto.shiga.jp",
        "ryuoh.shiga.jp",
        "takashima.shiga.jp",
        "takatsuki.shiga.jp",
        "torahime.shiga.jp",
        "toyosato.shiga.jp",
        "yasu.shiga.jp",
        "akagi.shimane.jp",
        "ama.shimane.jp",
        "gotsu.shimane.jp",
        "hamada.shimane.jp",
        "higashiizumo.shimane.jp",
        "hikawa.shimane.jp",
        "hikimi.shimane.jp",
        "izumo.shimane.jp",
        "kakinoki.shimane.jp",
        "masuda.shimane.jp",
        "matsue.shimane.jp",
        "misato.shimane.jp",
        "nishinoshima.shimane.jp",
        "ohda.shimane.jp",
        "okinoshima.shimane.jp",
        "okuizumo.shimane.jp",
        "shimane.shimane.jp",
        "tamayu.shimane.jp",
        "tsuwano.shimane.jp",
        "unnan.shimane.jp",
        "yakumo.shimane.jp",
        "yasugi.shimane.jp",
        "yatsuka.shimane.jp",
        "arai.shizuoka.jp",
        "atami.shizuoka.jp",
        "fuji.shizuoka.jp",
        "fujieda.shizuoka.jp",
        "fujikawa.shizuoka.jp",
        "fujinomiya.shizuoka.jp",
        "fukuroi.shizuoka.jp",
        "gotemba.shizuoka.jp",
        "haibara.shizuoka.jp",
        "hamamatsu.shizuoka.jp",
        "higashiizu.shizuoka.jp",
        "ito.shizuoka.jp",
        "iwata.shizuoka.jp",
        "izu.shizuoka.jp",
        "izunokuni.shizuoka.jp",
        "kakegawa.shizuoka.jp",
        "kannami.shizuoka.jp",
        "kawanehon.shizuoka.jp",
        "kawazu.shizuoka.jp",
        "kikugawa.shizuoka.jp",
        "kosai.shizuoka.jp",
        "makinohara.shizuoka.jp",
        "matsuzaki.shizuoka.jp",
        "minamiizu.shizuoka.jp",
        "mishima.shizuoka.jp",
        "morimachi.shizuoka.jp",
        "nishiizu.shizuoka.jp",
        "numazu.shizuoka.jp",
        "omaezaki.shizuoka.jp",
        "shimada.shizuoka.jp",
        "shimizu.shizuoka.jp",
        "shimoda.shizuoka.jp",
        "shizuoka.shizuoka.jp",
        "susono.shizuoka.jp",
        "yaizu.shizuoka.jp",
        "yoshida.shizuoka.jp",
        "ashikaga.tochigi.jp",
        "bato.tochigi.jp",
        "haga.tochigi.jp",
        "ichikai.tochigi.jp",
        "iwafune.tochigi.jp",
        "kaminokawa.tochigi.jp",
        "kanuma.tochigi.jp",
        "karasuyama.tochigi.jp",
        "kuroiso.tochigi.jp",
        "mashiko.tochigi.jp",
        "mibu.tochigi.jp",
        "moka.tochigi.jp",
        "motegi.tochigi.jp",
        "nasu.tochigi.jp",
        "nasushiobara.tochigi.jp",
        "nikko.tochigi.jp",
        "nishikata.tochigi.jp",
        "nogi.tochigi.jp",
        "ohira.tochigi.jp",
        "ohtawara.tochigi.jp",
        "oyama.tochigi.jp",
        "sakura.tochigi.jp",
        "sano.tochigi.jp",
        "shimotsuke.tochigi.jp",
        "shioya.tochigi.jp",
        "takanezawa.tochigi.jp",
        "tochigi.tochigi.jp",
        "tsuga.tochigi.jp",
        "ujiie.tochigi.jp",
        "utsunomiya.tochigi.jp",
        "yaita.tochigi.jp",
        "aizumi.tokushima.jp",
        "anan.tokushima.jp",
        "ichiba.tokushima.jp",
        "itano.tokushima.jp",
        "kainan.tokushima.jp",
        "komatsushima.tokushima.jp",
        "matsushige.tokushima.jp",
        "mima.tokushima.jp",
        "minami.tokushima.jp",
        "miyoshi.tokushima.jp",
        "mugi.tokushima.jp",
        "nakagawa.tokushima.jp",
        "naruto.tokushima.jp",
        "sanagochi.tokushima.jp",
        "shishikui.tokushima.jp",
        "tokushima.tokushima.jp",
        "wajiki.tokushima.jp",
        "adachi.tokyo.jp",
        "akiruno.tokyo.jp",
        "akishima.tokyo.jp",
        "aogashima.tokyo.jp",
        "arakawa.tokyo.jp",
        "bunkyo.tokyo.jp",
        "chiyoda.tokyo.jp",
        "chofu.tokyo.jp",
        "chuo.tokyo.jp",
        "edogawa.tokyo.jp",
        "fuchu.tokyo.jp",
        "fussa.tokyo.jp",
        "hachijo.tokyo.jp",
        "hachioji.tokyo.jp",
        "hamura.tokyo.jp",
        "higashikurume.tokyo.jp",
        "higashimurayama.tokyo.jp",
        "higashiyamato.tokyo.jp",
        "hino.tokyo.jp",
        "hinode.tokyo.jp",
        "hinohara.tokyo.jp",
        "inagi.tokyo.jp",
        "itabashi.tokyo.jp",
        "katsushika.tokyo.jp",
        "kita.tokyo.jp",
        "kiyose.tokyo.jp",
        "kodaira.tokyo.jp",
        "koganei.tokyo.jp",
        "kokubunji.tokyo.jp",
        "komae.tokyo.jp",
        "koto.tokyo.jp",
        "kouzushima.tokyo.jp",
        "kunitachi.tokyo.jp",
        "machida.tokyo.jp",
        "meguro.tokyo.jp",
        "minato.tokyo.jp",
        "mitaka.tokyo.jp",
        "mizuho.tokyo.jp",
        "musashimurayama.tokyo.jp",
        "musashino.tokyo.jp",
        "nakano.tokyo.jp",
        "nerima.tokyo.jp",
        "ogasawara.tokyo.jp",
        "okutama.tokyo.jp",
        "ome.tokyo.jp",
        "oshima.tokyo.jp",
        "ota.tokyo.jp",
        "setagaya.tokyo.jp",
        "shibuya.tokyo.jp",
        "shinagawa.tokyo.jp",
        "shinjuku.tokyo.jp",
        "suginami.tokyo.jp",
        "sumida.tokyo.jp",
        "tachikawa.tokyo.jp",
        "taito.tokyo.jp",
        "tama.tokyo.jp",
        "toshima.tokyo.jp",
        "chizu.tottori.jp",
        "hino.tottori.jp",
        "kawahara.tottori.jp",
        "koge.tottori.jp",
        "kotoura.tottori.jp",
        "misasa.tottori.jp",
        "nanbu.tottori.jp",
        "nichinan.tottori.jp",
        "sakaiminato.tottori.jp",
        "tottori.tottori.jp",
        "wakasa.tottori.jp",
        "yazu.tottori.jp",
        "yonago.tottori.jp",
        "asahi.toyama.jp",
        "fuchu.toyama.jp",
        "fukumitsu.toyama.jp",
        "funahashi.toyama.jp",
        "himi.toyama.jp",
        "imizu.toyama.jp",
        "inami.toyama.jp",
        "johana.toyama.jp",
        "kamiichi.toyama.jp",
        "kurobe.toyama.jp",
        "nakaniikawa.toyama.jp",
        "namerikawa.toyama.jp",
        "nanto.toyama.jp",
        "nyuzen.toyama.jp",
        "oyabe.toyama.jp",
        "taira.toyama.jp",
        "takaoka.toyama.jp",
        "tateyama.toyama.jp",
        "toga.toyama.jp",
        "tonami.toyama.jp",
        "toyama.toyama.jp",
        "unazuki.toyama.jp",
        "uozu.toyama.jp",
        "yamada.toyama.jp",
        "arida.wakayama.jp",
        "aridagawa.wakayama.jp",
        "gobo.wakayama.jp",
        "hashimoto.wakayama.jp",
        "hidaka.wakayama.jp",
        "hirogawa.wakayama.jp",
        "inami.wakayama.jp",
        "iwade.wakayama.jp",
        "kainan.wakayama.jp",
        "kamitonda.wakayama.jp",
        "katsuragi.wakayama.jp",
        "kimino.wakayama.jp",
        "kinokawa.wakayama.jp",
        "kitayama.wakayama.jp",
        "koya.wakayama.jp",
        "koza.wakayama.jp",
        "kozagawa.wakayama.jp",
        "kudoyama.wakayama.jp",
        "kushimoto.wakayama.jp",
        "mihama.wakayama.jp",
        "misato.wakayama.jp",
        "nachikatsuura.wakayama.jp",
        "shingu.wakayama.jp",
        "shirahama.wakayama.jp",
        "taiji.wakayama.jp",
        "tanabe.wakayama.jp",
        "wakayama.wakayama.jp",
        "yuasa.wakayama.jp",
        "yura.wakayama.jp",
        "asahi.yamagata.jp",
        "funagata.yamagata.jp",
        "higashine.yamagata.jp",
        "iide.yamagata.jp",
        "kahoku.yamagata.jp",
        "kaminoyama.yamagata.jp",
        "kaneyama.yamagata.jp",
        "kawanishi.yamagata.jp",
        "mamurogawa.yamagata.jp",
        "mikawa.yamagata.jp",
        "murayama.yamagata.jp",
        "nagai.yamagata.jp",
        "nakayama.yamagata.jp",
        "nanyo.yamagata.jp",
        "nishikawa.yamagata.jp",
        "obanazawa.yamagata.jp",
        "oe.yamagata.jp",
        "oguni.yamagata.jp",
        "ohkura.yamagata.jp",
        "oishida.yamagata.jp",
        "sagae.yamagata.jp",
        "sakata.yamagata.jp",
        "sakegawa.yamagata.jp",
        "shinjo.yamagata.jp",
        "shirataka.yamagata.jp",
        "shonai.yamagata.jp",
        "takahata.yamagata.jp",
        "tendo.yamagata.jp",
        "tozawa.yamagata.jp",
        "tsuruoka.yamagata.jp",
        "yamagata.yamagata.jp",
        "yamanobe.yamagata.jp",
        "yonezawa.yamagata.jp",
        "yuza.yamagata.jp",
        "abu.yamaguchi.jp",
        "hagi.yamaguchi.jp",
        "hikari.yamaguchi.jp",
        "hofu.yamaguchi.jp",
        "iwakuni.yamaguchi.jp",
        "kudamatsu.yamaguchi.jp",
        "mitou.yamaguchi.jp",
        "nagato.yamaguchi.jp",
        "oshima.yamaguchi.jp",
        "shimonoseki.yamaguchi.jp",
        "shunan.yamaguchi.jp",
        "tabuse.yamaguchi.jp",
        "tokuyama.yamaguchi.jp",
        "toyota.yamaguchi.jp",
        "ube.yamaguchi.jp",
        "yuu.yamaguchi.jp",
        "chuo.yamanashi.jp",
        "doshi.yamanashi.jp",
        "fuefuki.yamanashi.jp",
        "fujikawa.yamanashi.jp",
        "fujikawaguchiko.yamanashi.jp",
        "fujiyoshida.yamanashi.jp",
        "hayakawa.yamanashi.jp",
        "hokuto.yamanashi.jp",
        "ichikawamisato.yamanashi.jp",
        "kai.yamanashi.jp",
        "kofu.yamanashi.jp",
        "koshu.yamanashi.jp",
        "kosuge.yamanashi.jp",
        "minami-alps.yamanashi.jp",
        "minobu.yamanashi.jp",
        "nakamichi.yamanashi.jp",
        "nanbu.yamanashi.jp",
        "narusawa.yamanashi.jp",
        "nirasaki.yamanashi.jp",
        "nishikatsura.yamanashi.jp",
        "oshino.yamanashi.jp",
        "otsuki.yamanashi.jp",
        "showa.yamanashi.jp",
        "tabayama.yamanashi.jp",
        "tsuru.yamanashi.jp",
        "uenohara.yamanashi.jp",
        "yamanakako.yamanashi.jp",
        "yamanashi.yamanashi.jp",
        "ke",
        "ac.ke",
        "co.ke",
        "go.ke",
        "info.ke",
        "me.ke",
        "mobi.ke",
        "ne.ke",
        "or.ke",
        "sc.ke",
        "kg",
        "org.kg",
        "net.kg",
        "com.kg",
        "edu.kg",
        "gov.kg",
        "mil.kg",
        "*.kh",
        "ki",
        "edu.ki",
        "biz.ki",
        "net.ki",
        "org.ki",
        "gov.ki",
        "info.ki",
        "com.ki",
        "km",
        "org.km",
        "nom.km",
        "gov.km",
        "prd.km",
        "tm.km",
        "edu.km",
        "mil.km",
        "ass.km",
        "com.km",
        "coop.km",
        "asso.km",
        "presse.km",
        "medecin.km",
        "notaires.km",
        "pharmaciens.km",
        "veterinaire.km",
        "gouv.km",
        "kn",
        "net.kn",
        "org.kn",
        "edu.kn",
        "gov.kn",
        "kp",
        "com.kp",
        "edu.kp",
        "gov.kp",
        "org.kp",
        "rep.kp",
        "tra.kp",
        "kr",
        "ac.kr",
        "co.kr",
        "es.kr",
        "go.kr",
        "hs.kr",
        "kg.kr",
        "mil.kr",
        "ms.kr",
        "ne.kr",
        "or.kr",
        "pe.kr",
        "re.kr",
        "sc.kr",
        "busan.kr",
        "chungbuk.kr",
        "chungnam.kr",
        "daegu.kr",
        "daejeon.kr",
        "gangwon.kr",
        "gwangju.kr",
        "gyeongbuk.kr",
        "gyeonggi.kr",
        "gyeongnam.kr",
        "incheon.kr",
        "jeju.kr",
        "jeonbuk.kr",
        "jeonnam.kr",
        "seoul.kr",
        "ulsan.kr",
        "kw",
        "com.kw",
        "edu.kw",
        "emb.kw",
        "gov.kw",
        "ind.kw",
        "net.kw",
        "org.kw",
        "ky",
        "edu.ky",
        "gov.ky",
        "com.ky",
        "org.ky",
        "net.ky",
        "kz",
        "org.kz",
        "edu.kz",
        "net.kz",
        "gov.kz",
        "mil.kz",
        "com.kz",
        "la",
        "int.la",
        "net.la",
        "info.la",
        "edu.la",
        "gov.la",
        "per.la",
        "com.la",
        "org.la",
        "lb",
        "com.lb",
        "edu.lb",
        "gov.lb",
        "net.lb",
        "org.lb",
        "lc",
        "com.lc",
        "net.lc",
        "co.lc",
        "org.lc",
        "edu.lc",
        "gov.lc",
        "li",
        "lk",
        "gov.lk",
        "sch.lk",
        "net.lk",
        "int.lk",
        "com.lk",
        "org.lk",
        "edu.lk",
        "ngo.lk",
        "soc.lk",
        "web.lk",
        "ltd.lk",
        "assn.lk",
        "grp.lk",
        "hotel.lk",
        "ac.lk",
        "lr",
        "com.lr",
        "edu.lr",
        "gov.lr",
        "org.lr",
        "net.lr",
        "ls",
        "ac.ls",
        "biz.ls",
        "co.ls",
        "edu.ls",
        "gov.ls",
        "info.ls",
        "net.ls",
        "org.ls",
        "sc.ls",
        "lt",
        "gov.lt",
        "lu",
        "lv",
        "com.lv",
        "edu.lv",
        "gov.lv",
        "org.lv",
        "mil.lv",
        "id.lv",
        "net.lv",
        "asn.lv",
        "conf.lv",
        "ly",
        "com.ly",
        "net.ly",
        "gov.ly",
        "plc.ly",
        "edu.ly",
        "sch.ly",
        "med.ly",
        "org.ly",
        "id.ly",
        "ma",
        "co.ma",
        "net.ma",
        "gov.ma",
        "org.ma",
        "ac.ma",
        "press.ma",
        "mc",
        "tm.mc",
        "asso.mc",
        "md",
        "me",
        "co.me",
        "net.me",
        "org.me",
        "edu.me",
        "ac.me",
        "gov.me",
        "its.me",
        "priv.me",
        "mg",
        "org.mg",
        "nom.mg",
        "gov.mg",
        "prd.mg",
        "tm.mg",
        "edu.mg",
        "mil.mg",
        "com.mg",
        "co.mg",
        "mh",
        "mil",
        "mk",
        "com.mk",
        "org.mk",
        "net.mk",
        "edu.mk",
        "gov.mk",
        "inf.mk",
        "name.mk",
        "ml",
        "com.ml",
        "edu.ml",
        "gouv.ml",
        "gov.ml",
        "net.ml",
        "org.ml",
        "presse.ml",
        "*.mm",
        "mn",
        "gov.mn",
        "edu.mn",
        "org.mn",
        "mo",
        "com.mo",
        "net.mo",
        "org.mo",
        "edu.mo",
        "gov.mo",
        "mobi",
        "mp",
        "mq",
        "mr",
        "gov.mr",
        "ms",
        "com.ms",
        "edu.ms",
        "gov.ms",
        "net.ms",
        "org.ms",
        "mt",
        "com.mt",
        "edu.mt",
        "net.mt",
        "org.mt",
        "mu",
        "com.mu",
        "net.mu",
        "org.mu",
        "gov.mu",
        "ac.mu",
        "co.mu",
        "or.mu",
        "museum",
        "academy.museum",
        "agriculture.museum",
        "air.museum",
        "airguard.museum",
        "alabama.museum",
        "alaska.museum",
        "amber.museum",
        "ambulance.museum",
        "american.museum",
        "americana.museum",
        "americanantiques.museum",
        "americanart.museum",
        "amsterdam.museum",
        "and.museum",
        "annefrank.museum",
        "anthro.museum",
        "anthropology.museum",
        "antiques.museum",
        "aquarium.museum",
        "arboretum.museum",
        "archaeological.museum",
        "archaeology.museum",
        "architecture.museum",
        "art.museum",
        "artanddesign.museum",
        "artcenter.museum",
        "artdeco.museum",
        "arteducation.museum",
        "artgallery.museum",
        "arts.museum",
        "artsandcrafts.museum",
        "asmatart.museum",
        "assassination.museum",
        "assisi.museum",
        "association.museum",
        "astronomy.museum",
        "atlanta.museum",
        "austin.museum",
        "australia.museum",
        "automotive.museum",
        "aviation.museum",
        "axis.museum",
        "badajoz.museum",
        "baghdad.museum",
        "bahn.museum",
        "bale.museum",
        "baltimore.museum",
        "barcelona.museum",
        "baseball.museum",
        "basel.museum",
        "baths.museum",
        "bauern.museum",
        "beauxarts.museum",
        "beeldengeluid.museum",
        "bellevue.museum",
        "bergbau.museum",
        "berkeley.museum",
        "berlin.museum",
        "bern.museum",
        "bible.museum",
        "bilbao.museum",
        "bill.museum",
        "birdart.museum",
        "birthplace.museum",
        "bonn.museum",
        "boston.museum",
        "botanical.museum",
        "botanicalgarden.museum",
        "botanicgarden.museum",
        "botany.museum",
        "brandywinevalley.museum",
        "brasil.museum",
        "bristol.museum",
        "british.museum",
        "britishcolumbia.museum",
        "broadcast.museum",
        "brunel.museum",
        "brussel.museum",
        "brussels.museum",
        "bruxelles.museum",
        "building.museum",
        "burghof.museum",
        "bus.museum",
        "bushey.museum",
        "cadaques.museum",
        "california.museum",
        "cambridge.museum",
        "can.museum",
        "canada.museum",
        "capebreton.museum",
        "carrier.museum",
        "cartoonart.museum",
        "casadelamoneda.museum",
        "castle.museum",
        "castres.museum",
        "celtic.museum",
        "center.museum",
        "chattanooga.museum",
        "cheltenham.museum",
        "chesapeakebay.museum",
        "chicago.museum",
        "children.museum",
        "childrens.museum",
        "childrensgarden.museum",
        "chiropractic.museum",
        "chocolate.museum",
        "christiansburg.museum",
        "cincinnati.museum",
        "cinema.museum",
        "circus.museum",
        "civilisation.museum",
        "civilization.museum",
        "civilwar.museum",
        "clinton.museum",
        "clock.museum",
        "coal.museum",
        "coastaldefence.museum",
        "cody.museum",
        "coldwar.museum",
        "collection.museum",
        "colonialwilliamsburg.museum",
        "coloradoplateau.museum",
        "columbia.museum",
        "columbus.museum",
        "communication.museum",
        "communications.museum",
        "community.museum",
        "computer.museum",
        "computerhistory.museum",
        "comunicações.museum",
        "contemporary.museum",
        "contemporaryart.museum",
        "convent.museum",
        "copenhagen.museum",
        "corporation.museum",
        "correios-e-telecomunicações.museum",
        "corvette.museum",
        "costume.museum",
        "countryestate.museum",
        "county.museum",
        "crafts.museum",
        "cranbrook.museum",
        "creation.museum",
        "cultural.museum",
        "culturalcenter.museum",
        "culture.museum",
        "cyber.museum",
        "cymru.museum",
        "dali.museum",
        "dallas.museum",
        "database.museum",
        "ddr.museum",
        "decorativearts.museum",
        "delaware.museum",
        "delmenhorst.museum",
        "denmark.museum",
        "depot.museum",
        "design.museum",
        "detroit.museum",
        "dinosaur.museum",
        "discovery.museum",
        "dolls.museum",
        "donostia.museum",
        "durham.museum",
        "eastafrica.museum",
        "eastcoast.museum",
        "education.museum",
        "educational.museum",
        "egyptian.museum",
        "eisenbahn.museum",
        "elburg.museum",
        "elvendrell.museum",
        "embroidery.museum",
        "encyclopedic.museum",
        "england.museum",
        "entomology.museum",
        "environment.museum",
        "environmentalconservation.museum",
        "epilepsy.museum",
        "essex.museum",
        "estate.museum",
        "ethnology.museum",
        "exeter.museum",
        "exhibition.museum",
        "family.museum",
        "farm.museum",
        "farmequipment.museum",
        "farmers.museum",
        "farmstead.museum",
        "field.museum",
        "figueres.museum",
        "filatelia.museum",
        "film.museum",
        "fineart.museum",
        "finearts.museum",
        "finland.museum",
        "flanders.museum",
        "florida.museum",
        "force.museum",
        "fortmissoula.museum",
        "fortworth.museum",
        "foundation.museum",
        "francaise.museum",
        "frankfurt.museum",
        "franziskaner.museum",
        "freemasonry.museum",
        "freiburg.museum",
        "fribourg.museum",
        "frog.museum",
        "fundacio.museum",
        "furniture.museum",
        "gallery.museum",
        "garden.museum",
        "gateway.museum",
        "geelvinck.museum",
        "gemological.museum",
        "geology.museum",
        "georgia.museum",
        "giessen.museum",
        "glas.museum",
        "glass.museum",
        "gorge.museum",
        "grandrapids.museum",
        "graz.museum",
        "guernsey.museum",
        "halloffame.museum",
        "hamburg.museum",
        "handson.museum",
        "harvestcelebration.museum",
        "hawaii.museum",
        "health.museum",
        "heimatunduhren.museum",
        "hellas.museum",
        "helsinki.museum",
        "hembygdsforbund.museum",
        "heritage.museum",
        "histoire.museum",
        "historical.museum",
        "historicalsociety.museum",
        "historichouses.museum",
        "historisch.museum",
        "historisches.museum",
        "history.museum",
        "historyofscience.museum",
        "horology.museum",
        "house.museum",
        "humanities.museum",
        "illustration.museum",
        "imageandsound.museum",
        "indian.museum",
        "indiana.museum",
        "indianapolis.museum",
        "indianmarket.museum",
        "intelligence.museum",
        "interactive.museum",
        "iraq.museum",
        "iron.museum",
        "isleofman.museum",
        "jamison.museum",
        "jefferson.museum",
        "jerusalem.museum",
        "jewelry.museum",
        "jewish.museum",
        "jewishart.museum",
        "jfk.museum",
        "journalism.museum",
        "judaica.museum",
        "judygarland.museum",
        "juedisches.museum",
        "juif.museum",
        "karate.museum",
        "karikatur.museum",
        "kids.museum",
        "koebenhavn.museum",
        "koeln.museum",
        "kunst.museum",
        "kunstsammlung.museum",
        "kunstunddesign.museum",
        "labor.museum",
        "labour.museum",
        "lajolla.museum",
        "lancashire.museum",
        "landes.museum",
        "lans.museum",
        "läns.museum",
        "larsson.museum",
        "lewismiller.museum",
        "lincoln.museum",
        "linz.museum",
        "living.museum",
        "livinghistory.museum",
        "localhistory.museum",
        "london.museum",
        "losangeles.museum",
        "louvre.museum",
        "loyalist.museum",
        "lucerne.museum",
        "luxembourg.museum",
        "luzern.museum",
        "mad.museum",
        "madrid.museum",
        "mallorca.museum",
        "manchester.museum",
        "mansion.museum",
        "mansions.museum",
        "manx.museum",
        "marburg.museum",
        "maritime.museum",
        "maritimo.museum",
        "maryland.museum",
        "marylhurst.museum",
        "media.museum",
        "medical.museum",
        "medizinhistorisches.museum",
        "meeres.museum",
        "memorial.museum",
        "mesaverde.museum",
        "michigan.museum",
        "midatlantic.museum",
        "military.museum",
        "mill.museum",
        "miners.museum",
        "mining.museum",
        "minnesota.museum",
        "missile.museum",
        "missoula.museum",
        "modern.museum",
        "moma.museum",
        "money.museum",
        "monmouth.museum",
        "monticello.museum",
        "montreal.museum",
        "moscow.museum",
        "motorcycle.museum",
        "muenchen.museum",
        "muenster.museum",
        "mulhouse.museum",
        "muncie.museum",
        "museet.museum",
        "museumcenter.museum",
        "museumvereniging.museum",
        "music.museum",
        "national.museum",
        "nationalfirearms.museum",
        "nationalheritage.museum",
        "nativeamerican.museum",
        "naturalhistory.museum",
        "naturalhistorymuseum.museum",
        "naturalsciences.museum",
        "nature.museum",
        "naturhistorisches.museum",
        "natuurwetenschappen.museum",
        "naumburg.museum",
        "naval.museum",
        "nebraska.museum",
        "neues.museum",
        "newhampshire.museum",
        "newjersey.museum",
        "newmexico.museum",
        "newport.museum",
        "newspaper.museum",
        "newyork.museum",
        "niepce.museum",
        "norfolk.museum",
        "north.museum",
        "nrw.museum",
        "nyc.museum",
        "nyny.museum",
        "oceanographic.museum",
        "oceanographique.museum",
        "omaha.museum",
        "online.museum",
        "ontario.museum",
        "openair.museum",
        "oregon.museum",
        "oregontrail.museum",
        "otago.museum",
        "oxford.museum",
        "pacific.museum",
        "paderborn.museum",
        "palace.museum",
        "paleo.museum",
        "palmsprings.museum",
        "panama.museum",
        "paris.museum",
        "pasadena.museum",
        "pharmacy.museum",
        "philadelphia.museum",
        "philadelphiaarea.museum",
        "philately.museum",
        "phoenix.museum",
        "photography.museum",
        "pilots.museum",
        "pittsburgh.museum",
        "planetarium.museum",
        "plantation.museum",
        "plants.museum",
        "plaza.museum",
        "portal.museum",
        "portland.museum",
        "portlligat.museum",
        "posts-and-telecommunications.museum",
        "preservation.museum",
        "presidio.museum",
        "press.museum",
        "project.museum",
        "public.museum",
        "pubol.museum",
        "quebec.museum",
        "railroad.museum",
        "railway.museum",
        "research.museum",
        "resistance.museum",
        "riodejaneiro.museum",
        "rochester.museum",
        "rockart.museum",
        "roma.museum",
        "russia.museum",
        "saintlouis.museum",
        "salem.museum",
        "salvadordali.museum",
        "salzburg.museum",
        "sandiego.museum",
        "sanfrancisco.museum",
        "santabarbara.museum",
        "santacruz.museum",
        "santafe.museum",
        "saskatchewan.museum",
        "satx.museum",
        "savannahga.museum",
        "schlesisches.museum",
        "schoenbrunn.museum",
        "schokoladen.museum",
        "school.museum",
        "schweiz.museum",
        "science.museum",
        "scienceandhistory.museum",
        "scienceandindustry.museum",
        "sciencecenter.museum",
        "sciencecenters.museum",
        "science-fiction.museum",
        "sciencehistory.museum",
        "sciences.museum",
        "sciencesnaturelles.museum",
        "scotland.museum",
        "seaport.museum",
        "settlement.museum",
        "settlers.museum",
        "shell.museum",
        "sherbrooke.museum",
        "sibenik.museum",
        "silk.museum",
        "ski.museum",
        "skole.museum",
        "society.museum",
        "sologne.museum",
        "soundandvision.museum",
        "southcarolina.museum",
        "southwest.museum",
        "space.museum",
        "spy.museum",
        "square.museum",
        "stadt.museum",
        "stalbans.museum",
        "starnberg.museum",
        "state.museum",
        "stateofdelaware.museum",
        "station.museum",
        "steam.museum",
        "steiermark.museum",
        "stjohn.museum",
        "stockholm.museum",
        "stpetersburg.museum",
        "stuttgart.museum",
        "suisse.museum",
        "surgeonshall.museum",
        "surrey.museum",
        "svizzera.museum",
        "sweden.museum",
        "sydney.museum",
        "tank.museum",
        "tcm.museum",
        "technology.museum",
        "telekommunikation.museum",
        "television.museum",
        "texas.museum",
        "textile.museum",
        "theater.museum",
        "time.museum",
        "timekeeping.museum",
        "topology.museum",
        "torino.museum",
        "touch.museum",
        "town.museum",
        "transport.museum",
        "tree.museum",
        "trolley.museum",
        "trust.museum",
        "trustee.museum",
        "uhren.museum",
        "ulm.museum",
        "undersea.museum",
        "university.museum",
        "usa.museum",
        "usantiques.museum",
        "usarts.museum",
        "uscountryestate.museum",
        "usculture.museum",
        "usdecorativearts.museum",
        "usgarden.museum",
        "ushistory.museum",
        "ushuaia.museum",
        "uslivinghistory.museum",
        "utah.museum",
        "uvic.museum",
        "valley.museum",
        "vantaa.museum",
        "versailles.museum",
        "viking.museum",
        "village.museum",
        "virginia.museum",
        "virtual.museum",
        "virtuel.museum",
        "vlaanderen.museum",
        "volkenkunde.museum",
        "wales.museum",
        "wallonie.museum",
        "war.museum",
        "washingtondc.museum",
        "watchandclock.museum",
        "watch-and-clock.museum",
        "western.museum",
        "westfalen.museum",
        "whaling.museum",
        "wildlife.museum",
        "williamsburg.museum",
        "windmill.museum",
        "workshop.museum",
        "york.museum",
        "yorkshire.museum",
        "yosemite.museum",
        "youth.museum",
        "zoological.museum",
        "zoology.museum",
        "ירושלים.museum",
        "иком.museum",
        "mv",
        "aero.mv",
        "biz.mv",
        "com.mv",
        "coop.mv",
        "edu.mv",
        "gov.mv",
        "info.mv",
        "int.mv",
        "mil.mv",
        "museum.mv",
        "name.mv",
        "net.mv",
        "org.mv",
        "pro.mv",
        "mw",
        "ac.mw",
        "biz.mw",
        "co.mw",
        "com.mw",
        "coop.mw",
        "edu.mw",
        "gov.mw",
        "int.mw",
        "museum.mw",
        "net.mw",
        "org.mw",
        "mx",
        "com.mx",
        "org.mx",
        "gob.mx",
        "edu.mx",
        "net.mx",
        "my",
        "com.my",
        "net.my",
        "org.my",
        "gov.my",
        "edu.my",
        "mil.my",
        "name.my",
        "mz",
        "ac.mz",
        "adv.mz",
        "co.mz",
        "edu.mz",
        "gov.mz",
        "mil.mz",
        "net.mz",
        "org.mz",
        "na",
        "info.na",
        "pro.na",
        "name.na",
        "school.na",
        "or.na",
        "dr.na",
        "us.na",
        "mx.na",
        "ca.na",
        "in.na",
        "cc.na",
        "tv.na",
        "ws.na",
        "mobi.na",
        "co.na",
        "com.na",
        "org.na",
        "name",
        "nc",
        "asso.nc",
        "nom.nc",
        "ne",
        "net",
        "nf",
        "com.nf",
        "net.nf",
        "per.nf",
        "rec.nf",
        "web.nf",
        "arts.nf",
        "firm.nf",
        "info.nf",
        "other.nf",
        "store.nf",
        "ng",
        "com.ng",
        "edu.ng",
        "gov.ng",
        "i.ng",
        "mil.ng",
        "mobi.ng",
        "name.ng",
        "net.ng",
        "org.ng",
        "sch.ng",
        "ni",
        "ac.ni",
        "biz.ni",
        "co.ni",
        "com.ni",
        "edu.ni",
        "gob.ni",
        "in.ni",
        "info.ni",
        "int.ni",
        "mil.ni",
        "net.ni",
        "nom.ni",
        "org.ni",
        "web.ni",
        "nl",
        "no",
        "fhs.no",
        "vgs.no",
        "fylkesbibl.no",
        "folkebibl.no",
        "museum.no",
        "idrett.no",
        "priv.no",
        "mil.no",
        "stat.no",
        "dep.no",
        "kommune.no",
        "herad.no",
        "aa.no",
        "ah.no",
        "bu.no",
        "fm.no",
        "hl.no",
        "hm.no",
        "jan-mayen.no",
        "mr.no",
        "nl.no",
        "nt.no",
        "of.no",
        "ol.no",
        "oslo.no",
        "rl.no",
        "sf.no",
        "st.no",
        "svalbard.no",
        "tm.no",
        "tr.no",
        "va.no",
        "vf.no",
        "gs.aa.no",
        "gs.ah.no",
        "gs.bu.no",
        "gs.fm.no",
        "gs.hl.no",
        "gs.hm.no",
        "gs.jan-mayen.no",
        "gs.mr.no",
        "gs.nl.no",
        "gs.nt.no",
        "gs.of.no",
        "gs.ol.no",
        "gs.oslo.no",
        "gs.rl.no",
        "gs.sf.no",
        "gs.st.no",
        "gs.svalbard.no",
        "gs.tm.no",
        "gs.tr.no",
        "gs.va.no",
        "gs.vf.no",
        "akrehamn.no",
        "åkrehamn.no",
        "algard.no",
        "ålgård.no",
        "arna.no",
        "brumunddal.no",
        "bryne.no",
        "bronnoysund.no",
        "brønnøysund.no",
        "drobak.no",
        "drøbak.no",
        "egersund.no",
        "fetsund.no",
        "floro.no",
        "florø.no",
        "fredrikstad.no",
        "hokksund.no",
        "honefoss.no",
        "hønefoss.no",
        "jessheim.no",
        "jorpeland.no",
        "jørpeland.no",
        "kirkenes.no",
        "kopervik.no",
        "krokstadelva.no",
        "langevag.no",
        "langevåg.no",
        "leirvik.no",
        "mjondalen.no",
        "mjøndalen.no",
        "mo-i-rana.no",
        "mosjoen.no",
        "mosjøen.no",
        "nesoddtangen.no",
        "orkanger.no",
        "osoyro.no",
        "osøyro.no",
        "raholt.no",
        "råholt.no",
        "sandnessjoen.no",
        "sandnessjøen.no",
        "skedsmokorset.no",
        "slattum.no",
        "spjelkavik.no",
        "stathelle.no",
        "stavern.no",
        "stjordalshalsen.no",
        "stjørdalshalsen.no",
        "tananger.no",
        "tranby.no",
        "vossevangen.no",
        "afjord.no",
        "åfjord.no",
        "agdenes.no",
        "al.no",
        "ål.no",
        "alesund.no",
        "ålesund.no",
        "alstahaug.no",
        "alta.no",
        "áltá.no",
        "alaheadju.no",
        "álaheadju.no",
        "alvdal.no",
        "amli.no",
        "åmli.no",
        "amot.no",
        "åmot.no",
        "andebu.no",
        "andoy.no",
        "andøy.no",
        "andasuolo.no",
        "ardal.no",
        "årdal.no",
        "aremark.no",
        "arendal.no",
        "ås.no",
        "aseral.no",
        "åseral.no",
        "asker.no",
        "askim.no",
        "askvoll.no",
        "askoy.no",
        "askøy.no",
        "asnes.no",
        "åsnes.no",
        "audnedaln.no",
        "aukra.no",
        "aure.no",
        "aurland.no",
        "aurskog-holand.no",
        "aurskog-høland.no",
        "austevoll.no",
        "austrheim.no",
        "averoy.no",
        "averøy.no",
        "balestrand.no",
        "ballangen.no",
        "balat.no",
        "bálát.no",
        "balsfjord.no",
        "bahccavuotna.no",
        "báhccavuotna.no",
        "bamble.no",
        "bardu.no",
        "beardu.no",
        "beiarn.no",
        "bajddar.no",
        "bájddar.no",
        "baidar.no",
        "báidár.no",
        "berg.no",
        "bergen.no",
        "berlevag.no",
        "berlevåg.no",
        "bearalvahki.no",
        "bearalváhki.no",
        "bindal.no",
        "birkenes.no",
        "bjarkoy.no",
        "bjarkøy.no",
        "bjerkreim.no",
        "bjugn.no",
        "bodo.no",
        "bodø.no",
        "badaddja.no",
        "bådåddjå.no",
        "budejju.no",
        "bokn.no",
        "bremanger.no",
        "bronnoy.no",
        "brønnøy.no",
        "bygland.no",
        "bykle.no",
        "barum.no",
        "bærum.no",
        "bo.telemark.no",
        "bø.telemark.no",
        "bo.nordland.no",
        "bø.nordland.no",
        "bievat.no",
        "bievát.no",
        "bomlo.no",
        "bømlo.no",
        "batsfjord.no",
        "båtsfjord.no",
        "bahcavuotna.no",
        "báhcavuotna.no",
        "dovre.no",
        "drammen.no",
        "drangedal.no",
        "dyroy.no",
        "dyrøy.no",
        "donna.no",
        "dønna.no",
        "eid.no",
        "eidfjord.no",
        "eidsberg.no",
        "eidskog.no",
        "eidsvoll.no",
        "eigersund.no",
        "elverum.no",
        "enebakk.no",
        "engerdal.no",
        "etne.no",
        "etnedal.no",
        "evenes.no",
        "evenassi.no",
        "evenášši.no",
        "evje-og-hornnes.no",
        "farsund.no",
        "fauske.no",
        "fuossko.no",
        "fuoisku.no",
        "fedje.no",
        "fet.no",
        "finnoy.no",
        "finnøy.no",
        "fitjar.no",
        "fjaler.no",
        "fjell.no",
        "flakstad.no",
        "flatanger.no",
        "flekkefjord.no",
        "flesberg.no",
        "flora.no",
        "fla.no",
        "flå.no",
        "folldal.no",
        "forsand.no",
        "fosnes.no",
        "frei.no",
        "frogn.no",
        "froland.no",
        "frosta.no",
        "frana.no",
        "fræna.no",
        "froya.no",
        "frøya.no",
        "fusa.no",
        "fyresdal.no",
        "forde.no",
        "førde.no",
        "gamvik.no",
        "gangaviika.no",
        "gáŋgaviika.no",
        "gaular.no",
        "gausdal.no",
        "gildeskal.no",
        "gildeskål.no",
        "giske.no",
        "gjemnes.no",
        "gjerdrum.no",
        "gjerstad.no",
        "gjesdal.no",
        "gjovik.no",
        "gjøvik.no",
        "gloppen.no",
        "gol.no",
        "gran.no",
        "grane.no",
        "granvin.no",
        "gratangen.no",
        "grimstad.no",
        "grong.no",
        "kraanghke.no",
        "kråanghke.no",
        "grue.no",
        "gulen.no",
        "hadsel.no",
        "halden.no",
        "halsa.no",
        "hamar.no",
        "hamaroy.no",
        "habmer.no",
        "hábmer.no",
        "hapmir.no",
        "hápmir.no",
        "hammerfest.no",
        "hammarfeasta.no",
        "hámmárfeasta.no",
        "haram.no",
        "hareid.no",
        "harstad.no",
        "hasvik.no",
        "aknoluokta.no",
        "ákŋoluokta.no",
        "hattfjelldal.no",
        "aarborte.no",
        "haugesund.no",
        "hemne.no",
        "hemnes.no",
        "hemsedal.no",
        "heroy.more-og-romsdal.no",
        "herøy.møre-og-romsdal.no",
        "heroy.nordland.no",
        "herøy.nordland.no",
        "hitra.no",
        "hjartdal.no",
        "hjelmeland.no",
        "hobol.no",
        "hobøl.no",
        "hof.no",
        "hol.no",
        "hole.no",
        "holmestrand.no",
        "holtalen.no",
        "holtålen.no",
        "hornindal.no",
        "horten.no",
        "hurdal.no",
        "hurum.no",
        "hvaler.no",
        "hyllestad.no",
        "hagebostad.no",
        "hægebostad.no",
        "hoyanger.no",
        "høyanger.no",
        "hoylandet.no",
        "høylandet.no",
        "ha.no",
        "hå.no",
        "ibestad.no",
        "inderoy.no",
        "inderøy.no",
        "iveland.no",
        "jevnaker.no",
        "jondal.no",
        "jolster.no",
        "jølster.no",
        "karasjok.no",
        "karasjohka.no",
        "kárášjohka.no",
        "karlsoy.no",
        "galsa.no",
        "gálsá.no",
        "karmoy.no",
        "karmøy.no",
        "kautokeino.no",
        "guovdageaidnu.no",
        "klepp.no",
        "klabu.no",
        "klæbu.no",
        "kongsberg.no",
        "kongsvinger.no",
        "kragero.no",
        "kragerø.no",
        "kristiansand.no",
        "kristiansund.no",
        "krodsherad.no",
        "krødsherad.no",
        "kvalsund.no",
        "rahkkeravju.no",
        "ráhkkerávju.no",
        "kvam.no",
        "kvinesdal.no",
        "kvinnherad.no",
        "kviteseid.no",
        "kvitsoy.no",
        "kvitsøy.no",
        "kvafjord.no",
        "kvæfjord.no",
        "giehtavuoatna.no",
        "kvanangen.no",
        "kvænangen.no",
        "navuotna.no",
        "návuotna.no",
        "kafjord.no",
        "kåfjord.no",
        "gaivuotna.no",
        "gáivuotna.no",
        "larvik.no",
        "lavangen.no",
        "lavagis.no",
        "loabat.no",
        "loabát.no",
        "lebesby.no",
        "davvesiida.no",
        "leikanger.no",
        "leirfjord.no",
        "leka.no",
        "leksvik.no",
        "lenvik.no",
        "leangaviika.no",
        "leaŋgaviika.no",
        "lesja.no",
        "levanger.no",
        "lier.no",
        "lierne.no",
        "lillehammer.no",
        "lillesand.no",
        "lindesnes.no",
        "lindas.no",
        "lindås.no",
        "lom.no",
        "loppa.no",
        "lahppi.no",
        "láhppi.no",
        "lund.no",
        "lunner.no",
        "luroy.no",
        "lurøy.no",
        "luster.no",
        "lyngdal.no",
        "lyngen.no",
        "ivgu.no",
        "lardal.no",
        "lerdal.no",
        "lærdal.no",
        "lodingen.no",
        "lødingen.no",
        "lorenskog.no",
        "lørenskog.no",
        "loten.no",
        "løten.no",
        "malvik.no",
        "masoy.no",
        "måsøy.no",
        "muosat.no",
        "muosát.no",
        "mandal.no",
        "marker.no",
        "marnardal.no",
        "masfjorden.no",
        "meland.no",
        "meldal.no",
        "melhus.no",
        "meloy.no",
        "meløy.no",
        "meraker.no",
        "meråker.no",
        "moareke.no",
        "moåreke.no",
        "midsund.no",
        "midtre-gauldal.no",
        "modalen.no",
        "modum.no",
        "molde.no",
        "moskenes.no",
        "moss.no",
        "mosvik.no",
        "malselv.no",
        "målselv.no",
        "malatvuopmi.no",
        "málatvuopmi.no",
        "namdalseid.no",
        "aejrie.no",
        "namsos.no",
        "namsskogan.no",
        "naamesjevuemie.no",
        "nååmesjevuemie.no",
        "laakesvuemie.no",
        "nannestad.no",
        "narvik.no",
        "narviika.no",
        "naustdal.no",
        "nedre-eiker.no",
        "nes.akershus.no",
        "nes.buskerud.no",
        "nesna.no",
        "nesodden.no",
        "nesseby.no",
        "unjarga.no",
        "unjárga.no",
        "nesset.no",
        "nissedal.no",
        "nittedal.no",
        "nord-aurdal.no",
        "nord-fron.no",
        "nord-odal.no",
        "norddal.no",
        "nordkapp.no",
        "davvenjarga.no",
        "davvenjárga.no",
        "nordre-land.no",
        "nordreisa.no",
        "raisa.no",
        "ráisa.no",
        "nore-og-uvdal.no",
        "notodden.no",
        "naroy.no",
        "nærøy.no",
        "notteroy.no",
        "nøtterøy.no",
        "odda.no",
        "oksnes.no",
        "øksnes.no",
        "oppdal.no",
        "oppegard.no",
        "oppegård.no",
        "orkdal.no",
        "orland.no",
        "ørland.no",
        "orskog.no",
        "ørskog.no",
        "orsta.no",
        "ørsta.no",
        "os.hedmark.no",
        "os.hordaland.no",
        "osen.no",
        "osteroy.no",
        "osterøy.no",
        "ostre-toten.no",
        "østre-toten.no",
        "overhalla.no",
        "ovre-eiker.no",
        "øvre-eiker.no",
        "oyer.no",
        "øyer.no",
        "oygarden.no",
        "øygarden.no",
        "oystre-slidre.no",
        "øystre-slidre.no",
        "porsanger.no",
        "porsangu.no",
        "porsáŋgu.no",
        "porsgrunn.no",
        "radoy.no",
        "radøy.no",
        "rakkestad.no",
        "rana.no",
        "ruovat.no",
        "randaberg.no",
        "rauma.no",
        "rendalen.no",
        "rennebu.no",
        "rennesoy.no",
        "rennesøy.no",
        "rindal.no",
        "ringebu.no",
        "ringerike.no",
        "ringsaker.no",
        "rissa.no",
        "risor.no",
        "risør.no",
        "roan.no",
        "rollag.no",
        "rygge.no",
        "ralingen.no",
        "rælingen.no",
        "rodoy.no",
        "rødøy.no",
        "romskog.no",
        "rømskog.no",
        "roros.no",
        "røros.no",
        "rost.no",
        "røst.no",
        "royken.no",
        "røyken.no",
        "royrvik.no",
        "røyrvik.no",
        "rade.no",
        "råde.no",
        "salangen.no",
        "siellak.no",
        "saltdal.no",
        "salat.no",
        "sálát.no",
        "sálat.no",
        "samnanger.no",
        "sande.more-og-romsdal.no",
        "sande.møre-og-romsdal.no",
        "sande.vestfold.no",
        "sandefjord.no",
        "sandnes.no",
        "sandoy.no",
        "sandøy.no",
        "sarpsborg.no",
        "sauda.no",
        "sauherad.no",
        "sel.no",
        "selbu.no",
        "selje.no",
        "seljord.no",
        "sigdal.no",
        "siljan.no",
        "sirdal.no",
        "skaun.no",
        "skedsmo.no",
        "ski.no",
        "skien.no",
        "skiptvet.no",
        "skjervoy.no",
        "skjervøy.no",
        "skierva.no",
        "skiervá.no",
        "skjak.no",
        "skjåk.no",
        "skodje.no",
        "skanland.no",
        "skånland.no",
        "skanit.no",
        "skánit.no",
        "smola.no",
        "smøla.no",
        "snillfjord.no",
        "snasa.no",
        "snåsa.no",
        "snoasa.no",
        "snaase.no",
        "snåase.no",
        "sogndal.no",
        "sokndal.no",
        "sola.no",
        "solund.no",
        "songdalen.no",
        "sortland.no",
        "spydeberg.no",
        "stange.no",
        "stavanger.no",
        "steigen.no",
        "steinkjer.no",
        "stjordal.no",
        "stjørdal.no",
        "stokke.no",
        "stor-elvdal.no",
        "stord.no",
        "stordal.no",
        "storfjord.no",
        "omasvuotna.no",
        "strand.no",
        "stranda.no",
        "stryn.no",
        "sula.no",
        "suldal.no",
        "sund.no",
        "sunndal.no",
        "surnadal.no",
        "sveio.no",
        "svelvik.no",
        "sykkylven.no",
        "sogne.no",
        "søgne.no",
        "somna.no",
        "sømna.no",
        "sondre-land.no",
        "søndre-land.no",
        "sor-aurdal.no",
        "sør-aurdal.no",
        "sor-fron.no",
        "sør-fron.no",
        "sor-odal.no",
        "sør-odal.no",
        "sor-varanger.no",
        "sør-varanger.no",
        "matta-varjjat.no",
        "mátta-várjjat.no",
        "sorfold.no",
        "sørfold.no",
        "sorreisa.no",
        "sørreisa.no",
        "sorum.no",
        "sørum.no",
        "tana.no",
        "deatnu.no",
        "time.no",
        "tingvoll.no",
        "tinn.no",
        "tjeldsund.no",
        "dielddanuorri.no",
        "tjome.no",
        "tjøme.no",
        "tokke.no",
        "tolga.no",
        "torsken.no",
        "tranoy.no",
        "tranøy.no",
        "tromso.no",
        "tromsø.no",
        "tromsa.no",
        "romsa.no",
        "trondheim.no",
        "troandin.no",
        "trysil.no",
        "trana.no",
        "træna.no",
        "trogstad.no",
        "trøgstad.no",
        "tvedestrand.no",
        "tydal.no",
        "tynset.no",
        "tysfjord.no",
        "divtasvuodna.no",
        "divttasvuotna.no",
        "tysnes.no",
        "tysvar.no",
        "tysvær.no",
        "tonsberg.no",
        "tønsberg.no",
        "ullensaker.no",
        "ullensvang.no",
        "ulvik.no",
        "utsira.no",
        "vadso.no",
        "vadsø.no",
        "cahcesuolo.no",
        "čáhcesuolo.no",
        "vaksdal.no",
        "valle.no",
        "vang.no",
        "vanylven.no",
        "vardo.no",
        "vardø.no",
        "varggat.no",
        "várggát.no",
        "vefsn.no",
        "vaapste.no",
        "vega.no",
        "vegarshei.no",
        "vegårshei.no",
        "vennesla.no",
        "verdal.no",
        "verran.no",
        "vestby.no",
        "vestnes.no",
        "vestre-slidre.no",
        "vestre-toten.no",
        "vestvagoy.no",
        "vestvågøy.no",
        "vevelstad.no",
        "vik.no",
        "vikna.no",
        "vindafjord.no",
        "volda.no",
        "voss.no",
        "varoy.no",
        "værøy.no",
        "vagan.no",
        "vågan.no",
        "voagat.no",
        "vagsoy.no",
        "vågsøy.no",
        "vaga.no",
        "vågå.no",
        "valer.ostfold.no",
        "våler.østfold.no",
        "valer.hedmark.no",
        "våler.hedmark.no",
        "*.np",
        "nr",
        "biz.nr",
        "info.nr",
        "gov.nr",
        "edu.nr",
        "org.nr",
        "net.nr",
        "com.nr",
        "nu",
        "nz",
        "ac.nz",
        "co.nz",
        "cri.nz",
        "geek.nz",
        "gen.nz",
        "govt.nz",
        "health.nz",
        "iwi.nz",
        "kiwi.nz",
        "maori.nz",
        "mil.nz",
        "māori.nz",
        "net.nz",
        "org.nz",
        "parliament.nz",
        "school.nz",
        "om",
        "co.om",
        "com.om",
        "edu.om",
        "gov.om",
        "med.om",
        "museum.om",
        "net.om",
        "org.om",
        "pro.om",
        "onion",
        "org",
        "pa",
        "ac.pa",
        "gob.pa",
        "com.pa",
        "org.pa",
        "sld.pa",
        "edu.pa",
        "net.pa",
        "ing.pa",
        "abo.pa",
        "med.pa",
        "nom.pa",
        "pe",
        "edu.pe",
        "gob.pe",
        "nom.pe",
        "mil.pe",
        "org.pe",
        "com.pe",
        "net.pe",
        "pf",
        "com.pf",
        "org.pf",
        "edu.pf",
        "*.pg",
        "ph",
        "com.ph",
        "net.ph",
        "org.ph",
        "gov.ph",
        "edu.ph",
        "ngo.ph",
        "mil.ph",
        "i.ph",
        "pk",
        "com.pk",
        "net.pk",
        "edu.pk",
        "org.pk",
        "fam.pk",
        "biz.pk",
        "web.pk",
        "gov.pk",
        "gob.pk",
        "gok.pk",
        "gon.pk",
        "gop.pk",
        "gos.pk",
        "info.pk",
        "pl",
        "com.pl",
        "net.pl",
        "org.pl",
        "aid.pl",
        "agro.pl",
        "atm.pl",
        "auto.pl",
        "biz.pl",
        "edu.pl",
        "gmina.pl",
        "gsm.pl",
        "info.pl",
        "mail.pl",
        "miasta.pl",
        "media.pl",
        "mil.pl",
        "nieruchomosci.pl",
        "nom.pl",
        "pc.pl",
        "powiat.pl",
        "priv.pl",
        "realestate.pl",
        "rel.pl",
        "sex.pl",
        "shop.pl",
        "sklep.pl",
        "sos.pl",
        "szkola.pl",
        "targi.pl",
        "tm.pl",
        "tourism.pl",
        "travel.pl",
        "turystyka.pl",
        "gov.pl",
        "ap.gov.pl",
        "ic.gov.pl",
        "is.gov.pl",
        "us.gov.pl",
        "kmpsp.gov.pl",
        "kppsp.gov.pl",
        "kwpsp.gov.pl",
        "psp.gov.pl",
        "wskr.gov.pl",
        "kwp.gov.pl",
        "mw.gov.pl",
        "ug.gov.pl",
        "um.gov.pl",
        "umig.gov.pl",
        "ugim.gov.pl",
        "upow.gov.pl",
        "uw.gov.pl",
        "starostwo.gov.pl",
        "pa.gov.pl",
        "po.gov.pl",
        "psse.gov.pl",
        "pup.gov.pl",
        "rzgw.gov.pl",
        "sa.gov.pl",
        "so.gov.pl",
        "sr.gov.pl",
        "wsa.gov.pl",
        "sko.gov.pl",
        "uzs.gov.pl",
        "wiih.gov.pl",
        "winb.gov.pl",
        "pinb.gov.pl",
        "wios.gov.pl",
        "witd.gov.pl",
        "wzmiuw.gov.pl",
        "piw.gov.pl",
        "wiw.gov.pl",
        "griw.gov.pl",
        "wif.gov.pl",
        "oum.gov.pl",
        "sdn.gov.pl",
        "zp.gov.pl",
        "uppo.gov.pl",
        "mup.gov.pl",
        "wuoz.gov.pl",
        "konsulat.gov.pl",
        "oirm.gov.pl",
        "augustow.pl",
        "babia-gora.pl",
        "bedzin.pl",
        "beskidy.pl",
        "bialowieza.pl",
        "bialystok.pl",
        "bielawa.pl",
        "bieszczady.pl",
        "boleslawiec.pl",
        "bydgoszcz.pl",
        "bytom.pl",
        "cieszyn.pl",
        "czeladz.pl",
        "czest.pl",
        "dlugoleka.pl",
        "elblag.pl",
        "elk.pl",
        "glogow.pl",
        "gniezno.pl",
        "gorlice.pl",
        "grajewo.pl",
        "ilawa.pl",
        "jaworzno.pl",
        "jelenia-gora.pl",
        "jgora.pl",
        "kalisz.pl",
        "kazimierz-dolny.pl",
        "karpacz.pl",
        "kartuzy.pl",
        "kaszuby.pl",
        "katowice.pl",
        "kepno.pl",
        "ketrzyn.pl",
        "klodzko.pl",
        "kobierzyce.pl",
        "kolobrzeg.pl",
        "konin.pl",
        "konskowola.pl",
        "kutno.pl",
        "lapy.pl",
        "lebork.pl",
        "legnica.pl",
        "lezajsk.pl",
        "limanowa.pl",
        "lomza.pl",
        "lowicz.pl",
        "lubin.pl",
        "lukow.pl",
        "malbork.pl",
        "malopolska.pl",
        "mazowsze.pl",
        "mazury.pl",
        "mielec.pl",
        "mielno.pl",
        "mragowo.pl",
        "naklo.pl",
        "nowaruda.pl",
        "nysa.pl",
        "olawa.pl",
        "olecko.pl",
        "olkusz.pl",
        "olsztyn.pl",
        "opoczno.pl",
        "opole.pl",
        "ostroda.pl",
        "ostroleka.pl",
        "ostrowiec.pl",
        "ostrowwlkp.pl",
        "pila.pl",
        "pisz.pl",
        "podhale.pl",
        "podlasie.pl",
        "polkowice.pl",
        "pomorze.pl",
        "pomorskie.pl",
        "prochowice.pl",
        "pruszkow.pl",
        "przeworsk.pl",
        "pulawy.pl",
        "radom.pl",
        "rawa-maz.pl",
        "rybnik.pl",
        "rzeszow.pl",
        "sanok.pl",
        "sejny.pl",
        "slask.pl",
        "slupsk.pl",
        "sosnowiec.pl",
        "stalowa-wola.pl",
        "skoczow.pl",
        "starachowice.pl",
        "stargard.pl",
        "suwalki.pl",
        "swidnica.pl",
        "swiebodzin.pl",
        "swinoujscie.pl",
        "szczecin.pl",
        "szczytno.pl",
        "tarnobrzeg.pl",
        "tgory.pl",
        "turek.pl",
        "tychy.pl",
        "ustka.pl",
        "walbrzych.pl",
        "warmia.pl",
        "warszawa.pl",
        "waw.pl",
        "wegrow.pl",
        "wielun.pl",
        "wlocl.pl",
        "wloclawek.pl",
        "wodzislaw.pl",
        "wolomin.pl",
        "wroclaw.pl",
        "zachpomor.pl",
        "zagan.pl",
        "zarow.pl",
        "zgora.pl",
        "zgorzelec.pl",
        "pm",
        "pn",
        "gov.pn",
        "co.pn",
        "org.pn",
        "edu.pn",
        "net.pn",
        "post",
        "pr",
        "com.pr",
        "net.pr",
        "org.pr",
        "gov.pr",
        "edu.pr",
        "isla.pr",
        "pro.pr",
        "biz.pr",
        "info.pr",
        "name.pr",
        "est.pr",
        "prof.pr",
        "ac.pr",
        "pro",
        "aaa.pro",
        "aca.pro",
        "acct.pro",
        "avocat.pro",
        "bar.pro",
        "cpa.pro",
        "eng.pro",
        "jur.pro",
        "law.pro",
        "med.pro",
        "recht.pro",
        "ps",
        "edu.ps",
        "gov.ps",
        "sec.ps",
        "plo.ps",
        "com.ps",
        "org.ps",
        "net.ps",
        "pt",
        "net.pt",
        "gov.pt",
        "org.pt",
        "edu.pt",
        "int.pt",
        "publ.pt",
        "com.pt",
        "nome.pt",
        "pw",
        "co.pw",
        "ne.pw",
        "or.pw",
        "ed.pw",
        "go.pw",
        "belau.pw",
        "py",
        "com.py",
        "coop.py",
        "edu.py",
        "gov.py",
        "mil.py",
        "net.py",
        "org.py",
        "qa",
        "com.qa",
        "edu.qa",
        "gov.qa",
        "mil.qa",
        "name.qa",
        "net.qa",
        "org.qa",
        "sch.qa",
        "re",
        "asso.re",
        "com.re",
        "nom.re",
        "ro",
        "arts.ro",
        "com.ro",
        "firm.ro",
        "info.ro",
        "nom.ro",
        "nt.ro",
        "org.ro",
        "rec.ro",
        "store.ro",
        "tm.ro",
        "www.ro",
        "rs",
        "ac.rs",
        "co.rs",
        "edu.rs",
        "gov.rs",
        "in.rs",
        "org.rs",
        "ru",
        "rw",
        "ac.rw",
        "co.rw",
        "coop.rw",
        "gov.rw",
        "mil.rw",
        "net.rw",
        "org.rw",
        "sa",
        "com.sa",
        "net.sa",
        "org.sa",
        "gov.sa",
        "med.sa",
        "pub.sa",
        "edu.sa",
        "sch.sa",
        "sb",
        "com.sb",
        "edu.sb",
        "gov.sb",
        "net.sb",
        "org.sb",
        "sc",
        "com.sc",
        "gov.sc",
        "net.sc",
        "org.sc",
        "edu.sc",
        "sd",
        "com.sd",
        "net.sd",
        "org.sd",
        "edu.sd",
        "med.sd",
        "tv.sd",
        "gov.sd",
        "info.sd",
        "se",
        "a.se",
        "ac.se",
        "b.se",
        "bd.se",
        "brand.se",
        "c.se",
        "d.se",
        "e.se",
        "f.se",
        "fh.se",
        "fhsk.se",
        "fhv.se",
        "g.se",
        "h.se",
        "i.se",
        "k.se",
        "komforb.se",
        "kommunalforbund.se",
        "komvux.se",
        "l.se",
        "lanbib.se",
        "m.se",
        "n.se",
        "naturbruksgymn.se",
        "o.se",
        "org.se",
        "p.se",
        "parti.se",
        "pp.se",
        "press.se",
        "r.se",
        "s.se",
        "t.se",
        "tm.se",
        "u.se",
        "w.se",
        "x.se",
        "y.se",
        "z.se",
        "sg",
        "com.sg",
        "net.sg",
        "org.sg",
        "gov.sg",
        "edu.sg",
        "per.sg",
        "sh",
        "com.sh",
        "net.sh",
        "gov.sh",
        "org.sh",
        "mil.sh",
        "si",
        "sj",
        "sk",
        "sl",
        "com.sl",
        "net.sl",
        "edu.sl",
        "gov.sl",
        "org.sl",
        "sm",
        "sn",
        "art.sn",
        "com.sn",
        "edu.sn",
        "gouv.sn",
        "org.sn",
        "perso.sn",
        "univ.sn",
        "so",
        "com.so",
        "edu.so",
        "gov.so",
        "me.so",
        "net.so",
        "org.so",
        "sr",
        "ss",
        "biz.ss",
        "com.ss",
        "edu.ss",
        "gov.ss",
        "net.ss",
        "org.ss",
        "st",
        "co.st",
        "com.st",
        "consulado.st",
        "edu.st",
        "embaixada.st",
        "gov.st",
        "mil.st",
        "net.st",
        "org.st",
        "principe.st",
        "saotome.st",
        "store.st",
        "su",
        "sv",
        "com.sv",
        "edu.sv",
        "gob.sv",
        "org.sv",
        "red.sv",
        "sx",
        "gov.sx",
        "sy",
        "edu.sy",
        "gov.sy",
        "net.sy",
        "mil.sy",
        "com.sy",
        "org.sy",
        "sz",
        "co.sz",
        "ac.sz",
        "org.sz",
        "tc",
        "td",
        "tel",
        "tf",
        "tg",
        "th",
        "ac.th",
        "co.th",
        "go.th",
        "in.th",
        "mi.th",
        "net.th",
        "or.th",
        "tj",
        "ac.tj",
        "biz.tj",
        "co.tj",
        "com.tj",
        "edu.tj",
        "go.tj",
        "gov.tj",
        "int.tj",
        "mil.tj",
        "name.tj",
        "net.tj",
        "nic.tj",
        "org.tj",
        "test.tj",
        "web.tj",
        "tk",
        "tl",
        "gov.tl",
        "tm",
        "com.tm",
        "co.tm",
        "org.tm",
        "net.tm",
        "nom.tm",
        "gov.tm",
        "mil.tm",
        "edu.tm",
        "tn",
        "com.tn",
        "ens.tn",
        "fin.tn",
        "gov.tn",
        "ind.tn",
        "intl.tn",
        "nat.tn",
        "net.tn",
        "org.tn",
        "info.tn",
        "perso.tn",
        "tourism.tn",
        "edunet.tn",
        "rnrt.tn",
        "rns.tn",
        "rnu.tn",
        "mincom.tn",
        "agrinet.tn",
        "defense.tn",
        "turen.tn",
        "to",
        "com.to",
        "gov.to",
        "net.to",
        "org.to",
        "edu.to",
        "mil.to",
        "tr",
        "av.tr",
        "bbs.tr",
        "bel.tr",
        "biz.tr",
        "com.tr",
        "dr.tr",
        "edu.tr",
        "gen.tr",
        "gov.tr",
        "info.tr",
        "mil.tr",
        "k12.tr",
        "kep.tr",
        "name.tr",
        "net.tr",
        "org.tr",
        "pol.tr",
        "tel.tr",
        "tsk.tr",
        "tv.tr",
        "web.tr",
        "nc.tr",
        "gov.nc.tr",
        "tt",
        "co.tt",
        "com.tt",
        "org.tt",
        "net.tt",
        "biz.tt",
        "info.tt",
        "pro.tt",
        "int.tt",
        "coop.tt",
        "jobs.tt",
        "mobi.tt",
        "travel.tt",
        "museum.tt",
        "aero.tt",
        "name.tt",
        "gov.tt",
        "edu.tt",
        "tv",
        "tw",
        "edu.tw",
        "gov.tw",
        "mil.tw",
        "com.tw",
        "net.tw",
        "org.tw",
        "idv.tw",
        "game.tw",
        "ebiz.tw",
        "club.tw",
        "網路.tw",
        "組織.tw",
        "商業.tw",
        "tz",
        "ac.tz",
        "co.tz",
        "go.tz",
        "hotel.tz",
        "info.tz",
        "me.tz",
        "mil.tz",
        "mobi.tz",
        "ne.tz",
        "or.tz",
        "sc.tz",
        "tv.tz",
        "ua",
        "com.ua",
        "edu.ua",
        "gov.ua",
        "in.ua",
        "net.ua",
        "org.ua",
        "cherkassy.ua",
        "cherkasy.ua",
        "chernigov.ua",
        "chernihiv.ua",
        "chernivtsi.ua",
        "chernovtsy.ua",
        "ck.ua",
        "cn.ua",
        "cr.ua",
        "crimea.ua",
        "cv.ua",
        "dn.ua",
        "dnepropetrovsk.ua",
        "dnipropetrovsk.ua",
        "dominic.ua",
        "donetsk.ua",
        "dp.ua",
        "if.ua",
        "ivano-frankivsk.ua",
        "kh.ua",
        "kharkiv.ua",
        "kharkov.ua",
        "kherson.ua",
        "khmelnitskiy.ua",
        "khmelnytskyi.ua",
        "kiev.ua",
        "kirovograd.ua",
        "km.ua",
        "kr.ua",
        "krym.ua",
        "ks.ua",
        "kv.ua",
        "kyiv.ua",
        "lg.ua",
        "lt.ua",
        "lugansk.ua",
        "lutsk.ua",
        "lv.ua",
        "lviv.ua",
        "mk.ua",
        "mykolaiv.ua",
        "nikolaev.ua",
        "od.ua",
        "odesa.ua",
        "odessa.ua",
        "pl.ua",
        "poltava.ua",
        "rivne.ua",
        "rovno.ua",
        "rv.ua",
        "sb.ua",
        "sebastopol.ua",
        "sevastopol.ua",
        "sm.ua",
        "sumy.ua",
        "te.ua",
        "ternopil.ua",
        "uz.ua",
        "uzhgorod.ua",
        "vinnica.ua",
        "vinnytsia.ua",
        "vn.ua",
        "volyn.ua",
        "yalta.ua",
        "zaporizhzhe.ua",
        "zaporizhzhia.ua",
        "zhitomir.ua",
        "zhytomyr.ua",
        "zp.ua",
        "zt.ua",
        "ug",
        "co.ug",
        "or.ug",
        "ac.ug",
        "sc.ug",
        "go.ug",
        "ne.ug",
        "com.ug",
        "org.ug",
        "uk",
        "ac.uk",
        "co.uk",
        "gov.uk",
        "ltd.uk",
        "me.uk",
        "net.uk",
        "nhs.uk",
        "org.uk",
        "plc.uk",
        "police.uk",
        "*.sch.uk",
        "us",
        "dni.us",
        "fed.us",
        "isa.us",
        "kids.us",
        "nsn.us",
        "ak.us",
        "al.us",
        "ar.us",
        "as.us",
        "az.us",
        "ca.us",
        "co.us",
        "ct.us",
        "dc.us",
        "de.us",
        "fl.us",
        "ga.us",
        "gu.us",
        "hi.us",
        "ia.us",
        "id.us",
        "il.us",
        "in.us",
        "ks.us",
        "ky.us",
        "la.us",
        "ma.us",
        "md.us",
        "me.us",
        "mi.us",
        "mn.us",
        "mo.us",
        "ms.us",
        "mt.us",
        "nc.us",
        "nd.us",
        "ne.us",
        "nh.us",
        "nj.us",
        "nm.us",
        "nv.us",
        "ny.us",
        "oh.us",
        "ok.us",
        "or.us",
        "pa.us",
        "pr.us",
        "ri.us",
        "sc.us",
        "sd.us",
        "tn.us",
        "tx.us",
        "ut.us",
        "vi.us",
        "vt.us",
        "va.us",
        "wa.us",
        "wi.us",
        "wv.us",
        "wy.us",
        "k12.ak.us",
        "k12.al.us",
        "k12.ar.us",
        "k12.as.us",
        "k12.az.us",
        "k12.ca.us",
        "k12.co.us",
        "k12.ct.us",
        "k12.dc.us",
        "k12.de.us",
        "k12.fl.us",
        "k12.ga.us",
        "k12.gu.us",
        "k12.ia.us",
        "k12.id.us",
        "k12.il.us",
        "k12.in.us",
        "k12.ks.us",
        "k12.ky.us",
        "k12.la.us",
        "k12.ma.us",
        "k12.md.us",
        "k12.me.us",
        "k12.mi.us",
        "k12.mn.us",
        "k12.mo.us",
        "k12.ms.us",
        "k12.mt.us",
        "k12.nc.us",
        "k12.ne.us",
        "k12.nh.us",
        "k12.nj.us",
        "k12.nm.us",
        "k12.nv.us",
        "k12.ny.us",
        "k12.oh.us",
        "k12.ok.us",
        "k12.or.us",
        "k12.pa.us",
        "k12.pr.us",
        "k12.ri.us",
        "k12.sc.us",
        "k12.tn.us",
        "k12.tx.us",
        "k12.ut.us",
        "k12.vi.us",
        "k12.vt.us",
        "k12.va.us",
        "k12.wa.us",
        "k12.wi.us",
        "k12.wy.us",
        "cc.ak.us",
        "cc.al.us",
        "cc.ar.us",
        "cc.as.us",
        "cc.az.us",
        "cc.ca.us",
        "cc.co.us",
        "cc.ct.us",
        "cc.dc.us",
        "cc.de.us",
        "cc.fl.us",
        "cc.ga.us",
        "cc.gu.us",
        "cc.hi.us",
        "cc.ia.us",
        "cc.id.us",
        "cc.il.us",
        "cc.in.us",
        "cc.ks.us",
        "cc.ky.us",
        "cc.la.us",
        "cc.ma.us",
        "cc.md.us",
        "cc.me.us",
        "cc.mi.us",
        "cc.mn.us",
        "cc.mo.us",
        "cc.ms.us",
        "cc.mt.us",
        "cc.nc.us",
        "cc.nd.us",
        "cc.ne.us",
        "cc.nh.us",
        "cc.nj.us",
        "cc.nm.us",
        "cc.nv.us",
        "cc.ny.us",
        "cc.oh.us",
        "cc.ok.us",
        "cc.or.us",
        "cc.pa.us",
        "cc.pr.us",
        "cc.ri.us",
        "cc.sc.us",
        "cc.sd.us",
        "cc.tn.us",
        "cc.tx.us",
        "cc.ut.us",
        "cc.vi.us",
        "cc.vt.us",
        "cc.va.us",
        "cc.wa.us",
        "cc.wi.us",
        "cc.wv.us",
        "cc.wy.us",
        "lib.ak.us",
        "lib.al.us",
        "lib.ar.us",
        "lib.as.us",
        "lib.az.us",
        "lib.ca.us",
        "lib.co.us",
        "lib.ct.us",
        "lib.dc.us",
        "lib.fl.us",
        "lib.ga.us",
        "lib.gu.us",
        "lib.hi.us",
        "lib.ia.us",
        "lib.id.us",
        "lib.il.us",
        "lib.in.us",
        "lib.ks.us",
        "lib.ky.us",
        "lib.la.us",
        "lib.ma.us",
        "lib.md.us",
        "lib.me.us",
        "lib.mi.us",
        "lib.mn.us",
        "lib.mo.us",
        "lib.ms.us",
        "lib.mt.us",
        "lib.nc.us",
        "lib.nd.us",
        "lib.ne.us",
        "lib.nh.us",
        "lib.nj.us",
        "lib.nm.us",
        "lib.nv.us",
        "lib.ny.us",
        "lib.oh.us",
        "lib.ok.us",
        "lib.or.us",
        "lib.pa.us",
        "lib.pr.us",
        "lib.ri.us",
        "lib.sc.us",
        "lib.sd.us",
        "lib.tn.us",
        "lib.tx.us",
        "lib.ut.us",
        "lib.vi.us",
        "lib.vt.us",
        "lib.va.us",
        "lib.wa.us",
        "lib.wi.us",
        "lib.wy.us",
        "pvt.k12.ma.us",
        "chtr.k12.ma.us",
        "paroch.k12.ma.us",
        "ann-arbor.mi.us",
        "cog.mi.us",
        "dst.mi.us",
        "eaton.mi.us",
        "gen.mi.us",
        "mus.mi.us",
        "tec.mi.us",
        "washtenaw.mi.us",
        "uy",
        "com.uy",
        "edu.uy",
        "gub.uy",
        "mil.uy",
        "net.uy",
        "org.uy",
        "uz",
        "co.uz",
        "com.uz",
        "net.uz",
        "org.uz",
        "va",
        "vc",
        "com.vc",
        "net.vc",
        "org.vc",
        "gov.vc",
        "mil.vc",
        "edu.vc",
        "ve",
        "arts.ve",
        "co.ve",
        "com.ve",
        "e12.ve",
        "edu.ve",
        "firm.ve",
        "gob.ve",
        "gov.ve",
        "info.ve",
        "int.ve",
        "mil.ve",
        "net.ve",
        "org.ve",
        "rec.ve",
        "store.ve",
        "tec.ve",
        "web.ve",
        "vg",
        "vi",
        "co.vi",
        "com.vi",
        "k12.vi",
        "net.vi",
        "org.vi",
        "vn",
        "com.vn",
        "net.vn",
        "org.vn",
        "edu.vn",
        "gov.vn",
        "int.vn",
        "ac.vn",
        "biz.vn",
        "info.vn",
        "name.vn",
        "pro.vn",
        "health.vn",
        "vu",
        "com.vu",
        "edu.vu",
        "net.vu",
        "org.vu",
        "wf",
        "ws",
        "com.ws",
        "net.ws",
        "org.ws",
        "gov.ws",
        "edu.ws",
        "yt",
        "امارات",
        "հայ",
        "বাংলা",
        "бг",
        "бел",
        "中国",
        "中國",
        "الجزائر",
        "مصر",
        "ею",
        "موريتانيا",
        "გე",
        "ελ",
        "香港",
        "公司.香港",
        "教育.香港",
        "政府.香港",
        "個人.香港",
        "網絡.香港",
        "組織.香港",
        "ಭಾರತ",
        "ଭାରତ",
        "ভাৰত",
        "भारतम्",
        "भारोत",
        "ڀارت",
        "ഭാരതം",
        "भारत",
        "بارت",
        "بھارت",
        "భారత్",
        "ભારત",
        "ਭਾਰਤ",
        "ভারত",
        "இந்தியா",
        "ایران",
        "ايران",
        "عراق",
        "الاردن",
        "한국",
        "қаз",
        "ලංකා",
        "இலங்கை",
        "المغرب",
        "мкд",
        "мон",
        "澳門",
        "澳门",
        "مليسيا",
        "عمان",
        "پاکستان",
        "پاكستان",
        "فلسطين",
        "срб",
        "пр.срб",
        "орг.срб",
        "обр.срб",
        "од.срб",
        "упр.срб",
        "ак.срб",
        "рф",
        "قطر",
        "السعودية",
        "السعودیة",
        "السعودیۃ",
        "السعوديه",
        "سودان",
        "新加坡",
        "சிங்கப்பூர்",
        "سورية",
        "سوريا",
        "ไทย",
        "ศึกษา.ไทย",
        "ธุรกิจ.ไทย",
        "รัฐบาล.ไทย",
        "ทหาร.ไทย",
        "เน็ต.ไทย",
        "องค์กร.ไทย",
        "تونس",
        "台灣",
        "台湾",
        "臺灣",
        "укр",
        "اليمن",
        "xxx",
        "*.ye",
        "ac.za",
        "agric.za",
        "alt.za",
        "co.za",
        "edu.za",
        "gov.za",
        "grondar.za",
        "law.za",
        "mil.za",
        "net.za",
        "ngo.za",
        "nic.za",
        "nis.za",
        "nom.za",
        "org.za",
        "school.za",
        "tm.za",
        "web.za",
        "zm",
        "ac.zm",
        "biz.zm",
        "co.zm",
        "com.zm",
        "edu.zm",
        "gov.zm",
        "info.zm",
        "mil.zm",
        "net.zm",
        "org.zm",
        "sch.zm",
        "zw",
        "ac.zw",
        "co.zw",
        "gov.zw",
        "mil.zw",
        "org.zw",
        "aaa",
        "aarp",
        "abarth",
        "abb",
        "abbott",
        "abbvie",
        "abc",
        "able",
        "abogado",
        "abudhabi",
        "academy",
        "accenture",
        "accountant",
        "accountants",
        "aco",
        "actor",
        "adac",
        "ads",
        "adult",
        "aeg",
        "aetna",
        "afamilycompany",
        "afl",
        "africa",
        "agakhan",
        "agency",
        "aig",
        "aigo",
        "airbus",
        "airforce",
        "airtel",
        "akdn",
        "alfaromeo",
        "alibaba",
        "alipay",
        "allfinanz",
        "allstate",
        "ally",
        "alsace",
        "alstom",
        "amazon",
        "americanexpress",
        "americanfamily",
        "amex",
        "amfam",
        "amica",
        "amsterdam",
        "analytics",
        "android",
        "anquan",
        "anz",
        "aol",
        "apartments",
        "app",
        "apple",
        "aquarelle",
        "arab",
        "aramco",
        "archi",
        "army",
        "art",
        "arte",
        "asda",
        "associates",
        "athleta",
        "attorney",
        "auction",
        "audi",
        "audible",
        "audio",
        "auspost",
        "author",
        "auto",
        "autos",
        "avianca",
        "aws",
        "axa",
        "azure",
        "baby",
        "baidu",
        "banamex",
        "bananarepublic",
        "band",
        "bank",
        "bar",
        "barcelona",
        "barclaycard",
        "barclays",
        "barefoot",
        "bargains",
        "baseball",
        "basketball",
        "bauhaus",
        "bayern",
        "bbc",
        "bbt",
        "bbva",
        "bcg",
        "bcn",
        "beats",
        "beauty",
        "beer",
        "bentley",
        "berlin",
        "best",
        "bestbuy",
        "bet",
        "bharti",
        "bible",
        "bid",
        "bike",
        "bing",
        "bingo",
        "bio",
        "black",
        "blackfriday",
        "blockbuster",
        "blog",
        "bloomberg",
        "blue",
        "bms",
        "bmw",
        "bnpparibas",
        "boats",
        "boehringer",
        "bofa",
        "bom",
        "bond",
        "boo",
        "book",
        "booking",
        "bosch",
        "bostik",
        "boston",
        "bot",
        "boutique",
        "box",
        "bradesco",
        "bridgestone",
        "broadway",
        "broker",
        "brother",
        "brussels",
        "budapest",
        "bugatti",
        "build",
        "builders",
        "business",
        "buy",
        "buzz",
        "bzh",
        "cab",
        "cafe",
        "cal",
        "call",
        "calvinklein",
        "cam",
        "camera",
        "camp",
        "cancerresearch",
        "canon",
        "capetown",
        "capital",
        "capitalone",
        "car",
        "caravan",
        "cards",
        "care",
        "career",
        "careers",
        "cars",
        "casa",
        "case",
        "caseih",
        "cash",
        "casino",
        "catering",
        "catholic",
        "cba",
        "cbn",
        "cbre",
        "cbs",
        "ceb",
        "center",
        "ceo",
        "cern",
        "cfa",
        "cfd",
        "chanel",
        "channel",
        "charity",
        "chase",
        "chat",
        "cheap",
        "chintai",
        "christmas",
        "chrome",
        "church",
        "cipriani",
        "circle",
        "cisco",
        "citadel",
        "citi",
        "citic",
        "city",
        "cityeats",
        "claims",
        "cleaning",
        "click",
        "clinic",
        "clinique",
        "clothing",
        "cloud",
        "club",
        "clubmed",
        "coach",
        "codes",
        "coffee",
        "college",
        "cologne",
        "comcast",
        "commbank",
        "community",
        "company",
        "compare",
        "computer",
        "comsec",
        "condos",
        "construction",
        "consulting",
        "contact",
        "contractors",
        "cooking",
        "cookingchannel",
        "cool",
        "corsica",
        "country",
        "coupon",
        "coupons",
        "courses",
        "cpa",
        "credit",
        "creditcard",
        "creditunion",
        "cricket",
        "crown",
        "crs",
        "cruise",
        "cruises",
        "csc",
        "cuisinella",
        "cymru",
        "cyou",
        "dabur",
        "dad",
        "dance",
        "data",
        "date",
        "dating",
        "datsun",
        "day",
        "dclk",
        "dds",
        "deal",
        "dealer",
        "deals",
        "degree",
        "delivery",
        "dell",
        "deloitte",
        "delta",
        "democrat",
        "dental",
        "dentist",
        "desi",
        "design",
        "dev",
        "dhl",
        "diamonds",
        "diet",
        "digital",
        "direct",
        "directory",
        "discount",
        "discover",
        "dish",
        "diy",
        "dnp",
        "docs",
        "doctor",
        "dog",
        "domains",
        "dot",
        "download",
        "drive",
        "dtv",
        "dubai",
        "duck",
        "dunlop",
        "dupont",
        "durban",
        "dvag",
        "dvr",
        "earth",
        "eat",
        "eco",
        "edeka",
        "education",
        "email",
        "emerck",
        "energy",
        "engineer",
        "engineering",
        "enterprises",
        "epson",
        "equipment",
        "ericsson",
        "erni",
        "esq",
        "estate",
        "esurance",
        "etisalat",
        "eurovision",
        "eus",
        "events",
        "exchange",
        "expert",
        "exposed",
        "express",
        "extraspace",
        "fage",
        "fail",
        "fairwinds",
        "faith",
        "family",
        "fan",
        "fans",
        "farm",
        "farmers",
        "fashion",
        "fast",
        "fedex",
        "feedback",
        "ferrari",
        "ferrero",
        "fiat",
        "fidelity",
        "fido",
        "film",
        "final",
        "finance",
        "financial",
        "fire",
        "firestone",
        "firmdale",
        "fish",
        "fishing",
        "fit",
        "fitness",
        "flickr",
        "flights",
        "flir",
        "florist",
        "flowers",
        "fly",
        "foo",
        "food",
        "foodnetwork",
        "football",
        "ford",
        "forex",
        "forsale",
        "forum",
        "foundation",
        "fox",
        "free",
        "fresenius",
        "frl",
        "frogans",
        "frontdoor",
        "frontier",
        "ftr",
        "fujitsu",
        "fujixerox",
        "fun",
        "fund",
        "furniture",
        "futbol",
        "fyi",
        "gal",
        "gallery",
        "gallo",
        "gallup",
        "game",
        "games",
        "gap",
        "garden",
        "gay",
        "gbiz",
        "gdn",
        "gea",
        "gent",
        "genting",
        "george",
        "ggee",
        "gift",
        "gifts",
        "gives",
        "giving",
        "glade",
        "glass",
        "gle",
        "global",
        "globo",
        "gmail",
        "gmbh",
        "gmo",
        "gmx",
        "godaddy",
        "gold",
        "goldpoint",
        "golf",
        "goo",
        "goodyear",
        "goog",
        "google",
        "gop",
        "got",
        "grainger",
        "graphics",
        "gratis",
        "green",
        "gripe",
        "grocery",
        "group",
        "guardian",
        "gucci",
        "guge",
        "guide",
        "guitars",
        "guru",
        "hair",
        "hamburg",
        "hangout",
        "haus",
        "hbo",
        "hdfc",
        "hdfcbank",
        "health",
        "healthcare",
        "help",
        "helsinki",
        "here",
        "hermes",
        "hgtv",
        "hiphop",
        "hisamitsu",
        "hitachi",
        "hiv",
        "hkt",
        "hockey",
        "holdings",
        "holiday",
        "homedepot",
        "homegoods",
        "homes",
        "homesense",
        "honda",
        "horse",
        "hospital",
        "host",
        "hosting",
        "hot",
        "hoteles",
        "hotels",
        "hotmail",
        "house",
        "how",
        "hsbc",
        "hughes",
        "hyatt",
        "hyundai",
        "ibm",
        "icbc",
        "ice",
        "icu",
        "ieee",
        "ifm",
        "ikano",
        "imamat",
        "imdb",
        "immo",
        "immobilien",
        "inc",
        "industries",
        "infiniti",
        "ing",
        "ink",
        "institute",
        "insurance",
        "insure",
        "intel",
        "international",
        "intuit",
        "investments",
        "ipiranga",
        "irish",
        "ismaili",
        "ist",
        "istanbul",
        "itau",
        "itv",
        "iveco",
        "jaguar",
        "java",
        "jcb",
        "jcp",
        "jeep",
        "jetzt",
        "jewelry",
        "jio",
        "jll",
        "jmp",
        "jnj",
        "joburg",
        "jot",
        "joy",
        "jpmorgan",
        "jprs",
        "juegos",
        "juniper",
        "kaufen",
        "kddi",
        "kerryhotels",
        "kerrylogistics",
        "kerryproperties",
        "kfh",
        "kia",
        "kim",
        "kinder",
        "kindle",
        "kitchen",
        "kiwi",
        "koeln",
        "komatsu",
        "kosher",
        "kpmg",
        "kpn",
        "krd",
        "kred",
        "kuokgroup",
        "kyoto",
        "lacaixa",
        "lamborghini",
        "lamer",
        "lancaster",
        "lancia",
        "land",
        "landrover",
        "lanxess",
        "lasalle",
        "lat",
        "latino",
        "latrobe",
        "law",
        "lawyer",
        "lds",
        "lease",
        "leclerc",
        "lefrak",
        "legal",
        "lego",
        "lexus",
        "lgbt",
        "liaison",
        "lidl",
        "life",
        "lifeinsurance",
        "lifestyle",
        "lighting",
        "like",
        "lilly",
        "limited",
        "limo",
        "lincoln",
        "linde",
        "link",
        "lipsy",
        "live",
        "living",
        "lixil",
        "llc",
        "llp",
        "loan",
        "loans",
        "locker",
        "locus",
        "loft",
        "lol",
        "london",
        "lotte",
        "lotto",
        "love",
        "lpl",
        "lplfinancial",
        "ltd",
        "ltda",
        "lundbeck",
        "lupin",
        "luxe",
        "luxury",
        "macys",
        "madrid",
        "maif",
        "maison",
        "makeup",
        "man",
        "management",
        "mango",
        "map",
        "market",
        "marketing",
        "markets",
        "marriott",
        "marshalls",
        "maserati",
        "mattel",
        "mba",
        "mckinsey",
        "med",
        "media",
        "meet",
        "melbourne",
        "meme",
        "memorial",
        "men",
        "menu",
        "merckmsd",
        "metlife",
        "miami",
        "microsoft",
        "mini",
        "mint",
        "mit",
        "mitsubishi",
        "mlb",
        "mls",
        "mma",
        "mobile",
        "moda",
        "moe",
        "moi",
        "mom",
        "monash",
        "money",
        "monster",
        "mormon",
        "mortgage",
        "moscow",
        "moto",
        "motorcycles",
        "mov",
        "movie",
        "msd",
        "mtn",
        "mtr",
        "mutual",
        "nab",
        "nadex",
        "nagoya",
        "nationwide",
        "natura",
        "navy",
        "nba",
        "nec",
        "netbank",
        "netflix",
        "network",
        "neustar",
        "new",
        "newholland",
        "news",
        "next",
        "nextdirect",
        "nexus",
        "nfl",
        "ngo",
        "nhk",
        "nico",
        "nike",
        "nikon",
        "ninja",
        "nissan",
        "nissay",
        "nokia",
        "northwesternmutual",
        "norton",
        "now",
        "nowruz",
        "nowtv",
        "nra",
        "nrw",
        "ntt",
        "nyc",
        "obi",
        "observer",
        "off",
        "office",
        "okinawa",
        "olayan",
        "olayangroup",
        "oldnavy",
        "ollo",
        "omega",
        "one",
        "ong",
        "onl",
        "online",
        "onyourside",
        "ooo",
        "open",
        "oracle",
        "orange",
        "organic",
        "origins",
        "osaka",
        "otsuka",
        "ott",
        "ovh",
        "page",
        "panasonic",
        "paris",
        "pars",
        "partners",
        "parts",
        "party",
        "passagens",
        "pay",
        "pccw",
        "pet",
        "pfizer",
        "pharmacy",
        "phd",
        "philips",
        "phone",
        "photo",
        "photography",
        "photos",
        "physio",
        "pics",
        "pictet",
        "pictures",
        "pid",
        "pin",
        "ping",
        "pink",
        "pioneer",
        "pizza",
        "place",
        "play",
        "playstation",
        "plumbing",
        "plus",
        "pnc",
        "pohl",
        "poker",
        "politie",
        "porn",
        "pramerica",
        "praxi",
        "press",
        "prime",
        "prod",
        "productions",
        "prof",
        "progressive",
        "promo",
        "properties",
        "property",
        "protection",
        "pru",
        "prudential",
        "pub",
        "pwc",
        "qpon",
        "quebec",
        "quest",
        "qvc",
        "racing",
        "radio",
        "raid",
        "read",
        "realestate",
        "realtor",
        "realty",
        "recipes",
        "red",
        "redstone",
        "redumbrella",
        "rehab",
        "reise",
        "reisen",
        "reit",
        "reliance",
        "ren",
        "rent",
        "rentals",
        "repair",
        "report",
        "republican",
        "rest",
        "restaurant",
        "review",
        "reviews",
        "rexroth",
        "rich",
        "richardli",
        "ricoh",
        "rightathome",
        "ril",
        "rio",
        "rip",
        "rmit",
        "rocher",
        "rocks",
        "rodeo",
        "rogers",
        "room",
        "rsvp",
        "rugby",
        "ruhr",
        "run",
        "rwe",
        "ryukyu",
        "saarland",
        "safe",
        "safety",
        "sakura",
        "sale",
        "salon",
        "samsclub",
        "samsung",
        "sandvik",
        "sandvikcoromant",
        "sanofi",
        "sap",
        "sarl",
        "sas",
        "save",
        "saxo",
        "sbi",
        "sbs",
        "sca",
        "scb",
        "schaeffler",
        "schmidt",
        "scholarships",
        "school",
        "schule",
        "schwarz",
        "science",
        "scjohnson",
        "scor",
        "scot",
        "search",
        "seat",
        "secure",
        "security",
        "seek",
        "select",
        "sener",
        "services",
        "ses",
        "seven",
        "sew",
        "sex",
        "sexy",
        "sfr",
        "shangrila",
        "sharp",
        "shaw",
        "shell",
        "shia",
        "shiksha",
        "shoes",
        "shop",
        "shopping",
        "shouji",
        "show",
        "showtime",
        "shriram",
        "silk",
        "sina",
        "singles",
        "site",
        "ski",
        "skin",
        "sky",
        "skype",
        "sling",
        "smart",
        "smile",
        "sncf",
        "soccer",
        "social",
        "softbank",
        "software",
        "sohu",
        "solar",
        "solutions",
        "song",
        "sony",
        "soy",
        "spa",
        "space",
        "sport",
        "spot",
        "spreadbetting",
        "srl",
        "stada",
        "staples",
        "star",
        "statebank",
        "statefarm",
        "stc",
        "stcgroup",
        "stockholm",
        "storage",
        "store",
        "stream",
        "studio",
        "study",
        "style",
        "sucks",
        "supplies",
        "supply",
        "support",
        "surf",
        "surgery",
        "suzuki",
        "swatch",
        "swiftcover",
        "swiss",
        "sydney",
        "symantec",
        "systems",
        "tab",
        "taipei",
        "talk",
        "taobao",
        "target",
        "tatamotors",
        "tatar",
        "tattoo",
        "tax",
        "taxi",
        "tci",
        "tdk",
        "team",
        "tech",
        "technology",
        "temasek",
        "tennis",
        "teva",
        "thd",
        "theater",
        "theatre",
        "tiaa",
        "tickets",
        "tienda",
        "tiffany",
        "tips",
        "tires",
        "tirol",
        "tjmaxx",
        "tjx",
        "tkmaxx",
        "tmall",
        "today",
        "tokyo",
        "tools",
        "top",
        "toray",
        "toshiba",
        "total",
        "tours",
        "town",
        "toyota",
        "toys",
        "trade",
        "trading",
        "training",
        "travel",
        "travelchannel",
        "travelers",
        "travelersinsurance",
        "trust",
        "trv",
        "tube",
        "tui",
        "tunes",
        "tushu",
        "tvs",
        "ubank",
        "ubs",
        "unicom",
        "university",
        "uno",
        "uol",
        "ups",
        "vacations",
        "vana",
        "vanguard",
        "vegas",
        "ventures",
        "verisign",
        "versicherung",
        "vet",
        "viajes",
        "video",
        "vig",
        "viking",
        "villas",
        "vin",
        "vip",
        "virgin",
        "visa",
        "vision",
        "vistaprint",
        "viva",
        "vivo",
        "vlaanderen",
        "vodka",
        "volkswagen",
        "volvo",
        "vote",
        "voting",
        "voto",
        "voyage",
        "vuelos",
        "wales",
        "walmart",
        "walter",
        "wang",
        "wanggou",
        "watch",
        "watches",
        "weather",
        "weatherchannel",
        "webcam",
        "weber",
        "website",
        "wed",
        "wedding",
        "weibo",
        "weir",
        "whoswho",
        "wien",
        "wiki",
        "williamhill",
        "win",
        "windows",
        "wine",
        "winners",
        "wme",
        "wolterskluwer",
        "woodside",
        "work",
        "works",
        "world",
        "wow",
        "wtc",
        "wtf",
        "xbox",
        "xerox",
        "xfinity",
        "xihuan",
        "xin",
        "कॉम",
        "セール",
        "佛山",
        "慈善",
        "集团",
        "在线",
        "大众汽车",
        "点看",
        "คอม",
        "八卦",
        "موقع",
        "公益",
        "公司",
        "香格里拉",
        "网站",
        "移动",
        "我爱你",
        "москва",
        "католик",
        "онлайн",
        "сайт",
        "联通",
        "קום",
        "时尚",
        "微博",
        "淡马锡",
        "ファッション",
        "орг",
        "नेट",
        "ストア",
        "アマゾン",
        "삼성",
        "商标",
        "商店",
        "商城",
        "дети",
        "ポイント",
        "新闻",
        "工行",
        "家電",
        "كوم",
        "中文网",
        "中信",
        "娱乐",
        "谷歌",
        "電訊盈科",
        "购物",
        "クラウド",
        "通販",
        "网店",
        "संगठन",
        "餐厅",
        "网络",
        "ком",
        "亚马逊",
        "诺基亚",
        "食品",
        "飞利浦",
        "手表",
        "手机",
        "ارامكو",
        "العليان",
        "اتصالات",
        "بازار",
        "ابوظبي",
        "كاثوليك",
        "همراه",
        "닷컴",
        "政府",
        "شبكة",
        "بيتك",
        "عرب",
        "机构",
        "组织机构",
        "健康",
        "招聘",
        "рус",
        "珠宝",
        "大拿",
        "みんな",
        "グーグル",
        "世界",
        "書籍",
        "网址",
        "닷넷",
        "コム",
        "天主教",
        "游戏",
        "vermögensberater",
        "vermögensberatung",
        "企业",
        "信息",
        "嘉里大酒店",
        "嘉里",
        "广东",
        "政务",
        "xyz",
        "yachts",
        "yahoo",
        "yamaxun",
        "yandex",
        "yodobashi",
        "yoga",
        "yokohama",
        "you",
        "youtube",
        "yun",
        "zappos",
        "zara",
        "zero",
        "zip",
        "zone",
        "zuerich",
        "cc.ua",
        "inf.ua",
        "ltd.ua",
        "adobeaemcloud.com",
        "adobeaemcloud.net",
        "*.dev.adobeaemcloud.com",
        "beep.pl",
        "barsy.ca",
        "*.compute.estate",
        "*.alces.network",
        "altervista.org",
        "alwaysdata.net",
        "cloudfront.net",
        "*.compute.amazonaws.com",
        "*.compute-1.amazonaws.com",
        "*.compute.amazonaws.com.cn",
        "us-east-1.amazonaws.com",
        "cn-north-1.eb.amazonaws.com.cn",
        "cn-northwest-1.eb.amazonaws.com.cn",
        "elasticbeanstalk.com",
        "ap-northeast-1.elasticbeanstalk.com",
        "ap-northeast-2.elasticbeanstalk.com",
        "ap-northeast-3.elasticbeanstalk.com",
        "ap-south-1.elasticbeanstalk.com",
        "ap-southeast-1.elasticbeanstalk.com",
        "ap-southeast-2.elasticbeanstalk.com",
        "ca-central-1.elasticbeanstalk.com",
        "eu-central-1.elasticbeanstalk.com",
        "eu-west-1.elasticbeanstalk.com",
        "eu-west-2.elasticbeanstalk.com",
        "eu-west-3.elasticbeanstalk.com",
        "sa-east-1.elasticbeanstalk.com",
        "us-east-1.elasticbeanstalk.com",
        "us-east-2.elasticbeanstalk.com",
        "us-gov-west-1.elasticbeanstalk.com",
        "us-west-1.elasticbeanstalk.com",
        "us-west-2.elasticbeanstalk.com",
        "*.elb.amazonaws.com",
        "*.elb.amazonaws.com.cn",
        "s3.amazonaws.com",
        "s3-ap-northeast-1.amazonaws.com",
        "s3-ap-northeast-2.amazonaws.com",
        "s3-ap-south-1.amazonaws.com",
        "s3-ap-southeast-1.amazonaws.com",
        "s3-ap-southeast-2.amazonaws.com",
        "s3-ca-central-1.amazonaws.com",
        "s3-eu-central-1.amazonaws.com",
        "s3-eu-west-1.amazonaws.com",
        "s3-eu-west-2.amazonaws.com",
        "s3-eu-west-3.amazonaws.com",
        "s3-external-1.amazonaws.com",
        "s3-fips-us-gov-west-1.amazonaws.com",
        "s3-sa-east-1.amazonaws.com",
        "s3-us-gov-west-1.amazonaws.com",
        "s3-us-east-2.amazonaws.com",
        "s3-us-west-1.amazonaws.com",
        "s3-us-west-2.amazonaws.com",
        "s3.ap-northeast-2.amazonaws.com",
        "s3.ap-south-1.amazonaws.com",
        "s3.cn-north-1.amazonaws.com.cn",
        "s3.ca-central-1.amazonaws.com",
        "s3.eu-central-1.amazonaws.com",
        "s3.eu-west-2.amazonaws.com",
        "s3.eu-west-3.amazonaws.com",
        "s3.us-east-2.amazonaws.com",
        "s3.dualstack.ap-northeast-1.amazonaws.com",
        "s3.dualstack.ap-northeast-2.amazonaws.com",
        "s3.dualstack.ap-south-1.amazonaws.com",
        "s3.dualstack.ap-southeast-1.amazonaws.com",
        "s3.dualstack.ap-southeast-2.amazonaws.com",
        "s3.dualstack.ca-central-1.amazonaws.com",
        "s3.dualstack.eu-central-1.amazonaws.com",
        "s3.dualstack.eu-west-1.amazonaws.com",
        "s3.dualstack.eu-west-2.amazonaws.com",
        "s3.dualstack.eu-west-3.amazonaws.com",
        "s3.dualstack.sa-east-1.amazonaws.com",
        "s3.dualstack.us-east-1.amazonaws.com",
        "s3.dualstack.us-east-2.amazonaws.com",
        "s3-website-us-east-1.amazonaws.com",
        "s3-website-us-west-1.amazonaws.com",
        "s3-website-us-west-2.amazonaws.com",
        "s3-website-ap-northeast-1.amazonaws.com",
        "s3-website-ap-southeast-1.amazonaws.com",
        "s3-website-ap-southeast-2.amazonaws.com",
        "s3-website-eu-west-1.amazonaws.com",
        "s3-website-sa-east-1.amazonaws.com",
        "s3-website.ap-northeast-2.amazonaws.com",
        "s3-website.ap-south-1.amazonaws.com",
        "s3-website.ca-central-1.amazonaws.com",
        "s3-website.eu-central-1.amazonaws.com",
        "s3-website.eu-west-2.amazonaws.com",
        "s3-website.eu-west-3.amazonaws.com",
        "s3-website.us-east-2.amazonaws.com",
        "amsw.nl",
        "t3l3p0rt.net",
        "tele.amune.org",
        "apigee.io",
        "on-aptible.com",
        "user.aseinet.ne.jp",
        "gv.vc",
        "d.gv.vc",
        "user.party.eus",
        "pimienta.org",
        "poivron.org",
        "potager.org",
        "sweetpepper.org",
        "myasustor.com",
        "myfritz.net",
        "*.awdev.ca",
        "*.advisor.ws",
        "b-data.io",
        "backplaneapp.io",
        "balena-devices.com",
        "app.banzaicloud.io",
        "betainabox.com",
        "bnr.la",
        "blackbaudcdn.net",
        "boomla.net",
        "boxfuse.io",
        "square7.ch",
        "bplaced.com",
        "bplaced.de",
        "square7.de",
        "bplaced.net",
        "square7.net",
        "browsersafetymark.io",
        "uk0.bigv.io",
        "dh.bytemark.co.uk",
        "vm.bytemark.co.uk",
        "mycd.eu",
        "carrd.co",
        "crd.co",
        "uwu.ai",
        "ae.org",
        "ar.com",
        "br.com",
        "cn.com",
        "com.de",
        "com.se",
        "de.com",
        "eu.com",
        "gb.com",
        "gb.net",
        "hu.com",
        "hu.net",
        "jp.net",
        "jpn.com",
        "kr.com",
        "mex.com",
        "no.com",
        "qc.com",
        "ru.com",
        "sa.com",
        "se.net",
        "uk.com",
        "uk.net",
        "us.com",
        "uy.com",
        "za.bz",
        "za.com",
        "africa.com",
        "gr.com",
        "in.net",
        "us.org",
        "co.com",
        "c.la",
        "certmgr.org",
        "xenapponazure.com",
        "discourse.group",
        "virtueeldomein.nl",
        "cleverapps.io",
        "*.lcl.dev",
        "*.stg.dev",
        "c66.me",
        "cloud66.ws",
        "cloud66.zone",
        "jdevcloud.com",
        "wpdevcloud.com",
        "cloudaccess.host",
        "freesite.host",
        "cloudaccess.net",
        "cloudcontrolled.com",
        "cloudcontrolapp.com",
        "cloudera.site",
        "trycloudflare.com",
        "workers.dev",
        "wnext.app",
        "co.ca",
        "*.otap.co",
        "co.cz",
        "c.cdn77.org",
        "cdn77-ssl.net",
        "r.cdn77.net",
        "rsc.cdn77.org",
        "ssl.origin.cdn77-secure.org",
        "cloudns.asia",
        "cloudns.biz",
        "cloudns.club",
        "cloudns.cc",
        "cloudns.eu",
        "cloudns.in",
        "cloudns.info",
        "cloudns.org",
        "cloudns.pro",
        "cloudns.pw",
        "cloudns.us",
        "cloudeity.net",
        "cnpy.gdn",
        "co.nl",
        "co.no",
        "webhosting.be",
        "hosting-cluster.nl",
        "ac.ru",
        "edu.ru",
        "gov.ru",
        "int.ru",
        "mil.ru",
        "test.ru",
        "dyn.cosidns.de",
        "dynamisches-dns.de",
        "dnsupdater.de",
        "internet-dns.de",
        "l-o-g-i-n.de",
        "dynamic-dns.info",
        "feste-ip.net",
        "knx-server.net",
        "static-access.net",
        "realm.cz",
        "*.cryptonomic.net",
        "cupcake.is",
        "*.customer-oci.com",
        "*.oci.customer-oci.com",
        "*.ocp.customer-oci.com",
        "*.ocs.customer-oci.com",
        "cyon.link",
        "cyon.site",
        "daplie.me",
        "localhost.daplie.me",
        "dattolocal.com",
        "dattorelay.com",
        "dattoweb.com",
        "mydatto.com",
        "dattolocal.net",
        "mydatto.net",
        "biz.dk",
        "co.dk",
        "firm.dk",
        "reg.dk",
        "store.dk",
        "*.dapps.earth",
        "*.bzz.dapps.earth",
        "builtwithdark.com",
        "edgestack.me",
        "debian.net",
        "dedyn.io",
        "dnshome.de",
        "online.th",
        "shop.th",
        "drayddns.com",
        "dreamhosters.com",
        "mydrobo.com",
        "drud.io",
        "drud.us",
        "duckdns.org",
        "dy.fi",
        "tunk.org",
        "dyndns-at-home.com",
        "dyndns-at-work.com",
        "dyndns-blog.com",
        "dyndns-free.com",
        "dyndns-home.com",
        "dyndns-ip.com",
        "dyndns-mail.com",
        "dyndns-office.com",
        "dyndns-pics.com",
        "dyndns-remote.com",
        "dyndns-server.com",
        "dyndns-web.com",
        "dyndns-wiki.com",
        "dyndns-work.com",
        "dyndns.biz",
        "dyndns.info",
        "dyndns.org",
        "dyndns.tv",
        "at-band-camp.net",
        "ath.cx",
        "barrel-of-knowledge.info",
        "barrell-of-knowledge.info",
        "better-than.tv",
        "blogdns.com",
        "blogdns.net",
        "blogdns.org",
        "blogsite.org",
        "boldlygoingnowhere.org",
        "broke-it.net",
        "buyshouses.net",
        "cechire.com",
        "dnsalias.com",
        "dnsalias.net",
        "dnsalias.org",
        "dnsdojo.com",
        "dnsdojo.net",
        "dnsdojo.org",
        "does-it.net",
        "doesntexist.com",
        "doesntexist.org",
        "dontexist.com",
        "dontexist.net",
        "dontexist.org",
        "doomdns.com",
        "doomdns.org",
        "dvrdns.org",
        "dyn-o-saur.com",
        "dynalias.com",
        "dynalias.net",
        "dynalias.org",
        "dynathome.net",
        "dyndns.ws",
        "endofinternet.net",
        "endofinternet.org",
        "endoftheinternet.org",
        "est-a-la-maison.com",
        "est-a-la-masion.com",
        "est-le-patron.com",
        "est-mon-blogueur.com",
        "for-better.biz",
        "for-more.biz",
        "for-our.info",
        "for-some.biz",
        "for-the.biz",
        "forgot.her.name",
        "forgot.his.name",
        "from-ak.com",
        "from-al.com",
        "from-ar.com",
        "from-az.net",
        "from-ca.com",
        "from-co.net",
        "from-ct.com",
        "from-dc.com",
        "from-de.com",
        "from-fl.com",
        "from-ga.com",
        "from-hi.com",
        "from-ia.com",
        "from-id.com",
        "from-il.com",
        "from-in.com",
        "from-ks.com",
        "from-ky.com",
        "from-la.net",
        "from-ma.com",
        "from-md.com",
        "from-me.org",
        "from-mi.com",
        "from-mn.com",
        "from-mo.com",
        "from-ms.com",
        "from-mt.com",
        "from-nc.com",
        "from-nd.com",
        "from-ne.com",
        "from-nh.com",
        "from-nj.com",
        "from-nm.com",
        "from-nv.com",
        "from-ny.net",
        "from-oh.com",
        "from-ok.com",
        "from-or.com",
        "from-pa.com",
        "from-pr.com",
        "from-ri.com",
        "from-sc.com",
        "from-sd.com",
        "from-tn.com",
        "from-tx.com",
        "from-ut.com",
        "from-va.com",
        "from-vt.com",
        "from-wa.com",
        "from-wi.com",
        "from-wv.com",
        "from-wy.com",
        "ftpaccess.cc",
        "fuettertdasnetz.de",
        "game-host.org",
        "game-server.cc",
        "getmyip.com",
        "gets-it.net",
        "go.dyndns.org",
        "gotdns.com",
        "gotdns.org",
        "groks-the.info",
        "groks-this.info",
        "ham-radio-op.net",
        "here-for-more.info",
        "hobby-site.com",
        "hobby-site.org",
        "home.dyndns.org",
        "homedns.org",
        "homeftp.net",
        "homeftp.org",
        "homeip.net",
        "homelinux.com",
        "homelinux.net",
        "homelinux.org",
        "homeunix.com",
        "homeunix.net",
        "homeunix.org",
        "iamallama.com",
        "in-the-band.net",
        "is-a-anarchist.com",
        "is-a-blogger.com",
        "is-a-bookkeeper.com",
        "is-a-bruinsfan.org",
        "is-a-bulls-fan.com",
        "is-a-candidate.org",
        "is-a-caterer.com",
        "is-a-celticsfan.org",
        "is-a-chef.com",
        "is-a-chef.net",
        "is-a-chef.org",
        "is-a-conservative.com",
        "is-a-cpa.com",
        "is-a-cubicle-slave.com",
        "is-a-democrat.com",
        "is-a-designer.com",
        "is-a-doctor.com",
        "is-a-financialadvisor.com",
        "is-a-geek.com",
        "is-a-geek.net",
        "is-a-geek.org",
        "is-a-green.com",
        "is-a-guru.com",
        "is-a-hard-worker.com",
        "is-a-hunter.com",
        "is-a-knight.org",
        "is-a-landscaper.com",
        "is-a-lawyer.com",
        "is-a-liberal.com",
        "is-a-libertarian.com",
        "is-a-linux-user.org",
        "is-a-llama.com",
        "is-a-musician.com",
        "is-a-nascarfan.com",
        "is-a-nurse.com",
        "is-a-painter.com",
        "is-a-patsfan.org",
        "is-a-personaltrainer.com",
        "is-a-photographer.com",
        "is-a-player.com",
        "is-a-republican.com",
        "is-a-rockstar.com",
        "is-a-socialist.com",
        "is-a-soxfan.org",
        "is-a-student.com",
        "is-a-teacher.com",
        "is-a-techie.com",
        "is-a-therapist.com",
        "is-an-accountant.com",
        "is-an-actor.com",
        "is-an-actress.com",
        "is-an-anarchist.com",
        "is-an-artist.com",
        "is-an-engineer.com",
        "is-an-entertainer.com",
        "is-by.us",
        "is-certified.com",
        "is-found.org",
        "is-gone.com",
        "is-into-anime.com",
        "is-into-cars.com",
        "is-into-cartoons.com",
        "is-into-games.com",
        "is-leet.com",
        "is-lost.org",
        "is-not-certified.com",
        "is-saved.org",
        "is-slick.com",
        "is-uberleet.com",
        "is-very-bad.org",
        "is-very-evil.org",
        "is-very-good.org",
        "is-very-nice.org",
        "is-very-sweet.org",
        "is-with-theband.com",
        "isa-geek.com",
        "isa-geek.net",
        "isa-geek.org",
        "isa-hockeynut.com",
        "issmarterthanyou.com",
        "isteingeek.de",
        "istmein.de",
        "kicks-ass.net",
        "kicks-ass.org",
        "knowsitall.info",
        "land-4-sale.us",
        "lebtimnetz.de",
        "leitungsen.de",
        "likes-pie.com",
        "likescandy.com",
        "merseine.nu",
        "mine.nu",
        "misconfused.org",
        "mypets.ws",
        "myphotos.cc",
        "neat-url.com",
        "office-on-the.net",
        "on-the-web.tv",
        "podzone.net",
        "podzone.org",
        "readmyblog.org",
        "saves-the-whales.com",
        "scrapper-site.net",
        "scrapping.cc",
        "selfip.biz",
        "selfip.com",
        "selfip.info",
        "selfip.net",
        "selfip.org",
        "sells-for-less.com",
        "sells-for-u.com",
        "sells-it.net",
        "sellsyourhome.org",
        "servebbs.com",
        "servebbs.net",
        "servebbs.org",
        "serveftp.net",
        "serveftp.org",
        "servegame.org",
        "shacknet.nu",
        "simple-url.com",
        "space-to-rent.com",
        "stuff-4-sale.org",
        "stuff-4-sale.us",
        "teaches-yoga.com",
        "thruhere.net",
        "traeumtgerade.de",
        "webhop.biz",
        "webhop.info",
        "webhop.net",
        "webhop.org",
        "worse-than.tv",
        "writesthisblog.com",
        "ddnss.de",
        "dyn.ddnss.de",
        "dyndns.ddnss.de",
        "dyndns1.de",
        "dyn-ip24.de",
        "home-webserver.de",
        "dyn.home-webserver.de",
        "myhome-server.de",
        "ddnss.org",
        "definima.net",
        "definima.io",
        "bci.dnstrace.pro",
        "ddnsfree.com",
        "ddnsgeek.com",
        "giize.com",
        "gleeze.com",
        "kozow.com",
        "loseyourip.com",
        "ooguy.com",
        "theworkpc.com",
        "casacam.net",
        "dynu.net",
        "accesscam.org",
        "camdvr.org",
        "freeddns.org",
        "mywire.org",
        "webredirect.org",
        "myddns.rocks",
        "blogsite.xyz",
        "dynv6.net",
        "e4.cz",
        "en-root.fr",
        "mytuleap.com",
        "onred.one",
        "staging.onred.one",
        "enonic.io",
        "customer.enonic.io",
        "eu.org",
        "al.eu.org",
        "asso.eu.org",
        "at.eu.org",
        "au.eu.org",
        "be.eu.org",
        "bg.eu.org",
        "ca.eu.org",
        "cd.eu.org",
        "ch.eu.org",
        "cn.eu.org",
        "cy.eu.org",
        "cz.eu.org",
        "de.eu.org",
        "dk.eu.org",
        "edu.eu.org",
        "ee.eu.org",
        "es.eu.org",
        "fi.eu.org",
        "fr.eu.org",
        "gr.eu.org",
        "hr.eu.org",
        "hu.eu.org",
        "ie.eu.org",
        "il.eu.org",
        "in.eu.org",
        "int.eu.org",
        "is.eu.org",
        "it.eu.org",
        "jp.eu.org",
        "kr.eu.org",
        "lt.eu.org",
        "lu.eu.org",
        "lv.eu.org",
        "mc.eu.org",
        "me.eu.org",
        "mk.eu.org",
        "mt.eu.org",
        "my.eu.org",
        "net.eu.org",
        "ng.eu.org",
        "nl.eu.org",
        "no.eu.org",
        "nz.eu.org",
        "paris.eu.org",
        "pl.eu.org",
        "pt.eu.org",
        "q-a.eu.org",
        "ro.eu.org",
        "ru.eu.org",
        "se.eu.org",
        "si.eu.org",
        "sk.eu.org",
        "tr.eu.org",
        "uk.eu.org",
        "us.eu.org",
        "eu-1.evennode.com",
        "eu-2.evennode.com",
        "eu-3.evennode.com",
        "eu-4.evennode.com",
        "us-1.evennode.com",
        "us-2.evennode.com",
        "us-3.evennode.com",
        "us-4.evennode.com",
        "twmail.cc",
        "twmail.net",
        "twmail.org",
        "mymailer.com.tw",
        "url.tw",
        "apps.fbsbx.com",
        "ru.net",
        "adygeya.ru",
        "bashkiria.ru",
        "bir.ru",
        "cbg.ru",
        "com.ru",
        "dagestan.ru",
        "grozny.ru",
        "kalmykia.ru",
        "kustanai.ru",
        "marine.ru",
        "mordovia.ru",
        "msk.ru",
        "mytis.ru",
        "nalchik.ru",
        "nov.ru",
        "pyatigorsk.ru",
        "spb.ru",
        "vladikavkaz.ru",
        "vladimir.ru",
        "abkhazia.su",
        "adygeya.su",
        "aktyubinsk.su",
        "arkhangelsk.su",
        "armenia.su",
        "ashgabad.su",
        "azerbaijan.su",
        "balashov.su",
        "bashkiria.su",
        "bryansk.su",
        "bukhara.su",
        "chimkent.su",
        "dagestan.su",
        "east-kazakhstan.su",
        "exnet.su",
        "georgia.su",
        "grozny.su",
        "ivanovo.su",
        "jambyl.su",
        "kalmykia.su",
        "kaluga.su",
        "karacol.su",
        "karaganda.su",
        "karelia.su",
        "khakassia.su",
        "krasnodar.su",
        "kurgan.su",
        "kustanai.su",
        "lenug.su",
        "mangyshlak.su",
        "mordovia.su",
        "msk.su",
        "murmansk.su",
        "nalchik.su",
        "navoi.su",
        "north-kazakhstan.su",
        "nov.su",
        "obninsk.su",
        "penza.su",
        "pokrovsk.su",
        "sochi.su",
        "spb.su",
        "tashkent.su",
        "termez.su",
        "togliatti.su",
        "troitsk.su",
        "tselinograd.su",
        "tula.su",
        "tuva.su",
        "vladikavkaz.su",
        "vladimir.su",
        "vologda.su",
        "channelsdvr.net",
        "fastly-terrarium.com",
        "fastlylb.net",
        "map.fastlylb.net",
        "freetls.fastly.net",
        "map.fastly.net",
        "a.prod.fastly.net",
        "global.prod.fastly.net",
        "a.ssl.fastly.net",
        "b.ssl.fastly.net",
        "global.ssl.fastly.net",
        "fastpanel.direct",
        "fastvps-server.com",
        "fhapp.xyz",
        "fedorainfracloud.org",
        "fedorapeople.org",
        "cloud.fedoraproject.org",
        "app.os.fedoraproject.org",
        "app.os.stg.fedoraproject.org",
        "mydobiss.com",
        "filegear.me",
        "filegear-au.me",
        "filegear-de.me",
        "filegear-gb.me",
        "filegear-ie.me",
        "filegear-jp.me",
        "filegear-sg.me",
        "firebaseapp.com",
        "flynnhub.com",
        "flynnhosting.net",
        "0e.vc",
        "freebox-os.com",
        "freeboxos.com",
        "fbx-os.fr",
        "fbxos.fr",
        "freebox-os.fr",
        "freeboxos.fr",
        "freedesktop.org",
        "*.futurecms.at",
        "*.ex.futurecms.at",
        "*.in.futurecms.at",
        "futurehosting.at",
        "futuremailing.at",
        "*.ex.ortsinfo.at",
        "*.kunden.ortsinfo.at",
        "*.statics.cloud",
        "service.gov.uk",
        "gehirn.ne.jp",
        "usercontent.jp",
        "gentapps.com",
        "lab.ms",
        "github.io",
        "githubusercontent.com",
        "gitlab.io",
        "glitch.me",
        "lolipop.io",
        "cloudapps.digital",
        "london.cloudapps.digital",
        "homeoffice.gov.uk",
        "ro.im",
        "shop.ro",
        "goip.de",
        "run.app",
        "a.run.app",
        "web.app",
        "*.0emm.com",
        "appspot.com",
        "*.r.appspot.com",
        "blogspot.ae",
        "blogspot.al",
        "blogspot.am",
        "blogspot.ba",
        "blogspot.be",
        "blogspot.bg",
        "blogspot.bj",
        "blogspot.ca",
        "blogspot.cf",
        "blogspot.ch",
        "blogspot.cl",
        "blogspot.co.at",
        "blogspot.co.id",
        "blogspot.co.il",
        "blogspot.co.ke",
        "blogspot.co.nz",
        "blogspot.co.uk",
        "blogspot.co.za",
        "blogspot.com",
        "blogspot.com.ar",
        "blogspot.com.au",
        "blogspot.com.br",
        "blogspot.com.by",
        "blogspot.com.co",
        "blogspot.com.cy",
        "blogspot.com.ee",
        "blogspot.com.eg",
        "blogspot.com.es",
        "blogspot.com.mt",
        "blogspot.com.ng",
        "blogspot.com.tr",
        "blogspot.com.uy",
        "blogspot.cv",
        "blogspot.cz",
        "blogspot.de",
        "blogspot.dk",
        "blogspot.fi",
        "blogspot.fr",
        "blogspot.gr",
        "blogspot.hk",
        "blogspot.hr",
        "blogspot.hu",
        "blogspot.ie",
        "blogspot.in",
        "blogspot.is",
        "blogspot.it",
        "blogspot.jp",
        "blogspot.kr",
        "blogspot.li",
        "blogspot.lt",
        "blogspot.lu",
        "blogspot.md",
        "blogspot.mk",
        "blogspot.mr",
        "blogspot.mx",
        "blogspot.my",
        "blogspot.nl",
        "blogspot.no",
        "blogspot.pe",
        "blogspot.pt",
        "blogspot.qa",
        "blogspot.re",
        "blogspot.ro",
        "blogspot.rs",
        "blogspot.ru",
        "blogspot.se",
        "blogspot.sg",
        "blogspot.si",
        "blogspot.sk",
        "blogspot.sn",
        "blogspot.td",
        "blogspot.tw",
        "blogspot.ug",
        "blogspot.vn",
        "cloudfunctions.net",
        "cloud.goog",
        "codespot.com",
        "googleapis.com",
        "googlecode.com",
        "pagespeedmobilizer.com",
        "publishproxy.com",
        "withgoogle.com",
        "withyoutube.com",
        "awsmppl.com",
        "fin.ci",
        "free.hr",
        "caa.li",
        "ua.rs",
        "conf.se",
        "hs.zone",
        "hs.run",
        "hashbang.sh",
        "hasura.app",
        "hasura-app.io",
        "hepforge.org",
        "herokuapp.com",
        "herokussl.com",
        "myravendb.com",
        "ravendb.community",
        "ravendb.me",
        "development.run",
        "ravendb.run",
        "bpl.biz",
        "orx.biz",
        "ng.city",
        "biz.gl",
        "ng.ink",
        "col.ng",
        "firm.ng",
        "gen.ng",
        "ltd.ng",
        "ngo.ng",
        "ng.school",
        "sch.so",
        "häkkinen.fi",
        "*.moonscale.io",
        "moonscale.net",
        "iki.fi",
        "dyn-berlin.de",
        "in-berlin.de",
        "in-brb.de",
        "in-butter.de",
        "in-dsl.de",
        "in-dsl.net",
        "in-dsl.org",
        "in-vpn.de",
        "in-vpn.net",
        "in-vpn.org",
        "biz.at",
        "info.at",
        "info.cx",
        "ac.leg.br",
        "al.leg.br",
        "am.leg.br",
        "ap.leg.br",
        "ba.leg.br",
        "ce.leg.br",
        "df.leg.br",
        "es.leg.br",
        "go.leg.br",
        "ma.leg.br",
        "mg.leg.br",
        "ms.leg.br",
        "mt.leg.br",
        "pa.leg.br",
        "pb.leg.br",
        "pe.leg.br",
        "pi.leg.br",
        "pr.leg.br",
        "rj.leg.br",
        "rn.leg.br",
        "ro.leg.br",
        "rr.leg.br",
        "rs.leg.br",
        "sc.leg.br",
        "se.leg.br",
        "sp.leg.br",
        "to.leg.br",
        "pixolino.com",
        "ipifony.net",
        "mein-iserv.de",
        "test-iserv.de",
        "iserv.dev",
        "iobb.net",
        "myjino.ru",
        "*.hosting.myjino.ru",
        "*.landing.myjino.ru",
        "*.spectrum.myjino.ru",
        "*.vps.myjino.ru",
        "*.triton.zone",
        "*.cns.joyent.com",
        "js.org",
        "kaas.gg",
        "khplay.nl",
        "keymachine.de",
        "kinghost.net",
        "uni5.net",
        "knightpoint.systems",
        "oya.to",
        "co.krd",
        "edu.krd",
        "git-repos.de",
        "lcube-server.de",
        "svn-repos.de",
        "leadpages.co",
        "lpages.co",
        "lpusercontent.com",
        "lelux.site",
        "co.business",
        "co.education",
        "co.events",
        "co.financial",
        "co.network",
        "co.place",
        "co.technology",
        "app.lmpm.com",
        "linkitools.space",
        "linkyard.cloud",
        "linkyard-cloud.ch",
        "members.linode.com",
        "nodebalancer.linode.com",
        "we.bs",
        "loginline.app",
        "loginline.dev",
        "loginline.io",
        "loginline.services",
        "loginline.site",
        "krasnik.pl",
        "leczna.pl",
        "lubartow.pl",
        "lublin.pl",
        "poniatowa.pl",
        "swidnik.pl",
        "uklugs.org",
        "glug.org.uk",
        "lug.org.uk",
        "lugs.org.uk",
        "barsy.bg",
        "barsy.co.uk",
        "barsyonline.co.uk",
        "barsycenter.com",
        "barsyonline.com",
        "barsy.club",
        "barsy.de",
        "barsy.eu",
        "barsy.in",
        "barsy.info",
        "barsy.io",
        "barsy.me",
        "barsy.menu",
        "barsy.mobi",
        "barsy.net",
        "barsy.online",
        "barsy.org",
        "barsy.pro",
        "barsy.pub",
        "barsy.shop",
        "barsy.site",
        "barsy.support",
        "barsy.uk",
        "*.magentosite.cloud",
        "mayfirst.info",
        "mayfirst.org",
        "hb.cldmail.ru",
        "miniserver.com",
        "memset.net",
        "cloud.metacentrum.cz",
        "custom.metacentrum.cz",
        "flt.cloud.muni.cz",
        "usr.cloud.muni.cz",
        "meteorapp.com",
        "eu.meteorapp.com",
        "co.pl",
        "azurecontainer.io",
        "azurewebsites.net",
        "azure-mobile.net",
        "cloudapp.net",
        "mozilla-iot.org",
        "bmoattachments.org",
        "net.ru",
        "org.ru",
        "pp.ru",
        "ui.nabu.casa",
        "pony.club",
        "of.fashion",
        "on.fashion",
        "of.football",
        "in.london",
        "of.london",
        "for.men",
        "and.mom",
        "for.mom",
        "for.one",
        "for.sale",
        "of.work",
        "to.work",
        "nctu.me",
        "bitballoon.com",
        "netlify.com",
        "4u.com",
        "ngrok.io",
        "nh-serv.co.uk",
        "nfshost.com",
        "dnsking.ch",
        "mypi.co",
        "n4t.co",
        "001www.com",
        "ddnslive.com",
        "myiphost.com",
        "forumz.info",
        "16-b.it",
        "32-b.it",
        "64-b.it",
        "soundcast.me",
        "tcp4.me",
        "dnsup.net",
        "hicam.net",
        "now-dns.net",
        "ownip.net",
        "vpndns.net",
        "dynserv.org",
        "now-dns.org",
        "x443.pw",
        "now-dns.top",
        "ntdll.top",
        "freeddns.us",
        "crafting.xyz",
        "zapto.xyz",
        "nsupdate.info",
        "nerdpol.ovh",
        "blogsyte.com",
        "brasilia.me",
        "cable-modem.org",
        "ciscofreak.com",
        "collegefan.org",
        "couchpotatofries.org",
        "damnserver.com",
        "ddns.me",
        "ditchyourip.com",
        "dnsfor.me",
        "dnsiskinky.com",
        "dvrcam.info",
        "dynns.com",
        "eating-organic.net",
        "fantasyleague.cc",
        "geekgalaxy.com",
        "golffan.us",
        "health-carereform.com",
        "homesecuritymac.com",
        "homesecuritypc.com",
        "hopto.me",
        "ilovecollege.info",
        "loginto.me",
        "mlbfan.org",
        "mmafan.biz",
        "myactivedirectory.com",
        "mydissent.net",
        "myeffect.net",
        "mymediapc.net",
        "mypsx.net",
        "mysecuritycamera.com",
        "mysecuritycamera.net",
        "mysecuritycamera.org",
        "net-freaks.com",
        "nflfan.org",
        "nhlfan.net",
        "no-ip.ca",
        "no-ip.co.uk",
        "no-ip.net",
        "noip.us",
        "onthewifi.com",
        "pgafan.net",
        "point2this.com",
        "pointto.us",
        "privatizehealthinsurance.net",
        "quicksytes.com",
        "read-books.org",
        "securitytactics.com",
        "serveexchange.com",
        "servehumour.com",
        "servep2p.com",
        "servesarcasm.com",
        "stufftoread.com",
        "ufcfan.org",
        "unusualperson.com",
        "workisboring.com",
        "3utilities.com",
        "bounceme.net",
        "ddns.net",
        "ddnsking.com",
        "gotdns.ch",
        "hopto.org",
        "myftp.biz",
        "myftp.org",
        "myvnc.com",
        "no-ip.biz",
        "no-ip.info",
        "no-ip.org",
        "noip.me",
        "redirectme.net",
        "servebeer.com",
        "serveblog.net",
        "servecounterstrike.com",
        "serveftp.com",
        "servegame.com",
        "servehalflife.com",
        "servehttp.com",
        "serveirc.com",
        "serveminecraft.net",
        "servemp3.com",
        "servepics.com",
        "servequake.com",
        "sytes.net",
        "webhop.me",
        "zapto.org",
        "stage.nodeart.io",
        "nodum.co",
        "nodum.io",
        "pcloud.host",
        "nyc.mn",
        "nom.ae",
        "nom.af",
        "nom.ai",
        "nom.al",
        "nym.by",
        "nym.bz",
        "nom.cl",
        "nym.ec",
        "nom.gd",
        "nom.ge",
        "nom.gl",
        "nym.gr",
        "nom.gt",
        "nym.gy",
        "nym.hk",
        "nom.hn",
        "nym.ie",
        "nom.im",
        "nom.ke",
        "nym.kz",
        "nym.la",
        "nym.lc",
        "nom.li",
        "nym.li",
        "nym.lt",
        "nym.lu",
        "nym.me",
        "nom.mk",
        "nym.mn",
        "nym.mx",
        "nom.nu",
        "nym.nz",
        "nym.pe",
        "nym.pt",
        "nom.pw",
        "nom.qa",
        "nym.ro",
        "nom.rs",
        "nom.si",
        "nym.sk",
        "nom.st",
        "nym.su",
        "nym.sx",
        "nom.tj",
        "nym.tw",
        "nom.ug",
        "nom.uy",
        "nom.vc",
        "nom.vg",
        "static.observableusercontent.com",
        "cya.gg",
        "cloudycluster.net",
        "nid.io",
        "opencraft.hosting",
        "operaunite.com",
        "skygearapp.com",
        "outsystemscloud.com",
        "ownprovider.com",
        "own.pm",
        "ox.rs",
        "oy.lc",
        "pgfog.com",
        "pagefrontapp.com",
        "art.pl",
        "gliwice.pl",
        "krakow.pl",
        "poznan.pl",
        "wroc.pl",
        "zakopane.pl",
        "pantheonsite.io",
        "gotpantheon.com",
        "mypep.link",
        "perspecta.cloud",
        "on-web.fr",
        "*.platform.sh",
        "*.platformsh.site",
        "dyn53.io",
        "co.bn",
        "xen.prgmr.com",
        "priv.at",
        "prvcy.page",
        "*.dweb.link",
        "protonet.io",
        "chirurgiens-dentistes-en-france.fr",
        "byen.site",
        "pubtls.org",
        "qualifioapp.com",
        "instantcloud.cn",
        "ras.ru",
        "qa2.com",
        "qcx.io",
        "*.sys.qcx.io",
        "dev-myqnapcloud.com",
        "alpha-myqnapcloud.com",
        "myqnapcloud.com",
        "*.quipelements.com",
        "vapor.cloud",
        "vaporcloud.io",
        "rackmaze.com",
        "rackmaze.net",
        "*.on-k3s.io",
        "*.on-rancher.cloud",
        "*.on-rio.io",
        "readthedocs.io",
        "rhcloud.com",
        "app.render.com",
        "onrender.com",
        "repl.co",
        "repl.run",
        "resindevice.io",
        "devices.resinstaging.io",
        "hzc.io",
        "wellbeingzone.eu",
        "ptplus.fit",
        "wellbeingzone.co.uk",
        "git-pages.rit.edu",
        "sandcats.io",
        "logoip.de",
        "logoip.com",
        "schokokeks.net",
        "gov.scot",
        "scrysec.com",
        "firewall-gateway.com",
        "firewall-gateway.de",
        "my-gateway.de",
        "my-router.de",
        "spdns.de",
        "spdns.eu",
        "firewall-gateway.net",
        "my-firewall.org",
        "myfirewall.org",
        "spdns.org",
        "biz.ua",
        "co.ua",
        "pp.ua",
        "shiftedit.io",
        "myshopblocks.com",
        "shopitsite.com",
        "mo-siemens.io",
        "1kapp.com",
        "appchizi.com",
        "applinzi.com",
        "sinaapp.com",
        "vipsinaapp.com",
        "siteleaf.net",
        "bounty-full.com",
        "alpha.bounty-full.com",
        "beta.bounty-full.com",
        "stackhero-network.com",
        "static.land",
        "dev.static.land",
        "sites.static.land",
        "apps.lair.io",
        "*.stolos.io",
        "spacekit.io",
        "customer.speedpartner.de",
        "api.stdlib.com",
        "storj.farm",
        "utwente.io",
        "soc.srcf.net",
        "user.srcf.net",
        "temp-dns.com",
        "applicationcloud.io",
        "scapp.io",
        "*.s5y.io",
        "*.sensiosite.cloud",
        "syncloud.it",
        "diskstation.me",
        "dscloud.biz",
        "dscloud.me",
        "dscloud.mobi",
        "dsmynas.com",
        "dsmynas.net",
        "dsmynas.org",
        "familyds.com",
        "familyds.net",
        "familyds.org",
        "i234.me",
        "myds.me",
        "synology.me",
        "vpnplus.to",
        "direct.quickconnect.to",
        "taifun-dns.de",
        "gda.pl",
        "gdansk.pl",
        "gdynia.pl",
        "med.pl",
        "sopot.pl",
        "edugit.org",
        "telebit.app",
        "telebit.io",
        "*.telebit.xyz",
        "gwiddle.co.uk",
        "thingdustdata.com",
        "cust.dev.thingdust.io",
        "cust.disrec.thingdust.io",
        "cust.prod.thingdust.io",
        "cust.testing.thingdust.io",
        "arvo.network",
        "azimuth.network",
        "bloxcms.com",
        "townnews-staging.com",
        "12hp.at",
        "2ix.at",
        "4lima.at",
        "lima-city.at",
        "12hp.ch",
        "2ix.ch",
        "4lima.ch",
        "lima-city.ch",
        "trafficplex.cloud",
        "de.cool",
        "12hp.de",
        "2ix.de",
        "4lima.de",
        "lima-city.de",
        "1337.pictures",
        "clan.rip",
        "lima-city.rocks",
        "webspace.rocks",
        "lima.zone",
        "*.transurl.be",
        "*.transurl.eu",
        "*.transurl.nl",
        "tuxfamily.org",
        "dd-dns.de",
        "diskstation.eu",
        "diskstation.org",
        "dray-dns.de",
        "draydns.de",
        "dyn-vpn.de",
        "dynvpn.de",
        "mein-vigor.de",
        "my-vigor.de",
        "my-wan.de",
        "syno-ds.de",
        "synology-diskstation.de",
        "synology-ds.de",
        "uber.space",
        "*.uberspace.de",
        "hk.com",
        "hk.org",
        "ltd.hk",
        "inc.hk",
        "virtualuser.de",
        "virtual-user.de",
        "lib.de.us",
        "2038.io",
        "router.management",
        "v-info.info",
        "voorloper.cloud",
        "v.ua",
        "wafflecell.com",
        "*.webhare.dev",
        "wedeploy.io",
        "wedeploy.me",
        "wedeploy.sh",
        "remotewd.com",
        "wmflabs.org",
        "half.host",
        "xnbay.com",
        "u2.xnbay.com",
        "u2-local.xnbay.com",
        "cistron.nl",
        "demon.nl",
        "xs4all.space",
        "yandexcloud.net",
        "storage.yandexcloud.net",
        "website.yandexcloud.net",
        "official.academy",
        "yolasite.com",
        "ybo.faith",
        "yombo.me",
        "homelink.one",
        "ybo.party",
        "ybo.review",
        "ybo.science",
        "ybo.trade",
        "nohost.me",
        "noho.st",
        "za.net",
        "za.org",
        "now.sh",
        "bss.design",
        "basicserver.io",
        "virtualserver.io",
        "site.builder.nu",
        "enterprisecloud.nu"
    ];
},
/* tough-cookie/lib/store.js */ function _(require, module, exports) {
    /*jshint unused:false */
    function Store() {
    }
    exports.Store = Store;
    // Stores may be synchronous, but are still required to use a
    // Continuation-Passing Style API.  The CookieJar itself will expose a "*Sync"
    // API that converts from synchronous-callbacks to imperative style.
    Store.prototype.synchronous = false;
    Store.prototype.findCookie = function (domain, path, key, cb) {
        throw new Error('findCookie is not implemented');
    };
    Store.prototype.findCookies = function (domain, path, cb) {
        throw new Error('findCookies is not implemented');
    };
    Store.prototype.putCookie = function (cookie, cb) {
        throw new Error('putCookie is not implemented');
    };
    Store.prototype.updateCookie = function (oldCookie, newCookie, cb) {
        // recommended default implementation:
        // return this.putCookie(newCookie, cb);
        throw new Error('updateCookie is not implemented');
    };
    Store.prototype.removeCookie = function (domain, path, key, cb) {
        throw new Error('removeCookie is not implemented');
    };
    Store.prototype.removeCookies = function (domain, path, cb) {
        throw new Error('removeCookies is not implemented');
    };
    Store.prototype.removeAllCookies = function (cb) {
        throw new Error('removeAllCookies is not implemented');
    };
    Store.prototype.getAllCookies = function (cb) {
        throw new Error('getAllCookies is not implemented (therefore jar cannot be serialized)');
    };
},
/* tough-cookie/lib/memstore.js */ function _(require, module, exports) {
    var Store = require(840) /* ./store */.Store;
    var permuteDomain = require(842) /* ./permuteDomain */.permuteDomain;
    var pathMatch = require(843) /* ./pathMatch */.pathMatch;
    var util = require('util');
    function MemoryCookieStore() {
        Store.call(this);
        this.idx = {};
    }
    util.inherits(MemoryCookieStore, Store);
    exports.MemoryCookieStore = MemoryCookieStore;
    MemoryCookieStore.prototype.idx = null;
    // Since it's just a struct in RAM, this Store is synchronous
    MemoryCookieStore.prototype.synchronous = true;
    // force a default depth:
    MemoryCookieStore.prototype.inspect = function () {
        return "{ idx: " + util.inspect(this.idx, false, 2) + ' }';
    };
    // Use the new custom inspection symbol to add the custom inspect function if
    // available.
    if (util.inspect.custom) {
        MemoryCookieStore.prototype[util.inspect.custom] = MemoryCookieStore.prototype.inspect;
    }
    MemoryCookieStore.prototype.findCookie = function (domain, path, key, cb) {
        if (!this.idx[domain]) {
            return cb(null, undefined);
        }
        if (!this.idx[domain][path]) {
            return cb(null, undefined);
        }
        return cb(null, this.idx[domain][path][key] || null);
    };
    MemoryCookieStore.prototype.findCookies = function (domain, path, cb) {
        var results = [];
        if (!domain) {
            return cb(null, []);
        }
        var pathMatcher;
        if (!path) {
            // null means "all paths"
            pathMatcher = function matchAll(domainIndex) {
                for (var curPath in domainIndex) {
                    var pathIndex = domainIndex[curPath];
                    for (var key in pathIndex) {
                        results.push(pathIndex[key]);
                    }
                }
            };
        }
        else {
            pathMatcher = function matchRFC(domainIndex) {
                //NOTE: we should use path-match algorithm from S5.1.4 here
                //(see : https://github.com/ChromiumWebApps/chromium/blob/b3d3b4da8bb94c1b2e061600df106d590fda3620/net/cookies/canonical_cookie.cc#L299)
                Object.keys(domainIndex).forEach(function (cookiePath) {
                    if (pathMatch(path, cookiePath)) {
                        var pathIndex = domainIndex[cookiePath];
                        for (var key in pathIndex) {
                            results.push(pathIndex[key]);
                        }
                    }
                });
            };
        }
        var domains = permuteDomain(domain) || [domain];
        var idx = this.idx;
        domains.forEach(function (curDomain) {
            var domainIndex = idx[curDomain];
            if (!domainIndex) {
                return;
            }
            pathMatcher(domainIndex);
        });
        cb(null, results);
    };
    MemoryCookieStore.prototype.putCookie = function (cookie, cb) {
        if (!this.idx[cookie.domain]) {
            this.idx[cookie.domain] = {};
        }
        if (!this.idx[cookie.domain][cookie.path]) {
            this.idx[cookie.domain][cookie.path] = {};
        }
        this.idx[cookie.domain][cookie.path][cookie.key] = cookie;
        cb(null);
    };
    MemoryCookieStore.prototype.updateCookie = function (oldCookie, newCookie, cb) {
        // updateCookie() may avoid updating cookies that are identical.  For example,
        // lastAccessed may not be important to some stores and an equality
        // comparison could exclude that field.
        this.putCookie(newCookie, cb);
    };
    MemoryCookieStore.prototype.removeCookie = function (domain, path, key, cb) {
        if (this.idx[domain] && this.idx[domain][path] && this.idx[domain][path][key]) {
            delete this.idx[domain][path][key];
        }
        cb(null);
    };
    MemoryCookieStore.prototype.removeCookies = function (domain, path, cb) {
        if (this.idx[domain]) {
            if (path) {
                delete this.idx[domain][path];
            }
            else {
                delete this.idx[domain];
            }
        }
        return cb(null);
    };
    MemoryCookieStore.prototype.removeAllCookies = function (cb) {
        this.idx = {};
        return cb(null);
    };
    MemoryCookieStore.prototype.getAllCookies = function (cb) {
        var cookies = [];
        var idx = this.idx;
        var domains = Object.keys(idx);
        domains.forEach(function (domain) {
            var paths = Object.keys(idx[domain]);
            paths.forEach(function (path) {
                var keys = Object.keys(idx[domain][path]);
                keys.forEach(function (key) {
                    if (key !== null) {
                        cookies.push(idx[domain][path][key]);
                    }
                });
            });
        });
        // Sort by creationIndex so deserializing retains the creation order.
        // When implementing your own store, this SHOULD retain the order too
        cookies.sort(function (a, b) {
            return (a.creationIndex || 0) - (b.creationIndex || 0);
        });
        cb(null, cookies);
    };
},
/* tough-cookie/lib/permuteDomain.js */ function _(require, module, exports) {
    var pubsuffix = require(837) /* ./pubsuffix-psl */;
    // Gives the permutation of all possible domainMatch()es of a given domain. The
    // array is in shortest-to-longest order.  Handy for indexing.
    function permuteDomain(domain) {
        var pubSuf = pubsuffix.getPublicSuffix(domain);
        if (!pubSuf) {
            return null;
        }
        if (pubSuf == domain) {
            return [domain];
        }
        var prefix = domain.slice(0, -(pubSuf.length + 1)); // ".example.com"
        var parts = prefix.split('.').reverse();
        var cur = pubSuf;
        var permutations = [cur];
        while (parts.length) {
            cur = parts.shift() + '.' + cur;
            permutations.push(cur);
        }
        return permutations;
    }
    exports.permuteDomain = permuteDomain;
},
/* tough-cookie/lib/pathMatch.js */ function _(require, module, exports) {
    /*
     * "A request-path path-matches a given cookie-path if at least one of the
     * following conditions holds:"
     */
    function pathMatch(reqPath, cookiePath) {
        // "o  The cookie-path and the request-path are identical."
        if (cookiePath === reqPath) {
            return true;
        }
        var idx = reqPath.indexOf(cookiePath);
        if (idx === 0) {
            // "o  The cookie-path is a prefix of the request-path, and the last
            // character of the cookie-path is %x2F ("/")."
            if (cookiePath.substr(-1) === "/") {
                return true;
            }
            // " o  The cookie-path is a prefix of the request-path, and the first
            // character of the request-path that is not included in the cookie- path
            // is a %x2F ("/") character."
            if (reqPath.substr(cookiePath.length, 1) === "/") {
                return true;
            }
        }
        return false;
    }
    exports.pathMatch = pathMatch;
},
/* tough-cookie/lib/version.js */ function _(require, module, exports) {
    // generated by genversion
    module.exports = '2.5.0';
},
/* request/lib/helpers.js */ function _(require, module, exports) {
    var jsonSafeStringify = require(846) /* json-stringify-safe */;
    var crypto = require('crypto');
    var Buffer = require(70) /* safe-buffer */.Buffer;
    var defer = typeof setImmediate === 'undefined'
        ? process.nextTick
        : setImmediate;
    function paramsHaveRequestBody(params) {
        return (params.body ||
            params.requestBodyStream ||
            (params.json && typeof params.json !== 'boolean') ||
            params.multipart);
    }
    function safeStringify(obj, replacer) {
        var ret;
        try {
            ret = JSON.stringify(obj, replacer);
        }
        catch (e) {
            ret = jsonSafeStringify(obj, replacer);
        }
        return ret;
    }
    function md5(str) {
        return crypto.createHash('md5').update(str).digest('hex');
    }
    function isReadStream(rs) {
        return rs.readable && rs.path && rs.mode;
    }
    function toBase64(str) {
        return Buffer.from(str || '', 'utf8').toString('base64');
    }
    function copy(obj) {
        var o = {};
        Object.keys(obj).forEach(function (i) {
            o[i] = obj[i];
        });
        return o;
    }
    function version() {
        var numbers = process.version.replace('v', '').split('.');
        return {
            major: parseInt(numbers[0], 10),
            minor: parseInt(numbers[1], 10),
            patch: parseInt(numbers[2], 10)
        };
    }
    exports.paramsHaveRequestBody = paramsHaveRequestBody;
    exports.safeStringify = safeStringify;
    exports.md5 = md5;
    exports.isReadStream = isReadStream;
    exports.toBase64 = toBase64;
    exports.copy = copy;
    exports.version = version;
    exports.defer = defer;
},
/* json-stringify-safe/stringify.js */ function _(require, module, exports) {
    exports = module.exports = stringify;
    exports.getSerialize = serializer;
    function stringify(obj, replacer, spaces, cycleReplacer) {
        return JSON.stringify(obj, serializer(replacer, cycleReplacer), spaces);
    }
    function serializer(replacer, cycleReplacer) {
        var stack = [], keys = [];
        if (cycleReplacer == null)
            cycleReplacer = function (key, value) {
                if (stack[0] === value)
                    return "[Circular ~]";
                return "[Circular ~." + keys.slice(0, stack.indexOf(value)).join(".") + "]";
            };
        return function (key, value) {
            if (stack.length > 0) {
                var thisPos = stack.indexOf(this);
                ~thisPos ? stack.splice(thisPos + 1) : stack.push(this);
                ~thisPos ? keys.splice(thisPos, Infinity, key) : keys.push(key);
                if (~stack.indexOf(value))
                    value = cycleReplacer.call(this, key, value);
            }
            else
                stack.push(value);
            return replacer == null ? value : replacer.call(this, key, value);
        };
    }
},
/* request/request.js */ function _(require, module, exports) {
    var http = require('http');
    var https = require('https');
    var url = require('url');
    var util = require('util');
    var stream = require('stream');
    var zlib = require('zlib');
    var aws2 = require(848) /* aws-sign2 */;
    var aws4 = require(849) /* aws4 */;
    var httpSignature = require(851) /* http-signature */;
    var mime = require(900) /* mime-types */;
    var caseless = require(903) /* caseless */;
    var ForeverAgent = require(904) /* forever-agent */;
    var FormData = require(905) /* form-data */;
    var extend = require(834) /* extend */;
    var isstream = require(919) /* isstream */;
    var isTypedArray = require(920) /* is-typedarray */.strict;
    var helpers = require(845) /* ./lib/helpers */;
    var cookies = require(835) /* ./lib/cookies */;
    var getProxyFromURI = require(921) /* ./lib/getProxyFromURI */;
    var Querystring = require(922) /* ./lib/querystring */.Querystring;
    var Har = require(928) /* ./lib/har */.Har;
    var Auth = require(951) /* ./lib/auth */.Auth;
    var OAuth = require(955) /* ./lib/oauth */.OAuth;
    var hawk = require(957) /* ./lib/hawk */;
    var Multipart = require(958) /* ./lib/multipart */.Multipart;
    var Redirect = require(959) /* ./lib/redirect */.Redirect;
    var Tunnel = require(960) /* ./lib/tunnel */.Tunnel;
    var now = require(962) /* performance-now */;
    var Buffer = require(70) /* safe-buffer */.Buffer;
    var safeStringify = helpers.safeStringify;
    var isReadStream = helpers.isReadStream;
    var toBase64 = helpers.toBase64;
    var defer = helpers.defer;
    var copy = helpers.copy;
    var version = helpers.version;
    var globalCookieJar = cookies.jar();
    var globalPool = {};
    function filterForNonReserved(reserved, options) {
        // Filter out properties that are not reserved.
        // Reserved values are passed in at call site.
        var object = {};
        for (var i in options) {
            var notReserved = (reserved.indexOf(i) === -1);
            if (notReserved) {
                object[i] = options[i];
            }
        }
        return object;
    }
    function filterOutReservedFunctions(reserved, options) {
        // Filter out properties that are functions and are reserved.
        // Reserved values are passed in at call site.
        var object = {};
        for (var i in options) {
            var isReserved = !(reserved.indexOf(i) === -1);
            var isFunction = (typeof options[i] === 'function');
            if (!(isReserved && isFunction)) {
                object[i] = options[i];
            }
        }
        return object;
    }
    // Return a simpler request object to allow serialization
    function requestToJSON() {
        var self = this;
        return {
            uri: self.uri,
            method: self.method,
            headers: self.headers
        };
    }
    // Return a simpler response object to allow serialization
    function responseToJSON() {
        var self = this;
        return {
            statusCode: self.statusCode,
            body: self.body,
            headers: self.headers,
            request: requestToJSON.call(self.request)
        };
    }
    function Request(options) {
        // if given the method property in options, set property explicitMethod to true
        // extend the Request instance with any non-reserved properties
        // remove any reserved functions from the options object
        // set Request instance to be readable and writable
        // call init
        var self = this;
        // start with HAR, then override with additional options
        if (options.har) {
            self._har = new Har(self);
            options = self._har.options(options);
        }
        stream.Stream.call(self);
        var reserved = Object.keys(Request.prototype);
        var nonReserved = filterForNonReserved(reserved, options);
        extend(self, nonReserved);
        options = filterOutReservedFunctions(reserved, options);
        self.readable = true;
        self.writable = true;
        if (options.method) {
            self.explicitMethod = true;
        }
        self._qs = new Querystring(self);
        self._auth = new Auth(self);
        self._oauth = new OAuth(self);
        self._multipart = new Multipart(self);
        self._redirect = new Redirect(self);
        self._tunnel = new Tunnel(self);
        self.init(options);
    }
    util.inherits(Request, stream.Stream);
    // Debugging
    Request.debug = process.env.NODE_DEBUG && /\brequest\b/.test(process.env.NODE_DEBUG);
    function debug() {
        if (Request.debug) {
            console.error('REQUEST %s', util.format.apply(util, arguments));
        }
    }
    Request.prototype.debug = debug;
    Request.prototype.init = function (options) {
        // init() contains all the code to setup the request object.
        // the actual outgoing request is not started until start() is called
        // this function is called from both the constructor and on redirect.
        var self = this;
        if (!options) {
            options = {};
        }
        self.headers = self.headers ? copy(self.headers) : {};
        // Delete headers with value undefined since they break
        // ClientRequest.OutgoingMessage.setHeader in node 0.12
        for (var headerName in self.headers) {
            if (typeof self.headers[headerName] === 'undefined') {
                delete self.headers[headerName];
            }
        }
        caseless.httpify(self, self.headers);
        if (!self.method) {
            self.method = options.method || 'GET';
        }
        if (!self.localAddress) {
            self.localAddress = options.localAddress;
        }
        self._qs.init(options);
        debug(options);
        if (!self.pool && self.pool !== false) {
            self.pool = globalPool;
        }
        self.dests = self.dests || [];
        self.__isRequestRequest = true;
        // Protect against double callback
        if (!self._callback && self.callback) {
            self._callback = self.callback;
            self.callback = function () {
                if (self._callbackCalled) {
                    return; // Print a warning maybe?
                }
                self._callbackCalled = true;
                self._callback.apply(self, arguments);
            };
            self.on('error', self.callback.bind());
            self.on('complete', self.callback.bind(self, null));
        }
        // People use this property instead all the time, so support it
        if (!self.uri && self.url) {
            self.uri = self.url;
            delete self.url;
        }
        // If there's a baseUrl, then use it as the base URL (i.e. uri must be
        // specified as a relative path and is appended to baseUrl).
        if (self.baseUrl) {
            if (typeof self.baseUrl !== 'string') {
                return self.emit('error', new Error('options.baseUrl must be a string'));
            }
            if (typeof self.uri !== 'string') {
                return self.emit('error', new Error('options.uri must be a string when using options.baseUrl'));
            }
            if (self.uri.indexOf('//') === 0 || self.uri.indexOf('://') !== -1) {
                return self.emit('error', new Error('options.uri must be a path when using options.baseUrl'));
            }
            // Handle all cases to make sure that there's only one slash between
            // baseUrl and uri.
            var baseUrlEndsWithSlash = self.baseUrl.lastIndexOf('/') === self.baseUrl.length - 1;
            var uriStartsWithSlash = self.uri.indexOf('/') === 0;
            if (baseUrlEndsWithSlash && uriStartsWithSlash) {
                self.uri = self.baseUrl + self.uri.slice(1);
            }
            else if (baseUrlEndsWithSlash || uriStartsWithSlash) {
                self.uri = self.baseUrl + self.uri;
            }
            else if (self.uri === '') {
                self.uri = self.baseUrl;
            }
            else {
                self.uri = self.baseUrl + '/' + self.uri;
            }
            delete self.baseUrl;
        }
        // A URI is needed by this point, emit error if we haven't been able to get one
        if (!self.uri) {
            return self.emit('error', new Error('options.uri is a required argument'));
        }
        // If a string URI/URL was given, parse it into a URL object
        if (typeof self.uri === 'string') {
            self.uri = url.parse(self.uri);
        }
        // Some URL objects are not from a URL parsed string and need href added
        if (!self.uri.href) {
            self.uri.href = url.format(self.uri);
        }
        // DEPRECATED: Warning for users of the old Unix Sockets URL Scheme
        if (self.uri.protocol === 'unix:') {
            return self.emit('error', new Error('`unix://` URL scheme is no longer supported. Please use the format `http://unix:SOCKET:PATH`'));
        }
        // Support Unix Sockets
        if (self.uri.host === 'unix') {
            self.enableUnixSocket();
        }
        if (self.strictSSL === false) {
            self.rejectUnauthorized = false;
        }
        if (!self.uri.pathname) {
            self.uri.pathname = '/';
        }
        if (!(self.uri.host || (self.uri.hostname && self.uri.port)) && !self.uri.isUnix) {
            // Invalid URI: it may generate lot of bad errors, like 'TypeError: Cannot call method `indexOf` of undefined' in CookieJar
            // Detect and reject it as soon as possible
            var faultyUri = url.format(self.uri);
            var message = 'Invalid URI "' + faultyUri + '"';
            if (Object.keys(options).length === 0) {
                // No option ? This can be the sign of a redirect
                // As this is a case where the user cannot do anything (they didn't call request directly with this URL)
                // they should be warned that it can be caused by a redirection (can save some hair)
                message += '. This can be caused by a crappy redirection.';
            }
            // This error was fatal
            self.abort();
            return self.emit('error', new Error(message));
        }
        if (!self.hasOwnProperty('proxy')) {
            self.proxy = getProxyFromURI(self.uri);
        }
        self.tunnel = self._tunnel.isEnabled();
        if (self.proxy) {
            self._tunnel.setup(options);
        }
        self._redirect.onRequest(options);
        self.setHost = false;
        if (!self.hasHeader('host')) {
            var hostHeaderName = self.originalHostHeaderName || 'host';
            self.setHeader(hostHeaderName, self.uri.host);
            // Drop :port suffix from Host header if known protocol.
            if (self.uri.port) {
                if ((self.uri.port === '80' && self.uri.protocol === 'http:') ||
                    (self.uri.port === '443' && self.uri.protocol === 'https:')) {
                    self.setHeader(hostHeaderName, self.uri.hostname);
                }
            }
            self.setHost = true;
        }
        self.jar(self._jar || options.jar);
        if (!self.uri.port) {
            if (self.uri.protocol === 'http:') {
                self.uri.port = 80;
            }
            else if (self.uri.protocol === 'https:') {
                self.uri.port = 443;
            }
        }
        if (self.proxy && !self.tunnel) {
            self.port = self.proxy.port;
            self.host = self.proxy.hostname;
        }
        else {
            self.port = self.uri.port;
            self.host = self.uri.hostname;
        }
        if (options.form) {
            self.form(options.form);
        }
        if (options.formData) {
            var formData = options.formData;
            var requestForm = self.form();
            var appendFormValue = function (key, value) {
                if (value && value.hasOwnProperty('value') && value.hasOwnProperty('options')) {
                    requestForm.append(key, value.value, value.options);
                }
                else {
                    requestForm.append(key, value);
                }
            };
            for (var formKey in formData) {
                if (formData.hasOwnProperty(formKey)) {
                    var formValue = formData[formKey];
                    if (formValue instanceof Array) {
                        for (var j = 0; j < formValue.length; j++) {
                            appendFormValue(formKey, formValue[j]);
                        }
                    }
                    else {
                        appendFormValue(formKey, formValue);
                    }
                }
            }
        }
        if (options.qs) {
            self.qs(options.qs);
        }
        if (self.uri.path) {
            self.path = self.uri.path;
        }
        else {
            self.path = self.uri.pathname + (self.uri.search || '');
        }
        if (self.path.length === 0) {
            self.path = '/';
        }
        // Auth must happen last in case signing is dependent on other headers
        if (options.aws) {
            self.aws(options.aws);
        }
        if (options.hawk) {
            self.hawk(options.hawk);
        }
        if (options.httpSignature) {
            self.httpSignature(options.httpSignature);
        }
        if (options.auth) {
            if (Object.prototype.hasOwnProperty.call(options.auth, 'username')) {
                options.auth.user = options.auth.username;
            }
            if (Object.prototype.hasOwnProperty.call(options.auth, 'password')) {
                options.auth.pass = options.auth.password;
            }
            self.auth(options.auth.user, options.auth.pass, options.auth.sendImmediately, options.auth.bearer);
        }
        if (self.gzip && !self.hasHeader('accept-encoding')) {
            self.setHeader('accept-encoding', 'gzip, deflate');
        }
        if (self.uri.auth && !self.hasHeader('authorization')) {
            var uriAuthPieces = self.uri.auth.split(':').map(function (item) { return self._qs.unescape(item); });
            self.auth(uriAuthPieces[0], uriAuthPieces.slice(1).join(':'), true);
        }
        if (!self.tunnel && self.proxy && self.proxy.auth && !self.hasHeader('proxy-authorization')) {
            var proxyAuthPieces = self.proxy.auth.split(':').map(function (item) { return self._qs.unescape(item); });
            var authHeader = 'Basic ' + toBase64(proxyAuthPieces.join(':'));
            self.setHeader('proxy-authorization', authHeader);
        }
        if (self.proxy && !self.tunnel) {
            self.path = (self.uri.protocol + '//' + self.uri.host + self.path);
        }
        if (options.json) {
            self.json(options.json);
        }
        if (options.multipart) {
            self.multipart(options.multipart);
        }
        if (options.time) {
            self.timing = true;
            // NOTE: elapsedTime is deprecated in favor of .timings
            self.elapsedTime = self.elapsedTime || 0;
        }
        function setContentLength() {
            if (isTypedArray(self.body)) {
                self.body = Buffer.from(self.body);
            }
            if (!self.hasHeader('content-length')) {
                var length;
                if (typeof self.body === 'string') {
                    length = Buffer.byteLength(self.body);
                }
                else if (Array.isArray(self.body)) {
                    length = self.body.reduce(function (a, b) { return a + b.length; }, 0);
                }
                else {
                    length = self.body.length;
                }
                if (length) {
                    self.setHeader('content-length', length);
                }
                else {
                    self.emit('error', new Error('Argument error, options.body.'));
                }
            }
        }
        if (self.body && !isstream(self.body)) {
            setContentLength();
        }
        if (options.oauth) {
            self.oauth(options.oauth);
        }
        else if (self._oauth.params && self.hasHeader('authorization')) {
            self.oauth(self._oauth.params);
        }
        var protocol = self.proxy && !self.tunnel ? self.proxy.protocol : self.uri.protocol;
        var defaultModules = { 'http:': http, 'https:': https };
        var httpModules = self.httpModules || {};
        self.httpModule = httpModules[protocol] || defaultModules[protocol];
        if (!self.httpModule) {
            return self.emit('error', new Error('Invalid protocol: ' + protocol));
        }
        if (options.ca) {
            self.ca = options.ca;
        }
        if (!self.agent) {
            if (options.agentOptions) {
                self.agentOptions = options.agentOptions;
            }
            if (options.agentClass) {
                self.agentClass = options.agentClass;
            }
            else if (options.forever) {
                var v = version();
                // use ForeverAgent in node 0.10- only
                if (v.major === 0 && v.minor <= 10) {
                    self.agentClass = protocol === 'http:' ? ForeverAgent : ForeverAgent.SSL;
                }
                else {
                    self.agentClass = self.httpModule.Agent;
                    self.agentOptions = self.agentOptions || {};
                    self.agentOptions.keepAlive = true;
                }
            }
            else {
                self.agentClass = self.httpModule.Agent;
            }
        }
        if (self.pool === false) {
            self.agent = false;
        }
        else {
            self.agent = self.agent || self.getNewAgent();
        }
        self.on('pipe', function (src) {
            if (self.ntick && self._started) {
                self.emit('error', new Error('You cannot pipe to this stream after the outbound request has started.'));
            }
            self.src = src;
            if (isReadStream(src)) {
                if (!self.hasHeader('content-type')) {
                    self.setHeader('content-type', mime.lookup(src.path));
                }
            }
            else {
                if (src.headers) {
                    for (var i in src.headers) {
                        if (!self.hasHeader(i)) {
                            self.setHeader(i, src.headers[i]);
                        }
                    }
                }
                if (self._json && !self.hasHeader('content-type')) {
                    self.setHeader('content-type', 'application/json');
                }
                if (src.method && !self.explicitMethod) {
                    self.method = src.method;
                }
            }
            // self.on('pipe', function () {
            //   console.error('You have already piped to this stream. Pipeing twice is likely to break the request.')
            // })
        });
        defer(function () {
            if (self._aborted) {
                return;
            }
            var end = function () {
                if (self._form) {
                    if (!self._auth.hasAuth) {
                        self._form.pipe(self);
                    }
                    else if (self._auth.hasAuth && self._auth.sentAuth) {
                        self._form.pipe(self);
                    }
                }
                if (self._multipart && self._multipart.chunked) {
                    self._multipart.body.pipe(self);
                }
                if (self.body) {
                    if (isstream(self.body)) {
                        self.body.pipe(self);
                    }
                    else {
                        setContentLength();
                        if (Array.isArray(self.body)) {
                            self.body.forEach(function (part) {
                                self.write(part);
                            });
                        }
                        else {
                            self.write(self.body);
                        }
                        self.end();
                    }
                }
                else if (self.requestBodyStream) {
                    console.warn('options.requestBodyStream is deprecated, please pass the request object to stream.pipe.');
                    self.requestBodyStream.pipe(self);
                }
                else if (!self.src) {
                    if (self._auth.hasAuth && !self._auth.sentAuth) {
                        self.end();
                        return;
                    }
                    if (self.method !== 'GET' && typeof self.method !== 'undefined') {
                        self.setHeader('content-length', 0);
                    }
                    self.end();
                }
            };
            if (self._form && !self.hasHeader('content-length')) {
                // Before ending the request, we had to compute the length of the whole form, asyncly
                self.setHeader(self._form.getHeaders(), true);
                self._form.getLength(function (err, length) {
                    if (!err && !isNaN(length)) {
                        self.setHeader('content-length', length);
                    }
                    end();
                });
            }
            else {
                end();
            }
            self.ntick = true;
        });
    };
    Request.prototype.getNewAgent = function () {
        var self = this;
        var Agent = self.agentClass;
        var options = {};
        if (self.agentOptions) {
            for (var i in self.agentOptions) {
                options[i] = self.agentOptions[i];
            }
        }
        if (self.ca) {
            options.ca = self.ca;
        }
        if (self.ciphers) {
            options.ciphers = self.ciphers;
        }
        if (self.secureProtocol) {
            options.secureProtocol = self.secureProtocol;
        }
        if (self.secureOptions) {
            options.secureOptions = self.secureOptions;
        }
        if (typeof self.rejectUnauthorized !== 'undefined') {
            options.rejectUnauthorized = self.rejectUnauthorized;
        }
        if (self.cert && self.key) {
            options.key = self.key;
            options.cert = self.cert;
        }
        if (self.pfx) {
            options.pfx = self.pfx;
        }
        if (self.passphrase) {
            options.passphrase = self.passphrase;
        }
        var poolKey = '';
        // different types of agents are in different pools
        if (Agent !== self.httpModule.Agent) {
            poolKey += Agent.name;
        }
        // ca option is only relevant if proxy or destination are https
        var proxy = self.proxy;
        if (typeof proxy === 'string') {
            proxy = url.parse(proxy);
        }
        var isHttps = (proxy && proxy.protocol === 'https:') || this.uri.protocol === 'https:';
        if (isHttps) {
            if (options.ca) {
                if (poolKey) {
                    poolKey += ':';
                }
                poolKey += options.ca;
            }
            if (typeof options.rejectUnauthorized !== 'undefined') {
                if (poolKey) {
                    poolKey += ':';
                }
                poolKey += options.rejectUnauthorized;
            }
            if (options.cert) {
                if (poolKey) {
                    poolKey += ':';
                }
                poolKey += options.cert.toString('ascii') + options.key.toString('ascii');
            }
            if (options.pfx) {
                if (poolKey) {
                    poolKey += ':';
                }
                poolKey += options.pfx.toString('ascii');
            }
            if (options.ciphers) {
                if (poolKey) {
                    poolKey += ':';
                }
                poolKey += options.ciphers;
            }
            if (options.secureProtocol) {
                if (poolKey) {
                    poolKey += ':';
                }
                poolKey += options.secureProtocol;
            }
            if (options.secureOptions) {
                if (poolKey) {
                    poolKey += ':';
                }
                poolKey += options.secureOptions;
            }
        }
        if (self.pool === globalPool && !poolKey && Object.keys(options).length === 0 && self.httpModule.globalAgent) {
            // not doing anything special.  Use the globalAgent
            return self.httpModule.globalAgent;
        }
        // we're using a stored agent.  Make sure it's protocol-specific
        poolKey = self.uri.protocol + poolKey;
        // generate a new agent for this setting if none yet exists
        if (!self.pool[poolKey]) {
            self.pool[poolKey] = new Agent(options);
            // properly set maxSockets on new agents
            if (self.pool.maxSockets) {
                self.pool[poolKey].maxSockets = self.pool.maxSockets;
            }
        }
        return self.pool[poolKey];
    };
    Request.prototype.start = function () {
        // start() is called once we are ready to send the outgoing HTTP request.
        // this is usually called on the first write(), end() or on nextTick()
        var self = this;
        if (self.timing) {
            // All timings will be relative to this request's startTime.  In order to do this,
            // we need to capture the wall-clock start time (via Date), immediately followed
            // by the high-resolution timer (via now()).  While these two won't be set
            // at the _exact_ same time, they should be close enough to be able to calculate
            // high-resolution, monotonically non-decreasing timestamps relative to startTime.
            var startTime = new Date().getTime();
            var startTimeNow = now();
        }
        if (self._aborted) {
            return;
        }
        self._started = true;
        self.method = self.method || 'GET';
        self.href = self.uri.href;
        if (self.src && self.src.stat && self.src.stat.size && !self.hasHeader('content-length')) {
            self.setHeader('content-length', self.src.stat.size);
        }
        if (self._aws) {
            self.aws(self._aws, true);
        }
        // We have a method named auth, which is completely different from the http.request
        // auth option.  If we don't remove it, we're gonna have a bad time.
        var reqOptions = copy(self);
        delete reqOptions.auth;
        debug('make request', self.uri.href);
        // node v6.8.0 now supports a `timeout` value in `http.request()`, but we
        // should delete it for now since we handle timeouts manually for better
        // consistency with node versions before v6.8.0
        delete reqOptions.timeout;
        try {
            self.req = self.httpModule.request(reqOptions);
        }
        catch (err) {
            self.emit('error', err);
            return;
        }
        if (self.timing) {
            self.startTime = startTime;
            self.startTimeNow = startTimeNow;
            // Timing values will all be relative to startTime (by comparing to startTimeNow
            // so we have an accurate clock)
            self.timings = {};
        }
        var timeout;
        if (self.timeout && !self.timeoutTimer) {
            if (self.timeout < 0) {
                timeout = 0;
            }
            else if (typeof self.timeout === 'number' && isFinite(self.timeout)) {
                timeout = self.timeout;
            }
        }
        self.req.on('response', self.onRequestResponse.bind(self));
        self.req.on('error', self.onRequestError.bind(self));
        self.req.on('drain', function () {
            self.emit('drain');
        });
        self.req.on('socket', function (socket) {
            // `._connecting` was the old property which was made public in node v6.1.0
            var isConnecting = socket._connecting || socket.connecting;
            if (self.timing) {
                self.timings.socket = now() - self.startTimeNow;
                if (isConnecting) {
                    var onLookupTiming = function () {
                        self.timings.lookup = now() - self.startTimeNow;
                    };
                    var onConnectTiming = function () {
                        self.timings.connect = now() - self.startTimeNow;
                    };
                    socket.once('lookup', onLookupTiming);
                    socket.once('connect', onConnectTiming);
                    // clean up timing event listeners if needed on error
                    self.req.once('error', function () {
                        socket.removeListener('lookup', onLookupTiming);
                        socket.removeListener('connect', onConnectTiming);
                    });
                }
            }
            var setReqTimeout = function () {
                // This timeout sets the amount of time to wait *between* bytes sent
                // from the server once connected.
                //
                // In particular, it's useful for erroring if the server fails to send
                // data halfway through streaming a response.
                self.req.setTimeout(timeout, function () {
                    if (self.req) {
                        self.abort();
                        var e = new Error('ESOCKETTIMEDOUT');
                        e.code = 'ESOCKETTIMEDOUT';
                        e.connect = false;
                        self.emit('error', e);
                    }
                });
            };
            if (timeout !== undefined) {
                // Only start the connection timer if we're actually connecting a new
                // socket, otherwise if we're already connected (because this is a
                // keep-alive connection) do not bother. This is important since we won't
                // get a 'connect' event for an already connected socket.
                if (isConnecting) {
                    var onReqSockConnect = function () {
                        socket.removeListener('connect', onReqSockConnect);
                        self.clearTimeout();
                        setReqTimeout();
                    };
                    socket.on('connect', onReqSockConnect);
                    self.req.on('error', function (err) {
                        socket.removeListener('connect', onReqSockConnect);
                    });
                    // Set a timeout in memory - this block will throw if the server takes more
                    // than `timeout` to write the HTTP status and headers (corresponding to
                    // the on('response') event on the client). NB: this measures wall-clock
                    // time, not the time between bytes sent by the server.
                    self.timeoutTimer = setTimeout(function () {
                        socket.removeListener('connect', onReqSockConnect);
                        self.abort();
                        var e = new Error('ETIMEDOUT');
                        e.code = 'ETIMEDOUT';
                        e.connect = true;
                        self.emit('error', e);
                    }, timeout);
                }
                else {
                    // We're already connected
                    setReqTimeout();
                }
            }
            self.emit('socket', socket);
        });
        self.emit('request', self.req);
    };
    Request.prototype.onRequestError = function (error) {
        var self = this;
        if (self._aborted) {
            return;
        }
        if (self.req && self.req._reusedSocket && error.code === 'ECONNRESET' &&
            self.agent.addRequestNoreuse) {
            self.agent = { addRequest: self.agent.addRequestNoreuse.bind(self.agent) };
            self.start();
            self.req.end();
            return;
        }
        self.clearTimeout();
        self.emit('error', error);
    };
    Request.prototype.onRequestResponse = function (response) {
        var self = this;
        if (self.timing) {
            self.timings.response = now() - self.startTimeNow;
        }
        debug('onRequestResponse', self.uri.href, response.statusCode, response.headers);
        response.on('end', function () {
            if (self.timing) {
                self.timings.end = now() - self.startTimeNow;
                response.timingStart = self.startTime;
                // fill in the blanks for any periods that didn't trigger, such as
                // no lookup or connect due to keep alive
                if (!self.timings.socket) {
                    self.timings.socket = 0;
                }
                if (!self.timings.lookup) {
                    self.timings.lookup = self.timings.socket;
                }
                if (!self.timings.connect) {
                    self.timings.connect = self.timings.lookup;
                }
                if (!self.timings.response) {
                    self.timings.response = self.timings.connect;
                }
                debug('elapsed time', self.timings.end);
                // elapsedTime includes all redirects
                self.elapsedTime += Math.round(self.timings.end);
                // NOTE: elapsedTime is deprecated in favor of .timings
                response.elapsedTime = self.elapsedTime;
                // timings is just for the final fetch
                response.timings = self.timings;
                // pre-calculate phase timings as well
                response.timingPhases = {
                    wait: self.timings.socket,
                    dns: self.timings.lookup - self.timings.socket,
                    tcp: self.timings.connect - self.timings.lookup,
                    firstByte: self.timings.response - self.timings.connect,
                    download: self.timings.end - self.timings.response,
                    total: self.timings.end
                };
            }
            debug('response end', self.uri.href, response.statusCode, response.headers);
        });
        if (self._aborted) {
            debug('aborted', self.uri.href);
            response.resume();
            return;
        }
        self.response = response;
        response.request = self;
        response.toJSON = responseToJSON;
        // XXX This is different on 0.10, because SSL is strict by default
        if (self.httpModule === https &&
            self.strictSSL && (!response.hasOwnProperty('socket') ||
            !response.socket.authorized)) {
            debug('strict ssl error', self.uri.href);
            var sslErr = response.hasOwnProperty('socket') ? response.socket.authorizationError : self.uri.href + ' does not support SSL';
            self.emit('error', new Error('SSL Error: ' + sslErr));
            return;
        }
        // Save the original host before any redirect (if it changes, we need to
        // remove any authorization headers).  Also remember the case of the header
        // name because lots of broken servers expect Host instead of host and we
        // want the caller to be able to specify this.
        self.originalHost = self.getHeader('host');
        if (!self.originalHostHeaderName) {
            self.originalHostHeaderName = self.hasHeader('host');
        }
        if (self.setHost) {
            self.removeHeader('host');
        }
        self.clearTimeout();
        var targetCookieJar = (self._jar && self._jar.setCookie) ? self._jar : globalCookieJar;
        var addCookie = function (cookie) {
            // set the cookie if it's domain in the href's domain.
            try {
                targetCookieJar.setCookie(cookie, self.uri.href, { ignoreError: true });
            }
            catch (e) {
                self.emit('error', e);
            }
        };
        response.caseless = caseless(response.headers);
        if (response.caseless.has('set-cookie') && (!self._disableCookies)) {
            var headerName = response.caseless.has('set-cookie');
            if (Array.isArray(response.headers[headerName])) {
                response.headers[headerName].forEach(addCookie);
            }
            else {
                addCookie(response.headers[headerName]);
            }
        }
        if (self._redirect.onResponse(response)) {
            return; // Ignore the rest of the response
        }
        else {
            // Be a good stream and emit end when the response is finished.
            // Hack to emit end on close because of a core bug that never fires end
            response.on('close', function () {
                if (!self._ended) {
                    self.response.emit('end');
                }
            });
            response.once('end', function () {
                self._ended = true;
            });
            var noBody = function (code) {
                return (self.method === 'HEAD' ||
                    // Informational
                    (code >= 100 && code < 200) ||
                    // No Content
                    code === 204 ||
                    // Not Modified
                    code === 304);
            };
            var responseContent;
            if (self.gzip && !noBody(response.statusCode)) {
                var contentEncoding = response.headers['content-encoding'] || 'identity';
                contentEncoding = contentEncoding.trim().toLowerCase();
                // Be more lenient with decoding compressed responses, since (very rarely)
                // servers send slightly invalid gzip responses that are still accepted
                // by common browsers.
                // Always using Z_SYNC_FLUSH is what cURL does.
                var zlibOptions = {
                    flush: zlib.Z_SYNC_FLUSH,
                    finishFlush: zlib.Z_SYNC_FLUSH
                };
                if (contentEncoding === 'gzip') {
                    responseContent = zlib.createGunzip(zlibOptions);
                    response.pipe(responseContent);
                }
                else if (contentEncoding === 'deflate') {
                    responseContent = zlib.createInflate(zlibOptions);
                    response.pipe(responseContent);
                }
                else {
                    // Since previous versions didn't check for Content-Encoding header,
                    // ignore any invalid values to preserve backwards-compatibility
                    if (contentEncoding !== 'identity') {
                        debug('ignoring unrecognized Content-Encoding ' + contentEncoding);
                    }
                    responseContent = response;
                }
            }
            else {
                responseContent = response;
            }
            if (self.encoding) {
                if (self.dests.length !== 0) {
                    console.error('Ignoring encoding parameter as this stream is being piped to another stream which makes the encoding option invalid.');
                }
                else {
                    responseContent.setEncoding(self.encoding);
                }
            }
            if (self._paused) {
                responseContent.pause();
            }
            self.responseContent = responseContent;
            self.emit('response', response);
            self.dests.forEach(function (dest) {
                self.pipeDest(dest);
            });
            responseContent.on('data', function (chunk) {
                if (self.timing && !self.responseStarted) {
                    self.responseStartTime = (new Date()).getTime();
                    // NOTE: responseStartTime is deprecated in favor of .timings
                    response.responseStartTime = self.responseStartTime;
                }
                self._destdata = true;
                self.emit('data', chunk);
            });
            responseContent.once('end', function (chunk) {
                self.emit('end', chunk);
            });
            responseContent.on('error', function (error) {
                self.emit('error', error);
            });
            responseContent.on('close', function () { self.emit('close'); });
            if (self.callback) {
                self.readResponseBody(response);
            }
            else { // if no callback
                self.on('end', function () {
                    if (self._aborted) {
                        debug('aborted', self.uri.href);
                        return;
                    }
                    self.emit('complete', response);
                });
            }
        }
        debug('finish init function', self.uri.href);
    };
    Request.prototype.readResponseBody = function (response) {
        var self = this;
        debug("reading response's body");
        var buffers = [];
        var bufferLength = 0;
        var strings = [];
        self.on('data', function (chunk) {
            if (!Buffer.isBuffer(chunk)) {
                strings.push(chunk);
            }
            else if (chunk.length) {
                bufferLength += chunk.length;
                buffers.push(chunk);
            }
        });
        self.on('end', function () {
            debug('end event', self.uri.href);
            if (self._aborted) {
                debug('aborted', self.uri.href);
                // `buffer` is defined in the parent scope and used in a closure it exists for the life of the request.
                // This can lead to leaky behavior if the user retains a reference to the request object.
                buffers = [];
                bufferLength = 0;
                return;
            }
            if (bufferLength) {
                debug('has body', self.uri.href, bufferLength);
                response.body = Buffer.concat(buffers, bufferLength);
                if (self.encoding !== null) {
                    response.body = response.body.toString(self.encoding);
                }
                // `buffer` is defined in the parent scope and used in a closure it exists for the life of the Request.
                // This can lead to leaky behavior if the user retains a reference to the request object.
                buffers = [];
                bufferLength = 0;
            }
            else if (strings.length) {
                // The UTF8 BOM [0xEF,0xBB,0xBF] is converted to [0xFE,0xFF] in the JS UTC16/UCS2 representation.
                // Strip this value out when the encoding is set to 'utf8', as upstream consumers won't expect it and it breaks JSON.parse().
                if (self.encoding === 'utf8' && strings[0].length > 0 && strings[0][0] === '\uFEFF') {
                    strings[0] = strings[0].substring(1);
                }
                response.body = strings.join('');
            }
            if (self._json) {
                try {
                    response.body = JSON.parse(response.body, self._jsonReviver);
                }
                catch (e) {
                    debug('invalid JSON received', self.uri.href);
                }
            }
            debug('emitting complete', self.uri.href);
            if (typeof response.body === 'undefined' && !self._json) {
                response.body = self.encoding === null ? Buffer.alloc(0) : '';
            }
            self.emit('complete', response, response.body);
        });
    };
    Request.prototype.abort = function () {
        var self = this;
        self._aborted = true;
        if (self.req) {
            self.req.abort();
        }
        else if (self.response) {
            self.response.destroy();
        }
        self.clearTimeout();
        self.emit('abort');
    };
    Request.prototype.pipeDest = function (dest) {
        var self = this;
        var response = self.response;
        // Called after the response is received
        if (dest.headers && !dest.headersSent) {
            if (response.caseless.has('content-type')) {
                var ctname = response.caseless.has('content-type');
                if (dest.setHeader) {
                    dest.setHeader(ctname, response.headers[ctname]);
                }
                else {
                    dest.headers[ctname] = response.headers[ctname];
                }
            }
            if (response.caseless.has('content-length')) {
                var clname = response.caseless.has('content-length');
                if (dest.setHeader) {
                    dest.setHeader(clname, response.headers[clname]);
                }
                else {
                    dest.headers[clname] = response.headers[clname];
                }
            }
        }
        if (dest.setHeader && !dest.headersSent) {
            for (var i in response.headers) {
                // If the response content is being decoded, the Content-Encoding header
                // of the response doesn't represent the piped content, so don't pass it.
                if (!self.gzip || i !== 'content-encoding') {
                    dest.setHeader(i, response.headers[i]);
                }
            }
            dest.statusCode = response.statusCode;
        }
        if (self.pipefilter) {
            self.pipefilter(response, dest);
        }
    };
    Request.prototype.qs = function (q, clobber) {
        var self = this;
        var base;
        if (!clobber && self.uri.query) {
            base = self._qs.parse(self.uri.query);
        }
        else {
            base = {};
        }
        for (var i in q) {
            base[i] = q[i];
        }
        var qs = self._qs.stringify(base);
        if (qs === '') {
            return self;
        }
        self.uri = url.parse(self.uri.href.split('?')[0] + '?' + qs);
        self.url = self.uri;
        self.path = self.uri.path;
        if (self.uri.host === 'unix') {
            self.enableUnixSocket();
        }
        return self;
    };
    Request.prototype.form = function (form) {
        var self = this;
        if (form) {
            if (!/^application\/x-www-form-urlencoded\b/.test(self.getHeader('content-type'))) {
                self.setHeader('content-type', 'application/x-www-form-urlencoded');
            }
            self.body = (typeof form === 'string')
                ? self._qs.rfc3986(form.toString('utf8'))
                : self._qs.stringify(form).toString('utf8');
            return self;
        }
        // create form-data object
        self._form = new FormData();
        self._form.on('error', function (err) {
            err.message = 'form-data: ' + err.message;
            self.emit('error', err);
            self.abort();
        });
        return self._form;
    };
    Request.prototype.multipart = function (multipart) {
        var self = this;
        self._multipart.onRequest(multipart);
        if (!self._multipart.chunked) {
            self.body = self._multipart.body;
        }
        return self;
    };
    Request.prototype.json = function (val) {
        var self = this;
        if (!self.hasHeader('accept')) {
            self.setHeader('accept', 'application/json');
        }
        if (typeof self.jsonReplacer === 'function') {
            self._jsonReplacer = self.jsonReplacer;
        }
        self._json = true;
        if (typeof val === 'boolean') {
            if (self.body !== undefined) {
                if (!/^application\/x-www-form-urlencoded\b/.test(self.getHeader('content-type'))) {
                    self.body = safeStringify(self.body, self._jsonReplacer);
                }
                else {
                    self.body = self._qs.rfc3986(self.body);
                }
                if (!self.hasHeader('content-type')) {
                    self.setHeader('content-type', 'application/json');
                }
            }
        }
        else {
            self.body = safeStringify(val, self._jsonReplacer);
            if (!self.hasHeader('content-type')) {
                self.setHeader('content-type', 'application/json');
            }
        }
        if (typeof self.jsonReviver === 'function') {
            self._jsonReviver = self.jsonReviver;
        }
        return self;
    };
    Request.prototype.getHeader = function (name, headers) {
        var self = this;
        var result, re, match;
        if (!headers) {
            headers = self.headers;
        }
        Object.keys(headers).forEach(function (key) {
            if (key.length !== name.length) {
                return;
            }
            re = new RegExp(name, 'i');
            match = key.match(re);
            if (match) {
                result = headers[key];
            }
        });
        return result;
    };
    Request.prototype.enableUnixSocket = function () {
        // Get the socket & request paths from the URL
        var unixParts = this.uri.path.split(':');
        var host = unixParts[0];
        var path = unixParts[1];
        // Apply unix properties to request
        this.socketPath = host;
        this.uri.pathname = path;
        this.uri.path = path;
        this.uri.host = host;
        this.uri.hostname = host;
        this.uri.isUnix = true;
    };
    Request.prototype.auth = function (user, pass, sendImmediately, bearer) {
        var self = this;
        self._auth.onRequest(user, pass, sendImmediately, bearer);
        return self;
    };
    Request.prototype.aws = function (opts, now) {
        var self = this;
        if (!now) {
            self._aws = opts;
            return self;
        }
        if (opts.sign_version === 4 || opts.sign_version === '4') {
            // use aws4
            var options = {
                host: self.uri.host,
                path: self.uri.path,
                method: self.method,
                headers: self.headers,
                body: self.body
            };
            if (opts.service) {
                options.service = opts.service;
            }
            var signRes = aws4.sign(options, {
                accessKeyId: opts.key,
                secretAccessKey: opts.secret,
                sessionToken: opts.session
            });
            self.setHeader('authorization', signRes.headers.Authorization);
            self.setHeader('x-amz-date', signRes.headers['X-Amz-Date']);
            if (signRes.headers['X-Amz-Security-Token']) {
                self.setHeader('x-amz-security-token', signRes.headers['X-Amz-Security-Token']);
            }
        }
        else {
            // default: use aws-sign2
            var date = new Date();
            self.setHeader('date', date.toUTCString());
            var auth = {
                key: opts.key,
                secret: opts.secret,
                verb: self.method.toUpperCase(),
                date: date,
                contentType: self.getHeader('content-type') || '',
                md5: self.getHeader('content-md5') || '',
                amazonHeaders: aws2.canonicalizeHeaders(self.headers)
            };
            var path = self.uri.path;
            if (opts.bucket && path) {
                auth.resource = '/' + opts.bucket + path;
            }
            else if (opts.bucket && !path) {
                auth.resource = '/' + opts.bucket;
            }
            else if (!opts.bucket && path) {
                auth.resource = path;
            }
            else if (!opts.bucket && !path) {
                auth.resource = '/';
            }
            auth.resource = aws2.canonicalizeResource(auth.resource);
            self.setHeader('authorization', aws2.authorization(auth));
        }
        return self;
    };
    Request.prototype.httpSignature = function (opts) {
        var self = this;
        httpSignature.signRequest({
            getHeader: function (header) {
                return self.getHeader(header, self.headers);
            },
            setHeader: function (header, value) {
                self.setHeader(header, value);
            },
            method: self.method,
            path: self.path
        }, opts);
        debug('httpSignature authorization', self.getHeader('authorization'));
        return self;
    };
    Request.prototype.hawk = function (opts) {
        var self = this;
        self.setHeader('Authorization', hawk.header(self.uri, self.method, opts));
    };
    Request.prototype.oauth = function (_oauth) {
        var self = this;
        self._oauth.onRequest(_oauth);
        return self;
    };
    Request.prototype.jar = function (jar) {
        var self = this;
        var cookies;
        if (self._redirect.redirectsFollowed === 0) {
            self.originalCookieHeader = self.getHeader('cookie');
        }
        if (!jar) {
            // disable cookies
            cookies = false;
            self._disableCookies = true;
        }
        else {
            var targetCookieJar = jar.getCookieString ? jar : globalCookieJar;
            var urihref = self.uri.href;
            // fetch cookie in the Specified host
            if (targetCookieJar) {
                cookies = targetCookieJar.getCookieString(urihref);
            }
        }
        // if need cookie and cookie is not empty
        if (cookies && cookies.length) {
            if (self.originalCookieHeader) {
                // Don't overwrite existing Cookie header
                self.setHeader('cookie', self.originalCookieHeader + '; ' + cookies);
            }
            else {
                self.setHeader('cookie', cookies);
            }
        }
        self._jar = jar;
        return self;
    };
    // Stream API
    Request.prototype.pipe = function (dest, opts) {
        var self = this;
        if (self.response) {
            if (self._destdata) {
                self.emit('error', new Error('You cannot pipe after data has been emitted from the response.'));
            }
            else if (self._ended) {
                self.emit('error', new Error('You cannot pipe after the response has been ended.'));
            }
            else {
                stream.Stream.prototype.pipe.call(self, dest, opts);
                self.pipeDest(dest);
                return dest;
            }
        }
        else {
            self.dests.push(dest);
            stream.Stream.prototype.pipe.call(self, dest, opts);
            return dest;
        }
    };
    Request.prototype.write = function () {
        var self = this;
        if (self._aborted) {
            return;
        }
        if (!self._started) {
            self.start();
        }
        if (self.req) {
            return self.req.write.apply(self.req, arguments);
        }
    };
    Request.prototype.end = function (chunk) {
        var self = this;
        if (self._aborted) {
            return;
        }
        if (chunk) {
            self.write(chunk);
        }
        if (!self._started) {
            self.start();
        }
        if (self.req) {
            self.req.end();
        }
    };
    Request.prototype.pause = function () {
        var self = this;
        if (!self.responseContent) {
            self._paused = true;
        }
        else {
            self.responseContent.pause.apply(self.responseContent, arguments);
        }
    };
    Request.prototype.resume = function () {
        var self = this;
        if (!self.responseContent) {
            self._paused = false;
        }
        else {
            self.responseContent.resume.apply(self.responseContent, arguments);
        }
    };
    Request.prototype.destroy = function () {
        var self = this;
        this.clearTimeout();
        if (!self._ended) {
            self.end();
        }
        else if (self.response) {
            self.response.destroy();
        }
    };
    Request.prototype.clearTimeout = function () {
        if (this.timeoutTimer) {
            clearTimeout(this.timeoutTimer);
            this.timeoutTimer = null;
        }
    };
    Request.defaultProxyHeaderWhiteList =
        Tunnel.defaultProxyHeaderWhiteList.slice();
    Request.defaultProxyHeaderExclusiveList =
        Tunnel.defaultProxyHeaderExclusiveList.slice();
    // Exports
    Request.prototype.toJSON = requestToJSON;
    module.exports = Request;
},
/* aws-sign2/index.js */ function _(require, module, exports) {
    /*!
     *  Copyright 2010 LearnBoost <dev@learnboost.com>
     *
     * Licensed under the Apache License, Version 2.0 (the "License");
     * you may not use this file except in compliance with the License.
     * You may obtain a copy of the License at
     *
     *     http://www.apache.org/licenses/LICENSE-2.0
     *
     * Unless required by applicable law or agreed to in writing, software
     * distributed under the License is distributed on an "AS IS" BASIS,
     * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
     * See the License for the specific language governing permissions and
     * limitations under the License.
     */
    /**
     * Module dependencies.
     */
    var crypto = require('crypto'), parse = require('url').parse;
    /**
     * Valid keys.
     */
    var keys = ['acl',
        'location',
        'logging',
        'notification',
        'partNumber',
        'policy',
        'requestPayment',
        'torrent',
        'uploadId',
        'uploads',
        'versionId',
        'versioning',
        'versions',
        'website'
    ];
    /**
     * Return an "Authorization" header value with the given `options`
     * in the form of "AWS <key>:<signature>"
     *
     * @param {Object} options
     * @return {String}
     * @api private
     */
    function authorization(options) {
        return 'AWS ' + options.key + ':' + sign(options);
    }
    module.exports = authorization;
    module.exports.authorization = authorization;
    /**
     * Simple HMAC-SHA1 Wrapper
     *
     * @param {Object} options
     * @return {String}
     * @api private
     */
    function hmacSha1(options) {
        return crypto.createHmac('sha1', options.secret).update(options.message).digest('base64');
    }
    module.exports.hmacSha1 = hmacSha1;
    /**
     * Create a base64 sha1 HMAC for `options`.
     *
     * @param {Object} options
     * @return {String}
     * @api private
     */
    function sign(options) {
        options.message = stringToSign(options);
        return hmacSha1(options);
    }
    module.exports.sign = sign;
    /**
     * Create a base64 sha1 HMAC for `options`.
     *
     * Specifically to be used with S3 presigned URLs
     *
     * @param {Object} options
     * @return {String}
     * @api private
     */
    function signQuery(options) {
        options.message = queryStringToSign(options);
        return hmacSha1(options);
    }
    module.exports.signQuery = signQuery;
    /**
     * Return a string for sign() with the given `options`.
     *
     * Spec:
     *
     *    <verb>\n
     *    <md5>\n
     *    <content-type>\n
     *    <date>\n
     *    [headers\n]
     *    <resource>
     *
     * @param {Object} options
     * @return {String}
     * @api private
     */
    function stringToSign(options) {
        var headers = options.amazonHeaders || '';
        if (headers)
            headers += '\n';
        var r = [options.verb,
            options.md5,
            options.contentType,
            options.date ? options.date.toUTCString() : '',
            headers + options.resource
        ];
        return r.join('\n');
    }
    module.exports.stringToSign = stringToSign;
    /**
     * Return a string for sign() with the given `options`, but is meant exclusively
     * for S3 presigned URLs
     *
     * Spec:
     *
     *    <date>\n
     *    <resource>
     *
     * @param {Object} options
     * @return {String}
     * @api private
     */
    function queryStringToSign(options) {
        return 'GET\n\n\n' + options.date + '\n' + options.resource;
    }
    module.exports.queryStringToSign = queryStringToSign;
    /**
     * Perform the following:
     *
     *  - ignore non-amazon headers
     *  - lowercase fields
     *  - sort lexicographically
     *  - trim whitespace between ":"
     *  - join with newline
     *
     * @param {Object} headers
     * @return {String}
     * @api private
     */
    function canonicalizeHeaders(headers) {
        var buf = [], fields = Object.keys(headers);
        for (var i = 0, len = fields.length; i < len; ++i) {
            var field = fields[i], val = headers[field], field = field.toLowerCase();
            if (0 !== field.indexOf('x-amz'))
                continue;
            buf.push(field + ':' + val);
        }
        return buf.sort().join('\n');
    }
    module.exports.canonicalizeHeaders = canonicalizeHeaders;
    /**
     * Perform the following:
     *
     *  - ignore non sub-resources
     *  - sort lexicographically
     *
     * @param {String} resource
     * @return {String}
     * @api private
     */
    function canonicalizeResource(resource) {
        var url = parse(resource, true), path = url.pathname, buf = [];
        Object.keys(url.query).forEach(function (key) {
            if (!~keys.indexOf(key))
                return;
            var val = '' == url.query[key] ? '' : '=' + encodeURIComponent(url.query[key]);
            buf.push(key + val);
        });
        return path + (buf.length ? '?' + buf.sort().join('&') : '');
    }
    module.exports.canonicalizeResource = canonicalizeResource;
},
/* aws4/aws4.js */ function _(require, module, exports) {
    var aws4 = exports, url = require('url'), querystring = require('querystring'), crypto = require('crypto'), lru = require(850) /* ./lru */, credentialsCache = lru(1000);
    // http://docs.amazonwebservices.com/general/latest/gr/signature-version-4.html
    function hmac(key, string, encoding) {
        return crypto.createHmac('sha256', key).update(string, 'utf8').digest(encoding);
    }
    function hash(string, encoding) {
        return crypto.createHash('sha256').update(string, 'utf8').digest(encoding);
    }
    // This function assumes the string has already been percent encoded
    function encodeRfc3986(urlEncodedString) {
        return urlEncodedString.replace(/[!'()*]/g, function (c) {
            return '%' + c.charCodeAt(0).toString(16).toUpperCase();
        });
    }
    function encodeRfc3986Full(str) {
        return encodeRfc3986(encodeURIComponent(str));
    }
    // request: { path | body, [host], [method], [headers], [service], [region] }
    // credentials: { accessKeyId, secretAccessKey, [sessionToken] }
    function RequestSigner(request, credentials) {
        if (typeof request === 'string')
            request = url.parse(request);
        var headers = request.headers = (request.headers || {}), hostParts = this.matchHost(request.hostname || request.host || headers.Host || headers.host);
        this.request = request;
        this.credentials = credentials || this.defaultCredentials();
        this.service = request.service || hostParts[0] || '';
        this.region = request.region || hostParts[1] || 'us-east-1';
        // SES uses a different domain from the service name
        if (this.service === 'email')
            this.service = 'ses';
        if (!request.method && request.body)
            request.method = 'POST';
        if (!headers.Host && !headers.host) {
            headers.Host = request.hostname || request.host || this.createHost();
            // If a port is specified explicitly, use it as is
            if (request.port)
                headers.Host += ':' + request.port;
        }
        if (!request.hostname && !request.host)
            request.hostname = headers.Host || headers.host;
        this.isCodeCommitGit = this.service === 'codecommit' && request.method === 'GIT';
    }
    RequestSigner.prototype.matchHost = function (host) {
        var match = (host || '').match(/([^\.]+)\.(?:([^\.]*)\.)?amazonaws\.com(\.cn)?$/);
        var hostParts = (match || []).slice(1, 3);
        // ES's hostParts are sometimes the other way round, if the value that is expected
        // to be region equals ‘es’ switch them back
        // e.g. search-cluster-name-aaaa00aaaa0aaa0aaaaaaa0aaa.us-east-1.es.amazonaws.com
        if (hostParts[1] === 'es')
            hostParts = hostParts.reverse();
        return hostParts;
    };
    // http://docs.aws.amazon.com/general/latest/gr/rande.html
    RequestSigner.prototype.isSingleRegion = function () {
        // Special case for S3 and SimpleDB in us-east-1
        if (['s3', 'sdb'].indexOf(this.service) >= 0 && this.region === 'us-east-1')
            return true;
        return ['cloudfront', 'ls', 'route53', 'iam', 'importexport', 'sts']
            .indexOf(this.service) >= 0;
    };
    RequestSigner.prototype.createHost = function () {
        var region = this.isSingleRegion() ? '' :
            (this.service === 's3' && this.region !== 'us-east-1' ? '-' : '.') + this.region, service = this.service === 'ses' ? 'email' : this.service;
        return service + region + '.amazonaws.com';
    };
    RequestSigner.prototype.prepareRequest = function () {
        this.parsePath();
        var request = this.request, headers = request.headers, query;
        if (request.signQuery) {
            this.parsedPath.query = query = this.parsedPath.query || {};
            if (this.credentials.sessionToken)
                query['X-Amz-Security-Token'] = this.credentials.sessionToken;
            if (this.service === 's3' && !query['X-Amz-Expires'])
                query['X-Amz-Expires'] = 86400;
            if (query['X-Amz-Date'])
                this.datetime = query['X-Amz-Date'];
            else
                query['X-Amz-Date'] = this.getDateTime();
            query['X-Amz-Algorithm'] = 'AWS4-HMAC-SHA256';
            query['X-Amz-Credential'] = this.credentials.accessKeyId + '/' + this.credentialString();
            query['X-Amz-SignedHeaders'] = this.signedHeaders();
        }
        else {
            if (!request.doNotModifyHeaders && !this.isCodeCommitGit) {
                if (request.body && !headers['Content-Type'] && !headers['content-type'])
                    headers['Content-Type'] = 'application/x-www-form-urlencoded; charset=utf-8';
                if (request.body && !headers['Content-Length'] && !headers['content-length'])
                    headers['Content-Length'] = Buffer.byteLength(request.body);
                if (this.credentials.sessionToken && !headers['X-Amz-Security-Token'] && !headers['x-amz-security-token'])
                    headers['X-Amz-Security-Token'] = this.credentials.sessionToken;
                if (this.service === 's3' && !headers['X-Amz-Content-Sha256'] && !headers['x-amz-content-sha256'])
                    headers['X-Amz-Content-Sha256'] = hash(this.request.body || '', 'hex');
                if (headers['X-Amz-Date'] || headers['x-amz-date'])
                    this.datetime = headers['X-Amz-Date'] || headers['x-amz-date'];
                else
                    headers['X-Amz-Date'] = this.getDateTime();
            }
            delete headers.Authorization;
            delete headers.authorization;
        }
    };
    RequestSigner.prototype.sign = function () {
        if (!this.parsedPath)
            this.prepareRequest();
        if (this.request.signQuery) {
            this.parsedPath.query['X-Amz-Signature'] = this.signature();
        }
        else {
            this.request.headers.Authorization = this.authHeader();
        }
        this.request.path = this.formatPath();
        return this.request;
    };
    RequestSigner.prototype.getDateTime = function () {
        if (!this.datetime) {
            var headers = this.request.headers, date = new Date(headers.Date || headers.date || new Date);
            this.datetime = date.toISOString().replace(/[:\-]|\.\d{3}/g, '');
            // Remove the trailing 'Z' on the timestamp string for CodeCommit git access
            if (this.isCodeCommitGit)
                this.datetime = this.datetime.slice(0, -1);
        }
        return this.datetime;
    };
    RequestSigner.prototype.getDate = function () {
        return this.getDateTime().substr(0, 8);
    };
    RequestSigner.prototype.authHeader = function () {
        return [
            'AWS4-HMAC-SHA256 Credential=' + this.credentials.accessKeyId + '/' + this.credentialString(),
            'SignedHeaders=' + this.signedHeaders(),
            'Signature=' + this.signature(),
        ].join(', ');
    };
    RequestSigner.prototype.signature = function () {
        var date = this.getDate(), cacheKey = [this.credentials.secretAccessKey, date, this.region, this.service].join(), kDate, kRegion, kService, kCredentials = credentialsCache.get(cacheKey);
        if (!kCredentials) {
            kDate = hmac('AWS4' + this.credentials.secretAccessKey, date);
            kRegion = hmac(kDate, this.region);
            kService = hmac(kRegion, this.service);
            kCredentials = hmac(kService, 'aws4_request');
            credentialsCache.set(cacheKey, kCredentials);
        }
        return hmac(kCredentials, this.stringToSign(), 'hex');
    };
    RequestSigner.prototype.stringToSign = function () {
        return [
            'AWS4-HMAC-SHA256',
            this.getDateTime(),
            this.credentialString(),
            hash(this.canonicalString(), 'hex'),
        ].join('\n');
    };
    RequestSigner.prototype.canonicalString = function () {
        if (!this.parsedPath)
            this.prepareRequest();
        var pathStr = this.parsedPath.path, query = this.parsedPath.query, headers = this.request.headers, queryStr = '', normalizePath = this.service !== 's3', decodePath = this.service === 's3' || this.request.doNotEncodePath, decodeSlashesInPath = this.service === 's3', firstValOnly = this.service === 's3', bodyHash;
        if (this.service === 's3' && this.request.signQuery) {
            bodyHash = 'UNSIGNED-PAYLOAD';
        }
        else if (this.isCodeCommitGit) {
            bodyHash = '';
        }
        else {
            bodyHash = headers['X-Amz-Content-Sha256'] || headers['x-amz-content-sha256'] ||
                hash(this.request.body || '', 'hex');
        }
        if (query) {
            var reducedQuery = Object.keys(query).reduce(function (obj, key) {
                if (!key)
                    return obj;
                obj[encodeRfc3986Full(key)] = !Array.isArray(query[key]) ? query[key] :
                    (firstValOnly ? query[key][0] : query[key]);
                return obj;
            }, {});
            var encodedQueryPieces = [];
            Object.keys(reducedQuery).sort().forEach(function (key) {
                if (!Array.isArray(reducedQuery[key])) {
                    encodedQueryPieces.push(key + '=' + encodeRfc3986Full(reducedQuery[key]));
                }
                else {
                    reducedQuery[key].map(encodeRfc3986Full).sort()
                        .forEach(function (val) { encodedQueryPieces.push(key + '=' + val); });
                }
            });
            queryStr = encodedQueryPieces.join('&');
        }
        if (pathStr !== '/') {
            if (normalizePath)
                pathStr = pathStr.replace(/\/{2,}/g, '/');
            pathStr = pathStr.split('/').reduce(function (path, piece) {
                if (normalizePath && piece === '..') {
                    path.pop();
                }
                else if (!normalizePath || piece !== '.') {
                    if (decodePath)
                        piece = decodeURIComponent(piece).replace(/\+/g, ' ');
                    path.push(encodeRfc3986Full(piece));
                }
                return path;
            }, []).join('/');
            if (pathStr[0] !== '/')
                pathStr = '/' + pathStr;
            if (decodeSlashesInPath)
                pathStr = pathStr.replace(/%2F/g, '/');
        }
        return [
            this.request.method || 'GET',
            pathStr,
            queryStr,
            this.canonicalHeaders() + '\n',
            this.signedHeaders(),
            bodyHash,
        ].join('\n');
    };
    RequestSigner.prototype.canonicalHeaders = function () {
        var headers = this.request.headers;
        function trimAll(header) {
            return header.toString().trim().replace(/\s+/g, ' ');
        }
        return Object.keys(headers)
            .sort(function (a, b) { return a.toLowerCase() < b.toLowerCase() ? -1 : 1; })
            .map(function (key) { return key.toLowerCase() + ':' + trimAll(headers[key]); })
            .join('\n');
    };
    RequestSigner.prototype.signedHeaders = function () {
        return Object.keys(this.request.headers)
            .map(function (key) { return key.toLowerCase(); })
            .sort()
            .join(';');
    };
    RequestSigner.prototype.credentialString = function () {
        return [
            this.getDate(),
            this.region,
            this.service,
            'aws4_request',
        ].join('/');
    };
    RequestSigner.prototype.defaultCredentials = function () {
        var env = process.env;
        return {
            accessKeyId: env.AWS_ACCESS_KEY_ID || env.AWS_ACCESS_KEY,
            secretAccessKey: env.AWS_SECRET_ACCESS_KEY || env.AWS_SECRET_KEY,
            sessionToken: env.AWS_SESSION_TOKEN,
        };
    };
    RequestSigner.prototype.parsePath = function () {
        var path = this.request.path || '/';
        // S3 doesn't always encode characters > 127 correctly and
        // all services don't encode characters > 255 correctly
        // So if there are non-reserved chars (and it's not already all % encoded), just encode them all
        if (/[^0-9A-Za-z;,/?:@&=+$\-_.!~*'()#%]/.test(path)) {
            path = encodeURI(decodeURI(path));
        }
        var queryIx = path.indexOf('?'), query = null;
        if (queryIx >= 0) {
            query = querystring.parse(path.slice(queryIx + 1));
            path = path.slice(0, queryIx);
        }
        this.parsedPath = {
            path: path,
            query: query,
        };
    };
    RequestSigner.prototype.formatPath = function () {
        var path = this.parsedPath.path, query = this.parsedPath.query;
        if (!query)
            return path;
        // Services don't support empty query string keys
        if (query[''] != null)
            delete query[''];
        return path + '?' + encodeRfc3986(querystring.stringify(query));
    };
    aws4.RequestSigner = RequestSigner;
    aws4.sign = function (request, credentials) {
        return new RequestSigner(request, credentials).sign();
    };
},
/* aws4/lru.js */ function _(require, module, exports) {
    module.exports = function (size) {
        return new LruCache(size);
    };
    function LruCache(size) {
        this.capacity = size | 0;
        this.map = Object.create(null);
        this.list = new DoublyLinkedList();
    }
    LruCache.prototype.get = function (key) {
        var node = this.map[key];
        if (node == null)
            return undefined;
        this.used(node);
        return node.val;
    };
    LruCache.prototype.set = function (key, val) {
        var node = this.map[key];
        if (node != null) {
            node.val = val;
        }
        else {
            if (!this.capacity)
                this.prune();
            if (!this.capacity)
                return false;
            node = new DoublyLinkedNode(key, val);
            this.map[key] = node;
            this.capacity--;
        }
        this.used(node);
        return true;
    };
    LruCache.prototype.used = function (node) {
        this.list.moveToFront(node);
    };
    LruCache.prototype.prune = function () {
        var node = this.list.pop();
        if (node != null) {
            delete this.map[node.key];
            this.capacity++;
        }
    };
    function DoublyLinkedList() {
        this.firstNode = null;
        this.lastNode = null;
    }
    DoublyLinkedList.prototype.moveToFront = function (node) {
        if (this.firstNode == node)
            return;
        this.remove(node);
        if (this.firstNode == null) {
            this.firstNode = node;
            this.lastNode = node;
            node.prev = null;
            node.next = null;
        }
        else {
            node.prev = null;
            node.next = this.firstNode;
            node.next.prev = node;
            this.firstNode = node;
        }
    };
    DoublyLinkedList.prototype.pop = function () {
        var lastNode = this.lastNode;
        if (lastNode != null) {
            this.remove(lastNode);
        }
        return lastNode;
    };
    DoublyLinkedList.prototype.remove = function (node) {
        if (this.firstNode == node) {
            this.firstNode = node.next;
        }
        else if (node.prev != null) {
            node.prev.next = node.next;
        }
        if (this.lastNode == node) {
            this.lastNode = node.prev;
        }
        else if (node.next != null) {
            node.next.prev = node.prev;
        }
    };
    function DoublyLinkedNode(key, val) {
        this.key = key;
        this.val = val;
        this.prev = null;
        this.next = null;
    }
},
/* http-signature/lib/index.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    var parser = require(852) /* ./parser */;
    var signer = require(893) /* ./signer */;
    var verify = require(899) /* ./verify */;
    var utils = require(854) /* ./utils */;
    ///--- API
    module.exports = {
        parse: parser.parseRequest,
        parseRequest: parser.parseRequest,
        sign: signer.signRequest,
        signRequest: signer.signRequest,
        createSigner: signer.createSigner,
        isSigner: signer.isSigner,
        sshKeyToPEM: utils.sshKeyToPEM,
        sshKeyFingerprint: utils.fingerprint,
        pemToRsaSSHKey: utils.pemToRsaSSHKey,
        verify: verify.verifySignature,
        verifySignature: verify.verifySignature,
        verifyHMAC: verify.verifyHMAC
    };
},
/* http-signature/lib/parser.js */ function _(require, module, exports) {
    // Copyright 2012 Joyent, Inc.  All rights reserved.
    var assert = require(853) /* assert-plus */;
    var util = require('util');
    var utils = require(854) /* ./utils */;
    ///--- Globals
    var HASH_ALGOS = utils.HASH_ALGOS;
    var PK_ALGOS = utils.PK_ALGOS;
    var HttpSignatureError = utils.HttpSignatureError;
    var InvalidAlgorithmError = utils.InvalidAlgorithmError;
    var validateAlgorithm = utils.validateAlgorithm;
    var State = {
        New: 0,
        Params: 1
    };
    var ParamsState = {
        Name: 0,
        Quote: 1,
        Value: 2,
        Comma: 3
    };
    ///--- Specific Errors
    function ExpiredRequestError(message) {
        HttpSignatureError.call(this, message, ExpiredRequestError);
    }
    util.inherits(ExpiredRequestError, HttpSignatureError);
    function InvalidHeaderError(message) {
        HttpSignatureError.call(this, message, InvalidHeaderError);
    }
    util.inherits(InvalidHeaderError, HttpSignatureError);
    function InvalidParamsError(message) {
        HttpSignatureError.call(this, message, InvalidParamsError);
    }
    util.inherits(InvalidParamsError, HttpSignatureError);
    function MissingHeaderError(message) {
        HttpSignatureError.call(this, message, MissingHeaderError);
    }
    util.inherits(MissingHeaderError, HttpSignatureError);
    function StrictParsingError(message) {
        HttpSignatureError.call(this, message, StrictParsingError);
    }
    util.inherits(StrictParsingError, HttpSignatureError);
    ///--- Exported API
    module.exports = {
        /**
         * Parses the 'Authorization' header out of an http.ServerRequest object.
         *
         * Note that this API will fully validate the Authorization header, and throw
         * on any error.  It will not however check the signature, or the keyId format
         * as those are specific to your environment.  You can use the options object
         * to pass in extra constraints.
         *
         * As a response object you can expect this:
         *
         *     {
         *       "scheme": "Signature",
         *       "params": {
         *         "keyId": "foo",
         *         "algorithm": "rsa-sha256",
         *         "headers": [
         *           "date" or "x-date",
         *           "digest"
         *         ],
         *         "signature": "base64"
         *       },
         *       "signingString": "ready to be passed to crypto.verify()"
         *     }
         *
         * @param {Object} request an http.ServerRequest.
         * @param {Object} options an optional options object with:
         *                   - clockSkew: allowed clock skew in seconds (default 300).
         *                   - headers: required header names (def: date or x-date)
         *                   - algorithms: algorithms to support (default: all).
         *                   - strict: should enforce latest spec parsing
         *                             (default: false).
         * @return {Object} parsed out object (see above).
         * @throws {TypeError} on invalid input.
         * @throws {InvalidHeaderError} on an invalid Authorization header error.
         * @throws {InvalidParamsError} if the params in the scheme are invalid.
         * @throws {MissingHeaderError} if the params indicate a header not present,
         *                              either in the request headers from the params,
         *                              or not in the params from a required header
         *                              in options.
         * @throws {StrictParsingError} if old attributes are used in strict parsing
         *                              mode.
         * @throws {ExpiredRequestError} if the value of date or x-date exceeds skew.
         */
        parseRequest: function parseRequest(request, options) {
            assert.object(request, 'request');
            assert.object(request.headers, 'request.headers');
            if (options === undefined) {
                options = {};
            }
            if (options.headers === undefined) {
                options.headers = [request.headers['x-date'] ? 'x-date' : 'date'];
            }
            assert.object(options, 'options');
            assert.arrayOfString(options.headers, 'options.headers');
            assert.optionalFinite(options.clockSkew, 'options.clockSkew');
            var authzHeaderName = options.authorizationHeaderName || 'authorization';
            if (!request.headers[authzHeaderName]) {
                throw new MissingHeaderError('no ' + authzHeaderName + ' header ' +
                    'present in the request');
            }
            options.clockSkew = options.clockSkew || 300;
            var i = 0;
            var state = State.New;
            var substate = ParamsState.Name;
            var tmpName = '';
            var tmpValue = '';
            var parsed = {
                scheme: '',
                params: {},
                signingString: ''
            };
            var authz = request.headers[authzHeaderName];
            for (i = 0; i < authz.length; i++) {
                var c = authz.charAt(i);
                switch (Number(state)) {
                    case State.New:
                        if (c !== ' ')
                            parsed.scheme += c;
                        else
                            state = State.Params;
                        break;
                    case State.Params:
                        switch (Number(substate)) {
                            case ParamsState.Name:
                                var code = c.charCodeAt(0);
                                // restricted name of A-Z / a-z
                                if ((code >= 0x41 && code <= 0x5a) || // A-Z
                                    (code >= 0x61 && code <= 0x7a)) { // a-z
                                    tmpName += c;
                                }
                                else if (c === '=') {
                                    if (tmpName.length === 0)
                                        throw new InvalidHeaderError('bad param format');
                                    substate = ParamsState.Quote;
                                }
                                else {
                                    throw new InvalidHeaderError('bad param format');
                                }
                                break;
                            case ParamsState.Quote:
                                if (c === '"') {
                                    tmpValue = '';
                                    substate = ParamsState.Value;
                                }
                                else {
                                    throw new InvalidHeaderError('bad param format');
                                }
                                break;
                            case ParamsState.Value:
                                if (c === '"') {
                                    parsed.params[tmpName] = tmpValue;
                                    substate = ParamsState.Comma;
                                }
                                else {
                                    tmpValue += c;
                                }
                                break;
                            case ParamsState.Comma:
                                if (c === ',') {
                                    tmpName = '';
                                    substate = ParamsState.Name;
                                }
                                else {
                                    throw new InvalidHeaderError('bad param format');
                                }
                                break;
                            default:
                                throw new Error('Invalid substate');
                        }
                        break;
                    default:
                        throw new Error('Invalid substate');
                }
            }
            if (!parsed.params.headers || parsed.params.headers === '') {
                if (request.headers['x-date']) {
                    parsed.params.headers = ['x-date'];
                }
                else {
                    parsed.params.headers = ['date'];
                }
            }
            else {
                parsed.params.headers = parsed.params.headers.split(' ');
            }
            // Minimally validate the parsed object
            if (!parsed.scheme || parsed.scheme !== 'Signature')
                throw new InvalidHeaderError('scheme was not "Signature"');
            if (!parsed.params.keyId)
                throw new InvalidHeaderError('keyId was not specified');
            if (!parsed.params.algorithm)
                throw new InvalidHeaderError('algorithm was not specified');
            if (!parsed.params.signature)
                throw new InvalidHeaderError('signature was not specified');
            // Check the algorithm against the official list
            parsed.params.algorithm = parsed.params.algorithm.toLowerCase();
            try {
                validateAlgorithm(parsed.params.algorithm);
            }
            catch (e) {
                if (e instanceof InvalidAlgorithmError)
                    throw (new InvalidParamsError(parsed.params.algorithm + ' is not ' +
                        'supported'));
                else
                    throw (e);
            }
            // Build the signingString
            for (i = 0; i < parsed.params.headers.length; i++) {
                var h = parsed.params.headers[i].toLowerCase();
                parsed.params.headers[i] = h;
                if (h === 'request-line') {
                    if (!options.strict) {
                        /*
                         * We allow headers from the older spec drafts if strict parsing isn't
                         * specified in options.
                         */
                        parsed.signingString +=
                            request.method + ' ' + request.url + ' HTTP/' + request.httpVersion;
                    }
                    else {
                        /* Strict parsing doesn't allow older draft headers. */
                        throw (new StrictParsingError('request-line is not a valid header ' +
                            'with strict parsing enabled.'));
                    }
                }
                else if (h === '(request-target)') {
                    parsed.signingString +=
                        '(request-target): ' + request.method.toLowerCase() + ' ' +
                            request.url;
                }
                else {
                    var value = request.headers[h];
                    if (value === undefined)
                        throw new MissingHeaderError(h + ' was not in the request');
                    parsed.signingString += h + ': ' + value;
                }
                if ((i + 1) < parsed.params.headers.length)
                    parsed.signingString += '\n';
            }
            // Check against the constraints
            var date;
            if (request.headers.date || request.headers['x-date']) {
                if (request.headers['x-date']) {
                    date = new Date(request.headers['x-date']);
                }
                else {
                    date = new Date(request.headers.date);
                }
                var now = new Date();
                var skew = Math.abs(now.getTime() - date.getTime());
                if (skew > options.clockSkew * 1000) {
                    throw new ExpiredRequestError('clock skew of ' +
                        (skew / 1000) +
                        's was greater than ' +
                        options.clockSkew + 's');
                }
            }
            options.headers.forEach(function (hdr) {
                // Remember that we already checked any headers in the params
                // were in the request, so if this passes we're good.
                if (parsed.params.headers.indexOf(hdr.toLowerCase()) < 0)
                    throw new MissingHeaderError(hdr + ' was not a signed header');
            });
            if (options.algorithms) {
                if (options.algorithms.indexOf(parsed.params.algorithm) === -1)
                    throw new InvalidParamsError(parsed.params.algorithm +
                        ' is not a supported algorithm');
            }
            parsed.algorithm = parsed.params.algorithm.toUpperCase();
            parsed.keyId = parsed.params.keyId;
            return parsed;
        }
    };
},
/* assert-plus/assert.js */ function _(require, module, exports) {
    // Copyright (c) 2012, Mark Cavage. All rights reserved.
    // Copyright 2015 Joyent, Inc.
    var assert = require('assert');
    var Stream = require('stream').Stream;
    var util = require('util');
    ///--- Globals
    /* JSSTYLED */
    var UUID_REGEXP = /^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/;
    ///--- Internal
    function _capitalize(str) {
        return (str.charAt(0).toUpperCase() + str.slice(1));
    }
    function _toss(name, expected, oper, arg, actual) {
        throw new assert.AssertionError({
            message: util.format('%s (%s) is required', name, expected),
            actual: (actual === undefined) ? typeof (arg) : actual(arg),
            expected: expected,
            operator: oper || '===',
            stackStartFunction: _toss.caller
        });
    }
    function _getClass(arg) {
        return (Object.prototype.toString.call(arg).slice(8, -1));
    }
    function noop() {
        // Why even bother with asserts?
    }
    ///--- Exports
    var types = {
        bool: {
            check: function (arg) { return typeof (arg) === 'boolean'; }
        },
        func: {
            check: function (arg) { return typeof (arg) === 'function'; }
        },
        string: {
            check: function (arg) { return typeof (arg) === 'string'; }
        },
        object: {
            check: function (arg) {
                return typeof (arg) === 'object' && arg !== null;
            }
        },
        number: {
            check: function (arg) {
                return typeof (arg) === 'number' && !isNaN(arg);
            }
        },
        finite: {
            check: function (arg) {
                return typeof (arg) === 'number' && !isNaN(arg) && isFinite(arg);
            }
        },
        buffer: {
            check: function (arg) { return Buffer.isBuffer(arg); },
            operator: 'Buffer.isBuffer'
        },
        array: {
            check: function (arg) { return Array.isArray(arg); },
            operator: 'Array.isArray'
        },
        stream: {
            check: function (arg) { return arg instanceof Stream; },
            operator: 'instanceof',
            actual: _getClass
        },
        date: {
            check: function (arg) { return arg instanceof Date; },
            operator: 'instanceof',
            actual: _getClass
        },
        regexp: {
            check: function (arg) { return arg instanceof RegExp; },
            operator: 'instanceof',
            actual: _getClass
        },
        uuid: {
            check: function (arg) {
                return typeof (arg) === 'string' && UUID_REGEXP.test(arg);
            },
            operator: 'isUUID'
        }
    };
    function _setExports(ndebug) {
        var keys = Object.keys(types);
        var out;
        /* re-export standard assert */
        if (process.env.NODE_NDEBUG) {
            out = noop;
        }
        else {
            out = function (arg, msg) {
                if (!arg) {
                    _toss(msg, 'true', arg);
                }
            };
        }
        /* standard checks */
        keys.forEach(function (k) {
            if (ndebug) {
                out[k] = noop;
                return;
            }
            var type = types[k];
            out[k] = function (arg, msg) {
                if (!type.check(arg)) {
                    _toss(msg, k, type.operator, arg, type.actual);
                }
            };
        });
        /* optional checks */
        keys.forEach(function (k) {
            var name = 'optional' + _capitalize(k);
            if (ndebug) {
                out[name] = noop;
                return;
            }
            var type = types[k];
            out[name] = function (arg, msg) {
                if (arg === undefined || arg === null) {
                    return;
                }
                if (!type.check(arg)) {
                    _toss(msg, k, type.operator, arg, type.actual);
                }
            };
        });
        /* arrayOf checks */
        keys.forEach(function (k) {
            var name = 'arrayOf' + _capitalize(k);
            if (ndebug) {
                out[name] = noop;
                return;
            }
            var type = types[k];
            var expected = '[' + k + ']';
            out[name] = function (arg, msg) {
                if (!Array.isArray(arg)) {
                    _toss(msg, expected, type.operator, arg, type.actual);
                }
                var i;
                for (i = 0; i < arg.length; i++) {
                    if (!type.check(arg[i])) {
                        _toss(msg, expected, type.operator, arg, type.actual);
                    }
                }
            };
        });
        /* optionalArrayOf checks */
        keys.forEach(function (k) {
            var name = 'optionalArrayOf' + _capitalize(k);
            if (ndebug) {
                out[name] = noop;
                return;
            }
            var type = types[k];
            var expected = '[' + k + ']';
            out[name] = function (arg, msg) {
                if (arg === undefined || arg === null) {
                    return;
                }
                if (!Array.isArray(arg)) {
                    _toss(msg, expected, type.operator, arg, type.actual);
                }
                var i;
                for (i = 0; i < arg.length; i++) {
                    if (!type.check(arg[i])) {
                        _toss(msg, expected, type.operator, arg, type.actual);
                    }
                }
            };
        });
        /* re-export built-in assertions */
        Object.keys(assert).forEach(function (k) {
            if (k === 'AssertionError') {
                out[k] = assert[k];
                return;
            }
            if (ndebug) {
                out[k] = noop;
                return;
            }
            out[k] = assert[k];
        });
        /* export ourselves (for unit tests _only_) */
        out._setExports = _setExports;
        return out;
    }
    module.exports = _setExports(process.env.NODE_NDEBUG);
},
/* http-signature/lib/utils.js */ function _(require, module, exports) {
    // Copyright 2012 Joyent, Inc.  All rights reserved.
    var assert = require(853) /* assert-plus */;
    var sshpk = require(855) /* sshpk */;
    var util = require('util');
    var HASH_ALGOS = {
        'sha1': true,
        'sha256': true,
        'sha512': true
    };
    var PK_ALGOS = {
        'rsa': true,
        'dsa': true,
        'ecdsa': true
    };
    function HttpSignatureError(message, caller) {
        if (Error.captureStackTrace)
            Error.captureStackTrace(this, caller || HttpSignatureError);
        this.message = message;
        this.name = caller.name;
    }
    util.inherits(HttpSignatureError, Error);
    function InvalidAlgorithmError(message) {
        HttpSignatureError.call(this, message, InvalidAlgorithmError);
    }
    util.inherits(InvalidAlgorithmError, HttpSignatureError);
    function validateAlgorithm(algorithm) {
        var alg = algorithm.toLowerCase().split('-');
        if (alg.length !== 2) {
            throw (new InvalidAlgorithmError(alg[0].toUpperCase() + ' is not a ' +
                'valid algorithm'));
        }
        if (alg[0] !== 'hmac' && !PK_ALGOS[alg[0]]) {
            throw (new InvalidAlgorithmError(alg[0].toUpperCase() + ' type keys ' +
                'are not supported'));
        }
        if (!HASH_ALGOS[alg[1]]) {
            throw (new InvalidAlgorithmError(alg[1].toUpperCase() + ' is not a ' +
                'supported hash algorithm'));
        }
        return (alg);
    }
    ///--- API
    module.exports = {
        HASH_ALGOS: HASH_ALGOS,
        PK_ALGOS: PK_ALGOS,
        HttpSignatureError: HttpSignatureError,
        InvalidAlgorithmError: InvalidAlgorithmError,
        validateAlgorithm: validateAlgorithm,
        /**
         * Converts an OpenSSH public key (rsa only) to a PKCS#8 PEM file.
         *
         * The intent of this module is to interoperate with OpenSSL only,
         * specifically the node crypto module's `verify` method.
         *
         * @param {String} key an OpenSSH public key.
         * @return {String} PEM encoded form of the RSA public key.
         * @throws {TypeError} on bad input.
         * @throws {Error} on invalid ssh key formatted data.
         */
        sshKeyToPEM: function sshKeyToPEM(key) {
            assert.string(key, 'ssh_key');
            var k = sshpk.parseKey(key, 'ssh');
            return (k.toString('pem'));
        },
        /**
         * Generates an OpenSSH fingerprint from an ssh public key.
         *
         * @param {String} key an OpenSSH public key.
         * @return {String} key fingerprint.
         * @throws {TypeError} on bad input.
         * @throws {Error} if what you passed doesn't look like an ssh public key.
         */
        fingerprint: function fingerprint(key) {
            assert.string(key, 'ssh_key');
            var k = sshpk.parseKey(key, 'ssh');
            return (k.fingerprint('md5').toString('hex'));
        },
        /**
         * Converts a PKGCS#8 PEM file to an OpenSSH public key (rsa)
         *
         * The reverse of the above function.
         */
        pemToRsaSSHKey: function pemToRsaSSHKey(pem, comment) {
            assert.equal('string', typeof (pem), 'typeof pem');
            var k = sshpk.parseKey(pem, 'pem');
            k.comment = comment;
            return (k.toString('ssh'));
        }
    };
},
/* sshpk/lib/index.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    var Key = require(856) /* ./key */;
    var Fingerprint = require(859) /* ./fingerprint */;
    var Signature = require(862) /* ./signature */;
    var PrivateKey = require(861) /* ./private-key */;
    var Certificate = require(888) /* ./certificate */;
    var Identity = require(889) /* ./identity */;
    var errs = require(860) /* ./errors */;
    module.exports = {
        /* top-level classes */
        Key: Key,
        parseKey: Key.parse,
        Fingerprint: Fingerprint,
        parseFingerprint: Fingerprint.parse,
        Signature: Signature,
        parseSignature: Signature.parse,
        PrivateKey: PrivateKey,
        parsePrivateKey: PrivateKey.parse,
        generatePrivateKey: PrivateKey.generate,
        Certificate: Certificate,
        parseCertificate: Certificate.parse,
        createSelfSignedCertificate: Certificate.createSelfSigned,
        createCertificate: Certificate.create,
        Identity: Identity,
        identityFromDN: Identity.parseDN,
        identityForHost: Identity.forHost,
        identityForUser: Identity.forUser,
        identityForEmail: Identity.forEmail,
        identityFromArray: Identity.fromArray,
        /* errors */
        FingerprintFormatError: errs.FingerprintFormatError,
        InvalidAlgorithmError: errs.InvalidAlgorithmError,
        KeyParseError: errs.KeyParseError,
        SignatureParseError: errs.SignatureParseError,
        KeyEncryptedError: errs.KeyEncryptedError,
        CertificateParseError: errs.CertificateParseError
    };
},
/* sshpk/lib/key.js */ function _(require, module, exports) {
    // Copyright 2018 Joyent, Inc.
    module.exports = Key;
    var assert = require(853) /* assert-plus */;
    var algs = require(857) /* ./algs */;
    var crypto = require('crypto');
    var Fingerprint = require(859) /* ./fingerprint */;
    var Signature = require(862) /* ./signature */;
    var DiffieHellman = require(874) /* ./dhe */.DiffieHellman;
    var errs = require(860) /* ./errors */;
    var utils = require(863) /* ./utils */;
    var PrivateKey = require(861) /* ./private-key */;
    var edCompat;
    try {
        edCompat = require(877) /* ./ed-compat */;
    }
    catch (e) {
        /* Just continue through, and bail out if we try to use it. */
    }
    var InvalidAlgorithmError = errs.InvalidAlgorithmError;
    var KeyParseError = errs.KeyParseError;
    var formats = {};
    formats['auto'] = require(878) /* ./formats/auto */;
    formats['pem'] = require(879) /* ./formats/pem */;
    formats['pkcs1'] = require(880) /* ./formats/pkcs1 */;
    formats['pkcs8'] = require(881) /* ./formats/pkcs8 */;
    formats['rfc4253'] = require(883) /* ./formats/rfc4253 */;
    formats['ssh'] = require(885) /* ./formats/ssh */;
    formats['ssh-private'] = require(882) /* ./formats/ssh-private */;
    formats['openssh'] = formats['ssh-private'];
    formats['dnssec'] = require(886) /* ./formats/dnssec */;
    formats['putty'] = require(887) /* ./formats/putty */;
    formats['ppk'] = formats['putty'];
    function Key(opts) {
        assert.object(opts, 'options');
        assert.arrayOfObject(opts.parts, 'options.parts');
        assert.string(opts.type, 'options.type');
        assert.optionalString(opts.comment, 'options.comment');
        var algInfo = algs.info[opts.type];
        if (typeof (algInfo) !== 'object')
            throw (new InvalidAlgorithmError(opts.type));
        var partLookup = {};
        for (var i = 0; i < opts.parts.length; ++i) {
            var part = opts.parts[i];
            partLookup[part.name] = part;
        }
        this.type = opts.type;
        this.parts = opts.parts;
        this.part = partLookup;
        this.comment = undefined;
        this.source = opts.source;
        /* for speeding up hashing/fingerprint operations */
        this._rfc4253Cache = opts._rfc4253Cache;
        this._hashCache = {};
        var sz;
        this.curve = undefined;
        if (this.type === 'ecdsa') {
            var curve = this.part.curve.data.toString();
            this.curve = curve;
            sz = algs.curves[curve].size;
        }
        else if (this.type === 'ed25519' || this.type === 'curve25519') {
            sz = 256;
            this.curve = 'curve25519';
        }
        else {
            var szPart = this.part[algInfo.sizePart];
            sz = szPart.data.length;
            sz = sz * 8 - utils.countZeros(szPart.data);
        }
        this.size = sz;
    }
    Key.formats = formats;
    Key.prototype.toBuffer = function (format, options) {
        if (format === undefined)
            format = 'ssh';
        assert.string(format, 'format');
        assert.object(formats[format], 'formats[format]');
        assert.optionalObject(options, 'options');
        if (format === 'rfc4253') {
            if (this._rfc4253Cache === undefined)
                this._rfc4253Cache = formats['rfc4253'].write(this);
            return (this._rfc4253Cache);
        }
        return (formats[format].write(this, options));
    };
    Key.prototype.toString = function (format, options) {
        return (this.toBuffer(format, options).toString());
    };
    Key.prototype.hash = function (algo, type) {
        assert.string(algo, 'algorithm');
        assert.optionalString(type, 'type');
        if (type === undefined)
            type = 'ssh';
        algo = algo.toLowerCase();
        if (algs.hashAlgs[algo] === undefined)
            throw (new InvalidAlgorithmError(algo));
        var cacheKey = algo + '||' + type;
        if (this._hashCache[cacheKey])
            return (this._hashCache[cacheKey]);
        var buf;
        if (type === 'ssh') {
            buf = this.toBuffer('rfc4253');
        }
        else if (type === 'spki') {
            buf = formats.pkcs8.pkcs8ToBuffer(this);
        }
        else {
            throw (new Error('Hash type ' + type + ' not supported'));
        }
        var hash = crypto.createHash(algo).update(buf).digest();
        this._hashCache[cacheKey] = hash;
        return (hash);
    };
    Key.prototype.fingerprint = function (algo, type) {
        if (algo === undefined)
            algo = 'sha256';
        if (type === undefined)
            type = 'ssh';
        assert.string(algo, 'algorithm');
        assert.string(type, 'type');
        var opts = {
            type: 'key',
            hash: this.hash(algo, type),
            algorithm: algo,
            hashType: type
        };
        return (new Fingerprint(opts));
    };
    Key.prototype.defaultHashAlgorithm = function () {
        var hashAlgo = 'sha1';
        if (this.type === 'rsa')
            hashAlgo = 'sha256';
        if (this.type === 'dsa' && this.size > 1024)
            hashAlgo = 'sha256';
        if (this.type === 'ed25519')
            hashAlgo = 'sha512';
        if (this.type === 'ecdsa') {
            if (this.size <= 256)
                hashAlgo = 'sha256';
            else if (this.size <= 384)
                hashAlgo = 'sha384';
            else
                hashAlgo = 'sha512';
        }
        return (hashAlgo);
    };
    Key.prototype.createVerify = function (hashAlgo) {
        if (hashAlgo === undefined)
            hashAlgo = this.defaultHashAlgorithm();
        assert.string(hashAlgo, 'hash algorithm');
        /* ED25519 is not supported by OpenSSL, use a javascript impl. */
        if (this.type === 'ed25519' && edCompat !== undefined)
            return (new edCompat.Verifier(this, hashAlgo));
        if (this.type === 'curve25519')
            throw (new Error('Curve25519 keys are not suitable for ' +
                'signing or verification'));
        var v, nm, err;
        try {
            nm = hashAlgo.toUpperCase();
            v = crypto.createVerify(nm);
        }
        catch (e) {
            err = e;
        }
        if (v === undefined || (err instanceof Error &&
            err.message.match(/Unknown message digest/))) {
            nm = 'RSA-';
            nm += hashAlgo.toUpperCase();
            v = crypto.createVerify(nm);
        }
        assert.ok(v, 'failed to create verifier');
        var oldVerify = v.verify.bind(v);
        var key = this.toBuffer('pkcs8');
        var curve = this.curve;
        var self = this;
        v.verify = function (signature, fmt) {
            if (Signature.isSignature(signature, [2, 0])) {
                if (signature.type !== self.type)
                    return (false);
                if (signature.hashAlgorithm &&
                    signature.hashAlgorithm !== hashAlgo)
                    return (false);
                if (signature.curve && self.type === 'ecdsa' &&
                    signature.curve !== curve)
                    return (false);
                return (oldVerify(key, signature.toBuffer('asn1')));
            }
            else if (typeof (signature) === 'string' ||
                Buffer.isBuffer(signature)) {
                return (oldVerify(key, signature, fmt));
                /*
                 * Avoid doing this on valid arguments, walking the prototype
                 * chain can be quite slow.
                 */
            }
            else if (Signature.isSignature(signature, [1, 0])) {
                throw (new Error('signature was created by too old ' +
                    'a version of sshpk and cannot be verified'));
            }
            else {
                throw (new TypeError('signature must be a string, ' +
                    'Buffer, or Signature object'));
            }
        };
        return (v);
    };
    Key.prototype.createDiffieHellman = function () {
        if (this.type === 'rsa')
            throw (new Error('RSA keys do not support Diffie-Hellman'));
        return (new DiffieHellman(this));
    };
    Key.prototype.createDH = Key.prototype.createDiffieHellman;
    Key.parse = function (data, format, options) {
        if (typeof (data) !== 'string')
            assert.buffer(data, 'data');
        if (format === undefined)
            format = 'auto';
        assert.string(format, 'format');
        if (typeof (options) === 'string')
            options = { filename: options };
        assert.optionalObject(options, 'options');
        if (options === undefined)
            options = {};
        assert.optionalString(options.filename, 'options.filename');
        if (options.filename === undefined)
            options.filename = '(unnamed)';
        assert.object(formats[format], 'formats[format]');
        try {
            var k = formats[format].read(data, options);
            if (k instanceof PrivateKey)
                k = k.toPublic();
            if (!k.comment)
                k.comment = options.filename;
            return (k);
        }
        catch (e) {
            if (e.name === 'KeyEncryptedError')
                throw (e);
            throw (new KeyParseError(options.filename, format, e));
        }
    };
    Key.isKey = function (obj, ver) {
        return (utils.isCompatible(obj, Key, ver));
    };
    /*
     * API versions for Key:
     * [1,0] -- initial ver, may take Signature for createVerify or may not
     * [1,1] -- added pkcs1, pkcs8 formats
     * [1,2] -- added auto, ssh-private, openssh formats
     * [1,3] -- added defaultHashAlgorithm
     * [1,4] -- added ed support, createDH
     * [1,5] -- first explicitly tagged version
     * [1,6] -- changed ed25519 part names
     * [1,7] -- spki hash types
     */
    Key.prototype._sshpkApiVersion = [1, 7];
    Key._oldVersionDetect = function (obj) {
        assert.func(obj.toBuffer);
        assert.func(obj.fingerprint);
        if (obj.createDH)
            return ([1, 4]);
        if (obj.defaultHashAlgorithm)
            return ([1, 3]);
        if (obj.formats['auto'])
            return ([1, 2]);
        if (obj.formats['pkcs1'])
            return ([1, 1]);
        return ([1, 0]);
    };
},
/* sshpk/lib/algs.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algInfo = {
        'dsa': {
            parts: ['p', 'q', 'g', 'y'],
            sizePart: 'p'
        },
        'rsa': {
            parts: ['e', 'n'],
            sizePart: 'n'
        },
        'ecdsa': {
            parts: ['curve', 'Q'],
            sizePart: 'Q'
        },
        'ed25519': {
            parts: ['A'],
            sizePart: 'A'
        }
    };
    algInfo['curve25519'] = algInfo['ed25519'];
    var algPrivInfo = {
        'dsa': {
            parts: ['p', 'q', 'g', 'y', 'x']
        },
        'rsa': {
            parts: ['n', 'e', 'd', 'iqmp', 'p', 'q']
        },
        'ecdsa': {
            parts: ['curve', 'Q', 'd']
        },
        'ed25519': {
            parts: ['A', 'k']
        }
    };
    algPrivInfo['curve25519'] = algPrivInfo['ed25519'];
    var hashAlgs = {
        'md5': true,
        'sha1': true,
        'sha256': true,
        'sha384': true,
        'sha512': true
    };
    /*
     * Taken from
     * http://csrc.nist.gov/groups/ST/toolkit/documents/dss/NISTReCur.pdf
     */
    var curves = {
        'nistp256': {
            size: 256,
            pkcs8oid: '1.2.840.10045.3.1.7',
            p: Buffer.from(('00' +
                'ffffffff 00000001 00000000 00000000' +
                '00000000 ffffffff ffffffff ffffffff').
                replace(/ /g, ''), 'hex'),
            a: Buffer.from(('00' +
                'FFFFFFFF 00000001 00000000 00000000' +
                '00000000 FFFFFFFF FFFFFFFF FFFFFFFC').
                replace(/ /g, ''), 'hex'),
            b: Buffer.from(('5ac635d8 aa3a93e7 b3ebbd55 769886bc' +
                '651d06b0 cc53b0f6 3bce3c3e 27d2604b').
                replace(/ /g, ''), 'hex'),
            s: Buffer.from(('00' +
                'c49d3608 86e70493 6a6678e1 139d26b7' +
                '819f7e90').
                replace(/ /g, ''), 'hex'),
            n: Buffer.from(('00' +
                'ffffffff 00000000 ffffffff ffffffff' +
                'bce6faad a7179e84 f3b9cac2 fc632551').
                replace(/ /g, ''), 'hex'),
            G: Buffer.from(('04' +
                '6b17d1f2 e12c4247 f8bce6e5 63a440f2' +
                '77037d81 2deb33a0 f4a13945 d898c296' +
                '4fe342e2 fe1a7f9b 8ee7eb4a 7c0f9e16' +
                '2bce3357 6b315ece cbb64068 37bf51f5').
                replace(/ /g, ''), 'hex')
        },
        'nistp384': {
            size: 384,
            pkcs8oid: '1.3.132.0.34',
            p: Buffer.from(('00' +
                'ffffffff ffffffff ffffffff ffffffff' +
                'ffffffff ffffffff ffffffff fffffffe' +
                'ffffffff 00000000 00000000 ffffffff').
                replace(/ /g, ''), 'hex'),
            a: Buffer.from(('00' +
                'FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF' +
                'FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFE' +
                'FFFFFFFF 00000000 00000000 FFFFFFFC').
                replace(/ /g, ''), 'hex'),
            b: Buffer.from(('b3312fa7 e23ee7e4 988e056b e3f82d19' +
                '181d9c6e fe814112 0314088f 5013875a' +
                'c656398d 8a2ed19d 2a85c8ed d3ec2aef').
                replace(/ /g, ''), 'hex'),
            s: Buffer.from(('00' +
                'a335926a a319a27a 1d00896a 6773a482' +
                '7acdac73').
                replace(/ /g, ''), 'hex'),
            n: Buffer.from(('00' +
                'ffffffff ffffffff ffffffff ffffffff' +
                'ffffffff ffffffff c7634d81 f4372ddf' +
                '581a0db2 48b0a77a ecec196a ccc52973').
                replace(/ /g, ''), 'hex'),
            G: Buffer.from(('04' +
                'aa87ca22 be8b0537 8eb1c71e f320ad74' +
                '6e1d3b62 8ba79b98 59f741e0 82542a38' +
                '5502f25d bf55296c 3a545e38 72760ab7' +
                '3617de4a 96262c6f 5d9e98bf 9292dc29' +
                'f8f41dbd 289a147c e9da3113 b5f0b8c0' +
                '0a60b1ce 1d7e819d 7a431d7c 90ea0e5f').
                replace(/ /g, ''), 'hex')
        },
        'nistp521': {
            size: 521,
            pkcs8oid: '1.3.132.0.35',
            p: Buffer.from(('01ffffff ffffffff ffffffff ffffffff' +
                'ffffffff ffffffff ffffffff ffffffff' +
                'ffffffff ffffffff ffffffff ffffffff' +
                'ffffffff ffffffff ffffffff ffffffff' +
                'ffff').replace(/ /g, ''), 'hex'),
            a: Buffer.from(('01FF' +
                'FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF' +
                'FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF' +
                'FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF' +
                'FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFC').
                replace(/ /g, ''), 'hex'),
            b: Buffer.from(('51' +
                '953eb961 8e1c9a1f 929a21a0 b68540ee' +
                'a2da725b 99b315f3 b8b48991 8ef109e1' +
                '56193951 ec7e937b 1652c0bd 3bb1bf07' +
                '3573df88 3d2c34f1 ef451fd4 6b503f00').
                replace(/ /g, ''), 'hex'),
            s: Buffer.from(('00' +
                'd09e8800 291cb853 96cc6717 393284aa' +
                'a0da64ba').replace(/ /g, ''), 'hex'),
            n: Buffer.from(('01ff' +
                'ffffffff ffffffff ffffffff ffffffff' +
                'ffffffff ffffffff ffffffff fffffffa' +
                '51868783 bf2f966b 7fcc0148 f709a5d0' +
                '3bb5c9b8 899c47ae bb6fb71e 91386409').
                replace(/ /g, ''), 'hex'),
            G: Buffer.from(('04' +
                '00c6 858e06b7 0404e9cd 9e3ecb66 2395b442' +
                '9c648139 053fb521 f828af60 6b4d3dba' +
                'a14b5e77 efe75928 fe1dc127 a2ffa8de' +
                '3348b3c1 856a429b f97e7e31 c2e5bd66' +
                '0118 39296a78 9a3bc004 5c8a5fb4 2c7d1bd9' +
                '98f54449 579b4468 17afbd17 273e662c' +
                '97ee7299 5ef42640 c550b901 3fad0761' +
                '353c7086 a272c240 88be9476 9fd16650').
                replace(/ /g, ''), 'hex')
        }
    };
    module.exports = {
        info: algInfo,
        privInfo: algPrivInfo,
        hashAlgs: hashAlgs,
        curves: curves
    };
},
/* safer-buffer/safer.js */ function _(require, module, exports) {
    var buffer = require('buffer');
    var Buffer = buffer.Buffer;
    var safer = {};
    var key;
    for (key in buffer) {
        if (!buffer.hasOwnProperty(key))
            continue;
        if (key === 'SlowBuffer' || key === 'Buffer')
            continue;
        safer[key] = buffer[key];
    }
    var Safer = safer.Buffer = {};
    for (key in Buffer) {
        if (!Buffer.hasOwnProperty(key))
            continue;
        if (key === 'allocUnsafe' || key === 'allocUnsafeSlow')
            continue;
        Safer[key] = Buffer[key];
    }
    safer.Buffer.prototype = Buffer.prototype;
    if (!Safer.from || Safer.from === Uint8Array.from) {
        Safer.from = function (value, encodingOrOffset, length) {
            if (typeof value === 'number') {
                throw new TypeError('The "value" argument must not be of type number. Received type ' + typeof value);
            }
            if (value && typeof value.length === 'undefined') {
                throw new TypeError('The first argument must be one of type string, Buffer, ArrayBuffer, Array, or Array-like Object. Received type ' + typeof value);
            }
            return Buffer(value, encodingOrOffset, length);
        };
    }
    if (!Safer.alloc) {
        Safer.alloc = function (size, fill, encoding) {
            if (typeof size !== 'number') {
                throw new TypeError('The "size" argument must be of type number. Received type ' + typeof size);
            }
            if (size < 0 || size >= 2 * (1 << 30)) {
                throw new RangeError('The value "' + size + '" is invalid for option "size"');
            }
            var buf = Buffer(size);
            if (!fill || fill.length === 0) {
                buf.fill(0);
            }
            else if (typeof encoding === 'string') {
                buf.fill(fill, encoding);
            }
            else {
                buf.fill(fill);
            }
            return buf;
        };
    }
    if (!safer.kStringMaxLength) {
        try {
            safer.kStringMaxLength = process.binding('buffer').kStringMaxLength;
        }
        catch (e) {
            // we can't determine kStringMaxLength in environments where process.binding
            // is unsupported, so let's not set it
        }
    }
    if (!safer.constants) {
        safer.constants = {
            MAX_LENGTH: safer.kMaxLength
        };
        if (safer.kStringMaxLength) {
            safer.constants.MAX_STRING_LENGTH = safer.kStringMaxLength;
        }
    }
    module.exports = safer;
},
/* sshpk/lib/fingerprint.js */ function _(require, module, exports) {
    // Copyright 2018 Joyent, Inc.
    module.exports = Fingerprint;
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ./algs */;
    var crypto = require('crypto');
    var errs = require(860) /* ./errors */;
    var Key = require(856) /* ./key */;
    var PrivateKey = require(861) /* ./private-key */;
    var Certificate = require(888) /* ./certificate */;
    var utils = require(863) /* ./utils */;
    var FingerprintFormatError = errs.FingerprintFormatError;
    var InvalidAlgorithmError = errs.InvalidAlgorithmError;
    function Fingerprint(opts) {
        assert.object(opts, 'options');
        assert.string(opts.type, 'options.type');
        assert.buffer(opts.hash, 'options.hash');
        assert.string(opts.algorithm, 'options.algorithm');
        this.algorithm = opts.algorithm.toLowerCase();
        if (algs.hashAlgs[this.algorithm] !== true)
            throw (new InvalidAlgorithmError(this.algorithm));
        this.hash = opts.hash;
        this.type = opts.type;
        this.hashType = opts.hashType;
    }
    Fingerprint.prototype.toString = function (format) {
        if (format === undefined) {
            if (this.algorithm === 'md5' || this.hashType === 'spki')
                format = 'hex';
            else
                format = 'base64';
        }
        assert.string(format);
        switch (format) {
            case 'hex':
                if (this.hashType === 'spki')
                    return (this.hash.toString('hex'));
                return (addColons(this.hash.toString('hex')));
            case 'base64':
                if (this.hashType === 'spki')
                    return (this.hash.toString('base64'));
                return (sshBase64Format(this.algorithm, this.hash.toString('base64')));
            default:
                throw (new FingerprintFormatError(undefined, format));
        }
    };
    Fingerprint.prototype.matches = function (other) {
        assert.object(other, 'key or certificate');
        if (this.type === 'key' && this.hashType !== 'ssh') {
            utils.assertCompatible(other, Key, [1, 7], 'key with spki');
            if (PrivateKey.isPrivateKey(other)) {
                utils.assertCompatible(other, PrivateKey, [1, 6], 'privatekey with spki support');
            }
        }
        else if (this.type === 'key') {
            utils.assertCompatible(other, Key, [1, 0], 'key');
        }
        else {
            utils.assertCompatible(other, Certificate, [1, 0], 'certificate');
        }
        var theirHash = other.hash(this.algorithm, this.hashType);
        var theirHash2 = crypto.createHash(this.algorithm).
            update(theirHash).digest('base64');
        if (this.hash2 === undefined)
            this.hash2 = crypto.createHash(this.algorithm).
                update(this.hash).digest('base64');
        return (this.hash2 === theirHash2);
    };
    /*JSSTYLED*/
    var base64RE = /^[A-Za-z0-9+\/=]+$/;
    /*JSSTYLED*/
    var hexRE = /^[a-fA-F0-9]+$/;
    Fingerprint.parse = function (fp, options) {
        assert.string(fp, 'fingerprint');
        var alg, hash, enAlgs;
        if (Array.isArray(options)) {
            enAlgs = options;
            options = {};
        }
        assert.optionalObject(options, 'options');
        if (options === undefined)
            options = {};
        if (options.enAlgs !== undefined)
            enAlgs = options.enAlgs;
        if (options.algorithms !== undefined)
            enAlgs = options.algorithms;
        assert.optionalArrayOfString(enAlgs, 'algorithms');
        var hashType = 'ssh';
        if (options.hashType !== undefined)
            hashType = options.hashType;
        assert.string(hashType, 'options.hashType');
        var parts = fp.split(':');
        if (parts.length == 2) {
            alg = parts[0].toLowerCase();
            if (!base64RE.test(parts[1]))
                throw (new FingerprintFormatError(fp));
            try {
                hash = Buffer.from(parts[1], 'base64');
            }
            catch (e) {
                throw (new FingerprintFormatError(fp));
            }
        }
        else if (parts.length > 2) {
            alg = 'md5';
            if (parts[0].toLowerCase() === 'md5')
                parts = parts.slice(1);
            parts = parts.map(function (p) {
                while (p.length < 2)
                    p = '0' + p;
                if (p.length > 2)
                    throw (new FingerprintFormatError(fp));
                return (p);
            });
            parts = parts.join('');
            if (!hexRE.test(parts) || parts.length % 2 !== 0)
                throw (new FingerprintFormatError(fp));
            try {
                hash = Buffer.from(parts, 'hex');
            }
            catch (e) {
                throw (new FingerprintFormatError(fp));
            }
        }
        else {
            if (hexRE.test(fp)) {
                hash = Buffer.from(fp, 'hex');
            }
            else if (base64RE.test(fp)) {
                hash = Buffer.from(fp, 'base64');
            }
            else {
                throw (new FingerprintFormatError(fp));
            }
            switch (hash.length) {
                case 32:
                    alg = 'sha256';
                    break;
                case 16:
                    alg = 'md5';
                    break;
                case 20:
                    alg = 'sha1';
                    break;
                case 64:
                    alg = 'sha512';
                    break;
                default:
                    throw (new FingerprintFormatError(fp));
            }
            /* Plain hex/base64: guess it's probably SPKI unless told. */
            if (options.hashType === undefined)
                hashType = 'spki';
        }
        if (alg === undefined)
            throw (new FingerprintFormatError(fp));
        if (algs.hashAlgs[alg] === undefined)
            throw (new InvalidAlgorithmError(alg));
        if (enAlgs !== undefined) {
            enAlgs = enAlgs.map(function (a) { return a.toLowerCase(); });
            if (enAlgs.indexOf(alg) === -1)
                throw (new InvalidAlgorithmError(alg));
        }
        return (new Fingerprint({
            algorithm: alg,
            hash: hash,
            type: options.type || 'key',
            hashType: hashType
        }));
    };
    function addColons(s) {
        /*JSSTYLED*/
        return (s.replace(/(.{2})(?=.)/g, '$1:'));
    }
    function base64Strip(s) {
        /*JSSTYLED*/
        return (s.replace(/=*$/, ''));
    }
    function sshBase64Format(alg, h) {
        return (alg.toUpperCase() + ':' + base64Strip(h));
    }
    Fingerprint.isFingerprint = function (obj, ver) {
        return (utils.isCompatible(obj, Fingerprint, ver));
    };
    /*
     * API versions for Fingerprint:
     * [1,0] -- initial ver
     * [1,1] -- first tagged ver
     * [1,2] -- hashType and spki support
     */
    Fingerprint.prototype._sshpkApiVersion = [1, 2];
    Fingerprint._oldVersionDetect = function (obj) {
        assert.func(obj.toString);
        assert.func(obj.matches);
        return ([1, 0]);
    };
},
/* sshpk/lib/errors.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    var assert = require(853) /* assert-plus */;
    var util = require('util');
    function FingerprintFormatError(fp, format) {
        if (Error.captureStackTrace)
            Error.captureStackTrace(this, FingerprintFormatError);
        this.name = 'FingerprintFormatError';
        this.fingerprint = fp;
        this.format = format;
        this.message = 'Fingerprint format is not supported, or is invalid: ';
        if (fp !== undefined)
            this.message += ' fingerprint = ' + fp;
        if (format !== undefined)
            this.message += ' format = ' + format;
    }
    util.inherits(FingerprintFormatError, Error);
    function InvalidAlgorithmError(alg) {
        if (Error.captureStackTrace)
            Error.captureStackTrace(this, InvalidAlgorithmError);
        this.name = 'InvalidAlgorithmError';
        this.algorithm = alg;
        this.message = 'Algorithm "' + alg + '" is not supported';
    }
    util.inherits(InvalidAlgorithmError, Error);
    function KeyParseError(name, format, innerErr) {
        if (Error.captureStackTrace)
            Error.captureStackTrace(this, KeyParseError);
        this.name = 'KeyParseError';
        this.format = format;
        this.keyName = name;
        this.innerErr = innerErr;
        this.message = 'Failed to parse ' + name + ' as a valid ' + format +
            ' format key: ' + innerErr.message;
    }
    util.inherits(KeyParseError, Error);
    function SignatureParseError(type, format, innerErr) {
        if (Error.captureStackTrace)
            Error.captureStackTrace(this, SignatureParseError);
        this.name = 'SignatureParseError';
        this.type = type;
        this.format = format;
        this.innerErr = innerErr;
        this.message = 'Failed to parse the given data as a ' + type +
            ' signature in ' + format + ' format: ' + innerErr.message;
    }
    util.inherits(SignatureParseError, Error);
    function CertificateParseError(name, format, innerErr) {
        if (Error.captureStackTrace)
            Error.captureStackTrace(this, CertificateParseError);
        this.name = 'CertificateParseError';
        this.format = format;
        this.certName = name;
        this.innerErr = innerErr;
        this.message = 'Failed to parse ' + name + ' as a valid ' + format +
            ' format certificate: ' + innerErr.message;
    }
    util.inherits(CertificateParseError, Error);
    function KeyEncryptedError(name, format) {
        if (Error.captureStackTrace)
            Error.captureStackTrace(this, KeyEncryptedError);
        this.name = 'KeyEncryptedError';
        this.format = format;
        this.keyName = name;
        this.message = 'The ' + format + ' format key ' + name + ' is ' +
            'encrypted (password-protected), and no passphrase was ' +
            'provided in `options`';
    }
    util.inherits(KeyEncryptedError, Error);
    module.exports = {
        FingerprintFormatError: FingerprintFormatError,
        InvalidAlgorithmError: InvalidAlgorithmError,
        KeyParseError: KeyParseError,
        SignatureParseError: SignatureParseError,
        KeyEncryptedError: KeyEncryptedError,
        CertificateParseError: CertificateParseError
    };
},
/* sshpk/lib/private-key.js */ function _(require, module, exports) {
    // Copyright 2017 Joyent, Inc.
    module.exports = PrivateKey;
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ./algs */;
    var crypto = require('crypto');
    var Fingerprint = require(859) /* ./fingerprint */;
    var Signature = require(862) /* ./signature */;
    var errs = require(860) /* ./errors */;
    var util = require('util');
    var utils = require(863) /* ./utils */;
    var dhe = require(874) /* ./dhe */;
    var generateECDSA = dhe.generateECDSA;
    var generateED25519 = dhe.generateED25519;
    var edCompat = require(877) /* ./ed-compat */;
    var nacl = require(872) /* tweetnacl */;
    var Key = require(856) /* ./key */;
    var InvalidAlgorithmError = errs.InvalidAlgorithmError;
    var KeyParseError = errs.KeyParseError;
    var KeyEncryptedError = errs.KeyEncryptedError;
    var formats = {};
    formats['auto'] = require(878) /* ./formats/auto */;
    formats['pem'] = require(879) /* ./formats/pem */;
    formats['pkcs1'] = require(880) /* ./formats/pkcs1 */;
    formats['pkcs8'] = require(881) /* ./formats/pkcs8 */;
    formats['rfc4253'] = require(883) /* ./formats/rfc4253 */;
    formats['ssh-private'] = require(882) /* ./formats/ssh-private */;
    formats['openssh'] = formats['ssh-private'];
    formats['ssh'] = formats['ssh-private'];
    formats['dnssec'] = require(886) /* ./formats/dnssec */;
    function PrivateKey(opts) {
        assert.object(opts, 'options');
        Key.call(this, opts);
        this._pubCache = undefined;
    }
    util.inherits(PrivateKey, Key);
    PrivateKey.formats = formats;
    PrivateKey.prototype.toBuffer = function (format, options) {
        if (format === undefined)
            format = 'pkcs1';
        assert.string(format, 'format');
        assert.object(formats[format], 'formats[format]');
        assert.optionalObject(options, 'options');
        return (formats[format].write(this, options));
    };
    PrivateKey.prototype.hash = function (algo, type) {
        return (this.toPublic().hash(algo, type));
    };
    PrivateKey.prototype.fingerprint = function (algo, type) {
        return (this.toPublic().fingerprint(algo, type));
    };
    PrivateKey.prototype.toPublic = function () {
        if (this._pubCache)
            return (this._pubCache);
        var algInfo = algs.info[this.type];
        var pubParts = [];
        for (var i = 0; i < algInfo.parts.length; ++i) {
            var p = algInfo.parts[i];
            pubParts.push(this.part[p]);
        }
        this._pubCache = new Key({
            type: this.type,
            source: this,
            parts: pubParts
        });
        if (this.comment)
            this._pubCache.comment = this.comment;
        return (this._pubCache);
    };
    PrivateKey.prototype.derive = function (newType) {
        assert.string(newType, 'type');
        var priv, pub, pair;
        if (this.type === 'ed25519' && newType === 'curve25519') {
            priv = this.part.k.data;
            if (priv[0] === 0x00)
                priv = priv.slice(1);
            pair = nacl.box.keyPair.fromSecretKey(new Uint8Array(priv));
            pub = Buffer.from(pair.publicKey);
            return (new PrivateKey({
                type: 'curve25519',
                parts: [
                    { name: 'A', data: utils.mpNormalize(pub) },
                    { name: 'k', data: utils.mpNormalize(priv) }
                ]
            }));
        }
        else if (this.type === 'curve25519' && newType === 'ed25519') {
            priv = this.part.k.data;
            if (priv[0] === 0x00)
                priv = priv.slice(1);
            pair = nacl.sign.keyPair.fromSeed(new Uint8Array(priv));
            pub = Buffer.from(pair.publicKey);
            return (new PrivateKey({
                type: 'ed25519',
                parts: [
                    { name: 'A', data: utils.mpNormalize(pub) },
                    { name: 'k', data: utils.mpNormalize(priv) }
                ]
            }));
        }
        throw (new Error('Key derivation not supported from ' + this.type +
            ' to ' + newType));
    };
    PrivateKey.prototype.createVerify = function (hashAlgo) {
        return (this.toPublic().createVerify(hashAlgo));
    };
    PrivateKey.prototype.createSign = function (hashAlgo) {
        if (hashAlgo === undefined)
            hashAlgo = this.defaultHashAlgorithm();
        assert.string(hashAlgo, 'hash algorithm');
        /* ED25519 is not supported by OpenSSL, use a javascript impl. */
        if (this.type === 'ed25519' && edCompat !== undefined)
            return (new edCompat.Signer(this, hashAlgo));
        if (this.type === 'curve25519')
            throw (new Error('Curve25519 keys are not suitable for ' +
                'signing or verification'));
        var v, nm, err;
        try {
            nm = hashAlgo.toUpperCase();
            v = crypto.createSign(nm);
        }
        catch (e) {
            err = e;
        }
        if (v === undefined || (err instanceof Error &&
            err.message.match(/Unknown message digest/))) {
            nm = 'RSA-';
            nm += hashAlgo.toUpperCase();
            v = crypto.createSign(nm);
        }
        assert.ok(v, 'failed to create verifier');
        var oldSign = v.sign.bind(v);
        var key = this.toBuffer('pkcs1');
        var type = this.type;
        var curve = this.curve;
        v.sign = function () {
            var sig = oldSign(key);
            if (typeof (sig) === 'string')
                sig = Buffer.from(sig, 'binary');
            sig = Signature.parse(sig, type, 'asn1');
            sig.hashAlgorithm = hashAlgo;
            sig.curve = curve;
            return (sig);
        };
        return (v);
    };
    PrivateKey.parse = function (data, format, options) {
        if (typeof (data) !== 'string')
            assert.buffer(data, 'data');
        if (format === undefined)
            format = 'auto';
        assert.string(format, 'format');
        if (typeof (options) === 'string')
            options = { filename: options };
        assert.optionalObject(options, 'options');
        if (options === undefined)
            options = {};
        assert.optionalString(options.filename, 'options.filename');
        if (options.filename === undefined)
            options.filename = '(unnamed)';
        assert.object(formats[format], 'formats[format]');
        try {
            var k = formats[format].read(data, options);
            assert.ok(k instanceof PrivateKey, 'key is not a private key');
            if (!k.comment)
                k.comment = options.filename;
            return (k);
        }
        catch (e) {
            if (e.name === 'KeyEncryptedError')
                throw (e);
            throw (new KeyParseError(options.filename, format, e));
        }
    };
    PrivateKey.isPrivateKey = function (obj, ver) {
        return (utils.isCompatible(obj, PrivateKey, ver));
    };
    PrivateKey.generate = function (type, options) {
        if (options === undefined)
            options = {};
        assert.object(options, 'options');
        switch (type) {
            case 'ecdsa':
                if (options.curve === undefined)
                    options.curve = 'nistp256';
                assert.string(options.curve, 'options.curve');
                return (generateECDSA(options.curve));
            case 'ed25519':
                return (generateED25519());
            default:
                throw (new Error('Key generation not supported with key ' +
                    'type "' + type + '"'));
        }
    };
    /*
     * API versions for PrivateKey:
     * [1,0] -- initial ver
     * [1,1] -- added auto, pkcs[18], openssh/ssh-private formats
     * [1,2] -- added defaultHashAlgorithm
     * [1,3] -- added derive, ed, createDH
     * [1,4] -- first tagged version
     * [1,5] -- changed ed25519 part names and format
     * [1,6] -- type arguments for hash() and fingerprint()
     */
    PrivateKey.prototype._sshpkApiVersion = [1, 6];
    PrivateKey._oldVersionDetect = function (obj) {
        assert.func(obj.toPublic);
        assert.func(obj.createSign);
        if (obj.derive)
            return ([1, 3]);
        if (obj.defaultHashAlgorithm)
            return ([1, 2]);
        if (obj.formats['auto'])
            return ([1, 1]);
        return ([1, 0]);
    };
},
/* sshpk/lib/signature.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    module.exports = Signature;
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ./algs */;
    var crypto = require('crypto');
    var errs = require(860) /* ./errors */;
    var utils = require(863) /* ./utils */;
    var asn1 = require(864) /* asn1 */;
    var SSHBuffer = require(873) /* ./ssh-buffer */;
    var InvalidAlgorithmError = errs.InvalidAlgorithmError;
    var SignatureParseError = errs.SignatureParseError;
    function Signature(opts) {
        assert.object(opts, 'options');
        assert.arrayOfObject(opts.parts, 'options.parts');
        assert.string(opts.type, 'options.type');
        var partLookup = {};
        for (var i = 0; i < opts.parts.length; ++i) {
            var part = opts.parts[i];
            partLookup[part.name] = part;
        }
        this.type = opts.type;
        this.hashAlgorithm = opts.hashAlgo;
        this.curve = opts.curve;
        this.parts = opts.parts;
        this.part = partLookup;
    }
    Signature.prototype.toBuffer = function (format) {
        if (format === undefined)
            format = 'asn1';
        assert.string(format, 'format');
        var buf;
        var stype = 'ssh-' + this.type;
        switch (this.type) {
            case 'rsa':
                switch (this.hashAlgorithm) {
                    case 'sha256':
                        stype = 'rsa-sha2-256';
                        break;
                    case 'sha512':
                        stype = 'rsa-sha2-512';
                        break;
                    case 'sha1':
                    case undefined:
                        break;
                    default:
                        throw (new Error('SSH signature ' +
                            'format does not support hash ' +
                            'algorithm ' + this.hashAlgorithm));
                }
                if (format === 'ssh') {
                    buf = new SSHBuffer({});
                    buf.writeString(stype);
                    buf.writePart(this.part.sig);
                    return (buf.toBuffer());
                }
                else {
                    return (this.part.sig.data);
                }
                break;
            case 'ed25519':
                if (format === 'ssh') {
                    buf = new SSHBuffer({});
                    buf.writeString(stype);
                    buf.writePart(this.part.sig);
                    return (buf.toBuffer());
                }
                else {
                    return (this.part.sig.data);
                }
                break;
            case 'dsa':
            case 'ecdsa':
                var r, s;
                if (format === 'asn1') {
                    var der = new asn1.BerWriter();
                    der.startSequence();
                    r = utils.mpNormalize(this.part.r.data);
                    s = utils.mpNormalize(this.part.s.data);
                    der.writeBuffer(r, asn1.Ber.Integer);
                    der.writeBuffer(s, asn1.Ber.Integer);
                    der.endSequence();
                    return (der.buffer);
                }
                else if (format === 'ssh' && this.type === 'dsa') {
                    buf = new SSHBuffer({});
                    buf.writeString('ssh-dss');
                    r = this.part.r.data;
                    if (r.length > 20 && r[0] === 0x00)
                        r = r.slice(1);
                    s = this.part.s.data;
                    if (s.length > 20 && s[0] === 0x00)
                        s = s.slice(1);
                    if ((this.hashAlgorithm &&
                        this.hashAlgorithm !== 'sha1') ||
                        r.length + s.length !== 40) {
                        throw (new Error('OpenSSH only supports ' +
                            'DSA signatures with SHA1 hash'));
                    }
                    buf.writeBuffer(Buffer.concat([r, s]));
                    return (buf.toBuffer());
                }
                else if (format === 'ssh' && this.type === 'ecdsa') {
                    var inner = new SSHBuffer({});
                    r = this.part.r.data;
                    inner.writeBuffer(r);
                    inner.writePart(this.part.s);
                    buf = new SSHBuffer({});
                    /* XXX: find a more proper way to do this? */
                    var curve;
                    if (r[0] === 0x00)
                        r = r.slice(1);
                    var sz = r.length * 8;
                    if (sz === 256)
                        curve = 'nistp256';
                    else if (sz === 384)
                        curve = 'nistp384';
                    else if (sz === 528)
                        curve = 'nistp521';
                    buf.writeString('ecdsa-sha2-' + curve);
                    buf.writeBuffer(inner.toBuffer());
                    return (buf.toBuffer());
                }
                throw (new Error('Invalid signature format'));
            default:
                throw (new Error('Invalid signature data'));
        }
    };
    Signature.prototype.toString = function (format) {
        assert.optionalString(format, 'format');
        return (this.toBuffer(format).toString('base64'));
    };
    Signature.parse = function (data, type, format) {
        if (typeof (data) === 'string')
            data = Buffer.from(data, 'base64');
        assert.buffer(data, 'data');
        assert.string(format, 'format');
        assert.string(type, 'type');
        var opts = {};
        opts.type = type.toLowerCase();
        opts.parts = [];
        try {
            assert.ok(data.length > 0, 'signature must not be empty');
            switch (opts.type) {
                case 'rsa':
                    return (parseOneNum(data, type, format, opts));
                case 'ed25519':
                    return (parseOneNum(data, type, format, opts));
                case 'dsa':
                case 'ecdsa':
                    if (format === 'asn1')
                        return (parseDSAasn1(data, type, format, opts));
                    else if (opts.type === 'dsa')
                        return (parseDSA(data, type, format, opts));
                    else
                        return (parseECDSA(data, type, format, opts));
                default:
                    throw (new InvalidAlgorithmError(type));
            }
        }
        catch (e) {
            if (e instanceof InvalidAlgorithmError)
                throw (e);
            throw (new SignatureParseError(type, format, e));
        }
    };
    function parseOneNum(data, type, format, opts) {
        if (format === 'ssh') {
            try {
                var buf = new SSHBuffer({ buffer: data });
                var head = buf.readString();
            }
            catch (e) {
                /* fall through */
            }
            if (buf !== undefined) {
                var msg = 'SSH signature does not match expected ' +
                    'type (expected ' + type + ', got ' + head + ')';
                switch (head) {
                    case 'ssh-rsa':
                        assert.strictEqual(type, 'rsa', msg);
                        opts.hashAlgo = 'sha1';
                        break;
                    case 'rsa-sha2-256':
                        assert.strictEqual(type, 'rsa', msg);
                        opts.hashAlgo = 'sha256';
                        break;
                    case 'rsa-sha2-512':
                        assert.strictEqual(type, 'rsa', msg);
                        opts.hashAlgo = 'sha512';
                        break;
                    case 'ssh-ed25519':
                        assert.strictEqual(type, 'ed25519', msg);
                        opts.hashAlgo = 'sha512';
                        break;
                    default:
                        throw (new Error('Unknown SSH signature ' +
                            'type: ' + head));
                }
                var sig = buf.readPart();
                assert.ok(buf.atEnd(), 'extra trailing bytes');
                sig.name = 'sig';
                opts.parts.push(sig);
                return (new Signature(opts));
            }
        }
        opts.parts.push({ name: 'sig', data: data });
        return (new Signature(opts));
    }
    function parseDSAasn1(data, type, format, opts) {
        var der = new asn1.BerReader(data);
        der.readSequence();
        var r = der.readString(asn1.Ber.Integer, true);
        var s = der.readString(asn1.Ber.Integer, true);
        opts.parts.push({ name: 'r', data: utils.mpNormalize(r) });
        opts.parts.push({ name: 's', data: utils.mpNormalize(s) });
        return (new Signature(opts));
    }
    function parseDSA(data, type, format, opts) {
        if (data.length != 40) {
            var buf = new SSHBuffer({ buffer: data });
            var d = buf.readBuffer();
            if (d.toString('ascii') === 'ssh-dss')
                d = buf.readBuffer();
            assert.ok(buf.atEnd(), 'extra trailing bytes');
            assert.strictEqual(d.length, 40, 'invalid inner length');
            data = d;
        }
        opts.parts.push({ name: 'r', data: data.slice(0, 20) });
        opts.parts.push({ name: 's', data: data.slice(20, 40) });
        return (new Signature(opts));
    }
    function parseECDSA(data, type, format, opts) {
        var buf = new SSHBuffer({ buffer: data });
        var r, s;
        var inner = buf.readBuffer();
        var stype = inner.toString('ascii');
        if (stype.slice(0, 6) === 'ecdsa-') {
            var parts = stype.split('-');
            assert.strictEqual(parts[0], 'ecdsa');
            assert.strictEqual(parts[1], 'sha2');
            opts.curve = parts[2];
            switch (opts.curve) {
                case 'nistp256':
                    opts.hashAlgo = 'sha256';
                    break;
                case 'nistp384':
                    opts.hashAlgo = 'sha384';
                    break;
                case 'nistp521':
                    opts.hashAlgo = 'sha512';
                    break;
                default:
                    throw (new Error('Unsupported ECDSA curve: ' +
                        opts.curve));
            }
            inner = buf.readBuffer();
            assert.ok(buf.atEnd(), 'extra trailing bytes on outer');
            buf = new SSHBuffer({ buffer: inner });
            r = buf.readPart();
        }
        else {
            r = { data: inner };
        }
        s = buf.readPart();
        assert.ok(buf.atEnd(), 'extra trailing bytes');
        r.name = 'r';
        s.name = 's';
        opts.parts.push(r);
        opts.parts.push(s);
        return (new Signature(opts));
    }
    Signature.isSignature = function (obj, ver) {
        return (utils.isCompatible(obj, Signature, ver));
    };
    /*
     * API versions for Signature:
     * [1,0] -- initial ver
     * [2,0] -- support for rsa in full ssh format, compat with sshpk-agent
     *          hashAlgorithm property
     * [2,1] -- first tagged version
     */
    Signature.prototype._sshpkApiVersion = [2, 1];
    Signature._oldVersionDetect = function (obj) {
        assert.func(obj.toBuffer);
        if (obj.hasOwnProperty('hashAlgorithm'))
            return ([2, 0]);
        return ([1, 0]);
    };
},
/* sshpk/lib/utils.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    module.exports = {
        bufferSplit: bufferSplit,
        addRSAMissing: addRSAMissing,
        calculateDSAPublic: calculateDSAPublic,
        calculateED25519Public: calculateED25519Public,
        calculateX25519Public: calculateX25519Public,
        mpNormalize: mpNormalize,
        mpDenormalize: mpDenormalize,
        ecNormalize: ecNormalize,
        countZeros: countZeros,
        assertCompatible: assertCompatible,
        isCompatible: isCompatible,
        opensslKeyDeriv: opensslKeyDeriv,
        opensshCipherInfo: opensshCipherInfo,
        publicFromPrivateECDSA: publicFromPrivateECDSA,
        zeroPadToLength: zeroPadToLength,
        writeBitString: writeBitString,
        readBitString: readBitString,
        pbkdf2: pbkdf2
    };
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var PrivateKey = require(861) /* ./private-key */;
    var Key = require(856) /* ./key */;
    var crypto = require('crypto');
    var algs = require(857) /* ./algs */;
    var asn1 = require(864) /* asn1 */;
    var ec = require(870) /* ecc-jsbn/lib/ec */;
    var jsbn = require(871) /* jsbn */.BigInteger;
    var nacl = require(872) /* tweetnacl */;
    var MAX_CLASS_DEPTH = 3;
    function isCompatible(obj, klass, needVer) {
        if (obj === null || typeof (obj) !== 'object')
            return (false);
        if (needVer === undefined)
            needVer = klass.prototype._sshpkApiVersion;
        if (obj instanceof klass &&
            klass.prototype._sshpkApiVersion[0] == needVer[0])
            return (true);
        var proto = Object.getPrototypeOf(obj);
        var depth = 0;
        while (proto.constructor.name !== klass.name) {
            proto = Object.getPrototypeOf(proto);
            if (!proto || ++depth > MAX_CLASS_DEPTH)
                return (false);
        }
        if (proto.constructor.name !== klass.name)
            return (false);
        var ver = proto._sshpkApiVersion;
        if (ver === undefined)
            ver = klass._oldVersionDetect(obj);
        if (ver[0] != needVer[0] || ver[1] < needVer[1])
            return (false);
        return (true);
    }
    function assertCompatible(obj, klass, needVer, name) {
        if (name === undefined)
            name = 'object';
        assert.ok(obj, name + ' must not be null');
        assert.object(obj, name + ' must be an object');
        if (needVer === undefined)
            needVer = klass.prototype._sshpkApiVersion;
        if (obj instanceof klass &&
            klass.prototype._sshpkApiVersion[0] == needVer[0])
            return;
        var proto = Object.getPrototypeOf(obj);
        var depth = 0;
        while (proto.constructor.name !== klass.name) {
            proto = Object.getPrototypeOf(proto);
            assert.ok(proto && ++depth <= MAX_CLASS_DEPTH, name + ' must be a ' + klass.name + ' instance');
        }
        assert.strictEqual(proto.constructor.name, klass.name, name + ' must be a ' + klass.name + ' instance');
        var ver = proto._sshpkApiVersion;
        if (ver === undefined)
            ver = klass._oldVersionDetect(obj);
        assert.ok(ver[0] == needVer[0] && ver[1] >= needVer[1], name + ' must be compatible with ' + klass.name + ' klass ' +
            'version ' + needVer[0] + '.' + needVer[1]);
    }
    var CIPHER_LEN = {
        'des-ede3-cbc': { key: 24, iv: 8 },
        'aes-128-cbc': { key: 16, iv: 16 },
        'aes-256-cbc': { key: 32, iv: 16 }
    };
    var PKCS5_SALT_LEN = 8;
    function opensslKeyDeriv(cipher, salt, passphrase, count) {
        assert.buffer(salt, 'salt');
        assert.buffer(passphrase, 'passphrase');
        assert.number(count, 'iteration count');
        var clen = CIPHER_LEN[cipher];
        assert.object(clen, 'supported cipher');
        salt = salt.slice(0, PKCS5_SALT_LEN);
        var D, D_prev, bufs;
        var material = Buffer.alloc(0);
        while (material.length < clen.key + clen.iv) {
            bufs = [];
            if (D_prev)
                bufs.push(D_prev);
            bufs.push(passphrase);
            bufs.push(salt);
            D = Buffer.concat(bufs);
            for (var j = 0; j < count; ++j)
                D = crypto.createHash('md5').update(D).digest();
            material = Buffer.concat([material, D]);
            D_prev = D;
        }
        return ({
            key: material.slice(0, clen.key),
            iv: material.slice(clen.key, clen.key + clen.iv)
        });
    }
    /* See: RFC2898 */
    function pbkdf2(hashAlg, salt, iterations, size, passphrase) {
        var hkey = Buffer.alloc(salt.length + 4);
        salt.copy(hkey);
        var gen = 0, ts = [];
        var i = 1;
        while (gen < size) {
            var t = T(i++);
            gen += t.length;
            ts.push(t);
        }
        return (Buffer.concat(ts).slice(0, size));
        function T(I) {
            hkey.writeUInt32BE(I, hkey.length - 4);
            var hmac = crypto.createHmac(hashAlg, passphrase);
            hmac.update(hkey);
            var Ti = hmac.digest();
            var Uc = Ti;
            var c = 1;
            while (c++ < iterations) {
                hmac = crypto.createHmac(hashAlg, passphrase);
                hmac.update(Uc);
                Uc = hmac.digest();
                for (var x = 0; x < Ti.length; ++x)
                    Ti[x] ^= Uc[x];
            }
            return (Ti);
        }
    }
    /* Count leading zero bits on a buffer */
    function countZeros(buf) {
        var o = 0, obit = 8;
        while (o < buf.length) {
            var mask = (1 << obit);
            if ((buf[o] & mask) === mask)
                break;
            obit--;
            if (obit < 0) {
                o++;
                obit = 8;
            }
        }
        return (o * 8 + (8 - obit) - 1);
    }
    function bufferSplit(buf, chr) {
        assert.buffer(buf);
        assert.string(chr);
        var parts = [];
        var lastPart = 0;
        var matches = 0;
        for (var i = 0; i < buf.length; ++i) {
            if (buf[i] === chr.charCodeAt(matches))
                ++matches;
            else if (buf[i] === chr.charCodeAt(0))
                matches = 1;
            else
                matches = 0;
            if (matches >= chr.length) {
                var newPart = i + 1;
                parts.push(buf.slice(lastPart, newPart - matches));
                lastPart = newPart;
                matches = 0;
            }
        }
        if (lastPart <= buf.length)
            parts.push(buf.slice(lastPart, buf.length));
        return (parts);
    }
    function ecNormalize(buf, addZero) {
        assert.buffer(buf);
        if (buf[0] === 0x00 && buf[1] === 0x04) {
            if (addZero)
                return (buf);
            return (buf.slice(1));
        }
        else if (buf[0] === 0x04) {
            if (!addZero)
                return (buf);
        }
        else {
            while (buf[0] === 0x00)
                buf = buf.slice(1);
            if (buf[0] === 0x02 || buf[0] === 0x03)
                throw (new Error('Compressed elliptic curve points ' +
                    'are not supported'));
            if (buf[0] !== 0x04)
                throw (new Error('Not a valid elliptic curve point'));
            if (!addZero)
                return (buf);
        }
        var b = Buffer.alloc(buf.length + 1);
        b[0] = 0x0;
        buf.copy(b, 1);
        return (b);
    }
    function readBitString(der, tag) {
        if (tag === undefined)
            tag = asn1.Ber.BitString;
        var buf = der.readString(tag, true);
        assert.strictEqual(buf[0], 0x00, 'bit strings with unused bits are ' +
            'not supported (0x' + buf[0].toString(16) + ')');
        return (buf.slice(1));
    }
    function writeBitString(der, buf, tag) {
        if (tag === undefined)
            tag = asn1.Ber.BitString;
        var b = Buffer.alloc(buf.length + 1);
        b[0] = 0x00;
        buf.copy(b, 1);
        der.writeBuffer(b, tag);
    }
    function mpNormalize(buf) {
        assert.buffer(buf);
        while (buf.length > 1 && buf[0] === 0x00 && (buf[1] & 0x80) === 0x00)
            buf = buf.slice(1);
        if ((buf[0] & 0x80) === 0x80) {
            var b = Buffer.alloc(buf.length + 1);
            b[0] = 0x00;
            buf.copy(b, 1);
            buf = b;
        }
        return (buf);
    }
    function mpDenormalize(buf) {
        assert.buffer(buf);
        while (buf.length > 1 && buf[0] === 0x00)
            buf = buf.slice(1);
        return (buf);
    }
    function zeroPadToLength(buf, len) {
        assert.buffer(buf);
        assert.number(len);
        while (buf.length > len) {
            assert.equal(buf[0], 0x00);
            buf = buf.slice(1);
        }
        while (buf.length < len) {
            var b = Buffer.alloc(buf.length + 1);
            b[0] = 0x00;
            buf.copy(b, 1);
            buf = b;
        }
        return (buf);
    }
    function bigintToMpBuf(bigint) {
        var buf = Buffer.from(bigint.toByteArray());
        buf = mpNormalize(buf);
        return (buf);
    }
    function calculateDSAPublic(g, p, x) {
        assert.buffer(g);
        assert.buffer(p);
        assert.buffer(x);
        g = new jsbn(g);
        p = new jsbn(p);
        x = new jsbn(x);
        var y = g.modPow(x, p);
        var ybuf = bigintToMpBuf(y);
        return (ybuf);
    }
    function calculateED25519Public(k) {
        assert.buffer(k);
        var kp = nacl.sign.keyPair.fromSeed(new Uint8Array(k));
        return (Buffer.from(kp.publicKey));
    }
    function calculateX25519Public(k) {
        assert.buffer(k);
        var kp = nacl.box.keyPair.fromSeed(new Uint8Array(k));
        return (Buffer.from(kp.publicKey));
    }
    function addRSAMissing(key) {
        assert.object(key);
        assertCompatible(key, PrivateKey, [1, 1]);
        var d = new jsbn(key.part.d.data);
        var buf;
        if (!key.part.dmodp) {
            var p = new jsbn(key.part.p.data);
            var dmodp = d.mod(p.subtract(1));
            buf = bigintToMpBuf(dmodp);
            key.part.dmodp = { name: 'dmodp', data: buf };
            key.parts.push(key.part.dmodp);
        }
        if (!key.part.dmodq) {
            var q = new jsbn(key.part.q.data);
            var dmodq = d.mod(q.subtract(1));
            buf = bigintToMpBuf(dmodq);
            key.part.dmodq = { name: 'dmodq', data: buf };
            key.parts.push(key.part.dmodq);
        }
    }
    function publicFromPrivateECDSA(curveName, priv) {
        assert.string(curveName, 'curveName');
        assert.buffer(priv);
        var params = algs.curves[curveName];
        var p = new jsbn(params.p);
        var a = new jsbn(params.a);
        var b = new jsbn(params.b);
        var curve = new ec.ECCurveFp(p, a, b);
        var G = curve.decodePointHex(params.G.toString('hex'));
        var d = new jsbn(mpNormalize(priv));
        var pub = G.multiply(d);
        pub = Buffer.from(curve.encodePointHex(pub), 'hex');
        var parts = [];
        parts.push({ name: 'curve', data: Buffer.from(curveName) });
        parts.push({ name: 'Q', data: pub });
        var key = new Key({ type: 'ecdsa', curve: curve, parts: parts });
        return (key);
    }
    function opensshCipherInfo(cipher) {
        var inf = {};
        switch (cipher) {
            case '3des-cbc':
                inf.keySize = 24;
                inf.blockSize = 8;
                inf.opensslName = 'des-ede3-cbc';
                break;
            case 'blowfish-cbc':
                inf.keySize = 16;
                inf.blockSize = 8;
                inf.opensslName = 'bf-cbc';
                break;
            case 'aes128-cbc':
            case 'aes128-ctr':
            case 'aes128-gcm@openssh.com':
                inf.keySize = 16;
                inf.blockSize = 16;
                inf.opensslName = 'aes-128-' + cipher.slice(7, 10);
                break;
            case 'aes192-cbc':
            case 'aes192-ctr':
            case 'aes192-gcm@openssh.com':
                inf.keySize = 24;
                inf.blockSize = 16;
                inf.opensslName = 'aes-192-' + cipher.slice(7, 10);
                break;
            case 'aes256-cbc':
            case 'aes256-ctr':
            case 'aes256-gcm@openssh.com':
                inf.keySize = 32;
                inf.blockSize = 16;
                inf.opensslName = 'aes-256-' + cipher.slice(7, 10);
                break;
            default:
                throw (new Error('Unsupported openssl cipher "' + cipher + '"'));
        }
        return (inf);
    }
},
/* asn1/lib/index.js */ function _(require, module, exports) {
    // Copyright 2011 Mark Cavage <mcavage@gmail.com> All rights reserved.
    // If you have no idea what ASN.1 or BER is, see this:
    // ftp://ftp.rsa.com/pub/pkcs/ascii/layman.asc
    var Ber = require(865) /* ./ber/index */;
    // --- Exported API
    module.exports = {
        Ber: Ber,
        BerReader: Ber.Reader,
        BerWriter: Ber.Writer
    };
},
/* asn1/lib/ber/index.js */ function _(require, module, exports) {
    // Copyright 2011 Mark Cavage <mcavage@gmail.com> All rights reserved.
    var errors = require(866) /* ./errors */;
    var types = require(867) /* ./types */;
    var Reader = require(868) /* ./reader */;
    var Writer = require(869) /* ./writer */;
    // --- Exports
    module.exports = {
        Reader: Reader,
        Writer: Writer
    };
    for (var t in types) {
        if (types.hasOwnProperty(t))
            module.exports[t] = types[t];
    }
    for (var e in errors) {
        if (errors.hasOwnProperty(e))
            module.exports[e] = errors[e];
    }
},
/* asn1/lib/ber/errors.js */ function _(require, module, exports) {
    // Copyright 2011 Mark Cavage <mcavage@gmail.com> All rights reserved.
    module.exports = {
        newInvalidAsn1Error: function (msg) {
            var e = new Error();
            e.name = 'InvalidAsn1Error';
            e.message = msg || '';
            return e;
        }
    };
},
/* asn1/lib/ber/types.js */ function _(require, module, exports) {
    // Copyright 2011 Mark Cavage <mcavage@gmail.com> All rights reserved.
    module.exports = {
        EOC: 0,
        Boolean: 1,
        Integer: 2,
        BitString: 3,
        OctetString: 4,
        Null: 5,
        OID: 6,
        ObjectDescriptor: 7,
        External: 8,
        Real: 9,
        Enumeration: 10,
        PDV: 11,
        Utf8String: 12,
        RelativeOID: 13,
        Sequence: 16,
        Set: 17,
        NumericString: 18,
        PrintableString: 19,
        T61String: 20,
        VideotexString: 21,
        IA5String: 22,
        UTCTime: 23,
        GeneralizedTime: 24,
        GraphicString: 25,
        VisibleString: 26,
        GeneralString: 28,
        UniversalString: 29,
        CharacterString: 30,
        BMPString: 31,
        Constructor: 32,
        Context: 128
    };
},
/* asn1/lib/ber/reader.js */ function _(require, module, exports) {
    // Copyright 2011 Mark Cavage <mcavage@gmail.com> All rights reserved.
    var assert = require('assert');
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var ASN1 = require(867) /* ./types */;
    var errors = require(866) /* ./errors */;
    // --- Globals
    var newInvalidAsn1Error = errors.newInvalidAsn1Error;
    // --- API
    function Reader(data) {
        if (!data || !Buffer.isBuffer(data))
            throw new TypeError('data must be a node Buffer');
        this._buf = data;
        this._size = data.length;
        // These hold the "current" state
        this._len = 0;
        this._offset = 0;
    }
    Object.defineProperty(Reader.prototype, 'length', {
        enumerable: true,
        get: function () { return (this._len); }
    });
    Object.defineProperty(Reader.prototype, 'offset', {
        enumerable: true,
        get: function () { return (this._offset); }
    });
    Object.defineProperty(Reader.prototype, 'remain', {
        get: function () { return (this._size - this._offset); }
    });
    Object.defineProperty(Reader.prototype, 'buffer', {
        get: function () { return (this._buf.slice(this._offset)); }
    });
    /**
     * Reads a single byte and advances offset; you can pass in `true` to make this
     * a "peek" operation (i.e., get the byte, but don't advance the offset).
     *
     * @param {Boolean} peek true means don't move offset.
     * @return {Number} the next byte, null if not enough data.
     */
    Reader.prototype.readByte = function (peek) {
        if (this._size - this._offset < 1)
            return null;
        var b = this._buf[this._offset] & 0xff;
        if (!peek)
            this._offset += 1;
        return b;
    };
    Reader.prototype.peek = function () {
        return this.readByte(true);
    };
    /**
     * Reads a (potentially) variable length off the BER buffer.  This call is
     * not really meant to be called directly, as callers have to manipulate
     * the internal buffer afterwards.
     *
     * As a result of this call, you can call `Reader.length`, until the
     * next thing called that does a readLength.
     *
     * @return {Number} the amount of offset to advance the buffer.
     * @throws {InvalidAsn1Error} on bad ASN.1
     */
    Reader.prototype.readLength = function (offset) {
        if (offset === undefined)
            offset = this._offset;
        if (offset >= this._size)
            return null;
        var lenB = this._buf[offset++] & 0xff;
        if (lenB === null)
            return null;
        if ((lenB & 0x80) === 0x80) {
            lenB &= 0x7f;
            if (lenB === 0)
                throw newInvalidAsn1Error('Indefinite length not supported');
            if (lenB > 4)
                throw newInvalidAsn1Error('encoding too long');
            if (this._size - offset < lenB)
                return null;
            this._len = 0;
            for (var i = 0; i < lenB; i++)
                this._len = (this._len << 8) + (this._buf[offset++] & 0xff);
        }
        else {
            // Wasn't a variable length
            this._len = lenB;
        }
        return offset;
    };
    /**
     * Parses the next sequence in this BER buffer.
     *
     * To get the length of the sequence, call `Reader.length`.
     *
     * @return {Number} the sequence's tag.
     */
    Reader.prototype.readSequence = function (tag) {
        var seq = this.peek();
        if (seq === null)
            return null;
        if (tag !== undefined && tag !== seq)
            throw newInvalidAsn1Error('Expected 0x' + tag.toString(16) +
                ': got 0x' + seq.toString(16));
        var o = this.readLength(this._offset + 1); // stored in `length`
        if (o === null)
            return null;
        this._offset = o;
        return seq;
    };
    Reader.prototype.readInt = function () {
        return this._readTag(ASN1.Integer);
    };
    Reader.prototype.readBoolean = function () {
        return (this._readTag(ASN1.Boolean) === 0 ? false : true);
    };
    Reader.prototype.readEnumeration = function () {
        return this._readTag(ASN1.Enumeration);
    };
    Reader.prototype.readString = function (tag, retbuf) {
        if (!tag)
            tag = ASN1.OctetString;
        var b = this.peek();
        if (b === null)
            return null;
        if (b !== tag)
            throw newInvalidAsn1Error('Expected 0x' + tag.toString(16) +
                ': got 0x' + b.toString(16));
        var o = this.readLength(this._offset + 1); // stored in `length`
        if (o === null)
            return null;
        if (this.length > this._size - o)
            return null;
        this._offset = o;
        if (this.length === 0)
            return retbuf ? Buffer.alloc(0) : '';
        var str = this._buf.slice(this._offset, this._offset + this.length);
        this._offset += this.length;
        return retbuf ? str : str.toString('utf8');
    };
    Reader.prototype.readOID = function (tag) {
        if (!tag)
            tag = ASN1.OID;
        var b = this.readString(tag, true);
        if (b === null)
            return null;
        var values = [];
        var value = 0;
        for (var i = 0; i < b.length; i++) {
            var byte = b[i] & 0xff;
            value <<= 7;
            value += byte & 0x7f;
            if ((byte & 0x80) === 0) {
                values.push(value);
                value = 0;
            }
        }
        value = values.shift();
        values.unshift(value % 40);
        values.unshift((value / 40) >> 0);
        return values.join('.');
    };
    Reader.prototype._readTag = function (tag) {
        assert.ok(tag !== undefined);
        var b = this.peek();
        if (b === null)
            return null;
        if (b !== tag)
            throw newInvalidAsn1Error('Expected 0x' + tag.toString(16) +
                ': got 0x' + b.toString(16));
        var o = this.readLength(this._offset + 1); // stored in `length`
        if (o === null)
            return null;
        if (this.length > 4)
            throw newInvalidAsn1Error('Integer too long: ' + this.length);
        if (this.length > this._size - o)
            return null;
        this._offset = o;
        var fb = this._buf[this._offset];
        var value = 0;
        for (var i = 0; i < this.length; i++) {
            value <<= 8;
            value |= (this._buf[this._offset++] & 0xff);
        }
        if ((fb & 0x80) === 0x80 && i !== 4)
            value -= (1 << (i * 8));
        return value >> 0;
    };
    // --- Exported API
    module.exports = Reader;
},
/* asn1/lib/ber/writer.js */ function _(require, module, exports) {
    // Copyright 2011 Mark Cavage <mcavage@gmail.com> All rights reserved.
    var assert = require('assert');
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var ASN1 = require(867) /* ./types */;
    var errors = require(866) /* ./errors */;
    // --- Globals
    var newInvalidAsn1Error = errors.newInvalidAsn1Error;
    var DEFAULT_OPTS = {
        size: 1024,
        growthFactor: 8
    };
    // --- Helpers
    function merge(from, to) {
        assert.ok(from);
        assert.equal(typeof (from), 'object');
        assert.ok(to);
        assert.equal(typeof (to), 'object');
        var keys = Object.getOwnPropertyNames(from);
        keys.forEach(function (key) {
            if (to[key])
                return;
            var value = Object.getOwnPropertyDescriptor(from, key);
            Object.defineProperty(to, key, value);
        });
        return to;
    }
    // --- API
    function Writer(options) {
        options = merge(DEFAULT_OPTS, options || {});
        this._buf = Buffer.alloc(options.size || 1024);
        this._size = this._buf.length;
        this._offset = 0;
        this._options = options;
        // A list of offsets in the buffer where we need to insert
        // sequence tag/len pairs.
        this._seq = [];
    }
    Object.defineProperty(Writer.prototype, 'buffer', {
        get: function () {
            if (this._seq.length)
                throw newInvalidAsn1Error(this._seq.length + ' unended sequence(s)');
            return (this._buf.slice(0, this._offset));
        }
    });
    Writer.prototype.writeByte = function (b) {
        if (typeof (b) !== 'number')
            throw new TypeError('argument must be a Number');
        this._ensure(1);
        this._buf[this._offset++] = b;
    };
    Writer.prototype.writeInt = function (i, tag) {
        if (typeof (i) !== 'number')
            throw new TypeError('argument must be a Number');
        if (typeof (tag) !== 'number')
            tag = ASN1.Integer;
        var sz = 4;
        while ((((i & 0xff800000) === 0) || ((i & 0xff800000) === 0xff800000 >> 0)) &&
            (sz > 1)) {
            sz--;
            i <<= 8;
        }
        if (sz > 4)
            throw newInvalidAsn1Error('BER ints cannot be > 0xffffffff');
        this._ensure(2 + sz);
        this._buf[this._offset++] = tag;
        this._buf[this._offset++] = sz;
        while (sz-- > 0) {
            this._buf[this._offset++] = ((i & 0xff000000) >>> 24);
            i <<= 8;
        }
    };
    Writer.prototype.writeNull = function () {
        this.writeByte(ASN1.Null);
        this.writeByte(0x00);
    };
    Writer.prototype.writeEnumeration = function (i, tag) {
        if (typeof (i) !== 'number')
            throw new TypeError('argument must be a Number');
        if (typeof (tag) !== 'number')
            tag = ASN1.Enumeration;
        return this.writeInt(i, tag);
    };
    Writer.prototype.writeBoolean = function (b, tag) {
        if (typeof (b) !== 'boolean')
            throw new TypeError('argument must be a Boolean');
        if (typeof (tag) !== 'number')
            tag = ASN1.Boolean;
        this._ensure(3);
        this._buf[this._offset++] = tag;
        this._buf[this._offset++] = 0x01;
        this._buf[this._offset++] = b ? 0xff : 0x00;
    };
    Writer.prototype.writeString = function (s, tag) {
        if (typeof (s) !== 'string')
            throw new TypeError('argument must be a string (was: ' + typeof (s) + ')');
        if (typeof (tag) !== 'number')
            tag = ASN1.OctetString;
        var len = Buffer.byteLength(s);
        this.writeByte(tag);
        this.writeLength(len);
        if (len) {
            this._ensure(len);
            this._buf.write(s, this._offset);
            this._offset += len;
        }
    };
    Writer.prototype.writeBuffer = function (buf, tag) {
        if (typeof (tag) !== 'number')
            throw new TypeError('tag must be a number');
        if (!Buffer.isBuffer(buf))
            throw new TypeError('argument must be a buffer');
        this.writeByte(tag);
        this.writeLength(buf.length);
        this._ensure(buf.length);
        buf.copy(this._buf, this._offset, 0, buf.length);
        this._offset += buf.length;
    };
    Writer.prototype.writeStringArray = function (strings) {
        if ((!strings instanceof Array))
            throw new TypeError('argument must be an Array[String]');
        var self = this;
        strings.forEach(function (s) {
            self.writeString(s);
        });
    };
    // This is really to solve DER cases, but whatever for now
    Writer.prototype.writeOID = function (s, tag) {
        if (typeof (s) !== 'string')
            throw new TypeError('argument must be a string');
        if (typeof (tag) !== 'number')
            tag = ASN1.OID;
        if (!/^([0-9]+\.){3,}[0-9]+$/.test(s))
            throw new Error('argument is not a valid OID string');
        function encodeOctet(bytes, octet) {
            if (octet < 128) {
                bytes.push(octet);
            }
            else if (octet < 16384) {
                bytes.push((octet >>> 7) | 0x80);
                bytes.push(octet & 0x7F);
            }
            else if (octet < 2097152) {
                bytes.push((octet >>> 14) | 0x80);
                bytes.push(((octet >>> 7) | 0x80) & 0xFF);
                bytes.push(octet & 0x7F);
            }
            else if (octet < 268435456) {
                bytes.push((octet >>> 21) | 0x80);
                bytes.push(((octet >>> 14) | 0x80) & 0xFF);
                bytes.push(((octet >>> 7) | 0x80) & 0xFF);
                bytes.push(octet & 0x7F);
            }
            else {
                bytes.push(((octet >>> 28) | 0x80) & 0xFF);
                bytes.push(((octet >>> 21) | 0x80) & 0xFF);
                bytes.push(((octet >>> 14) | 0x80) & 0xFF);
                bytes.push(((octet >>> 7) | 0x80) & 0xFF);
                bytes.push(octet & 0x7F);
            }
        }
        var tmp = s.split('.');
        var bytes = [];
        bytes.push(parseInt(tmp[0], 10) * 40 + parseInt(tmp[1], 10));
        tmp.slice(2).forEach(function (b) {
            encodeOctet(bytes, parseInt(b, 10));
        });
        var self = this;
        this._ensure(2 + bytes.length);
        this.writeByte(tag);
        this.writeLength(bytes.length);
        bytes.forEach(function (b) {
            self.writeByte(b);
        });
    };
    Writer.prototype.writeLength = function (len) {
        if (typeof (len) !== 'number')
            throw new TypeError('argument must be a Number');
        this._ensure(4);
        if (len <= 0x7f) {
            this._buf[this._offset++] = len;
        }
        else if (len <= 0xff) {
            this._buf[this._offset++] = 0x81;
            this._buf[this._offset++] = len;
        }
        else if (len <= 0xffff) {
            this._buf[this._offset++] = 0x82;
            this._buf[this._offset++] = len >> 8;
            this._buf[this._offset++] = len;
        }
        else if (len <= 0xffffff) {
            this._buf[this._offset++] = 0x83;
            this._buf[this._offset++] = len >> 16;
            this._buf[this._offset++] = len >> 8;
            this._buf[this._offset++] = len;
        }
        else {
            throw newInvalidAsn1Error('Length too long (> 4 bytes)');
        }
    };
    Writer.prototype.startSequence = function (tag) {
        if (typeof (tag) !== 'number')
            tag = ASN1.Sequence | ASN1.Constructor;
        this.writeByte(tag);
        this._seq.push(this._offset);
        this._ensure(3);
        this._offset += 3;
    };
    Writer.prototype.endSequence = function () {
        var seq = this._seq.pop();
        var start = seq + 3;
        var len = this._offset - start;
        if (len <= 0x7f) {
            this._shift(start, len, -2);
            this._buf[seq] = len;
        }
        else if (len <= 0xff) {
            this._shift(start, len, -1);
            this._buf[seq] = 0x81;
            this._buf[seq + 1] = len;
        }
        else if (len <= 0xffff) {
            this._buf[seq] = 0x82;
            this._buf[seq + 1] = len >> 8;
            this._buf[seq + 2] = len;
        }
        else if (len <= 0xffffff) {
            this._shift(start, len, 1);
            this._buf[seq] = 0x83;
            this._buf[seq + 1] = len >> 16;
            this._buf[seq + 2] = len >> 8;
            this._buf[seq + 3] = len;
        }
        else {
            throw newInvalidAsn1Error('Sequence too long');
        }
    };
    Writer.prototype._shift = function (start, len, shift) {
        assert.ok(start !== undefined);
        assert.ok(len !== undefined);
        assert.ok(shift);
        this._buf.copy(this._buf, start + shift, start, start + len);
        this._offset += shift;
    };
    Writer.prototype._ensure = function (len) {
        assert.ok(len);
        if (this._size - this._offset < len) {
            var sz = this._size * this._options.growthFactor;
            if (sz - this._offset < len)
                sz += len;
            var buf = Buffer.alloc(sz);
            this._buf.copy(buf, 0, 0, this._offset);
            this._buf = buf;
            this._size = sz;
        }
    };
    // --- Exported API
    module.exports = Writer;
},
/* ecc-jsbn/lib/ec.js */ function _(require, module, exports) {
    // Basic Javascript Elliptic Curve implementation
    // Ported loosely from BouncyCastle's Java EC code
    // Only Fp curves implemented for now
    // Requires jsbn.js and jsbn2.js
    var BigInteger = require(871) /* jsbn */.BigInteger;
    var Barrett = BigInteger.prototype.Barrett;
    // ----------------
    // ECFieldElementFp
    // constructor
    function ECFieldElementFp(q, x) {
        this.x = x;
        // TODO if(x.compareTo(q) >= 0) error
        this.q = q;
    }
    function feFpEquals(other) {
        if (other == this)
            return true;
        return (this.q.equals(other.q) && this.x.equals(other.x));
    }
    function feFpToBigInteger() {
        return this.x;
    }
    function feFpNegate() {
        return new ECFieldElementFp(this.q, this.x.negate().mod(this.q));
    }
    function feFpAdd(b) {
        return new ECFieldElementFp(this.q, this.x.add(b.toBigInteger()).mod(this.q));
    }
    function feFpSubtract(b) {
        return new ECFieldElementFp(this.q, this.x.subtract(b.toBigInteger()).mod(this.q));
    }
    function feFpMultiply(b) {
        return new ECFieldElementFp(this.q, this.x.multiply(b.toBigInteger()).mod(this.q));
    }
    function feFpSquare() {
        return new ECFieldElementFp(this.q, this.x.square().mod(this.q));
    }
    function feFpDivide(b) {
        return new ECFieldElementFp(this.q, this.x.multiply(b.toBigInteger().modInverse(this.q)).mod(this.q));
    }
    ECFieldElementFp.prototype.equals = feFpEquals;
    ECFieldElementFp.prototype.toBigInteger = feFpToBigInteger;
    ECFieldElementFp.prototype.negate = feFpNegate;
    ECFieldElementFp.prototype.add = feFpAdd;
    ECFieldElementFp.prototype.subtract = feFpSubtract;
    ECFieldElementFp.prototype.multiply = feFpMultiply;
    ECFieldElementFp.prototype.square = feFpSquare;
    ECFieldElementFp.prototype.divide = feFpDivide;
    // ----------------
    // ECPointFp
    // constructor
    function ECPointFp(curve, x, y, z) {
        this.curve = curve;
        this.x = x;
        this.y = y;
        // Projective coordinates: either zinv == null or z * zinv == 1
        // z and zinv are just BigIntegers, not fieldElements
        if (z == null) {
            this.z = BigInteger.ONE;
        }
        else {
            this.z = z;
        }
        this.zinv = null;
        //TODO: compression flag
    }
    function pointFpGetX() {
        if (this.zinv == null) {
            this.zinv = this.z.modInverse(this.curve.q);
        }
        var r = this.x.toBigInteger().multiply(this.zinv);
        this.curve.reduce(r);
        return this.curve.fromBigInteger(r);
    }
    function pointFpGetY() {
        if (this.zinv == null) {
            this.zinv = this.z.modInverse(this.curve.q);
        }
        var r = this.y.toBigInteger().multiply(this.zinv);
        this.curve.reduce(r);
        return this.curve.fromBigInteger(r);
    }
    function pointFpEquals(other) {
        if (other == this)
            return true;
        if (this.isInfinity())
            return other.isInfinity();
        if (other.isInfinity())
            return this.isInfinity();
        var u, v;
        // u = Y2 * Z1 - Y1 * Z2
        u = other.y.toBigInteger().multiply(this.z).subtract(this.y.toBigInteger().multiply(other.z)).mod(this.curve.q);
        if (!u.equals(BigInteger.ZERO))
            return false;
        // v = X2 * Z1 - X1 * Z2
        v = other.x.toBigInteger().multiply(this.z).subtract(this.x.toBigInteger().multiply(other.z)).mod(this.curve.q);
        return v.equals(BigInteger.ZERO);
    }
    function pointFpIsInfinity() {
        if ((this.x == null) && (this.y == null))
            return true;
        return this.z.equals(BigInteger.ZERO) && !this.y.toBigInteger().equals(BigInteger.ZERO);
    }
    function pointFpNegate() {
        return new ECPointFp(this.curve, this.x, this.y.negate(), this.z);
    }
    function pointFpAdd(b) {
        if (this.isInfinity())
            return b;
        if (b.isInfinity())
            return this;
        // u = Y2 * Z1 - Y1 * Z2
        var u = b.y.toBigInteger().multiply(this.z).subtract(this.y.toBigInteger().multiply(b.z)).mod(this.curve.q);
        // v = X2 * Z1 - X1 * Z2
        var v = b.x.toBigInteger().multiply(this.z).subtract(this.x.toBigInteger().multiply(b.z)).mod(this.curve.q);
        if (BigInteger.ZERO.equals(v)) {
            if (BigInteger.ZERO.equals(u)) {
                return this.twice(); // this == b, so double
            }
            return this.curve.getInfinity(); // this = -b, so infinity
        }
        var THREE = new BigInteger("3");
        var x1 = this.x.toBigInteger();
        var y1 = this.y.toBigInteger();
        var x2 = b.x.toBigInteger();
        var y2 = b.y.toBigInteger();
        var v2 = v.square();
        var v3 = v2.multiply(v);
        var x1v2 = x1.multiply(v2);
        var zu2 = u.square().multiply(this.z);
        // x3 = v * (z2 * (z1 * u^2 - 2 * x1 * v^2) - v^3)
        var x3 = zu2.subtract(x1v2.shiftLeft(1)).multiply(b.z).subtract(v3).multiply(v).mod(this.curve.q);
        // y3 = z2 * (3 * x1 * u * v^2 - y1 * v^3 - z1 * u^3) + u * v^3
        var y3 = x1v2.multiply(THREE).multiply(u).subtract(y1.multiply(v3)).subtract(zu2.multiply(u)).multiply(b.z).add(u.multiply(v3)).mod(this.curve.q);
        // z3 = v^3 * z1 * z2
        var z3 = v3.multiply(this.z).multiply(b.z).mod(this.curve.q);
        return new ECPointFp(this.curve, this.curve.fromBigInteger(x3), this.curve.fromBigInteger(y3), z3);
    }
    function pointFpTwice() {
        if (this.isInfinity())
            return this;
        if (this.y.toBigInteger().signum() == 0)
            return this.curve.getInfinity();
        // TODO: optimized handling of constants
        var THREE = new BigInteger("3");
        var x1 = this.x.toBigInteger();
        var y1 = this.y.toBigInteger();
        var y1z1 = y1.multiply(this.z);
        var y1sqz1 = y1z1.multiply(y1).mod(this.curve.q);
        var a = this.curve.a.toBigInteger();
        // w = 3 * x1^2 + a * z1^2
        var w = x1.square().multiply(THREE);
        if (!BigInteger.ZERO.equals(a)) {
            w = w.add(this.z.square().multiply(a));
        }
        w = w.mod(this.curve.q);
        //this.curve.reduce(w);
        // x3 = 2 * y1 * z1 * (w^2 - 8 * x1 * y1^2 * z1)
        var x3 = w.square().subtract(x1.shiftLeft(3).multiply(y1sqz1)).shiftLeft(1).multiply(y1z1).mod(this.curve.q);
        // y3 = 4 * y1^2 * z1 * (3 * w * x1 - 2 * y1^2 * z1) - w^3
        var y3 = w.multiply(THREE).multiply(x1).subtract(y1sqz1.shiftLeft(1)).shiftLeft(2).multiply(y1sqz1).subtract(w.square().multiply(w)).mod(this.curve.q);
        // z3 = 8 * (y1 * z1)^3
        var z3 = y1z1.square().multiply(y1z1).shiftLeft(3).mod(this.curve.q);
        return new ECPointFp(this.curve, this.curve.fromBigInteger(x3), this.curve.fromBigInteger(y3), z3);
    }
    // Simple NAF (Non-Adjacent Form) multiplication algorithm
    // TODO: modularize the multiplication algorithm
    function pointFpMultiply(k) {
        if (this.isInfinity())
            return this;
        if (k.signum() == 0)
            return this.curve.getInfinity();
        var e = k;
        var h = e.multiply(new BigInteger("3"));
        var neg = this.negate();
        var R = this;
        var i;
        for (i = h.bitLength() - 2; i > 0; --i) {
            R = R.twice();
            var hBit = h.testBit(i);
            var eBit = e.testBit(i);
            if (hBit != eBit) {
                R = R.add(hBit ? this : neg);
            }
        }
        return R;
    }
    // Compute this*j + x*k (simultaneous multiplication)
    function pointFpMultiplyTwo(j, x, k) {
        var i;
        if (j.bitLength() > k.bitLength())
            i = j.bitLength() - 1;
        else
            i = k.bitLength() - 1;
        var R = this.curve.getInfinity();
        var both = this.add(x);
        while (i >= 0) {
            R = R.twice();
            if (j.testBit(i)) {
                if (k.testBit(i)) {
                    R = R.add(both);
                }
                else {
                    R = R.add(this);
                }
            }
            else {
                if (k.testBit(i)) {
                    R = R.add(x);
                }
            }
            --i;
        }
        return R;
    }
    ECPointFp.prototype.getX = pointFpGetX;
    ECPointFp.prototype.getY = pointFpGetY;
    ECPointFp.prototype.equals = pointFpEquals;
    ECPointFp.prototype.isInfinity = pointFpIsInfinity;
    ECPointFp.prototype.negate = pointFpNegate;
    ECPointFp.prototype.add = pointFpAdd;
    ECPointFp.prototype.twice = pointFpTwice;
    ECPointFp.prototype.multiply = pointFpMultiply;
    ECPointFp.prototype.multiplyTwo = pointFpMultiplyTwo;
    // ----------------
    // ECCurveFp
    // constructor
    function ECCurveFp(q, a, b) {
        this.q = q;
        this.a = this.fromBigInteger(a);
        this.b = this.fromBigInteger(b);
        this.infinity = new ECPointFp(this, null, null);
        this.reducer = new Barrett(this.q);
    }
    function curveFpGetQ() {
        return this.q;
    }
    function curveFpGetA() {
        return this.a;
    }
    function curveFpGetB() {
        return this.b;
    }
    function curveFpEquals(other) {
        if (other == this)
            return true;
        return (this.q.equals(other.q) && this.a.equals(other.a) && this.b.equals(other.b));
    }
    function curveFpGetInfinity() {
        return this.infinity;
    }
    function curveFpFromBigInteger(x) {
        return new ECFieldElementFp(this.q, x);
    }
    function curveReduce(x) {
        this.reducer.reduce(x);
    }
    // for now, work with hex strings because they're easier in JS
    function curveFpDecodePointHex(s) {
        switch (parseInt(s.substr(0, 2), 16)) { // first byte
            case 0:
                return this.infinity;
            case 2:
            case 3:
                // point compression not supported yet
                return null;
            case 4:
            case 6:
            case 7:
                var len = (s.length - 2) / 2;
                var xHex = s.substr(2, len);
                var yHex = s.substr(len + 2, len);
                return new ECPointFp(this, this.fromBigInteger(new BigInteger(xHex, 16)), this.fromBigInteger(new BigInteger(yHex, 16)));
            default: // unsupported
                return null;
        }
    }
    function curveFpEncodePointHex(p) {
        if (p.isInfinity())
            return "00";
        var xHex = p.getX().toBigInteger().toString(16);
        var yHex = p.getY().toBigInteger().toString(16);
        var oLen = this.getQ().toString(16).length;
        if ((oLen % 2) != 0)
            oLen++;
        while (xHex.length < oLen) {
            xHex = "0" + xHex;
        }
        while (yHex.length < oLen) {
            yHex = "0" + yHex;
        }
        return "04" + xHex + yHex;
    }
    ECCurveFp.prototype.getQ = curveFpGetQ;
    ECCurveFp.prototype.getA = curveFpGetA;
    ECCurveFp.prototype.getB = curveFpGetB;
    ECCurveFp.prototype.equals = curveFpEquals;
    ECCurveFp.prototype.getInfinity = curveFpGetInfinity;
    ECCurveFp.prototype.fromBigInteger = curveFpFromBigInteger;
    ECCurveFp.prototype.reduce = curveReduce;
    //ECCurveFp.prototype.decodePointHex = curveFpDecodePointHex;
    ECCurveFp.prototype.encodePointHex = curveFpEncodePointHex;
    // from: https://github.com/kaielvin/jsbn-ec-point-compression
    ECCurveFp.prototype.decodePointHex = function (s) {
        var yIsEven;
        switch (parseInt(s.substr(0, 2), 16)) { // first byte
            case 0:
                return this.infinity;
            case 2:
                yIsEven = false;
            case 3:
                if (yIsEven == undefined)
                    yIsEven = true;
                var len = s.length - 2;
                var xHex = s.substr(2, len);
                var x = this.fromBigInteger(new BigInteger(xHex, 16));
                var alpha = x.multiply(x.square().add(this.getA())).add(this.getB());
                var beta = alpha.sqrt();
                if (beta == null)
                    throw "Invalid point compression";
                var betaValue = beta.toBigInteger();
                if (betaValue.testBit(0) != yIsEven) {
                    // Use the other root
                    beta = this.fromBigInteger(this.getQ().subtract(betaValue));
                }
                return new ECPointFp(this, x, beta);
            case 4:
            case 6:
            case 7:
                var len = (s.length - 2) / 2;
                var xHex = s.substr(2, len);
                var yHex = s.substr(len + 2, len);
                return new ECPointFp(this, this.fromBigInteger(new BigInteger(xHex, 16)), this.fromBigInteger(new BigInteger(yHex, 16)));
            default: // unsupported
                return null;
        }
    };
    ECCurveFp.prototype.encodeCompressedPointHex = function (p) {
        if (p.isInfinity())
            return "00";
        var xHex = p.getX().toBigInteger().toString(16);
        var oLen = this.getQ().toString(16).length;
        if ((oLen % 2) != 0)
            oLen++;
        while (xHex.length < oLen)
            xHex = "0" + xHex;
        var yPrefix;
        if (p.getY().toBigInteger().isEven())
            yPrefix = "02";
        else
            yPrefix = "03";
        return yPrefix + xHex;
    };
    ECFieldElementFp.prototype.getR = function () {
        if (this.r != undefined)
            return this.r;
        this.r = null;
        var bitLength = this.q.bitLength();
        if (bitLength > 128) {
            var firstWord = this.q.shiftRight(bitLength - 64);
            if (firstWord.intValue() == -1) {
                this.r = BigInteger.ONE.shiftLeft(bitLength).subtract(this.q);
            }
        }
        return this.r;
    };
    ECFieldElementFp.prototype.modMult = function (x1, x2) {
        return this.modReduce(x1.multiply(x2));
    };
    ECFieldElementFp.prototype.modReduce = function (x) {
        if (this.getR() != null) {
            var qLen = q.bitLength();
            while (x.bitLength() > (qLen + 1)) {
                var u = x.shiftRight(qLen);
                var v = x.subtract(u.shiftLeft(qLen));
                if (!this.getR().equals(BigInteger.ONE)) {
                    u = u.multiply(this.getR());
                }
                x = u.add(v);
            }
            while (x.compareTo(q) >= 0) {
                x = x.subtract(q);
            }
        }
        else {
            x = x.mod(q);
        }
        return x;
    };
    ECFieldElementFp.prototype.sqrt = function () {
        if (!this.q.testBit(0))
            throw "unsupported";
        // p mod 4 == 3
        if (this.q.testBit(1)) {
            var z = new ECFieldElementFp(this.q, this.x.modPow(this.q.shiftRight(2).add(BigInteger.ONE), this.q));
            return z.square().equals(this) ? z : null;
        }
        // p mod 4 == 1
        var qMinusOne = this.q.subtract(BigInteger.ONE);
        var legendreExponent = qMinusOne.shiftRight(1);
        if (!(this.x.modPow(legendreExponent, this.q).equals(BigInteger.ONE))) {
            return null;
        }
        var u = qMinusOne.shiftRight(2);
        var k = u.shiftLeft(1).add(BigInteger.ONE);
        var Q = this.x;
        var fourQ = modDouble(modDouble(Q));
        var U, V;
        do {
            var P;
            do {
                P = new BigInteger(this.q.bitLength(), new SecureRandom());
            } while (P.compareTo(this.q) >= 0
                || !(P.multiply(P).subtract(fourQ).modPow(legendreExponent, this.q).equals(qMinusOne)));
            var result = this.lucasSequence(P, Q, k);
            U = result[0];
            V = result[1];
            if (this.modMult(V, V).equals(fourQ)) {
                // Integer division by 2, mod q
                if (V.testBit(0)) {
                    V = V.add(q);
                }
                V = V.shiftRight(1);
                return new ECFieldElementFp(q, V);
            }
        } while (U.equals(BigInteger.ONE) || U.equals(qMinusOne));
        return null;
    };
    ECFieldElementFp.prototype.lucasSequence = function (P, Q, k) {
        var n = k.bitLength();
        var s = k.getLowestSetBit();
        var Uh = BigInteger.ONE;
        var Vl = BigInteger.TWO;
        var Vh = P;
        var Ql = BigInteger.ONE;
        var Qh = BigInteger.ONE;
        for (var j = n - 1; j >= s + 1; --j) {
            Ql = this.modMult(Ql, Qh);
            if (k.testBit(j)) {
                Qh = this.modMult(Ql, Q);
                Uh = this.modMult(Uh, Vh);
                Vl = this.modReduce(Vh.multiply(Vl).subtract(P.multiply(Ql)));
                Vh = this.modReduce(Vh.multiply(Vh).subtract(Qh.shiftLeft(1)));
            }
            else {
                Qh = Ql;
                Uh = this.modReduce(Uh.multiply(Vl).subtract(Ql));
                Vh = this.modReduce(Vh.multiply(Vl).subtract(P.multiply(Ql)));
                Vl = this.modReduce(Vl.multiply(Vl).subtract(Ql.shiftLeft(1)));
            }
        }
        Ql = this.modMult(Ql, Qh);
        Qh = this.modMult(Ql, Q);
        Uh = this.modReduce(Uh.multiply(Vl).subtract(Ql));
        Vl = this.modReduce(Vh.multiply(Vl).subtract(P.multiply(Ql)));
        Ql = this.modMult(Ql, Qh);
        for (var j = 1; j <= s; ++j) {
            Uh = this.modMult(Uh, Vl);
            Vl = this.modReduce(Vl.multiply(Vl).subtract(Ql.shiftLeft(1)));
            Ql = this.modMult(Ql, Ql);
        }
        return [Uh, Vl];
    };
    var exports = {
        ECCurveFp: ECCurveFp,
        ECPointFp: ECPointFp,
        ECFieldElementFp: ECFieldElementFp
    };
    module.exports = exports;
},
/* jsbn/index.js */ function _(require, module, exports) {
    (function () {
        // Copyright (c) 2005  Tom Wu
        // All Rights Reserved.
        // See "LICENSE" for details.
        // Basic JavaScript BN library - subset useful for RSA encryption.
        // Bits per digit
        var dbits;
        // JavaScript engine analysis
        var canary = 0xdeadbeefcafe;
        var j_lm = ((canary & 0xffffff) == 0xefcafe);
        // (public) Constructor
        function BigInteger(a, b, c) {
            if (a != null)
                if ("number" == typeof a)
                    this.fromNumber(a, b, c);
                else if (b == null && "string" != typeof a)
                    this.fromString(a, 256);
                else
                    this.fromString(a, b);
        }
        // return new, unset BigInteger
        function nbi() { return new BigInteger(null); }
        // am: Compute w_j += (x*this_i), propagate carries,
        // c is initial carry, returns final carry.
        // c < 3*dvalue, x < 2*dvalue, this_i < dvalue
        // We need to select the fastest one that works in this environment.
        // am1: use a single mult and divide to get the high bits,
        // max digit bits should be 26 because
        // max internal value = 2*dvalue^2-2*dvalue (< 2^53)
        function am1(i, x, w, j, c, n) {
            while (--n >= 0) {
                var v = x * this[i++] + w[j] + c;
                c = Math.floor(v / 0x4000000);
                w[j++] = v & 0x3ffffff;
            }
            return c;
        }
        // am2 avoids a big mult-and-extract completely.
        // Max digit bits should be <= 30 because we do bitwise ops
        // on values up to 2*hdvalue^2-hdvalue-1 (< 2^31)
        function am2(i, x, w, j, c, n) {
            var xl = x & 0x7fff, xh = x >> 15;
            while (--n >= 0) {
                var l = this[i] & 0x7fff;
                var h = this[i++] >> 15;
                var m = xh * l + h * xl;
                l = xl * l + ((m & 0x7fff) << 15) + w[j] + (c & 0x3fffffff);
                c = (l >>> 30) + (m >>> 15) + xh * h + (c >>> 30);
                w[j++] = l & 0x3fffffff;
            }
            return c;
        }
        // Alternately, set max digit bits to 28 since some
        // browsers slow down when dealing with 32-bit numbers.
        function am3(i, x, w, j, c, n) {
            var xl = x & 0x3fff, xh = x >> 14;
            while (--n >= 0) {
                var l = this[i] & 0x3fff;
                var h = this[i++] >> 14;
                var m = xh * l + h * xl;
                l = xl * l + ((m & 0x3fff) << 14) + w[j] + c;
                c = (l >> 28) + (m >> 14) + xh * h;
                w[j++] = l & 0xfffffff;
            }
            return c;
        }
        var inBrowser = typeof navigator !== "undefined";
        if (inBrowser && j_lm && (navigator.appName == "Microsoft Internet Explorer")) {
            BigInteger.prototype.am = am2;
            dbits = 30;
        }
        else if (inBrowser && j_lm && (navigator.appName != "Netscape")) {
            BigInteger.prototype.am = am1;
            dbits = 26;
        }
        else { // Mozilla/Netscape seems to prefer am3
            BigInteger.prototype.am = am3;
            dbits = 28;
        }
        BigInteger.prototype.DB = dbits;
        BigInteger.prototype.DM = ((1 << dbits) - 1);
        BigInteger.prototype.DV = (1 << dbits);
        var BI_FP = 52;
        BigInteger.prototype.FV = Math.pow(2, BI_FP);
        BigInteger.prototype.F1 = BI_FP - dbits;
        BigInteger.prototype.F2 = 2 * dbits - BI_FP;
        // Digit conversions
        var BI_RM = "0123456789abcdefghijklmnopqrstuvwxyz";
        var BI_RC = new Array();
        var rr, vv;
        rr = "0".charCodeAt(0);
        for (vv = 0; vv <= 9; ++vv)
            BI_RC[rr++] = vv;
        rr = "a".charCodeAt(0);
        for (vv = 10; vv < 36; ++vv)
            BI_RC[rr++] = vv;
        rr = "A".charCodeAt(0);
        for (vv = 10; vv < 36; ++vv)
            BI_RC[rr++] = vv;
        function int2char(n) { return BI_RM.charAt(n); }
        function intAt(s, i) {
            var c = BI_RC[s.charCodeAt(i)];
            return (c == null) ? -1 : c;
        }
        // (protected) copy this to r
        function bnpCopyTo(r) {
            for (var i = this.t - 1; i >= 0; --i)
                r[i] = this[i];
            r.t = this.t;
            r.s = this.s;
        }
        // (protected) set from integer value x, -DV <= x < DV
        function bnpFromInt(x) {
            this.t = 1;
            this.s = (x < 0) ? -1 : 0;
            if (x > 0)
                this[0] = x;
            else if (x < -1)
                this[0] = x + this.DV;
            else
                this.t = 0;
        }
        // return bigint initialized to value
        function nbv(i) { var r = nbi(); r.fromInt(i); return r; }
        // (protected) set from string and radix
        function bnpFromString(s, b) {
            var k;
            if (b == 16)
                k = 4;
            else if (b == 8)
                k = 3;
            else if (b == 256)
                k = 8; // byte array
            else if (b == 2)
                k = 1;
            else if (b == 32)
                k = 5;
            else if (b == 4)
                k = 2;
            else {
                this.fromRadix(s, b);
                return;
            }
            this.t = 0;
            this.s = 0;
            var i = s.length, mi = false, sh = 0;
            while (--i >= 0) {
                var x = (k == 8) ? s[i] & 0xff : intAt(s, i);
                if (x < 0) {
                    if (s.charAt(i) == "-")
                        mi = true;
                    continue;
                }
                mi = false;
                if (sh == 0)
                    this[this.t++] = x;
                else if (sh + k > this.DB) {
                    this[this.t - 1] |= (x & ((1 << (this.DB - sh)) - 1)) << sh;
                    this[this.t++] = (x >> (this.DB - sh));
                }
                else
                    this[this.t - 1] |= x << sh;
                sh += k;
                if (sh >= this.DB)
                    sh -= this.DB;
            }
            if (k == 8 && (s[0] & 0x80) != 0) {
                this.s = -1;
                if (sh > 0)
                    this[this.t - 1] |= ((1 << (this.DB - sh)) - 1) << sh;
            }
            this.clamp();
            if (mi)
                BigInteger.ZERO.subTo(this, this);
        }
        // (protected) clamp off excess high words
        function bnpClamp() {
            var c = this.s & this.DM;
            while (this.t > 0 && this[this.t - 1] == c)
                --this.t;
        }
        // (public) return string representation in given radix
        function bnToString(b) {
            if (this.s < 0)
                return "-" + this.negate().toString(b);
            var k;
            if (b == 16)
                k = 4;
            else if (b == 8)
                k = 3;
            else if (b == 2)
                k = 1;
            else if (b == 32)
                k = 5;
            else if (b == 4)
                k = 2;
            else
                return this.toRadix(b);
            var km = (1 << k) - 1, d, m = false, r = "", i = this.t;
            var p = this.DB - (i * this.DB) % k;
            if (i-- > 0) {
                if (p < this.DB && (d = this[i] >> p) > 0) {
                    m = true;
                    r = int2char(d);
                }
                while (i >= 0) {
                    if (p < k) {
                        d = (this[i] & ((1 << p) - 1)) << (k - p);
                        d |= this[--i] >> (p += this.DB - k);
                    }
                    else {
                        d = (this[i] >> (p -= k)) & km;
                        if (p <= 0) {
                            p += this.DB;
                            --i;
                        }
                    }
                    if (d > 0)
                        m = true;
                    if (m)
                        r += int2char(d);
                }
            }
            return m ? r : "0";
        }
        // (public) -this
        function bnNegate() { var r = nbi(); BigInteger.ZERO.subTo(this, r); return r; }
        // (public) |this|
        function bnAbs() { return (this.s < 0) ? this.negate() : this; }
        // (public) return + if this > a, - if this < a, 0 if equal
        function bnCompareTo(a) {
            var r = this.s - a.s;
            if (r != 0)
                return r;
            var i = this.t;
            r = i - a.t;
            if (r != 0)
                return (this.s < 0) ? -r : r;
            while (--i >= 0)
                if ((r = this[i] - a[i]) != 0)
                    return r;
            return 0;
        }
        // returns bit length of the integer x
        function nbits(x) {
            var r = 1, t;
            if ((t = x >>> 16) != 0) {
                x = t;
                r += 16;
            }
            if ((t = x >> 8) != 0) {
                x = t;
                r += 8;
            }
            if ((t = x >> 4) != 0) {
                x = t;
                r += 4;
            }
            if ((t = x >> 2) != 0) {
                x = t;
                r += 2;
            }
            if ((t = x >> 1) != 0) {
                x = t;
                r += 1;
            }
            return r;
        }
        // (public) return the number of bits in "this"
        function bnBitLength() {
            if (this.t <= 0)
                return 0;
            return this.DB * (this.t - 1) + nbits(this[this.t - 1] ^ (this.s & this.DM));
        }
        // (protected) r = this << n*DB
        function bnpDLShiftTo(n, r) {
            var i;
            for (i = this.t - 1; i >= 0; --i)
                r[i + n] = this[i];
            for (i = n - 1; i >= 0; --i)
                r[i] = 0;
            r.t = this.t + n;
            r.s = this.s;
        }
        // (protected) r = this >> n*DB
        function bnpDRShiftTo(n, r) {
            for (var i = n; i < this.t; ++i)
                r[i - n] = this[i];
            r.t = Math.max(this.t - n, 0);
            r.s = this.s;
        }
        // (protected) r = this << n
        function bnpLShiftTo(n, r) {
            var bs = n % this.DB;
            var cbs = this.DB - bs;
            var bm = (1 << cbs) - 1;
            var ds = Math.floor(n / this.DB), c = (this.s << bs) & this.DM, i;
            for (i = this.t - 1; i >= 0; --i) {
                r[i + ds + 1] = (this[i] >> cbs) | c;
                c = (this[i] & bm) << bs;
            }
            for (i = ds - 1; i >= 0; --i)
                r[i] = 0;
            r[ds] = c;
            r.t = this.t + ds + 1;
            r.s = this.s;
            r.clamp();
        }
        // (protected) r = this >> n
        function bnpRShiftTo(n, r) {
            r.s = this.s;
            var ds = Math.floor(n / this.DB);
            if (ds >= this.t) {
                r.t = 0;
                return;
            }
            var bs = n % this.DB;
            var cbs = this.DB - bs;
            var bm = (1 << bs) - 1;
            r[0] = this[ds] >> bs;
            for (var i = ds + 1; i < this.t; ++i) {
                r[i - ds - 1] |= (this[i] & bm) << cbs;
                r[i - ds] = this[i] >> bs;
            }
            if (bs > 0)
                r[this.t - ds - 1] |= (this.s & bm) << cbs;
            r.t = this.t - ds;
            r.clamp();
        }
        // (protected) r = this - a
        function bnpSubTo(a, r) {
            var i = 0, c = 0, m = Math.min(a.t, this.t);
            while (i < m) {
                c += this[i] - a[i];
                r[i++] = c & this.DM;
                c >>= this.DB;
            }
            if (a.t < this.t) {
                c -= a.s;
                while (i < this.t) {
                    c += this[i];
                    r[i++] = c & this.DM;
                    c >>= this.DB;
                }
                c += this.s;
            }
            else {
                c += this.s;
                while (i < a.t) {
                    c -= a[i];
                    r[i++] = c & this.DM;
                    c >>= this.DB;
                }
                c -= a.s;
            }
            r.s = (c < 0) ? -1 : 0;
            if (c < -1)
                r[i++] = this.DV + c;
            else if (c > 0)
                r[i++] = c;
            r.t = i;
            r.clamp();
        }
        // (protected) r = this * a, r != this,a (HAC 14.12)
        // "this" should be the larger one if appropriate.
        function bnpMultiplyTo(a, r) {
            var x = this.abs(), y = a.abs();
            var i = x.t;
            r.t = i + y.t;
            while (--i >= 0)
                r[i] = 0;
            for (i = 0; i < y.t; ++i)
                r[i + x.t] = x.am(0, y[i], r, i, 0, x.t);
            r.s = 0;
            r.clamp();
            if (this.s != a.s)
                BigInteger.ZERO.subTo(r, r);
        }
        // (protected) r = this^2, r != this (HAC 14.16)
        function bnpSquareTo(r) {
            var x = this.abs();
            var i = r.t = 2 * x.t;
            while (--i >= 0)
                r[i] = 0;
            for (i = 0; i < x.t - 1; ++i) {
                var c = x.am(i, x[i], r, 2 * i, 0, 1);
                if ((r[i + x.t] += x.am(i + 1, 2 * x[i], r, 2 * i + 1, c, x.t - i - 1)) >= x.DV) {
                    r[i + x.t] -= x.DV;
                    r[i + x.t + 1] = 1;
                }
            }
            if (r.t > 0)
                r[r.t - 1] += x.am(i, x[i], r, 2 * i, 0, 1);
            r.s = 0;
            r.clamp();
        }
        // (protected) divide this by m, quotient and remainder to q, r (HAC 14.20)
        // r != q, this != m.  q or r may be null.
        function bnpDivRemTo(m, q, r) {
            var pm = m.abs();
            if (pm.t <= 0)
                return;
            var pt = this.abs();
            if (pt.t < pm.t) {
                if (q != null)
                    q.fromInt(0);
                if (r != null)
                    this.copyTo(r);
                return;
            }
            if (r == null)
                r = nbi();
            var y = nbi(), ts = this.s, ms = m.s;
            var nsh = this.DB - nbits(pm[pm.t - 1]); // normalize modulus
            if (nsh > 0) {
                pm.lShiftTo(nsh, y);
                pt.lShiftTo(nsh, r);
            }
            else {
                pm.copyTo(y);
                pt.copyTo(r);
            }
            var ys = y.t;
            var y0 = y[ys - 1];
            if (y0 == 0)
                return;
            var yt = y0 * (1 << this.F1) + ((ys > 1) ? y[ys - 2] >> this.F2 : 0);
            var d1 = this.FV / yt, d2 = (1 << this.F1) / yt, e = 1 << this.F2;
            var i = r.t, j = i - ys, t = (q == null) ? nbi() : q;
            y.dlShiftTo(j, t);
            if (r.compareTo(t) >= 0) {
                r[r.t++] = 1;
                r.subTo(t, r);
            }
            BigInteger.ONE.dlShiftTo(ys, t);
            t.subTo(y, y); // "negative" y so we can replace sub with am later
            while (y.t < ys)
                y[y.t++] = 0;
            while (--j >= 0) {
                // Estimate quotient digit
                var qd = (r[--i] == y0) ? this.DM : Math.floor(r[i] * d1 + (r[i - 1] + e) * d2);
                if ((r[i] += y.am(0, qd, r, j, 0, ys)) < qd) { // Try it out
                    y.dlShiftTo(j, t);
                    r.subTo(t, r);
                    while (r[i] < --qd)
                        r.subTo(t, r);
                }
            }
            if (q != null) {
                r.drShiftTo(ys, q);
                if (ts != ms)
                    BigInteger.ZERO.subTo(q, q);
            }
            r.t = ys;
            r.clamp();
            if (nsh > 0)
                r.rShiftTo(nsh, r); // Denormalize remainder
            if (ts < 0)
                BigInteger.ZERO.subTo(r, r);
        }
        // (public) this mod a
        function bnMod(a) {
            var r = nbi();
            this.abs().divRemTo(a, null, r);
            if (this.s < 0 && r.compareTo(BigInteger.ZERO) > 0)
                a.subTo(r, r);
            return r;
        }
        // Modular reduction using "classic" algorithm
        function Classic(m) { this.m = m; }
        function cConvert(x) {
            if (x.s < 0 || x.compareTo(this.m) >= 0)
                return x.mod(this.m);
            else
                return x;
        }
        function cRevert(x) { return x; }
        function cReduce(x) { x.divRemTo(this.m, null, x); }
        function cMulTo(x, y, r) { x.multiplyTo(y, r); this.reduce(r); }
        function cSqrTo(x, r) { x.squareTo(r); this.reduce(r); }
        Classic.prototype.convert = cConvert;
        Classic.prototype.revert = cRevert;
        Classic.prototype.reduce = cReduce;
        Classic.prototype.mulTo = cMulTo;
        Classic.prototype.sqrTo = cSqrTo;
        // (protected) return "-1/this % 2^DB"; useful for Mont. reduction
        // justification:
        //         xy == 1 (mod m)
        //         xy =  1+km
        //   xy(2-xy) = (1+km)(1-km)
        // x[y(2-xy)] = 1-k^2m^2
        // x[y(2-xy)] == 1 (mod m^2)
        // if y is 1/x mod m, then y(2-xy) is 1/x mod m^2
        // should reduce x and y(2-xy) by m^2 at each step to keep size bounded.
        // JS multiply "overflows" differently from C/C++, so care is needed here.
        function bnpInvDigit() {
            if (this.t < 1)
                return 0;
            var x = this[0];
            if ((x & 1) == 0)
                return 0;
            var y = x & 3; // y == 1/x mod 2^2
            y = (y * (2 - (x & 0xf) * y)) & 0xf; // y == 1/x mod 2^4
            y = (y * (2 - (x & 0xff) * y)) & 0xff; // y == 1/x mod 2^8
            y = (y * (2 - (((x & 0xffff) * y) & 0xffff))) & 0xffff; // y == 1/x mod 2^16
            // last step - calculate inverse mod DV directly;
            // assumes 16 < DB <= 32 and assumes ability to handle 48-bit ints
            y = (y * (2 - x * y % this.DV)) % this.DV; // y == 1/x mod 2^dbits
            // we really want the negative inverse, and -DV < y < DV
            return (y > 0) ? this.DV - y : -y;
        }
        // Montgomery reduction
        function Montgomery(m) {
            this.m = m;
            this.mp = m.invDigit();
            this.mpl = this.mp & 0x7fff;
            this.mph = this.mp >> 15;
            this.um = (1 << (m.DB - 15)) - 1;
            this.mt2 = 2 * m.t;
        }
        // xR mod m
        function montConvert(x) {
            var r = nbi();
            x.abs().dlShiftTo(this.m.t, r);
            r.divRemTo(this.m, null, r);
            if (x.s < 0 && r.compareTo(BigInteger.ZERO) > 0)
                this.m.subTo(r, r);
            return r;
        }
        // x/R mod m
        function montRevert(x) {
            var r = nbi();
            x.copyTo(r);
            this.reduce(r);
            return r;
        }
        // x = x/R mod m (HAC 14.32)
        function montReduce(x) {
            while (x.t <= this.mt2) // pad x so am has enough room later
                x[x.t++] = 0;
            for (var i = 0; i < this.m.t; ++i) {
                // faster way of calculating u0 = x[i]*mp mod DV
                var j = x[i] & 0x7fff;
                var u0 = (j * this.mpl + (((j * this.mph + (x[i] >> 15) * this.mpl) & this.um) << 15)) & x.DM;
                // use am to combine the multiply-shift-add into one call
                j = i + this.m.t;
                x[j] += this.m.am(0, u0, x, i, 0, this.m.t);
                // propagate carry
                while (x[j] >= x.DV) {
                    x[j] -= x.DV;
                    x[++j]++;
                }
            }
            x.clamp();
            x.drShiftTo(this.m.t, x);
            if (x.compareTo(this.m) >= 0)
                x.subTo(this.m, x);
        }
        // r = "x^2/R mod m"; x != r
        function montSqrTo(x, r) { x.squareTo(r); this.reduce(r); }
        // r = "xy/R mod m"; x,y != r
        function montMulTo(x, y, r) { x.multiplyTo(y, r); this.reduce(r); }
        Montgomery.prototype.convert = montConvert;
        Montgomery.prototype.revert = montRevert;
        Montgomery.prototype.reduce = montReduce;
        Montgomery.prototype.mulTo = montMulTo;
        Montgomery.prototype.sqrTo = montSqrTo;
        // (protected) true iff this is even
        function bnpIsEven() { return ((this.t > 0) ? (this[0] & 1) : this.s) == 0; }
        // (protected) this^e, e < 2^32, doing sqr and mul with "r" (HAC 14.79)
        function bnpExp(e, z) {
            if (e > 0xffffffff || e < 1)
                return BigInteger.ONE;
            var r = nbi(), r2 = nbi(), g = z.convert(this), i = nbits(e) - 1;
            g.copyTo(r);
            while (--i >= 0) {
                z.sqrTo(r, r2);
                if ((e & (1 << i)) > 0)
                    z.mulTo(r2, g, r);
                else {
                    var t = r;
                    r = r2;
                    r2 = t;
                }
            }
            return z.revert(r);
        }
        // (public) this^e % m, 0 <= e < 2^32
        function bnModPowInt(e, m) {
            var z;
            if (e < 256 || m.isEven())
                z = new Classic(m);
            else
                z = new Montgomery(m);
            return this.exp(e, z);
        }
        // protected
        BigInteger.prototype.copyTo = bnpCopyTo;
        BigInteger.prototype.fromInt = bnpFromInt;
        BigInteger.prototype.fromString = bnpFromString;
        BigInteger.prototype.clamp = bnpClamp;
        BigInteger.prototype.dlShiftTo = bnpDLShiftTo;
        BigInteger.prototype.drShiftTo = bnpDRShiftTo;
        BigInteger.prototype.lShiftTo = bnpLShiftTo;
        BigInteger.prototype.rShiftTo = bnpRShiftTo;
        BigInteger.prototype.subTo = bnpSubTo;
        BigInteger.prototype.multiplyTo = bnpMultiplyTo;
        BigInteger.prototype.squareTo = bnpSquareTo;
        BigInteger.prototype.divRemTo = bnpDivRemTo;
        BigInteger.prototype.invDigit = bnpInvDigit;
        BigInteger.prototype.isEven = bnpIsEven;
        BigInteger.prototype.exp = bnpExp;
        // public
        BigInteger.prototype.toString = bnToString;
        BigInteger.prototype.negate = bnNegate;
        BigInteger.prototype.abs = bnAbs;
        BigInteger.prototype.compareTo = bnCompareTo;
        BigInteger.prototype.bitLength = bnBitLength;
        BigInteger.prototype.mod = bnMod;
        BigInteger.prototype.modPowInt = bnModPowInt;
        // "constants"
        BigInteger.ZERO = nbv(0);
        BigInteger.ONE = nbv(1);
        // Copyright (c) 2005-2009  Tom Wu
        // All Rights Reserved.
        // See "LICENSE" for details.
        // Extended JavaScript BN functions, required for RSA private ops.
        // Version 1.1: new BigInteger("0", 10) returns "proper" zero
        // Version 1.2: square() API, isProbablePrime fix
        // (public)
        function bnClone() { var r = nbi(); this.copyTo(r); return r; }
        // (public) return value as integer
        function bnIntValue() {
            if (this.s < 0) {
                if (this.t == 1)
                    return this[0] - this.DV;
                else if (this.t == 0)
                    return -1;
            }
            else if (this.t == 1)
                return this[0];
            else if (this.t == 0)
                return 0;
            // assumes 16 < DB < 32
            return ((this[1] & ((1 << (32 - this.DB)) - 1)) << this.DB) | this[0];
        }
        // (public) return value as byte
        function bnByteValue() { return (this.t == 0) ? this.s : (this[0] << 24) >> 24; }
        // (public) return value as short (assumes DB>=16)
        function bnShortValue() { return (this.t == 0) ? this.s : (this[0] << 16) >> 16; }
        // (protected) return x s.t. r^x < DV
        function bnpChunkSize(r) { return Math.floor(Math.LN2 * this.DB / Math.log(r)); }
        // (public) 0 if this == 0, 1 if this > 0
        function bnSigNum() {
            if (this.s < 0)
                return -1;
            else if (this.t <= 0 || (this.t == 1 && this[0] <= 0))
                return 0;
            else
                return 1;
        }
        // (protected) convert to radix string
        function bnpToRadix(b) {
            if (b == null)
                b = 10;
            if (this.signum() == 0 || b < 2 || b > 36)
                return "0";
            var cs = this.chunkSize(b);
            var a = Math.pow(b, cs);
            var d = nbv(a), y = nbi(), z = nbi(), r = "";
            this.divRemTo(d, y, z);
            while (y.signum() > 0) {
                r = (a + z.intValue()).toString(b).substr(1) + r;
                y.divRemTo(d, y, z);
            }
            return z.intValue().toString(b) + r;
        }
        // (protected) convert from radix string
        function bnpFromRadix(s, b) {
            this.fromInt(0);
            if (b == null)
                b = 10;
            var cs = this.chunkSize(b);
            var d = Math.pow(b, cs), mi = false, j = 0, w = 0;
            for (var i = 0; i < s.length; ++i) {
                var x = intAt(s, i);
                if (x < 0) {
                    if (s.charAt(i) == "-" && this.signum() == 0)
                        mi = true;
                    continue;
                }
                w = b * w + x;
                if (++j >= cs) {
                    this.dMultiply(d);
                    this.dAddOffset(w, 0);
                    j = 0;
                    w = 0;
                }
            }
            if (j > 0) {
                this.dMultiply(Math.pow(b, j));
                this.dAddOffset(w, 0);
            }
            if (mi)
                BigInteger.ZERO.subTo(this, this);
        }
        // (protected) alternate constructor
        function bnpFromNumber(a, b, c) {
            if ("number" == typeof b) {
                // new BigInteger(int,int,RNG)
                if (a < 2)
                    this.fromInt(1);
                else {
                    this.fromNumber(a, c);
                    if (!this.testBit(a - 1)) // force MSB set
                        this.bitwiseTo(BigInteger.ONE.shiftLeft(a - 1), op_or, this);
                    if (this.isEven())
                        this.dAddOffset(1, 0); // force odd
                    while (!this.isProbablePrime(b)) {
                        this.dAddOffset(2, 0);
                        if (this.bitLength() > a)
                            this.subTo(BigInteger.ONE.shiftLeft(a - 1), this);
                    }
                }
            }
            else {
                // new BigInteger(int,RNG)
                var x = new Array(), t = a & 7;
                x.length = (a >> 3) + 1;
                b.nextBytes(x);
                if (t > 0)
                    x[0] &= ((1 << t) - 1);
                else
                    x[0] = 0;
                this.fromString(x, 256);
            }
        }
        // (public) convert to bigendian byte array
        function bnToByteArray() {
            var i = this.t, r = new Array();
            r[0] = this.s;
            var p = this.DB - (i * this.DB) % 8, d, k = 0;
            if (i-- > 0) {
                if (p < this.DB && (d = this[i] >> p) != (this.s & this.DM) >> p)
                    r[k++] = d | (this.s << (this.DB - p));
                while (i >= 0) {
                    if (p < 8) {
                        d = (this[i] & ((1 << p) - 1)) << (8 - p);
                        d |= this[--i] >> (p += this.DB - 8);
                    }
                    else {
                        d = (this[i] >> (p -= 8)) & 0xff;
                        if (p <= 0) {
                            p += this.DB;
                            --i;
                        }
                    }
                    if ((d & 0x80) != 0)
                        d |= -256;
                    if (k == 0 && (this.s & 0x80) != (d & 0x80))
                        ++k;
                    if (k > 0 || d != this.s)
                        r[k++] = d;
                }
            }
            return r;
        }
        function bnEquals(a) { return (this.compareTo(a) == 0); }
        function bnMin(a) { return (this.compareTo(a) < 0) ? this : a; }
        function bnMax(a) { return (this.compareTo(a) > 0) ? this : a; }
        // (protected) r = this op a (bitwise)
        function bnpBitwiseTo(a, op, r) {
            var i, f, m = Math.min(a.t, this.t);
            for (i = 0; i < m; ++i)
                r[i] = op(this[i], a[i]);
            if (a.t < this.t) {
                f = a.s & this.DM;
                for (i = m; i < this.t; ++i)
                    r[i] = op(this[i], f);
                r.t = this.t;
            }
            else {
                f = this.s & this.DM;
                for (i = m; i < a.t; ++i)
                    r[i] = op(f, a[i]);
                r.t = a.t;
            }
            r.s = op(this.s, a.s);
            r.clamp();
        }
        // (public) this & a
        function op_and(x, y) { return x & y; }
        function bnAnd(a) { var r = nbi(); this.bitwiseTo(a, op_and, r); return r; }
        // (public) this | a
        function op_or(x, y) { return x | y; }
        function bnOr(a) { var r = nbi(); this.bitwiseTo(a, op_or, r); return r; }
        // (public) this ^ a
        function op_xor(x, y) { return x ^ y; }
        function bnXor(a) { var r = nbi(); this.bitwiseTo(a, op_xor, r); return r; }
        // (public) this & ~a
        function op_andnot(x, y) { return x & ~y; }
        function bnAndNot(a) { var r = nbi(); this.bitwiseTo(a, op_andnot, r); return r; }
        // (public) ~this
        function bnNot() {
            var r = nbi();
            for (var i = 0; i < this.t; ++i)
                r[i] = this.DM & ~this[i];
            r.t = this.t;
            r.s = ~this.s;
            return r;
        }
        // (public) this << n
        function bnShiftLeft(n) {
            var r = nbi();
            if (n < 0)
                this.rShiftTo(-n, r);
            else
                this.lShiftTo(n, r);
            return r;
        }
        // (public) this >> n
        function bnShiftRight(n) {
            var r = nbi();
            if (n < 0)
                this.lShiftTo(-n, r);
            else
                this.rShiftTo(n, r);
            return r;
        }
        // return index of lowest 1-bit in x, x < 2^31
        function lbit(x) {
            if (x == 0)
                return -1;
            var r = 0;
            if ((x & 0xffff) == 0) {
                x >>= 16;
                r += 16;
            }
            if ((x & 0xff) == 0) {
                x >>= 8;
                r += 8;
            }
            if ((x & 0xf) == 0) {
                x >>= 4;
                r += 4;
            }
            if ((x & 3) == 0) {
                x >>= 2;
                r += 2;
            }
            if ((x & 1) == 0)
                ++r;
            return r;
        }
        // (public) returns index of lowest 1-bit (or -1 if none)
        function bnGetLowestSetBit() {
            for (var i = 0; i < this.t; ++i)
                if (this[i] != 0)
                    return i * this.DB + lbit(this[i]);
            if (this.s < 0)
                return this.t * this.DB;
            return -1;
        }
        // return number of 1 bits in x
        function cbit(x) {
            var r = 0;
            while (x != 0) {
                x &= x - 1;
                ++r;
            }
            return r;
        }
        // (public) return number of set bits
        function bnBitCount() {
            var r = 0, x = this.s & this.DM;
            for (var i = 0; i < this.t; ++i)
                r += cbit(this[i] ^ x);
            return r;
        }
        // (public) true iff nth bit is set
        function bnTestBit(n) {
            var j = Math.floor(n / this.DB);
            if (j >= this.t)
                return (this.s != 0);
            return ((this[j] & (1 << (n % this.DB))) != 0);
        }
        // (protected) this op (1<<n)
        function bnpChangeBit(n, op) {
            var r = BigInteger.ONE.shiftLeft(n);
            this.bitwiseTo(r, op, r);
            return r;
        }
        // (public) this | (1<<n)
        function bnSetBit(n) { return this.changeBit(n, op_or); }
        // (public) this & ~(1<<n)
        function bnClearBit(n) { return this.changeBit(n, op_andnot); }
        // (public) this ^ (1<<n)
        function bnFlipBit(n) { return this.changeBit(n, op_xor); }
        // (protected) r = this + a
        function bnpAddTo(a, r) {
            var i = 0, c = 0, m = Math.min(a.t, this.t);
            while (i < m) {
                c += this[i] + a[i];
                r[i++] = c & this.DM;
                c >>= this.DB;
            }
            if (a.t < this.t) {
                c += a.s;
                while (i < this.t) {
                    c += this[i];
                    r[i++] = c & this.DM;
                    c >>= this.DB;
                }
                c += this.s;
            }
            else {
                c += this.s;
                while (i < a.t) {
                    c += a[i];
                    r[i++] = c & this.DM;
                    c >>= this.DB;
                }
                c += a.s;
            }
            r.s = (c < 0) ? -1 : 0;
            if (c > 0)
                r[i++] = c;
            else if (c < -1)
                r[i++] = this.DV + c;
            r.t = i;
            r.clamp();
        }
        // (public) this + a
        function bnAdd(a) { var r = nbi(); this.addTo(a, r); return r; }
        // (public) this - a
        function bnSubtract(a) { var r = nbi(); this.subTo(a, r); return r; }
        // (public) this * a
        function bnMultiply(a) { var r = nbi(); this.multiplyTo(a, r); return r; }
        // (public) this^2
        function bnSquare() { var r = nbi(); this.squareTo(r); return r; }
        // (public) this / a
        function bnDivide(a) { var r = nbi(); this.divRemTo(a, r, null); return r; }
        // (public) this % a
        function bnRemainder(a) { var r = nbi(); this.divRemTo(a, null, r); return r; }
        // (public) [this/a,this%a]
        function bnDivideAndRemainder(a) {
            var q = nbi(), r = nbi();
            this.divRemTo(a, q, r);
            return new Array(q, r);
        }
        // (protected) this *= n, this >= 0, 1 < n < DV
        function bnpDMultiply(n) {
            this[this.t] = this.am(0, n - 1, this, 0, 0, this.t);
            ++this.t;
            this.clamp();
        }
        // (protected) this += n << w words, this >= 0
        function bnpDAddOffset(n, w) {
            if (n == 0)
                return;
            while (this.t <= w)
                this[this.t++] = 0;
            this[w] += n;
            while (this[w] >= this.DV) {
                this[w] -= this.DV;
                if (++w >= this.t)
                    this[this.t++] = 0;
                ++this[w];
            }
        }
        // A "null" reducer
        function NullExp() { }
        function nNop(x) { return x; }
        function nMulTo(x, y, r) { x.multiplyTo(y, r); }
        function nSqrTo(x, r) { x.squareTo(r); }
        NullExp.prototype.convert = nNop;
        NullExp.prototype.revert = nNop;
        NullExp.prototype.mulTo = nMulTo;
        NullExp.prototype.sqrTo = nSqrTo;
        // (public) this^e
        function bnPow(e) { return this.exp(e, new NullExp()); }
        // (protected) r = lower n words of "this * a", a.t <= n
        // "this" should be the larger one if appropriate.
        function bnpMultiplyLowerTo(a, n, r) {
            var i = Math.min(this.t + a.t, n);
            r.s = 0; // assumes a,this >= 0
            r.t = i;
            while (i > 0)
                r[--i] = 0;
            var j;
            for (j = r.t - this.t; i < j; ++i)
                r[i + this.t] = this.am(0, a[i], r, i, 0, this.t);
            for (j = Math.min(a.t, n); i < j; ++i)
                this.am(0, a[i], r, i, 0, n - i);
            r.clamp();
        }
        // (protected) r = "this * a" without lower n words, n > 0
        // "this" should be the larger one if appropriate.
        function bnpMultiplyUpperTo(a, n, r) {
            --n;
            var i = r.t = this.t + a.t - n;
            r.s = 0; // assumes a,this >= 0
            while (--i >= 0)
                r[i] = 0;
            for (i = Math.max(n - this.t, 0); i < a.t; ++i)
                r[this.t + i - n] = this.am(n - i, a[i], r, 0, 0, this.t + i - n);
            r.clamp();
            r.drShiftTo(1, r);
        }
        // Barrett modular reduction
        function Barrett(m) {
            // setup Barrett
            this.r2 = nbi();
            this.q3 = nbi();
            BigInteger.ONE.dlShiftTo(2 * m.t, this.r2);
            this.mu = this.r2.divide(m);
            this.m = m;
        }
        function barrettConvert(x) {
            if (x.s < 0 || x.t > 2 * this.m.t)
                return x.mod(this.m);
            else if (x.compareTo(this.m) < 0)
                return x;
            else {
                var r = nbi();
                x.copyTo(r);
                this.reduce(r);
                return r;
            }
        }
        function barrettRevert(x) { return x; }
        // x = x mod m (HAC 14.42)
        function barrettReduce(x) {
            x.drShiftTo(this.m.t - 1, this.r2);
            if (x.t > this.m.t + 1) {
                x.t = this.m.t + 1;
                x.clamp();
            }
            this.mu.multiplyUpperTo(this.r2, this.m.t + 1, this.q3);
            this.m.multiplyLowerTo(this.q3, this.m.t + 1, this.r2);
            while (x.compareTo(this.r2) < 0)
                x.dAddOffset(1, this.m.t + 1);
            x.subTo(this.r2, x);
            while (x.compareTo(this.m) >= 0)
                x.subTo(this.m, x);
        }
        // r = x^2 mod m; x != r
        function barrettSqrTo(x, r) { x.squareTo(r); this.reduce(r); }
        // r = x*y mod m; x,y != r
        function barrettMulTo(x, y, r) { x.multiplyTo(y, r); this.reduce(r); }
        Barrett.prototype.convert = barrettConvert;
        Barrett.prototype.revert = barrettRevert;
        Barrett.prototype.reduce = barrettReduce;
        Barrett.prototype.mulTo = barrettMulTo;
        Barrett.prototype.sqrTo = barrettSqrTo;
        // (public) this^e % m (HAC 14.85)
        function bnModPow(e, m) {
            var i = e.bitLength(), k, r = nbv(1), z;
            if (i <= 0)
                return r;
            else if (i < 18)
                k = 1;
            else if (i < 48)
                k = 3;
            else if (i < 144)
                k = 4;
            else if (i < 768)
                k = 5;
            else
                k = 6;
            if (i < 8)
                z = new Classic(m);
            else if (m.isEven())
                z = new Barrett(m);
            else
                z = new Montgomery(m);
            // precomputation
            var g = new Array(), n = 3, k1 = k - 1, km = (1 << k) - 1;
            g[1] = z.convert(this);
            if (k > 1) {
                var g2 = nbi();
                z.sqrTo(g[1], g2);
                while (n <= km) {
                    g[n] = nbi();
                    z.mulTo(g2, g[n - 2], g[n]);
                    n += 2;
                }
            }
            var j = e.t - 1, w, is1 = true, r2 = nbi(), t;
            i = nbits(e[j]) - 1;
            while (j >= 0) {
                if (i >= k1)
                    w = (e[j] >> (i - k1)) & km;
                else {
                    w = (e[j] & ((1 << (i + 1)) - 1)) << (k1 - i);
                    if (j > 0)
                        w |= e[j - 1] >> (this.DB + i - k1);
                }
                n = k;
                while ((w & 1) == 0) {
                    w >>= 1;
                    --n;
                }
                if ((i -= n) < 0) {
                    i += this.DB;
                    --j;
                }
                if (is1) { // ret == 1, don't bother squaring or multiplying it
                    g[w].copyTo(r);
                    is1 = false;
                }
                else {
                    while (n > 1) {
                        z.sqrTo(r, r2);
                        z.sqrTo(r2, r);
                        n -= 2;
                    }
                    if (n > 0)
                        z.sqrTo(r, r2);
                    else {
                        t = r;
                        r = r2;
                        r2 = t;
                    }
                    z.mulTo(r2, g[w], r);
                }
                while (j >= 0 && (e[j] & (1 << i)) == 0) {
                    z.sqrTo(r, r2);
                    t = r;
                    r = r2;
                    r2 = t;
                    if (--i < 0) {
                        i = this.DB - 1;
                        --j;
                    }
                }
            }
            return z.revert(r);
        }
        // (public) gcd(this,a) (HAC 14.54)
        function bnGCD(a) {
            var x = (this.s < 0) ? this.negate() : this.clone();
            var y = (a.s < 0) ? a.negate() : a.clone();
            if (x.compareTo(y) < 0) {
                var t = x;
                x = y;
                y = t;
            }
            var i = x.getLowestSetBit(), g = y.getLowestSetBit();
            if (g < 0)
                return x;
            if (i < g)
                g = i;
            if (g > 0) {
                x.rShiftTo(g, x);
                y.rShiftTo(g, y);
            }
            while (x.signum() > 0) {
                if ((i = x.getLowestSetBit()) > 0)
                    x.rShiftTo(i, x);
                if ((i = y.getLowestSetBit()) > 0)
                    y.rShiftTo(i, y);
                if (x.compareTo(y) >= 0) {
                    x.subTo(y, x);
                    x.rShiftTo(1, x);
                }
                else {
                    y.subTo(x, y);
                    y.rShiftTo(1, y);
                }
            }
            if (g > 0)
                y.lShiftTo(g, y);
            return y;
        }
        // (protected) this % n, n < 2^26
        function bnpModInt(n) {
            if (n <= 0)
                return 0;
            var d = this.DV % n, r = (this.s < 0) ? n - 1 : 0;
            if (this.t > 0)
                if (d == 0)
                    r = this[0] % n;
                else
                    for (var i = this.t - 1; i >= 0; --i)
                        r = (d * r + this[i]) % n;
            return r;
        }
        // (public) 1/this % m (HAC 14.61)
        function bnModInverse(m) {
            var ac = m.isEven();
            if ((this.isEven() && ac) || m.signum() == 0)
                return BigInteger.ZERO;
            var u = m.clone(), v = this.clone();
            var a = nbv(1), b = nbv(0), c = nbv(0), d = nbv(1);
            while (u.signum() != 0) {
                while (u.isEven()) {
                    u.rShiftTo(1, u);
                    if (ac) {
                        if (!a.isEven() || !b.isEven()) {
                            a.addTo(this, a);
                            b.subTo(m, b);
                        }
                        a.rShiftTo(1, a);
                    }
                    else if (!b.isEven())
                        b.subTo(m, b);
                    b.rShiftTo(1, b);
                }
                while (v.isEven()) {
                    v.rShiftTo(1, v);
                    if (ac) {
                        if (!c.isEven() || !d.isEven()) {
                            c.addTo(this, c);
                            d.subTo(m, d);
                        }
                        c.rShiftTo(1, c);
                    }
                    else if (!d.isEven())
                        d.subTo(m, d);
                    d.rShiftTo(1, d);
                }
                if (u.compareTo(v) >= 0) {
                    u.subTo(v, u);
                    if (ac)
                        a.subTo(c, a);
                    b.subTo(d, b);
                }
                else {
                    v.subTo(u, v);
                    if (ac)
                        c.subTo(a, c);
                    d.subTo(b, d);
                }
            }
            if (v.compareTo(BigInteger.ONE) != 0)
                return BigInteger.ZERO;
            if (d.compareTo(m) >= 0)
                return d.subtract(m);
            if (d.signum() < 0)
                d.addTo(m, d);
            else
                return d;
            if (d.signum() < 0)
                return d.add(m);
            else
                return d;
        }
        var lowprimes = [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127, 131, 137, 139, 149, 151, 157, 163, 167, 173, 179, 181, 191, 193, 197, 199, 211, 223, 227, 229, 233, 239, 241, 251, 257, 263, 269, 271, 277, 281, 283, 293, 307, 311, 313, 317, 331, 337, 347, 349, 353, 359, 367, 373, 379, 383, 389, 397, 401, 409, 419, 421, 431, 433, 439, 443, 449, 457, 461, 463, 467, 479, 487, 491, 499, 503, 509, 521, 523, 541, 547, 557, 563, 569, 571, 577, 587, 593, 599, 601, 607, 613, 617, 619, 631, 641, 643, 647, 653, 659, 661, 673, 677, 683, 691, 701, 709, 719, 727, 733, 739, 743, 751, 757, 761, 769, 773, 787, 797, 809, 811, 821, 823, 827, 829, 839, 853, 857, 859, 863, 877, 881, 883, 887, 907, 911, 919, 929, 937, 941, 947, 953, 967, 971, 977, 983, 991, 997];
        var lplim = (1 << 26) / lowprimes[lowprimes.length - 1];
        // (public) test primality with certainty >= 1-.5^t
        function bnIsProbablePrime(t) {
            var i, x = this.abs();
            if (x.t == 1 && x[0] <= lowprimes[lowprimes.length - 1]) {
                for (i = 0; i < lowprimes.length; ++i)
                    if (x[0] == lowprimes[i])
                        return true;
                return false;
            }
            if (x.isEven())
                return false;
            i = 1;
            while (i < lowprimes.length) {
                var m = lowprimes[i], j = i + 1;
                while (j < lowprimes.length && m < lplim)
                    m *= lowprimes[j++];
                m = x.modInt(m);
                while (i < j)
                    if (m % lowprimes[i++] == 0)
                        return false;
            }
            return x.millerRabin(t);
        }
        // (protected) true if probably prime (HAC 4.24, Miller-Rabin)
        function bnpMillerRabin(t) {
            var n1 = this.subtract(BigInteger.ONE);
            var k = n1.getLowestSetBit();
            if (k <= 0)
                return false;
            var r = n1.shiftRight(k);
            t = (t + 1) >> 1;
            if (t > lowprimes.length)
                t = lowprimes.length;
            var a = nbi();
            for (var i = 0; i < t; ++i) {
                //Pick bases at random, instead of starting at 2
                a.fromInt(lowprimes[Math.floor(Math.random() * lowprimes.length)]);
                var y = a.modPow(r, this);
                if (y.compareTo(BigInteger.ONE) != 0 && y.compareTo(n1) != 0) {
                    var j = 1;
                    while (j++ < k && y.compareTo(n1) != 0) {
                        y = y.modPowInt(2, this);
                        if (y.compareTo(BigInteger.ONE) == 0)
                            return false;
                    }
                    if (y.compareTo(n1) != 0)
                        return false;
                }
            }
            return true;
        }
        // protected
        BigInteger.prototype.chunkSize = bnpChunkSize;
        BigInteger.prototype.toRadix = bnpToRadix;
        BigInteger.prototype.fromRadix = bnpFromRadix;
        BigInteger.prototype.fromNumber = bnpFromNumber;
        BigInteger.prototype.bitwiseTo = bnpBitwiseTo;
        BigInteger.prototype.changeBit = bnpChangeBit;
        BigInteger.prototype.addTo = bnpAddTo;
        BigInteger.prototype.dMultiply = bnpDMultiply;
        BigInteger.prototype.dAddOffset = bnpDAddOffset;
        BigInteger.prototype.multiplyLowerTo = bnpMultiplyLowerTo;
        BigInteger.prototype.multiplyUpperTo = bnpMultiplyUpperTo;
        BigInteger.prototype.modInt = bnpModInt;
        BigInteger.prototype.millerRabin = bnpMillerRabin;
        // public
        BigInteger.prototype.clone = bnClone;
        BigInteger.prototype.intValue = bnIntValue;
        BigInteger.prototype.byteValue = bnByteValue;
        BigInteger.prototype.shortValue = bnShortValue;
        BigInteger.prototype.signum = bnSigNum;
        BigInteger.prototype.toByteArray = bnToByteArray;
        BigInteger.prototype.equals = bnEquals;
        BigInteger.prototype.min = bnMin;
        BigInteger.prototype.max = bnMax;
        BigInteger.prototype.and = bnAnd;
        BigInteger.prototype.or = bnOr;
        BigInteger.prototype.xor = bnXor;
        BigInteger.prototype.andNot = bnAndNot;
        BigInteger.prototype.not = bnNot;
        BigInteger.prototype.shiftLeft = bnShiftLeft;
        BigInteger.prototype.shiftRight = bnShiftRight;
        BigInteger.prototype.getLowestSetBit = bnGetLowestSetBit;
        BigInteger.prototype.bitCount = bnBitCount;
        BigInteger.prototype.testBit = bnTestBit;
        BigInteger.prototype.setBit = bnSetBit;
        BigInteger.prototype.clearBit = bnClearBit;
        BigInteger.prototype.flipBit = bnFlipBit;
        BigInteger.prototype.add = bnAdd;
        BigInteger.prototype.subtract = bnSubtract;
        BigInteger.prototype.multiply = bnMultiply;
        BigInteger.prototype.divide = bnDivide;
        BigInteger.prototype.remainder = bnRemainder;
        BigInteger.prototype.divideAndRemainder = bnDivideAndRemainder;
        BigInteger.prototype.modPow = bnModPow;
        BigInteger.prototype.modInverse = bnModInverse;
        BigInteger.prototype.pow = bnPow;
        BigInteger.prototype.gcd = bnGCD;
        BigInteger.prototype.isProbablePrime = bnIsProbablePrime;
        // JSBN-specific extension
        BigInteger.prototype.square = bnSquare;
        // Expose the Barrett function
        BigInteger.prototype.Barrett = Barrett;
        // BigInteger interfaces not implemented in jsbn:
        // BigInteger(int signum, byte[] magnitude)
        // double doubleValue()
        // float floatValue()
        // int hashCode()
        // long longValue()
        // static BigInteger valueOf(long val)
        // Random number generator - requires a PRNG backend, e.g. prng4.js
        // For best results, put code like
        // <body onClick='rng_seed_time();' onKeyPress='rng_seed_time();'>
        // in your main HTML document.
        var rng_state;
        var rng_pool;
        var rng_pptr;
        // Mix in a 32-bit integer into the pool
        function rng_seed_int(x) {
            rng_pool[rng_pptr++] ^= x & 255;
            rng_pool[rng_pptr++] ^= (x >> 8) & 255;
            rng_pool[rng_pptr++] ^= (x >> 16) & 255;
            rng_pool[rng_pptr++] ^= (x >> 24) & 255;
            if (rng_pptr >= rng_psize)
                rng_pptr -= rng_psize;
        }
        // Mix in the current time (w/milliseconds) into the pool
        function rng_seed_time() {
            rng_seed_int(new Date().getTime());
        }
        // Initialize the pool with junk if needed.
        if (rng_pool == null) {
            rng_pool = new Array();
            rng_pptr = 0;
            var t;
            if (typeof window !== "undefined" && window.crypto) {
                if (window.crypto.getRandomValues) {
                    // Use webcrypto if available
                    var ua = new Uint8Array(32);
                    window.crypto.getRandomValues(ua);
                    for (t = 0; t < 32; ++t)
                        rng_pool[rng_pptr++] = ua[t];
                }
                else if (navigator.appName == "Netscape" && navigator.appVersion < "5") {
                    // Extract entropy (256 bits) from NS4 RNG if available
                    var z = window.crypto.random(32);
                    for (t = 0; t < z.length; ++t)
                        rng_pool[rng_pptr++] = z.charCodeAt(t) & 255;
                }
            }
            while (rng_pptr < rng_psize) { // extract some randomness from Math.random()
                t = Math.floor(65536 * Math.random());
                rng_pool[rng_pptr++] = t >>> 8;
                rng_pool[rng_pptr++] = t & 255;
            }
            rng_pptr = 0;
            rng_seed_time();
            //rng_seed_int(window.screenX);
            //rng_seed_int(window.screenY);
        }
        function rng_get_byte() {
            if (rng_state == null) {
                rng_seed_time();
                rng_state = prng_newstate();
                rng_state.init(rng_pool);
                for (rng_pptr = 0; rng_pptr < rng_pool.length; ++rng_pptr)
                    rng_pool[rng_pptr] = 0;
                rng_pptr = 0;
                //rng_pool = null;
            }
            // TODO: allow reseeding after first request
            return rng_state.next();
        }
        function rng_get_bytes(ba) {
            var i;
            for (i = 0; i < ba.length; ++i)
                ba[i] = rng_get_byte();
        }
        function SecureRandom() { }
        SecureRandom.prototype.nextBytes = rng_get_bytes;
        // prng4.js - uses Arcfour as a PRNG
        function Arcfour() {
            this.i = 0;
            this.j = 0;
            this.S = new Array();
        }
        // Initialize arcfour context from key, an array of ints, each from [0..255]
        function ARC4init(key) {
            var i, j, t;
            for (i = 0; i < 256; ++i)
                this.S[i] = i;
            j = 0;
            for (i = 0; i < 256; ++i) {
                j = (j + this.S[i] + key[i % key.length]) & 255;
                t = this.S[i];
                this.S[i] = this.S[j];
                this.S[j] = t;
            }
            this.i = 0;
            this.j = 0;
        }
        function ARC4next() {
            var t;
            this.i = (this.i + 1) & 255;
            this.j = (this.j + this.S[this.i]) & 255;
            t = this.S[this.i];
            this.S[this.i] = this.S[this.j];
            this.S[this.j] = t;
            return this.S[(t + this.S[this.i]) & 255];
        }
        Arcfour.prototype.init = ARC4init;
        Arcfour.prototype.next = ARC4next;
        // Plug in your RNG constructor here
        function prng_newstate() {
            return new Arcfour();
        }
        // Pool size must be a multiple of 4 and greater than 32.
        // An array of bytes the size of the pool will be passed to init()
        var rng_psize = 256;
        BigInteger.SecureRandom = SecureRandom;
        BigInteger.BigInteger = BigInteger;
        if (typeof exports !== 'undefined') {
            exports = module.exports = BigInteger;
        }
        else {
            this.BigInteger = BigInteger;
            this.SecureRandom = SecureRandom;
        }
    }).call(this);
},
/* tweetnacl/nacl-fast.js */ function _(require, module, exports) {
    (function (nacl) {
        'use strict';
        // Ported in 2014 by Dmitry Chestnykh and Devi Mandiri.
        // Public domain.
        //
        // Implementation derived from TweetNaCl version 20140427.
        // See for details: http://tweetnacl.cr.yp.to/
        var gf = function (init) {
            var i, r = new Float64Array(16);
            if (init)
                for (i = 0; i < init.length; i++)
                    r[i] = init[i];
            return r;
        };
        //  Pluggable, initialized in high-level API below.
        var randombytes = function ( /* x, n */) { throw new Error('no PRNG'); };
        var _0 = new Uint8Array(16);
        var _9 = new Uint8Array(32);
        _9[0] = 9;
        var gf0 = gf(), gf1 = gf([1]), _121665 = gf([0xdb41, 1]), D = gf([0x78a3, 0x1359, 0x4dca, 0x75eb, 0xd8ab, 0x4141, 0x0a4d, 0x0070, 0xe898, 0x7779, 0x4079, 0x8cc7, 0xfe73, 0x2b6f, 0x6cee, 0x5203]), D2 = gf([0xf159, 0x26b2, 0x9b94, 0xebd6, 0xb156, 0x8283, 0x149a, 0x00e0, 0xd130, 0xeef3, 0x80f2, 0x198e, 0xfce7, 0x56df, 0xd9dc, 0x2406]), X = gf([0xd51a, 0x8f25, 0x2d60, 0xc956, 0xa7b2, 0x9525, 0xc760, 0x692c, 0xdc5c, 0xfdd6, 0xe231, 0xc0a4, 0x53fe, 0xcd6e, 0x36d3, 0x2169]), Y = gf([0x6658, 0x6666, 0x6666, 0x6666, 0x6666, 0x6666, 0x6666, 0x6666, 0x6666, 0x6666, 0x6666, 0x6666, 0x6666, 0x6666, 0x6666, 0x6666]), I = gf([0xa0b0, 0x4a0e, 0x1b27, 0xc4ee, 0xe478, 0xad2f, 0x1806, 0x2f43, 0xd7a7, 0x3dfb, 0x0099, 0x2b4d, 0xdf0b, 0x4fc1, 0x2480, 0x2b83]);
        function ts64(x, i, h, l) {
            x[i] = (h >> 24) & 0xff;
            x[i + 1] = (h >> 16) & 0xff;
            x[i + 2] = (h >> 8) & 0xff;
            x[i + 3] = h & 0xff;
            x[i + 4] = (l >> 24) & 0xff;
            x[i + 5] = (l >> 16) & 0xff;
            x[i + 6] = (l >> 8) & 0xff;
            x[i + 7] = l & 0xff;
        }
        function vn(x, xi, y, yi, n) {
            var i, d = 0;
            for (i = 0; i < n; i++)
                d |= x[xi + i] ^ y[yi + i];
            return (1 & ((d - 1) >>> 8)) - 1;
        }
        function crypto_verify_16(x, xi, y, yi) {
            return vn(x, xi, y, yi, 16);
        }
        function crypto_verify_32(x, xi, y, yi) {
            return vn(x, xi, y, yi, 32);
        }
        function core_salsa20(o, p, k, c) {
            var j0 = c[0] & 0xff | (c[1] & 0xff) << 8 | (c[2] & 0xff) << 16 | (c[3] & 0xff) << 24, j1 = k[0] & 0xff | (k[1] & 0xff) << 8 | (k[2] & 0xff) << 16 | (k[3] & 0xff) << 24, j2 = k[4] & 0xff | (k[5] & 0xff) << 8 | (k[6] & 0xff) << 16 | (k[7] & 0xff) << 24, j3 = k[8] & 0xff | (k[9] & 0xff) << 8 | (k[10] & 0xff) << 16 | (k[11] & 0xff) << 24, j4 = k[12] & 0xff | (k[13] & 0xff) << 8 | (k[14] & 0xff) << 16 | (k[15] & 0xff) << 24, j5 = c[4] & 0xff | (c[5] & 0xff) << 8 | (c[6] & 0xff) << 16 | (c[7] & 0xff) << 24, j6 = p[0] & 0xff | (p[1] & 0xff) << 8 | (p[2] & 0xff) << 16 | (p[3] & 0xff) << 24, j7 = p[4] & 0xff | (p[5] & 0xff) << 8 | (p[6] & 0xff) << 16 | (p[7] & 0xff) << 24, j8 = p[8] & 0xff | (p[9] & 0xff) << 8 | (p[10] & 0xff) << 16 | (p[11] & 0xff) << 24, j9 = p[12] & 0xff | (p[13] & 0xff) << 8 | (p[14] & 0xff) << 16 | (p[15] & 0xff) << 24, j10 = c[8] & 0xff | (c[9] & 0xff) << 8 | (c[10] & 0xff) << 16 | (c[11] & 0xff) << 24, j11 = k[16] & 0xff | (k[17] & 0xff) << 8 | (k[18] & 0xff) << 16 | (k[19] & 0xff) << 24, j12 = k[20] & 0xff | (k[21] & 0xff) << 8 | (k[22] & 0xff) << 16 | (k[23] & 0xff) << 24, j13 = k[24] & 0xff | (k[25] & 0xff) << 8 | (k[26] & 0xff) << 16 | (k[27] & 0xff) << 24, j14 = k[28] & 0xff | (k[29] & 0xff) << 8 | (k[30] & 0xff) << 16 | (k[31] & 0xff) << 24, j15 = c[12] & 0xff | (c[13] & 0xff) << 8 | (c[14] & 0xff) << 16 | (c[15] & 0xff) << 24;
            var x0 = j0, x1 = j1, x2 = j2, x3 = j3, x4 = j4, x5 = j5, x6 = j6, x7 = j7, x8 = j8, x9 = j9, x10 = j10, x11 = j11, x12 = j12, x13 = j13, x14 = j14, x15 = j15, u;
            for (var i = 0; i < 20; i += 2) {
                u = x0 + x12 | 0;
                x4 ^= u << 7 | u >>> (32 - 7);
                u = x4 + x0 | 0;
                x8 ^= u << 9 | u >>> (32 - 9);
                u = x8 + x4 | 0;
                x12 ^= u << 13 | u >>> (32 - 13);
                u = x12 + x8 | 0;
                x0 ^= u << 18 | u >>> (32 - 18);
                u = x5 + x1 | 0;
                x9 ^= u << 7 | u >>> (32 - 7);
                u = x9 + x5 | 0;
                x13 ^= u << 9 | u >>> (32 - 9);
                u = x13 + x9 | 0;
                x1 ^= u << 13 | u >>> (32 - 13);
                u = x1 + x13 | 0;
                x5 ^= u << 18 | u >>> (32 - 18);
                u = x10 + x6 | 0;
                x14 ^= u << 7 | u >>> (32 - 7);
                u = x14 + x10 | 0;
                x2 ^= u << 9 | u >>> (32 - 9);
                u = x2 + x14 | 0;
                x6 ^= u << 13 | u >>> (32 - 13);
                u = x6 + x2 | 0;
                x10 ^= u << 18 | u >>> (32 - 18);
                u = x15 + x11 | 0;
                x3 ^= u << 7 | u >>> (32 - 7);
                u = x3 + x15 | 0;
                x7 ^= u << 9 | u >>> (32 - 9);
                u = x7 + x3 | 0;
                x11 ^= u << 13 | u >>> (32 - 13);
                u = x11 + x7 | 0;
                x15 ^= u << 18 | u >>> (32 - 18);
                u = x0 + x3 | 0;
                x1 ^= u << 7 | u >>> (32 - 7);
                u = x1 + x0 | 0;
                x2 ^= u << 9 | u >>> (32 - 9);
                u = x2 + x1 | 0;
                x3 ^= u << 13 | u >>> (32 - 13);
                u = x3 + x2 | 0;
                x0 ^= u << 18 | u >>> (32 - 18);
                u = x5 + x4 | 0;
                x6 ^= u << 7 | u >>> (32 - 7);
                u = x6 + x5 | 0;
                x7 ^= u << 9 | u >>> (32 - 9);
                u = x7 + x6 | 0;
                x4 ^= u << 13 | u >>> (32 - 13);
                u = x4 + x7 | 0;
                x5 ^= u << 18 | u >>> (32 - 18);
                u = x10 + x9 | 0;
                x11 ^= u << 7 | u >>> (32 - 7);
                u = x11 + x10 | 0;
                x8 ^= u << 9 | u >>> (32 - 9);
                u = x8 + x11 | 0;
                x9 ^= u << 13 | u >>> (32 - 13);
                u = x9 + x8 | 0;
                x10 ^= u << 18 | u >>> (32 - 18);
                u = x15 + x14 | 0;
                x12 ^= u << 7 | u >>> (32 - 7);
                u = x12 + x15 | 0;
                x13 ^= u << 9 | u >>> (32 - 9);
                u = x13 + x12 | 0;
                x14 ^= u << 13 | u >>> (32 - 13);
                u = x14 + x13 | 0;
                x15 ^= u << 18 | u >>> (32 - 18);
            }
            x0 = x0 + j0 | 0;
            x1 = x1 + j1 | 0;
            x2 = x2 + j2 | 0;
            x3 = x3 + j3 | 0;
            x4 = x4 + j4 | 0;
            x5 = x5 + j5 | 0;
            x6 = x6 + j6 | 0;
            x7 = x7 + j7 | 0;
            x8 = x8 + j8 | 0;
            x9 = x9 + j9 | 0;
            x10 = x10 + j10 | 0;
            x11 = x11 + j11 | 0;
            x12 = x12 + j12 | 0;
            x13 = x13 + j13 | 0;
            x14 = x14 + j14 | 0;
            x15 = x15 + j15 | 0;
            o[0] = x0 >>> 0 & 0xff;
            o[1] = x0 >>> 8 & 0xff;
            o[2] = x0 >>> 16 & 0xff;
            o[3] = x0 >>> 24 & 0xff;
            o[4] = x1 >>> 0 & 0xff;
            o[5] = x1 >>> 8 & 0xff;
            o[6] = x1 >>> 16 & 0xff;
            o[7] = x1 >>> 24 & 0xff;
            o[8] = x2 >>> 0 & 0xff;
            o[9] = x2 >>> 8 & 0xff;
            o[10] = x2 >>> 16 & 0xff;
            o[11] = x2 >>> 24 & 0xff;
            o[12] = x3 >>> 0 & 0xff;
            o[13] = x3 >>> 8 & 0xff;
            o[14] = x3 >>> 16 & 0xff;
            o[15] = x3 >>> 24 & 0xff;
            o[16] = x4 >>> 0 & 0xff;
            o[17] = x4 >>> 8 & 0xff;
            o[18] = x4 >>> 16 & 0xff;
            o[19] = x4 >>> 24 & 0xff;
            o[20] = x5 >>> 0 & 0xff;
            o[21] = x5 >>> 8 & 0xff;
            o[22] = x5 >>> 16 & 0xff;
            o[23] = x5 >>> 24 & 0xff;
            o[24] = x6 >>> 0 & 0xff;
            o[25] = x6 >>> 8 & 0xff;
            o[26] = x6 >>> 16 & 0xff;
            o[27] = x6 >>> 24 & 0xff;
            o[28] = x7 >>> 0 & 0xff;
            o[29] = x7 >>> 8 & 0xff;
            o[30] = x7 >>> 16 & 0xff;
            o[31] = x7 >>> 24 & 0xff;
            o[32] = x8 >>> 0 & 0xff;
            o[33] = x8 >>> 8 & 0xff;
            o[34] = x8 >>> 16 & 0xff;
            o[35] = x8 >>> 24 & 0xff;
            o[36] = x9 >>> 0 & 0xff;
            o[37] = x9 >>> 8 & 0xff;
            o[38] = x9 >>> 16 & 0xff;
            o[39] = x9 >>> 24 & 0xff;
            o[40] = x10 >>> 0 & 0xff;
            o[41] = x10 >>> 8 & 0xff;
            o[42] = x10 >>> 16 & 0xff;
            o[43] = x10 >>> 24 & 0xff;
            o[44] = x11 >>> 0 & 0xff;
            o[45] = x11 >>> 8 & 0xff;
            o[46] = x11 >>> 16 & 0xff;
            o[47] = x11 >>> 24 & 0xff;
            o[48] = x12 >>> 0 & 0xff;
            o[49] = x12 >>> 8 & 0xff;
            o[50] = x12 >>> 16 & 0xff;
            o[51] = x12 >>> 24 & 0xff;
            o[52] = x13 >>> 0 & 0xff;
            o[53] = x13 >>> 8 & 0xff;
            o[54] = x13 >>> 16 & 0xff;
            o[55] = x13 >>> 24 & 0xff;
            o[56] = x14 >>> 0 & 0xff;
            o[57] = x14 >>> 8 & 0xff;
            o[58] = x14 >>> 16 & 0xff;
            o[59] = x14 >>> 24 & 0xff;
            o[60] = x15 >>> 0 & 0xff;
            o[61] = x15 >>> 8 & 0xff;
            o[62] = x15 >>> 16 & 0xff;
            o[63] = x15 >>> 24 & 0xff;
        }
        function core_hsalsa20(o, p, k, c) {
            var j0 = c[0] & 0xff | (c[1] & 0xff) << 8 | (c[2] & 0xff) << 16 | (c[3] & 0xff) << 24, j1 = k[0] & 0xff | (k[1] & 0xff) << 8 | (k[2] & 0xff) << 16 | (k[3] & 0xff) << 24, j2 = k[4] & 0xff | (k[5] & 0xff) << 8 | (k[6] & 0xff) << 16 | (k[7] & 0xff) << 24, j3 = k[8] & 0xff | (k[9] & 0xff) << 8 | (k[10] & 0xff) << 16 | (k[11] & 0xff) << 24, j4 = k[12] & 0xff | (k[13] & 0xff) << 8 | (k[14] & 0xff) << 16 | (k[15] & 0xff) << 24, j5 = c[4] & 0xff | (c[5] & 0xff) << 8 | (c[6] & 0xff) << 16 | (c[7] & 0xff) << 24, j6 = p[0] & 0xff | (p[1] & 0xff) << 8 | (p[2] & 0xff) << 16 | (p[3] & 0xff) << 24, j7 = p[4] & 0xff | (p[5] & 0xff) << 8 | (p[6] & 0xff) << 16 | (p[7] & 0xff) << 24, j8 = p[8] & 0xff | (p[9] & 0xff) << 8 | (p[10] & 0xff) << 16 | (p[11] & 0xff) << 24, j9 = p[12] & 0xff | (p[13] & 0xff) << 8 | (p[14] & 0xff) << 16 | (p[15] & 0xff) << 24, j10 = c[8] & 0xff | (c[9] & 0xff) << 8 | (c[10] & 0xff) << 16 | (c[11] & 0xff) << 24, j11 = k[16] & 0xff | (k[17] & 0xff) << 8 | (k[18] & 0xff) << 16 | (k[19] & 0xff) << 24, j12 = k[20] & 0xff | (k[21] & 0xff) << 8 | (k[22] & 0xff) << 16 | (k[23] & 0xff) << 24, j13 = k[24] & 0xff | (k[25] & 0xff) << 8 | (k[26] & 0xff) << 16 | (k[27] & 0xff) << 24, j14 = k[28] & 0xff | (k[29] & 0xff) << 8 | (k[30] & 0xff) << 16 | (k[31] & 0xff) << 24, j15 = c[12] & 0xff | (c[13] & 0xff) << 8 | (c[14] & 0xff) << 16 | (c[15] & 0xff) << 24;
            var x0 = j0, x1 = j1, x2 = j2, x3 = j3, x4 = j4, x5 = j5, x6 = j6, x7 = j7, x8 = j8, x9 = j9, x10 = j10, x11 = j11, x12 = j12, x13 = j13, x14 = j14, x15 = j15, u;
            for (var i = 0; i < 20; i += 2) {
                u = x0 + x12 | 0;
                x4 ^= u << 7 | u >>> (32 - 7);
                u = x4 + x0 | 0;
                x8 ^= u << 9 | u >>> (32 - 9);
                u = x8 + x4 | 0;
                x12 ^= u << 13 | u >>> (32 - 13);
                u = x12 + x8 | 0;
                x0 ^= u << 18 | u >>> (32 - 18);
                u = x5 + x1 | 0;
                x9 ^= u << 7 | u >>> (32 - 7);
                u = x9 + x5 | 0;
                x13 ^= u << 9 | u >>> (32 - 9);
                u = x13 + x9 | 0;
                x1 ^= u << 13 | u >>> (32 - 13);
                u = x1 + x13 | 0;
                x5 ^= u << 18 | u >>> (32 - 18);
                u = x10 + x6 | 0;
                x14 ^= u << 7 | u >>> (32 - 7);
                u = x14 + x10 | 0;
                x2 ^= u << 9 | u >>> (32 - 9);
                u = x2 + x14 | 0;
                x6 ^= u << 13 | u >>> (32 - 13);
                u = x6 + x2 | 0;
                x10 ^= u << 18 | u >>> (32 - 18);
                u = x15 + x11 | 0;
                x3 ^= u << 7 | u >>> (32 - 7);
                u = x3 + x15 | 0;
                x7 ^= u << 9 | u >>> (32 - 9);
                u = x7 + x3 | 0;
                x11 ^= u << 13 | u >>> (32 - 13);
                u = x11 + x7 | 0;
                x15 ^= u << 18 | u >>> (32 - 18);
                u = x0 + x3 | 0;
                x1 ^= u << 7 | u >>> (32 - 7);
                u = x1 + x0 | 0;
                x2 ^= u << 9 | u >>> (32 - 9);
                u = x2 + x1 | 0;
                x3 ^= u << 13 | u >>> (32 - 13);
                u = x3 + x2 | 0;
                x0 ^= u << 18 | u >>> (32 - 18);
                u = x5 + x4 | 0;
                x6 ^= u << 7 | u >>> (32 - 7);
                u = x6 + x5 | 0;
                x7 ^= u << 9 | u >>> (32 - 9);
                u = x7 + x6 | 0;
                x4 ^= u << 13 | u >>> (32 - 13);
                u = x4 + x7 | 0;
                x5 ^= u << 18 | u >>> (32 - 18);
                u = x10 + x9 | 0;
                x11 ^= u << 7 | u >>> (32 - 7);
                u = x11 + x10 | 0;
                x8 ^= u << 9 | u >>> (32 - 9);
                u = x8 + x11 | 0;
                x9 ^= u << 13 | u >>> (32 - 13);
                u = x9 + x8 | 0;
                x10 ^= u << 18 | u >>> (32 - 18);
                u = x15 + x14 | 0;
                x12 ^= u << 7 | u >>> (32 - 7);
                u = x12 + x15 | 0;
                x13 ^= u << 9 | u >>> (32 - 9);
                u = x13 + x12 | 0;
                x14 ^= u << 13 | u >>> (32 - 13);
                u = x14 + x13 | 0;
                x15 ^= u << 18 | u >>> (32 - 18);
            }
            o[0] = x0 >>> 0 & 0xff;
            o[1] = x0 >>> 8 & 0xff;
            o[2] = x0 >>> 16 & 0xff;
            o[3] = x0 >>> 24 & 0xff;
            o[4] = x5 >>> 0 & 0xff;
            o[5] = x5 >>> 8 & 0xff;
            o[6] = x5 >>> 16 & 0xff;
            o[7] = x5 >>> 24 & 0xff;
            o[8] = x10 >>> 0 & 0xff;
            o[9] = x10 >>> 8 & 0xff;
            o[10] = x10 >>> 16 & 0xff;
            o[11] = x10 >>> 24 & 0xff;
            o[12] = x15 >>> 0 & 0xff;
            o[13] = x15 >>> 8 & 0xff;
            o[14] = x15 >>> 16 & 0xff;
            o[15] = x15 >>> 24 & 0xff;
            o[16] = x6 >>> 0 & 0xff;
            o[17] = x6 >>> 8 & 0xff;
            o[18] = x6 >>> 16 & 0xff;
            o[19] = x6 >>> 24 & 0xff;
            o[20] = x7 >>> 0 & 0xff;
            o[21] = x7 >>> 8 & 0xff;
            o[22] = x7 >>> 16 & 0xff;
            o[23] = x7 >>> 24 & 0xff;
            o[24] = x8 >>> 0 & 0xff;
            o[25] = x8 >>> 8 & 0xff;
            o[26] = x8 >>> 16 & 0xff;
            o[27] = x8 >>> 24 & 0xff;
            o[28] = x9 >>> 0 & 0xff;
            o[29] = x9 >>> 8 & 0xff;
            o[30] = x9 >>> 16 & 0xff;
            o[31] = x9 >>> 24 & 0xff;
        }
        function crypto_core_salsa20(out, inp, k, c) {
            core_salsa20(out, inp, k, c);
        }
        function crypto_core_hsalsa20(out, inp, k, c) {
            core_hsalsa20(out, inp, k, c);
        }
        var sigma = new Uint8Array([101, 120, 112, 97, 110, 100, 32, 51, 50, 45, 98, 121, 116, 101, 32, 107]);
        // "expand 32-byte k"
        function crypto_stream_salsa20_xor(c, cpos, m, mpos, b, n, k) {
            var z = new Uint8Array(16), x = new Uint8Array(64);
            var u, i;
            for (i = 0; i < 16; i++)
                z[i] = 0;
            for (i = 0; i < 8; i++)
                z[i] = n[i];
            while (b >= 64) {
                crypto_core_salsa20(x, z, k, sigma);
                for (i = 0; i < 64; i++)
                    c[cpos + i] = m[mpos + i] ^ x[i];
                u = 1;
                for (i = 8; i < 16; i++) {
                    u = u + (z[i] & 0xff) | 0;
                    z[i] = u & 0xff;
                    u >>>= 8;
                }
                b -= 64;
                cpos += 64;
                mpos += 64;
            }
            if (b > 0) {
                crypto_core_salsa20(x, z, k, sigma);
                for (i = 0; i < b; i++)
                    c[cpos + i] = m[mpos + i] ^ x[i];
            }
            return 0;
        }
        function crypto_stream_salsa20(c, cpos, b, n, k) {
            var z = new Uint8Array(16), x = new Uint8Array(64);
            var u, i;
            for (i = 0; i < 16; i++)
                z[i] = 0;
            for (i = 0; i < 8; i++)
                z[i] = n[i];
            while (b >= 64) {
                crypto_core_salsa20(x, z, k, sigma);
                for (i = 0; i < 64; i++)
                    c[cpos + i] = x[i];
                u = 1;
                for (i = 8; i < 16; i++) {
                    u = u + (z[i] & 0xff) | 0;
                    z[i] = u & 0xff;
                    u >>>= 8;
                }
                b -= 64;
                cpos += 64;
            }
            if (b > 0) {
                crypto_core_salsa20(x, z, k, sigma);
                for (i = 0; i < b; i++)
                    c[cpos + i] = x[i];
            }
            return 0;
        }
        function crypto_stream(c, cpos, d, n, k) {
            var s = new Uint8Array(32);
            crypto_core_hsalsa20(s, n, k, sigma);
            var sn = new Uint8Array(8);
            for (var i = 0; i < 8; i++)
                sn[i] = n[i + 16];
            return crypto_stream_salsa20(c, cpos, d, sn, s);
        }
        function crypto_stream_xor(c, cpos, m, mpos, d, n, k) {
            var s = new Uint8Array(32);
            crypto_core_hsalsa20(s, n, k, sigma);
            var sn = new Uint8Array(8);
            for (var i = 0; i < 8; i++)
                sn[i] = n[i + 16];
            return crypto_stream_salsa20_xor(c, cpos, m, mpos, d, sn, s);
        }
        /*
        * Port of Andrew Moon's Poly1305-donna-16. Public domain.
        * https://github.com/floodyberry/poly1305-donna
        */
        var poly1305 = function (key) {
            this.buffer = new Uint8Array(16);
            this.r = new Uint16Array(10);
            this.h = new Uint16Array(10);
            this.pad = new Uint16Array(8);
            this.leftover = 0;
            this.fin = 0;
            var t0, t1, t2, t3, t4, t5, t6, t7;
            t0 = key[0] & 0xff | (key[1] & 0xff) << 8;
            this.r[0] = (t0) & 0x1fff;
            t1 = key[2] & 0xff | (key[3] & 0xff) << 8;
            this.r[1] = ((t0 >>> 13) | (t1 << 3)) & 0x1fff;
            t2 = key[4] & 0xff | (key[5] & 0xff) << 8;
            this.r[2] = ((t1 >>> 10) | (t2 << 6)) & 0x1f03;
            t3 = key[6] & 0xff | (key[7] & 0xff) << 8;
            this.r[3] = ((t2 >>> 7) | (t3 << 9)) & 0x1fff;
            t4 = key[8] & 0xff | (key[9] & 0xff) << 8;
            this.r[4] = ((t3 >>> 4) | (t4 << 12)) & 0x00ff;
            this.r[5] = ((t4 >>> 1)) & 0x1ffe;
            t5 = key[10] & 0xff | (key[11] & 0xff) << 8;
            this.r[6] = ((t4 >>> 14) | (t5 << 2)) & 0x1fff;
            t6 = key[12] & 0xff | (key[13] & 0xff) << 8;
            this.r[7] = ((t5 >>> 11) | (t6 << 5)) & 0x1f81;
            t7 = key[14] & 0xff | (key[15] & 0xff) << 8;
            this.r[8] = ((t6 >>> 8) | (t7 << 8)) & 0x1fff;
            this.r[9] = ((t7 >>> 5)) & 0x007f;
            this.pad[0] = key[16] & 0xff | (key[17] & 0xff) << 8;
            this.pad[1] = key[18] & 0xff | (key[19] & 0xff) << 8;
            this.pad[2] = key[20] & 0xff | (key[21] & 0xff) << 8;
            this.pad[3] = key[22] & 0xff | (key[23] & 0xff) << 8;
            this.pad[4] = key[24] & 0xff | (key[25] & 0xff) << 8;
            this.pad[5] = key[26] & 0xff | (key[27] & 0xff) << 8;
            this.pad[6] = key[28] & 0xff | (key[29] & 0xff) << 8;
            this.pad[7] = key[30] & 0xff | (key[31] & 0xff) << 8;
        };
        poly1305.prototype.blocks = function (m, mpos, bytes) {
            var hibit = this.fin ? 0 : (1 << 11);
            var t0, t1, t2, t3, t4, t5, t6, t7, c;
            var d0, d1, d2, d3, d4, d5, d6, d7, d8, d9;
            var h0 = this.h[0], h1 = this.h[1], h2 = this.h[2], h3 = this.h[3], h4 = this.h[4], h5 = this.h[5], h6 = this.h[6], h7 = this.h[7], h8 = this.h[8], h9 = this.h[9];
            var r0 = this.r[0], r1 = this.r[1], r2 = this.r[2], r3 = this.r[3], r4 = this.r[4], r5 = this.r[5], r6 = this.r[6], r7 = this.r[7], r8 = this.r[8], r9 = this.r[9];
            while (bytes >= 16) {
                t0 = m[mpos + 0] & 0xff | (m[mpos + 1] & 0xff) << 8;
                h0 += (t0) & 0x1fff;
                t1 = m[mpos + 2] & 0xff | (m[mpos + 3] & 0xff) << 8;
                h1 += ((t0 >>> 13) | (t1 << 3)) & 0x1fff;
                t2 = m[mpos + 4] & 0xff | (m[mpos + 5] & 0xff) << 8;
                h2 += ((t1 >>> 10) | (t2 << 6)) & 0x1fff;
                t3 = m[mpos + 6] & 0xff | (m[mpos + 7] & 0xff) << 8;
                h3 += ((t2 >>> 7) | (t3 << 9)) & 0x1fff;
                t4 = m[mpos + 8] & 0xff | (m[mpos + 9] & 0xff) << 8;
                h4 += ((t3 >>> 4) | (t4 << 12)) & 0x1fff;
                h5 += ((t4 >>> 1)) & 0x1fff;
                t5 = m[mpos + 10] & 0xff | (m[mpos + 11] & 0xff) << 8;
                h6 += ((t4 >>> 14) | (t5 << 2)) & 0x1fff;
                t6 = m[mpos + 12] & 0xff | (m[mpos + 13] & 0xff) << 8;
                h7 += ((t5 >>> 11) | (t6 << 5)) & 0x1fff;
                t7 = m[mpos + 14] & 0xff | (m[mpos + 15] & 0xff) << 8;
                h8 += ((t6 >>> 8) | (t7 << 8)) & 0x1fff;
                h9 += ((t7 >>> 5)) | hibit;
                c = 0;
                d0 = c;
                d0 += h0 * r0;
                d0 += h1 * (5 * r9);
                d0 += h2 * (5 * r8);
                d0 += h3 * (5 * r7);
                d0 += h4 * (5 * r6);
                c = (d0 >>> 13);
                d0 &= 0x1fff;
                d0 += h5 * (5 * r5);
                d0 += h6 * (5 * r4);
                d0 += h7 * (5 * r3);
                d0 += h8 * (5 * r2);
                d0 += h9 * (5 * r1);
                c += (d0 >>> 13);
                d0 &= 0x1fff;
                d1 = c;
                d1 += h0 * r1;
                d1 += h1 * r0;
                d1 += h2 * (5 * r9);
                d1 += h3 * (5 * r8);
                d1 += h4 * (5 * r7);
                c = (d1 >>> 13);
                d1 &= 0x1fff;
                d1 += h5 * (5 * r6);
                d1 += h6 * (5 * r5);
                d1 += h7 * (5 * r4);
                d1 += h8 * (5 * r3);
                d1 += h9 * (5 * r2);
                c += (d1 >>> 13);
                d1 &= 0x1fff;
                d2 = c;
                d2 += h0 * r2;
                d2 += h1 * r1;
                d2 += h2 * r0;
                d2 += h3 * (5 * r9);
                d2 += h4 * (5 * r8);
                c = (d2 >>> 13);
                d2 &= 0x1fff;
                d2 += h5 * (5 * r7);
                d2 += h6 * (5 * r6);
                d2 += h7 * (5 * r5);
                d2 += h8 * (5 * r4);
                d2 += h9 * (5 * r3);
                c += (d2 >>> 13);
                d2 &= 0x1fff;
                d3 = c;
                d3 += h0 * r3;
                d3 += h1 * r2;
                d3 += h2 * r1;
                d3 += h3 * r0;
                d3 += h4 * (5 * r9);
                c = (d3 >>> 13);
                d3 &= 0x1fff;
                d3 += h5 * (5 * r8);
                d3 += h6 * (5 * r7);
                d3 += h7 * (5 * r6);
                d3 += h8 * (5 * r5);
                d3 += h9 * (5 * r4);
                c += (d3 >>> 13);
                d3 &= 0x1fff;
                d4 = c;
                d4 += h0 * r4;
                d4 += h1 * r3;
                d4 += h2 * r2;
                d4 += h3 * r1;
                d4 += h4 * r0;
                c = (d4 >>> 13);
                d4 &= 0x1fff;
                d4 += h5 * (5 * r9);
                d4 += h6 * (5 * r8);
                d4 += h7 * (5 * r7);
                d4 += h8 * (5 * r6);
                d4 += h9 * (5 * r5);
                c += (d4 >>> 13);
                d4 &= 0x1fff;
                d5 = c;
                d5 += h0 * r5;
                d5 += h1 * r4;
                d5 += h2 * r3;
                d5 += h3 * r2;
                d5 += h4 * r1;
                c = (d5 >>> 13);
                d5 &= 0x1fff;
                d5 += h5 * r0;
                d5 += h6 * (5 * r9);
                d5 += h7 * (5 * r8);
                d5 += h8 * (5 * r7);
                d5 += h9 * (5 * r6);
                c += (d5 >>> 13);
                d5 &= 0x1fff;
                d6 = c;
                d6 += h0 * r6;
                d6 += h1 * r5;
                d6 += h2 * r4;
                d6 += h3 * r3;
                d6 += h4 * r2;
                c = (d6 >>> 13);
                d6 &= 0x1fff;
                d6 += h5 * r1;
                d6 += h6 * r0;
                d6 += h7 * (5 * r9);
                d6 += h8 * (5 * r8);
                d6 += h9 * (5 * r7);
                c += (d6 >>> 13);
                d6 &= 0x1fff;
                d7 = c;
                d7 += h0 * r7;
                d7 += h1 * r6;
                d7 += h2 * r5;
                d7 += h3 * r4;
                d7 += h4 * r3;
                c = (d7 >>> 13);
                d7 &= 0x1fff;
                d7 += h5 * r2;
                d7 += h6 * r1;
                d7 += h7 * r0;
                d7 += h8 * (5 * r9);
                d7 += h9 * (5 * r8);
                c += (d7 >>> 13);
                d7 &= 0x1fff;
                d8 = c;
                d8 += h0 * r8;
                d8 += h1 * r7;
                d8 += h2 * r6;
                d8 += h3 * r5;
                d8 += h4 * r4;
                c = (d8 >>> 13);
                d8 &= 0x1fff;
                d8 += h5 * r3;
                d8 += h6 * r2;
                d8 += h7 * r1;
                d8 += h8 * r0;
                d8 += h9 * (5 * r9);
                c += (d8 >>> 13);
                d8 &= 0x1fff;
                d9 = c;
                d9 += h0 * r9;
                d9 += h1 * r8;
                d9 += h2 * r7;
                d9 += h3 * r6;
                d9 += h4 * r5;
                c = (d9 >>> 13);
                d9 &= 0x1fff;
                d9 += h5 * r4;
                d9 += h6 * r3;
                d9 += h7 * r2;
                d9 += h8 * r1;
                d9 += h9 * r0;
                c += (d9 >>> 13);
                d9 &= 0x1fff;
                c = (((c << 2) + c)) | 0;
                c = (c + d0) | 0;
                d0 = c & 0x1fff;
                c = (c >>> 13);
                d1 += c;
                h0 = d0;
                h1 = d1;
                h2 = d2;
                h3 = d3;
                h4 = d4;
                h5 = d5;
                h6 = d6;
                h7 = d7;
                h8 = d8;
                h9 = d9;
                mpos += 16;
                bytes -= 16;
            }
            this.h[0] = h0;
            this.h[1] = h1;
            this.h[2] = h2;
            this.h[3] = h3;
            this.h[4] = h4;
            this.h[5] = h5;
            this.h[6] = h6;
            this.h[7] = h7;
            this.h[8] = h8;
            this.h[9] = h9;
        };
        poly1305.prototype.finish = function (mac, macpos) {
            var g = new Uint16Array(10);
            var c, mask, f, i;
            if (this.leftover) {
                i = this.leftover;
                this.buffer[i++] = 1;
                for (; i < 16; i++)
                    this.buffer[i] = 0;
                this.fin = 1;
                this.blocks(this.buffer, 0, 16);
            }
            c = this.h[1] >>> 13;
            this.h[1] &= 0x1fff;
            for (i = 2; i < 10; i++) {
                this.h[i] += c;
                c = this.h[i] >>> 13;
                this.h[i] &= 0x1fff;
            }
            this.h[0] += (c * 5);
            c = this.h[0] >>> 13;
            this.h[0] &= 0x1fff;
            this.h[1] += c;
            c = this.h[1] >>> 13;
            this.h[1] &= 0x1fff;
            this.h[2] += c;
            g[0] = this.h[0] + 5;
            c = g[0] >>> 13;
            g[0] &= 0x1fff;
            for (i = 1; i < 10; i++) {
                g[i] = this.h[i] + c;
                c = g[i] >>> 13;
                g[i] &= 0x1fff;
            }
            g[9] -= (1 << 13);
            mask = (c ^ 1) - 1;
            for (i = 0; i < 10; i++)
                g[i] &= mask;
            mask = ~mask;
            for (i = 0; i < 10; i++)
                this.h[i] = (this.h[i] & mask) | g[i];
            this.h[0] = ((this.h[0]) | (this.h[1] << 13)) & 0xffff;
            this.h[1] = ((this.h[1] >>> 3) | (this.h[2] << 10)) & 0xffff;
            this.h[2] = ((this.h[2] >>> 6) | (this.h[3] << 7)) & 0xffff;
            this.h[3] = ((this.h[3] >>> 9) | (this.h[4] << 4)) & 0xffff;
            this.h[4] = ((this.h[4] >>> 12) | (this.h[5] << 1) | (this.h[6] << 14)) & 0xffff;
            this.h[5] = ((this.h[6] >>> 2) | (this.h[7] << 11)) & 0xffff;
            this.h[6] = ((this.h[7] >>> 5) | (this.h[8] << 8)) & 0xffff;
            this.h[7] = ((this.h[8] >>> 8) | (this.h[9] << 5)) & 0xffff;
            f = this.h[0] + this.pad[0];
            this.h[0] = f & 0xffff;
            for (i = 1; i < 8; i++) {
                f = (((this.h[i] + this.pad[i]) | 0) + (f >>> 16)) | 0;
                this.h[i] = f & 0xffff;
            }
            mac[macpos + 0] = (this.h[0] >>> 0) & 0xff;
            mac[macpos + 1] = (this.h[0] >>> 8) & 0xff;
            mac[macpos + 2] = (this.h[1] >>> 0) & 0xff;
            mac[macpos + 3] = (this.h[1] >>> 8) & 0xff;
            mac[macpos + 4] = (this.h[2] >>> 0) & 0xff;
            mac[macpos + 5] = (this.h[2] >>> 8) & 0xff;
            mac[macpos + 6] = (this.h[3] >>> 0) & 0xff;
            mac[macpos + 7] = (this.h[3] >>> 8) & 0xff;
            mac[macpos + 8] = (this.h[4] >>> 0) & 0xff;
            mac[macpos + 9] = (this.h[4] >>> 8) & 0xff;
            mac[macpos + 10] = (this.h[5] >>> 0) & 0xff;
            mac[macpos + 11] = (this.h[5] >>> 8) & 0xff;
            mac[macpos + 12] = (this.h[6] >>> 0) & 0xff;
            mac[macpos + 13] = (this.h[6] >>> 8) & 0xff;
            mac[macpos + 14] = (this.h[7] >>> 0) & 0xff;
            mac[macpos + 15] = (this.h[7] >>> 8) & 0xff;
        };
        poly1305.prototype.update = function (m, mpos, bytes) {
            var i, want;
            if (this.leftover) {
                want = (16 - this.leftover);
                if (want > bytes)
                    want = bytes;
                for (i = 0; i < want; i++)
                    this.buffer[this.leftover + i] = m[mpos + i];
                bytes -= want;
                mpos += want;
                this.leftover += want;
                if (this.leftover < 16)
                    return;
                this.blocks(this.buffer, 0, 16);
                this.leftover = 0;
            }
            if (bytes >= 16) {
                want = bytes - (bytes % 16);
                this.blocks(m, mpos, want);
                mpos += want;
                bytes -= want;
            }
            if (bytes) {
                for (i = 0; i < bytes; i++)
                    this.buffer[this.leftover + i] = m[mpos + i];
                this.leftover += bytes;
            }
        };
        function crypto_onetimeauth(out, outpos, m, mpos, n, k) {
            var s = new poly1305(k);
            s.update(m, mpos, n);
            s.finish(out, outpos);
            return 0;
        }
        function crypto_onetimeauth_verify(h, hpos, m, mpos, n, k) {
            var x = new Uint8Array(16);
            crypto_onetimeauth(x, 0, m, mpos, n, k);
            return crypto_verify_16(h, hpos, x, 0);
        }
        function crypto_secretbox(c, m, d, n, k) {
            var i;
            if (d < 32)
                return -1;
            crypto_stream_xor(c, 0, m, 0, d, n, k);
            crypto_onetimeauth(c, 16, c, 32, d - 32, c);
            for (i = 0; i < 16; i++)
                c[i] = 0;
            return 0;
        }
        function crypto_secretbox_open(m, c, d, n, k) {
            var i;
            var x = new Uint8Array(32);
            if (d < 32)
                return -1;
            crypto_stream(x, 0, 32, n, k);
            if (crypto_onetimeauth_verify(c, 16, c, 32, d - 32, x) !== 0)
                return -1;
            crypto_stream_xor(m, 0, c, 0, d, n, k);
            for (i = 0; i < 32; i++)
                m[i] = 0;
            return 0;
        }
        function set25519(r, a) {
            var i;
            for (i = 0; i < 16; i++)
                r[i] = a[i] | 0;
        }
        function car25519(o) {
            var i, v, c = 1;
            for (i = 0; i < 16; i++) {
                v = o[i] + c + 65535;
                c = Math.floor(v / 65536);
                o[i] = v - c * 65536;
            }
            o[0] += c - 1 + 37 * (c - 1);
        }
        function sel25519(p, q, b) {
            var t, c = ~(b - 1);
            for (var i = 0; i < 16; i++) {
                t = c & (p[i] ^ q[i]);
                p[i] ^= t;
                q[i] ^= t;
            }
        }
        function pack25519(o, n) {
            var i, j, b;
            var m = gf(), t = gf();
            for (i = 0; i < 16; i++)
                t[i] = n[i];
            car25519(t);
            car25519(t);
            car25519(t);
            for (j = 0; j < 2; j++) {
                m[0] = t[0] - 0xffed;
                for (i = 1; i < 15; i++) {
                    m[i] = t[i] - 0xffff - ((m[i - 1] >> 16) & 1);
                    m[i - 1] &= 0xffff;
                }
                m[15] = t[15] - 0x7fff - ((m[14] >> 16) & 1);
                b = (m[15] >> 16) & 1;
                m[14] &= 0xffff;
                sel25519(t, m, 1 - b);
            }
            for (i = 0; i < 16; i++) {
                o[2 * i] = t[i] & 0xff;
                o[2 * i + 1] = t[i] >> 8;
            }
        }
        function neq25519(a, b) {
            var c = new Uint8Array(32), d = new Uint8Array(32);
            pack25519(c, a);
            pack25519(d, b);
            return crypto_verify_32(c, 0, d, 0);
        }
        function par25519(a) {
            var d = new Uint8Array(32);
            pack25519(d, a);
            return d[0] & 1;
        }
        function unpack25519(o, n) {
            var i;
            for (i = 0; i < 16; i++)
                o[i] = n[2 * i] + (n[2 * i + 1] << 8);
            o[15] &= 0x7fff;
        }
        function A(o, a, b) {
            for (var i = 0; i < 16; i++)
                o[i] = a[i] + b[i];
        }
        function Z(o, a, b) {
            for (var i = 0; i < 16; i++)
                o[i] = a[i] - b[i];
        }
        function M(o, a, b) {
            var v, c, t0 = 0, t1 = 0, t2 = 0, t3 = 0, t4 = 0, t5 = 0, t6 = 0, t7 = 0, t8 = 0, t9 = 0, t10 = 0, t11 = 0, t12 = 0, t13 = 0, t14 = 0, t15 = 0, t16 = 0, t17 = 0, t18 = 0, t19 = 0, t20 = 0, t21 = 0, t22 = 0, t23 = 0, t24 = 0, t25 = 0, t26 = 0, t27 = 0, t28 = 0, t29 = 0, t30 = 0, b0 = b[0], b1 = b[1], b2 = b[2], b3 = b[3], b4 = b[4], b5 = b[5], b6 = b[6], b7 = b[7], b8 = b[8], b9 = b[9], b10 = b[10], b11 = b[11], b12 = b[12], b13 = b[13], b14 = b[14], b15 = b[15];
            v = a[0];
            t0 += v * b0;
            t1 += v * b1;
            t2 += v * b2;
            t3 += v * b3;
            t4 += v * b4;
            t5 += v * b5;
            t6 += v * b6;
            t7 += v * b7;
            t8 += v * b8;
            t9 += v * b9;
            t10 += v * b10;
            t11 += v * b11;
            t12 += v * b12;
            t13 += v * b13;
            t14 += v * b14;
            t15 += v * b15;
            v = a[1];
            t1 += v * b0;
            t2 += v * b1;
            t3 += v * b2;
            t4 += v * b3;
            t5 += v * b4;
            t6 += v * b5;
            t7 += v * b6;
            t8 += v * b7;
            t9 += v * b8;
            t10 += v * b9;
            t11 += v * b10;
            t12 += v * b11;
            t13 += v * b12;
            t14 += v * b13;
            t15 += v * b14;
            t16 += v * b15;
            v = a[2];
            t2 += v * b0;
            t3 += v * b1;
            t4 += v * b2;
            t5 += v * b3;
            t6 += v * b4;
            t7 += v * b5;
            t8 += v * b6;
            t9 += v * b7;
            t10 += v * b8;
            t11 += v * b9;
            t12 += v * b10;
            t13 += v * b11;
            t14 += v * b12;
            t15 += v * b13;
            t16 += v * b14;
            t17 += v * b15;
            v = a[3];
            t3 += v * b0;
            t4 += v * b1;
            t5 += v * b2;
            t6 += v * b3;
            t7 += v * b4;
            t8 += v * b5;
            t9 += v * b6;
            t10 += v * b7;
            t11 += v * b8;
            t12 += v * b9;
            t13 += v * b10;
            t14 += v * b11;
            t15 += v * b12;
            t16 += v * b13;
            t17 += v * b14;
            t18 += v * b15;
            v = a[4];
            t4 += v * b0;
            t5 += v * b1;
            t6 += v * b2;
            t7 += v * b3;
            t8 += v * b4;
            t9 += v * b5;
            t10 += v * b6;
            t11 += v * b7;
            t12 += v * b8;
            t13 += v * b9;
            t14 += v * b10;
            t15 += v * b11;
            t16 += v * b12;
            t17 += v * b13;
            t18 += v * b14;
            t19 += v * b15;
            v = a[5];
            t5 += v * b0;
            t6 += v * b1;
            t7 += v * b2;
            t8 += v * b3;
            t9 += v * b4;
            t10 += v * b5;
            t11 += v * b6;
            t12 += v * b7;
            t13 += v * b8;
            t14 += v * b9;
            t15 += v * b10;
            t16 += v * b11;
            t17 += v * b12;
            t18 += v * b13;
            t19 += v * b14;
            t20 += v * b15;
            v = a[6];
            t6 += v * b0;
            t7 += v * b1;
            t8 += v * b2;
            t9 += v * b3;
            t10 += v * b4;
            t11 += v * b5;
            t12 += v * b6;
            t13 += v * b7;
            t14 += v * b8;
            t15 += v * b9;
            t16 += v * b10;
            t17 += v * b11;
            t18 += v * b12;
            t19 += v * b13;
            t20 += v * b14;
            t21 += v * b15;
            v = a[7];
            t7 += v * b0;
            t8 += v * b1;
            t9 += v * b2;
            t10 += v * b3;
            t11 += v * b4;
            t12 += v * b5;
            t13 += v * b6;
            t14 += v * b7;
            t15 += v * b8;
            t16 += v * b9;
            t17 += v * b10;
            t18 += v * b11;
            t19 += v * b12;
            t20 += v * b13;
            t21 += v * b14;
            t22 += v * b15;
            v = a[8];
            t8 += v * b0;
            t9 += v * b1;
            t10 += v * b2;
            t11 += v * b3;
            t12 += v * b4;
            t13 += v * b5;
            t14 += v * b6;
            t15 += v * b7;
            t16 += v * b8;
            t17 += v * b9;
            t18 += v * b10;
            t19 += v * b11;
            t20 += v * b12;
            t21 += v * b13;
            t22 += v * b14;
            t23 += v * b15;
            v = a[9];
            t9 += v * b0;
            t10 += v * b1;
            t11 += v * b2;
            t12 += v * b3;
            t13 += v * b4;
            t14 += v * b5;
            t15 += v * b6;
            t16 += v * b7;
            t17 += v * b8;
            t18 += v * b9;
            t19 += v * b10;
            t20 += v * b11;
            t21 += v * b12;
            t22 += v * b13;
            t23 += v * b14;
            t24 += v * b15;
            v = a[10];
            t10 += v * b0;
            t11 += v * b1;
            t12 += v * b2;
            t13 += v * b3;
            t14 += v * b4;
            t15 += v * b5;
            t16 += v * b6;
            t17 += v * b7;
            t18 += v * b8;
            t19 += v * b9;
            t20 += v * b10;
            t21 += v * b11;
            t22 += v * b12;
            t23 += v * b13;
            t24 += v * b14;
            t25 += v * b15;
            v = a[11];
            t11 += v * b0;
            t12 += v * b1;
            t13 += v * b2;
            t14 += v * b3;
            t15 += v * b4;
            t16 += v * b5;
            t17 += v * b6;
            t18 += v * b7;
            t19 += v * b8;
            t20 += v * b9;
            t21 += v * b10;
            t22 += v * b11;
            t23 += v * b12;
            t24 += v * b13;
            t25 += v * b14;
            t26 += v * b15;
            v = a[12];
            t12 += v * b0;
            t13 += v * b1;
            t14 += v * b2;
            t15 += v * b3;
            t16 += v * b4;
            t17 += v * b5;
            t18 += v * b6;
            t19 += v * b7;
            t20 += v * b8;
            t21 += v * b9;
            t22 += v * b10;
            t23 += v * b11;
            t24 += v * b12;
            t25 += v * b13;
            t26 += v * b14;
            t27 += v * b15;
            v = a[13];
            t13 += v * b0;
            t14 += v * b1;
            t15 += v * b2;
            t16 += v * b3;
            t17 += v * b4;
            t18 += v * b5;
            t19 += v * b6;
            t20 += v * b7;
            t21 += v * b8;
            t22 += v * b9;
            t23 += v * b10;
            t24 += v * b11;
            t25 += v * b12;
            t26 += v * b13;
            t27 += v * b14;
            t28 += v * b15;
            v = a[14];
            t14 += v * b0;
            t15 += v * b1;
            t16 += v * b2;
            t17 += v * b3;
            t18 += v * b4;
            t19 += v * b5;
            t20 += v * b6;
            t21 += v * b7;
            t22 += v * b8;
            t23 += v * b9;
            t24 += v * b10;
            t25 += v * b11;
            t26 += v * b12;
            t27 += v * b13;
            t28 += v * b14;
            t29 += v * b15;
            v = a[15];
            t15 += v * b0;
            t16 += v * b1;
            t17 += v * b2;
            t18 += v * b3;
            t19 += v * b4;
            t20 += v * b5;
            t21 += v * b6;
            t22 += v * b7;
            t23 += v * b8;
            t24 += v * b9;
            t25 += v * b10;
            t26 += v * b11;
            t27 += v * b12;
            t28 += v * b13;
            t29 += v * b14;
            t30 += v * b15;
            t0 += 38 * t16;
            t1 += 38 * t17;
            t2 += 38 * t18;
            t3 += 38 * t19;
            t4 += 38 * t20;
            t5 += 38 * t21;
            t6 += 38 * t22;
            t7 += 38 * t23;
            t8 += 38 * t24;
            t9 += 38 * t25;
            t10 += 38 * t26;
            t11 += 38 * t27;
            t12 += 38 * t28;
            t13 += 38 * t29;
            t14 += 38 * t30;
            // t15 left as is
            // first car
            c = 1;
            v = t0 + c + 65535;
            c = Math.floor(v / 65536);
            t0 = v - c * 65536;
            v = t1 + c + 65535;
            c = Math.floor(v / 65536);
            t1 = v - c * 65536;
            v = t2 + c + 65535;
            c = Math.floor(v / 65536);
            t2 = v - c * 65536;
            v = t3 + c + 65535;
            c = Math.floor(v / 65536);
            t3 = v - c * 65536;
            v = t4 + c + 65535;
            c = Math.floor(v / 65536);
            t4 = v - c * 65536;
            v = t5 + c + 65535;
            c = Math.floor(v / 65536);
            t5 = v - c * 65536;
            v = t6 + c + 65535;
            c = Math.floor(v / 65536);
            t6 = v - c * 65536;
            v = t7 + c + 65535;
            c = Math.floor(v / 65536);
            t7 = v - c * 65536;
            v = t8 + c + 65535;
            c = Math.floor(v / 65536);
            t8 = v - c * 65536;
            v = t9 + c + 65535;
            c = Math.floor(v / 65536);
            t9 = v - c * 65536;
            v = t10 + c + 65535;
            c = Math.floor(v / 65536);
            t10 = v - c * 65536;
            v = t11 + c + 65535;
            c = Math.floor(v / 65536);
            t11 = v - c * 65536;
            v = t12 + c + 65535;
            c = Math.floor(v / 65536);
            t12 = v - c * 65536;
            v = t13 + c + 65535;
            c = Math.floor(v / 65536);
            t13 = v - c * 65536;
            v = t14 + c + 65535;
            c = Math.floor(v / 65536);
            t14 = v - c * 65536;
            v = t15 + c + 65535;
            c = Math.floor(v / 65536);
            t15 = v - c * 65536;
            t0 += c - 1 + 37 * (c - 1);
            // second car
            c = 1;
            v = t0 + c + 65535;
            c = Math.floor(v / 65536);
            t0 = v - c * 65536;
            v = t1 + c + 65535;
            c = Math.floor(v / 65536);
            t1 = v - c * 65536;
            v = t2 + c + 65535;
            c = Math.floor(v / 65536);
            t2 = v - c * 65536;
            v = t3 + c + 65535;
            c = Math.floor(v / 65536);
            t3 = v - c * 65536;
            v = t4 + c + 65535;
            c = Math.floor(v / 65536);
            t4 = v - c * 65536;
            v = t5 + c + 65535;
            c = Math.floor(v / 65536);
            t5 = v - c * 65536;
            v = t6 + c + 65535;
            c = Math.floor(v / 65536);
            t6 = v - c * 65536;
            v = t7 + c + 65535;
            c = Math.floor(v / 65536);
            t7 = v - c * 65536;
            v = t8 + c + 65535;
            c = Math.floor(v / 65536);
            t8 = v - c * 65536;
            v = t9 + c + 65535;
            c = Math.floor(v / 65536);
            t9 = v - c * 65536;
            v = t10 + c + 65535;
            c = Math.floor(v / 65536);
            t10 = v - c * 65536;
            v = t11 + c + 65535;
            c = Math.floor(v / 65536);
            t11 = v - c * 65536;
            v = t12 + c + 65535;
            c = Math.floor(v / 65536);
            t12 = v - c * 65536;
            v = t13 + c + 65535;
            c = Math.floor(v / 65536);
            t13 = v - c * 65536;
            v = t14 + c + 65535;
            c = Math.floor(v / 65536);
            t14 = v - c * 65536;
            v = t15 + c + 65535;
            c = Math.floor(v / 65536);
            t15 = v - c * 65536;
            t0 += c - 1 + 37 * (c - 1);
            o[0] = t0;
            o[1] = t1;
            o[2] = t2;
            o[3] = t3;
            o[4] = t4;
            o[5] = t5;
            o[6] = t6;
            o[7] = t7;
            o[8] = t8;
            o[9] = t9;
            o[10] = t10;
            o[11] = t11;
            o[12] = t12;
            o[13] = t13;
            o[14] = t14;
            o[15] = t15;
        }
        function S(o, a) {
            M(o, a, a);
        }
        function inv25519(o, i) {
            var c = gf();
            var a;
            for (a = 0; a < 16; a++)
                c[a] = i[a];
            for (a = 253; a >= 0; a--) {
                S(c, c);
                if (a !== 2 && a !== 4)
                    M(c, c, i);
            }
            for (a = 0; a < 16; a++)
                o[a] = c[a];
        }
        function pow2523(o, i) {
            var c = gf();
            var a;
            for (a = 0; a < 16; a++)
                c[a] = i[a];
            for (a = 250; a >= 0; a--) {
                S(c, c);
                if (a !== 1)
                    M(c, c, i);
            }
            for (a = 0; a < 16; a++)
                o[a] = c[a];
        }
        function crypto_scalarmult(q, n, p) {
            var z = new Uint8Array(32);
            var x = new Float64Array(80), r, i;
            var a = gf(), b = gf(), c = gf(), d = gf(), e = gf(), f = gf();
            for (i = 0; i < 31; i++)
                z[i] = n[i];
            z[31] = (n[31] & 127) | 64;
            z[0] &= 248;
            unpack25519(x, p);
            for (i = 0; i < 16; i++) {
                b[i] = x[i];
                d[i] = a[i] = c[i] = 0;
            }
            a[0] = d[0] = 1;
            for (i = 254; i >= 0; --i) {
                r = (z[i >>> 3] >>> (i & 7)) & 1;
                sel25519(a, b, r);
                sel25519(c, d, r);
                A(e, a, c);
                Z(a, a, c);
                A(c, b, d);
                Z(b, b, d);
                S(d, e);
                S(f, a);
                M(a, c, a);
                M(c, b, e);
                A(e, a, c);
                Z(a, a, c);
                S(b, a);
                Z(c, d, f);
                M(a, c, _121665);
                A(a, a, d);
                M(c, c, a);
                M(a, d, f);
                M(d, b, x);
                S(b, e);
                sel25519(a, b, r);
                sel25519(c, d, r);
            }
            for (i = 0; i < 16; i++) {
                x[i + 16] = a[i];
                x[i + 32] = c[i];
                x[i + 48] = b[i];
                x[i + 64] = d[i];
            }
            var x32 = x.subarray(32);
            var x16 = x.subarray(16);
            inv25519(x32, x32);
            M(x16, x16, x32);
            pack25519(q, x16);
            return 0;
        }
        function crypto_scalarmult_base(q, n) {
            return crypto_scalarmult(q, n, _9);
        }
        function crypto_box_keypair(y, x) {
            randombytes(x, 32);
            return crypto_scalarmult_base(y, x);
        }
        function crypto_box_beforenm(k, y, x) {
            var s = new Uint8Array(32);
            crypto_scalarmult(s, x, y);
            return crypto_core_hsalsa20(k, _0, s, sigma);
        }
        var crypto_box_afternm = crypto_secretbox;
        var crypto_box_open_afternm = crypto_secretbox_open;
        function crypto_box(c, m, d, n, y, x) {
            var k = new Uint8Array(32);
            crypto_box_beforenm(k, y, x);
            return crypto_box_afternm(c, m, d, n, k);
        }
        function crypto_box_open(m, c, d, n, y, x) {
            var k = new Uint8Array(32);
            crypto_box_beforenm(k, y, x);
            return crypto_box_open_afternm(m, c, d, n, k);
        }
        var K = [
            0x428a2f98, 0xd728ae22, 0x71374491, 0x23ef65cd,
            0xb5c0fbcf, 0xec4d3b2f, 0xe9b5dba5, 0x8189dbbc,
            0x3956c25b, 0xf348b538, 0x59f111f1, 0xb605d019,
            0x923f82a4, 0xaf194f9b, 0xab1c5ed5, 0xda6d8118,
            0xd807aa98, 0xa3030242, 0x12835b01, 0x45706fbe,
            0x243185be, 0x4ee4b28c, 0x550c7dc3, 0xd5ffb4e2,
            0x72be5d74, 0xf27b896f, 0x80deb1fe, 0x3b1696b1,
            0x9bdc06a7, 0x25c71235, 0xc19bf174, 0xcf692694,
            0xe49b69c1, 0x9ef14ad2, 0xefbe4786, 0x384f25e3,
            0x0fc19dc6, 0x8b8cd5b5, 0x240ca1cc, 0x77ac9c65,
            0x2de92c6f, 0x592b0275, 0x4a7484aa, 0x6ea6e483,
            0x5cb0a9dc, 0xbd41fbd4, 0x76f988da, 0x831153b5,
            0x983e5152, 0xee66dfab, 0xa831c66d, 0x2db43210,
            0xb00327c8, 0x98fb213f, 0xbf597fc7, 0xbeef0ee4,
            0xc6e00bf3, 0x3da88fc2, 0xd5a79147, 0x930aa725,
            0x06ca6351, 0xe003826f, 0x14292967, 0x0a0e6e70,
            0x27b70a85, 0x46d22ffc, 0x2e1b2138, 0x5c26c926,
            0x4d2c6dfc, 0x5ac42aed, 0x53380d13, 0x9d95b3df,
            0x650a7354, 0x8baf63de, 0x766a0abb, 0x3c77b2a8,
            0x81c2c92e, 0x47edaee6, 0x92722c85, 0x1482353b,
            0xa2bfe8a1, 0x4cf10364, 0xa81a664b, 0xbc423001,
            0xc24b8b70, 0xd0f89791, 0xc76c51a3, 0x0654be30,
            0xd192e819, 0xd6ef5218, 0xd6990624, 0x5565a910,
            0xf40e3585, 0x5771202a, 0x106aa070, 0x32bbd1b8,
            0x19a4c116, 0xb8d2d0c8, 0x1e376c08, 0x5141ab53,
            0x2748774c, 0xdf8eeb99, 0x34b0bcb5, 0xe19b48a8,
            0x391c0cb3, 0xc5c95a63, 0x4ed8aa4a, 0xe3418acb,
            0x5b9cca4f, 0x7763e373, 0x682e6ff3, 0xd6b2b8a3,
            0x748f82ee, 0x5defb2fc, 0x78a5636f, 0x43172f60,
            0x84c87814, 0xa1f0ab72, 0x8cc70208, 0x1a6439ec,
            0x90befffa, 0x23631e28, 0xa4506ceb, 0xde82bde9,
            0xbef9a3f7, 0xb2c67915, 0xc67178f2, 0xe372532b,
            0xca273ece, 0xea26619c, 0xd186b8c7, 0x21c0c207,
            0xeada7dd6, 0xcde0eb1e, 0xf57d4f7f, 0xee6ed178,
            0x06f067aa, 0x72176fba, 0x0a637dc5, 0xa2c898a6,
            0x113f9804, 0xbef90dae, 0x1b710b35, 0x131c471b,
            0x28db77f5, 0x23047d84, 0x32caab7b, 0x40c72493,
            0x3c9ebe0a, 0x15c9bebc, 0x431d67c4, 0x9c100d4c,
            0x4cc5d4be, 0xcb3e42b6, 0x597f299c, 0xfc657e2a,
            0x5fcb6fab, 0x3ad6faec, 0x6c44198c, 0x4a475817
        ];
        function crypto_hashblocks_hl(hh, hl, m, n) {
            var wh = new Int32Array(16), wl = new Int32Array(16), bh0, bh1, bh2, bh3, bh4, bh5, bh6, bh7, bl0, bl1, bl2, bl3, bl4, bl5, bl6, bl7, th, tl, i, j, h, l, a, b, c, d;
            var ah0 = hh[0], ah1 = hh[1], ah2 = hh[2], ah3 = hh[3], ah4 = hh[4], ah5 = hh[5], ah6 = hh[6], ah7 = hh[7], al0 = hl[0], al1 = hl[1], al2 = hl[2], al3 = hl[3], al4 = hl[4], al5 = hl[5], al6 = hl[6], al7 = hl[7];
            var pos = 0;
            while (n >= 128) {
                for (i = 0; i < 16; i++) {
                    j = 8 * i + pos;
                    wh[i] = (m[j + 0] << 24) | (m[j + 1] << 16) | (m[j + 2] << 8) | m[j + 3];
                    wl[i] = (m[j + 4] << 24) | (m[j + 5] << 16) | (m[j + 6] << 8) | m[j + 7];
                }
                for (i = 0; i < 80; i++) {
                    bh0 = ah0;
                    bh1 = ah1;
                    bh2 = ah2;
                    bh3 = ah3;
                    bh4 = ah4;
                    bh5 = ah5;
                    bh6 = ah6;
                    bh7 = ah7;
                    bl0 = al0;
                    bl1 = al1;
                    bl2 = al2;
                    bl3 = al3;
                    bl4 = al4;
                    bl5 = al5;
                    bl6 = al6;
                    bl7 = al7;
                    // add
                    h = ah7;
                    l = al7;
                    a = l & 0xffff;
                    b = l >>> 16;
                    c = h & 0xffff;
                    d = h >>> 16;
                    // Sigma1
                    h = ((ah4 >>> 14) | (al4 << (32 - 14))) ^ ((ah4 >>> 18) | (al4 << (32 - 18))) ^ ((al4 >>> (41 - 32)) | (ah4 << (32 - (41 - 32))));
                    l = ((al4 >>> 14) | (ah4 << (32 - 14))) ^ ((al4 >>> 18) | (ah4 << (32 - 18))) ^ ((ah4 >>> (41 - 32)) | (al4 << (32 - (41 - 32))));
                    a += l & 0xffff;
                    b += l >>> 16;
                    c += h & 0xffff;
                    d += h >>> 16;
                    // Ch
                    h = (ah4 & ah5) ^ (~ah4 & ah6);
                    l = (al4 & al5) ^ (~al4 & al6);
                    a += l & 0xffff;
                    b += l >>> 16;
                    c += h & 0xffff;
                    d += h >>> 16;
                    // K
                    h = K[i * 2];
                    l = K[i * 2 + 1];
                    a += l & 0xffff;
                    b += l >>> 16;
                    c += h & 0xffff;
                    d += h >>> 16;
                    // w
                    h = wh[i % 16];
                    l = wl[i % 16];
                    a += l & 0xffff;
                    b += l >>> 16;
                    c += h & 0xffff;
                    d += h >>> 16;
                    b += a >>> 16;
                    c += b >>> 16;
                    d += c >>> 16;
                    th = c & 0xffff | d << 16;
                    tl = a & 0xffff | b << 16;
                    // add
                    h = th;
                    l = tl;
                    a = l & 0xffff;
                    b = l >>> 16;
                    c = h & 0xffff;
                    d = h >>> 16;
                    // Sigma0
                    h = ((ah0 >>> 28) | (al0 << (32 - 28))) ^ ((al0 >>> (34 - 32)) | (ah0 << (32 - (34 - 32)))) ^ ((al0 >>> (39 - 32)) | (ah0 << (32 - (39 - 32))));
                    l = ((al0 >>> 28) | (ah0 << (32 - 28))) ^ ((ah0 >>> (34 - 32)) | (al0 << (32 - (34 - 32)))) ^ ((ah0 >>> (39 - 32)) | (al0 << (32 - (39 - 32))));
                    a += l & 0xffff;
                    b += l >>> 16;
                    c += h & 0xffff;
                    d += h >>> 16;
                    // Maj
                    h = (ah0 & ah1) ^ (ah0 & ah2) ^ (ah1 & ah2);
                    l = (al0 & al1) ^ (al0 & al2) ^ (al1 & al2);
                    a += l & 0xffff;
                    b += l >>> 16;
                    c += h & 0xffff;
                    d += h >>> 16;
                    b += a >>> 16;
                    c += b >>> 16;
                    d += c >>> 16;
                    bh7 = (c & 0xffff) | (d << 16);
                    bl7 = (a & 0xffff) | (b << 16);
                    // add
                    h = bh3;
                    l = bl3;
                    a = l & 0xffff;
                    b = l >>> 16;
                    c = h & 0xffff;
                    d = h >>> 16;
                    h = th;
                    l = tl;
                    a += l & 0xffff;
                    b += l >>> 16;
                    c += h & 0xffff;
                    d += h >>> 16;
                    b += a >>> 16;
                    c += b >>> 16;
                    d += c >>> 16;
                    bh3 = (c & 0xffff) | (d << 16);
                    bl3 = (a & 0xffff) | (b << 16);
                    ah1 = bh0;
                    ah2 = bh1;
                    ah3 = bh2;
                    ah4 = bh3;
                    ah5 = bh4;
                    ah6 = bh5;
                    ah7 = bh6;
                    ah0 = bh7;
                    al1 = bl0;
                    al2 = bl1;
                    al3 = bl2;
                    al4 = bl3;
                    al5 = bl4;
                    al6 = bl5;
                    al7 = bl6;
                    al0 = bl7;
                    if (i % 16 === 15) {
                        for (j = 0; j < 16; j++) {
                            // add
                            h = wh[j];
                            l = wl[j];
                            a = l & 0xffff;
                            b = l >>> 16;
                            c = h & 0xffff;
                            d = h >>> 16;
                            h = wh[(j + 9) % 16];
                            l = wl[(j + 9) % 16];
                            a += l & 0xffff;
                            b += l >>> 16;
                            c += h & 0xffff;
                            d += h >>> 16;
                            // sigma0
                            th = wh[(j + 1) % 16];
                            tl = wl[(j + 1) % 16];
                            h = ((th >>> 1) | (tl << (32 - 1))) ^ ((th >>> 8) | (tl << (32 - 8))) ^ (th >>> 7);
                            l = ((tl >>> 1) | (th << (32 - 1))) ^ ((tl >>> 8) | (th << (32 - 8))) ^ ((tl >>> 7) | (th << (32 - 7)));
                            a += l & 0xffff;
                            b += l >>> 16;
                            c += h & 0xffff;
                            d += h >>> 16;
                            // sigma1
                            th = wh[(j + 14) % 16];
                            tl = wl[(j + 14) % 16];
                            h = ((th >>> 19) | (tl << (32 - 19))) ^ ((tl >>> (61 - 32)) | (th << (32 - (61 - 32)))) ^ (th >>> 6);
                            l = ((tl >>> 19) | (th << (32 - 19))) ^ ((th >>> (61 - 32)) | (tl << (32 - (61 - 32)))) ^ ((tl >>> 6) | (th << (32 - 6)));
                            a += l & 0xffff;
                            b += l >>> 16;
                            c += h & 0xffff;
                            d += h >>> 16;
                            b += a >>> 16;
                            c += b >>> 16;
                            d += c >>> 16;
                            wh[j] = (c & 0xffff) | (d << 16);
                            wl[j] = (a & 0xffff) | (b << 16);
                        }
                    }
                }
                // add
                h = ah0;
                l = al0;
                a = l & 0xffff;
                b = l >>> 16;
                c = h & 0xffff;
                d = h >>> 16;
                h = hh[0];
                l = hl[0];
                a += l & 0xffff;
                b += l >>> 16;
                c += h & 0xffff;
                d += h >>> 16;
                b += a >>> 16;
                c += b >>> 16;
                d += c >>> 16;
                hh[0] = ah0 = (c & 0xffff) | (d << 16);
                hl[0] = al0 = (a & 0xffff) | (b << 16);
                h = ah1;
                l = al1;
                a = l & 0xffff;
                b = l >>> 16;
                c = h & 0xffff;
                d = h >>> 16;
                h = hh[1];
                l = hl[1];
                a += l & 0xffff;
                b += l >>> 16;
                c += h & 0xffff;
                d += h >>> 16;
                b += a >>> 16;
                c += b >>> 16;
                d += c >>> 16;
                hh[1] = ah1 = (c & 0xffff) | (d << 16);
                hl[1] = al1 = (a & 0xffff) | (b << 16);
                h = ah2;
                l = al2;
                a = l & 0xffff;
                b = l >>> 16;
                c = h & 0xffff;
                d = h >>> 16;
                h = hh[2];
                l = hl[2];
                a += l & 0xffff;
                b += l >>> 16;
                c += h & 0xffff;
                d += h >>> 16;
                b += a >>> 16;
                c += b >>> 16;
                d += c >>> 16;
                hh[2] = ah2 = (c & 0xffff) | (d << 16);
                hl[2] = al2 = (a & 0xffff) | (b << 16);
                h = ah3;
                l = al3;
                a = l & 0xffff;
                b = l >>> 16;
                c = h & 0xffff;
                d = h >>> 16;
                h = hh[3];
                l = hl[3];
                a += l & 0xffff;
                b += l >>> 16;
                c += h & 0xffff;
                d += h >>> 16;
                b += a >>> 16;
                c += b >>> 16;
                d += c >>> 16;
                hh[3] = ah3 = (c & 0xffff) | (d << 16);
                hl[3] = al3 = (a & 0xffff) | (b << 16);
                h = ah4;
                l = al4;
                a = l & 0xffff;
                b = l >>> 16;
                c = h & 0xffff;
                d = h >>> 16;
                h = hh[4];
                l = hl[4];
                a += l & 0xffff;
                b += l >>> 16;
                c += h & 0xffff;
                d += h >>> 16;
                b += a >>> 16;
                c += b >>> 16;
                d += c >>> 16;
                hh[4] = ah4 = (c & 0xffff) | (d << 16);
                hl[4] = al4 = (a & 0xffff) | (b << 16);
                h = ah5;
                l = al5;
                a = l & 0xffff;
                b = l >>> 16;
                c = h & 0xffff;
                d = h >>> 16;
                h = hh[5];
                l = hl[5];
                a += l & 0xffff;
                b += l >>> 16;
                c += h & 0xffff;
                d += h >>> 16;
                b += a >>> 16;
                c += b >>> 16;
                d += c >>> 16;
                hh[5] = ah5 = (c & 0xffff) | (d << 16);
                hl[5] = al5 = (a & 0xffff) | (b << 16);
                h = ah6;
                l = al6;
                a = l & 0xffff;
                b = l >>> 16;
                c = h & 0xffff;
                d = h >>> 16;
                h = hh[6];
                l = hl[6];
                a += l & 0xffff;
                b += l >>> 16;
                c += h & 0xffff;
                d += h >>> 16;
                b += a >>> 16;
                c += b >>> 16;
                d += c >>> 16;
                hh[6] = ah6 = (c & 0xffff) | (d << 16);
                hl[6] = al6 = (a & 0xffff) | (b << 16);
                h = ah7;
                l = al7;
                a = l & 0xffff;
                b = l >>> 16;
                c = h & 0xffff;
                d = h >>> 16;
                h = hh[7];
                l = hl[7];
                a += l & 0xffff;
                b += l >>> 16;
                c += h & 0xffff;
                d += h >>> 16;
                b += a >>> 16;
                c += b >>> 16;
                d += c >>> 16;
                hh[7] = ah7 = (c & 0xffff) | (d << 16);
                hl[7] = al7 = (a & 0xffff) | (b << 16);
                pos += 128;
                n -= 128;
            }
            return n;
        }
        function crypto_hash(out, m, n) {
            var hh = new Int32Array(8), hl = new Int32Array(8), x = new Uint8Array(256), i, b = n;
            hh[0] = 0x6a09e667;
            hh[1] = 0xbb67ae85;
            hh[2] = 0x3c6ef372;
            hh[3] = 0xa54ff53a;
            hh[4] = 0x510e527f;
            hh[5] = 0x9b05688c;
            hh[6] = 0x1f83d9ab;
            hh[7] = 0x5be0cd19;
            hl[0] = 0xf3bcc908;
            hl[1] = 0x84caa73b;
            hl[2] = 0xfe94f82b;
            hl[3] = 0x5f1d36f1;
            hl[4] = 0xade682d1;
            hl[5] = 0x2b3e6c1f;
            hl[6] = 0xfb41bd6b;
            hl[7] = 0x137e2179;
            crypto_hashblocks_hl(hh, hl, m, n);
            n %= 128;
            for (i = 0; i < n; i++)
                x[i] = m[b - n + i];
            x[n] = 128;
            n = 256 - 128 * (n < 112 ? 1 : 0);
            x[n - 9] = 0;
            ts64(x, n - 8, (b / 0x20000000) | 0, b << 3);
            crypto_hashblocks_hl(hh, hl, x, n);
            for (i = 0; i < 8; i++)
                ts64(out, 8 * i, hh[i], hl[i]);
            return 0;
        }
        function add(p, q) {
            var a = gf(), b = gf(), c = gf(), d = gf(), e = gf(), f = gf(), g = gf(), h = gf(), t = gf();
            Z(a, p[1], p[0]);
            Z(t, q[1], q[0]);
            M(a, a, t);
            A(b, p[0], p[1]);
            A(t, q[0], q[1]);
            M(b, b, t);
            M(c, p[3], q[3]);
            M(c, c, D2);
            M(d, p[2], q[2]);
            A(d, d, d);
            Z(e, b, a);
            Z(f, d, c);
            A(g, d, c);
            A(h, b, a);
            M(p[0], e, f);
            M(p[1], h, g);
            M(p[2], g, f);
            M(p[3], e, h);
        }
        function cswap(p, q, b) {
            var i;
            for (i = 0; i < 4; i++) {
                sel25519(p[i], q[i], b);
            }
        }
        function pack(r, p) {
            var tx = gf(), ty = gf(), zi = gf();
            inv25519(zi, p[2]);
            M(tx, p[0], zi);
            M(ty, p[1], zi);
            pack25519(r, ty);
            r[31] ^= par25519(tx) << 7;
        }
        function scalarmult(p, q, s) {
            var b, i;
            set25519(p[0], gf0);
            set25519(p[1], gf1);
            set25519(p[2], gf1);
            set25519(p[3], gf0);
            for (i = 255; i >= 0; --i) {
                b = (s[(i / 8) | 0] >> (i & 7)) & 1;
                cswap(p, q, b);
                add(q, p);
                add(p, p);
                cswap(p, q, b);
            }
        }
        function scalarbase(p, s) {
            var q = [gf(), gf(), gf(), gf()];
            set25519(q[0], X);
            set25519(q[1], Y);
            set25519(q[2], gf1);
            M(q[3], X, Y);
            scalarmult(p, q, s);
        }
        function crypto_sign_keypair(pk, sk, seeded) {
            var d = new Uint8Array(64);
            var p = [gf(), gf(), gf(), gf()];
            var i;
            if (!seeded)
                randombytes(sk, 32);
            crypto_hash(d, sk, 32);
            d[0] &= 248;
            d[31] &= 127;
            d[31] |= 64;
            scalarbase(p, d);
            pack(pk, p);
            for (i = 0; i < 32; i++)
                sk[i + 32] = pk[i];
            return 0;
        }
        var L = new Float64Array([0xed, 0xd3, 0xf5, 0x5c, 0x1a, 0x63, 0x12, 0x58, 0xd6, 0x9c, 0xf7, 0xa2, 0xde, 0xf9, 0xde, 0x14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0x10]);
        function modL(r, x) {
            var carry, i, j, k;
            for (i = 63; i >= 32; --i) {
                carry = 0;
                for (j = i - 32, k = i - 12; j < k; ++j) {
                    x[j] += carry - 16 * x[i] * L[j - (i - 32)];
                    carry = (x[j] + 128) >> 8;
                    x[j] -= carry * 256;
                }
                x[j] += carry;
                x[i] = 0;
            }
            carry = 0;
            for (j = 0; j < 32; j++) {
                x[j] += carry - (x[31] >> 4) * L[j];
                carry = x[j] >> 8;
                x[j] &= 255;
            }
            for (j = 0; j < 32; j++)
                x[j] -= carry * L[j];
            for (i = 0; i < 32; i++) {
                x[i + 1] += x[i] >> 8;
                r[i] = x[i] & 255;
            }
        }
        function reduce(r) {
            var x = new Float64Array(64), i;
            for (i = 0; i < 64; i++)
                x[i] = r[i];
            for (i = 0; i < 64; i++)
                r[i] = 0;
            modL(r, x);
        }
        // Note: difference from C - smlen returned, not passed as argument.
        function crypto_sign(sm, m, n, sk) {
            var d = new Uint8Array(64), h = new Uint8Array(64), r = new Uint8Array(64);
            var i, j, x = new Float64Array(64);
            var p = [gf(), gf(), gf(), gf()];
            crypto_hash(d, sk, 32);
            d[0] &= 248;
            d[31] &= 127;
            d[31] |= 64;
            var smlen = n + 64;
            for (i = 0; i < n; i++)
                sm[64 + i] = m[i];
            for (i = 0; i < 32; i++)
                sm[32 + i] = d[32 + i];
            crypto_hash(r, sm.subarray(32), n + 32);
            reduce(r);
            scalarbase(p, r);
            pack(sm, p);
            for (i = 32; i < 64; i++)
                sm[i] = sk[i];
            crypto_hash(h, sm, n + 64);
            reduce(h);
            for (i = 0; i < 64; i++)
                x[i] = 0;
            for (i = 0; i < 32; i++)
                x[i] = r[i];
            for (i = 0; i < 32; i++) {
                for (j = 0; j < 32; j++) {
                    x[i + j] += h[i] * d[j];
                }
            }
            modL(sm.subarray(32), x);
            return smlen;
        }
        function unpackneg(r, p) {
            var t = gf(), chk = gf(), num = gf(), den = gf(), den2 = gf(), den4 = gf(), den6 = gf();
            set25519(r[2], gf1);
            unpack25519(r[1], p);
            S(num, r[1]);
            M(den, num, D);
            Z(num, num, r[2]);
            A(den, r[2], den);
            S(den2, den);
            S(den4, den2);
            M(den6, den4, den2);
            M(t, den6, num);
            M(t, t, den);
            pow2523(t, t);
            M(t, t, num);
            M(t, t, den);
            M(t, t, den);
            M(r[0], t, den);
            S(chk, r[0]);
            M(chk, chk, den);
            if (neq25519(chk, num))
                M(r[0], r[0], I);
            S(chk, r[0]);
            M(chk, chk, den);
            if (neq25519(chk, num))
                return -1;
            if (par25519(r[0]) === (p[31] >> 7))
                Z(r[0], gf0, r[0]);
            M(r[3], r[0], r[1]);
            return 0;
        }
        function crypto_sign_open(m, sm, n, pk) {
            var i, mlen;
            var t = new Uint8Array(32), h = new Uint8Array(64);
            var p = [gf(), gf(), gf(), gf()], q = [gf(), gf(), gf(), gf()];
            mlen = -1;
            if (n < 64)
                return -1;
            if (unpackneg(q, pk))
                return -1;
            for (i = 0; i < n; i++)
                m[i] = sm[i];
            for (i = 0; i < 32; i++)
                m[i + 32] = pk[i];
            crypto_hash(h, m, n);
            reduce(h);
            scalarmult(p, q, h);
            scalarbase(q, sm.subarray(32));
            add(p, q);
            pack(t, p);
            n -= 64;
            if (crypto_verify_32(sm, 0, t, 0)) {
                for (i = 0; i < n; i++)
                    m[i] = 0;
                return -1;
            }
            for (i = 0; i < n; i++)
                m[i] = sm[i + 64];
            mlen = n;
            return mlen;
        }
        var crypto_secretbox_KEYBYTES = 32, crypto_secretbox_NONCEBYTES = 24, crypto_secretbox_ZEROBYTES = 32, crypto_secretbox_BOXZEROBYTES = 16, crypto_scalarmult_BYTES = 32, crypto_scalarmult_SCALARBYTES = 32, crypto_box_PUBLICKEYBYTES = 32, crypto_box_SECRETKEYBYTES = 32, crypto_box_BEFORENMBYTES = 32, crypto_box_NONCEBYTES = crypto_secretbox_NONCEBYTES, crypto_box_ZEROBYTES = crypto_secretbox_ZEROBYTES, crypto_box_BOXZEROBYTES = crypto_secretbox_BOXZEROBYTES, crypto_sign_BYTES = 64, crypto_sign_PUBLICKEYBYTES = 32, crypto_sign_SECRETKEYBYTES = 64, crypto_sign_SEEDBYTES = 32, crypto_hash_BYTES = 64;
        nacl.lowlevel = {
            crypto_core_hsalsa20: crypto_core_hsalsa20,
            crypto_stream_xor: crypto_stream_xor,
            crypto_stream: crypto_stream,
            crypto_stream_salsa20_xor: crypto_stream_salsa20_xor,
            crypto_stream_salsa20: crypto_stream_salsa20,
            crypto_onetimeauth: crypto_onetimeauth,
            crypto_onetimeauth_verify: crypto_onetimeauth_verify,
            crypto_verify_16: crypto_verify_16,
            crypto_verify_32: crypto_verify_32,
            crypto_secretbox: crypto_secretbox,
            crypto_secretbox_open: crypto_secretbox_open,
            crypto_scalarmult: crypto_scalarmult,
            crypto_scalarmult_base: crypto_scalarmult_base,
            crypto_box_beforenm: crypto_box_beforenm,
            crypto_box_afternm: crypto_box_afternm,
            crypto_box: crypto_box,
            crypto_box_open: crypto_box_open,
            crypto_box_keypair: crypto_box_keypair,
            crypto_hash: crypto_hash,
            crypto_sign: crypto_sign,
            crypto_sign_keypair: crypto_sign_keypair,
            crypto_sign_open: crypto_sign_open,
            crypto_secretbox_KEYBYTES: crypto_secretbox_KEYBYTES,
            crypto_secretbox_NONCEBYTES: crypto_secretbox_NONCEBYTES,
            crypto_secretbox_ZEROBYTES: crypto_secretbox_ZEROBYTES,
            crypto_secretbox_BOXZEROBYTES: crypto_secretbox_BOXZEROBYTES,
            crypto_scalarmult_BYTES: crypto_scalarmult_BYTES,
            crypto_scalarmult_SCALARBYTES: crypto_scalarmult_SCALARBYTES,
            crypto_box_PUBLICKEYBYTES: crypto_box_PUBLICKEYBYTES,
            crypto_box_SECRETKEYBYTES: crypto_box_SECRETKEYBYTES,
            crypto_box_BEFORENMBYTES: crypto_box_BEFORENMBYTES,
            crypto_box_NONCEBYTES: crypto_box_NONCEBYTES,
            crypto_box_ZEROBYTES: crypto_box_ZEROBYTES,
            crypto_box_BOXZEROBYTES: crypto_box_BOXZEROBYTES,
            crypto_sign_BYTES: crypto_sign_BYTES,
            crypto_sign_PUBLICKEYBYTES: crypto_sign_PUBLICKEYBYTES,
            crypto_sign_SECRETKEYBYTES: crypto_sign_SECRETKEYBYTES,
            crypto_sign_SEEDBYTES: crypto_sign_SEEDBYTES,
            crypto_hash_BYTES: crypto_hash_BYTES
        };
        /* High-level API */
        function checkLengths(k, n) {
            if (k.length !== crypto_secretbox_KEYBYTES)
                throw new Error('bad key size');
            if (n.length !== crypto_secretbox_NONCEBYTES)
                throw new Error('bad nonce size');
        }
        function checkBoxLengths(pk, sk) {
            if (pk.length !== crypto_box_PUBLICKEYBYTES)
                throw new Error('bad public key size');
            if (sk.length !== crypto_box_SECRETKEYBYTES)
                throw new Error('bad secret key size');
        }
        function checkArrayTypes() {
            var t, i;
            for (i = 0; i < arguments.length; i++) {
                if ((t = Object.prototype.toString.call(arguments[i])) !== '[object Uint8Array]')
                    throw new TypeError('unexpected type ' + t + ', use Uint8Array');
            }
        }
        function cleanup(arr) {
            for (var i = 0; i < arr.length; i++)
                arr[i] = 0;
        }
        // TODO: Completely remove this in v0.15.
        if (!nacl.util) {
            nacl.util = {};
            nacl.util.decodeUTF8 = nacl.util.encodeUTF8 = nacl.util.encodeBase64 = nacl.util.decodeBase64 = function () {
                throw new Error('nacl.util moved into separate package: https://github.com/dchest/tweetnacl-util-js');
            };
        }
        nacl.randomBytes = function (n) {
            var b = new Uint8Array(n);
            randombytes(b, n);
            return b;
        };
        nacl.secretbox = function (msg, nonce, key) {
            checkArrayTypes(msg, nonce, key);
            checkLengths(key, nonce);
            var m = new Uint8Array(crypto_secretbox_ZEROBYTES + msg.length);
            var c = new Uint8Array(m.length);
            for (var i = 0; i < msg.length; i++)
                m[i + crypto_secretbox_ZEROBYTES] = msg[i];
            crypto_secretbox(c, m, m.length, nonce, key);
            return c.subarray(crypto_secretbox_BOXZEROBYTES);
        };
        nacl.secretbox.open = function (box, nonce, key) {
            checkArrayTypes(box, nonce, key);
            checkLengths(key, nonce);
            var c = new Uint8Array(crypto_secretbox_BOXZEROBYTES + box.length);
            var m = new Uint8Array(c.length);
            for (var i = 0; i < box.length; i++)
                c[i + crypto_secretbox_BOXZEROBYTES] = box[i];
            if (c.length < 32)
                return false;
            if (crypto_secretbox_open(m, c, c.length, nonce, key) !== 0)
                return false;
            return m.subarray(crypto_secretbox_ZEROBYTES);
        };
        nacl.secretbox.keyLength = crypto_secretbox_KEYBYTES;
        nacl.secretbox.nonceLength = crypto_secretbox_NONCEBYTES;
        nacl.secretbox.overheadLength = crypto_secretbox_BOXZEROBYTES;
        nacl.scalarMult = function (n, p) {
            checkArrayTypes(n, p);
            if (n.length !== crypto_scalarmult_SCALARBYTES)
                throw new Error('bad n size');
            if (p.length !== crypto_scalarmult_BYTES)
                throw new Error('bad p size');
            var q = new Uint8Array(crypto_scalarmult_BYTES);
            crypto_scalarmult(q, n, p);
            return q;
        };
        nacl.scalarMult.base = function (n) {
            checkArrayTypes(n);
            if (n.length !== crypto_scalarmult_SCALARBYTES)
                throw new Error('bad n size');
            var q = new Uint8Array(crypto_scalarmult_BYTES);
            crypto_scalarmult_base(q, n);
            return q;
        };
        nacl.scalarMult.scalarLength = crypto_scalarmult_SCALARBYTES;
        nacl.scalarMult.groupElementLength = crypto_scalarmult_BYTES;
        nacl.box = function (msg, nonce, publicKey, secretKey) {
            var k = nacl.box.before(publicKey, secretKey);
            return nacl.secretbox(msg, nonce, k);
        };
        nacl.box.before = function (publicKey, secretKey) {
            checkArrayTypes(publicKey, secretKey);
            checkBoxLengths(publicKey, secretKey);
            var k = new Uint8Array(crypto_box_BEFORENMBYTES);
            crypto_box_beforenm(k, publicKey, secretKey);
            return k;
        };
        nacl.box.after = nacl.secretbox;
        nacl.box.open = function (msg, nonce, publicKey, secretKey) {
            var k = nacl.box.before(publicKey, secretKey);
            return nacl.secretbox.open(msg, nonce, k);
        };
        nacl.box.open.after = nacl.secretbox.open;
        nacl.box.keyPair = function () {
            var pk = new Uint8Array(crypto_box_PUBLICKEYBYTES);
            var sk = new Uint8Array(crypto_box_SECRETKEYBYTES);
            crypto_box_keypair(pk, sk);
            return { publicKey: pk, secretKey: sk };
        };
        nacl.box.keyPair.fromSecretKey = function (secretKey) {
            checkArrayTypes(secretKey);
            if (secretKey.length !== crypto_box_SECRETKEYBYTES)
                throw new Error('bad secret key size');
            var pk = new Uint8Array(crypto_box_PUBLICKEYBYTES);
            crypto_scalarmult_base(pk, secretKey);
            return { publicKey: pk, secretKey: new Uint8Array(secretKey) };
        };
        nacl.box.publicKeyLength = crypto_box_PUBLICKEYBYTES;
        nacl.box.secretKeyLength = crypto_box_SECRETKEYBYTES;
        nacl.box.sharedKeyLength = crypto_box_BEFORENMBYTES;
        nacl.box.nonceLength = crypto_box_NONCEBYTES;
        nacl.box.overheadLength = nacl.secretbox.overheadLength;
        nacl.sign = function (msg, secretKey) {
            checkArrayTypes(msg, secretKey);
            if (secretKey.length !== crypto_sign_SECRETKEYBYTES)
                throw new Error('bad secret key size');
            var signedMsg = new Uint8Array(crypto_sign_BYTES + msg.length);
            crypto_sign(signedMsg, msg, msg.length, secretKey);
            return signedMsg;
        };
        nacl.sign.open = function (signedMsg, publicKey) {
            if (arguments.length !== 2)
                throw new Error('nacl.sign.open accepts 2 arguments; did you mean to use nacl.sign.detached.verify?');
            checkArrayTypes(signedMsg, publicKey);
            if (publicKey.length !== crypto_sign_PUBLICKEYBYTES)
                throw new Error('bad public key size');
            var tmp = new Uint8Array(signedMsg.length);
            var mlen = crypto_sign_open(tmp, signedMsg, signedMsg.length, publicKey);
            if (mlen < 0)
                return null;
            var m = new Uint8Array(mlen);
            for (var i = 0; i < m.length; i++)
                m[i] = tmp[i];
            return m;
        };
        nacl.sign.detached = function (msg, secretKey) {
            var signedMsg = nacl.sign(msg, secretKey);
            var sig = new Uint8Array(crypto_sign_BYTES);
            for (var i = 0; i < sig.length; i++)
                sig[i] = signedMsg[i];
            return sig;
        };
        nacl.sign.detached.verify = function (msg, sig, publicKey) {
            checkArrayTypes(msg, sig, publicKey);
            if (sig.length !== crypto_sign_BYTES)
                throw new Error('bad signature size');
            if (publicKey.length !== crypto_sign_PUBLICKEYBYTES)
                throw new Error('bad public key size');
            var sm = new Uint8Array(crypto_sign_BYTES + msg.length);
            var m = new Uint8Array(crypto_sign_BYTES + msg.length);
            var i;
            for (i = 0; i < crypto_sign_BYTES; i++)
                sm[i] = sig[i];
            for (i = 0; i < msg.length; i++)
                sm[i + crypto_sign_BYTES] = msg[i];
            return (crypto_sign_open(m, sm, sm.length, publicKey) >= 0);
        };
        nacl.sign.keyPair = function () {
            var pk = new Uint8Array(crypto_sign_PUBLICKEYBYTES);
            var sk = new Uint8Array(crypto_sign_SECRETKEYBYTES);
            crypto_sign_keypair(pk, sk);
            return { publicKey: pk, secretKey: sk };
        };
        nacl.sign.keyPair.fromSecretKey = function (secretKey) {
            checkArrayTypes(secretKey);
            if (secretKey.length !== crypto_sign_SECRETKEYBYTES)
                throw new Error('bad secret key size');
            var pk = new Uint8Array(crypto_sign_PUBLICKEYBYTES);
            for (var i = 0; i < pk.length; i++)
                pk[i] = secretKey[32 + i];
            return { publicKey: pk, secretKey: new Uint8Array(secretKey) };
        };
        nacl.sign.keyPair.fromSeed = function (seed) {
            checkArrayTypes(seed);
            if (seed.length !== crypto_sign_SEEDBYTES)
                throw new Error('bad seed size');
            var pk = new Uint8Array(crypto_sign_PUBLICKEYBYTES);
            var sk = new Uint8Array(crypto_sign_SECRETKEYBYTES);
            for (var i = 0; i < 32; i++)
                sk[i] = seed[i];
            crypto_sign_keypair(pk, sk, true);
            return { publicKey: pk, secretKey: sk };
        };
        nacl.sign.publicKeyLength = crypto_sign_PUBLICKEYBYTES;
        nacl.sign.secretKeyLength = crypto_sign_SECRETKEYBYTES;
        nacl.sign.seedLength = crypto_sign_SEEDBYTES;
        nacl.sign.signatureLength = crypto_sign_BYTES;
        nacl.hash = function (msg) {
            checkArrayTypes(msg);
            var h = new Uint8Array(crypto_hash_BYTES);
            crypto_hash(h, msg, msg.length);
            return h;
        };
        nacl.hash.hashLength = crypto_hash_BYTES;
        nacl.verify = function (x, y) {
            checkArrayTypes(x, y);
            // Zero length arguments are considered not equal.
            if (x.length === 0 || y.length === 0)
                return false;
            if (x.length !== y.length)
                return false;
            return (vn(x, 0, y, 0, x.length) === 0) ? true : false;
        };
        nacl.setPRNG = function (fn) {
            randombytes = fn;
        };
        (function () {
            // Initialize PRNG if environment provides CSPRNG.
            // If not, methods calling randombytes will throw.
            var crypto = typeof self !== 'undefined' ? (self.crypto || self.msCrypto) : null;
            if (crypto && crypto.getRandomValues) {
                // Browsers.
                var QUOTA = 65536;
                nacl.setPRNG(function (x, n) {
                    var i, v = new Uint8Array(n);
                    for (i = 0; i < n; i += QUOTA) {
                        crypto.getRandomValues(v.subarray(i, i + Math.min(n - i, QUOTA)));
                    }
                    for (i = 0; i < n; i++)
                        x[i] = v[i];
                    cleanup(v);
                });
            }
            else if (typeof require !== 'undefined') {
                // Node.js.
                crypto = require('crypto');
                if (crypto && crypto.randomBytes) {
                    nacl.setPRNG(function (x, n) {
                        var i, v = crypto.randomBytes(n);
                        for (i = 0; i < n; i++)
                            x[i] = v[i];
                        cleanup(v);
                    });
                }
            }
        })();
    })(typeof module !== 'undefined' && module.exports ? module.exports : (self.nacl = self.nacl || {}));
},
/* sshpk/lib/ssh-buffer.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    module.exports = SSHBuffer;
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    function SSHBuffer(opts) {
        assert.object(opts, 'options');
        if (opts.buffer !== undefined)
            assert.buffer(opts.buffer, 'options.buffer');
        this._size = opts.buffer ? opts.buffer.length : 1024;
        this._buffer = opts.buffer || Buffer.alloc(this._size);
        this._offset = 0;
    }
    SSHBuffer.prototype.toBuffer = function () {
        return (this._buffer.slice(0, this._offset));
    };
    SSHBuffer.prototype.atEnd = function () {
        return (this._offset >= this._buffer.length);
    };
    SSHBuffer.prototype.remainder = function () {
        return (this._buffer.slice(this._offset));
    };
    SSHBuffer.prototype.skip = function (n) {
        this._offset += n;
    };
    SSHBuffer.prototype.expand = function () {
        this._size *= 2;
        var buf = Buffer.alloc(this._size);
        this._buffer.copy(buf, 0);
        this._buffer = buf;
    };
    SSHBuffer.prototype.readPart = function () {
        return ({ data: this.readBuffer() });
    };
    SSHBuffer.prototype.readBuffer = function () {
        var len = this._buffer.readUInt32BE(this._offset);
        this._offset += 4;
        assert.ok(this._offset + len <= this._buffer.length, 'length out of bounds at +0x' + this._offset.toString(16) +
            ' (data truncated?)');
        var buf = this._buffer.slice(this._offset, this._offset + len);
        this._offset += len;
        return (buf);
    };
    SSHBuffer.prototype.readString = function () {
        return (this.readBuffer().toString());
    };
    SSHBuffer.prototype.readCString = function () {
        var offset = this._offset;
        while (offset < this._buffer.length &&
            this._buffer[offset] !== 0x00)
            offset++;
        assert.ok(offset < this._buffer.length, 'c string does not terminate');
        var str = this._buffer.slice(this._offset, offset).toString();
        this._offset = offset + 1;
        return (str);
    };
    SSHBuffer.prototype.readInt = function () {
        var v = this._buffer.readUInt32BE(this._offset);
        this._offset += 4;
        return (v);
    };
    SSHBuffer.prototype.readInt64 = function () {
        assert.ok(this._offset + 8 < this._buffer.length, 'buffer not long enough to read Int64');
        var v = this._buffer.slice(this._offset, this._offset + 8);
        this._offset += 8;
        return (v);
    };
    SSHBuffer.prototype.readChar = function () {
        var v = this._buffer[this._offset++];
        return (v);
    };
    SSHBuffer.prototype.writeBuffer = function (buf) {
        while (this._offset + 4 + buf.length > this._size)
            this.expand();
        this._buffer.writeUInt32BE(buf.length, this._offset);
        this._offset += 4;
        buf.copy(this._buffer, this._offset);
        this._offset += buf.length;
    };
    SSHBuffer.prototype.writeString = function (str) {
        this.writeBuffer(Buffer.from(str, 'utf8'));
    };
    SSHBuffer.prototype.writeCString = function (str) {
        while (this._offset + 1 + str.length > this._size)
            this.expand();
        this._buffer.write(str, this._offset);
        this._offset += str.length;
        this._buffer[this._offset++] = 0;
    };
    SSHBuffer.prototype.writeInt = function (v) {
        while (this._offset + 4 > this._size)
            this.expand();
        this._buffer.writeUInt32BE(v, this._offset);
        this._offset += 4;
    };
    SSHBuffer.prototype.writeInt64 = function (v) {
        assert.buffer(v, 'value');
        if (v.length > 8) {
            var lead = v.slice(0, v.length - 8);
            for (var i = 0; i < lead.length; ++i) {
                assert.strictEqual(lead[i], 0, 'must fit in 64 bits of precision');
            }
            v = v.slice(v.length - 8, v.length);
        }
        while (this._offset + 8 > this._size)
            this.expand();
        v.copy(this._buffer, this._offset);
        this._offset += 8;
    };
    SSHBuffer.prototype.writeChar = function (v) {
        while (this._offset + 1 > this._size)
            this.expand();
        this._buffer[this._offset++] = v;
    };
    SSHBuffer.prototype.writePart = function (p) {
        this.writeBuffer(p.data);
    };
    SSHBuffer.prototype.write = function (buf) {
        while (this._offset + buf.length > this._size)
            this.expand();
        buf.copy(this._buffer, this._offset);
        this._offset += buf.length;
    };
},
/* sshpk/lib/dhe.js */ function _(require, module, exports) {
    // Copyright 2017 Joyent, Inc.
    module.exports = {
        DiffieHellman: DiffieHellman,
        generateECDSA: generateECDSA,
        generateED25519: generateED25519
    };
    var assert = require(853) /* assert-plus */;
    var crypto = require('crypto');
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ./algs */;
    var utils = require(863) /* ./utils */;
    var nacl = require(872) /* tweetnacl */;
    var Key = require(856) /* ./key */;
    var PrivateKey = require(861) /* ./private-key */;
    var CRYPTO_HAVE_ECDH = (crypto.createECDH !== undefined);
    var ecdh = require(875) /* ecc-jsbn */;
    var ec = require(870) /* ecc-jsbn/lib/ec */;
    var jsbn = require(871) /* jsbn */.BigInteger;
    function DiffieHellman(key) {
        utils.assertCompatible(key, Key, [1, 4], 'key');
        this._isPriv = PrivateKey.isPrivateKey(key, [1, 3]);
        this._algo = key.type;
        this._curve = key.curve;
        this._key = key;
        if (key.type === 'dsa') {
            if (!CRYPTO_HAVE_ECDH) {
                throw (new Error('Due to bugs in the node 0.10 ' +
                    'crypto API, node 0.12.x or later is required ' +
                    'to use DH'));
            }
            this._dh = crypto.createDiffieHellman(key.part.p.data, undefined, key.part.g.data, undefined);
            this._p = key.part.p;
            this._g = key.part.g;
            if (this._isPriv)
                this._dh.setPrivateKey(key.part.x.data);
            this._dh.setPublicKey(key.part.y.data);
        }
        else if (key.type === 'ecdsa') {
            if (!CRYPTO_HAVE_ECDH) {
                this._ecParams = new X9ECParameters(this._curve);
                if (this._isPriv) {
                    this._priv = new ECPrivate(this._ecParams, key.part.d.data);
                }
                return;
            }
            var curve = {
                'nistp256': 'prime256v1',
                'nistp384': 'secp384r1',
                'nistp521': 'secp521r1'
            }[key.curve];
            this._dh = crypto.createECDH(curve);
            if (typeof (this._dh) !== 'object' ||
                typeof (this._dh.setPrivateKey) !== 'function') {
                CRYPTO_HAVE_ECDH = false;
                DiffieHellman.call(this, key);
                return;
            }
            if (this._isPriv)
                this._dh.setPrivateKey(key.part.d.data);
            this._dh.setPublicKey(key.part.Q.data);
        }
        else if (key.type === 'curve25519') {
            if (this._isPriv) {
                utils.assertCompatible(key, PrivateKey, [1, 5], 'key');
                this._priv = key.part.k.data;
            }
        }
        else {
            throw (new Error('DH not supported for ' + key.type + ' keys'));
        }
    }
    DiffieHellman.prototype.getPublicKey = function () {
        if (this._isPriv)
            return (this._key.toPublic());
        return (this._key);
    };
    DiffieHellman.prototype.getPrivateKey = function () {
        if (this._isPriv)
            return (this._key);
        else
            return (undefined);
    };
    DiffieHellman.prototype.getKey = DiffieHellman.prototype.getPrivateKey;
    DiffieHellman.prototype._keyCheck = function (pk, isPub) {
        assert.object(pk, 'key');
        if (!isPub)
            utils.assertCompatible(pk, PrivateKey, [1, 3], 'key');
        utils.assertCompatible(pk, Key, [1, 4], 'key');
        if (pk.type !== this._algo) {
            throw (new Error('A ' + pk.type + ' key cannot be used in ' +
                this._algo + ' Diffie-Hellman'));
        }
        if (pk.curve !== this._curve) {
            throw (new Error('A key from the ' + pk.curve + ' curve ' +
                'cannot be used with a ' + this._curve +
                ' Diffie-Hellman'));
        }
        if (pk.type === 'dsa') {
            assert.deepEqual(pk.part.p, this._p, 'DSA key prime does not match');
            assert.deepEqual(pk.part.g, this._g, 'DSA key generator does not match');
        }
    };
    DiffieHellman.prototype.setKey = function (pk) {
        this._keyCheck(pk);
        if (pk.type === 'dsa') {
            this._dh.setPrivateKey(pk.part.x.data);
            this._dh.setPublicKey(pk.part.y.data);
        }
        else if (pk.type === 'ecdsa') {
            if (CRYPTO_HAVE_ECDH) {
                this._dh.setPrivateKey(pk.part.d.data);
                this._dh.setPublicKey(pk.part.Q.data);
            }
            else {
                this._priv = new ECPrivate(this._ecParams, pk.part.d.data);
            }
        }
        else if (pk.type === 'curve25519') {
            var k = pk.part.k;
            if (!pk.part.k)
                k = pk.part.r;
            this._priv = k.data;
            if (this._priv[0] === 0x00)
                this._priv = this._priv.slice(1);
            this._priv = this._priv.slice(0, 32);
        }
        this._key = pk;
        this._isPriv = true;
    };
    DiffieHellman.prototype.setPrivateKey = DiffieHellman.prototype.setKey;
    DiffieHellman.prototype.computeSecret = function (otherpk) {
        this._keyCheck(otherpk, true);
        if (!this._isPriv)
            throw (new Error('DH exchange has not been initialized with ' +
                'a private key yet'));
        var pub;
        if (this._algo === 'dsa') {
            return (this._dh.computeSecret(otherpk.part.y.data));
        }
        else if (this._algo === 'ecdsa') {
            if (CRYPTO_HAVE_ECDH) {
                return (this._dh.computeSecret(otherpk.part.Q.data));
            }
            else {
                pub = new ECPublic(this._ecParams, otherpk.part.Q.data);
                return (this._priv.deriveSharedSecret(pub));
            }
        }
        else if (this._algo === 'curve25519') {
            pub = otherpk.part.A.data;
            while (pub[0] === 0x00 && pub.length > 32)
                pub = pub.slice(1);
            var priv = this._priv;
            assert.strictEqual(pub.length, 32);
            assert.strictEqual(priv.length, 32);
            var secret = nacl.box.before(new Uint8Array(pub), new Uint8Array(priv));
            return (Buffer.from(secret));
        }
        throw (new Error('Invalid algorithm: ' + this._algo));
    };
    DiffieHellman.prototype.generateKey = function () {
        var parts = [];
        var priv, pub;
        if (this._algo === 'dsa') {
            this._dh.generateKeys();
            parts.push({ name: 'p', data: this._p.data });
            parts.push({ name: 'q', data: this._key.part.q.data });
            parts.push({ name: 'g', data: this._g.data });
            parts.push({ name: 'y', data: this._dh.getPublicKey() });
            parts.push({ name: 'x', data: this._dh.getPrivateKey() });
            this._key = new PrivateKey({
                type: 'dsa',
                parts: parts
            });
            this._isPriv = true;
            return (this._key);
        }
        else if (this._algo === 'ecdsa') {
            if (CRYPTO_HAVE_ECDH) {
                this._dh.generateKeys();
                parts.push({ name: 'curve',
                    data: Buffer.from(this._curve) });
                parts.push({ name: 'Q', data: this._dh.getPublicKey() });
                parts.push({ name: 'd', data: this._dh.getPrivateKey() });
                this._key = new PrivateKey({
                    type: 'ecdsa',
                    curve: this._curve,
                    parts: parts
                });
                this._isPriv = true;
                return (this._key);
            }
            else {
                var n = this._ecParams.getN();
                var r = new jsbn(crypto.randomBytes(n.bitLength()));
                var n1 = n.subtract(jsbn.ONE);
                priv = r.mod(n1).add(jsbn.ONE);
                pub = this._ecParams.getG().multiply(priv);
                priv = Buffer.from(priv.toByteArray());
                pub = Buffer.from(this._ecParams.getCurve().
                    encodePointHex(pub), 'hex');
                this._priv = new ECPrivate(this._ecParams, priv);
                parts.push({ name: 'curve',
                    data: Buffer.from(this._curve) });
                parts.push({ name: 'Q', data: pub });
                parts.push({ name: 'd', data: priv });
                this._key = new PrivateKey({
                    type: 'ecdsa',
                    curve: this._curve,
                    parts: parts
                });
                this._isPriv = true;
                return (this._key);
            }
        }
        else if (this._algo === 'curve25519') {
            var pair = nacl.box.keyPair();
            priv = Buffer.from(pair.secretKey);
            pub = Buffer.from(pair.publicKey);
            priv = Buffer.concat([priv, pub]);
            assert.strictEqual(priv.length, 64);
            assert.strictEqual(pub.length, 32);
            parts.push({ name: 'A', data: pub });
            parts.push({ name: 'k', data: priv });
            this._key = new PrivateKey({
                type: 'curve25519',
                parts: parts
            });
            this._isPriv = true;
            return (this._key);
        }
        throw (new Error('Invalid algorithm: ' + this._algo));
    };
    DiffieHellman.prototype.generateKeys = DiffieHellman.prototype.generateKey;
    /* These are helpers for using ecc-jsbn (for node 0.10 compatibility). */
    function X9ECParameters(name) {
        var params = algs.curves[name];
        assert.object(params);
        var p = new jsbn(params.p);
        var a = new jsbn(params.a);
        var b = new jsbn(params.b);
        var n = new jsbn(params.n);
        var h = jsbn.ONE;
        var curve = new ec.ECCurveFp(p, a, b);
        var G = curve.decodePointHex(params.G.toString('hex'));
        this.curve = curve;
        this.g = G;
        this.n = n;
        this.h = h;
    }
    X9ECParameters.prototype.getCurve = function () { return (this.curve); };
    X9ECParameters.prototype.getG = function () { return (this.g); };
    X9ECParameters.prototype.getN = function () { return (this.n); };
    X9ECParameters.prototype.getH = function () { return (this.h); };
    function ECPublic(params, buffer) {
        this._params = params;
        if (buffer[0] === 0x00)
            buffer = buffer.slice(1);
        this._pub = params.getCurve().decodePointHex(buffer.toString('hex'));
    }
    function ECPrivate(params, buffer) {
        this._params = params;
        this._priv = new jsbn(utils.mpNormalize(buffer));
    }
    ECPrivate.prototype.deriveSharedSecret = function (pubKey) {
        assert.ok(pubKey instanceof ECPublic);
        var S = pubKey._pub.multiply(this._priv);
        return (Buffer.from(S.getX().toBigInteger().toByteArray()));
    };
    function generateED25519() {
        var pair = nacl.sign.keyPair();
        var priv = Buffer.from(pair.secretKey);
        var pub = Buffer.from(pair.publicKey);
        assert.strictEqual(priv.length, 64);
        assert.strictEqual(pub.length, 32);
        var parts = [];
        parts.push({ name: 'A', data: pub });
        parts.push({ name: 'k', data: priv.slice(0, 32) });
        var key = new PrivateKey({
            type: 'ed25519',
            parts: parts
        });
        return (key);
    }
    /* Generates a new ECDSA private key on a given curve. */
    function generateECDSA(curve) {
        var parts = [];
        var key;
        if (CRYPTO_HAVE_ECDH) {
            /*
             * Node crypto doesn't expose key generation directly, but the
             * ECDH instances can generate keys. It turns out this just
             * calls into the OpenSSL generic key generator, and we can
             * read its output happily without doing an actual DH. So we
             * use that here.
             */
            var osCurve = {
                'nistp256': 'prime256v1',
                'nistp384': 'secp384r1',
                'nistp521': 'secp521r1'
            }[curve];
            var dh = crypto.createECDH(osCurve);
            dh.generateKeys();
            parts.push({ name: 'curve',
                data: Buffer.from(curve) });
            parts.push({ name: 'Q', data: dh.getPublicKey() });
            parts.push({ name: 'd', data: dh.getPrivateKey() });
            key = new PrivateKey({
                type: 'ecdsa',
                curve: curve,
                parts: parts
            });
            return (key);
        }
        else {
            var ecParams = new X9ECParameters(curve);
            /* This algorithm taken from FIPS PUB 186-4 (section B.4.1) */
            var n = ecParams.getN();
            /*
             * The crypto.randomBytes() function can only give us whole
             * bytes, so taking a nod from X9.62, we round up.
             */
            var cByteLen = Math.ceil((n.bitLength() + 64) / 8);
            var c = new jsbn(crypto.randomBytes(cByteLen));
            var n1 = n.subtract(jsbn.ONE);
            var priv = c.mod(n1).add(jsbn.ONE);
            var pub = ecParams.getG().multiply(priv);
            priv = Buffer.from(priv.toByteArray());
            pub = Buffer.from(ecParams.getCurve().
                encodePointHex(pub), 'hex');
            parts.push({ name: 'curve', data: Buffer.from(curve) });
            parts.push({ name: 'Q', data: pub });
            parts.push({ name: 'd', data: priv });
            key = new PrivateKey({
                type: 'ecdsa',
                curve: curve,
                parts: parts
            });
            return (key);
        }
    }
},
/* ecc-jsbn/index.js */ function _(require, module, exports) {
    var crypto = require("crypto");
    var BigInteger = require(871) /* jsbn */.BigInteger;
    var ECPointFp = require(870) /* ./lib/ec.js */.ECPointFp;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    exports.ECCurves = require(876) /* ./lib/sec.js */;
    // zero prepad
    function unstupid(hex, len) {
        return (hex.length >= len) ? hex : unstupid("0" + hex, len);
    }
    exports.ECKey = function (curve, key, isPublic) {
        var priv;
        var c = curve();
        var n = c.getN();
        var bytes = Math.floor(n.bitLength() / 8);
        if (key) {
            if (isPublic) {
                var curve = c.getCurve();
                //      var x = key.slice(1,bytes+1); // skip the 04 for uncompressed format
                //      var y = key.slice(bytes+1);
                //      this.P = new ECPointFp(curve,
                //        curve.fromBigInteger(new BigInteger(x.toString("hex"), 16)),
                //        curve.fromBigInteger(new BigInteger(y.toString("hex"), 16)));      
                this.P = curve.decodePointHex(key.toString("hex"));
            }
            else {
                if (key.length != bytes)
                    return false;
                priv = new BigInteger(key.toString("hex"), 16);
            }
        }
        else {
            var n1 = n.subtract(BigInteger.ONE);
            var r = new BigInteger(crypto.randomBytes(n.bitLength()));
            priv = r.mod(n1).add(BigInteger.ONE);
            this.P = c.getG().multiply(priv);
        }
        if (this.P) {
            //  var pubhex = unstupid(this.P.getX().toBigInteger().toString(16),bytes*2)+unstupid(this.P.getY().toBigInteger().toString(16),bytes*2);
            //  this.PublicKey = Buffer.from("04"+pubhex,"hex");
            this.PublicKey = Buffer.from(c.getCurve().encodeCompressedPointHex(this.P), "hex");
        }
        if (priv) {
            this.PrivateKey = Buffer.from(unstupid(priv.toString(16), bytes * 2), "hex");
            this.deriveSharedSecret = function (key) {
                if (!key || !key.P)
                    return false;
                var S = key.P.multiply(priv);
                return Buffer.from(unstupid(S.getX().toBigInteger().toString(16), bytes * 2), "hex");
            };
        }
    };
},
/* ecc-jsbn/lib/sec.js */ function _(require, module, exports) {
    // Named EC curves
    // Requires ec.js, jsbn.js, and jsbn2.js
    var BigInteger = require(871) /* jsbn */.BigInteger;
    var ECCurveFp = require(870) /* ./ec.js */.ECCurveFp;
    // ----------------
    // X9ECParameters
    // constructor
    function X9ECParameters(curve, g, n, h) {
        this.curve = curve;
        this.g = g;
        this.n = n;
        this.h = h;
    }
    function x9getCurve() {
        return this.curve;
    }
    function x9getG() {
        return this.g;
    }
    function x9getN() {
        return this.n;
    }
    function x9getH() {
        return this.h;
    }
    X9ECParameters.prototype.getCurve = x9getCurve;
    X9ECParameters.prototype.getG = x9getG;
    X9ECParameters.prototype.getN = x9getN;
    X9ECParameters.prototype.getH = x9getH;
    // ----------------
    // SECNamedCurves
    function fromHex(s) { return new BigInteger(s, 16); }
    function secp128r1() {
        // p = 2^128 - 2^97 - 1
        var p = fromHex("FFFFFFFDFFFFFFFFFFFFFFFFFFFFFFFF");
        var a = fromHex("FFFFFFFDFFFFFFFFFFFFFFFFFFFFFFFC");
        var b = fromHex("E87579C11079F43DD824993C2CEE5ED3");
        //byte[] S = Hex.decode("000E0D4D696E6768756151750CC03A4473D03679");
        var n = fromHex("FFFFFFFE0000000075A30D1B9038A115");
        var h = BigInteger.ONE;
        var curve = new ECCurveFp(p, a, b);
        var G = curve.decodePointHex("04"
            + "161FF7528B899B2D0C28607CA52C5B86"
            + "CF5AC8395BAFEB13C02DA292DDED7A83");
        return new X9ECParameters(curve, G, n, h);
    }
    function secp160k1() {
        // p = 2^160 - 2^32 - 2^14 - 2^12 - 2^9 - 2^8 - 2^7 - 2^3 - 2^2 - 1
        var p = fromHex("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFAC73");
        var a = BigInteger.ZERO;
        var b = fromHex("7");
        //byte[] S = null;
        var n = fromHex("0100000000000000000001B8FA16DFAB9ACA16B6B3");
        var h = BigInteger.ONE;
        var curve = new ECCurveFp(p, a, b);
        var G = curve.decodePointHex("04"
            + "3B4C382CE37AA192A4019E763036F4F5DD4D7EBB"
            + "938CF935318FDCED6BC28286531733C3F03C4FEE");
        return new X9ECParameters(curve, G, n, h);
    }
    function secp160r1() {
        // p = 2^160 - 2^31 - 1
        var p = fromHex("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7FFFFFFF");
        var a = fromHex("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7FFFFFFC");
        var b = fromHex("1C97BEFC54BD7A8B65ACF89F81D4D4ADC565FA45");
        //byte[] S = Hex.decode("1053CDE42C14D696E67687561517533BF3F83345");
        var n = fromHex("0100000000000000000001F4C8F927AED3CA752257");
        var h = BigInteger.ONE;
        var curve = new ECCurveFp(p, a, b);
        var G = curve.decodePointHex("04"
            + "4A96B5688EF573284664698968C38BB913CBFC82"
            + "23A628553168947D59DCC912042351377AC5FB32");
        return new X9ECParameters(curve, G, n, h);
    }
    function secp192k1() {
        // p = 2^192 - 2^32 - 2^12 - 2^8 - 2^7 - 2^6 - 2^3 - 1
        var p = fromHex("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFEE37");
        var a = BigInteger.ZERO;
        var b = fromHex("3");
        //byte[] S = null;
        var n = fromHex("FFFFFFFFFFFFFFFFFFFFFFFE26F2FC170F69466A74DEFD8D");
        var h = BigInteger.ONE;
        var curve = new ECCurveFp(p, a, b);
        var G = curve.decodePointHex("04"
            + "DB4FF10EC057E9AE26B07D0280B7F4341DA5D1B1EAE06C7D"
            + "9B2F2F6D9C5628A7844163D015BE86344082AA88D95E2F9D");
        return new X9ECParameters(curve, G, n, h);
    }
    function secp192r1() {
        // p = 2^192 - 2^64 - 1
        var p = fromHex("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFFFFFFFFFF");
        var a = fromHex("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFFFFFFFFFC");
        var b = fromHex("64210519E59C80E70FA7E9AB72243049FEB8DEECC146B9B1");
        //byte[] S = Hex.decode("3045AE6FC8422F64ED579528D38120EAE12196D5");
        var n = fromHex("FFFFFFFFFFFFFFFFFFFFFFFF99DEF836146BC9B1B4D22831");
        var h = BigInteger.ONE;
        var curve = new ECCurveFp(p, a, b);
        var G = curve.decodePointHex("04"
            + "188DA80EB03090F67CBF20EB43A18800F4FF0AFD82FF1012"
            + "07192B95FFC8DA78631011ED6B24CDD573F977A11E794811");
        return new X9ECParameters(curve, G, n, h);
    }
    function secp224r1() {
        // p = 2^224 - 2^96 + 1
        var p = fromHex("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000001");
        var a = fromHex("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFFFFFFFFFFFFFFFFFE");
        var b = fromHex("B4050A850C04B3ABF54132565044B0B7D7BFD8BA270B39432355FFB4");
        //byte[] S = Hex.decode("BD71344799D5C7FCDC45B59FA3B9AB8F6A948BC5");
        var n = fromHex("FFFFFFFFFFFFFFFFFFFFFFFFFFFF16A2E0B8F03E13DD29455C5C2A3D");
        var h = BigInteger.ONE;
        var curve = new ECCurveFp(p, a, b);
        var G = curve.decodePointHex("04"
            + "B70E0CBD6BB4BF7F321390B94A03C1D356C21122343280D6115C1D21"
            + "BD376388B5F723FB4C22DFE6CD4375A05A07476444D5819985007E34");
        return new X9ECParameters(curve, G, n, h);
    }
    function secp256r1() {
        // p = 2^224 (2^32 - 1) + 2^192 + 2^96 - 1
        var p = fromHex("FFFFFFFF00000001000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFF");
        var a = fromHex("FFFFFFFF00000001000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFC");
        var b = fromHex("5AC635D8AA3A93E7B3EBBD55769886BC651D06B0CC53B0F63BCE3C3E27D2604B");
        //byte[] S = Hex.decode("C49D360886E704936A6678E1139D26B7819F7E90");
        var n = fromHex("FFFFFFFF00000000FFFFFFFFFFFFFFFFBCE6FAADA7179E84F3B9CAC2FC632551");
        var h = BigInteger.ONE;
        var curve = new ECCurveFp(p, a, b);
        var G = curve.decodePointHex("04"
            + "6B17D1F2E12C4247F8BCE6E563A440F277037D812DEB33A0F4A13945D898C296"
            + "4FE342E2FE1A7F9B8EE7EB4A7C0F9E162BCE33576B315ECECBB6406837BF51F5");
        return new X9ECParameters(curve, G, n, h);
    }
    // TODO: make this into a proper hashtable
    function getSECCurveByName(name) {
        if (name == "secp128r1")
            return secp128r1();
        if (name == "secp160k1")
            return secp160k1();
        if (name == "secp160r1")
            return secp160r1();
        if (name == "secp192k1")
            return secp192k1();
        if (name == "secp192r1")
            return secp192r1();
        if (name == "secp224r1")
            return secp224r1();
        if (name == "secp256r1")
            return secp256r1();
        return null;
    }
    module.exports = {
        "secp128r1": secp128r1,
        "secp160k1": secp160k1,
        "secp160r1": secp160r1,
        "secp192k1": secp192k1,
        "secp192r1": secp192r1,
        "secp224r1": secp224r1,
        "secp256r1": secp256r1
    };
},
/* sshpk/lib/ed-compat.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    module.exports = {
        Verifier: Verifier,
        Signer: Signer
    };
    var nacl = require(872) /* tweetnacl */;
    var stream = require('stream');
    var util = require('util');
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var Signature = require(862) /* ./signature */;
    function Verifier(key, hashAlgo) {
        if (hashAlgo.toLowerCase() !== 'sha512')
            throw (new Error('ED25519 only supports the use of ' +
                'SHA-512 hashes'));
        this.key = key;
        this.chunks = [];
        stream.Writable.call(this, {});
    }
    util.inherits(Verifier, stream.Writable);
    Verifier.prototype._write = function (chunk, enc, cb) {
        this.chunks.push(chunk);
        cb();
    };
    Verifier.prototype.update = function (chunk) {
        if (typeof (chunk) === 'string')
            chunk = Buffer.from(chunk, 'binary');
        this.chunks.push(chunk);
    };
    Verifier.prototype.verify = function (signature, fmt) {
        var sig;
        if (Signature.isSignature(signature, [2, 0])) {
            if (signature.type !== 'ed25519')
                return (false);
            sig = signature.toBuffer('raw');
        }
        else if (typeof (signature) === 'string') {
            sig = Buffer.from(signature, 'base64');
        }
        else if (Signature.isSignature(signature, [1, 0])) {
            throw (new Error('signature was created by too old ' +
                'a version of sshpk and cannot be verified'));
        }
        assert.buffer(sig);
        return (nacl.sign.detached.verify(new Uint8Array(Buffer.concat(this.chunks)), new Uint8Array(sig), new Uint8Array(this.key.part.A.data)));
    };
    function Signer(key, hashAlgo) {
        if (hashAlgo.toLowerCase() !== 'sha512')
            throw (new Error('ED25519 only supports the use of ' +
                'SHA-512 hashes'));
        this.key = key;
        this.chunks = [];
        stream.Writable.call(this, {});
    }
    util.inherits(Signer, stream.Writable);
    Signer.prototype._write = function (chunk, enc, cb) {
        this.chunks.push(chunk);
        cb();
    };
    Signer.prototype.update = function (chunk) {
        if (typeof (chunk) === 'string')
            chunk = Buffer.from(chunk, 'binary');
        this.chunks.push(chunk);
    };
    Signer.prototype.sign = function () {
        var sig = nacl.sign.detached(new Uint8Array(Buffer.concat(this.chunks)), new Uint8Array(Buffer.concat([
            this.key.part.k.data, this.key.part.A.data
        ])));
        var sigBuf = Buffer.from(sig);
        var sigObj = Signature.parse(sigBuf, 'ed25519', 'raw');
        sigObj.hashAlgorithm = 'sha512';
        return (sigObj);
    };
},
/* sshpk/lib/formats/auto.js */ function _(require, module, exports) {
    // Copyright 2018 Joyent, Inc.
    module.exports = {
        read: read,
        write: write
    };
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var utils = require(863) /* ../utils */;
    var Key = require(856) /* ../key */;
    var PrivateKey = require(861) /* ../private-key */;
    var pem = require(879) /* ./pem */;
    var ssh = require(885) /* ./ssh */;
    var rfc4253 = require(883) /* ./rfc4253 */;
    var dnssec = require(886) /* ./dnssec */;
    var putty = require(887) /* ./putty */;
    var DNSSEC_PRIVKEY_HEADER_PREFIX = 'Private-key-format: v1';
    function read(buf, options) {
        if (typeof (buf) === 'string') {
            if (buf.trim().match(/^[-]+[ ]*BEGIN/))
                return (pem.read(buf, options));
            if (buf.match(/^\s*ssh-[a-z]/))
                return (ssh.read(buf, options));
            if (buf.match(/^\s*ecdsa-/))
                return (ssh.read(buf, options));
            if (buf.match(/^putty-user-key-file-2:/i))
                return (putty.read(buf, options));
            if (findDNSSECHeader(buf))
                return (dnssec.read(buf, options));
            buf = Buffer.from(buf, 'binary');
        }
        else {
            assert.buffer(buf);
            if (findPEMHeader(buf))
                return (pem.read(buf, options));
            if (findSSHHeader(buf))
                return (ssh.read(buf, options));
            if (findPuTTYHeader(buf))
                return (putty.read(buf, options));
            if (findDNSSECHeader(buf))
                return (dnssec.read(buf, options));
        }
        if (buf.readUInt32BE(0) < buf.length)
            return (rfc4253.read(buf, options));
        throw (new Error('Failed to auto-detect format of key'));
    }
    function findPuTTYHeader(buf) {
        var offset = 0;
        while (offset < buf.length &&
            (buf[offset] === 32 || buf[offset] === 10 || buf[offset] === 9))
            ++offset;
        if (offset + 22 <= buf.length &&
            buf.slice(offset, offset + 22).toString('ascii').toLowerCase() ===
                'putty-user-key-file-2:')
            return (true);
        return (false);
    }
    function findSSHHeader(buf) {
        var offset = 0;
        while (offset < buf.length &&
            (buf[offset] === 32 || buf[offset] === 10 || buf[offset] === 9))
            ++offset;
        if (offset + 4 <= buf.length &&
            buf.slice(offset, offset + 4).toString('ascii') === 'ssh-')
            return (true);
        if (offset + 6 <= buf.length &&
            buf.slice(offset, offset + 6).toString('ascii') === 'ecdsa-')
            return (true);
        return (false);
    }
    function findPEMHeader(buf) {
        var offset = 0;
        while (offset < buf.length &&
            (buf[offset] === 32 || buf[offset] === 10))
            ++offset;
        if (buf[offset] !== 45)
            return (false);
        while (offset < buf.length &&
            (buf[offset] === 45))
            ++offset;
        while (offset < buf.length &&
            (buf[offset] === 32))
            ++offset;
        if (offset + 5 > buf.length ||
            buf.slice(offset, offset + 5).toString('ascii') !== 'BEGIN')
            return (false);
        return (true);
    }
    function findDNSSECHeader(buf) {
        // private case first
        if (buf.length <= DNSSEC_PRIVKEY_HEADER_PREFIX.length)
            return (false);
        var headerCheck = buf.slice(0, DNSSEC_PRIVKEY_HEADER_PREFIX.length);
        if (headerCheck.toString('ascii') === DNSSEC_PRIVKEY_HEADER_PREFIX)
            return (true);
        // public-key RFC3110 ?
        // 'domain.com. IN KEY ...' or 'domain.com. IN DNSKEY ...'
        // skip any comment-lines
        if (typeof (buf) !== 'string') {
            buf = buf.toString('ascii');
        }
        var lines = buf.split('\n');
        var line = 0;
        /* JSSTYLED */
        while (lines[line].match(/^\;/))
            line++;
        if (lines[line].toString('ascii').match(/\. IN KEY /))
            return (true);
        if (lines[line].toString('ascii').match(/\. IN DNSKEY /))
            return (true);
        return (false);
    }
    function write(key, options) {
        throw (new Error('"auto" format cannot be used for writing'));
    }
},
/* sshpk/lib/formats/pem.js */ function _(require, module, exports) {
    // Copyright 2018 Joyent, Inc.
    module.exports = {
        read: read,
        write: write
    };
    var assert = require(853) /* assert-plus */;
    var asn1 = require(864) /* asn1 */;
    var crypto = require('crypto');
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ../algs */;
    var utils = require(863) /* ../utils */;
    var Key = require(856) /* ../key */;
    var PrivateKey = require(861) /* ../private-key */;
    var pkcs1 = require(880) /* ./pkcs1 */;
    var pkcs8 = require(881) /* ./pkcs8 */;
    var sshpriv = require(882) /* ./ssh-private */;
    var rfc4253 = require(883) /* ./rfc4253 */;
    var errors = require(860) /* ../errors */;
    var OID_PBES2 = '1.2.840.113549.1.5.13';
    var OID_PBKDF2 = '1.2.840.113549.1.5.12';
    var OID_TO_CIPHER = {
        '1.2.840.113549.3.7': '3des-cbc',
        '2.16.840.1.101.3.4.1.2': 'aes128-cbc',
        '2.16.840.1.101.3.4.1.42': 'aes256-cbc'
    };
    var CIPHER_TO_OID = {};
    Object.keys(OID_TO_CIPHER).forEach(function (k) {
        CIPHER_TO_OID[OID_TO_CIPHER[k]] = k;
    });
    var OID_TO_HASH = {
        '1.2.840.113549.2.7': 'sha1',
        '1.2.840.113549.2.9': 'sha256',
        '1.2.840.113549.2.11': 'sha512'
    };
    var HASH_TO_OID = {};
    Object.keys(OID_TO_HASH).forEach(function (k) {
        HASH_TO_OID[OID_TO_HASH[k]] = k;
    });
    /*
     * For reading we support both PKCS#1 and PKCS#8. If we find a private key,
     * we just take the public component of it and use that.
     */
    function read(buf, options, forceType) {
        var input = buf;
        if (typeof (buf) !== 'string') {
            assert.buffer(buf, 'buf');
            buf = buf.toString('ascii');
        }
        var lines = buf.trim().split(/[\r\n]+/g);
        var m;
        var si = -1;
        while (!m && si < lines.length) {
            m = lines[++si].match(/*JSSTYLED*/ /[-]+[ ]*BEGIN ([A-Z0-9][A-Za-z0-9]+ )?(PUBLIC|PRIVATE) KEY[ ]*[-]+/);
        }
        assert.ok(m, 'invalid PEM header');
        var m2;
        var ei = lines.length;
        while (!m2 && ei > 0) {
            m2 = lines[--ei].match(/*JSSTYLED*/ /[-]+[ ]*END ([A-Z0-9][A-Za-z0-9]+ )?(PUBLIC|PRIVATE) KEY[ ]*[-]+/);
        }
        assert.ok(m2, 'invalid PEM footer');
        /* Begin and end banners must match key type */
        assert.equal(m[2], m2[2]);
        var type = m[2].toLowerCase();
        var alg;
        if (m[1]) {
            /* They also must match algorithms, if given */
            assert.equal(m[1], m2[1], 'PEM header and footer mismatch');
            alg = m[1].trim();
        }
        lines = lines.slice(si, ei + 1);
        var headers = {};
        while (true) {
            lines = lines.slice(1);
            m = lines[0].match(/*JSSTYLED*/ /^([A-Za-z0-9-]+): (.+)$/);
            if (!m)
                break;
            headers[m[1].toLowerCase()] = m[2];
        }
        /* Chop off the first and last lines */
        lines = lines.slice(0, -1).join('');
        buf = Buffer.from(lines, 'base64');
        var cipher, key, iv;
        if (headers['proc-type']) {
            var parts = headers['proc-type'].split(',');
            if (parts[0] === '4' && parts[1] === 'ENCRYPTED') {
                if (typeof (options.passphrase) === 'string') {
                    options.passphrase = Buffer.from(options.passphrase, 'utf-8');
                }
                if (!Buffer.isBuffer(options.passphrase)) {
                    throw (new errors.KeyEncryptedError(options.filename, 'PEM'));
                }
                else {
                    parts = headers['dek-info'].split(',');
                    assert.ok(parts.length === 2);
                    cipher = parts[0].toLowerCase();
                    iv = Buffer.from(parts[1], 'hex');
                    key = utils.opensslKeyDeriv(cipher, iv, options.passphrase, 1).key;
                }
            }
        }
        if (alg && alg.toLowerCase() === 'encrypted') {
            var eder = new asn1.BerReader(buf);
            var pbesEnd;
            eder.readSequence();
            eder.readSequence();
            pbesEnd = eder.offset + eder.length;
            var method = eder.readOID();
            if (method !== OID_PBES2) {
                throw (new Error('Unsupported PEM/PKCS8 encryption ' +
                    'scheme: ' + method));
            }
            eder.readSequence(); /* PBES2-params */
            eder.readSequence(); /* keyDerivationFunc */
            var kdfEnd = eder.offset + eder.length;
            var kdfOid = eder.readOID();
            if (kdfOid !== OID_PBKDF2)
                throw (new Error('Unsupported PBES2 KDF: ' + kdfOid));
            eder.readSequence();
            var salt = eder.readString(asn1.Ber.OctetString, true);
            var iterations = eder.readInt();
            var hashAlg = 'sha1';
            if (eder.offset < kdfEnd) {
                eder.readSequence();
                var hashAlgOid = eder.readOID();
                hashAlg = OID_TO_HASH[hashAlgOid];
                if (hashAlg === undefined) {
                    throw (new Error('Unsupported PBKDF2 hash: ' +
                        hashAlgOid));
                }
            }
            eder._offset = kdfEnd;
            eder.readSequence(); /* encryptionScheme */
            var cipherOid = eder.readOID();
            cipher = OID_TO_CIPHER[cipherOid];
            if (cipher === undefined) {
                throw (new Error('Unsupported PBES2 cipher: ' +
                    cipherOid));
            }
            iv = eder.readString(asn1.Ber.OctetString, true);
            eder._offset = pbesEnd;
            buf = eder.readString(asn1.Ber.OctetString, true);
            if (typeof (options.passphrase) === 'string') {
                options.passphrase = Buffer.from(options.passphrase, 'utf-8');
            }
            if (!Buffer.isBuffer(options.passphrase)) {
                throw (new errors.KeyEncryptedError(options.filename, 'PEM'));
            }
            var cinfo = utils.opensshCipherInfo(cipher);
            cipher = cinfo.opensslName;
            key = utils.pbkdf2(hashAlg, salt, iterations, cinfo.keySize, options.passphrase);
            alg = undefined;
        }
        if (cipher && key && iv) {
            var cipherStream = crypto.createDecipheriv(cipher, key, iv);
            var chunk, chunks = [];
            cipherStream.once('error', function (e) {
                if (e.toString().indexOf('bad decrypt') !== -1) {
                    throw (new Error('Incorrect passphrase ' +
                        'supplied, could not decrypt key'));
                }
                throw (e);
            });
            cipherStream.write(buf);
            cipherStream.end();
            while ((chunk = cipherStream.read()) !== null)
                chunks.push(chunk);
            buf = Buffer.concat(chunks);
        }
        /* The new OpenSSH internal format abuses PEM headers */
        if (alg && alg.toLowerCase() === 'openssh')
            return (sshpriv.readSSHPrivate(type, buf, options));
        if (alg && alg.toLowerCase() === 'ssh2')
            return (rfc4253.readType(type, buf, options));
        var der = new asn1.BerReader(buf);
        der.originalInput = input;
        /*
         * All of the PEM file types start with a sequence tag, so chop it
         * off here
         */
        der.readSequence();
        /* PKCS#1 type keys name an algorithm in the banner explicitly */
        if (alg) {
            if (forceType)
                assert.strictEqual(forceType, 'pkcs1');
            return (pkcs1.readPkcs1(alg, type, der));
        }
        else {
            if (forceType)
                assert.strictEqual(forceType, 'pkcs8');
            return (pkcs8.readPkcs8(alg, type, der));
        }
    }
    function write(key, options, type) {
        assert.object(key);
        var alg = {
            'ecdsa': 'EC',
            'rsa': 'RSA',
            'dsa': 'DSA',
            'ed25519': 'EdDSA'
        }[key.type];
        var header;
        var der = new asn1.BerWriter();
        if (PrivateKey.isPrivateKey(key)) {
            if (type && type === 'pkcs8') {
                header = 'PRIVATE KEY';
                pkcs8.writePkcs8(der, key);
            }
            else {
                if (type)
                    assert.strictEqual(type, 'pkcs1');
                header = alg + ' PRIVATE KEY';
                pkcs1.writePkcs1(der, key);
            }
        }
        else if (Key.isKey(key)) {
            if (type && type === 'pkcs1') {
                header = alg + ' PUBLIC KEY';
                pkcs1.writePkcs1(der, key);
            }
            else {
                if (type)
                    assert.strictEqual(type, 'pkcs8');
                header = 'PUBLIC KEY';
                pkcs8.writePkcs8(der, key);
            }
        }
        else {
            throw (new Error('key is not a Key or PrivateKey'));
        }
        var tmp = der.buffer.toString('base64');
        var len = tmp.length + (tmp.length / 64) +
            18 + 16 + header.length * 2 + 10;
        var buf = Buffer.alloc(len);
        var o = 0;
        o += buf.write('-----BEGIN ' + header + '-----\n', o);
        for (var i = 0; i < tmp.length;) {
            var limit = i + 64;
            if (limit > tmp.length)
                limit = tmp.length;
            o += buf.write(tmp.slice(i, limit), o);
            buf[o++] = 10;
            i = limit;
        }
        o += buf.write('-----END ' + header + '-----\n', o);
        return (buf.slice(0, o));
    }
},
/* sshpk/lib/formats/pkcs1.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    module.exports = {
        read: read,
        readPkcs1: readPkcs1,
        write: write,
        writePkcs1: writePkcs1
    };
    var assert = require(853) /* assert-plus */;
    var asn1 = require(864) /* asn1 */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ../algs */;
    var utils = require(863) /* ../utils */;
    var Key = require(856) /* ../key */;
    var PrivateKey = require(861) /* ../private-key */;
    var pem = require(879) /* ./pem */;
    var pkcs8 = require(881) /* ./pkcs8 */;
    var readECDSACurve = pkcs8.readECDSACurve;
    function read(buf, options) {
        return (pem.read(buf, options, 'pkcs1'));
    }
    function write(key, options) {
        return (pem.write(key, options, 'pkcs1'));
    }
    /* Helper to read in a single mpint */
    function readMPInt(der, nm) {
        assert.strictEqual(der.peek(), asn1.Ber.Integer, nm + ' is not an Integer');
        return (utils.mpNormalize(der.readString(asn1.Ber.Integer, true)));
    }
    function readPkcs1(alg, type, der) {
        switch (alg) {
            case 'RSA':
                if (type === 'public')
                    return (readPkcs1RSAPublic(der));
                else if (type === 'private')
                    return (readPkcs1RSAPrivate(der));
                throw (new Error('Unknown key type: ' + type));
            case 'DSA':
                if (type === 'public')
                    return (readPkcs1DSAPublic(der));
                else if (type === 'private')
                    return (readPkcs1DSAPrivate(der));
                throw (new Error('Unknown key type: ' + type));
            case 'EC':
            case 'ECDSA':
                if (type === 'private')
                    return (readPkcs1ECDSAPrivate(der));
                else if (type === 'public')
                    return (readPkcs1ECDSAPublic(der));
                throw (new Error('Unknown key type: ' + type));
            case 'EDDSA':
            case 'EdDSA':
                if (type === 'private')
                    return (readPkcs1EdDSAPrivate(der));
                throw (new Error(type + ' keys not supported with EdDSA'));
            default:
                throw (new Error('Unknown key algo: ' + alg));
        }
    }
    function readPkcs1RSAPublic(der) {
        // modulus and exponent
        var n = readMPInt(der, 'modulus');
        var e = readMPInt(der, 'exponent');
        // now, make the key
        var key = {
            type: 'rsa',
            parts: [
                { name: 'e', data: e },
                { name: 'n', data: n }
            ]
        };
        return (new Key(key));
    }
    function readPkcs1RSAPrivate(der) {
        var version = readMPInt(der, 'version');
        assert.strictEqual(version[0], 0);
        // modulus then public exponent
        var n = readMPInt(der, 'modulus');
        var e = readMPInt(der, 'public exponent');
        var d = readMPInt(der, 'private exponent');
        var p = readMPInt(der, 'prime1');
        var q = readMPInt(der, 'prime2');
        var dmodp = readMPInt(der, 'exponent1');
        var dmodq = readMPInt(der, 'exponent2');
        var iqmp = readMPInt(der, 'iqmp');
        // now, make the key
        var key = {
            type: 'rsa',
            parts: [
                { name: 'n', data: n },
                { name: 'e', data: e },
                { name: 'd', data: d },
                { name: 'iqmp', data: iqmp },
                { name: 'p', data: p },
                { name: 'q', data: q },
                { name: 'dmodp', data: dmodp },
                { name: 'dmodq', data: dmodq }
            ]
        };
        return (new PrivateKey(key));
    }
    function readPkcs1DSAPrivate(der) {
        var version = readMPInt(der, 'version');
        assert.strictEqual(version.readUInt8(0), 0);
        var p = readMPInt(der, 'p');
        var q = readMPInt(der, 'q');
        var g = readMPInt(der, 'g');
        var y = readMPInt(der, 'y');
        var x = readMPInt(der, 'x');
        // now, make the key
        var key = {
            type: 'dsa',
            parts: [
                { name: 'p', data: p },
                { name: 'q', data: q },
                { name: 'g', data: g },
                { name: 'y', data: y },
                { name: 'x', data: x }
            ]
        };
        return (new PrivateKey(key));
    }
    function readPkcs1EdDSAPrivate(der) {
        var version = readMPInt(der, 'version');
        assert.strictEqual(version.readUInt8(0), 1);
        // private key
        var k = der.readString(asn1.Ber.OctetString, true);
        der.readSequence(0xa0);
        var oid = der.readOID();
        assert.strictEqual(oid, '1.3.101.112', 'the ed25519 curve identifier');
        der.readSequence(0xa1);
        var A = utils.readBitString(der);
        var key = {
            type: 'ed25519',
            parts: [
                { name: 'A', data: utils.zeroPadToLength(A, 32) },
                { name: 'k', data: k }
            ]
        };
        return (new PrivateKey(key));
    }
    function readPkcs1DSAPublic(der) {
        var y = readMPInt(der, 'y');
        var p = readMPInt(der, 'p');
        var q = readMPInt(der, 'q');
        var g = readMPInt(der, 'g');
        var key = {
            type: 'dsa',
            parts: [
                { name: 'y', data: y },
                { name: 'p', data: p },
                { name: 'q', data: q },
                { name: 'g', data: g }
            ]
        };
        return (new Key(key));
    }
    function readPkcs1ECDSAPublic(der) {
        der.readSequence();
        var oid = der.readOID();
        assert.strictEqual(oid, '1.2.840.10045.2.1', 'must be ecPublicKey');
        var curveOid = der.readOID();
        var curve;
        var curves = Object.keys(algs.curves);
        for (var j = 0; j < curves.length; ++j) {
            var c = curves[j];
            var cd = algs.curves[c];
            if (cd.pkcs8oid === curveOid) {
                curve = c;
                break;
            }
        }
        assert.string(curve, 'a known ECDSA named curve');
        var Q = der.readString(asn1.Ber.BitString, true);
        Q = utils.ecNormalize(Q);
        var key = {
            type: 'ecdsa',
            parts: [
                { name: 'curve', data: Buffer.from(curve) },
                { name: 'Q', data: Q }
            ]
        };
        return (new Key(key));
    }
    function readPkcs1ECDSAPrivate(der) {
        var version = readMPInt(der, 'version');
        assert.strictEqual(version.readUInt8(0), 1);
        // private key
        var d = der.readString(asn1.Ber.OctetString, true);
        der.readSequence(0xa0);
        var curve = readECDSACurve(der);
        assert.string(curve, 'a known elliptic curve');
        der.readSequence(0xa1);
        var Q = der.readString(asn1.Ber.BitString, true);
        Q = utils.ecNormalize(Q);
        var key = {
            type: 'ecdsa',
            parts: [
                { name: 'curve', data: Buffer.from(curve) },
                { name: 'Q', data: Q },
                { name: 'd', data: d }
            ]
        };
        return (new PrivateKey(key));
    }
    function writePkcs1(der, key) {
        der.startSequence();
        switch (key.type) {
            case 'rsa':
                if (PrivateKey.isPrivateKey(key))
                    writePkcs1RSAPrivate(der, key);
                else
                    writePkcs1RSAPublic(der, key);
                break;
            case 'dsa':
                if (PrivateKey.isPrivateKey(key))
                    writePkcs1DSAPrivate(der, key);
                else
                    writePkcs1DSAPublic(der, key);
                break;
            case 'ecdsa':
                if (PrivateKey.isPrivateKey(key))
                    writePkcs1ECDSAPrivate(der, key);
                else
                    writePkcs1ECDSAPublic(der, key);
                break;
            case 'ed25519':
                if (PrivateKey.isPrivateKey(key))
                    writePkcs1EdDSAPrivate(der, key);
                else
                    writePkcs1EdDSAPublic(der, key);
                break;
            default:
                throw (new Error('Unknown key algo: ' + key.type));
        }
        der.endSequence();
    }
    function writePkcs1RSAPublic(der, key) {
        der.writeBuffer(key.part.n.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.e.data, asn1.Ber.Integer);
    }
    function writePkcs1RSAPrivate(der, key) {
        var ver = Buffer.from([0]);
        der.writeBuffer(ver, asn1.Ber.Integer);
        der.writeBuffer(key.part.n.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.e.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.d.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.p.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.q.data, asn1.Ber.Integer);
        if (!key.part.dmodp || !key.part.dmodq)
            utils.addRSAMissing(key);
        der.writeBuffer(key.part.dmodp.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.dmodq.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.iqmp.data, asn1.Ber.Integer);
    }
    function writePkcs1DSAPrivate(der, key) {
        var ver = Buffer.from([0]);
        der.writeBuffer(ver, asn1.Ber.Integer);
        der.writeBuffer(key.part.p.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.q.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.g.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.y.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.x.data, asn1.Ber.Integer);
    }
    function writePkcs1DSAPublic(der, key) {
        der.writeBuffer(key.part.y.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.p.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.q.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.g.data, asn1.Ber.Integer);
    }
    function writePkcs1ECDSAPublic(der, key) {
        der.startSequence();
        der.writeOID('1.2.840.10045.2.1'); /* ecPublicKey */
        var curve = key.part.curve.data.toString();
        var curveOid = algs.curves[curve].pkcs8oid;
        assert.string(curveOid, 'a known ECDSA named curve');
        der.writeOID(curveOid);
        der.endSequence();
        var Q = utils.ecNormalize(key.part.Q.data, true);
        der.writeBuffer(Q, asn1.Ber.BitString);
    }
    function writePkcs1ECDSAPrivate(der, key) {
        var ver = Buffer.from([1]);
        der.writeBuffer(ver, asn1.Ber.Integer);
        der.writeBuffer(key.part.d.data, asn1.Ber.OctetString);
        der.startSequence(0xa0);
        var curve = key.part.curve.data.toString();
        var curveOid = algs.curves[curve].pkcs8oid;
        assert.string(curveOid, 'a known ECDSA named curve');
        der.writeOID(curveOid);
        der.endSequence();
        der.startSequence(0xa1);
        var Q = utils.ecNormalize(key.part.Q.data, true);
        der.writeBuffer(Q, asn1.Ber.BitString);
        der.endSequence();
    }
    function writePkcs1EdDSAPrivate(der, key) {
        var ver = Buffer.from([1]);
        der.writeBuffer(ver, asn1.Ber.Integer);
        der.writeBuffer(key.part.k.data, asn1.Ber.OctetString);
        der.startSequence(0xa0);
        der.writeOID('1.3.101.112');
        der.endSequence();
        der.startSequence(0xa1);
        utils.writeBitString(der, key.part.A.data);
        der.endSequence();
    }
    function writePkcs1EdDSAPublic(der, key) {
        throw (new Error('Public keys are not supported for EdDSA PKCS#1'));
    }
},
/* sshpk/lib/formats/pkcs8.js */ function _(require, module, exports) {
    // Copyright 2018 Joyent, Inc.
    module.exports = {
        read: read,
        readPkcs8: readPkcs8,
        write: write,
        writePkcs8: writePkcs8,
        pkcs8ToBuffer: pkcs8ToBuffer,
        readECDSACurve: readECDSACurve,
        writeECDSACurve: writeECDSACurve
    };
    var assert = require(853) /* assert-plus */;
    var asn1 = require(864) /* asn1 */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ../algs */;
    var utils = require(863) /* ../utils */;
    var Key = require(856) /* ../key */;
    var PrivateKey = require(861) /* ../private-key */;
    var pem = require(879) /* ./pem */;
    function read(buf, options) {
        return (pem.read(buf, options, 'pkcs8'));
    }
    function write(key, options) {
        return (pem.write(key, options, 'pkcs8'));
    }
    /* Helper to read in a single mpint */
    function readMPInt(der, nm) {
        assert.strictEqual(der.peek(), asn1.Ber.Integer, nm + ' is not an Integer');
        return (utils.mpNormalize(der.readString(asn1.Ber.Integer, true)));
    }
    function readPkcs8(alg, type, der) {
        /* Private keys in pkcs#8 format have a weird extra int */
        if (der.peek() === asn1.Ber.Integer) {
            assert.strictEqual(type, 'private', 'unexpected Integer at start of public key');
            der.readString(asn1.Ber.Integer, true);
        }
        der.readSequence();
        var next = der.offset + der.length;
        var oid = der.readOID();
        switch (oid) {
            case '1.2.840.113549.1.1.1':
                der._offset = next;
                if (type === 'public')
                    return (readPkcs8RSAPublic(der));
                else
                    return (readPkcs8RSAPrivate(der));
            case '1.2.840.10040.4.1':
                if (type === 'public')
                    return (readPkcs8DSAPublic(der));
                else
                    return (readPkcs8DSAPrivate(der));
            case '1.2.840.10045.2.1':
                if (type === 'public')
                    return (readPkcs8ECDSAPublic(der));
                else
                    return (readPkcs8ECDSAPrivate(der));
            case '1.3.101.112':
                if (type === 'public') {
                    return (readPkcs8EdDSAPublic(der));
                }
                else {
                    return (readPkcs8EdDSAPrivate(der));
                }
            case '1.3.101.110':
                if (type === 'public') {
                    return (readPkcs8X25519Public(der));
                }
                else {
                    return (readPkcs8X25519Private(der));
                }
            default:
                throw (new Error('Unknown key type OID ' + oid));
        }
    }
    function readPkcs8RSAPublic(der) {
        // bit string sequence
        der.readSequence(asn1.Ber.BitString);
        der.readByte();
        der.readSequence();
        // modulus
        var n = readMPInt(der, 'modulus');
        var e = readMPInt(der, 'exponent');
        // now, make the key
        var key = {
            type: 'rsa',
            source: der.originalInput,
            parts: [
                { name: 'e', data: e },
                { name: 'n', data: n }
            ]
        };
        return (new Key(key));
    }
    function readPkcs8RSAPrivate(der) {
        der.readSequence(asn1.Ber.OctetString);
        der.readSequence();
        var ver = readMPInt(der, 'version');
        assert.equal(ver[0], 0x0, 'unknown RSA private key version');
        // modulus then public exponent
        var n = readMPInt(der, 'modulus');
        var e = readMPInt(der, 'public exponent');
        var d = readMPInt(der, 'private exponent');
        var p = readMPInt(der, 'prime1');
        var q = readMPInt(der, 'prime2');
        var dmodp = readMPInt(der, 'exponent1');
        var dmodq = readMPInt(der, 'exponent2');
        var iqmp = readMPInt(der, 'iqmp');
        // now, make the key
        var key = {
            type: 'rsa',
            parts: [
                { name: 'n', data: n },
                { name: 'e', data: e },
                { name: 'd', data: d },
                { name: 'iqmp', data: iqmp },
                { name: 'p', data: p },
                { name: 'q', data: q },
                { name: 'dmodp', data: dmodp },
                { name: 'dmodq', data: dmodq }
            ]
        };
        return (new PrivateKey(key));
    }
    function readPkcs8DSAPublic(der) {
        der.readSequence();
        var p = readMPInt(der, 'p');
        var q = readMPInt(der, 'q');
        var g = readMPInt(der, 'g');
        // bit string sequence
        der.readSequence(asn1.Ber.BitString);
        der.readByte();
        var y = readMPInt(der, 'y');
        // now, make the key
        var key = {
            type: 'dsa',
            parts: [
                { name: 'p', data: p },
                { name: 'q', data: q },
                { name: 'g', data: g },
                { name: 'y', data: y }
            ]
        };
        return (new Key(key));
    }
    function readPkcs8DSAPrivate(der) {
        der.readSequence();
        var p = readMPInt(der, 'p');
        var q = readMPInt(der, 'q');
        var g = readMPInt(der, 'g');
        der.readSequence(asn1.Ber.OctetString);
        var x = readMPInt(der, 'x');
        /* The pkcs#8 format does not include the public key */
        var y = utils.calculateDSAPublic(g, p, x);
        var key = {
            type: 'dsa',
            parts: [
                { name: 'p', data: p },
                { name: 'q', data: q },
                { name: 'g', data: g },
                { name: 'y', data: y },
                { name: 'x', data: x }
            ]
        };
        return (new PrivateKey(key));
    }
    function readECDSACurve(der) {
        var curveName, curveNames;
        var j, c, cd;
        if (der.peek() === asn1.Ber.OID) {
            var oid = der.readOID();
            curveNames = Object.keys(algs.curves);
            for (j = 0; j < curveNames.length; ++j) {
                c = curveNames[j];
                cd = algs.curves[c];
                if (cd.pkcs8oid === oid) {
                    curveName = c;
                    break;
                }
            }
        }
        else {
            // ECParameters sequence
            der.readSequence();
            var version = der.readString(asn1.Ber.Integer, true);
            assert.strictEqual(version[0], 1, 'ECDSA key not version 1');
            var curve = {};
            // FieldID sequence
            der.readSequence();
            var fieldTypeOid = der.readOID();
            assert.strictEqual(fieldTypeOid, '1.2.840.10045.1.1', 'ECDSA key is not from a prime-field');
            var p = curve.p = utils.mpNormalize(der.readString(asn1.Ber.Integer, true));
            /*
             * p always starts with a 1 bit, so count the zeros to get its
             * real size.
             */
            curve.size = p.length * 8 - utils.countZeros(p);
            // Curve sequence
            der.readSequence();
            curve.a = utils.mpNormalize(der.readString(asn1.Ber.OctetString, true));
            curve.b = utils.mpNormalize(der.readString(asn1.Ber.OctetString, true));
            if (der.peek() === asn1.Ber.BitString)
                curve.s = der.readString(asn1.Ber.BitString, true);
            // Combined Gx and Gy
            curve.G = der.readString(asn1.Ber.OctetString, true);
            assert.strictEqual(curve.G[0], 0x4, 'uncompressed G is required');
            curve.n = utils.mpNormalize(der.readString(asn1.Ber.Integer, true));
            curve.h = utils.mpNormalize(der.readString(asn1.Ber.Integer, true));
            assert.strictEqual(curve.h[0], 0x1, 'a cofactor=1 curve is ' +
                'required');
            curveNames = Object.keys(algs.curves);
            var ks = Object.keys(curve);
            for (j = 0; j < curveNames.length; ++j) {
                c = curveNames[j];
                cd = algs.curves[c];
                var equal = true;
                for (var i = 0; i < ks.length; ++i) {
                    var k = ks[i];
                    if (cd[k] === undefined)
                        continue;
                    if (typeof (cd[k]) === 'object' &&
                        cd[k].equals !== undefined) {
                        if (!cd[k].equals(curve[k])) {
                            equal = false;
                            break;
                        }
                    }
                    else if (Buffer.isBuffer(cd[k])) {
                        if (cd[k].toString('binary')
                            !== curve[k].toString('binary')) {
                            equal = false;
                            break;
                        }
                    }
                    else {
                        if (cd[k] !== curve[k]) {
                            equal = false;
                            break;
                        }
                    }
                }
                if (equal) {
                    curveName = c;
                    break;
                }
            }
        }
        return (curveName);
    }
    function readPkcs8ECDSAPrivate(der) {
        var curveName = readECDSACurve(der);
        assert.string(curveName, 'a known elliptic curve');
        der.readSequence(asn1.Ber.OctetString);
        der.readSequence();
        var version = readMPInt(der, 'version');
        assert.equal(version[0], 1, 'unknown version of ECDSA key');
        var d = der.readString(asn1.Ber.OctetString, true);
        var Q;
        if (der.peek() == 0xa0) {
            der.readSequence(0xa0);
            der._offset += der.length;
        }
        if (der.peek() == 0xa1) {
            der.readSequence(0xa1);
            Q = der.readString(asn1.Ber.BitString, true);
            Q = utils.ecNormalize(Q);
        }
        if (Q === undefined) {
            var pub = utils.publicFromPrivateECDSA(curveName, d);
            Q = pub.part.Q.data;
        }
        var key = {
            type: 'ecdsa',
            parts: [
                { name: 'curve', data: Buffer.from(curveName) },
                { name: 'Q', data: Q },
                { name: 'd', data: d }
            ]
        };
        return (new PrivateKey(key));
    }
    function readPkcs8ECDSAPublic(der) {
        var curveName = readECDSACurve(der);
        assert.string(curveName, 'a known elliptic curve');
        var Q = der.readString(asn1.Ber.BitString, true);
        Q = utils.ecNormalize(Q);
        var key = {
            type: 'ecdsa',
            parts: [
                { name: 'curve', data: Buffer.from(curveName) },
                { name: 'Q', data: Q }
            ]
        };
        return (new Key(key));
    }
    function readPkcs8EdDSAPublic(der) {
        if (der.peek() === 0x00)
            der.readByte();
        var A = utils.readBitString(der);
        var key = {
            type: 'ed25519',
            parts: [
                { name: 'A', data: utils.zeroPadToLength(A, 32) }
            ]
        };
        return (new Key(key));
    }
    function readPkcs8X25519Public(der) {
        var A = utils.readBitString(der);
        var key = {
            type: 'curve25519',
            parts: [
                { name: 'A', data: utils.zeroPadToLength(A, 32) }
            ]
        };
        return (new Key(key));
    }
    function readPkcs8EdDSAPrivate(der) {
        if (der.peek() === 0x00)
            der.readByte();
        der.readSequence(asn1.Ber.OctetString);
        var k = der.readString(asn1.Ber.OctetString, true);
        k = utils.zeroPadToLength(k, 32);
        var A;
        if (der.peek() === asn1.Ber.BitString) {
            A = utils.readBitString(der);
            A = utils.zeroPadToLength(A, 32);
        }
        else {
            A = utils.calculateED25519Public(k);
        }
        var key = {
            type: 'ed25519',
            parts: [
                { name: 'A', data: utils.zeroPadToLength(A, 32) },
                { name: 'k', data: utils.zeroPadToLength(k, 32) }
            ]
        };
        return (new PrivateKey(key));
    }
    function readPkcs8X25519Private(der) {
        if (der.peek() === 0x00)
            der.readByte();
        der.readSequence(asn1.Ber.OctetString);
        var k = der.readString(asn1.Ber.OctetString, true);
        k = utils.zeroPadToLength(k, 32);
        var A = utils.calculateX25519Public(k);
        var key = {
            type: 'curve25519',
            parts: [
                { name: 'A', data: utils.zeroPadToLength(A, 32) },
                { name: 'k', data: utils.zeroPadToLength(k, 32) }
            ]
        };
        return (new PrivateKey(key));
    }
    function pkcs8ToBuffer(key) {
        var der = new asn1.BerWriter();
        writePkcs8(der, key);
        return (der.buffer);
    }
    function writePkcs8(der, key) {
        der.startSequence();
        if (PrivateKey.isPrivateKey(key)) {
            var sillyInt = Buffer.from([0]);
            der.writeBuffer(sillyInt, asn1.Ber.Integer);
        }
        der.startSequence();
        switch (key.type) {
            case 'rsa':
                der.writeOID('1.2.840.113549.1.1.1');
                if (PrivateKey.isPrivateKey(key))
                    writePkcs8RSAPrivate(key, der);
                else
                    writePkcs8RSAPublic(key, der);
                break;
            case 'dsa':
                der.writeOID('1.2.840.10040.4.1');
                if (PrivateKey.isPrivateKey(key))
                    writePkcs8DSAPrivate(key, der);
                else
                    writePkcs8DSAPublic(key, der);
                break;
            case 'ecdsa':
                der.writeOID('1.2.840.10045.2.1');
                if (PrivateKey.isPrivateKey(key))
                    writePkcs8ECDSAPrivate(key, der);
                else
                    writePkcs8ECDSAPublic(key, der);
                break;
            case 'ed25519':
                der.writeOID('1.3.101.112');
                if (PrivateKey.isPrivateKey(key))
                    throw (new Error('Ed25519 private keys in pkcs8 ' +
                        'format are not supported'));
                writePkcs8EdDSAPublic(key, der);
                break;
            default:
                throw (new Error('Unsupported key type: ' + key.type));
        }
        der.endSequence();
    }
    function writePkcs8RSAPrivate(key, der) {
        der.writeNull();
        der.endSequence();
        der.startSequence(asn1.Ber.OctetString);
        der.startSequence();
        var version = Buffer.from([0]);
        der.writeBuffer(version, asn1.Ber.Integer);
        der.writeBuffer(key.part.n.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.e.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.d.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.p.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.q.data, asn1.Ber.Integer);
        if (!key.part.dmodp || !key.part.dmodq)
            utils.addRSAMissing(key);
        der.writeBuffer(key.part.dmodp.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.dmodq.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.iqmp.data, asn1.Ber.Integer);
        der.endSequence();
        der.endSequence();
    }
    function writePkcs8RSAPublic(key, der) {
        der.writeNull();
        der.endSequence();
        der.startSequence(asn1.Ber.BitString);
        der.writeByte(0x00);
        der.startSequence();
        der.writeBuffer(key.part.n.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.e.data, asn1.Ber.Integer);
        der.endSequence();
        der.endSequence();
    }
    function writePkcs8DSAPrivate(key, der) {
        der.startSequence();
        der.writeBuffer(key.part.p.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.q.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.g.data, asn1.Ber.Integer);
        der.endSequence();
        der.endSequence();
        der.startSequence(asn1.Ber.OctetString);
        der.writeBuffer(key.part.x.data, asn1.Ber.Integer);
        der.endSequence();
    }
    function writePkcs8DSAPublic(key, der) {
        der.startSequence();
        der.writeBuffer(key.part.p.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.q.data, asn1.Ber.Integer);
        der.writeBuffer(key.part.g.data, asn1.Ber.Integer);
        der.endSequence();
        der.endSequence();
        der.startSequence(asn1.Ber.BitString);
        der.writeByte(0x00);
        der.writeBuffer(key.part.y.data, asn1.Ber.Integer);
        der.endSequence();
    }
    function writeECDSACurve(key, der) {
        var curve = algs.curves[key.curve];
        if (curve.pkcs8oid) {
            /* This one has a name in pkcs#8, so just write the oid */
            der.writeOID(curve.pkcs8oid);
        }
        else {
            // ECParameters sequence
            der.startSequence();
            var version = Buffer.from([1]);
            der.writeBuffer(version, asn1.Ber.Integer);
            // FieldID sequence
            der.startSequence();
            der.writeOID('1.2.840.10045.1.1'); // prime-field
            der.writeBuffer(curve.p, asn1.Ber.Integer);
            der.endSequence();
            // Curve sequence
            der.startSequence();
            var a = curve.p;
            if (a[0] === 0x0)
                a = a.slice(1);
            der.writeBuffer(a, asn1.Ber.OctetString);
            der.writeBuffer(curve.b, asn1.Ber.OctetString);
            der.writeBuffer(curve.s, asn1.Ber.BitString);
            der.endSequence();
            der.writeBuffer(curve.G, asn1.Ber.OctetString);
            der.writeBuffer(curve.n, asn1.Ber.Integer);
            var h = curve.h;
            if (!h) {
                h = Buffer.from([1]);
            }
            der.writeBuffer(h, asn1.Ber.Integer);
            // ECParameters
            der.endSequence();
        }
    }
    function writePkcs8ECDSAPublic(key, der) {
        writeECDSACurve(key, der);
        der.endSequence();
        var Q = utils.ecNormalize(key.part.Q.data, true);
        der.writeBuffer(Q, asn1.Ber.BitString);
    }
    function writePkcs8ECDSAPrivate(key, der) {
        writeECDSACurve(key, der);
        der.endSequence();
        der.startSequence(asn1.Ber.OctetString);
        der.startSequence();
        var version = Buffer.from([1]);
        der.writeBuffer(version, asn1.Ber.Integer);
        der.writeBuffer(key.part.d.data, asn1.Ber.OctetString);
        der.startSequence(0xa1);
        var Q = utils.ecNormalize(key.part.Q.data, true);
        der.writeBuffer(Q, asn1.Ber.BitString);
        der.endSequence();
        der.endSequence();
        der.endSequence();
    }
    function writePkcs8EdDSAPublic(key, der) {
        der.endSequence();
        utils.writeBitString(der, key.part.A.data);
    }
    function writePkcs8EdDSAPrivate(key, der) {
        der.endSequence();
        var k = utils.mpNormalize(key.part.k.data, true);
        der.startSequence(asn1.Ber.OctetString);
        der.writeBuffer(k, asn1.Ber.OctetString);
        der.endSequence();
    }
},
/* sshpk/lib/formats/ssh-private.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    module.exports = {
        read: read,
        readSSHPrivate: readSSHPrivate,
        write: write
    };
    var assert = require(853) /* assert-plus */;
    var asn1 = require(864) /* asn1 */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ../algs */;
    var utils = require(863) /* ../utils */;
    var crypto = require('crypto');
    var Key = require(856) /* ../key */;
    var PrivateKey = require(861) /* ../private-key */;
    var pem = require(879) /* ./pem */;
    var rfc4253 = require(883) /* ./rfc4253 */;
    var SSHBuffer = require(873) /* ../ssh-buffer */;
    var errors = require(860) /* ../errors */;
    var bcrypt;
    function read(buf, options) {
        return (pem.read(buf, options));
    }
    var MAGIC = 'openssh-key-v1';
    function readSSHPrivate(type, buf, options) {
        buf = new SSHBuffer({ buffer: buf });
        var magic = buf.readCString();
        assert.strictEqual(magic, MAGIC, 'bad magic string');
        var cipher = buf.readString();
        var kdf = buf.readString();
        var kdfOpts = buf.readBuffer();
        var nkeys = buf.readInt();
        if (nkeys !== 1) {
            throw (new Error('OpenSSH-format key file contains ' +
                'multiple keys: this is unsupported.'));
        }
        var pubKey = buf.readBuffer();
        if (type === 'public') {
            assert.ok(buf.atEnd(), 'excess bytes left after key');
            return (rfc4253.read(pubKey));
        }
        var privKeyBlob = buf.readBuffer();
        assert.ok(buf.atEnd(), 'excess bytes left after key');
        var kdfOptsBuf = new SSHBuffer({ buffer: kdfOpts });
        switch (kdf) {
            case 'none':
                if (cipher !== 'none') {
                    throw (new Error('OpenSSH-format key uses KDF "none" ' +
                        'but specifies a cipher other than "none"'));
                }
                break;
            case 'bcrypt':
                var salt = kdfOptsBuf.readBuffer();
                var rounds = kdfOptsBuf.readInt();
                var cinf = utils.opensshCipherInfo(cipher);
                if (bcrypt === undefined) {
                    bcrypt = require(884) /* bcrypt-pbkdf */;
                }
                if (typeof (options.passphrase) === 'string') {
                    options.passphrase = Buffer.from(options.passphrase, 'utf-8');
                }
                if (!Buffer.isBuffer(options.passphrase)) {
                    throw (new errors.KeyEncryptedError(options.filename, 'OpenSSH'));
                }
                var pass = new Uint8Array(options.passphrase);
                var salti = new Uint8Array(salt);
                /* Use the pbkdf to derive both the key and the IV. */
                var out = new Uint8Array(cinf.keySize + cinf.blockSize);
                var res = bcrypt.pbkdf(pass, pass.length, salti, salti.length, out, out.length, rounds);
                if (res !== 0) {
                    throw (new Error('bcrypt_pbkdf function returned ' +
                        'failure, parameters invalid'));
                }
                out = Buffer.from(out);
                var ckey = out.slice(0, cinf.keySize);
                var iv = out.slice(cinf.keySize, cinf.keySize + cinf.blockSize);
                var cipherStream = crypto.createDecipheriv(cinf.opensslName, ckey, iv);
                cipherStream.setAutoPadding(false);
                var chunk, chunks = [];
                cipherStream.once('error', function (e) {
                    if (e.toString().indexOf('bad decrypt') !== -1) {
                        throw (new Error('Incorrect passphrase ' +
                            'supplied, could not decrypt key'));
                    }
                    throw (e);
                });
                cipherStream.write(privKeyBlob);
                cipherStream.end();
                while ((chunk = cipherStream.read()) !== null)
                    chunks.push(chunk);
                privKeyBlob = Buffer.concat(chunks);
                break;
            default:
                throw (new Error('OpenSSH-format key uses unknown KDF "' + kdf + '"'));
        }
        buf = new SSHBuffer({ buffer: privKeyBlob });
        var checkInt1 = buf.readInt();
        var checkInt2 = buf.readInt();
        if (checkInt1 !== checkInt2) {
            throw (new Error('Incorrect passphrase supplied, could not ' +
                'decrypt key'));
        }
        var ret = {};
        var key = rfc4253.readInternal(ret, 'private', buf.remainder());
        buf.skip(ret.consumed);
        var comment = buf.readString();
        key.comment = comment;
        return (key);
    }
    function write(key, options) {
        var pubKey;
        if (PrivateKey.isPrivateKey(key))
            pubKey = key.toPublic();
        else
            pubKey = key;
        var cipher = 'none';
        var kdf = 'none';
        var kdfopts = Buffer.alloc(0);
        var cinf = { blockSize: 8 };
        var passphrase;
        if (options !== undefined) {
            passphrase = options.passphrase;
            if (typeof (passphrase) === 'string')
                passphrase = Buffer.from(passphrase, 'utf-8');
            if (passphrase !== undefined) {
                assert.buffer(passphrase, 'options.passphrase');
                assert.optionalString(options.cipher, 'options.cipher');
                cipher = options.cipher;
                if (cipher === undefined)
                    cipher = 'aes128-ctr';
                cinf = utils.opensshCipherInfo(cipher);
                kdf = 'bcrypt';
            }
        }
        var privBuf;
        if (PrivateKey.isPrivateKey(key)) {
            privBuf = new SSHBuffer({});
            var checkInt = crypto.randomBytes(4).readUInt32BE(0);
            privBuf.writeInt(checkInt);
            privBuf.writeInt(checkInt);
            privBuf.write(key.toBuffer('rfc4253'));
            privBuf.writeString(key.comment || '');
            var n = 1;
            while (privBuf._offset % cinf.blockSize !== 0)
                privBuf.writeChar(n++);
            privBuf = privBuf.toBuffer();
        }
        switch (kdf) {
            case 'none':
                break;
            case 'bcrypt':
                var salt = crypto.randomBytes(16);
                var rounds = 16;
                var kdfssh = new SSHBuffer({});
                kdfssh.writeBuffer(salt);
                kdfssh.writeInt(rounds);
                kdfopts = kdfssh.toBuffer();
                if (bcrypt === undefined) {
                    bcrypt = require(884) /* bcrypt-pbkdf */;
                }
                var pass = new Uint8Array(passphrase);
                var salti = new Uint8Array(salt);
                /* Use the pbkdf to derive both the key and the IV. */
                var out = new Uint8Array(cinf.keySize + cinf.blockSize);
                var res = bcrypt.pbkdf(pass, pass.length, salti, salti.length, out, out.length, rounds);
                if (res !== 0) {
                    throw (new Error('bcrypt_pbkdf function returned ' +
                        'failure, parameters invalid'));
                }
                out = Buffer.from(out);
                var ckey = out.slice(0, cinf.keySize);
                var iv = out.slice(cinf.keySize, cinf.keySize + cinf.blockSize);
                var cipherStream = crypto.createCipheriv(cinf.opensslName, ckey, iv);
                cipherStream.setAutoPadding(false);
                var chunk, chunks = [];
                cipherStream.once('error', function (e) {
                    throw (e);
                });
                cipherStream.write(privBuf);
                cipherStream.end();
                while ((chunk = cipherStream.read()) !== null)
                    chunks.push(chunk);
                privBuf = Buffer.concat(chunks);
                break;
            default:
                throw (new Error('Unsupported kdf ' + kdf));
        }
        var buf = new SSHBuffer({});
        buf.writeCString(MAGIC);
        buf.writeString(cipher); /* cipher */
        buf.writeString(kdf); /* kdf */
        buf.writeBuffer(kdfopts); /* kdfoptions */
        buf.writeInt(1); /* nkeys */
        buf.writeBuffer(pubKey.toBuffer('rfc4253'));
        if (privBuf)
            buf.writeBuffer(privBuf);
        buf = buf.toBuffer();
        var header;
        if (PrivateKey.isPrivateKey(key))
            header = 'OPENSSH PRIVATE KEY';
        else
            header = 'OPENSSH PUBLIC KEY';
        var tmp = buf.toString('base64');
        var len = tmp.length + (tmp.length / 70) +
            18 + 16 + header.length * 2 + 10;
        buf = Buffer.alloc(len);
        var o = 0;
        o += buf.write('-----BEGIN ' + header + '-----\n', o);
        for (var i = 0; i < tmp.length;) {
            var limit = i + 70;
            if (limit > tmp.length)
                limit = tmp.length;
            o += buf.write(tmp.slice(i, limit), o);
            buf[o++] = 10;
            i = limit;
        }
        o += buf.write('-----END ' + header + '-----\n', o);
        return (buf.slice(0, o));
    }
},
/* sshpk/lib/formats/rfc4253.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    module.exports = {
        read: read.bind(undefined, false, undefined),
        readType: read.bind(undefined, false),
        write: write,
        /* semi-private api, used by sshpk-agent */
        readPartial: read.bind(undefined, true),
        /* shared with ssh format */
        readInternal: read,
        keyTypeToAlg: keyTypeToAlg,
        algToKeyType: algToKeyType
    };
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ../algs */;
    var utils = require(863) /* ../utils */;
    var Key = require(856) /* ../key */;
    var PrivateKey = require(861) /* ../private-key */;
    var SSHBuffer = require(873) /* ../ssh-buffer */;
    function algToKeyType(alg) {
        assert.string(alg);
        if (alg === 'ssh-dss')
            return ('dsa');
        else if (alg === 'ssh-rsa')
            return ('rsa');
        else if (alg === 'ssh-ed25519')
            return ('ed25519');
        else if (alg === 'ssh-curve25519')
            return ('curve25519');
        else if (alg.match(/^ecdsa-sha2-/))
            return ('ecdsa');
        else
            throw (new Error('Unknown algorithm ' + alg));
    }
    function keyTypeToAlg(key) {
        assert.object(key);
        if (key.type === 'dsa')
            return ('ssh-dss');
        else if (key.type === 'rsa')
            return ('ssh-rsa');
        else if (key.type === 'ed25519')
            return ('ssh-ed25519');
        else if (key.type === 'curve25519')
            return ('ssh-curve25519');
        else if (key.type === 'ecdsa')
            return ('ecdsa-sha2-' + key.part.curve.data.toString());
        else
            throw (new Error('Unknown key type ' + key.type));
    }
    function read(partial, type, buf, options) {
        if (typeof (buf) === 'string')
            buf = Buffer.from(buf);
        assert.buffer(buf, 'buf');
        var key = {};
        var parts = key.parts = [];
        var sshbuf = new SSHBuffer({ buffer: buf });
        var alg = sshbuf.readString();
        assert.ok(!sshbuf.atEnd(), 'key must have at least one part');
        key.type = algToKeyType(alg);
        var partCount = algs.info[key.type].parts.length;
        if (type && type === 'private')
            partCount = algs.privInfo[key.type].parts.length;
        while (!sshbuf.atEnd() && parts.length < partCount)
            parts.push(sshbuf.readPart());
        while (!partial && !sshbuf.atEnd())
            parts.push(sshbuf.readPart());
        assert.ok(parts.length >= 1, 'key must have at least one part');
        assert.ok(partial || sshbuf.atEnd(), 'leftover bytes at end of key');
        var Constructor = Key;
        var algInfo = algs.info[key.type];
        if (type === 'private' || algInfo.parts.length !== parts.length) {
            algInfo = algs.privInfo[key.type];
            Constructor = PrivateKey;
        }
        assert.strictEqual(algInfo.parts.length, parts.length);
        if (key.type === 'ecdsa') {
            var res = /^ecdsa-sha2-(.+)$/.exec(alg);
            assert.ok(res !== null);
            assert.strictEqual(res[1], parts[0].data.toString());
        }
        var normalized = true;
        for (var i = 0; i < algInfo.parts.length; ++i) {
            var p = parts[i];
            p.name = algInfo.parts[i];
            /*
             * OpenSSH stores ed25519 "private" keys as seed + public key
             * concat'd together (k followed by A). We want to keep them
             * separate for other formats that don't do this.
             */
            if (key.type === 'ed25519' && p.name === 'k')
                p.data = p.data.slice(0, 32);
            if (p.name !== 'curve' && algInfo.normalize !== false) {
                var nd;
                if (key.type === 'ed25519') {
                    nd = utils.zeroPadToLength(p.data, 32);
                }
                else {
                    nd = utils.mpNormalize(p.data);
                }
                if (nd.toString('binary') !==
                    p.data.toString('binary')) {
                    p.data = nd;
                    normalized = false;
                }
            }
        }
        if (normalized)
            key._rfc4253Cache = sshbuf.toBuffer();
        if (partial && typeof (partial) === 'object') {
            partial.remainder = sshbuf.remainder();
            partial.consumed = sshbuf._offset;
        }
        return (new Constructor(key));
    }
    function write(key, options) {
        assert.object(key);
        var alg = keyTypeToAlg(key);
        var i;
        var algInfo = algs.info[key.type];
        if (PrivateKey.isPrivateKey(key))
            algInfo = algs.privInfo[key.type];
        var parts = algInfo.parts;
        var buf = new SSHBuffer({});
        buf.writeString(alg);
        for (i = 0; i < parts.length; ++i) {
            var data = key.part[parts[i]].data;
            if (algInfo.normalize !== false) {
                if (key.type === 'ed25519')
                    data = utils.zeroPadToLength(data, 32);
                else
                    data = utils.mpNormalize(data);
            }
            if (key.type === 'ed25519' && parts[i] === 'k')
                data = Buffer.concat([data, key.part.A.data]);
            buf.writeBuffer(data);
        }
        return (buf.toBuffer());
    }
},
/* bcrypt-pbkdf/index.js */ function _(require, module, exports) {
    var crypto_hash_sha512 = require(872) /* tweetnacl */.lowlevel.crypto_hash;
    /*
     * This file is a 1:1 port from the OpenBSD blowfish.c and bcrypt_pbkdf.c. As a
     * result, it retains the original copyright and license. The two files are
     * under slightly different (but compatible) licenses, and are here combined in
     * one file.
     *
     * Credit for the actual porting work goes to:
     *  Devi Mandiri <me@devi.web.id>
     */
    /*
     * The Blowfish portions are under the following license:
     *
     * Blowfish block cipher for OpenBSD
     * Copyright 1997 Niels Provos <provos@physnet.uni-hamburg.de>
     * All rights reserved.
     *
     * Implementation advice by David Mazieres <dm@lcs.mit.edu>.
     *
     * Redistribution and use in source and binary forms, with or without
     * modification, are permitted provided that the following conditions
     * are met:
     * 1. Redistributions of source code must retain the above copyright
     *    notice, this list of conditions and the following disclaimer.
     * 2. Redistributions in binary form must reproduce the above copyright
     *    notice, this list of conditions and the following disclaimer in the
     *    documentation and/or other materials provided with the distribution.
     * 3. The name of the author may not be used to endorse or promote products
     *    derived from this software without specific prior written permission.
     *
     * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
     * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
     * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
     * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
     * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
     * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
     * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
     * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
     * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
     * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
     */
    /*
     * The bcrypt_pbkdf portions are under the following license:
     *
     * Copyright (c) 2013 Ted Unangst <tedu@openbsd.org>
     *
     * Permission to use, copy, modify, and distribute this software for any
     * purpose with or without fee is hereby granted, provided that the above
     * copyright notice and this permission notice appear in all copies.
     *
     * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
     * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
     * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
     * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
     * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
     * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
     * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
     */
    /*
     * Performance improvements (Javascript-specific):
     *
     * Copyright 2016, Joyent Inc
     * Author: Alex Wilson <alex.wilson@joyent.com>
     *
     * Permission to use, copy, modify, and distribute this software for any
     * purpose with or without fee is hereby granted, provided that the above
     * copyright notice and this permission notice appear in all copies.
     *
     * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
     * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
     * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
     * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
     * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
     * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
     * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
     */
    // Ported from OpenBSD bcrypt_pbkdf.c v1.9
    var BLF_J = 0;
    var Blowfish = function () {
        this.S = [
            new Uint32Array([
                0xd1310ba6, 0x98dfb5ac, 0x2ffd72db, 0xd01adfb7,
                0xb8e1afed, 0x6a267e96, 0xba7c9045, 0xf12c7f99,
                0x24a19947, 0xb3916cf7, 0x0801f2e2, 0x858efc16,
                0x636920d8, 0x71574e69, 0xa458fea3, 0xf4933d7e,
                0x0d95748f, 0x728eb658, 0x718bcd58, 0x82154aee,
                0x7b54a41d, 0xc25a59b5, 0x9c30d539, 0x2af26013,
                0xc5d1b023, 0x286085f0, 0xca417918, 0xb8db38ef,
                0x8e79dcb0, 0x603a180e, 0x6c9e0e8b, 0xb01e8a3e,
                0xd71577c1, 0xbd314b27, 0x78af2fda, 0x55605c60,
                0xe65525f3, 0xaa55ab94, 0x57489862, 0x63e81440,
                0x55ca396a, 0x2aab10b6, 0xb4cc5c34, 0x1141e8ce,
                0xa15486af, 0x7c72e993, 0xb3ee1411, 0x636fbc2a,
                0x2ba9c55d, 0x741831f6, 0xce5c3e16, 0x9b87931e,
                0xafd6ba33, 0x6c24cf5c, 0x7a325381, 0x28958677,
                0x3b8f4898, 0x6b4bb9af, 0xc4bfe81b, 0x66282193,
                0x61d809cc, 0xfb21a991, 0x487cac60, 0x5dec8032,
                0xef845d5d, 0xe98575b1, 0xdc262302, 0xeb651b88,
                0x23893e81, 0xd396acc5, 0x0f6d6ff3, 0x83f44239,
                0x2e0b4482, 0xa4842004, 0x69c8f04a, 0x9e1f9b5e,
                0x21c66842, 0xf6e96c9a, 0x670c9c61, 0xabd388f0,
                0x6a51a0d2, 0xd8542f68, 0x960fa728, 0xab5133a3,
                0x6eef0b6c, 0x137a3be4, 0xba3bf050, 0x7efb2a98,
                0xa1f1651d, 0x39af0176, 0x66ca593e, 0x82430e88,
                0x8cee8619, 0x456f9fb4, 0x7d84a5c3, 0x3b8b5ebe,
                0xe06f75d8, 0x85c12073, 0x401a449f, 0x56c16aa6,
                0x4ed3aa62, 0x363f7706, 0x1bfedf72, 0x429b023d,
                0x37d0d724, 0xd00a1248, 0xdb0fead3, 0x49f1c09b,
                0x075372c9, 0x80991b7b, 0x25d479d8, 0xf6e8def7,
                0xe3fe501a, 0xb6794c3b, 0x976ce0bd, 0x04c006ba,
                0xc1a94fb6, 0x409f60c4, 0x5e5c9ec2, 0x196a2463,
                0x68fb6faf, 0x3e6c53b5, 0x1339b2eb, 0x3b52ec6f,
                0x6dfc511f, 0x9b30952c, 0xcc814544, 0xaf5ebd09,
                0xbee3d004, 0xde334afd, 0x660f2807, 0x192e4bb3,
                0xc0cba857, 0x45c8740f, 0xd20b5f39, 0xb9d3fbdb,
                0x5579c0bd, 0x1a60320a, 0xd6a100c6, 0x402c7279,
                0x679f25fe, 0xfb1fa3cc, 0x8ea5e9f8, 0xdb3222f8,
                0x3c7516df, 0xfd616b15, 0x2f501ec8, 0xad0552ab,
                0x323db5fa, 0xfd238760, 0x53317b48, 0x3e00df82,
                0x9e5c57bb, 0xca6f8ca0, 0x1a87562e, 0xdf1769db,
                0xd542a8f6, 0x287effc3, 0xac6732c6, 0x8c4f5573,
                0x695b27b0, 0xbbca58c8, 0xe1ffa35d, 0xb8f011a0,
                0x10fa3d98, 0xfd2183b8, 0x4afcb56c, 0x2dd1d35b,
                0x9a53e479, 0xb6f84565, 0xd28e49bc, 0x4bfb9790,
                0xe1ddf2da, 0xa4cb7e33, 0x62fb1341, 0xcee4c6e8,
                0xef20cada, 0x36774c01, 0xd07e9efe, 0x2bf11fb4,
                0x95dbda4d, 0xae909198, 0xeaad8e71, 0x6b93d5a0,
                0xd08ed1d0, 0xafc725e0, 0x8e3c5b2f, 0x8e7594b7,
                0x8ff6e2fb, 0xf2122b64, 0x8888b812, 0x900df01c,
                0x4fad5ea0, 0x688fc31c, 0xd1cff191, 0xb3a8c1ad,
                0x2f2f2218, 0xbe0e1777, 0xea752dfe, 0x8b021fa1,
                0xe5a0cc0f, 0xb56f74e8, 0x18acf3d6, 0xce89e299,
                0xb4a84fe0, 0xfd13e0b7, 0x7cc43b81, 0xd2ada8d9,
                0x165fa266, 0x80957705, 0x93cc7314, 0x211a1477,
                0xe6ad2065, 0x77b5fa86, 0xc75442f5, 0xfb9d35cf,
                0xebcdaf0c, 0x7b3e89a0, 0xd6411bd3, 0xae1e7e49,
                0x00250e2d, 0x2071b35e, 0x226800bb, 0x57b8e0af,
                0x2464369b, 0xf009b91e, 0x5563911d, 0x59dfa6aa,
                0x78c14389, 0xd95a537f, 0x207d5ba2, 0x02e5b9c5,
                0x83260376, 0x6295cfa9, 0x11c81968, 0x4e734a41,
                0xb3472dca, 0x7b14a94a, 0x1b510052, 0x9a532915,
                0xd60f573f, 0xbc9bc6e4, 0x2b60a476, 0x81e67400,
                0x08ba6fb5, 0x571be91f, 0xf296ec6b, 0x2a0dd915,
                0xb6636521, 0xe7b9f9b6, 0xff34052e, 0xc5855664,
                0x53b02d5d, 0xa99f8fa1, 0x08ba4799, 0x6e85076a
            ]),
            new Uint32Array([
                0x4b7a70e9, 0xb5b32944, 0xdb75092e, 0xc4192623,
                0xad6ea6b0, 0x49a7df7d, 0x9cee60b8, 0x8fedb266,
                0xecaa8c71, 0x699a17ff, 0x5664526c, 0xc2b19ee1,
                0x193602a5, 0x75094c29, 0xa0591340, 0xe4183a3e,
                0x3f54989a, 0x5b429d65, 0x6b8fe4d6, 0x99f73fd6,
                0xa1d29c07, 0xefe830f5, 0x4d2d38e6, 0xf0255dc1,
                0x4cdd2086, 0x8470eb26, 0x6382e9c6, 0x021ecc5e,
                0x09686b3f, 0x3ebaefc9, 0x3c971814, 0x6b6a70a1,
                0x687f3584, 0x52a0e286, 0xb79c5305, 0xaa500737,
                0x3e07841c, 0x7fdeae5c, 0x8e7d44ec, 0x5716f2b8,
                0xb03ada37, 0xf0500c0d, 0xf01c1f04, 0x0200b3ff,
                0xae0cf51a, 0x3cb574b2, 0x25837a58, 0xdc0921bd,
                0xd19113f9, 0x7ca92ff6, 0x94324773, 0x22f54701,
                0x3ae5e581, 0x37c2dadc, 0xc8b57634, 0x9af3dda7,
                0xa9446146, 0x0fd0030e, 0xecc8c73e, 0xa4751e41,
                0xe238cd99, 0x3bea0e2f, 0x3280bba1, 0x183eb331,
                0x4e548b38, 0x4f6db908, 0x6f420d03, 0xf60a04bf,
                0x2cb81290, 0x24977c79, 0x5679b072, 0xbcaf89af,
                0xde9a771f, 0xd9930810, 0xb38bae12, 0xdccf3f2e,
                0x5512721f, 0x2e6b7124, 0x501adde6, 0x9f84cd87,
                0x7a584718, 0x7408da17, 0xbc9f9abc, 0xe94b7d8c,
                0xec7aec3a, 0xdb851dfa, 0x63094366, 0xc464c3d2,
                0xef1c1847, 0x3215d908, 0xdd433b37, 0x24c2ba16,
                0x12a14d43, 0x2a65c451, 0x50940002, 0x133ae4dd,
                0x71dff89e, 0x10314e55, 0x81ac77d6, 0x5f11199b,
                0x043556f1, 0xd7a3c76b, 0x3c11183b, 0x5924a509,
                0xf28fe6ed, 0x97f1fbfa, 0x9ebabf2c, 0x1e153c6e,
                0x86e34570, 0xeae96fb1, 0x860e5e0a, 0x5a3e2ab3,
                0x771fe71c, 0x4e3d06fa, 0x2965dcb9, 0x99e71d0f,
                0x803e89d6, 0x5266c825, 0x2e4cc978, 0x9c10b36a,
                0xc6150eba, 0x94e2ea78, 0xa5fc3c53, 0x1e0a2df4,
                0xf2f74ea7, 0x361d2b3d, 0x1939260f, 0x19c27960,
                0x5223a708, 0xf71312b6, 0xebadfe6e, 0xeac31f66,
                0xe3bc4595, 0xa67bc883, 0xb17f37d1, 0x018cff28,
                0xc332ddef, 0xbe6c5aa5, 0x65582185, 0x68ab9802,
                0xeecea50f, 0xdb2f953b, 0x2aef7dad, 0x5b6e2f84,
                0x1521b628, 0x29076170, 0xecdd4775, 0x619f1510,
                0x13cca830, 0xeb61bd96, 0x0334fe1e, 0xaa0363cf,
                0xb5735c90, 0x4c70a239, 0xd59e9e0b, 0xcbaade14,
                0xeecc86bc, 0x60622ca7, 0x9cab5cab, 0xb2f3846e,
                0x648b1eaf, 0x19bdf0ca, 0xa02369b9, 0x655abb50,
                0x40685a32, 0x3c2ab4b3, 0x319ee9d5, 0xc021b8f7,
                0x9b540b19, 0x875fa099, 0x95f7997e, 0x623d7da8,
                0xf837889a, 0x97e32d77, 0x11ed935f, 0x16681281,
                0x0e358829, 0xc7e61fd6, 0x96dedfa1, 0x7858ba99,
                0x57f584a5, 0x1b227263, 0x9b83c3ff, 0x1ac24696,
                0xcdb30aeb, 0x532e3054, 0x8fd948e4, 0x6dbc3128,
                0x58ebf2ef, 0x34c6ffea, 0xfe28ed61, 0xee7c3c73,
                0x5d4a14d9, 0xe864b7e3, 0x42105d14, 0x203e13e0,
                0x45eee2b6, 0xa3aaabea, 0xdb6c4f15, 0xfacb4fd0,
                0xc742f442, 0xef6abbb5, 0x654f3b1d, 0x41cd2105,
                0xd81e799e, 0x86854dc7, 0xe44b476a, 0x3d816250,
                0xcf62a1f2, 0x5b8d2646, 0xfc8883a0, 0xc1c7b6a3,
                0x7f1524c3, 0x69cb7492, 0x47848a0b, 0x5692b285,
                0x095bbf00, 0xad19489d, 0x1462b174, 0x23820e00,
                0x58428d2a, 0x0c55f5ea, 0x1dadf43e, 0x233f7061,
                0x3372f092, 0x8d937e41, 0xd65fecf1, 0x6c223bdb,
                0x7cde3759, 0xcbee7460, 0x4085f2a7, 0xce77326e,
                0xa6078084, 0x19f8509e, 0xe8efd855, 0x61d99735,
                0xa969a7aa, 0xc50c06c2, 0x5a04abfc, 0x800bcadc,
                0x9e447a2e, 0xc3453484, 0xfdd56705, 0x0e1e9ec9,
                0xdb73dbd3, 0x105588cd, 0x675fda79, 0xe3674340,
                0xc5c43465, 0x713e38d8, 0x3d28f89e, 0xf16dff20,
                0x153e21e7, 0x8fb03d4a, 0xe6e39f2b, 0xdb83adf7
            ]),
            new Uint32Array([
                0xe93d5a68, 0x948140f7, 0xf64c261c, 0x94692934,
                0x411520f7, 0x7602d4f7, 0xbcf46b2e, 0xd4a20068,
                0xd4082471, 0x3320f46a, 0x43b7d4b7, 0x500061af,
                0x1e39f62e, 0x97244546, 0x14214f74, 0xbf8b8840,
                0x4d95fc1d, 0x96b591af, 0x70f4ddd3, 0x66a02f45,
                0xbfbc09ec, 0x03bd9785, 0x7fac6dd0, 0x31cb8504,
                0x96eb27b3, 0x55fd3941, 0xda2547e6, 0xabca0a9a,
                0x28507825, 0x530429f4, 0x0a2c86da, 0xe9b66dfb,
                0x68dc1462, 0xd7486900, 0x680ec0a4, 0x27a18dee,
                0x4f3ffea2, 0xe887ad8c, 0xb58ce006, 0x7af4d6b6,
                0xaace1e7c, 0xd3375fec, 0xce78a399, 0x406b2a42,
                0x20fe9e35, 0xd9f385b9, 0xee39d7ab, 0x3b124e8b,
                0x1dc9faf7, 0x4b6d1856, 0x26a36631, 0xeae397b2,
                0x3a6efa74, 0xdd5b4332, 0x6841e7f7, 0xca7820fb,
                0xfb0af54e, 0xd8feb397, 0x454056ac, 0xba489527,
                0x55533a3a, 0x20838d87, 0xfe6ba9b7, 0xd096954b,
                0x55a867bc, 0xa1159a58, 0xcca92963, 0x99e1db33,
                0xa62a4a56, 0x3f3125f9, 0x5ef47e1c, 0x9029317c,
                0xfdf8e802, 0x04272f70, 0x80bb155c, 0x05282ce3,
                0x95c11548, 0xe4c66d22, 0x48c1133f, 0xc70f86dc,
                0x07f9c9ee, 0x41041f0f, 0x404779a4, 0x5d886e17,
                0x325f51eb, 0xd59bc0d1, 0xf2bcc18f, 0x41113564,
                0x257b7834, 0x602a9c60, 0xdff8e8a3, 0x1f636c1b,
                0x0e12b4c2, 0x02e1329e, 0xaf664fd1, 0xcad18115,
                0x6b2395e0, 0x333e92e1, 0x3b240b62, 0xeebeb922,
                0x85b2a20e, 0xe6ba0d99, 0xde720c8c, 0x2da2f728,
                0xd0127845, 0x95b794fd, 0x647d0862, 0xe7ccf5f0,
                0x5449a36f, 0x877d48fa, 0xc39dfd27, 0xf33e8d1e,
                0x0a476341, 0x992eff74, 0x3a6f6eab, 0xf4f8fd37,
                0xa812dc60, 0xa1ebddf8, 0x991be14c, 0xdb6e6b0d,
                0xc67b5510, 0x6d672c37, 0x2765d43b, 0xdcd0e804,
                0xf1290dc7, 0xcc00ffa3, 0xb5390f92, 0x690fed0b,
                0x667b9ffb, 0xcedb7d9c, 0xa091cf0b, 0xd9155ea3,
                0xbb132f88, 0x515bad24, 0x7b9479bf, 0x763bd6eb,
                0x37392eb3, 0xcc115979, 0x8026e297, 0xf42e312d,
                0x6842ada7, 0xc66a2b3b, 0x12754ccc, 0x782ef11c,
                0x6a124237, 0xb79251e7, 0x06a1bbe6, 0x4bfb6350,
                0x1a6b1018, 0x11caedfa, 0x3d25bdd8, 0xe2e1c3c9,
                0x44421659, 0x0a121386, 0xd90cec6e, 0xd5abea2a,
                0x64af674e, 0xda86a85f, 0xbebfe988, 0x64e4c3fe,
                0x9dbc8057, 0xf0f7c086, 0x60787bf8, 0x6003604d,
                0xd1fd8346, 0xf6381fb0, 0x7745ae04, 0xd736fccc,
                0x83426b33, 0xf01eab71, 0xb0804187, 0x3c005e5f,
                0x77a057be, 0xbde8ae24, 0x55464299, 0xbf582e61,
                0x4e58f48f, 0xf2ddfda2, 0xf474ef38, 0x8789bdc2,
                0x5366f9c3, 0xc8b38e74, 0xb475f255, 0x46fcd9b9,
                0x7aeb2661, 0x8b1ddf84, 0x846a0e79, 0x915f95e2,
                0x466e598e, 0x20b45770, 0x8cd55591, 0xc902de4c,
                0xb90bace1, 0xbb8205d0, 0x11a86248, 0x7574a99e,
                0xb77f19b6, 0xe0a9dc09, 0x662d09a1, 0xc4324633,
                0xe85a1f02, 0x09f0be8c, 0x4a99a025, 0x1d6efe10,
                0x1ab93d1d, 0x0ba5a4df, 0xa186f20f, 0x2868f169,
                0xdcb7da83, 0x573906fe, 0xa1e2ce9b, 0x4fcd7f52,
                0x50115e01, 0xa70683fa, 0xa002b5c4, 0x0de6d027,
                0x9af88c27, 0x773f8641, 0xc3604c06, 0x61a806b5,
                0xf0177a28, 0xc0f586e0, 0x006058aa, 0x30dc7d62,
                0x11e69ed7, 0x2338ea63, 0x53c2dd94, 0xc2c21634,
                0xbbcbee56, 0x90bcb6de, 0xebfc7da1, 0xce591d76,
                0x6f05e409, 0x4b7c0188, 0x39720a3d, 0x7c927c24,
                0x86e3725f, 0x724d9db9, 0x1ac15bb4, 0xd39eb8fc,
                0xed545578, 0x08fca5b5, 0xd83d7cd3, 0x4dad0fc4,
                0x1e50ef5e, 0xb161e6f8, 0xa28514d9, 0x6c51133c,
                0x6fd5c7e7, 0x56e14ec4, 0x362abfce, 0xddc6c837,
                0xd79a3234, 0x92638212, 0x670efa8e, 0x406000e0
            ]),
            new Uint32Array([
                0x3a39ce37, 0xd3faf5cf, 0xabc27737, 0x5ac52d1b,
                0x5cb0679e, 0x4fa33742, 0xd3822740, 0x99bc9bbe,
                0xd5118e9d, 0xbf0f7315, 0xd62d1c7e, 0xc700c47b,
                0xb78c1b6b, 0x21a19045, 0xb26eb1be, 0x6a366eb4,
                0x5748ab2f, 0xbc946e79, 0xc6a376d2, 0x6549c2c8,
                0x530ff8ee, 0x468dde7d, 0xd5730a1d, 0x4cd04dc6,
                0x2939bbdb, 0xa9ba4650, 0xac9526e8, 0xbe5ee304,
                0xa1fad5f0, 0x6a2d519a, 0x63ef8ce2, 0x9a86ee22,
                0xc089c2b8, 0x43242ef6, 0xa51e03aa, 0x9cf2d0a4,
                0x83c061ba, 0x9be96a4d, 0x8fe51550, 0xba645bd6,
                0x2826a2f9, 0xa73a3ae1, 0x4ba99586, 0xef5562e9,
                0xc72fefd3, 0xf752f7da, 0x3f046f69, 0x77fa0a59,
                0x80e4a915, 0x87b08601, 0x9b09e6ad, 0x3b3ee593,
                0xe990fd5a, 0x9e34d797, 0x2cf0b7d9, 0x022b8b51,
                0x96d5ac3a, 0x017da67d, 0xd1cf3ed6, 0x7c7d2d28,
                0x1f9f25cf, 0xadf2b89b, 0x5ad6b472, 0x5a88f54c,
                0xe029ac71, 0xe019a5e6, 0x47b0acfd, 0xed93fa9b,
                0xe8d3c48d, 0x283b57cc, 0xf8d56629, 0x79132e28,
                0x785f0191, 0xed756055, 0xf7960e44, 0xe3d35e8c,
                0x15056dd4, 0x88f46dba, 0x03a16125, 0x0564f0bd,
                0xc3eb9e15, 0x3c9057a2, 0x97271aec, 0xa93a072a,
                0x1b3f6d9b, 0x1e6321f5, 0xf59c66fb, 0x26dcf319,
                0x7533d928, 0xb155fdf5, 0x03563482, 0x8aba3cbb,
                0x28517711, 0xc20ad9f8, 0xabcc5167, 0xccad925f,
                0x4de81751, 0x3830dc8e, 0x379d5862, 0x9320f991,
                0xea7a90c2, 0xfb3e7bce, 0x5121ce64, 0x774fbe32,
                0xa8b6e37e, 0xc3293d46, 0x48de5369, 0x6413e680,
                0xa2ae0810, 0xdd6db224, 0x69852dfd, 0x09072166,
                0xb39a460a, 0x6445c0dd, 0x586cdecf, 0x1c20c8ae,
                0x5bbef7dd, 0x1b588d40, 0xccd2017f, 0x6bb4e3bb,
                0xdda26a7e, 0x3a59ff45, 0x3e350a44, 0xbcb4cdd5,
                0x72eacea8, 0xfa6484bb, 0x8d6612ae, 0xbf3c6f47,
                0xd29be463, 0x542f5d9e, 0xaec2771b, 0xf64e6370,
                0x740e0d8d, 0xe75b1357, 0xf8721671, 0xaf537d5d,
                0x4040cb08, 0x4eb4e2cc, 0x34d2466a, 0x0115af84,
                0xe1b00428, 0x95983a1d, 0x06b89fb4, 0xce6ea048,
                0x6f3f3b82, 0x3520ab82, 0x011a1d4b, 0x277227f8,
                0x611560b1, 0xe7933fdc, 0xbb3a792b, 0x344525bd,
                0xa08839e1, 0x51ce794b, 0x2f32c9b7, 0xa01fbac9,
                0xe01cc87e, 0xbcc7d1f6, 0xcf0111c3, 0xa1e8aac7,
                0x1a908749, 0xd44fbd9a, 0xd0dadecb, 0xd50ada38,
                0x0339c32a, 0xc6913667, 0x8df9317c, 0xe0b12b4f,
                0xf79e59b7, 0x43f5bb3a, 0xf2d519ff, 0x27d9459c,
                0xbf97222c, 0x15e6fc2a, 0x0f91fc71, 0x9b941525,
                0xfae59361, 0xceb69ceb, 0xc2a86459, 0x12baa8d1,
                0xb6c1075e, 0xe3056a0c, 0x10d25065, 0xcb03a442,
                0xe0ec6e0e, 0x1698db3b, 0x4c98a0be, 0x3278e964,
                0x9f1f9532, 0xe0d392df, 0xd3a0342b, 0x8971f21e,
                0x1b0a7441, 0x4ba3348c, 0xc5be7120, 0xc37632d8,
                0xdf359f8d, 0x9b992f2e, 0xe60b6f47, 0x0fe3f11d,
                0xe54cda54, 0x1edad891, 0xce6279cf, 0xcd3e7e6f,
                0x1618b166, 0xfd2c1d05, 0x848fd2c5, 0xf6fb2299,
                0xf523f357, 0xa6327623, 0x93a83531, 0x56cccd02,
                0xacf08162, 0x5a75ebb5, 0x6e163697, 0x88d273cc,
                0xde966292, 0x81b949d0, 0x4c50901b, 0x71c65614,
                0xe6c6c7bd, 0x327a140a, 0x45e1d006, 0xc3f27b9a,
                0xc9aa53fd, 0x62a80f00, 0xbb25bfe2, 0x35bdd2f6,
                0x71126905, 0xb2040222, 0xb6cbcf7c, 0xcd769c2b,
                0x53113ec0, 0x1640e3d3, 0x38abbd60, 0x2547adf0,
                0xba38209c, 0xf746ce76, 0x77afa1c5, 0x20756060,
                0x85cbfe4e, 0x8ae88dd8, 0x7aaaf9b0, 0x4cf9aa7e,
                0x1948c25c, 0x02fb8a8c, 0x01c36ae4, 0xd6ebe1f9,
                0x90d4f869, 0xa65cdea0, 0x3f09252d, 0xc208e69f,
                0xb74e6132, 0xce77e25b, 0x578fdfe3, 0x3ac372e6
            ])
        ];
        this.P = new Uint32Array([
            0x243f6a88, 0x85a308d3, 0x13198a2e, 0x03707344,
            0xa4093822, 0x299f31d0, 0x082efa98, 0xec4e6c89,
            0x452821e6, 0x38d01377, 0xbe5466cf, 0x34e90c6c,
            0xc0ac29b7, 0xc97c50dd, 0x3f84d5b5, 0xb5470917,
            0x9216d5d9, 0x8979fb1b
        ]);
    };
    function F(S, x8, i) {
        return (((S[0][x8[i + 3]] +
            S[1][x8[i + 2]]) ^
            S[2][x8[i + 1]]) +
            S[3][x8[i]]);
    }
    ;
    Blowfish.prototype.encipher = function (x, x8) {
        if (x8 === undefined) {
            x8 = new Uint8Array(x.buffer);
            if (x.byteOffset !== 0)
                x8 = x8.subarray(x.byteOffset);
        }
        x[0] ^= this.P[0];
        for (var i = 1; i < 16; i += 2) {
            x[1] ^= F(this.S, x8, 0) ^ this.P[i];
            x[0] ^= F(this.S, x8, 4) ^ this.P[i + 1];
        }
        var t = x[0];
        x[0] = x[1] ^ this.P[17];
        x[1] = t;
    };
    Blowfish.prototype.decipher = function (x) {
        var x8 = new Uint8Array(x.buffer);
        if (x.byteOffset !== 0)
            x8 = x8.subarray(x.byteOffset);
        x[0] ^= this.P[17];
        for (var i = 16; i > 0; i -= 2) {
            x[1] ^= F(this.S, x8, 0) ^ this.P[i];
            x[0] ^= F(this.S, x8, 4) ^ this.P[i - 1];
        }
        var t = x[0];
        x[0] = x[1] ^ this.P[0];
        x[1] = t;
    };
    function stream2word(data, databytes) {
        var i, temp = 0;
        for (i = 0; i < 4; i++, BLF_J++) {
            if (BLF_J >= databytes)
                BLF_J = 0;
            temp = (temp << 8) | data[BLF_J];
        }
        return temp;
    }
    ;
    Blowfish.prototype.expand0state = function (key, keybytes) {
        var d = new Uint32Array(2), i, k;
        var d8 = new Uint8Array(d.buffer);
        for (i = 0, BLF_J = 0; i < 18; i++) {
            this.P[i] ^= stream2word(key, keybytes);
        }
        BLF_J = 0;
        for (i = 0; i < 18; i += 2) {
            this.encipher(d, d8);
            this.P[i] = d[0];
            this.P[i + 1] = d[1];
        }
        for (i = 0; i < 4; i++) {
            for (k = 0; k < 256; k += 2) {
                this.encipher(d, d8);
                this.S[i][k] = d[0];
                this.S[i][k + 1] = d[1];
            }
        }
    };
    Blowfish.prototype.expandstate = function (data, databytes, key, keybytes) {
        var d = new Uint32Array(2), i, k;
        for (i = 0, BLF_J = 0; i < 18; i++) {
            this.P[i] ^= stream2word(key, keybytes);
        }
        for (i = 0, BLF_J = 0; i < 18; i += 2) {
            d[0] ^= stream2word(data, databytes);
            d[1] ^= stream2word(data, databytes);
            this.encipher(d);
            this.P[i] = d[0];
            this.P[i + 1] = d[1];
        }
        for (i = 0; i < 4; i++) {
            for (k = 0; k < 256; k += 2) {
                d[0] ^= stream2word(data, databytes);
                d[1] ^= stream2word(data, databytes);
                this.encipher(d);
                this.S[i][k] = d[0];
                this.S[i][k + 1] = d[1];
            }
        }
        BLF_J = 0;
    };
    Blowfish.prototype.enc = function (data, blocks) {
        for (var i = 0; i < blocks; i++) {
            this.encipher(data.subarray(i * 2));
        }
    };
    Blowfish.prototype.dec = function (data, blocks) {
        for (var i = 0; i < blocks; i++) {
            this.decipher(data.subarray(i * 2));
        }
    };
    var BCRYPT_BLOCKS = 8, BCRYPT_HASHSIZE = 32;
    function bcrypt_hash(sha2pass, sha2salt, out) {
        var state = new Blowfish(), cdata = new Uint32Array(BCRYPT_BLOCKS), i, ciphertext = new Uint8Array([79, 120, 121, 99, 104, 114, 111, 109, 97, 116, 105,
            99, 66, 108, 111, 119, 102, 105, 115, 104, 83, 119, 97, 116, 68, 121, 110, 97, 109,
            105, 116, 101]); //"OxychromaticBlowfishSwatDynamite"
        state.expandstate(sha2salt, 64, sha2pass, 64);
        for (i = 0; i < 64; i++) {
            state.expand0state(sha2salt, 64);
            state.expand0state(sha2pass, 64);
        }
        for (i = 0; i < BCRYPT_BLOCKS; i++)
            cdata[i] = stream2word(ciphertext, ciphertext.byteLength);
        for (i = 0; i < 64; i++)
            state.enc(cdata, cdata.byteLength / 8);
        for (i = 0; i < BCRYPT_BLOCKS; i++) {
            out[4 * i + 3] = cdata[i] >>> 24;
            out[4 * i + 2] = cdata[i] >>> 16;
            out[4 * i + 1] = cdata[i] >>> 8;
            out[4 * i + 0] = cdata[i];
        }
    }
    ;
    function bcrypt_pbkdf(pass, passlen, salt, saltlen, key, keylen, rounds) {
        var sha2pass = new Uint8Array(64), sha2salt = new Uint8Array(64), out = new Uint8Array(BCRYPT_HASHSIZE), tmpout = new Uint8Array(BCRYPT_HASHSIZE), countsalt = new Uint8Array(saltlen + 4), i, j, amt, stride, dest, count, origkeylen = keylen;
        if (rounds < 1)
            return -1;
        if (passlen === 0 || saltlen === 0 || keylen === 0 ||
            keylen > (out.byteLength * out.byteLength) || saltlen > (1 << 20))
            return -1;
        stride = Math.floor((keylen + out.byteLength - 1) / out.byteLength);
        amt = Math.floor((keylen + stride - 1) / stride);
        for (i = 0; i < saltlen; i++)
            countsalt[i] = salt[i];
        crypto_hash_sha512(sha2pass, pass, passlen);
        for (count = 1; keylen > 0; count++) {
            countsalt[saltlen + 0] = count >>> 24;
            countsalt[saltlen + 1] = count >>> 16;
            countsalt[saltlen + 2] = count >>> 8;
            countsalt[saltlen + 3] = count;
            crypto_hash_sha512(sha2salt, countsalt, saltlen + 4);
            bcrypt_hash(sha2pass, sha2salt, tmpout);
            for (i = out.byteLength; i--;)
                out[i] = tmpout[i];
            for (i = 1; i < rounds; i++) {
                crypto_hash_sha512(sha2salt, tmpout, tmpout.byteLength);
                bcrypt_hash(sha2pass, sha2salt, tmpout);
                for (j = 0; j < out.byteLength; j++)
                    out[j] ^= tmpout[j];
            }
            amt = Math.min(amt, keylen);
            for (i = 0; i < amt; i++) {
                dest = i * stride + (count - 1);
                if (dest >= origkeylen)
                    break;
                key[dest] = out[i];
            }
            keylen -= i;
        }
        return 0;
    }
    ;
    module.exports = {
        BLOCKS: BCRYPT_BLOCKS,
        HASHSIZE: BCRYPT_HASHSIZE,
        hash: bcrypt_hash,
        pbkdf: bcrypt_pbkdf
    };
},
/* sshpk/lib/formats/ssh.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    module.exports = {
        read: read,
        write: write
    };
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var rfc4253 = require(883) /* ./rfc4253 */;
    var utils = require(863) /* ../utils */;
    var Key = require(856) /* ../key */;
    var PrivateKey = require(861) /* ../private-key */;
    var sshpriv = require(882) /* ./ssh-private */;
    /*JSSTYLED*/
    var SSHKEY_RE = /^([a-z0-9-]+)[ \t]+([a-zA-Z0-9+\/]+[=]*)([ \t]+([^ \t][^\n]*[\n]*)?)?$/;
    /*JSSTYLED*/
    var SSHKEY_RE2 = /^([a-z0-9-]+)[ \t\n]+([a-zA-Z0-9+\/][a-zA-Z0-9+\/ \t\n=]*)([^a-zA-Z0-9+\/ \t\n=].*)?$/;
    function read(buf, options) {
        if (typeof (buf) !== 'string') {
            assert.buffer(buf, 'buf');
            buf = buf.toString('ascii');
        }
        var trimmed = buf.trim().replace(/[\\\r]/g, '');
        var m = trimmed.match(SSHKEY_RE);
        if (!m)
            m = trimmed.match(SSHKEY_RE2);
        assert.ok(m, 'key must match regex');
        var type = rfc4253.algToKeyType(m[1]);
        var kbuf = Buffer.from(m[2], 'base64');
        /*
         * This is a bit tricky. If we managed to parse the key and locate the
         * key comment with the regex, then do a non-partial read and assert
         * that we have consumed all bytes. If we couldn't locate the key
         * comment, though, there may be whitespace shenanigans going on that
         * have conjoined the comment to the rest of the key. We do a partial
         * read in this case to try to make the best out of a sorry situation.
         */
        var key;
        var ret = {};
        if (m[4]) {
            try {
                key = rfc4253.read(kbuf);
            }
            catch (e) {
                m = trimmed.match(SSHKEY_RE2);
                assert.ok(m, 'key must match regex');
                kbuf = Buffer.from(m[2], 'base64');
                key = rfc4253.readInternal(ret, 'public', kbuf);
            }
        }
        else {
            key = rfc4253.readInternal(ret, 'public', kbuf);
        }
        assert.strictEqual(type, key.type);
        if (m[4] && m[4].length > 0) {
            key.comment = m[4];
        }
        else if (ret.consumed) {
            /*
             * Now the magic: trying to recover the key comment when it's
             * gotten conjoined to the key or otherwise shenanigan'd.
             *
             * Work out how much base64 we used, then drop all non-base64
             * chars from the beginning up to this point in the the string.
             * Then offset in this and try to make up for missing = chars.
             */
            var data = m[2] + (m[3] ? m[3] : '');
            var realOffset = Math.ceil(ret.consumed / 3) * 4;
            data = data.slice(0, realOffset - 2). /*JSSTYLED*/
                replace(/[^a-zA-Z0-9+\/=]/g, '') +
                data.slice(realOffset - 2);
            var padding = ret.consumed % 3;
            if (padding > 0 &&
                data.slice(realOffset - 1, realOffset) !== '=')
                realOffset--;
            while (data.slice(realOffset, realOffset + 1) === '=')
                realOffset++;
            /* Finally, grab what we think is the comment & clean it up. */
            var trailer = data.slice(realOffset);
            trailer = trailer.replace(/[\r\n]/g, ' ').
                replace(/^\s+/, '');
            if (trailer.match(/^[a-zA-Z0-9]/))
                key.comment = trailer;
        }
        return (key);
    }
    function write(key, options) {
        assert.object(key);
        if (!Key.isKey(key))
            throw (new Error('Must be a public key'));
        var parts = [];
        var alg = rfc4253.keyTypeToAlg(key);
        parts.push(alg);
        var buf = rfc4253.write(key);
        parts.push(buf.toString('base64'));
        if (key.comment)
            parts.push(key.comment);
        return (Buffer.from(parts.join(' ')));
    }
},
/* sshpk/lib/formats/dnssec.js */ function _(require, module, exports) {
    // Copyright 2017 Joyent, Inc.
    module.exports = {
        read: read,
        write: write
    };
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var Key = require(856) /* ../key */;
    var PrivateKey = require(861) /* ../private-key */;
    var utils = require(863) /* ../utils */;
    var SSHBuffer = require(873) /* ../ssh-buffer */;
    var Dhe = require(874) /* ../dhe */;
    var supportedAlgos = {
        'rsa-sha1': 5,
        'rsa-sha256': 8,
        'rsa-sha512': 10,
        'ecdsa-p256-sha256': 13,
        'ecdsa-p384-sha384': 14
        /*
         * ed25519 is hypothetically supported with id 15
         * but the common tools available don't appear to be
         * capable of generating/using ed25519 keys
         */
    };
    var supportedAlgosById = {};
    Object.keys(supportedAlgos).forEach(function (k) {
        supportedAlgosById[supportedAlgos[k]] = k.toUpperCase();
    });
    function read(buf, options) {
        if (typeof (buf) !== 'string') {
            assert.buffer(buf, 'buf');
            buf = buf.toString('ascii');
        }
        var lines = buf.split('\n');
        if (lines[0].match(/^Private-key-format\: v1/)) {
            var algElems = lines[1].split(' ');
            var algoNum = parseInt(algElems[1], 10);
            var algoName = algElems[2];
            if (!supportedAlgosById[algoNum])
                throw (new Error('Unsupported algorithm: ' + algoName));
            return (readDNSSECPrivateKey(algoNum, lines.slice(2)));
        }
        // skip any comment-lines
        var line = 0;
        /* JSSTYLED */
        while (lines[line].match(/^\;/))
            line++;
        // we should now have *one single* line left with our KEY on it.
        if ((lines[line].match(/\. IN KEY /) ||
            lines[line].match(/\. IN DNSKEY /)) && lines[line + 1].length === 0) {
            return (readRFC3110(lines[line]));
        }
        throw (new Error('Cannot parse dnssec key'));
    }
    function readRFC3110(keyString) {
        var elems = keyString.split(' ');
        //unused var flags = parseInt(elems[3], 10);
        //unused var protocol = parseInt(elems[4], 10);
        var algorithm = parseInt(elems[5], 10);
        if (!supportedAlgosById[algorithm])
            throw (new Error('Unsupported algorithm: ' + algorithm));
        var base64key = elems.slice(6, elems.length).join();
        var keyBuffer = Buffer.from(base64key, 'base64');
        if (supportedAlgosById[algorithm].match(/^RSA-/)) {
            // join the rest of the body into a single base64-blob
            var publicExponentLen = keyBuffer.readUInt8(0);
            if (publicExponentLen != 3 && publicExponentLen != 1)
                throw (new Error('Cannot parse dnssec key: ' +
                    'unsupported exponent length'));
            var publicExponent = keyBuffer.slice(1, publicExponentLen + 1);
            publicExponent = utils.mpNormalize(publicExponent);
            var modulus = keyBuffer.slice(1 + publicExponentLen);
            modulus = utils.mpNormalize(modulus);
            // now, make the key
            var rsaKey = {
                type: 'rsa',
                parts: []
            };
            rsaKey.parts.push({ name: 'e', data: publicExponent });
            rsaKey.parts.push({ name: 'n', data: modulus });
            return (new Key(rsaKey));
        }
        if (supportedAlgosById[algorithm] === 'ECDSA-P384-SHA384' ||
            supportedAlgosById[algorithm] === 'ECDSA-P256-SHA256') {
            var curve = 'nistp384';
            var size = 384;
            if (supportedAlgosById[algorithm].match(/^ECDSA-P256-SHA256/)) {
                curve = 'nistp256';
                size = 256;
            }
            var ecdsaKey = {
                type: 'ecdsa',
                curve: curve,
                size: size,
                parts: [
                    { name: 'curve', data: Buffer.from(curve) },
                    { name: 'Q', data: utils.ecNormalize(keyBuffer) }
                ]
            };
            return (new Key(ecdsaKey));
        }
        throw (new Error('Unsupported algorithm: ' +
            supportedAlgosById[algorithm]));
    }
    function elementToBuf(e) {
        return (Buffer.from(e.split(' ')[1], 'base64'));
    }
    function readDNSSECRSAPrivateKey(elements) {
        var rsaParams = {};
        elements.forEach(function (element) {
            if (element.split(' ')[0] === 'Modulus:')
                rsaParams['n'] = elementToBuf(element);
            else if (element.split(' ')[0] === 'PublicExponent:')
                rsaParams['e'] = elementToBuf(element);
            else if (element.split(' ')[0] === 'PrivateExponent:')
                rsaParams['d'] = elementToBuf(element);
            else if (element.split(' ')[0] === 'Prime1:')
                rsaParams['p'] = elementToBuf(element);
            else if (element.split(' ')[0] === 'Prime2:')
                rsaParams['q'] = elementToBuf(element);
            else if (element.split(' ')[0] === 'Exponent1:')
                rsaParams['dmodp'] = elementToBuf(element);
            else if (element.split(' ')[0] === 'Exponent2:')
                rsaParams['dmodq'] = elementToBuf(element);
            else if (element.split(' ')[0] === 'Coefficient:')
                rsaParams['iqmp'] = elementToBuf(element);
        });
        // now, make the key
        var key = {
            type: 'rsa',
            parts: [
                { name: 'e', data: utils.mpNormalize(rsaParams['e']) },
                { name: 'n', data: utils.mpNormalize(rsaParams['n']) },
                { name: 'd', data: utils.mpNormalize(rsaParams['d']) },
                { name: 'p', data: utils.mpNormalize(rsaParams['p']) },
                { name: 'q', data: utils.mpNormalize(rsaParams['q']) },
                { name: 'dmodp',
                    data: utils.mpNormalize(rsaParams['dmodp']) },
                { name: 'dmodq',
                    data: utils.mpNormalize(rsaParams['dmodq']) },
                { name: 'iqmp',
                    data: utils.mpNormalize(rsaParams['iqmp']) }
            ]
        };
        return (new PrivateKey(key));
    }
    function readDNSSECPrivateKey(alg, elements) {
        if (supportedAlgosById[alg].match(/^RSA-/)) {
            return (readDNSSECRSAPrivateKey(elements));
        }
        if (supportedAlgosById[alg] === 'ECDSA-P384-SHA384' ||
            supportedAlgosById[alg] === 'ECDSA-P256-SHA256') {
            var d = Buffer.from(elements[0].split(' ')[1], 'base64');
            var curve = 'nistp384';
            var size = 384;
            if (supportedAlgosById[alg] === 'ECDSA-P256-SHA256') {
                curve = 'nistp256';
                size = 256;
            }
            // DNSSEC generates the public-key on the fly (go calculate it)
            var publicKey = utils.publicFromPrivateECDSA(curve, d);
            var Q = publicKey.part['Q'].data;
            var ecdsaKey = {
                type: 'ecdsa',
                curve: curve,
                size: size,
                parts: [
                    { name: 'curve', data: Buffer.from(curve) },
                    { name: 'd', data: d },
                    { name: 'Q', data: Q }
                ]
            };
            return (new PrivateKey(ecdsaKey));
        }
        throw (new Error('Unsupported algorithm: ' + supportedAlgosById[alg]));
    }
    function dnssecTimestamp(date) {
        var year = date.getFullYear() + ''; //stringify
        var month = (date.getMonth() + 1);
        var timestampStr = year + month + date.getUTCDate();
        timestampStr += '' + date.getUTCHours() + date.getUTCMinutes();
        timestampStr += date.getUTCSeconds();
        return (timestampStr);
    }
    function rsaAlgFromOptions(opts) {
        if (!opts || !opts.hashAlgo || opts.hashAlgo === 'sha1')
            return ('5 (RSASHA1)');
        else if (opts.hashAlgo === 'sha256')
            return ('8 (RSASHA256)');
        else if (opts.hashAlgo === 'sha512')
            return ('10 (RSASHA512)');
        else
            throw (new Error('Unknown or unsupported hash: ' +
                opts.hashAlgo));
    }
    function writeRSA(key, options) {
        // if we're missing parts, add them.
        if (!key.part.dmodp || !key.part.dmodq) {
            utils.addRSAMissing(key);
        }
        var out = '';
        out += 'Private-key-format: v1.3\n';
        out += 'Algorithm: ' + rsaAlgFromOptions(options) + '\n';
        var n = utils.mpDenormalize(key.part['n'].data);
        out += 'Modulus: ' + n.toString('base64') + '\n';
        var e = utils.mpDenormalize(key.part['e'].data);
        out += 'PublicExponent: ' + e.toString('base64') + '\n';
        var d = utils.mpDenormalize(key.part['d'].data);
        out += 'PrivateExponent: ' + d.toString('base64') + '\n';
        var p = utils.mpDenormalize(key.part['p'].data);
        out += 'Prime1: ' + p.toString('base64') + '\n';
        var q = utils.mpDenormalize(key.part['q'].data);
        out += 'Prime2: ' + q.toString('base64') + '\n';
        var dmodp = utils.mpDenormalize(key.part['dmodp'].data);
        out += 'Exponent1: ' + dmodp.toString('base64') + '\n';
        var dmodq = utils.mpDenormalize(key.part['dmodq'].data);
        out += 'Exponent2: ' + dmodq.toString('base64') + '\n';
        var iqmp = utils.mpDenormalize(key.part['iqmp'].data);
        out += 'Coefficient: ' + iqmp.toString('base64') + '\n';
        // Assume that we're valid as-of now
        var timestamp = new Date();
        out += 'Created: ' + dnssecTimestamp(timestamp) + '\n';
        out += 'Publish: ' + dnssecTimestamp(timestamp) + '\n';
        out += 'Activate: ' + dnssecTimestamp(timestamp) + '\n';
        return (Buffer.from(out, 'ascii'));
    }
    function writeECDSA(key, options) {
        var out = '';
        out += 'Private-key-format: v1.3\n';
        if (key.curve === 'nistp256') {
            out += 'Algorithm: 13 (ECDSAP256SHA256)\n';
        }
        else if (key.curve === 'nistp384') {
            out += 'Algorithm: 14 (ECDSAP384SHA384)\n';
        }
        else {
            throw (new Error('Unsupported curve'));
        }
        var base64Key = key.part['d'].data.toString('base64');
        out += 'PrivateKey: ' + base64Key + '\n';
        // Assume that we're valid as-of now
        var timestamp = new Date();
        out += 'Created: ' + dnssecTimestamp(timestamp) + '\n';
        out += 'Publish: ' + dnssecTimestamp(timestamp) + '\n';
        out += 'Activate: ' + dnssecTimestamp(timestamp) + '\n';
        return (Buffer.from(out, 'ascii'));
    }
    function write(key, options) {
        if (PrivateKey.isPrivateKey(key)) {
            if (key.type === 'rsa') {
                return (writeRSA(key, options));
            }
            else if (key.type === 'ecdsa') {
                return (writeECDSA(key, options));
            }
            else {
                throw (new Error('Unsupported algorithm: ' + key.type));
            }
        }
        else if (Key.isKey(key)) {
            /*
             * RFC3110 requires a keyname, and a keytype, which we
             * don't really have a mechanism for specifying such
             * additional metadata.
             */
            throw (new Error('Format "dnssec" only supports ' +
                'writing private keys'));
        }
        else {
            throw (new Error('key is not a Key or PrivateKey'));
        }
    }
},
/* sshpk/lib/formats/putty.js */ function _(require, module, exports) {
    // Copyright 2018 Joyent, Inc.
    module.exports = {
        read: read,
        write: write
    };
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var rfc4253 = require(883) /* ./rfc4253 */;
    var Key = require(856) /* ../key */;
    var errors = require(860) /* ../errors */;
    function read(buf, options) {
        var lines = buf.toString('ascii').split(/[\r\n]+/);
        var found = false;
        var parts;
        var si = 0;
        while (si < lines.length) {
            parts = splitHeader(lines[si++]);
            if (parts &&
                parts[0].toLowerCase() === 'putty-user-key-file-2') {
                found = true;
                break;
            }
        }
        if (!found) {
            throw (new Error('No PuTTY format first line found'));
        }
        var alg = parts[1];
        parts = splitHeader(lines[si++]);
        assert.equal(parts[0].toLowerCase(), 'encryption');
        parts = splitHeader(lines[si++]);
        assert.equal(parts[0].toLowerCase(), 'comment');
        var comment = parts[1];
        parts = splitHeader(lines[si++]);
        assert.equal(parts[0].toLowerCase(), 'public-lines');
        var publicLines = parseInt(parts[1], 10);
        if (!isFinite(publicLines) || publicLines < 0 ||
            publicLines > lines.length) {
            throw (new Error('Invalid public-lines count'));
        }
        var publicBuf = Buffer.from(lines.slice(si, si + publicLines).join(''), 'base64');
        var keyType = rfc4253.algToKeyType(alg);
        var key = rfc4253.read(publicBuf);
        if (key.type !== keyType) {
            throw (new Error('Outer key algorithm mismatch'));
        }
        key.comment = comment;
        return (key);
    }
    function splitHeader(line) {
        var idx = line.indexOf(':');
        if (idx === -1)
            return (null);
        var header = line.slice(0, idx);
        ++idx;
        while (line[idx] === ' ')
            ++idx;
        var rest = line.slice(idx);
        return ([header, rest]);
    }
    function write(key, options) {
        assert.object(key);
        if (!Key.isKey(key))
            throw (new Error('Must be a public key'));
        var alg = rfc4253.keyTypeToAlg(key);
        var buf = rfc4253.write(key);
        var comment = key.comment || '';
        var b64 = buf.toString('base64');
        var lines = wrap(b64, 64);
        lines.unshift('Public-Lines: ' + lines.length);
        lines.unshift('Comment: ' + comment);
        lines.unshift('Encryption: none');
        lines.unshift('PuTTY-User-Key-File-2: ' + alg);
        return (Buffer.from(lines.join('\n') + '\n'));
    }
    function wrap(txt, len) {
        var lines = [];
        var pos = 0;
        while (pos < txt.length) {
            lines.push(txt.slice(pos, pos + 64));
            pos += 64;
        }
        return (lines);
    }
},
/* sshpk/lib/certificate.js */ function _(require, module, exports) {
    // Copyright 2016 Joyent, Inc.
    module.exports = Certificate;
    var assert = require(853) /* assert-plus */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ./algs */;
    var crypto = require('crypto');
    var Fingerprint = require(859) /* ./fingerprint */;
    var Signature = require(862) /* ./signature */;
    var errs = require(860) /* ./errors */;
    var util = require('util');
    var utils = require(863) /* ./utils */;
    var Key = require(856) /* ./key */;
    var PrivateKey = require(861) /* ./private-key */;
    var Identity = require(889) /* ./identity */;
    var formats = {};
    formats['openssh'] = require(890) /* ./formats/openssh-cert */;
    formats['x509'] = require(891) /* ./formats/x509 */;
    formats['pem'] = require(892) /* ./formats/x509-pem */;
    var CertificateParseError = errs.CertificateParseError;
    var InvalidAlgorithmError = errs.InvalidAlgorithmError;
    function Certificate(opts) {
        assert.object(opts, 'options');
        assert.arrayOfObject(opts.subjects, 'options.subjects');
        utils.assertCompatible(opts.subjects[0], Identity, [1, 0], 'options.subjects');
        utils.assertCompatible(opts.subjectKey, Key, [1, 0], 'options.subjectKey');
        utils.assertCompatible(opts.issuer, Identity, [1, 0], 'options.issuer');
        if (opts.issuerKey !== undefined) {
            utils.assertCompatible(opts.issuerKey, Key, [1, 0], 'options.issuerKey');
        }
        assert.object(opts.signatures, 'options.signatures');
        assert.buffer(opts.serial, 'options.serial');
        assert.date(opts.validFrom, 'options.validFrom');
        assert.date(opts.validUntil, 'optons.validUntil');
        assert.optionalArrayOfString(opts.purposes, 'options.purposes');
        this._hashCache = {};
        this.subjects = opts.subjects;
        this.issuer = opts.issuer;
        this.subjectKey = opts.subjectKey;
        this.issuerKey = opts.issuerKey;
        this.signatures = opts.signatures;
        this.serial = opts.serial;
        this.validFrom = opts.validFrom;
        this.validUntil = opts.validUntil;
        this.purposes = opts.purposes;
    }
    Certificate.formats = formats;
    Certificate.prototype.toBuffer = function (format, options) {
        if (format === undefined)
            format = 'x509';
        assert.string(format, 'format');
        assert.object(formats[format], 'formats[format]');
        assert.optionalObject(options, 'options');
        return (formats[format].write(this, options));
    };
    Certificate.prototype.toString = function (format, options) {
        if (format === undefined)
            format = 'pem';
        return (this.toBuffer(format, options).toString());
    };
    Certificate.prototype.fingerprint = function (algo) {
        if (algo === undefined)
            algo = 'sha256';
        assert.string(algo, 'algorithm');
        var opts = {
            type: 'certificate',
            hash: this.hash(algo),
            algorithm: algo
        };
        return (new Fingerprint(opts));
    };
    Certificate.prototype.hash = function (algo) {
        assert.string(algo, 'algorithm');
        algo = algo.toLowerCase();
        if (algs.hashAlgs[algo] === undefined)
            throw (new InvalidAlgorithmError(algo));
        if (this._hashCache[algo])
            return (this._hashCache[algo]);
        var hash = crypto.createHash(algo).
            update(this.toBuffer('x509')).digest();
        this._hashCache[algo] = hash;
        return (hash);
    };
    Certificate.prototype.isExpired = function (when) {
        if (when === undefined)
            when = new Date();
        return (!((when.getTime() >= this.validFrom.getTime()) &&
            (when.getTime() < this.validUntil.getTime())));
    };
    Certificate.prototype.isSignedBy = function (issuerCert) {
        utils.assertCompatible(issuerCert, Certificate, [1, 0], 'issuer');
        if (!this.issuer.equals(issuerCert.subjects[0]))
            return (false);
        if (this.issuer.purposes && this.issuer.purposes.length > 0 &&
            this.issuer.purposes.indexOf('ca') === -1) {
            return (false);
        }
        return (this.isSignedByKey(issuerCert.subjectKey));
    };
    Certificate.prototype.getExtension = function (keyOrOid) {
        assert.string(keyOrOid, 'keyOrOid');
        var ext = this.getExtensions().filter(function (maybeExt) {
            if (maybeExt.format === 'x509')
                return (maybeExt.oid === keyOrOid);
            if (maybeExt.format === 'openssh')
                return (maybeExt.name === keyOrOid);
            return (false);
        })[0];
        return (ext);
    };
    Certificate.prototype.getExtensions = function () {
        var exts = [];
        var x509 = this.signatures.x509;
        if (x509 && x509.extras && x509.extras.exts) {
            x509.extras.exts.forEach(function (ext) {
                ext.format = 'x509';
                exts.push(ext);
            });
        }
        var openssh = this.signatures.openssh;
        if (openssh && openssh.exts) {
            openssh.exts.forEach(function (ext) {
                ext.format = 'openssh';
                exts.push(ext);
            });
        }
        return (exts);
    };
    Certificate.prototype.isSignedByKey = function (issuerKey) {
        utils.assertCompatible(issuerKey, Key, [1, 2], 'issuerKey');
        if (this.issuerKey !== undefined) {
            return (this.issuerKey.
                fingerprint('sha512').matches(issuerKey));
        }
        var fmt = Object.keys(this.signatures)[0];
        var valid = formats[fmt].verify(this, issuerKey);
        if (valid)
            this.issuerKey = issuerKey;
        return (valid);
    };
    Certificate.prototype.signWith = function (key) {
        utils.assertCompatible(key, PrivateKey, [1, 2], 'key');
        var fmts = Object.keys(formats);
        var didOne = false;
        for (var i = 0; i < fmts.length; ++i) {
            if (fmts[i] !== 'pem') {
                var ret = formats[fmts[i]].sign(this, key);
                if (ret === true)
                    didOne = true;
            }
        }
        if (!didOne) {
            throw (new Error('Failed to sign the certificate for any ' +
                'available certificate formats'));
        }
    };
    Certificate.createSelfSigned = function (subjectOrSubjects, key, options) {
        var subjects;
        if (Array.isArray(subjectOrSubjects))
            subjects = subjectOrSubjects;
        else
            subjects = [subjectOrSubjects];
        assert.arrayOfObject(subjects);
        subjects.forEach(function (subject) {
            utils.assertCompatible(subject, Identity, [1, 0], 'subject');
        });
        utils.assertCompatible(key, PrivateKey, [1, 2], 'private key');
        assert.optionalObject(options, 'options');
        if (options === undefined)
            options = {};
        assert.optionalObject(options.validFrom, 'options.validFrom');
        assert.optionalObject(options.validUntil, 'options.validUntil');
        var validFrom = options.validFrom;
        var validUntil = options.validUntil;
        if (validFrom === undefined)
            validFrom = new Date();
        if (validUntil === undefined) {
            assert.optionalNumber(options.lifetime, 'options.lifetime');
            var lifetime = options.lifetime;
            if (lifetime === undefined)
                lifetime = 10 * 365 * 24 * 3600;
            validUntil = new Date();
            validUntil.setTime(validUntil.getTime() + lifetime * 1000);
        }
        assert.optionalBuffer(options.serial, 'options.serial');
        var serial = options.serial;
        if (serial === undefined)
            serial = Buffer.from('0000000000000001', 'hex');
        var purposes = options.purposes;
        if (purposes === undefined)
            purposes = [];
        if (purposes.indexOf('signature') === -1)
            purposes.push('signature');
        /* Self-signed certs are always CAs. */
        if (purposes.indexOf('ca') === -1)
            purposes.push('ca');
        if (purposes.indexOf('crl') === -1)
            purposes.push('crl');
        /*
         * If we weren't explicitly given any other purposes, do the sensible
         * thing and add some basic ones depending on the subject type.
         */
        if (purposes.length <= 3) {
            var hostSubjects = subjects.filter(function (subject) {
                return (subject.type === 'host');
            });
            var userSubjects = subjects.filter(function (subject) {
                return (subject.type === 'user');
            });
            if (hostSubjects.length > 0) {
                if (purposes.indexOf('serverAuth') === -1)
                    purposes.push('serverAuth');
            }
            if (userSubjects.length > 0) {
                if (purposes.indexOf('clientAuth') === -1)
                    purposes.push('clientAuth');
            }
            if (userSubjects.length > 0 || hostSubjects.length > 0) {
                if (purposes.indexOf('keyAgreement') === -1)
                    purposes.push('keyAgreement');
                if (key.type === 'rsa' &&
                    purposes.indexOf('encryption') === -1)
                    purposes.push('encryption');
            }
        }
        var cert = new Certificate({
            subjects: subjects,
            issuer: subjects[0],
            subjectKey: key.toPublic(),
            issuerKey: key.toPublic(),
            signatures: {},
            serial: serial,
            validFrom: validFrom,
            validUntil: validUntil,
            purposes: purposes
        });
        cert.signWith(key);
        return (cert);
    };
    Certificate.create =
        function (subjectOrSubjects, key, issuer, issuerKey, options) {
            var subjects;
            if (Array.isArray(subjectOrSubjects))
                subjects = subjectOrSubjects;
            else
                subjects = [subjectOrSubjects];
            assert.arrayOfObject(subjects);
            subjects.forEach(function (subject) {
                utils.assertCompatible(subject, Identity, [1, 0], 'subject');
            });
            utils.assertCompatible(key, Key, [1, 0], 'key');
            if (PrivateKey.isPrivateKey(key))
                key = key.toPublic();
            utils.assertCompatible(issuer, Identity, [1, 0], 'issuer');
            utils.assertCompatible(issuerKey, PrivateKey, [1, 2], 'issuer key');
            assert.optionalObject(options, 'options');
            if (options === undefined)
                options = {};
            assert.optionalObject(options.validFrom, 'options.validFrom');
            assert.optionalObject(options.validUntil, 'options.validUntil');
            var validFrom = options.validFrom;
            var validUntil = options.validUntil;
            if (validFrom === undefined)
                validFrom = new Date();
            if (validUntil === undefined) {
                assert.optionalNumber(options.lifetime, 'options.lifetime');
                var lifetime = options.lifetime;
                if (lifetime === undefined)
                    lifetime = 10 * 365 * 24 * 3600;
                validUntil = new Date();
                validUntil.setTime(validUntil.getTime() + lifetime * 1000);
            }
            assert.optionalBuffer(options.serial, 'options.serial');
            var serial = options.serial;
            if (serial === undefined)
                serial = Buffer.from('0000000000000001', 'hex');
            var purposes = options.purposes;
            if (purposes === undefined)
                purposes = [];
            if (purposes.indexOf('signature') === -1)
                purposes.push('signature');
            if (options.ca === true) {
                if (purposes.indexOf('ca') === -1)
                    purposes.push('ca');
                if (purposes.indexOf('crl') === -1)
                    purposes.push('crl');
            }
            var hostSubjects = subjects.filter(function (subject) {
                return (subject.type === 'host');
            });
            var userSubjects = subjects.filter(function (subject) {
                return (subject.type === 'user');
            });
            if (hostSubjects.length > 0) {
                if (purposes.indexOf('serverAuth') === -1)
                    purposes.push('serverAuth');
            }
            if (userSubjects.length > 0) {
                if (purposes.indexOf('clientAuth') === -1)
                    purposes.push('clientAuth');
            }
            if (userSubjects.length > 0 || hostSubjects.length > 0) {
                if (purposes.indexOf('keyAgreement') === -1)
                    purposes.push('keyAgreement');
                if (key.type === 'rsa' &&
                    purposes.indexOf('encryption') === -1)
                    purposes.push('encryption');
            }
            var cert = new Certificate({
                subjects: subjects,
                issuer: issuer,
                subjectKey: key,
                issuerKey: issuerKey.toPublic(),
                signatures: {},
                serial: serial,
                validFrom: validFrom,
                validUntil: validUntil,
                purposes: purposes
            });
            cert.signWith(issuerKey);
            return (cert);
        };
    Certificate.parse = function (data, format, options) {
        if (typeof (data) !== 'string')
            assert.buffer(data, 'data');
        if (format === undefined)
            format = 'auto';
        assert.string(format, 'format');
        if (typeof (options) === 'string')
            options = { filename: options };
        assert.optionalObject(options, 'options');
        if (options === undefined)
            options = {};
        assert.optionalString(options.filename, 'options.filename');
        if (options.filename === undefined)
            options.filename = '(unnamed)';
        assert.object(formats[format], 'formats[format]');
        try {
            var k = formats[format].read(data, options);
            return (k);
        }
        catch (e) {
            throw (new CertificateParseError(options.filename, format, e));
        }
    };
    Certificate.isCertificate = function (obj, ver) {
        return (utils.isCompatible(obj, Certificate, ver));
    };
    /*
     * API versions for Certificate:
     * [1,0] -- initial ver
     * [1,1] -- openssh format now unpacks extensions
     */
    Certificate.prototype._sshpkApiVersion = [1, 1];
    Certificate._oldVersionDetect = function (obj) {
        return ([1, 0]);
    };
},
/* sshpk/lib/identity.js */ function _(require, module, exports) {
    // Copyright 2017 Joyent, Inc.
    module.exports = Identity;
    var assert = require(853) /* assert-plus */;
    var algs = require(857) /* ./algs */;
    var crypto = require('crypto');
    var Fingerprint = require(859) /* ./fingerprint */;
    var Signature = require(862) /* ./signature */;
    var errs = require(860) /* ./errors */;
    var util = require('util');
    var utils = require(863) /* ./utils */;
    var asn1 = require(864) /* asn1 */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    /*JSSTYLED*/
    var DNS_NAME_RE = /^([*]|[a-z0-9][a-z0-9\-]{0,62})(?:\.([*]|[a-z0-9][a-z0-9\-]{0,62}))*$/i;
    var oids = {};
    oids.cn = '2.5.4.3';
    oids.o = '2.5.4.10';
    oids.ou = '2.5.4.11';
    oids.l = '2.5.4.7';
    oids.s = '2.5.4.8';
    oids.c = '2.5.4.6';
    oids.sn = '2.5.4.4';
    oids.postalCode = '2.5.4.17';
    oids.serialNumber = '2.5.4.5';
    oids.street = '2.5.4.9';
    oids.x500UniqueIdentifier = '2.5.4.45';
    oids.role = '2.5.4.72';
    oids.telephoneNumber = '2.5.4.20';
    oids.description = '2.5.4.13';
    oids.dc = '0.9.2342.19200300.100.1.25';
    oids.uid = '0.9.2342.19200300.100.1.1';
    oids.mail = '0.9.2342.19200300.100.1.3';
    oids.title = '2.5.4.12';
    oids.gn = '2.5.4.42';
    oids.initials = '2.5.4.43';
    oids.pseudonym = '2.5.4.65';
    oids.emailAddress = '1.2.840.113549.1.9.1';
    var unoids = {};
    Object.keys(oids).forEach(function (k) {
        unoids[oids[k]] = k;
    });
    function Identity(opts) {
        var self = this;
        assert.object(opts, 'options');
        assert.arrayOfObject(opts.components, 'options.components');
        this.components = opts.components;
        this.componentLookup = {};
        this.components.forEach(function (c) {
            if (c.name && !c.oid)
                c.oid = oids[c.name];
            if (c.oid && !c.name)
                c.name = unoids[c.oid];
            if (self.componentLookup[c.name] === undefined)
                self.componentLookup[c.name] = [];
            self.componentLookup[c.name].push(c);
        });
        if (this.componentLookup.cn && this.componentLookup.cn.length > 0) {
            this.cn = this.componentLookup.cn[0].value;
        }
        assert.optionalString(opts.type, 'options.type');
        if (opts.type === undefined) {
            if (this.components.length === 1 &&
                this.componentLookup.cn &&
                this.componentLookup.cn.length === 1 &&
                this.componentLookup.cn[0].value.match(DNS_NAME_RE)) {
                this.type = 'host';
                this.hostname = this.componentLookup.cn[0].value;
            }
            else if (this.componentLookup.dc &&
                this.components.length === this.componentLookup.dc.length) {
                this.type = 'host';
                this.hostname = this.componentLookup.dc.map(function (c) {
                    return (c.value);
                }).join('.');
            }
            else if (this.componentLookup.uid &&
                this.components.length ===
                    this.componentLookup.uid.length) {
                this.type = 'user';
                this.uid = this.componentLookup.uid[0].value;
            }
            else if (this.componentLookup.cn &&
                this.componentLookup.cn.length === 1 &&
                this.componentLookup.cn[0].value.match(DNS_NAME_RE)) {
                this.type = 'host';
                this.hostname = this.componentLookup.cn[0].value;
            }
            else if (this.componentLookup.uid &&
                this.componentLookup.uid.length === 1) {
                this.type = 'user';
                this.uid = this.componentLookup.uid[0].value;
            }
            else if (this.componentLookup.mail &&
                this.componentLookup.mail.length === 1) {
                this.type = 'email';
                this.email = this.componentLookup.mail[0].value;
            }
            else if (this.componentLookup.cn &&
                this.componentLookup.cn.length === 1) {
                this.type = 'user';
                this.uid = this.componentLookup.cn[0].value;
            }
            else {
                this.type = 'unknown';
            }
        }
        else {
            this.type = opts.type;
            if (this.type === 'host')
                this.hostname = opts.hostname;
            else if (this.type === 'user')
                this.uid = opts.uid;
            else if (this.type === 'email')
                this.email = opts.email;
            else
                throw (new Error('Unknown type ' + this.type));
        }
    }
    Identity.prototype.toString = function () {
        return (this.components.map(function (c) {
            var n = c.name.toUpperCase();
            /*JSSTYLED*/
            n = n.replace(/=/g, '\\=');
            var v = c.value;
            /*JSSTYLED*/
            v = v.replace(/,/g, '\\,');
            return (n + '=' + v);
        }).join(', '));
    };
    Identity.prototype.get = function (name, asArray) {
        assert.string(name, 'name');
        var arr = this.componentLookup[name];
        if (arr === undefined || arr.length === 0)
            return (undefined);
        if (!asArray && arr.length > 1)
            throw (new Error('Multiple values for attribute ' + name));
        if (!asArray)
            return (arr[0].value);
        return (arr.map(function (c) {
            return (c.value);
        }));
    };
    Identity.prototype.toArray = function (idx) {
        return (this.components.map(function (c) {
            return ({
                name: c.name,
                value: c.value
            });
        }));
    };
    /*
     * These are from X.680 -- PrintableString allowed chars are in section 37.4
     * table 8. Spec for IA5Strings is "1,6 + SPACE + DEL" where 1 refers to
     * ISO IR #001 (standard ASCII control characters) and 6 refers to ISO IR #006
     * (the basic ASCII character set).
     */
    /* JSSTYLED */
    var NOT_PRINTABLE = /[^a-zA-Z0-9 '(),+.\/:=?-]/;
    /* JSSTYLED */
    var NOT_IA5 = /[^\x00-\x7f]/;
    Identity.prototype.toAsn1 = function (der, tag) {
        der.startSequence(tag);
        this.components.forEach(function (c) {
            der.startSequence(asn1.Ber.Constructor | asn1.Ber.Set);
            der.startSequence();
            der.writeOID(c.oid);
            /*
             * If we fit in a PrintableString, use that. Otherwise use an
             * IA5String or UTF8String.
             *
             * If this identity was parsed from a DN, use the ASN.1 types
             * from the original representation (otherwise this might not
             * be a full match for the original in some validators).
             */
            if (c.asn1type === asn1.Ber.Utf8String ||
                c.value.match(NOT_IA5)) {
                var v = Buffer.from(c.value, 'utf8');
                der.writeBuffer(v, asn1.Ber.Utf8String);
            }
            else if (c.asn1type === asn1.Ber.IA5String ||
                c.value.match(NOT_PRINTABLE)) {
                der.writeString(c.value, asn1.Ber.IA5String);
            }
            else {
                var type = asn1.Ber.PrintableString;
                if (c.asn1type !== undefined)
                    type = c.asn1type;
                der.writeString(c.value, type);
            }
            der.endSequence();
            der.endSequence();
        });
        der.endSequence();
    };
    function globMatch(a, b) {
        if (a === '**' || b === '**')
            return (true);
        var aParts = a.split('.');
        var bParts = b.split('.');
        if (aParts.length !== bParts.length)
            return (false);
        for (var i = 0; i < aParts.length; ++i) {
            if (aParts[i] === '*' || bParts[i] === '*')
                continue;
            if (aParts[i] !== bParts[i])
                return (false);
        }
        return (true);
    }
    Identity.prototype.equals = function (other) {
        if (!Identity.isIdentity(other, [1, 0]))
            return (false);
        if (other.components.length !== this.components.length)
            return (false);
        for (var i = 0; i < this.components.length; ++i) {
            if (this.components[i].oid !== other.components[i].oid)
                return (false);
            if (!globMatch(this.components[i].value, other.components[i].value)) {
                return (false);
            }
        }
        return (true);
    };
    Identity.forHost = function (hostname) {
        assert.string(hostname, 'hostname');
        return (new Identity({
            type: 'host',
            hostname: hostname,
            components: [{ name: 'cn', value: hostname }]
        }));
    };
    Identity.forUser = function (uid) {
        assert.string(uid, 'uid');
        return (new Identity({
            type: 'user',
            uid: uid,
            components: [{ name: 'uid', value: uid }]
        }));
    };
    Identity.forEmail = function (email) {
        assert.string(email, 'email');
        return (new Identity({
            type: 'email',
            email: email,
            components: [{ name: 'mail', value: email }]
        }));
    };
    Identity.parseDN = function (dn) {
        assert.string(dn, 'dn');
        var parts = [''];
        var idx = 0;
        var rem = dn;
        while (rem.length > 0) {
            var m;
            /*JSSTYLED*/
            if ((m = /^,/.exec(rem)) !== null) {
                parts[++idx] = '';
                rem = rem.slice(m[0].length);
                /*JSSTYLED*/
            }
            else if ((m = /^\\,/.exec(rem)) !== null) {
                parts[idx] += ',';
                rem = rem.slice(m[0].length);
                /*JSSTYLED*/
            }
            else if ((m = /^\\./.exec(rem)) !== null) {
                parts[idx] += m[0];
                rem = rem.slice(m[0].length);
                /*JSSTYLED*/
            }
            else if ((m = /^[^\\,]+/.exec(rem)) !== null) {
                parts[idx] += m[0];
                rem = rem.slice(m[0].length);
            }
            else {
                throw (new Error('Failed to parse DN'));
            }
        }
        var cmps = parts.map(function (c) {
            c = c.trim();
            var eqPos = c.indexOf('=');
            while (eqPos > 0 && c.charAt(eqPos - 1) === '\\')
                eqPos = c.indexOf('=', eqPos + 1);
            if (eqPos === -1) {
                throw (new Error('Failed to parse DN'));
            }
            /*JSSTYLED*/
            var name = c.slice(0, eqPos).toLowerCase().replace(/\\=/g, '=');
            var value = c.slice(eqPos + 1);
            return ({ name: name, value: value });
        });
        return (new Identity({ components: cmps }));
    };
    Identity.fromArray = function (components) {
        assert.arrayOfObject(components, 'components');
        components.forEach(function (cmp) {
            assert.object(cmp, 'component');
            assert.string(cmp.name, 'component.name');
            if (!Buffer.isBuffer(cmp.value) &&
                !(typeof (cmp.value) === 'string')) {
                throw (new Error('Invalid component value'));
            }
        });
        return (new Identity({ components: components }));
    };
    Identity.parseAsn1 = function (der, top) {
        var components = [];
        der.readSequence(top);
        var end = der.offset + der.length;
        while (der.offset < end) {
            der.readSequence(asn1.Ber.Constructor | asn1.Ber.Set);
            var after = der.offset + der.length;
            der.readSequence();
            var oid = der.readOID();
            var type = der.peek();
            var value;
            switch (type) {
                case asn1.Ber.PrintableString:
                case asn1.Ber.IA5String:
                case asn1.Ber.OctetString:
                case asn1.Ber.T61String:
                    value = der.readString(type);
                    break;
                case asn1.Ber.Utf8String:
                    value = der.readString(type, true);
                    value = value.toString('utf8');
                    break;
                case asn1.Ber.CharacterString:
                case asn1.Ber.BMPString:
                    value = der.readString(type, true);
                    value = value.toString('utf16le');
                    break;
                default:
                    throw (new Error('Unknown asn1 type ' + type));
            }
            components.push({ oid: oid, asn1type: type, value: value });
            der._offset = after;
        }
        der._offset = end;
        return (new Identity({
            components: components
        }));
    };
    Identity.isIdentity = function (obj, ver) {
        return (utils.isCompatible(obj, Identity, ver));
    };
    /*
     * API versions for Identity:
     * [1,0] -- initial ver
     */
    Identity.prototype._sshpkApiVersion = [1, 0];
    Identity._oldVersionDetect = function (obj) {
        return ([1, 0]);
    };
},
/* sshpk/lib/formats/openssh-cert.js */ function _(require, module, exports) {
    // Copyright 2017 Joyent, Inc.
    module.exports = {
        read: read,
        verify: verify,
        sign: sign,
        signAsync: signAsync,
        write: write,
        /* Internal private API */
        fromBuffer: fromBuffer,
        toBuffer: toBuffer
    };
    var assert = require(853) /* assert-plus */;
    var SSHBuffer = require(873) /* ../ssh-buffer */;
    var crypto = require('crypto');
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ../algs */;
    var Key = require(856) /* ../key */;
    var PrivateKey = require(861) /* ../private-key */;
    var Identity = require(889) /* ../identity */;
    var rfc4253 = require(883) /* ./rfc4253 */;
    var Signature = require(862) /* ../signature */;
    var utils = require(863) /* ../utils */;
    var Certificate = require(888) /* ../certificate */;
    function verify(cert, key) {
        /*
         * We always give an issuerKey, so if our verify() is being called then
         * there was no signature. Return false.
         */
        return (false);
    }
    var TYPES = {
        'user': 1,
        'host': 2
    };
    Object.keys(TYPES).forEach(function (k) { TYPES[TYPES[k]] = k; });
    var ECDSA_ALGO = /^ecdsa-sha2-([^@-]+)-cert-v01@openssh.com$/;
    function read(buf, options) {
        if (Buffer.isBuffer(buf))
            buf = buf.toString('ascii');
        var parts = buf.trim().split(/[ \t\n]+/g);
        if (parts.length < 2 || parts.length > 3)
            throw (new Error('Not a valid SSH certificate line'));
        var algo = parts[0];
        var data = parts[1];
        data = Buffer.from(data, 'base64');
        return (fromBuffer(data, algo));
    }
    function fromBuffer(data, algo, partial) {
        var sshbuf = new SSHBuffer({ buffer: data });
        var innerAlgo = sshbuf.readString();
        if (algo !== undefined && innerAlgo !== algo)
            throw (new Error('SSH certificate algorithm mismatch'));
        if (algo === undefined)
            algo = innerAlgo;
        var cert = {};
        cert.signatures = {};
        cert.signatures.openssh = {};
        cert.signatures.openssh.nonce = sshbuf.readBuffer();
        var key = {};
        var parts = (key.parts = []);
        key.type = getAlg(algo);
        var partCount = algs.info[key.type].parts.length;
        while (parts.length < partCount)
            parts.push(sshbuf.readPart());
        assert.ok(parts.length >= 1, 'key must have at least one part');
        var algInfo = algs.info[key.type];
        if (key.type === 'ecdsa') {
            var res = ECDSA_ALGO.exec(algo);
            assert.ok(res !== null);
            assert.strictEqual(res[1], parts[0].data.toString());
        }
        for (var i = 0; i < algInfo.parts.length; ++i) {
            parts[i].name = algInfo.parts[i];
            if (parts[i].name !== 'curve' &&
                algInfo.normalize !== false) {
                var p = parts[i];
                p.data = utils.mpNormalize(p.data);
            }
        }
        cert.subjectKey = new Key(key);
        cert.serial = sshbuf.readInt64();
        var type = TYPES[sshbuf.readInt()];
        assert.string(type, 'valid cert type');
        cert.signatures.openssh.keyId = sshbuf.readString();
        var principals = [];
        var pbuf = sshbuf.readBuffer();
        var psshbuf = new SSHBuffer({ buffer: pbuf });
        while (!psshbuf.atEnd())
            principals.push(psshbuf.readString());
        if (principals.length === 0)
            principals = ['*'];
        cert.subjects = principals.map(function (pr) {
            if (type === 'user')
                return (Identity.forUser(pr));
            else if (type === 'host')
                return (Identity.forHost(pr));
            throw (new Error('Unknown identity type ' + type));
        });
        cert.validFrom = int64ToDate(sshbuf.readInt64());
        cert.validUntil = int64ToDate(sshbuf.readInt64());
        var exts = [];
        var extbuf = new SSHBuffer({ buffer: sshbuf.readBuffer() });
        var ext;
        while (!extbuf.atEnd()) {
            ext = { critical: true };
            ext.name = extbuf.readString();
            ext.data = extbuf.readBuffer();
            exts.push(ext);
        }
        extbuf = new SSHBuffer({ buffer: sshbuf.readBuffer() });
        while (!extbuf.atEnd()) {
            ext = { critical: false };
            ext.name = extbuf.readString();
            ext.data = extbuf.readBuffer();
            exts.push(ext);
        }
        cert.signatures.openssh.exts = exts;
        /* reserved */
        sshbuf.readBuffer();
        var signingKeyBuf = sshbuf.readBuffer();
        cert.issuerKey = rfc4253.read(signingKeyBuf);
        /*
         * OpenSSH certs don't give the identity of the issuer, just their
         * public key. So, we use an Identity that matches anything. The
         * isSignedBy() function will later tell you if the key matches.
         */
        cert.issuer = Identity.forHost('**');
        var sigBuf = sshbuf.readBuffer();
        cert.signatures.openssh.signature =
            Signature.parse(sigBuf, cert.issuerKey.type, 'ssh');
        if (partial !== undefined) {
            partial.remainder = sshbuf.remainder();
            partial.consumed = sshbuf._offset;
        }
        return (new Certificate(cert));
    }
    function int64ToDate(buf) {
        var i = buf.readUInt32BE(0) * 4294967296;
        i += buf.readUInt32BE(4);
        var d = new Date();
        d.setTime(i * 1000);
        d.sourceInt64 = buf;
        return (d);
    }
    function dateToInt64(date) {
        if (date.sourceInt64 !== undefined)
            return (date.sourceInt64);
        var i = Math.round(date.getTime() / 1000);
        var upper = Math.floor(i / 4294967296);
        var lower = Math.floor(i % 4294967296);
        var buf = Buffer.alloc(8);
        buf.writeUInt32BE(upper, 0);
        buf.writeUInt32BE(lower, 4);
        return (buf);
    }
    function sign(cert, key) {
        if (cert.signatures.openssh === undefined)
            cert.signatures.openssh = {};
        try {
            var blob = toBuffer(cert, true);
        }
        catch (e) {
            delete (cert.signatures.openssh);
            return (false);
        }
        var sig = cert.signatures.openssh;
        var hashAlgo = undefined;
        if (key.type === 'rsa' || key.type === 'dsa')
            hashAlgo = 'sha1';
        var signer = key.createSign(hashAlgo);
        signer.write(blob);
        sig.signature = signer.sign();
        return (true);
    }
    function signAsync(cert, signer, done) {
        if (cert.signatures.openssh === undefined)
            cert.signatures.openssh = {};
        try {
            var blob = toBuffer(cert, true);
        }
        catch (e) {
            delete (cert.signatures.openssh);
            done(e);
            return;
        }
        var sig = cert.signatures.openssh;
        signer(blob, function (err, signature) {
            if (err) {
                done(err);
                return;
            }
            try {
                /*
                 * This will throw if the signature isn't of a
                 * type/algo that can be used for SSH.
                 */
                signature.toBuffer('ssh');
            }
            catch (e) {
                done(e);
                return;
            }
            sig.signature = signature;
            done();
        });
    }
    function write(cert, options) {
        if (options === undefined)
            options = {};
        var blob = toBuffer(cert);
        var out = getCertType(cert.subjectKey) + ' ' + blob.toString('base64');
        if (options.comment)
            out = out + ' ' + options.comment;
        return (out);
    }
    function toBuffer(cert, noSig) {
        assert.object(cert.signatures.openssh, 'signature for openssh format');
        var sig = cert.signatures.openssh;
        if (sig.nonce === undefined)
            sig.nonce = crypto.randomBytes(16);
        var buf = new SSHBuffer({});
        buf.writeString(getCertType(cert.subjectKey));
        buf.writeBuffer(sig.nonce);
        var key = cert.subjectKey;
        var algInfo = algs.info[key.type];
        algInfo.parts.forEach(function (part) {
            buf.writePart(key.part[part]);
        });
        buf.writeInt64(cert.serial);
        var type = cert.subjects[0].type;
        assert.notStrictEqual(type, 'unknown');
        cert.subjects.forEach(function (id) {
            assert.strictEqual(id.type, type);
        });
        type = TYPES[type];
        buf.writeInt(type);
        if (sig.keyId === undefined) {
            sig.keyId = cert.subjects[0].type + '_' +
                (cert.subjects[0].uid || cert.subjects[0].hostname);
        }
        buf.writeString(sig.keyId);
        var sub = new SSHBuffer({});
        cert.subjects.forEach(function (id) {
            if (type === TYPES.host)
                sub.writeString(id.hostname);
            else if (type === TYPES.user)
                sub.writeString(id.uid);
        });
        buf.writeBuffer(sub.toBuffer());
        buf.writeInt64(dateToInt64(cert.validFrom));
        buf.writeInt64(dateToInt64(cert.validUntil));
        var exts = sig.exts;
        if (exts === undefined)
            exts = [];
        var extbuf = new SSHBuffer({});
        exts.forEach(function (ext) {
            if (ext.critical !== true)
                return;
            extbuf.writeString(ext.name);
            extbuf.writeBuffer(ext.data);
        });
        buf.writeBuffer(extbuf.toBuffer());
        extbuf = new SSHBuffer({});
        exts.forEach(function (ext) {
            if (ext.critical === true)
                return;
            extbuf.writeString(ext.name);
            extbuf.writeBuffer(ext.data);
        });
        buf.writeBuffer(extbuf.toBuffer());
        /* reserved */
        buf.writeBuffer(Buffer.alloc(0));
        sub = rfc4253.write(cert.issuerKey);
        buf.writeBuffer(sub);
        if (!noSig)
            buf.writeBuffer(sig.signature.toBuffer('ssh'));
        return (buf.toBuffer());
    }
    function getAlg(certType) {
        if (certType === 'ssh-rsa-cert-v01@openssh.com')
            return ('rsa');
        if (certType === 'ssh-dss-cert-v01@openssh.com')
            return ('dsa');
        if (certType.match(ECDSA_ALGO))
            return ('ecdsa');
        if (certType === 'ssh-ed25519-cert-v01@openssh.com')
            return ('ed25519');
        throw (new Error('Unsupported cert type ' + certType));
    }
    function getCertType(key) {
        if (key.type === 'rsa')
            return ('ssh-rsa-cert-v01@openssh.com');
        if (key.type === 'dsa')
            return ('ssh-dss-cert-v01@openssh.com');
        if (key.type === 'ecdsa')
            return ('ecdsa-sha2-' + key.curve + '-cert-v01@openssh.com');
        if (key.type === 'ed25519')
            return ('ssh-ed25519-cert-v01@openssh.com');
        throw (new Error('Unsupported key type ' + key.type));
    }
},
/* sshpk/lib/formats/x509.js */ function _(require, module, exports) {
    // Copyright 2017 Joyent, Inc.
    module.exports = {
        read: read,
        verify: verify,
        sign: sign,
        signAsync: signAsync,
        write: write
    };
    var assert = require(853) /* assert-plus */;
    var asn1 = require(864) /* asn1 */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ../algs */;
    var utils = require(863) /* ../utils */;
    var Key = require(856) /* ../key */;
    var PrivateKey = require(861) /* ../private-key */;
    var pem = require(879) /* ./pem */;
    var Identity = require(889) /* ../identity */;
    var Signature = require(862) /* ../signature */;
    var Certificate = require(888) /* ../certificate */;
    var pkcs8 = require(881) /* ./pkcs8 */;
    /*
     * This file is based on RFC5280 (X.509).
     */
    /* Helper to read in a single mpint */
    function readMPInt(der, nm) {
        assert.strictEqual(der.peek(), asn1.Ber.Integer, nm + ' is not an Integer');
        return (utils.mpNormalize(der.readString(asn1.Ber.Integer, true)));
    }
    function verify(cert, key) {
        var sig = cert.signatures.x509;
        assert.object(sig, 'x509 signature');
        var algParts = sig.algo.split('-');
        if (algParts[0] !== key.type)
            return (false);
        var blob = sig.cache;
        if (blob === undefined) {
            var der = new asn1.BerWriter();
            writeTBSCert(cert, der);
            blob = der.buffer;
        }
        var verifier = key.createVerify(algParts[1]);
        verifier.write(blob);
        return (verifier.verify(sig.signature));
    }
    function Local(i) {
        return (asn1.Ber.Context | asn1.Ber.Constructor | i);
    }
    function Context(i) {
        return (asn1.Ber.Context | i);
    }
    var SIGN_ALGS = {
        'rsa-md5': '1.2.840.113549.1.1.4',
        'rsa-sha1': '1.2.840.113549.1.1.5',
        'rsa-sha256': '1.2.840.113549.1.1.11',
        'rsa-sha384': '1.2.840.113549.1.1.12',
        'rsa-sha512': '1.2.840.113549.1.1.13',
        'dsa-sha1': '1.2.840.10040.4.3',
        'dsa-sha256': '2.16.840.1.101.3.4.3.2',
        'ecdsa-sha1': '1.2.840.10045.4.1',
        'ecdsa-sha256': '1.2.840.10045.4.3.2',
        'ecdsa-sha384': '1.2.840.10045.4.3.3',
        'ecdsa-sha512': '1.2.840.10045.4.3.4',
        'ed25519-sha512': '1.3.101.112'
    };
    Object.keys(SIGN_ALGS).forEach(function (k) {
        SIGN_ALGS[SIGN_ALGS[k]] = k;
    });
    SIGN_ALGS['1.3.14.3.2.3'] = 'rsa-md5';
    SIGN_ALGS['1.3.14.3.2.29'] = 'rsa-sha1';
    var EXTS = {
        'issuerKeyId': '2.5.29.35',
        'altName': '2.5.29.17',
        'basicConstraints': '2.5.29.19',
        'keyUsage': '2.5.29.15',
        'extKeyUsage': '2.5.29.37'
    };
    function read(buf, options) {
        if (typeof (buf) === 'string') {
            buf = Buffer.from(buf, 'binary');
        }
        assert.buffer(buf, 'buf');
        var der = new asn1.BerReader(buf);
        der.readSequence();
        if (Math.abs(der.length - der.remain) > 1) {
            throw (new Error('DER sequence does not contain whole byte ' +
                'stream'));
        }
        var tbsStart = der.offset;
        der.readSequence();
        var sigOffset = der.offset + der.length;
        var tbsEnd = sigOffset;
        if (der.peek() === Local(0)) {
            der.readSequence(Local(0));
            var version = der.readInt();
            assert.ok(version <= 3, 'only x.509 versions up to v3 supported');
        }
        var cert = {};
        cert.signatures = {};
        var sig = (cert.signatures.x509 = {});
        sig.extras = {};
        cert.serial = readMPInt(der, 'serial');
        der.readSequence();
        var after = der.offset + der.length;
        var certAlgOid = der.readOID();
        var certAlg = SIGN_ALGS[certAlgOid];
        if (certAlg === undefined)
            throw (new Error('unknown signature algorithm ' + certAlgOid));
        der._offset = after;
        cert.issuer = Identity.parseAsn1(der);
        der.readSequence();
        cert.validFrom = readDate(der);
        cert.validUntil = readDate(der);
        cert.subjects = [Identity.parseAsn1(der)];
        der.readSequence();
        after = der.offset + der.length;
        cert.subjectKey = pkcs8.readPkcs8(undefined, 'public', der);
        der._offset = after;
        /* issuerUniqueID */
        if (der.peek() === Local(1)) {
            der.readSequence(Local(1));
            sig.extras.issuerUniqueID =
                buf.slice(der.offset, der.offset + der.length);
            der._offset += der.length;
        }
        /* subjectUniqueID */
        if (der.peek() === Local(2)) {
            der.readSequence(Local(2));
            sig.extras.subjectUniqueID =
                buf.slice(der.offset, der.offset + der.length);
            der._offset += der.length;
        }
        /* extensions */
        if (der.peek() === Local(3)) {
            der.readSequence(Local(3));
            var extEnd = der.offset + der.length;
            der.readSequence();
            while (der.offset < extEnd)
                readExtension(cert, buf, der);
            assert.strictEqual(der.offset, extEnd);
        }
        assert.strictEqual(der.offset, sigOffset);
        der.readSequence();
        after = der.offset + der.length;
        var sigAlgOid = der.readOID();
        var sigAlg = SIGN_ALGS[sigAlgOid];
        if (sigAlg === undefined)
            throw (new Error('unknown signature algorithm ' + sigAlgOid));
        der._offset = after;
        var sigData = der.readString(asn1.Ber.BitString, true);
        if (sigData[0] === 0)
            sigData = sigData.slice(1);
        var algParts = sigAlg.split('-');
        sig.signature = Signature.parse(sigData, algParts[0], 'asn1');
        sig.signature.hashAlgorithm = algParts[1];
        sig.algo = sigAlg;
        sig.cache = buf.slice(tbsStart, tbsEnd);
        return (new Certificate(cert));
    }
    function readDate(der) {
        if (der.peek() === asn1.Ber.UTCTime) {
            return (utcTimeToDate(der.readString(asn1.Ber.UTCTime)));
        }
        else if (der.peek() === asn1.Ber.GeneralizedTime) {
            return (gTimeToDate(der.readString(asn1.Ber.GeneralizedTime)));
        }
        else {
            throw (new Error('Unsupported date format'));
        }
    }
    function writeDate(der, date) {
        if (date.getUTCFullYear() >= 2050 || date.getUTCFullYear() < 1950) {
            der.writeString(dateToGTime(date), asn1.Ber.GeneralizedTime);
        }
        else {
            der.writeString(dateToUTCTime(date), asn1.Ber.UTCTime);
        }
    }
    /* RFC5280, section 4.2.1.6 (GeneralName type) */
    var ALTNAME = {
        OtherName: Local(0),
        RFC822Name: Context(1),
        DNSName: Context(2),
        X400Address: Local(3),
        DirectoryName: Local(4),
        EDIPartyName: Local(5),
        URI: Context(6),
        IPAddress: Context(7),
        OID: Context(8)
    };
    /* RFC5280, section 4.2.1.12 (KeyPurposeId) */
    var EXTPURPOSE = {
        'serverAuth': '1.3.6.1.5.5.7.3.1',
        'clientAuth': '1.3.6.1.5.5.7.3.2',
        'codeSigning': '1.3.6.1.5.5.7.3.3',
        /* See https://github.com/joyent/oid-docs/blob/master/root.md */
        'joyentDocker': '1.3.6.1.4.1.38678.1.4.1',
        'joyentCmon': '1.3.6.1.4.1.38678.1.4.2'
    };
    var EXTPURPOSE_REV = {};
    Object.keys(EXTPURPOSE).forEach(function (k) {
        EXTPURPOSE_REV[EXTPURPOSE[k]] = k;
    });
    var KEYUSEBITS = [
        'signature', 'identity', 'keyEncryption',
        'encryption', 'keyAgreement', 'ca', 'crl'
    ];
    function readExtension(cert, buf, der) {
        der.readSequence();
        var after = der.offset + der.length;
        var extId = der.readOID();
        var id;
        var sig = cert.signatures.x509;
        if (!sig.extras.exts)
            sig.extras.exts = [];
        var critical;
        if (der.peek() === asn1.Ber.Boolean)
            critical = der.readBoolean();
        switch (extId) {
            case (EXTS.basicConstraints):
                der.readSequence(asn1.Ber.OctetString);
                der.readSequence();
                var bcEnd = der.offset + der.length;
                var ca = false;
                if (der.peek() === asn1.Ber.Boolean)
                    ca = der.readBoolean();
                if (cert.purposes === undefined)
                    cert.purposes = [];
                if (ca === true)
                    cert.purposes.push('ca');
                var bc = { oid: extId, critical: critical };
                if (der.offset < bcEnd && der.peek() === asn1.Ber.Integer)
                    bc.pathLen = der.readInt();
                sig.extras.exts.push(bc);
                break;
            case (EXTS.extKeyUsage):
                der.readSequence(asn1.Ber.OctetString);
                der.readSequence();
                if (cert.purposes === undefined)
                    cert.purposes = [];
                var ekEnd = der.offset + der.length;
                while (der.offset < ekEnd) {
                    var oid = der.readOID();
                    cert.purposes.push(EXTPURPOSE_REV[oid] || oid);
                }
                /*
                 * This is a bit of a hack: in the case where we have a cert
                 * that's only allowed to do serverAuth or clientAuth (and not
                 * the other), we want to make sure all our Subjects are of
                 * the right type. But we already parsed our Subjects and
                 * decided if they were hosts or users earlier (since it appears
                 * first in the cert).
                 *
                 * So we go through and mutate them into the right kind here if
                 * it doesn't match. This might not be hugely beneficial, as it
                 * seems that single-purpose certs are not often seen in the
                 * wild.
                 */
                if (cert.purposes.indexOf('serverAuth') !== -1 &&
                    cert.purposes.indexOf('clientAuth') === -1) {
                    cert.subjects.forEach(function (ide) {
                        if (ide.type !== 'host') {
                            ide.type = 'host';
                            ide.hostname = ide.uid ||
                                ide.email ||
                                ide.components[0].value;
                        }
                    });
                }
                else if (cert.purposes.indexOf('clientAuth') !== -1 &&
                    cert.purposes.indexOf('serverAuth') === -1) {
                    cert.subjects.forEach(function (ide) {
                        if (ide.type !== 'user') {
                            ide.type = 'user';
                            ide.uid = ide.hostname ||
                                ide.email ||
                                ide.components[0].value;
                        }
                    });
                }
                sig.extras.exts.push({ oid: extId, critical: critical });
                break;
            case (EXTS.keyUsage):
                der.readSequence(asn1.Ber.OctetString);
                var bits = der.readString(asn1.Ber.BitString, true);
                var setBits = readBitField(bits, KEYUSEBITS);
                setBits.forEach(function (bit) {
                    if (cert.purposes === undefined)
                        cert.purposes = [];
                    if (cert.purposes.indexOf(bit) === -1)
                        cert.purposes.push(bit);
                });
                sig.extras.exts.push({ oid: extId, critical: critical,
                    bits: bits });
                break;
            case (EXTS.altName):
                der.readSequence(asn1.Ber.OctetString);
                der.readSequence();
                var aeEnd = der.offset + der.length;
                while (der.offset < aeEnd) {
                    switch (der.peek()) {
                        case ALTNAME.OtherName:
                        case ALTNAME.EDIPartyName:
                            der.readSequence();
                            der._offset += der.length;
                            break;
                        case ALTNAME.OID:
                            der.readOID(ALTNAME.OID);
                            break;
                        case ALTNAME.RFC822Name:
                            /* RFC822 specifies email addresses */
                            var email = der.readString(ALTNAME.RFC822Name);
                            id = Identity.forEmail(email);
                            if (!cert.subjects[0].equals(id))
                                cert.subjects.push(id);
                            break;
                        case ALTNAME.DirectoryName:
                            der.readSequence(ALTNAME.DirectoryName);
                            id = Identity.parseAsn1(der);
                            if (!cert.subjects[0].equals(id))
                                cert.subjects.push(id);
                            break;
                        case ALTNAME.DNSName:
                            var host = der.readString(ALTNAME.DNSName);
                            id = Identity.forHost(host);
                            if (!cert.subjects[0].equals(id))
                                cert.subjects.push(id);
                            break;
                        default:
                            der.readString(der.peek());
                            break;
                    }
                }
                sig.extras.exts.push({ oid: extId, critical: critical });
                break;
            default:
                sig.extras.exts.push({
                    oid: extId,
                    critical: critical,
                    data: der.readString(asn1.Ber.OctetString, true)
                });
                break;
        }
        der._offset = after;
    }
    var UTCTIME_RE = /^([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})?Z$/;
    function utcTimeToDate(t) {
        var m = t.match(UTCTIME_RE);
        assert.ok(m, 'timestamps must be in UTC');
        var d = new Date();
        var thisYear = d.getUTCFullYear();
        var century = Math.floor(thisYear / 100) * 100;
        var year = parseInt(m[1], 10);
        if (thisYear % 100 < 50 && year >= 60)
            year += (century - 1);
        else
            year += century;
        d.setUTCFullYear(year, parseInt(m[2], 10) - 1, parseInt(m[3], 10));
        d.setUTCHours(parseInt(m[4], 10), parseInt(m[5], 10));
        if (m[6] && m[6].length > 0)
            d.setUTCSeconds(parseInt(m[6], 10));
        return (d);
    }
    var GTIME_RE = /^([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})?Z$/;
    function gTimeToDate(t) {
        var m = t.match(GTIME_RE);
        assert.ok(m);
        var d = new Date();
        d.setUTCFullYear(parseInt(m[1], 10), parseInt(m[2], 10) - 1, parseInt(m[3], 10));
        d.setUTCHours(parseInt(m[4], 10), parseInt(m[5], 10));
        if (m[6] && m[6].length > 0)
            d.setUTCSeconds(parseInt(m[6], 10));
        return (d);
    }
    function zeroPad(n, m) {
        if (m === undefined)
            m = 2;
        var s = '' + n;
        while (s.length < m)
            s = '0' + s;
        return (s);
    }
    function dateToUTCTime(d) {
        var s = '';
        s += zeroPad(d.getUTCFullYear() % 100);
        s += zeroPad(d.getUTCMonth() + 1);
        s += zeroPad(d.getUTCDate());
        s += zeroPad(d.getUTCHours());
        s += zeroPad(d.getUTCMinutes());
        s += zeroPad(d.getUTCSeconds());
        s += 'Z';
        return (s);
    }
    function dateToGTime(d) {
        var s = '';
        s += zeroPad(d.getUTCFullYear(), 4);
        s += zeroPad(d.getUTCMonth() + 1);
        s += zeroPad(d.getUTCDate());
        s += zeroPad(d.getUTCHours());
        s += zeroPad(d.getUTCMinutes());
        s += zeroPad(d.getUTCSeconds());
        s += 'Z';
        return (s);
    }
    function sign(cert, key) {
        if (cert.signatures.x509 === undefined)
            cert.signatures.x509 = {};
        var sig = cert.signatures.x509;
        sig.algo = key.type + '-' + key.defaultHashAlgorithm();
        if (SIGN_ALGS[sig.algo] === undefined)
            return (false);
        var der = new asn1.BerWriter();
        writeTBSCert(cert, der);
        var blob = der.buffer;
        sig.cache = blob;
        var signer = key.createSign();
        signer.write(blob);
        cert.signatures.x509.signature = signer.sign();
        return (true);
    }
    function signAsync(cert, signer, done) {
        if (cert.signatures.x509 === undefined)
            cert.signatures.x509 = {};
        var sig = cert.signatures.x509;
        var der = new asn1.BerWriter();
        writeTBSCert(cert, der);
        var blob = der.buffer;
        sig.cache = blob;
        signer(blob, function (err, signature) {
            if (err) {
                done(err);
                return;
            }
            sig.algo = signature.type + '-' + signature.hashAlgorithm;
            if (SIGN_ALGS[sig.algo] === undefined) {
                done(new Error('Invalid signing algorithm "' +
                    sig.algo + '"'));
                return;
            }
            sig.signature = signature;
            done();
        });
    }
    function write(cert, options) {
        var sig = cert.signatures.x509;
        assert.object(sig, 'x509 signature');
        var der = new asn1.BerWriter();
        der.startSequence();
        if (sig.cache) {
            der._ensure(sig.cache.length);
            sig.cache.copy(der._buf, der._offset);
            der._offset += sig.cache.length;
        }
        else {
            writeTBSCert(cert, der);
        }
        der.startSequence();
        der.writeOID(SIGN_ALGS[sig.algo]);
        if (sig.algo.match(/^rsa-/))
            der.writeNull();
        der.endSequence();
        var sigData = sig.signature.toBuffer('asn1');
        var data = Buffer.alloc(sigData.length + 1);
        data[0] = 0;
        sigData.copy(data, 1);
        der.writeBuffer(data, asn1.Ber.BitString);
        der.endSequence();
        return (der.buffer);
    }
    function writeTBSCert(cert, der) {
        var sig = cert.signatures.x509;
        assert.object(sig, 'x509 signature');
        der.startSequence();
        der.startSequence(Local(0));
        der.writeInt(2);
        der.endSequence();
        der.writeBuffer(utils.mpNormalize(cert.serial), asn1.Ber.Integer);
        der.startSequence();
        der.writeOID(SIGN_ALGS[sig.algo]);
        if (sig.algo.match(/^rsa-/))
            der.writeNull();
        der.endSequence();
        cert.issuer.toAsn1(der);
        der.startSequence();
        writeDate(der, cert.validFrom);
        writeDate(der, cert.validUntil);
        der.endSequence();
        var subject = cert.subjects[0];
        var altNames = cert.subjects.slice(1);
        subject.toAsn1(der);
        pkcs8.writePkcs8(der, cert.subjectKey);
        if (sig.extras && sig.extras.issuerUniqueID) {
            der.writeBuffer(sig.extras.issuerUniqueID, Local(1));
        }
        if (sig.extras && sig.extras.subjectUniqueID) {
            der.writeBuffer(sig.extras.subjectUniqueID, Local(2));
        }
        if (altNames.length > 0 || subject.type === 'host' ||
            (cert.purposes !== undefined && cert.purposes.length > 0) ||
            (sig.extras && sig.extras.exts)) {
            der.startSequence(Local(3));
            der.startSequence();
            var exts = [];
            if (cert.purposes !== undefined && cert.purposes.length > 0) {
                exts.push({
                    oid: EXTS.basicConstraints,
                    critical: true
                });
                exts.push({
                    oid: EXTS.keyUsage,
                    critical: true
                });
                exts.push({
                    oid: EXTS.extKeyUsage,
                    critical: true
                });
            }
            exts.push({ oid: EXTS.altName });
            if (sig.extras && sig.extras.exts)
                exts = sig.extras.exts;
            for (var i = 0; i < exts.length; ++i) {
                der.startSequence();
                der.writeOID(exts[i].oid);
                if (exts[i].critical !== undefined)
                    der.writeBoolean(exts[i].critical);
                if (exts[i].oid === EXTS.altName) {
                    der.startSequence(asn1.Ber.OctetString);
                    der.startSequence();
                    if (subject.type === 'host') {
                        der.writeString(subject.hostname, Context(2));
                    }
                    for (var j = 0; j < altNames.length; ++j) {
                        if (altNames[j].type === 'host') {
                            der.writeString(altNames[j].hostname, ALTNAME.DNSName);
                        }
                        else if (altNames[j].type ===
                            'email') {
                            der.writeString(altNames[j].email, ALTNAME.RFC822Name);
                        }
                        else {
                            /*
                             * Encode anything else as a
                             * DN style name for now.
                             */
                            der.startSequence(ALTNAME.DirectoryName);
                            altNames[j].toAsn1(der);
                            der.endSequence();
                        }
                    }
                    der.endSequence();
                    der.endSequence();
                }
                else if (exts[i].oid === EXTS.basicConstraints) {
                    der.startSequence(asn1.Ber.OctetString);
                    der.startSequence();
                    var ca = (cert.purposes.indexOf('ca') !== -1);
                    var pathLen = exts[i].pathLen;
                    der.writeBoolean(ca);
                    if (pathLen !== undefined)
                        der.writeInt(pathLen);
                    der.endSequence();
                    der.endSequence();
                }
                else if (exts[i].oid === EXTS.extKeyUsage) {
                    der.startSequence(asn1.Ber.OctetString);
                    der.startSequence();
                    cert.purposes.forEach(function (purpose) {
                        if (purpose === 'ca')
                            return;
                        if (KEYUSEBITS.indexOf(purpose) !== -1)
                            return;
                        var oid = purpose;
                        if (EXTPURPOSE[purpose] !== undefined)
                            oid = EXTPURPOSE[purpose];
                        der.writeOID(oid);
                    });
                    der.endSequence();
                    der.endSequence();
                }
                else if (exts[i].oid === EXTS.keyUsage) {
                    der.startSequence(asn1.Ber.OctetString);
                    /*
                     * If we parsed this certificate from a byte
                     * stream (i.e. we didn't generate it in sshpk)
                     * then we'll have a ".bits" property on the
                     * ext with the original raw byte contents.
                     *
                     * If we have this, use it here instead of
                     * regenerating it. This guarantees we output
                     * the same data we parsed, so signatures still
                     * validate.
                     */
                    if (exts[i].bits !== undefined) {
                        der.writeBuffer(exts[i].bits, asn1.Ber.BitString);
                    }
                    else {
                        var bits = writeBitField(cert.purposes, KEYUSEBITS);
                        der.writeBuffer(bits, asn1.Ber.BitString);
                    }
                    der.endSequence();
                }
                else {
                    der.writeBuffer(exts[i].data, asn1.Ber.OctetString);
                }
                der.endSequence();
            }
            der.endSequence();
            der.endSequence();
        }
        der.endSequence();
    }
    /*
     * Reads an ASN.1 BER bitfield out of the Buffer produced by doing
     * `BerReader#readString(asn1.Ber.BitString)`. That function gives us the raw
     * contents of the BitString tag, which is a count of unused bits followed by
     * the bits as a right-padded byte string.
     *
     * `bits` is the Buffer, `bitIndex` should contain an array of string names
     * for the bits in the string, ordered starting with bit #0 in the ASN.1 spec.
     *
     * Returns an array of Strings, the names of the bits that were set to 1.
     */
    function readBitField(bits, bitIndex) {
        var bitLen = 8 * (bits.length - 1) - bits[0];
        var setBits = {};
        for (var i = 0; i < bitLen; ++i) {
            var byteN = 1 + Math.floor(i / 8);
            var bit = 7 - (i % 8);
            var mask = 1 << bit;
            var bitVal = ((bits[byteN] & mask) !== 0);
            var name = bitIndex[i];
            if (bitVal && typeof (name) === 'string') {
                setBits[name] = true;
            }
        }
        return (Object.keys(setBits));
    }
    /*
     * `setBits` is an array of strings, containing the names for each bit that
     * sould be set to 1. `bitIndex` is same as in `readBitField()`.
     *
     * Returns a Buffer, ready to be written out with `BerWriter#writeString()`.
     */
    function writeBitField(setBits, bitIndex) {
        var bitLen = bitIndex.length;
        var blen = Math.ceil(bitLen / 8);
        var unused = blen * 8 - bitLen;
        var bits = Buffer.alloc(1 + blen); // zero-filled
        bits[0] = unused;
        for (var i = 0; i < bitLen; ++i) {
            var byteN = 1 + Math.floor(i / 8);
            var bit = 7 - (i % 8);
            var mask = 1 << bit;
            var name = bitIndex[i];
            if (name === undefined)
                continue;
            var bitVal = (setBits.indexOf(name) !== -1);
            if (bitVal) {
                bits[byteN] |= mask;
            }
        }
        return (bits);
    }
},
/* sshpk/lib/formats/x509-pem.js */ function _(require, module, exports) {
    // Copyright 2016 Joyent, Inc.
    var x509 = require(891) /* ./x509 */;
    module.exports = {
        read: read,
        verify: x509.verify,
        sign: x509.sign,
        write: write
    };
    var assert = require(853) /* assert-plus */;
    var asn1 = require(864) /* asn1 */;
    var Buffer = require(858) /* safer-buffer */.Buffer;
    var algs = require(857) /* ../algs */;
    var utils = require(863) /* ../utils */;
    var Key = require(856) /* ../key */;
    var PrivateKey = require(861) /* ../private-key */;
    var pem = require(879) /* ./pem */;
    var Identity = require(889) /* ../identity */;
    var Signature = require(862) /* ../signature */;
    var Certificate = require(888) /* ../certificate */;
    function read(buf, options) {
        if (typeof (buf) !== 'string') {
            assert.buffer(buf, 'buf');
            buf = buf.toString('ascii');
        }
        var lines = buf.trim().split(/[\r\n]+/g);
        var m;
        var si = -1;
        while (!m && si < lines.length) {
            m = lines[++si].match(/*JSSTYLED*/ /[-]+[ ]*BEGIN CERTIFICATE[ ]*[-]+/);
        }
        assert.ok(m, 'invalid PEM header');
        var m2;
        var ei = lines.length;
        while (!m2 && ei > 0) {
            m2 = lines[--ei].match(/*JSSTYLED*/ /[-]+[ ]*END CERTIFICATE[ ]*[-]+/);
        }
        assert.ok(m2, 'invalid PEM footer');
        lines = lines.slice(si, ei + 1);
        var headers = {};
        while (true) {
            lines = lines.slice(1);
            m = lines[0].match(/*JSSTYLED*/ /^([A-Za-z0-9-]+): (.+)$/);
            if (!m)
                break;
            headers[m[1].toLowerCase()] = m[2];
        }
        /* Chop off the first and last lines */
        lines = lines.slice(0, -1).join('');
        buf = Buffer.from(lines, 'base64');
        return (x509.read(buf, options));
    }
    function write(cert, options) {
        var dbuf = x509.write(cert, options);
        var header = 'CERTIFICATE';
        var tmp = dbuf.toString('base64');
        var len = tmp.length + (tmp.length / 64) +
            18 + 16 + header.length * 2 + 10;
        var buf = Buffer.alloc(len);
        var o = 0;
        o += buf.write('-----BEGIN ' + header + '-----\n', o);
        for (var i = 0; i < tmp.length;) {
            var limit = i + 64;
            if (limit > tmp.length)
                limit = tmp.length;
            o += buf.write(tmp.slice(i, limit), o);
            buf[o++] = 10;
            i = limit;
        }
        o += buf.write('-----END ' + header + '-----\n', o);
        return (buf.slice(0, o));
    }
},
/* http-signature/lib/signer.js */ function _(require, module, exports) {
    // Copyright 2012 Joyent, Inc.  All rights reserved.
    var assert = require(853) /* assert-plus */;
    var crypto = require('crypto');
    var http = require('http');
    var util = require('util');
    var sshpk = require(855) /* sshpk */;
    var jsprim = require(894) /* jsprim */;
    var utils = require(854) /* ./utils */;
    var sprintf = require('util').format;
    var HASH_ALGOS = utils.HASH_ALGOS;
    var PK_ALGOS = utils.PK_ALGOS;
    var InvalidAlgorithmError = utils.InvalidAlgorithmError;
    var HttpSignatureError = utils.HttpSignatureError;
    var validateAlgorithm = utils.validateAlgorithm;
    ///--- Globals
    var AUTHZ_FMT = 'Signature keyId="%s",algorithm="%s",headers="%s",signature="%s"';
    ///--- Specific Errors
    function MissingHeaderError(message) {
        HttpSignatureError.call(this, message, MissingHeaderError);
    }
    util.inherits(MissingHeaderError, HttpSignatureError);
    function StrictParsingError(message) {
        HttpSignatureError.call(this, message, StrictParsingError);
    }
    util.inherits(StrictParsingError, HttpSignatureError);
    /* See createSigner() */
    function RequestSigner(options) {
        assert.object(options, 'options');
        var alg = [];
        if (options.algorithm !== undefined) {
            assert.string(options.algorithm, 'options.algorithm');
            alg = validateAlgorithm(options.algorithm);
        }
        this.rs_alg = alg;
        /*
         * RequestSigners come in two varieties: ones with an rs_signFunc, and ones
         * with an rs_signer.
         *
         * rs_signFunc-based RequestSigners have to build up their entire signing
         * string within the rs_lines array and give it to rs_signFunc as a single
         * concat'd blob. rs_signer-based RequestSigners can add a line at a time to
         * their signing state by using rs_signer.update(), thus only needing to
         * buffer the hash function state and one line at a time.
         */
        if (options.sign !== undefined) {
            assert.func(options.sign, 'options.sign');
            this.rs_signFunc = options.sign;
        }
        else if (alg[0] === 'hmac' && options.key !== undefined) {
            assert.string(options.keyId, 'options.keyId');
            this.rs_keyId = options.keyId;
            if (typeof (options.key) !== 'string' && !Buffer.isBuffer(options.key))
                throw (new TypeError('options.key for HMAC must be a string or Buffer'));
            /*
             * Make an rs_signer for HMACs, not a rs_signFunc -- HMACs digest their
             * data in chunks rather than requiring it all to be given in one go
             * at the end, so they are more similar to signers than signFuncs.
             */
            this.rs_signer = crypto.createHmac(alg[1].toUpperCase(), options.key);
            this.rs_signer.sign = function () {
                var digest = this.digest('base64');
                return ({
                    hashAlgorithm: alg[1],
                    toString: function () { return (digest); }
                });
            };
        }
        else if (options.key !== undefined) {
            var key = options.key;
            if (typeof (key) === 'string' || Buffer.isBuffer(key))
                key = sshpk.parsePrivateKey(key);
            assert.ok(sshpk.PrivateKey.isPrivateKey(key, [1, 2]), 'options.key must be a sshpk.PrivateKey');
            this.rs_key = key;
            assert.string(options.keyId, 'options.keyId');
            this.rs_keyId = options.keyId;
            if (!PK_ALGOS[key.type]) {
                throw (new InvalidAlgorithmError(key.type.toUpperCase() + ' type ' +
                    'keys are not supported'));
            }
            if (alg[0] !== undefined && key.type !== alg[0]) {
                throw (new InvalidAlgorithmError('options.key must be a ' +
                    alg[0].toUpperCase() + ' key, was given a ' +
                    key.type.toUpperCase() + ' key instead'));
            }
            this.rs_signer = key.createSign(alg[1]);
        }
        else {
            throw (new TypeError('options.sign (func) or options.key is required'));
        }
        this.rs_headers = [];
        this.rs_lines = [];
    }
    /**
     * Adds a header to be signed, with its value, into this signer.
     *
     * @param {String} header
     * @param {String} value
     * @return {String} value written
     */
    RequestSigner.prototype.writeHeader = function (header, value) {
        assert.string(header, 'header');
        header = header.toLowerCase();
        assert.string(value, 'value');
        this.rs_headers.push(header);
        if (this.rs_signFunc) {
            this.rs_lines.push(header + ': ' + value);
        }
        else {
            var line = header + ': ' + value;
            if (this.rs_headers.length > 0)
                line = '\n' + line;
            this.rs_signer.update(line);
        }
        return (value);
    };
    /**
     * Adds a default Date header, returning its value.
     *
     * @return {String}
     */
    RequestSigner.prototype.writeDateHeader = function () {
        return (this.writeHeader('date', jsprim.rfc1123(new Date())));
    };
    /**
     * Adds the request target line to be signed.
     *
     * @param {String} method, HTTP method (e.g. 'get', 'post', 'put')
     * @param {String} path
     */
    RequestSigner.prototype.writeTarget = function (method, path) {
        assert.string(method, 'method');
        assert.string(path, 'path');
        method = method.toLowerCase();
        this.writeHeader('(request-target)', method + ' ' + path);
    };
    /**
     * Calculate the value for the Authorization header on this request
     * asynchronously.
     *
     * @param {Func} callback (err, authz)
     */
    RequestSigner.prototype.sign = function (cb) {
        assert.func(cb, 'callback');
        if (this.rs_headers.length < 1)
            throw (new Error('At least one header must be signed'));
        var alg, authz;
        if (this.rs_signFunc) {
            var data = this.rs_lines.join('\n');
            var self = this;
            this.rs_signFunc(data, function (err, sig) {
                if (err) {
                    cb(err);
                    return;
                }
                try {
                    assert.object(sig, 'signature');
                    assert.string(sig.keyId, 'signature.keyId');
                    assert.string(sig.algorithm, 'signature.algorithm');
                    assert.string(sig.signature, 'signature.signature');
                    alg = validateAlgorithm(sig.algorithm);
                    authz = sprintf(AUTHZ_FMT, sig.keyId, sig.algorithm, self.rs_headers.join(' '), sig.signature);
                }
                catch (e) {
                    cb(e);
                    return;
                }
                cb(null, authz);
            });
        }
        else {
            try {
                var sigObj = this.rs_signer.sign();
            }
            catch (e) {
                cb(e);
                return;
            }
            alg = (this.rs_alg[0] || this.rs_key.type) + '-' + sigObj.hashAlgorithm;
            var signature = sigObj.toString();
            authz = sprintf(AUTHZ_FMT, this.rs_keyId, alg, this.rs_headers.join(' '), signature);
            cb(null, authz);
        }
    };
    ///--- Exported API
    module.exports = {
        /**
         * Identifies whether a given object is a request signer or not.
         *
         * @param {Object} object, the object to identify
         * @returns {Boolean}
         */
        isSigner: function (obj) {
            if (typeof (obj) === 'object' && obj instanceof RequestSigner)
                return (true);
            return (false);
        },
        /**
         * Creates a request signer, used to asynchronously build a signature
         * for a request (does not have to be an http.ClientRequest).
         *
         * @param {Object} options, either:
         *                   - {String} keyId
         *                   - {String|Buffer} key
         *                   - {String} algorithm (optional, required for HMAC)
         *                 or:
         *                   - {Func} sign (data, cb)
         * @return {RequestSigner}
         */
        createSigner: function createSigner(options) {
            return (new RequestSigner(options));
        },
        /**
         * Adds an 'Authorization' header to an http.ClientRequest object.
         *
         * Note that this API will add a Date header if it's not already set. Any
         * other headers in the options.headers array MUST be present, or this
         * will throw.
         *
         * You shouldn't need to check the return type; it's just there if you want
         * to be pedantic.
         *
         * The optional flag indicates whether parsing should use strict enforcement
         * of the version draft-cavage-http-signatures-04 of the spec or beyond.
         * The default is to be loose and support
         * older versions for compatibility.
         *
         * @param {Object} request an instance of http.ClientRequest.
         * @param {Object} options signing parameters object:
         *                   - {String} keyId required.
         *                   - {String} key required (either a PEM or HMAC key).
         *                   - {Array} headers optional; defaults to ['date'].
         *                   - {String} algorithm optional (unless key is HMAC);
         *                              default is the same as the sshpk default
         *                              signing algorithm for the type of key given
         *                   - {String} httpVersion optional; defaults to '1.1'.
         *                   - {Boolean} strict optional; defaults to 'false'.
         * @return {Boolean} true if Authorization (and optionally Date) were added.
         * @throws {TypeError} on bad parameter types (input).
         * @throws {InvalidAlgorithmError} if algorithm was bad or incompatible with
         *                                 the given key.
         * @throws {sshpk.KeyParseError} if key was bad.
         * @throws {MissingHeaderError} if a header to be signed was specified but
         *                              was not present.
         */
        signRequest: function signRequest(request, options) {
            assert.object(request, 'request');
            assert.object(options, 'options');
            assert.optionalString(options.algorithm, 'options.algorithm');
            assert.string(options.keyId, 'options.keyId');
            assert.optionalArrayOfString(options.headers, 'options.headers');
            assert.optionalString(options.httpVersion, 'options.httpVersion');
            if (!request.getHeader('Date'))
                request.setHeader('Date', jsprim.rfc1123(new Date()));
            if (!options.headers)
                options.headers = ['date'];
            if (!options.httpVersion)
                options.httpVersion = '1.1';
            var alg = [];
            if (options.algorithm) {
                options.algorithm = options.algorithm.toLowerCase();
                alg = validateAlgorithm(options.algorithm);
            }
            var i;
            var stringToSign = '';
            for (i = 0; i < options.headers.length; i++) {
                if (typeof (options.headers[i]) !== 'string')
                    throw new TypeError('options.headers must be an array of Strings');
                var h = options.headers[i].toLowerCase();
                if (h === 'request-line') {
                    if (!options.strict) {
                        /**
                         * We allow headers from the older spec drafts if strict parsing isn't
                         * specified in options.
                         */
                        stringToSign +=
                            request.method + ' ' + request.path + ' HTTP/' +
                                options.httpVersion;
                    }
                    else {
                        /* Strict parsing doesn't allow older draft headers. */
                        throw (new StrictParsingError('request-line is not a valid header ' +
                            'with strict parsing enabled.'));
                    }
                }
                else if (h === '(request-target)') {
                    stringToSign +=
                        '(request-target): ' + request.method.toLowerCase() + ' ' +
                            request.path;
                }
                else {
                    var value = request.getHeader(h);
                    if (value === undefined || value === '') {
                        throw new MissingHeaderError(h + ' was not in the request');
                    }
                    stringToSign += h + ': ' + value;
                }
                if ((i + 1) < options.headers.length)
                    stringToSign += '\n';
            }
            /* This is just for unit tests. */
            if (request.hasOwnProperty('_stringToSign')) {
                request._stringToSign = stringToSign;
            }
            var signature;
            if (alg[0] === 'hmac') {
                if (typeof (options.key) !== 'string' && !Buffer.isBuffer(options.key))
                    throw (new TypeError('options.key must be a string or Buffer'));
                var hmac = crypto.createHmac(alg[1].toUpperCase(), options.key);
                hmac.update(stringToSign);
                signature = hmac.digest('base64');
            }
            else {
                var key = options.key;
                if (typeof (key) === 'string' || Buffer.isBuffer(key))
                    key = sshpk.parsePrivateKey(options.key);
                assert.ok(sshpk.PrivateKey.isPrivateKey(key, [1, 2]), 'options.key must be a sshpk.PrivateKey');
                if (!PK_ALGOS[key.type]) {
                    throw (new InvalidAlgorithmError(key.type.toUpperCase() + ' type ' +
                        'keys are not supported'));
                }
                if (alg[0] !== undefined && key.type !== alg[0]) {
                    throw (new InvalidAlgorithmError('options.key must be a ' +
                        alg[0].toUpperCase() + ' key, was given a ' +
                        key.type.toUpperCase() + ' key instead'));
                }
                var signer = key.createSign(alg[1]);
                signer.update(stringToSign);
                var sigObj = signer.sign();
                if (!HASH_ALGOS[sigObj.hashAlgorithm]) {
                    throw (new InvalidAlgorithmError(sigObj.hashAlgorithm.toUpperCase() +
                        ' is not a supported hash algorithm'));
                }
                options.algorithm = key.type + '-' + sigObj.hashAlgorithm;
                signature = sigObj.toString();
                assert.notStrictEqual(signature, '', 'empty signature produced');
            }
            var authzHeaderName = options.authorizationHeaderName || 'Authorization';
            request.setHeader(authzHeaderName, sprintf(AUTHZ_FMT, options.keyId, options.algorithm, options.headers.join(' '), signature));
            return true;
        }
    };
},
/* jsprim/lib/jsprim.js */ function _(require, module, exports) {
    /*
     * lib/jsprim.js: utilities for primitive JavaScript types
     */
    var mod_assert = require(853) /* assert-plus */;
    var mod_util = require('util');
    var mod_extsprintf = require(895) /* extsprintf */;
    var mod_verror = require(896) /* verror */;
    var mod_jsonschema = require(898) /* json-schema */;
    /*
     * Public interface
     */
    exports.deepCopy = deepCopy;
    exports.deepEqual = deepEqual;
    exports.isEmpty = isEmpty;
    exports.hasKey = hasKey;
    exports.forEachKey = forEachKey;
    exports.pluck = pluck;
    exports.flattenObject = flattenObject;
    exports.flattenIter = flattenIter;
    exports.validateJsonObject = validateJsonObjectJS;
    exports.validateJsonObjectJS = validateJsonObjectJS;
    exports.randElt = randElt;
    exports.extraProperties = extraProperties;
    exports.mergeObjects = mergeObjects;
    exports.startsWith = startsWith;
    exports.endsWith = endsWith;
    exports.parseInteger = parseInteger;
    exports.iso8601 = iso8601;
    exports.rfc1123 = rfc1123;
    exports.parseDateTime = parseDateTime;
    exports.hrtimediff = hrtimeDiff;
    exports.hrtimeDiff = hrtimeDiff;
    exports.hrtimeAccum = hrtimeAccum;
    exports.hrtimeAdd = hrtimeAdd;
    exports.hrtimeNanosec = hrtimeNanosec;
    exports.hrtimeMicrosec = hrtimeMicrosec;
    exports.hrtimeMillisec = hrtimeMillisec;
    /*
     * Deep copy an acyclic *basic* Javascript object.  This only handles basic
     * scalars (strings, numbers, booleans) and arbitrarily deep arrays and objects
     * containing these.  This does *not* handle instances of other classes.
     */
    function deepCopy(obj) {
        var ret, key;
        var marker = '__deepCopy';
        if (obj && obj[marker])
            throw (new Error('attempted deep copy of cyclic object'));
        if (obj && obj.constructor == Object) {
            ret = {};
            obj[marker] = true;
            for (key in obj) {
                if (key == marker)
                    continue;
                ret[key] = deepCopy(obj[key]);
            }
            delete (obj[marker]);
            return (ret);
        }
        if (obj && obj.constructor == Array) {
            ret = [];
            obj[marker] = true;
            for (key = 0; key < obj.length; key++)
                ret.push(deepCopy(obj[key]));
            delete (obj[marker]);
            return (ret);
        }
        /*
         * It must be a primitive type -- just return it.
         */
        return (obj);
    }
    function deepEqual(obj1, obj2) {
        if (typeof (obj1) != typeof (obj2))
            return (false);
        if (obj1 === null || obj2 === null || typeof (obj1) != 'object')
            return (obj1 === obj2);
        if (obj1.constructor != obj2.constructor)
            return (false);
        var k;
        for (k in obj1) {
            if (!obj2.hasOwnProperty(k))
                return (false);
            if (!deepEqual(obj1[k], obj2[k]))
                return (false);
        }
        for (k in obj2) {
            if (!obj1.hasOwnProperty(k))
                return (false);
        }
        return (true);
    }
    function isEmpty(obj) {
        var key;
        for (key in obj)
            return (false);
        return (true);
    }
    function hasKey(obj, key) {
        mod_assert.equal(typeof (key), 'string');
        return (Object.prototype.hasOwnProperty.call(obj, key));
    }
    function forEachKey(obj, callback) {
        for (var key in obj) {
            if (hasKey(obj, key)) {
                callback(key, obj[key]);
            }
        }
    }
    function pluck(obj, key) {
        mod_assert.equal(typeof (key), 'string');
        return (pluckv(obj, key));
    }
    function pluckv(obj, key) {
        if (obj === null || typeof (obj) !== 'object')
            return (undefined);
        if (obj.hasOwnProperty(key))
            return (obj[key]);
        var i = key.indexOf('.');
        if (i == -1)
            return (undefined);
        var key1 = key.substr(0, i);
        if (!obj.hasOwnProperty(key1))
            return (undefined);
        return (pluckv(obj[key1], key.substr(i + 1)));
    }
    /*
     * Invoke callback(row) for each entry in the array that would be returned by
     * flattenObject(data, depth).  This is just like flattenObject(data,
     * depth).forEach(callback), except that the intermediate array is never
     * created.
     */
    function flattenIter(data, depth, callback) {
        doFlattenIter(data, depth, [], callback);
    }
    function doFlattenIter(data, depth, accum, callback) {
        var each;
        var key;
        if (depth === 0) {
            each = accum.slice(0);
            each.push(data);
            callback(each);
            return;
        }
        mod_assert.ok(data !== null);
        mod_assert.equal(typeof (data), 'object');
        mod_assert.equal(typeof (depth), 'number');
        mod_assert.ok(depth >= 0);
        for (key in data) {
            each = accum.slice(0);
            each.push(key);
            doFlattenIter(data[key], depth - 1, each, callback);
        }
    }
    function flattenObject(data, depth) {
        if (depth === 0)
            return ([data]);
        mod_assert.ok(data !== null);
        mod_assert.equal(typeof (data), 'object');
        mod_assert.equal(typeof (depth), 'number');
        mod_assert.ok(depth >= 0);
        var rv = [];
        var key;
        for (key in data) {
            flattenObject(data[key], depth - 1).forEach(function (p) {
                rv.push([key].concat(p));
            });
        }
        return (rv);
    }
    function startsWith(str, prefix) {
        return (str.substr(0, prefix.length) == prefix);
    }
    function endsWith(str, suffix) {
        return (str.substr(str.length - suffix.length, suffix.length) == suffix);
    }
    function iso8601(d) {
        if (typeof (d) == 'number')
            d = new Date(d);
        mod_assert.ok(d.constructor === Date);
        return (mod_extsprintf.sprintf('%4d-%02d-%02dT%02d:%02d:%02d.%03dZ', d.getUTCFullYear(), d.getUTCMonth() + 1, d.getUTCDate(), d.getUTCHours(), d.getUTCMinutes(), d.getUTCSeconds(), d.getUTCMilliseconds()));
    }
    var RFC1123_MONTHS = [
        'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
        'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'
    ];
    var RFC1123_DAYS = [
        'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'
    ];
    function rfc1123(date) {
        return (mod_extsprintf.sprintf('%s, %02d %s %04d %02d:%02d:%02d GMT', RFC1123_DAYS[date.getUTCDay()], date.getUTCDate(), RFC1123_MONTHS[date.getUTCMonth()], date.getUTCFullYear(), date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds()));
    }
    /*
     * Parses a date expressed as a string, as either a number of milliseconds since
     * the epoch or any string format that Date accepts, giving preference to the
     * former where these two sets overlap (e.g., small numbers).
     */
    function parseDateTime(str) {
        /*
         * This is irritatingly implicit, but significantly more concise than
         * alternatives.  The "+str" will convert a string containing only a
         * number directly to a Number, or NaN for other strings.  Thus, if the
         * conversion succeeds, we use it (this is the milliseconds-since-epoch
         * case).  Otherwise, we pass the string directly to the Date
         * constructor to parse.
         */
        var numeric = +str;
        if (!isNaN(numeric)) {
            return (new Date(numeric));
        }
        else {
            return (new Date(str));
        }
    }
    /*
     * Number.*_SAFE_INTEGER isn't present before node v0.12, so we hardcode
     * the ES6 definitions here, while allowing for them to someday be higher.
     */
    var MAX_SAFE_INTEGER = Number.MAX_SAFE_INTEGER || 9007199254740991;
    var MIN_SAFE_INTEGER = Number.MIN_SAFE_INTEGER || -9007199254740991;
    /*
     * Default options for parseInteger().
     */
    var PI_DEFAULTS = {
        base: 10,
        allowSign: true,
        allowPrefix: false,
        allowTrailing: false,
        allowImprecise: false,
        trimWhitespace: false,
        leadingZeroIsOctal: false
    };
    var CP_0 = 0x30;
    var CP_9 = 0x39;
    var CP_A = 0x41;
    var CP_B = 0x42;
    var CP_O = 0x4f;
    var CP_T = 0x54;
    var CP_X = 0x58;
    var CP_Z = 0x5a;
    var CP_a = 0x61;
    var CP_b = 0x62;
    var CP_o = 0x6f;
    var CP_t = 0x74;
    var CP_x = 0x78;
    var CP_z = 0x7a;
    var PI_CONV_DEC = 0x30;
    var PI_CONV_UC = 0x37;
    var PI_CONV_LC = 0x57;
    /*
     * A stricter version of parseInt() that provides options for changing what
     * is an acceptable string (for example, disallowing trailing characters).
     */
    function parseInteger(str, uopts) {
        mod_assert.string(str, 'str');
        mod_assert.optionalObject(uopts, 'options');
        var baseOverride = false;
        var options = PI_DEFAULTS;
        if (uopts) {
            baseOverride = hasKey(uopts, 'base');
            options = mergeObjects(options, uopts);
            mod_assert.number(options.base, 'options.base');
            mod_assert.ok(options.base >= 2, 'options.base >= 2');
            mod_assert.ok(options.base <= 36, 'options.base <= 36');
            mod_assert.bool(options.allowSign, 'options.allowSign');
            mod_assert.bool(options.allowPrefix, 'options.allowPrefix');
            mod_assert.bool(options.allowTrailing, 'options.allowTrailing');
            mod_assert.bool(options.allowImprecise, 'options.allowImprecise');
            mod_assert.bool(options.trimWhitespace, 'options.trimWhitespace');
            mod_assert.bool(options.leadingZeroIsOctal, 'options.leadingZeroIsOctal');
            if (options.leadingZeroIsOctal) {
                mod_assert.ok(!baseOverride, '"base" and "leadingZeroIsOctal" are ' +
                    'mutually exclusive');
            }
        }
        var c;
        var pbase = -1;
        var base = options.base;
        var start;
        var mult = 1;
        var value = 0;
        var idx = 0;
        var len = str.length;
        /* Trim any whitespace on the left side. */
        if (options.trimWhitespace) {
            while (idx < len && isSpace(str.charCodeAt(idx))) {
                ++idx;
            }
        }
        /* Check the number for a leading sign. */
        if (options.allowSign) {
            if (str[idx] === '-') {
                idx += 1;
                mult = -1;
            }
            else if (str[idx] === '+') {
                idx += 1;
            }
        }
        /* Parse the base-indicating prefix if there is one. */
        if (str[idx] === '0') {
            if (options.allowPrefix) {
                pbase = prefixToBase(str.charCodeAt(idx + 1));
                if (pbase !== -1 && (!baseOverride || pbase === base)) {
                    base = pbase;
                    idx += 2;
                }
            }
            if (pbase === -1 && options.leadingZeroIsOctal) {
                base = 8;
            }
        }
        /* Parse the actual digits. */
        for (start = idx; idx < len; ++idx) {
            c = translateDigit(str.charCodeAt(idx));
            if (c !== -1 && c < base) {
                value *= base;
                value += c;
            }
            else {
                break;
            }
        }
        /* If we didn't parse any digits, we have an invalid number. */
        if (start === idx) {
            return (new Error('invalid number: ' + JSON.stringify(str)));
        }
        /* Trim any whitespace on the right side. */
        if (options.trimWhitespace) {
            while (idx < len && isSpace(str.charCodeAt(idx))) {
                ++idx;
            }
        }
        /* Check for trailing characters. */
        if (idx < len && !options.allowTrailing) {
            return (new Error('trailing characters after number: ' +
                JSON.stringify(str.slice(idx))));
        }
        /* If our value is 0, we return now, to avoid returning -0. */
        if (value === 0) {
            return (0);
        }
        /* Calculate our final value. */
        var result = value * mult;
        /*
         * If the string represents a value that cannot be precisely represented
         * by JavaScript, then we want to check that:
         *
         * - We never increased the value past MAX_SAFE_INTEGER
         * - We don't make the result negative and below MIN_SAFE_INTEGER
         *
         * Because we only ever increment the value during parsing, there's no
         * chance of moving past MAX_SAFE_INTEGER and then dropping below it
         * again, losing precision in the process. This means that we only need
         * to do our checks here, at the end.
         */
        if (!options.allowImprecise &&
            (value > MAX_SAFE_INTEGER || result < MIN_SAFE_INTEGER)) {
            return (new Error('number is outside of the supported range: ' +
                JSON.stringify(str.slice(start, idx))));
        }
        return (result);
    }
    /*
     * Interpret a character code as a base-36 digit.
     */
    function translateDigit(d) {
        if (d >= CP_0 && d <= CP_9) {
            /* '0' to '9' -> 0 to 9 */
            return (d - PI_CONV_DEC);
        }
        else if (d >= CP_A && d <= CP_Z) {
            /* 'A' - 'Z' -> 10 to 35 */
            return (d - PI_CONV_UC);
        }
        else if (d >= CP_a && d <= CP_z) {
            /* 'a' - 'z' -> 10 to 35 */
            return (d - PI_CONV_LC);
        }
        else {
            /* Invalid character code */
            return (-1);
        }
    }
    /*
     * Test if a value matches the ECMAScript definition of trimmable whitespace.
     */
    function isSpace(c) {
        return (c === 0x20) ||
            (c >= 0x0009 && c <= 0x000d) ||
            (c === 0x00a0) ||
            (c === 0x1680) ||
            (c === 0x180e) ||
            (c >= 0x2000 && c <= 0x200a) ||
            (c === 0x2028) ||
            (c === 0x2029) ||
            (c === 0x202f) ||
            (c === 0x205f) ||
            (c === 0x3000) ||
            (c === 0xfeff);
    }
    /*
     * Determine which base a character indicates (e.g., 'x' indicates hex).
     */
    function prefixToBase(c) {
        if (c === CP_b || c === CP_B) {
            /* 0b/0B (binary) */
            return (2);
        }
        else if (c === CP_o || c === CP_O) {
            /* 0o/0O (octal) */
            return (8);
        }
        else if (c === CP_t || c === CP_T) {
            /* 0t/0T (decimal) */
            return (10);
        }
        else if (c === CP_x || c === CP_X) {
            /* 0x/0X (hexadecimal) */
            return (16);
        }
        else {
            /* Not a meaningful character */
            return (-1);
        }
    }
    function validateJsonObjectJS(schema, input) {
        var report = mod_jsonschema.validate(input, schema);
        if (report.errors.length === 0)
            return (null);
        /* Currently, we only do anything useful with the first error. */
        var error = report.errors[0];
        /* The failed property is given by a URI with an irrelevant prefix. */
        var propname = error['property'];
        var reason = error['message'].toLowerCase();
        var i, j;
        /*
         * There's at least one case where the property error message is
         * confusing at best.  We work around this here.
         */
        if ((i = reason.indexOf('the property ')) != -1 &&
            (j = reason.indexOf(' is not defined in the schema and the ' +
                'schema does not allow additional properties')) != -1) {
            i += 'the property '.length;
            if (propname === '')
                propname = reason.substr(i, j - i);
            else
                propname = propname + '.' + reason.substr(i, j - i);
            reason = 'unsupported property';
        }
        var rv = new mod_verror.VError('property "%s": %s', propname, reason);
        rv.jsv_details = error;
        return (rv);
    }
    function randElt(arr) {
        mod_assert.ok(Array.isArray(arr) && arr.length > 0, 'randElt argument must be a non-empty array');
        return (arr[Math.floor(Math.random() * arr.length)]);
    }
    function assertHrtime(a) {
        mod_assert.ok(a[0] >= 0 && a[1] >= 0, 'negative numbers not allowed in hrtimes');
        mod_assert.ok(a[1] < 1e9, 'nanoseconds column overflow');
    }
    /*
     * Compute the time elapsed between hrtime readings A and B, where A is later
     * than B.  hrtime readings come from Node's process.hrtime().  There is no
     * defined way to represent negative deltas, so it's illegal to diff B from A
     * where the time denoted by B is later than the time denoted by A.  If this
     * becomes valuable, we can define a representation and extend the
     * implementation to support it.
     */
    function hrtimeDiff(a, b) {
        assertHrtime(a);
        assertHrtime(b);
        mod_assert.ok(a[0] > b[0] || (a[0] == b[0] && a[1] >= b[1]), 'negative differences not allowed');
        var rv = [a[0] - b[0], 0];
        if (a[1] >= b[1]) {
            rv[1] = a[1] - b[1];
        }
        else {
            rv[0]--;
            rv[1] = 1e9 - (b[1] - a[1]);
        }
        return (rv);
    }
    /*
     * Convert a hrtime reading from the array format returned by Node's
     * process.hrtime() into a scalar number of nanoseconds.
     */
    function hrtimeNanosec(a) {
        assertHrtime(a);
        return (Math.floor(a[0] * 1e9 + a[1]));
    }
    /*
     * Convert a hrtime reading from the array format returned by Node's
     * process.hrtime() into a scalar number of microseconds.
     */
    function hrtimeMicrosec(a) {
        assertHrtime(a);
        return (Math.floor(a[0] * 1e6 + a[1] / 1e3));
    }
    /*
     * Convert a hrtime reading from the array format returned by Node's
     * process.hrtime() into a scalar number of milliseconds.
     */
    function hrtimeMillisec(a) {
        assertHrtime(a);
        return (Math.floor(a[0] * 1e3 + a[1] / 1e6));
    }
    /*
     * Add two hrtime readings A and B, overwriting A with the result of the
     * addition.  This function is useful for accumulating several hrtime intervals
     * into a counter.  Returns A.
     */
    function hrtimeAccum(a, b) {
        assertHrtime(a);
        assertHrtime(b);
        /*
         * Accumulate the nanosecond component.
         */
        a[1] += b[1];
        if (a[1] >= 1e9) {
            /*
             * The nanosecond component overflowed, so carry to the seconds
             * field.
             */
            a[0]++;
            a[1] -= 1e9;
        }
        /*
         * Accumulate the seconds component.
         */
        a[0] += b[0];
        return (a);
    }
    /*
     * Add two hrtime readings A and B, returning the result as a new hrtime array.
     * Does not modify either input argument.
     */
    function hrtimeAdd(a, b) {
        assertHrtime(a);
        var rv = [a[0], a[1]];
        return (hrtimeAccum(rv, b));
    }
    /*
     * Check an object for unexpected properties.  Accepts the object to check, and
     * an array of allowed property names (strings).  Returns an array of key names
     * that were found on the object, but did not appear in the list of allowed
     * properties.  If no properties were found, the returned array will be of
     * zero length.
     */
    function extraProperties(obj, allowed) {
        mod_assert.ok(typeof (obj) === 'object' && obj !== null, 'obj argument must be a non-null object');
        mod_assert.ok(Array.isArray(allowed), 'allowed argument must be an array of strings');
        for (var i = 0; i < allowed.length; i++) {
            mod_assert.ok(typeof (allowed[i]) === 'string', 'allowed argument must be an array of strings');
        }
        return (Object.keys(obj).filter(function (key) {
            return (allowed.indexOf(key) === -1);
        }));
    }
    /*
     * Given three sets of properties "provided" (may be undefined), "overrides"
     * (required), and "defaults" (may be undefined), construct an object containing
     * the union of these sets with "overrides" overriding "provided", and
     * "provided" overriding "defaults".  None of the input objects are modified.
     */
    function mergeObjects(provided, overrides, defaults) {
        var rv, k;
        rv = {};
        if (defaults) {
            for (k in defaults)
                rv[k] = defaults[k];
        }
        if (provided) {
            for (k in provided)
                rv[k] = provided[k];
        }
        if (overrides) {
            for (k in overrides)
                rv[k] = overrides[k];
        }
        return (rv);
    }
},
/* extsprintf/lib/extsprintf.js */ function _(require, module, exports) {
    /*
     * extsprintf.js: extended POSIX-style sprintf
     */
    var mod_assert = require('assert');
    var mod_util = require('util');
    /*
     * Public interface
     */
    exports.sprintf = jsSprintf;
    exports.printf = jsPrintf;
    exports.fprintf = jsFprintf;
    /*
     * Stripped down version of s[n]printf(3c).  We make a best effort to throw an
     * exception when given a format string we don't understand, rather than
     * ignoring it, so that we won't break existing programs if/when we go implement
     * the rest of this.
     *
     * This implementation currently supports specifying
     *	- field alignment ('-' flag),
     * 	- zero-pad ('0' flag)
     *	- always show numeric sign ('+' flag),
     *	- field width
     *	- conversions for strings, decimal integers, and floats (numbers).
     *	- argument size specifiers.  These are all accepted but ignored, since
     *	  Javascript has no notion of the physical size of an argument.
     *
     * Everything else is currently unsupported, most notably precision, unsigned
     * numbers, non-decimal numbers, and characters.
     */
    function jsSprintf(fmt) {
        var regex = [
            '([^%]*)',
            '%',
            '([\'\\-+ #0]*?)',
            '([1-9]\\d*)?',
            '(\\.([1-9]\\d*))?',
            '[lhjztL]*?',
            '([diouxXfFeEgGaAcCsSp%jr])' /* conversion */
        ].join('');
        var re = new RegExp(regex);
        var args = Array.prototype.slice.call(arguments, 1);
        var flags, width, precision, conversion;
        var left, pad, sign, arg, match;
        var ret = '';
        var argn = 1;
        mod_assert.equal('string', typeof (fmt));
        while ((match = re.exec(fmt)) !== null) {
            ret += match[1];
            fmt = fmt.substring(match[0].length);
            flags = match[2] || '';
            width = match[3] || 0;
            precision = match[4] || '';
            conversion = match[6];
            left = false;
            sign = false;
            pad = ' ';
            if (conversion == '%') {
                ret += '%';
                continue;
            }
            if (args.length === 0)
                throw (new Error('too few args to sprintf'));
            arg = args.shift();
            argn++;
            if (flags.match(/[\' #]/))
                throw (new Error('unsupported flags: ' + flags));
            if (precision.length > 0)
                throw (new Error('non-zero precision not supported'));
            if (flags.match(/-/))
                left = true;
            if (flags.match(/0/))
                pad = '0';
            if (flags.match(/\+/))
                sign = true;
            switch (conversion) {
                case 's':
                    if (arg === undefined || arg === null)
                        throw (new Error('argument ' + argn +
                            ': attempted to print undefined or null ' +
                            'as a string'));
                    ret += doPad(pad, width, left, arg.toString());
                    break;
                case 'd':
                    arg = Math.floor(arg);
                /*jsl:fallthru*/
                case 'f':
                    sign = sign && arg > 0 ? '+' : '';
                    ret += sign + doPad(pad, width, left, arg.toString());
                    break;
                case 'x':
                    ret += doPad(pad, width, left, arg.toString(16));
                    break;
                case 'j': /* non-standard */
                    if (width === 0)
                        width = 10;
                    ret += mod_util.inspect(arg, false, width);
                    break;
                case 'r': /* non-standard */
                    ret += dumpException(arg);
                    break;
                default:
                    throw (new Error('unsupported conversion: ' +
                        conversion));
            }
        }
        ret += fmt;
        return (ret);
    }
    function jsPrintf() {
        var args = Array.prototype.slice.call(arguments);
        args.unshift(process.stdout);
        jsFprintf.apply(null, args);
    }
    function jsFprintf(stream) {
        var args = Array.prototype.slice.call(arguments, 1);
        return (stream.write(jsSprintf.apply(this, args)));
    }
    function doPad(chr, width, left, str) {
        var ret = str;
        while (ret.length < width) {
            if (left)
                ret += chr;
            else
                ret = chr + ret;
        }
        return (ret);
    }
    /*
     * This function dumps long stack traces for exceptions having a cause() method.
     * See node-verror for an example.
     */
    function dumpException(ex) {
        var ret;
        if (!(ex instanceof Error))
            throw (new Error(jsSprintf('invalid type for %%r: %j', ex)));
        /* Note that V8 prepends "ex.stack" with ex.toString(). */
        ret = 'EXCEPTION: ' + ex.constructor.name + ': ' + ex.stack;
        if (ex.cause && typeof (ex.cause) === 'function') {
            var cex = ex.cause();
            if (cex) {
                ret += '\nCaused by: ' + dumpException(cex);
            }
        }
        return (ret);
    }
},
/* verror/lib/verror.js */ function _(require, module, exports) {
    /*
     * verror.js: richer JavaScript errors
     */
    var mod_assertplus = require(853) /* assert-plus */;
    var mod_util = require('util');
    var mod_extsprintf = require(895) /* extsprintf */;
    var mod_isError = require(897) /* core-util-is */.isError;
    var sprintf = mod_extsprintf.sprintf;
    /*
     * Public interface
     */
    /* So you can 'var VError = require('verror')' */
    module.exports = VError;
    /* For compatibility */
    VError.VError = VError;
    /* Other exported classes */
    VError.SError = SError;
    VError.WError = WError;
    VError.MultiError = MultiError;
    /*
     * Common function used to parse constructor arguments for VError, WError, and
     * SError.  Named arguments to this function:
     *
     *     strict		force strict interpretation of sprintf arguments, even
     *     			if the options in "argv" don't say so
     *
     *     argv		error's constructor arguments, which are to be
     *     			interpreted as described in README.md.  For quick
     *     			reference, "argv" has one of the following forms:
     *
     *          [ sprintf_args... ]           (argv[0] is a string)
     *          [ cause, sprintf_args... ]    (argv[0] is an Error)
     *          [ options, sprintf_args... ]  (argv[0] is an object)
     *
     * This function normalizes these forms, producing an object with the following
     * properties:
     *
     *    options           equivalent to "options" in third form.  This will never
     *    			be a direct reference to what the caller passed in
     *    			(i.e., it may be a shallow copy), so it can be freely
     *    			modified.
     *
     *    shortmessage      result of sprintf(sprintf_args), taking options.strict
     *    			into account as described in README.md.
     */
    function parseConstructorArguments(args) {
        var argv, options, sprintf_args, shortmessage, k;
        mod_assertplus.object(args, 'args');
        mod_assertplus.bool(args.strict, 'args.strict');
        mod_assertplus.array(args.argv, 'args.argv');
        argv = args.argv;
        /*
         * First, figure out which form of invocation we've been given.
         */
        if (argv.length === 0) {
            options = {};
            sprintf_args = [];
        }
        else if (mod_isError(argv[0])) {
            options = { 'cause': argv[0] };
            sprintf_args = argv.slice(1);
        }
        else if (typeof (argv[0]) === 'object') {
            options = {};
            for (k in argv[0]) {
                options[k] = argv[0][k];
            }
            sprintf_args = argv.slice(1);
        }
        else {
            mod_assertplus.string(argv[0], 'first argument to VError, SError, or WError ' +
                'constructor must be a string, object, or Error');
            options = {};
            sprintf_args = argv;
        }
        /*
         * Now construct the error's message.
         *
         * extsprintf (which we invoke here with our caller's arguments in order
         * to construct this Error's message) is strict in its interpretation of
         * values to be processed by the "%s" specifier.  The value passed to
         * extsprintf must actually be a string or something convertible to a
         * String using .toString().  Passing other values (notably "null" and
         * "undefined") is considered a programmer error.  The assumption is
         * that if you actually want to print the string "null" or "undefined",
         * then that's easy to do that when you're calling extsprintf; on the
         * other hand, if you did NOT want that (i.e., there's actually a bug
         * where the program assumes some variable is non-null and tries to
         * print it, which might happen when constructing a packet or file in
         * some specific format), then it's better to stop immediately than
         * produce bogus output.
         *
         * However, sometimes the bug is only in the code calling VError, and a
         * programmer might prefer to have the error message contain "null" or
         * "undefined" rather than have the bug in the error path crash the
         * program (making the first bug harder to identify).  For that reason,
         * by default VError converts "null" or "undefined" arguments to their
         * string representations and passes those to extsprintf.  Programmers
         * desiring the strict behavior can use the SError class or pass the
         * "strict" option to the VError constructor.
         */
        mod_assertplus.object(options);
        if (!options.strict && !args.strict) {
            sprintf_args = sprintf_args.map(function (a) {
                return (a === null ? 'null' :
                    a === undefined ? 'undefined' : a);
            });
        }
        if (sprintf_args.length === 0) {
            shortmessage = '';
        }
        else {
            shortmessage = sprintf.apply(null, sprintf_args);
        }
        return ({
            'options': options,
            'shortmessage': shortmessage
        });
    }
    /*
     * See README.md for reference documentation.
     */
    function VError() {
        var args, obj, parsed, cause, ctor, message, k;
        args = Array.prototype.slice.call(arguments, 0);
        /*
         * This is a regrettable pattern, but JavaScript's built-in Error class
         * is defined to work this way, so we allow the constructor to be called
         * without "new".
         */
        if (!(this instanceof VError)) {
            obj = Object.create(VError.prototype);
            VError.apply(obj, arguments);
            return (obj);
        }
        /*
         * For convenience and backwards compatibility, we support several
         * different calling forms.  Normalize them here.
         */
        parsed = parseConstructorArguments({
            'argv': args,
            'strict': false
        });
        /*
         * If we've been given a name, apply it now.
         */
        if (parsed.options.name) {
            mod_assertplus.string(parsed.options.name, 'error\'s "name" must be a string');
            this.name = parsed.options.name;
        }
        /*
         * For debugging, we keep track of the original short message (attached
         * this Error particularly) separately from the complete message (which
         * includes the messages of our cause chain).
         */
        this.jse_shortmsg = parsed.shortmessage;
        message = parsed.shortmessage;
        /*
         * If we've been given a cause, record a reference to it and update our
         * message appropriately.
         */
        cause = parsed.options.cause;
        if (cause) {
            mod_assertplus.ok(mod_isError(cause), 'cause is not an Error');
            this.jse_cause = cause;
            if (!parsed.options.skipCauseMessage) {
                message += ': ' + cause.message;
            }
        }
        /*
         * If we've been given an object with properties, shallow-copy that
         * here.  We don't want to use a deep copy in case there are non-plain
         * objects here, but we don't want to use the original object in case
         * the caller modifies it later.
         */
        this.jse_info = {};
        if (parsed.options.info) {
            for (k in parsed.options.info) {
                this.jse_info[k] = parsed.options.info[k];
            }
        }
        this.message = message;
        Error.call(this, message);
        if (Error.captureStackTrace) {
            ctor = parsed.options.constructorOpt || this.constructor;
            Error.captureStackTrace(this, ctor);
        }
        return (this);
    }
    mod_util.inherits(VError, Error);
    VError.prototype.name = 'VError';
    VError.prototype.toString = function ve_toString() {
        var str = (this.hasOwnProperty('name') && this.name ||
            this.constructor.name || this.constructor.prototype.name);
        if (this.message)
            str += ': ' + this.message;
        return (str);
    };
    /*
     * This method is provided for compatibility.  New callers should use
     * VError.cause() instead.  That method also uses the saner `null` return value
     * when there is no cause.
     */
    VError.prototype.cause = function ve_cause() {
        var cause = VError.cause(this);
        return (cause === null ? undefined : cause);
    };
    /*
     * Static methods
     *
     * These class-level methods are provided so that callers can use them on
     * instances of Errors that are not VErrors.  New interfaces should be provided
     * only using static methods to eliminate the class of programming mistake where
     * people fail to check whether the Error object has the corresponding methods.
     */
    VError.cause = function (err) {
        mod_assertplus.ok(mod_isError(err), 'err must be an Error');
        return (mod_isError(err.jse_cause) ? err.jse_cause : null);
    };
    VError.info = function (err) {
        var rv, cause, k;
        mod_assertplus.ok(mod_isError(err), 'err must be an Error');
        cause = VError.cause(err);
        if (cause !== null) {
            rv = VError.info(cause);
        }
        else {
            rv = {};
        }
        if (typeof (err.jse_info) == 'object' && err.jse_info !== null) {
            for (k in err.jse_info) {
                rv[k] = err.jse_info[k];
            }
        }
        return (rv);
    };
    VError.findCauseByName = function (err, name) {
        var cause;
        mod_assertplus.ok(mod_isError(err), 'err must be an Error');
        mod_assertplus.string(name, 'name');
        mod_assertplus.ok(name.length > 0, 'name cannot be empty');
        for (cause = err; cause !== null; cause = VError.cause(cause)) {
            mod_assertplus.ok(mod_isError(cause));
            if (cause.name == name) {
                return (cause);
            }
        }
        return (null);
    };
    VError.hasCauseWithName = function (err, name) {
        return (VError.findCauseByName(err, name) !== null);
    };
    VError.fullStack = function (err) {
        mod_assertplus.ok(mod_isError(err), 'err must be an Error');
        var cause = VError.cause(err);
        if (cause) {
            return (err.stack + '\ncaused by: ' + VError.fullStack(cause));
        }
        return (err.stack);
    };
    VError.errorFromList = function (errors) {
        mod_assertplus.arrayOfObject(errors, 'errors');
        if (errors.length === 0) {
            return (null);
        }
        errors.forEach(function (e) {
            mod_assertplus.ok(mod_isError(e));
        });
        if (errors.length == 1) {
            return (errors[0]);
        }
        return (new MultiError(errors));
    };
    VError.errorForEach = function (err, func) {
        mod_assertplus.ok(mod_isError(err), 'err must be an Error');
        mod_assertplus.func(func, 'func');
        if (err instanceof MultiError) {
            err.errors().forEach(function iterError(e) { func(e); });
        }
        else {
            func(err);
        }
    };
    /*
     * SError is like VError, but stricter about types.  You cannot pass "null" or
     * "undefined" as string arguments to the formatter.
     */
    function SError() {
        var args, obj, parsed, options;
        args = Array.prototype.slice.call(arguments, 0);
        if (!(this instanceof SError)) {
            obj = Object.create(SError.prototype);
            SError.apply(obj, arguments);
            return (obj);
        }
        parsed = parseConstructorArguments({
            'argv': args,
            'strict': true
        });
        options = parsed.options;
        VError.call(this, options, '%s', parsed.shortmessage);
        return (this);
    }
    /*
     * We don't bother setting SError.prototype.name because once constructed,
     * SErrors are just like VErrors.
     */
    mod_util.inherits(SError, VError);
    /*
     * Represents a collection of errors for the purpose of consumers that generally
     * only deal with one error.  Callers can extract the individual errors
     * contained in this object, but may also just treat it as a normal single
     * error, in which case a summary message will be printed.
     */
    function MultiError(errors) {
        mod_assertplus.array(errors, 'list of errors');
        mod_assertplus.ok(errors.length > 0, 'must be at least one error');
        this.ase_errors = errors;
        VError.call(this, {
            'cause': errors[0]
        }, 'first of %d error%s', errors.length, errors.length == 1 ? '' : 's');
    }
    mod_util.inherits(MultiError, VError);
    MultiError.prototype.name = 'MultiError';
    MultiError.prototype.errors = function me_errors() {
        return (this.ase_errors.slice(0));
    };
    /*
     * See README.md for reference details.
     */
    function WError() {
        var args, obj, parsed, options;
        args = Array.prototype.slice.call(arguments, 0);
        if (!(this instanceof WError)) {
            obj = Object.create(WError.prototype);
            WError.apply(obj, args);
            return (obj);
        }
        parsed = parseConstructorArguments({
            'argv': args,
            'strict': false
        });
        options = parsed.options;
        options['skipCauseMessage'] = true;
        VError.call(this, options, '%s', parsed.shortmessage);
        return (this);
    }
    mod_util.inherits(WError, VError);
    WError.prototype.name = 'WError';
    WError.prototype.toString = function we_toString() {
        var str = (this.hasOwnProperty('name') && this.name ||
            this.constructor.name || this.constructor.prototype.name);
        if (this.message)
            str += ': ' + this.message;
        if (this.jse_cause && this.jse_cause.message)
            str += '; caused by ' + this.jse_cause.toString();
        return (str);
    };
    /*
     * For purely historical reasons, WError's cause() function allows you to set
     * the cause.
     */
    WError.prototype.cause = function we_cause(c) {
        if (mod_isError(c))
            this.jse_cause = c;
        return (this.jse_cause);
    };
},
/* core-util-is/lib/util.js */ function _(require, module, exports) {
    // Copyright Joyent, Inc. and other Node contributors.
    //
    // Permission is hereby granted, free of charge, to any person obtaining a
    // copy of this software and associated documentation files (the
    // "Software"), to deal in the Software without restriction, including
    // without limitation the rights to use, copy, modify, merge, publish,
    // distribute, sublicense, and/or sell copies of the Software, and to permit
    // persons to whom the Software is furnished to do so, subject to the
    // following conditions:
    //
    // The above copyright notice and this permission notice shall be included
    // in all copies or substantial portions of the Software.
    //
    // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
    // OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    // MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
    // NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
    // DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
    // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
    // USE OR OTHER DEALINGS IN THE SOFTWARE.
    // NOTE: These type checking functions intentionally don't use `instanceof`
    // because it is fragile and can be easily faked with `Object.create()`.
    function isArray(arg) {
        if (Array.isArray) {
            return Array.isArray(arg);
        }
        return objectToString(arg) === '[object Array]';
    }
    exports.isArray = isArray;
    function isBoolean(arg) {
        return typeof arg === 'boolean';
    }
    exports.isBoolean = isBoolean;
    function isNull(arg) {
        return arg === null;
    }
    exports.isNull = isNull;
    function isNullOrUndefined(arg) {
        return arg == null;
    }
    exports.isNullOrUndefined = isNullOrUndefined;
    function isNumber(arg) {
        return typeof arg === 'number';
    }
    exports.isNumber = isNumber;
    function isString(arg) {
        return typeof arg === 'string';
    }
    exports.isString = isString;
    function isSymbol(arg) {
        return typeof arg === 'symbol';
    }
    exports.isSymbol = isSymbol;
    function isUndefined(arg) {
        return arg === void 0;
    }
    exports.isUndefined = isUndefined;
    function isRegExp(re) {
        return objectToString(re) === '[object RegExp]';
    }
    exports.isRegExp = isRegExp;
    function isObject(arg) {
        return typeof arg === 'object' && arg !== null;
    }
    exports.isObject = isObject;
    function isDate(d) {
        return objectToString(d) === '[object Date]';
    }
    exports.isDate = isDate;
    function isError(e) {
        return (objectToString(e) === '[object Error]' || e instanceof Error);
    }
    exports.isError = isError;
    function isFunction(arg) {
        return typeof arg === 'function';
    }
    exports.isFunction = isFunction;
    function isPrimitive(arg) {
        return arg === null ||
            typeof arg === 'boolean' ||
            typeof arg === 'number' ||
            typeof arg === 'string' ||
            typeof arg === 'symbol' || // ES6 symbol
            typeof arg === 'undefined';
    }
    exports.isPrimitive = isPrimitive;
    exports.isBuffer = Buffer.isBuffer;
    function objectToString(o) {
        return Object.prototype.toString.call(o);
    }
},
/* json-schema/lib/validate.js */ function _(require, module, exports) {
    /**
     * JSONSchema Validator - Validates JavaScript objects using JSON Schemas
     *	(http://www.json.com/json-schema-proposal/)
     *
     * Copyright (c) 2007 Kris Zyp SitePen (www.sitepen.com)
     * Licensed under the MIT (MIT-LICENSE.txt) license.
    To use the validator call the validate function with an instance object and an optional schema object.
    If a schema is provided, it will be used to validate. If the instance object refers to a schema (self-validating),
    that schema will be used to validate and the schema parameter is not necessary (if both exist,
    both validations will occur).
    The validate method will return an array of validation errors. If there are no errors, then an
    empty list will be returned. A validation error will have two properties:
    "property" which indicates which property had the error
    "message" which indicates what the error was
     */
    (function (root, factory) {
        if (typeof define === 'function' && define.amd) {
            // AMD. Register as an anonymous module.
            define([], function () {
                return factory();
            });
        }
        else if (typeof module === 'object' && module.exports) {
            // Node. Does not work with strict CommonJS, but
            // only CommonJS-like environments that support module.exports,
            // like Node.
            module.exports = factory();
        }
        else {
            // Browser globals
            root.jsonSchema = factory();
        }
    }(this, function () {
        var exports = validate;
        exports.Integer = { type: "integer" };
        var primitiveConstructors = {
            String: String,
            Boolean: Boolean,
            Number: Number,
            Object: Object,
            Array: Array,
            Date: Date
        };
        exports.validate = validate;
        function validate(/*Any*/ instance, /*Object*/ schema) {
            // Summary:
            //  	To use the validator call JSONSchema.validate with an instance object and an optional schema object.
            // 		If a schema is provided, it will be used to validate. If the instance object refers to a schema (self-validating),
            // 		that schema will be used to validate and the schema parameter is not necessary (if both exist,
            // 		both validations will occur).
            // 		The validate method will return an object with two properties:
            // 			valid: A boolean indicating if the instance is valid by the schema
            // 			errors: An array of validation errors. If there are no errors, then an
            // 					empty list will be returned. A validation error will have two properties:
            // 						property: which indicates which property had the error
            // 						message: which indicates what the error was
            //
            return validate(instance, schema, { changing: false }); //, coerce: false, existingOnly: false});
        }
        ;
        exports.checkPropertyChange = function (/*Any*/ value, /*Object*/ schema, /*String*/ property) {
            // Summary:
            // 		The checkPropertyChange method will check to see if an value can legally be in property with the given schema
            // 		This is slightly different than the validate method in that it will fail if the schema is readonly and it will
            // 		not check for self-validation, it is assumed that the passed in value is already internally valid.
            // 		The checkPropertyChange method will return the same object type as validate, see JSONSchema.validate for
            // 		information.
            //
            return validate(value, schema, { changing: property || "property" });
        };
        var validate = exports._validate = function (/*Any*/ instance, /*Object*/ schema, /*Object*/ options) {
            if (!options)
                options = {};
            var _changing = options.changing;
            function getType(schema) {
                return schema.type || (primitiveConstructors[schema.name] == schema && schema.name.toLowerCase());
            }
            var errors = [];
            // validate a value against a property definition
            function checkProp(value, schema, path, i) {
                var l;
                path += path ? typeof i == 'number' ? '[' + i + ']' : typeof i == 'undefined' ? '' : '.' + i : i;
                function addError(message) {
                    errors.push({ property: path, message: message });
                }
                if ((typeof schema != 'object' || schema instanceof Array) && (path || typeof schema != 'function') && !(schema && getType(schema))) {
                    if (typeof schema == 'function') {
                        if (!(value instanceof schema)) {
                            addError("is not an instance of the class/constructor " + schema.name);
                        }
                    }
                    else if (schema) {
                        addError("Invalid schema/property definition " + schema);
                    }
                    return null;
                }
                if (_changing && schema.readonly) {
                    addError("is a readonly field, it can not be changed");
                }
                if (schema['extends']) { // if it extends another schema, it must pass that schema as well
                    checkProp(value, schema['extends'], path, i);
                }
                // validate a value against a type definition
                function checkType(type, value) {
                    if (type) {
                        if (typeof type == 'string' && type != 'any' &&
                            (type == 'null' ? value !== null : typeof value != type) &&
                            !(value instanceof Array && type == 'array') &&
                            !(value instanceof Date && type == 'date') &&
                            !(type == 'integer' && value % 1 === 0)) {
                            return [{ property: path, message: (typeof value) + " value found, but a " + type + " is required" }];
                        }
                        if (type instanceof Array) {
                            var unionErrors = [];
                            for (var j = 0; j < type.length; j++) { // a union type
                                if (!(unionErrors = checkType(type[j], value)).length) {
                                    break;
                                }
                            }
                            if (unionErrors.length) {
                                return unionErrors;
                            }
                        }
                        else if (typeof type == 'object') {
                            var priorErrors = errors;
                            errors = [];
                            checkProp(value, type, path);
                            var theseErrors = errors;
                            errors = priorErrors;
                            return theseErrors;
                        }
                    }
                    return [];
                }
                if (value === undefined) {
                    if (schema.required) {
                        addError("is missing and it is required");
                    }
                }
                else {
                    errors = errors.concat(checkType(getType(schema), value));
                    if (schema.disallow && !checkType(schema.disallow, value).length) {
                        addError(" disallowed value was matched");
                    }
                    if (value !== null) {
                        if (value instanceof Array) {
                            if (schema.items) {
                                var itemsIsArray = schema.items instanceof Array;
                                var propDef = schema.items;
                                for (i = 0, l = value.length; i < l; i += 1) {
                                    if (itemsIsArray)
                                        propDef = schema.items[i];
                                    if (options.coerce)
                                        value[i] = options.coerce(value[i], propDef);
                                    errors.concat(checkProp(value[i], propDef, path, i));
                                }
                            }
                            if (schema.minItems && value.length < schema.minItems) {
                                addError("There must be a minimum of " + schema.minItems + " in the array");
                            }
                            if (schema.maxItems && value.length > schema.maxItems) {
                                addError("There must be a maximum of " + schema.maxItems + " in the array");
                            }
                        }
                        else if (schema.properties || schema.additionalProperties) {
                            errors.concat(checkObj(value, schema.properties, path, schema.additionalProperties));
                        }
                        if (schema.pattern && typeof value == 'string' && !value.match(schema.pattern)) {
                            addError("does not match the regex pattern " + schema.pattern);
                        }
                        if (schema.maxLength && typeof value == 'string' && value.length > schema.maxLength) {
                            addError("may only be " + schema.maxLength + " characters long");
                        }
                        if (schema.minLength && typeof value == 'string' && value.length < schema.minLength) {
                            addError("must be at least " + schema.minLength + " characters long");
                        }
                        if (typeof schema.minimum !== undefined && typeof value == typeof schema.minimum &&
                            schema.minimum > value) {
                            addError("must have a minimum value of " + schema.minimum);
                        }
                        if (typeof schema.maximum !== undefined && typeof value == typeof schema.maximum &&
                            schema.maximum < value) {
                            addError("must have a maximum value of " + schema.maximum);
                        }
                        if (schema['enum']) {
                            var enumer = schema['enum'];
                            l = enumer.length;
                            var found;
                            for (var j = 0; j < l; j++) {
                                if (enumer[j] === value) {
                                    found = 1;
                                    break;
                                }
                            }
                            if (!found) {
                                addError("does not have a value in the enumeration " + enumer.join(", "));
                            }
                        }
                        if (typeof schema.maxDecimal == 'number' &&
                            (value.toString().match(new RegExp("\\.[0-9]{" + (schema.maxDecimal + 1) + ",}")))) {
                            addError("may only have " + schema.maxDecimal + " digits of decimal places");
                        }
                    }
                }
                return null;
            }
            // validate an object against a schema
            function checkObj(instance, objTypeDef, path, additionalProp) {
                if (typeof objTypeDef == 'object') {
                    if (typeof instance != 'object' || instance instanceof Array) {
                        errors.push({ property: path, message: "an object is required" });
                    }
                    for (var i in objTypeDef) {
                        if (objTypeDef.hasOwnProperty(i)) {
                            var value = instance[i];
                            // skip _not_ specified properties
                            if (value === undefined && options.existingOnly)
                                continue;
                            var propDef = objTypeDef[i];
                            // set default
                            if (value === undefined && propDef["default"]) {
                                value = instance[i] = propDef["default"];
                            }
                            if (options.coerce && i in instance) {
                                value = instance[i] = options.coerce(value, propDef);
                            }
                            checkProp(value, propDef, path, i);
                        }
                    }
                }
                for (i in instance) {
                    if (instance.hasOwnProperty(i) && !(i.charAt(0) == '_' && i.charAt(1) == '_') && objTypeDef && !objTypeDef[i] && additionalProp === false) {
                        if (options.filter) {
                            delete instance[i];
                            continue;
                        }
                        else {
                            errors.push({ property: path, message: (typeof value) + "The property " + i +
                                    " is not defined in the schema and the schema does not allow additional properties" });
                        }
                    }
                    var requires = objTypeDef && objTypeDef[i] && objTypeDef[i].requires;
                    if (requires && !(requires in instance)) {
                        errors.push({ property: path, message: "the presence of the property " + i + " requires that " + requires + " also be present" });
                    }
                    value = instance[i];
                    if (additionalProp && (!(objTypeDef && typeof objTypeDef == 'object') || !(i in objTypeDef))) {
                        if (options.coerce) {
                            value = instance[i] = options.coerce(value, additionalProp);
                        }
                        checkProp(value, additionalProp, path, i);
                    }
                    if (!_changing && value && value.$schema) {
                        errors = errors.concat(checkProp(value, value.$schema, path, i));
                    }
                }
                return errors;
            }
            if (schema) {
                checkProp(instance, schema, '', _changing || '');
            }
            if (!_changing && instance && instance.$schema) {
                checkProp(instance, instance.$schema, '', '');
            }
            return { valid: !errors.length, errors: errors };
        };
        exports.mustBeValid = function (result) {
            //	summary:
            //		This checks to ensure that the result is valid and will throw an appropriate error message if it is not
            // result: the result returned from checkPropertyChange or validate
            if (!result.valid) {
                throw new TypeError(result.errors.map(function (error) { return "for property " + error.property + ': ' + error.message; }).join(", \n"));
            }
        };
        return exports;
    }));
},
/* http-signature/lib/verify.js */ function _(require, module, exports) {
    // Copyright 2015 Joyent, Inc.
    var assert = require(853) /* assert-plus */;
    var crypto = require('crypto');
    var sshpk = require(855) /* sshpk */;
    var utils = require(854) /* ./utils */;
    var HASH_ALGOS = utils.HASH_ALGOS;
    var PK_ALGOS = utils.PK_ALGOS;
    var InvalidAlgorithmError = utils.InvalidAlgorithmError;
    var HttpSignatureError = utils.HttpSignatureError;
    var validateAlgorithm = utils.validateAlgorithm;
    ///--- Exported API
    module.exports = {
        /**
         * Verify RSA/DSA signature against public key.  You are expected to pass in
         * an object that was returned from `parse()`.
         *
         * @param {Object} parsedSignature the object you got from `parse`.
         * @param {String} pubkey RSA/DSA private key PEM.
         * @return {Boolean} true if valid, false otherwise.
         * @throws {TypeError} if you pass in bad arguments.
         * @throws {InvalidAlgorithmError}
         */
        verifySignature: function verifySignature(parsedSignature, pubkey) {
            assert.object(parsedSignature, 'parsedSignature');
            if (typeof (pubkey) === 'string' || Buffer.isBuffer(pubkey))
                pubkey = sshpk.parseKey(pubkey);
            assert.ok(sshpk.Key.isKey(pubkey, [1, 1]), 'pubkey must be a sshpk.Key');
            var alg = validateAlgorithm(parsedSignature.algorithm);
            if (alg[0] === 'hmac' || alg[0] !== pubkey.type)
                return (false);
            var v = pubkey.createVerify(alg[1]);
            v.update(parsedSignature.signingString);
            return (v.verify(parsedSignature.params.signature, 'base64'));
        },
        /**
         * Verify HMAC against shared secret.  You are expected to pass in an object
         * that was returned from `parse()`.
         *
         * @param {Object} parsedSignature the object you got from `parse`.
         * @param {String} secret HMAC shared secret.
         * @return {Boolean} true if valid, false otherwise.
         * @throws {TypeError} if you pass in bad arguments.
         * @throws {InvalidAlgorithmError}
         */
        verifyHMAC: function verifyHMAC(parsedSignature, secret) {
            assert.object(parsedSignature, 'parsedHMAC');
            assert.string(secret, 'secret');
            var alg = validateAlgorithm(parsedSignature.algorithm);
            if (alg[0] !== 'hmac')
                return (false);
            var hashAlg = alg[1].toUpperCase();
            var hmac = crypto.createHmac(hashAlg, secret);
            hmac.update(parsedSignature.signingString);
            /*
             * Now double-hash to avoid leaking timing information - there's
             * no easy constant-time compare in JS, so we use this approach
             * instead. See for more info:
             * https://www.isecpartners.com/blog/2011/february/double-hmac-
             * verification.aspx
             */
            var h1 = crypto.createHmac(hashAlg, secret);
            h1.update(hmac.digest());
            h1 = h1.digest();
            var h2 = crypto.createHmac(hashAlg, secret);
            h2.update(new Buffer(parsedSignature.params.signature, 'base64'));
            h2 = h2.digest();
            /* Node 0.8 returns strings from .digest(). */
            if (typeof (h1) === 'string')
                return (h1 === h2);
            /* And node 0.10 lacks the .equals() method on Buffers. */
            if (Buffer.isBuffer(h1) && !h1.equals)
                return (h1.toString('binary') === h2.toString('binary'));
            return (h1.equals(h2));
        }
    };
},
/* mime-types/index.js */ function _(require, module, exports) {
    /**
     * Module dependencies.
     * @private
     */
    var db = require(901) /* mime-db */;
    var extname = require('path').extname;
    /**
     * Module variables.
     * @private
     */
    var EXTRACT_TYPE_REGEXP = /^\s*([^;\s]*)(?:;|\s|$)/;
    var TEXT_TYPE_REGEXP = /^text\//i;
    /**
     * Module exports.
     * @public
     */
    exports.charset = charset;
    exports.charsets = { lookup: charset };
    exports.contentType = contentType;
    exports.extension = extension;
    exports.extensions = Object.create(null);
    exports.lookup = lookup;
    exports.types = Object.create(null);
    // Populate the extensions/types maps
    populateMaps(exports.extensions, exports.types);
    /**
     * Get the default charset for a MIME type.
     *
     * @param {string} type
     * @return {boolean|string}
     */
    function charset(type) {
        if (!type || typeof type !== 'string') {
            return false;
        }
        // TODO: use media-typer
        var match = EXTRACT_TYPE_REGEXP.exec(type);
        var mime = match && db[match[1].toLowerCase()];
        if (mime && mime.charset) {
            return mime.charset;
        }
        // default text/* to utf-8
        if (match && TEXT_TYPE_REGEXP.test(match[1])) {
            return 'UTF-8';
        }
        return false;
    }
    /**
     * Create a full Content-Type header given a MIME type or extension.
     *
     * @param {string} str
     * @return {boolean|string}
     */
    function contentType(str) {
        // TODO: should this even be in this module?
        if (!str || typeof str !== 'string') {
            return false;
        }
        var mime = str.indexOf('/') === -1
            ? exports.lookup(str)
            : str;
        if (!mime) {
            return false;
        }
        // TODO: use content-type or other module
        if (mime.indexOf('charset') === -1) {
            var charset = exports.charset(mime);
            if (charset)
                mime += '; charset=' + charset.toLowerCase();
        }
        return mime;
    }
    /**
     * Get the default extension for a MIME type.
     *
     * @param {string} type
     * @return {boolean|string}
     */
    function extension(type) {
        if (!type || typeof type !== 'string') {
            return false;
        }
        // TODO: use media-typer
        var match = EXTRACT_TYPE_REGEXP.exec(type);
        // get extensions
        var exts = match && exports.extensions[match[1].toLowerCase()];
        if (!exts || !exts.length) {
            return false;
        }
        return exts[0];
    }
    /**
     * Lookup the MIME type for a file path/extension.
     *
     * @param {string} path
     * @return {boolean|string}
     */
    function lookup(path) {
        if (!path || typeof path !== 'string') {
            return false;
        }
        // get the extension ("ext" or ".ext" or full path)
        var extension = extname('x.' + path)
            .toLowerCase()
            .substr(1);
        if (!extension) {
            return false;
        }
        return exports.types[extension] || false;
    }
    /**
     * Populate the extensions and types maps.
     * @private
     */
    function populateMaps(extensions, types) {
        // source preference (least -> most)
        var preference = ['nginx', 'apache', undefined, 'iana'];
        Object.keys(db).forEach(function forEachMimeType(type) {
            var mime = db[type];
            var exts = mime.extensions;
            if (!exts || !exts.length) {
                return;
            }
            // mime -> extensions
            extensions[type] = exts;
            // extension -> mime
            for (var i = 0; i < exts.length; i++) {
                var extension = exts[i];
                if (types[extension]) {
                    var from = preference.indexOf(db[types[extension]].source);
                    var to = preference.indexOf(mime.source);
                    if (types[extension] !== 'application/octet-stream' &&
                        (from > to || (from === to && types[extension].substr(0, 12) === 'application/'))) {
                        // skip the remapping
                        continue;
                    }
                }
                // set the extension -> mime
                types[extension] = type;
            }
        });
    }
},
/* mime-db/index.js */ function _(require, module, exports) {
    /*!
     * mime-db
     * Copyright(c) 2014 Jonathan Ong
     * MIT Licensed
     */
    /**
     * Module exports.
     */
    module.exports = require(902) /* ./db.json */;
},
/* mime-db/db.json */ function _(require, module, exports) {
    module.exports = {
        "application/1d-interleaved-parityfec": {
            "source": "iana"
        },
        "application/3gpdash-qoe-report+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/3gpp-ims+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/a2l": {
            "source": "iana"
        },
        "application/activemessage": {
            "source": "iana"
        },
        "application/activity+json": {
            "source": "iana",
            "compressible": true
        },
        "application/alto-costmap+json": {
            "source": "iana",
            "compressible": true
        },
        "application/alto-costmapfilter+json": {
            "source": "iana",
            "compressible": true
        },
        "application/alto-directory+json": {
            "source": "iana",
            "compressible": true
        },
        "application/alto-endpointcost+json": {
            "source": "iana",
            "compressible": true
        },
        "application/alto-endpointcostparams+json": {
            "source": "iana",
            "compressible": true
        },
        "application/alto-endpointprop+json": {
            "source": "iana",
            "compressible": true
        },
        "application/alto-endpointpropparams+json": {
            "source": "iana",
            "compressible": true
        },
        "application/alto-error+json": {
            "source": "iana",
            "compressible": true
        },
        "application/alto-networkmap+json": {
            "source": "iana",
            "compressible": true
        },
        "application/alto-networkmapfilter+json": {
            "source": "iana",
            "compressible": true
        },
        "application/aml": {
            "source": "iana"
        },
        "application/andrew-inset": {
            "source": "iana",
            "extensions": ["ez"]
        },
        "application/applefile": {
            "source": "iana"
        },
        "application/applixware": {
            "source": "apache",
            "extensions": ["aw"]
        },
        "application/atf": {
            "source": "iana"
        },
        "application/atfx": {
            "source": "iana"
        },
        "application/atom+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["atom"]
        },
        "application/atomcat+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["atomcat"]
        },
        "application/atomdeleted+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["atomdeleted"]
        },
        "application/atomicmail": {
            "source": "iana"
        },
        "application/atomsvc+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["atomsvc"]
        },
        "application/atsc-dwd+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["dwd"]
        },
        "application/atsc-held+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["held"]
        },
        "application/atsc-rdt+json": {
            "source": "iana",
            "compressible": true
        },
        "application/atsc-rsat+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rsat"]
        },
        "application/atxml": {
            "source": "iana"
        },
        "application/auth-policy+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/bacnet-xdd+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/batch-smtp": {
            "source": "iana"
        },
        "application/bdoc": {
            "compressible": false,
            "extensions": ["bdoc"]
        },
        "application/beep+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/calendar+json": {
            "source": "iana",
            "compressible": true
        },
        "application/calendar+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xcs"]
        },
        "application/call-completion": {
            "source": "iana"
        },
        "application/cals-1840": {
            "source": "iana"
        },
        "application/cbor": {
            "source": "iana"
        },
        "application/cbor-seq": {
            "source": "iana"
        },
        "application/cccex": {
            "source": "iana"
        },
        "application/ccmp+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/ccxml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["ccxml"]
        },
        "application/cdfx+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["cdfx"]
        },
        "application/cdmi-capability": {
            "source": "iana",
            "extensions": ["cdmia"]
        },
        "application/cdmi-container": {
            "source": "iana",
            "extensions": ["cdmic"]
        },
        "application/cdmi-domain": {
            "source": "iana",
            "extensions": ["cdmid"]
        },
        "application/cdmi-object": {
            "source": "iana",
            "extensions": ["cdmio"]
        },
        "application/cdmi-queue": {
            "source": "iana",
            "extensions": ["cdmiq"]
        },
        "application/cdni": {
            "source": "iana"
        },
        "application/cea": {
            "source": "iana"
        },
        "application/cea-2018+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/cellml+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/cfw": {
            "source": "iana"
        },
        "application/clue+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/clue_info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/cms": {
            "source": "iana"
        },
        "application/cnrp+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/coap-group+json": {
            "source": "iana",
            "compressible": true
        },
        "application/coap-payload": {
            "source": "iana"
        },
        "application/commonground": {
            "source": "iana"
        },
        "application/conference-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/cose": {
            "source": "iana"
        },
        "application/cose-key": {
            "source": "iana"
        },
        "application/cose-key-set": {
            "source": "iana"
        },
        "application/cpl+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/csrattrs": {
            "source": "iana"
        },
        "application/csta+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/cstadata+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/csvm+json": {
            "source": "iana",
            "compressible": true
        },
        "application/cu-seeme": {
            "source": "apache",
            "extensions": ["cu"]
        },
        "application/cwt": {
            "source": "iana"
        },
        "application/cybercash": {
            "source": "iana"
        },
        "application/dart": {
            "compressible": true
        },
        "application/dash+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["mpd"]
        },
        "application/dashdelta": {
            "source": "iana"
        },
        "application/davmount+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["davmount"]
        },
        "application/dca-rft": {
            "source": "iana"
        },
        "application/dcd": {
            "source": "iana"
        },
        "application/dec-dx": {
            "source": "iana"
        },
        "application/dialog-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/dicom": {
            "source": "iana"
        },
        "application/dicom+json": {
            "source": "iana",
            "compressible": true
        },
        "application/dicom+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/dii": {
            "source": "iana"
        },
        "application/dit": {
            "source": "iana"
        },
        "application/dns": {
            "source": "iana"
        },
        "application/dns+json": {
            "source": "iana",
            "compressible": true
        },
        "application/dns-message": {
            "source": "iana"
        },
        "application/docbook+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["dbk"]
        },
        "application/dskpp+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/dssc+der": {
            "source": "iana",
            "extensions": ["dssc"]
        },
        "application/dssc+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xdssc"]
        },
        "application/dvcs": {
            "source": "iana"
        },
        "application/ecmascript": {
            "source": "iana",
            "compressible": true,
            "extensions": ["ecma", "es"]
        },
        "application/edi-consent": {
            "source": "iana"
        },
        "application/edi-x12": {
            "source": "iana",
            "compressible": false
        },
        "application/edifact": {
            "source": "iana",
            "compressible": false
        },
        "application/efi": {
            "source": "iana"
        },
        "application/emergencycalldata.comment+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/emergencycalldata.control+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/emergencycalldata.deviceinfo+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/emergencycalldata.ecall.msd": {
            "source": "iana"
        },
        "application/emergencycalldata.providerinfo+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/emergencycalldata.serviceinfo+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/emergencycalldata.subscriberinfo+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/emergencycalldata.veds+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/emma+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["emma"]
        },
        "application/emotionml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["emotionml"]
        },
        "application/encaprtp": {
            "source": "iana"
        },
        "application/epp+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/epub+zip": {
            "source": "iana",
            "compressible": false,
            "extensions": ["epub"]
        },
        "application/eshop": {
            "source": "iana"
        },
        "application/exi": {
            "source": "iana",
            "extensions": ["exi"]
        },
        "application/expect-ct-report+json": {
            "source": "iana",
            "compressible": true
        },
        "application/fastinfoset": {
            "source": "iana"
        },
        "application/fastsoap": {
            "source": "iana"
        },
        "application/fdt+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["fdt"]
        },
        "application/fhir+json": {
            "source": "iana",
            "compressible": true
        },
        "application/fhir+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/fido.trusted-apps+json": {
            "compressible": true
        },
        "application/fits": {
            "source": "iana"
        },
        "application/flexfec": {
            "source": "iana"
        },
        "application/font-sfnt": {
            "source": "iana"
        },
        "application/font-tdpfr": {
            "source": "iana",
            "extensions": ["pfr"]
        },
        "application/font-woff": {
            "source": "iana",
            "compressible": false
        },
        "application/framework-attributes+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/geo+json": {
            "source": "iana",
            "compressible": true,
            "extensions": ["geojson"]
        },
        "application/geo+json-seq": {
            "source": "iana"
        },
        "application/geopackage+sqlite3": {
            "source": "iana"
        },
        "application/geoxacml+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/gltf-buffer": {
            "source": "iana"
        },
        "application/gml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["gml"]
        },
        "application/gpx+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["gpx"]
        },
        "application/gxf": {
            "source": "apache",
            "extensions": ["gxf"]
        },
        "application/gzip": {
            "source": "iana",
            "compressible": false,
            "extensions": ["gz"]
        },
        "application/h224": {
            "source": "iana"
        },
        "application/held+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/hjson": {
            "extensions": ["hjson"]
        },
        "application/http": {
            "source": "iana"
        },
        "application/hyperstudio": {
            "source": "iana",
            "extensions": ["stk"]
        },
        "application/ibe-key-request+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/ibe-pkg-reply+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/ibe-pp-data": {
            "source": "iana"
        },
        "application/iges": {
            "source": "iana"
        },
        "application/im-iscomposing+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/index": {
            "source": "iana"
        },
        "application/index.cmd": {
            "source": "iana"
        },
        "application/index.obj": {
            "source": "iana"
        },
        "application/index.response": {
            "source": "iana"
        },
        "application/index.vnd": {
            "source": "iana"
        },
        "application/inkml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["ink", "inkml"]
        },
        "application/iotp": {
            "source": "iana"
        },
        "application/ipfix": {
            "source": "iana",
            "extensions": ["ipfix"]
        },
        "application/ipp": {
            "source": "iana"
        },
        "application/isup": {
            "source": "iana"
        },
        "application/its+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["its"]
        },
        "application/java-archive": {
            "source": "apache",
            "compressible": false,
            "extensions": ["jar", "war", "ear"]
        },
        "application/java-serialized-object": {
            "source": "apache",
            "compressible": false,
            "extensions": ["ser"]
        },
        "application/java-vm": {
            "source": "apache",
            "compressible": false,
            "extensions": ["class"]
        },
        "application/javascript": {
            "source": "iana",
            "charset": "UTF-8",
            "compressible": true,
            "extensions": ["js", "mjs"]
        },
        "application/jf2feed+json": {
            "source": "iana",
            "compressible": true
        },
        "application/jose": {
            "source": "iana"
        },
        "application/jose+json": {
            "source": "iana",
            "compressible": true
        },
        "application/jrd+json": {
            "source": "iana",
            "compressible": true
        },
        "application/json": {
            "source": "iana",
            "charset": "UTF-8",
            "compressible": true,
            "extensions": ["json", "map"]
        },
        "application/json-patch+json": {
            "source": "iana",
            "compressible": true
        },
        "application/json-seq": {
            "source": "iana"
        },
        "application/json5": {
            "extensions": ["json5"]
        },
        "application/jsonml+json": {
            "source": "apache",
            "compressible": true,
            "extensions": ["jsonml"]
        },
        "application/jwk+json": {
            "source": "iana",
            "compressible": true
        },
        "application/jwk-set+json": {
            "source": "iana",
            "compressible": true
        },
        "application/jwt": {
            "source": "iana"
        },
        "application/kpml-request+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/kpml-response+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/ld+json": {
            "source": "iana",
            "compressible": true,
            "extensions": ["jsonld"]
        },
        "application/lgr+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["lgr"]
        },
        "application/link-format": {
            "source": "iana"
        },
        "application/load-control+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/lost+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["lostxml"]
        },
        "application/lostsync+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/lxf": {
            "source": "iana"
        },
        "application/mac-binhex40": {
            "source": "iana",
            "extensions": ["hqx"]
        },
        "application/mac-compactpro": {
            "source": "apache",
            "extensions": ["cpt"]
        },
        "application/macwriteii": {
            "source": "iana"
        },
        "application/mads+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["mads"]
        },
        "application/manifest+json": {
            "charset": "UTF-8",
            "compressible": true,
            "extensions": ["webmanifest"]
        },
        "application/marc": {
            "source": "iana",
            "extensions": ["mrc"]
        },
        "application/marcxml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["mrcx"]
        },
        "application/mathematica": {
            "source": "iana",
            "extensions": ["ma", "nb", "mb"]
        },
        "application/mathml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["mathml"]
        },
        "application/mathml-content+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mathml-presentation+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbms-associated-procedure-description+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbms-deregister+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbms-envelope+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbms-msk+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbms-msk-response+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbms-protection-description+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbms-reception-report+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbms-register+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbms-register-response+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbms-schedule+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbms-user-service-description+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mbox": {
            "source": "iana",
            "extensions": ["mbox"]
        },
        "application/media-policy-dataset+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/media_control+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/mediaservercontrol+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["mscml"]
        },
        "application/merge-patch+json": {
            "source": "iana",
            "compressible": true
        },
        "application/metalink+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["metalink"]
        },
        "application/metalink4+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["meta4"]
        },
        "application/mets+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["mets"]
        },
        "application/mf4": {
            "source": "iana"
        },
        "application/mikey": {
            "source": "iana"
        },
        "application/mipc": {
            "source": "iana"
        },
        "application/mmt-aei+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["maei"]
        },
        "application/mmt-usd+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["musd"]
        },
        "application/mods+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["mods"]
        },
        "application/moss-keys": {
            "source": "iana"
        },
        "application/moss-signature": {
            "source": "iana"
        },
        "application/mosskey-data": {
            "source": "iana"
        },
        "application/mosskey-request": {
            "source": "iana"
        },
        "application/mp21": {
            "source": "iana",
            "extensions": ["m21", "mp21"]
        },
        "application/mp4": {
            "source": "iana",
            "extensions": ["mp4s", "m4p"]
        },
        "application/mpeg4-generic": {
            "source": "iana"
        },
        "application/mpeg4-iod": {
            "source": "iana"
        },
        "application/mpeg4-iod-xmt": {
            "source": "iana"
        },
        "application/mrb-consumer+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xdf"]
        },
        "application/mrb-publish+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xdf"]
        },
        "application/msc-ivr+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/msc-mixer+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/msword": {
            "source": "iana",
            "compressible": false,
            "extensions": ["doc", "dot"]
        },
        "application/mud+json": {
            "source": "iana",
            "compressible": true
        },
        "application/multipart-core": {
            "source": "iana"
        },
        "application/mxf": {
            "source": "iana",
            "extensions": ["mxf"]
        },
        "application/n-quads": {
            "source": "iana",
            "extensions": ["nq"]
        },
        "application/n-triples": {
            "source": "iana",
            "extensions": ["nt"]
        },
        "application/nasdata": {
            "source": "iana"
        },
        "application/news-checkgroups": {
            "source": "iana"
        },
        "application/news-groupinfo": {
            "source": "iana"
        },
        "application/news-transmission": {
            "source": "iana"
        },
        "application/nlsml+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/node": {
            "source": "iana"
        },
        "application/nss": {
            "source": "iana"
        },
        "application/ocsp-request": {
            "source": "iana"
        },
        "application/ocsp-response": {
            "source": "iana"
        },
        "application/octet-stream": {
            "source": "iana",
            "compressible": false,
            "extensions": ["bin", "dms", "lrf", "mar", "so", "dist", "distz", "pkg", "bpk", "dump", "elc", "deploy", "exe", "dll", "deb", "dmg", "iso", "img", "msi", "msp", "msm", "buffer"]
        },
        "application/oda": {
            "source": "iana",
            "extensions": ["oda"]
        },
        "application/odm+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/odx": {
            "source": "iana"
        },
        "application/oebps-package+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["opf"]
        },
        "application/ogg": {
            "source": "iana",
            "compressible": false,
            "extensions": ["ogx"]
        },
        "application/omdoc+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["omdoc"]
        },
        "application/onenote": {
            "source": "apache",
            "extensions": ["onetoc", "onetoc2", "onetmp", "onepkg"]
        },
        "application/oscore": {
            "source": "iana"
        },
        "application/oxps": {
            "source": "iana",
            "extensions": ["oxps"]
        },
        "application/p2p-overlay+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["relo"]
        },
        "application/parityfec": {
            "source": "iana"
        },
        "application/passport": {
            "source": "iana"
        },
        "application/patch-ops-error+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xer"]
        },
        "application/pdf": {
            "source": "iana",
            "compressible": false,
            "extensions": ["pdf"]
        },
        "application/pdx": {
            "source": "iana"
        },
        "application/pem-certificate-chain": {
            "source": "iana"
        },
        "application/pgp-encrypted": {
            "source": "iana",
            "compressible": false,
            "extensions": ["pgp"]
        },
        "application/pgp-keys": {
            "source": "iana"
        },
        "application/pgp-signature": {
            "source": "iana",
            "extensions": ["asc", "sig"]
        },
        "application/pics-rules": {
            "source": "apache",
            "extensions": ["prf"]
        },
        "application/pidf+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/pidf-diff+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/pkcs10": {
            "source": "iana",
            "extensions": ["p10"]
        },
        "application/pkcs12": {
            "source": "iana"
        },
        "application/pkcs7-mime": {
            "source": "iana",
            "extensions": ["p7m", "p7c"]
        },
        "application/pkcs7-signature": {
            "source": "iana",
            "extensions": ["p7s"]
        },
        "application/pkcs8": {
            "source": "iana",
            "extensions": ["p8"]
        },
        "application/pkcs8-encrypted": {
            "source": "iana"
        },
        "application/pkix-attr-cert": {
            "source": "iana",
            "extensions": ["ac"]
        },
        "application/pkix-cert": {
            "source": "iana",
            "extensions": ["cer"]
        },
        "application/pkix-crl": {
            "source": "iana",
            "extensions": ["crl"]
        },
        "application/pkix-pkipath": {
            "source": "iana",
            "extensions": ["pkipath"]
        },
        "application/pkixcmp": {
            "source": "iana",
            "extensions": ["pki"]
        },
        "application/pls+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["pls"]
        },
        "application/poc-settings+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/postscript": {
            "source": "iana",
            "compressible": true,
            "extensions": ["ai", "eps", "ps"]
        },
        "application/ppsp-tracker+json": {
            "source": "iana",
            "compressible": true
        },
        "application/problem+json": {
            "source": "iana",
            "compressible": true
        },
        "application/problem+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/provenance+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["provx"]
        },
        "application/prs.alvestrand.titrax-sheet": {
            "source": "iana"
        },
        "application/prs.cww": {
            "source": "iana",
            "extensions": ["cww"]
        },
        "application/prs.hpub+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/prs.nprend": {
            "source": "iana"
        },
        "application/prs.plucker": {
            "source": "iana"
        },
        "application/prs.rdf-xml-crypt": {
            "source": "iana"
        },
        "application/prs.xsf+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/pskc+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["pskcxml"]
        },
        "application/qsig": {
            "source": "iana"
        },
        "application/raml+yaml": {
            "compressible": true,
            "extensions": ["raml"]
        },
        "application/raptorfec": {
            "source": "iana"
        },
        "application/rdap+json": {
            "source": "iana",
            "compressible": true
        },
        "application/rdf+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rdf", "owl"]
        },
        "application/reginfo+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rif"]
        },
        "application/relax-ng-compact-syntax": {
            "source": "iana",
            "extensions": ["rnc"]
        },
        "application/remote-printing": {
            "source": "iana"
        },
        "application/reputon+json": {
            "source": "iana",
            "compressible": true
        },
        "application/resource-lists+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rl"]
        },
        "application/resource-lists-diff+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rld"]
        },
        "application/rfc+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/riscos": {
            "source": "iana"
        },
        "application/rlmi+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/rls-services+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rs"]
        },
        "application/route-apd+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rapd"]
        },
        "application/route-s-tsid+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["sls"]
        },
        "application/route-usd+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rusd"]
        },
        "application/rpki-ghostbusters": {
            "source": "iana",
            "extensions": ["gbr"]
        },
        "application/rpki-manifest": {
            "source": "iana",
            "extensions": ["mft"]
        },
        "application/rpki-publication": {
            "source": "iana"
        },
        "application/rpki-roa": {
            "source": "iana",
            "extensions": ["roa"]
        },
        "application/rpki-updown": {
            "source": "iana"
        },
        "application/rsd+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["rsd"]
        },
        "application/rss+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["rss"]
        },
        "application/rtf": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rtf"]
        },
        "application/rtploopback": {
            "source": "iana"
        },
        "application/rtx": {
            "source": "iana"
        },
        "application/samlassertion+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/samlmetadata+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/sbml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["sbml"]
        },
        "application/scaip+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/scim+json": {
            "source": "iana",
            "compressible": true
        },
        "application/scvp-cv-request": {
            "source": "iana",
            "extensions": ["scq"]
        },
        "application/scvp-cv-response": {
            "source": "iana",
            "extensions": ["scs"]
        },
        "application/scvp-vp-request": {
            "source": "iana",
            "extensions": ["spq"]
        },
        "application/scvp-vp-response": {
            "source": "iana",
            "extensions": ["spp"]
        },
        "application/sdp": {
            "source": "iana",
            "extensions": ["sdp"]
        },
        "application/secevent+jwt": {
            "source": "iana"
        },
        "application/senml+cbor": {
            "source": "iana"
        },
        "application/senml+json": {
            "source": "iana",
            "compressible": true
        },
        "application/senml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["senmlx"]
        },
        "application/senml-exi": {
            "source": "iana"
        },
        "application/sensml+cbor": {
            "source": "iana"
        },
        "application/sensml+json": {
            "source": "iana",
            "compressible": true
        },
        "application/sensml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["sensmlx"]
        },
        "application/sensml-exi": {
            "source": "iana"
        },
        "application/sep+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/sep-exi": {
            "source": "iana"
        },
        "application/session-info": {
            "source": "iana"
        },
        "application/set-payment": {
            "source": "iana"
        },
        "application/set-payment-initiation": {
            "source": "iana",
            "extensions": ["setpay"]
        },
        "application/set-registration": {
            "source": "iana"
        },
        "application/set-registration-initiation": {
            "source": "iana",
            "extensions": ["setreg"]
        },
        "application/sgml": {
            "source": "iana"
        },
        "application/sgml-open-catalog": {
            "source": "iana"
        },
        "application/shf+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["shf"]
        },
        "application/sieve": {
            "source": "iana",
            "extensions": ["siv", "sieve"]
        },
        "application/simple-filter+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/simple-message-summary": {
            "source": "iana"
        },
        "application/simplesymbolcontainer": {
            "source": "iana"
        },
        "application/sipc": {
            "source": "iana"
        },
        "application/slate": {
            "source": "iana"
        },
        "application/smil": {
            "source": "iana"
        },
        "application/smil+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["smi", "smil"]
        },
        "application/smpte336m": {
            "source": "iana"
        },
        "application/soap+fastinfoset": {
            "source": "iana"
        },
        "application/soap+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/sparql-query": {
            "source": "iana",
            "extensions": ["rq"]
        },
        "application/sparql-results+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["srx"]
        },
        "application/spirits-event+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/sql": {
            "source": "iana"
        },
        "application/srgs": {
            "source": "iana",
            "extensions": ["gram"]
        },
        "application/srgs+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["grxml"]
        },
        "application/sru+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["sru"]
        },
        "application/ssdl+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["ssdl"]
        },
        "application/ssml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["ssml"]
        },
        "application/stix+json": {
            "source": "iana",
            "compressible": true
        },
        "application/swid+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["swidtag"]
        },
        "application/tamp-apex-update": {
            "source": "iana"
        },
        "application/tamp-apex-update-confirm": {
            "source": "iana"
        },
        "application/tamp-community-update": {
            "source": "iana"
        },
        "application/tamp-community-update-confirm": {
            "source": "iana"
        },
        "application/tamp-error": {
            "source": "iana"
        },
        "application/tamp-sequence-adjust": {
            "source": "iana"
        },
        "application/tamp-sequence-adjust-confirm": {
            "source": "iana"
        },
        "application/tamp-status-query": {
            "source": "iana"
        },
        "application/tamp-status-response": {
            "source": "iana"
        },
        "application/tamp-update": {
            "source": "iana"
        },
        "application/tamp-update-confirm": {
            "source": "iana"
        },
        "application/tar": {
            "compressible": true
        },
        "application/taxii+json": {
            "source": "iana",
            "compressible": true
        },
        "application/tei+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["tei", "teicorpus"]
        },
        "application/tetra_isi": {
            "source": "iana"
        },
        "application/thraud+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["tfi"]
        },
        "application/timestamp-query": {
            "source": "iana"
        },
        "application/timestamp-reply": {
            "source": "iana"
        },
        "application/timestamped-data": {
            "source": "iana",
            "extensions": ["tsd"]
        },
        "application/tlsrpt+gzip": {
            "source": "iana"
        },
        "application/tlsrpt+json": {
            "source": "iana",
            "compressible": true
        },
        "application/tnauthlist": {
            "source": "iana"
        },
        "application/toml": {
            "compressible": true,
            "extensions": ["toml"]
        },
        "application/trickle-ice-sdpfrag": {
            "source": "iana"
        },
        "application/trig": {
            "source": "iana"
        },
        "application/ttml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["ttml"]
        },
        "application/tve-trigger": {
            "source": "iana"
        },
        "application/tzif": {
            "source": "iana"
        },
        "application/tzif-leap": {
            "source": "iana"
        },
        "application/ulpfec": {
            "source": "iana"
        },
        "application/urc-grpsheet+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/urc-ressheet+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rsheet"]
        },
        "application/urc-targetdesc+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/urc-uisocketdesc+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vcard+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vcard+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vemmi": {
            "source": "iana"
        },
        "application/vividence.scriptfile": {
            "source": "apache"
        },
        "application/vnd.1000minds.decision-model+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["1km"]
        },
        "application/vnd.3gpp-prose+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp-prose-pc3ch+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp-v2x-local-service-information": {
            "source": "iana"
        },
        "application/vnd.3gpp.access-transfer-events+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.bsf+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.gmop+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mc-signalling-ear": {
            "source": "iana"
        },
        "application/vnd.3gpp.mcdata-affiliation-command+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcdata-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcdata-payload": {
            "source": "iana"
        },
        "application/vnd.3gpp.mcdata-service-config+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcdata-signalling": {
            "source": "iana"
        },
        "application/vnd.3gpp.mcdata-ue-config+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcdata-user-profile+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcptt-affiliation-command+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcptt-floor-request+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcptt-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcptt-location-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcptt-mbms-usage-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcptt-service-config+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcptt-signed+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcptt-ue-config+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcptt-ue-init-config+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcptt-user-profile+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcvideo-affiliation-command+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcvideo-affiliation-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcvideo-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcvideo-location-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcvideo-mbms-usage-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcvideo-service-config+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcvideo-transmission-request+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcvideo-ue-config+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mcvideo-user-profile+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.mid-call+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.pic-bw-large": {
            "source": "iana",
            "extensions": ["plb"]
        },
        "application/vnd.3gpp.pic-bw-small": {
            "source": "iana",
            "extensions": ["psb"]
        },
        "application/vnd.3gpp.pic-bw-var": {
            "source": "iana",
            "extensions": ["pvb"]
        },
        "application/vnd.3gpp.sms": {
            "source": "iana"
        },
        "application/vnd.3gpp.sms+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.srvcc-ext+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.srvcc-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.state-and-event-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp.ussd+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp2.bcmcsinfo+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.3gpp2.sms": {
            "source": "iana"
        },
        "application/vnd.3gpp2.tcap": {
            "source": "iana",
            "extensions": ["tcap"]
        },
        "application/vnd.3lightssoftware.imagescal": {
            "source": "iana"
        },
        "application/vnd.3m.post-it-notes": {
            "source": "iana",
            "extensions": ["pwn"]
        },
        "application/vnd.accpac.simply.aso": {
            "source": "iana",
            "extensions": ["aso"]
        },
        "application/vnd.accpac.simply.imp": {
            "source": "iana",
            "extensions": ["imp"]
        },
        "application/vnd.acucobol": {
            "source": "iana",
            "extensions": ["acu"]
        },
        "application/vnd.acucorp": {
            "source": "iana",
            "extensions": ["atc", "acutc"]
        },
        "application/vnd.adobe.air-application-installer-package+zip": {
            "source": "apache",
            "compressible": false,
            "extensions": ["air"]
        },
        "application/vnd.adobe.flash.movie": {
            "source": "iana"
        },
        "application/vnd.adobe.formscentral.fcdt": {
            "source": "iana",
            "extensions": ["fcdt"]
        },
        "application/vnd.adobe.fxp": {
            "source": "iana",
            "extensions": ["fxp", "fxpl"]
        },
        "application/vnd.adobe.partial-upload": {
            "source": "iana"
        },
        "application/vnd.adobe.xdp+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xdp"]
        },
        "application/vnd.adobe.xfdf": {
            "source": "iana",
            "extensions": ["xfdf"]
        },
        "application/vnd.aether.imp": {
            "source": "iana"
        },
        "application/vnd.afpc.afplinedata": {
            "source": "iana"
        },
        "application/vnd.afpc.afplinedata-pagedef": {
            "source": "iana"
        },
        "application/vnd.afpc.foca-charset": {
            "source": "iana"
        },
        "application/vnd.afpc.foca-codedfont": {
            "source": "iana"
        },
        "application/vnd.afpc.foca-codepage": {
            "source": "iana"
        },
        "application/vnd.afpc.modca": {
            "source": "iana"
        },
        "application/vnd.afpc.modca-formdef": {
            "source": "iana"
        },
        "application/vnd.afpc.modca-mediummap": {
            "source": "iana"
        },
        "application/vnd.afpc.modca-objectcontainer": {
            "source": "iana"
        },
        "application/vnd.afpc.modca-overlay": {
            "source": "iana"
        },
        "application/vnd.afpc.modca-pagesegment": {
            "source": "iana"
        },
        "application/vnd.ah-barcode": {
            "source": "iana"
        },
        "application/vnd.ahead.space": {
            "source": "iana",
            "extensions": ["ahead"]
        },
        "application/vnd.airzip.filesecure.azf": {
            "source": "iana",
            "extensions": ["azf"]
        },
        "application/vnd.airzip.filesecure.azs": {
            "source": "iana",
            "extensions": ["azs"]
        },
        "application/vnd.amadeus+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.amazon.ebook": {
            "source": "apache",
            "extensions": ["azw"]
        },
        "application/vnd.amazon.mobi8-ebook": {
            "source": "iana"
        },
        "application/vnd.americandynamics.acc": {
            "source": "iana",
            "extensions": ["acc"]
        },
        "application/vnd.amiga.ami": {
            "source": "iana",
            "extensions": ["ami"]
        },
        "application/vnd.amundsen.maze+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.android.ota": {
            "source": "iana"
        },
        "application/vnd.android.package-archive": {
            "source": "apache",
            "compressible": false,
            "extensions": ["apk"]
        },
        "application/vnd.anki": {
            "source": "iana"
        },
        "application/vnd.anser-web-certificate-issue-initiation": {
            "source": "iana",
            "extensions": ["cii"]
        },
        "application/vnd.anser-web-funds-transfer-initiation": {
            "source": "apache",
            "extensions": ["fti"]
        },
        "application/vnd.antix.game-component": {
            "source": "iana",
            "extensions": ["atx"]
        },
        "application/vnd.apache.thrift.binary": {
            "source": "iana"
        },
        "application/vnd.apache.thrift.compact": {
            "source": "iana"
        },
        "application/vnd.apache.thrift.json": {
            "source": "iana"
        },
        "application/vnd.api+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.aplextor.warrp+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.apothekende.reservation+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.apple.installer+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["mpkg"]
        },
        "application/vnd.apple.keynote": {
            "source": "iana",
            "extensions": ["keynote"]
        },
        "application/vnd.apple.mpegurl": {
            "source": "iana",
            "extensions": ["m3u8"]
        },
        "application/vnd.apple.numbers": {
            "source": "iana",
            "extensions": ["numbers"]
        },
        "application/vnd.apple.pages": {
            "source": "iana",
            "extensions": ["pages"]
        },
        "application/vnd.apple.pkpass": {
            "compressible": false,
            "extensions": ["pkpass"]
        },
        "application/vnd.arastra.swi": {
            "source": "iana"
        },
        "application/vnd.aristanetworks.swi": {
            "source": "iana",
            "extensions": ["swi"]
        },
        "application/vnd.artisan+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.artsquare": {
            "source": "iana"
        },
        "application/vnd.astraea-software.iota": {
            "source": "iana",
            "extensions": ["iota"]
        },
        "application/vnd.audiograph": {
            "source": "iana",
            "extensions": ["aep"]
        },
        "application/vnd.autopackage": {
            "source": "iana"
        },
        "application/vnd.avalon+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.avistar+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.balsamiq.bmml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["bmml"]
        },
        "application/vnd.balsamiq.bmpr": {
            "source": "iana"
        },
        "application/vnd.banana-accounting": {
            "source": "iana"
        },
        "application/vnd.bbf.usp.error": {
            "source": "iana"
        },
        "application/vnd.bbf.usp.msg": {
            "source": "iana"
        },
        "application/vnd.bbf.usp.msg+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.bekitzur-stech+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.bint.med-content": {
            "source": "iana"
        },
        "application/vnd.biopax.rdf+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.blink-idb-value-wrapper": {
            "source": "iana"
        },
        "application/vnd.blueice.multipass": {
            "source": "iana",
            "extensions": ["mpm"]
        },
        "application/vnd.bluetooth.ep.oob": {
            "source": "iana"
        },
        "application/vnd.bluetooth.le.oob": {
            "source": "iana"
        },
        "application/vnd.bmi": {
            "source": "iana",
            "extensions": ["bmi"]
        },
        "application/vnd.bpf": {
            "source": "iana"
        },
        "application/vnd.bpf3": {
            "source": "iana"
        },
        "application/vnd.businessobjects": {
            "source": "iana",
            "extensions": ["rep"]
        },
        "application/vnd.byu.uapi+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.cab-jscript": {
            "source": "iana"
        },
        "application/vnd.canon-cpdl": {
            "source": "iana"
        },
        "application/vnd.canon-lips": {
            "source": "iana"
        },
        "application/vnd.capasystems-pg+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.cendio.thinlinc.clientconf": {
            "source": "iana"
        },
        "application/vnd.century-systems.tcp_stream": {
            "source": "iana"
        },
        "application/vnd.chemdraw+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["cdxml"]
        },
        "application/vnd.chess-pgn": {
            "source": "iana"
        },
        "application/vnd.chipnuts.karaoke-mmd": {
            "source": "iana",
            "extensions": ["mmd"]
        },
        "application/vnd.ciedi": {
            "source": "iana"
        },
        "application/vnd.cinderella": {
            "source": "iana",
            "extensions": ["cdy"]
        },
        "application/vnd.cirpack.isdn-ext": {
            "source": "iana"
        },
        "application/vnd.citationstyles.style+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["csl"]
        },
        "application/vnd.claymore": {
            "source": "iana",
            "extensions": ["cla"]
        },
        "application/vnd.cloanto.rp9": {
            "source": "iana",
            "extensions": ["rp9"]
        },
        "application/vnd.clonk.c4group": {
            "source": "iana",
            "extensions": ["c4g", "c4d", "c4f", "c4p", "c4u"]
        },
        "application/vnd.cluetrust.cartomobile-config": {
            "source": "iana",
            "extensions": ["c11amc"]
        },
        "application/vnd.cluetrust.cartomobile-config-pkg": {
            "source": "iana",
            "extensions": ["c11amz"]
        },
        "application/vnd.coffeescript": {
            "source": "iana"
        },
        "application/vnd.collabio.xodocuments.document": {
            "source": "iana"
        },
        "application/vnd.collabio.xodocuments.document-template": {
            "source": "iana"
        },
        "application/vnd.collabio.xodocuments.presentation": {
            "source": "iana"
        },
        "application/vnd.collabio.xodocuments.presentation-template": {
            "source": "iana"
        },
        "application/vnd.collabio.xodocuments.spreadsheet": {
            "source": "iana"
        },
        "application/vnd.collabio.xodocuments.spreadsheet-template": {
            "source": "iana"
        },
        "application/vnd.collection+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.collection.doc+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.collection.next+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.comicbook+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.comicbook-rar": {
            "source": "iana"
        },
        "application/vnd.commerce-battelle": {
            "source": "iana"
        },
        "application/vnd.commonspace": {
            "source": "iana",
            "extensions": ["csp"]
        },
        "application/vnd.contact.cmsg": {
            "source": "iana",
            "extensions": ["cdbcmsg"]
        },
        "application/vnd.coreos.ignition+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.cosmocaller": {
            "source": "iana",
            "extensions": ["cmc"]
        },
        "application/vnd.crick.clicker": {
            "source": "iana",
            "extensions": ["clkx"]
        },
        "application/vnd.crick.clicker.keyboard": {
            "source": "iana",
            "extensions": ["clkk"]
        },
        "application/vnd.crick.clicker.palette": {
            "source": "iana",
            "extensions": ["clkp"]
        },
        "application/vnd.crick.clicker.template": {
            "source": "iana",
            "extensions": ["clkt"]
        },
        "application/vnd.crick.clicker.wordbank": {
            "source": "iana",
            "extensions": ["clkw"]
        },
        "application/vnd.criticaltools.wbs+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["wbs"]
        },
        "application/vnd.cryptii.pipe+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.crypto-shade-file": {
            "source": "iana"
        },
        "application/vnd.ctc-posml": {
            "source": "iana",
            "extensions": ["pml"]
        },
        "application/vnd.ctct.ws+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.cups-pdf": {
            "source": "iana"
        },
        "application/vnd.cups-postscript": {
            "source": "iana"
        },
        "application/vnd.cups-ppd": {
            "source": "iana",
            "extensions": ["ppd"]
        },
        "application/vnd.cups-raster": {
            "source": "iana"
        },
        "application/vnd.cups-raw": {
            "source": "iana"
        },
        "application/vnd.curl": {
            "source": "iana"
        },
        "application/vnd.curl.car": {
            "source": "apache",
            "extensions": ["car"]
        },
        "application/vnd.curl.pcurl": {
            "source": "apache",
            "extensions": ["pcurl"]
        },
        "application/vnd.cyan.dean.root+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.cybank": {
            "source": "iana"
        },
        "application/vnd.d2l.coursepackage1p0+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.dart": {
            "source": "iana",
            "compressible": true,
            "extensions": ["dart"]
        },
        "application/vnd.data-vision.rdz": {
            "source": "iana",
            "extensions": ["rdz"]
        },
        "application/vnd.datapackage+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.dataresource+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.debian.binary-package": {
            "source": "iana"
        },
        "application/vnd.dece.data": {
            "source": "iana",
            "extensions": ["uvf", "uvvf", "uvd", "uvvd"]
        },
        "application/vnd.dece.ttml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["uvt", "uvvt"]
        },
        "application/vnd.dece.unspecified": {
            "source": "iana",
            "extensions": ["uvx", "uvvx"]
        },
        "application/vnd.dece.zip": {
            "source": "iana",
            "extensions": ["uvz", "uvvz"]
        },
        "application/vnd.denovo.fcselayout-link": {
            "source": "iana",
            "extensions": ["fe_launch"]
        },
        "application/vnd.desmume.movie": {
            "source": "iana"
        },
        "application/vnd.dir-bi.plate-dl-nosuffix": {
            "source": "iana"
        },
        "application/vnd.dm.delegation+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.dna": {
            "source": "iana",
            "extensions": ["dna"]
        },
        "application/vnd.document+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.dolby.mlp": {
            "source": "apache",
            "extensions": ["mlp"]
        },
        "application/vnd.dolby.mobile.1": {
            "source": "iana"
        },
        "application/vnd.dolby.mobile.2": {
            "source": "iana"
        },
        "application/vnd.doremir.scorecloud-binary-document": {
            "source": "iana"
        },
        "application/vnd.dpgraph": {
            "source": "iana",
            "extensions": ["dpg"]
        },
        "application/vnd.dreamfactory": {
            "source": "iana",
            "extensions": ["dfac"]
        },
        "application/vnd.drive+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.ds-keypoint": {
            "source": "apache",
            "extensions": ["kpxx"]
        },
        "application/vnd.dtg.local": {
            "source": "iana"
        },
        "application/vnd.dtg.local.flash": {
            "source": "iana"
        },
        "application/vnd.dtg.local.html": {
            "source": "iana"
        },
        "application/vnd.dvb.ait": {
            "source": "iana",
            "extensions": ["ait"]
        },
        "application/vnd.dvb.dvbj": {
            "source": "iana"
        },
        "application/vnd.dvb.esgcontainer": {
            "source": "iana"
        },
        "application/vnd.dvb.ipdcdftnotifaccess": {
            "source": "iana"
        },
        "application/vnd.dvb.ipdcesgaccess": {
            "source": "iana"
        },
        "application/vnd.dvb.ipdcesgaccess2": {
            "source": "iana"
        },
        "application/vnd.dvb.ipdcesgpdd": {
            "source": "iana"
        },
        "application/vnd.dvb.ipdcroaming": {
            "source": "iana"
        },
        "application/vnd.dvb.iptv.alfec-base": {
            "source": "iana"
        },
        "application/vnd.dvb.iptv.alfec-enhancement": {
            "source": "iana"
        },
        "application/vnd.dvb.notif-aggregate-root+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.dvb.notif-container+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.dvb.notif-generic+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.dvb.notif-ia-msglist+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.dvb.notif-ia-registration-request+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.dvb.notif-ia-registration-response+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.dvb.notif-init+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.dvb.pfr": {
            "source": "iana"
        },
        "application/vnd.dvb.service": {
            "source": "iana",
            "extensions": ["svc"]
        },
        "application/vnd.dxr": {
            "source": "iana"
        },
        "application/vnd.dynageo": {
            "source": "iana",
            "extensions": ["geo"]
        },
        "application/vnd.dzr": {
            "source": "iana"
        },
        "application/vnd.easykaraoke.cdgdownload": {
            "source": "iana"
        },
        "application/vnd.ecdis-update": {
            "source": "iana"
        },
        "application/vnd.ecip.rlp": {
            "source": "iana"
        },
        "application/vnd.ecowin.chart": {
            "source": "iana",
            "extensions": ["mag"]
        },
        "application/vnd.ecowin.filerequest": {
            "source": "iana"
        },
        "application/vnd.ecowin.fileupdate": {
            "source": "iana"
        },
        "application/vnd.ecowin.series": {
            "source": "iana"
        },
        "application/vnd.ecowin.seriesrequest": {
            "source": "iana"
        },
        "application/vnd.ecowin.seriesupdate": {
            "source": "iana"
        },
        "application/vnd.efi.img": {
            "source": "iana"
        },
        "application/vnd.efi.iso": {
            "source": "iana"
        },
        "application/vnd.emclient.accessrequest+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.enliven": {
            "source": "iana",
            "extensions": ["nml"]
        },
        "application/vnd.enphase.envoy": {
            "source": "iana"
        },
        "application/vnd.eprints.data+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.epson.esf": {
            "source": "iana",
            "extensions": ["esf"]
        },
        "application/vnd.epson.msf": {
            "source": "iana",
            "extensions": ["msf"]
        },
        "application/vnd.epson.quickanime": {
            "source": "iana",
            "extensions": ["qam"]
        },
        "application/vnd.epson.salt": {
            "source": "iana",
            "extensions": ["slt"]
        },
        "application/vnd.epson.ssf": {
            "source": "iana",
            "extensions": ["ssf"]
        },
        "application/vnd.ericsson.quickcall": {
            "source": "iana"
        },
        "application/vnd.espass-espass+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.eszigno3+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["es3", "et3"]
        },
        "application/vnd.etsi.aoc+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.asic-e+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.etsi.asic-s+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.etsi.cug+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.iptvcommand+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.iptvdiscovery+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.iptvprofile+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.iptvsad-bc+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.iptvsad-cod+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.iptvsad-npvr+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.iptvservice+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.iptvsync+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.iptvueprofile+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.mcid+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.mheg5": {
            "source": "iana"
        },
        "application/vnd.etsi.overload-control-policy-dataset+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.pstn+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.sci+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.simservs+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.timestamp-token": {
            "source": "iana"
        },
        "application/vnd.etsi.tsl+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.etsi.tsl.der": {
            "source": "iana"
        },
        "application/vnd.eudora.data": {
            "source": "iana"
        },
        "application/vnd.evolv.ecig.profile": {
            "source": "iana"
        },
        "application/vnd.evolv.ecig.settings": {
            "source": "iana"
        },
        "application/vnd.evolv.ecig.theme": {
            "source": "iana"
        },
        "application/vnd.exstream-empower+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.exstream-package": {
            "source": "iana"
        },
        "application/vnd.ezpix-album": {
            "source": "iana",
            "extensions": ["ez2"]
        },
        "application/vnd.ezpix-package": {
            "source": "iana",
            "extensions": ["ez3"]
        },
        "application/vnd.f-secure.mobile": {
            "source": "iana"
        },
        "application/vnd.fastcopy-disk-image": {
            "source": "iana"
        },
        "application/vnd.fdf": {
            "source": "iana",
            "extensions": ["fdf"]
        },
        "application/vnd.fdsn.mseed": {
            "source": "iana",
            "extensions": ["mseed"]
        },
        "application/vnd.fdsn.seed": {
            "source": "iana",
            "extensions": ["seed", "dataless"]
        },
        "application/vnd.ffsns": {
            "source": "iana"
        },
        "application/vnd.ficlab.flb+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.filmit.zfc": {
            "source": "iana"
        },
        "application/vnd.fints": {
            "source": "iana"
        },
        "application/vnd.firemonkeys.cloudcell": {
            "source": "iana"
        },
        "application/vnd.flographit": {
            "source": "iana",
            "extensions": ["gph"]
        },
        "application/vnd.fluxtime.clip": {
            "source": "iana",
            "extensions": ["ftc"]
        },
        "application/vnd.font-fontforge-sfd": {
            "source": "iana"
        },
        "application/vnd.framemaker": {
            "source": "iana",
            "extensions": ["fm", "frame", "maker", "book"]
        },
        "application/vnd.frogans.fnc": {
            "source": "iana",
            "extensions": ["fnc"]
        },
        "application/vnd.frogans.ltf": {
            "source": "iana",
            "extensions": ["ltf"]
        },
        "application/vnd.fsc.weblaunch": {
            "source": "iana",
            "extensions": ["fsc"]
        },
        "application/vnd.fujitsu.oasys": {
            "source": "iana",
            "extensions": ["oas"]
        },
        "application/vnd.fujitsu.oasys2": {
            "source": "iana",
            "extensions": ["oa2"]
        },
        "application/vnd.fujitsu.oasys3": {
            "source": "iana",
            "extensions": ["oa3"]
        },
        "application/vnd.fujitsu.oasysgp": {
            "source": "iana",
            "extensions": ["fg5"]
        },
        "application/vnd.fujitsu.oasysprs": {
            "source": "iana",
            "extensions": ["bh2"]
        },
        "application/vnd.fujixerox.art-ex": {
            "source": "iana"
        },
        "application/vnd.fujixerox.art4": {
            "source": "iana"
        },
        "application/vnd.fujixerox.ddd": {
            "source": "iana",
            "extensions": ["ddd"]
        },
        "application/vnd.fujixerox.docuworks": {
            "source": "iana",
            "extensions": ["xdw"]
        },
        "application/vnd.fujixerox.docuworks.binder": {
            "source": "iana",
            "extensions": ["xbd"]
        },
        "application/vnd.fujixerox.docuworks.container": {
            "source": "iana"
        },
        "application/vnd.fujixerox.hbpl": {
            "source": "iana"
        },
        "application/vnd.fut-misnet": {
            "source": "iana"
        },
        "application/vnd.futoin+cbor": {
            "source": "iana"
        },
        "application/vnd.futoin+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.fuzzysheet": {
            "source": "iana",
            "extensions": ["fzs"]
        },
        "application/vnd.genomatix.tuxedo": {
            "source": "iana",
            "extensions": ["txd"]
        },
        "application/vnd.gentics.grd+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.geo+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.geocube+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.geogebra.file": {
            "source": "iana",
            "extensions": ["ggb"]
        },
        "application/vnd.geogebra.tool": {
            "source": "iana",
            "extensions": ["ggt"]
        },
        "application/vnd.geometry-explorer": {
            "source": "iana",
            "extensions": ["gex", "gre"]
        },
        "application/vnd.geonext": {
            "source": "iana",
            "extensions": ["gxt"]
        },
        "application/vnd.geoplan": {
            "source": "iana",
            "extensions": ["g2w"]
        },
        "application/vnd.geospace": {
            "source": "iana",
            "extensions": ["g3w"]
        },
        "application/vnd.gerber": {
            "source": "iana"
        },
        "application/vnd.globalplatform.card-content-mgt": {
            "source": "iana"
        },
        "application/vnd.globalplatform.card-content-mgt-response": {
            "source": "iana"
        },
        "application/vnd.gmx": {
            "source": "iana",
            "extensions": ["gmx"]
        },
        "application/vnd.google-apps.document": {
            "compressible": false,
            "extensions": ["gdoc"]
        },
        "application/vnd.google-apps.presentation": {
            "compressible": false,
            "extensions": ["gslides"]
        },
        "application/vnd.google-apps.spreadsheet": {
            "compressible": false,
            "extensions": ["gsheet"]
        },
        "application/vnd.google-earth.kml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["kml"]
        },
        "application/vnd.google-earth.kmz": {
            "source": "iana",
            "compressible": false,
            "extensions": ["kmz"]
        },
        "application/vnd.gov.sk.e-form+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.gov.sk.e-form+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.gov.sk.xmldatacontainer+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.grafeq": {
            "source": "iana",
            "extensions": ["gqf", "gqs"]
        },
        "application/vnd.gridmp": {
            "source": "iana"
        },
        "application/vnd.groove-account": {
            "source": "iana",
            "extensions": ["gac"]
        },
        "application/vnd.groove-help": {
            "source": "iana",
            "extensions": ["ghf"]
        },
        "application/vnd.groove-identity-message": {
            "source": "iana",
            "extensions": ["gim"]
        },
        "application/vnd.groove-injector": {
            "source": "iana",
            "extensions": ["grv"]
        },
        "application/vnd.groove-tool-message": {
            "source": "iana",
            "extensions": ["gtm"]
        },
        "application/vnd.groove-tool-template": {
            "source": "iana",
            "extensions": ["tpl"]
        },
        "application/vnd.groove-vcard": {
            "source": "iana",
            "extensions": ["vcg"]
        },
        "application/vnd.hal+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.hal+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["hal"]
        },
        "application/vnd.handheld-entertainment+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["zmm"]
        },
        "application/vnd.hbci": {
            "source": "iana",
            "extensions": ["hbci"]
        },
        "application/vnd.hc+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.hcl-bireports": {
            "source": "iana"
        },
        "application/vnd.hdt": {
            "source": "iana"
        },
        "application/vnd.heroku+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.hhe.lesson-player": {
            "source": "iana",
            "extensions": ["les"]
        },
        "application/vnd.hp-hpgl": {
            "source": "iana",
            "extensions": ["hpgl"]
        },
        "application/vnd.hp-hpid": {
            "source": "iana",
            "extensions": ["hpid"]
        },
        "application/vnd.hp-hps": {
            "source": "iana",
            "extensions": ["hps"]
        },
        "application/vnd.hp-jlyt": {
            "source": "iana",
            "extensions": ["jlt"]
        },
        "application/vnd.hp-pcl": {
            "source": "iana",
            "extensions": ["pcl"]
        },
        "application/vnd.hp-pclxl": {
            "source": "iana",
            "extensions": ["pclxl"]
        },
        "application/vnd.httphone": {
            "source": "iana"
        },
        "application/vnd.hydrostatix.sof-data": {
            "source": "iana",
            "extensions": ["sfd-hdstx"]
        },
        "application/vnd.hyper+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.hyper-item+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.hyperdrive+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.hzn-3d-crossword": {
            "source": "iana"
        },
        "application/vnd.ibm.afplinedata": {
            "source": "iana"
        },
        "application/vnd.ibm.electronic-media": {
            "source": "iana"
        },
        "application/vnd.ibm.minipay": {
            "source": "iana",
            "extensions": ["mpy"]
        },
        "application/vnd.ibm.modcap": {
            "source": "iana",
            "extensions": ["afp", "listafp", "list3820"]
        },
        "application/vnd.ibm.rights-management": {
            "source": "iana",
            "extensions": ["irm"]
        },
        "application/vnd.ibm.secure-container": {
            "source": "iana",
            "extensions": ["sc"]
        },
        "application/vnd.iccprofile": {
            "source": "iana",
            "extensions": ["icc", "icm"]
        },
        "application/vnd.ieee.1905": {
            "source": "iana"
        },
        "application/vnd.igloader": {
            "source": "iana",
            "extensions": ["igl"]
        },
        "application/vnd.imagemeter.folder+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.imagemeter.image+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.immervision-ivp": {
            "source": "iana",
            "extensions": ["ivp"]
        },
        "application/vnd.immervision-ivu": {
            "source": "iana",
            "extensions": ["ivu"]
        },
        "application/vnd.ims.imsccv1p1": {
            "source": "iana"
        },
        "application/vnd.ims.imsccv1p2": {
            "source": "iana"
        },
        "application/vnd.ims.imsccv1p3": {
            "source": "iana"
        },
        "application/vnd.ims.lis.v2.result+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.ims.lti.v2.toolconsumerprofile+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.ims.lti.v2.toolproxy+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.ims.lti.v2.toolproxy.id+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.ims.lti.v2.toolsettings+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.ims.lti.v2.toolsettings.simple+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.informedcontrol.rms+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.informix-visionary": {
            "source": "iana"
        },
        "application/vnd.infotech.project": {
            "source": "iana"
        },
        "application/vnd.infotech.project+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.innopath.wamp.notification": {
            "source": "iana"
        },
        "application/vnd.insors.igm": {
            "source": "iana",
            "extensions": ["igm"]
        },
        "application/vnd.intercon.formnet": {
            "source": "iana",
            "extensions": ["xpw", "xpx"]
        },
        "application/vnd.intergeo": {
            "source": "iana",
            "extensions": ["i2g"]
        },
        "application/vnd.intertrust.digibox": {
            "source": "iana"
        },
        "application/vnd.intertrust.nncp": {
            "source": "iana"
        },
        "application/vnd.intu.qbo": {
            "source": "iana",
            "extensions": ["qbo"]
        },
        "application/vnd.intu.qfx": {
            "source": "iana",
            "extensions": ["qfx"]
        },
        "application/vnd.iptc.g2.catalogitem+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.iptc.g2.conceptitem+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.iptc.g2.knowledgeitem+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.iptc.g2.newsitem+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.iptc.g2.newsmessage+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.iptc.g2.packageitem+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.iptc.g2.planningitem+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.ipunplugged.rcprofile": {
            "source": "iana",
            "extensions": ["rcprofile"]
        },
        "application/vnd.irepository.package+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["irp"]
        },
        "application/vnd.is-xpr": {
            "source": "iana",
            "extensions": ["xpr"]
        },
        "application/vnd.isac.fcs": {
            "source": "iana",
            "extensions": ["fcs"]
        },
        "application/vnd.iso11783-10+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.jam": {
            "source": "iana",
            "extensions": ["jam"]
        },
        "application/vnd.japannet-directory-service": {
            "source": "iana"
        },
        "application/vnd.japannet-jpnstore-wakeup": {
            "source": "iana"
        },
        "application/vnd.japannet-payment-wakeup": {
            "source": "iana"
        },
        "application/vnd.japannet-registration": {
            "source": "iana"
        },
        "application/vnd.japannet-registration-wakeup": {
            "source": "iana"
        },
        "application/vnd.japannet-setstore-wakeup": {
            "source": "iana"
        },
        "application/vnd.japannet-verification": {
            "source": "iana"
        },
        "application/vnd.japannet-verification-wakeup": {
            "source": "iana"
        },
        "application/vnd.jcp.javame.midlet-rms": {
            "source": "iana",
            "extensions": ["rms"]
        },
        "application/vnd.jisp": {
            "source": "iana",
            "extensions": ["jisp"]
        },
        "application/vnd.joost.joda-archive": {
            "source": "iana",
            "extensions": ["joda"]
        },
        "application/vnd.jsk.isdn-ngn": {
            "source": "iana"
        },
        "application/vnd.kahootz": {
            "source": "iana",
            "extensions": ["ktz", "ktr"]
        },
        "application/vnd.kde.karbon": {
            "source": "iana",
            "extensions": ["karbon"]
        },
        "application/vnd.kde.kchart": {
            "source": "iana",
            "extensions": ["chrt"]
        },
        "application/vnd.kde.kformula": {
            "source": "iana",
            "extensions": ["kfo"]
        },
        "application/vnd.kde.kivio": {
            "source": "iana",
            "extensions": ["flw"]
        },
        "application/vnd.kde.kontour": {
            "source": "iana",
            "extensions": ["kon"]
        },
        "application/vnd.kde.kpresenter": {
            "source": "iana",
            "extensions": ["kpr", "kpt"]
        },
        "application/vnd.kde.kspread": {
            "source": "iana",
            "extensions": ["ksp"]
        },
        "application/vnd.kde.kword": {
            "source": "iana",
            "extensions": ["kwd", "kwt"]
        },
        "application/vnd.kenameaapp": {
            "source": "iana",
            "extensions": ["htke"]
        },
        "application/vnd.kidspiration": {
            "source": "iana",
            "extensions": ["kia"]
        },
        "application/vnd.kinar": {
            "source": "iana",
            "extensions": ["kne", "knp"]
        },
        "application/vnd.koan": {
            "source": "iana",
            "extensions": ["skp", "skd", "skt", "skm"]
        },
        "application/vnd.kodak-descriptor": {
            "source": "iana",
            "extensions": ["sse"]
        },
        "application/vnd.las": {
            "source": "iana"
        },
        "application/vnd.las.las+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.las.las+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["lasxml"]
        },
        "application/vnd.laszip": {
            "source": "iana"
        },
        "application/vnd.leap+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.liberty-request+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.llamagraphics.life-balance.desktop": {
            "source": "iana",
            "extensions": ["lbd"]
        },
        "application/vnd.llamagraphics.life-balance.exchange+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["lbe"]
        },
        "application/vnd.logipipe.circuit+zip": {
            "source": "iana",
            "compressible": false
        },
        "application/vnd.loom": {
            "source": "iana"
        },
        "application/vnd.lotus-1-2-3": {
            "source": "iana",
            "extensions": ["123"]
        },
        "application/vnd.lotus-approach": {
            "source": "iana",
            "extensions": ["apr"]
        },
        "application/vnd.lotus-freelance": {
            "source": "iana",
            "extensions": ["pre"]
        },
        "application/vnd.lotus-notes": {
            "source": "iana",
            "extensions": ["nsf"]
        },
        "application/vnd.lotus-organizer": {
            "source": "iana",
            "extensions": ["org"]
        },
        "application/vnd.lotus-screencam": {
            "source": "iana",
            "extensions": ["scm"]
        },
        "application/vnd.lotus-wordpro": {
            "source": "iana",
            "extensions": ["lwp"]
        },
        "application/vnd.macports.portpkg": {
            "source": "iana",
            "extensions": ["portpkg"]
        },
        "application/vnd.mapbox-vector-tile": {
            "source": "iana"
        },
        "application/vnd.marlin.drm.actiontoken+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.marlin.drm.conftoken+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.marlin.drm.license+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.marlin.drm.mdcf": {
            "source": "iana"
        },
        "application/vnd.mason+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.maxmind.maxmind-db": {
            "source": "iana"
        },
        "application/vnd.mcd": {
            "source": "iana",
            "extensions": ["mcd"]
        },
        "application/vnd.medcalcdata": {
            "source": "iana",
            "extensions": ["mc1"]
        },
        "application/vnd.mediastation.cdkey": {
            "source": "iana",
            "extensions": ["cdkey"]
        },
        "application/vnd.meridian-slingshot": {
            "source": "iana"
        },
        "application/vnd.mfer": {
            "source": "iana",
            "extensions": ["mwf"]
        },
        "application/vnd.mfmp": {
            "source": "iana",
            "extensions": ["mfm"]
        },
        "application/vnd.micro+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.micrografx.flo": {
            "source": "iana",
            "extensions": ["flo"]
        },
        "application/vnd.micrografx.igx": {
            "source": "iana",
            "extensions": ["igx"]
        },
        "application/vnd.microsoft.portable-executable": {
            "source": "iana"
        },
        "application/vnd.microsoft.windows.thumbnail-cache": {
            "source": "iana"
        },
        "application/vnd.miele+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.mif": {
            "source": "iana",
            "extensions": ["mif"]
        },
        "application/vnd.minisoft-hp3000-save": {
            "source": "iana"
        },
        "application/vnd.mitsubishi.misty-guard.trustweb": {
            "source": "iana"
        },
        "application/vnd.mobius.daf": {
            "source": "iana",
            "extensions": ["daf"]
        },
        "application/vnd.mobius.dis": {
            "source": "iana",
            "extensions": ["dis"]
        },
        "application/vnd.mobius.mbk": {
            "source": "iana",
            "extensions": ["mbk"]
        },
        "application/vnd.mobius.mqy": {
            "source": "iana",
            "extensions": ["mqy"]
        },
        "application/vnd.mobius.msl": {
            "source": "iana",
            "extensions": ["msl"]
        },
        "application/vnd.mobius.plc": {
            "source": "iana",
            "extensions": ["plc"]
        },
        "application/vnd.mobius.txf": {
            "source": "iana",
            "extensions": ["txf"]
        },
        "application/vnd.mophun.application": {
            "source": "iana",
            "extensions": ["mpn"]
        },
        "application/vnd.mophun.certificate": {
            "source": "iana",
            "extensions": ["mpc"]
        },
        "application/vnd.motorola.flexsuite": {
            "source": "iana"
        },
        "application/vnd.motorola.flexsuite.adsi": {
            "source": "iana"
        },
        "application/vnd.motorola.flexsuite.fis": {
            "source": "iana"
        },
        "application/vnd.motorola.flexsuite.gotap": {
            "source": "iana"
        },
        "application/vnd.motorola.flexsuite.kmr": {
            "source": "iana"
        },
        "application/vnd.motorola.flexsuite.ttc": {
            "source": "iana"
        },
        "application/vnd.motorola.flexsuite.wem": {
            "source": "iana"
        },
        "application/vnd.motorola.iprm": {
            "source": "iana"
        },
        "application/vnd.mozilla.xul+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xul"]
        },
        "application/vnd.ms-3mfdocument": {
            "source": "iana"
        },
        "application/vnd.ms-artgalry": {
            "source": "iana",
            "extensions": ["cil"]
        },
        "application/vnd.ms-asf": {
            "source": "iana"
        },
        "application/vnd.ms-cab-compressed": {
            "source": "iana",
            "extensions": ["cab"]
        },
        "application/vnd.ms-color.iccprofile": {
            "source": "apache"
        },
        "application/vnd.ms-excel": {
            "source": "iana",
            "compressible": false,
            "extensions": ["xls", "xlm", "xla", "xlc", "xlt", "xlw"]
        },
        "application/vnd.ms-excel.addin.macroenabled.12": {
            "source": "iana",
            "extensions": ["xlam"]
        },
        "application/vnd.ms-excel.sheet.binary.macroenabled.12": {
            "source": "iana",
            "extensions": ["xlsb"]
        },
        "application/vnd.ms-excel.sheet.macroenabled.12": {
            "source": "iana",
            "extensions": ["xlsm"]
        },
        "application/vnd.ms-excel.template.macroenabled.12": {
            "source": "iana",
            "extensions": ["xltm"]
        },
        "application/vnd.ms-fontobject": {
            "source": "iana",
            "compressible": true,
            "extensions": ["eot"]
        },
        "application/vnd.ms-htmlhelp": {
            "source": "iana",
            "extensions": ["chm"]
        },
        "application/vnd.ms-ims": {
            "source": "iana",
            "extensions": ["ims"]
        },
        "application/vnd.ms-lrm": {
            "source": "iana",
            "extensions": ["lrm"]
        },
        "application/vnd.ms-office.activex+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.ms-officetheme": {
            "source": "iana",
            "extensions": ["thmx"]
        },
        "application/vnd.ms-opentype": {
            "source": "apache",
            "compressible": true
        },
        "application/vnd.ms-outlook": {
            "compressible": false,
            "extensions": ["msg"]
        },
        "application/vnd.ms-package.obfuscated-opentype": {
            "source": "apache"
        },
        "application/vnd.ms-pki.seccat": {
            "source": "apache",
            "extensions": ["cat"]
        },
        "application/vnd.ms-pki.stl": {
            "source": "apache",
            "extensions": ["stl"]
        },
        "application/vnd.ms-playready.initiator+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.ms-powerpoint": {
            "source": "iana",
            "compressible": false,
            "extensions": ["ppt", "pps", "pot"]
        },
        "application/vnd.ms-powerpoint.addin.macroenabled.12": {
            "source": "iana",
            "extensions": ["ppam"]
        },
        "application/vnd.ms-powerpoint.presentation.macroenabled.12": {
            "source": "iana",
            "extensions": ["pptm"]
        },
        "application/vnd.ms-powerpoint.slide.macroenabled.12": {
            "source": "iana",
            "extensions": ["sldm"]
        },
        "application/vnd.ms-powerpoint.slideshow.macroenabled.12": {
            "source": "iana",
            "extensions": ["ppsm"]
        },
        "application/vnd.ms-powerpoint.template.macroenabled.12": {
            "source": "iana",
            "extensions": ["potm"]
        },
        "application/vnd.ms-printdevicecapabilities+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.ms-printing.printticket+xml": {
            "source": "apache",
            "compressible": true
        },
        "application/vnd.ms-printschematicket+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.ms-project": {
            "source": "iana",
            "extensions": ["mpp", "mpt"]
        },
        "application/vnd.ms-tnef": {
            "source": "iana"
        },
        "application/vnd.ms-windows.devicepairing": {
            "source": "iana"
        },
        "application/vnd.ms-windows.nwprinting.oob": {
            "source": "iana"
        },
        "application/vnd.ms-windows.printerpairing": {
            "source": "iana"
        },
        "application/vnd.ms-windows.wsd.oob": {
            "source": "iana"
        },
        "application/vnd.ms-wmdrm.lic-chlg-req": {
            "source": "iana"
        },
        "application/vnd.ms-wmdrm.lic-resp": {
            "source": "iana"
        },
        "application/vnd.ms-wmdrm.meter-chlg-req": {
            "source": "iana"
        },
        "application/vnd.ms-wmdrm.meter-resp": {
            "source": "iana"
        },
        "application/vnd.ms-word.document.macroenabled.12": {
            "source": "iana",
            "extensions": ["docm"]
        },
        "application/vnd.ms-word.template.macroenabled.12": {
            "source": "iana",
            "extensions": ["dotm"]
        },
        "application/vnd.ms-works": {
            "source": "iana",
            "extensions": ["wps", "wks", "wcm", "wdb"]
        },
        "application/vnd.ms-wpl": {
            "source": "iana",
            "extensions": ["wpl"]
        },
        "application/vnd.ms-xpsdocument": {
            "source": "iana",
            "compressible": false,
            "extensions": ["xps"]
        },
        "application/vnd.msa-disk-image": {
            "source": "iana"
        },
        "application/vnd.mseq": {
            "source": "iana",
            "extensions": ["mseq"]
        },
        "application/vnd.msign": {
            "source": "iana"
        },
        "application/vnd.multiad.creator": {
            "source": "iana"
        },
        "application/vnd.multiad.creator.cif": {
            "source": "iana"
        },
        "application/vnd.music-niff": {
            "source": "iana"
        },
        "application/vnd.musician": {
            "source": "iana",
            "extensions": ["mus"]
        },
        "application/vnd.muvee.style": {
            "source": "iana",
            "extensions": ["msty"]
        },
        "application/vnd.mynfc": {
            "source": "iana",
            "extensions": ["taglet"]
        },
        "application/vnd.ncd.control": {
            "source": "iana"
        },
        "application/vnd.ncd.reference": {
            "source": "iana"
        },
        "application/vnd.nearst.inv+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.nervana": {
            "source": "iana"
        },
        "application/vnd.netfpx": {
            "source": "iana"
        },
        "application/vnd.neurolanguage.nlu": {
            "source": "iana",
            "extensions": ["nlu"]
        },
        "application/vnd.nimn": {
            "source": "iana"
        },
        "application/vnd.nintendo.nitro.rom": {
            "source": "iana"
        },
        "application/vnd.nintendo.snes.rom": {
            "source": "iana"
        },
        "application/vnd.nitf": {
            "source": "iana",
            "extensions": ["ntf", "nitf"]
        },
        "application/vnd.noblenet-directory": {
            "source": "iana",
            "extensions": ["nnd"]
        },
        "application/vnd.noblenet-sealer": {
            "source": "iana",
            "extensions": ["nns"]
        },
        "application/vnd.noblenet-web": {
            "source": "iana",
            "extensions": ["nnw"]
        },
        "application/vnd.nokia.catalogs": {
            "source": "iana"
        },
        "application/vnd.nokia.conml+wbxml": {
            "source": "iana"
        },
        "application/vnd.nokia.conml+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.nokia.iptv.config+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.nokia.isds-radio-presets": {
            "source": "iana"
        },
        "application/vnd.nokia.landmark+wbxml": {
            "source": "iana"
        },
        "application/vnd.nokia.landmark+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.nokia.landmarkcollection+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.nokia.n-gage.ac+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["ac"]
        },
        "application/vnd.nokia.n-gage.data": {
            "source": "iana",
            "extensions": ["ngdat"]
        },
        "application/vnd.nokia.n-gage.symbian.install": {
            "source": "iana",
            "extensions": ["n-gage"]
        },
        "application/vnd.nokia.ncd": {
            "source": "iana"
        },
        "application/vnd.nokia.pcd+wbxml": {
            "source": "iana"
        },
        "application/vnd.nokia.pcd+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.nokia.radio-preset": {
            "source": "iana",
            "extensions": ["rpst"]
        },
        "application/vnd.nokia.radio-presets": {
            "source": "iana",
            "extensions": ["rpss"]
        },
        "application/vnd.novadigm.edm": {
            "source": "iana",
            "extensions": ["edm"]
        },
        "application/vnd.novadigm.edx": {
            "source": "iana",
            "extensions": ["edx"]
        },
        "application/vnd.novadigm.ext": {
            "source": "iana",
            "extensions": ["ext"]
        },
        "application/vnd.ntt-local.content-share": {
            "source": "iana"
        },
        "application/vnd.ntt-local.file-transfer": {
            "source": "iana"
        },
        "application/vnd.ntt-local.ogw_remote-access": {
            "source": "iana"
        },
        "application/vnd.ntt-local.sip-ta_remote": {
            "source": "iana"
        },
        "application/vnd.ntt-local.sip-ta_tcp_stream": {
            "source": "iana"
        },
        "application/vnd.oasis.opendocument.chart": {
            "source": "iana",
            "extensions": ["odc"]
        },
        "application/vnd.oasis.opendocument.chart-template": {
            "source": "iana",
            "extensions": ["otc"]
        },
        "application/vnd.oasis.opendocument.database": {
            "source": "iana",
            "extensions": ["odb"]
        },
        "application/vnd.oasis.opendocument.formula": {
            "source": "iana",
            "extensions": ["odf"]
        },
        "application/vnd.oasis.opendocument.formula-template": {
            "source": "iana",
            "extensions": ["odft"]
        },
        "application/vnd.oasis.opendocument.graphics": {
            "source": "iana",
            "compressible": false,
            "extensions": ["odg"]
        },
        "application/vnd.oasis.opendocument.graphics-template": {
            "source": "iana",
            "extensions": ["otg"]
        },
        "application/vnd.oasis.opendocument.image": {
            "source": "iana",
            "extensions": ["odi"]
        },
        "application/vnd.oasis.opendocument.image-template": {
            "source": "iana",
            "extensions": ["oti"]
        },
        "application/vnd.oasis.opendocument.presentation": {
            "source": "iana",
            "compressible": false,
            "extensions": ["odp"]
        },
        "application/vnd.oasis.opendocument.presentation-template": {
            "source": "iana",
            "extensions": ["otp"]
        },
        "application/vnd.oasis.opendocument.spreadsheet": {
            "source": "iana",
            "compressible": false,
            "extensions": ["ods"]
        },
        "application/vnd.oasis.opendocument.spreadsheet-template": {
            "source": "iana",
            "extensions": ["ots"]
        },
        "application/vnd.oasis.opendocument.text": {
            "source": "iana",
            "compressible": false,
            "extensions": ["odt"]
        },
        "application/vnd.oasis.opendocument.text-master": {
            "source": "iana",
            "extensions": ["odm"]
        },
        "application/vnd.oasis.opendocument.text-template": {
            "source": "iana",
            "extensions": ["ott"]
        },
        "application/vnd.oasis.opendocument.text-web": {
            "source": "iana",
            "extensions": ["oth"]
        },
        "application/vnd.obn": {
            "source": "iana"
        },
        "application/vnd.ocf+cbor": {
            "source": "iana"
        },
        "application/vnd.oftn.l10n+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oipf.contentaccessdownload+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oipf.contentaccessstreaming+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oipf.cspg-hexbinary": {
            "source": "iana"
        },
        "application/vnd.oipf.dae.svg+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oipf.dae.xhtml+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oipf.mippvcontrolmessage+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oipf.pae.gem": {
            "source": "iana"
        },
        "application/vnd.oipf.spdiscovery+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oipf.spdlist+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oipf.ueprofile+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oipf.userprofile+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.olpc-sugar": {
            "source": "iana",
            "extensions": ["xo"]
        },
        "application/vnd.oma-scws-config": {
            "source": "iana"
        },
        "application/vnd.oma-scws-http-request": {
            "source": "iana"
        },
        "application/vnd.oma-scws-http-response": {
            "source": "iana"
        },
        "application/vnd.oma.bcast.associated-procedure-parameter+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.bcast.drm-trigger+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.bcast.imd+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.bcast.ltkm": {
            "source": "iana"
        },
        "application/vnd.oma.bcast.notification+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.bcast.provisioningtrigger": {
            "source": "iana"
        },
        "application/vnd.oma.bcast.sgboot": {
            "source": "iana"
        },
        "application/vnd.oma.bcast.sgdd+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.bcast.sgdu": {
            "source": "iana"
        },
        "application/vnd.oma.bcast.simple-symbol-container": {
            "source": "iana"
        },
        "application/vnd.oma.bcast.smartcard-trigger+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.bcast.sprov+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.bcast.stkm": {
            "source": "iana"
        },
        "application/vnd.oma.cab-address-book+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.cab-feature-handler+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.cab-pcc+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.cab-subs-invite+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.cab-user-prefs+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.dcd": {
            "source": "iana"
        },
        "application/vnd.oma.dcdc": {
            "source": "iana"
        },
        "application/vnd.oma.dd2+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["dd2"]
        },
        "application/vnd.oma.drm.risd+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.group-usage-list+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.lwm2m+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.lwm2m+tlv": {
            "source": "iana"
        },
        "application/vnd.oma.pal+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.poc.detailed-progress-report+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.poc.final-report+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.poc.groups+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.poc.invocation-descriptor+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.poc.optimized-progress-report+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.push": {
            "source": "iana"
        },
        "application/vnd.oma.scidm.messages+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oma.xcap-directory+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.omads-email+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.omads-file+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.omads-folder+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.omaloc-supl-init": {
            "source": "iana"
        },
        "application/vnd.onepager": {
            "source": "iana"
        },
        "application/vnd.onepagertamp": {
            "source": "iana"
        },
        "application/vnd.onepagertamx": {
            "source": "iana"
        },
        "application/vnd.onepagertat": {
            "source": "iana"
        },
        "application/vnd.onepagertatp": {
            "source": "iana"
        },
        "application/vnd.onepagertatx": {
            "source": "iana"
        },
        "application/vnd.openblox.game+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["obgx"]
        },
        "application/vnd.openblox.game-binary": {
            "source": "iana"
        },
        "application/vnd.openeye.oeb": {
            "source": "iana"
        },
        "application/vnd.openofficeorg.extension": {
            "source": "apache",
            "extensions": ["oxt"]
        },
        "application/vnd.openstreetmap.data+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["osm"]
        },
        "application/vnd.openxmlformats-officedocument.custom-properties+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.customxmlproperties+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.drawing+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.drawingml.chart+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.drawingml.chartshapes+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.drawingml.diagramcolors+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.drawingml.diagramdata+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.drawingml.diagramlayout+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.drawingml.diagramstyle+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.extended-properties+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.commentauthors+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.comments+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.handoutmaster+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.notesmaster+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.notesslide+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.presentation": {
            "source": "iana",
            "compressible": false,
            "extensions": ["pptx"]
        },
        "application/vnd.openxmlformats-officedocument.presentationml.presentation.main+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.presprops+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.slide": {
            "source": "iana",
            "extensions": ["sldx"]
        },
        "application/vnd.openxmlformats-officedocument.presentationml.slide+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.slidelayout+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.slidemaster+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.slideshow": {
            "source": "iana",
            "extensions": ["ppsx"]
        },
        "application/vnd.openxmlformats-officedocument.presentationml.slideshow.main+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.slideupdateinfo+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.tablestyles+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.tags+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.template": {
            "source": "iana",
            "extensions": ["potx"]
        },
        "application/vnd.openxmlformats-officedocument.presentationml.template.main+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.presentationml.viewprops+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.calcchain+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.chartsheet+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.comments+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.connections+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.dialogsheet+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.externallink+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.pivotcachedefinition+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.pivotcacherecords+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.pivottable+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.querytable+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.revisionheaders+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.revisionlog+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.sharedstrings+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet": {
            "source": "iana",
            "compressible": false,
            "extensions": ["xlsx"]
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheetmetadata+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.styles+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.table+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.tablesinglecells+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.template": {
            "source": "iana",
            "extensions": ["xltx"]
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.template.main+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.usernames+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.volatiledependencies+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.theme+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.themeoverride+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.vmldrawing": {
            "source": "iana"
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.comments+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.document": {
            "source": "iana",
            "compressible": false,
            "extensions": ["docx"]
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.document.glossary+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.endnotes+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.fonttable+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.footer+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.footnotes+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.numbering+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.settings+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.styles+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.template": {
            "source": "iana",
            "extensions": ["dotx"]
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.template.main+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-officedocument.wordprocessingml.websettings+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-package.core-properties+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-package.digital-signature-xmlsignature+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.openxmlformats-package.relationships+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oracle.resource+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.orange.indata": {
            "source": "iana"
        },
        "application/vnd.osa.netdeploy": {
            "source": "iana"
        },
        "application/vnd.osgeo.mapguide.package": {
            "source": "iana",
            "extensions": ["mgp"]
        },
        "application/vnd.osgi.bundle": {
            "source": "iana"
        },
        "application/vnd.osgi.dp": {
            "source": "iana",
            "extensions": ["dp"]
        },
        "application/vnd.osgi.subsystem": {
            "source": "iana",
            "extensions": ["esa"]
        },
        "application/vnd.otps.ct-kip+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.oxli.countgraph": {
            "source": "iana"
        },
        "application/vnd.pagerduty+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.palm": {
            "source": "iana",
            "extensions": ["pdb", "pqa", "oprc"]
        },
        "application/vnd.panoply": {
            "source": "iana"
        },
        "application/vnd.paos.xml": {
            "source": "iana"
        },
        "application/vnd.patentdive": {
            "source": "iana"
        },
        "application/vnd.patientecommsdoc": {
            "source": "iana"
        },
        "application/vnd.pawaafile": {
            "source": "iana",
            "extensions": ["paw"]
        },
        "application/vnd.pcos": {
            "source": "iana"
        },
        "application/vnd.pg.format": {
            "source": "iana",
            "extensions": ["str"]
        },
        "application/vnd.pg.osasli": {
            "source": "iana",
            "extensions": ["ei6"]
        },
        "application/vnd.piaccess.application-licence": {
            "source": "iana"
        },
        "application/vnd.picsel": {
            "source": "iana",
            "extensions": ["efif"]
        },
        "application/vnd.pmi.widget": {
            "source": "iana",
            "extensions": ["wg"]
        },
        "application/vnd.poc.group-advertisement+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.pocketlearn": {
            "source": "iana",
            "extensions": ["plf"]
        },
        "application/vnd.powerbuilder6": {
            "source": "iana",
            "extensions": ["pbd"]
        },
        "application/vnd.powerbuilder6-s": {
            "source": "iana"
        },
        "application/vnd.powerbuilder7": {
            "source": "iana"
        },
        "application/vnd.powerbuilder7-s": {
            "source": "iana"
        },
        "application/vnd.powerbuilder75": {
            "source": "iana"
        },
        "application/vnd.powerbuilder75-s": {
            "source": "iana"
        },
        "application/vnd.preminet": {
            "source": "iana"
        },
        "application/vnd.previewsystems.box": {
            "source": "iana",
            "extensions": ["box"]
        },
        "application/vnd.proteus.magazine": {
            "source": "iana",
            "extensions": ["mgz"]
        },
        "application/vnd.psfs": {
            "source": "iana"
        },
        "application/vnd.publishare-delta-tree": {
            "source": "iana",
            "extensions": ["qps"]
        },
        "application/vnd.pvi.ptid1": {
            "source": "iana",
            "extensions": ["ptid"]
        },
        "application/vnd.pwg-multiplexed": {
            "source": "iana"
        },
        "application/vnd.pwg-xhtml-print+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.qualcomm.brew-app-res": {
            "source": "iana"
        },
        "application/vnd.quarantainenet": {
            "source": "iana"
        },
        "application/vnd.quark.quarkxpress": {
            "source": "iana",
            "extensions": ["qxd", "qxt", "qwd", "qwt", "qxl", "qxb"]
        },
        "application/vnd.quobject-quoxdocument": {
            "source": "iana"
        },
        "application/vnd.radisys.moml+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-audit+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-audit-conf+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-audit-conn+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-audit-dialog+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-audit-stream+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-conf+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-dialog+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-dialog-base+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-dialog-fax-detect+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-dialog-fax-sendrecv+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-dialog-group+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-dialog-speech+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.radisys.msml-dialog-transform+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.rainstor.data": {
            "source": "iana"
        },
        "application/vnd.rapid": {
            "source": "iana"
        },
        "application/vnd.rar": {
            "source": "iana"
        },
        "application/vnd.realvnc.bed": {
            "source": "iana",
            "extensions": ["bed"]
        },
        "application/vnd.recordare.musicxml": {
            "source": "iana",
            "extensions": ["mxl"]
        },
        "application/vnd.recordare.musicxml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["musicxml"]
        },
        "application/vnd.renlearn.rlprint": {
            "source": "iana"
        },
        "application/vnd.restful+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.rig.cryptonote": {
            "source": "iana",
            "extensions": ["cryptonote"]
        },
        "application/vnd.rim.cod": {
            "source": "apache",
            "extensions": ["cod"]
        },
        "application/vnd.rn-realmedia": {
            "source": "apache",
            "extensions": ["rm"]
        },
        "application/vnd.rn-realmedia-vbr": {
            "source": "apache",
            "extensions": ["rmvb"]
        },
        "application/vnd.route66.link66+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["link66"]
        },
        "application/vnd.rs-274x": {
            "source": "iana"
        },
        "application/vnd.ruckus.download": {
            "source": "iana"
        },
        "application/vnd.s3sms": {
            "source": "iana"
        },
        "application/vnd.sailingtracker.track": {
            "source": "iana",
            "extensions": ["st"]
        },
        "application/vnd.sbm.cid": {
            "source": "iana"
        },
        "application/vnd.sbm.mid2": {
            "source": "iana"
        },
        "application/vnd.scribus": {
            "source": "iana"
        },
        "application/vnd.sealed.3df": {
            "source": "iana"
        },
        "application/vnd.sealed.csf": {
            "source": "iana"
        },
        "application/vnd.sealed.doc": {
            "source": "iana"
        },
        "application/vnd.sealed.eml": {
            "source": "iana"
        },
        "application/vnd.sealed.mht": {
            "source": "iana"
        },
        "application/vnd.sealed.net": {
            "source": "iana"
        },
        "application/vnd.sealed.ppt": {
            "source": "iana"
        },
        "application/vnd.sealed.tiff": {
            "source": "iana"
        },
        "application/vnd.sealed.xls": {
            "source": "iana"
        },
        "application/vnd.sealedmedia.softseal.html": {
            "source": "iana"
        },
        "application/vnd.sealedmedia.softseal.pdf": {
            "source": "iana"
        },
        "application/vnd.seemail": {
            "source": "iana",
            "extensions": ["see"]
        },
        "application/vnd.sema": {
            "source": "iana",
            "extensions": ["sema"]
        },
        "application/vnd.semd": {
            "source": "iana",
            "extensions": ["semd"]
        },
        "application/vnd.semf": {
            "source": "iana",
            "extensions": ["semf"]
        },
        "application/vnd.shade-save-file": {
            "source": "iana"
        },
        "application/vnd.shana.informed.formdata": {
            "source": "iana",
            "extensions": ["ifm"]
        },
        "application/vnd.shana.informed.formtemplate": {
            "source": "iana",
            "extensions": ["itp"]
        },
        "application/vnd.shana.informed.interchange": {
            "source": "iana",
            "extensions": ["iif"]
        },
        "application/vnd.shana.informed.package": {
            "source": "iana",
            "extensions": ["ipk"]
        },
        "application/vnd.shootproof+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.shopkick+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.sigrok.session": {
            "source": "iana"
        },
        "application/vnd.simtech-mindmapper": {
            "source": "iana",
            "extensions": ["twd", "twds"]
        },
        "application/vnd.siren+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.smaf": {
            "source": "iana",
            "extensions": ["mmf"]
        },
        "application/vnd.smart.notebook": {
            "source": "iana"
        },
        "application/vnd.smart.teacher": {
            "source": "iana",
            "extensions": ["teacher"]
        },
        "application/vnd.software602.filler.form+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["fo"]
        },
        "application/vnd.software602.filler.form-xml-zip": {
            "source": "iana"
        },
        "application/vnd.solent.sdkm+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["sdkm", "sdkd"]
        },
        "application/vnd.spotfire.dxp": {
            "source": "iana",
            "extensions": ["dxp"]
        },
        "application/vnd.spotfire.sfs": {
            "source": "iana",
            "extensions": ["sfs"]
        },
        "application/vnd.sqlite3": {
            "source": "iana"
        },
        "application/vnd.sss-cod": {
            "source": "iana"
        },
        "application/vnd.sss-dtf": {
            "source": "iana"
        },
        "application/vnd.sss-ntf": {
            "source": "iana"
        },
        "application/vnd.stardivision.calc": {
            "source": "apache",
            "extensions": ["sdc"]
        },
        "application/vnd.stardivision.draw": {
            "source": "apache",
            "extensions": ["sda"]
        },
        "application/vnd.stardivision.impress": {
            "source": "apache",
            "extensions": ["sdd"]
        },
        "application/vnd.stardivision.math": {
            "source": "apache",
            "extensions": ["smf"]
        },
        "application/vnd.stardivision.writer": {
            "source": "apache",
            "extensions": ["sdw", "vor"]
        },
        "application/vnd.stardivision.writer-global": {
            "source": "apache",
            "extensions": ["sgl"]
        },
        "application/vnd.stepmania.package": {
            "source": "iana",
            "extensions": ["smzip"]
        },
        "application/vnd.stepmania.stepchart": {
            "source": "iana",
            "extensions": ["sm"]
        },
        "application/vnd.street-stream": {
            "source": "iana"
        },
        "application/vnd.sun.wadl+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["wadl"]
        },
        "application/vnd.sun.xml.calc": {
            "source": "apache",
            "extensions": ["sxc"]
        },
        "application/vnd.sun.xml.calc.template": {
            "source": "apache",
            "extensions": ["stc"]
        },
        "application/vnd.sun.xml.draw": {
            "source": "apache",
            "extensions": ["sxd"]
        },
        "application/vnd.sun.xml.draw.template": {
            "source": "apache",
            "extensions": ["std"]
        },
        "application/vnd.sun.xml.impress": {
            "source": "apache",
            "extensions": ["sxi"]
        },
        "application/vnd.sun.xml.impress.template": {
            "source": "apache",
            "extensions": ["sti"]
        },
        "application/vnd.sun.xml.math": {
            "source": "apache",
            "extensions": ["sxm"]
        },
        "application/vnd.sun.xml.writer": {
            "source": "apache",
            "extensions": ["sxw"]
        },
        "application/vnd.sun.xml.writer.global": {
            "source": "apache",
            "extensions": ["sxg"]
        },
        "application/vnd.sun.xml.writer.template": {
            "source": "apache",
            "extensions": ["stw"]
        },
        "application/vnd.sus-calendar": {
            "source": "iana",
            "extensions": ["sus", "susp"]
        },
        "application/vnd.svd": {
            "source": "iana",
            "extensions": ["svd"]
        },
        "application/vnd.swiftview-ics": {
            "source": "iana"
        },
        "application/vnd.symbian.install": {
            "source": "apache",
            "extensions": ["sis", "sisx"]
        },
        "application/vnd.syncml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xsm"]
        },
        "application/vnd.syncml.dm+wbxml": {
            "source": "iana",
            "extensions": ["bdm"]
        },
        "application/vnd.syncml.dm+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xdm"]
        },
        "application/vnd.syncml.dm.notification": {
            "source": "iana"
        },
        "application/vnd.syncml.dmddf+wbxml": {
            "source": "iana"
        },
        "application/vnd.syncml.dmddf+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["ddf"]
        },
        "application/vnd.syncml.dmtnds+wbxml": {
            "source": "iana"
        },
        "application/vnd.syncml.dmtnds+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.syncml.ds.notification": {
            "source": "iana"
        },
        "application/vnd.tableschema+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.tao.intent-module-archive": {
            "source": "iana",
            "extensions": ["tao"]
        },
        "application/vnd.tcpdump.pcap": {
            "source": "iana",
            "extensions": ["pcap", "cap", "dmp"]
        },
        "application/vnd.think-cell.ppttc+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.tmd.mediaflex.api+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.tml": {
            "source": "iana"
        },
        "application/vnd.tmobile-livetv": {
            "source": "iana",
            "extensions": ["tmo"]
        },
        "application/vnd.tri.onesource": {
            "source": "iana"
        },
        "application/vnd.trid.tpt": {
            "source": "iana",
            "extensions": ["tpt"]
        },
        "application/vnd.triscape.mxs": {
            "source": "iana",
            "extensions": ["mxs"]
        },
        "application/vnd.trueapp": {
            "source": "iana",
            "extensions": ["tra"]
        },
        "application/vnd.truedoc": {
            "source": "iana"
        },
        "application/vnd.ubisoft.webplayer": {
            "source": "iana"
        },
        "application/vnd.ufdl": {
            "source": "iana",
            "extensions": ["ufd", "ufdl"]
        },
        "application/vnd.uiq.theme": {
            "source": "iana",
            "extensions": ["utz"]
        },
        "application/vnd.umajin": {
            "source": "iana",
            "extensions": ["umj"]
        },
        "application/vnd.unity": {
            "source": "iana",
            "extensions": ["unityweb"]
        },
        "application/vnd.uoml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["uoml"]
        },
        "application/vnd.uplanet.alert": {
            "source": "iana"
        },
        "application/vnd.uplanet.alert-wbxml": {
            "source": "iana"
        },
        "application/vnd.uplanet.bearer-choice": {
            "source": "iana"
        },
        "application/vnd.uplanet.bearer-choice-wbxml": {
            "source": "iana"
        },
        "application/vnd.uplanet.cacheop": {
            "source": "iana"
        },
        "application/vnd.uplanet.cacheop-wbxml": {
            "source": "iana"
        },
        "application/vnd.uplanet.channel": {
            "source": "iana"
        },
        "application/vnd.uplanet.channel-wbxml": {
            "source": "iana"
        },
        "application/vnd.uplanet.list": {
            "source": "iana"
        },
        "application/vnd.uplanet.list-wbxml": {
            "source": "iana"
        },
        "application/vnd.uplanet.listcmd": {
            "source": "iana"
        },
        "application/vnd.uplanet.listcmd-wbxml": {
            "source": "iana"
        },
        "application/vnd.uplanet.signal": {
            "source": "iana"
        },
        "application/vnd.uri-map": {
            "source": "iana"
        },
        "application/vnd.valve.source.material": {
            "source": "iana"
        },
        "application/vnd.vcx": {
            "source": "iana",
            "extensions": ["vcx"]
        },
        "application/vnd.vd-study": {
            "source": "iana"
        },
        "application/vnd.vectorworks": {
            "source": "iana"
        },
        "application/vnd.vel+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.verimatrix.vcas": {
            "source": "iana"
        },
        "application/vnd.veryant.thin": {
            "source": "iana"
        },
        "application/vnd.ves.encrypted": {
            "source": "iana"
        },
        "application/vnd.vidsoft.vidconference": {
            "source": "iana"
        },
        "application/vnd.visio": {
            "source": "iana",
            "extensions": ["vsd", "vst", "vss", "vsw"]
        },
        "application/vnd.visionary": {
            "source": "iana",
            "extensions": ["vis"]
        },
        "application/vnd.vividence.scriptfile": {
            "source": "iana"
        },
        "application/vnd.vsf": {
            "source": "iana",
            "extensions": ["vsf"]
        },
        "application/vnd.wap.sic": {
            "source": "iana"
        },
        "application/vnd.wap.slc": {
            "source": "iana"
        },
        "application/vnd.wap.wbxml": {
            "source": "iana",
            "extensions": ["wbxml"]
        },
        "application/vnd.wap.wmlc": {
            "source": "iana",
            "extensions": ["wmlc"]
        },
        "application/vnd.wap.wmlscriptc": {
            "source": "iana",
            "extensions": ["wmlsc"]
        },
        "application/vnd.webturbo": {
            "source": "iana",
            "extensions": ["wtb"]
        },
        "application/vnd.wfa.p2p": {
            "source": "iana"
        },
        "application/vnd.wfa.wsc": {
            "source": "iana"
        },
        "application/vnd.windows.devicepairing": {
            "source": "iana"
        },
        "application/vnd.wmc": {
            "source": "iana"
        },
        "application/vnd.wmf.bootstrap": {
            "source": "iana"
        },
        "application/vnd.wolfram.mathematica": {
            "source": "iana"
        },
        "application/vnd.wolfram.mathematica.package": {
            "source": "iana"
        },
        "application/vnd.wolfram.player": {
            "source": "iana",
            "extensions": ["nbp"]
        },
        "application/vnd.wordperfect": {
            "source": "iana",
            "extensions": ["wpd"]
        },
        "application/vnd.wqd": {
            "source": "iana",
            "extensions": ["wqd"]
        },
        "application/vnd.wrq-hp3000-labelled": {
            "source": "iana"
        },
        "application/vnd.wt.stf": {
            "source": "iana",
            "extensions": ["stf"]
        },
        "application/vnd.wv.csp+wbxml": {
            "source": "iana"
        },
        "application/vnd.wv.csp+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.wv.ssp+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.xacml+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.xara": {
            "source": "iana",
            "extensions": ["xar"]
        },
        "application/vnd.xfdl": {
            "source": "iana",
            "extensions": ["xfdl"]
        },
        "application/vnd.xfdl.webform": {
            "source": "iana"
        },
        "application/vnd.xmi+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/vnd.xmpie.cpkg": {
            "source": "iana"
        },
        "application/vnd.xmpie.dpkg": {
            "source": "iana"
        },
        "application/vnd.xmpie.plan": {
            "source": "iana"
        },
        "application/vnd.xmpie.ppkg": {
            "source": "iana"
        },
        "application/vnd.xmpie.xlim": {
            "source": "iana"
        },
        "application/vnd.yamaha.hv-dic": {
            "source": "iana",
            "extensions": ["hvd"]
        },
        "application/vnd.yamaha.hv-script": {
            "source": "iana",
            "extensions": ["hvs"]
        },
        "application/vnd.yamaha.hv-voice": {
            "source": "iana",
            "extensions": ["hvp"]
        },
        "application/vnd.yamaha.openscoreformat": {
            "source": "iana",
            "extensions": ["osf"]
        },
        "application/vnd.yamaha.openscoreformat.osfpvg+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["osfpvg"]
        },
        "application/vnd.yamaha.remote-setup": {
            "source": "iana"
        },
        "application/vnd.yamaha.smaf-audio": {
            "source": "iana",
            "extensions": ["saf"]
        },
        "application/vnd.yamaha.smaf-phrase": {
            "source": "iana",
            "extensions": ["spf"]
        },
        "application/vnd.yamaha.through-ngn": {
            "source": "iana"
        },
        "application/vnd.yamaha.tunnel-udpencap": {
            "source": "iana"
        },
        "application/vnd.yaoweme": {
            "source": "iana"
        },
        "application/vnd.yellowriver-custom-menu": {
            "source": "iana",
            "extensions": ["cmp"]
        },
        "application/vnd.youtube.yt": {
            "source": "iana"
        },
        "application/vnd.zul": {
            "source": "iana",
            "extensions": ["zir", "zirz"]
        },
        "application/vnd.zzazz.deck+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["zaz"]
        },
        "application/voicexml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["vxml"]
        },
        "application/voucher-cms+json": {
            "source": "iana",
            "compressible": true
        },
        "application/vq-rtcpxr": {
            "source": "iana"
        },
        "application/wasm": {
            "compressible": true,
            "extensions": ["wasm"]
        },
        "application/watcherinfo+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/webpush-options+json": {
            "source": "iana",
            "compressible": true
        },
        "application/whoispp-query": {
            "source": "iana"
        },
        "application/whoispp-response": {
            "source": "iana"
        },
        "application/widget": {
            "source": "iana",
            "extensions": ["wgt"]
        },
        "application/winhlp": {
            "source": "apache",
            "extensions": ["hlp"]
        },
        "application/wita": {
            "source": "iana"
        },
        "application/wordperfect5.1": {
            "source": "iana"
        },
        "application/wsdl+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["wsdl"]
        },
        "application/wspolicy+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["wspolicy"]
        },
        "application/x-7z-compressed": {
            "source": "apache",
            "compressible": false,
            "extensions": ["7z"]
        },
        "application/x-abiword": {
            "source": "apache",
            "extensions": ["abw"]
        },
        "application/x-ace-compressed": {
            "source": "apache",
            "extensions": ["ace"]
        },
        "application/x-amf": {
            "source": "apache"
        },
        "application/x-apple-diskimage": {
            "source": "apache",
            "extensions": ["dmg"]
        },
        "application/x-arj": {
            "compressible": false,
            "extensions": ["arj"]
        },
        "application/x-authorware-bin": {
            "source": "apache",
            "extensions": ["aab", "x32", "u32", "vox"]
        },
        "application/x-authorware-map": {
            "source": "apache",
            "extensions": ["aam"]
        },
        "application/x-authorware-seg": {
            "source": "apache",
            "extensions": ["aas"]
        },
        "application/x-bcpio": {
            "source": "apache",
            "extensions": ["bcpio"]
        },
        "application/x-bdoc": {
            "compressible": false,
            "extensions": ["bdoc"]
        },
        "application/x-bittorrent": {
            "source": "apache",
            "extensions": ["torrent"]
        },
        "application/x-blorb": {
            "source": "apache",
            "extensions": ["blb", "blorb"]
        },
        "application/x-bzip": {
            "source": "apache",
            "compressible": false,
            "extensions": ["bz"]
        },
        "application/x-bzip2": {
            "source": "apache",
            "compressible": false,
            "extensions": ["bz2", "boz"]
        },
        "application/x-cbr": {
            "source": "apache",
            "extensions": ["cbr", "cba", "cbt", "cbz", "cb7"]
        },
        "application/x-cdlink": {
            "source": "apache",
            "extensions": ["vcd"]
        },
        "application/x-cfs-compressed": {
            "source": "apache",
            "extensions": ["cfs"]
        },
        "application/x-chat": {
            "source": "apache",
            "extensions": ["chat"]
        },
        "application/x-chess-pgn": {
            "source": "apache",
            "extensions": ["pgn"]
        },
        "application/x-chrome-extension": {
            "extensions": ["crx"]
        },
        "application/x-cocoa": {
            "source": "nginx",
            "extensions": ["cco"]
        },
        "application/x-compress": {
            "source": "apache"
        },
        "application/x-conference": {
            "source": "apache",
            "extensions": ["nsc"]
        },
        "application/x-cpio": {
            "source": "apache",
            "extensions": ["cpio"]
        },
        "application/x-csh": {
            "source": "apache",
            "extensions": ["csh"]
        },
        "application/x-deb": {
            "compressible": false
        },
        "application/x-debian-package": {
            "source": "apache",
            "extensions": ["deb", "udeb"]
        },
        "application/x-dgc-compressed": {
            "source": "apache",
            "extensions": ["dgc"]
        },
        "application/x-director": {
            "source": "apache",
            "extensions": ["dir", "dcr", "dxr", "cst", "cct", "cxt", "w3d", "fgd", "swa"]
        },
        "application/x-doom": {
            "source": "apache",
            "extensions": ["wad"]
        },
        "application/x-dtbncx+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["ncx"]
        },
        "application/x-dtbook+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["dtb"]
        },
        "application/x-dtbresource+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["res"]
        },
        "application/x-dvi": {
            "source": "apache",
            "compressible": false,
            "extensions": ["dvi"]
        },
        "application/x-envoy": {
            "source": "apache",
            "extensions": ["evy"]
        },
        "application/x-eva": {
            "source": "apache",
            "extensions": ["eva"]
        },
        "application/x-font-bdf": {
            "source": "apache",
            "extensions": ["bdf"]
        },
        "application/x-font-dos": {
            "source": "apache"
        },
        "application/x-font-framemaker": {
            "source": "apache"
        },
        "application/x-font-ghostscript": {
            "source": "apache",
            "extensions": ["gsf"]
        },
        "application/x-font-libgrx": {
            "source": "apache"
        },
        "application/x-font-linux-psf": {
            "source": "apache",
            "extensions": ["psf"]
        },
        "application/x-font-pcf": {
            "source": "apache",
            "extensions": ["pcf"]
        },
        "application/x-font-snf": {
            "source": "apache",
            "extensions": ["snf"]
        },
        "application/x-font-speedo": {
            "source": "apache"
        },
        "application/x-font-sunos-news": {
            "source": "apache"
        },
        "application/x-font-type1": {
            "source": "apache",
            "extensions": ["pfa", "pfb", "pfm", "afm"]
        },
        "application/x-font-vfont": {
            "source": "apache"
        },
        "application/x-freearc": {
            "source": "apache",
            "extensions": ["arc"]
        },
        "application/x-futuresplash": {
            "source": "apache",
            "extensions": ["spl"]
        },
        "application/x-gca-compressed": {
            "source": "apache",
            "extensions": ["gca"]
        },
        "application/x-glulx": {
            "source": "apache",
            "extensions": ["ulx"]
        },
        "application/x-gnumeric": {
            "source": "apache",
            "extensions": ["gnumeric"]
        },
        "application/x-gramps-xml": {
            "source": "apache",
            "extensions": ["gramps"]
        },
        "application/x-gtar": {
            "source": "apache",
            "extensions": ["gtar"]
        },
        "application/x-gzip": {
            "source": "apache"
        },
        "application/x-hdf": {
            "source": "apache",
            "extensions": ["hdf"]
        },
        "application/x-httpd-php": {
            "compressible": true,
            "extensions": ["php"]
        },
        "application/x-install-instructions": {
            "source": "apache",
            "extensions": ["install"]
        },
        "application/x-iso9660-image": {
            "source": "apache",
            "extensions": ["iso"]
        },
        "application/x-java-archive-diff": {
            "source": "nginx",
            "extensions": ["jardiff"]
        },
        "application/x-java-jnlp-file": {
            "source": "apache",
            "compressible": false,
            "extensions": ["jnlp"]
        },
        "application/x-javascript": {
            "compressible": true
        },
        "application/x-keepass2": {
            "extensions": ["kdbx"]
        },
        "application/x-latex": {
            "source": "apache",
            "compressible": false,
            "extensions": ["latex"]
        },
        "application/x-lua-bytecode": {
            "extensions": ["luac"]
        },
        "application/x-lzh-compressed": {
            "source": "apache",
            "extensions": ["lzh", "lha"]
        },
        "application/x-makeself": {
            "source": "nginx",
            "extensions": ["run"]
        },
        "application/x-mie": {
            "source": "apache",
            "extensions": ["mie"]
        },
        "application/x-mobipocket-ebook": {
            "source": "apache",
            "extensions": ["prc", "mobi"]
        },
        "application/x-mpegurl": {
            "compressible": false
        },
        "application/x-ms-application": {
            "source": "apache",
            "extensions": ["application"]
        },
        "application/x-ms-shortcut": {
            "source": "apache",
            "extensions": ["lnk"]
        },
        "application/x-ms-wmd": {
            "source": "apache",
            "extensions": ["wmd"]
        },
        "application/x-ms-wmz": {
            "source": "apache",
            "extensions": ["wmz"]
        },
        "application/x-ms-xbap": {
            "source": "apache",
            "extensions": ["xbap"]
        },
        "application/x-msaccess": {
            "source": "apache",
            "extensions": ["mdb"]
        },
        "application/x-msbinder": {
            "source": "apache",
            "extensions": ["obd"]
        },
        "application/x-mscardfile": {
            "source": "apache",
            "extensions": ["crd"]
        },
        "application/x-msclip": {
            "source": "apache",
            "extensions": ["clp"]
        },
        "application/x-msdos-program": {
            "extensions": ["exe"]
        },
        "application/x-msdownload": {
            "source": "apache",
            "extensions": ["exe", "dll", "com", "bat", "msi"]
        },
        "application/x-msmediaview": {
            "source": "apache",
            "extensions": ["mvb", "m13", "m14"]
        },
        "application/x-msmetafile": {
            "source": "apache",
            "extensions": ["wmf", "wmz", "emf", "emz"]
        },
        "application/x-msmoney": {
            "source": "apache",
            "extensions": ["mny"]
        },
        "application/x-mspublisher": {
            "source": "apache",
            "extensions": ["pub"]
        },
        "application/x-msschedule": {
            "source": "apache",
            "extensions": ["scd"]
        },
        "application/x-msterminal": {
            "source": "apache",
            "extensions": ["trm"]
        },
        "application/x-mswrite": {
            "source": "apache",
            "extensions": ["wri"]
        },
        "application/x-netcdf": {
            "source": "apache",
            "extensions": ["nc", "cdf"]
        },
        "application/x-ns-proxy-autoconfig": {
            "compressible": true,
            "extensions": ["pac"]
        },
        "application/x-nzb": {
            "source": "apache",
            "extensions": ["nzb"]
        },
        "application/x-perl": {
            "source": "nginx",
            "extensions": ["pl", "pm"]
        },
        "application/x-pilot": {
            "source": "nginx",
            "extensions": ["prc", "pdb"]
        },
        "application/x-pkcs12": {
            "source": "apache",
            "compressible": false,
            "extensions": ["p12", "pfx"]
        },
        "application/x-pkcs7-certificates": {
            "source": "apache",
            "extensions": ["p7b", "spc"]
        },
        "application/x-pkcs7-certreqresp": {
            "source": "apache",
            "extensions": ["p7r"]
        },
        "application/x-rar-compressed": {
            "source": "apache",
            "compressible": false,
            "extensions": ["rar"]
        },
        "application/x-redhat-package-manager": {
            "source": "nginx",
            "extensions": ["rpm"]
        },
        "application/x-research-info-systems": {
            "source": "apache",
            "extensions": ["ris"]
        },
        "application/x-sea": {
            "source": "nginx",
            "extensions": ["sea"]
        },
        "application/x-sh": {
            "source": "apache",
            "compressible": true,
            "extensions": ["sh"]
        },
        "application/x-shar": {
            "source": "apache",
            "extensions": ["shar"]
        },
        "application/x-shockwave-flash": {
            "source": "apache",
            "compressible": false,
            "extensions": ["swf"]
        },
        "application/x-silverlight-app": {
            "source": "apache",
            "extensions": ["xap"]
        },
        "application/x-sql": {
            "source": "apache",
            "extensions": ["sql"]
        },
        "application/x-stuffit": {
            "source": "apache",
            "compressible": false,
            "extensions": ["sit"]
        },
        "application/x-stuffitx": {
            "source": "apache",
            "extensions": ["sitx"]
        },
        "application/x-subrip": {
            "source": "apache",
            "extensions": ["srt"]
        },
        "application/x-sv4cpio": {
            "source": "apache",
            "extensions": ["sv4cpio"]
        },
        "application/x-sv4crc": {
            "source": "apache",
            "extensions": ["sv4crc"]
        },
        "application/x-t3vm-image": {
            "source": "apache",
            "extensions": ["t3"]
        },
        "application/x-tads": {
            "source": "apache",
            "extensions": ["gam"]
        },
        "application/x-tar": {
            "source": "apache",
            "compressible": true,
            "extensions": ["tar"]
        },
        "application/x-tcl": {
            "source": "apache",
            "extensions": ["tcl", "tk"]
        },
        "application/x-tex": {
            "source": "apache",
            "extensions": ["tex"]
        },
        "application/x-tex-tfm": {
            "source": "apache",
            "extensions": ["tfm"]
        },
        "application/x-texinfo": {
            "source": "apache",
            "extensions": ["texinfo", "texi"]
        },
        "application/x-tgif": {
            "source": "apache",
            "extensions": ["obj"]
        },
        "application/x-ustar": {
            "source": "apache",
            "extensions": ["ustar"]
        },
        "application/x-virtualbox-hdd": {
            "compressible": true,
            "extensions": ["hdd"]
        },
        "application/x-virtualbox-ova": {
            "compressible": true,
            "extensions": ["ova"]
        },
        "application/x-virtualbox-ovf": {
            "compressible": true,
            "extensions": ["ovf"]
        },
        "application/x-virtualbox-vbox": {
            "compressible": true,
            "extensions": ["vbox"]
        },
        "application/x-virtualbox-vbox-extpack": {
            "compressible": false,
            "extensions": ["vbox-extpack"]
        },
        "application/x-virtualbox-vdi": {
            "compressible": true,
            "extensions": ["vdi"]
        },
        "application/x-virtualbox-vhd": {
            "compressible": true,
            "extensions": ["vhd"]
        },
        "application/x-virtualbox-vmdk": {
            "compressible": true,
            "extensions": ["vmdk"]
        },
        "application/x-wais-source": {
            "source": "apache",
            "extensions": ["src"]
        },
        "application/x-web-app-manifest+json": {
            "compressible": true,
            "extensions": ["webapp"]
        },
        "application/x-www-form-urlencoded": {
            "source": "iana",
            "compressible": true
        },
        "application/x-x509-ca-cert": {
            "source": "apache",
            "extensions": ["der", "crt", "pem"]
        },
        "application/x-xfig": {
            "source": "apache",
            "extensions": ["fig"]
        },
        "application/x-xliff+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["xlf"]
        },
        "application/x-xpinstall": {
            "source": "apache",
            "compressible": false,
            "extensions": ["xpi"]
        },
        "application/x-xz": {
            "source": "apache",
            "extensions": ["xz"]
        },
        "application/x-zmachine": {
            "source": "apache",
            "extensions": ["z1", "z2", "z3", "z4", "z5", "z6", "z7", "z8"]
        },
        "application/x400-bp": {
            "source": "iana"
        },
        "application/xacml+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/xaml+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["xaml"]
        },
        "application/xcap-att+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xav"]
        },
        "application/xcap-caps+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xca"]
        },
        "application/xcap-diff+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xdf"]
        },
        "application/xcap-el+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xel"]
        },
        "application/xcap-error+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xer"]
        },
        "application/xcap-ns+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xns"]
        },
        "application/xcon-conference-info+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/xcon-conference-info-diff+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/xenc+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xenc"]
        },
        "application/xhtml+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xhtml", "xht"]
        },
        "application/xhtml-voice+xml": {
            "source": "apache",
            "compressible": true
        },
        "application/xliff+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xlf"]
        },
        "application/xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xml", "xsl", "xsd", "rng"]
        },
        "application/xml-dtd": {
            "source": "iana",
            "compressible": true,
            "extensions": ["dtd"]
        },
        "application/xml-external-parsed-entity": {
            "source": "iana"
        },
        "application/xml-patch+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/xmpp+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/xop+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xop"]
        },
        "application/xproc+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["xpl"]
        },
        "application/xslt+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xslt"]
        },
        "application/xspf+xml": {
            "source": "apache",
            "compressible": true,
            "extensions": ["xspf"]
        },
        "application/xv+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["mxml", "xhvml", "xvml", "xvm"]
        },
        "application/yang": {
            "source": "iana",
            "extensions": ["yang"]
        },
        "application/yang-data+json": {
            "source": "iana",
            "compressible": true
        },
        "application/yang-data+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/yang-patch+json": {
            "source": "iana",
            "compressible": true
        },
        "application/yang-patch+xml": {
            "source": "iana",
            "compressible": true
        },
        "application/yin+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["yin"]
        },
        "application/zip": {
            "source": "iana",
            "compressible": false,
            "extensions": ["zip"]
        },
        "application/zlib": {
            "source": "iana"
        },
        "application/zstd": {
            "source": "iana"
        },
        "audio/1d-interleaved-parityfec": {
            "source": "iana"
        },
        "audio/32kadpcm": {
            "source": "iana"
        },
        "audio/3gpp": {
            "source": "iana",
            "compressible": false,
            "extensions": ["3gpp"]
        },
        "audio/3gpp2": {
            "source": "iana"
        },
        "audio/aac": {
            "source": "iana"
        },
        "audio/ac3": {
            "source": "iana"
        },
        "audio/adpcm": {
            "source": "apache",
            "extensions": ["adp"]
        },
        "audio/amr": {
            "source": "iana"
        },
        "audio/amr-wb": {
            "source": "iana"
        },
        "audio/amr-wb+": {
            "source": "iana"
        },
        "audio/aptx": {
            "source": "iana"
        },
        "audio/asc": {
            "source": "iana"
        },
        "audio/atrac-advanced-lossless": {
            "source": "iana"
        },
        "audio/atrac-x": {
            "source": "iana"
        },
        "audio/atrac3": {
            "source": "iana"
        },
        "audio/basic": {
            "source": "iana",
            "compressible": false,
            "extensions": ["au", "snd"]
        },
        "audio/bv16": {
            "source": "iana"
        },
        "audio/bv32": {
            "source": "iana"
        },
        "audio/clearmode": {
            "source": "iana"
        },
        "audio/cn": {
            "source": "iana"
        },
        "audio/dat12": {
            "source": "iana"
        },
        "audio/dls": {
            "source": "iana"
        },
        "audio/dsr-es201108": {
            "source": "iana"
        },
        "audio/dsr-es202050": {
            "source": "iana"
        },
        "audio/dsr-es202211": {
            "source": "iana"
        },
        "audio/dsr-es202212": {
            "source": "iana"
        },
        "audio/dv": {
            "source": "iana"
        },
        "audio/dvi4": {
            "source": "iana"
        },
        "audio/eac3": {
            "source": "iana"
        },
        "audio/encaprtp": {
            "source": "iana"
        },
        "audio/evrc": {
            "source": "iana"
        },
        "audio/evrc-qcp": {
            "source": "iana"
        },
        "audio/evrc0": {
            "source": "iana"
        },
        "audio/evrc1": {
            "source": "iana"
        },
        "audio/evrcb": {
            "source": "iana"
        },
        "audio/evrcb0": {
            "source": "iana"
        },
        "audio/evrcb1": {
            "source": "iana"
        },
        "audio/evrcnw": {
            "source": "iana"
        },
        "audio/evrcnw0": {
            "source": "iana"
        },
        "audio/evrcnw1": {
            "source": "iana"
        },
        "audio/evrcwb": {
            "source": "iana"
        },
        "audio/evrcwb0": {
            "source": "iana"
        },
        "audio/evrcwb1": {
            "source": "iana"
        },
        "audio/evs": {
            "source": "iana"
        },
        "audio/flexfec": {
            "source": "iana"
        },
        "audio/fwdred": {
            "source": "iana"
        },
        "audio/g711-0": {
            "source": "iana"
        },
        "audio/g719": {
            "source": "iana"
        },
        "audio/g722": {
            "source": "iana"
        },
        "audio/g7221": {
            "source": "iana"
        },
        "audio/g723": {
            "source": "iana"
        },
        "audio/g726-16": {
            "source": "iana"
        },
        "audio/g726-24": {
            "source": "iana"
        },
        "audio/g726-32": {
            "source": "iana"
        },
        "audio/g726-40": {
            "source": "iana"
        },
        "audio/g728": {
            "source": "iana"
        },
        "audio/g729": {
            "source": "iana"
        },
        "audio/g7291": {
            "source": "iana"
        },
        "audio/g729d": {
            "source": "iana"
        },
        "audio/g729e": {
            "source": "iana"
        },
        "audio/gsm": {
            "source": "iana"
        },
        "audio/gsm-efr": {
            "source": "iana"
        },
        "audio/gsm-hr-08": {
            "source": "iana"
        },
        "audio/ilbc": {
            "source": "iana"
        },
        "audio/ip-mr_v2.5": {
            "source": "iana"
        },
        "audio/isac": {
            "source": "apache"
        },
        "audio/l16": {
            "source": "iana"
        },
        "audio/l20": {
            "source": "iana"
        },
        "audio/l24": {
            "source": "iana",
            "compressible": false
        },
        "audio/l8": {
            "source": "iana"
        },
        "audio/lpc": {
            "source": "iana"
        },
        "audio/melp": {
            "source": "iana"
        },
        "audio/melp1200": {
            "source": "iana"
        },
        "audio/melp2400": {
            "source": "iana"
        },
        "audio/melp600": {
            "source": "iana"
        },
        "audio/midi": {
            "source": "apache",
            "extensions": ["mid", "midi", "kar", "rmi"]
        },
        "audio/mobile-xmf": {
            "source": "iana",
            "extensions": ["mxmf"]
        },
        "audio/mp3": {
            "compressible": false,
            "extensions": ["mp3"]
        },
        "audio/mp4": {
            "source": "iana",
            "compressible": false,
            "extensions": ["m4a", "mp4a"]
        },
        "audio/mp4a-latm": {
            "source": "iana"
        },
        "audio/mpa": {
            "source": "iana"
        },
        "audio/mpa-robust": {
            "source": "iana"
        },
        "audio/mpeg": {
            "source": "iana",
            "compressible": false,
            "extensions": ["mpga", "mp2", "mp2a", "mp3", "m2a", "m3a"]
        },
        "audio/mpeg4-generic": {
            "source": "iana"
        },
        "audio/musepack": {
            "source": "apache"
        },
        "audio/ogg": {
            "source": "iana",
            "compressible": false,
            "extensions": ["oga", "ogg", "spx"]
        },
        "audio/opus": {
            "source": "iana"
        },
        "audio/parityfec": {
            "source": "iana"
        },
        "audio/pcma": {
            "source": "iana"
        },
        "audio/pcma-wb": {
            "source": "iana"
        },
        "audio/pcmu": {
            "source": "iana"
        },
        "audio/pcmu-wb": {
            "source": "iana"
        },
        "audio/prs.sid": {
            "source": "iana"
        },
        "audio/qcelp": {
            "source": "iana"
        },
        "audio/raptorfec": {
            "source": "iana"
        },
        "audio/red": {
            "source": "iana"
        },
        "audio/rtp-enc-aescm128": {
            "source": "iana"
        },
        "audio/rtp-midi": {
            "source": "iana"
        },
        "audio/rtploopback": {
            "source": "iana"
        },
        "audio/rtx": {
            "source": "iana"
        },
        "audio/s3m": {
            "source": "apache",
            "extensions": ["s3m"]
        },
        "audio/silk": {
            "source": "apache",
            "extensions": ["sil"]
        },
        "audio/smv": {
            "source": "iana"
        },
        "audio/smv-qcp": {
            "source": "iana"
        },
        "audio/smv0": {
            "source": "iana"
        },
        "audio/sp-midi": {
            "source": "iana"
        },
        "audio/speex": {
            "source": "iana"
        },
        "audio/t140c": {
            "source": "iana"
        },
        "audio/t38": {
            "source": "iana"
        },
        "audio/telephone-event": {
            "source": "iana"
        },
        "audio/tetra_acelp": {
            "source": "iana"
        },
        "audio/tone": {
            "source": "iana"
        },
        "audio/uemclip": {
            "source": "iana"
        },
        "audio/ulpfec": {
            "source": "iana"
        },
        "audio/usac": {
            "source": "iana"
        },
        "audio/vdvi": {
            "source": "iana"
        },
        "audio/vmr-wb": {
            "source": "iana"
        },
        "audio/vnd.3gpp.iufp": {
            "source": "iana"
        },
        "audio/vnd.4sb": {
            "source": "iana"
        },
        "audio/vnd.audiokoz": {
            "source": "iana"
        },
        "audio/vnd.celp": {
            "source": "iana"
        },
        "audio/vnd.cisco.nse": {
            "source": "iana"
        },
        "audio/vnd.cmles.radio-events": {
            "source": "iana"
        },
        "audio/vnd.cns.anp1": {
            "source": "iana"
        },
        "audio/vnd.cns.inf1": {
            "source": "iana"
        },
        "audio/vnd.dece.audio": {
            "source": "iana",
            "extensions": ["uva", "uvva"]
        },
        "audio/vnd.digital-winds": {
            "source": "iana",
            "extensions": ["eol"]
        },
        "audio/vnd.dlna.adts": {
            "source": "iana"
        },
        "audio/vnd.dolby.heaac.1": {
            "source": "iana"
        },
        "audio/vnd.dolby.heaac.2": {
            "source": "iana"
        },
        "audio/vnd.dolby.mlp": {
            "source": "iana"
        },
        "audio/vnd.dolby.mps": {
            "source": "iana"
        },
        "audio/vnd.dolby.pl2": {
            "source": "iana"
        },
        "audio/vnd.dolby.pl2x": {
            "source": "iana"
        },
        "audio/vnd.dolby.pl2z": {
            "source": "iana"
        },
        "audio/vnd.dolby.pulse.1": {
            "source": "iana"
        },
        "audio/vnd.dra": {
            "source": "iana",
            "extensions": ["dra"]
        },
        "audio/vnd.dts": {
            "source": "iana",
            "extensions": ["dts"]
        },
        "audio/vnd.dts.hd": {
            "source": "iana",
            "extensions": ["dtshd"]
        },
        "audio/vnd.dts.uhd": {
            "source": "iana"
        },
        "audio/vnd.dvb.file": {
            "source": "iana"
        },
        "audio/vnd.everad.plj": {
            "source": "iana"
        },
        "audio/vnd.hns.audio": {
            "source": "iana"
        },
        "audio/vnd.lucent.voice": {
            "source": "iana",
            "extensions": ["lvp"]
        },
        "audio/vnd.ms-playready.media.pya": {
            "source": "iana",
            "extensions": ["pya"]
        },
        "audio/vnd.nokia.mobile-xmf": {
            "source": "iana"
        },
        "audio/vnd.nortel.vbk": {
            "source": "iana"
        },
        "audio/vnd.nuera.ecelp4800": {
            "source": "iana",
            "extensions": ["ecelp4800"]
        },
        "audio/vnd.nuera.ecelp7470": {
            "source": "iana",
            "extensions": ["ecelp7470"]
        },
        "audio/vnd.nuera.ecelp9600": {
            "source": "iana",
            "extensions": ["ecelp9600"]
        },
        "audio/vnd.octel.sbc": {
            "source": "iana"
        },
        "audio/vnd.presonus.multitrack": {
            "source": "iana"
        },
        "audio/vnd.qcelp": {
            "source": "iana"
        },
        "audio/vnd.rhetorex.32kadpcm": {
            "source": "iana"
        },
        "audio/vnd.rip": {
            "source": "iana",
            "extensions": ["rip"]
        },
        "audio/vnd.rn-realaudio": {
            "compressible": false
        },
        "audio/vnd.sealedmedia.softseal.mpeg": {
            "source": "iana"
        },
        "audio/vnd.vmx.cvsd": {
            "source": "iana"
        },
        "audio/vnd.wave": {
            "compressible": false
        },
        "audio/vorbis": {
            "source": "iana",
            "compressible": false
        },
        "audio/vorbis-config": {
            "source": "iana"
        },
        "audio/wav": {
            "compressible": false,
            "extensions": ["wav"]
        },
        "audio/wave": {
            "compressible": false,
            "extensions": ["wav"]
        },
        "audio/webm": {
            "source": "apache",
            "compressible": false,
            "extensions": ["weba"]
        },
        "audio/x-aac": {
            "source": "apache",
            "compressible": false,
            "extensions": ["aac"]
        },
        "audio/x-aiff": {
            "source": "apache",
            "extensions": ["aif", "aiff", "aifc"]
        },
        "audio/x-caf": {
            "source": "apache",
            "compressible": false,
            "extensions": ["caf"]
        },
        "audio/x-flac": {
            "source": "apache",
            "extensions": ["flac"]
        },
        "audio/x-m4a": {
            "source": "nginx",
            "extensions": ["m4a"]
        },
        "audio/x-matroska": {
            "source": "apache",
            "extensions": ["mka"]
        },
        "audio/x-mpegurl": {
            "source": "apache",
            "extensions": ["m3u"]
        },
        "audio/x-ms-wax": {
            "source": "apache",
            "extensions": ["wax"]
        },
        "audio/x-ms-wma": {
            "source": "apache",
            "extensions": ["wma"]
        },
        "audio/x-pn-realaudio": {
            "source": "apache",
            "extensions": ["ram", "ra"]
        },
        "audio/x-pn-realaudio-plugin": {
            "source": "apache",
            "extensions": ["rmp"]
        },
        "audio/x-realaudio": {
            "source": "nginx",
            "extensions": ["ra"]
        },
        "audio/x-tta": {
            "source": "apache"
        },
        "audio/x-wav": {
            "source": "apache",
            "extensions": ["wav"]
        },
        "audio/xm": {
            "source": "apache",
            "extensions": ["xm"]
        },
        "chemical/x-cdx": {
            "source": "apache",
            "extensions": ["cdx"]
        },
        "chemical/x-cif": {
            "source": "apache",
            "extensions": ["cif"]
        },
        "chemical/x-cmdf": {
            "source": "apache",
            "extensions": ["cmdf"]
        },
        "chemical/x-cml": {
            "source": "apache",
            "extensions": ["cml"]
        },
        "chemical/x-csml": {
            "source": "apache",
            "extensions": ["csml"]
        },
        "chemical/x-pdb": {
            "source": "apache"
        },
        "chemical/x-xyz": {
            "source": "apache",
            "extensions": ["xyz"]
        },
        "font/collection": {
            "source": "iana",
            "extensions": ["ttc"]
        },
        "font/otf": {
            "source": "iana",
            "compressible": true,
            "extensions": ["otf"]
        },
        "font/sfnt": {
            "source": "iana"
        },
        "font/ttf": {
            "source": "iana",
            "compressible": true,
            "extensions": ["ttf"]
        },
        "font/woff": {
            "source": "iana",
            "extensions": ["woff"]
        },
        "font/woff2": {
            "source": "iana",
            "extensions": ["woff2"]
        },
        "image/aces": {
            "source": "iana",
            "extensions": ["exr"]
        },
        "image/apng": {
            "compressible": false,
            "extensions": ["apng"]
        },
        "image/avci": {
            "source": "iana"
        },
        "image/avcs": {
            "source": "iana"
        },
        "image/bmp": {
            "source": "iana",
            "compressible": true,
            "extensions": ["bmp"]
        },
        "image/cgm": {
            "source": "iana",
            "extensions": ["cgm"]
        },
        "image/dicom-rle": {
            "source": "iana",
            "extensions": ["drle"]
        },
        "image/emf": {
            "source": "iana",
            "extensions": ["emf"]
        },
        "image/fits": {
            "source": "iana",
            "extensions": ["fits"]
        },
        "image/g3fax": {
            "source": "iana",
            "extensions": ["g3"]
        },
        "image/gif": {
            "source": "iana",
            "compressible": false,
            "extensions": ["gif"]
        },
        "image/heic": {
            "source": "iana",
            "extensions": ["heic"]
        },
        "image/heic-sequence": {
            "source": "iana",
            "extensions": ["heics"]
        },
        "image/heif": {
            "source": "iana",
            "extensions": ["heif"]
        },
        "image/heif-sequence": {
            "source": "iana",
            "extensions": ["heifs"]
        },
        "image/hej2k": {
            "source": "iana",
            "extensions": ["hej2"]
        },
        "image/hsj2": {
            "source": "iana",
            "extensions": ["hsj2"]
        },
        "image/ief": {
            "source": "iana",
            "extensions": ["ief"]
        },
        "image/jls": {
            "source": "iana",
            "extensions": ["jls"]
        },
        "image/jp2": {
            "source": "iana",
            "compressible": false,
            "extensions": ["jp2", "jpg2"]
        },
        "image/jpeg": {
            "source": "iana",
            "compressible": false,
            "extensions": ["jpeg", "jpg", "jpe"]
        },
        "image/jph": {
            "source": "iana",
            "extensions": ["jph"]
        },
        "image/jphc": {
            "source": "iana",
            "extensions": ["jhc"]
        },
        "image/jpm": {
            "source": "iana",
            "compressible": false,
            "extensions": ["jpm"]
        },
        "image/jpx": {
            "source": "iana",
            "compressible": false,
            "extensions": ["jpx", "jpf"]
        },
        "image/jxr": {
            "source": "iana",
            "extensions": ["jxr"]
        },
        "image/jxra": {
            "source": "iana",
            "extensions": ["jxra"]
        },
        "image/jxrs": {
            "source": "iana",
            "extensions": ["jxrs"]
        },
        "image/jxs": {
            "source": "iana",
            "extensions": ["jxs"]
        },
        "image/jxsc": {
            "source": "iana",
            "extensions": ["jxsc"]
        },
        "image/jxsi": {
            "source": "iana",
            "extensions": ["jxsi"]
        },
        "image/jxss": {
            "source": "iana",
            "extensions": ["jxss"]
        },
        "image/ktx": {
            "source": "iana",
            "extensions": ["ktx"]
        },
        "image/naplps": {
            "source": "iana"
        },
        "image/pjpeg": {
            "compressible": false
        },
        "image/png": {
            "source": "iana",
            "compressible": false,
            "extensions": ["png"]
        },
        "image/prs.btif": {
            "source": "iana",
            "extensions": ["btif"]
        },
        "image/prs.pti": {
            "source": "iana",
            "extensions": ["pti"]
        },
        "image/pwg-raster": {
            "source": "iana"
        },
        "image/sgi": {
            "source": "apache",
            "extensions": ["sgi"]
        },
        "image/svg+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["svg", "svgz"]
        },
        "image/t38": {
            "source": "iana",
            "extensions": ["t38"]
        },
        "image/tiff": {
            "source": "iana",
            "compressible": false,
            "extensions": ["tif", "tiff"]
        },
        "image/tiff-fx": {
            "source": "iana",
            "extensions": ["tfx"]
        },
        "image/vnd.adobe.photoshop": {
            "source": "iana",
            "compressible": true,
            "extensions": ["psd"]
        },
        "image/vnd.airzip.accelerator.azv": {
            "source": "iana",
            "extensions": ["azv"]
        },
        "image/vnd.cns.inf2": {
            "source": "iana"
        },
        "image/vnd.dece.graphic": {
            "source": "iana",
            "extensions": ["uvi", "uvvi", "uvg", "uvvg"]
        },
        "image/vnd.djvu": {
            "source": "iana",
            "extensions": ["djvu", "djv"]
        },
        "image/vnd.dvb.subtitle": {
            "source": "iana",
            "extensions": ["sub"]
        },
        "image/vnd.dwg": {
            "source": "iana",
            "extensions": ["dwg"]
        },
        "image/vnd.dxf": {
            "source": "iana",
            "extensions": ["dxf"]
        },
        "image/vnd.fastbidsheet": {
            "source": "iana",
            "extensions": ["fbs"]
        },
        "image/vnd.fpx": {
            "source": "iana",
            "extensions": ["fpx"]
        },
        "image/vnd.fst": {
            "source": "iana",
            "extensions": ["fst"]
        },
        "image/vnd.fujixerox.edmics-mmr": {
            "source": "iana",
            "extensions": ["mmr"]
        },
        "image/vnd.fujixerox.edmics-rlc": {
            "source": "iana",
            "extensions": ["rlc"]
        },
        "image/vnd.globalgraphics.pgb": {
            "source": "iana"
        },
        "image/vnd.microsoft.icon": {
            "source": "iana",
            "extensions": ["ico"]
        },
        "image/vnd.mix": {
            "source": "iana"
        },
        "image/vnd.mozilla.apng": {
            "source": "iana"
        },
        "image/vnd.ms-dds": {
            "extensions": ["dds"]
        },
        "image/vnd.ms-modi": {
            "source": "iana",
            "extensions": ["mdi"]
        },
        "image/vnd.ms-photo": {
            "source": "apache",
            "extensions": ["wdp"]
        },
        "image/vnd.net-fpx": {
            "source": "iana",
            "extensions": ["npx"]
        },
        "image/vnd.radiance": {
            "source": "iana"
        },
        "image/vnd.sealed.png": {
            "source": "iana"
        },
        "image/vnd.sealedmedia.softseal.gif": {
            "source": "iana"
        },
        "image/vnd.sealedmedia.softseal.jpg": {
            "source": "iana"
        },
        "image/vnd.svf": {
            "source": "iana"
        },
        "image/vnd.tencent.tap": {
            "source": "iana",
            "extensions": ["tap"]
        },
        "image/vnd.valve.source.texture": {
            "source": "iana",
            "extensions": ["vtf"]
        },
        "image/vnd.wap.wbmp": {
            "source": "iana",
            "extensions": ["wbmp"]
        },
        "image/vnd.xiff": {
            "source": "iana",
            "extensions": ["xif"]
        },
        "image/vnd.zbrush.pcx": {
            "source": "iana",
            "extensions": ["pcx"]
        },
        "image/webp": {
            "source": "apache",
            "extensions": ["webp"]
        },
        "image/wmf": {
            "source": "iana",
            "extensions": ["wmf"]
        },
        "image/x-3ds": {
            "source": "apache",
            "extensions": ["3ds"]
        },
        "image/x-cmu-raster": {
            "source": "apache",
            "extensions": ["ras"]
        },
        "image/x-cmx": {
            "source": "apache",
            "extensions": ["cmx"]
        },
        "image/x-freehand": {
            "source": "apache",
            "extensions": ["fh", "fhc", "fh4", "fh5", "fh7"]
        },
        "image/x-icon": {
            "source": "apache",
            "compressible": true,
            "extensions": ["ico"]
        },
        "image/x-jng": {
            "source": "nginx",
            "extensions": ["jng"]
        },
        "image/x-mrsid-image": {
            "source": "apache",
            "extensions": ["sid"]
        },
        "image/x-ms-bmp": {
            "source": "nginx",
            "compressible": true,
            "extensions": ["bmp"]
        },
        "image/x-pcx": {
            "source": "apache",
            "extensions": ["pcx"]
        },
        "image/x-pict": {
            "source": "apache",
            "extensions": ["pic", "pct"]
        },
        "image/x-portable-anymap": {
            "source": "apache",
            "extensions": ["pnm"]
        },
        "image/x-portable-bitmap": {
            "source": "apache",
            "extensions": ["pbm"]
        },
        "image/x-portable-graymap": {
            "source": "apache",
            "extensions": ["pgm"]
        },
        "image/x-portable-pixmap": {
            "source": "apache",
            "extensions": ["ppm"]
        },
        "image/x-rgb": {
            "source": "apache",
            "extensions": ["rgb"]
        },
        "image/x-tga": {
            "source": "apache",
            "extensions": ["tga"]
        },
        "image/x-xbitmap": {
            "source": "apache",
            "extensions": ["xbm"]
        },
        "image/x-xcf": {
            "compressible": false
        },
        "image/x-xpixmap": {
            "source": "apache",
            "extensions": ["xpm"]
        },
        "image/x-xwindowdump": {
            "source": "apache",
            "extensions": ["xwd"]
        },
        "message/cpim": {
            "source": "iana"
        },
        "message/delivery-status": {
            "source": "iana"
        },
        "message/disposition-notification": {
            "source": "iana",
            "extensions": [
                "disposition-notification"
            ]
        },
        "message/external-body": {
            "source": "iana"
        },
        "message/feedback-report": {
            "source": "iana"
        },
        "message/global": {
            "source": "iana",
            "extensions": ["u8msg"]
        },
        "message/global-delivery-status": {
            "source": "iana",
            "extensions": ["u8dsn"]
        },
        "message/global-disposition-notification": {
            "source": "iana",
            "extensions": ["u8mdn"]
        },
        "message/global-headers": {
            "source": "iana",
            "extensions": ["u8hdr"]
        },
        "message/http": {
            "source": "iana",
            "compressible": false
        },
        "message/imdn+xml": {
            "source": "iana",
            "compressible": true
        },
        "message/news": {
            "source": "iana"
        },
        "message/partial": {
            "source": "iana",
            "compressible": false
        },
        "message/rfc822": {
            "source": "iana",
            "compressible": true,
            "extensions": ["eml", "mime"]
        },
        "message/s-http": {
            "source": "iana"
        },
        "message/sip": {
            "source": "iana"
        },
        "message/sipfrag": {
            "source": "iana"
        },
        "message/tracking-status": {
            "source": "iana"
        },
        "message/vnd.si.simp": {
            "source": "iana"
        },
        "message/vnd.wfa.wsc": {
            "source": "iana",
            "extensions": ["wsc"]
        },
        "model/3mf": {
            "source": "iana",
            "extensions": ["3mf"]
        },
        "model/gltf+json": {
            "source": "iana",
            "compressible": true,
            "extensions": ["gltf"]
        },
        "model/gltf-binary": {
            "source": "iana",
            "compressible": true,
            "extensions": ["glb"]
        },
        "model/iges": {
            "source": "iana",
            "compressible": false,
            "extensions": ["igs", "iges"]
        },
        "model/mesh": {
            "source": "iana",
            "compressible": false,
            "extensions": ["msh", "mesh", "silo"]
        },
        "model/stl": {
            "source": "iana",
            "extensions": ["stl"]
        },
        "model/vnd.collada+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["dae"]
        },
        "model/vnd.dwf": {
            "source": "iana",
            "extensions": ["dwf"]
        },
        "model/vnd.flatland.3dml": {
            "source": "iana"
        },
        "model/vnd.gdl": {
            "source": "iana",
            "extensions": ["gdl"]
        },
        "model/vnd.gs-gdl": {
            "source": "apache"
        },
        "model/vnd.gs.gdl": {
            "source": "iana"
        },
        "model/vnd.gtw": {
            "source": "iana",
            "extensions": ["gtw"]
        },
        "model/vnd.moml+xml": {
            "source": "iana",
            "compressible": true
        },
        "model/vnd.mts": {
            "source": "iana",
            "extensions": ["mts"]
        },
        "model/vnd.opengex": {
            "source": "iana",
            "extensions": ["ogex"]
        },
        "model/vnd.parasolid.transmit.binary": {
            "source": "iana",
            "extensions": ["x_b"]
        },
        "model/vnd.parasolid.transmit.text": {
            "source": "iana",
            "extensions": ["x_t"]
        },
        "model/vnd.rosette.annotated-data-model": {
            "source": "iana"
        },
        "model/vnd.usdz+zip": {
            "source": "iana",
            "compressible": false,
            "extensions": ["usdz"]
        },
        "model/vnd.valve.source.compiled-map": {
            "source": "iana",
            "extensions": ["bsp"]
        },
        "model/vnd.vtu": {
            "source": "iana",
            "extensions": ["vtu"]
        },
        "model/vrml": {
            "source": "iana",
            "compressible": false,
            "extensions": ["wrl", "vrml"]
        },
        "model/x3d+binary": {
            "source": "apache",
            "compressible": false,
            "extensions": ["x3db", "x3dbz"]
        },
        "model/x3d+fastinfoset": {
            "source": "iana",
            "extensions": ["x3db"]
        },
        "model/x3d+vrml": {
            "source": "apache",
            "compressible": false,
            "extensions": ["x3dv", "x3dvz"]
        },
        "model/x3d+xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["x3d", "x3dz"]
        },
        "model/x3d-vrml": {
            "source": "iana",
            "extensions": ["x3dv"]
        },
        "multipart/alternative": {
            "source": "iana",
            "compressible": false
        },
        "multipart/appledouble": {
            "source": "iana"
        },
        "multipart/byteranges": {
            "source": "iana"
        },
        "multipart/digest": {
            "source": "iana"
        },
        "multipart/encrypted": {
            "source": "iana",
            "compressible": false
        },
        "multipart/form-data": {
            "source": "iana",
            "compressible": false
        },
        "multipart/header-set": {
            "source": "iana"
        },
        "multipart/mixed": {
            "source": "iana"
        },
        "multipart/multilingual": {
            "source": "iana"
        },
        "multipart/parallel": {
            "source": "iana"
        },
        "multipart/related": {
            "source": "iana",
            "compressible": false
        },
        "multipart/report": {
            "source": "iana"
        },
        "multipart/signed": {
            "source": "iana",
            "compressible": false
        },
        "multipart/vnd.bint.med-plus": {
            "source": "iana"
        },
        "multipart/voice-message": {
            "source": "iana"
        },
        "multipart/x-mixed-replace": {
            "source": "iana"
        },
        "text/1d-interleaved-parityfec": {
            "source": "iana"
        },
        "text/cache-manifest": {
            "source": "iana",
            "compressible": true,
            "extensions": ["appcache", "manifest"]
        },
        "text/calendar": {
            "source": "iana",
            "extensions": ["ics", "ifb"]
        },
        "text/calender": {
            "compressible": true
        },
        "text/cmd": {
            "compressible": true
        },
        "text/coffeescript": {
            "extensions": ["coffee", "litcoffee"]
        },
        "text/css": {
            "source": "iana",
            "charset": "UTF-8",
            "compressible": true,
            "extensions": ["css"]
        },
        "text/csv": {
            "source": "iana",
            "compressible": true,
            "extensions": ["csv"]
        },
        "text/csv-schema": {
            "source": "iana"
        },
        "text/directory": {
            "source": "iana"
        },
        "text/dns": {
            "source": "iana"
        },
        "text/ecmascript": {
            "source": "iana"
        },
        "text/encaprtp": {
            "source": "iana"
        },
        "text/enriched": {
            "source": "iana"
        },
        "text/flexfec": {
            "source": "iana"
        },
        "text/fwdred": {
            "source": "iana"
        },
        "text/grammar-ref-list": {
            "source": "iana"
        },
        "text/html": {
            "source": "iana",
            "compressible": true,
            "extensions": ["html", "htm", "shtml"]
        },
        "text/jade": {
            "extensions": ["jade"]
        },
        "text/javascript": {
            "source": "iana",
            "compressible": true
        },
        "text/jcr-cnd": {
            "source": "iana"
        },
        "text/jsx": {
            "compressible": true,
            "extensions": ["jsx"]
        },
        "text/less": {
            "compressible": true,
            "extensions": ["less"]
        },
        "text/markdown": {
            "source": "iana",
            "compressible": true,
            "extensions": ["markdown", "md"]
        },
        "text/mathml": {
            "source": "nginx",
            "extensions": ["mml"]
        },
        "text/mdx": {
            "compressible": true,
            "extensions": ["mdx"]
        },
        "text/mizar": {
            "source": "iana"
        },
        "text/n3": {
            "source": "iana",
            "compressible": true,
            "extensions": ["n3"]
        },
        "text/parameters": {
            "source": "iana"
        },
        "text/parityfec": {
            "source": "iana"
        },
        "text/plain": {
            "source": "iana",
            "compressible": true,
            "extensions": ["txt", "text", "conf", "def", "list", "log", "in", "ini"]
        },
        "text/provenance-notation": {
            "source": "iana"
        },
        "text/prs.fallenstein.rst": {
            "source": "iana"
        },
        "text/prs.lines.tag": {
            "source": "iana",
            "extensions": ["dsc"]
        },
        "text/prs.prop.logic": {
            "source": "iana"
        },
        "text/raptorfec": {
            "source": "iana"
        },
        "text/red": {
            "source": "iana"
        },
        "text/rfc822-headers": {
            "source": "iana"
        },
        "text/richtext": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rtx"]
        },
        "text/rtf": {
            "source": "iana",
            "compressible": true,
            "extensions": ["rtf"]
        },
        "text/rtp-enc-aescm128": {
            "source": "iana"
        },
        "text/rtploopback": {
            "source": "iana"
        },
        "text/rtx": {
            "source": "iana"
        },
        "text/sgml": {
            "source": "iana",
            "extensions": ["sgml", "sgm"]
        },
        "text/shex": {
            "extensions": ["shex"]
        },
        "text/slim": {
            "extensions": ["slim", "slm"]
        },
        "text/strings": {
            "source": "iana"
        },
        "text/stylus": {
            "extensions": ["stylus", "styl"]
        },
        "text/t140": {
            "source": "iana"
        },
        "text/tab-separated-values": {
            "source": "iana",
            "compressible": true,
            "extensions": ["tsv"]
        },
        "text/troff": {
            "source": "iana",
            "extensions": ["t", "tr", "roff", "man", "me", "ms"]
        },
        "text/turtle": {
            "source": "iana",
            "charset": "UTF-8",
            "extensions": ["ttl"]
        },
        "text/ulpfec": {
            "source": "iana"
        },
        "text/uri-list": {
            "source": "iana",
            "compressible": true,
            "extensions": ["uri", "uris", "urls"]
        },
        "text/vcard": {
            "source": "iana",
            "compressible": true,
            "extensions": ["vcard"]
        },
        "text/vnd.a": {
            "source": "iana"
        },
        "text/vnd.abc": {
            "source": "iana"
        },
        "text/vnd.ascii-art": {
            "source": "iana"
        },
        "text/vnd.curl": {
            "source": "iana",
            "extensions": ["curl"]
        },
        "text/vnd.curl.dcurl": {
            "source": "apache",
            "extensions": ["dcurl"]
        },
        "text/vnd.curl.mcurl": {
            "source": "apache",
            "extensions": ["mcurl"]
        },
        "text/vnd.curl.scurl": {
            "source": "apache",
            "extensions": ["scurl"]
        },
        "text/vnd.debian.copyright": {
            "source": "iana"
        },
        "text/vnd.dmclientscript": {
            "source": "iana"
        },
        "text/vnd.dvb.subtitle": {
            "source": "iana",
            "extensions": ["sub"]
        },
        "text/vnd.esmertec.theme-descriptor": {
            "source": "iana"
        },
        "text/vnd.ficlab.flt": {
            "source": "iana"
        },
        "text/vnd.fly": {
            "source": "iana",
            "extensions": ["fly"]
        },
        "text/vnd.fmi.flexstor": {
            "source": "iana",
            "extensions": ["flx"]
        },
        "text/vnd.gml": {
            "source": "iana"
        },
        "text/vnd.graphviz": {
            "source": "iana",
            "extensions": ["gv"]
        },
        "text/vnd.hgl": {
            "source": "iana"
        },
        "text/vnd.in3d.3dml": {
            "source": "iana",
            "extensions": ["3dml"]
        },
        "text/vnd.in3d.spot": {
            "source": "iana",
            "extensions": ["spot"]
        },
        "text/vnd.iptc.newsml": {
            "source": "iana"
        },
        "text/vnd.iptc.nitf": {
            "source": "iana"
        },
        "text/vnd.latex-z": {
            "source": "iana"
        },
        "text/vnd.motorola.reflex": {
            "source": "iana"
        },
        "text/vnd.ms-mediapackage": {
            "source": "iana"
        },
        "text/vnd.net2phone.commcenter.command": {
            "source": "iana"
        },
        "text/vnd.radisys.msml-basic-layout": {
            "source": "iana"
        },
        "text/vnd.senx.warpscript": {
            "source": "iana"
        },
        "text/vnd.si.uricatalogue": {
            "source": "iana"
        },
        "text/vnd.sosi": {
            "source": "iana"
        },
        "text/vnd.sun.j2me.app-descriptor": {
            "source": "iana",
            "extensions": ["jad"]
        },
        "text/vnd.trolltech.linguist": {
            "source": "iana"
        },
        "text/vnd.wap.si": {
            "source": "iana"
        },
        "text/vnd.wap.sl": {
            "source": "iana"
        },
        "text/vnd.wap.wml": {
            "source": "iana",
            "extensions": ["wml"]
        },
        "text/vnd.wap.wmlscript": {
            "source": "iana",
            "extensions": ["wmls"]
        },
        "text/vtt": {
            "source": "iana",
            "charset": "UTF-8",
            "compressible": true,
            "extensions": ["vtt"]
        },
        "text/x-asm": {
            "source": "apache",
            "extensions": ["s", "asm"]
        },
        "text/x-c": {
            "source": "apache",
            "extensions": ["c", "cc", "cxx", "cpp", "h", "hh", "dic"]
        },
        "text/x-component": {
            "source": "nginx",
            "extensions": ["htc"]
        },
        "text/x-fortran": {
            "source": "apache",
            "extensions": ["f", "for", "f77", "f90"]
        },
        "text/x-gwt-rpc": {
            "compressible": true
        },
        "text/x-handlebars-template": {
            "extensions": ["hbs"]
        },
        "text/x-java-source": {
            "source": "apache",
            "extensions": ["java"]
        },
        "text/x-jquery-tmpl": {
            "compressible": true
        },
        "text/x-lua": {
            "extensions": ["lua"]
        },
        "text/x-markdown": {
            "compressible": true,
            "extensions": ["mkd"]
        },
        "text/x-nfo": {
            "source": "apache",
            "extensions": ["nfo"]
        },
        "text/x-opml": {
            "source": "apache",
            "extensions": ["opml"]
        },
        "text/x-org": {
            "compressible": true,
            "extensions": ["org"]
        },
        "text/x-pascal": {
            "source": "apache",
            "extensions": ["p", "pas"]
        },
        "text/x-processing": {
            "compressible": true,
            "extensions": ["pde"]
        },
        "text/x-sass": {
            "extensions": ["sass"]
        },
        "text/x-scss": {
            "extensions": ["scss"]
        },
        "text/x-setext": {
            "source": "apache",
            "extensions": ["etx"]
        },
        "text/x-sfv": {
            "source": "apache",
            "extensions": ["sfv"]
        },
        "text/x-suse-ymp": {
            "compressible": true,
            "extensions": ["ymp"]
        },
        "text/x-uuencode": {
            "source": "apache",
            "extensions": ["uu"]
        },
        "text/x-vcalendar": {
            "source": "apache",
            "extensions": ["vcs"]
        },
        "text/x-vcard": {
            "source": "apache",
            "extensions": ["vcf"]
        },
        "text/xml": {
            "source": "iana",
            "compressible": true,
            "extensions": ["xml"]
        },
        "text/xml-external-parsed-entity": {
            "source": "iana"
        },
        "text/yaml": {
            "extensions": ["yaml", "yml"]
        },
        "video/1d-interleaved-parityfec": {
            "source": "iana"
        },
        "video/3gpp": {
            "source": "iana",
            "extensions": ["3gp", "3gpp"]
        },
        "video/3gpp-tt": {
            "source": "iana"
        },
        "video/3gpp2": {
            "source": "iana",
            "extensions": ["3g2"]
        },
        "video/bmpeg": {
            "source": "iana"
        },
        "video/bt656": {
            "source": "iana"
        },
        "video/celb": {
            "source": "iana"
        },
        "video/dv": {
            "source": "iana"
        },
        "video/encaprtp": {
            "source": "iana"
        },
        "video/flexfec": {
            "source": "iana"
        },
        "video/h261": {
            "source": "iana",
            "extensions": ["h261"]
        },
        "video/h263": {
            "source": "iana",
            "extensions": ["h263"]
        },
        "video/h263-1998": {
            "source": "iana"
        },
        "video/h263-2000": {
            "source": "iana"
        },
        "video/h264": {
            "source": "iana",
            "extensions": ["h264"]
        },
        "video/h264-rcdo": {
            "source": "iana"
        },
        "video/h264-svc": {
            "source": "iana"
        },
        "video/h265": {
            "source": "iana"
        },
        "video/iso.segment": {
            "source": "iana"
        },
        "video/jpeg": {
            "source": "iana",
            "extensions": ["jpgv"]
        },
        "video/jpeg2000": {
            "source": "iana"
        },
        "video/jpm": {
            "source": "apache",
            "extensions": ["jpm", "jpgm"]
        },
        "video/mj2": {
            "source": "iana",
            "extensions": ["mj2", "mjp2"]
        },
        "video/mp1s": {
            "source": "iana"
        },
        "video/mp2p": {
            "source": "iana"
        },
        "video/mp2t": {
            "source": "iana",
            "extensions": ["ts"]
        },
        "video/mp4": {
            "source": "iana",
            "compressible": false,
            "extensions": ["mp4", "mp4v", "mpg4"]
        },
        "video/mp4v-es": {
            "source": "iana"
        },
        "video/mpeg": {
            "source": "iana",
            "compressible": false,
            "extensions": ["mpeg", "mpg", "mpe", "m1v", "m2v"]
        },
        "video/mpeg4-generic": {
            "source": "iana"
        },
        "video/mpv": {
            "source": "iana"
        },
        "video/nv": {
            "source": "iana"
        },
        "video/ogg": {
            "source": "iana",
            "compressible": false,
            "extensions": ["ogv"]
        },
        "video/parityfec": {
            "source": "iana"
        },
        "video/pointer": {
            "source": "iana"
        },
        "video/quicktime": {
            "source": "iana",
            "compressible": false,
            "extensions": ["qt", "mov"]
        },
        "video/raptorfec": {
            "source": "iana"
        },
        "video/raw": {
            "source": "iana"
        },
        "video/rtp-enc-aescm128": {
            "source": "iana"
        },
        "video/rtploopback": {
            "source": "iana"
        },
        "video/rtx": {
            "source": "iana"
        },
        "video/smpte291": {
            "source": "iana"
        },
        "video/smpte292m": {
            "source": "iana"
        },
        "video/ulpfec": {
            "source": "iana"
        },
        "video/vc1": {
            "source": "iana"
        },
        "video/vc2": {
            "source": "iana"
        },
        "video/vnd.cctv": {
            "source": "iana"
        },
        "video/vnd.dece.hd": {
            "source": "iana",
            "extensions": ["uvh", "uvvh"]
        },
        "video/vnd.dece.mobile": {
            "source": "iana",
            "extensions": ["uvm", "uvvm"]
        },
        "video/vnd.dece.mp4": {
            "source": "iana"
        },
        "video/vnd.dece.pd": {
            "source": "iana",
            "extensions": ["uvp", "uvvp"]
        },
        "video/vnd.dece.sd": {
            "source": "iana",
            "extensions": ["uvs", "uvvs"]
        },
        "video/vnd.dece.video": {
            "source": "iana",
            "extensions": ["uvv", "uvvv"]
        },
        "video/vnd.directv.mpeg": {
            "source": "iana"
        },
        "video/vnd.directv.mpeg-tts": {
            "source": "iana"
        },
        "video/vnd.dlna.mpeg-tts": {
            "source": "iana"
        },
        "video/vnd.dvb.file": {
            "source": "iana",
            "extensions": ["dvb"]
        },
        "video/vnd.fvt": {
            "source": "iana",
            "extensions": ["fvt"]
        },
        "video/vnd.hns.video": {
            "source": "iana"
        },
        "video/vnd.iptvforum.1dparityfec-1010": {
            "source": "iana"
        },
        "video/vnd.iptvforum.1dparityfec-2005": {
            "source": "iana"
        },
        "video/vnd.iptvforum.2dparityfec-1010": {
            "source": "iana"
        },
        "video/vnd.iptvforum.2dparityfec-2005": {
            "source": "iana"
        },
        "video/vnd.iptvforum.ttsavc": {
            "source": "iana"
        },
        "video/vnd.iptvforum.ttsmpeg2": {
            "source": "iana"
        },
        "video/vnd.motorola.video": {
            "source": "iana"
        },
        "video/vnd.motorola.videop": {
            "source": "iana"
        },
        "video/vnd.mpegurl": {
            "source": "iana",
            "extensions": ["mxu", "m4u"]
        },
        "video/vnd.ms-playready.media.pyv": {
            "source": "iana",
            "extensions": ["pyv"]
        },
        "video/vnd.nokia.interleaved-multimedia": {
            "source": "iana"
        },
        "video/vnd.nokia.mp4vr": {
            "source": "iana"
        },
        "video/vnd.nokia.videovoip": {
            "source": "iana"
        },
        "video/vnd.objectvideo": {
            "source": "iana"
        },
        "video/vnd.radgamettools.bink": {
            "source": "iana"
        },
        "video/vnd.radgamettools.smacker": {
            "source": "iana"
        },
        "video/vnd.sealed.mpeg1": {
            "source": "iana"
        },
        "video/vnd.sealed.mpeg4": {
            "source": "iana"
        },
        "video/vnd.sealed.swf": {
            "source": "iana"
        },
        "video/vnd.sealedmedia.softseal.mov": {
            "source": "iana"
        },
        "video/vnd.uvvu.mp4": {
            "source": "iana",
            "extensions": ["uvu", "uvvu"]
        },
        "video/vnd.vivo": {
            "source": "iana",
            "extensions": ["viv"]
        },
        "video/vnd.youtube.yt": {
            "source": "iana"
        },
        "video/vp8": {
            "source": "iana"
        },
        "video/webm": {
            "source": "apache",
            "compressible": false,
            "extensions": ["webm"]
        },
        "video/x-f4v": {
            "source": "apache",
            "extensions": ["f4v"]
        },
        "video/x-fli": {
            "source": "apache",
            "extensions": ["fli"]
        },
        "video/x-flv": {
            "source": "apache",
            "compressible": false,
            "extensions": ["flv"]
        },
        "video/x-m4v": {
            "source": "apache",
            "extensions": ["m4v"]
        },
        "video/x-matroska": {
            "source": "apache",
            "compressible": false,
            "extensions": ["mkv", "mk3d", "mks"]
        },
        "video/x-mng": {
            "source": "apache",
            "extensions": ["mng"]
        },
        "video/x-ms-asf": {
            "source": "apache",
            "extensions": ["asf", "asx"]
        },
        "video/x-ms-vob": {
            "source": "apache",
            "extensions": ["vob"]
        },
        "video/x-ms-wm": {
            "source": "apache",
            "extensions": ["wm"]
        },
        "video/x-ms-wmv": {
            "source": "apache",
            "compressible": false,
            "extensions": ["wmv"]
        },
        "video/x-ms-wmx": {
            "source": "apache",
            "extensions": ["wmx"]
        },
        "video/x-ms-wvx": {
            "source": "apache",
            "extensions": ["wvx"]
        },
        "video/x-msvideo": {
            "source": "apache",
            "extensions": ["avi"]
        },
        "video/x-sgi-movie": {
            "source": "apache",
            "extensions": ["movie"]
        },
        "video/x-smv": {
            "source": "apache",
            "extensions": ["smv"]
        },
        "x-conference/x-cooltalk": {
            "source": "apache",
            "extensions": ["ice"]
        },
        "x-shader/x-fragment": {
            "compressible": true
        },
        "x-shader/x-vertex": {
            "compressible": true
        }
    };
},
/* caseless/index.js */ function _(require, module, exports) {
    function Caseless(dict) {
        this.dict = dict || {};
    }
    Caseless.prototype.set = function (name, value, clobber) {
        if (typeof name === 'object') {
            for (var i in name) {
                this.set(i, name[i], value);
            }
        }
        else {
            if (typeof clobber === 'undefined')
                clobber = true;
            var has = this.has(name);
            if (!clobber && has)
                this.dict[has] = this.dict[has] + ',' + value;
            else
                this.dict[has || name] = value;
            return has;
        }
    };
    Caseless.prototype.has = function (name) {
        var keys = Object.keys(this.dict), name = name.toLowerCase();
        for (var i = 0; i < keys.length; i++) {
            if (keys[i].toLowerCase() === name)
                return keys[i];
        }
        return false;
    };
    Caseless.prototype.get = function (name) {
        name = name.toLowerCase();
        var result, _key;
        var headers = this.dict;
        Object.keys(headers).forEach(function (key) {
            _key = key.toLowerCase();
            if (name === _key)
                result = headers[key];
        });
        return result;
    };
    Caseless.prototype.swap = function (name) {
        var has = this.has(name);
        if (has === name)
            return;
        if (!has)
            throw new Error('There is no header than matches "' + name + '"');
        this.dict[name] = this.dict[has];
        delete this.dict[has];
    };
    Caseless.prototype.del = function (name) {
        var has = this.has(name);
        return delete this.dict[has || name];
    };
    module.exports = function (dict) { return new Caseless(dict); };
    module.exports.httpify = function (resp, headers) {
        var c = new Caseless(headers);
        resp.setHeader = function (key, value, clobber) {
            if (typeof value === 'undefined')
                return;
            return c.set(key, value, clobber);
        };
        resp.hasHeader = function (key) {
            return c.has(key);
        };
        resp.getHeader = function (key) {
            return c.get(key);
        };
        resp.removeHeader = function (key) {
            return c.del(key);
        };
        resp.headers = c.dict;
        return c;
    };
},
/* forever-agent/index.js */ function _(require, module, exports) {
    module.exports = ForeverAgent;
    ForeverAgent.SSL = ForeverAgentSSL;
    var util = require('util'), Agent = require('http').Agent, net = require('net'), tls = require('tls'), AgentSSL = require('https').Agent;
    function getConnectionName(host, port) {
        var name = '';
        if (typeof host === 'string') {
            name = host + ':' + port;
        }
        else {
            // For node.js v012.0 and iojs-v1.5.1, host is an object. And any existing localAddress is part of the connection name.
            name = host.host + ':' + host.port + ':' + (host.localAddress ? (host.localAddress + ':') : ':');
        }
        return name;
    }
    function ForeverAgent(options) {
        var self = this;
        self.options = options || {};
        self.requests = {};
        self.sockets = {};
        self.freeSockets = {};
        self.maxSockets = self.options.maxSockets || Agent.defaultMaxSockets;
        self.minSockets = self.options.minSockets || ForeverAgent.defaultMinSockets;
        self.on('free', function (socket, host, port) {
            var name = getConnectionName(host, port);
            if (self.requests[name] && self.requests[name].length) {
                self.requests[name].shift().onSocket(socket);
            }
            else if (self.sockets[name].length < self.minSockets) {
                if (!self.freeSockets[name])
                    self.freeSockets[name] = [];
                self.freeSockets[name].push(socket);
                // if an error happens while we don't use the socket anyway, meh, throw the socket away
                var onIdleError = function () {
                    socket.destroy();
                };
                socket._onIdleError = onIdleError;
                socket.on('error', onIdleError);
            }
            else {
                // If there are no pending requests just destroy the
                // socket and it will get removed from the pool. This
                // gets us out of timeout issues and allows us to
                // default to Connection:keep-alive.
                socket.destroy();
            }
        });
    }
    util.inherits(ForeverAgent, Agent);
    ForeverAgent.defaultMinSockets = 5;
    ForeverAgent.prototype.createConnection = net.createConnection;
    ForeverAgent.prototype.addRequestNoreuse = Agent.prototype.addRequest;
    ForeverAgent.prototype.addRequest = function (req, host, port) {
        var name = getConnectionName(host, port);
        if (typeof host !== 'string') {
            var options = host;
            port = options.port;
            host = options.host;
        }
        if (this.freeSockets[name] && this.freeSockets[name].length > 0 && !req.useChunkedEncodingByDefault) {
            var idleSocket = this.freeSockets[name].pop();
            idleSocket.removeListener('error', idleSocket._onIdleError);
            delete idleSocket._onIdleError;
            req._reusedSocket = true;
            req.onSocket(idleSocket);
        }
        else {
            this.addRequestNoreuse(req, host, port);
        }
    };
    ForeverAgent.prototype.removeSocket = function (s, name, host, port) {
        if (this.sockets[name]) {
            var index = this.sockets[name].indexOf(s);
            if (index !== -1) {
                this.sockets[name].splice(index, 1);
            }
        }
        else if (this.sockets[name] && this.sockets[name].length === 0) {
            // don't leak
            delete this.sockets[name];
            delete this.requests[name];
        }
        if (this.freeSockets[name]) {
            var index = this.freeSockets[name].indexOf(s);
            if (index !== -1) {
                this.freeSockets[name].splice(index, 1);
                if (this.freeSockets[name].length === 0) {
                    delete this.freeSockets[name];
                }
            }
        }
        if (this.requests[name] && this.requests[name].length) {
            // If we have pending requests and a socket gets closed a new one
            // needs to be created to take over in the pool for the one that closed.
            this.createSocket(name, host, port).emit('free');
        }
    };
    function ForeverAgentSSL(options) {
        ForeverAgent.call(this, options);
    }
    util.inherits(ForeverAgentSSL, ForeverAgent);
    ForeverAgentSSL.prototype.createConnection = createConnectionSSL;
    ForeverAgentSSL.prototype.addRequestNoreuse = AgentSSL.prototype.addRequest;
    function createConnectionSSL(port, host, options) {
        if (typeof port === 'object') {
            options = port;
        }
        else if (typeof host === 'object') {
            options = host;
        }
        else if (typeof options === 'object') {
            options = options;
        }
        else {
            options = {};
        }
        if (typeof port === 'number') {
            options.port = port;
        }
        if (typeof host === 'string') {
            options.host = host;
        }
        return tls.connect(options);
    }
},
/* form-data/lib/form_data.js */ function _(require, module, exports) {
    var CombinedStream = require(906) /* combined-stream */;
    var util = require('util');
    var path = require('path');
    var http = require('http');
    var https = require('https');
    var parseUrl = require('url').parse;
    var fs = require('fs');
    var mime = require(900) /* mime-types */;
    var asynckit = require(908) /* asynckit */;
    var populate = require(918) /* ./populate.js */;
    // Public API
    module.exports = FormData;
    // make it a Stream
    util.inherits(FormData, CombinedStream);
    /**
     * Create readable "multipart/form-data" streams.
     * Can be used to submit forms
     * and file uploads to other web applications.
     *
     * @constructor
     * @param {Object} options - Properties to be added/overriden for FormData and CombinedStream
     */
    function FormData(options) {
        if (!(this instanceof FormData)) {
            return new FormData();
        }
        this._overheadLength = 0;
        this._valueLength = 0;
        this._valuesToMeasure = [];
        CombinedStream.call(this);
        options = options || {};
        for (var option in options) {
            this[option] = options[option];
        }
    }
    FormData.LINE_BREAK = '\r\n';
    FormData.DEFAULT_CONTENT_TYPE = 'application/octet-stream';
    FormData.prototype.append = function (field, value, options) {
        options = options || {};
        // allow filename as single option
        if (typeof options == 'string') {
            options = { filename: options };
        }
        var append = CombinedStream.prototype.append.bind(this);
        // all that streamy business can't handle numbers
        if (typeof value == 'number') {
            value = '' + value;
        }
        // https://github.com/felixge/node-form-data/issues/38
        if (util.isArray(value)) {
            // Please convert your array into string
            // the way web server expects it
            this._error(new Error('Arrays are not supported.'));
            return;
        }
        var header = this._multiPartHeader(field, value, options);
        var footer = this._multiPartFooter();
        append(header);
        append(value);
        append(footer);
        // pass along options.knownLength
        this._trackLength(header, value, options);
    };
    FormData.prototype._trackLength = function (header, value, options) {
        var valueLength = 0;
        // used w/ getLengthSync(), when length is known.
        // e.g. for streaming directly from a remote server,
        // w/ a known file a size, and not wanting to wait for
        // incoming file to finish to get its size.
        if (options.knownLength != null) {
            valueLength += +options.knownLength;
        }
        else if (Buffer.isBuffer(value)) {
            valueLength = value.length;
        }
        else if (typeof value === 'string') {
            valueLength = Buffer.byteLength(value);
        }
        this._valueLength += valueLength;
        // @check why add CRLF? does this account for custom/multiple CRLFs?
        this._overheadLength +=
            Buffer.byteLength(header) +
                FormData.LINE_BREAK.length;
        // empty or either doesn't have path or not an http response
        if (!value || (!value.path && !(value.readable && value.hasOwnProperty('httpVersion')))) {
            return;
        }
        // no need to bother with the length
        if (!options.knownLength) {
            this._valuesToMeasure.push(value);
        }
    };
    FormData.prototype._lengthRetriever = function (value, callback) {
        if (value.hasOwnProperty('fd')) {
            // take read range into a account
            // `end` = Infinity –> read file till the end
            //
            // TODO: Looks like there is bug in Node fs.createReadStream
            // it doesn't respect `end` options without `start` options
            // Fix it when node fixes it.
            // https://github.com/joyent/node/issues/7819
            if (value.end != undefined && value.end != Infinity && value.start != undefined) {
                // when end specified
                // no need to calculate range
                // inclusive, starts with 0
                callback(null, value.end + 1 - (value.start ? value.start : 0));
                // not that fast snoopy
            }
            else {
                // still need to fetch file size from fs
                fs.stat(value.path, function (err, stat) {
                    var fileSize;
                    if (err) {
                        callback(err);
                        return;
                    }
                    // update final size based on the range options
                    fileSize = stat.size - (value.start ? value.start : 0);
                    callback(null, fileSize);
                });
            }
            // or http response
        }
        else if (value.hasOwnProperty('httpVersion')) {
            callback(null, +value.headers['content-length']);
            // or request stream http://github.com/mikeal/request
        }
        else if (value.hasOwnProperty('httpModule')) {
            // wait till response come back
            value.on('response', function (response) {
                value.pause();
                callback(null, +response.headers['content-length']);
            });
            value.resume();
            // something else
        }
        else {
            callback('Unknown stream');
        }
    };
    FormData.prototype._multiPartHeader = function (field, value, options) {
        // custom header specified (as string)?
        // it becomes responsible for boundary
        // (e.g. to handle extra CRLFs on .NET servers)
        if (typeof options.header == 'string') {
            return options.header;
        }
        var contentDisposition = this._getContentDisposition(value, options);
        var contentType = this._getContentType(value, options);
        var contents = '';
        var headers = {
            // add custom disposition as third element or keep it two elements if not
            'Content-Disposition': ['form-data', 'name="' + field + '"'].concat(contentDisposition || []),
            // if no content type. allow it to be empty array
            'Content-Type': [].concat(contentType || [])
        };
        // allow custom headers.
        if (typeof options.header == 'object') {
            populate(headers, options.header);
        }
        var header;
        for (var prop in headers) {
            if (!headers.hasOwnProperty(prop))
                continue;
            header = headers[prop];
            // skip nullish headers.
            if (header == null) {
                continue;
            }
            // convert all headers to arrays.
            if (!Array.isArray(header)) {
                header = [header];
            }
            // add non-empty headers.
            if (header.length) {
                contents += prop + ': ' + header.join('; ') + FormData.LINE_BREAK;
            }
        }
        return '--' + this.getBoundary() + FormData.LINE_BREAK + contents + FormData.LINE_BREAK;
    };
    FormData.prototype._getContentDisposition = function (value, options) {
        var filename, contentDisposition;
        if (typeof options.filepath === 'string') {
            // custom filepath for relative paths
            filename = path.normalize(options.filepath).replace(/\\/g, '/');
        }
        else if (options.filename || value.name || value.path) {
            // custom filename take precedence
            // formidable and the browser add a name property
            // fs- and request- streams have path property
            filename = path.basename(options.filename || value.name || value.path);
        }
        else if (value.readable && value.hasOwnProperty('httpVersion')) {
            // or try http response
            filename = path.basename(value.client._httpMessage.path);
        }
        if (filename) {
            contentDisposition = 'filename="' + filename + '"';
        }
        return contentDisposition;
    };
    FormData.prototype._getContentType = function (value, options) {
        // use custom content-type above all
        var contentType = options.contentType;
        // or try `name` from formidable, browser
        if (!contentType && value.name) {
            contentType = mime.lookup(value.name);
        }
        // or try `path` from fs-, request- streams
        if (!contentType && value.path) {
            contentType = mime.lookup(value.path);
        }
        // or if it's http-reponse
        if (!contentType && value.readable && value.hasOwnProperty('httpVersion')) {
            contentType = value.headers['content-type'];
        }
        // or guess it from the filepath or filename
        if (!contentType && (options.filepath || options.filename)) {
            contentType = mime.lookup(options.filepath || options.filename);
        }
        // fallback to the default content type if `value` is not simple value
        if (!contentType && typeof value == 'object') {
            contentType = FormData.DEFAULT_CONTENT_TYPE;
        }
        return contentType;
    };
    FormData.prototype._multiPartFooter = function () {
        return function (next) {
            var footer = FormData.LINE_BREAK;
            var lastPart = (this._streams.length === 0);
            if (lastPart) {
                footer += this._lastBoundary();
            }
            next(footer);
        }.bind(this);
    };
    FormData.prototype._lastBoundary = function () {
        return '--' + this.getBoundary() + '--' + FormData.LINE_BREAK;
    };
    FormData.prototype.getHeaders = function (userHeaders) {
        var header;
        var formHeaders = {
            'content-type': 'multipart/form-data; boundary=' + this.getBoundary()
        };
        for (header in userHeaders) {
            if (userHeaders.hasOwnProperty(header)) {
                formHeaders[header.toLowerCase()] = userHeaders[header];
            }
        }
        return formHeaders;
    };
    FormData.prototype.getBoundary = function () {
        if (!this._boundary) {
            this._generateBoundary();
        }
        return this._boundary;
    };
    FormData.prototype._generateBoundary = function () {
        // This generates a 50 character boundary similar to those used by Firefox.
        // They are optimized for boyer-moore parsing.
        var boundary = '--------------------------';
        for (var i = 0; i < 24; i++) {
            boundary += Math.floor(Math.random() * 10).toString(16);
        }
        this._boundary = boundary;
    };
    // Note: getLengthSync DOESN'T calculate streams length
    // As workaround one can calculate file size manually
    // and add it as knownLength option
    FormData.prototype.getLengthSync = function () {
        var knownLength = this._overheadLength + this._valueLength;
        // Don't get confused, there are 3 "internal" streams for each keyval pair
        // so it basically checks if there is any value added to the form
        if (this._streams.length) {
            knownLength += this._lastBoundary().length;
        }
        // https://github.com/form-data/form-data/issues/40
        if (!this.hasKnownLength()) {
            // Some async length retrievers are present
            // therefore synchronous length calculation is false.
            // Please use getLength(callback) to get proper length
            this._error(new Error('Cannot calculate proper length in synchronous way.'));
        }
        return knownLength;
    };
    // Public API to check if length of added values is known
    // https://github.com/form-data/form-data/issues/196
    // https://github.com/form-data/form-data/issues/262
    FormData.prototype.hasKnownLength = function () {
        var hasKnownLength = true;
        if (this._valuesToMeasure.length) {
            hasKnownLength = false;
        }
        return hasKnownLength;
    };
    FormData.prototype.getLength = function (cb) {
        var knownLength = this._overheadLength + this._valueLength;
        if (this._streams.length) {
            knownLength += this._lastBoundary().length;
        }
        if (!this._valuesToMeasure.length) {
            process.nextTick(cb.bind(this, null, knownLength));
            return;
        }
        asynckit.parallel(this._valuesToMeasure, this._lengthRetriever, function (err, values) {
            if (err) {
                cb(err);
                return;
            }
            values.forEach(function (length) {
                knownLength += length;
            });
            cb(null, knownLength);
        });
    };
    FormData.prototype.submit = function (params, cb) {
        var request, options, defaults = { method: 'post' };
        // parse provided url if it's string
        // or treat it as options object
        if (typeof params == 'string') {
            params = parseUrl(params);
            options = populate({
                port: params.port,
                path: params.pathname,
                host: params.hostname,
                protocol: params.protocol
            }, defaults);
            // use custom params
        }
        else {
            options = populate(params, defaults);
            // if no port provided use default one
            if (!options.port) {
                options.port = options.protocol == 'https:' ? 443 : 80;
            }
        }
        // put that good code in getHeaders to some use
        options.headers = this.getHeaders(params.headers);
        // https if specified, fallback to http in any other case
        if (options.protocol == 'https:') {
            request = https.request(options);
        }
        else {
            request = http.request(options);
        }
        // get content length and fire away
        this.getLength(function (err, length) {
            if (err) {
                this._error(err);
                return;
            }
            // add content length
            request.setHeader('Content-Length', length);
            this.pipe(request);
            if (cb) {
                request.on('error', cb);
                request.on('response', cb.bind(this, null));
            }
        }.bind(this));
        return request;
    };
    FormData.prototype._error = function (err) {
        if (!this.error) {
            this.error = err;
            this.pause();
            this.emit('error', err);
        }
    };
    FormData.prototype.toString = function () {
        return '[object FormData]';
    };
},
/* combined-stream/lib/combined_stream.js */ function _(require, module, exports) {
    var util = require('util');
    var Stream = require('stream').Stream;
    var DelayedStream = require(907) /* delayed-stream */;
    module.exports = CombinedStream;
    function CombinedStream() {
        this.writable = false;
        this.readable = true;
        this.dataSize = 0;
        this.maxDataSize = 2 * 1024 * 1024;
        this.pauseStreams = true;
        this._released = false;
        this._streams = [];
        this._currentStream = null;
        this._insideLoop = false;
        this._pendingNext = false;
    }
    util.inherits(CombinedStream, Stream);
    CombinedStream.create = function (options) {
        var combinedStream = new this();
        options = options || {};
        for (var option in options) {
            combinedStream[option] = options[option];
        }
        return combinedStream;
    };
    CombinedStream.isStreamLike = function (stream) {
        return (typeof stream !== 'function')
            && (typeof stream !== 'string')
            && (typeof stream !== 'boolean')
            && (typeof stream !== 'number')
            && (!Buffer.isBuffer(stream));
    };
    CombinedStream.prototype.append = function (stream) {
        var isStreamLike = CombinedStream.isStreamLike(stream);
        if (isStreamLike) {
            if (!(stream instanceof DelayedStream)) {
                var newStream = DelayedStream.create(stream, {
                    maxDataSize: Infinity,
                    pauseStream: this.pauseStreams,
                });
                stream.on('data', this._checkDataSize.bind(this));
                stream = newStream;
            }
            this._handleErrors(stream);
            if (this.pauseStreams) {
                stream.pause();
            }
        }
        this._streams.push(stream);
        return this;
    };
    CombinedStream.prototype.pipe = function (dest, options) {
        Stream.prototype.pipe.call(this, dest, options);
        this.resume();
        return dest;
    };
    CombinedStream.prototype._getNext = function () {
        this._currentStream = null;
        if (this._insideLoop) {
            this._pendingNext = true;
            return; // defer call
        }
        this._insideLoop = true;
        try {
            do {
                this._pendingNext = false;
                this._realGetNext();
            } while (this._pendingNext);
        }
        finally {
            this._insideLoop = false;
        }
    };
    CombinedStream.prototype._realGetNext = function () {
        var stream = this._streams.shift();
        if (typeof stream == 'undefined') {
            this.end();
            return;
        }
        if (typeof stream !== 'function') {
            this._pipeNext(stream);
            return;
        }
        var getStream = stream;
        getStream(function (stream) {
            var isStreamLike = CombinedStream.isStreamLike(stream);
            if (isStreamLike) {
                stream.on('data', this._checkDataSize.bind(this));
                this._handleErrors(stream);
            }
            this._pipeNext(stream);
        }.bind(this));
    };
    CombinedStream.prototype._pipeNext = function (stream) {
        this._currentStream = stream;
        var isStreamLike = CombinedStream.isStreamLike(stream);
        if (isStreamLike) {
            stream.on('end', this._getNext.bind(this));
            stream.pipe(this, { end: false });
            return;
        }
        var value = stream;
        this.write(value);
        this._getNext();
    };
    CombinedStream.prototype._handleErrors = function (stream) {
        var self = this;
        stream.on('error', function (err) {
            self._emitError(err);
        });
    };
    CombinedStream.prototype.write = function (data) {
        this.emit('data', data);
    };
    CombinedStream.prototype.pause = function () {
        if (!this.pauseStreams) {
            return;
        }
        if (this.pauseStreams && this._currentStream && typeof (this._currentStream.pause) == 'function')
            this._currentStream.pause();
        this.emit('pause');
    };
    CombinedStream.prototype.resume = function () {
        if (!this._released) {
            this._released = true;
            this.writable = true;
            this._getNext();
        }
        if (this.pauseStreams && this._currentStream && typeof (this._currentStream.resume) == 'function')
            this._currentStream.resume();
        this.emit('resume');
    };
    CombinedStream.prototype.end = function () {
        this._reset();
        this.emit('end');
    };
    CombinedStream.prototype.destroy = function () {
        this._reset();
        this.emit('close');
    };
    CombinedStream.prototype._reset = function () {
        this.writable = false;
        this._streams = [];
        this._currentStream = null;
    };
    CombinedStream.prototype._checkDataSize = function () {
        this._updateDataSize();
        if (this.dataSize <= this.maxDataSize) {
            return;
        }
        var message = 'DelayedStream#maxDataSize of ' + this.maxDataSize + ' bytes exceeded.';
        this._emitError(new Error(message));
    };
    CombinedStream.prototype._updateDataSize = function () {
        this.dataSize = 0;
        var self = this;
        this._streams.forEach(function (stream) {
            if (!stream.dataSize) {
                return;
            }
            self.dataSize += stream.dataSize;
        });
        if (this._currentStream && this._currentStream.dataSize) {
            this.dataSize += this._currentStream.dataSize;
        }
    };
    CombinedStream.prototype._emitError = function (err) {
        this._reset();
        this.emit('error', err);
    };
},
/* delayed-stream/lib/delayed_stream.js */ function _(require, module, exports) {
    var Stream = require('stream').Stream;
    var util = require('util');
    module.exports = DelayedStream;
    function DelayedStream() {
        this.source = null;
        this.dataSize = 0;
        this.maxDataSize = 1024 * 1024;
        this.pauseStream = true;
        this._maxDataSizeExceeded = false;
        this._released = false;
        this._bufferedEvents = [];
    }
    util.inherits(DelayedStream, Stream);
    DelayedStream.create = function (source, options) {
        var delayedStream = new this();
        options = options || {};
        for (var option in options) {
            delayedStream[option] = options[option];
        }
        delayedStream.source = source;
        var realEmit = source.emit;
        source.emit = function () {
            delayedStream._handleEmit(arguments);
            return realEmit.apply(source, arguments);
        };
        source.on('error', function () { });
        if (delayedStream.pauseStream) {
            source.pause();
        }
        return delayedStream;
    };
    Object.defineProperty(DelayedStream.prototype, 'readable', {
        configurable: true,
        enumerable: true,
        get: function () {
            return this.source.readable;
        }
    });
    DelayedStream.prototype.setEncoding = function () {
        return this.source.setEncoding.apply(this.source, arguments);
    };
    DelayedStream.prototype.resume = function () {
        if (!this._released) {
            this.release();
        }
        this.source.resume();
    };
    DelayedStream.prototype.pause = function () {
        this.source.pause();
    };
    DelayedStream.prototype.release = function () {
        this._released = true;
        this._bufferedEvents.forEach(function (args) {
            this.emit.apply(this, args);
        }.bind(this));
        this._bufferedEvents = [];
    };
    DelayedStream.prototype.pipe = function () {
        var r = Stream.prototype.pipe.apply(this, arguments);
        this.resume();
        return r;
    };
    DelayedStream.prototype._handleEmit = function (args) {
        if (this._released) {
            this.emit.apply(this, args);
            return;
        }
        if (args[0] === 'data') {
            this.dataSize += args[1].length;
            this._checkIfMaxDataSizeExceeded();
        }
        this._bufferedEvents.push(args);
    };
    DelayedStream.prototype._checkIfMaxDataSizeExceeded = function () {
        if (this._maxDataSizeExceeded) {
            return;
        }
        if (this.dataSize <= this.maxDataSize) {
            return;
        }
        this._maxDataSizeExceeded = true;
        var message = 'DelayedStream#maxDataSize of ' + this.maxDataSize + ' bytes exceeded.';
        this.emit('error', new Error(message));
    };
},
/* asynckit/index.js */ function _(require, module, exports) {
    module.exports =
        {
            parallel: require(909) /* ./parallel.js */,
            serial: require(916) /* ./serial.js */,
            serialOrdered: require(917) /* ./serialOrdered.js */
        };
},
/* asynckit/parallel.js */ function _(require, module, exports) {
    var iterate = require(910) /* ./lib/iterate.js */, initState = require(914) /* ./lib/state.js */, terminator = require(915) /* ./lib/terminator.js */;
    // Public API
    module.exports = parallel;
    /**
     * Runs iterator over provided array elements in parallel
     *
     * @param   {array|object} list - array or object (named list) to iterate over
     * @param   {function} iterator - iterator to run
     * @param   {function} callback - invoked when all elements processed
     * @returns {function} - jobs terminator
     */
    function parallel(list, iterator, callback) {
        var state = initState(list);
        while (state.index < (state['keyedList'] || list).length) {
            iterate(list, iterator, state, function (error, result) {
                if (error) {
                    callback(error, result);
                    return;
                }
                // looks like it's the last one
                if (Object.keys(state.jobs).length === 0) {
                    callback(null, state.results);
                    return;
                }
            });
            state.index++;
        }
        return terminator.bind(state, callback);
    }
},
/* asynckit/lib/iterate.js */ function _(require, module, exports) {
    var async = require(911) /* ./async.js */, abort = require(913) /* ./abort.js */;
    // API
    module.exports = iterate;
    /**
     * Iterates over each job object
     *
     * @param {array|object} list - array or object (named list) to iterate over
     * @param {function} iterator - iterator to run
     * @param {object} state - current job status
     * @param {function} callback - invoked when all elements processed
     */
    function iterate(list, iterator, state, callback) {
        // store current index
        var key = state['keyedList'] ? state['keyedList'][state.index] : state.index;
        state.jobs[key] = runJob(iterator, key, list[key], function (error, output) {
            // don't repeat yourself
            // skip secondary callbacks
            if (!(key in state.jobs)) {
                return;
            }
            // clean up jobs
            delete state.jobs[key];
            if (error) {
                // don't process rest of the results
                // stop still active jobs
                // and reset the list
                abort(state);
            }
            else {
                state.results[key] = output;
            }
            // return salvaged results
            callback(error, state.results);
        });
    }
    /**
     * Runs iterator over provided job element
     *
     * @param   {function} iterator - iterator to invoke
     * @param   {string|number} key - key/index of the element in the list of jobs
     * @param   {mixed} item - job description
     * @param   {function} callback - invoked after iterator is done with the job
     * @returns {function|mixed} - job abort function or something else
     */
    function runJob(iterator, key, item, callback) {
        var aborter;
        // allow shortcut if iterator expects only two arguments
        if (iterator.length == 2) {
            aborter = iterator(item, async(callback));
        }
        // otherwise go with full three arguments
        else {
            aborter = iterator(item, key, async(callback));
        }
        return aborter;
    }
},
/* asynckit/lib/async.js */ function _(require, module, exports) {
    var defer = require(912) /* ./defer.js */;
    // API
    module.exports = async;
    /**
     * Runs provided callback asynchronously
     * even if callback itself is not
     *
     * @param   {function} callback - callback to invoke
     * @returns {function} - augmented callback
     */
    function async(callback) {
        var isAsync = false;
        // check if async happened
        defer(function () { isAsync = true; });
        return function async_callback(err, result) {
            if (isAsync) {
                callback(err, result);
            }
            else {
                defer(function nextTick_callback() {
                    callback(err, result);
                });
            }
        };
    }
},
/* asynckit/lib/defer.js */ function _(require, module, exports) {
    module.exports = defer;
    /**
     * Runs provided function on next iteration of the event loop
     *
     * @param {function} fn - function to run
     */
    function defer(fn) {
        var nextTick = typeof setImmediate == 'function'
            ? setImmediate
            : (typeof process == 'object' && typeof process.nextTick == 'function'
                ? process.nextTick
                : null);
        if (nextTick) {
            nextTick(fn);
        }
        else {
            setTimeout(fn, 0);
        }
    }
},
/* asynckit/lib/abort.js */ function _(require, module, exports) {
    // API
    module.exports = abort;
    /**
     * Aborts leftover active jobs
     *
     * @param {object} state - current state object
     */
    function abort(state) {
        Object.keys(state.jobs).forEach(clean.bind(state));
        // reset leftover jobs
        state.jobs = {};
    }
    /**
     * Cleans up leftover job by invoking abort function for the provided job id
     *
     * @this  state
     * @param {string|number} key - job id to abort
     */
    function clean(key) {
        if (typeof this.jobs[key] == 'function') {
            this.jobs[key]();
        }
    }
},
/* asynckit/lib/state.js */ function _(require, module, exports) {
    // API
    module.exports = state;
    /**
     * Creates initial state object
     * for iteration over list
     *
     * @param   {array|object} list - list to iterate over
     * @param   {function|null} sortMethod - function to use for keys sort,
     *                                     or `null` to keep them as is
     * @returns {object} - initial state object
     */
    function state(list, sortMethod) {
        var isNamedList = !Array.isArray(list), initState = {
            index: 0,
            keyedList: isNamedList || sortMethod ? Object.keys(list) : null,
            jobs: {},
            results: isNamedList ? {} : [],
            size: isNamedList ? Object.keys(list).length : list.length
        };
        if (sortMethod) {
            // sort array keys based on it's values
            // sort object's keys just on own merit
            initState.keyedList.sort(isNamedList ? sortMethod : function (a, b) {
                return sortMethod(list[a], list[b]);
            });
        }
        return initState;
    }
},
/* asynckit/lib/terminator.js */ function _(require, module, exports) {
    var abort = require(913) /* ./abort.js */, async = require(911) /* ./async.js */;
    // API
    module.exports = terminator;
    /**
     * Terminates jobs in the attached state context
     *
     * @this  AsyncKitState#
     * @param {function} callback - final callback to invoke after termination
     */
    function terminator(callback) {
        if (!Object.keys(this.jobs).length) {
            return;
        }
        // fast forward iteration index
        this.index = this.size;
        // abort jobs
        abort(this);
        // send back results we have so far
        async(callback)(null, this.results);
    }
},
/* asynckit/serial.js */ function _(require, module, exports) {
    var serialOrdered = require(917) /* ./serialOrdered.js */;
    // Public API
    module.exports = serial;
    /**
     * Runs iterator over provided array elements in series
     *
     * @param   {array|object} list - array or object (named list) to iterate over
     * @param   {function} iterator - iterator to run
     * @param   {function} callback - invoked when all elements processed
     * @returns {function} - jobs terminator
     */
    function serial(list, iterator, callback) {
        return serialOrdered(list, iterator, null, callback);
    }
},
/* asynckit/serialOrdered.js */ function _(require, module, exports) {
    var iterate = require(910) /* ./lib/iterate.js */, initState = require(914) /* ./lib/state.js */, terminator = require(915) /* ./lib/terminator.js */;
    // Public API
    module.exports = serialOrdered;
    // sorting helpers
    module.exports.ascending = ascending;
    module.exports.descending = descending;
    /**
     * Runs iterator over provided sorted array elements in series
     *
     * @param   {array|object} list - array or object (named list) to iterate over
     * @param   {function} iterator - iterator to run
     * @param   {function} sortMethod - custom sort function
     * @param   {function} callback - invoked when all elements processed
     * @returns {function} - jobs terminator
     */
    function serialOrdered(list, iterator, sortMethod, callback) {
        var state = initState(list, sortMethod);
        iterate(list, iterator, state, function iteratorHandler(error, result) {
            if (error) {
                callback(error, result);
                return;
            }
            state.index++;
            // are we there yet?
            if (state.index < (state['keyedList'] || list).length) {
                iterate(list, iterator, state, iteratorHandler);
                return;
            }
            // done here
            callback(null, state.results);
        });
        return terminator.bind(state, callback);
    }
    /*
     * -- Sort methods
     */
    /**
     * sort helper to sort array elements in ascending order
     *
     * @param   {mixed} a - an item to compare
     * @param   {mixed} b - an item to compare
     * @returns {number} - comparison result
     */
    function ascending(a, b) {
        return a < b ? -1 : a > b ? 1 : 0;
    }
    /**
     * sort helper to sort array elements in descending order
     *
     * @param   {mixed} a - an item to compare
     * @param   {mixed} b - an item to compare
     * @returns {number} - comparison result
     */
    function descending(a, b) {
        return -1 * ascending(a, b);
    }
},
/* form-data/lib/populate.js */ function _(require, module, exports) {
    // populates missing values
    module.exports = function (dst, src) {
        Object.keys(src).forEach(function (prop) {
            dst[prop] = dst[prop] || src[prop];
        });
        return dst;
    };
},
/* isstream/isstream.js */ function _(require, module, exports) {
    var stream = require('stream');
    function isStream(obj) {
        return obj instanceof stream.Stream;
    }
    function isReadable(obj) {
        return isStream(obj) && typeof obj._read == 'function' && typeof obj._readableState == 'object';
    }
    function isWritable(obj) {
        return isStream(obj) && typeof obj._write == 'function' && typeof obj._writableState == 'object';
    }
    function isDuplex(obj) {
        return isReadable(obj) && isWritable(obj);
    }
    module.exports = isStream;
    module.exports.isReadable = isReadable;
    module.exports.isWritable = isWritable;
    module.exports.isDuplex = isDuplex;
},
/* is-typedarray/index.js */ function _(require, module, exports) {
    module.exports = isTypedArray;
    isTypedArray.strict = isStrictTypedArray;
    isTypedArray.loose = isLooseTypedArray;
    var toString = Object.prototype.toString;
    var names = {
        '[object Int8Array]': true,
        '[object Int16Array]': true,
        '[object Int32Array]': true,
        '[object Uint8Array]': true,
        '[object Uint8ClampedArray]': true,
        '[object Uint16Array]': true,
        '[object Uint32Array]': true,
        '[object Float32Array]': true,
        '[object Float64Array]': true
    };
    function isTypedArray(arr) {
        return (isStrictTypedArray(arr)
            || isLooseTypedArray(arr));
    }
    function isStrictTypedArray(arr) {
        return (arr instanceof Int8Array
            || arr instanceof Int16Array
            || arr instanceof Int32Array
            || arr instanceof Uint8Array
            || arr instanceof Uint8ClampedArray
            || arr instanceof Uint16Array
            || arr instanceof Uint32Array
            || arr instanceof Float32Array
            || arr instanceof Float64Array);
    }
    function isLooseTypedArray(arr) {
        return names[toString.call(arr)];
    }
},
/* request/lib/getProxyFromURI.js */ function _(require, module, exports) {
    function formatHostname(hostname) {
        // canonicalize the hostname, so that 'oogle.com' won't match 'google.com'
        return hostname.replace(/^\.*/, '.').toLowerCase();
    }
    function parseNoProxyZone(zone) {
        zone = zone.trim().toLowerCase();
        var zoneParts = zone.split(':', 2);
        var zoneHost = formatHostname(zoneParts[0]);
        var zonePort = zoneParts[1];
        var hasPort = zone.indexOf(':') > -1;
        return { hostname: zoneHost, port: zonePort, hasPort: hasPort };
    }
    function uriInNoProxy(uri, noProxy) {
        var port = uri.port || (uri.protocol === 'https:' ? '443' : '80');
        var hostname = formatHostname(uri.hostname);
        var noProxyList = noProxy.split(',');
        // iterate through the noProxyList until it finds a match.
        return noProxyList.map(parseNoProxyZone).some(function (noProxyZone) {
            var isMatchedAt = hostname.indexOf(noProxyZone.hostname);
            var hostnameMatched = (isMatchedAt > -1 &&
                (isMatchedAt === hostname.length - noProxyZone.hostname.length));
            if (noProxyZone.hasPort) {
                return (port === noProxyZone.port) && hostnameMatched;
            }
            return hostnameMatched;
        });
    }
    function getProxyFromURI(uri) {
        // Decide the proper request proxy to use based on the request URI object and the
        // environmental variables (NO_PROXY, HTTP_PROXY, etc.)
        // respect NO_PROXY environment variables (see: https://lynx.invisible-island.net/lynx2.8.7/breakout/lynx_help/keystrokes/environments.html)
        var noProxy = process.env.NO_PROXY || process.env.no_proxy || '';
        // if the noProxy is a wildcard then return null
        if (noProxy === '*') {
            return null;
        }
        // if the noProxy is not empty and the uri is found return null
        if (noProxy !== '' && uriInNoProxy(uri, noProxy)) {
            return null;
        }
        // Check for HTTP or HTTPS Proxy in environment Else default to null
        if (uri.protocol === 'http:') {
            return process.env.HTTP_PROXY ||
                process.env.http_proxy || null;
        }
        if (uri.protocol === 'https:') {
            return process.env.HTTPS_PROXY ||
                process.env.https_proxy ||
                process.env.HTTP_PROXY ||
                process.env.http_proxy || null;
        }
        // if none of that works, return null
        // (What uri protocol are you using then?)
        return null;
    }
    module.exports = getProxyFromURI;
},
/* request/lib/querystring.js */ function _(require, module, exports) {
    var qs = require(923) /* qs */;
    var querystring = require('querystring');
    function Querystring(request) {
        this.request = request;
        this.lib = null;
        this.useQuerystring = null;
        this.parseOptions = null;
        this.stringifyOptions = null;
    }
    Querystring.prototype.init = function (options) {
        if (this.lib) {
            return;
        }
        this.useQuerystring = options.useQuerystring;
        this.lib = (this.useQuerystring ? querystring : qs);
        this.parseOptions = options.qsParseOptions || {};
        this.stringifyOptions = options.qsStringifyOptions || {};
    };
    Querystring.prototype.stringify = function (obj) {
        return (this.useQuerystring)
            ? this.rfc3986(this.lib.stringify(obj, this.stringifyOptions.sep || null, this.stringifyOptions.eq || null, this.stringifyOptions))
            : this.lib.stringify(obj, this.stringifyOptions);
    };
    Querystring.prototype.parse = function (str) {
        return (this.useQuerystring)
            ? this.lib.parse(str, this.parseOptions.sep || null, this.parseOptions.eq || null, this.parseOptions)
            : this.lib.parse(str, this.parseOptions);
    };
    Querystring.prototype.rfc3986 = function (str) {
        return str.replace(/[!'()*]/g, function (c) {
            return '%' + c.charCodeAt(0).toString(16).toUpperCase();
        });
    };
    Querystring.prototype.unescape = querystring.unescape;
    exports.Querystring = Querystring;
},
/* qs/lib/index.js */ function _(require, module, exports) {
    var stringify = require(924) /* ./stringify */;
    var parse = require(927) /* ./parse */;
    var formats = require(926) /* ./formats */;
    module.exports = {
        formats: formats,
        parse: parse,
        stringify: stringify
    };
},
/* qs/lib/stringify.js */ function _(require, module, exports) {
    var utils = require(925) /* ./utils */;
    var formats = require(926) /* ./formats */;
    var has = Object.prototype.hasOwnProperty;
    var arrayPrefixGenerators = {
        brackets: function brackets(prefix) {
            return prefix + '[]';
        },
        comma: 'comma',
        indices: function indices(prefix, key) {
            return prefix + '[' + key + ']';
        },
        repeat: function repeat(prefix) {
            return prefix;
        }
    };
    var isArray = Array.isArray;
    var push = Array.prototype.push;
    var pushToArray = function (arr, valueOrArray) {
        push.apply(arr, isArray(valueOrArray) ? valueOrArray : [valueOrArray]);
    };
    var toISO = Date.prototype.toISOString;
    var defaults = {
        addQueryPrefix: false,
        allowDots: false,
        charset: 'utf-8',
        charsetSentinel: false,
        delimiter: '&',
        encode: true,
        encoder: utils.encode,
        encodeValuesOnly: false,
        formatter: formats.formatters[formats['default']],
        // deprecated
        indices: false,
        serializeDate: function serializeDate(date) {
            return toISO.call(date);
        },
        skipNulls: false,
        strictNullHandling: false
    };
    var stringify = function stringify(// eslint-disable-line func-name-matching
    object, prefix, generateArrayPrefix, strictNullHandling, skipNulls, encoder, filter, sort, allowDots, serializeDate, formatter, encodeValuesOnly, charset) {
        var obj = object;
        if (typeof filter === 'function') {
            obj = filter(prefix, obj);
        }
        else if (obj instanceof Date) {
            obj = serializeDate(obj);
        }
        else if (generateArrayPrefix === 'comma' && isArray(obj)) {
            obj = obj.join(',');
        }
        if (obj === null) {
            if (strictNullHandling) {
                return encoder && !encodeValuesOnly ? encoder(prefix, defaults.encoder, charset) : prefix;
            }
            obj = '';
        }
        if (typeof obj === 'string' || typeof obj === 'number' || typeof obj === 'boolean' || utils.isBuffer(obj)) {
            if (encoder) {
                var keyValue = encodeValuesOnly ? prefix : encoder(prefix, defaults.encoder, charset);
                return [formatter(keyValue) + '=' + formatter(encoder(obj, defaults.encoder, charset))];
            }
            return [formatter(prefix) + '=' + formatter(String(obj))];
        }
        var values = [];
        if (typeof obj === 'undefined') {
            return values;
        }
        var objKeys;
        if (isArray(filter)) {
            objKeys = filter;
        }
        else {
            var keys = Object.keys(obj);
            objKeys = sort ? keys.sort(sort) : keys;
        }
        for (var i = 0; i < objKeys.length; ++i) {
            var key = objKeys[i];
            if (skipNulls && obj[key] === null) {
                continue;
            }
            if (isArray(obj)) {
                pushToArray(values, stringify(obj[key], typeof generateArrayPrefix === 'function' ? generateArrayPrefix(prefix, key) : prefix, generateArrayPrefix, strictNullHandling, skipNulls, encoder, filter, sort, allowDots, serializeDate, formatter, encodeValuesOnly, charset));
            }
            else {
                pushToArray(values, stringify(obj[key], prefix + (allowDots ? '.' + key : '[' + key + ']'), generateArrayPrefix, strictNullHandling, skipNulls, encoder, filter, sort, allowDots, serializeDate, formatter, encodeValuesOnly, charset));
            }
        }
        return values;
    };
    var normalizeStringifyOptions = function normalizeStringifyOptions(opts) {
        if (!opts) {
            return defaults;
        }
        if (opts.encoder !== null && opts.encoder !== undefined && typeof opts.encoder !== 'function') {
            throw new TypeError('Encoder has to be a function.');
        }
        var charset = opts.charset || defaults.charset;
        if (typeof opts.charset !== 'undefined' && opts.charset !== 'utf-8' && opts.charset !== 'iso-8859-1') {
            throw new TypeError('The charset option must be either utf-8, iso-8859-1, or undefined');
        }
        var format = formats['default'];
        if (typeof opts.format !== 'undefined') {
            if (!has.call(formats.formatters, opts.format)) {
                throw new TypeError('Unknown format option provided.');
            }
            format = opts.format;
        }
        var formatter = formats.formatters[format];
        var filter = defaults.filter;
        if (typeof opts.filter === 'function' || isArray(opts.filter)) {
            filter = opts.filter;
        }
        return {
            addQueryPrefix: typeof opts.addQueryPrefix === 'boolean' ? opts.addQueryPrefix : defaults.addQueryPrefix,
            allowDots: typeof opts.allowDots === 'undefined' ? defaults.allowDots : !!opts.allowDots,
            charset: charset,
            charsetSentinel: typeof opts.charsetSentinel === 'boolean' ? opts.charsetSentinel : defaults.charsetSentinel,
            delimiter: typeof opts.delimiter === 'undefined' ? defaults.delimiter : opts.delimiter,
            encode: typeof opts.encode === 'boolean' ? opts.encode : defaults.encode,
            encoder: typeof opts.encoder === 'function' ? opts.encoder : defaults.encoder,
            encodeValuesOnly: typeof opts.encodeValuesOnly === 'boolean' ? opts.encodeValuesOnly : defaults.encodeValuesOnly,
            filter: filter,
            formatter: formatter,
            serializeDate: typeof opts.serializeDate === 'function' ? opts.serializeDate : defaults.serializeDate,
            skipNulls: typeof opts.skipNulls === 'boolean' ? opts.skipNulls : defaults.skipNulls,
            sort: typeof opts.sort === 'function' ? opts.sort : null,
            strictNullHandling: typeof opts.strictNullHandling === 'boolean' ? opts.strictNullHandling : defaults.strictNullHandling
        };
    };
    module.exports = function (object, opts) {
        var obj = object;
        var options = normalizeStringifyOptions(opts);
        var objKeys;
        var filter;
        if (typeof options.filter === 'function') {
            filter = options.filter;
            obj = filter('', obj);
        }
        else if (isArray(options.filter)) {
            filter = options.filter;
            objKeys = filter;
        }
        var keys = [];
        if (typeof obj !== 'object' || obj === null) {
            return '';
        }
        var arrayFormat;
        if (opts && opts.arrayFormat in arrayPrefixGenerators) {
            arrayFormat = opts.arrayFormat;
        }
        else if (opts && 'indices' in opts) {
            arrayFormat = opts.indices ? 'indices' : 'repeat';
        }
        else {
            arrayFormat = 'indices';
        }
        var generateArrayPrefix = arrayPrefixGenerators[arrayFormat];
        if (!objKeys) {
            objKeys = Object.keys(obj);
        }
        if (options.sort) {
            objKeys.sort(options.sort);
        }
        for (var i = 0; i < objKeys.length; ++i) {
            var key = objKeys[i];
            if (options.skipNulls && obj[key] === null) {
                continue;
            }
            pushToArray(keys, stringify(obj[key], key, generateArrayPrefix, options.strictNullHandling, options.skipNulls, options.encode ? options.encoder : null, options.filter, options.sort, options.allowDots, options.serializeDate, options.formatter, options.encodeValuesOnly, options.charset));
        }
        var joined = keys.join(options.delimiter);
        var prefix = options.addQueryPrefix === true ? '?' : '';
        if (options.charsetSentinel) {
            if (options.charset === 'iso-8859-1') {
                // encodeURIComponent('&#10003;'), the "numeric entity" representation of a checkmark
                prefix += 'utf8=%26%2310003%3B&';
            }
            else {
                // encodeURIComponent('✓')
                prefix += 'utf8=%E2%9C%93&';
            }
        }
        return joined.length > 0 ? prefix + joined : '';
    };
},
/* qs/lib/utils.js */ function _(require, module, exports) {
    var has = Object.prototype.hasOwnProperty;
    var isArray = Array.isArray;
    var hexTable = (function () {
        var array = [];
        for (var i = 0; i < 256; ++i) {
            array.push('%' + ((i < 16 ? '0' : '') + i.toString(16)).toUpperCase());
        }
        return array;
    }());
    var compactQueue = function compactQueue(queue) {
        while (queue.length > 1) {
            var item = queue.pop();
            var obj = item.obj[item.prop];
            if (isArray(obj)) {
                var compacted = [];
                for (var j = 0; j < obj.length; ++j) {
                    if (typeof obj[j] !== 'undefined') {
                        compacted.push(obj[j]);
                    }
                }
                item.obj[item.prop] = compacted;
            }
        }
    };
    var arrayToObject = function arrayToObject(source, options) {
        var obj = options && options.plainObjects ? Object.create(null) : {};
        for (var i = 0; i < source.length; ++i) {
            if (typeof source[i] !== 'undefined') {
                obj[i] = source[i];
            }
        }
        return obj;
    };
    var merge = function merge(target, source, options) {
        if (!source) {
            return target;
        }
        if (typeof source !== 'object') {
            if (isArray(target)) {
                target.push(source);
            }
            else if (target && typeof target === 'object') {
                if ((options && (options.plainObjects || options.allowPrototypes)) || !has.call(Object.prototype, source)) {
                    target[source] = true;
                }
            }
            else {
                return [target, source];
            }
            return target;
        }
        if (!target || typeof target !== 'object') {
            return [target].concat(source);
        }
        var mergeTarget = target;
        if (isArray(target) && !isArray(source)) {
            mergeTarget = arrayToObject(target, options);
        }
        if (isArray(target) && isArray(source)) {
            source.forEach(function (item, i) {
                if (has.call(target, i)) {
                    var targetItem = target[i];
                    if (targetItem && typeof targetItem === 'object' && item && typeof item === 'object') {
                        target[i] = merge(targetItem, item, options);
                    }
                    else {
                        target.push(item);
                    }
                }
                else {
                    target[i] = item;
                }
            });
            return target;
        }
        return Object.keys(source).reduce(function (acc, key) {
            var value = source[key];
            if (has.call(acc, key)) {
                acc[key] = merge(acc[key], value, options);
            }
            else {
                acc[key] = value;
            }
            return acc;
        }, mergeTarget);
    };
    var assign = function assignSingleSource(target, source) {
        return Object.keys(source).reduce(function (acc, key) {
            acc[key] = source[key];
            return acc;
        }, target);
    };
    var decode = function (str, decoder, charset) {
        var strWithoutPlus = str.replace(/\+/g, ' ');
        if (charset === 'iso-8859-1') {
            // unescape never throws, no try...catch needed:
            return strWithoutPlus.replace(/%[0-9a-f]{2}/gi, unescape);
        }
        // utf-8
        try {
            return decodeURIComponent(strWithoutPlus);
        }
        catch (e) {
            return strWithoutPlus;
        }
    };
    var encode = function encode(str, defaultEncoder, charset) {
        // This code was originally written by Brian White (mscdex) for the io.js core querystring library.
        // It has been adapted here for stricter adherence to RFC 3986
        if (str.length === 0) {
            return str;
        }
        var string = typeof str === 'string' ? str : String(str);
        if (charset === 'iso-8859-1') {
            return escape(string).replace(/%u[0-9a-f]{4}/gi, function ($0) {
                return '%26%23' + parseInt($0.slice(2), 16) + '%3B';
            });
        }
        var out = '';
        for (var i = 0; i < string.length; ++i) {
            var c = string.charCodeAt(i);
            if (c === 0x2D // -
                || c === 0x2E // .
                || c === 0x5F // _
                || c === 0x7E // ~
                || (c >= 0x30 && c <= 0x39) // 0-9
                || (c >= 0x41 && c <= 0x5A) // a-z
                || (c >= 0x61 && c <= 0x7A) // A-Z
            ) {
                out += string.charAt(i);
                continue;
            }
            if (c < 0x80) {
                out = out + hexTable[c];
                continue;
            }
            if (c < 0x800) {
                out = out + (hexTable[0xC0 | (c >> 6)] + hexTable[0x80 | (c & 0x3F)]);
                continue;
            }
            if (c < 0xD800 || c >= 0xE000) {
                out = out + (hexTable[0xE0 | (c >> 12)] + hexTable[0x80 | ((c >> 6) & 0x3F)] + hexTable[0x80 | (c & 0x3F)]);
                continue;
            }
            i += 1;
            c = 0x10000 + (((c & 0x3FF) << 10) | (string.charCodeAt(i) & 0x3FF));
            out += hexTable[0xF0 | (c >> 18)]
                + hexTable[0x80 | ((c >> 12) & 0x3F)]
                + hexTable[0x80 | ((c >> 6) & 0x3F)]
                + hexTable[0x80 | (c & 0x3F)];
        }
        return out;
    };
    var compact = function compact(value) {
        var queue = [{ obj: { o: value }, prop: 'o' }];
        var refs = [];
        for (var i = 0; i < queue.length; ++i) {
            var item = queue[i];
            var obj = item.obj[item.prop];
            var keys = Object.keys(obj);
            for (var j = 0; j < keys.length; ++j) {
                var key = keys[j];
                var val = obj[key];
                if (typeof val === 'object' && val !== null && refs.indexOf(val) === -1) {
                    queue.push({ obj: obj, prop: key });
                    refs.push(val);
                }
            }
        }
        compactQueue(queue);
        return value;
    };
    var isRegExp = function isRegExp(obj) {
        return Object.prototype.toString.call(obj) === '[object RegExp]';
    };
    var isBuffer = function isBuffer(obj) {
        if (!obj || typeof obj !== 'object') {
            return false;
        }
        return !!(obj.constructor && obj.constructor.isBuffer && obj.constructor.isBuffer(obj));
    };
    var combine = function combine(a, b) {
        return [].concat(a, b);
    };
    module.exports = {
        arrayToObject: arrayToObject,
        assign: assign,
        combine: combine,
        compact: compact,
        decode: decode,
        encode: encode,
        isBuffer: isBuffer,
        isRegExp: isRegExp,
        merge: merge
    };
},
/* qs/lib/formats.js */ function _(require, module, exports) {
    var replace = String.prototype.replace;
    var percentTwenties = /%20/g;
    module.exports = {
        'default': 'RFC3986',
        formatters: {
            RFC1738: function (value) {
                return replace.call(value, percentTwenties, '+');
            },
            RFC3986: function (value) {
                return value;
            }
        },
        RFC1738: 'RFC1738',
        RFC3986: 'RFC3986'
    };
},
/* qs/lib/parse.js */ function _(require, module, exports) {
    var utils = require(925) /* ./utils */;
    var has = Object.prototype.hasOwnProperty;
    var defaults = {
        allowDots: false,
        allowPrototypes: false,
        arrayLimit: 20,
        charset: 'utf-8',
        charsetSentinel: false,
        comma: false,
        decoder: utils.decode,
        delimiter: '&',
        depth: 5,
        ignoreQueryPrefix: false,
        interpretNumericEntities: false,
        parameterLimit: 1000,
        parseArrays: true,
        plainObjects: false,
        strictNullHandling: false
    };
    var interpretNumericEntities = function (str) {
        return str.replace(/&#(\d+);/g, function ($0, numberStr) {
            return String.fromCharCode(parseInt(numberStr, 10));
        });
    };
    // This is what browsers will submit when the ✓ character occurs in an
    // application/x-www-form-urlencoded body and the encoding of the page containing
    // the form is iso-8859-1, or when the submitted form has an accept-charset
    // attribute of iso-8859-1. Presumably also with other charsets that do not contain
    // the ✓ character, such as us-ascii.
    var isoSentinel = 'utf8=%26%2310003%3B'; // encodeURIComponent('&#10003;')
    // These are the percent-encoded utf-8 octets representing a checkmark, indicating that the request actually is utf-8 encoded.
    var charsetSentinel = 'utf8=%E2%9C%93'; // encodeURIComponent('✓')
    var parseValues = function parseQueryStringValues(str, options) {
        var obj = {};
        var cleanStr = options.ignoreQueryPrefix ? str.replace(/^\?/, '') : str;
        var limit = options.parameterLimit === Infinity ? undefined : options.parameterLimit;
        var parts = cleanStr.split(options.delimiter, limit);
        var skipIndex = -1; // Keep track of where the utf8 sentinel was found
        var i;
        var charset = options.charset;
        if (options.charsetSentinel) {
            for (i = 0; i < parts.length; ++i) {
                if (parts[i].indexOf('utf8=') === 0) {
                    if (parts[i] === charsetSentinel) {
                        charset = 'utf-8';
                    }
                    else if (parts[i] === isoSentinel) {
                        charset = 'iso-8859-1';
                    }
                    skipIndex = i;
                    i = parts.length; // The eslint settings do not allow break;
                }
            }
        }
        for (i = 0; i < parts.length; ++i) {
            if (i === skipIndex) {
                continue;
            }
            var part = parts[i];
            var bracketEqualsPos = part.indexOf(']=');
            var pos = bracketEqualsPos === -1 ? part.indexOf('=') : bracketEqualsPos + 1;
            var key, val;
            if (pos === -1) {
                key = options.decoder(part, defaults.decoder, charset);
                val = options.strictNullHandling ? null : '';
            }
            else {
                key = options.decoder(part.slice(0, pos), defaults.decoder, charset);
                val = options.decoder(part.slice(pos + 1), defaults.decoder, charset);
            }
            if (val && options.interpretNumericEntities && charset === 'iso-8859-1') {
                val = interpretNumericEntities(val);
            }
            if (val && options.comma && val.indexOf(',') > -1) {
                val = val.split(',');
            }
            if (has.call(obj, key)) {
                obj[key] = utils.combine(obj[key], val);
            }
            else {
                obj[key] = val;
            }
        }
        return obj;
    };
    var parseObject = function (chain, val, options) {
        var leaf = val;
        for (var i = chain.length - 1; i >= 0; --i) {
            var obj;
            var root = chain[i];
            if (root === '[]' && options.parseArrays) {
                obj = [].concat(leaf);
            }
            else {
                obj = options.plainObjects ? Object.create(null) : {};
                var cleanRoot = root.charAt(0) === '[' && root.charAt(root.length - 1) === ']' ? root.slice(1, -1) : root;
                var index = parseInt(cleanRoot, 10);
                if (!options.parseArrays && cleanRoot === '') {
                    obj = { 0: leaf };
                }
                else if (!isNaN(index)
                    && root !== cleanRoot
                    && String(index) === cleanRoot
                    && index >= 0
                    && (options.parseArrays && index <= options.arrayLimit)) {
                    obj = [];
                    obj[index] = leaf;
                }
                else {
                    obj[cleanRoot] = leaf;
                }
            }
            leaf = obj;
        }
        return leaf;
    };
    var parseKeys = function parseQueryStringKeys(givenKey, val, options) {
        if (!givenKey) {
            return;
        }
        // Transform dot notation to bracket notation
        var key = options.allowDots ? givenKey.replace(/\.([^.[]+)/g, '[$1]') : givenKey;
        // The regex chunks
        var brackets = /(\[[^[\]]*])/;
        var child = /(\[[^[\]]*])/g;
        // Get the parent
        var segment = brackets.exec(key);
        var parent = segment ? key.slice(0, segment.index) : key;
        // Stash the parent if it exists
        var keys = [];
        if (parent) {
            // If we aren't using plain objects, optionally prefix keys that would overwrite object prototype properties
            if (!options.plainObjects && has.call(Object.prototype, parent)) {
                if (!options.allowPrototypes) {
                    return;
                }
            }
            keys.push(parent);
        }
        // Loop through children appending to the array until we hit depth
        var i = 0;
        while ((segment = child.exec(key)) !== null && i < options.depth) {
            i += 1;
            if (!options.plainObjects && has.call(Object.prototype, segment[1].slice(1, -1))) {
                if (!options.allowPrototypes) {
                    return;
                }
            }
            keys.push(segment[1]);
        }
        // If there's a remainder, just add whatever is left
        if (segment) {
            keys.push('[' + key.slice(segment.index) + ']');
        }
        return parseObject(keys, val, options);
    };
    var normalizeParseOptions = function normalizeParseOptions(opts) {
        if (!opts) {
            return defaults;
        }
        if (opts.decoder !== null && opts.decoder !== undefined && typeof opts.decoder !== 'function') {
            throw new TypeError('Decoder has to be a function.');
        }
        if (typeof opts.charset !== 'undefined' && opts.charset !== 'utf-8' && opts.charset !== 'iso-8859-1') {
            throw new Error('The charset option must be either utf-8, iso-8859-1, or undefined');
        }
        var charset = typeof opts.charset === 'undefined' ? defaults.charset : opts.charset;
        return {
            allowDots: typeof opts.allowDots === 'undefined' ? defaults.allowDots : !!opts.allowDots,
            allowPrototypes: typeof opts.allowPrototypes === 'boolean' ? opts.allowPrototypes : defaults.allowPrototypes,
            arrayLimit: typeof opts.arrayLimit === 'number' ? opts.arrayLimit : defaults.arrayLimit,
            charset: charset,
            charsetSentinel: typeof opts.charsetSentinel === 'boolean' ? opts.charsetSentinel : defaults.charsetSentinel,
            comma: typeof opts.comma === 'boolean' ? opts.comma : defaults.comma,
            decoder: typeof opts.decoder === 'function' ? opts.decoder : defaults.decoder,
            delimiter: typeof opts.delimiter === 'string' || utils.isRegExp(opts.delimiter) ? opts.delimiter : defaults.delimiter,
            depth: typeof opts.depth === 'number' ? opts.depth : defaults.depth,
            ignoreQueryPrefix: opts.ignoreQueryPrefix === true,
            interpretNumericEntities: typeof opts.interpretNumericEntities === 'boolean' ? opts.interpretNumericEntities : defaults.interpretNumericEntities,
            parameterLimit: typeof opts.parameterLimit === 'number' ? opts.parameterLimit : defaults.parameterLimit,
            parseArrays: opts.parseArrays !== false,
            plainObjects: typeof opts.plainObjects === 'boolean' ? opts.plainObjects : defaults.plainObjects,
            strictNullHandling: typeof opts.strictNullHandling === 'boolean' ? opts.strictNullHandling : defaults.strictNullHandling
        };
    };
    module.exports = function (str, opts) {
        var options = normalizeParseOptions(opts);
        if (str === '' || str === null || typeof str === 'undefined') {
            return options.plainObjects ? Object.create(null) : {};
        }
        var tempObj = typeof str === 'string' ? parseValues(str, options) : str;
        var obj = options.plainObjects ? Object.create(null) : {};
        // Iterate over the keys and setup the new object
        var keys = Object.keys(tempObj);
        for (var i = 0; i < keys.length; ++i) {
            var key = keys[i];
            var newObj = parseKeys(key, tempObj[key], options);
            obj = utils.merge(obj, newObj, options);
        }
        return utils.compact(obj);
    };
},
/* request/lib/har.js */ function _(require, module, exports) {
    var fs = require('fs');
    var qs = require('querystring');
    var validate = require(929) /* har-validator */;
    var extend = require(834) /* extend */;
    function Har(request) {
        this.request = request;
    }
    Har.prototype.reducer = function (obj, pair) {
        // new property ?
        if (obj[pair.name] === undefined) {
            obj[pair.name] = pair.value;
            return obj;
        }
        // existing? convert to array
        var arr = [
            obj[pair.name],
            pair.value
        ];
        obj[pair.name] = arr;
        return obj;
    };
    Har.prototype.prep = function (data) {
        // construct utility properties
        data.queryObj = {};
        data.headersObj = {};
        data.postData.jsonObj = false;
        data.postData.paramsObj = false;
        // construct query objects
        if (data.queryString && data.queryString.length) {
            data.queryObj = data.queryString.reduce(this.reducer, {});
        }
        // construct headers objects
        if (data.headers && data.headers.length) {
            // loweCase header keys
            data.headersObj = data.headers.reduceRight(function (headers, header) {
                headers[header.name] = header.value;
                return headers;
            }, {});
        }
        // construct Cookie header
        if (data.cookies && data.cookies.length) {
            var cookies = data.cookies.map(function (cookie) {
                return cookie.name + '=' + cookie.value;
            });
            if (cookies.length) {
                data.headersObj.cookie = cookies.join('; ');
            }
        }
        // prep body
        function some(arr) {
            return arr.some(function (type) {
                return data.postData.mimeType.indexOf(type) === 0;
            });
        }
        if (some([
            'multipart/mixed',
            'multipart/related',
            'multipart/form-data',
            'multipart/alternative'
        ])) {
            // reset values
            data.postData.mimeType = 'multipart/form-data';
        }
        else if (some([
            'application/x-www-form-urlencoded'
        ])) {
            if (!data.postData.params) {
                data.postData.text = '';
            }
            else {
                data.postData.paramsObj = data.postData.params.reduce(this.reducer, {});
                // always overwrite
                data.postData.text = qs.stringify(data.postData.paramsObj);
            }
        }
        else if (some([
            'text/json',
            'text/x-json',
            'application/json',
            'application/x-json'
        ])) {
            data.postData.mimeType = 'application/json';
            if (data.postData.text) {
                try {
                    data.postData.jsonObj = JSON.parse(data.postData.text);
                }
                catch (e) {
                    this.request.debug(e);
                    // force back to text/plain
                    data.postData.mimeType = 'text/plain';
                }
            }
        }
        return data;
    };
    Har.prototype.options = function (options) {
        // skip if no har property defined
        if (!options.har) {
            return options;
        }
        var har = {};
        extend(har, options.har);
        // only process the first entry
        if (har.log && har.log.entries) {
            har = har.log.entries[0];
        }
        // add optional properties to make validation successful
        har.url = har.url || options.url || options.uri || options.baseUrl || '/';
        har.httpVersion = har.httpVersion || 'HTTP/1.1';
        har.queryString = har.queryString || [];
        har.headers = har.headers || [];
        har.cookies = har.cookies || [];
        har.postData = har.postData || {};
        har.postData.mimeType = har.postData.mimeType || 'application/octet-stream';
        har.bodySize = 0;
        har.headersSize = 0;
        har.postData.size = 0;
        if (!validate.request(har)) {
            return options;
        }
        // clean up and get some utility properties
        var req = this.prep(har);
        // construct new options
        if (req.url) {
            options.url = req.url;
        }
        if (req.method) {
            options.method = req.method;
        }
        if (Object.keys(req.queryObj).length) {
            options.qs = req.queryObj;
        }
        if (Object.keys(req.headersObj).length) {
            options.headers = req.headersObj;
        }
        function test(type) {
            return req.postData.mimeType.indexOf(type) === 0;
        }
        if (test('application/x-www-form-urlencoded')) {
            options.form = req.postData.paramsObj;
        }
        else if (test('application/json')) {
            if (req.postData.jsonObj) {
                options.body = req.postData.jsonObj;
                options.json = true;
            }
        }
        else if (test('multipart/form-data')) {
            options.formData = {};
            req.postData.params.forEach(function (param) {
                var attachment = {};
                if (!param.fileName && !param.contentType) {
                    options.formData[param.name] = param.value;
                    return;
                }
                // attempt to read from disk!
                if (param.fileName && !param.value) {
                    attachment.value = fs.createReadStream(param.fileName);
                }
                else if (param.value) {
                    attachment.value = param.value;
                }
                if (param.fileName) {
                    attachment.options = {
                        filename: param.fileName,
                        contentType: param.contentType ? param.contentType : null
                    };
                }
                options.formData[param.name] = attachment;
            });
        }
        else {
            if (req.postData.text) {
                options.body = req.postData.text;
            }
        }
        return options;
    };
    exports.Har = Har;
},
/* har-validator/lib/promise.js */ function _(require, module, exports) {
    var Ajv = require(424) /* ajv */;
    var HARError = require(930) /* ./error */;
    var schemas = require(931) /* har-schema */;
    var ajv;
    function createAjvInstance() {
        var ajv = new Ajv({
            allErrors: true
        });
        ajv.addMetaSchema(require(950) /* ajv/lib/refs/json-schema-draft-06.json */);
        ajv.addSchema(schemas);
        return ajv;
    }
    function validate(name, data) {
        data = data || {};
        // validator config
        ajv = ajv || createAjvInstance();
        var validate = ajv.getSchema(name + '.json');
        return new Promise(function (resolve, reject) {
            var valid = validate(data);
            !valid ? reject(new HARError(validate.errors)) : resolve(data);
        });
    }
    exports.afterRequest = function (data) {
        return validate('afterRequest', data);
    };
    exports.beforeRequest = function (data) {
        return validate('beforeRequest', data);
    };
    exports.browser = function (data) {
        return validate('browser', data);
    };
    exports.cache = function (data) {
        return validate('cache', data);
    };
    exports.content = function (data) {
        return validate('content', data);
    };
    exports.cookie = function (data) {
        return validate('cookie', data);
    };
    exports.creator = function (data) {
        return validate('creator', data);
    };
    exports.entry = function (data) {
        return validate('entry', data);
    };
    exports.har = function (data) {
        return validate('har', data);
    };
    exports.header = function (data) {
        return validate('header', data);
    };
    exports.log = function (data) {
        return validate('log', data);
    };
    exports.page = function (data) {
        return validate('page', data);
    };
    exports.pageTimings = function (data) {
        return validate('pageTimings', data);
    };
    exports.postData = function (data) {
        return validate('postData', data);
    };
    exports.query = function (data) {
        return validate('query', data);
    };
    exports.request = function (data) {
        return validate('request', data);
    };
    exports.response = function (data) {
        return validate('response', data);
    };
    exports.timings = function (data) {
        return validate('timings', data);
    };
},
/* har-validator/lib/error.js */ function _(require, module, exports) {
    function HARError(errors) {
        var message = 'validation failed';
        this.name = 'HARError';
        this.message = message;
        this.errors = errors;
        if (typeof Error.captureStackTrace === 'function') {
            Error.captureStackTrace(this, this.constructor);
        }
        else {
            this.stack = (new Error(message)).stack;
        }
    }
    HARError.prototype = Error.prototype;
    module.exports = HARError;
},
/* har-schema/lib/index.js */ function _(require, module, exports) {
    module.exports = {
        afterRequest: require(932) /* ./afterRequest.json */,
        beforeRequest: require(933) /* ./beforeRequest.json */,
        browser: require(934) /* ./browser.json */,
        cache: require(935) /* ./cache.json */,
        content: require(936) /* ./content.json */,
        cookie: require(937) /* ./cookie.json */,
        creator: require(938) /* ./creator.json */,
        entry: require(939) /* ./entry.json */,
        har: require(940) /* ./har.json */,
        header: require(941) /* ./header.json */,
        log: require(942) /* ./log.json */,
        page: require(943) /* ./page.json */,
        pageTimings: require(944) /* ./pageTimings.json */,
        postData: require(945) /* ./postData.json */,
        query: require(946) /* ./query.json */,
        request: require(947) /* ./request.json */,
        response: require(948) /* ./response.json */,
        timings: require(949) /* ./timings.json */
    };
},
/* har-schema/lib/afterRequest.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "afterRequest.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "optional": true,
        "required": [
            "lastAccess",
            "eTag",
            "hitCount"
        ],
        "properties": {
            "expires": {
                "type": "string",
                "pattern": "^(\\d{4})(-)?(\\d\\d)(-)?(\\d\\d)(T)?(\\d\\d)(:)?(\\d\\d)(:)?(\\d\\d)(\\.\\d+)?(Z|([+-])(\\d\\d)(:)?(\\d\\d))?"
            },
            "lastAccess": {
                "type": "string",
                "pattern": "^(\\d{4})(-)?(\\d\\d)(-)?(\\d\\d)(T)?(\\d\\d)(:)?(\\d\\d)(:)?(\\d\\d)(\\.\\d+)?(Z|([+-])(\\d\\d)(:)?(\\d\\d))?"
            },
            "eTag": {
                "type": "string"
            },
            "hitCount": {
                "type": "integer"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/beforeRequest.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "beforeRequest.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "optional": true,
        "required": [
            "lastAccess",
            "eTag",
            "hitCount"
        ],
        "properties": {
            "expires": {
                "type": "string",
                "pattern": "^(\\d{4})(-)?(\\d\\d)(-)?(\\d\\d)(T)?(\\d\\d)(:)?(\\d\\d)(:)?(\\d\\d)(\\.\\d+)?(Z|([+-])(\\d\\d)(:)?(\\d\\d))?"
            },
            "lastAccess": {
                "type": "string",
                "pattern": "^(\\d{4})(-)?(\\d\\d)(-)?(\\d\\d)(T)?(\\d\\d)(:)?(\\d\\d)(:)?(\\d\\d)(\\.\\d+)?(Z|([+-])(\\d\\d)(:)?(\\d\\d))?"
            },
            "eTag": {
                "type": "string"
            },
            "hitCount": {
                "type": "integer"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/browser.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "browser.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "required": [
            "name",
            "version"
        ],
        "properties": {
            "name": {
                "type": "string"
            },
            "version": {
                "type": "string"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/cache.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "cache.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "properties": {
            "beforeRequest": {
                "oneOf": [
                    { "type": "null" },
                    { "$ref": "beforeRequest.json#" }
                ]
            },
            "afterRequest": {
                "oneOf": [
                    { "type": "null" },
                    { "$ref": "afterRequest.json#" }
                ]
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/content.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "content.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "required": [
            "size",
            "mimeType"
        ],
        "properties": {
            "size": {
                "type": "integer"
            },
            "compression": {
                "type": "integer"
            },
            "mimeType": {
                "type": "string"
            },
            "text": {
                "type": "string"
            },
            "encoding": {
                "type": "string"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/cookie.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "cookie.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "required": [
            "name",
            "value"
        ],
        "properties": {
            "name": {
                "type": "string"
            },
            "value": {
                "type": "string"
            },
            "path": {
                "type": "string"
            },
            "domain": {
                "type": "string"
            },
            "expires": {
                "type": ["string", "null"],
                "format": "date-time"
            },
            "httpOnly": {
                "type": "boolean"
            },
            "secure": {
                "type": "boolean"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/creator.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "creator.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "required": [
            "name",
            "version"
        ],
        "properties": {
            "name": {
                "type": "string"
            },
            "version": {
                "type": "string"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/entry.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "entry.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "optional": true,
        "required": [
            "startedDateTime",
            "time",
            "request",
            "response",
            "cache",
            "timings"
        ],
        "properties": {
            "pageref": {
                "type": "string"
            },
            "startedDateTime": {
                "type": "string",
                "format": "date-time",
                "pattern": "^(\\d{4})(-)?(\\d\\d)(-)?(\\d\\d)(T)?(\\d\\d)(:)?(\\d\\d)(:)?(\\d\\d)(\\.\\d+)?(Z|([+-])(\\d\\d)(:)?(\\d\\d))"
            },
            "time": {
                "type": "number",
                "min": 0
            },
            "request": {
                "$ref": "request.json#"
            },
            "response": {
                "$ref": "response.json#"
            },
            "cache": {
                "$ref": "cache.json#"
            },
            "timings": {
                "$ref": "timings.json#"
            },
            "serverIPAddress": {
                "type": "string",
                "oneOf": [
                    { "format": "ipv4" },
                    { "format": "ipv6" }
                ]
            },
            "connection": {
                "type": "string"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/har.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "har.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "required": [
            "log"
        ],
        "properties": {
            "log": {
                "$ref": "log.json#"
            }
        }
    };
},
/* har-schema/lib/header.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "header.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "required": [
            "name",
            "value"
        ],
        "properties": {
            "name": {
                "type": "string"
            },
            "value": {
                "type": "string"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/log.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "log.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "required": [
            "version",
            "creator",
            "entries"
        ],
        "properties": {
            "version": {
                "type": "string"
            },
            "creator": {
                "$ref": "creator.json#"
            },
            "browser": {
                "$ref": "browser.json#"
            },
            "pages": {
                "type": "array",
                "items": {
                    "$ref": "page.json#"
                }
            },
            "entries": {
                "type": "array",
                "items": {
                    "$ref": "entry.json#"
                }
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/page.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "page.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "optional": true,
        "required": [
            "startedDateTime",
            "id",
            "title",
            "pageTimings"
        ],
        "properties": {
            "startedDateTime": {
                "type": "string",
                "format": "date-time",
                "pattern": "^(\\d{4})(-)?(\\d\\d)(-)?(\\d\\d)(T)?(\\d\\d)(:)?(\\d\\d)(:)?(\\d\\d)(\\.\\d+)?(Z|([+-])(\\d\\d)(:)?(\\d\\d))"
            },
            "id": {
                "type": "string",
                "unique": true
            },
            "title": {
                "type": "string"
            },
            "pageTimings": {
                "$ref": "pageTimings.json#"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/pageTimings.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "pageTimings.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "properties": {
            "onContentLoad": {
                "type": "number",
                "min": -1
            },
            "onLoad": {
                "type": "number",
                "min": -1
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/postData.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "postData.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "optional": true,
        "required": [
            "mimeType"
        ],
        "properties": {
            "mimeType": {
                "type": "string"
            },
            "text": {
                "type": "string"
            },
            "params": {
                "type": "array",
                "required": [
                    "name"
                ],
                "properties": {
                    "name": {
                        "type": "string"
                    },
                    "value": {
                        "type": "string"
                    },
                    "fileName": {
                        "type": "string"
                    },
                    "contentType": {
                        "type": "string"
                    },
                    "comment": {
                        "type": "string"
                    }
                }
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/query.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "query.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "required": [
            "name",
            "value"
        ],
        "properties": {
            "name": {
                "type": "string"
            },
            "value": {
                "type": "string"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/request.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "request.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "required": [
            "method",
            "url",
            "httpVersion",
            "cookies",
            "headers",
            "queryString",
            "headersSize",
            "bodySize"
        ],
        "properties": {
            "method": {
                "type": "string"
            },
            "url": {
                "type": "string",
                "format": "uri"
            },
            "httpVersion": {
                "type": "string"
            },
            "cookies": {
                "type": "array",
                "items": {
                    "$ref": "cookie.json#"
                }
            },
            "headers": {
                "type": "array",
                "items": {
                    "$ref": "header.json#"
                }
            },
            "queryString": {
                "type": "array",
                "items": {
                    "$ref": "query.json#"
                }
            },
            "postData": {
                "$ref": "postData.json#"
            },
            "headersSize": {
                "type": "integer"
            },
            "bodySize": {
                "type": "integer"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/response.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "response.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "type": "object",
        "required": [
            "status",
            "statusText",
            "httpVersion",
            "cookies",
            "headers",
            "content",
            "redirectURL",
            "headersSize",
            "bodySize"
        ],
        "properties": {
            "status": {
                "type": "integer"
            },
            "statusText": {
                "type": "string"
            },
            "httpVersion": {
                "type": "string"
            },
            "cookies": {
                "type": "array",
                "items": {
                    "$ref": "cookie.json#"
                }
            },
            "headers": {
                "type": "array",
                "items": {
                    "$ref": "header.json#"
                }
            },
            "content": {
                "$ref": "content.json#"
            },
            "redirectURL": {
                "type": "string"
            },
            "headersSize": {
                "type": "integer"
            },
            "bodySize": {
                "type": "integer"
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* har-schema/lib/timings.json */ function _(require, module, exports) {
    module.exports = {
        "$id": "timings.json#",
        "$schema": "http://json-schema.org/draft-06/schema#",
        "required": [
            "send",
            "wait",
            "receive"
        ],
        "properties": {
            "dns": {
                "type": "number",
                "min": -1
            },
            "connect": {
                "type": "number",
                "min": -1
            },
            "blocked": {
                "type": "number",
                "min": -1
            },
            "send": {
                "type": "number",
                "min": -1
            },
            "wait": {
                "type": "number",
                "min": -1
            },
            "receive": {
                "type": "number",
                "min": -1
            },
            "ssl": {
                "type": "number",
                "min": -1
            },
            "comment": {
                "type": "string"
            }
        }
    };
},
/* ajv/lib/refs/json-schema-draft-06.json */ function _(require, module, exports) {
    module.exports = {
        "$schema": "http://json-schema.org/draft-06/schema#",
        "$id": "http://json-schema.org/draft-06/schema#",
        "title": "Core schema meta-schema",
        "definitions": {
            "schemaArray": {
                "type": "array",
                "minItems": 1,
                "items": { "$ref": "#" }
            },
            "nonNegativeInteger": {
                "type": "integer",
                "minimum": 0
            },
            "nonNegativeIntegerDefault0": {
                "allOf": [
                    { "$ref": "#/definitions/nonNegativeInteger" },
                    { "default": 0 }
                ]
            },
            "simpleTypes": {
                "enum": [
                    "array",
                    "boolean",
                    "integer",
                    "null",
                    "number",
                    "object",
                    "string"
                ]
            },
            "stringArray": {
                "type": "array",
                "items": { "type": "string" },
                "uniqueItems": true,
                "default": []
            }
        },
        "type": ["object", "boolean"],
        "properties": {
            "$id": {
                "type": "string",
                "format": "uri-reference"
            },
            "$schema": {
                "type": "string",
                "format": "uri"
            },
            "$ref": {
                "type": "string",
                "format": "uri-reference"
            },
            "title": {
                "type": "string"
            },
            "description": {
                "type": "string"
            },
            "default": {},
            "examples": {
                "type": "array",
                "items": {}
            },
            "multipleOf": {
                "type": "number",
                "exclusiveMinimum": 0
            },
            "maximum": {
                "type": "number"
            },
            "exclusiveMaximum": {
                "type": "number"
            },
            "minimum": {
                "type": "number"
            },
            "exclusiveMinimum": {
                "type": "number"
            },
            "maxLength": { "$ref": "#/definitions/nonNegativeInteger" },
            "minLength": { "$ref": "#/definitions/nonNegativeIntegerDefault0" },
            "pattern": {
                "type": "string",
                "format": "regex"
            },
            "additionalItems": { "$ref": "#" },
            "items": {
                "anyOf": [
                    { "$ref": "#" },
                    { "$ref": "#/definitions/schemaArray" }
                ],
                "default": {}
            },
            "maxItems": { "$ref": "#/definitions/nonNegativeInteger" },
            "minItems": { "$ref": "#/definitions/nonNegativeIntegerDefault0" },
            "uniqueItems": {
                "type": "boolean",
                "default": false
            },
            "contains": { "$ref": "#" },
            "maxProperties": { "$ref": "#/definitions/nonNegativeInteger" },
            "minProperties": { "$ref": "#/definitions/nonNegativeIntegerDefault0" },
            "required": { "$ref": "#/definitions/stringArray" },
            "additionalProperties": { "$ref": "#" },
            "definitions": {
                "type": "object",
                "additionalProperties": { "$ref": "#" },
                "default": {}
            },
            "properties": {
                "type": "object",
                "additionalProperties": { "$ref": "#" },
                "default": {}
            },
            "patternProperties": {
                "type": "object",
                "additionalProperties": { "$ref": "#" },
                "default": {}
            },
            "dependencies": {
                "type": "object",
                "additionalProperties": {
                    "anyOf": [
                        { "$ref": "#" },
                        { "$ref": "#/definitions/stringArray" }
                    ]
                }
            },
            "propertyNames": { "$ref": "#" },
            "const": {},
            "enum": {
                "type": "array",
                "minItems": 1,
                "uniqueItems": true
            },
            "type": {
                "anyOf": [
                    { "$ref": "#/definitions/simpleTypes" },
                    {
                        "type": "array",
                        "items": { "$ref": "#/definitions/simpleTypes" },
                        "minItems": 1,
                        "uniqueItems": true
                    }
                ]
            },
            "format": { "type": "string" },
            "allOf": { "$ref": "#/definitions/schemaArray" },
            "anyOf": { "$ref": "#/definitions/schemaArray" },
            "oneOf": { "$ref": "#/definitions/schemaArray" },
            "not": { "$ref": "#" }
        },
        "default": {}
    };
},
/* request/lib/auth.js */ function _(require, module, exports) {
    var caseless = require(903) /* caseless */;
    var uuid = require(952) /* uuid/v4 */;
    var helpers = require(845) /* ./helpers */;
    var md5 = helpers.md5;
    var toBase64 = helpers.toBase64;
    function Auth(request) {
        // define all public properties here
        this.request = request;
        this.hasAuth = false;
        this.sentAuth = false;
        this.bearerToken = null;
        this.user = null;
        this.pass = null;
    }
    Auth.prototype.basic = function (user, pass, sendImmediately) {
        var self = this;
        if (typeof user !== 'string' || (pass !== undefined && typeof pass !== 'string')) {
            self.request.emit('error', new Error('auth() received invalid user or password'));
        }
        self.user = user;
        self.pass = pass;
        self.hasAuth = true;
        var header = user + ':' + (pass || '');
        if (sendImmediately || typeof sendImmediately === 'undefined') {
            var authHeader = 'Basic ' + toBase64(header);
            self.sentAuth = true;
            return authHeader;
        }
    };
    Auth.prototype.bearer = function (bearer, sendImmediately) {
        var self = this;
        self.bearerToken = bearer;
        self.hasAuth = true;
        if (sendImmediately || typeof sendImmediately === 'undefined') {
            if (typeof bearer === 'function') {
                bearer = bearer();
            }
            var authHeader = 'Bearer ' + (bearer || '');
            self.sentAuth = true;
            return authHeader;
        }
    };
    Auth.prototype.digest = function (method, path, authHeader) {
        // TODO: More complete implementation of RFC 2617.
        //   - handle challenge.domain
        //   - support qop="auth-int" only
        //   - handle Authentication-Info (not necessarily?)
        //   - check challenge.stale (not necessarily?)
        //   - increase nc (not necessarily?)
        // For reference:
        // http://tools.ietf.org/html/rfc2617#section-3
        // https://github.com/bagder/curl/blob/master/lib/http_digest.c
        var self = this;
        var challenge = {};
        var re = /([a-z0-9_-]+)=(?:"([^"]+)"|([a-z0-9_-]+))/gi;
        while (true) {
            var match = re.exec(authHeader);
            if (!match) {
                break;
            }
            challenge[match[1]] = match[2] || match[3];
        }
        /**
         * RFC 2617: handle both MD5 and MD5-sess algorithms.
         *
         * If the algorithm directive's value is "MD5" or unspecified, then HA1 is
         *   HA1=MD5(username:realm:password)
         * If the algorithm directive's value is "MD5-sess", then HA1 is
         *   HA1=MD5(MD5(username:realm:password):nonce:cnonce)
         */
        var ha1Compute = function (algorithm, user, realm, pass, nonce, cnonce) {
            var ha1 = md5(user + ':' + realm + ':' + pass);
            if (algorithm && algorithm.toLowerCase() === 'md5-sess') {
                return md5(ha1 + ':' + nonce + ':' + cnonce);
            }
            else {
                return ha1;
            }
        };
        var qop = /(^|,)\s*auth\s*($|,)/.test(challenge.qop) && 'auth';
        var nc = qop && '00000001';
        var cnonce = qop && uuid().replace(/-/g, '');
        var ha1 = ha1Compute(challenge.algorithm, self.user, challenge.realm, self.pass, challenge.nonce, cnonce);
        var ha2 = md5(method + ':' + path);
        var digestResponse = qop
            ? md5(ha1 + ':' + challenge.nonce + ':' + nc + ':' + cnonce + ':' + qop + ':' + ha2)
            : md5(ha1 + ':' + challenge.nonce + ':' + ha2);
        var authValues = {
            username: self.user,
            realm: challenge.realm,
            nonce: challenge.nonce,
            uri: path,
            qop: qop,
            response: digestResponse,
            nc: nc,
            cnonce: cnonce,
            algorithm: challenge.algorithm,
            opaque: challenge.opaque
        };
        authHeader = [];
        for (var k in authValues) {
            if (authValues[k]) {
                if (k === 'qop' || k === 'nc' || k === 'algorithm') {
                    authHeader.push(k + '=' + authValues[k]);
                }
                else {
                    authHeader.push(k + '="' + authValues[k] + '"');
                }
            }
        }
        authHeader = 'Digest ' + authHeader.join(', ');
        self.sentAuth = true;
        return authHeader;
    };
    Auth.prototype.onRequest = function (user, pass, sendImmediately, bearer) {
        var self = this;
        var request = self.request;
        var authHeader;
        if (bearer === undefined && user === undefined) {
            self.request.emit('error', new Error('no auth mechanism defined'));
        }
        else if (bearer !== undefined) {
            authHeader = self.bearer(bearer, sendImmediately);
        }
        else {
            authHeader = self.basic(user, pass, sendImmediately);
        }
        if (authHeader) {
            request.setHeader('authorization', authHeader);
        }
    };
    Auth.prototype.onResponse = function (response) {
        var self = this;
        var request = self.request;
        if (!self.hasAuth || self.sentAuth) {
            return null;
        }
        var c = caseless(response.headers);
        var authHeader = c.get('www-authenticate');
        var authVerb = authHeader && authHeader.split(' ')[0].toLowerCase();
        request.debug('reauth', authVerb);
        switch (authVerb) {
            case 'basic':
                return self.basic(self.user, self.pass, true);
            case 'bearer':
                return self.bearer(self.bearerToken, true);
            case 'digest':
                return self.digest(request.method, request.path, authHeader);
        }
    };
    exports.Auth = Auth;
},
/* uuid/v4.js */ function _(require, module, exports) {
    var rng = require(953) /* ./lib/rng */;
    var bytesToUuid = require(954) /* ./lib/bytesToUuid */;
    function v4(options, buf, offset) {
        var i = buf && offset || 0;
        if (typeof (options) == 'string') {
            buf = options === 'binary' ? new Array(16) : null;
            options = null;
        }
        options = options || {};
        var rnds = options.random || (options.rng || rng)();
        // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`
        rnds[6] = (rnds[6] & 0x0f) | 0x40;
        rnds[8] = (rnds[8] & 0x3f) | 0x80;
        // Copy bytes to buffer, if provided
        if (buf) {
            for (var ii = 0; ii < 16; ++ii) {
                buf[i + ii] = rnds[ii];
            }
        }
        return buf || bytesToUuid(rnds);
    }
    module.exports = v4;
},
/* uuid/lib/rng.js */ function _(require, module, exports) {
    // Unique ID creation requires a high quality random # generator.  In node.js
    // this is pretty straight-forward - we use the crypto API.
    var crypto = require('crypto');
    module.exports = function nodeRNG() {
        return crypto.randomBytes(16);
    };
},
/* uuid/lib/bytesToUuid.js */ function _(require, module, exports) {
    /**
     * Convert array of 16 byte values to UUID string format of the form:
     * XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
     */
    var byteToHex = [];
    for (var i = 0; i < 256; ++i) {
        byteToHex[i] = (i + 0x100).toString(16).substr(1);
    }
    function bytesToUuid(buf, offset) {
        var i = offset || 0;
        var bth = byteToHex;
        // join used to fix memory issue caused by concatenation: https://bugs.chromium.org/p/v8/issues/detail?id=3175#c4
        return ([
            bth[buf[i++]], bth[buf[i++]],
            bth[buf[i++]], bth[buf[i++]], '-',
            bth[buf[i++]], bth[buf[i++]], '-',
            bth[buf[i++]], bth[buf[i++]], '-',
            bth[buf[i++]], bth[buf[i++]], '-',
            bth[buf[i++]], bth[buf[i++]],
            bth[buf[i++]], bth[buf[i++]],
            bth[buf[i++]], bth[buf[i++]]
        ]).join('');
    }
    module.exports = bytesToUuid;
},
/* request/lib/oauth.js */ function _(require, module, exports) {
    var url = require('url');
    var qs = require(923) /* qs */;
    var caseless = require(903) /* caseless */;
    var uuid = require(952) /* uuid/v4 */;
    var oauth = require(956) /* oauth-sign */;
    var crypto = require('crypto');
    var Buffer = require(70) /* safe-buffer */.Buffer;
    function OAuth(request) {
        this.request = request;
        this.params = null;
    }
    OAuth.prototype.buildParams = function (_oauth, uri, method, query, form, qsLib) {
        var oa = {};
        for (var i in _oauth) {
            oa['oauth_' + i] = _oauth[i];
        }
        if (!oa.oauth_version) {
            oa.oauth_version = '1.0';
        }
        if (!oa.oauth_timestamp) {
            oa.oauth_timestamp = Math.floor(Date.now() / 1000).toString();
        }
        if (!oa.oauth_nonce) {
            oa.oauth_nonce = uuid().replace(/-/g, '');
        }
        if (!oa.oauth_signature_method) {
            oa.oauth_signature_method = 'HMAC-SHA1';
        }
        var consumer_secret_or_private_key = oa.oauth_consumer_secret || oa.oauth_private_key; // eslint-disable-line camelcase
        delete oa.oauth_consumer_secret;
        delete oa.oauth_private_key;
        var token_secret = oa.oauth_token_secret; // eslint-disable-line camelcase
        delete oa.oauth_token_secret;
        var realm = oa.oauth_realm;
        delete oa.oauth_realm;
        delete oa.oauth_transport_method;
        var baseurl = uri.protocol + '//' + uri.host + uri.pathname;
        var params = qsLib.parse([].concat(query, form, qsLib.stringify(oa)).join('&'));
        oa.oauth_signature = oauth.sign(oa.oauth_signature_method, method, baseurl, params, consumer_secret_or_private_key, // eslint-disable-line camelcase
        token_secret // eslint-disable-line camelcase
        );
        if (realm) {
            oa.realm = realm;
        }
        return oa;
    };
    OAuth.prototype.buildBodyHash = function (_oauth, body) {
        if (['HMAC-SHA1', 'RSA-SHA1'].indexOf(_oauth.signature_method || 'HMAC-SHA1') < 0) {
            this.request.emit('error', new Error('oauth: ' + _oauth.signature_method +
                ' signature_method not supported with body_hash signing.'));
        }
        var shasum = crypto.createHash('sha1');
        shasum.update(body || '');
        var sha1 = shasum.digest('hex');
        return Buffer.from(sha1, 'hex').toString('base64');
    };
    OAuth.prototype.concatParams = function (oa, sep, wrap) {
        wrap = wrap || '';
        var params = Object.keys(oa).filter(function (i) {
            return i !== 'realm' && i !== 'oauth_signature';
        }).sort();
        if (oa.realm) {
            params.splice(0, 0, 'realm');
        }
        params.push('oauth_signature');
        return params.map(function (i) {
            return i + '=' + wrap + oauth.rfc3986(oa[i]) + wrap;
        }).join(sep);
    };
    OAuth.prototype.onRequest = function (_oauth) {
        var self = this;
        self.params = _oauth;
        var uri = self.request.uri || {};
        var method = self.request.method || '';
        var headers = caseless(self.request.headers);
        var body = self.request.body || '';
        var qsLib = self.request.qsLib || qs;
        var form;
        var query;
        var contentType = headers.get('content-type') || '';
        var formContentType = 'application/x-www-form-urlencoded';
        var transport = _oauth.transport_method || 'header';
        if (contentType.slice(0, formContentType.length) === formContentType) {
            contentType = formContentType;
            form = body;
        }
        if (uri.query) {
            query = uri.query;
        }
        if (transport === 'body' && (method !== 'POST' || contentType !== formContentType)) {
            self.request.emit('error', new Error('oauth: transport_method of body requires POST ' +
                'and content-type ' + formContentType));
        }
        if (!form && typeof _oauth.body_hash === 'boolean') {
            _oauth.body_hash = self.buildBodyHash(_oauth, self.request.body.toString());
        }
        var oa = self.buildParams(_oauth, uri, method, query, form, qsLib);
        switch (transport) {
            case 'header':
                self.request.setHeader('Authorization', 'OAuth ' + self.concatParams(oa, ',', '"'));
                break;
            case 'query':
                var href = self.request.uri.href += (query ? '&' : '?') + self.concatParams(oa, '&');
                self.request.uri = url.parse(href);
                self.request.path = self.request.uri.path;
                break;
            case 'body':
                self.request.body = (form ? form + '&' : '') + self.concatParams(oa, '&');
                break;
            default:
                self.request.emit('error', new Error('oauth: transport_method invalid'));
        }
    };
    exports.OAuth = OAuth;
},
/* oauth-sign/index.js */ function _(require, module, exports) {
    var crypto = require('crypto');
    function sha(key, body, algorithm) {
        return crypto.createHmac(algorithm, key).update(body).digest('base64');
    }
    function rsa(key, body) {
        return crypto.createSign('RSA-SHA1').update(body).sign(key, 'base64');
    }
    function rfc3986(str) {
        return encodeURIComponent(str)
            .replace(/!/g, '%21')
            .replace(/\*/g, '%2A')
            .replace(/\(/g, '%28')
            .replace(/\)/g, '%29')
            .replace(/'/g, '%27');
    }
    // Maps object to bi-dimensional array
    // Converts { foo: 'A', bar: [ 'b', 'B' ]} to
    // [ ['foo', 'A'], ['bar', 'b'], ['bar', 'B'] ]
    function map(obj) {
        var key, val, arr = [];
        for (key in obj) {
            val = obj[key];
            if (Array.isArray(val))
                for (var i = 0; i < val.length; i++)
                    arr.push([key, val[i]]);
            else if (typeof val === 'object')
                for (var prop in val)
                    arr.push([key + '[' + prop + ']', val[prop]]);
            else
                arr.push([key, val]);
        }
        return arr;
    }
    // Compare function for sort
    function compare(a, b) {
        return a > b ? 1 : a < b ? -1 : 0;
    }
    function generateBase(httpMethod, base_uri, params) {
        // adapted from https://dev.twitter.com/docs/auth/oauth and 
        // https://dev.twitter.com/docs/auth/creating-signature
        // Parameter normalization
        // http://tools.ietf.org/html/rfc5849#section-3.4.1.3.2
        var normalized = map(params)
            // 1.  First, the name and value of each parameter are encoded
            .map(function (p) {
            return [rfc3986(p[0]), rfc3986(p[1] || '')];
        })
            // 2.  The parameters are sorted by name, using ascending byte value
            //     ordering.  If two or more parameters share the same name, they
            //     are sorted by their value.
            .sort(function (a, b) {
            return compare(a[0], b[0]) || compare(a[1], b[1]);
        })
            // 3.  The name of each parameter is concatenated to its corresponding
            //     value using an "=" character (ASCII code 61) as a separator, even
            //     if the value is empty.
            .map(function (p) { return p.join('='); })
            // 4.  The sorted name/value pairs are concatenated together into a
            //     single string by using an "&" character (ASCII code 38) as
            //     separator.
            .join('&');
        var base = [
            rfc3986(httpMethod ? httpMethod.toUpperCase() : 'GET'),
            rfc3986(base_uri),
            rfc3986(normalized)
        ].join('&');
        return base;
    }
    function hmacsign(httpMethod, base_uri, params, consumer_secret, token_secret) {
        var base = generateBase(httpMethod, base_uri, params);
        var key = [
            consumer_secret || '',
            token_secret || ''
        ].map(rfc3986).join('&');
        return sha(key, base, 'sha1');
    }
    function hmacsign256(httpMethod, base_uri, params, consumer_secret, token_secret) {
        var base = generateBase(httpMethod, base_uri, params);
        var key = [
            consumer_secret || '',
            token_secret || ''
        ].map(rfc3986).join('&');
        return sha(key, base, 'sha256');
    }
    function rsasign(httpMethod, base_uri, params, private_key, token_secret) {
        var base = generateBase(httpMethod, base_uri, params);
        var key = private_key || '';
        return rsa(key, base);
    }
    function plaintext(consumer_secret, token_secret) {
        var key = [
            consumer_secret || '',
            token_secret || ''
        ].map(rfc3986).join('&');
        return key;
    }
    function sign(signMethod, httpMethod, base_uri, params, consumer_secret, token_secret) {
        var method;
        var skipArgs = 1;
        switch (signMethod) {
            case 'RSA-SHA1':
                method = rsasign;
                break;
            case 'HMAC-SHA1':
                method = hmacsign;
                break;
            case 'HMAC-SHA256':
                method = hmacsign256;
                break;
            case 'PLAINTEXT':
                method = plaintext;
                skipArgs = 4;
                break;
            default:
                throw new Error('Signature method not supported: ' + signMethod);
        }
        return method.apply(null, [].slice.call(arguments, skipArgs));
    }
    exports.hmacsign = hmacsign;
    exports.hmacsign256 = hmacsign256;
    exports.rsasign = rsasign;
    exports.plaintext = plaintext;
    exports.sign = sign;
    exports.rfc3986 = rfc3986;
    exports.generateBase = generateBase;
},
/* request/lib/hawk.js */ function _(require, module, exports) {
    var crypto = require('crypto');
    function randomString(size) {
        var bits = (size + 1) * 6;
        var buffer = crypto.randomBytes(Math.ceil(bits / 8));
        var string = buffer.toString('base64').replace(/\+/g, '-').replace(/\//g, '_').replace(/=/g, '');
        return string.slice(0, size);
    }
    function calculatePayloadHash(payload, algorithm, contentType) {
        var hash = crypto.createHash(algorithm);
        hash.update('hawk.1.payload\n');
        hash.update((contentType ? contentType.split(';')[0].trim().toLowerCase() : '') + '\n');
        hash.update(payload || '');
        hash.update('\n');
        return hash.digest('base64');
    }
    exports.calculateMac = function (credentials, opts) {
        var normalized = 'hawk.1.header\n' +
            opts.ts + '\n' +
            opts.nonce + '\n' +
            (opts.method || '').toUpperCase() + '\n' +
            opts.resource + '\n' +
            opts.host.toLowerCase() + '\n' +
            opts.port + '\n' +
            (opts.hash || '') + '\n';
        if (opts.ext) {
            normalized = normalized + opts.ext.replace('\\', '\\\\').replace('\n', '\\n');
        }
        normalized = normalized + '\n';
        if (opts.app) {
            normalized = normalized + opts.app + '\n' + (opts.dlg || '') + '\n';
        }
        var hmac = crypto.createHmac(credentials.algorithm, credentials.key).update(normalized);
        var digest = hmac.digest('base64');
        return digest;
    };
    exports.header = function (uri, method, opts) {
        var timestamp = opts.timestamp || Math.floor((Date.now() + (opts.localtimeOffsetMsec || 0)) / 1000);
        var credentials = opts.credentials;
        if (!credentials || !credentials.id || !credentials.key || !credentials.algorithm) {
            return '';
        }
        if (['sha1', 'sha256'].indexOf(credentials.algorithm) === -1) {
            return '';
        }
        var artifacts = {
            ts: timestamp,
            nonce: opts.nonce || randomString(6),
            method: method,
            resource: uri.pathname + (uri.search || ''),
            host: uri.hostname,
            port: uri.port || (uri.protocol === 'http:' ? 80 : 443),
            hash: opts.hash,
            ext: opts.ext,
            app: opts.app,
            dlg: opts.dlg
        };
        if (!artifacts.hash && (opts.payload || opts.payload === '')) {
            artifacts.hash = calculatePayloadHash(opts.payload, credentials.algorithm, opts.contentType);
        }
        var mac = exports.calculateMac(credentials, artifacts);
        var hasExt = artifacts.ext !== null && artifacts.ext !== undefined && artifacts.ext !== '';
        var header = 'Hawk id="' + credentials.id +
            '", ts="' + artifacts.ts +
            '", nonce="' + artifacts.nonce +
            (artifacts.hash ? '", hash="' + artifacts.hash : '') +
            (hasExt ? '", ext="' + artifacts.ext.replace(/\\/g, '\\\\').replace(/"/g, '\\"') : '') +
            '", mac="' + mac + '"';
        if (artifacts.app) {
            header = header + ', app="' + artifacts.app + (artifacts.dlg ? '", dlg="' + artifacts.dlg : '') + '"';
        }
        return header;
    };
},
/* request/lib/multipart.js */ function _(require, module, exports) {
    var uuid = require(952) /* uuid/v4 */;
    var CombinedStream = require(906) /* combined-stream */;
    var isstream = require(919) /* isstream */;
    var Buffer = require(70) /* safe-buffer */.Buffer;
    function Multipart(request) {
        this.request = request;
        this.boundary = uuid();
        this.chunked = false;
        this.body = null;
    }
    Multipart.prototype.isChunked = function (options) {
        var self = this;
        var chunked = false;
        var parts = options.data || options;
        if (!parts.forEach) {
            self.request.emit('error', new Error('Argument error, options.multipart.'));
        }
        if (options.chunked !== undefined) {
            chunked = options.chunked;
        }
        if (self.request.getHeader('transfer-encoding') === 'chunked') {
            chunked = true;
        }
        if (!chunked) {
            parts.forEach(function (part) {
                if (typeof part.body === 'undefined') {
                    self.request.emit('error', new Error('Body attribute missing in multipart.'));
                }
                if (isstream(part.body)) {
                    chunked = true;
                }
            });
        }
        return chunked;
    };
    Multipart.prototype.setHeaders = function (chunked) {
        var self = this;
        if (chunked && !self.request.hasHeader('transfer-encoding')) {
            self.request.setHeader('transfer-encoding', 'chunked');
        }
        var header = self.request.getHeader('content-type');
        if (!header || header.indexOf('multipart') === -1) {
            self.request.setHeader('content-type', 'multipart/related; boundary=' + self.boundary);
        }
        else {
            if (header.indexOf('boundary') !== -1) {
                self.boundary = header.replace(/.*boundary=([^\s;]+).*/, '$1');
            }
            else {
                self.request.setHeader('content-type', header + '; boundary=' + self.boundary);
            }
        }
    };
    Multipart.prototype.build = function (parts, chunked) {
        var self = this;
        var body = chunked ? new CombinedStream() : [];
        function add(part) {
            if (typeof part === 'number') {
                part = part.toString();
            }
            return chunked ? body.append(part) : body.push(Buffer.from(part));
        }
        if (self.request.preambleCRLF) {
            add('\r\n');
        }
        parts.forEach(function (part) {
            var preamble = '--' + self.boundary + '\r\n';
            Object.keys(part).forEach(function (key) {
                if (key === 'body') {
                    return;
                }
                preamble += key + ': ' + part[key] + '\r\n';
            });
            preamble += '\r\n';
            add(preamble);
            add(part.body);
            add('\r\n');
        });
        add('--' + self.boundary + '--');
        if (self.request.postambleCRLF) {
            add('\r\n');
        }
        return body;
    };
    Multipart.prototype.onRequest = function (options) {
        var self = this;
        var chunked = self.isChunked(options);
        var parts = options.data || options;
        self.setHeaders(chunked);
        self.chunked = chunked;
        self.body = self.build(parts, chunked);
    };
    exports.Multipart = Multipart;
},
/* request/lib/redirect.js */ function _(require, module, exports) {
    var url = require('url');
    var isUrl = /^https?:/;
    function Redirect(request) {
        this.request = request;
        this.followRedirect = true;
        this.followRedirects = true;
        this.followAllRedirects = false;
        this.followOriginalHttpMethod = false;
        this.allowRedirect = function () { return true; };
        this.maxRedirects = 10;
        this.redirects = [];
        this.redirectsFollowed = 0;
        this.removeRefererHeader = false;
    }
    Redirect.prototype.onRequest = function (options) {
        var self = this;
        if (options.maxRedirects !== undefined) {
            self.maxRedirects = options.maxRedirects;
        }
        if (typeof options.followRedirect === 'function') {
            self.allowRedirect = options.followRedirect;
        }
        if (options.followRedirect !== undefined) {
            self.followRedirects = !!options.followRedirect;
        }
        if (options.followAllRedirects !== undefined) {
            self.followAllRedirects = options.followAllRedirects;
        }
        if (self.followRedirects || self.followAllRedirects) {
            self.redirects = self.redirects || [];
        }
        if (options.removeRefererHeader !== undefined) {
            self.removeRefererHeader = options.removeRefererHeader;
        }
        if (options.followOriginalHttpMethod !== undefined) {
            self.followOriginalHttpMethod = options.followOriginalHttpMethod;
        }
    };
    Redirect.prototype.redirectTo = function (response) {
        var self = this;
        var request = self.request;
        var redirectTo = null;
        if (response.statusCode >= 300 && response.statusCode < 400 && response.caseless.has('location')) {
            var location = response.caseless.get('location');
            request.debug('redirect', location);
            if (self.followAllRedirects) {
                redirectTo = location;
            }
            else if (self.followRedirects) {
                switch (request.method) {
                    case 'PATCH':
                    case 'PUT':
                    case 'POST':
                    case 'DELETE':
                        // Do not follow redirects
                        break;
                    default:
                        redirectTo = location;
                        break;
                }
            }
        }
        else if (response.statusCode === 401) {
            var authHeader = request._auth.onResponse(response);
            if (authHeader) {
                request.setHeader('authorization', authHeader);
                redirectTo = request.uri;
            }
        }
        return redirectTo;
    };
    Redirect.prototype.onResponse = function (response) {
        var self = this;
        var request = self.request;
        var redirectTo = self.redirectTo(response);
        if (!redirectTo || !self.allowRedirect.call(request, response)) {
            return false;
        }
        request.debug('redirect to', redirectTo);
        // ignore any potential response body.  it cannot possibly be useful
        // to us at this point.
        // response.resume should be defined, but check anyway before calling. Workaround for browserify.
        if (response.resume) {
            response.resume();
        }
        if (self.redirectsFollowed >= self.maxRedirects) {
            request.emit('error', new Error('Exceeded maxRedirects. Probably stuck in a redirect loop ' + request.uri.href));
            return false;
        }
        self.redirectsFollowed += 1;
        if (!isUrl.test(redirectTo)) {
            redirectTo = url.resolve(request.uri.href, redirectTo);
        }
        var uriPrev = request.uri;
        request.uri = url.parse(redirectTo);
        // handle the case where we change protocol from https to http or vice versa
        if (request.uri.protocol !== uriPrev.protocol) {
            delete request.agent;
        }
        self.redirects.push({ statusCode: response.statusCode, redirectUri: redirectTo });
        if (self.followAllRedirects && request.method !== 'HEAD' &&
            response.statusCode !== 401 && response.statusCode !== 307) {
            request.method = self.followOriginalHttpMethod ? request.method : 'GET';
        }
        // request.method = 'GET' // Force all redirects to use GET || commented out fixes #215
        delete request.src;
        delete request.req;
        delete request._started;
        if (response.statusCode !== 401 && response.statusCode !== 307) {
            // Remove parameters from the previous response, unless this is the second request
            // for a server that requires digest authentication.
            delete request.body;
            delete request._form;
            if (request.headers) {
                request.removeHeader('host');
                request.removeHeader('content-type');
                request.removeHeader('content-length');
                if (request.uri.hostname !== request.originalHost.split(':')[0]) {
                    // Remove authorization if changing hostnames (but not if just
                    // changing ports or protocols).  This matches the behavior of curl:
                    // https://github.com/bagder/curl/blob/6beb0eee/lib/http.c#L710
                    request.removeHeader('authorization');
                }
            }
        }
        if (!self.removeRefererHeader) {
            request.setHeader('referer', uriPrev.href);
        }
        request.emit('redirect');
        request.init();
        return true;
    };
    exports.Redirect = Redirect;
},
/* request/lib/tunnel.js */ function _(require, module, exports) {
    var url = require('url');
    var tunnel = require(961) /* tunnel-agent */;
    var defaultProxyHeaderWhiteList = [
        'accept',
        'accept-charset',
        'accept-encoding',
        'accept-language',
        'accept-ranges',
        'cache-control',
        'content-encoding',
        'content-language',
        'content-location',
        'content-md5',
        'content-range',
        'content-type',
        'connection',
        'date',
        'expect',
        'max-forwards',
        'pragma',
        'referer',
        'te',
        'user-agent',
        'via'
    ];
    var defaultProxyHeaderExclusiveList = [
        'proxy-authorization'
    ];
    function constructProxyHost(uriObject) {
        var port = uriObject.port;
        var protocol = uriObject.protocol;
        var proxyHost = uriObject.hostname + ':';
        if (port) {
            proxyHost += port;
        }
        else if (protocol === 'https:') {
            proxyHost += '443';
        }
        else {
            proxyHost += '80';
        }
        return proxyHost;
    }
    function constructProxyHeaderWhiteList(headers, proxyHeaderWhiteList) {
        var whiteList = proxyHeaderWhiteList
            .reduce(function (set, header) {
            set[header.toLowerCase()] = true;
            return set;
        }, {});
        return Object.keys(headers)
            .filter(function (header) {
            return whiteList[header.toLowerCase()];
        })
            .reduce(function (set, header) {
            set[header] = headers[header];
            return set;
        }, {});
    }
    function constructTunnelOptions(request, proxyHeaders) {
        var proxy = request.proxy;
        var tunnelOptions = {
            proxy: {
                host: proxy.hostname,
                port: +proxy.port,
                proxyAuth: proxy.auth,
                headers: proxyHeaders
            },
            headers: request.headers,
            ca: request.ca,
            cert: request.cert,
            key: request.key,
            passphrase: request.passphrase,
            pfx: request.pfx,
            ciphers: request.ciphers,
            rejectUnauthorized: request.rejectUnauthorized,
            secureOptions: request.secureOptions,
            secureProtocol: request.secureProtocol
        };
        return tunnelOptions;
    }
    function constructTunnelFnName(uri, proxy) {
        var uriProtocol = (uri.protocol === 'https:' ? 'https' : 'http');
        var proxyProtocol = (proxy.protocol === 'https:' ? 'Https' : 'Http');
        return [uriProtocol, proxyProtocol].join('Over');
    }
    function getTunnelFn(request) {
        var uri = request.uri;
        var proxy = request.proxy;
        var tunnelFnName = constructTunnelFnName(uri, proxy);
        return tunnel[tunnelFnName];
    }
    function Tunnel(request) {
        this.request = request;
        this.proxyHeaderWhiteList = defaultProxyHeaderWhiteList;
        this.proxyHeaderExclusiveList = [];
        if (typeof request.tunnel !== 'undefined') {
            this.tunnelOverride = request.tunnel;
        }
    }
    Tunnel.prototype.isEnabled = function () {
        var self = this;
        var request = self.request;
        // Tunnel HTTPS by default. Allow the user to override this setting.
        // If self.tunnelOverride is set (the user specified a value), use it.
        if (typeof self.tunnelOverride !== 'undefined') {
            return self.tunnelOverride;
        }
        // If the destination is HTTPS, tunnel.
        if (request.uri.protocol === 'https:') {
            return true;
        }
        // Otherwise, do not use tunnel.
        return false;
    };
    Tunnel.prototype.setup = function (options) {
        var self = this;
        var request = self.request;
        options = options || {};
        if (typeof request.proxy === 'string') {
            request.proxy = url.parse(request.proxy);
        }
        if (!request.proxy || !request.tunnel) {
            return false;
        }
        // Setup Proxy Header Exclusive List and White List
        if (options.proxyHeaderWhiteList) {
            self.proxyHeaderWhiteList = options.proxyHeaderWhiteList;
        }
        if (options.proxyHeaderExclusiveList) {
            self.proxyHeaderExclusiveList = options.proxyHeaderExclusiveList;
        }
        var proxyHeaderExclusiveList = self.proxyHeaderExclusiveList.concat(defaultProxyHeaderExclusiveList);
        var proxyHeaderWhiteList = self.proxyHeaderWhiteList.concat(proxyHeaderExclusiveList);
        // Setup Proxy Headers and Proxy Headers Host
        // Only send the Proxy White Listed Header names
        var proxyHeaders = constructProxyHeaderWhiteList(request.headers, proxyHeaderWhiteList);
        proxyHeaders.host = constructProxyHost(request.uri);
        proxyHeaderExclusiveList.forEach(request.removeHeader, request);
        // Set Agent from Tunnel Data
        var tunnelFn = getTunnelFn(request);
        var tunnelOptions = constructTunnelOptions(request, proxyHeaders);
        request.agent = tunnelFn(tunnelOptions);
        return true;
    };
    Tunnel.defaultProxyHeaderWhiteList = defaultProxyHeaderWhiteList;
    Tunnel.defaultProxyHeaderExclusiveList = defaultProxyHeaderExclusiveList;
    exports.Tunnel = Tunnel;
},
/* tunnel-agent/index.js */ function _(require, module, exports) {
    var net = require('net'), tls = require('tls'), http = require('http'), https = require('https'), events = require('events'), assert = require('assert'), util = require('util'), Buffer = require(70) /* safe-buffer */.Buffer;
    exports.httpOverHttp = httpOverHttp;
    exports.httpsOverHttp = httpsOverHttp;
    exports.httpOverHttps = httpOverHttps;
    exports.httpsOverHttps = httpsOverHttps;
    function httpOverHttp(options) {
        var agent = new TunnelingAgent(options);
        agent.request = http.request;
        return agent;
    }
    function httpsOverHttp(options) {
        var agent = new TunnelingAgent(options);
        agent.request = http.request;
        agent.createSocket = createSecureSocket;
        agent.defaultPort = 443;
        return agent;
    }
    function httpOverHttps(options) {
        var agent = new TunnelingAgent(options);
        agent.request = https.request;
        return agent;
    }
    function httpsOverHttps(options) {
        var agent = new TunnelingAgent(options);
        agent.request = https.request;
        agent.createSocket = createSecureSocket;
        agent.defaultPort = 443;
        return agent;
    }
    function TunnelingAgent(options) {
        var self = this;
        self.options = options || {};
        self.proxyOptions = self.options.proxy || {};
        self.maxSockets = self.options.maxSockets || http.Agent.defaultMaxSockets;
        self.requests = [];
        self.sockets = [];
        self.on('free', function onFree(socket, host, port) {
            for (var i = 0, len = self.requests.length; i < len; ++i) {
                var pending = self.requests[i];
                if (pending.host === host && pending.port === port) {
                    // Detect the request to connect same origin server,
                    // reuse the connection.
                    self.requests.splice(i, 1);
                    pending.request.onSocket(socket);
                    return;
                }
            }
            socket.destroy();
            self.removeSocket(socket);
        });
    }
    util.inherits(TunnelingAgent, events.EventEmitter);
    TunnelingAgent.prototype.addRequest = function addRequest(req, options) {
        var self = this;
        // Legacy API: addRequest(req, host, port, path)
        if (typeof options === 'string') {
            options = {
                host: options,
                port: arguments[2],
                path: arguments[3]
            };
        }
        if (self.sockets.length >= this.maxSockets) {
            // We are over limit so we'll add it to the queue.
            self.requests.push({ host: options.host, port: options.port, request: req });
            return;
        }
        // If we are under maxSockets create a new one.
        self.createConnection({ host: options.host, port: options.port, request: req });
    };
    TunnelingAgent.prototype.createConnection = function createConnection(pending) {
        var self = this;
        self.createSocket(pending, function (socket) {
            socket.on('free', onFree);
            socket.on('close', onCloseOrRemove);
            socket.on('agentRemove', onCloseOrRemove);
            pending.request.onSocket(socket);
            function onFree() {
                self.emit('free', socket, pending.host, pending.port);
            }
            function onCloseOrRemove(err) {
                self.removeSocket(socket);
                socket.removeListener('free', onFree);
                socket.removeListener('close', onCloseOrRemove);
                socket.removeListener('agentRemove', onCloseOrRemove);
            }
        });
    };
    TunnelingAgent.prototype.createSocket = function createSocket(options, cb) {
        var self = this;
        var placeholder = {};
        self.sockets.push(placeholder);
        var connectOptions = mergeOptions({}, self.proxyOptions, { method: 'CONNECT',
            path: options.host + ':' + options.port,
            agent: false
        });
        if (connectOptions.proxyAuth) {
            connectOptions.headers = connectOptions.headers || {};
            connectOptions.headers['Proxy-Authorization'] = 'Basic ' +
                Buffer.from(connectOptions.proxyAuth).toString('base64');
        }
        debug('making CONNECT request');
        var connectReq = self.request(connectOptions);
        connectReq.useChunkedEncodingByDefault = false; // for v0.6
        connectReq.once('response', onResponse); // for v0.6
        connectReq.once('upgrade', onUpgrade); // for v0.6
        connectReq.once('connect', onConnect); // for v0.7 or later
        connectReq.once('error', onError);
        connectReq.end();
        function onResponse(res) {
            // Very hacky. This is necessary to avoid http-parser leaks.
            res.upgrade = true;
        }
        function onUpgrade(res, socket, head) {
            // Hacky.
            process.nextTick(function () {
                onConnect(res, socket, head);
            });
        }
        function onConnect(res, socket, head) {
            connectReq.removeAllListeners();
            socket.removeAllListeners();
            if (res.statusCode === 200) {
                assert.equal(head.length, 0);
                debug('tunneling connection has established');
                self.sockets[self.sockets.indexOf(placeholder)] = socket;
                cb(socket);
            }
            else {
                debug('tunneling socket could not be established, statusCode=%d', res.statusCode);
                var error = new Error('tunneling socket could not be established, ' + 'statusCode=' + res.statusCode);
                error.code = 'ECONNRESET';
                options.request.emit('error', error);
                self.removeSocket(placeholder);
            }
        }
        function onError(cause) {
            connectReq.removeAllListeners();
            debug('tunneling socket could not be established, cause=%s\n', cause.message, cause.stack);
            var error = new Error('tunneling socket could not be established, ' + 'cause=' + cause.message);
            error.code = 'ECONNRESET';
            options.request.emit('error', error);
            self.removeSocket(placeholder);
        }
    };
    TunnelingAgent.prototype.removeSocket = function removeSocket(socket) {
        var pos = this.sockets.indexOf(socket);
        if (pos === -1)
            return;
        this.sockets.splice(pos, 1);
        var pending = this.requests.shift();
        if (pending) {
            // If we have pending requests and a socket gets closed a new one
            // needs to be created to take over in the pool for the one that closed.
            this.createConnection(pending);
        }
    };
    function createSecureSocket(options, cb) {
        var self = this;
        TunnelingAgent.prototype.createSocket.call(self, options, function (socket) {
            // 0 is dummy port for v0.6
            var secureSocket = tls.connect(0, mergeOptions({}, self.options, { servername: options.host,
                socket: socket
            }));
            self.sockets[self.sockets.indexOf(socket)] = secureSocket;
            cb(secureSocket);
        });
    }
    function mergeOptions(target) {
        for (var i = 1, len = arguments.length; i < len; ++i) {
            var overrides = arguments[i];
            if (typeof overrides === 'object') {
                var keys = Object.keys(overrides);
                for (var j = 0, keyLen = keys.length; j < keyLen; ++j) {
                    var k = keys[j];
                    if (overrides[k] !== undefined) {
                        target[k] = overrides[k];
                    }
                }
            }
        }
        return target;
    }
    var debug;
    if (process.env.NODE_DEBUG && /\btunnel\b/.test(process.env.NODE_DEBUG)) {
        debug = function () {
            var args = Array.prototype.slice.call(arguments);
            if (typeof args[0] === 'string') {
                args[0] = 'TUNNEL: ' + args[0];
            }
            else {
                args.unshift('TUNNEL:');
            }
            console.error.apply(console, args);
        };
    }
    else {
        debug = function () { };
    }
    exports.debug = debug; // for test
},
/* performance-now/lib/performance-now.js */ function _(require, module, exports) {
    // Generated by CoffeeScript 1.12.2
    (function () {
        var getNanoSeconds, hrtime, loadTime, moduleLoadTime, nodeLoadTime, upTime;
        if ((typeof performance !== "undefined" && performance !== null) && performance.now) {
            module.exports = function () {
                return performance.now();
            };
        }
        else if ((typeof process !== "undefined" && process !== null) && process.hrtime) {
            module.exports = function () {
                return (getNanoSeconds() - nodeLoadTime) / 1e6;
            };
            hrtime = process.hrtime;
            getNanoSeconds = function () {
                var hr;
                hr = hrtime();
                return hr[0] * 1e9 + hr[1];
            };
            moduleLoadTime = getNanoSeconds();
            upTime = process.uptime() * 1e9;
            nodeLoadTime = moduleLoadTime - upTime;
        }
        else if (Date.now) {
            module.exports = function () {
                return Date.now() - loadTime;
            };
            loadTime = Date.now();
        }
        else {
            module.exports = function () {
                return new Date().getTime() - loadTime;
            };
            loadTime = new Date().getTime();
        }
    }).call(this);
},
/* image-size/lib/index.js */ function _(require, module, exports) {
    var fs = require('fs');
    var path = require('path');
    var detector = require(964) /* ./detector */;
    var handlers = {};
    var types = require(965) /* ./types */;
    // load all available handlers
    types.forEach(function (type) {
        handlers[type] = require('./types/' + type);
    });
    // Maximum buffer size, with a default of 128 kilobytes.
    // TO-DO: make this adaptive based on the initial signature of the image
    var MaxBufferSize = 128 * 1024;
    function lookup(buffer, filepath) {
        // detect the file type.. don't rely on the extension
        var type = detector(buffer, filepath);
        // find an appropriate handler for this file type
        if (type in handlers) {
            var size = handlers[type].calculate(buffer, filepath);
            if (size !== false) {
                size.type = type;
                return size;
            }
        }
        // throw up, if we don't understand the file
        throw new TypeError('unsupported file type: ' + type + ' (file: ' + filepath + ')');
    }
    function asyncFileToBuffer(filepath, callback) {
        // open the file in read only mode
        fs.open(filepath, 'r', function (err, descriptor) {
            if (err) {
                return callback(err);
            }
            var size = fs.fstatSync(descriptor).size;
            if (size <= 0) {
                return callback(new Error("File size is not greater than 0 —— " + filepath));
            }
            var bufferSize = Math.min(size, MaxBufferSize);
            var buffer = new Buffer(bufferSize);
            // read first buffer block from the file, asynchronously
            fs.read(descriptor, buffer, 0, bufferSize, 0, function (err) {
                if (err) {
                    return callback(err);
                }
                // close the file, we are done
                fs.close(descriptor, function (err) {
                    callback(err, buffer);
                });
            });
        });
    }
    function syncFileToBuffer(filepath) {
        // read from the file, synchronously
        var descriptor = fs.openSync(filepath, 'r');
        var size = fs.fstatSync(descriptor).size;
        var bufferSize = Math.min(size, MaxBufferSize);
        var buffer = new Buffer(bufferSize);
        fs.readSync(descriptor, buffer, 0, bufferSize, 0);
        fs.closeSync(descriptor);
        return buffer;
    }
    /**
     * @params input - buffer or relative/absolute path of the image file
     * @params callback - optional function for async detection
     */
    module.exports = function (input, callback) {
        // Handle buffer input
        if (Buffer.isBuffer(input)) {
            return lookup(input);
        }
        // input should be a string at this point
        if (typeof input !== 'string') {
            throw new TypeError('invalid invocation');
        }
        // resolve the file path
        var filepath = path.resolve(input);
        if (typeof callback === 'function') {
            asyncFileToBuffer(filepath, function (err, buffer) {
                if (err) {
                    return callback(err);
                }
                // return the dimensions
                var dimensions;
                try {
                    dimensions = lookup(buffer, filepath);
                }
                catch (e) {
                    err = e;
                }
                callback(err, dimensions);
            });
        }
        else {
            var buffer = syncFileToBuffer(filepath);
            return lookup(buffer, filepath);
        }
    };
    module.exports.types = types;
},
/* image-size/lib/detector.js */ function _(require, module, exports) {
    var typeMap = {};
    var types = require(965) /* ./types */;
    // load all available handlers
    types.forEach(function (type) {
        typeMap[type] = require('./types/' + type).detect;
    });
    module.exports = function (buffer, filepath) {
        var type, result;
        for (type in typeMap) {
            result = typeMap[type](buffer, filepath);
            if (result) {
                return type;
            }
        }
    };
},
/* image-size/lib/types.js */ function _(require, module, exports) {
    module.exports = [
        'bmp',
        'gif',
        'jpg',
        'png',
        'psd',
        'svg',
        'tiff',
        'webp',
        'dds'
    ];
},
], 0, {"main":0,"sys":43,"build":58,"compiler":59,"transforms":65,"linker":66,"prelude":75,"tsconfig.ext.json":76,"compile":823}, {"path":true,"fs":true,"util":true,"crypto":true,"@microsoft/typescript-etw":true,"os":true,"buffer":true,"inspector":true,"module":true,"child_process":true,"readline":true,"tty":true,"assert":true,"events":true,"url":true,"constants":true,"stream":true,"net":true,"punycode":true,"http":true,"https":true,"zlib":true,"querystring":true,"tls":true});
})

//# sourceMappingURL=compiler.js.map
